// DiffractionImageADSC.cpp
// maintained by G.Winter
// 17th December 2003
// 
// The bits of the DiffractionImage library which pertain to ADSC
// images.
// 
// 
// 

#include "DiffractionImage.h"
#include <string>
// if we are in development mode, then include things pertaining to assert.

#ifdef TEST_CODE
#define NDEBUG
#include <assert.h>
#endif

using namespace std;

namespace DI 
	{
	// strip whitespace, make lower case
	string tidyKey(string key)
		{
		string result = "";
		int i, length;
		length = key.length();
		char littleString[2];
		littleString[1] = '\0';
		for(i = 0; i < length; i++)
			{
			if ((key[i] > 32) && (key[i] != '='))
				{
				littleString[0] = (char) tolower(key[i]);
				result.append(littleString);
				}
			}
		return result;
		}

    // strip whitespace from the ends
    string tidyValue(string value)
		{
		string result = "";
		int i, j, length;
		char littleString[2];
		littleString[1] = '\0';
		length = value.length();
		for(i = 0; i < length; i++)
			{
			if (value[i] > 32)
				break;
			}
		for(j = i; j < length; j++)
			{
			if (value[j] != ';')
				{
				littleString[0] = value[j];
				result.append(littleString);
				}
			}
		return result;
		}

	//simulate the header writing to keep track of number of bytes that will be written
	int DiffractionImage::newHeaderSize()
		{
		int size = 0;
		char value[30];
		string printline;
		printline = "{\n";
		size += printline.size();
		int hsize = (int) headerKeys.size();
		for(int i=0; i < hsize; i++)
			{
			if(headerKeys[i].compare("beam_center_x") == 0 && fabs(atof(headerMap["beam_center_x"].c_str()) - beamX) > 0.0001)
				{
				sprintf(value,"%f",beamX);
				headerMap["beam_center_x"] = value;
				}
			if(headerKeys[i].compare("beam_center_y") == 0 && fabs(atof(headerMap["beam_center_y"].c_str()) - beamY) > 0.0001)
				{
				sprintf(value,"%f",beamY);
				headerMap["beam_center_y"] = value;
				}
			if(headerKeys[i].compare("wavelength") == 0 && fabs(atof(headerMap["wavelength"].c_str()) - wavelength) > 0.0001)
				{
				sprintf(value,"%f",wavelength);
				headerMap["wavelength"] = value;
				}
			if(headerKeys[i].compare("distance") == 0 && fabs(atof(headerMap["distance"].c_str()) - distance) > 0.0001)
				{
				sprintf(value,"%f",distance);
				headerMap["distance"] = value;
				}
			printline+=headerKeys[i];
			printline+="=";
			printline+=headerMap[tidyKey(headerKeys[i])];
			printline+=";\n";
			size += printline.size();
			}
		printline = "}\n";
		size += printline.size();
		return size;
		}
  
	// read in an ADSC image - this will read the header first, then 
	// reopen the file to read the unsigned shorts out - byte 
	// swapping if needed. NOTE WELL - this is hacky as there is 
	// an assumption here that the images will be represented as
	// unsigned shorts, which could well be wrong!
	void DiffractionImage::loadADSC()
		{
		loadADSCHeader();  // read the header
		if(format.compare("saturn") == 0 || format.compare("raxisd") == 0)
			return;
		allocateImage(); // allocate the image structure
#ifdef TEST_CODE
		assert(image != NULL);
#endif
		// after this stage, the rest of the image is just a big pile
		// of unsigned shorts - another chance to share code! :o)
		// this routine will be defined in DiffractionImageRaw.cpp
		loadRawImage(filename, originalHeaderLength);
		}
  
	void DiffractionImage::loadADSCHeader()
		{
		// how shall I read the header? probably using a hash table 
		// (map) is best..
		fstream fin;
		char* scanaxis;
		int equals, semicolon,index;
		string line, key, value;
		bool headerComplete;
		index=0;
		headerComplete = false;
		fin.open(filename.c_str(), ios::in|ios::binary); //open the file for input  
		if (!fin.good())
			{
			string msg="error opening ADSC file: ";
			msg+=filename;
			throw(DiffractionImageException(msg));
			}
		getline(fin, line);
		if (line.compare("{") != 0) // an ADSC header begins with { and ends with } alone 
			{
			fin.close();
			throw(DiffractionImageException("not an ADSC header")); // this is not an ADSC image
			}
		// now work through the header
		while (headerComplete == false)
			{
			if (fin.eof())
				throw(DiffractionImageException("EOF before ADSC header end"));
			getline(fin, line);
			if (line[0] == '}')
				{
				// then the header is finished
				headerComplete = true;
				break;
				}
#ifdef TEST_CODE_SHOW_HEADER
			cerr << line << endl;
#endif	
			equals = line.find('=');
			if (equals == (int) string::npos)
				throw(DiffractionImageException("'=' not found in ADSC header"));
			semicolon = line.find(';');
			if (semicolon == (int) string::npos)
				throw(DiffractionImageException("';' not found in ADSC header"));
			// delete these things from the line now - or just 
			// select the limits carefully
			key = line.substr(0, equals);
			equals += 1; // advance equals to miss the sign
			value = line.substr(equals, semicolon - equals);
			// these are defined locally
			headerKeys.push_back(key);
			key = tidyKey(key);
			value = tidyValue(value);
#ifdef TEST_CODE
			cout << "header key = \"" << key << "\" value = \"" << value 
			<< "\"" << endl;
#endif
			if(key.find("ccd_detector_description")!=(int) string::npos &&
			   value.find("Saturn")!=(int) string::npos)
				format="saturn";
			if(key.find("rx_detector_description")!=(int) string::npos &&
			   value.find("RAXIS")!=(int) string::npos)
				format="raxisd";

			headerMap[key] = value;
			index++;
			}
		if(format.compare("saturn") == 0 || format.compare("raxisd") == 0)
			return;
		else
			{
			format="SMV";
			manufact="ADSC";
			}
		// now parse what we stored in the map,
		// verify that the header structure is ok - this needs to be 
		// done via a macro - and now here is a prototype of what I want
		if (headerMap.find("header_bytes") == headerMap.end())
			throw(DiffractionImageException("header_bytes not found in ADSC header"));

		ASSERT_IN_HEADER("size1");
		ASSERT_IN_HEADER("size2");
		ASSERT_IN_HEADER("byte_order");
		ASSERT_IN_HEADER("distance");
		ASSERT_IN_HEADER("wavelength");
		ASSERT_IN_HEADER("beam_center_x");
		ASSERT_IN_HEADER("beam_center_y");
		ASSERT_IN_HEADER("pixel_size");
		ASSERT_IN_HEADER("time");
		ASSERT_IN_HEADER("osc_start");
		ASSERT_IN_HEADER("detector_sn");
		// do not assert this - be gentle if not in header!
		// ASSERT_IN_HEADER("date");

		// now check that the rest of the oscillation 
		// information is in there
		if ((headerMap.find("osc_end") == headerMap.end()) &&
			(headerMap.find("osc_range") == headerMap.end())) // then there isn't enough information in there
			throw(DiffractionImageException("oscillation information not ADSC inheader"));
    
		// if we get here then everything we want is in the header
		// now fetch it all out into the header data structure
		// this will also be done via macro, with the following prototype
		sscanf(headerMap["header_bytes"].c_str(), "%d",&originalHeaderLength);
		GET_FROM_HEADER("size1", "%d", width);
		GET_FROM_HEADER("size2", "%d", height);
		GET_FROM_HEADER("beam_center_x", "%f", beamX);
		GET_FROM_HEADER("beam_center_y", "%f", beamY);
		GET_FROM_HEADER("wavelength", "%f", wavelength);
		GET_FROM_HEADER("distance", "%f", distance);
		GET_FROM_HEADER("pixel_size", "%f", pixelX);	
		GET_FROM_HEADER("pixel_size", "%f", pixelY);
		GET_FROM_HEADER("time", "%f", exposureTime);
		GET_FROM_HEADER("osc_start", "%f", oscStart);
		float range;
		if (headerMap.find("axis") != headerMap.end())
			{
			oscAxis=headerMap["axis"];
			}
		else
			oscAxis="";
		if (headerMap.find("phi") != headerMap.end())
			{
			float phiStartTemp;
			phiStartTemp = oscStart;
			GET_FROM_HEADER("phi", "%f", oscStart);
			if(strcmp(oscAxis.c_str(),"")==0)
				oscAxis="phi";
			if (fabs(phiStartTemp - oscStart) > 0.01)
				{
				// something dodgy is going on
				warning = true;
				message = "ambiguous phi values;";
				}
			}
		if (headerMap.find("osc_end") == headerMap.end())
			{
			// then we have to figure it out from the range
			GET_FROM_HEADER("osc_range", "%f", range);
			oscEnd = oscStart + range;
			}
		else
			GET_FROM_HEADER("osc_end", "%f", oscEnd);  // we can just get it straight from the header
		
		if (headerMap.find("twotheta") != headerMap.end())
			{
			GET_FROM_HEADER("twotheta", "%f", twoTheta);
			}
		else
			twoTheta=0;
    
		// get the detector serial number out
		serialNo = headerMap["detector_sn"];
		if (headerMap.find("date") != headerMap.end()) //we can read the date out
			date = headerMap["date"];
		if (headerMap["byte_order"].compare("big_endian") == 0)
			bigEndian = true;
		else
			bigEndian = false;
		if(strcmp(oscAxis.c_str(),"")==0)
			oscAxis="unknown";
		}

/*	void DiffractionImage::saveADSC()
		{
		int i,sizetest,sizediff;
		FILE* myoutput;
		char value[30];
		string printline;

		// For an ADSC file the header is text-format so we can use the simple
		// function for writing string to write back the header with the 
		// modifications if any
		myoutput = fopen(filename.c_str(),"wb");

		//checking the new header size by simulating lines to be written 
		sizetest = newHeaderSize();

		// updating field containing header size if we write more 
		// than the previous size
		if(sizetest > atoi(headerMap["header_bytes"].c_str()))
			{
			sizetest = sizetest/512;
			originalHeaderLength = (sizetest + 1) * 512;
			sprintf(value,"%d",originalHeaderLength);
			headerMap["header_bytes"] = value;
			}
		fprintf(myoutput,"{\n");
		int hsize = (int) headerKeys.size();
		for(i = 0; i < hsize; i++)
			{
			printline = headerKeys[i] + "=" + headerMap[tidyKey(headerKeys[i])] + ";";
			fprintf(myoutput,"%s\n",printline.c_str());
			}
		fprintf(myoutput,"}\n");

		//writing blanks to effectively write originalHeaderLength bytes
		sizediff = originalHeaderLength - sizetest;
		for(i = 0; i < sizediff; i++)
			fprintf(myoutput," ");
		fclose(myoutput);

		// then open the image in write mode using file descriptor
		int fd = open(filename.c_str(), MYWRITE);
		if (fd < 0)
			throw(DiffractionImageException("error opening raw image"));
		int amount; 
		//we already wrote the header so we go to its end for writing back the image
		lseek(fd,originalHeaderLength,SEEK_SET);
		if (image == NULL)
			{
			close(fd);
			throw(DiffractionImageException("error image is empty"));
			}
		// in reading we read and then swap bytes, therefore for writing i swap first
		if (!(bigEndian && machineBigEndian()) && (bigEndian || machineBigEndian()))
			{
			// byte swap - this shouldn't be implemented as a 
			// function call as that will be very expensive - perhaps
			// there is a clever way to perform a byte swap
			// using something like 
			// a = ((a & 0xff00) >> 8) & ((a & 0x00ff) << 8);
			for (int i = 0; i < width * height; i++)
				byteSwap(&(image[i]));
			}
		// and finally write the image at once.
		amount = write(fd, (unsigned char *) image, width * height * 2);
		close(fd);
		if (amount < width * height * 2)
			throw(DiffractionImageException("error writing back raw image"));
		close(fd);	
		}*/
	};
