// DiffractionImageRaxis.cpp
// Author: F. Remacle
// 25 January 2005
// 
// The bits of the DiffractionImage library which pertain to Raxis images
// images.

#include "DiffractionImage.h"
#include "Rigaku/RaxisImage.h"

// if we are in development mode, then include things pertaining to
// assert.

#ifdef TEST_CODE
#define NDEBUG
#include <assert.h>
#endif

using namespace std;

namespace DI 
	{	
	void DiffractionImage::loadRaxis()
		{
		unsigned short* tmpimg;
		ifstream filein;  
		filein.open(filename.c_str(), ifstream::in|ios::binary);
		if (!filein.good())
			throw(DiffractionImageException("error opening RAXIS file: "+filename));
		// read the header		
		loadRaxisHeader();
		tmpimg=new unsigned short[width*height];
		allocateImage();
#ifdef TEST_CODE
		assert(image != NULL);
#endif
		// load the image
		if (!ReadImageBuffer(filein,tmpimg))
			throw(DiffractionImageException("error reading RAXIS file: "+filename));
		for(int i=0;i<width*height;i++)
			image[i]=tmpimg[i];
		free(tmpimg);
		filein.close();
		}
    
	void DiffractionImage::loadRaxisHeader()
		{
		RigakuHeader header;
		ifstream fin;
		char gonaxis[5][10];
		char* gonaxisraw;
		char* token;
		char* token2;
		int i,scanaxis;
			
		gonaxisraw=new char[40];
		// open the stream to the file
		fin.open(filename.c_str(), ifstream::in);
		if (!fin.good())
			throw(DiffractionImageException("error opening RAXIS file: "+filename));
		//loading header content from file
		ReadHeader(fin, header);
    
		// loading the value into the corresponding fields
		//! raxis beam center coordinate is is pixels
		// therefore it needs to be converted in mm
		beamX = header.drtx * header.xsiz; 
		beamY = header.drtz * header.zsiz; 
		distance = header.camr;
		wavelength = header.wlng;
		width = header.xpxl;
		height = header.zpxl;
		pixelX = header.xsiz;
		pixelY = header.zsiz;
		//strcpy(gonaxisraw,header.gnom);
		scanaxis=-1;
		scanaxis=header.saxs;
		//need this safety check to ensure we are not reading rubbish from header
		if(scanaxis > 4 || scanaxis < 0)
			{
			oscStart = header.phis;
			oscEnd = header.phie;
			oscAxis= "phi";
			}
		else
			{
			oscStart = header.gst[scanaxis];
			oscEnd = header.gend[scanaxis];
			if(oscStart==0 && oscEnd==0)
				{
				oscStart = header.phis;
				oscEnd = header.phie;
				oscAxis= "phi";
				}	
			else
				{
				strcpy(gonaxisraw,header.gnom);
				token=strtok(gonaxisraw," ,");
				i=0;
				while(token!=NULL)
					{
					strcpy(gonaxis[i],token);
					i++;
					token=strtok(NULL," ,");
					}
				oscAxis= gonaxis[scanaxis];
				}
			}
		exposureTime = header.fext;
		twoTheta = header.thta;
		token2=strtok(header.date," ");
		date=token2;
		serialNo = "N/A";
	    
		//0 means from left to right (viewing from our Z axis)
		if(header.dr_x==0)
			{
			rapidAxis[0]=1;
			rapidAxis[1]=0;
			rapidAxis[2]=0;
			}
		else
			{
			rapidAxis[0]=-1;
			rapidAxis[1]=0;
			rapidAxis[2]=0;
			}
		//0 means from bottom to top (viewing from our Z axis)
		if(header.dr_z==0)
			{
			slowAxis[0]=0;
			slowAxis[1]=1;
			slowAxis[2]=0;
			}
		else
			{
			slowAxis[0]=0;
			slowAxis[1]=-1;
			slowAxis[2]=0;
			}
		//1 means that the axis previously defined refer to the back of the 
		//detector so we need to flip the R axis.
		if(header.drxz==1)
			rapidAxis[0]=-rapidAxis[0];
		
		fin.close();
		format="R-AXIS";
		manufact="RIGAKU";
		return;
		}

	//WARNING THIS FUNCTION DOES NOT WORK YET !
/*	void DiffractionImage::saveRaxis() 
		{
		RigakuHeader header;
		ifstream fin;
		fstream fin2;
		// open the stream to the file
		fin.open(filename.c_str(), ifstream::in);
		if (!fin.good())
			throw(DiffractionImageException("error opening file"));
 
		//loading header content from file
		ReadHeader(fin, header);
	
		// raxis beam center coordinate is is pixels therefore it needs 
		// to be converted
		if(fabs((header.drtx*header.xsiz)-beamX)>0.0001) 
			header.drtx=beamX/header.xsiz; 
		if(fabs((header.drtz*header.zsiz)-beamY)>0.0001) 
			header.drtz=beamY/header.zsiz;
		if(fabs(header.camr-distance)>0.0001) 
			header.camr=distance;
		if(fabs(header.wlng-wavelength)>0.0001) 
			header.wlng=wavelength;
		fin.close();
		fin2.open(filename.c_str(), ifstream::out);
	
		// Do we need to swap the data?
		unsigned short one = 1;
		char *p1 = reinterpret_cast<char *>(&one);
		if(1 == static_cast<int>(*p1)) 
			Swap(header);
		fin2.write(reinterpret_cast<char *>(&header),MinimumRigakuHeaderLength);
		long size = header.rlng - MinimumRigakuHeaderLength;
		char *blanks = new char [size];
		fin2.write(blanks,size);

		// Do we need to swap the data?
		unsigned short one1 = 1;
		char *p = reinterpret_cast<char *>(&one1);
		if(1 == static_cast<int>(*p))
			{
			// Swap each pixel value in the image buffer
			unsigned short *p = const_cast<unsigned short *>(image);
			unsigned short *pEnd = p+size;
			for(NULL; p < pEnd; p++)
				*p = Swap(*p);
			}
		fin2.write(reinterpret_cast<char *>(image),header.xpxl*header.zpxl);
		fin2.close();
		}*/

	};
