// DiffractionImageRaw.cpp
// maintained by G.Winter
// 17th December 2003
// 
// Things for handling the raw images in the DiffractionImage module - 
// which could be pretty useful I guess... would make extending
// the image handling a bit easier - take the XDS approach!
// 
// 
// $Id: DiffractionImageRaw.cpp,v 1.5 2008/05/07 11:00:52 fr45 Exp $

#include "DiffractionImage.h"


using namespace std;

namespace DI 
{
    void DiffractionImage::allocateImage(void)
		{
		// allocate the image, trapping errors as we go
		if ((width <= 0) || (height <= 0))
			throw(DiffractionImageException("bad size"));
		if (image != NULL)
			{
			// free the image before we restart
			delete [] image;
			image = NULL;
			}
		image = new unsigned int[width * height];
		if (image == NULL)
			throw(DiffractionImageException("error allocating image"));
		}


    void DiffractionImage::loadRawImage(string filename,int offset)
		{
		unsigned short* tmpimg;
		// then open the image
		int fd = open(filename.c_str(), MYREAD);
		if (fd < 0)
			throw(DiffractionImageException("error opening raw image"));
	
		// then advance - unfortunately this can only be performed 
		// by reading :o( however, we can read this into the originalHeader
		// entry in the class
		int amount = read(fd, originalHeader, offset);
		if (amount != offset)
			{
			close(fd);
			throw(DiffractionImageException("error reading header"));
			}
		
		// now read the image
		if (image == NULL) // allocate it
			image = new unsigned int[width * height];
		if (image == NULL)
			{
			close(fd);
			throw(DiffractionImageException("error allocating image"));
			}
		tmpimg = new unsigned short[width * height];
		amount = read(fd, (unsigned char *) tmpimg, width * height * 2); // that should be the "big read" so

		close(fd);
		if (amount < width * height * 2)
			throw(DiffractionImageException("error reading raw image"));

		if (!(bigEndian && machineBigEndian()) && (bigEndian || machineBigEndian()))
			{
			// byte swap - this shouldn't be implemented as a 
			// function call as that will be very expensive - perhaps
			// there is a clever way to perform a byte swap
			// using something like 
			// a = ((a & 0xff00) >> 8) & ((a & 0x00ff) << 8);
			for (int i = 0; i < width * height; i++)
				byteSwap(&(tmpimg[i]));
			}
		for (int i = 0; i < width * height; i++)
			image[i]=tmpimg[i];
		free(tmpimg);
		close(fd);
		}

	};
