#include <stdlib.h>
#include <stdio.h>
#include "DiffractionImage.h"
#include "Peak.h"
using namespace DI;

int printHeader(char* filename,bool printgain,bool printaxis)
	{
	float *R,*S,*P;
	DiffractionImage* diff=new DiffractionImage();
	if(printgain)
		diff->load(filename);
	else
		diff->loadHeader(filename);
	printf("Format : %s\n",diff->getFormat());
	printf("Manufacturer : %s\n",diff->getManufacturer());
	printf("Collection date : %s\n",diff->getDate());
	printf("Exposure time : %f s\n",diff->getExposureTime());
	printf("Detector S/N : %s\n",diff->getSerialNo());
	printf("Wavelength : %f Ang\n",diff->getWavelength());
	printf("Beam center : (%f mm,%f mm)\n",diff->getBeamX(),diff->getBeamY());
	printf("Distance to detector : %f mm\n",diff->getDistance());
	printf("Image Size : (%d px, %d px)\n",diff->getWidth(),diff->getHeight());
	printf("Pixel Size : (%f mm, %f mm)\n",diff->getPixelX(),diff->getPixelY());
	printf("Oscillation (%s) : %f -> %f deg\n",diff->getOscAxis(),
	       diff->getOscStart(),diff->getOscEnd());
	printf("Two Theta value: %f deg\n",diff->getTwoTheta());
	if(printgain)
		printf("Estimation of gain: %f\n",diff->gain());
	if(printaxis)
		{
		R=diff->getR();
		S=diff->getS();
		//P=diff->getP();
		printf("R in real space: %.5f %.5f %.5f\n",R[0],R[1],R[2]);
		printf("S in real space: %.5f %.5f %.5f\n",S[0],S[1],S[2]);
		//printf("R in real space: %f\n",diff->gain());
		}
	}

int main (int argc, char *argv[])
	{
	if(argc<2 || argc>4)
		{
		printf("Wrong Number of arguments!\n\n");
		printf("Usage is  'diffdump [-gain] [-axis] <filename>' \n");
		return -1;
		}
	else
		{
		if(argc==2)
			{
			printHeader(argv[1],false,false);
			return 0;
			}
		else
			{
			bool printA,printG;
			printA=false;
			printG=false;
			if(strcmp(argv[1],"-gain")!=0 && strcmp(argv[1],"-axis")!=0)
				{
				printf("Unknown keyword!\n\n");
				printf("Usage is  'diffdump [-gain -axis] <filename>' \n");
				return -1;
				}
			else
				{
				if(argc==4 && (strcmp(argv[2],"-gain")!=0 && strcmp(argv[2],"-axis")!=0))
					{
					printf("Unknown keyword!\n\n");
					printf("Usage is  'diffdump [-gain -axis] <filename>' \n");
					return -1;
					}
				else
					{
					if(strcmp(argv[1],"-gain")==0 || strcmp(argv[2],"-gain")==0)
						printG=true;
					if(strcmp(argv[1],"-axis")==0 || strcmp(argv[2],"-axis")==0)
						printA=true;
					printHeader(argv[2],printG,printA);
					}
				}	
			return 1;
			}
		}
	}
