                                                               /* -*-C-*- */
/* $Id: CCIF_VMS.C,v 1.5 2008/06/09 17:37:54 mdw Exp $ */
#include "stdpccts.h"

/********************************************************************
* Copyright (c) 1995 - 2004, EMBL, Peter Keller
* 
* CCIF: a library to access and output data conforming to the
* CIF (Crystallographic Information File) and mmCIF (Macromolecular
* CIF) specifications, and other related specifications.
* 
* This library is free software: you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* version 3, modified in accordance with the provisions of the 
* license to address the requirements of UK law.
*
* You should have received a copy of the modified GNU Lesser General 
* Public License along with this library.  If not, copies may be 
* downloaded from http://www.ccp4.ac.uk/ccp4license.php
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
********************************************************************/


#if 0

A note on the members of 'struct fibdef' (defined in 
sys$library:fibdef.h [VAX/VMS] or module fibdef in 
sys$library:sys$starlet_c.tlb [AXP/VMS] ).

The bit overlays are defined as:

struct fibdef {
    union  {
        unsigned long int fib$l_acctl;   /* ACCESS CONTROL BITS              */
        struct  {
            .....
           }
        } fib$r_acctl_overlay;
        ....
   };

On the VAX, the bits can be set by assigning something to

   fib.fib$r_acctl_overlay.fib$l_acctl = ... ;      

as you would expect. BUT, under AXP/VMS, the following macros are defined:

#if !defined(__VAXC) && !defined(VAXC) 
#define fib$l_acctl fib$r_acctl_overlay.fib$l_acctl
....
#endif

i.e. the above form results in substitution for fib$l_acctl! (The obvious 
thing to do would have been to use a different name for fib$l_acctl, but 
perhaps that isn't obvious in Colorado....).

SO, for code to work under both systems, every reference to an overlay 
which has been recursively #define'd in this way, has to be encapsulated 
with the same condition :-{

#endif /* 0 */



#ifndef ZZLEXBUFSIZE
#define ZZLEXBUFSIZE    2000
#endif


__export__ __noshare__ MAP_FILE_RECORD *ccif_map_file_list[MAX_MAPPED_FILES];


#define $DESCRIPTOR_NULL(name) struct dsc$descriptor_s name = \
      { 0, DSC$K_DTYPE_T, DSC$K_CLASS_S, NULL }

struct acp$descriptor {
      unsigned short int acp$w_length;
      unsigned short int acp$w_not_used;
      char * acp$a_pointer;
};

static unsigned short int null_fid[3] = {0, 0, 0};  /* use memcmp with this, to identify DECnet files */


/* VMS error message handling */
static char errbuf[257];
static $DESCRIPTOR(errdesc,errbuf);
static unsigned short int errmsglen;
static unsigned char outadr[4];
#define ERR(status, pre)   if ( !(status & 1) ) { \
                              sys$getmsg(status, &errmsglen, &errdesc, 15, outadr); \
                              errbuf[errmsglen] = '\0';  \
                              ccif_signal(VMS_SYSMSG, pre, __FILE__, __LINE__, errbuf); \
                           }

/* Macro to allow strerror to be called, or EXIT_ON_SYS_ERROR to be invoked, after a 
   system/library/RMS etc. error 
   Use this to allow Unix routine clones to have their errors handled by the
   calling routine
*/
#define C_ERR(status)      errno = EVMSERR; vaxc$errno = status;

typedef unsigned long int UL;
typedef unsigned short int US;
typedef unsigned char UC;

typedef struct {
   struct FAB * fab;
   struct NAM * nam;
   struct RAB * rab;
} rms_blocks;

#ifndef __DECC
/* __DECC is defined as 1 with DEC C, even when called with /STANDARD=VAXC
*/

extern long int sys$crmpsc( UL*, UL*, UL, UL, void*, void*, UL, US, UL, 
          UL, UL, UL);
/* OK, it's declared in starlet.h, but without a prototype. $CRMPSC is just
   going to pull stuff off the stack, so the argument type conversion has to be
   done at this end. Declaration of system stuff in VAX C is chronic, so it
   really needs to be done properly.
*/

extern long int sys$expreg( UL, UL*, UL, UL);
#endif

/* Keep number of blocks after doing $CLOSE */
static UL ebk;

static FILE *cifin;

/*
**++
**  FUNCTIONAL DESCRIPTION: ccif_mapfile
**
**	Maps a cif file into memory. If the file has STREAM_LF record format
**	(i.e. equivalent to a standard Unix stream file), the file is mapped
**	directly using $CRMPSC. Otherwise, memory is calloc'd for the file,
**	which is read one record at a time, leading and trailing
**	carriage-control information removed, and the records copied with
**	trailing line-feeds into memory.  
**
**  FORMAL PARAMETERS:
**
**      file:
**          pointer to the name of the file to be mapped.
**
**  RETURN VALUE:
**
**      Pointer to the start of the mapped file or allocated memory
**
**  SIDE EFFECTS:
**
**      Leaves file open on $OPEN if STREAM_LF
**
**
**
**--
*/


/* Fixme! It should be possible to map VFC files with cr carriage control,
 * and use the ANTLRf macro */

char * ccif_mapfile (char * const ioname, FILE **const fileptr, 
                     const CIF_FILE_LIST * const file_entry) {
   long status;
   UL   inadr[2]={ 0ul, 0ul }, /* map file to P0 */
        retadr[2],
        acmode;
   char *mapptr, line[ZZLEXBUFSIZE], *lineptr, *ptr, *filename;
   int i=0, j=0, maxreclen = ZZLEXBUFSIZE - 1, decnet = 0;
   size_t filesize;

   struct FAB infab, mapfab;
   struct XABFHC inxab;
   struct NAM innam;
   char rsa[NAM$C_MAXRSS+1], esa[NAM$C_MAXRSS+1];
   MAP_FILE_RECORD *map_ptr;

   for ( i = 0; i < MAX_MAPPED_FILES; i++ ) {
      if ( ! ccif_map_file_list[i] ) break;
   }

   if ( i >= MAX_MAPPED_FILES ) return (-1);
   

/* If file is a CIF, will already have everything we need in file_entry.
   Assume non-CIF files should just be mapped without checking file structure */



/* Set up FAB and file header XAB for $OPEN */
   infab = cc$rms_fab;
   inxab = cc$rms_xabfhc;
   innam = cc$rms_nam;

   if ( file_entry ) {
      decnet = file_entry->decnet;
      if ( decnet ) {
         return (-2);
      }
      else {
         infab.fab$l_nam = &(file_entry->namblock);
         infab.fab$l_fop |= FAB$M_NAM;  /* Use DVI, DID and FID from file_entry->namblock */
      }
   }
   else {
      infab.fab$l_fna =  ioname;
      infab.fab$b_fns = strlen (ioname);
      infab.fab$l_nam = &innam;

      innam.nam$l_rsa = rsa;
      innam.nam$b_rss = NAM$C_MAXRSS;
      innam.nam$l_esa = esa;
      innam.nam$b_ess = NAM$C_MAXRSS;
   }

   infab.fab$b_fac = FAB$M_GET;  /* Read access is the default, but it
                                    doesn't do any harm to make it explicit */
   infab.fab$b_shr = FAB$M_SHRGET; /* Allow shared read access */
   infab.fab$l_xab = (char *) &inxab;

   inxab.xab$l_nxt = NULL;

/* Do it... */
   status = sys$open (&infab);
   ERR(status,"ccif_mapfile: $OPEN:")

   if ( !file_entry ) {
      rsa[innam.nam$b_rsl] = '\0';
      decnet = !memcmp(innam.nam$w_fid, null_fid, sizeof null_fid);
   }

/* Keep number of pages (VMS) / pagelets (AXP) of data */
   ebk = inxab.xab$l_ebk;

/* Length of data in bytes */
   filesize = (inxab.xab$l_ebk - 1) * 512 + inxab.xab$w_ffb;


   if ( infab.fab$b_org != FAB$C_SEQ ) 
     ccif_signal(RMS_NOTSEQ, "ccif_mapfile", NULL, NULL, 0, rsa);



   switch ( infab.fab$b_rfm ) {

    case ( FAB$C_STMCR ):
    case ( FAB$C_STMLF ):

     infab.fab$l_nam = NULL;    /* Preserve nameblock */
     infab.fab$l_xab = NULL;    /* and XAB on close */
     status = sys$close(&infab);
     ERR(status, "ccif_mapfile: $CLOSE: ")

      if ( ! decnet ) { 

         if ( *fileptr ) {
            fclose (*fileptr);
	    *fileptr = NULL;
         }
      
        /* Re-open file in UFO mode for mapping */
        mapfab = cc$rms_fab;
        mapfab.fab$b_fac = FAB$M_GET;
        mapfab.fab$l_fop = FAB$M_UFO | FAB$M_NAM; /* No rms processing; use file/device info */
        mapfab.fab$l_nam = file_entry ? &(file_entry->namblock) : &innam; 
        mapfab.fab$b_shr = FAB$M_SHRGET;           /* Shared access for read */
        status = sys$open(&mapfab);
        ERR(status, "ccif_mapfile: $OPEN(UFO)")
        

        if ( file_entry ) {   /* a CIF */

           /* This bit is a real drag - if CIF data is an exact multiple of 512 bytes,
              must make sure that space for mapping has an extra page at the end.
              This is because DLG needs run-off at the end of the data.
              $CRMPSC does not give the full number of requested pages if the file is shorter
           */
	  status = sys$expreg(inxab.xab$l_ebk, inadr, 0, 0); /* Sets inadr for $CRMPSC */
	  ERR(status, "ccif_mapfile: $EXPREG: ")

	  status = sys$crmpsc(inadr, retadr, PSL$C_USER, 
			      SEC$M_CRF | SEC$M_WRT , 
			      NULL, NULL , 0, mapfab.fab$l_stv, inxab.xab$l_ebk, 0, 0, 0);
	  ERR(status, "ccif_mapfile: $CRMPSC: ")
	    
	  mapptr = (char *) retadr[0];
	  mapptr[filesize] = '\0';    /* For DLG */

        }
        else {   /* Not a CIF */
           status = sys$crmpsc(inadr, retadr, PSL$C_USER, 
	       SEC$M_EXPREG | SEC$M_CRF | SEC$M_WRT , 
	       NULL, NULL , 0, mapfab.fab$l_stv, inxab.xab$l_ebk, 0, 0, 0);
           ERR(status, "ccif_mapfile: $CRMPSC: ")

           mapptr = (char *) retadr[0];

        }
	   break;

    }  /* if ( ! decnet ) */



    case ( FAB$C_VFC ):
    case ( FAB$C_VAR ):
	if ( infab.fab$b_rat == 0 ) 
	  ccif_signal(RMS_NOCRINFO, "ccif_mapfile", NULL, NULL, 0, rsa);

    case ( FAB$C_FIX ):
    case ( FAB$C_STM ):


	if (inxab.xab$w_lrl > ZZLEXBUFSIZE-1) 
	  ccif_signal(RMS_RECTOOLONG, "ccif_mapfile", NULL, NULL, 0, rsa);

         if ( file_entry )
	    return (-2);   /* send back for stream I/O if a CIF, otherwise assume that it
	                      is something like MMCIFDIC.LIB, which should be block-read into
			      memory regardless of record format */

        if ( infab.fab$w_ifi ) {
           infab.fab$l_nam = NULL;    /* Preserve nameblock */
           infab.fab$l_xab = NULL;    /* Preserve XAB */
           status = sys$close(&infab);
           ERR(status, "ccif_mapfile: $CLOSE: ")
        }

        if ( ! *fileptr )
           *fileptr = fopen(ioname, "rb");
        if ( !*fileptr )
           EXIT_ON_SYS_ERROR("ccif_mapfile", "Calling fopen (mode rb)");
	CCIF_MALLOC(filesize, char, mapptr, "ccif_mapfile", (0))
        fread (mapptr, 1, filesize, *fileptr);
        if ( ferror(*fileptr) )
           EXIT_ON_SYS_ERROR("ccif_mapfile", "Calling fread")

	break;

    default:
        ccif_signal(RMS_NORECTYPE, "ccif_mapfile", NULL, NULL, 0, rsa);

    }


    CCIF_CALLOC(1, MAP_FILE_RECORD, map_ptr, "ccif_mapfile", 0)
    map_ptr->start = (char *) mapptr;
    map_ptr->length = filesize;
    filename = file_entry ? file_entry->namblock.nam$l_rsa : rsa ;
    map_ptr->filename = ccif_strdup(filename);

    if ( ! *fileptr ) {
#ifdef DEBUG
       printf("\n\n Mapping file %s on channel %d\n\n", filename, mapfab.fab$l_stv);
#endif
       map_ptr->channel = (unsigned short int) mapfab.fab$l_stv;
       map_ptr->end = (char *) inadr[1] ? inadr[1] : retadr[1];  /* Include last page
                                                                    for $DELTVA */
       map_ptr->mapped = 1;
    }

    ccif_map_file_list[i] = map_ptr;

    return (i);
}

#if 0
void closecif (void){
    if (mapped)
	sys$close(&infab);
    else {
	if (!cifin) {
	    fclose(cifin);
	    cifin = NULL;
	}
   }
}
    
#endif


void ccif_exit_c_rtl_error(char * pre, char * msg) {
   
   if ( errno == EVMSERR )
      ccif_signal(VMS_CRTL, pre, NULL, NULL, 0, msg, strerror(errno, vaxc$errno) );
   else
     ccif_signal(VMS_CRTL, pre, NULL, NULL, 0, msg, strerror(errno));
}  



/* Return RMS nameblock of a file.
   retnam->nam$l_rsa, retnam->nam$b_rss, retnam->nam$l_esa and retnam->nam$b_ess  
   MUST be set appropriately to receive resultant name and expanded name
   (i.e. point to a buffer of at least NAM$C_MAXRSS+1)
   Expanded name is returned with DECNET password intact; resultant name (result of
   actual lookup in directory) has DECNET password changed to "password".

   Careless use of the returned nameblock with the $OPEN service, may cause
   the resultant and expanded names to be clobbered. For DECNET files, cannot use
   this nameblock at all for $OPEN - just set fab$l_fna/fab$b_fns with the expanded
   strings. For non-DECNET files, make sure that the FAB$M_NAM bit of fab$l_fop is
   set.

   Use the C_ERR macro here, because we are treating it like Unix stat(). stat() won't
   work with DECnet files, but this routine will return the FID's in the name block
   for non-network files, so we don't need both this routine and stat() .
*/

int ccif_get_nameblock(const char * const filename, struct NAM *retnam) {

   struct FAB infab;
   struct NAM innam;
   char esatmp[NAM$C_MAXRSS+1], rsatmp[NAM$C_MAXRSS+1];
   long status;

   infab = cc$rms_fab;
   innam = cc$rms_nam;

/*  Set up FAB and NAM for $PARSE */

   infab.fab$l_fna = (char *) filename;
   infab.fab$b_fns = strlen(filename);
   infab.fab$l_nam = &innam;
   infab.fab$w_ifi = 0;

   innam.nam$l_esa = retnam->nam$l_esa;
   innam.nam$b_ess = retnam->nam$b_ess;
   innam.nam$b_nop |= NAM$M_PWD ; /* ( 1 << NAM$V_PWD ) */ /* Don't conceal password */

/* Initialise data structures */
   status = sys$parse(&infab);
   if ( !(status & 1) ) {
      C_ERR(status)
      return (-1);
   }

/* Get resultant file name */
   innam.nam$b_nop = NAM$M_NOCONCEAL;  /* | NAM$C_SRCHXABS ? May need this for DECnet files.
                                        * N.B. - not using |= operator, so clears previous
                                        * NAM$M_PWD setting */
   innam.nam$l_rsa = retnam->nam$l_rsa;
   innam.nam$b_rss = retnam->nam$b_rss;

   status = sys$search(&infab);
/* 'C' string terminator (this is why there should be NAM$C_MAXRSS+1 bytes in the buffer */
   if (innam.nam$b_rsl > 0) innam.nam$l_rsa[innam.nam$b_rsl] = '\0';
   if (innam.nam$b_esl > 0) innam.nam$l_esa[innam.nam$b_esl] = '\0';
   if ( status == RMS$_NMF || status == RMS$_FNF ) {    /* No files found - simulate Unix stat() return condition */
      errno = ENOENT;
      return (-1);
   }

   if ( !(status & 1) ) {
      C_ERR(status)
      return (-1);
   }

/* Save NAM block, and release data structures set up by $PARSE */
   *retnam = innam;
   innam.nam$l_rsa = rsatmp;  /* Don't allow retnam->nam$l_rsa to be clobbered */
   innam.nam$b_rss = NAM$C_MAXRSS;
   innam.nam$l_esa = esatmp;  /* Don't allow retnam->nam$l_esa to be clobbered */
   innam.nam$b_ess = NAM$C_MAXRSS;
   innam.nam$b_nop = NAM$M_SYNCHK;
   infab.fab$b_dns = 0;
   innam.nam$l_rlf = NULL;

   status = sys$parse(&infab);
   if ( !(status & 1) ) {
      C_ERR(status)
      return (-1);
   }

   return (0);
}

/* getenv()-style function to return resultant string name of a filename */
char * ccif_logname2filename(const char * const logname) {

   struct NAM innam = cc$rms_nam;
   char rsa[NAM$C_MAXRSS+1], esa[NAM$C_MAXRSS+1], *filename = getenv(logname);
   int status;

   if ( !filename ) 
      return (NULL);

   innam.nam$l_rsa = rsa;
   innam.nam$b_rss = NAM$C_MAXRSS;
   innam.nam$l_esa = esa;
   innam.nam$b_ess = NAM$C_MAXRSS;

   status = ccif_get_nameblock(filename, &innam);
   if ( status && errno != ENOENT ) {
     sprintf(errmsg, "Cannot get nameblock of file %s", filename);
     EXIT_ON_SYS_ERROR("ccif_logname2filename",errmsg)
   }

#ifdef NO_RTL_GETENV
   free(filename);
#endif
   return (ccif_strdup(rsa));
}

/* Routine to set the record attributes and file organisation to appropriate values
   (i.e. those which will allow convenient mapping of the output file).

   After much thought, I decided to use the fiddly, but documented, QIO services to
   set these, rather than the easier, undocumented RMS $MODIFY service.

   Be careful about using this routine with un-truncated files.

   This will not work with DECnet files, but we are using nam$l_rsa to compare those,
   so it is not important to preserve the FID.
*/


int ccif_setup_output_file(const char * const filename) {

   long status;
   $DESCRIPTOR_NULL(devnam);
   int decnet;


   struct FAB infab;
   struct NAM qionam;
   char qiorsa[NAM$C_MAXRSS];

   struct FAT infat;
   struct atrdef inatr[2];
   struct fibdef infib;
   struct acp$descriptor fib_desc = {sizeof(struct fibdef), 0, &infib};
   unsigned short int qiochan;


/* Set up structures for QIO operations now */
   memset(&infib, 0, sizeof infib);                             /* Initialise file information block */

#if ( defined (__VAXC) || defined (VAXC) ) && !defined (__DECC)
   infib.fib$r_acctl_overlay.fib$l_acctl = FIB$M_WRITE;         /* Write access */
#else
   infib.fib$l_acctl = FIB$M_WRITE;          /* Write access */
#endif

   inatr[0].atr$w_size = sizeof infat;                           /* Request file attribute information */
   inatr[0].atr$w_type = ATR$C_RECATTR;
   inatr[0].atr$l_addr = &infat;
   inatr[1].atr$w_size = 0;                                      /* Zero longword terminator */
   inatr[1].atr$w_type = 0;



   /* Use RMS services to find FID of file and device name. Could use QIO, but I don't
      hate myself that much */

   infab = cc$rms_fab;
   qionam = cc$rms_nam;

   infab.fab$l_fna = (char *) filename;
   infab.fab$b_fns = strlen(filename);
   infab.fab$b_fac = FAB$M_GET;
   infab.fab$l_nam = &qionam;
   infab.fab$l_xab = NULL;

   qionam.nam$b_nop = NAM$M_NOCONCEAL;
   qionam.nam$b_ess = 0;
   qionam.nam$l_esa = NULL;
   qionam.nam$b_rss = NAM$C_MAXRSS;
   qionam.nam$l_rsa = qiorsa;

   status = sys$open(&infab);
   ERR(status, "ccif_setup_output_file: ")

/* Set up a descriptor to the device name, ready for $ASSIGN */
   devnam.dsc$a_pointer = qionam.nam$l_dev;
   devnam.dsc$w_length = qionam.nam$b_dev;

/* Keep FID for QIO functions */
#if ( defined (__VAXC) || defined (VAXC) ) && !defined (__DECC)
   infib.fib$r_fid_overlay.fib$w_fid[0] = qionam.nam$w_fid[0];
   infib.fib$r_fid_overlay.fib$w_fid[1] = qionam.nam$w_fid[1];
   infib.fib$r_fid_overlay.fib$w_fid[2] = qionam.nam$w_fid[2];
#else
   infib.fib$w_fid[0] = qionam.nam$w_fid[0];
   infib.fib$w_fid[1] = qionam.nam$w_fid[1];
   infib.fib$w_fid[2] = qionam.nam$w_fid[2];
#endif

   status = sys$close(&infab);
   ERR(status, "ccif_setup_output_file: ")


/* Open an I/O channel to device */
   status = sys$assign(&devnam, &qiochan, 0L, NULL);
   ERR(status, "ccif_setup_output_file: ")



   /* Access file. IO$_ACCESS does a read attributes when P5 is not NULL
      fib_desc has been initialised to refer to infib */
   status = sys$qiow(0L, (unsigned long int) qiochan, IO$_ACCESS|IO$M_ACCESS,
                      NULL, NULL, 0L, &fib_desc, NULL, NULL, NULL, &inatr, NULL);
   ERR(status, "ccif_setup_output_file: ")

   /* Set new file/record type information */
   infat.fat$v_rtype = FAT$C_STREAMLF;
   infat.fat$v_fileorg = FAT$C_SEQUENTIAL;
   infat.fat$b_rattrib =  FAT$M_IMPLIEDCC;
   infat.fat$w_rsize = 0;
   infat.fat$w_maxrec = 0;

   /* Deacess file. IO$_DEACCESS does a write attributes when P5 is not NULL. */
   status = sys$qiow(0L, (unsigned long int) qiochan, IO$_DEACCESS,
                      NULL, NULL, 0L, &fib_desc, NULL, NULL, NULL, &inatr, NULL);
   ERR(status, "ccif_setup_output_file: ")

   /* Deassign I/O channel to device */
   status = sys$dassgn(qiochan);
   ERR(status, "ccif_setup_output_file: ")

   return (0);
}

   
/* Are we attached to a terminal on device? Can't be an exact clone of
 * U**x's isatty - the getdvi call isn't general enough */
   
   
int devtty(const char * const device) {
   
   
   unsigned short int length;

   $DESCRIPTOR(devnam, device);
   unsigned long int devchar;
   
   struct {
      unsigned short int buflen;
      unsigned short int itmcode;
      unsigned long int bufadr;
      unsigned long int retadr;
      unsigned long int last;
   } item = { sizeof devchar, DVI$_TRM, &devchar, &length, 0L }; 

   /* The $DESCRIPTOR macro gets this wrong in the particular
      case where the string is a pointer to char passed as
      a function argument :-{  */
   devnam.dsc$w_length = strlen(device);

   sys$getdvi(0, 0, &devnam, &item, 0, 0, 0, 0);
   return ( devchar != 0 );
}


   
   
   

/* VMS emulations of various Unix routines */

#ifdef NO_RTL_GETENV

char * getenv ( const char * namenv ) 
{

/* N.B. - like its Unix equivalent, this will not work if namenv
   has leading or trailing whitespace.
*/

   unsigned long int ierr, attr=LNM$M_CASE_BLIND;
   unsigned char acmode=PSL$C_USER;
   auto $DESCRIPTOR(tabnam, "LNM$PROCESS_TABLE"); 

   unsigned long int length_l;
   unsigned short int length_s;
   char *valenv;

/* Cannot do this:
   auto $DESCRIPTOR(lognam, namenv);
   because sizeof(namenv) as used in sys$library:descrip.h 
   doesn't count the null byte :-{
*/

   struct dsc$descriptor_s lognam;
   struct lnm$_itmlist {
      unsigned short int buf_len, item_code;
      unsigned long int buf_addr, len_addr;
   } itemlist[2];
 

   lognam.dsc$a_pointer = (char *) namenv;
   lognam.dsc$w_length = strlen(namenv);
   lognam.dsc$b_class = DSC$K_CLASS_S;
   lognam.dsc$b_dtype = DSC$K_DTYPE_T;


/* Check for existence of name first, and return length */
   itemlist[0].buf_len = sizeof(length_l);
   itemlist[0].item_code = LNM$_LENGTH;
   itemlist[0].buf_addr = (unsigned long int) &length_l;
   itemlist[0].len_addr = (unsigned long int) NULL;

   itemlist[1].buf_len = ( itemlist[1].item_code = 0) ; /* Terminator */


   ierr = sys$trnlnm( &attr, &tabnam, &lognam, &acmode, itemlist);

   if ( ierr == SS$_NOLOGNAM ) /* Not found */
      return NULL;

   ERR(ierr, "getenv: $TRNLNM:")

/* Now, allocate buffer for translation, and find name */

     CCIF_CALLOC(length_l + 1, char, valenv, "getenv(CCIF_VMS.C)", (0))

   itemlist[0].buf_len = length_l;
   itemlist[0].item_code = LNM$_STRING;
   itemlist[0].buf_addr = (unsigned long int) valenv;
   itemlist[0].len_addr = (unsigned long int) &length_s;

   ierr = sys$trnlnm( &attr, &tabnam, &lognam, &acmode, itemlist);
   
   ERR(ierr, "getenv: $TRNLNM: ")

   if ( length_s == length_l )  /* consistency, although $trnlnm should really
                                 return SS$_BUFFEROVF if there is a problem here */
	return valenv;  /* already null-terminated from calloc */
   ccif_signal(VMS_TRNLNM, "getenv", NULL, NULL, 0);

}


#endif /* NO_RTL_GETENV */

/* Note that this is not a true clone of Unix unlink, since it takes no
 * note of other references (logical names, etc) to the file */

int unlink (const char *path) {

   struct FAB infab;
   long status;


   /* Set up FAB for $OPEN */
      infab = cc$rms_fab;
      
      infab.fab$l_fna = (char *) path;
      infab.fab$b_fns = strlen (path);
      infab.fab$l_fop |= FAB$M_DLT  /* Delete on close */;
      infab.fab$l_xab = NULL;
   
      status = sys$open (&infab);
      if ( !(status & 1) ) {
         C_ERR(status)
         return (-1);
      }

      status = sys$close (&infab);
      if ( !(status & 1) ) {
         C_ERR(status)
         return (-1);
      }

      return (0);
}

#ifdef __DECC
static int truncate_callback ( rms_blocks *, struct FAB *, struct RAB *); 
#endif

int truncate(const char *path, long int offset) {

   struct FAB infab;
   struct RAB inrab;
   unsigned short int rfa[3];
   long int status;
   int fd;
   rms_blocks block_ptr;

#ifdef __DECC
   fd = open(path, O_RDONLY, 0, "acc", truncate_callback, &block_ptr);
   if ( fd == -1 ) return (-1);
#else
   fd = open(path, O_RDONLY, 0);
   if ( fd == -1 ) return (-1);
   _fstat(fd, &block_ptr.fab, &block_ptr.nam, &block_ptr.rab);
#endif

   lseek(fd, offset, SEEK_SET);

   memcpy(rfa, block_ptr.rab->rab$w_rfa, 3*sizeof(unsigned short int) );
   close(fd);


   /* Set up FAB for $OPEN */
      infab = cc$rms_fab;
      
      infab.fab$l_fna = (char *) path;
      infab.fab$b_fns = strlen (path);
      infab.fab$b_fac = FAB$M_TRN; /* truncate access */
      infab.fab$l_xab = NULL;
   
      status = sys$open (&infab);
      if ( !(status & 1) ) {
         C_ERR(status)
         return (-1);
      }
      
      inrab = cc$rms_rab;
      inrab.rab$l_fab = &infab;
      inrab.rab$w_isi = 0;

      status = sys$connect (&inrab);
      if ( !(status & 1) ) {
         C_ERR(status)
         return (-1);
      }

      inrab.rab$b_rac = RAB$C_RFA;
      memcpy(inrab.rab$w_rfa, rfa, 3*sizeof(unsigned short int) );
      status = sys$find (&inrab);
      if ( !(status & 1) ) { 
        if ( status != RMS$_EOF ) {
           C_ERR(status)
           return (-1);
        }
      }
      else {

         inrab.rab$l_rop = RAB$M_ASY; /* Truncate must be done asynchronously... */
         status = sys$truncate (&inrab);
         if ( !(status & 1) ) {
            C_ERR(status)
            return (-1);
         }

         status = sys$wait(&inrab);  /* ... so wait until it has finished */
         if ( !(status & 1) ) {
            C_ERR(status)
            return (-1);
         }

      } /* if ( !(status & 1) )  ($FIND) */


   infab.fab$l_fop = FAB$M_TEF;   /* Drop excess allocated blocks */
   status = sys$close (&infab); 
   if ( !(status & 1) ) {
      C_ERR(status)
      return (-1);
   }

   return (0);
}


#ifdef __DECC
int truncate_callback ( rms_blocks * block_ptr, 
                                  struct FAB * fab, struct RAB * rab) {
   block_ptr->fab = fab;
   block_ptr->rab = rab;
   block_ptr->nam = (struct NAM *) fab->fab$l_nam;   
   return (0);
      
}
#endif

/* This is not called munmap, since it has a different parameter list to
   Unix's munmap, but it does more-or-less the same. It also deassigns the
   I/O channel (equivalent to closing the file under Unix), but CCIF opens
   a new channel for each mapped file, so there is no point in keeping it open
*/

int ccif_vms_munmap( char ** const addr, 
                        const unsigned short int channel) {


   long status;
 
   unsigned long int **inadr, retadr[2], acmode=0;

   inadr = (unsigned long int **) addr; 

   status = sys$deltva( *inadr,retadr,acmode );
   if ( !(status & 1) ) {
      C_ERR(status);
      return (-1);
   }

   status = sys$dassgn(channel);
   if ( !(status & 1) ) {
      C_ERR(status);
      return (-1);
   }

   return (0);
}
 


