dnl $Id: aclocal.m4,v 1.2 2005/09/08 20:21:40 mdw Exp $

dnl ####################################################################
dnl 
dnl  Copyright (c) 1995 - 2004, EMBL, Peter Keller
dnl  
dnl  CCIF: a library to access and output data conforming to the
dnl  CIF (Crystallographic Information File) and mmCIF (Macromolecular
dnl  CIF) specifications, and other related specifications.
dnl  
dnl  Where the terms and conditions of the CCP4 licence agreement apply
dnl  to this code (including, but not limited to, its acquisition as a
dnl  component of the CCP4 distribution and/or its use in building an
dnl  executable program that uses other CCP4 components), this code is
dnl  distributed as a CCP4 Library.
dnl  
dnl ####################################################################

AC_DEFUN(CCIF_HEADER_UNISTD_ABSENT, [

dnl The form of this check is CCP4 practice, from $CLIBS/library.h, rather 
dnl than standard Autoconf-style stuff.
AC_CACHE_CHECK(for unistd.h,
               ccif_cv_header_unistd_h,
               [AC_CHECK_HEADER(unistd.h,
                                ccif_cv_header_unistd_h='yes',
                                ccif_cv_header_unistd_h='no'
                               )
               ]
              )
if test "$ccif_cv_header_unistd_h" = "no" ; then
   AC_DEFINE(NOUNISTD)
fi
])

AC_DEFUN(CCIF_FUNC_STRERROR_NEEDED, [
dnl We use TRY_RUN rather than TRY_LINK, because old linkers don't always return an 
dnl error status for an undefined symbol.
AC_CACHE_CHECK(whether we need to define strerror, 
               ccif_cv_func_strerror_needed,
               [AC_RUN_IFELSE(
		[AC_LANG_PROGRAM([[#include <string.h>]],
                                 [[strerror(1); 
                                   exit(0);]])],
                           ccif_cv_func_strerror_needed='no',
                           ccif_cv_func_strerror_needed='yes'
                          )
               ]
              )

if test "$ccif_cv_func_strerror_needed" = "yes" ; then
   AC_DEFINE(NEED_STRERROR)
fi
])


AC_DEFUN(CCIF_STRDUP_DECL, [
dnl This check works by trying to link a program with a declaration of strdup
dnl that is incompatible with the proper one, if it exists.
dnl This is necessary because the Xopen/UNIX95/UNIX98 stuff doesn't allow strdup,
dnl even though most rtl's have it.
dnl WARNING: this check assumes that the linker will find strdup, even if
dnl the compiler is unable to find a declaration of it!
AC_CACHE_CHECK(whether strdup declared in string.h,
               ccif_cv_strdup_decl,
               [AC_LINK_IFELSE(
		[AC_LANG_PROGRAM([[#include <string.h>
                                   float *strdup (const char *s1);]],
                                 [[ float *f = strdup("hello");]])],
                            ccif_cv_strdup_decl='no',
                            ccif_cv_strdup_decl='yes'
                           )
               ]
             )
if test "$ccif_cv_strdup_decl" = "yes" ; then
   AC_DEFINE(STRDUP_DECL)
fi
])

AC_DEFUN(CCIF_FUNC_REALLOC_OLD, [

dnl Checks whether:
dnl    ptr1 = NULL; ptr2 = realloc(ptr1, size);
dnl is equivalent to:
dnl    ptr2 = malloc(size);
dnl
dnl I have seen this test fail under two circumstances:
dnl  (1) ESV: realloc only works properly when ptr1 != NULL
dnl  (2) VAX C: optimized programs fail when ptr1 == NULL
dnl      as the optimizer expects a literal 'NULL' as the first
dnl      parameter in this case. To be fair, this is documented
dnl      in the VAX C manual. This is not a problem for DEC C, 
dnl      of course.


AC_CACHE_CHECK(for old-style realloc,
               ccif_cv_func_realloc_old,
               [AC_RUN_IFELSE(
		[AC_LANG_PROGRAM([[ ]], 
				 [[char *ptr1=0, *ptr2=0;
                                   ptr1 = (char *) realloc(ptr2, 1024);
                                   exit (ptr1==ptr2);  /* exit(0) => realloc is standard */ ]]
				)],
                           ccif_cv_func_realloc_old='no',
                           ccif_cv_func_realloc_old='yes'
                          )
               ]
              )

if test "$ccif_cv_func_realloc_old" = "yes" ; then
   AC_DEFINE(OLD_REALLOC)
fi
])


AC_DEFUN(CCIF_FUNC_SPRINTF_OLDBSD, [
AC_CACHE_CHECK(for old BSD sprintf,
               ccif_cv_func_sprintf_oldbsd,
               [AC_RUN_IFELSE(
		[AC_LANG_PROGRAM([[#include <stdio.h>]],
                                 [[char buf[50]; int res;
                                   res = sprintf(buf, "%d" ,20);
                                   exit( buf != (char *) res );]]
				)],
                           ccif_cv_func_sprintf_oldbsd='yes',
                           ccif_cv_func_sprintf_oldbsd='no'
                          )
               ]
              )

if test "$ccif_cv_func_sprintf_oldbsd" = "yes" ; then
   AC_DEFINE(OLD_SPRINTF)
fi
])



dnl Some systems don't seem to support mapping to specified areas
dnl of memory, but are quite happy to find a suitable area and return
dnl it to the program (i.e. don't use the MAP_FIXED option to mmap).
dnl This test checks this behaviour. In this case, the dependence on 
dnl getpagesize and valloc are internal to the RTL, so don't have to be
dnl checked for by the application programmer.

dnl Another moan with certain systems, is that they don't distinguish
dnl between language constructs, and system/library stuff. So to enable
dnl certain things (like mmap, POSIX stuff), it is necessary to compile
dnl in an extended mode, which will accept non-ANSI constructs (the
dnl extensions are system dependent). This compilation mode is, in general,
dnl not appropriate for generating portable applications. The only way 
dnl around this, is to manually define the various symbols which enable
dnl the extended library stuff.

dnl Silicon Graphics is exemplary in providing three options (-ansi, 
dnl -ansiposix, -xansi) which deal with these things cleanly. Others 
dnl need _POSIX_SOURCE (or something similar) defined, or this test 
dnl will fail with strict ANSI compiliation, when it should succeed.

AC_DEFUN(CCIF_FUNC_MMAP_NONFIXED,[
AC_CHECK_HEADERS(standards.h)
AC_CACHE_CHECK(for working non-fixed mmap, ccif_cv_func_mmap_nonfixed,
[AC_RUN_IFELSE(
 [AC_LANG_PROGRAM(
[[
/* Adapted from autoconf's standard HAVE_MMAP test in acspecific.m4 */
#ifdef HAVE_STANDARDS_H
#include <standards.h>   /* Always a good idea under AIX */
#endif

#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE
#endif

#ifndef _HPUX_SOURCE
#define _HPUX_SOURCE
#endif

#include <sys/types.h>
#include <fcntl.h>
#include <sys/mman.h>

#ifdef __cplusplus
extern "C" { *malloc(unsigned); }
#else
char  *malloc();
#endif


/* Systems which don't define MAP_VARIABLE (e.g. IRIX) seem to assume
   it by default, if they support this behaviour at all */
#ifndef MAP_VARIABLE
#define MAP_VARIABLE 0
#endif
]],
[[
  char *buf1, *buf2, *buf3;
  int i=4096, /* Used to be getpagesize() - now anything sensible is
                 appropriate */
       j;
  int i2 = i * 2;
  int fd;

  buf1 = (char *)malloc(i2);
  buf2 = 0;
  buf3 = (char *)malloc(i2);

  for (j = 0; j < i2; ++j)
    *(buf1 + j) = rand();
  fd = open("conftestmmap", O_CREAT | O_RDWR, 0666);
  write(fd, buf1, i2);
  buf2 = mmap(0, i, PROT_READ | PROT_WRITE, MAP_VARIABLE | MAP_PRIVATE, fd, 0);
  for (j = 0; j < i; ++j)
    if (*(buf1 + j) != *(buf2 + j))
      exit(1);
  lseek(fd, (long)i, 0);
  read(fd, buf2, i); /* read into mapped memory -- file should not change */
  /* (it does in i386 SVR4.0 - Jim Avera, jima@netcom.com) */
  lseek(fd, (long)0, 0);
  read(fd, buf3, i2);
  for (j = 0; j < i2; ++j)
    if (*(buf1 + j) != *(buf3 + j))
      exit(1);
  exit(0);

]])], ccif_cv_func_mmap_nonfixed=yes, ccif_cv_func_mmap_nonfixed=no, 
                                  ccif_cv_func_mmap_nonfixed=no)])

if test $ccif_cv_func_mmap_nonfixed = yes; then
  AC_DEFINE(HAVE_MMAP)
fi
])

# CCIF_ANTLR_USE_PROTOS.
# Essentially the same as the standard macro AC_C_PROTOTYPES, but
# defines __USE_PROTOS as well.
# Requires a new test, becuase the AC_C_PROTOTYPES does not set anything
# that we can check for later
# ---------------
# Check if the C compiler supports prototypes, included if it needs
# options.
AC_DEFUN([CCIF_ANTLR_USE_PROTOS],
[AC_REQUIRE([AC_PROG_CC])dnl
AC_MSG_CHECKING([for function prototypes])
if test "$ac_cv_prog_cc_stdc" != no; then
  AC_MSG_RESULT([yes])
  AC_DEFINE(PROTOTYPES, 1,
            [Define to 1 if the C compiler supports function prototypes.])
  AC_DEFINE(__PROTOTYPES, 1,
            [Define like PROTOTYPES; this can be used by system headers.])
  AC_DEFINE(__USE_PROTOS, 1,
            [Preprocessor symbol used by ANTLR code to indicate support for prototypes])
else
  AC_MSG_RESULT([no])
fi
])# CCIF_ANTLR_USE_PROTOS

