(*
 * Copyright (c) 1997-1999, 2003 Massachusetts Institute of Technology
 *
 * See the file COPYING for license information.
 *
 *)

(* $Id: expr.ml,v 1.1 2003/08/18 08:57:31 mdw Exp $ *)

(* Here, we define the data type encapsulating a symbolic arithmetic
   expression, and provide some routines for manipulating it.  (See
   also simplify.ml for functions to do symbolic simplifications.) *)

type expr =
    Num of Number.number
  | Var of Variable.variable
  | Plus of expr list
  | Times of expr * expr
  | Uminus of expr
  | Integer of int

type assignment = Assign of Variable.variable * expr

let rec find_vars x =
  match x with
  | Var y -> [y]
  | Plus l -> List.flatten (List.map find_vars l)
  | Times (a, b) -> (find_vars a) @ (find_vars b)
  | Uminus a -> find_vars a
  | _ -> []
