/*
 * Copyright (c) 1997-1999, 2003 Massachusetts Institute of Technology
 *
 * See the file COPYING for license information.
 *
 */

#ifndef FFTW_F77_MPI_H
#define FFTW_F77_MPI_H

#include "fftw_mpi.h"
#include "fftw-int.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/***********************************************************************/

/* How do we pass MPI_Comm data types from Fortran?  Here are three
   possibilities, selected among by defining the appropriate cpp
   symbol (in fftw/config.h, preferably using the configure script):

   HAVE_MPI_COMM_F2C -- the MPI_Comm_f2c function is available
                        (this function is supplied e.g. by MPICH)

   FFTW_USE_F77_MPI_COMM -- Fortran gives us an (MPI_Comm *)

   FFTW_USE_F77_MPI_COMM_P -- MPI_Comm is a pointer, and Fortran
                              passes it to us directly by value (seems
			      unlikely).

   [default] -- ignore the comm parameter, and just use MPI_COMM_WORLD
                (this at least will always work, at the expense of flexibility)
*/

#if defined(HAVE_MPI_COMM_F2C)
#  define FFTW_MPI_COMM_F2C(comm) MPI_Comm_f2c(*((MPI_Comm *) comm))
#elif defined(FFTW_USE_F77_MPI_COMM)
#  define FFTW_MPI_COMM_F2C(comm) (* ((MPI_Comm *) comm))
#elif defined(FFTW_USE_F77_MPI_COMM_P)
#  define FFTW_MPI_COMM_F2C(comm) ((MPI_Comm) comm)
#else
#  define FFTW_MPI_COMM_F2C(comm) MPI_COMM_WORLD
#endif

/***********************************************************************/

#ifdef __cplusplus
} /* extern "C" */
#endif /* __cplusplus */

#endif /* FFTW_F77_MPI_H */
