//  $Id: machine_.cpp,v 1.14 2008/06/23 16:55:30 mdw Exp $
//  =================================================================
//
//   CCP4 Coordinate Library: support of coordinate-related
//   functionality in protein crystallography applications.
//
//   Copyright (C) Eugene Krissinel 2000-2008.
//
//    This library is free software: you can redistribute it and/or 
//    modify it under the terms of the GNU Lesser General Public 
//    License version 3, modified in accordance with the provisions 
//    of the license to address the requirements of UK law.
//
//    You should have received a copy of the modified GNU Lesser 
//    General Public License along with this library. If not, copies 
//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//  =================================================================
//
//    23.10.00   <--  Date of Last Modification.
//                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  -----------------------------------------------------------------
//
//  **** Module  :   machine_  <implementation>
//       ~~~~~~~~~
//  **** Functions : GetMachineID   - returns ID code for the machine
//       ~~~~~~~~~~~ GetMachineName - returns name of a machine
//
//  (C) E. Krissinel 2000-2008
//
//  =================================================================
//

#ifndef  __Machine__
#include "machine_.h"
#endif


#ifdef CALL_LIKE_SUN

int GetMachineID()  {
int k = CALL_LIKE_SUN;
  switch (k)  {
    case 1  : return MACHINE_ALLIANT;
    case 2  : return MACHINE_CONVEX;
    case 3  : return MACHINE_ESV;
    case 4  : return MACHINE_SGI;
    case 5  : return MACHINE_SOLBOURNE;
    case 6  : return MACHINE_SOLARIS;
    case 7  : return MACHINE_ALPHA;
    case 8  : return MACHINE_F2C_G77;
    case 9  : return MACHINE_LINUX;
    default : return MACHINE_UNKNOWN;
  }
}

#elif defined(CALL_LIKE_HPUX)

int GetMachineID()  {
int k = CALL_LIKE_HPUX;
  switch (k)  {
    case 1  : return MACHINE_RS6000;
    case 2  : return MACHINE_HP9000;
    default : return MACHINE_UNKNOWN;
  }
}

#elif defined(CALL_LIKE_STARDENT)

int GetMachineID()  {
int k = CALL_LIKE_STARDENT;
  switch (k)  {
    case 1  : return MACHINE_ARDENT;
    case 2  : return MACHINE_TITAN;
    case 3  : return MACHINE_STARDENT;
    default : return MACHINE_UNKNOWN;
  }
}

#elif defined(CALL_LIKE_VMS)

int GetMachineID()  {
  return MACHINE_VMS;
}

#elif defined(CALL_LIKE_MVS)

int GetMachineID()  {
  return MACHINE_MVS;
}

#else

int GetMachineID()  {
  return MACHINE_UNKNOWN;
}

#endif

static pstr MCH_SGI       = pstr("Silicon Graphics");
static pstr MCH_RS6000    = pstr("IBM RS/6000");
static pstr MCH_ALLIANT   = pstr("Alliant");
static pstr MCH_ARDENT    = pstr("Ardent");
static pstr MCH_TITAN     = pstr("Titan");
static pstr MCH_STARDENT  = pstr("Stardent");
static pstr MCH_CONVEX    = pstr("Convex");
static pstr MCH_ESV       = pstr("Evans or Sutherland");
static pstr MCH_HP9000    = pstr("Hewlett Packard 9000");
static pstr MCH_SOLBOURNE = pstr("Solbourne");
static pstr MCH_SOLARIS   = pstr("Solaris");
static pstr MCH_ALPHA     = pstr("DEC Alpha");
static pstr MCH_VMS       = pstr("A VMS machine");
static pstr MCH_MVS       = pstr("MS Windows");
static pstr MCH_F2C_G77   = pstr("SUN compatible");
static pstr MCH_LINUX     = pstr("Linux");

pstr GetMachineName ( int MachineID )  {
  switch (MachineID)  {
    case MACHINE_SGI       : return MCH_SGI;
    case MACHINE_RS6000    : return MCH_RS6000;
    case MACHINE_ALLIANT   : return MCH_ALLIANT;
    case MACHINE_ARDENT    : return MCH_ARDENT;
    case MACHINE_TITAN     : return MCH_TITAN;
    case MACHINE_STARDENT  : return MCH_STARDENT;
    case MACHINE_CONVEX    : return MCH_CONVEX;
    case MACHINE_ESV       : return MCH_ESV;
    case MACHINE_HP9000    : return MCH_HP9000;
    case MACHINE_SOLBOURNE : return MCH_SOLBOURNE;
    case MACHINE_SOLARIS   : return MCH_SOLARIS;
    case MACHINE_ALPHA     : return MCH_ALPHA;
    case MACHINE_VMS       : return MCH_VMS;
    case MACHINE_MVS       : return MCH_MVS;
    case MACHINE_F2C_G77   : return MCH_F2C_G77;
    case MACHINE_LINUX     : return MCH_LINUX;
    default                :
    case MACHINE_UNKNOWN   : return pstr("Unidentified machine");
  }
}

pstr GetMachineName()  {
  return GetMachineName ( GetMachineID() );
}

