//  $Id: mmdb_tables.h,v 1.18 2008/06/23 16:55:32 mdw Exp $
//  =================================================================
//
//   CCP4 Coordinate Library: support of coordinate-related
//   functionality in protein crystallography applications.
//
//   Copyright (C) Eugene Krissinel 2000-2008.
//
//    This library is free software: you can redistribute it and/or 
//    modify it under the terms of the GNU Lesser General Public 
//    License version 3, modified in accordance with the provisions 
//    of the license to address the requirements of UK law.
//
//    You should have received a copy of the modified GNU Lesser 
//    General Public License along with this library. If not, copies 
//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//  =================================================================
//
//    30.05.03   <--  Date of Last Modification.
//                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  -----------------------------------------------------------------
//
//  **** Module  :   MMDBF_Tables <interface>
//       ~~~~~~~~~
//  **** Project :   MacroMolecular Data Base (MMDB)
//       ~~~~~~~~~
//
//  **** Functions : 
//       ~~~~~~~~~~~ 
//
//  **** Constants : AName  ( array of 2-character atom names       )
//       ~~~~~~~~~~~ HAName ( array of 2=character heteroatom names )
//                   RName  ( 3-characters amino acid names         )
//                   RName1 ( 1-characters amino acid names         )
//
//
//  (C) E. Krissinel  2000-2008
//
//  =================================================================
//


#ifndef __MMDB_Tables__
#define __MMDB_Tables__


#ifndef __MatType__
#include "mattype_.h"
#endif



//  =================================================================


#define nElementNames  117
#define nElementMetals 91
#define nHydAtomNames  14

extern pstr     const ElementName   [nElementNames];
extern pstr     const ElementMetal  [nElementMetals];
extern pstr     const HydAtomName   [nHydAtomNames];
extern realtype const MolecWeight   [nElementNames];
extern realtype const CovalentRadius[nElementNames];
extern realtype const VdWaalsRadius [nElementNames];
extern realtype const IonicRadius   [nElementNames];

extern Boolean isMetal ( const pstr element );

#define  ELEMENT_UNKNOWN    -1

extern int      getElementNo      ( const pstr element );
extern realtype getMolecWeight    ( const pstr element );
extern realtype getCovalentRadius ( const pstr element );
extern realtype getVdWaalsRadius  ( const pstr element );

#define nResNames  26

extern pstr const ResidueName [nResNames];
extern char const ResidueName1[nResNames];

extern int getResidueNo ( pstr resName );

#define nSolventNames     12
#define nAminoacidNames   23
#define nNucleotideNames  24


DefineStructure(SAAProperty)

struct SAAProperty  {
  char     name[4];
  realtype hydropathy;
  realtype charge;
  realtype relSolvEnergy;
};

extern SAAProperty const AAProperty[nAminoacidNames];

extern realtype GetAAHydropathy ( const pstr resName );  // -4.5...+4.5
extern realtype GetAACharge     ( const pstr resName );
extern realtype GetAASolvationEnergy ( const pstr resName );
extern int      GetAASimilarity ( const pstr resName1,
                                  const pstr resName2 );  // 0..5

extern pstr const StdSolventName[nSolventNames];
//extern pstr const AminoacidName [nAminoacidNames];
extern pstr const NucleotideName[nNucleotideNames];

extern Boolean isSolvent    ( const pstr resName );
extern Boolean isAminoacid  ( const pstr resName );
extern Boolean isNucleotide ( const pstr resName );
extern Boolean isSugar      ( const pstr resName );

extern void  Get1LetterCode ( const pstr res3name, pstr res1code );


#endif

