C
C     ACT
C     Copyright (C) 1992 Wojtek Rypniewski
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C*************************************************************
C
C
C**************************************************************

C  ACT - analysis of coordinates.
C
C  The program is in the process of being expanded but it already
C  does some useful analyses.
C  For example it quickly finds ALL the crystal contacts,
C  no matter where the molecule is placed with respect to the origin
C  (otherwise the fast algorithm for finding contacts is the same
C  as in Tadeusz Skarzynski's program CONTACT).
C
C  The program always gives statistics for the temperature factors
C  averaged by residue, for the main chain and side chains.
C
C  Lists atoms with B-values .gt. bmoni (default = 50).
C
C  Lists B-values for Ox1 and Nx2 atoms for each ASN and GLN
C  - if the difference is large the atoms could be wrong way round.
C
C  Prints a histogram of solvent molecules in ranges of B-values.
C
C  Calculates F000 (the number of electrons in the unit cell).
C
C
C  Compulsory input cards:
C  -----------------------
C
C  SYMM followed by space group name or number.
C
C
C  Optional input cards:
C  ---------------------
C
C  The following options are currently available:
C
C
C  CONTACT - analysis of interatomic contacts.
C
C     followed by ALL (default)
C              or INTERmolecular
C              or INTRAmolecular
C
C                  followed by dmin dmax - distance limits for listing
C                                          contacts, in any order
C                                          (default = 0.0, 5.0)
C                                          If only one value given,
C                                          it is assumed to be dmax
C
C  SHORT - listing of short contacts
C
C     followed by dmoni (default = 2.5)
C                     short contacts, up to dmoni, are listed.
C
C  HBOND - list of possible hydrogen bonds (by distance criterion)
C
C     followed by ALL (default)
C              or INTERmolecular
C              or INTRAmolecular
C
C                  followed by hdmin hdmax - distance limits for listing
C                                            contacts in any order
C                                            (default = 2.7, 3.3)
C
C     A list of unpaired hydrogen bonding atoms is given with HBOND ALL option.
C
C
C  BMONITOR - followed by bmoni. Monitor B-values greater than bmoni.
C             (default = 50.0)
C
C
C  BDUMP - dump ascii plot files (BRES.PLT) for the B-factors to disk
C
C
C  SEQU - extract sequence and analyse the amino acid composition.
C
C
C  END - last card, ends input.
C
C
C  Plotting:
C
C  If you have a NAG plotting library you can uncomment the call
C  to plotit and the plotit code at the end.  You can then plot
C  B-values by residue with keyword:
C
C
C  BPLT - plot B-values by residue 
C
C--------------------------------------------------------------------------------
C Example:
C
C #!/bin/csh -f
C act XYZIN /Throb1/Wojtek/sp/frodo/sp170b.brk << EOF-act
C SYMM 4
C BMONI 55
C CONTACT INTER
C HBOND
C SHORT
C BDUMP
C SEQU
C END
C EOF-act
C
C--------------------------------------------------------------------------------
      PROGRAM ACT
C
      PARAMETER (MXATOM=99999,MAXBAD=20000,MAXCH=30,MAXRES=9999) 
C
C This is the maximum possible number of equivalent positions
C for a biological substance (F432), quite unlikely to occur in practice.
      PARAMETER (MAXSYM=96)
C
      REAL XYZ(3,MXATOM),OCC(MXATOM),BF(MXATOM),BPLOT(2,MAXRES)
      INTEGER NRES(MXATOM),CHPTR(MAXCH),RESCTR(2,MAXRES),RESLST(MAXRES)
      INTEGER IXYZIN,F000,NCH,RESPTR,NRESID(MAXCH),NATOMS
      CHARACTER*40000 RESIDS
      CHARACTER*80 FULNAM
      CHARACTER*4 ANAME(MXATOM),RNAME(MXATOM)
      CHARACTER*1 CH(MXATOM),IDCH(MAXCH)
C
C..RF - matrix for converting cartesian coordinates to fractional
C..RO - matrix for converting fractional coordinates to cartesian
C..RSYM - symmetry matrices
      REAL RF(4,4),RO(4,4),RSYM(4,4,MAXSYM)
      INTEGER NSYM
C
      REAL DMONI,DMIN,DMAX,HDMIN,HDMAX
C
      LOGICAL CONTCT,INTER,INTRA,ALL,HBOND,HINTER,HINTRA,HALL,
     +        SHORT,SALL,SEQUEN,BYRES,CHFILE,SYMM
C
      LOGICAL FALS
      FALS=.FALSE.
C
C     *****************
      CALL CCPFYP
      CALL XYZINIT
C     *****************
C
      IUIN=5
      IUOT=6
      LDUM = 0
      IFAIL = 0
C
C     ****************************************************
      CALL CCPRCS(IUOT,'ACT','$Date: 2007/09/17 09:10:15 $')
C     ****************************************************
C Read input cards
C
      CALL KEYIN(IUOT,SEQUEN,BYRES,BMONI,CHFILE,
     +           CONTCT,INTER,INTRA,ALL,DMIN,DMAX,
     +           HBOND,HINTER,HINTRA,HALL,HDMIN,HDMAX,
     +           SHORT,SALL,DMONI,SYMM,NSYM,RSYM)
C
      IXYZIN = 0
C Read in coordinates
C     **********************************************
      CALL XYZOPEN('XYZIN','INPUT',' ',IXYZIN,IFAIL)
      INQUIRE (FILE='XYZIN',NAME=FULNAM)
C     **********************************************
C
      CALL GETXYZ(IXYZIN,XYZ,ANAME,RNAME,NRES,OCC,BF,IDCH,NATOMS,
     +            CH,NCH,CHPTR,RF,RO)
      print *, 'natoms = ',natoms
C
C Get the B-factor statistics and F000 for main/side chain.
C
      CALL BSTATS(IUOT,ANAME,RNAME,NRES,OCC,BF,IDCH,NATOMS,
     +            NCH,CH,CHPTR,BMONI,F000,RESCTR,BPLOT,
     +            RESPTR,RESLST,NRESID,RESIDS,NSYM)
C
C Plotting B-values
C
      IF (BYRES.OR.CHFILE) THEN
        CALL PLBYRS(IUOT,RESCTR,BPLOT,RESPTR,RESLST,BYRES,CHFILE)
      ENDIF
C
C Search for hydrogen bonds
C
      IF (HBOND) THEN
        CALL CONTACT(IUOT,NATOMS,NCH,CHPTR,XYZ,ANAME,RNAME,NRES,OCC,
     +               NRESID,IDCH,CH,RF,RO,NSYM,RSYM,HDMIN,HDMAX,
     +               HBOND,HINTER,HINTRA,HALL,FALS,DMONI)
      ENDIF
C
C Search for contacts
C
      IF (CONTCT) THEN
        CALL CONTACT(IUOT,NATOMS,NCH,CHPTR,XYZ,ANAME,RNAME,NRES,OCC,
     +               NRESID,IDCH,CH,RF,RO,NSYM,RSYM,DMIN,DMAX,
     +               FALS,INTER,INTRA,ALL,FALS,DMONI)
      ENDIF
C
C List short contacts
C
      IF (SHORT) THEN
        CALL CONTACT(IUOT,NATOMS,NCH,CHPTR,XYZ,ANAME,RNAME,NRES,OCC,
     +               NRESID,IDCH,CH,RF,RO,NSYM,RSYM,DMIN,DMAX,
     +               FALS,FALS,FALS,SALL,SHORT,DMONI)
      ENDIF
C
C  Sequence information
C
      IF (SEQUEN) THEN
        CALL SEQ(IUOT,NCH,NRESID,RESIDS)
      ENDIF
C
      CALL XYZCLOSE(IXYZIN)
      CALL CCPERR(0,'NORMAL TERMINATION')
      END
C
C------------------------------------------------------------------
      SUBROUTINE KEYIN(IUOT,SEQUEN,BYRES,BMONI,CHFILE,
     +                 CONTCT,INTER,INTRA,ALL,DMIN,DMAX,
     +                 HBOND,HINTER,HINTRA,HALL,HDMIN,HDMAX,
     +                 SHORT,SALL,DMONI,SYMM,NSYM,RSYM)
C------------------------------------------------------------------
C Read keyworded input.
C
      PARAMETER (NPAR=200)
      PARAMETER (NKEY=9,NKEYS=9)
C
      REAL FVALUE(NPAR)
      INTEGER IBEG(NPAR),IDEC(NPAR),IEND(NPAR),ITYP(NPAR)
      CHARACTER KEYWRD(NKEY)*4,CVALUE(NPAR)*4,LINE*400,
     +   KEY*4,CWORK*10,ERRLIN*100
      LOGICAL CONTCT,INTER,INTRA,ALL,HBOND,HINTER,HINTRA,HALL,
     +        SHORT,SALL,SYMM,LEND,SEQUEN,BYRES,
     +        CHFILE
C
      INTEGER NSYM,NSYMP
      CHARACTER*10 SPGNAM,PGNAME
      REAL RSYM(4,4,*)
C
      REAL DMONI,DMIN,DMAX,HDMIN,HDMAX
C
      DATA KEYWRD/'SHOR','BMON','BPLT','SEQU','BDUM',
     +            'CONT','HBON','SYMM','END '/
C
C Defaults for cards
C
      SEQUEN = .FALSE.
      BYRES =  .FALSE.
      CHFILE = .FALSE.
C
      CONTCT = .FALSE.
      INTER = .FALSE.
      INTRA = .FALSE.
      ALL = .FALSE.
C
      HBOND = .FALSE.
      HINTER = .FALSE.
      HINTRA = .FALSE.
      HALL = .FALSE.
C
      SHORT = .FALSE.
      SALL = .FALSE.
C
      SYMM = .FALSE.
C
C Default distances (code needs these, even if not used)
C
      DMONI = 2.5
      DMIN = 0.0
      DMAX = 5.0
      HDMIN = 2.7
      HDMAX = 3.3
      BMONI = 50.0
C
   10 CONTINUE
      LINE = '   '
      KEY = ' '
      NTOK = NPAR
C
C     *****************************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +            .TRUE.)
C     *****************************************************************
      IF (LEND) THEN
        CALL CCPERR(1,'UNEXPECTED END OF FILE ON INPUT CONTROL DATA')
      ELSE
        KEY = LINE(IBEG(1) :4)
C
C            **********
        CALL CCPUPC(KEY)
C            **********
C
C---- extra check for comment line (!)
C
        IF (KEY(1:1).EQ.'!') THEN
          GO TO 10
        ELSE
C
C----  Loop over possible key-words
C
          DO 20 I = 1,NKEYS
            IF (KEY.EQ.KEYWRD(I)) GO TO 30
   20     CONTINUE
C
          ERRLIN = ' *** ERROR - UNRECOGNIZED KEYWORD: '//
     +              LINE(IBEG(1) :IEND(1))
          CALL CCPERR(2,ERRLIN)
C
   30     CONTINUE
C
C
C      DATA KEYWRD/'SHOR','BMON','BPLT','SEQU','BDUM',
C     +            'CONT','HBON','SYMM','END '/
C
C
          GO TO (50,60,70,80,100,110,120,130,500) I
C
C
C   "SHOR"  List short contacts
   50     SHORT=.TRUE.
          SALL=.TRUE.
          IF(NTOK.EQ.1)THEN
            DMONI=2.5
            WRITE(ERRLIN,FMT='(A,A,F5.2,A)')
     +       ' SHORT: Default values used: short contacts will be',
     +       ' listed up to ',DMONI,' angstroms.'
            CALL CCPERR(3,ERRLIN)
          ENDIF
C
          IF(NTOK.EQ.2)THEN
            N=1
            CALL KEYNUM(N,2,LINE,IBEG,IEND,ITYP,NTOK)
            DMONI = FVALUE(2)
          ENDIF
C
          IF(NTOK.GT.2)THEN
             CALL CCPERR(1,'KEYIN: Card SHORT has too many tokens')
          ENDIF
C
          GO TO 10
C
C   "BMON"  B values monitored above this value
   60     IF (NTOK.LE.1) THEN
            BMONI = 50.0
            WRITE(ERRLIN,FMT='(A,F6.2)')
     +       ' No value read from BMON card: using default of ',BMONI
            CALL CCPERR(3,ERRLIN)
          ELSE
            N=1
            CALL KEYNUM(N,2,LINE,IBEG,IEND,ITYP,NTOK)
            BMONI = FVALUE(2)
          ENDIF
          GO TO 10
C
C   "BPLT"  B-value by residue plot
   70     BYRES = .TRUE.
          CALL CCPERR(2,' SORRY, PLOTTING NOT IMPLEMENTED.')
          ERRLIN = ' IF YOU HAVE A NAG LIBRARY FIRST UNCOMMENT THE'//
     +             ' CALL TO PLOTIT IN PLBYRS,'
          CALL CCPERR(2,ERRLIN)
          CALL CCPERR(2,' THEN LINK ACT WITH ACTPLOT AND THE LIBRARY.')
          GO TO 10
C
C   "SEQU"  Amino acid sequence output for sequence programs
   80     SEQUEN = .TRUE.
          GO TO 10
C
C   "BDUM" Dump ascii plot files to disk
  100     CHFILE = .TRUE.
          GO TO 10
C
C   "CONT" list contacts
  110     CONTCT = .TRUE.
          IF(NTOK.EQ.1) THEN
            ALL=.TRUE.
            DMIN=0.
            DMAX=5.
            ERRLIN = ' CONTACT: Default values used: ALL contacts'//
     +               ' will be listed between 0 and 5 angstroms.'
            CALL CCPERR(2,ERRLIN)
          ENDIF
C
          IF(NTOK.EQ.2) THEN
             IF(ITYP(2).EQ.1) THEN
                CWORK=LINE(IBEG(2):IEND(2))
                CALL CCPUPC(CWORK)
                IF (CWORK(1:4).EQ.'INTE') INTER=.TRUE.
                IF (CWORK(1:4).EQ.'INTR') INTRA=.TRUE.
                IF (CWORK(1:3).EQ.'ALL')  ALL=.TRUE.
                IF (INTER.OR.INTRA.OR.ALL) THEN
                   DMIN=0.
                   DMAX=5.
                   WRITE (ERRLIN,FMT='(A,A,F5.2,A,F5.2,A)')
     +              ' CONTACT: Default values used:  contacts will be',
     +              ' listed between ',DMIN,' and ',DMAX,' angstroms.'
                   CALL CCPERR(3,ERRLIN)
                ELSE
                   ERRLIN = 'KEYIN: CONTACT can only be followed'//
     +             ' by INTER, INTRA, ALL or values of dmin and dmax'
                   CALL CCPERR(1,ERRLIN)
                ENDIF
             ELSE IF(ITYP(2).EQ.2) THEN
                     ALL=.TRUE.
                     DMAX=FVALUE(2)
                     DMIN=0.
                     ERRLIN = ' CONTACT: Default values used: ALL '//
     +                        ' contacts will be listed.'
                     CALL CCPERR(3,ERRLIN)
                     WRITE(ERRLIN,FMT='(A,F5.2)')
     +                 ' CONTACT: Default value used for dmin: ',DMIN
                      CALL CCPERR(3,ERRLIN)
                  ELSE
                     ERRLIN = 'KEYIN: CONTACT can only be followed'//
     +                ' by INTER, INTRA, ALL or values of dmin and dmax'
                     CALL CCPERR(1,ERRLIN)
             ENDIF
          ENDIF
C
          IF(NTOK.EQ.3) THEN
             IF(ITYP(2).EQ.1) THEN
                CWORK=LINE(IBEG(2):IEND(2))
                CALL CCPUPC(CWORK)
                IF (CWORK(1:4).EQ.'INTE') INTER=.TRUE.
                IF (CWORK(1:4).EQ.'INTR') INTRA=.TRUE.
                IF (CWORK(1:3).EQ.'ALL')  ALL=.TRUE.
                IF (INTER.OR.INTRA.OR.ALL) THEN
                   N=1
                   CALL KEYNUM(N,3,LINE,IBEG,IEND,ITYP,NTOK)
                   DMAX=FVALUE(3)
                   DMIN=0.
                   WRITE(ERRLIN,FMT='(A,F5.2)')
     +               ' CONTACT: Default value used for dmin: ',DMIN
                   CALL CCPERR(3,ERRLIN)
                ELSE
                   ERRLIN = 'KEYIN: CONTACT can only be followed'//
     +              ' by INTER, INTRA, ALL or values of dmin and dmax'
                   CALL CCPERR(1,ERRLIN)
                ENDIF
             ELSE IF(ITYP(2).EQ.2) THEN
                     ALL=.TRUE.
                     N=2
                     CALL KEYNUM(N,2,LINE,IBEG,IEND,ITYP,NTOK)
                     DMIN=FVALUE(2)
                     DMAX=FVALUE(3)
                     IF(DMIN.GT.DMAX) THEN
                        TEMP=DMIN
                        DMIN=DMAX
                        DMAX=TEMP
                     ENDIF
                        ERRLIN = ' CONTACT: Default used: ALL conta'//
     +              'cts will be listed between the specified limits.'
                        CALL CCPERR(3,ERRLIN)
                  ELSE
                     ERRLIN = 'KEYIN: CONTACT can only be followed by'//
     +                 ' INTER, INTRA, ALL or values of dmin and dmax'
                     CALL CCPERR(1,ERRLIN)
             ENDIF
          ENDIF
C
          IF(NTOK.EQ.4) THEN
             IF(ITYP(2).EQ.1) THEN
                CWORK=LINE(IBEG(2):IEND(2))
                CALL CCPUPC(CWORK)
                IF (CWORK(1:4).EQ.'INTE') INTER=.TRUE.
                IF (CWORK(1:4).EQ.'INTR') INTRA=.TRUE.
                IF (CWORK(1:3).EQ.'ALL')  ALL=.TRUE.
                IF (INTER.OR.INTRA.OR.ALL) THEN
                  N=2
                  CALL KEYNUM(N,3,LINE,IBEG,IEND,ITYP,NTOK)
                  DMIN=FVALUE(3)
                  DMAX=FVALUE(4)
                  IF(DMIN.GT.DMAX) THEN
                     TEMP=DMIN
                     DMIN=DMAX
                     DMAX=TEMP
                  ENDIF
                ELSE
                  ERRLIN = 'KEYIN: CONTACT can only be followed by'//
     +                 ' INTER, INTRA, ALL or values of dmin and dmax'
                  CALL CCPERR(1,ERRLIN)
                ENDIF
             ELSE
                ERRLIN = 'KEYIN: CONTACT can only be followed by'//
     +               ' INTER, INTRA, ALL or values of dmin and dmax'
                CALL CCPERR(1,ERRLIN)
             ENDIF
          ENDIF
C
          IF(NTOK.GT.4) THEN
             CALL CCPERR(1,'KEYIN: Card CONTACT has too many tokens')
          ENDIF
C
          GO TO 10
C
C   "HBON" list possible hydrogen bonds
  120     HBOND = .TRUE.
          IF(NTOK.EQ.1) THEN
             HALL=.TRUE.
             HDMIN=2.7
             HDMAX=3.3
             ERRLIN = ' HBOND: Default values used: ALL contacts will'//
     +                ' be listed between 2.7 and 3.3 angstroms'
             CALL CCPERR(3,ERRLIN)
          ENDIF
C
          IF(NTOK.EQ.2) THEN
             IF(ITYP(2).EQ.1) THEN
                CWORK=LINE(IBEG(2):IEND(2))
                CALL CCPUPC(CWORK)
                IF (CWORK(1:4).EQ.'INTE') HINTER=.TRUE.
                IF (CWORK(1:4).EQ.'INTR') HINTRA=.TRUE.
                IF (CWORK(1:3).EQ.'ALL')  HALL=.TRUE.
                IF (HINTER.OR.HINTRA.OR.HALL) THEN
                   HDMIN=2.7
                   HDMAX=3.3
                   ERRLIN = ' HBOND: Default values used: contacts '//
     +               ' will be listed between 2.7 and 3.3 angstroms'
                   CALL CCPERR(3,ERRLIN)
                ELSE
                   ERRLIN = 'KEYIN: HBOND can only be followed by'//
     +             ' INTER, INTRA, ALL or values of dmin and dmax'
                   CALL CCPERR(1,ERRLIN)
                ENDIF
             ELSE IF(ITYP(2).EQ.2) THEN
                     ERRLIN = 'KEYIN: HBOND: Please specify '//
     +                        'values for both dmin and dmax.'
                     CALL CCPERR(1,ERRLIN)
                  ELSE
                     ERRLIN = 'KEYIN: HBOND can only be followed by'//
     +               ' INTER, INTRA, ALL or values of dmin and dmax'
                     CALL CCPERR(1,ERRLIN)
             ENDIF
C
          ENDIF
C
          IF(NTOK.EQ.3) THEN
             IF(ITYP(2).EQ.2) THEN
                HALL=.TRUE.
                N=2
                CALL KEYNUM(N,2,LINE,IBEG,IEND,ITYP,NTOK)
                HDMIN=FVALUE(2)
                HDMAX=FVALUE(3)
                IF(HDMIN.GT.HDMAX) THEN
                   TEMP=HDMIN
                   HDMIN=HDMAX
                   HDMAX=TEMP
                ENDIF
                   ERRLIN = ' HBOND: Default used: ALL contacts '//
     +              ' will be listed between the specified limits.'
                   CALL CCPERR(3,ERRLIN)
             ELSE
                ERRLIN = 'KEYIN: HBOND: Please specify values'//
     +                   ' for both dmin and dmax'
                CALL CCPERR(1,ERRLIN)
             ENDIF
          ENDIF
C
          IF(NTOK.EQ.4) THEN
             IF(ITYP(2).EQ.1) THEN
                CWORK=LINE(IBEG(2):IEND(2))
                CALL CCPUPC(CWORK)
                IF (CWORK(1:4).EQ.'INTE') HINTER=.TRUE.
                IF (CWORK(1:4).EQ.'INTR') HINTRA=.TRUE.
                IF (CWORK(1:3).EQ.'ALL')  HALL=.TRUE.
                IF (HINTER.OR.HINTRA.OR.HALL) THEN
                   N=2
                   CALL KEYNUM(N,3,LINE,IBEG,IEND,ITYP,NTOK)
                   HDMIN=FVALUE(3)
                   HDMAX=FVALUE(4)
                   IF(HDMIN.GT.HDMAX) THEN
                      TEMP=HDMIN
                      HDMIN=HDMAX
                      HDMAX=TEMP
                   ENDIF
                ELSE
                   ERRLIN = 'KEYIN: HBOND can only be followed by'//
     +             ' INTER, INTRA, ALL or values of dmin and dmax'
                   CALL CCPERR(1,ERRLIN)
                ENDIF
             ELSE
                ERRLIN = 'KEYIN: HBOND can only be followed by'//
     +            ' INTER, INTRA, ALL or values of dmin and dmax'
                CALL CCPERR(1,ERRLIN)
             ENDIF
          ENDIF
C
          IF(NTOK.GT.4) THEN
             CALL CCPERR(1,'KEYIN: Card HBOND has too many tokens')
          ENDIF
C
          GO TO 10
C
C   "SYMM" read symmetry (space group number or name)
  130     NSYM=0
          JTOK=2
C         *************************************************
          CALL RDSYMM(JTOK,LINE,IBEG,IEND,ITYP,FVALUE,NTOK,
     +                SPGNAM,NUMSGP,PGNAME,NSYM,NSYMP,RSYM)
C         *************************************************
          SYMM = .TRUE.
          GO TO 10
C
C   "END"
C
  500     GO TO 1000
        ENDIF
      ENDIF
 1000 CONTINUE
C
      IF(.NOT.SYMM) THEN
        CALL CCPERR(1,'KEYIN: SYMMETRY CARD IS COMPULSORY!')
      ENDIF
C
      RETURN
      END
C
C--------------------------------------------------------------
      SUBROUTINE GETXYZ(IXYZIN,XYZ,ANAME,RNAME,NRES,OCC,BF,IDCH,
     +                  NATOMS,CH,NCH,CHPTR,RFRAC,RORTH)
C--------------------------------------------------------------
C Read in coordinates from Brookhaven file.
C
      PARAMETER (MXATOM=99999,MAXCH=30)
C
      REAL BISO,XYZ(3,MXATOM),OCC(MXATOM),BF(MXATOM),B(6)
      REAL RFRAC(4,4),RORTH(4,4)
      INTEGER IXYZIN,NRES(MXATOM),CHPTR(MAXCH)
      CHARACTER*4 ANAME(MXATOM),RNAME(MXATOM),RESNO,ID,SEGID
      CHARACTER*1 CH(MXATOM),CURCH,IDCH(MAXCH),ALTCOD,INSCOD
      CHARACTER*80 LINE
      REAL Q,PI
C
      PI = 3.1415927
      CURCH = '?'
      NCH = 0
      IFLAG = 1
C
      N = 1
 10   CALL XYZADVANCE(IXYZIN,0,0,*20,*20)
      CALL XYZATOM(IXYZIN,ISER,ANAME(N),RNAME(N),CH(N),NRES(N),
     +                RESNO,INSCOD,ALTCOD,SEGID,IZ,ID)
      CALL XYZCOORD(IXYZIN,'O','U',X,Y,Z,Q,BISO,B)
C
C---- Check for anisotropic Bs, skip ANISOU
C
      IF (B(2).NE.0.0 .OR. B(3).NE.0.0) THEN
        IF (X.EQ.0.0 .AND. Y.EQ.0.0 .AND. Z.EQ.0.0) GOTO 10
      ENDIF
      BF(N) = BISO

      XYZ(1,N) = X
      XYZ(2,N) = Y
      XYZ(3,N) = Z
      OCC(N) = Q
C
      IF (CH(N).NE.CURCH) THEN
        NCH = NCH + 1
        IF (NCH.GT.MAXCH) THEN
          WRITE(LINE,'(A,I3,A)')
     + 'TOO MANY CHAINS IN XYZIN. INCREASE MAXCH (CURRENTLY ',MAXCH,')'
          CALL CCPERR(1,LINE)
        ENDIF
        CURCH = CH(N)
        IDCH(NCH) = CH(N)
        CHPTR(NCH) = N
      ENDIF
C
      N = N + 1
      IF (N.GT.MXATOM) THEN
          WRITE(LINE,'(A,I6,A)')
     + 'TOO MANY ATOMS IN XYZIN. INCREASE MXATOM (CURRENTLY ',MXATOM,')'
          CALL CCPERR(1,LINE)
      ENDIF

      GO TO 10
C
 20   CONTINUE
C
      NATOMS = N - 1
C
      RORTH(1,1) = 0.0
      CALL RBRORF(RORTH,RFRAC)

      IF (RORTH(1,1) .EQ. 0.0) 
     +       CALL CCPERR(1,'NO CRYST CARD IN BROOK FILE')

      RETURN
      END
C
C---------------------------------------------------------------------
      SUBROUTINE BSTATS(IUOT,ANAME,RNAME,NRES,OCC,BF,IDCH,NATOMS,
     +                  NCH,CH,CHPTR,BMONI,F000,RESCTR,BPLOT,
     +                  RESPTR,RESLST,NRESID,RESIDS,NSYM)
C---------------------------------------------------------------------
C Temperature factor analysis.
C
      PARAMETER(MAXBAD=20000,MAXCH=30,MAXMET=11,MXATOM=99999,
     +           MAXRES=9999,MAXUNK=500)
C
      INTEGER IBADR(MAXBAD),ICHN(MAXBAD),
     +        NO(MAXCH),NWO(MAXCH),NPO(MAXCH),NMAIN(MAXCH),
     +        NSOLV(MAXCH),NMET(MAXCH),NSIDE(MAXCH),NRESID(MAXCH),
     +        NPROT(MAXCH),NTOT(MAXCH),MXRES1(MAXCH),MXRES2(MAXCH),
     +        MXRES3(MAXCH),OTHER(MAXCH),CHPTR(MAXCH),
     +        NRES(MXATOM),RESCTR(2,MAXRES),RESLST(MAXRES),F000,
     +        RESPTR,ICH,
     +        ISIGDEVM(MAXCH,4),MEANDEVM(MAXCH,4),
     +        ISIGDEVS(MAXCH,4),MEANDEVS(MAXCH,4),
     +        MBADPT(MAXCH),SBADPT(MAXCH),
     +        BH(21)
      REAL OCC(MXATOM),BF(MXATOM),MXPB(10,MAXCH),BADB(MAXBAD),
     +     TBFAC(MAXCH),SOLBFAC(MAXCH),FETBFAC(MAXCH),
     +     SIDBFAC(MAXCH),AVMNB(MAXCH),AVSDB(MAXCH),AVMTB(MAXCH),
     +     AVTOTB(MAXCH),AVPRB(MAXCH),AVSLB(MAXCH),OTBFAC(MAXCH),
     +     AVOTHB(MAXCH),BPLOT(2,MAXRES),
     +     VALUESM(MAXRES),VALUESS(MAXRES),
     +     AVEM(MAXCH),ADEVM(MAXCH),SDEVM(MAXCH),
     +     AVES(MAXCH),ADEVS(MAXCH),SDEVS(MAXCH),
     +     MBADLIST(MAXBAD),SBADLIST(MAXBAD),BMONI,TOTH
      CHARACTER*40000 RESIDS
      CHARACTER*1 CH(MXATOM),IDCH(MAXCH),SHORT(20)
      CHARACTER*4 MCHAIN(5),SOLV(4),METAL(MAXMET),AACID(20),
     +            BADRES(MAXBAD),BADAT(MAXBAD),ANAME(MXATOM),
     +            RNAME(MXATOM)
      CHARACTER*40 UNKNOWN(MAXUNK),UNK1
      CHARACTER*12 RLIST(MAXRES),RLABEL,RLISTM(MAXRES),RLISTS(MAXRES),
     +             MBADLABEL(MAXBAD),SBADLABEL(MAXBAD)
      CHARACTER*50 HISTO
      CHARACTER*80 LINE
C
C..Pointers to Ox1 and Nx2 atoms of ASN and GLN
      PARAMETER(MXASN = 3000)
      INTEGER ASNGLN(2,MXASN)
C..Flags for ASN and GLN
      LOGICAL GLN,ASN
C
      DATA AACID  /'ALA ','ARG ','ASN ','ASP ','CYS ','GLN ','GLU ',
     +             'GLY ','HIS ','ILE ','LEU ','LYS ','MET ','PHE ',
     +             'PRO ','SER ','THR ','TRP ','TYR ','VAL '/
      DATA SHORT  /'A','R','N','D','C','Q','E','G','H','I','L','K',
     +             'M','F','P','S','T','W','Y','V'/
      DATA MCHAIN /'N   ','C   ','CA  ','O   ','OE  '/
      DATA METAL  /'CO  ','MG  ','FE  ','ZN  ','PB  ','HG  ','CM  ',
     +             'VO  ','V   ','CU  ','PT  '/
      DATA SOLV   /'OW1 ','OW0 ','OW  ','HOH '/
      DATA HISTO  /'**************************************************'/
C
      IF(BMONI.EQ.0.0) BMONI=50.
C
      CALL CALCF0(NATOMS,ANAME,RNAME,F000)
      F000=F000*NSYM
      WRITE (IUOT,10) F000,NSYM
  10  FORMAT(1X,'F(000) = ',I8,' for ',I2,' molecules in',
     +        ' the unit cell.'/)
C
      DO 976 I=1,21
        BH(I)=0
  976 CONTINUE
C
      RESPTR = 0
      LPTR=1
      NBAD = 0
      IASNGLN = 0
      INK = 0
      INRES = 0
      INSOLV = 0
      INMET = 0
      TB = 0.0
      SB = 0.0
      SLB = 0.0
      TM = 0.0
      ITO = 0
      IMAIN = 0
      ISIDE = 0
      ISOLV = 0
      IMET = 0
      ITOTHER = 0
      ITOTHR = 0
      ITOTA = 0
      TAVMN = 0.0
      TAVSD = 0.0
      TAVSLV = 0.0
      TAVPR = 0.0
      TMET = 0.0
      TOTH = 0.0
C
      DO 387 I=1,MAXCH
        AVSLB(I) = 0.0
        AVMTB(I) = 0.0
        AVMNB(I) = 0.0
        AVSDB(I) = 0.0
        AVPRB(I) = 0.0
        AVOTHB(I) = 0.0
        AVTOTB(I) = 0.0
         MXRES1(I) = 0
         MXRES2(I) = 0
         MXRES3(I) = 0
         NRESID(I) = 0
         NO(I) = 0
         NWO(I) = 0
         NPO(I) = 0
         TBFAC(I) = 0.0
         NMAIN(I) = 0
         NSIDE(I) = 0
         SOLBFAC(I) = 0.0
         NSOLV(I) = 0
         NMET(I) = 0
         FETBFAC(I) = 0
         OTHER(I) = 0
         OTBFAC(I) = 0
         SIDBFAC(I) = 0.0
         DO 389 J=1,10
           MXPB(J, I) = 0
 389     CONTINUE 
         DO 390 J=1,4
           ISIGDEVM(I,J) = 0
           ISIGDEVS(I,J) = 0
           MEANDEVM(I,J) = 0
           MEANDEVS(I,J) = 0
 390     CONTINUE 
 387  CONTINUE
C
      DO 388 I=1,2
      DO 388 J=1,MAXRES
        BPLOT(I,J) = 0.0
        RESCTR(I,J) = 0
 388  CONTINUE
      DO 386 I=1,2
         DO 386 J=1,MXASN
            ASNGLN(I,J)=0
 386        CONTINUE
C
C..Loop over chains
      DO 5000 ICH = 1,NCH
C
        ILOW = CHPTR(ICH)
        IF (ICH.EQ.NCH) THEN
          IHIGH = NATOMS
        ELSE
          IHIGH = CHPTR(ICH+1) - 1
        ENDIF
C
        IRESPR = 0
C
C..Loop over atoms within a chain.
        DO 4000 IATOM = ILOW,IHIGH
C
          IF (NRES(IATOM).NE.IRESPR) THEN
C
C This is a new residue so see if it is an amino acid
C
            DO 400 I = 1,20
              IF (RNAME(IATOM).EQ.AACID(I)) THEN
                NRESID(ICH) = NRESID(ICH) + 1
                RESPTR = RESPTR + 1
                IF (RESPTR.GT.MAXRES) THEN
                  WRITE(LINE,'(A,I5,A)')
     + 'TOO MANY RESIDUES IN XYZIN. INCREASE MAXRES (CURRENTLY ',
     +  MAXRES,')'
                  CALL CCPERR(1,LINE)
                ENDIF
                RESLST(RESPTR) = NRES(IATOM)
                RESIDS(LPTR:LPTR+3)=RNAME(IATOM)
                LPTR=LPTR+4
                IF (LPTR.GT.40000) THEN
                  CALL CCPERR(1,
     +  'INCREASE LENGTH OF CHARACTER STRING RESIDS (CURRENTLY 40000)')
                ENDIF
                WRITE(RLABEL,1753)NRES(IATOM),IDCH(ICH),RNAME(IATOM)
1753            FORMAT(1X,I4,A,2X,A4)
                RLIST(RESPTR)=RLABEL
              ENDIF
 400        CONTINUE
            IRESPR = NRES(IATOM)
C
            IF(RNAME(IATOM).EQ.'GLN ')THEN
              GLN=.TRUE.
              NUMRES=RESPTR
              IASNGLN=IASNGLN+1
              IF(IASNGLN.GT.MXASN) CALL CCPERR(1,
     +            'TOO MANY ASN AND GLN: INCREASE PARAMETER MXASN')
            ELSE
              GLN=.FALSE.
            ENDIF
            IF(RNAME(IATOM).EQ.'ASN ')THEN
              ASN=.TRUE.
              NUMRES=RESPTR
              IASNGLN=IASNGLN+1
              IF(IASNGLN.GT.MXASN)CALL CCPERR(1,
     +              'TOO MANY ASN AND GLN: INCREASE PARAMETER MXASN')
            ELSE
              ASN=.FALSE.
            ENDIF
C
          ENDIF
C
C Don't include any atoms with zero occupancy in calculations
C but just count them
C
          IF (OCC(IATOM).EQ.0.0) THEN
            NO(ICH) = NO(ICH) + 1
            IF (RNAME(IATOM).EQ.'WAT '
     +          .OR.RNAME(IATOM).EQ.'HOH ') THEN
              NWO(ICH) = NWO(ICH) + 1
            ELSE
              NPO(ICH) = NPO(ICH) + 1
            ENDIF
            GOTO 3950
          ENDIF
C
C Check for the bad B-factors
C
          IF (BF(IATOM).GT.BMONI) THEN
            NBAD = NBAD + 1
            BADB(NBAD) = BF(IATOM)
            BADAT(NBAD) = ANAME(IATOM)
            BADRES(NBAD) = RNAME(IATOM)
            IBADR(NBAD) = NRES(IATOM)
            ICHN(NBAD) = ICH
            IF (NBAD.GT.MAXBAD) CALL CCPERR(1,
     +  'No. of B-factors to be monitored > MAXBAD: INCREASE <bmoni>')
          ENDIF
C
C  Solvent
C
          DO 700 I = 1,4
C
            IF(ANAME(IATOM)(1:1).EQ.SOLV(I)(1:1).AND.
     +         RNAME(IATOM).EQ.'WAT '.OR.RNAME(IATOM).EQ.'HOH ')THEN
              SOLBFAC(ICH) = SOLBFAC(ICH) + BF(IATOM)
              NSOLV(ICH) = NSOLV(ICH) + 1
C
C..prepare a histogram
              NBIN=0
              DO 288 KK=0,95,5
                NBIN=NBIN+1
                IF(BF(IATOM).GT.FLOAT(KK).AND.
     +             BF(IATOM).LE.(FLOAT(KK)+5.0))
     +                BH(NBIN)=BH(NBIN)+1
 288          CONTINUE
              IF(BF(IATOM).GT.100.0) BH(21)=BH(21)+1
C
              IF (BF(IATOM).GT.MXPB(3,ICH)) THEN
                MXPB(3,ICH) = BF(IATOM)
                MXRES3(ICH) = NRES(IATOM)
              ENDIF
C
              GOTO 3950
C
            ENDIF
 700      CONTINUE
C
C  Main chain
C
          DO 600 J = 1,20
C
C  Allow both long and short residue names
            IF (RNAME(IATOM).EQ.AACID(J) .OR.
     +          (RNAME(IATOM)(1:1).EQ.SHORT(J)
     +          .and. ichar(rname(iatom)(2:2)) .eq. 32 
     +          .and. ichar(rname(iatom)(3:3)) .eq. 32 )) then     
C
              DO 500 I = 1,5
                IF (ANAME(IATOM).EQ.MCHAIN(I)) THEN
                  TBFAC(ICH) = TBFAC(ICH) + BF(IATOM)
                  NMAIN(ICH) = NMAIN(ICH) + 1
                  BPLOT(1,RESPTR) =  BPLOT(1,RESPTR) + BF(IATOM)
                  RESCTR(1,RESPTR) = RESCTR(1,RESPTR) + 1
                  IF (BF(IATOM).GT.MXPB(1,ICH)) THEN
                    MXPB(1,ICH) = BF(IATOM)
                    MXRES1(ICH) = NRES(IATOM)
                  ENDIF
                  GOTO 3950
                ENDIF
 500          CONTINUE
C
C  Side chain
C
              SIDBFAC(ICH) = SIDBFAC(ICH) + BF(IATOM)
              NSIDE(ICH) = NSIDE(ICH) + 1
              BPLOT(2,RESPTR) = BPLOT(2,RESPTR) + BF(IATOM)
              RESCTR(2,RESPTR) = RESCTR(2,RESPTR) + 1
              IF (BF(IATOM).GT.MXPB(2,ICH)) THEN
                MXPB(2,ICH) = BF(IATOM)
                MXRES2(ICH) = NRES(IATOM)
              ENDIF
C
C..Set pointers for ASN and GLN Ox1 and Nx2 atoms
              IF (ASN) THEN
                IF(NUMRES.EQ.RESPTR) THEN
                  IF(ANAME(IATOM).EQ.'OD1 ')THEN
                    ASNGLN(1,IASNGLN)=IATOM
                  ENDIF
                  IF(ANAME(IATOM).EQ.'ND2 ')THEN
                    ASNGLN(2,IASNGLN)=IATOM
                  ENDIF
                ELSE
                  ASN=.FALSE.
                ENDIF
              ENDIF
C
              IF (GLN) THEN
                IF(NUMRES.EQ.RESPTR) THEN
                  IF(ANAME(IATOM).EQ.'OE1 ')THEN
                    ASNGLN(1,IASNGLN)=IATOM
                  ENDIF
                  IF(ANAME(IATOM).EQ.'NE2 ')THEN
                    ASNGLN(2,IASNGLN)=IATOM
                  ENDIF
                ELSE
                  GLN=.FALSE.
                ENDIF
              ENDIF
              GOTO 3950
C
            ENDIF
 600      CONTINUE
C
C  Metals, etc
C
          DO 800 I = 1,MAXMET
C
            IF (ANAME(IATOM).EQ.METAL(I)) THEN
              FETBFAC(ICH) = FETBFAC(ICH) + BF(IATOM)
              NMET(ICH) = NMET(ICH) + 1
              GOTO 3950
            ENDIF
 800      CONTINUE
C
C  Anything else (inhibitors, sulphate,NAD,ATP......)
C
          INK  =  INK +1
          IF (INK.GT.MAXUNK) THEN
            WRITE(6,*)
     +      'Parameter MAXUNK (currently ',MAXUNK,
     +      ') exceeded in s/r FILSHEL'
            CALL CCPERR(1,'IN BSTATS: too many unknown atoms')
          END IF
          WRITE(UNK1,850)NRES(IATOM),RNAME(IATOM),ANAME(IATOM),IDCH(ICH)
 850      FORMAT(1X,'RES.',I3.3,1X,A,3X,A,' CHAIN ',A)
          UNKNOWN(INK)  =  UNK1
          IF (ICH.GT.MAXCH) CALL CCPERR(1,' BSTATS: ICH EXCEEDED MAXCH')
          OTHER(ICH) = OTHER(ICH) + 1
          OTBFAC(ICH) = OTBFAC(ICH) + BF(IATOM)
C
3950      CONTINUE
C
C..End of loop over atoms within a chain
4000    CONTINUE
C
C..End of loop over chains
5000  CONTINUE
C
C CALCULATE SOME TOTALS
C
      DO 6000 I = 1,NCH
C
        INRES = INRES + NRESID(I)
        INSOLV = INSOLV + NSOLV(I)
        INMET = INMET + NMET(I)
        TB = TB + TBFAC(I)
        SB = SB + SIDBFAC(I)
        SLB = SLB + SOLBFAC(I)
        TM = TM + FETBFAC(I)
        ITO  =  ITO  +  OTBFAC(I)
        IMAIN = IMAIN + NMAIN(I)
        ISIDE = ISIDE + NSIDE(I)
        ISOLV = ISOLV + NSOLV(I)
        IMET = IMET + NMET(I)
        ITOTHR = ITOTHR + OTHER(I)
6000  CONTINUE
C
      IPROT = IMAIN + ISIDE
      ITOTA = IPROT + ISOLV + IMET + ITOTHR
      IF (IMAIN.GT.0) TAVMN = TB/IMAIN
      IF (ISIDE.GT.0) TAVSD = SB/ISIDE
      IF (ISOLV.GT.0) TAVSLV = SLB/ISOLV
      IF (IPROT.GT.0) TAVPR = (TB + SB)/IPROT
      IF (IMET.GT.0)TMET = TM/IMET
      IF (ITOTHR.GT.0) TOTH = ITO/ITOTHR
      TAVALL = (TB + SB + SLB + TM + ITO)/ITOTA
      WRITE(IUOT,6010) INRES,INSOLV
6010  FORMAT(1X,'and the file contains ',I4,' amino acid residues',/,
     +         '                   and ',I4,' solvent molecules.'//)
      IF (INMET.GT.0) WRITE(IUOT,6015) INMET
6015  FORMAT(1X,'There are ',I3,' metals present '/)
      IF (ITOTHR.GT.0) WRITE(IUOT,6020) ITOTHR
6020  FORMAT(1X,'There are ',I3,' other atoms present '/)
      WRITE(IUOT,7942)
7942  FORMAT( ' ==========================='/
     *        ' TEMPERATURE FACTOR ANALYSIS'/
     *        ' ==========================='/
     *,1X,'Note: Standard Deviation and Average Deviation are ',
     * 'calculated after temperature'/,1X,'factors are averaged ',
     * 'for each residue (main chain and side chains separately).'/)
C
C..B-factor statistics by residue.
C
C..counter of bad residues (main chain)
      MBADCOUNT=0
C..counter of bad side chains
      ISBADCOUNT=0
      DO 3574 ICH=1,NCH
C..not a protein
        IF(NRESID(ICH).EQ.0) GOTO 3574
C
        IF(ICH.EQ.1) THEN
          ILOW=1
        ELSE
          ILOW=ILOW+NRESID(ICH-1)
        ENDIF
C
        IHIGH=ILOW+NRESID(ICH)-1
C
C..Main chain.
        ICOUNT=0
        DO 27 K=ILOW,IHIGH
C..Skip residues if all atoms have 0 occupancy
          IF(RESCTR(1,K).EQ.0) GOTO 27
          ICOUNT=ICOUNT+1
          VALUESM(ICOUNT)=BPLOT(1,K)/RESCTR(1,K)
          RLISTM(ICOUNT)=RLIST(K)
27      CONTINUE
C
C..Do statistics
        CALL MOMENT(VALUESM,ICOUNT,AVEM(ICH),ADEVM(ICH),
     *              SDEVM(ICH))
C
C Now tabulate some results
C
C..deviations by sigma
        DO 300 K=1,ICOUNT
          IF(VALUESM(K).LE.(AVEM(ICH)+SDEVM(ICH)).AND.
     *       VALUESM(K).GE.(AVEM(ICH)-SDEVM(ICH)))
     *               ISIGDEVM(ICH,1)=ISIGDEVM(ICH,1)+1
C
          IF(VALUESM(K).GT.(AVEM(ICH)+SDEVM(ICH)).AND.
     *       VALUESM(K).LE.(AVEM(ICH)+2*SDEVM(ICH)))
     *               ISIGDEVM(ICH,2)=ISIGDEVM(ICH,2)+1
          IF(VALUESM(K).LT.(AVEM(ICH)-SDEVM(ICH)).AND.
     *       VALUESM(K).GE.(AVEM(ICH)-2*SDEVM(ICH)))
     *               ISIGDEVM(ICH,2)=ISIGDEVM(ICH,2)+1
C
          IF(VALUESM(K).GT.(AVEM(ICH)+2*SDEVM(ICH)).AND.
     *       VALUESM(K).LE.(AVEM(ICH)+3*SDEVM(ICH)))
     *               ISIGDEVM(ICH,3)=ISIGDEVM(ICH,3)+1
          IF(VALUESM(K).LT.(AVEM(ICH)-2*SDEVM(ICH)).AND.
     *       VALUESM(K).GE.(AVEM(ICH)-3*SDEVM(ICH)))
     *               ISIGDEVM(ICH,3)=ISIGDEVM(ICH,3)+1
C
          IF(VALUESM(K).GT.(AVEM(ICH)+3*SDEVM(ICH))) THEN
                     ISIGDEVM(ICH,4)=ISIGDEVM(ICH,4)+1
C..make a note of this residue.
            MBADCOUNT=MBADCOUNT+1
            MBADLABEL(MBADCOUNT)=RLISTM(K)
            MBADLIST(MBADCOUNT)=VALUESM(K)
          ENDIF

          IF(VALUESM(K).LT.(AVEM(ICH)-3*SDEVM(ICH))) THEN
                     ISIGDEVM(ICH,4)=ISIGDEVM(ICH,4)+1
C..make a note of this residue.
            MBADCOUNT=MBADCOUNT+1
            MBADLABEL(MBADCOUNT)=RLISTM(K)
            MBADLIST(MBADCOUNT)=VALUESM(K)
          ENDIF
C
C..by average deviations
          IF(VALUESM(K).LE.(AVEM(ICH)+ADEVM(ICH)).AND.
     *       VALUESM(K).GE.(AVEM(ICH)-ADEVM(ICH)))
     *               MEANDEVM(ICH,1)=MEANDEVM(ICH,1)+1
C
          IF(VALUESM(K).GT.(AVEM(ICH)+ADEVM(ICH)).AND.
     *       VALUESM(K).LE.(AVEM(ICH)+2*ADEVM(ICH)))
     *               MEANDEVM(ICH,2)=MEANDEVM(ICH,2)+1
          IF(VALUESM(K).LT.(AVEM(ICH)-ADEVM(ICH)).AND.
     *       VALUESM(K).GE.(AVEM(ICH)-2*ADEVM(ICH)))
     *               MEANDEVM(ICH,2)=MEANDEVM(ICH,2)+1
C
          IF(VALUESM(K).GT.(AVEM(ICH)+2*ADEVM(ICH)).AND.
     *       VALUESM(K).LE.(AVEM(ICH)+3*ADEVM(ICH)))
     *               MEANDEVM(ICH,3)=MEANDEVM(ICH,3)+1
          IF(VALUESM(K).LT.(AVEM(ICH)-2*ADEVM(ICH)).AND.
     *       VALUESM(K).GE.(AVEM(ICH)-3*ADEVM(ICH)))
     *               MEANDEVM(ICH,3)=MEANDEVM(ICH,3)+1
C
          IF(VALUESM(K).GT.(AVEM(ICH)+3*ADEVM(ICH)))
     *               MEANDEVM(ICH,4)=MEANDEVM(ICH,4)+1

          IF(VALUESM(K).LT.(AVEM(ICH)-3*ADEVM(ICH)))
     *               MEANDEVM(ICH,4)=MEANDEVM(ICH,4)+1
C
300     CONTINUE
C
        MBADPT(ICH)=MBADCOUNT
C
C..Side chains.
        ICOUNT=0
        DO 28 K=ILOW,IHIGH
C..Skip residues if all atoms have 0 occupancy
          IF(RESCTR(2,K).EQ.0) GOTO 28
          ICOUNT=ICOUNT+1
          VALUESS(ICOUNT)=BPLOT(2,K)/RESCTR(2,K)
          RLISTS(ICOUNT)=RLIST(K)
28      CONTINUE
C
C..Do statistics
        CALL MOMENT(VALUESS,ICOUNT,AVES(ICH),ADEVS(ICH),
     *              SDEVS(ICH))
C
C Now tabulate some results
C
C..deviations by sigma
        DO 310 K=1,ICOUNT
          IF(VALUESS(K).LE.(AVES(ICH)+SDEVS(ICH)).AND.
     *       VALUESS(K).GE.(AVES(ICH)-SDEVS(ICH)))
     *               ISIGDEVS(ICH,1)=ISIGDEVS(ICH,1)+1
C
          IF(VALUESS(K).GT.(AVES(ICH)+SDEVS(ICH)).AND.
     *       VALUESS(K).LE.(AVES(ICH)+2*SDEVS(ICH)))
     *               ISIGDEVS(ICH,2)=ISIGDEVS(ICH,2)+1
          IF(VALUESS(K).LT.(AVES(ICH)-SDEVS(ICH)).AND.
     *       VALUESS(K).GE.(AVES(ICH)-2*SDEVS(ICH)))
     *               ISIGDEVS(ICH,2)=ISIGDEVS(ICH,2)+1
C
          IF(VALUESS(K).GT.(AVES(ICH)+2*SDEVS(ICH)).AND.
     *       VALUESS(K).LE.(AVES(ICH)+3*SDEVS(ICH)))
     *               ISIGDEVS(ICH,3)=ISIGDEVS(ICH,3)+1
          IF(VALUESS(K).LT.(AVES(ICH)-2*SDEVS(ICH)).AND.
     *       VALUESS(K).GE.(AVES(ICH)-3*SDEVS(ICH)))
     *               ISIGDEVS(ICH,3)=ISIGDEVS(ICH,3)+1
C
          IF(VALUESS(K).GT.(AVES(ICH)+3*SDEVS(ICH))) THEN
                     ISIGDEVS(ICH,4)=ISIGDEVS(ICH,4)+1
C..make a note of this residue.
            ISBADCOUNT=ISBADCOUNT+1
            SBADLABEL(ISBADCOUNT)=RLISTS(K)
            SBADLIST(ISBADCOUNT)=VALUESS(K)
          ENDIF

          IF(VALUESS(K).LT.(AVES(ICH)-3*SDEVS(ICH))) THEN
                     ISIGDEVS(ICH,4)=ISIGDEVS(ICH,4)+1
C..make a note of this residue.
            ISBADCOUNT=ISBADCOUNT+1
            SBADLABEL(ISBADCOUNT)=RLISTS(K)
            SBADLIST(ISBADCOUNT)=VALUESS(K)
          ENDIF
C
C..by average deviations
          IF(VALUESS(K).LE.(AVES(ICH)+ADEVS(ICH)).AND.
     *       VALUESS(K).GE.(AVES(ICH)-ADEVS(ICH)))
     *               MEANDEVS(ICH,1)=MEANDEVS(ICH,1)+1
C
          IF(VALUESS(K).GT.(AVES(ICH)+ADEVS(ICH)).AND.
     *       VALUESS(K).LE.(AVES(ICH)+2*ADEVS(ICH)))
     *               MEANDEVS(ICH,2)=MEANDEVS(ICH,2)+1
          IF(VALUESS(K).LT.(AVES(ICH)-ADEVS(ICH)).AND.
     *       VALUESS(K).GE.(AVES(ICH)-2*ADEVS(ICH)))
     *               MEANDEVS(ICH,2)=MEANDEVS(ICH,2)+1
C
          IF(VALUESS(K).GT.(AVES(ICH)+2*ADEVS(ICH)).AND.
     *       VALUESS(K).LE.(AVES(ICH)+3*ADEVS(ICH)))
     *               MEANDEVS(ICH,3)=MEANDEVS(ICH,3)+1
          IF(VALUESS(K).LT.(AVES(ICH)-2*ADEVS(ICH)).AND.
     *       VALUESS(K).GE.(AVES(ICH)-3*ADEVS(ICH)))
     *               MEANDEVS(ICH,3)=MEANDEVS(ICH,3)+1
C
          IF(VALUESS(K).GT.(AVES(ICH)+3*ADEVS(ICH)))
     *               MEANDEVS(ICH,4)=MEANDEVS(ICH,4)+1

          IF(VALUESS(K).LT.(AVES(ICH)-3*ADEVS(ICH)))
     *               MEANDEVS(ICH,4)=MEANDEVS(ICH,4)+1
C
310     CONTINUE
C
        SBADPT(ICH)=ISBADCOUNT
C
3574  CONTINUE
C
      DO 7000 I = 1,NCH
C
        IF (NMAIN(I).GT.0) AVMNB(I) = TBFAC(I)/NMAIN(I)
        IF (NSIDE(I).GT.0) AVSDB(I) = SIDBFAC(I)/NSIDE(I)
        NPROT(I) = NMAIN(I) + NSIDE(I)
        IF (NPROT(I).GT.0) AVPRB(I) = (TBFAC(I) + SIDBFAC(I))/NPROT(I)
        NTOT(I) = NPROT(I) + NSOLV(I) + NMET(I) + OTHER(I)
        IF (NTOT(I).GT.0) AVTOTB(I) = (TBFAC(I) + SIDBFAC(I) +
     +      FETBFAC(I) + SOLBFAC(I) + OTBFAC(I))/NTOT(I)
        IF (NSOLV(I).GT.0) THEN
          AVSLB(I) = SOLBFAC(I)/NSOLV(I)
        ENDIF
        IF (NMET(I).GT.0) THEN
          AVMTB(I) = FETBFAC(I)/NMET(I)
        ENDIF
        IF (OTHER(I).GT.0) THEN
          AVOTHB(I) = OTBFAC(I)/OTHER(I)
        ENDIF
C
        IF (NCH.GT.1)WRITE(IUOT,6025)IDCH(I)
6025    FORMAT(/,1X,'STATISTICS FOR CHAIN ',A,/,
     +                          ' ----------------------')
        WRITE(IUOT,6030)
        WRITE(IUOT,6035)NMAIN(I),NSIDE(I),NPROT(I),NSOLV(I),NMET(I),
     +             OTHER(I),NTOT(I)
        WRITE(IUOT,6040)AVMNB(I),AVSDB(I),AVPRB(I),AVSLB(I),AVMTB(I),
     +             AVOTHB(I),AVTOTB(I)
        WRITE (IUOT,6045) (MXPB(J,I),J = 1,3)
        WRITE(IUOT,6050)MXRES1(I),MXRES2(I),MXRES3(I)
6030    FORMAT(13X,'Main chain',3X,'Side chain',3X,'Protein',3X,
     +         'Solvent',3X,'Metal',3X,'Others',3X,'Total'/)
6035    FORMAT(1X,'No. atoms',5X,I6,7X,I6,6X,I6,4X,I6,4X,I4,4X,I4,2X,I6)
6040    FORMAT(1X,'Average B',5X,F6.2,7X,F6.2,6X,F6.2,4X,F6.2,2X,
     +         F6.2,2X,F6.2,2X,F6.2)
6045    FORMAT(1X,'Maximum B',5X,F6.2,7X,F6.2,16X,F6.2)
6050    FORMAT(1X,'on residue',6X,I4,9X,I4,18X,I4)
C
        IF(NRESID(I).GT.1) THEN 
        WRITE(IUOT,6051)SDEVM(I),SDEVS(I)
6051    FORMAT(1X,'Sigma',9X,F6.2,7X,F6.2)
        WRITE(IUOT,6052)ADEVM(I),ADEVS(I)
6052    FORMAT(1X,'Average Dev.',2X,F6.2,7X,F6.2)
        WRITE(IUOT,1030)
        WRITE(IUOT,1040)(ISIGDEVM(I,J),J=1,4)
        WRITE(IUOT,1050)(MEANDEVM(I,J),J=1,4)
1030    FORMAT(/,' Number of residues (averaged MAIN CHAIN)',
     *          ' which deviate from mean by :',/)
1040    FORMAT(' <1 sigma    1-2 sigma    2-3 sigma     >3 sigma',
     *         /,2X,4(I5,8X),/)
1050    FORMAT(' <1 av dev   1-2 av dev   2-3 av dev    >3 av dev',
     *         /,2X,4(I5,8X),/)
C List residues with large deviations
        WRITE(IUOT,1070)
1070    FORMAT(1X,' Residues which deviate from mean by more',
     *           ' than 3 sigma:',/)
C
        IF(I.EQ.1) THEN
          LABLOW=1
        ELSE
          LABLOW=LABHIGH+1
        ENDIF
C
        LABHIGH=MBADPT(I)
C
        DO 79 LOOP=LABLOW,LABHIGH
          WRITE(IUOT,1080)MBADLABEL(LOOP),MBADLIST(LOOP)
1080      FORMAT(5X,A12,4X,'Averaged B-value : ',F6.2)
 79     CONTINUE
C
        WRITE(IUOT,1090)
        WRITE(IUOT,1100)(ISIGDEVS(I,J),J=1,4)
        WRITE(IUOT,1110)(MEANDEVS(I,J),J=1,4)
1090    FORMAT(/,' Number of SIDE CHAINS which deviate from mean by :',
     *          /)
1100    FORMAT(' <1 sigma    1-2 sigma    2-3 sigma     >3 sigma',
     *         /,2X,4(I5,8X),/)
1110    FORMAT(' <1 av dev   1-2 av dev   2-3 av dev    >3 av dev',
     *         /,2X,4(I5,8X),/)
C
C List side chains with large deviations
        WRITE(IUOT,1120)
1120    FORMAT(1X,' Side chains which deviate from mean by more',
     *           ' than 3 sigma:',/)
C
        IF(I.EQ.1) THEN
          LABSLOW=1
        ELSE
          LABSLOW=LABSHIGH+1
        ENDIF
C
        LABSHIGH=SBADPT(I)
C
        DO 81 LOOP=LABSLOW,LABSHIGH
          WRITE(IUOT,1130)SBADLABEL(LOOP),SBADLIST(LOOP)
1130      FORMAT(5X,A12,4X,'Averaged B-value : ',F6.2)
  81    CONTINUE
C
        ENDIF
C
        IF (NO(I).GT.0) WRITE(IUOT,6055) NPO(I),NWO(I)
6055    FORMAT(/,1X,'Results do not include',I4,
     +         ' protein atoms  and ',I4,' solvent atoms',/,
     +         ' with zero occupancy. ',/)
7000  CONTINUE
C
      IF (NCH.GT.1) THEN
         WRITE(IUOT,7005)
7005     FORMAT(//,1X,'TOTALS:',/)
         WRITE(IUOT,6030)
         WRITE(IUOT,6035)IMAIN,ISIDE,IPROT,ISOLV,IMET,ITOTHR,ITOTA
         WRITE(IUOT,6040)TAVMN,TAVSD,TAVPR,TAVSLV,TMET,TOTH,TAVALL
      ENDIF
C
      IF (NBAD.EQ.0) GOTO 7030
      WRITE(IUOT,7010) BMONI
7010  FORMAT(///,1X,' Listing of atoms with B''s larger than',F5.1,':',
     +        //,1X,' Atom        Residue        B',/)
C
      DO 7020 I = 1,NBAD
C
         WRITE(IUOT,7015)BADAT(I),BADRES(I),IBADR(I),IDCH(ICHN(I)),
     +                   BADB(I)
7015     FORMAT(1X,A4,8X,A4,1X,I3,A,4X,F6.2)
7020  CONTINUE
C
      WRITE(IUOT,7025) NBAD
7025  FORMAT(/,1X,I3,' monitored atoms',/)
7030  CONTINUE
C
C..List B-factors for Ox1 and Nx2 atoms for GLN and ASN
      IF (IASNGLN.GT.0) THEN
         WRITE(IUOT,7035)
7035  FORMAT (//,1X,' Listing of B-values for Ox1 and Nx2 atoms',
     +  ' of GLN and ASN:',//,1X,' Residue     Oxygen    B-factor',
     +  '   Nitrogen    B-factor   Difference')
         DO 798 I=1,IASNGLN
           IF(ASNGLN(1,I).GT.0.AND.ASNGLN(2,I).GT.0)THEN
             BDIFF=ABS(BF(ASNGLN(1,I))-BF(ASNGLN(2,I)))
             WRITE(IUOT,'(2X,A,I4,A,5X,A,5X,F6.2,7X,A,6X,F6.2,5X,F6.2)')
     +             RNAME(ASNGLN(1,I)),NRES(ASNGLN(1,I)),CH(ASNGLN(1,I)),
     +             ANAME(ASNGLN(1,I)),BF(ASNGLN(1,I)),
     +             ANAME(ASNGLN(2,I)),BF(ASNGLN(2,I)),BDIFF
           ENDIF
  798    CONTINUE
      ENDIF

      IF (TOTH.GT.0) THEN
       WRITE(IUOT,'(///,1X,'' List of unrecognised residues follows'')')
        DO 7045 I  = 1,INK
          WRITE(IUOT,'(A)') UNKNOWN(I)
7045    CONTINUE
      ENDIF
C
C Histogram of B-values for waters
      SCHIST=1.
      IBHMAX=0
      DO 376 I=1,21
        IF(BH(I).GT.IBHMAX)IBHMAX=BH(I)
 376  CONTINUE
C
      IF(IBHMAX.GT.50)THEN
        SCHIST=50./REAL(IBHMAX)
      ENDIF
C
      RMN=-5.
      RMX=0.
      WRITE(IUOT,7050)
7050  FORMAT (///,' HISTOGRAM OF SOLVENT MOLECULES IN RANGES',
     +  ' OF B-VALUES',/,' ----------------------------',
     +  '------------------------')
      WRITE(IUOT,7060)
7060  FORMAT (7X,' Range',5X,' Number of solvent molecules',/)
      DO 265 I=1,20
        RMN=RMN+5.
        RMX=RMX+5.
        WRITE(IUOT,2389)RMN,RMX,BH(I),HISTO(1:NINT(BH(I)*SCHIST))
2389    FORMAT(1X,F6.1,'  -',F6.1,3X,I5,1X,A)
265   CONTINUE
      WRITE(IUOT,2390)RMX,BH(21),HISTO(1:NINT(BH(21)*SCHIST))
2390  FORMAT(3X,' .gt.',F6.1,5X,I5,1X,A,/)
C
      RETURN
      END
C
C--------------------------------------------------------------------
      SUBROUTINE PLBYRS(IUOT,RESCTR,BPLOT,RESPTR,RESLST,BYRES,CHFILE)
C--------------------------------------------------------------------
C
      PARAMETER (MAXRES=9999,MXDIM=2,MXBIN=50)
C
      INTEGER RESCTR(MXDIM,MAXRES),RESLST(MAXRES)
      INTEGER RESPTR
C
      REAL BPLOT(MXDIM,MAXRES)
      DOUBLE PRECISION VALUES(MXDIM,MAXRES)
      DOUBLE PRECISION YMIN,YMAX
      LOGICAL BYRES,CHFILE
      CHARACTER*20 FNAME
      CHARACTER*30 LABEL
      CHARACTER*4 CH(2)
C
      DATA LABEL /'B-Factor by residue'/
      DATA CH /'main','side'/
      YMIN = 1000.0
      YMAX = -1000.0
      DO 50 I = 1,RESPTR
C
        DO 40 K = 1,2
C
          VALUES(K,I) = 0.0D0
          NAT = RESCTR(K,I)
          IF (NAT.GT.0) THEN
            VALUES(K,I) = BPLOT(K,I)/RESCTR(K,I)
          ENDIF
          IF (K.EQ.2) VALUES(K,I) = VALUES(K,I)*(-1.0)
          YMAX = MAX(YMAX,VALUES(K,I))
          YMIN = MIN(YMIN,VALUES(K,I))
  40    CONTINUE
  50  CONTINUE
      NBINS = MXBIN
      NDIM = MXDIM
C
      IF (CHFILE) THEN
        FNAME = 'BRES.PLT'
        CALL DUMP(IUOT,FNAME,VALUES,LABEL,RESPTR,RESLST,CH)
      ENDIF
C
      IF (BYRES) THEN
        WRITE(IUOT,'(/,'' ********************************'')')
        WRITE(IUOT,'('' SORRY, PLOTTING NOT IMPLEMENTED.'')')
        WRITE(IUOT,'('' ********************************'')')
C
C For plotting using NAG library uncomment the following lines and link
C ACT with ACTPLOT (and NAG library).
C
C        CALL PLOTIT(AY,NBINS,NDIM,RESLST,YCHAR,VALUES,YMIN,YMAX,RESPTR,
C     +              LABEL)
      ENDIF
C
      RETURN
      END
C
C--------------------------------------------------------------
      SUBROUTINE DUMP(IUOT,FNAME,VALUES,LABEL,RESPTR,RESLST,CH)
C--------------------------------------------------------------
C
      PARAMETER (MAXRES=9999,MXDIM=2)
C
      INTEGER RESPTR,RESLST(MAXRES)
C
      DOUBLE PRECISION VALUES(MXDIM,MAXRES)
C
      CHARACTER*20 FNAME
      CHARACTER*30 LABEL
      CHARACTER*4 CH(2)
C
      IUNIT = 0
      CALL CCPDPN(IUNIT,FNAME,'NEW','F',0,0)
      WRITE(IUOT,10)FNAME
 10   FORMAT(1X,'Writing plot information to ',A)
C
      WRITE(IUNIT,20)LABEL
 20   FORMAT(A/)
      WRITE(IUNIT,25)(CH(I),I=1,2)
 25   FORMAT(2X,'Residue',6X,A,' chain',12X,A,' chain'/)
      DO 40 I = 1,RESPTR
        WRITE(IUNIT,30) RESLST(I),(VALUES(K,I),K=1,2)
 30     FORMAT(3X,I4,4X,D15.6,7X,D15.6)
 40   CONTINUE
C
      CLOSE(UNIT=IUNIT,STATUS='KEEP')
      RETURN
      END
C
C-----------------------------------------------
      SUBROUTINE CALCF0(NATOMS,ANAME,RNAME,F000)
C-----------------------------------------------
C Calculate number of electrons
C
      PARAMETER (MXATOM=99999)
C
      INTEGER NATOMS,F000,ATNUM(6)
C
      CHARACTER*4 ANAME(MXATOM),RNAME(MXATOM)
      CHARACTER*1 AT,ATOMS(6)
C
      DATA ATOMS /'C','O','N','H','S','P'/
      DATA ATNUM / 6,  8,  7,  1, 16, 15/
C
      F000 = 0
      DO 20 I = 1,NATOMS
        AT = ANAME(I)(1:1)
        DO 10 J = 1,6
          IF (AT.EQ.ATOMS(J)) THEN
            F000 = F000 + ATNUM(J)
          ENDIF
   10   CONTINUE
        IF (RNAME(I).EQ.'WAT '.OR.RNAME(I).EQ.'HOH ')
     +    F000 = F000 + 2
   20 CONTINUE
C
      RETURN
      END
C----------------------------------------------------------
      SUBROUTINE MOMENT(DATA,N,AVE,ADEV,SDEV)
C----------------------------------------------------------
C Given an array of DATA of length N, calculate AVErage value,
C AVErage Deviation, Standard DEViation.
C
      DIMENSION DATA(N)
      IF(N.LE.0) CALL CCPERR(1, 'MOMENT: n must be at least 1')
      if (n.eq.1) then
        ave = data(1)
        adev = 0.0
        sdev = 0.0
        return
      endif      
      S=0.
      DO 11 J=1,N
        S=S+DATA(J)
11    CONTINUE
      AVE=S/N
      ADEV=0.
      VAR=0.
      DO 12 J=1,N
        S=DATA(J)-AVE
        ADEV=ADEV+ABS(S)
        P=S*S
        VAR=VAR+P
12    CONTINUE
      ADEV=ADEV/N
      VAR=VAR/(N-1)
      SDEV=SQRT(VAR)
      RETURN
      END
C
C-----------------------------------------------------------------------
      SUBROUTINE CONTACT(IUOT,NATOMS,NCH,CHPTR,XYZ,ANAME,RNAME,NRES,OCC,
     +                   NRESID,IDCH,CH,RF,RO,NSYM,RSYM,DMIN,DMAX,
     +                   HBOND,INTER,INTRA,ALL,SHORT,DMONI)
C-----------------------------------------------------------------------
C Calculate contacts between atoms
C
      PARAMETER(MXBOX=100000,MXATSYM=400000)
C
      INTEGER NET(81*MXBOX),NRES(NATOMS),NRESID(NCH),CHPTR(NCH)
C
C..Arrays for coordinates of the primary atoms 
C  and in the symmetry related shell around the model
C  N.B. MXATSYM=40000 should be enough except for unusually large molecules,
C  then it can be increased.
C  PJX: 17/03/00 indeed MXATSYM increased to 100000!
      REAL XYZ(3,NATOMS),XYZS(3,MXATSYM),OCC(NATOMS)
      REAL XYZMAX(3),XYZMIN(3),
     +     XYZSMIN(3),XYZSMAX(3),
     +     XYZFMIN(3),XYZFMAX(3)
C
      CHARACTER*4 ANAME(NATOMS),RNAME(NATOMS)
      CHARACTER*1 CH(NATOMS),IDCH(NCH)
C
C..RF - matrix for converting cartesian coordinates to fractional
C..RO - matrix for converting fractional coordinates to cartesian
C..RSYM - symmetry operations
C..OPS  - record of how symmetry related atom was reduced to the box
C         around the primary molecule:
C         (symop # as in rsym, translations along x, y, z)
C..IPOINT - pointer from symmetry related atom to the original atom
      REAL RF(4,4),RO(4,4),RSYM(4,4,*)
      INTEGER OPS(4,MXATSYM),IPOINT(MXATSYM)
C
      INTEGER NSYM
      INTEGER NATOMSYM
C
      REAL DMIN,DMAX,DMONI
C
      LOGICAL HBOND,INTER,INTRA,ALL,SHORT,SYMATOM
C
C..(re)initialise - this routine can be called several times
      NATOMSYM=0
      NARRAY=81*MXBOX
      DO 100 I=1,NARRAY
         NET(I)=0
  100 CONTINUE
C
C
      IF (HBOND) THEN 
         WRITE(IUOT,1062)
1062     FORMAT(///,' ===================================',/,
     *              ' ANALYSIS OF POSSIBLE HYDROGEN BONDS',/,
     *              ' ===================================')
      ENDIF
C
      IF (ALL) THEN 
         WRITE(IUOT,1063)
1063     FORMAT(//,' ==============================================',/,
     *             ' ANALYSIS OF INTRA- AND INTERMOLECULAR CONTACTS',/,
     *             ' ==============================================')
      ENDIF
C
      IF (INTRA) THEN
         WRITE(IUOT,1064)
1064     FORMAT(//,' ===================================',/,
     *             ' ANALYSIS OF INTRAMOLECULAR CONTACTS',/,
     *             ' ===================================')
      ENDIF
C
      IF (INTER) THEN 
         WRITE(IUOT,1065)
1065     FORMAT(//,' ===================================',/,
     *             ' ANALYSIS OF INTERMOLECULAR CONTACTS',/,
     *             ' ===================================')
      ENDIF
C
C
      IF (SHORT) THEN 
         WRITE(IUOT,1067)
1067     FORMAT(//,' =========================',/,
     *             ' LISTING OF SHORT CONTACTS',/,
     *             ' =========================')
      ENDIF
C
      WRITE(IUOT,1069)
 1069 FORMAT (/,' Atoms with zero occupancy will be ignored.',/)
C
C Find limits for atoms.
      DO 10 I=1,3
         XYZMAX(I)=-1000.0
         XYZMIN(I)= 1000.0
10    CONTINUE
C
      DO 20 I=1,NATOMS
         DO 30 J=1,3
            XYZMIN(J)=AMIN1(XYZMIN(J),XYZ(J,I))
            XYZMAX(J)=AMAX1(XYZMAX(J),XYZ(J,I))
30       CONTINUE
20    CONTINUE
C
C Calculate the number of divisions along x, y, z.
      NX=INT((XYZMAX(1)-XYZMIN(1))/6)+3
      NY=INT((XYZMAX(2)-XYZMIN(2))/6)+3
      NZ=INT((XYZMAX(3)-XYZMIN(3))/6)+3
C
C      WRITE(IUOT,1010)NX,NY,NZ
C1010  FORMAT(/,' Number of boxes along X Y Z : ',3(2X,I5),/)
C
      IF(NX*NY*NZ.GT.MXBOX) CALL CCPERR(1,
     +         ' Number of boxes .gt. MXBOX - increase size of NET')
C
C..Extend the limits to make room for a shell of symmetry related atoms
        DO 40 I=1,3
           XYZSMIN(I)=XYZMIN(I)-6.
           XYZSMAX(I)=XYZMAX(I)+6.
   40   CONTINUE
C
C..Fill in net with atoms
      SYMATOM=.FALSE.
      CALL SETNET(IUOT,NET,NX,NY,NZ,XYZ,NATOMS,XYZSMIN,
     +            SYMATOM,HBOND,ANAME,OCC)
C
C..Inter- (and intra)molecular contacts
C
      IF(ALL.OR.INTER) THEN
C
C..Convert the limits to fractional
        CALL XYZMAT(XYZSMIN,XYZFMIN,RF)
        CALL XYZMAT(XYZSMAX,XYZFMAX,RF)
C
C..The box may extend over more than one unit cell (but not more than three)
        DO 50 I=1,3
           IF(ABS(XYZFMAX(I)-XYZFMIN(I)).GT.4.) THEN
              CALL CCPERR(1,'CONTACT: BOX WIDER THAN 4 UNIT CELLS')
           ENDIF
   50   CONTINUE
C
C..Generate symmetry related atoms that fit in the shell around the molecule
        CALL FILSHEL(IUOT,XYZ,NATOMS,XYZS,NATOMSYM,XYZSMIN,XYZSMAX,
     +               XYZFMIN,RSYM,NSYM,RF,RO,OPS,IPOINT,
     +               HBOND,ANAME,OCC)
C
C..Fill in net with symmetry related atoms
        SYMATOM=.TRUE.
        CALL SETNET(IUOT,NET,NX,NY,NZ,XYZS,NATOMSYM,XYZSMIN,
     +              SYMATOM,HBOND,ANAME,OCC)
C
C..End of intermolecular contacts
      ENDIF
C
C..Find contacts
        CALL FINDCNT(IUOT,NET,NX,NY,NZ,XYZ,NATOMS,XYZMIN,
     +               XYZS,NATOMSYM,OPS,IPOINT,
     +               ANAME,RNAME,NRES,NCH,CH,NRESID,CHPTR,IDCH,
     +               HBOND,INTER,INTRA,ALL,DMIN,DMAX,SHORT,DMONI)
C
      RETURN
      END
C
C--------------------------------------------------------------------
      SUBROUTINE SETNET(IUOT,NET,NX,NY,NZ,XYZ,NATOMS,XYZSMIN,
     +                  SYMATOM,HBOND,ANAME,OCC)
C--------------------------------------------------------------------
C Set up a grid.
      INTEGER NET(81,NX,NY,NZ),INMAX
      REAL XYZ(3,NATOMS),XYZSMIN(3),OCC(NATOMS)
      CHARACTER*4 ANAME(NATOMS)
      LOGICAL SYMATOM,HBOND
C
C
      INMAX=0
C
      DO 10 I=1,NATOMS
         IF(.NOT.SYMATOM) THEN
C
C..For HBOND option select only nitrogens and oxygens
           IF(HBOND)THEN
             IF(ANAME(I)(1:1).NE.'N'.AND.ANAME(I)(1:1).NE.'O')GOTO 10
           ENDIF
C
C..Ignore atoms with zero occupancy
           IF(OCC(I).EQ.0.0) GOTO 10
         ENDIF
C
C..N.B. The limits are offset to make room for a shell of symmetry related atoms,
         X=XYZ(1,I)-XYZSMIN(1)
         Y=XYZ(2,I)-XYZSMIN(2)
         Z=XYZ(3,I)-XYZSMIN(3)
C
         IX=INT((X+6)/6)
         IY=INT((Y+6)/6)
         IZ=INT((Z+6)/6)
C
C..first element: # atoms in this box
         NET(1,IX,IY,IZ)=NET(1,IX,IY,IZ)+1
         IN=NET(1,IX,IY,IZ)
         IF(INMAX.LT.IN) INMAX=IN
C
C..next elements: numbers of those atoms, -ve for symmetry related atoms
         IF(SYMATOM) THEN
            NET(IN+1,IX,IY,IZ)=-I
         ELSE
            NET(IN+1,IX,IY,IZ)=I
         ENDIF
C
10    CONTINUE
C
      IF(INMAX.GT.80) THEN
        CALL CCPERR(1,'SETNET: There are more than 80 atoms in one box')
      ENDIF
C
      RETURN
      END
C
C--------------------------------------------------------------------
      SUBROUTINE FINDCNT(IUOT,NET,NX,NY,NZ,XYZ,NATOMS,XYZMIN,
     +                   XYZS,NATOMSYM,OPS,IPOINT,
     +                   ANAME,RNAME,NRES,NCH,CH,NRESID,CHPTR,IDCH,
     +                   HBOND,INTER,INTRA,ALL,DMIN,DMAX,SHORT,DMONI)
C--------------------------------------------------------------------
C Find the contacts.
C
      PARAMETER(NUNPAIR=2000)
C
      INTEGER NET(81,NX,NY,NZ),NRES(NATOMS),NRESID(NCH),CHPTR(NCH)
      REAL XYZ(3,NATOMS),XYZS(3,NATOMSYM)
      REAL XYZMIN(3)
      CHARACTER*4 ANAME(NATOMS),RNAME(NATOMS)
      CHARACTER*1 CH(NATOMS),IDCH(NCH)
C
C..OPS  - record of how symmetry related atom was reduced to the box
C         around the primary molecule:
C         (symop # as in rsym, translations along x, y, z)
C..IPOINT - pointer from symmetry related atom to the original atom
      INTEGER OPS(4,NATOMSYM),IPOINT(NATOMSYM)
C
      LOGICAL HBOND,INTER,INTRA,ALL,SHORT
C
C..pointers and flag for unpaired hydrogen bonding atoms
      INTEGER IPUNPAIR(NUNPAIR)
      LOGICAL HBPAIR
C
      REAL DMIN,DMAX,DMONI

      DMIN2=DMIN*DMIN
      DMAX2=DMAX*DMAX
      DMONI2=DMONI*DMONI
C
      IF(SHORT) THEN
         WRITE(IUOT,1000) DMONI
1000     FORMAT (' Short contacts up to ',F5.2,
     +        ' angstroms will be listed.',/)
      ELSE
         WRITE(IUOT,1010) DMIN,DMAX
1010     FORMAT (' Contacts between ',F5.2,' and ',F5.2,
     +           ' angstroms will be listed.',/)
      ENDIF
C
C..loop over chains
      DO 10 ICH=1,NCH
C..not a protein
         IF(NRESID(ICH).EQ.0) GOTO 10
C
         WRITE(IUOT,1067)IDCH(ICH)
1067     FORMAT(/,1X,'CONTACTS FOR CHAIN ',A,/,1X,
     +                                 '--------------------',/)
C
         IF(INTER.OR.ALL) WRITE(IUOT,1077)
1077     FORMAT(1X,'Contacts with symmetry related atoms are marked ',
     +            'by ''*''',/,1X,'followed by NSYM and number of ',
     +            'translations of one unit cell ',/,1X,
     +            'along x, y and z.',/)
C
         WRITE(IUOT,'(57X,''DIST'',/)')
C
         ILOW=CHPTR(ICH)
C
         IF(ICH.EQ.NCH) THEN
            IHIGH=NATOMS
         ELSE
            IHIGH=CHPTR(ICH+1)-1
         ENDIF
C
C..initialise counter of unpaired hydrogen bonding atoms
         NN=0
C
C..loop over atoms within a chain
         DO 20 I=ILOW,IHIGH
C
C..For HBOND option select only nitrogens and oxygens
            IF(HBOND)THEN
              IF(ANAME(I)(1:1).NE.'N'.AND.ANAME(I)(1:1).NE.'O') THEN
                 GOTO 20
              ELSE
C
C..Initialise flag for hydrogen bonding pair
                 HBPAIR=.FALSE.
              ENDIF
            ENDIF
C
C..find out which box it's in.
            X=XYZ(1,I)-XYZMIN(1)
            Y=XYZ(2,I)-XYZMIN(2)
            Z=XYZ(3,I)-XYZMIN(3)
            IX=INT(X/6)+2
            IY=INT(Y/6)+2
            IZ=INT(Z/6)+2
C
C..loop over 27 bricks
            DO 30 J=IX-1,IX+1
            DO 30 K=IY-1,IY+1
            DO 30 L=IZ-1,IZ+1
C
C..# atoms in this brick
               IL=NET(1,J,K,L)
C..skip empty bricks
               IF(IL.EQ.0) GOTO 30
C..loop over atoms within one brick
               DO 40 II=2,IL+1
                  IA=NET(II,J,K,L)
C
C..if INTER set pick only symmetry related atoms or atoms belonging
C  to a different chain.
                  IF(INTER.AND.IA.GT.0) THEN
                    IF(CH(I).EQ.CH(IA)) GOTO 40
                  ENDIF
C
C..Not a symmetry related contact
C
                  IF(IA.GT.0) THEN

C Hydrogen bonding option selects only nitrogens and oxygens
                    IF(HBOND) THEN

C This section used to exclude various combinations of N and O:
C..exclude OG + OG and OH + OH
C..include O + OG and O + OH
C..exclude N + OG and N + OH
C This seemed to be too drastic and has been removed.

                      IF(CH(I).EQ.CH(IA))THEN
                        IF(NRES(I).EQ.NRES(IA)) GOTO 40
                      ENDIF

                    ELSE

C..ignore adjacent residues belonging to the same chain
                      IF(CH(I).EQ.CH(IA))THEN
                        IF(NRES(I).GE.(NRES(IA)-1).AND.
     +                     NRES(I).LE.(NRES(IA)+1)) GOTO 40
                      ENDIF

                    ENDIF
C
                    XDIF=XYZ(1,I)-XYZ(1,IA)
                    YDIF=XYZ(2,I)-XYZ(2,IA)
                    ZDIF=XYZ(3,I)-XYZ(3,IA)
C
C..Symmetry related contact
C
                  ELSE

C Hydrogen bonding option selects only nitrogens and oxygens
                    IF(HBOND) THEN

C This section used to exclude various combinations of N and O:
C..exclude OG + OG and OH + OH
C..include O + OG and O + OH
C..exclude N + OG and N + OH
C This seemed to be too drastic and has been removed.
                      CONTINUE

                    ENDIF
C
                    XDIF=XYZ(1,I)-XYZS(1,ABS(IA))
                    YDIF=XYZ(2,I)-XYZS(2,ABS(IA))
                    ZDIF=XYZ(3,I)-XYZS(3,ABS(IA))

                  ENDIF
C..check distance
                  DIST2=XDIF*XDIF+YDIF*YDIF+ZDIF*ZDIF
C
                  IF(SHORT) THEN
C
C..Checking for short contacts
                     IF(DIST2.GT.DMONI2) GOTO 40
C
                     DIST=SQRT(DIST2)
C..output
                     IF(IA.GT.0) THEN
                        WRITE(IUOT,1050)NRES(I),CH(I),RNAME(I),ANAME(I),
     +                              NRES(IA),CH(IA),RNAME(IA),ANAME(IA),
     +                              DIST
                     ELSE
                        WRITE(IUOT,1055)NRES(I),CH(I),RNAME(I),ANAME(I),
     +                                  NRES(IPOINT(ABS(IA))),
     +                                  CH(IPOINT(ABS(IA))),
     +                                  RNAME(IPOINT(ABS(IA))),
     +                                  ANAME(IPOINT(ABS(IA))),
     +                                  (OPS(LL,ABS(IA)),LL=1,4),DIST
                     ENDIF
C
C..Checking other contacts
                  ELSE
                     IF(DIST2.GT.DMAX2.OR.DIST2.LT.DMIN2) GOTO 40
C
                     DIST=SQRT(DIST2)
C
C..set up a flag for hydrogen bonding pair (used if HATOM set)
                     HBPAIR=.TRUE.
C..output
                     IF(IA.GT.0) THEN
                        WRITE(IUOT,1050)NRES(I),CH(I),RNAME(I),ANAME(I),
     +                              NRES(IA),CH(IA),RNAME(IA),ANAME(IA),
     +                              DIST
1050                    FORMAT(1X,I5,1X,A,1X,A4,1X,A4,'....',
     +                            I5,1X,A,1X,A4,1X,A4,17X,F5.2)
                     ELSE
                        WRITE(IUOT,1055)NRES(I),CH(I),RNAME(I),ANAME(I),
     +                                  NRES(IPOINT(ABS(IA))),
     +                                  CH(IPOINT(ABS(IA))),
     +                                  RNAME(IPOINT(ABS(IA))),
     +                                  ANAME(IPOINT(ABS(IA))),
     +                                  (OPS(LL,ABS(IA)),LL=1,4),DIST
1055                    FORMAT(1X,I5,1X,A,1X,A4,1X,A4,'....',
     +                            I5,1X,A,1X,A4,1X,A4,
     +                            '* ',4(1X,I2),3X,F5.2)
                     ENDIF
                  ENDIF
C
C..end of atoms-in-one-brick loop
   40          CONTINUE
C
C..end of 27-brick loop
   30       CONTINUE
C
C..sep up a pointer if hydrogen bonding atom is unpaired
            IF(HBOND.AND.ALL)THEN
              IF(.NOT.HBPAIR)THEN
                NN=NN+1
                IF (NN.GT.NUNPAIR) CALL CCPERR(1,
     +               'IPUNPAIR too small: increase NUNPAIR in FINDCNT')
                IPUNPAIR(NN)=I
              ENDIF
            ENDIF
C
C..end of loop of atoms within a chain
   20    CONTINUE
C
C..output a list of unpaired hydrogen bonding atoms
      IF(HBOND.AND.ALL) THEN
         WRITE(IUOT,1060)
1060     FORMAT (/,1X,' The following hydrogen bonding atoms do not',
     +    ' have a pair',/,1X,' (within the specified limits):',/)
         DO 50 KK=1,NN
            WRITE(IUOT,'(1X,I5,1X,A,1X,A4,1X,A4)')NRES(IPUNPAIR(KK)),
     +                                        CH(IPUNPAIR(KK)),
     +                                        RNAME(IPUNPAIR(KK)),
     +                                        ANAME(IPUNPAIR(KK))
   50    CONTINUE
      ENDIF
C
C..end of chains loop
   10 CONTINUE
C
      RETURN
      END
C
C----------------------------------------------------------------------
      SUBROUTINE FILSHEL(IUOT,XYZ,NATOMS,XYZS,NATOMSYM,XYZSMIN,XYZSMAX,
     +                   XYZFMIN,RSYM,NSYM,RF,RO,OPS,IPOINT,
     +                   HBOND,ANAME,OCC)
C----------------------------------------------------------------------
C Generate symmetry related atoms and select those that can be translated
c to the box around the primary molecule.
      PARAMETER(MXATSYM=400000)
C
      REAL XYZ(3,NATOMS),OCC(NATOMS),XYZS(3,MXATSYM)
      REAL XYZSMAX(3),XYZSMIN(3)
      REAL XYZFMIN(3)
      REAL FRXYZ(3),FRXYZS(3),W(3),WO(3)
      INTEGER NATOMSYM
      CHARACTER*4 ANAME(NATOMS)
C
C..RF - matrix for converting cartesian coordinates to fractional
C..RO - matrix for converting fractional coordinates to cartesian
C..RSYM - symmetry operations
C..OPS  - record of how symmetry related atom was reduced to the box
C         around the primary molecule:
C         (symop # as in rsym, translations along x, y, z)
C..IPOINT - pointer from symmetry related atom to the original atom
      REAL RF(4,4),RO(4,4),RSYM(4,4,*)
      INTEGER OPS(4,MXATSYM),IPOINT(MXATSYM)
C
      INTEGER NSYM
C
      LOGICAL HBOND
C
C..Loop over all atoms..
      NATOMSYM=0
      DO 10 I=1,NATOMS
C
C..for HBOND option select only nitrogens and oxygens
         IF(HBOND)THEN
            IF(ANAME(I)(1:1).NE.'N'.AND.ANAME(I)(1:1).NE.'O') GOTO 10
         ENDIF
C
C..Ignore atoms with zero occupancy
         IF(OCC(I).EQ.0.0) GOTO 10
C
C..Convert atom to fractional coordinates (more convenient to manipulate)
         CALL XYZMAT(XYZ(1,I),FRXYZ,RF)
C
C..First test translations of -+1 cell from the original position
         DO 20 J=-1,1
            DO 30 K=-1,1
               DO 40 L=-1,1
                  W(1)=FRXYZ(1)+J
                  W(2)=FRXYZ(2)+K
                  W(3)=FRXYZ(3)+L
C
C..Transform back to cartesian and check if in box
                  CALL XYZMAT(W,WO,RO)
C
                  IF(WO(1).LT.XYZSMIN(1)) GOTO 45
                  IF(WO(2).LT.XYZSMIN(2)) GOTO 45
                  IF(WO(3).LT.XYZSMIN(3)) GOTO 45
                  IF(WO(1).GT.XYZSMAX(1)) GOTO 45
                  IF(WO(2).GT.XYZSMAX(2)) GOTO 45
                  IF(WO(3).GT.XYZSMAX(3)) GOTO 45
C
C..Exclude the original position
                  IF(J.EQ.0.AND.K.EQ.0.AND.L.EQ.0) GOTO 45
C
                  NATOMSYM=NATOMSYM+1
                  IF (NATOMSYM.GT.MXATSYM) THEN
                    WRITE(6,FMT='(X,A,I8,A)')
     +              'Parameter MXATSYM (currently ',MXATSYM,
     +              ') exceeded in s/r FILSHEL'
                    CALL CCPERR(1,
     +              'IN FILSHEL: too many symmetry generated atoms')
                  END IF
C     
                  XYZS(1,NATOMSYM)=WO(1)
                  XYZS(2,NATOMSYM)=WO(2)
                  XYZS(3,NATOMSYM)=WO(3)
C
C..Make a record of this transformation
                  OPS(1,NATOMSYM)=1
                  OPS(2,NATOMSYM)=J
                  OPS(3,NATOMSYM)=K
                  OPS(4,NATOMSYM)=L
C
C..set up a pointer to the original atom
                  IPOINT(NATOMSYM)=I
C
   45             CONTINUE
C
   40          CONTINUE
   30       CONTINUE
   20    CONTINUE
C
C..Loop over symmetry operations
         DO 60 J=2,NSYM
C
C..Transform
            CALL XYZTR(FRXYZ,FRXYZS,RSYM,J)
C
C..Test translations to put atom in the box, starting at the lower edge
            IXSHIFT=NINT(XYZFMIN(1)-FRXYZS(1))
            IYSHIFT=NINT(XYZFMIN(2)-FRXYZS(2))
            IZSHIFT=NINT(XYZFMIN(3)-FRXYZS(3))
            FRXYZS(1)=FRXYZS(1)+IXSHIFT
            FRXYZS(2)=FRXYZS(2)+IYSHIFT
            FRXYZS(3)=FRXYZS(3)+IZSHIFT
            DO 70 JJ=0,2
              DO 80 KK=0,2
                DO 90 LL=0,2
                  W(1)=FRXYZS(1)+JJ
                  W(2)=FRXYZS(2)+KK
                  W(3)=FRXYZS(3)+LL
C
C..Transform back to cartesian and check if in box
                  CALL XYZMAT(W,WO,RO)
C
                  IF(WO(1).LT.XYZSMIN(1)) GOTO 95
                  IF(WO(2).LT.XYZSMIN(2)) GOTO 95
                  IF(WO(3).LT.XYZSMIN(3)) GOTO 95
                  IF(WO(1).GT.XYZSMAX(1)) GOTO 95
                  IF(WO(2).GT.XYZSMAX(2)) GOTO 95
                  IF(WO(3).GT.XYZSMAX(3)) GOTO 95
C
                  NATOMSYM=NATOMSYM+1
C
                  XYZS(1,NATOMSYM)=WO(1)
                  XYZS(2,NATOMSYM)=WO(2)
                  XYZS(3,NATOMSYM)=WO(3)
C
C..Make a record of this transformation
                  OPS(1,NATOMSYM)=J
                  OPS(2,NATOMSYM)=IXSHIFT+JJ
                  OPS(3,NATOMSYM)=IYSHIFT+KK
                  OPS(4,NATOMSYM)=IZSHIFT+LL
C
C..set up a pointer to the original atom
                  IPOINT(NATOMSYM)=I
C
   95             CONTINUE
C
C..End of atom testing loops
   90           CONTINUE
   80         CONTINUE
   70       CONTINUE
C
C..End of symmetry loop
   60    CONTINUE
C
C..End of atom loop
   10 CONTINUE
C
      RETURN
      END
C
C-----------------------------------------
      SUBROUTINE XYZMAT(XYZIN,XYZOUT,RMAT)
C-----------------------------------------
C Multiply coordinates by a matrix
C
      REAL XYZIN(3),XYZOUT(3),RMAT(4,4)
C
      XYZOUT(1)=RMAT(1,1)*XYZIN(1)
     +        + RMAT(1,2)*XYZIN(2)
     +        + RMAT(1,3)*XYZIN(3)

      XYZOUT(2)=RMAT(2,1)*XYZIN(1)
     +        + RMAT(2,2)*XYZIN(2)
     +        + RMAT(2,3)*XYZIN(3)

      XYZOUT(3)=RMAT(3,1)*XYZIN(1)
     +        + RMAT(3,2)*XYZIN(2)
     +        + RMAT(3,3)*XYZIN(3)
C
      RETURN
      END
C
C------------------------------------------
      SUBROUTINE XYZTR(XYZIN,XYZOUT,RSYM,J)
C------------------------------------------
C Transform coordinates through a symmetry operation.
C
      REAL XYZIN(3),XYZOUT(3)
      REAL RSYM(4,4,*)
C
C
      XYZOUT(1)=RSYM(1,1,J)*XYZIN(1)
     +        + RSYM(1,2,J)*XYZIN(2)
     +        + RSYM(1,3,J)*XYZIN(3)
C
      XYZOUT(1)=XYZOUT(1)+RSYM(1,4,J)
C
C
      XYZOUT(2)=RSYM(2,1,J)*XYZIN(1)
     +        + RSYM(2,2,J)*XYZIN(2)
     +        + RSYM(2,3,J)*XYZIN(3)
C
      XYZOUT(2)=XYZOUT(2)+RSYM(2,4,J)
C
C
      XYZOUT(3)=RSYM(3,1,J)*XYZIN(1)
     +        + RSYM(3,2,J)*XYZIN(2)
     +        + RSYM(3,3,J)*XYZIN(3)
C
      XYZOUT(3)=XYZOUT(3)+RSYM(3,4,J)
C
C
      RETURN
      END
C
C-------------------------------------------
      SUBROUTINE SEQ(IUOT,NCH,NRESID,RESIDS)
C-------------------------------------------
C Extract sequence.
C
      PARAMETER (MAXCH=30,LONGLN=80,SHORTLN=60,LONGWD=4,SHORTWD=2,
     +           MAXAA=20)

      CHARACTER*1 BLANK
      INTEGER HIST(MAXAA), NRESID(MAXCH),SPTR
      CHARACTER*4 AACID(MAXAA)
      CHARACTER*1 SHORT(MAXAA)
      CHARACTER HGM*52
      CHARACTER*3000 SLINE
      CHARACTER*40000 RESIDS
      CHARACTER*80 FULNAM
      COMMON /NAME/ FULNAM
      LOGICAL FOUND
      DATA BLANK /' '/
      DATA AACID  /'ALA ','ARG ','ASN ','ASP ','CYS ','GLN ','GLU ',
     +             'GLY ','HIS ','ILE ','LEU ','LYS ','MET ','PHE ',
     +             'PRO ','SER ','THR ','TRP ','TYR ','VAL '/
      DATA SHORT  /'A','R','N','D','C','Q','E','G','H','I','L','K',
     +             'M','F','P','S','T','W','Y','V'/
      DATA HGM /' '/

      WRITE(IUOT,106)
106   FORMAT(//' ================================='/
     *         ' SEQUENCE INFORMATION AND ANALYSIS'/
     *         ' ================================='/)
      DO 124 J=1,MAXAA
        HIST(J)=0
124   CONTINUE
      MK = 0
      DO 125 J=1,3000
        SLINE(J:J)=BLANK
125   CONTINUE
      LETT=LONGWD
      LLINE=LONGLN
C
      CALL PRINTSEQ(IUOT,NCH,NRESID,RESIDS,LETT,LLINE)
C
      LPTR=1
      SPTR=2
      DO 2000 I=1,NCH
         DO 1500 K=1,NRESID(I)
         FOUND=.FALSE.
         DO 135 J=1,MAXAA
            IF(RESIDS(LPTR:LPTR+3).EQ.AACID(J)) THEN
               FOUND=.TRUE.
               NAA=J
            ENDIF
135      CONTINUE
         IF(.NOT.FOUND) THEN
            WRITE(IUOT,137)RESIDS(LPTR:LPTR+3)
137         FORMAT(1X,'Failed to recognise residue ',A)
            NOTRES=NOTRES+1
            SLINE(SPTR:SPTR)='X'
         ELSE
            HIST(NAA)=HIST(NAA)+1
            SLINE(SPTR:SPTR)=SHORT(NAA)
         ENDIF
         SPTR=SPTR+2
         LPTR=LPTR+4
1500     CONTINUE
2000  CONTINUE
      LETT=SHORTWD
      LLINE=SHORTLN
C
      CALL PRINTSEQ(IUOT,NCH,NRESID,SLINE,LETT,LLINE)
C
      WRITE(IUOT,150)
150   FORMAT(//,1X,'Amino acid composition',//)
      NTOT=0
      DO 160 I=1,MAXAA
         WRITE(IUOT,155)HIST(I),AACID(I)
155      FORMAT(1X,I4,1X,A,' residues')
         NTOT=NTOT+HIST(I)
160   CONTINUE
      WRITE(IUOT,163)NTOT
163   FORMAT(/,1X,'Total number of amino acid residues =',I4)
      WRITE(IUOT,165)BLANK
165   FORMAT(A)
      DO 170 I=1,MAXAA
         IF(HIST(I).GT.MK) MK=HIST(I)
170   CONTINUE
      DO 180 I=MK,1,-1
         DO 175 J=1,MAXAA
            JJ=J*2
            IF(HIST(J).EQ.I)HGM(JJ-1:JJ)=' *'
175      CONTINUE
         WRITE(IUOT,185)HGM
180   CONTINUE
185   FORMAT(1X,A)
      WRITE(IUOT,190)(SHORT(I),I=1,MAXAA)
190   FORMAT(1X,20(1X,A1))
      RETURN
      END
C
C-------------------------------------------------------------
         SUBROUTINE PRINTSEQ(IUOT,NCHAIN,NRESID,CH,LETT,LLINE)
C-------------------------------------------------------------
         PARAMETER (MAXCH=30)
         CHARACTER*(*) CH
         CHARACTER*1 BLANK
         INTEGER NRESID(MAXCH)
         BLANK=' '
         NBASE=1
         DO 150 NN=1,NCHAIN
            IF(NCHAIN.GT.1)WRITE(IUOT,50)NN
 50         FORMAT(1X,'Chain ',I2)
            LENGTH=NRESID(NN)*LETT
            NREM=MOD(LENGTH,LLINE)
            IF (NREM.GT.0) THEN
               LINES=(LENGTH/LLINE)+1
            ELSE
               LINES=LENGTH/LLINE
            ENDIF
         DO 130 I=1,LINES
            IF(I.EQ.1)THEN
              NP=NBASE
            ELSE
              NP=NQ+1
            ENDIF
            NQ=NP+LLINE-1
            IF ((NQ-NBASE).GT.LENGTH) THEN
               NUMLEFT=NRESID(NN)-((LLINE/LETT)*(I-1))
               NQ=NP+(NUMLEFT*LETT)-1
               NBASE=NQ+1
            ENDIF
            WRITE(IUOT,100)CH(NP:NQ)
100         FORMAT(1X,A)
130      CONTINUE
         WRITE(IUOT,100)BLANK
150      CONTINUE
         RETURN
         END
CCCC
CCCC-----------------------------------------------------------------
CCC      SUBROUTINE PLOTIT(AY,NBINS,NDIM,XLAB,YCHAR,VALUES,YMIN,YMAX,
CCC     +                  NTOTAL,LABEL)
CCCC-----------------------------------------------------------------
CCCC Plot a histogram of B-values by residue, using the NAG graphics library.
CCCC
CCCC  XLAB IS AN INTEGER ARRAY OF THE X AXIS INCREMENTS (RESIDUE
CCCC     NUMBERS)
CCCC  YCHAR is CHARACTER LABEL FOR THE Y-AXIS
CCCC  VALUES IS A 2-D ARRAY OF THE VALUES TO BE PLOTTED (ONE FOR MAIN
CCCC     CHAIN, ONE FOR SIDE CHAIN)
CCCC  NDIM IS SIZE OF FIRST DIMENSION OF VALUES
CCCC  NBINS IS NUMBER OF BINS ALONG HORIZONTAL
CCCC  AY IS ADJUSTABLE ARRAY OF DIMENSION NDIM x NBINS INTO WHICH VALUES
CCCC     ARE READ FOR PLOTTING
CCCC  YMIN AND YMAX ARE 'ADJUSTED' MIN AND MAX OF VALUES. THE NAG MANUAL
CCCC     DESCRIBES THE BEST MIN AND MAX TO USE.
CCCC  NTOTAL IS THE NUMBER OF BINS IN TOTAL TO BE PLOTTED (NO. OF RESIDUES)
CCCC  LABEL IS A CHARACTER STRING USED AS HEADER LABEL FOR THE PLOT
CCCC
CCCC
CCC      INTEGER MXPLOT,MXNVAL,MXNDIM,MXNBIN,MXWORK
CCC      PARAMETER (MXPLOT=10,MXNVAL=1200,MXNDIM=4,MXNBIN=100,
CCC     +           MXWORK=500)
CCCC
CCC      INTEGER XLAB(MXNVAL)
CCC      INTEGER NOUT,MARGIN,IVAX,IANNOT,IBRUSH,IPALET,NBINS,NLABEL,NFILL,
CCC     +        MREP,NRESID,I,N,M,J,K,IR,NTOTAL,NPLOTS,IFAIL,NDIM,NWORK
CCCC
CCC      DOUBLE PRECISION VALUES(2,MXNVAL),WORK(MXWORK),
CCC     +       AY(NDIM,NBINS)
CCC      DOUBLE PRECISION UMIN,UMAX,VMIN,VMAX,YMIN,YMAX,XMIN,XMAX,
CCC     +       FRAC,YRANGE,ZERO,AMIN,AMAX,SCALE
CCCC
CCC      CHARACTER*20 YCHAR
CCC      CHARACTER*30 LABEL
CCCC
CCC      SCALE = 1.45
CCC      ZERO = 0.0D0
CCC      NOUT = 6
CCC      NWORK = MXWORK
CCC      MARGIN = 1
CCC      UMIN = 0.0
CCC      UMAX = SCALE
CCC      VMIN = 0.0
CCC      VMAX = 1.0
CCC      IVAX = 1                       
CCC      IANNOT = 1
CCC      IBRUSH = 2
CCC      IPALET = 2
CCC      NLABEL = NBINS/5
CCC      NFILL = NDIM
CCC      FRAC = 0.7E0
CCC      MREP = -1
CCCC
CCC      YMIN = MIN(ZERO,YMIN)
CCCC
CCC      CALL J06VAF(1,NOUT)
CCCC
CCC      CALL PAPER(1)
CCCC
CCCC  Setting up of fill areas as in the NAG manual
CCCC                                           
CCC      CALL J06WAF
CCC      CALL J06WCF(UMIN,UMAX,VMIN,VMAX)
CCCC
CCC      DO 5 I = 1,2
CCC        CALL J06XKF(I,IPALET,I,0,I)
CCC   5  CONTINUE
CCC      CALL J06XJF(IPALET-1,.FALSE.,.FALSE.)
CCC      CALL J06XMF(0.33E0)
CCCC
CCC      CALL J06XFF(2)                                            
CCCC
CCC      YRANGE = YMAX - YMIN
CCC      YMAX = YMAX + 0.1E0*YRANGE
CCCC             
CCC      CALL J06WBF(0.0,1.0,YMIN,YMAX,MARGIN)
CCC      CALL J06WCF(UMIN,UMAX,VMIN,VMAX)
CCC      IR = MOD(NTOTAL,NBINS)
CCC      IF (IR.EQ.0) THEN
CCC        NPLOTS = NTOTAL/NBINS
CCC      ELSE
CCC        NPLOTS = NTOTAL/NBINS + 1
CCC      ENDIF
CCC      IFAIL = 0
CCC      NRESID = 0
CCCC
CCC      DO 50 I=1,NPLOTS
CCCC
CCC        DO 20 N=1,NBINS
CCC          DO 10 M=1,NDIM
CCC            AY(M,N) = 0.0
CCC  10      CONTINUE
CCC  20    CONTINUE
CCCc
CCC        DO 40 J=1,NBINS
CCC          NRESID = NRESID + 1
CCC          IRES = NBINS * (I-1) + J
CCC          IF (NRESID.LE.NTOTAL) THEN
CCC            DO 30 K=1,NDIM
CCC              AY(K,J) = VALUES(K,IRES)
CCC  30        CONTINUE
CCC          ELSE
CCC            GO TO 45
CCC          ENDIF
CCC  40    CONTINUE
CCC  45    CONTINUE
CCCC
CCC        CALL J06JBF(AY,NDIM,NFILL,NBINS,FRAC,MREP,IANNOT,IBRUSH,
CCC     +              YCHAR,IVAX,WORK,NWORK,NLABEL,IFAIL)
CCC        CALL J06AHF(LABEL)
CCC        CALL J06WDF
CCCC
CCC  50  CONTINUE
CCC      CALL J06WZF
CCCC
CCC      RETURN 
CCC      END

