C
C     ANISOANL
C     Copyright (C) 2000-2004 Martyn Winn
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
      PROGRAM ANISOANL

C     A program for bringing together various analyses
C     of anisotropic displacement parameters

      INCLUDE 'anisoanl.fh'

      CHARACTER TLSFIL*200

C----Initialize ccp4 like file organization
      CALL CCP4_PROG_VERSION('0.1',0)
      CALL CCPFYP
      CALL CCPRCS(6,'ANISOANL','$Date: 2008/07/09 13:29:36 $')

      CALL XYZINIT

C----Read in keyworded input
      CALL RCARD

      WRITE (6,FMT=9020) TITLE(1:LENSTR(TITLE))
 9020 FORMAT (/,' Title : ',A,/)

C---Read in XYZIN
      CALL RDXYZ

C----Not using lower-triangular matrix formulation
      USELTM = .FALSE.

      CALL UGTENV('TLSIN',TLSFIL)
      IF (LFITTLS.OR.LRIGBODY.OR.TLSFIL.NE.' ') THEN 
C---If doing TLS fitting, rigid-body analysis and/or TLSIN is set, 
C   define and initialise tls groups. If not fitting, but TLSIN is set,
C   then use TLSIN to define rigid groups for other analyses.
         CALL TLS_INIT
      ENDIF

C---Fit TLS parameters to U_OBS
C   U's from TLS parameters are U_ANISO
C   Includes writing TLS file.
      IF (LFITTLS) CALL TLS_FIT

C---Assess 'rigid-body' postulate for U_OBS
      IF (LRIGBODY) CALL RIGIDBODY

C---Plot initial and (if LFITTLS) residual U's
      IF (LDOPLOTS) CALL UPLOT

C---Write out XYZOUT
      IF (LFITTLS) CALL WRTXYZ

      CALL CCPERR(0,'End of anisoanl version 0.1')
      END

      SUBROUTINE RCARD

      INCLUDE 'anisoanl.fh'

      PARAMETER (NPARS = 200)
C
c---Things for parser
      CHARACTER LINE*600,KEY*4,CVALUE(NPARS)*4,CONOF*2
      INTEGER IBEG(NPARS),IEND(NPARS),IDEC(NPARS),ITYP(NPARS)
      REAL FVALUE(NPARS)
      LOGICAL LEND,LPRINT
C
      PI     = ATAN2(1.0,1.0)*4.0
      PISQ8  = PI*PI*8.0
      TOL    = 1.0E-8
      RTODEG = 180.0/PI
      DEGTOR = PI/180.0
C
C---Atom defaults
      DO I=1,QQA
        IGROUP(I) = 0
        DO J=1,6
          U_ANISO(I,J) = 0.0
        ENDDO
      ENDDO
C
C---TLS defaults
      DO    ITLSGRP=1,MAXTLSGRP
        ITLSGRP_PIECES(ITLSGRP) = 0
        TLSGRP_TITLE(ITLSGRP)    = 'From REFMAC'
        DO   IPIECE=1,MAXPIECES_TLS
          TLSGRP_CHN(IPIECE,ITLSGRP)    = ' '
          ITLSGRP_FROM(IPIECE,ITLSGRP) = -1
          ITLSGRP_TO(IPIECE,ITLSGRP)   = -1
          DO ISEL = 1,MAXSELECT
            TLSGRP_SELECT(ISEL,IPIECE,ITLSGRP) = ' '
          ENDDO
        ENDDO
      ENDDO
      DO I=1,MAXTLSGRP20
        SHFT_TLS(I)=0.0
      ENDDO
      NTLSGRP    = 1

C--- Keyword defaults
      NTLS_CYCLE = 2
      DELTAURANGE = 0.3
      NDELTAUBINS = 30
      NPSBINS = 10
      LFITTLS  = .TRUE.
      LDOPLOTS = .TRUE.
      LRIGBODY = .TRUE.
      LMAINCHAIN = .FALSE.
      VERBOSE  = .FALSE.

C--- PARSER
      NSYSFIL = 1
 10   CONTINUE
      LINE = ' '
      KEY  = ' '
      NTOK = NPARS
      LPRINT = .FALSE.
C          ***********************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +            .TRUE.)
C          **************
      IF(LEND) GOTO 999
C
      CALL CCPUPC(KEY)
      DO    IKEY=2,NTOK
        CALL CCPUPC(CVALUE(IKEY))
      ENDDO
C
C----If comment don't do anything, go and read next key
      IF(KEY(1:1).EQ.'!') THEN
        GOTO 10
C
C===========================================================
      ELSEIF(KEY(1:1).EQ.'#') THEN
        GOTO 10

      ELSEIF(KEY.EQ.'TITL') THEN
C
C---Job title
         IF(NTOK.GE.2) 
     +      TITLE = LINE(IBEG(2) :IEND(NTOK))
         GOTO 10

      ELSEIF(KEY.EQ.'TLSC') THEN
C
C---No. of cycles of TLS refinement. 
         IF(NTOK.GE.2) CALL GTNINT(2,1,NTLS_CYCLE,NTOK,ITYP,FVALUE)
         GOTO 10

      ELSEIF(KEY.EQ.'FITT') THEN
C
C---Whether or not to fit TLS parameters
         LFITTLS  = .TRUE.
         IF (NTOK.GE.2) THEN
           CONOF = LINE(IBEG(2):IBEG(2)+1)
           IF (CONOF.EQ.'OF') LFITTLS  = .FALSE.
         ENDIF
         GOTO 10

      ELSEIF(KEY.EQ.'PLOT') THEN
C
C---Whether or not to plot
         LDOPLOTS  = .TRUE.
         IF (NTOK.GE.2) THEN
           CONOF = LINE(IBEG(2):IBEG(2)+1)
           IF (CONOF.EQ.'OF') LDOPLOTS  = .FALSE.
         ENDIF
         GOTO 10

      ELSEIF(KEY.EQ.'RIGI') THEN
C
C---Whether or not to assess 'rigid-body' postulate
         LRIGBODY  = .TRUE.
         IF (NTOK.GE.2) THEN
           CONOF = LINE(IBEG(2):IBEG(2)+1)
           IF (CONOF.EQ.'OF') LRIGBODY  = .FALSE.
         ENDIF
         GOTO 10

      ELSEIF(KEY.EQ.'DUBI') THEN
C
C---Range of deltaU to be used in RIGIDBODY analysis
         IF(NTOK.GE.2) CALL GTNINT(2,1,NDELTAUBINS,NTOK,ITYP,FVALUE)
         IF(NTOK.GE.3) CALL GTNINT(3,1,NPSBINS,NTOK,ITYP,FVALUE)
         GOTO 10

      ELSEIF(KEY.EQ.'DURA') THEN
C
C---Range of deltaU to be used in RIGIDBODY analysis
         IF(NTOK.GE.2) CALL GTNREA(2,1,DELTAURANGE,NTOK,ITYP,FVALUE)
         GOTO 10

      ELSEIF(KEY.EQ.'VERB') THEN
C
C---Print extra information
         VERBOSE  = .TRUE.
         GOTO 10

      ELSEIF(KEY.EQ.'MAIN') THEN
C
C---Whether or not to include main chain atoms only
         LMAINCHAIN  = .TRUE.
         GOTO 10
C
C     END
C===========================================================
      ELSEIF(KEY(1:3).EQ.'END') THEN
        GOTO 999
      ELSE
        CALL CCPERR(1,'Unrecognized keyword.')
        GOTO 10
      ENDIF
c
c----Normal return from subroutine RCARD. 
999   CONTINUE

      RETURN
      END

      SUBROUTINE RDXYZ
C     ================
C
C  Read in atomic parameters
C
      INCLUDE 'anisoanl.fh'
C
      INTEGER I,INXYZ,ISERQ,IRES,IZZ
      REAL XQ,YQ,ZQ,OCCQ,BISO,BQ(6)
      CHARACTER ATNAM*4,RESNAM*4,CHNNAM*1,ERRLINE*200
      CHARACTER RESNO*4,ID*4,ALTCOD*1,INSCOD*1,SEGID*4
      REAL UANISO0(6),EVECTOR(9) 
      LOGICAL HAVE_NONPOSDEF
C
      NBROOK = 0
      NOMIT = 0
      IATM = 0
      NANISO_ATOMS = 0
      HAVE_NONPOSDEF=.FALSE.

      WRITE(6,'(/,A,/)') ' ** READING INPUT COORDINATE FILE **'

      IFAIL = 0
      INXYZ = 0
      CALL XYZOPEN('XYZIN','INPUT',' ',INXYZ,IFAIL)

C---Loop over atoms in XYZIN
   10 CONTINUE
C
      CALL XYZADVANCE(INXYZ,0,0,*20,*20)
      CALL XYZATOM(INXYZ,ISERQ,ATNAM,RESNAM,CHNNAM,IRES,RESNO,
     +          INSCOD,ALTCOD,SEGID,IZZ,ID)
      CALL XYZCOORD(INXYZ,'O','O',XQ,YQ,ZQ,OCCQ,BISO,BQ)

C  Don't count atoms with OCC = 0.0
      IF (OCCQ.EQ.0.0) THEN
        NBROOK = NBROOK + 1
        NOMIT = NOMIT + 1
        GOTO 10
      ENDIF

      NBROOK = NBROOK + 1
      IATM = IATM + 1
      IF (IATM.GT.QQA) THEN
        WRITE(ERRLINE,'(A,I6)') 'No of atoms exceeds QQA = ',QQA
        CALL CCPERR(1,ERRLINE)
      ENDIF

C---If atom has aniso U values
      IF (BQ(2).NE.0.0 .OR. BQ(3).NE.0.0) THEN
         DO IANISO=1,6
            U_OBS(IATM,IANISO) = BQ(IANISO)
         ENDDO
         NANISO_ATOMS = NANISO_ATOMS + 1
      ELSE
        U_OBS(IATM,1) = BQ(1)/PISQ8
        U_OBS(IATM,2) = BQ(1)/PISQ8
        U_OBS(IATM,3) = BQ(1)/PISQ8
        U_OBS(IATM,4) = 0.0
        U_OBS(IATM,5) = 0.0
        U_OBS(IATM,6) = 0.0
      ENDIF

      WRITE (ATOM(IATM),FMT=6000) RESNAM,IRES,ATNAM(1:3),CHNNAM
 6000 FORMAT (A1,I4,A3,A1)

      UANISO0(1) = U_OBS(IATM,1)
      UANISO0(2) = U_OBS(IATM,4)
      UANISO0(3) = U_OBS(IATM,2)
      UANISO0(4) = U_OBS(IATM,5)
      UANISO0(5) = U_OBS(IATM,6)
      UANISO0(6) = U_OBS(IATM,3)
      CALL EIGEN_RS_ASC(UANISO0,EVECTOR,3,0)
      IF (UANISO0(1).LE.0 .OR. UANISO0(3).LE.0 
     +                    .OR. UANISO0(6).LE.0) THEN
        IF (.NOT. HAVE_NONPOSDEF) THEN
           CALL CCPERR(2,
     +   'XYZIN contains atoms with non-positive-definite aniso-Us.')
          WRITE(6,'(A,/)') 
     +   ' The following atoms have non-positive-definite ADPs:'
          HAVE_NONPOSDEF=.TRUE.
        ENDIF
        WRITE(6,'(3X,A,6(X,F8.3))') ATOM(IATM),
     +       (U_OBS(IATM,I),I=1,6)
        WRITE(6,'(7X,A,3(X,F8.3))') 'Eigenvalues: ',
     +       UANISO0(1),UANISO0(3),UANISO0(6)
      ENDIF

      ALTID(IATM) = ALTCOD
      XYZ1(IATM) = XQ
      XYZ2(IATM) = YQ
      XYZ3(IATM) = ZQ
      XORT0(1,IATM) = XQ
      XORT0(2,IATM) = YQ
      XORT0(3,IATM) = ZQ
      QOCC(IATM) = OCCQ

      GOTO 10

   20 CONTINUE
C
      NA = IATM

      WRITE (6,FMT=6004) NBROOK,NOMIT,NA,NANISO_ATOMS
 6004 FORMAT (//
     +          ' Number of atoms read from PDB file      = ',I6,/,
     +          ' Number of atoms with zero occupancy     = ',I6,/,
     +          ' Number of atoms available for fitting   = ',I6,/,
     +          ' Number of atoms with anisotropic Us     = ',I6,//)

C  It is possible you have not yet read the cell.
C  If not take it from XYZIN.
      IF(CELL(1).LT.0.1) THEN
         CALL RBCELL(CELL,VOL)
      END IF

C  Calculate CoMs for chains, and atom coordinates w.r.t. these.
      CALL CHAIN_COMS
C
      CALL XYZCLOSE(INXYZ)

      END
C
      SUBROUTINE CHAIN_COMS

      INCLUDE 'anisoanl.fh'

      CHARACTER CHNNAM*1,CHNNAM_PREV*1
      INTEGER I0
      REAL COMX,COMY,COMZ

      I0 = 1
      COMX = 0.0
      COMY = 0.0
      COMZ = 0.0

      READ(ATOM(1),FMT=6003) IRES_PREV,CHNNAM_PREV
      DO    IA=1,NA
        READ(ATOM(IA),FMT=6003) IRES,CHNNAM
        IF (CHNNAM.EQ.CHNNAM_PREV) THEN
          COMX = COMX + XYZ1(IA)
          COMY = COMY + XYZ2(IA)
          COMZ = COMZ + XYZ3(IA)
        ELSE
          COMX = COMX / REAL(IA - I0)
          COMY = COMY / REAL(IA - I0)
          COMZ = COMZ / REAL(IA - I0)
          DO IA1 = I0, IA - 1
            XCOM(1,IA1) = XYZ1(IA1) - COMX
            XCOM(2,IA1) = XYZ2(IA1) - COMY
            XCOM(3,IA1) = XYZ3(IA1) - COMZ
          ENDDO
          WRITE(6,6004) CHNNAM_PREV,COMX,COMY,COMZ
          I0 = IA
          COMX = XYZ1(IA)
          COMY = XYZ2(IA)
          COMZ = XYZ3(IA)
        ENDIF
        CHNNAM_PREV = CHNNAM
      ENDDO
      COMX = COMX / REAL(NA + 1 - I0)
      COMY = COMY / REAL(NA + 1 - I0)
      COMZ = COMZ / REAL(NA + 1 - I0)
      DO IA1 = I0, NA
         XCOM(1,IA1) = XYZ1(IA1) - COMX
         XCOM(2,IA1) = XYZ2(IA1) - COMY
         XCOM(3,IA1) = XYZ3(IA1) - COMZ
      ENDDO
      WRITE(6,6004) CHNNAM_PREV,COMX,COMY,COMZ

 6003 FORMAT (1X,I4,3X,A1)
 6004 FORMAT(' Centre of mass of chain ',A1,' is    (',
     +   F8.3,',',F8.3,',',F8.3,')')

      END

      SUBROUTINE WRTXYZ
C     =================
C
C  Read in atomic parameters
C
      INCLUDE 'anisoanl.fh'
C
      INTEGER INXYZ,ISERQ,IRES,IZZ
      REAL XQ,YQ,ZQ,OCCQ,BISO,BQ(6)
      CHARACTER ATNAM*4,RESNAM*4,CHNNAM*1
      CHARACTER RESNO*4,ID*4,ALTCOD*1,INSCOD*1,SEGID*4

      IFAIL = 0
      INXYZ = 0
      CALL XYZOPEN('XYZIN','INPUT',' ',INXYZ,IFAIL)
      IOUTX = 0
      IFAIL = 0
      CALL XYZOPEN('XYZOUT','OUTPUT',' ',IOUTX,IFAIL)

      IATM = 0
C---Loop over atoms in XYZIN
   10 CONTINUE

C---Note that input is NOT connected to output via second argument
C   of XYZADVANCE. Thus XYZOUT has no header information. This is 
C   deliberate, since XYZOUT is a bit of a dud PDB file.
      CALL XYZADVANCE(INXYZ,0,0,*20,*20)
      CALL XYZATOM(INXYZ,ISERQ,ATNAM,RESNAM,CHNNAM,IRES,RESNO,
     +       INSCOD,ALTCOD,SEGID,IZZ,ID)
      CALL XYZCOORD(INXYZ,'O','O',XQ,YQ,ZQ,OCCQ,BISO,BQ)

C---If OCCQ = 0.0 output it without any changes, and no ANISOU line.
      IF (OCCQ.EQ.0.0 ) THEN
        GO TO 10
      END IF

      IATM = IATM + 1

C---If part of TLS group, output residual U's
C   cf. Harata et al JMB 287, 347 (1999), Watanabe et al Acta Cryst
C   D62, 784 (2006) who scale U_ANISO to make sure BQ is positive
C   definite
      IF (IGROUP(IATM).NE.0) THEN
        DO    IANISO=1,6
          BQ(IANISO) = U_OBS(IATM,IANISO) - U_ANISO(IATM,IANISO)
        ENDDO
C---If not part of TLS group, decide if ANISOU line needed
      ELSEIF (U_OBS(IATM,4).NE.0.0 .OR. 
     +        U_OBS(IATM,5).NE.0.0 .OR. 
     +        U_OBS(IATM,6).NE.0.0 ) THEN
        DO    IANISO=1,6
          BQ(IANISO) = U_OBS(IATM,IANISO) 
        ENDDO
      ELSE
        BQ(1) = U_OBS(IATM,1)* PISQ8
        DO    IANISO=2,6
          BQ(IANISO) = 0.0
        ENDDO
      ENDIF

      CALL XYZATOM(IOUTX,IATM,ATNAM,RESNAM,CHNNAM,IRES,RESNO,
     +       INSCOD,ALTCOD,SEGID,IZZ,ID)
      CALL XYZCOORD(IOUTX,'O','O',XYZ1(IATM),XYZ2(IATM),XYZ3(IATM),
     +                                              QOCC(IATM),BISO,BQ)
      CALL XYZADVANCE(IOUTX,0,0,*20,*20)

      GOTO 10

   20 CONTINUE
C
      CALL XYZCLOSE(INXYZ)
      CALL XYZCLOSE(IOUTX)

      END

      SUBROUTINE RIGIDBODY

C  Routine to assess 'rigid-body' postulate of Rosenfield et al
C  Acta Cryst A34, 828 (1978)

      INCLUDE 'anisoanl.fh'

      PARAMETER (MAXBIN=100)

      CHARACTER ATNMP*3,TLSCOL*200,TLSNAM*200,LINE*200
      CHARACTER IALTCOD*1,JALTCOD*1
      INTEGER NDELBIN1(MAXTLSGRP,MAXBIN),NDELBIN2(MAXBIN)
      INTEGER NTOTBIN1(MAXTLSGRP),NTOTBIN2,NTOTMISS,NTOTPAIR
      REAL RIOCC,RJOCC,OCCWEIGHT
      REAL XYBINS(MAXBIN,MAXBIN),RXYBINS(MAXBIN,MAXBIN)
      REAL XYMIN,XYMAX,DELBIN1(MAXTLSGRP),DELBIN2
      REAL UOBS1(6),UOBS2(6),RDIFF(3)
      LOGICAL WEIGHTED

      WRITE(6,'(/,/,10X,A,/)')
     +   ' *** RIGID-BODY ANALYSIS *** '
      
      IF (NDELTAUBINS.GT.MAXBIN) THEN
        NDELTAUBINS = MAXBIN
        WRITE(LINE,'(A,A,I4,A)') 'Number of bins requested greater ',
     +  'than permitted maximum: ',MAXBIN,' - resetting to this value.'
        CALL CCPERR(2,LINE)
      ENDIF
      
      IF (NPSBINS.GT.MAXBIN) THEN
        NPSBINS = MAXBIN
        WRITE(LINE,'(A,A,I4,A)') 'Number of bins requested greater ',
     +  'than permitted maximum: ',MAXBIN,' - resetting to this value.'
        CALL CCPERR(2,LINE)
      ENDIF

C     weight or not delta-U
      WEIGHTED=.TRUE.

C  Initialise binning
      DO IBIN=1,MAXBIN
        DO ITLS = 1,MAXTLSGRP
          NDELBIN1(ITLS,IBIN) = 0
        ENDDO
        NDELBIN2(IBIN) = 0
      ENDDO
      DO ITLS = 1,MAXTLSGRP
        NTOTBIN1(ITLS) = 0
      ENDDO
      NTOTBIN2 = 0
      NTOTMISS = 0
        
      BINWIDTH = DELTAURANGE / REAL(NDELTAUBINS)

      MINATOM = 1
      MAXATOM = NA

C  Set up binning for postscript plot

C  Count residues to be included
      NRBRESI = 0
      IRES0 = 0
      DO 50 IA=MINATOM,MAXATOM

C  Atom selection is done via TLS groups 
        IGRP = IGROUP(IA)
        IF (IGRP .LE. 0) GOTO 50

        READ(ATOM(IA),'(1X,I4)') IRES1

        IF (IRES1.NE.IRES0) THEN
           NRBRESI = NRBRESI + 1
           IRES0 = IRES1
        ENDIF

 50   CONTINUE

      XYMIN = 0.0
      XYMAX = NRBRESI

      DO I = 1,MAXBIN
        DO J = 1,MAXBIN
          XYBINS(I,J) = 0.0
          RXYBINS(I,J) = 0.0
        ENDDO
      ENDDO

C  Variables for scoring choice of rigid groups
      SCORE = 0.0
      RNSCORE = 0.0

      IF (VERBOSE) THEN
        WRITE(6,'(/,A,/)') 
     + ' List of delta-U values calculated along interatomic vector: '
      ENDIF   

      IRES0 = 0
      IRES = 0
      DO 100 IA=MINATOM,MAXATOM

C  Atom selection is done via TLS groups 

        IGRP = IGROUP(IA)
        IF (IGRP .LE. 0) GOTO 100

        DO I = 1,6
          UOBS1(I) = U_OBS(IA,I)
        ENDDO
        IALTCOD = ALTID(IA)
        RIOCC = QOCC(IA)

C  x-bin for shading
        READ(ATOM(IA),'(1X,I4)') IRES1
        IF (IRES1.NE.IRES0) THEN
           IRES = IRES + 1
           IRES0 = IRES1
        ENDIF
        IXBIN = INT(REAL((IRES-1)*NPSBINS)/REAL(NRBRESI)) + 1
      
        JRES0 = 0
        JRES = 0
        DO 200 JA=MINATOM,MAXATOM

          JGRP = IGROUP(JA)
          IF (JGRP .LE. 0) GOTO 200

          DO I = 1,6
            UOBS2(I) = U_OBS(JA,I)
          ENDDO
          JALTCOD = ALTID(JA)
          RJOCC = QOCC(JA)

C  y-bin for shading
          READ(ATOM(JA),'(1X,I4)') JRES1
          IF (JRES1.NE.JRES0) THEN
             JRES = JRES + 1
             JRES0 = JRES1
          ENDIF
          IYBIN = INT(REAL((JRES-1)*NPSBINS)/REAL(NRBRESI)) + 1

C Don't include diagonal terms. But this must come after JRES is updated!
          IF (JA.EQ.IA) GOTO 200

C Deal with multiple conformations
          IF (IALTCOD.NE.' ' .AND. JALTCOD.NE.' ') THEN
            IF (IALTCOD.NE.JALTCOD) GOTO 200
            OCCWEIGHT = RIOCC
          ELSEIF (IALTCOD.NE.' ') THEN
            OCCWEIGHT = RIOCC
          ELSEIF (JALTCOD.NE.' ') THEN
            OCCWEIGHT = RJOCC
          ELSE
            OCCWEIGHT = 1.0
          ENDIF

          RDIFF(1) = XYZ1(JA) - XYZ1(IA)
          RDIFF(2) = XYZ2(JA) - XYZ2(IA)
          RDIFF(3) = XYZ3(JA) - XYZ3(IA)

          CALL PROJECT_U(UOBS1,RDIFF,URAD1)
          CALL PROJECT_U(UOBS2,RDIFF,URAD2)

C **** here is calculation of delta-U ****
          IF (WEIGHTED) THEN
C    express as fraction of mean value:
            DELTA = 2.0*(URAD2 - URAD1)/(URAD2 + URAD1)
          ELSE
            DELTA = URAD2 - URAD1
          ENDIF

C  accumulate sums for loggraph plot
          IBIN = INT(ABS(DELTA)/BINWIDTH) + 1
          IF (IBIN.GE.1 .AND. IBIN.LE.NDELTAUBINS) THEN
            IF (IGRP.EQ.JGRP) THEN
              NDELBIN1(IGRP,IBIN) = NDELBIN1(IGRP,IBIN) + 1
              NTOTBIN1(IGRP) = NTOTBIN1(IGRP) + 1
            ELSE
              NDELBIN2(IBIN) = NDELBIN2(IBIN) + 1
              NTOTBIN2 = NTOTBIN2 + 1
            ENDIF  
          ELSE          
            NTOTMISS = NTOTMISS + 1
          ENDIF 

C  accumulate sums for postscript plot
          RXYBINS(IXBIN,IYBIN) = RXYBINS(IXBIN,IYBIN) + 
     +                                    ABS(DELTA)*OCCWEIGHT
          XYBINS(IXBIN,IYBIN) = XYBINS(IXBIN,IYBIN) + OCCWEIGHT

C   calculate scoring function
          IF (IGRP.EQ.JGRP .AND. IA.NE.JA) THEN
            RNSCORE = RNSCORE + OCCWEIGHT
            SCORE = SCORE + ABS(DELTA)*OCCWEIGHT
          ENDIF

          IF (VERBOSE) THEN
            WRITE(6,'(5X,A9,8X,A9,8X,F8.2)') ATOM(IA),ATOM(JA),DELTA
          ENDIF   

 200    CONTINUE
 100  CONTINUE

      SCORE = SCORE / RNSCORE
      WRITE(6,'(/,'' Score for rigid-body criterion = '',F8.2)') SCORE
      WRITE(6,'('' Based on '',F12.1,'' comparisons '')') RNSCORE
      WRITE(6,'('' You should vary the TLS groups to'',
     +          '' lower this score. '',/)') 

      NTOTPAIR = 0
      DO ITLS = 1,NTLSGRP
        NTOTPAIR = NTOTPAIR + NTOTBIN1(ITLS)
      ENDDO
      NTOTPAIR = NTOTPAIR + NTOTBIN2 + NTOTMISS

      WRITE(6,'(/,'' Plotting distributions of delta-U '',
     +   '' for each TLS group defined.'')') 
      WRITE(6,'('' Each distribution plotted up to '',F8.2,
     +   '' and divided into '',I4,'' bins'')') 
     +     DELTAURANGE,NDELTAUBINS
      WRITE(6,'(I8,'' pairs out of a total of '',I8,
     +   '' considered had a delta-U larger than '',F8.2,/)') 
     +     NTOTMISS,NTOTPAIR,DELTAURANGE

      TLSCOL = ' '
      TLSNAM = ' '
      DO ITLS = 1,NTLSGRP
        WRITE(TLSCOL(3*ITLS-2:3*ITLS),'(I2,A1)') ITLS+3,','
        WRITE(TLSNAM(7*ITLS-6:7*ITLS),'(A4,I2.2,A1)') ' GRP',ITLS,' '
      ENDDO
      WRITE(TLSCOL(3*NTLSGRP+1:3*NTLSGRP+2),'(I2)') NTLSGRP+4
      WRITE(TLSNAM(7*NTLSGRP+1:7*NTLSGRP+9),'(A9)') ' OFFDIAG '
      WRITE(6,6002) TLSCOL(1:LENSTR(TLSCOL)),TLSNAM(1:LENSTR(TLSNAM))
      WRITE (6,'(A)') ' $$ $$'
      DO IBIN=1,NDELTAUBINS
        DO ITLS = 1,NTLSGRP
          DELBIN1(ITLS) = 0.0
          IF (NTOTBIN1(ITLS).GT.0) 
     +  DELBIN1(ITLS) = REAL(NDELBIN1(ITLS,IBIN))/REAL(NTOTBIN1(ITLS))
        ENDDO
        DELBIN2 = 0.0
        IF (NTOTBIN2.GT.0) 
     +      DELBIN2 = REAL(NDELBIN2(IBIN))/REAL(NTOTBIN2)
        WRITE(6,'(I5,100F8.4)') 
     +     IBIN, REAL(IBIN-1)*BINWIDTH, REAL(IBIN)*BINWIDTH, 
     +     (DELBIN1(ITLS),ITLS=1,NTLSGRP),DELBIN2
      ENDDO
      WRITE (6,'(A)') ' $$'

      WRITE(6,'(/,'' Writing delta-U matrix to postscript file '',
     +   ''with residues divided into '',I4,'' bins'')') 
     +     NPSBINS
      IF (WEIGHTED) THEN
        WRITE(6,'('' delta-U is calculated as fraction of mean U '',/)')
      ELSE
        WRITE(6,'('' unweighted delta-U is used '',/)') 
      ENDIF

      DO I = 1,MAXBIN
        DO J = 1,MAXBIN
          IF (XYBINS(I,J).GT.0.0) 
     +          RXYBINS(I,J) = RXYBINS(I,J) / XYBINS(I,J)
        ENDDO
      ENDDO

      CALL RIGIDBODYPS(XYMIN,XYMAX,RXYBINS)

 6001 FORMAT(5X,A3,1X)
 6002 FORMAT(/,/,
     +  ' $TABLE: Rigid-body test :',/,
     +  ' $GRAPHS: Distribution of delta-Us :A:3,',A,': $$',/,/,
     +  '  IBIN   DELTAUlow   DELTAUupp ',A)

      END

      SUBROUTINE RIGIDBODYPS(XYMIN,XYMAX,RXYBINS)

      INCLUDE 'anisoanl.fh'

      PARAMETER (MAXBIN=100)
      
      REAL XYMIN,XYMAX,RXYBINS(MAXBIN,MAXBIN)

      INTEGER PBOXX1, PBOXX2, PBOXY1, PBOXY2
      REAL XW,XL,XR,YW,YL,YR,XORIGIN,YORIGIN

      PARAMETER ( MXCOLR = 19 )
      REAL RGB(3,MXCOLR)
      CHARACTER*12 COLNAM(MXCOLR)
      INTEGER ICOL,ILINE

      DATA ((RGB(ILINE,ICOL), ILINE=1,3), COLNAM(ICOL), ICOL=1,MXCOLR) /
     +     0.0000, 0.0000, 0.0000, 'BLACK',
     +     1.0000, 1.0000, 1.0000, 'WHITE',
     +     1.0000, 0.0000, 0.0000, 'RED',
     +     0.0000, 1.0000, 0.0000, 'GREEN',
     +     0.0000, 0.0000, 1.0000, 'BLUE',
     +     1.0000, 1.0000, 0.0000, 'YELLOW',
     +     0.8000, 0.5000, 0.0000, 'ORANGE',
     +     0.5000, 1.0000, 0.0000, 'LIME GREEN',
     +     0.5000, 0.0000, 1.0000, 'PURPLE',
     +     0.5000, 1.0000, 1.0000, 'CYAN',
     +     1.0000, 0.5000, 1.0000, 'PINK',
     +     0.3000, 0.8000, 1.0000, 'SKY BLUE',
     +     1.0000, 1.0000, 0.7000, 'CREAM',
     +     0.0000, 1.0000, 1.0000, 'TURQUOISE',
     +     1.0000, 0.0000, 1.0000, 'LILAC',
     +     0.8000, 0.0000, 0.0000, 'BRICK RED',
     +     0.5000, 0.0000, 0.0000, 'BROWN',
     +     0.9700, 0.9700, 0.9700, 'LIGHT GREY',
     +     1.0000, 1.0000, 1.0000, 'WHITE' /

C  Bounding box of page
      PBOXX1 = 50
      PBOXX2 = 500
      PBOXY1 = 150
      PBOXY2 = 600

C  Origin of plot on page
      XORIGIN = 100.0
      YORIGIN = 200.0

C  Inner box for plot
      XW = 300.0
      XL = 0.0
      XR = XL + XW
      YW = 300.0
      YL = 0.0
      YR = YL + YW

      CALL PSFUNIT (54)

C   Just the one page
      CALL PSSETPG(1)

      CALL PSOPEN (PBOXX1,PBOXX2,PBOXY1,PBOXY2,MXCOLR,RGB,.FALSE.,1)

C   Set origin of plot compared to origin of BB
      CALL PSTRANSLATE (XORIGIN,YORIGIN)

C   Set axes
      CALL AXES(XL,XR,YL,YR,NPSBINS,NPSBINS,
     +     XYMIN,XYMAX,XYMIN,XYMAX,
     +     10.0,0,0,15.0,.TRUE.,.TRUE.,
     +     .TRUE.,.FALSE.,.FALSE.,.FALSE.)

      CALL PSTEXT(XL+60.0,YL+330.0,30.0,'DELTA-MATRIX')

      CALL PSTEXT(XL+60.0,YL-30.0,15.0,'First residue')
      CALL PSRTXT(XL-30.0,YL+60.0,15.0,'Second residue')

C   Scale RXYBINS so maximum is 1.0
      RMAXRXY = 0.0
      DO I=1,NPSBINS
        DO J=1,NPSBINS
          IF (ABS(RXYBINS(I,J)).GT.RMAXRXY)
     +      RMAXRXY = ABS(RXYBINS(I,J))
        ENDDO
      ENDDO
      SHADESCALE = 1.0/RMAXRXY

      BINSTEP = XW/REAL(NPSBINS)
      DO I=1,NPSBINS
        XXL = REAL(I-1)*BINSTEP
        XXR = REAL(I)*BINSTEP
        DO J=1,NPSBINS
          YYL = REAL(J-1)*BINSTEP
          YYR = REAL(J)*BINSTEP
          SHADE = 1.0 - ABS(RXYBINS(I,J))*SHADESCALE
          CALL PSHADE(SHADE,1,RGB,MXCOLR,.FALSE.)
          CALL PSUBOX(XXL,YYL,XXL,YYR,XXR,YYR,XXR,YYL)
C          CALL PSHBOX(XXL,XXR,YYL,YYR,SHADE)
        ENDDO
      ENDDO

C   text, lines, points, colours

      CALL PSCLOS(PBOXX1,PBOXX2,PBOXY1,PBOXY2)

      RETURN
      END

      SUBROUTINE UPLOT

      INCLUDE 'anisoanl.fh'

      PARAMETER (MAXBIN=100)
      INTEGER NBIN(MAXBIN)
      REAL RADBIN(MAXBIN),TANGBIN(MAXBIN)

      CHARACTER CHNNAM*1,CHNNAM_PREV*1,ATNAM*3
      REAL U_CALC(6),UOBS0(6)

      WRITE(6,'(/,/,10X,A,/)')
     +   ' *** Generating plots for loggraph *** '

C  Initialise binning
      DO IBIN=1,MAXBIN
        NBIN(IBIN) = 0
        RADBIN(IBIN) = 0.0
        TANGBIN(IBIN) = 0.0
      ENDDO
      R2RANGE = 1000.0
      BINWIDTH = R2RANGE / REAL(MAXBIN)

      SCALE_R2_TO_U = 3000.0

      NRES = 1
      NATOM = 0
      UISOMEAN1 = 0.0
      UISOSIGMA1 = 0.0
      UISOMEAN2 = 0.0
      UISOSIGMA2 = 0.0
      ANISOMEAN1 = 0.0
      ANISOSIGMA1 = 0.0
      PROLMEAN1 = 0.0
      ANISOMEAN2 = 0.0
      ANISOSIGMA2 = 0.0
      PROLMEAN2 = 0.0
      ANISOMEAN_H1 = 0.0
      ANISOSIGMA_H1 = 0.0
      ANISOMEAN_H2 = 0.0
      ANISOSIGMA_H2 = 0.0
      R2FROMORIG = 0.0
      URADMEAN1 = 0.0
      UTANGMEAN1 = 0.0

      IF (LFITTLS) THEN
        WRITE(6,'(/,A,/,A,/)')
     +   ' Radius from origin and radial/tangential projections',
     +   ' of U are calculated using origins of TLS groups.'
      ELSE
        WRITE(6,'(/,A,/,A,/)')
     +   ' Radius from origin and radial/tangential projections',
     +   ' of U are calculated using centres of mass of chains.'
      ENDIF

      READ(ATOM(1),FMT=6003) IRES_PREV,CHNNAM_PREV

      IF (LFITTLS) THEN
         WRITE (6,6000) CHNNAM_PREV
      ELSE
         WRITE (6,6001) CHNNAM_PREV
      ENDIF
      WRITE (6,'(A)') ' $$ $$'
      
      DO  1000  IA=1,NA

        IF (LMAINCHAIN) THEN
          IMC = 0
          READ(ATOM(IA),FMT=6005) ATNAM
C Check for main chain atoms and Calcium ....
          IF (ATNAM.EQ.'CA ' .OR. ATNAM.EQ.'N  ' .OR. 
     +      ATNAM.EQ.'O  ' .OR. ATNAM.EQ.'C  ') IMC = 1
          IF (IMC.EQ.0) GOTO 1000
        ENDIF

        DO I = 1,6
          UOBS0(I) = U_OBS(IA,I)
        ENDDO

C---uiso and anisotropy of original U from XYZIN
        CALL U_ANISOTROPY(UOBS0,UISO1,ANISO1,ANISO_H1,PROLATE1)
        UISO1SQ = UISO1*UISO1
        ANISO1SQ = ANISO1*ANISO1
        ANISO_H1SQ = ANISO_H1*ANISO_H1
C
C---uiso and anisotropy of U from TLS 
        U_CALC(1) = U_ANISO(IA,1)
        U_CALC(2) = U_ANISO(IA,2)
        U_CALC(3) = U_ANISO(IA,3)
        U_CALC(4) = U_ANISO(IA,4)
        U_CALC(5) = U_ANISO(IA,5)
        U_CALC(6) = U_ANISO(IA,6)
        CALL U_ANISOTROPY(U_CALC,UISO2,ANISO2,ANISO_H2,PROLATE2)
        UISO2SQ = UISO2*UISO2
        ANISO2SQ = ANISO2*ANISO2
        ANISO_H2SQ = ANISO_H2*ANISO_H2

C---if we have TLS groups, use their origins
        IF (LFITTLS) THEN
          IF(IGROUP(IA).GT.0) THEN
            R2 = XORT0(1,IA)**2 + XORT0(2,IA)**2 + XORT0(3,IA)**2
            CALL PROJECT_U(UOBS0,XORT0(1,IA),URAD)
            UTANG = 0.5*(3.0*UISO1-URAD)
          ELSE
            R2 = 0.0
            URAD = 0.0
            UTANG = 0.0
          ENDIF
C---else use chain CoMs
        ELSE
          R2 = XCOM(1,IA)**2 + XCOM(2,IA)**2 + XCOM(3,IA)**2
          CALL PROJECT_U(UOBS0,XCOM(1,IA),URAD)
          UTANG = 0.5*(3.0*UISO1-URAD)
        ENDIF

        READ(ATOM(IA),FMT=6003) IRES,CHNNAM

C---If same residue as last atom, accumulate totals
        IF (IRES.EQ.IRES_PREV .AND. CHNNAM.EQ.CHNNAM_PREV) THEN
          NATOM = NATOM + 1
          UISOMEAN1 = UISOMEAN1 + UISO1
          UISOSIGMA1 = UISOSIGMA1 + UISO1SQ
          UISOMEAN2 = UISOMEAN2 + UISO2
          UISOSIGMA2 = UISOSIGMA2 + UISO2SQ
          ANISOMEAN1 = ANISOMEAN1 + ANISO1
          ANISOSIGMA1 = ANISOSIGMA1 + ANISO1SQ
          PROLMEAN1 = PROLMEAN1 + PROLATE1
          ANISOMEAN2 = ANISOMEAN2 + ANISO2
          ANISOSIGMA2 = ANISOSIGMA2 + ANISO2SQ
          PROLMEAN2 = PROLMEAN2 + PROLATE2
          ANISOMEAN_H1 = ANISOMEAN_H1 + ANISO_H1
          ANISOSIGMA_H1 = ANISOSIGMA_H1 + ANISO_H1SQ
          ANISOMEAN_H2 = ANISOMEAN_H2 + ANISO_H2
          ANISOSIGMA_H2 = ANISOSIGMA_H2 + ANISO_H2SQ
          R2FROMORIG = R2FROMORIG + R2
          URADMEAN1 = URADMEAN1 + URAD
          UTANGMEAN1 = UTANGMEAN1 + UTANG

C---New residue
        ELSE
C---Finish off previous residue
          UISOMEAN1 = UISOMEAN1 / NATOM
          IF (NATOM.EQ.1) THEN
            UISOSIGMA1 = 0.0
          ELSE
            UISOSIGMA1 = SQRT(UISOSIGMA1 / NATOM - UISOMEAN1**2)
          ENDIF
          UISOMEAN2 = UISOMEAN2 / NATOM
          IF (NATOM.EQ.1) THEN
            UISOSIGMA2 = 0.0
          ELSE
            UISOSIGMA2 = SQRT(UISOSIGMA2 / NATOM - UISOMEAN2**2)
          ENDIF
          ANISOMEAN1 = ANISOMEAN1 / NATOM
          IF (NATOM.EQ.1) THEN
            ANISOSIGMA1 = 0.0
          ELSE
            ANISOSIGMA1 = SQRT(ANISOSIGMA1 / NATOM - ANISOMEAN1**2)
          ENDIF
          PROLMEAN1 = PROLMEAN1 / NATOM
          ANISOMEAN2 = ANISOMEAN2 / NATOM
          IF (NATOM.EQ.1) THEN
            ANISOSIGMA2 = 0.0
          ELSE
            ANISOSIGMA2 = SQRT(ANISOSIGMA2 / NATOM - ANISOMEAN2**2)
          ENDIF
          PROLMEAN2 = PROLMEAN2 / NATOM
          ANISOMEAN_H1 = ANISOMEAN_H1 / NATOM
          IF (NATOM.EQ.1) THEN
            ANISOSIGMA_H1 = 0.0
          ELSE
            ANISOSIGMA_H1 = SQRT(ANISOSIGMA_H1/NATOM - ANISOMEAN_H1**2)
          ENDIF
          ANISOMEAN_H2 = ANISOMEAN_H2 / NATOM
          IF (NATOM.EQ.1) THEN
            ANISOSIGMA_H2 = 0.0
          ELSE
            ANISOSIGMA_H2 = SQRT(ANISOSIGMA_H2/NATOM - ANISOMEAN_H2**2)
          ENDIF
          R2FROMORIG = R2FROMORIG / NATOM
          URADMEAN1 = URADMEAN1 / NATOM
          UTANGMEAN1 = UTANGMEAN1 / NATOM
          IF (LFITTLS) THEN
            WRITE(6,'(X,I6,17F10.3)') 
     +           NRES,UISOMEAN1,UISOMEAN2,ANISOMEAN1,ANISOMEAN2,
     +           UISOSIGMA1,UISOSIGMA2,ANISOSIGMA1,ANISOSIGMA2,
     +           R2FROMORIG/SCALE_R2_TO_U,
     +           URADMEAN1,UTANGMEAN1,
     +           ANISOMEAN_H1,ANISOMEAN_H2,ANISOSIGMA_H1,ANISOSIGMA_H2,
     +           PROLMEAN1,PROLMEAN2
          ELSE
            WRITE(6,'(X,I6,10F10.3)') 
     +           NRES,UISOMEAN1,ANISOMEAN1,
     +           UISOSIGMA1,ANISOSIGMA1,
     +           R2FROMORIG/SCALE_R2_TO_U,
     +           URADMEAN1,UTANGMEAN1,
     +           ANISOMEAN_H1,ANISOSIGMA_H1,
     +           PROLMEAN1
          ENDIF
C---Start totals for this residue
          NRES = NRES + 1
          NATOM = 1
          UISOMEAN1 = UISO1
          UISOSIGMA1 = UISO1SQ
          UISOMEAN2 = UISO2
          UISOSIGMA2 = UISO2SQ
          ANISOMEAN1 = ANISO1
          ANISOSIGMA1 = ANISO1SQ
          PROLMEAN1 = PROLATE1 
          ANISOMEAN2 = ANISO2
          ANISOSIGMA2 = ANISO2SQ
          PROLMEAN2 = PROLATE2 
          ANISOMEAN_H1 = ANISO_H1
          ANISOSIGMA_H1 = ANISO_H1SQ
          ANISOMEAN_H2 = ANISO_H2
          ANISOSIGMA_H2 = ANISO_H2SQ
          R2FROMORIG = R2
          URADMEAN1 = URAD
          UTANGMEAN1 = UTANG
          IF (CHNNAM.NE.CHNNAM_PREV) THEN
            WRITE (6,'(A)') ' $$'
            NRES = 1
            IF (LFITTLS) THEN
              WRITE (6,6000) CHNNAM
            ELSE
              WRITE (6,6001) CHNNAM
            ENDIF
            WRITE (6,'(A)') ' $$ $$'
          ENDIF
        ENDIF

        IF (R2.GT.0.0) THEN
          IBIN = INT(R2/BINWIDTH) + 1
          IF (IBIN.GE.1 .AND. IBIN.LE.MAXBIN) THEN 
            NBIN(IBIN) = NBIN(IBIN) + 1
            RADBIN(IBIN) = RADBIN(IBIN) + URAD
            TANGBIN(IBIN) = TANGBIN(IBIN) + UTANG
          ENDIF
        ENDIF

        CHNNAM_PREV = CHNNAM
        IRES_PREV = IRES

C---End of loop over atoms
 1000 CONTINUE

C---Finish off last residue
      UISOMEAN1 = UISOMEAN1 / NATOM
      IF (NATOM.EQ.1) THEN
        UISOSIGMA1 = 0.0
      ELSE
        UISOSIGMA1 = SQRT(UISOSIGMA1 / NATOM - UISOMEAN1**2)
      ENDIF
      UISOMEAN2 = UISOMEAN2 / NATOM
      IF (NATOM.EQ.1) THEN
        UISOSIGMA2 = 0.0
      ELSE
        UISOSIGMA2 = SQRT(UISOSIGMA2 / NATOM - UISOMEAN2**2)
      ENDIF
      ANISOMEAN1 = ANISOMEAN1 / NATOM
      IF (NATOM.EQ.1) THEN
        ANISOSIGMA1 = 0.0
      ELSE
        ANISOSIGMA1 = SQRT(ANISOSIGMA1 / NATOM - ANISOMEAN1**2)
      ENDIF
      PROLMEAN1 = PROLMEAN1 / NATOM
      ANISOMEAN2 = ANISOMEAN2 / NATOM
      IF (NATOM.EQ.1) THEN
        ANISOSIGMA2 = 0.0
      ELSE
        ANISOSIGMA2 = SQRT(ANISOSIGMA2 / NATOM - ANISOMEAN2**2)
      ENDIF
      PROLMEAN2 = PROLMEAN2 / NATOM
      ANISOMEAN_H1 = ANISOMEAN_H1 / NATOM
      IF (NATOM.EQ.1) THEN
        ANISOSIGMA_H1 = 0.0
      ELSE
        ANISOSIGMA_H1 = SQRT(ANISOSIGMA_H1 / NATOM - ANISOMEAN_H1**2)
      ENDIF
      ANISOMEAN_H2 = ANISOMEAN_H2 / NATOM
      IF (NATOM.EQ.1) THEN
        ANISOSIGMA_H2 = 0.0
      ELSE
        ANISOSIGMA_H2 = SQRT(ANISOSIGMA_H2 / NATOM - ANISOMEAN_H2**2)
      ENDIF
      R2FROMORIG = R2FROMORIG / NATOM
      URADMEAN1 = URADMEAN1 / NATOM
      UTANGMEAN1 = UTANGMEAN1 / NATOM
      IF (LFITTLS) THEN
         WRITE(6,'(X,I6,17F10.3)') 
     +           NRES,UISOMEAN1,UISOMEAN2,ANISOMEAN1,ANISOMEAN2,
     +           UISOSIGMA1,UISOSIGMA2,ANISOSIGMA1,ANISOSIGMA2,
     +           R2FROMORIG/SCALE_R2_TO_U,
     +           URADMEAN1,UTANGMEAN1,
     +           ANISOMEAN_H1,ANISOMEAN_H2,ANISOSIGMA_H1,ANISOSIGMA_H2,
     +           PROLMEAN1,PROLMEAN2
      ELSE
         WRITE(6,'(X,I6,10F10.3)') 
     +           NRES,UISOMEAN1,ANISOMEAN1,
     +           UISOSIGMA1,ANISOSIGMA1,
     +           R2FROMORIG/SCALE_R2_TO_U,
     +           URADMEAN1,UTANGMEAN1,
     +           ANISOMEAN_H1,ANISOSIGMA_H1,
     +           PROLMEAN1
      ENDIF
      WRITE (6,'(A)') ' $$'

C---Plot of radial variation of URAD and UTANG
      WRITE(6,6004)
      WRITE (6,'(A)') ' $$ $$'
      DO IBIN=1,MAXBIN
        IF (NBIN(IBIN).GE.1) 
     +  WRITE(6,'(2i8,2f8.2,2G15.5)') IBIN, NBIN(IBIN),
     +     REAL(IBIN-1)*BINWIDTH, REAL(IBIN)*BINWIDTH, 
     +     RADBIN(IBIN)/REAL(NBIN(IBIN)),
     +     TANGBIN(IBIN)/REAL(NBIN(IBIN))
      ENDDO
      WRITE (6,'(A)') ' $$'

 6000 FORMAT(/,/,
     +  ' $TABLE: Initial and residual Us for chain ',
     +  A1,':',/,
     +  ' $GRAPHS: Uiso  vs. residue :A:1,2,3,10: ',/,
     +  ' : Anisotropy vs. residue :A:1,4,5,17,18,10: ',/,
     +  ' : Projections of U vs. residue :A:1,11,12,10: $$',/,/,
     +  '  NRES  UISOMEAN1  UISOMEAN2  ANISOMEAN1  ANISOMEAN2  ',
     +  '  UISOSIGMA1 UISOSIGMA2 ANISOSIGMA1 ANISOSIGMA2 R2FROMORIG ',
     +  '  URADMEAN1 UTANGMEAN1 ANISOMEAN_H1  ANISOMEAN_H2  ',
     +  '  ANISOSIGMA_H1  ANISOSIGMA_H2  PROLMEAN1  PROLMEAN2')
 6001 FORMAT(/,/,
     +  ' $TABLE: Refined Us for chain ',
     +  A1,':',/,
     +  ' $GRAPHS: Uiso  vs. residue :A:1,2,6: ',/,
     +  ' : Anisotropy vs. residue :A:1,3,11,6: ',/,
     +  ' : Projections of U vs. residue :A:1,7,8,6: $$',/,/,
     +  '  NRES  UISOMEAN1  ANISOMEAN1  ',
     +  '  UISOSIGMA1  ANISOSIGMA1 R2FROMORIG ',
     +  '  URADMEAN1  UTANGMEAN1  ANISOMEAN_H1  ANISOSIGMA_H1 ',
     +  '  PROLMEAN1')
 6003 FORMAT (1X,I4,3X,A1)
 6004 FORMAT(/,/,
     + ' $TABLE: Radial distribution of Urad and Utang :',/,
     + ' $GRAPHS: Radial distribution of Urad and Utang :A:1,5,6: $$',
     +  /,/,'  R2   Num  R2-MIN  R2-MAX   URAD   UTANG  ')
 6005 FORMAT (5X,A3,1X)

      END

      SUBROUTINE U_ANISOTROPY(UMAT,UISO,ANISO1,ANISO2,PROLATE)

C---From anisotropic UMAT, calculate isotropic UISO,
C   and 2 definitions of anisotropy:
C   ANISO1 ratio of smallest to largest eigenvalue
C   ANISO2 as defined by Harata '99
C   PROLATE ratio of middle to largest eigenvalue
C    = 1 if oblate (disc), = ANISO1 if prolate (cigar)

      REAL UEIGEN_MIN,UEIGEN_MID,UEIGEN_MAX,TMP
      REAL UMAT(6),UANISO0(6),EVECTOR(9) 

      UISO = (UMAT(1)+UMAT(2)+UMAT(3))/3.0

      UANISO0(1) = UMAT(1)
      UANISO0(2) = UMAT(4)
      UANISO0(3) = UMAT(2)
      UANISO0(4) = UMAT(5)
      UANISO0(5) = UMAT(6)
      UANISO0(6) = UMAT(3)
      CALL EIGEN_RS_ASC(UANISO0,EVECTOR,3,0)
      UEIGEN_MIN = UANISO0(1)
      UEIGEN_MID = UANISO0(3)
      UEIGEN_MAX = UANISO0(6)
      IF (UEIGEN_MIN.GT.UEIGEN_MAX) THEN
        TMP = UEIGEN_MAX
        UEIGEN_MAX = UEIGEN_MIN
        UEIGEN_MIN = TMP
      ENDIF
      IF (UEIGEN_MIN.GT.UEIGEN_MID) THEN
        TMP = UEIGEN_MID
        UEIGEN_MID = UEIGEN_MIN
        UEIGEN_MIN = TMP
      ENDIF
      IF (UEIGEN_MID.GT.UEIGEN_MAX) THEN
        TMP = UEIGEN_MAX
        UEIGEN_MAX = UEIGEN_MID
        UEIGEN_MID = TMP
      ENDIF

      IF (UEIGEN_MAX .GT. 0.0) THEN
        ANISO1 = UEIGEN_MIN/UEIGEN_MAX
        PROLATE = UEIGEN_MID/UEIGEN_MAX

        ANISO2 = SQRT(((UANISO0(1)-UISO)**2 + (UANISO0(3)-UISO)**2 + 
     +         (UANISO0(6)-UISO)**2)/3.0) / UISO
C  This gives best agreement with ANISO1 .... no idea why!
C      ANISO2 = 0.8 - ANISO2

      ELSE
        ANISO1 = 1.0
        PROLATE = 1.0
        ANISO2 = 0.0
      ENDIF

      END
C
      SUBROUTINE PROJECT_U(UMAT,VEC,UPROJ)

C   Calculate projection of UMAT parallel to vector VEC
C   Assume orthogonal coordinates

      REAL UMAT(6),VEC(3),UPROJ
      REAL V1(3)

      VEC2 = VEC(1)**2 + VEC(2)**2 + VEC(3)**2 

      V1(1) = UMAT(1)*VEC(1) + UMAT(4)*VEC(2) + UMAT(5)*VEC(3) 
      V1(2) = UMAT(4)*VEC(1) + UMAT(2)*VEC(2) + UMAT(6)*VEC(3) 
      V1(3) = UMAT(5)*VEC(1) + UMAT(6)*VEC(2) + UMAT(3)*VEC(3) 

      UPROJ = (VEC(1)*V1(1) + VEC(2)*V1(2) + VEC(3)*V1(3))/VEC2 

      END
C
      SUBROUTINE TLS_FIT
C
C---Fitting of TLS parameters to observed U's

C   cf The small molecule program THMA
C   Schomaker and Trueblood Acta Cryst B54, 507-514, 1998
C   Some notes on THMA for comparison:
C   Normal equations are set up in s/r SOLVE.
C   P is 1st derivative evaluated at TLSpars = 0, i.e. Ucalc = 0
C   AN is 2nd derivative evaluated at TLSpars = 0
C   s/r DERIV calculates AO which is T&S G/H and my DUDTLSGRP
C   Normal eqs solved by MATNVV by matrix inversion, and TLS
C   parameters returned in P.

C   TLS tensors are held in TMAT(6,MAXTLSGRP),LMAT(6,MAXTLSGRP),SMAT(8,MAXTLSGRP)
C   in orthogonal coordinates in A and radians (though radians are converted
C   to degrees on input/output).
      INCLUDE 'anisoanl.fh'

C---Local variables
      REAL SHIFTWGT, CVM(MAXTLSGRP20,MAXTLSGRP20)

C---Calculate A matrices for particular set of orthogonal coordinates
C   obtained from XORT0(3,QQA). Elements of A are orthogonal coordinates
C   w.r.t. TLS origins, and are placed back in XORT0(3,QQA).
      CALL CALC_AMAT
C
      NTLSGRP20 = NTLSGRP*20
      NTLSGRP21 = NTLSGRP*21
      DO    ICYCL=1,NTLS_CYCLE

         WRITE(6,'(/,/,A,I4,/)') ' ***TLS fitting cycle*** ',ICYCL

C---Calculate U matrices from TLS parameters
         CALL TLS2ANISOU
C
C---Calculate derivatives wrt TLS parameters using chain rule
         CALL DERIVS_OF_TLS

         IF (USELTM) THEN
C---Calculate derivatives wrt LTM parameters using chain rule
           CALL DERIVS_OF_LTM
C---Solve linear equations using singular value decomposition
           CALL SVSOLV(HX_LTM,NTLSGRP21,NTLSGRP21,MAXTLSGRP21,
     +        MAXTLSGRP21,GX_LTM,SHFT_LTM,CVM)
           DO I = 1,NTLSGRP21
             SHFT_LTM(I) = SHFT_LTM(I)*1.0
           ENDDO
C---Derive TLS shifts from LTM shifts
           CALL SHFTLTM2SHFTTLS
           SHIFTWGT = 1.0
         ELSE
C---Solve linear equations using singular value decomposition
           CALL SVSOLV(HX_TLS,NTLSGRP20,NTLSGRP20,MAXTLSGRP20,
     +        MAXTLSGRP20,GX_TLS,SHFT_TLS,CVM)
           SHIFTWGT = 1.0
         ENDIF

C---Apply shifts to TLS parameters 
         IPAR = 0
         DO IDOM=1,NTLSGRP
          DO I = 1,6
           IPAR = IPAR +1
           DELTMAT(I,IDOM) = SHFT_TLS(IPAR)*SHIFTWGT
           TMAT(I,IDOM) = TMAT(I,IDOM) + DELTMAT(I,IDOM)
          ENDDO
          DO I = 1,6
           IPAR = IPAR +1
           DELLMAT(I,IDOM) = SHFT_TLS(IPAR)*SHIFTWGT
           LMAT(I,IDOM) = LMAT(I,IDOM) + DELLMAT(I,IDOM)
          ENDDO
          DO I = 1,8
           IPAR = IPAR +1
           DELSMAT(I,IDOM) = SHFT_TLS(IPAR)*SHIFTWGT
           SMAT(I,IDOM) = SMAT(I,IDOM) + DELSMAT(I,IDOM)
          ENDDO

          IF (VERBOSE.OR.ICYCL.EQ.NTLS_CYCLE) THEN
           WRITE(6,'(/,A,I4,A2,A)') ' TLS group ',IDOM,': ',
     +                    TLSGRP_TITLE(IDOM)
           WRITE(6,'(/,A11,I2,A4,6F10.4)') ' T tensor (',IDOM,') = ',
     +                    (TMAT(I,IDOM),I=1,6)
           WRITE(6,'(/,A11,I2,A4,6F10.4)') ' L tensor (',IDOM,') = ',
     +                    (LMAT(I,IDOM)*RTODEG*RTODEG,I=1,6)
           WRITE(6,'(/,A11,I2,A4,9F10.4)') ' S tensor (',IDOM,') = ',
     +                    (SMAT(I,IDOM)*RTODEG,I=1,8)
           IF (USELTM) 
     +       WRITE(6,'(/,A11,I2,A4,6F10.4,/,6F10.4,/,6F10.4,/,3F10.4)') 
     +         ' LTM (',IDOM,') = ',(LTMMAT(I,IDOM),I=1,21)
          ENDIF
         ENDDO

      ENDDO

C---Derive final aniso-U from TLS parameters
      CALL TLS2ANISOU

C---Transfer estimated standard deviations
      DO ITLSGRP=1,NTLSGRP
        ISTART = (ITLSGRP-1)*20 
        DO I = 1,6
          TMATSD(I,ITLSGRP) = SQRT(CVM(ISTART+I,ISTART+I))
        ENDDO
        DO I = 1,6
          LMATSD(I,ITLSGRP) = SQRT(CVM(ISTART+6+I,ISTART+6+I))
        ENDDO
        DO I = 1,8
          SMATSD(I,ITLSGRP) = SQRT(CVM(ISTART+12+I,ISTART+12+I))
        ENDDO
      ENDDO

C---Write new TLS parameters file
      CALL WRITE_TLS_FILE

      RETURN
      END

      SUBROUTINE TLS_INIT
C
C--Defines groups according to read parameters
      INCLUDE 'anisoanl.fh'
C
C---Local variables
      CHARACTER RESNAMP*1,ATNMP*3,CHNNMP*1,TLSFIL*200,LINE*200
      CHARACTER STATUS*5
      INTEGER IDOM
      REAL RNUMTLSGRP(MAXTLSGRP),UANISO_ORT(6),UANISO_CELL(6)
      LOGICAL LTLSSET(MAXTLSGRP)

C---Initialisations
      DO IDOM=1,MAXTLSGRP
        DO I = 1,6
          TMAT(I,IDOM) = 0.0
          LMAT(I,IDOM) = 0.0
        ENDDO
        DO I = 1,8
          SMAT(I,IDOM) = 0.0
        ENDDO
        DO I = 1,21
          LTMMAT(I,IDOM) = 0.0
        ENDDO
        DO I = 1,3
          TLSORIGIN(I,IDOM) = 0.0
        ENDDO
        RNUMTLSGRP(IDOM) = 0.0
        LTLSSET(IDOM) = .FALSE.
      ENDDO

C---If TLSIN defined, read parameters in
      CALL UGTENV('TLSIN',TLSFIL)
      IF (TLSFIL.NE.' ') CALL READ_TLS_FILE

C  Move the NTLSGRP count here 
      NTLSGRP = 0
      DO   II=1,MAXTLSGRP
        IF(ITLSGRP_PIECES(II).GT.0) NTLSGRP = NTLSGRP+1
      ENDDO

      NTLSATOMS = 0
      IF(NTLSGRP.GT.0) THEN
C---Set IGROUP(IA) for each atom based on input residue ranges 
C   and atom selection
C   IGROUP is positive if atom used for fitting, and negative if
C   only used when applying TLS
       DO   IA = 1,NA
        READ(ATOM(IA),'(A1,I4,A3,A1)')RESNAMP,JRS,ATNMP,CHNNMP
        IGROUP(IA) = 0
C---Loop over TLS groups
        DO IDOM=1,NTLSGRP
         IF(ITLSGRP_PIECES(IDOM).GT.0) THEN
          IPIECE = ITLSGRP_PIECES(IDOM)
C---Loop over pieces of TLS group
          DO IP1 = 1,IPIECE
           IF(CHNNMP.EQ.TLSGRP_CHN(IP1,IDOM).AND.
     +                  JRS.GE.ITLSGRP_FROM(IP1,IDOM).AND.
     +                  JRS.LE.ITLSGRP_TO(IP1,IDOM)) THEN
C---Loop over atom selections for this piece
C   Allow atom names (char*3), 'ALL ', 'MNCH', 'SDCH'
            DO ISEL=1,MAXSELECT
             IF (TLSGRP_SELECT(ISEL,IP1,IDOM).EQ.'ALL ') THEN
              IF (ITLSGRP_STATUS(IP1,IDOM).EQ.0) THEN
                IGROUP(IA) = IDOM
              ELSEIF (ITLSGRP_STATUS(IP1,IDOM).EQ.1) THEN
                IGROUP(IA) = - IDOM
              ENDIF
              NTLSATOMS = NTLSATOMS + 1 
              GOTO 10
             ELSEIF (TLSGRP_SELECT(ISEL,IP1,IDOM).EQ.'MNCH' .AND.
     +         (ATNMP(1:2).EQ.'CA'.OR.ATNMP(1:2).EQ.'N '.OR.
     +          ATNMP(1:2).EQ.'C '.OR.ATNMP(1:2).EQ.'O ')) THEN
              IF (ITLSGRP_STATUS(IP1,IDOM).EQ.0) THEN
                IGROUP(IA) = IDOM
              ELSEIF (ITLSGRP_STATUS(IP1,IDOM).EQ.1) THEN
                IGROUP(IA) = - IDOM
              ENDIF
              NTLSATOMS = NTLSATOMS + 1 
              GOTO 10
             ELSEIF (TLSGRP_SELECT(ISEL,IP1,IDOM).EQ.'SDCH' .AND.
     +         (ATNMP(1:2).NE.'CA'.AND.ATNMP(1:2).NE.'N '.AND.
     +          ATNMP(1:2).NE.'C '.AND.ATNMP(1:2).NE.'O ')) THEN
              IF (ITLSGRP_STATUS(IP1,IDOM).EQ.0) THEN
                IGROUP(IA) = IDOM
              ELSEIF (ITLSGRP_STATUS(IP1,IDOM).EQ.1) THEN
                IGROUP(IA) = - IDOM
              ENDIF
              NTLSATOMS = NTLSATOMS + 1 
              GOTO 10
             ELSEIF (TLSGRP_SELECT(ISEL,IP1,IDOM)(1:3).EQ.ATNMP) THEN
              IF (ITLSGRP_STATUS(IP1,IDOM).EQ.0) THEN
                IGROUP(IA) = IDOM
              ELSEIF (ITLSGRP_STATUS(IP1,IDOM).EQ.1) THEN
                IGROUP(IA) = - IDOM
              ENDIF
              NTLSATOMS = NTLSATOMS + 1 
              GOTO 10
             ENDIF
            ENDDO
           ENDIF
          ENDDO
         ENDIF
        ENDDO
 10     CONTINUE
       ENDDO
      ELSE
C---If no groups set up yet, default is whole molecule
        NTLSGRP    = 1
        ITLSGRP_PIECES(1) = 1
        DO   IA=1,NA
          IGROUP(IA) = 1
        ENDDO
        NTLSATOMS = NA
      ENDIF

C---Report TLS Groups

      WRITE(6,'(A)') ' '
      WRITE(6,'(A)') ' '
      WRITE(6,'(A)') '######  TLS Group Definitions ######'
      WRITE(6,'(A)') ' '
      DO II = 1,NTLSGRP
        WRITE(LINE,'(A,I4,A,I4)')'  Group:',II, 
     +  ':    No. of pieces:',ITLSGRP_PIECES(II)
        WRITE(6,'(A)') LINE(1:LENSTR(LINE))
        DO III = 1,ITLSGRP_PIECES(II)
          IF (ITLSGRP_STATUS(III,II).EQ.0) THEN
            WRITE(STATUS,'(A5)') ' FIT '
          ELSEIF (ITLSGRP_STATUS(III,II).EQ.1) THEN
            WRITE(STATUS,'(A5)') 'APPLY'
          ENDIF
          WRITE(LINE,'(A,A,A,2I5,A,A5,A,10A)') 
     +     ' Chain:  ',TLSGRP_CHN(iii,ii),
     +     ' Span: ',ITLSGRP_FROM(III,II),ITLSGRP_TO(III,II),
     +     ' Status: ',STATUS,
     +     ' Atom selection: ',
     +     (TLSGRP_SELECT(ISEL,III,II),ISEL=1,MAXSELECT)
          WRITE(6,'(A)') LINE(1:LENSTR(LINE))
        END DO
        WRITE(6,'(A)') ' '
      END DO
      WRITE(6,'(I5,A,I5,A,/)') NTLSATOMS,' out of ',NA,
     +               ' atoms are included in TLS groups.'

C **** Set origins for TLS groups ****

C---Return TLS group origins in TLSORIGIN
      CALL FIND_TLS_ORIGIN

C **** Set parameter values for TLS groups ****

C---Check if set already
      DO IDOM=1,NTLSGRP
        IF (TMAT(1,IDOM).NE.0.0) LTLSSET(IDOM) = .TRUE.        
      ENDDO 

C---Calculate initial T matrices for groups where not already set
      DO    IA=1,NA
        IDOM = IGROUP(IA)

        IF(IDOM.GT.0 .AND. .NOT.LTLSSET(IDOM)) THEN

C---Calculate averages for initial T matrix
         Q = QOCC(IA)

         DO I = 1,6
           TMAT(I,IDOM) = TMAT(I,IDOM) + U_OBS(IA,I)*Q     
         ENDDO
         RNUMTLSGRP(IDOM) = RNUMTLSGRP(IDOM) + Q       

        ENDIF

      ENDDO 

      DO IDOM=1,NTLSGRP

        IF(.NOT.LTLSSET(IDOM)) THEN
         DO I = 1,6
          TMAT(I,IDOM) = TMAT(I,IDOM)/RNUMTLSGRP(IDOM)
         ENDDO
        ENDIF

         IF (USELTM) THEN
        LTMMAT(1,IDOM) = SQRT(TMAT(1,IDOM))
        LTMMAT(2,IDOM) = SQRT(TMAT(2,IDOM))
        LTMMAT(3,IDOM) = SQRT(TMAT(3,IDOM))
        LTMMAT(7,IDOM) = 0.01
        LTMMAT(8,IDOM) = 0.01
        LTMMAT(9,IDOM) = 0.01
        LMAT(1,IDOM) = LTMMAT(7,IDOM)**2
        LMAT(2,IDOM) = LTMMAT(8,IDOM)**2
        LMAT(3,IDOM) = LTMMAT(9,IDOM)**2
         ENDIF

C---TLS2ANISOU uses DELxMAT to calculate shift in U_ANISO.
C   In first call, shift is from 0 to xMAT.
        DO I = 1,6
          DELTMAT(I,IDOM) = TMAT(I,IDOM) 
          DELLMAT(I,IDOM) = LMAT(I,IDOM)
        ENDDO
        DO I = 1,8
          DELSMAT(I,IDOM) = SMAT(I,IDOM)
        ENDDO

        IF (LFITTLS) THEN
          WRITE(6,'(/,A)') ' Initial TLS parameters '
          WRITE(6,'(/,A,I4,A2,A)') ' TLS group ',IDOM,': ',
     +                    TLSGRP_TITLE(IDOM)
          WRITE(6,'(/,A11,I2,A4,6F10.4)') ' T tensor (',IDOM,') = ',
     +                    (TMAT(I,IDOM),I=1,6)
          WRITE(6,'(/,A11,I2,A4,6F10.4)') ' L tensor (',IDOM,') = ',
     +                    (LMAT(I,IDOM)*RTODEG*RTODEG,I=1,6)
          WRITE(6,'(/,A11,I2,A4,9F10.4)') ' S tensor (',IDOM,') = ',
     +                    (SMAT(I,IDOM)*RTODEG,I=1,8)
          IF (USELTM) 
     +      WRITE(6,'(/,A11,I2,A4,21F10.4)') ' LTM (',IDOM,') = ',
     +                    (LTMMAT(I,IDOM),I=1,21)
        ENDIF
      ENDDO 

      RETURN
      END

      SUBROUTINE READ_TLS_FILE

C---Read TLS parameters from TLSIN

      INCLUDE 'anisoanl.fh'

      INTEGER NPARS
      PARAMETER (NPARS = 200)

c---Things for parser
      CHARACTER LINE*600,KEY*4,CVALUE(NPARS)*4
      INTEGER IBEG(NPARS),IEND(NPARS),IDEC(NPARS),ITYP(NPARS),NTOK
      REAL FVALUE(NPARS)
      LOGICAL LEND,LPRINT

      CHARACTER RANGEFROM*6,RANGETO*6,ERRLIN*200
      INTEGER ITLSIN,IFAIL

      WRITE(6,'(/,A,/)') ' ** READING INPUT TLS FILE **'

      ITLSIN = 0
      IFAIL = 0
      CALL CCPDPN(ITLSIN,'TLSIN','READONLY','F',0,IFAIL)

      IGR = 0
 10   CONTINUE

      LINE = ' '
      READ(ITLSIN,'(A)',END=999) LINE
      KEY  = ' '
      NTOK = NPARS
      LPRINT = .TRUE.

      CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +            LPRINT)

      IF(KEY.EQ.'TLS ') THEN
        IGR = IGR + 1
        TLSGRP_TITLE(IGR) = LINE(5:84)
      ELSEIF(KEY.EQ.'RANG') THEN
        ITLSGRP_PIECES(IGR) = ITLSGRP_PIECES(IGR) + 1
        IPIECE              = ITLSGRP_PIECES(IGR)
        RANGEFROM = LINE (IBEG(2):IEND(2))
        RANGETO = LINE (IBEG(3):IEND(3))
        READ(RANGEFROM,'(A1)') TLSGRP_CHN(IPIECE,IGR)
        READ(RANGEFROM,'(X,I4)') ITLSGRP_FROM(IPIECE,IGR)
        READ(RANGETO,'(X,I4)') ITLSGRP_TO(IPIECE,IGR)
        ITLSGRP_STATUS(IPIECE,IGR) = 0
        ISEL = 0
        TLSGRP_SELECT(1,IPIECE,IGR)='ALL '
C---Is status or atom selection present?
        IF (NTOK.GE.4) THEN
         DO ITOK=4,NTOK
          IF (LINE(IBEG(ITOK):IEND(ITOK)).EQ.'FIT') THEN
            ITLSGRP_STATUS(IPIECE,IGR) = 0
          ELSEIF (LINE(IBEG(ITOK):IEND(ITOK)).EQ.'APPLY') THEN
            ITLSGRP_STATUS(IPIECE,IGR) = 1
          ELSE
            ISEL=ISEL + 1
            IF (ISEL.GT.MAXSELECT) THEN
              WRITE(ERRLIN,'(A,/,A,I4)') 
     +          'Too many atom selections made.',
     +          'Maximum allowed: ',MAXSELECT
              GOTO 10
            ENDIF
            TLSGRP_SELECT(ISEL,IPIECE,IGR)=LINE(IBEG(ITOK):IEND(ITOK))
          ENDIF
         ENDDO
        ENDIF
      ELSEIF(KEY.EQ.'ORIG') THEN
        CALL GTNREA(2,3,TLSORIGIN(1,IGR),NTOK,ITYP,FVALUE)
      ELSEIF(KEY.EQ.'T   ') THEN
        CALL GTNREA(2,6,TMAT(1,IGR),NTOK,ITYP,FVALUE)
      ELSEIF(KEY.EQ.'L   ') THEN
        CALL GTNREA(2,6,LMAT(1,IGR),NTOK,ITYP,FVALUE)
C---TLSIN holds L in units of degrees**2
        DO I = 1,6
          LMAT(I,IGR) = LMAT(I,IGR)*DEGTOR*DEGTOR
        ENDDO
      ELSEIF(KEY.EQ.'S   ') THEN
        CALL GTNREA(2,8,SMAT(1,IGR),NTOK,ITYP,FVALUE)
C---TLSIN holds S in units of degrees*Angstrom
        DO I = 1,8
          SMAT(I,IGR) = SMAT(I,IGR)*DEGTOR
        ENDDO
      ENDIF

      GOTO 10
 999  CONTINUE

      CLOSE(ITLSIN)

      RETURN
      END

      SUBROUTINE WRITE_TLS_FILE

C---Write TLS parameters to TLSOUT
C   Can be analysed with CCP4 program TLSANL

      INCLUDE 'anisoanl.fh'

      INTEGER ITLSOUT,IFAIL,IDOM,III
      CHARACTER RANGEFROM*6,RANGETO*6,STATUS*5,LS(2,20)*1

      ITLSOUT = 0
      IFAIL = 0
      CALL CCPDPN(ITLSOUT,'TLSOUT','NEW','F',0,IFAIL)

      WRITE(ITLSOUT,'(/,A)') 'REFMAC'

      DO IDOM=1,NTLSGRP

        WRITE(ITLSOUT,'(/,A4,A)') 'TLS ',TLSGRP_TITLE(IDOM)
        DO III = 1,ITLSGRP_PIECES(IDOM)
          WRITE(RANGEFROM,'(A1,I4,A1)') TLSGRP_CHN(III,IDOM),
     +          ITLSGRP_FROM(III,IDOM),'.'
          WRITE(RANGETO,'(A1,I4,A1)') TLSGRP_CHN(III,IDOM),
     +          ITLSGRP_TO(III,IDOM),'.'
          IF (ITLSGRP_STATUS(III,IDOM).EQ.0) THEN
            WRITE(STATUS,'(A5)') ' FIT '
          ELSEIF (ITLSGRP_STATUS(III,IDOM).EQ.1) THEN
            WRITE(STATUS,'(A5)') 'APPLY'
          ENDIF

          WRITE(ITLSOUT,'(A7,A1,A6,A3,A6,A2,A5,X,10A)') 'RANGE  ',
     +            '''',RANGEFROM,''' ''',RANGETO,''' ',STATUS,
     +            (TLSGRP_SELECT(ISEL,III,IDOM),ISEL=1,MAXSELECT)
        ENDDO
        WRITE(ITLSOUT,'(A7,3F8.3)') 'ORIGIN ',
     +                          (TLSORIGIN(I,IDOM),I=1,3)

        DO I=1,6
          IF (ABS(TMAT(I,IDOM)).LE.TMATSD(I,IDOM)) THEN
            LS(1,I)='('
            LS(2,I)=')'
          ELSE
            LS(1,I)='<'
            LS(2,I)='>'
          ENDIF
        ENDDO          
        DO I=1,6
          IF (ABS(LMAT(I,IDOM)).LE.LMATSD(I,IDOM)) THEN
            LS(1,6+I)='('
            LS(2,6+I)=')'
          ELSE
            LS(1,6+I)='<'
            LS(2,6+I)='>'
          ENDIF
        ENDDO          
        DO I=1,8
          IF (ABS(SMAT(I,IDOM)).LE.SMATSD(I,IDOM)) THEN
            LS(1,12+I)='('
            LS(2,12+I)=')'
          ELSE
            LS(1,12+I)='<'
            LS(2,12+I)='>'
          ENDIF
        ENDDO          

C--- T, L and S written out in REFMAC order which is different
C    from RESTRAIN. TLSANL altered to allow this.
        WRITE(ITLSOUT,'(A4,6F8.4)') 'T   ',
     +                          (TMAT(I,IDOM),I=1,6)
        WRITE(ITLSOUT,'(A4,X,6(A1,F6.4,A1))') '!   ',
     +      (LS(1,I),TMATSD(I,IDOM),LS(2,I),I=1,6)
        WRITE(ITLSOUT,'(A4,6F8.4)') 'L   ',
     +                          (LMAT(I,IDOM)*RTODEG*RTODEG,I=1,6)
        WRITE(ITLSOUT,'(A4,X,6(A1,F6.4,A1))') '!   ',
     +      (LS(1,6+I),LMATSD(I,IDOM)*RTODEG*RTODEG,LS(2,6+I),I=1,6)
        WRITE(ITLSOUT,'(A4,8F8.4)') 'S   ',
     +                          (SMAT(I,IDOM)*RTODEG,I=1,8)
        WRITE(ITLSOUT,'(A4,X,8(A1,F6.4,A1))') '!   ',
     +      (LS(1,12+I),SMATSD(I,IDOM)*RTODEG,LS(2,12+I),I=1,8)

      ENDDO

      CLOSE(ITLSOUT)

      RETURN
      END

      SUBROUTINE TLS2ANISOU

C---Derive aniso-U from TLS tensors
C   These are held in U_ANISO(QQA,6) in A**2 and orthogonal units

      INCLUDE 'anisoanl.fh'

      REAL UANISO_ORT(6),UANISO_CELL(6),X,Y,Z
      LOGICAL LRECALC

 100  CONTINUE

      DO    IA=1,NA
C---calculate anisoU for all atoms in TLS group
        IDOM = ABS(IGROUP(IA))

        IF(IDOM.GT.0.AND.QOCC(IA).GT.0.0) THEN
          X = XORT0(1,IA)
          Y = XORT0(2,IA)
          Z = XORT0(3,IA)

C---Symmetric tensors are in the order (1,1),(2,2),(3,3),(1,2),(1,3),(2,3)
C---The S tensor is in the order 
C   (2,2)-(1,1),(1,1)-(3,3),(1,2),(1,3),(2,3),(2,1),(3,1),(3,2)
C   Beware! RESTRAIN has different order.

C---If atom included in TLS analysis, derive U tensor in orthogonal units
C   See Table 2 of Schomaker and Trueblood
          U_ANISO(IA,1) = TMAT(1,IDOM) + 
     +      LMAT(2,IDOM)*Z**2 + LMAT(3,IDOM)*Y**2 -
     +      2.0*LMAT(6,IDOM)*Y*Z +
     +      2.0*SMAT(6,IDOM)*Z - 2.0*SMAT(7,IDOM)*Y
          U_ANISO(IA,2) = TMAT(2,IDOM) + 
     +      LMAT(3,IDOM)*X**2 + LMAT(1,IDOM)*Z**2 -
     +      2.0*LMAT(5,IDOM)*Z*X +
     +      2.0*SMAT(8,IDOM)*X - 2.0*SMAT(3,IDOM)*Z
          U_ANISO(IA,3) = TMAT(3,IDOM) + 
     +      LMAT(1,IDOM)*Y**2 + LMAT(2,IDOM)*X**2 -
     +      2.0*LMAT(4,IDOM)*X*Y +
     +      2.0*SMAT(4,IDOM)*Y - 2.0*SMAT(5,IDOM)*X
          U_ANISO(IA,4) = TMAT(4,IDOM) - 
     +      LMAT(4,IDOM)*Z**2 + LMAT(5,IDOM)*Y*Z +
     +      LMAT(6,IDOM)*X*Z - LMAT(3,IDOM)*X*Y +
     +      SMAT(7,IDOM)*X - SMAT(8,IDOM)*Y +
     +      SMAT(1,IDOM)*Z
          U_ANISO(IA,5) = TMAT(5,IDOM) - 
     +      LMAT(5,IDOM)*Y**2 + LMAT(4,IDOM)*Y*Z +
     +      LMAT(6,IDOM)*X*Y - LMAT(2,IDOM)*X*Z +
     +      SMAT(5,IDOM)*Z - SMAT(6,IDOM)*X +
     +      SMAT(2,IDOM)*Y
          U_ANISO(IA,6) = TMAT(6,IDOM) - 
     +      LMAT(6,IDOM)*X**2 + LMAT(5,IDOM)*Y*X +
     +      LMAT(4,IDOM)*X*Z - LMAT(1,IDOM)*Z*Y +
     +      SMAT(3,IDOM)*Y - SMAT(4,IDOM)*Z -
     +      (SMAT(1,IDOM) + SMAT(2,IDOM))*X

        ENDIF
      ENDDO

C---Check positive definite
C      CALL CHECK_U_VALUES
C      CALL CHECK_POSDEF_US(LRECALC) - CVS 1.19 and earlier if needed
      LRECALC = .false.
      IF (LRECALC) THEN
        DO IDOM=1,NTLSGRP
          IF (VERBOSE) THEN
           WRITE(6,'(/,A,I4,A2,A)') ' TLS group ',IDOM,': ',
     +                    TLSGRP_TITLE(IDOM)
           WRITE(6,'(/,A11,I2,A4,6F10.4)') ' T tensor (',IDOM,') = ',
     +                    (TMAT(I,IDOM),I=1,6)
           WRITE(6,'(/,A11,I2,A4,6F10.4)') ' L tensor (',IDOM,') = ',
     +                    (LMAT(I,IDOM)*RTODEG*RTODEG,I=1,6)
           WRITE(6,'(/,A11,I2,A4,9F10.4)') ' S tensor (',IDOM,') = ',
     +                    (SMAT(I,IDOM)*RTODEG,I=1,8)
          ENDIF
        ENDDO
        write(6,*) 'Recalculating U_ANISO'
        GOTO 100
      ENDIF

      RETURN
      END

      SUBROUTINE CALC_AMAT

C---Calculates A matrices for particular set of orthogonal coordinates
C   obtained from XORT0(3,QQA). Elements of A are orthogonal coordinates
C   w.r.t. TLS origins, and are placed back in XORT0(3,QQA).

C   Changing origin or equivalently XORT0 shouldn't affect converged values
C   of U, but does change corresponding values of T and S.

      INCLUDE 'anisoanl.fh'

      DO    IA = 1,NA
         ITLSGRP = ABS(IGROUP(IA))
         IF(ITLSGRP.GT.0) THEN
           DO   IX=1,3
             XORT0(IX,IA) = XORT0(IX,IA)-TLSORIGIN(IX,ITLSGRP)
           ENDDO
         ENDIF
      ENDDO

      RETURN
      END
       
      SUBROUTINE FIND_TLS_ORIGIN
C
C---Finds origin of each TLS group as centroid of atoms.
      INCLUDE 'anisoanl.fh'

      REAL QATOM(MAXTLSGRP)
      LOGICAL LORIGINSET(MAXTLSGRP)

      WRITE(6,*) ' '
      DO ITLSGRP = 1,NTLSGRP
        QATOM(ITLSGRP) = 0.0
        IF (TLSORIGIN(1,ITLSGRP).EQ.0.0) THEN
          LORIGINSET(ITLSGRP) = .FALSE.
          WRITE(6,'(A,I2,A)') 'Calculating TLS origin of group ',
     +                              ITLSGRP,' from centroid of group.'
        ELSE
          LORIGINSET(ITLSGRP) = .TRUE.
        ENDIF
      ENDDO
      WRITE(6,*) ' '

      DO    IA=1,NA
        ITLSGRP  = IGROUP(IA)
        IF(ITLSGRP.GT.0 .AND. .NOT.LORIGINSET(ITLSGRP)) THEN
          DO    IX=1,3
            TLSORIGIN(IX,ITLSGRP) = TLSORIGIN(IX,ITLSGRP) +
     +                                      XORT0(IX,IA)*QOCC(IA)
          ENDDO
          QATOM(ITLSGRP) = QATOM(ITLSGRP) + QOCC(IA)
        ENDIF
      ENDDO

      DO    ITLSGRP = 1,NTLSGRP
        IF(QATOM(ITLSGRP).GT.0.0 .AND. .NOT.LORIGINSET(ITLSGRP)) THEN
          DO   IX=1,3
            TLSORIGIN(IX,ITLSGRP) = TLSORIGIN(IX,ITLSGRP)/QATOM(ITLSGRP)
          ENDDO
        ENDIF
      ENDDO

      DO ITLSGRP = 1,NTLSGRP
        write(6,'(a,i5,3f15.6)') 'TLS origin for group ',ITLSGRP,
     +                           (TLSORIGIN(IX,ITLSGRP),IX=1,3)
      ENDDO

      RETURN
      END

      SUBROUTINE DERIVS_OF_TLS
C
C---Calculates derivatives of minimised function wrt TLS parameters.
C---using chain rule. 
C
C--- 1) Neglects second derivatives of U, but U linear in TLS.
C--- 2) Only diagonal terms w.r.t. atoms of second derivative of function 
C     wrt U are used. But HUU doesn't contain off-diagonal terms.
C--- 3) No cross-derivatives w.r.t. different TLS groups are included.
C     But these would need terms of 2) which aren't available.

      INCLUDE 'anisoanl.fh'

      INTEGER IA
      REAL DUDTLSGRP(6,20)
     
C---Initialise
C
      DO    IPAR=1,MAXTLSGRP20
        GX_TLS(IPAR) = 0.0
        DO    IPAR1=1,MAXTLSGRP20
          HX_TLS(IPAR1,IPAR) = 0.0
        ENDDO
      ENDDO
C
C---Evaluate GU1 and HUU
      CALL DERIVS_OF_RESFN

C---Calculate derivatives of minimised function wrt tlsgrp
C---parameters using chain rule. 
      DO   IA=1,NA
       ITLSGRP = IGROUP(IA)
       IF(ITLSGRP.GT.0) THEN
        IU = 6*IA - 5
        IUU = 21*IA - 20

        CALL DERIVS_WRT_TLSPARS(IA,DUDTLSGRP)

        IPAR0   = 20*ITLSGRP-20
        DO   IPAR = 1,20
          IPAR0 = IPAR0 + 1
C---Note SVSOLV requires -(gradient)
          GX_TLS(IPAR0) = GX_TLS(IPAR0) -
     +                       (GU1(IU)*DUDTLSGRP(1,IPAR)+
     +                        GU1(IU+1)*DUDTLSGRP(2,IPAR)+
     +                        GU1(IU+2)*DUDTLSGRP(3,IPAR)+
     +                        GU1(IU+3)*DUDTLSGRP(4,IPAR)+
     +                        GU1(IU+4)*DUDTLSGRP(5,IPAR)+
     +                        GU1(IU+5)*DUDTLSGRP(6,IPAR))
          IPAR10 = 20*ITLSGRP-20
          DO   IPAR1 = 1,20
            IPAR10 = IPAR10 + 1

C---First contribution: diagonal HUU elements
            HX_TLS(IPAR0,IPAR10) = HX_TLS(IPAR0,IPAR10) + 
     +                      (HUU(IUU)*DUDTLSGRP(1,IPAR)*
     +                                 DUDTLSGRP(1,IPAR1)+
     +                       HUU(IUU+1)*DUDTLSGRP(2,IPAR)*
     +                                 DUDTLSGRP(2,IPAR1)+
     +                       HUU(IUU+2)*DUDTLSGRP(3,IPAR)*
     +                                 DUDTLSGRP(3,IPAR1)+
     +                       HUU(IUU+3)*DUDTLSGRP(4,IPAR)*
     +                                 DUDTLSGRP(4,IPAR1)+
     +                       HUU(IUU+4)*DUDTLSGRP(5,IPAR)*
     +                                 DUDTLSGRP(5,IPAR1)+
     +                       HUU(IUU+5)*DUDTLSGRP(6,IPAR)*
     +                                 DUDTLSGRP(6,IPAR1))

C---Second contribution: off-diagonal HUU elements
            HX_TLS(IPAR0,IPAR10) = HX_TLS(IPAR0,IPAR10) + (
     +    HUU(IUU+6) *(DUDTLSGRP(1,IPAR)*DUDTLSGRP(2,IPAR1) +
     +                 DUDTLSGRP(2,IPAR)*DUDTLSGRP(1,IPAR1)) +
     +    HUU(IUU+7) *(DUDTLSGRP(1,IPAR)*DUDTLSGRP(3,IPAR1) +
     +                 DUDTLSGRP(3,IPAR)*DUDTLSGRP(1,IPAR1)) +
     +    HUU(IUU+8) *(DUDTLSGRP(1,IPAR)*DUDTLSGRP(4,IPAR1) +
     +                 DUDTLSGRP(4,IPAR)*DUDTLSGRP(1,IPAR1)) +
     +    HUU(IUU+9) *(DUDTLSGRP(1,IPAR)*DUDTLSGRP(5,IPAR1) +
     +                 DUDTLSGRP(5,IPAR)*DUDTLSGRP(1,IPAR1)) +
     +    HUU(IUU+10)*(DUDTLSGRP(1,IPAR)*DUDTLSGRP(6,IPAR1) +
     +                 DUDTLSGRP(6,IPAR)*DUDTLSGRP(1,IPAR1)) +
     +    HUU(IUU+11)*(DUDTLSGRP(2,IPAR)*DUDTLSGRP(3,IPAR1) +
     +                 DUDTLSGRP(3,IPAR)*DUDTLSGRP(2,IPAR1)) +
     +    HUU(IUU+12)*(DUDTLSGRP(2,IPAR)*DUDTLSGRP(4,IPAR1) +
     +                 DUDTLSGRP(4,IPAR)*DUDTLSGRP(2,IPAR1)) +
     +    HUU(IUU+13)*(DUDTLSGRP(2,IPAR)*DUDTLSGRP(5,IPAR1) +
     +                 DUDTLSGRP(5,IPAR)*DUDTLSGRP(2,IPAR1)) +
     +    HUU(IUU+14)*(DUDTLSGRP(2,IPAR)*DUDTLSGRP(6,IPAR1) +
     +                 DUDTLSGRP(6,IPAR)*DUDTLSGRP(2,IPAR1)) +
     +    HUU(IUU+15)*(DUDTLSGRP(3,IPAR)*DUDTLSGRP(4,IPAR1) +
     +                 DUDTLSGRP(4,IPAR)*DUDTLSGRP(3,IPAR1)) +
     +    HUU(IUU+16)*(DUDTLSGRP(3,IPAR)*DUDTLSGRP(5,IPAR1) +
     +                 DUDTLSGRP(5,IPAR)*DUDTLSGRP(3,IPAR1)) +
     +    HUU(IUU+17)*(DUDTLSGRP(3,IPAR)*DUDTLSGRP(6,IPAR1) +
     +                 DUDTLSGRP(6,IPAR)*DUDTLSGRP(3,IPAR1)) +
     +    HUU(IUU+18)*(DUDTLSGRP(4,IPAR)*DUDTLSGRP(5,IPAR1) +
     +                 DUDTLSGRP(5,IPAR)*DUDTLSGRP(4,IPAR1)) +
     +    HUU(IUU+19)*(DUDTLSGRP(4,IPAR)*DUDTLSGRP(6,IPAR1) +
     +                 DUDTLSGRP(6,IPAR)*DUDTLSGRP(4,IPAR1)) +
     +    HUU(IUU+20)*(DUDTLSGRP(5,IPAR)*DUDTLSGRP(6,IPAR1) +
     +                 DUDTLSGRP(6,IPAR)*DUDTLSGRP(5,IPAR1))
     +                                 )

           ENDDO
         ENDDO
        ENDIF
      ENDDO

      RETURN
      END

      SUBROUTINE DERIVS_OF_RESFN

C---This routine calculates GU1 and HUU which are 1st and 2nd derivatives
C   of chosen residual function wrt to calculated aniso Us.
C   Should be able to change function here.

C   First try L2 norm.

      INCLUDE 'anisoanl.fh'

      RESL2_FIT = 0.0
      RESL2_APPLY = 0.0
      RESL2_ALL = 0.0
      SUMOBS_FIT = 0.0
      SUMOBS_APPLY = 0.0
      DO    IA=1,NA
        IU = 6*(IA - 1)
        IUU = 21*(IA - 1)

        DO J=1,3
          RESL2_ALL = RESL2_ALL + (U_ANISO(IA,J) - U_OBS(IA,J))**2
        ENDDO
        DO J=4,6
          RESL2_ALL = RESL2_ALL + 2.0 * (U_ANISO(IA,J) - U_OBS(IA,J))**2
        ENDDO

        IF (IGROUP(IA).NE.0) THEN
          DO J=1,3
            RESL2_APPLY = RESL2_APPLY + (U_ANISO(IA,J) - U_OBS(IA,J))**2
            SUMOBS_APPLY = SUMOBS_APPLY + U_OBS(IA,J)**2
          ENDDO
          DO J=4,6
            RESL2_APPLY = RESL2_APPLY + 
     +             2.0 * (U_ANISO(IA,J) - U_OBS(IA,J))**2
            SUMOBS_APPLY = SUMOBS_APPLY + 2.0 * U_OBS(IA,J)**2
          ENDDO
        ENDIF

C---derivatives just for those atoms in fitting parts of TLS groups
        IF (IGROUP(IA).GT.0) THEN
          DO J=1,3
            RESL2_FIT = RESL2_FIT + (U_ANISO(IA,J) - U_OBS(IA,J))**2
            SUMOBS_FIT = SUMOBS_FIT + U_OBS(IA,J)**2
            GU1(IU+J) = 2.0 * (U_ANISO(IA,J) - U_OBS(IA,J))
            HUU(IUU+J) = 2.0 
          ENDDO
          DO J=4,6
            RESL2_FIT = RESL2_FIT + 
     +            2.0 * (U_ANISO(IA,J) - U_OBS(IA,J))**2
            SUMOBS_FIT = SUMOBS_FIT + 2.0 * U_OBS(IA,J)**2
            GU1(IU+J) = 4.0 * (U_ANISO(IA,J) - U_OBS(IA,J))
            HUU(IUU+J) = 4.0 
          ENDDO
          DO J=7,21
            HUU(IUU+J) = 0.0
          ENDDO
        ENDIF

      ENDDO

C---estimate Goodness of Fit for fitting atoms
C   uses definition of Trueblood, assuming sigU = 0.001
      GOF = SQRT(RESL2_FIT/REAL(6*NTLSATOMS - 20*NTLSGRP))*1000.0
C---R values
C   Note that these equal 1 for T/L/S = 0 so the initial values
C   reflect initial TLS values chosen by program. Currently, T is
C   set to average U's and this accounts for most of drop in R.
      R_FIT = SQRT(RESL2_FIT / SUMOBS_FIT)
      R_APPLY = SQRT(RESL2_APPLY / SUMOBS_APPLY)

      WRITE(6,*) 
     +  'Value of L2 residual (atoms used in fitting TLS)  = ',
     +       RESL2_FIT
      WRITE(6,*) 
     +  'Value of L2 residual (atoms to which TLS applied) = ',
     +       RESL2_APPLY
      WRITE(6,*) 
      WRITE(6,*) 'R value (atoms used in fitting TLS)  = ',R_FIT
      WRITE(6,*) 'R value (atoms to which TLS applied) = ',R_APPLY
      WRITE(6,*) 
      WRITE(6,*) 'Goodness of Fit (atoms used in fitting TLS) = ',
     +       GOF

      RETURN
      END

      SUBROUTINE DERIVS_OF_L2N(IPOW)

C---This routine calculates GU1 and HUU which are 1st and 2nd derivatives
C   of chosen residual function wrt to calculated aniso Us.
C   Should be able to change function here.

C   L2-N residual
C   Quadratic if Ucalc > Uobs and power IPOW if Ucalc < Uobs
C   i.e. favours Ucalc < Uobs and positive Uresid

C   NOT USED: TLS parameters are less well determined, while remaining
C   similar to the least-squares result

      INCLUDE 'anisoanl.fh'

      INTEGER IPOW
      LOGICAL GREATER

      RESL2_FIT = 0.0
      RESL2_APPLY = 0.0
      RESL2N_FIT = 0.0
      RESL2N_APPLY = 0.0
      SUMOBS_FIT = 0.0
      SUMOBS_APPLY = 0.0
      DO    IA=1,NA
        IU = 6*(IA - 1)
        IUU = 21*(IA - 1)

        GREATER = .TRUE.
        UCALCTR = U_ANISO(IA,1)+U_ANISO(IA,2)+U_ANISO(IA,3)
        UOBSTR = U_OBS(IA,1)+U_OBS(IA,2)+U_OBS(IA,3)
        IF (UCALCTR.LT.UOBSTR) GREATER = .FALSE.

        IF (IGROUP(IA).NE.0) THEN
C Accumulate sums for R_APPLY
         DO J=1,3
           RESL2_APPLY = RESL2_APPLY + (U_ANISO(IA,J) - U_OBS(IA,J))**2
           SUMOBS_APPLY = SUMOBS_APPLY + U_OBS(IA,J)**2
         ENDDO
         DO J=4,6
           RESL2_APPLY = RESL2_APPLY + 
     +             2.0 * (U_ANISO(IA,J) - U_OBS(IA,J))**2
           SUMOBS_APPLY = SUMOBS_APPLY + 2.0 * U_OBS(IA,J)**2
         ENDDO
C Accumulate sums for RESL2N_APPLY
         IF (GREATER) THEN
          DO J=1,3
            RESL2N_APPLY = RESL2N_APPLY + 
     +                (U_ANISO(IA,J) - U_OBS(IA,J))**2
          ENDDO
          DO J=4,6
            RESL2N_APPLY = RESL2N_APPLY + 
     +             2.0 * (U_ANISO(IA,J) - U_OBS(IA,J))**2
          ENDDO
         ELSE
          DO J=1,3
            RESL2N_APPLY = RESL2N_APPLY + (-1)**IPOW *
     +                (U_ANISO(IA,J)-U_OBS(IA,J))**IPOW
          ENDDO
          DO J=4,6
            RESL2N_APPLY = RESL2N_APPLY + (-1)**IPOW *
     +             2.0 * (U_ANISO(IA,J) - U_OBS(IA,J))**IPOW
          ENDDO
         ENDIF
        ENDIF

C---derivatives just for those atoms in fitting parts of TLS groups
        IF (IGROUP(IA).GT.0) THEN
C Accumulate sums for R_FIT
         DO J=1,3
           RESL2_FIT = RESL2_FIT + (U_ANISO(IA,J) - U_OBS(IA,J))**2
           SUMOBS_FIT = SUMOBS_FIT + U_OBS(IA,J)**2
         ENDDO
         DO J=4,6
           RESL2_FIT = RESL2_FIT + 
     +             2.0 * (U_ANISO(IA,J) - U_OBS(IA,J))**2
           SUMOBS_FIT = SUMOBS_FIT + 2.0 * U_OBS(IA,J)**2
         ENDDO
C Accumulate sums for RESL2N_FIT, and derivatives
         IF (GREATER) THEN
          DO J=1,3
            RESL2N_FIT = RESL2N_FIT + 
     +                 (U_ANISO(IA,J) - U_OBS(IA,J))**2
            GU1(IU+J) = 2.0 * (U_ANISO(IA,J) - U_OBS(IA,J))
            HUU(IUU+J) = 2.0 
          ENDDO
          DO J=4,6
            RESL2N_FIT = RESL2N_FIT + 
     +            2.0 * (U_ANISO(IA,J) - U_OBS(IA,J))**2
            GU1(IU+J) = 4.0 * (U_ANISO(IA,J) - U_OBS(IA,J))
            HUU(IUU+J) = 4.0 
          ENDDO
          DO J=7,21
            HUU(IUU+J) = 0.0
          ENDDO
         ELSE
          DO J=1,3
            RESL2N_FIT = RESL2N_FIT +  (-1)**IPOW *
     +              (U_ANISO(IA,J) - U_OBS(IA,J))**IPOW
            GU1(IU+J) =  (-1)**IPOW * REAL(IPOW) * 
     +                 (U_ANISO(IA,J) - U_OBS(IA,J))**(IPOW-1)
            HUU(IUU+J) =  (-1)**IPOW * REAL(IPOW*(IPOW-1)) * 
     +                 (U_ANISO(IA,J) - U_OBS(IA,J))**(IPOW-2)
          ENDDO
          DO J=4,6
            RESL2N_FIT = RESL2N_FIT +  (-1)**IPOW *
     +            2.0 * (U_ANISO(IA,J) - U_OBS(IA,J))**IPOW
            GU1(IU+J) =  (-1)**IPOW * REAL(IPOW*2) * 
     +                  (U_ANISO(IA,J) - U_OBS(IA,J))**(IPOW-1)
            HUU(IUU+J) =  (-1)**IPOW * REAL(IPOW*(IPOW-1)*2) * 
     +                  (U_ANISO(IA,J) - U_OBS(IA,J))**(IPOW-2)
          ENDDO
          DO J=7,21
            HUU(IUU+J) = 0.0
          ENDDO
         ENDIF
        ENDIF

      ENDDO

C---estimate Goodness of Fit for fitting atoms
C   uses definition of Trueblood, assuming sigU = 0.001
      GOF = SQRT(RESL2_FIT/REAL(6*NTLSATOMS - 20*NTLSGRP))*1000.0
C---R values
C   Note that these equal 1 for T/L/S = 0 so the initial values
C   reflect initial TLS values chosen by program. Currently, T is
C   set to average U's and this accounts for most of drop in R.
      R_FIT = SQRT(RESL2_FIT / SUMOBS_FIT)
      R_APPLY = SQRT(RESL2_APPLY / SUMOBS_APPLY)

      WRITE(6,*) 
     +  'Value of L2N residual (atoms used in fitting TLS)  = ',
     +       RESL2N_FIT
      WRITE(6,*) 
     +  'Value of L2N residual (atoms to which TLS applied) = ',
     +       RESL2N_APPLY
      WRITE(6,*) 
      WRITE(6,*) 'R value (atoms used in fitting TLS)  = ',R_FIT
      WRITE(6,*) 'R value (atoms to which TLS applied) = ',R_APPLY
      WRITE(6,*) 
      WRITE(6,*) 'Goodness of Fit (atoms used in fitting TLS) = ',
     +       GOF

      RETURN
      END

      SUBROUTINE DERIVS_WRT_TLSPARS(IA,DUDTLSGRP)
C
C---Calculates derivatives of anisotropic displacement parameters wrt 
C---TLS parameters for atom IA and stores them in DUDTLSGRP

C---cf. RESTRAIN s/r lscalc line 9001
      INCLUDE 'anisoanl.fh'

      INTEGER IA
      REAL DUDTLSGRP(6,20)

      REAL X,Y,Z,XX,YY,ZZ,XY,XZ,YZ

      DO   ITLS = 1,20
       DO   IU = 1,6
         DUDTLSGRP(IU,ITLS) = 0.0
       ENDDO
      ENDDO

C---Contribution of T tensor
      DUDTLSGRP(1,1) = 1.0
      DUDTLSGRP(2,2) = 1.0
      DUDTLSGRP(3,3) = 1.0
      DUDTLSGRP(4,4) = 1.0
      DUDTLSGRP(5,5) = 1.0
      DUDTLSGRP(6,6) = 1.0

C---Contribution of L tensor
C   See G in Table 2 of Schomaker and Trueblood
      X = XORT0(1,IA)
      Y = XORT0(2,IA)
      Z = XORT0(3,IA)
      XX = X**2
      YY = Y**2
      ZZ = Z**2
      XY = X*Y
      XZ = X*Z
      YZ = Y*Z
      DUDTLSGRP(1, 8) = ZZ
      DUDTLSGRP(1, 9) = YY
      DUDTLSGRP(1,12) = -2.0*YZ
      DUDTLSGRP(2, 7) = ZZ
      DUDTLSGRP(2, 9) = XX
      DUDTLSGRP(2,11) = -2.0*XZ
      DUDTLSGRP(3, 7) = YY
      DUDTLSGRP(3, 8) = XX
      DUDTLSGRP(3,10) = -2.0*XY
      DUDTLSGRP(4, 9) = -XY
      DUDTLSGRP(4,10) = -ZZ
      DUDTLSGRP(4,11) = YZ
      DUDTLSGRP(4,12) = XZ
      DUDTLSGRP(5, 8) = -XZ
      DUDTLSGRP(5,10) = YZ
      DUDTLSGRP(5,11) = -YY
      DUDTLSGRP(5,12) = XY
      DUDTLSGRP(6, 7) = -YZ
      DUDTLSGRP(6,10) = XZ
      DUDTLSGRP(6,11) = XY
      DUDTLSGRP(6,12) = -XX

C---Contribution of S tensor
C   See H in Table 2 of Schomaker and Trueblood
      DUDTLSGRP(1,18) = 2.0*Z 
      DUDTLSGRP(1,19) = -2.0*Y
      DUDTLSGRP(2,20) = 2.0*X
      DUDTLSGRP(2,15) = -2.0*Z
      DUDTLSGRP(3,16) = 2.0*Y
      DUDTLSGRP(3,17) = -2.0*X
      DUDTLSGRP(4,19) = X
      DUDTLSGRP(4,20) = -Y
      DUDTLSGRP(4,13) = Z
      DUDTLSGRP(5,17) = Z
      DUDTLSGRP(5,18) = -X
      DUDTLSGRP(5,14) = Y
      DUDTLSGRP(6,15) = Y
      DUDTLSGRP(6,16) = -Z
      DUDTLSGRP(6,13) = -X
      DUDTLSGRP(6,14) = -X

      RETURN
      END

      SUBROUTINE DERIVS_OF_LTM
C
C---Calculates derivatives of minimised function wrt LTM parameters.
C---using chain rule. 
C   I.e. GX_LTM, HX_LTM from GX_TLS, HX_TLS using first derivative
C   of TLS wrt LTM (ignore contribution of 2nd derivative)
C
C      IMPLICIT NONE

      INCLUDE 'anisoanl.fh'

      INTEGER IPARLTM1,IPARLTM11,IPARTLS1,IPARTLS11,
     +        IPARLTM2,IPARLTM22,IPARTLS2,IPARTLS22,
     +        ITLSGRP
      REAL DTLSDLTM(20,21)
     
C---Initialise
C
      DO    IPARLTM1=1,MAXTLSGRP21
        GX_LTM(IPARLTM1) = 0.0
        DO    IPARLTM2=1,MAXTLSGRP21
          HX_LTM(IPARLTM2,IPARLTM1) = 0.0
        ENDDO
      ENDDO

      DO ITLSGRP = 1,NTLSGRP
        CALL DERIVS_WRT_LTMPARS(ITLSGRP,DTLSDLTM)

        IPARLTM1   = 21*ITLSGRP-21
        DO IPARLTM11 = 1,21
         IPARLTM1 = IPARLTM1 + 1

         IPARTLS1   = 20*ITLSGRP-20
         DO IPARTLS11 = 1,20
          IPARTLS1 = IPARTLS1 + 1
          
          GX_LTM(IPARLTM1) = GX_LTM(IPARLTM1) +
     +       GX_TLS(IPARTLS1)*DTLSDLTM(IPARTLS11,IPARLTM11)

          IPARLTM2   = 21*ITLSGRP-21
          DO IPARLTM22 = 1,21
           IPARLTM2 = IPARLTM2 + 1

           IPARTLS2   = 20*ITLSGRP-20
           DO IPARTLS22 = 1,20
            IPARTLS2 = IPARTLS2 + 1
           
            HX_LTM(IPARLTM2,IPARLTM1) = HX_LTM(IPARLTM2,IPARLTM1) +
     +        HX_TLS(IPARTLS2,IPARTLS1)*DTLSDLTM(IPARTLS22,IPARLTM22)
     +                     *DTLSDLTM(IPARTLS11,IPARLTM11)

           ENDDO
          ENDDO
         ENDDO
        ENDDO
      ENDDO

      RETURN
      END

      SUBROUTINE DERIVS_WRT_LTMPARS(ITLSGRP,DTLSDLTM)
C
C---Calculates derivatives of TLS parameters wrt parameters of lower
C---triangular matrix for TLS group ITLSGRP and stores them in DTLSDLTM

C      IMPLICIT NONE

      INCLUDE 'anisoanl.fh'

      INTEGER ITLSGRP,ILTM,ITLS
      REAL DTLSDLTM(20,21)

      DO   ILTM = 1,21
       DO   ITLS = 1,20
         DTLSDLTM(ITLS,ILTM) = 0.0
       ENDDO
      ENDDO

C---T, L tensors are in the order (1,1),(2,2),(3,3),(1,2),(1,3),(2,3)
C---The S tensor is in the order 
C   (2,2)-(1,1),(1,1)-(3,3),(1,2),(1,3),(2,3),(2,1),(3,1),(3,2)
C   Beware! RESTRAIN has different order.

C---Gradients of T
      DTLSDLTM(1,1) = 2.0*LTMMAT(1,ITLSGRP)
      DTLSDLTM(2,2) = 2.0*LTMMAT(2,ITLSGRP) 
      DTLSDLTM(2,4) = 2.0*LTMMAT(4,ITLSGRP) 
      DTLSDLTM(3,3) = 2.0*LTMMAT(3,ITLSGRP) 
      DTLSDLTM(3,5) = 2.0*LTMMAT(5,ITLSGRP) 
      DTLSDLTM(3,6) = 2.0*LTMMAT(6,ITLSGRP) 
      DTLSDLTM(4,1) = LTMMAT(4,ITLSGRP)
      DTLSDLTM(4,4) = LTMMAT(1,ITLSGRP)
      DTLSDLTM(5,1) = LTMMAT(5,ITLSGRP)
      DTLSDLTM(5,5) = LTMMAT(1,ITLSGRP)
      DTLSDLTM(6,2) = LTMMAT(6,ITLSGRP)
      DTLSDLTM(6,4) = LTMMAT(5,ITLSGRP)
      DTLSDLTM(6,5) = LTMMAT(4,ITLSGRP)
      DTLSDLTM(6,6) = LTMMAT(2,ITLSGRP)

C---Gradients of L
      DTLSDLTM(7,7) = 2.0*LTMMAT(7,ITLSGRP)
      DTLSDLTM(7,13) = 2.0*LTMMAT(13,ITLSGRP)
      DTLSDLTM(7,18) = 2.0*LTMMAT(18,ITLSGRP)
      DTLSDLTM(7,19) = 2.0*LTMMAT(19,ITLSGRP)
      DTLSDLTM(8,8) = 2.0*LTMMAT(8,ITLSGRP)
      DTLSDLTM(8,10) = 2.0*LTMMAT(10,ITLSGRP)
      DTLSDLTM(8,14) = 2.0*LTMMAT(14,ITLSGRP)
      DTLSDLTM(8,15) = 2.0*LTMMAT(15,ITLSGRP)
      DTLSDLTM(8,20) = 2.0*LTMMAT(20,ITLSGRP)
      DTLSDLTM(9,9) = 2.0*LTMMAT(9,ITLSGRP)
      DTLSDLTM(9,11) = 2.0*LTMMAT(11,ITLSGRP)
      DTLSDLTM(9,12) = 2.0*LTMMAT(12,ITLSGRP)
      DTLSDLTM(9,16) = 2.0*LTMMAT(16,ITLSGRP)
      DTLSDLTM(9,17) = 2.0*LTMMAT(17,ITLSGRP)
      DTLSDLTM(9,21) = 2.0*LTMMAT(21,ITLSGRP)
      DTLSDLTM(10,7)  = LTMMAT(10,ITLSGRP)
      DTLSDLTM(10,13) = LTMMAT(15,ITLSGRP)
      DTLSDLTM(10,18) = LTMMAT(14,ITLSGRP)
      DTLSDLTM(10,19) = LTMMAT(20,ITLSGRP)
      DTLSDLTM(10,10) = LTMMAT(7,ITLSGRP)
      DTLSDLTM(10,14) = LTMMAT(18,ITLSGRP)
      DTLSDLTM(10,15) = LTMMAT(13,ITLSGRP)
      DTLSDLTM(10,20) = LTMMAT(19,ITLSGRP)
      DTLSDLTM(11,7) = LTMMAT(11,ITLSGRP)
      DTLSDLTM(11,13) = LTMMAT(16,ITLSGRP)
      DTLSDLTM(11,18) = LTMMAT(17,ITLSGRP)
      DTLSDLTM(11,19) = LTMMAT(21,ITLSGRP)
      DTLSDLTM(11,11) = LTMMAT(7,ITLSGRP)
      DTLSDLTM(11,16) = LTMMAT(13,ITLSGRP)
      DTLSDLTM(11,17) = LTMMAT(18,ITLSGRP)
      DTLSDLTM(11,21) = LTMMAT(19,ITLSGRP)
      DTLSDLTM(12,15) = LTMMAT(16,ITLSGRP)
      DTLSDLTM(12,14) = LTMMAT(17,ITLSGRP)
      DTLSDLTM(12,20) = LTMMAT(21,ITLSGRP)
      DTLSDLTM(12,10) = LTMMAT(11,ITLSGRP)
      DTLSDLTM(12,8) = LTMMAT(12,ITLSGRP)
      DTLSDLTM(12,16) = LTMMAT(15,ITLSGRP)
      DTLSDLTM(12,17) = LTMMAT(14,ITLSGRP)
      DTLSDLTM(12,21) = LTMMAT(20,ITLSGRP)
      DTLSDLTM(12,11) = LTMMAT(10,ITLSGRP)
      DTLSDLTM(12,12) = LTMMAT(8,ITLSGRP)

C---Gradients of S

      DTLSDLTM(13,4) = LTMMAT(15,ITLSGRP)
      DTLSDLTM(13,15) = LTMMAT(4,ITLSGRP)
      DTLSDLTM(13,2) = LTMMAT(14,ITLSGRP)
      DTLSDLTM(13,14) = LTMMAT(2,ITLSGRP)
      DTLSDLTM(13,1) = - LTMMAT(13,ITLSGRP)
      DTLSDLTM(13,13) = - LTMMAT(1,ITLSGRP)
      DTLSDLTM(14,1) = LTMMAT(13,ITLSGRP)
      DTLSDLTM(14,13) = LTMMAT(1,ITLSGRP)
      DTLSDLTM(14,5) = - LTMMAT(16,ITLSGRP)
      DTLSDLTM(14,6) = - LTMMAT(17,ITLSGRP)
      DTLSDLTM(14,3) = - LTMMAT(21,ITLSGRP)
      DTLSDLTM(14,16) = - LTMMAT(5,ITLSGRP)
      DTLSDLTM(14,17) = - LTMMAT(6,ITLSGRP)
      DTLSDLTM(14,21) = - LTMMAT(3,ITLSGRP)
      DTLSDLTM(15,1) = LTMMAT(15,ITLSGRP)
      DTLSDLTM(15,15) = LTMMAT(1,ITLSGRP)
      DTLSDLTM(16,1) = LTMMAT(16,ITLSGRP)
      DTLSDLTM(16,16) = LTMMAT(1,ITLSGRP)
      DTLSDLTM(17,4) = LTMMAT(16,ITLSGRP)
      DTLSDLTM(17,2) = LTMMAT(17,ITLSGRP)
      DTLSDLTM(17,16) = LTMMAT(4,ITLSGRP)
      DTLSDLTM(17,17) = LTMMAT(2,ITLSGRP)
      DTLSDLTM(18,4) = LTMMAT(13,ITLSGRP)
      DTLSDLTM(18,2) = LTMMAT(18,ITLSGRP)
      DTLSDLTM(18,13) = LTMMAT(4,ITLSGRP)
      DTLSDLTM(18,18) = LTMMAT(2,ITLSGRP)
      DTLSDLTM(19,5) = LTMMAT(13,ITLSGRP)
      DTLSDLTM(19,6) = LTMMAT(18,ITLSGRP)
      DTLSDLTM(19,3) = LTMMAT(19,ITLSGRP)
      DTLSDLTM(19,13) = LTMMAT(5,ITLSGRP)
      DTLSDLTM(19,18) = LTMMAT(6,ITLSGRP)
      DTLSDLTM(19,19) = LTMMAT(3,ITLSGRP)
      DTLSDLTM(20,5) = LTMMAT(15,ITLSGRP)
      DTLSDLTM(20,6) = LTMMAT(14,ITLSGRP)
      DTLSDLTM(20,3) = LTMMAT(20,ITLSGRP)
      DTLSDLTM(20,15) = LTMMAT(5,ITLSGRP)
      DTLSDLTM(20,14) = LTMMAT(6,ITLSGRP)
      DTLSDLTM(20,20) = LTMMAT(3,ITLSGRP)

      RETURN
      END

      SUBROUTINE SHFTLTM2SHFTTLS

C---Derives SHFT_TLS from SHFT_LTM

C      IMPLICIT NONE

      INCLUDE 'anisoanl.fh'

      INTEGER ITLSGRP,IPAR,IPAR0,IPAR1
      REAL LTM(6,6),DELLTM(6,6),AMAT1(6,6),AMAT2(6,6),AMAT3(6,6)
      LOGICAL ERROR

      DO ITLSGRP = 1,NTLSGRP
        IPAR0   = 20*ITLSGRP-20
        IPAR1   = 21*ITLSGRP-21

C---Construct lower triangular matrix for this group
        CALL VEC21_TO_LTM66(LTMMAT(1,ITLSGRP),LTM)

C---Construct LTM shift matrix for this group
        CALL VEC21_TO_LTM66(SHFT_LTM(IPAR1+1),DELLTM)

        CALL MAT2MATT(6,6,LTM,DELLTM,AMAT1,ERROR)
        CALL MAT2MATT(6,6,DELLTM,LTM,AMAT2,ERROR)
        CALL MAT2MATT(6,6,DELLTM,DELLTM,AMAT3,ERROR)

C---TMAT shifts
        SHFT_TLS(IPAR0+1) = AMAT1(1,1) + AMAT2(1,1) + AMAT3(1,1)
        SHFT_TLS(IPAR0+2) = AMAT1(2,2) + AMAT2(2,2) + AMAT3(2,2)
        SHFT_TLS(IPAR0+3) = AMAT1(3,3) + AMAT2(3,3) + AMAT3(3,3)
        SHFT_TLS(IPAR0+4) = AMAT1(1,2) + AMAT2(1,2) + AMAT3(1,2)
        SHFT_TLS(IPAR0+5) = AMAT1(1,3) + AMAT2(1,3) + AMAT3(1,3)
        SHFT_TLS(IPAR0+6) = AMAT1(2,3) + AMAT2(2,3) + AMAT3(2,3)

C---LMAT shifts
        SHFT_TLS(IPAR0+7) = AMAT1(4,4) + AMAT2(4,4) + AMAT3(4,4)
        SHFT_TLS(IPAR0+8) = AMAT1(5,5) + AMAT2(5,5) + AMAT3(5,5)
        SHFT_TLS(IPAR0+9) = AMAT1(6,6) + AMAT2(6,6) + AMAT3(6,6)
        SHFT_TLS(IPAR0+10) = AMAT1(4,5) + AMAT2(4,5) + AMAT3(4,5)
        SHFT_TLS(IPAR0+11) = AMAT1(4,6) + AMAT2(4,6) + AMAT3(4,6)
        SHFT_TLS(IPAR0+12) = AMAT1(5,6) + AMAT2(5,6) + AMAT3(5,6)

C---SMAT shifts
        SHFT_TLS(IPAR0+13) = AMAT1(2,5) + AMAT2(2,5) + AMAT3(2,5) -
     +                       AMAT1(1,4) - AMAT2(1,4) - AMAT3(1,4)
        SHFT_TLS(IPAR0+14) = AMAT1(1,4) + AMAT2(1,4) + AMAT3(1,4) -
     +                       AMAT1(3,6) - AMAT2(3,6) - AMAT3(3,6)
        SHFT_TLS(IPAR0+15) = AMAT1(1,5) + AMAT2(1,5) + AMAT3(1,5)
        SHFT_TLS(IPAR0+16) = AMAT1(1,6) + AMAT2(1,6) + AMAT3(1,6)
        SHFT_TLS(IPAR0+17) = AMAT1(2,6) + AMAT2(2,6) + AMAT3(2,6)
        SHFT_TLS(IPAR0+18) = AMAT1(2,4) + AMAT2(2,4) + AMAT3(2,4)
        SHFT_TLS(IPAR0+19) = AMAT1(3,4) + AMAT2(3,4) + AMAT3(3,4)
        SHFT_TLS(IPAR0+20) = AMAT1(3,5) + AMAT2(3,5) + AMAT3(3,5)

C---Apply shifts to LTMMAT
        DO IPAR = 1,21
          LTMMAT(IPAR,ITLSGRP) = LTMMAT(IPAR,ITLSGRP) + 
     +                         SHFT_LTM(IPAR1+IPAR)
        ENDDO
      ENDDO

      RETURN
      END
C
      SUBROUTINE MAT2MATT(NP,N,AMAT1,AMAT2,AMAT_OUT,ERROR)
C
C---Multiples AMAT1 to transpose of AMAT2 and writes results
C---to AMAT_OUT
C
C-----Input and output parameters
C
C---NP       - maximal size
C---N        -  current size
C---AMAT1    - Input left matrix
C---AMAT2    - Input right matrix
C---AMAT_OUT - resultant output matrix
C---ERROR    - simple error checks
C--------------------------------------------------------------------
      REAL AMAT1(NP,NP),AMAT2(NP,NP),AMAT_OUT(NP,NP)
      LOGICAL ERROR
C
      ERROR = .FALSE.
      IF(N.LE.0 .OR. N.GT.NP) THEN
        ERROR = .TRUE.
        RETURN
      ENDIF
      DO   I1=1,N
        DO   I2=1,N
          AMAT_OUT(I1,I2) = 0.0
          DO    I3=1,N
            AMAT_OUT(I1,I2) =
     +          AMAT_OUT(I1,I2) + AMAT1(I1,I3)*AMAT2(I2,I3)
          ENDDO
        ENDDO
      ENDDO
      RETURN
      END

      SUBROUTINE VEC21_TO_LTM66(VEC21,MAT66)

C---Convert vector of 21 elements to lower triangular 6 x 6 matrix
C   This constitutes definition of order of 21.

      IMPLICIT NONE

      INTEGER I,J
      REAL VEC21(21),MAT66(6,6)

      DO J = 2,6
        DO I = 1,J-1
          MAT66(I,J) = 0.0
        ENDDO
      ENDDO

      MAT66(1,1) = VEC21(1)
      MAT66(2,1) = VEC21(4)
      MAT66(2,2) = VEC21(2)
      MAT66(3,1) = VEC21(5)
      MAT66(3,2) = VEC21(6)
      MAT66(3,3) = VEC21(3)
      MAT66(4,1) = VEC21(13)
      MAT66(4,2) = VEC21(18)
      MAT66(4,3) = VEC21(19)
      MAT66(4,4) = VEC21(7)
      MAT66(5,1) = VEC21(15)
      MAT66(5,2) = VEC21(14)
      MAT66(5,3) = VEC21(20)
      MAT66(5,4) = VEC21(10)
      MAT66(5,5) = VEC21(8)
      MAT66(6,1) = VEC21(16)
      MAT66(6,2) = VEC21(17)
      MAT66(6,3) = VEC21(21)
      MAT66(6,4) = VEC21(11)
      MAT66(6,5) = VEC21(12)
      MAT66(6,6) = VEC21(9)

      RETURN
      END
c
C----linalgebra_block
C Subroutines and functions to deal with problems of linear algebra
C
C  Strange enough it begins with singular value decomposition.
C  But I need it now.
C
      SUBROUTINE SVDCMP(A,M,N,MP,NP,W,V)

C----This subroutine makes singular value decomposition
C----as   A = U.W.VT
C----See Press, Flannery, Teukolsky, Vetterling  Numerical recipes
C----M and N are physical sizes of matrix MP and NP are logical
C---sizes. M >= N. Otherwise routine will fail
C---Note: Output is VT but not V
C---A will be destroyed and replaced by U
C
      PARAMETER (NMAX = 1000)
      REAL A(MP,NP),W(NP),V(NP,NP),RV1(NMAX)
C
      IF(M.LT.N) CALL CCPERR(1,'SVDMP: M Should be > N')
      G     = 0.0
      SCALE = 0.0
      ANORM = 0.0
      L     = 2
      NM    = 1
      DO   I=1,N
        L      = I+1
        RV1(I) = SCALE*G
        G      = 0.0
        S      = 0.0
        SCALE  = 0.0
        IF(I.LE.M) THEN
          DO   K=I,M
            SCALE = SCALE+ABS(A(K,I))
          ENDDO
          IF(SCALE.NE.0.0) THEN
            DO    K=I,M
              A(K,I) = A(K,I)/SCALE
              S      = S + A(K,I)*A(K,I)
            ENDDO
            F = A(I,I)
            G = -SIGN(SQRT(S),F)
            H = F*G-S
            A(I,I) = F-G 
            DO   J=L,N
              S = 0.0
              DO   K=I,M
                S = S+A(K,I)*A(K,J)
              ENDDO
              F = S/H
              DO    K=I,M
                A(K,J) =A(K,J)+F*A(K,I)
              ENDDO
            ENDDO
            DO   K=I,M
              A(K,I) = SCALE*A(K,I)
            ENDDO
          ENDIF
        ENDIF
        W(I)  = SCALE*G
        G     = 0.0
        S     = 0.0
        SCALE = 0.0
        IF((I.LE.M).AND.(I.NE.N)) THEN
          DO   K=L,N
            SCALE = SCALE+ABS(A(I,K))
          ENDDO
          IF(SCALE.NE.0.0) THEN
            DO   K=L,N
              A(I,K) = A(I,K)/SCALE
              S      = S + A(I,K)*A(I,K)
            ENDDO
            F = A(I,L)
            G = -SIGN(SQRT(S),F)
            H = F*G-S
            A(I,L) = F-G
            DO   K=L,N
              RV1(K) = A(I,K)/H
            ENDDO  
            DO   J=L,M
              S = 0.0
              DO   K=L,N
                S = S+A(J,K)*A(I,K)
              ENDDO
              DO   K=L,N
                A(J,K) = A(J,K)+S*RV1(K)
              ENDDO
            ENDDO
            DO    K=L,N
              A(I,K) = SCALE*A(I,K)
            ENDDO
          ENDIF
        ENDIF
        ANORM = MAX(ANORM,(ABS(W(I))+ABS(RV1(I))))
      ENDDO
C
C Accumulation of right hand transformation
C
      DO   I=N,1,-1
        IF(I.LT.N) THEN
          IF(G.NE.0.0) THEN
            DO   J=L,N
C
C   Double division to avoid underflow (????
C
              V(J,I) = (A(I,J)/A(I,L))/G   
            ENDDO
            DO   J=L,N
              S = 0.0
              DO   K=L,N
                S = S+A(I,K)*V(K,J)
              ENDDO
              DO   K=L,N
                V(K,J) = V(K,J)+S*V(K,I)
              ENDDO
            ENDDO
          ENDIF
          DO   J=L,N
            V(I,J) = 0.0
            V(J,I) = 0.0
          ENDDO
        ENDIF
        V(I,I) = 1.0
        G      = RV1(I)
        L      = I
      ENDDO
C
C Accumulation of left hand transformation
C
      DO    I=min(m,N),1,-1 
        L = I+1
        G = W(I)
        DO  J=L,N
          A(I,J) = 0.0
        ENDDO 
        IF(G.NE.0.0) THEN
          G = 1.0/G
          DO   J=L,N
            S = 0.0
            DO   K=L,M
              S=S+A(K,I)*A(K,J)
            ENDDO
            F = (S/A(I,I))*G
            DO    K=I,M
              A(K,J) = A(K,J)+F*A(K,I)
            ENDDO
          ENDDO 
          DO    J=I,M
            A(J,I) = A(J,I)*G
          ENDDO
        ELSE
          DO    J=I,M
            A(J,I) = 0.0
          ENDDO
        ENDIF
        A(I,I) = A(I,I)+1.0
      ENDDO
C
C  Diagonalisation of bidiagonal form
C
      DO   K=N,1,-1
C
C  Loop over singular values
C
        DO   ITS=1,30
C
C Loop over iteration. 
C
          DO    L=K,1,-1
C
C  Test for splitting
C
            NM = L-1
            IF((ABS(RV1(L))+ANORM).EQ.ANORM) GOTO 2
            IF((ABS(W(NM))+ANORM).EQ.ANORM) GOTO 1
          ENDDO
 1        C = 0.0
          S = 1.0
          DO   I=L,K
            F = S*RV1(I)
            rv1(i)=c*rv1(i)
            IF((ABS(F)+ANORM).eq.ANORM) goto 2
            G = W(I)
            h = pythag(f,g)
            W(I) = H
            H = 1.0/H
            C = G*H
            S = -F*H
            DO    J=1,M
              Y = A(J,NM)
              Z = A(J,I)
              A(J,NM) = (Y*C)+(Z*S)
              A(J,I)  = -(Y*S)+(Z*C)
            ENDDO
          ENDDO
 2        Z = W(K)
          IF(L.EQ.K) THEN
            IF(Z.LT.0.0) THEN
              W(K) = -Z
              DO   J=1,N
                V(J,K) = -V(J,K)
              ENDDO
            ENDIF
            GOTO 3
          ENDIF
          IF(ITS.EQ.30) CALL CCPERR(2,
     +         'SVDCMP: No convergence in 30 iterations')
          X = W(L)
          NM= K-1
          Y = W(NM)
          G = RV1(NM)
          H = RV1(K)
          F = ((Y-Z)*(Y+Z)+(G-H)*(G+H))/(2.0*H*Y)
          g = pythag(f,1.0)
          F = ((X-Z)*(X+Z)+H*((Y/(F+SIGN(G,F)))-H))/X
C
C  Next QR transformation
C
          C = 1.0
          S = 1.0
          DO   J=L,NM
            I = J+1
            G = RV1(I)
            Y = W(I)
            H = S*G
            G = C*G
            z = pythag(f,h)
            RV1(J) = Z
            C = F/Z
            S = H/Z
            F =  X*C+G*S
            G = -X*S+G*C
            H = Y*S
            Y = Y*C
            DO   JJ=1,N
              X = V(JJ,J)
              Z = V(JJ,I)
              V(JJ,J) =  (X*C)+(Z*S)
              V(JJ,I) = -(X*S)+(Z*C)
            ENDDO
            z = pythag(f,h)
            W(J) = Z
            IF(Z.NE.0.0) THEN
              Z = 1.0/Z
              C = F*Z
              S = H*Z
            ENDIF
            F =  C*G+S*Y
            X = -S*G+C*Y
            DO   JJ=1,M
              Y = A(JJ,J)
              Z = A(JJ,I)
              A(JJ,J) =   (Y*C)+(Z*S)
              A(JJ,I) =  -(Y*S)+(Z*C)
            ENDDO
          ENDDO
          RV1(L) = 0.0
          RV1(K) = F
          W(K)   = X
        ENDDO
 3      CONTINUE
      ENDDO
      RETURN
      END      
C
      function pythag(a,b)
      real a,b,pythag
      real absa, absb
      absa = abs(a)
      absb = abs(b)
      if(absa.gt.absb) then
        pythag = absa*sqrt(1.+(absb/absa)**2)
      else
        if(absb.eq.0.0) then
          pythag = 0.0
        else
          pythag = absb*sqrt(1.+(absa/absb)**2)
        endif
      endif
      return
      end 
C
      SUBROUTINE SVBKSB(U,W,V,M,N,MP,NP,B,X)
C
C  Solves AX = B. A should be decomposed by SVDCMP subroutine
C  A = U.W.VT
      PARAMETER (NMAX = 1000)
      REAL U(MP,NP),W(NP),V(NP,NP),B(NP),X(NP),TMP(NMAX)
C
C Calculate Ut.B
      DO   J=1,N
        S = 0.0
        IF(W(J).NE.0.0) THEN
          DO   I=1,M
            S = S+U(I,J)*B(I)
          ENDDO
          S = S/W(J)
        ENDIF
        TMP(J) = S
      ENDDO
C
C Multiply by V to get answer
      DO   J=1,N
        S = 0.0
        DO   JJ=1,N
          S = S+V(J,JJ)*TMP(JJ)
        ENDDO
        X(J) = S
      ENDDO
      RETURN
      END
C
      SUBROUTINE SVSOLV(A,M,N,MP,NP,B,X,CVM)
C
C   Solve Ax=B by calling singular value decomposition 
C   subroutines

C---Needed for TOL
      INCLUDE 'anisoanl.fh'

      PARAMETER (NMAX = 1000)
      REAL A(MP,NP),B(NP),X(NP),CVM(NP,NP),WV(NMAX+NMAX*NMAX)
C
      DO I = 1, NP
        WV(I)      = 0.
      END DO
      DO I = 1, NP*NP
        WV(NMAX+I) = 0.
      END DO
C---First call SVDCMP to decompose A = U.W.Vt
      CALL SVDCMP(A,M,N,MP,NP,WV(1),WV(NMAX+1))
C
C  Now edit W-s
      WMAX = 0.0
      DO    I=1,N
        IF(WMAX.LT.WV(I)) WMAX = WV(I)
      ENDDO
      WMIN = WMAX*TOL
      DO   I=1,N
        IF(WV(I).LT.WMIN) WV(I) = 0.0
      ENDDO
C
C--Now call SVBKSB to solve equation
      CALL SVBKSB(A,WV(1),WV(NMAX+1),M,N,MP,NP,B,X)

C--Now call SVDVAR to get covariance matrix
      CALL SVDVAR(WV(NMAX+1),NP,WV(1),CVM,NP)

      RETURN
      END
C
      SUBROUTINE SVDVAR(V,MA,W,CVM,NCVM)
C
C---Evaluation of covariance matrix after using singular
C---value decomposition. Parameters with singular contribution
C---to matrix will have 0.0 variance.
      PARAMETER (NMAX = 1000)
      REAL V(MA,MA),W(MA),CVM(NCVM,NCVM),WTI(NMAX)
C
      DO   I=1,MA
        WTI(I) = 0.0
        IF(W(I).NE.0.0) WTI(I) = 1.0/(W(I)*W(I))
      ENDDO
      DO   I=1,MA
        DO   J=1,I
          SUM = 0.0
          DO    K=1,MA
            SUM = SUM + V(I,K)*V(J,K)*WTI(K)
          ENDDO
          CVM(I,J) = SUM
          CVM(J,I) = SUM
        ENDDO
      ENDDO
      RETURN
      END

C**************************************************************************
C
C  PS.F    - PostScript subroutines
C
C            Written by Roman Laskowski, University College, London,
C            November 1993.
C
C            Original version was part of v.3.0 of the PROCHECK suite
C            of programs. (Previously the routines were incorporated in
C            pplot.f).
C
C            Subsequent amendments will be labelled by CHECK v.m.n-->
C            and CHECK v.m.n<-- where m.n is the version number
C            corresponding to the change
C
C  v.3.0.1 - Change of plot-file names. All plot-file names now have the
C            form: <filename>_nn.ps  where <filename> is the name of the
C            input data file, and nn is a sequential number 01, 02, ...
C                                             Roman Laskowski (29 Mar 1994)
C
C  v.4.0   - Modification for allowing the call from SFCHECK, and various
C            improvements:
C            Use of latin1_encoding, hijacked from a PostScript file
C            generated by genscript by Markku Rossi <mtr@iki.fi>
C            <http://iki.fi/~mtr/>.
C            Creation of multipage PostScript files.
C            Right justified text; use of the Courier and Symbol fonts.
C                                             Jean Richelle (Oct 1996)
C                                             Alexei Vagin (Dec 1996)
C  v.4.0.1 - Add new routines, DrawSect, ArrowHead, OrthoBox.
C            Other small changes
c                                             Jean Richelle (20 Mar 1997)
C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE PSSETPG  -  Set page specifications
C
C----------------------------------------------------------------------+---

      SUBROUTINE PSSETPG(TOTALNBPAGES)

      INCLUDE 'anisoanl.fh'

      INTEGER TOTALNBPAGES

      TOTPAG = TOTALNBPAGES
      CURPAG = 1

      RETURN
      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE PSOPEN  -  Open PostScript file and write out header records
C
C----------------------------------------------------------------------+---

      SUBROUTINE PSOPEN(BBOXX1,BBOXX2,BBOXY1,BBOXY2,MXCOLR,RGB,
     -    INCOL,BAKCOL)

      INCLUDE 'anisoanl.fh'

      CHARACTER*2    COLNO
      INTEGER        BAKCOL, BBOXX1, BBOXX2, BBOXY1, BBOXY2, I, ICOL,
     -               MXCOLR,IFAIL
      LOGICAL        INCOL
      REAL           RGB(3,MXCOLR)

C---- Open output file
      IFAIL=0
      CALL CCPDPN(UNPS,'PSOUT','UNKNOWN','F',0,IFAIL)
      IF(IFAIL.NE.0) THEN
        CALL CCPERR(1,' ERROR: can''t open PSOUT')
        RETURN
      ENDIF

C---- Write out headings to PostScript file
      WRITE(UNPS,10) TOTPAG
 10   FORMAT(
     -    '%!PS-Adobe-3.0',/,
     -    '%%Creator: SfCheck',/,
     -'%%DocumentNeededResources: font Times-Roman Symbol Times-Bold',
     -    ' Courier',/,
     -    '%%BoundingBox: (atend)',/,
     -    '%%Pages:',I5,/,
     -    '%%EndComments',/,
     -    '%%BeginProlog')

      CALL PSRESOURCE (UNPS)

      WRITE(UNPS,20)
 20   FORMAT(
     -    '/L { moveto lineto stroke } bind def',/,
     -    '/Col { setrgbcolor } bind def')

C---- Loop to define the colours
      IF (INCOL) THEN
          DO 100, ICOL = 1, MXCOLR
              WRITE(COLNO,40) ICOL
 40           FORMAT(I2)
              IF (COLNO(1:1).EQ.' ') COLNO(1:1) = '0'
              WRITE(UNPS,60) COLNO, (RGB(I,ICOL), I = 1, 3)
 60           FORMAT('/Col',A2,' {gsave ',3F8.4,' setrgbcolor } def')
 100      CONTINUE
      ENDIF

      WRITE(UNPS,120)
 120  FORMAT(
c     -    '/Poly3 { moveto lineto lineto fill grestore } bind ',
     -    '/Poly3 { moveto lineto lineto fill } bind ',
     -    'def',/,
c     -    '/Pl3 { 6 copy Poly3 moveto moveto moveto closepath ',
c     -    'stroke } bind def',/,
     -    '/Pl3 { gsave newpath 6 copy Poly3 moveto moveto moveto ',
     -    'closepath stroke grestore } bind def',/,
c     -    '/Pline3 { 6 copy Poly3 moveto lineto lineto closepa',
c     -    'th stroke } bind def')
     -    '/Pline3 { gsave newpath 6 copy Poly3 moveto lineto lineto ',
     -    'closepath stroke grestore } bind def')
      WRITE(UNPS,250)
 250  FORMAT(
     -    '/Poly4 { moveto lineto lineto lineto fill grestore } bind ',
     -    'def',/,
     -    '/Pl4 { 8 copy Poly4 moveto moveto moveto moveto closepath ',
     -    'stroke } bind def',/,
     -    '/Pline4 { 8 copy Poly4 moveto lineto lineto lineto closepa',
     -    'th stroke } bind def',/,
     -    '/Circle { gsave newpath 0.0 setgray 3 copy 0 360 arc fill ',
     -    'stroke grestore }',/
     -    'bind def',/,
     -    '/Circleu { gsave newpath 3 copy 0 360 arc ',
     -    'stroke grestore }',/
     -    'bind def')
      WRITE(UNPS,302)
 302  FORMAT(
     -     '/DrawSect',/,
     -     '  { /grayshade exch def',/,
     -     '    /endangle exch def',/,
     -     '    /startangle exch def',/,
     -     '    /radhigh exch def',/,
     -     '    /radlow exch def',/,
     -     '    /cy exch def',/,
     -     '    /cx exch def',/,
     -     '    newpath cx cy radlow startangle endangle arc',/,
     -     '    cx cy radhigh endangle startangle arcn closepath',/,
     -     '    1.415 setmiterlimit',/,
     -     '    gsave grayshade setgray fill grestore',/,
     -     '    stroke',/,
     -     '  } def')
      WRITE(UNPS,312)
 312  FORMAT(
     -     '/DrawUSect',/,
     -     '  { /grayshade exch def',/,
     -     '    /endangle exch def',/,
     -     '    /startangle exch def',/,
     -     '    /radhigh exch def',/,
     -     '    /radlow exch def',/,
     -     '    /cy exch def',/,
     -     '    /cx exch def',/,
     -     '    newpath cx cy radlow startangle endangle arc',/,
     -     '    cx cy radhigh endangle startangle arcn closepath',/,
     -     '    1.415 setmiterlimit',/,
     -     '    gsave grayshade setgray fill grestore',/,
     -     '  } def')
      WRITE(UNPS,303)
 303  FORMAT(
     -     '/DrawBadSect',/,
     -     '  { /endangle exch def',/,
     -     '    /startangle exch def',/,
     -     '    /radhigh exch def',/,
     -     '    /radlow exch def',/,
     -     '    /cy exch def',/,
     -     '    /cx exch def',/,
     -     '    newpath cx cy radlow startangle endangle arc',/,
     -     '    cx cy radhigh startangle endangle arc closepath',/,
     -     '    1.415 setmiterlimit',/,
     -     '    stroke',/,
     -     '  } def')
      WRITE(UNPS,304)
 304  FORMAT(
     -     '/OrthoBox',/,
     -     '  { /grayshade exch def',/,
     -     '    /YHigh exch def',/,
     -     '    /YLow exch def',/,
     -     '    /XRight exch def',/,
     -     '    /XLeft exch def',/,
     -     '    newpath XLeft YHigh moveto XRight YHigh lineto',/,
     -     '    XRight YLow lineto XLeft YLow lineto closepath',/,
     -     '    gsave grayshade setgray fill grestore',/,
     -     '    stroke',/,
     -     '  } def')
      WRITE(UNPS,305)
 305  FORMAT(
     -     '/ArrowHead',/,
     -     '  { /WidthHalf exch def',/,
     -     '    /Length exch def',/,
     -     '    /Angle exch def',/,
     -     '    /YBase exch def',/,
     -     '    /XBase exch def',/,
     -     '    gsave XBase YBase translate Angle rotate',/,
     -     '    newpath 0 0 moveto 0 WidthHalf neg lineto',/,
     -     '    Length 0 lineto 0 WidthHalf lineto 0 0 lineto',/,
     -     '    closepath 0 setgray fill grestore',/,
     -     '  } def')
      WRITE(UNPS,301)
 301  FORMAT(
     -    '/RightJ {',/,
     -    '  dup /Times-Roman-Extd findfont exch scalefont setfont',/,
     -    '  exch stringwidth pop -1 mul exch pop 0 rmoveto',/,
     -    ' } bind def',/,
     -    '/CCenterRot90 {',/,
     -    '  dup /Courier findfont exch scalefont setfont',/,
     -    '  exch stringwidth pop -2 div exch 3 div exch rmoveto',/,
     -    ' } bind def',/,
     -    '/Rot45 { gsave currentpoint translate 45 rotate } bind def',
     -    /,'/CPrint { /Courier findfont exch scalefont setfont show',
     -    ' } bind def',/)
      WRITE(UNPS,300)
 300  FORMAT(
     - '/Print { /Times-Roman-Extd findfont exch scalefont setfont',
     -    ' show } bind def',/,
     -    '/Bprint { /Times-Bold findfont exch scalefont setfont show',
     -    ' } bind def',/,
     -    '/Gprint { /Symbol findfont exch scalefont setfont show }',
     -    ' bind def',/,
     -    '/Center {',/,
     -    '  dup /Times-Roman-Extd findfont exch scalefont setfont',/,
     -    '  exch stringwidth pop -2 div exch -3 div rmoveto',/,
     -    ' } bind def',/,
     -    '/CenterRot90 {',/,
     -    '  dup /Times-Roman-Extd findfont exch scalefont setfont',/,
     -    '  exch stringwidth pop -2 div exch 3 div exch rmoveto',/,
     -    ' } bind def',/,
     -    '/UncenterRot90 {',/,
     -    '  dup /Times-Roman-Extd findfont exch scalefont setfont',/,
     -    '  exch stringwidth } bind def',/,
     -    '/Rot90 { gsave currentpoint translate 90 rotate } bind def'
     -    ,/,'%%EndProlog')
      WRITE(UNPS,400) CURPAG, TOTPAG
 400  FORMAT(
     -    '%%BeginSetup',/,
     -    '1 setlinecap 1 setlinejoin 1 setlinewidth 0 setgray [ ] 0 ',
     -    'setdash newpath',/,
     -    '/Times-Roman /Times-Roman-Extd encode_vector ReEncodeSmall'
     -    ,/,'%%EndSetup',/,
     -    '%%Page:',I5,I5)
      WRITE(UNPS,500) REAL(BBOXX1), REAL(BBOXY1), REAL(BBOXX2),
     -     REAL(BBOXY1), REAL(BBOXX2), REAL(BBOXY2), REAL(BBOXX1),
     -     REAL(BBOXY2)
 500  FORMAT(
     -    '/SfCheckSave save def',/,
     -    2F7.2,' moveto',2(2F7.2,' lineto'),/,
     -    2F7.2,' lineto closepath gsave',/,
     -    'gsave 1.0000 setgray fill grestore',/,
     -    'clip newpath')

C---- Draw in the background colour
      IF (INCOL) THEN
          CALL PSHADE(0.0,BAKCOL,RGB,MXCOLR,INCOL)
          CALL PSUBOX(REAL(BBOXX1), REAL(BBOXY1), REAL(BBOXX2),
     -        REAL(BBOXY1), REAL(BBOXX2), REAL(BBOXY2), REAL(BBOXX1),
     -        REAL(BBOXY2))
      ENDIF

      RETURN
      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE PSRESOURCE  -  Write resource to PostScript file
C
C----------------------------------------------------------------------+---

      SUBROUTINE PSRESOURCE(UN)

      INTEGER       UN

      CHARACTER LINE*80
      INTEGER   I,IERR,IUN

      IERR=0
      IUN = 52
      CALL CCPDPN(IUN,'PSRESOURCE','READONLY','F',0,IERR)
      IF(IERR.NE.0) THEN
        CALL CCPERR(1,' ERROR: can''t open ps.resource')
        RETURN
      ENDIF
C---- Read in the resource and write to requested unit
      DO I = 1, 104
         READ(IUN,'(A)',END=900,ERR=900) LINE
         WRITE(UN,'(A)') LINE(1:LENSTR(LINE))
      ENDDO
      GOTO 999

 900  CONTINUE
      CALL CCPERR(1,'ERROR: Problem with file ps.resource.')
      IERR=1
 999  CONTINUE

      CLOSE(IUN)

      RETURN
      END
C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE PSCLOS  -  Write final lines to PostScript file and close
C
C----------------------------------------------------------------------+---

      SUBROUTINE PSCLOS(BBOXX1,BBOXX2,BBOXY1,BBOXY2)

      INCLUDE 'anisoanl.fh'

      INTEGER       BBOXX1, BBOXX2, BBOXY1, BBOXY2

C---- Write out closing lines to PostScript file
      WRITE(UNPS,100) BBOXX1 - 1, BBOXY1 - 1, BBOXX2 + 1, BBOXY2 + 1
 100  FORMAT(
     -    'grestore stroke',/,
     -    'SfCheckSave restore',/,
     -    'showpage',/,
     -    '%%Trailer',/,
     -    '%%BoundingBox:',4I4,/,
     -    '%%EOF')


C---- Close the file
      CLOSE(UNPS)

      RETURN
      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE PSCTXT  -  Write out centred text to PostScript file
C
C----------------------------------------------------------------------+---

      SUBROUTINE PSCTXT(X,Y,SIZE,TEXT)

      INCLUDE 'anisoanl.fh'

      CHARACTER*(*) TEXT
      REAL          SIZE, X, Y

      WRITE(UNPS,100) X, Y
 100  FORMAT(2F7.2,' moveto')
      SZ=SIZE
      IF(SZ.LT.0.0.OR.SZ.GT.99.9) SZ=0.0
      WRITE(UNPS,200) TEXT(1:LENSTR(TEXT)), SZ
 200  FORMAT('(',A,')',/,F4.1,' Center')
      WRITE(UNPS,300) TEXT(1:LENSTR(TEXT)), SZ
 300  FORMAT('(',A,')',/,F4.1,' Print')
      RETURN
      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE PSLINE  -  Write line out to PostScript file
C
C----------------------------------------------------------------------+---

      SUBROUTINE PSLINE(X1,Y1,X2,Y2)

      INCLUDE 'anisoanl.fh'

      REAL          X1, X2, Y1, Y2

      IF(ABS(X1).GT.999.0) X1=0.0
      IF(ABS(X2).GT.999.0) X2=0.0
      IF(ABS(Y1).GT.999.0) Y1=0.0
      IF(ABS(Y2).GT.999.0) Y2=0.0

      WRITE(UNPS,100) X1, Y1, X2, Y2
 100  FORMAT(4F7.2,' L')

      RETURN
      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE PSLWID  -  Write line-width out to PostScript file
C
C----------------------------------------------------------------------+---

      SUBROUTINE PSLWID(LWIDTH)

      INCLUDE 'anisoanl.fh'

      REAL          LWIDTH

      WRITE(UNPS,100) LWIDTH
 100  FORMAT(F5.2,' setlinewidth')

      RETURN
      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE PSRTXT  -  Write out text rotated through 90 degrees to
C                        PostScript file
C
C----------------------------------------------------------------------+---

      SUBROUTINE PSRTXT(X,Y,SIZE,TEXT)

      INCLUDE 'anisoanl.fh'

      CHARACTER*(*) TEXT
      REAL          SIZE, X, Y

      WRITE(UNPS,100) X, Y
 100  FORMAT(2F7.2,' moveto')
      WRITE(UNPS,200) TEXT(1:LENSTR(TEXT)), SIZE
 200  FORMAT('(',A,') ',F4.1,' UncenterRot90 Rot90')
      WRITE(UNPS,300) TEXT(1:LENSTR(TEXT)), SIZE
 300  FORMAT('(',A,') ',F4.1,' Print')
      WRITE(UNPS,400)
 400  FORMAT('grestore')

      RETURN
      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE PSHADE  -  Write shading level out to PostScript file
C
C----------------------------------------------------------------------+---

      SUBROUTINE PSHADE(SHADE,COLOUR,RGB,MXCOLR,INCOL)

      INCLUDE 'anisoanl.fh'

      CHARACTER*2   COLNO
      INTEGER       COLOUR, MXCOLR
      LOGICAL       INCOL
      REAL          RGB(3,MXCOLR), SHADE

      IF (.NOT.INCOL) THEN
          WRITE(UNPS,100) SHADE
 100      FORMAT('gsave',F7.4,' setgray')
      ELSE
          WRITE(COLNO,140) COLOUR
 140      FORMAT(I2.0)
          IF (COLNO(1:1).EQ.' ') COLNO(1:1) = '0'
          WRITE(UNPS,200) COLNO
 200      FORMAT('Col',A2)
      ENDIF

      RETURN
      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE PSTEXT  -  Write out text to PostScript file
C
C----------------------------------------------------------------------+---

      SUBROUTINE PSTEXT(X,Y,SIZE,TEXT)

      INCLUDE 'anisoanl.fh'

      CHARACTER*(*) TEXT
      REAL          SIZE, X, Y

      T1=X
      T2=Y - SIZE / 4.0
      IF(ABS(T1).GT.999.0) T1=1.0
      IF(ABS(T2).GT.999.0) T2=1.0
      WRITE(UNPS,100) T1,T2 
 100  FORMAT(2F7.2,' moveto')
      SZ=SIZE
      IF(SZ.LT.0.5.OR.SZ.GT.99.9) SZ=1.0
      WRITE(UNPS,300) TEXT(1:LENSTR(TEXT)), SZ
 300  FORMAT('(',A,')',/,F4.1,' Print')
      RETURN
      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE PSUBOX  -  Write out unbounded box to PostScript file
C
C----------------------------------------------------------------------+---

      SUBROUTINE PSUBOX(X1,Y1,X2,Y2,X3,Y3,X4,Y4)

      INCLUDE 'anisoanl.fh'

      REAL          X1, X2, Y1, Y2, X3, Y3, X4, Y4


      IF(ABS(X1).GT.999.0) X1=0.0
      IF(ABS(X2).GT.999.0) X2=0.0
      IF(ABS(Y1).GT.999.0) Y1=0.0
      IF(ABS(Y2).GT.999.0) Y2=0.0

      IF(ABS(X3).GT.999.0) X3=0.0
      IF(ABS(X4).GT.999.0) X4=0.0
      IF(ABS(Y3).GT.999.0) Y3=0.0
      IF(ABS(Y4).GT.999.0) Y4=0.0

      WRITE(UNPS,100) X1, Y1, X2, Y2, X3, Y3, X4, Y4
 100  FORMAT(8F7.2,' Pl4')

      RETURN
      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE AXES  -  Draw graph axes and axis-labels, writing out to
C                      PostScript file
C
C----------------------------------------------------------------------+---

      SUBROUTINE AXES(XLIM1,XLIM2,YLIM1,YLIM2,NPTSX,NPTSY,XMIN,XMAX,
     -    YMIN,YMAX,SIZLAB,INTRLX,INTRLY,TSIZE,NUMBEX,NUMBEY,ZEROS,
     -    NOENDX,NOENDY,FULTIC)

      CHARACTER*5    LABEL
      INTEGER        IFROM, IMARK, INTRLX, INTRLY, IVALUE, NPTSX, NPTSY
      LOGICAL        FULTIC, NOENDX, NOENDY, NUMBEX, NUMBEY, ZEROS
      REAL           SIZLAB, TSIZE, UGAP, VALUE, VGAP, X, XCENTR, XGAP,
     -               XLIM1, XLIM2, XMAX, XMIN, X1, X2, Y, YCENTR, YGAP,
     -               YLIM1, YLIM2, YMAX, YMIN, Y1, Y2

C Routine parameters:-
C ------------------
C XLIM1, XLIM2, YLIM1, YLIM2 - x- and y-extent of graph area (ie coords
C          of graph-box). In PostScript coords.
C NPTSX, NPTSY - Numbers of ticks along x- and y-axes
C XMIN, XMAX, YMIN, YMAX - Minimum and maximum x- and y-values on plotted
C          axes
C SIZLAB - Label-size for numbers on the axis (in PostScript coords).
C INTRLX, INTRLY - Integer values indicating how the numbers on the axes
C          are to be formatted. (O=format as integer,1=format as real with
C          1 number after decimal point,2=real with 2 numbers after decimal
C          point.
C TSIZE  - A measure of the distance of axis labels from the axis (in
C          PostScript coords).
C NUMBEX, NUMBEY - Flags indicating whether numbers are actually required
C          on the x- and y-axes
C ZEROS  - Flag indicating whether zero value on the y-axis is required
C NOEND  - Flag indicating whether the last numbers on the x- and y-axes
C          are required
C FULTIC - Flag indicating whether axis ticks are to be full-ticks,
C          extending a little to the other side of the axis, or half-ticks
C          only.

C---- Initialise variables
      IF (NPTSX.GT.0) THEN
          XGAP = (XLIM2 - XLIM1) / NPTSX
          UGAP = (XMAX - XMIN) / NPTSX
      ENDIF
      IF (NPTSY.GT.0) THEN
          YGAP = (YLIM2 - YLIM1) / NPTSY
          VGAP = (YMAX - YMIN) / NPTSY
      ENDIF
      XCENTR = (XLIM1 + XLIM2) / 2.0
      YCENTR = (YLIM1 + YLIM2) / 2.0

C---- Draw box round graph
      CALL PSLWID(0.6)
      CALL PSLINE(XLIM1,YLIM1,XLIM1,YLIM2)
      CALL PSLINE(XLIM1,YLIM2,XLIM2,YLIM2)
      CALL PSLINE(XLIM2,YLIM2,XLIM2,YLIM1)
      CALL PSLINE(XLIM2,YLIM1,XLIM1,YLIM1)

C---- x-axis point markers and labels
      X = XLIM1
      Y = YLIM1 - TSIZE / 2.0
      IF (NPTSX.GT.0) THEN
          DO 100, IMARK = 1, NPTSX + 1
              VALUE = XMIN + (IMARK - 1) * UGAP
              IF (VALUE.GE.0.0) THEN
                  IVALUE = VALUE + 0.5
              ELSE
                  IVALUE = VALUE - 0.5
              ENDIF
              IF (INTRLX.EQ.0) THEN
                  WRITE(LABEL,20) IVALUE
 20               FORMAT(I5)
              ELSE IF (INTRLX.EQ.1) THEN
                  WRITE(LABEL,40) VALUE
 40               FORMAT(F5.1)
              ELSE IF (INTRLX.EQ.2) THEN
                  WRITE(LABEL,60) VALUE
 60               FORMAT(F5.2)
              ENDIF
              IFROM = 1
              IF (LABEL(1:1).EQ.' ') IFROM = 2
              IF (LABEL(2:2).EQ.' ') IFROM = 3
              IF (LABEL(3:3).EQ.' ') IFROM = 4
              IF (LABEL(4:4).EQ.' ') IFROM = 5
              IF (NUMBEX) THEN
                  IF (.NOT.(NOENDX .AND. IMARK.EQ.NPTSX + 1)) THEN
                      Y1 = YLIM1
                      Y2 = YLIM1 - SIZLAB / 3.0
                      IF (FULTIC) Y1 = Y1 + SIZLAB / 6.0
                      CALL PSCTXT(X,Y,SIZLAB,LABEL(IFROM:5))
                      CALL PSLINE(X,Y1,X,Y2)
                  ENDIF
              ENDIF
              X = X + XGAP
 100      CONTINUE
      ENDIF

C---- y-axis point markers and labels
      X = XLIM1 - TSIZE
      Y = YLIM1
      IF (NPTSY.GT.0) THEN
          DO 200, IMARK = 1, NPTSY + 1
              VALUE = YMIN + (IMARK - 1) * VGAP
              IF (VALUE.GE.0.0) THEN
                  IVALUE = VALUE + 0.5
              ELSE
                  IVALUE = VALUE - 0.5
              ENDIF
              IF (INTRLY.EQ.0) THEN
                  WRITE(LABEL,20) IVALUE
              ELSE IF (INTRLY.EQ.1) THEN
                  WRITE(LABEL,40) VALUE
              ELSE
                  WRITE(LABEL,60) VALUE
              ENDIF
              IFROM = 1
              IF (NUMBEY .AND. (ZEROS .OR. IMARK.GT.1)) THEN
                  IF (.NOT.(NOENDY .AND. IMARK.EQ.NPTSY + 1)) THEN
                      X1 = XLIM1 - SIZLAB / 3.0
                      X2 = XLIM1
                      IF (FULTIC) X2 = X2 + SIZLAB / 6.0
                      CALL PSCTXT(X,Y,SIZLAB,LABEL(1:5))
                      CALL PSLINE(X1,Y,X2,Y)
                  ENDIF
              ENDIF
              Y = Y + YGAP
 200      CONTINUE
      ENDIF

      RETURN
      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C      SUBROUTINE PSHBOX - Draw a shaded bounded rectangle parallel to
C                          the x and y direction of the page
C
C----------------------------------------------------------------------+---

      SUBROUTINE PSHBOX (XL,XR,YL,YH,SHADE)

      INCLUDE 'anisoanl.fh'

      REAL XL,XR,YL,YH,SHADE

      WRITE(UNPS,100) XL, XR, YL, YH, SHADE
 100  FORMAT(5F7.2,' OrthoBox')

      RETURN

      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C      SUBROUTINE PSTRANSLATE - Translate user coordinates to position
C                               x and y of the page
C
C----------------------------------------------------------------------+---

      SUBROUTINE PSTRANSLATE (X,Y)

      INCLUDE 'anisoanl.fh'

      REAL X,Y

      WRITE(UNPS,100) X, Y
 100  FORMAT(2F10.2,' translate')

      RETURN

      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C      SUBROUTINE PSFUNIT - Define the value Unit to be used
C
C----------------------------------------------------------------------+---

      SUBROUTINE PSFUNIT (UN)

      INCLUDE 'anisoanl.fh'

      INTEGER UN

      UNPS = UN

      RETURN

      END
