/*
  Copyright (c) Navraj Pannu       1998 - 2007
  Copyright (c) Leiden University  2002 - 2007

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef AFROLIKELIHOOD_H
#define AFROLIKELIHOOD_H	1

#include "cmtzlib.h"
#include "crystal.h"
#include "likelihood.h"
#include "misc.h"
#include "model.h"
#include "tab.h"

class Afrolikelihood : public Likelihood
{
 public:
    Afrolikelihood(Model &, Crystal &);
    virtual ~Afrolikelihood(){;}
    void   setup(const bool = true);
    void   checkdata();
    void   finitedifftest();
    void   writescript() const;
    double calcanomcorrel() const;
    void   outputeas();
    void   outputevalues();
    virtual void print(const int = -1) const { mdl.print(); }

    // minimizer routines
    double grad(const bool check = false)
    {double func = sadsignalgradient(check); calcsigmah(); return func;}

    double function()
    {return sadsignalgradient(); }

    virtual double gradient(const vector<double> &pars, vector<double> &grd)
    { double func(grad()); grd = Getgradient(); return func;;}

    virtual double function1dim (const double stepsize, const vector<double> &pars,
                                 const vector<double> &direction)
    {shift(stepsize,pars,direction); calcsigmah(); return grad();}

    virtual double gradient1dim (double &dfunc, vector<double> &grd,
                                 const double stepsize,
                                 const vector<double> &pars,
                                 const vector<double> &direction)
    { shift(stepsize,pars,direction); calcsigmah();
      double func(grad()); grd = Getgradient();
      dfunc = dot_prod(grd,direction); return func;;}
    // set functions for parser
    Afrolikelihood &Setascii(const bool input){ascii = input; return *this;}
    Afrolikelihood &Setsubhires(const double input){
      if ((input < 0.0) || (input > 50.0))
	Bp3Error("Afrolikelihood::Setsubhires","Resolution cutoff is irreasonable");
      subhires = input; return *this;}
 private:
    vector<double> fa, sigfa, sigmafa, sigmfa;
    vector<double> ssfpp, ssfpfpp, dLdssfpfpp, dLdssfpp;
    Tab<double> tab;
    double subhires;
    unsigned cd;
    bool anodiff, ascii;
    // laguerre nodes and weights
    vector<double> lnode, lweight;
    void calcsigmah();
    void calccorrel();
    double giacovazzo(const bool check=false)
    {Bp3Error("Afrolikelihood::giacovazzo","not in this version"); return 0.0;}
    double sadfunction(const bool check=false)
    {Bp3Error("Afrolikelihood::sadfunction","not in this version"); return 0.0;}
    double sadsignalgradient(const bool check=false)
    {Bp3Error("Afrolikelihood::sadsignalgradient","not in this version"); return 0.0;}
    void resize();
    void Setresocutoff();
    void calcanodiff();
    void calcdeltafa();
    void downweighoutliers();
    void calcsigmafa();
    void renormalize();
    void writedrear();
    void writemtz();
};
#endif  
