/*
  Copyright (c) Navraj Pannu       1998 - 2005
  Copyright (c) Leiden University  2002 - 2005

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "afroinput.h"
#include "afrolikelihood.h"
#include "crystal.h"
#include "minimizer.h"
#include "misc.h"
#include "model.h"

// local functions
void Banner(int argc, char** argv);
void GetInput(Model &, Crystal &, Afrolikelihood &, Minimizer &);

int main(int argc, char** argv)
{
  Banner(argc,argv);

  // initialize classes for our parameters and data
  Model mdl;
  Crystal xtal;
  Afrolikelihood likelihood(mdl, xtal);
  Minimizer minimizer(likelihood);

  GetInput(mdl, xtal, likelihood, minimizer);

  likelihood.setup();

  if ( (likelihood.Getmode()    == "DIFFE") || (likelihood.Getmode() == "GIAC") )
  {
   printf("Refinement cycle\n\n");
   minimizer.Setiterations(100);
   minimizer.refine();
   
    printf("Outputting difference evalues\n\n");
    likelihood.outputeas();
  }
  else if (likelihood.Getmode() == "DELTA")
  {
    printf("Outputting delta evalues\n\n");
    likelihood.outputeas();
  }
  else if (likelihood.Getmode() == "EVALUES")
  {
    printf("Outputting evalues\n\n");
    likelihood.outputevalues();
  }
  else if (likelihood.Getmode() == "INTENSITY")
  {
    printf("Outputting intensities\n\n");
    likelihood.outputintensities();
  }
  else
    Bp3Error("main", "unknown mode of operation");

  Termination();
}

void Banner(int argc, char **argv)
{
  char version[]  = "0.85";
  CCP4Banner(argc,argv,version);
  printf("                           Afro\n");
  printf("  Multivariate likelihood substructure amplitude estimation\n");
  printf("                  Developmental Version\n");
  printf("                    19 February 2007\n\n");
  printf("        http:://www.bfsc.leidenuniv.nl/software/afro\n\n");


}
void GetInput(Model &mdl, Crystal &xtal,
	      Afrolikelihood &like, Minimizer &minimizer)
{
  // gets input and stores it in data structures
  Afroinput input(mdl, xtal, like, minimizer);
}
