/*
  Copyright (c) Navraj Pannu       1998 - 2007
  Copyright (c) Leiden University  2002 - 2007

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef GCXLIKELIHOOD_H
#define GCXLIKELIHOOD_H	1

#include "misc.h"
#include "cmtzlib.h"
#include "crystal.h"
#include "likelihood.h"
#include "matrix.h"
#include "model.h"

class Gcxlikelihood : public Likelihood
{
 public:
  Gcxlikelihood(Model &, Crystal &);
  virtual ~Gcxlikelihood(){;}
  void   setup();

  unsigned Getstepsize() const
  { return step.size(); }

  string Getstep(const unsigned i) const
  { if (i >= step.size())
      Bp3Error("Gcxlikelihood::Getstep", "out of bound");
    return step[i];
  }
    
  // set functions for parser
  Gcxlikelihood &Setdiskspace(const string input)
  { diskspace = input; return *this; }

  Gcxlikelihood &Setstep(const string input)
  { step.push_back(input); return *this; }

  Gcxlikelihood &Setrefltype(const string input)
  { refltype  = input; return *this; }

  Gcxlikelihood &Setfirststep(const string input)
  { firststep = input; return *this; }

  Gcxlikelihood &Setlaststep(const string input)
  { laststep  = input; return *this; }

  Gcxlikelihood &Setsolvent(const double input)
  { if ( (input    > 0.9) || (input < 0.1) )
      Bp3Error("Gcxlikelihood::Setsolvent", "solvent must be < 0.9 and > 0.1");
    solvent        = input; return *this;
  }

  Gcxlikelihood &Setsadonly(const bool input)
  { sadonly        = input; return *this; }

  Gcxlikelihood &Setnresidues(const unsigned input)
  { nresidues      = input; return *this; }

  Gcxlikelihood &Setnnucleotides(const unsigned input) 
  { nnucleotides   = input; return *this; }

  Gcxlikelihood &Setweight(const double input) 
  { weight         = input; return *this; }

  Gcxlikelihood &Setnmonomers(const unsigned input) 
  { nmonomers      = input; return *this; }

  Gcxlikelihood &Setoverallbfactor(const double input)
  { if (input      < ZERO)
      Bp3Error("Gcxlikelihood::Setoverallbfactor", "bfactor must be positive");
    overallbfactor = input; return *this;
  }

  Gcxlikelihood &Setautoreso(const bool input)
  { 
    autoreso       = input; return *this;
  }

  Gcxlikelihood &Setsubhirescut(const double input)
  { if (input      < ZERO)
      Bp3Error("Gcxlikelihood::Setsubhirescut", "subhirescut must be non-negative");
    subhirescut    = input; return *this;
  }

  Gcxlikelihood &Setrescalef(const unsigned input)
  { 
    rescalef       = input; return *this;
  }

  Gcxlikelihood &Setafrosigf(const double input)
  { if (input      < ZERO)
      Bp3Error("Gcxlikelihood::Setafrosigf", "sigma-cutoff must be non-negative");
    afrosigf       = input; return *this;
  }

  Gcxlikelihood &Setafrosano(const double input)
  { if (input      < ZERO)
      Bp3Error("Gcxlikelihood::Setafrosano", "sigma-cutoff must be non-negative");
    afrosano       = input; return *this;
  }

  Gcxlikelihood &Setafrosiso(const double input)
  { if (input      < ZERO)
      Bp3Error("Gcxlikelihood::Setafrosiso", "sigma-cutoff must be non-negative");
    afrosiso       = input; return *this;
  }

  Gcxlikelihood &Setcrunchptrials(const unsigned input)
  {
    crunchptrials  = input; return *this;
  }

  Gcxlikelihood &Setcrunchtrials(const unsigned input)
  { 
    crunchtrials   = input; return *this;
  }

  Gcxlikelihood &Setcrunchbp3(const unsigned input)
  { 
    crunchbp3      = input; return *this;
  }

  Gcxlikelihood &Setshelxdtrials(const unsigned input)
  { 
    shelxdtrials   = input; return *this;
  }

  Gcxlikelihood &Setcrunchthres(const double input)
  { if (input      < ZERO)
      Bp3Error("Gcxlikelihood::Setcrunchthres", "crunchthres must be non-negative");
    crunchthres    = input; return *this;
  }

  Gcxlikelihood &Setcrunchspec(const bool input)
  {  crunchspec    = input; return *this; }

  Gcxlikelihood &Setshelxdmind(const double input)
  { if (input      < 0)
      Bp3Error("Gcxlikelihood::Setshelxdmind", "shelxd mind must be non-negative");
    shelxdmind     = input; return *this;
  }

  Gcxlikelihood &Setshelxdmtrials(const unsigned input)
  { 
    shelxdmtrials   = input; return *this;
  }

  Gcxlikelihood &Setshelxdmdeq(const double input)
  {
    shelxdmdeq     = input; return *this;
  }

  Gcxlikelihood &Setshelxdspec(const bool input)
  {  shelxdspec    = input; return *this; }

  Gcxlikelihood &Setcrunchdev(const double input)
  { if (input      < ZERO)
      Bp3Error("Gcxlikelihood::Setcrunchdev", "crunchdev must be non-negative");
    crunchdev      = input; return *this;
  }
  
  Gcxlikelihood &Setshelxdthres(const double input)
  { if (input      < ZERO)
      Bp3Error("Gcxlikelihood::Setshelxdthres", "shelxdthres must be non-negative");
    shelxdthres    = input; return *this;
  }

  Gcxlikelihood &Setshelxddsul(const unsigned input)
  { 
    shelxddsul     = input; return *this;
  }

  Gcxlikelihood &Setbp3stopfom(const double input)
  { if (input      < ZERO)
      Bp3Error("Gcxlikelihood::Setbp3stopfom", "bp3stopfom must be non-negative");
    bp3stopfom     = input; return *this;
  }
  
  Gcxlikelihood &Setbp3phase(const string input)
  { if ( (input   == "FAST") || (input == "NORMAL") )
      bp3phase     = input;
    else
      Bp3Error("Gcxlikelihood::Setbp3phase", "bp3phase must be fast or normal");
    return *this;
  }

  Gcxlikelihood &Setrefmacsolvent(const bool input)
  { 
    refmacsolvent   = input; return *this;
  }

  Gcxlikelihood &Setrefmachand(const bool input)
  { 
    refmachand      = input; return *this;
  }

  Gcxlikelihood &Setsolomonocycles(const unsigned input)
  {  solomonocycles = input; return *this; }

  Gcxlikelihood &Setsolomonhcycles(const unsigned input)
  {  solomonhcycles = input; return *this; }

  Gcxlikelihood &Setsolomonncycles(const unsigned input)
  {  solomonncycles = input; return *this; }

  Gcxlikelihood &Setsolomonhand(const bool input)
  {  solomonhand    = input; return *this; }

  Gcxlikelihood &Setsolomonopt(const bool input)
  {  solomonopt     = input; return *this; }

  Gcxlikelihood &Setsolomondenmod(const bool input)
  {  solomondenmod  = input; return *this; }

  Gcxlikelihood &Setsolomonnomtz(const bool input)
  {  solomonnomtz   = input; return *this; }

  Gcxlikelihood &Setsolomonnomap(const bool input)
  {  solomonnomap   = input; return *this; }

  Gcxlikelihood &Setdmhand(const bool input)
  {  dmhand         = input; return *this; }

  Gcxlikelihood &Setdmopt(const bool input)
  {  dmopt          = input; return *this; }

  Gcxlikelihood &Setdmdenmod(const bool input)
  {  dmdenmod       = input; return *this; }

  Gcxlikelihood &Setdmhist(const bool input)
  {  dmhist         = input; return *this; }

  Gcxlikelihood &Setpiratedenmod(const bool input)
  {  piratedenmod   = input; return *this; }

  Gcxlikelihood &Setpiratencs(const bool input)
  {  piratencs      = input; return *this; }
  
  Gcxlikelihood &Setpiratehl(const bool input)
  {  piratehl       = input; return *this; }

  Gcxlikelihood &Setpiratencycles(const unsigned input)
  {  piratencycles  = input; return *this; }

  Gcxlikelihood &Setpiratewihl(const double input)
  {  piratewihl     = input; return *this; }

  Gcxlikelihood &Setbuccnofreer(const bool input)
  {  buccnofreer    = input; return *this; }

  Gcxlikelihood &Setbuccusepdb(const bool input)
  {  buccusepdb     = input; return *this; }

  Gcxlikelihood &Setbuccfragments(const unsigned input)
  {  buccfragments  = input; return *this; }

  Gcxlikelihood &Setbuccreli(const double input)
  {  buccreli       = input; return *this; }

  Gcxlikelihood &Setbucccycles(const unsigned input)
  {  bucccycles     = input; return *this; }
  
  Gcxlikelihood &Setbuccfast(const bool input)
  {  buccfast       = input; return *this; }
  
  Gcxlikelihood &Setresolvedmncs(const bool input)
  {  resolvedmncs   = input; return *this; }

  Gcxlikelihood &Setresolvembncs(const bool input)
  {  resolvembncs   = input; return *this; }

  Gcxlikelihood &Setresolvehl(const bool input)
  {  resolvehl      = input; return *this; }

  Gcxlikelihood &Setshelxeocycles(const unsigned input)
  {  shelxeocycles  = input; return *this; }

  Gcxlikelihood &Setshelxehcycles(const unsigned input)
  {  shelxehcycles  = input; return *this; }

  Gcxlikelihood &Setshelxencycles(const unsigned input)
  {  shelxencycles  = input; return *this; }
  
  Gcxlikelihood &Setshelxehand(const bool input)
  {  shelxehand     = input; return *this; }

  Gcxlikelihood &Setshelxeopt(const bool input)
  {  shelxeopt      = input; return *this; }

  Gcxlikelihood &Setfreelunch(const bool input)
  {  freelunch      = input; return *this; }

  Gcxlikelihood &Setfreelunchreso(const double input)
  {  freelunchreso  = input; return *this; }

  Gcxlikelihood &Setshelxedenmod(const bool input)
  {  shelxedenmod   = input; return *this; }

  Gcxlikelihood &Setarpcycles(const unsigned input)
  { 
    arpcycles       = input; return *this;
  }

  Gcxlikelihood &Setarpsmallcycles(const unsigned input)
  { 
    arpsmallcycles  = input; return *this;
  }

  Gcxlikelihood &Setarpbigcycles(const unsigned input)
  { 
    arpbigcycles    = input; return *this;
  }

  Gcxlikelihood &Setarpusesix(const bool input)
  { 
    arpusesix       = input; return *this;
  }

  Gcxlikelihood &Setarpusepdb(const bool input)
  { 
    arpusepdb       = input; return *this;
  }

  Gcxlikelihood &Setarpsideafter(const unsigned input)
  { 
    arpsideafter    = input; return *this;
  }

  Gcxlikelihood &Setrefmactarget(const string input)
  { 
    refmactarget    = input; return *this;
  }

  Gcxlikelihood &Setresolvetype(const string input)
  { if (!( (input  != "quick") || (input != "superquick") || (input != "thorough") ||
	   (input  != "aggressive") || (input != "conservative") ) )
      Bp3Error("Gcxlikelihood::Setresolvetype", "Build type must be QUICk SUPErquick THORough AGGRessive or CONServative");
    resolvetype     = input; return *this;
  }

  Gcxlikelihood &Setcheck(const bool input)
  {  check          = input; return *this; }

  Gcxlikelihood &Setnotrans(const bool input)
  {  notrans        = input; return *this; }

  Gcxlikelihood &Setfinalpdb(const string input)
  {  finalpdb       = input; return *this; }

  Gcxlikelihood &Setfinalsubstructure(const string input)
  {  finalsubstructure = input; return *this; }

  Gcxlikelihood &Setpipeline(const unsigned input);
  
  virtual void print(const int = -1) const {;}

  // minimizer routines
  virtual vector<double> Setpars() const
  {vector<double> dummy; return dummy;}

  virtual double Getstepmax(const vector<double> &) const
  { return ONE;}

  virtual double gradient(const vector<double> &pars, vector<double> &grd)
  { return ONE;}

  virtual double function1dim (const double stepsize, const vector<double> &pars,
			       const vector<double> &direction)
  { return ONE;}

  virtual double gradient1dim (double &dfunc, vector<double> &grd,
			       const double stepsize,
			       const vector<double> &pars,
			       const vector<double> &direction)
  { return ONE;}
    
 private:
  string version, diskspace;
  // string that has pipeline
  vector<string> step;
  string refltype, firststep, laststep;
  // tags for the columns
  string expcoltag, fcoltag, icoltag, hltag, phasetag, eatag, fatag, alphatag, freertag;
  string subtag, pdbtag, lastshelxdtag, lastbp3tag;
  // overall properties about the structure
  string sequence;
  double solvent, overallbfactor, weight;
  unsigned nresidues, nnucleotides, nmonomers;
  unsigned refd, refc, bp3refd;
  bool autoreso, sadonly;
  double subhires, subhirescut;

  // program specific variables
  // scaling
  unsigned rescalef;
  // fa values
  double afrosigf, afrosano, afrosiso;
  // phasing
  double bp3stopfom;
  bool bp3hand, refmachand, refmacsolvent;
  // substructure detection
  double crunchthres, crunchdev, shelxdthres, shelxdmind, shelxdmdeq;
  bool crunchspec, shelxdspec;
  unsigned crunchtrials, crunchptrials, crunchbp3;
  unsigned shelxdtrials, shelxddsul, shelxdmtrials;
  // phasing
  string bp3phase;
  // density modification
  bool solomonhand, solomonopt, solomondenmod;
  bool solomonnomtz, solomonnomap;
  unsigned solomonhcycles, solomonocycles, solomonncycles;
  unsigned shelxehcycles, shelxeocycles, shelxencycles;
  unsigned piratencycles, buccfragments, bucccycles;
  bool shelxehand, shelxeopt, shelxedenmod;
  bool dmhand, dmopt, dmdenmod, dmhist;
  bool piratehand, pirateopt, piratedenmod;
  bool piratencs, piratehl, buccnofreer, buccfast, buccusepdb;
  bool resolvedmncs, resolvembncs, resolvehl;
  bool shelxeinphase, freelunch;
  double piratewihl, buccreli;
  double freelunchreso;
  double dmhandmargin, solomonhandmargin;
  // model building
  bool arpusesix, arpusepdb;
  unsigned arpsmallcycles, arpbigcycles, arpcycles, arpsideafter;
  string refmactarget;
  string resolvetype;
  // model comparison
  bool check, notrans;
  string finalpdb, finalsubstructure;
  
  FILE *xmlfile;

  void parseseqin(const char *);
  
  void writexml(); 
  void writeparameters();
  void writeinputfile();
  void writeoutputfile();
  void writemtzin();
  void writemtzout();
  void writeprep(const unsigned);
  void writesetup(const unsigned);
  void writeafro(const unsigned);
  void writecrunch2(const unsigned);
  void writebp3(const unsigned);
  void writerefmac(const unsigned);
  void writesolomon(const unsigned);
  void writedm(const unsigned);
  void writepirate(const unsigned);
  void writebuccaneer(const unsigned);
  void writeresolvedm(const unsigned);
  void writeresolvebuild(const unsigned);
  void writearpwarp(const unsigned);
  void writecheck(const unsigned);
  void checkshelx();
  void writeshelxc(const unsigned);
  void writeshelxd(const unsigned);
  void writeshelxe(const unsigned);
  void writeexpcol(const string = "BOTH") const;
  void writefcol() const;
  void writeicol() const;
  void writeeacol() const;
  void writefacol() const;
  void writephasecol() const;
  void writehlcol() const;  
};

#endif
