/*
  Copyright (c) Navraj Pannu       1998 - 2005
  Copyright (c) Leiden University  2002 - 2005

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <math.h>
#include "bp3likelihood.h"

double Bp3likelihood::sadfunction(const bool check)
{
  // Calculates likelihood for a sad data set (d=0) assuming correlated errors.

  likelihood                  = ZERO;

  unsigned flagged(0), d(cd);

  if (check)
    xtal.Setselected("REFINE");

  Matrix recovmodel(2), recovinvmodel(2);

  vector<double> detmodel(xtal.sf[d].nbins, ONE);

  for (unsigned s             = 0; s < xtal.sf[d].nbins; s++)
  {
    // covariance matrix for the model

    const double model[ ]     = {xtal.sf[d].sigmah[s],
				 xtal.sf[d].sigmah[s]-sumfpp[d][d][s],
				 xtal.sf[d].sigmah[s]-sumfpp[d][d][s],
				 xtal.sf[d].sigmah[s]};
    covmodel[s].assign(model);
    inverse2by2(covmodel[s], covinvmodel[s], detmodel[s]);
  }

  for (unsigned r             = 0; r < xtal.maxselref; r++)
  {
    unsigned counts(0);

    if (xtal.sf[d].use(r))
    { 
      counts++;
      if (!xtal.centric[r]   && xtal.sf[d].anouse(r))
        counts++;
    }
    
    if (counts                > 1)
    { 
      double eps((double) xtal.epsilon[r]);
      
      unsigned sa(xtal.bin(d,r));
      
      double sfpp(sumfpp[d][d][sa]), sigman(xtal.sf[d].sigman[sa]);
      double sd(xtal.sf[d].sdluz[sa]);
      
      // epsilon correct model covariance
      const double model[ ]   = {covmodel[sa](0,0)*eps, covmodel[sa](0,1)*eps,
				 covmodel[sa](1,0)*eps, covmodel[sa](1,1)*eps};
      recovmodel.assign(model);
    
      const double invmodl[ ] = {covinvmodel[sa](0,0)/eps, covinvmodel[sa](0,1)/eps,
				 covinvmodel[sa](1,0)/eps, covinvmodel[sa](1,1)/eps};

      recovinvmodel.assign(invmodl);

      double det2(detmodel[sa]*eps*eps);

      const double cov[ ]     = {eps*sigman+xtal.sf[d].devp[r]*xtal.sf[d].devp[r],
                                 eps*(sigman-sfpp), sd*recovmodel(0,0),
                                 sd*recovmodel(0,1),
                                 eps*(sigman-sfpp),
                                 eps*sigman+xtal.sf[d].devm[r]*xtal.sf[d].devm[r],
                                 sd*recovmodel(0,1), sd*recovmodel(0,0),
                                 sd*recovmodel(0,0), sd*recovmodel(0,1),
                                 recovmodel(0,0), recovmodel(0,1),
                                 sd*recovmodel(0,1), sd*recovmodel(0,0),
                                 recovmodel(0,1), recovmodel(0,0)};

      recov.assign(cov);

      double det4(ONE);

      bool filter             = inverse(recov, recovinv,det4);

      double cosdiff          = tab.tcos(xtal.sf[d].pcalcp[r] - xtal.sf[d].pcalcm[r]);

      likelihood             += (recovinv(0,0)*xtal.sf[d].datap[r]*xtal.sf[d].datap[r]+
                                 recovinv(1,1)*xtal.sf[d].datam[r]*xtal.sf[d].datam[r]+
                                 (recovinv(2,2) - recovinvmodel(0,0))*
                                 xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcp[r] +
                                 (recovinv(3,3) - recovinvmodel(1,1))*
                                 xtal.sf[d].fcalcm[r]*xtal.sf[d].fcalcm[r]+
                                 TWO*xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcm[r]*
                                 (recovinv(2,3) - recovinvmodel(0,1))*cosdiff);

      double integral(ZERO), maxexpval(ZERO);

      if (!filter)
      {
      // precalculate arguments in summation/integration

        double cospcalcp      = tab.tcos(xtal.sf[d].pcalcp[r]);
        double sinpcalcp      = tab.tsin2(xtal.sf[d].pcalcp[r]);
        double cospcalcm      = tab.tcos(xtal.sf[d].pcalcm[r]);
        double sinpcalcm      = tab.tsin2(xtal.sf[d].pcalcm[r]);

        double coscoeff1      = (-TWO*xtal.sf[d].datam[r]*
                                 (xtal.sf[d].fcalcp[r]*cospcalcp*recovinv(1,2) +
                                  xtal.sf[d].fcalcm[r]*cospcalcm*recovinv(1,3)));

        double sincoeff1      = (-TWO*xtal.sf[d].datam[r]*
                                 (xtal.sf[d].fcalcp[r]*sinpcalcp*recovinv(1,2) +
                                  xtal.sf[d].fcalcm[r]*sinpcalcm*recovinv(1,3)));

        double bessarg        = (recovinv(0,1)*recovinv(0,1)*
                                 xtal.sf[d].datam[r]*xtal.sf[d].datam[r] +
                                 recovinv(0,2)*recovinv(0,2)*
                                 xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcp[r] +
                                 recovinv(0,3)*recovinv(0,3)*
                                 xtal.sf[d].fcalcm[r]*xtal.sf[d].fcalcm[r] +
                                 TWO*xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcm[r]*
                                 recovinv(0,2)*recovinv(0,3)*cosdiff);

        double temp1          = recovinv(0,1)*recovinv(0,2);

        double temp2          = recovinv(0,1)*recovinv(0,3);

        double coscoeff2      = (TWO*xtal.sf[d].datam[r]*
                                 (xtal.sf[d].fcalcp[r]*temp1*cospcalcp +
                                  xtal.sf[d].fcalcm[r]*temp2*cospcalcm));

        double sincoeff2      = (TWO*xtal.sf[d].datam[r]*
                                 (xtal.sf[d].fcalcp[r]*temp1*sinpcalcp +
                                  xtal.sf[d].fcalcm[r]*temp2*sinpcalcm));

	vector<double> exparg(sadweight.size(), ZERO);
        vector<double> besselarg(sadweight.size(), ZERO);

        for (unsigned i       = 0; i < sadweight.size(); i++)
        {
          exparg[i]           = coscoeff1*sadcos[i]+sincoeff1*sadsin[i];
          temp1               = std::max(coscoeff2*sadcos[i] +
                                         sincoeff2*sadsin[i] +
                                         bessarg, ZERO);
          besselarg[i]        = TWO*xtal.sf[d].datap[r]*sqrt(temp1);
          exparg[i]          += besselarg[i];
          maxexpval           = std::max(maxexpval, exparg[i]);
        }

        for (unsigned i       = 0; i < sadweight.size(); i++)
          integral           += (tab.texp(-maxexpval + exparg[i])*
                                 tab.ti0e(besselarg[i])*sadweight[i]);
      }

      if ( (integral          > DSMALL)  && (!filter) )
        likelihood           -= (log(det2*integral/det4) + maxexpval);
      else
      {
        likelihood           += 1000.0*(log(std::max(fabs(det4-0.001), ONE)) + 1.75);
        flagged++;
      }
    }
  }

  if (xtal.verbose            > 1)
    printf("LIKELIHOOD = %f\n", likelihood);

  if (flagged && check)
    printf("Number of reflections with low likelihood values: %u\n", flagged);

  return likelihood;
}

double Bp3likelihood::sadgradient(const bool check, const bool outputmtz)
{
  // Calculates likelihood for a sad data set (d=0) assuming correlated errors.
  likelihood                      = ZERO;

  unsigned flagged(0), d(cd), nstop(0);
  double fomreso((double)xtal.sf[d].hires);
  double stopfom(ZERO), meansdluz(ZERO), sdsdluz(ZERO);
  
  if (check)
    xtal.Setselected( (outputmtz ? "PHASE" : "REFINE" ) );
  
  CMtz::MTZ *MTZOUT(NULL),  *MTZIN(NULL);
  FILE *hfile(NULL);   

  if (outputmtz)
  {
    if (allin)
    {
      if (getenv("HKLIN")        != NULL)
	MTZIN                     = CMtz::MtzGet("HKLIN",0);
      else if (xtal.sf[0].mtzin.size())
	MTZIN                     = CMtz::MtzGet(xtal.sf[0].mtzin.c_str(),0);

      if (!MTZIN)
	Bp3Error("Bp3likelihood::sadgradient","HKLIN could not be opened for reading");
    }

    MTZOUT                        = CMtz::MtzMalloc(0,0);

    if (!MTZOUT)
      Bp3Error("Bp3likelihood::sadgradient","HKLOUT could not be opened for writing");
    
    setupmtz(MTZOUT,MTZIN);

    if (outputhcalc)
    {
      string name                 = output + "hfcalc" + ".fmt";
      hfile                       = fopen(name.c_str(),"w");
    }
    
  }
  
  unsigned inc                    = (allin) ? colin.size() : 3;

  vector<double> detmodel(xtal.sf[d].nbins, ONE);

  Matrix recovmodel(2), recovinvmodel(2);

  for (unsigned s                 = 0; s < xtal.sf[d].nbins; s++)
  {
    dLdsigmah[d][s]               = dLdsumfpp[d][s]   = ZERO;
    dLdsdluz[d][s]                = ZERO;
    afom[s]                       = cfom[s]           = ZERO;
    
    // covariance matrix for the model

    const double model[ ]         = {xtal.sf[d].sigmah[s],
				     xtal.sf[d].sigmah[s]-sumfpp[d][d][s],
				     xtal.sf[d].sigmah[s]-sumfpp[d][d][s],
				     xtal.sf[d].sigmah[s]};
    
    covmodel[s].assign(model);
    inverse2by2(covmodel[s], covinvmodel[s], detmodel[s]);
  }

  xtal.sf[d].initstat();

  for (unsigned r                 = 0; r < xtal.maxselref; r++)
  {
    // Default value for columns to be written out is MNF
    vector<float> fdata(10+inc, CCP4::ccp4_nan().f);

    if (outputmtz)
      storeinitialcolumns(MTZIN, fdata, inc, r);
    
    dLdAp[d][r]                   = dLdBp[d][r] = dLdAm[d][r] =  dLdBm[d][r] = ZERO;

    double eps((double) xtal.epsilon[r]);
    unsigned sa(xtal.bin(d,r));

    // epsilon correct model covariance
    const double tmodel[ ]        = {covmodel[sa](0,0)*eps, covmodel[sa](0,1)*eps,
				     covmodel[sa](1,0)*eps, covmodel[sa](1,1)*eps};

    recovmodel.assign(tmodel);
    
    const double tinvmodel[ ]     = {covinvmodel[sa](0,0)/eps, covinvmodel[sa](0,1)/eps,
				     covinvmodel[sa](1,0)/eps, covinvmodel[sa](1,1)/eps};

    recovinvmodel.assign(tinvmodel);

    double det2(detmodel[sa]*eps*eps);
    double sigmah(xtal.sf[d].sigmah[sa]), sfpp(sumfpp[d][d][sa]);
    double sigman(xtal.sf[d].sigman[sa]), sd(xtal.sf[d].sdluz[sa]);
    unsigned counts(0);

    if (xtal.sf[d].use(r))
    {
      counts++;
      if (xtal.sf[d].anouse(r) && (outputmtz || !xtal.centric[r]))
	counts++;
    }

    if (counts                    > 1)
    {
      // total covariance matrices
      const double cov[ ]         = {eps*sigman+xtal.sf[d].devp[r]*xtal.sf[d].devp[r],
				     eps*(sigman-sfpp), sd*recovmodel(0,0),
				     sd*recovmodel(0,1),
				     eps*(sigman-sfpp),
				     eps*sigman+xtal.sf[d].devm[r]*xtal.sf[d].devm[r],
				     sd*recovmodel(0,1), sd*recovmodel(0,0),
				     sd*recovmodel(0,0), sd*recovmodel(0,1),
				     recovmodel(0,0), recovmodel(0,1),
				     sd*recovmodel(0,1), sd*recovmodel(0,0),
				     recovmodel(0,1), recovmodel(0,0)}; 

      recov.assign(cov);

      double det4(ONE);
      
      bool filter                 = inverse(recov,recovinv,det4);

      /*
      if (fabs(recov(2,2))        > DSMALL)
	sd                        = xtal.sf[d].sdluz[sa] = recov(0,2)/recov(2,2);
      */

      // derivatives of the matrices

      // wrt sdluz

      const double rtmp1[ ]       = {ZERO, ZERO, -recovmodel(0,0), -recovmodel(0,1),
				     ZERO, ZERO, -recovmodel(0,1), -recovmodel(0,0),
				     -recovmodel(0,0), -recovmodel(0,1), ZERO, ZERO,
				     -recovmodel(0,1), -recovmodel(0,0), ZERO, ZERO};
      
      matrixprod(redAdsd,recovinv,rtmp1);

      const double rtmp2[ ]       = {ZERO, ZERO, -eps*sd, -eps*sd,
				     ZERO, ZERO, -eps*sd, -eps*sd,
				     -eps*sd, -eps*sd, -eps, -eps,
				     -eps*sd, -eps*sd, -eps, -eps};

      matrixprod(redAdsigh,recovinv,rtmp2);

      // wrt sumfpp
      const double rtmp3[ ]       = {ZERO, TWO*eps, ZERO, TWO*eps*sd,
				     TWO*eps, ZERO, TWO*eps*sd, ZERO,
				     ZERO, TWO*eps*sd, ZERO, TWO*eps,
				     TWO*eps*sd, ZERO, TWO*eps, ZERO};

      matrixprod(redAdsfpp,recovinv,rtmp3);

      double cosdiff              = tab.tcos(xtal.sf[d].pcalcp[r] - xtal.sf[d].pcalcm[r]);

      double sindiff              = tab.tsin2(xtal.sf[d].pcalcp[r] - xtal.sf[d].pcalcm[r]);
      
      likelihood                 += (recovinv(0,0)*xtal.sf[d].datap[r]*
				     xtal.sf[d].datap[r]+
				     recovinv(1,1)*xtal.sf[d].datam[r]*
				     xtal.sf[d].datam[r]+
				     (recovinv(2,2) - recovinvmodel(0,0))*
				     xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcp[r] +
				     (recovinv(3,3) - recovinvmodel(1,1))*
				     xtal.sf[d].fcalcm[r]*xtal.sf[d].fcalcm[r]+
				     TWO*xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcm[r]*
				     (recovinv(2,3) - recovinvmodel(0,1))*cosdiff);

      double cospcalcp            = tab.tcos(xtal.sf[d].pcalcp[r]);
      double sinpcalcp            = tab.tsin2(xtal.sf[d].pcalcp[r]);
      double cospcalcm            = tab.tcos(xtal.sf[d].pcalcm[r]);
      double sinpcalcm            = tab.tsin2(xtal.sf[d].pcalcm[r]);

      double dldfp                = (TWO*(xtal.sf[d].fcalcp[r]*
					  (recovinv(2,2)-recovinvmodel(0,0)) +
					  xtal.sf[d].fcalcm[r]*
					  (recovinv(2,3)-recovinvmodel(0,1))*
					  cosdiff));
      double dldfm                = (TWO*(xtal.sf[d].fcalcm[r]*
					  (recovinv(3,3)-recovinvmodel(1,1)) +
					  xtal.sf[d].fcalcp[r]*
					  (recovinv(2,3)-recovinvmodel(0,1))*
					  cosdiff));
      double dldpp                = (-TWO*xtal.sf[d].fcalcm[r]*
				     (recovinv(2,3)-recovinvmodel(0,1))*sindiff);
      double dldpm                = (TWO*xtal.sf[d].fcalcp[r]*
				     (recovinv(2,3)-recovinvmodel(0,1))*sindiff);
      
      dLdAp[d][r]                 = dldfp*cospcalcp - dldpp*sinpcalcp;
      dLdBp[d][r]                 = dldfp*sinpcalcp + dldpp*cospcalcp;
      dLdAm[d][r]                 = dldfm*cospcalcm - dldpm*sinpcalcm;
      dLdBm[d][r]                 = dldfm*sinpcalcm + dldpm*cospcalcm;
      
      dLdsdluz[d][sa]            += (redAdsd(0,0)*xtal.sf[d].datap[r]*
				     xtal.sf[d].datap[r]+
				     redAdsd(1,1)*xtal.sf[d].datam[r]*
				     xtal.sf[d].datam[r]+
				     redAdsd(2,2)*xtal.sf[d].fcalcp[r]*
				     xtal.sf[d].fcalcp[r]+
				     redAdsd(3,3)*xtal.sf[d].fcalcm[r]*
				     xtal.sf[d].fcalcm[r]+
				     TWO*xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcm[r]*
				     redAdsd(2,3)*cosdiff);	  

      double temp                 = recovinvmodel(0,0) + recovinvmodel(0,1);
      double temp1                = -eps*temp*temp;

      dLdsigmah[d][sa]           += (redAdsigh(0,0)*xtal.sf[d].datap[r]*
				     xtal.sf[d].datap[r]+
				     redAdsigh(1,1)*xtal.sf[d].datam[r]*
				     xtal.sf[d].datam[r]+
				     (redAdsigh(2,2) - temp1)*
				     xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcp[r]+
				     (redAdsigh(3,3) - temp1)*
				     xtal.sf[d].fcalcm[r]*xtal.sf[d].fcalcm[r]+
				     TWO*xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcm[r]*
				     (redAdsigh(2,3) - temp1)*cosdiff);
	
      temp1                       = TWO*eps*(recovinvmodel(0,0)*recovinvmodel(0,0) +
					     recovinvmodel(0,1)*recovinvmodel(0,1));
      double temp2                = FOUR*eps*recovinvmodel(0,0)*recovinvmodel(0,1);

      dLdsumfpp[d][sa]           += (redAdsfpp(0,0)*xtal.sf[d].datap[r]*
				     xtal.sf[d].datap[r]+
				     redAdsfpp(1,1)*xtal.sf[d].datam[r]*
				     xtal.sf[d].datam[r]+
				     (redAdsfpp(2,2) - temp2)*
				     xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcp[r]+
				     (redAdsfpp(3,3) - temp2)*
				     xtal.sf[d].fcalcm[r]*xtal.sf[d].fcalcm[r]+
				     TWO*xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcm[r]*
				     (redAdsfpp(2,3) - temp1)*cosdiff);

      double integral(ZERO), totcos(ZERO), totsin(ZERO); 
      double totlogcos(ZERO), totlogsin(ZERO), totlogcos2(ZERO), totlogsin2(ZERO);
      double dldfcalcp(ZERO), dldfcalcm(ZERO), dldpcalcp(ZERO), dldpcalcm(ZERO);
      double dldsd(ZERO), dldsigh(ZERO), dldsfpp(ZERO);
      double maxexpval(ZERO);

      if (!filter)
      {
	// precalculate arguments in summation/integration      
	double dcos1dfp           = -TWO*xtal.sf[d].datam[r]*cospcalcp*recovinv(1,2);
	double dcos1dfm           = -TWO*xtal.sf[d].datam[r]*cospcalcm*recovinv(1,3);
	double cos1               = (dcos1dfp*xtal.sf[d].fcalcp[r] +
				     dcos1dfm*xtal.sf[d].fcalcm[r]);
	double dcos1dpp           = (TWO*xtal.sf[d].datam[r]*
				     sinpcalcp*recovinv(1,2));
	double dcos1dpm           = (TWO*xtal.sf[d].datam[r]*
				     sinpcalcm*recovinv(1,3)); 
	double dcos1dsd           = (-TWO*xtal.sf[d].datam[r]*
				     (xtal.sf[d].fcalcp[r]*
				      cospcalcp*redAdsd(1,2) +
				      xtal.sf[d].fcalcm[r]*
				      cospcalcm*redAdsd(1,3)));
	double dcos1dsigh         = (-TWO*xtal.sf[d].datam[r]*
				     (xtal.sf[d].fcalcp[r]*
				      cospcalcp*redAdsigh(1,2) +
				      xtal.sf[d].fcalcm[r]*
				      cospcalcm*redAdsigh(1,3)));
	double dcos1dsfpp         = (-TWO*xtal.sf[d].datam[r]*
				     (xtal.sf[d].fcalcp[r]*
				      cospcalcp*redAdsfpp(1,2) +
				      xtal.sf[d].fcalcm[r]*
				      cospcalcm*redAdsfpp(1,3)));

	double dsin1dfp           = (-TWO*xtal.sf[d].datam[r]*
				     sinpcalcp*recovinv(1,2));
	double dsin1dfm           = (-TWO*xtal.sf[d].datam[r]*
				     sinpcalcm*recovinv(1,3));
	double sin1               = (dsin1dfp*xtal.sf[d].fcalcp[r] +
				     dsin1dfm*xtal.sf[d].fcalcm[r]);
	double dsin1dpp           = (-TWO*xtal.sf[d].datam[r]*
				     cospcalcp*recovinv(1,2));
	double dsin1dpm           = (-TWO*xtal.sf[d].datam[r]*
				     cospcalcm*recovinv(1,3));
	double dsin1dsd           = (-TWO*xtal.sf[d].datam[r]*
				     (xtal.sf[d].fcalcp[r]*
				      sinpcalcp*redAdsd(1,2) +
				      xtal.sf[d].fcalcm[r]*
				      sinpcalcm*redAdsd(1,3)));
	double dsin1dsigh         = (-TWO*xtal.sf[d].datam[r]*
				     (xtal.sf[d].fcalcp[r]*
				      sinpcalcp*redAdsigh(1,2) +
				      xtal.sf[d].fcalcm[r]*
				      sinpcalcm*redAdsigh(1,3)));
	double dsin1dsfpp         = (-TWO*xtal.sf[d].datam[r]*
				     (xtal.sf[d].fcalcp[r]*
				      sinpcalcp*redAdsfpp(1,2) +
				      xtal.sf[d].fcalcm[r]*
				      sinpcalcm*redAdsfpp(1,3)));

	temp                      = (recovinv(0,2)*recovinv(0,3)*cosdiff);
      
	double bessarg            = (recovinv(0,1)*recovinv(0,1)*
				     xtal.sf[d].datam[r]*xtal.sf[d].datam[r] +
				     recovinv(0,2)*recovinv(0,2)*
				     xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcp[r] +
				     recovinv(0,3)*recovinv(0,3)*
				     xtal.sf[d].fcalcm[r]*xtal.sf[d].fcalcm[r] +
				     TWO*xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcm[r]*temp);
      
	double dbessargdfp        = TWO*(xtal.sf[d].fcalcp[r]*
					 recovinv(0,2)*recovinv(0,2) +
					 xtal.sf[d].fcalcm[r]*temp);

	double dbessargdfm        = TWO*(xtal.sf[d].fcalcm[r]*
					 recovinv(0,3)*recovinv(0,3) +
					 xtal.sf[d].fcalcp[r]*temp);

	double dbessargdpp        = -TWO*(xtal.sf[d].fcalcm[r]*
					  recovinv(0,2)*recovinv(0,3)*sindiff);
      
	double dbessargdpm        = TWO*(xtal.sf[d].fcalcp[r]*
					 recovinv(0,2)*recovinv(0,3)*sindiff);
      
	double dbessargdsd        = TWO*(recovinv(0,1)*redAdsd(0,1)*
					 xtal.sf[d].datam[r]*xtal.sf[d].datam[r] +
					 recovinv(0,2)*redAdsd(0,2)*
					 xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcp[r] +
					 recovinv(0,3)*redAdsd(0,3)*
					 xtal.sf[d].fcalcm[r]*xtal.sf[d].fcalcm[r] +
					 xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcm[r]*
					 ((recovinv(0,2)*redAdsd(0,3) +
					   redAdsd(0,2)*recovinv(0,3))*cosdiff));

	double dbessargdsigh      = TWO*(recovinv(0,1)*redAdsigh(0,1)*
 					 xtal.sf[d].datam[r]*xtal.sf[d].datam[r] +
					 recovinv(0,2)*redAdsigh(0,2)*
					 xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcp[r] +
					 recovinv(0,3)*redAdsigh(0,3)*
					 xtal.sf[d].fcalcm[r]*xtal.sf[d].fcalcm[r] +
					 xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcm[r]*
					 ((recovinv(0,2)*redAdsigh(0,3) +
					   redAdsigh(0,2)*recovinv(0,3))*cosdiff));
      
	double dbessargdsfpp      = TWO*(recovinv(0,1)*redAdsfpp(0,1)*
					 xtal.sf[d].datam[r]*xtal.sf[d].datam[r] +
					 recovinv(0,2)*redAdsfpp(0,2)*
					 xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcp[r] +
					 recovinv(0,3)*redAdsfpp(0,3)*
					 xtal.sf[d].fcalcm[r]*xtal.sf[d].fcalcm[r] +
					 xtal.sf[d].fcalcp[r]*xtal.sf[d].fcalcm[r]*
					 ((recovinv(0,2)*redAdsfpp(0,3) +
					   redAdsfpp(0,2)*recovinv(0,3))*cosdiff));
      
	temp1                     = (TWO*xtal.sf[d].datam[r]*
				     recovinv(0,1)*recovinv(0,2));
	temp2                     = (TWO*xtal.sf[d].datam[r]*
				     recovinv(0,1)*recovinv(0,3));

	double dcos2dfp           = temp1*cospcalcp;
	double dcos2dfm           = temp2*cospcalcm;
	double cos2               = (dcos2dfp*xtal.sf[d].fcalcp[r] +
				     dcos2dfm*xtal.sf[d].fcalcm[r]);
	double dcos2dpp           = -temp1*sinpcalcp;
	double dcos2dpm           = -temp2*sinpcalcm;

	double dcos2dsd           = (TWO*xtal.sf[d].datam[r]*
				     (xtal.sf[d].fcalcp[r]*
				      ((recovinv(0,1)*redAdsd(0,2) +
					redAdsd(0,1)*recovinv(0,2))*cospcalcp) +
				      xtal.sf[d].fcalcm[r]*
				      ((recovinv(0,1)*redAdsd(0,3) +
					redAdsd(0,1)*recovinv(0,3))*cospcalcm)));
      
	double dcos2dsigh         = (TWO*xtal.sf[d].datam[r]*
				     (xtal.sf[d].fcalcp[r]*
				      ((recovinv(0,1)*redAdsigh(0,2) +
					redAdsigh(0,1)*recovinv(0,2))*cospcalcp) +
				      xtal.sf[d].fcalcm[r]*
				      ((recovinv(0,1)*redAdsigh(0,3) +
					redAdsigh(0,1)*recovinv(0,3))*cospcalcm)));

	double dcos2dsfpp         = (TWO*xtal.sf[d].datam[r]*
				     (xtal.sf[d].fcalcp[r]*
				      ((recovinv(0,1)*redAdsfpp(0,2) +
					redAdsfpp(0,1)*recovinv(0,2))*cospcalcp) +
				      xtal.sf[d].fcalcm[r]*
				      ((recovinv(0,1)*redAdsfpp(0,3) +
					redAdsfpp(0,1)*recovinv(0,3))*cospcalcm)));
      
	double dsin2dfp           = temp1*sinpcalcp;
	double dsin2dfm           = temp2*sinpcalcm;
	double sin2               = (dsin2dfp*xtal.sf[d].fcalcp[r] + 
				     dsin2dfm*xtal.sf[d].fcalcm[r]);
	double dsin2dpp           = temp1*cospcalcp;
	double dsin2dpm           = temp2*cospcalcm;

	double dsin2dsd           = (TWO*xtal.sf[d].datam[r]*
				     (xtal.sf[d].fcalcp[r]*
				      ((recovinv(0,1)*redAdsd(0,2) +
					redAdsd(0,1)*recovinv(0,2))*sinpcalcp) +
				      xtal.sf[d].fcalcm[r]*
				      ((recovinv(0,1)*redAdsd(0,3) +
					redAdsd(0,1)*recovinv(0,3))*sinpcalcm)));

	double dsin2dsigh         = (TWO*xtal.sf[d].datam[r]*
				     (xtal.sf[d].fcalcp[r]*
				      ((recovinv(0,1)*redAdsigh(0,2) +
					redAdsigh(0,1)*recovinv(0,2))*sinpcalcp) +
				      xtal.sf[d].fcalcm[r]*
				      ((recovinv(0,1)*redAdsigh(0,3) +
					redAdsigh(0,1)*recovinv(0,3))*sinpcalcm)));

	double dsin2dsfpp         = (TWO*xtal.sf[d].datam[r]*
				     (xtal.sf[d].fcalcp[r]*
				      ((recovinv(0,1)*redAdsfpp(0,2) +
					redAdsfpp(0,1)*recovinv(0,2))*sinpcalcp) +
				      xtal.sf[d].fcalcm[r]*
				      ((recovinv(0,1)*redAdsfpp(0,3) +
					redAdsfpp(0,1)*recovinv(0,3))*sinpcalcm)));

	
	// For stability in the numerical integral, calculate the maximum value
	// of the exponential.  And, since all the values are calculated, store
	// the arguments of exponential and the bessel functions in arrays.

	vector<double> exparg(sadweight.size(), ZERO);
	vector<double> besselarg(sadweight.size(), ZERO);
      
	for (unsigned i           = 0; i < sadweight.size(); i++)
	{ 
	  exparg[i]               = cos1*sadcos[i] + sin1*sadsin[i];
	  double tempbess         = sqrt(std::max(cos2*sadcos[i] +
				 		  sin2*sadsin[i] +
						  bessarg, ZERO));
	  besselarg[i]            = TWO*xtal.sf[d].datap[r]*tempbess;
	  exparg[i]              += besselarg[i];
	  maxexpval               = std::max(maxexpval, exparg[i]);
	}
	
	for (unsigned i           = 0; i < sadweight.size(); i++)
	{
	  double io               = tab.ti0e(besselarg[i]);
	  double simarg           = (TWO*xtal.sf[d].datap[r]*xtal.sf[d].datap[r]*
				     tab.tsimoverx(besselarg[i], io));
	  double prob             = tab.texp(-maxexpval + exparg[i])*io;
	  double wprob            = prob*sadweight[i];
	  integral               += wprob;
	  totcos                 += wprob*sadcos[i];
	  totsin                 += wprob*sadsin[i];

	  dldfcalcp              -= ((dbessargdfp + dcos2dfp*sadcos[i] +
			 	      dsin2dfp*sadsin[i])*simarg +
				     (dcos1dfp*sadcos[i] + dsin1dfp*sadsin[i]))*wprob;
	  dldfcalcm              -= ((dbessargdfm + dcos2dfm*sadcos[i] +
			 	      dsin2dfm*sadsin[i])*simarg +
				     (dcos1dfm*sadcos[i] + dsin1dfm*sadsin[i]))*wprob;
	  dldpcalcp              -= ((dbessargdpp +  dcos2dpp*sadcos[i] +
				      dsin2dpp*sadsin[i])*simarg +
				     (dcos1dpp*sadcos[i] + dsin1dpp*sadsin[i]))*wprob; 
	  dldpcalcm              -= ((dbessargdpm + dcos2dpm*sadcos[i] +
				      dsin2dpm*sadsin[i])*simarg +
				     (dcos1dpm*sadcos[i] + dsin1dpm*sadsin[i]))*wprob;
	  dldsd                  -= ((dbessargdsd + dcos2dsd*sadcos[i] +
			 	      dsin2dsd*sadsin[i])*simarg +
				     (dcos1dsd*sadcos[i] + dsin1dsd*sadsin[i]))*wprob;
	  dldsigh                -= ((dbessargdsigh + dcos2dsigh*sadcos[i] +
			 	      dsin2dsigh*sadsin[i])*simarg +
				     (dcos1dsigh*sadcos[i] + dsin1dsigh*sadsin[i]))*wprob;
	  dldsfpp                -= ((dbessargdsfpp + dcos2dsfpp*sadcos[i] +
				      dsin2dsfpp*sadsin[i])*simarg +
				     (dcos1dsfpp*sadcos[i] + dsin1dsfpp*sadsin[i]))*wprob;
	  if (prob                > SMALLESTD)
	  {
	    prob                  = log(prob)*sadweight[i];
	    totlogcos            += prob*sadcos[i];
	    totlogsin            += prob*sadsin[i];
	    totlogcos2           += prob*(sadcos[i]*sadcos[i] - sadsin[i]*sadsin[i]);
	    totlogsin2           += prob*TWO*sadcos[i]*sadsin[i];
	  }
	}
      }
      
      if ( (integral              > DSMALL) && !filter) 
      {
	likelihood               -= (log(det2*integral/det4) + maxexpval);

	dLdAp[d][r]              += (dldfcalcp*cospcalcp - 
				     dldpcalcp*sinpcalcp)/integral;
	dLdBp[d][r]              += (dldfcalcp*sinpcalcp + 
				     dldpcalcp*cospcalcp)/integral;
	dLdAm[d][r]              += (dldfcalcm*cospcalcm - 
				     dldpcalcm*sinpcalcm)/integral;
	dLdBm[d][r]              += (dldfcalcm*sinpcalcm + 
				     dldpcalcm*cospcalcm)/integral;

	dLdsdluz[d][sa]          += (dldsd/integral +
				     TWO*((recovmodel(0,0)*(recovinv(0,2) +
							    recovinv(1,3)) +
					   recovmodel(0,1)*(recovinv(0,3) +
							    recovinv(1,2)))));
	
	dLdsigmah[d][sa]         += (dldsigh/integral +
				     eps*((recovinv(0,2) + recovinv(0,3) +
	 				   recovinv(1,2) + recovinv(1,3))*TWO*sd +
					  recovinv(2,2) + TWO*recovinv(2,3) +
					  recovinv(3,3) -
					  (recovinvmodel(0,0)+TWO*recovinvmodel(0,1) +
					   recovinvmodel(1,1))));
	
	dLdsumfpp[d][sa]         += (dldsfpp/integral -
				     FOUR*eps*(recovinv(0,1) + recovinv(2,3) +
					       (recovinv(0,3) + recovinv(1,2))*sd -
					       recovinvmodel(0,1)));	
	
	double fom                = sqrt(totcos*totcos + totsin*totsin)/integral;

	if (xtal.Getres(d,r)     >= fomreso)
	{
	  stopfom                += fom;
	  meansdluz              += sd;
	  sdsdluz                += sd*sd;
	  nstop++;
	}

	double phib               = atan2(totsin, totcos);
	if (xtal.centric[r])
	  phib                    =  ((cos(phib - xtal.centricphase[r]) >= ZERO) ?
				      xtal.centricphase[r] : xtal.centricphase[r] + PI);
	  
	unsigned s                = xtal.bin(d,r);
	xtal.sf[d].sanonshl[s]++;

	if (xtal.centric[r])
	{
	  cfom[s]                += fom;
	  xtal.sf[d].cnshl[s]++;
	}
	else
	{
	  afom[s]                += fom;
	  xtal.sf[d].anshl[s]++;
	}
	  
	if (outputmtz)
	{
	  fdata[inc + 2]          = (float) (fom*xtal.sf[d].datap[r]);
	  fdata[inc + 3]          = (float) (phib/DEGREEtoRAD);
	  fdata[inc + 4]          = (float) fom;
	  fdata[inc + 5]          = (float) totlogcos;
	  fdata[inc + 6]          = (float) totlogsin;
	  fdata[inc + 7]          = (float) totlogcos2;
	  fdata[inc + 8]          = (float) totlogsin2;
	  fdata[inc + 9]          = (float) dldfcalcp/integral;
	}
      }
      else
      {
	double tmp(std::max(fabs(det4-0.001), ONE));
	likelihood                += 1000.0*(log(tmp) + 1.75);
        
	if (tmp                    > ONE)
	{
	  dLdsdluz[d][sa]         += 1000.0*det4*(TWO*((recovinv(0,2)+recovinv(1,3))*recov(2,2) +
                                                       (recovinv(0,3)+recovinv(1,2))*recov(2,3)))/(0.001-det4);
          dLdsigmah[d][sa]        += 1000.0*det4*eps*(TWO*sd*(recovinv(0,2)+recovinv(0,3)+recovinv(1,2)+
                                                             recovinv(1,3))+recovinv(2,2)+recovinv(3,3)+
                                                      TWO*recovinv(2,3))/(0.001-det4);
          dLdsumfpp[d][sa]        += 1000.0*det4*(-FOUR*eps*(recovinv(0,1)+sd*(recovinv(0,3)+recovinv(1,2))+
                                                             recovinv(2,3)))/(0.001-det4);
        }
	
	flagged++;
      }
    }
    else if (outputmtz && (xtal.sf[d].usem(r) || xtal.sf[d].usep(r)) )
    {
      double fom(CCP4::ccp4_nan().f), fobs, arg(ZERO);
      double var                  = std::max(eps*(sigman - sd*sd*sigmah), EPSILON);
	
      if (xtal.sf[d].usep(r))
      {
	fobs                      = xtal.sf[d].datap[r];
	var                      += xtal.sf[d].devp[r]*xtal.sf[d].devp[r];
	arg                       = TWO*fobs*sd*xtal.sf[d].fcalcp[r]/var;
      }
      else
      {
	fobs                      = xtal.sf[d].datam[r];
	var                      += xtal.sf[d].devm[r]*xtal.sf[d].devm[r];
	arg                       = TWO*fobs*sd*xtal.sf[d].fcalcm[r]/var;
      }
	
      double phib                 = xtal.sf[d].pcalcp[r];

      unsigned s                  = xtal.bin(d,r);

      xtal.sf[d].sanonshl[s]++;

      fom                         = tab.tsim(arg);

      afom[s]                    += fom;
      xtal.sf[d].anshl[s]++;
 
      fdata[inc + 2]              = (float) fom*fobs;
      fdata[inc + 3]              = (float) (phib/DEGREEtoRAD);
      fdata[inc + 4]              = (float) fom;
      double temp                 = atanh(fom);
      fdata[inc + 5]              = (float) temp*tab.tcos(phib);
      fdata[inc + 6]              = (float) temp*tab.tsin2(phib);
      fdata[inc + 7]              = (float) ZERO;
      fdata[inc + 8]              = (float) ZERO;
      fdata[inc + 9]              = (float) ZERO;
    }
    
    if (outputmtz)
    {      
      CMtz::ccp4_lwrefl(MTZOUT, &fdata[0], &colout[0], fdata.size(), r+1);
      if (outputhcalc && (counts > 1))
      {
	unsigned sa(xtal.bin(d,r)), sb(sa);
	double wa(ONE), wb(ZERO);
	double sig               = wa*xtal.sf[d].sigmah[sa] + wb*xtal.sf[d].sigmah[sb];
	xtal.binweights(d, r, sa, sb, wa, wb);
	fprintf(hfile, "%3d %3d %3d %10.3f %10.3f %10.3f %10.3f\n",
		xtal.miller[r][0], xtal.miller[r][1], xtal.miller[r][2],
		xtal.sf[d].fcalcp[r]/sqrt(eps*sig), xtal.sf[d].pcalcp[r],
		xtal.sf[d].fcalcm[r]/sqrt(eps*sig), xtal.sf[d].pcalcm[r]);
      }
    }
  }

  if (outputmtz)
  {
    if (nstop)
    {
      double dnstop((double)nstop);
      printf("The overall FOM is %.3f to %.2f resolution\n\n",stopfom/dnstop,fomreso);
      printf("The average anomalous Luzzati error is %.3f to %.2f resolution\n\n",meansdluz/dnstop,fomreso);
      printf("The standard deviation of the anomalous Luzzati error is %.3f to %.2f resolution\n\n",
	     sqrt(fabs(dnstop*sdsdluz - meansdluz*meansdluz)/(dnstop*(dnstop-1))), fomreso);
    }
    
    if (outputhcalc)
      fclose(hfile);
    CMtz::MtzPut(MTZOUT, " ");
    if (MTZOUT)
      CMtz::MtzFree(MTZOUT);
    if (MTZIN)
      CMtz::MtzFree(MTZIN);

  }  
  
  if (xtal.verbose                > 1)
    printf("LIKELIHOOD = %f\n",likelihood);
  
  if (flagged && check)
    printf("Number of reflections with low likelihood values: %u\n", flagged);

  return likelihood;
}
