/*
  Copyright (c) Navraj Pannu       1998 - 2005
  Copyright (c) Leiden University  2002 - 2005

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "sfdata.h"

Sfdata::Sfdata()
{
  MINK      = ZERO;  MAXK    = 100.0; MINBSCLISO = -200.0;  MAXBSCLISO = 200.0;
  MINDLUZ   = ZERO;  MAXDLUZ = 25.0;  MINADLUZ   = ZERO;    MAXADLUZ   = ONE;
  MINELUZ   = ZERO;  MAXELUZ = 25.0;  MINSDLUZ   = ZERO;    MAXSDLUZ   = 2.25;
  MINBINS   = 5;     MAXBINS = 20;    MINRES     = ZERO;    MAXRES     = 1000.0;
  defsdluz  = 0.65;
  anomalous = noniso         = false;
  nwave     = nxtal          = nbins             = 0;
  refined   = refinead       = refinesd          = true;
  refinee   = false;
  kscale    = ONE;
  biso      = ZERO;
  refinek   = refineb        = true;
  hires     = MAXRES;
  lowres    = MINRES;
  sigmacut  = anosigmacut    = isosigmacut       = -1000000.1;
  sdatap    = sdevp          = sdatam            = sdevm               = "";
  type      = "";
}

void Sfdata::resize(const unsigned maxref, const bool anom)
{
  anomalous           = anom;
  if (datap.size()   != maxref)
  {
    datap.resize(maxref, NOTUSED);
    devp.resize(maxref, NOTUSED);
    fcalcp.resize(maxref, ZERO);
    pcalcp.resize(maxref, ZERO);
    if (sfmodel.size() != 0)
    {
      fmodel.resize(maxref, ZERO);
      pmodel.resize(maxref, ZERO);
    }
  }
  
  if (anomalous)
  {
    if (datam.size() != maxref)
    {
      datam.resize(maxref, NOTUSED);
      devm.resize(maxref, NOTUSED);
      fcalcm.resize(maxref, ZERO);
      pcalcm.resize(maxref, ZERO);
    }
  }
  else // // if there is no anomalous data, don't refine adluz!!
    refinead = false;
}

void Sfdata::resizebin()
{
  if (sigman.size() != nbins)
  {
    sigman.resize(nbins, ZERO);
    sigmanref.resize(nbins, ZERO);
    sigmadano.resize(nbins, ZERO);
    sigmah.resize(nbins, ZERO);
    nshl.resize(nbins, 0);
    anonshl.resize(nbins, 0);
    astolsq.resize(nbins, ZERO);
    fovers.resize(nbins, ZERO);
    danoovers.resize(nbins, ZERO);  
    aisolofc.resize(nbins, ZERO);
    cisolofc.resize(nbins, ZERO);
    anolofc.resize(nbins, ZERO);
    asumfph.resize(nbins, ZERO);
    csumfph.resize(nbins, ZERO);
    anosumfph.resize(nbins, ZERO);
    asumfh.resize(nbins, ZERO);
    csumfh.resize(nbins, ZERO);
    sumimfh.resize(nbins, ZERO);
    asumdiff.resize(nbins, ZERO);
    csumdiff.resize(nbins, ZERO);
    sumdano.resize(nbins, ZERO);
    anshl.resize(nbins, 0);
    cnshl.resize(nbins, 0);
    sanonshl.resize(nbins, 0);
  }
}

Sfdata &Sfdata::Setdluz(const unsigned s, const double din)
{
  if (s     < nbins)
    dluz[s] = between(MINDLUZ,din,MAXDLUZ);
  else
    Bp3Error("Sfdata::Setdluz", "out of bounds array indices");
  
  return *this;
}

Sfdata &Sfdata::Seteluz(const unsigned s, const double ein)
{
  if (s     < nbins )
    eluz[s] = between(MINELUZ,ein,MAXELUZ);
  else
    Bp3Error("Sfdata::Seteluz", "out of bounds array indices");
  
  return *this;
}

Sfdata &Sfdata::Setadluz(const unsigned s, const double adin)
{
  if (s      < nbins )
    adluz[s] = between(MINADLUZ,adin,MAXADLUZ);
  else
    Bp3Error("Crystal::Setadluz", "out of bounds array indices");
  
  return *this;
}

Sfdata &Sfdata::Setsdluz(const unsigned s, const double sdin)
{
  if (s      < nbins )
    sdluz[s] = between(MINSDLUZ,sdin,MAXSDLUZ);
  else
    Bp3Error("Crystal::Setsdluz", "out of bounds array indices");
  
  return *this;
}

Sfdata &Sfdata::Setsdluzref(const unsigned s, const double sdin)
{
  if (s         < nbins )
    sdluzref[s] = between(MINSDLUZ,sdin,MAXSDLUZ);
  else
    Bp3Error("Crystal::Setsdluz", "out of bounds array indices");
  
  return *this;
}

Sfdata &Sfdata::Setdluz(const vector<double> &din)
{
  for (unsigned i = 0; i < din.size(); i++)
    dluz.push_back(between(MINDLUZ,din[i],MAXDLUZ));
  
  return *this;
}

Sfdata &Sfdata::Seteluz(const vector<double> &ein)
{
  for (unsigned i = 0; i < ein.size(); i++)
    eluz.push_back(between(MINELUZ,ein[i],MAXELUZ));
  
  return *this;
}

Sfdata &Sfdata::Setadluz(const vector<double> &adin)
{
  for (unsigned i = 0; i < adin.size(); i++)
    adluz.push_back(between(MINADLUZ,adin[i],MAXADLUZ));
  
  return *this;
}

Sfdata &Sfdata::Setsdluz(const vector<double> &sdin)
{
  for (unsigned i = 0; i < sdin.size(); i++)
    sdluz.push_back(between(MINSDLUZ,sdin[i],MAXSDLUZ));
  
  return *this;
}

Sfdata &Sfdata::Setsdluzref(const vector<double> &sdin)
{
  for (unsigned i = 0; i < sdin.size(); i++)
    sdluzref.push_back(between(MINSDLUZ,sdin[i],MAXSDLUZ));
  
  return *this;
}

Sfdata &Sfdata::Setnref()
{
  nref            = 0;
  for (unsigned r = 0; r < datap.size(); r++)    
    if (use(r))
      nref++;

  printf("Number of reflections in for Dataset %s is %u\n\n",name.c_str(), nref);
  
  return *this;
}

Sfdata &Sfdata::Setshell()
{
  if (lowres > EPSILON)
    lowsq    = QUARTER/(lowres*lowres);
  else
    Bp3Error("Sfdata::Setshell", "low resolution limit is zero");
  
  if (hires  > EPSILON)
    hisq     = QUARTER/(hires*hires);
  else
    Bp3Error("Sfdata::Setshell","high resolution limit is zero");

  if (hisq  != lowsq)
    vshell   = (work)(nbins)/(hisq - lowsq);
  else
    Bp3Error("Sfdata::Setshell","high and low resolution limits are the same");

  return *this;  
}

void Sfdata::checkparameters()
{
  // do not want to refine reference set scale and Luzzati parameters
  if ((!nxtal) && (!nwave))
  {
    refinek           = false;
    refineb           = false;
    refined           = false;
    refinesdref       = false;
  }

  /*     ***NSP
  if ( (dluz.size()  != nbins) || (adluz.size() != nbins) || 
       (eluz.size()  != nbins) || (sdluz.size() != nbins)   )    
    Bp3Error("Sfdata::checkparameters",
	     "nbins and Luzzati parameters don't match");
  */
}

void Sfdata::printLuzzati(const string target, const int cyc, const bool verbose) const
{
  if (verbose)
  {
    if (cyc           > 0)
      printf(" $TABLE: Cycle %d Dataset %s Luzzati parameters vs. STOL2 - iso, ano:\n",
	     cyc, name.c_str());
    else
      printf(" $TABLE: Dataset %s Luzzati parameters vs. STOL2 - iso, ano:\n",
	     name.c_str());
    
    printf("$GRAPHS");
  }
  
  if ( (target        == "UNCO") || (target == "") )
    printf(": Luzzati iso parameters vs. STOL2:N:4,5:\n");
  if (anomalous)
    if ( (target      == "UNCO") || (target == "") )
      printf(": Luzzati anom parameters vs. STOL2:N:4,6:\n");
    else
      printf(": Luzzati anom parameters vs. STOL2:N:4,5:\n");

  printf("$$\n Bin   HiRes  LoRes   STOL2 ");
  if ( (target        == "UNCO") || (target == "") )
    printf("  Isomorphous");
  if (anomalous)
    printf("   Anomalous  $$\n$$\n");
  else
    printf("$$\n$$\n");
  for (unsigned s      = 0; s < nbins; s++)
  {
    printf(" %2d  %6.2f  %6.2f  %7.5f ",s+1,Getlores(s),Gethires(s),astolsq[s]);
    if ( (target      == "UNCO") || (target == "") )
      printf("%10.5f  ",dluz[s]);
    if (anomalous)
    {
      if ( (target   == "UNCO") || (target == "") )
	printf("%10.5f     ", adluz[s]);
      else if ( (target == "SAD") || (target == "MAD") )
	printf("%10.5f    ", sdluz[s]);
    }
    printf("\n");
  }
  printf("$$\n\n\n");
  fflush(stdout);  
}

void Sfdata::printnorm(const bool verbose) const
{
  printf("\n");
  if (verbose)
  {
    printf(" $TABLE: Dataset %s Normalization statistics vs. STOL2:\n", name.c_str());
    printf("$GRAPHS: SigmaN    vs. STOL2:N:4,6:\n");
  }
  
  printf("       : SigmaH    vs. STOL2:N:4,8:\n");
  if (anomalous)
    printf("       : SigmaDano vs. STOL2:N:4,9:\n");
    printf("       : Variance  vs. STOL2:N:4,10:\n");

  printf("$$\n Bin  HiRes  LoRes   STOL2     Refls    SigmaN    SigmaNref    ");
  printf("  SigmaH    SigmaDano   Variance $$\n$$\n");
  for (unsigned s  = 0; s < nbins; s++)
  {
    if (anomalous)
    {
      printf(    " %2d  %6.2f %6.2f  %7.5f   %5i  %10.2f  %10.2f  %10.2f  %10.2f",
		   s+1, Getlores(s), Gethires(s), astolsq[s], nshl[s],
		 sigman[s], sigmanref[s], sigmah[s], sigmadano[s]);
      printf(" %10.2f\n", ( (sigman[s] - dluz[s]*dluz[s]*sigmanref[s]) < EPSILON ) ?
	     ZERO : sigman[s] - dluz[s]*dluz[s]*sigmanref[s]);
    }
    else
    {
      printf(      " %2d  %6.2f %6.2f  %7.5f   %5i  %10.2f  %10.2f  %10.2f  %10.2f",
		   s+1, Getlores(s), Gethires(s), astolsq[s], nshl[s],
		   sigman[s], sigmanref[s], sigmah[s], 0.00);
      printf(" %10.2f\n", ( sigman[s] - dluz[s]*dluz[s]*sigmanref[s]-sigmah[s] < EPSILON ) ?
	     ZERO : sigman[s] - dluz[s]*dluz[s]*sigmanref[s]-sigmah[s]);
    }
  }
  printf("$$\n\n");
  fflush(stdout);  
}

void Sfdata::printsignal() const
{
  printf("\n");
  printf(" $TABLE: Dataset %s Signal to noise ratio vs. STOL2:\n", name.c_str());
  string ston( ( (type == "AMPLITUDE") ? "F/SigmaF" : "I/SigmaI" ) );
  printf("$GRAPHS: %s    vs. STOL2:N:4,6:\n",ston.c_str());
  if (anomalous)
    printf("       : Dano/SigDano vs. STOL2:N:4,8:\n");

  printf("$$\n Bin  HiRes  LoRes   STOL2    Refls     %s",ston.c_str());
  if (anomalous)
    printf("   AnoRefls    Dano/SigDano");
  printf(" $$\n$$\n");
  for (unsigned s  = 0; s < nbins; s++)
  {
    printf(" %2d  %6.2f %6.2f  %7.5f   %5i  %10.2f",
	   s+1, Getlores(s), Gethires(s), astolsq[s], nshl[s],
	   fovers[s]);
    if (anomalous)
      printf("     %5i    %10.2f\n",anonshl[s], danoovers[s]);
    else
      printf("\n");
  }
  printf("$$\n\n");
  fflush(stdout);  
}

void Sfdata::initstat()
{
  for (unsigned s  = 0; s < nbins; s++)
  {
    aisolofc[s]    = cisolofc[s] = anolofc[s]   = ZERO;
    asumfph[s]     = csumfph[s]  = anosumfph[s] = ZERO;
    asumfh[s]      = csumfh[s]   = sumimfh[s]   = ZERO;
    asumdiff[s]    = csumdiff[s] = sumdano[s]   = ZERO;
    anshl[s]       = cnshl[s]    = sanonshl[s]  =  0;
  }
}

void Sfdata::printstat() const
{
  vector<double> aphasepower(nbins, ZERO), cphasepower(nbins, ZERO);
  vector<double> arkraut(nbins, ZERO), crkraut(nbins, ZERO);
  vector<double> arcullis(nbins, ZERO), crcullis(nbins, ZERO);
  vector<double> anophasepower(nbins, ZERO), anorkraut(nbins, ZERO);
  vector<double> anorcullis(nbins, ZERO);

  for (unsigned s        = 0; s < nbins; s++)
  {
    if (aisolofc[s])      
      aphasepower[s]     = asumfh[s]/aisolofc[s];
    if (cisolofc[s])
      cphasepower[s]     = csumfh[s]/cisolofc[s];
    if (asumfph[s])
      arkraut[s]         = aisolofc[s]/asumfph[s];
    if (csumfph[s])
      crkraut[s]         = cisolofc[s]/csumfph[s];
    if (asumdiff[s])
      arcullis[s]        = aisolofc[s]/asumdiff[s];
    if (csumdiff[s])
      crcullis[s]        = cisolofc[s]/csumdiff[s];
    if (anomalous)
    {
      if (anolofc[s])
	anophasepower[s] = sumimfh[s]/anolofc[s];
      if (anosumfph[s])
	anorkraut[s]     = anolofc[s]/anosumfph[s];
      if (sumdano[s])
	anorcullis[s]    = anolofc[s]/sumdano[s];
    }
  }
  printonestat("R Kraut", crkraut, arkraut, anorkraut);
  printonestat("R Cullis", crcullis, arcullis, anorcullis);
  printonestat("Phasing Power", cphasepower, aphasepower, anophasepower);
}

void Sfdata::printonestat(const string stat, const vector<double> &cstat,
			  const vector<double> &astat,
			  const vector<double> &anostat) const
{
  
  printf(" $TABLE: Dataset %s  %s vs. STOL2-centric, acentric, overall, anom:\n",
	 name.c_str(), stat.c_str());
  if (anomalous)
    printf("$GRAPHS: %s vs. STOL2:N:4,6,8,10,12:\n", stat.c_str());
  else  
    printf("$GRAPHS: %s vs. STOL2:N:4,6,8,10:\n", stat.c_str());

  double totalcstat(ZERO);
  unsigned totalcnshl(0);

  double totalastat(ZERO);
  unsigned totalanshl(0);

  double totalanostat(ZERO);
  unsigned totalanonshl(0);

  printf("$$\n Bin   HiRes  LoRes   STOL2    Refls   Centr   Refls  Acentr");
  printf("   Refls    All");
  
  if (anomalous)
    printf("    Refls   Anom  $$\n$$\n");
  else
    printf("  $$\n$$\n");
  printf("\n");
  for (unsigned s   = 0; s < nbins; s++)
  {
    double statbin  = ((anshl[s] + cnshl[s]) ?
		       (astat[s]*anshl[s]+cstat[s]*cnshl[s])/
		       ((double)anshl[s]+cnshl[s]) : ZERO);
    printf(" %2d  %6.2f  %6.2f  %7.4f  %5d  %7.4f  %5d  %7.4f  %5d  %7.4f",
	   s+1, Getlores(s), Gethires(s), astolsq[s],
	   cnshl[s],cstat[s],anshl[s],astat[s],anshl[s] + cnshl[s], statbin);
    if (anomalous)
      printf("  %5d  %7.4f\n", sanonshl[s], anostat[s]);
    else
      printf("\n");
      
    totalcstat     += cstat[s]*cnshl[s];
    totalcnshl     += cnshl[s];
    totalastat     += astat[s]*anshl[s];
    totalanshl     += anshl[s];
    if (anomalous)
    {
      totalanostat += anostat[s]*sanonshl[s];
      totalanonshl += sanonshl[s];
    }
  }
  double totcstat   = (totalcnshl) ? totalcstat/((double)totalcnshl) : ZERO;
  double totastat   = (totalanshl) ? totalastat/((double)totalanshl) : ZERO;
  double totstat    = ((totalanshl+totalcnshl) ?
		       (totalastat+totalcstat)/((double)(totalanshl+totalcnshl))
		       : ZERO);
  printf("$$\nTOTAL\n");  
  printf("     %6.2f  %6.2f           %5d  %7.4f  %5d  %7.4f  %5d  %7.4f",
	 Getlores(0),Gethires(nbins-1), totalcnshl, totcstat,
	 totalanshl, totastat, totalanshl+totalcnshl, totstat);

  if (anomalous)
  {
    double totano   = (totalanonshl) ? totalanostat/totalanonshl : ZERO;
    printf("  %5d  %7.4f\n", totalanonshl, totano);
  }		 
  else
    printf("\n");
    
  printf("\n\n");
  fflush(stdout);  
}

void Sfdata::print() const
{
  printf("Structure factor data\n");
  printf("Number of reflections: %u\n", nref);
  fflush(stdout);  
}
