C
C     CAD
C     Copyright (C) 1992 Eleanor Dodson 
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C*************************************************************
C
C
C**************************************************************
C
C     ==============
      PROGRAM CAD
C     ==============
C
C
C
C----   I thought many facilities were unnecessary.
C
C       no need to reduce I SIGI to F SIGF - do it in TRUNCATE>
C       no need to SCALE or collect statistics- do it in ANISOSC_NEW
C
C   Subroutines in program
C  CAD.f:      MAIN PROGRAM
C  CADKEY.f:      Input
C  SORTINI.f:     Initialise Sort o/p
C  LCTPIN.f:      Deciphers CTYP line - assigns column types.
C                 (Useful in mtzlib?)
C  ASUPHP.f:      Gets correct phase for symmetry operator.
C                 Belongs in symlib.for
C                 Something called ASUPHS there which should be replaced.
C
C 
C     The program CAD may be used to combine  and  sort  
C     reflection  data from a  number  of  input  reflection  
C     data  files  with  various  possible operations being 
C     performed on the input data items. Apart from manipulating
C     the data values, data may be converted from one area of 
C     reciprocal space to another. Other special functions allow 
C     for the generation of input data for expansion  of  the  data  
C     to  a  lower symmetry if required, and for the generation of 
C     data for input to  the  'R' factor search function program. 
C     The  output data from  the  program  may  be combined with data 
C     from a master file.
C 
C                Preparation_of_Search_Function_data
C 
C       In this case the input is  a  partial  Fcalc  value  and  a
C       phase.  The  output  is  NSYM  sets  of  Fcalc  and   phase
C       (corrected) values.
C 
C       Names of input items are:    FCPART  PHI
C       Output labels required for:  Fcalc phase
C       Input assignments for:       H K L S FCPART PHI
C 

C    automatically expanded to give NSYM pairs of output  labels
C    by appending the numbers 1 to NSYM to the given labels.
C 
C   PROGRAM_FUNCTION
C 
C     The program CAD may be used for  a  number  of  
C     different  purposes which are outlined below. 
C     Simply stated, the program reads one or more sets
C     of reflection data, applies  various  operations  to  
C     them  and  outputs  a combined and sorted set of data.
C 
C   a)   Convert_data_to_a_new_region_of_reciprocal_space
C 
C     The program may be used to convert  reflection  data  to  a  
C different region of reciprocal space. Symmetry operations are applied  
C to  the  input hkl data in order to generate the reflections in new 
C regions of  reciprocal space. The program selects  the  data,  from  
C this  generated  data,  which satisfies the conditions for the 
C requested asymmetric unit of the requested Laue group. The SYMM 
C control cards specify the symmetry  operations  to  be performed on 
C the input data  whilst  a  LAUE  control  card  specifies  the
C desired region of reciprocal space for the output data.
C 
C    b)   Combine_data_from_various_sources
C 
C   The program may be used  to  combine  data  from  various  
C sources  to produce a merged and scaled output file for the 
C required part of reciprocal space. The data items may be 
C manipulated before being  transferred  to  the output file 
C with a number of options being available:-
C 
C       (i)  F, sigma(F)  -> F, sigma(F)
C 
C   c)   Group_Friedel_pairs_of_data
C 
C     Freidel pairs of data may be grouped under a single  
C  set  of  indices. The program expects to read values for  
C  F(hkl)  and  F(-h,-k,-l)  and  will create an output set 
C  of four items per reflection.
C 
C 
C                  F+/sigma(F+)**2 + F-/sigma(F-)**2
C            F =  ___________________________________
C                  1/sigma**2(F+) + 1/sigma**2(F-)
C
C         sigma(F) = [1/sigma**2(F+) + 1/sigma**2(F-)]**1/2
C
C         DeltaF = (F+) - (F-)
C
C         sigma(DeltaF) = [sigma**2(F+) + sigma**2(F-)]**1/2
C
C
C  e)   Preparing_data_for_input_to_the_Search_Function_program
C 
C     A special option is available for use in conjunction with  
C the  Search Function program. Briefly, the input data is a set 
C of structure factor data calculated in space group P1 from a 
C unit  cell  containing  only  a  single asymmetric unit. The program 
C CAD enables the  partial  Fcalc  and  phase data from the NSYM 
C related reflections to be collected under a  single  set
C of indices. The phases are adjusted to correspond to that set of 
C indices so that the result is, in effect, to obtain a set of data 
C with  partial  Fcalc and phase values for each asymmetric unit for 
C each set of indices.
C 
C
C-------------------- Example -------------------------------
C
C
C    #!/bin/csh -f
C    #
C    #
C    cad hklin1  /ccpe0/tms/fvb/fvb18_sg18_f \
C           hklin2 /ccpe0/tms/fvb/sg18_fvs_rot\
C           hklout cad_fvb_sg18\
C           <<'eof-cad'
C    CELL 71.710 86.160 111.930  90.0 90.0 90.0 
C    RESOLUTION OVERALL 100 2.
C    SYMM  18
C    TITLE P2122   eule 28.6 41.6 81.8 Fvsearch model data assuming sg18
C    !
C    LABIN   FILE 1 E1=fvb_F        E2=fvb_SIGF
C    CTYPE   FILE 1 E1=F            E2=Q
C    LABOUT  FILE 1 E1=fvb_F        E2=fvb_SIGF
C    !
C    LABIN   FILE 2 E1=RFV_model_FC E2=RFV_model_PHCAL
C    CTYPE   FILE 2 E1=U            E2=V
C    LABOUT  FILE 2 E1=FCP          E2=PHICP
C    END
C    'eof-cad'
C    #
C--------------------------------------------------------------------------
C
C
C
C          implicit none
C
C
C     .. Parameters ..
      INTEGER MCOLS
      PARAMETER (MCOLS=500)
      INTEGER MAXSETS
      PARAMETER (MAXSETS=MCOLS)
      INTEGER MAXBAT
      PARAMETER (MAXBAT=5000)
      INTEGER MAXHIS
      PARAMETER (MAXHIS=30)
      INTEGER MAXFILES
      PARAMETER (MAXFILES=9)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
      INTEGER NPARM
      PARAMETER (NPARM=2*MCOLS)
      INTEGER NLPRGI
      PARAMETER (NLPRGI=MCOLS)
      INTEGER ENDSRT
      PARAMETER (ENDSRT=-1)
      INTEGER MAXSAVE
      PARAMETER (MAXSAVE=150)
C     ..
C     .. Scalars in Common ..
      REAL RESMAX_OVERALL,RESMIN_OVERALL,S_MAX_OVERALL,S_MIN_OVERALL,   
     +     SCALE_ALL,TEMP_ALL,TEMP_ANIS
      INTEGER MTZPRT,NFILES_IN,NUM_NEW_HIST,NMON
      LOGICAL DO_ADD_TITLE,DO_NEW_HIST,USE_CENTRIC,USE_INPUT_SYMM,
     +        USE_MASTER_SYMM,NO_KEEP_SYSABS,OUTPUT_MISS,GOT_MISS
      CHARACTER TITLE_ADD*70
C     ..
C     .. Arrays in Common ..
C
C---- Asymmetric unit symmetry variables
C
      INTEGER NASU_CODE,NLAUE_ASU,
     +        HMIN_ASU,HMAX_ASU,
     +                  KMIN_ASU,KMAX_ASU,LMIN_ASU,LMAX_ASU
C
C---- Master symmetry variables
C
      INTEGER NSPGRP_MASTER,   NLAUE_MASTER,
     +        NSYMP_MASTER,    NSYM_MASTER
      CHARACTER NAMSPG_MASTER*10, PGNAME_MASTER*10,LATTYP_MASTER*1
      CHARACTER  PGNAM*10
      REAL    RSYMT_MASTER,    RSYM_MASTER
C
C
C
      REAL    S_MAX,
     +        S_MIN,
     +        DCELL_OUT,DCELL_BYNAME,
     +        DWAVE_OUT,DWAVE_BYNAME
      INTEGER PROCESSING_CODE
      LOGICAL GOT_LABIN,
     +        GOT_LABOUT, GOT_XNAME, GOT_DNAME, GOT_DCELL, 
     +        GOT_DWAVE, GOT_DRENAME, GOT_DPNAME
      CHARACTER CTPRGI_NFILE*1,
     +          LSPRGI_NFILE*30,
     +          NEW_HIST*80,
     +          LABIN_SAVE*2048,
     +          LABOUT_SAVE*2048,
     +          XNAME_SAVE*2048,DNAME_SAVE*2048
C     ..
C     .. Local Scalars ..
      REAL SSQ0,PHASE_IN,PHASE_OUT,RECSUM,REF_MULT_FACTOR,VALM,
     +     VRSET_MAGIC
      integer nl,NBATX
      LOGICAL SETVAL
C
C----Column matching stuff, used to swap F+ and F- etc..
C
       integer ncolgf(MCOLS,maxfiles),ncolql(MCOLS,maxfiles),
     +         ncolki(MCOLS,maxfiles),nset(MCOLS,maxfiles),
     +         MTZBLS(MAXBAT)
C
C---- Do loop counters
C
      INTEGER
     +        JDO10,JDO20,JDO30,JDO70,JDO80,JDO81,JDO100,JDO120,
     +        JDO130,JDO140,JDO170,JDO180,JDO200,
     +        JDO240,JDO280,JDO290,JDO300,JDO310,JDO340,JDO350,
     +        JDO360,JDO370,JDO380,JDO390,JDO400,JDO420,JDO260,JDO5
C
C
      INTEGER HM,HKL_SAVE_SIGN,I,II,IRECIN,IRECOUT,ISTAT,ISYM_NUMBER,
     +        ILAB,IS_IT_CENTRIC,IS_IT_SYS_ABS,ITOK,LENTIT,LUNIN,
     +        LUNOUT,LWORK1,LWORK2,LWORK3,LWORK4,LWORK5,MTITOT,MTZAPN,
     +        MTZERR,MTZIN,MTZOUT,NFILE,NFROMSORT,NHISOUT,
     +        NREFLX,NSAVE,NTOK,NUMCOL,JFILE,JSET
     +        NUM_MISS,NUM_COLS_OUT,TOTAL_READ,
     +        TOTAL_REJECT_SST,TOTAL_SYS_ABS,TOTAL_TOSORT,
     +        LDUMM,LFAIL,NUM_TO_OUTPUT
      LOGICAL DO_MASTER,LEND,MTZEOF,USED_REF,GET_MISS
      CHARACTER FNAME*1,CWORK*4,DWORK*2,KEY*4,MTZ_FILE_NAME*6,
     +          MTZDATE*8,VERSNX*10,TITOUT*70,LINERR*100,
     +          MTZOUT_LABELS*2048,LABOUT_LINE*2048,LINE*2048
C     ..
C     .. Local Arrays ..
      REAL CELMTZ(6),RCELMTZ(6),RMETRIC(6),CELOUT(6),FVALUE(NPARM),
     +     RECIN(MCOLS),RECOUT(MCOLS),RECWORK(MAXSAVE,MCOLS),
     +     RNGMTZ(2,MCOLS)
      real bdata(MCOLS)
      LOGICAL RECIN_MISS(MCOLS)
C
C---- Parser stuff
C
      INTEGER
     +        IBEG(NPARM),   IDEC(NPARM),  IEND(NPARM),  ITYP(NPARM)
C
C---- HKL work stuff
C
      INTEGER
     +        HKL_IN(3),
     +        HKL_OUT(3),
     +        HKL_UNIQUE(3)
C
C
      INTEGER   COLS_OP_PER_FILEIN(MAXFILES),
     +          LOOKUP_WORK(NLPRGI),
     +          MSORTX_MASTER(5),
     +          MTZ_LOOKUP(NLPRGI,MAXFILES),
     +          NHISIN(MAXFILES),
     +          NUM_NON_CENTRIC(MAXFILES),
     +          NUM_READ(MAXFILES),
     +          NUM_REF_OUT(MAXFILES),
     +          NUM_REJECT_SST(MAXFILES),
     +          NUM_SYS_ABSENT(MAXFILES),
     +          OP_COLS_STORE(MAXFILES),
     +          SORTKYS
C
C
      CHARACTER CTPRGO(MCOLS)*1,CTYWORK(3)*1,CVALUE(NPARM)*4,
     +          LABWORK(3)*30,LSPRGO(MCOLS)*30,
     +          LSUSRI_WORK(NLPRGI)*30,LSUSRO_WORK(NLPRGI)*30,
     +          TITIN(MAXFILES)*70,HISTIN(MAXHIS,MAXFILES)*80,
     +          HISTOUT(MAXHIS)*80, CTCHECK(MCOLS)*1,
     +          LABCHK(MCOLS)*30
      CHARACTER LAB_STORE_WRITE(MCOLS)*60,
     +          MTZ_STORE_WRITE(MCOLS)*30
      INTEGER   NUM_STORE_WRITE
C 
C----MTZ dataset variables
C
      INTEGER NDATASETS(MAXFILES),ISETS(MAXSETS,MAXFILES),
     +          CSETID_NFILE(NLPRGI,MAXFILES),SETID,ISET,
     +          NSTART,NUM2_COLS_OUT,NTOTDATASETS,
     +          XNAME_LOOKUP(MCOLS),DNAME_LOOKUP(MCOLS)
      REAL DATCELL(6,MAXSETS,MAXFILES),DATWAVE(MAXSETS,MAXFILES)
      CHARACTER*64 PNAME(MAXSETS,MAXFILES),XNAME(MAXSETS,MAXFILES), 
     +         DNAME(MAXSETS,MAXFILES),DCELL_XNAME,
     +         DCELL_DNAME,DWAVE_XNAME,DWAVE_DNAME,
     +         DRENAME_PNAME_OUT,DRENAME_XNAME_OUT,DRENAME_DNAME_OUT,
     +         XNAME_OUT(MCOLS),DNAME_OUT(MCOLS),
     +         DRENAME_XNAME,DRENAME_DNAME,
     +         DPNAME_XNAME,DPNAME_DNAME,
     +         DRENAME_BYNAME_PNAME_OUT,DRENAME_BYNAME_XNAME_OUT,
     +         DRENAME_BYNAME_DNAME_OUT
      CHARACTER*64 XNAME_NEW(MCOLS),DNAME_NEW(MCOLS),PDNAMCHK,
     +         XNAME_CHECK
C     ..
C     .. External Functions ..
      REAL STHLSQ
      INTEGER CCP4SPG_F_INASU,LENSTR,SRTMRG,SRTRET,SRTRLS
      LOGICAL QISNAN
      EXTERNAL STHLSQ,CCP4SPG_F_INASU,LENSTR,SRTMRG,SRTRET,SRTRLS,
     +   QISNAN
C     ..
C     .. External Subroutines ..
      EXTERNAL ASUPHP,ASUSET,CADINI,CCPERR,CCPDAT,CCPFYP,CCPDPN,CCPRCS,
     +         CENTR,CENTRIC,EPSLN,EPSLON,INVSYM,LERROR,LKYIN,LKYOUT,
     +         LKYSET,LRASSN,LRCELL,LRCLOS,LRHIST,LRINFO,LROPEN,LRREFF,
     +         LRSORT,LRSYMI,LRSYMM,LRTITL,LWASSN,LWCELL,LWCLOS,LWHIST,
     +         LWOPEN,LWREFL,LWSORT,LWSYMM,LWTITL,MTZINI,PARSER,CADKEY,
     +         RESET_MAGIC,SET_MAGIC,SETRSL,SORTINI
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,NINT,REAL
C     ..
C     .. Common blocks ..
      COMMON /ASU_INFO/ NASU_CODE,NLAUE_ASU,HMIN_ASU,HMAX_ASU,
     +                  KMIN_ASU,KMAX_ASU,LMIN_ASU,LMAX_ASU
C
C
      COMMON /ATSYM/  RSYM_MASTER(4,4,MAXSYM),
     +                RSYMT_MASTER(4,4,MAXSYM),
     +                NSYM_MASTER,
     +                NSYMP_MASTER,
     +                NSPGRP_MASTER,
     +                NLAUE_MASTER
      COMMON /ATSYM_CHAR/ NAMSPG_MASTER, PGNAME_MASTER, LATTYP_MASTER
C
C
      COMMON /CELLS/  DCELL_OUT(6,MAXSETS,MAXFILES),
     +        DCELL_BYNAME(6,MAXSETS,MAXFILES),
     +        DCELL_XNAME(MAXSETS,MAXFILES),
     +        DCELL_DNAME(MAXSETS,MAXFILES),
     +        DWAVE_OUT(MAXSETS,MAXFILES),
     +        DWAVE_BYNAME(MAXSETS,MAXFILES),
     +        DWAVE_XNAME(MAXSETS,MAXFILES),
     +        DWAVE_DNAME(MAXSETS,MAXFILES),
     +        DRENAME_PNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_XNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_DNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_XNAME(MAXSETS,MAXFILES),
     +        DRENAME_DNAME(MAXSETS,MAXFILES),
     +        DPNAME_XNAME(MAXSETS,MAXFILES),
     +        DPNAME_DNAME(MAXSETS,MAXFILES),
     +        DRENAME_BYNAME_PNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_BYNAME_XNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_BYNAME_DNAME_OUT(MAXSETS,MAXFILES)
      COMMON /GENERAL/ NFILES_IN,
     +                 NUM_NEW_HIST,
     +                 MTZPRT,
     +                 NMON,VAL_MISSING
      COMMON /GENERAL_C/NEW_HIST(MAXHIS)
      COMMON /GENERAL_L/
     +             GOT_LABOUT(MAXFILES),
     +             GOT_LABIN(MAXFILES),
     +             GOT_XNAME(MAXFILES),
     +             GOT_DNAME(MAXFILES),
     +             GOT_DCELL(MAXFILES),
     +             GOT_DWAVE(MAXFILES),
     +             GOT_DRENAME(MAXFILES),
     +             GOT_DPNAME(MAXFILES),
     +             USE_MASTER_SYMM,
     +             USE_INPUT_SYMM,
     +             USE_CENTRIC,
     +             NO_KEEP_SYSABS,
     +             DO_NEW_HIST,
     +             DO_ADD_TITLE,
     +             OUTPUT_MISS,GOT_MISS
      COMMON /LSPRG_INFO/
     +                LSPRGI_NFILE(NLPRGI),
     +                CTPRGI_NFILE(NLPRGI,MAXFILES),
     +                LABIN_SAVE(MAXFILES),
     +                LABOUT_SAVE(MAXFILES),
     +                XNAME_SAVE(MAXFILES),
     +                DNAME_SAVE(MAXFILES)
      COMMON /PROCESS_INFO/   PROCESSING_CODE(MAXFILES),  
     +                        SCALE_ALL(MAXFILES),TEMP_ALL(MAXFILES),
     +                        TEMP_ANIS(6,MAXFILES)
      COMMON /RESO_INFO/
     +             RESMAX_OVERALL,
     +             RESMIN_OVERALL,
     +             S_MAX_OVERALL,
     +             S_MIN_OVERALL,
     +             S_MAX(MAXFILES),
     +             S_MIN(MAXFILES)
      COMMON /SORT_INFO/  SORTKYS(3)
      COMMON /TITL_INFO_C/  TITLE_ADD
C     .. Data statements ..
C
C---- Number of Input columns required
C  ejd change to all the same
C      processing options 1  2  3
C  code = 1  - Master (all in)
C  code = 2  - standard - controlled by LABIN_NFILE
C  code = 3  - includes FCPART PHIPART
C
      DATA NUM_MISS/0/,NUM_COLS_OUT/0/,NUM2_COLS_OUT/0/,MTZAPN/0/
      DATA OP_COLS_STORE/MAXFILES*0/
      DATA TOTAL_READ/0/,TOTAL_TOSORT/0/,TOTAL_REJECT_SST/0/,
     +     TOTAL_SYS_ABS/0/
      DATA COLS_OP_PER_FILEIN/MAXFILES*0/,NUM_READ/MAXFILES*0/,
     +     NUM_REJECT_SST/MAXFILES*0/,NUM_SYS_ABSENT/MAXFILES*0/,
     +     NUM_REF_OUT/MAXFILES*0/,
     +     NUM_NON_CENTRIC/MAXFILES*0/
      DATA LABWORK/'H','K','L'/
      DATA LAB_STORE_WRITE/MCOLS*' '/,
     +     MTZ_STORE_WRITE/MCOLS*' '/
      DATA BDATA/MCOLS*0.0/
      DATA NUM_STORE_WRITE/0/
      DATA NUM_TO_OUTPUT/0/
      DATA XNAME_OUT/MCOLS*' '/,DNAME_OUT/MCOLS*' '/
      DATA XNAME_LOOKUP/MCOLS*0/,DNAME_LOOKUP/MCOLS*0/
cc
cc      data ctypes/'h','s','j','f','d','q','p','m','w','a','b','y','i',
cc     +     'r'/  allowed values
cc
      DATA CTYWORK/'H','H','H'/
C     ..
C
C
C
      CONV = 3.1415927/180.0
      LUNOUT = 6
      LUNIN = 5
      LDUMM = 80
      LFAIL = 0
      VRSET_MAGIC = -9.0E+12
      DO I = 1,MAXFILES
        DO J = 1,MAXSETS
          PNAME(J,I) = ' '
          XNAME(J,I) = ' '
          DNAME(J,I) = ' '
          datwave(j,i) = 0.0
        ENDDO
      ENDDO
      do i=1,mcols
        lsprgo(i) = ' '
      enddo
C
C          ****************************************
      CALL CCPFYP
      CALL MTZINI
      CALL CCPDPN(LUNIN,'DATA','READONLY','F',LDUMM,LFAIL)
      CALL CCPDPN(LUNOUT,'PRINTER','PRINTER','F',LDUMM,LFAIL)
      CALL CCPRCS(6,'CAD','$Date: 2008/07/03 15:52:09 $')
      CALL CADINI
      CALL CCPDAT(MTZDATE)
C          ****************************************
C
C---- Lookup table for processing options, only ever
C     require a max 6 lookup items to match program labels
C
      DO 5  JDO5  = 1,3
          LSPRGI_NFILE(JDO5) = LABWORK(JDO5)
          LSPRGO(JDO5)       = LABWORK(JDO5)
          CTPRGO(JDO5)       = CTYWORK(JDO5)
5     CONTINUE
C
C
      DO 30 JDO30 = 1,MAXFILES
C
C---- set up program expected labels and column types
C
        DO 10 JDO10 = 1,3
          LOOKUP_WORK(JDO10)   = -1
          MTZ_LOOKUP(JDO10,JDO30)   = -1
          CTPRGI_NFILE(JDO10,JDO30) = CTYWORK(JDO10)
   10   CONTINUE
C
C----  make labels E1 E2 E3 .. E9 E10 ... E19 E20 ..  E27
C
        DO 20 JDO20 = 4,NLPRGI
          LOOKUP_WORK(JDO20)   = 0
          MTZ_LOOKUP(JDO20,JDO30) = 0
          II = JDO20 - 3
C
C
          IF (II.LE.9) THEN
            WRITE (CWORK,FMT='(I1)') II
            LSPRGI_NFILE(JDO20) = 'E'//CWORK(1:1)
          END IF
C
C
          IF (II.GT.9.AND.II.LE.99) THEN
            WRITE (CWORK,FMT='(I2)') II
            LSPRGI_NFILE(JDO20) = 'E'//CWORK(1:2)
          END IF
C
          IF (II.GT.99) THEN
            WRITE (CWORK,FMT='(I3)') II
            LSPRGI_NFILE(JDO20) = 'E'//CWORK(1:3)
          END IF
C
          CTPRGI_NFILE(JDO20,JDO30) = ' '
   20   CONTINUE
   30 CONTINUE
C
C
C     =====================
C---- Read key-worded input
C     =====================
C
C          ******
      CALL CADKEY
C          ******
C
C---- Finished Key_word Input
C     =======================
C
C---- Check all input/output labels given
C
      DO 50 NFILE = 1,NFILES_IN
C
C
        IF (.NOT.GOT_LABIN(NFILE)) THEN
          CALL CCPERR(1, ' Error, NOT all LABIN data lines given')
        END IF
C
C
        IF (.NOT.GOT_LABOUT(NFILE)) THEN
          WRITE (LUNOUT,FMT=6002)
 6002     FORMAT (' Warning, NOT all LABOUT data lines given')
        END IF
C
C
   50 CONTINUE
C
C
      MTZIN = 1
C
C----  start opening files...
C      First sort out symmetry information from MASTER if required and
C      build number of output columns
C     (i) always output H K L S
C
      NUM_COLS_OUT = 3
      MTZOUT_LABELS = 'H K L   '
      LABOUT_LINE  = 'LABOUT  '
C
C
      DO 110 NFILE = 1,NFILES_IN
C
C            ******
        CALL MTZINI
C            ******
C
C---- get logical name for input file - may not need to open it..
C     can use same file several times!!
C
        WRITE (FNAME,FMT='(I1)') NFILE
        MTZ_FILE_NAME = 'HKLIN'//FNAME
C
C            *****************************************
        CALL LROPEN(MTZIN,MTZ_FILE_NAME,MTZPRT,MTZERR)
C            *****************************************
C
        IF (MTZERR.EQ.-1) THEN
          WRITE (LUNOUT,FMT=*) ' Error in LROPEN for file name ',
     +      ' Input_File_Name: ',MTZ_FILE_NAME
          CALL CCPERR(1, ' Fatal Error')
        END IF

C    Check we don't have multi-record file
        CALL LRBATS(MTZIN,NBATX,MTZBLS)
        IF (NBATX.GT.0) THEN
          WRITE (6,'(/,A,I3,A)') ' File ', NFILE,
     +        ' is a multi-record MTZ file.'
          WRITE (6,'(A,A)') ' Inputting a multi-record file to CAD ',
     +        'is an unmitigated disaster!'
          WRITE (6,'(A,/)') ' Please use MTZUTILS.'
          CALL CCPERR(1,'Input file is unmerged MTZ file.')
        ENDIF
C
C----- Master_File is always the first file HKLIN1
C
        IF (NFILE.EQ.1)  THEN
          IF ( USE_MASTER_SYMM) THEN
C
C              *****************************************************
          CALL LRSYMI(MTZIN,NSYMP_MASTER,LATTYP_MASTER, NSPGRP_MASTER,
     +                      NAMSPG_MASTER, PGNAME_MASTER)
C              *****************************************************
             IF(NSPGRP_MASTER.GT.0)THEN
             CALL LRSYMM(MTZIN,           NSYM_MASTER,  RSYM_MASTER)
             ELSE
             WRITE (LUNOUT,FMT=*) ' No Symmetry found in Master_Input',
     +        ' File: HKLIN1'
          CALL CCPERR(1, ' Fatal Error')
             END IF
          END IF
C
C---- use key_word input symmetry to
C     set NLAUE code  for this spacegroup unless set previously.
C
C
C EJD  Symmetry should have either come from input or from the MASTER file.
C   Must call ASUSET for the real symmetry to make sure
C information used for phase changes in ASUPHP  is set up properly.
C
                 PGNAM = PGNAME_MASTER
C                **************************************************
            CALL ASUSET(NAMSPG_MASTER,   NSPGRP_MASTER,
     +                  PGNAME_MASTER,   NSYM_MASTER,
     +                  RSYM_MASTER,     NSYMP_MASTER,
     +                  NLAUE_MASTER,    .TRUE.)
C                **************************************************
                 PGNAME_MASTER = PGNAM
C   Invert  symmetry operators
C
          DO 55 ISYM_NUMBER = 1,NSYMP_MASTER
C
C                *************************************
            CALL INVSYM(RSYM_MASTER(1,1,ISYM_NUMBER),
     +                  RSYMT_MASTER(1,1,ISYM_NUMBER))
C                *************************************
C
  55     CONTINUE
C
C   Use symmetry operators to find NASU_CODE if this is not set
C
          IF (NASU_CODE.EQ.0) THEN
            NASU_CODE = NLAUE_MASTER
          END IF 
C
C  print information about reciprocal space limits.
C
        WRITE(LUNOUT,'(//,a)')
     .           ' Chosen Asymmetric unit of reciprocal space: '
C        
           IF (NASU_CODE .LT. 0) THEN
             WRITE(LUNOUT,'(a,3(3x,2i5))')
     .             ' HKL limits are: ',
     + HMIN_ASU,HMAX_ASU,KMIN_ASU,KMAX_ASU,LMIN_ASU,LMAX_ASU
           ELSEIF (NASU_CODE .EQ. 3) THEN
             WRITE(LUNOUT,'(a)')
     .             ' [-1] hkl:l>=0  hk0:h>=0  0k0:k>=0'
           ELSEIF (NASU_CODE .EQ. 4) THEN
             WRITE(LUNOUT,'(a)')
     .             ' [2/m] hkl:k>=0, l>=0  hk0:h>=0'
           ELSEIF (NASU_CODE .EQ. 6) THEN
             WRITE(LUNOUT,'(a)')
     .             ' [mmm] hkl:h>=0, k>=0, l>=0'
           ELSEIF (NASU_CODE .EQ. 7) THEN
             WRITE(LUNOUT,'(a,/,a)')
     .             ' [4/m] hkl:h>=0, l>=0 with k>=0 if h=0',
     .       ' and k>0 if h>0'
           ELSEIF (NASU_CODE .EQ. 8) THEN
             WRITE(LUNOUT,'(a)')
     .             ' [4/mmm] hkl:h>=0, k>=0, l>=0 and h>=k'
           ELSEIF (NASU_CODE .EQ. 9) THEN
             WRITE(LUNOUT,'(a)')
     .             ' [-3] hkl:h>=0, k>0  00l:l>0'
           ELSEIF (NASU_CODE .EQ. 10) THEN
             WRITE(LUNOUT,'(a,/,a)')
     .             ' [312] hkl:h>=0, k>=0 with k<=h ',
     .         ' for all l. If k = 0  l>=0'
           ELSEIF (NASU_CODE .EQ. 11) THEN
             WRITE(LUNOUT,'(a,/,a)')
     .             ' [321] hkl:h>=0, k>=0 with k<=h ',
     .         ' for all l. If k = h  l>=0'
           ELSEIF (NASU_CODE .EQ. 12) THEN
             WRITE(LUNOUT,'(a,/,a)')
     .             ' [6/m] hkl:h>=0, k>=0, l>=0 with k>=0',
     .          ' if h=0, and k> 0 if h>0'
           ELSEIF (NASU_CODE .EQ. 13) THEN
             WRITE(LUNOUT,'(a)')
     .             ' [6/mmm] hkl:h>=0, k>=0, l>=0 with h>=k'
           ELSEIF (NASU_CODE .EQ. 14) THEN
             WRITE(LUNOUT,'(a,/,a)')
     .             ' [m3] hkl:h>=0, k>=0, l>=0 with l>=h,',
     .          ' k>=h if l=h, k> h if l>h'
           ELSEIF (NASU_CODE .EQ. 15) THEN
             WRITE(LUNOUT,'(a)')
     .         ' [m3m] hkl:h>=0, k>=0, l>=0 with k>=l, and l>=h'
C
           END IF
C   Check "missing" value; is it set on input? If so use it.
          IF(GOT_MISS) THEN
            CALL SET_MAGIC(MTZIN,VAL_MISSING,GOT_MISS)
               IF( QISNAN(VAL_MISSING))  THEN
                 WRITE(6,'(//,A)') 
     +     ' ** "Missing" flag set to program input value: Nan'
               ELSE
                 WRITE(6,'(//,A,F18.3)') 
     +     ' ** "Missing" flag set to program input value:',VAL_MISSING
               END IF
          ELSE
            GET_MISS = .FALSE.
            CALL SET_MAGIC(MTZIN,VAL_MISSING,GET_MISS)
               IF( QISNAN(VAL_MISSING))  THEN
                 WRITE(6,'(//,A)')
     +     ' ** "Missing" flag set in HKLIN1 to Nan:'
               ELSE
                 WRITE(6,'(//,A,F18.3)')
     +     ' ** "Missing" flag set in HKLIN1 to:',VAL_MISSING
               END IF
          END IF
          VALM = -999
                 WRITE(6,'(//,A,F18.3)')
     +     ' ** "Missing" entries LISTED as ',VALM
          SETVAL = .TRUE.
        END IF
C
C
C
C---- Now find total number of o/p columns
C     op_cols_store(NFILE) is first position for file NFILE
C
        OP_COLS_STORE(NFILE) = NUM_COLS_OUT + 1
C
        CALL LRCLAB(MTZIN,LABCHK,CTCHECK,NUMCOL)
C
C---- If ALL columns wanted  processing_code = 1
C     Get NUMCOL from FILE
C
        IF (PROCESSING_CODE(NFILE).EQ.1) THEN
C
C              *****************************************
        CALL LRINFO(MTZIN,VERSNX,NUMCOL,NREFLX,RNGMTZ)
        CALL LRCLAB(MTZIN,LSUSRI_WORK,CTPRGI_NFILE(1,NFILE),NUMCOL)
C              *****************************************
C  Construct LABIN_SAVE(NFILE)
        IF (NUMCOL.GT.NLPRGI ) THEN 
           WRITE(6,'(//,A,I4,A,A,I4,A,A)')
     +   ' *** Problem: Only ',NLPRGI,' columns can be copied from ',
     +   ' each HKLINi - this file contains',NUMCOL,' columns ***',
     +   ' You will have to split it into several parts'
           CALL CCPERR(1,' Too many columns in input file.')
        ENDIF
        WRITE(LABIN_SAVE(NFILE),'(A,200(2x,3A))') 'LABIN',
     +  (LSPRGI_NFILE(JDO20)(1:LENSTR(LSPRGI_NFILE(JDO20))),'=',
     +  LSUSRI_WORK(JDO20)(1:LENSTR(LSUSRI_WORK(JDO20))),JDO20=4,NUMCOL)
C
C---- Do I mean -3??
C
C          COLS_OP_PER_FILEIN(NFILE) = NUMCOL - 3
C          NUM_COLS_OUT               = NUM_COLS_OUT + NUMCOL - 3
C
C----  Start getting LABOUT labels ready.
C
C          MTZAPN = 1
        END IF
C
C----   If   columns assigned  processing_code =  2 or 3
C       LKYSET returns MTZ_LOOKUP(II,NFILE)
C       set to -1 for any LSPRGI_NFILE label set.
C
C----  Clear arrays  to check label assignments
C
        IF (PROCESSING_CODE(NFILE).GT.1) THEN
          DO 70 JDO70 = 1,NLPRGI
            LSUSRI_WORK(JDO70)       = ' '
            IF(JDO70.GT.3)MTZ_LOOKUP(JDO70,NFILE) = 0
   70     CONTINUE
        END IF 
C
C       IF (PROCESSING_CODE(NFILE).GT.1) THEN
C
C---- UNPACK LABIN_SAVE line
C
          NTOK = NPARM
          LINE = LABIN_SAVE(NFILE)
C
C              *******************************************************
          CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,
     +                LEND,.TRUE.)
C              *******************************************************
C
          ITOK = 2
C
C   LSPRGI_NFILE are E1 E2 ...
C  returns LSUSRI_WORK as the set labels.
C
C              *****************************************************
          CALL LKYSET(LSPRGI_NFILE,NLPRGI,LSUSRI_WORK,
     +                MTZ_LOOKUP(1,NFILE),ITOK,NTOK,LINE,IBEG,IEND)
C              *****************************************************
C
C     Check to ensure E1 to EX has no gaps.
C
          DO 75 I=1, ((NTOK -3)/2)
            IF (MTZ_LOOKUP((I + 3),NFILE) .GT. -1) THEN
              CALL CCPERR (1,' ERR: LABIN COLUMN IDENTIFIERS MISSING')
            ENDIF
 75       CONTINUE
C
C----  Now UNPACK LABOUT_SAVE line -
C      default is output label = input label
C
          DO 80 JDO80 = 1,NLPRGI
            LSUSRO_WORK(JDO80) = LSUSRI_WORK(JDO80)
            LOOKUP_WORK(JDO80) = 0
C     Assign CTYP equal to input CTYPE unless already assigned.
             IF(CTPRGI_NFILE(JDO80,NFILE).NE.' ') GO TO 80
             DO 81 JDO81 = 1,NUMCOL
             IF(LSUSRI_WORK(JDO80) .EQ. LABCHK(JDO81) )
     +       CTPRGI_NFILE(JDO80,NFILE) = CTCHECK(JDO81)
   81        CONTINUE
   80     CONTINUE
C
C
          IF (LABOUT_SAVE(NFILE).NE.' ') THEN
            LINE = LABOUT_SAVE(NFILE)
C
C                ******************************************************
            CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,
     +                  LEND,.TRUE.)
C                ******************************************************
C
            ITOK = 2
C
C                **************************************************
            CALL LKYSET(LSPRGI_NFILE,NLPRGI,LSUSRO_WORK,LOOKUP_WORK,
     +                  ITOK,NTOK,LINE,IBEG,IEND)
C                **************************************************
C
          END IF
C        END IF 
C
C----  Count number of assignments
C
          COLS_OP_PER_FILEIN(NFILE) = 0
          ILAB = 0
C
C
          DO 100 JDO100 = 4,NLPRGI
            IF (MTZ_LOOKUP(JDO100,NFILE).EQ.0) GO TO 100
              NUM_COLS_OUT               = NUM_COLS_OUT + 1
              COLS_OP_PER_FILEIN(NFILE) = 
     +             COLS_OP_PER_FILEIN(NFILE) +  1
C
C----  Assign output program labels here.
C      Build LSPRGO E1nfile E2nfile ... etc
C
C
C   Actually MAXFILES = 9  - 
              IF (NFILE.GT.9) CALL CCPERR(1,' Too many output files')
C
C
              IF (CTPRGI_NFILE(JDO100,NFILE).NE.'U' .AND.
     +            CTPRGI_NFILE(JDO100,NFILE).NE.'V') THEN
C
              ILAB               = ILAB + 1
C              LWORK2 = LENSTR(LSPRGI_NFILE(JDO100))
C
              IF (ILAB.LE.9) 
     +          WRITE (CWORK,FMT='(A1,I1,I1)') 'E',ILAB,NFILE
              IF (ILAB.GT.99) 
     +          WRITE (CWORK,FMT='(A1,I3,I1)') 'E',ILAB,NFILE
              IF (ILAB.GT.9.AND.ILAB.LE.99) 
     +          WRITE (CWORK,FMT='(A1,I2,I1)') 'E',ILAB,NFILE
                LWORK5 = LENSTR(CWORK)
                LSPRGO(NUM_COLS_OUT) = CWORK(1:LWORK5)
C----  construct LABOUT line
C      it will contain LSPRGO(..)=LSUSRO(..) for every column.
C
                LWORK1 = LENSTR(LABOUT_LINE)
                LWORK2 = LENSTR(LSPRGO(NUM_COLS_OUT))
                LWORK3 = LENSTR(LSUSRO_WORK(JDO100))
                LWORK4 = LENSTR(MTZOUT_LABELS)
C
C
                NUM_STORE_WRITE = NUM_STORE_WRITE + 1
                LAB_STORE_WRITE(NUM_STORE_WRITE) = ' '//
     +                    LSPRGO(NUM_COLS_OUT) (1:LWORK2)//'='//
     +                    LSUSRO_WORK(JDO100) (1:LWORK3)
                LABOUT_LINE = LABOUT_LINE(1:LWORK1)//
     +             LAB_STORE_WRITE(NUM_STORE_WRITE)
     +             (1:LENSTR(LAB_STORE_WRITE(NUM_STORE_WRITE)))
C
C
                MTZ_STORE_WRITE(NUM_STORE_WRITE) = ' '//
     +                         LSUSRO_WORK(JDO100) (1:LWORK3)
                MTZOUT_LABELS = MTZOUT_LABELS(1:LWORK4)//
     +             MTZ_STORE_WRITE(NUM_STORE_WRITE)
     +             (1:LENSTR(MTZ_STORE_WRITE(NUM_STORE_WRITE)))
C
C
                CTPRGO(NUM_COLS_OUT) = CTPRGI_NFILE(JDO100,NFILE)
              END IF
C
C
              IF (CTPRGI_NFILE(JDO100,NFILE).EQ.'U' .OR.
     +            CTPRGI_NFILE(JDO100,NFILE).EQ.'V') THEN
C
C----  fix up FCPART now
C
                LWORK2 = LENSTR(LSPRGO(NUM_COLS_OUT))
                LWORK3 = LENSTR(LSUSRO_WORK(JDO100))
C   Remove all labl "E" additions.

C
                DO 90 ISYM_NUMBER = 1,NSYMP_MASTER
                   IF (CTPRGI_NFILE(JDO100,NFILE).EQ.'U')
     +         CTPRGO(NUM_COLS_OUT+2*ISYM_NUMBER - 2) = 'F'
                   IF (CTPRGI_NFILE(JDO100,NFILE).EQ.'V')
     +         CTPRGO(NUM_COLS_OUT+2*ISYM_NUMBER - 2) = 'P'
C
C----   OUTPUT  label is FCisym_number   or PHICisym_number
C
C   Plabels - count Ei,nfile, Ei+1,nfile ..Ei+nsymp_master,nfile
C     File number must be < 10 I think
C
C  First remove file number
                  ILAB               = ILAB + 1
                  IF (ILAB.LT.10) 
     +               WRITE (CWORK,FMT='(A1,I1,I1)') 'E',ILAB,NFILE
                  IF (ILAB.GE.10.AND.ILAB.LE.99) 
     +               WRITE (CWORK,FMT='(A1,I2,I1)') 'E',ILAB,NFILE
C
                  IF (ILAB.GE.100) 
     +               WRITE (CWORK,FMT='(A1,I3,I1)') 'E',ILAB,NFILE
C
                  IF (ISYM_NUMBER.LT.10) 
     +               WRITE (DWORK,FMT='(I1,A1)') ISYM_NUMBER,' '
                  IF (ISYM_NUMBER.GE.10) 
     +               WRITE (DWORK,FMT='(I2)') ISYM_NUMBER
C
                    LWORK5 = LENSTR(CWORK)
C
                    LSPRGO(NUM_COLS_OUT+2*ISYM_NUMBER - 2) = 
     +              CWORK(1:LWORK5)
                    LWORK1 = LENSTR(LABOUT_LINE)
                    LWORK4 = LENSTR(MTZOUT_LABELS)
C
C
                NUM_STORE_WRITE = NUM_STORE_WRITE + 1
                    LAB_STORE_WRITE(NUM_STORE_WRITE) = ' '//
     +                            CWORK(1:LWORK5)//'='//
     +                            LSUSRO_WORK(JDO100)(1:LWORK3)//
     +                             DWORK
C
C
                    MTZ_STORE_WRITE(NUM_STORE_WRITE) = ' '//
     +                             LSUSRO_WORK(JDO100) (1:LWORK3)//
     +                             DWORK
C
C
C
C
   90           CONTINUE
C
              END IF
  100     CONTINUE
C
Cejd - shit..all the  FC1 phi1 .... has to be reordered
C
            IF(PROCESSING_CODE(NFILE) .EQ.3) THEN
                NUM_STORE_WRITE = NUM_STORE_WRITE - 2*NSYMP_MASTER
               DO 101 ISYM_NUMBER = 1,NSYMP_MASTER
                    LWORK1 = LENSTR(LABOUT_LINE)
                    LWORK4 = LENSTR(MTZOUT_LABELS)
C
C----   PROGRAM label is FCisym_number   or PHICisym_number
C
C
                NUM_STORE_WRITE = NUM_STORE_WRITE + 1
                    LABOUT_LINE = LABOUT_LINE(1:LWORK1)//
     +               LAB_STORE_WRITE(NUM_STORE_WRITE)
     +             (1:LENSTR(LAB_STORE_WRITE(NUM_STORE_WRITE)))//
     +   LAB_STORE_WRITE(NUM_STORE_WRITE+NSYMP_MASTER)
     +   (1:LENSTR(LAB_STORE_WRITE(NUM_STORE_WRITE+NSYMP_MASTER)))
C
C
                    MTZOUT_LABELS = MTZOUT_LABELS(1:LWORK4)//
     +               MTZ_STORE_WRITE(NUM_STORE_WRITE)
     +             (1:LENSTR(MTZ_STORE_WRITE(NUM_STORE_WRITE)))//
     +  MTZ_STORE_WRITE(NUM_STORE_WRITE+NSYMP_MASTER)
     + (1:LENSTR(MTZ_STORE_WRITE(NUM_STORE_WRITE+NSYMP_MASTER)))
C
C
  101           CONTINUE
                NUM_STORE_WRITE = NUM_STORE_WRITE + 2*NSYMP_MASTER
C
C----   If FCPART PHIPART - increase number of columns output
C
            NUM_COLS_OUT = 2*NSYMP_MASTER + NUM_COLS_OUT - 2
            COLS_OP_PER_FILEIN(NFILE) = COLS_OP_PER_FILEIN(NFILE) +
     +                                   2*NSYMP_MASTER - 2
            END IF
C
          WRITE (LUNOUT,FMT=6661) NFILE,NUM_COLS_OUT,
     +     (LAB_STORE_WRITE(II)(1:LENSTR(LAB_STORE_WRITE(II))),
     +      II=1,NUM_STORE_WRITE)
          WRITE (LUNOUT,FMT=6662)
     +     (MTZ_STORE_WRITE(II)(1:LENSTR(MTZ_STORE_WRITE(II))),
     +      II=1,NUM_STORE_WRITE)
6661     FORMAT(//,' After Processing File_Number: ',I2,/,
     +             ' Number of columns so far    : ',I3,/,
     +             ' Accumulated Out_Put_MTZ_labels are:',/,
     +           99(' ',A,' ',A,' ',A,' ',A,/))
6662     FORMAT(//,' MTZOUT_LABELS are H K L and -',/,
     +           99(' ',A,' ',A,' ',A,' ',A,/))
c         write(lunout,'(2a)') ' labout_line ',labout_line
c         write(lunout,'(2a)') ' mtzout_labels ',mtzout_labels
C        END IF
C
C            *************
        CALL LRCLOS(MTZIN)
C            *************
C
  110 CONTINUE
C
C---- Initialise SORTING o/p
C
 6663   FORMAT(///)
C
C          ****************************
      CALL SORTINI(NUM_COLS_OUT,LUNOUT)
C          ****************************
C
      WRITE(LUNOUT,6663)
      TOTAL_TOSORT = 0
C
C
      DO 270 NFILE = 1,NFILES_IN
        MTZIN = 2
        IF (NFILE.EQ.1) DO_MASTER = .TRUE.
C
C---- Now check if this file has resolution limits or should we use
C     overall ones?
C
        IF (S_MAX(NFILE).EQ.0.0 .AND. S_MIN(NFILE).EQ.0.0) THEN
          S_MAX(NFILE) = S_MAX_OVERALL
          S_MIN(NFILE) = S_MIN_OVERALL
        END IF
C
C
        IF (S_MAX(NFILE).EQ.0.0 .AND. S_MIN(NFILE).EQ.0.0) THEN
          S_MAX(NFILE) = 10.0
          S_MIN(NFILE) = 0.0
        END IF
C
C---- Now want column labels and column numbers for
C     current input file
Cejd   - what do I do if I want ALL columns back??
Cejd  - here is a junkie fix...
C        IF(PROCESSING_CODE(NFILE).EQ.1) THEN
CC        MTZIN = 1
C        DO 119 JDO120 = 1,NLPRGI
C          MTZ_LOOKUP(JDO120,NFILE) = JDO120
C          LOOKUP_WORK(JDO120) = MTZ_LOOKUP(JDO120,NFILE)
C  119   CONTINUE
C        END IF
C
C        IF(PROCESSING_CODE(NFILE).GT.1) THEN
        IF(PROCESSING_CODE(NFILE).GE.1) THEN
C
        NTOK = NPARM
        LINE = LABIN_SAVE(NFILE)
C
C            *******************************************************
        CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,
     +              LEND,.TRUE.)
C
C----  Always re-initialise the MTZLIB common blocks
C      Panic ---- is this enough to reset all data for LKYIN etc
C?????
        call mtzini
        CALL LKYIN(MTZIN,LSPRGI_NFILE,NLPRGI,NTOK,LINE,IBEG,IEND)
C            *******************************************************
C
C---- Transfer into Lookup_Work the LOOKUP values for
C     this Processing_Code
C
        DO 120 JDO120 = 1,NLPRGI
          LOOKUP_WORK(JDO120) = MTZ_LOOKUP(JDO120,NFILE)
  120   CONTINUE
        END IF
C
C---- get logical name for input file
C     can use same file several times!!
C
        WRITE (FNAME,FMT='(I1)') NFILE
        MTZ_FILE_NAME = 'HKLIN'//FNAME
C
C            *****************************************
        CALL LROPEN(MTZIN,MTZ_FILE_NAME,MTZPRT,MTZERR)
        IF(PROCESSING_CODE(NFILE).EQ.1)  THEN
         nl = 0
        END IF
        IF(PROCESSING_CODE(NFILE).GT.1)  THEN
         nl = -720
        END IF
C            *****************************************
C
        IF (MTZERR.EQ.-1) THEN
          WRITE (LUNOUT,FMT=*) ' Error in LROPEN for file name ',
     +      ' Input_File_Name: ',MTZ_FILE_NAME
          CALL CCPERR(1, ' Fatal Error')
        END IF

C     Inform LRIDX on available space
        NDATASETS(NFILE) = MAXSETS
C---- Read in list of datasets in input file
        CALL LRIDX(MTZIN,PNAME(1,NFILE),XNAME(1,NFILE),
     +            DNAME(1,NFILE),ISETS(1,NFILE),DATCELL(1,1,NFILE),
     +            DATWAVE(1,NFILE),NDATASETS(NFILE))

C---- Update cell dimensions from keyworded input if requested 
C     (DCELL <setid>)
        IF (NDATASETS(NFILE).GT.0 .AND. GOT_DCELL(NFILE)) THEN
          DO ISET = 1,NDATASETS(NFILE)

           IF (ISETS(ISET,NFILE).GT.0 .AND.
     +         ISETS(ISET,NFILE).LE.MAXSETS) THEN
            IF (DCELL_OUT(1,ISETS(ISET,NFILE),NFILE).GT.0.0) THEN
              DO I = 1,6
                DATCELL(I,ISET,NFILE) = 
     +              DCELL_OUT(I,ISETS(ISET,NFILE),NFILE)
              ENDDO
            END IF
           ENDIF

          ENDDO
        ENDIF

C---- Update wavelengths from keyworded input if requested
C     (DWAVE <setid>)
        IF (NDATASETS(NFILE).GT.0 .AND. GOT_DWAVE(NFILE)) THEN
          DO ISET = 1,NDATASETS(NFILE)

           IF (ISETS(ISET,NFILE).GT.0 .AND.
     +         ISETS(ISET,NFILE).LE.MAXSETS) THEN
            IF (DWAVE_OUT(ISETS(ISET,NFILE),NFILE).GT.0.0) THEN
              DATWAVE(ISET,NFILE) = 
     +           DWAVE_OUT(ISETS(ISET,NFILE),NFILE)
            END IF
           ENDIF

          ENDDO
        ENDIF

C---- Update pname from keyworded input if requested
C     (DPNAME <setid>)
        IF (NDATASETS(NFILE).GT.0 .AND. GOT_DPNAME(NFILE)) THEN
          DO ISET = 1,NDATASETS(NFILE)

           IF (ISETS(ISET,NFILE).GT.0 .AND.
     +         ISETS(ISET,NFILE).LE.MAXSETS) THEN
            IF (DRENAME_PNAME_OUT(ISETS(ISET,NFILE),NFILE).NE.' ') THEN
              PNAME(ISET,NFILE) = 
     +           DRENAME_PNAME_OUT(ISETS(ISET,NFILE),NFILE)
            END IF
           ENDIF

          ENDDO
        ENDIF

C---- Update xname/dname from keyworded input if requested (DRENAME)
        IF (NDATASETS(NFILE).GT.0 .AND. GOT_DRENAME(NFILE)) THEN
          DO ISET = 1,NDATASETS(NFILE)

            DO I = 1,MAXSETS
              IF (DRENAME_XNAME(I,NFILE).EQ.XNAME(ISET,NFILE)
     +       .AND.DRENAME_DNAME(I,NFILE).EQ.DNAME(ISET,NFILE)) THEN
                XNAME(ISET,NFILE)=DRENAME_BYNAME_XNAME_OUT(I,NFILE)
                DNAME(ISET,NFILE)=DRENAME_BYNAME_DNAME_OUT(I,NFILE)
              ENDIF
            ENDDO

           IF (ISETS(ISET,NFILE).GT.0 .AND.
     +         ISETS(ISET,NFILE).LE.MAXSETS) THEN
            IF (DRENAME_XNAME_OUT(ISETS(ISET,NFILE),NFILE).NE.' ') THEN
              XNAME(ISET,NFILE) = 
     +           DRENAME_XNAME_OUT(ISETS(ISET,NFILE),NFILE)
            END IF
            IF (DRENAME_DNAME_OUT(ISETS(ISET,NFILE),NFILE).NE.' ') THEN
              DNAME(ISET,NFILE) = 
     +           DRENAME_DNAME_OUT(ISETS(ISET,NFILE),NFILE)
            END IF
           ENDIF

          ENDDO

        ENDIF

C            ********************************************
        CALL LRASSN(MTZIN,LSPRGI_NFILE,NLPRGI,LOOKUP_WORK,
     +              CTPRGI_NFILE(1,NFILE))
        CALL LRINFO(MTZIN,VERSNX,NUMCOL,NREFLX,RNGMTZ)
C            ********************************************

C---- Get dataset IDs for file columns
        IF (NDATASETS(NFILE).GT.0) 
     +    CALL LRCLID(MTZIN,CSETID_NFILE(1,NFILE),NUMCOL)

C---- If XNAM and DNAM cards were given, get new dataset names

        DO ICOL = 1,NUMCOL
          XNAME_NEW(ICOL) = ' '
          XNAME_LOOKUP(ICOL) = 0
        ENDDO

        IF (GOT_XNAME(NFILE)) THEN
C---- UNPACK XNAME_SAVE line into XNAME_NEW and XNAME_LOOKUP
C
          NTOK = NPARM
          LINE = XNAME_SAVE(NFILE)
C
C              *******************************************************
          CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,
     +                LEND,.TRUE.)
C              *******************************************************
C
          ITOK = 2
          PDNAMCHK = ' '
 127      CONTINUE

          IF (LINE(IBEG(ITOK):IBEG(ITOK)+2) .EQ.'ALL' .OR.
     +        LINE(IBEG(ITOK):IBEG(ITOK)+2) .EQ.'all' ) THEN

            PDNAMCHK = LINE(IBEG(ITOK+1):IEND(ITOK+1))
            DO ICOL = 1,NUMCOL
              XNAME_NEW(ICOL) = PDNAMCHK
              XNAME_LOOKUP(ICOL) = -1
            ENDDO
            GOTO 128

          ELSEIF (LINE(IBEG(ITOK):IBEG(ITOK)+2) .EQ.'HKL' ) THEN

            PDNAMCHK = LINE(IBEG(ITOK+1):IEND(ITOK+1))
            DO ICOL = 1,3
              XNAME_NEW(ICOL) = PDNAMCHK
              XNAME_LOOKUP(ICOL) = -1
            ENDDO
            ITOK = ITOK + 2
            IF (ITOK.LE.NTOK) GOTO 127
 
          ELSEIF (LINE(IBEG(ITOK):IBEG(ITOK)) .EQ.'E' ) THEN

            ILENGTH = IEND(ITOK) - IBEG(ITOK)
            IF (ILENGTH.EQ.1)
     +         READ(LINE(IBEG(ITOK)+1:IEND(ITOK)),'(I1)') ICOL1
            IF (ILENGTH.EQ.2)
     +         READ(LINE(IBEG(ITOK)+1:IEND(ITOK)),'(I2)') ICOL1
            IF (ILENGTH.EQ.3)
     +         READ(LINE(IBEG(ITOK)+1:IEND(ITOK)),'(I3)') ICOL1
            ICOL2 = ICOL1
            ITOK = ITOK + 1
C--- Allow for column ranges
            IF (LINE(IBEG(ITOK):IBEG(ITOK)+2) .EQ.'TO ' .OR.
     +          LINE(IBEG(ITOK):IBEG(ITOK)+2) .EQ.'to ' ) THEN
              ITOK = ITOK + 1
              ILENGTH = IEND(ITOK) - IBEG(ITOK)
              IF (ILENGTH.EQ.1)
     +          READ(LINE(IBEG(ITOK)+1:IEND(ITOK)),'(I1)') ICOL2
              IF (ILENGTH.EQ.2)
     +          READ(LINE(IBEG(ITOK)+1:IEND(ITOK)),'(I2)') ICOL2
              IF (ILENGTH.EQ.3)
     +          READ(LINE(IBEG(ITOK)+1:IEND(ITOK)),'(I3)') ICOL2
              ITOK = ITOK + 1
            ENDIF
            PDNAMCHK = LINE(IBEG(ITOK):IEND(ITOK))
            DO ICOL = ICOL1,ICOL2
C--- E1 is 4th program label, etc.
              XNAME_NEW(ICOL+3) = PDNAMCHK
              XNAME_LOOKUP(ICOL+3) = -1
            ENDDO
            ITOK = ITOK + 1
            IF (ITOK.LE.NTOK) GOTO 127

          ELSE

            CALL CCPERR(1,'Syntax error on XNAME line!')

          ENDIF

        ENDIF

 128    CONTINUE

        DO ICOL = 1,NUMCOL
          DNAME_NEW(ICOL) = ' '
          DNAME_LOOKUP(ICOL) = 0
        ENDDO

        IF (GOT_DNAME(NFILE)) THEN
C---- UNPACK DNAME_SAVE line into DNAME_NEW and DNAME_LOOKUP
C
          NTOK = NPARM
          LINE = DNAME_SAVE(NFILE)
C
C              *******************************************************
          CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,
     +                LEND,.TRUE.)
C              *******************************************************
C
          ITOK = 2
          PDNAMCHK = ' '
 132      CONTINUE
          IF (LINE(IBEG(ITOK):IBEG(ITOK)+2) .EQ.'ALL' .OR.
     +        LINE(IBEG(ITOK):IBEG(ITOK)+2) .EQ.'all' ) THEN

            PDNAMCHK = LINE(IBEG(ITOK+1):IEND(ITOK+1))
            DO ICOL = 1,NUMCOL
              DNAME_NEW(ICOL) = PDNAMCHK
              DNAME_LOOKUP(ICOL) = -1
            ENDDO
            GOTO 133

          ELSEIF (LINE(IBEG(ITOK):IBEG(ITOK)+2) .EQ.'HKL' ) THEN

            PDNAMCHK = LINE(IBEG(ITOK+1):IEND(ITOK+1))
            DO ICOL = 1,3
              DNAME_NEW(ICOL) = PDNAMCHK
              DNAME_LOOKUP(ICOL) = -1
            ENDDO
            ITOK = ITOK + 2
            IF (ITOK.LE.NTOK) GOTO 132
 
          ELSEIF (LINE(IBEG(ITOK):IBEG(ITOK)) .EQ.'E' ) THEN

            ILENGTH = IEND(ITOK) - IBEG(ITOK)
            IF (ILENGTH.EQ.1)
     +         READ(LINE(IBEG(ITOK)+1:IEND(ITOK)),'(I1)') ICOL1
            IF (ILENGTH.EQ.2)
     +         READ(LINE(IBEG(ITOK)+1:IEND(ITOK)),'(I2)') ICOL1
            IF (ILENGTH.EQ.3)
     +         READ(LINE(IBEG(ITOK)+1:IEND(ITOK)),'(I3)') ICOL1
            ICOL2 = ICOL1
            ITOK = ITOK + 1
C--- Allow for column ranges
            IF (LINE(IBEG(ITOK):IBEG(ITOK)+2) .EQ.'TO ' .OR.
     +          LINE(IBEG(ITOK):IBEG(ITOK)+2) .EQ.'to ' ) THEN
              ITOK = ITOK + 1
              ILENGTH = IEND(ITOK) - IBEG(ITOK)
              IF (ILENGTH.EQ.1)
     +          READ(LINE(IBEG(ITOK)+1:IEND(ITOK)),'(I1)') ICOL2
              IF (ILENGTH.EQ.2)
     +          READ(LINE(IBEG(ITOK)+1:IEND(ITOK)),'(I2)') ICOL2
              IF (ILENGTH.EQ.3)
     +          READ(LINE(IBEG(ITOK)+1:IEND(ITOK)),'(I3)') ICOL2
              ITOK = ITOK + 1
            ENDIF
            PDNAMCHK = LINE(IBEG(ITOK):IEND(ITOK))
            DO ICOL = ICOL1,ICOL2
C--- E1 is 4th program label, etc.
              DNAME_NEW(ICOL+3) = PDNAMCHK
              DNAME_LOOKUP(ICOL+3) = -1
            ENDDO
            ITOK = ITOK + 1
            IF (ITOK.LE.NTOK) GOTO 132

          ELSE

            CALL CCPERR(1,'Syntax error on DNAME line!')

          ENDIF

        ENDIF

 133    CONTINUE

C  Check consistency of crystals with those from previous files
C  Note, this comes after the effects of DRENAME keyword.
C  It comes after DCELL <i> but before DCELL <xname> <dname>
C  It also doesn't cover new crystals added by XNAME.
C  It has to be done here so that XNAME_OUT gets correct names.
        DO JFILE = 1, NFILE - 1
          DO JSET = 1, NDATASETS(JFILE)
            XNAME_CHECK = XNAME(JSET,JFILE)
            IF (XNAME_CHECK.EQ.'HKL_base') GOTO 134
            DO ISET = 1,NDATASETS(NFILE)
              IF (XNAME(ISET,NFILE).EQ.XNAME_CHECK) THEN
C  Duplicate crystal name - cell dimensions must agree
                CALL CELLCHK(DATCELL(1,ISET,NFILE),
     +                       DATCELL(1,JSET,JFILE),0.0001,JERR)
                IF (JERR.NE.0) THEN
                  WRITE(6,'(A,A,A,I4,A,/,A,I4)') ' Warning: crystal ',
     +      XNAME(ISET,NFILE)(1:LENSTR(XNAME(ISET,NFILE))),' of file ',
     +      NFILE,
     +      ' has different cell dimensions to corresponding',
     +      ' crystal of file ',JFILE
                  XNAME(ISET,NFILE) = 
     +      XNAME(ISET,NFILE)(1:LENSTR(XNAME(ISET,NFILE)))//'renamed'
                  WRITE(6,'(A,A)')
     +      ' Renamed to ',
     +      XNAME(ISET,NFILE)(1:LENSTR(XNAME(ISET,NFILE)))
                  WRITE(6,'(A,A,/,A)')
     +       ' If this is in error, use Edit MTZ Datasets task, to ',
     +       ' edit cell dimensions and/or ',
     +       ' transfer columns to correct dataset. '
                  CALL CCPERR(2,
     +        'Duplicate crystal renamed on basis of cell dimensions.')
                ENDIF
              ENDIF
            ENDDO
 134        CONTINUE
          ENDDO
        ENDDO

C---- Assign crystal name/dataset name for output columns
        NTOTDATASETS = NDATASETS(NFILE)
        NSTART = 4
        IF (NFILE.EQ.1) NSTART = 1
C---- Cycle through program labels that are assigned
        DO 115 JDO115 = NSTART,NLPRGI
          IF (LOOKUP_WORK(JDO115).EQ.0) GO TO 115
          NUM2_COLS_OUT = NUM2_COLS_OUT + 1
C---- If input file had datasets assigned, match dataset ID to names
          IF (NDATASETS(NFILE).GT.0) THEN
            SETID = CSETID_NFILE(LOOKUP_WORK(JDO115),NFILE)
            DO ISET = 1,NDATASETS(NFILE)
              IF (ISETS(ISET,NFILE).EQ.SETID) THEN
                XNAME_OUT(NUM2_COLS_OUT) = XNAME(ISET,NFILE)
                DNAME_OUT(NUM2_COLS_OUT) = DNAME(ISET,NFILE)
              END IF
            ENDDO
          END IF
C---- Were new or replacement names given in input cards (XNAME/DNAME) for
C     either crystal name or dataset name?
          IF (XNAME_LOOKUP(JDO115).EQ.-1.OR.
     +        DNAME_LOOKUP(JDO115).EQ.-1) THEN
             IF (XNAME_LOOKUP(JDO115).EQ.-1) 
     +         XNAME_OUT(NUM2_COLS_OUT) = XNAME_NEW(JDO115)(1:64)
             IF (DNAME_LOOKUP(JDO115).EQ.-1)
     +         DNAME_OUT(NUM2_COLS_OUT) = DNAME_NEW(JDO115)(1:64)
C---- Is this an existing dataset?
             DO ISET = 1,NTOTDATASETS
               IF (XNAME_OUT(NUM2_COLS_OUT).EQ.XNAME(ISET,NFILE)
     +        .AND.DNAME_OUT(NUM2_COLS_OUT).EQ.DNAME(ISET,NFILE)) THEN
                 IISET = ISET
                 GOTO 116 
               ENDIF
             ENDDO
C----  Add in new dataset. Library will remove duplicates.
             NTOTDATASETS = NTOTDATASETS + 1
             IISET = NTOTDATASETS
             XNAME(NTOTDATASETS,NFILE) = XNAME_OUT(NUM2_COLS_OUT)(1:64)
             DNAME(NTOTDATASETS,NFILE) = DNAME_OUT(NUM2_COLS_OUT)(1:64)
 116         CONTINUE
             CSETID_NFILE(LOOKUP_WORK(JDO115),NFILE) = IISET
          ENDIF
 115    CONTINUE
        NDATASETS(NFILE) = NTOTDATASETS

C---- Update pname from keyworded input if requested
C     (DPNAME <xname> <dname>)
        IF (NDATASETS(NFILE).GT.0 .AND. GOT_DPNAME(NFILE)) THEN
          DO ISET = 1,NDATASETS(NFILE)

            DO I = 1,MAXSETS
              IF (DPNAME_XNAME(I,NFILE).EQ.XNAME(ISET,NFILE)
     +       .AND.DPNAME_DNAME(I,NFILE).EQ.DNAME(ISET,NFILE)) THEN
                PNAME(ISET,NFILE)=DRENAME_BYNAME_PNAME_OUT(I,NFILE)
              ENDIF
            ENDDO

          ENDDO
        ENDIF

C---- Update cell dimensions from keyworded input if requested
C     (DCELL <xname> <dname>)
        IF (NDATASETS(NFILE).GT.0 .AND. GOT_DCELL(NFILE)) THEN
          DO ISET = 1,NDATASETS(NFILE)

            DO I = 1,MAXSETS
              IF (DCELL_XNAME(I,NFILE).EQ.XNAME(ISET,NFILE)
     +       .AND.DCELL_DNAME(I,NFILE).EQ.DNAME(ISET,NFILE)) THEN
                DO II = 1,6
                  DATCELL(II,ISET,NFILE)=DCELL_BYNAME(II,I,NFILE)
                ENDDO
              ENDIF
            ENDDO

          ENDDO
        ENDIF

C---- Update wavelengths from keyworded input if requested
C     (DWAVE <xname> <dname>)
        IF (NDATASETS(NFILE).GT.0 .AND. GOT_DWAVE(NFILE)) THEN
          DO ISET = 1,NDATASETS(NFILE)

            DO I = 1,MAXSETS
              IF (DWAVE_XNAME(I,NFILE).EQ.XNAME(ISET,NFILE)
     +       .AND.DWAVE_DNAME(I,NFILE).EQ.DNAME(ISET,NFILE)) THEN
                DATWAVE(ISET,NFILE)=DWAVE_BYNAME(I,NFILE)
              ENDIF
            ENDDO

          ENDDO
        ENDIF
C
C----   If FCPART PHIPART (CTYPIN U and V), increase number of columns output
C       and copy XNAME, DNAME. This assumes that FCPART PHIPART are the
C       last two columns for this file, which is a documented requirement.
C
        IF(PROCESSING_CODE(NFILE) .EQ.3) THEN
          I_FCPART_COL = NUM2_COLS_OUT - 1
          DO ISYM_NUMBER = 2,NSYMP_MASTER
            NUM2_COLS_OUT = NUM2_COLS_OUT + 1
            XNAME_OUT(NUM2_COLS_OUT) = XNAME_OUT(I_FCPART_COL)
            DNAME_OUT(NUM2_COLS_OUT) = DNAME_OUT(I_FCPART_COL)
            NUM2_COLS_OUT = NUM2_COLS_OUT + 1
            XNAME_OUT(NUM2_COLS_OUT) = XNAME_OUT(I_FCPART_COL+1)
            DNAME_OUT(NUM2_COLS_OUT) = DNAME_OUT(I_FCPART_COL+1)
          ENDDO
        END IF

C End of dataset section

           do ifil = 1,maxfiles
             do icol = 1,MCOLS
              ncolgf(icol,ifil) = 0
              ncolql(icol,ifil) = 0
              ncolki(icol,ifil) = 0
              nset(icol,ifil) = 0
             end do
           end do
c
C          200HHHGLGLFQDQ
C
C----  Check for F+ F-, or I+ I- or Q+ Q- pairs
      nanogf = 0
      nanoql = 0
      nanoki = 0
      do  i = 1,nlprgi
C  Anomalous difference
         if (ctprgi_nfile(i,nfile) .eq. 'G') then
C  F+ F- pair; store column number ( labelled G)
            nanogf = nanogf + 1
            ncolgf(nanogf,nfile) = i
         elseif (ctprgi_nfile(i,nfile) .eq. 'L'
     +       .or.ctprgi_nfile(i,nfile) .eq. 'M') then
C  Q+ Q- pair; store column number ( labelled L (SIGF) or M(SIGI))
            nanoql = nanoql + 1
            ncolql(nanoql,nfile) = i
         elseif (ctprgi_nfile(i,nfile) .eq. 'K') then
C  I+ I- pair; store column number
            nanoki = nanoki + 1
            ncolki(nanoki,nfile) = i
         endif
      end do
C
C--- Check there are pairs of ncolgf and ncolki for the same dataset
       ichk = mod(nanogf,2)
         if(ichk .ne. 0) write (6, '(/a/)')
     $        ' **** Something WRONG with F+ F- pairing ****'
         if(ichk .ne. 0) call ccperr(1,' **** Check data set ****')
C
       ichk = mod(nanoql,2)
         if(ichk .ne. 0) write (6, '(/a/)')
     $        ' **** Something WRONG with Q+ Q- pairing ****'
         if(ichk .ne. 0) call ccperr(1,' **** Check data set ****')
C
       ichk = mod(nanoki,2)
         if(ichk .ne. 0) write (6, '(/a/)')
     $        ' **** Something WRONG with I+ I- pairing ****'
         if(ichk .ne. 0) call ccperr(1,' **** Check data set ****')
C
C--- Match partners using dataset info, if available
       IF (NDATASETS(NFILE).GT.0) THEN

C--- Loop over F+ F- columns
       do icset = 1,nanogf
        if(nset(ncolgf(icset,nfile),nfile) .eq. 0) then
         do jcset = icset+1,nanogf
          if(csetid_nfile(LOOKUP_WORK(ncolgf(icset,nfile)),nfile) .eq. 
     +      csetid_nfile(LOOKUP_WORK(ncolgf(jcset,nfile)),nfile) ) then
            nset(ncolgf(icset,nfile),nfile) = ncolgf(jcset,nfile)
            nset(ncolgf(jcset,nfile),nfile) = ncolgf(icset,nfile)
            go to 52
          end if
         end do
        end if
 52     continue
C  Has this worked?
        if(nset(ncolgf(icset,nfile),nfile) .eq. 0) then
         write (6, '(/a,i3,a/)')
     +        ' **** No match for F+/F- column ',icset,' ****'
         call ccperr(1,' **** Check data set ****')
        else
        write(6, '(/a,i3,3x,i3/)')
     +  ' Column pairs- F+ F-:',ncolgf(icset,nfile), 
     +                           nset(ncolgf(icset,nfile),nfile)
        end if
       end do

C--- Loop over Q+ Q- columns
       do icset = 1,nanoql
        if(nset(ncolql(icset,nfile),nfile) .eq. 0) then
         do jcset = icset+1,nanoql
          if(csetid_nfile(LOOKUP_WORK(ncolql(icset,nfile)),nfile) .eq.
     +       csetid_nfile(LOOKUP_WORK(ncolql(jcset,nfile)),nfile) .and.
     +       ctprgi_nfile(LOOKUP_WORK(ncolql(icset,nfile)),nfile)
     +  .eq. ctprgi_nfile(LOOKUP_WORK(ncolql(icset,nfile)),nfile)) then
            nset(ncolql(icset,nfile),nfile) = ncolql(jcset,nfile)
            nset(ncolql(jcset,nfile),nfile) = ncolql(icset,nfile)
            go to 54
          end if
         end do
        end if
 54     continue
C  Has this worked?
        if(nset(ncolql(icset,nfile),nfile) .eq. 0) then
         write (6, '(/a,i3,a/)')
     +        ' **** No match for Q+/Q- column ',icset,' ****'
         call ccperr(1,' **** Check data set ****')
        else
        write(6, '(/a,i3,3x,i3/)')
     +  ' Column pairs - SIGs:',ncolql(icset,nfile), 
     +                           nset(ncolql(icset,nfile),nfile)
        end if
       end do

C--- Loop over I+ I- columns
       do icset = 1,nanoki
        if(nset(ncolki(icset,nfile),nfile) .eq. 0) then
         do jcset = icset+1,nanoki
          if(csetid_nfile(LOOKUP_WORK(ncolki(icset,nfile)),nfile) 
     + .eq. csetid_nfile(LOOKUP_WORK(ncolki(jcset,nfile)),nfile) ) then
            nset(ncolki(icset,nfile),nfile) = ncolki(jcset,nfile)
            nset(ncolki(jcset,nfile),nfile) = ncolki(icset,nfile)
            go to 56
          end if
         end do
        end if
 56     continue
C  Has this worked?
        if(nset(ncolki(icset,nfile),nfile) .eq. 0) then
         write (6, '(/a,i3,a/)')
     +        ' **** No match for I+/I- column ',icset,' ****'
         call ccperr(1,' **** Check data set ****')
        else
        write(6, '(/a,i3,3x,i3/)')
     +  ' Column pairs - I+ I-:',ncolki(icset,nfile), 
     +                           nset(ncolki(icset,nfile),nfile)
        end if
       end do

       ENDIF
C
        DO 125 I=4,NUMCOL
          IF (RNGMTZ(1,I) .LE. VRSET_MAGIC) CALL CCPERR(1,
     +    ' *** Datum less than VRSET_MAGIC, it MUST be changed ***')
  125   CONTINUE
 
       WRITE (LUNOUT,FMT=6004) FNAME,VERSNX,FNAME,NUMCOL,NREFLX
 6004   FORMAT (/,
     +   ' Reading from HKLIN mtz file_Number = :',A,/,
     +   ' This file written with MTZLIB Version Number : ',A,/,
     +   ' File HKLIN',A,' contains a total of ',I6,' Columns',/,
     +   ' and a total of ',I10,' Reflections')
C
C            *******************************************
        CALL LRTITL(MTZIN,TITIN(NFILE),LENTIT)
c Must set NHISIN to max allowed number of listory lines
        NHISIN(NFILE) = MAXHIS
        CALL LRHIST(MTZIN,HISTIN(1,NFILE),NHISIN(NFILE))
C            *******************************************
C
        IF (DO_MASTER) THEN
C
C              ********************
          CALL LRCELL(MTZIN,CELMTZ)
C              ********************
C  Get ready for possible anisotropic scaling
           CALL CALC_RCELL(CELMTZ,RCELMTZ)
           RMETRIC(1)=RCELMTZ(1)*RCELMTZ(1)
           RMETRIC(2)=RCELMTZ(2)*RCELMTZ(2)
           RMETRIC(3)=RCELMTZ(3)*RCELMTZ(3)
           RMETRIC(4)=2.0*RCELMTZ(1)*RCELMTZ(2)
           RMETRIC(5)=2.0*RCELMTZ(1)*RCELMTZ(3)
           RMETRIC(6)=2.0*RCELMTZ(2)*RCELMTZ(3)
C
          DO 130 JDO130 = 1,6
            CELOUT(JDO130) = CELMTZ(JDO130)
  130     CONTINUE
C
C
C---- set up cell parameters for calculation of SST
C
C              ********************************************************
          CALL SETRSL(CELOUT(1),CELOUT(2),CELOUT(3),CELOUT(4),CELOUT(5),
     +                CELOUT(6))
C              ********************************************************
C
C---- set up for systematic absences
C
C              *************************************
          CALL EPSLN(NSYM_MASTER,NSYMP_MASTER,RSYM_MASTER,MTZPRT)
C              *************************************
C
C---- set up for Centric zones
C
C              ***************************************
          CALL CENTRIC(NSYM_MASTER,RSYM_MASTER,MTZPRT)
C              ***************************************
C
C---- sort info - is this ever going to be used ??
C
C                ***************************
            CALL LRSORT(MTZIN,MSORTX_MASTER)
C                ***************************
C
        END IF
C
C---- Finished with Do_Master
C
        IF (DO_MASTER) DO_MASTER = .FALSE.
C
C---- Now Read Input data in column order assigned
C
  160   CONTINUE
C
        USED_REF = .FALSE.
C
C
C---- return the record in Lookup order
C
C            **************************
        CALL LRREFF(MTZIN,SSQ0, RECIN,MTZEOF)
C            **************************
C
        IF (MTZEOF) GO TO 260
C   Get logical flags to see if a DATA item is "missing" or not.
        NUM_READ(NFILE) = NUM_READ(NFILE) + 1
C
C---- Get HKL input
C
        DO 180 JDO180 = 1,3
          HKL_IN(JDO180) = NINT(RECIN(JDO180))
  180   CONTINUE
C
C---- Ignore Input S and recalculate it
C     only do this on output??
C
C                   *************************************
        SSQ0 = 4.0*STHLSQ(HKL_IN(1),HKL_IN(2),HKL_IN(3))
C                   *************************************
C
C---- Check if within resolution limits
C
        IF ((SSQ0 - S_MIN(NFILE)).LT.0.0) THEN
          NUM_REJECT_SST(NFILE) = NUM_REJECT_SST(NFILE) + 1
          GO TO 160
        ELSE IF ((SSQ0 - S_MAX(NFILE)).GT.0.0) THEN
          NUM_REJECT_SST(NFILE) = NUM_REJECT_SST(NFILE) + 1
          GO TO 160
        END IF
C
        CALL LRREFM(MTZIN, RECIN_MISS)
c        write(6,'(3i4)')
c     + hkl_in
c        write(6,*)
c     + (recin(jdo180),RECIN_MISS(jdo180),jdo180=4,7)
C
C---- Check if Centric
C
C            ****************************
        CALL CENTR(HKL_IN, IS_IT_CENTRIC)
C            ****************************
C
        IF (IS_IT_CENTRIC.EQ.1) THEN
          IF (NMON.GT.0) THEN
          WRITE (LUNOUT,FMT=6006) MTZ_FILE_NAME, HKL_IN
 6006     FORMAT (' For file ',A,' the reflection: ',3I6,
     +           /' is Centric.')
          END IF
        ELSE
          NUM_NON_CENTRIC(NFILE) = NUM_NON_CENTRIC(NFILE) + 1
          IF (USE_CENTRIC) GO TO 160
        END IF
C
C---- Check if reflection is in  asymmetric unit
C     defined by call to ASUSET for
C     point group defined by SYMOUT
C
C----  The inverse of real-space symmetry matrices are applied by
C      premultiplying them  by a row vector hkl,
C      ie  (h'k'l') = (hkl)[R] inv
C
CEJD  - this is a BIIIGGG loop - may pick up more than 1 equiv h k l..
        DO 250 ISYM_NUMBER = 1,NSYMP_MASTER
C
C---- First Initialise ALL possible output columns
C
C
        DO 170 JDO170 = 4,NUM_COLS_OUT
          RECOUT(JDO170) = VRSET_MAGIC
  170   CONTINUE
C
C----  h' = h R   ie row vector h premultiplies symmetry matrix
C
          DO 200 JDO200 = 1,3
         HKL_UNIQUE(JDO200) = 
     +       HKL_IN(1)*NINT(RSYMT_MASTER(1,JDO200,ISYM_NUMBER)) +
     +       HKL_IN(2)*NINT(RSYMT_MASTER(2,JDO200,ISYM_NUMBER)) +
     +       HKL_IN(3)*NINT(RSYMT_MASTER(3,JDO200,ISYM_NUMBER))
  200     CONTINUE
C
C
C
C---- Test this index against the asymmetric unit
C     Function INASU returns 0 if outside au,
C     else +-1 depending on sign
C
          HKL_SAVE_SIGN = 0
          HM = 0
C
          IF(NASU_CODE.LT.0)THEN
           IF(HKL_UNIQUE(1) .GE. HMIN_ASU .AND. 
     +        HKL_UNIQUE(1) .LE. HMAX_ASU .AND.
     +        HKL_UNIQUE(2) .GE. KMIN_ASU .AND.
     +        HKL_UNIQUE(2) .LE. KMAX_ASU .AND.
     +        HKL_UNIQUE(3) .GE. LMIN_ASU .AND.
     +        HKL_UNIQUE(3) .LE. LMAX_ASU)
     +        HKL_SAVE_SIGN = 1
C  Need to consider cases where HKL_SAVE_SIGN = both 1 and -1 
                HM = 0
           IF(-HKL_UNIQUE(1) .GE. HMIN_ASU .AND. 
     +        -HKL_UNIQUE(1) .LE. HMAX_ASU .AND.
     +        -HKL_UNIQUE(2) .GE. KMIN_ASU .AND.
     +        -HKL_UNIQUE(2) .LE. KMAX_ASU .AND.
     +        -HKL_UNIQUE(3) .GE. LMIN_ASU .AND.
     +        -HKL_UNIQUE(3) .LE. LMAX_ASU) THEN
              HM = -1
              IF(HKL_SAVE_SIGN .NE. 1 ) THEN 
                HKL_SAVE_SIGN = HM
                HM = 0
              END IF
           END IF
          END IF

          IF (NLAUE_ASU.GT.0) THEN
            HKL_SAVE_SIGN = CCP4SPG_F_INASU(1,HKL_UNIQUE)
          ELSE IF (NASU_CODE.GT.0) THEN
            HKL_SAVE_SIGN = INASU(HKL_UNIQUE,NASU_CODE)
          END IF
C
          IF (HKL_SAVE_SIGN.EQ.0) GO TO 250
C
C---- Succesful, set USED_REF = true , multiply by sign
 205      CONTINUE
C
          DO 210 I = 1,3
            HKL_UNIQUE(I) = HKL_UNIQUE(I)*HKL_SAVE_SIGN
  210     CONTINUE
C
C
          USED_REF = .TRUE.
C
C---- Check if request to remove systematic absences
C     Must have first called  subroutine epsln(nsm,rsm,iprint)
C
C              **************************************************
          CALL EPSLON(HKL_UNIQUE, REF_MULT_FACTOR, IS_IT_SYS_ABS)
C              **************************************************
C
          IF (IS_IT_SYS_ABS.EQ.1) THEN
            NUM_SYS_ABSENT(NFILE) = NUM_SYS_ABSENT(NFILE) + 1
            IF(NO_KEEP_SYSABS) 
     +      WRITE(6,'(A,3I4)') ' Systematic absent reflection rejected',
     +        HKL_UNIQUE
            IF (NO_KEEP_SYSABS) GO TO 160
            WRITE(6,'(A,3I4)') ' Systematic absent reflection kept',
     +        HKL_UNIQUE
          END IF
C
C
225       CONTINUE
C
C---- set output to scratch file columns
C
          DO 220 JDO260 = 1,3
            HKL_OUT(JDO260) = HKL_UNIQUE(JDO260)
            RECOUT(JDO260) = REAL(HKL_OUT(JDO260))
  220     CONTINUE
C
C---- Now process this reflection
C
          IRECIN = 4
          IRECOUT = OP_COLS_STORE(NFILE)
C
C---- columns returned from HKLINx in lookup order
C
          JDO240CHK = 0
C  Get Isotropic or aniso Scaleexp(- B * (sin theta/lambda)**2) for these Fs, Ds, Qs
            SCAL = 1.0
          IF(TEMP_ALL(NFILE).GT.-998.0)
     +    SCAL = SCALE_ALL(NFILE)*exp(-0.25*TEMP_ALL(NFILE)*SSQ0)
          IF(TEMP_ANIS(1,NFILE).GT.-998.0) THEN
            XX = TEMP_ANIS(1,NFILE)*HKL_IN(1)*HKL_IN(1)*RMETRIC(1) +
     +           TEMP_ANIS(2,NFILE)*HKL_IN(2)*HKL_IN(2)*RMETRIC(2) +
     +           TEMP_ANIS(3,NFILE)*HKL_IN(3)*HKL_IN(3)*RMETRIC(3) +
     +           TEMP_ANIS(4,NFILE)*HKL_IN(1)*HKL_IN(2)*RMETRIC(4) +
     +           TEMP_ANIS(5,NFILE)*HKL_IN(1)*HKL_IN(3)*RMETRIC(5) +
     +           TEMP_ANIS(6,NFILE)*HKL_IN(2)*HKL_IN(3)*RMETRIC(6)  
            SCAL = SCALE_ALL(NFILE)*exp(-XX)
          ELSE 
             SCAL = SCALE_ALL(NFILE)
          ENDIF

          DO 240 JDO240 = 4,NLPRGI
C   Only here to skip 4 JDO240 when HLA coefficients given
            IF (JDO240 .LE. JDO240CHK) GO TO 240
            IF (MTZ_LOOKUP(JDO240,NFILE).EQ.0) GO TO 240
C   In general IRECIN = JDO240   Maybe there is some exception?
C   Cant check here - some anomalous columns may need to be swapped.
C            IF (RECIN_MISS(JDO240)) GO TO 230
C
C
C----  Check Possible Anomalous pairs first: G K L or M
C
            IF (CTPRGI_NFILE(JDO240,NFILE).EQ.'G' .OR.
     +          CTPRGI_NFILE(JDO240,NFILE).EQ.'K' .OR.
     +          CTPRGI_NFILE(JDO240,NFILE).EQ.'L' .OR.
     +          CTPRGI_NFILE(JDO240,NFILE).EQ.'M') THEN
              IF (HKL_SAVE_SIGN.EQ.-1) THEN 
C  If RECIN(NSET.. is a MNF then we cannot apply SCALE
               IF (RECIN_MISS(NSET(JDO240,NFILE)) ) GO TO 230 
                RECOUT(IRECOUT) = 
     +             SCAL*RECIN(NSET(IRECIN,NFILE))
              ELSE
                IF ( RECIN_MISS(JDO240))  GO TO 230
                RECOUT(IRECOUT) = SCAL*RECIN(IRECIN)
              ENDIF
              GO TO 230
            ENDIF
C
C  Now check for MNF for all others and proceed as before..

            IF (RECIN_MISS(JDO240)) GO TO 230
C----  check ctprgi and act appropriately
C      phase.
C
            IF (CTPRGI_NFILE(JDO240,NFILE).EQ.'P') THEN
              PHASE_IN = RECIN(IRECIN)
C
C                  **************************************************
              CALL ASUPHP(HKL_OUT,ISYM_NUMBER,HKL_SAVE_SIGN,PHASE_IN,
     +                    PHASE_OUT)
C                  **************************************************
C
              RECOUT(IRECOUT) = PHASE_OUT
              GO TO 230
            END IF
C
C--- Hendrickson Lattman: need to know phase change.
C  A_out = A_in *COS(  PhiDiff) - B * Sin(  PhiDiff)
C  B_out = A_in *SIN(  PhiDiff) + B * Cos(  PhiDiff)
C  C_out = C_in *COS(2*PhiDiff) - D * Sin(2*PhiDiff)
C  D_out = C_in *SIN(2*PhiDiff) + D * Cos(2*PhiDiff)
C
C  Check all HLA HLB HLC HLD are present, in this file and in order
            IF (CTPRGI_NFILE(JDO240  ,NFILE).EQ.'A'  .AND.
     +          CTPRGI_NFILE(JDO240+1,NFILE).EQ.'A'  .AND.
     +          CTPRGI_NFILE(JDO240+2,NFILE).EQ.'A'  .AND.
     +          CTPRGI_NFILE(JDO240+3,NFILE).EQ.'A'  ) THEN
              HLA_IN = RECIN(IRECIN)
              HLB_IN = RECIN(IRECIN+1)
              HLC_IN = RECIN(IRECIN+2)
              HLD_IN = RECIN(IRECIN+3)
C   If there has been a change of hand need to convert to A, -B, C -D
              HLB_IN =  HKL_SAVE_SIGN *RECIN(IRECIN+1)
              HLD_IN =  HKL_SAVE_SIGN *RECIN(IRECIN+3)
              PHASE_IN = 0.0
C
C                  **************************************************
              CALL ASUPHP(HKL_OUT,ISYM_NUMBER,HKL_SAVE_SIGN,PHASE_IN,
     +                    PHASE_OUT)
C                  **************************************************
              PHIDIFF = PHASE_OUT -PHASE_IN
              COSD = COS(CONV*PHIDIFF)
              SIND = SIN(CONV*PHIDIFF)
              COS2D = COS(2*CONV*PHIDIFF)
              SIN2D = SIN(2*CONV*PHIDIFF)
C
              RECOUT(IRECOUT  ) = HLA_IN*COSD  - HLB_IN*SIND
              RECOUT(IRECOUT+1) = HLA_IN*SIND  + HLB_IN*COSD
              RECOUT(IRECOUT+2) = HLC_IN*COS2D - HLD_IN*SIN2D
              RECOUT(IRECOUT+3) = HLC_IN*SIN2D + HLD_IN*COS2D
              JDO240CHK = JDO240 + 3
              IRECIN  = IRECIN + 3
              IRECOUT = IRECOUT + 3
              GO TO 230
            END IF
C
C----  anomalous difference.
C
            IF (CTPRGI_NFILE(JDO240,NFILE).EQ.'D') THEN
              RECOUT(IRECOUT) = SCAL*
     +                          RECIN(IRECIN)*HKL_SAVE_SIGN
              GO TO 230
            END IF
C
C----  B part .
C
            IF (CTPRGI_NFILE(JDO240,NFILE).EQ.'B') THEN
              RECOUT(IRECOUT) = SCAL*
     +                          RECIN(IRECIN)*HKL_SAVE_SIGN
              GO TO 230
            END IF
C
C----  FCPART
C
            IF (CTPRGI_NFILE(JDO240,NFILE).EQ.'U') THEN
              RECOUT(IRECOUT+2*ISYM_NUMBER-2) = 
     +                        SCAL*RECIN(IRECIN)
              GO TO 230
            END IF
C
C----  PHIPART
C
            IF (CTPRGI_NFILE(JDO240,NFILE).EQ.'V') THEN
              PHASE_IN = RECIN(IRECIN)
C
C                  **************************************************
C   24-feb 1992 - Dont change phase - only sign if necessary.
C  all translation program will deal with screw axes etc themselves.
C              CALL ASUPHP(HKL_OUT,ISYM_NUMBER,HKL_SAVE_SIGN,PHASE_IN,
C     +                    PHASE_OUT)
C                  **************************************************
C
             PHASE_OUT =  PHASE_IN
             IF(HKL_SAVE_SIGN.EQ.-1) PHASE_OUT = - PHASE_IN
             RECOUT(IRECOUT+2*ISYM_NUMBER-2) = PHASE_OUT
              GO TO 230
            END IF
C
C----  Anything else: apply scale to F J Q A (B D) only
C
            IF (CTPRGI_NFILE(JDO240,NFILE).EQ.'F' .OR.
     +          CTPRGI_NFILE(JDO240,NFILE).EQ.'J' .OR.
     +          CTPRGI_NFILE(JDO240,NFILE).EQ.'Q' .OR.
     +          CTPRGI_NFILE(JDO240,NFILE).EQ.'A') THEN
              RECOUT(IRECOUT) = SCAL*RECIN(IRECIN)
            ELSE
              RECOUT(IRECOUT) = RECIN(IRECIN)
            ENDIF
C
  230       CONTINUE
            IRECIN  = IRECIN + 1
            IRECOUT = IRECOUT + 1
  240     CONTINUE
C
C---- Pass keys and record to sort
C
          NUM_REF_OUT(NFILE) = NUM_REF_OUT(NFILE) + 1
C
C                 **************
          ISTAT = SRTRLS(RECOUT)
C                 **************
C
          IF (ISTAT.NE.0) THEN
            WRITE (LUNOUT,FMT=6008) ISTAT
 6008       FORMAT (
     +  ' CAD failed to release record to sort procedure, status = ',
     +    I9,/)
          CALL CCPERR(1, ' Fatal Error')
          END IF
C   Is HM still = -1 ? 
          IF(HM.EQ.-1) THEN
          HKL_SAVE_SIGN = HM
          HM = 0
          GO TO 205 
          END IF 
C
C----  Try next symmetry
C
  250   CONTINUE
C
C
        IF (.NOT.USED_REF) THEN
C
C---- Shouldn't get here, can't reduce reflection to asymmetric unit
C
          WRITE (LINERR,FMT='(A,3I4,A)')
     +      'ASUPUT: can''t put reflection ',HKL_IN,
     +      ' into asymmetric unit'
C
C---- Fatal error, stop in routine
C
C              *******************
          CALL LERROR(2,-1,LINERR)
C              *******************
C
        END IF
C
C---- Go back and get another reflection for the current input file
C
        GO TO 160
C
C---- Come here for end of an input file
C
  260   CONTINUE
C
C----  write out some stuff for this input file
C
        WRITE (LUNOUT,FMT=6010) FNAME,NUM_READ(NFILE),
     +                          NUM_SYS_ABSENT(NFILE),
     +                          NUM_REJECT_SST(NFILE),
     +                          NUM_REF_OUT(NFILE)
 6010   FORMAT (//,
     + ' For file HKLIN',A,/,
     + ' Total Number of HKL read                         = ',I10,/,
     + ' Total Number of HKL systematic absent            = ',I10,/,
     + ' Total Number of HKL reject for resolution limits = ',I10,/,
     + ' Total Number of HKL passed to sort for this file = ',I10)
C
C---- Close the input file, and reuse the MTZIN index = 1
C
C            *************
        CALL LRCLOS(MTZIN)
C            *************
C
  270 CONTINUE

C
C---- Stats on total number of reflections handled
C     from all input files
C
      DO 280 JDO280 = 1,NFILES_IN
        TOTAL_READ       = NUM_READ(JDO280)       + TOTAL_READ
        TOTAL_TOSORT     = NUM_REF_OUT(JDO280)    + TOTAL_TOSORT
        TOTAL_SYS_ABS    = NUM_SYS_ABSENT(JDO280) + TOTAL_SYS_ABS
        TOTAL_REJECT_SST = NUM_REJECT_SST(JDO280) + TOTAL_REJECT_SST
  280 CONTINUE
C
C---- write out some more info
C
      WRITE (LUNOUT,FMT=6012) TOTAL_READ,
     +                        TOTAL_TOSORT,
     +                        TOTAL_SYS_ABS,
     +                        TOTAL_REJECT_SST
 6012 FORMAT (//,
     + ' Total number read for all files    = ',I10,/,
     + ' Total passed to sort for all files = ',I10,/,
     + ' Total number systematic absent     = ',I10,/,
     + ' Total number rejected outside resol= ',I10)
C
C---- Summary of columns selected per file
C
      WRITE (LUNOUT,6112) 
     +      (II,COLS_OP_PER_FILEIN(II),II=1,NFILES_IN)
6112  FORMAT(//,' Number of columns per file:',/,
     +    (' File_Number = ',I4,' Number_Columns_Selected = ',I5,/))
C
C---- All records passed, now sort them
C
C             ********
      ISTAT = SRTMRG()
C             ********
C
      IF (ISTAT.NE.0) THEN
        WRITE (LUNOUT,FMT=6014) ISTAT
 6014   FORMAT (
     + ' CAD detected failure in merge phase of sort procedure,',
     + ' status = ',I9,/)
          CALL CCPERR(1, ' Fatal Error')
      END IF
C
C---- Now open final output file
C
      MTZOUT = 1
C
C          ***********************
      CALL LWOPEN(MTZOUT,'HKLOUT')
C          ***********************

C   Transfer "missing" value to output and overwrite any default
C   It may have come from  program input, or HKLIN1
          SETVAL = .TRUE.
          CALL SET_MAGIC(MTZOUT,VAL_MISSING,SETVAL)
C
C---- Build TITLE from MASTER File + ADD_TITLE
C     Final output TITLE
C
      IF (.NOT.DO_ADD_TITLE) THEN
        TITOUT = TITIN(1)
      ELSE
        LWORK1 = LENSTR(TITLE_ADD)
        LWORK2 = LENSTR(TITIN(1))
        IF (LWORK1.EQ.0) LWORK1 = 1
        IF (LWORK2.EQ.0) LWORK2 = 1
        TITOUT = TITLE_ADD(1:LWORK1)//' '//TITIN(1) (1:LWORK2)
      END IF
C
        LWORK1 = LENSTR(TITOUT)
        IF(LWORK1.GT.70) THEN
        TITOUT = TITOUT(1:70)
        END IF
C
      MTITOT = 0
C
C          ****************************
      CALL LWTITL(MTZOUT,TITOUT,MTITOT)
C          ****************************


C Store the project, crystal and dataset names in the mtz header:
      DO 285 NFILE = 1,NFILES_IN
       DO 286 ISET = 1,NDATASETS(NFILE)
        CALL LWIDX(MTZOUT,PNAME(ISET,NFILE),XNAME(ISET,NFILE),
     +    DNAME(ISET,NFILE),DATCELL(1,ISET,NFILE),DATWAVE(ISET,NFILE))
 286   CONTINUE
 285  CONTINUE
C
C---- Final o/p of history
C
      IF (.NOT.DO_NEW_HIST) THEN
C
C---- Just add master history lines If any?
C
        NHISOUT = 0
C
C
        IF (NHISIN(1).GT.0) THEN
          DO 290 JDO290 = 1,NHISIN(1)
            HISTOUT(JDO290) = HISTIN(JDO290,1)
            NHISOUT = NHISOUT + 1
  290     CONTINUE
        END IF
C
C
        IF (NHISOUT.LT.MAXHIS) THEN
          NHISOUT = NHISOUT + 1
          HISTOUT(NHISOUT) = ' data from CAD on '//MTZDATE
        END IF
C
C---- add new history lines
C
      ELSE
        NHISOUT = 0
        DO 300 JDO300 = 1,NUM_NEW_HIST
          NHISOUT = NHISOUT + 1
          HISTOUT(JDO300) = NEW_HIST(JDO300)
  300   CONTINUE
C
C
        IF (NHISOUT.LT.MAXHIS) THEN
          NHISOUT = NHISOUT + 1
          HISTOUT(NHISOUT) = ' data from CAD on '//MTZDATE
        END IF
C
C---- Add at end master history lines if any
C
        IF (NHISIN(1).GT.0) THEN
          DO 310 JDO310 = 1,NHISIN(1)
            IF (NHISOUT.EQ.MAXHIS) GO TO 320
            NHISOUT = NHISOUT + 1
            HISTOUT(NHISOUT) = HISTIN(JDO310,1)
  310     CONTINUE
        END IF
  320   CONTINUE
      END IF
C
C---- final write it out
C
C          ******************************
      CALL LWHIST(MTZOUT,HISTOUT,NHISOUT)
C          ******************************
C
C---- Final o/p symmetry information
C
      LATTYP_MASTER = NAMSPG_MASTER(1:1)
C
cc-help should this be MASTER or input from Key_Word ASUSPACEGRPOUP ??
C          ***************************************************
      CALL LWSYMM(MTZOUT,NSYM_MASTER,NSYMP_MASTER,RSYM_MASTER,
     +            LATTYP_MASTER,NSPGRP_MASTER,NAMSPG_MASTER,
     +            PGNAME_MASTER)
C          ***************************************************
C
C---- final o/p cell data
C
C          *********************
      CALL LWCELL(MTZOUT,CELOUT)
C          *********************
C
C---- final o/p sort data
          MSORTX_MASTER(1) = SORTKYS(1)
          MSORTX_MASTER(2) = SORTKYS(2)
          MSORTX_MASTER(3) = SORTKYS(3)
          MSORTX_MASTER(4) = 0
          MSORTX_MASTER(5) = 0
C
C          ****************************
      CALL LWSORT(MTZOUT,MSORTX_MASTER)
C          ****************************
C
C---- Now want column labels and column numbers for
C     output file
C
C
      NTOK = NPARM
      LINE = LABOUT_LINE
C
C          ************************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +            .TRUE.)
      CALL LKYOUT(MTZOUT,LSPRGO,NUM_COLS_OUT,NTOK,LINE,IBEG,IEND)
      CALL LWASSN(MTZOUT,LSPRGO,NUM_COLS_OUT,CTPRGO,MTZAPN)
      CALL LWIDASX(MTZOUT,NUM_COLS_OUT,XNAME_OUT,DNAME_OUT,MTZAPN)
C          ************************************************************
C
      NFROMSORT = 0
      NSAVE     = 0
C
C
  330 CONTINUE
C
C---- Now pick up records and output
C
C             *************
      ISTAT = SRTRET(RECIN)
C             *************
        NSAVE = NSAVE + 1
C
      IF (ISTAT.NE.ENDSRT) THEN
C
C
        IF (ISTAT.NE.0) THEN
C
C----    error in sort read back
C
          WRITE (LUNOUT,FMT=6016) ISTAT
 6016     FORMAT (
     +' CAD detected error on obtaining record from sort procedure'/
     +' in return phase, status = ',I4,/)
          CALL CCPERR(1, ' Fatal Error')
        END IF
C
C---- Now get all reflections with common HKL
C     and patch all the columns from RECIN into a single
C
C        NSAVE = NSAVE + 1
        NFROMSORT = NFROMSORT + 1
C
C
        IF (NSAVE.GT.MAXSAVE) THEN
          WRITE (LUNOUT,FMT=6018) NSAVE,MAXSAVE
 6018     FORMAT (
     + ' Error: ',I6,' reflections to be MERGED',/,
     + ' From the SORT routine: maximum of ',I6,' allowed')
          CALL CCPERR(1, ' Parameter MAXSAVE exceeded.')
        END IF
C
C
        DO 340 JDO340 = 1,NUM_COLS_OUT
          RECWORK(NSAVE,JDO340) = RECIN(JDO340)
  340   CONTINUE
      END IF
C
C
      IF (ISTAT.NE.ENDSRT) THEN
        IF (NSAVE.EQ.1) GO TO 330
C
C---- Have more than one record saved Test if lastest
C     record equal hkl to last saved record, if so
C     just store and get another
C
        IF (NINT(RECWORK(NSAVE,1)).EQ.NINT(RECWORK(NSAVE-1,1)) .AND.
     +      NINT(RECWORK(NSAVE,2)).EQ.NINT(RECWORK(NSAVE-1,2)) .AND.
     +      NINT(RECWORK(NSAVE,3)).EQ.NINT(RECWORK(NSAVE-1,3))) 
     +      GO TO 330
      END IF
C
C---- current record doesnt match (hkl) in recwork(nsave-1)
C     therefore build output record
C
      IF (NSAVE.EQ.2) THEN
C
C---- There is only one unique record
C
        DO 350 JDO350 = 1,NUM_COLS_OUT
          RECOUT(JDO350) = RECWORK(NSAVE-1,JDO350)
C
C---- move record in NSAVE to NSAVE -1
C
          RECWORK(NSAVE-1,JDO350) = RECWORK(NSAVE,JDO350)
  350   CONTINUE
C
C---- reset NSAVE counter
C
        NSAVE = 1
        GO TO 410
      END IF
C
C---- more than one record with the same HKL
C     now merge these
C
      IF (NSAVE.GT.2) THEN
C
C---- First transfer H K L S from recwork(1,n) to recout
C
        DO 360 JDO360 = 1,3
          RECOUT(JDO360) = RECWORK(1,JDO360)
  360   CONTINUE
C
C---- Now initialise remaining recout columns
C
        DO 370 JDO370 = 4,NUM_COLS_OUT
          RECOUT(JDO370) = VRSET_MAGIC
  370   CONTINUE
C
C---- Now pull in all columns from NSAVEdrecords; have to handle Nan seperately
C
        DO 390 JDO390 = 1, (NSAVE-1)
          DO 380 JDO380 = 4,NUM_COLS_OUT
            IF (RECWORK(JDO390,JDO380) .GT. VRSET_MAGIC) 
     +                  RECOUT(JDO380) = RECWORK(JDO390,JDO380)
  380     CONTINUE
  390   CONTINUE
C
C---- move last record in NSAVE to first save position 1
C
        DO 400 JDO400 = 1,NUM_COLS_OUT
          RECWORK(1,JDO400) = RECWORK(NSAVE,JDO400)
  400   CONTINUE
C
C---- reset save counter
C
        NSAVE = 1
        GO TO 410
      END IF
C
C---- now write it out
C
  410 CONTINUE
C
C---- Do you want to output a reflection where all elements have 
C-----    "missing value"? Do so if OUTPUT_MISS true.
C
      RECSUM = 0
C
      DO 420 JDO420 = 4,NUM_COLS_OUT
         IF( RECOUT(JDO420).GT.VRSET_MAGIC)  RECSUM = RECSUM + 1
  420 CONTINUE
C
      IF (RECSUM.LT.0.0000000005) THEN
        NUM_MISS = NUM_MISS + 1
        IF(.NOT.OUTPUT_MISS) GO TO 425
      END IF
C
C---- Reset MNF from VRSET_MAGIC to VAL_MISSING to write out
C
C          *******************************************************
      CALL RESET_MAGIC(0,RECOUT,RECIN,NUM_COLS_OUT,VRSET_MAGIC,
     +                                                VAL_MISSING)
C          *******************************************************
C
C---- Do you apply overall Resolution limits NOW
C
C          ********************
      CALL LWREFL(MTZOUT,RECIN)
C          ********************
C
      IF (NMON.GT.0) THEN
         JDO420 = MOD(NUM_TO_OUTPUT,NMON)
         CALL RESET_MAGIC(0,RECOUT,BDATA,NUM_COLS_OUT,VRSET_MAGIC,VALM)
         IF(JDO420.EQ.0) WRITE(LUNOUT,FMT= '(A,/,3F4.0,12F8.1)')
     +       ' Monitoring first 15 items',
     +         (BDATA(JDO400),JDO400 = 1,15)
      END IF
      NUM_TO_OUTPUT = NUM_TO_OUTPUT + 1 
C
 425  CONTINUE
C
      IF (ISTAT.NE.ENDSRT) GO TO 330
C
C---- end of sort reached
C     Close sort and end file
C     Close MTZOUT file
C
C
C          *********************
      CALL LWCLOS(MTZOUT,MTZPRT)
C          *********************
C
C---- Write out nfromsort
C
      WRITE (LUNOUT,FMT=6020) NFROMSORT, NUM_TO_OUTPUT,NUM_MISS
 6020 FORMAT (//,
     +        ' Total number from Sorting Routine = ',I10,/,
     +        ' Final Total of Unique records to HKLOUT = ',I10,/,
     +        ' Final Total of Missing records to HKLOUT = ',I10)
C
C
      CALL CCPERR(0,' *** Normal Termination of CAD ***')
      END
C
C
C
C     =================
      SUBROUTINE CADINI
C     =================
C
C
C     .. Parameters ..
      INTEGER MCOLS
      PARAMETER (MCOLS=500)
      INTEGER MAXSETS
      PARAMETER (MAXSETS=MCOLS)
      INTEGER MAXHIS
      PARAMETER (MAXHIS=30)
      INTEGER MAXFILES
      PARAMETER (MAXFILES=9)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
      INTEGER NLPRGI
      PARAMETER (NLPRGI=MCOLS)
      REAL VAL_MISSING
C   This will have to be set after first mtz file opened for reading
C      PARAMETER (VAL_MISSING=-9999.0)
C      PARAMETER (VAL_MISSING=0.0)
C     ..
C     .. Scalars in Common ..
C
C---- Asymmetric unit symmetry variables
C
      INTEGER NASU_CODE,NLAUE_ASU,
     +        HMIN_ASU,HMAX_ASU,
     +                  KMIN_ASU,KMAX_ASU,LMIN_ASU,LMAX_ASU
C
C
C---- Master symmetry variables
C
      INTEGER NSPGRP_MASTER,   NLAUE_MASTER,
     +        NSYMP_MASTER,    NSYM_MASTER
      CHARACTER NAMSPG_MASTER*10, PGNAME_MASTER*10,LATTYP_MASTER*1
      REAL    RSYMT_MASTER,    RSYM_MASTER
C
C
      REAL RESMAX_OVERALL,RESMIN_OVERALL,S_MAX_OVERALL,S_MIN_OVERALL
      INTEGER MTZPRT,NFILES_IN,NUM_NEW_HIST,NMON
      LOGICAL DO_ADD_TITLE,DO_NEW_HIST,USE_CENTRIC,USE_INPUT_SYMM,
     +        USE_MASTER_SYMM,NO_KEEP_SYSABS,OUTPUT_MISS,GOT_MISS
      CHARACTER TITLE_ADD*70
C     ..
C     .. Arrays in Common ..
      REAL S_MAX,S_MIN,DCELL_OUT,DCELL_BYNAME,DWAVE_OUT,
     +     DWAVE_BYNAME
      INTEGER PROCESSING_CODE, SORTKYS
      LOGICAL GOT_LABIN,GOT_LABOUT,GOT_XNAME,GOT_DNAME,
     +         GOT_DCELL,GOT_DWAVE,GOT_DRENAME,GOT_DPNAME
      CHARACTER CTPRGI_NFILE*1,LSPRGI_NFILE*30,NEW_HIST*80,
     +          LABIN_SAVE*2048,LABOUT_SAVE*2048,
     +          XNAME_SAVE*2048,DNAME_SAVE*2048,
     +         DCELL_XNAME*64,DCELL_DNAME*64,
     +         DWAVE_XNAME*64,DWAVE_DNAME*64,
     +         DRENAME_PNAME_OUT*64,DRENAME_XNAME_OUT*64,
     +         DRENAME_DNAME_OUT*64,
     +         DRENAME_XNAME*64,DRENAME_DNAME*64,
     +         DPNAME_XNAME*64,DPNAME_DNAME*64,
     +         DRENAME_BYNAME_PNAME_OUT*64,DRENAME_BYNAME_XNAME_OUT*64,
     +         DRENAME_BYNAME_DNAME_OUT*64
C     ..
C     .. Local Scalars ..
      INTEGER JJ
C     ..
C     .. Common blocks ..
      COMMON /ASU_INFO/ NASU_CODE,NLAUE_ASU,HMIN_ASU,HMAX_ASU,
     +                  KMIN_ASU,KMAX_ASU,LMIN_ASU,LMAX_ASU
C
C
      COMMON /ATSYM/  RSYM_MASTER(4,4,MAXSYM),
     +                RSYMT_MASTER(4,4,MAXSYM),
     +                NSYM_MASTER,
     +                NSYMP_MASTER,
     +                NSPGRP_MASTER,
     +                NLAUE_MASTER
      COMMON /ATSYM_CHAR/ NAMSPG_MASTER, PGNAME_MASTER, LATTYP_MASTER
C
C
      COMMON /CELLS/DCELL_OUT(6,MAXSETS,MAXFILES),
     +        DCELL_BYNAME(6,MAXSETS,MAXFILES),
     +        DCELL_XNAME(MAXSETS,MAXFILES),
     +        DCELL_DNAME(MAXSETS,MAXFILES),
     +        DWAVE_OUT(MAXSETS,MAXFILES),
     +        DWAVE_BYNAME(MAXSETS,MAXFILES),
     +        DWAVE_XNAME(MAXSETS,MAXFILES),
     +        DWAVE_DNAME(MAXSETS,MAXFILES),
     +        DRENAME_PNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_XNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_DNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_XNAME(MAXSETS,MAXFILES),
     +        DRENAME_DNAME(MAXSETS,MAXFILES),
     +        DPNAME_XNAME(MAXSETS,MAXFILES),
     +        DPNAME_DNAME(MAXSETS,MAXFILES),
     +        DRENAME_BYNAME_PNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_BYNAME_XNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_BYNAME_DNAME_OUT(MAXSETS,MAXFILES)
      COMMON /GENERAL/NFILES_IN,NUM_NEW_HIST,MTZPRT,
     +                NMON,VAL_MISSING
      COMMON /GENERAL_C/NEW_HIST(MAXHIS)
      COMMON /GENERAL_L/GOT_LABOUT(MAXFILES),GOT_LABIN(MAXFILES),
     +             GOT_XNAME(MAXFILES),
     +             GOT_DNAME(MAXFILES),
     +             GOT_DCELL(MAXFILES),
     +             GOT_DWAVE(MAXFILES),
     +             GOT_DRENAME(MAXFILES),
     +             GOT_DPNAME(MAXFILES),
     +       USE_MASTER_SYMM,USE_INPUT_SYMM,USE_CENTRIC,
     +       NO_KEEP_SYSABS,DO_NEW_HIST,DO_ADD_TITLE,OUTPUT_MISS,
     +       GOT_MISS
      COMMON /LSPRG_INFO/LSPRGI_NFILE(NLPRGI),
     +       CTPRGI_NFILE(NLPRGI,MAXFILES),LABIN_SAVE(MAXFILES),
     +       LABOUT_SAVE(MAXFILES),
     +       XNAME_SAVE(MAXFILES),DNAME_SAVE(MAXFILES)
      COMMON /PROCESS_INFO/   PROCESSING_CODE(MAXFILES),  
     +                        SCALE_ALL(MAXFILES),TEMP_ALL(MAXFILES),
     +                        TEMP_ANIS(6,MAXFILES)
      COMMON /RESO_INFO/RESMAX_OVERALL,RESMIN_OVERALL,S_MAX_OVERALL,
     +       S_MIN_OVERALL,S_MAX(MAXFILES),S_MIN(MAXFILES)
      COMMON /SORT_INFO/  SORTKYS(3)
      COMMON /TITL_INFO_C/TITLE_ADD
C     ..
      SAVE
C
      NSYM_MASTER  = 0
      NSYMP_MASTER = 0
      MTZPRT       = 1
      NFILES_IN    = 0
      NUM_NEW_HIST = 0
      NASU_CODE    = 0
      NLAUE_ASU    = 0
      NMON         = 0
      VAL_MISSING  = 0.0
      GOT_MISS = .FALSE.
      SORTKYS(1) = 1
      SORTKYS(2) = 2
      SORTKYS(3) = 3
C
C
      DO 10 JJ = 1,MAXFILES
        PROCESSING_CODE(JJ) = 0
        SCALE_ALL(JJ)       = 1
        TEMP_ALL(JJ)        = 0
        GOT_LABOUT(JJ)      = .FALSE.
        GOT_LABIN(JJ)       = .FALSE.
        GOT_XNAME(JJ)      = .FALSE.
        GOT_DNAME(JJ)       = .FALSE.
        GOT_DCELL(JJ)       = .FALSE.
        GOT_DWAVE(JJ)       = .FALSE.
        GOT_DRENAME(JJ)     = .FALSE.
        GOT_DPNAME(JJ)     = .FALSE.
        LABIN_SAVE(JJ)      = ' '
        LABOUT_SAVE(JJ)     = ' '
        XNAME_SAVE(JJ)      = ' '
        DNAME_SAVE(JJ)     = ' '
        S_MAX(JJ)        = 0.0
        S_MIN(JJ)        = 0.0
        DO KK = 1,MAXSETS
          DO LL = 1,6
            DCELL_OUT(LL,KK,JJ) = 0.0
            DCELL_BYNAME(LL,KK,JJ) = 0.0
          ENDDO
          DCELL_XNAME(KK,JJ) = ' '
          DCELL_DNAME(KK,JJ) = ' '
          DWAVE_OUT(KK,JJ) = 0.0
          DWAVE_BYNAME(KK,JJ) = 0.0
          DWAVE_XNAME(KK,JJ) = ' '
          DWAVE_DNAME(KK,JJ) = ' '
          DRENAME_PNAME_OUT(KK,JJ) = ' '
          DRENAME_XNAME_OUT(KK,JJ) = ' '
          DRENAME_DNAME_OUT(KK,JJ) = ' '
          DRENAME_XNAME(KK,JJ) = ' '
          DRENAME_DNAME(KK,JJ) = ' '
          DPNAME_XNAME(KK,JJ) = ' '
          DPNAME_DNAME(KK,JJ) = ' '
          DRENAME_BYNAME_PNAME_OUT(KK,JJ) = ' '
          DRENAME_BYNAME_XNAME_OUT(KK,JJ) = ' '
          DRENAME_BYNAME_DNAME_OUT(KK,JJ) = ' '
        ENDDO
   10 CONTINUE
C
C
      USE_MASTER_SYMM    = .TRUE.
      USE_INPUT_SYMM     = .FALSE.
      USE_CENTRIC        = .FALSE.
      NO_KEEP_SYSABS     = .TRUE.
      DO_NEW_HIST        = .FALSE.
      DO_ADD_TITLE       = .FALSE.
      OUTPUT_MISS        = .TRUE.
C
C
      DO 20 JJ = 1,MAXHIS
        NEW_HIST(JJ) = ' '
   20 CONTINUE
C
C
      RESMIN_OVERALL = 1000.0
      RESMAX_OVERALL = 0.10
      S_MIN_OVERALL  = 0.0
      S_MAX_OVERALL  = 10.0
C
C
      RETURN
      END
C
C
C     =================
      SUBROUTINE CADKEY
C     =================
C
C
C---- Read in the control data
C
C          implicit none
C
C
C     .. Parameters ..
      INTEGER MCOLS
      PARAMETER (MCOLS=500)
      INTEGER MAXSETS
      PARAMETER (MAXSETS=MCOLS)
      INTEGER MAXHIS
      PARAMETER (MAXHIS=30)
      INTEGER MAXFILES
      PARAMETER (MAXFILES=9)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
      INTEGER NPARM
      PARAMETER (NPARM=2*MCOLS)
      INTEGER NLPRGI
      PARAMETER (NLPRGI=MCOLS)
      INTEGER NKEYS
      PARAMETER (NKEYS=24)
C     ..
C     .. Scalars in Common ..
      REAL RESMAX_OVERALL,      RESMIN_OVERALL,
     +     S_MAX_OVERALL,    S_MIN_OVERALL
C
C
      INTEGER MTZPRT,          NFILES_IN,
     +        NUM_NEW_HIST
C
C---- Asymmetric unit symmetry variables
C
      INTEGER NLAUE_ASU,  NASU_CODE,   NSPGRP_ASU,
     +        NSYMP_ASU,  NSYM_ASU,
     +        HMIN_ASU,HMAX_ASU,
     +                  KMIN_ASU,KMAX_ASU,LMIN_ASU,LMAX_ASU
      CHARACTER NAMSPG_ASU*10,  PGNAME_ASU*10
      REAL      RSYM_ASU(4,4,MAXSYM)
C
C---- Master symmetry variables
C
      INTEGER NSPGRP_MASTER,   NLAUE_MASTER,
     +        NSYMP_MASTER,    NSYM_MASTER
      CHARACTER NAMSPG_MASTER*10, PGNAME_MASTER*10,LATTYP_MASTER*1
      REAL    RSYMT_MASTER,    RSYM_MASTER
C
C
      LOGICAL DO_ADD_TITLE,    DO_NEW_HIST,
     +        USE_CENTRIC,USE_INPUT_SYMM,
     +        USE_MASTER_SYMM, NO_KEEP_SYSABS,OUTPUT_MISS,GOT_MISS
      CHARACTER TITLE_ADD*70
C     ..
C     .. Arrays in Common ..
      REAL   S_MAX,        S_MIN,   
     +     SCALE_ALL,TEMP_ALL,TEMP_ANIS
C
C
      INTEGER   PROCESSING_CODE,SORTKYS
      LOGICAL   GOT_LABIN,GOT_LABOUT,GOT_XNAME,GOT_DNAME,
     +          GOT_DCELL,GOT_DWAVE,GOT_DRENAME,GOT_DPNAME
      CHARACTER CTPRGI_NFILE*1,   LSPRGI_NFILE*30,  NEW_HIST*80,
     +          LABIN_SAVE*2048,   LABOUT_SAVE*2048,
     +          XNAME_SAVE*2048,DNAME_SAVE*2048
C     ..
C     .. Local Scalars ..
      REAL RESMAX,  RESMIN,
     +     SMAX,SMIN,DCELL_OUT,DCELL_BYNAME,DWAVE_OUT,DWAVE_BYNAME
      INTEGER I,II,ITOK,IWORK,LUNOUT,NFILE,NMON,NTOK,FIRST_ARG
      LOGICAL LEND
      CHARACTER CWORK*4,KEY*4,LINE*2048,
     +         DCELL_XNAME*64,DCELL_DNAME*64,
     +         DWAVE_XNAME*64,DWAVE_DNAME*64,
     +         DRENAME_PNAME_OUT*64,DRENAME_XNAME_OUT*64,
     +         DRENAME_DNAME_OUT*64,
     +         DRENAME_XNAME*64,DRENAME_DNAME*64,
     +         DPNAME_XNAME*64,DPNAME_DNAME*64,
     +         DRENAME_BYNAME_PNAME_OUT*64,DRENAME_BYNAME_XNAME_OUT*64,
     +         DRENAME_BYNAME_DNAME_OUT*64
C     ..
C     .. Local Arrays ..
C
C---- Parser Stuff
C
      REAL      FVALUE(NPARM)
      INTEGER   IBEG(NPARM),IDEC(NPARM),IEND(NPARM),ITYP(NPARM)
      CHARACTER CVALUE(NPARM)*4
C
      INTEGER   CTYP_PROC_CODE(MAXFILES)
C
      CHARACTER CTCHECK(MCOLS)*1,  KEYWRD(NKEYS)*4,
     +          LAUNAM_ASU*10
C     ..
C     .. External Functions ..
      INTEGER LENSTR
      EXTERNAL LENSTR
      LOGICAL QISNAN
      EXTERNAL QISNAN
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPUPC,GTPINT, LCTPIN,PARSER,RDCELL,RDRESO, RDSYMM,
     + CCPERR
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC MAX
C     ..
C     .. Common blocks ..
      COMMON /ASU_INFO/ NASU_CODE,NLAUE_ASU,HMIN_ASU,HMAX_ASU,
     +                  KMIN_ASU,KMAX_ASU,LMIN_ASU,LMAX_ASU
C
C
      COMMON /ATSYM/  RSYM_MASTER(4,4,MAXSYM),
     +                RSYMT_MASTER(4,4,MAXSYM),
     +                NSYM_MASTER,
     +                NSYMP_MASTER,
     +                NSPGRP_MASTER,
     +                NLAUE_MASTER
      COMMON /ATSYM_CHAR/ NAMSPG_MASTER, PGNAME_MASTER, LATTYP_MASTER
C
C
      COMMON /CELLS/DCELL_OUT(6,MAXSETS,MAXFILES),
     +        DCELL_BYNAME(6,MAXSETS,MAXFILES),
     +        DCELL_XNAME(MAXSETS,MAXFILES),
     +        DCELL_DNAME(MAXSETS,MAXFILES),
     +        DWAVE_OUT(MAXSETS,MAXFILES),
     +        DWAVE_BYNAME(MAXSETS,MAXFILES),
     +        DWAVE_XNAME(MAXSETS,MAXFILES),
     +        DWAVE_DNAME(MAXSETS,MAXFILES),
     +        DRENAME_PNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_XNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_DNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_XNAME(MAXSETS,MAXFILES),
     +        DRENAME_DNAME(MAXSETS,MAXFILES),
     +        DPNAME_XNAME(MAXSETS,MAXFILES),
     +        DPNAME_DNAME(MAXSETS,MAXFILES),
     +        DRENAME_BYNAME_PNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_BYNAME_XNAME_OUT(MAXSETS,MAXFILES),
     +        DRENAME_BYNAME_DNAME_OUT(MAXSETS,MAXFILES)
      COMMON /GENERAL/NFILES_IN,NUM_NEW_HIST,MTZPRT,
     +                NMON,VAL_MISSING
      COMMON /GENERAL_C/NEW_HIST(MAXHIS)
      COMMON /GENERAL_L/GOT_LABOUT(MAXFILES),GOT_LABIN(MAXFILES),
     +             GOT_XNAME(MAXFILES),
     +             GOT_DNAME(MAXFILES),
     +             GOT_DCELL(MAXFILES),
     +             GOT_DWAVE(MAXFILES),
     +             GOT_DRENAME(MAXFILES),
     +             GOT_DPNAME(MAXFILES),
     +       USE_MASTER_SYMM,USE_INPUT_SYMM,USE_CENTRIC,
     +       NO_KEEP_SYSABS,DO_NEW_HIST,DO_ADD_TITLE,OUTPUT_MISS,
     +       GOT_MISS
      COMMON /LSPRG_INFO/LSPRGI_NFILE(NLPRGI),
     +       CTPRGI_NFILE(NLPRGI,MAXFILES),LABIN_SAVE(MAXFILES),
     +       LABOUT_SAVE(MAXFILES),
     +       XNAME_SAVE(MAXFILES),DNAME_SAVE(MAXFILES)
      COMMON /PROCESS_INFO/   PROCESSING_CODE(MAXFILES),  
     +                        SCALE_ALL(MAXFILES),TEMP_ALL(MAXFILES),
     +                        TEMP_ANIS(6,MAXFILES)
      COMMON /RESO_INFO/RESMAX_OVERALL,RESMIN_OVERALL,S_MAX_OVERALL,
     +       S_MIN_OVERALL,S_MAX(MAXFILES),S_MIN(MAXFILES)
      COMMON /SORT_INFO/  SORTKYS(3)
      COMMON /TITL_INFO_C/TITLE_ADD
C     ..
      SAVE
C     .. Data statements ..
      DATA CTYP_PROC_CODE/MAXFILES*0/
C
C---- Key words
C
      DATA KEYWRD/
     +         'TITL', 'CELL', 'RESO', 'OUTL', 'SYMM',
     +         'LABI', 'LABO', 'CTYP', 'SYSA', 'CENT','VALM'
     +        ,'REFM', 'HIST', 'MONI', 'END ', 'SORT','SCAL'
     +        ,'PNAM', 'XNAM', 'DNAM', 'DCEL', 'DWAV', 'DREN'
     +        ,'DPNA'/
C
C  ejd change to all the same
C      processing options 1  2  3
C  code = 1  - Master (all in)
C  code = 2  - standard - controlled by LABIN_NFILE
C  code = 3  - includes FCPART PHIPART
C
C   Use these to drive output. I have needed to introduce another
C   code to cover FCPART PHIPART.
C     CTPPRG
C      H K L BATCH M/ISYM I FRACTI RAT
C      H H H   B     Y    J   R     R
C      H K L F SIGF DEL PHI FOM W FCPART PHIPART
C      H H H F   Q   D   P   M  W   U       V
C
C
      LUNOUT = 6
      NSETC = 0
      NSETW = 0
      NSETR = 0
      NSETP = 0
C
C----  Default to calculate sfs and gradients in P1
C
   10 CONTINUE
      LINE = '   '
      KEY = ' '
      NTOK = NPARM
C
C          ********************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +            .TRUE.)
C          ********************************************************
C
C---- End of file?
C
      IF (LEND) GO TO 220
C
C---- Get chars of 1st token
C
      KEY(1:4) = LINE(IBEG(1) :IEND(1))
C
C---- Comment !(Extra check also in Parse)
C
      IF (KEY(1:1).EQ.'!') GO TO 10
      IF (KEY(1:1).EQ.'#') GO TO 10
C
C---- Convert to Uppercase
C
C          ***********
      CALL CCPUPC(KEY)
C          ***********
C---- Loop over possible key-words
C
      DO 20 I = 1,NKEYS
        IF (KEY.EQ.KEYWRD(I)) GO TO 30
   20 CONTINUE
C
C
      WRITE (LUNOUT,FMT=6000) LINE(1:LENSTR(LINE))
 6000 FORMAT (' The following line Not Understood:',/' ',A)
      GO TO 200
C
C
   30 CONTINUE
C
c      DATA KEYWRD/
c     +         'TITL CELL RESO OUTL SYMM',
c     +         'LABI LABO CTYP SYSA CENT','VALM'
c     +        ,'REFM','HIST','MONI','END ',SORT','SCAL',
C     +         'PNAM','XNAM','DNAM','DCEL','DWAV','DREN'/
C     +        ,'DPNA'/
C
      GO TO (     40,  50,  60,  70,  80,
     +           100, 110, 120, 150, 160, 165,
     +           170, 180, 210, 220, 190, 195, 
     +           500, 505, 510, 520, 530, 540,
     +           545) I
C
C---- TITLE
C     =====
C
   40 CONTINUE
      IF(NTOK.GE.2)TITLE_ADD    = LINE(IBEG(2) :IEND(NTOK))
      IF(NTOK.GE.2)DO_ADD_TITLE = .TRUE.
      GO TO 10
C
C---- CELL
C     =====
C
   50 CONTINUE

      WRITE (LUNOUT,FMT='(A,/,A)')
     +    ' CELL keyword obsolete - please use DCELL',
     +    ' Keyword ignored.'
      GO TO 10
C
C---- RESOLUTION
C     ==========
C
   60 CONTINUE
      ITOK = 2
C
C     RESOLUTION OVERALL       x1,x2
C     RESOLUTION FILE_NUMBER n x1,x2
C     ==============================
C
C     NOTE:    Default =  0.1 and 1000.0 Angstrom
C
C    DMAX, DMIN are the resolution limits for  the  data  to  be
C    included, i.e. data are included for which
C
C    (1/DMAX)**2 >= 4 sin**2theta/lambda**2 >= (1/DMIN)**2
C
C
C
      IF (NTOK.LE.2) THEN
C
C---- test if enough arguments
C
        WRITE (LUNOUT,FMT='(a,a)')
     +    ' Key_Word Error, Not Enough Tokens for  RESOLUTION ',
     +    'line, Ignoring this line.'
        GO TO 10
      END IF
C
C
      IF (ITYP(2).NE.1) THEN
C
C---- test if second argument is character
C
        WRITE (LUNOUT,FMT='(A,A,/,A,A,A,/,A)')
     +    ' Key_Word Error, Wrong type of input for  RESOLUTIO',
     +    'N line:-','  Second Argument  MUST be a Character ',
     +    'string, either OVER_ALL  or FILE_NUMBER, Ignoring th',
     +    'is line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF
C
C
      CWORK = LINE(IBEG(2) :IEND(2))
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
C---- see whether second argument = 'O' or 'F'
C
      IF (CWORK(1:1).EQ.'O') THEN
C
C---- Over_all Resolution being input
C
        ITOK = 3
C
C            ********************************************************
        CALL RDRESO(ITOK,ITYP,FVALUE,NTOK,RESMIN_OVERALL,
     +              RESMAX_OVERALL,S_MIN_OVERALL,S_MAX_OVERALL)
C            ********************************************************
C
        GO TO 10
      END IF
C
C
      IF (CWORK(1:1).EQ.'F') THEN
C
C---- second argument = 'FILE_NUMBER'
C     test if third argument is integer
C
        IF (ITYP(3).NE.2) THEN
          WRITE (LUNOUT,FMT='(A,A,/,A,A,A,/,A)')
     +      ' Key_Word Error, Wrong type of input for  RESOLUT',
     +      'ION line:-',' If Sub_KeyWord = FILE_NUMBER  then ',
     +      'third  Argument MUST be an Integer  Ignoring this',
     +      ' line -',LINE(1:LENSTR(LINE))
          GO TO 10
        END IF
C
C---- Resolution For an input file being given
C     Get File_Number and check if valid
C
C            ********************************
        CALL GTPINT(3,NFILE,NTOK,ITYP,FVALUE)
C            ********************************
C
C---- Check if allowed range of files ... 1 to 5 (is this enough ???)
C
        IF (NFILE.LE.0 .OR. NFILE.GT.MAXFILES) THEN
          WRITE (LUNOUT,FMT='(A,/,A,I4,A,I2,A,/,A)')
     +      ' Key_Word Error, For RESOLUTION FILE_NUMBER line:-',
     +      '  File_Number Given = ',NFILE,
     +      ' Is invalid only files 1 to ',MAXFILES,
     +      ' Allowed, Ignoring this line -',LINE(1:LENSTR(LINE))
          GO TO 10
        END IF
C
C---- Get Resolution for File_Number
C
        ITOK = 4
C
C            *******************************************
        CALL RDRESO(ITOK,ITYP,FVALUE,NTOK,RESMIN,RESMAX,
     +              SMIN,SMAX)
C            *******************************************
C
        S_MAX(NFILE) = SMAX
        S_MIN(NFILE) = SMIN
        GO TO 10
      END IF
C
C---- OUTLIM SPACEGROUP or OUTLIM HKL  
C     ==========
C
   70 CONTINUE
C
      IF (NTOK.LE.2) THEN
C
C---- test if enough arguments
C
        WRITE (LUNOUT,FMT='(a,a)')
     +    ' Key_Word Error, Not Enough Tokens for  OUTLIM ',
     +    'line, Ignoring this line.'
        GO TO 10
      END IF
C
C
      IF (ITYP(2).NE.1) THEN
C
C---- test if second argument is character
C
        WRITE (LUNOUT,FMT='(A,A,/,A,A,A,/,A)')
     +    ' Key_Word Error, Wrong type of input for  OUTLIM',
     +    ' line:-','  Second Argument  MUST be a Character ',
     +    'string, either SPACEGROUP  or HKL, Ignoring th',
     +    'is line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF
C
C
      CWORK = LINE(IBEG(2) :IEND(2))
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
C---- see whether second argument = 'S' or 'H'
        ITOK = 3
C
      IF (CWORK(1:1).EQ.'S') THEN
C
C---- Asummetric unit for a SPACEGROUP defined.
C
C          *********************************************************
      CALL RDSYMM(ITOK,LINE,IBEG,IEND,ITYP,FVALUE,NTOK,NAMSPG_ASU,
     +            NSPGRP_ASU,PGNAME_ASU,NSYM_ASU,NSYMP_ASU,RSYM_ASU)
C          *********************************************************
C
C---- Set up symmetry (RDSYMM must be called first )

        CALL CCP4SPG_F_LOAD_BY_OPS(1,NSYM_ASU,RSYM_ASU)
        CALL CCP4SPG_F_GET_LAUE(1,NLAUE_ASU,LAUNAM_ASU)
        NASU_CODE    = NLAUE_ASU
      END IF 


      IF (CWORK(1:1).EQ.'H') THEN
C
C---- Read limits for H K and L - hmin hmax kmin kmax lmin lmax.
C
      IF(NTOK .LT.8) CALL CCPERR(1,' Error on OUTLIM line')
        CALL GTPINT(ITOK  ,HMIN_ASU,NTOK,ITYP,FVALUE)
        CALL GTPINT(ITOK+1,HMAX_ASU,NTOK,ITYP,FVALUE)
        CALL GTPINT(ITOK+2,KMIN_ASU,NTOK,ITYP,FVALUE)
        CALL GTPINT(ITOK+3,KMAX_ASU,NTOK,ITYP,FVALUE)
        CALL GTPINT(ITOK+4,LMIN_ASU,NTOK,ITYP,FVALUE)
        CALL GTPINT(ITOK+5,LMAX_ASU,NTOK,ITYP,FVALUE)
        NASU_CODE    = -999
      END IF 
      GO TO 10
C
C---- SYMMETRY
C     ========
C
   80 CONTINUE
      ITOK = 2
C
C          ************************************************************
      CALL RDSYMM(ITOK,LINE,IBEG,IEND,ITYP,FVALUE,NTOK,NAMSPG_MASTER,
     +            NSPGRP_MASTER,PGNAME_MASTER,NSYM_MASTER,NSYMP_MASTER,
     +            RSYM_MASTER)
C          ************************************************************
C
C
Cejd wrong!!      NASU_CODE       = 0
      USE_INPUT_SYMM  = .TRUE.
      USE_MASTER_SYMM = .FALSE.
      GO TO 10
C
C---- LABIN   (File_Number)  "a=b c=d ...."
C     ==========
C
  100 CONTINUE
      CWORK = LINE(IBEG(2) :IEND(2))
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
C
      IF (CWORK(1:1).EQ.'F') THEN
C
C---- second argument = 'FILE_NUMBER'
C     test if third argument is integer
C
        IF (ITYP(3).NE.2) THEN
          WRITE (LUNOUT,FMT='(A,A,/,A,A,A,/,A)')
     +      ' Key_Word Error, Wrong type of input for  LABIN',' line:-',
     +      ' If Sub_KeyWord = FILE_NUMBER  then ',
     +      'third  Argument MUST be an Integer  Ignoring this',
     +      ' line -',LINE(1:LENSTR(LINE))
          GO TO 10
        END IF
C
C---- For an input file being given
C     Get File_Number and check if valid
C
C            ********************************
        CALL GTPINT(3,NFILE,NTOK,ITYP,FVALUE)
C            ********************************
C
C       Point to the token with the first label
        FIRST_ARG = 4
C
      ELSE
C
C---- Second argument wasn't FILE - set the file number to be
C     the first file by default and warn the user
C     This is an undocumented feature to allow backwards compatibility
C     with Arp/Warp 6.0.
C
        NFILE = 1
C       Point to the token with the first label
        FIRST_ARG = 2
        CALL CCPERR(2,'LABIN line missing FILE number')
        WRITE (LUNOUT,FMT='(/,a,i2,/)')
     +       ' File_Number defaulted to ',NFILE
C
      ENDIF
C
      NFILES_IN = MAX(NFILES_IN,NFILE)
C
C---- Check if allowed range of files ... 1 to 5 (is this enough ???)
C
      IF (NFILE.LE.0 .OR. NFILE.GT.MAXFILES) THEN
        WRITE (LUNOUT,FMT='(a,/,a,i4,a,i2,a,/,a)')
     +       ' Key_Word Error, For LABIN FILE_NUMBER line:-',
     +       '  File_Number Given = ',NFILE,
     +       ' Is invalid only files 1 to ',MAXFILES,
     +       ' Allowed, Ignoring this line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF
C
C---- Check is the next key word ALL?
C
C
C---- Now save labels for this input file
C
      LABIN_SAVE(NFILE) = 'LABIN  '//LINE(IBEG(FIRST_ARG) :IEND(NTOK))
      GOT_LABIN(NFILE)  = .TRUE.
C
C----      processing options 1  2  3
C
C  code = 1  - Master (all in)
C  code = 2  - standard - controlled by LABIN_NFILE
C  code = 3  - includes FCPART PHIPART  ( requires CTYP card)
C
      CWORK = LINE(IBEG(FIRST_ARG) :IEND(FIRST_ARG))
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
C----   If labi keyword is ALL - this is to be the MASTER file
C       and we will set PROCESSING_CODE(NFILE) = 1
C
      IF (CWORK(1:3).EQ.'ALL') THEN
        PROCESSING_CODE(NFILE) = 1
      ELSE
        PROCESSING_CODE(NFILE) = 2
      END IF
C
C
      GO TO 10
C
C---- LABOUT  
C     ======
C
  110 CONTINUE
C
C---- Now save labels for this input file
C
      CWORK = LINE(IBEG(2) :IEND(2))
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
      IF (CWORK(1:1).EQ.'F') THEN
C
C---- second argument = 'FILE_NUMBER'
C     test if third argument is integer
C
        IF (ITYP(3).NE.2) THEN
          WRITE (LUNOUT,FMT='(A,A,/,A,A,A,/,A)')
     +      ' Key_Word Error, Wrong type of input for  LABOUT',
     +      ' line:-',' If Sub_KeyWord = FILE_NUMBER  then ',
     +      'third  Argument MUST be an Integer  Ignoring this',
     +      ' line -',LINE(1:LENSTR(LINE))
          GO TO 10
        END IF
C
C---- For an input file being given
C     Get File_Number and check if valid
C
C            ********************************
        CALL GTPINT(3,NFILE,NTOK,ITYP,FVALUE)
C            ********************************
C
C       Point to the token with the first label
        FIRST_ARG = 4
C
      ELSE
C
C---- Second argument wasn't FILE - set the file number to be
C     the first file by default and warn the user
C     This is an undocumented feature to allow backwards compatibility
C     with Arp/Warp 6.0.
C
        NFILE = 1
C       Point to the token with the first label
        FIRST_ARG = 2
        CALL CCPERR(2,'LABOUT line missing FILE number')
        WRITE (LUNOUT,FMT='(/,a,i2,/)')
     +       ' File_Number defaulted to ',NFILE
C
      ENDIF
C
C---- Check if allowed range of files ... 1 to 5 (is this enough ???)
C
      IF (NFILE.LE.0 .OR. NFILE.GT.MAXFILES) THEN
        WRITE (LUNOUT,FMT='(a,/,a,i4,a,i2,a,/,a)')
     +       ' Key_Word Error, For LABOUT FILE_NUMBER line:-',
     +       '  File_Number Given = ',NFILE,
     +       ' Is invalid only files 1 to ',MAXFILES,
     +       ' Allowed, Ignoring this line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF
C
C
      LABOUT_SAVE(NFILE) = 'LABOUT '//LINE(IBEG(FIRST_ARG) :IEND(NTOK))
      GOT_LABOUT(NFILE)  = .TRUE.
      GO TO 10
C
C---- CTYPIN FILE_NUMBER n Ei=??? Ei+1=??? Ei+2=??? etc
C     ==================================================
C
  120 CONTINUE
      CWORK = LINE(IBEG(2) :IEND(2))
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
C
      IF (CWORK(1:1).EQ.'F') THEN
C
C---- second argument = 'FILE_NUMBER'
C     test if third argument is integer
C
        IF (ITYP(3).NE.2) THEN
          WRITE (LUNOUT,FMT='(a,a,/,a,a,a,/,a)')
     +      ' Key_Word Error, Wrong type of input for  CTYPIN',
     +      ' line:-',' If Sub_KeyWord = FILE_NUMBER  then ',
     +      'third  Argument MUST be an Integer  Ignoring this',
     +      ' line -',LINE(1:LENSTR(LINE))
          GO TO 10
        END IF
C
C---- For an input file being given
C     Get File_Number and check if valid
C
C            ********************************
        CALL GTPINT(3,NFILE,NTOK,ITYP,FVALUE)
C            ********************************
C
C---- Check if allowed range of files ... 1 to 5 (is this enough ???)
C
        IF (NFILE.LE.0 .OR. NFILE.GT.MAXFILES) THEN
          WRITE (LUNOUT,FMT='(a,/,a,i4,a,i2,a,/,a)')
     +      ' Key_Word Error, For CTYPIN FILE_NUMBER line:-',
     +      '  File_Number Given = ',NFILE,
     +      ' Is invalid only files 1 to ',MAXFILES,
     +      ' Allowed, Ignoring this line -',LINE(1:LENSTR(LINE))
          GO TO 10
        END IF
      END IF
C
      CTYP_PROC_CODE(NFILE) = 1
C
      DO 130 II = 1,NLPRGI
        CTCHECK(II) = ' '
  130 CONTINUE
C
C
      ITOK = 4
C
C          ************************************************************
      CALL LCTPIN(LSPRGI_NFILE,NLPRGI,CTCHECK,ITOK,NTOK,LINE,IBEG,IEND)
C          ************************************************************
C
C----   Set FCPART flag  - processing code =3
C       Check if Column type U and V set - if so be ready for FCPART..
C       Also check there is only one FCPART PHIPART for each file,
C       and they are consecutive types.
C
      DO 140 II = 1,NLPRGI
C       Only overwrite if set
        IF (CTCHECK(II).NE.' ') CTPRGI_NFILE(II,NFILE) = CTCHECK(II)
        IF (CTPRGI_NFILE(II,NFILE).NE.'U') GO TO 140
C
C
        IF (CTPRGI_NFILE(II,NFILE).EQ.'U') THEN
          IF (CTCHECK(II+1).NE.'V') THEN
            WRITE (LUNOUT,FMT='(a)')
     +     '  Illegal column types - FCpart must be followed by PHIpart'
          CALL CCPERR(1, ' Fatal Error')
          END IF
C
C
          IF (PROCESSING_CODE(NFILE).EQ.3) THEN
            WRITE (LUNOUT,FMT='(a)')
     +    '  Illegal column types - Only one FCpart etc  per Input file'
          CALL CCPERR(1, ' Fatal Error')
          END IF
C
            WRITE (LUNOUT,FMT='(/,a,a,/,a,a)')
     +  '  FCpart PHICpart input - Symmetry is that of POINT GROUP ',
     +  ' only: NO phase changes applied.',
     +  ' It is assumed that all translation program will deal',
     +  ' with screw axes etc themselves.'
C
          PROCESSING_CODE(NFILE) = 3
        END IF
  140 CONTINUE
C
C
      GO TO 10
C
C---- SYSAB_KEEP    keep systematic absences
C     ======================================
C
  150 CONTINUE
      NO_KEEP_SYSABS = .FALSE.
      GO TO 10
C
C---- VALMISSING  - Reset from default.  Request don't OUTPUT missing data
C     ============
C     e.g.   VALM NaN NOOUTPUT
C
 165  CONTINUE
        
      KEY = LINE(IBEG(2):IEND(2))
      CALL CCPUPC(KEY)
      GOT_MISS = .TRUE.
      IF(KEY(1:3).EQ.'NAN') THEN
         CALL QNAN(VAL_MISSING)
         WRITE (LUNOUT,FMT='(a,F12.3)') 
     +   '  Missing observations flagged as NAN'
      ELSE
         CALL GTPREA(2,VAL_MISSING,NTOK,ITYP,FVALUE)
         WRITE (LUNOUT,FMT='(a,F12.3)') 
     +   '  Missing observations flagged as ',VAL_MISSING
      END IF
C
      IF (NTOK.GT.2) THEN
         KEY = LINE(IBEG(3):IEND(3))
         CALL CCPUPC(KEY)
         IF(KEY(1:4).EQ.'NOOU') OUTPUT_MISS=.FALSE.
         WRITE (LUNOUT,FMT='(a)')
     +   '    Indices not output if all data items flagged "missing"'
      END IF

      GO TO 10
C
C---- CENTRIC_ONLY  only output centric terms.
C     ============
C
  160 CONTINUE
      USE_CENTRIC = .TRUE.
      WRITE (LUNOUT,FMT=*) '  Centric reflections only'
      GO TO 10
C
C---- REFMONITOR
C     ==========
C
C     Monitoring NUMBER _ program prints lots of information
C     about every nmon th reflection
C
  170 CONTINUE
C
C          *******************************
      CALL GTPINT(2,NMON,NTOK,ITYP,FVALUE)
C          *******************************
C
      WRITE (LUNOUT,FMT=6002) NMON
 6002 FORMAT ('  Monitoring',i5)
      GO TO 10
C
C---- HISTORY "string"
C     ================

  180 CONTINUE
C
C---- History strings to be added to  mtz o/p file HKLOUT
C
      NUM_NEW_HIST = NUM_NEW_HIST + 1
C
C
      IF (NUM_NEW_HIST.GT.MAXHIS) THEN
        WRITE (LUNOUT,FMT='(A,/,A,I6,A)')
     +    ' Sorry HISTORY now full no more lines accepted','   ONLY ',
     +    MAXHIS,' Lines of HISTORY ALLOWED'
        NUM_NEW_HIST = NUM_NEW_HIST - 1
        GO TO 10
      END IF
C
C
      IWORK = LENSTR(LINE)
      IF (IWORK.GT.80) IWORK = 80
      NEW_HIST(NUM_NEW_HIST) = LINE(IBEG(1) :IWORK)
      DO_NEW_HIST = .TRUE.
      GO TO 10
C
C  How lucky = ready for SORT 
cc
  190 CONTINUE
      CALL CCPUPC(LINE)
        IF(NTOK.LT.4) CALL CCPERR(1,' NOT ENOUGH KEYWORDS FOR SORT')
      I  = 0
      DO 191 II = 2,4
       IF(LINE(IBEG(II):IBEG(II)) .EQ.'H') 
     +                        SORTKYS(II-1) = 1
       IF(LINE(IBEG(II):IBEG(II)) .EQ.'K') 
     +                        SORTKYS(II-1) = 2
       IF(LINE(IBEG(II):IBEG(II)) .EQ.'L') 
     +                        SORTKYS(II-1) = 3
       I  = I + SORTKYS(II-1)
191   CONTINUE
       IF(I.NE.6)WRITE(LUNOUT,*)LINE(1:LENSTR(LINE)) 
       IF(I.NE.6) 
     +  CALL CCPERR(1,' Error_ Sort on EACH of H K and L in some order')
      GO TO 10
C
C
C---- SCALE FILE_NUMBER n  SCALE_ALL(n)   TEMP_ALL
C     ==================================================
C
  195 CONTINUE
      CWORK = LINE(IBEG(2) :IEND(2))
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
C
      IF (CWORK(1:1).EQ.'F') THEN
C
C---- second argument = 'FILE_NUMBER'
C     test if third argument is integer
C
        IF (ITYP(3).NE.2) THEN
          WRITE (LUNOUT,FMT='(a,a,/,a,a,a,/,a)')
     +      ' Key_Word Error, Wrong type of input for  CTYPIN',
     +      ' line:-',' If Sub_KeyWord = FILE_NUMBER  then ',
     +      'third  Argument MUST be an Integer  Ignoring this',
     +      ' line -',LINE(1:LENSTR(LINE))
          GO TO 10
        END IF
C
C---- For an input file being given
C     Get File_Number and check if valid
C
C            ********************************
        CALL GTPINT(3,NFILE,NTOK,ITYP,FVALUE)
C            ********************************
C
C---- Check if allowed range of files ... 1 to 5 (is this enough ???)
C
        IF (NFILE.LE.0 .OR. NFILE.GT.MAXFILES) THEN
          WRITE (LUNOUT,FMT='(a,/,a,i4,a,i2,a,/,a)')
     +      ' Key_Word Error, For CTYPIN FILE_NUMBER line:-',
     +      '  File_Number Given = ',NFILE,
     +      ' Is invalid only files 1 to ',MAXFILES,
     +      ' Allowed, Ignoring this line -',LINE(1:LENSTR(LINE))
          GO TO 10
        END IF
      END IF
C            ********************************
        CALL GTPREA(4,SCALE_ALL(NFILE),NTOK,ITYP,FVALUE)
        TEMP_ALL(NFILE) = -999.0
        IF(NTOK.EQ.5) CALL GTPREA(5,TEMP_ALL(NFILE),NTOK,ITYP,FVALUE)
        TEMP_ANIS(1,NFILE) = -999.0
        IF(NTOK.EQ.10) THEN
            CALL GTPREA(5,TEMP_ANIS(1,NFILE),NTOK,ITYP,FVALUE)
            CALL GTPREA(6,TEMP_ANIS(2,NFILE),NTOK,ITYP,FVALUE)
            CALL GTPREA(7,TEMP_ANIS(3,NFILE),NTOK,ITYP,FVALUE)
            CALL GTPREA(8,TEMP_ANIS(4,NFILE),NTOK,ITYP,FVALUE)
            CALL GTPREA(9,TEMP_ANIS(5,NFILE),NTOK,ITYP,FVALUE)
            CALL GTPREA(10,TEMP_ANIS(6,NFILE),NTOK,ITYP,FVALUE)
        END IF
        IF(NTOK.GT.5 .AND. NTOK.NE.10) 
     +   CALL CCPERR(1,' Wrong number of items on SCALE line')
C            ********************************
        IF(TEMP_ALL(NFILE).GT.-998.0) 
     +  WRITE (LUNOUT,'(a,I4,a,2f7.3,/,a,/,a)')
     +  ' Overall Scale and Temperature factor for File',NFILE,
     +  ' are ',SCALE_ALL(NFILE),TEMP_ALL(NFILE),
     +  ' Applied scale is Scale * exp(- B * (sin theta/lambda)**2);',
     +  '  ** Scale and B applied to all F and SIGF in file **'
        IF(TEMP_ANIS(1,NFILE).GT.-998.0) 
     +  WRITE (LUNOUT,'(a,I4,a,f7.3,3x,6F7.3/,a,/2a,/,5x,a)')
     +' Overall Scale and Anisotropic Temperature factor for File',NFILE
     +, ' are ',SCALE_ALL(NFILE),
     +   TEMP_ANIS(1,NFILE),TEMP_ANIS(2,NFILE),TEMP_ANIS(3,NFILE),
     +   TEMP_ANIS(4,NFILE),TEMP_ANIS(5,NFILE),TEMP_ANIS(6,NFILE),
     +  ' Applied scale is Scale * ',
     + ' exp',
     +' (- ( B11*h*h*(a*)(a*) + B22*k*k*(b*)(b*) + B33*l*l*(c*)(c*) + ',
     +'2.0*(B12*h*k*(a*)*(b*) + B13*h*l*(a*)*(c*) + B23*k*l*(b*)*(c*)))'
C   
C   where
C   h,k,l are Miller indices
C   a*,b*,c* are reciprocal space cell dimensions

          GO TO 10
C
C    Unrecognised Key word
C
  200 WRITE (LUNOUT,FMT='(/,a,/,5x,a)') '  Error in keyword:',
     +  LINE(IBEG(1) :IEND(1))
          CALL CCPERR(1, ' Fatal Error')
C
C---- MONITOR
C     =======
  210 CONTINUE
C
C
C---- MONITOR "string"   [OPTIONAL INPUT]  Printing mTZ info
C
C    string is one of the following;
C
C NONE sets MTZPRT = 0 (default)
C
C     no header o/p
C
C BRIEF sets MTZPRT = 1
C     brief header o/p
C
C HIST sets MTZPRT = 2
C     brief + mtz history
C
C FULL sets MTZPRT = 3
C     full header o/p from mtz reads
C
      MTZPRT = 0
C
C
      IF (NTOK.LE.1) THEN
        WRITE (LUNOUT,FMT=6004)
 6004   FORMAT (//' **** KeyWord Warning ****',/' No argument given on',
     +         ' MONITOR KEY using default of  NONE ie MTZPRT = 0')
        GO TO 50
      END IF
C
C
      IF (ITYP(2).EQ.1) THEN
        CWORK = LINE(IBEG(2) :IEND(2))
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
        IF (CWORK(1:1).EQ.'N') MTZPRT = 0
        IF (CWORK(1:1).EQ.'B') MTZPRT = 1
        IF (CWORK(1:1).EQ.'H') MTZPRT = 2
        IF (CWORK(1:1).EQ.'F') MTZPRT = 3
        IF (CWORK(1:1).NE.'N' .AND. CWORK(1:1).NE.'B' .AND.
     +      CWORK(1:1).NE.'H' .AND. CWORK(1:1).NE.'F') WRITE (LUNOUT,
     +      FMT=6006)
 6006   FORMAT (//' **** KeyWord Warning ****',/' Character string giv',
     +         'en on MONITOR KEY not equal to',/'    ANY expected val',
     +         'ue, using default of NONE')
      ELSE
        WRITE (LUNOUT,FMT=6008)
 6008   FORMAT (//' **** KeyWord Warning ****',/'  Integer argument gi',
     +         'ven on MONITOR KEY when expecting',/'   character stri',
     +         'ng  using default for MTZPRT = 0')
      END IF
      GO TO 10
C
C---- PNAME 
C     Obsolete - assign columns according to XNAME/DNAME only
C     ==========
C
  500 CONTINUE
C
      WRITE (LUNOUT,FMT='(/,A,/)')
     +  ' WARNING: PNAME keyword is obsolete - see documentation!'

      GO TO 10
C
C---- XNAME FILE <File_Number>  "E1=xxx E2=xxx ...."
C     XNAME FILE <File_Number>  "E1 TO E5 xxx ...."
C     XNAME FILE <File_Number>  "ALL xxx ...."
C     ==========
C
 505  CONTINUE
      CWORK = LINE(IBEG(2) :IEND(2))
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
      IF (CWORK(1:1).NE.'F') THEN
          WRITE (LUNOUT,FMT='(A,/,A,/,A)')
     +      ' Key_Word Error, XNAME requires FILE argument.',
     +      ' Ignoring this line -',LINE(1:LENSTR(LINE))
          GO TO 10
      END IF
C
C---- second argument = 'FILE_NUMBER'
C     test if third argument is integer
C
      IF (ITYP(3).NE.2) THEN
        WRITE (LUNOUT,FMT='(A,A,/,A,A,A,/,A)')
     +      ' Key_Word Error, Wrong type of input for  XNAME',' line:-',
     +      ' If Sub_KeyWord = FILE_NUMBER  then ',
     +      'third  Argument MUST be an Integer  Ignoring this',
     +      ' line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF
C
C---- For an input file being given
C     Get File_Number and check if valid
C
C            ********************************
      CALL GTPINT(3,NFILE,NTOK,ITYP,FVALUE)
C            ********************************
C
C---- Check if allowed range of files ... 1 to 5 (is this enough ???)
C
      IF (NFILE.LE.0 .OR. NFILE.GT.MAXFILES) THEN
        WRITE (LUNOUT,FMT='(a,/,a,i4,a,i2,a,/,a)')
     +      ' Key_Word Error, For XNAME FILE_NUMBER line:-',
     +      '  File_Number Given = ',NFILE,
     +      ' Is invalid only files 1 to ',MAXFILES,
     +      ' Allowed, Ignoring this line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF
C
C---- Now save labels for this input file
C
      XNAME_SAVE(NFILE) = 'XNAME  '//LINE(IBEG(4) :IEND(NTOK))
      GOT_XNAME(NFILE)  = .TRUE.
C
      GO TO 10
C
C---- DNAME FILE <File_Number>  "E1=xxx E2=xxx ...."
C     DNAME FILE <File_Number>  "E1 TO E5 xxx ...."
C     DNAME FILE <File_Number>  "ALL xxx ...."
C     ==========
C
  510 CONTINUE
      CWORK = LINE(IBEG(2) :IEND(2))
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
      IF (CWORK(1:1).NE.'F') THEN
          WRITE (LUNOUT,FMT='(A,/,A,/,A)')
     +      ' Key_Word Error, DNAME requires FILE argument.',
     +      ' Ignoring this line -',LINE(1:LENSTR(LINE))
          GO TO 10
      END IF
C
C---- second argument = 'FILE_NUMBER'
C     test if third argument is integer
C
      IF (ITYP(3).NE.2) THEN
        WRITE (LUNOUT,FMT='(A,A,/,A,A,A,/,A)')
     +      ' Key_Word Error, Wrong type of input for  DNAME',' line:-',
     +      ' If Sub_KeyWord = FILE_NUMBER  then ',
     +      'third  Argument MUST be an Integer  Ignoring this',
     +      ' line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF
C
C---- For an input file being given
C     Get File_Number and check if valid
C
C            ********************************
      CALL GTPINT(3,NFILE,NTOK,ITYP,FVALUE)
C            ********************************
C
C---- Check if allowed range of files ... 1 to 5 (is this enough ???)
C
      IF (NFILE.LE.0 .OR. NFILE.GT.MAXFILES) THEN
        WRITE (LUNOUT,FMT='(a,/,a,i4,a,i2,a,/,a)')
     +      ' Key_Word Error, For DNAME FILE_NUMBER line:-',
     +      '  File_Number Given = ',NFILE,
     +      ' Is invalid only files 1 to ',MAXFILES,
     +      ' Allowed, Ignoring this line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF
C
C---- Now save labels for this input file
C
      DNAME_SAVE(NFILE) = 'DNAME  '//LINE(IBEG(4) :IEND(NTOK))
      GOT_DNAME(NFILE)  = .TRUE.
C
      GO TO 10
C
C---- DCELL FILE <File_Number> [ <xname> <dname> | <Set_number> ] <cell dimensions>
C     ==========
C
  520 CONTINUE
      CWORK = LINE(IBEG(2) :IEND(2))
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
      IF (CWORK(1:1).NE.'F') THEN
          WRITE (LUNOUT,FMT='(A,/,A,/,A)')
     +      ' Key_Word Error, DCELL requires FILE argument.',
     +      ' Ignoring this line -',LINE(1:LENSTR(LINE))
          GO TO 10
      END IF
C
C---- second argument = 'FILE_NUMBER'
C     test if third argument is integer
C
      IF (ITYP(3).NE.2) THEN
        WRITE (LUNOUT,FMT='(A,A,/,A,A,A,/,A)')
     +      ' Key_Word Error, Wrong type of input for  DCELL',' line:-',
     +      ' If Sub_KeyWord = FILE_NUMBER  then ',
     +      'third  Argument MUST be an Integer  Ignoring this',
     +      ' line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF
C
C---- For an input file being given
C     Get File_Number and check if valid
C
C            ********************************
      CALL GTPINT(3,NFILE,NTOK,ITYP,FVALUE)
C            ********************************
C
C---- Check if allowed range of files ... 1 to 5 (is this enough ???)
C
      IF (NFILE.LE.0 .OR. NFILE.GT.MAXFILES) THEN
        WRITE (LUNOUT,FMT='(a,/,a,i4,a,i2,a,/,a)')
     +      ' Key_Word Error, For DCELL FILE_NUMBER line:-',
     +      '  File_Number Given = ',NFILE,
     +      ' Is invalid only files 1 to ',MAXFILES,
     +      ' Allowed, Ignoring this line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF

      IF (ITYP(4).EQ.2) THEN

C---- Get set number
        CALL GTPINT(4,NSET,NTOK,ITYP,FVALUE)
        CALL RDCELL(5,ITYP,FVALUE,NTOK,DCELL_OUT(1,NSET,NFILE))

        GOT_DCELL(NFILE) = .TRUE.

      ELSEIF (ITYP(4).EQ.1) THEN

        NSETC = NSETC + 1
        DCELL_XNAME(NSETC,NFILE) = LINE(IBEG(4):IEND(4))
        DCELL_DNAME(NSETC,NFILE) = LINE(IBEG(5):IEND(5))
        CALL RDCELL(6,ITYP,FVALUE,NTOK,DCELL_BYNAME(1,NSETC,NFILE))

        GOT_DCELL(NFILE) = .TRUE.

      ENDIF
C
      GO TO 10
C
C---- DWAVE FILE <File_Number> [ <xname> <dname> | <Set_number> ] <wavelength>
C     ==========
C
  530 CONTINUE
      CWORK = LINE(IBEG(2) :IEND(2))
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
      IF (CWORK(1:1).NE.'F') THEN
          WRITE (LUNOUT,FMT='(A,/,A,/,A)')
     +      ' Key_Word Error, DWAVE requires FILE argument.',
     +      ' Ignoring this line -',LINE(1:LENSTR(LINE))
          GO TO 10
      END IF
C
C---- second argument = 'FILE_NUMBER'
C     test if third argument is integer
C
      IF (ITYP(3).NE.2) THEN
        WRITE (LUNOUT,FMT='(A,A,/,A,A,A,/,A)')
     +      ' Key_Word Error, Wrong type of input for  DWAVE',' line:-',
     +      ' If Sub_KeyWord = FILE_NUMBER  then ',
     +      'third  Argument MUST be an Integer  Ignoring this',
     +      ' line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF
C
C---- For an input file being given
C     Get File_Number and check if valid
C
C            ********************************
      CALL GTPINT(3,NFILE,NTOK,ITYP,FVALUE)
C            ********************************
C
C---- Check if allowed range of files ... 1 to 5 (is this enough ???)
C
      IF (NFILE.LE.0 .OR. NFILE.GT.MAXFILES) THEN
        WRITE (LUNOUT,FMT='(a,/,a,i4,a,i2,a,/,a)')
     +      ' Key_Word Error, For DWAVE FILE_NUMBER line:-',
     +      '  File_Number Given = ',NFILE,
     +      ' Is invalid only files 1 to ',MAXFILES,
     +      ' Allowed, Ignoring this line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF

      IF (ITYP(4).EQ.2) THEN

C---- Get set number
        CALL GTPINT(4,NSET,NTOK,ITYP,FVALUE)
        CALL GTPREA(5,DWAVE_OUT(NSET,NFILE),NTOK,ITYP,FVALUE)

        GOT_DWAVE(NFILE) = .TRUE.

      ELSEIF (ITYP(4).EQ.1) THEN

        NSETW = NSETW + 1
        DWAVE_XNAME(NSETW,NFILE) = LINE(IBEG(4):IEND(4))
        DWAVE_DNAME(NSETW,NFILE) = LINE(IBEG(5):IEND(5))
        CALL GTPREA(6,DWAVE_BYNAME(NSETW,NFILE),NTOK,ITYP,FVALUE)

        GOT_DWAVE(NFILE) = .TRUE.

      ENDIF
C
      GO TO 10
C
C---- DRENAME FILE <File_Number> [ <xname> <dname> | <Set_number> ] <xname_new> <dname_new>
C     ==========
C
  540 CONTINUE
      CWORK = LINE(IBEG(2) :IEND(2))
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
      IF (CWORK(1:1).NE.'F') THEN
          WRITE (LUNOUT,FMT='(A,/,A,/,A)')
     +      ' Key_Word Error, DRENAME requires FILE argument.',
     +      ' Ignoring this line -',LINE(1:LENSTR(LINE))
          GO TO 10
      END IF
C
C---- second argument = 'FILE_NUMBER'
C     test if third argument is integer
C
      IF (ITYP(3).NE.2) THEN
        WRITE (LUNOUT,FMT='(A,A,/,A,A,A,/,A)')
     +    ' Key_Word Error, Wrong type of input for  DRENAME',' line:-',
     +    ' If Sub_KeyWord = FILE_NUMBER  then ',
     +    'third  Argument MUST be an Integer  Ignoring this',
     +    ' line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF
C
C---- For an input file being given
C     Get File_Number and check if valid
C
C            ********************************
      CALL GTPINT(3,NFILE,NTOK,ITYP,FVALUE)
C            ********************************
C
C---- Check if allowed range of files ... 1 to 5 (is this enough ???)
C
      IF (NFILE.LE.0 .OR. NFILE.GT.MAXFILES) THEN
        WRITE (LUNOUT,FMT='(a,/,a,i4,a,i2,a,/,a)')
     +      ' Key_Word Error, For DRENAME FILE_NUMBER line:-',
     +      '  File_Number Given = ',NFILE,
     +      ' Is invalid only files 1 to ',MAXFILES,
     +      ' Allowed, Ignoring this line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF

      IF (ITYP(4).EQ.2) THEN

C---- Get set number
        CALL GTPINT(4,NSET,NTOK,ITYP,FVALUE)
        DRENAME_XNAME_OUT(NSET,NFILE) = LINE(IBEG(5):IEND(5))
        DRENAME_DNAME_OUT(NSET,NFILE) = LINE(IBEG(6):IEND(6))

        GOT_DRENAME(NFILE) = .TRUE.

      ELSEIF (ITYP(4).EQ.1) THEN

        NSETR = NSETR + 1
        DRENAME_XNAME(NSETR,NFILE) = LINE(IBEG(4):IEND(4))
        DRENAME_DNAME(NSETR,NFILE) = LINE(IBEG(5):IEND(5))
        DRENAME_BYNAME_XNAME_OUT(NSETR,NFILE) = LINE(IBEG(6):IEND(6))
        DRENAME_BYNAME_DNAME_OUT(NSETR,NFILE) = LINE(IBEG(7):IEND(7))

        GOT_DRENAME(NFILE) = .TRUE.

      ENDIF
C
      GO TO 10
C
C---- DPNAME FILE <File_Number> [ <xname> <dname> | <Set_number> ] <pname_new>
C     ==========
C
 545  CONTINUE
      CWORK = LINE(IBEG(2) :IEND(2))
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
      IF (CWORK(1:1).NE.'F') THEN
          WRITE (LUNOUT,FMT='(A,/,A,/,A)')
     +      ' Key_Word Error, DPNAME requires FILE argument.',
     +      ' Ignoring this line -',LINE(1:LENSTR(LINE))
          GO TO 10
      END IF
C
C---- second argument = 'FILE_NUMBER'
C     test if third argument is integer
C
      IF (ITYP(3).NE.2) THEN
        WRITE (LUNOUT,FMT='(A,A,/,A,A,A,/,A)')
     +    ' Key_Word Error, Wrong type of input for DPNAME',' line:-',
     +    ' If Sub_KeyWord = FILE_NUMBER  then ',
     +    'third  Argument MUST be an Integer  Ignoring this',
     +    ' line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF
C
C---- For an input file being given
C     Get File_Number and check if valid
C
C            ********************************
      CALL GTPINT(3,NFILE,NTOK,ITYP,FVALUE)
C            ********************************
C
C---- Check if allowed range of files ... 1 to 5 (is this enough ???)
C
      IF (NFILE.LE.0 .OR. NFILE.GT.MAXFILES) THEN
        WRITE (LUNOUT,FMT='(a,/,a,i4,a,i2,a,/,a)')
     +      ' Key_Word Error, For DPNAME FILE_NUMBER line:-',
     +      '  File_Number Given = ',NFILE,
     +      ' Is invalid only files 1 to ',MAXFILES,
     +      ' Allowed, Ignoring this line -',LINE(1:LENSTR(LINE))
        GO TO 10
      END IF

      IF (ITYP(4).EQ.2) THEN

C---- Get set number
        CALL GTPINT(4,NSET,NTOK,ITYP,FVALUE)
        DRENAME_PNAME_OUT(NSET,NFILE) = LINE(IBEG(5):IEND(5))

        GOT_DPNAME(NFILE) = .TRUE.

      ELSEIF (ITYP(4).EQ.1) THEN

        NSETP = NSETP + 1
        DPNAME_XNAME(NSETP,NFILE) = LINE(IBEG(4):IEND(4))
        DPNAME_DNAME(NSETP,NFILE) = LINE(IBEG(5):IEND(5))
        DRENAME_BYNAME_PNAME_OUT(NSETP,NFILE) = LINE(IBEG(6):IEND(6))

        GOT_DPNAME(NFILE) = .TRUE.

      ENDIF
C
      GO TO 10
C
C---- END or RUN or GO 
C     ========
  220 CONTINUE

      DO 221 I = 1,NFILES_IN
        IF(CTYP_PROC_CODE(I) .EQ.0) 
     1  write(6,'(a,i3)') ' No CTYP lines input for file:',I
221   CONTINUE
      IF (GOT_MISS) THEN
        IF(.NOT.QISNAN(VAL_MISSING)) WRITE (LUNOUT,FMT='(a,F12.3)')
     +   '  Missing observations flagged as ',VAL_MISSING
        IF(QISNAN(VAL_MISSING)) WRITE (LUNOUT,FMT='(a,3x,A)')
     +   '  Missing observations flagged as ','Nan'
      ENDIF
      IF(OUTPUT_MISS)
     +    WRITE (LUNOUT,FMT='(a)')
     +   '    Indices output even if all data items flagged "missing"'
      RETURN
C
C
      END
C
C
C
C     ================================================================
      SUBROUTINE LCTPIN(LSPRGI,NLPRGI,CTCHECK,ITOK,NTOK,LINE,IBEG,IEND)
C     ================================================================
C
C
C
C-----Subroutine to read standard input lines of the form
C              LABIN  item1=name1 item2=name2 ...
C     Use this to read the inputs, and then LRASSN to setup the
C     program label to file label assignments.
C     NB : there should only be one LABIN card for each program,
C     with continuation lines if necessary.
C
C---- Arguments:
C
C     LSPRGI    CHARACTER*30    program label strings (array)
C                               L(abel) S(tring) PRG(rammme) I(nput)
C
C     NLPRGI    INTEGER         number of program input labels
C                               N(umber of) L(abels) PRG(ramme) I(nput)
C
C     NTOK      INTEGER         from Parser, number of tokens on line
C
C     LINE      CHARACTER*400   the input line
C
C     IBEG,IEND INTEGER         arrays from the parser, delimiters
C                               for each token
C
C
C     .. Parameters ..
      INTEGER           MCOLS
      PARAMETER         (MCOLS=500)
C     ..
C     .. Scalar Arguments ..
      INTEGER           ITOK,NLPRGI,NTOK
      CHARACTER         LINE* (*)
C     ..
C     .. Array Arguments ..
      INTEGER           IBEG(*),IEND(*)
      CHARACTER         CTCHECK(*)*1,LSPRGI(*)*30
C     ..
C     .. Local Scalars ..
      INTEGER           JDO,JLENG,JLOOP,JSTART,JTOK
      CHARACTER         LC1*1,LC2*1,CWORK*30,CWORK2*30,STROUT*400
C     ..
C     .. External Functions ..
      INTEGER           LENSTR
      EXTERNAL          LENSTR
C     ..
C     .. External Subroutines ..
      EXTERNAL          CCPUPC,PUTLIN
C     ..
C
      JTOK = NTOK
      JSTART = ITOK
C
C---- Keyword  CTYPIN - ASSIGN_HKL_LABELS  item1=name1 item2=name2 ...
C
C      item == program label
C      name == user label
C
      IF (NLPRGI.LE.0) THEN
        STROUT = ' Error this program has no PROGRAM LABELS'
C
C            ***********************
        CALL PUTLIN(STROUT,'ERRWIN')
C            ***********************
C
      ELSE IF (JTOK.LE.1) THEN
        STROUT = ' **** Warning no argument given for LABIN '
C
C            ***********************
        CALL PUTLIN(STROUT,'ERRWIN')
C            ***********************
C
      ELSE
C
C---- Find input label assignments
C
        DO 40 JLOOP = JSTART,NTOK,2
C
C
          IF ((JLOOP+1).GT.JTOK) THEN
            GO TO 50
          ELSE
            CWORK = LINE(IBEG(JLOOP) :IEND(JLOOP))
            LC1 = CWORK(1:1)
C
C                *************
            CALL CCPUPC(CWORK)
C                *************
C
            CWORK2 = LINE(IBEG(JLOOP+1) :IEND(JLOOP+1))
            LC2 = CWORK2(1:1)
C
C                **************
            CALL CCPUPC(CWORK2)
C                **************
C
            DO 10 JDO = 4,NLPRGI
              JLENG = LENSTR(LSPRGI(JDO))
C
C
              IF (CWORK.EQ.LSPRGI(JDO)) THEN
                GO TO 30
              ELSE IF (CWORK2.EQ.LSPRGI(JDO)) THEN
                GO TO 20
              END IF
C
C
   10       CONTINUE
C
C
            STROUT = ' **** Error input assignment does not match'//
     +               ' program labels'
C
C                ***********************
            CALL PUTLIN(STROUT,'ERRWIN')
C                ***********************
C
            WRITE (STROUT,FMT=
     +        '(''  Neither '',A,'' nor '',A,'' recognised'')') CWORK(1:
     +        LENSTR(CWORK)),CWORK2(1:LENSTR(CWORK2))
C
C                ***********************
            CALL PUTLIN(STROUT,'ERRWIN')
C                ***********************
C
            GO TO 40
   20       CONTINUE
            CTCHECK(JDO) = LC1
            GO TO 40
   30       CONTINUE
            CTCHECK(JDO) = LC2
          END IF
   40   CONTINUE
C
C
        RETURN
   50   STROUT = ' **** Error !!!! for CTYPIN ****'
C
C            ***********************
        CALL PUTLIN(STROUT,'ERRWIN')
        STROUT = ' **** NOT ENOUGH ARGUMENT PAIRS of type '
        CALL PUTLIN(STROUT,'ERRWIN')
        STROUT = 'Prog_label = User_label'
        CALL PUTLIN(STROUT,'ERRWIN')
C            ***********************
C
      END IF
C
C
      END
C
C
C
C     =======================================
      SUBROUTINE SORTINI(NUM_COLS_OUT,LUNOUT)
C     =======================================
C
C---- sort type is Real
C      Initialise SORTING o/p
C
C     .. Parameters ..
      INTEGER            SORTYP
      PARAMETER          (SORTYP=7)
      INTEGER            MAXSORT_KEY
      PARAMETER          (MAXSORT_KEY=5)
C     ..
C     .. Scalar Arguments ..
      INTEGER            LUNOUT,NUM_COLS_OUT
C     ..
C     .. Local Scalars ..
      INTEGER            ISTAT,J,JDO10,KEYSIZ,LASCND,LRECL_SORT,
     +                   NSORTKEYS,SORTKYS,KEYBF
C     ..
C     .. Local Arrays ..
      INTEGER            SORTKEYS(MAXSORT_KEY)
CEJD      INTEGER*2          KEYBUF(41)
      INTEGER          KEYBUF(MAXSORT_KEY*10 + 1)
      COMMON /SORT_INFO/  SORTKYS(3)
      SAVE
C     ..
C     .. External Functions ..
      INTEGER            SRTBEG
      LOGICAL            VAXVMS, WINMVS
      EXTERNAL           SRTBEG,VAXVMS, WINMVS
C     ..
C
C
      LASCND = 0
      LRECL_SORT = NUM_COLS_OUT*4
      KEYSIZ = 4
C
C---- sort on H K L only  - faster to assume one nsortkeys 
C
      NSORTKEYS = 3
      KEYBF = 1
C
C----   SORTKEYS are columns 1 2 3
C
CEJD      SORTKEYS(1) = 1
CEJD      SORTKEYS(2) = 2
CEJD      SORTKEYS(3) = 3
      SORTKEYS(1) = SORTKYS(1)
      SORTKEYS(2) = SORTKYS(2)
      SORTKEYS(3) = SORTKYS(3)
C
C----  Check if order 1/2/3 - but not on a VAX or windows (doesn't use binsort)
CMDW   Now not for anything, unless we re-instate binsort.
C
C       IF(.NOT.VAXVMS() .AND. .NOT.WINMVS() .AND.
C     +    SORTKEYS(1) .EQ. 1 .AND.
C     +    SORTKEYS(2) .EQ. 2 .AND.
C     +    SORTKEYS(3) .EQ. 3) THEN
C          NSORTKEYS = 1
C          KEYBF     = 3
C       END IF
C
C---- Set up key buffer for number of keys. A descriptor consists of:
C     2-byte length (in bytes)
C     2-byte class & type (probably not used)
C     4-byte address of array
C
      DO 10 JDO10 = 1,NSORTKEYS
C        J = (JDO10-1)*4 + 1
        J = (JDO10-1)*MAXSORT_KEY + 1
C        J = 1
C
C---- Sort Type is REAL (SORTYP set up in PARAMETER statement)
C
        KEYBUF(J) = SORTYP
C
C---- Sort Order ascending/descending
C
        KEYBUF(J+1) = LASCND
C
C---- position of 1st byte in key
C
        KEYBUF(J+2) = (SORTKEYS(JDO10)-1)*KEYSIZ
C
C---- keylength in data units 
C
C        KEYBUF(J+3) = 3
        KEYBUF(J+3) = KEYBF
        KEYBUF(J+4) = 0
   10 CONTINUE
C
C---- sort on H K L only
C
Cejd      NSORTKEYS = 3
C
C----   SORTKEYS are columns 1 2 3
C
Cejd      SORTKEYS(1) = 1
Cejd      SORTKEYS(2) = 2
Cejd      SORTKEYS(3) = 3
C
C---- Set up key buffer for number of keys. A descriptor consists of:
C     2-byte length (in bytes)
C     2-byte class & type (probably not used)
C     4-byte address of array
C
Cejd      DO 10 JDO10 = 1,NSORTKEYS
CEJD        J = (JDO10-1)*4 + 1
Cejd        J = (JDO10-1)*MAXSORT_KEY + 1
C
C---- Sort Type is REAL (SORTYP set up in PARAMETER statement)
C
Cejd        KEYBUF(J) = SORTYP
C
C---- Sort Order ascending/descending
C
Cejd        KEYBUF(J+1) = LASCND
C
C---- position of 1st byte in key
C
Cejd        KEYBUF(J+2) = (SORTKEYS(JDO10)-1)*KEYSIZ
C
C---- keylength in BYTES (4)
C
CEJD        KEYBUF(J+3) = KEYSIZ
Cejd        KEYBUF(J+3) = 1
Cejd        KEYBUF(J+4) = 0
Cejd   10 CONTINUE
C
C
      WRITE (LUNOUT,FMT=6000) NSORTKEYS, (SORTKEYS(J),J=1,NSORTKEYS)
 6000 FORMAT (1X,I8,' sort keys, in columns ',10(30I4,/))
C
C---- Initialise sort, set parameters, etc
C
C             ***************************************
CEJD      ISTAT = SRTBEG(NSORTKEYS,KEYBUF,LRECL_SORT,0,0)
      ISTAT = SRTBEG(NSORTKEYS,KEYBUF,LRECL_SORT,0)
C             ***************************************
C
      IF (ISTAT.NE.0) THEN
        WRITE (LUNOUT,FMT=6002) ISTAT
 6002   FORMAT (' SORTINI failed to initialise sort, status = ',I8,/)
          CALL CCPERR(1, ' Fatal Error')
      END IF
C
C
      RETURN
      END

c
      subroutine calc_rcell(cell,rcell)
c
      real cell(6),rcell(6),a,b,c,alph,beta,gamm,sum,vol,
     +     conv, as,bs,gs,alphas,betas,gammas,
     +     cosa,sina,cosb,sinb,cosg,sing,
     +     cosas,sinas,cosbs,sinbs,cosgs,sings
c
c
      CONV = 3.1415927/180.0
      a=cell(1)
      b=cell(2)
      c=cell(3)
      alph=conv*cell(4)
      beta=conv*cell(5)
      gamm=conv*cell(6)
      sum=0.5*(alph+beta+gamm)
      vol=2.0*a*b*c*
     + sqrt(sin(sum)*sin(sum-alph)*sin(sum-beta)*sin(sum-gamm))
      cosas=(cos(beta)*cos(gamm)-cos(alph))/(sin(beta)*sin(gamm))
c
      SINA = SIN(ALPH)
      COSA = COS(ALPH)
      SINB = SIN(BETA)
      COSB = COS(BETA)
      SING = SIN(GAMM)
      COSG = COS(GAMM)
      COSAS = (COSG*COSB-COSA)/ (SINB*SING)
      SINAS = SQRT(1.0-COSAS*COSAS)
      COSBS = (COSA*COSG-COSB)/ (SINA*SING)
      SINBS = SQRT(1.0-COSBS*COSBS)
      COSGS = (COSA*COSB-COSG)/ (SINA*SINB)
      SINGS = SQRT(1.0-COSGS*COSGS)
      AS = B*C*SINA/VOL
      BS = C*A*SINB/VOL
      CS = A*B*SING/VOL
      ALPHAS = ATAN2(SINAS,COSAS)/CONV
      BETAS  = ATAN2(SINBS,COSBS)/CONV
      GAMMAS = ATAN2(SINGS,COSGS)/CONV
      RCELL(1) = AS
      RCELL(2) = BS
      RCELL(3) = CS
      RCELL(4) = ALPHAS
      RCELL(5) = BETAS
      RCELL(6) = GAMMAS
      RETURN
      END
C
