C
C     CAVENV
C     Copyright (C) 2000 ANNE VOLBEDA
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
        PROGRAM CAVENV
C 
C	PROGRAM VERSION MARCH 2000 BY ANNE VOLBEDA, IBS/LCCP GRENOBLE.
C
C	Calculates cavities in macromolecular structures:
C
C	PLEASE ACKNOWLEDGE AS: A. Volbeda, private communication
C	or with reference (in french):
C	Anne Volbeda, Splologie des hydrognases  nickel et  fer. In:
C	"Les Ecoles Physique et Chimie du Vivant, numro 1 - avril 1999,
C	Analyse de l'organisation tridimensionnelle des protines", pp 47-52.	
C
C	The input structure is mapped onto a 3-d grid.
C	Each grid point is given a value equal to the distance of the
C	closest atom, minus its Van der Waals radius, up to an 
C	input maximum of radmax (maximal probe radius to be tested).
C
C	Van der Waals radii are obtained from X-PLOR.
C
C	Program history:
C	1st version was called CAVITY
C	Written during my PhD in Groningen, The Netherlands, 1986
C	Has been part of BIOMOL package, with modifications by others, 
C       but at about 1990 it was completely changed.
C	My original version used to give a map with only zero's and ones
C	Now modified to give an "accessibility" map in which the value of each 
C	grid point corresponds to the minimal accessible probe radius. 
C
C	The output map may be inspected at a computer graphics display, using
C	standard display programs as O, TURBO or FRODO.
C	Output map format: CCP4 => may be converted with MAPPAGE or MAPMAN
C
C	compilation: with ccp4-library $CLIB/libccp4.a
C	=> f77 cavenv.f $CLIB/libccp4.a -o cavenv.exe
C
C       April 2000
C       Martyn Winn: adapted for inclusion in CCP4 suite. 
C
C	N.B. the program is dimensioned for a maximum of 50000 atoms,
C	a maximal map size of 3375000 (150*150*150) grid points 
C	and a maximum of 60 NCS operations.
C	Adapt the parameter statements (main program and subroutines)
C	if you need larger values.
C
C       MDW: The real killer is holding the entire map in memory. The map
C       could be calculated a section at a time, except for the
C       SHRINK option.
C-----------------------------------------------------------------------

      INTEGER NPARM
      PARAMETER  (NPARM=200)
      PARAMETER (NMAP=150, MAXATM=50000, NCSMAX=60, MAXSYM=192 ) 

      DIMENSION G1(NMAP*NMAP*NMAP),T1(NMAP*NMAP*NMAP)
      DIMENSION NT(MAXATM),Z(MAXATM),AC(MAXATM)
      DIMENSION XX(3),X1(3),X2(3),X3(3),X4(3)
      DIMENSION RO3(3,3),SC(3,3),FTOG(3,3),TC(3),PTOG(3,3),XPDB(3)
      DIMENSION MXYZ(3),CELL(6),STP(3),XXMIN(3),XXMAX(3)
      DIMENSION RNCS(3,3,NCSMAX),TNCS(3,NCSMAX),MUVW1(3),MUVW2(3)
      LOGICAL E1(NMAP*NMAP*NMAP),E2(NMAP*NMAP*NMAP)
      COMMON /CAV/ MU1,MV1,MW1,JUVW(3),STMUVW(3)
      COMMON /XYZB/ GTOP(3,3),YY(4,MAXATM),NMOL(MAXATM),DET,CELLVOL
      COMMON /SGSYM/ NSYM,RSYM(4,4,MAXSYM)
      COMMON /LOGIC/ ENV,ENV1,SOLVNT
      COMMON / / RADIUS(MAXATM),XGRID(3,MAXATM),ED(MAXATM)
      CHARACTER*80 CARD,TITLE
      CHARACTER*4 NAME,RES,RESNAM,RESNO,IDENT,ID
      CHARACTER*3 EXCL(NMAP),CXYZ
      CHARACTER*1 AXISF,AXISM,AXISS,CHNNAM,INSCOD,ALTCOD
      CHARACTER*80 NAMSPG,NAMSPGCif, PGNAME
      INTEGER LSPGRP,NSYMP,IATOM,IZ,IXYZIN,NRES,NX,NY,NZ
      REAL U(6),RHOMIN,RHOMAX,RHMEAN,RRMS
      LOGICAL ENV,ENV1,SOLVNT
C    
C---- things for parser
C
      CHARACTER KEY*4,SUBKEY*4,LINE*600,ERRLINE*600,CVALUE(NPARM)*4
      REAL      FVALUE(NPARM)
      LOGICAL   LEND,LOGDEBUG
      INTEGER   NTOK,IBEG(NPARM),IEND(NPARM),ITYPE(NPARM),
     +          IDEC(NPARM)
C
      DATA FTOG / 9*0.0 /
      DATA TC   / 3*0.0 /
      DATA XXMIN / 3*1000.0 /
      DATA XXMAX / 3*-1000.0 /
      DATA RNCS, TNCS / 720*0.0 /
      DATA CXYZ /'XYZ'/

      RNCS(1,1,1) = 1.0
      RNCS(2,2,1) = 1.0
      RNCS(3,3,1) = 1.0
      IUC=1
      NCS = 1
      JEXCL1 = 0
      JEXCL2 = 0
      JEXCL3 = 0
      RMAX1 = 0.0
      NMAP3 = NMAP*NMAP*NMAP

C Defaults for keywords
      TITLE = 'From CAVENV'
      NFIRST = 1
      NL = 10000
      NEXCL = 0
      RMAX = 3.0
      CELL(1) = 0.0
      LSPGRP = 0
      AXISF = 'X'
      AXISM = 'Y'
      AXISS = 'Z'
      JUVW(1)=index(cxyz,axisf)
      JUVW(2)=index(cxyz,axism)
      JUVW(3)=index(cxyz,axiss)
      IU = JUVW(1)
      IV = JUVW(2)
      IW = JUVW(3)
      MXYZ(1) = 0
      MUVW1(1) = 0
      ENV = .FALSE.
      ENV1 = .FALSE.
      SOLVNT = .FALSE.
      NENV = 0
C 
      CALL CCPFYP
      CALL XYZINIT
      CALL CCPRCS(6, 'CAVENV', '$Date: 2006/12/20 16:01:39 $')

      WRITE(6,15)
 15   FORMAT(//,' Visualise cavities in a macromolecular structure',//)

      LOGDEBUG = .TRUE.
 10   CONTINUE
      NTOK=NPARM
      LINE=' '
      CALL PARSER (KEY,LINE,IBEG,IEND,ITYPE,FVALUE,CVALUE,IDEC,NTOK,
     +             LEND,LOGDEBUG)
      IF (LEND) GO TO 1000
      CALL CCPUPC(KEY)
C
      IF (KEY .EQ. 'END')  THEN
           GO TO 1000
C
      ELSE IF (KEY.EQ.'TITL') THEN
        IF(NTOK.GE.2) THEN
          TITLE = LINE(IBEG(2) :IEND(NTOK))
        ENDIF
      ELSE IF (KEY.EQ.'RANG') THEN
        IF(NTOK.GE.3) THEN 
          CALL GTPINT(2,NFIRST,NTOK,ITYPE,FVALUE)
          CALL GTPINT(3,NL,NTOK,ITYPE,FVALUE)
        ENDIF
      ELSE IF (KEY.EQ.'RADM') THEN
        IF(NTOK.GE.2) THEN
          CALL GTPREA (2,RMAX,NTOK,ITYPE,FVALUE)
        ENDIF
      ELSE IF (KEY.EQ.'EXCL') THEN
        IF(NTOK.GE.2) THEN
          NEXCL = NEXCL + 1
          IF (NEXCL.GT.NMAP) THEN
            CALL CCPERR(2,'Too many EXCLUDE cards! Card ignored.')
            NEXCL = NEXCL - 1
            GOTO 10
          ENDIF
          EXCL(NEXCL) = LINE(IBEG(2):IBEG(2)+2)
        ENDIF
      ELSE IF (KEY.EQ.'CELL') THEN
        CALL RDCELL(2,ITYPE,FVALUE,NTOK,CELL)
  
C Used to be 'XYZ' with JUVW as arguments
      ELSE IF (KEY.EQ.'AXIS') THEN
        IF(NTOK.GE.4) THEN
          AXISF = LINE(IBEG(2):IBEG(2)+1)
          AXISM = LINE(IBEG(3):IBEG(3)+1)
          AXISS = LINE(IBEG(4):IBEG(4)+1)
          call ccpupc(axisf)
          call ccpupc(axism)
          call ccpupc(axiss)
          JUVW(1)=index(cxyz,axisf)
          JUVW(2)=index(cxyz,axism)
          JUVW(3)=index(cxyz,axiss)
          IU = JUVW(1)
          IV = JUVW(2)
          IW = JUVW(3)
        ENDIF

C Used to be NGRID
      ELSE IF (KEY.EQ.'GRID') THEN
        IF(NTOK.GE.4) THEN
          CALL GTPINT(2,MXYZ(1),NTOK,ITYPE,FVALUE)
          CALL GTPINT(3,MXYZ(2),NTOK,ITYPE,FVALUE)
          CALL GTPINT(4,MXYZ(3),NTOK,ITYPE,FVALUE)
        ENDIF  

C Used to be 'LIMITS'
      ELSE IF (KEY.EQ.'XYZL') THEN
        IF(NTOK.GE.7) THEN
          CALL GTPINT(2,MUVW1(1),NTOK,ITYPE,FVALUE)
          CALL GTPINT(3,MUVW2(1),NTOK,ITYPE,FVALUE)
          CALL GTPINT(4,MUVW1(2),NTOK,ITYPE,FVALUE)
          CALL GTPINT(5,MUVW2(2),NTOK,ITYPE,FVALUE)
          CALL GTPINT(6,MUVW1(3),NTOK,ITYPE,FVALUE)
          CALL GTPINT(7,MUVW2(3),NTOK,ITYPE,FVALUE)
        ENDIF  

C Used to be   'SPACE'
      ELSE IF (KEY.EQ.'SYMM') THEN
        SUBKEY = LINE(IBEG(2):IEND(2))
        CALL CCPUPC(SUBKEY)
c Special option to disable symmetry
        IF (SUBKEY(1:3).EQ.'OFF') THEN
          IUC=0
          NTOK=NPARM
          LINE='SYMM 1'
          CALL PARSER (KEY,LINE,IBEG,IEND,ITYPE,FVALUE,CVALUE,IDEC,NTOK,
     +             LEND,.FALSE.)
        ENDIF
        CALL RDSYMM(2,LINE,IBEG,IEND,ITYPE,FVALUE,NTOK,NAMSPG,
     +                LSPGRP,PGNAME,NSYM,NSYMP,RSYM)

      ELSE IF (KEY.EQ.'SHRI') THEN
	SOLVNT = .TRUE.
        IF(NTOK.GE.2) THEN
          SUBKEY = LINE(IBEG(2):IEND(2))
          CALL CCPUPC(SUBKEY)
          IF (SUBKEY(1:3).EQ.'CAV') THEN
	    PRINT *,' SHRINK option: find internal cavities'
	    NENV = 0
          ELSEIF (SUBKEY(1:3).EQ.'ENV') THEN
	    PRINT *,' SHRINK option: find minimal protein envelope'
	    NENV = 1
          ENDIF
	  PRINT *,' N.B. THIS OPTION IS STILL UNDER DEVELOPMENT'
	  PRINT *,' '
        ENDIF
  
      ELSE IF (KEY.EQ.'ENVN') THEN
        IF(NTOK.GE.2) THEN
          ENV = .TRUE.
          CALL GTPINT(2,NCS,NTOK,ITYPE,FVALUE)
	  DO 80 K=1,NCS
	    READ(5,*) (RNCS(1,J,K),J=1,3),TNCS(1,K)
	    READ(5,*) (RNCS(2,J,K),J=1,3),TNCS(2,K)
	    READ(5,*) (RNCS(3,J,K),J=1,3),TNCS(3,K)
	    WRITE(6,*) (RNCS(1,J,K),J=1,3),TNCS(1,K)
	    WRITE(6,*) (RNCS(2,J,K),J=1,3),TNCS(2,K)
	    WRITE(6,*) (RNCS(3,J,K),J=1,3),TNCS(3,K)
   80	  CONTINUE
        ENDIF
  
C Used to be 'ENV'
      ELSE IF (KEY.EQ.'ENVP') THEN
          ENV = .TRUE.
  
C Used to be 'ENV1'
      ELSE IF (KEY.EQ.'ENVS') THEN
          ENV = .TRUE.
	  ENV1 = .TRUE.

      ELSE
C
C----- ????? not understood
C
        WRITE(ERRLINE,'(A,A,A)') 
     +          ' Keyword ',KEY,' not recognised: ignored!'
        CALL CCPERR(2,ERRLINE)
      END IF
C
      GO TO 10
C        
1000  CONTINUE

C Open input coordinate file
      IXYZIN = 0
      IFAIL = 0
      CALL XYZOPEN('XYZIN','INPUT',' ',IXYZIN,IFAIL)

C If CELL keyword not given, try to get from XYZIN
      IF (CELL(1) .EQ. 0.0) THEN
        call rbcell(cell, vol)
        IF (CELL(1) .EQ. 0.0) CALL CCPERR(1,
     +    ' No cell dimensions in keyworded input or XYZIN!')
        WRITE(6,25) CELL
 25     FORMAT(' unit cell from XYZIN :',6F8.3,/)
      ENDIF

C If GRID keyword not given, default to 0.5 A spacing
      IF (MXYZ(1).EQ.0) THEN
        MXYZ(1) = NINT(CELL(1)*2.0)
        MXYZ(2) = NINT(CELL(2)*2.0)
        MXYZ(3) = NINT(CELL(3)*2.0)
      ENDIF

C If XYZLIMIT keyword not given, default to whole unit cell
      IF (MUVW1(1).EQ.0) THEN
        MUVW1(1) = 1
        MUVW2(1) = MXYZ(1)
        MUVW1(2) = 1
        MUVW2(2) = MXYZ(2)
        MUVW1(3) = 1
        MUVW2(3) = MXYZ(3)
      ENDIF

C If SYMM keyword not given, try to get from XYZIN
      IF (LSPGRP.EQ.0) THEN
        CALL RBSPGRP(NAMSPG)
        IF (NAMSPG.EQ.' ') CALL CCPERR(1,
     +    ' No spacegroup in keyworded input or XYZIN!')
        CALL MSYMLB3(10,LSPGRP,NAMSPG,NAMSPGCif,
     +      PGNAME,NSYMP,NSYM,RSYM)
        IF (LSPGRP.EQ.0) CALL CCPERR(1,
     +    ' No spacegroup in keyworded input or XYZIN!')
        WRITE(6,35) LSPGRP
 35     FORMAT(' spacegroup from XYZIN :',I6,/)
      ENDIF

C Report input/default parameters
      call ccp4h_summary_beg()
      WRITE(6,'(/,/,A,/)') ' INPUT/DEFAULT PARAMETERS:'
      PRINT *,'title for output map: ',TITLE
      PRINT *,'maximal probe size  :',RMAX
      IF (ENV) THEN
        IF (ENV1) THEN
          PRINT *,'solvent regions are determined from the input pdb'
        ELSE
          PRINT *,'an envelope is calculated around the input pdb'
        ENDIF
      ELSE
        PRINT *,'a cavity map is produced from input pdb'
      ENDIF
      PRINT *,'pdb residue range:',NFIRST,NL
      IF (NEXCL.GT.0) THEN
        WRITE(6,*) ' Atom types excluded:'
        WRITE(6,*) '  ',(EXCL(I),I=1,NEXCL)
      ENDIF
      IF (NCS.GT.1) THEN
	  PRINT *,' number of ncs related molecules =',NCS
          PRINT *,' for each of these an envelope is calculated ',
     +		  '(and numbered according to NCS operation)'
      ENDIF
      WRITE(*,20) CELL
 20   FORMAT(' unit cell :',6F8.3)
      PRINT *,'spacegroup:',LSPGRP
      WRITE(6,'(A,3A2)') ' Axis order: ',AXISF,AXISM,AXISS
      WRITE(6,40) MXYZ
 40   FORMAT(' grid size: ',3I5)
      WRITE(*,30) (MUVW1(I),MUVW2(I),I=1,3)
 30   FORMAT(' output map limits: ',6I5)
      call ccp4h_summary_end()

C
      MU1 = MUVW1(JUVW(1))
      MU2 = MUVW2(JUVW(1))
      MV1 = MUVW1(JUVW(2))
      MV2 = MUVW2(JUVW(2))
      MW1 = MUVW1(JUVW(3))
      MW2 = MUVW2(JUVW(3))
      M1 = MU2 - MU1 + 1
      M2 = MV2 - MV1 + 1
      MS = MW2 - MW1 + 1
      MMAP3 = M1*M2*MS
      IF(MMAP3.GT.NMAP3) THEN
        PRINT *,
     +      ' map too big => reduce map limits with XYZLIM keyword'
        PRINT *,' requested size is',M1,M2,MS
        PRINT *,' corresponding to',MMAP3,' grid points'
        PRINT *,' maximum allowed:',NMAP3,' grid points'
        CALL CCPERR(1,' REDUCE MAP SIZE')
      ENDIF
C
      STMUVW(1) = CELL(IU)/MXYZ(IU)
      STMUVW(2) = CELL(IV)/MXYZ(IV)
      STMUVW(3) = CELL(IW)/MXYZ(IW)
C
C---open output cavity map/envelope
      IO4 = 8
      LMODE = 2
      CALL MWRHDL(IO4,'MAPOUT',TITLE,MS,JUVW,MXYZ,MW1,
     1              MU1,MU2,MV1,MV2,CELL,LSPGRP,LMODE)
      CALL MSYPUT(10,LSPGRP,IO4)
C
C---calculate scale matrix from cell dimensions
      CALL CCP4UC_F_FRAC_ORTH_MAT(CELL, 1, RO3, SC, CELLVOL)

        PRINT *,' '
        PRINT *,' TRANSFORMATION TO FRACTIONAL COORDINATES:'
        WRITE(*,111) ((SC(I,J),J=1,3),I=1,3)
  111   FORMAT(1X,3F10.7)

C---calculate FVOL which is used in developmental SHRINK option
C   not sure that it is correct
      A = CELL(1)
      B = CELL(2)
      C = CELL(3)
      ALP = CELL(4)
      BET = CELL(5)
      GAM = CELL(6)
      DEG2RAD = ATAN(1.0)/45.0
      SING = SIN(GAM*DEG2RAD)
      SINA = SIN(ALP*DEG2RAD)
      SINB = SIN(BET*DEG2RAD)
      FVOL = SINA*SINB*SING*A*B*C/FLOAT(MXYZ(1)*MXYZ(2)*MXYZ(3))
C
C---transformation from pdb to grid coordinates:
      FTOG(1,1) = MXYZ(1)
      FTOG(2,2) = MXYZ(2)
      FTOG(3,3) = MXYZ(3)
      CALL XMAT33(FTOG,SC,PTOG)
      CALL INVRS(PTOG,DET,GTOP)
        PRINT *,' '
        PRINT *,' TRANSFORMATION TO GRID COORDINATES:'
        WRITE(*,111) ((PTOG(I,J),J=1,3),I=1,3)
C
C---read all atoms
      FN = FLOAT(MXYZ(IW)+1)
      IA = 0
      IAT = 0

 115  CONTINUE
      CALL XYZADVANCE(IXYZIN, 0, 0, *300, *300)
      CALL XYZCOORD(IXYZIN,'O','O',XPDB(1),XPDB(2),XPDB(3),Q,B,U)
      CALL XYZATOM(IXYZIN, IATOM, NAME, RES, CHNNAM, NRES, 
     +             RESNO, INSCOD, ALTCOD, IDENT, IZ, ID)
      IA = IA + 1

C---filter according to keywords RANGE and EXCLUDE
        IF(NRES.GE.NFIRST .AND. NRES.LE.NL) THEN
          DO 125 J=1,NEXCL
            IF(NAME(1:3).EQ.EXCL(J)) THEN
              JEXCL1 = JEXCL1 + 1
              GOTO 115
            ENDIF 
  125     CONTINUE
C 
C---select the Van der Waals radius:
          IF(RMAX .GT. 0.0) THEN
            CALL VDWRAD(NAME,RES,RAD)
            IF(RAD+RMAX .GT. RMAX1) RMAX1 = RAD + RMAX
          ELSE
            RAD = 0.0
          ENDIF
C
C---for all non-crystallographic symmetry operations:
          DO 180 J=1,NCS

C---Generate coordinates X1 for NCS related molecules
c   By default, first one is molecule from PDB file
           CALL TRAFO(X1,RNCS(1,1,J),XPDB,TNCS(1,J))

           DO 170 K=1,NSYM

C---Convert to fractional coordinates
             CALL TRAFO(X2,SC,X1,TC)

C---Generate coordinates X2 for symmetry related molecules
             DO I=1,3
               X3(I)=RSYM(I,4,K)
               DO II=1,3
                 X3(I)=X3(I)+RSYM(I,II,K)*X2(II)
               ENDDO
             ENDDO

C---Loop over lattive translations (up to one in any direction)
             DO 161 NX=-IUC,IUC
              X4(1) = X3(1) + REAL(NX)
              DO 162 NY=-IUC,IUC
               X4(2) = X3(2) + REAL(NY)
               DO 163 NZ=-IUC,IUC
                X4(3) = X3(3) + REAL(NZ)

                IAT = IAT + 1
                IF (IAT.GT.MAXATM) THEN
                  WRITE(6,'(/,A,A,I7,/,A,A,/)')
     +  ' FATAL ERROR: number of atoms (including symmetry mates)',
     +  ' exceeds maximum allowed: ',MAXATM,
     +  ' Increase parameter MAXATM and recompile, or apply RANGE',
     +  ' and EXCL keywords.'
                  CALL CCPERR(1,'Maximum number of atoms exceeded.')
                ENDIF
C
C---transform to grid coordinates:
                CALL TRAFO(XX,FTOG,X4,TC)

                DO 150 I=1,3
                 IF (K.EQ.1.AND.NX.EQ.0.AND.NY.EQ.0.AND.NZ.EQ.0) THEN
                   XXMIN(I) = MIN(XX(I),XXMIN(I))
                   XXMAX(I) = MAX(XX(I),XXMAX(I))
                 ENDIF
                 IF(XX(I)+RAD+RMAX .LT. REAL(MUVW1(I)-1) .OR. 
     $               XX(I)-RAD-RMAX .GT. REAL(MUVW2(I))) THEN
                   JEXCL2 = JEXCL2 + 1
                   IAT = IAT - 1
                   GOTO 163
                 ENDIF
                 YY(I,IAT) = XX(I) 
  150           CONTINUE

                NT(IAT) = IAT
                RADIUS(IAT) = RAD
                Z(IAT) = XX(IW)/FN
                NMOL(IAT) = J

  163          CONTINUE
  162         CONTINUE
  161        CONTINUE
  170      CONTINUE
  180     CONTINUE
        ELSE
          JEXCL3 = JEXCL3 + 1
        ENDIF
      GOTO 115

 300  CONTINUE
C Summarise atom input
      call ccp4h_summary_beg()

      WRITE(6,'(A)') ' '
      WRITE(6,'(15X,I6,A)') IA,' atoms found in model'
      WRITE(6,'(A15,I6,A)') 
     +  'of which ',JEXCL3,' outside specified residue range'
      WRITE(6,'(A15,I6,A)') 
     +  'and ',JEXCL1,' excluded by EXCLUDE keyword'
      WRITE(6,'(A)') ' '
      WRITE(6,'(15X,I6,A)') NCS,' NCS operation(s) included'
      WRITE(6,'(A15,I6,A)') 
     +  'generating',NCS*(IA-JEXCL3-JEXCL1),' atoms'
      WRITE(6,'(A)') ' '
      WRITE(6,'(2X,A)') 'Minimum grid coordinates along a/b/c:'
      WRITE(6,'(2X,3F10.3)') XXMIN
      WRITE(6,'(2X,A)') 'Maximum grid coordinates along a/b/c:'
      WRITE(6,'(2X,3F10.3)') XXMAX
      WRITE(6,'(A)') ' '
      WRITE(6,'(2X,A)') 'After generation of symmetry mates:'
      WRITE(6,'(11X,I10,A)') JEXCL2,' atoms outside map limits'
      WRITE(6,'(14X,I7,A)') IAT,' atoms used for map calculation'
      WRITE(6,'(A)') ' '
      RMAX1 = MAX(RMAX1,RMAX)

      call ccp4h_summary_end()
C 
C---sort atoms according to section 
      CALL SORT1 (IAT,Z,NT)
C
C---calculate cavities
      IF(NENV.EQ.1) IAT = -IAT
      CALL CALRHO(G1,T1,E1,E2,NT,M1,M2,MS,IAT,RMAX,RMAX1,FVOL)

C   check map and write header information
      CALL MRDHDR(IO4,'MAPOUT',TITLE,MS,JUVW,MXYZ,MW1,
     +              MU1,MU2,MV1,MV2,CELL,LSPGRP,LMODE,
     +              RHOMIN,RHOMAX,RHMEAN,RRMS)
      CALL MRCLOS(IO4)

      CALL CCPERR(0, 'Normal termination')
      END
C
C--------*---------*---------*---------*---------*---------*---------*--
      SUBROUTINE SORT1(N,A0,I0)
C     =========================
C
C****
C**** N IS THE LENGTH OF THE ARRAY TO BE SORTED I.E. THE NUMBER OF
C**** ITEMS TO BE SORTED
C**** THE ELEMENTS OF ARRAY A0(N) ARE SORTED INTO INCREASING ORDER OF
C**** MAGNITUDE.  ARRAY I0 IS SORTED ACCORDING TO A0
C****
      DIMENSION A0(N),I0(N)
      NO2 = N/2
      DO 100 I=1,NO2
      C = A0(I)
      D = C
      IH = I
      K = I
      IP1 = I+1
      NP1 = N+1-I
      DO 20 J=IP1,NP1
      A = A0(J)
      IF ( A.GE.C ) GO TO 15
      IH = J
      C = A
      GO TO 20
   15 IF ( D.GE.A ) GO TO 20
      D = A
      K = J
   20 CONTINUE
      IF ( C.EQ.D ) GO TO 200
      IF ( IH.EQ.I ) GO TO 30
    2 C = I0(IH)
      I0(IH) = I0(I)
      I0(I) = C
    1 C = A0(IH)
      A0(IH) = A0(I)
      A0(I) = C
   30 IF ( K.EQ.I ) K=IH
      IF ( K.EQ.NP1 ) GO TO 100
   32 D = I0(K)
      I0(K) = I0(NP1)
      I0(NP1) = D
   31 D = A0(K)
      A0(K) = A0(NP1)
      A0(NP1) = D
  100 CONTINUE
  200 RETURN
      END
C
      SUBROUTINE TRAFO (A,B,C,D)
C 
C---- A = [B]*C + D 
C 
      DIMENSION A(3),B(3,3),C(3),D(3) 
      DO 20 I=1,3 
        A(I) = D(I) 
        DO 10 J=1,3 
          A(I) = A(I) + B(I,J)*C(J) 
   10   CONTINUE
   20 CONTINUE
      RETURN
      END 
C
      SUBROUTINE XMAT33(A,B,C)
C///  VERSION 83/07/28 (LJL (INS.)) 
* ------------- COMPUTE [C] = [A] * [B]         A,B,C ARE 3*3 MATRICES
*      [C] MAY BE THE SAME AS EITHER [A] OR [B].
C\\\
      DIMENSION A(3,3),B(3,3),C(3,3), A1(3,3), B1(3,3)
      DO 1 N=1,3
        DO 1 M=1,3
          A1(N,M) = A(N,M)
          B1(N,M) = B(N,M)
1         C(N,M) = 0. 
      DO 30 N=1,3 
        DO 20 M=1,3 
          DO 10 J=1,3 
10          C(N,M) = C(N,M) + A1(N,J)*B1(J,M) 
20      CONTINUE
30    CONTINUE
      RETURN
      END 
C
C
      SUBROUTINE INVRS(X, DETX, XINV) 
C///
* 
*      CALCULATES THE INVERS [XINV] AND THE DETERMINANT 'DETX' OF A 
*      3X3 MATRIX [X];  [XINV] MAY BE THE SAME ARRAY AS [X].
*      IF DETX = 0, [XINV] WILL BE LEFT UNDEFINED.
C\\\
*     SLIGHTLY MODIFIED     27/10/76     LJL
      DIMENSION X(9), XINV(9) 
      DIMENSION XX(9),ZZ(9) 
      DO 10 I = 1,9 
        XX(I) = X(I)
10    CONTINUE
      ZZ(1) = XX(5)*XX(9)-XX(6)*XX(8) 
      ZZ(2) = XX(3)*XX(8)-XX(2)*XX(9) 
      ZZ(3) = XX(2)*XX(6)-XX(3)*XX(5) 
      ZZ(4) = XX(6)*XX(7)-XX(4)*XX(9) 
      ZZ(5) = XX(1)*XX(9)-XX(3)*XX(7) 
      ZZ(6) = XX(3)*XX(4)-XX(1)*XX(6) 
      ZZ(7) = XX(4)*XX(8)-XX(5)*XX(7) 
      ZZ(8) = XX(2)*XX(7)-XX(1)*XX(8) 
      ZZ(9) = XX(1)*XX(5)-XX(2)*XX(4) 
      DETX = ZZ(1)*XX(1) + ZZ(2)*XX(4) + ZZ(3)*XX(7)
      IF (DETX.EQ.0.) RETURN
      DO 20 I = 1,9 
        XINV(I) = ZZ(I)/DETX
20    CONTINUE
      RETURN
      END 
C
C-----------------------------------------------------------------------
      SUBROUTINE CALRHO(R1,DMAP,S1,S2,NT,M1,M2,MS,IA,RMAX,RMAX1,FVOL)
      PARAMETER ( MAXATM=50000 )
      COMMON /CAV/ MU1,MV1,MW1,JUVW(3),STMUVW(3)
      COMMON /XYZB/ GTOP(3,3),YY(4,MAXATM),NMOL(MAXATM),DET,CELLVOL
      COMMON /LOGIC/ ENV,ENV1,SOLVNT
      COMMON / / RADIUS(MAXATM),XGRID(3,MAXATM)
      DIMENSION R1(M1,M2,MS),NT(IA),DMAP(M1,M2,MS)
      LOGICAL S1(M1,M2,MS),S2(M1,M2,MS)
      LOGICAL ENV,ENV1,SOLVNT
      PI = 4.0 * ATAN(1.0)
      ITOT = 0
      NREJ = 0
      RHMEAN = 0.0
      RHOMIN = 9999999.
      RHOMAX =-9999999.
      RMAX  = ABS(RMAX)
      RMAX2 = RMAX1**2
      RRMS   = 0.0
      NFIRST = 1
      IO4 = 8
      J1 = JUVW(1)
      J2 = JUVW(2)
      J3 = JUVW(3)
      IMAX = NINT(ABS(RMAX1))
C
       STEP = 0.0
       DO 10 J=1,3
       DO 10 I=1,3
         STEP = MAX(STEP,GTOP(I,J))
   10  CONTINUE
       PRINT *,' grid stepsize =',STEP
C
C---do transformation from crystallographic Angstroem
C---to grid coordinates
      IF(IA.LT.0) THEN
        IA = -IA
      NENV = 1
      ELSE
        NENV = 0
      ENDIF
      DO 40 L=NFIRST,IA
      JA = NT(L)
      DO 30 J=1,3
        JJ = JUVW(J)
        XGRID(J,L) = YY(JJ,JA)
   30      CONTINUE
      XGRID(1,L) = XGRID(1,L) - FLOAT(MU1-1)
      XGRID(2,L) = XGRID(2,L) - FLOAT(MV1-1)
      XGRID(3,L) = XGRID(3,L) - FLOAT(MW1-1)
      IG1 = NINT(XGRID(1,L))
      IG2 = NINT(XGRID(2,L))
      IG3 = NINT(XGRID(3,L))
      IF(IG1.LT.1 .OR. IG1.GT.M1 .OR. IG2.LT.1 .OR.
     $     IG2.GT.M2 .OR. IG3.LT.1 .OR. IG3.GT.MS) THEN
        NREJ = NREJ + 1
        ENDIF
   40 CONTINUE
C 
C---START BIG LOOP OVER ALL ATOMS
C***************************************************************
      DO 450 K=1,MS
      G3 = FLOAT(K)
      NTOT = 0
C
C---initialize section
      DO 50 J=1,M2
      DO 50 I=1,M1
        IF(ENV) THEN
          R1(I,J,K) = 0.0
          DMAP(I,J,K) = 999.99
        ELSE IF(ENV1) THEN
          R1(I,J,K) = 1.0
          DMAP(I,J,K) = 999.99
        ELSE
          R1(I,J,K) = RMAX
          S1(I,J,K) = .TRUE.
        ENDIF
   50      CONTINUE
C
C---search first atom relevant for this section
      DO 100 L=NFIRST,IA
        XG3 = XGRID(3,L)
        CALL DORTHO(DZ2,J3,XG3,G3)
        IF(DZ2.LT.RMAX2) GOTO 220
  100   CONTINUE
C
C---no more atoms within rmax1 angstroem of this section
      GOTO 450
  220   NFIRST = L
C
C---for all atoms contributing to this section:
      DO 400 L=NFIRST,IA
        JA = NT(L)
        XG1 = XGRID(1,L)
        XG2 = XGRID(2,L)
        XG3 = XGRID(3,L)
        IMOL = NMOL(JA)
          ITOT = ITOT + 1
C
C---last relevant atom:
        CALL DORTHO(DZ2,J3,XG3,G3)
        IF(DZ2.GT.RMAX2) GOTO 450
        RAD = RADIUS(JA)
        RADT = RAD+RMAX
        SQRRAD = RADT**2
        IF(DZ2.GT.SQRRAD) GOTO 400
C
C---grid point boundaries:
        R2 = SQRRAD-DZ2
        R = SQRT(R2)
        DELI = R/STMUVW(1)
        DELJ = R/STMUVW(2)
C
        DIFX = XG1 - DELI
        SUMX = XG1 + DELI
        IFIRST = MAX(INT(DIFX)-1 , 1)
        ILAST  = MIN(INT(SUMX)+1 , M1)
C
        DIFX = XG2 - DELJ
        SUMX = XG2 + DELJ
        JFIRST = MAX(INT(DIFX)-1 , 1)
        JLAST  = MIN(INT(SUMX)+1 , M2)
C
C---calculate cavity map
        DO 350 I=IFIRST,ILAST
            G1 = FLOAT(I)
          CALL DORTHO(RAD1,J1,XG1,G1)
            IF(RAD1.GT.R2) GOTO 350
          DO 300 J=JFIRST,JLAST
            G2 = FLOAT(J)
            CALL DORTHO(RAD2,J2,XG2,G2)
            RAD12 = RAD1 + RAD2
            IF(RAD12.GT.R2) GOTO 300
            NTOT = NTOT + 1
            DIST = SQRT(DZ2+RAD12)
            IF(ENV) THEN
              IF(DIST.LT.RADT) THEN
              IF(DIST.LT.DMAP(I,J,K)) THEN
                R1(I,J,K) = IMOL
                DMAP(I,J,K) = DIST
              ENDIF
            ENDIF
            ELSE IF(DIST.LT.RAD) THEN
            R1(I,J,K) = 0.0
            S1(I,J,K) = .FALSE.
            ELSE
            DELTA = DIST - RAD
            R1(I,J,K) = MIN(R1(I,J,K),DELTA)
            S1(I,J,K) = .FALSE.
            ENDIF
  300          CONTINUE
  350        CONTINUE
  400      CONTINUE
  450 CONTINUE
C
C---update map statistics
      IF(SOLVNT) THEN
        IF(NENV.EQ.1) STEP = -STEP
        CALL SHRINK(R1,DMAP,S1,S2,M1,M2,MS,STEP,IA,FVOL,RMAX)
      ELSE
        WRITE(*,480)
  480   FORMAT(/,' OUTPUT MAP:',/,' section   minimum   maximum')
      ENDIF
C

      NUM_GRID_SOLV=0
      NUM_GRID_PROT=0

      DO 600 K=1,MS
      RHMIN = 1000.
      RHMAX =-1000.
        DO 560 J=1,M2
      DO 560 I=1,M1
        IF(ENV1) THEN
          IF(R1(I,J,K) .NE. 0.0) THEN
            R1(I,J,K) = 0.0
          ELSE
            R1(I,J,K) = 1.0
          ENDIF
        ENDIF
C
        VAL = R1(I,J,K)
        RHMIN = MIN(RHMIN,VAL)
        RHMAX = MAX(RHMAX,VAL)
        RRMS = RRMS + VAL*VAL
        RHMEAN = RHMEAN + VAL

CMDW  Estimate solvent fraction
        IF (R1(I,J,K).GT.0.0001) THEN
          IF(ENV .AND. .NOT.ENV1) THEN
            NUM_GRID_PROT=NUM_GRID_PROT+1
          ELSE  
            NUM_GRID_SOLV=NUM_GRID_SOLV+1
          ENDIF
        ELSE
          IF(ENV .AND. .NOT.ENV1) THEN
            NUM_GRID_SOLV=NUM_GRID_SOLV+1
          ELSE
            NUM_GRID_PROT=NUM_GRID_PROT+1
          ENDIF
        ENDIF
 
  560   CONTINUE
        KSEC = MW1 + K - 1
      IF(.NOT.SOLVNT) WRITE(*,580) KSEC,RHMIN,RHMAX
  580      FORMAT(1X,I4,2X,2F10.3)
      RHOMIN = MIN(RHOMIN,RHMIN)
      RHOMAX = MAX(RHOMAX,RHMAX)
      CALL MSPEW(IO4,R1(1,1,K))
  600 CONTINUE
C
C---CLOSE OUTPUT FILE
      CALL MCLOSE(IO4,RHOMIN,RHOMAX,RHMEAN,RRMS)

      WRITE(6,*)
      WRITE(6,*) ' STATISTICS OF PROTEIN AND SOLVENT VOLUMES' 
      WRITE(6,*)
      WRITE(6,*) '   Unit cell colume (A**3) = ',CELLVOL
      WRITE(6,*) '   Total number of map grid points = ',M1*M2*MS
      WRITE(6,*) '   Grid point volume (A**3) = ',1.0/DET
      WRITE(6,*)
      WRITE(6,*) '   Number of solvent grid points = ',NUM_GRID_SOLV
      WRITE(6,*) '   Volume of solvent (A**3) = ',
     +           REAL(NUM_GRID_SOLV)/DET
      WRITE(6,*) '   Fraction of solvent = ',
     +           REAL(NUM_GRID_SOLV)/REAL(M1*M2*MS)
      WRITE(6,*)
      WRITE(6,*) '   Number of protein grid points = ',NUM_GRID_PROT
      WRITE(6,*) '   Volume of protein (A**3) = ',
     +           REAL(NUM_GRID_PROT)/DET
      WRITE(6,*) '   Fraction of protein = ',
     +           REAL(NUM_GRID_PROT)/REAL(M1*M2*MS)

      RETURN
      END
C
C---------------------------------------------------------------
      SUBROUTINE DORTHO(D2,JJ,P1,P2)
      PARAMETER (MAXATM=50000)
        COMMON /XYZB/ GTOP(3,3),YY(4,MAXATM),NMOL(MAXATM),DET,CELLVOL
      DA = P1-P2
      DEL1 = GTOP(1,JJ) * DA
      DEL2 = GTOP(2,JJ) * DA
      DEL3 = GTOP(3,JJ) * DA
      D2 = DEL1**2 +DEL2**2 +DEL3**2
      RETURN
      END
C
C---------------------------------------------------------------
      SUBROUTINE VDWRAD(NAME,RES,RAD)
      CHARACTER*4 NAME,RES
C
C---Initialize Van der Waals radii according to XPLOR
      IF(NAME.EQ.'C   ') THEN
      RAD = 1.87
      ELSE IF(NAME(1:1).EQ.'H') THEN
      RAD = 1.07
      ELSE IF(NAME(1:1).EQ.'O') THEN
      RAD = 1.425
      ELSE IF(NAME(1:1).EQ.'D') THEN
      RAD = 1.2
C
      ELSE IF(NAME(1:1).EQ.'N') THEN
      IF(RES.EQ.'PRO ') THEN
        RAD = 1.87
      ELSE IF(RES.EQ.'ADE ') THEN
        IF(NAME(1:2).EQ.'N1'.OR.NAME(1:2).EQ.'N6') THEN
          RAD = 1.425
        ELSE
          RAD = 1.87
        ENDIF
      ELSE
        RAD = 1.425
      ENDIF
C
      ELSE IF(NAME(1:1).EQ.'S') THEN
      RAD = 1.69
C
      ELSE IF(NAME(1:1).EQ.'P') THEN
      RAD = 1.69
C
      ELSE IF(NAME.EQ.'CA  ') THEN
      IF(RES.EQ.'GLY ') THEN
        RAD = 2.00
      ELSE
        RAD = 2.11
      ENDIF
C
      ELSE IF(NAME.EQ.'CB  ') THEN
      IF(RES.EQ.'ALA ') THEN
        RAD = 1.98
      ELSE IF(RES.EQ.'ILE '.OR.RES.EQ.'THR '.OR.RES.EQ.'VAL ') THEN
        RAD = 2.11
      ELSE
        RAD = 2.00
      ENDIF
C
      ELSE IF(NAME.EQ.'CG  ') THEN
      IF(RES.EQ.'LEU ') THEN
        RAD = 2.11
      ELSE IF(RES.EQ.'ASN '.OR.RES.EQ.'ASP '.OR.RES.EQ.'HIS '
     $      .OR.RES.EQ.'PHE '.OR.RES.EQ.'TYR '.OR.RES.EQ.'TRP ') THEN
        RAD = 1.87
      ELSE
        RAD = 2.00
      ENDIF
C
      ELSE IF(NAME.EQ.'CG1 ') THEN
      IF(RES.EQ.'VAL ') THEN
        RAD = 1.98
      ELSE
        RAD = 2.00
      ENDIF
C
      ELSE IF(NAME.EQ.'CG2 ') THEN
      RAD = 1.98
C
      ELSE IF(NAME(1:2).EQ.'CD') THEN
      IF(RES.EQ.'ARG '.OR.RES.EQ.'LYS '.OR.RES.EQ.'PRO ') THEN
        RAD = 2.00
      ELSE IF(RES.EQ.'ILE '.OR.RES.EQ.'LEU ') THEN
        RAD = 1.98
      ELSE
        RAD = 1.87
      ENDIF
C
      ELSE IF(NAME(1:2).EQ.'CE') THEN
      IF(RES.EQ.'LYS ') THEN
        RAD = 2.00
      ELSE IF(RES.EQ.'MET ') THEN
        RAD = 1.98
      ELSE
        RAD = 1.87
      ENDIF
C
      ELSE IF(NAME(1:2).EQ.'CZ') THEN
      RAD = 1.87
C
      ELSE IF(NAME.EQ.'CH2 ') THEN
      RAD = 1.87
C
      ELSE IF(NAME.EQ.'CU  ') THEN
      RAD = 1.4
C
      ELSE IF(NAME.EQ.'ZN  ') THEN
      RAD = 1.4
C
      ELSE IF(NAME.EQ.'FE  ') THEN
      RAD = 1.4
C
      ELSE IF(NAME.EQ.'MN  ') THEN
      RAD = 1.4
C
      ELSE IF(NAME.EQ.'NI  ') THEN
      RAD = 1.4
C
      ELSE IF(NAME(1:2).EQ.'C1') THEN
      RAD = 2.11
C
      ELSE IF(NAME(1:2).EQ.'C2') THEN
      IF(RES.EQ.'ADE ') THEN
        IF(NAME.EQ.'C2  ') THEN
          RAD = 1.87
        ELSE
          RAD = 2.00
        ENDIF
      ELSE
        RAD = 2.11
      ENDIF
C
      ELSE IF(NAME(1:2).EQ.'C3') THEN
      RAD = 2.11
C
      ELSE IF(NAME(1:2).EQ.'C4') THEN
      IF(RES.EQ.'ADE ') THEN
        IF(NAME.EQ.'C4  ') THEN
          RAD = 1.87
        ELSE
          RAD = 2.11
        ENDIF
      ELSE
        RAD = 2.11
      ENDIF
C
      ELSE IF(NAME(1:2).EQ.'C5') THEN
      IF(RES.EQ.'ADE ') THEN
        IF(NAME.EQ.'C5  ') THEN
          RAD = 1.87
        ELSE
          RAD = 2.00
        ENDIF
      ELSE
        RAD = 2.11
      ENDIF
C
      ELSE IF(NAME.EQ.'C6  ') THEN
      IF(RES.EQ.'ADE ') THEN
        RAD = 1.87
      ELSE
        RAD = 2.00
      ENDIF
C
      ELSE IF(NAME.EQ.'C7  ') THEN
      RAD = 1.87
C
      ELSE IF(NAME.EQ.'C8  ') THEN
      IF(RES.EQ.'ADE ') THEN
        RAD = 1.87
      ELSE
        RAD = 1.98
      ENDIF
C
      ELSE IF(NAME(1:2).EQ.'CH' .OR. NAME(1:2).EQ.'CG') THEN
      RAD = 1.87
C
      ELSE IF(NAME(1:3).EQ.'CAB' .OR. NAME(1:2).EQ.'CAC') THEN
      RAD = 1.87
C
      ELSE IF(NAME(1:2).EQ.'CM') THEN
      RAD = 1.93
C
      ELSE IF(NAME(1:1).EQ.'C') THEN
      RAD = 1.99
C
      ELSE
      WRITE(*,10) NAME,RES
   10   FORMAT(/1X,'unknown atom ',A4,' in residue ',A4,
     $         ': assumed radius = 1.40 A')
      RAD = 1.40
      ENDIF
C
      RETURN
      END
C
      SUBROUTINE SHRINK(R1,R2,S1,S2,M1,M2,MS,STEP,NAT,FVOL,RMAX)
      DIMENSION R1(M1,M2,MS),R2(M1,M2,MS)
      LOGICAL TEST,ENV,S1(M1,M2,MS),S2(M1,M2,MS)
      TEST = .TRUE.
      IF(STEP .LT. 0.0) THEN
        ENV = .TRUE.
        EDMIN = 1.4
      ELSE
        ENV = .FALSE.
        EDMIN = 0.0
      ENDIF
C
        THRES = RMAX
        NSOL = 0
        DO 100 K=1,MS
        DO 100 J=1,M2
        DO 100 I=1,M1
          IF(S1(I,J,K)) NSOL = NSOL + 1
C
C---take care of map edges and most distant bulk solvent
          IF(I.EQ.1 .OR. J.EQ.1 .OR. K.EQ.1 .OR.
     $            I.EQ.M1 .OR. J.EQ.M2 .OR. K.EQ.MS) THEN
                 IF(R1(I,J,K).GT.0 .AND. .NOT.S1(I,J,K)) THEN
              NMOD = NMOD + 1
              S1(I,J,K) = .TRUE.
            ENDIF
            ENDIF
          IF(.NOT.S1(I,J,K) .AND. R1(I,J,K).EQ.THRES) THEN
            S1(I,J,K) = .TRUE.
            NMOD = NMOD + 1
          ENDIF 
          S2(I,J,K) = S1(I,J,K)
  100        CONTINUE
C
          NG = M1*M2*MS
        NP = NG - NSOL
          PRINT *,' number of grid points in map:             ', NG
          PRINT *,' number of solvent points before shrinking:', NSOL
        PRINT *,' number of protein points before shrinking:', NP 
        PRINT *,' number of edge points added too solvent:  ', NMOD
        PRINT *,' '
C
C---gradually add layers of gridpoints to bulk solvent
      T1 = THRES
      THRES = THRES - STEP
      NMIN = 3
      PRINT *,' THRES,T1,NMIN=',THRES,T1,NMIN
      DO 500 NPASS=1,200
       NMOD = 0
       DIFMAX = 0.0
C
       DO 200 I=1,M1
        I1 = MAX(I-1,1)
        I2 = MIN(I+1,M1)
        DO 180 J=1,M2
         J1 = MAX(J-1,1)
         J2 = MIN(J+1,M2)
         DO 160 K=1,MS
C
C---for all points not yet assigned to bulk solvent:
          IF(.NOT.S1(I,J,K)) THEN
            K1 = MAX(K-1,1)
            K2 = MIN(K+1,MS)
            NSOLV = 0
            RAD1 = 0.0
            RAD1A = R1(I,J,K)
C
C---check if higher than current threshold
            IF(RAD1A .GT. THRES) THEN
              NCH = 0
              DO 130 II=I1,I2
C
C---count direct connections to bulk solvent
                IF(S1(II,J,K)) THEN
                NCH = NCH + 1
                RAD1 = RAD1 + R1(II,J,K)
              ENDIF
  130            CONTINUE
C
              DO 140 JJ=J1,J2
                IF(S1(I,JJ,K)) THEN
                NCH = NCH + 1
                RAD1 = RAD1 + R1(I,JJ,K)
              ENDIF
  140            CONTINUE
C
              DO 150 KK=K1,K2
                IF(S1(I,J,KK)) THEN
                NCH = NCH + 1
                RAD1 = RAD1+ R1(I,J,KK)
              ENDIF
  150            CONTINUE
C
C---accept if average value of bulk solvent neighbours is higher or
C---if at least 3 bulk solvent neighbours + leftover from previous level
            IF(NCH.GT.0) THEN
             RAD1 = RAD1/FLOAT(NCH)
             IF(RAD1.GT.RAD1A.OR.(NCH.GE.NMIN.AND.RAD1A.GT.T1)) THEN
                    NMOD = NMOD + 1
              S2(I,J,K) = .TRUE.
             ENDIF
            ENDIF
              ENDIF
          ENDIF
  160         CONTINUE
  180        CONTINUE
  200       CONTINUE
C
         IF(NMOD.EQ.0 .AND. T1 .EQ. 0.0) THEN
         GOTO 501
       ELSE IF(NMOD.EQ.0) THEN
         T1 = THRES
         IF(T1.LE.STEP) NMIN = 4
         THRES = THRES - STEP
c         NPASS = NPASS - 1
         PRINT *,' THRES,T1,NMIN=',THRES,T1,NMIN
       ELSE IF(T1 .EQ. 0.0) THEN
         WRITE(*,210) NMOD,NPASS,T1
       ELSE
         WRITE(*,210) NMOD,NPASS,THRES
  210         FORMAT(1X,I10,' gp added to bulk solvent in pass',I4,
     $            ' at level',F8.3)
       ENDIF
C
       NSOL = 0
       DO 300 K=1,MS
       DO 300 J=1,M2
       DO 300 I=1,M1
         S1(I,J,K) = S2(I,J,K)
         IF(S1(I,J,K)) NSOL = NSOL + 1
  300       CONTINUE
c         PRINT *,NSOL,' bulk solvent points in current cavity map'
  500      CONTINUE
C
  501      NPROT = 0
      NSOL = 0
      NCAV = 0
      N05 = 0
      N10 = 0
      N15 = 0
      N20 = 0
      NFIN = 0
      RHOMAX = 0.0
      IMAX = 0
      JMAX = 0
      KMAX = 0
C
      IF(TEST) THEN
       PRINT *,' '
       PRINT *,' 103 18 92',R1(103,18,92)
       WRITE(*,650) ((R1(I,J,90),J=16,20),I=101,105)
       WRITE(*,650) ((R1(I,J,91),J=16,20),I=101,105)
       WRITE(*,650) ((R1(I,J,92),J=16,20),I=101,105)
       WRITE(*,650) ((R1(I,J,93),J=16,20),I=101,105)
       WRITE(*,650) ((R1(I,J,94),J=16,20),I=101,105)
      ENDIF
C
      DO 600 K=1,MS
        K1 = MAX(K-1,1)
        K2 = MIN(K+1,MS)
      DO 600 J=1,M2
        J1 = MAX(J-1,1)
        J2 = MIN(J+1,M2)
      DO 600 I=1,M1
        I1 = MAX(I-1,1)
        I2 = MIN(I+1,M1)
C
C---first check that point is not isolated
        IF(.NOT.S1(I,J,K)) THEN
          NGH = 0
          NCH = 0
          DO 520 II=I1,I2
            IF(II.NE.I) THEN
              NGH = NGH + 1
              IF(S1(II,J,K)) NCH = NCH + 1
            ENDIF
  520          CONTINUE
          DO 540 JJ=J1,J2
            IF(JJ.NE.J) THEN
              NGH = NGH + 1
              IF(S1(I,JJ,K)) NCH = NCH + 1
            ENDIF
  540          CONTINUE
          DO 560 KK=K1,K2
            IF(KK.NE.K) THEN
              NGH = NGH + 1
              IF(S1(I,J,KK)) NCH = NCH + 1
            ENDIF
  560          CONTINUE
            IF(NCH.GT.3) THEN
            NFIN = NFIN + 1
            S1(I,J,K) = .TRUE.
          ENDIF
        ENDIF
C
        IF(S1(I,J,K)) THEN
          IF(ENV) THEN
            R1(I,J,K) = 0.0
          ELSE
            R1(I,J,K) = -1.0
          ENDIF
          NSOL = NSOL + 1
        ELSE
          NPROT = NPROT + 1
          IF(R1(I,J,K) .GT. 0.0) NCAV = NCAV + 1
          IF(R1(I,J,K) .GT. 0.5) N05 = N05 + 1
          IF(R1(I,J,K) .GT. 1.0) N10 = N10 + 1
          IF(R1(I,J,K) .GT. 1.5) N15 = N15 + 1
          IF(R1(I,J,K) .GT. 2.0) N20 = N20 + 1
          IF(ENV) R1(I,J,K) = 1.0
        ENDIF
C
        IF(R1(I,J,K).GT.RHOMAX) THEN
          RHOMAX = R1(I,J,K)
          IMAX = I
          JMAX = J
          KMAX = K
        ENDIF
  600      CONTINUE
C
      IF(TEST) THEN
       PRINT *,' '
       PRINT *,IMAX,JMAX,KMAX,RHOMAX
       WRITE(*,650) ((R1(I,J,KMAX-2),J=JMAX-2,JMAX+2),I=IMAX-2,IMAX+2)
       WRITE(*,650) ((R1(I,J,KMAX-1),J=JMAX-2,JMAX+2),I=IMAX-2,IMAX+2)
       WRITE(*,650) ((R1(I,J,KMAX),J=JMAX-2,JMAX+2),I=IMAX-2,IMAX+2)
       WRITE(*,650) ((R1(I,J,KMAX+1),J=JMAX-2,JMAX+2),I=IMAX-2,IMAX+2)
       WRITE(*,650) ((R1(I,J,KMAX+2),J=JMAX-2,JMAX+2),I=IMAX-2,IMAX+2)
      ENDIF
  650      FORMAT(5(1X,5F4.1))
C      
      PRINT *,' '
      PRINT *,NFIN,' points added to bulk solvent in final pass'
      PRINT *,NSOL,' bulk solvent points in final cavity map' 
      PRINT *,NPROT,' protein points in final cavity map' 
      PRINT *,' '
      VOLP = FVOL * NPROT
      AV = VOLP/FLOAT(NAT)
      PRINT *,' protein volume (in cubic Angstroems) =',VOLP
      PRINT *,' average volume per atom              =',AV
      PRINT *,' '
      ESV = FVOL * NCAV
      ACV = FVOL * N05
      PRINT *,' total volume of internal empty space =',ESV
      PRINT *,' accessible volume for probe with 0.5 Angstroem radius:',
     $            ACV 
      ACV = FVOL * N10
      PRINT *,' accessible volume for probe with 1.0 Angstroem radius:',
     $            ACV 
      ACV = FVOL * N15
      PRINT *,' accessible volume for probe with 1.5 Angstroem radius:',
     $            ACV 
      ACV = FVOL * N20
      PRINT *,' accessible volume for probe with 2.0 Angstroem radius:',
     $            ACV 
      PRINT *,' '
C
      RETURN
      END
