/*
     c_contour.h: header file for 2d contouring functions
     Copyright (C) 2001  CCLRC, Peter Briggs

     This code is distributed under the terms and conditions of the
     CCP4 Program Suite Licence Agreement as a CCP4 Library.
     A copy of the CCP4 licence can be obtained by writing to the
     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
*/

/*******************************************************************/

/* c_contour.h

   Header file for 2d contouring functions
*/


/*------------------------------------------------------------------------------*/
/* Macro definitions */
/*------------------------------------------------------------------------------*/

/* These define different situations for contours crossing
   the edge of a cell */
#define CROSSING       1
#define NO_EXIT        0
/* These define the status of the loop for tracing a contour */
#define CONTINUE       1
#define STOP           0
/* Set DIAG to 1 for diagnostic output */
#define DIAG           0

/*------------------------------------------------------------------------------*/
/* Function Prototypes */
/*------------------------------------------------------------------------------*/

/* contour_main constitutes the "user interface" to the contouring
   functionality */

int contour_main(int nx, int ny, float **farray, int ncontours, float *fcontour,
		 int outfunc(float isovalue, int n, float *x, float *y));

/* The rest of the functions are internal and shouldn't be accessed
   directly from an application */

int find_contours(float isovalue, int nx, int ny, float **farray, int ncell,
		  int *workspace,
		  int outfunc(float isovalue, int n, float *x, float *y));

int examine_cell(int ix, int iy, int nx, int ny, float isovalue, float **farray,
		 int ncell, int *workspace);

int cell_index(int ix, int iy, int nx);

int test_cell(int ix, int iy, int iedge,
	      float isovalue, int nx, int ny, float **farray,
	      int ncell, int *workspace,
	      float *xpos, float *ypos, int *nextx, int *nexty, int *nextedge);

int cell_edge_index(int ix, int iy, int iedge, int nx, int ny, int ncell);

int get_neighbouring_cell_for_edge(int ix, int iy, int iedge, int nx, int ny,
				   int *jx, int *jy);

int get_neighbouring_cell_for_corner(int ix, int iy, int icorner, int nx, int ny,
				     float isovalue, float **farray,
				     int *workspace, int *jx, int *jy);

int get_crossover_position(int ix, int iy, int iedge, float rho1, float rho2,
			   float *xpos, float *ypos);

float linear_interp(float v0, float v1);

int initial_point_is_corner(int ncoords, float **xptr, float **yptr);

int start_new_contour(int *nptr, float **xptr, float **yptr);

int follow_contour(int init_x, int init_y,
		   float isovalue, int nx, int ny, float **farray,
		   int ncell, int *workspace,
		   int *ncoords, float **xptr, float **yptr);

int extend_contour(float xpos,float ypos,int *nptr,
		   float **xptr, float **yptr);

int flip_contour(int ncoords,float **xptr,float **yptr);

int close_contour(int *nptr, float **xptr, float **yptr, int nx, int ny);
