/*
     ccp4mapwish.h: header file for ccp4mapwish
     Copyright (C) 2000  CCLRC, Peter Briggs

     This code is distributed under the terms and conditions of the
     CCP4 Program Suite Licence Agreement as a CCP4 Application.
     A copy of the CCP4 licence can be obtained by writing to the
     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
*/

/*******************************************************************/

/* ccp4mapwish.h

   Header file for ccp4mapwish

   Initial version: Peter Briggs Nov 2000
   Revised Oct 2001
   CVS id string: $Id: ccp4mapwish.h,v 1.12 2005/09/08 13:53:25 mdw Exp $
*/

/*******************************************************************/

/* Include external header files */

/*******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

/* We need tcl.h for the definition of Tcl_Obj
   Must be version 8.0 or better */
#include <tcl.h>
#include <tk.h>

/* CCP4 CMap include file */
#include <cmaplib.h>

/*******************************************************************/

/* Define statements */

/*******************************************************************/

/* Package name */
# define CCP4MAPWISH_PACKAGE_NAME "ccp4map"

/* Version */
# define CCP4MAPWISH_MAJOR_VERSION 3
# define CCP4MAPWISH_MINOR_VERSION 2
# define CCP4MAPWISH_VERSION "4.0"

/* Useful hardcoded limits */
# define IPTMAX  200
# define CCP4_LTMPSTR 500
# define LEN_AXISLABEL 30

/* Diagnostics */
# define CCP4MAPWISH_DIAG 0
# define CCP4MAPWISH_MAP_DIAG 0
# define CCP4MAPWISH_SECTION_DIAG 0
# define CCP4MAPWISH_UTILS_DIAG 0
# define CCP4MAPWISH_CANVAS_DIAG 0
# define CCP4MAPWISH_CONTOUR_DIAG 0

/*******************************************************************/

/* Function Macros */

/*******************************************************************/

/* This is required for Microsoft Visual Studio

   On UNIX: fabsf is for FLOATS, fabs for DOUBLES on UNIX
   On NT/MVS: fabsf doesn't exist, fabs is for FLOATS on NT/MVS

   So we need to rename fabsf to the correct name before
   compiling on this system.
*/
#if defined(_MSC_VER) || defined (sun) || defined (__sun)
#  define fabsf(x) fabs(x)
#endif


/*******************************************************************/

/* Structures and typedefs */

/*******************************************************************/

/* Structure for map storage */
typedef struct {
  char  *mapid;        /* Handle name for map structure */ 
  char  *filename;     /* Filename from which map was read in  */
  char  *title;        /* Title taken from map header */
  float cell[6];       /* Cell parameters */
  int   iorth;         /* Orthogonalisation code */
  float frac_to_orth[3][3]; /* Conversion matrix (fractional->orthogonal) */
  float orth_to_frac[3][3]; /* Conversion matrix (orthogonal->fractional) */
  float rhmin,rhmax,rhmean,rhrms; /* Stats on the density */
  int   lspgrp;        /* Spacegroup number */
  int   mxyz[3];       /* Number of intervals along whole cell edges xyz */
  int   ext[3];        /* Number of grid intervals along xyz in map */
  int   start[3];      /* Number of first grid point along xyz */
  int   mode;          /* Mode of storage used in the mapfile */
  float ***map;        /* values in the map */
} CCP4MAP;

/* Array of maps */
typedef struct {
  int     nmaps;       /* Number of maps in array */
  CCP4MAP **map_ptr;   /* Array of pointers to map structures */
} CCP4MAPARRAY;

/* Structure for section/slab storage */
typedef struct {
  char  *sectionid;    /* Handle name for section structure */
  char  *mapid;        /* Handle for parent map */
  /* Section information */
  char  axis;          /* Axis stored as a charcter (x,y or z) */
  int   iu,iv;         /* Indices of "x" and "y" axes */
  int   iaxis;         /* Index of "z" axis */
  int   nsec;          /* Section number (for single section) */
  float fsec;          /* Section in fractional units */
  int   ulimits[2];    /* Start-stop on u/x axis (grid units) */
  int   vlimits[2];    /* Start-stop on v/y axis (grid units) */
  float frc_ulimits[2];/* Start-stop on u/x axis (fractional units) */
  float frc_vlimits[2];/* Start-stop on v/y axis (grid units) */
  /* Slab information */
  int   slab_start;    /* First section for slab */
  int   slab_end;      /* Last section for slab */
  int   slab_step;     /* Intervals between sections in the slab */
  int   fslab_start;   /* First section in fractional units */
  int   fslab_end;     /* Last section in fractional units */
  int   fslab_step;    /* Intervals between sections in fractional units */
  /* Contour information */
  int   nlevels;       /* The number of contour levels stored */
  float *levels;       /* Array of contour levels */
  /* Display information */
  int   precision;     /* Maximum no of dp.s to display on grid labels */
  /* Axis labels */
  char  axislabel[3][LEN_AXISLABEL];
  /* Display flags for visible features */
  int   bbox;          /* 1 if bounding box is to be shown, 0 if not */
  int   grid;          /* 1 if grid is to be shown, 0 if not */
  float uspace,vspace; /* Grid spacing intervals (fractional) */
  int   labels;        /* 1 if labels are to be shown, 0 if not */
  int   titles;        /* 1 if titles are to be shown, 0 if not */
  int   axes;          /* 1 if axes are to be shown, 0 if not */
  /* View orientation */
  char  view_orient[4]; /* Orientation of the view (rhs or lhs) */
} CCP4SECTION;

/* Array of sections */
typedef struct {
  int   nsections;           /* Number of sections in array */
  CCP4SECTION **section_ptr; /* Array of pointers to section
				structures */
} CCP4SECTIONARRAY;

/* Structure for storage of canvas and contour information */
typedef struct {
  char  *canvas;       /* Path name of the Tcl canvas */
  char  *sectionid;    /* Handle for currently displayed section structure */
  /* Canvas attributes */
  float scale;         /* No of pixels per angstrom on canvas */
  int   xoffset,yoffset; /* No of pixels from canvas origin to section origin */
} CCP4CONTOURCANVAS;

/* Array of contour canvases */
typedef struct {
  int   ncanvas;                  /* Number of canvases in array */
  CCP4CONTOURCANVAS **canvas_ptr; /* Array of pointers to canvas
				     structures */
} CCP4CONTOURCANVASARRAY;

/*******************************************************************/

/* Function prototypes */

/*******************************************************************/

/* Tcl-specific functions
   ccp4mapwish_main.c
*/

int Tk_AppInit(Tcl_Interp *interp);

int Ccp4map_Init(Tcl_Interp *interp);

/* ccp4mapwish_tcl.c */

Tcl_Interp* get_tcl_interp_ptr(Tcl_Interp* new_interp_Ptr);

char* append_tcl_command(char *commandstr, char *appendstr);

/* Functions implementing Tcl commands for maps
   (ccp4mapwish_map.c)
*/

int ReadMap(ClientData clientData, Tcl_Interp *interp,
	    int objc, Tcl_Obj *CONST objv[]);

int MapInfo(ClientData clientData, Tcl_Interp *interp,
	    int objc, Tcl_Obj *CONST objv[]);

int DeleteMap(ClientData clientData, Tcl_Interp *interp,
	      int objc, Tcl_Obj *CONST objv[]);

int init_ccp4maparray(void);

CCP4MAP* fetch_map_ptr(char *mapid);

CCP4MAP* new_map_ptr(char *mapid);

int zap_map_ptr(char *mapid);

int deletemap(CCP4MAP *mapPtr);

/* Functions implementing Tcl commands for CCP4 contour canvases
   (ccp4mapwish_canvas.c) */

int init_ccp4canvasarray(void);

CCP4CONTOURCANVAS* fetch_canvas_ptr(char *canvas);

CCP4CONTOURCANVAS* new_canvas_ptr(char *canvas);

int zap_canvas_ptr(char *canvas);

int canvas_draw_rectangle(char *canvas, int npts, float *x, float *y,
			  char *tags, char *color, char units);

int canvas_draw_line(char *canvas, int npts, float *x, float *y, char *tags,
		     char *arrow, char units);

int canvas_draw_label(char *canvas, char *text, float x, float y,
		      char *anchor, char *tags, char units);

int canvas_scale_canvas(char *canvas, char *tag, float scale);

int canvas_shift_canvas(char *canvas);

int canvas_shift_element(char *canvas, char *tag, int shiftx, int shifty);

int canvas_delete(char *canvas, char *tag);

int canvas_exists(char *canvas);

float canvas_get_tk_scaling(char *canvas, char units);

int canvas_get_limits(char *canvas, int limits[4]);

/* Functions implementing Tcl commands for CCP4 sections
   (ccp4mapwish_section.c)
*/

int SectionCmd(ClientData clientData, Tcl_Interp *interp,
	       int objc, Tcl_Obj *CONST objv[]);

int section_init(char *sectionid, CCP4SECTION *section, ClientData clientData,
		 Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);

int section_config(CCP4SECTION *section, ClientData clientData, Tcl_Interp
		   *interp, int objc, Tcl_Obj *CONST objv[]);

int section_contours(CCP4SECTION *section, ClientData clientData, Tcl_Interp
		     *interp, int objc, Tcl_Obj *CONST objv[]);

int section_render(CCP4SECTION *section, ClientData clientData, Tcl_Interp
		   *interp, int objc, Tcl_Obj *CONST objv[]);

int section_delete(CCP4SECTION *section, ClientData clientData, Tcl_Interp
		   *interp, int objc, Tcl_Obj *CONST objv[]);

int section_coords(CCP4SECTION  *section, ClientData clientData, Tcl_Interp
		   *interp, int objc, Tcl_Obj *CONST objv[]);

int section_exists(CCP4SECTION *section, ClientData clientData, Tcl_Interp
		   *interp, int objc, Tcl_Obj *CONST objv[]);

int section_info(CCP4SECTION *section, ClientData clientData, Tcl_Interp
		 *interp, int objc, Tcl_Obj *CONST objv[]);

int init_ccp4sectionarray(void);

CCP4SECTION* fetch_section_ptr(char *sectionid);

CCP4SECTION* new_section_ptr(char *sectionid);

CCP4SECTION* init_section_strct(char *sectionid, char *mapid, char *axis,
				int iaxis, int iu, int iv, int first_sec,
				int last_sec, int slab_step);

int update_section_strct(CCP4SECTION *section, char *axis, int iu, int iv,
			 int iaxis, int first_sec, int last_sec, int slab_step,
			 int ulimits[], int vlimits[], float frc_ulimits[],
			 float frc_vlimits[], int nlevels, float *levels, int bbox,
			 int grid, float uspace, float vspace, int precision,
			 int labels, int titles, int axes,
			 char axislabel[][LEN_AXISLABEL], char view_orient[]);

int zap_section_ptr(char *sectionid);

int deletesection(CCP4SECTION *section);

int section_process_args(char *mode, CCP4SECTION *section, int first_arg,
			 ClientData clientData, Tcl_Interp
			 *interp, int objc, Tcl_Obj *CONST objv[]);

int section_process_contour_list_args(ClientData clientData, Tcl_Interp
				      *interp, int objc, Tcl_Obj *CONST objv[],
				      int first_arg, int *nvalues,
				      float **values_ptr);

int section_process_contour_range_args(ClientData clientData, Tcl_Interp
				       *interp, int objc, Tcl_Obj *CONST objv[],
				       int first_arg, int *nvalues,
				       float **values_ptr);

/* Contour Rendering Functions
   (ccp4mapwish_contour.c)
*/

int init_contour_canvas(char *canvas, CCP4SECTION *section, char units);

char* fetch_contour_canvas(void);

char fetch_contour_units(void);

int contour_section(CCP4SECTION *section);

int greyscale_section(CCP4SECTION *section);

int write_contour_line(float isovalue, int npts, float *x, float *y);

int write_contour_boundingbox(CCP4SECTION *section);

int write_contour_gridlines(CCP4SECTION *section);

int write_contour_gridlabel(CCP4SECTION *section, char *canvas, float *x, float *y);

int write_contour_axes(CCP4SECTION *section, char *canvas, float scale);

int write_contour_axis(CCP4SECTION *section, char *canvas, float *x, float*y,
		       int iaxis, float scale);

int write_contour_titles(CCP4SECTION *section, char *canvas, float scale);

/* Utility Functions
   (ccp4mapwish_utils.c)
*/

int get_scale_matrix(float cell[6], int iorth, float frac_to_orth[3][3],
		     float orth_to_frac[3][3]);

int invert_3matrix(float m[3][3], float invm[3][3]);

int transform_coordinates(CCP4SECTION *section, int npts, float *x, float *y);

int inv_transform_coordinates(CCP4SECTION *section, int xcanvas, int ycanvas,
			      int *xgrid, int *ygrid,
			      float *xfrac, float *yfrac,
			      float *xorth, float *yorth,
			      float scale);

int blankstr(char blank_str[], int nblanks);

int asc_cmp(const void *vp, const void *vq);

int get_axis_id(char *axis, int *iaxis, int *iu, int *iv);

char get_axis_name(int iaxis);

/* Internal C functions */

/* None at present */

/*******************************************************************/

/* End of prototypes */

/*******************************************************************/

/* End of mapwish.h */
