/*
     ccp4mapwish_init.c: initialise the CCP4map package
     Copyright (C) 2002  CCLRC, Peter Briggs

     This code is distributed under the terms and conditions of the
     CCP4 Program Suite Licence Agreement as a CCP4 Application.
     A copy of the CCP4 licence can be obtained by writing to the
     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
*/

/*******************************************************************/

/* ccp4mapwish_init.c */

/*******************************************************************/

/* CCP4map Package 2.1

   Package providing new Tcl commands for reading CCP4 map files,
   storing sections and generating contoured sections which can be
   displayed on a canvas widget.

   Nov 2000
   Initial version 1.0 (Peter Briggs pjx@dl.ac.uk).

   Oct 2001
   Version 2.0
   Substantial amount of old Fortran code (including countouring
   function) replaced by C code; complete restructuring.

   Feb 2002
   Version 2.1
   New Ccp4map_Init allows building of ccp4map as shared library.

*/

/*******************************************************************/

/* Include header file */

/*******************************************************************/

#include "ccp4mapwish.h"

/*******************************************************************/

/* Initialise the ccp4map package */

/*******************************************************************/

int Ccp4map_Init(Tcl_Interp *interp)
{
  /* Set the global pointer for the interpreter */
  get_tcl_interp_ptr(interp);

  /* Define the application-specific commands here */

  /* MAP commands */
  Tcl_CreateObjCommand(interp, "readmap", ReadMap,
		       (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateObjCommand(interp, "mapinfo", MapInfo,
		       (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateObjCommand(interp, "deletemap", DeleteMap,
		       (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  /* SECTION commands */
  Tcl_CreateObjCommand(interp, "section", SectionCmd,
		       (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  /* Initialise the array for storing maps in memory */
  init_ccp4maparray();

  /* Initialise the array for storing canvas information in memory */
  init_ccp4canvasarray();

  /* Initialise the array for storing section information in memory */
  init_ccp4sectionarray();

  /* Declare that we implement the ccp4map package
     so scripts that do "package require ccp4map" can load the
     library automatically
  */
  Tcl_PkgProvide(interp, CCP4MAPWISH_PACKAGE_NAME,CCP4MAPWISH_VERSION);

  /* Okay all done */
  return TCL_OK;
}

/*------------------------------------------------------------------------------*/
