C
C     DETWIN
C     Copyright (C) 1999 Andrew Leslie
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C*************************************************************
C
C
C**************************************************************
c
c  Detwin. Given the twinning fraction, detwin an input dataset. 
C  For merohedral twinning only. Note that detwinning only possible if
C  the twin fraction is less than 0.5 (not equal to or close to 0.5).
c
c  Read an MTZ file output by SCALA and write out a SCALA style MTZ file for
C  input to truncate.
C
c  Input:
c     HKLIN    reflection file
c
c  Output:
c     HKLOUT  
c     
cKEYWORDED INPUT
c
c
c   The various data control lines are identified by keywords Only the
c   first 4 letters of each keyword are necessary.
c   
c   OPERATOR, TWIN, DEBUG, SIG
c   
cDESCRIPTION OF KEYWORDS
c
c  TITLE <title> [OPTIONAL INPUT]
c  
c   (default TITLE='From Truncate') [OPTIONAL INPUT]
c   
c   Title to write to output reflection file
c
c  OPERATOR <string> [COMPULSORY INPUT]
c
c   Twinning operator, given as the indices of the reflection that is
c   related to the reflection "h,k,l" by the twin operator.
c
c   eg OPERATOR -h,h+k,-l
c   eg OPERATOR -h,h+k,-l
c
c  TWIN_FRACTION <twfac> [COMPULSORY INPUT]
c
c   The twin fraction (value between 0 and 0.5). This can be estimated
c   by running the program several times and looking at the resulting
c   cumulative intensity statistics (the N(z) test) when the output MTZ
c   file is input to TRUNCATE. The value which gives the best fit to
c   the theoretical distribution for the acentric terms should be used.
c
c   Once a refined model has been obtained, it is possible to use the
c   Fcalc values from this model to obtain a better estimate of the
c   twin fraction.
c
c   Alternatively, the program can be run several times and the value
c   which gives a correlation coefficient between the detwinned
c   intensities of closest to zero can be used. However, this may be
c   unreliable if there is any non-crystallographic symmetry present.
c
c
c  DEBUG <ndebug> [OPTIONAL INPUT]
c
c   Debug output will be printed for <ndebug> reflections
c
c   Author: Andrew G.W. Leslie, June 1998
c
      implicit none
c
C Things for PARSER ----
      integer maxtok
      parameter (maxtok=100)
      character key*4,cvalue(maxtok)*4,line*800,sline*800
      integer ibeg(maxtok),iend(maxtok),ityp(maxtok),
     .     idec(maxtok),ntok
      real    fvalue(maxtok)
      logical lend,hklswap,laniso
c
c Locals
Cpete the final table expects nrange = 10
      integer nrange
      parameter (nrange=10)
      integer maxind
C  Divide up for bigger problems
      parameter (maxind=8000000)
      integer ncols,msets
      parameter (ncols=500,msets=100)
      integer nlprgi
      parameter (nlprgi=25)
C   Can take 2% steps in twinning fraction, twfac 0 - 0.48
      integer nalpha
      parameter (nalpha=24)
      integer mtzin, mtzout, iprint, ifail, ihkl1(3), ivec(3), izone,
     $   nskip, nref, nout, nacen, ncent, nnzero, nzero(24), i,j,k,mdbg,
     $   nnrep,nrep(24),npair,ihkl2(3),twinmat(3,3),ih,ik,il,ialpha,
     $   ihklt(3),ksym,nlaue,nequ,nmiss,ndebug,ndbg,nbin,lxx,ii,loop,
     $   ksm2,ihtest(41,2),it,nhtest(2),IHTESTR(nrange,2),is(2),indd1,
     $   twinmats(3,3,4),ncol,nlprgo,ilok,iappnd,IImn,IFMn,IIano,IFano,
     $   numneg(2,nalpha),maxh,maxk,maxl,maxh21,maxh21k21,indd,idiff
C      real ff(-maxind:maxind, -maxind:maxind,-maxind:maxind),
C     +     sigff(-maxind:maxind, -maxind:maxind,-maxind:maxind),
       double precision amom,bmom(nalpha),amom2(nrange,nalpha),
     +         bmom2(nrange,nalpha),bmomm
      real ff(maxind),
     +     sigff(maxind),int1,int2,int1a,int2a,sd1,sd1a,var1a,var2a,
     +     iobs1,iobs2,iobs1a,iobs2a,denom,x,var1,var2,fact1,fact2,
     +     sdobs1,sdobs2,sdobs1a,sdobs2a,symmat(4,4,1), alpha,
     +      numer,corr1(nalpha),corr2(nalpha),s,range,sumai,tumai,fsig
      real suma(nrange,nalpha),si1a(nrange,nalpha),si2a(nrange,nalpha),
     +     si1sqa(nrange,nalpha),
     +     si2sqa(nrange,nalpha),si1i2a(nrange,nalpha),
     +     sai1a(nrange,nalpha),sai2a(nrange,nalpha),
     +     sai1sqa(nrange,nalpha),sai2sqa(nrange,nalpha),
     +     sai1i2a(nrange,nalpha),
     +     tuma(nrange,nalpha),ti1a(nrange,nalpha),
     +     ti1sqa(nrange,nalpha),
     +     tai1a(nrange,nalpha),tai1sqa(nrange,nalpha),
     +     corr1a(nrange,nalpha),corr2a(nrange,nalpha),dmin(nrange),
     +     sum(nalpha), si1(nalpha), si2(nalpha), si1sq(nalpha), 
     +     si2sq(nalpha), si1i2(nalpha), sai1(nalpha), sai2(nalpha), 
     +     sai1sq(nalpha),sai2sq(nalpha),sai1i2(nalpha),FHTEST(41,2),
     +     tum(nalpha)
      real cell(6),test,htest(2),FHTESTR(nrange,2),ai12aver,ai12min,
     +     ai12max,fs(2)
      real data(ncols),data2(19),ranges(2,ncols),TF0,TF1,TF2,TF3,TF4
      logical logmss(ncols)
      logical eof, lznot, keep, lcent
      real rminwl,rmaxwl,sminwl,smaxwl,rmin,rmax,smin,smax,twfac
      integer idot,icheck,ichk,itwin
      external idot
c
      integer nsym, nsymp, numspg, ic, n, nsymtwin, lentit, mtitot
      character  ltype*1, spgnam*10, pgname*10, titin*70, titout*70,
     +           datemt*8,SWAP*1
      real rsym(4,4,96)
      logical intitle,inreso
c
c---- MTZ arrays
c
      integer mtzlok(nlprgi)
      character*30 lsprgi(ncols),lsprgo(9),lsusri(ncols)
      character*1 ctprgi(ncols),ctprgo(9),ctusri(ncols)

C----Harvesting stuff
C
      INTEGER NDATASETS,ISETS(MSETS),CSETID(NCOLS),setid,set_index,iset
      REAL DCELL(6,MSETS),DWAVE(MSETS)
      CHARACTER*64 PNAME(MSETS),XNAME(MSETS),DNAME(MSETS),
     +             XNAME_OUT(NCOLS),DNAME_OUT(NCOLS)

C     ..
C     .. External Functions ..
      INTEGER LENSTR
      EXTERNAL LENSTR
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPFYP, MTZINI, CCPDAT, PARSER, GTPINT, GTPREA, CCPUPC,
     +         SYMFR2, CCPERR, LROPEN, LRTITL, LRSYMI, LRSYMM, ASUSET,
     +         LRCELL, LRRSOL, CENTRIC, LRASSN, LWOPEN, LWTITL, LRREFF,
     +         LRREFM, LRREWD, LRREFL, LWREFL, LWCLOS,CENTR,
     +   ccp4h_init,ccp4h_toc_beg,ccp4h_toc_ent,
     +   ccp4h_toc_end, ccp4h_pre_beg,ccp4h_pre_end,ccp4h_link_key,
     +   ccp4h_graph_beg,ccp4h_graph_end,ccp4h_link,ccp4h_header,
     +   ccp4h_rule  
      data twinmat/1,0,0, 0,1,0, 0,0,1/
      data twinmats/ 1, 0,0,  0, 1,0, 0,0, 1,
     +              -1, 0,0,  0,-1,0, 0,0, 1,
     +               0, 1,0,  1, 0,0, 0,0,-1,
     +               0,-1,0, -1, 0,0, 0,0,-1 /
C
C---- LSPRGI   array containing the program label strings
C     labels input
C
      data (lsprgi(j),j=1,19)/'H','K','L','IMEAN',
     +     'SIGIMEAN','IW(+)','SIGIW(+)','I(+)','SIGI(+)',
     +     'IW(-)','SIGIW(-)','I(-)','SIGI(-)', 'F','SIGF',
     +     'F(+)','SIGF(+)','F(-)','SIGF(-)'/
C
C---- CTPRGI   array containing the program label types
C
      DATA (CTPRGI(J),J=1,19)/'H','H','H','J','Q','K','M','K',
     +     'M','K','M','K','M','F','Q','G','L','G','L'/
c
C
C---- LSPRGO   array containing the program label strings
C     labels input
C
      data (lsprgo(j),j=1,3)/'H','K','L'/
C
C---- CTPRGI   array containing the program label types
C
      DATA (CTPRGO(J),J=1,3)/'H','H','H'/
c
C---- MTZLOK  array containing index from program labels
c     to file labels. On input these should be
c     set to -1 for compulsory labels and 1
c     for optional labels, and exit labels
c     which are not present in the file have
c     their lookup entry set to 0.
c     Default is to expect H K L IMEAN SIGIMEAN
c     Can be reset if a LABI line given
c
      data mtzlok/5*-1,20*0/
c
      call ccp4h_init()
      call ccp4h_header('DETWIN','detwin',2)
      call ccp4h_pre_beg()
      call ccpfyp
      CALL CCPRCS(6,'DETWIN','$Date: 2007/08/23 14:47:22 $')
      call mtzini
      call ccpdat(datemt)

      LANISO=.FALSE.
      HKLSWAP=.FALSE.
      inreso = .false.
      intitle = .false.
      twfac = 0
      smin = 0
      smax = 4
      rmin = 100000
      rmax = 0.25
      nhtest(1) = 0
      nhtest(2) = 0
      loop = 1
      nref = 0
      nout = 0
      nequ  = 0
      nmiss = 0
      ndbg = 0
      mdbg = 500
      ndebug = 10
      alpha = 0.0
      fsig = 3.0
      ai12aver = 0.0
Cpete ai12min = 0.0 doesn't catch the minimum if it's .gt. zero
      ai12min = 100000
      ai12max = 0.0
      htest(1) = 0.0
      htest(2) = 0.0
        do i = 1,41
          fhtest(i,1) = 0
          ihtest(i,1) = 0
          fhtest(i,2) = 0
          ihtest(i,2) = 0
        end do 
      mtzin  = 1
      mtzout = mtzin
      nsymtwin = 0
      titout = ' '
      write(6,98)
 98   format(/,1x,'Program DETWIN, beta test version 1/9/98',/,1X,
     =        '*** Remember to include keywords ANOMALOUS NO in the',
     +        ' input to TRUNCATE ***',//)
c
c--- Open mtz file : Twinning operator may be defined by space group.
c
      call ccp4h_summary_beg()
      call ccp4h_header('Input File Details','input',3)
      call ccp4h_summary_end()  
      iprint = 1
      ifail  = 0
      call lropen(mtzin,  'HKLIN', iprint, ifail)
      if (ifail .ne. 0) then
         go to 999
      endif
C
C---- CALL to return the title and it's length from the
C     header of an MTZ file.
C
c     ==========================
      call lrtitl(mtzin,titin,lentit)
c     ==========================
c
      if (lentit.le.1) titin = ' From Detwin on the '//datemt
c Set up symmetry
      call lrsymi(mtzin, nsymp, ltype, numspg, spgnam, pgname)
C
C   Check default twinning operators
         itwin = 0
C  PG4 PG6 H3 PG23 can only twin with operator k,h,-l
         write(6,*)' spgnam, pgname ',spgnam, pgname
         if( pgname(3:4) .eq. '4 '.or.pgname(3:4) .eq. '6 '
     +  .or. pgname(3:4) .eq. '23' .or. spgnam(1:2).eq.'H3' ) then 
           itwin = 3
           sline = 'k,h,-l'
           do i = 1,3
             do j = 1,3
               twinmat(i,j) = twinmats(i,j,itwin)
             end do
           end do
           nsymtwin = nsymtwin + 1
           write(6,*)' itwin,twinmat', itwin,twinmat
         end if
      call ccp4h_pre_end()
      call ccp4h_summary_beg()
      call ccp4h_toc_beg()
      call ccp4h_toc_ent('Command Input','#command')
      call ccp4h_toc_ent('Twinning Relationship','#twin')
      call ccp4h_toc_ent('Input File Details','#input')
      call ccp4h_toc_ent('Output File Details','#output')
      call ccp4h_summary_end()
      call ccp4h_toc_ent('Correlation Coefficient as a 
     +       Function of Various Twinning Fractions','#corr-coeff')
      call ccp4h_toc_ent('Plot Showing Correlation of 
     +       Measured and Detwined Intensities','#corr')
      call ccp4h_toc_ent('Second Moments and Correlation Coefficient
     + of Observed and Detwinned Z vs. Twinning Factor','#moments')
      call ccp4h_toc_end()
      call ccp4h_rule()
      call ccp4h_summary_beg()
      call ccp4h_header('Command Input','command',3)
      call ccp4h_link('DEBUG','detwin.html#debug') 
      call ccp4h_link('LABIN','detwin.html#labin') 
      call ccp4h_link('RESOLUTION','detwin.html#resolution') 
      call ccp4h_link('SYMMETRY','detwin.html#symmetry') 
      call ccp4h_link('TITLE','detwin.html#title') 
      call ccp4h_link('TWIN','detwin.html#twinfrac')
      call ccp4h_summary_end() 
      call ccp4h_pre_beg()
c
c---- read keyword input
c
c
 10   line=' '
      ntok=maxtok
      call parser(
     .  key,line,ibeg,iend,ityp,fvalue,cvalue,idec,ntok,lend,.true.)
      if (lend) go to 100
      if (ntok.eq.0) go to 10
      call ccpupc(key)
       write(6,*)' key:',key
c
c---- END
c
      if (key(1:3) .eq. 'END') go to 100
c
c---- OPERATOR "twin operator"
c
      if (key(1:3) .eq. 'SYM' .OR. key(1:3) .eq. 'OPE') then
        sline = line(ibeg(2):)
        nsymtwin =  1
        itwin = -1
        call symfr2 (sline, 1, nsymtwin, symmat)
        do 22 i = 1,3
          do 20 j = 1,3
            twinmat(i,j) = symmat(i,j,1)
 20       continue
 22     continue
c
C==== RESOLUTION
c
      else if (key .eq. 'RESO') then
       write(6,*)' after reso'
      CALL RDRESO(2,ITYP,FVALUE,NTOK,rmin,rmax,smin,smax)
         inreso = .true.
c
c---- TWIN_FRACTION <alpha>
c
      else if (key.eq.'TWIN') then
         call gtprea(2,twfac,ntok,ityp,fvalue)
c
c
c---- Sigma cut off (fsig)
c
      else if (key(1:3).eq.'SIG') then
         call gtprea(2,fsig,ntok,ityp,fvalue)
c
      ELSE IF (KEY.EQ.'LABI') THEN
C
C---- LABIN   (File_Number)  "a=b c=d ---"
C     ==========
C  No longer compulsory to set IMEAN and SIGIMEAN; could use F or I+
      mtzlok(4) = 0
      mtzlok(5) = 0
C
      CALL LKYIN(mtzin,LSPRGI,NLPRGI,NTOK,LINE,IBEG,IEND)
C
c---- DEBUG <ndebug> [ <mdbg> ]
c
      else if (key.eq.'DEBU') then
         call gtpint(2,ndebug,ntok,ityp,fvalue)
         if(ntok.gt.2) call gtpint(3,mdbg,ntok,ityp,fvalue)
        write(6,'(a,i4,a,i3,a)') 
     +  ' List every ',mdbg,'-th reflection till ',ndebug,' listed.'
c
c---- PLOT MURRay-rust
c
      else if (key.eq.'PLOT') then
         call ccpupc(line)
         IF (line(ibeg(2):ibeg(2)+1).EQ.'MU') then
          write(6,'(a)')
     +  ' *** Murray Rust plot requested - no DEBUG info given ***'
          ndebug = 0
         end IF 
c
c---- TITLE "Title"
c
      else if (key.eq.'TITL') then
         if (ntok.le.1) goto 10
         lxx = len(line)
         if (lxx.gt.80) lxx = 80
         titout = line(ibeg(2) :lxx)
         intitle = .true.
      endif
      go to 10

 100   continue
       if ( nsymtwin .eq.0) 
     + call ccperr(1,' DETWIN error: No twinning operator defined')
      call ccp4h_pre_end()
      call ccp4h_rule()
      call ccp4h_summary_beg()
      call ccp4h_header('Twinning Relationship','twin',3)
      call ccp4h_pre_beg()
c
       write(6,fmt=1004) sline(1:lenstr(sline)),
     +               ((twinmat(i,j),j=1,3),i=1,3)
 1004 format(/,1x,'Twinning relationship relates reflection with ',
     +       'indices h,k,l to : ',A,/,1X,'In matrix form',/,/,
     +        3(1x,3i5,/)/)
      call ccp4h_summary_end()
Cpete alpha is now twfac
      if (twfac.gt.0.5) then
        write(6,fmt=1006)
 1006   format(/,1X,'The twinning fraction must be in the range',
     +           ' 0-0.5.')
c       call ccperr(1,'DETWIN error: invalid twin fraction')
c       stop
      else if (twfac.eq.0.5) then
        write(6,fmt=1008)
 1008   format(/,1X,'The equations used to detwin the data become',
     +           ' indeterminate for a twin fraction',/,1X,
     +           'of exactly 0.5.')
        call ccperr(1,'DETWIN error: invalid twin fraction')
c       stop
      else if (abs(0.5-twfac).lt.0.05) then
        write(6,fmt=1010)
 1010   format(/,1X,'***** WARNING *****',/,1X,'The equations ',
     +          'used to detwin the data are indeterminate for',
     +          'a twin fraction',/,1X,'of 0.5, and will be ',
     +          'increasingly poorly determined for twin fractions',
     +          /,1X,'above 0.45.')
      end if  
      call ccp4h_summary _beg()  
      write(6,fmt=1012) twfac
 1012 format(1x,'Estimated twin fraction',F6.3,/,/)
      call ccp4h_summary_end()
      call ccp4h_rule()
c
      write(6,fmt=1013) fsig
 1013 format(1x,' Sigma cut off:',F6.3,/,/)
c
c Set up symmetry
      call lrsymm(mtzin, nsym, rsym)
      call asuset(spgnam,numspg,pgname,nsym,rsym,nsymp,nlaue,.false.)
c
c---- call to read cell parameters from the header common block
c     ====================
      call lrcell(mtzin,cell)
c     ====================
C     Notify available space and Read project and dat set names
      NDATASETS = MSETS
      CALL LRIDX(MTZIN,PNAME,XNAME,DNAME,ISETS,DCELL,DWAVE,NDATASETS)

C---- Get dataset ID for column and match to dataset header info.
      IF (NDATASETS.GT.0) THEN
        CALL LRCLID(MTZIN,CSETID,NCOL)
      ENDIF
C
 
C
C---- Call to read resolution limits
c     sminwl,smaxwl are 1/d**2
      call lrrsol(mtzin,sminwl,smaxwl)
      rminwl = 10000
      rmaxwl = 0.5
      if (sminwl.gt.0.0) rminwl=1./sqrt(sminwl)
      if (smaxwl.gt.0.0) rmaxwl=1./sqrt(smaxwl)
      if( inreso) then
        if (smin.gt.sminwl) then
            sminwl = smin
            rminwl = rmin
        end if 
        if (smax.lt.smaxwl) then
            smaxwl = smax
            rmaxwl = rmax
        end if 
      end if
      range = (smaxwl-sminwl)/nrange
      iprint = 0
      maxh = 1 + cell(1)/rmaxwl 
      maxk = 1 + cell(2)/rmaxwl
      maxl = 1 + cell(3)/rmaxwl
C We will have to pack indices into ff and sigff as 
C    (1 + ih+maxh + (2maxh+1)*k+maxk) + (2maxh+1)*(2maxh+1)*(il+maxl)
       maxh21 = 2*maxh+1
       maxh21k21 = maxh21*(2*maxk+1)
      write(6,'(A,2F11.3/,A,3i4)')
     +   ' Resolution limits in As',rminwl,rmaxwl,
     +   ' Maximum H K and L', maxh,maxk,maxl
      if(( (2*maxh+1) + (2*maxh+1)*(2*maxk+1) + 
     +       (2*maxh+1)*(2*maxk+1)*(2*maxl+1)) .gt. maxind) then
        write(6,fmt=1000) maxh,maxk,maxl,maxind
 1000   format(1x,'*** FATAL ERROR ***',/,1X,
     +  'Maximum reflection indices',3I5,' exceed maximum allowed',
     +  /,'Increase PARAMETER MAXIND',i10,
     +  ' and recompile or reduce resolution.')
        call ccperr(1,'Maximum permissible index exceeded.')
      end if
      call centric(nsym, rsym, iprint)
C
C---- CALL to apply the input column label assignments to the
C     MTZ file which is open for read and setup the Lookup pointer array
C     Also note that having an unassigned program label is a
C     fatal error, but this CALL uses LERROR to report
C     this as a warning, so that all the program label assignments
C     can be checked before stopping.
C
c     ==============================================
      call lrassn(mtzin,lsprgi,nlprgi,mtzlok,ctprgi)
c     ==============================================
C
       IIMn = 0
       IFMn = 0
       IIano = 0
       IFano = 0
        SETID = 0
        SET_INDEX = 0
      IF(MTZLOK(4) .GT.1 .and. MTZLOK(5) .GT.1) then
       IIMn = 1
       WRITE(6,*)'  Detwinning will deconvolute  < I > and < SIGI >'
C     Get setid of FP column and position index of the set
          SETID = CSETID(MTZLOK(4))
          DO ISET = 1,NDATASETS
            IF (ISETS(ISET).EQ.SETID) THEN
              SET_INDEX = ISET
            END IF
          ENDDO
      END IF
C
Cpete <F> stops the Java applets from running
      IF(MTZLOK(14) .GT.1 .and. MTZLOK(15) .GT.1) then
       WRITE(6,*)'  Detwinning will deconvolute  < F > and < SIGF >'
       IFMn = 1
          SETID = CSETID(MTZLOK(14))
          DO ISET = 1,NDATASETS
            IF (ISETS(ISET).EQ.SETID) THEN
              SET_INDEX = ISET
            END IF
          ENDDO
      END IF
C

      IF(MTZLOK(8) .GT.1  .and. MTZLOK(9) .GT.1 .and. 
     +   MTZLOK(12) .GT.1 .and. MTZLOK(13) .GT.1 )  then
       WRITE(6,*)'  Detwinning will deconvolute  I(+), I(-)'
       IIano = 1
          SETID = CSETID(MTZLOK(8))
          DO ISET = 1,NDATASETS
            IF (ISETS(ISET).EQ.SETID) THEN
              SET_INDEX = ISET
            END IF
          ENDDO
      END IF
C
      IF(MTZLOK(16) .GT.1 .and. MTZLOK(17) .GT.1 .and. 
     +   MTZLOK(18) .GT.1 .and. MTZLOK(19) .GT.1 )  then
       WRITE(6,*)'  Detwinning will deconvolute  F(+), F(-)'
       IFano = 1
          SETID = CSETID(MTZLOK(16))
          DO ISET = 1,NDATASETS
            IF (ISETS(ISET).EQ.SETID) THEN
              SET_INDEX = ISET
            END IF
          ENDDO
      END IF
C       Set output dataset and correct choice of cell
        IF (SET_INDEX.GT.0) THEN
          DO I=1,6
            CELL(I) = DCELL(I,SET_INDEX)
          ENDDO
        ENDIF
        ICHK = IFano + IIano + IFMn +IIMn
        If(ichk.eq.0) call ccperr(1,' Input labels improperly given')
C
C            ******************************
      CALL LRCLAB(MTZIN,lsusri,ctusri,NCOL)
C            ******************************
      call ccp4h_pre_end()
      call ccp4h_rule()
      call ccp4h_summary_beg()
      call ccp4h_header('Output File Details','output',3)
      call ccp4h_summary_end()
      call ccp4h_pre_beg()
c
      if(twfac.gt.0.001)then
         call lwopen(mtzout, 'HKLOUT')

            nlprgo=3
             Do ilok = 4,19
               if(mtzlok(ilok) .gt.0) then
                 nlprgo = nlprgo + 1
        sline = lsusri(mtzlok(ilok))
                  lsprgo(nlprgo) = 
     +  sline(1:lenstr(sline))//"_detw"
                  ctprgo(nlprgo) = ctusri(mtzlok(ilok))
               end if
             end Do 
        IAPPND = 0
        CALL LWASSN(MTZOUT,LSPRGO,NLPRGO,CTPRGO,IAPPND)
C
C---- If TITLE Key_Word used then put in New Title
C
Cpete if titout is empty, java applets are broken
Cpete so need to do this outside the if(twfac.gt.0.001) statement
C      if (.not.intitle) titout = titin
C
C          ============================
           call lwtitl(mtzout,titout,mtitot)
C          ============================
      end if
C
      if (.not.intitle) titout = titin
c
c
c---- This is too time-consuming, but still has to be done!
       do 6 k = 1,maxind
             ff(k) = 0.0
             sigff(k) = 0.0
  6    continue
c---- Read input MTZ file and store in 3D array in program order.
c
 120  call lrreff(mtzin, s, data, eof)
      if (eof) go to 200
C  mtzlok(4)              - IMEAN
C  mtzlok(14)             - F
C  mtzlok(8)   mtzlok(12) - I(+) I(-)
C  mtzlok(16)  mtzlok(18) - F(+) F(-)
      call lrrefm(mtzin,logmss)
       if(s.gt.smaxwl .or. s.lt.sminwl) go to 120
      if (IIMn .ne.0) then
         if (logmss(4).or.logmss(5)) goto 120
         if (data(5) .eq. 0.0)  goto 120
      end if 
      if (IFMn .ne.0) then
         if (logmss(14).or.logmss(15)) goto 120
         if (data(15) .eq. 0.0)  goto 120
      end if 
      if (IIano .ne.0) then
         if ( (logmss(8) .or.logmss(9) )
     $  .and.  (logmss(12).or.logmss(13)) )goto 120
         if (.not.logmss(9) )then
           if( data(9) .eq. 0.0) go to 120
         end if
         if (.not.logmss(13) )then
           if( data(13) .eq. 0.0) go to 120
         end if
      end if 
      if (IFano .ne.0) then
         if ( (logmss(16) .or.logmss(17)) 
     $  .and.  (logmss(18).or.logmss(19)) )goto 120
         if (.not.logmss(17) )then
           if( data(17) .eq. 0.0) go to 120
         end if
         if (.not.logmss(19) )then
           if( data(19) .eq. 0.0) go to 120
         end if
      end if 
C      if (mtzlok(4).ne.0 .and. (logmss(4).or.logmss(5))) goto 120
C      if (mtzlok(14).ne.0 .and. (logmss(14).or.logmss(15))) goto 120
c       write(6,'(3f5.0,6f10.2)') 
c     + data(1),data(2),data(3),(data(ii),ii=14,19)
      nref = nref+1
        ndbg = ndbg + 1
C  Make sure data in asymmetric unit.
      ihkl1(1) = nint(data(1))
      ihkl1(2) = nint(data(2))
      ihkl1(3) = nint(data(3))
      call asuput(ihkl1,ihklt,ksym)
C    KSYM       symmetry number for output
C                 odd  numbers are for I+ ( icheck = 1, HKLSWAP=.FALSE.)
C                 even numbers are for I-( icheck = 0, HKLSWAP=.TRUE.)
        icheck = mod(ksym,2)
        ih = ihklt(1)
        ik = ihklt(2)
        il = ihklt(3)
       HKLSWAP=.FALSE.
       if(icheck.eq.0) HKLSWAP=.TRUE.
      if ((abs(ih).gt.maxh).or.(abs(ik).gt.maxk).or.
     +    (abs(il).gt.maxl)) then
        call ccperr(1,' Indices outside limits')
      end if
      indd = 1+ ih+maxh + maxh21*(ik+maxk) + maxh21k21*(il+maxl)
C  indd1 for I(-)  - indices are -ih, -ik, -il
      indd1 = 1 -ih+maxh + maxh21*(-ik+maxk) + maxh21k21*(-il+maxl)
c       write(6,*) ih,ik,il,hklswap,indd,indd1
c      if (mtzlok(4).gt.0) then
c      if (data(5).eq.0.0) goto 120
cejd       maxh21 = 2*maxh+1
cejd       maxh21k21 = maxh21*(2*maxk+1)
c      ff(ih,ik,il) = data(4)
c      sigff(ih,ik,il) = data(5)
      if (IIMn .ne.0) then
       ff(indd) = data(4)
       sigff(indd) = data(5)
       ff(indd1) = data(4)
       sigff(indd1) = data(5)
      end if 
     
      if (IFMn .ne.0) then
c      if (mtzlok(14).gt.0) then
c      if (data(15).eq.0.0) goto 120
      ff(indd)    = data(14)*data(14)*0.0009
      sigff(indd) = 2*data(14)*data(15)*0.0009
      ff(indd1)    = data(14)*data(14)*0.0009
      sigff(indd1) = 2*data(14)*data(15)*0.0009
      end if 

c      data (lsprgi(j),j=1,19)/'H','K','L','IMEAN',
c     +     'SIGIMEAN','IW(+)','SIGIW(+)','I(+)','SIGI(+)',
c     +     'IW(-)','SIGIW(-)','I(-)','SIGI(-)', 'F','SIGF',
c     +     'F(+)','SIGF(+)','F(-)','SIGF(-)'/
C  Anomalous pairs
      IF(IIano .ne. 0) then
       IF  (HKLSWAP) THEN
        if(.not. (logmss(8).or.logmss(9)))then
         ff(indd1) = data(8)
         sigff(indd1) = data(9)
        end if
        if(.not. (logmss(12).or.logmss(13)))then
         ff(indd) = data(12)
         sigff(indd) = data(13)
        end if
       ELSE 
        if(.not. (logmss(8).or.logmss(9)))then
         ff(indd) = data(8)
         sigff(indd) = data(9)
        end if
        if(.not. (logmss(12).or.logmss(13)))then
         ff(indd1) = data(12)
         sigff(indd1) = data(13)
        end if
       END IF 
      END IF

      IF(IFano .ne. 0) then
       IF  (HKLSWAP) THEN
        if(.not. (logmss(16).or.logmss(17)))then
          ff(indd1)    = data(16)*data(16)*0.0009
          sigff(indd1) = 2*data(16)*data(17)*0.0009
        end if
        if(.not. (logmss(18).or.logmss(19)))then
          ff(indd)    = data(18)*data(18)*0.0009
          sigff(indd) = 2*data(18)*data(19)*0.0009
        end if
       ELSE 
        if(.not. (logmss(16).or.logmss(17)))then
          ff(indd)    = data(16)*data(16)*0.0009
          sigff(indd) = 2*data(16)*data(17)*0.0009
        end if
        if(.not. (logmss(18).or.logmss(19)))then
          ff(indd1)    = data(18)*data(18)*0.0009
          sigff(indd1) = 2*data(18)*data(19)*0.0009
        end if
       END IF 
      END IF
c      write(6,*)ih,ik,il,ff(indd),-ih,-ik,-il,ff(indd1)
c     do i=1,loop  - to pick up alternates for hkl -h,-k,-l

c----
C      call centr(ihkl1, ic)
c  IC = 1 centric, = 0 not
c  
      go to 120

c
c---- Rewind the  input MTZ file
 200  call lrrewd(mtzin)
      ichk = 0
      call ccp4h_summary_beg()
      write(6,fmt=1002) nref
 1002 format(1x,'Number of reflections read:',I6)
      call ccp4h_summary_end()
c
c---- Go through the file again, this time detwinning the data and
c     writing out a new MTZ file.
        ndbg = 0
      do 201 ialpha = 1,24
        numneg(1,ialpha) = 0
        numneg(2,ialpha) = 0
        sum(ialpha) = 0
        si1(ialpha) = 0
        si2(ialpha) = 0
        si1sq(ialpha) = 0
        si2sq(ialpha) = 0
        si1i2(ialpha) = 0
        sai1(ialpha) = 0
        sai2(ialpha) = 0
        sai1sq(ialpha) = 0
        sai2sq(ialpha) = 0
        sai1i2(ialpha) = 0
        tum(ialpha) = 0
          corr1(ialpha) = 0
          corr2(ialpha) = 0
          bmom(ialpha) = 0
        do 202 i=1,nrange
          if (range.gt.0.0) dmin(i) = sqrt(1.0/(sminwl+i*range))
          fhtestr(i,1) = 0
          ihtestr(i,1) = 0
          fhtestr(i,2) = 0
          ihtestr(i,2) = 0
          suma(i,ialpha) = 0
          tuma(i,ialpha) = 0
          amom2(i,ialpha) = 0
          bmom2(i,ialpha) = 0
          si1a(i,ialpha) = 0
          si2a(i,ialpha) = 0
          si1sqa(i,ialpha) = 0
          si2sqa(i,ialpha) = 0
          si1i2a(i,ialpha) = 0
          sai1a(i,ialpha) = 0
          sai2a(i,ialpha) = 0
          sai1sqa(i,ialpha) = 0
          sai2sqa(i,ialpha) = 0
          sai1i2a(i,ialpha) = 0
          ti1a(i,ialpha) = 0
          ti1sqa(i,ialpha) = 0
          tai1a(i,ialpha) = 0
          tai1sqa(i,ialpha) = 0
          corr1a(i,ialpha) = 0
          corr2a(i,ialpha) = 0
 202    continue
 201  continue
C  Set up Murray Rust Plot
           if(ndebug.eq.0)WRITE(6,'(4(/,a))')
     &     ' $TABLE :Murray Rust Plot Fo1/Fo2 :',
     &     ' $SCATTER : |Fo1| vs |Fo2| :A:1,2 :',
     &     ' $$ Fo1  Fo2 $$ $$'


 210  call lrrefl(mtzin, s, data, eof)
      if (eof) go to 300
       if(s.gt.smaxwl .or. s.lt.sminwl) go to 210
      call lrrefm(mtzin,logmss)
      if (IIMn .ne.0) then
         if (logmss(mtzlok(4)).or.logmss(mtzlok(5))) goto 210
         if (data(mtzlok(5))  .eq. 0.0 ) goto 210
      end if 
      if (IFMn .ne.0) then
         if (logmss(mtzlok(14)).or.logmss(mtzlok(15))) goto 210
         if (data(mtzlok(15)) .eq. 0.0 ) goto 210
      end if 
Cpete change the logic to skip reflex with mnfs
      if (IIano .ne.0) then
         if ((logmss(mtzlok(8)) .or.logmss(mtzlok(9))) 
     $  .or. (logmss(mtzlok(12)).or.logmss(mtzlok(13))) ) goto 210
         if (.not.logmss(mtzlok(9))) then
           if( data(mtzlok(9))   .eq. 0.0) goto 210
         end if 
         if (.not.logmss(mtzlok(13))) then
           if( data(mtzlok(13))   .eq. 0.0) goto 210
         end if 
      end if 
Cpete change the logic to skip reflex with mnfs
      if (IFano .ne.0) then
         if ((logmss(mtzlok(16)) .or.logmss(mtzlok(17))) 
     $  .or.(logmss(mtzlok(18)).or.logmss(mtzlok(19))) ) goto 210
         if (.not.logmss(mtzlok(17))) then
           if( data(mtzlok(17))   .eq. 0.0) goto 210
         end if 
         if (.not.logmss(mtzlok(19))) then
           if( data(mtzlok(19))   .eq. 0.0) goto 210
         end if 
      end if 
c       write(6,*) data(1),data(2),data(3),data(14),data(15)
c       write(6,*) (logmss(ii),ii=1,15)
c      if (mtzlok(4).ne.0 .and.
c     +    (logmss(mtzlok(4)).or.logmss(mtzlok(5)))) goto 210
c      if (mtzlok(14).ne.0 .and.
c     +    (logmss(mtzlok(14)).or.logmss(mtzlok(15)))) goto 210
      ihkl1(1) = nint(data(1))
      ihkl1(2) = nint(data(2))
      ihkl1(3) = nint(data(3))
C  Need IH IK IL to check for twin overlaps
      call asuput(ihkl1,ihklt,ksym)
C    KSYM  ta     symmetry number for output
C                 odd  numbers are for I+ ( icheck = 1, HKLSWAP=.FALSE.)
C                 even numbers are for I-( icheck = 0, HKLSWAP=.TRUE.)
C       icheck = mod(ksym,2)
C        HKLSWAP=.FALSE.
C       if(icheck.eq.0) HKLSWAP=.TRUE.
      ih = ihklt(1)
      ik = ihklt(2)
      il = ihklt(3)
      iobs1 = 0
      sdobs1 = 0
      iobs1a = 0
      sdobs1a = 0
C
      if (IIMn.ne.0) then
      iobs1 = data(mtzlok(4))
      sdobs1 = data(mtzlok(5))
      iobs1a = iobs1
      sdobs1a = sdobs1
      end if

      if (IFMn.ne.0) then
      iobs1   = data(mtzlok(14))*data(mtzlok(14)) *0.0009
c      sdobs1  = 2*data(mtzlok(14))*data(mtzlok(15)) *0.0009
      sdobs1  = 0.0018*(data(mtzlok(14))*data(mtzlok(15)) + 
     +                  data(mtzlok(15))*data(mtzlok(15)))
      iobs1a = iobs1
      sdobs1a = sdobs1
      end if

      IF(IIano .ne. 0) then
        if(.not. (logmss(mtzlok(8)).and.logmss(mtzlok(9))))then
         iobs1  = data(mtzlok(8))
         sdobs1 = data(mtzlok(9))
        end if
        if(.not. (logmss(mtzlok(12)).and.logmss(mtzlok(13))))then
         iobs1a  = data(mtzlok(12))
         sdobs1a = data(mtzlok(13))
        end if
      end if

      IF(IFano .ne. 0) then
        if(.not. (logmss(mtzlok(16)).and.logmss(mtzlok(17))))then
         iobs1  = data(mtzlok(16))*data(mtzlok(16)) *0.0009
c         sdobs1 = 2*data(mtzlok(16))*data(mtzlok(17)) *0.0009
         sdobs1  = 0.0018*(data(mtzlok(16))*data(mtzlok(17)) + 
     +                     data(mtzlok(17))*data(mtzlok(17)))
        end if
        if(.not. (logmss(mtzlok(18)).and.logmss(mtzlok(19))))then
         iobs1a  = data(mtzlok(18))*data(mtzlok(18)) *0.0009
c         sdobs1a = 2*data(mtzlok(18))*data(mtzlok(19)) *0.0009
         sdobs1a  = 0.0018*(data(mtzlok(18))*data(mtzlok(19)) + 
     +                     data(mtzlok(19))*data(mtzlok(19)))
        end if
      end if
C  No need to swap terms if necessary - ie HKLSWAP TRUE
C       IF( HKLSWAP) then
C        iobs2 = iobs1
C        iobs1 = iobs1a
C        iobs1a = iobs2
C        iobs2 = sdobs1
C        sdobs1 = sdobs1a
C        sdobs1a = iobs2
C        iobs2 = 0.0
C       end if
c
      nbin = (s-sminwl)/range
      if (nbin.lt.nrange) nbin = nbin + 1
      if (nbin.eq.0) nbin = 1
c
c----get the twin indices
c
      call imatvec(ihklt,twinmat,ihkl1)
c---- put twin indices into the asu
c
      call asuput(ihklt,ihkl2,ksm2)
C
c         icheck = 0
c         if( IFano.eq.0 .and. IIano.eq.0 .and.
c     +      ih.eq.ihkl2(1) .and. ik.eq.ihkl2(2) 
c     +.and. il.eq.ihkl2(3)) icheck = 1
c         if((IFano.ne.0 .or. IIano.ne.0) .and.
c     +      ih.eq.ihkl2(1) .and. ik.eq.ihkl2(2) 
c     +.and. il.eq.ihkl2(3) .and. ksym.eq.ksm2) icheck = 1
c         if( icheck.eq.1) then
c            if(mtzlok(4) .gt. 0) then 
c               data2(4) = data(mtzlok(4))
c               data2(5) = data(mtzlok(5))
c            end if
c            if(mtzlok(14) .gt. 0) then 
c               data2(4) = data(mtzlok(14))
c               data2(5) = data(mtzlok(15))
c            end if
c            if(mtzlok(8) .gt. 0) then 
c               data2(4) = data(mtzlok(8))
c               data2(5) = data(mtzlok(9))
c               data2(6) = data(mtzlok(12))
c               data2(7) = data(mtzlok(13))
c            end if
c            if(mtzlok(16) .gt. 0) then 
c               data2(4) = data(mtzlok(16))
c               data2(5) = data(mtzlok(17))
c               data2(6) = data(mtzlok(18))
c               data2(7) = data(mtzlok(19))
c            end if
c           go to 216
c         end if
cc
      if ((abs(ihkl2(1)).gt.maxh).or.(abs(ihkl2(2)).gt.maxk).or.
     +    (abs(ihkl2(3)).gt.maxl)) then
        write(6,fmt=1016) ihkl2(1),ihkl2(2),ihkl2(3),maxh,maxk,maxl
 1016   format(1x,'*** FATAL ERROR ***',/,1X,
     +  'One of Twin related reflection indices',3I5,' exceed maximum',
     +       ' allowed values (',3I5,')',/,1X,
     +  'Increase PARAMETER MAXIND and recompile.')
        call ccperr(1,'Maximum permissible index exceeded.')
      end if
c
C    KSYM       symmetry number for output
C                 odd  numbers are for I+ ( icheck = 1, HKLSWAP=.FALSE.)
C                 even numbers are for I-( icheck = 0, HKLSWAP=.TRUE.)
C  Shouldnt this check if ksym and ksm2 are DIFFERENT? then change pair?
       icheck = mod(ksm2,2)
        HKLSWAP=.FALSE.
       if(icheck.eq.0) HKLSWAP=.TRUE.
c
c
c---- twin intensity
c
       indd = 1+ ihkl2(1)+maxh + maxh21*(ihkl2(2)+maxk)
     +                       + maxh21k21*(ihkl2(3)+maxl)
       indd1 = 1- ihkl2(1)+maxh + maxh21*(-ihkl2(2)+maxk)
     +                       + maxh21k21*(-ihkl2(3)+maxl)
c      iobs2 =     ff(ihkl2(1),ihkl2(2),ihkl2(3))
c      sdobs2 = sigff(ihkl2(1),ihkl2(2),ihkl2(3))
C
C  Need to  worry about HKLSWAP here - hand of ihkl2 might by modified by asuput
       iobs2 =     ff(indd )
       sdobs2 = sigff(indd )
       iobs2a =     ff(indd1)
       sdobs2a = sigff(indd1)
          SWAP='F'
       IF  (HKLSWAP) THEN
          iobs2 =     ff(indd1)
          sdobs2 = sigff(indd1)
          iobs2a =     ff(indd)
          sdobs2a = sigff(indd)
          SWAP='T'
       END IF 

c
c---- check that this has been measured
c
      if (sdobs2.eq.0.0) then
       if(ndebug.gt.0) then
       write(6,'(/,a)')
     +  ' Missing twin pair'
        write(6,'(2a,/,x,3i4,i3,5x,3i4,6x,3i4,i3,12x,a)')
     + ' Input hkl(sym_op),  Detwinned hkl, Detwinned hkl_asu(sym_op) ',
     + 'handchange?',
     + ihkl1,ksym,ihklt,ihkl2,ksm2, swap
        write(6,1035) ihkl1,iobs1,sdobs1,iobs1a,sdobs1a,
     +                ihklt,iobs2,sdobs2,iobs2a,sdobs2a
       end if
        nmiss = nmiss + 1
        goto 210
      end if
      if ((IIano.ne.0 .or. IFano .ne.0) .and. sdobs2a.eq.0.0) then
C   List every mdbg-th reflection till ndebug listed
       if(ndebug.gt.0) then
        write(6,'(/,a)')
     +  ' Missing twin pair'
        write(6,'(2a,/,x,3i4,i3,5x,3i4,6x,3i4,i3,12x,a)')
     + ' Input hkl(sym_op),  Detwinned hkl, Detwinned hkl_asu(sym_op) ',
     + 'handchange?',
     + ihkl1,ksym,ihklt,ihkl2,ksm2, swap
Cpete can't write int1, int1a, in2, in2a yet since reflex hasn't been 
C     detwinned yet
C       write(6,1040) ihkl1,iobs1,int1,iobs1a,int1a,
C    +                ihklt,iobs2,int2,iobs2a,int2a
        write(6,1035) ihkl1,iobs1,sdobs1,iobs1a,sdobs1a,
     +                ihklt,iobs2,sdobs2,iobs2a,sdobs2a
 1035   format(' Reflection',3I5,' obs.intensity+',F8.0,
     +         ' s.d. intensity+ ',F8.0,/,26x,
     +         ' obs.intensity-',F8.0,
     +         ' s.d. intensity- ',F8.0,/,
     +         '  twin mate',3I5,' obs.intensity+',F8.0,
     +         ' s.d. intensity+ ',F8.0,/,26x,
     +         ' obs.intensity-',F8.0,
     +         ' s.d. intensity- ',F8.0)
       end if
        nmiss = nmiss + 1
        goto 210
      end if 
c
c---- solve for the two twin components
c
c     iobs(h1) = (1-x)*i(h1)  +     x*i(h2)
c     iobs(h2) =     x*i(h1)  + (1-x)*i(h2)
c
c     where x is the twin fraction
c
c
c     thus i(h1) = ((1-x)*iobs(h1) -x*iobs(h2)) / (1-2x)
c     thus i(h2) = ((1-x)*iobs(h2) -x*iobs(h1)) / (1-2x)
c     also
c     var(h1) = ((1-x)/(1-2x))**2 * sdobs(h1)**2 + (x/(1-2x))**2 * sdobs(h2)**2
c     var(h2) = ((1-x)/(1-2x))**2 * sdobs(h2)**2 + (x/(1-2x))**2 * sdobs(h1)**2
c
c     Note that this is singular if x=0.5
c
 215   continue
C   If twfac not set, no need to generate output
      IF(twfac.le.0.0) go to 216
      denom = 1-2.0*twfac
        fact1 = (1.0-twfac)/(1.0-2*twfac)
        fact2 = (twfac)/(1.0-2*twfac)
        fact1 = fact1**2
        fact2 = fact2**2
C      write(6,*) 'iobs1,iobs2,denom',iobs1,iobs2,denom
      int1 = ((1.0-twfac)*iobs1  -twfac*iobs2)  / denom
      int2 = ((1.0-twfac)*iobs2  -twfac*iobs1)  / denom
      int1a= ((1.0-twfac)*iobs1a -twfac*iobs2a) / denom
      int2a= ((1.0-twfac)*iobs2a -twfac*iobs1a) / denom
C   List every mdbg-th reflection till ndebug listed
      if (ndebug.gt.0 ) ichk = mod(nout,mdbg)
      if (ndebug.gt.0 .and. ichk.eq.0) then
       write(6,'(/,a)')
     +  ' Debug output'
        write(6,'(2a,/,x,3i4,i3,5x,3i4,6x,3i4,i3,12x,a)')
     + ' Input hkl(sym op),  Detwinned hkl, Detwinned hkl_asu(sym op) ',
     + 'handchange?',
     + ihkl1,ksym,ihklt,ihkl2,ksm2, swap
        write(6,1040) ihkl1,iobs1,int1,iobs1a,int1a,
     +                ihklt,iobs2,int2,iobs2a,int2a
 1040   format(' Reflection',3I5,' obs.intensity+',F8.0,
     +         ' detwinned intensity+ ',F8.0,/,26x,
     +         ' obs.intensity-',F8.0,
     +         ' detwinned intensity- ',F8.0,/,
     +         '  twin mate',3I5,' obs.intensity+',F8.0,
     +         ' detwinned intensity+ ',F8.0,/,26x,
     +         ' obs.intensity-',F8.0,
     +         ' detwinned intensity- ',F8.0)
        ndbg = ndbg + 1
        if(ndbg.gt.ndebug) ndbg = 0
      end if
C      write(6,*) 'nout,sdobs1,sdobs2',nout,sdobs1,sdobs2
      var1 = fact1*sdobs1**2 + fact2*sdobs2**2
      var1a = fact1*sdobs1a**2 + fact2*sdobs2a**2
      sd1 = sqrt(var1)
      sd1a = sqrt(var1a)
      if(IImn .gt. 0) then 
c         data(mtzlok(4)) = int1
c         data(mtzlok(5)) = sqrt(var1)
         data2(4) = int1
         data2(5) = sd1
      end if
      if(IFmn .gt. 0) then 
c         data(mtzlok(14)) = 0
c         data(mtzlok(15)) = 0
         data2(4) = 0
         data2(5) = 0
         if( int1.gt.0) then 
           data2(4) = sqrt(int1) 
c           data2(5) = sd1/(2*data2(4))
c  better
           data2(5) = sqrt( sd1 +int1) - data2(4)
C  Compensate for reducing scale by 0.03 above
           data2(4) = 33.333*data2(4)
           data2(5) = 33.333*data2(5)
c           data(mtzlok(15)) = 33.333*data(mtzlok(15))
         end if
      end if
      if(IIano .gt. 0) then 
         data2(4) = int1
c         data2(5) = sqrt(var1)
         data2(5) = sd1
         data2(6) = int1a
c         data2(7) = sqrt(var1a)
         data2(7) = sd1a
c         data(mtzlok(14)) = 0
c         data(mtzlok(15)) = 0
      end if
      if(IFano .gt. 0) then 
         data2(4) = 0
         data2(5) = 0
         if( int1.gt.0) then 
           data2(4) = sqrt(int1) 
c           data2(5) = sqrt(var1)/(2*data2(4))
c  better
           data2(5) = sqrt( sd1 +int1) - data2(4)
C  Compensate for reducing scale by 0.03 above
           data2(4) = 33.333*data2(4)
           data2(5) = 33.333*data2(5)
c           data(mtzlok(15)) = 33.333*data(mtzlok(15))
         end if
         if( int1a.gt.0) then 
           data2(6) = sqrt(int1a) 
c           data2(7) = sqrt(var1a)/(2*data2(6))
c  better
           data2(7) = sqrt( sd1a +int1a) - data2(6)
C  Compensate for reducing scale by 0.03 above
           data2(6) = 33.333*data2(6)
           data2(7) = 33.333*data2(7)
c           data(mtzlok(15)) = 33.333*data(mtzlok(15))
         end if
      end if
 216  continue
      if(twfac.gt.0.001) then
           data2(1) = data(1)
           data2(2) = data(2)
           data2(3) = data(3)
c            write(6,'(3f4.0,4f10.1)')(data2(i),i=1,7)
           call lwrefl(mtzout, data2)
           nout = nout + 1
      end if
c
 217  continue
C  Carry out all twinning checks
C----- Do not include those reflections where the twinned indices coincide for analysis.
         if(ih.eq.ihkl2(1) .and. ik.eq.ihkl2(2) 
     +.and. il.eq.ihkl2(3)) then
           nequ = nequ +1
           go to 210
         end if
CC  Output for Murray Rust Plot
         if(ndebug.eq.0.and.iobs1.gt.0.0 .and. iobs2 .gt.0.0) 
     &     write(6,'(2F8.2)') sqrt(iobs1),sqrt(iobs2)
C   Yeates test AAV
Cpete should use acentrics only
       call centr(ihkl1, ic)
c  IC = 1 centric, = 0 not
       if (ic.eq.0) then
       test = abs(iobs1-iobs2)/(iobs1+iobs2)
Cpete the value of test could be very large and negative
Cpete if the denominator is negative and close to zero
Cpete so the graph is better than the number 0.5 - HTEST(1)
C
C      it = test*40 + 1.5
              HTEST(1)  = HTEST(1)  + TEST
              NHTEST(1) = NHTEST(1) + 1
              IT = TEST*40.0 + 1.5
              IF(IT.GT.41) IT = 41
              IF(IT.LT. 1) IT =  1
              IHTEST(IT,1)    = IHTEST(IT,1)    + 1
              IHTESTR(NBIN,1) = IHTESTR(NBIN,1) + 1
              FHTESTR(NBIN,1) = FHTESTR(NBIN,1) + TEST
              AI12AVER = AI12AVER + (iobs1+iobs2)
              IF((iobs1+iobs2).LE.AI12MIN) AI12MIN = (iobs1+iobs2)
              IF((iobs1+iobs2).GE.AI12MAX) AI12MAX = (iobs1+iobs2)
Cpete use of abs here not needed
C      if( abs(iobs1).gt.fsig*sdobs1 .and. abs(iobs2).gt.fsig*sdobs2) 
C    +                                                          then
       if( iobs1.gt.fsig*sdobs1 .and. iobs2.gt.fsig*sdobs2) then
              HTEST(2)  = HTEST(2)  + TEST
              NHTEST(2) = NHTEST(2) + 1
              IHTEST(IT,2)    = IHTEST(IT,2)    + 1
              IHTESTR(NBIN,2) = IHTESTR(NBIN,2) + 1
              FHTESTR(NBIN,2) = FHTESTR(NBIN,2) + TEST
       end if
       if(IIano.ne.0 .or. IFano.ne.0) then
Cpete next 6 lines new, include I(-) in calculation of H, 
Cpete rather than including I(+) twice 
Cpete note mean summed int was wrong becuz NHTEST(1) wasn't incremented
Cpete while AI12AVER was updated
              test = abs(iobs1a-iobs2a)/(iobs1a+iobs2a)
              HTEST(1)  = HTEST(1)  + TEST
              NHTEST(1) = NHTEST(1) + 1
              IT = TEST*40.0 + 1.5
              IF(IT.GT.41) IT = 41
              IF(IT.LT. 1) IT =  1
              IHTEST(IT,1)    = IHTEST(IT,1)    + 1
              IHTESTR(NBIN,1) = IHTESTR(NBIN,1) + 1
              FHTESTR(NBIN,1) = FHTESTR(NBIN,1) + TEST
              AI12AVER = AI12AVER + (iobs1a+iobs2a)
              IF((iobs1a+iobs2a).LE.AI12MIN) AI12MIN = (iobs1a+iobs2a)
              IF((iobs1a+iobs2a).GE.AI12MAX) AI12MAX = (iobs1a+iobs2a)
Cpete toss abs
C      if( abs(iobs1a).gt.fsig*sdobs1a .and. 
C    +     abs(iobs2a).gt.fsig*sdobs2a) then
       if( iobs1a.gt.fsig*sdobs1a .and.
     +     iobs2a.gt.fsig*sdobs2a) then
              HTEST(2)  = HTEST(2)  + TEST
              NHTEST(2) = NHTEST(2) + 1
              IHTEST(IT,2)    = IHTEST(IT,2)    + 1
              IHTESTR(NBIN,2) = IHTESTR(NBIN,2) + 1
              FHTESTR(NBIN,2) = FHTESTR(NBIN,2) + TEST
       end if
       end if
       end if

c---- Correlation coefficient between twinned reflections
c
Cpete Also second moments, with original code having
Cpete acentrics and centrics mixed together
C
      do 211 ialpha = 1,24
        alpha = float(ialpha)/50.0
        fact1 = (1.0-alpha)**2/(1.0-2*alpha)
        fact2 = (alpha)**2/(1.0-2*alpha)
C
      denom = 1-2.0*alpha
C      write(6,*) 'iobs1,iobs2,denom',iobs1,iobs2,denom
      int1 = ((1.0-alpha)*iobs1 -alpha*iobs2) / denom
      int2 = ((1.0-alpha)*iobs2 -alpha*iobs1) / denom
C
      if(int1.lt.0.0 .or. int2.lt.0) then
       numneg(1,ialpha) = numneg(1,ialpha) + 1
Cpete abs(iobs) made no sense
c      if( abs(iobs1).gt.fsig*sdobs1 .and. abs(iobs2).gt.fsig*sdobs2)
c    +  numneg(2,ialpha) = numneg(2,ialpha) + 1
       if( iobs1.gt.fsig*sdobs1 .and. iobs2.gt.fsig*sdobs2)
     +  numneg(2,ialpha) = numneg(2,ialpha) + 1
       go to 211
      end if
c
      sum(ialpha) = sum(ialpha) + 1
      suma(nbin,ialpha) = suma(nbin,ialpha) + 1
      si1(ialpha) = si1(ialpha) + iobs1
      si2(ialpha) = si2(ialpha) + iobs2
      si1sq(ialpha) = si1sq(ialpha) + iobs1*iobs1
      si2sq(ialpha) = si2sq(ialpha) + iobs2*iobs2
      si1i2(ialpha) = si1i2(ialpha) + iobs1*iobs2
c
c---- Correlation coefficient  as a function of resln
c
      si1a(nbin,ialpha) = si1a(nbin,ialpha) + iobs1
      si2a(nbin,ialpha) = si2a(nbin,ialpha) + iobs2
      si1sqa(nbin,ialpha) = si1sqa(nbin,ialpha) + iobs1*iobs1
      si2sqa(nbin,ialpha) = si2sqa(nbin,ialpha) + iobs2*iobs2
      si1i2a(nbin,ialpha) = si1i2a(nbin,ialpha) + iobs1*iobs2
c---- Correlation coefficient between twinned reflections after detwinning
c
      sai1(ialpha) = sai1(ialpha) + int1
      sai2(ialpha) = sai2(ialpha) + int2
      sai1sq(ialpha) = sai1sq(ialpha) + int1*int1
      sai2sq(ialpha) = sai2sq(ialpha) + int2*int2
      sai1i2(ialpha) = sai1i2(ialpha) + int1*int2
c
c---- Correlation coefficient  as a function of resln
c
      sai1a(nbin,ialpha) = sai1a(nbin,ialpha) + int1
      sai2a(nbin,ialpha) = sai2a(nbin,ialpha) + int2
      sai1sqa(nbin,ialpha) = sai1sqa(nbin,ialpha) + int1*int1
      sai2sqa(nbin,ialpha) = sai2sqa(nbin,ialpha) + int2*int2
      sai1i2a(nbin,ialpha) = sai1i2a(nbin,ialpha) + int1*int2
c
cpete Again with acentrics only
c     the C.C. and 2nd moments never used iobs1a or int2a
c     this is all done more comprehensively in truncate
c
      if (ic.eq.0) then
      tum(ialpha) = tum(ialpha) + 1
      tuma(nbin,ialpha) = tuma(nbin,ialpha) + 1

      ti1a(nbin,ialpha) = ti1a(nbin,ialpha) + iobs1
      ti1sqa(nbin,ialpha) = ti1sqa(nbin,ialpha) + iobs1*iobs1

      tai1a(nbin,ialpha) = tai1a(nbin,ialpha) + int1
      tai1sqa(nbin,ialpha) = tai1sqa(nbin,ialpha) + int1*int1

      end if
211   continue

      goto 210


 300  continue
      if(twfac.gt.0.001)call lwclos(mtzout, iprint)
c  Write final $$ for scatter
      if(ndebug.eq.0) write(6,*)'$$'
      call ccp4h_summary_beg()
      if(twfac.gt.0.001) write(6,1020) twfac,nout
      write(6,1028) nequ,nmiss
 1020 format(/,
     +    'Output file written for  twinning fraction',f6.3,/, 
     +    'Number of reflections written to output file',I7)
 1028 format(/,
     +    'Number omitted from analysis because the twin mate same as ',
     +    'original',I7,/,
     +    'Number omitted from analysis because the twin mate was not ',
     +    'observed',I7,/,
     +    'Twinning tests for all fractions: see log graphs',/)
      call ccp4h_summary_end()
      call ccp4h_pre_end()
      call ccp4h_rule()
      call ccp4h_header('Correlation Coefficient as a Function
     +  of Resolution for Various Twinning Fractions ','corr-coeff',3)
      call ccp4h_pre_beg()
c---- calculate correlation coefficients
c
Cpete how did the next 5 lines get here
c             HTEST(2)  = HTEST(2)  + TEST
c             NHTEST(2) = NHTEST(2) + 1
c             IHTEST(IT,2)    = IHTEST(IT,2)    + 1
c             IHTESTR(NBIN,2) = IHTESTR(NBIN,2) + 1
c             FHTESTR(NBIN,2) = FHTESTR(NBIN,2) + TEST
      IF(nhtest(1).gt.0 .and. NHTEST(2).GT.0) THEN
        HTEST(1) = HTEST(1)/NHTEST(1)
        HTEST(2) = HTEST(2)/NHTEST(2)

      ALPHA = 0.5 - HTEST(1)
      WRITE(6,'(2(/,a,f7.3,/,a,f7.3,/,a,i7))') 
     +  ' H value for all reflections:       ', HTEST(1),
     +  ' Suggests Twinning factor (0.5-H):  ',0.5 - HTEST(1),
     +  ' Number of pairs:                   ',NHTEST(1),
     +  ' H value for reflections > Nsig*obs:', HTEST(2),
     +  ' Suggests Twinning factor (0.5-H):  ',0.5 - HTEST(2),
     +  ' Number of pairs:                   ',NHTEST(2)
        AI12AVER = AI12AVER/NHTEST(1)

        WRITE(6,'(/,a,F10.2,F8.2,F15.2)')
     +  ' Mean, Minimum and Maximum summed intensity:',
     +  ai12aver,ai12min,ai12max

        IS(1) = 0
        IS(2) = 0
        DO I=1,41
          IS(1) = IS(1) + IHTEST(I,1)
          IS(2) = IS(2) + IHTEST(I,2)
        ENDDO
        FS(1) = 0.0
        FS(2) = 0.0
        call ccp4h_graph_beg(0,0)
        write(6,2021) titout(1:20)
 2021 FORMAT(//,
     $ ' $TABLE: Twin Test v <H> = <|Itw1 -Itw2|/(Itw1 +Itw2)>',
     $    A,' :',/,
     $ ' $GRAPHS',/,
     $ ' : Observed and Expected  reflection distributions  ',
     $ ':A:1,9,10,4,5,6,7,8:',/,
     $ ' $$', ' <H> countAll',
     $ ' count_NSig <Hcount-TF=0.0> <Hcount-TF=0.1> <Hcount-TF=0.2> ',
     $ '<Hcount-TF=0.3> <Hcount-TF=0.4> <Hcount=obs> <Hcount=obs_Nsig>',
     $ /,' $$',
     $/,' $$',/)
Cpete potential division by zero
        DO I=1,41
          FS(1) = FS(1) + FLOAT(IHTEST(I,1))/IS(1)
          FHTEST(I,1) = FS(1)
          FS(2) = FS(2) + FLOAT(IHTEST(I,2))/IS(2)
          FHTEST(I,2) = FS(2)
          TF1 = 1.0 
          TF2 = 1.0 
          TF3 = 1.0 
          TF4 = 1.0 
          IF(I.LT.9)  TF4 = (I-1)/8.0 
          IF(I.LT.17) TF3 = (I-1)/16.0 
          IF(I.LT.25) TF2 = (I-1)/24.0 
          IF(I.LT.33) TF1 = (I-1)/32.0 
          IF(I.LT.41) TF0 = (I-1)/40.0 
        WRITE(6,'(X,F5.3,2I8,7F8.3)')(I-1)/80.0,IHTEST(I,1),IHTEST(I,2),
     $  TF0,TF1,TF2,TF3,TF4, FHTEST(I,1),FHTEST(I,2)
        ENDDO
      write(6,*)' $$'
      call ccp4h_graph_end()

        call ccp4h_graph_beg(0,0)
        write(6,2022) titout(1:20)
 2022 FORMAT(//,
     $ ' $TABLE: Twin factor estimate ( TF = 1/2 - <H>) v resn - ',A,
     $ ' :',/,
     $ ' $GRAPHS',/,
     $ ' : TF estimate (1/2 - <H>) from all data and with NSig cuttoff',
     $ ':N:1,4,5:',/,
     $ ' $$',
     $ '  1/resol^2 countall Count_NSig Twin_fac_all Twin_fac_NSig',/,
     $ ' $$',
     $/,' $$',/)
       do i=1,nrange
          s = 1.0/ (dmin(i)*dmin(i))
          IF(IHTESTR(I,1).NE.0) FHTESTR(I,1)=FHTESTR(I,1)/IHTESTR(I,1)
          IF(IHTESTR(I,2).NE.0) FHTESTR(I,2)=FHTESTR(I,2)/IHTESTR(I,2)
            TF1 = 0.5 - FHTESTR(I,1)
            TF2 = 0.5 - FHTESTR(I,2)
            if(IHTESTR(I,1).NE.0) write(6,'(F7.3,2I6,2F6.2)') 
     +       s,IHTESTR(I,1),IHTESTR(I,2),TF1,TF2
       end do 
       write(6,*)' $$'
       call ccp4h_graph_end()
C        WRITE(6,'(4(/,10F6.3),F6.3)') (I,IHTEST(I,1),FHTEST(I,1),I= 1,41)
      END IF

       do 311 ialpha=1,nalpha
        alpha = float(ialpha)/50.0
       write(6,'(A,f6.3)') '  Twinning fraction: ', alpha
       if (sum(ialpha).le.0.0) go to 311
        numer = si1i2(ialpha) - si1(ialpha)*si2(ialpha)/sum(ialpha)
        denom = sqrt(si1sq(ialpha) -si1(ialpha)**2/sum(ialpha))
     +         *sqrt(si2sq(ialpha)-si2(ialpha)**2/sum(ialpha))
        if (denom.gt.0.0) corr1(ialpha) = numer/denom
c        numer = si1sq(ialpha)/sum(ialpha)
c        denom = si1(ialpha)/sum(ialpha)
c        denom = denom*denom
c        bmom(ialpha) = numer/denom
c        bmom(ialpha) = sum(ialpha)*si1sq(ialpha)/(si1(ialpha)*si1(ialpha))
        numer = sai1i2(ialpha) - sai1(ialpha)*sai2(ialpha)/sum(ialpha)
        denom = sqrt(sai1sq(ialpha) -sai1(ialpha)**2/sum(ialpha))
     +         *sqrt(sai2sq(ialpha)-sai2(ialpha)**2/sum(ialpha))
        if (denom.gt.0.0) corr2(ialpha) = numer/denom
c

c       write(6,'(i3,a,4f15.3,2(/,a,/,5F15.1))')
c     + ialpha,' bmom, corr1,corr2 sum',
c     + bmom(ialpha), corr1(ialpha),corr2(ialpha),sum(ialpha),
c     + ' si1, si2,si1sq,si2sq, si1i2',
c     + si1(ialpha), si2(ialpha),si1sq(ialpha),
c     + si2sq(ialpha), si1i2(ialpha),  
c     + ' sai1, sai2,sai1sq, sai2sq, sai1i2',
c     + sai1(ialpha), sai2(ialpha),sai1sq(ialpha), 
c     + sai2sq(ialpha), sai1i2(ialpha)       
c      write(6,7022) alpha,corr1(ialpha),corr2(ialpha), 
c     + numneg(1,ialpha),amom,bmom
c 7022 format(/,1x,'  Twinning factor',F6.2,/,
c     +       ' Correlation coefficient (on intensities) for input',
c     +       ' data',F6.3,/,1X,'Correlation coefficient after ',
c     +       'detwinning',F6.3,/,1X,'Number of negative Is',I18,/,
c     + ' amom'f12.2,' bmom',f12.2)

        do 310 i=1,nrange
          if (suma(i,ialpha).gt.0) then
            sumai = suma(i,ialpha)
            numer = si1i2a(i,ialpha) - 
     +              si1a(i,ialpha)*si2a(i,ialpha)/sumai
            denom = sqrt(si1sqa(i,ialpha) -
     +                   si1a(i,ialpha)**2/sumai)*
     +              sqrt(si2sqa(i,ialpha)-si2a(i,ialpha)**2/sumai)
            if (denom.gt.0.0) corr1a(i,ialpha) = numer/denom
            if (tuma(i,ialpha).gt.0) then
              tumai = tuma(i,ialpha)
              numer = ti1sqa(i,ialpha)/tumai
              denom = ti1a(i,ialpha)/tumai
              denom = denom*denom
              amom2(i,ialpha) = numer/denom
C  Get second moments too
              numer = tai1sqa(i,ialpha)/tumai
              denom = tai1a(i,ialpha)/tumai
              bmom2(i,ialpha) = numer/(denom**2)
        bmom(ialpha) = bmom(ialpha) + bmom2(i,ialpha)*tuma(i,ialpha)
            end if
c
            numer = sai1i2a(i,ialpha) - 
     +              sai1a(i,ialpha)*sai2a(i,ialpha)/sumai
            denom = sqrt(sai1sqa(i,ialpha) -
     +                   sai1a(i,ialpha)**2/sumai)*
     +              sqrt(sai2sqa(i,ialpha)-sai2a(i,ialpha)**2/sumai)
            if (denom.gt.0.0) corr2a(i,ialpha) = numer/denom
          end if
 310    continue
c
Cpete number in table now right for c.c. but 2nd moments are based 
Cpete on acentrics only
c
      write(6,1030) (dmin(i),nint(suma(i,ialpha)),corr1a(i,ialpha),
     +               corr2a(i,ialpha),
     +               amom2(i,ialpha),bmom2(i,ialpha),i=1,nrange)
 1030 format(/,1x,' Correlation coefficient as a function of ',
     +            'resolution.',/,
     +            '    Dmin   Number   CC_before    CC_after ',
     +            '2nd_moment_before   and_after',/,
     +            '                   detwinning   detwinning',
     +            '    detwinning     detwinning',/,
     +            (1x,F7.2,I9,F10.3,F14.3,F10.3,F14.3))
 311   continue
C      write(6,1022) alpha,corr1,corr2, numneg(ialpha) 
C 1022 format(/,1x,'  Twinning factor',F6.2,/,
C     +       ' Correlation coefficient (on intensities) for input',
C     +       ' data',F6.3,/,1X,'Correlation coefficient after ',
C     +       'detwinning',F6.3,/,1X,'Number of negative Is',I8)
      call ccp4h_rule()
      call ccp4h_header('Plot Showing Correlation of Measured and 
     +Detwinned Intensities','corr',3)
      call ccp4h_graph_beg(0,0)
      WRITE (6,1021) titout(1:20)
 1021 FORMAT(//,
     $ ' $TABLE: ',
     $ 'Britton plot, Correlations and 2nd Moments v Tw_factor - ',
     $  A,' :',/,
     $ ' $GRAPHS',/,
     $  ' : Britton plot; Number of negative intensities and change',
     $  ' after detwinning :A:1,4,5,6:',/,
     $ ' :Correlation of measured and detwinned intensities ',
     $  ':0|0.5x-1|1:1,2,3:',/,
     $ ' : Second moments (exp. 2.0) of obs and detwinned Z v Tw_f',
     $ ':0|0.5x0|4:1,7:',/,
     $ ' $$',/
     $ '  Twin_fac Corr_meas   Corr_detw NnegIs-all NnegIs-NSig',/,
     $  41x,'Diff_negIs-all Mom2_detw ',/,
     $ ' $$',
     $/,' $$',/)
      DO 325 ialpha=1,nalpha
        alpha = float(ialpha)/50.0
        idiff = 0
        if(ialpha.gt.1)idiff = numneg(1,ialpha) - numneg(1,(ialpha-1))
        if (tum(ialpha).gt.0) then
          write(6,1022) 
     +    alpha,corr1(ialpha),corr2(ialpha),numneg(1,ialpha),
     +    numneg(2,ialpha),idiff,bmom(ialpha)/tum(ialpha)
 1022     format(F8.2, 2F12.3,3I8,F10.3)
        else
          write(6,fmt='(F8.2, 2F12.3,3I8,A)')
     +    alpha,corr1(ialpha),corr2(ialpha),numneg(1,ialpha),
     +    numneg(2,ialpha),idiff,' ??'
        end if
       
 325  continue
c
Cpete number in table now right for c.c. not 2nd moments are based
Cpete on acentrics only
c
      write(6,*)' $$'
      call ccp4h_graph_end()
      call ccp4h_rule()
      call ccp4h_header('Second Moments and Correlation Coefficient of
     + Observed and Detwinned Intensities vs.Twinning Factor'
     + ,'moments',3)
      call ccp4h_graph_beg(0,0)
      WRITE (6,1023)titout(1:20)
 1023 FORMAT(///,
     $ '  Correlation coefficients and second moments as a function ',
     $ 'of twinning factor &resolution.:',/,
     +            '    Dmin   Number   CC_before    CC_after ',
     +            '2nd_moment_before   and_after',/,
     +            '                   detwinning   detwinning',
     +            '    detwinning     detwinning',///,
     $ ' $TABLE: ',
     $ 'Resln dependent Correlations and  moments  v Tw_factor ',A,',:',
     $ /,' $GRAPHS',/,
     $' : Second moments (exp. 2.0) for ranges(1-5) of obs and ',
     # 'detwinned Z v Tw_f',/,
     $ ':0|0.5x0|4:1,7,12,13,19,24,25,31:',/,
     $' : Second moments (exp. 2.0) for ranges(6-10) of obs and ',
     # 'detwinned Z v Tw_f',/,
     $ ':0|0.5x0|4:1,37,42,43,49,54,55,61:',/,
     $' : Correlation for ranges(1-5) of obs and detwinned Is v Tw_f',/,
     $ ':0|0.5x-1|1:1,4,5,10,11,16,17,22,23,28,29:',/,
     $' : Correlation for ranges(6-10) of obs and detwinned Is v Tw_f',
     $ /,':0|0.5x-1|1:1,34,35,40,41,46,47,52,53,58,59:',/,
     $ ' $$',/
     $ ' Twinning_factor',/,
     $ ' Res_1 Nrf_1 Corr_meas_1 corr_detw_1 2Mom_meas_1 2Mom_detw_1',/,
     $ ' Res_2 Nrf_2 Corr_meas_2 corr_detw_2 2Mom_meas_2 2Mom_detw_2',/,
     $ ' Res_3 Nrf_3 Corr_meas_3 corr_detw_3 2Mom_meas_3 2Mom_detw_3',/,
     $ ' Res_4 Nrf_4 Corr_meas_4 corr_detw_4 2Mom_meas_4 2Mom_detw_4',/,
     $ ' Res_5 Nrf_5 Corr_meas_5 corr_detw_5 2Mom_meas_5 2Mom_detw_5',/,
     $ ' Res_6 Nrf_6 Corr_meas_6 corr_detw_6 2Mom_meas_6 2Mom_detw_6',/,
     $ ' Res_7 Nrf_7 Corr_meas_7 corr_detw_7 2Mom_meas_7 2Mom_detw_7',/,
     $ ' Res_8 Nrf_8 Corr_meas_8 corr_detw_8 2Mom_meas_8 2Mom_detw_8',/,
     $ ' Res_9 Nrf_9 Corr_meas_9 corr_detw_9 2Mom_meas_9 2Mom_detw_9',/,
     $ ' Res_10 Nrf_10 Corr_meas_10 corr_detw_10 2Mom_meas_10',
     $ ' 2Mom_detw_10',/,
     $ ' $$',
     $/,' $$',/)
      DO 335 ialpha=1,nalpha
        alpha = float(ialpha)/50.0
      write(6,1024) 
     +  alpha,
     + (dmin(i),nint(suma(i,ialpha)),corr1a(i,ialpha), corr2a(i,ialpha),
     +               amom2(i,ialpha),bmom2(i,ialpha),i=1,nrange)
c
cpjx - Can't have the newlines in format statement below because of loggraph
cpjx   It needs columns demarced by spaces, rows by newlines
cpjx 1024 format(F8.2, 10(/,10x,F7.2,I9,F10.3,F14.3,F10.3,F14.3))
 1024 format(F8.2, 10(10x,F7.2,I9,F10.3,F14.3,F10.3,F14.3))
 335  continue
      write(6,*)' $$'
      call ccp4h_graph_end()
      call ccperr(0, ' ** Normal termination **')
 999  call ccperr(1, ' ** Error in input **')
      call ccp4h_pre_end()
      end

