C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
c
c ----------------------------------------------------------------------
c
c Non-crystallographic symmetry library
c
c ----------------------------------------------------------------------
c
      subroutine ncsinii()
      include 'dmheader.fh'
      include 'crystal.fh'
      include 'ncssym.fh'
c ncsini - convert ncs matrices to fractional coords
c
      integer i,j
      real temp(4,4)
c
      do 200 i=1,ndomain
       do 200 j=1,nncsym(i)
       call symmatmul(rsymncs(1,1,j,i),q,temp)
       call symmatmul(qt,temp,rsymncs(1,1,j,i))
 200  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
c map averaging routines
c
c ----------------------------------------------------------------------
c
      subroutine ncsref(hkls,lfa,lfb,rhospl,ncsmsk)
      include 'dmheader.fh'
      include 'uvwdata.fh'
      include 'cycl.fh'
c NCSAVER - non-crystallographic symmetry averaging
      real hkls(*),rhospl(*)
      integer ncsmsk(*)
      integer lfa,lfb
c
      external ncsplinewt
c
      call wfo(hkls,lfa,lfb)
      call hklfpab(hkls,lfa,lfb)
      call copylistmap(hkls,lfa,lfb,rhospl,nu,nv,nw)
      call forallrfl(ncsplinewt,rhospl)
      call fft3hr(rhospl,nu,nv,nw)
      call ncsrefine(rhospl,ncsmsk)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine ncsaver(hkls,lfa,lfb,rhomap,rhospl,ncsmsk)
      include 'dmheader.fh'
      include 'uvwdata.fh'
      include 'cycl.fh'
c NCSAVER - non-crystallographic symmetry averaging
      real hkls(*),rhomap(*),rhospl(*)
      integer lfa,lfb,ncsmsk(*)
c
      integer icorrel
      external ncsplinewt
c
      icorrel=2
      if (icycl.eq.1) icorrel=1
c
      call copylistmap(hkls,lfa,lfb,rhospl,nu,nv,nw)
      call forallrfl(ncsplinewt,rhospl)
      call fft3hr(rhospl,nu,nv,nw)
      call ncsavmap(rhomap,rhospl,ncsmsk,icorrel)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine ncsavmap(rhomap,rhospl,ncsmsk,icorrel)
      include 'dmheader.fh'
      include 'io.fh'
      include 'crystal.fh'
      include 'ncssym.fh'
      include 'uvwdata.fh'
      include 'output.fh'
c ncsavmap - apply ncs avergaing to the map
c first we will average over all ncs domains and symmetries,
c then we will expand to fill the unit cell.
c
      real rhomap(0:nu-1,0:nv-1,0:nw-1),rhospl(0:nu-1,0:nv-1,0:nw-1)
      integer ncsmsk(msku1:msku2,mskv1:mskv2,mskw1:mskw2),icorrel
c
      real sn(maxncs,maxncs),sx(maxncs,maxncs),sy(maxncs,maxncs),
     +     sx2(maxncs,maxncs),sy2(maxncs,maxncs),sxy(maxncs,maxncs)
      integer iu,iv,iw,ju,jv,jw,ku,kv,kw,lmod
      integer i,j,idom,incs,jncs,nncs,isym,nlim
      real rinvncs(4,4),u,v,w,u1,v1,w1,u2,v2,w2,rx,ry,bsplinterp
      external bsplinterp,lmod
c
c
c FIRST AVERAGE INSIDE EACH NCS OF EACH DOMAIN FROM ALL OTHER
c NCS TRANSFORMATIONS OF THAT DOMAIN
c
      if (icorrel.eq.1)
     +  call ccp4h_header('Initial NCS Averaging Correlations',
     +                    'ncscorr',3)
c loop though each independant set in turn
      do 500 idom=1,ndomain
c
       nncs=nncsym(idom)
c
c zero statistics arrays
       do 50 j=1,maxncs
       do 50 i=1,maxncs
        sn(i,j)=0.0
        sx(i,j)=0.0
        sy(i,j)=0.0
        sx2(i,j)=0.0
        sy2(i,j)=0.0
        sxy(i,j)=0.0
 50    continue
c
c loop though each region in turn, averaging into it from the others
c
       do 300 incs=1,nncs
c
c first generate the transformation to get us back in the identity region
        call invsym(rsymncs(1,1,incs,idom),rinvncs(1,1))
c
c now loop through each point in this region
        do 200 iw=nuvwlim(3,incs,idom),nuvwlim(6,incs,idom)
        jw=lmod(iw,nw)
        w=real(iw)/nw
        do 200 iv=nuvwlim(2,incs,idom),nuvwlim(5,incs,idom)
        jv=lmod(iv,nv)
        v=real(iv)/nv
        do 200 iu=nuvwlim(1,incs,idom),nuvwlim(4,incs,idom)
        ju=lmod(iu,nu)
        u=real(iu)/nu
c shift point back to identity domain, average from all equivs (inc this)
         call symuvw(rinvncs,u,v,w,u1,v1,w1)
         ku=nint(nu*u1)
         kv=nint(nv*v1)
         kw=nint(nw*w1)
c check if we are in the masked region
         if (ku.ge.msku1.and.kv.ge.mskv1.and.kw.ge.mskw1.and.
     +       ku.le.msku2.and.kv.le.mskv2.and.kw.le.mskw2) then
          if (ncsmsk(ku,kv,kw).eq.idom) then
c
           rx=rhomap(ju,jv,jw)
           if (rx.gt.-500.0) then
c note: density values -1000 have been modified (a bit inelegant)
            rhomap(ju,jv,jw)=-1000.0
            do 100 jncs=1,nncs
             call symuvw(rsymncs(1,1,jncs,idom),u1,v1,w1,u2,v2,w2)
             ry=bsplinterp(rhospl,u2,v2,w2)
             rhomap(ju,jv,jw)=rhomap(ju,jv,jw)+ry/nncs
c and do some stats
             sn (incs,jncs)=sn (incs,jncs)+1.0
             sx (incs,jncs)=sx (incs,jncs)+rx
             sy (incs,jncs)=sy (incs,jncs)+ry
             sx2(incs,jncs)=sx2(incs,jncs)+rx**2
             sy2(incs,jncs)=sy2(incs,jncs)+ry**2
             sxy(incs,jncs)=sxy(incs,jncs)+rx*ry
 100        continue
           endif
c
          endif
         endif
c next point
 200    continue
c
c next region
 300   continue
c
c Done the averaging calculation for this domain
c Now print out the averaging statistics
c
       write (lpt,905)idom
 905   format (
     +   '<h4>NCS AVERAGING CORRELATIONS BETWEEN REGIONS IN DOMAIN'
     +   ,i2,'</h4>')
       ncscrl(idom,icorrel)=0.0
       do 910 i=1,nncs
       do 910 j=1,nncs
        if (sn(i,j).ne.0.0)
     +    sn(i,j)=(sn(i,j)*sxy(i,j)-sx(i,j)*sy(i,j)) / sqrt(
     +    (sn(i,j)*sx2(i,j)-sx(i,j)**2)*(sn(i,j)*sy2(i,j)-sy(i,j)**2) )
        if (i.ne.j) ncscrl(idom,icorrel)=ncscrl(idom,icorrel)+sn(i,j)
 910   continue
       ncscrl(idom,icorrel)=ncscrl(idom,icorrel)/(nncs*(nncs-1))
       nlim=min(nncs,20)
       do 920 i=1,nlim
        write (lpt,930)(sn(i,j),j=1,nlim)
 920   continue
       write(lpt,*)
 930   format (1x,20f6.3)
c
c next domain
 500  continue
c
c NOW EXPAND CRYSTALLOGRAPHICALLY FROM ALL MASKED DOMAINS TO THE REST OF
c THE UNIT CELL.
c
      do 800 iw=0,nw-1
      w=real(iw)/nw
      do 800 iv=0,nv-1
      v=real(iv)/nv
      do 800 iu=0,nu-1
      u=real(iu)/nu
c if this point has been averaged, then expand it
       if (rhomap(iu,iv,iw).lt.-500.0) then
        rhomap(iu,iv,iw)=rhomap(iu,iv,iw)+1000.0
c loop through crystallographic symmetries
        do 700 isym=1,nsym
         call symuvw(rsym(1,1,isym),u,v,w,u1,v1,w1)
         ku=lmod(nint(u1*nu),nu)
         kv=lmod(nint(v1*nv),nv)
         kw=lmod(nint(w1*nw),nw)
         rhomap(ku,kv,kw)=rhomap(iu,iv,iw)
 700    continue
       endif
 800  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
c matrix refinement routines
c
c ----------------------------------------------------------------------
c
      subroutine ncsrefine(rhospl,ncsmsk)
      include 'dmheader.fh'
      include 'io.fh'
      include 'ncssym.fh'
c ncsrefine - refine ncs matrices
c
      integer ncsmsk(*)
      real rhospl(*)
c
      integer idom,incs,nncs
c
      write (lpt,50)
 50   format (' NCS Matrix Refinement:'/30x,
     +  ' d(alpha) d(beta) d(gamma) d(x)    d(y)    d(z)')
c
c loop through each domain and each symmetry (but not the identity!)
      do 500 idom=1,ndomain
       nncs=nncsym(idom)
c check refinement steps are non-zero
       if (refdr(idom).gt.0.0) then
        do 400 incs=2,nncs
         write (lpt,800)idom,incs
         call matrefine(rhospl,ncsmsk,idom,incs)
 400    continue
       endif
 500  continue
c
 800  format ('  Domain ',i2,' Symmetry ',i3)
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine matrefine(rhospl,ncsmsk,idom,incs)
      include 'dmheader.fh'
      include 'io.fh'
      include 'ncssym.fh'
      include 'crystal.fh'
c matrefine - refine one ncs matrix
c
      integer ncsmsk(msku1:msku2,mskv1:mskv2,mskw1:mskw2),idom,incs
      real rhospl(*)
c
      integer i,j,k,iparam
      real temp1(4,4),temp2(4,4),drdx(4,4,6),rncs(4,4,0:2)
      real xc,yc,zc,a,b,x,y,y2,step,dp(6),dq(6),dy(6),z(0:2)
c
      call symcpy(rsymncs(1,1,incs,idom),rncs(1,1,0))
c
c first we must make the fractionalised derivative matrices for the six
c refinement parameters (this is a bit complicated)
c
c pick up molelcule centre (in orthogonal coords)
      xc=xyzcen(1,incs,idom)
      yc=xyzcen(2,incs,idom)
      zc=xyzcen(3,incs,idom)
c make partial derivative matrices (jacobian)
      do 60 k=1,6
      do 60 j=1,4
      do 60 i=1,4
       drdx(i,j,k)=0.0
 60   continue
      drdx(2,3,1)= 1.0
      drdx(2,4,1)=-zc
      drdx(3,2,1)=-1.0
      drdx(3,4,1)= yc
      drdx(3,1,2)= 1.0
      drdx(3,4,2)=-xc
      drdx(1,3,2)=-1.0
      drdx(1,4,2)= zc
      drdx(1,2,3)= 1.0
      drdx(1,4,3)=-yc
      drdx(2,1,3)=-1.0
      drdx(2,4,3)= xc
      drdx(1,4,4)=1.0
      drdx(2,4,5)=1.0
      drdx(3,4,6)=1.0
c make coordinate shifts
      do 100 i=1,3
       dp(i)=dtor*refdphi(idom)
       dp(i+3)=refdr(idom)
 100  continue
c
      step=1.0
      i=0
c
 200   i=i+1
c FIRST WE WORK OUT THE STEP DIRECTION
c calculate the correlation, and its gradient with each of the 6 params
       call ncscorgrd(rhospl,ncsmsk,idom,rncs,drdx,y,dy)
       y2=sqrt(dy(1)**2+dy(2)**2+dy(3)**2+dy(4)**2+dy(5)**2+dy(6)**2)
c
c NOW WE USE QUADRATIC TRIAL TO GET STEP LENGTH
c make matrices for shifts of lengths 1 and 2
       call symcpy(rncs(1,1,0),rncs(1,1,1))
       call symcpy(rncs(1,1,0),rncs(1,1,2))
       do 340 iparam=1,6
        dq(iparam)=step*dy(iparam)*dp(iparam)/y2
c 1/2 x input step
        call makemat(temp1,iparam,0.5*dq(iparam),xc,yc,zc)
        call symcpy(rncs(1,1,1),temp2)
        call symmatmul(temp1,temp2,rncs(1,1,1))
c  1  x input step
        call makemat(temp1,iparam,dq(iparam),xc,yc,zc)
        call symcpy(rncs(1,1,2),temp2)
        call symmatmul(temp1,temp2,rncs(1,1,2))
 340   continue
c now generate correlations
       z(0)=y
       call ncscorrel(rhospl,ncsmsk,idom,rncs(1,1,1),z(1),2)
c
       b=4.0*z(1)-3.0*z(0)-z(2)
       a=2.0*z(2)+2.0*z(0)-4.0*z(1)
       x=0.0
       if (a.lt.0.0) then
        x=min(max(-0.5*b/a,0.0),1.0)
       else
        if (z(2).gt.z(0)) x=1.0
       endif
c NOW DO THE SHIFT
       do 360 iparam=1,6
        call makemat(temp1,iparam,x*dq(iparam),xc,yc,zc)
        call symcpy(rncs(1,1,0),temp2)
        call symmatmul(temp1,temp2,rncs(1,1,0))
 360   continue
       write (lpt,810)z(0),(x*dq(j)/dtor,j=1,3),(x*dq(j),j=4,6)
c guess a step size for next step
       step=step*sqrt(x)
c next refinement cycle
      if (step.gt.0.25.and.i.lt.8) goto 200
c
      call symcpy(rncs(1,1,0),rsymncs(1,1,incs,idom))
 810  format ('   Correlation: ',f6.3,'  Step: ',6f8.2)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine ncscorrel(rhospl,ncsmsk,idom,rmat,correl,nmat)
      include 'dmheader.fh'
      include 'io.fh'
      include 'ncssym.fh'
      include 'uvwdata.fh'
c ncscorrel - perform a ncs correlation using the given matrix
c   (can speed this up by trying lots of matrices at once)
c
      real rhospl(*),rmat(4,4,*),correl(*)
      integer ncsmsk(msku1:msku2,mskv1:mskv2,mskw1:mskw2),idom,nmat
c
      integer i,iu,iv,iw
      real u,v,w,u1,v1,w1,rx,ry,sn,sx,sx2,sy(100),sy2(100),sxy(100)
      real bsplinterp
      external bsplinterp
c
      sn=0.0
      sx=0.0
      sx2=0.0
      do 50 i=1,nmat
       sy(i)=0.0
       sy2(i)=0.0
       sxy(i)=0.0
 50   continue
c
c accumulate correlation statistics
      do 100 iw=mskw1,mskw2
      w=real(iw)/nw
      do 100 iv=mskv1,mskv2
      v=real(iv)/nv
      do 100 iu=msku1,msku2
      u=real(iu)/nu
       if (ncsmsk(iu,iv,iw).eq.idom) then
        rx=bsplinterp(rhospl,u,v,w)
        sn=sn+1.0
        sx=sx+rx
        sx2=sx2+rx**2
        do 90 i=1,nmat
         call symuvw(rmat(1,1,i),u,v,w,u1,v1,w1)
         ry=bsplinterp(rhospl,u1,v1,w1)
         sy(i)=sy(i)+ry
         sy2(i)=sy2(i)+ry**2
         sxy(i)=sxy(i)+rx*ry
 90     continue
       endif
 100  continue
c
c and calculate correlations
      do 150 i=1,nmat
       correl(i)=(sn*sxy(i)-sx*sy(i))/
     +   sqrt((sn*sx2-sx**2)*(sn*sy2(i)-sy(i)**2))
 150   continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine ncscorgrd
     +  (rhospl,ncsmsk,idom,rmat,rderiv,correl,dcorrel)
      include 'dmheader.fh'
      include 'io.fh'
      include 'ncssym.fh'
      include 'crystal.fh'
      include 'uvwdata.fh'
c ncscorgrd - perform a ncs correlation using the given matrix
c  and calculate correlation gradiants along directions given by
c  partial derivative tensors rderiv.
c
      real rhospl(*),rmat(4,4),rderiv(4,4,6),correl,dcorrel(6)
      integer ncsmsk(msku1:msku2,mskv1:mskv2,mskw1:mskw2),idom
c
      integer i,iu,iv,iw
      real u,v,w,u1,v1,w1,x1,y1,z1,dx1,dy1,dz1,gu(3),gx(3),dr(6)
      real rx,ry,sn,sx,sy,sx2,sy2,sxy,gy(6),gy2(6),gxy(6)
      real bsplinterp
      external bsplinterp
c
c
      sn=0.0
      sx=0.0
      sy=0.0
      sx2=0.0
      sy2=0.0
      sxy=0.0
      do 50 i=1,6
       gy(i)=0.0
       gy2(i)=0.0
       gxy(i)=0.0
 50   continue
c
c accumulate correlation statistics
      do 100 iw=mskw1,mskw2
      w=real(iw)/nw
      do 100 iv=mskv1,mskv2
      v=real(iv)/nv
      do 100 iu=msku1,msku2
      u=real(iu)/nu
       if (ncsmsk(iu,iv,iw).eq.idom) then
        call symuvw(rmat(1,1),u,v,w,u1,v1,w1)
        rx=bsplinterp(rhospl,u,v,w)
        call bsplintgrd(rhospl,u1,v1,w1,ry,gu)
        gu(1)=gu(1)*nu
        gu(2)=gu(2)*nv
        gu(3)=gu(3)*nw
        gx(1)=qt(1,1)*gu(1)+qt(2,1)*gu(2)+qt(3,1)*gu(3)
        gx(2)=qt(1,2)*gu(1)+qt(2,2)*gu(2)+qt(3,2)*gu(3)
        gx(3)=qt(1,3)*gu(1)+qt(2,3)*gu(2)+qt(3,3)*gu(3)
        sn=sn+1.0
        sx=sx+rx
        sx2=sx2+rx**2
        sy=sy+ry
        sy2=sy2+ry**2
        sxy=sxy+rx*ry
c calculate gradient along parameter directions
        do 90 i=1,6
c note: derivs and grads are on orthogonal coordinates
         call symuvw(q,u1,v1,w1,x1,y1,z1)
         call symuvw(rderiv(1,1,i),x1,y1,z1,dx1,dy1,dz1)
         dr(i)=dx1*gx(1)+dy1*gx(2)+dz1*gx(3)
c now calc correl gradient accumulants
         gy(i)=gy(i)+dr(i)
         gy2(i)=gy2(i)+2.0*ry*dr(i)
         gxy(i)=gxy(i)+rx*dr(i)
 90     continue
       endif
 100  continue
c
c and calculate correlation and gradients
      correl=(sn*sxy-sx*sy)/sqrt((sn*sx2-sx**2)*(sn*sy2-sy**2))
      do 150 i=1,6
       dcorrel(i)=(sn*gxy(i)-sx*gy(i))/
     +   sqrt((sn*sx2-sx**2)*(sn*sy2-sy**2))
c FUDGE: assumes variance is constant (tests OK). correct code is:
c     +   ( (sn*gxy(i)-sx*gy(i))*(sn*sx2-sx**2)*(sn*sy2-sy**2) -
c     +   0.5*(sn*sxy-sx*sy)*(sn*sx2-sx**2)*(sn*gy2(i)-2.0*sy*gy(i)) )/
c     +   ((sn*sx2-sx**2)*(sn*sy2-sy**2))**1.5
 150  continue
c
      return
      end
c
c this code checks the gradients but not rderiv:-
c         x2=x1+0.001*
c     +(rderiv(1,1,i)*x1+rderiv(1,2,i)*y1+rderiv(1,3,i)*z1+rderiv(1,4,i))
c         y2=y1+0.001*
c     +(rderiv(2,1,i)*x1+rderiv(2,2,i)*y1+rderiv(2,3,i)*z1+rderiv(2,4,i))
c         z2=z1+0.001*
c     +(rderiv(3,1,i)*x1+rderiv(3,2,i)*y1+rderiv(3,3,i)*z1+rderiv(3,4,i))
c         call symuvw(qt,x2,y2,z2,u2,v2,w2)
c         dr1(i)=(bsplinterp(rhospl,u2,v2,w2)-ry)/0.001
c
c ----------------------------------------------------------------------
c
      subroutine makemat(mat,iparam,dp,cx,cy,cz)
      include 'dmheader.fh'
      include 'crystal.fh'
c makemat - make a matrix to shift parameter i
c
      integer iparam
      real mat(4,4),cx,cy,cz,dp
c
      real temp(4,4),dphi
      integer i,j
c
      dphi=dp
c
      do 110 j=1,4
       do 100 i=1,4
        mat(i,j)=0.0
 100   continue
       mat(j,j)=1.0
 110  continue
c
      if (iparam.le.3) then
       i=mod(iparam,3)+1
       j=mod(i,3)+1
c rotation about 0 0 0
       mat(i,i)=cos(dphi)
       mat(j,j)=cos(dphi)
       mat(i,j)=sin(dphi)
       mat(j,i)=-sin(dphi)
c adjust for rotation about cx cy cz
       mat(1,4)=cx-(mat(1,1)*cx+mat(1,2)*cy+mat(1,3)*cz)
       mat(2,4)=cy-(mat(2,1)*cx+mat(2,2)*cy+mat(2,3)*cz)
       mat(3,4)=cz-(mat(3,1)*cx+mat(3,2)*cy+mat(3,3)*cz)
      else
c translation along x,y,z
       mat(iparam-3,4)=dp
      endif
c
c now fractionalise the operation
      call symmatmul(mat,q,temp)
      call symmatmul(qt,temp,mat)
c
      return
      end
c
c ----------------------------------------------------------------------
c
c Now we have the routines relating to mask input and creation
c
c ----------------------------------------------------------------------
c
      subroutine ncsmasksize()
      include 'dmheader.fh'
      include 'io.fh'
      include 'crystal.fh'
      include 'ncssym.fh'
      include 'uvwdata.fh'
c ncsmasksize - read mask headers and set up array sizes.
c
      character name*10
      integer lsec(maxsec)
      integer mf1,mm1,ms1,mf2,mm2,ms2,msec,mspgrp,mode,ifail
      integer ifast,imedm,islow,lfast,lmedm,lslow
      integer jfms(3),juvw(3),mxyz(3)
      integer i,j,ju,jv,jw,mu,mv,mw,idom,incs,nncs,lmod
      real u,v,w,u1,v1,w1,fms(3),mcell(6),rmin,rmax,rmean,rrms,cerr
      character title*80
      external lmod
c
c Note: juvw convert from fast/med/slow to u/v/w
c       jfms convert from u/v/w to fast/med/slow
c
      do 50 j=1,maxdom
      do 50 i=1,maxncs
       nuvwlim(1,i,j)=10000
       nuvwlim(2,i,j)=10000
       nuvwlim(3,i,j)=10000
       nuvwlim(4,i,j)=-10000
       nuvwlim(5,i,j)=-10000
       nuvwlim(6,i,j)=-10000
 50   continue
      msku1=1000
      mskv1=1000
      mskw1=1000
      msku2=-1000
      mskv2=-1000
      mskw2=-1000
c
c loop through domains
      do 400 idom=1,ndomain
c
       nncs=nncsym(idom)
c
c make mask name
c
       write (name,100)idom
 100   format ('NCSIN',i1)
c
c now open map header and read map, re-ordering it as necessary
c
       ifail=0
       call mrdhds(lmsks,name,title,msec,jfms,mxyz,ms1,mf1,mf2,
     +   mm1,mm2,mcell,mspgrp,mode,rmin,rmax,rmean,rrms,ifail,0)
       ms2=ms1+msec-1
       do 110 i=1,3
        juvw(jfms(i))=i
 110   continue
c find out the mask grid dimensions
       lfast=mf2-mf1+1
       lmedm=mm2-mm1+1
       lslow=ms2-ms1+1
       mu=mxyz(1)
       mv=mxyz(2)
       mw=mxyz(3)
c check map format OK
       cerr=0.0
       do 120 i=1,6
        cerr=cerr+abs(cell(i)-mcell(i))/(cell(i)+mcell(i))
 120   continue
       if (cerr.gt.0.01)
     +   call ccperr(1,' ncsmsksize - Inconsistent cell info')
       if (lfast*lmedm.gt.maxsec)
     +   call ccperr(1,' ncsmsksize - Mask section > maxsec: recompile')
       if (mode.ne.0)
     +   call ccperr(1,' ncsmsksize - Mask file is not logical*1')
c now read the map in the order it is on file
       do 200 islow=0,lslow-1
c get a section
        ifail=0
        call mgulp(lmsks,lsec,ifail)
        if (ifail.ne.0) call ccperr(1,' ncsmasksize - ccp4 read fail')
c now sort into uvw mask
        fms(3)=real(islow+ms1)
        do 190 imedm=0,lmedm-1
        fms(2)=real(imedm+mm1)
        do 190 ifast=0,lfast-1
        fms(1)=real(ifast+mf1)
         call ccpgtb(i,lsec,ifast+lfast*imedm+1)
         if (i.ne.0) then
          do 150 incs=1,nncs
           u=fms(juvw(1))/mu
           v=fms(juvw(2))/mv
           w=fms(juvw(3))/mw
           call symuvw(rsymncs(1,1,incs,idom),u,v,w,u1,v1,w1)
           ju=nint(u1*nu)
           jv=nint(v1*nv)
           jw=nint(w1*nw)
           nuvwlim(1,incs,idom)=min(nuvwlim(1,incs,idom),ju)
           nuvwlim(2,incs,idom)=min(nuvwlim(2,incs,idom),jv)
           nuvwlim(3,incs,idom)=min(nuvwlim(3,incs,idom),jw)
           nuvwlim(4,incs,idom)=max(nuvwlim(4,incs,idom),ju)
           nuvwlim(5,incs,idom)=max(nuvwlim(5,incs,idom),jv)
           nuvwlim(6,incs,idom)=max(nuvwlim(6,incs,idom),jw)
 150      continue
         endif
 190    continue
 200   continue
c get limits of identity box
       msku1=min(msku1,nuvwlim(1,1,idom))
       mskv1=min(mskv1,nuvwlim(2,1,idom))
       mskw1=min(mskw1,nuvwlim(3,1,idom))
       msku2=max(msku2,nuvwlim(4,1,idom))
       mskv2=max(mskv2,nuvwlim(5,1,idom))
       mskw2=max(mskw2,nuvwlim(6,1,idom))
c close the map file
       call mrclos(lmsks)
 400  continue
c
      ncssize=(msku2-msku1+1)*(mskv2-mskv1+1)*(mskw2-mskw1+1)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine ncsmaskin(ncsmsk,lmask)
      include 'dmheader.fh'
      include 'crystal.fh'
      include 'ncssym.fh'
      include 'uvwdata.fh'
c ncsmaskin - read in ncs mask for monomers and expand to multimer
c
      integer ncsmsk(msku1:msku2,mskv1:mskv2,mskw1:mskw2),lmask(0:*)
c
      integer mu,mv,mw,mu1,mv1,mw1,mu2,mv2,mw2,nspgrp1
      integer i,iu,iv,iw,idom,lmod
      real cell1(6),bsplinterp
      character name*10,title*600
      external bsplinterp,lmod
c
c first clear stat data and mask
c
      do 60 iw=mskw1,mskw2
      do 60 iv=mskv1,mskv2
      do 60 iu=msku1,msku2
       ncsmsk(iu,iv,iw)=0
 60   continue
c
c MASK READ: loop though masks, reading them in and storing in the map.
c
      do 800 idom=ndomain,1,-1
c
c read in the mask for this domain
c
       write (name,100)idom
 100   format ('NCSIN',i1)
c
c read this mask as 1's and copy onto grid
       call ccpmaphead(name,nspgrp1,cell1,
     +   mu,mv,mw,mu1,mv1,mw1,mu2,mv2,mw2)
       call ccpmskin(name,title,lmask,mu1,mv1,mw1,mu2,mv2,mw2)
       call maskgrid(lmask,mu,mv,mw,mu1,mv1,mw1,mu2,mv2,mw2,
     +   ncsmsk,nu,nv,nw,msku1,mskv1,mskw1,msku2,mskv2,mskw2)
c
c now change 1's to domain numbers
       do 200 iw=mskw1,mskw2
       do 200 iv=mskv1,mskv2
       do 200 iu=msku1,msku2
        if (ncsmsk(iu,iv,iw).eq.1) ncsmsk(iu,iv,iw)=idom
 200   continue
c
c now go on to next domain and next mask
c
 800  continue
c
c MASK READ DONE
c
c now remove overlap from mask
      call ncstrim(ncsmsk,lmask)
c
c and display a section
      call maskstats(ncsmsk,msku2-msku1+1,mskv2-mskv1+1,mskw2-mskw1+1,
     +  (mskw2-mskw1+1)/2)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine ncstrim(ncsmsk,ncsmap)
      include 'dmheader.fh'
      include 'io.fh'
      include 'crystal.fh'
      include 'ncssym.fh'
      include 'uvwdata.fh'
c ncstrim - remove overlap due to crystallographic symmtry from mask
c     s/r for convenience of array indexing.
c
      integer ncsmsk(msku1:msku2,mskv1:mskv2,mskw1:mskw2)
      integer ncsmap(0:nu-1,0:nv-1,0:nw-1)
c
      integer iu,iv,iw,ju,jv,jw,ku,kv,kw
      integer i,j,idom,incs,nncs,m,lmod
      real u,v,w,u1,v1,w1,x1,y1,z1,rinvncs(4,4)
      real sn(-1:maxncs*maxdom),suvw(4,maxdom)
      external lmod
c
c clear work arrays
c
      do 30 i=-1,maxncs*maxdom
       sn(i)=0.0
 30   continue
      do 50 j=1,maxdom
       do 40 i=1,4
        suvw(i,j)=0.0
 40    continue
 50   continue
c
      do 80 iw=0,nw-1
      do 80 iv=0,nv-1
      do 80 iu=0,nu-1
       ncsmap(iu,iv,iw)=0
 80   continue
c
c first remove any previous overlap markers and accumulate CoM (before trim)
c
      do 100 iw=mskw1,mskw2
      do 100 iv=mskv1,mskv2
      do 100 iu=msku1,msku2
c ..remove -ve
       ncsmsk(iu,iv,iw)=abs(ncsmsk(iu,iv,iw))
c ..and accumulate centre of mass data
       idom=ncsmsk(iu,iv,iw)
       if (idom.ge.1.and.idom.le.ndomain) then
        suvw(1,idom)=suvw(1,idom)+real(iu)/nu
        suvw(2,idom)=suvw(2,idom)+real(iv)/nv
        suvw(3,idom)=suvw(3,idom)+real(iw)/nw
        suvw(4,idom)=suvw(4,idom)+1.0
       endif
 100  continue
c
c 1. MAKE MODEL OF MULTIMER IN UNIT CELL
c loop through each domain in turn and apply all ncs to make model of
c multimer in unit cell
c
      do 400 idom=1,ndomain
c
       nncs=nncsym(idom)
c
       do 300 incs=1,nncs
c
c first generate the transformation to get us back in the identity region
        call invsym(rsymncs(1,1,incs,idom),rinvncs(1,1))
c
c now loop through each point in this region
        do 200 iw=nuvwlim(3,incs,idom),nuvwlim(6,incs,idom)
        jw=lmod(iw,nw)
        w=real(iw)/nw
        do 200 iv=nuvwlim(2,incs,idom),nuvwlim(5,incs,idom)
        jv=lmod(iv,nv)
        v=real(iv)/nv
        do 200 iu=nuvwlim(1,incs,idom),nuvwlim(4,incs,idom)
        ju=lmod(iu,nu)
        u=real(iu)/nu
c shift point back to identity domain, average from all equivs (inc this)
         call symuvw(rinvncs,u,v,w,u1,v1,w1)
         ku=nint(nu*u1)
         kv=nint(nv*v1)
         kw=nint(nw*w1)
c check if we are in the masked region
         if (ku.ge.msku1.and.kv.ge.mskv1.and.kw.ge.mskw1.and.
     +       ku.le.msku2.and.kv.le.mskv2.and.kw.le.mskw2) then
          if (ncsmsk(ku,kv,kw).eq.idom) then
           if (ncsmap(ju,jv,jw).eq.0) then
            ncsmap(ju,jv,jw)=incs+maxncs*idom
           else
            ncsmap(ju,jv,jw)=-1
           endif
          endif
         endif
 200    continue
 300   continue
 400  continue
c
c 2. REMOVE XTAL SYMM OVERLAP
c now loop through xtal sym to build whole unit cell, marking overlap
c
      do 700 iw=0,nw-1
      w=real(iw)/nw
      do 700 iv=0,nv-1
      v=real(iv)/nv
      do 700 iu=0,nu-1
      u=real(iu)/nu
       if (ncsmap(iu,iv,iw).gt.0) then
        do 600 m=2,nsym
         call symuvw(rsym(1,1,m),u,v,w,u1,v1,w1)
         ju=lmod(nint(nu*u1),nu)
         jv=lmod(nint(nv*v1),nv)
         jw=lmod(nint(nw*w1),nw)
         if (iu.ne.ju.and.iv.ne.jv.and.iw.ne.jw.and.
     +       ncsmap(ju,jv,jw).ne.0) ncsmap(iu,iv,iw)=-1
 600    continue
       endif
 700  continue
c
c 3. UPDATE MASK
c finally, loop through mask again, checking against the cell to see
c if there is overlap
c
c (skip this if we have a multimer mask)
      if (ovlprm.eq.1) then
c remove any overlapped points
       do 800 iw=mskw1,mskw2
       jw=lmod(iw,nw)
       do 800 iv=mskv1,mskv2
       jv=lmod(iv,nv)
       do 800 iu=msku1,msku2
       ju=lmod(iu,nu)
        if (ncsmsk(iu,iv,iw).gt.0.and.ncsmap(ju,jv,jw).le.0) then
         ncsmsk(iu,iv,iw)=-abs(ncsmsk(iu,iv,iw))
        endif
 800   continue
      endif
c
c MASK IS NOW FINISHED...
c loop through symmetries generating centres of molecules
c
      do 850 idom=1,ndomain
       nncs=nncsym(idom)
       u=suvw(1,idom)/suvw(4,idom)
       v=suvw(2,idom)/suvw(4,idom)
       w=suvw(3,idom)/suvw(4,idom)
       do 840 incs=1,nncs
        call symuvw(rsymncs(1,1,incs,idom), u, v, w,u1,v1,w1)
        call symuvw(q,                     u1,v1,w1,x1,y1,z1)
        uvwcen(1,incs,idom)=u1
        uvwcen(2,incs,idom)=v1
        uvwcen(3,incs,idom)=w1
        xyzcen(1,incs,idom)=x1
        xyzcen(2,incs,idom)=y1
        xyzcen(3,incs,idom)=z1
 840   continue
 850  continue
c
c now output some stats to see if it has worked:
c
      do 900 iw=0,nw-1
      do 900 iv=0,nv-1
      do 900 iu=0,nu-1
       sn(ncsmap(iu,iv,iw))=sn(ncsmap(iu,iv,iw))+1.0
 900  continue
c
      write (lpt,910)
 910  format (
     +/' NCS MASK INPUT AND INTERPOLATION:'
     +/' Index  Mask  Symm.  Frac.of Cell   Centre of Mass   ',
     + 'nu1 nu2 nv1 nv2 nw1 nw2'
     +/'                   (non-overlapped)  (fractional)')
      do 960 idom=1,ndomain
       nncs=nncsym(idom)
       do 940 incs=1,nncs
        write (lpt,920)incs,idom,incs,sn(incs+maxncs*idom)/(nu*nv*nw),
     +    (uvwcen(i,incs,idom),i=1,3),
     +    (nuvwlim(i,incs,idom),nuvwlim(i+3,incs,idom),i=1,3)
 920    format (1x,i3,4x,i4,2x,i4,4x,f8.4,3x,3f6.3,1x,6i4)
 940   continue
 960  continue
      write (lpt,980)sn(-1)/(nu*nv*nw)
 980  format (' Total overlap volume:',f8.4)
      write (lpt,990)msku1,msku2,mskv1,mskv2,mskw1,mskw2
 990  format (' Box containing whole monomer',23x,6i4/)
c
      if (ovlprm.eq.0) write (lpt,995)
 995  format (' WARNING: NO OVERLAP REMOVAL PERFORMED'/
     +  '  If you are using a multimer mask the NCS correlation'/
     +  '  statistics may be misleading. Ignore all values below the'/
     +  '  top line of the NCS AVERAGING CORRELATION matrix.'/)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine ncsmskout(ncsmsk)
      include 'dmheader.fh'
      include 'crystal.fh'
      include 'ncssym.fh'
      include 'uvwdata.fh'
c ncsmaskin - read in ncs mask for monomers and expand to multimer
      integer ncsmsk(*)
      integer lspgrp
c
      lspgrp = 1
      call ccpmskout('NCSOUT','ncsautomask from dm',ncsmsk,lspgrp,cell,
     +               nu,nv,nw,msku1,mskv1,mskw1,msku2,mskv2,mskw2)
c
      return
      end
c
c ----------------------------------------------------------------------
c
c interpolation routines
c
c ----------------------------------------------------------------------
c
      subroutine ncsplinewt(ih,ik,il,s,f)
      include 'dmheader.fh'
      include 'uvwdata.fh'
      include 'crystal.fh'
c ncSPLINEWT - apply reciprocal space weights to correct for B-spline
      integer ih,ik,il
      real s
      complex f
c
      real x1,x2,x3,wt
c
      x1=max(pi*abs(ih)/real(nu),0.0001)
      x2=max(pi*abs(ik)/real(nv),0.0001)
      x3=max(pi*abs(il)/real(nw),0.0001)
      wt=(sin(x1)*sin(x2)*sin(x3)/(x1*x2*x3))**3
      f=f/(wt*vol)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      real function bsplinterp(rho,x,y,z)
      include 'dmheader.fh'
      include 'uvwdata.fh'
c bsplinterp - Interpolate a density point by bspline convolution
c
      real rho(0:nu-1,0:nv-1,0:nw-1),x,y,z
c
      integer iu,iv,iw,iu1,iv1,iw1,iu2,iv2,iw2,iu3,iv3,iw3,lmod
      real u,v,w,du,dv,dw,bu1,bv1,bw1,bu2,bv2,bw2,bu3,bv3,bw3
      real r1,r2,r3,r11,r12,r13,r21,r22,r23,r31,r32,r33
      external lmod
c
      u=x*nu
      v=y*nv
      w=z*nw
      iu=nint(u)
      iv=nint(v)
      iw=nint(w)
      du=real(iu)-u
      dv=real(iv)-v
      dw=real(iw)-w
      iu2=lmod(iu,nu)
      iv2=lmod(iv,nv)
      iw2=lmod(iw,nw)
      iu1=lmod(iu2-1,nu)
      iv1=lmod(iv2-1,nv)
      iw1=lmod(iw2-1,nw)
      iu3=lmod(iu2+1,nu)
      iv3=lmod(iv2+1,nv)
      iw3=lmod(iw2+1,nw)
c
      bu1=0.5*(du+0.5)**2
      bu2=-du**2+0.75
      bu3=0.5*(du-0.5)**2
      bv1=0.5*(dv+0.5)**2
      bv2=-dv**2+0.75
      bv3=0.5*(dv-0.5)**2
      bw1=0.5*(dw+0.5)**2
      bw2=-dw**2+0.75
      bw3=0.5*(dw-0.5)**2
c
      r11=bw1*rho(iu1,iv1,iw1)+bw2*rho(iu1,iv1,iw2)+bw3*rho(iu1,iv1,iw3)
      r21=bw1*rho(iu2,iv1,iw1)+bw2*rho(iu2,iv1,iw2)+bw3*rho(iu2,iv1,iw3)
      r31=bw1*rho(iu3,iv1,iw1)+bw2*rho(iu3,iv1,iw2)+bw3*rho(iu3,iv1,iw3)
      r12=bw1*rho(iu1,iv2,iw1)+bw2*rho(iu1,iv2,iw2)+bw3*rho(iu1,iv2,iw3)
      r22=bw1*rho(iu2,iv2,iw1)+bw2*rho(iu2,iv2,iw2)+bw3*rho(iu2,iv2,iw3)
      r32=bw1*rho(iu3,iv2,iw1)+bw2*rho(iu3,iv2,iw2)+bw3*rho(iu3,iv2,iw3)
      r13=bw1*rho(iu1,iv3,iw1)+bw2*rho(iu1,iv3,iw2)+bw3*rho(iu1,iv3,iw3)
      r23=bw1*rho(iu2,iv3,iw1)+bw2*rho(iu2,iv3,iw2)+bw3*rho(iu2,iv3,iw3)
      r33=bw1*rho(iu3,iv3,iw1)+bw2*rho(iu3,iv3,iw2)+bw3*rho(iu3,iv3,iw3)
      r1=bv1*r11+bv2*r12+bv3*r13
      r2=bv1*r21+bv2*r22+bv3*r23
      r3=bv1*r31+bv2*r32+bv3*r33
      bsplinterp=bu1*r1+bu2*r2+bu3*r3
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine bsplintgrd(rho,x,y,z,r,g)
      include 'dmheader.fh'
      include 'uvwdata.fh'
      include 'crystal.fh'
c bsplinterp - Interpolate a density point by bspline convolution
c
      real rho(0:nu-1,0:nv-1,0:nw-1),x,y,z,r,g(3)
c
      integer iu,iv,iw,iu1,iv1,iw1,iu2,iv2,iw2,iu3,iv3,iw3,lmod
      real u,v,w,du,dv,dw,r1,r2,r3
      real bu1,bv1,bw1,bu2,bv2,bw2,bu3,bv3,bw3
      real gu1,gv1,gw1,gu2,gv2,gw2,gu3,gv3,gw3
      real r11,r12,r13,r21,r22,r23,r31,r32,r33
      external lmod
c
      u=x*nu
      v=y*nv
      w=z*nw
      iu=nint(u)
      iv=nint(v)
      iw=nint(w)
      du=real(iu)-u
      dv=real(iv)-v
      dw=real(iw)-w
      iu2=lmod(iu,nu)
      iv2=lmod(iv,nv)
      iw2=lmod(iw,nw)
      iu1=lmod(iu2-1,nu)
      iv1=lmod(iv2-1,nv)
      iw1=lmod(iw2-1,nw)
      iu3=lmod(iu2+1,nu)
      iv3=lmod(iv2+1,nv)
      iw3=lmod(iw2+1,nw)
c
      bu1=0.5*(du+0.5)**2
      bu2=-du**2+0.75
      bu3=0.5*(du-0.5)**2
      bv1=0.5*(dv+0.5)**2
      bv2=-dv**2+0.75
      bv3=0.5*(dv-0.5)**2
      bw1=0.5*(dw+0.5)**2
      bw2=-dw**2+0.75
      bw3=0.5*(dw-0.5)**2
      gu1=-du-0.5
      gu2=2.0*du
      gu3=-du+0.5
      gv1=-dv-0.5
      gv2=2.0*dv
      gv3=-dv+0.5
      gw1=-dw-0.5
      gw2=2.0*dw
      gw3=-dw+0.5
c
      r11=bw1*rho(iu1,iv1,iw1)+bw2*rho(iu1,iv1,iw2)+bw3*rho(iu1,iv1,iw3)
      r21=bw1*rho(iu2,iv1,iw1)+bw2*rho(iu2,iv1,iw2)+bw3*rho(iu2,iv1,iw3)
      r31=bw1*rho(iu3,iv1,iw1)+bw2*rho(iu3,iv1,iw2)+bw3*rho(iu3,iv1,iw3)
      r12=bw1*rho(iu1,iv2,iw1)+bw2*rho(iu1,iv2,iw2)+bw3*rho(iu1,iv2,iw3)
      r22=bw1*rho(iu2,iv2,iw1)+bw2*rho(iu2,iv2,iw2)+bw3*rho(iu2,iv2,iw3)
      r32=bw1*rho(iu3,iv2,iw1)+bw2*rho(iu3,iv2,iw2)+bw3*rho(iu3,iv2,iw3)
      r13=bw1*rho(iu1,iv3,iw1)+bw2*rho(iu1,iv3,iw2)+bw3*rho(iu1,iv3,iw3)
      r23=bw1*rho(iu2,iv3,iw1)+bw2*rho(iu2,iv3,iw2)+bw3*rho(iu2,iv3,iw3)
      r33=bw1*rho(iu3,iv3,iw1)+bw2*rho(iu3,iv3,iw2)+bw3*rho(iu3,iv3,iw3)
      r1=bv1*r11+bv2*r12+bv3*r13
      r2=bv1*r21+bv2*r22+bv3*r23
      r3=bv1*r31+bv2*r32+bv3*r33
      r=bu1*r1+bu2*r2+bu3*r3
c
      g(1)=gu1*r1+gu2*r2+gu3*r3
      r1=gv1*r11+gv2*r12+gv3*r13
      r2=gv1*r21+gv2*r22+gv3*r23
      r3=gv1*r31+gv2*r32+gv3*r33
      g(2)=bu1*r1+bu2*r2+bu3*r3
      r11=gw1*rho(iu1,iv1,iw1)+gw2*rho(iu1,iv1,iw2)+gw3*rho(iu1,iv1,iw3)
      r21=gw1*rho(iu2,iv1,iw1)+gw2*rho(iu2,iv1,iw2)+gw3*rho(iu2,iv1,iw3)
      r31=gw1*rho(iu3,iv1,iw1)+gw2*rho(iu3,iv1,iw2)+gw3*rho(iu3,iv1,iw3)
      r12=gw1*rho(iu1,iv2,iw1)+gw2*rho(iu1,iv2,iw2)+gw3*rho(iu1,iv2,iw3)
      r22=gw1*rho(iu2,iv2,iw1)+gw2*rho(iu2,iv2,iw2)+gw3*rho(iu2,iv2,iw3)
      r32=gw1*rho(iu3,iv2,iw1)+gw2*rho(iu3,iv2,iw2)+gw3*rho(iu3,iv2,iw3)
      r13=gw1*rho(iu1,iv3,iw1)+gw2*rho(iu1,iv3,iw2)+gw3*rho(iu1,iv3,iw3)
      r23=gw1*rho(iu2,iv3,iw1)+gw2*rho(iu2,iv3,iw2)+gw3*rho(iu2,iv3,iw3)
      r33=gw1*rho(iu3,iv3,iw1)+gw2*rho(iu3,iv3,iw2)+gw3*rho(iu3,iv3,iw3)
      r1=bv1*r11+bv2*r12+bv3*r13
      r2=bv1*r21+bv2*r22+bv3*r23
      r3=bv1*r31+bv2*r32+bv3*r33
      g(3)=bu1*r1+bu2*r2+bu3*r3
c
      return
      end
c      g(1)=(sxrho/sw)*(nu/q(1,1))
c      g(2)=(syrho/sw)*(nv/q(2,2))
c      g(3)=(szrho/sw)*(nw/q(3,3))
c
c ----------------------------------------------------------------------
c
