C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
c
c ----------------------------------------------------------------------
c
      subroutine getfreemask(solmsk,mask)
c Locate good free spheres in protein and solvent
      include 'dmheader.fh'
      include 'io.fh'
      include 'crystal.fh'
      include 'params.fh'
      include 'uvwdata.fh'
      integer solmsk(0:nu-1,0:nv-1,0:nw-1)
      real    mask(0:nu-1,0:nv-1,0:nw-1)
c
      real frprad,frsrad
      parameter (frprad=5.0**2,frsrad=5.0**2)
c
      integer i,m,iu,iv,iw,pu,pv,pw,su,sv,sw,lmod
c
      external lmod
c
c set up the mask on -0.5..0.5
      do 100 iw=0,nw-1
      do 100 iv=0,nv-1
      do 100 iu=0,nu-1
       if (solmsk(iu,iv,iw).ge.1) then
        mask(iu,iv,iw)=0.5
       else
        mask(iu,iv,iw)=-0.5
       endif
 100  continue
c
c now mark with approx. dist from the solvent boundary
      call markdist(mask)
c
c find most distant points
      pu=0
      pv=0
      pw=0
      su=0
      sv=0
      sw=0
      do 200 iw=0,nw-1
      do 200 iv=0,nv-1
      do 200 iu=0,nu-1
       if (mask(iu,iv,iw).gt.mask(pu,pv,pw)) then
        pu=iu
        pv=iv
        pw=iw
       endif
       if (mask(iu,iv,iw).lt.mask(su,sv,sw)) then
        su=iu
        sv=iv
        sw=iw
       endif
 200  continue
c
      if (frpx.lt.-0.9e6) then
       call frac2orth
     +   (real(pu)/nu,real(pv)/nv,real(pw)/nw,frpx,frpy,frpz)
       write (lpt,210)'protein',frpx,frpy,frpz,mask(pu,pv,pw)
      endif
      if (frsx.lt.-0.9e6) then
       call frac2orth
     +   (real(su)/nu,real(sv)/nv,real(sw)/nw,frsx,frsy,frsz)
       write (lpt,210)'solvent',frsx,frsy,frsz,mask(su,sv,sw)
      endif
 210  format (/
     +  ' REAL/FREE ',a7,' sphere at ',3f8.3,' remoteness ',f8.3)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine markdist(mask)
      include 'dmheader.fh'
      include 'crystal.fh'
      include 'params.fh'
      include 'uvwdata.fh'
c markdist - mark up the mask, marking each point with the (approximate)
c  distance to the nearest mask boundary. Max error ~ 15% (cubic)
c  Distances are in approximate grid points.
      real mask(0:nu-1,0:nv-1,0:nw-1)
c
      real olddif,newdif,maxht,minht,uphill,dnhill,dist,distm,cut
      real u,v,w,distab(-1:1,-1:1,-1:1)
      integer iu,iv,iw,ju,jv,jw,ku,kv,kw,lmod
c
      external lmod
c
c make table of orthogonal pixel distances to nearby pixels
      distm=1.0e12
      distab(0,0,0)=1.0e12
      do 100 jw=-1,1
      w=real(jw)/nw
      do 100 jv=-1,1
      v=real(jv)/nv
      do 100 ju=-1,1
      u=real(ju)/nu
       if (ju.ne.0.or.jv.ne.0.or.jw.ne.0)
     +  distab(ju,jv,jw)=sqrt(rlm11*u**2+rlm22*v**2+rlm33*w**2+
     +                        rlm12*u*v +rlm13*u*w +rlm23*v*w )
       distm=min(distm,distab(ju,jv,jw))
 100  continue
c
c and mark up - iterative refinement
      cut=0.0
      newdif=0.0
 150   olddif=newdif+0.001
       maxht=0.0
       minht=0.0
c loop to increase stuff inside mask
       do 200 iw=0,nw-1
       do 200 iv=0,nv-1
       do 200 iu=0,nu-1
c only treat points inside mask (to speed things up)
        if (abs(mask(iu,iv,iw)).gt.cut) then
         uphill= 1.0e12
         dnhill=-1.0e12
         do 180 jw=-1,1
         kw=lmod(iw+jw,nw)
         do 180 jv=-1,1
         kv=lmod(iv+jv,nv)
         do 180 ju=-1,1
         ku=lmod(iu+ju,nu)
          dist=distab(ju,jv,jw)
c mark with highest/lowest neighbour+distance
          uphill=min(uphill,mask(ku,kv,kw)+dist)
          dnhill=max(dnhill,mask(ku,kv,kw)-dist)
 180     continue
         if (mask(iu,iv,iw).gt.0.0) then
          mask(iu,iv,iw)=uphill
         else
          mask(iu,iv,iw)=dnhill
         endif
        endif
        maxht=max(maxht,mask(iu,iv,iw))
        minht=min(minht,mask(iu,iv,iw))
 200   continue
c we can increase the cutoff to speed things up
       cut=cut+distm
       newdif=maxht-minht
      if (newdif.gt.olddif) goto 150
      return
      end
c
c ----------------------------------------------------------------------
c
      logical function setfreemask(solmsk)
      include 'dmheader.fh'
      include 'io.fh'
      include 'crystal.fh'
      include 'params.fh'
      include 'uvwdata.fh'
      integer solmsk(0:nu-1,0:nv-1,0:nw-1)
c
      real frprad,frsrad
      parameter (frprad=5.0**2,frsrad=5.0**2)
c
      integer m,iu,iv,iw,lmod
      real u1,v1,w1,du,dv,dw,fpu,fpv,fpw,fsu,fsv,fsw,r2
      real s,p,sf,pf,rfracp
c
      external rfracp,lmod
c
c make spheres
      call orth2frac(frpx,frpy,frpz,fpu,fpv,fpw)
      call orth2frac(frsx,frsy,frsz,fsu,fsv,fsw)
      s=0.1
      p=0.1
      sf=0.1
      pf=0.1
      do 300 iw=0,nw-1
      do 300 iv=0,nv-1
      do 300 iu=0,nu-1
       dw=rfracp(real(iw)/nw-fpw)
       dv=rfracp(real(iv)/nv-fpv)
       du=rfracp(real(iu)/nu-fpu)
       r2=rlm11*du**2+rlm22*dv**2+rlm33*dw**2+
     +    rlm12*du*dv+rlm13*du*dw+rlm23*dv*dw
       if (r2.lt.frpr**2) then
        p=p+1.0
        if (solmsk(iu,iv,iw).eq.1) then
         solmsk(iu,iv,iw)=10
         pf=pf+1.0
        endif
       endif
       dw=rfracp(real(iw)/nw-fsw)
       dv=rfracp(real(iv)/nv-fsv)
       du=rfracp(real(iu)/nu-fsu)
       r2=rlm11*du**2+rlm22*dv**2+rlm33*dw**2+
     +    rlm12*du*dv+rlm13*du*dw+rlm23*dv*dw
       if (r2.lt.frsr**2) then
        s=s+1.0
        if (solmsk(iu,iv,iw).eq.0) then
         solmsk(iu,iv,iw)=-10
         sf=sf+1.0
        endif
       endif
 300  continue
c
c symmetry expand
      do 310 iw=0,nw-1
      do 310 iv=0,nv-1
      do 310 iu=0,nu-1
       if (abs(solmsk(iu,iv,iw)).eq.10) then
        do 305 m=2,nsym
         call symuvw(rsym(1,1,m),
     +               real(iu)/nu,real(iv)/nv,real(iw)/nw,u1,v1,w1)
         solmsk(lmod(nint(nu*u1),nu),
     +          lmod(nint(nv*v1),nv),
     +          lmod(nint(nw*w1),nw))=solmsk(iu,iv,iw)
 305    continue
       endif
 310  continue
c
      sf=sf/s
      pf=pf/p
      write (lpt,350)sf,pf
 350  format(/' REAL/FREE-R MASKS:'/
     +  '  ',f5.2,' of free solvent sphere lies in solvent'/
     +  '  ',f5.2,' of free protein sphere lies in protein'/)
c
c Check for mask changing beyond free spheres
c
      setfreemask=(pf.gt.0.75.and.sf.gt.0.75)
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine resetfreemask(solmsk)
      include 'dmheader.fh'
      include 'uvwdata.fh'
      integer solmsk(*)
      integer i
      do 100 i=1,nu*nv*nw
       if (solmsk(i).eq.-10) then
        solmsk(i)=0
       else if (solmsk(i).eq.10 ) then
        solmsk(i)=1
       endif
 100  continue
      return
      end
c
c ----------------------------------------------------------------------
c
      logical function freestat(rhomap,rhomod,solmsk,reseff,realfree)
      include 'dmheader.fh'
      include 'params.fh'
      include 'uvwdata.fh'
      include 'cycl.fh'
      include 'io.fh'
      include 'output.fh'
      real rhomap(*),rhomod(*),reseff,realfree
      integer solmsk(*)
      integer i
      real r1,r2,r3,r4,r5,r6,c1,c2,c3,c4,r12,r22,r32,r42
      logical setfreemask,dum
      real flagstd,flaghm
      external setfreemask,flagstd,flaghm
c
c set flags:
      if (setfreemask(solmsk)) goto 150
      write (lpt,90)
 90   format(' WARNING: Solvent mask has changed significantly.',/,
     +  ' Recalculating free spheres. Real-free-resid may jump')
      call resetfreemask(solmsk)
      realfree=1.0e6
      frpx=-1.0e6
      frsx=-1.0e6
      call getfreemask(solmsk,rhomod)
      if (setfreemask(solmsk)) goto 150
c failed to get a new freemask
      call ccperr(1,
     +  ' FREESTAT - not enough solv/prot for REALFREE or NCYC AUTO')
c
c make map
 150  continue
c       call resetfreemask(solmsk)
c       call copy3d(rhomap,rhomod)
c c .. solvent flattening
c       call solvflat(rhomod,solmsk)
c c .. histogram mapping
c       call histmap(rhomod,solmsk,reseff,.false.)
c c stats
c       dum=setfreemask(solmsk)
c       c1=0.1
c       c2=0.1
c       c3=0.1
c       c4=0.1
c       r1=0.0
c       r2=0.0
c       r3=0.0
c       r4=0.0
c       r12=0.0
c       r22=0.0
c       r32=0.0
c       r42=0.0
c       do 200 i=1,nu*nv*nw
c        if (solmsk(i).eq.0) then
c         c1=c1+1.0
c         r1 =r1 +(rhomap(i)-rhomod(i))
c         r12=r12+(rhomap(i)-rhomod(i))**2
c        else if (solmsk(i).eq.-10) then
c         c2=c2+1.0
c         r2 =r2 +(rhomap(i)-rhomod(i))
c         r22=r22+(rhomap(i)-rhomod(i))**2
c        else if (solmsk(i).eq.1) then
c         c3=c3+1.0
c         r3 =r3 +(rhomap(i)-rhomod(i))
c         r32=r32+(rhomap(i)-rhomod(i))**2
c        else if (solmsk(i).eq.10) then
c         c4=c4+1.0
c         r4 =r4 +(rhomap(i)-rhomod(i))
c         r42=r42+(rhomap(i)-rhomod(i))**2
c        endif
c  200  continue
c       call resetfreemask(solmsk)
c c
c       r1=sqrt(c1*r12-r1**2)/c1
c       r2=sqrt(c2*r22-r2**2)/c2
c       r3=sqrt(c3*r32-r3**2)/c3
c       r4=sqrt(c4*r42-r4**2)/c4
c       r5=solc*r1+(1.0-solc)*r3
c       r6=solc*r2+(1.0-solc)*r4
c       write (lpt,991)r1,r2,r3,r4,r5,r6
c  991  format(
c      +  /'*************************************************'
c      +  /'* Solvent Resid : ',f6.3,' FreeSol Resid : ',f6.3,' *'
c      +  /'* Protein Resid : ',f6.3,' FreePrt Resid : ',f6.3,' *'
c      +  /'* Combine Resid : ',f6.3,' FreeCmb Resid : ',f6.3,' *'
c      +  /'*************************************************'/)
c
      r2=0.0
      r4=0.0
c
      r1=flagstd(rhomap,solmsk,0)
      if (frsr.gt.0.0) r2=flagstd(rhomap,solmsk,-10)
      r3=flaghm(rhomap,solmsk,1,reseff)
      if (frpr.gt.0.0) r4=flaghm(rhomap,solmsk,10,reseff)
      r5=solc*r1+(1.0-solc)*r3
      r6=solc*r2+(1.0-solc)*r4
      call resetfreemask(solmsk)
      write (*,991)r1,r2,r3,r4,r5,r6
 991  format(
     +  /'*************************************************'
     +  /'* Solvent Resid : ',f6.3,' FreeSol Resid : ',f6.3,' *'
     +  /'* Protein Resid : ',f6.3,' FreePrt Resid : ',f6.3,' *'
     +  /'* Combine Resid : ',f6.3,' FreeCmb Resid : ',f6.3,' *'
     +  /'*************************************************'/)
      realr(icycl)=r6
c
c set termination condition
      freestat=(r6.gt.realfree)
c update real free r
      realfree=r6
      return
      end
c
c ----------------------------------------------------------------------
c
c       subroutine freestat(rhomap,solmsk,reseff,realfree)
c       include 'dmheader.fh'
c       include 'params.fh'
c       include 'cycl.fh'
c       include 'output.fh'
c       real rhomap(*),reseff,realfree
c       integer solmsk(*)
c       real r1,r2,r3,r4,r5,r6,flagstd,flaghm
c       external flagstd,flaghm
c c
c       r2=0.0
c       r4=0.0
c c
c       r1=flagstd(rhomap,solmsk,0)
c       if (frsr.gt.0.0) r2=flagstd(rhomap,solmsk,-10)
c       r3=flaghm(rhomap,solmsk,1,reseff)
c       if (frpr.gt.0.0) r4=flaghm(rhomap,solmsk,10,reseff)
c       r5=solc*r1+(1.0-solc)*r3
c       r6=solc*r2+(1.0-solc)*r4
c       write (*,991)r1,r2,r3,r4,r5,r6
c  991  format(
c      +  /'*************************************************'
c      +  /'* Solvent Resid : ',f6.3,' FreeSol Resid : ',f6.3,' *'
c      +  /'* Protein Resid : ',f6.3,' FreePrt Resid : ',f6.3,' *'
c      +  /'* Combine Resid : ',f6.3,' FreeCmb Resid : ',f6.3,' *'
c      +  /'*************************************************'/)
c c
c       realr(icycl)=r6
c       realfree=r6
c c
c       return
c       end
c
c ----------------------------------------------------------------------
c
      real function flagstd(map,mask,flag)
      include 'dmheader.fh'
      include 'uvwdata.fh'
c
      real map(nu,nv,nw)
      integer mask(nu,nv,nw),flag
c
      real s,sr,sr2
      integer iu,iv,iw,i
c
      s=0.0
      sr=0.0
      sr2=0.0
      do 100 iw=1,nw
      do 100 iv=1,nv
      do 100 iu=1,nu
       i=mask(iu,iv,iw)
       if (i.eq.flag) then
        s=s+1.0
        sr=sr+map(iu,iv,iw)
        sr2=sr2+map(iu,iv,iw)**2
       endif
 100  continue
c
      flagstd=sqrt(sr2*s-sr**2)/s
c
      return
      end
c
c ----------------------------------------------------------------------
c
      real function flaghm(map,solmsk,flag,hmres)
      include 'dmheader.fh'
      include 'io.fh'
      include 'crystal.fh'
      include 'params.fh'
      include 'uvwdata.fh'
c HISTMAP - perform histogram mapping
c
      real map(nu,nv,nw),hmres
      integer solmsk(nu,nv,nw),flag
c
      real hresl,hrmin,hrmax,hrave,hdata
      common /histograms/hresl(50),hrmin(50),hrmax(50),hrave(50),
     +  hdata(200,50),nresl,nhist
      save   /histograms/
c
      real hobs(200),hcal(200),hmod(200),cobs(200),ccal(200),rtran(200)
      integer i,j,k,nr,nrh,nrm,jres,ih,ihm,nresl,nhist
      real hmres1,sr,sr2,srh,srm,srh2,srm2,dj,r,dhr,dr,robs,rcal,rh,dh
      real rhomax,rhomin,hrhomax,hrhomin,hrav,hrst,hmav,hmst
c
      hmres1=hmres
      nr=0
      nrh=0
      nrm=0
      sr=0.0
      srh=0.0
      srm=0.0
      sr2=0.0
      srh2=0.0
      srm2=0.0
      rhomax=-1000.0
      rhomin=1000.0
      do 100 i=1,nhist
      hobs(i)=0.0
      hmod(i)=0.0
 100  continue
c pick the best resolution ranges for the Fcalc histograms
      do 150 jres=1,nresl
      if (hresl(jres).lt.hmres1) goto 160
 150  continue
 160  if (jres.eq.1) then
      jres=2
      dj=0.0
      else if (jres.eq.nresl+1) then
      jres=nresl
      dj=1.0
      else
      dj=(hmres1-hresl(jres-1))/(hresl(jres)-hresl(jres-1))
      endif
c make interpolated histogram from two calculated histograms
      do 320 i=1,nhist
      rcal=(i-0.5)*(hrmax(jres)-hrmin(jres))/(nhist-1)+hrmin(jres)
      hcal(i)=(1.0-dj)*hdata(i,jres-1)+dj*hdata(i,jres)
      nrh=nrh+hcal(i)
      srh=srh+hcal(i)*rcal
      srh2=srh2+hcal(i)*rcal**2
 320  continue
c now calc the scale of the Fcalc histogram
      hrav=srh/nrh
      hrst=sqrt(srh2*nrh-srh**2)/nrh
      hrhomax=hrmax(jres)
cc *aveprt/hrav
      hrhomin=hrmin(jres)
cc *aveprt/hrav
      dhr=(hrhomax-hrhomin)/(nhist-1)
c compile histogram stats for observed data
      do 200 k=1,nw
      do 200 j=1,nv
      do 200 i=1,nu
      if (solmsk(i,j,k).ne.flag) goto 200
      nr=nr+1
      sr=sr+map(i,j,k)
      sr2=sr2+map(i,j,k)**2
      rhomax=max(rhomax,map(i,j,k))
      rhomin=min(rhomin,map(i,j,k))
 200  continue
      dr=(rhomax-rhomin)/(nhist-1)
      do 300 k=1,nw
      do 300 j=1,nv
      do 300 i=1,nu
      if (solmsk(i,j,k).ne.flag) goto 300
      ih=int((map(i,j,k)-rhomin)/dr)+1
      hobs(ih)=hobs(ih)+1.0
 300  continue
c put on a scale of fractions of the unit cell
      do 360 i=1,nhist
      hobs(i)=hobs(i)/nr
      hcal(i)=hcal(i)/nrh
 360  continue
c generate cumulative density distributions
      cobs(1)=0.0
      ccal(1)=0.0
      cobs(nhist)=1.0
      ccal(nhist)=1.0
      do 400 i=2,nhist-1
      cobs(i)=cobs(i-1)+hobs(i-1)
      ccal(i)=ccal(i-1)+hcal(i-1)
 400  continue
c generate the density transformation list
      do 500 i=1,nhist
      robs=(i-1)*dr+rhomin
      do 480 j=2,nhist-1
      if (ccal(j).ge.cobs(i)) goto 490
 480  continue
 490  rcal=(j-2+(cobs(i)-ccal(j-1))/max(ccal(j)-ccal(j-1),1.0e-6))
     +  *dhr+hrhomin
      rtran(i)=rcal
 500  continue
c now do shift calculations
      do 600 k=1,nw
      do 600 j=1,nv
      do 600 i=1,nu
      if (solmsk(i,j,k).ne.flag) goto 600
      robs=map(i,j,k)
      rh=(robs-rhomin)/dr+1.0
      ih=int(rh)
      dh=rh-ih
      ih=max(min(ih,nhist-1),1)
      rcal=rtran(ih)+dh*(rtran(ih+1)-rtran(ih))
      nrm=nrm+1
      srm=srm+(rcal-map(i,j,k))
      srm2=srm2+(rcal-map(i,j,k))**2
 600  continue
      hmav=srm/nrm
      hmst=sqrt(srm2*nrm-srm**2)/nrm
c
      flaghm=hmst
      return
      end
c
c ----------------------------------------------------------------------
c
