C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
c
c ----------------------------------------------------------------------
c
c iterative skeletonisation
c
c ----------------------------------------------------------------------
c
      subroutine itskel(hkls,bigmap,rhomap,solmsk,reseff)
      include 'dmheader.fh'
      include 'params.fh'
      include 'hkldata.fh'
      include 'uvwdata.fh'
      include 'io.fh'
c itskel - iterative skeletonisation using core tracing
c NOTE: bigmap must be at least 4 x map in size (overrunning other maps)
c
      integer solmsk(*)
      real hkls(nhklc,0:*),bigmap(*),rhomap(*),reseff
c
      integer lfa,lfb
      parameter (lfa=20,lfb=21)
c
      integer fuvw(3,maxfeature),joinptr(0:maxjoin),jointo(0:maxjoin)
      integer i,j,nus,nvs,nws,startptr,nabove(0:levels),njoin,nfeature
      real r(0:6),s
c
c
      write (lpt,10)
 10   format(/' ITERATIVE SKELETONISATION')
c
c need a BIG grid!
      nus=0
      nvs=0
      nws=0
      call gridres(0.67*rmax,nus,nvs,nws)
c
c Un-sharpen the data
      do 100 i=1,nrflo
       s=exp(-sklbf*hkls(4,i)/4.0)
       hkls(lfa,i)=s*hkls(lfa,i)
       hkls(lfb,i)=s*hkls(lfb,i)
 100  continue

c do an BIG fft (faster than doing a small fft and interpolating!)
      call fftlistmapn(bigmap,hkls,lfa,lfb,nus,nvs,nws)
c
      call qmapsort(bigmap,nus,nvs,nws,startptr,nabove)
c
      call coretrace(bigmap,nus,nvs,nws,startptr,nabove,
     +  fuvw,joinptr,jointo,nfeature)
c
      call ridgelevel
     +  (bigmap,nus,nvs,nws,fuvw,joinptr,jointo,nfeature)
c
c build the density in the real array 'rhomap'
      call buildden(bigmap,nus,nvs,nws,rhomap,solmsk,reseff)
c
      do 200 i=1,nu*nv*nw
       if (solmsk(i).eq.0) then
        bigmap(i)=avesol
       else
        bigmap(i)=rhomap(i)
       endif
 200  continue
c
      return
      end
c      call ccpdpn(18,'QSKL','UNKNOWN','U',0,ifail)
c       call frac2orth(u1,v1,w1,x1,y1,z1)
c       call frac2orth(u2,v2,w2,x2,y2,z2)
c       write (18)2.0,x1,y1,z1
c       write (18)3.0,x2,y2,z2
c      close (unit=18,status='keep')
c
c ----------------------------------------------------------------------
c
c FAST MAP SORT
c
c ----------------------------------------------------------------------
c
      subroutine qmapsort(bigmap,nus,nvs,nws,startptr,nabove)
      include 'dmheader.fh'
      include 'params.fh'
      include 'skldat.fh'
c qmapsort - sort a map in order of density: fast and approximate
c   in  : bigmap = real map
c   out : bigmap = map of pointers, starting with startptr
      integer bigmap(0:*),nus,nvs,nws,startptr,nabove(0:*)
c
      integer binfirst(0:levels),binlast(0:levels)
      integer i,j,nmap,irho,ipt
      real rho,sr,sr2,rhocut
      real realval
c
      external realval
c
      nmap=nus*nvs*nws
      do 50 i=0,levels
       binfirst(i)=-1
       binlast(i)=-1
       nabove(i)=0
 50   continue
c
c find the limits of the initial density
      rhomin= 1000.0
      rhomax=-1000.0
      sr=0.0
      sr2=0.0
      do 100 i=0,nmap-1
       rho=realval(bigmap(i))
       sr=sr+rho
       sr2=sr2+rho**2
       rhomin=min(rhomin,rho)
       rhomax=max(rhomax,rho)
 100  continue
      rhoave=sr/nmap
      rhostd=sqrt(sr2*nmap-sr**2)/nmap
c
c NOW TURN THE MAP INTO AN APPROXIMATELY ORDERED LINKED LIST OF DENSITY VALUES
c
c now we turn the map into linked lists of 'equal' density values
      do 200 i=0,nmap-1
       rho=realval(bigmap(i))
       irho=int(levels*(rhomax-rho)/(rhomax-rhomin))
       irho=min(max(irho,0),levels)
       nabove(irho)=nabove(irho)+1
       ipt=binlast(irho)
       if (ipt.lt.0) then
c first in bin: first and last pointers point to it
        binfirst(irho)=i
       else
c add to bin: last point and last point pointer point to it
        bigmap(ipt)=i
       endif
c either way, this point is now a chain end (marker is redundant).
       binlast(irho)=i
 200  continue
c
c fill in links from one density level to the next to make one huge list
c (assume bin(0) cannot be empty)
      do 300 i=1,levels
       if (binlast(i).lt.0) then
c if this bin is empty propogate end of last chain
        binlast(i)=binlast(i-1)
       else
c if not, then point end of last chain to beginning of this one
        bigmap(binlast(i-1))=binfirst(i)
       endif
 300  continue
c end flag!!
      bigmap(binlast(i-1))=100000000
      startptr=binfirst(0)
c
c make a list of how many points above different sigma levels
c
      do 400 i=1,levels
       nabove(i)=nabove(i)+nabove(i-1)
 400  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      real function realval(val)
      include 'dmheader.fh'
c realval - get a real from a variable 
c (presumably one that is not declared as real in the calling routine)
      real val
      realval=val
      return
      end
c
c ----------------------------------------------------------------------
c
c skeletonisation
c
c ----------------------------------------------------------------------
c
      subroutine coretrace(sklmap,nus,nvs,nws,startptr,nabove,
     +                     fuvw,joinptr,jointo,nfeature)
      include 'dmheader.fh'
      include 'crystal.fh'
      include 'params.fh'
      include 'skldat.fh'
c Build a skelton by the core tracing method of Stanley Swanson
c (Acta Cryst D50)
      integer nus,nvs,nws,sklmap(0:*),startptr,nfeature
      integer fuvw(3,*),joinptr(0:*),jointo(0:*),nabove(0:*)
c
      integer iu,iv,iw,ju,jv,jw,nuvs,nuvws
      integer ipoint,npoint,npeak,npass,njoin,siglvl
      integer i,j,ifeature,jfeature,ptr,thispt,nextpt,nearpt
      integer envftr(26),envgen(26),nenv,ftr,gen,ilevel,rlevel
      integer lmod
      real du,dv,dw,thisd2,neard2,d2core
      real u1,v1,w1,u2,v2,w2,x1,y1,z1,x2,y2,z2,rhocut
      logical dojoin,mapfeat,arejoined
c
      external mapfeat,arejoined,lmod
c
c distance^2 for nearest neighbours: 2.5->18 neighbours
      parameter (d2core=2.5)
c
c This subroutine uses a linked list to store features and their links.
c Features are peaks, which appear in the map, and joins, which do not.
c The first (maxfeature) elements of this list are list headers, each
c corresponding to the feature of the same number. The corresponding value
c contains the feature number (-ve for joins), and a pointer to its
c links.
c During the core trace, peaks are linked with neigbouring peaks whenever
c a join is created; the join is listed with the peaks it connects. Thus:
c  1. Peaks are linked only to other peaks
c  2. Joins are linked only to peaks
c This is so that it can be quickly determined whether peaks are joined.
c After the core trace, the list can be re-written, so that peaks are
c linked to joins and joins are linked to peaks.
c
      nuvs =nus*nvs
      nuvws=nuvs*nws
c
c Set up list of features and linked list of joins
      call linkedlist(joinptr,maxjoin)
      do 10 i=1,maxfeature
       call inserthead(joinptr,ptr)
       jointo(ptr)=0
 10   continue
c
c set up a list of neigbours (calc on arbitrary 1.0A scale)
      ndn=0
      do 20 iw=-1,1
      do 20 iv=-1,1
      do 20 iu=-1,1
       du=real(iu)/cell(1)
       dv=real(iv)/cell(2)
       dw=real(iw)/cell(3)
       thisd2=rlm11*du**2+rlm22*dv**2+rlm33*dw**2+
     +        rlm12*du*dv+rlm13*du*dw+rlm23*dv*dw
       if (thisd2.gt.0.5.and.thisd2.lt.d2core) then
        ndn=ndn+1
        ndu(ndn)=iu
        ndv(ndn)=iv
        ndw(ndn)=iw
       endif
 20   continue
      write (*,*)' neighbourhood ',ndn
c
c NOW DO THE CORE TRACE
c
      thispt=startptr
      nfeature=0
      npeak=0
      npass=0
      njoin=0
c note: siglvl can be used to speed things up by ignoring low density.
c It is not so used here.
      siglvl=1000000
      ilevel=0
c
c CORE TRACING:
c
c We consider each point in turn, in order of density value.
c The map has been turned into a linked list of pointers, so this
c is easy, although it is harder to get back to the coordinates.
c
c get each point in order
      do 250 ipoint=1,nuvws
       if (thispt.ge.nuvws)
     +   call ccperr(1,' coretrace - internal error: email Kevin')
       nextpt=sklmap(thispt)
c work out density level
 100   if (ipoint.gt.nabove(ilevel)) then
        ilevel=ilevel+1
        goto 100
       endif
       rlevel=rhomax-ilevel*(rhomax-rhomin)/levels
c check we have some feature space left - if not then flag the map and skip
       if (nfeature.ge.maxfeature) then
        call setmapfeat(sklmap(thispt),maxfeature+1,levels+1)
       else
c calculate its coordinates
        iw=thispt/nuvs
        iv=mod(thispt/nus,nvs)
        iu=mod(thispt,nus)
c clear the environment list
        nenv=0
c
c Now we are going to make a list of the values of neighbouring cells.
c Pointers (cells not yet visited, +ve) are ignored.
c
c now loop through points and offsets
        do 130 j=1,ndn
         ju=lmod(iu+ndu(j),nus)
         jv=lmod(iv+ndv(j),nvs)
         jw=lmod(iw+ndw(j),nws)
         nearpt=sklmap(ju+nus*(jv+nvs*jw))
c if found a new feature marker add it to the list
         if (mapfeat(nearpt,ftr,gen)) then
          do 110 i=1,nenv
           if (ftr.eq.envftr(i)) then
            envgen(i)=max(envgen(i),gen)
            goto 120
           endif
 110      continue
          nenv=nenv+1
          envftr(nenv)=ftr
          envgen(nenv)=gen
 120      continue
         endif
 130    continue
c
c Now we deal with the point:
c Points with no neigbours are peaks, points with one type of neighbour
c add to that peaks, points with two or more neighbours are joins
c unless any of the neighbours are already joined.
c
c if it has no neighbours, it is a peak and gets a new number
        if (nenv.eq.0) then
         nfeature=nfeature+1
         npeak=npeak+1
         call setmapfeat(sklmap(thispt),nfeature,ilevel)
         fuvw(1,nfeature)=iu
         fuvw(2,nfeature)=iv
         fuvw(3,nfeature)=iw
         jointo(nfeature)=nfeature
        else if (nenv.eq.1) then
c if there is exactly one neigbour, then it adds to that peak
         call setmapfeat(sklmap(thispt),envftr(1),
     +     max(ilevel,envgen(1)+1) )
        else
c two or more neigbours: connect to the nearest peak
         nearpt=0
         neard2=1.0e12
         do 150 i=1,nenv
          ptr=envftr(i)
          du=real(iu-fuvw(1,ptr))/nus
          dv=real(iv-fuvw(2,ptr))/nvs
          dw=real(iw-fuvw(3,ptr))/nws
          thisd2=rlm11*du**2+rlm22*dv**2+rlm33*dw**2+
     +           rlm12*du*dv+rlm13*du*dw+rlm23*dv*dw
          if (thisd2.lt.neard2) then
           neard2=thisd2
           nearpt=i
          endif
 150     continue
         call setmapfeat(sklmap(thispt),envftr(nearpt),
     +     max(ilevel,envgen(nearpt)+1) )
c consider whether to join the peaks
         dojoin=.true.
         do 170 i=1,nenv-1
          ifeature=envftr(i)
          do 160 j=i+1,nenv
           jfeature=envftr(j)
           if (arejoined(ifeature,jfeature,joinptr,jointo)) goto 210
 160      continue
 170     continue
c .. if this is a new join, join two peaks together
c ... add a non-map feature
          nfeature=nfeature+1
          npass=npass+1
          fuvw(1,nfeature)=iu
          fuvw(2,nfeature)=iv
          fuvw(3,nfeature)=iw
          jointo(nfeature)=-nfeature
c ... and store links from join point
          ptr=nfeature
          do 180 i=1,nenv
           call insertafter(joinptr,ptr)
           jointo(ptr)=envftr(i)
           njoin=njoin+1
 180      continue
c ... and join peaks
          do 200 i=1,nenv-1
           ifeature=envftr(i)
           do 190 j=i+1,nenv
            jfeature=envftr(j)
            call joinfeat(ifeature,jfeature,joinptr,jointo)
 190       continue
 200      continue
c .. if we find a join, we skip to here
 210     continue
c done the case of two or more peaks
        endif
       endif
       thispt=nextpt
 250  continue
c
 260  write (*,*)' peaks ',npeak,' passes ',npass,' joins ',njoin
      write (*,*)' ilevel',ilevel
c
c CORE TRACE IS FINISHED - NOW TO MANIPULATE THE RESULTS
c
c Now we are going to tidy up the skeleton: first delete all links
c from peaks.
c
      do 320 ifeature=1,nfeature
       if (jointo(ifeature).gt.0) then
c if this is a peak, delete all its neigbours
 310    if (joinptr(ifeature).ne.0) then
         call deleteafter(joinptr,ifeature)
         goto 310
        endif
       endif
 320  continue
c
c now make links from the peaks to the joins
c
      do 350 ifeature=1,nfeature
       if (jointo(ifeature).lt.0) then
c if this is a join, fill in the links from its neighbouring peaks
        ptr=joinptr(ifeature)
 340    if (ptr.ne.0) then
         j=jointo(ptr)
         call insertafter(joinptr,j)
         jointo(j)=ifeature
         ptr=joinptr(ptr)
         goto 340
        endif
       endif
 350  continue
c
      i=0
      j=0
      do 683 iw=0,nws-1
      do 683 iv=0,nvs-1
      do 683 iu=0,nus-1
       dojoin=mapfeat(sklmap(iu+nus*(iv+nvs*iw)),ftr,gen)
       i=max(i,ftr)
       j=max(j,gen)
       if (gen.gt.levels+1)
     +   write(*,*)iu,iv,iw,sklmap(iu+nus*(iv+nvs*iw)),ftr,gen,dojoin
cxx       sklmap(iu,iv,iw)=-abs(sklmap(iu,iv,iw))
 683  continue
      write (*,*)'maxftr',i,' maxgen ',j
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine setmapfeat(point,ftr,gen)
      include 'dmheader.fh'
c set a map point with a given feature and generation no as negative
      integer point,ftr,gen,temp
      call ccpsi2(ftr,temp,1)
      call ccpsi2(gen,temp,2)
      point=-temp
      return
      end
c
      logical function mapfeat(point,ftr,gen)
      include 'dmheader.fh'
c get feature and generation no from a map point, return true if negative
      integer point,ftr,gen,temp
      temp=abs(point)
      call ccpgi2(ftr,temp,1)
      call ccpgi2(gen,temp,2)
      mapfeat=(point.lt.0)
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine joinfeat(ifeature,jfeature,joinptr,jointo)
      include 'dmheader.fh'
c joinfeat - join feature i to feature j
      integer ifeature,jfeature,joinptr(0:*),jointo(0:*)
      integer ptr
c
c join i to j
      ptr=ifeature
      call insertafter(joinptr,ptr)
      jointo(ptr)=jfeature
c join j to i
      ptr=jfeature
      call insertafter(joinptr,ptr)
      jointo(ptr)=ifeature
      return
      end
c
c ----------------------------------------------------------------------
c
      logical function arejoined(ifeature,jfeature,joinptr,jointo)
      include 'dmheader.fh'
c arejoined - are feature i and feature j linked?
      integer ifeature,jfeature,joinptr(0:*),jointo(0:*)
      integer ptr
c
      ptr=joinptr(ifeature)
 100  if (ptr.ne.0) then
       if (jointo(ptr).eq.jfeature) goto 200
       ptr=joinptr(ptr)
       goto 100
      endif
      arejoined=.false.
      return
 200  arejoined=.true.
      return
      end
c
c ----------------------------------------------------------------------
c
c Automatic level detection:
c
c ----------------------------------------------------------------------
c
      subroutine ridgelevel
     +  (map,nu,nv,nw,fuvw,joinptr,jointo,nfeature)
      include 'dmheader.fh'
      include 'crystal.fh'
      include 'params.fh'
      include 'skldat.fh'
      include 'cycl.fh'
      include 'io.fh'
c ridgelevel - set up ridge cutoff levels
      integer nu,nv,nw,map(0:nu-1,0:nv-1,0:nw-1)
      integer fuvw(3,*),joinptr(0:*),jointo(0:*),nfeature
c
      integer ntry
      real a1,b1,a2,b2
      parameter (ntry=6,a1=0.15,a2=0.30,b1=0.80)
      integer i
      real r(0:ntry),s(0:ntry),rjoin,rendp,sksig,skend,resids
c
c
      resids=vol*(1.0-solc)/(140.0)
      write (lpt,900)nint(resids),nint(resids/nsym)
      rjoin=0.0
      rendp=0.0
c
c FIRST TRY A RIDGE TRACE TO SEE IF LEVELS NEED CHANGING...
c  (unless this is the first cycle)
      if (icycl.gt.ncskl) then
       call ridgetrace(map,nu,nv,nw,fuvw,joinptr,jointo,nfeature,
     +   sklsig,1.0e6,rjoin)
       call ridgetrace(map,nu,nv,nw,fuvw,joinptr,jointo,nfeature,
     +   sklsig,sklend,rendp)
      endif
c
c were the values reasonable?
c??      if (abs(rjoin/resids-ljoin).gt.0.75.or.
c??     +  abs((rendp-rjoin)/resids-lendp).gt.1.5) then
      if (icycl.le.ncskl) then
c
c LEVELS NEED CHANGING:
c TRY RIDGE TRACE TO FIND JOIN LEVEL...
c
       skend=1.0e6
       do 300 i=0,ntry
c try some join cutoff levels
        sksig=a1*i+b1
        call ridgetrace(map,nu,nv,nw,fuvw,joinptr,jointo,nfeature,
     +    sksig,skend,r(i))
c convert to length per residue
        r(i)=r(i)/resids
 300   continue
c
c search for a good join level
       do 400 i=1,ntry-1
        if (r(i).lt.ljoin) goto 410
 400   continue
 410   sksig=a1*(real(i-1)+(r(i-1)-ljoin)/(r(i-1)-r(i)))+b1
c
c and check the amount of ridge
       call ridgetrace(map,nu,nv,nw,fuvw,joinptr,jointo,nfeature,
     +   sksig,skend,rjoin)
c
c TRY THE RIDGE TRACE TO FIND THE END LEVEL...
       b2=sksig
       do 500 i=0,ntry
c try some end point cutoff levels
        skend=a2*i+b2
        call ridgetrace(map,nu,nv,nw,fuvw,joinptr,jointo,nfeature,
     +    sksig,skend,s(i))
c convert to length per residue
        s(i)=(s(i)-rjoin)/resids
 500   continue
c
c search for a good endpoint level
       do 600 i=1,ntry-1
        if (s(i).lt.lendp) goto 610
 600   continue
 610   skend=a2*(real(i-1)+(s(i-1)-lendp)/(s(i-1)-s(i)))+b2
c
c now do the final trace:
       call ridgetrace(map,nu,nv,nw,fuvw,joinptr,jointo,nfeature,
     +   sksig,skend,rendp)
c
c now write some results:
       write (lpt,905)
       write (lpt,906)(a1*i+b1,i=0,ntry)
       write (lpt,907)(r(i),i=0,ntry)
       write (lpt,908)
       write (lpt,906)(a2*i+b2,i=0,ntry)
       write (lpt,907)(s(i),i=0,ntry)
       sklsig=sksig
       sklend=skend
c done auto-level detection
      endif
c
c LEVEL HAS BEEN SET AND RISGE TRACE HAS BEEN DONE
c Write out the final levels..
c
      write (lpt,909)sklsig,rjoin/resids,sklend,(rendp-rjoin)/resids
c
 900  format(/' Automatic skeleton parameter determination:'
     +  /'  Estimated number of residues is ',i6,' or ',i6,
     +  ' per asymmetric unit'/)
 905  format('  Join level sigma cutoff vs. length of joined chain:')
 906  format('   cutoff ',20f6.2)
 907  format('   length ',20f6.2)
 908  format('  End  level sigma cutoff vs. length of trailing chain:')
 909  format(/'  Join level cutoff is ',f5.2,
     +  ' sigma giving ',f5.1,' Angstroms/residue'
     +       /'  End  level cutoff is ',f5.2,
     +  ' sigma giving ',f5.1,' Angstroms/residue')
c
      return
      end
c
c ----------------------------------------------------------------------
c
c Ridge tracing and marking
c
c ----------------------------------------------------------------------
c
      subroutine ridgetrace
     +  (map,nu,nv,nw,fuvw,joinptr,jointo,nfeature,sksig,skend,r)
      include 'dmheader.fh'
      include 'io.fh'
      include 'crystal.fh'
      include 'skldat.fh'
c ridgetrace - trace ridges and mark in the map
      integer nu,nv,nw,map(0:nu-1,0:nv-1,0:nw-1)
      integer fuvw(3,*),joinptr(0:*),jointo(0:*),nfeature
      real sksig,skend,r
c
      integer i,j,k,iu,iv,iw,ju,jv,jw,pu,pv,pw,siglvl,endlvl
      integer ptr,ftr,lftr,gen,igen,jgen,lgen,pgen,near,lmod
      real x,y,z,du,dv,dw,r2,thisd2,rhocut
      logical mapfeat,dum
c
      external mapfeat,lmod
c
c
      rhocut=rhoave+sksig*rhostd
      siglvl=int(levels*(rhomax-rhocut)/(rhomax-rhomin))
      rhocut=rhoave+skend*rhostd
      endlvl=int(levels*(rhomax-rhocut)/(rhomax-rhomin))
c
      do 100 iw=0,nw-1
      do 100 iv=0,nv-1
      do 100 iu=0,nu-1
       map(iu,iv,iw)=-abs(map(iu,iv,iw))
 100  continue
c
c loop through joins
      k=0
      do 400 j=1,nfeature
       if (jointo(j).lt.0) then
        ju=fuvw(1,j)
        jv=fuvw(2,j)
        jw=fuvw(3,j)
        dum=mapfeat(map(ju,jv,jw),ftr,jgen)
        if (.not.dum) jgen=0
        ptr=joinptr(j)
c . loop through join links
 200    if (ptr.ne.0) then
         lftr=jointo(ptr)
         pu=fuvw(1,lftr)
         pv=fuvw(2,lftr)
         pw=fuvw(3,lftr)
         dum=mapfeat(map(pu,pv,pw),ftr,pgen)
         if (.not.dum) pgen=0
c .. if the join is above the cutoff, or the peak is greater than the
c .. end point value, then trace the ridge
         iu=ju
         iv=jv
         iw=jw
c .. write a point
 250      dum=mapfeat(map(iu,iv,iw),ftr,gen)
          if (.not.dum) gen=0
          if (jgen.le.siglvl.or.gen.le.endlvl)
     +      map(iu,iv,iw)=abs(map(iu,iv,iw))
          igen=0
          lgen=32768
          do 270 i=1,ndn
           near=map(lmod(iu+ndu(i),nu),lmod(iv+ndv(i),nv),
     +              lmod(iw+ndw(i),nw))
           dum=mapfeat(near,ftr,gen)
           if (.not.dum) gen=0
           if (ftr.eq.lftr.and.gen.lt.lgen) then
            igen=i
            lgen=gen
           endif
 270      continue
          if (igen.eq.0) call ccperr(1,' ridgetrace - internal error')
c .. move to eldest neighbour
          iu=lmod(iu+ndu(igen),nu)
          iv=lmod(iv+ndv(igen),nv)
          iw=lmod(iw+ndw(igen),nw)
c . until peak or a ridge is found
         if (lgen.ne.pgen.and.lgen.ne.0) goto 250
         dum=mapfeat(map(iu,iv,iw),ftr,gen)
         if (.not.dum) gen=0
         if (jgen.le.siglvl.or.gen.le.endlvl)
     +     map(iu,iv,iw)=abs(map(iu,iv,iw))
         ptr=joinptr(ptr)
         goto 200
        endif
       endif
 400  continue
c
c done ridge trace: now calc total length of ridge
c
      r=0.0
      do 500 iw=0,nw-1
      do 500 iv=0,nv-1
      do 500 iu=0,nu-1
c loop through 1/2 of neighbours (to count links only 1 way)
       if (map(iu,iv,iw).ge.0) then
        do 480 i=1,ndn/2
         ju=lmod(iu+ndu(i),nu)
         jv=lmod(iv+ndv(i),nv)
         jw=lmod(iw+ndw(i),nw)
         if (map(ju,jv,jw).ge.0) then
          du=real(ndu(i))/nu
          dv=real(ndv(i))/nv
          dw=real(ndw(i))/nw
          r2=rlm11*du**2+rlm22*dv**2+rlm33*dw**2+
     +       rlm12*du*dv+rlm13*du*dw+rlm23*dv*dw
          r=r+sqrt(r2)
         endif
 480    continue
       endif
 500  continue
c      write (*,*)r
c
c      i=0
c      do iw=0,nw-1
c       do iv=0,nv-1
c        do iu=0,nu-1
c         if (map(iu,iv,iw).ge.0) i=i+1
c        enddo
c       enddo
c      enddo
c      write (*,*)' i,n ',i,nu*nv*nw
c
      return
      end
c
c ----------------------------------------------------------------------
c
c Linked lists (yeuch!) - used for storing feature links
c
c ----------------------------------------------------------------------
c
      subroutine linkedlist(listptr,listlen)
      include 'dmheader.fh'
c initialise a linked list with listptr(0) as freeptr
      integer listlen,listptr(0:listlen)
      integer i
c initial list is a free-list, with each element pointing to the next
      do 100 i=0,listlen
       listptr(i)=i+1
 100  continue
      listptr(listlen)=0
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine inserthead(listptr,ptr)
      include 'dmheader.fh'
c insert val as new list in linked list and return its pointer
      integer listptr(0:*),ptr
c insert in next free slot
      ptr=listptr(0)
      if (ptr.eq.0)
     +  call ccperr(1,' inserthead - linked list overrun')
c this=end, advance free ptr
      listptr(0)=listptr(ptr)
      listptr(ptr)=0
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine insertafter(listptr,ptr)
      include 'dmheader.fh'
c insert val in linked list after value at ptr
      integer listptr(0:*),ptr
      integer newptr
c insert in next free slot
      newptr=listptr(0)
      if (newptr.eq.0)
     +  call ccperr(1,' insertafter - linked list overrun')
c this -> new, new -> next, advance free ptr
      listptr(0)=listptr(newptr)
      listptr(newptr)=listptr(ptr)
      listptr(ptr)=newptr
      ptr=newptr
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine deletehead(listptr,ptr)
      include 'dmheader.fh'
c delete list header element (don't do this to linked eles!)
      integer listptr(0:*),ptr
c
      if (listptr(0).eq.0)
     +  call ccperr(1,' deletehead - linked list underrun')
      listptr(ptr)=listptr(0)
      listptr(0)=ptr
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine deleteafter(listptr,ptr)
      include 'dmheader.fh'
c delete element after this from the list
      integer listptr(0:*),ptr
      integer delptr
c
      delptr=listptr(ptr)
      if (delptr.eq.0)
     +  call ccperr(1,' deleteafter - linked list underrun')
c old ele -> free list, this ele points to the one after
      listptr(ptr)=listptr(delptr)
      listptr(delptr)=listptr(0)
      listptr(0)=delptr
      return
      end
c
c ----------------------------------------------------------------------
c
c Density building
c
c ----------------------------------------------------------------------
c
      subroutine buildden(bigmap,nus,nvs,nws,rho,solmsk,reseff)
      include 'dmheader.fh'
      include 'io.fh'
      include 'params.fh'
      include 'hkldata.fh'
      include 'crystal.fh'
      include 'uvwdata.fh'
c build density around the skeleton
      integer nus,nvs,nws,bigmap(0:nus-1,0:nvs-1,0:nws-1)
      integer solmsk(0:nu-1,0:nv-1,0:nw-1)
      real rho(0:nu-1,0:nv-1,0:nw-1),reseff
c
      integer nres
      real drad
      parameter (nres=14,drad=0.2)
      real resl(nres),rhor(0:15,nres)
c
      integer iu,iv,iw,i,j,k,i1,j1,k1,i2,j2,k2,ires,ir,ftr,gen,lmod
      real x,y,z,u,v,w,u1,v1,w1,du,dv,dw
      real r,r2,dr,rad,ratom
      logical mapfeat
c
      parameter (ratom=2.4)
c
      external lmod,mapfeat
c
      data resl/5.000, 4.500, 4.000, 3.600, 3.300, 3.000, 2.800,
     +          2.600, 2.400, 2.200, 2.000, 1.800, 1.600, 1.500/
      data ((rhor (i,j), i=0,15),j=1,7) /
     + 0.407, 0.404, 0.396, 0.382, 0.364, 0.341, 0.314, 0.284,
     +  0.251, 0.217, 0.182, 0.146, 0.112, 0.079, 0.048, 0.020,
     + 0.539, 0.534, 0.519, 0.495, 0.463, 0.423, 0.378, 0.329,
     +  0.278, 0.226, 0.175, 0.127, 0.083, 0.044, 0.011,-0.015,
     + 0.603, 0.596, 0.578, 0.548, 0.508, 0.459, 0.404, 0.343,
     +  0.281, 0.219, 0.159, 0.103, 0.053, 0.010,-0.025,-0.051,
     + 0.815, 0.803, 0.769, 0.713, 0.641, 0.555, 0.460, 0.363,
     +  0.267, 0.177, 0.098, 0.032,-0.019,-0.054,-0.074,-0.081,
     + 0.929, 0.913, 0.868, 0.795, 0.701, 0.590, 0.471, 0.349,
     +  0.233, 0.129, 0.040,-0.029,-0.077,-0.105,-0.114,-0.106,
     + 1.093, 1.071, 1.007, 0.907, 0.778, 0.630, 0.475, 0.323,
     +  0.185, 0.068,-0.022,-0.083,-0.114,-0.120,-0.105,-0.076,
     + 1.246, 1.217, 1.133, 1.002, 0.836, 0.652, 0.464, 0.289,
     +  0.139, 0.023,-0.055,-0.096,-0.105,-0.088,-0.055,-0.016/
      data ((rhor (i,j), i=0,15),j=8,14) /
     + 1.394, 1.357, 1.251, 1.086, 0.881, 0.657, 0.435, 0.236,
     +  0.073,-0.043,-0.111,-0.134,-0.122,-0.087,-0.042, 0.000,
     + 1.601, 1.551, 1.408, 1.191, 0.927, 0.649, 0.387, 0.168,
     +  0.007,-0.089,-0.124,-0.112,-0.071,-0.019, 0.028, 0.058,
     + 1.815, 1.749, 1.562, 1.282, 0.951, 0.614, 0.315, 0.083,
     + -0.065,-0.130,-0.128,-0.083,-0.021, 0.035, 0.069, 0.077,
     + 2.036, 1.951, 1.711, 1.357, 0.947, 0.546, 0.207,-0.032,
     + -0.158,-0.184,-0.138,-0.057, 0.022, 0.073, 0.085, 0.061,
     + 2.403, 2.280, 1.936, 1.447, 0.914, 0.435, 0.083,-0.111,
     + -0.159,-0.106,-0.013, 0.070, 0.109, 0.098, 0.054, 0.001,
     + 2.833, 2.652, 2.161, 1.492, 0.811, 0.268,-0.056,-0.159,
     + -0.106, 0.007, 0.097, 0.119, 0.077, 0.005,-0.054,-0.073,
     + 3.076, 2.857, 2.270, 1.490, 0.733, 0.175,-0.108,-0.147,
     + -0.046, 0.074, 0.131, 0.106, 0.031,-0.039,-0.064,-0.039/
c
c clear the map to a solvent mask
c
      do 100 iw=0,nw-1
      do 100 iv=0,nv-1
      do 100 iu=0,nu-1
       rho(iu,iv,iw)=0.0
 100  continue
c
c pick model resolution
c
      do 110 i=1,nres-1
       if (resl(i).le.reseff) goto 120
 110  continue
 120  ires=i
c
c now go through and fill in density for each link
c
      do 450 iw=0,nws-1
      w1=real(iw)/nws
      do 450 iv=0,nvs-1
      v1=real(iv)/nvs
      do 450 iu=0,nus-1
      u1=real(iu)/nus
       if (bigmap(iu,iv,iw).ge.0) then
c fill in mask within 2A of point
        i=nint(u1*nu)
        j=nint(v1*nv)
        k=nint(w1*nw)
        do 400 k1=k-4,k+4
        do 400 j1=j-4,j+4
        do 400 i1=i-4,i+4
         du=real(i1)/nu-u1
         dv=real(j1)/nv-v1
         dw=real(k1)/nw-w1
         r2=rlm11*du**2+rlm22*dv**2+rlm33*dw**2+
     +      rlm12*du*dv+rlm13*du*dw+rlm23*dv*dw
         rad=sqrt(r2)
         if (rad.le.ratom) then
          i2=lmod(i1,nu)
          j2=lmod(j1,nv)
          k2=lmod(k1,nw)
          r=rad/drad
          ir=int(r)
          dr=r-ir
          rho(i2,j2,k2)=max(rho(i2,j2,k2),
     +      rhor(ir,ires)+dr*(rhor(ir+1,ires)-rhor(ir,ires)) )
         endif
 400    continue
c      
       endif
 450  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine frac2orth(u,v,w,x,y,z)
      include 'dmheader.fh'
      include 'crystal.fh'
      real u,v,w,x,y,z
      x=q(1,1)*u+q(1,2)*v+q(1,3)*w+q(1,4)
      y=q(2,1)*u+q(2,2)*v+q(2,3)*w+q(2,4)
      z=q(3,1)*u+q(3,2)*v+q(3,3)*w+q(3,4)
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine orth2frac(x,y,z,u,v,w)
      include 'dmheader.fh'
      include 'crystal.fh'
      real u,v,w,x,y,z
      u=qt(1,1)*x+qt(1,2)*y+qt(1,3)*z+qt(1,4)
      v=qt(2,1)*x+qt(2,2)*y+qt(2,3)*z+qt(2,4)
      w=qt(3,1)*x+qt(3,2)*y+qt(3,3)*z+qt(3,4)
      return
      end
c
c ----------------------------------------------------------------------
c


