C
C     DMMULTI density modification and multi-crystal averaging in core
C     Copyright (C) 1995-2004 Kevin Cowtan
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
c
c ----------------------------------------------------------------------
c
      program dmmulti
c
c dmmulti - Density modification and multi-crystal averaging in core.
c
c  Hello people! :-)
c
c  Density modification and multi-crystal averaging in core.
c  By Kevin Cowtan (cowtan@yorvic.york.ac.uk)
c   `dm`  code date: 25/1/95 (alpha skeletonisation release)
c   multi code date:  6/2/95 - 
c
c   Algorithms and insight:          Peter Main, Kam Zhang, Dave Schuller
c   Spectral interpolation:                      Eric Grosse
c   Library support and program portability:     Dave Love
c   Music and cutscenes (some platforms only):   Rob Povey
c
c   Scientific background:                     see manual references
c
c   Thanks also to LucasArts for Xwing, which is great fun and less
c   damaging to the braincells than writing this code.
c                                   General Rabbit 1 - Death Star 0
c
c   No thanks to DEC, for their zealous interpretation of the
c   Fortran77 array parsing standards. Also for VMS.     >:-[
c
c   People with a sensitive disposition may read this program with
c   confidence, there is no swearing in my comments. Neither are
c   there strings of ascii characters (!@#$%^&*) in lieu of the above.
c               ...oops     ;-)
c
c   If it ever works you had better thank Yeshua ben Yosef (known as
c   Jesus in English), `cos this is so complicated I don`t see much 
c   hope of me ever fixing it.
c
c Also many niggling bugs spotted by Clemens Vonrhein!
c
c Kevin 9/2/95
c               ...usually  8^)
c
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xtalinfo.fh'
c
      integer narrays,nmaps
      parameter (nmaps=6,narrays=nmaps+2)
      integer uvwsize(maxxtal),hklsize(maxxtal),domsize(maxdomn)
      integer uvwmax,hklmax,dommax
      integer size(narrays)
      integer i
      integer lunsti,lunsto,ccpnun
      character*1 type(narrays)
c
      external domulti,lunsti,lunsto,ccpnun
c
      data type/narrays*'R'/size/narrays*0/
c
c Standard setup:
      call ccpfyp()
      call mtzini()
      call ccprcs(6, 'dmmulti',
     +     '$Date: 2007/04/23 10:14:55 $')

      lpt=lunsto(i)
      lcd=lunsti(i)
      latom=ccpnun()
      lhist=ccpnun()+1
      lmsks=ccpnun()+2
      ltmp =ccpnun()+3
      call rseed(54321)
c
c Clear the size arrays.
      do 100 i=1,maxxtal
       uvwsize(i)=0
       hklsize(i)=0
 100  continue
      do 120 i=1,maxdomn
       domsize(i)=0
 120  continue
      do 140 i=1,narrays
       size(i)=0
 140  continue
c
c Now see what sort of arrays we need - big ones I guess:
c
      call rcards()
      call sizes(uvwsize,hklsize,domsize,uvwmax,hklmax,dommax)
c
c 1 hkl multi-array, 1 domain multi-array, many map arrays.
c
      do 220 i=1,nxtal
       size(1)=size(1)+hklsize(i)
 220  continue
      do 240 i=1,ndomn
       size(2)=size(2)+domsize(i)
 240  continue
c 2 maps for basic dm
      size(3)=uvwmax
      size(4)=uvwmax
c and 1 solvent mask
      size(8)=uvwmax
c Now the mammoth array for skeletonisation
      do 260 i=1,nxtal
       if (nint(xtaldat(lmskel,i)).ne.0) size(4)=max(size(4),uvwsize(i))
 260  continue
c don`t risk allocating less than 256 bytes...
      do 280 i=1,narrays
       size(i)=max(size(i),64)
 280  continue
c
c Prepare offsets for accessing the multiplexed arrays:
c  the multiplexed arrays for the hkls or masks will be accessed
c  by offsets into giant 1-d arrays (1...n)
c
      hkloff(1)=1
      domoff(1)=1
      do 40 i=2,nxtal
       hkloff(i)=hkloff(i-1)+hklsize(i-1)
 40   continue
      do 60 i=2,ndomn
       domoff(i)=domoff(i-1)+domsize(i-1)
 60   continue
c
c That was the easy bit %(
c
      write(6,'(/,a/,a,/)') ' memory allocation:'
     +  ,' Array Type       Size'
      do i=1,narrays
        write(6,'(i5,5x,a1,2x,i15)') i,type(i),size(i)
      enddo
      call ccpalc(domulti,narrays,type,size)
c
      stop
      end
c
c ----------------------------------------------------------------------
c
      subroutine domulti(lhklmul,hklmul,ldommul,dommul,lmap1,map1,
     +  lmap2,map2,lmap3,map3,lmap4,map4,lmap5,map5,lmap6,map6)
      include 'dmmulti.fh'
c DOMULTI - main multi-xtal program routine
c  It goes something like this:
c
c++++++++++++++++++++++++++++++++++++++++
c  for crystal=1,ncrytals
c    read hkl(crystal)
c  endfor
c  for domain=1,ndomains
c    read mask(domain)
c  endfor
c
c  for cycle=1,ncycles
c
c    for crossvl=1,ncrossvl
c
c      for crystal=1,ncrytals
c        make (spectral) map(crystal)
c        average from map(crystal) to dommap(domain(1..ndomains))
c      endfor
c
c      for domain=1,ndomains
c        normalise (spectral) dommap(domain)
c      endfor
c
c      for crystal=1,ncrytals
c        build map(crystal) from dommap(domain(1..ndomains))
c        density modify map(crystal)
c      endfor
c
c    endfor
c
c    for crystal=1,ncrytals
c      phase recombination on crystal
c    endfor
c
c  endfor
c
c  for crystal=1,ncrytals
c    write hkl(crystal)
c  endfor
c++++++++++++++++++++++++++++++++++++++++
c
      include 'io.fh'
      include 'xtalinfo.fh'
      include 'cyclsync.fh'
      include 'xtlparam.fh'
      include 'xavginfo.fh'
c
c map arrays
      integer lmap1,lmap2,lmap3,lmap4,lmap5,lmap6
      real map1(lmap1),map2(lmap2),map3(lmap3)
      real map4(lmap4),map5(lmap5),map6(lmap6)
c multiplexed reflection and domain arrays (sorry)
      integer lhklmul,ldommul
      real hklmul(lhklmul),dommul(ldommul)
c
c local variables
      real reseff(maxxtal),xtlwt(maxxtal)
      integer xtal,domn,nrfls,icfr,incs,idom,iref
      logical print
      integer lfa,lfb,lfwa,lfwb
      parameter (lfa=20,lfb=21,lfwa=22,lfwb=23)
c
      character*8 mtzin(maxxtal),mtzout(maxxtal),
     +  solin(maxxtal),solout(maxxtal),mskin(maxdomn)
      common /filename/mtzin,mtzout,solin,solout,mskin
      save   /filename/
c
      real rrand
      external rrand
c
c Now do some crystallography!
c
      call readhist()
c
c Read in the reflection data
c
      do 200 xtal=1,nxtal
       call rseed(54321)
       call getxtal(xtal)
       call hklin(hklmul(hkloff(xtal)),nrfls)
       call calcscale(hklmul(hkloff(xtal)),scale,bfac)
       xtaldat(lnrflo,xtal)=real(nrfls)
       xtaldat(lscale,xtal)=scale
       xtaldat(lscale+1,xtal)=bfac
       call phextsetup(hklmul(hkloff(xtal)))
 200  continue
c
c Read in the domain masks (real x1000 marked as inside,edge,outside)
c
      do 220 domn=1,ndomn
       call getdomn(domn)
       call domnin(dommul(domoff(domn)),mskin(domn))
 220  continue
c
c Start the phase improvement loop
c
      do 800 icycl=1,ncycl
c
c . zero the domains
c
       do 240 domn=1,ndomn
        call getdomn(domn)
        call domzero(dommul(domoff(domn)))
        domndat(ldomnwt,domn)=0.0
 240   continue
c
c . set up this phase improvement cycle
c
       do 250 xtal=1,nxtal
        call getxtal(xtal)
c ..    zero the sigma-q totals
        call setxfr(hklmul(hkloff(xtal)))
c ..    pick the reflections to include
        call phextstep(hklmul(hkloff(xtal)),print,reseff(xtal))
 250   continue
c
c . refine the averaging matrices
c
       if (maver.eq.1.and.mod(icycl-1,ncref).eq.0) then
        do 300 xtal=1,nxtal
         call getxtal(xtal)
c? use all reflections for matrix refinement?
c?         call setfreer(hklmul(hkloff(xtal)),icfr,ncfr,icycl)
         call wfo(hklmul(hkloff(xtal)),lfa,lfb,xtlwt(xtal))
         if (weight.gt.0.) xtlwt(xtal)=weight
         call hklfpab(hklmul(hkloff(xtal)),lfa,lfb)
         call bsplinewt(hklmul(hkloff(xtal)),lfa,lfb,lfwa,lfwb)
 300    continue
        call refinefromcell(map1,hklmul,dommul,xtlwt,lfwa,lfwb)
       endif
c
c . start the cross-validation set loop
c
       do 700 icfr=1,ncfr
c
c . prepare the hkl data and average the domains
c
        do 350 xtal=1,nxtal
         call getxtal(xtal)
c ...    set up this cycle`s free-r set
         call setfreer(hklmul(hkloff(xtal)),icfr,ncfr,icycl)
c ...    generate wFo`s
         call wfo(hklmul(hkloff(xtal)),lfa,lfb,xtlwt(xtal))
         if (weight.gt.0.) xtlwt(xtal)=weight
         call hklfpab(hklmul(hkloff(xtal)),lfa,lfb)
c ...    if we are averaging we need spline coefficients too
         if (maver.eq.1) then
          call bsplinewt(hklmul(hkloff(xtal)),lfa,lfb,lfwa,lfwb)
          if (nxtal.gt.1) write (lpt,315)icycl,xtal,xtlwt(xtal)
 315      format (/' Cycle:',i3,'  Crystal:',i2,'  Weight:',f6.3/)
          call fftlistmap(map1,hklmul(hkloff(xtal)),lfwa,lfwb)
          call averagefromcell
     +      (hklmul(hkloff(xtal)),map1,dommul,domoff,xtlwt(xtal))
         endif
 350    continue
c
c .. average back into each xtal and density modify
c
        do 550 xtal=1,nxtal
         call getxtal(xtal)
c ...    calculate the map
         call fftlistmap(map1,hklmul(hkloff(xtal)),lfa,lfb)
c ...    now do the averaging back into the crystal cell
         if (maver.eq.1) then
          call averagetocell
     +      (hklmul(hkloff(xtal)),map1,dommul,domoff,reseff(xtal))
         endif
c ...    solvent mask in (every cycle)
         if (xtaldat(lsolin,xtal).gt.0.5) then
          call solmskin(solin(xtal),map6,map2,icycl)
         else
          call copy3d(map1,map2)
          call wangmask(map2,map6,print)
         endif
c ...    density modification
         call dm
     +      (hklmul(hkloff(xtal)),map1,map2,map6,reseff(xtal),.true.)
c ...    solvent mask out (last cycle)
         if (icycl.eq.ncycl.and.icfr.eq.ncfr.and.
     +     xtaldat(lsolout,xtal).gt.0.5) then
          call solmskout(solout(xtal),map6)
         endif
c ...    accumulate phase recombination and free-r statistics
         call invfftmaplist(map2,hklmul(hkloff(xtal)),lfa,lfb)
         call hklabfp(hklmul(hkloff(xtal)),lfa,lfb)
         call accxfr(hklmul(hkloff(xtal)),lfa,lfb)
 550    continue
c
c .. go back and do exactly the same thing again with a different free set
c
 700   continue
c
c . finish phase improvement cycle with phase recombination
c
       do 750 xtal=1,nxtal
        call getxtal(xtal)
        call combxfr(hklmul(hkloff(xtal)),lfa,lfb)
 750   continue
c
c . next cycle
c
 800  continue
c
c write out the results
c
      do 900 xtal=1,nxtal
       call getxtal(xtal)
       call hklout(hklmul(hkloff(xtal)),lfa,lfb)
       call lptout()
 900  continue
c
c Phew!
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine rcards()
c RCARDS - read punched card input
c err...hmm. Maybe this is the twentieth century
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xtalinfo.fh'
      include 'xcrystal.fh'
      include 'xdominfo.fh'
      include 'cyclsync.fh'
c
      integer xtal,domn,i,j,n,nu,nv,nw,nrflc,ifail,iappnd
      integer nlprgi,nlprgo,nncs,jpoint(mcols)
      character ctprgi(mcols)*1,ctprgo(mcols)*1,
     +          lsprgi(mcols)*30,lsprgo(mcols)*30
      character modes*80,filnam*200,ltype*1,namspg*10,nampg*10,namlau*10
      real rmin,rmax,smin,smax,maxres
      real alpha,beta,gamma,omega,phi,kappa
      real mat(4,4)
      logical exist,cont
c
      character*8 mtzin(maxxtal),mtzout(maxxtal),
     +  solin(maxxtal),solout(maxxtal),mskin(maxdomn)
      common /filename/mtzin,mtzout,solin,solout,mskin
      save   /filename/
c
      integer numres
c
      external numres
c
c input and output filenames
      data mtzin /'HKLIN1 ','HKLIN2 ','HKLIN3 ','HKLIN4 ','HKLIN5 ',
     +            'HKLIN6 ','HKLIN7 ','HKLIN8 ','HKLIN9 '/
      data mtzout/'HKLOUT1','HKLOUT2','HKLOUT3','HKLOUT4','HKLOUT5',
     +            'HKLOUT6','HKLOUT7','HKLOUT8','HKLOUT9'/
      data solin /'SOLIN1 ','SOLIN2 ','SOLIN3 ','SOLIN4 ','SOLIN5 ',
     +            'SOLIN6 ','SOLIN7 ','SOLIN8 ','SOLIN9 '/
      data solout/'SOLOUT1','SOLOUT2','SOLOUT3','SOLOUT4','SOLOUT5',
     +            'SOLOUT6','SOLOUT7','SOLOUT8','SOLOUT9'/
      data mskin /'MSKIN1 ','MSKIN2 ','MSKIN3 ','MSKIN4 ','MSKIN5 ',
     +            'MSKIN6 ','MSKIN7 ','MSKIN8 ','MSKIN9 ','MSKIN10',
     +            'MSKIN11','MSKIN12','MSKIN13','MSKIN14','MSKIN15'/
c input and output column labels
      data nlprgi,lsprgi,ctprgi/15,
     +  'H','K','L','FP','SIGFP','PHIO','FOMO','HLA','HLB','HLC',
     +  'HLD','FDM','PHIDM','FOMDM','FREE',5*' ',
     +  'H','H','H', 'F',    'Q',   'P',   'W',  'A',  'A',  'A',
     +    'A',  'F',    'P',    'W',   'I',5*' '/
      data nlprgo,lsprgo,ctprgo/4,
     +'PHIDM','FOMDM','FCDM','PHICDM',16*' ',
     +    'P',    'W',   'F',     'P',16*' '/
c
c
c Set up the defaults:
c
c .. the general stuff
      ncycl=10
      ncfr=1
      ncskl=3
      ncref=2
      do 100 j=1,4
      do 100 i=1,4
       mat(i,j)=0.0
       if (i.eq.j) mat(i,j)=1.0
 100  continue
c .. and the xtal-specific
      do 150 xtal=1,maxxtal
       do 110 i=1,maxxdat
        xtaldat(i,xtal)=0.0
 110   continue
       xtaldat(lweight,xtal)=-1.0
       xtaldat(lsolcmsk,xtal)=-1.0
       xtaldat(lrmin,xtal)=10000.0
       xtaldat(lsolmean,xtal)=0.32
       xtaldat(lprtmean,xtal)=0.43
       xtaldat(lwangrd,xtal)=8.0
       xtaldat(lwangmd,xtal)=1.0
       xtaldat(lwngmin,xtal)=0.32
       xtaldat(lwngmax,xtal)=2.00
       xtaldat(lskjoin,xtal)=6.0
       xtaldat(lskendp,xtal)=6.0
       xtaldat(lskbfac,xtal)=45.0
       xtaldat(lrefdr,xtal)=-999.0
       xtaldat(lrefdp,xtal)= 2.5
       xtaldat(lsolin ,xtal)=0.0
       xtaldat(lsolout,xtal)=0.0
 150  continue
c
c
c xtal is the number of the xtal to which the current input refers:
      xtal=1
      nxtal=0
      ndomn=0
c
c ****************************************
c Keyword input loop...
 200  call memoparse(.true.)
c XTAL
      call parseint('XTAL',xtal)
c MODE
      exist=.false.
      call parsesubkey('MODE','SOLV',exist)
      if (exist) xtaldat(lmsolv,xtal)=1.0
      exist=.false.
      call parsesubkey('MODE','HIST',exist)
      if (exist) xtaldat(lmhist,xtal)=1.0
      exist=.false.
      call parsesubkey('MODE','AVER',exist)
      if (exist) xtaldat(lmaver,xtal)=1.0
      exist=.false.
      call parsesubkey('MODE','SKEL',exist)
      if (exist) xtaldat(lmskel,xtal)=1.0
      exist=.false.
      call parsesubkey('MODE','SAYR',exist)
      if (exist) xtaldat(lmsayr,xtal)=1.0
      call parsekeyarg('MODE',modes)
c RESO
      call parsesubreal('RESO','    ',1,.true.,xtaldat(lrmin,xtal))
      call parsesubreal('RESO','    ',2,.true.,xtaldat(lrmax,xtal))
c WEIG
      call parsereal('WEIG',xtaldat(lweight,xtal))
c SOLC
      call parsesubreal('SOLC','    ',1,.true.,xtaldat(lsolc,xtal))
      call parsesubreal('SOLC','MASK',1,.true.,xtaldat(lsolcmsk,xtal))
      call parsesubreal('SOLC','MASK',2,.true.,xtaldat(lprtcmsk,xtal))
      call parsesubreal('SOLC','MEAN',1,.true.,xtaldat(lsolmean,xtal))
      call parsesubreal('SOLC','MEAN',2,.true.,xtaldat(lprtmean,xtal))
c GRID (we will read the ints as reals for the benefit of xtaldat)
      n=3
      call parsenreals ('GRID',n,xtaldat(lgrid,xtal))
c SCALE
      n=2
      call parsenreals ('SCAL',n,xtaldat(lscale,xtal))
c NCYCLE
      call parsesubint ('NCYC','    ',1,.true.,ncycl)
      call parsesubint ('NCYC','FREE',1,.true.,ncfr )
c SCHEME (AUTO is inferred)
      exist=.false.
      call parsesubkey('SCHE','AUTO',exist)
      exist=.false.
      call parsesubkey('SCHE','MAG',exist)
      if (exist) xtaldat(lwmag,xtal)=1.0
      exist=.false.
      call parsesubkey('SCHE','FOM',exist)
      if (exist) xtaldat(lwfom,xtal)=1.0
      exist=.false.
      call parsesubkey('SCHE','RES',exist)
      if (exist) xtaldat(lwres,xtal)=1.0
      call parsesubreal('SCHE','FRAC',1,.true.,xtaldat(lfrac,xtal))
      call parsesubreal('SCHE','FROM',1,.true.,xtaldat(lfrom,xtal))
c WANG
      call parsesubreal('WANG','    ',1,.true.,xtaldat(lwangrd,xtal))
      call parsesubreal('WANG','    ',2,.true.,xtaldat(lwangmd,xtal))
      call parsesubreal('WANG','LIMI',1,.true.,xtaldat(lwngmin,xtal))
      call parsesubreal('WANG','LIMI',2,.true.,xtaldat(lwngmax,xtal))
c SKEL
      call parsesubreal('SKEL','LENG',1,.true.,xtaldat(lskjoin,xtal))
      call parsesubreal('SKEL','LENG',2,.true.,xtaldat(lskendp,xtal))
      call parsesubreal('SKEL','BFAC',1,.true.,xtaldat(lskbfac,xtal))
      call parsesubint ('SKEL','EVER',1,.true.,ncskl)
c AVER
c .. this is a complex multi-line keyword
      exist=.false.
      call parsesubkey('AVER','    ',exist)
c .. spot if we have a keyword
      if (exist) then
       xtaldat(lnncs,xtal)=xtaldat(lnncs,xtal)+1.0
       nncs=nint(xtaldat(lnncs,xtal))
       domn=1
       call parsesubint ('AVER','DOMA',1,.true.,domn)
       call parsesubint ('AVER','EVER',1,.true.,ncref)
       exist=.false.
       call parsesubkey('AVER','REFI',exist)
       if (exist) xtaldat(lrefdr,xtal)=0.5
       call parsesubreal('AVER','STEP',1,.true.,xtaldat(lrefdr,xtal))
       call parsesubreal('AVER','STEP',2,.true.,xtaldat(lrefdp,xtal))
       call parsediagnose(cont)
c .. now look at next line: is it ROTA?
       call memoparse(.true.)
       exist=.false.
       call parsesubkey('ROTA','    ',exist)
       if (exist) then
c ... deal with ROTA
        do 240 i=1,3
        do 240 j=1,3
         call parsesubreal('ROTA','MATR',3*(i-1)+j,.true.,mat(i,j))
 240    continue
        alpha=-1000.0
        omega=-1000.0
        call parsesubreal('ROTA','EULE',1,.true.,alpha)
        call parsesubreal('ROTA','EULE',2,.true.,beta)
        call parsesubreal('ROTA','EULE',3,.true.,gamma)
        call parsesubreal('ROTA','POLA',1,.true.,omega)
        call parsesubreal('ROTA','POLA',2,.true.,phi)
        call parsesubreal('ROTA','POLA',3,.true.,kappa)
        if (alpha.gt.-999.0) call eulertomatrix(alpha,beta,gamma,mat)
        if (omega.gt.-999.0) call polartomatrix(omega,phi,kappa,mat)
        call parsediagnose(cont)
c ... deal with TRANS
        call memoparse(.true.)
        n=3
        call parsenreals('TRAN',n,mat(1,4))
       endif
c .. deal with `O` matrices
       exist=.false.
       call parsesubkey('OMAT','    ',exist)
       if (exist) then
        read (lcd,*)((mat(i,j),i=1,3),j=1,4)
       endif
c . now copy the matrix and domain flag into the crystal store
       xtaldat(lsrcdom+nncs,xtal)=real(domn)
       call symcpy(mat,xtaldat(lsrcncs+16*nncs,xtal))
c . check how many domains are used
       ndomn=max(ndomn,domn)
      endif
c LABI
      call parselabin(xtal,lsprgi,nlprgi)
c LABO
      call parselabout(xtal,lsprgo,nlprgo)
c
c check on how many xtals are used:
      nxtal=max(nxtal,xtal)
c
c next keyword
      call parsediagnose(cont)
      if (cont) goto 200
c Done keyword input
c ****************************************
c
c now look up the names of the input/output files
      call ugtenv(mtzin (1),filnam)
      if (filnam.eq.' ') mtzin(1) ='HKLIN'
      call ugtenv(mtzout(1),filnam)
      if (filnam.eq.' ') mtzout(1)='HKLOUT'
      call ugtenv(solin (1),filnam)
      if (filnam.eq.' ') solin(1) ='SOLIN'
      call ugtenv(solout(1),filnam)
      if (filnam.eq.' ') solout(1)='SOLOUT'
      call ugtenv(mskin (1),filnam)
      if (filnam.eq.' ') mskin(1) ='MSKIN'
c
c open the mtz`s, pick up cell and space group, plus anything else
c
      maxres=1000.0
c
      do 300 xtal=1,nxtal
c .. open mtz input file
       call lropen(xtal,mtzin(xtal),1,ifail)
       if (ifail.ne.0) goto 9010
c .. column assignments
       do 280 i=6,mcols
        jpoint(i)=0
 280   continue
c .. h k l fp sigfp are compulsory
       do 290 i=1,5
        jpoint(i)=-1
 290   continue
       call lrassn(xtal,lsprgi,nlprgi,jpoint,ctprgi)
       call ccpmvr(xtaldat(ljpoint,xtal),jpoint,mcols)
c .. cell, symm
       call lrcell(xtal,xtaldat(lcell,xtal))
       call calcmetric(xtaldat(lcell,xtal),
     +   vol,q,qt,rcm11,rcm12,rcm13,rcm22,rcm23,rcm33,
     +            rlm11,rlm12,rlm13,rlm22,rlm23,rlm33)
       xtaldat(lvol,xtal)=vol
       xtaldat(lrcm  ,xtal)=rcm11
       xtaldat(lrcm+1,xtal)=rcm12
       xtaldat(lrcm+2,xtal)=rcm13
       xtaldat(lrcm+3,xtal)=rcm22
       xtaldat(lrcm+4,xtal)=rcm23
       xtaldat(lrcm+5,xtal)=rcm33
       xtaldat(lrlm  ,xtal)=rlm11
       xtaldat(lrlm+1,xtal)=rlm12
       xtaldat(lrlm+2,xtal)=rlm13
       xtaldat(lrlm+3,xtal)=rlm22
       xtaldat(lrlm+4,xtal)=rlm23
       xtaldat(lrlm+5,xtal)=rlm33
       call symcpy(q,xtaldat(lmatq,xtal))
       call symcpy(qt,xtaldat(lmatqt,xtal))
c .. spacegroup
       call lrsymi(xtal,nsymp,ltype,nspgrp,namspg,nampg)
       call msymlb(ltmp,nspgrp,namspg,nampg,nsymp,nsym,rsym)
       call pgnlau(nampg,nlaue,namlau)
       xtaldat(lspgrp,xtal)=real(nspgrp)
       xtaldat(lnlaue,xtal)=real(nlaue)
       xtaldat(lnsym,xtal)=real(nsym)
       xtaldat(lnsymp,xtal)=real(nsymp)
       call ccpmvr(xtaldat(lrsym,xtal),rsym,96*16)
c .. res limits
       call lrrsol(xtal,smin,smax)
       xtaldat(lrmax,xtal)=max(xtaldat(lrmax,xtal),sqrt(1.0/smax))
       xtaldat(lrmin,xtal)=min(xtaldat(lrmin,xtal),sqrt(1.0/smin))
c .. starting fraction
       if (xtaldat(lfrom,xtal).ne.0.0) xtaldat(lfrac,xtal)=
     +   min((xtaldat(lrmax,xtal)/xtaldat(lfrom,xtal))**3,1.0)
c .. fill out all the data on this xtal
       call getxtal(xtal)
c .. so that we can set up the grid and hkl arrays
c ... grid
       nu=nint(xtaldat(lgrid  ,xtal))
       nv=nint(xtaldat(lgrid+1,xtal))
       nw=nint(xtaldat(lgrid+2,xtal))
       if (nu.eq.0) then
        call gridres(xtaldat(lrmax,xtal),nu,nv,nw)
        xtaldat(lgrid  ,xtal)=real(nu)
        xtaldat(lgrid+1,xtal)=real(nv)
        xtaldat(lgrid+2,xtal)=real(nw)
       endif
c ... number of reflections
       nrflc=nint(xtaldat(lnrflc,xtal))
       if (nrflc.eq.0) then
        nrflc=100+numres(xtaldat(lrmax,xtal))
        xtaldat(lnrflc,xtal)=real(nrflc)
       endif
c
c ... open mtz output file
       iappnd = 1
       call lwopen(xtal,mtzout(xtal))
       call lwassn(xtal,lsprgo,nlprgo,ctprgo,iappnd)
c
c ... check for input and output solvent masks
       call ugtenv(solin (xtal),filnam)
       if (filnam.ne.' ') xtaldat(lsolin ,xtal)=1.0
       call ugtenv(solout(xtal),filnam)
       if (filnam.ne.' ') xtaldat(lsolout,xtal)=1.0
c
c ... maximum resolution (for mask)
       maxres=min(maxres,xtaldat(lrmax,xtal))
 300  continue
c
c open the masks and get the mask cells and grids
c
      maxres=maxres/2.2
      do 400 domn=1,ndomn
       call ccpmskhead
     +   (mskin(domn),domndat(ldcell,domn),domndat(ldlim,domn))
       mu=nint(domndat(lcell  ,domn)/maxres)
       mv=nint(domndat(lcell+1,domn)/maxres)
       mw=nint(domndat(lcell+2,domn)/maxres)
       domndat(lmgrid  ,domn)=real(mu)
       domndat(lmgrid+1,domn)=real(mv)
       domndat(lmgrid+2,domn)=real(mw)
       mu1=nint(mu*domndat(ldlim  ,domn)-1.0)
       mu2=nint(mu*domndat(ldlim+1,domn)+1.0)
       mv1=nint(mv*domndat(ldlim+2,domn)-1.0)
       mv2=nint(mv*domndat(ldlim+3,domn)+1.0)
       mw1=nint(mw*domndat(ldlim+4,domn)-1.0)
       mw2=nint(mw*domndat(ldlim+5,domn)+1.0)
       domndat(lmlim  ,domn)=mu1
       domndat(lmlim+1,domn)=mu2
       domndat(lmlim+2,domn)=mv1
       domndat(lmlim+3,domn)=mv2
       domndat(lmlim+4,domn)=mw1
       domndat(lmlim+5,domn)=mw2
       call calcmetric(domndat(ldcell,domn),
     +   vol,q,qt,rcm11,rcm12,rcm13,rcm22,rcm23,rcm33,
     +            rlm11,rlm12,rlm13,rlm22,rlm23,rlm33)
       call symcpy(q,domndat(ldmatq,domn))
       call symcpy(qt,domndat(ldmatqt,domn))
 400  continue
c
c done for now
c
      return
 9010 call ccperr(1,' RCARDS - error opening input mtz file')
      end
c
c ------------------------------------------------------------------------
c
      subroutine sizes(uvwsize,hklsize,domsize,uvwmax,hklmax,dommax)
c SIZES - work out the number we need to set up the array sizes
      include 'dmmulti.fh'
c the multi-xtal datablock:
      include 'xtalinfo.fh'
c
      integer uvwsize(maxxtal),hklsize(maxxtal),domsize(maxdomn)
      integer uvwmax,hklmax,dommax
c
      integer xtal,domn,lu,lv,lw
      real rmax
c
      uvwmax=0
      hklmax=0
      dommax=0
      rmax=1000.0
c
c first size the xtal arrays:
c  .. one array for each hkl, and n arrays of size uvwmax
c
      do 100 xtal=1,nxtal
c xtal datablock sizes
       uvwsize(xtal)=nint(xtaldat(lgrid,xtal))*
     +   nint(xtaldat(lgrid+1,xtal))*(nint(xtaldat(lgrid+2,xtal)+2))
       uvwmax=max(uvwmax,uvwsize(xtal))
       hklsize(xtal)=nhklc*nint(xtaldat(lnrflc,xtal))
       hklmax=max(hklmax,hklsize(xtal))
c and get maximum res for domain grid
       rmax=max(rmax,xtaldat(lrmax,xtal))
 100  continue
c
c now size the domain arrays
c  .. one for each domain, at res > rmax(xtal)
c
      rmax=2.2*rmax
      do 200 domn=1,ndomn
       lu=nint(domndat(lmlim+1,domn))-nint(domndat(lmlim  ,domn))+1
       lv=nint(domndat(lmlim+3,domn))-nint(domndat(lmlim+2,domn))+1
       lw=nint(domndat(lmlim+5,domn))-nint(domndat(lmlim+4,domn))+1
       domsize(domn)=lu*lv*lw
 200  continue
c
      return
      end
c
c ------------------------------------------------------------------------
c
      subroutine getxtal(xtal)
c GETXTAL - get all the information concerning a single crystal form
      include 'dmmulti.fh'
      include 'io.fh'
c the multi-xtal datablock:
      include 'xtalinfo.fh'
c the particular xtal datablocks:
      include 'xcrystal.fh'
      include 'xtlparam.fh'
      include 'xhklinfo.fh'
      include 'xavginfo.fh'
c
      integer xtal,i,j,idum,ifail
c
      character*20 namspg,nampg,namlau
c
c
c Notes: The zero`th element of the jpoint list, the zero`th element of
c        the srcdom list and the zero`th 16 elements of the srcmat list
c        are not used.
c
      thisxtal=xtal
      write (lpt,100)thisxtal
 100  format ('XTAL ',i1,' ----------------------------------------')
c
c get all the xtal info from xtaldat
      msolv=nint(xtaldat(lmsolv,xtal))
      mhist=nint(xtaldat(lmhist,xtal))
      maver=nint(xtaldat(lmaver,xtal))
      mskel=nint(xtaldat(lmskel,xtal))
      msayr=nint(xtaldat(lmsayr,xtal))
      wmag=xtaldat(lwmag,xtal)
      wfom=xtaldat(lwmag,xtal)
      wres=xtaldat(lwmag,xtal)
      fracinit=xtaldat(lfrac,xtal)
      rmin=xtaldat(lrmin,xtal)
      rmax=xtaldat(lrmax,xtal)
      solc=xtaldat(lsolc,xtal)
      weight=xtaldat(lweight,xtal)
      avesol=xtaldat(lsolmean,xtal)
      aveprt=xtaldat(lprtmean,xtal)
      scmask=xtaldat(lsolcmsk,xtal)
      pcmask=xtaldat(lprtcmsk,xtal)
      wangrd=xtaldat(lwangrd,xtal)
      wangmd=nint(xtaldat(lwangmd,xtal))
      wngmin=xtaldat(lwngmin,xtal)
      wngmax=xtaldat(lwngmax,xtal)
      scale=xtaldat(lscale,xtal)
      bfac=xtaldat(lscale+1,xtal)
      ljoin=xtaldat(lskjoin,xtal)
      lendp=xtaldat(lskendp,xtal)
      sklbf=xtaldat(lskbfac,xtal)
      nncs=nint(xtaldat(lnncs,xtal))
      refdr=xtaldat(lrefdr,xtal)
      refdphi=xtaldat(lrefdp,xtal)
      do 120 j=1,nncs
       srcdom(j)=nint(xtaldat(lsrcdom+j,xtal))
       call symcpy(xtaldat(lsrcncs+16*j,xtal),rsymncs(1,1,j))
 120  continue
      call ccpmvr(cell,xtaldat(lcell,xtal),6)
      call ccpmvr(jpoint,xtaldat(ljpoint,xtal),mcols)
c (the following may be blank on the first call..)
      nu=nint(xtaldat(lgrid  ,xtal))
      nv=nint(xtaldat(lgrid+1,xtal))
      nw=nint(xtaldat(lgrid+2,xtal))
      nrflc=nint(xtaldat(lnrflc,xtal))
      nrflo=nint(xtaldat(lnrflo,xtal))
      call symcpy(xtaldat(lmatq,xtal),q)
      call symcpy(xtaldat(lmatqt,xtal),qt)
      vol=xtaldat(lvol,xtal)
      rcm11=xtaldat(lrcm  ,xtal)
      rcm12=xtaldat(lrcm+1,xtal)
      rcm13=xtaldat(lrcm+2,xtal)
      rcm22=xtaldat(lrcm+3,xtal)
      rcm23=xtaldat(lrcm+4,xtal)
      rcm33=xtaldat(lrcm+5,xtal)
      rlm11=xtaldat(lrlm  ,xtal)
      rlm12=xtaldat(lrlm+1,xtal)
      rlm13=xtaldat(lrlm+2,xtal)
      rlm22=xtaldat(lrlm+3,xtal)
      rlm23=xtaldat(lrlm+4,xtal)
      rlm33=xtaldat(lrlm+5,xtal)
      nspgrp=nint(xtaldat(lspgrp,xtal))
      nlaue=nint(xtaldat(lnlaue,xtal))
      nsym=nint(xtaldat(lnsym,xtal))
      nsymp=nint(xtaldat(lnsymp,xtal))
      call ccpmvr(rsym,xtaldat(lrsym,xtal),96*16)
c
c Now generate the rest of the info
c
c resolution
      smin=1.0/rmin**2
      smax=1.0/rmax**2
c reflection limits
      hmax=int(cell(1)/rmax)
      kmax=int(cell(2)/rmax)
      lmax=int(cell(3)/rmax)
      hmin=-hmax
      kmin=-kmax
      lmin=0
c F(000) term
      f000=vol*(avesol*solc+aveprt*(1.0-solc))
c
c      write (xtal+10,*)rmin,rmax,smin,smax,f000,
c     +  nrflc,nrflo,hmin,kmin,lmin,hmax,kmax,lmax,jpoint,
c     +  solc,scmask,pcmask,aveprt,avesol,wangrd,wngmin,wngmax,
c     +  wmag,wfom,wres,fracinit,scale,bfac,ljoin,lendp,sklbf,
c     +  msayr,mhist,msolv,maver,mskel,mcode,wangmd,
c     +  nu,nv,nw,
c     +  cell,vol,q,qt,rcm11,rcm12,rcm13,rcm22,rcm23,rcm33,
c     +  rlm11,rlm12,rlm13,rlm22,rlm23,rlm33,nspgrp,nlaue,nsym,nsymp
c
      return
      end
c
c ------------------------------------------------------------------------
c
      subroutine getdomn(domn)
c GETDOMN - get parameters for a particular averaging domain
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xtalinfo.fh'
      include 'xdominfo.fh'
c
      integer domn
c
      mu=nint(domndat(lmgrid  ,domn))
      mv=nint(domndat(lmgrid+1,domn))
      mw=nint(domndat(lmgrid+2,domn))
      mu1=nint(domndat(lmlim  ,domn))
      mu2=nint(domndat(lmlim+1,domn))
      mv1=nint(domndat(lmlim+2,domn))
      mv2=nint(domndat(lmlim+3,domn))
      mw1=nint(domndat(lmlim+4,domn))
      mw2=nint(domndat(lmlim+5,domn))
      call symcpy(domndat(ldmatq,domn),dq)
      call symcpy(domndat(ldmatqt,domn),dqt)
      call ccpmvr(dcell,domndat(ldcell,domn),6)
      domnwt=domndat(ldomnwt,domn)
c
      return
      end
c
c ------------------------------------------------------------------------
c
      subroutine ccpmskhead(filnam,cell,uvwlim)
c CCPMSKHEAD - read ccp4 mask header and return some stuff
      include 'dmmulti.fh'
      include 'io.fh'
c
      real cell(6),uvwlim(6)
      integer mxyz(3),juvw(3),jfms(3),m1(3),m2(3),msec,mode
      integer i,ifail,iprint,nspgrp
      real rmin,rmax,rmean,rrms
      character title*80,filnam*(*)
c
      ifail=0
      iprint=0
      call mrdhds(lmsks,filnam,title,msec,jfms,mxyz,m1(3),m1(1),m2(1),
     +  m1(2),m2(2),cell,nspgrp,mode,rmin,rmax,rmean,rrms,ifail,iprint)
      m2(3)=m1(3)+msec-1
      call mrclos(lmsks)
c
      do 110 i=1,3
       juvw(jfms(i))=i
 110  continue
c
      uvwlim(1)=(real(m1(juvw(1)))-0.5)/real(mxyz(1))
      uvwlim(2)=(real(m2(juvw(1)))+0.5)/real(mxyz(1))
      uvwlim(3)=(real(m1(juvw(2)))-0.5)/real(mxyz(2))
      uvwlim(4)=(real(m2(juvw(2)))+0.5)/real(mxyz(2))
      uvwlim(5)=(real(m1(juvw(3)))-0.5)/real(mxyz(3))
      uvwlim(6)=(real(m2(juvw(3)))+0.5)/real(mxyz(3))
c
      return
      end
c
c ------------------------------------------------------------------------
c
      subroutine hklin(hkls,nrflx)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xcrystal.fh'
      include 'xhklinfo.fh'
c HKLIN - read columns from .mtz file
c
      integer nrflx
      real hkls(nhklc,0:*)
c
      logical eof,centre,sysabs,logmss(200)
      real adata(200),sh(nhklc),sh2(nhklc),hmx(nhklc),hmn(nhklc)
      character colnam(nhklc)*6
c
      integer in(3),nfile,nhires,nsysab,notasu,nunobs,ncen,ncenerr
      integer ih,ik,il,inasuold,i
      integer ifo,isfo,iphio,ifomo,ihla,ihlb,ihlc,ihld
      integer ifdm,iphidm,ifomdm,ifree
      real s,eps,phicen,fo,sfo,phio,fomo,fdm,phidm,fomdm,fpc,phipc
      real rrand,flag,hla,hlb,hlc,hld,x,atanh,siminv
      logical assigned(nhklc)
c
      external inasuold,rrand,atanh,siminv
c
      data colnam/'H','K','L','S','EPS','FO','PHIO','FOMO',
     +            'HL-A','HL-B','HL-C','HL-D',2*'x',
     +            'FDM','PHIDM','FOMDM',7*'x','FREE'/
c
      do 10 i=1,nhklc
       assigned(i)=(i.le.8)
       sh(i)=0.0
       sh2(i)=0.0
       hmx(i)=-1.0e6
       hmn(i)= 1.0e6
 10   continue
c
c set up pointers for input columns
      ifo=jpoint(4)
      isfo=jpoint(5)
      iphio=jpoint(6)
      ifomo=jpoint(7)
      ihla=jpoint(8)
      ihlb=jpoint(9)
      ihlc=jpoint(10)
      ihld=jpoint(11)
      ifdm=jpoint(12)
      iphidm=jpoint(13)
      ifomdm=jpoint(14)
      ifree=jpoint(15)
      assigned(lfdm)  =(ifdm.ne.0)  
      assigned(lphidm)=(iphidm.ne.0)
      assigned(lfomdm)=(ifomdm.ne.0)
      assigned(lfree) =(ifree.ne.0)
      assigned(lhla)=(ihla.ne.0)
      assigned(lhlb)=(ihlb.ne.0)
      assigned(lhlc)=(ihlc.ne.0)
      assigned(lhld)=(ihld.ne.0)
c
      nfile=0
      nrflo=0
      nhires=0
      nsysab=0
      notasu=0
      nunobs=0
      ncen=0
      ncenerr=0
c
c
 100    call lrrefl(thisxtal,s,adata,eof)
        if (eof) goto 200
        call lrrefm(thisxtal,logmss)
        do 101 i=1,200
         if (logmss(i)) adata(i)=0.0
 101    continue
        nfile=nfile+1
        ih=nint(adata(1))
        ik=nint(adata(2))
        il=nint(adata(3))
        in(1)=ih
        in(2)=ik
        in(3)=il
c Get epsilon zone and centric flag
        call epscen(ih,ik,il,centre,sysabs,eps,phicen,nsym,rsym)
        if (centre) eps=-eps
c Get standard deviation of F - zero for unobserved
        sfo=1.0
        if (isfo.ne.0) sfo=adata(isfo)
c reject reflections
        if (s.lt.smin.or.s.gt.smax) then
          nhires=nhires+1
          goto 100
        endif
        if (sysabs) then
          nsysab=nsysab+1
          goto 100
        endif
        if (inasuold(in,nlaue).ne.1) then
          notasu=notasu+1
          goto 100
        endif
        if (sfo.eq.0.0) then
          nunobs=nunobs+1
          goto 100
        endif
        nrflo=nrflo+1
        if (nrflo.gt.nrflc) goto 9000
c Get reflection data ....
        fo=adata(ifo)
        phio=0.0
        fomo=0.0
        if (iphio.ne.0) phio=dtor*adata(iphio)
        if (ifomo.ne.0) fomo=adata(ifomo)
c get starting points
        fdm=fo
        phidm=phio
        fomdm=fomo
        if (ifdm.ne.0)   fomdm=1.0
        if (ifdm.ne.0)   fdm=adata(ifdm)
        if (iphidm.ne.0) phidm=dtor*adata(iphidm)
        if (ifomdm.ne.0) fomdm=adata(ifomdm)
c Get free-r flag - 1...20 for 5% free-r sets
c if there is a free-r set on file then those get 1.0
        if (ifree.ne.0) then
cv         flag=adata(ifree)*real(int(rrand()*19.0)+1)+1.0
         flag=adata(ifree)
        else
         flag=real(int(rrand()*20.0)+1)
        endif
c Get H-L coeffs
        if (ihla.ne.0) then
          hla=adata(ihla)
          hlb=adata(ihlb)
          hlc=adata(ihlc)
          hld=adata(ihld)
          if (iphio.eq.0.or.ifomo.eq.0) then
           if (centre) then
            call probcen(hla,hlb,phio,fomo)
           else
            call probhl(hla,hlb,hlc,hld,phio,fomo)
           endif
          endif
        else
          if (centre) then
            x=atanh(fomo)
          else
            x=siminv(fomo)
          endif
          hla=x*cos(phio)
          hlb=x*sin(phio)
          hlc=0.0
          hld=0.0
        endif
c correct centric phases and count erroneous phases ...
        if (centre) then
          ncen=ncen+1
	    if (iphio.ne.0 .and. (cos(2.0*(phio-phicen)).lt.0.99)) then
              if (.not.logmss(iphio)) ncenerr=ncenerr+1
	    endif
	    if (iphio.eq.0 .and. ihla.ne.0 .and.
     +	  (cos(2.0*(phio-phicen)).lt.0.99)) then
              if (.not.logmss(ihla)) ncenerr=ncenerr+1
	    endif
c
          phio=nint((phio-phicen)/pi)*pi+phicen
        endif
c store in reflection array
        hkls(1,nrflo)=adata(1)
        hkls(2,nrflo)=adata(2)
        hkls(3,nrflo)=adata(3)
        hkls(4,nrflo)=s
        hkls(5,nrflo)=eps
        hkls(lfo,nrflo)=fo
        hkls(lphio,nrflo)=phio
        hkls(lfomo,nrflo)=fomo
        hkls(lhla,nrflo)=hla
        hkls(lhlb,nrflo)=hlb
        hkls(lhlc,nrflo)=hlc
        hkls(lhld,nrflo)=hld
        hkls(lfdm,nrflo)=fdm
        hkls(lphidm,nrflo)=phidm
        hkls(lfomdm,nrflo)=fomdm
        hkls(lfree,nrflo)=flag
c
c accumulate some stats
        do 190 i=1,nhklc
         if (assigned(i)) then
          sh(i)=sh(i)+hkls(i,nrflo)
          sh2(i)=sh2(i)+hkls(i,nrflo)**2
          hmx(i)=max(hmx(i),hkls(i,nrflo))
          hmn(i)=min(hmn(i),hkls(i,nrflo))
         endif
 190    continue
c next reflection
        goto 100
c
c store F(000) term
 200  hkls(lfo,0)=f000
c write out some information
      do 205 i=1,nhklc
       sh(i)=sh(i)/nrflo
       sh2(i)=sh2(i)/nrflo
 205  continue
c
      write (lpt,210)nfile,nhires,nsysab,notasu,nunobs,nrflo
 210  format (/' Number of reflections on file:   ',i6
     +        /' Number outside resolution limit: ',i6
     +        /' Number of systematic absences:   ',i6
     +        /' Number outside asymmetric unit:  ',i6
     +        /' Number of unobserved reflections:',i6
     +        /' Number of reflections used:      ',i6)
c
      write (lpt,215)ncen,ncenerr
 215  format (/' Number of centric reflections:                   ',i6
     +        /' Number of centric phases more than 4 degrees out:',i6)
      if(float(ncen)/ncenerr .gt. 0.2) call ccperr(2,
     + ' Too many centric reflections differ from the expected value')
c
      write (lpt,220)
 220  format (/' Program data read:'
     +  /' Column:      MIN       MAX       MEAN X    MEAN X^2')
      do 240 i=1,nhklc
       if (assigned(i))
     +   write (lpt,230)colnam(i),hmn(i),hmx(i),sh(i),sh2(i)
 240  continue
 230  format (4x,a6,1x,f9.3,1x,f9.3,1x,f9.3,1x,f11.3)
c
      nrflx=nrflo
c
      return
 9000 call ccperr(1,' (HKLIN) Too many reflections - supply only ASU')
      end
c
c ----------------------------------------------------------------------
c
      subroutine hklout(hkls,lfa,lfb)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xtlparam.fh'
      include 'xhklinfo.fh'
c HKLOUT - write out modified data
c
      real hkls(nhklc,0:*)
      integer lfa,lfb
c
      real adata(200)
      logical eof
      integer ncols,nfile,nlist
      real s,sc
c
c
c rewind the input file and get the number of columns
      call lrrewd(thisxtal)
      call lrncol(thisxtal,ncols)
c
c write modified data to .mtz file
      nfile=1
      nlist=1
 100    call lrrefl(thisxtal,s,adata,eof)
        if (eof) goto 200
        if (nint(adata(1)).eq.nint(hkls(1,nlist)).and.
     +      nint(adata(2)).eq.nint(hkls(2,nlist)).and.
     +      nint(adata(3)).eq.nint(hkls(3,nlist))) then
          sc=sqrt(scale)*exp(bfac*s/4.0)
          adata(ncols+1)=hkls(lphidm,nlist)/dtor
          adata(ncols+2)=hkls(lfomdm,nlist)
          adata(ncols+3)=hkls(lfa,nlist)/sc
          adata(ncols+4)=hkls(lfb,nlist)/dtor
          nlist=nlist+1
        else
          adata(ncols+1)=0.0
          adata(ncols+2)=0.0
          adata(ncols+3)=0.0
          adata(ncols+4)=0.0
        endif
        nfile=nfile+1
        call lwrefl(thisxtal,adata)
        goto 100
c done write
 200  call lrclos(thisxtal)
      call lwclos(thisxtal,1)
c
      write (lpt,220)nfile-1,nlist-1
 220  format (/' Number of reflections on file:  ',i6/
     +         ' Number of reflections modified: ',i6)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine lptout()
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xavginfo.fh'
c produce log file output for end of run
      integer i,j,k
c
      write (lpt,110)
      do 100 k=1,nncs
       write (lpt,120)srcdom(k),
     +   ((rsymncs(i,j,k),j=1,3),i=1,3),(rsymncs(i,4,k),i=1,3)
 100  continue
c
 110  format (/'FINAL AVERAGING MATRICES:'/)
 120  format ('AVER DOMAIN ',i3/'ROTA MATR ',3f9.5,' -'/
     +  '          ',3f9.5,' -'/'          ',3f9.5/
     +  'TRANS     ',3f9.2)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine calcscale(hkls,scalex,bfacx)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xcrystal.fh'
      include 'xtlparam.fh'
      include 'xhklinfo.fh'
c SCALEF - calculate the scale of the input data
c uses columns h,k,l,s,eps,fo to calculate bfac,scale
c
      real hkls(nhklc,0:*),scalex,bfacx
c
      real res(50),std(50),rres(50),sres(50),rstd(50)
      real ncal(50),f2cal(50),nobs(50),f2obs(50)
      integer ndatares
c
      integer i,j,nr,nres,nrstep,numres
      real srmin,srmax,varprt,varsol,rhoave,rhovar,s,sc
      real x,y1,y2,w,sw,sx,sx2,sdy,sdyx
c
      external numres
c data from ins
      data ndatares/19/
      data res/999.00,10.00, 8.00, 7.00, 6.00, 5.00, 4.50, 4.00, 3.60,
     +     3.30, 3.00, 2.80, 2.60, 2.40, 2.20, 2.00, 1.80, 1.60, 1.50,
     +     31*0.0/
      data std/ 0.000,0.076,0.107,0.121,0.143,0.200,0.248,0.316,0.375,
     +    0.427,0.476,0.512,0.552,0.599,0.654,0.718,0.783,0.850,0.890,
     +     31*0.0/
c
c first pick the resolution ranges
c ================================
      nrstep=25
      rres(1)=0.0
      rstd(1)=0.0
      nres=0
      nr=0
      do 100 i=2,ndatares
        srmin=1.0/res(i-1)**2
        srmax=1.0/res(i)**2
        do 50 j=1,nrflo
          if (hkls(4,j).ge.srmin.and.hkls(4,j).lt.srmax) nr=nr+1
 50     continue
        if (nr.gt.nrstep) then
          nr=0
          nres=nres+1
          rres(nres)=res(i)
          rstd(nres)=std(i)
        endif
 100  continue
c
c calculate estimated variances with resolution for this structure
c ================================================================
      do 110 i=1,nres
c pick up variance of protein region (res.dep.) and solvent (almost flat)
      varprt=rstd(i)**2
      varsol=varprt/10.0
c now calc overall mean and variance
      rhoave=aveprt*(1.0-solc)+avesol*solc
      rhovar=(varprt+(aveprt-rhoave)**2)*(1.0-solc) +
     +       (varsol+(avesol-rhoave)**2)*solc
c store expected data sizes
      ncal(i)=numres(rres(i))
      f2cal(i)=rhovar*vol**2/(2.0*nsym)
 110  continue
c now do differences for individual ranges
      do 120 i=nres,2,-1
      ncal(i)=ncal(i)-ncal(i-1)
      f2cal(i)=f2cal(i)-f2cal(i-1)
 120  continue
c
c now produce obs data stats for each resolution range
c ====================================================
      call calcsobs(hkls,nres,rres,sres,nobs,f2obs)
      if (scale.ne.0.0) goto 350
      if (rmax.gt.res(5)) goto 9010
c
c calculate scale and B-factor to produce best fit between obs and calc
c =====================================================================
      sw=0.0
      sx=0.0
      sx2=0.0
      sdy=0.0
      sdyx=0.0
      do 300 i=1,nres
        x=sres(i)/nobs(i)
        y1=log(f2cal(i)/ncal(i))
        y2=log(f2obs(i)/nobs(i))
c Empirical weighting scheme:
c uniform weight gives too little influence to high res data.
c (but there are more ranges at high res)
c weighting with number of reflections give too much influence to high res data.
c (but taking into account the SD`s might put it right)
        w=1.0
        sw=sw+w
        sx=sx+w*x
        sx2=sx2+w*x**2
        sdy=sdy+w*(y2-y1)
        sdyx=sdyx+w*(y2-y1)*x
 300  continue
      scale=exp((sx2*sdy-sx*sdyx)/(sx**2-sw*sx2))
      bfac=(sw*sdyx-sx*sdy)/(sx**2-sw*sx2)*2.0
c
c rescale the input data to match the calc distributions
c ======================================================
 350  do 400 i=1,nrflo
        s=hkls(4,i)
        sc=sqrt(scale)*exp(bfac*s/4.0)
        hkls(lfo,i) =sc*hkls(lfo,i)
        hkls(lfdm,i)=sc*hkls(lfdm,i)
 400  continue
c
c produce stats for rescaled obs data
c ===================================
      call calcsobs(hkls,nres,rres,sres,nobs,f2obs)
c
c output data
c ===========
      scalex=scale
      bfacx=bfac
      write (lpt,500)scale,bfac
 500  format (//
     +' Input data analysis:'/
     +'   Estimated intensity scale factor = ',f10.3/
     +'   Estimated temperature factor     = ',f10.3)
      write (lpt,510)
 510  format (/
     +' ------------------ xloggraph ------------------'/
     +' $TABLE: Analysis of input structure factors :'/
     +' $GRAPHS: Data completeness with resolution :N:2,4:'/
     +'        : Mean F^2 with resolution          :N:2,5,6:'/
     +' $$'/
     +'  res   4sin2(th)  number    %present      expect.   obs. $$'/
     +' (lim)  /lambda2  (reflns) (completeness)    (mean F^2)   $$')
      do 520 i=1,nres
      write (lpt,530)rres(i),sres(i)/nobs(i),
     +  nobs(i),100.0*nobs(i)/ncal(i),f2cal(i)/ncal(i),f2obs(i)/nobs(i)
 520  continue
 530  format(1x,f5.2,2x,f6.3,4x,f7.0,6x,f6.1,2x,2f9.0)
      write (lpt,540)
 540  format (' $$'/' ------------------ xloggraph ------------------')
c
      return
 9010 call ccperr(1,
     +  ' (CALCSCALE) - insufficent resolution: enter a SCALE card')
      end
c
c ----------------------------------------------------------------------
c
      subroutine calcsobs(hkls,nres,rres,sres,nobs,f2obs)
      include 'dmmulti.fh'
      include 'xhklinfo.fh'
c CALCSOBS - accumulate some statistics on observed data for CALCSCALE
c
      real hkls(nhklc,0:*)
      real rres(*),sres(*),nobs(*),f2obs(*)
c
      integer i,j,ir,nres
      real s,eps,fo
c
      do 140 i=1,nres
      sres(i)=0.0
      nobs(i)=0.0
      f2obs(i)=0.0
 140  continue
c
      do 200 j=1,nrflo
        s=hkls(4,j)
        eps=abs(hkls(5,j))
        fo=hkls(lfo,j)
        do 150 ir=1,nres-1
         if (s.lt.1.0/rres(ir)**2) goto 160
 150    continue
 160    nobs(ir)=nobs(ir)+1.0
        f2obs(ir)=f2obs(ir)+fo**2/eps
        sres(ir)=sres(ir)+s
 200  continue
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine wfo(hkls,lf,lphi,meanfom)
      include 'dmmulti.fh'
      include 'xhklinfo.fh'
c calculate initial map coefficients from input data and copy into
c fdm and phidm columns. Meanfom is weight of current map.
      real hkls(nhklc,0:*),meanfom
      integer lf,lphi
c
      integer i
c
      hkls(lf,0)=hkls(lfo,0)
      hkls(lphi,0)=0.0
      meanfom=1.0
c
      do 100 i=1,nrflo
       if (hkls(lflag,i).gt.0.0) then
        hkls(lf,i)=hkls(lfdm,i)*hkls(lfomdm,i)
        hkls(lphi,i)=hkls(lphidm,i)
        meanfom=meanfom+hkls(lfomdm,i)
       else
        hkls(lf,i)=0.0
        hkls(lphi,i)=0.0
       endif
 100  continue
c
      meanfom=meanfom/real(nrflo)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine setfreer(hkls,icfr,ncfr,icycl)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xhklinfo.fh'
c SETFREER - pick an excluded set of reflections for free R-factor calc
c
      real hkls(nhklc,0:*)
      integer icfr,ncfr,icycl
c
      integer i,ipick,ninc,nexc,nincfr,nexcfr,lmod
c
      external lmod
c
c pick a fraction of the non-centric reflections
      if (ncfr.gt.1.and.icfr.eq.ncfr) then
       ipick=-1
      else if (ncfr.eq.1) then
       ipick=mod(icycl,20)+1
      else
       ipick=icfr
      endif
      nexc=0
      ninc=0
      nexcfr=0
      nincfr=0
      do 100 i=1,nrflo
c remove old flags
       hkls(lflag,i)=real(lmod(nint(hkls(lflag,i)),2))
c pick a validation set and flag them
       if (hkls(lflag,i).eq.0) nexc=nexc+1
       if (hkls(lflag,i).eq.1) ninc=ninc+1
       if (nint(hkls(lfree,i)).eq.ipick) then
        if (hkls(lflag,i).eq.0) nexcfr=nexcfr+1
        if (hkls(lflag,i).eq.1) nincfr=nincfr+1
        hkls(lflag,i)=hkls(lflag,i)-2.0
       endif
c next reflection
 100  continue
c
      write (lpt,200)nincfr,ninc,nexcfr,nexc,nincfr+nexcfr,ninc+nexc
 200  format (
     +  '  Included reflections used for Free-R: ',i7,' of ',i7/
     +  '  Excluded reflections used for Free-R: ',i7,' of ',i7/
     +  '     Total reflections used for Free-R: ',i7,' of ',i7/)
c
      return
      end
c
c ----------------------------------------------------------------------
c
