C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
c
c ----------------------------------------------------------------------
c
      subroutine dm(hkls,rho,rhodm,solmsk,reseff,print)
      include 'dmmulti.fh'
      include 'xtlparam.fh'
      include 'cyclsync.fh'
c DM - subroutine to do pure real-space density modification.
c rho is input map, rhodm is modified map
      real hkls(*),rho(*),rhodm(*),reseff
      integer solmsk(*)
      logical print
c
      call copy3d(rho,rhodm)
c .. skeletonisation
      if (mskel .eq.1.and.mod(icycl,ncskl).eq.0) then
       call itskel(hkls,rhodm,rho,solmsk,reseff)
       return
      endif
c .. solvent flattening
      if (msolv.eq.1) call solvflat(rhodm,solmsk)
c .. histogram mapping
      if (mhist.eq.1) call histmap(rhodm,solmsk,reseff,print)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine wangmask(map,solmsk,print)
      include 'dmmulti.fh'
      include 'xtlparam.fh'
c WANGMASK - mask a solvent mask by Wang's method
      real map(nu,nv,nw)
      integer solmsk(nu,nv,nw)
      logical print
c
      integer i,j,k
      external wangav
c
c make a mask according to Wang's method
c first truncate the density to the solvent level
      do 100 k=1,nw
      do 100 j=1,nv
      do 100 i=1,nu
      map(i,j,k)=min(max(map(i,j,k),wngmin),wngmax)
      solmsk(i,j,k)=0
 100  continue
c now perform local averaging (by ffts)
      call setwangav(wangrd,wangmd)
      call inv3d(map)
      call forallrfl(wangav,map)
      call fft3d(map)
c produce a mask from the high density in the averaged map
      if (scmask.ge.0.0) then
       call maskmake(map,solmsk,1.0-scmask,-1)
       call maskmake(map,solmsk,pcmask,1)
      else
       call maskmake(map,solmsk,1.0-solc,1)
      endif
c
      if (print) call maskstats(solmsk,nu,nv,nw,0)
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine setwangav(rdwang,mdwang)
      include 'dmmulti.fh'
c SETWANGAV - set up wang weighting
      common /wangpar/rwang,mwang
      save   /wangpar/
      integer mdwang,mwang
      real    rdwang,rwang
      rwang=rdwang
      mwang=mdwang
      return
      end
c
      subroutine wangav(ih,ik,il,s,fh)
      include 'dmmulti.fh'
c WANGAV - apply wang weights in reciprocal space to perform
c smoothing of the map in real space
      common /wangpar/rwang,mwang
      save   /wangpar/
      integer ih,ik,il,mwang
      real s,wwt,recipwt,rwang
      complex fh
      external recipwt
      wwt=recipwt(sqrt(s),rwang,mwang)
      fh=wwt*fh
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine maskmake(map,msk,frac,flag)
      include 'dmmulti.fh'
      include 'xtlparam.fh'
c MASKMAKE - make a mask by selecting all density above a value chosen
c to include a specified fraction of the cell.
c
      real map(nu,nv,nw),frac
      integer msk(nu,nv,nw),flag
c
      integer nhist
      parameter (nhist=1000)
      real hist(nhist)
      integer i,j,k,ih
      real rhomax,rhomin,dr,finc,fcut,rcut
c
      do 10 i=1,nhist
      hist(i)=0.0
 10   continue
c find bounds of quantity in map
      rhomax=map(1,1,1)
      rhomin=map(1,1,1)
      do 100 k=1,nw
      do 100 j=1,nv
      do 100 i=1,nu
      rhomax=max(map(i,j,k),rhomax)
      rhomin=min(map(i,j,k),rhomin)
 100  continue
c accumulate histogram
      dr=(rhomax-rhomin)/(nhist-1)
      do 200 k=1,nw
      do 200 j=1,nv
      do 200 i=1,nu
      ih=int((map(i,j,k)-rhomin)/dr)+1
      hist(ih)=hist(ih)+1.0
 200  continue
c now find cutoff
      fcut=(1.0-frac)*real(nu*nv*nw)
      finc=hist(1)
      do 300 i=2,nhist-1
      finc=finc+hist(i)
      if (finc.gt.fcut) goto 310
 300  continue
 310  rcut=(real(i)-(finc-fcut)/hist(i))*dr+rhomin
c
c cutoff found. now make the mask.
c
      do 400 k=1,nw
      do 400 j=1,nv
      do 400 i=1,nu
      if (map(i,j,k).gt.rcut) msk(i,j,k)=flag
 400  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine pdbmask(msk)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xcrystal.fh'
      include 'xtlparam.fh'
c PDBMASK - make a protein mask from coordinates in a .pdb file
c
      integer msk(nu,nv,nw)
c
      character*4 atnam,resnam,resno
      character*1 chnnam
c
      real x,y,z,u,v,w,u1,v1,w1,du,dv,dw,ratom,r2cut,r2,occ,b
      integer i,j,k,i1,j1,k1,i2,j2,k2,m,nread,nused,lmod
      integer iser,irs,is,iz,ifail,idum
      external lmod
c
      ratom=2.4
      r2cut=ratom**2
c
      do 100 k=1,nw
      do 100 j=1,nv
      do 100 i=1,nu
      msk(i,j,k)=0
 100  continue
c
c open coordinate file
      ifail=0
      call ccpdpn(latom,'XYZIN','READONLY','F',idum,ifail)
      if (ifail.ne.0) goto 9000
      call rbinit(latom)
c
      nread=0
      nused=0
c Read atomic coordinates, convert to fractional
 300    continue
        call rbrook(latom,iser,atnam,resnam,chnnam,irs,resno,is,x,y,z,
     +    occ,b,iz,0,0,0,0,*300,*500)
        nread=nread+1
c ignore solvent atoms (I hope)
        if (resnam.eq.'WAT'.or.resnam.eq.'HOH'.or.resnam.eq.'H2O')
     +    goto 300
c convert to fractional coordinates
        u=qt(1,1)*x+qt(1,2)*y+qt(1,3)*z+qt(1,4)
        v=qt(2,1)*x+qt(2,2)*y+qt(2,3)*z+qt(2,4)
        w=qt(3,1)*x+qt(3,2)*y+qt(3,3)*z+qt(3,4)
c loop through symmetry equivalents
        do 400 m=1,nsym
          u1=rsym(1,1,m)*u+rsym(1,2,m)*v+rsym(1,3,m)*w+rsym(1,4,m)
          v1=rsym(2,1,m)*u+rsym(2,2,m)*v+rsym(2,3,m)*w+rsym(2,4,m)
          w1=rsym(3,1,m)*u+rsym(3,2,m)*v+rsym(3,3,m)*w+rsym(3,4,m)
c fill in mask within 2A of atom
          i=nint(u1*nu)
          j=nint(v1*nv)
          k=nint(w1*nw)
          do 400 k1=k-4,k+4
          do 400 j1=j-4,j+4
          do 400 i1=i-4,i+4
            du=real(i1)/nu-u1
            dv=real(j1)/nv-v1
            dw=real(k1)/nw-w1
            r2=rlm11*du**2+rlm22*dv**2+rlm33*dw**2+
     +         rlm12*du*dv+rlm13*du*dw+rlm23*dv*dw
            i2=lmod(i1,nu)+1
            j2=lmod(j1,nv)+1
            k2=lmod(k1,nw)+1
            if (r2.lt.r2cut) msk(i2,j2,k2)=1
 400    continue
c next atom
        nused=nused+1
        go to 300
c
c end of .pdb file
c
 500  write (lpt,510)nread,nused
 510  format (' Number of atoms read: ',i6/
     +        ' Number of atoms used: ',i6/)
c end of .pdb file
c check the input mask
      call maskstats(msk,nu,nv,nw,0)
      return
c
 9000 call ccperr(1,' (PDBMASK) Error opening file XYZIN')
      end
c
c ----------------------------------------------------------------------
c
      subroutine solmskin(filename,solmsk,lmask,icycl)
      include 'dmmulti.fh'
      include 'xtlparam.fh'
c SOLMSKIN - read in a mask and expand by xtal sym
      integer solmsk(*),lmask(*)
      character filename*(*)
c
      integer mu,mv,mw,mu1,mv1,mw1,mu2,mv2,mw2,lu,lv,lw,icycl,iprint
c
      iprint=0
      if (icycl.eq.1) iprint=1
c read in input mask
      call ccpmskin(filename,lmask,mu1,mv1,mw1,mu2,mv2,mw2,lu,lv,lw,
     +  iprint)
c
c symmetry expand it (s/r for convenient array indexing)
      call solmsksym(solmsk,lmask,mu1,mv1,mw1,mu2,mv2,mw2,lu,lv,lw)
c
c display the mask
      if (icycl.eq.1) call maskstats(solmsk,nu,nv,nw,0)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine solmsksym
     +  (solmsk,lmask,mu1,mv1,mw1,mu2,mv2,mw2,lu,lv,lw)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xcrystal.fh'
      include 'xtlparam.fh'
c solmsksym - symmetry expand a mask and put it on the program grid
c
      integer mu1,mv1,mw1,mu2,mv2,mw2,lu,lv,lw
      integer solmsk(0:nu-1,0:nv-1,0:nw-1)
      integer lmask(mu1:mu2,mv1:mv2,mw1:mw2)
c
      integer iu,iv,iw,iu1,iv1,iw1,ju,jv,jw,mu0,mv0,mw0,m,lmod
      real u,v,w,u1,v1,w1
      external lmod
c
c loop through each point in the unit cell mask
      do 500 iw=0,nw-1
      w=real(iw)/nw
      do 500 iv=0,nv-1
      v=real(iv)/nv
      do 500 iu=0,nu-1
      u=real(iu)/nu
       solmsk(iu,iv,iw)=0
c try each sym to see if it maps back onto the input mask
       do 400 m=1,nsym
        call symuvw(rsym(1,1,m),u,v,w,u1,v1,w1)
        iu1=nint(nu*u1)
        iv1=nint(nv*v1)
        iw1=nint(nw*w1)
        mu0=lmod(iu1-mu1,nu)+mu1
        mv0=lmod(iv1-mv1,nv)+mv1
        mw0=lmod(iw1-mw1,nw)+mw1
c loop through all reapeats of this cell coordinate in the mask
        do 300 jw=mw0,mw2,nw
        do 300 jv=mv0,mv2,nv
        do 300 ju=mu0,mu2,nu
         if (lmask(ju,jv,jw).ne.0) then
          solmsk(iu,iv,iw)=1
         endif
 300    continue
 400   continue
 500  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine solmskout(filename,solmsk)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xcrystal.fh'
      include 'xtlparam.fh'
c SOLMSKOUT - output ccp4 solvent mask
      integer solmsk(*)
      character filename*(*)
c
      integer mu1,mv1,mw1,mu2,mv2,mw2
      real xyzlim(2,3)
c
      write (lpt,100)
 100  format (/' SOLVENT MASK OUTPUT')
c
      call setlim(nspgrp,xyzlim)
      if (xyzlim(1,1).lt.0.0)
     +  call ccperr(1,' ccpmskout - Space group not recognized')
c
      mu1=int(nu*xyzlim(1,1))
      mv1=int(nv*xyzlim(1,2))
      mw1=int(nw*xyzlim(1,3))
      mu2=int(nu*xyzlim(2,1))
      mv2=int(nv*xyzlim(2,2))
      mw2=int(nw*xyzlim(2,3))
c
      call ccpmskout(filename,solmsk,nu,nv,nw,mu1,mv1,mw1,mu2,mv2,mw2)
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine maskstats(mask,mu,mv,mw,jw)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xtlparam.fh'
c MASKSTATS - output a mask section to log file
c
      integer mu,mv,mw,mask(0:mu-1,0:mv-1,0:mw-1),jw
c
      integer iu,iv,iw,nmask
c
c calculate fraction of cell masked
      nmask=0
      do 100 iw=0,mw-1
      do 100 iv=0,mv-1
      do 100 iu=0,mu-1
      if (mask(iu,iv,iw).gt.0) nmask=nmask+1
cwr      if (mask(iu,iv,iw).gt.1.or.mask(iu,iv,iw).lt.0)
cwr     +  write (*,*)iu,iv,iw,mask(iu,iv,iw)
 100  continue
      write (lpt,110)jw,mw,100.0*nmask/(nu*nv*nw)
 110  format (' Section',i4,' of',i4/
     +  12x,'The masked region occupies ',f5.1,' % of the cell.')
c
c print a section
      call masksec(mask(0,0,jw),mu,mv)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine rhostats(map,solmsk)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xtlparam.fh'
c RHOSTATS - output some statistics on the electron density
c
      real map(nu,nv,nw)
      integer solmsk(nu,nv,nw)
c
      integer nhist
      real rhomin,rhomax,pmin,pmax
      parameter (nhist=100,rhomin=-5.0,rhomax=20.0,pmin=-0.9,pmax=2.4)
      integer hist(nhist,-1:1)
      integer i,j,k,l,n,ih,imin,imax,nr(-1:1)
      real s(-1:1),s2(-1:1),rmax(-1:1),rmin(-1:1),rho
c
      do 20 i=-1,1
       nr(i)=0
       s(i)=0.0
       s2(i)=0.0
       rmin(i)= 1000.0
       rmax(i)=-1000.0
       do 10 j=1,nhist
        hist(j,i)=0
 10    continue
 20   continue
c
      do 100 k=1,nw
      do 100 j=1,nv
      do 100 i=1,nu
       rho=map(i,j,k)
       ih=int(nhist*(rho-rhomin)/(rhomax-rhomin))
       ih=max(min(ih,nhist),1)
       l=solmsk(i,j,k)
       nr(l)=nr(l)+1
       s(l)=s(l)+rho
       s2(l)=s2(l)+rho**2
       rmax(l)=max(rmax(l),rho)
       rmin(l)=min(rmin(l),rho)
       hist(ih,l)=hist(ih,l)+1
 100  continue
c
      n=nu*nv*nw
      write (lpt,200)
      write (lpt,201)'Protein',(100.0*nr(1))/n,rmin(1),rmax(1),
     +  s(1)/nr(1),sqrt(max(s2(1)*nr(1)-s(1)**2,0.0))/nr(1)
      write (lpt,201)'Solvent',(100.0*nr(0))/n,rmin(0),rmax(0),
     +  s(0)/nr(0),sqrt(max(s2(0)*nr(0)-s(0)**2,0.0))/nr(0)
      if (nr(-1).ne.0) write (lpt,201)'Unknown',(100.0*nr(-1))/n,
     +  rmin(-1),rmax(-1),s(-1)/nr(-1),
     +  sqrt(max(s2(-1)*nr(-1)-s(-1)**2,0.0))/nr(-1)
 200  format(/9x,'% of cell   Min Rho   Max Rho   Mean Rho   Std.Dev.')
 201  format(1x,a7,': ',f8.1,'  ',f8.3,'  ',f8.3,'   ',f8.3,'   ',f8.3)
      imin=int(nhist*(pmin-rhomin)/(rhomax-rhomin))
      imax=int(nhist*(pmax-rhomin)/(rhomax-rhomin))
      write (lpt,205)
 205  format (/' Density       Number of counts:   Number of counts:'
     +        /' Range:        (Protein)           (Solvent)')
      do 220 i=imin,imax
      rho=(i+0.5)*(rhomax-rhomin)/nhist+rhomin
      write (lpt,210)rho,hist(i,1),hist(i,0)
 210  format(1x,f8.3,8x,i10,10x,i10)
 220  continue
      write (lpt,*)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine solvflat(map,solmsk)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xtlparam.fh'
c SOLVFLAT - FLATten the SOLVent a bit.
c
      real map(nu,nv,nw)
      integer solmsk(nu,nv,nw)
c
c
      integer i,j,k
c
      do 100 k=1,nw
      do 100 j=1,nv
      do 100 i=1,nu
      if (solmsk(i,j,k).eq.0) map(i,j,k)=avesol
 100  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine readhist()
      include 'dmmulti.fh'
      include 'io.fh'
c READHIST - read in histogram data file
c
      real hresl,hrmin,hrmax,hrave,hdata
      common /histograms/hresl(50),hrmin(50),hrmax(50),hrave(50),
     +  hdata(200,50),nresl,nhist
      save   /histograms/
c
      integer npars
      parameter (npars=50)
      real fvalue(npars)
      integer ibeg(npars),iend(npars),idec(npars),itype(npars)
      character line*600,cvalue(npars)*4
      integer i,j,ntok,idum,ifail,nresl,nhist
c
c
      call ccpdpn(lhist,'HISTLIB','READONLY','F',idum,ifail)
c read the first line
 10   read (lhist,90)line
      if (line(1:4).ne.'HIST') goto 9000
      ntok=-npars
      call parse(line,ibeg,iend,itype,fvalue,cvalue,idec,ntok)
      call gtpint(2,nresl,ntok,itype,fvalue)
      call gtpint(3,nhist,ntok,itype,fvalue)
      write (lpt,12)nresl,nhist
 12   format (/' HISTLIB opened:'
     +/'  Number of histograms            =',i5
     +/'  Number of entries per histogram =',i5)
c now read the keyword lines
 20   read (lhist,90)line
      ntok=-npars
      call parse(line,ibeg,iend,itype,fvalue,cvalue,idec,ntok)
      if (line(1:4).eq.'RESL') call ccpmvi(hresl,fvalue(2),nresl)
      if (line(1:4).eq.'RMAX') call ccpmvi(hrmax,fvalue(2),nresl)
      if (line(1:4).eq.'RMIN') call ccpmvi(hrmin,fvalue(2),nresl)
      if (line(1:4).eq.'RAVE') call ccpmvi(hrave,fvalue(2),nresl)
      if (line(1:4).ne.'DATA') goto 20
c now read the histograms
      do 50 i=1,nhist
      read (lhist,*)(hdata(i,j),j=1,nresl)
 50   continue
c
 90   format (a)
c
      close (unit=lhist,status='keep')
      write (lpt,92)(hresl(i),i=1,min(nresl,20))
      write (lpt,*)
 92   format (' RESL ',20f5.1)
      return
 9000 call ccperr(1,' READHIST - error in histogram data file')
      end
c
c ----------------------------------------------------------------------
c
      subroutine histmap(map,solmsk,hmres,print)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xtlparam.fh'
c HISTMAP - perform histogram mapping
c
      real map(nu,nv,nw),hmres
      integer solmsk(nu,nv,nw)
      logical print
c
      real hresl,hrmin,hrmax,hrave,hdata
      common /histograms/hresl(50),hrmin(50),hrmax(50),hrave(50),
     +  hdata(200,50),nresl,nhist
      save   /histograms/
c
      real hobs(200),hcal(200),hmod(200),cobs(200),ccal(200),rtran(200)
      integer i,j,k,nr,nrh,nrm,jres,ih,ihm,nresl,nhist
      real hmres1,sr,sr2,srh,srm,srh2,srm2,dj,r,dhr,dr,robs,rcal,rh,dh
      real rhomax,rhomin,hrhomax,hrhomin,hrav,hrst,hmav,hmst
c
      hmres1=hmres
      nr=0
      nrh=0
      nrm=0
      sr=0.0
      srh=0.0
      srm=0.0
      sr2=0.0
      srh2=0.0
      srm2=0.0
      rhomax=-1000.0
      rhomin=1000.0
      do 100 i=1,nhist
      hobs(i)=0.0
      hmod(i)=0.0
 100  continue
c pick the best resolution ranges for the Fcalc histograms
      do 150 jres=1,nresl
      if (hresl(jres).lt.hmres1) goto 160
 150  continue
 160  if (jres.eq.1) then
      jres=2
      dj=0.0
      else if (jres.eq.nresl+1) then
      jres=nresl
      dj=1.0
      else
      dj=(hmres1-hresl(jres-1))/(hresl(jres)-hresl(jres-1))
      endif
c make interpolated histogram from two calculated histograms
      do 320 i=1,nhist
      rcal=(i-0.5)*(hrmax(jres)-hrmin(jres))/(nhist-1)+hrmin(jres)
      hcal(i)=(1.0-dj)*hdata(i,jres-1)+dj*hdata(i,jres)
      nrh=nrh+hcal(i)
      srh=srh+hcal(i)*rcal
      srh2=srh2+hcal(i)*rcal**2
 320  continue
c now calc the scale of the Fcalc histogram
      hrav=srh/nrh
      hrst=sqrt(srh2*nrh-srh**2)/nrh
      hrhomax=hrmax(jres)
cc *aveprt/hrav
      hrhomin=hrmin(jres)
cc *aveprt/hrav
      dhr=(hrhomax-hrhomin)/(nhist-1)
c compile histogram stats for observed data
      do 200 k=1,nw
      do 200 j=1,nv
      do 200 i=1,nu
      if (solmsk(i,j,k).ne.1) goto 200
      nr=nr+1
      sr=sr+map(i,j,k)
      sr2=sr2+map(i,j,k)**2
      rhomax=max(rhomax,map(i,j,k))
      rhomin=min(rhomin,map(i,j,k))
 200  continue
      dr=(rhomax-rhomin)/(nhist-1)
      do 300 k=1,nw
      do 300 j=1,nv
      do 300 i=1,nu
      if (solmsk(i,j,k).ne.1) goto 300
      ih=int((map(i,j,k)-rhomin)/dr)+1
      hobs(ih)=hobs(ih)+1.0
 300  continue
c put on a scale of fractions of the unit cell
      do 360 i=1,nhist
      hobs(i)=hobs(i)/nr
      hcal(i)=hcal(i)/nrh
 360  continue
c generate cumulative density distributions
      cobs(1)=0.0
      ccal(1)=0.0
      cobs(nhist)=1.0
      ccal(nhist)=1.0
      do 400 i=2,nhist-1
      cobs(i)=cobs(i-1)+hobs(i-1)
      ccal(i)=ccal(i-1)+hcal(i-1)
 400  continue
c generate the density transformation list
      do 500 i=1,nhist
      robs=(i-1)*dr+rhomin
      do 480 j=2,nhist-1
      if (ccal(j).ge.cobs(i)) goto 490
 480  continue
 490  rcal=(j-2+(cobs(i)-ccal(j-1))/max(ccal(j)-ccal(j-1),1.0e-6))
     +  *dhr+hrhomin
      rtran(i)=rcal
 500  continue
c now transform the density
      do 600 k=1,nw
      do 600 j=1,nv
      do 600 i=1,nu
      if (solmsk(i,j,k).ne.1) goto 600
      robs=map(i,j,k)
      rh=(robs-rhomin)/dr+1.0
      ih=int(rh)
      dh=rh-ih
      ih=max(min(ih,nhist-1),1)
      rcal=rtran(ih)+dh*(rtran(ih+1)-rtran(ih))
      map(i,j,k)=rcal
      ihm=(rcal-hrhomin)/dhr+1
      ihm=max(min(ihm,nhist-1),1)
      hmod(ihm)=hmod(ihm)+1.0
      nrm=nrm+1
      srm=srm+rcal
      srm2=srm2+rcal**2
 600  continue
      hmav=srm/nrm
      hmst=sqrt(srm2*nrm-srm**2)/nrm
c
      if (.not.print) return
c
c Now write out some samples to prove it has worked
      write (lpt,850)
 850  format (/' HISTOGRAM MAPPING:')
      write (lpt,860)hresl(jres-1),hresl(jres),1.0-dj,dj
 860  format ('  Fcalc histograms chosen: resolution =',2f5.2,' A'
     +       /'                              weights =',2f5.2)
      write (lpt,870)hrav,hrst,hmav,hmst
 870  format ('  Fcalc mean,std ',2f7.3,'  Adjusted mean,std ',2f7.3)
      write (lpt,880)
 880  format (/'    RHO       Counts    Counts       '
     +        /'              (Desired) (Transformed)'/)
      do 900 i=1,nhist,nhist/12
      r=(i-0.5)*dhr+hrhomin
      hmod(i)=hmod(i)/nr
      write (lpt,890)r,hcal(i),hmod(i)
 890  format (1x,f9.3,3x,2f10.6)
 900  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine molrep(map,mol,msk)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xtlparam.fh'
c MOLREP - do the real space modification for molecular replacement
c set anything in the model region to zero.
c
      real map(nu,nv,nw),mol(nu,nv,nw)
      integer msk(nu,nv,nw)
c
      integer i,j,k
      real sm,sx,sy,sx2,sy2,sxy,frac,correl
      external wangav
c
      call pdbmask(msk)
c
c now zero the stuff above the cutoff. calc correlation and volume
c
      sm=0.0
      sx=0.0
      sy=0.0
      sx2=0.0
      sy2=0.0
      sxy=0.0
      do 200 k=1,nw
      do 200 j=1,nv
      do 200 i=1,nu
       if (msk(i,j,k).ne.0) then
        sm=sm+1.0
        sx=sx+mol(i,j,k)
        sy=sy+map(i,j,k)
        sx2=sx2+mol(i,j,k)**2
        sy2=sy2+map(i,j,k)**2
        sxy=sxy+mol(i,j,k)*map(i,j,k)
        map(i,j,k)=0.0
       endif
 200  continue
c
      frac=sm/(nu*nv*nw)
      correl=(sm*sxy-sx*sy)/sqrt(max((sm*sx2-sx**2)*(sm*sy2-sy**2),0.0))
c
      write (lpt,300)frac,correl
 300  format (
     +  /' MOLECULAR REPLACEMENT:'
     +  /'  Fraction of unit cell covered by model : ',f6.3
     +  /'  Correlation between model and DM map   : ',f6.3)
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine sayrdm(rho,rhodm,l5,l6,l7,l8,solmsk,hkls,reseff)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xtlparam.fh'
      include 'xhklinfo.fh'
      real hkls(nhklc,0:*),reseff
      real rho(*),rhodm(*),l5(*),l6(*),l7(*),l8(*)
      integer solmsk(*)
c
      integer lfha,lfhb
      parameter (lfha=20,lfhb=21)
      integer i,nrcyc
      real xcor,correl
c
c
      do 300 nrcyc=1,10
c check for density modification
        if (mhist+msolv.ne.0) then
c .. do real space density modification
          call dm(hkls,rho,rhodm,solmsk,reseff,.false.)
c .. convert dm map to difference map: rhodm=rho-rhodm
          call sub3d(rhodm,rho,rhodm)
        endif
c solve least squares for dm with Sayres equation
c        call conjgrd(rho,rhodm,l5,l6,l7,l8,hkls)
        call diagaprx(rho,rhodm,l5,l6,l7,l8,hkls)
c now calc modified map and correlation
        call sub3d(rhodm,rho,rhodm)
        call mapcorrel(rho,rhodm,correl)
        call copy3d(rhodm,rho)
c zero all non-included reflections
        call invfftmaplist(rho,hkls,lfha,lfhb)
        do 200 i=1,nrflo
          if (hkls(lflag,i).eq.0.0) then
            hkls(lfha,i)=0.0
            hkls(lfhb,i)=0.0
          endif
 200    continue
        call fftlistmap(rho,hkls,lfha,lfhb)
        call mapcorrel(rho,rhodm,xcor)
        write (lpt,*)' corr ',xcor
c check for convergence
        write (lpt,290)correl
 290    format (' Correlation between old and new maps: ',f6.3)
        if (correl.gt.0.99) goto 310
 300  continue
 310  write (lpt,320)nrcyc
 320  format (' Newton-Raphson converged after ',i2,' cycles.')
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine diagaprx(rho,xk,rk,pk,qk,work,hkls)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xcrystal.fh'
      include 'xhklinfo.fh'
      include 'xtlparam.fh'
c Solve Sayre/dm equations usin diagonal approximation - quick and dirty
c ............ solve JTJ x = JT r
c
      real xk(nu,nv,nw),rk(nu,nv,nw),pk(nu,nv,nw),qk(nu,nv,nw)
      real rho(nu,nv,nw),work(nu,nv,nw),hkls(*)
c
      integer i,j,k,n,ih,ik,il
      real rtr,stheta,stheta2
      real s12,s13,s23,s2,s3,s,theta
      logical hemi
c
      real sctheta
      common /scaletheta/sctheta
      save   /scaletheta/
c
      external hemi,theta
c
      n=nu*nv*nw
c
c first calc rhs of system
      call jtr(rk, work,xk,rho,hkls)
c
c now calc diagonal elements
c ...accumulate sum_h(theta^2)
c
      stheta2=0.0
      stheta=0.0
      do 100 il=-lmax,lmax
      s13 = il*rcm13
      s23 = il*rcm23
      s3 = il**2*rcm33
      do 100 ik=-kmax,kmax
      s12 = ik*rcm12 + s13
      s2 = ik*(ik*rcm22 + s23) + s3
      do 100 ih=-hmax,hmax
      s  = ih*(ih*rcm11 + s12) + s2
      if (s.gt.0.0.and.s.le.smax) then
       if (hemi(ih,ik,il)) stheta2=stheta2+(sctheta*theta(s))**2
       if (hemi(ih,ik,il)) stheta =stheta +(sctheta*theta(s))
c? .and.not.extinct?
      endif
 100  continue
c
      do 200 k=1,nw
      do 200 j=1,nv
      do 200 i=1,nu
c      rtr=(4.0/n)*rho(i,j,k)*(rho(i,j,k)*stheta2-sctheta*theta(0.0))
c     +   +2.0
      rtr=(4.0/n)*rho(i,j,k)*(rho(i,j,k)*stheta2-stheta) + 2.0
      xk(i,j,k)=rk(i,j,k)/rtr
 200  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine conjgrd(rho,xk,rk,pk,qk,work,hkls)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xtlparam.fh'
c Solve a system of linear equations by the conjugate gradient method.
c ............ solve JTJ x = JT r
c
      real xk(*),rk(*),pk(*),qk(*)
      real rho(*),work(*),hkls(*)
c
      integer i,k,n
      real rtr,rtr0,rtr1,ptq,alpha,beta
c
      n=nu*nv*nw
c
c first calc rhs of system
      call jtr(rk, work,xk,rho,hkls)
c
      rtr=0.0
      do 100 i=1,n
        xk(i)=0.0
        pk(i)=rk(i)
        rtr=rtr+rk(i)**2
 100  continue
c
      rtr0=rtr
c
      do 200 k=1,10
c calc qk = JTJ pk
        call jtj(pk,qk, work,rho,hkls)
c calc alpha
        ptq=0.0
        do 120 i=1,n
          ptq=ptq+pk(i)*qk(i)
 120    continue
        alpha=rtr/ptq
c update estimated soln xk, calc beta
        rtr1=0.0
        do 140 i=1,n
          xk(i)=xk(i)+alpha*pk(i)
          rk(i)=rk(i)-alpha*qk(i)
          rtr1=rtr1+rk(i)*rk(i)
 140    continue
        beta=rtr1/rtr
c update conjugate direction pk
        do 160 i=1,n
          pk(i)=rk(i)+beta*pk(i)
 160    continue
c check for convergence
        rtr=rtr1
        if ((rtr/rtr0).lt.0.0001) goto 210
 200  continue
c
 210  write (lpt,220)k
 220  format (' Conjugate gradient converged after ',i2,' cycles.')
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine jtr(rk, work,rhodm,rho,hkls)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xtlparam.fh'
      include 'xhklinfo.fh'
c calculate rhs of least sayre's equation system
c
      real hkls(nhklc,0:*)
      real rho(nu,nv,nw),rhodm(nu,nv,nw),work(nu,nv,nw),rk(nu,nv,nw)
c
      real sctheta
      common /scaletheta/sctheta
      save   /scaletheta/
c
      integer lfha,lfhb,lgha,lghb,lrha,lrhb,lrh1a,lrh1b
      parameter (lfha=20,lfhb=21,lgha=22, lghb=23 )
      parameter (lrha=20,lrhb=21,lrh1a=22,lrh1b=23)
c
      integer i,j,k
      real s,f,g,sfg,sgg,w1,w2,r1,r2,theta
      complex fh,gh,rh,rh1
      external theta
c
      do 100 k=1,nw
      do 100 j=1,nv
      do 100 i=1,nu
        work(i,j,k)=rho(i,j,k)
        rk(i,j,k)=rho(i,j,k)**2
 100  continue
      call invfftmaplist(work,hkls,lfha,lfhb)
      call invfftmaplist(rk,hkls,lgha,lghb)
c
      sfg=0.0
      sgg=0.0
      do 190 i=1,nrflo
        s=hkls(4,i)
        f=abs(cmplx(hkls(lfha,i),hkls(lfhb,i)))
        g=abs(theta(s)*cmplx(hkls(lgha,i),hkls(lghb,i)))
        sfg=sfg+f*g
        sgg=sgg+g*g
 190  continue
      sctheta=sfg/sgg
c
      hkls(lrha,0)=0.0
      hkls(lrhb,0)=0.0
      hkls(lrh1a,0)=0.0
      hkls(lrh1b,0)=0.0
      do 200 i=1,nrflo
        s=hkls(4,i)
        fh=cmplx(hkls(lfha,i),hkls(lfhb,i))
        gh=cmplx(hkls(lgha,i),hkls(lghb,i))
c
        rh=sctheta*theta(s)*gh-fh
        rh1=sctheta*theta(s)*rh
c
        hkls(lrha,i)=real(rh)
        hkls(lrhb,i)=aimag(rh)
        hkls(lrh1a,i)=real(rh1)
        hkls(lrh1b,i)=aimag(rh1)
 200  continue
c
c?? bug to make Sayre work better
      hkls(lrh1a,0)=0.6*f000
c
      call fftlistmap(rk,hkls,lrha,lrhb)
      call fftlistmap(work,hkls,lrh1a,lrh1b)
c
c set weight for dm term
      w2=0.0
      if (msolv+mhist.ne.0) w2=1.0
c
c calc jtr and residuals
      r1=0.0
      r2=0.0
      do 300 k=1,nw
      do 300 j=1,nv
      do 300 i=1,nu
        r1=r1+rk(i,j,k)**2
        r2=r2+(w2*rhodm(i,j,k))**2
        rk(i,j,k)=2.0*rho(i,j,k)*work(i,j,k)-rk(i,j,k)+
     +  w2**2*rhodm(i,j,k)
 300  continue
c
      write (lpt,400)r1,r2
      write (lpt,*)' scfg ',sctheta
 400  format (
     +/' Sayre residual:                r1 = ',g12.5
     +/' Density modification residual: r2 = ',g12.5)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine jtj(pk,qk, work,rho,hkls)
      include 'dmmulti.fh'
      include 'xtlparam.fh'
      include 'xhklinfo.fh'
c calculate q(x)=JtJ p
c
      real hkls(nhklc,0:*)
      real qk(nu,nv,nw),pk(nu,nv,nw)
      real rho(nu,nv,nw),work(nu,nv,nw)
c
      real sctheta
      common /scaletheta/sctheta
      save   /scaletheta/
c
      integer lpha,lphb,lph1a,lph1b,lph2a,lph2b,lph3a,lph3b
      parameter (lpha=20, lphb=21, lph1a=22,lph1b=23)
      parameter (lph2a=20,lph2b=21,lph3a=22,lph3b=23)
      integer i,j,k
      real s,w2,theta
      complex ph,ph1,ph2,ph3
      external theta
c
      do 100 k=1,nw
      do 100 j=1,nv
      do 100 i=1,nu
        qk(i,j,k)=pk(i,j,k)
        work(i,j,k)=rho(i,j,k)*pk(i,j,k)
 100  continue
c
      call invfftmaplist(qk,hkls,lpha,lphb)
      call invfftmaplist(work,hkls,lph1a,lph1b)
c
      hkls(lph2a,0)=0.0
      hkls(lph2b,0)=0.0
      hkls(lph3a,0)=0.0
      hkls(lph3b,0)=0.0
      do 200 i=1,nrflo
        s=hkls(4,i)
        ph =cmplx(hkls(lpha ,i),hkls(lphb ,i))
        ph1=cmplx(hkls(lph1a,i),hkls(lph1b,i))
c
        ph2=2.0*sctheta*theta(s)*ph1-ph
        ph3=sctheta*theta(s)*ph2
c
        hkls(lph2a,i)=real(ph2)
        hkls(lph2b,i)=aimag(ph2)
        hkls(lph3a,i)=real(ph3)
        hkls(lph3b,i)=aimag(ph3)
 200  continue
c
      call fftlistmap(qk,hkls,lph2a,lph2b)
      call fftlistmap(work,hkls,lph3a,lph3b)
c
c set weight for dm term
      w2=0.0
      if (msolv+mhist.ne.0) w2=1.0
c
      do 300 k=1,nw
      do 300 j=1,nv
      do 300 i=1,nu
        qk(i,j,k)=2.0*rho(i,j,k)*work(i,j,k)-qk(i,j,k)+
     +  w2**2*pk(i,j,k)
 300  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine calctheta(hkls,rho,work1,work2)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xhklinfo.fh'
      include 'xtlparam.fh'
c calculate theta function for sayre calculation
c
      real hkls(nhklc,0:*)
      real rho(nu,nv,nw),work1(nu,nv,nw),work2(nu,nv,nw)
c
      real thc,ds
      common /thetacv/thc(0:200),ds,irmax
      save   /thetacv/
c
      integer lfha,lfhb,lgha,lghb
      parameter (lfha=20, lfhb=21, lgha=22,lghb=23)
      integer i,j,k,ires,irmax
      real thf(0:200),thg(0:200),thn(0:200)
      real s,w,x,y,sw,sx,sy,sx2,sy2,sxy,wf,wg,tha,thb,theta
      complex fh,gh
      external theta
c
c
      ds=0.005
      irmax=int(smax/ds+0.999)
      sw=0.0
      sx=0.0
      sy=0.0
      sx2=0.0
      sy2=0.0
      sxy=0.0
      do 10 i=0,irmax
        thf(i)=0.0
        thg(i)=0.0
        thn(i)=0.0
 10   continue
c
      do 100 k=1,nw
      do 100 j=1,nv
      do 100 i=1,nu
        work1(i,j,k)=rho(i,j,k)
        work2(i,j,k)=rho(i,j,k)**2
 100  continue
c
      call invfftmaplist(work1,hkls,lfha,lfhb)
      call invfftmaplist(work2,hkls,lgha,lghb)
c
      do 200 i=1,nrflo
        if (hkls(lflag,i).le.0) goto 200
        s=hkls(4,i)
        fh=cmplx(hkls(lfha,i),hkls(lfhb,i))
        gh=cmplx(hkls(lgha,i),hkls(lghb,i))
        ires=int(s/ds+0.999)
c take real part, since summation is over +/-hkl
        thf(ires)=thf(ires)+abs(fh)/max(abs(gh),1.0)
        thg(ires)=thg(ires)+abs(gh)
        thn(ires)=thn(ires)+1.0
 200  continue
c
c origin term
      thf(0)=hkls(lfha,0)/hkls(lgha,0)
      thg(0)=thf(0)
      thc(0)=thf(0)
c
c now fit a Gaussian to the bit of theta we have
      do 300 i=1,irmax
c .. calc theta
        if (abs(thf(i)).lt.1.0.or.abs(thg(i)).lt.1.0) then
          thn(i)=0.0
        else
          thf(i)=thf(i)/thn(i)
        endif
        w=thn(i)
        x=(i-0.5)*ds
        y=log(max(thf(i),1.0e-6))
        sw=sw+w
        sx=sx+w*x
        sy=sy+w*y
        sx2=sx2+w*x**2
        sy2=sy2+w*y**2
        sxy=sxy+w*x*y
 300  continue
      tha=(sxy*sw-sx*sy)/(sx2*sw-sx**2)
      thb=(sx2*sy-sxy*sx)/(sx2*sw-sx**2)
c
c now produce extrapolated curve
      do 400 i=1,irmax
        s=(i-0.5)*ds
        thg(i)=exp(tha*s+thb)
        wf=thn(i)
        wg=nrflo/irmax
        thc(i)=(wf*thf(i)+wg*thg(i))/(wf+wg)
 400  continue
c
c output results
      write (lpt,510)
      write (lpt,520)0.0,thn(0),thc(0),thf(0),thg(0),theta(0.0)
      write (lpt,520)((i-0.5)*ds,
     + thn(i),thc(i),thf(i),thg(i),theta((i-0.5)*ds),i=1,irmax)
 510  format (/' Theta Curve Estimation:'
     +/'      S      REFLECTIONS    THETA      EMPIRICAL   GAUSSIAN ')
 520  format (4x,f8.3,f12.0,4f12.4)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      real function theta(s)
      include 'dmmulti.fh'
c calulate theta(s) from lookup
c
      real s,r,dr,thc,ds
      common /thetacv/thc(0:200),ds,irmax
      save   /thetacv/
      integer ir,irmax
c
      if (s.eq.0.0) then
        theta=thc(0)
      else
        r=s/ds+0.5
        ir=int(r)
        ir=min(max(ir,1),irmax)
        dr=r-real(ir)
        theta=(1.0-dr)*thc(ir)+dr*thc(ir+1)
      endif
      return
      end
c
c ----------------------------------------------------------------------
c

