C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
c
c ----------------------------------------------------------------------
c
      subroutine calcmetric(cell,vol,q,qt,
     +  rcm11,rcm12,rcm13,rcm22,rcm23,rcm33,
     +  rlm11,rlm12,rlm13,rlm22,rlm23,rlm33)
      include 'dmmulti.fh'
      include 'io.fh'
c Calculate the orthonormal - crystallographic transform
      real cell(6),vol,q(4,4),qt(4,4)
      real rcm11,rcm12,rcm13,rcm22,rcm23,rcm33
      real rlm11,rlm12,rlm13,rlm22,rlm23,rlm33
c
      integer i,j
      real a,b,c,alph,beta,gamm,sum,cosas,q123
c
      do 10 j=1,4
      do 10 i=1,4
      q(i,j)=0.0
      qt(i,j)=0.0
 10   continue
c
      a=cell(1)
      b=cell(2)
      c=cell(3)
      alph=dtor*cell(4)
      beta=dtor*cell(5)
      gamm=dtor*cell(6)
      sum=0.5*(alph+beta+gamm)
      vol=2.0*a*b*c*
     + sqrt(sin(sum)*sin(sum-alph)*sin(sum-beta)*sin(sum-gamm))
      cosas=(cos(beta)*cos(gamm)-cos(alph))/(sin(beta)*sin(gamm))
c orthogonalisation matrix
      q(1,1)=a
      q(1,2)=b*cos(gamm)
      q(1,3)=c*cos(beta)
      q(2,2)=b*sin(gamm)
      q(2,3)=-c*sin(beta)*cosas
      q(3,3)=c*sin(beta)*sqrt(1.0-cosas**2)
      q(4,4)=1.0
c and its inverse
      q123=q(1,1)*q(2,2)*q(3,3)
      qt(1,1)=1.0/q(1,1)
      qt(1,2)=-q(1,2)*q(3,3)/q123
      qt(1,3)=(q(1,2)*q(2,3)-q(2,2)*q(1,3))/q123
      qt(2,2)=1.0/q(2,2)
      qt(2,3)=-q(1,1)*q(2,3)/q123
      qt(3,3)=1.0/q(3,3)
      qt(4,4)=1.0
c
      write (lpt,4)vol
 4    format (' Unit cell volume:',f10.1)
      write (lpt,1)((q(i,j),j=1,3),(qt(i,j),j=1,3),i=1,3)
 1    format (' Orthogonalisation matrices:',3(/1x,3f8.3,8x,3f8.3))
c Calculate real space metric tensor
      rlm11=q(1,1)*q(1,1)+q(2,1)*q(2,1)+q(3,1)*q(3,1)
      rlm12=2.0*(q(1,1)*q(1,2)+q(2,1)*q(2,2)+q(3,1)*q(3,2))
      rlm13=2.0*(q(1,1)*q(1,3)+q(2,1)*q(2,3)+q(3,1)*q(3,3))
      rlm22=q(1,2)*q(1,2)+q(2,2)*q(2,2)+q(3,2)*q(3,2)
      rlm23=2.0*(q(1,2)*q(1,3)+q(2,2)*q(2,3)+q(3,2)*q(3,3))
      rlm33=q(1,3)*q(1,3)+q(2,3)*q(2,3)+q(3,3)*q(3,3)
      write (lpt,2)rlm11,rlm22,rlm33,rlm12,rlm13,rlm23
 2    format (' Real space metric tensor:      '/1x,6f9.1)
c Calculate reciprocal space metric tensor
      rcm11=qt(1,1)*qt(1,1)+qt(1,2)*qt(1,2)+qt(1,3)*qt(1,3)
      rcm12=2.0*(qt(1,1)*qt(2,1)+qt(1,2)*qt(2,2)+qt(1,3)*qt(2,3))
      rcm13=2.0*(qt(1,1)*qt(3,1)+qt(1,2)*qt(3,2)+qt(1,3)*qt(3,3))
      rcm22=qt(2,1)*qt(2,1)+qt(2,2)*qt(2,2)+qt(2,3)*qt(2,3)
      rcm23=2.0*(qt(2,1)*qt(3,1)+qt(2,2)*qt(3,2)+qt(2,3)*qt(3,3))
      rcm33=qt(3,1)*qt(3,1)+qt(3,2)*qt(3,2)+qt(3,3)*qt(3,3)
      write (lpt,3)rcm11,rcm22,rcm33,rcm12,rcm13,rcm23
 3    format (' Reciprocal space metric tensor:'/1x,6f9.6)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine eulertomatrix(alpha,beta,gamma,matrix)
      include 'dmmulti.fh'
c calculate rotation matrix from Euler angles
      real alpha,beta,gamma,matrix(4,4)
c
      real ca,cb,cg,sa,sb,sg
c
      ca = cos(dtor*alpha)
      sa = sin(dtor*alpha)
      cb = cos(dtor*beta)
      sb = sin(dtor*beta)
      cg = cos(dtor*gamma)
      sg = sin(dtor*gamma)
      matrix(1,1) = cg*cb*ca - sg*sa
      matrix(1,2) = -sg*cb*ca - cg*sa
      matrix(1,3) = sb*ca
      matrix(2,1) = cg*cb*sa + sg*ca
      matrix(2,2) = -sg*cb*sa + cg*ca
      matrix(2,3) = sb*sa
      matrix(3,1) = -cg*sb
      matrix(3,2) = sg*sb
      matrix(3,3) = cb
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine polartomatrix(omega,phi,kappa,matrix)
      include 'dmmulti.fh'
c calculate rotation matrix from Euler angles
      real omega,phi,kappa,matrix(4,4)
c
      real someg,comeg,sinph,cosph,sinchi,coschi,cosch1,dc1,dc2,dc3
c
      someg = sin(omega*dtor)
      comeg = cos(omega*dtor)
      sinph = sin(phi*dtor)
      cosph = cos(phi*dtor)
      dc1 = someg*cosph
      dc2 = someg*sinph
      dc3 = comeg
      sinchi = sin(kappa*dtor)
      coschi = cos(kappa*dtor)
      cosch1 = 1.0 - coschi
      matrix(1,1) = dc1*dc1*cosch1 + coschi
      matrix(1,2) = dc1*dc2*cosch1 - dc3*sinchi
      matrix(1,3) = dc3*dc1*cosch1 + dc2*sinchi
      matrix(2,1) = dc1*dc2*cosch1 + dc3*sinchi
      matrix(2,2) = dc2*dc2*cosch1 + coschi
      matrix(2,3) = -dc1*sinchi + dc2*dc3*cosch1
      matrix(3,1) = -dc2*sinchi + dc3*dc1*cosch1
      matrix(3,2) = dc2*dc3*cosch1 + dc1*sinchi
      matrix(3,3) = dc3*dc3*cosch1 + coschi
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine gridres(res,nu,nv,nw)
      include 'dmmulti.fh'
      include 'xcrystal.fh'
c calculate grid required for a particular data resolution
      integer nu0,nv0,nw0,nu,nv,nw
      real res
      nu0=2*int(cell(1)/res)+1
      nv0=2*int(cell(2)/res)+1
      nw0=2*int(cell(3)/res)+1
      call gridsize(nu0,nv0,nw0,nu,nv,nw)
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine gridsize(nu0,nv0,nw0,nu,nv,nw)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xcrystal.fh'
c Calculate a valid grid size
c
      integer nu0,nv0,nw0,nu,nv,nw
c
      integer mu,mv,mw,i,fftfac
      external fftfac
c
c Set up factors required by symmetry and fft
      mu=2
      mv=2
      mw=2
      do 100 i=1,nsym
       if (rsym(1,4,i).ne.0.0) mu=max(mu,nint(1.0/rsym(1,4,i)))
       if (rsym(2,4,i).ne.0.0) mv=max(mv,nint(1.0/rsym(2,4,i)))
       if (rsym(3,4,i).ne.0.0) mw=max(mw,nint(1.0/rsym(3,4,i)))
100   continue
      if (mod(mu,2).eq.1) mu=mu*2
      if (mod(mv,2).eq.1) mv=mv*2
      if (mod(mw,2).eq.1) mw=mw*2
c
c if no grid set then set grid sizes
      if (nu+nv+nw.eq.0) then
       nu=fftfac(nu0,mu)
       nv=fftfac(nv0,mv)
       nw=fftfac(nw0,mw)
      endif
c
c print the result
      write (lpt,110)nu,nv,nw,mu,mv,mw
 110  format(
     +/' Grid dimensions ',3i4,'  must contain the following prime'
     +/' factors for agreement with symmetry restrictions- ',3i4/)
c check the input grid - should only fail if input by user
      if (mod(nu,mu)+mod(nv,mv)+mod(nw,mw).ne.0) call ccperr(1,
     +  ' GRIDSIZE - GRID card does not fit symmetry restrictions')
c
      return
      end
c
c ----------------------------------------------------------------------
c
      integer function fftfac(n0,m0)
      include 'dmmulti.fh'
c FFTFAC - find the composite number greater n than n0 which
c will optimise the speed of the fft and has m0 as factor
c (naive calculation: time = (p1.p2.p3...pn).(p1+p2+p3+..pn) )
c
      integer n0,m0
c
      integer nprime
      parameter (nprime=5)
      integer iprime(10),i,j,n,n1,nmin,time,timemin,sum,prod
      data iprime/2,3,5,7,11,13,17,19,23,27/
c
      n1=((n0+m0-1)/m0)*m0
      nmin=0
      timemin=1000000
c Try a number
      do 200 n=n1,2*n1,m0
c decompose n and add its factors to sum and prod
       i=n
       sum=0
       prod=1
       do 150 j=1,nprime
 100    if (mod(i,iprime(j)).eq.0) then
         i=i/iprime(j)
         sum=sum+iprime(j)
         prod=prod*iprime(j)
         goto 100
        endif
 150   continue
       time=sum*prod
       if (i.eq.1.and.time.lt.timemin) then
        nmin=n
        timemin=time
       endif
 200  continue
c
      fftfac=nmin
c
      return
      end
c
c ----------------------------------------------------------------------
c
      integer function numres(reso)
      include 'dmmulti.fh'
      include 'xcrystal.fh'
      include 'xhklinfo.fh'
c NUMRES - calculate number of reflections at a particular resolution
c this is the slow but accurate counting method!
c
      real reso
c
      integer in(3),ih,ik,il,n,inasuold
      real s12,s13,s23,s2,s3,s,slim,eps,phicen
      logical centre,sysabs
      external inasuold
      equivalence (in(1),ih)
      equivalence (in(2),ik)
      equivalence (in(3),il)
c
      n=0
      slim=1.0/reso**2
c *** optimise this bit ***
      do 100 il=-lmax,lmax
      s13 = il*rcm13
      s23 = il*rcm23
      s3 = il**2*rcm33
      do 100 ik=-kmax,kmax
      s12 = ik*rcm12 + s13
      s2 = ik*(ik*rcm22 + s23) + s3
      do 100 ih=-hmax,hmax
      s  = ih*(ih*rcm11 + s12) + s2
      if (s.gt.0.0.and.s.le.slim) then
       if (inasuold(in,nlaue).eq.1) then
        call epscen(ih,ik,il,centre,sysabs,eps,phicen,nsym,rsym)
        if (.not.sysabs) n=n+1
       endif
      endif
 100  continue
c
      numres=n
c
cc Fast method underestimates in many space groups
cc      numres=(2.0*pi*vol)/(3.0*reso**3*nsym)
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine ccpmskin(name,lmask,nu1,nv1,nw1,nu2,nv2,nw2,
     +  lu,lv,lw,iprint)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xcrystal.fh'
      include 'xtlparam.fh'
c CCPMSKIN - read a ccp4 logical mask and store in xyz order
c
      integer lmask(0:*)
      integer nu1,nv1,nw1,nu2,nv2,nw2,lu,lv,lw,iprint
      character name*(*)
c
      integer lsec(maxsec)
c
      integer ifast,imedm,islow,lfast,lmedm,lslow,ierr,ifail
      integer m1(3),m2(3),ifms(3),jfms(3),juvw(3),mxyz(3),maxmsk
      integer i,iu,iv,iw,ju,jv,jw,iu1,iv1,iw1,iu2,iv2,iw2,nn,mm
      integer mu,mv,mw,msec,mspgrp,mode,lmod,lint
      real mcell(6),rmin,rmax,rmean,rrms,cerr
      character title*80
c
      external lmod,lint
c
c Note: juvw convert from fast/med/slow to u/v/w
c       jfms convert from u/v/w to fast/med/slow
c
c set maximum mask size
c
      maxmsk=nu*nv*nw
c
c now open map header and read map, re-ordering it as necessary
c
      nn=0
      mm=0
c
      ifail=0
      call mrdhds(lmsks,name,title,msec,jfms,mxyz,m1(3),m1(1),m2(1),
     +  m1(2),m2(2),mcell,mspgrp,mode,rmin,rmax,rmean,rrms,ifail,iprint)
      m2(3)=m1(3)+msec-1
      do 110 i=1,3
       juvw(jfms(i))=i
 110  continue
c find out the mask grid dimensions
      lfast=m2(1)-m1(1)+1
      lmedm=m2(2)-m1(2)+1
      lslow=m2(3)-m1(3)+1
      mu=mxyz(1)
      mv=mxyz(2)
      mw=mxyz(3)
c now generate program grid dimensions
      nu1=nint(real(m1(juvw(1)))*nu/mu)
      nu2=nint(real(m2(juvw(1)))*nu/mu)
      nv1=nint(real(m1(juvw(2)))*nv/mv)
      nv2=nint(real(m2(juvw(2)))*nv/mv)
      nw1=nint(real(m1(juvw(3)))*nw/mw)
      nw2=nint(real(m2(juvw(3)))*nw/mw)
      lu=nu2-nu1+1
      lv=nv2-nv1+1
      lw=nw2-nw1+1
      do 115 i=0,lu*lv*lw-1
       lmask(i)=0
 115  continue
c check map format OK
      cerr=0.0
      do 120 i=1,6
       cerr=cerr+abs(cell(i)-mcell(i))/(cell(i)+mcell(i))
 120  continue
      if (cerr.gt.0.01)
     +  call ccperr(1,' ccpmskin - Inconsistent cell info')
      if (lu*lv*lw.gt.maxmsk)
     +  call ccperr(1,' ccpmskin - Mask file bigger than unit cell?!')
      if (lfast*lmedm.gt.4*maxsec)
     +  call ccperr(1,' ccpmskin - Mask section > lsec: recompile')
      if (mode.ne.0)
     +  call ccperr(1,' ccpmskin - Mask file is not logical*1')
c now read the map in the order it is on file
      do 200 islow=0,lslow-1
c get a section
       ierr=0
       call mgulp(lmsks,lsec,ierr)
       if (ierr.ne.0) call ccperr(1,' ccpmskin - ccp4 read error')
c now sort into uvw mask and extrapolate onto program grid
       ifms(3)=islow+m1(3)
       do 190 imedm=0,lmedm-1
       ifms(2)=imedm+m1(2)
       do 190 ifast=0,lfast-1
       ifms(1)=ifast+m1(1)
        call ccpgtb(i,lsec,ifast+lfast*imedm+1)
        if (i.ne.0) mm=mm+1
        iu=ifms(juvw(1))
        iv=ifms(juvw(2))
        iw=ifms(juvw(3))
        iu1=lint((real(iu)-0.50)*nu/mu)
        iu2=lint((real(iu)+0.50)*nu/mu)
        iv1=lint((real(iv)-0.50)*nv/mv)
        iv2=lint((real(iv)+0.50)*nv/mv)
        iw1=lint((real(iw)-0.50)*nw/mw)
        iw2=lint((real(iw)+0.50)*nw/mw)
c we must fill every program grid point covered by this mask point
        do 180 iw=iw1+1,iw2
        jw=iw-nw1
        do 180 iv=iv1+1,iv2
        jv=iv-nv1
        do 180 iu=iu1+1,iu2
        ju=iu-nu1
         if (ju.ge.0 .and.jv.ge.0 .and.jw.ge.0 .and.
     +       ju.lt.lu.and.jv.lt.lv.and.jw.lt.lw) then
          if (i.ne.0) lmask(ju+lu*(jv+lv*jw))=1
         endif
 180    continue
 190   continue
 200  continue
c close the map file
      call mrclos(lmsks)
c
      do 250 i=0,lu*lv*lw-1
       if (lmask(i).ne.0) nn=nn+1
 250  continue
      if (iprint.ne.0) write (lpt,260)
     +  mm,real(mm)/real(mu*mv*mw),nn,real(nn)/real(nu*nv*nw)
 260  format(
     + ' Mask grid   :  Grid points set- ',i7,'  Cell fraction- ',f6.3/
     + ' Program grid:  Grid points set- ',i7,'  Cell fraction- ',f6.3/)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine ccpmskout(name,lmask,mu,mv,mw,mu1,mv1,mw1,mu2,mv2,mw2)
      include 'dmmulti.fh'
      include 'io.fh'
      include 'xcrystal.fh'
      include 'xtlparam.fh'
c CCPMSKOUT - write a ccp4 logical mask in xyz or xzy order
c
      integer mu,mv,mw,mu1,mv1,mw1,mu2,mv2,mw2
      integer lmask(0:mu-1,0:mv-1,0:mw-1)
      character name*(*)
c
      integer lsec(maxsec)
      integer jfms(3),juvw(3),mxyz(3),mxyz1(3),mxyz2(3)
      integer i,iu,iv,iw,nn,msec,mode
      integer m1(3),m2(3),ifms(3),ifast,imedm,islow,lfast,lmedm,lslow
      character title*80
c
c spacegroups with yxz=1 and zxy=2 axis ordering
      integer axis(230)
      data axis/2,2,2,2,1,1,1,1,1,2,1,1,1,1,1,2,2,2,1,2,2,1,2,207*1/
c
c Set up axis order
c
      jfms(1)=2
      jfms(2)=1
      jfms(3)=3
      if (axis(mod(nspgrp,1000)).eq.2) then
       jfms(1)=3
       jfms(2)=1
       jfms(3)=2
      endif
      do 110 i=1,3
       juvw(jfms(i))=i
 110  continue
c
c set grid limits
c
      mxyz(1)=mu
      mxyz(2)=mv
      mxyz(3)=mw
      mxyz1(1)=mu1
      mxyz1(2)=mv1
      mxyz1(3)=mw1
      mxyz2(1)=mu2
      mxyz2(2)=mv2
      mxyz2(3)=mw2
c
      do 120 i=1,3
       m1(i)=mxyz1(jfms(i))
       m2(i)=mxyz2(jfms(i))
 120  continue
c
c check maximum mask size
c
      lfast=m2(1)-m1(1)+1
      lmedm=m2(2)-m1(2)+1
      lslow=m2(3)-m1(3)+1
      if (lfast*lmedm.gt.4*maxsec)
     +  call ccperr(1,' ccpmskout - Mask section > lsec: recompile')
c
c now open map header and read map, re-ordering it as necessary
c
      title='mask from dm'
      msec=mw2-mw1+1
      mxyz(1)=mu
      mxyz(2)=mv
      mxyz(3)=mw
      mode=0
      msec=m2(3)-m1(3)+1
      call mwrhdl(lmsks,name,title,msec,jfms,mxyz,
     +  m1(3),m1(1),m2(1),m1(2),m2(2),cell,nspgrp,mode)
      call msywrt(lmsks,nsym,rsym)
      do 200 islow=0,lslow-1
      ifms(3)=islow+m1(3)
c put a section
       do 180 imedm=0,lmedm-1
       ifms(2)=imedm+m1(2)
       do 180 ifast=0,lfast-1
       ifms(1)=ifast+m1(1)
        iu=ifms(juvw(1))
        iv=ifms(juvw(2))
        iw=ifms(juvw(3))
        i=0
        if (lmask(iu,iv,iw).ne.0) i=1
        call ccpstb(i,lsec,(ifast-m1(1))+lfast*(imedm-m1(2))+1  )
 180   continue
       call mspew(lmsks,lsec)
 200  continue
c close the map file
      call mwclose(lmsks)
c
      nn=0
      do 250 iw=mw1,mw2
      do 250 iv=mv1,mv2
      do 250 iu=mu1,mu2
       if (lmask(iu,iv,iw).ne.0) nn=nn+1
 250  continue
      write (lpt,260)nn,real(nn)/real(mu*mv*mw)
 260  format (' Number of grid points set: ',i7
     +       /' Fraction of unit cell:     ',f7.3)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine masksec(mask,mu,mv)
      include 'dmmulti.fh'
      include 'io.fh'
c MASKSEC - output a mask section to log file
c
      integer mu,mv,mask(0:mu-1,0:mv-1)
c
      integer iu,iv,ju,jv,lu,lv,lmod
      real scx,scy
      character str*72
      external lmod
c
      write (lpt,120)
 120  format (12x,'Maps are x-y sections with the x-axis horizontal'/
     +        12x,'and the origin top left.')
c
c display sections of the mask: width 64, y/x=1.25
      str=' '
      scx=real(max(mu,mv))/64.0
      scy=scx*1.25
      lu=nint(mu/scx)
      lv=nint(mv/scy)
c print a section
      do 300 iv=1,lv
      jv=lmod(nint(scy*real(iv-1)),mv)
      do 290 iu=1,lu
      ju=lmod(nint(scx*real(iu-1)),mu)
      str(iu:iu)=' '
      if (iu.eq.1.or.iu.eq.lu.or.iv.eq.1.or.iv.eq.lv) str(iu:iu)='+'
      if (mask(ju,jv).gt.0) str(iu:iu)='*'
      if (mask(ju,jv).lt.0) str(iu:iu)='?'
 290  continue
      write (lpt,295)str
 295  format (1x,a)
 300  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine mapstats(rho,rave,rstd)
      include 'dmmulti.fh'
      include 'xtlparam.fh'
c Produce stats for a map
c
      real rho(nu,nv,nw),rave,rstd
c
      integer i,j,k
      real sn,sx,sx2
c
      sn=0.0
      sx=0.0
      sx2=0.0
      do 10 k=1,nw
      do 10 j=1,nv
      do 10 i=1,nu
      sn=sn+1.0
      sx=sx+rho(i,j,k)
      sx2=sx2+rho(i,j,k)**2
 10   continue
c
      rave=sx/sn
      rstd=sqrt(max(sn*sx2-sx**2,0.0))/sn
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine mapcorrel(rho1,rho2,correl)
      include 'dmmulti.fh'
      include 'xtlparam.fh'
c Produce correl between two maps
c
      real rho1(nu,nv,nw),rho2(nu,nv,nw),correl
c
      integer i,j,k
      real sn,sx,sy,sx2,sy2,sxy
c
      sn=0.0
      sx=0.0
      sy=0.0
      sx2=0.0
      sy2=0.0
      sxy=0.0
      do 10 k=1,nw
      do 10 j=1,nv
      do 10 i=1,nu
      sn=sn+1.0
      sx=sx+rho1(i,j,k)
      sy=sy+rho2(i,j,k)
      sx2=sx2+rho1(i,j,k)**2
      sy2=sy2+rho2(i,j,k)**2
      sxy=sxy+rho1(i,j,k)*rho2(i,j,k)
 10   continue
c
      correl=(sn*sxy-sx*sy)/sqrt(max((sn*sx2-sx**2)*(sn*sy2-sy**2),0.0))
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine symuvw(mat,u,v,w,u1,v1,w1)
      include 'dmmulti.fh'
c symuvw - multiply a 4-matrix by a 3-vector
      real mat(4,4),u,v,w,u1,v1,w1
      u1=mat(1,1)*u+mat(1,2)*v+mat(1,3)*w+mat(1,4)
      v1=mat(2,1)*u+mat(2,2)*v+mat(2,3)*w+mat(2,4)
      w1=mat(3,1)*u+mat(3,2)*v+mat(3,3)*w+mat(3,4)
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine symcpy(a,b)
      include 'dmmulti.fh'
c symcpy - copy a symmetry matrix
      real a(16),b(16)
      integer i
      do 100 i=1,16
       b(i)=a(i)
 100  continue
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine symmatmul(a,b,c)
      include 'dmmulti.fh'
c symmetmaul - 4x4 matrix multiply, c=a*b
      real a(4,4),b(4,4),c(4,4)
      integer i,j,k
c
      do 20 k=1,4
      do 20 i=1,4
      c(i,k)=0.0
      do 20 j=1,4
      c(i,k)=c(i,k)+a(i,j)*b(j,k)
20    continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine copy3d(x1,x2)
      include 'dmmulti.fh'
      include 'xtlparam.fh'
c copy data from one array to another: x1->x2
      real x1(*),x2(*)
c
      integer i
c
      do 10 i=1,nu*nv*nw
      x2(i)=x1(i)
 10   continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine add3d(x1,x2,x3)
      include 'dmmulti.fh'
      include 'xtlparam.fh'
c add map arrays x1=x2+x3
      real x1(*),x2(*),x3(*)
c
      integer i
c
      do 10 i=1,nu*nv*nw
      x1(i)=x2(i)+x3(i)
 10   continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine sub3d(x1,x2,x3)
      include 'dmmulti.fh'
      include 'xtlparam.fh'
c subtract map arrays x1=x2-x3
      real x1(*),x2(*),x3(*)
c
      integer i
c
      do 10 i=1,nu*nv*nw
      x1(i)=x2(i)-x3(i)
 10   continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine mul3d(x1,x2,x3)
      include 'dmmulti.fh'
      include 'xtlparam.fh'
c multiply map arrays x1=x2*x3
      real x1(*),x2(*),x3(*)
c
      integer i
c
      do 10 i=1,nu*nv*nw
      x1(i)=x2(i)*x3(i)
 10   continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine hklfpab(hkls,lf,lphi)
      include 'dmmulti.fh'
      include 'xhklinfo.fh'
c generate A,B list from F,PHI list
      integer lf,lphi
      real hkls(nhklc,0:*)
c
      integer i,lfa,lfb
      real f,phi
c
      lfa=lf
      lfb=lphi
c
c first the origin term
      hkls(lfa,0)=hkls(lf,0)
      hkls(lfb,0)=0.0
c now go through the rest of the list
      do 100 i=1,nrflo
       f=hkls(lf,i)
       phi=hkls(lphi,i)
       hkls(lfa,i)=f*cos(phi)
       hkls(lfb,i)=f*sin(phi)
 100  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine hklabfp(hkls,lfa,lfb)
      include 'dmmulti.fh'
      include 'xhklinfo.fh'
c generate F,PHI list from A,B list
      integer lfa,lfb
      real hkls(nhklc,0:*)
c
      integer i,lf,lphi
      real phaseof
      complex fh
      external phaseof
c
      lf  =lfa
      lphi=lfb
c
c first the origin term
      hkls(lf,0)=hkls(lfa,0)
      hkls(lphi,0)=0.0
c now go through the rest of the list
      do 100 i=1,nrflo
       fh=cmplx(hkls(lfa,i),hkls(lfb,i))
       hkls(lf,i)=abs(fh)
       hkls(lphi,i)=phaseof(fh)
 100  continue
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine fftlistmapn(rfl3d,hkls,lfa,lfb,nu,nv,nw)
      include 'dmmulti.fh'
      include 'xcrystal.fh'
      include 'xhklinfo.fh'
c FFT two columns of data from the reflection list into a map
c of arbitrary size (for averaging, skeletons, whatever)
c I don't think I need an inverse for this.
c
      integer lfa,lfb,nu,nv,nw
      real hkls(nhklc,0:*)
      real rfl3d(0:nu-1,0:nv-1,0:nw-1)
c
      real fftscale
      integer i,m,iu,iv,iw,ih,ik,il,ihm,ikm,ilm,lmod
      complex fh,fm,phase
      logical hemi
      external phase,hemi,lmod
c
      fftscale=1.0/vol
c
c clear the 3d array
      do 100 iw=0,nw-1
      do 100 iv=0,nv-1
      do 100 iu=0,nu-1
      rfl3d(iu,iv,iw)=0.0
 100  continue
c
c now fill it with reflections
c loop through reflections
      do 200 i=1,nrflo
c fetch a reflection (complex)
       ih=nint(hkls(1,i))
       ik=nint(hkls(2,i))
       il=nint(hkls(3,i))
       fh=fftscale*cmplx(hkls(lfa,i),hkls(lfb,i))
c calculate symmetry equivalents for this reflection
       do 150 m=1,nsymp
        ihm=nint(rsym(1,1,m)*ih+rsym(2,1,m)*ik+rsym(3,1,m)*il)
        ikm=nint(rsym(1,2,m)*ih+rsym(2,2,m)*ik+rsym(3,2,m)*il)
        ilm=nint(rsym(1,3,m)*ih+rsym(2,3,m)*ik+rsym(3,3,m)*il)
        fm=fh*
     +   phase(-2.0*pi*(ih*rsym(1,4,m)+ik*rsym(2,4,m)+il*rsym(3,4,m)))
c and store, filling in all the l=0 plane
        if (ilm.ge.0) then
         iu=  lmod(ihm,nu)
         iv=  lmod(ikm,nv)
         iw=2*lmod(ilm,nw)
         rfl3d(iu,iv,iw)  = real(fm)
         rfl3d(iu,iv,iw+1)= aimag(fm)
        endif
        if (-ilm.ge.0) then
         iu=  lmod(-ihm,nu)
         iv=  lmod(-ikm,nv)
         iw=2*lmod(-ilm,nw)
         rfl3d(iu,iv,iw)  = real(fm)
         rfl3d(iu,iv,iw+1)=-aimag(fm)
        endif
c next symmetry equivalent
 150   continue
c next reflection
 200  continue
c
c include (000)
      rfl3d(0,0,0)=fftscale*hkls(lfa,0)
c
c Now perform hermitian-real FFT
c
      call fft3hr(rfl3d,nu,nv,nw)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine fftlistmap(rfl3d,hkls,lfa,lfb)
      include 'dmmulti.fh'
      include 'xcrystal.fh'
      include 'xhklinfo.fh'
      include 'xtlparam.fh'
c FFT two columns of data from the reflection list into a map
c
      integer lfa,lfb
      real hkls(nhklc,0:*)
      real rfl3d(0:nu-1,0:nv-1,0:nw-1)
c
      real fftscale
      integer i,m,iu,iv,iw,ih,ik,il,ihm,ikm,ilm,lmod
      complex fh,fm,phase
      logical hemi
      external phase,hemi,lmod
c
      fftscale=1.0/vol
c
c clear the 3d array
      do 100 iw=0,nw-1
      do 100 iv=0,nv-1
      do 100 iu=0,nu-1
      rfl3d(iu,iv,iw)=0.0
 100  continue
c
c now fill it with reflections
c loop through reflections
      do 200 i=1,nrflo
c fetch a reflection (complex)
       ih=nint(hkls(1,i))
       ik=nint(hkls(2,i))
       il=nint(hkls(3,i))
       fh=fftscale*cmplx(hkls(lfa,i),hkls(lfb,i))
c calculate symmetry equivalents for this reflection
       do 150 m=1,nsymp
        ihm=nint(rsym(1,1,m)*ih+rsym(2,1,m)*ik+rsym(3,1,m)*il)
        ikm=nint(rsym(1,2,m)*ih+rsym(2,2,m)*ik+rsym(3,2,m)*il)
        ilm=nint(rsym(1,3,m)*ih+rsym(2,3,m)*ik+rsym(3,3,m)*il)
        fm=fh*
     +   phase(-2.0*pi*(ih*rsym(1,4,m)+ik*rsym(2,4,m)+il*rsym(3,4,m)))
c and store, filling in all the l=0 plane
        if (ilm.ge.0) then
         iu=  lmod(ihm,nu)
         iv=  lmod(ikm,nv)
         iw=2*lmod(ilm,nw)
         rfl3d(iu,iv,iw)  = real(fm)
         rfl3d(iu,iv,iw+1)= aimag(fm)
        endif
        if (-ilm.ge.0) then
         iu=  lmod(-ihm,nu)
         iv=  lmod(-ikm,nv)
         iw=2*lmod(-ilm,nw)
         rfl3d(iu,iv,iw)  = real(fm)
         rfl3d(iu,iv,iw+1)=-aimag(fm)
        endif
c next symmetry equivalent
 150   continue
c next reflection
 200  continue
c
c include (000)
      rfl3d(0,0,0)=fftscale*hkls(lfa,0)
c
c Now perform hermitian-real FFT
c
      call fft3hr(rfl3d,nu,nv,nw)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine invfftmaplist(rfl3d,hkls,lfa,lfb)
      include 'dmmulti.fh'
      include 'xcrystal.fh'
      include 'xhklinfo.fh'
      include 'xtlparam.fh'
c inverse FFT from map into reflection list
c
      integer lfa,lfb
      real hkls(nhklc,0:*)
      real rfl3d(0:nu-1,0:nv-1,0:nw-1)
c
      real fftscale0,fftscale,dphim
      integer i,m,iu,iv,iw,ih,ik,il,ihm,ikm,ilm,lmod
      complex sf,phase
      logical hemi
      external phase,hemi,lmod
c
c NB scale includes term for averaging over primative symmetry elements
      fftscale0=vol/real(nu*nv*nw)
      fftscale=vol/real(nu*nv*nw*nsymp)
c
c Perform Real-Hermitian FFT and scale the result
c
      call fft3rh(rfl3d,nu,nv,nw)
c
c fill the reflection list - loop through reflections
      do 200 i=1,nrflo
       ih=nint(hkls(1,i))
       ik=nint(hkls(2,i))
       il=nint(hkls(3,i))
c average over symmetry equivalents for this reflection
       sf=cmplx(0.0,0.0)
       do 150 m=1,nsymp
        ihm=nint(rsym(1,1,m)*ih+rsym(2,1,m)*ik+rsym(3,1,m)*il)
        ikm=nint(rsym(1,2,m)*ih+rsym(2,2,m)*ik+rsym(3,2,m)*il)
        ilm=nint(rsym(1,3,m)*ih+rsym(2,3,m)*ik+rsym(3,3,m)*il)
        dphim=2.0*pi*(ih*rsym(1,4,m)+ik*rsym(2,4,m)+il*rsym(3,4,m))
c note: we perform conjugate for inv fft
        if (hemi(ihm,ikm,ilm)) then
         iu=  lmod(ihm,nu)
         iv=  lmod(ikm,nv)
         iw=2*lmod(ilm,nw)
         sf=sf+cmplx(rfl3d(iu,iv,iw),-rfl3d(iu,iv,iw+1))*phase(dphim)
        else
         iu=  lmod(-ihm,nu)
         iv=  lmod(-ikm,nv)
         iw=2*lmod(-ilm,nw)
         sf=sf+cmplx(rfl3d(iu,iv,iw), rfl3d(iu,iv,iw+1))*phase(dphim)
        endif
 150   continue
c and store
       hkls(lfa,i)=fftscale*real(sf)
       hkls(lfb,i)=fftscale*aimag(sf)
c next reflection
 200  continue
c
c get (000) term
      hkls(lfa,0)=fftscale0*rfl3d(0,0,0)
      hkls(lfb,0)=0.0
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine fft3d(xrho)
      include 'dmmulti.fh'
      include 'xcrystal.fh'
      include 'xhklinfo.fh'
      include 'xtlparam.fh'
c Crystallographic Hermitian to Real FFT
c In:  complex xrfl(h,k,l)  (l=0...nl/2)
c Out: complex xrho(X,Y,Z)  (real)
c
      real xrho(0:nu-1,0:nv-1,0:nw-1)
c
      integer i,j,k,ih,jh,jh1,ik,jk,jk1,lmod
      real fftscale
      logical hemi
      external hemi,lmod
c
      fftscale=1.0/vol
c
c Fill in l=0 plane
c
      do 160 ik=kmin,kmax
      jk =lmod( ik,nv)
      jk1=lmod(-ik,nv)
      do 160 ih=hmin,hmax
      jh =lmod( ih,nu)
      jh1=lmod(-ih,nu)
      if (hemi(ih,ik,0)) then
       xrho(jh1,jk1,0)= xrho(jh,jk,0)
       xrho(jh1,jk1,1)=-xrho(jh,jk,1)
      endif
 160  continue
c
c Now perform hermitian-real FFT and fftscale the result
c
      call fft3hr(xrho,nu,nv,nw)
c
      do 100 k=0,nw-1
      do 100 j=0,nv-1
      do 100 i=0,nu-1
       xrho(i,j,k)=fftscale*xrho(i,j,k)
 100  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine inv3d(xrho)
      include 'dmmulti.fh'
      include 'xcrystal.fh'
      include 'xtlparam.fh'
c Crystallographic Real to Hermitian INVERSE FFT
c In:  complex xrho(X,Y,Z) (real)
c Out: complex xrfl(h,k,l)  (l=0...nl/2)
c
      real xrho(0:nu-1,0:nv-1,0:nw-1)
c
      integer i,j,k
      real fftscale
c
      fftscale=vol/real(nu*nv*nw)
c
c Perform Real-Hermitian FFT and fftscale, conjugate the result
c
      call fft3rh(xrho,nu,nv,nw)
c
      do 100 k=0,nw/2-1
      do 100 j=0,nv-1
      do 100 i=0,nu-1
       xrho(i,j,2*k)  = fftscale*xrho(i,j,2*k)
       xrho(i,j,2*k+1)=-fftscale*xrho(i,j,2*k+1)
 100  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine forallrfl(func,xrfl)
      include 'dmmulti.fh'
      include 'xcrystal.fh'
      include 'xhklinfo.fh'
      include 'xtlparam.fh'
c for all the rfls in the array xrfl perform function 'func'
c
      real xrfl(0:nu-1,0:nv-1,0:nw-1)
c
      integer ih,ik,il,jh,jk,jl,lmod
      real s12,s13,s23,s2,s3,s
      complex fh
c
      external func,lmod
c
c *** optimise this bit ***
      do 100 jl=0,nw/2-1
      il=lmod(jl-lmin,nw)+lmin
      s13 = il*rcm13
      s23 = il*rcm23
      s3 = il**2*rcm33
      do 100 jk=0,nv-1
      ik=lmod(jk-kmin,nv)+kmin
      s12 = ik*rcm12 + s13
      s2 = ik*(ik*rcm22 + s23) + s3
      do 100 jh=0,nu-1
      ih=lmod(jh-hmin,nu)+hmin
      s  = ih*(ih*rcm11 + s12) + s2
      if (s.le.smax) then
       fh=cmplx(xrfl(jh,jk,2*jl),xrfl(jh,jk,2*jl+1))
       call func(ih,ik,il,s,fh)
       xrfl(jh,jk,2*jl)  =real(fh)
       xrfl(jh,jk,2*jl+1)=aimag(fh)
      else
       xrfl(jh,jk,2*jl)  =0.0
       xrfl(jh,jk,2*jl+1)=0.0
      endif
 100  continue
c
c
      return
      end
c
c ----------------------------------------------------------------------
c
C
      INTEGER FUNCTION INASUold(IH, NLAUE)
C     =================================
C
C   This used to be FUNCTION XRY173(NLAUE,IH,ISGN)
C***********************************************************************
C***********************************************************************
C*****                         *****************************************
C***** ROUTINE ( XRY173        *****************************************
C*****                         *****************************************
C*****     From XRAY77 or something like it ****************************
C***********************************************************************
C***********************************************************************
C
C  Arguments:
C   NLAUE - code number for this pointgroup
C   IH(3) - indices
C
C Returns:
C   INASU = +1  if  h k l chosen
C   INASU = -1  if -h-k-l chosen
C   INASU =  0   if reflection is out-of-bounds
C
C  Reciprocal space zones: please check spacegroup numbers.
C
Code:3 pg1     1bar      hkl:l>=0  hk0:h>=0  0k0:k>=0
C           Space group numbers :   1,2
Code:4 pg2    2/m        hkl:k>=0, l>=0  hk0:h>=0       3/b,4/b....
C           Space group numbers :   3 -15 B axis unique.
Code:5 pg2    2/m        hkl:k>=0, l>=0  h0l:>=0        1003,1004
C           Space group numbers :   3 -15 C axis unique.
Code:6 pg222  mmm        hkl:h>=0, k>=0, l>=0            16 ...
C           Space group numbers :   16-74
Code:7 pg4    4/m        hkl:h>=0, l>=0 with k>=0 if  h=0  and
C                                           k> 0 if  h>0.
C           Space group numbers :   75-88
Code:8 pg422 4/mmm       hkl:h>=0, k>=0, l>=0 and h>=k
C           Space group numbers :   89-142
Code:9  pg3     3bar      hkl:h>=0, k>0  00l:l>0
C           Space group numbers :   143-148
Code:  10 pg312  3/m        hkl:h>=0, k>=0 with k<=h for all l.
C                           if k = 0  l>=0
C           Space group numbers :   149-151-153
Code:  11 pg321  3/m        hkl:h>=0, k>=0 with k<=h for all l.
C                           if h = k  l>=0
C           Space group numbers :   150-152-154
Code:12 pg6    6/m        hkl:h>=0, k>=0, l>=0 with k>=0 if  h=0
C                                            and  k> 0 if  h>0.
C           Space group numbers :   168-176
Code:13 pg622 6/mmm       hkl:h>=0, k>=0, l>=0 with h>=k
C           Space group numbers :   177-194
Code:14 pg23   m3         hkl:h>=0, k>=0, l>=0 with l>=h,
C                                                 k>=h if l=h
C                                                 k> h if l>h.
C           Space group numbers :   195-206
Code:15 pg432  m3m        hkl:h>=0, k>=0, l>=0 with k>=l, and l>=h.
C           Space group numbers :   207-232
C
C -- TEST FOR HKL IN ASYMMETRIC UNIT
C     INCORPORATED INTO DATCO5 JUNE 70
C
C Arguments
      INTEGER IH(3), NLAUE
C Locals
      INTEGER J,K,L,ISGN
C
C----MOVE INDICES INTO J,K, AND L
      J = IH(1)
      K = IH(2)
      L = IH(3)
      ISGN=1
C----GO TO TEST IF INDICES ARE IN DESIRED UNIT
1     GO TO (100,150,200,250,300,350,400,450,550,570,600,650,700,750,
     1  800),NLAUE
 100  CONTINUE
C----1 BAR (ALTERNATE 1)
C     HKL --    H.GE.0
C     0KL --    K.GE.0
C     00L --    L.GE.0
      IF (J) 7,105,5
 105  IF (K) 7,110,5
 110  IF (L) 7,5,5
C
 150  CONTINUE
C----1 BAR (ALTERNATE I)
C     HKL --    K.GE.0
C     H0L --    L.GE.0
C     H00 --    H.GE.0
      IF (K) 7,155,5
 155  IF (L) 7,160,5
 160  IF (J) 7,5,5
C
C  Corresponds to Data reduction unique set for  pg1
C   3 pg1     1bar      hkl:l>=0  hk0:h>=0  0k0:k>=0   1,2
 200  CONTINUE
C----1 BAR (ALTERNATE 3)
C     HKL --    L.GE.0
C     HK0 --    H.GE.0
C     0K0 --    K.GE.0
      IF (L) 7,205,5
 205  IF (J) 7,210,5
 210  IF (K) 7,5,5
C
C  Corresponds to Data reduction unique set for  pg2
C   4 pg2    2/m        hkl:k>=0, l>=0  hk0:h>=0       3/b,4/b....
 250  CONTINUE
C----2/M (ALTERNATE 1)
C     HKL --    K.GE.0 AND L.GE.0
C     HK0 --    H.GE.0
      IF (K) 7,255,255
 255  IF (L) 7,260,5
 260  IF (J) 7,5,5
C
C  Corresponds to Data reduction unique set for  pg2
C   5 pg2    2/m        hkl:k>=0, l>=0  h0l:h>=0       1003,1004
 300  CONTINUE
C----2/M (ALTERNATE 2)
C     HKL --    L.GE.0 AND H.GE.0
C     H0L --    H.GE.0
      IF (L) 7,305,305
 305  IF (K) 7,310,5
 310  IF (J) 7,5,5
C
C  Corresponds to Data reduction unique set for  pg222
C   6 pg222  mmm        hkl:h>=0, k>=0, l>=0            16 ...
 350  CONTINUE
C----MMM
C     HKL --    H.GE.0, K.GE.0, AND L.GE.0
      IF (J) 7,355,355
 355  IF (K) 7,361,361
C****NO 3-SICKLIES PLEASE
 361  IF (L) 7,5,5
C
C  Corresponds to Data reduction unique set for  pg4
C   7 pg4    4/m        hkl:h>=0, l>=0 with k>=0 if  h=0  and
C                                           k> 0 if  h>0.
 400  CONTINUE
C----4/M
C     HKL --    H.GE.0, L.GE.0, WITH K.GE.0 IF H.EQ.0 OR
C               K.GE.1 IF H.GT.0
      IF (L) 7,405,405
 405  IF (J) 7,410,415
 410  IF (K) 7,5,5
 415  IF (K) 7,7,5
C
C  Corresponds to Data reduction unique set for  pg422
C   8 pg422 4/mmm       hkl:h>=0, k>=0, l>=0 and h>=k   89..
 450  CONTINUE
C----4/MMM
C     HKL --    H.GE.0, K.GE.0, AND L.GE.0 WITH H.GE.K
      IF (J) 7,455,455
 455  IF (K) 7,460,460
 460  IF (L) 7,465,465
 465  IF (J-K) 7,5,5
C
C 500  CONTINUE
C----3 BAR
C      HKL ---   H.LE.0, L.GE.0, WITH K.GE.0 IF H.LE.0 OR
C               K.GE.1 IF H.LT.0
C      HK0  --  K.GT.-H
CC
C  1978 CHANGED TO FIT N.ISAACS
C  INCLUDE 0 0 L,  HK0 K.LT.0 H.GE.-K     HKL H.GE.0 K.LT.0
CC
CCC      IF(J)7,505,505
CCC505   IF(K)510,510,7
CCC510   IF(L)7,515,520
CCC515   IF(K)516,7,7
CCC516   IF(J+K)7,5,5
CCC520   IF(J-K)521,5,521
CCC521   IF(K)5,7,7
C
C  Corresponds to Data reduction unique set for  pg3
C  9 pg3     3bar      hkl:h>=0, k>0  00l:l>0         143..
550   CONTINUE
C   ALTERNATIVE FOR R3
C  H.GE.0 K.GT.0    ALL L
C  H=0 K=0 L.GT.0
      IF(J)7,555,556
555   IF(K)7,557,5
556   IF(K)7,7,5
557   IF(L)7,7,5
C
C  Corresponds to Data reduction unique set for  pg312 
C  10 pg312        hkl:h>=0, k>=0 with k<=h for all l.
C                           if k = 0  l>=0
 570  CONTINUE
C----3 BAR M
C  H.GE.0 K.GE.0 K.LE.H   ALL L
C  H=K  L.GE.0
      IF(J)7,575,575
575   IF(K)7,577,576
576   IF(J-K)7,5,5
577   IF(L)7,5,5
C
C  Corresponds to Data reduction unique set for  pg321
C  11 pg321        hkl:h>=0, k>=0 with k<=h for all l.
C                           if h = k  l>=0
 600  CONTINUE
C----3 BAR M
C  H.GE.0 K.GE.0 K.LE.H   ALL L
C  H=K  L.GE.0
      IF(J)7,605,605
605   IF(K)7,606,606
606   IF(J-K)7,607,5
607   IF(L)7,5,5
C
C  Corresponds to Data reduction unique set for  pg6
C  12 pg6    6/m        hkl:h>=0, k>=0, l>=0 with k>=0 if  h=0
C                                            and  k> 0 if  h>0.
 650  CONTINUE
C----6/M
C     HKL --    H.GE.0, L.GE.0, WITH K.GE.0 IF H.EQ.0 OR
C               K.GE.1 IF H.GT.0
      IF (L) 7,655,655
 655  IF (J) 7,660,665
 660  IF (K) 7,5,5
 665  IF (K) 7,7,5
C
C  Corresponds to Data reduction unique set for  pg622
C  13 pg622 6/mmm       hkl:h>=0, k>=0, l>=0 with h>=k 177..
  700 CONTINUE
C----6/MMM
C     HKL --    H.GE.0, K.GE.0, AND L.GE.0 WITH H.GE.K
      IF (J) 7,705,705
 705  IF (K) 7,710,710
710   IF(J-K)7,715,715
715   IF(L)7,5,5
C
C  Corresponds to Data reduction unique set for  pg23
C  14 pg23   m3         hkl:h>=0, k>=0, l>=0 with l>=h,
C                                                 k>=h if l=h
C                                                 k> h if l>h.
 750  CONTINUE
C----M3
C     HKL --    H.GE.0, K.GE.0, AND L.GE.0 WITH L.GE.H AND WITH
C               K.GE.H IF L.EQ.H OR K.GT.H IF L.GT.H
      IF (J) 7,755,755
 755  IF (K) 7,760,760
 760  IF (L) 7,765,765
 765  IF (L-J) 7,770,775
 770  IF (K-J) 7,5,5
 775  IF (K-J) 7,7,5
C
C  Corresponds to Data reduction unique set for  pg432
C  15 pg432  m3m        hkl:h>=0, k>=0, l>=0 with k>=l, and l>=h.
 800  CONTINUE
C----M3M
C     HKL --    H.GE.0, K.GE.0, AND L.GE.0 WITH
C              K.GE.L AND L.GE.H
      IF (J) 7,805,805
 805  IF (K) 7,810,810
 810  IF (L) 7,815,815
 815  IF (K-L) 7,820,820
 820  IF (L-J) 7,5,5
C
C----REFLECTION IS IN BOUNDS
 5    INASUold = ISGN
      RETURN
C     ======
C
C----REFLECTION IS OUT OF BOUNDS
 7    IF (ISGN .EQ. +1) THEN
C     TRY -H -K -L
        ISGN=-1
        J=-J
        K=-K
        L=-L
        GO TO 1
      ENDIF
C Index failed, exit
      INASUold = 0
      RETURN
C     ======
C
      END
C
c
c ----------------------------------------------------------------------
c
