C
C     DTREK2MTZ
C     Copyright (C) 2000 Jim Pflugrath
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
      PROGRAM DTREK2MTZ
C
C     This produces an HKLIN file suitable for input to TRUNCATE from
C     an ASCII d*TREK reflnlist file.  Binary d*TREK reflnlist files should
C     be converted to text/ASCII with 'dtreflnmerge infile outfile -text'
C
C     A d*TREK ASCII reflnlist has a header and columns
C       nH nK nL [nOther ...] fIntensity fSigmaI 
C                            [fIntensity+ fSigmaI+ fIntensity- fSigmaI-]
C                            [fOther ...]
C     in FREE FORMAT.  This takes a dtscalemerge output reflection file
C     Assumption: if I+ and I- are on separate lines, then I- appears first.
C     This assumption is checked and a count of the failure of the assumption
C     is output AND I+/I- are swapped.  BUT if this happens, you should
C     be very wary of the resultant mtz file since d*TREK designates I+ and I-
C     properly.
C
C     These are converted to the weighted means Imean and SigImean
C     Imean = (I+/SIGI+  +  I-/SIGI-) / (1/SIGI+  +  1/SIGI-)
C     SigImean = 1 / (1/SIGI+  +  1/SIGI-)
C
C     Authors: Eleanor Dodson, Adam Ralph
C
C     Changes: Martyn Winn (Dec. 1996) missing data handling
C                                      ANOM/CELL keywords
C
C              J. W. Pflugrath (Dec. 1999) Modified from SCALEPACK2MTZ
C              J. W. Pflugrath (Jan. 2001) Modified for v7 d*TREK reflnlists
C
      IMPLICIT NONE
      INTEGER NPARM, MCOLS, NREJ, IREJ
      PARAMETER (NPARM=200, MCOLS=500,NREJ=200)
      REAL SCFF
      REAL SSQ4, FPLUS, SIGFPLUS, FNEG, SIGFNEG,
     +     FHIGH, SIGIMEAN
      INTEGER IFAIL, NSYM, NSYMP, NLAUE, NCHK, 
     +        IRECL, I, N
      REAL RSYM(4, 4, 192)
C
      REAL    DTREKIN(MCOLS), FTEMP, SIGFTEMP
      INTEGER NUMINT, NUMFLT, NUMSTR, IDXANO, IHKLD(3), JHKL(3), ISYM
      INTEGER NUMCRY
      INTEGER ISTAT
      INTEGER NUMSWP
      LOGICAL LDREAD
C
      REAL ADATA(MCOLS), CELL(6), RESMIN, RESMAX, SMIN,
     +     SMAX, DUMMY, WAVEL
      LOGICAL LFIN
      INTEGER NLPRGO, NREAD, NREF, KANOM,
     +        NMON, IHIGH, KHIGH, 
     +        LHIGH, II, NSPGRP,
     +        NSMP, NEGSIG, MTZOUT
      INTEGER IHKL(3), HKLREJ(3,NREJ)
      INTEGER ICELLIN, ISYMMIN
      LOGICAL CCPONL, LWAVEL
      CHARACTER*80 TITLE
      CHARACTER date*8,date6*6
      CHARACTER*10 NAMSPG, PGNAME
      character*64 PROJECTNAME,CRYSTALNAME,DATASETNAME
      character*64 XNAME(mcols),DNAME(mcols)

      CHARACTER CVALUE(NPARM)*4, LSPRGO(MCOLS)*30, CTPRGO(MCOLS)*1,
     +          HISOUT(20)*80, DATEMT*8, CTIME*8
      CHARACTER LTYPE*1
C     ..
C---- things for parser
C
      CHARACTER KEY*4, SUBKEY*4, ERRLINE*200, LINE*600, CWORK*20
      REAL FVALUE(NPARM)
      LOGICAL LEND, IPRINT, FIRST
      INTEGER LUNOUT, ITOK, NTOK, IBEG(NPARM), IEND(NPARM),
     +     ITYPE(NPARM), IDEC(NPARM)
C
C---- External Routines and Functions
C
      REAL LSTLSQ
      INTEGER LENSTR, LUNSTO
      EXTERNAL ASUSET,CCPDAT,CCPDPN,CCPERR,CCPFYP,CCPONL,CCPRCS,
     +         CCPUPC,GTPINT,GTPREA,LENSTR,LSTLSQ,LSTRSL,
     +         LUNSTO,LWCELL,LWCLAB,LWCLOS,LWHIST,LWHSTL,LWOPEN,
     +         LWREFL,LWSYMM,LWTITL,MTZINI,PARSER,RDRESO,RDSYMM,
     +         CCP4H_INIT,CCP4H_TOC_BEG,CCP4H_TOC_ENT,CCP4H_TOC_END,
     +         CCP4H_PRE_BEG,CCP4H_RULE,CCP4H_LINK,CCP4H_LINK_KEY,
     +         CCP4H_HEADER,CCP4H_PRE_END
C
C---- Data Statements
C
      DATA IPRINT/.TRUE./, FIRST/.TRUE./
      DATA LSPRGO/'H', 'K', 'L', 'IMEAN', 'SIGIMEAN', 'I(+)', 'SIGI(+)',
     +            'I(-)', 'SIGI(-)', 491*' '/
      DATA CTPRGO/'H', 'H', 'H', 'J', 'Q', 'K', 'M', 'K', 'M',
     +            491*' '/
      DATA FHIGH /0.0/
      DATA TITLE /' '/, NAMSPG /' '/
C
      LUNOUT = LUNSTO(1)
      CALL MTZINI
      CALL CCPFYP
      call ccp4h_init()
      call ccp4h_header('dtrek2mtz','dtr',2)
      call ccp4h_pre_beg() 
      CALL CCPRCS(LUNOUT, 'DTREK2MTZ',
     +     '$Date: 2005/09/06 11:25:11 $')
      WRITE (LUNOUT, '(///,a,a,/,10X,a,/)')
     +  ' ********* PROGRAM TO PREPARE a d*TREK',
     +  ' reflnlist TO GO INTO TRUNCATE *********',
     +  ' (you will need to run SORTMTZ or CAD afterwards)'
      call ccp4h_pre_end()
      call ccp4h_toc_beg()
      call ccp4h_toc_ent('Command input','#command')
      call ccp4h_toc_ent('Input and output files','#io')
      call ccp4h_toc_ent('Cell Details','#cel')
      call ccp4h_toc_ent('Format conversion','#conversion')
      call ccp4h_toc_ent('Header for output MTZ files','#out')
      call ccp4h_toc_end()
c    
C Set default PROJECTNAME, CRYSTALNAME and DATASETNAME
      CALL CCPDAT(DATE)
      CALL UTIME(CTIME)
      WRITE(DATE6,'(A2,A2,A2)') DATE(1:2),DATE(4:5),DATE(7:8)
 5    I = INDEX(DATE6,' ')
      IF (I.GT.0) THEN
        WRITE(DATE6(I:I),'(A1)') '0'
        GOTO 5
      ENDIF
      PROJECTNAME   = 'unknown'
      CRYSTALNAME   = 'unknown'
      DATASETNAME   = 'unknown'//DATE6//':'//CTIME
      NSYM = 0
      NREAD = 0
      NREF = 0
      NUMSWP = 0
      IRECL = 0
      KANOM = 1
      IREJ = 0
      SCFF = 1.0
      SMIN = 0.0
      SMAX = 10000.0
      WAVEL = 0.0
      FPLUS = 0
      FNEG = 0
      MTZOUT = 1
      LFIN = .FALSE.
      LWAVEL=.FALSE.
      NMON = 1000
      ICELLIN= 0
      ISYMMIN= 0
      CELL(1) = 0.0
      CELL(2) = 0.0
      CELL(3) = 0.0
      CELL(4) = 90.0
      CELL(5) = 90.0
      CELL(6) = 90.0
      call ccp4h_rule()
      call ccp4h_header('Command Input','command',3)
      call ccp4h_pre_beg()
      call ccp4h_link('SYMM      ','dtrek2mtz.html#symmetry')
      call ccp4h_link('CELL      ','dtrek2mtz.html#cell')
      call ccp4h_link('ANOMALOUS ','dtrek2mtz.html#anomalous')
      call ccp4h_link('MONITOR   ','dtrek2mtz.html#monitor')
      call ccp4h_link('RESOLUTION','dtrek2mtz.html#resolution')
      call ccp4h_link('SCALE     ','dtrek2mtz.html#scale')
      call ccp4h_link('TITLE     ','dtrek2mtz.html#title')
      call ccp4h_link('REJECT    ','dtrek2mtz.html#reject')
      call ccp4h_link('NAME      ','dtrek2mtz.html#name')
      call ccp4h_link('WAVELENGTH','dtrek2mtz.html#wavel')
      IF (CCPONL (1)) THEN
        WRITE (LUNOUT, '(//,A/,A)')
     +  ' Compulsory inputs are SYMMETRY followed by space group no.',
     +  '       PROJECTNAME, CRYSTALNAME, DATAsetNAME and Wavelength.'
        WRITE (LUNOUT, '(/,a,a)')
     +  '   CELL followed by cell dimensions'
        WRITE (LUNOUT, '(///,a)') ' Optional input follows.'
        WRITE (LUNOUT, '(/,a)') '   TITLE followed by title'
        WRITE (LUNOUT, '(/,a)')
     +  '   SCALE followed by scale factor for FSQ of input file'
        WRITE (LUNOUT, '(/,a,a)')
     +  '   MONITOR followed by an integer nmon.',
     +  ' Every nmon-th reflection monitored '
        WRITE (LUNOUT, '(/,a)')
     +  '   ANOMALOUS YES|NO (default YES)  '
        WRITE (LUNOUT, '(/,a,a)')
     +  '   RESOlution followed by 2 real numbers  ',
     +  ' resmin resmax (As or 4s**2/l**2)'
        WRITE (LUNOUT, '(/,a,a,/,a,a)')
     +  '   REJECT followed by 3 numbers:',
     +  ' These give indices h k l AFTER conversion to asymmetric',
     +  ' unit of reflections to reject.'
        WRITE (LUNOUT, '(/,a)')
     +  '   END ends input and starts program  '
      ENDIF
C
   30 CONTINUE
      NTOK = NPARM
      LINE = ' '
      CALL PARSER(KEY, LINE, IBEG, IEND, ITYPE, FVALUE, CVALUE, IDEC,
     +            NTOK, LEND, IPRINT)
      IF (LEND) GO TO 70
C
C
C----- TITLE
C      =====
C
      IF (KEY.EQ.'TITL') THEN
        TITLE = LINE(IBEG(2) :80)
         goto 30

c     ==========================================
c---- name project <pname> crystal <xname> dataset <dname>
c     ==========================================
c---- assign project/crystal/dataset names for new mtz file

      ELSEIF (KEY.EQ.'NAME') THEN

        if (ntok.lt.7) then
          call ccperr(2,'NAME keyword line incomplete.')
          goto 30
        endif

        n = 2
 451    if (n .le. ntok) then
         subkey = cvalue(n)(1:4)
         call ccpupc(subkey)
         if (subkey .eq. 'PROJ') then
c  set current pname
            PROJECTNAME = line(ibeg(n+1):iend(n+1))
            n = n+2
         elseif (subkey .eq. 'CRYS') then
c  set current xname
            CRYSTALNAME = line(ibeg(n+1):iend(n+1))
            n = n+2
         elseif (subkey .eq. 'DATA') then
            DATASETNAME = line(ibeg(n+1):iend(n+1))
            n = n+2
         else
            write(errline,'(a,a)') 
     +            'wrong subkeyword after NAME: ',cvalue(n)
            call ccperr(1,errline)
         endif
         go to 451
        endif

        go to 30
C
C     ==========================================
C---- PNAME    >> PROJECTNAME [pname]  <string>
C     ==========================================
C
      ELSEIF (KEY.EQ.'PNAM'.OR.KEY.EQ.'PROJ') THEN
         if (ntok .ge. 2) PROJECTNAME = line(ibeg(2):iend(2))
         goto 30
C
C     ===========================================
C---- XNAME    >> CRYSTALNAME  [xname]  <string>
C     ===========================================
C     
      ELSEIF (KEY.EQ.'XNAM'.OR.KEY.EQ.'CRYS') THEN
         if (ntok .ge. 2) CRYSTALNAME = line(ibeg(2):iend(2))
         goto 30
C
C     ===========================================
C---- DNAME    >> DATASETNAME  [dname]  <string>
C     ===========================================
C
      ELSEIF (KEY.EQ.'DNAM'.OR.KEY.Eq.'DATA') THEN
         if (ntok .ge. 2) DATASETNAME = line(ibeg(2):iend(2))
         goto 30
C
C--- WAVElength
C    ==========
C
C
      ELSEIF (KEY.EQ.'WAVE') THEN
        CALL GTPREA(2, WAVEL, NTOK, ITYPE, FVALUE)
         LWAVEL=.TRUE.
         goto 30
C
C---- RESOlution
C     ==========
C
C     read resolution limits in A, if only one treat as high
C     resolution limit
C
      ELSE IF (KEY.EQ.'RESO') THEN
        ITOK = 2
C
C       *****************************************************
        CALL RDRESO(ITOK, ITYPE, FVALUE, NTOK, RESMIN, RESMAX, SMIN,
     +              SMAX)
C       *****************************************************
C
        WRITE (LUNOUT, FMT='(A,2F10.5)')
     +       '  Resolution limits: SMIN SMAX', SMIN, SMAX
C
C---- MONITOR information
C     ===================
C
      ELSE IF (KEY.EQ.'MONI') THEN
        CALL GTPINT(2, NMON, NTOK, ITYPE, FVALUE)
        WRITE (LUNOUT, '(/,A,I8,a/)') ' Every ', NMON,
     +    'th reflection listed'
C
C---- SCALE information
C     =================
C
      ELSE IF (KEY.EQ.'SCAL') THEN
        CALL GTPREA(2, SCFF, NTOK, ITYPE, FVALUE)
        WRITE (LUNOUT, '(/,A,/,10X,F8.2,/)') ' Output fsq  scaled by ',
     +    SCFF
      ELSE IF (KEY.EQ.'ANOM') THEN
        IF (NTOK.LE.1) THEN
          WRITE (LUNOUT,FMT=6010)
 6010     FORMAT (//,' **** KEYWORD WARNING ****',/,
     +  ' No argument given on ANOMALOUS KEY, using default of YES')
          KANOM = 1
        ELSE IF (ITYPE(2).EQ.2) THEN
          WRITE (LUNOUT,FMT=6012)
 6012     FORMAT (//,' **** KEYWORD WARNING ****',/,
     +' Integer argument given on ANOMALOUS KEY, using default of YES')
          KANOM = 1
        ELSE
          CWORK = LINE(IBEG(2) :IEND(2))
C
C              *************
          CALL CCPUPC(CWORK)
C              *************
C
          IF (CWORK(1:3).EQ.'YES') KANOM = 1
          IF (CWORK(1:2).EQ.'NO') KANOM = 0
        END IF
C
C---- CELL information
C     ================
C
      ELSE IF (KEY.EQ.'CELL') THEN
         CALL RDCELL(2,ITYPE,FVALUE,NTOK,CELL)
         ICELLIN=1
C
C---- REJECT information
C     ==================
C
      ELSE IF (KEY.EQ.'REJE') THEN
        WRITE (LUNOUT, '(/,A,/,10X,I8/)')
        IREJ = IREJ + 1
        CALL GTPINT(2, HKLREJ(1,IREJ), NTOK, ITYPE, FVALUE)
        CALL GTPINT(3, HKLREJ(2,IREJ), NTOK, ITYPE, FVALUE)
        CALL GTPINT(4, HKLREJ(3,IREJ), NTOK, ITYPE, FVALUE)
        WRITE (LUNOUT, '(/,A,/,10X,3I4,/)')
     +    ' Reject reflections with indices: ', (HKLREJ(I,IREJ),I=1,3)
C
C---- SYMMETRY
C     ========
C
      ELSE IF (KEY.EQ.'SYMM') THEN
        ITOK = 2
        CALL RDSYMM(ITOK, LINE, IBEG, IEND, ITYPE, FVALUE, NTOK, NAMSPG,
     +              NSPGRP, PGNAME, NSYM, NSYMP, RSYM)
        LTYPE = NAMSPG(1:1)
        NSMP = NSYMP
        ISYMMIN = 1
C
C---- END
C     ===
C
      ELSE IF (KEY.EQ.'END') THEN
        GO TO 70
      ELSE
        CALL CCPERR(1, ' Unrecognised keyword')
      END IF
C     next keyword
      GO TO 30
C
   70 CONTINUE
C
      IF (KANOM .EQ. 0) THEN
C----    No anomalous data present
         WRITE (LUNOUT, '(/,/,a,/,5X,a,/,/)') 
     +  ' No anomalous data expected in input file; ',
     +  ' output file has H K L IMEAN SIGIMEAN'
         NLPRGO = 5
      ELSE
C----    Anomalous data present
         WRITE (LUNOUT, '(/,/,a,/,5X,a,a,/,/)') 
     +  ' Anomalous data expected in input file; ',
     +  ' output file has H K L IMEAN SIGIMEAN I(+) SIGI(+)', 
     +  ' I(-) SIGI(-)'
         NLPRGO = 9
      END IF

      call ccp4h_pre_end()
      NEGSIG = 0
      IFAIL = 0
      call ccp4h_rule() 
      call ccp4h_header('Input and Output files','io',3)
      call ccp4h_pre_beg()
C
C---- Open the ASCII reflnlist file from d*TREK
C
C          =================================================
      CALL CCPDPN(21, 'HKLIN ', 'READONLY', 'F', 0, IFAIL)
C          =================================================
C Determine d*TREK columns etc
C
      READ (21, '(A)') LINE
C
C See if there are 3 (pre-v7) or 4 (d*TREK v7 and higher) numbers on the line
C
      READ (LINE, *, IOSTAT=ISTAT) NUMINT, NUMFLT, NUMSTR, NUMCRY
      IF (0 .NE. ISTAT) THEN
         READ (LINE, *, IOSTAT=ISTAT) NUMINT, NUMFLT, NUMSTR
         NUMCRY = 0
      ENDIF
      IF (0 .NE. ISTAT) THEN
         goto 140
      ENDIF
C
C Read or skip crystal information lines. Save spacegroup and cell info
C
      DO 45 I = 1, NUMCRY
         READ (21, '(A)') LINE
         IF ( (1 .NE. ICELLIN) 
     +      .AND. ('CRYSTAL_UNIT_CELL=' .EQ. LINE(1:18)) ) THEN
C
C Read unit cell only if input did not specify unit cell parameters,
C (that is, ignore unit cell in input file if CELL card already specified)
C
            READ (LINE(19:(LENSTR(LINE)-1)), *) CELL
            ICELLIN = 2
         ELSE IF ( (1 .NE. ISYMMIN)
     +            .AND. ('CRYSTAL_SPACEGROUP=' .EQ. LINE(1:19)) ) THEN
C
C Read spacegroup number only if input did not specify unit cell parameters,
C (that is, ignore spacegroup in input file if SYMM card already specified)
C
           LINE = 'SYMM ' // LINE(20:)
           ITOK = INDEX(LINE, ';')
           LINE(ITOK:ITOK) = ' '
           IBEG(1) = 1
           IEND(1) = 4
           IBEG(2) = 6
           IEND(2) = ITOK - 1
           ITYPE(1) = 1
           ITYPE(2) = 2
           FVALUE(1) = 0.0
           NSYM = 0
           READ (LINE(IBEG(2):IEND(2)), *) FVALUE(2)
           ITOK = 2    
           NTOK = 2
           CALL RDSYMM(ITOK, LINE, IBEG, IEND, ITYPE, FVALUE, NTOK, 
     +                 NAMSPG, NSPGRP, PGNAME, NSYM, NSYMP, RSYM)
           LTYPE = PGNAME(1:1)
           NSMP  = NSYMP
           ISYMMIN = 2
         ENDIF
 45   CONTINUE
C
C Skip fields, but save index of fIntensity+ field
C
      IDXANO = 0
      DO 50 I = 1, NUMINT+NUMFLT+NUMSTR
         READ (21, '(A)') LINE
         IF ('fIntensity+' .EQ. LINE(1:11)) THEN
            IDXANO = I-3
            WRITE (LUNOUT, '(/,A,/)') ' Anomalous info on SAME line!'
         ENDIF
  50  CONTINUE
C
C If KANOM is not set for anomalous, reset IDXANO to 0 if it is set
C
      IF ( (0 .EQ. KANOM) .AND. (0 .NE. IDXANO) ) THEN
         WRITE (LUNOUT, '(/, A, /)') 
     +          'WARNING! Ignoring anomalous info in input file!'
         IDXANO = 0
      ENDIF
C
C
C---- Open output MTZ file
      MTZOUT = 1
C
C     HKLOUT (OUT) output file
C
C     Column labels (anomalous data):
C
C     H K L      indices
C     IMEAN, SIGIMEAN    weighted mean intensity between I+ and I- and sig
C     I(+), SIGI(+)      intensity and sig for hkl
C     I(-), SIGI(-)      intensity and sig for -h -k -l
C
C     Column labels (no anomalous data):
C
C     H K L      indices
C     I, SIGI    mean intensity and sig
C
C---- CALL to open an MTZ file for write.
C     =================================
C
      IF(.NOT.LWAVEL) CALL CCPERR(2,' No wavelength provided')
      CALL LWOPEN(MTZOUT, 'HKLOUT')
      CALL LWTITL(MTZOUT, TITLE, 0)
      CALL LWCLAB(MTZOUT, LSPRGO, NLPRGO, CTPRGO, 0)
      CALL LWHSTL(MTZOUT,'from DTREK2MTZ')
C
C---- No Key_Word HISTORY therefore just add one new history line
C
      CALL CCPDAT(DATEMT)
      HISOUT(1) = ' From DTREK2mtz run on '//DATEMT
C
C     ================================
      CALL LWHIST(MTZOUT, HISOUT, 1)
C     ================================
C
C       A d*TREK reflnlist file usually starts like this:
C    nNumIntCols nNumFloatCols nNumStringCols
C    column_names
C    ...
C   4   0   3 51178.9  2796.6
C
      call ccp4h_rule() 
      call ccp4h_header('Cell and Symmetry  Details','cel',3) 
      call ccp4h_pre_beg()
      IF (1 .EQ. ICELLIN) THEN
         WRITE (LUNOUT, '(/,A,/,10X,6F8.2/)') 
     +  ' Cell from CELL keyword ', CELL
      ELSE IF (2 .EQ. ICELLIN) THEN
         WRITE (LUNOUT, '(/,A,/,10X,6F8.2/)') 
     +  ' Cell from d*TREK file ', CELL
      ELSE
         CALL CCPERR(1, ' You must supply CELL input!')
      ENDIF
C
C---- check CELL given in input file or keyworded input
      IF(CELL(1).LT.0.001) 
     +     CALL CCPERR(1, 'You MUST give your CELL dimensions')
      CALL LWCELL(MTZOUT, CELL)
      CALL LSTRSL(MTZOUT, CELL(1), CELL(2), CELL(3), CELL(4), CELL(5),
     +            CELL(6))

C Store the project, crystal and dataset names in the mtz header:
      if (PROJECTNAME.eq.'unknown')
     +   call ccperr(2,' PROJECTNAME not assigned') 
      if (CRYSTALNAME.eq.'unknown')
     +   call ccperr(2,' CRYSTALNAME not assigned') 
      if (DATASETNAME(1:7).eq.'unknown')
     +   call ccperr(2,' DATASETNAME not assigned') 
      call lwidx(MTZOUT,PROJECTNAME,CRYSTALNAME,DATASETNAME,CELL,WAVEL)
C
C Associate columns with datasets
      do i=1,nlprgo
        xname(i) = CRYSTALNAME
        dname(i) = DATASETNAME
      enddo
      call lwidasx(MTZOUT, nlprgo, xname, dname , 0)
C     
      IF (NAMSPG.EQ.' ') THEN
         CALL CCPERR (1, 'You must specify the SYMMETRY')
      ELSE IF (1 .EQ. ISYMMIN) THEN
         WRITE (LUNOUT, '(/,A,/)') ' Symmetry from SYMM keyword '
      ELSE IF (2 .EQ. ISYMMIN) THEN
         WRITE (LUNOUT, '(/,A,A,I3,/)') ' Symmetry from d*TREK file,',
     +                             ' spacegroup number is ', NSPGRP
      ENDIF  
      CALL LWSYMM(MTZOUT, NSYM, NSYMP, RSYM, LTYPE, NSPGRP, NAMSPG,
     +            PGNAME)
C
C---- Set up symmetry (RDSYMM or LRSYMM must be called first )
C
      CALL ASUSET(NAMSPG, NSPGRP, PGNAME, NSYM, RSYM, NSYMP, NLAUE,
     +            .TRUE.)
C
C---- Set up a test for centric/acentric test
C
      CALL CENTRIC(NSYM, RSYM, 0)

      DO 60 I = 1, MCOLS
        DTREKIN(I) = 0.0
 60   CONTINUE
C
C     LDREAD is TRUE if a refln has been read and is buffered, so that
C                    you do not need to read a new refln
C
      LDREAD = .FALSE.
C
C  ...loop over reflections
C
      call ccp4h_rule() 
      call ccp4h_header('Format Conversion','conversion',3)
   90 CONTINUE
      FPLUS    = 0.0
      SIGFPLUS = 0.0
      FNEG     = 0.0
      SIGFNEG  = 0.0
      IF (LDREAD) THEN 
C
C        Use the previously read info
C         
         IHKL(1) = IHKLD(1)
         IHKL(2) = IHKLD(2)
         IHKL(3) = IHKLD(3)
         DO 100 I = 1, NUMFLT+NUMINT-3
            DTREKIN(I) = DTREKIN(I+100)
 100     CONTINUE
         LDREAD = .FALSE.
C         WRITE (LUNOUT, '(A, 3I5)') 'Old ref: ', IHKL(1), IHKL(2), IHKL(3)
      ELSE
         READ (21, *, END=120, ERR=140) IHKL, 
     +        (DTREKIN(I), I=1, NUMFLT+NUMINT-3)
         NREAD    = NREAD + 1
      ENDIF

      FPLUS    = DTREKIN(NUMINT-3+1)
      SIGFPLUS = DTREKIN(NUMINT-3+2)
      IF (IDXANO .NE. 0) THEN
C
C        fIntensity+ and fIntensity- found in reflnlist
C
         FPLUS    = DTREKIN(IDXANO)
         SIGFPLUS = DTREKIN(IDXANO+1)
         FNEG     = DTREKIN(IDXANO+2)
         SIGFNEG  = DTREKIN(IDXANO+3)
      ELSE IF (KANOM .NE. 0) THEN
C
C        Read next refln if and only if anomalous and IDXANO = 0
C        Note: There is no guarantee that I+ comes before I- in the file
C              although they should be paired if KANOM .ne. 0.
C              So check later if FPLUS and FNEG need to be swapped.
C
         FNEG    = 0.0
         SIGFNEG = 0.0
         READ (21, *, END=110, ERR=140) IHKLD, 
     +        (DTREKIN(I+100), I=1, NUMFLT+NUMINT-3)
C
C        Problem: if this is last of pair, then this one does not
C        get written out
C
         NREAD = NREAD + 1
         LDREAD  = .TRUE.
C
C     Compare IHKL and IHKLD to see if they are Friedel mates
C
         IF (      (IHKL(1) .EQ. -IHKLD(1)) 
     +       .AND. (IHKL(2) .EQ. -IHKLD(2)) 
     +       .AND. (IHKL(3) .EQ. -IHKLD(3)) ) THEN
C
C     If IHKLD is Friedel of IHKL, then save Intensity and SigmaI
C
            FNEG    = DTREKIN(101 + NUMINT-3)
            SIGFNEG = DTREKIN(102 + NUMINT-3)
            LDREAD  = .FALSE.
C
C           Input file is normally I- first, then I+ on next line,
C           so swap I+ and I-, and select the hkl that belongs to I+
C
            FTEMP    = FNEG
            SIGFTEMP = SIGFNEG
            FNEG     = FPLUS
            SIGFNEG  = SIGFPLUS
            FPLUS    = FTEMP
            SIGFPLUS = SIGFTEMP
            IHKL(1)  = IHKLD(1)
            IHKL(2)  = IHKLD(2)
            IHKL(3)  = IHKLD(3)
C         ELSE
C
C           OK, we have only one of I+ or I-, but we cannot be sure of which
C           one it is... unless we check the HKL carefully.
C           Below we will check HKL and decide if it's I+ or I-
         ENDIF

      ENDIF
C
C  At this point we have an IHKL, but is it I+ or I-?
C
 110  CONTINUE
      ISYM = 1
      CALL ASUPUT(IHKL, JHKL, ISYM)
      IF (0 .EQ. MOD(ISYM, 2)) THEN
C
C        IHKL is really an I-, so swap FPLUS and FNEG, and use JHKL
C        Unfortunately, we have centric reflections that might be in here
C        and the CCP4 lib does not necessarily assign them to I+ and I-
C
         IHKL(1)  = JHKL(1)
         IHKL(2)  = JHKL(2)
         IHKL(3)  = JHKL(3)
         IF (IDXANO.GT.0 .OR. KANOM.GT.0) THEN
           FTEMP    = FNEG
           SIGFTEMP = SIGFNEG
           FNEG     = FPLUS
           SIGFNEG  = SIGFPLUS
           FPLUS    = FTEMP
           SIGFPLUS = SIGFTEMP

           CALL CENTR(IHKL, ISYM)
           IF (1 .EQ. ISYM) THEN
C
C           Centric!
C
C            WRITE (LUNOUT, '(1X, A, 3I5)') 
C     +                     'WARNING! Centric swap hkl: ',
C     +                     IHKL(1), IHKL(2), IHKL(3)
           ELSE IF ( (FNEG .NE. 0.0) .AND. (FPLUS .NE. 0.0) ) THEN
C
C           Warn about swapping if both I+ and I- are present for acentrics
C
              NUMSWP = NUMSWP + 1
C            WRITE (LUNOUT, *) 'Swapped I+, I-'
           ENDIF
         ENDIF
      ENDIF

      IF (NREAD.LE.2) THEN
        IF (KANOM.EQ.0) THEN
C         WRITE (LUNOUT, *) IHKL, (DTREKIN(I), I = 1, 6)
         WRITE (LUNOUT, 9130) IHKL, FPLUS, SIGFPLUS
 9130    FORMAT (/,' First (or second) reflection  of d*TREK:',
     +           /,5X,3I4,2F8.1,/)
        ELSE
C         WRITE (LUNOUT, *) IHKL, (DTREKIN(I), I = 1, 6)
 4         WRITE (LUNOUT, 9135) IHKL, FPLUS, SIGFPLUS, FNEG, SIGFNEG
 9135    FORMAT (/,' First (or second) reflection  of d*TREK:',
     +           /,5X,3I4,4F8.1,/)
        ENDIF
      ENDIF

      SSQ4 = 4.0*LSTLSQ(MTZOUT, IHKL(1), IHKL(2), IHKL(3))
      IF (SSQ4.LT.SMIN .OR. SSQ4.GT.SMAX) GO TO 90
C
C     Get suitable indices - exclude terminators etc
      IF (IABS(IHKL(1)).GE.999 .OR. IABS(IHKL(2)).GE.999 .OR.
     +    IABS(IHKL(3)).GE.999) GO TO 90

C     Rejections from REJECT keyword
      IF (IREJ.GT.0) THEN
        DO I=1,IREJ
          IF (HKLREJ(1,I).EQ.IHKL(1) .AND. 
     +        HKLREJ(2,I).EQ.IHKL(2) .AND. 
     +        HKLREJ(3,I).EQ.IHKL(3)) GOTO 90
        ENDDO
      ENDIF
C
      NREF = NREF + 1
      NCHK = NREF - NMON* (NREF/NMON)
      IF (NCHK.EQ.0) THEN
        IF (KANOM.EQ.0) THEN
C         WRITE (LUNOUT, *) IHKL, (DTREKIN(I), I = 1, 6)
          WRITE (LUNOUT,' (a,I8,/,1X,3i4,2f8.1)')
     +    ' DATA IN (h k l I sigI). NREF = ', NREF,
     +    (IHKL(II), II=1, 3), FPLUS, SIGFPLUS
        ELSE
C         WRITE (LUNOUT, *) IHKL, (DTREKIN(I), I = 1, 6)
          WRITE (LUNOUT,' (a,I8,/,1X,3i4,4f8.1)')
     +    ' DATA IN (h k l I+ sigI+ I- sigI-). NREF = ', NREF,
     +    (IHKL(II), II=1, 3), FPLUS, SIGFPLUS, FNEG, SIGFNEG
        ENDIF
      ENDIF

      IF (FPLUS.GT.FHIGH) THEN
        FHIGH = FPLUS
        IHIGH = IHKL(1)
        KHIGH = IHKL(2)
        LHIGH = IHKL(3)
      END IF
      IF (FNEG.GT.FHIGH) THEN
        FHIGH = FNEG
        IHIGH = IHKL(1)
        KHIGH = IHKL(2)
        LHIGH = IHKL(3)
      END IF

C---- put output data into ADATA

      CALL EQUAL_MAGIC(MTZOUT,ADATA,NLPRGO)

      ADATA(1) = IHKL(1)
      ADATA(2) = IHKL(2)
      ADATA(3) = IHKL(3)
C
C---- No I- reflection, no I+ reflection and both I+ and I-
C     Missing data is indicated by missing columns.
C     I = 0, SIGI = -1 indicates the reflection is overloaded.
C
      IF (KANOM.EQ.0) THEN
        IF (SIGFPLUS .GT. 0.0) THEN
          ADATA(4) = SCFF*FPLUS
          ADATA(5) = SCFF*SIGFPLUS
          IF (NCHK.EQ.0) THEN
            WRITE (LUNOUT,' (a,I8,/,1X,3i4,2F8.1)') 
     +      ' DATA OUT (I SIGI). NREF = ', 
     +      NREF, IHKL, (ADATA(II), II=4, 5)
          ENDIF
        ELSE
          IF (NCHK.EQ.0) THEN
            WRITE (LUNOUT,' (a,I8,/,1X,3i4,2A8)') 
     +      ' DATA OUT (I SIGI). NREF = ', 
     +      NREF, IHKL, 'MNF', 'MNF'
          ENDIF
        ENDIF
      ELSE
        IF (SIGFPLUS.LE.0.0 .AND. SIGFNEG.LE.0.0) THEN
          IF (NCHK.EQ.0) THEN
            WRITE (LUNOUT,' (a,I8,/,1X,3i4,6A8)') 
     +      ' DATA OUT (IMEAN SIGIMEAN etc.). NREF = ', 
     +      NREF, IHKL,'MNF','MNF','MNF','MNF','MNF','MNF'
          ENDIF
        ELSE IF (SIGFNEG .LE. 0.0) THEN
          ADATA(4) = SCFF*FPLUS
          ADATA(5) = SCFF*SIGFPLUS
          ADATA(6) = SCFF*FPLUS
          ADATA(7) = SCFF*SIGFPLUS
          IF (NCHK.EQ.0) THEN
            WRITE (LUNOUT,' (a,I8,/,1X,3i4,4F8.1,2A8)') 
     +      ' DATA OUT (IMEAN SIGIMEAN etc.). NREF = ', 
     +      NREF, IHKL, (ADATA(II), II=4,7),'MNF','MNF'
          ENDIF
        ELSE IF (SIGFPLUS .LE. 0.0) THEN
          ADATA(4) = SCFF*FNEG
          ADATA(5) = SCFF*SIGFNEG
          ADATA(8) = SCFF*FNEG
          ADATA(9) = SCFF*SIGFNEG
          IF (NCHK.EQ.0) THEN
            WRITE (LUNOUT,' (a,I8,/,1X,3i4,2F8.1,2A8,2F8.1)') 
     +      ' DATA OUT (IMEAN SIGIMEAN etc.). NREF = ', 
     +   NREF,IHKL,(ADATA(II),II=4,5),'MNF','MNF',(ADATA(II),II=8,9)
          ENDIF
        ELSE
          SIGIMEAN = SIGFPLUS*SIGFNEG/(SIGFPLUS+SIGFNEG)
          ADATA(4) = SCFF*(FPLUS/SIGFPLUS + FNEG/SIGFNEG)*SIGIMEAN
          ADATA(5) = SCFF*SIGIMEAN
          ADATA(6) = SCFF*FPLUS
          ADATA(7) = SCFF*SIGFPLUS
          ADATA(8) = SCFF*FNEG
          ADATA(9) = SCFF*SIGFNEG
          IF (NCHK.EQ.0) THEN
            WRITE (LUNOUT,' (a,I8,/,1X,3i4,6F8.1)') 
     +      ' DATA OUT (IMEAN SIGIMEAN etc.). NREF = ', 
     +      NREF, IHKL, (ADATA(II), II=4, 9)
          ENDIF
        ENDIF
      ENDIF

      IF (ABS(ADATA(1)).LT.998) CALL LWREFL(MTZOUT,ADATA)
C
      GO TO 90
C
C  ...back for next reflection
C
  120 CONTINUE
C
      WRITE (LUNOUT, 9210) NREAD, NREF
 9210 FORMAT (/,' END OF INPUT FILE ',//,' NUMBER OF REFL. READ : ', I6,
     +       ' -    PROCESSED : ', I6)
      IF (0 .NE. NUMSWP) THEN
         WRITE (LUNOUT, '(A, I10, A)') 
     +                 'WARNING: I+, I- not ordered for ', NUMSWP,
     +                 ' reflections, so they were swapped.'
      ENDIF
      WRITE (LUNOUT, 9220) IHIGH, KHIGH, LHIGH, FHIGH
 9220 FORMAT (/,/,' HIGHEST I or SIGI before scaling: ',3i4,f12.1,/)
      call ccp4h_pre_end()
      call ccp4h_rule() 
      call ccp4h_header('Header for Output MTZ File','out',3)
      call ccp4h_pre_beg() 
C
C---- Close files and finish
C
      CALL LWCLOS(MTZOUT, 1)
      CLOSE(UNIT=21,STATUS='KEEP')
      IF (NREF.LE.0) CALL CCPERR (1, 'No reflexions passed')
      IF (NEGSIG.GT.0) THEN
        WRITE (LUNOUT, '(a,i4,a)') ' ******   YOU HAVE ',
     +       NEGSIG, 'NEGATIVE SIGMAS!!! - Do something! **** '
        CALL CCPERR (1, 'Negative sigmas')
      END IF
      CALL CCPERR(0, 'Normal termination')
C
 140  WRITE (LUNOUT, '('' *** Bad input format on record '',I8)') NREAD
      CLOSE(UNIT=21,STATUS='KEEP')
      CALL LWCLOS(MTZOUT, 1)
      CALL CCPERR(1, 'Check your data!')
      call ccp4h_rule() 
      call ccp4h_pre_end()
      END
