C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
      SUBROUTINE BEMVEC(IHKL,GONANG,UB,E1,E2,E3,S0,PB,SB,S1L)
C     =======================================================
C
C Calculate primary and secondary beam directions for reflection
C in spindle frame.
C The diffractometer equation in x = R U B h
C Then Pb(spindle frame) = R**-1 (-s0)      [s0 antiparallel to beam]
C      scattering vector s = UB h
C      Sb  =  s - Pb
C
C      Scattering vector in laboratory frame  SL  = R U B h
C      Secondary beam vector in laboratory frame  S1L  = SL - S0
C
C
C On entry:
C
C  IHKL(3)      indices
C  GONANG(3)    goniometer angles (degrees)
C  UB(3,3)      orientation matrix for this batch
C  E1,E2,E3     goniostat vectors
C  S0           source vector (anti-parallel to beam)
C
C On exit:
C
C  PB(3)        direction cosines of primary beam in spindle frame
C  SB(3)        direction cosines of diffracted beam in spindle frame
C  S1L(3)       direction cosines of diffracted beam in laboratory frame
C
C
      IMPLICIT NONE
C
      INTEGER IHKL(3)
      REAL GONANG(3),UB(3,3),E1(3),E2(3),E3(3),S0(3),PB(3),SB(3),S1L(3)
C
      INTEGER I
      REAL DTORAD,R(3,3),RM1(3,3),S(3),V(3),SL(3),DET
C
      DTORAD=ATAN(1.)/45.
C
C Goniostat matrix R
      CALL GN3CMP(
     .   DTORAD*GONANG(1),DTORAD*GONANG(2),DTORAD*GONANG(3),
     .   E1,E2,E3,0,0,R)
C Invert to RM1
      CALL MINV3(RM1,R,DET)
C
C Incident beam (remember that s0 is antiparallel to beam)
      DO 1,I=1,3
1     V(I)=-S0(I)
      CALL MATVEC(PB,RM1,V)
C
C Scattering vector S in spindle frame
      DO 2,I=1,3
2     V(I)=IHKL(I)
      CALL MATVEC(S,UB,V)
C
C Scattering vector in laboratory frame
      CALL MATVEC(SL,R,S)
C
C Sb = s + Pb
      CALL VSUM(SB,S,PB)
C
      CALL VDIF(S1L,SL,S0)
      CALL VNORM(S1L,S1L,I)
      IF (I.NE.0) GO TO 999
C
C Normalise both vectors
      CALL VNORM(PB,PB,I)
      IF(I.NE.0) GO TO 999
      CALL VNORM(SB,SB,I)
      IF(I.NE.0) GO TO 999
C
C
      RETURN
C
999   WRITE(6,FMT='(A)') ' BEMVEC PB or SB error '
      CALL CCPERR(1,'Error in BEMVEC')
      END
