C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
      SUBROUTINE BMATRX(B, CELL, R, WAVE)
C     ===========================
C
C        [B] is an orthogonalization matrix to convert indices h,k,l to
C        cartesian crystal axes x,y,z.
C
C        [B] =  ( a*     b* cos gamma*   c* cos beta*            )
C               ( 0      b* sin gamma*  -c* sin beta* cos alpha  )
C               ( 0           0            lambda / c            )
C
C        reciprocal lettice modified by multiplication by lamdba.
C
C
C Build cell orthogonalization matrix B from reciprocal cell dimensions
C R (in rlu), CA = cos alpha
C
C
      IMPLICIT NONE
C
C     .. Scalar Arguments ..
      REAL CA, CBS, SBS, CGS, SGS, WAVE
C     ..
C     .. Array Arguments ..
      REAL B(3, 3), CELL(6), R(6)
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC SQRT
C     ..
C
      CA = COS(CELL(4)*3.141592654/180.0)
      CBS = COS(R(5)*3.141592654/180.0)
      CGS = COS(R(6)*3.141592654/180.0)
      SBS = SQRT(ABS(1.0-CBS*CBS))
      SGS = SQRT(ABS(1.0-CGS*CGS))
      B(1, 1) = WAVE*R(1)
      B(1, 2) = WAVE*R(2)*CGS
      B(1, 3) = WAVE*R(3)*CBS
      B(2, 1) = 0.0
      B(2, 2) = WAVE*SGS*R(2)
      B(2, 3) = -WAVE*SBS*R(3)*CA
      B(3, 1) = 0.0
      B(3, 2) = 0.0
      B(3, 3) = WAVE*1.0/CELL(3)
      RETURN
      END
