C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
      SUBROUTINE DCOSFD(ROT,DIRCOS,KAPPA)
C     ===================================
C******************************************************
C       Given a rotation matrix ROTN, expressing a rotation
C       through an angle KAPPA right-handedly about an axis
C       with direction cosines DIRCOS(I),
C       DCOSFD determines DIRCOS() and KAPPA, given
C       that DIRCOS(1) is defined to be positive (otherwise
C       one would have the other end of the rotation axis).
C       KAPPA is returned in degrees
C
C       Expression for rotation matrix is (see J & J, "Mathl.
C               Phys.", p. 122):-
C
C  cw + n(1)n(1)(1-cw)    n(1)n(2)(1-cw)-n(3)sw  n(1)n(3)(1-cw)+n(3)sw
C  n(1)n(2)(1-cw)+n(3)sw  cw + n(2)n(2)(1-cw)    n(2)n(3)(1-cw)-n(1)sw
C  n(3)n(1)(1-cw)-n(2)sw  n(3)n(2)(1-cw)+n(1)sw  cw + n(3)n(3)(1-cw)
C
C       where cw = cos(KAPPA), sw = sin(KAPPA),
C       & n(1), n(2), n(3) are the direction cosines.
C
C*******************************************************}
C
      IMPLICIT NONE
C
      REAL ROT(3,3),DIRCOS(3),S(3),P(3),D(3)
      REAL KAPPA
      REAL DIFF,PI,TRACE,R2,R3,SINKAP,COSKAP
      INTEGER NS(3),NP(3),NCS(3),ND(3),KNTRL,J,JMX
C
C
      DIFF=1.0E-04
      PI=4.0*ATAN(1.)
      KNTRL=0
C     Trace = 1 + 2.cos w
      TRACE=ROT(1,1)+ROT(2,2)+ROT(3,3)
C     S(1)=2.n(1).sin w
C     S(2)=2.n(2).sin w
C     S(3)=2.n(3).sin w
      S(1)=ROT(3,2)-ROT(2,3)
      S(2)=ROT(1,3)-ROT(3,1)
      S(3)=ROT(2,1)-ROT(1,2)
      CALL ORDR3(KNTRL,S,NS)
C--Is biggest S() zero (i.e. is sin w = 0)?
      IF (ABS(S(NS(1))).GT.DIFF) THEN
        R2=S(NS(2))/S(NS(1))
        R3=S(NS(3))/S(NS(1))
        DIRCOS(NS(1))=1.0/SQRT(1 + R2**2 + R3**2)
        DIRCOS(NS(2))=R2*DIRCOS(NS(1))
        DIRCOS(NS(3))=R3*DIRCOS(NS(1))
        GO TO 123
      END IF
C--Calculation when sin w = 0 (esp. when w = 180').
C     P(1)=n(2).n(3).{0(W=0)or2(w=180)}
C     P(2)=n(3).n(1).{0(W=0)or2(w=180)}
C     P(3)=n(1).n(2).{0(W=0)or2(w=180)}
 30   P(1)=ROT(3,2)
      P(2)=ROT(1,3)
      P(3)=ROT(2,1)
      CALL ORDR3(KNTRL,P,NP)
C--Is biggest P() zero (i.e. are all off-diag. terms zero)?
      IF (ABS(P(NP(1))).LT.DIFF) THEN
        IF (TRACE.GT.0.0) THEN
C--Matrix is unit matrix.
          KAPPA=0.0
          DIRCOS(1)=0.0
          DIRCOS(2)=0.0
          DIRCOS(3)=1.0
          RETURN
        END IF
C--Trace -ve, so dyad about x,y or z.
        KAPPA=PI
        DO 40 J=1,3
 40     D(J)=ROT(J,J)+1.0
        CALL ORDR3(KNTRL,D,ND)
        DIRCOS(ND(1))=1.0
        DIRCOS(ND(2))=0.0
        DIRCOS(ND(3))=0.0
        RETURN
      END IF
      IF (ABS(P(NP(2))).LT.DIFF) THEN
C--One d.c. is zero.
        DIRCOS(NP(1))=0.0
        DIRCOS(NP(2))=SQRT(0.5*(1.0+ROT(NP(2),NP(2))))
        DIRCOS(NP(3))=SQRT(0.5*(1.0+ROT(NP(3),NP(3))))
        IF (P(NP(1)).LT.0.0) DIRCOS(NP(2))=-DIRCOS(NP(2))
        GO TO 123
      END IF
C--All d.c's are nonzero.
C     R2=n(NP(1))/n(NP(2))
C     R3=n(NP(1))/n(NP(3))
      R2=P(NP(2))/P(NP(1))
      R3=P(NP(3))/P(NP(1))
      DIRCOS(NP(2))=1.0/SQRT(1 + R2**2 + (R2/R3)**2)
      DIRCOS(NP(3))=1.0/SQRT(1 + R3**2 + (R3/R2)**2)
      DIRCOS(NP(1))=SQRT(1.0-DIRCOS(NP(2))**2-DIRCOS(NP(3))**2)
C--Adjust signs of DIRCOS().
      JMX=0
      DO 20 J=1,3
        IF (P(J).GT.0.0) THEN
          IF (JMX.GT.0) JMX=-1
          IF (JMX.EQ.0) JMX=J
        END IF
 20   CONTINUE
      IF (JMX.GT.0) DIRCOS(JMX)=-DIRCOS(JMX)
      IF (JMX.EQ.0) WRITE(*,1358)
 1358 FORMAT(' All P()s. are negative or zero')
C--Given d.c's, calculate KAPPA.
 123  KNTRL=1
      CALL ORDR3(KNTRL,DIRCOS,NCS)
C     Find KAPPA.
      SINKAP=ROT(NCS(3),NCS(2))-ROT(NCS(2),NCS(3))
      SINKAP=0.5*SINKAP/DIRCOS(NCS(1))
      COSKAP=0.5*(TRACE-1.0)
      KAPPA=ATAN2(SINKAP,COSKAP)
      KAPPA=KAPPA*180.0/PI
C
      RETURN
      END
