C
C     DTREK2SCALA
C     Copyright (C) 2003 Gwyndaf Evans
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
*************************************************************
C
C
C**************************************************************
C
C                                DTREK2SCALA
C                                ===========
C
C==================================================================
C  Read integrated intensity data from from D*TREK 
C
C  Reject data on error flags
C  Reduce data to asymmetric unit
C  Write MTZ file with orientation information suitable for 
C  input to SCALA
C
C==================================================================
C   Gwyndaf Evans   MRC Laboratory of Molecular Biology,
C                   Hills Road, Cambridge UK.
C                   (1998 -- 2000)
C          and      Global Phasing Ltd.
C                   Sheraton House, Castle Park,
C                   Cambridge CB3 0AX, UK.
C                   (2000 -- 2002)
C==================================================================
C
C ADOPTED FROM THE PROGRAM   ABSURD
C
C  by Phil Evans   MRC Laboratory of Molecular Biology,
C                  Hills Road, Cambridge UK
C                  May 1988   July 1988
C
C +++++++++++++++++++   CHANGES  ++++++++++++++++++++++++++++++++++
C
C
C    08-Aug-2002  Named changed from ABSURD to DTREK2SCALA in
C                 preparation for inclusion into the CCP4 suite 
C                 again. (GE)
C
C    05-Nov-1998  A number subroutines added and
C                 others changed to allow integrated data from Jim 
C                 Pflugrath's D*TREK to be read. The parts of the code
C                 particular to Madnes removed.
C                 Output file is suitable for input into SORTMTZ and
C                 then SCALA. (GE)
C     
C    14-Nov-1994  INTOPT = 0, for antique data (no special input) PRE
C
C     9-Oct-1992  Forbid reindexing from inverting hand  PRE
C
C     9-Jul-1992  Portability mods made at Daresbury.  The VMS
C                 `debugging comments' (`D' in column 1) have been
C                 changed to start with CDEBUG
C    19-Jun-1992  Changed keyword "HEADER" to "BTITLE",
C                 changed Ncryst default
C                 Output file now contains
C                 XDET (=Yms), YDET (=Zms), ROT (=Psi/Phi or whatever)
C                 No goniometer angles in output
C
C    24-Jan-1992  Allowed reindex translations
C
C    10-Jan-1992  added REINDEX option   PRE
C
C    17-Jun-1991  New-style orientation block   PRE
C
C    20-May-1991  Added Ian Tickle's UFILE option
C                 also saving orientation matrix for absorption curve (UBA)
C
C     1-May-1991  MTZ Version   PRE
C
C    18-May-1990  Change to use new CCP4 parser
C
C    28-Nov-1989  Don't let INTOPT be .le. 0 (shouldn't be anyway) PRE
C
C    18-Jul-1989  Stop program when keyword unrecognised or ambiguous
C                 when running batch job.  Introduce calls to CCPERR
C                 when error detected.                         PB.
C
C     4-Jul-1989  TOOFAR option, test for large differences between
C                 calculated and observed postions on detector. P.B.
C
C     1-Jun-1989  Don't initialize PSI0 for each file, work out
C               average absorption correctly, get correct Batch title
C                  PRE
C
C    24-APR-89  Trap zero standard deviations after applying scale factor,
C               reset to 1. Andrew Leslie, LMB
C
C    12-Sep-88  fix to read several input files properly into 1 batch PRE
C
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C   Other bits of code were stolen from various places (MADLCF, ABSCALE etc)
C
C
      PROGRAM DTREK2SCALA
C     ===================
C
C Main program
C
      IMPLICIT NONE
C
C     Hint to linker for block data:
      EXTERNAL ABBLK
C Things for PARSER ----
C
C     Parameter definitions
C
      INTEGER MAXLAB, MAXTOK
      PARAMETER (MAXLAB=80, MAXTOK=100)
      CHARACTER KEY*4,SUBKEY*4,ERRLINE*200,CVALUE(MAXTOK)*4,LINE*200
      INTEGER IBEG(MAXTOK),IEND(MAXTOK),ITYP(MAXTOK),
     .     IDEC(MAXTOK),NTOK
      REAL    FVALUE(MAXTOK)
      LOGICAL LEND,LFORM
C
C Main input keywords
      INTEGER NKEYS
      PARAMETER (NKEYS=26)
      CHARACTER*30 KEYS(NKEYS)
C
C  Commons
      INTEGER NBITS
      PARAMETER (NBITS=16)
C
      include 'flags.fh'
      include 'batch.fh'
      include 'reindx.fh'
c
c     include madsav so I can set a default fvalue for LAXES in the case
c     that d*trek header do not have the permutation matrix.
c
      include 'madsav.fh'
C
      REAL TMPMAT(4,4),DET
C
C Common block used when REMADN and REDTRE were created
C
      include 'subcom.fh'
C
C Functions
C      INTEGER LENSTR
C Local variables
      CHARACTER FNAME*80,CFLAGS(2)*6,BTITLE*80,HNAME*80
      INTEGER   IKEY,I,J,K,JRFLAG(NBITS),IDUM,IERR,IFAIL,
     .          FNAME_BEG,FNAME_END,N
      LOGICAL   LCHGRJ,CCPONL,LHKLTR
C
C  Symmetry
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
      include 'symmet.fh'
      include 'csymme.fh'
      include 'harvest.fh'
C
      DATA CFLAGS/'accept','reject'/
C
      DATA KEYS/
     .       'PNAME'
     .      ,'NAME'
     .      ,'XNAME'
     .      ,'DNAME'
     .      ,'SYMMETRY'
     .      ,'FILE'
     .      ,'UFILE'
     .      ,'SCALE' 
     .      ,'TITLE' 
     .      ,'BATCH' 
     .      ,'ACCEPT' 
     .      ,'REJECT'
     .      ,'SPLIT' 
     .      ,'BTITLE' 
     .      ,'HFILE' 
     .      ,'CRYSTAL' 
     .      ,'LIMITS'
     .      ,'TOOFAR' 
     .      ,'REINDEX' 
     .      ,'CELL' 
     .      ,'PHIBAT' 
     .      ,'READOUT' 
     .      ,'STARTIME' 
     .      ,'REALTIME' 
     .      ,'UGCB' 
     .      ,'PROCESS'/
C
C  JRFLAG(I) is flag to accept (=0) or reject (=1) reflections marked
C    with error bit I
C  Set defaults
C       Bit number  1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16
      DATA JRFLAG/0,0,0,0,1,1,1,1,1, 1, 1, 1, 1, 0, 1, 1/
C
      IDUM = 0
C
      CALL CCPFYP
      CALL MTZINI
C
      IFAIL=0
      CALL CCPDPN(5,'DATA','READONLY','F',0,IFAIL)
      IFAIL=0
      CALL CCPDPN(6,'PRINTER','PRINTER','F',0,IFAIL)
      CALL CCPRCS(6,'DTREK2SCALA','$Date: 2005/09/06 11:29:05 $')
C
C=========================================================
C         SET DEFAULTS
C=========================================================
      include 'defaults.fh'
C Set default PROJECTNAME, CRYSTALNAME and DATASETNAME
      CALL CCPDAT(DATE)
      WRITE(DATE6,'(A2,A2,A2)') DATE(1:2),DATE(4:5),DATE(7:8)
 5    I = INDEX(DATE6,' ')
      IF (I.GT.0) THEN
        WRITE(DATE6(I:I),'(A1)') '0'
        GOTO 5
      ENDIF
      PROJECTNAME   = 'unknown'
      CRYSTALNAME   = 'unknown'
      DATASETNAME   = 'unknown'//DATE6

C  Set input error counter
 2    IERR=0
C
C
C=========================================================
C           LOOP TO READ CONTROL INPUT
C=========================================================
C
 1    LINE=' '
      NTOK=MAXTOK
      CALL PARSER(
     .  KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,.TRUE.)
      IF(LEND) GO TO 1000
      IF(NTOK.EQ.0) GO TO 1
      CALL CCPUPC(KEY)
      CALL CHKKEY(KEY,KEYS,NKEYS,IKEY)
      IF(IKEY.EQ.0) THEN
        WRITE(6,FMT='(A)') ' ** Unrecognized keyword **'
        IF(CCPONL(IDUM)) THEN
          GOTO 1
        ELSE
          IERR=IERR+1
        ENDIF
      ELSE IF(IKEY.LT.0) THEN
        WRITE(6,FMT='(A)') ' ** Ambiguous keyword **'
        IF(CCPONL(IDUM)) THEN
          GOTO 1
        ELSE
          IERR=IERR+1
        ENDIF
      ENDIF
C
      GO TO (3,4,6,7,10,40,45,50,60,70,80,90,100,110,115,120,130,
     .    150,160,180,190,200,210,220,240,
     .    900),IKEY
C
C
C
C     ==========================================
C---- PNAME    >> PROJECTNAME [pname]  <string>
C     ==========================================
C
 3    if (ntok .ge. 2) PROJECTNAME = LINE(ibeg(2):)
      goto 1
C
c     ==========================================
c---- name project <pname> crystal <xname> dataset <dname>
c     ==========================================
c---- assign project/crystal/dataset names for new mtz file

 4    if (ntok.lt.7) then
        call ccperr(2,'NAME keyword line incomplete.')
        goto 1
      endif

      n = 2
 451  if (n .le. ntok) then
        subkey = cvalue(n)(1:4)
        call ccpupc(subkey)
        if (subkey .eq. 'PROJ') then
c  set current pname
          PROJECTNAME = line(ibeg(n+1):iend(n+1))
          n = n+2
        elseif (subkey .eq. 'CRYS') then
c  set current xname
          CRYSTALNAME = line(ibeg(n+1):iend(n+1))
          n = n+2
        elseif (subkey .eq. 'DATA') then
          DATASETNAME = line(ibeg(n+1):iend(n+1))
          n = n+2
        else
          write(errline,'(a,a)') 
     +          'wrong subkeyword after NAME: ',cvalue(n)
            call ccperr(1,errline)
        endif
        go to 451
      endif

      go to 1

C     ==========================================
C---- XNAME    >> CRYSTALNAME [xname]  <string>
C     ==========================================
C
 6    if (ntok .ge. 2) CRYSTALNAME = LINE(ibeg(2):)
        goto 1

C     ===========================================
C---- DNAME    >> DATASETNAME  [dname]  <string>
C     ===========================================
C
 7    if (ntok .ge. 2) DATASETNAME = LINE(ibeg(2):)
         goto 1
C=============================================================================
C SYMMETRY read  real-space symmetry as space-group name, number or operations
C=============================================================================
 10   J = 2
      CALL RDSYMM(J,LINE,IBEG,IEND,ITYP,FVALUE,NTOK,
     .    SPGNAM,NUMSGP,PGNAME,NSYM,NSYMP,RSYM)
      GO TO 1

C================================================
C FILE  filename for reflection input
C================================================
 40   IF (NTOK.EQ.2) THEN
        FNAME=LINE(IBEG(2):IEND(NTOK))
      ELSE IF (NTOK.EQ.4) THEN
        FNAME = LINE(IBEG(2):IEND(2))
        FNAME_BEG = NINT(FVALUE(3))
        FNAME_END = NINT(FVALUE(4))
        HNAME = 'NONE'
      END IF
      LFORM=.TRUE.
      GO TO 1

C================================================
C UFILE  filename for unformatted reflection input
C================================================
 45   FNAME=LINE(IBEG(2):IEND(NTOK))
      LFORM=.FALSE.
      GO TO 1

C================================================
C SCALE scale factor for output
C================================================
50    CALL GTPREA(2,SCALE,NTOK,ITYP,FVALUE)
      GO TO 1

C================================================
C TITLE  for output MTZ file
C================================================
60    TITLE=LINE(IBEG(2):)
      BTITLE=TITLE
      GO TO 1

C================================================
C BATCH  number for MTZ file
C================================================
70    CALL GTPINT(2,IBATCH,NTOK,ITYP,FVALUE)

C================================================
C  MBATCH is optional maximum batch number, default = 0 ie don't use
C================================================
      MBATCH = 0
      CALL GTPINT(3,MBATCH,NTOK,ITYP,FVALUE)
C Change of Batch number, write out batch headers if there are any to do
      IF(LBCHDR) THEN
         WRITE(6,*) 'Writing out batch orientation block'
         CALL OUTHDR
         LBCHDR=.FALSE.
      ENDIF
C  Note that INIHDR clears PSI0 when new batch is specified
      CALL INIHDR(JCRYST)
      NEWBCH=.TRUE.
      GO TO 1

C================================================
C ACCEPT  set accept mask for error flags (cf REJECT)
C================================================
80    K=0
C
81    DO 82,I=2,NTOK
      CALL GTPINT(I,J,NTOK,ITYP,FVALUE)
      IF(J.GT.0.AND.J.LE.NBITS) THEN
C  Set accept or reject flag
            JRFLAG(J)=K
      ELSE
            WRITE(6,1080) J,NBITS
1080      FORMAT(' !!! Error bit number out of range: ',2I5)
            IERR=IERR+1
      ENDIF
82    CONTINUE
      LCHGRJ=.TRUE.
      GO TO 1

C================================================
C REJECT  set reject mask for error flags (cf ACCEPT)
C================================================
90    K=1
      GO TO 81

C================================================
C SPLIT  into psi ranges
C================================================
100   CALL GTPREA(2,DELPSI,NTOK,ITYP,FVALUE)
      IF(DELPSI.LE.0.000001.OR.DELPSI.GT.360.) THEN
            WRITE(6,1100) DELPSI
1100      FORMAT(/' *** Warning! silly value of DELPSI: ',F10.3/)
      ENDIF
      GO TO 1

C================================================
C BTITLE read batch title
C================================================
110   BTITLE=LINE(IBEG(2):)
      GO TO 1

C================================================
C HEADER read name of d*trek header file
C================================================
 115  HNAME=LINE(IBEG(2):)
      GO TO 1

C================================================
C CRYSTAL read crystal number
C================================================
120   CALL GTPINT(2,JCRYST,NTOK,ITYP,FVALUE)
      IF(JCRYST.LT.0) THEN
            WRITE(6,1120)
1120      FORMAT(/
     .  ' ***** !! WARNING: crystal number must be .ge. 0 !! *****'/)
      ENDIF
      GO TO 1

C================================================
C LIMITS read Ymin, Ymax, Zmin, Zmax
C================================================
130   IF(NTOK.LT.5) THEN
            WRITE(6,*)
     .       ' **** You must give Ymin, Ymax, Zmin, Zmax ****'
            GO TO 1
      ENDIF
      K=1
      DO 131,J=1,2
      DO 131,I=1,2
      K=K+1
      CALL GTPINT(K,LIMITS(I,J),NTOK,ITYP,FVALUE)
131   CONTINUE
      GO TO 1


C================================================
C TOOFAR read Yshift, Zshift, Phishift
C================================================
150   IF(NTOK.LT.4) THEN
            WRITE(6,*)
     .       ' **** You must give Yshift, Zshift, Phishift ****'
            GO TO 1
      ENDIF
      CALL GTPREA(2,SHIFT(1),NTOK,ITYP,FVALUE)
      CALL GTPREA(3,SHIFT(2),NTOK,ITYP,FVALUE)
      CALL GTPREA(4,SHIFT(3),NTOK,ITYP,FVALUE)
      GO TO 1

C================================================
C REINDEX read reindexing order
C================================================
 160  LRINDX = .TRUE.
      J = 1
        CALL SYMFR2(LINE,IBEG(2),J,TMPMAT)
      IF (J .NE. 1) THEN
         WRITE (6, '(A)') ' *** Illegal REINDEX command ***'
         IERR = IERR + 1
      ELSE
C copy to 3x3 matrix & translations
         LHKLTR = .FALSE.
         DO 161, J = 1,3
            HKLTRN(J) = TMPMAT(J,4)
            IF (ABS(HKLTRN(J)) .GT. 0.05) LHKLTR = .TRUE.
            DO 162, I = 1,3
               HKLMAT(I,J) = TMPMAT(I,J)
 162        CONTINUE
 161     CONTINUE
C Invert matrix
         CALL MINV3(HKLINV,HKLMAT,DET)
C
         IF (DET .LT. 0.0) THEN
            WRITE (6,'(/A/A)')
     $        '  !!!! WARNING - reindexing matrix INVERTS hand !!!!',
     $        '  !!!!      You are NOT allowed to do this      !!!!'
            CALL CCPERR(1,'*** Illegal reindexing ***')
         ENDIF
C
         WRITE (6,'(/A)')
     $   '  Reflections will be reindexed, and unit cell recalculated'
         CALL WRMATX(HKLMAT,' for reindexing')
         IF (LHKLTR) WRITE (6,'(/A/A/A,3F8.1/)')
     $        ' !!!! WARNING -  you are translating the indices !!!!',
     $        ' !!!! orientation data may be in error           !!!!',
     $        ' hkl translation vector : ' , HKLTRN
      ENDIF
      GO TO 1

C================================================
C     CELL
C     (used for DENZO option)
C================================================
 180  DO I = 1 , 6
         CALL GTPREA(I+1,OCELL(I),NTOK,ITYP,FVALUE)
      END DO
      GO TO 1

C================================================
C     PHIBAT - phi value at start of run where BATCH No = 1 
C     (used for DENZO option)
C================================================
 190  CALL GTPREA(2,PHIBAT,NTOK,ITYP,FVALUE)
      GO TO 1

C================================================
C     READOUT:
C     TIMDED - Detector readout time (total delay between exposures)
C     (used in D*TREK option only)
C================================================
 200  CALL GTPREA(2,TIMDED,NTOK,ITYP,FVALUE)
      GO TO 1

C================================================
C     STARTIME:
C     EXPTIM0 - Time in seconds corresponding to the start of this set
C     of reflections being process. i.e The total time that this crystal
C     has been used to measure data already. This is added to the TIMEs
C     calculated for each reflection based on the image exposure time.
C================================================
 210  CALL GTPREA(2,EXPTIM0,NTOK,ITYP,FVALUE)
      GO TO 1

C================================================
C     REALTIME:
C     EXPRTIM0 - Similar to EXPTIM0 however this value includes the dead
C     /readout times for the detectors and corresponds therefore to time
C     that the crystal has been exposed to radiation. This is added to
C     the RTIME column in the output file.
C================================================
 220  CALL GTPREA(2,EXPRTIM0,NTOK,ITYP,FVALUE)
      GO TO 1

C================================================
C     These are here for future use when we may be able to model the
C     dose and/or time dependent effects of radiation damage
C     (used in D*TREK option only)
C
C
C     UGCB:
C     TIMDED - Detector readout time (total delay between exposures)
C     (used in D*TREK option only)
C================================================
 240  LUGCB = .TRUE.
      GO TO 1

C================================================
C PROCESS  current file
C================================================
C  Check that all necessary data has been given
900   IF (IERR.GT.0) THEN
            WRITE(6,'(A)')
     *          ' **** Some input above was wrong, stop now ****'
            CALL CCPERR(1,
     *          ' **** Some input wrong, stop now ****')
      ENDIF
      IF(NSYM .LE.0) THEN
            WRITE(6,9001)
9001      FORMAT(' !!! No symmetry given !!!')
            CALL CCPERR(1,'No symmetry')
      ENDIF
C
C Set up symmetry
      IF (LFIRST) CALL ASUSET(
     .     SPGNAM,NUMSGP,PGNAME,NSYM,RSYM,NSYMP,NLAUE,.TRUE.)

C Set mask for rejections
      MASK=0
      DO 910,I=1,NBITS
      IF(JRFLAG(I).EQ.1) MASK=MASK+2**(I-1)
910   CONTINUE
      IF(LCHGRJ) WRITE(6,1910) (CFLAGS(JRFLAG(I)+1),I=1,NBITS)
1910  FORMAT(//' Reflection rejection conditions:'//
     .         ' Type              Error',/
     *         ' Number   Flag     Condition',/,
     *  /,     '  1     ',A,'     Not found',
     *  /,     '  2     ',A,'     On YMS edge',
     *  /,     '  3     ',A,'     On ZMS edge',
     *  /,     '  4     ',A,'     On Phi edge',
     *  /,     '  5     ',A,'     Too far from YMS',
     *  /,     '  6     ',A,'     Too far from ZMS',
     *  /,     '  7     ',A,'     Too far from Phi',
     *  /,     '  8     ',A,'     Too big in YMS',
     *  /,     '  9     ',A,'     Too big in ZMS',
     *  /,     ' 10     ',A,'     Too big in Phi',
     *  /,     ' 11     ',A,'     Background bad',
     *  /,     ' 12     ',A,'     Background sd bad',
     *  /,     ' 13     ',A,'     Negative sd',
     *  /,     ' 14     ',A,'     Fobs <= 0.0',
     *  /,     ' 15     ',A,'     Bad pixels',
     *  /,     ' 16     ',A,'     Overflow',/)
      LCHGRJ=.FALSE.
C
      IF(LIMITS(1,1).GE.0) WRITE(6,1131) LIMITS
1131  FORMAT(/' Reflections only accepted if they lie ',
     .   'between detector pixel limits: '/
     .   11X,'   Ymin   Ymax   Zmin   Zmax'/
     .   11X,4I7/)
C
      IF(SHIFT(1).GE.0.0) WRITE(6,1132) SHIFT
1132  FORMAT(/' Reflections only accepted if difference between',
     .   'calculated and observed positions is less than: '/
     .   11X,'   Yshift   Zshift  Phishift'/
     .   11X,2F7.1,F7.2/)
C
C     Process input data from D*TREK only
C
C
C     Reflection and Header files from d*trek
C     FNAME = dtprofit.ref ; HNAME = dtintegrate.head
C
      WRITE(6,*) ' Data processed with D*TREK'
      WRITE(6,'(A,/,A,/,A)') PROJECTNAME, CRYSTALNAME, DATASETNAME
      CALL REDTRE(FNAME, HNAME, BTITLE )
C
        GO TO 2
C
C--------------------------------------------------------------------------
C
C All input finished, close down
C
 1000 IF(FOPEN) THEN
C Write group statistics, calculate average cell (NOT in DENZO case)
         CALL PRTGRP
C   Write batch headers if necessary
         IF(LBCHDR) CALL OUTHDR
C   Close MTZ file
         CALL CLSOPF
      ENDIF
C
      CALL CCPERR(0,'Normal termination')
C
      END
