C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C 26-Nov-1988       J. W. Pflugrath           Cold Spring Harbor Laboratory
C    Edited to conform to Fortran 77.  Renamed from Goniostat_3_compose to
C    GN3CMP
C
C =============================================================================
C                     ! to compose rotation matrix for general 3-circle geometry
      SUBROUTINE GN3CMP
C                     ! inputs: goniostat angles
     1 (ANGLE1, ANGLE2, ANGLE3
C                     ! inputs: goniostat axes at datum position
     2 , AXIS1, AXIS2, AXIS3
C                     ! input: maximum possible number of derivatives
     3 , NDERMX
C                     ! input: required number of derivatives
     4 , NDER
C                     ! outputs: rotation matrix and derivatives
     5 , MATS)
C
CEE Composes the rotation matrix for a 3-circle goniostat and
C   any number of derivatives with respect to the angles of rotation.
C Goniostat_3_compose
Created:      14-NOV-1986   D.J.Thomas, MRC Laboratory of Molecular Biology,
C                                       Hills Road, Cambridge, CB2 2QH, England
C Goniostat_3_compose
C
      IMPLICIT NONE
C
C                     ! 1st goniostat angle (usually omega) / radians
      REAL     ANGLE1
C                     ! 2nd goniostat angle (usually kappa/chi) / radians
      REAL     ANGLE2
C                     ! 3rd goniostat angle (usually phi) / radians
      REAL     ANGLE3
C                     ! dummy variable in statement function
      INTEGER  ORDER
C                     ! ORDER of derivative of product matrix wrt ANGLE1
      INTEGER  ORDER1
C                     ! ORDER of derivative of product matrix wrt ANGLE2
      INTEGER  ORDER2
C                     ! ORDER of derivative of product matrix wrt ANGLE3
      INTEGER  ORDER3
C                     ! number of derivatives required
      INTEGER  NDER
C                     ! maximum possible number of derivatives
      INTEGER  NDERMX
C                     ! statement function
      INTEGER  PRNPAL
C Goniostat_3_compose
C
C                     ! 1st axis (usually omega) (right-handed unit-vector)
      REAL AXIS1 (1:3)
C                     ! 2nd axis (usually kappa/chi) (right-handed unit-vector)
      REAL AXIS2 (1:3)
C                     ! 3rd axis (usually phi) (right-handed unit-vector)
      REAL AXIS3 (1:3)
C                     ! overall rotation matrix and requested derivatives
      REAL MATS (1:3, 1:3, 0:NDERMX, 0:NDERMX, 0:NDERMX)
C                          ! 1st angle rotation matrix and requested derivatives
      REAL MATS1 (1:3,1:3,0:4)
C                          ! 2nd angle rotation matrix and requested derivatives
      REAL MATS2 (1:3,1:3,0:4)
C                          ! 3rd angle rotation matrix and requested derivatives
      REAL MATS3 (1:3,1:3,0:4)
C                      ! statement function to cycle round derivatives correctly
      PRNPAL(ORDER) = 1+MOD(ORDER-1,4)
C
C                   ! make rotation matrix and requested derivatives (not > 4th)
      CALL RTMATS(ANGLE1, AXIS1, MIN(NDER,4), MATS1)
C                   ! make rotation matrix and requested derivatives (not > 4th)
      CALL RTMATS(ANGLE2, AXIS2, MIN(NDER,4), MATS2)
C                   ! make rotation matrix and requested derivatives (not > 4th)
      CALL RTMATS(ANGLE3, AXIS3, MIN(NDER,4), MATS3)
C
      DO 100 ORDER1 = 0, NDER
         DO 100 ORDER2 = 0, NDER-ORDER1
            DO 100 ORDER3 = 0, NDER-ORDER1-ORDER2
C
C! calculate all of the derivatives of the rotation matrix
C!  up to the requested ORDER
C
               CALL ML3MAT(
     2                     3, MATS1(1,1,PRNPAL(ORDER1)),
     3                     3, MATS2(1,1,PRNPAL(ORDER2)),
     4                     3, MATS3(1,1,PRNPAL(ORDER3)),
     5                     3, MATS(1,1,ORDER1,ORDER2,ORDER3))
100         CONTINUE
         CONTINUE
      CONTINUE
C
      RETURN
      END
