C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C     
      SUBROUTINE MKBMAT(CELL, BMAT, LAMDA, LAXV)
C     ==========================================
C     
C     Makes reciprocal cell orthogonalization matrix BMAT from
C   direct cell CELL and wavelength LAMDA, using permutation
C   flags LAXV
C
C Based on subroutines CONDTR & MATFST
C     A. Messerschmidt
C     Max-Planck-Institut fuer Biochemie
C     D8033 Martinsried
C     West Germany
C
C     FORTRAN 77: No exceptions.
C
C  Phil Evans  April 1988
C
      IMPLICIT NONE
C
C  Dummy variables
      REAL CELL(6), BMAT(3,3), LAMDA
      INTEGER LAXV(3)
C
C Local variables
      INTEGER LAXS(3)
C
      REAL           PARM(9), STAR(9), WINKEL(3)
C
      REAL AA,BB,CC,CA,CB,CG,SA,SB,SG
      REAL AS,BS,CS,CAS,CBS,CGS,SAS,SBS,SGS
      EQUIVALENCE    (STAR(1), AS), (STAR(2), BS), (STAR(3), CS),
     *             (STAR(4),CAS), (STAR(5),CBS), (STAR(6),CGS),
     *             (STAR(7),SAS), (STAR(8),SBS), (STAR(9),SGS)
C
      REAL SINE, COSANG, PV, PVSTAR, DG2RD
      INTEGER I, LAX, LUP, LIN
C
C     Multiply by this to convert degrees to radians:
C
      PARAMETER (DG2RD = 3.1415926 / 180.)
C
C     **** Inline function definitions ****
C
      SINE(COSANG) = SQRT( 1. - COSANG * COSANG)
C
      AA = CELL(1)
      BB = CELL(2)
      CC = CELL(3)
      CA = COS ( CELL(4) * DG2RD )
      CB = COS ( CELL(5) * DG2RD )
      CG = COS ( CELL(6) * DG2RD )
      SA = SIN ( CELL(4) * DG2RD )
      SB = SIN ( CELL(5) * DG2RD )
      SG = SIN ( CELL(6) * DG2RD )
C
C     Look at those equivalences above!
C
      PV = SQRT(1. - CA * CA - CB * CB - CG * CG + 2. * CA * CB * CG)
C
C * AS,BS,CS,CAS,CBS,CGS * RECIPROCAL LATTICE (A*,B*,C*,COS(ALPHA*),...)
C
      AS = SA / (AA * PV)
      BS = SB / (BB * PV)
      CS = SG / (CC * PV)
      CAS = (CB * CG - CA) / (SB * SG)
      CBS = (CG * CA - CB) / (SG * SA)
      CGS = (CA * CB - CG) / (SA * SB)
      DO 200 I = 4, 6
C
         STAR(I+3) = SINE(STAR(I))
200   CONTINUE
C
      DO 300 I = 1, 3
         WINKEL(I) = ACOS(STAR(I+3)) / DG2RD
C
C        * Normalize Ewald-sphere, radius = 1
C
         STAR(I) = STAR(I) * LAMDA
         PARM(I+3) = STAR(I)
         PARM(I+6) = WINKEL(I) * DG2RD
C
300   CONTINUE
C
      PVSTAR = SQRT( 1. -CAS*CAS - CBS*CBS - CGS*CGS+ 2. *CAS*CBS*CGS)
C
      LAX = IABS(LAXV(1))
      LUP = IABS(LAXV(2))
      LIN = IABS(LAXV(3))
      DO 305 I = 1, 3
         LAXS(I)=ISIGN(1,LAXV(I))
305   CONTINUE
C
C     Form orientation matrix of reciprocal lattice
C
cv
      write(6,*) 'cv-1',laxv,lax,lup,lin
cv
      BMAT(2, LAX) = 0.
      BMAT(3, LAX) = 0.
      BMAT(3, LUP) = 0.
      BMAT(1, LAX) = STAR(LAX) * LAXS(1)
      BMAT(1, LUP) = STAR(LUP) * STAR(3 + LIN) * LAXS(1)
      BMAT(1, LIN) = STAR(LIN) * STAR(3 + LUP) * LAXS(1)
      BMAT(2, LUP) = STAR(LUP) * STAR(6 + LIN) * LAXS(2)
      BMAT(2, LIN) =-STAR(LIN) * LAXS(2) / STAR(6 + LIN)
     *        * ( STAR(3 + LUP) * STAR(3 + LIN) - STAR(3 + LAX) )
      BMAT(3, LIN) = STAR(LIN) / STAR(6 + LIN) * LAXS(3)
     *        * PVSTAR
      RETURN
C     ======
C
      END
