C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
      SUBROUTINE MKORDTR(RCELL)
C     =======================
C
C Converts d*trek header information to orientation block for MTZ file,
C  & generates matrices etc for this program
C
C Returns RCELL,  cell dimensions
C
      IMPLICIT NONE
C
C Common containing Madnes data
C
      include 'madsav.fh'
      include 'madsvc.fh'
C
      SAVE /MADSVC/ , /MADSAV/
C
C
C  Commons
      INTEGER NBITS
      PARAMETER (NBITS=16)
C
      include 'reindx.fh'
      include 'flags.fh'
C
C
C     Orientation block data
C
C     This contains slots for all information that seems to be essential
C     at present. Each group of parameters is padded at the end for future
C     expansion.
C
C **** CAMBRIDGE CONVENTION **** 
C
C     Data in the orientation block are
C     referred to the "Cambridge" laboratory axis frame: x along the
C     (idealized) X-ray beam, z along usual rotation axis (omega on
C     3-axis system).  Cambridge frame has z along the rotation axis and
C     x in plane containing z and X-ray beam (parallel to beam). y
C     completes a right handed orthonormal basis.
C
C     The general orientation equation is
C
C     x  =   R M U B h
C
C     where x    position in laboratory frame
C     R    goniostat matrix
C     M    missetting angle matrix (if relevant, see MISFLG)
C                 PhiZ PhiY PhiX (PHIXYZ)
C     U    crystal orientation matrix UMAT
C     B    cell orthogonalization matrix, derived from cell dimensions
C     h    reflection indices
C
C **** D*TREK CONVENTION ****
C
C     The d*trek laboratory frame has X along the omega axis (towards
C     base plate), Z antiparallel to the X-ray beam and Y completing a right-handed
C     system. All rotations are right-handed. 
C     In D*TREK the equation and conventions are as follows:
C
C     r =   R G C B h
C
C     where r    position in laboratory frame
C     R    rotation matrix describing crystal poition about the data
C          collection axis
C     G    rotation matrix describing goniometer settings
C          Datum position is defined by the position of these goniometer axes
C          when the crystal rotation axis has a value of 0.
C     C    rotation matrix describing crystal orientation away from the
C          standard setting (a* along X, b* in the XY plane).
C     B    converts a r.l.p described by Miller indices h into a
C          position in the laboratory frame r  r=Bh where
C
C           a*     b* cos(gamma*)    c* cos(beta*)
C     B  =  0      b* sin(gamma*)   -c* sin(beta*) cos(alpha)
C           0      0                 c* sib(beta*) sin(alpha)
C
C          and a,  b,  c,  alpha,  beta,  gamma    cell dimensions
C              a*, b*, c*, alpha*, beta*, gamma*  reciprocal cell dimensions
C
C *************************
C
C     Note that the description below is NOT is the same order as in the
C     common block, in which all the integers come before all the reals
C     (flagged as I or R in the description below)
C
CI    NWORDS       number of words in orientation block
CI    NINTGR       number of integers (first part of block,
C                        includes these counts)
CI    NREALS       number of reals
CI    IORTYP       type of orientation block (for possible future use, now = 0)
CI    INTPAD(9)    padding for future use (integers)
C
C---  Information for this crystal
C
CR    CELL(6)      cell dimensions  (A & degrees)
CI    LBCELL(6)    refinement flags for cell dimensions
CR    UMAT(3,3)    orientation matrix U. If MISFLG .gt. 0, U is the
C                   "standard" setting when PhiXYZ ==0
CI    MISFLG       status of "missetting" angles PHIXYZ
C                   = 0  PHIXYZ not used, all orientation in UMAT
C                   = 1  1 set of missetting angles (PHIXYZ(I,1))
C                   = 2  2 sets PHIXYZ(I,J), J=1,2
CR    PHIXYZ(3,2)  missetting angles at beginning & end of rotation
CI    JUMPAX       reciprocal axis closest to principle goniostat axis E1
C                       (only used for printing)
CI    NCRYST       crystal number: a crystal may contain several batches
CI    LCRFLG       type of crystal mosaicity information
C                  (=0 for isotropic, =1 anisotropic)
C     *** CRYDAT(12) equivalenced to following ***
CR    ETAD         reflection width (full width) (degrees)  (if LCRFLG=0)
C      or
CR    ETADH,ETADV  horizontal & vertical reflection width   (if LCRFLG=1)
CR    rest of CRYDAT: padding for crystal information (eg more complicated
C                           mosaicity model)
C     ***
C
C---  Information for this batch
C
CI    LDTYPE       type of data
C                   = 1    oscillation data   (2D spots)
C                   = 2    area detector data (3D spots)
C                   = 3    Laue data
CR    DATUM(3)     datum values of goniostat axes, from which Phi is measured
C                      (degrees)
CR    PHISTT,PHIEND start & stop values of Phi (degrees) relative to datum
CI    JSCAXS       goniostat scan axis number (=1,2,3, or =0 for
C                        multiple axis scan
CR    SCANAX(3)    rotation axis in laboratory frame (not yet implemented:
C                      only relevant if JSCAXS=0)
CR    TIME1, TIME2 start & stop times in minutes
CI    NBSCAL       number of batch scales & Bfactors plus SD's
C                   (4 at present, BSCALE, BBFAC & sd's)
C                  set = 0 if batch scales unset
CR    BSCALE       batch scale
CR    BBFAC        batch temperature factor
C                   corresponding scale is exp(-2 B (sin theta/lambda)**2)
CR    SDBSCL       sd (Bscale)
CR    SDBFAC       sd (BBfac)
CR    BATPAD(12)   padding for batch information
C
C---  Crystal goniostat information
C
CI    NGONAX       number of goniostat axes (normally 1 or 3)
CI    E1(3),E2(3),E3(3) vectors (in "Cambridge" laboratory frame, see below)
C                            defining the NGONAX goniostat axes
CC    GONLAB(3)  names of the three goniostat axes
CR    GONPAD(12) padding for goniostat information
C
C---  Beam information
C
CR    SOURCE(3)    Idealized (ie excluding tilts) source vector
C                   (antiparallel to beam), in "Cambridge" laboratory frame
CR    S0(3)        Source vector (antiparallel ! to beam), in
C                   "Cambridge" laboratory frame, including tilts
CI    LBMFLG       flag for type of beam information following
C                   = 0 for ALAMBD, DELAMB only (laboratory source)
C                   = 1     ALAMBD,DELAMB,DELCOR,DIVHD,DIVVD (synchrotron)
C                           (other options could include white beam)
C     *** BEMDAT(25) equivalenced to following ***
CR    ALAMBD       Wavelength in Angstroms
CR    DELAMB       dispersion Deltalambda / lambda.
CR    DELCOR       Correlated component of wavelength dispersion.
CR    DIVHD        Horizontal beam divergence in degrees.
CR    DIVVD        Vertical beam divergence (may be 0.0 for isotropic beam
C                        divergence.
CR    rest of BEMDAT: padding for beam information
C                      (*** How much here for Laue? ***)
C     ***
C
C---  Detector information
C
CI    NDET         number of detectors (current maximum 2)
C     -- for each detector
CR    DXn          crystal to detector distance (mm)
CR    THETAn       detector tilt angle (=Madnes:tau2) (degrees)
CR    DETLMn(2,2)  minimum & maximum values of detector coordinates (pixels)
C                     (i,j): i = 1 minimum, = 2 maximum
C                            j = 1 Xdet,    = 2 Ydet
C                  The exact detector frame is not important, but Ydet
C                  should be the axis ~ parallel to the pricipal
C                  rotation axis
CR    DETPAD(34)     padding for detector information
C
C
C Lengths: MBLENG is total length of block, MBLINT, MBLREA are numbers
C          of integers & reals
      INTEGER MBLENG,MBLINT,MBLREA
      PARAMETER (MBLENG=185,MBLINT=29,MBLREA=156)
C     ..
C     .. Common blocks ..
      include 'orient.fh'
      include 'corient.fh'
C
C     *** Note well the following equivalences: these are to allow for
C     alternative definitions of crystal mosaicity, beam parameters, etc
      EQUIVALENCE (ETAD,CRYDAT(1))
      EQUIVALENCE (ETADH,CRYDAT(1)),(ETADV,CRYDAT(2))
      EQUIVALENCE (BEMDAT(1),ALAMBD)
      EQUIVALENCE (BEMDAT(2),DELAMB)
      EQUIVALENCE (BEMDAT(3),DELCOR)
      EQUIVALENCE (BEMDAT(4),DIVHD)
      EQUIVALENCE (BEMDAT(5),DIVVD)
C ***
C This equivalence is strictly for convenience in printing
      REAL E123(3,3)
      EQUIVALENCE (E123(1,1),E1(1))
C     ..
C
C Save it
      SAVE /CORIEN/,/ORIENT/,/REINDX/,/MATRCS_R/, /MATRCS_I/,/MORIEN/,
     .  /GRPDAT/
C
C
C  Data for rest of this program
C
      include 'matrcs.fh'
      include 'morien.fh'
      include 'batch.fh'
C
C Storage for recording change of orientation etc during data collection
C  These variables apply to data between 2 successive Save sets in Madnes
C  output.
      INTEGER MAXGRP
      PARAMETER (MAXGRP=200)
      include 'grpdat.fh'
C
C Local things for group calculation
      REAL UB0(3,3),RMAT(3,3),UBI(3,3),V(3)
      INTEGER MGROUP
      SAVE UB0, MGROUP
C
C Local variables
      REAL V1(3),D(3,3),
     .  BM1(3,3),T1(3,3),T2(3,3),DTORAD,TEMP,DET
      REAL QINV(3,3)
      REAL RCELL(6)
      REAL C(3,3), G(3,3), UDTR(3,3), UBDTR(3,3), S(3,3)
C
C
      EXTERNAL MATMUL
C
      LOGICAL DEBUG
      INTEGER I , J , IAXES(3) , JAXES(3)
      DATA IAXES/1,2,3/
      DATA JAXES/1,2,3/
      DATA MGROUP/0/
C
      DTORAD=ATAN(1.)/45.
C
C     Debug = .TRUE. prints out all the matrices during the
C     conversion from d*trek frame to Cambridge frame.
C
C      DEBUG = .TRUE.
      DEBUG = .FALSE.
C     
C Some defaults
      LCRFLG=0
      LBMFLG=0
C
C     First we work in D*TREK laboratory frame using values directly
C     from the d*trek header file.
C
C     Make d*trek C matrix: CRYSTAL_ORIENT_******
C     RPHI(3)     == CRYSTAL_ORIENT_VALUES
C     CRORVC(3,3) == CRYSTAL_ORIENT_VECTORS
C     CRORVC(I,J),I=1,3 applies to the J'th axis
C
      CALL GN3CMP(
     .  RPHI(3)*DTORAD,RPHI(2)*DTORAD,RPHI(1)*DTORAD,
     .  CRORVC(1,3),CRORVC(1,2),CRORVC(1,1),0,0,C)
      IF(DEBUG)CALL WRMATX(C,'Crystal rotation [C]')
C
C     Make d*trek [G] matrix: goniostat rotation matrix at Datum position
C     DATUMM(3)   == CRYSTAL_GONIO_VALUES
C     GONVEC(3,3) == CRYSTAL_GONIO_VECTORS
C
      CALL GN3CMP(
     .  DATUMM(3)*DTORAD,DATUMM(2)*DTORAD,DATUMM(1)*DTORAD,
     .  GONVEC(1,3),GONVEC(1,2),GONVEC(1,1),0,0,G)
      IF(DEBUG)CALL WRMATX(G,'Crystal goniostat rotation [G]')
C
C     Now create d*trek rotation matrix including all except the R
C     matrix. This should be equivalent to the orientation matrix which
C     is to be written to the mtz file but in the incorrect frame of
C     reference.
C
C     D*trek orientation matrix
C                                             
C
C     If LUGCB = .FALSE. then UDTR = [C]     (default)
C     If LUGCB = .TRUE.  then UDTR = [G][C]
C                                             
      IF (LUGCB) THEN
         CALL MATMUL(UDTR,G,C)
         CALL WRMATX(UDTR,
     .    'D*TREK orientation matrix [G][C] (including goniostat)')
C     Must also set Datum values to zero now for writing to MTZ file
C     batch header
         DO I = 1 , 3
            DATUMM(I) = 0.0
         ENDDO
      ELSE
         CALL MOVEIT(C,UDTR,9)
         CALL WRMATX(UDTR,
     .    'D*TREK orientation matrix [C] (crystal orientation only)')
      ENDIF
C
C
C     Make B matrix: cell orthoganalization matrix
C
      CALL MKBMAT(DCELL,B,LAMBDA,IAXES)
      IF(DEBUG)CALL WRMATX(B,'Cell orthogonalization B (d*trek frame)')
      CALL MATMUL(UBDTR,UDTR,B)
      IF(DEBUG)CALL WRMATX(UBDTR,
     . 'UB = ([G])[C][B] in D*TREK frame  ([G] optionally included)')
C
C=============================================================================
C                *** Explanation of the transformation from ***
C                ***     D*trek to Cambridge lab frames     ***
C
C     Transformation matrix Q converts from d*trek laboratory frame to
C     Cambridge frame
C
C     Old basis P  = (x ,y ,z )              - D*trek
C     New basis P' = (x',y',z')              - Cambridge
C     
C     where x,y,z and x',y',z' are basis vectors describing the basis P
C     and P' respectively
C     
C     Transformation matrix from P ---> P'   is Q = [Qij]
C
C     where [Qij] = ith component in P' of the jth basis vector in P
C
C     N.B. This is the classic transformation matrix defined in
C     "Physical properties of Crystals" by J. F. Nye, p9.  and "Vectors
C     and tensors in Crystallography", by Donald E. Sands, sec3 .3, p50.
C
C     That is the 1st column (j=1, i=1,2,3) are the components in B'(new) of
C     the 1st basis vector in B (old).
C
C     Alternatively we can construct the matrix [Q] by putting the ith
c     row equal to the components in P of the ith basis vector in P'.
C
C     Here are the definitions of the basis vectors in d*trek lab-frame
C     and the Cambridge lab-frame. In both systems hardware rotations
C     are all right-handed. That is they are clockwise when viewed along
C     a vector away from the origin. Thus we may transform all rotation 
C     matrices R in d*trek frame to R' in Cambridge frame by
C
C            [R'] = [Q] [R] [Q-1]
C     
C                     
C  ------>   <--------o
C  x-ray        z     |\
C                     | \
C                     |  \         D*TREK
C                     |   \ y
C                   x |    
C                     V
C              Goniometer Baseplate
C
C
C  ------>            o--------->
C  x-ray              |\       x'
C                     | \
C                     |  \         CAMBRIDGE
C                     |   \ y'
C                   z'|    
C                     V
C                  rot axis (SCAN_ROTATION_VECTOR or SCROVC)
C
C Note that MTZ files in principal allow for the definition of a general
C rotation axis vector using JSCAXS=0 and a vector definition in SCANAX(3).
CI    JSCAXS       goniostat scan axis number (=1,2,3, or =0 for
C                  multiple axis scan
CR    SCANAX(3)    rotation axis in laboratory frame (not yet
C                  implemented:
C     
C     However this appears not to be implemented so we will continue
C     using the definition of the lab-frame using SCROVC(3).
C
C=============================================================================
C     Now constuct the transformation matrix Q.
C
      CALL MKTRDTR ( SOURCM(1,1) , SCROVC , Q )
C
C=============================================================================
C     Now calculate UMAT (Cambridge frame orientation matrix)
C
C     Now if Q is the transition matrix
C
C     Cambridge frame      x'  = [R'] [M'] [U'] [B']  h
C                          -                          -
C     here we assume that M' = [1] (identity) since we will have
C     all the misset angles included in the U matrix. (MISFLG=0)
C
C
C     D*TREK    frame      x   = [R]  [G] [C]   [B]   h
C                          -                          -
C     Now x' = Q x
C
C     Substituting for x' gives
C     
C     x' = [Q] [R] [G] [C] [B] h = [R'] [U'] [B'] h
C     -                        -                  -
C
C     Equating both sides gives 
C
C          [Q] [R] [G] [C] [B] h = [R'] [U'] [B'] h
C                              -                  -
C
C     or inserting  [Q] [Q-1] (= [1])
C
C     [Q] [R] [Q-1] [Q] [G] [C] [B] h = [R'] [U'] [B'] h
C                                   -                  -
C
C     and therefore because R' = Q R Q-1
C
C     [U'] = [Q] [G] [C] [B] [B'-1]
C
C     i.e. [U'] = [UMAT] = [Q] [G] [C] [B] [B'-1] 
C     
C     [UB] = [UMAT][B'] = [U'] [B']
C
C          [UMAT] = [Q] [UDTR] [B] [B'-1]
C     
C     [UMAT] = [Q] [UBDTR] [B'-1]
C
C     We use variables
C
C     UMAT  = [U']
C     UB    = [U'] [B']
C     UBDTR = [G] [C] [B]
C     UDTR  = [G] [C]
C     B     = [B]
C     BC    = [B']
C     BM1   = [B'-1]
C
C     Now do calculation
C
C     Get inverse of Q matrix
C
      CALL MINV3(QINV,Q,DET)
      IF(DEBUG)CALL WRMATX(QINV,'Inverse Q')
C
C     Create [B']. i.e. BMAT in Cambridge frame.
C
      CALL MKBMAT(DCELL,BC,LAMBDA,JAXES)
      IF(DEBUG)CALL WRMATX
     .      (BC,"Cell orthogonalization B' (cambridge frame)")
C     and the inverse
      CALL MINV3(BM1,BC,DET)
      IF(DEBUG)CALL WRMATX(BM1,"Inverse of B'")
C     Orientation matrix in Cambridge frame  [U'] = [Q] [UDTR] [B] [B'-1]
      CALL MATMUL(T1,UBDTR,BM1)
      CALL MATMUL(UMAT,Q,T1)
      CALL WRMATX(UMAT,' Cambridge UMAT')
      CALL MINV3(T1,UMAT,DET)
C Check if determinant of UMAT is 1.0 (or at least very close to it)
      WRITE(6,*) 'Determinant of UMAT = ' , DET
      IF (ABS(1.0 - DET) .GT. 0.001) THEN
         WRITE(6,'(A,A)')
     .         'Error: U matrix in Cambridge frame has determinant',
     .         ' of ' , DET,'. It should be +1.0.'
         STOP
      END IF
C Matrix UB
      CALL MATMUL(UB,UMAT,BC)
      IF(DEBUG)CALL WRMATX(UB,'UB in Cambridge frame')
C
C=============================================================================
C Now get all the orientation block data
C
      NWORDS=MBLENG
      NINTGR=MBLINT
      NREALS=MBLREA
C
C--- Crystal data -----------------------------------------
C Cell
      CALL MOVEIT(DCELL,CELL,6)
C Cell refinement flags LBCELL , not properly set
      CALL ZEROIA(LBCELL,6)
C
C----  If reindexing, transform UB matrix, & redetermine new Cell
      IF (LRINDX) THEN
C New indices h' = H h  (where H = HKLMAT)
C  we want (UB)' h' = (UB) h
C  so (UB)' = (UB) H**-1  (H**-1 = HKLINV)
C         CALL MATMUL(T1, UB, HKLINV)
        CALL MOVEIT(T1, UB, 9)
C Decompose new UB matrix into U & B to get new cell
        CALL TRANSP(T1, UB)
C Reciprocal metric tensor G**-1 in T2 = (UB)T . (UB)
        CALL MATMUL(T2, T1, UB)
C Invert to get real metric tensor in T1
        CALL MINV3(T1,T2,DET)
C Get cell dimensions from real metric tensor
C     (CLCALC returns a,b,c/lambda, cos alpha, beta, gamma)
        CALL CLCALC(CELL,T1)
C Convert cell
        DO  I = 1 , 3
           CELL(I) = CELL(I) * LAMBDA
           CELL(I+3) = ACOS(CELL(I+3))/DTORAD
        END DO
C     New B matrix
        CALL MKBMAT(CELL,BC,LAMBDA,IAXES)
C     BM1 = B**-1
        CALL MINV3(BM1,BC,DET)
C     U = (UB) . B**-1
        CALL MATMUL(UMAT, UB, BM1)
C
        WRITE (6,'(A,6F8.2)')
     +    '  New unit cell determined from reindexing: ',CELL
      ENDIF
C
C Missetting angles are dummy so UMAT contains crystal orientation
C and goiniostat setting angles
      MISFLG=0
      CALL ZERORA(PHIXYZ,6)
C
C      NCRYST=LCRYST
      NCRYST=1
C Mosaicity (isotropic)
      LCRFLG=0
      ETAD=EPS
C
C--- Batch data  -----------------------------------------
C Type of data: area detector (is actually LABTYP(J) J=2 in orientation block)
      LDTYPE=2
      CALL MOVEIT(DATUMM,DATUM,3)
      PHISTT=PSIRNG(1)
      PHIEND=PSIRNG(2)
C Scan axis, general setting SCANAX not currently implemented
      JSCAXS=LSCAXS
      WRITE(6,*) ' **** Scan axis number JSCAXS = ',JSCAXS
      CALL ZERORA(SCANAX,3)
C Time
      TIME1 = EXPTIM0
      TIME2 = EXPTIM0 + 
     .      ( 
     .      ( (PSIRNG(2) - PSIRNG(1) ) / WIDINC ) * ( TIMIMG )
     .      )
C Clear batch scales
      NBSCAL=0
      BSCALE=0.0
      BBFAC=0.0
      SDBSCL=0.0
      SDBFAC=0.0
C
C--- Crystal goniostat data -----------------------------------------
      NGONAX=MGONAX
C Convert goniostat vectors to Cambridge frame
      CALL MATVEC(E1,Q,GONVEC(1,1))
      CALL MATVEC(E2,Q,GONVEC(1,2))
      CALL MATVEC(E3,Q,GONVEC(1,3))
C Copy goniostat axis labels
      DO I = 1 , NGONAX
         GONLAB(I)=GONAX(I)
      END DO
C
C--- Beam data ----------------------------------------- 
C
C     Get S0 & Source in Cambridge frame Idealised source in Cambridge
c     frame is always 1 0 0 in this case because the source is used in
c     the definition of both D*trek and Cambridge lab-frames.
C
      CALL GN3CMP(
     .  0.0,MU(1)*DTORAD,MU(2)*DTORAD,
     .  SOURCM(1,1),SOURCM(1,2),SOURCM(1,3),0,0,S)
      IF(DEBUG)CALL WRMATX(S,'Source rotation [S]')
      CALL MATVEC(S0M,S,SOURCM(1,1))
      IF(DEBUG)WRITE(6,*) ' S0M in d*trek:',S0M
      CALL MATVEC(S0,Q,S0M)
      IF(DEBUG)WRITE(6,*) ' SO in Cambri:',S0
      CALL MATVEC(SOURCE,Q,SOURCM(1,1))
C Beam flag for following data
      LBMFLG=0
      ALAMBD=LAMBDA
      DELAMB=DISP
      DELCOR=SYN(3)
      DIVHD=SYN(1)
      DIVVD=SYN(2)
C
C--- Detector data --------------------------------------
C
      NDET=1
      DX1=DXMM
      THETA1=DTAU(2)
      CALL MOVEIT(DTAU,TAU,3)
C Store data for rest of program
      DX=DX1
      THETA=THETA1
C
C Convert detector vectors to Cambridge frame
      CALL MATVEC(D1,Q,DETVEC(1,1))
      CALL MATVEC(D2,Q,DETVEC(1,2))
      CALL MATVEC(D3,Q,DETVEC(1,3))
      CALL MATVEC(DT(1,1),Q,DETTRN(1,1))
      CALL MATVEC(DT(1,2),Q,DETTRN(1,2))
C
      DO I=1,2
        DO J=1,2
C LIMDET(,1) = Yms == Xdet = DETLM1(,2)
C LIMDET(,2) = Zms == Ydet = DETLM1(,1)
          DETLM1(I,3-J)=LIMDET(I,J)
        END DO
      END DO
C
C --- This bit belongs with the crystal data above, but depends on the
C setting of E1,E2,E3 & datum
C JUMPAX is nearest reciprocal axis to principle goniostat axis e1
C V1 = (D U B)**-1 e1, ie e1 vector in reciprocal space
C Datum matrix D from datum angles and goniostat vectors (New frame)
      CALL GN3CMP(
     .  DATUM(1)*DTORAD,DATUM(2)*DTORAD,DATUM(3)*DTORAD,
     .  E1,E2,E3,0,0,D)
      CALL MATMUL(T1,D,UB)
C  T1 is (DUB)**-1
      CALL MINV3(T2,T1,DET)
C  V1 = (DUB)**-1 e1
      CALL MATVEC(V1,T2,E1)
      TEMP=0.0
      DO I = 1 , 3
         IF(ABS(V1(I)).GT.TEMP) THEN
            TEMP=ABS(V1(I))
            JUMPAX=I
         ENDIF
      END DO
      WRITE(6,'(A,I5)') ' ****JUMPAX = ',JUMPAX
C     
C*******************************************************************
C=============================================================================
C That's all the orientation block stuff, are other things needed?
C
      CALL MOVEIT(CELL,RCELL,6)
      IPFILE=INTOPT
C
C --- Group analysis stuff -------------------------
C     Set group number
C
      IF (MGROUP .EQ. 0) THEN
        NGROUP = 1
        MGROUP = 1
        NBTGRP(NGROUP) = 1
      ELSE
        NGROUP = NGROUP+1
C
C Increment NBTGRP if new batch series
C
        IF (PSI0 .LT. -999999.) THEN
          NBTGRP(NGROUP) = NBTGRP(NGROUP-1) +1
        ELSE
          NBTGRP(NGROUP) = NBTGRP(NGROUP-1)
        ENDIF
      ENDIF
      IF (NGROUP .GT. MAXGRP) THEN
        WRITE (6,'(A)') ' *** Too many Save Groups to store ***'
        GO TO 9100
      ENDIF
C
C
C     Store things for this save group
C
      CALL MOVEIT(CELL,CELLGR(1,NGROUP),6)
      CALL MOVEIT(RPHI,RPHGRP(1,NGROUP),3)
      EPSGRP(NGROUP)=EPS
C
C     Psi range: clear minimum & maximum values
C
      PSIGRP(1,NGROUP) =  100000.
      PSIGRP(2,NGROUP) = -100000.
C
C Detector offsets
C
      CALL MOVEIT(DETOFF,DTOGRP(1,NGROUP),2)
C     RMS refine errors
      RMSGRP(1,NGROUP) = RMSCFD
      RMSGRP(2,NGROUP) = RMSDEG
C     Orientation shift
C     If first file of set of batches, record initial orientation
      IF (PSI0 .LT. -999999.) THEN
        CALL MOVEIT(UB,UB0,9)
      ENDIF
C     get rotation from beginning (UB0)
C     rotation = UB0 . UB**-1
C     rotation angle RANGLE in degrees
      CALL MINV3(UBI,UB,DET)
      CALL MATMUL(RMAT,UB0,UBI)
      CALL DCOSFD(RMAT,V,RANGLE(NGROUP))
C Distance
      DXMGRP(NGROUP) = DXMM
      CALL MOVEIT(DTAU,TAUGRP(1,NGROUP),3)
C
C
 9100 RETURN
C
      END

