C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
      SUBROUTINE MKTRDTR
     +      (
C     On entry:
C    1   SOURCE_VECTOR        - Source vector in d*trek lab frame
     1      SRCVEC , 
C
C    2   SCAN_ROTATION_VECTOR - Vector describing rotation axis
C    2                          in d*trek lab frame
     2      ROTVEC , 
C     On exit:
C    3   Transformation matrix for d*trek --> Cambridge frame
     3      Q 
     +      )
C
C     Declarations
C
      REAL Q(3,3)
      REAL SRCVEC(3) , ROTVEC(3)
      REAL XQ(3) , YQ(3) , ZQ(3)
      INTEGER IERR
      LOGICAL DEBUG
C
C     SRCVEC is the source vector i.e. vector pointing towards the
C     source (towrads centre of Ewald sphere from origin).  
C
C     ROTVEC is vector describing the data collection rotation axis
C     (right handed rotations)
C
C      DEBUG = .TRUE.
      DEBUG = .FALSE.
C
C     1st row of Q are components of x' in old basis.
C
      DO I = 1 , 3
         XQ(I) = -SRCVEC(I)
      ENDDO
      CALL VNORM(XQ,XQ,IERR)
      IF(DEBUG)WRITE(6,'(a, i6/)')' Vector nomalisation error flag:'
     .      ,IERR
      IF(IERR.NE.0) GO TO 999
      IF(DEBUG)WRITE(6,'(a,3f8.4)')'   XQ ( after normalisation):',XQ
C
C     3rd row of Q are components of z' in old basis. i.e. the
C     SCAN_ROTATION_VECTOR as defined in the d*trek header files.
C
      DO I = 1 , 3
         ZQ(I) = ROTVEC(I)
      ENDDO
      CALL VNORM(ZQ,ZQ,IERR)
      IF(DEBUG)WRITE(6,'(a, i6/)')' Vector nomalisation error flag:'
     .      ,IERR
      IF(IERR.NE.0) GO TO 999
      IF(DEBUG)WRITE(6,'(a,3f8.4)')'   ZQ ( after normalisation):',ZQ
C
C     2nd row completes the right handed system YQ = ZQ x XQ
C
      CALL CROSS(YQ,ZQ,XQ)
      CALL VNORM(YQ,YQ,I)
      IF(DEBUG)WRITE(6,'(a, i6)')' Vector nomalisation error flag:',IERR
      IF(IERR.NE.0) GO TO 999
      IF(DEBUG)WRITE(6,'(a,3f8.4)')'   YQ ( after normalization):',YQ
C
C     Now X is defined as perpendicular to Z and lying in the plane
C     containing Z and the X-ray beam (parallel to beam). So we must
C     recalculate XQ to get it orthogonal to YQ and ZQ.
C
      IF(DEBUG)WRITE(6,'(a, i6)')'Recalculate X basis vector:'
      CALL CROSS(XQ,YQ,ZQ)
      CALL VNORM(XQ,XQ,I)
      IF(DEBUG)WRITE(6,'(a, i6)')' Vector nomalisation error flag:',IERR
      IF(IERR.NE.0) GO TO 999
      IF(DEBUG)WRITE(6,'(a,3f8.4)')'   XQ ( after normalization):',XQ
C
C     Construct Transformation matrix Q
C
      DO I = 1 , 3
         Q(1,I)=XQ(I)
         Q(2,I)=YQ(I)
         Q(3,I)=ZQ(I)
      END DO
      CALL WRMATX(Q,'Transformation matrix Q')
C
C     Now in general the only vector that both frames have in common is
C     the vector along the rotation axis, i.e. ROTVEC in d*trek and the
C     Z' axis in Cambridge.
C
      RETURN
C
C Error
 999  WRITE(6,FMT='(A)') 'MKORN: normalization error'
      CALL CCPERR(1,'Normalization error in MKORN')
      END

