C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
      SUBROUTINE PLABSN(NTHETA,NPHI,TITLE)
C     =====================================
C
C  Plot observed (T1) and calculated (T2) transmission values
C  Omitted observations are dashed, calculated values dotted
C  TITLE  title for plot
C
      IMPLICIT NONE
C
      INTEGER NTHETA,NPHI
      CHARACTER*(*) TITLE
C
C/ABSORPTION/
C/ABSORPTION/
      INTEGER MOMEGA,MTHETA,MPHI,MTHPHI,MAXORD,LORD,MAXCLM
      PARAMETER (MOMEGA=21,
     *           MPHI=50,
     *           MTHETA=MOMEGA,
     *           MTHPHI=MTHETA*MPHI,
     *           MAXORD=10,
     *           LORD=2,
     *           MAXCLM=(MAXORD+1)*(MAXORD+LORD)/LORD )
C
      include 'tabl1.fh'
C
      SAVE /TABL1/
C/ABSORPTION/
C/ABSORPTION/
C
      REAL TMAX,THETA,VWPORT(2,2),XLOW,XHIGH,YLOW,YHIGH,THMIN,THMAX,
     .   REP1,DASH1,REP2,DASH2,X(50),Y(50),PHIMIN,PHIMAX,DLTHET,RTOD
      INTEGER I,J,IT,IP,NXTICK,NXSTCK,NYTICK,NYSTCK,IOBS
      LOGICAL LDASH,LOMIT(50)
C
C  VWPORT defines the area of the graph (whole page = 0 to 1 )
      DATA VWPORT/0.2,0.2,0.9,0.9/
C Range on Phi to plot
      DATA PHIMIN,PHIMAX/-180.,+180./
C  REP1, DASH1 are repeat & dash parameters (fractions of 1 as whole page)
C              for omitted observations
C  REP2, DASH2  for calculated values
      DATA REP1,DASH1/0.02,0.008/
      DATA REP2,DASH2/0.006,0.002/
C
C
      RTOD=45./ATAN(1.)
C
C  Get ranges for scaling
      TMAX=-1000000.
      THMIN=1000000.
      THMAX=-1000000.
      DO 1,IT=1,NTHETA
        THMIN=MIN(THMIN,THET1(IT))
        THMAX=MAX(THMAX,THET1(IT))
        DO 1,IP=1,NPHI
          TMAX=MAX(TMAX,T1(IT,IP),T1A(IT,IP))
1     CONTINUE
      TMAX=TMAX*1.1
C
C
C Start plot
      CALL PLTINI
C
C Draw axes.  Assume equal ranges for theta.  Each theta step will be divided
C  into a range of TMAX intensity
      XLOW=PHIMIN
      XHIGH=PHIMAX
      NXTICK=-5
      NXSTCK=2
      DLTHET=ABS(THET1(2)-THET1(1))*RTOD
      YLOW=THMIN*RTOD
      YHIGH=THMAX*RTOD+DLTHET
      NYTICK=-(NTHETA+1)
      NYSTCK=0
      CALL PLTAXS(VWPORT,XLOW,XHIGH,NXTICK,NXSTCK,+1,'Phi (polar)',
     .                   YLOW,YHIGH,NYTICK,NYSTCK,-1,'Theta (polar)')
C
      CALL PLTCTX('Transmission curves',0.55,0.98,1.3,2,0.0)
      CALL PLTCTX(TITLE,0.55,0.94,1.3,2,0.0)
C
C  Loop graphs for each theta value
      IOBS=0
      DO 10,IT=1,NTHETA
        THETA=THET1(IT)*RTOD
C
C Draw horizontal separating line except for bottom
        IF(THETA.NE.THMIN) THEN
          CALL PLTMVU(PHIMIN,THETA)
          CALL PLTDWU(PHIMAX,THETA)
        ENDIF
C
C Store values to plot
        DO 20,IP=1,NPHI
          X(IP)=PHI1(IP)*RTOD
          Y(IP)=T1(IT,IP)*DLTHET/TMAX+THETA
C Flag as .false. if omitted
          IOBS=IOBS+1
          IF(T2(IOBS,4).LT.0.00001) THEN
            LOMIT(IP)=.FALSE.
          ELSE
            LOMIT(IP)=.TRUE.
          ENDIF
 20     CONTINUE
C
C Duplicate 1st point as last
        IP=NPHI
        IF(X(IP).NE.X(1)) THEN
          IP=NPHI+1
          X(IP)=X(1)+SIGN(360.,X(NPHI))
          Y(IP)=Y(1)
          LOMIT(IP)=LOMIT(1)
        ENDIF
C
C Plot, assuming Phi varies monotonically
C
        J=1
        LDASH=LOMIT(1)
        DO 25,I=2,IP
          IF(LOMIT(I)) THEN
C new point is solid
            IF(LDASH) THEN
C so is old point, so OK
              CONTINUE
            ELSE
C was dashed, so plot dashed to this point
              CALL PLTDSH(1,REP1,DASH1,0.)
              CALL PLTLIN(X(J),Y(J),I-J+1)
              J=I
            ENDIF
            LDASH=.TRUE.
          ELSE
C new point is dashed
            IF(LDASH) THEN
C   was solid, so plot solid to previous point
              IF(I.GT.2) THEN
                CALL PLTDSH(0,0.,0.,0.)
                CALL PLTLIN(X(J),Y(J),I-J)
                J=I-1
              ENDIF
            ELSE
C   was dashed so OK
              CONTINUE
            ENDIF
            LDASH=.FALSE.
          ENDIF
 25     CONTINUE
C
C plot to last point
        IF(LDASH) THEN
          CALL PLTDSH(0,0.,0.,0.)
        ELSE
          CALL PLTDSH(1,REP1,DASH1,0.)
        ENDIF
        I=IP
        CALL PLTLIN(X(J),Y(J),I-J+1)
C
C Plot calculated values
        DO 30,IP=1,NPHI
          X(IP)=PHI1(IP)*RTOD
          Y(IP)=T1A(IT,IP)*DLTHET/TMAX+THETA
 30     CONTINUE
C Duplicate 1st point as last
        IP=NPHI
        IF(X(IP).NE.X(1)) THEN
          IP=NPHI+1
          X(IP)=X(1)+SIGN(360.,X(NPHI))
          Y(IP)=Y(1)
        ENDIF
        CALL PLTDSH(1,REP2,DASH2,0.)
        CALL PLTLIN(X,Y,IP)
C
 10   CONTINUE
C
C End plot
      CALL PLTSTP
C
      RETURN
      END
