C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C New subroutine added by G. Evans to read d*trek header files
C
      SUBROUTINE READHD(LUN,ISTAT,HNAME)
C     ============================
C
C Modified from READSV by Gwyndaf Evans 5th November 98
C
C
C     Read header file 'dtintegrate.head' from d*trek after integration
C     and profile fitting.
C
C On entry:
C   LUN        unit number to read
C
C On exit:
C   ISTAT = 0  file positioned to first reflection record
C         =+1  end of file found (shouldn't happen)
C         =-1  save data not found, probably reflection data
C
      IMPLICIT NONE
C
C Variable passed by subroutine
      INTEGER LUN,ISTAT
C
C KEYS - List of KEYWORDS required from the header
C NKEYS - Number of KEYS in list
C
      INTEGER NKEYS
      PARAMETER (NKEYS=28)
      CHARACTER*30 KEYS(NKEYS)
      CHARACTER*80 HNAME
C
C Common block to store all variables read from D*TREK header file
C
C     $DETNAM is the detector name and is determined by a first look
c     through the header file. It is then used to search keyword
c     starting with that name to get detector parameters
C
C INTOPT        Was used for Madnes data - not used for D*TREK
C DATUMM(3)     CRYSTAL_GONIO_VALUES        Datum position on MGONAX
c                                           goniostat axes (degrees)
C MGONAX        CRYSTAL_GONIO_NUM_VALUES    number of crystal goniostat axes
C GONAX(3)      CRYSTAL_GONIO_NAMES         names of the MGONAX goniostat axes
C GONVEC(3,3)   CRYSTAL_GONIO_VECTORS       vectors defining the directions 
C                                           of the MGONAX goniostat axes,
C                                           in the d*trek laboratory frame.
C                                           GONVEC(I,J),I=1,3 applies to the J'th axis
C DCELL(6)      CRYSTAL_UNIT_CELL           Cell dimensions (A & degrees)
C LSPGRP        CRYSTAL_SPACEGROUP          space group number
C EPS           CRYSTAL_MOSAICITY           reflection width (degrees)
C CRORVC(3,3)   CRYSTAL_ORIENT_VECTORS      vectors defining the three
C                                           rotations of the crystal
C                                           orientation. They always default
C                                           to (1 0 0)(0 1 0)(0 0 1).
C                                           CRORVC(I,J),I=1,3 applies to the J'th axis
C LAXES(3)      CRYSTAL_ORIENT_VECTORS      Axis permutation from d*trek
C                                           These are not always present in
C                                           the d*trek header file and are
C                                           set to defaults in absurd.f
C                                           LAXES(1,2,3) = (1,2,3)
C RPHI(3)       CRYSTAL_ORIENT_ANGLES       crystal rotation angles (degrees)
C
C DXMM          $DETNAM_GONIO_VALUES(6)     Crystal to detector distance (mm)
C DTAU(3)       $DETNAM_GONIO_VALUES(1,2,3) detector tilts: DTAU(2) = theta 
C                                           detector swing angle (degrees)
C MU(2)         SOURCE_ORIENT_ANGLES        beam tilt angles (degrees)
C LAMBDA        SCAN_WAVELENGTH             wavelength (A)
C DISP          SOURCE_SPECTRAL_DISPERSION  dispersion (delta lambda/lambda)
C SYN(4)        SOURCE_CROSSFIRE            synchrotron beam parameters: 
C                                           gammaH, gammaV, Delcor, ?syn4?
C LSCAXS                                    scan axis number (1 -> MGONAX)
C SCROVC(3)     SCAN_ROTATION_VECTOR        vector describing data 
C                                           collection axis (used here in 
C                                           definition of Cambridge frame
C                                           Z' basis vector)
C PSIRNG(2)     SCAN_ROTATION(1,2)          start and stop values of psi 
C                                           (D*trek scan axes - usually Omega)  (degrees)
C WIDINC        SCAN_ROTATION(3)            rotation width of each image (degrees)
C TIMIMG        SCAN_ROTATION(4)            time for each image (seconds)
C EXTIM0        exposure time (hours)
C SOURCM(3)     SOURCE_VECTORS(1,2,3)       idealized main beam vector 
C                                           (anti-parallel to beam!), in d*trek
C                                           laboratory frame (excluding the 
C                                           tilts parameterized by MU)
C S0M(3)        SOURCE_VECTORS(1,2,3)       main beam vector (anti -parallel 
C                                           to beam!), in d*trek laboratory frame 
C                                           (including the tilts parameterized by MU)
C LIMDET(2,2)   $DETNAM_DETECTOR_DIMENSIONS detector limits minimum, maximum Yms, Zms
C DETVEC(3,3)   $DETNAM_GONIO_VECTORS       vectors defining detector rotations
C DETTRN(3,2)   $DETNAM_DETECTOR_VECTORS    vectors defining detector translations
C RMSCFD        DTREFINE_RMS_MM             rms positional errors from last refinement
C RMSDEG        DTREFINE_RMS_DEG            rms rotational errors from last refinement
C DETOFF(2)     $DETNAM_GONIO_VALUES(4,5)      detector offsets ccx, ccy
C IMPBAT        DTP_DTINTEGRATE_OPTIONS(11) number of images per batch used in dtintegrate
C
C     New common block for d*trek specific things
C
C               SCAXIS                      scan axis
C               GONAX(3)                    names of the MGONAX goniostat axes
C               DETAX(3)                    names of the detector rotation angles
C               COMMENT                     crystal description
C
C
C                   D*TREK LABORATORY FRAME
C                   =======================
C
C     The d*trek laboratory frame has X along the omega axis (towards
C     base plate), Z antiparallel to the X-ray beam and Y completing a right-handed
C     system. All rotations are right-handed. This information is encoded in
C     GONVEC & S0, so these are used to define the frame.
C
C
C
      include 'madsav.fh'
      include 'madsvc.fh'
C
      SAVE /MADSVC/, /MADSAV/
C
C Local variables
C
C CRORVC - CRYSTAL_ORIENT_VECTORS from which LAXES axes permutation
C          is derived
C BEMNAM - name of beamline; defined by BEAMLINE_NAME keyword
C BEMLEN - integer containing length of BEMNAM
C DETNAM - name of detector; defined by DETECTOR_NAMES keyword
C DETLEN - integer containing length of DETNAM
C     N.B. I have only coded provision for one detector.
C
      INTEGER I,J,K,IKEY
      REAL CRYVEC(3,3)
      INTEGER DETLEN,BEMLEN
      CHARACTER*30 DETNAM,BEMNAM
C
C Things for PARSER ----
C
      INTEGER MAXTOK
      PARAMETER (MAXTOK=100)
      CHARACTER KEY*30,CVALUE(MAXTOK)*4,LINE*140
      INTEGER IBEG(MAXTOK),IEND(MAXTOK),ITYP(MAXTOK),
     .     IDEC(MAXTOK),NTOK
      REAL    FVALUE(MAXTOK)
C
C
C These are the equivalent d*trek keyword in corresponding positions
C except for RMSCHK which is split into two
C
C     Keywords related to DETECTOR parameters have DETECTOR_NAMES
C     prepending later on after we find out the detector name
C
      DATA KEYS /
     .       'INTOPT'
     .      ,'CRYSTAL_GONIO_VALUES'
     .      ,'CRYSTAL_UNIT_CELL'
     .      ,'CRYSTAL_SPACEGROUP'
     .      ,'CRYSTAL_ORIENT_VECTORS'
     .      ,'CRYSTAL_ORIENT_ANGLES'
     .      ,'GONIO_VALUES'
     .      ,'SOURCE_ORIENT_ANGLES'
     .      ,'CRYSTAL_MOSAICITY'
     .      ,'SOURCE_SPECTRAL_DISPERSION'
     .      ,'SOURCE_CROSSFIRE'
     .      ,'SCAN_ROTATION_AXIS_NAME'
     .      ,'SCAN_ROTATION_VECTOR'
     .      ,'SCAN_ROTATION'
     .      ,'EXP_TIME'
     .      ,'CRYSTAL_GONIO_NUM_VALUES'
     .      ,'CRYSTAL_GONIO_NAMES'
     .      ,'CRYSTAL_GONIO_VECTORS'
     .      ,'SOURCE_VECTORS'
     .      ,'SCAN_WAVELENGTH'
     .      ,'DETECTOR_DIMENSIONS'
     .      ,'CRYSTAL_DESCRIPTION'
     .      ,'GONIO_NAMES'
     .      ,'GONIO_VECTORS'
     .      ,'DETECTOR_VECTORS'
     .      ,'SOURCE_WAVELENGTH'
     .      ,'DTREFINE_RMS_MM'
     .      ,'DTREFINE_RMS_DEG'
     .      /
C     .      ,'DTP_DTINTEGRATE_OPTIONS'
C
C
C Set non-standard Parser delimiters for d*trek header files
      CALL PARSDL(' ;=,',4,1)
C
C     Default INTOPT to type = 0, for old data (slightly different from
C     INTOPT 1
C     Not required for d*trek header files yet!
      INTOPT = 0
C
C
C
C     Before parsing d*trek header file we have to determine the
C     DETECTOR_NAME and BEAMLINE_NAME.
C     so loop once through header file to find DETECTOR_NAME and then
C     do it again with modified keys
C
C Read a line from the header     
 3    READ(LUN,1001,END=999,ERR=998)LINE
C Check for last line
      IF(LINE.EQ.'}') GO TO 4
C Check for first line
      IF(LINE.EQ.'{') GOTO 3
      NTOK=-MAXTOK
      CALL PARSE(LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK)
      IF(NTOK.LE.0) GO TO 1
C Select first token as the KEY
      KEY=LINE(IBEG(1):IEND(1))
C
C Convert string to upper case
      CALL CCPUPC(KEY)
C
C Parse header file for BEAMLINE and DETECTOR name
C
C     Now try and fine the detector name from the keyword DETECTOR_NAMES
      IF(KEY.EQ.'DETECTOR_NAMES')THEN
C     Assign the second token to DETNAM
         DETNAM=LINE(IBEG(2):IEND(2))
C     Get length of DETNAM
         DETLEN=IEND(2)-IBEG(2)+1
C     Output detector name to STDOUT
         WRITE(6,'(A16,A30)')' Detector name: ' , DETNAM(1:DETLEN-1)
      END IF
C
C
C     Now try and fine the beamline name from the keyword BEAMLINE_NAME
      IF(KEY.EQ.'BEAMLINE_NAME')THEN
C     Assign the second token to BEMNAM
         BEMNAM=LINE(IBEG(2):IEND(2))
C     Get length of BEMNAM
         BEMLEN=IEND(2)-IBEG(2)+1
C     Output beamline name to STDOUT
         WRITE(6,'(A16,A30)')' Beamline name: ' , BEMNAM(1:BEMLEN-1)
      END IF
C
C If we haven't found it yet go to start of loop and read another line
      GO TO 3
C
C Rewind header file ready for parsing the important keyword
C
 4    REWIND LUN

C
C
C
C
C
C Now read in keywords of interest
C
C--- Read loop
 1    READ(LUN,1001,END=999,ERR=998)LINE
1001  FORMAT(A140)
C
C Check for end of file with "}" (end of save file)
 2    IF(LINE.EQ.'}') THEN
C  Get scan axis number from name
C         WRITE(6,*)' Finished reading header file'
C
         DO I = 1 , MGONAX
            IF(SCAXIS.EQ.GONAX(I)) LSCAXS=I
         END DO
C     End of save file found, return
         ISTAT=0
         GO TO 9000
C     ==========
      ENDIF
C
C Check for first line with "{" and ignore
C
      IF(LINE.EQ.'{') THEN
         GO TO 1
      END IF
C Divide line into tokens
      NTOK=-MAXTOK
      CALL PARSE(LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK)
C Check keys
      IF(NTOK.LE.0) GO TO 1
C
      KEY=LINE(IBEG(1):IEND(1))

C
C Check for LINE beginning with DETECTOR_NAME (DETNAM) and if so
C remove from LINE 
C
      IF(INDEX(LINE,DETNAM(1:DETLEN)).EQ.1) THEN
         KEY=KEY(DETLEN+1:)
      END IF
C Convert string to upper case
      CALL CCPUPC(KEY)
C
      CALL CHKHDKEY(KEY,KEYS,NKEYS,IKEY)
      IF(IKEY.LE.0) GO TO 1
C
C
C Depending on KEY go to correct line to select relevant information
C pertaining to that key
C
C
      GO TO (10,20,30,40,50,60,70,80,90,100,110,120,125,130,
     .    160,170,180,190,200,210,230,250,260,270,280,290,
     .    300,320), IKEY
C
C
C
C
C Here are the lines that actually extract the important values from the
C items in the header file
C
C     INTOPT - Not used for d*trek data
C
10    CALL GTPINT(2,INTOPT,NTOK,ITYP,FVALUE)
C  Default INTOPT to 1
      IF (INTOPT .LE. 0) INTOPT = 1
      WRITE(6,2310)KEY,INTOPT
      GO TO 1
C=======================================================================
C
C     DATUM  read goniostat datum values 
C     CRYSTAL_GONIO_VALUES
C
 20   DO I = 1 , 3
         CALL GTPREA(I+1,DATUMM(I),NTOK,ITYP,FVALUE)
      END DO
      WRITE(6,2200)KEY,DATUMM
      GO TO 1
C=======================================================================
C
C     CELL    read cell dimensions    
C     CRYSTAL_UNIT_CELL
C
30    DO I = 1 , 6 
         CALL GTPREA(I+1,DCELL(I),NTOK,ITYP,FVALUE)
      END DO
      WRITE(6,2030)KEY,DCELL
      GO TO 1
C=======================================================================
C
C     SPCGRP  space group number
C     CRYSTAL_SPACEGROUP
C
40    CALL GTPINT(2,LSPGRP,NTOK,ITYP,FVALUE)
      WRITE(6,2310)KEY,LSPGRP
      GO TO 1
C=======================================================================
C
C     LAXES   axis permutation
C     CRYSTAL_ORIENT_VECTORS
C     Read in CRYSTAL_ORIENT_VECTORS, find axes permutation and 
C     write to LAXES
C
 50   K=1
      DO J = 1 , 3
         DO I = 1 , 3 
            K=K+1
            CALL GTPREA(K,CRORVC(I,J),NTOK,ITYP,FVALUE)
            IF(CRORVC(I,J).EQ.1)LAXES(J)=I
         END DO
      END DO
      WRITE(6,2050)KEY,CRORVC
      WRITE(6,2051)LAXES
 2051 FORMAT('Axes permutation LAXES        : ',3i3)
      GO TO 1
C=======================================================================
C
C     MISSET  "missetting" angles
C     CRYSTAL_ORIENT_ANGLES
C
 60   DO I = 1 , 3 
         CALL GTPREA(I+1,RPHI(I),NTOK,ITYP,FVALUE)
      END DO
      WRITE(6,2200)KEY,RPHI
      GO TO 1
C=======================================================================
C
C     DIST   detector distance in mm
C     TAU    detector tilts
C     CAMERA beam offsets
C     $DETNAM_GONIO_VALUES
C
70    CALL GTPREA(7,DXMM,NTOK,ITYP,FVALUE)
C
      DO I=1,3
         CALL GTPREA(I+1,DTAU(I),NTOK,ITYP,FVALUE)
      END DO
      CALL GTPREA(5,DETOFF(1),NTOK,ITYP,FVALUE)
      CALL GTPREA(6,DETOFF(2),NTOK,ITYP,FVALUE)
      WRITE(6,2030)DETNAM(1:DETLEN)//KEY,DTAU,DETOFF,DXMM
      GO TO 1
C=======================================================================
C
C MU     beam tilts
C     SOURCE_ORIENT_ANGLES
C
 80   DO I = 1 , 2
         CALL GTPREA(I+1,MU(I),NTOK,ITYP,FVALUE)
      END DO
      WRITE(6,2080)KEY,MU
      GO TO 1
C=======================================================================
C
C     EPS     mosaicicity / divergence
C     CRYSTAL_MOSAICITY
C
90    CALL GTPREA(2,EPS,NTOK,ITYP,FVALUE)
      WRITE(6,2160)KEY,EPS
      GO TO 1
C=======================================================================
C
C     DISP   dispersion
C     SOURCE_SPECTRAL_DISPERSION
C
 100  CALL GTPREA(2,DISP,NTOK,ITYP,FVALUE)
      WRITE(6,2160)KEY,DISP
      GO TO 1
C=======================================================================
C
C     SYN   sunchrotron parameters
C     SOURCE_CROSSFIRE
C
110   DO I = 1 , 4
         CALL GTPREA(I+1,SYN(I),NTOK,ITYP,FVALUE)
      END DO
      WRITE(6,2110)KEY,SYN
      GO TO 1
C=======================================================================
C
C     SCAN   scan axis 
C     SCAN_ROTATION_AXIS_NAME
C
120   SCAXIS=LINE(IBEG(2):IEND(2))
      WRITE(6,2120)KEY,SCAXIS
      GO TO 1
C=======================================================================
C
C     SCAN_ROTATION_VECTOR
C
 125  DO I = 1 , 3
         CALL GTPREA(I+1,SCROVC(I),NTOK,ITYP,FVALUE)
      END DO
      WRITE(6,2200)KEY,SCROVC
      GO TO 1
C=======================================================================
C     PSIRNG  start & stop values of psi
C     WIDINC  image width (degrees)
C     TIMIMG  exposure time per image (seconds) ---> minutes
C     SCAN_ROTATION=PHIRNG(1) PSIRNG(2) WIDINC TIMIMG ..... ;
 130  DO I = 1 , 2 
         CALL GTPREA(I+1,PSIRNG(I),NTOK,ITYP,FVALUE)
      END DO
      CALL GTPREA(4,WIDINC,NTOK,ITYP,FVALUE)
      CALL GTPREA(5,TIMIMG,NTOK,ITYP,FVALUE)
C     Convert image exposure time into minutes
      TIMIMG=TIMIMG/60.0
      WRITE(6,2110)KEY,PSIRNG,WIDINC,TIMIMG
      IF (TIMIMG .LE. 0.0) THEN
         WRITE(6,'(A/A/2A/A)')
     .          '**************************************'
     .         ,'Warning EXPOSURE TIME value is not set'
     .         ,'Edit fourth item in SCAN_ROTATION=... header'
     .         ,'line of ', HNAME
     .         ,'**************************************'
      END IF
      GO TO 1
C=======================================================================
C
C    ****** NOT USED YET!!!! *****
C     EXPTIM  starting exposure time (hours) ?? To be done still
C     It would be nice if Jim P. wrote a data set satrt time to the
C     header or reflection file
C     EXP_TIME
C     
 160  CALL GTPREA(2,EXTIM0,NTOK,ITYP,FVALUE)
      WRITE(6,2160)KEY,EXTIM0
      GO TO 1
C=======================================================================
C    *****************************
C
C     MGONAX  number of goniostat axes
C     CRYSTAL_GONIO_NUM_VALUES
C
 170  CALL GTPINT(2,MGONAX,NTOK,ITYP,FVALUE)
      WRITE(6,2310)KEY,MGONAX
      GO TO 1
C=======================================================================
C
C     GONAX   names of axes
C     CRYSTAL_GONIO_NAMES
C
 180  DO I=1,3
         GONAX(I)=LINE(IBEG(I+1):IEND(I+1))
      END DO
      WRITE(6,2260)KEY,GONAX
      GO TO 1
C=======================================================================
C     
C     GONVEC  goniostat vectors
C     CRYSTAL_GONIO_VECTORS
C
190   K=1
cv
c fix for cases with
c
c     CRYSTAL_GONIO_NUM_VALUES=3;
c     CRYSTAL_GONIO_VECTORS=<18 values>;
c
c     -> take second 9 numbers
c
      IF (NTOK.GT.11) THEN
c
        K=1
        DO J=1,3
          DO I=1,3
            K=K+1
            CALL GTPREA(K,CRYVEC(I,J),NTOK,ITYP,FVALUE)
C            IF(CRYVEC(I,J).EQ.1)LAXES(J)=I
          END DO
        END DO
        WRITE(6,2050)KEY,CRYVEC
        WRITE(6,2051)LAXES
c
        K = 10
c
      END IF
cv
      DO J=1,3
         DO I=1,3
            K=K+1
            CALL GTPREA(K,GONVEC(I,J),NTOK,ITYP,FVALUE)
         END DO
      END DO
      WRITE(6,2050)KEY,GONVEC
      GO TO 1
C=======================================================================
C     
C     S0            source vector
C     SOURCE        source vector excluding tilts
C     These are taken to be the same for now!
C     SOURCE_VECTORS
C
 200  K = 1
      DO J = 1 , 3
         DO I =1 , 3 
            K = K + 1
            CALL GTPREA(K,SOURCM(I,J),NTOK,ITYP,FVALUE)
         END DO
      END DO
      WRITE(6,2050)KEY,SOURCM
      GO TO 1
C=======================================================================
C
C     LAMBDA  wavelength
C     SCAN_WAVELENGTH
C
 210  CALL GTPREA(2,LAMBDA,NTOK,ITYP,FVALUE)
      WRITE(6,2210)KEY,LAMBDA
      GO TO 1
C=======================================================================
C
C     DETLIM  detector limits (min values fixed at 0 because d*trek
C     header only define the max size in along X and Y in pixels or mm
C     I chose pixel since the reflection files output the reflection
C     coordinate in pixels
C     $DETNAM_DETECTOR_DIMENSIONS
C
230   K=1
      LIMDET(1,1)=0
      LIMDET(1,2)=0
      DO J = 1 , 2
         K=K+1
         CALL GTPINT(K,LIMDET(2,J),NTOK,ITYP,FVALUE)
      END DO
      WRITE(6,2230)DETNAM(1:DETLEN)//KEY,LIMDET
 2230 FORMAT(a30,': ',4i6)
      GO TO 1
C=======================================================================
C
C     COMMENT       skip next 3 lines
C     CRYSTAL_DESCRIPTION
C
250   COMMENT=LINE(IBEG(2):IEND(NTOK-1))
      WRITE(6,2250)KEY,COMMENT
 2250 FORMAT(a30,': ',a40)
      GO TO 1
C=======================================================================
C
C     DETAX   detector angle names
C     $DETNAM_GONIO_NAMES
C     
260   DO I = 1 , 3
         DETAX(I)=LINE(IBEG(I+1):IEND(I+1))
      END DO
      WRITE(6,2260)DETNAM(1:DETLEN)//KEY,DETAX
      GO TO 1
C=======================================================================
C
C     DETVEC   detector angle vectors
C     $DETNAM_GONIO_VECTORS
C
270   K=10
      DO J = 1 , 3
         DO I = 1 , 3
            K=K+1
            CALL GTPREA(K,DETVEC(I,J),NTOK,ITYP,FVALUE)
         END DO
      END DO
      WRITE(6,2050)DETNAM(1:DETLEN)//KEY,DETVEC
      GO TO 1
C=======================================================================
C
C     DETTRN   detector translation vectors 
C     $DETNAM_DETECTOR_VECTORS
C
280   K=1
      DO J = 1 , 2
         DO I = 1 , 3
            K=K+1
            CALL GTPREA(K,DETTRN(I,J),NTOK,ITYP,FVALUE)
         END DO
      END DO
      WRITE(6,2030)DETNAM(1:DETLEN)//KEY,DETTRN
      GO TO 1
C=======================================================================
C
C     IMPBAT - No. of images per batch
C     DTP_DTINTEGRATE_OPTIONS (11)
C
C 310  CALL GTPINT(12,IMPBAT,NTOK,ITYP,FVALUE)
C      WRITE(6,2310)KEY,IMPBAT
C      GO TO 1
C
C     LAMBDA
C     SOURCE_WAVELENGTH: second value if LAMBDA not yet set
C     i.e. it will take the SCAN_WAVELENGTH value if it is available
C
C
 290  CALL GTPREA(3,LAMBDA,NTOK,ITYP,FVALUE)
      WRITE(6,2210)KEY,LAMBDA
      GO TO 1
C=======================================================================
C
C     RMSCHK - RMSCFD
C     DTREFINE_RMS_MM (RMSCHK)   rms mm from dtrefine
C
 300  CALL GTPREA(2,RMSCFD,NTOK,ITYP,FVALUE)
      WRITE(6,2160)KEY,RMSCFD
      GO TO 1
C=======================================================================
C
C     RMSCHK - RMSDEG
C     DTREFINE_RMS_DEG (RMSCHK)   rms deg from dtrefine
C
 320  CALL GTPREA(2,RMSDEG,NTOK,ITYP,FVALUE)
      WRITE(6,2160)KEY,RMSDEG
      GO TO 1
C=======================================================================
C
C
C End of lines to extract header data
C
C
C     Format statements
C     Writing Key and a wavelength
 2210 FORMAT(a30,': ', f9.5)
C     Writing Key and a single real
 2160 FORMAT(a30,': ', f9.4)
C     Writing Key and two reals
 2080 FORMAT(a30,': ',2f8.2)
C     Writing Key and a vector
 2200 FORMAT(a30,': ',3f9.4)
C     Writing Key and 4 reals (Source crossfire, SCAN_ROTATION)
 2110 FORMAT(a30,': ',4f9.4)
C     Writing Key and 6 reals (Cell for example)
 2030 FORMAT(a30,': ',6f9.4)
C     Writing key and a 3x3 matrix
 2050 FORMAT(a30,': ',9f9.4)
C     Writing Key and an integer
 2310 FORMAT(a30,': ',i8)
C     Writing Key and three strings
 2260 FORMAT(a30,': ',3a15)
C     Writing Key and a string
 2120 FORMAT(a30,': ',a12)



C
C
C----------------------
C Error
998   WRITE(6,9998) LINE
9998  FORMAT(' Warning: Problem reading from header file:'/1X,A)
      GO TO 1
C
C ------------- End of file  (shouldn't get here)
C
999   WRITE(6,9999)
 9999 FORMAT(
     +    /' Premature end of file in dtrek header file !'/)
      ISTAT=1
      GO TO 9000
C
C***********************************************************************
C
C Exit: reset Parser delimiters to standard
C
 9000 CALL PARSDL(' ',0,0)
      RETURN
C     ======
C
      END
