C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C 26-Nov-1988       J. W. Pflugrath           Cold Spring Harbor Laboratory
C    Edited to conform to Fortran 77.  Renamed from Rotation_matrices to
C    RTMATS
C
C ==============================================================================
C
C! to calculate a rotation matrix and derivatives
C
      SUBROUTINE  RTMATS
C                 ! input: rotation angle
     1   (ANGLE
C                 ! input: rotation axis
     2   ,AXIS
C                 ! input: required number of derivatives
     3   ,NDER
C                 ! output: rotation matrix and up to 4 requested derivatives
     4   ,MATS)
C
CEE Creates a rotation matrix and optional derivatives up to the 4th with
CEE respect to the angle of rotation.  The direction of
CEE the input vector is taken as the axis down which the rotation represented
CEE by the matrix is seen to be clockwise,
CEE and the angle of rotation is taken to be in radians.
C rotation_matrices
Created: 14-NOV-1986 D.J.Thomas,   MRC Laboratory of Molecular Biology,
C                                  Hills Road, Cambridge, CB2 2QH, England
C    - from dRotation_matrix routine
C rotation_matrices
C
      IMPLICIT NONE
C
C                 ! rotation angle / radians
      REAL   ANGLE
C                 ! cos(rotation angle)
      REAL   CSR
C                 ! loop counters
      INTEGER   I, J
C                 ! number of derivatives required (on [0,4])
      INTEGER   NDER
C                 ! sin(rotation angle)
      REAL   SNR
C                 ! 1-cos(rotation angle)
      REAL   VCR
C
C                 ! rotation axis (unit vector)
      REAL   AXIS  (1:3)
C                 ! required rotation matrix and requested derivatives
      REAL   MATS (1:3,1:3,0:*)
C
      CSR = COS(ANGLE)
      VCR = 1.0 - CSR
      SNR = SIN(ANGLE)
C
      IF (NDER .LT. 0) THEN
C                  ! warning message, no other action necessary
         PRINT *, 'RTMATS: negative number of derivatives?'
      ENDIF
C
C                  ! evaluate the rotation matrix explicitly
      MATS(1,1,0) = VCR * AXIS(1) * AXIS(1)  +  CSR
      MATS(2,1,0) = VCR * AXIS(2) * AXIS(1)  +  SNR * AXIS(3)
      MATS(3,1,0) = VCR * AXIS(3) * AXIS(1)  -  SNR * AXIS(2)
      MATS(1,2,0) = VCR * AXIS(1) * AXIS(2)  -  SNR * AXIS(3)
      MATS(2,2,0) = VCR * AXIS(2) * AXIS(2)  +  CSR
      MATS(3,2,0) = VCR * AXIS(3) * AXIS(2)  +  SNR * AXIS(1)
      MATS(1,3,0) = VCR * AXIS(1) * AXIS(3)  +  SNR * AXIS(2)
      MATS(2,3,0) = VCR * AXIS(2) * AXIS(3)  -  SNR * AXIS(1)
      MATS(3,3,0) = VCR * AXIS(3) * AXIS(3)  +  CSR
C
C                  ! evaluate the (4n+1)th derivative matrix explicitly
      IF (NDER .GE. 1) THEN
         MATS(1,1,1) = SNR * AXIS(1) * AXIS(1)  -  SNR
         MATS(2,1,1) = SNR * AXIS(2) * AXIS(1)  +  CSR * AXIS(3)
         MATS(3,1,1) = SNR * AXIS(3) * AXIS(1)  -  CSR * AXIS(2)
         MATS(1,2,1) = SNR * AXIS(1) * AXIS(2)  -  CSR * AXIS(3)
         MATS(2,2,1) = SNR * AXIS(2) * AXIS(2)  -  SNR
         MATS(3,2,1) = SNR * AXIS(3) * AXIS(2)  +  CSR * AXIS(1)
         MATS(1,3,1) = SNR * AXIS(1) * AXIS(3)  +  CSR * AXIS(2)
         MATS(2,3,1) = SNR * AXIS(2) * AXIS(3)  -  CSR * AXIS(1)
         MATS(3,3,1) = SNR * AXIS(3) * AXIS(3)  -  SNR
C
C                   ! evaluate the (4n+2)th derivative matrix explicitly
         IF (NDER .GE. 2) THEN
            MATS(1,1,2) = CSR * AXIS(1) * AXIS(1)  -  CSR
            MATS(2,1,2) = CSR * AXIS(2) * AXIS(1)  -  SNR * AXIS(3)
            MATS(3,1,2) = CSR * AXIS(3) * AXIS(1)  +  SNR * AXIS(2)
            MATS(1,2,2) = CSR * AXIS(1) * AXIS(2)  +  SNR * AXIS(3)
            MATS(2,2,2) = CSR * AXIS(2) * AXIS(2)  -  CSR
            MATS(3,2,2) = CSR * AXIS(3) * AXIS(2)  -  SNR * AXIS(1)
            MATS(1,3,2) = CSR * AXIS(1) * AXIS(3)  -  SNR * AXIS(2)
            MATS(2,3,2) = CSR * AXIS(2) * AXIS(3)  +  SNR * AXIS(1)
            MATS(3,3,2) = CSR * AXIS(3) * AXIS(3)  -  CSR
C
C                    ! evaluate the (4n+3)th derivative matrix
            IF (NDER .GE. 3) THEN
               DO 100 J = 1, 3
                  DO 100 I = 1, 3
C                    ! (4n+3)th derivative is minus (4n+1)th derivative
100                  MATS(I,J,3) =  -  MATS(I,J,1)
                  CONTINUE
               CONTINUE
C
C                    ! evaluate the (4n+4)th derivative matrix
               IF (NDER .GE. 4) THEN
                  DO 200 J = 1, 3
                     DO 200 I = 1, 3
C                       ! (4n+4)th derivative is minus (4n+2)th derivative
200                     MATS(I,J,4) =  -  MATS(I,J,2)
                     CONTINUE
                  CONTINUE
C
                  IF (NDER .GE. 5) THEN
                     PRINT *, 'RTMATS: > 4 derivatives requested!'
                     PRINT *, ' Only the first 4 have been evaluated '
                     PRINT *, ' since they repeat after this.'
                     PRINT *, ' Additionally, d3=-d1, d4=-d2.'
                  ENDIF
               ENDIF
            ENDIF
         ENDIF
      ENDIF
C
      RETURN
      END
