C
C     F2MTZ
C     Copyright (C) 1991 Morten Kjeldgaard
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C*************************************************************
C
C
C**************************************************************
C

      program f2mtz
* Formatted_2_MTZ
* Convert a formatted reflection file of some sort to mtz format.
* Written at the CCP4/ESF workshop in Cambridge, Oct-1991.
* Morten Kjeldgaard.

C

      integer maxlab, maxtok, maxfrm
* Max number of labels
      parameter (maxlab = 80)
* Max number of tokens for 'parser'
      parameter (maxtok = 2*maxlab+1)
* Max length of user-specified format string
      parameter (maxfrm = 200)

      integer i, k, ii, jj, ct, lun, dummy, ntok, nskip
      integer ibeg(maxtok), iend(maxtok), ityp(maxtok), idec(maxtok)
      integer nsymx, nsympx, nspgrx, nzero
      real fvalue(maxtok), cell(6), adata(maxtok-1),SCAL(MAXLAB)
      character str*500,date*8,date6*6
      character*64 PROJECTNAME,CRYSTALNAME,DATASETNAME
      character*64 XNAME(maxlab),DNAME(maxlab)
      character*4 key, subkey, cvalue(maxtok)
      character spgrnx*10, pgnamx*10
      character lsprgo(maxlab)*30, ctprgo(maxlab)
      integer nlprgo
      character frmat*200,frmatx*200,errline*200
      real rsymx(4,4,192)
      real nan
      logical qisnan
      external qnan,qisnan
      logical lsymm, lfile, lcell, labout, lctyp, xpfreer(maxlab), lend
      data lsymm, lfile, lcell, labout, lctyp /5*.false./
      data lun /1/
      data frmat /'*'/
      data xpfreer /maxlab*.false./

      ntok = maxtok
      nsymx = 0
      nskip = 0
      DO 1 I = 1,MAXLAB
1      SCAL(I) = 1.0

      call ccpfyp
      call mtzini
      CALL CCPRCS(6,'f2mtz','$Date: 2005/09/06 11:25:12 $')
      write (*,*) 'Maximum number of columns in this version:',maxlab

C Set default PROJECTNAME, CRYSTALNAME and DATASETNAME
      CALL CCPDAT(DATE)
      WRITE(DATE6,'(A2,A2,A2)') DATE(1:2),DATE(4:5),DATE(7:8)
 5    I = INDEX(DATE6,' ')
      IF (I.GT.0) THEN
        WRITE(DATE6(I:I),'(A1)') '0'
        GOTO 5
      ENDIF
      PROJECTNAME   = 'unknown'
      CRYSTALNAME   = 'unknown'
      DATASETNAME   = 'unknown'//DATE6

* Open mtz file 
      call lwopen(1,'HKLOUT')

* Parse user input
 10   continue

      str = ' '
      ntok = maxtok
      call parser (key, str, ibeg, iend, ityp, fvalue, cvalue,
     $     idec, ntok, lend, .true.)
      if (lend) go to 500

*     TITLE keyword 
      if (key .eq. 'TITL') then
         call lwtitl (1, str(ibeg(2):iend(ntok)), 0)   
         goto 10

*     CELL keyword - read in the cell parameters
      else if (key .eq. 'CELL') then
         call rdcell (2, ityp, fvalue, ntok, cell)
         call lwcell(1, cell)
         lcell = .true.
         go to 10

*     LABOUT keyword
      else if (key .eq. 'LABO') then
         k = 0
         if (ntok.gt.maxlab) call ccperr (1, 'Too many labels')
         do 20 i = 2, ntok
            k = k+1
            lsprgo(k) = str(ibeg(i):iend(i))
 20      continue
         nlprgo = (ntok-1)
         write (*,*) 'Number of columns to be read in:', nlprgo
         labout = .true.
         goto 10

*     CTYPOUT keyword
      else if (key .eq. 'CTYP') then
         k = 0
         if (ntok.gt.maxlab) call ccperr (1, 'Too many columns')
         do 22 i = 2, ntok
            k = k+1
            ctprgo(k) = str(ibeg(i):iend(i))
            call ccpupc (ctprgo(k))
            if (ctprgo(k).eq.'X') then
*             fudge for X-PLOR free R input in opposite sense, indicated
C             by column type X, which is converted to W.  the column
C             will have its value fixed if xpfreer is true
              ctprgo(k) = 'I'
              xpfreer(k) = .true.
            endif
 22      continue
         lctyp = .true.
         goto 10
C
*     SCALE keyword
      else if (key .eq. 'SCAL') then
         k = 0
         do 21 i = 2, ntok
            k = k+1
            SCAL(k) = fvalue(i)
 21      continue
         goto 10

*     SYMM keyword
      else if (key .eq. 'SYMM') then
         call rdsymm (2, str, ibeg, iend, ityp, fvalue, ntok,
     $        spgrnx, nspgrx, pgnamx, nsymx, nsympx, rsymx)
         lsymm = .true.
         goto 10

*     FILE keyword
      else if (key .eq. 'FILE') then
         str = str(ibeg(2):iend(2))
         call ccpdpn(lun, str, 'READONLY', 'F', dummy, 0)
         lfile = .true.
         goto 10

*     SKIP keyword
      else if (key .eq. 'SKIP') then
         if (ntok .le. 1) goto 10
         if (ityp(2) .eq. 2) then
            call keynum(1, 2, str, ibeg, iend, ityp, ntok)
            nskip = int(fvalue(2))
            write (*,'(a,i10,a)')
     &           'First', nskip,' lines of file will be skipped.'
         endif
         goto 10

*     FORMAT keyword
      else if (key .eq. 'FORM') then
       call ccpupc(str)
         if (ntok .ne. 2)
     +       call fatal(lun,'FORMAT must have only one argument')
         if (iend(2)-ibeg(2).gt.maxfrm) then
           call ccperr(1,'FORMAT format string too long')
         else
C  Format must not contain an A descriptor
         if ( index(frmat,'A').ne.0) then
           write (*,FMT='(A,/,A)') 
     +  ' *** Format contains an "A"  - this is not permitted'
           call ccperr (1, ' Format contains A descriptor')
         end if
C  Format must contain SOMETHING; () is not enough!
         if (iend(2)-ibeg(2).lt.2) then
           call ccperr(2,
     +   'FORMAT format string is empty - will default to free format')
           go to 10
         else
           frmat = ' '
           frmatx = str(ibeg(2):iend(2))
         endif
c
C Check for I descriptors and replace with F
c
           lenfmtx = iend(2) +1 - ibeg(2)
            jj = 1
            ii = 1
 234       continue
             if(frmatx(ii:ii).ne.'I') then
              frmat(jj:jj) = frmatx(ii:ii)
               jj = jj + 1
               ii = ii + 1
             else
              frmat(jj:jj) = 'F'
               ii = ii + 1
               jj = jj + 1
               frmat(jj:jj+2) = frmatx(ii:ii)//'.0'
               ii = ii + 1
               jj = jj + 3
             endif
               if(ii.le.lenfmtx) go to 234
               if (jj.gt.ii) write (*,FMT='(A)') 
     +  ' *** Format contains Ix.  -     Replaced with Fx.0'
           call ccperr (2, ' Format I replaced with F')
         end if
         goto 10

c     ==========================================
c---- name project <pname> crystal <xname> dataset <dname>
c     ==========================================
c---- assign project/crystal/dataset names for new mtz file

      ELSEIF (KEY.EQ.'NAME') THEN

        if (ntok.lt.7) then
          call ccperr(2,'name keyword line incomplete.')
          goto 10
        endif

        n = 2
 451    if (n .le. ntok) then
         subkey = cvalue(n)(1:4)
         call ccpupc(subkey)
         if (subkey .eq. 'PROJ') then
c  set current pname
            PROJECTNAME = str(ibeg(n+1):iend(n+1))
            n = n+2
         elseif (subkey .eq. 'CRYS') then
c  set current xname
            CRYSTALNAME = str(ibeg(n+1):iend(n+1))
            n = n+2
         elseif (subkey .eq. 'DATA') then
            DATASETNAME = str(ibeg(n+1):iend(n+1))
            n = n+2
         else
            write(errline,'(a,a)') 
     +            'wrong subkeyword after NAME: ',cvalue(n)
            call ccperr(1,errline)
         endif
         go to 451
        endif

        go to 10
C
C     ==========================================
C---- PNAME    >> PROJECTNAME [pname]  <string>
C     ==========================================
C     
      ELSEIF (KEY.EQ.'PNAM') THEN
         if (ntok .ge. 2) PROJECTNAME = str(ibeg(2):iend(2))
         goto 10
C
C     ===========================================
C---- XNAME    >> CRYSTALNAME  [xname]  <string>
C     ===========================================
C     
      ELSEIF (KEY.EQ.'XNAM') THEN
         if (ntok .ge. 2) CRYSTALNAME = str(ibeg(2):iend(2))
         goto 10
C
C     ===========================================
C---- DNAME    >> DATASETNAME  [dname]  <string>
C     ===========================================
C     
      ELSEIF (KEY.EQ.'DNAM') THEN
         if (ntok .ge. 2) DATASETNAME = str(ibeg(2):iend(2))
         goto 10

*     END keyword
      else if (key .eq. 'END') then
         goto 500
*     else error
      else
        call ccperr (1, 'Bad keyword: '//key)
      endif

*** End of input parsing.

 500  continue

      if (.not. lcell) call fatal(lun,'CELL must be specified')
      if (.not. lsymm) call fatal(lun,'SYMMETRY must be specified')
      if (.not. labout) call fatal(lun,'LABOUT must be specified')
      if (.not. lctyp) call fatal(lun,'CTYPOUT must be specified')
      if (.not. lfile) call ccpdpn(lun, 'HKLIN', 'READONLY',
     $     'F', dummy, 0)

* Store the project name and dataset name in the mtz header:
      if (PROJECTNAME.eq.'unknown')
     +   call ccperr(2,' PROJECTNAME not assigned') 
      if (CRYSTALNAME.eq.'unknown')
     +   call ccperr(2,' CRYSTALNAME not assigned') 
      if (DATASETNAME(1:7).eq.'unknown')
     +   call ccperr(2,' DATASETNAME not assigned') 
      call lwidx(1,PROJECTNAME,CRYSTALNAME,DATASETNAME,CELL,0.0)

* Store the column labels in the mtz header:
      call lwclab(1, lsprgo, nlprgo, ctprgo, 0)

* Associate columns with datasets
      do i=1,nlprgo
        xname(i) = CRYSTALNAME
        dname(i) = DATASETNAME
      enddo
      call lwidasx(1, nlprgo, xname, dname , 0)

* Store the symmetry in the mtz header:
      call lwsymm (1, nsymx, nsympx, rsymx, spgrnx(1:1), nspgrx,
     $     spgrnx, pgnamx)

* Store a history header:
      call lwhstl (1, ' ')

*** Read all reflections from input file...
      if (nskip .ne. 0) then
         do 25 i = 1,nskip
            read (lun,*)
 25      continue
      endif

      ct = 0
      call qnan(nan)
 30   continue
c   Skip lines beginning with #
       read(lun,'(A)',end=50, err=1000) str
      if(str(1:1).eq.'#') then
        write(*,*)' Comment line ignored:   ',str(1:50)
        go to 30
      end if
      backspace lun
      if (frmat.eq.'*') then
        do 32 i=1,nlprgo
C         You can default data to `missing' with free format.
          adata(i) = nan
 32     continue
         read (lun, *, end=50, err=1000) (adata(i),i=1,nlprgo)
      else
         read (lun, frmat, end=50, err=1000) (adata(i),i=1,nlprgo)
      endif
*     scale if required
      nzero = 0
      do 31 i = 1,nlprgo
        if (qisnan(adata(i))) goto 31
        if (adata(i) .eq. 0.0) nzero = nzero + 1
        adata(i)= scal(i)*adata(i)
        if (xpfreer(i)) then
*         fiddle X-PLOR or SHELX free R to opposite sense
          if (adata(i) .ge. 0.000001) then
*           exclude
            adata (i) = 0.0
          else
*           include
            adata (i) = 1.0
          endif
        endif
 31   continue
      if (nzero .eq. nlprgo) then
         call ccp4h_summary_beg()
         write(*,*) ' ****Reflection rejected: record contains only ',
     +              ' zeros (blank line?)****.'
         call ccp4h_summary_end()
      else
         ct = ct+1
         call lwrefl (1, adata)
      endif
      goto 30

* Close mtz file, print full header:
 50   call ccp4h_summary_beg()
      call lwclos (1, 3)
      close(unit=lun,status='keep')
      write (*,'(i7, '' input reflections processed'')') ct
      call ccp4h_summary_end()
      call ccperr(0,'Normal termination')

 1000 write (*, FMT='(A,/,A)') '*** Read error',
     +     'Check FORMAT -- especially the need for REALs'
      write (str, '(''problems reading reflection '',i7)') ct
      call fatal (lun,str)
      end

c=======================================================================
       subroutine fatal (lun,str)
* Fatal -- issue fatal error message and die.
       integer lun
       character str*(*)
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

       close(unit=lun,status='keep')
       call ccperr(1,str)
       end
