C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
c
c ----------------------------------------------------------------------
c
      subroutine calcmetric()
      include 'fffear.fh'
      include 'io.fh'
      include 'crystal.fh'
c Calculate the orthonormal - crystallographic transform
c
      integer i,j
      real a,b,c,alph,beta,gamm,sum,cosas,q123
c
      do 10 j=1,4
      do 10 i=1,4
      q(i,j)=0.0
      qt(i,j)=0.0
 10   continue
c
      a=cell(1)
      b=cell(2)
      c=cell(3)
      alph=dtor*cell(4)
      beta=dtor*cell(5)
      gamm=dtor*cell(6)
      sum=0.5*(alph+beta+gamm)
      vol=2.0*a*b*c*
     + sqrt(sin(sum)*sin(sum-alph)*sin(sum-beta)*sin(sum-gamm))
      cosas=(cos(beta)*cos(gamm)-cos(alph))/(sin(beta)*sin(gamm))
c orthogonalisation matrix
      q(1,1)=a
      q(1,2)=b*cos(gamm)
      q(1,3)=c*cos(beta)
      q(2,2)=b*sin(gamm)
      q(2,3)=-c*sin(beta)*cosas
      q(3,3)=c*sin(beta)*sqrt(1.0-cosas**2)
      q(4,4)=1.0
c and its inverse
      q123=q(1,1)*q(2,2)*q(3,3)
      qt(1,1)=1.0/q(1,1)
      qt(1,2)=-q(1,2)*q(3,3)/q123
      qt(1,3)=(q(1,2)*q(2,3)-q(2,2)*q(1,3))/q123
      qt(2,2)=1.0/q(2,2)
      qt(2,3)=-q(1,1)*q(2,3)/q123
      qt(3,3)=1.0/q(3,3)
      qt(4,4)=1.0
c
      write (lpt,4)vol
 4    format (' Unit cell volume:',f10.1)
      write (lpt,1)((q(i,j),j=1,3),(qt(i,j),j=1,3),i=1,3)
 1    format (' Orthogonalisation matrices:',3(/1x,3f8.3,8x,3f8.3))
c Calculate real space metric tensor
      rlm11=q(1,1)*q(1,1)+q(2,1)*q(2,1)+q(3,1)*q(3,1)
      rlm12=2.0*(q(1,1)*q(1,2)+q(2,1)*q(2,2)+q(3,1)*q(3,2))
      rlm13=2.0*(q(1,1)*q(1,3)+q(2,1)*q(2,3)+q(3,1)*q(3,3))
      rlm22=q(1,2)*q(1,2)+q(2,2)*q(2,2)+q(3,2)*q(3,2)
      rlm23=2.0*(q(1,2)*q(1,3)+q(2,2)*q(2,3)+q(3,2)*q(3,3))
      rlm33=q(1,3)*q(1,3)+q(2,3)*q(2,3)+q(3,3)*q(3,3)
      write (lpt,2)rlm11,rlm22,rlm33,rlm12,rlm13,rlm23
 2    format (' Real space metric tensor:      '/1x,6f9.1)
c Calculate reciprocal space metric tensor
      rcm11=qt(1,1)*qt(1,1)+qt(1,2)*qt(1,2)+qt(1,3)*qt(1,3)
      rcm12=2.0*(qt(1,1)*qt(2,1)+qt(1,2)*qt(2,2)+qt(1,3)*qt(2,3))
      rcm13=2.0*(qt(1,1)*qt(3,1)+qt(1,2)*qt(3,2)+qt(1,3)*qt(3,3))
      rcm22=qt(2,1)*qt(2,1)+qt(2,2)*qt(2,2)+qt(2,3)*qt(2,3)
      rcm23=2.0*(qt(2,1)*qt(3,1)+qt(2,2)*qt(3,2)+qt(2,3)*qt(3,3))
      rcm33=qt(3,1)*qt(3,1)+qt(3,2)*qt(3,2)+qt(3,3)*qt(3,3)
      write (lpt,3)rcm11,rcm22,rcm33,rcm12,rcm13,rcm23
 3    format (' Reciprocal space metric tensor:'/1x,6f9.6)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine polartomatrix(omega,phi,kappa,matrix)
      include 'fffear.fh'
c calculate rotation matrix from Euler angles
      real omega,phi,kappa,matrix(4,4)
c
      real someg,comeg,sinph,cosph,sinchi,coschi,cosch1,dc1,dc2,dc3
c
      someg = sin(omega*dtor)
      comeg = cos(omega*dtor)
      sinph = sin(phi*dtor)
      cosph = cos(phi*dtor)
      dc1 = someg*cosph
      dc2 = someg*sinph
      dc3 = comeg
      sinchi = sin(kappa*dtor)
      coschi = cos(kappa*dtor)
      cosch1 = 1.0 - coschi
      matrix(1,1) = dc1*dc1*cosch1 + coschi
      matrix(1,2) = dc1*dc2*cosch1 - dc3*sinchi
      matrix(1,3) = dc3*dc1*cosch1 + dc2*sinchi
      matrix(2,1) = dc1*dc2*cosch1 + dc3*sinchi
      matrix(2,2) = dc2*dc2*cosch1 + coschi
      matrix(2,3) = -dc1*sinchi + dc2*dc3*cosch1
      matrix(3,1) = -dc2*sinchi + dc3*dc1*cosch1
      matrix(3,2) = dc2*dc3*cosch1 + dc1*sinchi
      matrix(3,3) = dc3*dc3*cosch1 + coschi
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine eulertomatrix(alpha,beta,gamma,matrix)
      include 'fffear.fh'
c calculate rotation matrix from Euler angles
      real alpha,beta,gamma,matrix(4,4)
c
      real ca,cb,cg,sa,sb,sg
c
      ca = cos(dtor*alpha)
      sa = sin(dtor*alpha)
      cb = cos(dtor*beta)
      sb = sin(dtor*beta)
      cg = cos(dtor*gamma)
      sg = sin(dtor*gamma)
      matrix(1,1) = cg*cb*ca - sg*sa
      matrix(1,2) = -sg*cb*ca - cg*sa
      matrix(1,3) = sb*ca
      matrix(2,1) = cg*cb*sa + sg*ca
      matrix(2,2) = -sg*cb*sa + cg*ca
      matrix(2,3) = sb*sa
      matrix(3,1) = -cg*sb
      matrix(3,2) = sg*sb
      matrix(3,3) = cb
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine matrixtoeuler(matrix,alpha,beta,gamma)
      include 'fffear.fh'
c calculate Euler angles from rotation matrix
      real alpha,beta,gamma,matrix(4,4)
c
      integer i,j,k,l
      real ca,cb,cg,sa,sb,sg,a(3),dr,r11,r22,r12,mat1(4,4),mat2(4,4)
      parameter (dr=0.001)
c
c get initial estimate
      cb=matrix(3,3)
      sb=sqrt(0.5*(matrix(1,3)**2+matrix(2,3)**2+
     +             matrix(3,1)**2+matrix(3,2)**2))
      if (sb.gt.0.01) then
       ca= matrix(1,3)/sb
       sa= matrix(2,3)/sb
       cg=-matrix(3,1)/sb
       sg= matrix(3,2)/sb
      else
       ca=1.0
       sa=0.0
       cg=matrix(2,2)
       sg=matrix(2,1)
      endif
      a(1)=atan2(sa,ca)
      a(2)=atan2(sb,cb)
      a(3)=atan2(sg,cg)
c refine the estimate
      do k=1,2
       do l=1,3
        call eulertomatrix(a(1)/dtor,a(2)/dtor,a(3)/dtor,mat1)
        a(l)=a(l)+dr
        call eulertomatrix(a(1)/dtor,a(2)/dtor,a(3)/dtor,mat2)
        a(l)=a(l)-dr
        r11=0.0
        r22=0.0
        r12=0.0
        do i=1,3
         do j=1,3
          r11=r11+(mat2(i,j)-mat1(i,j))**2
          r22=r22+(matrix(i,j)-mat1(i,j))**2
          r12=r12+(matrix(i,j)-mat1(i,j))*(mat2(i,j)-mat1(i,j))
         enddo
        enddo
        a(l)=a(l)+r12/r11*dr
       enddo
      enddo
c
      alpha=mod(a(1)/dtor+360.0,360.0)
      beta =mod(a(2)/dtor+360.0,360.0)
      gamma=mod(a(3)/dtor+360.0,360.0)
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine gridres(res,nu,nv,nw)
      include 'fffear.fh'
      include 'crystal.fh'
c calculate grid required for a particular data resolution
      integer nu0,nv0,nw0,nu,nv,nw
      real res
      nu0=2*int(cell(1)/res)+1
      nv0=2*int(cell(2)/res)+1
      nw0=2*int(cell(3)/res)+1
      call gridsize(nu0,nv0,nw0,nu,nv,nw)
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine gridsize(nu0,nv0,nw0,nu,nv,nw)
      include 'fffear.fh'
      include 'io.fh'
      include 'crystal.fh'
c Calculate a valid grid size
c
      integer nu0,nv0,nw0,nu,nv,nw
c
      integer mu,mv,mw,i,fftfac
      external fftfac
c
c Set up factors required by symmetry and fft
      mu=2
      mv=2
      mw=2
      do 100 i=1,nsym
       if (rsym(1,4,i).ne.0.0) mu=max(mu,nint(1.0/rsym(1,4,i)))
       if (rsym(2,4,i).ne.0.0) mv=max(mv,nint(1.0/rsym(2,4,i)))
       if (rsym(3,4,i).ne.0.0) mw=max(mw,nint(1.0/rsym(3,4,i)))
100   continue
      if (mod(mu,2).eq.1) mu=mu*2
      if (mod(mv,2).eq.1) mv=mv*2
      if (mod(mw,2).eq.1) mw=mw*2
c
c if no grid set then set grid sizes
      if (nu+nv+nw.eq.0) then
       nu=fftfac(nu0,mu)
       nv=fftfac(nv0,mv)
       nw=fftfac(nw0,mw)
      endif
c
c print the result
      write (lpt,110)nu,nv,nw,mu,mv,mw
 110  format(
     +/' Grid dimensions ',3i4,'  must contain the following prime'
     +/' factors for agreement with symmetry restrictions- ',3i4/)
c check the input grid - should only fail if input by user
      if (mod(nu,mu)+mod(nv,mv)+mod(nw,mw).ne.0) call ccperr(1,
     +  ' GRIDSIZE - GRID card does not fit symmetry restrictions')
c
      return
      end
c
c ----------------------------------------------------------------------
c
      integer function fftfac(n0,m0)
      include 'fffear.fh'
c FFTFAC - find the composite number greater n than n0 which
c will optimise the speed of the fft and has m0 as factor
c (naive calculation: time = (p1.p2.p3...pn).(p1+p2+p3+..pn) )
c
      integer n0,m0
c
      integer nprime
      parameter (nprime=5)
      integer iprime(10),i,j,n,n1,nmin,time,timemin,sum,prod
      data iprime/2,3,5,7,11,13,17,19,23,27/
c
      n1=((n0+m0-1)/m0)*m0
      nmin=0
      timemin=1000000
c Try a number
      do 200 n=n1,2*n1,m0
c decompose n and add its factors to sum and prod
       i=n
       sum=0
       prod=1
       do 150 j=1,nprime
 100    if (mod(i,iprime(j)).eq.0) then
         i=i/iprime(j)
         sum=sum+iprime(j)
         prod=prod*iprime(j)
         goto 100
        endif
 150   continue
       time=sum*prod
       if (i.eq.1.and.time.lt.timemin) then
        nmin=n
        timemin=time
       endif
 200  continue
c
      fftfac=nmin
c
      return
      end
c
c ----------------------------------------------------------------------
c
      integer function numres(reso)
      include 'fffear.fh'
      include 'crystal.fh'
      include 'hkldata.fh'
      include 'hkllim.fh'
c NUMRES - calculate number of reflections at a particular resolution
c this is the slow but accurate counting method!
c
      real reso
c
      integer in(3),ih,ik,il,n,inasu
      real s12,s13,s23,s2,s3,s,slim,eps,phicen
      logical centre,sysabs
      external inasu
      equivalence (in(1),ih)
      equivalence (in(2),ik)
      equivalence (in(3),il)
c
      n=0
      slim=1.0/reso**2
c *** optimise this bit ***
      do 100 il=-lmax,lmax
      s13 = il*rcm13
      s23 = il*rcm23
      s3 = il**2*rcm33
      do 100 ik=-kmax,kmax
      s12 = ik*rcm12 + s13
      s2 = ik*(ik*rcm22 + s23) + s3
      do 100 ih=-hmax,hmax
      s  = ih*(ih*rcm11 + s12) + s2
      if (s.gt.0.0.and.s.le.slim) then
       if (inasu(in,nlaue).eq.1) then
        call epscen(ih,ik,il,centre,sysabs,eps,phicen,nsym,rsym)
        if (.not.sysabs) n=n+1
       endif
      endif
 100  continue
c
      numres=n
c
cc Fast method underestimates in many space groups
cc      numres=(2.0*pi*vol)/(3.0*reso**3*nsym)
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine ccpmaphead
     +  (name,nspgrp,cell,nu,nv,nw,nu1,nv1,nw1,nu2,nv2,nw2)
      include 'fffear.fh'
      include 'io.fh'
c CCPMAPhead - get limits of mask from file
c
      character name*(*)
      integer nspgrp,nu,nv,nw,nu1,nv1,nw1,nu2,nv2,nw2
      real cell(6)
c
      integer jfms(3),juvw(3),mxyz(3),m1(3),m2(3),nsec,mode
      integer i,ifail,iprint
      real rmin,rmax,rmean,rrms,qmin,qmax,qmean,qrms,qcell(6)
      character title*80
c
c read the file headers
      ifail=0
      iprint=1
      call mrdhds(lmsks,name,title,nsec,jfms,mxyz,m1(3),m1(1),m2(1),
     +  m1(2),m2(2),cell,nspgrp,mode,rmin,rmax,rmean,rrms,ifail,iprint)
      call mrclos(lmsks)
      m2(3)=m1(3)+nsec-1
      do 110 i=1,3
       juvw(jfms(i))=i
 110  continue
      nu1=m1(juvw(1))
      nu2=m2(juvw(1))
      nv1=m1(juvw(2))
      nv2=m2(juvw(2))
      nw1=m1(juvw(3))
      nw2=m2(juvw(3))
      nu=mxyz(1)
      nv=mxyz(2)
      nw=mxyz(3)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine ccpmskin
     +  (name,title,msk,nu1,nv1,nw1,nu2,nv2,nw2)
      include 'fffear.fh'
      include 'io.fh'
c CCPMSKIN - read a ccp4 logical msk and store in xyz order
c
      integer nu1,nv1,nw1,nu2,nv2,nw2
      integer msk(nu1:nu2,nv1:nv2,nw1:nw2)
      character name*(*),title*(*)
c
      real lsec(0:maxsec)
c
      integer ifast,imedm,islow,lfast,lmedm,lslow,ierr,ifail,iprint
      integer m1(3),m2(3),ifms(3),jfms(3),juvw(3),mxyz(3)
      integer i,iu,iv,iw
      integer nsec,mode,mspgrp
      real cell(6),rmin,rmax,rmean,rrms
c
c
c Note: juvw convert from fast/med/slow to u/v/w
c       jfms convert from u/v/w to fast/med/slow
c
c now open msk header and read msk, re-ordering it as necessary
c
      ifail=0
      iprint=0
      call mrdhds(lmsks,name,title,nsec,jfms,mxyz,m1(3),m1(1),m2(1),
     +  m1(2),m2(2),cell,mspgrp,mode,rmin,rmax,rmean,rrms,ifail,iprint)
      m2(3)=m1(3)+nsec-1
      do 110 i=1,3
       juvw(jfms(i))=i
 110  continue
c check we got the right header info:
      if (nu1.ne.m1(juvw(1)).or.nu2.ne.m2(juvw(1)).or.
     +    nv1.ne.m1(juvw(2)).or.nv2.ne.m2(juvw(2)).or.
     +    nw1.ne.m1(juvw(3)).or.nw2.ne.m2(juvw(3)))
     +  call ccperr(1,'ccpmskin - mask grid sizes are inconsistent')
c find out the msk grid dimensions
      lfast=m2(1)-m1(1)+1
      lmedm=m2(2)-m1(2)+1
      lslow=m2(3)-m1(3)+1
c
      if (lfast*lmedm.gt.maxsec)
     +  call ccperr(1,' ccpmskin - Msk section > lsec: recompile')
c now read the msk in the order it is on file
      do 200 islow=0,lslow-1
c get a section
       ierr=0
       call mgulpr(lmsks,lsec,ierr)
       if (ierr.ne.0) call ccperr(1,' ccpmskin - ccp4 read error')
c now sort into uvw msk
       ifms(3)=islow+m1(3)
       do 190 imedm=0,lmedm-1
       ifms(2)=imedm+m1(2)
       do 190 ifast=0,lfast-1
       ifms(1)=ifast+m1(1)
        iu=ifms(juvw(1))
        iv=ifms(juvw(2))
        iw=ifms(juvw(3))
        msk(iu,iv,iw)=nint(lsec(ifast+lfast*imedm))
 190   continue
 200  continue
c close the msk file
      call mrclos(lmsks)
c
      write (lpt,910)
 910  format (/' MAP/MASK READ SUCCESSFUL'//)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine ccpmskout
     +  (name,title,msk,nspgrp,cell,nu,nv,nw,nu1,nv1,nw1,nu2,nv2,nw2)
      include 'fffear.fh'
      include 'io.fh'
c CCPMAPIN - read a ccp4 logical map and store in xyz order
c
      integer nspgrp,nu,nv,nw,nu1,nv1,nw1,nu2,nv2,nw2
      integer msk(nu1:nu2,nv1:nv2,nw1:nw2)
      real cell(6)
      character name*(*),title*(*)
c
      integer lsec(maxsec)
c
      integer ifast,imedm,islow,lfast,lmedm,lslow,ierr,ifail,iprint
      integer m1(3),m2(3),ifms(3),jfms(3),juvw(3),mxyz(3)
      integer i,m,iu,iv,iw
      integer nsec,mode,nsym,nsymp
      real rmin,rmax,rmean,rrms,rsym(4,4,192),u,v,w,u1,v1,w1,rho
      character*10 namspg,nampg
c
c
c spacegroups with yxz=1 and zxy=2 axis ordering
      integer axis(230)
      data axis/2,2,2,2,1,1,1,1,1,2,1,1,1,1,1,2,2,2,1,2,2,1,2,207*1/
c
c Note: juvw convert from fast/med/slow to u/v/w
c       jfms convert from u/v/w to fast/med/slow
c
c now open map header and read map, re-ordering it as necessary
c
      if (axis(mod(nspgrp,1000)).eq.1) then
       jfms(1)=2
       jfms(2)=1
       jfms(3)=3
      else
       jfms(1)=3
       jfms(2)=1
       jfms(3)=2
      endif
c
      do 110 i=1,3
       juvw(jfms(i))=i
 110  continue
c
      mxyz(1)=nu
      mxyz(2)=nv
      mxyz(3)=nw
      m1(juvw(1))=nu1
      m2(juvw(1))=nu2
      m1(juvw(2))=nv1
      m2(juvw(2))=nv2
      m1(juvw(3))=nw1
      m2(juvw(3))=nw2
      nsec=m2(3)-m1(3)+1
      mode=0
c
      call msymlb(latom,nspgrp,namspg,nampg,nsym,nsymp,rsym)
c
      call mwrhdl(lmsks,name,title,nsec,jfms,mxyz,
     +  m1(3),m1(1),m2(1),m1(2),m2(2),cell,nspgrp,mode)
      call msywrt(lmsks,nsym,rsym)
c
c find out the mask grid dimensions
      lfast=m2(1)-m1(1)+1
      lmedm=m2(2)-m1(2)+1
      lslow=m2(3)-m1(3)+1
c
      if (lfast*lmedm.gt.4*maxsec)
     +  call ccperr(1,' ccpmapin - Mask section > lsec: recompile')
c now write the map in the new order
      do 200 islow=0,lslow-1
c sort onto section
       ifms(3)=islow+m1(3)
       do 190 imedm=0,lmedm-1
       ifms(2)=imedm+m1(2)
       do 190 ifast=0,lfast-1
       ifms(1)=ifast+m1(1)
        iu=ifms(juvw(1))
        iv=ifms(juvw(2))
        iw=ifms(juvw(3))
        call ccpstb(msk(iu,iv,iw),lsec,ifast+lfast*imedm+1)
 190   continue
c write the section
       call mspew(lmsks,lsec)
 200  continue
c close the map file
      call mwclose(lmsks)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine maskgrid(ma,au,av,aw,au1,av1,aw1,au2,av2,aw2,
     +                    mb,bu,bv,bw,bu1,bv1,bw1,bu2,bv2,bw2)
      include 'fffear.fh'
c Covert a mask from one grid to another.
c note: masks may contain multiple discrete levels, so we may not
c interpolate.
c mask B is not zeroed and zero values in A are unchanged.
c
      integer au,av,aw,au1,av1,aw1,au2,av2,aw2
      integer bu,bv,bw,bu1,bv1,bw1,bu2,bv2,bw2
      integer ma(au1:au2,av1:av2,aw1:aw2),mb(bu1:bu2,bv1:bv2,bw1:bw2)
c
      integer iu,iv,iw,ju,jv,jw
c
      do 100 iw=bw1,bw2
      jw=nint(real(iw)*real(aw)/real(bw))
      do 100 iv=bv1,bv2
      jv=nint(real(iv)*real(av)/real(bv))
      do 100 iu=bu1,bu2
      ju=nint(real(iu)*real(au)/real(bu))
      if (ju.ge.au1.and.ju.le.au2.and.jv.ge.av1.and.
     +    jv.le.av2.and.jw.ge.aw1.and.jw.le.aw2) then
       if (ma(ju,jv,jw).ne.0) mb(iu,iv,iw)=ma(ju,jv,jw)
      endif
 100  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine mapstats(rho,rave,rstd)
      include 'fffear.fh'
      include 'uvwdata.fh'
c Produce stats for a map
c
      real rho(nu,nv,nw),rave,rstd
c
      integer i,j,k
      real sn,sx,sx2
c
      sn=0.0
      sx=0.0
      sx2=0.0
      do 10 k=1,nw
      do 10 j=1,nv
      do 10 i=1,nu
      sn=sn+1.0
      sx=sx+rho(i,j,k)
      sx2=sx2+rho(i,j,k)**2
 10   continue
c
      rave=sx/sn
      rstd=sqrt(max(sn*sx2-sx**2,0.0))/sn
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine mapcorrel(rho1,rho2,correl)
      include 'fffear.fh'
      include 'uvwdata.fh'
c Produce correl between two maps
c
      real rho1(nu,nv,nw),rho2(nu,nv,nw),correl
c
      integer i,j,k
      real sn,sx,sy,sx2,sy2,sxy
c
      sn=0.0
      sx=0.0
      sy=0.0
      sx2=0.0
      sy2=0.0
      sxy=0.0
      do 10 k=1,nw
      do 10 j=1,nv
      do 10 i=1,nu
      sn=sn+1.0
      sx=sx+rho1(i,j,k)
      sy=sy+rho2(i,j,k)
      sx2=sx2+rho1(i,j,k)**2
      sy2=sy2+rho2(i,j,k)**2
      sxy=sxy+rho1(i,j,k)*rho2(i,j,k)
 10   continue
c
      correl=(sn*sxy-sx*sy)/sqrt(max((sn*sx2-sx**2)*(sn*sy2-sy**2),0.0))
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine symuvw(mat,u,v,w,u1,v1,w1)
      include 'fffear.fh'
c symuvw - multiply a 4-matrix by a 3-vector
      real mat(4,4),u,v,w,u1,v1,w1
      u1=mat(1,1)*u+mat(1,2)*v+mat(1,3)*w+mat(1,4)
      v1=mat(2,1)*u+mat(2,2)*v+mat(2,3)*w+mat(2,4)
      w1=mat(3,1)*u+mat(3,2)*v+mat(3,3)*w+mat(3,4)
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine symcpy(a,b)
      include 'fffear.fh'
c symcpy - copy a symmetry matrix
      real a(16),b(16)
      integer i
      do 100 i=1,16
       b(i)=a(i)
 100  continue
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine symmatmul(a,b,c)
      include 'fffear.fh'
c symmetmaul - 4x4 matrix multiply, c=a*b
      real a(4,4),b(4,4),c(4,4)
      integer i,j,k
c
      do 20 k=1,4
      do 20 i=1,4
      c(i,k)=0.0
      do 20 j=1,4
      c(i,k)=c(i,k)+a(i,j)*b(j,k)
20    continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine copy3d(x1,x2)
      include 'fffear.fh'
      include 'uvwdata.fh'
c copy data from one array to another: x1->x2
      real x1(*),x2(*)
c
      integer i
c
      do 10 i=1,nu*nv*nw
      x2(i)=x1(i)
 10   continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine add3d(x1,x2,x3)
      include 'fffear.fh'
      include 'uvwdata.fh'
c add map arrays x1=x2+x3
      real x1(*),x2(*),x3(*)
c
      integer i
c
      do 10 i=1,nu*nv*nw
      x1(i)=x2(i)+x3(i)
 10   continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine sub3d(x1,x2,x3)
      include 'fffear.fh'
      include 'uvwdata.fh'
c subtract map arrays x1=x2-x3
      real x1(*),x2(*),x3(*)
c
      integer i
c
      do 10 i=1,nu*nv*nw
      x1(i)=x2(i)-x3(i)
 10   continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine mul3d(x1,x2,x3)
      include 'fffear.fh'
      include 'uvwdata.fh'
c multiply map arrays x1=x2*x3
      real x1(*),x2(*),x3(*)
c
      integer i
c
      do 10 i=1,nu*nv*nw
      x1(i)=x2(i)*x3(i)
 10   continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine hklfpab(hkls,lf,lphi)
      include 'fffear.fh'
      include 'hkldata.fh'
c generate A,B list from F,PHI list
      integer lf,lphi
      real hkls(nhklc,0:*)
c
      integer i,lfa,lfb
      real f,phi
c
      lfa=lf
      lfb=lphi
c
c first the origin term
      hkls(lfa,0)=hkls(lf,0)
      hkls(lfb,0)=0.0
c now go through the rest of the list
      do 100 i=1,nrflo
       f=hkls(lf,i)
       phi=hkls(lphi,i)
       hkls(lfa,i)=f*cos(phi)
       hkls(lfb,i)=f*sin(phi)
 100  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine hklabfp(hkls,lfa,lfb)
      include 'fffear.fh'
      include 'hkldata.fh'
c generate F,PHI list from A,B list
      integer lfa,lfb
      real hkls(nhklc,0:*)
c
      integer i,lf,lphi
      real phaseof
      complex fh
      external phaseof
c
      lf  =lfa
      lphi=lfb
c
c first the origin term
      hkls(lf,0)=hkls(lfa,0)
      hkls(lphi,0)=0.0
c now go through the rest of the list
      do 100 i=1,nrflo
       fh=cmplx(hkls(lfa,i),hkls(lfb,i))
       hkls(lf,i)=abs(fh)
       hkls(lphi,i)=phaseof(fh)
 100  continue
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine fftlistmapn(rfl3d,hkls,lfa,lfb,nu,nv,nw)
      include 'fffear.fh'
      include 'crystal.fh'
      include 'hkldata.fh'
c FFT two columns of data from the reflection list into a map
c of arbitrary size (for averaging, skeletons, whatever)
c I don't think I need an inverse for this.
c
      integer lfa,lfb,nu,nv,nw
      real hkls(nhklc,0:*)
      real rfl3d(0:nu-1,0:nv-1,0:nw-1)
c
      real fftscale
      integer iu,iv,iw
c
c expand list to map
      call copylistmap(hkls,lfa,lfb,rfl3d,nu,nv,nw)
c
c scale the 3d array
      fftscale=1.0/vol
      do 100 iw=0,nw-1
      do 100 iv=0,nv-1
      do 100 iu=0,nu-1
      rfl3d(iu,iv,iw)=rfl3d(iu,iv,iw)*fftscale
 100  continue
c
c Now perform hermitian-real FFT
      call fft3hr(rfl3d,nu,nv,nw)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine fftlistmap(rfl3d,hkls,lfa,lfb)
      include 'fffear.fh'
      include 'crystal.fh'
      include 'hkldata.fh'
      include 'uvwdata.fh'
c FFT two columns of data from the reflection list into a map
c
      integer lfa,lfb
      real hkls(nhklc,0:*)
      real rfl3d(0:nu-1,0:nv-1,0:nw-1)
c
      real fftscale
      integer iu,iv,iw
c
c expand list to map
      call copylistmap(hkls,lfa,lfb,rfl3d,nu,nv,nw)
c
c scale the 3d array
      fftscale=1.0/vol
      do 100 iw=0,nw-1
      do 100 iv=0,nv-1
      do 100 iu=0,nu-1
      rfl3d(iu,iv,iw)=rfl3d(iu,iv,iw)*fftscale
 100  continue
c
c Now perform hermitian-real FFT
      call fft3hr(rfl3d,nu,nv,nw)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine copylistmap(hkls,lfa,lfb,rfl3d,nu,nv,nw)
      include 'fffear.fh'
      include 'crystal.fh'
      include 'hkldata.fh'
c copy two columns of data from the reflection list into a map with sym
c
      integer lfa,lfb,nu,nv,nw
      real hkls(nhklc,0:*)
      real rfl3d(0:nu-1,0:nv-1,0:nw-1)
c
      integer i,m,iu,iv,iw,ih,ik,il,ihm,ikm,ilm,lmod
      complex fh,fm,phase
      logical hemi
      external phase,hemi,lmod
c
c clear the 3d array
      do 100 iw=0,nw-1
      do 100 iv=0,nv-1
      do 100 iu=0,nu-1
      rfl3d(iu,iv,iw)=0.0
 100  continue
c
c now fill it with reflections
c loop through reflections
      do 200 i=1,nrflo
c fetch a reflection (complex)
       ih=nint(hkls(1,i))
       ik=nint(hkls(2,i))
       il=nint(hkls(3,i))
       fh=cmplx(hkls(lfa,i),hkls(lfb,i))
c calculate symmetry equivalents for this reflection
       do 150 m=1,nsymp
        ihm=nint(rsym(1,1,m)*ih+rsym(2,1,m)*ik+rsym(3,1,m)*il)
        ikm=nint(rsym(1,2,m)*ih+rsym(2,2,m)*ik+rsym(3,2,m)*il)
        ilm=nint(rsym(1,3,m)*ih+rsym(2,3,m)*ik+rsym(3,3,m)*il)
        fm=fh*
     +   phase(-2.0*pi*(ih*rsym(1,4,m)+ik*rsym(2,4,m)+il*rsym(3,4,m)))
c and store, filling in all the l=0 plane
        if (ilm.ge.0) then
         iu=  lmod(ihm,nu)
         iv=  lmod(ikm,nv)
         iw=2*lmod(ilm,nw)
         rfl3d(iu,iv,iw)  = real(fm)
         rfl3d(iu,iv,iw+1)= aimag(fm)
        endif
        if (-ilm.ge.0) then
         iu=  lmod(-ihm,nu)
         iv=  lmod(-ikm,nv)
         iw=2*lmod(-ilm,nw)
         rfl3d(iu,iv,iw)  = real(fm)
         rfl3d(iu,iv,iw+1)=-aimag(fm)
        endif
c next symmetry equivalent
 150   continue
c next reflection
 200  continue
c
c include (000)
      rfl3d(0,0,0)=hkls(lfa,0)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine invfftmaplist(rfl3d,hkls,lfa,lfb)
      include 'fffear.fh'
      include 'crystal.fh'
      include 'hkldata.fh'
      include 'uvwdata.fh'
c inverse FFT from map into reflection list
c
      integer lfa,lfb
      real hkls(nhklc,0:*)
      real rfl3d(0:nu-1,0:nv-1,0:nw-1)
c
      real fftscale0,fftscale,dphim
      integer i,m,iu,iv,iw,ih,ik,il,ihm,ikm,ilm,lmod
      complex sf,phase
      logical hemi
      external phase,hemi,lmod
c
c NB scale includes term for averaging over primative symmetry elements
      fftscale0=vol/real(nu*nv*nw)
      fftscale=vol/real(nu*nv*nw*nsymp)
c
c Perform Real-Hermitian FFT and scale the result
c
      call fft3rh(rfl3d,nu,nv,nw)
c
c fill the reflection list - loop through reflections
      do 200 i=1,nrflo
       ih=nint(hkls(1,i))
       ik=nint(hkls(2,i))
       il=nint(hkls(3,i))
c average over symmetry equivalents for this reflection
       sf=cmplx(0.0,0.0)
       do 150 m=1,nsymp
        ihm=nint(rsym(1,1,m)*ih+rsym(2,1,m)*ik+rsym(3,1,m)*il)
        ikm=nint(rsym(1,2,m)*ih+rsym(2,2,m)*ik+rsym(3,2,m)*il)
        ilm=nint(rsym(1,3,m)*ih+rsym(2,3,m)*ik+rsym(3,3,m)*il)
        dphim=2.0*pi*(ih*rsym(1,4,m)+ik*rsym(2,4,m)+il*rsym(3,4,m))
c note: we perform conjugate for inv fft
        if (hemi(ihm,ikm,ilm)) then
         iu=  lmod(ihm,nu)
         iv=  lmod(ikm,nv)
         iw=2*lmod(ilm,nw)
         sf=sf+cmplx(rfl3d(iu,iv,iw),-rfl3d(iu,iv,iw+1))*phase(dphim)
        else
         iu=  lmod(-ihm,nu)
         iv=  lmod(-ikm,nv)
         iw=2*lmod(-ilm,nw)
         sf=sf+cmplx(rfl3d(iu,iv,iw), rfl3d(iu,iv,iw+1))*phase(dphim)
        endif
 150   continue
c and store
       hkls(lfa,i)=fftscale*real(sf)
       hkls(lfb,i)=fftscale*aimag(sf)
c next reflection
 200  continue
c
c get (000) term
      hkls(lfa,0)=fftscale0*rfl3d(0,0,0)
      hkls(lfb,0)=0.0
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine fft3d(xrho)
      include 'fffear.fh'
      include 'crystal.fh'
      include 'hkldata.fh'
      include 'uvwdata.fh'
      include 'hkllim.fh'
c Crystallographic Hermitian to Real FFT
c In:  complex xrfl(h,k,l)  (l=0...nl/2)
c Out: complex xrho(X,Y,Z)  (real)
c
      real xrho(0:nu-1,0:nv-1,0:nw-1)
c
      integer i,j,k,ih,jh,jh1,ik,jk,jk1,lmod
      real scale
      logical hemi
      external hemi,lmod
c
      scale=1.0/vol
c
c Fill in l=0 plane
c
      do 160 ik=kmin,kmax
      jk =lmod( ik,nv)
      jk1=lmod(-ik,nv)
      do 160 ih=hmin,hmax
      jh =lmod( ih,nu)
      jh1=lmod(-ih,nu)
      if (hemi(ih,ik,0)) then
       xrho(jh1,jk1,0)= xrho(jh,jk,0)
       xrho(jh1,jk1,1)=-xrho(jh,jk,1)
      endif
 160  continue
c
c Now perform hermitian-real FFT and scale the result
c
      call fft3hr(xrho,nu,nv,nw)
c
      do 100 k=0,nw-1
      do 100 j=0,nv-1
      do 100 i=0,nu-1
       xrho(i,j,k)=scale*xrho(i,j,k)
 100  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine inv3d(xrho)
      include 'fffear.fh'
      include 'crystal.fh'
      include 'uvwdata.fh'
c Crystallographic Real to Hermitian INVERSE FFT
c In:  complex xrho(X,Y,Z) (real)
c Out: complex xrfl(h,k,l)  (l=0...nl/2)
c
      real xrho(0:nu-1,0:nv-1,0:nw-1)
c
      integer i,j,k
      real scale
c
      scale=vol/real(nu*nv*nw)
c
c Perform Real-Hermitian FFT and scale, conjugate the result
c
      call fft3rh(xrho,nu,nv,nw)
c
      do 100 k=0,nw/2-1
      do 100 j=0,nv-1
      do 100 i=0,nu-1
       xrho(i,j,2*k)  = scale*xrho(i,j,2*k)
       xrho(i,j,2*k+1)=-scale*xrho(i,j,2*k+1)
 100  continue
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine forallrfl(func,xrfl)
      include 'fffear.fh'
      include 'crystal.fh'
      include 'hkldata.fh'
      include 'uvwdata.fh'
      include 'hkllim.fh'
c for all the rfls in the array xrfl perform function 'func'
c
      real xrfl(0:nu-1,0:nv-1,0:nw-1)
c
      integer ih,ik,il,jh,jk,jl,lmod
      real s12,s13,s23,s2,s3,s
      complex fh
c
      external func,lmod
c
c *** optimise this bit ***
      do 100 jl=0,nw/2-1
      il=lmod(jl-lmin,nw)+lmin
      s13 = il*rcm13
      s23 = il*rcm23
      s3 = il**2*rcm33
      do 100 jk=0,nv-1
      ik=lmod(jk-kmin,nv)+kmin
      s12 = ik*rcm12 + s13
      s2 = ik*(ik*rcm22 + s23) + s3
      do 100 jh=0,nu-1
      ih=lmod(jh-hmin,nu)+hmin
      s  = ih*(ih*rcm11 + s12) + s2
      if (s.le.smax) then
       fh=cmplx(xrfl(jh,jk,2*jl),xrfl(jh,jk,2*jl+1))
       call func(ih,ik,il,s,fh)
       xrfl(jh,jk,2*jl)  =real(fh)
       xrfl(jh,jk,2*jl+1)=aimag(fh)
      else
       xrfl(jh,jk,2*jl)  =0.0
       xrfl(jh,jk,2*jl+1)=0.0
      endif
 100  continue
c
c
      return
      end
c
c ----------------------------------------------------------------------
c
