C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
c
c ----------------------------------------------------------------------
c
c   Routines to calculate sphere of complex structure factors
c   from a model or map
c
c ----------------------------------------------------------------------
c
      subroutine atmdensity(fh,eh,atoms,nused,tx0,ty0,tz0)
c calculate mask and map and generate structure factors by fft
      include 'fffear.fh'
      include 'params.fh'
      include 'uvwdata.fh'
      include 'srcdata.fh'
      include 'hkldata.fh'
      include 'crystal.fh'
      include 'io.fh'
c
      complex fh(0:*),eh(0:*)
      real atoms(6,*),tx0,ty0,tz0
      integer nused
c
      real u,v,w,b,occ,f,r,rr,ratom2,rmask2,maxrad,rhomap,rhomsk
      real x,y,z,gx,gy,gz,rh,rk,rl,sa1,sa2,frgrad
      integer i,j,k,n,ix,iy,iz,hcent,dcent
      complex c
c
c      real alt(50),ast(50),blt(50),bst(50),clt(50)
c
      integer lmod
      external lmod
c
c calculate model radius and centre
      maxrad=max(mdlrad,mskrad)
      tx0=0.0
      ty0=0.0
      tz0=0.0
      do i=1,nused
       tx0=tx0+atoms(1,i)
       ty0=ty0+atoms(2,i)
       tz0=tz0+atoms(3,i)
      enddo
      tx0=-tx0/real(nused)
      ty0=-ty0/real(nused)
      tz0=-tz0/real(nused)
      frgrad=0.0
      do i=1,nused
       frgrad=max(frgrad,sqrt((atoms(1,i)+tx0)**2+
     +    (atoms(2,i)+ty0)**2+(atoms(3,i)+tz0)**2)+maxrad)
      enddo
c calculate grids
      n=nu*nv*nw/2
c get grid size...
      hgrid=int(real(n)**0.333)
      hcent=hgrid/2
      dcent=hcent+hgrid*(hcent+hgrid*(hcent))
c
      do i=0,hgrid*hgrid*hgrid-1
       fh(i)=(0.0,0.0)
       eh(i)=(0.0,0.0)
      enddo
c
c Electron density calculation
c
      rscale=(2.0*frgrad)/real(hgrid)
      ratom2=mdlrad**2
      rmask2=mskrad**2
c
      do i=1,nused
c add contributions from all atoms
       x=atoms(1,i)+tx0
       y=atoms(2,i)+ty0
       z=atoms(3,i)+tz0
       b=atoms(4,i)
       occ=atoms(5,i)
       j=nint(atoms(6,i))
       sa1=fa1(j)*(12.56637/(fb1(j)+b))**1.5
       sa2=fa2(j)*(12.56637/(fb2(j)+b))**1.5
       do iz=int((z-maxrad)/rscale),int((z+maxrad)/rscale)+1
       gz=real(iz)*rscale
       do iy=int((y-maxrad)/rscale),int((y+maxrad)/rscale)+1
       gy=real(iy)*rscale
       do ix=int((x-maxrad)/rscale),int((x+maxrad)/rscale)+1
       gx=real(ix)*rscale
        k=ix+hgrid*(iy+hgrid*iz)+dcent
        r=(x-gx)**2+(y-gy)**2+(z-gz)**2
        if (r.lt.rmask2) then
         eh(k)=(1.0,0.0)
        endif
        if (r.lt.ratom2) then
         rr=39.47842*r
         f=sa1*exp(-rr/(fb1(j)+b))+sa2*exp(-rr/(fb2(j)+b))
         fh(k)=fh(k)+cmplx(occ*f,0.0)
        endif
       enddo
       enddo
       enddo
      enddo
c
c fh=product of mask and map
      do i=0,hgrid*hgrid*hgrid-1
       fh(i)=fh(i)*eh(i)
      enddo
c
      write(lpt,510)frgrad,-tx0,-ty0,-tz0
 510  format(/,' SEARCH MODEL FROM ATOMIC COORDINATES',
     +       /,'  Radius ',f5.1,'A, Centre ',3f6.1)
c
      return
      end
c
c!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
c test: if the origin term is right, this should exactly double the results
c       eh(i)=2.0*eh(i)
c!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
c
c ----------------------------------------------------------------------
c
      subroutine mapdensity(fh,eh,tx0,ty0,tz0)
c calculate mask and map and generate structure factors by fft
      include 'fffear.fh'
      include 'params.fh'
      include 'uvwdata.fh'
      include 'srcdata.fh'
      include 'hkldata.fh'
      include 'crystal.fh'
      include 'io.fh'
c
      complex fh(0:*),eh(0:*)
      real tx0,ty0,tz0
c
      real lsec(0:maxsec)
      integer n,hcent,dcent
      integer ifast,imedm,islow,lfast,lmedm,lslow,ierr,ifail,iprint
      integer m1(3),m2(3),ifms(3),jfms(3),juvw(3),mxyz(3)
      integer i,iu,iv,iw
      integer nsec,mode,mspgrp
      real mcell(6),mrmin,mrmax,mrmean,mrrms
      real r,rhop,rhof,varp,varf
      complex c
      character title*240
      real rint,rfrac
      external rint,rfrac
c
c      real alt(50),ast(50),blt(50),bst(50),clt(50)
c
c calculate grids
      n=uvwsize/2
c get the grid size...
      hgrid=int(real(n)**0.333)
      hcent=hgrid/2
      dcent=hcent+hgrid*(hcent+hgrid*(hcent))
c zero the grid
      do i=0,hgrid*hgrid*hgrid-1
       fh(i)=(0.0,0.0)
       eh(i)=(0.0,0.0)
      enddo
c
c READ THE ELECTRON DENSITY
c
      ifail=0
      iprint=0
      call mrdhds(lmsks,'MAPIN',title,nsec,jfms,mxyz,m1(3),m1(1),m2(1),
     +  m1(2),m2(2),mcell,mspgrp,mode,mrmin,mrmax,mrmean,mrrms,ifail,
     +  iprint)
      m2(3)=m1(3)+nsec-1
      do 110 i=1,3
       juvw(jfms(i))=i
 110  continue
c
      if (m2(1)-m1(1).ge.hgrid.or.m2(2)-m1(2).ge.hgrid.or.
     +    m2(3)-m1(3).ge.hgrid) goto 9010
      if (mcell(4).ne.90.0.or.mcell(5).ne.90.0.or.mcell(6).ne.90.0.or.
     +    mcell(1).ne.mcell(2).or.mcell(1).ne.mcell(3)) goto 9020
      if (mxyz(1).ne.mxyz(2).or.mxyz(1).ne.mxyz(3)) goto 9030

c find out the msk grid dimensions
      lfast=m2(1)-m1(1)+1
      lmedm=m2(2)-m1(2)+1
      lslow=m2(3)-m1(3)+1
c
      if (lfast*lmedm.gt.maxsec)

     +  call ccperr(1,' ccpmskin - Map section > maxsec: recompile')
c now read the msk in the order it is on file
      do 200 islow=0,lslow-1
c get a section
       ierr=0
       call mgulpr(lmsks,lsec,ierr)
       if (ierr.ne.0) call ccperr(1,' ccpmskin - ccp4 read error')
c now sort into uvw map and store in centre of map
       ifms(3)=islow-lslow/2
       do 190 imedm=0,lmedm-1
       ifms(2)=imedm-lmedm/2
       do 190 ifast=0,lfast-1
       ifms(1)=ifast-lfast/2
        iu=ifms(juvw(1))
        iv=ifms(juvw(2))
        iw=ifms(juvw(3))
        fh(iu+hgrid*(iv+hgrid*iw)+dcent)=
     +    cmplx(lsec(ifast+lfast*imedm),0.0)
 190   continue
 200  continue
c close the map file
      call mrclos(lmsks)
c
c NOW GET MASK FROM MAP
c
c mask is 1 where map is non-zero
      do i=0,hgrid*hgrid*hgrid-1
       if (abs(fh(i)).ne.0.0) eh(i)=(1.0,0.0)
      enddo
c
c save the shift we applied to the map
      rscale=mcell(1)/mxyz(1)
      tx0=rscale*real(-m1(juvw(1))-(m2(juvw(1))-m1(juvw(1))+1)/2)
      ty0=rscale*real(-m1(juvw(2))-(m2(juvw(2))-m1(juvw(2))+1)/2)
      tz0=rscale*real(-m1(juvw(3))-(m2(juvw(3))-m1(juvw(3))+1)/2)
      write(lpt,510)-tx0,-ty0,-tz0
 510  format(/,' SEARCH MODEL FROM INPUT MAP',
     +       /,'  Centre ',3f6.1)
c
      return
 9010 write (lpt,9011)hgrid
 9011 format('frgmapin - Fragment map is too big for current calc.'
     +  ,/,' Either increase the GRID in fffear, trim the map to'
     +  ,/,' remove unused sections, or calculate the fragment density'
     +  ,/,' on a coarser grid. Maximum allowed dimension is ',i4)
      call ccperr(1,'frgmapin - Error: fragment map grid>program grid')
 9020 call ccperr(1,'frgmapin - Error: fragment map cell not cubic')
 9030 call ccperr(1,'frgmapin - Error: fragment map grid not uniform')
      end
c
c ----------------------------------------------------------------------
c
      subroutine maxdensity(fh,eh,tx0,ty0,tz0,mapscl,mapstd)
c calculate mask and map and generate structure factors by fft
      include 'fffear.fh'
      include 'params.fh'
      include 'uvwdata.fh'
      include 'srcdata.fh'
      include 'hkldata.fh'
      include 'crystal.fh'
      include 'io.fh'
c
      complex fh(0:*),eh(0:*)
      real tx0,ty0,tz0,mapscl,mapstd
c
      real lsec(0:maxsec)
      integer n,hcent,dcent
      integer ifast,imedm,islow,lfast,lmedm,lslow,ierr,ifail,iprint
      integer m1(3),m2(3),ifms(3),jfms(3),juvw(3),mxyz(3)
      integer i,iu,iv,iw
      integer nsec,mode,mspgrp
      real mcell(6),mrmin,mrmax,mrmean,mrrms
      real r,rhop,rhof,varp,varf
      complex c
      character title*240
      real rint,rfrac
      external rint,rfrac
c
c      real alt(50),ast(50),blt(50),bst(50),clt(50)
c
c calculate grids
      n=uvwsize/2
c get the grid size...
      hgrid=int(real(n)**0.333)
      hcent=hgrid/2
      dcent=hcent+hgrid*(hcent+hgrid*(hcent))
c zero the grid
      do i=0,hgrid*hgrid*hgrid-1
       fh(i)=(0.0,0.0)
       eh(i)=(0.0,0.0)
      enddo
c
c READ THE ELECTRON DENSITY
c
      ifail=0
      iprint=0
      call mrdhds(lmsks,'MAXIN',title,nsec,jfms,mxyz,m1(3),m1(1),m2(1),
     +  m1(2),m2(2),mcell,mspgrp,mode,mrmin,mrmax,mrmean,mrrms,ifail,
     +  iprint)
      m2(3)=m1(3)+nsec-1
      do 110 i=1,3
       juvw(jfms(i))=i
 110  continue
c
      if (m2(1)-m1(1).ge.hgrid.or.m2(2)-m1(2).ge.hgrid.or.
     +    m2(3)-m1(3).ge.hgrid) goto 9010
      if (mcell(4).ne.90.0.or.mcell(5).ne.90.0.or.mcell(6).ne.90.0.or.
     +    mcell(1).ne.mcell(2).or.mcell(1).ne.mcell(3)) goto 9020
      if (mxyz(1).ne.mxyz(2).or.mxyz(1).ne.mxyz(3)) goto 9030

c find out the msk grid dimensions
      lfast=m2(1)-m1(1)+1
      lmedm=m2(2)-m1(2)+1
      lslow=m2(3)-m1(3)+1
c
      if (lfast*lmedm.gt.maxsec)

     +  call ccperr(1,' ccpmskin - Map section > maxsec: recompile')
c now read the msk in the order it is on file
      do 200 islow=0,lslow-1
c get a section
       ierr=0
       call mgulpr(lmsks,lsec,ierr)
       if (ierr.ne.0) call ccperr(1,' ccpmskin - ccp4 read error')
c now sort into uvw map and store in centre of map
       ifms(3)=islow-lslow/2
       do 190 imedm=0,lmedm-1
       ifms(2)=imedm-lmedm/2
       do 190 ifast=0,lfast-1
       ifms(1)=ifast-lfast/2
        iu=ifms(juvw(1))
        iv=ifms(juvw(2))
        iw=ifms(juvw(3))
        fh(iu+hgrid*(iv+hgrid*iw)+dcent)=
     +    cmplx(lsec(ifast+lfast*imedm),0.0)
 190   continue
 200  continue
c close the map file
      call mrclos(lmsks)
c
c NOW GET WEIGHTS FROM VARIANCES
c extract standard dev from low order bits generated by 'map2ml.f'
c and generate ml weights
c
c extract map mean and variance
      do i=0,hgrid*hgrid*hgrid-1
       rhof=  rint(real(fh(i))*1000.0)/1000.0
       varf=(rfrac(real(fh(i))*1000.0))**2+mapstd**2
       fh(i)=cmplx(rhof,0.0)
       eh(i)=cmplx(varf,0.0)
      enddo
c get extreme values
      rhop=0.0
      varp=0.0
      r=0.0
      do iw=0,hgrid-1
      do iv=0,hgrid-1
      do iu=0,hgrid-1
       if ((2*iu-hgrid)**2+(2*iv-hgrid)**2+(2*iw-hgrid)**2 .gt.
     +   (lfast**2+lmedm**2+lslow**2)/2) then
        i=iu+hgrid*(iv+hgrid*iw)
        if (real(fh(i)).ne.0.0) then
         rhop=rhop+real(fh(i))
         varp=varp+real(eh(i))
         r=r+1.0          
        endif
       endif
      enddo
      enddo
      enddo
      rhop=rhop/r
      varp=varp/r
c       write (17,*)r,rhop,varp
c fill in any edge sites
      do i=0,hgrid*hgrid*hgrid-1
       if (real(fh(i)).eq.0.0) then
        fh(i)=cmplx(rhop,0.0)
        eh(i)=cmplx(varp,0.0)
       endif
      enddo
c now generate ml target and inverse variance weight
c rhof, varf are mean and variance for correctly oriented fragment
      mrmin=1.0e6
      mrmax=-1.0e6
      do i=0,hgrid*hgrid*hgrid-1
       rhof=real(fh(i))
       varf=real(eh(i))
       if (varf.lt.0.99*varp) then
        rhof=rhop+mapscl*(rhof-rhop)
        fh(i)=cmplx( rhof+(rhof-rhop)*varf/(varp-varf) ,0.0)
        eh(i)=cmplx( (varp-varf)/(2.0*varp*varf)       ,0.0)
        mrmin=min(mrmin,real(eh(i)))
        mrmax=max(mrmax,real(eh(i)))
       else
        eh(i)=(0.0,0.0)
       endif
      enddo
      write (lpt,405)rhop,sqrt(varp)
 405  format(/'  Unknown density mean ',f8.4,'   Std ',f8.4)
      write (lpt,410)mrmin,mrmax,mapstd
 410  format( '  Likelihood weights range ',2f8.4,'   Map std ',f8.4)
c
c
c save the shift we applied to the map
      rscale=mcell(1)/mxyz(1)
      tx0=rscale*real(-m1(juvw(1))-(m2(juvw(1))-m1(juvw(1))+1)/2)
      ty0=rscale*real(-m1(juvw(2))-(m2(juvw(2))-m1(juvw(2))+1)/2)
      tz0=rscale*real(-m1(juvw(3))-(m2(juvw(3))-m1(juvw(3))+1)/2)
      write(lpt,510)-tx0,-ty0,-tz0
 510  format(/,' SEARCH MODEL FROM INPUT MAP',
     +       /,'  Centre ',3f6.1)
c
      return
 9010 write (lpt,9011)hgrid
 9011 format('frgmapin - Fragment map is too big for current calc.'
     +  ,/,' Either increase the GRID in fffear, trim the map to'
     +  ,/,' remove unused sections, or calculate the fragment density'
     +  ,/,' on a coarser grid. Maximum allowed dimension is ',i4)
      call ccperr(1,'frgmapin - Error: fragment map grid>program grid')
 9020 call ccperr(1,'frgmapin - Error: fragment map cell not cubic')
 9030 call ccperr(1,'frgmapin - Error: fragment map grid not uniform')
      end
c
c ----------------------------------------------------------------------
c
      subroutine fftsf(fh,eh,work)
c calculate mask and map and generate structure factors by fft
      include 'fffear.fh'
      include 'hkldata.fh'
      include 'srcdata.fh'
      include 'crystal.fh'
      include 'params.fh'
      include 'io.fh'
c
      complex fh(0:*),eh(0:*),work(0:*)
c
      real lsec(maxsec)
      integer i,j,hgridbig
c
      integer fftfac
      external fftfac
c
c CALC FFT GRID
c
      hscale=(2.0*sqrt(1.2*tmax))/real(hgrid)
      hgridbig=nint(1.0/(rscale*hscale))
c shrink to find an efficient fft grid
      do i=hgridbig,1,-1
       j=fftfac(i,4)
       if (j.lt.hgridbig) goto 100
      enddo
 100  hgridbig=j
      hscale=1.0/(real(hgridbig)*rscale)
c
c SCALE FOR FFT
c
      senv=0.0
      sfrag=0.0
      sfrag2=0.0
c correct scale from e-/A3 to e-
      do i=0,hgrid*hgrid*hgrid-1
       senv  =senv  +real(eh(i))
       sfrag =sfrag +real(eh(i))*real(fh(i))
       sfrag2=sfrag2+real(eh(i))*real(fh(i))**2
c note: sfrag2 is (e x f^2)
       fh(i)=eh(i)*fh(i)*(rscale**3)
       eh(i)=eh(i)*(rscale**3)
      enddo
      senv  =senv  *(rscale**3)
      sfrag =sfrag *(rscale**3)
      sfrag2=sfrag2*(rscale**3)
c
c now fft
c
c flip alternate elements for centred structure factors
c (note if hcent is odd, the sign flips, but cancels later)
      call fftsignoffset(fh,hgrid)
      call fftsignoffset(eh,hgrid)
c fft, using huge intermediate arrays
      call hugesfcalc(fh,work,hgrid,hgridbig)
      call hugesfcalc(eh,work,hgrid,hgridbig)
      do i=0,hgrid*hgrid*hgrid-1
       fh(i)=conjg(fh(i))
       eh(i)=conjg(eh(i))
      enddo
c flip alternate elements for centred model
      call fftsignoffset(fh,hgrid)
      call fftsignoffset(eh,hgrid)
c 
      write (lpt,510)hgrid,hgridbig,rscale,hscale
      write (lpt,515)senv,sfrag,sfrag2
      write (lpt,520)fh(hgrid/2+hgrid*(hgrid/2+hgrid*(hgrid/2))),
     +               eh(hgrid/2+hgrid*(hgrid/2+hgrid*(hgrid/2)))
 510  format(/,' FFT SEARCH MODEL AND MASK',
     +       /,'  Model grid',i4,/,'  FFT grid  ',i4,
     +       /,'  Map grid scale',f7.4,/,'  SF grid scale ',f7.4)
 515  format('  Mask volume',f10.1,
     +       '   &lt;rho&gt;',f10.1,'   &lt;rho^2&gt;',f10.1)
 520  format('  FH(000)  (',f10.2,1x,f6.2,')',/,
     +       '  EH(000)  (',f10.2,1x,f6.2,')')
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine sfcalc(fh,eh,atoms,nused)
c calculate mask and map and generate structure factors by direct
c fourier summation
      include 'fffear.fh'
      include 'params.fh'
      include 'uvwdata.fh'
      include 'srcdata.fh'
      include 'hkldata.fh'
      include 'crystal.fh'
      include 'io.fh'
c
      real atoms(6,*)
      integer nused
c
      complex fh(0:*),eh(0:*)
c
      real u,v,w,b,occ,f,r,rr,atmrad,rcut,rhomap,rhomsk
      real xmin,xmax,ymin,ymax,zmin,zmax
      real x,y,z,gx,gy,gz,rh,rk,rl
      real sa,sb,sc
      integer i,j,ix,iy,iz,ih,ik,il,jh,jk,jl,hcent,dcent
      complex c
c
      integer lmod
      external lmod
c
c
      atmrad=mskrad
c
      hgrid=2*(int(sqrt(1.2*tmax)/hscale)+1)
      hcent=hgrid/2
      dcent=hcent+hgrid*(hcent+hgrid*(hcent))
c
      senv=0.0
      sfrag=0.0
      sfrag2=0.0
c
      call arryzro(fh,nu*nv*nw)
      call arryzro(eh,nu*nv*nw)
c
      rcut=atmrad**2
c
      xmin=0.0
      xmax=0.0
      ymin=0.0
      ymax=0.0
      zmin=0.0
      zmax=0.0
c
      do i=1,nused
       x=atoms(1,i)
       y=atoms(2,i)
       z=atoms(3,i)
       if (x-atmrad.lt.xmin) xmin=x-atmrad
       if (y-atmrad.lt.ymin) ymin=y-atmrad
       if (z-atmrad.lt.zmin) zmin=z-atmrad
       if (x+atmrad.gt.xmax) xmax=x+atmrad
       if (y+atmrad.gt.ymax) ymax=y+atmrad
       if (z+atmrad.gt.zmax) zmax=z+atmrad
      enddo
c
c Scattering factor scaling: if needed, pre-figure into a,b,c
c        sa=(12.56637/(ast(j)+b))**1.5
c        sb=(12.56637/(bst(j)+b))**1.5
c        sc=(12.56637/(b))**1.5
c
c Now actually do the FT
c
      do iz=int(zmin/rscale),int(zmax/rscale)+1
      gz=real(iz)*rscale
      do iy=int(ymin/rscale),int(ymax/rscale)+1
      gy=real(iy)*rscale
      do ix=int(xmin/rscale),int(xmax/rscale)+1
      gx=real(ix)*rscale
       rhomap=0.0
       rhomsk=0.0
       do i=1,nused
c add contributions from all atoms
        x=atoms(1,i)
        y=atoms(2,i)
        z=atoms(3,i)
        b=atoms(4,i)
        occ=atoms(5,i)
        j=nint(atoms(6,i))
        r=(x-gx)**2+(y-gy)**2+(z-gz)**2
        if (r.lt.rcut) then
         rr=39.47842*r
         f=fa1(j)*exp(-rr/(fb1(j)+b))+fa2(j)*exp(-rr/(fb2(j)+b))
         rhomap=rhomap+occ*f
         rhomsk=1.0
        endif
       enddo
c correct scale to from e-/A-3 e-
       senv=senv+rhomsk*(rscale**3)
       sfrag=sfrag+rhomap*(rscale**3)
       sfrag2=sfrag2+rhomap**2*(rscale**3)
       rhomsk=rhomsk*(rscale**3)
       rhomap=rhomap*(rscale**3)
c Now we have the density. Calculate its contribution to every reflection
c in reciprocal space hemisphere.
       if (rhomsk.gt.0.0) then
        do il=     0,hcent
        jl=il+hcent
        rl=hscale*il
        do ik=-hcent,hcent
        jk=ik+hgrid*il
        rk=hscale*ik
        do ih=-hcent,hcent
        jh=ih+hgrid*jk+dcent
        rh=hscale*ih
         if (rh*rh+rk*rk+rl*rl.lt.1.2*tmax) then
          r=2*pi*(rh*gx+rk*gy+rl*gz)
          c=cmplx(cos(r),sin(r))
          fh(jh)=fh(jh)+rhomap*c
          eh(jh)=eh(jh)+rhomsk*c
         endif
        enddo
        enddo
        enddo
       endif
c
      enddo
      enddo
      enddo
c fill in other hemisphere
      do il=     0,hcent
      jl=il+hcent
      do ik=-hcent,hcent
      jk=ik+hgrid*il
      do ih=-hcent,hcent
      jh=ih+hgrid*jk
       fh(dcent-jh)=conjg(fh(dcent+jh))
       eh(dcent-jh)=conjg(eh(dcent+jh))
      enddo
      enddo
      enddo
c
      write (lpt,510)hgrid,rscale,hscale
      write (lpt,515)senv,sfrag,sfrag2
      write (lpt,520)fh(hgrid/2+hgrid*(hgrid/2+hgrid*(hgrid/2))),
     +               eh(hgrid/2+hgrid*(hgrid/2+hgrid*(hgrid/2)))
 510  format(/,' FFT SEARCH MODEL AND MASK',
     +       /,'  Model grid',i4,
     +       /,'  Map grid scale',f7.4,/,'  SF grid scale ',f7.4)
 515  format('  Mask volume',f10.1,
     +       '   &lt;rho&gt;',f10.1,'   &lt;rho^2&gt;',f10.1)
 520  format('  FH(000)  (',f10.2,1x,f6.2,')',/,
     +       '  EH(000)  (',f10.2,1x,f6.2,')')
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine setshapefn()
c set form factors
      include 'fffear.fh'
      include 'srcdata.fh'
c
c---- atomic scattering factors for above atom types
c     f = fa1 * exp(-fb1 * rho) + fa2 * exp(-fb2 * rho)
c
      data fa1/0.79, 1.00, 1.50, 2.00, 2.50, 3.00, 2.99, 2.45, 2.97,
     +   3.30, 3.63, 3.96, 4.29, 4.62, 4.95, 5.55, 5.61, 5.94, 6.27,
     +   6.60, 6.93, 7.26, 7.59, 7.92, 8.25, 8.58, 8.91, 9.24, 9.57,
     +   9.90,10.23,10.56,10.89,11.22,11.55,11.88,12.21,12.54,12.87,
     +  13.20,13.53,13.86,14.19,14.52,14.85,15.18,15.51,15.84,16.17,
     +  16.50,16.83,17.16,17.49,17.82,18.15,18.48,18.81,19.14,19.47,
     +  19.80,20.13,20.46,20.79,21.12,21.45,21.78,22.11,22.44,22.77,
     +  23.10,23.43,23.76,24.09,24.42,24.75,25.08,25.41,25.74,26.07,
     +  26.40,26.73,27.06,27.39,27.72,28.05,28.38,28.71,29.04,29.37,
     +  29.70,30.03,30.36,30.69,31.02,31.35,31.68,32.01,1,1,1/
      data fa2/0.19, 1.00, 1.50, 2.00, 2.50, 2.98, 4.00, 5.56, 6.03,
     +   6.70, 7.37, 8.04, 8.71, 9.38,10.05,10.42,11.39,12.06,12.73,
     +  13.40,14.07,14.74,15.41,16.08,16.75,17.42,18.09,18.76,19.43,
     +  20.10,20.77,21.44,22.11,22.78,23.45,24.12,24.79,25.46,26.13,
     +  26.80,27.47,28.14,28.81,29.48,30.15,30.82,31.49,32.16,32.83,
     +  33.50,34.17,34.84,35.51,36.18,36.85,37.52,38.19,38.86,39.53,
     +  40.20,40.87,41.54,42.21,42.88,43.55,44.22,44.89,45.56,46.23,
     +  46.90,47.57,48.24,48.91,49.58,50.25,50.92,51.59,52.26,52.93,
     +  53.60,54.27,54.94,55.61,56.28,56.95,57.62,58.29,58.96,59.63,
     +  60.30,60.97,61.64,62.31,62.98,63.65,64.32,64.99,1,1,1/
      data fb1/24.22,25.00,25.00,25.00,25.00,30.02,25.38,24.76,30.00,
     +  30.00,30.00,30.00,30.00,30.00,30.00,33.71,30.00,30.00,30.00,
     +  30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,
     +  30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,
     +  30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,
     +  30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,
     +  30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,
     +  30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,
     +  30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,
     +  30.00,30.00,30.00,30.00,30.00,30.00,30.00,30.00,1,1,1/
      data fb2/2.11, 3.00, 3.00, 3.00, 3.00, 2.89, 3.50, 4.14, 3.00,
     +   3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 1.90, 3.00, 3.00, 3.00,
     +   3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00,
     +   3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00,
     +   3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00,
     +   3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00,
     +   3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00,
     +   3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00,
     +   3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00,
     +   3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00, 3.00,1,1,1/
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine resshapefn(hkls,lf)
c truncate atomic shape functions to some resolution
      include 'fffear.fh'
      include 'crystal.fh'
      include 'hkldata.fh'
      include 'srcdata.fh'
      include 'params.fh'
      include 'io.fh'
c
      integer lf
      real hkls(nhklc,0:*)
c
      integer hshells,xshells
      real dh,dx
      parameter (hshells=50,xshells=50,dh=0.02,dx=0.1)
c
      integer i,j,ih,ix,lmod
      real sf(0:hshells),sh(0:hshells),ss(0:hshells),srho(0:xshells)
      real x,y,w,sw,sx,sx2,sdy,sdyx,r,s,eps,f
      real a,b
      logical hemi
      external phase,hemi,lmod
c
c do we want res truncated scattering factors?
      if (mdlres.gt.0.0) then
c first deal with resolution truncation case...
c prepare for resolution truncation
       do ih=0,hshells
        sf(ih)=0.0
        ss(ih)=0.0
       enddo
c loop through reflections and get fom curve
       do i=1,nrflo
        s=sqrt(hkls(4,i))
        eps=abs(hkls(5,i))
        f=hkls(lf,i)
        ih=int(s/dh)
        sf(ih)=sf(ih)+f/sqrt(eps)
        ss(ih)=ss(ih)+1.0
       enddo
       do ih=0,hshells
        sf(ih)=sf(ih)/max(ss(ih),1.0)
       enddo
       sf(0)=1.0
c
c loop through atomic types
c
       do j=1,50
c multiply by scattering factor
        do ih=0,hshells
         s=(dh*(real(ih)+0.5))
         s=s**2/4.0
         b=mdlbfc
c?? fom weighting
c        sh(ih)=sf(ih)*exp(-b*s)*
c     +    (alt(j)*exp(-ast(j)*s)+blt(j)*exp(-bst(j)*s)+clt(j))
c?? data res weighting
c        sh(ih)=0.0
c        if (sf(ih).gt.0.025) sh(ih)=exp(-b*s)*
c     +    (alt(j)*exp(-ast(j)*s)+blt(j)*exp(-bst(j)*s)+clt(j))
c resolution filter
         sh(ih)=0.0
         if ((dh*ih)**2.lt.mdlres) sh(ih)=exp(-b*s)*
     +     (fa1(j)*exp(-fb1(j)*s)+fa2(j)*exp(-fb2(j)*s))
        enddo
c fourier transform:
        do ix=0,xshells
         srho(ix)=0.0
         x=dx*(real(ix)+0.5)
         do ih=0,hshells
          s=dh*(real(ih)+0.5)
          srho(ix)=srho(ix)+2.0*dh*s*sh(ih)*sin(2.0*pi*s*x)/x
         enddo
        enddo
c Now we'll fit a spherical gaussian
        do i=1,xshells
         if (srho(i).lt.srho(0)/4.0) goto 200
        enddo
 200    sw=0.0
        sx=0.0
        sx2=0.0
        sdy=0.0
        sdyx=0.0
        do ix=0,i-1
         x=39.47842*(dx*(real(ix)+0.5))**2
         y=log(srho(ix))
         w=1.0
         sw=sw+w
         sx=sx+w*x
         sx2=sx2+w*x**2
         sdy=sdy+w*y
         sdyx=sdyx+w*x*y
        enddo
        a=exp(-(sx2*sdy-sx*sdyx)/(sx**2-sw*sx2))
        b=1.0/((sw*sdyx-sx*sdy)/(sx**2-sw*sx2))
        fa1(j)=a/(12.56637/b)**1.5
        fb1(j)=b
        fa2(j)=0.0
        fb2(j)=1.0
c next atomic type
       enddo
       write (lpt,510)1.0/sqrt(mdlres)
 510   format(' Truncating formfactors to ',f4.1,'A.   f=a1 exp(b1.s)')
c done res truncation
      else
c otherwise deal with fixed resolution case
       do j=1,50
        fb1(j)=fb1(j)+mdlbfc
        fb2(j)=fb2(j)+mdlbfc
       enddo
       write (lpt,515)
 515   format(' Using natural formfactors f=a1 exp(b1.s)+a2 exp(b2.s)')
      endif
c
       write (lpt,525)'H ',1,fa1(1),fb1(1),fa2(1),fb2(1)
       write (lpt,525)'C ',6,fa1(6),fb1(6),fa2(6),fb2(6)
       write (lpt,525)'N ',7,fa1(7),fb1(7),fa2(7),fb2(7)
       write (lpt,525)'O ',8,fa1(8),fb1(8),fa2(8),fb2(8)
       write (lpt,525)'S ',16,fa1(16),fb1(16),fa2(16),fb2(16)
       write (lpt,525)'Zn',30,fa1(30),fb1(30),fa2(30),fb2(30)
 525   format(2x,a2,' Z=',i2,
     +   '   a1=',f7.3,' b1=',f7.3,'   a2=',f7.3,' b2=',f7.3)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine fftsignoffset(x,n)
c apply checkerboard sign change to x to recentre its Fourier transform
      include 'fffear.fh'
      integer n
      complex x(0:n-1,0:n-1,0:n-1)
c
      integer ix,iy,iz
      real f
c
      do iz=0,n-1
      do iy=0,n-1
      do ix=0,n-1
       f=1.0-2.0*real(mod(ix+iy+iz,2))
       x(ix,iy,iz)=f*x(ix,iy,iz)
      enddo
      enddo
      enddo
      return
      end
c
c ----------------------------------------------------------------------
c
c    The huge-section inverse FFT, used to calculate finely spaced
c    low resolution stucture factors from a finely sampled low-radius
c    model.
c
c ----------------------------------------------------------------------
c
      subroutine hugesfcalc(x,y,nsml,nbig)
c complex FFT a small grid using huge sections.
c i.e. transform a finely sampled but small extent function to get
c another finely sampled but small extent function.
      include 'fffear.fh'
      integer nsml,nbig
      complex x(0:nsml-1,0:nsml-1,0:nsml-1),y(0:nbig-1,0:nsml-1)
c
      integer iu,iv,iw,csml,cbig
      complex c
c
      csml=nsml/2
      cbig=nbig/2
c
c transform along x
c
      do iw=-csml,csml-1
       call arryzro(y,2*nbig*nsml)
c get x-y section
       do iv=-csml,csml-1
        do iu=-csml,csml-1
         y(iu+cbig,iv+csml)=x(iu+csml,iv+csml,iw+csml)
        enddo
       enddo
c transform along x
       call cplxft2(y,nbig,nsml)
c store x-y section
       do iv=-csml,csml-1
        do iu=-csml,csml-1
         x(iu+csml,iv+csml,iw+csml)=y(iu+cbig,iv+csml)
        enddo
       enddo
      enddo
c
c transform along y
c
      do iw=-csml,csml-1
       call arryzro(y,2*nbig*nsml)
c get x-y section
       do iv=-csml,csml-1
        do iu=-csml,csml-1
         y(iv+cbig,iu+csml)=x(iu+csml,iv+csml,iw+csml)
        enddo
       enddo
c transform along y
       call cplxft2(y,nbig,nsml)
c store x-y section
       do iv=-csml,csml-1
        do iu=-csml,csml-1
         x(iu+csml,iv+csml,iw+csml)=y(iv+cbig,iu+csml)
        enddo
       enddo
      enddo
c
c transform along z
c
      do iv=-csml,csml-1
       call arryzro(y,2*nbig*nsml)
c get x-y section
       do iw=-csml,csml-1
        do iu=-csml,csml-1
         y(iw+cbig,iu+csml)=x(iu+csml,iv+csml,iw+csml)
        enddo
       enddo
c transform along y
       call cplxft2(y,nbig,nsml)
c store x-y section
       do iw=-csml,csml-1
        do iu=-csml,csml-1
         x(iu+csml,iv+csml,iw+csml)=y(iw+cbig,iu+csml)
        enddo
       enddo
      enddo
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine cplxft2(y,n,m)
c perform m complex transforms of length n
      include 'fffear.fh'
      integer n,m
      real y(0:*)
c
      integer d(5)
c
      d(1)=2*n*m
      d(2)=2
      d(3)=2*n*m
      d(4)=2*n*m
      d(5)=2*n
      call cmplft(y(0),y(1),n,d)
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine gridreduce(l,s,nu,nv,nw,nus,nvs,nws)
c copy structure factors from a larger grid to a smaller
      include 'fffear.fh'
      integer nu,nv,nw,nus,nvs,nws
      real l(0:nu-1,0:nv-1,0:nw-1),s(0:nus-1,0:nvs-1,0:nws-1)
c
      integer ih,ik,il,lmod
      external lmod
c
      do il=0,nws-1
      do ik=-nvs/2,nvs/2-1
      do ih=-nus/2,nus/2-1
      s(lmod(ih,nus),lmod(ik,nvs),il)=l(lmod(ih,nu),lmod(ik,nv),il)
      enddo
      enddo
      enddo
c
      return
      end
c
c ----------------------------------------------------------------------
c
c    The grid doubling FFT, used to calculate a fine map
c    quickly from lower resolution structure factors, allowing
c    a simple peak search.
c
c ----------------------------------------------------------------------
c
      subroutine fft3df(rhof,fu,fv,fw)
c interpolating fft: calc spline coeffs on coarse grid, fft, interpolate.
      include 'fffear.fh'
      include 'crystal.fh'
      include 'hkllim.fh'
      integer fu,fv,fw
      real rhof(0:fu-1,0:fv-1,0:fw-1)
c
      real work(0:999999),s,x1,x2,x3,wt
      integer iu,iv,iw,ju,jv,jw,ih,ik,il,lmod
      integer cu,cv,cw
      external lmod
c
      cu=fu/2
      cv=fv/2
      cw=fw/2
c
c      call ccptim(0,x1,x2)
      call arryzro(work,cu*cv*cw)
c      call ccptim(1,x1,x2)
c      write (*,*)'zro',x1,x2
c copy rfls to coarse grid
      s=1.0/vol
      do 100 il=0,lmaxt
      iw=2*il
      jw=iw
      x3=max(pi*abs(il)/real(cw),0.001)
      x3=sin(x3)/x3
      do 100 ik=kmint,kmaxt
      iv=lmod(ik,cv)
      jv=lmod(ik,fv)
      x2=max(pi*abs(ik)/real(cv),0.001)
      x2=x3*sin(x2)/x2
      do 100 ih=hmint,hmaxt
      iu=lmod(ih,cu)
      ju=lmod(ih,fu)
      x1=max(pi*abs(ih)/real(cu),0.001)
      x1=x2*sin(x1)/x1
      wt=x1**3
      work(iu+cu*(iv+cv*(iw  )))=s*rhof(ju,jv,jw  )/wt
      work(iu+cu*(iv+cv*(iw+1)))=s*rhof(ju,jv,jw+1)/wt
 100  continue
c      call ccptim(1,x1,x2)
c      write (*,*)'shp',x1,x2
c fft
      call fft3hr(work,cu,cv,cw)
c      call ccptim(1,x1,x2)
c      write (*,*)'fft',x1,x2
c interpolate to fine grid
      call fine2(work,rhof,cu,cv,cw)
c      call ccptim(1,x1,x2)
c      write (*,*)'int',x1,x2,' ---'
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine fine2(rhoc,rhof,cu,cv,cw)
c 3-d grid doubler
      include 'fffear.fh'
      integer cu,cv,cw
      real rhoc(0:*)
      real rhof(0:*)
c
      integer iu,iv,iw,ju,jv,jw,ku,kv,kw,fu,fv,fw,fuv
c
      fu=cu*2
      fv=cv*2
      fw=cw*2
      fuv=fu*fv
c
      do iw=0,fw-1,2
       jw=fv*iw
       kw=cv*iw/2
       do iv=0,fv-1,2
        jv=fu*(iv+jw)
        kv=cu*(iv/2+kw)
        do iu=0,fu-1,2
         ju=iu+jv
         ku=iu/2+kv
         rhof(ju)=rhoc(ku)/512.0
        enddo
        call fineb2f(rhof(jv),fu,1)
       enddo
      enddo
      do iw=0,fw-1,2
       jw=fuv*iw
       do iu=0,fu-1
        ju=iu+jw
        call fineb2f(rhof(ju),fv,fu)
       enddo
      enddo
      do iv=0,fv-1
       jv=fu*iv
       do iu=0,fu-1
        ju=iu+jv
        call fineb2f(rhof(ju),fw,fuv)
       enddo
      enddo
c
      return
      end
c
c ----------------------------------------------------------------------
c
      subroutine fineb2f(fx,nx,df)
c High speed cyclic doubler. First coarse point=First fine point.
c fx=function
c nx=length of fx
c df=spacing between adjacent elements of fx
      include 'fffear.fh'
      real fx(0:*)
      integer nx,df
c
      real l1,l2,l3,l4
      integer ix,ixlim,df2
c
      df2=2*df
      l2=fx((nx-2)*df)
      l3=fx(0)
      l4=l3
      ix=0
      ixlim=nx*df-df2
 100   l1=l2
       l2=l3
       l3=fx(ix+df2)
       fx(ix)=l1+6*l2+l3
       ix=ix+df
       fx(ix)=4*(l2+l3)
       ix=ix+df
      if (ix.lt.ixlim) goto 100
      fx(ix)=l2+6*l3+l4
      fx(ix+df)=4*(l3+l4)
c
      return
      end
c
c ----------------------------------------------------------------------
c
c Miscellaneous routines.
c
c ----------------------------------------------------------------------
c
      subroutine arryzro(x,n)
      include 'fffear.fh'
      integer i,n
      real x(n)
      do 100 i=1,n
       x(i)=0.0
 100  continue
      return
      end
c
c ----------------------------------------------------------------------
c
