C
C     GENSYM
C     Copyright (C) 1990 Phil Evans
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C*************************************************************
C
C
C**************************************************************
C
C-------------------------------------------------------------------------
C
C   GENSYM
C
C                    Phil Evans
C                    MRC LMB Cambridge
C                    August 1990
C
C Generate all symmetry-related sites from a list of input atoms, and
C produce a list of all atoms which fall within the defined volume
C
C-------------------------------------------------------------------------
C
C  Input:
C
C    Control input (DATA)
C
C TITLE         title for the run, written to the output file
C
C SYMMETRY  space-group name OR number OR symmetry operation
C       Space-group symmetry for the atoms (NOT the Patterson)
C       This may be given in 3 ways:-
C       (a) space-group name
C       (b) space-group number
C       (c) symmetry operations as in International Tables, separated
C               by '*', on a series of SYMMETRY lines if necessary
C       For options (a)& (b), symmetry operations are read from the
C       library file SYMOP
C
C NCODE  orthogonalization code
C       Vectors are written out to a PDB file in an orthogonal frame
C       defined by this code. The default is NCODE = 1, which is the
C       usual frame for Brookhaven files, so normally this command
C       can be omitted
C
C XYZLIM  Xmin Xmax Ymin Ymax Zmin Zmax 
C       Limits of volume (fractional or grid coordinates)
C
C SPHERE  [ORTHOGONAL] Xcen Ycen Zcen Radius
C       Define spherical volume (alternative to XYZLIM): centre is
C       in fractional coordinates unless keyword ORTHOGONAL is
C       present. Radius in A
C
C CELL   a b c alpha beta gamma
C       Cell dimensions for orthogonalization (for Brookhaven file)
C
C GRID  NX NY NZ
C       Sampling intervals along x,y,z, for grid coordinates
C
C BROOKHAVEN
C       Write Brookhaven file
C
C RESIDUE  residue_name
C       Input a residue name for following atoms, until changed,
C       for labelling output
C
C ATOM  atomname   x  y  z
C       Input an atom. "atomname" is a
C       1-character unique identifier for this site, x,y,z are the
C       FRACTIONAL coordinates
C
C NOPRINT
C 	Don't print coordinates on output (default do)
C
C READ
C       Read coordinates from PDB file (orthogonal A), instead of
C       from ATOM commands. The file is read from XYZIN.
C
C SORT CHAINS | SYMMETRY
C       Sorting of output file: SYMMETRY gives old output (default)
C       whereby atoms are grouped together with symmetry mates.
C       This destroys atom sequence. CHAINS preserves the atom
C       ordering.
C
C CHAIN SYMMETRY Nsym [old_chain] new_chain
C       Based on CHAIN keyword from PDBSET
C       Set chain id of symmetry-generated atoms from sym. op.
C       Nsym to new_chain (If old_chain also given then this
C       only applied to atoms which originally had chain id of
C       old_chain).
C
C END
C       End of input, also end-of-file will do
C
C-------------------------------------------------------------------------
C
C
C
C-------------------------------------------------------------------------
C
      IMPLICIT NONE
C
C Atoms
      INTEGER MAXATM
      PARAMETER (MAXATM=5000)
      CHARACTER*1 ATNAM(MAXATM),ATMNAM(MAXATM)
      CHARACTER*4 SITNAM(MAXATM),RSANAM(MAXATM),RSTNAM(MAXATM)
      REAL XS(3,MAXATM),XSS(3,MAXATM),XYZ(3)
      REAL BIS(MAXATM),BISS(MAXATM)
      INTEGER NATOMS, ISYM(MAXATM), NSITE
C                                                     
C Orthogonalization (common block fro Brookhaven library routine)
C
C        RO    ORTHOGONALISING MATRIX (ONLY SET IF 'CRYST1' OR 'SCALE'
C              CARDS PRESENT - CHECK 'MATRIX' FLAG)
C        RF    FRACTIONALISING MATRIX (ONLY SET IF 'CRYST1' OR 'SCALE'
C              CARDS PRESENT - CHECK 'MATRIX' FLAG)
C     NCODE    FLAG INDICATING SETTING FOUND, 0 IF NOT ONE THAT WAS
C              RECOGNISED
C
      INTEGER NCODE
      REAL RO(4,4),RF(4,4)
      REAL RCEL(6), RVOL
C
C Symmetry
C  Space-group symmetry for atoms
      INTEGER NATSYM,LATSGP,NSYMP
      REAL    ATMROT(4,4,96)
      CHARACTER*12 NAMSGP,PGNAME
      CHARACTER*80 SYMCHS(96)
C
C Things for PARSER ----
      INTEGER MAXTOK
      PARAMETER (MAXTOK=100)
      CHARACTER KEY*4,CVALUE(MAXTOK)*4,LINE*80
      INTEGER IBEG(MAXTOK),IEND(MAXTOK),ITYP(MAXTOK),IDEC(MAXTOK),NTOK
      REAL FVALUE(MAXTOK)
      LOGICAL LEND
C
C Main input keywords
      INTEGER NKEYS
      PARAMETER (NKEYS=15)
      CHARACTER*8 KEYS(NKEYS)
C
      INTEGER MXYZ(3)
      REAL CELL(6)
C
C Chains etc
      INTEGER MAXCHNS
      PARAMETER (MAXCHNS=30)
      INTEGER ISYMCH,IATOM,ISYMOP,II,SYMOP(MAXCHNS)
      CHARACTER*1 ORIGCHN(MAXATM),CHAIN(MAXATM),OLDCH(MAXCHNS),
     $     NEWCH(MAXCHNS)
C General
      INTRINSIC NINT
      INTEGER I,IXYZIN,IXYZOUT,J,K,L,IX,IY,IZ,IFLAG,IRES,NCT(3),
     $     IFAIL,ISER,JFLMIN(3),JFLMAX(3),SORTMODE,I1,I2,NLIMIT1,
     $     NLIMIT2
      REAL Y(3),Z(3),Q,B(6),FLMMAP(2,3),FBOX(3),GRIDPT,XX,YY,ZZ,BISO
      REAL XYZCNF(3),XYZCNO(3),RADIUS,R, UNITCL(2,3),VOL
      LOGICAL CELMUL,PRINT,BROOKF,SPHERE,SPORTH,LCELL
      CHARACTER*1 CHNNAM,INSCOD,ALTCOD,SITE
      CHARACTER*4 RESNAM,ATNAME,RESNO,ID,SEGID
      CHARACTER*72 VTITLE
      CHARACTER*80 INFILE, ERRLIN
C     ..
C     .. External Functions ..
      INTEGER JMINT
      LOGICAL LINBOX
      EXTERNAL JMINT, LINBOX 
C     ..
C     .. External Routines ..
      EXTERNAL CCPERR, CCPFYP, CCPMVI, CCPRCS, CCPUPC, CELTRN, CHKKEY, 
     +         CVFRAC2, GTPINT, GTPREA, MATVC4, PARSER, RBFRAC2, 
     +         RBRCEL, RBRORF, RDSYMM, XYZADVANCE, XYZATOM, 
     +         XYZCLOSE, XYZCOORD, XYZINIT, XYZOPEN, 
     +         WBCELL, WREMARK, SYMTR4
C     ..
C     .. Data Statements ..
      DATA UNITCL/0.0,1.0,0.0,1.0,0.0,1.0/
      DATA KEYS/'TITLE','SYMMETRY','XYZLIM','NCODE','ATOM',
     .   'CELL','GRID','BROOKHAV','RESIDUE','SPHERE','NOPRINT',
     .   'READ','SORT','CHAI',
     .   'END'/
C
C********************************************************************
C
      CALL CCPFYP
C
      CALL CCPRCS(6,'GENSYM', '$Date: 2005/09/06 11:25:14 $')
      CALL XYZINIT
C
C Initializations
      ISYMCH = 0
      NATSYM = 0
      NCODE  = 1
      PRINT = .TRUE.
      RESNAM = 'ATM '
      LCELL = .FALSE.
      SPHERE = .FALSE.
      INFILE = ' '
      VTITLE = 'From GENSYM.'
      NAMSGP = ' '
      SORTMODE = 1
C No Brookhaven
      BROOKF = .FALSE.
      DO 2 I=1,3
        MXYZ(I) = 100
        CELL(I) = 1.0
        CELL(I+3) = 90.0
        FLMMAP(1,I) = 0.0
        FLMMAP(2,I) = 1.0
2     CONTINUE
      DO 4 I=1,4
        DO 4 J=1,4
          RO(J,I) = 0.0
          RF(J,I) = 0.0
4     CONTINUE
      NATOMS = 0
C
C
10    NTOK=MAXTOK
      LINE=' '
      CALL PARSER(
     .  KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,.TRUE.)
C
      IF(LEND) GO TO 1000
C Ignore blank line
      IF(NTOK.EQ.0) GO TO 10
C
C Make uppercase
      CALL CCPUPC(KEY)
      CALL CHKKEY(KEY,KEYS,NKEYS,K)
      IF(K.EQ.0) THEN
        CALL CCPERR(2,' ** Unrecognized keyword **')
        GO TO 10
      ELSE IF(K.LT.0) THEN
        CALL CCPERR(2,' ** Ambiguous keyword **')
        GO TO 10
      ENDIF
C
C
19    GO TO
     . (20,30,35,40,50,60,70,80,90,100,110,120,130,140,1000),K
C---------------------------------------------------------------------
C
C TITLE
20    VTITLE = LINE(IBEG(2):)
      GO TO 10
C---------------------------------------------------------------------
C                    
C SYMMETRY read atom symmetry (all operations)
30    CONTINUE
      J=2
      CALL RDSYMM(J,LINE,IBEG,IEND,ITYP,FVALUE,NTOK,
     .    NAMSGP,LATSGP,PGNAME,NATSYM,NSYMP,ATMROT)
      GO TO 10
C
C---------------------------------------------------------------------
C  XYZLIM  read fractional or grid limits
35    K=1
      DO 36, J=1,3
      DO 36, I=1,2
         K=K+1
         CALL GTPREA(K,FLMMAP(I,J),NTOK,ITYP,FVALUE)
36    CONTINUE
      GO TO 10
C
C---------------------------------------------------------------------
C
C  NCODE read orthogonalization code
C
40    CALL GTPINT(2,NCODE,NTOK,ITYP,FVALUE)
      GO TO 10
C---------------------------------------------------------------------
C
C ATOM  read atom name (1-character) x,y,z
C
50    CONTINUE
      IF (INFILE .NE. ' ') THEN
         CALL CCPERR(2,' *** ATOM command invalid with READ ***')
         CALL CCPERR(1,'*** Invalid input ***')
      ENDIF
      IF (NTOK .LT. 5) THEN
            CALL CCPERR(2,' *** Too few fields for ATOM ***')
            GO TO 10
      ENDIF
C
      IF (NATOMS .GE. MAXATM) THEN
        WRITE (ERRLIN,FMT='(A,I8,A)')
     +     ' *** More than ',MAXATM,' atoms given. Line ignored. ***'
        CALL CCPERR(2,ERRLIN)
        GO TO 1000
      ENDIF
C
      NATOMS = NATOMS + 1
      ATMNAM(NATOMS) = CVALUE(2)(1:1)
      ISYM(NATOMS) = 1
      DO 51, I=1,3
        CALL GTPREA(I+2,XS(I,NATOMS),NTOK,ITYP,FVALUE)
51    CONTINUE
      RSANAM(NATOMS) = RESNAM
      BIS(NATOMS) = 1.0
      GO TO 10
C---------------------------------------------------------------------
C
C CELL  read cell dimensions
60    DO 61, I=1,6
      CALL GTPREA(I+1, CELL(I), NTOK,ITYP,FVALUE)
61    CONTINUE
      LCELL = .TRUE.
      GO TO 10
C---------------------------------------------------------------------
C GRID read grid sampling
70    DO 71, I=1,3
      CALL GTPINT(I+1, MXYZ(I), NTOK,ITYP,FVALUE)
71    CONTINUE
      GO TO 10
C---------------------------------------------------------------------
C BROOKHAVen, set flag to write Brookhaven file
80    BROOKF = .TRUE.
      GO TO 10
C---------------------------------------------------------------------
C RESIDUE read residue name
90    RESNAM = CVALUE(2)
      GO TO 10
C---------------------------------------------------------------------
C SPHERE [ORTHOGONAL] Xcen Ycen Zcen Radius
C   Check for keyword
100   IF (CVALUE(2) .EQ. 'ORTH') THEN
        J = 2
        SPORTH = .TRUE.
      ELSE
        J = 1
        SPORTH = .FALSE.
      ENDIF
      DO 101, I=1,3
        CALL GTPREA(I+J,XYZCNF(I),NTOK,ITYP,FVALUE)
101   CONTINUE
      CALL GTPREA(J+4,RADIUS,NTOK,ITYP,FVALUE)
      SPHERE = .TRUE.
      GO TO 10
C---------------------------------------------------------------------
C NOPRINT suppress printing of all atoms
110   PRINT = .FALSE.
      GO TO 10
C---------------------------------------------------------------------
C READ set up to read from input file (XYZIN)
 120  IF (NATOMS .GT. 0) THEN
         CALL CCPERR(2,
     $        ' **** READ command not valid with ATOM command ****')
         CALL CCPERR(1,'*** Invalid input ***')
      ENDIF
      INFILE = 'XYZIN'
      GO TO 10
C---------------------------------------------------------------------
C SORT sets how to sort atoms in output file
 130  CALL CCPUPC(CVALUE(2))
      IF (CVALUE(2) .EQ. 'SYMM') THEN
        SORTMODE = 1
      ELSE IF (CVALUE(2) .EQ. 'CHAI') THEN
        SORTMODE = 2
      ELSE
        CALL CCPERR(1,
     $       ' **** Unrecognised subkeyword for SORT command ****')
      ENDIF
      GO TO 10
C---------------------------------------------------------------------
C CHAIN SYMMETRY sets chain names in output file
 140  CALL CCPUPC(CVALUE(2))
C
      IF (CVALUE(2) .NE. 'SYMM') CALL CCPERR(1,
     $       ' **** CHAIN keyword must be followed by SYMMETRY ****')
C
      IF (ITYP(3).NE.2) THEN
        CALL CCPERR(1,
     $       ' **** CHAIN SYMMETRY must be followed by number ****')
      ELSE
        ISYMCH = ISYMCH + 1
        IF (ISYMCH.GT.MAXCHNS) CALL CCPERR(1,
     $  ' **** Maximum number of chains exceeded - increase MAXCHS ***')
        SYMOP(ISYMCH) = FVALUE(3)
      END IF
C
      IF (NTOK.EQ.5) THEN
        OLDCH(ISYMCH) = LINE(IBEG(4):IBEG(4))
      ELSE
        OLDCH(ISYMCH) = ' '
      END IF
C
      NEWCH(ISYMCH) = LINE(IBEG(NTOK):IBEG(NTOK))
      IF (NTOK.GT.5) CALL CCPERR(1,
     $       ' **** Too many arguments for CHAIN keyword ****')
C
      IF (OLDCH(ISYMCH).EQ.' ') THEN
        WRITE(6,'(A,I2.2,A,A1)')
     $       ' All positions generated by sym. op. ',ISYMCH,
     $       ' will be assigned chain id ',NEWCH(ISYMCH)
      ELSE
        WRITE(6,'(A,A1,A,I2.2,A,/,A,A1)')
     $       ' All positions generated from chain ',OLDCH(ISYMCH),
     $       ' by sym. op. ',ISYMCH,' will be assigned',
     $       ' chain id ',NEWCH(ISYMCH)
      END IF

      GO TO 10
C---------------------------------------------------------------------
C
C GO, calculate sites
C
1000  CONTINUE
C
C---- First, print some info
C
      IF (SORTMODE.EQ.1) THEN
        WRITE(6,'(X,A,/,X,A)')
     $  'Symmetry-generated atoms with the same source atom',
     $  'will be grouped together'
      ELSE IF (SORTMODE.EQ.2) THEN
        WRITE(6,'(X,A,/,X,A)')
     $  'Symmetry-generated atoms from the same symmetry operation',
     $  'will be grouped together'
      END IF
C
C---- Check necessary keywords

      IF (NATSYM.EQ.0)
     +   CALL CCPERR(1,'SYMMETRY keyword required.')

      IF (INFILE .NE. ' ') THEN
C  Read file in PDB format
         IFAIL = 0
         IXYZIN = 0
         CALL XYZOPEN(INFILE,'INPUT',' ',IXYZIN,IFAIL)
C
         IF (.NOT.LCELL) CALL RBCELL(CELL,VOL)
C
 1005    CALL XYZADVANCE(IXYZIN,0,0,*1020,*1020)
         CALL XYZATOM(IXYZIN,ISER,ATNAME,RESNAM,CHNNAM,IRES,
     $            RESNO,INSCOD,ALTCOD,SEGID,IZ,ID)
         CALL XYZCOORD(IXYZIN,'F','U',XX,YY,ZZ,Q,BISO,B)
C
C---- If ANISO card then skip, b-factor not used.
C
         IF (B(2).NE.0.0 .OR. B(3).NE.0.0) THEN
           IF (XX.EQ.0.0 .AND. YY.EQ.0.0 .AND. ZZ.EQ.0.0) GOTO 1005
         ENDIF

         NATOMS = NATOMS+1
         IF (NATOMS .GT. MAXATM) GOTO 1020
         ATMNAM(NATOMS) = ATNAME(1:1)
         ISYM(NATOMS) = 1
         XS(1,NATOMS) = XX
         XS(2,NATOMS) = YY
         XS(3,NATOMS) = ZZ
         RSANAM(NATOMS) = RESNAM
         BIS(NATOMS) = BISO
C
C---- Store the original chain names
         ORIGCHN(NATOMS) = CHNNAM
C
C---- Get next atom
         GO TO 1005
      ENDIF
C
 1020 WRITE(6,'(1X,I8,A,/)') NATOMS,' atoms read'
      IF (NATOMS.EQ.0)
     +   CALL CCPERR(1,'No atoms read in. Use ATOM or READ keyword.')
      IF (NATOMS .GT. MAXATM) THEN
         WRITE(6,6080) MAXATM
         CALL CCPERR(1,'Maximum no. of atoms exceeded.')
      ENDIF
 6080 FORMAT(' The number of atoms read in exceeds the maximum',I8,/,
     . ' The parameter MAXATM must be changed.')
C
      WRITE(6,6090)
     .  'Space group for atoms    : ',NAMSGP,', ',NATSYM,
     .  ' symmetry operations'
6090  FORMAT(/,1X,A,A,A,I4,A,/)
C
C Get symmetry operations as character strings
C This gives a list of which operations correspond to which numbers
C in the output to operations
C
      WRITE(6,'(A,/,A)')
     +      ' List of symmetry operations:',
     +      ' ----------------------------'
      CALL SYMTR4(NATSYM,ATMROT,SYMCHS)
      DO I=1,NATSYM
        WRITE(6,FMT='(X,A,I2.2,A,A20)')
     +  'Sym.op. ',I,' : ',SYMCHS(I)
      END DO
      WRITE(6,'(/,A,/,A,/,A)')
     +' Symmetry-generated atoms in output file will be labelled',
     +' with the two-digit code number identifying the relevant',
     +' symmetry operation from the list above.'
C
C Set up matrices needed
C
C Orthogonalization
      CALL RBFRAC2(CELL(1),CELL(2),CELL(3),CELL(4),CELL(5),CELL(6),
     $                                                         NCODE)
C Get reciprocal cell
      CALL RBRCEL(RCEL,RVOL)
      CALL RBRORF(RO,RF)
      WRITE (6,6100) NCODE, ((RO(I,J),J=1,3),I=1,3)
6100  FORMAT(/,
     .   ' Fractional to Orthogonal matrix for orthogonalization code ',
     .   I3,//,3(11X,3F10.5,/))
C
C
C
      IF (SPHERE) THEN
C  Sphere mode, set centre (fractional & orthogonal)
        IF (SPORTH) THEN
C          Orthogonal, convert to fractional
C             Copy orthogonal centre to XYZCNO
          CALL CCPMVI(XYZCNO,XYZCNF,6)
C               XYZCNF contains fractional centre
          CALL CVFRAC2(XYZCNO(1),XYZCNO(2),XYZCNO(3),
     .                  XYZCNF(1),XYZCNF(2),XYZCNF(3),1)
        ELSE
C          Fractional, set orthogonal centre
          CALL CVFRAC2(XYZCNF(1),XYZCNF(2),XYZCNF(3),
     .                  XYZCNO(1),XYZCNO(2),XYZCNO(3),0)
        ENDIF
C
        DO 1010, I=1,3
C          Fractional limits at RADIUS (Angstrom) are at r a*, r b*, r c*
          FLMMAP(1,I) = XYZCNF(I) - RADIUS*RCEL(I)
          FLMMAP(2,I) = XYZCNF(I) + RADIUS*RCEL(I)
 1010   CONTINUE
C Square the radius
        RADIUS = RADIUS**2
C
      ELSE
        GRIDPT = (FLMMAP(2,1)-FLMMAP(1,1))* 
     .           (FLMMAP(2,2)-FLMMAP(1,2))* 
     .           (FLMMAP(2,3)-FLMMAP(1,3))
        IF ((FLMMAP(2,1)-FLMMAP(1,1)).EQ.0.0) 
     .    GRIDPT = (FLMMAP(2,2)-FLMMAP(1,2))* (FLMMAP(2,3)-FLMMAP(1,3))
        IF ((FLMMAP(2,2)-FLMMAP(1,2)).EQ.0.0) 
     .    GRIDPT = (FLMMAP(2,3)-FLMMAP(1,3))* (FLMMAP(2,1)-FLMMAP(1,1))
        IF ((FLMMAP(2,3)-FLMMAP(1,3)).EQ.0.0) 
     .    GRIDPT = (FLMMAP(2,2)-FLMMAP(1,2))* (FLMMAP(2,1)-FLMMAP(1,1))
C
C---- Allow limits to be input in grid points or
C     fractions of cell edge.
C
          IF (GRIDPT.GT.50.0) THEN
            FLMMAP(1,1) = FLMMAP(1,1)/MXYZ(1)
            FLMMAP(2,1) = FLMMAP(2,1)/MXYZ(1)
            FLMMAP(1,2) = FLMMAP(1,2)/MXYZ(2)
            FLMMAP(2,2) = FLMMAP(2,2)/MXYZ(2)
            FLMMAP(1,3) = FLMMAP(1,3)/MXYZ(3)
            FLMMAP(2,3) = FLMMAP(2,3)/MXYZ(3)
          END IF                 
      ENDIF
C Check volume is less than 1 cell
C CELMUL is .true. if the map volume is more than 1 unit cell in any direction
      CELMUL = .FALSE.         
      DO 1, J=1,3              
C  FBOX is length of box in each direction
          FBOX(J) = FLMMAP(2,J) - FLMMAP(1,J)
C  NCT  is the number of unit cells in each direction, = 0 if smaller than cell
          NCT(J) = INT(FBOX(J))
C JFLMIN is next integer below minimum fractional coordinate
C JFLMAX is next integer above maximum fractional coordinate
          JFLMIN(J) = JMINT(FLMMAP(1,J), -1)
          JFLMAX(J) = JMINT(FLMMAP(2,J), +1)
1     CONTINUE
C
C
C
C Generate all symmetry related atoms
      K = 0
C
C Depending on SORTMODE do either all symops for each atom (def.)
C or all atoms for each symop
      IF (SORTMODE.EQ.1) THEN
        NLIMIT1 = NATOMS
        NLIMIT2 = NATSYM
      ELSE IF (SORTMODE.EQ.2) THEN
        NLIMIT1 = NATSYM
        NLIMIT2 = NATOMS
      ELSE
        CALL CCPERR(1,' **** SORTMODE out of range ****')
      END IF
C
      DO 1030, I1=1,NLIMIT1
      DO 1030, I2=1,NLIMIT2
C  Set symop and atom values
         IF (SORTMODE.EQ.1) THEN
           J = I1
           L = I2
         ELSE
           J = I2
           L = I1
         ENDIF
C  Apply symmetry         
         CALL MATVC4(XYZ,ATMROT(1,1,L),XS(1,J))
C  Reduce to unit cell
         CALL CELTRN(XYZ,UNITCL,IFLAG)
         IF (IFLAG .NE. 0) THEN
            CALL CCPERR(1,'*** Impossible coordinate ***')
         ENDIF
         DO 1031, IX = JFLMIN(1),JFLMAX(1)
            DO 1032, IY = JFLMIN(2),JFLMAX(2)
               DO 1033, IZ = JFLMIN(3),JFLMAX(3)
                  IF (K .GE. MAXATM) THEN
                     WRITE(ERRLIN,FMT='(A,I8,A)')
     .                    ' *** More than ',MAXATM
     $                    ,' symmetry-generated atoms ***'
                     CALL CCPERR(2,ERRLIN)
                     GO TO 1100
                  ELSE
                     K = K+1
                     ATNAM(K) = ATMNAM(J)
                     RSTNAM(K) = RSANAM(J)
                     BISS(K) = BIS(J)
                     WRITE (SITNAM(K),'(A,I2.2,1X)') ATNAM(K),L
                     ISYM(K) = L
                     XSS(1,K) = XYZ(1) + IX
                     XSS(2,K) = XYZ(2) + IY
                     XSS(3,K) = XYZ(3) + IZ
C Set the chain id for output
C This is done on basis of symmetry operation and optionally
C on the original chain (see CHAIN SYMMETRY keyword)
                     IF (ISYMCH.GT.0) THEN
                       IF (SORTMODE.EQ.1) THEN
                         IATOM  = I1
                         ISYMOP = I2
                       ELSE
                         IATOM  = I2
                         ISYMOP = I1
                       ENDIF
                       CHAIN(K) = ' '
                       DO II = 1,ISYMCH
                         IF (ISYMOP.EQ.SYMOP(II)) THEN
                           IF (ORIGCHN(IATOM).EQ.OLDCH(II) .OR.
     +                          OLDCH(II).EQ.' ') THEN
                             CHAIN(K) = NEWCH(II)
                           ENDIF
                         ENDIF
                       END DO
                     ELSE
                       CHAIN(K) = ' '
                     END IF
C Is it in desired volume ?
                     IF (.NOT.LINBOX(XSS(1,K), FLMMAP)) THEN
C Failed
                        K = K - 1
                     ELSE
C Passed, check sphere if necessary
                        IF (SPHERE) THEN
                           CALL CVFRAC2(XSS(1,L),XSS(2,L),XSS(3,L),
     $                                      Z(1),Z(2),Z(3),0)
                           R =  (Z(1)-XYZCNO(1))**2+
     .                          (Z(2)-XYZCNO(2))**2+
     .                          (Z(3)-XYZCNO(3))**2
C     Check for distance: note RADIUS is radius**2
                           IF (R .GT. RADIUS) K = K - 1
                        ENDIF
                     ENDIF
                  ENDIF
 1033          CONTINUE
 1032       CONTINUE
 1031    CONTINUE
 1030 CONTINUE
C
 1100 NSITE = K
C
C******************************************************************
C
C Now write out all sites to Brookhaven format file (orthogonal)
C and (optionally) to printer
C
      IF (BROOKF) THEN
         IFAIL = 0
         IXYZOUT = 0
         CALL XYZOPEN('XYZOUT','OUTPUT','PDB',IXYZOUT,IFAIL)
         
         ERRLIN = 'REMARK  '//VTITLE
         CALL WREMARK(IXYZOUT,ERRLIN)
C
         CALL WBCELL(IXYZOUT,CELL,NCODE)
      ENDIF

      WRITE(6,6205) NSITE
6205   FORMAT(/,' Number of accepted positions             : ',I6,/)
      IF (PRINT) WRITE(6,6210)
6210  FORMAT(//,'   List of sites',//,
     .  '  Number        Fractional coordinate         ',
C        XiiiiiixxxxffffffffffFFFFFFFFFFffffffffffxxxxx
     .  '  Orthogonal coordinate       Grid coordinate  ',
C        ffffffffFFFFFFFFffffffffxxxxxiiiiiiIIIIIIiiiiii
     .  '    Symmetry',/)
C        xxxxxaaaaxxiiiIII
C
      Q=1.0                                              
      RESNAM = RSTNAM(1)
      CHNNAM=' '
      INSCOD = ' '
      ALTCOD = ' '
      RESNO = ' '
      SEGID = ' '
      IRES = 1
      IZ = 0
      DO 2000 I=1,6
        B(I)=0.0
2000  CONTINUE
      ID = ' '
      SITE = SITNAM(1)(1:1)
C
      DO 2100, L=1,NSITE
C Orthogonalize
C
      IF (RSTNAM(L) .NE. RESNAM) THEN
C  New residue
         RESNAM = RSTNAM(L)
         IRES = IRES + 1
      ENDIF
      B(1)=BISS(L)
      CHNNAM = CHAIN(L)
      IF (BROOKF) THEN
        CALL XYZATOM(IXYZOUT,L,SITNAM(L),RSTNAM(L),CHNNAM,IRES,
     .               RESNO,INSCOD,ALTCOD,SEGID,IZ,ID)
        CALL XYZCOORD(IXYZOUT,'F','U',XSS(1,L),XSS(2,L),XSS(3,L),Q,
     .                                                          BISO,B)
        CALL XYZADVANCE(IXYZOUT,0,0,*2100,*2100)
      ENDIF
C
      IF (PRINT) THEN
C Calculate grid units
            CALL CVFRAC2(XSS(1,L),XSS(2,L),XSS(3,L),Z(1),Z(2),Z(3),0)
            DO 2110, I=1,3
2110        NCT(I) = NINT(XSS(I,L) * MXYZ(I))
            IF (SITNAM(L)(1:1) .NE. SITE) THEN
               SITE = SITNAM(L)(1:1)
               WRITE(6,*)
            ENDIF
            WRITE (6,6300) L, (XSS(I,L), I=1,3), Z, 
     .                     NCT, SITNAM(L), RSTNAM(L), ISYM(L)
6300      FORMAT(1X,I6,4X,3F10.5,5X,3F8.2,5X,3I6,2(4X,A),2X,2I3)
      ENDIF
2100  CONTINUE
C
      WRITE (6, '(//,1X,I6,A,I6,A,/)')
     $     NSITE, ' atoms generated from',
     $     NATOMS, ' input sites'

C
      IF (INFILE .NE. ' ') CALL XYZCLOSE(IXYZIN)
      IF (BROOKF) CALL XYZCLOSE(IXYZOUT)
      CALL CCPERR(0, ' *** Normal termination ***')
      END
C
C
      SUBROUTINE CELTRN(Z,FLMMAP,IFLAG)
C     ================================
C
C  Translate position Z into box defined by FLMMAP if possible
C  Returns IFLAG = 0 if OK, = 1 if impossible
C
      REAL Z(3), FLMMAP(2,3)
      INTEGER IFLAG
C
      INTEGER K
      REAL FR,FD
C See if this site can be put back into the volume by cell translation
            DO 1220, K=1,3
                FR = Z(K) - FLMMAP(1,K)
                FD = AMOD(FR, 1.0)
                IF (FD.LT.0.0) FD = FD + 1.0
C FD is coordinate relative to start of box
C  Test against box size, jump out as soon as it fails
                IF (FD .GT. (FLMMAP(2,K)-FLMMAP(1,K))) GO TO 1210
C  Required translation is FD - FR
                Z(K) = Z(K) + FD - FR
1220      CONTINUE
C
      IFLAG = 0
      RETURN
C
1210  IFLAG = 1
      RETURN
C
      END
C
      logical function linbox(z, box)
c     ===============================
c
c  Returns .true. if position z(3) is in box(1,xyz) to box(2,xyz)
c
      real z(3), box(2,3)
c
      integer i
c
      do 1, i = 1,3
         if (z(i) .lt. box(1,i) .or.
     $       z(i) .gt. box(2,i)) then
            linbox = .false.
            return
         endif
 1    continue 
      linbox = .true.
      return
      end
C
C
C*********************************************************************
      INTEGER FUNCTION JMINT(A,N)
C     ==================
C
C Truncate to next higher integer if N.GE.0, next lower if N.LT.0
C
      real a
      integer n

      if (abs(amod(a,1.0)) .gt. 0.000001) then
         if (n.gt.0) then
            if (a .gt. 0.0) then
               jmint=int(a+0.999999)
            else
               jmint=int(a)
            endif
         else
            if (a .lt. 0.0) then
               jmint=int(a-0.999999)
            else
               jmint=int(a)
            endif
         endif
      else
         jmint = nint(a)
      endif
      return
      end
