C
C     HGEN
C     Copyright (C) 1979 Simon Phillips
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C*************************************************************
C
C
C**************************************************************
C
C     HYDROGEN GENERATION PROGRAM FOR PROTEINS
C
C     ORIGINATED BY  S.E.V. PHILIPS    JAN  1979
C
C     COMPLETELY REWRITTEN  BY  Z. DAUTER   FEB  1990
C     PARAMETRISED BY VICTOR S. LAMZIN 18 AUG 1992
C
C     IT WORKS NOW WITH BROOKHAVEN COORDINATES (+/- SCALE CARDS)
C     AND ANY ORDER OF ATOMS WITHIN A RESIDUE
C     ALSO N-TERMINAL NH3 GROUPS AND PROLINES ARE TREATED PROPERLY
C
C     THE USER HAS TO ANSWER Y/N TO CREATE THE OUTPUT FILE
C     WITH HYDROGENS ONLY OR ALL INPUT ATOMS + HYDROGENS
C
C     TO INCLUDE A NEW RESIDUE IT IS NECESSARY TO PUT IT 
C     AT THE END OF RESLIB SUBROUTINE AND INCREASE NTYP
C     IN RESLIB:  RES(N) - N-TH RESIDUE ID
C                 NAT(N) - NUMBER OF ATOMS HAVING HYDROGENS ON THEM
C                          (ONLY THOSE ARE MENTIONED HERE)
C                 ACON(1,M,N) - NAME OF ATOM HAVING HYDROGEN(S)
C                 NCON(M,N)   - TYPE OF HYDROGEN(S), AS IN SUBR. HYDROS
C                 ACON(2-4,M,N) - ADDITIONAL ATOMS TO DEFINE HYDROGENS
C                                 (USUALLY CONNECTED TO PARENT ATOM)
C
C     .. 
C     .. Parameters ..
      PARAMETER(NATOMS=50)
C     ..
C     .. Local Arrays ..
      REAL BB(6),Y(3,3),X(3,NATOMS),B(7,NATOMS),OC(NATOMS)
      INTEGER  NAT(25),NCON(10,25),NR(NATOMS)
      CHARACTER*3 RES(25),ACON(4,10,25)
      CHARACTER*3 R(NATOMS)
      CHARACTER*1 CH(NATOMS),ALT(NATOMS),INS(NATOMS)
      CHARACTER*4 A(NATOMS),SEG(NATOMS),IAN(NATOMS)
C     ..
C     .. Local Scalars ..
      REAL BISO,OCC,XX,YY,ZZ
      INTEGER I,J,IRESN,ISER,IXYZIN,IXYZOUT,IZ,K
      INTEGER IFAIL,NMON,NSEP,NTYP,NADDED
      CHARACTER*80 LIN1
      CHARACTER*4 AH,BH,ATNAM,RESNAM,RESNO,ID,SEGID
      CHARACTER*1 CHNNAM,ALTCOD,INSCOD,INSC
      LOGICAL CONT,FLAG
C     ..
C     .. External Functions ..
      LOGICAL CCPONL
      EXTERNAL CCPONL
C     ..
C     .. External Routines ..
      EXTERNAL CCPERR,CCPFYP,CCPRCS,HYDROS,MEMOPARSE,PARSEDIAGNOSE,
     +         PARSESUBKEY,RESLIB,XYZADVANCE,XYZATOM,
     +         XYZCLOSE,XYZCOORD,XYZINIT,XYZOPEN,
     +         WREMARK
C     ..
      nmon = 0
      NSEP = 0
      NADDED = 0
      IFAIL = 0
      IXYZIN = 0
      IXYZOUT = 0
C
C          ***************************************************
      CALL CCPFYP
      call ccp4h_init()
      call ccp4h_summary_beg()
      call ccp4h_header('HGEN','hgen',1)
      call ccp4h_summary_end()
      call ccp4h_pre_beg() 
      CALL CCPRCS (6, 'HGEN', '$Date: 2005/11/18 14:02:48 $')

      CALL XYZINIT
      CALL XYZOPEN('XYZIN','INPUT',' ',IXYZIN,IFAIL)
      CALL XYZOPEN('XYZOUT','OUTPUT',' ',IXYZOUT,IFAIL)
C          ***************************************************
C
      call ccp4h_summary_beg()
      WRITE(6,10)
   10 FORMAT(/,'  HYDROGEN ATOM GENERATION',/)
      call ccp4h_summary_end()

   20 CALL MEMOPARSE(.TRUE.)
      FLAG = .FALSE.
C
C          *******************************
      CALL PARSESUBKEY('HYDR','SEPA',FLAG)
C          *******************************
C
      IF (FLAG) NSEP = 1
      FLAG = .FALSE.
C
C          *******************************
      CALL PARSESUBKEY('HYDR','APPE',FLAG)
C          *******************************
C
      IF (FLAG) NSEP = 0
C
C          *******************
      CALL PARSEDIAGNOSE(CONT)
C          *******************
C
      IF (CONT) GOTO 20
C
C---- End of Input
C
      call ccp4h_summary_beg()
      IF (NSEP .EQ. 0) THEN
        WRITE(6,25) 'XYZOUT contains all atoms including hydrogens.'
      ELSEIF (NSEP .EQ. 1) THEN
        WRITE(6,25) 'XYZOUT contains hydrogens only.'
      ENDIF
 25   FORMAT(/,2X,A,/)
      call ccp4h_summary_end()

C          *************************
      CALL RESLIB(RES,NAT,ACON,NCON)
C          *************************
C
      NTYP=20
      IEND=0
      NTOT=0
      NTOTN=0
      NTOTH=0
      N=1
C
C---- Copy REMARKS
C
      IF (NSEP .EQ. 0) THEN
        LIN1 = 'REMARK    HYDROGENS ADDED BY HGEN (CCP4)'
        CALL WREMARK(IXYZin,LIN1)
      ENDIF
C
C---- Start reading file:
C
C          ****************************************************
      CALL XYZADVANCE(IXYZIN,IXYZOUT,0,*400,*400)
      CALL XYZATOM(IXYZIN,ISER,ATNAM,RESNAM,CHNNAM,IRESN,
     +               RESNO,INSCOD,ALTCOD,SEGID,IZ,ID)
      CALL XYZCOORD(IXYZIN,'O','B',XX,YY,ZZ,OCC,BISO,BB)
C          ****************************************************
C
C---- READ FIRST RESIDUE INTO CORE
C
      A(N) = ATNAM
      R(N) = RESNAM(1:3)
      CH(N) = CHNNAM
      NR(N) = IRESN
      X(1,N) = XX
      X(2,N) = YY
      X(3,N) = ZZ
      OC(N) = OCC
      DO 30 I=1,6
        B(I,N) = BB(I)
   30 CONTINUE
      B(7,N) = BISO
      INS(N) = INSCOD
      ALT(N) = ALTCOD
      SEG(N) = SEGID
      IAN(N) = ID
C
      NRC=NR(N)
      INSC=INS(N)
      M1=N
      N=N+1
      A(N)=A(M1)
      R(N)=R(M1)
      CH(N)=CH(M1)
      NR(N)=NR(M1)
      X(1,N)=X(1,M1)
      X(2,N)=X(2,M1)
      X(3,N)=X(3,M1)
      OC(N)=OC(M1)
      DO 40 I=1,7
        B(I,N)=B(I,M1)
   40 CONTINUE
      INS(N) = INS(M1)
      ALT(N) = ALT(M1)
      SEG(N) = SEG(M1)
      IAN(N)=IAN(M1)
  100 N=N+1
C
C---- READ RESIDUE INTO CORE
C
C          ****************************************************
  101 CALL XYZADVANCE(IXYZIN,IXYZOUT,0,*110,*110)
      CALL XYZATOM(IXYZIN,ISER,ATNAM,RESNAM,CHNNAM,IRESN,
     +           RESNO,INSCOD,ALTCOD,SEGID,IZ,ID)
      CALL XYZCOORD(IXYZIN,'O','B',XX,YY,ZZ,OCC,BISO,BB)
C          ****************************************************
C
C---- CHECK FOR ANISO CARD
C
      IF (BB(2).NE.0.0 .OR. BB(3).NE.0.0) THEN
        IF (XX.EQ.0.0 .AND. YY.EQ.0.0 .AND. ZZ.EQ.0.0) THEN
          DO 60 I=N-1,1,-1
            IF (CHNNAM .EQ. CH(I)) THEN
              IF (IRESN .EQ. NR(I)) THEN
                IF (ATNAM .EQ. A(I)) THEN
                  DO 50 J=1,6
                    B(J,I) = BB(J)
   50             CONTINUE
                  GOTO 101
                ENDIF
              ENDIF
            ENDIF
   60     CONTINUE
          CALL CCPERR(2,' *** ANISO not matched with ATOM record ***')
          GOTO 101
        ENDIF
      ENDIF  

      A(N) = ATNAM
      R(N) = RESNAM(1:3)
      CH(N) = CHNNAM
      NR(N) = IRESN
      X(1,N) = XX
      X(2,N) = YY
      X(3,N) = ZZ
      OC(N) = OCC
      DO 70 I=1,6
        B(I,N) = BB(I)
   70 CONTINUE
      B(7,N) = BISO
      INS(N) = INSCOD
      ALT(N) = ALTCOD
      SEG(N) = SEGID
      IAN(N) = ID
      IF(A(N)(1:1).EQ.'H' .OR. ID.EQ.' H') GO TO 101
C---- If still on same residue, read another atom
      IF(NR(N).EQ.NRC .AND. INS(N).EQ.INSC) GO TO 100
      NRC=NR(N)
      INSC=INS(N)
      GOTO 120

  110 IEND=2

  120 NN1=N-1
C
C---- CHECK FOR DISORDERED ATOMS (APPEARING MORE THAN ONCE)
C
      IDIS=0
      NN2=NN1-1
      DO 122 I=2,NN2
        AH=A(I)
        AH(4:4)=ALT(I)
        I1=I+1
        DO 121 J=I1,NN1
          BH=A(J)
          BH(4:4)=ALT(J)
          IF(AH.EQ.BH) IDIS=1
  121   CONTINUE
  122 CONTINUE

      DO 350 I=2,NN1
        IF (NSEP.EQ.0) THEN
          NTOT=NTOT+1
          NTOTN=NTOTN+1
          ISER = NTOT
          ATNAM = A(I)
          RESNAM = R(I)
          CHNNAM = CH(I)
          IRESN = NR(I)
          XX = X(1,I)
          YY = X(2,I)
          ZZ = X(3,I)
          OCC = OC(I)
          DO 130 J=1,6
            BB(J) = B(J,I)
  130     CONTINUE
          INSCOD = INS(I)
          ALTCOD = ALT(I)
          SEGID = SEG(I)
          ID = IAN(I)
C
C              *****************************************************
          CALL XYZATOM(IXYZOUT,ISER,ATNAM,RESNAM,CHNNAM,IRESN,
     +                   RESNO,INSCOD,ALTCOD,SEGID,IZ,ID)
          CALL XYZCOORD(IXYZOUT,'O','B',XX,YY,ZZ,OCC,BISO,BB)
          CALL XYZADVANCE(IXYZOUT,0,0,*160,*160)
C              *****************************************************
C
        ENDIF

        DO 140 J=1,NTYP
          NT=J
          IF(R(I).EQ.RES(J)) GOTO 160
  140   CONTINUE
C---- test for water
        IF (R(I) .NE. 'WAT'.AND.R(I).NE.'HOH')
     *    WRITE(6,150) A(I),R(I),CH(I),NR(I),X(1,I),X(2,I),X(3,I),
     *    OC(I),BISO
  150     FORMAT(' FOR ATOM:',2X,A4,A3,1X,A1,I4,4X,3F8.3,2F6.2,
     *    '    NO RESIDUE TYPE IN LIBRARY')
        GOTO 350

  160   CONTINUE
        IF(IDIS.NE.0) NT=21
        NATH=NAT(NT)
        IF(.NOT.(IEND.EQ.0 .AND. A(I).EQ.'N  ')) THEN
          DO 170 J=1,NATH
            NATL=J
            IF(A(I).EQ.ACON(1,J,NT)) GOTO 180
  170     CONTINUE
          GOTO 350

  180     M=NCON(NATL,NT)
          N1=I
          IF (ACON(2,NATL,NT) .NE. '   ') THEN
            DO 190 J=1,NN1
              N2=J
              IF(A(J).EQ.ACON(2,NATL,NT)) GOTO 200
  190       CONTINUE
            GOTO 340
          ENDIF

  200     IF (ACON(3,NATL,NT) .NE. '   ') THEN
            DO 210 J=1,NN1
              N3=J
              IF(A(J).EQ.ACON(3,NATL,NT)) GOTO 220
  210       CONTINUE
            GOTO 340
          ENDIF

  220     IF(ACON(4,NATL,NT).EQ.'   ') GOTO 320
          DO 230 J=1,NN1
            N4=J
            IF(A(J).EQ.ACON(4,NATL,NT)) GOTO 320
  230     CONTINUE
          GOTO 340
        ENDIF
C
C---- N-TERMINUS
C
        IF (R(I) .NE. 'PRO') THEN
          M=5
          N1=I
          DO 250 J=2,NN1
            N2=J
            IF(A(J).EQ.'CA ') GOTO 260
  250     CONTINUE

  260     DO 270 J=2,NN1
            N3=J
            IF(A(J).EQ.'C  ') GOTO 320
  270     CONTINUE
        ELSE
C
C---- N-TERMINAL PROLINE
C
          M=2
          N1=I
          DO 290 J=2,NN1
            N2=J
            IF(A(J).EQ.'CA ') GOTO 300
  290     CONTINUE

  300     DO 310 J=2,NN1
            N3=J
            IF(A(J).EQ.'CD ') GOTO 320
  310     CONTINUE
        ENDIF
C
C            *******************************
  320   CALL HYDROS(M,N1,N2,N3,N4,NOHYD,X,Y)
C            *******************************
C
        BH=A(I)
        DO 330 J=1,NOHYD
          NTOT=NTOT+1
          AH(1:1)='H'
          IF(J.EQ.1) AH(2:2)='1'
          IF(J.EQ.2) AH(2:2)='2'
          IF(J.EQ.3) AH(2:2)='3'
          AH(3:3)=BH(2:2)
          AH(4:4)=BH(3:3)
          ID=' H'
          ISER = NTOT
          ATNAM = AH
          RESNAM = R(I)
          CHNNAM = CH(I)
          IRESN = NR(I)
          XX = Y(1,J)
          YY = Y(2,J)
          ZZ = Y(3,J)
          OCC = OC(I)
          INSCOD = INS(I)
          ALTCOD = ALT(I)
          SEGID = SEG(I)
C
C---- ISOTROPIC B FOR HYDROGENS
C
          BB(1) = B(7,I)
          DO 325 K=2,6
            BB(K) = 0.0
  325     CONTINUE
C
C              ******************************************************
          CALL XYZATOM(IXYZOUT,ISER,ATNAM,RESNAM,CHNNAM,IRESN,
     +                     RESNO,INSCOD,ALTCOD,SEGID,IZ,ID)
          CALL XYZCOORD(IXYZOUT,'O','B',XX,YY,ZZ,OCC,BISO,BB)
          CALL XYZADVANCE(IXYZOUT,0,0,*330,*330)
          NADDED = NADDED + 1
C              ******************************************************
C
  330   CONTINUE
        NTOTH=NTOTH+NOHYD
        GOTO 350
C
C---- CAN NOT FIND ATOMS DEFINING HYDROGENS
C
  340   WRITE(6,341) A(I),R(I),CH(I),NR(I)
  341   FORMAT(' CAN NOT DEFINE HYDROGEN ON ATOM:',2X,A4,A3,1X,A1,I4,
     1      '         SKIPPED')
  350 CONTINUE

      DO 360 J=2,NN1
        N1=J
        IF(A(J).EQ.'C  ') GOTO 370
  360 CONTINUE

  370 DO 380 K=1,3
        X(K,1)=X(K,N1)
        X(K,2)=X(K,N)
  380 CONTINUE
      A(1)='C-1'
      R(1)=R(N1)
      CH(1)=CH(N1)
      NR(1)=NR(N1)
      INS(1)=INS(N1)
      OC(1)=OC(N1)
      DO 385 J=1,7
        B(J,1)=B(J,N1)
  385 CONTINUE
      IAN(1)=IAN(N1)
      A(2)=A(N)
      R(2)=R(N)
      CH(2)=CH(N)
      NR(2)=NR(N)
      INS(2)=INS(N)
      OC(2)=OC(N)
      DO 390 J=1,7
        B(J,2)=B(J,N)
  390 CONTINUE
      IAN(2)=IAN(N)
      N=2
      IF(IEND.NE.2) IEND=1
      IF(IEND.LT.2) GOTO 100          

      CALL XYZCLOSE(IXYZIN)
      CALL XYZCLOSE(IXYZOUT)

      call ccp4h_summary_beg()
      WRITE(6,395) NADDED
 395  FORMAT(/,2X,I8,' hydrogens generated and written to XYZOUT.',/)
      call ccp4h_summary_end()

      CALL CCPERR (0, 'Normal termination')

400   CALL CCPERR(1, ' ERROR: PDB file is empty')
      END
C--------------------------------------------------------------
      SUBROUTINE RESLIB(RES,NAT,ACON,NCON)
      CHARACTER*3 RES(25),ACON(4,10,25)
      INTEGER NAT(25),NCON(10,25)
      DO 10 I=1,4
      DO 10 J=1,10
      DO 10 K=1,25
      ACON(I,J,K)='   '
   10 CONTINUE
C     ALA
      RES(1)='ALA'
      NAT(1)=3
      ACON(1,1,1)='N  '
      NCON(1,1)=1
      ACON(2,1,1)='CA '
      ACON(3,1,1)='C-1'
      ACON(4,1,1)='   '
      ACON(1,2,1)='CB '
      NCON(2,1)=5
      ACON(2,2,1)='CA '
      ACON(3,2,1)='C '
      ACON(4,2,1)='   '
      ACON(1,3,1)='CA '
      NCON(3,1)=3
      ACON(2,3,1)='CB '
      ACON(3,3,1)='N  '
      ACON(4,3,1)='C  '
C     ARG
      RES(2)='ARG'
      NAT(2)=8
      ACON(1,1,2)='N  '
      NCON(1,2)=1
      ACON(2,1,2)='CA '
      ACON(3,1,2)='C-1'
      ACON(4,1,2)='   '
      ACON(1,2,2)='NH2'
      NCON(2,2)=4
      ACON(2,2,2)='CZ '
      ACON(3,2,2)='NH1'
      ACON(4,2,2)='NE '
      ACON(1,3,2)='NH1'
      NCON(3,2)=4
      ACON(2,3,2)='CZ '
      ACON(3,3,2)='NH2'
      ACON(4,3,2)='NE '
      ACON(1,4,2)='NE '
      NCON(4,2)=1
      ACON(2,4,2)='CZ '
      ACON(3,4,2)='CD '
      ACON(4,4,2)='   '
      ACON(1,5,2)='CD '
      NCON(5,2)=2
      ACON(2,5,2)='NE '
      ACON(3,5,2)='CG '
      ACON(4,5,2)='   '
      ACON(1,6,2)='CG '
      NCON(6,2)=2
      ACON(2,6,2)='CD '
      ACON(3,6,2)='CB '
      ACON(4,6,2)='   '
      ACON(1,7,2)='CB '
      NCON(7,2)=2
      ACON(2,7,2)='CA '
      ACON(3,7,2)='CG '
      ACON(4,7,2)='   '
      ACON(1,8,2)='CA '
      NCON(8,2)=3
      ACON(2,8,2)='CB '
      ACON(3,8,2)='N  '
      ACON(4,8,2)='C  '
C     ASN
      RES(3)='ASN'
      NAT(3)=4
      ACON(1,1,3)='N  '
      NCON(1,3)=1
      ACON(2,1,3)='CA '
      ACON(3,1,3)='C-1'
      ACON(4,1,3)='   '
      ACON(1,2,3)='ND2'
      NCON(2,3)=4
      ACON(2,2,3)='CG '
      ACON(3,2,3)='OD1'
      ACON(4,2,3)='CB '
      ACON(1,3,3)='CB '
      NCON(3,3)=2
      ACON(2,3,3)='CA '
      ACON(3,3,3)='CG '
      ACON(4,3,3)='   '
      ACON(1,4,3)='CA '
      NCON(4,3)=3
      ACON(2,4,3)='CB '
      ACON(3,4,3)='N  '
      ACON(4,4,3)='C  '
C     ASP
      RES(4)='ASP'
      NAT(4)=3
      ACON(1,1,4)='N  '
      NCON(1,4)=1
      ACON(2,1,4)='CA '
      ACON(3,1,4)='C-1'
      ACON(4,1,4)='   '
      ACON(1,2,4)='CB '
      NCON(2,4)=2
      ACON(2,2,4)='CA '
      ACON(3,2,4)='CG '
      ACON(4,2,4)='   '
      ACON(1,3,4)='CA '
      NCON(3,4)=3
      ACON(2,3,4)='CB '
      ACON(3,3,4)='N  '
      ACON(4,3,4)='C  '
C     CYS
      RES(5)='CYS'
      NAT(5)=4
      ACON(1,1,5)='N  '
      NCON(1,5)=1
      ACON(2,1,5)='CA '
      ACON(3,1,5)='C-1'
      ACON(4,1,5)='   '
      ACON(1,2,5)='SG '
      NCON(2,5)=7
      ACON(2,2,5)='CB '
      ACON(3,2,5)='CA '
      ACON(4,2,5)='   '
      ACON(1,3,5)='CB '
      NCON(3,5)=2
      ACON(2,3,5)='SG '
      ACON(3,3,5)='CA '
      ACON(4,3,5)='   '
      ACON(1,4,5)='CA '
      NCON(4,5)=3
      ACON(2,4,5)='CB '
      ACON(3,4,5)='N  '
      ACON(4,4,5)='C  '
C     GLN
      RES(6)='GLN'
      NAT(6)=5
      ACON(1,1,6)='N  '
      NCON(1,6)=1
      ACON(2,1,6)='CA '
      ACON(3,1,6)='C-1'
      ACON(4,1,6)='   '
      ACON(1,2,6)='NE2'
      NCON(2,6)=4
      ACON(2,2,6)='CD '
      ACON(3,2,6)='OE1'
      ACON(4,2,6)='CG '
      ACON(1,3,6)='CG '
      NCON(3,6)=2
      ACON(2,3,6)='CD '
      ACON(3,3,6)='CB '
      ACON(4,3,6)='   '
      ACON(1,4,6)='CB '
      NCON(4,6)=2
      ACON(2,4,6)='CA '
      ACON(3,4,6)='CG '
      ACON(4,4,6)='   '
      ACON(1,5,6)='CA '
      NCON(5,6)=3
      ACON(2,5,6)='CB '
      ACON(3,5,6)='N  '
      ACON(4,5,6)='C  '
C     GLU
      RES(7)='GLU'
      NAT(7)=4
      ACON(1,1,7)='N  '
      NCON(1,7)=1
      ACON(2,1,7)='CA '
      ACON(3,1,7)='C-1'
      ACON(4,1,7)='   '
      ACON(1,2,7)='CG '
      NCON(2,7)=2
      ACON(2,2,7)='CD '
      ACON(3,2,7)='CB '
      ACON(4,2,7)='   '
      ACON(1,3,7)='CB '
      NCON(3,7)=2
      ACON(2,3,7)='CA '
      ACON(3,3,7)='CG '
      ACON(4,3,7)='   '
      ACON(1,4,7)='CA '
      NCON(4,7)=3
      ACON(2,4,7)='CB '
      ACON(3,4,7)='N  '
      ACON(4,4,7)='C  '
C     GLY
      RES(8)='GLY'
      NAT(8)=2
      ACON(1,1,8)='N  '
      NCON(1,8)=1
      ACON(2,1,8)='CA '
      ACON(3,1,8)='C-1'
      ACON(4,1,8)='   '
      ACON(1,2,8)='CA '
      NCON(2,8)=2
      ACON(2,2,8)='N  '
      ACON(3,2,8)='C  '
      ACON(4,2,8)='   '
C     HIS
      RES(9)='HIS'
      NAT(9)=6
      ACON(1,1,9)='N  '
      NCON(1,9)=1
      ACON(2,1,9)='CA '
      ACON(3,1,9)='C-1'
      ACON(4,1,9)='   '
      ACON(1,2,9)='CE1'
      NCON(2,9)=1
      ACON(2,2,9)='NE2'
      ACON(3,2,9)='ND1'
      ACON(4,2,9)='   '
      ACON(1,3,9)='CD2'
      NCON(3,9)=1
      ACON(2,3,9)='NE2'
      ACON(3,3,9)='CG '
      ACON(4,3,9)='   '
      ACON(1,4,9)='CB '
      NCON(4,9)=2
      ACON(2,4,9)='CA '
      ACON(3,4,9)='CG '
      ACON(4,4,9)='   '
      ACON(1,5,9)='CA '
      NCON(5,9)=3
      ACON(2,5,9)='CB '
      ACON(3,5,9)='N  '
      ACON(4,5,9)='C  '
      ACON(1,6,9)='NE2'
      NCON(6,9)=1
      ACON(2,6,9)='CE1'
      ACON(3,6,9)='CD2'
      ACON(4,6,9)='   '
C     ILE
      RES(10)='ILE'
      NAT(10)=6
      ACON(1,1,10)='N  '
      NCON(1,10)=1
      ACON(2,1,10)='CA '
      ACON(3,1,10)='C-1'
      ACON(4,1,10)='   '
      ACON(1,2,10)='CD1'
      NCON(2,10)=5
      ACON(2,2,10)='CG1'
      ACON(3,2,10)='CB '
      ACON(4,2,10)='   '
      ACON(1,3,10)='CG2'
      NCON(3,10)=5
      ACON(2,3,10)='CB '
      ACON(3,3,10)='CA '
      ACON(4,3,10)='   '
      ACON(1,4,10)='CG1'
      NCON(4,10)=2
      ACON(2,4,10)='CD1'
      ACON(3,4,10)='CB '
      ACON(4,4,10)='   '
      ACON(1,5,10)='CB '
      NCON(5,10)=3
      ACON(2,5,10)='CG2'
      ACON(3,5,10)='CG1'
      ACON(4,5,10)='CA '
      ACON(1,6,10)='CA '
      NCON(6,10)=3
      ACON(2,6,10)='CB '
      ACON(3,6,10)='N  '
      ACON(4,6,10)='C  '
C     LEU
      RES(11)='LEU'
      NAT(11)=6
      ACON(1,1,11)='N  '
      NCON(1,11)=1
      ACON(2,1,11)='CA '
      ACON(3,1,11)='C-1'            
      ACON(4,1,11)='   '
      ACON(1,2,11)='CD2'
      NCON(2,11)=5
      ACON(2,2,11)='CG '
      ACON(3,2,11)='CB '
      ACON(4,2,11)='   '
      ACON(1,3,11)='CD1'
      NCON(3,11)=5
      ACON(2,3,11)='CG '
      ACON(3,3,11)='CB '
      ACON(4,3,11)='   '
      ACON(1,4,11)='CG '
      NCON(4,11)=3
      ACON(2,4,11)='CD2'
      ACON(3,4,11)='CD1'
      ACON(4,4,11)='CB '
      ACON(1,5,11)='CB '
      NCON(5,11)=2
      ACON(2,5,11)='CA '
      ACON(3,5,11)='CG '
      ACON(4,5,11)='   '
      ACON(1,6,11)='CA '
      NCON(6,11)=3
      ACON(2,6,11)='CB '
      ACON(3,6,11)='N  '
      ACON(4,6,11)='C  '
C     LYS
      RES(12)='LYS'
      NAT(12)=7
      ACON(1,1,12)='N  '
      NCON(1,12)=1
      ACON(2,1,12)='CA '
      ACON(3,1,12)='C-1'
      ACON(4,1,12)='   '
      ACON(1,2,12)='NZ '
      NCON(2,12)=5
      ACON(2,2,12)='CE '
      ACON(3,2,12)='CD '
      ACON(4,2,12)='   '
      ACON(1,3,12)='CE '
      NCON(3,12)=2
      ACON(2,3,12)='NZ '
      ACON(3,3,12)='CD '
      ACON(4,3,12)='   '
      ACON(1,4,12)='CD '
      NCON(4,12)=2
      ACON(2,4,12)='CE '
      ACON(3,4,12)='CG '
      ACON(4,4,12)='   '
      ACON(1,5,12)='CG '
      NCON(5,12)=2
      ACON(2,5,12)='CD '
      ACON(3,5,12)='CB '
      ACON(4,5,12)='   '
      ACON(1,6,12)='CB '
      NCON(6,12)=2
      ACON(2,6,12)='CA '
      ACON(3,6,12)='CG '
      ACON(4,6,12)='   '
      ACON(1,7,12)='CA '
      NCON(7,12)=3
      ACON(2,7,12)='CB '
      ACON(3,7,12)='N  '
      ACON(4,7,12)='C  '
C     MET
      RES(13)='MET'
      NAT(13)=5
      ACON(1,1,13)='N  '
      NCON(1,13)=1
      ACON(2,1,13)='CA '
      ACON(3,1,13)='C-1'
      ACON(4,1,13)='   '
      ACON(1,2,13)='CE '
      NCON(2,13)=5
      ACON(2,2,13)='SD '
      ACON(3,2,13)='CG '
      ACON(4,2,13)='   '
      ACON(1,3,13)='CG '
      NCON(3,13)=2
      ACON(2,3,13)='SD '
      ACON(3,3,13)='CB '
      ACON(4,3,13)='   '
      ACON(1,4,13)='CB '
      NCON(4,13)=2
      ACON(2,4,13)='CA '
      ACON(3,4,13)='CG '
      ACON(4,4,13)='   '
      ACON(1,5,13)='CA '
      NCON(5,13)=3
      ACON(2,5,13)='CB '
      ACON(3,5,13)='N  '
      ACON(4,5,13)='C  '
C     PHE
      RES(14)='PHE'
      NAT(14)=8
      ACON(1,1,14)='N  '
      NCON(1,14)=1
      ACON(2,1,14)='CA '
      ACON(3,1,14)='C-1'
      ACON(4,1,14)='   '
      ACON(1,2,14)='CZ '
      NCON(2,14)=1
      ACON(2,2,14)='CE2'
      ACON(3,2,14)='CE1'
      ACON(4,2,14)='   '
      ACON(1,3,14)='CE2'
      NCON(3,14)=1
      ACON(2,3,14)='CZ '
      ACON(3,3,14)='CD2'
      ACON(4,3,14)='   '
      ACON(1,4,14)='CE1'
      NCON(4,14)=1
      ACON(2,4,14)='CZ '
      ACON(3,4,14)='CD1'
      ACON(4,4,14)='   '
      ACON(1,5,14)='CD2'
      NCON(5,14)=1
      ACON(2,5,14)='CE2'
      ACON(3,5,14)='CG '
      ACON(4,5,14)='   '
      ACON(1,6,14)='CD1'
      NCON(6,14)=1
      ACON(2,6,14)='CE1'
      ACON(3,6,14)='CG '
      ACON(4,6,14)='   '
      ACON(1,7,14)='CB '
      NCON(7,14)=2
      ACON(2,7,14)='CA '
      ACON(3,7,14)='CG '
      ACON(4,7,14)='   '
      ACON(1,8,14)='CA '
      NCON(8,14)=3
      ACON(2,8,14)='CB '
      ACON(3,8,14)='N  '
      ACON(4,8,14)='C  '
C     PRO
      RES(15)='PRO'
      NAT(15)=4
      ACON(1,1,15)='CD '
      NCON(1,15)=2
      ACON(2,1,15)='N  '
      ACON(3,1,15)='CG '
      ACON(4,1,15)='   '
      ACON(1,2,15)='CG '
      NCON(2,15)=2
      ACON(2,2,15)='CD '
      ACON(3,2,15)='CB '
      ACON(4,2,15)='   '
      ACON(1,3,15)='CB '
      NCON(3,15)=2
      ACON(2,3,15)='CG '
      ACON(3,3,15)='CA '
      ACON(4,3,15)='   '
      ACON(1,4,15)='CA '
      NCON(4,15)=3
      ACON(2,4,15)='CB '
      ACON(3,4,15)='N  '
      ACON(4,4,15)='C  '
C     SER
      RES(16)='SER'
      NAT(16)=4
      ACON(1,1,16)='N  '
      NCON(1,16)=1
      ACON(2,1,16)='CA '
      ACON(3,1,16)='C-1'
      ACON(4,1,16)='   '
      ACON(1,2,16)='OG '
      NCON(2,16)=7
      ACON(2,2,16)='CB '
      ACON(3,2,16)='CA '
      ACON(4,2,16)='   '
      ACON(1,3,16)='CB '
      NCON(3,16)=2
      ACON(2,3,16)='OG '
      ACON(3,3,16)='CA '
      ACON(4,3,16)='   '
      ACON(1,4,16)='CA '
      NCON(4,16)=3
      ACON(2,4,16)='CB '
      ACON(3,4,16)='N  '
      ACON(4,4,16)='C  '
C     THR
      RES(17)='THR'
      NAT(17)=5
      ACON(1,1,17)='N  '
      NCON(1,17)=1
      ACON(2,1,17)='CA '
      ACON(3,1,17)='C-1'
      ACON(4,1,17)='   '
      ACON(1,2,17)='CG2'
      NCON(2,17)=5
      ACON(2,2,17)='CB '
      ACON(3,2,17)='CA '
      ACON(4,2,17)='   '
      ACON(1,3,17)='OG1'
      NCON(3,17)=7
      ACON(2,3,17)='CB '
      ACON(3,3,17)='CA '
      ACON(4,3,17)='   '
      ACON(1,4,17)='CB '
      NCON(4,17)=3
      ACON(2,4,17)='CA '
      ACON(3,4,17)='CG2'
      ACON(4,4,17)='OG1'
      ACON(1,5,17)='CA '
      NCON(5,17)=3
      ACON(2,5,17)='CB '
      ACON(3,5,17)='N  '
      ACON(4,5,17)='C  '
C     TRP
      RES(18)='TRP'
      NAT(18)=9
      ACON(1,1,18)='N  '
      NCON(1,18)=1
      ACON(2,1,18)='CA '
      ACON(3,1,18)='C-1'
      ACON(4,1,18)='   '
      ACON(1,2,18)='CH2'
      NCON(2,18)=1
      ACON(2,2,18)='CZ3'
      ACON(3,2,18)='CZ2'
      ACON(4,2,18)='   '
      ACON(1,3,18)='CZ3'
      NCON(3,18)=1
      ACON(2,3,18)='CH2'
      ACON(3,3,18)='CE3'
      ACON(4,3,18)='   '
      ACON(1,4,18)='CZ2'
      NCON(4,18)=1
      ACON(2,4,18)='CH2'
      ACON(3,4,18)='CE2'
      ACON(4,4,18)='   '
      ACON(1,5,18)='CE3'
      NCON(5,18)=1
      ACON(2,5,18)='CZ3'
      ACON(3,5,18)='CD2'
      ACON(4,5,18)='   '
      ACON(1,6,18)='NE1'
      NCON(6,18)=1
      ACON(2,6,18)='CE2'
      ACON(3,6,18)='CD1'
      ACON(4,6,18)='   '
      ACON(1,7,18)='CD1'
      NCON(7,18)=1
      ACON(2,7,18)='NE1'
      ACON(3,7,18)='CG '
      ACON(4,7,18)='   '
      ACON(1,8,18)='CB '
      NCON(8,18)=2
      ACON(2,8,18)='CA '
      ACON(3,8,18)='CG '
      ACON(4,8,18)='   '
      ACON(1,9,18)='CA '
      NCON(9,18)=3
      ACON(2,9,18)='CB '
      ACON(3,9,18)='N  '
      ACON(4,9,18)='C  '
C     TYR
      RES(19)='TYR'
      NAT(19)=8
      ACON(1,1,19)='N  '
      NCON(1,19)=1
      ACON(2,1,19)='CA '
      ACON(3,1,19)='C-1'
      ACON(4,1,19)='   '
      ACON(1,2,19)='OH '
      NCON(2,19)=7
      ACON(2,2,19)='CZ '
      ACON(3,2,19)='CE2'
      ACON(4,2,19)='   '
      ACON(1,3,19)='CE2'
      NCON(3,19)=1
      ACON(2,3,19)='CZ '
      ACON(3,3,19)='CD2'
      ACON(4,3,19)='   '
      ACON(1,4,19)='CE1'
      NCON(4,19)=1
      ACON(2,4,19)='CZ '
      ACON(3,4,19)='CD1'
      ACON(4,4,19)='   '
      ACON(1,5,19)='CD2'
      NCON(5,19)=1
      ACON(2,5,19)='CE2'
      ACON(3,5,19)='CG '
      ACON(4,5,19)='   '
      ACON(1,6,19)='CD1'
      NCON(6,19)=1
      ACON(2,6,19)='CE1'
      ACON(3,6,19)='CG '
      ACON(4,6,19)='   '
      ACON(1,7,19)='CB '
      NCON(7,19)=2
      ACON(2,7,19)='CA '
      ACON(3,7,19)='CG '
      ACON(4,7,19)='   '
      ACON(1,8,19)='CA '
      NCON(8,19)=3
      ACON(2,8,19)='CB '
      ACON(3,8,19)='N  '
      ACON(4,8,19)='C  '
C     VAL
      RES(20)='VAL'
      NAT(20)=5
      ACON(1,1,20)='N  '
      NCON(1,20)=1
      ACON(2,1,20)='CA '
      ACON(3,1,20)='C-1'
      ACON(4,1,20)='   '
      ACON(1,2,20)='CG2'
      NCON(2,20)=5
      ACON(2,2,20)='CB '
      ACON(3,2,20)='CA '
      ACON(4,2,20)='   '
      ACON(1,3,20)='CG1'
      NCON(3,20)=5
      ACON(2,3,20)='CB '
      ACON(3,3,20)='CA '
      ACON(4,3,20)='   '
      ACON(1,4,20)='CB '
      NCON(4,20)=3
      ACON(2,4,20)='CA '
      ACON(3,4,20)='CG2'
      ACON(4,4,20)='CG1'
      ACON(1,5,20)='CA '
      NCON(5,20)=3
      ACON(2,5,20)='CB '
      ACON(3,5,20)='N  '
      ACON(4,5,20)='C  '
C     DISORDER (ONLY SINGLE HYDROGENS ON N AND CA)
      RES(21)='DIS'
      NAT(21)=2
      ACON(1,1,21)='N  '
      NCON(1,21)=1
      ACON(2,1,21)='CA '
      ACON(3,1,21)='C-1'
      ACON(4,1,21)='   '
      ACON(1,2,21)='CA '
      NCON(2,21)=3
      ACON(2,2,21)='CB '
      ACON(3,2,21)='N  '
      ACON(4,2,21)='C  '
C     PLACE TO EXTEND THE LIST
      RETURN
      END
C--------------------------------------------------------------
      SUBROUTINE HYDROS(M,N1,N2,N3,N4,NOHYD,X,Y)
C     ..
C     .. Parameters ..
      INTEGER NATOMS
      PARAMETER(NATOMS=50)
C     ..
C     .. Arguments ..
      INTEGER M,N1,N2,N3,N4,NOHYD
      REAL X(3,NATOMS),Y(3,3)
C     ..
C     .. Local Scalars ..
      REAL D195,D30,D352,D547,D60,D70,DET,DET1,DET2,DET3,DH,DAR,PI,S
      INTEGER K
C     ..
C     .. Local Arrays ..
      REAL R1(3),R2(3),R3(3),R4(3)
C     ..
C     .. External Routines ..
      EXTERNAL MULT,HGENUNIT
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,SQRT
C     ..
      PI=3.14159
      DAR=PI/180.0
      DH=1.0
      D547=DH*COS(54.73*DAR)
      D352=DH*COS(35.27*DAR)
      D30=DH*COS(30.0*DAR)
      D60=DH*COS(60.0*DAR)
      D70=DH*COS(70.53*DAR)
      D195=DH*COS(19.47*DAR)
C
C---- 1. BENZENE  CH
C
      IF (M.EQ.1 .OR. M.EQ.2) THEN
C
C            ************************
        CALL HGENUNIT(X(1,N1),X(1,N2),R1)
        CALL HGENUNIT(X(1,N1),X(1,N3),R2)
C            ************************
C
        DO 110 K=1,3
          R3(K)=R1(K)+R2(K)
  110   CONTINUE
        S=1.0/SQRT(R3(1)*R3(1)+R3(2)*R3(2)+R3(3)*R3(3))
        DO 120 K=1,3
          R3(K)=S*R3(K)
  120   CONTINUE
        IF (M .EQ. 1) THEN
          DO 130 K=1,3
            Y(K,1)=X(K,N1)+DH*R3(K)
  130     CONTINUE
        ENDIF
      ENDIF
C
C---- 2. METHYLENE  CH2
C
      IF (M.EQ.2) THEN
C
C            **************
        CALL MULT(R1,R2,R4)
C            **************
C
        DO 210 K=1,3
          Y(K,1)=X(K,N1)+D547*R3(K)+D352*R4(K)
          Y(K,2)=X(K,N1)+D547*R3(K)-D352*R4(K)
  210   CONTINUE
      ENDIF
C
C---- 3. TETRAHEDRAL  CH
C
      IF (M .EQ. 3) THEN
C
C            ************************
        CALL HGENUNIT(X(1,N1),X(1,N2),R1)
        CALL HGENUNIT(X(1,N1),X(1,N3),R2)
        CALL HGENUNIT(X(1,N1),X(1,N4),R3)
C            ************************
C
C---- NOW SOLVE EQUATIONS OF ORDER 3 BY CRAMERS RULE
C
        DET=R1(1)*(R2(2)*R3(3)-R2(3)*R3(2))
     *     +R1(2)*(R2(3)*R3(1)-R2(1)*R3(3))
     *     +R1(3)*(R2(1)*R3(2)-R2(2)*R3(1))

        IF (ABS(DET).LT.0.000001) THEN
          WRITE(6,910) N1
          GOTO 900
        ENDIF

        DET1=(R2(2)*R3(3)-R2(3)*R3(2))
     *      +R1(2)*(R2(3)-R3(3))
     *      +R1(3)*(R3(2)-R2(2))
        DET2=R1(1)*(R3(3)-R2(3))
     *      +(R2(3)*R3(1)-R2(1)*R3(3))
     *      +R1(3)*(R2(1)-R3(1))
        DET3=R1(1)*(R2(1)-R3(2))
     *      +R1(2)*(R3(1)-R2(1))
     *      +(R2(1)*R3(2)-R2(2)*R3(1))
        R1(1)=DET1/DET
        R1(2)=DET2/DET
        R1(3)=DET3/DET
        S=1.0/SQRT(R1(1)*R1(1)+R1(2)*R1(2)+R1(3)*R1(3))
        DO 310 K=1,3
          Y(K,1)=X(K,N1)+DH*S*R1(K)
  310   CONTINUE
      ENDIF
C
C---- 4. PLANAR  CH2  OR  NH2
C
      IF (M .EQ. 4) THEN
C
C            ************************
        CALL HGENUNIT(X(1,N1),X(1,N2),R1)
        CALL HGENUNIT(X(1,N4),X(1,N3),R2)
        CALL MULT(R1,R2,R3)
        CALL MULT(R1,R3,R4)
C            ************************
C
        DO 410 K=1,3
          Y(K,1)=X(K,N1)+D60*R1(K)+D30*R4(K)
          Y(K,2)=X(K,N1)+D60*R1(K)-D30*R4(K)
  410   CONTINUE
      ENDIF
C
C---- 5. METHYL  CH3  IN  STAGGERED  CONFORMATION
C
      IF (M .EQ. 5) THEN
C
C            ************************
        CALL HGENUNIT(X(1,N1),X(1,N2),R1)
        CALL HGENUNIT(X(1,N3),X(1,N2),R2)
        CALL MULT(R1,R2,R3)
        CALL MULT(R1,R3,R4)
C            ************************
C
        DO 510 K=1,3
          Y(K,1)=X(K,N1)+D70*R1(K)-D195*(D60*R4(K)+D30*R3(K))
          Y(K,2)=X(K,N1)+D70*R1(K)-D195*(D60*R4(K)-D30*R3(K))
          Y(K,3)=X(K,N1)+D70*R1(K)+D195*R4(K)
  510   CONTINUE
      ENDIF
C
C---- 6. GLUCOSE  OH
C
      IF (M .EQ. 6) THEN
C
C            ************************
        CALL HGENUNIT(X(1,N1),X(1,N2),R1)
C            ************************
C
        S=0.0
        DO 610 K=1,3
          R2(K)=0.5*(X(K,N4)-X(K,N3))-X(K,N2)
          S=S+R2(K)*R2(K)
  610   CONTINUE
        S=1.0/SQRT(S)
        DO 620 K=1,3
          R2(K)=S*R2(K)
  620   CONTINUE
C
C            **************
        CALL MULT(R1,R2,R3)
        CALL MULT(R3,R1,R4)
C            **************
C
        DO 630 K=1,3
          Y(K,1)=X(K,N1)+D70*R1(K)-D195*R4(K)
  630   CONTINUE
      ENDIF
C
C---- 7. HYDROXYL  CH2-OH
C
      IF (M .EQ. 7) THEN
C
C            ************************
        CALL HGENUNIT(X(1,N1),X(1,N2),R1)
        CALL HGENUNIT(X(1,N3),X(1,N2),R2)
        CALL MULT(R2,R1,R3)
        CALL MULT(R1,R3,R4)
C            ************************
C
        DO 710 K=1,3
          Y(K,1)=X(K,N1)+D70*R1(K)-D195*R4(K)
  710   CONTINUE
      ENDIF

  900 CONTINUE
      NOHYD=1
      IF(M.EQ.2.OR.M.EQ.4) NOHYD=2
      IF(M.EQ.5) NOHYD=3

      RETURN
  910 FORMAT(' *** ERROR ***  FOR H-(C) ON ATOM NO ',I4,
     *   ' - ALL FOUR ATOMS CO-PLANAR')
      END
C------------------------------------------------------------
      SUBROUTINE HGENUNIT(X,Y,R)
C     ..
C     .. Arguments ..
      REAL R(3), X(3), Y(3)
C     ..
C     .. Locals ..
      REAL S
      INTEGER K
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC SQRT
C     ..
      S=0.0

      DO 10 K=1,3
        R(K)=X(K)-Y(K)
        S=S+R(K)*R(K)
   10 CONTINUE

      S=1.0/SQRT(S)
      DO 20 K=1,3
        R(K)=S*R(K)
   20 CONTINUE

      RETURN
      END
C--------------------------------------------------------------
      SUBROUTINE MULT(X,Y,R)
C     ..
C     .. Arguments ..
      REAL R(3), X(3), Y(3)
C     ..
C     .. Locals ..
      REAL S
      INTEGER K
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC SQRT
C     ..
      R(1)=X(2)*Y(3)-X(3)*Y(2)
      R(2)=X(3)*Y(1)-X(1)*Y(3)
      R(3)=X(1)*Y(2)-X(2)*Y(1)
      S=1.0/SQRT(R(1)*R(1)+R(2)*R(2)+R(3)*R(3))

      DO 10 K=1,3
        R(K)=S*R(K)
   10 CONTINUE

      RETURN
      END
