C
C     MAPTONA4
C     Copyright (C) 1990-1992 Phil Evans, Ian Tickle
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C*************************************************************
C
C
C**************************************************************

C----------------------------------------------------------------------
C
C Convert map between binary (CCP4 format) & formatted
C
C        Phil Evans, MRC LMB, Cambridge, December 1990
C
C        MODIFIED: to use "NA4" format.  Ian Tickle, 5-3-92
C
C  Input files:
C     MAPIN (stream 1)
C               input map, formatted (ascii) or binary CCP4 format. The program
C               reads the first line of the map to dteremine automatically
C               whether the file is formatted or not
C     SYMOP (stream 4)
C               symmetry library, required if the output file is CCP4 format
C
C  Output file:
C     MAPOUT(stream 2)
C               output map: this will be formatted if the input map is
C               binary & vice versa
C
C----------------------------------------------------------------------
C
      program maptona4
C     ================


      logical lform
      character*80 title
      integer nsec,iuvw(3),mxyz(3),nw1,nu1,nu2,nv1,nv2,lspgrp,lmode,
     &        indxr4(2)
      real cell(6),rhmin,rhmax,rhmean,rhrms

      integer maxlin
      parameter (maxlin=1000)
      integer maxrho
      parameter (maxrho=100000)
      real rho(maxrho)
      integer nsize

C Open input file & find out if it is formatted or not
C LFORM is true if input file in formatted
      CALL CCPFYP
      CALL CCPRCS(6,'MAPTONA4','$Date: 2005/09/06 11:25:16 $')
      call opnfil(lform)

C Read header information
      call rdmaphd(lform,title,nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2,
     .         cell,lspgrp,lmode,rhmin,rhmax,rhmean,rhrms,indxr4)

C Check array size
      nsize = (nu2-nu1+1)*(nv2-nv1+1)
      if (nsize .gt. maxrho) then
         write(6,*) '****** Map array too small, must be:',nsize
         call ccperr(1,' stop array')
      endif

C Check line length
      if (nu2-nu1+1 .gt. maxlin) then
         write(6,*) '****** Line buffer too small: ',maxlin
         call ccperr(1,' stop line buffer')
      endif

C Check mode
      if (lmode .ne. 2) then
         write(6,*) '****** Only mode 2 maps supported ******'
         call ccperr(1,' stop mode wrong')
      endif


C Write header information
      call wtmaphd(lform,title,nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2,
     .         cell,lspgrp,lmode,rhmin,rhmax,rhmean,rhrms,indxr4)

C Copy sections
      call cpysec(lform,rho,maxrho,nsec,nw1,nu1,nu2,nv1,nv2,indxr4)

C Close output
      call mapfin(lform,rhmin,rhmax,rhmean,rhrms)

      write(6,*) 'Last section written'

      end

      subroutine opnfil(lform)
C     ========================

C Find out if input file MAPIN is formatted or not
C Leave file closed

C LFORM is true if input file in formatted
      logical lform
      intrinsic index
      character*80 line

      lform = .false.
      KDUMMY = 80
      KFAIL = 0
      call ccpdpn(1,'MAPIN','READONLY','F',KDUMMY,KFAIL)

C Read 1st line as is formatted, should contain string MAPTONA4 if so
      read (1,6001,err=100) line
 6001 format(A)

      if (index(line,'MAPTONA4') .gt. 0) then
         lform = .true.
      else
         lform = .false.
      endif

C Close file
 100  close (unit=1)
      return 
      end


      subroutine rdmaphd(lform,title,nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2,
     .         cell,lspgrp,lmode,rhmin,rhmax,rhmean,rhrms,indxr4)
C     ==================================================================

C Read header information
C
      logical lform
      character*80 title
      integer nsec,iuvw(3),mxyz(3),nw1,nu1,nu2,nv1,nv2,lspgrp,lmode,
     &        indxr4(2)
      real cell(6),rhmin,rhmax,rhmean,rhrms,range(2)


      if (lform) then
C  Formatted, read keyworded header
         call rdfhdr(1,title,nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2,
     .         cell,lspgrp,lmode,rhmin,rhmax,rhmean,rhrms,indxr4)
      else
C  Read Map format (CCP4) header
         call mrdhdr(1,'MAPIN',title,nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2,
     .         cell,lspgrp,lmode,rhmin,rhmax,rhmean,rhrms)

C  Get indices for REAL <--> CHARACTER*4 conversion.
         range(1)=rhmin
         range(2)=rhmax
         call cvr4a4(range,indxr4)
      endif

      return
      end



      subroutine wtmaphd(lform,title,nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2,
     .         cell,lspgrp,lmode,rhmin,rhmax,rhmean,rhrms,indxr4)
C     ==================================================================

C Write header information
C
      logical lform
      character*80 title
      integer nsec,iuvw(3),mxyz(3),nw1,nu1,nu2,nv1,nv2,lspgrp,lmode,
     &        indxr4(2)
      real cell(6),rhmin,rhmax,rhmean,rhrms

      if (.not.lform) then
C  Formatted, write keyworded header
         call wtfhdr(2,title,nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2,
     .         cell,lspgrp,lmode,rhmin,rhmax,rhmean,rhrms,indxr4)
      else
C  Write Map format (CCP4) header
         call mwrhdr(2,title,nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2,
     .         cell,lspgrp,lmode)
C  Write symmetry operators to output map based on space-group
         call msyput(4,lspgrp,2)
      endif

      return
      end

      subroutine cpysec(
     .        lform,rho,maxrho,nsec,nw1,nu1,nu2,nv1,nv2,indxr4)
C     =========================================================

C Copy all sections

      logical lform
      integer maxrho
      real rho(maxrho)
      integer nsec,nw1,nu1,nu2,nv1,nv2,indxr4(2)

      integer k,nu,nv,ier

      nu = nu2-nu1+1
      nv = nv2-nv1+1

      do 1, k = nw1,nw1+nsec-1

         if (lform) then
            call rdfsec(1,ier,rho,nu,nv,indxr4)
            if (ier .ne. k) then
               write(6,*) '****** Wrong section read :',ier
               call ccperr(1,' stop section error')
            endif
            call mspew(2,rho)
         else
            call mgulp(1,rho,ier)
            call wtfsec(2,k,rho,nu,nv,indxr4)
         endif

 1    continue

      return
      end

      subroutine wtfsec(lun,isec,rho,nu,nv,indxr4)
C     ============================================

C Write formatted section to unit lun

      integer lun,isec,nu,nv,indxr4(2)
      real rho(nu,nv)

      integer maxlin
      parameter (maxlin=1000)
      character a4real*4,line(maxlin)*4
      integer i,j

      write (lun,6001) isec
 6001 format(/'SECTION',I8/)

      do 10, j=1,nv
         do 20, i=1,nu
            line(i) = a4real(indxr4,rho(i,j))
 20      continue
         write(lun,6010) (line(i),i=1,nu)
 6010    format(20A)
 10   continue
      return
      end


      subroutine rdfsec(lun,isec,rho,nu,nv,indxr4)
C     ============================================

C Read formatted section from unit lun

      integer lun,isec,nu,nv,indxr4(2)
      real rho(nu,nv)

      integer maxlin
      parameter (maxlin=1000)
      character line(maxlin)*4
      integer i,j
      real reala4

      read (lun,6001) isec
 6001 format(/7X,I8/)

      do 10, j=1,nv
         read(lun,6010) (line(i),i=1,nu)
 6010    format(20A)
         do 20, i=1,nu
            rho(i,j) = reala4(indxr4,line(i))
 20      continue
 10   continue
      return
      end


      subroutine wtfhdr(lun,title,nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2,
     .         cell,lspgrp,lmode,rhmin,rhmax,rhmean,rhrms,indxr4)
C     ===============================================================

C  Formatted, write keyworded header
      integer lun
      character*80 title
      integer nsec,iuvw(3),mxyz(3),nw1,nu1,nu2,nv1,nv2,lspgrp,lmode,
     &        indxr4(2)
      real cell(6),rhmin,rhmax,rhmean,rhrms

      integer limits(2,3)
      character*1 xyz(3)
      data xyz/'X','Y','Z'/

C Open file
      KDUMMY = 80
      KFAIL = 0
      call ccpdpn(lun,'MAPOUT','NEW','F',KDUMMY,KFAIL)
    

C Set limits on xyz
      limits(1,iuvw(1)) = nu1
      limits(2,iuvw(1)) = nu2
      limits(1,iuvw(2)) = nv1
      limits(2,iuvw(2)) = nv2
      limits(1,iuvw(3)) = nw1
      limits(2,iuvw(3)) = nw1+nsec-1

      write(lun,6000) 
 6000 format('MAPTONA4 HEADER')
      write(lun,6010) title
 6010 format('TITLE   '/A)
      write(lun,6020) xyz(iuvw(1)),xyz(iuvw(2)),xyz(iuvw(3))
 6020 format('AXIS    ',3(7X,A1))
      write(lun,6030) mxyz
 6030 format('GRID    ',3I8)
      write(lun,6040) limits
 6040 format('XYZLIM  ',6I8)
      write(lun,6050) lspgrp
 6050 format('SPACEGROUP',6X,I8)
      write(lun,6060) lmode
 6060 format('MODE    ',I8)
      write(lun,6070) cell
 6070 format('CELL    ',6F10.3)
      write(lun,6080) rhmin,rhmax,rhmean,rhrms
 6080 format('RHOLIM  ',4G16.6)
      write(lun,'(A,2I4)') 'INDXR4  ',indxr4
      write(lun,6100) 
 6100 format('END HEADER')

      return
      end


      subroutine rdfhdr(lun,title,nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2,
     .         cell,lspgrp,lmode,rhmin,rhmax,rhmean,rhrms,indxr4)
C     ===============================================================

C  Formatted, read keyworded header, fixed format
      integer lun
      character*80 title
      integer nsec,iuvw(3),mxyz(3),nw1,nu1,nu2,nv1,nv2,lspgrp,lmode,
     &        indxr4(2)
      real cell(6),rhmin,rhmax,rhmean,rhrms

      integer i,j,limits(2,3)
      character*1 xyz(3),axes(3)
      data xyz/'X','Y','Z'/
      KDUMMY = 80
      KFAIL = 0
      call ccpdpn(lun,'MAPIN','READONLY','F',KDUMMY,KFAIL)

      read(lun,'(A)') 
      write(6,6100) 
 6100 format(' MAPTONA4 HEADER')
      read(lun,'(/A)') title
      write(6,6110) title
 6110 format(' TITLE'/1X,A)
      read(lun,'(8X,3(7X,A))') axes
      write(6,6120) axes
 6120 format(' AXIS    ',3(7X,A1))
      read(lun,'(8X,3I8)') mxyz
      write(6,6130) mxyz
 6130 format(' GRID    ',3I8)
      read(lun,'(8X,6I8)') limits
      write(6,6140) limits
 6140 format(' XYZLIM  ',6I8)
      read(lun,'(16X,I8)') lspgrp
      write(6,6150) lspgrp
 6150 format(' SPACEGROUP',6X,I8)
      read(lun,'(8X,I8)') lmode
      write(6,6160) lmode
 6160 format(' MODE    ',I8)
      read(lun,'(8X,6F10.3)') cell
      write(6,6170) cell
 6170 format(' CELL    ',6F10.3)
      read(lun,'(8X,4G16.6)') rhmin,rhmax,rhmean,rhrms
      write(6,6180) rhmin,rhmax,rhmean,rhrms
 6180 format(' RHOLIM  ',4G16.6)
      read(lun,'(8X,2I4)') indxr4
      write(6,'(A,2I4)') ' INDXR4  ',indxr4
      read(lun,'(A)') 
      write(6,6195) 
 6195 format(' END HEADER')

C Get axis order
      do 10, i=1,3
         do 20, j=1,3
            if (axes(i) .eq. xyz(j)) then
               iuvw(i) = j
            endif
 20      continue
 10   continue


C Get limits 
      nu1 = limits(1,iuvw(1))
      nu2 = limits(2,iuvw(1))
      nv1 = limits(1,iuvw(2))
      nv2 = limits(2,iuvw(2))
      nw1 = limits(1,iuvw(3))
      nsec= limits(2,iuvw(3))-nw1+1

      return
      end


      subroutine mapfin(lform,rhmin,rhmax,rhmean,rhrms)
C     =================================================

C close output file

      logical lform
      real rhmin,rhmax,rhmean,rhrms

      if (.not.lform) then
         close (unit=2)
      else
         call mclosc(2,rhmin,rhmax,rhmean,rhrms)
      endif

      return
      end
C
C
C
C
      SUBROUTINE CVR4A4(RANGE, INDXR4)
C
C==== Machine-independent routine to get indices for REAL*4 <--> CHARACTER*4
C==== conversions.
C
C==== REAL array argument    :  Minimum and maximum values of series of
C==== REALs to be converted.
C
C==== INTEGER array argument :  Indices for use by A4REAL and REALA4.
C
C==== Ian Tickle, Birkbeck.
C
C
      INTEGER INDXR4(2)
      REAL RANGE(2)
      INTEGER I, J, N
      REAL A, P
C
      IF (RANGE(1).EQ.0. .AND. RANGE(2).EQ.0.) THEN
        INDXR4(1) = 0
        INDXR4(2) = 999
      ELSE
        A = MAX(ABS(RANGE(1)),ABS(RANGE(2)))
        P = 24. - LOG(DBLE(A))/LOG(2.D0)
        IF (P.LE.0.) THEN
          I = P - 1.
        ELSE
          I = P
        ENDIF
C
        N = A * 2.**I
        IF (N.LT.2**23) THEN
          I = I + 1
        ELSEIF (N.GE.2**24) THEN
          I = I - 1
        ENDIF
C
        N = 0
        DO 10 J = 1, 2
          IF (RANGE(J).LT.0.) THEN
            N = N - 1
          ELSEIF (RANGE(J).GT.0.) THEN
            N = N + 1
          ENDIF
10      CONTINUE
C
        INDXR4(1) = N
        IF (N.EQ.0) THEN
          INDXR4(2) = I - 1
        ELSE
          INDXR4(2) = I
        ENDIF
      ENDIF
      END
C
C
      CHARACTER*4 FUNCTION A4REAL(INDXR4, RVAL)
C
C
C==== Machine-independent routine for REAL*4 --> CHARACTER*4 conversion.
C
C==== INTEGER array argument :  Indices from INDXR4 function call.
C
C==== REAL argument          :  REAL value to be converted.
C
C==== CHARACTER*4 return     :  Converted string.
C====                         A **** string returned indicates overflow;
C====                         this will happen if the absolute value of
C====                         the REAL falls outside the range specified
C====                         in the call to INDXR4.
C
      INTEGER INDXR4(2)
      REAL RVAL
      CHARACTER*64 CHRTAB
      INTEGER I, J, M, N
      DATA CHRTAB/
     &'0123456789-+ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'
     &/
C==== First convert REAL to INTEGER in the range 0 to 2**24-1.
C==== Negative numbers are converted to 2's complement.
C
      IF (RVAL.EQ.0.) THEN
        A4REAL = '0000'
      ELSEIF (INDXR4(2).EQ.999) THEN
        A4REAL = '****'
      ELSEIF (INDXR4(1).NE.0 .AND.
     &        (ISIGN(1, INDXR4(1)).NE.SIGN(1., RVAL))) THEN
        A4REAL = '****'
      ELSE
        N = NINT(ABS(RVAL) * 2.**INDXR4(2))
        IF (INDXR4(1).EQ.0) THEN
          M = 2**23
        ELSE
          M = 2**24
        ENDIF
        IF (N.EQ.M) THEN
          N = M - 1
        ELSEIF (N.GT.M) THEN
          A4REAL = '****'
          RETURN
        ENDIF
C
        IF (INDXR4(1).EQ.0 .AND. RVAL.LT.0.) N = 2**24 - N
C
C==== Then convert INTEGER to CHAR*4 using 64-character translation table.
C
        DO 10 I = 4, 1, -1
          M = N/64
          J = N - 64*M + 1
          A4REAL(I:I) = CHRTAB(J:J)
10        N = M
      ENDIF
      END
C
C
      REAL FUNCTION REALA4(INDXR4, A4)
C
C==== Machine-independent routine for CHARACTER*4 --> REAL*4 conversion.
C
C==== INTEGER array argument :  Indices from INDXR4 function call.
C
C==== CHARACTER*4 argument   :  String to be converted.
C
C==== REAL return            :  REAL converted value.
C
C
      INTEGER INDXR4(2)
      CHARACTER*4 A4
      CHARACTER*64 CHRTAB
      INTEGER I, J, N
      DATA CHRTAB/
     &'0123456789-+ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'
     &/
C
C==== First convert CHAR*4 to INTEGER using 64-character translation table.
C
      N = 0
      DO 10 I = 1, 4
        J = INDEX(CHRTAB, A4(I:I))
        IF (J.EQ.0) THEN
          WRITE (6,*)
     &    '**** A4REAL ERROR: Illegal character(s) >> '//A4//' <<.'
          REALA4 = 0.
          RETURN
        ENDIF
10      N = 64*N + J - 1
C
C==== Then convert INTEGER in the range 0 to 2**24-1 to REAL.
C==== Negative numbers are 2's complement.
C
      IF (INDXR4(1).NE.0) THEN
        REALA4 = ISIGN(N, INDXR4(1)) * .5**INDXR4(2)
      ELSEIF (N.LT.2**23) THEN
        REALA4 = N * .5**INDXR4(2)
      ELSE
        REALA4 = (N - 2**24) * .5**INDXR4(2)
      ENDIF
      END
