C
C     MATTHEWS_COEF
C     Copyright (C) 1996 Misha Isupov
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C*************************************************************
C
C
C**************************************************************
C
C     =====================
      PROGRAM MATTHEWS_COEF
C     =====================
C
C     Very useful program written by Misha Isupov
C
C     Rather attractively CCP4ised by Adam (16/05/96)
C
C     Probabilities from method of Kantardjieff and Rupp,
C     Protein Science (2003), 12:1865-1871
C     incorporated CCB (9/11/04)
C
C     some parameters
C     PROTDEN protein density
C     DNADEN dna density
C     PDDEN protein/dna complex density
C     PROTSOL protein solvent content
C     DNASOL dna solvent content
C     PDSOL protein/dna complex solvent content
      REAL PROTDEN, DNADEN, PDDEN, PROTSOL, DNASOL, PDSOL
      PARAMETER (PROTDEN=1.0/0.74)
      PARAMETER (DNADEN=1.0/0.50)
      PARAMETER (PROTSOL=0.47)
      PARAMETER (DNASOL=0.64)
      PARAMETER (PDDEN=PROTDEN*0.75+DNADEN*0.25)
      PARAMETER (PDSOL=0.60)
C     MAXMOL maximum number of molecules
      INTEGER MAXMOL
      PARAMETER (MAXMOL=48)
C     ..
C     .. Local Scalars ..
      INTEGER NMOL, NUMSGP, NSYM, NSYMP, XMLFILEUNIT, NRES, IMOL
      REAL PI, RO, VOLUME, WEIGHT, ULT, RESO, DIFF
      LOGICAL CONT, AUTO, XMLOUT, LRUPP, LDNA, LCOMP
      CHARACTER PGNAME*10, SPGNAM*10, LINE*80
C     ..
C     .. Local Arrays ..
      REAL CELL(6), RSYM(4,4,192), PROB(MAXMOL), PROBT(MAXMOL),
     +     PER2(MAXMOL), CMATH(MAXMOL)
      REAL RBIN(15),P0(15),VMBAR(15),WCOEFF(15),ACOEFF(15),SCOEFF(15)
C     ..
C Mattprob parameter file  NOV 2002 release nonredundant set, 1% dV, 1% dM
C
C B.Rupp June 1, 2003
C
C Parameters are graphically fine adjusted and interpolated
C
C Rows    :       1: rbin, 2: p0, 3: vmbar, 4: w, 5: a, 6: s
C columns :       1-13 protein data inclusive to corresponding binmax value
C                 14 DNA (all resolutions)
C                 15 DNA/Protein complexes(25%/75%)

      DATA RBIN/1.199,1.501,1.650,1.801,1.901,2.001,2.201,2.401,
     +          2.601,2.801,3.101,3.501,5.001,5.001,5.001/
      DATA P0/0.085,0.312,0.400,0.503,0.597,0.729,1.052,1.781,
     +          2.852,3.386,3.841,4.281,4.592,1.503,0.257/
      DATA VMBAR/2.052,2.102,2.122,2.132,2.140,2.155,2.171,2.182,
     +           2.191,2.192,2.205,2.211,2.210,2.256,2.324/
      DATA WCOEFF/0.213,0.214,0.220,0.225,0.226,0.231,0.236,0.241,
     +       0.242,0.244,0.244,0.244,0.245,0.446,0.327/
      DATA ACOEFF/28.38,102.7,187.5,339.3,434.1,540.5,686.2,767.9,
     +       835.9,856.9,854.0,849.6,846.7,136.6,47.10/
      DATA SCOEFF/0.953,0.807,0.775,0.702,0.648,0.640,0.635,0.589,
     +       0.584,0.542,0.500,0.485,0.480,1.180,0.466/
C     ..
C     .. External Routines ..
      EXTERNAL CCPERR, CCPFYP, CCPRCS, MEMOPARSE, PARSECELL, 
     +         PARSEDIAGNOSE, PARSEINT, PARSEREAL, PARSESYMM,
     +         CCP4H_INIT, CCP4H_HEADER, CCP4H_PRE_BEG, CCP4H_PRE_END,
     +         CCP4H_TOC_BEG, CCP4H_TOC_ENT, CCP4H_TOC_END
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC COS, SQRT
C     ..
      RESO = -1.0
      NMOL = 1
      NSYM = 0
      PI=3.1415927
      CELL(1) = 0.0
      CELL(2) = 0.0
      CELL(3) = 0.0
      CELL(4) = 90.0
      CELL(5) = 90.0
      CELL(6) = 90.0
      WEIGHT = 1.0
      NRES   = 0
      AUTO = .FALSE.
      XMLOUT = .FALSE.
      XMLFILEUNIT = 0
      LINE=' '
      IBIN=0
      LDNA=.FALSE.
      LCOMP=.FALSE.
      LRUPP=.FALSE.
      DO IMOL = 1, MAXMOL
        PROB(IMOL) = 0.0
        PROBT(IMOL) = 0.0
        PER2(IMOL) = 0.0
      ENDDO
C
      CALL CCPFYP
      CALL CCP4H_INIT()
      CALL CCP4H_HEADER('Matthews_coef','matthews_coef',1)
      CALL CCP4H_PRE_BEG()
      CALL CCPRCS(6,'MATTHEWS_COEF','$Date: 2006/11/24 11:35:57 $')
      CALL CCP4H_PRE_END() 

      CALL CCP4H_TOC_BEG()
      CALL CCP4H_TOC_ENT('Reference', '#refs')
      CALL CCP4H_TOC_ENT('Possible Keywords','#poss_key')
      CALL CCP4H_TOC_ENT('Results','#results')
      CALL CCP4H_TOC_END()

      CALL CCP4H_HEADER('Reference','refs',1)
      CALL CCP4H_PRE_BEG()
      WRITE (6, 6000)
 6000 FORMAT (/,' $TEXT:Reference1: $$ comment $$ ',//,
     +'  Matthews_coef: a Very useful program written by Misha Isupov',
     +     //,' $$',/,' $SUMMARY :Reference1:  $$ Matthews_coef: $$',/,
     +     ' :TEXT:Reference1: $$')
      CALL CCP4H_PRE_END()
C
C---- Read in data cards
      CALL CCP4H_HEADER('Possible Keywords','poss_key',1)
      CALL CCP4H_PRE_BEG()
      WRITE(6,*)
     + ' Keywords:  CELL SYMM MOLW NRES NMOL AUTO RESO MODE END'
      CALL CCP4H_PRE_END()
C
      CALL CCP4H_PRE_BEG()
10    CALL MEMOPARSE(.TRUE.)
C
C  Cell
C
      CALL PARSECELL (CELL)
C
C  Symm
C
      CALL PARSESYMM(SPGNAM, NUMSGP, PGNAME, NSYM, NSYMP, RSYM)
C
C  Molecular weight
C
      CALL PARSEREAL('MOLW',WEIGHT)
C
C  Resolution
C
      CALL PARSEREAL('RESO',RESO)
C
C  Number of Residues per molecule
C
      CALL PARSEINT('NRES',NRES)
C
C  Number of Molecules per asymmetric unit
C
      CALL PARSEINT('NMOL',NMOL)
C
C  Automatically give a range of possible molecules in Asymmetric unit.
C
      CALL PARSEKEY('AUTO', AUTO)
C
C  undoucmented keyword to output an XML file assigned by XMLFILE.
C
      CALL PARSEKEY('XMLO', XMLOUT)
C
C  DNA or COMP
C
      CALL PARSEKEYARG('MODE',LINE)
      IF(LINE(1:1).EQ.'D'.or.LINE(1:1).eq.'d') LDNA=.TRUE.
      IF(LINE(1:1).EQ.'C'.or.LINE(1:1).eq.'c') LCOMP=.TRUE.
      CALL PARSEKEY('DNA ', LDNA)
      CALL PARSEKEY('COMP', LCOMP)
C
C  Test data input
C
      CALL PARSEDIAGNOSE(CONT)
      IF (CONT) GOTO 10
      CALL CCP4H_PRE_END()
C
C---- End of input keywords
C
C  IF NRES set but not MOLW then:
C   calculate MOLW from NRES assuming 5 C, 1.35 N, 1.5 O, 8 H 0.05 S atoms 
C   per residue on average
       IF(NRES.GT.0 .AND. WEIGHT.EQ.1.0) THEN
         WEIGHT =  12.0*5 + 14.0*1.35 + 16.0*1.5 +1.0*8 +32.0*0.05
         WEIGHT = WEIGHT*NRES
         write(6,*)' Molecular weight estimated from NRES is ',WEIGHT
       END IF
C
C if we have a resolution and are not doing complex or dna, then have
C resolution specific calc
      IF (.NOT.(LDNA.OR.LCOMP).AND.RESO.GT.0.0) LRUPP=.TRUE.
C set the standard solvent contents and densities
      IF (LDNA) THEN
        FRACSOL=DNASOL
        DENSITY=DNADEN
      ELSE IF (LCOMP) THEN
        FRACSOL=PDSOL
        DENSITY=PDDEN
      ELSE
        FRACSOL=PROTSOL
        DENSITY=PROTDEN
      ENDIF
C
      IF (CELL(1) .EQ. 0.0) 
     +  CALL CCPERR(1,' *** The CELL keyword has not been given ***')
      IF (NSYM .EQ. 0) 
     +  CALL CCPERR(1,' *** The SYMM keyword has not been given ***')
      IF (WEIGHT .EQ. 1.0) CALL CCPERR(3,
     +' ** Neither the MOLWEIGHT nor the number of residues given **')
C
C---- If everything OK start to calculate the Matthews coefficient 
C
      CELL(4)=CELL(4)*PI/180.0
      CELL(5)=CELL(5)*PI/180.0
      CELL(6)=CELL(6)*PI/180.0
      ULT = 1.0 + 2.0*COS(CELL(4))*COS(CELL(5))*COS(CELL(6)) - 
     +      COS(CELL(4))**2 - COS(CELL(5))**2 - COS(CELL(6))**2
      IF (ULT .LE. 0.0) CALL CCPERR(1,
     +' *** The cell volume cannot be calculated; check CELL card ***')
      VOLUME=CELL(1)*CELL(2)*CELL(3)*SQRT(ULT)

      CALL CCP4H_HEADER('Results','results',1)

      IF (XMLOUT) THEN
        CALL XMLOpen(XMLFILEUNIT, 'matthews_run', ifail)
        call XMLOpenElement(XMLFILEUNIT,'meta_data',ifail)
        call XMLWriteJournalCitation(XMLFILEUNIT,'B.W.Matthews',
     +   'J.Mol.Biol.','33',' ','491','497','1968',ifail)
        call XMLCloseElement(XMLFILEUNIT,'meta_data',ifail)
        call XMLWriteElement(XMLFILEUNIT,'keyword',' ',ifail)
      ENDIF

      IF (XMLOUT) THEN
        write (LINE,*) VOLUME
        call XMLOpenElement(XMLFILEUNIT, 'cell', ifail)
        call XMLWriteAttribute(XMLFILEUNIT, 'cell', 'volume',
     $       LINE, ifail)
        call XMLCloseElement(XMLFILEUNIT, 'cell', ifail)
      ENDIF

      WRITE(6,'(A,f15.3)')' Cell volume:   ', VOLUME
C
      IF(WEIGHT.LE.1.0) THEN
C   Assume 64% of cell volume solvent in dna crystal..
C   Assume 60% of cell volume solvent in dna/protein crystal..
C   Assume 47% of cell volume solvent in protein crytal..
        FRAC2 = FRACSOL
        RO = (1-FRAC2)*DENSITY
        WEIGHT =RO*(0.602*VOLUME)/(NMOL*NSYM)

        IF (LDNA) THEN 
          WRITE(6,'(//,2A,/,a,F12.2,a)')
     +    ' *** Assuming 64% of cell is solvent ',
     +    '(density of solvent = 1.0, density of dna = 2.0)',
     +    ' The Molecular weight is estimated as: ',weight,' Daltons.'
        ELSEIF (LCOMP) THEN 
          WRITE(6,'(//,3A,/,a,F12.2,a)')
     +    ' *** Assuming 60% of cell is solvent ',
     +'(density of solvent = 1.0, density of protein/dna = 1.35/2.0',
     +    ' with ration 75%/25% )',
     +    ' The Molecular weight is estimated as: ',weight,' Daltons.'
        ELSE
          WRITE(6,'(//,2A,/,a,F12.2,a)') 
     +    ' *** Assuming 47% of cell is solvent ',
     +    '(density of solvent = 1.0, density of protein = 1.35)', 
     +    ' The Molecular weight is estimated as: ',weight,' Daltons.'
        ENDIF
      END IF
C
      CALL CCP4H_SUMMARY_BEG()
      CALL CCP4H_PRE_BEG()

      DO IMOL=1,MAXMOL-1
        CMATH(IMOL) = VOLUME/(WEIGHT*IMOL*NSYM)
        RO=WEIGHT*IMOL*NSYM/(0.602*VOLUME)
        PER2(IMOL)=(1-RO/DENSITY)*100.0

        IF (PER2(IMOL).LE.0.0) GOTO 25

        IF (LDNA) THEN
          PROBT(IMOL) =
     +      VM_PROB(CMATH(IMOL),P0(14),VMBAR(14),WCOEFF(14),
     +              ACOEFF(14),SCOEFF(14))
        ELSEIF (LCOMP) THEN
          PROBT(IMOL) =
     +      VM_PROB(CMATH(IMOL),P0(15),VMBAR(15),WCOEFF(15),
     +              ACOEFF(15),SCOEFF(15))
        ELSE
          DO 21 IBIN=13,1,-1
            P1=P0(IBIN)
            VM1=VMBAR(IBIN)
            W1=WCOEFF(IBIN)
            A1=ACOEFF(IBIN)
            S1=SCOEFF(IBIN)
            PROBT(IMOL)=PROBT(IMOL)
     +                  +VM_PROB(CMATH(IMOL),P1,VM1,W1,A1,S1)
            IF (LRUPP) THEN
              IF ( (IBIN.EQ.13.AND.RESO.GT.RBIN(13))
     +         .OR.(IBIN.EQ.1.AND.RESO.LT.RBIN(1))
     +         .OR.(RESO.GT.RBIN(IBIN-1).AND.RESO.LT.RBIN(IBIN)) )
     +         PROB(IMOL)=VM_PROB(CMATH(IMOL),P1,VM1,W1,A1,S1)
            ENDIF
 21       CONTINUE
        ENDIF
        PROBT(MAXMOL)=PROBT(MAXMOL)+PROBT(IMOL)
        IF(LRUPP) PROB(MAXMOL)=PROB(MAXMOL)+PROB(IMOL)
      ENDDO
     
  25  CONTINUE
 
      IF (AUTO) THEN
        WRITE (6,'(/,A,F8.0)')
     +       'For estimated molecular weight ',WEIGHT
        IF (LRUPP) THEN
        WRITE (6,'(A,F4.2,A)')
     + 'Nmol/asym  Matthews Coeff  %solvent       P(',reso,
     +  ')     P(tot)'
        WRITE (6,'(A)')
     +  '_____________________________________________________________'
        ELSE
        WRITE (6,'(A)')
     + 'Nmol/asym  Matthews Coeff  %solvent       P(tot)'
        WRITE (6,'(A)')
     +  '________________________________________________'
        ENDIF 
        IMOL=1
        DO WHILE (PER2(IMOL).GT.0.0)
          IF (LRUPP) THEN
            WRITE (6,'(I3,F13.2,F17.2,2F13.2)') 
     +            IMOL, CMATH(IMOL), PER2(IMOL)
     +          , PROB(IMOL)/PROB(MAXMOL)
     +          , PROBT(IMOL)/PROBT(MAXMOL)
          ELSE
            WRITE (6,'(I3,F13.2,F17.2,F13.2)') 
     +            IMOL, CMATH(IMOL), PER2(IMOL)
     +          , PROBT(IMOL)/PROBT(MAXMOL)
          ENDIF
          IF (XMLOUT) THEN
            call XMLOpenElement (XMLFILEUNIT, 'result', ifail)
            write (LINE,*) IMOL
            call XMLWriteAttribute (XMLFILEUNIT, 'result',
     $           'nmol_in_asu', LINE, ifail)
            write (LINE,*) CMATH(IMOL)
            call XMLWriteAttribute (XMLFILEUNIT, 'result',
     +           'matth_coef', LINE, ifail)
            write (LINE,*) PER2(IMOL)
            call XMLWriteAttribute (XMLFILEUNIT, 'result',
     +           'percent_solvent', LINE, ifail)
              write(line,*) PROBT(IMOL)/PROBT(MAXMOL)
            call XMLWriteAttribute (XMLFILEUNIT, 'result',
     +           'prob_matth', LINE, ifail)
            call XMLCloseElement (XMLFILEUNIT, 'result', ifail)
          ENDIF
          IMOL=IMOL+1
        ENDDO
        IF (LRUPP) THEN
        WRITE (6,'(A)')
     +  '_____________________________________________________________'
        ELSE
        WRITE (6,'(A)')
     +   '________________________________________________'
        ENDIF
      ELSE
        WRITE(6,'(//,A,F6.2)') ' The Matthews Coefficient is :',
     +        CMATH(NMOL)
        WRITE(6,'(A,F4.2,A,F6.2)') 
     +       ' Assuming protein density/other is ',DENSITY,
     +       ' the solvent % is:',PER2(NMOL)
        IF (XMLOUT) THEN
            call XMLOpenElement (XMLFILEUNIT, 'result', ifail)
            write (LINE,*) NMOL
            call XMLWriteAttribute (XMLFILEUNIT, 'result',
     $           'nmol_in_asu', LINE, ifail)
            write (LINE,*) CMATH(NMOL)
            call XMLWriteAttribute (XMLFILEUNIT, 'result',
     +           'matth_coef', LINE, ifail)
            write (LINE,*) PER2(NMOL)
            call XMLWriteAttribute (XMLFILEUNIT, 'result',
     +           'percent_solvent', LINE, ifail)
            write(line,*) PROBT(NMOL)/PROBT(MAXMOL)
            call XMLWriteAttribute (XMLFILEUNIT, 'result',
     +           'prob_matth', LINE, ifail)
            call XMLCloseElement (XMLFILEUNIT, 'result', ifail)
        ENDIF
      ENDIF

      CALL CCP4H_PRE_END()
      CALL CCP4H_SUMMARY_END()
      IF (XMLOUT) THEN
        call XMLWriteMessages(XMLFILEUNIT,0,'Normal Termination',ifail)
        CALL XMLCloseElement(XMLFILEUNIT, 'matthews_run', ifail)
      ENDIF

      CALL CCPERR(0,'Normal Termination')
      END

      REAL FUNCTION VM_PROB(X,YO,XC,WT,A,S)
      REAL X,YO,XC,WT,A,Z,S
      Z=(X-XC)/WT
      VM_PROB=YO+A*(EXP(-EXP(-Z)-Z*S+1))
      RETURN
      END
