void vnorm(float v[3]);
void cross(float u[3], float v[3], float w[3]);
void transpose(float a[3][3], float at[3][3]);
void matmult(float rmat1[3][3], float rmat[3][3], float prod[3][3]);
void transform(float rmat[3][3], float xold[3], float t[3], float xnew[3]);
float dot(float u[3], float v[3]);
float vecabs(float v[3]);

int do_mutate(PCResidue residue, char target_type, int mode);
int max_mutate(PCResidue residue, char target_type);
void read_clustal(FILE *filein, std::string &model_seq, std::string &target_seq, int filetype);
void read_phylip(FILE *filein, std::string &model_seq, std::string &target_seq);
void read_blast(FILE *filein, std::string &model_seq, std::string &target_seq);
void read_pir(FILE *filein, std::string &model_seq, std::string &target_seq); 
void read_oca(FILE *filein, std::string &model_seq, std::string &target_seq); 

void basic_align(std::string &model_seq, std::string &pdb_seq, int &nblanks);
void backtrack(int i, int j, int k, int g, std::string &model_seq, std::string &pdb_seq, char *B0, char *B1, int **h, 
			   int &mingaps, std::string &bestB0, std::string &bestB1);
void countgaps(int k, char *B0, char *B1, int &ngaps);
void revstring(std::string &str);
void print_alignment(std::string &model_seq, std::string &target_seq, int itarget);
void print_target(std::string &target_seq, int itarget);


char *RR[21][22][7] = {
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // Tg ALA
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 115 PHE
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 120 TYR
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG ",  " CD ",  " NE ",  " CZ ",  " NH1",  " NH2",   // Tg ARG
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  " CD ",  " NE ",  " CZ ",  " NH1",  " NH2",   // 102 ARG
  " CB ",  " CG ",  " OD1",  " ND2",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  " OD1",  " OD2",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  " CD ",  " NE2",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  " ND1",  " CE1",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  " CD1",  " CG2",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  " CD ",  " CE ",  " NZ ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  " SD ",  " CE ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  " CD1",  " CE1",  " CZ ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  " CD1",  " NE1",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  " CD1",  " CE1",  " CZ ",  " OH ",  "    ",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG ",  " OD1",  " ND2",  "    ",  "    ",  "    ",   // Tg ASN
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  " OD1",  " ND2",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  " OD1",  " OD2",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  " ND1",  " CD2",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  " SD ",  "    ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " CG2",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG ",  " OD1",  " OD2",  "    ",  "    ",  "    ",   // Tg ASP
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  " OD1",  " ND2",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  " OD1",  " OD2",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  " ND1",  " CD2",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  " SD ",  "    ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " CG2",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // Tg CYS
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // Tg CYH
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG ",  " CD ",  " OE1",  " NE2",  "    ",  "    ",   // Tg GLN
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  " CD ",  " NE ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  " OD1",  "    ",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  " OD1",  "    ",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  " CD ",  " OE1",  " NE2",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  " CD ",  " OE1",  " OE2",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  " ND1",  " CE1",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  " CD ",  " CE ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  " SD ",  " CE ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  " CD1",  " CE1",  "    ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " CG2",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  " CD1",  " NE1",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  " CD1",  " CE1",  " CZ ",  " CE2",  " CD2",   // 120 TYR
  " CB ",  " CG1",  " CG2",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG ",  " CD ",  " OE1",  " OE2",  "    ",  "    ",   // Tg GLU
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  " CD ",  " NE ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  " OD1",  "    ",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  " OD1",  "    ",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  " CD ",  " OE1",  " NE2",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  " CD ",  " OE1",  " OE2",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  " ND1",  " CE1",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  " CD ",  " CE ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  " SD ",  " CE ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  " CD1",  " CE1",  "    ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  " CG2",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  " CD1",  " NE1",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  " CD1",  " CE1",  "    ",  "    ",  "    ",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // Tg GLY
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 102 ARG
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 103 ASN
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 104 ASP
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 106 GLN
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 109 HIS
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 111 LEU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 112 LYS
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 113 MET
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 115 PHE
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 119 TRP
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 120 TYR
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG ",  " ND1",  " CE1",  " NE2",  " CD2",  "    ",   // Tg HIS
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  " CD ",  " NE ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  " OD1",  " ND2",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  " OD1",  " OD2",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  " ND1",  " CE1",  " NE2",  " CD2",  "    ",   // 109 HIS
  " CB ",  " CG1",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  " CD ",  " CE ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  " SD ",  " CE ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  " CD1",  " CE1",  " CE2",  " CD2",  "    ",   // 115 PHE
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  " CD1",  " NE1",  " CE2",  " CD2",  "    ",   // 119 TRP
  " CB ",  " CG ",  " CD1",  " CE1",  " CE2",  " CD2",  "    ",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG1",  " CD1",  " CG2",  "    ",  "    ",  "    ",   // Tg ILE
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  " OD1",  "    ",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  " OD1",  "    ",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  " ND1",  "    ",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  " CD1",  " CG2",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  " SD ",  "    ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  " CG2",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 120 TYR
  " CB ",  " CG1",  " CG2",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // Tg LEU
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  " OD1",  " ND2",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  " OD1",  " OD2",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  " ND1",  " CD2",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  " SD ",  "    ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  " CD1",  " CD2",  "    ",  "    ",  "    ",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG ",  " CD ",  " CE ",  " NZ ",  "    ",  "    ",   // Tg LYS
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  " CD ",  " NE ",  " CZ ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  " OD1",  "    ",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  " OD1",  "    ",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  " ND1",  " CE1",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  " CD ",  " CE ",  " NZ ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  " SD ",  " CE ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  " CD1",  " CE1",  " CZ ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  " CD1",  " NE1",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  " CD1",  " CE1",  " CZ ",  "    ",  "    ",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG ",  " SD ",  " CE ",  "    ",  "    ",  "    ",   // Tg MET
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  " CD ",  " NE ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  " OD1",  "    ",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  " OD1",  "    ",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  " ND1",  " CE1",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  " CD ",  " CE ",  " NZ ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  " SD ",  " CE ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  " CD1",  " CE1",  " CZ ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  " CD1",  " NE1",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  " CD1",  " CE1",  "    ",  "    ",  "    ",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG ",  " CD1",  " CE1",  " CZ ",  " CE2",  " CD2",   // Tg PHE
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  " CD ",  " NE ",  " CZ ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  " OD1",  " ND2",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  " OD1",  " OD2",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  " ND1",  " CE1",  "    ",  " NE2",  " CD2",   // 109 HIS
  " CB ",  " CG1",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  "    ",  " CD2",   // 111 LEU
  " CB ",  " CG ",  " CD ",  " CE ",  " NZ ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  " SD ",  " CE ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  " CD1",  " CE1",  " CZ ",  " CE2",  " CD2",   // 115 PHE
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  " CD1",  " NE1",  "    ",  " CE2",  " CD2",   // 119 TRP
  " CB ",  " CG ",  " CD1",  " CE1",  " CZ ",  " CE2",  " CD2",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // Tg PRO
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  " OD1",  "    ",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  " OD1",  "    ",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  " ND1",  "    ",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  " SD ",  "    ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // Tg SER
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " OG1",  " CG2",  "    ",  "    ",  "    ",  "    ",   // Tg THR
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  " CG2",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  " CG2",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 120 TYR
  " CB ",  " CG1",  " CG2",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG ",  " CD1",  " NE1",  " CE2",  " CD2",  " CE3",   // Tg TRP
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  " CD ",  " NE ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  " OD1",  "    ",  "    ",  " ND2",  "    ",   // 103 ASN
  " CB ",  " CG ",  " OD1",  "    ",  "    ",  " OD2",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  " CD ",  " OE1",  " NE2",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  " CD ",  " OE1",  " OE2",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  " ND1",  " CE1",  " NE2",  " CD2",  "    ",   // 109 HIS
  " CB ",  " CG1",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  " CD2",  "    ",   // 111 LEU
  " CB ",  " CG ",  " CD ",  " CE ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  " SD ",  " CE ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  " CD1",  " CE1",  " CE2",  " CD2",  "    ",   // 115 PHE
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  " CD1",  " NE1",  " CE2",  " CD2",  " CE3",   // 119 TRP
  " CB ",  " CG ",  " CD1",  " CE1",  " CE2",  " CD2",  "    ",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG ",  " CD1",  " CE1",  " CZ ",  " CE2",  " CD2",   // Tg TYR
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  " CD ",  " NE ",  " CZ ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  " OD1",  " ND2",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  " OD1",  " OD2",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  " CD ",  " OE1",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  " ND1",  " CE1",  "    ",  " NE2",  " CD2",   // 109 HIS
  " CB ",  " CG1",  " CD1",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  " CD1",  "    ",  "    ",  "    ",  " CD2",   // 111 LEU
  " CB ",  " CG ",  " CD ",  " CE ",  " NZ ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  " SD ",  " CE ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  " CD1",  " CE1",  " CZ ",  " CE2",  " CD2",   // 115 PHE
  " CB ",  " CG ",  " CD ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  " CD1",  " NE1",  "    ",  " CE2",  " CD2",   // 119 TRP
  " CB ",  " CG ",  " CD1",  " CE1",  " CZ ",  " CE2",  " CD2",   // 120 TYR
  " CB ",  " CG1",  "    ",  "    ",  "    ",  "    ",  "    ",   // 121 VAL

  " CB ",  " CG1",  " CG2",  "    ",  "    ",  "    ",  "    ",   // Tg VAL
  " CB ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 101 ALA
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 102 ARG
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 103 ASN
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 104 ASP
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYS
  " CB ",  " SG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 105 CYH
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 106 GLN
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 107 GLU
  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 108 GLY
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 109 HIS
  " CB ",  " CG1",  " CG2",  "    ",  "    ",  "    ",  "    ",   // 110 ILE
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 111 LEU
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 112 LYS
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 113 MET
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 115 PHE
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 116 PRO
  " CB ",  " OG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 117 SER
  " CB ",  " OG1",  " CG2",  "    ",  "    ",  "    ",  "    ",   // 118 THR
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 119 TRP
  " CB ",  " CG ",  "    ",  "    ",  "    ",  "    ",  "    ",   // 120 TYR
  " CB ",  " CG1",  " CG2",  "    ",  "    ",  "    ",  "    "};  // 121 VAL

