C
C     MTZMNF
C     Copyright (C) 1996 Eleanor Dodson, Martyn Winn
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
C     Authors:  EJD, MDW
C
C  mtzmnf.f
C STANDARD ITEM NAMES taken from $CDOC/mtzlib.doc
C        Standard names are normally used for the following items:
C
C       Name          Item
C
C       H, K, L        Miller indices.
C   .....
C       I              Intensity.
C       SIGI           sigma(I).
C   .....
C       FP             Native 'F' value.
C       FC             Calculated 'F'.
C       FPHn           'F' value for derivative 'n'.
C
C       DP             Anomalous difference for native data.
C       DPHn           Anomalous difference for derivative 'n'.
C
C       SIGFP          sigma(FP).
C       SIGDP          sigma(DP).
C       SIGFPHn        sigma(Fn).
C       SIGDPHn        sigma(DELn).
C
C       PHIC           Calculated Phase.
C       PHIM           Most probable phase.
C       PHIB           Phase.
C
C       FOM            figure of merit.
C       WT             weight
C
C       HLA            ABCD H/L coefficients
C       HLB
C       HLC
C       HLD


CCOLUMN TYPES
C       The column types are as follows:
C
C       H       index h,k,l
C       J       intensity
C       F       structure amplitude, F
C       D       anomalous difference
C       Q       standard deviation of anything: J,F,D or other
C       P       phase angle in degrees
C       W       weight (of some sort )
C       A       phase probability coefficients (Hendrickson/Lattman)
C   .....
C       I       any other integer
C       R       any other real

C*************************************************************************************
C   We can change F or I to Nans if SIGF or SIGI = 0.0
C   We can change D      to Nan  if SIGD = 0.0 and the reflection is not centric
C   We can change FC and PHIC to Nan if FC =0.0  ( not very usual though I hope..)
C   We can change PHIB and HLA HLB HLC HLD to Nan if associated WT = 0.0
C   Nothing else is safe.
C*************************************************************************************


C---------  description of Key_Worded Input --------------
C           ================================
C
C
C  (i)   TITLE      (optional)
C
C        Replaces title in MTZ file
C
C  (ii)  LABIN      (compulsory)
C
C        Examples:
C           LABI F1=FTOXD3 SIGF1=SIGFTOXD3 -
C                F2=FAU20  SIGF2=SIGFAU20 -
C                D2=ANAU20 SIGD2=SIGANAU20 -
C
C  (iii) END      (optional)
C
C        Ends keyword input
C
      PROGRAM MAIN
C     ..
C     .. Parameters ..
      INTEGER MCOLS,NKEYWD,LSYM
      PARAMETER (MCOLS=500,NKEYWD=200,LSYM=192)
C     ..
C     .. Local Arrays ..
      REAL ADATA(MCOLS),BDATA(MCOLS),RSYML(4,4,LSYM)
      INTEGER LOOKUP(MCOLS),IN(3),NCHANGES(MCOLS)
      CHARACTER*1 CTPRGI(MCOLS),CTUSRI(MCOLS)*1
      CHARACTER*30 LSPRGI(MCOLS),CLABS(MCOLS)
      LOGICAL LOGMSS(MCOLS),LMSSOLD(MCOLS)
C     ..
C     .. Local Scalars ..
      REAL S
      INTEGER I,IC,IDUM,IFAIL,IFIL1,IFLAG,IPRINT,JDO30,JDO35,JDO40,JDO60
      INTEGER JDO65,JDO70,JDO80,JDO90,LUNOUT,NCOL,NF1,NLPRGI,NSPGRP
      INTEGER NSYM,NSYMP,NUMFO,NUMFC,NUMI,NUMPH,NTOK,STARTFO,STARTFC
      INTEGER STARTI,STARTPH
      CHARACTER NAMSPG*10,PGNAM*10,TITLE*80
      CHARACTER LTYPEX*1
      LOGICAL ENDFIL,LTITLE
C     ..
C     .. Variables for Parser ..
      REAL FVALUE(NKEYWD)
      INTEGER IBEG(NKEYWD),IDEC(NKEYWD),IEND(NKEYWD),ITYPE(NKEYWD)
      CHARACTER KEY*4,CVALUE(NKEYWD)*4,LINE*2000
      LOGICAL LEND
C     ..
C     .. External Routines ..
      INTEGER LUNSTO
      EXTERNAL CCPERR,CCPFYP,CCPRCS,CENTR,CENTRIC,EQUAL_MAGIC,LKYIN,
     +         LROPEN,LRASSN,LRCLAB,LRCLOS,LRREFL,LRREFM,LRSYMI,LRSYMM,
     +         LWOPEN,LWREFL,LUNSTO,MTZINI,PARSER
C     ..
C     .. Data Statements ..
      DATA NLPRGI/133/
      DATA LTITLE/.FALSE./
C  Allow NUMFO sets of Fs
C  Up to NUMFC FCs
C  Up to NUMI Is
C  Up to NUMPH phase sets.
      DATA NUMFO,NUMFC,NUMI,NUMPH/20,5,5,5/
      DATA LSPRGI/'H','K','L',
     +      'F1' ,'SIGF1' ,'D1' ,'SIGD1' ,
     +      'F2' ,'SIGF2' ,'D2' ,'SIGD2' ,
     +      'F3' ,'SIGF3' ,'D3' ,'SIGD3' ,
     +      'F4' ,'SIGF4' ,'D4' ,'SIGD4' ,
     +      'F5' ,'SIGF5' ,'D5' ,'SIGD5' ,
     +      'F6' ,'SIGF6' ,'D6' ,'SIGD6' ,
     +      'F7' ,'SIGF7' ,'D7' ,'SIGD7' ,
     +      'F8' ,'SIGF8' ,'D8' ,'SIGD8' ,
     +      'F9' ,'SIGF9' ,'D9' ,'SIGD9' ,
     +      'F10','SIGF10','D10','SIGD10',
     +      'F11','SIGF11','D11','SIGD11',
     +      'F12','SIGF12','D12','SIGD12',
     +      'F13','SIGF13','D13','SIGD13',
     +      'F14','SIGF14','D14','SIGD14',
     +      'F15','SIGF15','D15','SIGD15',
     +      'F16','SIGF16','D16','SIGD16',
     +      'F17','SIGF17','D17','SIGD17',
     +      'F18','SIGF18','D18','SIGD18',
     +      'F19','SIGF19','D19','SIGD19',
     +      'F20','SIGF20','D20','SIGD20',
     +      'FC1','PHIC1' ,
     +      'FC2','PHIC2' ,
     +      'FC3','PHIC3' ,
     +      'FC4','PHIC4' ,
     +      'FC5','PHIC5' ,
     +      'I1' ,'SIGI1' ,
     +      'I2' ,'SIGI2' ,
     +      'I3' ,'SIGI3' ,
     +      'I4' ,'SIGI4' ,
     +      'I5' ,'SIGI5' ,
     +      'PHIB1','FOM1','HLA1','HLB1','HLC1','HLD1',
     +      'PHIB2','FOM2','HLA2','HLB2','HLC2','HLD2',
     +      'PHIB3','FOM3','HLA3','HLB3','HLC3','HLD3',
     +      'PHIB4','FOM4','HLA4','HLB4','HLC4','HLD4',
     +      'PHIB5','FOM5','HLA5','HLB5','HLC5','HLD5',
     +     367*' '/
      DATA CTPRGI/'H','H','H',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','P',
     +      'F','P',
     +      'F','P',
     +      'F','P',
     +      'F','P',
     +      'J','Q',
     +      'J','Q',
     +      'J','Q',
     +      'J','Q',
     +      'J','Q',
     +      'P','W','A','A','A','A',
     +      'P','W','A','A','A','A',
     +      'P','W','A','A','A','A',
     +      'P','W','A','A','A','A',
     +      'P','W','A','A','A','A',
     +     367*' '/
      DATA LOOKUP/-1,-1,-1,0,0,0,0,0,0,0,0,489*0/
      DATA NCHANGES/MCOLS*0/
C 
C     Starting indices of program label blocks
      STARTFO=4
      STARTFC=NUMFO*4 + STARTFO
      STARTI=NUMFC*2 + STARTFC
      STARTPH=NUMI*2 + STARTI
      LUNOUT=LUNSTO(IDUM)
      IFIL1 = 1

      CALL CCPFYP
      CALL MTZINI
      CALL CCPRCS(6,'MTZMNF','$Date: 2005/09/06 11:25:18 $')

C     ...Read keyword input
 20   CONTINUE
      KEY  = ' '
      LINE = ' '
C     Reset number of words on line from 20 to NKEYWD
C     Need lots for LABI lines
      NTOK = NKEYWD
C
C          **********************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYPE,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +     .TRUE.)
C          **********************************************************
C
      IF (LEND) GO TO 200

      IF(KEY.EQ.'TITL') THEN
         IF (NTOK.GE.2) THEN
            LTITLE = .TRUE.
            TITLE = LINE(IBEG(2) :IEND(NTOK))
            WRITE(LUNOUT,'(10X,a,/,15X,a80,/)')   
     +                        'TITLE OF MTZ FILE REPLACED BY:',TITLE
         ENDIF
         GO TO 20

      ELSE IF(KEY.EQ.'LABI') THEN
C----    LABIN     i.e. set input labels
C        *****************************************
         CALL LKYIN(IFIL1,LSPRGI,NLPRGI,NTOK,LINE,IBEG,IEND)
C        *****************************************
         IF (NTOK.EQ.1) CALL CCPERR(1,
     +           'No columns assigned. Nothing to do. Bye-bye!')
         WRITE(LUNOUT,'(10X,I3,X,a,/,10X,a,a,/)')   
     +        (NTOK-1)/2, 'column labels have been assigned.',
     +       'NOTE: missing number analysis only performed ',
     +       'for these columns!!'
         GO TO 20

      ELSE IF(KEY.EQ.'END') THEN
         WRITE(LUNOUT,'(/)')   
         GO TO 200
      ELSE 
         Call ccperr(1,'  Key word unrecognised')
      END IF
C
C----   End of keyword input
 200  CONTINUE
C
      IPRINT = 2
C     **********************************
      CALL LROPEN(IFIL1,'HKLIN',IPRINT,IFAIL)
C     **********************************
      IF(IFAIL.NE.0) THEN
         CALL CCPERR(1, ' No input file???????')
      END IF
C
C     ****************************************
      CALL LRASSN(IFIL1,LSPRGI,NLPRGI,LOOKUP,CTPRGI)
C     *****************************************
C
      NCOL = MCOLS
C     =========================================
      CALL LRCLAB(IFIL1,CLABS,CTUSRI,NCOL)
C     =========================================
C
C     Test correct column types.
C
      Do 205 I = 1,NCOL
         IF (LOOKUP(I).NE.0) THEN
            NF1 = LOOKUP(I)
            IF (CTUSRI(NF1).NE.CTPRGI(I) ) THEN
C     Allow F+/- in place of F, or I+/- in place of I
              IF ((CTUSRI(NF1).EQ.'G'.AND.CTPRGI(I).EQ.'F') .OR.
     +            (CTUSRI(NF1).EQ.'K'.AND.CTPRGI(I).EQ.'J') .OR.
     +            (CTUSRI(NF1).EQ.'L'.AND.CTPRGI(I).EQ.'Q') .OR.
     +            (CTUSRI(NF1).EQ.'M'.AND.CTPRGI(I).EQ.'Q')) GOTO 205
              WRITE (LUNOUT,FMT='(A,/,I3,3x,A20,A4,3x,A20,A4)')
     +        ' ** Column type does not match assignment **',
     +        I,LSPRGI(I), CTPRGI(I),CLABS(NF1),CTUSRI(NF1)
              IF (CTUSRI(NF1).EQ.'R'.OR.CTUSRI(NF1).EQ.'I') THEN
                WRITE (LUNOUT,FMT='(/,A,A,/)')
     +          ' Warning: - Input column type is REAL or INTEGER ',
     +          '- it has been reset to program assignment.'
              ELSE
                CALL CCPERR(1,
     +          ' Column type does not match assignment')
              ENDIF
            END IF 
         END IF
205   CONTINUE
C
C     Test that SIGF present if F specified, etc, etc
C     Test F present if D specified.
C
      NF1 = 0
      DO 30 JDO30 = STARTFO,STARTFO+(NUMFO-1)*4,4
         NF1 = NF1 + 1
         IF (LOOKUP(JDO30).NE.0 .NEQV. LOOKUP(JDO30+1).NE.0) THEN
            WRITE (LUNOUT,FMT='(/,A,I2,A,I2,A,/)')
     +    'Unmatched F',NF1,' or SIGF',NF1,' assigned.'
            CALL CCPERR(1,' Unmatched F/SIGF pair')
         ENDIF 
         IF (LOOKUP(JDO30+2).NE.0 .NEQV. LOOKUP(JDO30+3).NE.0) THEN
            WRITE (LUNOUT,FMT='(/,A,I2,A,I2,A,/)')
     +    'Unmatched D',NF1,' or SIGD',NF1,' assigned.'
            CALL CCPERR(1,' Unmatched D/SIGD pair')
         ENDIF 
         IF (LOOKUP(JDO30+2).NE.0 .AND. LOOKUP(JDO30).EQ.0) THEN
            WRITE (LUNOUT,FMT='(/,A,I2,A,I2,/)')
     +    'D',NF1,' assigned but no corresponding F',NF1
            CALL CCPERR(1,' Unmatched F/D pair')
         ENDIF 
 30   CONTINUE

      NF1 = 0
      DO 35 JDO35 = STARTI,STARTI+(NUMI-1)*2,2
         NF1 = NF1 + 1
         IF (LOOKUP(JDO35).NE.0 .NEQV. LOOKUP(JDO35+1).NE.0) THEN
            WRITE (LUNOUT,FMT='(/,A,I2,A,I2,A,/)')
     +    'Unmatched I',NF1,' or SIGI',NF1,' assigned.'
            CALL CCPERR(1,' Unmatched I/SIGI pair')
         ENDIF 
 35    CONTINUE

      CALL LRSYMI(IFIL1,NSYMP,LTYPEX,NSPGRP,NAMSPG,PGNAM)
      CALL LRSYMM(IFIL1,NSYM,RSYML)
      CALL CENTRIC(NSYM,RSYML,IPRINT)
C
C     **********************
      CALL LWOPEN(IFIL1,'HKLOUT')
C     **********************
C
C---- Iflag = 0 means replace old title.
C
      IFLAG = 0
      IF (LTITLE) CALL LWTITL(IFIL1,TITLE,IFLAG)
C
      CALL EQUAL_MAGIC(IFIL1,ADATA,NCOL)
C
C---- Loop for reflections----------------------------------------------------
C
 10   CONTINUE
C
C     **************************
      CALL LRREFL(IFIL1,S,ADATA,ENDFIL)
      IF (ENDFIL) GO TO 100
      CALL LRREFM(IFIL1,LOGMSS)
C     **************************
C
      IN(1) = NINT(ADATA(1))
      IN(2) = NINT(ADATA(2))
      IN(3) = NINT(ADATA(3))
C     Test whether a reflection is centric (return ic=1)
C     or not (ic=0) - needed for Anom observations
      CALL CENTR(IN,IC)
C
C     default is to leave columns unchanged
      DO 40 JDO40 = 1,NCOL
         BDATA(JDO40) = ADATA(JDO40)
         LMSSOLD(JDO40) = LOGMSS(JDO40)
 40   CONTINUE   
C
C     ...Loop over columns in program order
C
C     The following checks are made:
C     1. Is program column of correct type (should be!)? 
C        That program and file columns have same type was checked earlier.
C     2. Is a file column assigned to this program column?
C     3. Are file columns assigned to associated columns?
C        Some checked earlier.
C        If fails, LOOKUP is changed so that fails on 2. next time around
C        (marginally more efficient).
C     4. If column already assigned as missing, make sure associated columns
C        also missing.
C     5. Finally, do specific test for this type of column.
C
C
C     case (a): F, SIGF
      DO 60 JDO60 = STARTFO+1,STARTFO+NUMFO*4-3,4
         IF (CTPRGI(JDO60).NE.'Q'.AND.CTPRGI(JDO60).NE.'L')
     +         CALL CCPERR(1,'Not a standard deviation')
C     ...if SIGF column not assigned via LABIN, leave unchanged
C     ...if SIGF assigned then so is F - checked earlier.
         IF(LOOKUP(JDO60).EQ.0) GO TO 60
C     ...if SIGF already flagged as "Missing" 
         IF(LOGMSS(LOOKUP(JDO60))) THEN
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO60-1)),1) 
            LOGMSS(LOOKUP(JDO60-1)) = .TRUE.
           GO TO 60
         ENDIF
C     ...if F already flagged as "Missing" 
         IF(LOGMSS(LOOKUP(JDO60-1))) THEN
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO60)),1) 
            LOGMSS(LOOKUP(JDO60)) = .TRUE.
           GO TO 60
         ENDIF
C     ...test for SIGF = 0.0
         IF (ADATA(LOOKUP(JDO60)).LE.1.E-30) THEN
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO60-1)),1) 
            LOGMSS(LOOKUP(JDO60-1)) = .TRUE.
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO60)),1) 
            LOGMSS(LOOKUP(JDO60)) = .TRUE.
         ENDIF
 60   CONTINUE
 
C     case (b): D, SIGD
      DO 65 JDO65 = STARTFO+3,STARTFO+NUMFO*4-1,4
         IF (CTPRGI(JDO65).NE.'Q')
     +         CALL CCPERR(1,'Not a standard deviation')
C     ...if SIGD column not assigned via LABIN, leave unchanged
C     ...if SIGD assigned then so is D - checked earlier.
         IF(LOOKUP(JDO65).EQ.0) GO TO 65
C     ...if SIGD already flagged as "Missing"
         IF(LOGMSS(LOOKUP(JDO65))) THEN
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO65-1)),1) 
            LOGMSS(LOOKUP(JDO65-1)) = .TRUE.
            GO TO 65
         ENDIF
C     ...if D already flagged as "Missing"
         IF(LOGMSS(LOOKUP(JDO65-1))) THEN
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO65)),1) 
            LOGMSS(LOOKUP(JDO65)) = .TRUE.
            GO TO 65
         ENDIF
C     ...if F (and thus SIGF) already flagged as "Missing"
         IF(LOGMSS(LOOKUP(JDO65-3))) THEN
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO65-1)),1) 
            LOGMSS(LOOKUP(JDO65-1)) = .TRUE.
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO65)),1) 
            LOGMSS(LOOKUP(JDO65)) = .TRUE.
            GO TO 65
         ENDIF
C     ...test for SIGD = 0.0
C     if reflection is centric, test does not apply
         IF (IC.NE.1) THEN
            IF (ADATA(LOOKUP(JDO65)).LE.1.E-30) THEN
               CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO65-1)),1) 
               LOGMSS(LOOKUP(JDO65-1)) = .TRUE.
               CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO65)),1) 
               LOGMSS(LOOKUP(JDO65)) = .TRUE.
            ENDIF
         ENDIF
 65   CONTINUE
 
C     case (c): FC, SIGFC
      DO 70 JDO70 = STARTFC,STARTFC+NUMFC*2-2,2
         IF (CTPRGI(JDO70).NE.'F')
     +         CALL CCPERR(1,'Not a structure factor')
C     ...if column not assigned via LABIN, leave unchanged
         IF(LOOKUP(JDO70).EQ.0) GO TO 70
         IF(LOOKUP(JDO70+1).EQ.0) THEN
            LOOKUP(JDO70)=0
            GO TO 70
         ENDIF
C     ...if FC already flagged as "Missing" 
         IF(LOGMSS(LOOKUP(JDO70))) THEN
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO70+1)),1) 
            LOGMSS(LOOKUP(JDO70+1)) = .TRUE.
            GO TO 70
         ENDIF
C     ...if SIGFC already flagged as "Missing" 
         IF(LOGMSS(LOOKUP(JDO70+1))) THEN
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO70)),1) 
            LOGMSS(LOOKUP(JDO70)) = .TRUE.
            GO TO 70
         ENDIF
C     ...test for FC = 0.0
         IF (ADATA(LOOKUP(JDO70)).LE.1.E-30) THEN
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO70)),1) 
            LOGMSS(LOOKUP(JDO70)) = .TRUE.
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO70+1)),1) 
            LOGMSS(LOOKUP(JDO70+1)) = .TRUE.
         ENDIF
 70   CONTINUE
 
C     case (d): I, SIGI
      DO 80 JDO80 = STARTI+1,STARTI+NUMI*2-1,2
         IF (CTPRGI(JDO80).NE.'Q'.AND.CTPRGI(JDO60).NE.'M')
     +         CALL CCPERR(1,'Not a standard deviation')
C     ...if SIGI not assigned via LABIN, leave unchanged
C     ...if SIGI assigned then so is I - checked earlier.
         IF(LOOKUP(JDO80).EQ.0) GO TO 80
C     ...if SIGI already flagged as "Missing"
         IF(LOGMSS(LOOKUP(JDO80))) THEN
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO80-1)),1) 
            LOGMSS(LOOKUP(JDO80-1)) = .TRUE.
            GO TO 80
         ENDIF
C     ...if I already flagged as "Missing"
         IF(LOGMSS(LOOKUP(JDO80-1))) THEN
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO80)),1) 
            LOGMSS(LOOKUP(JDO80)) = .TRUE.
            GO TO 80
         ENDIF
C     ...test for SIGI = 0.0
         IF (ADATA(LOOKUP(JDO80)).LE.1.E-30) THEN
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO80-1)),1) 
            LOGMSS(LOOKUP(JDO80-1)) = .TRUE.
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO80)),1) 
            LOGMSS(LOOKUP(JDO80)) = .TRUE.
         ENDIF
 80   CONTINUE

C     case (e): PHIB, etc
      DO 90 JDO90 = STARTPH+1,STARTPH+NUMPH*6-5,6
         IF (CTPRGI(JDO90).NE.'W')
     +         CALL CCPERR(1,'Not a weight')
C     ...if column not assigned via LABIN, leave unchanged
         IF(LOOKUP(JDO90).EQ.0) GO TO 90
         IF(LOOKUP(JDO90-1).EQ.0) THEN
            LOOKUP(JDO90)=0
            GO TO 90
         ENDIF
C     ...if this term already flagged as "Missing"
         IF ( LOGMSS(LOOKUP(JDO90)) .OR. LOGMSS(LOOKUP(JDO90-1))
     +                 .OR. ADATA(LOOKUP(JDO90)).LE.1.E-30 ) THEN
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO90-1)),1) 
            LOGMSS(LOOKUP(JDO90-1)) = .TRUE.
            CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO90)),1) 
            LOGMSS(LOOKUP(JDO90)) = .TRUE.
C     Hendrickson-Lattman coeffs may or may not be there....
            IF (LOOKUP(JDO90+1).GT.0) THEN
               CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO90+1)),1) 
               LOGMSS(LOOKUP(JDO90+1)) = .TRUE.
            ENDIF
            IF (LOOKUP(JDO90+2).GT.0)  THEN
               CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO90+2)),1) 
               LOGMSS(LOOKUP(JDO90+2)) = .TRUE.
            ENDIF
            IF (LOOKUP(JDO90+3).GT.0) THEN 
               CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO90+3)),1) 
               LOGMSS(LOOKUP(JDO90+3)) = .TRUE.
            ENDIF
            IF (LOOKUP(JDO90+4).GT.0) THEN 
               CALL EQUAL_MAGIC(IFIL1,BDATA(LOOKUP(JDO90+4)),1) 
               LOGMSS(LOOKUP(JDO90+4)) = .TRUE.
            ENDIF
         ENDIF
 90   CONTINUE

      DO 140 JDO40 = 1,NCOL
         IF((.NOT.LMSSOLD(JDO40)).AND.LOGMSS(JDO40))
     +       NCHANGES(JDO40) = NCHANGES(JDO40) + 1
 140  CONTINUE   
 
      CALL LWREFL(IFIL1,BDATA)

C     ...Return for next reflection
      GOTO 10

 100  CONTINUE
      CALL LRCLOS(IFIL1)
      CALL LWCLOS(IFIL1,1)

      WRITE(LUNOUT,'(//,A,/,A,//)') 
     +         ' CHANGES MADE TO MTZ FILE',
     +         ' ========================'

      DO 240 JDO40 = 1,NCOL
         IF (LOOKUP(JDO40).NE.0) THEN
           IF (NCHANGES(LOOKUP(JDO40)).EQ.0) THEN
             WRITE(LUNOUT,'(9X,A,A30)') 
     +   'NO changes made to column ',CLABS(LOOKUP(JDO40))
           ELSE
             WRITE(LUNOUT,'(5X,I6,A,A30)') NCHANGES(LOOKUP(JDO40)),
     +   ' changes made to column ',CLABS(LOOKUP(JDO40))
           ENDIF
         ENDIF
 240  CONTINUE   

      WRITE(LUNOUT,'(/)') 

      CALL CCPERR(0,'Normal termination')
      END

