C
C     OMIT
C     Copyright (C) 1997 Bauke Dijkstra
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
C///
C=======================================================================
C
C                               O M I T
C
C       A PROGRAM TO CALCULATE OMIT-MAPS ACCORDING TO T. N. BHAT
C       SEE:           "CALCULATION OF AN OMIT MAP" BY T.N. BHAT
C                       J. APPL. CRYSTALLOGR. (1988) 21, 279-281
C
C=======================================================================
C
C       THE USER IS STRONGLY ADVISED TO READ THE ABOVE ARTICLE !
C
C       For reliable documentation, please see omit.html
C
C       PROGRAM HISTORY:
C       ================
C
C       PROGRAM WRITTEN BY BAUKE DIJKSTRA (APRIL 1994). MODIFIED
C       BY FRED. VELLIEUX TO INCLUDE HISTOGRAM MAPPING  AND  THE
C       OPTION OF SKIPPING THE MAP TRUNCATION STEP (FEBR. 1996).
C       KEYWORDED INPUT ADDED BY MARTYN WINN (MAY 1997).
C       CCP4 MAP OUTPUT ADDED BY MARTYN WINN (JULY 1997).
C
C       PROGRAM INPUT:
C       ==============
C
C       THE PRESENT PROGRAM HAS COMBINED THE ORIGINAL CONGLOMERATE OF
C       SEPARATE PROGRAMS, ALL NEEDED TO CALCULATE AN OMIT-MAP,  INTO
C       ONE SINGLE AUTOMATIC PROCEDURE. ONLY A FEW LINES OF INPUT ARE
C       REQUIRED.
C 
C INPUT AND OUTPUT FILES
C 
C   Input
C   
C    HKLIN
C           Input data in MTZ format. THIS FILE SHOULD CONTAIN OBSERVED
C           AND/OR CALCULATED STRUCTURE FACTOR AMPLITUDES, AND PHASES FROM
C           SOME SOURCE.
C           
C   Output
C   
C    MAPOUT
C           Output binary omit-map. This file is in CCP4 format unless
C           the MFFOUTPUT keyword is specified for MFF FORMAT
C           (GRONINGEN MASTER FOURIER FILE FORMAT).
C           
C KEYWORDED INPUT
C 
C   LABIN <program label>=<file label>...
C   
C    (Compulsory.)
C    
C    Program labels are:
C    FP
C           Observed structure factor amplitudes, Fobs.
C    FC
C           Calculated structure factor amplitudes, Fcalc.
C    PHI
C           Phases, Phi.
C           
C   SCALE <scale_fobs> <scale_fcalc>
C   
C    (Compulsory.)
C    
C    <scale_fobs> and <scale_fcalc> are scale factors for Fobs and Fcalc
C    respectively. For example, <scale_fobs> = 2.0 and <scale_fcalc> = -1.0
C    allows one to calculate a 2FO-FC omit-map.
C    
C   RESOLUTION <resmin> <resmax>
C   
C    Resolution limits, either 4(sin theta/lambda)**2 or d in Angstrom
C    (either order). If this card isn't given, resolution range will be
C    read from HKLIN.
C    
C   TRUNCATE
C   
C    Include this card to TRUNCATE INITIAL MAP, SO THAT THE DENSITY VALUES
C    FALL BETWEEN -1.0*RMSMAP AND 20.0*RMSMAP (ITRUNC=1). Default is
C    ITRUNC= 0 (recommended).
C    
C   HISTOGRAM
C   
C    Include this card to apply HISTOGRAM MAPPING TO THE OMIT MAP, SO THAT
C    THE HISTOGRAM OF DENSITIES OF THE OMIT MAP EQUALS THAT OF THE STARTING
C    MAP (IHIST=1). Default is IHIST= 0 (recommended).
C    
C   DLIMIT <dstlmt>
C   
C    POINTS IN THE ELECTRON DENSITY MAP WITHIN <dstlmt> FROM A SYMMETRY
C    RELATED GRID POINT ARE SET TO -RMS DENSITY VALUE OF THE MAP. SET
C    DSTLMT = 0.0 FOR NO MODIFICATION OF INPUT MAP; 3.0-3.5 ANGSTROM SEEMS
C    A GOOD VALUES OTHERWISE. [Default 3.0]
C
C   GRID <nx> <ny> <nz>
C
C    Number of sampling divisions along each unit cell edge for the
C    output map. The number of points along each axis should be
C    even, and a product of the factors 2, 3, 4 or 5 (each of
C    which may appear any number of times). There is also a maximum
C    value for the number of points set in the program. If any
C    of &lt;nx&gt; &lt;ny&gt; and &lt;nz&gt; do not satisfy these
C    conditions, then they will be reset appropriately. If this
C    keyword is omitted, then the program will choose appropriate
C    values automatically.
C
C   MFFOUTPUT
C
C    Output omit map in MFF format (Groningen Master Fourier File format).
C    The default is to output the omit map in CCP4 format.
C    
C   TITLE <title>
C   
C    <title> WILL BE WRITTEN TO THE OUTPUT MFF-FILE
C    
C   END
C   
C    End input and run program.
C       
C=======================================================================
C
C       SAMPLE JOB FOR UNIX :
C
C #!/bin/sh
C 
C set -e
C 
C /ccpdisk/xtal/iris-build/src/omit hklin toxd_sf.mtz \
C mapout output.map << 'EOF' > omit.log
C LABI FP=FTOXD3 FC=FC_sf PHI=AC_sf
C SCAL 2.0 -1.0 
C RESO 10.0 2.3
C #TRUN
C HIST
C DLIM 3.0
C TITLE some kind of meaningful title
C END
C EOF
C #
C
C=======================================================================
C
C       INTERNAL SETTINGS OF THE PROGRAM:
C       - THE NEUTRAL VOLUME (SEE BHAT'S ARTICLE !) HAS BEEN FIXED TO 
C         A VOLUME OF TWO (2!) GRIDPOINTS AROUND THE PHASED VOLUME.
C       - IN CONTRAST TO WHAT BHAT MENTIONS (STEP J OF HIS PROCEDURE)
C         IN THE CURRENT VERSION OF THE PROGRAM STRUCTURE FACTORS ARE
C         OBTAINED BY  FOURIER TRANSFORMATION  OF A MAP  IN WHICH THE
C         DENSITY VALUES IN THE PHASED AND NEUTRAL VOLUMES HAVE  BEEN 
C         SET TO 0.0.
C       - BEFORE STARTING THE OMIT PROCEDURE THE ELECTRON DENSITY MAP
C         MAY BE TRUNCATED  SUCH THAT THE DENSITY VALUES ARE  BETWEEN 
C         -1.0*RMSMAP AND 20.0*RMSMAP
C
C=======================================================================
C      VERSION MARCH 29, 1994 BAUKE DIJKSTRA
C      VERSION APRIL 11, 1997 FRED. VELLIEUX, BAUKE DIJKSTRA
C=======================================================================
C\\\
      PARAMETER         (MAXBUF=60, MAXRBF=6, MAXREF=200000)
      PARAMETER         (MAXNU=200,MAXNV=200,MAXNW=200)
      PARAMETER         (MXHIST=256)
      REAL              X(6*MAXREF)
      CHARACTER*80      RBFINF(MAXRBF)
      INTEGER           MAXR, MAXB, ISLOW, INTER, IFAST, KLASS, ICENTR,
     .                  ISCREW, IVERSN, NSYM, IHKL(3), NTOTAL,
     .                  IHKLE(3), J(24), K(24), L(24), NFOBS, NFCALC,
     .                  NPHASE, ISYM, MDFEND, N, NN, NOW, NREF, NREFL,
     .                  IHKLS(2*MAXREF), I1, JJ, NSCR1, NSCR2, NOWREF,
     .                  IH1(8,1023), IH2(8,1023), IH3(8,1023), INP(3),
     .                  ESTMR
      REAL              CELL, SYMROT, SYMTRL, SCLFOB, SCLFC, RESMIN, 
     .                  RESMAX, RES, RBUFFR(MAXRBF), BUFRIN(MAXBUF),
     .                  PHASE, PHASE2, PSHIFT, CONV, BUF(8,1023),
     .                  AAS(2*MAXREF), BBS(2*MAXREF), FOB(2*MAXREF)
      LOGICAL LOGRES,LOGGRID
      EQUIVALENCE       (IH1(1,1),BUF(1,1)), (IH2(1,1),BUF(2,1)),
     .                  (IH3(1,1),BUF(3,1))
      EQUIVALENCE       (X(1),AAS(1)), (BBS(1),X(2*MAXREF+1))
      EQUIVALENCE       (FOB(1),X(4*MAXREF+1))

      COMMON /IPARAM/ NFOBS, NFCALC, NPHASE, ITRUNC, IMFF
      COMMON /LPARAM/ LOGRES,LOGGRID
      COMMON /RPARAM/ SCLFOB, SCLFC, RESMIN, RESMAX, DSTLMT
      COMMON / MDFPAR / MAXR, MAXB, CELL(6), ISLOW, INTER, IFAST,
     .                  KLASS, ICENTR, ISCREW(3), IVERSN
      COMMON / MDFSYM / NSPGRP, NSYM, SYMROT(3,3,96), SYMTRL(3,96)
      COMMON / MDFRES / A11, A21, A22, A31, A32, A33
      COMMON / MINMAX / MINH, MAXH, MINK, MAXK, MINL, MAXL,
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, NX, NY, NZ
      COMMON / REFLNS / INDEXH(-511:512), NOWREF
      COMMON / BOX    / IBOX, MX1, MX2, MY1, MY2, MZ1, MZ2
      COMMON / HIST   / IHIST, VMIN1, VMIN2, VMAX1, VMAX2,HSCF1,HSCF2,
     .                  IHIST1(MXHIST), IHIST2(MXHIST), LOOKUP(MXHIST)

      EXTERNAL PACK
C
      WRITE (*,10)
   10 FORMAT(//,15X,47('*'),/,15X,'*',45X,'*',/,15X,
     .'*   P  R  O  G  R  A  M       O  M  I  T      *',/,15X,
     .'*',45X,'*',/,15X,
     .'* V E R S I O N   1 1 - A P R - 1 9 9 7 (BWD) *',/,15X,
     .'*',45X,'*',/,15X,47('*'),/////)

C INITIALIZE CCP4 ROUTINES
      CALL CCPFYP
      CALL MTZINI

C----Read keyworded input
      CALL RCARDS

C     READ TITLE RECORDS FROM INPUT MTZ AND CALCULATE CONSTANTS FOR THE
C     RAPID CALCULATION OF THE RESOLUTION OF A REFLECTION

      CALL RDMTZ (1, RBFINF, MAXRBF, MAXBUF)
      CALL DCALC (CELL)

      NOWREF = 0
      NTOTAL = 0
      NREFL  = 0
      NWRT1  = 0
      CONV   = ATAN(1.0)/45.0
      NSCR1  = 8
      NSCR2  = 9
      CALL CLEARARRAY (BUF, 8*1023)
      CALL CCPDPN(-NSCR1,'OMITTMP1','SCRATCH','U',0,0)
      CALL CCPDPN(-NSCR2,'OMITTMP2','SCRATCH','U',0,0)
      MINH   =  1000
      MINK   =  1000
      MINL   =  1000
      MAXH   = -1000
      MAXK   = -1000
      MAXL   = -1000
C
C     READ A REFLECTION RECORD, AND CHECK WHETHER IT IS A VALID RECORD
C 
      WRITE (*,80) '   IH', '   IK', '   IL'
   80 FORMAT(/,1X,3A5,6X,'FOBS     FCALC     PHASE (DEGREES)')
  200 CALL RDREC (10,RBUFFR,MAXR,IHKL,BUFRIN,MAXB,MDFEND,IVERSN)
      IF (MDFEND.NE.0) GOTO 600
      RES   = 1.0/SQRT(RBUFFR(1))
      IF (RES.GT.RESMAX.OR.RES.LT.RESMIN) GO TO 200
      IF (NFOBS .GT.0) THEN
	     IF (BUFRIN(NFOBS)  .LE. 0.) GOTO 200
      END IF
      IF (NFCALC.GT.0) THEN
	     IF (BUFRIN(NFCALC) .LE. 0.) GOTO 200
      END IF
      PHASE = BUFRIN(NPHASE)*36./1000.
      IF (NREFL.LT.25) THEN
         IF (NFOBS.GT.0 .AND. NFCALC.GT.0) THEN
            WRITE (*,210) IHKL,BUFRIN(NFOBS),BUFRIN(NFCALC), PHASE
         ELSE IF (NFOBS.GT.0) THEN
            WRITE (*,211) IHKL,BUFRIN(NFOBS), PHASE
         ELSE IF (NFCALC.GT.0) THEN
            WRITE (*,212) IHKL,BUFRIN(NFCALC), PHASE
         END IF
      END IF
  210 FORMAT(1X,3I5,3F10.2)
  211 FORMAT(1X,3I5,F10.2,10X,F10.2)
  212 FORMAT(1X,3I5,10X,2F10.2)
C
C     GENERATE SYMMETRY EQUIVALENT REFLECTIONS WITH THEIR PHASES
C     THE HKL LIST SHOULD CONTAIN HK0, H-K0, 0K0. NOT NEEDED ARE
C     -HK0, -H-K0, 0-K0, BUT WE LEAVE THEM IN TO MAKE SURE
C
      NOW = 0
      DO 500 N = 1,NSYM
         CALL SYMEQ (IHKL, IHKLE, PSHIFT, N)
	 ISYM = 1
	 IF (IHKLE(3) .LT. 0) THEN
	    IHKLE(1) = -IHKLE(1)
	    IHKLE(2) = -IHKLE(2)
	    IHKLE(3) = -IHKLE(3)
	    ISYM = 2
c	 ELSE IF (IHKL(3). EQ. 0) THEN
c	    IF (IHKLE(1) .LT. 0) THEN
c	       IHKLE(1) = -IHKLE(1)
c	       IHKLE(2) = -IHKLE(2)
c	       ISYM = 2
c	    ELSE IF (IHKLE(1) .EQ. 0 .AND. IHKLE(2) .LT. 0) THEN
c               IHKLE(2) = -IHKLE(2)
c	       ISYM = 2
c	    END IF
	 END IF
         PHASE2 = PHASE + PSHIFT
         IF (ISYM .NE. 1) PHASE2 = -PHASE2
         PHASE2 = AMOD (PHASE2,360.)
         IF (PHASE2 .LT. 0.0) PHASE2 = PHASE2 + 360.
         PHASE2 = PHASE2*CONV
C
C	 REMOVE REPEATING REFLECTIONS AMONG THE SYMMETRY RELATED LIST
C
     	 IF (N .GT. 1) THEN
            DO 420 NN = 1, NOW
            IF (IHKLE(3) .EQ. L(NN) .AND. IHKLE(2) .EQ. K(NN) .AND. 
     .      IHKLE(1) .EQ. J(NN)) GO TO 500
  420	    CONTINUE
         END IF
         NREFL = NREFL + 1
         CALL PACK (IHKLE(1),IHKLE(2),IHKLE(3),IHKLS(NREFL))
         BFO = 0.0
         IF (NFOBS .GT.0) BFO = BUFRIN(NFOBS)*SCLFOB
         BFC = 0.0
         IF (NFCALC.GT.0) BFC = BUFRIN(NFCALC)*SCLFC
         AAS(NREFL) = COS(PHASE2)*(BFO + BFC)
         BBS(NREFL) = SIN(PHASE2)*(BFO + BFC)
         FOB(NREFL) = 0.0
         IF (NFOBS.GT.0) FOB(NREFL) = BUFRIN(NFOBS)
         NOWREF = NOWREF + 1
         NOW    = NOW + 1
         J(NOW) = IHKLE(1)
         K(NOW) = IHKLE(2)
         L(NOW) = IHKLE(3)
         IF (IHKLE(1) .LT. MINH) MINH = IHKLE(1)
         IF (IHKLE(2) .LT. MINK) MINK = IHKLE(2)
         IF (IHKLE(3) .LT. MINL) MINL = IHKLE(3)
         IF (IHKLE(1) .GT. MAXH) MAXH = IHKLE(1)
         IF (IHKLE(2) .GT. MAXK) MAXK = IHKLE(2)
         IF (IHKLE(3) .GT. MAXL) MAXL = IHKLE(3)
         IF (NREFL.EQ.2*MAXREF) THEN
            CALL SORT3 (IHKLS, NREFL, AAS, BBS, FOB)
            I1 = MAXREF+1
            WRITE (NSCR1) MAXREF
            WRITE (NSCR1) (IHKLS(JJ),JJ=I1,NREFL)
            WRITE (NSCR1) (AAS  (JJ),JJ=I1,NREFL)
            WRITE (NSCR1) (BBS  (JJ),JJ=I1,NREFL)
            WRITE (NSCR1) (FOB  (JJ),JJ=I1,NREFL)
            NREFL = MAXREF
            NWRT1 = NWRT1 + 1
         ENDIF
  500 CONTINUE
      NTOTAL = NTOTAL + 1
C     Back for next reflection
      GO TO 200

 600  CONTINUE

C     Close HKLIN
      CALL LRCLOS(1)

      CALL SORT3(IHKLS,NREFL,AAS,BBS,FOB)
      NN = MAXREF
      IF (NWRT1.EQ.0) NN = NREFL
      CALL UNPCK(IHOLD,IHKLE(2),IHKLE(3),IHKLS(1))
      INDEXH(IHOLD) = 1
      IF (-MAXH.LT.IHOLD) THEN
         DO 605 JJ = -MAXH, IHOLD
  605    INDEXH(JJ) = 1
      END IF
      IN            = 0
      INREC         = 0
      CALL CCPDPN(-11,'OMITFORT11','SCRATCH','DU',8*1023,0)
      DO 610 JJ=1,NN
         CALL UNPCK (IHKLE(1),IHKLE(2),IHKLE(3),IHKLS(JJ))
         IF (IHKLE(1).NE.IHOLD) THEN
            INDEXH(IHKLE(1)) = JJ
            IHOLD = IHKLE(1)
         END IF
         IN = IN + 1
         BUF(4,IN) = FOB(JJ)
         BUF(5,IN) = AAS(JJ)
         BUF(6,IN) = BBS(JJ)
         IH1(1,IN) = IHKLE(1)
         IH2(1,IN) = IHKLE(2)
         IH3(1,IN) = IHKLE(3)
         IF (IN.EQ.1023) THEN
            INREC = INREC + 1
            WRITE (11,REC=INREC) BUF
            IN = 0
         ENDIF
  610 CONTINUE
      IF (NWRT1.EQ.0) GOTO 750
      WRITE (NSCR1) NREFL - MAXREF
      WRITE (NSCR1) (IHKLS(JJ),JJ=MAXREF+1,NREFL)
      WRITE (NSCR1) (AAS  (JJ),JJ=MAXREF+1,NREFL)
      WRITE (NSCR1) (BBS  (JJ),JJ=MAXREF+1,NREFL)
      WRITE (NSCR1) (FOB  (JJ),JJ=MAXREF+1,NREFL)
      WRITE (NSCR1) -1
C
C     READ PREVIOUSLY WRITTEN REFLECTIONS FROM SCRATCH UNIT 9
C
  725 CALL SRTMRG (NSCR1, NSCR2, IHKLS, AAS, BBS, FOB, MAXREF, NREF)
      IF (NREF.EQ.0) GOTO 750
      DO 730 JJ=1,NREF
         CALL UNPCK (IHKLE(1),IHKLE(2),IHKLE(3),IHKLS(JJ))
         IF (IHKLE(1).NE.IHOLD) THEN
            INDEXH(IHKLE(1)) = JJ + NN
            IHOLD = IHKLE(1)
         ENDIF
         IN = IN + 1
         BUF(4,IN) = FOB(JJ)
         BUF(5,IN) = AAS(JJ)
         BUF(6,IN) = BBS(JJ)
         IH1(1,IN) = IHKLE(1)
         IH2(1,IN) = IHKLE(2)
         IH3(1,IN) = IHKLE(3)
         IF (IN.EQ.1023) THEN
            INREC = INREC + 1
            WRITE (11,REC=INREC) BUF
            IN = 0
         ENDIF
  730 CONTINUE
      NN = NN + NREF
      CALL SRTMRG (NSCR2, NSCR1, IHKLS, AAS, BBS, FOB, MAXREF, NREF)
      IF (NREF.EQ.0) GOTO 750
      DO 740 JJ=1,NREF
         CALL UNPCK (IHKLE(1),IHKLE(2),IHKLE(3),IHKLS(JJ))
         IF (IHKLE(1).NE.IHOLD) THEN
            INDEXH(IHKLE(1)) = JJ + NN
            IHOLD = IHKLE(1)
         ENDIF
         IN = IN + 1
         BUF(4,IN) = FOB(JJ)
         BUF(5,IN) = AAS(JJ)
         BUF(6,IN) = BBS(JJ)
         IH1(1,IN) = IHKLE(1)
         IH2(1,IN) = IHKLE(2)
         IH3(1,IN) = IHKLE(3)
         IF (IN.EQ.1023) THEN
            INREC = INREC + 1
            WRITE (11,REC=INREC) BUF
            IN = 0
         ENDIF
  740 CONTINUE
      NN = NN + NREF
      GOTO 725
  750 IF (IN.NE.0) THEN
         INREC = INREC + 1
         WRITE (11,REC=INREC) BUF
      ENDIF
      INDEXH(IHOLD+1) = NN+1
      WRITE (*,760) NTOTAL, NOWREF
  760 FORMAT(/,' THE',I9,' INDEPENDENT REFLECTIONS HAVE BEEN ',
     .'EXPANDED TO',I10,' REFLECTIONS')

      IF (LOGGRID) THEN
C     If the GRID keyword was specified, check values asked for OK.

C     Are any less or equal to zero?

        INP(1) = NX
        IF (INP(1).LE.0) INP(1) = IGRIDR (CELL(1), MAXH, 20., 20.)
        INP(2) = NY
        IF (INP(2).LE.0) INP(2) = IGRIDR (CELL(2), MAXK, 20., 20.)
        INP(3) = NZ
        IF (INP(3).LE.0) INP(3) = IGRIDR (CELL(3), MAXL, 20., 20.)

C     THE NUMBER OF POINTS ALONG EACH AXIS SHOULD BE EVEN AND BE
C     A PRODUCT OF THE FACTORS  2, 3, 4 OR 5. EACH FACTOR CAN BE
C     USED MORE THAN ONCE.

        DO 280 I=1,3
          INP(I)=INP(I)+MOD(INP(I),2)
 250      CONTINUE
          NTEST=INP(I)
          DO 270 II=2,5
  260       CONTINUE
            IF(NTEST.NE.(NTEST/II)*II) GOTO 270
            NTEST=NTEST/II
            IF(NTEST.EQ.1) GOTO 280
            GO TO 260
 270      CONTINUE
          INP(I)=INP(I)+2
          GO TO 250
 280    CONTINUE

C     Are any too large?

        IF (INP(1).GT.MAXNV) INP(1) = MAXNV
        IF (INP(2).GT.MAXNW) INP(2) = MAXNW
        IF (INP(3).GT.MAXNU) INP(3) = MAXNU

        IF (INP(1).NE.NX) WRITE(*,285) INP(1)
 285    FORMAT(/,' Warning: NX reset to ',I4)
        IF (INP(2).NE.NY) WRITE(*,286) INP(2)
 286    FORMAT(/,' Warning: NY reset to ',I4)
        IF (INP(3).NE.NZ) WRITE(*,287) INP(3)
 287    FORMAT(/,' Warning: NZ reset to ',I4)

        NX = INP(1)
        NY = INP(2)
        NZ = INP(3)

      ELSE
C     Else calculate internally.
        NX = IGRIDR (CELL(1), MAXH, 20., 20.)
        NY = IGRIDR (CELL(2), MAXK, 20., 20.)
        NZ = IGRIDR (CELL(3), MAXL, 20., 20.)
      ENDIF
C
C     CALCULATE AN ELECTRON DENSITY MAP OF COMPLETE UNIT CELL
C
      MINX = 0
      MINY = 0
      MINZ = 0
      MAXX = NX - 1
      MAXY = NY - 1
      MAXZ = NZ - 1
      WRITE (*,770) MINH, MINK, MINL, MAXH, MAXK, MAXL, MINX, MINY, 
     .              MINZ, MAXX, MAXY, MAXZ, NX, NY, NZ
  770 FORMAT(/,' MINIMUM H, K, L   = ',3I5,8X,'  MAXIMUM H, K, L   = ',
     .   3I5,/,' MINIMUM X, Y, Z   = ',3I5,8X,'  MAXIMUM X, Y, Z   = ',
     .   3I5,/,' GRID ALONG A, B, C  DIRECTION  ',32X,'= ',3I5,/)
C
      RMSMAP = 0.0
      IBOX   = 0
      CALL FFTED (X, 6*MAXREF, RMSMAP)
C
C     DETERMINE ASYMMETRIC UNIT OF MAP
C     AND DETERMINE CONVENIENT BOXES TO DIVIDE THE A.U. IN
C
      CALL LIMITS (MX, MY, MZ)
      WRITE (*,780) MINX, MINY, MINZ, MX, MY, MZ
  780 FORMAT(/,' EXTENT OF THE ASYMMETRIC UNIT:',/,' MINIMUM ',
     .'X, Y, Z   = ',3I5,10X,'MAXIMUM X, Y, Z   = ',3I5)
      NPTS = (MX+1)*(MY+1)*(MZ+1)
      IF (IHIST.EQ.1) CALL MKHIST(X)
      MMY = MIN((6*MAXREF/((MX+1)*(MZ+1))) - 1, MY)

      IF (MMY .LT. MY) THEN
C estimate the required MAXREF size:
C    this should be: (MX+1)*(MY+1)*(MZ+1)/6, but that gives estimates
C    like '140400'.  better to take advantage of integer arithmetic
C    to round this up to the nearest ten thousand.
         ESTMR = ((((MX+1) * (MY+1) * (MZ+1)) / 60000) + 1) * 10000
         WRITE (*,781) MY, ESTMR
  781    FORMAT(//,' MAIN ROUTINE - MAXIMUM ARRAY SPACE EXCEEDED BY'
     .          ' THE NUMBER OF SECTIONS (',I5,' )',/,'   PLEASE'
     .          ' REDEFINE PARAMETER MAXREF (',I7,' SHOULD DO ),'
     .          ,/,'   RECOMPILE AND RERUN THE PROGRAM',/)
         CALL CCPERR(1,' ERROR ; MAXREF array size to small.')
      ENDIF

      CALL MAKEBX (X, MX, MMY, MZ, NX, NY, NZ, MY)
C
      MAXREC = 0 
      CALL CCPDPN(-45,'OMITTMP3','SCRATCH','U',0,0)
      IF (NINT(10.*DSTLMT).NE.0) THEN
         WRITE (*,790) DSTLMT
  790    FORMAT(//,' DISTANCE LIMIT FOR CONTACTS BETWEEN SYMMETRY',
     .             ' RELATED POINTS',F12.3,' ANGSTROM')
         CALL CONTCT (CELL, DSTLMT, MAXREC)
         WRITE (*,800) MAXREC
  800    FORMAT(' NUMBER OF CLOSE CONTACTS BETWEEN SYMMETRY',
     .          ' RELATED POINTS   : ',9X,I10,/)
      ELSE
         WRITE (*,810) 
  810    FORMAT(/,' SYMMETRY RELATED POINTS IN THE INPUT MAP ARE',
     .             ' NOT RESET TO  -RMS ELECTRON DENSITY')
      END IF
      CALL CED(X, 6*MAXREF, ITRUNC, RMSMAP, NPTS)
      CALL CCPERR(0, 'Normal termination')
      END

      SUBROUTINE RCARDS

      INTEGER MXHIST,NPARM
      PARAMETER  (MXHIST=256,NPARM=200)

      CHARACTER*80  MAPTITLE
      CHARACTER*200  LABIN_SAVE

      INTEGER NFOBS, NFCALC, NPHASE, ITRUNC, IHIST, IMFF
      REAL SCLFOB, SCLFC, RESMIN, RESMAX, DSTLMT
C    
C---- things for parser
C
      CHARACTER KEY*4,LINE*600,CVALUE(NPARM)*4
      REAL      FVALUE(NPARM)
      LOGICAL   LEND,LOGDEBUG,LOGRES,LOGSCAL,LOGLAB,LOGGRID
      INTEGER   ITOK,NTOK,IBEG(NPARM),IEND(NPARM),ITYPE(NPARM),
     +          IDEC(NPARM)

      COMMON /IPARAM/ NFOBS, NFCALC, NPHASE, ITRUNC, IMFF
      COMMON /RPARAM/ SCLFOB, SCLFC, RESMIN, RESMAX, DSTLMT
      COMMON /LPARAM/ LOGRES,LOGGRID
      COMMON /MFFTIT/ MAPTITLE
      COMMON /LABINFO/ LABIN_SAVE
      COMMON / HIST   / IHIST, VMIN1, VMIN2, VMAX1, VMAX2,HSCF1,HSCF2,
     .                  IHIST1(MXHIST), IHIST2(MXHIST), LOOKUP(MXHIST)
      COMMON / MINMAX / MINH, MAXH, MINK, MAXK, MINL, MAXL,
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, NX, NY, NZ

C     Set some defaults

      ITRUNC = 0
      IHIST = 0
      IMFF = 0
      DSTLMT = 3.0
      MAPTITLE = ' From OMIT.'
C
      LOGDEBUG = .TRUE.
      LEND = .FALSE.
      LOGRES = .FALSE.
      LOGGRID = .FALSE.
      LOGSCAL = .FALSE.
      LOGLAB = .FALSE.

 10   CONTINUE
      NTOK=NPARM
      LINE=' '
      CALL PARSER (KEY,LINE,IBEG,IEND,ITYPE,FVALUE,CVALUE,IDEC,NTOK,
     +             LEND,LOGDEBUG)
      IF (LEND) GO TO 1000
C
      IF (KEY .EQ. 'END')  THEN
           GO TO 1000
C
      ELSE IF (KEY.EQ.'LABI') THEN
C----Save for parsing in s/r RDMTZ
        LABIN_SAVE = LINE(IBEG(1) :IEND(NTOK))
        LOGLAB = .TRUE.
C
C---- RESOlution
C     ==========
C
C     read resolution limits for listing in A, 
C     if only one treat as high resolution limit
C
C
      ELSE IF (KEY.EQ.'RESO') THEN
        ITOK = 2
C
C          *****************************************************
        CALL RDRESO(ITOK,ITYPE,FVALUE,NTOK,RESMAX,RESMIN,SMIN,SMAX)
C          *****************************************************
C
        LOGRES = .TRUE.
C
C----  TITLE  <Title>
C      ===============
C
      ELSE IF (KEY.EQ.'TITL') THEN
        IF(NTOK.GE.2) MAPTITLE = LINE(IBEG(2) :IEND(NTOK))
C
C----  TRUNCATE
C      ========
C     READ FLAG ITRUNC (0/1) FOR (NO/YES) TRUNCATION OF 
C     ELECTRON DENSITY VALUES TO BETWEEN -1.0*RMS-DENSITY AND 20.0*RMS-
C     DENSITY.
C
      ELSE IF (KEY.EQ.'TRUN') THEN
        ITRUNC = 1

C
C----  HISTOGRAM
C      =========
C     READ FLAG IHIST (0/1) FOR (NO/YES) HISTOGRAM MAPPING.
C
      ELSE IF (KEY.EQ.'HIST') THEN
        IHIST = 1

C
C----  MFFOUT
C      =========
C     READ FLAG IMFF (0/1) FOR (NO/YES) OUTPUT MFF MAP.
C
      ELSE IF (KEY.EQ.'MFFO') THEN
        IMFF = 1
C
C ---- GRID
C      =========
C
      ELSE IF (KEY.EQ.'GRID') THEN
        IF (NTOK.LT.4) THEN
          WRITE (*,'(A)') 
     +      ' ** Not enough arguments for GRID: keyword ignored **'
        ELSE
          CALL GTPINT(2,NX,NTOK,ITYPE,FVALUE)
          CALL GTPINT(3,NY,NTOK,ITYPE,FVALUE)
          CALL GTPINT(4,NZ,NTOK,ITYPE,FVALUE)
          LOGGRID = .TRUE.
        ENDIF
C
C ---- DLIMIT
C      =========
C
      ELSE IF (KEY.EQ.'DLIM') THEN
        IF (NTOK.LT.2) THEN
          WRITE(*,100) DSTLMT
 100      FORMAT(' Argument expected for DLIMIT. Default used: ',
     +      F12.3,' ANGSTROM')
        ELSE
          CALL GTPREA (2,DSTLMT,NTOK,ITYPE,FVALUE)
        ENDIF

C
C ---- SCALES
C      =========
C     READ SCALE FACTORS TO SCALE UP FOBS AND FCALC
C     THESE SCALE FACTORS MAY BE USED TO GET FOBS, OR DELF, OR 2FO-FC.
C     E.G. FOR A DELF MAP USE SCLFOB = 1.0 AND SCLFC = -1.0
C
      ELSE IF (KEY.EQ.'SCAL') THEN
        IF (NTOK.LT.3) THEN
          CALL CCPERR(1,' Two arguments required for SCALES.')
        ELSE
          CALL GTPREA (2,SCLFOB,NTOK,ITYPE,FVALUE)
          CALL GTPREA (3,SCLFC,NTOK,ITYPE,FVALUE)
          LOGSCAL = .TRUE.
        ENDIF

      ELSE
C
C----- ????? not understood
C
        WRITE(6,*) ' Keyword ',KEY,' not recognised'
      END IF
C
      GO TO 10
C        
1000  CONTINUE

C----Were compulsory keywords given?
      IF (.NOT.LOGLAB) CALL CCPERR(1,' LABIN must be given!')
      IF (.NOT.LOGSCAL) CALL CCPERR(1,' No scale factors given!')

      IF (LOGRES) THEN
         WRITE (*,20) RESMIN, RESMAX
   20    FORMAT(/,' RESOLUTION LIMITS FOR SELECTION OF REFLECTIONS:'
     .        ,F7.2,' TO',F7.2,' ANGSTROM')
      ELSE
         WRITE (*,25)
 25      FORMAT(/,' RESOLUTION LIMITS WILL BE TAKEN FROM MTZ FILE')
      ENDIF

      WRITE (*,30) SCLFOB, SCLFC
   30 FORMAT(/,' A ',F4.1,'*FOBS +',F4.1,
     .'*FCALC-MAP WILL BE CALCULATED')

      IF (ITRUNC.EQ.1) THEN
         WRITE (*,50)
   50    FORMAT(' STARTING MAP WILL BE TRUNCATED TO VALUES ',
     .          'BETWEEN -1.0*RMSMAP AND 20.0*RMSMAP ')
      ELSE
         WRITE (*,'('' STARTING MAP WILL NOT BE TRUNCATED '')')
      ENDIF

      IF (IHIST.EQ.0) THEN
         WRITE (*,'('' HISTOGRAM MAPPING WILL NOT BE CARRIED OUT '')')
      ELSE
         WRITE (*,60)
   60    FORMAT(' HISTOGRAM MAPPING WILL BE CARRIED OUT',/,
     .          ' SO THAT THE HISTOGRAM OF DENSITIES OF THE OMIT MAP',/,
     .          ' WILL BE EQUAL TO THAT OF THE STARTING MAP')
      ENDIF

      WRITE (*,70) DSTLMT
   70 FORMAT(' DISTANCE LIMIT FOR CONTACTS BETWEEN SYMMETRY',
     .       ' RELATED POINTS',F12.3,' ANGSTROM')

      END

      SUBROUTINE SYMEQ (IHKL,IHKLE,PSHIFT,JSYM)
      COMMON     / MDFSYM / NSPGRP, NSYM, SYMROT(3,3,96), SYMTRL(3,96)
      DIMENSION             IHKL(3), IHKLE(3)
      INTEGER               NSYM, IHKL, IHKLE, JSYM, I
      REAL                  PSHIFT, SYMROT, SYMTRL
      PSHIFT = 0.0
      DO 10 I=1,3
         IHKLE(I) = NINT(SYMROT(1,I,JSYM)*IHKL(1)) 
     .            + NINT(SYMROT(2,I,JSYM)*IHKL(2))
     .            + NINT(SYMROT(3,I,JSYM)*IHKL(3))
   10    PSHIFT   = PSHIFT - 360.0*IHKL(I)*SYMTRL(I,JSYM)
      RETURN
      END
      SUBROUTINE SRTMRG (NSCR1, NSCR2, IHKLS, A1, A2, A3, MAXREF, NREF)
C///
C     SUBROUTINE READS DATA FROM SCRATCH FILE NSCR1 AND SORTS THEM.
C     SORTED DATA IN IHKLS, A1 AND A2. IF MORE THAN 2*MAXREF HKL'S 
C     ARE PRESENT, THE FIRST MAXREF DATA ARE PUT IN IHKLS, A1 AND A2
C     AND THE REMAINDER ARE WRITTEN TO SCRATCH FILE NSCR2.
C     NREF IS THE ACTUAL NUMBER OF SORTED DATA IN THE ARRAYS AFTER
C     RETURNING FROM THIS ROUTINE
C
C\\\
      REAL       A1(2*MAXREF), A2(2*MAXREF), A3(2*MAXREF)
      INTEGER    NSCR1, NSCR2, IHKLS(2*MAXREF), MAXREF, NREF, NREF1, 
     .           NREF2
C     
      REWIND NSCR1
      REWIND NSCR2
      NWRT2 = 0
C
C     NREF1 .LE. 0: NO DATA ON NSCR1 :
      NREF   = 0
      READ  (NSCR1) NREF1
      IF (NREF1.LE.0) RETURN
C
      READ  (NSCR1) (IHKLS(J),J=1,NREF1)
      READ  (NSCR1) (A1   (J),J=1,NREF1)
      READ  (NSCR1) (A2   (J),J=1,NREF1)
      READ  (NSCR1) (A3   (J),J=1,NREF1)
      I2 = NREF1
  100 READ  (NSCR1) NREF2
      IF (NREF2.GT.0) THEN
         I1 = NREF1 + 1
         I2 = NREF1 + NREF2
         READ  (NSCR1) (IHKLS(J),J=I1,I2)
         READ  (NSCR1) (A1   (J),J=I1,I2)
         READ  (NSCR1) (A2   (J),J=I1,I2)
         READ  (NSCR1) (A3   (J),J=I1,I2)
         CALL SORT3 (IHKLS, I2, A1, A2, A3)
         WRITE (NSCR2) I2-MAXREF
         WRITE (NSCR2) (IHKLS(J),J=MAXREF+1,I2)
         WRITE (NSCR2) (A1   (J),J=MAXREF+1,I2)
         WRITE (NSCR2) (A2   (J),J=MAXREF+1,I2)
         WRITE (NSCR2) (A3   (J),J=MAXREF+1,I2)
         NREF1 = MAXREF
         NWRT2 = NWRT2 + 1
         GOTO 100
      ELSE
C
C     FINISHED?
         IF (NWRT2 .EQ. 0) THEN
            WRITE (NSCR2) -1
            NREF = I2
            RETURN
         ENDIF
         WRITE (NSCR2) -1
         NREF = MAXREF
         RETURN
      ENDIF
      END
      SUBROUTINE SORT3 (IHKL, N, A1, A2, A3)
C///
C
C       SUBROUTINE SORT3
C       PURPOSE:
C        TO SORT A VECTOR INTO INCREASING ORDER FROM IHKL(1) TO IHKL(N),
C        IHKL IS TYPE INTEGER. THE REAL VECTORS A1 AND A2 ARE PERMUTED 
C        THE SAME AS VECTOR IHKL.
C
C        DESCRIPTION OF PARAMETERS
C
C           IHKL   - THE NAME OF THE N-VECTOR TO BE SORTED (INTEGER).
C           N      - THE NUMBER OF ELEMENTS IN THE VECTOR TO BE SORTED.
C           A1, A2 - TWO REAL ARRAYS TO BE CO-SORTED WITH IHKL.
C
C        REMARKS
C           THE PROCEDURE REQUIRES TWO ADDITIONAL ARRAYS IU(K) AND
C           IL(K) WHICH PERMIT SORTING UP TO 2**(K+1)-1 ELEMENTS.
C           THESE ARRAYS ARE SUPPLIED BY THE SUBROUTINE WITH K = 16.
C           THIS ALLOWS SORTING A MAXIMUM OF 131071 ELEMENTS.
C
C        METHOD
C           'AN EFFICIENT ALGORITHM FOR SORTING WITH MINIMAL STORAGE'
C           BY RICHARD C. SINGLETON.  PREPARED FOR INSTITUTE RESEARCH
C           AND DEVELOPMENT.  STANFORD RESEARCH INSTITUTE PROJECT
C           387531-132 SEPTEMBER 1968.
C
C=========================================================================
C      VERSION 25 SEPTEMBER 1991
C=========================================================================
C\\\
C
      DIMENSION IHKL(N), IU(16), IL(16), A1(N), A2(N), A3(N)
      INTEGER   IHKL, T, TT
      REAL      AA1, AA2, AA3, A1, A2, A3
      M=1
      I=1
      J=N
    5 IF(I .GE. J) GO TO 70
   10 K=I
      IJ=(J+I)/2
      T=IHKL(IJ)
      IF(IHKL(I) .LE. T) GO TO 20
      IHKL(IJ)=IHKL(I)
      IHKL(I)=T
      T=IHKL(IJ)
      AA1=A1(IJ)
      A1(IJ)=A1(I)
      A1(I)=AA1
      AA2=A2(IJ)
      A2(IJ)=A2(I)
      A2(I)=AA2
      AA3=A3(IJ)
      A3(IJ)=A3(I)
      A3(I)=AA3
   20 L=J
      IF(IHKL(J) .GE. T) GO TO 40
      IHKL(IJ)=IHKL(J)
      IHKL(J)=T
      T=IHKL(IJ)
      AA1=A1(IJ)
      A1(IJ)=A1(J)
      A1(J)=AA1
      AA2=A2(IJ)
      A2(IJ)=A2(J)
      A2(J)=AA2
      AA3=A3(IJ)
      A3(IJ)=A3(J)
      A3(J)=AA3
      IF(IHKL(I) .LE. T) GO TO 40
      IHKL(IJ)=IHKL(I)
      IHKL(I)=T
      T=IHKL(IJ)
      AA1=A1(IJ)
      A1(IJ)=A1(I)
      A1(I)=AA1
      AA2=A2(IJ)
      A2(IJ)=A2(I)
      A2(I)=AA2
      AA3=A3(IJ)
      A3(IJ)=A3(I)
      A3(I)=AA3
      GO TO 40
   30 IHKL(L)=IHKL(K)
      IHKL(K)=TT
      AA1=A1(L)
      A1(L)=A1(K)
      A1(K)=AA1
      AA2=A2(L)
      A2(L)=A2(K)
      A2(K)=AA2
      AA3=A3(L)
      A3(L)=A3(K)
      A3(K)=AA3
   40 L=L-1
      IF(IHKL(L) .GT. T) GO TO 40
      TT=IHKL(L)
   50 K=K+1
      IF(IHKL(K) .LT. T) GO TO 50
      IF(K .LE. L) GO TO 30
      IF(L-I .LE. J-K) GO TO 60
      IL(M)=I
      IU(M)=L
      I=K
      M=M+1
      GO TO 80
   60 IL(M)=K
      IU(M)=J
      J=L
      M=M+1
      GO TO 80
   70 M=M-1
      IF(M.EQ.0) RETURN
      I=IL(M)
      J=IU(M)
   80 IF(J-I.GE.1) GOTO 10
      IF(I.EQ.1) GOTO 5
      I=I-1
   90 I=I+1
      IF(I .EQ. J) GO TO 70
      T=IHKL(I+1)
      IF(IHKL(I) .LE. T) GO TO 90
      AA1=A1(I+1)
      AA2=A2(I+1)
      AA3=A3(I+1)
      K=I
  100 IHKL(K+1)=IHKL(K)
      A1(K+1)=A1(K)
      A2(K+1)=A2(K)
      A3(K+1)=A3(K)
      K=K-1
      IF(T .LT. IHKL(K)) GO TO 100
      IHKL(K+1)=T
      A1(K+1)=AA1
      A2(K+1)=AA2
      A3(K+1)=AA3
      GO TO 90
      END
      INTEGER FUNCTION IGRIDR (A, MAXH, BAVRG, BADD)
C///
C=======================================================================
C                             G R I D E R
C
C     THE INTEGER FUNCTION IGRIDR CALCULATES FROM THE CELL LENGTH, MAX.
C     RESOLUTION, AVERAGE TEMPERATURE FACTOR AND ADDITIONAL TEMPERATURE
C     FACTOR, THE GRID INTERVAL ALONG THAT AXIS THAT GIVES THE SMALLEST 
C     ERROR IN THE CALCULATED STRUCTURE FACTORS. THE GRID INTERVAL WILL
C     BE A  MULTIPLE OF 12 TO  ACCOMMODATE  ALL  POSSIBLE  SPACE  GROUP
C                            REQUIREMENTS.
C
C=======================================================================
C\\\
      INTEGER          MAXH
      REAL             A, BAVRG, BADD
C
      N = MAXH + SQRT(MAXH*MAXH - 4.0*A*A*LOG(0.0002)/(BADD+BAVRG))
      N = N/24
      IF (N.EQ.0) THEN
         N = N + 1
         GO TO 400
      ENDIF
C
  100 N  = N + 1
      NN = N
      DO 300 I = 19,2,-1
  200    N1 = NN/I
	 II = I*N1
	 IF (II .EQ. NN) THEN
	    NN = N1
	    GOTO 200
	 ENDIF
  300 CONTINUE
C
C     THE GRID IS O.K. IF NN=1; ELSE, INCREASE N BY 1 AND TRY AGAIN
C
      IF (NN.NE.1) GOTO 100
  400 IGRIDR = 24*N
      RETURN
      END
      SUBROUTINE LIMITS (MX, MY, MZ)
C
C     THIS SUBROUTINE DEFINES THE LIMITS OF THE ASYMMETRIC UNIT
C     ACCORDING TO THE INTERNATIONAL TABLES CONVENTIONS
C
      COMMON / MDFPAR / MAXR, MAXB, CELL(6), ISLOW, INTER, IFAST,
     .                  KLASS, ICENTR, ISCREW(3), IVERSN
      COMMON / MINMAX / MINH, MAXH, MINK, MAXK, MINL, MAXL,
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, NX, NY, NZ
C
      MX = -1000
      MY = -1000
      MZ = -1000
C
C     TRICLINIC OR R3 WITH RHOMBOHEDRAL AXES
C
      IF (KLASS .EQ. 1 .OR. KLASS .EQ.  7) THEN
         MX = NX - 1
         MY = NY - 1
         MZ = NZ - 1
C
C     MONOCLINIC WITH B-AXIS UNIQUE
C     ICENTR = 0: P-SPACEGROUPS; ICENTR = 3: C-CENTERING
C
      ELSE IF (KLASS .EQ.  2) THEN
         MX = NX - 1
         MY = NY - 1
         MZ = NZ/2
         IF (ICENTR .EQ. 3) THEN
            MX = NX/2
            MY = NY/2
            MZ = NZ - 1
         ENDIF
C
C     ORTHORHOMBIC SPACE GROUPS
C
      ELSE IF (KLASS .EQ.  3) THEN
         MX = NX/2
         MY = NY/2
         MZ = NZ-1
         IF (ICENTR .EQ. 3) THEN
            IF (ISCREW(3) .EQ. 0) MX = NX/4
            IF (ISCREW(3) .EQ. 2) MZ = NZ/2
         ELSE IF (ICENTR .EQ. 4) THEN
            MX = NX/4
            MY = NY/4
         ELSE IF (ICENTR .EQ. 5) THEN
            MZ = NZ/2
         ENDIF
C
C     TETRAGONAL SPACE GROUP WITHOUT 2-FOLDS PERPENDICULAR TO THE 4-FOLD
C
      ELSE IF (KLASS .EQ.  4) THEN
         MX = NX/2
         MY = NY/2
         MZ = NZ-1
         IF (ICENTR .EQ. 5) THEN
            IF (ISCREW(3) .EQ. 0) MZ = NZ/2
            IF (ISCREW(3) .EQ. 4) THEN
               MY = NY-1
               MZ = NZ/4
            ENDIF
         ENDIF
C
C     TETRAGONAL SPACE GROUP WITH A 2-FOLD PERPENDICULAR TO THE 4-FOLD
C
      ELSE IF (KLASS .EQ.  5) THEN
         IF (ISCREW(3) .EQ. 0) THEN
            MX = NX/2
            MY = NY/2
            MZ = NZ/2
            IF (ICENTR .EQ. 5) MZ = NZ/4
         ELSE IF (ISCREW(3) .EQ. 4) THEN
            MX = NX-1
            MY = NY-1
            MZ = NZ/8
            IF (ICENTR .EQ. 5) MX = NX/2
         ELSE IF (ISCREW(3) .EQ. 2) THEN
            MX = NX/2
            MY = NY-1
            MZ = NZ/4
            IF (ISCREW(2) .EQ. 2) THEN
                  MY = NY/2
                  MZ = NZ/2
            ENDIF
         ENDIF
C
C     TRIGONAL SPACE GROUPS WITHOUT TWO-FOLDS
C
      ELSE IF (KLASS .EQ.  6) THEN
         IF (ISCREW(3) .EQ. 0) THEN
            MX = 2*NX/3
            MY = 2*NY/3
            MZ = NZ-1
            IF (ICENTR .EQ. 6) MZ = NZ/3
         ELSE IF (ISCREW(3) .EQ. 3) THEN
            MX = NX-1
            MY = NY-1
            MZ = NZ/3
         ENDIF
C
C     TRIGONAL SPACE GROUPS WITH TWOFOLDS PERPENDICULAR TO THE 3-FOLD
C
      ELSE IF (KLASS .EQ.  8 .OR. KLASS .EQ.  9) THEN
         IF (ISCREW(3) .EQ. 0) THEN
            MX = 2*NX/3
            MY = 2*NY/3
            MZ = NZ/2
            IF (ICENTR .EQ. 6) MZ = NZ/6
         ELSE IF (ISCREW(3) .EQ. 3) THEN
            MX = NX-1
            MY = NY-1
            MZ = NZ/6
         ENDIF
C
C     SPACE GROUP 155 WITH RHOMBOHEDRAL AXES
C
      ELSE IF (KLASS .EQ.  10) THEN
         MX = NX-1
         MY = NY-1
         MZ = NZ/2
C
C     HEXAGONAL SPACEGROUPS WITHOUT 2-FOLDS
C
      ELSE IF (KLASS .EQ.  11) THEN
         IF (ISCREW(3) .EQ. 0) THEN
            MX = 2*NX/3
            MY = NY/2
            MZ = NZ-1
         ELSE IF (ISCREW(3) .EQ. 6) THEN
            MX = NX-1
            MY = NY-1
            MZ = NZ/6
         ELSE IF (ISCREW(3) .EQ. 3) THEN
            MX = NX-1
            MY = NY-1
            MZ = NZ/3
         ELSE IF (ISCREW(3) .EQ. 2) THEN
            MX = 2*NX/3
            MY = 2*NY/3
            MZ = NZ/2
         ENDIF
C
C     HEXAGONAL SPACEGROUPS WITH 2-FOLDS PERP. TO THE 6-FOLD
C
      ELSE IF (KLASS .EQ.  12) THEN
         IF (ISCREW(3) .EQ. 0) THEN
            MX = 2*NX/3
            MY = NY/2
            MZ = NZ/2
         ELSE IF (ISCREW(3) .EQ. 6) THEN
            MX = NX-1
            MY = NY-1
            MZ = NZ/12
         ELSE IF (ISCREW(3) .EQ. 3) THEN
            MX = NX-1
            MY = NY-1
            MZ = NZ/6
         ELSE IF (ISCREW(3) .EQ. 2) THEN
            MX = 2*NX/3
            MY = 2*NY/3
            MZ = NZ/4
         ENDIF
C
C     CUBIC SPACE GROUPS P23, F23, I23, P2(1)3, I2(1)3
C
      ELSE IF (KLASS .EQ.  13) THEN
         IF (ISCREW(3) .EQ. 0) THEN
            MX = NX-1
            MY = NY-1
            MZ = NZ/2
            IF (ICENTR .NE. 0) MY = NY/2
            IF (ICENTR .EQ. 4) MX = NX/2
         ELSE IF (ISCREW(3) .EQ. 2) THEN
            MX = NX/2
            MY = NY/2
            MZ = NZ-1
            IF (ICENTR .EQ. 5) MZ = NZ/2
         ENDIF
C
C     THE REMAINING CUBIC SPACE GROUPS 
C
      ELSE IF (KLASS .EQ. 14) THEN
         IF (ISCREW (1) .EQ. 0) THEN
            MX = NX-1
            MY = NY/2
            MZ = NZ/2
            IF (ICENTR .NE. 0) MX = NX/2
            IF (ICENTR .EQ. 4) MY = NY/4
            IF (ICENTR .EQ. 5) MZ = NZ/4
         ELSE IF (ISCREW(1) .EQ. 2) THEN
            MX = NX/2
            MY = NY/2
            MZ = NZ/2
         ELSE IF (ISCREW(1) .EQ. 4) THEN
            MX = NX/2
            MY = 3*NY/4
            MZ = 3*NZ/4
            WRITE (*,100)
  100       FORMAT(' SPACE GROUP NOT IMPLEMENTED IN SUBROUTINE LIMITS',
     .      /,' CHECK YOUR SPACE GROUP OR DO SOME PROGRAMMING .....')
         ENDIF
      ENDIF
      IF (MX .LE. 0 .OR. MY .LE. 0 .OR. MZ .LE. 0) THEN
         WRITE (*,120) MX,MY,MZ,KLASS,ICENTR,ISCREW
  120    FORMAT(' SUBROUTINE "LIMITS" FAILED TO DETERMINE THE ',
     .   'ASYMMETRIC UNIT OF YOUR SPACE GROUP',/,' MAXX, MAXY, ',
     .   'MAXZ ',3I5,/,' KLASS, ICENTR, ISCREW ',5I5)
         CALL CCPERR (1,' ERROR IN SUBROUTINE LIMITS')
      ENDIF
      RETURN
      END
      SUBROUTINE MAKEBX(ARRAY, MX, MY, MZ, NX, NY, NZ, NSECS)
      INTEGER           BXNUMB, BOXSZX, BOXSZY, BOXSZZ, STNEUX, STNEUY,
     .                  STNEUZ, ENDNUX, ENDNUY, ENDNUZ
      DIMENSION         ARRAY(0:MZ,0:MX,0:MY), BUFED(8191)
C
C     READ THE ELECTRON DENSITY MAP (FROM FFTED) FROM UNIT 4.  WE ONLY
C     NEED TO FILL THE ASYMMETRIC UNIT. NEXT, SET ALL SYMMETRY RELATED
C     POINTS THAT FALL WITHIN THE ASYMMETRIC UNIT TO 0.0.
C
      MY1    = 0
      MY2    = MY
      TOTDEN = 0.0
   50 INDEX  = 1
      READ (4,REC=INDEX) BUFED
      WRITE (*,'('' MAKEBX: READING SECTIONS'',I5,'' TO'',I5)') MY1, MY2
      K = 0
      DO 100 J = 0,NY-1
      DO 100 I = 0,NX-1
      DO 100 L = 0,NZ-1
         K = K + 1
         IF (K .GT. 8191) THEN
            INDEX = INDEX + 1
            READ  (4,REC=INDEX) BUFED
            K = 1
         END IF
         IF (J.LT.MY1 .OR. J.GT.MY2) GOTO 100
         IF (I.LE.MX.AND.L.LE.MZ) ARRAY(L,I,J-MY1) = BUFED(K)
  100 CONTINUE
      CALL CORECT (ARRAY, MX, MY, MZ, NX, NY, NZ, TOTDEN)
      IF (MY2.LT.NSECS) THEN
         CALL CLEARARRAY (ARRAY, (MX+1)*(MY+1)*(MZ+1))
         MY1 = MY2+1
         MY2 = MY2+MY+1
         IF (MY2 .GT. NSECS) MY2 = NSECS
         IF (MY1 .LT. NSECS) GO TO 50
      END IF
C
C     DETERMINE BOX SIZE: THE ASYMMETRIC UNIT SHOULD BE DIVIDED INTO
C     APPROXIMATELY 30 BOXES
C
      BOXSZX = MX/4 + 1
      BOXSZY = (NSECS)/4 + 1
      FRACT = (BOXSZX + 5)*(BOXSZY + 5)*(MZ/4 + 6)
     .        /FLOAT((MX+1)*(NSECS+1)*(MZ+1))
      WRITE (*,200) BOXSZX, BOXSZY, FRACT, FRACT*TOTDEN
  200 FORMAT(' IT WILL BE DIVIDED IN BOXES OF ',2I5,'   ??  GRIDPOINTS',
     .' ALONG A, B AND C-AXES',/,' FRACTION OF THE ELECTRON DENSITY PE',
     .'R BOX:',30X,F9.3,/,' TARGET ELECTRON DENSITY PER BOX         : ',
     .28X,F10.3,//,'   BOX    MINX  MAXX    MINY  MAXY    MINZ  MAXZ  ',
     .'   VOL %    DEN %       DENSITY')
      FRACT  = FRACT*TOTDEN
      TOTVOL = FLOAT( (MX+1) * (NSECS+1) * (MZ+1))
      CALL CCPDPN(-96,'OMITTMP4','SCRATCH','U',0,0)
C
C     LOOP OVER PHASED VOLUMES
C
C *** TEST ***
C THE FOLLOWING PART HAS TO BE CHANGED FOR THE CASE WHERE THE ASYMMETRIC
C UNIT IS LARGER THAN THE X-ARRAY SIZE
C ...
C
      SUMVOL = 0.
      SUMDEN = 0.
      BXNUMB = 0
      STNEUY = -BOXSZY
      ENDNUY = -1
  300 STNEUY = STNEUY + BOXSZY
      ENDNUY = ENDNUY + BOXSZY
      IF (ENDNUY.GT.MY) ENDNUY = MY
      IF (STNEUY.LE.MY) THEN
          STNEUX = -BOXSZX
          ENDNUX = -1
  400     STNEUX = STNEUX + BOXSZX
          ENDNUX = ENDNUX + BOXSZX
          IF (ENDNUX.GT.MX) ENDNUX = MX
          IF (STNEUX.LE.MX) THEN
C
C         CALCULATE TOTAL DENSITY FOR ALL Z
C
              SUM = 0.0
              DO 600 K = STNEUY, ENDNUY
              DO 600 J = STNEUX, ENDNUX
              DO 600 I = 0,MZ
  600         SUM = SUM + ARRAY(I,J,K)
C
              BOXSZZ = MZ/IFIX(SUM/FRACT + 1.0) + 1
              STNEUZ = -BOXSZZ
              ENDNUZ = -1
  700         STNEUZ = STNEUZ + BOXSZZ
              ENDNUZ = ENDNUZ + BOXSZZ
              IF (ENDNUZ.GT.MZ) ENDNUZ = MZ
              IF (STNEUZ.LE.MZ) THEN
                  SUM    = 0.0
                  DO 800 K = STNEUY, ENDNUY
                  DO 800 J = STNEUX, ENDNUX
                  DO 800 I = STNEUZ, ENDNUZ
  800             SUM = SUM + ARRAY(I,J,K)
                  BXNUMB = BXNUMB + 1
                  WRITE (96) BXNUMB, STNEUX, ENDNUX, STNEUY, 
     .            ENDNUY, STNEUZ, ENDNUZ
C
C                 PRINT OUT SOME USEFUL INFORMATION ABOUT EACH BOX: 
C                 NUMBER, LIMITS OF BOX, VOLUME % OF ASYMMETRIC UNIT, 
C                 AND POSITIVE DENSITY %
C
                  MASVOL = (ENDNUX - STNEUX + 1)*(ENDNUY - STNEUY + 1)*
     .                     (ENDNUZ - STNEUZ + 1)
                  VOLFRC = FLOAT( MASVOL) / TOTVOL
                  SUMVOL = SUMVOL + VOLFRC
                  DENFRC = SUM / TOTDEN
                  SUMDEN = SUMDEN + DENFRC
                  WRITE( *, 1000) BXNUMB, STNEUX, ENDNUX, STNEUY, 
     .            ENDNUY, STNEUZ, ENDNUZ, VOLFRC, DENFRC, SUM
 1000             FORMAT( 1X, I4, 2X, 3( 2I6, 2X), 2F9.3,E14.3)
                  GO TO 700
              END IF
              GO TO 400
          END IF
          GO TO 300
      END IF
      WRITE (*,'('' TOTAL '',3( 2I6, 2X), 2F9.3,E14.3)')
     .0, MX, 0, MY, 0, MZ, SUMVOL, SUMDEN
      RETURN
      END
      SUBROUTINE CORECT (X, MX, MY, MZ, NX, NY, NZ, SUM)
C
C     SUBROUTINE TO ENFORCE SPACEGROUP SYMMETRY TO THE ELECTRON DENSITY
C     SYMMETRY RELATED POINTS  OF HIGHEST INDEX  ARE SET TO  0.0 TO GET 
C     MORE CORRECT ESTIMATE OF SUITABLE BOX SIZE
C
      DIMENSION         X(0:MZ,0:MX,0:MY)
      COMMON / MDFSYM / NSPGRP, NSYM, SYMROT(3,3,96), SYMTRL(3,96)
      DO 500 IY = 0,MY
      DO 500 IX = 0,MX
      DO 500 IZ = 0,MZ
         IXYZ1  = IZ + 512*IX + 512*512*IY
         DO 400 NS = 2,NSYM
            IY2 = NINT(SYMROT(2,1,NS)*IX) + NINT(SYMROT(2,2,NS)*IY)
     .          + NINT(SYMROT(2,3,NS)*IZ) + NINT(SYMTRL(2,NS)*NY)
            IY2 = MOD (IY2,NY)
            IF (IY2.LT.0) IY2 = IY2 + NY
            IF (IY2.GT.MY) GOTO 400
            IX2 = NINT(SYMROT(1,1,NS)*IX) + NINT(SYMROT(1,2,NS)*IY)
     .          + NINT(SYMROT(1,3,NS)*IZ) + NINT(SYMTRL(1,NS)*NX)
            IX2 = MOD (IX2,NX)
            IF (IX2.LT.0)IX2 = IX2 + NX
            IF (IX2.GT.MX) GOTO 400
            IZ2 = NINT(SYMROT(3,1,NS)*IX) + NINT(SYMROT(3,2,NS)*IY)
     .          + NINT(SYMROT(3,3,NS)*IZ) + NINT(SYMTRL(3,NS)*NZ)
            IZ2 = MOD (IZ2,NZ)
            IF (IZ2 .LT. 0) IZ2 = IZ2 + NZ
            IF (IZ2.GT.MZ) GOTO 400
            IXYZ2 = IZ2 + 512*IX2 + 512*512*IY2
            IF (IXYZ2.GT.IXYZ1) THEN
               X(IZ2,IX2,IY2) = X(IZ,IX,IY)
               X(IZ2,IX2,IY2) = 0.0
            END IF
  400    CONTINUE
         X(IZ,IX,IY) = AMAX1( X(IZ,IX,IY), 0.0)
         SUM = SUM + X(IZ,IX,IY)
  500 CONTINUE
      RETURN
      END
      SUBROUTINE CONTCT (CELL, DSTLMT, MAXREC)
      INTEGER           MINH, MAXH, MINK, MAXK, MINL, MAXL, NS, NSYM, 
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, NX, NY, NZ
      COMMON / MDFSYM / NSPGRP, NSYM, SYMROT(3,3,96), SYMTRL(3,96)
      COMMON / MINMAX / MINH, MAXH, MINK, MAXK, MINL, MAXL,
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, NX, NY, NZ
      REAL              SYMROT, SYMTRL, TRF(3,3), CELL(6), A, DSTLMT,
     .                  CALFA, CBETA, CGAMM, CW, RAD, SBETA, SGAMM, SW,
     .                  DISTCE, XC2(3)
      INTEGER           I, J, K, J2, MAXREC, II, JJ, KK, IDX, IDY, IDZ
C
      IF (NSYM .EQ. 1) RETURN
      WRITE (*,20) DSTLMT
   20 FORMAT(//,' DISTANCE LIMIT FOR CONTACTS BETWEEN SYMMETRY ',
     .'RELATED POINTS',F12.3,' ANGSTROM')
C
C     CALCULATE MATRIX TO GO FROM GRID COORDINATES TO CARTESIAN COORDS
C
      RAD   = ATAN(1.0)/45.
      CALFA = COS(CELL(4)*RAD)
      CBETA = COS(CELL(5)*RAD)
      SBETA = SIN(CELL(5)*RAD)
      CGAMM = COS(CELL(6)*RAD)
      SGAMM = SIN(CELL(6)*RAD)
      CW    = (CALFA-CGAMM*CBETA)/(SBETA*SGAMM)
      SW    = SIN(ACOS(CW))
      TRF(1,1)=CELL(1)/FLOAT(NX)
      TRF(1,2)=CELL(2)*CGAMM/FLOAT(NY)
      TRF(1,3)=CELL(3)*CBETA/FLOAT(NZ)
      TRF(2,1)=0.0
      TRF(2,2)=CELL(2)*SGAMM/FLOAT(NY)
      TRF(2,3)=CELL(3)*SBETA*CW/FLOAT(NZ)
      TRF(3,1)=0.0
      TRF(3,2)=0.0
      TRF(3,3)=CELL(3)*SBETA*SW/FLOAT(NZ)
C
      DO 1000 J = 0,NY-1
      DO 1000 I = 0,NX-1 
      DO 1000 K = 0,NZ-1  
C	  
      DO 500 NS = 2,NSYM
C
C        SYMMETRY RELATED X:
	  A   = SYMROT(1,1,NS)*I + SYMROT(1,2,NS)*J + SYMROT(1,3,NS)*K + 
     .         SYMTRL(1,NS)*NX
		 II  = MOD (NINT(A),NX)
	     IF (II .LT. 0) II = II + NX
		 IDX = IABS(II - I)
	     IF (2*IDX .GT. NX) IDX = NX - IDX
		 IF (IDX .GT. DSTLMT*NX/CELL(1)) GO TO 500
C
C        SYMMETRY RELATED Y:
	  A   = SYMROT(2,1,NS)*I + SYMROT(2,2,NS)*J + SYMROT(2,3,NS)*K + 
     .         SYMTRL(2,NS)*NY
		 JJ  = MOD (NINT(A),NY)
	     IF (JJ .LT. 0.) JJ = JJ + NY
		 IDY = IABS(JJ - J)
	     IF (2*IDY .GT. NY) IDY = NY - IDY
		 IF (IDY .GT. DSTLMT*NY/CELL(2)) GO TO 500
C
C        SYMMETRY RELATED Z:
	  A   = SYMROT(3,1,NS)*I + SYMROT(3,2,NS)*J + SYMROT(3,3,NS)*K + 
     .         SYMTRL(3,NS)*NZ
		 KK  = MOD (NINT(A),NZ)
	     IF (KK .LT. 0.) KK = KK + NZ
		 IDZ = IABS(KK - K)
	     IF (2*IDZ .GT. NZ) IDZ = NZ - IDZ
		 IF (IDZ .GT. DSTLMT*NZ/CELL(3)) GO TO 500
C
C        NOW CALCULATE EXACT DISTANCE DIFFERENCE IN ANGSTROM BETWEEN 
C        SYMMETRY RELATED POINTS:
         DISTCE = 0.0
         DO 300 J2 = 1,3
            XC2(J2) = TRF(J2,1)*IDX + TRF(J2,2)*IDY + TRF(J2,3)*IDZ
            DISTCE  = DISTCE + XC2(J2)*XC2(J2)
  300    CONTINUE
         IF (SQRT(DISTCE).LE.DSTLMT) THEN
            MAXREC = MAXREC + 1
            WRITE (45) I, J, K
            GO TO 1000
         END IF		 
  500 CONTINUE
 1000 CONTINUE
      RETURN
      END
      SUBROUTINE CED (X, SIZE, ITRUNC, RMSMAP, NPTS)
C
      INTEGER           SIZE, MINH, MINK, MINL, MAXH, MAXK, MAXL, P1, P2
      INTEGER           MINZ, MINX, MINY, MAXZ, MAXX, MAXY, NX, NY, NZ
      REAL              X(SIZE), RMS
      COMMON / MINMAX / MINH, MAXH, MINK, MAXK, MINL, MAXL,
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, NX, NY, NZ
      COMMON / MDFSYM / NSPGRP, NSYM, SYMROT(3,3,96), SYMTRL(3,96)
      COMMON / BOX    / IBOX, MX1, MX2, MY1, MY2, MZ1, MZ2
C
      P1 = SIZE/(NX*(NZ+2))
      P2 = SIZE/(2*NY*(MAXL+1))
      IF (P1 .LE. 0 .OR. P2 .LE. 0) THEN
         IFC = MAX( NX*(NZ+2), 2*NY*(MAXL+1) )
         IFC = IFC/6 + 5
         WRITE (*,10) SIZE, MAX(NX*(NZ+2),2*NY*(MAXL+1)), IFC
   10    FORMAT(' SUBROUTINE CED: INCREASE SIZE FROM',I7,' TO',I7,/,
     .   ' TO DO THIS AUTOMATICALLY, CHANGE PARAMETER MAXREF TO',
     .   ' AT LEAST ',I10,' AND RECOMPILE.') 
         CALL CCPERR(1,'SIZE OF X-ARRAY TOO SMALL: INCREASE MAXREF')
      ENDIF
C
C     CALCULATE STRUCTURE FACTORS FROM ORIGINAL ELECTRON DENSITY MAP.
      IFC = 5
      CALL SFC (X, SIZE, IFC, 0, 0, 0, 0, 0, 0, SCL1)
C
C     CALCULATE STRUCTURE FACTORS FROM TRUNCATED ELECTRON DENSITY MAP.
      CALL SYMMAP(X, NZ-1, NX-1, RMSMAP, ITRUNC)
      IFC = 7
      CALL SFC (X, SIZE, IFC, 0, 0, 0, 0, 0, 0, SCL2)
C
C     END OF JOB; DETERMINE WEIGHTS
C      
      CALL WEIGHTS (SCL1, SCL2)
      WRITE (*,'('' '')')
      CALL CCPDPN(-68,'OMITTMP5','SCRATCH','U',0,0)
      REWIND 96
      RMS  = 0.0
  550 READ(96,END=900) IBOX, MX1, MX2, MY1, MY2, MZ1, MZ2
      CALL CLEARARRAY (X,SIZE)
      IFC = 5
      CALL SFC (X, SIZE, IFC, MX1, MX2, MY1, MY2, MZ1, MZ2, SCL3)
      CALL WEIGHTS2 (SCL3)
C
C     END OF JOB: CALCULATE ELECTRON DENSITY FOR THIS BOX
C
      CALL FFTED(X,SIZE,RMS)
      GO TO 550
  900 P1 = SIZE/(NZ*NX)-1
      SCALE = SQRT(FLOAT(NPTS)/RMS)
      CALL COLECT (NX-1, P1, NZ-1, X, SCALE)
      END
      SUBROUTINE RDMAP (X, N, NX, NY, R)
C///
C     SUBROUTINE TO READ ELECTRON DENSITY MAP FROM DIRECT ACCESS UNIT 4
C\\\
      REAL      X(N,NX,NY), BUFED(8191)
      INTEGER   N, NX, NY, R, I, L, J, INDEX
      SAVE      INDEX, K, BUFED
C
C     READ FIRST RECORD FROM DIRECT ACCESS FILE
C
      IF (R .EQ. 0) THEN
         INDEX  = 1
         READ (4,REC=INDEX) BUFED
         K = 1
      ENDIF
      DO 100 J = 1,NY
      DO 100 I = 1,NX
      DO 100 L = 1,N-2
         X(L,I,J) = BUFED(K)
         K = K + 1
         IF (K .GT. 8191) THEN
            INDEX = INDEX + 1
            READ  (4,REC=INDEX) BUFED
            K = 1
         END IF
  100 CONTINUE
      RETURN
      END
      SUBROUTINE SFC (X, SIZE, IFC, MX1, MX2, MY1, MY2, MZ1, MZ2, SCL)
C///
C     SFC: CALCULATE STRUCTURE FACTORS FROM AN ELECTRON DENSITY MAP
C\\\
      INTEGER           MINH, MAXH, MINK, MAXK, MINL, MAXL, SIZE, T2,
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, NX, NY, NZ
      INTEGER           P1, P2, R, INDEXH, NOWREF, D(5), IFC, SKIP, RECS
      INTEGER           MX1, MX2, MY1, MY2, MZ1, MZ2, IX, IY, IZ, NSYM
      INTEGER           I, J, K, NS, JI, IOLD
      REAL              X(SIZE), BUF, SYMROT, SYMTRL
      COMMON / MINMAX / MINH, MAXH, MINK, MAXK, MINL, MAXL,
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, NX, NY, NZ
      COMMON / REFLNS / INDEXH(-511:512), NOWREF
      COMMON / REFLN  / IOLD, BUF(8,1023), DUMMY(8)
      COMMON / MDFSYM / NSPGRP, NSYM, SYMROT(3,3,96), SYMTRL(3,96)
C
      T2 = 82
      CALL CCPDPN(-T2,'OMITTMP6','SCRATCH','U',0,0)
      SMFOFC = 0.0
      SMFCSQ = 0.0
C
      P1 = SIZE/(NX*(NZ+2))
      P2 = SIZE/(2*NY*(MAXL+1))
      R  =  0 
      j  =  1
  100 IF (R + P1 .GT. NY) P1 = NY-R
      MAXSEC = R + P1 - 1
      CALL CLEARARRAY (X,SIZE)
      CALL RDMAP (X, NZ+2, NX, P1, R)
      IF (MX2.EQ.0 .AND. MY2.EQ.0 .AND. MZ2.EQ.0) GOTO 1000
      if (j.eq.0) goto 1000
      DO 900 J = MY1-2, MY2+2
      DO 900 I = MX1-2, MX2+2
      DO 900 K = MZ1-2, MZ2+2
         DO 800 NS = 1,NSYM
            IY = NINT(SYMROT(2,1,NS)*I) + NINT(SYMROT(2,2,NS)*J) 
     .         + NINT(SYMROT(2,3,NS)*K) + NINT(SYMTRL(2,NS)*NY)
            IY = MOD(IY,NY)
            IF (IY .LT .0)  IY = IY + NY
            IF (IY .LT. R)  GOTO 800
            IF (IY .GT. MAXSEC) GOTO 800
            IX = NINT(SYMROT(1,1,NS)*I) + NINT(SYMROT(1,2,NS)*J) 
     .         + NINT(SYMROT(1,3,NS)*K) + NINT(SYMTRL(1,NS)*NX)
            IZ = NINT(SYMROT(3,1,NS)*I) + NINT(SYMROT(3,2,NS)*J) 
     .         + NINT(SYMROT(3,3,NS)*K) + NINT(SYMTRL(3,NS)*NZ)
            IX = MOD(IX,NX)
            IF (IX .LT. 0) IX = IX + NX
            IZ = MOD(IZ,NZ)
            IF (IZ .LT. 0) IZ = IZ + NZ
            JI = (IY-R)*NX*(NZ+2) + IX*(NZ+2) + IZ + 1
            X(JI) = 0.0
  800    CONTINUE
  900 CONTINUE
 1000 D(1) = (NZ+2)*NX*P1
      D(2) = 2
      D(3) = D(1)
      D(4) = D(1)
      D(5) = NZ+2
      CALL REAL FT(X(1),X(2),NZ/2,D)
      D(2) = NZ+2
      D(3) = NX*(NZ+2)
      D(4) = 2*(MAXL+1)
      D(5) = 2
      CALL CMPL FT(X(1),X(2),NX,D)
      CALL WRITEH (X,(NZ+2)/2,NX,P1,MAXL,MAXH,P2,T2)
      R = R+P1
      IF(R.LT.NY) GO TO 100
C
C     PASS TWO READS INTERMEDIATE RESULTS AND CALCULATES FOURIER
C     TRANSFORMS ON Y.
C
      REWIND T2
      SKIP = 0
      P1   = SIZE/(NX*(NZ+2))
      P2   = SIZE/(2*NY*(MAXL+1))
      RECS = (2*MAXH)/P2
      R    = -MAXH
 1100 IF (R+P2.GT.MAXH) P2 = MAXH + 1 - R
      CALL CLEARARRAY (X,SIZE)
      CALL READH (X, NY, MAXL+1, P2, P1, SKIP, RECS, T2)
      IF (R+P2.LE.MAXH) REWIND T2
      SKIP = SKIP+1
      D(1) = 2*NY*(MAXL + 1)*P2
      D(2) = 2
      D(3) = D(1)
      D(4) = D(1)
      D(5) = 2*NY
      CALL CMPL FT (X(1), X(2), NY, D)
      CALL SFMAP2 (X, NY, MAXL+1, P2, R, IFC, SMFOFC, SMFCSQ)
      R = R+P2
      IF(R.LE.MAXH) GO TO 1100
      CLOSE (UNIT=T2)
      SCL = 1.0
      IF (SMFCSQ.NE.0.) SCL = SMFOFC/SMFCSQ
      RETURN
      END
      SUBROUTINE SFMAP2 (X, NK, NL, NH, HS, IFC, SMFOFC, SMFCSQ)
      INTEGER           NK, NL, NH, HS, IFC, INDEXH, NOWREF, IHMAX
      INTEGER           IH(8,1023), IK(8,1023), IL(8,1023), IREC2, I
      INTEGER           INDEX1, INDEX2, IH1, JK, IL1, ISTART
      COMPLEX           X(NK,NL,NH)
      REAL              BUF, R, S
      COMMON / REFLNS / INDEXH(-511:512), NOWREF
      COMMON / REFLN  / IREC2, BUF(8,1023), DUMMY(8)
      EQUIVALENCE       (IH(1,1),BUF(1,1)),   (IK(1,1),BUF(2,1)),
     .                  (IL(1,1),BUF(3,1))
C
      IHMAX  = HS - 1 + NH
      INDEX1 = INDEXH(HS)
      INDEX2 = INDEXH(IHMAX+1) - 1
      IF (INDEX1 .EQ.      0) RETURN
      IF (INDEX1 .GT. INDEX2) RETURN
C
      IREC2  = ((INDEX1-1)/1023) + 1
      READ (11,REC=IREC2) BUF
      ISTART = INDEX1 - (IREC2-1)*1023
      DO 200 I=INDEX1,INDEX2
         IF(ISTART.GT.1023) THEN
            WRITE (11,REC=IREC2) BUF
            IREC2  = ((I-1)/1023) + 1
            READ (11,REC=IREC2) BUF
            ISTART = I - (IREC2-1)*1023
         END IF
         JK  = IK(1,ISTART) + 1        
         IL1 = IL(1,ISTART) + 1
         IH1 = IH(1,ISTART) - HS + 1
         IF(JK.LE.0) JK = JK+NK
         R =   REAL(X(JK,IL1,IH1))
         S = -AIMAG(X(JK,IL1,IH1))
         BUF(IFC,  ISTART) = R
         BUF(IFC+1,ISTART) = S
         SMFOFC = SMFOFC + BUF(4,ISTART)*SQRT(R*R+S*S)
         SMFCSQ = SMFCSQ + R*R + S*S
         ISTART=ISTART+1
  200 CONTINUE
      WRITE (11,REC=IREC2) BUF
      RETURN
      END
      SUBROUTINE SYMMAP (ARRAY, N1, N2, SOLVEL, ITRUNC)
C///
C
C     FIRST TIME THROUGH THIS SUBROUTINE CALCULATES THE RMS VALUE OF  THE
C     ELECTRON DENSITY  AND WRITES THE DENSITIES TO DIRECT ACCESS FILE 4.
C     SECOND TIME THROUGH THE MAXIMUM PEAK HEIGHT MAY BE TRUNCATED TO 20* 
C     RMS-VALUE  (VARIABLE PEKHGT), AND THE MINIMUM DENSITY MAY BE SET TO
C     MINUS THIS RMS-VALUE. FURTHERMORE, DENSITY FOR SYMMETRY CONTACTS IS
C     ALSO PUT TO -RMS-VALUE. MODIFIED DENSITIES ARE WRITTEN TO UNIT 4.
C
C\\\
      LOGICAL           ENDSYM
      REAL              ARRAY(0:N1,0:N2), SOLVEL, RMSMAP, PEKHGT, 
     .                  BUF(8191), RMAX, RMIN
C
      INTEGER           MINH, MAXH, MINK, MAXK, MINL, MAXL, N1, N2,
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, NX, NY, NZ,
     .                  NDELET, I1, J1, K1, IX, IY, IZ, K, INDEX
      COMMON / MINMAX / MINH, MAXH, MINK, MAXK, MINL, MAXL,
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, NX, NY, NZ
C
      INDEX  = 1
      K      = 1
      RMAX   = -100000.
      RMIN   =  100000.
      RMSMAP = 0.
      PEKHGT = 20.0*SOLVEL
C
C     COMPARE ALL GRID POINTS IN THIS SECTION WITH THE GRIDPOINTS
C     ON UNIT 45 (FROM SUBROUTINE CONTACT)
C
      NDELET = 0
      REWIND 45
      READ (45,END=300) IX, IY, IZ
      ENDSYM = .FALSE.
      GOTO 400
  300 ENDSYM = .TRUE.
  400 READ (4,REC=INDEX) BUF
      READ (4,REC=INDEX) BUF
      DO 700 J1 = 0, NY-1
      DO 700 I1 = 0, NX-1
      DO 700 K1 = 0, NZ-1
         IF (ENDSYM) GOTO 600
         IF (I1.EQ.IX .AND. J1.EQ.IY .AND. K1.EQ.IZ) THEN
            BUF(K) = -SOLVEL
            NDELET = NDELET + 1
            READ (45,END=500) IX, IY, IZ
            GO TO 600
  500       ENDSYM = .TRUE.
            CLOSE (UNIT=45)
         ENDIF
  600    IF (ITRUNC.EQ.1 .AND. BUF(K) .GT. PEKHGT) BUF(K) = PEKHGT
         IF (ITRUNC.EQ.1 .AND. BUF(K) .LT.-SOLVEL) BUF(K) = -SOLVEL
         IF (BUF(K).GT.RMAX) RMAX = BUF(K)
         IF (BUF(K).LT.RMIN) RMIN = BUF(K)
         RMSMAP = RMSMAP + BUF(K)*BUF(K)
         K = K + 1
         IF (K .GT. 8191) THEN
            WRITE (4,REC=INDEX) BUF
            INDEX = INDEX + 1
            READ  (4,REC=INDEX) BUF
            K = 1
         ENDIF
  700 CONTINUE
      WRITE (*,710) NDELET
  710 FORMAT(' NUMBER OF GRIDS DELETED FOR SYMMETRY CONTACTS:',
     .17X,I17,//,' AFTER TRUNCATION:')
      IF (K.NE.1) WRITE (4,REC=INDEX) BUF
      SOLVEL = SQRT (RMSMAP/FLOAT( NX * NY * NZ ))
      WRITE (*,'('' MINIMUM AND MAXIMUM DENSITY '',32X,2F10.3)')
     .RMIN, RMAX
      WRITE (*,'('' RMS ELECTRON DENSITY OF MAP'',9X,F44.3)') SOLVEL
      RETURN
      END
      SUBROUTINE READH (X,NY,NZ,NX,SIZE,SKIP,RECS,TAPE)
      INTEGER NY,NZ,NX,SIZE,SKIP,RECS,TAPE
      COMPLEX X(NY,NZ,NX)
C
      INTEGER H,K,L,P,Q
C
      IF(SKIP.LE.0) GO TO 20
      DO 10 H = 1,SKIP
10    READ(TAPE)
C
20    CONTINUE
      Q = 0
100   CONTINUE
      P = Q+1
      Q = Q+SIZE
      IF (Q .GT. NY) Q = NY
      READ(TAPE)(((X(K,L,H),H = 1,NX),K = P,Q),L = 1,NZ)
      IF(Q.EQ.NY) GO TO 300
      IF(RECS.LE.0) GO TO 100
      DO 200 H = 1,RECS
      READ(TAPE)
200   CONTINUE
      GO TO 100
C
300   CONTINUE
      RETURN
      END
      SUBROUTINE WRITEH (X,NZ,NX,NY,LMAX,HMAX,SIZE,TAPE)
      INTEGER NZ,NX,NY,LMAX,HMAX,SIZE,TAPE
      COMPLEX X(NZ,NX,NY)
C
      INTEGER H,K,L,LM,P,Q
C
      LM = LMAX+1
      Q = NX-HMAX
100   CONTINUE
      P = Q+1
      Q = Q+SIZE
      IF(Q.GT.NX) GO TO 300
      WRITE(TAPE)(((X(L,H,K),H = P,Q),K = 1,NY),L = 1,LM)
      IF(Q.LT.NX) GO TO 100
C
      Q = 0
200   CONTINUE
      P = Q+1
      Q = Q+SIZE
      IF(Q.GT.HMAX) Q = HMAX+1
      WRITE(TAPE)(((X(L,H,K),H = P,Q),K = 1,NY),L = 1,LM)
      IF(Q.LE.HMAX) GO TO 200
C
      RETURN
C
300   CONTINUE
      Q = Q - NX
      IF(Q.GT.HMAX) Q = HMAX+1
      WRITE(TAPE)(((X(L,H,K),H = P,NX),(X(L,H,K),H = 1,Q),K = 1,NY),
     .L = 1,LM)
      IF(Q.LE.HMAX) GO TO 200
      RETURN
C
      END
      SUBROUTINE WEIGHTS(SCL1, SCL2)
      INTEGER         NOWREF, I, IREC2, ISTART, INDEXH
      REAL            WF, A, BUF(8,1023), SCL1, SCL2, FC1, FC2
      REAL            SMDIF1, SMDIF2, SUMFO
      COMMON /REFLNS/ INDEXH(-511:512), NOWREF
C
      SMDIF1 = 0.0
      SMDIF2 = 0.0
      SUMFO  = 0.0
      IREC2  = 1
      ISTART = 1
      READ (11,REC=IREC2) BUF
      DO 100 I=1,NOWREF
         SUMFO  = SUMFO  + BUF(4,ISTART)
         FC1 = BUF(5,ISTART)*BUF(5,ISTART) + BUF(6,ISTART)*BUF(6,ISTART)
         SMDIF1 = SMDIF1 + ABS(BUF(4,ISTART)-SCL1*SQRT(FC1))
         FC2 = BUF(7,ISTART)*BUF(7,ISTART) + BUF(8,ISTART)*BUF(8,ISTART)
         SMDIF2 = SMDIF2 + ABS(BUF(4,ISTART)-SCL2*SQRT(FC2))
         WF = 0.0
         IF (FC1.NE.0) WF = SQRT(FC2/FC1)
         WF = AMAX1(0.01,WF)
         A  = 2. - WF
         IF (WF .GT. 1.0) WF = AMAX1(0.01,A)
         BUF(5,ISTART) = 0.0
         BUF(6,ISTART) = 0.0
         IF (ABS(BUF(7,ISTART)).LT.1.E-8 .AND.
     .       ABS(BUF(8,ISTART)).LT.1.E-8) THEN
           BUF(7,ISTART)=0.0
           BUF(8,ISTART)=0.0
         ELSE
           A = ATAN2(BUF(8,ISTART),BUF(7,ISTART))
           BUF(7,ISTART) = WF
           BUF(8,ISTART) = A
         ENDIF
         ISTART = ISTART + 1
         IF(ISTART.GT.1023) THEN
            WRITE (11,REC=IREC2) BUF
            IREC2 = IREC2 + 1
            ISTART = 1
            IF (I.LT.NOWREF) READ (11,REC=IREC2) BUF
         END IF
  100 CONTINUE
      WRITE (11,REC=IREC2) BUF
      WRITE (*,110) SMDIF1/SUMFO, SMDIF2/SUMFO
  110 FORMAT(/,' R-FACTORS: ',/,' SUM|Fo-Fc|/SUM|Fo| FROM ORIGINAL ',
     .' MAP ',32X,F10.3,/,' SUM|Fo-Fc|/SUM|Fo| FROM TRUNCATED MAP ',
     .32X,F10.3)
      RETURN
      END
      SUBROUTINE WEIGHTS2 (SCL)
      INTEGER          NOWREF, I, IREC2, ISTART, INDEXH
      REAL             SCL, SUMFOB, SUMDIF, FC
      REAL             WF, FO, PP, PM, WP, BUF(8,1023)
      INTEGER          IH1(8,1023), IH2(8,1023), IH3(8,1023)
      EQUIVALENCE      (IH1(1,1),BUF(1,1)), (IH2(1,1),BUF(2,1)),
     .                 (IH3(1,1),BUF(3,1))
      COMMON / REFLNS / INDEXH(-511:512), NOWREF
      COMMON / BOX    / IBOX, MX1, MX2, MY1, MY2, MZ1, MZ2
C
      IREC2  = 1
      ISTART = 1
      READ (11,REC=IREC2) BUF
      SUMFOB = 0.0
      SUMDIF = 0.0
      DO 100 I=1,NOWREF
         FO = BUF(4,ISTART)
         SUMFOB = SUMFOB + FO
         FC = BUF(5,ISTART)*BUF(5,ISTART) + BUF(6,ISTART)*BUF(6,ISTART) 
         FC = SCL*SQRT(FC)
         SUMDIF = SUMDIF + ABS(FO-FC)
         IF (ABS(BUF(5,ISTART)).LT.1.E-8 .AND.
     .       ABS(BUF(6,ISTART)).LT.1.E-8) GOTO 90
         PP = ATAN2(BUF(6,ISTART),BUF(5,ISTART))
         WF = BUF(7,ISTART)
         PM = BUF(8,ISTART)
         WP = COS(PM-PP)
         IF (WP.LT.0.) WP = 0.
         BUF(5,ISTART) = WP*WF*FO*COS(PP)
         BUF(6,ISTART) = WP*WF*FO*SIN(PP)
   90    CONTINUE
         ISTART = ISTART + 1
         IF(ISTART.GT.1023) THEN
            WRITE (11,REC=IREC2) BUF
            IREC2  = IREC2 + 1
            ISTART = 1
            IF(I.LT.NOWREF) READ (11,REC=IREC2) BUF
         END IF
  100 CONTINUE
      IF (ISTART.NE.1) WRITE (11,REC=IREC2) BUF
      WRITE (*,110) IBOX, 'X', MX1, MX2,'Y', MY1, MY2, 'Z', MZ1, MZ2, 
     .SUMDIF/SUMFOB
  110 FORMAT(' BOX NUMBER',I3,': ',3(1X,A1,'=',I3,'-',I3,';'),
     .' RFAC FROM PHASING VOLUME ',F6.3)
      RETURN
      END
      SUBROUTINE FFTED (X, NDIM, RMSMAP)
C///
C     FOURIER SYNTHESIS FOR SPACE GROUP P1
C\\\
      INTEGER           NDIM, P1, P2, R, D(5), NXTREC, RECS, SKIP
      REAL              X(NDIM)
      COMMON / MINMAX / MINH, MAXH, MINK, MAXK, MINL, MAXL,
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, NX, NY, NZ
C             
      SCALE = 1.0/FLOAT(NX*NY*NZ)
C
C     CHECK SIZE OF FFT-ARRAY AND OPEN DIRECT ACCESS SCRATCH FILE
C
      P1 = NDIM/(2*NY*(MAXL + 1))
      P2 = NDIM/(NZ*NX)
      IF (P1 .LE. 0 .OR. P2 .LE. 0) THEN
         WRITE (*,100) NDIM, MAX( NZ*NX, 2*NY*(MAXL+1) )
  100    FORMAT(' INCREASE FFT-ARRAY SIZE FROM',I10,' TO AT LEAST',I10)
         WRITE (*,105) MAX( NZ*NX, 2*NY*(MAXL+1) )/6
  105    FORMAT(' TO DO THIS: INCREASE MAXREF TO AT LEAST',I10,
     +       ' AND RECOMPILE.')
         CALL CCPERR (1,'REQUEST TOO LARGE: INCREASE MAXREF')
      END IF
      R = 2*P1*P2*(MAXL+1)
      CALL CCPDPN(-67,'OMITFORT67','SCRATCH','DU',R,0)
C
C     PASS ONE.  TRANSFORM ON K AND WRITE INTERMEDIATE RESULTS ON 67.
C
      NXTREC = 1
      R      = -MAXH
  110 IF (R + P1 .GT. MAXH) P1 = MAXH + 1 - R
      CALL CLEARARRAY (X, NDIM)
      CALL RDREFL (X, NY, MAXL+1, P1, MAXK, R)
      D(1) = 2*NY*(MAXL + 1)*P1
      D(2) = 2
      D(3) = D(1)
      D(4) = D(1)
      D(5) = 2*NY
      CALL CMPL FT (X(1), X(2), NY, D)
      CALL GHC682 (X, NY, MAXL+1, P1, P2, 67, NXTREC)
      R = R + P1
      IF (R .LE. MAXH) GO TO 110
C
C     PASS 2 READS BACK THE INTERMEDIATE RESULTS, CALCULATES THE
C     TRANSFORMS ON H AND L, AND WRITES OUT THE FINAL Y SECTIONS
C
      P1 = NDIM/(2*NY*(MAXL + 1))
      SKIP = 0
      RECS = MAXY/P2
      R = 0
  200 IF (R + P2 .GT. MAXY) P2 = MAXY - R + 1
      NXTREC = SKIP + 1
      CALL GHC683 (X,NZ/2,NX,P2,MAXH,MAXL,P1,NXTREC,RECS,67)
      SKIP = SKIP + 1
C
C	X NOW CONTAINS INTERMEDIATE RESULTS STORED WITH L DOWN THE
C	COLUMNS AND H ACROSS THE ROWS.
C
      D(1) = NX*NZ*P2
      D(2) = NZ
      D(3) = NX*NZ
      D(4) = 2*(MAXL + 1)
      D(5) = 2
      CALL CMPL FT (X(1), X(2), NX, D)
      D(2) = 2
      D(4) = NX*NZ
      D(5) = NZ
      CALL HERM FT (X(1), X(2), NZ/2, D)
      CALL OUTMAP  (X, NZ, NX, P2, R, SCALE, RMSMAP)
      R = R + P2
      IF (R .LE. MAXY) GO TO 200
C
C	END OF JOB
C
      CLOSE (67)
      RETURN
      END
      SUBROUTINE RDREFL(X, NY, NZ, NX, KMAX, HS)
C
C     INPUT ROUTINE FOR P1 PROGRAM: PUTS STRUCTURE FACTORS INTO X-ARRAY
C     IT READS THE SORTED STRUCTURE FACTORS FROM THE DIRECT ACCESS FILE
C     ON UNIT 2.  THESE STRUCTURE FACTORS ARE ASSUMED TO BE SORTED ON H 
C     BUT DO NOT HAVE TO BE SORTED ON K OR L. L IS ASSUMED TO HAVE ONLY 
C     NON-NEGATIVE VALUES.  THE LIST MUST CONTAIN HK0, H-K 0 & 0K0;  IT
C     NEEDS NOT TO CONTAIN (-H K 0), (-H-K 0) & (0-K 0).
C
      COMMON / REFLN  / IOLD, BUF(8,1023), DUMMY(8)
      COMMON / REFLNS / INDEXH(-511:512), NOWREF
      INTEGER           IH(8,1023), IK(8,1023), IL(8,1023), H, K, L,
     .                  HS, HM, NX, NY, NZ, KMAX, NOWREF, INDEXH, IOLD
      EQUIVALENCE       (IH(1,1),BUF(1,1)), (IK(1,1),BUF(2,1)), 
     .                  (IL(1,1),BUF(3,1))
      COMPLEX           X(NY,NZ,NX), F
C
C     GET POINTERS TO FIRST RECORD (WITH IH-START) AND LAST RECORD (THAT
C     CONTAINS IH-MAX) IN THE DIRECT ACCESS REFLECTION FILE AND READ THE
C     REFLECTIONS FROM HS TO HM. STORE THEM IN THE X-ARRAY
C
      HM     = HS + NX - 1
      INDEX1 = INDEXH(HS)
      INDEX2 = INDEXH(HM+1) - 1
      IREC2  = (INDEX1-1)/1023 + 1
      READ (11,REC=IREC2) BUF
      ISTART = INDEX1-(IREC2-1)*1023
      DO 200 INDEX = INDEX1,INDEX2
         IF (ISTART.GT.1023) THEN
            IREC2  = IREC2 + 1
            READ (11,REC=IREC2) BUF
            ISTART = 1
         END IF
         H = IH(1,ISTART) - HS + 1
         K = IK(1,ISTART) + 1
         L = IL(1,ISTART) + 1
         IF (K .LE. 0) K = K + NY
C *** TEST
         IF (H.LT.1) THEN
            WRITE (*,145) K, L, H, IREC2, ISTART
  145       FORMAT(' RDREFL: K L H IREC2 ISTART ', 5I6)
            H = 1
         END IF
         X(K,L,H) = CMPLX(BUF(5,ISTART),BUF(6,ISTART))
         ISTART = ISTART + 1
  200 CONTINUE
C
C     FILL IN THE 0 -K 0 STRUCTURE FACTORS
C
c      IF (HS .GT. 0 .OR. HM .LT. 0 .OR. KMAX .LT. 1) RETURN
c      H = 1 - HS
c      DO 300 INDEX = 2, KMAX + 1
c      F = CONJG(X(INDEX,1,H))
c      write (*,'('' K : '',3i5,2f10.3,''; -K : '',3i5,2f10.3)')
c     .0,INDEX,0,x(INDEX,1,H),0,NY + 2 - INDEX,0,F
c      X((NY + 2 - INDEX),1,H) = CONJG(X(INDEX,1,H))
c  300 CONTINUE
      RETURN
      END
      SUBROUTINE GHC682 (X, NY, NZ, NX, SIZE, TAPE, IREC)
      INTEGER NY, NZ, NX, SIZE, TAPE, IREC, H, K, L, P, Q
      COMPLEX X(NY,NZ,NX)
C
C     WRITES Y SECTIONS FOR COMPLEX INTERMEDIATE RESULTS
C
      Q = 0
  100 P = Q + 1
      Q = Q + SIZE
      IF (Q .GE. NY) Q = NY
      WRITE (TAPE,REC=IREC) (((X(K,L,H),H = 1,NX),K = P,Q),L = 1,NZ)
      IREC = IREC + 1
      IF (Q .LT. NY) GO TO 100
      RETURN
      END
      SUBROUTINE GHC683 (X, NZ, NX, NY, HMAX, LMAX, SIZE,IREC,RECS,TAPE)
      INTEGER NX, NZ, NY, HMAX, LMAX, SIZE, RECS, TAPE, IREC
      INTEGER H, K, L, HL, HU, LM, P
      COMPLEX X(NZ,NX,NY)
C
C     READS IN INTERMEDIATE RESULTS FOR ALL H AND L FOR A BLOCK OF K.
C     THE INPUT IS STORED WITH L DOWN THE COLUMNS AND H ON THE ROWS.
C     THIS VERSION OF THE SUBROUTINE MAY BE USED FOR ALL MONOCLINIC
C     SPACE GROUPS FOR WHICH THE INTERMEDIATE RESULTS ARE COMPLEX, AS
C     WELL AS THE TRICLINIC SPACE GROUPS.
C
      LM = LMAX + 1
      P  = SIZE
      HU = NX - HMAX
C
C     READ DATA FOR NEGATIVE H
C
  200 HL = HU + 1
      HU = HU + P
      IF (HU .GT. NX) GO TO 700
      READ (TAPE,REC=IREC) (((X(L,H,K),H = HL,HU),K = 1,NY),L = 1,LM)
      IREC = IREC + RECS + 1
      IF (HU .LT. NX) GO TO 200
C
C     READ RECORDS FOR POSITIVE H
C
      HU = 0
  500 HL = HU + 1
      HU = HU + P
      IF (HU .GT. HMAX) HU = HMAX + 1
      READ (TAPE,REC=IREC) (((X(L,H,K),H = HL,HU),K = 1,NY),L = 1,LM)
      IREC = IREC + 1
      IF (HU .GT. HMAX) GO TO 800
  550 CONTINUE
      IREC = IREC + RECS
      GO TO 500
C
C     SECTION TO READ A RECORD SPANNING H = 0
C
  700 HU = HU - NX
      IF (HU .GT. HMAX) HU = HMAX + 1
      READ (TAPE,REC=IREC) (((X(L,H,K),H = HL,NX),(X(L,H,K),H = 1,HU),
     .     K = 1, NY), L = 1, LM)
      IREC = IREC + 1
      IF (HU .LE. HMAX) GO TO 550
C
C     PAD THE ARRAY WITH ZERO WHERE THERE ARE NO DATA, AND FILL IN THE
C     -H K 0 RESULTS BY SYMMETRY.
C
  800 DO 900 H = 2, HU
      HL = NX + 2 - H
      DO 900 K = 1, NY
      X(1,HL,K) = CONJG(X(1,H,K))
  900 CONTINUE
      HL = HMAX + 2
      HU = NX - HMAX
      IF (HU .LT. HL) GO TO 920
      DO 910 L = 1, LM
      DO 910 K = 1, NY
      DO 910 H = HL, HU
      X(L,H,K) = (0.0, 0.0)
  910 CONTINUE
C
  920 IF (LM .GE. NZ) GO TO 940
      P = LM + 1
      DO 930 K = 1, NY
      DO 930 L = P, NZ
      DO 930 H = 1, NX
      X(L,H,K) = (0.0, 0.0)
  930 CONTINUE
C
C     END OF INPUT
C
  940 CONTINUE
      RETURN
      END
      SUBROUTINE OUTMAP (X, NZ, NX, NY, Y, SCALE, RMSROW)
C
      REAL              X(NZ,NX,NY), BUF(8191), SCALE, RMSROW
      INTEGER           NZ, NX, NY, Y, SEC, MX, MY, MZ
      COMMON / MINMAX / MINH, MAXH, MINK, MAXK, MINL, MAXL,
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, MX, MY, MZ
      COMMON / BOX    / IBOX, MX1, MX2, MY1, MY2, MZ1, MZ2
      SAVE              RHOMIN, RHOMAX, INDEX, L, BUF
C
C     WRITE DENSITY OF BOX-VOLUME TO SCRATCH FILE
C
      IF (IBOX .NE. 0) THEN
         IY1 = Y
         IF (IY1.LT.MY1) IY1 = MY1
         IF (IY1.GT.MY2) RETURN
         IY2 = Y+NY-1
         IF (IY2.LT.MY1) RETURN
         IF (IY2.GT.MY2) IY2 = MY2
         WRITE (68) MX1, MX2, IY1, IY2, MZ1, MZ2
         DO 200 K = IY1-Y+1, IY2-Y+1
         DO 200 J = MX1+1, MX2+1
            DO 100 I = MZ1+1,MZ2+1
  100       RMSROW = RMSROW + X(I,J,K)*X(I,J,K)
  200    WRITE (68) (X(I,J,K),I=MZ1+1,MZ2+1)
      ELSE
C
C     PUT OUT Y-SECTIONS
C
         IF (Y .EQ. MINY) THEN
            CALL CCPDPN(-4,'OMITFORT4','SCRATCH','DU',8191,0)
            RHOMIN = 10000.
            RHOMAX =-10000.
            RMSROW =     0.
            INDEX  = 1
            L      = 1
         ENDIF
         DO 900 K = 1, NY
            RMSSEC = 0.0
            RMIN = 10000.
            RMAX =-10000.
            SEC = Y - MINY + K - 1
            DO 350 J = 1, NX
               DO 300 I = 1, NZ
                  R = X(I,J,K) * SCALE
                  IF (R .LT. RMIN) RMIN = R
                  IF (R .GT. RMAX) RMAX = R
                  RMSSEC = RMSSEC + R * R
                  RMSROW = RMSROW + R * R
                  BUF(L) = R
                  L = L + 1
                  IF (L .GT. 8191) THEN
                     WRITE (4,REC=INDEX) BUF
                     INDEX = INDEX + 1
                     L = 1
                  ENDIF
  300          CONTINUE
  350       CONTINUE
            RMSSEC = SQRT(RMSSEC/((MAXX-MINX+1)*(MAXZ-MINZ+1)))
            WRITE (*,360) SEC,SEC/FLOAT(MY),RMIN,RMAX,RMSSEC
  360       FORMAT(' SECTION',I4,' ("Y" = ',F5.3,') DENSITY ',
     .      'RANGE =',F10.3,' TO ',F10.3,' RMS =',F9.3)
            IF (RMIN .LT. RHOMIN) RHOMIN = RMIN
            IF (RMAX .GT. RHOMAX) RHOMAX = RMAX
  900    CONTINUE
         IF (L.NE.1) WRITE (4,REC=INDEX) BUF
C
         IF (SEC .GE. MAXY ) THEN
            RMSROW = SQRT(RMSROW/(MX*MY*MZ))
            WRITE (*,920) RHOMIN, RHOMAX, RMSROW
  920       FORMAT(' OVERALL DENSITY:  MINIMUM, MAXIMUM, RMS =',
     .      F10.3,F14.3,F15.3)
         ENDIF
      END IF
      RETURN
      END

      SUBROUTINE COLECT (MX, MY, MZ, X, SCALE)
      PARAMETER         (MXHIST=256)
      DIMENSION         X(0:MZ,0:MX,0:MY), UVWMAT(3,3), R(0:1000)
      CHARACTER         MAPTITLE*80
      INTEGER           MX, MY, MZ, NX, NY, NZ, MINH, MAXH, MINK, MAXK,
     .                  MINL, MAXL, MINX, MAXX, MINY, MAXY, MINZ, MAXZ, 
     .                  I, J, K, MX1, MX2, MY1, MY2, MZ1, MZ2, NSYM,
     .                  IUN, IUVW(3), MXYZ(3), ISYUNIT, IMODE
      COMMON /MFFTIT/ MAPTITLE
      COMMON /IPARAM/ NFOBS, NFCALC, NPHASE, ITRUNC, IMFF
      COMMON / MINMAX / MINH, MAXH, MINK, MAXK, MINL, MAXL,
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, NX, NY, NZ
      COMMON / MDFPAR / MAXR, MAXB, CELL(6), ISLOW, INTER, IFAST,
     .                  KLASS, ICENTR, ISCREW(3), IVERSN
      COMMON / MDFSYM / NSPGRP, NSYM, SYMROT(3,3,96), SYMTRL(3,96)
      COMMON / HIST   / IHIST, VMIN1, VMIN2, VMAX1, VMAX2,HSCF1,HSCF2,
     .                  IHIST1(MXHIST), IHIST2(MXHIST), LOOKUP(MXHIST)

C     Open scratch file to hold map if histogram mapping
C     is to be done.

      CALL CCPDPN(-94,'OMITTMP7','SCRATCH','U',0,0)

      IUN=12
      IF (IMFF.EQ.0) THEN

C  Open MAPOUT for writing CCP4 map and write map header
        IUVW(1) = 3
        IUVW(2) = 1
        IUVW(3) = 2
        MXYZ(1) = NX
        MXYZ(2) = NY
        MXYZ(3) = NZ
        IMODE = 2
        call mwrhdr(IUN,MAPTITLE,NY,IUVW,MXYZ,0,0,NZ-1,0,NX-1,
     .         CELL,NSPGRP,IMODE)
C  Write symmetry operators to output map based on space-group
        ISYUNIT = 13
        call msyput(ISYUNIT,NSPGRP,IUN)

      ELSE IF (IMFF.EQ.1) THEN
C
C     OPEN OUTPUT-MFF WITH OMIT-MAP AND WRITE HEADER INFORMATION 
C
        IFAIL = 0
        CALL CCPDPN(IUN,'MAPOUT','NEW','U',0,IFAIL)
        WRITE(IUN)    MAPTITLE
        WRITE(*,10) MAPTITLE
   10   FORMAT(//,' OUTPUT_MFF WITH OMITMAP:',/,' TITLE',14X,A)
        WRITE(IUN)    CELL
        WRITE(*,'('' CELL DIMENSIONS    '',3F12.3,1X,3F8.2)') CELL
        WRITE(IUN)    CELL(3)/NZ, CELL(1)/NX, CELL(2)/NY
        WRITE(*,'('' STEPUVW (FRACTNL)  '',3F12.5)') 
     .  CELL(3)/NZ, CELL(1)/NX, CELL(2)/NY
        WRITE(IUN)    3, 1, 2
        WRITE(*,'('' UVW DIRECTION           '',I7,2I12)') 3, 1, 2
        WRITE(IUN)    NX, NY, NZ
        WRITE(*,'('' NR OF DIVISIONS OF A,B,C'',I7,2I12)') NX, NY, NZ
        WRITE(IUN)    NZ, NX, NY
        WRITE(*,'('' NR OF POINTS(U,V,W)'',3I12)') NZ, NX, NY
        WRITE(IUN)    0.0, 0.0, 0.0
        WRITE(*,'('' UVW ORIGIN'',9X,3F12.7)') 0.0, 0.0, 0.0
        CALL CLEARARRAY (UVWMAT, 9)
        UVWMAT(3,1) = 1./NZ
        UVWMAT(1,2) = 1./NX
        UVWMAT(2,3) = 1./NY
        WRITE(IUN)     UVWMAT
        WRITE(*,'(3('' UVW MATRIX'',9X,3F12.7,/))') UVWMAT

      ENDIF
C
      RHOMIN =  10000.0
      RHOMAX = -10000.0
      RMSROW =      0.0
      IYSEC1 = 0
   50 IYSEC2 = IYSEC1 + MY
      IF (IYSEC2.GT.NY-1) IYSEC2 = NY-1
C
C     START READING THE FILE WITH ELECTRON DENSITIES
      I = NX*NZ*(MY+1)
      CALL CLEARARRAY (X,I)
      REWIND 68
  100 READ (68,END=400) MX1, MX2, MY1, MY2, MZ1, MZ2
      DO 300 K = MY1, MY2
      DO 300 J = MX1, MX2
         READ (68) (R(I),I=MZ1,MZ2)
         DO 200 I = MZ1, MZ2
         R(I) = SCALE*R(I)
         DO 200 NS = 1,NSYM
            KK = NINT(SYMROT(2,1,NS)*J) + NINT(SYMROT(2,2,NS)*K)
     .         + NINT(SYMROT(2,3,NS)*I) + NINT(SYMTRL(2,NS)*NY)
            KK = MOD (KK,NY)
            IF (KK.LT.0) KK = KK + NY
            IF (KK.LT.IYSEC1) GOTO 200
            IF (KK.GT.IYSEC2) GOTO 200
            KK = KK - IYSEC1
            JJ = NINT(SYMROT(1,1,NS)*J) + NINT(SYMROT(1,2,NS)*K)
     .         + NINT(SYMROT(1,3,NS)*I) + NINT(SYMTRL(1,NS)*NX)
            JJ = MOD (JJ,NX)
            IF (JJ.LT.0) JJ = JJ + NX
            II = NINT(SYMROT(3,1,NS)*J) + NINT(SYMROT(3,2,NS)*K)
     .         + NINT(SYMROT(3,3,NS)*I) + NINT(SYMTRL(3,NS)*NZ)
            II = MOD (II,NZ)
            IF (II .LT. 0) II = II + NZ
  150       X(II,JJ,KK) = R(I)
  200    CONTINUE
  300 CONTINUE
      GOTO 100
  400 DO 500 ISEC = IYSEC1, IYSEC2
      RMIN   = 0.0
      RMAX   = 0.0
      RMSSEC = 0.0
      DO 450 J = 0,NX-1
      DO 450 I = 0,NZ-1
         IF (RMIN.GT.X(I,J,ISEC-IYSEC1)) RMIN = X(I,J,ISEC-IYSEC1)
         IF (RMAX.LT.X(I,J,ISEC-IYSEC1)) RMAX = X(I,J,ISEC-IYSEC1)
         RMSSEC = RMSSEC + X(I,J,ISEC-IYSEC1) * X(I,J,ISEC-IYSEC1)
  450 CONTINUE
      RMSROW = RMSROW + RMSSEC
      RMSSEC = SQRT(RMSSEC/(NX*NZ))
      WRITE (*,460) ISEC,ISEC/FLOAT(NY),RMIN,RMAX,RMSSEC
  460 FORMAT(' SECTION',I4,' ("Y" = ',F5.3,') DENSITY ',
     .'RANGE =',F10.3,' TO ',F10.3,' RMS =',F9.3)
      IF (RHOMIN .GT. RMIN) RHOMIN = RMIN
      IF (RHOMAX .LT. RMAX) RHOMAX = RMAX
      IF (IHIST.EQ.0) THEN
        IF (IMFF.EQ.0) THEN
          call MSPEW(IUN,X(0,0,ISEC-IYSEC1))
        ELSE IF (IMFF.EQ.1) THEN
          WRITE (IUN) ((X(I,J,ISEC-IYSEC1),I=0,NZ-1),J=0,NX-1)
        ENDIF
      ELSE
        WRITE (94) ((X(I,J,ISEC-IYSEC1),I=0,NZ-1),J=0,NX-1)
      ENDIF
  500 CONTINUE
      IYSEC1 = IYSEC2 + 1
      IF (IYSEC1.LT.NY) GO TO 50
 1000 CONTINUE
      RMSROW = SQRT(RMSROW/(NX*NY*NZ))
      WRITE (*,1010) RHOMIN, RHOMAX, RMSROW
 1010 FORMAT(' OVERALL DENSITY:  MINIMUM, MAXIMUM, RMS =',
     .F10.3,F14.3,F15.3)
      IF (IHIST.EQ.1) THEN
         REWIND 94
         CALL RESTOR(IUN,RHOMIN,RHOMAX,NZ,NX,NY)
      ENDIF

C     Close map file.
      IF (IMFF.EQ.0) THEN
        call MWCLOSE(IUN)
      ELSE IF (IMFF.EQ.1) THEN
        CLOSE(IUN)  
      ENDIF

      RETURN
      END

      SUBROUTINE CLEARARRAY  ( ANAME , N )
C///
C=======================================================================
C     PURPOSE :
C-----------------------------------------------------------------------
C
C             THIS ROUTINE SET THE FIRST N ELEMENTS OF ARRAY " NAME "
C         TO ZERO
C
C=======================================================================
C.......................................................................
C
C          THIS ROUTINE IS WRITTEN FOR CYBER 170/760 IN FTN5 BY
C
C                        W I L    G A Y K E M A
C
C                        DEPARTMENT  OF  CHEMICAL  PHYSICS
C                        STATE  UNIVERSITY  GRONINGEN
C                        NIJENBORGH  16
C                        GRONINGEN
C                        THE  NETHERLANDS
C
C
C          VERSION 82/11/12-WG
C
C.......................................................................
C=======================================================================
C     USAGE :
C-----------------------------------------------------------------------
C
C         CALL  CLEARARRAY  ( NAME , N )
C         =========================
C
C             ARGUMENTS:
C                 NAME - FIRST ELEMENT OF VARIABLE TO SET TO ZERO
C                 N    - AN INTEGER CONSTANT OR VARIABLE WHICH
C                           SPECIFIES THE NUMBER OF VARIABLES
C                           TO SET TO ZERO
C
C=======================================================================
C\\\
      DIMENSION ANAME(N)
      INTEGER I,N
 
      DO 100,I = 1, N
        ANAME(I) = 0.0
  100 CONTINUE
 
      END
      SUBROUTINE DCALC (CELL)
C///
C     THE RESOLUTION CAN BE CALCULATED AS FOLLOWS: 
C
C     DSTAR-SQUARED = IH*IH*A11 + IH*IK*A21 + IH*IL*A31
C                   + IK*IK*A22 + IK*IL*A32 + IL*IL*A33
C     SO:       RES = 1.0/SQRT(DSTAR_SQUARED)
C
C     IN THIS SUBROUTINE THE COEFFICIENTS A11, A21, A31, A22, A32, A33
C     ARE CALCULATED
C
C\\\
      DIMENSION         CELL(6)
      COMMON / MDFRES / A11, A21, A22, A31, A32, A33
      RAD  =  ATAN(1.0)/45.
      S    = RAD*CELL(4)
      CA   = COS(S)
      SA   = SIN(S)
      S    = RAD*CELL(5)
      CB   = COS(S)
      SB   = SIN(S)
      S    = RAD*CELL(6)
      CG   = COS(S)
      SG   = SIN(S)
      CAST = (CB*CG - CA)/ (SB*SG)
      CBST = (CG*CA - CB)/ (SG*SA)
      CGST = (CA*CB - CG)/ (SA*SB)
      SAST = SQRT(1.0 - CAST*CAST)
      SBST = SQRT(1.0 - CBST*CBST)
      SGST = SQRT(1.0 - CGST*CGST)
      AST  = 1.0/(CELL(1)*SB*SGST)
      BST  = 1.0/(CELL(2)*SG*SAST)
      CST  = 1.0/(CELL(3)*SA*SBST)
      A11  = AST*AST
      A21  = 2.0*AST*BST*CGST
      A22  = BST*BST
      A31  = 2.0*AST*CST*CBST
      A32  = 2.0*BST*CST*CAST
      A33  = CST*CST

      END
      SUBROUTINE LSTSYM (NSYM, RSYM)
C///
C=======================================================================
C
C                 S U B R O U T I N E      L S T S Y M
C
C      THIS SUBROUTINE CONVERTS THE SPACEGROUP SYMMETRY OPERATIONS
C      WRITTEN IN MATRIX NOTATION INTO THE  "INTERNATIONAL TABLES"
C      FORMAT,  AND ECHOES THEM TO THE OUTPUT-FILE IN THAT FORMAT.
C
C=======================================================================
C
C      VARIABLES IN CALLING STATEMENT :
C
C      NSYM     -  NUMBER OF SYMMETRY OPERATIONS
C      RSYM     -  REAL ARRAY WITH SYMMETRY OPERATIONS
C                  DIMENSIONED AS  RSYM (4, 4, NSYM)
C                  SYMMETRY OPERATIONS ARE TO BE USED AS:
C
C                  X' = R(1,1)*X + R(1,2)*Y + R(1,3)*Z + R(1,4)
C                  Y' = R(2,1)*X + R(2,2)*Y + R(2,3)*Z + R(2,4)
C                  Z' = R(3,1)*X + R(3,2)*Y + R(3,3)*Z + R(3,4)
C
C=======================================================================
C      VERSION 19 DECEMBER 1989  BAUKE DIJKSTRA
C=======================================================================
C\\\
      CHARACTER        LINE*80, I1TO9*9, CHXYZ*3, ITR*3, ISIGN*1, DUM*4
      REAL             RSYM ( 4, 4, NSYM)
      LOGICAL          NFIRST, ERROR
      INTEGER          NSYM, I, J, L, N, ISYMIJ, ITRANS
      COMMON /CHARS /  LINE, I1TO9, CHXYZ, ITR, ISIGN, DUM

C     INITIALIZE SOME CONSTANTS

      I1TO9  = '123456789'
      CHXYZ  = 'XYZ'
      ERROR  = .FALSE.

C     LOOP OVER ALL NSYM SYMMETRY OPERATIONS

      DO 500 N = 1, NSYM
         LINE = ' '
         L = 1

C     LOOP OVER (CHXYZ(I),I=1,3)  :  X, Y, Z

         DO 300 I = 1, 3
         IF (I .GT. 1) THEN
            LINE = LINE(1:L) // ',  '
            L = 1 + 20*(I-1)
         ENDIF
         LINE = LINE(1:L) // CHXYZ(I:I) // ''' = '
         L = L + 5
         NFIRST = .TRUE.

C     LOOP OVER ROTATION PART OF THE SYMMETRY OPERATOR

         DO 100 J = 1, 3
            ISYMIJ = NINT(RSYM(I,J,N))
            IF ( ISYMIJ .EQ. 0) THEN
                  GOTO 100
            ELSE IF ( ISYMIJ .LT. 0) THEN
                  ISIGN  = '-'
                  ISYMIJ = IABS (ISYMIJ)
            ELSE
                  ISIGN  = '+'
                  IF (NFIRST) ISIGN = ' '
            ENDIF
            NFIRST = .FALSE.
            IF (ISYMIJ .EQ. 1) THEN
                  LINE   = LINE(1:L) // ISIGN // CHXYZ(J:J)
                  L      = L + 2
            ELSE
                  LINE   = LINE(1:L) // ISIGN //
     .                     I1TO9 (ISYMIJ:ISYMIJ) // CHXYZ(J:J)
                  L      = L + 3
            ENDIF
  100    CONTINUE

C     THE TRANSLATION VECTOR IS STORED IN RSYM(I,4,N) AS A
C     TRANSLATION IN A FRACTION OF THE CELL AXIS

         ISIGN  = '+'
         ITRANS = NINT(12.0*RSYM(I,4,N))
         IF ( ITRANS .LT. 0 ) THEN
            ISIGN = '-'
            ITRANS = -ITRANS
         ENDIF
         IF ( ITRANS .EQ. 0 ) THEN
            GO TO 300
         ELSE IF ( ITRANS .EQ. 2) THEN
            ITR = '1/6'
         ELSE IF ( ITRANS .EQ. 3) THEN
            ITR = '1/4'
         ELSE IF ( ITRANS .EQ. 4) THEN
            ITR = '1/3'
         ELSE IF ( ITRANS .EQ. 6) THEN
            ITR = '1/2'
         ELSE IF ( ITRANS .EQ. 8) THEN
            ITR = '2/3'
         ELSE IF ( ITRANS .EQ. 9) THEN
            ITR = '3/4'
         ELSE IF ( ITRANS .EQ.10) THEN
            ITR = '5/6'
         ELSE
            ITR = '***'
            ERROR = .TRUE.
         ENDIF
         LINE = LINE (1:L) // ISIGN // ITR
         L = L + 4
  300 CONTINUE
      WRITE (*, '(I5,''. :  '',A) ') N, LINE (1:L)

  500 CONTINUE
      IF (ERROR) CALL CCPERR (1,'ILLEGAL SYMMETRY OPERATOR IN LIST.')

      END
      SUBROUTINE PACK (IH, IK, IL, IHKLI)
C///
C     PACKS IH,IK,IL INTO ONE WORD  -  USES 30 BITS OF THAT WORD
C     SHOULD THEREFORE WORK ON ALL COMPUTERS WITH WORD LENGTH OF
C     32 BITS AND MORE.
C     LIMITATION IS THAT MAXIMUM ABSOLUTE VALUE OF ANY ONE INDEX
C     IS 511 .  THIS MEANS  2.0  ANGSTROM RESOLUTION FOR A  1000
C     ANGSTROM AXIS, WHICH IS SUFFICIENT IN MOST CASES.
C\\\

      IF (IABS(IH) .GE. 512)  GOTO 10
      IF (IABS(IK) .GE. 512)  GOTO 10
      IF (IABS(IL) .GE. 512)  GOTO 10

C OLD CODE
      IHKLI  =  (IH+512)*1048576 + (IK+512)*1024 + (IL+512)

C THIS IS A BIT FASTER. 512*1048576+512*1024+512=537395712 (BART H6)
c     IHKLI  =  IH*1048576 + IK*1024 + IL + 537395712

      RETURN

   10 WRITE(*,20) IH,IK,IL
   20 FORMAT(' ERROR IN INDICES: IABS(IH, IK, IL) TOO LARGE: ',3I6)
      CALL CCPERR(1,'ERROR IN PACK: IH,IK OR IL OUT OF BOUNDS')

      END
      SUBROUTINE MKHIST(ARRAY)
      PARAMETER (MXHIST=256)
C
C     THIS SUBROUTINE COMPUTES THE DENSITY HISTOGRAM
C
      INTEGER           MINH, MAXH, MINK, MAXK, MINL, MAXL,
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, NX, NY, NZ
      COMMON / MINMAX / MINH, MAXH, MINK, MAXK, MINL, MAXL,
     .                  MINX, MAXX, MINY, MAXY, MINZ, MAXZ, NX, NY, NZ
      DIMENSION         ARRAY(MINZ:MAXZ,MINX:MAXX,MINY:MAXY),BUFED(8191)
      COMMON / HIST   / IHIST, VMIN1, VMIN2, VMAX1, VMAX2,HSCF1,HSCF2,
     .                  IHIST1(MXHIST), IHIST2(MXHIST), LOOKUP(MXHIST)
C
C     ZERO HISTOGRAM ARRAY
C
      DO 5,I=1,MXHIST
         IHIST1(I)=0
    5 CONTINUE
C
C     READ THE MAP
C
      INDEX  = 1
      READ (4,REC=INDEX) BUFED
      K = 1
      DO 100 J = MINY, MAXY
      DO 100 I = MINX, MAXX
      DO 100 L = MINZ, MAXZ
         ARRAY(L,I,J) = 100.*BUFED(K)
         K = K + 1
         IF (K .GT. 8191) THEN
            INDEX = INDEX + 1
            READ  (4,REC=INDEX) BUFED
            K = 1
         END IF
  100 CONTINUE
      VMIN1=+1.0E30
      VMAX1=-1.0E30
      NTOT=0
C
C     LOOP THROUGH THE MAP
C
      DO 140,K=MINY,MAXY
        DO 120,J=MINX,MAXX
          DO 110,I=MINZ,MAXZ
            DEN=ARRAY(I,J,K)
C
C     UPDATE MIN AND MAX DENSITIES
C
            VMIN1=MIN(VMIN1,DEN)
            VMAX1=MAX(VMAX1,DEN)
	    NTOT=NTOT+1
  110     CONTINUE
  120   CONTINUE
  140 CONTINUE
C
C     COMPUTE THE HISTOGRAM SCALE FACTOR
C
      HSCF1=INT(MXHIST-1)/(VMAX1-VMIN1)
C
C     SECOND PASS THROUGH THE MAP, TO COMPUTE THE HISTOGRAM
C
      DO 200,K=MINY,MAXY
	DO 180,J=MINX,MAXX
          DO 160,I=MINZ,MAXZ
            DEN=ARRAY(I,J,K)
C
C     COMPUTE HISTOGRAM INDEX
C
            INDH1=INT(HSCF1*(DEN-VMIN1)+1.)
C
C     SUM UP INTO HISTOGRAM 1
C
	    IHIST1(INDH1)=IHIST1(INDH1)+1
  160     CONTINUE
  180   CONTINUE
  200 CONTINUE
      WRITE (*,300) MXHIST,VMIN1,VMAX1,NTOT
  300 FORMAT(/,5X,'ELECTRON DENSITY HISTOGRAM FILE COMPUTED',
     .       /,5X,'NUMBER OF BINS: ',I5,
     .       /,5X,'MIN DENSITY: ',F10.3,
     .       /,5X,'MAX DENSITY: ',F10.3,
     .       /,5X,'TOTAL NUMBER OF POINTS IN THE HISTOGRAM: ',I10,//)
      RETURN
      END
      SUBROUTINE RESTOR(IUN,RHOMIN,RHOMAX,NZ,NX,NY)
C
C     THIS SUBROUTINE CONVERTS THE HISTOGRAM OF ELECTRON
C     DENSITIES OF THE OMIT MAP INTO THAT OF THE INITIAL
C     MAP, THEN WRITES THE MAP OUT
C
      PARAMETER (MAXNU=200,MAXNV=200)
      PARAMETER (MXHIST=256)
      COMMON /IPARAM/ NFOBS, NFCALC, NPHASE, ITRUNC, IMFF
      COMMON/HIST/IHIST,VMIN1,VMIN2,VMAX1,VMAX2,HSCF1,HSCF2,
     .            IHIST1(MXHIST),IHIST2(MXHIST),LOOKUP(MXHIST)
      REAL MAP(MAXNU,MAXNV)
      INTEGER IUN
C     INTEGER NUVW(3),IHIST3(MXHIST)
C     EQUIVALENCE (NUVW(1),NZ),(NUVW(2),NX),(NUVW(3),NY)
      DO 10,I=1,MXHIST
	IHIST2(I)=0
   10 CONTINUE
      ALLMIN=+1.0E30
      ALLMAX=-1.0E30
      VMIN2=100.*RHOMIN
      VMAX2=100.*RHOMAX
      HSCF2=INT(MXHIST-1)/(VMAX2-VMIN2)
      NTOT=0
C
C     START PASSING THROUGH THE MAP
C
      DO 80,K=1,NY
	READ (94) ((MAP(I,J),I=1,NZ),J=1,NX)
	DO 60,J=1,NX
	  DO 40,I=1,NZ
	    MAP(I,J)=MAP(I,J)*100.
C
C     COMPUTE HISTOGRAM INDEX
C
	    INDH2=INT(HSCF2*(MAP(I,J)-VMIN2)+1.)
	    NTOT=NTOT+1
C
C     SUM UP INTO HISTOGRAM 2
C
	    IHIST2(INDH2)=IHIST2(INDH2)+1
   40     CONTINUE
   60   CONTINUE
   80 CONTINUE
      WRITE (*,100) VMIN2,VMAX2,NTOT
  100 FORMAT(//,5X,'BEFORE HISTOGRAM RESTORATION',
     .       //,5X,'MIN RHO: ',F10.3,
     .        /,5X,'MAX RHO: ',F10.3,
     .        /,5X,'TOTAL NUMBER OF POINTS IN THE HISTOGRAM: ',I10,
     .       //,5X,'HISTOGRAM OF ELECTRON DENSITY GENERATED')
C
C     COMPUTE THE HISTOGRAM TO HISTOGRAM SCALE FACTOR
C
      SUMH1=0.
      SUMH2=0.
      DO 300,I=1,MXHIST
	SUMH1=SUMH1+REAL(IHIST1(I))
	SUMH2=SUMH2+REAL(IHIST2(I))
  300 CONTINUE
      HSCLF=SUMH2/SUMH1
      NTOT=0
C
C     SCALE HISTOGRAM 1 ONTO HISTOGRAM 2
C
      DO 320,I=1,MXHIST
	IHIST1(I)=NINT(REAL(IHIST1(I))*HSCLF)
	NTOT=NTOT+IHIST1(I)
  320 CONTINUE
      VALUE1=0.
      VALUE2=0.
      IVAL=1
C
C     COMPUTE THE HISTOGRAM MAPPING LOOKUP TABLE
C
      DO 580,I=1,MXHIST
	VALUE1=VALUE1+REAL(IHIST1(MXHIST-I+1))
	DO 540,J=IVAL,MXHIST
	  VALUE2=VALUE2+REAL(IHIST2(MXHIST-J+1))
	  LOOKUP(MXHIST-J+1)=MXHIST-I+1
	  IF (VALUE2.LE.VALUE1) GOTO 540
	  LOOKUP(MXHIST-J+1)=MXHIST-I+1
	  VALUE2=VALUE2-REAL(IHIST2(MXHIST-J+1))
	  GOTO 560
  540   CONTINUE
  560   CONTINUE
	IVAL=J
  580 CONTINUE
      WRITE (*,590) VMIN1,(REAL(MXHIST-1)/HSCF1)+VMIN1,NTOT
  590 FORMAT( /,5X,'HISTOGRAM MAPPING LOOKUP TABLE GENERATED',
     .       //,5X,'AFTER SCALING BY HISTOGRAM MAPPING',
     .       //,5X,'MIN RHO: ',F10.3,
     .        /,5X,'MAX RHO: ',F10.3,
     .        /,5X,'NUMBER OF POINTS IN HISTOGRAM 1: ',I10,
     .       //,5X,'OUTPUT ELECTRON DENSITY:',
     .        /,5X,'SECTION   MINIMUM   MAXIMUM')
C
C     NOW GENERATE THE OUTPUT FILE AFTER HAVING RESTORED THE
C     ORIGINAL HISTOGRAM OF E.D.
C
      REWIND 94
      DO 680,K=1,NY
	SECMIN=+1.0E30
	SECMAX=-1.0E30
	READ (94) ((MAP(I,J),I=1,NZ),J=1,NX)
	DO 640,J=1,NX
	  DO 620,I=1,NZ
	    MAP(I,J)=MAP(I,J)*100.
	    INDH2=INT(HSCF2*(MAP(I,J)-VMIN2)+1.)
	    MAP(I,J)=(REAL(LOOKUP(INDH2)-1)/HSCF1)+VMIN1
C 600       CONTINUE
            SECMIN=MIN(SECMIN,MAP(I,J))
            SECMAX=MAX(SECMAX,MAP(I,J))
            ALLMIN=MIN(ALLMIN,MAP(I,J))
            ALLMAX=MAX(ALLMAX,MAP(I,J))
  620     CONTINUE
  640   CONTINUE
        IF (IMFF.EQ.0) THEN
          call MWRSEC(IUN,MAP,MAXNU,MAXNV,1,NZ,1,NX)
        ELSE IF (IMFF.EQ.1) THEN
          WRITE (IUN) ((MAP(I,J),I=1,NZ),J=1,NX)
        ENDIF
        WRITE (*,660) K,SECMIN,SECMAX
  660 FORMAT(5X,I4,3X,2F10.4)
  680 CONTINUE
      WRITE (*,700) ALLMIN,ALLMAX
  700 FORMAT( /,5X,'AFTER HISTOGRAM RESTORATION',
     .       //,5X,'OVERALL MIN RHO: ',F10.3,
     .        /,5X,'OVERALL MAX RHO: ',F10.3,/)
      CLOSE (UNIT=94)
      RETURN
      END
      SUBROUTINE RDMTZ(IPRINT,RBFINF,MAXRBF,MAXBUF)

C SUBROUTINE TO READ THE HEADER RECORDS OF AN MTZ FILE AND TO EXTRACT
C FROM IT THE DATA NORMALLY OBTAINED FROM THE MDF HEADER. THIS HEADER
C INFORMATION IS PASSED TO THE CALLING ROUTINE BY THE COMMON BLOCKS
C /MDFPAR/, /MDFTTL/ AND /MDFSYM/. MTZ-SPECIFIC INFORMATION IS STORED
C IN COMMON BLOCK /MTZINF/. THIS COMMON BLOCK IS SAVED FOR USE IN THE
C SUBROUTINES WRMTZ, RDREC AND WRTREC. IF INFORMATION IN /MTZINF/ HAS
C TO BE MODIFIED THEN THE COMMON BLOCK HAS TO BE DECLARED AND SAVED IN
C THE ROUTINE WHICH WANTS TO MODIFY THIS DATA.
C !!! WARNING !!!, THIS ROUTINE CANNOT HANDLE MULTIPLE-RECORD MTZ FILES

      IMPLICIT NONE

      INTEGER NPARM
      PARAMETER  (NPARM=200)

C DECLARATION OF SUBROUTINE ARGUMENTS
      INTEGER IPRINT, MAXRBF, MAXBUF 
      CHARACTER*80 RBFINF(MAXRBF)

C DECLARATION OF MDF COMMON BLOCKS
      CHARACTER TITLE*80
      REAL CELL,SYMROT,SYMTRL
      INTEGER MAXR,MAXB,ISLOW,INTER,IFAST,KLASS,ICENTR,ISCREW,IVERSN,
     $  NSYM, NSPGRP
      COMMON /MDFPAR/ MAXR, MAXB, CELL(6), ISLOW, INTER, IFAST,
     $                KLASS, ICENTR, ISCREW(3), IVERSN
      COMMON /MDFTTL/ TITLE(20)
      COMMON /MDFSYM/ NSPGRP, NSYM, SYMROT(3,3,96), SYMTRL(3,96)

C DECLARATION OF MTZ-SPECIFIC COMMON BLOCK INFO
      INTEGER MCOLS
      PARAMETER (MCOLS=500)
      INTEGER NPRIM,NLPRGI,LOOKUP(MCOLS)
      INTEGER NFOBS, NFCALC, NPHASE, ITRUNC, IMFF
      REAL SCLFOB, SCLFC, RESMIN, RESMAX, DSTLMT
      LOGICAL LOGRES,LOGGRID
      CHARACTER CTYPIN(MCOLS)*1,CTYPOUT(MCOLS)*1,SPGNAM*10,PGNAM*10
      CHARACTER CENTR*1
      CHARACTER*200  LABIN_SAVE
      COMMON /MTZINF/NPRIM,CTYPIN,CTYPOUT,SPGNAM,PGNAM,CENTR
      COMMON /LABINFO/ LABIN_SAVE
      COMMON /IPARAM/ NFOBS, NFCALC, NPHASE, ITRUNC, IMFF
      COMMON /RPARAM/ SCLFOB, SCLFC, RESMIN, RESMAX, DSTLMT
      COMMON /LPARAM/ LOGRES,LOGGRID

      SAVE /MTZINF/
C    
C---- things for parser
C
      CHARACTER KEY*4,LINE*600,CVALUE(NPARM)*4
      REAL      FVALUE(NPARM)
      LOGICAL   LEND,LOGDEBUG
      INTEGER   NTOK,IBEG(NPARM),IEND(NPARM),ITYPE(NPARM),
     +          IDEC(NPARM)

C DECLARATION OF LOCAL VARIABLES
      REAL RANGES(2,MCOLS),SYMOP(4,4,96),SMINWL,SMAXWL
      INTEGER IFAIL,IDUM,L,I,M,N,SORTX(5),NCOL,NREFLX,ISCRW,ISPGRP
      CHARACTER CLABS(MCOLS)*30,MTZTIT(31)*80,VERSION*10
      CHARACTER HKL(3)*1,CCLASS(16)*10
      CHARACTER CTPRGI(MCOLS)*1,LSPRGI(MCOLS)*30

      DATA NLPRGI /6/
      DATA LOOKUP/-1,-1,-1,0,0,0,494*0/
C---- LSPRGI   array containing the program label strings
C     
      DATA LSPRGI/'H','K','L','FP','FC','PHI',494*' '/
C     
C---- CTPRGI   array containing the program label types
C     
      DATA CTPRGI/'H','H','H','F','F','P',494*' '/
C     
      DATA          HKL / 'H', 'K', 'L'/
      DATA   CCLASS/'TRICLINIC ','MONOCLIN-B','O-RHOMBIC ','TETRAGNL-1',
     $              'TETRAGNL-2','TRIGONAL-1','TRIGONAL-2','TRIGONAL-3',
     $              'TRIGONAL-4','TRIGONAL-5','HEXAGONL-1','HEXAGONL-2',
     $              'CUBIC-1   ','CUBIC-2   ','MONOCLIN-A','MONOCLIN-C'/

C EMPTY TITLE RECORDS
      DO 50,L=1,31
        DO 40,I=1,80
          MTZTIT(L)(I:I)=' '
   40   CONTINUE
   50 CONTINUE

C----Parse LABIN_SAVE for labels
      LEND = .FALSE.
      LOGDEBUG = .TRUE.
      NTOK = NPARM
      LINE = LABIN_SAVE
C
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYPE,FVALUE,CVALUE,IDEC,NTOK,
     +              LEND,LOGDEBUG)

C----Get labels. Needed for LRASSN
      CALL LKYIN(1,LSPRGI,NLPRGI,NTOK,LINE,IBEG,IEND)
 
      CALL LROPEN(1,'HKLIN',IPRINT,IFAIL)
      IF(IFAIL.NE.0)THEN
        WRITE(*,*)' ERROR IN OPENING FILE: HKLIN'
        WRITE(*,*)' CHECK YOUR INPUT AND YOUR MTZ'
        WRITE(*,*)
        CALL CCPERR(1,' ERROR: CANNOT OPEN INPUT MTZ-FILE')
      ENDIF

C READ TITLE AND HISTORY CARDS
      CALL LRTITL(1,MTZTIT(1),IDUM)
      idum = 19
c Need to set idum to max number of history lines on entry to LRHIST      
      CALL LRHIST(1,MTZTIT(2),IDUM)

      IF(MTZTIT(21)(1:10).NE.'          ')THEN
        WRITE(*,*)' !!! WARNING !!!'
        WRITE(*,*)IDUM,' HISTORY CARDS ON INPUT MTZ'
        WRITE(*,*)' ONLY THE FIRST 19 CARDS ARE STORED'
      ENDIF

C STORE TITLE PLUS FIRST 19 HISTORY CARDS IN THE MDF-TITLE ARRAY
C HISTORY CARDS 20 TO 30 (IF PRESENT) ARE LOST
      IF(IPRINT.EQ.1)THEN
        WRITE(*,'(/,A,/)')' TITLE CARDS ON INPUT MTZ:'
        DO 100,L=1,20
          TITLE(L)=MTZTIT(L)
          IF(TITLE(L)(1:5).NE.'     ') WRITE(*,*)TITLE(L)
  100  CONTINUE
      ENDIF

      CALL LRSORT(1,SORTX)

C READ THE SORT ORDER AND CHECK IF THE RECORDS ARE SORTED ON THE FIRST
C 3 COLUMNS, WHICH WE ASSUME TO BE H, K AND L
      DO 200,L=1,3
        IF(SORTX(L).EQ.0 .OR. SORTX(L).GT.3)THEN
          WRITE(*,*)' !!! WARNING !!!'
          WRITE(*,*)' POTENTIAL PROBLEM IN SORT ORDER OF INDICES'
          WRITE(*,*)' SORTING ORDER OF INDICES UNKNOWN'
          WRITE(*,*)' RUN FILE THROUGH SORTMTZ'
          STOP
        ENDIF
  200 CONTINUE

C STORE SORT ORDER IN MDF-VARIABLES ISLOW,INTER,IFAST. IT WILL BE CHECKED
C LATER (AFTER THE CALL TO LRCLAB) IF THE FIRS 3 COLUMNS INDEED CONTAIN
C REFLECTION INDICES
      ISLOW=SORTX(1)
      INTER=SORTX(2)
      IFAST=SORTX(3)

      IF(IPRINT.EQ.1)THEN
        WRITE(*,210)HKL(ISLOW),HKL(INTER),HKL(IFAST)
  210   FORMAT(//,4X,'SLOW, MEDIUM AND FAST CHANGING INDICES', 9X,
     $    A1,', ',A1,', ',A1,//)
      ENDIF

C GET THE CELL PARAMETERS
      CALL LRCELL(1,CELL)

C GET SOME GENERAL INFORMATION. SET MAXR TO 1 (1 COLUMN, WHICH WILL
C CONTAIN 1/D**2) AND MAXB TO NCOL-3, SINCE NCOL INCLUDES THE INDICES
C H, K AND L WHICH ARE STORED SEPARATELY IN THE MDF-FORMAT.
C CHECK THE VERSION TO SEE IF IT IS COMPATIBLE WITH THIS ROUTINE.
      CALL LRINFO(1,VERSION,NCOL,NREFLX,RANGES)

C CHECK THE MTZ VERSION. SET IVERSN TO 2. THIS TELLS THE CALLING PROGRAM
C THAT A MTZ FILES HAS BEEN READ.
      IVERSN = 2
      IF(IPRINT.EQ.1) WRITE(*,*)' MTZ VERSION : ',VERSION
      IF(VERSION .NE. 'MTZ:V1.1')THEN
        WRITE(*,*)
        WRITE(*,*)' !!! WARNING, MTZ-VERSION UPDATE !!!'
        WRITE(*,*)
        WRITE(*,*)' THE BIOMOL PROGRAMS HAVE BEEN TESTED FOR VERSION'
        WRITE(*,*)' MTZ:V1.1'
        WRITE(*,*)
        WRITE(*,*)' CONFIRM THAT THE NEW VERSION IS STILL COMPATIBLE'
        WRITE(*,*)' WITH THE CODE IN SUBROUTINES RDMTZ, WRMTZ,'
        WRITE(*,*)' RDREC AND WRTREC AND IF SO UPDATE ROUTINE RDMTZ'
        WRITE(*,*)' SUCH THAT IT NO LONGER POSTS THIS WARNING'
        WRITE(*,*)
      ENDIF

C CREATE RBUFFER COLUMN THAT WILL CONTAIN THE RESOLUTION
      MAXR=1
      RBFINF(1)='1/D**2'
      IF(IPRINT.EQ.1)THEN
        WRITE(*,*)' ONE RBUFFER COLUMN CREATED CONTAINING : ',RBFINF(1)
      ENDIF

C SET MAXB TO NCOL-3 SINCE IN THE MDF THE INDICES ARE STORED SEPARATELY
C CHECK IF MAXB IS WITHIN PROGRAM LIMITS
      MAXB=NCOL-3
      IF(MAXB.GT.MAXBUF)THEN
        WRITE(*,211) MAXB,MAXBUF
  211 FORMAT(/,'    MAXB LARGER THAN ',
     $  'MAXBUF IN SUBROUTINE RDIMDF :  STOP!',//,'    MAXB = ',
     $  I5,', BUT MAXBUF = ',I5,//)
        WRITE (*,212)
  212 FORMAT(/,'    CHANGE MAXBUF IN ROUTINE RDIMDF ',
     $  'STATEMENT IN CALLING ROUTINE!')
        STOP
      ENDIF

C---- If RESO keyword wasn't specified, read from file
      IF (.NOT.LOGRES) THEN
         CALL LRRSOL(1,SMINWL,SMAXWL)
         IF (SMINWL.EQ.0.0.OR.SMAXWL.EQ.0.0)
     $      CALL CCPERR(1,' No resolution range in MTZ file.')
         RESMIN=1./SQRT(SMAXWL)
         RESMAX=1./SQRT(SMINWL)
         WRITE (*,220) RESMIN, RESMAX
  220    FORMAT(/,' RESOLUTION LIMITS FOR SELECTION OF REFLECTIONS:'
     .        ,F7.2,' TO',F7.2,' ANGSTROM')
         LOGRES=.TRUE.
      ENDIF

C---- Setup the program label to file label assignments.
      CALL LRASSN(1,LSPRGI,NLPRGI,LOOKUP,CTPRGI)

C---- Set column numbers for use in MAIN
      NFOBS = LOOKUP(4) - 3
      NFCALC = LOOKUP(5) - 3
      NPHASE = LOOKUP(6) - 3

C GET COLUMN LABELS AND TYPES in file order
      CALL LRCLAB(1,CLABS,CTYPIN,NCOL)

C COPY CTYPIN TO CTYPOUT
      DO 250,L=1,NCOL
        CTYPOUT(L)=CTYPIN(L)
  250 CONTINUE

C CHECK THAT THE INDICES INDEED OCCUPY THE FIRST THREE COLUMNS
C UNFORTUNATELY IT CAN NOT BE CHECKED IF THE INDICES ARE ALSO STORED IN
C THE RIGHT ORDER, E.G. H,K,L. JUST HOPE PEOPLE HAVE SOME LOGIC.
      IF(CTYPIN(1).NE.'H'.OR.CTYPIN(2).NE.'H'.OR.CTYPIN(3).NE.'H')THEN
        WRITE(*,*)' H, K & L ARE NOT STORED IN COLUMNS 1, 2 AND 3'
        WRITE(*,*)' OR COLUMN TYPES ARE ABSENT/INCORRECT IN YOUR MTZ'
        STOP
      ENDIF

C GET THE SYMMETRY INFORMATION AND EXTRACT FROM THIS THE VALUES FOR THE
C MDF VARIABLES; ICENTR, ISCREW, KLASS, SYMROT AND SYMTRL.
      CALL LRSYMI(1,NPRIM,CENTR,NSPGRP,SPGNAM,PGNAM)

      IF(CENTR .EQ. 'P')THEN
        ICENTR = 0
      ELSE IF(CENTR .EQ. 'A')THEN
        ICENTR = 1
      ELSE IF(CENTR .EQ. 'B')THEN
        ICENTR = 2
      ELSE IF(CENTR .EQ. 'C')THEN
        ICENTR = 3
      ELSE IF(CENTR .EQ. 'F')THEN
        ICENTR = 4
      ELSE IF(CENTR .EQ. 'I')THEN
        ICENTR = 5
      ELSE IF(CENTR .EQ. 'R')THEN
        ICENTR = 6
      ELSE
        WRITE(*,*)' !!! WARNING !!!'
        WRITE(*,*)' UNKNOWN CENTERING TYPE GIVEN'
        WRITE(*,*)' ONLY P, A, B, C, F, I & R ARE ALLOWED'
        STOP
      ENDIF

C THE SPACE GROUP NUMBER IS REQUIRED TO FIND OUT THE CRYSTAL CLASS
C IF NOT PRESENT IN THE MTZ, THEN ASK THE USER TO PROVIDE THE NUMBER.
      IF(NSPGRP .EQ. 0)THEN
        WRITE(*,*)' GIVE INTERNATIONAL TABLES SPACEGROUP NUMBER'
        READ(*,*)NSPGRP
      ENDIF

C GET THE SYMMETRY OPERATIONS AND FIND SCREW AXES
      CALL LRSYMM(1,NSYM,SYMOP)

      ISCREW(1)=0
      ISCREW(2)=0
      ISCREW(3)=0
      DO 600,L=1,NPRIM
        DO 500,M=1,3
          IF(SYMOP(M,4,L).GT.0.)THEN
            ISCRW=NINT(1./SYMOP(M,4,L))
            ISCREW(M)=MAX(ISCREW(M),ISCRW)
          ENDIF
  500   CONTINUE
  600 CONTINUE

C IN CCP4 NON-STANDARD SPACEGROUPS ARE GIVEN A NORMAL SPACEGROUP NUMBER
C PLUS A MULTIPLE OF 1000.
      ISPGRP=MOD(NSPGRP,1000)

      IF(ISPGRP.EQ.1)THEN
        KLASS=1
      ELSE IF(3.LE.ISPGRP .AND. ISPGRP.LE.5 ) THEN
        KLASS = 0
        DO 310 N=1,NSYM
        IF (SYMOP(1,1,N).EQ.-1 .AND. SYMOP(2,2,N).EQ.1 
     .  .AND. SYMOP(3,3,N).EQ.-1) KLASS = 2
        IF (SYMOP(1,1,N).EQ. 1 .AND. SYMOP(2,2,N).EQ.-1 
     .  .AND. SYMOP(3,3,N).EQ.-1) KLASS = 15
        IF (SYMOP(1,1,N).EQ.-1 .AND. SYMOP(2,2,N).EQ.-1 
     .  .AND. SYMOP(3,3,N).EQ. 1) KLASS = 16
  310   CONTINUE
        WRITE (*,'('' ISPGRP = '',I5,''; KLASS = '',I5)') ISPGRP,KLASS
        IF (KLASS.EQ.0) CALL CCPERR (1,'WRONG KLASS')
      ELSE IF(16.LE.ISPGRP .AND. ISPGRP.LE.24)THEN
        KLASS=3
      ELSE IF(75.LE.ISPGRP .AND. ISPGRP.LE.80)THEN
        KLASS=4
      ELSE IF(89.LE.ISPGRP .AND. ISPGRP.LE.98)THEN
        KLASS=5
      ELSE IF(143.LE.ISPGRP .AND. ISPGRP.LE.145)THEN
        KLASS=6
      ELSE IF(146.EQ.ISPGRP .AND. CENTR.EQ.'P')THEN
        KLASS=6
      ELSE IF(146.EQ.ISPGRP .AND. CENTR.EQ.'R')THEN
        KLASS=7
      ELSE IF(ISPGRP.EQ.149 .OR. ISPGRP.EQ.151 .OR. ISPGRP.EQ.153)THEN
        KLASS=8
      ELSE IF(ISPGRP.EQ.150 .OR. ISPGRP.EQ.152 .OR. ISPGRP.EQ.154)THEN
        KLASS=9
      ELSE IF(155.EQ.ISPGRP .AND. CENTR.EQ.'R')THEN
        KLASS=9
      ELSE IF(155.EQ.ISPGRP .AND. CENTR.EQ.'P')THEN
        KLASS=10
      ELSE IF(168.LE.ISPGRP .AND. ISPGRP.LE.173)THEN
        KLASS=11
      ELSE IF(177.LE.ISPGRP .AND. ISPGRP.LE.182)THEN
        KLASS=12
      ELSE IF(195.LE.ISPGRP .AND. ISPGRP.LE.199)THEN
        KLASS=13
      ELSE IF(207.LE.ISPGRP .AND. ISPGRP.LE.214)THEN
        KLASS=14
      ELSE
        WRITE(*,*)' UNKNOWN SPACEGROUP NUMBER'
        WRITE(*,*)' ONLY THE ENANTIOMERIC SPACEGROUPS CAN BE USED'
        STOP
      ENDIF

      DO 650,L=1,3
        IF(ISCREW(L).EQ.1 .OR. ISCREW(L).EQ.5 .OR. ISCREW(L).GE.7)THEN
          WRITE(*,*)' PROBLEM IN YOUR SYMMETRY OPERATIONS'
          WRITE(*,*)ISCREW(L),'-FOLD SCREW AXIS OBSERVED'
          STOP
        ENDIF
  650 CONTINUE

      IF(IPRINT.EQ.1)THEN
        WRITE (*,610) CCLASS(KLASS), CENTR, ISCREW
  610   FORMAT(/,'    CRYSTAL CLASS           ',A10,';   CRYSTAL',
     $    ' LATTICE         ',A1,/,'    THERE IS A ',I1,'-FOLD SCR',
     $    'EW AXIS PARALLEL TO THE A-AXIS',/,'    THERE IS A ',I1,'-',
     $    'FOLD SCREW AXIS PARALLEL TO THE B-AXIS',/,'    THERE IS A',
     $    I2,'-FOLD SCREW AXIS PARALLEL TO THE C-AXIS')
      ENDIF

C CONVERT MTZ 4x4 MATRIX TO A 3x3 MDF MATRIX FOR ROTATIONAL SYMMETRY
C AND A 1x3 COLUMN WITH THE TRANSLATIONAL PART
      DO 900,L=1,NSYM
        DO 800,M=1,3
          SYMTRL(M,L)=SYMOP(M,4,L)
          DO 700,N=1,3
            SYMROT(N,M,L)=SYMOP(N,M,L)
  700     CONTINUE
  800   CONTINUE
  900 CONTINUE

      IF (IPRINT .EQ. 1) THEN
         WRITE (*,920)
  920 FORMAT(/,'    SYMMETRY OPERATIONS IN "INTERNATIONAL ',
     $   'TABLES" FORMAT    : ')
         CALL LSTSYM(NSYM,SYMOP)
      ENDIF

      RETURN

      END
      SUBROUTINE RDREC (MDFIN, RBUFFR, MAXR, IHKL, BUFRIN, MAXIB, 
     $                  MDFEND, IVERSN)
C///
C=======================================================================
C
C              S U B R O U T I N E      R D R E C
C
C=======================================================================
C
C      SUBROUTINE TO READ A REFLECTION RECORD FROM A STRUCTURE FACTOR
C      FILE IN EITHER MDF OR MTZ FORMAT.
C
C=======================================================================
C
C      MEANING OF THE PARAMETERS:
C
C      INPUT PARAMETERS FOR THIS SUBROUTINE:
C
C      - MDFIN          UNIT NUMBER ON WHICH THE INPUT MDF RESIDES
C
C      - MAXR           MAXIMUM LENGTH OF RBUFFR ARRAY
C
C      - MAXIB          MAXIMUM LENGTH OF BUFRIN ARRAY
C
C      - IVERSN         MDF VERSION NUMBER
C                       IVERSN = 1 : APRIL 15, 1987 VERSION
C                       IVERSN = 2 : MTZ FORMAT   
C
C        MAXR, MAXIB AND IVERSN SHOULD HAVE BEEN READ FROM THE
C        HEADER RECORDS (E.G. BY SUBROUTINE RDIMDF).  THIS IS FINE
C        FOR A PROGRAM THAT JUST READS.   FOR PROGRAMS THAT ALSO
C        PRODUCE AN OUTPUT FILE, YOU COULD SET MAXIB TO THE OVERALL 
C        MAXIMUM FOR BOTH THE INPUT AND OUTPUT FILES.  THEN BUFRIN
C        WILL  BE  FILLED  TO ITS MAXIMUM LENGTH WITH ABSENTS, AND
C        YOU  WON'T  HAVE  TO  WORRY ABOUT  ACCIDENTALLY OCCURRING
C        UNDEFINED VALUES IN IT LATER.
C
C
C      OUTPUT PARAMETERS FROM THIS SUBROUTINE:
C
C      - MDFEND         FLAG FOR END OF FILE ON MDF
C                       = 0 : END OF FILE NOT YET REACHED
C                       = 1 : END OF FILE REACHED
C
C      - IHKL           ARRAY IHKL(3) CONTAINS IH, IK, IL
C
C      - RBUFFR         ARRAY RBUFFR(MAXR) CONTAINS THE DATA 
C                       PRESENT IN THE RBUFFR
C
C      - BUFRIN         ARRAY BUFRIN(MAXIB) CONTAINS THE DATA 
C                       PRESENT IN THE BUFFER
C
C=======================================================================
C     VERSION MARCH 19, 1987  (BAUKE DIJKSTRA, RANDY READ)
C     ADAPTED TO READ ALSO THE MTZ FORMAT, DECEMBER 14, 1992   (BH6)
C     Change CCP4 'absent' definition automatically (24.11.94,HASSO)
C=======================================================================
C\\\

      IMPLICIT NONE

C SUBROUTINE ARGUMENT DECLARATIONS
      INTEGER MDFIN, MAXR, IHKL(3), MAXIB, MDFEND, IVERSN
      REAL RBUFFR(MAXR), BUFRIN(MAXIB)

C DECLARATION OF MTZ-SPECIFIC COMMON BLOCK INFO
      INTEGER MCOLS
      PARAMETER (MCOLS=500)
      INTEGER NPRIM
      CHARACTER CTYPIN(MCOLS)*1,CTYPOUT(MCOLS)*1,SPGNAM*10,PGNAM*10
      CHARACTER CENTR*1
      COMMON /MTZINF/NPRIM,CTYPIN,CTYPOUT,SPGNAM,PGNAM,CENTR

      SAVE /MTZINF/

C LOCAL DECLARATIONS
      INTEGER I,J,IPOS,IB
      REAL ADATA(MCOLS),ABSENT
      LOGICAL EOF,BDATA(MCOLS)

C flag to signal that warning message has already been written
      INTEGER NEGSIG
      SAVE NEGSIG
      DATA NEGSIG /0/

      DATA ABSENT /-1.0E+10/
C     CLEAR THE OUTPUT BUFRIN RECORD
 
      DO 10 I = 1,MAXIB
   10 BUFRIN(I) = ABSENT
      MDFEND = 0
 
C     READ A NEW RECORD
 
      IF (IVERSN.EQ.0) THEN
        READ (MDFIN,END=50,ERR=100) IPOS, IB, (RBUFFR(J),J=1,MAXR), 
     $  IHKL, (BUFRIN(J), J=1,IB)
      ELSE IF (IVERSN.EQ.1) THEN
        READ (MDFIN,END=50,ERR=100) IB, (RBUFFR(J),J=1,MAXR), 
     $  IHKL, (BUFRIN(J), J=1,IB)
      ELSE IF (IVERSN.EQ.2) THEN
C IN THE CALL TO LRREFL THE INDEX HAS BEEN SET TO 1001 INSTEAD OF 1. THIS
C INDICATES THAT BIOMOL ABSENCE FLAGS SHOULD BE RETURNED UNMODIFIED. IF
C AN INDEX OF 1 IS GIVEN, THEN THE ABSENCE FLAGS WOULD BE RESET TO ZERO
C TO COMPLY TO THE CCP4 DEFINITION
        CALL LRREFL(1001,RBUFFR(1),ADATA,EOF)
        IF(EOF) GOTO 50
C
C     CONVERT FROM NaN's TO ABSENCE FLAGS
C
C       CALL RESET_MAGIC(1001,ADATA,ADATA,MCOLS,DUMMY,ABSENT)
        CALL LRREFM(1,BDATA)
	DO 14,I=1,MAXIB
	  IF (BDATA(I+3)) ADATA(I+3)=ABSENT
   14   CONTINUE
C REORGANIZE THE DATA
        IHKL(1)=NINT(ADATA(1))
        IHKL(2)=NINT(ADATA(2))
        IHKL(3)=NINT(ADATA(3))
        DO 20,I=1,MAXIB
          IF(CTYPIN(I+3).EQ.'P')THEN
C CTYPIN = P INDICATES THAT THIS COLUMN CONTAINS PHASE INFORMATION.
C CONVERT THE PHASE FROM DEGREES (IN MTZ) TO CYCLES (IN MDF)
	    IF (ADATA(I+3).LE.ABSENT) GOTO 15
            BUFRIN(I)=ADATA(I+3)*10000.0/360.0
   15       CONTINUE
          ELSE IF(CTYPIN(I+3).EQ.'Q')THEN
C CTYPIN = Q INDICATES THAT THIS COLUMN CONTAINS SIGMA's
C CHECK THAT SIGMA'S ARE POSITIVE
            IF(ADATA(I+3).LE.0.0 .AND. ADATA(I+3).GT.-0.99E+10)THEN
              IF (NEGSIG.EQ.0) THEN 
                WRITE(*,*)
                WRITE(*,*)' !!! WARNING : NON-POSITIVE SIGMA FOUND !!!'
                WRITE(*,*)
                WRITE(*,*)' SIGMA VALUES HAVE TO BE LARGER THAN ZERO TO'
                WRITE(*,*)' MAKE SENSE. YOUR FILE MAY HAVE BEEN WRITTEN'
                WRITE(*,*)' BY A CCP4 PROGRAM WHICH SETS ABSENT DATA TO'
                WRITE(*,*)' ZERO. USE OPTION FULLMDF IN PROGRAM SFTOOLS'
                WRITE(*,*)' TO UPDATE YOUR STRUCTURE FACTOR FILE'
                WRITE(*,*)
                WRITE(*,*)' THE VIOLATION WAS IN COLUMN ',I,' OF:'
                WRITE(*,*)IHKL,(ADATA(J),J=4,MAXIB+3)
                WRITE(*,*)' COLUMN TYPES ARE:'
                WRITE(*,*)(CTYPIN(J),J=4,MAXIB+3)
                WRITE(*,*)
                WRITE(*,*)' FOR THIS INPUT ABSENT FLAGS ARE CONVERTED',
     $                    ' TO BIOMOL CONVENTION'
                WRITE(*,*)
                WRITE(*,*)' NO SUBSEQUENT WARNINGS OF THIS FACT ARE',
     $                    ' REPORTED !'
                WRITE(*,*)
                NEGSIG=1
              ENDIF
              IF(ADATA(I+3).LE.0.0 .AND. ADATA(I+3).GT.-1.0E-04)THEN 
                ADATA(I+3) = ABSENT
              ENDIF
            ENDIF
            BUFRIN(I)=ADATA(I+3)
          ELSE
            BUFRIN(I)=ADATA(I+3)
          ENDIF
   20   CONTINUE
      ELSE
        WRITE(*,*)' UNKNOWN STRUCTURE FACTOR FILE FORMAT'
        STOP
      ENDIF

      RETURN

   50 MDFEND = 1
   55 CONTINUE
      DO 60 I = 1,MAXR
   60 RBUFFR(I) = ABSENT

      RETURN

  100 CONTINUE
      WRITE(*,*)' ERROR READING STRUCTURE FACTOR FILE'

      STOP

      END

      SUBROUTINE UNPCK (IH, IK, IL, IPK)

C     UNPACK THE INTEGER WORD IPK INTO IH, IK AND IL

      ITMP = IPK
      IL = MOD(ITMP,1024) - 512
      ITMP = ITMP/1024
      IK = MOD(ITMP,1024) - 512
      ITMP = ITMP/1024
      IH = MOD(ITMP,1024) - 512
      END

