C
C     PEAKMAX
C     Copyright (C) 1992 Phil Evans
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
C**** MAP SEARCH PROGRAM.
C
C**** THIS VERSION USES A CRUDE INTERPOLATION PROCEDURE FOLLOWING BOOTH
C**** THIS WILL NOT BE SATISFACTORY FOR POORLY RESOLVED PEAKS.
C
C
C
C****  AUTHOR              ALLEN LARSON             CAMBRIDGE,1968
C****  MODIFICATIONS       SAM MOTHERWELL           CAMBRIDGE,1969
C
C
C  8th October 1993 Phil Evans MRC LMB Cambridge
C    Re-keyworded !
C    The program will still accept the old-style number input, but will
C    complain
C
C*****************changed last option to be an option (INEG, card 1) - rap88.
C
C JULY 1989 fixed bug in orthogonalization procedure (T.S.,Basel)
C 23-Jul-1993 Keyworded by I J Clifton
C
C**** ADAPTED FOR VAX BY MIKE LAWRENCE DECEMBER 1981.
C****  and by Phil Evans
C****  modified to look for negative peaks as well - T.Skarzynski (JAN.85)
C
C 30-Sep-92
C sort on abs.val.(peak height); sorted o/p includes peak/sigma
C optionally o/p peak hght, peak/sigma in occ, Btemp cols (see NOUT)
C
C  7-Oct-86
C  Change to use F77 map handling routines (+rms), add routine MATVEC,
C  adopt Brookhaven orthogonal convention (a, c*xa, c*) and output
C  orthogonal coords to file.
C  Changed ordering of sections for searching for a peak on last
C  section from (SEC1=last-1, SEC2=last, SEC3=last) to
C  (SEC1=last-1, SEC2=last, SEC3=last-1).
C
C  22-Oct-86
C  Allow for different orthogonalisation conventions.  Extra param on
C  input cards.
C
C 31-Oct-1986 various output options, see NOUT on card 1
C
C 01-Feb-1999 renamed MATVEC to MATVEC2, to avoid clash with routine
C             MATVEC in modlib.f
C
C
C     ==============================================
C
C Keywords:
C
C  THRESHOLD [RMS] <threshold> [NEGATIVES]
C  	[default THRESHOLD RMS 3]
C  
C  	Threshold for peaks: all peaks with  values above <threshold>
C  	will be selected. 
C  	Subkeywords:
C  	RMS  peaks will be chosen if they are more than
C  	   <threshold> * (rms density), instead of an absolute value.
C  	NEGATIVES negative "peaks" below -<threshold> will be selected
C  	   as well as positive peaks
C  
C  OUTPUT [NONE | BROOKHAVEN | PEAKS | FRAC]
C  	[default OUTPUT BROOKHAVEN]
C  
C  	By default, output Brookhaven files contain in the occupancy
C  	column, the peak height, and in the Bfactor column,
C  	height/rms(density). If the BFACTOR command is present (qv),
C  	these columns will be set to occupancy & Bfactor
C  
C  	Subkeywords:
C  	BROOKHAVEN (default)  write sorted peaks to a Brookhaven
C  		format coordinate file (XYZOUT)
C  	NONE   no output file
C  	PEAKS   write peaks file to logical name TO
C       FRAC    write peaks (fractional coords in format suitable for
C               MLPHARE) to XYZFRC
C  
C  BFACTOR <Bfactor>  [<Occupancy>]
C  	[default: set occupancy = peak height/rms(density),
C                        Bfactor = 20.00]
C  
C  	Define Bfactor [& occupancy, default = 1.0] for output file
C  
C  RESIDUE <Residue_type>
C       [default "HOH"]
C
C       Set residue type for output file
C
C  ATNAME <Atom_name>
C       [default "O"]
C
C       Set atom name for output file
C
C  CHAIN <Chain_ID>
C       [default "X"]
C
C       Set chain identifier for output file
C  
C  NUMPEAKS  <nop>
C  	(<= 8000) (Default: 800)  
C  
C  	Number of peaks to output in the list, sorted by height
C  
C  ORTHOGONALIZATION  <Ncode>
C  	[Default: 1  (Standard Brookhaven)]
C  
C           Set orthogonalisation convention
C                =1   A // XO, C* // ZO  (Standard Brookhaven)
C                =2   B // XO, A* // ZO
C                =3   C // XO, B* // ZO
C                =4  HEX A+B // XO, C* // ZO
C                =5   A* // XO, C // ZO      (Rollett)
C  
C  XYZLIM  NU1 NU2 NV1 NV2 NW1 NW2 
C  	[Default: If this keyword is omitted, the whole map is searched]
C  
C  	Limits for search in grid units on fast, medium, slow axes 
C  
C  EXCLUDE [ EDGE | NONE ]
C       [Default: EXCLUDE EDGE]
C
C       Suppress output of peaks at the very edge of the
C       map (EXCLUDE EDGE), otherwise output all peaks (EXLUDE NONE)
C
      PROGRAM PEAKMAXPROG
C
C Main program sets array dimensions
C
C
C     .. Parameters ..
      INTEGER ISIZE
      PARAMETER (ISIZE=500000)
      INTEGER LINESIZE
      PARAMETER (LINESIZE=1000)
C     ..
C     .. Scalars in Common ..
      REAL RHMAX,RHMEAN,RHMIN,RHRMS
      INTEGER LMODE,LSPGRP,NSEC,NU1,NU2,NV1,NV2,NW1
C     ..
C     .. Arrays in Common ..
      REAL CELL(6)
      INTEGER IUVW(3),MXYZ(3)
C     ..
C     .. Local Scalars ..
      INTEGER JS,JSIZE,NC,NR
      CHARACTER TITLE*80,ERRMSG*80
C     ..
C     .. Local Arrays ..
      REAL SEC1(ISIZE),SEC2(ISIZE),SEC3(ISIZE),MAPLINE(LINESIZE)
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPERR,CCPFYP,MRDHDR,PKMAX,CCPRCS,XYZINIT
C     ..
C     .. Common blocks ..
      COMMON /MAP/NSEC,IUVW,MXYZ,NW1,NU1,NU2,NV1,NV2,CELL,LSPGRP,LMODE,
     +       RHMIN,RHMAX,RHMEAN,RHRMS
C     ..
      CALL CCPFYP
C
      CALL CCPRCS(6,'PEAKMAX','$Date: 2006/11/24 11:36:41 $')
      CALL XYZINIT
C
C  Open map file and test for sufficient space
C
      CALL MRDHDR(3,'MAPIN',TITLE,NSEC,IUVW,MXYZ,NW1,NU1,NU2,NV1,NV2,
     +            CELL,LSPGRP,LMODE,RHMIN,RHMAX,RHMEAN,RHRMS)
      JSIZE = ISIZE
      NC = NU2 - NU1 + 3
      NR = NV2 - NV1 + 3
      JS = NC*NR
      IF (JS.GT.JSIZE) THEN
        WRITE (ERRMSG,FMT=6002) JSIZE,JS
        CALL CCPERR(1, ERRMSG)
      END IF
      IF (NC .GT. LINESIZE) THEN
        WRITE (ERRMSG,FMT=6004) LINESIZE, NC
        CALL CCPERR(1, ERRMSG)
      ENDIF
   10 CALL PKMAX(SEC1,SEC2,SEC3,MAPLINE,NR,NC,TITLE)
      CALL CCPERR(0,' Normal Termination ')
 6002 FORMAT (' ***** Array size too small, increase ISIZE from ',I8,
     +       ' to',I8)
 6004 FORMAT (' ***** Array size too small, increase LINESIZE from',
     +    I8,' to',I8)
      END
C
C
      SUBROUTINE PKMAX(SEC1,SEC2,SEC3,MAPLINE,MR,MC,TITLE)
C     ====================================================
C
C      implicit none
C
C Symmetry
C     .. Scalar Arguments ..
      INTEGER MC,MR
      CHARACTER TITLE*80
C     ..
C     .. Array Arguments ..
      REAL SEC1(MR,MC),SEC2(MR,MC),SEC3(MR,MC),MAPLINE(MC)
C     ..
C     .. Scalars in Common ..
      REAL RHMAX,RHMEAN,RHMIN,RHRMS
      INTEGER LMODE,LSPGRP,NSEC,NU1,NU2,NV1,NV2,NW1
C     ..
C     .. Arrays in Common ..
      REAL CELL(6)
      INTEGER IUVW(3),MXYZ(3)
C     ..
C     .. Local Scalars ..
      REAL A,ALPH,B,BET,BISO,C,CM,GAMM,HEIGHT,HSEC1,HSEC2,HSEC3,Q,
     +     QOCC,R1,R2,RMN,RMNI,RPK,XX,YY,ZZ,SIGV,W,VEC_LEN
      INTEGER I,IBUFF1,IBUFF2,INEG,IORTH,IR,IRES,ISER,
     +        ISYM,IXYZOUT,IZ,J,K,KFAIL,KL,M,M1,M2,M3,MSG,N,NC,NOP,
     +        NOUT,NPK,NPKS,NR,NSYM,NSYMP,NTOK,NW2,NX1,NX2,NXC,NY1,NY2,
     +        NYc,NZEROPK,NZ1,NZ2,NZC,SX1,SX2,SY1,SY2,LFLAG,IST
      LOGICAL LEND,LIMS,LRMS, LBFAC, LFIRST, LFRAC, LPEAK, XMLOUT
      CHARACTER IDCH*1,ID*4,ATNAM*4,KEY*4,RESTYP*4,LINE*400,RESNO*4,
     +          ALTCOD*1,SEGID*4,INSCOD*1,ATMTYP*4
C     ..
C     .. Local Arrays ..
      INTEGER MAXPK
      PARAMETER (MAXPK=8000)
      REAL BS(6),DIS(3),FVALUE(20),FR(4,4),OR(4,4),PKHT(MAXPK),R(3,2),
     +     RS(4,4,192),X(3),XCORR(3),XO(MAXPK),XYZ0(4),XYZ1(4),XYZ2(4),
     +     YO(MAXPK),ZO(MAXPK),APKHT(MAXPK),RIO(MAXPK),XYZE(2,3)
      INTEGER IBEG(20),IEND(20),IFMS(3),IO(MAXPK),ITYP(20),NGRID(3),
     +        NSITE(MAXPK),IDEC(20)
      CHARACTER CVALUE(20)*4,ORTH(5)*40,NAMSPG_CIF*20,NAMSPG_CIFS*10,
     +                   NAMPG*10

      LOGICAL PATTERSON, LNOEDGE
      EQUIVALENCE (IO,RIO)
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPDPN,CCPERR,MPOSN,MSYMOP,PARSER,RBFRAC2,
     +         RBRORF,RLINE,SORT2,TRANS,XYZADVANCE,XYZATOM,
     +         XYZCLOSE,XYZCOORD,XYZOPEN
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,AMOD,ATAN,COS,NINT,SIN,SQRT,REAL
C
      INTEGER CCPNUN
      EXTERNAL CCPNUN
C     ..
C     .. Common blocks ..
      COMMON /MAP/NSEC,IUVW,MXYZ,NW1,NU1,NU2,NV1,NV2,CELL,LSPGRP,LMODE,
     +       RHMIN,RHMAX,RHMEAN,RHRMS
C     ..
C     .. Data statements ..
C
      DATA ORTH/'A // XO, C* // ZO (Standard Brookhaven)',
     +     'B // XO, A* // ZO','C // XO, B* // ZO',
     +     'HEX A+B // XO, C* // ZO','A* // XO, C // ZO (Rollett)'/
C     ..
C
      ALTCOD = ' '
      INSCOD = ' '
      SEGID = ' '
      RESNO = ' '
      IZ = 0
      BISO = 20.0
      PATTERSON = .FALSE.
      SIGV=1./RHRMS
      NC = MC - 2
      NR = MR - 2
      NW2 = NW1 + NSEC - 1
      NPK = 1
C Read symmetry operations from map file
C
      CALL MSYMOP(3,NSYM,RS)
C
C IUVW(N) CONTAINS THE COORDINATES  FAST(N=1) MEDIUM(N=2) SLOW(N=3)
C    (X=1,Y=2,Z=3)
C IFMS(N) CONTAINS THE SPEED OF X(N=1),Y(N=2),Z(N=3)
C  (FAST=1,MEDIUM=2,SLOW=3)
      DO 10 N = 1,3
        IFMS(IUVW(N)) = N
   10 CONTINUE
C
C  Set up constants for orthogonal conversion
C
      ALPH = CELL(4)
      BET = CELL(5)
      GAMM = CELL(6)
      A = CELL(1)
      B = CELL(2)
      C = CELL(3)
C
      DO 30 I = 1,3
        DO 20 J = 1,3
          OR(J,I) = 0.0
   20   CONTINUE
   30 CONTINUE
C
C Initialisations
      LIMS = .FALSE.
      INEG = 0
      LFIRST = .TRUE.
      NZEROPK = 0
C Defaults
      NOP = 800
      RMNI = 3.0
      NOUT = 2
      IORTH = 1
      LRMS = .TRUE.
      QOCC = 1.0
      LBFAC = .FALSE.
      LFRAC = .FALSE.
      LPEAK = .FALSE.
      LNOEDGE = .TRUE.
      ATNAM = ' O'
      ATMTYP = ' O  '
      RESTYP = 'HOH'
      IDCH = 'X'
      BS(1) = 30.0
      DO 35 I=2,6
        BS(I) = 0.0
   35 CONTINUE
      XMLOUT = .FALSE.
C Read the keywords
   40 LINE = ' '
      NTOK = 20
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +            .TRUE.)
      IF (LEND) THEN
        GO TO 50
      ENDIF
C
      DO 41, I=1,NTOK
         CALL CCPUPC(CVALUE(I))
 41   CONTINUE
C
      IF (KEY.EQ.'END ') THEN
        GO TO 50
C
C---- PATTERSON
C
      else if (KEY.eq.'PATT') then
        PATTERSON= .TRUE.
        GOTO 40
C
      ELSE IF (KEY.EQ.'NUMP') THEN
        IF (NTOK.GE.2) NOP = FVALUE(2)

      ELSE IF (KEY.EQ.'THRE') THEN
         LRMS = .FALSE.
         N = 2
 42      IF (N .LE. NTOK) THEN
            IF (CVALUE(N) .EQ. 'RMS') THEN
C  RMS keyword, set flag for RMS threshold
               LRMS = .TRUE.
            ELSEIF (CVALUE(N) .EQ. 'NEGA') THEN
C  NEGATIVE set flag for negative peaks
               INEG = 1
            ELSE
               CALL GTTREA(N, RMNI, LFLAG, NTOK,ITYP,FVALUE)
               IF (LFLAG .NE. 0) THEN
                  CALL CCPERR(1,
     $                 ' *** Illegal number on THRESHOLD command ***')
               ENDIF
            ENDIF
            N = N+1
            GO TO 42
         ENDIF
      ELSE IF (KEY.EQ.'OUTP') THEN
         N = 2
         NOUT = 0
 43      IF (N .LE. NTOK) THEN
            IF (CVALUE(N).EQ.'BROO' .OR. CVALUE(N)(1:3).EQ.'PDB') THEN
C  BROOKHAVEN format output
               NOUT = 2
            ELSEIF (CVALUE(N) .EQ. 'NONE') THEN
C  No output
               NOUT = 0
               LPEAK = .FALSE.
               LFRAC = .FALSE.
            ELSEIF (CVALUE(N) .EQ. 'PEAK') THEN
C  Peak output 
               LPEAK = .TRUE.
            ELSEIF (CVALUE(N) .EQ. 'FRAC') THEN
C  Fractional coordinates (output suitable for mlphare)
               LFRAC = .TRUE.
            ELSE
               CALL CCPERR(1,' *** Unrecognized keyword ***')
            ENDIF
            N = N+1
            GO TO 43
         ENDIF
      ELSE IF (KEY.EQ.'BFAC') THEN
         CALL GTTREA(2, BS(1), LFLAG, NTOK,ITYP,FVALUE)
         QOCC = 1.0
         IF(NTOK.GT.2)CALL GTTREA(3, QOCC, LFLAG, NTOK,ITYP,FVALUE)
         LBFAC = .TRUE.
      ELSE IF (KEY.EQ.'ORTH') THEN
        IF (NTOK.GE.2) IORTH = FVALUE(2)
      ELSE IF (KEY.EQ.'XYZL') THEN
        IF (NTOK.LT.7) CALL CCPERR(1,
     $        ' *** Not enough limits given ***')
        NY1 = FVALUE(2)
        NY2 = FVALUE(3)
        NX1 = FVALUE(4)
        NX2 = FVALUE(5)
        NZ1 = FVALUE(6)
        NZ2 = FVALUE(7)
        LIMS = .TRUE.
      ELSE IF (KEY.EQ.'RESI') THEN
C Set residue type
         RESTYP = LINE(IBEG(2):IEND(2))
      ELSE IF (KEY.EQ.'ATNA') THEN
C Set atomname
         ATNAM = LINE(IBEG(2):IEND(2))
      ELSE IF (KEY.EQ.'CHAI') THEN
C Set chain ID
         IDCH = LINE(IBEG(2):IBEG(2))
C  undocumented keyword to output an XML file assigned by XMLFILE.
      ELSE IF (KEY.EQ.'XMLO') THEN
         XMLOUT = .TRUE.
C
C---- EXCLUDE
C
      ELSE IF (KEY.eq.'EXCL') THEN
        IF (NTOK .EQ. 2) THEN
          IF (CVALUE(2).EQ.'EDGE') THEN
C     Suppress peaks at the edge of the map
            LNOEDGE= .TRUE.
          ELSEIF (CVALUE(2) .EQ. 'NONE') THEN
            LNOEDGE= .FALSE.
C     Output all peaks
          ELSE
            CALL CCPERR(1,' *** Unrecognized keyword ***')
          ENDIF
        ELSE
          CALL CCPERR(1,' Wrong number of arguments for EXCLUDE')
        ENDIF
        GOTO 40
C
      ELSE
         IF (LFIRST .AND. ITYP(1).EQ.2) THEN
C First line with no keyword, treat like old version
            WRITE (6,'(//,A,/,A,/,A,//)')
     $           '  ***************************************',
     $           '  *  Use keywords, read the write-up !! *',
     $           '  ***************************************'
            NOP = NINT(FVALUE(1))
            RMNI = FVALUE(2)
            IF (RMNI .LT. 0.0) THEN
               LRMS = .TRUE.
               RMNI = ABS(RMNI)
            ELSE
               LRMS = .FALSE.
            ENDIF
            NOUT  = NINT(FVALUE(3))
            IF (NOUT .LT. 0) THEN
               NOUT = IABS(NOUT)
               LBFAC = .FALSE.
            ELSE
               LBFAC = .TRUE.
            ENDIF
            IORTH = NINT(FVALUE(4))
            INEG  = NINT(FVALUE(5))
         ELSE
            CALL CCPERR(1,' *** Unexpected keyword ***')
         ENDIF
      END IF
      LFIRST = .FALSE.
      GO TO 40
C
C --- End of input, setup options
C
   50 RMN = RMNI
      IF (LRMS) RMN = RMNI*RHRMS
      IF (NOP.LE.0 .OR. NOP.GT.MAXPK) NOP = MAXPK
      WRITE (6,FMT=6000) NOP,RMN
      IF (RMN .GT. RHMAX) THEN
         WRITE (6,'(/,A,F15.5,A,F15.5,/)')
     $        ' *** Peak threshold ',RMN,
     $        ' is greater than maximum density ',RHMAX
         CALL CCPERR(1,' *** Threshold too high ***')
      ENDIF
C
C
      IF (IORTH.GT.0 .OR. IORTH.LE.6) THEN
        WRITE (6,FMT=6002) IORTH,ORTH(IORTH)
        CALL RBFRAC2(A,B,C,ALPH,BET,GAMM,IORTH)
        FR(1,1) = 0.0
        CALL RBRORF(OR,FR)
      ELSE
        WRITE (6,FMT=6004) IORTH
        CALL CCPERR(1,' *** Illegal Orthogonalization code ***')
      END IF
C
      WRITE (6,FMT=6006)

C
      WRITE (6,FMT=6008) ((OR(I,J),J=1,3),I=1,3)

C
C SEARCH LIMITS...IF OMITTED ASSUME WHOLE GRID.
C INTERNALLY Y X AND Z ARE THE FAST MEDIUM SLOW AXES.
C IN THE ARRAYS SEC1,SEC2,SEC3  X IS THE FIRST INDEX,Y THE SECOND
C
C ARRAY LIMITS ARE AS FOLLOWS
C      NX1,NX2,NY1,NY2,NZ1,NZ2 : GRID UNITS DEFINING SEARCH BOX
C      SX1,SX2,SY1.SY2,SZ1,SZ2 : ARRAY LIMITS OF SEARCH BOX
C      NU1,NU2,NV1,NV2,NW1,NW2 : GRID UNITS DEFINING MAP BOX
C
      IF (.NOT.LIMS) GO TO 60
      WRITE (6,FMT=6010) NY1,NY2,NX1,NX2,NZ1,NZ2

C
C CHECK SEARCH LIMITS
      IF (NX1.GT.NX2 .OR. NX1.LT.NV1 .OR. NX2.GT.NV2) GO TO 340
      IF (NY1.GT.NY2 .OR. NY1.LT.NU1 .OR. NY2.GT.NU2) GO TO 340
      IF (NZ1.GT.NZ2 .OR. NZ1.LT.NW1 .OR. NZ2.GT.NW2) GO TO 340
      WRITE (6,FMT=6012)


      GO TO 70
C
C SEARCH ENTIRE MAP
   60 WRITE (6,FMT=6014)


      NX1 = NV1
      NX2 = NV2
      NY1 = NU1
      NY2 = NU2
      NZ1 = NW1
      NZ2 = NW2
C
C SEARCH LIMITED AREA OF MAP
   70 SX1 = NX1 + 2 - NV1
      SX2 = NX2 + 2 - NV1
      SY1 = NY1 + 2 - NU1
      SY2 = NY2 + 2 - NU1
        KFAIL = 0
C
C OUTPUT FRAC
      IF (LFRAC) THEN
        KFAIL = 0
        CALL CCPDPN(2,'XYZFRC','UNKNOWN','F',0,KFAIL)
        WRITE (2,FMT=6017) (MXYZ(I),I=1,3), (CELL(I),I=1,6)
      END IF
C
C OUTPUT PEAKS
      IF (LPEAK .OR. NOUT.EQ.1) THEN
        KFAIL = 0
        CALL CCPDPN(3,'PEAKS','NEW','F',0,KFAIL)
        WRITE (3,FMT=6016) (MXYZ(I),I=1,3), (CELL(I),I=1,6)
      END IF
C
C OUTPUT BROOKHAVEN
      IF (NOUT.GT.1) THEN
C   Output file in Brookhaven format as waters
        KFAIL = 0
C  Get sym ops again - need the spacegroup NAME now.
        NAMSPG_CIF = ' '
        IST = CCPNUN()
        CALL MSYMLB3(IST,LSPGRP,NAMSPG_CIF,NAMSPG_CIFS,
     +                   NAMPG,NSYMP,NSYM,RS)
        IXYZOUT = 0
        CALL XYZOPEN('XYZOUT','OUTPUT','PDB',IXYZOUT,KFAIL)
        MSG = 6
        TITLE = 'Peaks from map: '//TITLE(1:64)
        IF (.NOT.LBFAC) THEN
          QOCC = 1.0
        END IF
        ID = ' '
C        CALL WBCELL(IXYZOUT,CELL,IORTH)
        CALL WBCELLS(IXYZOUT,CELL,IORTH,NAMSPG_CIF)
      END IF
C
C INPUT FIRST BUFFER SECTION. IF NO PRECEEDING SECTION USE
C                             SECTION IMMEDIATELY FOLLOWING.
C                             THIS FORCES THE PEAK TO LIE ON THE
C                             BOUNDARY.
      IBUFF1 = NZ1 - 1
      IF (NZ1.EQ.NW1) IBUFF1 = NW1 + 1
      IF (NW1.EQ.NW2) IBUFF1 = NW1
      CALL MPOSN(3,IBUFF1)
      CALL RLINE(SEC1,MAPLINE,MR,MC)
C
C INPUT FIRST CURRENT PLANE
      CALL MPOSN(3,NZ1)
      CALL RLINE(SEC2,MAPLINE,MR,MC)
C
C BEGIN LOOP OVER SECTIONS..MGULP READS THE NEXT SECTION
C IF THE LAST SECTION CORRESPONDS TO THE EDGE OF THE MAP
C THEN USE IMMEDIATELY PRECEEDING SECTION TO FILL SEC3.
      WRITE (6,FMT=6018)


      NPK = 1
      IBUFF2 = NW2 - 1
      IF (NW2.EQ.NW1) IBUFF2 = NW2
      DO 210 NZC = NZ1,NZ2
        IF (NZC.NE.NZ2 .OR. NZ2.NE.NW2) GO TO 80
        IBUFF2 = IBUFF2 - 1
        CALL MPOSN(3,IBUFF2)
        CALL RLINE(SEC3,MAPLINE,MR,MC)
        GO TO 90

   80   IBUFF2 = NZC + 1
        CALL RLINE(SEC3,MAPLINE,MR,MC)
   90   CONTINUE
C
C SEARCH SECTION FOR PEAKS
        DO 200 NYC = SY1,SY2
          DO 190 NXC = SX1,SX2
C   IS POINT HEIGHT HIGH ENOUGH TO BE CONSIDERED?
C
            HEIGHT = SEC2(NXC,NYC)
            IF (INEG.EQ.1) HEIGHT = ABS(HEIGHT)
C
            IF (HEIGHT.LT.RMN) GO TO 190
            DO 130 IR = 1,3
              K = NYC + IR - 2
              DO 120 IS = 1,3
                KL = NXC + IS - 2
C   IS POINT HEIGHT HIGHER THAN NEIGHBOURS?
C
                IF (INEG.EQ.0) GO TO 100
                HSEC1 = ABS(SEC1(KL,K))
                HSEC3 = ABS(SEC3(KL,K))
                IF (IR.EQ.2 .AND. IS.EQ.2) GO TO 110
                HSEC2 = ABS(SEC2(KL,K))
                GO TO 110

  100           HSEC1 = SEC1(KL,K)
                HSEC3 = SEC3(KL,K)
                IF (IR.EQ.2 .AND. IS.EQ.2) GO TO 110
                HSEC2 = SEC2(KL,K)
  110           CONTINUE
C
                IF (HEIGHT.LT.HSEC1) GO TO 190
                IF (HEIGHT.LT.HSEC3) GO TO 190
                IF (IR.EQ.2 .AND. IS.EQ.2) GO TO 120
                IF (HEIGHT.LT.HSEC2) GO TO 190
  120         CONTINUE
  130       CONTINUE
C
C PEAK FOUND.....
C INTERPOLATION OF PEAK MAXIMUM BY BOOTH'S PARABOLA PROCEDURE
C
            M = 1
            NGRID(1) = NYC - 2 + NU1
            NGRID(2) = NXC - 2 + NV1
            NGRID(3) = NZC
            R(1,1) = SEC2(NXC,NYC-1)
            R(1,2) = SEC2(NXC,NYC+1)
            R(2,1) = SEC2(NXC-1,NYC)
            R(2,2) = SEC2(NXC+1,NYC)
            R(3,1) = SEC1(NXC,NYC)
            R(3,2) = SEC3(NXC,NYC)
            RPK = SEC2(NXC,NYC)
C
C N LOOPS OVER FAST,MEDIUM,SLOW FOR 1,2,3 RESPECTIVELY
            DO 180 N = 1,3
              IF (R(N,2).GT.R(N,1)) GO TO 140
              R2 = R(N,1) - R(N,2)
              R1 = RPK - R(N,2)
              Q = -1.0
              GO TO 150

  140         R2 = R(N,2) - R(N,1)
              R1 = RPK - R(N,1)
              Q = 1.0
  150         IF (ABS(R1).GE.0.001 .OR. ABS(R2).GE.0.001*RHRMS) GOTO 160
              WRITE (6,FMT=6020) NGRID(1),NGRID(2),NGRID(3),R1,R2


              CM = 0.0
              GO TO 170

  160         CM = (R2-4.0*R1)/ (2.0*R2-4.0*R1) - 1.0
  170         CONTINUE
C
C X(N) CONTAINS THE FRACTIONAL COORDINATE OF THE PEAK GRID POINT
              X(N) = REAL(NGRID(N))/MXYZ(IUVW(N))
C
C XCORR(N) CONTAINS THE FRACTIONAL COORDINATE OF THE FITTED PEAK POINT
              XCORR(N) = X(N) + Q*CM/MXYZ(IUVW(N))
  180       CONTINUE
C
C CALCULATE ORTHONORMAL AND FRACTIONAL COORDINATES OF FITTED PEAK
C NOTE THAT XCORR AND XX,YY,ZZ USE DIFFERING ORDERS FOR SAME VALUES.
            M1 = IFMS(1)
            M2 = IFMS(2)
            M3 = IFMS(3)
C Store fractional coordinates in XO,YO,ZO
            XO(NPK) = XCORR(M1)
            YO(NPK) = XCORR(M2)
            ZO(NPK) = XCORR(M3)
            APKHT(NPK)=ABS(RPK)
            PKHT(NPK) = RPK
            IO(NPK) = NPK
C Write PEAKS output (if set)
            IF (LPEAK .OR. NOUT.EQ.1) THEN
              XX = OR(1,1)*XO(NPK) + OR(1,2)*YO(NPK) + OR(1,3)*ZO(NPK)
              YY = OR(2,1)*XO(NPK) + OR(2,2)*YO(NPK) + OR(2,3)*ZO(NPK)
              ZZ = OR(3,1)*XO(NPK) + OR(3,2)*YO(NPK) + OR(3,3)*ZO(NPK)
              WRITE (3,FMT=6022) NPK,RPK,NGRID(M1),NGRID(M2),NGRID(M3),
     +          XO(NPK),YO(NPK),ZO(NPK),XX,YY,ZZ
            END IF
C
            NPK = NPK + 1
            IF (NPK.GT.MAXPK) GO TO 330
  190     CONTINUE
  200   CONTINUE
C
C START LOOPING TO NEXT SECTION
        CALL TRANS(SEC2,SEC1,MR,MC)
        CALL TRANS(SEC3,SEC2,MR,MC)
  210 CONTINUE
C
C SELECTION OF THE NOP HIGHEST PEAKS FOR OUTPUT
C
  220 NPKS = NPK - 1
C
C-----------------------------------------------------------------------
C
C Check for symmetry related peaks
C
C  Clear site numbers
      DO 230 I = 1,NPKS
        NSITE(I) = 0
  230 CONTINUE
      N = 0
C
C Set distance criteria for same peak, within one grid unit on each
C axis, tested in fractional coordinates
C
C-IJT changed it to 1.5 because some symmetry-related peaks on edge
C of map were not being eliminated.
C
      DO 240 I = 1,3
        DIS(I) = 1.5/MXYZ(I)
  240 CONTINUE
C
C-IJT fractional coords of map edges for symmetry test.
      XYZE(1,IUVW(1)) = REAL(NU1)/MXYZ(IUVW(1))
      XYZE(2,IUVW(1)) = REAL(NU2)/MXYZ(IUVW(1))
      XYZE(1,IUVW(2)) = REAL(NV1)/MXYZ(IUVW(2))
      XYZE(2,IUVW(2)) = REAL(NV2)/MXYZ(IUVW(2))
      XYZE(1,IUVW(3)) = REAL(NW1)/MXYZ(IUVW(3))
      XYZE(2,IUVW(3)) = REAL(NW2)/MXYZ(IUVW(3))
CC
C Loop peak 1
      DO 290 I = 1,NPKS - 1
C Is this peak already assigned a site number?
        IF (NSITE(I).GT.0) GO TO 290
C
C Fractional coordinates
        XYZ0(1) = XO(I)
        XYZ0(2) = YO(I)
        XYZ0(3) = ZO(I)
        XYZ0(4) = 1.0
C This is site N
        N = N + 1
        NSITE(I) = N
C
C Loop symmetry operations
        DO 280 ISYM = 1,NSYM
C Generate symmetry related position
          CALL MATMULGEN(4,4,1,XYZ1,RS(1,1,ISYM),XYZ0)
C Put back into unit cell
          DO 250 K = 1,3
            XYZ1(K) = AMOD(XYZ1(K),1.)
            IF (XYZ1(K).LT.0.) XYZ1(K) = XYZ1(K) + 1.
  250     CONTINUE
C
C Check against all unassigned peaks later in list
C  Loop peak 2
C
          DO 270 J = I + 1,NPKS
C Is peak already assigned?
            IF (NSITE(J).GT.0) GO TO 270
C Put into cell
            XYZ2(1) = AMOD(XO(J),1.)
            XYZ2(2) = AMOD(YO(J),1.)
            XYZ2(3) = AMOD(ZO(J),1.)
            DO 260 K = 1,3
              IF (XYZ2(K).LT.0.) XYZ2(K) = XYZ2(K) + 1.
C Is this the same site as peak 1?
C
C-IJT some symmetry-related peaks on edge of map were not being eliminated.
              IF (ABS(ABS(ABS(XYZ1(K)-XYZ2(K))-.5)-.5).GT.DIS(K))
     &        GO TO 270
  260       CONTINUE
C
C Same site (within one grid unit on all axes)
C  Store site number
            NSITE(J) = N
C-IJT set peak height of symmetry related peaks on edges of map to zero,
C so they are not output.
            IF (LNOEDGE) THEN
              K=I
              IF (APKHT(J).LE.APKHT(I)) K=J
              IF (XO(K).EQ.XYZE(1,1) .OR. XO(K).EQ.XYZE(2,1) .OR.
     &            YO(K).EQ.XYZE(1,2) .OR. YO(K).EQ.XYZE(2,2) .OR.
     &            ZO(K).EQ.XYZE(1,3) .OR. ZO(K).EQ.XYZE(2,3)) THEN
                IF(APKHT(K).GT.0.0001) NZEROPK = NZEROPK + 1
                APKHT(K) = 0.
              ENDIF
            ENDIF
C End loop peak 2
  270     CONTINUE
C End loop symmetry
  280   CONTINUE
C End loop peak 1
  290 CONTINUE
C
C-----------------------------------------------------------------------
C
C Write out peaks in the order found
C
      DO 300 J = 1,NPKS
        XX = OR(1,1)*XO(J) + OR(1,2)*YO(J) + OR(1,3)*ZO(J)
        YY = OR(2,1)*XO(J) + OR(2,2)*YO(J) + OR(2,3)*ZO(J)
        ZZ = OR(3,1)*XO(J) + OR(3,2)*YO(J) + OR(3,3)*ZO(J)
        NGRID(1) = NINT(MXYZ(1)*XO(J))
        NGRID(2) = NINT(MXYZ(2)*YO(J))
        NGRID(3) = NINT(MXYZ(3)*ZO(J))
        WRITE (6,FMT=6024) J,NSITE(J),PKHT(J),NGRID,XO(J),YO(J),ZO(J),
     +    XX,YY,ZZ


  300 CONTINUE
C
C
      W=RMN*SIGV
      WRITE (6,FMT=6026) NPKS,RMN,W
      IF (NPKS.EQ.0) CALL CCPERR(1,
     $     ' === Stopping - no peaks found ===')

      IF (NOP.GT.NPKS) NOP = NPKS
      IF(NOP .GT. (NPKS-NZEROPK)) NOP = NPKS - NZEROPK
      WRITE (6,FMT=6028) NOP,NZEROPK

C
C SORT PEAKS IN ORDER OF INCREASING HEIGHT
      CALL SORT2(NPKS,APKHT,PKHT,RIO)
C
C WRITE THE COORDINATES OF TOP NOP PEAKS
      K = NPKS
      IF (PATTERSON) THEN
C      Is there NCS translation? Test the first 6 peaks
        DO I = 1,6
         IF( PKHT(K-I)/PKHT(K) .GT.0.25) THEN
          IF( I.EQ.1) THEN
           J = IO(K)
           XX = OR(1,1)*XO(J) + OR(1,2)*YO(J) + OR(1,3)*ZO(J)
           YY = OR(2,1)*XO(J) + OR(2,2)*YO(J) + OR(2,3)*ZO(J)
           ZZ = OR(3,1)*XO(J) + OR(3,2)*YO(J) + OR(3,3)*ZO(J)
                WRITE (6,'(30x,A,/,A,F9.2,2x,3F6.3,2X,3F7.2)')
     + '   ***** NCS translation likely *****  ',
     + ' Origin peak:                                     ',
     +             PKHT(K), XO(J),YO(J),ZO(J),XX,YY,ZZ
          END IF
          J =  IO(K-I)
          XX = OR(1,1)*XO(J) + OR(1,2)*YO(J) + OR(1,3)*ZO(J)
          YY = OR(2,1)*XO(J) + OR(2,2)*YO(J) + OR(2,3)*ZO(J)
          ZZ = OR(3,1)*XO(J) + OR(3,2)*YO(J) + OR(3,3)*ZO(J)
               WRITE (6,'(A,F9.2,2x,3F6.3,2X,3F7.2,//)')
     +   ' Possible NCS Peak, fractional & Orthogonal coords',
     +            PKHT(K-I),XO(J),YO(J),ZO(J),XX,YY,ZZ
         END IF
        END DO 
         WRITE (6,FMT=6030)
 6030    FORMAT (2X,'Order No. Site Height/Rms    Grid     ',
     +       ' Fractional coordinates   Orthogonal coordinates',
     +       ' Vector_Lengths'/)
      ELSE
         WRITE (6,FMT=6130)
 6130    FORMAT (2X,'Order No. Site Height/Rms    Grid     ',
     +       ' Fractional coordinates   Orthogonal coordinates',/)
      END IF
C
C
      IRES = 1
      DO 310 I = 1,NOP
        J = IO(K)
        XX = OR(1,1)*XO(J) + OR(1,2)*YO(J) + OR(1,3)*ZO(J)
        YY = OR(2,1)*XO(J) + OR(2,2)*YO(J) + OR(2,3)*ZO(J)
        ZZ = OR(3,1)*XO(J) + OR(3,2)*YO(J) + OR(3,3)*ZO(J)
        NGRID(1) = NINT(MXYZ(1)*XO(J))
        NGRID(2) = NINT(MXYZ(2)*YO(J))
        NGRID(3) = NINT(MXYZ(3)*ZO(J))
        W=SIGV*PKHT(K)
C
        IF (PATTERSON) THEN
C
C---- get vector lengths
C
C
           vec_len = SQRT(XX*XX + YY*YY+ ZZ*ZZ)
           WRITE (6,FMT=6032) I,J,NSITE(J),W,NGRID,XO(J),YO(J),
     +                        ZO(J),XX,YY,ZZ,vec_len
 6032  FORMAT(1X,3I5,F8.2,2X,3I4,1X,3F8.4,2X,3F7.2,2X,F8.1)
C
        ELSE
           WRITE (6,FMT=6132) I,J,NSITE(J),W,NGRID,XO(J),YO(J),
     +                        ZO(J),XX,YY,ZZ
 6132  FORMAT(1X,3I5,F8.2,2X,3I4,1X,3F8.4,2X,3F7.2)
        ENDIF
C
C  Write MLPHARE file to XYZFRC
          IF (LFRAC) THEN
            IF(I.LE.9)
     +      WRITE(2,'(a,I1,A,3F8.4,F12.2,A)') 'ATOM',I,'   Ano ', 
     +      XO(J),YO(J),ZO(J),SIGV*PKHT(K), '  0.0 BFAC  20.0'
C
            IF(I.GT.9 .AND. I.LE.99)
     +      WRITE(2,'(a,I2,A,3F8.4,F12.2,A)') 'ATOM',I,'  Ano ', 
     +      XO(J),YO(J),ZO(J),SIGV*PKHT(K), '  0.0 BFAC  20.0'
C
            IF ( I.GT.99) 
     +      WRITE(2,'(a,3F8.4,F12.2,A)') 'ATOM    Ano ', 
     +      XO(J),YO(J),ZO(J),SIGV*PKHT(K), '  0.0 BFAC  20.0'
          END IF
C
C-IJT BUG: decrement of K must be after IF block!
        IF (NOUT.GT.1) THEN
C output peaks to file as waters
          IF(.NOT.LBFAC)THEN
            QOCC=MIN(MAX(W,-99.99),999.99)
            BS(1)=MIN(MAX(W,-99.99),999.99)
          ENDIF
          ISER = NSITE(J)
          IF (IRES.GT.999) THEN
C bump up the chain ID, wrap round to `A' if needed
            IRES = 1
            IF (IDCH.EQ.'Z') THEN
              IDCH = 'A'
            ELSE IF (IDCH.EQ.'W') THEN
              CALL CCPERR(1,
     $              ' === Too many peaks to have unique IDs ===')
            ELSE
              IDCH = CHAR(ICHAR(IDCH) + 1)
            END IF
          END IF
          CALL XYZATOM(IXYZOUT,ISER,ATNAM,RESTYP,IDCH,IRES,RESNO,
     $              INSCOD,ALTCOD,SEGID,IZ,ATMTYP)
          CALL XYZCOORD(IXYZOUT,'O','U',XX,YY,ZZ,QOCC,BISO,BS)
          CALL XYZADVANCE(IXYZOUT,0,0,*310,*310)
          IRES = IRES + 1
        END IF
        K = K - 1

  310 CONTINUE
C
      IF (XMLOUT) THEN
        CALL WRITE_XML_OUTPUT(NOP,NPKS,IO,NSITE,SIGV,PKHT,XO,YO,ZO)
      ENDIF
C
      WRITE (6,FMT=6034)
C
C CLOSE ANY OUTPUT FILES
      IF (LPEAK .OR. NOUT.EQ.1) CLOSE(UNIT=3,STATUS='KEEP')
      IF (LFRAC) CLOSE(UNIT=2,STATUS='KEEP')
      IF (NOUT.GT.1) CALL XYZCLOSE(IXYZOUT)
      CALL CCPERR(0,' Normal Termination of PEAKMAX')
C
C ERROR MESSAGES
C
  330 WRITE (6,FMT=6036) MAXPK


      GO TO 220

  340 WRITE (6,FMT=6038) NX1,NX2,NY1,NY2,NZ1,NZ2


      CALL CCPERR(1,' stop 9998')
 6000 FORMAT (/,' Find',I6,' highest peaks above',F9.3,/)
 6002 FORMAT (/,' Orthogonalisation code',I2,': ',A40,/)
 6004 FORMAT (' ** Illegal orthogonalisation code:',I6)
 6006 FORMAT (' Transformation matrix    Fractional to orthogonal',/)
 6008 FORMAT (1X,3F10.4)
 6010 FORMAT (/,' Requested limits on fast axis  :',2I4,/,' requested ',
     +       'limits on medium axis:',2I4,/,' requested limits on slow',
     +       ' axis  :',2I4)
 6012 FORMAT (/,' Limits are consistent with map',/)
 6014 FORMAT (/,' Entire map is to be searched',/)
 6016 FORMAT (3I4,/,6F10.4)
 6017 FORMAT ('GRID',3I4,/,'CELL',6F10.4)
 6018 FORMAT (/,' List of peaks in order of location',/,
     +       ' ==================================',//,
     +       ' Peaks related by symmetry are assigned the same site',
     +       ' number',//,' Count Site',
     +       ' Height        Grid         Fractional coordinates',
     +       '   Orthogonal coordinates',/)
 6020 FORMAT (' WARNING: delta Rho too small for peak determination at',
     $     3I4,', delrho values',2E12.4)
 6022 FORMAT (I5,F8.2,3I3,3F8.5,2X,3F8.3)
 6024 FORMAT (I4,1X,I4,F14.2,2X,3I4,1X,3F8.4,2X,3F7.2)
 6026 FORMAT (///,' There are',I6,' peaks higher than the threshold ',
     +       F12.5,' (',F10.5,' *sigma)')
 6028    FORMAT(/,
     + ' These peaks are sorted into descending order of height,',
     + ' the top',I4,' are selected for output',/,' The number of',
     + ' symmetry related peaks rejected for being too close to',
     + ' the map edge is ',i4,/,
     + ' Peaks related by symmetry are assigned the same site number',/)
 6034 FORMAT (//)
 6036 FORMAT (//,'***** ',I5,' peaks found, check value of RMN',/,
     +       '  NOP highest peaks are selected from these ')
 6038 FORMAT (/,' *****Error in indices****',6I5)
      END

      SUBROUTINE WRITE_XML_OUTPUT(NOP,NPKS,IO,NSITE,SIGV,PKHT,XO,YO,ZO)

      PARAMETER (MAXPK=8000)

      INTEGER NOP,NPKS,IO(MAXPK),NSITE(MAXPK)
      REAL SIGV,PKHT(MAXPK),XO(MAXPK),YO(MAXPK),ZO(MAXPK)

      CHARACTER LINE*80
      INTEGER IFAIL,XMLFILEUNIT

      XMLFILEUNIT = 0

      CALL XMLOpen(XMLFILEUNIT, 'peakmax_run', ifail)
C     keywords should be in file
      call XMLWriteElement(XMLFILEUNIT,'peakmax_keyword',' ',ifail)

      K = NPKS
      DO I = 1,NOP
        J = IO(K)
        W=SIGV*PKHT(K)

        call XMLOpenElement (XMLFILEUNIT, 'peakmax_result', ifail)
        write (LINE,'(i7)') I
        call XMLWriteAttribute (XMLFILEUNIT, 'peakmax_result',
     +           'order_number', LINE, ifail)
        write (LINE,'(i7)') NSITE(J)
        call XMLWriteAttribute (XMLFILEUNIT, 'peakmax_result',
     +           'site_number', LINE, ifail)
        write (LINE,'(f15.10)') W
        call XMLWriteAttribute (XMLFILEUNIT, 'peakmax_result',
     +           'height_over_rms', LINE, ifail)
        write (LINE,'(f15.10)') XO(J)
        call XMLWriteAttribute (XMLFILEUNIT, 'peakmax_result',
     +           'peak_frac_x', LINE, ifail)
        write (LINE,'(f15.10)') YO(J)
        call XMLWriteAttribute (XMLFILEUNIT, 'peakmax_result',
     +           'peak_frac_y', LINE, ifail)
        write (LINE,'(f15.10)') ZO(J)
        call XMLWriteAttribute (XMLFILEUNIT, 'peakmax_result',
     +           'peak_frac_z', LINE, ifail)
        call XMLCloseElement (XMLFILEUNIT, 'peakmax_result', ifail)
        K = K - 1
      enddo

      call XMLWriteMessages(XMLFILEUNIT,0,'Normal Termination',ifail)
      CALL XMLCloseElement(XMLFILEUNIT,'peakmax_run', ifail)

      END
C
      SUBROUTINE SORT2(N,A0,A1,A2)
C     =========================
C
C****
C**** N IS THE LENGTH OF THE ARRAY TO BE SORTED I.E. THE NUMBER OF
C**** ITEMS TO BE SORTED
C**** THE ELEMENTS OF ARRAY A0(N) ARE SORTED INTO INCREASING ORDER OF
C**** MAGNITUDE.  ARRAYS A1 TO A8 MAY BE CARRIED WITH A0 AS REQUIRED
C****

C     .. Scalar Arguments ..
      INTEGER N
C     ..
C     .. Array Arguments ..
      REAL A0(N),A1(N),A2(N)
C     ..
C     .. Local Scalars ..
      REAL A,C,D
      INTEGER I,IH,IP1,J,K,NO2,NP1
C     ..
      NO2 = N/2
      DO 80 I = 1,NO2
        C = A0(I)
        D = C
        IH = I
        K = I
        IP1 = I + 1
        NP1 = N + 1 - I
        DO 20 J = IP1,NP1
          A = A0(J)
          IF (A.GE.C) GO TO 10
          IH = J
          C = A
          GO TO 20

   10     IF (D.GE.A) GO TO 20
          D = A
          K = J
   20   CONTINUE
        IF (C.EQ.D) GO TO 90
        IF (IH.EQ.I) GO TO 50
   30   C = A1(IH)
        A1(IH) = A1(I)
        A1(I) = C
        C=A2(IH)
        A2(IH)=A2(I)
        A2(I)=C
   40   C = A0(IH)
        A0(IH) = A0(I)
        A0(I) = C
   50   IF (K.EQ.I) K = IH
        IF (K.EQ.NP1) GO TO 80
   60   D = A1(K)
        A1(K) = A1(NP1)
        A1(NP1) = D
        D=A2(K)
        A2(K)=A2(NP1)
        A2(NP1)=D
   70   D = A0(K)
        A0(K) = A0(NP1)
        A0(NP1) = D
   80 CONTINUE
   90 RETURN

      END
C
C
      SUBROUTINE RLINE(SEC,MAPLINE,MR,MC)
C     ===================================
C
C THIS SUBROUTINE READS IN A SECTION LINE BY LINE FROM THE MAP AND
C PLACES IT IN THE ARRAY SEC. IT LEAVES A BORDER OF ONE ARRAY UNIT
C AROUND THE MAP SECTION IN SEC AND FILLS THIS BY USING THE MAP VALUES
C IMMEDIATELY INSIDE THE OUTER VALUES OF THE MAP. IF THE MAP HAS ONLY
C ONE ROW OR COLUMN THEN THE BORDER IS FILLED BY DUPLICATION OF THE MAP
C VALUES. THIS HAS THE EFFECT OF FORCING PEAKS THAT ARE NEAR TO THE
C EDGE OF THE MAP TO LIE PRECISELY ON THE MAP BOUNDARY.
C
C
C     .. Scalar Arguments ..
      INTEGER MC,MR
C     ..
C     .. Array Arguments ..
      REAL SEC(MR,MC),MAPLINE(MC)
C     ..
C     .. Local Scalars ..
      INTEGER I,IER,J,NC,NR
C     ..
C     .. External Subroutines ..
      EXTERNAL MRDLIN
C     ..
      NR = MR - 2
      NC = MC - 2
C
      DO 20 I = 1,NR
        CALL MRDLIN(3,MAPLINE,IER)
        DO 10 J = 1,NC
          SEC(I+1,J+1) = MAPLINE(J)
   10   CONTINUE
   20 CONTINUE
      DO 30 I = 1,NR
        SEC(I+1,1) = SEC(I+1,3)
        SEC(I+1,NC+2) = SEC(I+1,NC)
   30 CONTINUE
      DO 40 J = 1,NC
        SEC(1,J+1) = SEC(3,J+1)
        SEC(NR+2,J+1) = SEC(NR,J+1)
   40 CONTINUE
      SEC (1,1) = 0.0
      SEC (MR,MC) = 0.0
      SEC (1,MC) = 0.0
      SEC (MR,1) = 0.0
      RETURN

      END
C
      SUBROUTINE TRANS(SEC1,SEC2,NR,NC)
C     =================================
C
C      DO 1 I=1,NR
C      DO 1 J=1,NC
C1     SEC2(I,J)=SEC1(I,J)
C Fast move
C     .. Scalar Arguments ..
      INTEGER NC,NR
C     ..
C     .. Array Arguments ..
      REAL SEC1(NR,NC),SEC2(NR,NC)
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPMVI
C     ..
      CALL CCPMVI(SEC2,SEC1,NR*NC)
      RETURN

      END

C
