C
C     POLARRFN
C     Copyright (C) 1986 Wolfgang Kabsch, Phil Evans
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
C***********************************************************************
C
C      POLARRFN
C
C
C  Fast polar rotation function program, by W. KABSCH
C
C  21-10-02 IJT: Increased max Bessel order to 100.
C
C  15/9/93 PRE: output weighted mean & RMS to map file
C
C   1/1/90 PRE replaced s/r DCOSFD with more robust alternative
C
C  26/6/89 PRE test maximum & minimum amplitude after applying B-factor
C          Apply temperature factor correctly ie multiply F by
C          exp ( - B sintheta/lambda)
C
C   2/6/89 PRE allow peak threshold to be given as multiple of RMS
C        Interpolate peaks
C
C  Fancy input and output by Phil Evans, Feb 1986
C
C  25/5/89 PRE edited variable names ALFA, BETA, GAMA to PHI, OMEG, KAPA
C        Changed plotting to more portable interface (see routines
C        PLT... at end)
C
C  3/3/87  PRE Correction in s/r MAPRED, allowed title change in READ
C        Added JOIN option
C
C  30/7/86 PRE increased map array size, and check
C
C VAX  compile /NOPT (Fortran v4.3 anyway)
C      link  with F77LCFLIB, MAPLIB, PLOT84, CCPLIB
C
C***********************************************************************
      PROGRAM POLARRFN_100
C     ====================
C
      COMMON /MATRIX/ ORTHOG(3,3,2),FRAC(3,3,2),SYM(4,4,192,2),NSYM(2),
     . MRSYM,JRSYM(3,3,12),LSYMFG,LHALF
      PARAMETER(MCONTR=50)
      COMMON /ALMFRF/ NCRYST,IPHI1,IPHI2,NPHI,DPHI,IOMEG1,IOMEG2,NOMEG,
     . DOMEG,IKAPA1,IKAPA2,NKAPA,DKAPA,ARAD,EPS,RESMIN,RESMAX,SMIN,SMAX,
     . NCODES(2),LSPGPS(2),TFACS(2),FRMINS(2),FRMAXS(2),LPRINT,LMAP,
     . LPLOT,CONTRS(MCONTR),NCONTR,LREAD,LSAVE,PEAK,MAXPEK,LPKRMS,RKAP1,
     . RKAP2,RMIN,RMEAN,RRMS,RWT,NPOINT,LDL
      CHARACTER*80 TITLE,FNAME
      COMMON /TEXTC/ TITLE,FNAME
C
      PARAMETER(MAXNPK=100)
      COMMON /PEAKS/ NPEAK,PEAKS(MAXNPK),PKPOS(3,MAXNPK),LPEAK,
     .  LPKORD(MAXNPK),LPKOUT
C
C--     Open main read and write files.
C
      CALL MTZINI
      CALL CCPFYP
      IFAIL=0
C
      CALL CCPDPN(5,'DATA','READONLY','F',1,IFAIL)
      CALL CCPDPN(6,'PRINTER','PRINTER','F',1,IFAIL)
      CALL CCPRCS(6,'POLARRFN', '$Date: 2007/01/15 10:17:36 $')
C
C  Read control data
C
      CALL DATAIN
C
      IF (LPEAK.GT.0) THEN
C Option to print peaks only
        CALL PPEAKS
        CALL CCPERR(0,' Normal Termination from PPEAKS')
      ENDIF
C
C  Option to read map calculated on a previous run
C
      IF (LREAD.GT.0) THEN
        CALL MAPRED
        CALL CCPERR(0,' Normal Termination from MAPRED')
      ENDIF
C
C Calculate ALMN coefficients
C
      IF (LSAVE.GE.0) CALL CALMN
C
C Sum rotation function
      CALL FRFSUM
      CALL CCPERR(0,' Normal termination from FRFSUM')
      END
C
C
C
      SUBROUTINE DATAIN
C     =================
C
C Read in all data
      INTEGER MCOLS,MaxSets
      PARAMETER(MCOLS=500,MaxSets=50)
      INTEGER NLPRGI
      PARAMETER(NLPRGI=5)
      INTEGER NPARM
      PARAMETER(NPARM=200)
      INTEGER MAXNPK
      PARAMETER(MAXNPK=100)
C
      COMMON /MATRIX/ ORTHOG(3,3,2),FRAC(3,3,2),SYM(4,4,192,2),NSYM(2),
     . MRSYM,JRSYM(3,3,12),LSYMFG,LHALF
      REAL KAPA1,KAPA2
      PARAMETER(MCONTR=50)
      COMMON /ALMFRF/ NCRYST,IPHI1,IPHI2,NPHI,DPHI,IOMEG1,IOMEG2,NOMEG,
     . DOMEG,IKAPA1,IKAPA2,NKAPA,DKAPA,ARAD,EPS,RESMIN,RESMAX,SMIN,SMAX,
     . NCODES(2),LSPGPS(2),TFACS(2),FRMINS(2),FRMAXS(2),LPRINT,LMAP,
     . LPLOT,CONTRS(MCONTR),NCONTR,LREAD,LSAVE,PEAK,MAXPEK,LPKRMS,RKAP1,
     . RKAP2,RMIN,RMEAN,RRMS,RWT,NPOINT,LDL
      CHARACTER*200 LABIN_SAVE(2)
      CHARACTER*200 ERRLIN
      CHARACTER*80 TITLE,FNAME
      CHARACTER*80 FILIN(2)
      COMMON /TEXTC/ TITLE,FNAME
C
      COMMON /PEAKS/ NPEAK,PEAKS(MAXNPK),PKPOS(3,MAXNPK),LPEAK,
     .  LPKORD(MAXNPK),LPKOUT
C
      COMMON /PKFILE/ PKFNAM
      CHARACTER*80 PKFNAM
C
      LOGICAL NOTCAL,EOF
      CHARACTER*10 NAMSGP,NAMSGS(2),PGNAME,VERSNX
C
C Maximum section size
      PARAMETER(NSMAP=181*361)
C
C JOIN stuff
      PARAMETER(MXJOIN=30)
      COMMON /VJOIN/ NJOIN,PEAKJ(3,2,MXJOIN),DASH(2,MXJOIN)
C
      CHARACTER CTPRGI(NLPRGI),LSPRGI(NLPRGI,2)*30
      LOGICAL LOGMSS(NLPRGI)
      REAL ADATA(NLPRGI),COLRNG(2,MCOLS)
      INTEGER LOOKUP(NLPRGI)
      INTEGER LSPGRP,NSM,IPRINT
      REAL RR(3,3,6),RO(4,4),RF(4,4)
C
      SAVE /PKFILE/, /VJOIN/
C
C     .. External Functions ..
      INTEGER LENSTR
      EXTERNAL LENSTR
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPUPC,GTPINT,GTPREA,
     +         PARSER,RBRINV,LKYIN,
     +         LRASSN,LRINFO,LROPEN,LRSYMM
      DIMENSION CELL(6,2),CELCHK(6)
C
      REAL DATCELL(6,MaxSets),DATWAVE(MaxSets)
      INTEGER ISETS(MaxSets),CSETID(MCOLS),SETID,SET_INDEX
      CHARACTER PNAME(MaxSets)*64,XNAME(MaxSets)*64,DNAME(MaxSets)*64
C
C Things for PARSER ----
      CHARACTER KEY*4,CVALUE(NPARM)*4,LINE*600,LTYPE*1
      INTEGER IBEG(NPARM),IEND(NPARM),ITYP(NPARM),IDEC(NPARM),NTOK
      REAL FVALUE(NPARM)
      LOGICAL LEND
C
      PARAMETER(NKEY=20)
      CHARACTER KEYS(NKEY)*4
      CHARACTER*15 CODES(5)
C
      REAL*8 X,X0
      PARAMETER(MDL=100, MDN=MDL/2)
      REAL*8 BJ(0:MDL),DJ(0:MDL)
      COMMON/POLDAT/ LBY2MX,NMAX(MDN),BZ(MDN,MDN)
C
      PARAMETER(TWOPI=2.*3.141592654D0)
C
      DATA KEYS/'TITL','SELF','CROS','RESO','CRYS','CELL','LABI',
     .   'LIMI','PRIN','NOPR','MAP ','PLOT','FIND','READ','SAVE',
     .   'SUM ','PEAK','JOIN','MAXO','END'/
C
      DATA CODES/'a,c*xa,c*','b,a*xb,a*','c,b*xc,b*',
     . 'a+b,c*x(a+b),c*','a*,cxa*,c'/
C
      DATA (LSPRGI(JJ,1),JJ=1,NLPRGI) /'H','K','L','F','SIGF'/,
     &     (LSPRGI(JJ,2),JJ=1,NLPRGI) /'H','K','L','F','SIGF'/
C
C
C---- This code signs which input columns are essential (LOOKUP)
C
      DATA CTPRGI/3*'H','F','Q'/
      DATA LOOKUP/5*-1/
      DATA FILIN/'HKLIN','HKLIN2'/
C
C Set defaults
C
      MAXFILES = 2
      NFILES_IN = 0
      NFILE = 0
      TITLE=' '
      FNAME='COEFFS'
      NCRYST=1
      PHI1=0.
      PHI2=180.
      DPHI=0.
      OMEG1=0.
      OMEG2=180.
      DOMEG=0.
      KAPA1=0.
      KAPA2=180.
      DKAPA=0.
      ARAD=20.
      EPS=0.
      RESMIN=10000.
      RESMAX=2.
      SMIN=1./RESMIN**2
      SMAX=1./RESMAX**2
      DO I=1,2
        NCODES(I)=1
        LSPGPS(I)=1
        CELL(1,I)=0.
        TFACS(I)=0.
        FRMINS(I)=0.
        FRMAXS(I)=1E20
C
CEJD        CALL ZEROI(SYM(1,1,1,I),16)
        DO J=1,4
          DO K=1,4
            SYM(K,J,1,I)=0.
          ENDDO
          SYM(J,J,1,I)=1.
        ENDDO
        NSYM(I)=0
      ENDDO
      LPRINT=-100
      LDL=MDL
      LMAP=0
      LPLOT=0
      NCONTR=0
      NPEAK=0
      NJOIN=0
      PEAK=10.
      MAXPEK=10
      LPKRMS=0
      LREAD=0
      LSAVE=0
      LPEAK=0
      NOTCAL = .TRUE.
C
      NERR=0
      IXTL=1
C
C
1     NTOK=NPARM
      LINE=' '
      CALL PARSER(
     .  KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,.TRUE.)
C
      IF (LEND) GOTO 1000
C Ignore blank line
      IF (NTOK.EQ.0) GOTO 1
C
C Make uppercase
      CALL CCPUPC(KEY)
      DO K=1,NKEY
        IF (KEY.EQ.KEYS(K)) GOTO 9
      ENDDO
C
      WRITE(6,7)
7     FORMAT(/' Unrecognized keyword'/)
      NERR=NERR+1
      GOTO 1
C      DATA KEYS/'TITL','SELF','CROS','RESO','CRYS','CELL','LABI',
C     .   'LIMI','PRIN','NOPR','MAP ','PLOT','FIND','READ','SAVE',
C     .   'SUM ','PEAK','JOIN','MAXO','END',' '/
C
9     GOTO
     . (           10,    20,    30,    40,    50,    55,    60,
     .      70,    80,    90,   100,   110,   120,   130,   140,
     .     150,   160,   170,   180,  1000),K
C
C            ********************************************************
C TITLe  read title
C
10    TITLE=LINE(IBEG(2):)
      GOTO 1
C
C            ********************************************************
C SELF, set flag for self rotation, read integration radius ARAD,
C smoothing
C       radius EPS
C
20    NCRYST=1
C
21    IF (NTOK.GE.2) CALL GTPREA(2,ARAD,NTOK,ITYP,FVALUE)
      IF (NTOK.GE.3) CALL GTPREA(3,EPS,NTOK,ITYP,FVALUE)
      GOTO 1
C
C            ********************************************************
C CROSS, set flag for self rotation, read integration radius ARAD,
C smoothing
C       radius EPS
30    NCRYST=2
      GOTO 21
C
C            ********************************************************
C RESOlution    read resolution limits in A, if only one treat as high
C              resolution limit
40    CONTINUE
C     read resolution limits in A, if only one treat as high
C     resolution limit
      ITOK = 2
C     ********************************************************
      CALL RDRESO(ITOK,ITYP,FVALUE,NTOK,RESMIN,RESMAX,SMIN,SMAX)
C     ********************************************************
      RESMAX=MAX(RESMAX,1.)
      SMAX=MIN(SMAX,1.)
      GOTO 1
C
C
C
C
C---- CRYSTAL   - subsiduary keywords; FILE  SYMM  ORTH  BFAC FLIMITS
C
C---- Check command line for
C---- subsidiary keyword FILE   or NUMBER - crystal number
C---- subsidiary keyword ORTH  - orthogonalisation code NCODE,
C---- subsidiary keyword BFAC  - temperature factor B
C                            Fused = Finput * exp(-Bssq/lsq)
C---- subsidiary keyword FLIMITS  - minimum F and maximum F(order
C---- unimportant)
C---- subsidiary keyword CELL  - a b c ( alpha beta  gamma - default =
C---- 90)
C                      rarely needed - just for peak analysis
C
C
50    CONTINUE
C
      NOTCAL = .FALSE.
C Set defaults
      F1 = 0.0
      F2 = 1E20
      NCODE = 1
      BB = 0.0
C
      ITOK = 1
51    ITOK = ITOK + 1
      IF (ITOK.GT.NTOK) THEN
        NCODES(IXTL) = NCODE
        TFACS(IXTL) = BB
        FRMINS(IXTL) = F1
        FRMAXS(IXTL) = F2
        GOTO 1
      ENDIF
      KEY = LINE(IBEG(ITOK) :IEND(ITOK))
      CALL CCPUPC(KEY)
      IF (KEY.EQ.'FILE' .OR. KEY.EQ.'NUMB') THEN
        ITOK = ITOK + 1
C     ********************************
        CALL GTPINT(ITOK,IXTL,NTOK,ITYP,FVALUE)
C     ********************************
C
C---- Check if allowed range of files ... 1 to 2 (is this enough ???)
C
        IF (IXTL.LE.0 .OR. IXTL.GT.MAXFILES) THEN
          WRITE (6,FMT='(a,/,a,i4,a,i2,a,/,a)')
     +    ' Key_Word Error, For CRYSTAL FILE_NUMBER line:-',
     +    '  File_Number Given = ',IXTL,
     +    ' Is invalid only files 1 to ',MAXFILES,
     +    ' Allowed, Ignoring this line -',LINE(:LENSTR(LINE))
          NERR = NERR+1
        ENDIF
        GOTO 51
      ENDIF
C
C
      IF (KEY.EQ.'ORTH') THEN
        ITOK = ITOK + 1
        CALL GTPINT(ITOK,NCODE,NTOK,ITYP,FVALUE)
        GOTO 51
      ENDIF
C
      IF (KEY.EQ.'BFAC') THEN
        ITOK = ITOK + 1
        CALL GTPREA(ITOK,BB,NTOK,ITYP,FVALUE)
        GOTO 51
      ENDIF
C
      IF (KEY.EQ.'FLIM') THEN
        ITOK = ITOK + 1
        CALL GTPREA(ITOK,F1,NTOK,ITYP,FVALUE)
        ITOK = ITOK + 1
        CALL GTPREA(ITOK,F2,NTOK,ITYP,FVALUE)
        IF (F1.LT.F2) GOTO 51
        F3 = F1
        F1 = F2
        F2 = F3
        GOTO 51
      ENDIF
C
      IF (KEY.EQ.'SYMM') THEN
C SYMMetry    read crystal number or  space group
        ITOK = ITOK + 1
C
C     ************************************************************
        CALL RDSYMM(ITOK,LINE,IBEG,IEND,ITYP,FVALUE,ITOK,NAMSGP,
     +  LSPGRP,PGNAME,NSYM(IXTL),NSYMP,SYM(1,1,1,IXTL))
C     ************************************************************
C
C   Only need primitive symmetry operators
        NSYM(IXTL) = NSYMP
        LSPGPS(IXTL) = LSPGRP
        NAMSGS(IXTL) = NAMSGP
        GOTO 51
      ENDIF
C
C CELL    read cell dimensions - not usually needed.
      IF (KEY.EQ.'CELL') THEN
        DO I=4,6
          CELL(I,IXTL)=90.
        ENDDO
        ITOK = ITOK + 1
        CALL GTPREA(ITOK,CELL(1,IXTL),NTOK,ITYP,FVALUE)
        ITOK = ITOK + 1
        CALL GTPREA(ITOK,CELL(2,IXTL),NTOK,ITYP,FVALUE)
        ITOK = ITOK + 1
        CALL GTPREA(ITOK,CELL(3,IXTL),NTOK,ITYP,FVALUE)
        IF (ITOK.LT.NTOK)THEN
          ITOK = ITOK + 1
          CALL GTPREA(ITOK,CELL(4,IXTL),NTOK,ITYP,FVALUE)
          ITOK = ITOK + 1
          CALL GTPREA(ITOK,CELL(5,IXTL),NTOK,ITYP,FVALUE)
          ITOK = ITOK + 1
          CALL GTPREA(ITOK,CELL(6,IXTL),NTOK,ITYP,FVALUE)
        ENDIF
        GOTO 51
      ENDIF
C Unrecognized subkeyword
      WRITE (6, FMT='(/A,A,A/)')
     $' Key_word Error, unrecognized sub-keyword',
     $' for CRYSTAL command:  ',KEY
      NERR = NERR+1
      GOTO 51
C
C
C---- LABIN   (File_Number)  "a=b c=d ...."
C     ==========
C
  60  CONTINUE
      ITOK = 2
      KEY = LINE(IBEG(ITOK) :IEND(ITOK))
C
C     *************
      CALL CCPUPC(KEY)
C     *************
C
      IF (KEY(1:1).EQ.'F') THEN
C
C---- second arguement = 'FILE_NUMBER'
C     test if third argument is integer
C
        IF (ITYP(3).NE.2) THEN
          WRITE (6,FMT='(A,A,/,A,A,A,/,A)')
     +    ' Key_Word Error, Wrong type of input for  LABIN',' line:-',
     +    ' If Sub_KeyWord = FILE_NUMBER  then ',
     +    'third  Arguement MUST be an Integer  Ignoring this',
     +    ' line -',LINE(:LENSTR(LINE))
          NERR = NERR+1
          GOTO 10
        ENDIF
C
C---- For an input file being given
C     Get File_Number and check if valid
C
C     ********************************
        CALL GTPINT(3,NFILE,NTOK,ITYP,FVALUE)
C     ********************************
C
        NFILES_IN = NFILES_IN + 1
Cc             max(nfiles_in,nfile)
C
C---- Check if allowed range of files ... 1 to 5 (is this enough ???)
C
        IF (NFILE.LE.0 .OR. NFILE.GT.MAXFILES) THEN
          WRITE (6,FMT='(a,/,a,i4,a,i2,a,/,a)')
     +    ' Key_Word Error, For LABIN FILE_NUMBER line:-',
     +    '  File_Number Given = ',NFILE,
     +    ' Is invalid only files 1 to ',MAXFILES,
     +    ' Allowed, Ignoring this line -',LINE(:LENSTR(LINE))
          GOTO 10
        ENDIF
      ENDIF
C
C---- Now save labels for this input file
C
      LABIN_SAVE(NFILE) = 'LABIN  '//LINE(IBEG(4) :IEND(NTOK))
Cc      GOT_LABIN(NFILE)  = .TRUE.
C
C
      GOTO 1
C
C  CELL   read cell dimensions
C
55    CONTINUE
      GOTO 1
C
C
C
C  LIMIts  read limits and steps on phi,omega,kappa
C
70    CALL GTPREA(2,PHI1,NTOK,ITYP,FVALUE)
      CALL GTPREA(3,PHI2,NTOK,ITYP,FVALUE)
      CALL GTPREA(4,D,NTOK,ITYP,FVALUE)
      IF (D.LT.1.) CALL CCPERR(1,'ERROR in phi step.')
      DPHI=30./NINT(30./D)
      IF (NINT(10.*DPHI).NE.NINT(10.*D)) WRITE(6,'(/2(A,F7.2))')
     & 'WARNING: Phi   step  changed from',D,' to',DPHI
      TEMP=MIN(MAX(PHI1,0.),360.)
      IPHI1=NINT(TEMP/DPHI)
      IF (NINT(IPHI1*DPHI).NE.NINT(TEMP)) WRITE(6,'(/2(A,F7.2))')
     & 'WARNING: Phi   start changed from',PHI1,' to',IPHI1*DPHI
      TEMP=MIN(MAX(PHI2,TEMP),360.)
      IPHI2=NINT(TEMP/DPHI)
      IF (NINT(IPHI2*DPHI).NE.NINT(TEMP)) WRITE(6,'(/2(A,F7.2))')
     & 'WARNING: Phi   end   changed from',PHI2,' to',IPHI2*DPHI
      IF (IPHI2.LE.IPHI1) CALL CCPERR(1,'ERROR in phi limits.')
C
      CALL GTPREA(5,OMEG1,NTOK,ITYP,FVALUE)
      CALL GTPREA(6,OMEG2,NTOK,ITYP,FVALUE)
      CALL GTPREA(7,D,NTOK,ITYP,FVALUE)
      IF (D.LT.1.) CALL CCPERR(1,'ERROR in omega step.')
      DOMEG=30./NINT(30./D)
      IF (NINT(10.*DOMEG).NE.NINT(10.*D)) WRITE(6,'(/2(A,F7.2))')
     & 'WARNING: Omega step changed from',D,' to',DOMEG
      TEMP=MIN(MAX(OMEG1,0.),360.)
      IOMEG1=NINT(TEMP/DOMEG)
      IF (NINT(IOMEG1*DOMEG).NE.NINT(TEMP)) WRITE(6,'(/2(A,F7.2))')
     & 'WARNING: Omega start changed from',OMEG1,' to',IOMEG1*DOMEG
      TEMP=MIN(MAX(OMEG2,TEMP),360.)
      IOMEG2=NINT(TEMP/DOMEG)
      IF (NINT(IOMEG2*DOMEG).NE.NINT(TEMP)) WRITE(6,'(/2(A,F7.2))')
     & 'WARNING: Omega end   changed from',OMEG2,' to',IOMEG2*DOMEG
      IF (IOMEG2.LE.IOMEG1) CALL CCPERR(1,'ERROR in omega limits.')
C
      CALL GTPREA(8,KAPA1,NTOK,ITYP,FVALUE)
      CALL GTPREA(9,KAPA2,NTOK,ITYP,FVALUE)
      CALL GTPREA(10,D,NTOK,ITYP,FVALUE)
      IF (D.LT.1.) CALL CCPERR(1,'ERROR in kappa step.')
      DKAPA=30./NINT(30./D)
      IF (NINT(10.*DKAPA).NE.NINT(10.*D)) WRITE(6,'(/2(A,F7.2))')
     & 'WARNING: Kappa step changed from',D,' to',DKAPA
      TEMP=MIN(MAX(KAPA1,0.),360.)
      IKAPA1=NINT(TEMP/DKAPA)
      IF (NINT(IKAPA1*DKAPA).NE.NINT(TEMP)) WRITE(6,'(/2(A,F7.2))')
     & 'WARNING: Kappa start changed from',KAPA1,' to',IKAPA1*DKAPA
      TEMP=MIN(MAX(KAPA2,TEMP),360.)
      IKAPA2=NINT(TEMP/DKAPA)
      IF (NINT(IKAPA2*DKAPA).NE.NINT(TEMP)) WRITE(6,'(/2(A,F7.2))')
     & 'WARNING: Kappa end   changed from',KAPA2,' to',IKAPA2*DKAPA
      IF (IKAPA1.LT.0 .OR. IKAPA2.LT.IKAPA1)
     & CALL CCPERR(1,'ERROR in kappa limits.')
      GOTO 1
C
C PRINt    read print threshold
C
80    LPRINT=1
      CALL GTPINT(2,LPRINT,NTOK,ITYP,FVALUE)
      GOTO 1
C
C NOPRint  switch off map print
C
90    LPRINT=0
      GOTO 1
C
C MAP   switch to write map to map file
C
100   LMAP=1
      GOTO 1
C
C PLOT  read contour start, interval , NOTItle flag
C
110   LPLOT=1
      C1=10.
      C2=100.
      C=10.
      IF (NTOK.GE.2) CALL GTPREA(2,C1,NTOK,ITYP,FVALUE)
      IF (NTOK.GE.3) CALL GTPREA(3,C,NTOK,ITYP,FVALUE)
      IF (C1.GT.C2 .OR. C.LE.0) THEN
        WRITE(6,9110) C1,C2,C
9110    FORMAT(/' Unreasonable contour limits:',3F10.2/)
        NERR=NERR+1
        GOTO 1
      ENDIF
C read NOTItle flag to suppress title plotting
      IF (NTOK.GE.4) THEN
        CALL CCPUPC(CVALUE(4))
        IF (CVALUE(4).EQ.'NOTI') LPLOT=-1
      ENDIF
      NCONTR=1
      CONTRS(1)=C1
111   CONTRS(NCONTR+1)=CONTRS(NCONTR)+C
      IF (CONTRS(NCONTR+1).GT.C2) GOTO 1
      NCONTR=NCONTR+1
      IF (NCONTR.LT.MCONTR) GOTO 111
C
C  FIND read peak threshold and maximum number of peaks
C    optional keyword "RMS" to indicate that threshold is in
C       multiples of RMS density rather than absolute
C    optional keyword OUTPUT filename to output unique peak list to file
C    NB RMS must precede OUTPUT if both are present
C
120   CALL GTPREA(2,PEAK,NTOK,ITYP,FVALUE)
      CALL GTPINT(3,MAXPEK,NTOK,ITYP,FVALUE)
      MAXPEK=MIN(MAXPEK,MAXNPK)
C  Process rest of line
      LPKRMS=0
      LPKOUT=0
      IF (NTOK.LT. 4) GOTO 1
      DO J = 4, NTOK
        IF (CVALUE(J).EQ.'RMS') THEN
          LPKRMS=1
        ELSEIF(CVALUE(J).EQ.'OUTP') THEN
          LPKOUT=1
C Store rest of line as filename if given
          IF (NTOK.GT.J) THEN
            PKFNAM=LINE(IBEG(J+1):)
            GOTO 1
          ELSE
            PKFNAM='PEAKS'
          ENDIF
        ENDIF
      ENDDO
      GOTO 1
C
C READ   set flag to read map in, instead of calculating it
C        read 1st and last sections to process (kappa values)
C
130   LREAD=1
      RKAP1=-1.
      RKAP2=-1.
      CALL GTPREA(2,RKAP1,NTOK,ITYP,FVALUE)
      CALL GTPREA(3,RKAP2,NTOK,ITYP,FVALUE)
      IF (RKAP2.LT.0.) RKAP2 = RKAP1
      IF (RKAP1.LT.0. .OR. RKAP2.LT.RKAP1) THEN
        WRITE (6, '(A)') ' *** Illegal read limits ***'
        NERR = NERR+1
      ENDIF
      GOTO 1
C
C SAVE  read file name for saving ALMN output
140   LSAVE=1
C
141   IF (NTOK.GT.1) THEN
        FNAME=LINE(IBEG(2):IEND(2))
C&&&VAX !!!
C Strip off any existing extension
        I=INDEX(FNAME,']')
        J=INDEX(FNAME(I+1:),'.')
        IF (J.GT.0) FNAME=FNAME(1:J-1)
      ENDIF
      GOTO 1
C
C SUM  read file name for reading previous ALMN results
C
150   LSAVE=-1
      GOTO 141
C
C PEAK read list of peaks omega, phi, kappa from following cards,
C      terminated by blank line or 'END'
C
160   LPEAK=1
      NPEAK=0
      IF (NTOK.GT.1) THEN
C  Test second token to see if it the keyword MATRices
        CALL CCPUPC(CVALUE(2))
        IF (CVALUE(2).EQ.'MATR') THEN
          LPEAK=2
          WRITE(6,*) ' Matrices will be output for each peak'
        ENDIF
      ENDIF
161   NTOK=NPARM
      LINE=' '
      CALL PARSER(
     .KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,.TRUE.)
C
      IF (LEND) GOTO 1000
C Make uppercase
      CALL CCPUPC(KEY)
      IF (NTOK.LE.0 .OR. KEY.EQ.'END') GOTO 1
      IF (NPEAK.GE.MAXNPK) THEN
        WRITE(6,162) MAXNPK
162     FORMAT(/' Too many peaks given, maximum =',I4/)
        NERR=NERR+1
        GOTO 1
      ENDIF
      IF (NTOK.LT.3) THEN
        WRITE(6,163)
163     FORMAT(/' !!! Three angles must be given !!!')
        GOTO 161
      ENDIF
      NPEAK=NPEAK+1
      CALL GTPREA(1,PKPOS(1,NPEAK),NTOK,ITYP,FVALUE)
      CALL GTPREA(2,PKPOS(2,NPEAK),NTOK,ITYP,FVALUE)
      CALL GTPREA(3,PKPOS(3,NPEAK),NTOK,ITYP,FVALUE)
      LPKORD(NPEAK)=NPEAK
      GOTO 161
C
C
C JOIN  read peak positions to join up
C  kappa, omega1, phi1, omega2, phi2, [DASHED  repeat  dash]
C                                         (in fractions of radius)
C
170   IF (NJOIN.GE.MXJOIN) THEN
        WRITE(6,1170) MXJOIN
1170    FORMAT(/' !!!! No more than ',I4,' joins can be stored'//)
        GOTO 1
      ENDIF
      IF (NTOK.LT.6) THEN
        WRITE(6,1171)
1171    FORMAT(' Too few numbers for JOIN !!!')
        NERR=NERR+1
        GOTO 1
      ENDIF
C
      NJOIN=NJOIN+1
C read kappa
      CALL GTPREA(2,PEAKJ(3,1,NJOIN),NTOK,ITYP,FVALUE)
C Read 2 two sets of omega, phi
      K=2
      DO J=1,2
        DO I=1,2
          K=K+1
          CALL GTPREA(K,PEAKJ(I,J,NJOIN),NTOK,ITYP,FVALUE)
        ENDDO
      ENDDO
C
C Check field 7 for keyword DASHed
      IF (NTOK.GE.7 .AND. ITYP(7).EQ.1) THEN
        CALL CCPUPC(CVALUE(7))
        IF (CVALUE(7).EQ.'DASH') THEN
C    Set default repat and dash length
          DASH(1,NJOIN)=0.05
          DASH(2,NJOIN)=0.025
          CALL GTPREA(8,DASH(1,NJOIN),NTOK,ITYP,FVALUE)
          CALL GTPREA(9,DASH(2,NJOIN),NTOK,ITYP,FVALUE)
        ELSE
C     Solid
          DASH(1,NJOIN)=0.0
        ENDIF
      ENDIF
      GOTO 1
C
C MAXOrd  max Bessel order to use (mostly for test purposes).
C
180   CALL GTPINT(2,LDL,NTOK,ITYP,FVALUE)
      IF (LDL.LT.2 .OR. LDL.GT.MDL)
     & CALL CCPERR(1,'ERROR: MAXORD out of range.')
      GOTO 1
C
C---------------------------------------------------------------------
C
C End of input
C
C Check for errors
1000  IF (NERR.GT.0) THEN
        WRITE(6,9901) NERR
9901    FORMAT(/1X,I6,' errors in input, see above !!!!'/)
        CALL CCPERR(1,' error in input')
      ENDIF
C   Open MTZ files
C---- Open input file for Read; print header...
C
      IF (NFILES_IN .GE.0) THEN
        WRITE(6,*) ' Using ',NFILES_IN,' files'
        DO IFIL=1,NFILES_IN
          IERR = -1
          IPRINT = 2
          LOOKUP(1) = -1
          LOOKUP(2) = -1
          LOOKUP(3) = -1
          LOOKUP(4) = -1
          LOOKUP(5) = -1
C
C     ==================================
          CALL LROPEN(IFIL,FILIN(IFIL),IPRINT,IERR)
C     ==================================
C
          ERRLIN = 'ERROR:HEADER NOT THERE, ILLEGAL FILE'
C
C     =========================================
          IF (IERR.EQ.3) CALL LERROR(2,1,ERRLIN(:LENSTR(ERRLIN)))
C     =========================================
C
C
C
Cc         line= labin_save(nfile)
          LINE= LABIN_SAVE(IFIL)
C
C     ======================================================
          CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,
     +    LEND,.TRUE.)
          CALL LKYIN(IFIL,LSPRGI(1,IFIL),NLPRGI,NTOK,LINE,IBEG,IEND)
          CALL LRASSN(IFIL,LSPRGI(1,IFIL),NLPRGI,LOOKUP,CTPRGI)
C     ======================================================
C
C     ======================
          CALL LRCELL(IFIL,CELCHK)
C     ======================
C---- Get dataset info using LRIDX:
C     Notify available space
      NDATASETS = MaxSets
      CALL LRIDX(IFIL,PNAME,XNAME,DNAME,ISETS,
     +                     DATCELL,DATWAVE,NDATASETS)
C
C---- We can only check the datasets if there is dataset
C     information in the mtz file
C
      IF (NDATASETS.GT.0) THEN
C
C---- Get the complete list of labels and types and the

C     datasets to which they belong
C
        NCOL = MCOLS
C       ****************************************
        CALL LRCLID(IFIL,CSETID,NCOL)
C       ****************************************
C     Get setid of FP column and use that DATCELL as master for anisotropic analysis
        SET_INDEX = 0
        SETID = 0
        IF (LOOKUP(4).GT.0) THEN
          SETID = CSETID(LOOKUP(4))
          DO ISET = 1,NDATASETS
            IF (ISETS(ISET).EQ.SETID) THEN
              SET_INDEX = ISET
            END IF
          ENDDO
        ENDIF
C     Set output dataset and correct choice of cell
        IF (SET_INDEX.GT.0) THEN
          DO I=1,6
            CELCHK(I) = DATCELL(I,SET_INDEX)
          ENDDO
        ENDIF
      ENDIF
C      check cell
          IF (CELL(1,IFIL).NE.0.0) THEN
            ACHK = 0.0
            DO II = 1,6
              ACHK = ACHK +
     +        ABS(CELCHK(II) - CELL(II,IFIL)) /
     +        (CELCHK(II) + CELL(II,IFIL))
              IF (ACHK.GT.0.02)CALL CCPERR(1,
     +        ' Large Difference in CELL DIMENSIONS')
              IF (ACHK.GT.0.01) WRITE(6,*)
     +        ' Small Difference in CELL DIMENSION'
            ENDDO
          ELSE
            DO II = 1,6
              CELL(II,IFIL) =  CELCHK(II)
            ENDDO
          ENDIF
C
C---- Find out how many columns in input file?
C
C     ========================================
          CALL LRINFO(IFIL,VERSNX,NCOL,NREFTOT,COLRNG)
C     ========================================
C
C    Read symmetry and check
          CALL LRSYMI(IFIL,NSYMP,LTYPE,LSPGRP,NAMSGP,PGNAME)
          CALL LRSYMM(IFIL,NSM,SYM(1,1,1,IFIL) )
C
          IF (NSYM(IFIL).EQ.0 )THEN
C   Only need primitive symmetry operators
            NSYM(IFIL) = NSYMP
            LSPGPS(IFIL) = LSPGRP
            NAMSGS(IFIL) = NAMSGP
          ENDIF
C
          IF (NSYM(IFIL).GT.0 .AND. NSYM(IFIL).NE.NSYMP)
     &      CALL CCPERR(1,' Inconsistent symmetry information')
C
C*****READ ALL REFLECTION INTENSITIES AND CALCULATE SPHERICAL
C*****HARMONIC COEFFICIENTS ALMN.
C
          TFAC2=TFACS(IFIL)/4.
          SMINR=100.
          SMAXR=0.
5         CALL LRREFF(IFIL,S,ADATA,EOF)
          IF (EOF) GOTO 8
          CALL LRREFM(IFIL,LOGMSS)
          DO I=1,5
            IF (LOGMSS(I)) GOTO 5
          ENDDO
C
C*****CHECK IF REFLECTION IS WITHIN SPECIFIED RESOLUTION RANGE
          IF (S.LT.SMIN .OR. S.GT.SMAX) GOTO 5
          IF (ADATA(5).EQ.0.) GOTO 5
C***** Apply temperature factor before squaring F
          FR=ADATA(4)
          IF (TFAC2.NE.0.) FR=FR*EXP(-TFAC2*S)
C
C*****CHECK IF AMPLITUDE ABOVE THRESHOLD
          IF (FR.LE.FRMINS(IFIL) .OR. FR.GT.FRMAXS(IFIL)) GOTO 5
          SMINR=MIN(SMINR,S)
          SMAXR=MAX(SMAXR,S)
          GOTO 5
C
8         CALL LRREWD(IFIL)
          SMIN=MAX(SMIN,SMINR)
          SMAX=MIN(SMAX,SMAXR)
        ENDDO
        RESMIN=1./SQRT(SMIN)
        RESMAX=1./SQRT(SMAX)
      ENDIF
C
      DO IFIL = 1,NCRYST
C  Call pgdefn to make sure you have a primitive set of symops.
C
C---- Check NCODE defined
C
        IF (NCODES(IFIL).LT.1 .OR. NCODES(IFIL).GT.6) THEN
          WRITE (6, 6101) J
6101      FORMAT(' !!!! Illegal or undefined NCODE for crystal ',I4/)
          CALL CCPERR(1,' error in ncode')
        ENDIF
C
C
C  Set up fractional to orthogonal matrix RR with appropriate NCODE.
        CALL RBFRO1(CELL(1,IFIL),VOLL,RR)
C
        DO I=1,3
          DO J=1,3
            RO(I,4)=0.
            RO(4,I)=0.
            RO(I,J)=RR(I,J,NCODES(IFIL))
            ORTHOG(I,J,IFIL)=RR(I,J,NCODES(IFIL))
          ENDDO
        ENDDO
        RO(4,4)=1.0
C
C  and its inverse
        CALL RBRINV(RO,RF)
        DO I=1,3
          DO J=1,3
            FRAC(I,J,IFIL)=RF(I,J)
          ENDDO
        ENDDO
      ENDDO
C
C Check that crystal cards were given and LCF files opened
C unless READ or SUM options used
        IF (NOTCAL .AND. LREAD.EQ.0 .AND. LSAVE.GE.0 .AND. LPEAK.EQ.0)
     &  THEN
          WRITE(6,9902)
9902      FORMAT(/' No CRYSTAL card given !!!'/)
          CALL CCPERR(1,' error in crystal')
        ENDIF
C
C If 1 crystal, copy symmetry etc to crystal 2
        IF (NCRYST.EQ.1) THEN
          DO IS =1,NSYM(1)
            DO I =1,4
              DO J =1,4
                SYM(I,J,IS,2) = SYM(I,J,IS,1)
              ENDDO
            ENDDO
          ENDDO
          NSYM(2)=NSYM(1)
          NCODES(2)=NCODES(1)
          LSPGPS(2)=LSPGPS(1)
CEJD          CALL MOVEI(ORTHOG(1,1,2),ORTHOG(1,1,1),9)
CEJD          CALL MOVEI(FRAC(1,1,2),FRAC(1,1,1),9)
          DO I =1,3
            DO J =1,3
              ORTHOG(J,I,2) = ORTHOG(J,I,1)
              FRAC(J,I,2) = FRAC(J,I,1)
            ENDDO
          ENDDO
        ENDIF
C
C  Default title
C
        IF (LENSTR(TITLE).EQ.0) THEN
          IF (NCRYST.EQ.1) THEN
            TITLE='Self-rotation function'
          ELSE
            TITLE='Cross-rotation function'
          ENDIF
        ENDIF
C
C Print control data
C
C  PEAK option
C
      IF (LPEAK.GT.0) THEN
        WRITE(6,9101) NPEAK
9101    FORMAT('1    Rotation function peak calculation'///
     .  ' All symmetry related peaks and matrices will be',
     .  ' calculated for ',I4,' input peaks'//)
        DO I=1,NCRYST
          WRITE(6,9102) I,NCODES(I),CODES(NCODES(I)),LSPGPS(I),
     .    NAMSGS(I)(:LENSTR(NAMSGS(I))),NSYM(I)
9102      FORMAT(/' Crystal',I3,': orthogonalisation code (NCODE) =',
     .    I2,'   x y z axes along ',A//' Space group ',I4,
     .    ' (',A,'),',I4,' symmetry operations'//)
          WRITE(6,9103) (CELL(J,I),J=1,6)
9103      FORMAT(' Cell dimensions: ',6F8.2/)
        ENDDO
        RETURN
      ENDIF
C
      IF (LREAD.EQ.0) THEN
C Set print default if not set explicitly
        IF (LPRINT.LT.-99) LPRINT=1
        IF (NCRYST.EQ.1) THEN
          WRITE(6,9001) 'Self',TITLE
9001      FORMAT(20X,'Polar angle rotation function:  ',A,
     &    '-rotation function'/20X,
     &    '======================================================='//
     &    ' Title:  ',A//)
        ELSE
          WRITE(6,9001) 'Cross',TITLE
        ENDIF
C
        A2PI=TWOPI*ARAD
        RESMIN=MIN(A2PI,RESMIN)
        SMIN=1./RESMIN**2
C
        LDN=LDL/2
        LDL=2*LDN
        CALL ZJVX(LDL+.5D0,0,BJ,DJ,X0)
C
        RMAX=X0/TWOPI
        IF (RESMAX.LT.ARAD/RMAX) THEN
C  Reset maximum resolution to integration radius/RMAX
          RESMAX=ARAD/RMAX
          SMAX=1./RESMAX**2
          WRITE(6,9002) RESMAX,ARAD,RMAX
9002      FORMAT(/'!!! Warning !!! maximum resolution reset to',F7.3,
     &    ' = integration radius',F8.3,' /',F6.2/)
        ELSE
          X0=A2PI/RESMAX
        ENDIF

C      This is the documented default. 
C      Note: after possible resetting of RESMAX.
        IF (EPS.EQ.0.0) EPS = 0.7 * RESMAX
C
C***** Compute zeroes of spherical Bessel functions.
        DO LBY2=1,LDN
          L=2*LBY2
          N=0
2         CALL ZJVX(L+.5D0,N,BJ,DJ,X)
          IF (X.LE.X0) THEN
            IF (N.EQ.LDN) CALL CCPERR(1,'N > LDN')
            N=N+1
            BZ(N,LBY2)=X
            GOTO 2
          ENDIF
          IF (N.EQ.0) GOTO 3
          NMAX(LBY2)=N
        ENDDO
C
3       LBY2MX=LBY2-1
C        DO LBY2=1,LBY2MX
C          WRITE(6,'(/A,2I4)') 'L, NMAX =',2*LBY2,NMAX(LBY2)
C          WRITE(6,'(5F12.6)') (BZ(N,LBY2),N=1,NMAX(LBY2))
C        ENDDO
C
        WRITE(6,9003) ARAD,RESMIN,RESMAX
9003    FORMAT(/' Integration radius = ',F9.2,'A',4X,
     .  'Resolution limits ',2F9.2)
C
        IF (EPS.GT.0.) WRITE(6,9004) EPS
9004    FORMAT(/' Radius for averaging (smoothing) ',F10.3)
C
        DO I=1,NCRYST
          WRITE(6,9010) I,NCODES(I),CODES(NCODES(I)),LSPGPS(I),
     &    NAMSGS(I)(:LENSTR(NAMSGS(I))),NSYM(I),TFACS(I),FRMINS(I),
     &    FRMAXS(I)
9010      FORMAT(/' Crystal',I3,': orthogonalisation code (NCODE) =',
     .    I2,'   x y z axes along ',A//' Space group ',I4,' (',A,'),',
     .    I4,' symmetry operations'//' Temperature factor =',F10.3/
     .    5X,'Fmin =',F10.0,4X,'Fmax =',1PG14.4/)
          WRITE(6,9015) (CELL(J,I),J=1,6)
9015      FORMAT(' Cell dimensions: ',6F8.2)
        ENDDO
C
        L=LENSTR(FNAME)
        IF (LSAVE.GT.0) THEN
          WRITE(6,9011) FNAME(:L)
9011      FORMAT(/
     .    ' Output from ALMN stage will be saved in file(s) : ',A)
        ELSEIF(LSAVE.LT.0) THEN
          WRITE(6,9012) FNAME(:L)
9012      FORMAT(/
     .    ' Output from ALMN stage will be read from file(s) : ',A)
        ENDIF
C
C Check for maximum section size
C
        IF (DPHI.EQ.0.) THEN
          DPHI=30./NINT(30./RESMAX)
          IPHI1=NINT(PHI1/DPHI)
          IPHI2=NINT(PHI2/DPHI)
        ENDIF
        IF (DOMEG.EQ.0.) THEN
          DOMEG=30./NINT(30./RESMAX)
          IOMEG1=NINT(OMEG1/DOMEG)
          IOMEG2=NINT(OMEG2/DOMEG)
        ENDIF
        IF (DKAPA.EQ.0.) THEN
          DKAPA=30./NINT(30./RESMAX)
          IKAPA1=NINT(KAPA1/DKAPA)
          IKAPA2=NINT(KAPA2/DKAPA)
        ENDIF
        WRITE(6,'(3(/A,3X,3F8.2))')
     &  'Ranges & steps in phi   =',IPHI1*DPHI,IPHI2*DPHI,DPHI,
     &  'Ranges & steps in omega =',IOMEG1*DOMEG,IOMEG2*DOMEG,DOMEG,
     &  'Ranges & steps in kappa =',IKAPA1*DKAPA,IKAPA2*DKAPA,DKAPA
C
        NPHI=IPHI2-IPHI1+1
        NOMEG=IOMEG2-IOMEG1+1
        NKAPA=IKAPA2-IKAPA1+1
        NSECSZ=NPHI*NOMEG
        IF (NSECSZ.GT.NSMAP) THEN
          WRITE(6,9050) NSECSZ,NSMAP
9050      FORMAT(//' Section size too large, =',I8,'    maximum=',I8///)
          CALL CCPERR(1,' error in section')
        ENDIF
      ELSE
C  READ option
        WRITE(6,9020) TITLE
9020    FORMAT(/,' Previously calculated map will be read from MAPIN'//
     .  ' Title: ',A/)
C no save, no map output
        LSAVE=0
        LMAP=0
      ENDIF

C Report FIND parameters
      IF (MAXPEK.NE.0) THEN
        IF (LPKRMS.EQ.0) THEN
          WRITE(6,9030) MAXPEK,PEAK
 9030     FORMAT(/,' Sections will be searched for a maximum of ',I5,
     +    ' peaks above a threshold of ',F10.3)
        ELSE
          WRITE(6,9035) MAXPEK,PEAK
 9035     FORMAT(/,' Sections will be searched for a maximum of ',I5,
     +    ' peaks above a threshold of ',F10.3,
     +    ' * RMS density')
        ENDIF
      ENDIF

      END
C
C***********************************************************************
C******************* ND500-ALMN ********************** W.KABSCH 10-1982
C***********************************************************************
C************************ FILES ****************************************
C
      SUBROUTINE CALMN
C     ===============
C
C
C OUTPUT FILE
C -----------
C THE OUTPUT DATA FILE CONTAINS BINARY DATA AS FOLLOWS:
C
C         L,M,N,ALMNR,ALMNI
C
C L,M,N     ARE THE INDICES OF THE SPHERICAL HARMONIC COEFFICIENT.
C ALMNR,    ARE REAL AND IMAGINARY PART OF SPHERICAL HARMONIC
C ALMNI     EXPANSION COEFFICIENT
C
C ----------
C
C PARAMETERS
C    ARAD                 RADIUS (ANGSTROM) OF CUT-OFF SPHERE IN
C                         PATTERSON FUNCTION. THIS PROGRAM ALLOWS A
C                         LARGEST VALUE FOR L OF 100. THIS LEADS TO THE
C                         CONDITION ARAD < RESMAX*17.4.
C    RESMIN               LOW  RESOLUTION LIMIT (ANGSTROM),
C    RESMAX               HIGH RESOLUTION LIMIT FOR ACCEPTING DATA.
C    EPS                  RADIUS (ANGSTROM) OF SPHERE WITHIN WHICH THE
C                         PATTERSON IS AVERAGED. EPS SHOULD BE LESS
C                         THAN 0.7*RESMAX.
C    FRMIN                REFLECTIONS WITH INTENSITY EQUAL OR LESS THAN
C                         FRMIN ARE NOT INCLUDED IN THE CALCULATION.
C    NEQ                  NUMBER OF EQUIVALENT POSITIONS IN SPACE GROUP
C                         OF PATTERSON. THIS INCLUDES THE UNIT MATRIX,
C                         BUT NOT POSITIONS RELATED BY A CENTRE OF
C                         SYMMETRY AT THE ORIGIN.
C    LIST                 0 - NO LISTING
C                         1 - LIST SPHERICAL HARMONIC COEFFICIENTS
C
C 3.)UNIT CELL ORIENTATION
C    BASIS(1),(2),(3)  X,Y,Z COMPONENTS (ANGSTROM) OF A-AXIS,
C    BASIS(4),(5),(6)  X,Y,Z COMPONENTS (ANGSTROM) OF B-AXIS,
C    BASIS(7),(8),(9)  X,Y,Z COMPONENTS (ANGSTROM) OF C-AXIS
C                         OF UNIT CELL RELATIVE TO REFERENCE FRAME.
C
C 4.)EQUIVALENT POSITIONS (NEQ OF THEM)
C    MEQ(1),...,(9)    H'=MEQ(1)*H+MEQ(2)*K+MEQ(3)*L,
C                         K'=MEQ(4)*H+MEQ(5)*K+MEQ(6)*L,
C                         L'=MEQ(7)*H+MEQ(8)*K+MEQ(9)*L,
C                         WHERE H',K',L' IS A SYMMETRY RELATED
C                         REFLECTION TO H,K,L.
C
C***********************************************************************
C
C
      INTEGER NLPRGI
      PARAMETER(NLPRGI=5)
C
      REAL ADATA(NLPRGI)
      LOGICAL LOGMSS(NLPRGI)
      LOGICAL EOF
      DIMENSION RSYM(4,4)
      COMMON /MATRIX/ ORTHOG(3,3,2),FRAC(3,3,2),SYM(4,4,192,2),NSYM(2),
     . MRSYM,JRSYM(3,3,12),LSYMFG,LHALF
C
      PARAMETER(MCONTR=50)
      COMMON /ALMFRF/ NCRYST,IPHI1,IPHI2,NPHI,DPHI,IOMEG1,IOMEG2,NOMEG,
     . DOMEG,IKAPA1,IKAPA2,NKAPA,DKAPA,ARAD,EPS,RESMIN,RESMAX,SMIN,SMAX,
     . NCODES(2),LSPGPS(2),TFACS(2),FRMINS(2),FRMAXS(2),LPRINT,LMAP,
     . LPLOT,CONTRS(MCONTR),NCONTR,LREAD,LSAVE,PEAK,MAXPEK,LPKRMS,RKAP1,
     . RKAP2,RMIN,RMEAN,RRMS,RWT,NPOINT,LDL
      CHARACTER*80 TITLE,FNAME
      COMMON /TEXTC/ TITLE,FNAME
C
      PARAMETER(NSRANG=10)
      DIMENSION NRRANG(NSRANG)
C
      INTEGER TAPOT
      PARAMETER(MDL=100, MDL1=MDL+1, MDN=MDL/2, MDALMN=99999)
      COMMON/POLDAT/ LBY2MX,NMAX(MDN),BZ(MDN,MDN)
      DIMENSION IH(48),IK(48),IL(48),MEQ(216)
      DIMENSION BASIS(9),RBASIS(9),SPHBES(MDL),RLN(MDN)
C
      REAL*8 COSTH,COTTH,SINTH,PHI,T0,T1,T2,VM,BJ(0:MDL),FACTOR(MDN),
     & FAC1(MDN,MDL),FAC2(MDN,MDL),CMPHI(MDL1),SMPHI(MDL1),PLM(MDL1),
     & ARRAY(MDL*MDL1),ALMN(MDALMN)
C
      CHARACTER NUMS(2)*1,FILEN*80,STATUS*8
      DATA NUMS/'1','2'/
C
      PARAMETER(PI=3.141592654D0, TWOPI=2.*PI)
C
      LIST=0
C
C-----------------------------------------------------------------------
C
C Loop on 1 or two crystals
C
C
      LMAX=2*LBY2MX
      LMAX1=LMAX+1
      LDL1N=LMAX1*LBY2MX
      TAPOT = 8
      NCRYST=MIN(MAX(NCRYST,1),2)
      LDN=LDL/2
      A2PI=TWOPI*ARAD
      RESINT=1./(RESMAX**2*NSRANG)
C
      T0=1D0
      LGTH=0
      DO LBY2=1,LBY2MX
        L=2*LBY2
        L2=2*L
        T0=T0*(L2-3)*(L2-1)/(L2*(L2-2))
        FACTOR(LBY2)=SQRT((L2+1)*T0)
        LGTH=LGTH+(L+1)*NMAX(LBY2)
        DO M1=1,L
          T1=(L+1-M1)*(L+M1)
          FAC1(LBY2,M1)=-2*M1/SQRT(T1)
          FAC2(LBY2,M1)=-SQRT((L-M1)*(L+M1+1)/T1)
        ENDDO
      ENDDO
C
      LGTH2=2*LGTH
      WRITE(6,1040) LMAX,LGTH,(NMAX(LBY2),LBY2=1,LBY2MX)
1040  FORMAT(//'LMAX =',I4,4X,
     &'NUMBER OF SPHERICAL HARMONIC COEFFICIENTS =',I6/'NMAX(L)',
     &20I4:/(7X,20I4))
      WRITE(6,'()')
      IF (LGTH2.GT.MDALMN) CALL CCPERR(1,'INCREASE PARAMETER MDALMN')
C
      DO IXTL=1,NCRYST
C
C Open output file for ALMN coefficients
        FILEN=FNAME(:LENSTR(FNAME))//NUMS(IXTL)
C Status normally SCRATCH, but NEW if SAVE option used
        IF (LSAVE.GT.0) THEN
          STATUS='NEW'
        ELSE
          STATUS='SCRATCH'
        ENDIF
        IFAIL=0
        CALL CCPDPN(TAPOT,FILEN,STATUS,'U',1,IFAIL)
C
C Set up orthogonalisation matrix  (basis set)
        DO I=1,3
          BASIS(3*I-2) = ORTHOG(1,I,IXTL)
          BASIS(3*I-1) = ORTHOG(2,I,IXTL)
          BASIS(3*I) = ORTHOG(3,I,IXTL)
        ENDDO
CEJD        CALL MOVEI(BASIS(1),ORTHOG(1,1,IXTL),9)
        WRITE(6,1002) IXTL,(BASIS(I),BASIS(I+3),BASIS(I+6),I=1,3)
1002    FORMAT(/' Crystal number : ',I5/
     .  ' Basis set (orthogonalisation matrix) :',3F10.5,2(/39X,3F10.5))
C
C
C Invert and transpose symmetry to get reciprocal space matrices
        DO L=1,NSYM(IXTL)
          CALL RBRINV(SYM(1,1,L,IXTL),RSYM)
          L0=(L-1)*9
          DO I=1,3
            DO J=1,3
              K=(I-1)*3+J+L0
              MEQ(K)=NINT(RSYM(J,I))
            ENDDO
          ENDDO
          WRITE(6,1003) L,(MEQ(I+L0),I=1,9)
1003      FORMAT(/' Symmetry operation ',I3,': ',3I4,2(/25X,3I4)/)
        ENDDO
        NEQ=NSYM(IXTL)
        FRMIN=FRMINS(IXTL)
        FRMAX=FRMAXS(IXTL)
        TFAC2=TFACS(IXTL)/4.
C Counts of reflections in resolution ranges
CEJD        CALL ZEROI(NRRANG,NSRANG)
        DO ISRANG = 1,NSRANG
          NRRANG(ISRANG) = 0
        ENDDO
C
C*****DETERMINE COMPONENTS OF RECIPROCAL CELL VECTORS
        RBASIS(1)=BASIS(5)*BASIS(9)-BASIS(8)*BASIS(6)
        RBASIS(2)=BASIS(6)*BASIS(7)-BASIS(9)*BASIS(4)
        RBASIS(3)=BASIS(4)*BASIS(8)-BASIS(7)*BASIS(5)
        RBASIS(4)=BASIS(8)*BASIS(3)-BASIS(2)*BASIS(9)
        RBASIS(5)=BASIS(9)*BASIS(1)-BASIS(3)*BASIS(7)
        RBASIS(6)=BASIS(7)*BASIS(2)-BASIS(1)*BASIS(8)
        RBASIS(7)=BASIS(2)*BASIS(6)-BASIS(5)*BASIS(3)
        RBASIS(8)=BASIS(3)*BASIS(4)-BASIS(6)*BASIS(1)
        RBASIS(9)=BASIS(1)*BASIS(5)-BASIS(4)*BASIS(2)
        X=BASIS(1)*RBASIS(1)+BASIS(4)*RBASIS(4)+BASIS(7)*RBASIS(7)
        DO J1=1,9
          RBASIS(J1)=RBASIS(J1)/X
        ENDDO
C*****CLEAR ARRAY ALMN WHICH WILL BE USED FOR STORAGE OF SPHERICAL
C*****HARMONIC COEFFICIENTS ALMN.
        DO J1=1,LGTH2
          ALMN(J1)=0D0
        ENDDO
C
        NREFL=0
        NREJMX=0
C*****READ ALL REFLECTION INTENSITIES AND CALCULATE SPHERICAL
C*****HARMONIC COEFFICIENTS ALMN.
C
100     CALL LRREFF(IXTL,S,ADATA,EOF)
        IF (EOF) GOTO 400
        CALL LRREFM(IXTL,LOGMSS)
        DO I=1,5
          IF (LOGMSS(I)) GOTO 100
        ENDDO
C
C*****CHECK IF REFLECTION IS WITHIN SPECIFIED RESOLUTION RANGE
        IF (S.LT.SMIN .OR. S.GT.SMAX) GOTO 100
        IF (ADATA(5).EQ.0.) GOTO 100
C***** Apply temperature factor before squaring F
        FR=ADATA(4)
        IF (TFAC2.NE.0.) FR=FR*EXP(-TFAC2*S)
C
C*****CHECK IF AMPLITUDE ABOVE THRESHOLD
        IF (FR.LE.FRMIN) GOTO 100
        IF (FR.GT.FRMAX) THEN
          NREJMX=NREJMX+1
          GOTO 100
        ENDIF
        FR=1E-4*FR**2
C
        NREFL=NREFL+1
C Count in resolution ranges
        NR=INT(S/RESINT)+1
        NR=MIN(NR,NSRANG)
        NRRANG(NR)=NRRANG(NR)+1
C
C***** ATTACH SIGMA-FACTOR TO INTENSITY
        R=SQRT(S)
        X=TWOPI*R*EPS
        IF (X.GT..01) FR=FR*3.*(SIN(X)-X*COS(X))/X**3
C
C***** CALCULATE BESSEL FUNCTIONS BY BACKWARDS RECURSION
        A2PIR=A2PI*R
        CALL JVX(LMAX+.5D0,DBLE(A2PIR),VM,BJ)
        IF (INT(VM).NE.LMAX) CALL CCPERR(1,'VM != LMAX in JVX.')
        T0=SQRT(.5D0*PI/A2PIR)
        DO L=1,LMAX
          SPHBES(L)=T0*BJ(L)
        ENDDO
C
C*****GENERATE ALL SYMMETRY RELATED REFLECTIONS
        J0=0
        NRFL=1
        IH(1)=NINT(ADATA(1))
        IK(1)=NINT(ADATA(2))
        IL(1)=NINT(ADATA(3))
        DO ISYM=1,NEQ
          IH0=IH(1)*MEQ(J0+1)+IK(1)*MEQ(J0+2)+IL(1)*MEQ(J0+3)
          IK0=IH(1)*MEQ(J0+4)+IK(1)*MEQ(J0+5)+IL(1)*MEQ(J0+6)
          IL0=IH(1)*MEQ(J0+7)+IK(1)*MEQ(J0+8)+IL(1)*MEQ(J0+9)
          DO J2=1,NRFL
            IF (IH0.NE.IH(J2)) GOTO 230
            IF (IK0.NE.IK(J2)) GOTO 230
            IF (IL0.NE.IL(J2)) GOTO 230
            GOTO 250
230         IF (-IH0.NE.IH(J2)) GOTO 240
            IF (-IK0.NE.IK(J2)) GOTO 240
            IF (-IL0.NE.IL(J2)) GOTO 240
            GOTO 250
240         CONTINUE
          ENDDO
          NRFL=NRFL+1
          IH(NRFL)=IH0
          IK(NRFL)=IK0
          IL(NRFL)=IL0
250       J0=J0+9
        ENDDO
C
        DO J0=1,LMAX*LMAX1
          ARRAY(J0)=0.
        ENDDO
C
        DO J0=1,NRFL
          X=IH(J0)*RBASIS(1)+IK(J0)*RBASIS(4)+IL(J0)*RBASIS(7)
          Y=IH(J0)*RBASIS(2)+IK(J0)*RBASIS(5)+IL(J0)*RBASIS(8)
          Z=IH(J0)*RBASIS(3)+IK(J0)*RBASIS(6)+IL(J0)*RBASIS(9)
          COSTH=Z/R
          SINTH=SQRT(MAX(1D0-COSTH**2,0D0))
C
          IF (SINTH.LT..01) THEN
            J1=1
            DO LBY2=1,LBY2MX
              MM=2*LBY2+1
              ARRAY(J1)=ARRAY(J1)+SQRT(REAL(2*MM-1))
              J1=J1+MM
            ENDDO
C
          ELSE
            COTTH=COSTH/SINTH
            PHI=ATAN2(Y,X)
            DO M=1,LMAX1
              CMPHI(M)=COS((M-1)*PHI)
              SMPHI(M)=SIN((M-1)*PHI)
            ENDDO
C
            J1=0
            DO LBY2=1,LBY2MX
              MM=2*LBY2+1
              M=MM-1
              T1=FACTOR(LBY2)*(SINTH**M)
              T2=0.
              PLM(MM)=T1
C
290           T0=T1*COTTH*FAC1(LBY2,M)+T2*FAC2(LBY2,M)
              PLM(M)=T0
              T2=T1
              T1=T0
              M=M-1
              IF (M.GT.0) GOTO 290
C
              DO M=1,MM
                J1=J1+1
                ARRAY(J1)=ARRAY(J1)+PLM(M)*CMPHI(M)
                ARRAY(J1+LDL1N)=ARRAY(J1+LDL1N)-PLM(M)*SMPHI(M)
              ENDDO
            ENDDO
          ENDIF
        ENDDO
C
C
        J0=0
        J1=0
        T1=FR
        DO LBY2=1,LBY2MX
          MM=2*LBY2+1
          NN=NMAX(LBY2)
          T1=-T1
          DO N=1,NN
            Z=BZ(N,LBY2)
            X=A2PIR-Z
            Y=A2PIR+Z
            IF (ABS(X).LE..1) THEN
              Z=Z*A2PIR*SPHBES(MM-2)/(Y*(A2PIR+MM*X))
            ELSE
              Z=Z*SPHBES(MM-1)/(X*Y)
            ENDIF
            RLN(N)=T1*Z
          ENDDO
          DO M=1,MM
            J1=J1+1
            DO N=1,NN
              J0=J0+1
              ALMN(J0)=ALMN(J0)+ARRAY(J1)*RLN(N)
              ALMN(J0+LGTH)=ALMN(J0+LGTH)+ARRAY(J1+LDL1N)*RLN(N)
            ENDDO
          ENDDO
        ENDDO
        GOTO 100
C*****END OF MAIN LOOP************************************************
C
400     WRITE(6,5001) IXTL,NREFL,NREJMX,(I,NRRANG(I),I=1,NSRANG)
5001    FORMAT(/1X,' Crystal ',I3,8X,I8,' reflections read',
     .  6X,I8,' reflections too large'///
     .  '  Numbers of reflections in ranges of sin**2 theta / lambda**2'
     .  //30(1X,I6,5X,I6/))
C
C*****OUTPUT SPHERICAL HARMONIC COEFFICIENTS ALMN
        T0=0.
        DO L=1,LGTH2
          T0=MAX(T0,ABS(ALMN(L)))
        ENDDO
        T0=1D-6*T0
C
        J0=0
        DO LBY2=1,LBY2MX
          L=2*LBY2
          MM=L+1
          NN=NMAX(LBY2)
          DO M1=1,MM
            M=M1-1
            DO N=1,NN
              J0=J0+1
              T1=ALMN(J0)
              T2=ALMN(J0+LGTH)
              IF (ABS(T1).LT.T0) T1=0D0
              IF (ABS(T2).LT.T0) T2=0D0
              IF (LIST.GT.0) WRITE(6,1050) L,M,N,T1,T2
1050          FORMAT(3I4,1P,2E12.4)
              IF (T1.NE.0D0 .OR. T2.NE.0D0) WRITE(TAPOT) L,M,N,T1,T2
            ENDDO
          ENDDO
        ENDDO
        TAPOT=TAPOT+1
C
C-----------------------------------------------------------------------
C End of crystal loop
      ENDDO
      END
C
C
C
C
C
C
      SUBROUTINE FRFSUM
C     =================
C
C***********************************************************************
C***********************************************************************
C***********************************************************************
C
C           ND500-FRFSUM                  W.KABSCH  10-1982
C
C***********************************************************************
C***********************************************************************
C***********************************************************************
C***** FAST ROTATION FUNCTION SUMMATION PROGRAM
C       COMPARES THE ROTATED PATTERSON DENSITY OF THE SECOND CRYSTAL
C       WITH THE UNROTATED PATTERSON DENSITY OF THE FIRST CRYSTAL. THE
C       ROTATION-ANGLE IS KAPPA ABOUT A ROTATION AXIS U=(UX,UY,UZ),
C       WITH POLAR COORDINATES PHI,OMEGA, SUCH THAT
C       UX=SIN(omega)*COS(phi)
C       UY=SIN(omega)*SIN(phi)
C       UZ=COS(omega).
C       THE PRINTED VALUES OF THE OVERLAP-INTEGRALS APPEAR IN SECTIONS
C       AT CONSTANT KAPPA, EACH LINE AT A CONSTANT VALUE OF PHI.
C       OMEGA INCREASES FROM LEFT TO RIGHT, PHI INCREASES DOWNWARDS.
C
C************************* FILES **************************************
C
C
C 2.)CONTROL PARAMETER CARD                  FORMAT(10I5,E15.5)
C    PHI1                    STARTING VALUE OF PHI (DEGREES)
C    PHI2                    MAXIMUM VALUE OF PHI (DEGREES)
C    DPHI                    STEP SIZE IN PHI
C    OMEG1                   STARTING VALUE OF OMEG
C    OMEG2                   MAXIMUM VALUE OF OMEG
C    DOMEG                   STEP SIZE IN OMEG
C    KAPA1                   STARTING VALUE OF KAPA
C    KAPA2                   MAXIMUM VALUE OF KAPA
C    DKAPA                   STEP SIZE IN KAPA
C    LIST                    0 : NO LISTING,
C                            1 : LIST EXPANSION COEFFICIENTS ALMN,BLMN.
C    RMAX                    MAXIMUM VALUE OF OVERLAP INTEGRAL.
C                            RMAX MUST BE ZERO , OTHERWISE IT IS
C                            ASSUMED THAT A PREVIOUSLY COMPUTED
C                            MAP NEEDS TO BE PRINTED AGAIN USING THE
C                            KNOWN VALUE OF RMAX.
C
C***********************************************************************
C
C
      REAL KAPA
      PARAMETER(MCONTR=50)
      COMMON /ALMFRF/ NCRYST,IPHI1,IPHI2,NPHI,DPHI,IOMEG1,IOMEG2,NOMEG,
     . DOMEG,IKAPA1,IKAPA2,NKAPA,DKAPA,ARAD,EPS,RESMIN,RESMAX,SMIN,SMAX,
     . NCODES(2),LSPGPS(2),TFACS(2),FRMINS(2),FRMAXS(2),LPRINT,LMAP,
     . LPLOT,CONTRS(MCONTR),NCONTR,LREAD,LSAVE,PEAK,MAXPEK,LPKRMS,RKAP1,
     . RKAP2,RMIN,RMEAN,RRMS,RWT,NPOINT,LDL
      CHARACTER*80 TITLE,FNAME
      COMMON /TEXTC/ TITLE,FNAME
C
      COMMON /RHMOUT/ WRMEAN, WRMS
      SAVE /RHMOUT/
      REAL WRMEAN, WRMS
C
C Maximum section size
      PARAMETER(NSMAP=181*361)
C
      PARAMETER(MDL=100, MDL2=2*MDL, MDN=MDL/2)
      REAL*8 S1,S2,S3,S4,F(4),FF(-MDL:MDL,MDN,4),CCR(-MDL:MDL),
     &CCI(-MDL:MDL)
      REAL*8 ANG,CB,DLCA,DLSA,DHI,DLO,DN,VS
      REAL*8 DLMM(0:MDL,-MDL:MDL),FACT(0:MDL2),PCOS(0:MDL2),PSIN(0:MDL2)
      DIMENSION FRQ(0:MDL2),FIQ(0:MDL2),R(361),RROT(NSMAP),
     & FRPQ(0:MDL,-MDL2:MDL2),FIPQ(0:MDL,-MDL2:MDL2)
C
      DIMENSION IBUF(600),RBUF(2200)
      REAL*8 DBUF(800)
      EQUIVALENCE(RBUF(1),IBUF(1)),(RBUF(601),DBUF(1))
C
C Map file stuff
      DIMENSION IUVW(3),MXYZ(3),DCELL(6)
C
      DATA ITAPE1,ITAPE2,ITAPE3,IOUT/8,9,64,10/
      PARAMETER(PI=3.141592654D0, TWOPI=2.*PI, DTR=PI/180.)
C
C
C
      LIST=0
      RMAX=0.
      RMIN=0.
      RMEAN=0.
      RRMS=0.
      EPS=1E-10
      LDN=LDL/2
C
C***********************************************************************
C
      IF (LSAVE.GE.0) THEN
        REWIND ITAPE1
        IF (NCRYST.EQ.2) REWIND ITAPE2
      ELSE
C  SUM option, read ALMN files from previous run
        L=LENSTR(FNAME)
        IFAIL=0
        CALL CCPDPN(ITAPE1,FNAME(:L)//'1','READONLY','U',1,IFAIL)
        IF (NCRYST.EQ.2)
     &  CALL CCPDPN(ITAPE2,FNAME(:L)//'2','READONLY','U',1,IFAIL)
      ENDIF
C
C If writing out map file, open it
C   Note on header for map file:
C   Since this map is not on a crystallographic cell, the 'cell' is set
C   to be 360,360,360,90,90,90 and the number of grid points in a unit
C   cell translation set to 360/step_size.
C
      IF (LMAP.GT.0) THEN
         DO I=1,3
           IUVW(I)=I
           DCELL(I)=360.
           DCELL(I+3)=90.
         ENDDO
         MXYZ(1)=NINT(360./DPHI)
         MXYZ(2)=NINT(360./DOMEG)
         MXYZ(3)=NINT(360./DKAPA)
         CALL MWRHDR(4,TITLE,NKAPA,IUVW,MXYZ,IKAPA1,IPHI1,IPHI2,IOMEG1,
     &   IOMEG2,DCELL,1,2)
      ENDIF
C
C
C
      NPOINT=NPHI*NOMEG*NKAPA
C
C          ******************************
      CALL QOPEN(IOUT,'ROTFUN','SCRATCH')
C          ******************************
C
      IF (IOUT.LE.0) CALL CCPERR(1,
     $ '*** POLARRFN: failed to open file ROTFUN ***')
C
C          ********************************
      CALL QMODE(IOUT,2,NCH)
      CALL QOPEN(ITAPE3,'ITAPE3','SCRATCH')
C          ********************************
C
      IF (ITAPE3.LE.0) CALL CCPERR(1,
     $ '*** POLARRFN: failed to open file ITAPE3 ***')
C
C          *******************
      CALL QMODE(ITAPE3,2,NCH)
C          *******************
C
C  Ejd Use IBSCR to count BLOCKS written out;
C          IPSCR to count within each BLOCK.
C
      IPSCR=0
      IBSCR=0
C
C---- form clm1m2 coefficients
C
      L0=0
      LEND=0
40    READ(ITAPE1,END=120) L,M,N,F(1),F(2)
      IF (NCRYST.EQ.1) THEN
        F(3)=F(1)
        F(4)=F(2)
        IF (LIST.GT.0) WRITE(6,1050) L,M,N,F(1),F(2)
1050    FORMAT(3I8,1P,4E12.4)
      ELSE
        READ(ITAPE2,END=120) L,M,N,F(3),F(4)
        IF (LIST.GT.0) WRITE(6,1050) L,M,N,F
      ENDIF
      IF (L.GT.L0) GOTO 70
      NMAX=MAX(NMAX,N)
C
50    DO J1=1,4
        FF(M,N,J1)=F(J1)
      ENDDO
C
      S1=1-2*MOD(M,2)
      FF(-M,N,3)=S1*F(3)
      FF(-M,N,4)=-S1*F(4)
      GOTO 40
C
70    IF (L0.EQ.0) GOTO 100
C
      DO M1=0,L0
        DO M2=-L0,L0
          CCR(M2)=0D0
          CCI(M2)=0D0
C
          DO N1=1,NMAX
            CCR(M2)=CCR(M2)+FF(M1,N1,1)*FF(M2,N1,3)+
     &      FF(M1,N1,2)*FF(M2,N1,4)
            CCI(M2)=CCI(M2)+FF(M1,N1,1)*FF(M2,N1,4)-
     &      FF(M1,N1,2)*FF(M2,N1,3)
          ENDDO
        ENDDO
C
        DO M2=-L0,L0
          IBUF(3*IPSCR+1) = L0
          IBUF(3*IPSCR+2) = M1
          IBUF(3*IPSCR+3) = M2
          DBUF(4*IPSCR+1) = CCR(M2)
          DBUF(4*IPSCR+2) = CCI(M2)
          DBUF(4*IPSCR+3) = CCR(-M2)
          DBUF(4*IPSCR+4) = CCI(-M2)
C
          IPSCR=IPSCR+1
          IF (IPSCR.EQ.200) THEN
            CALL QWRITE(ITAPE3,RBUF,2200)
            IPSCR=0
            IBSCR=IBSCR+1
          ENDIF
        ENDDO
      ENDDO
C
100   IF (LEND.GT.0) GOTO 130
      L0=L
      NMAX=N
      DO M1=-LDL,LDL
        DO N1=1,LDN
          DO J1=1,4
            FF(M1,N1,J1)=0D0
          ENDDO
        ENDDO
      ENDDO
      GOTO 50
C
120   LEND=1
      GOTO 70
C
130   IBUF(3*IPSCR+1)=-1
      CALL QWRITE(ITAPE3,RBUF,2200)
C
C----  Ejd  Keep count of number of records.
C
      NBSCR=IBSCR+1
      CLOSE(ITAPE1)
      IF (NCRYST.EQ.2) CLOSE(ITAPE2)
C
      LMAX=L0
      L2MAX=2*L0
C
C---- Calculate rotation function at given eulerian angles
C
      DKAPAR=DTR*DKAPA
      DPHIR=DTR*DPHI
C
      IB=0
      IOMEG=IOMEG1
      FACT(0) = 0D0
      DO I = 1,L2MAX
        FACT(I) = FACT(I-1)+LOG(DBLE(I))
      ENDDO
C
      WRITE(6,'(/A/2X,A,2(10X,A,4X,A,3X,A)/)')
     &'Printed norms should be exactly 1:','Omega','L','M','Norm (min)',
     &'L','M','Norm (max)'
C
C----  Ejd  - Now to unpack things..
C
170   IBSCR=-1
      CALL QSEEK(ITAPE3,1,1,1)
C
      OMEG=IOMEG*DOMEG
C
      IF (OMEG.GT.0. .AND. OMEG.LT.180.) THEN
        ANG = DTR*OMEG
        CB = COS(ANG)
        VS = -LOG(.5D0*SIN(ANG))
        KS = VS
        VS = MOD(VS,1D0)
        IF (VS.GT.0D0) THEN
          KS = KS+1
          VS = VS-1
        ENDIF
        VS = EXP(VS)
C
        ANG = .5D0*ANG
        DLCA = LOG(COS(ANG))
        DLSA = LOG(SIN(ANG))
        PCOS(0) = 0D0
        PSIN(0) = 0D0
        DO I = 1,L2MAX
          PCOS(I) = PCOS(I-1)+DLCA
          PSIN(I) = PSIN(I-1)+DLSA
        ENDDO
      ENDIF
C
      LLO = 0
      MLO = 0
      DLO = 1D0
      LHI = 0
      MHI = 0
      DHI = 1D0
C
      DO IP=0,LMAX
        DO IQ=-L2MAX,L2MAX
          FRPQ(IP,IQ)=0.
          FIPQ(IP,IQ)=0.
        ENDDO
      ENDDO
C
C---- sum over l and m2 to form fpq
C
      L0=-1
C
      DO IBSCR = 1,NBSCR
        CALL QREAD(ITAPE3,RBUF,2200,IEND)
        IF (IEND.NE.0) CALL CCPERR(1,'stop ITAPE3')
C
        DO IRC = 1,200
C
C      Unpack array IBUF
C
          L = IBUF(3*IRC-2)
          IF (L.LT.0) GOTO 400
          M1 = IBUF(3*IRC-1)
          M2 = IBUF(3*IRC)
          S1 = DBUF(4*IRC-3)
          S2 = DBUF(4*IRC-2)
          S3 = DBUF(4*IRC-1)
          S4 = DBUF(4*IRC)
          IF (L.GT.L0) THEN
            CALL DLMMS(OMEG,L,CB,KS,VS,FACT,PCOS,PSIN,DLMM)
            L0 = L
C
            DO MM1 = 0,L
              DN = 0D0
              DO MM2 = -L,L
                DN = DN+DLMM(MM1,MM2)**2
              ENDDO
C
              IF (DN.LT.DLO) THEN
                LLO = L
                MLO = MM1
                DLO = DN
              ELSEIF (DN.GT.DHI) THEN
                LHI = L
                MHI = MM1
                DHI = DN
              ENDIF
            ENDDO
          ENDIF
C
          Y=1-2*MOD(ABS(M1+M2),2)
          J1=M1-M2
          J2=-M1-M2
          DO M=0,L
            X=DLMM(M,M1)*DLMM(M,M2)
            FRPQ(M,J1)=FRPQ(M,J1)+S1*X
            FIPQ(M,J1)=FIPQ(M,J1)+S2*X
            IF (M1.GT.0) THEN
              X=Y*DLMM(M,-M1)*DLMM(M,M2)
              FRPQ(M,J2)=FRPQ(M,J2)+S3*X
              FIPQ(M,J2)=FIPQ(M,J2)-S4*X
            ENDIF
          ENDDO
        ENDDO
      ENDDO
C
C---- sum over p to form fq at given omega and kappa
C
400   IKAPA=IKAPA1
      WRITE(6,'(F7.2,2X,2(4X,2I5,1PE15.6))') OMEG,LLO,MLO,SQRT(DLO),LHI,
     &MHI,SQRT(DHI)
C
410   DO IQ=0,L2MAX
        X=FRPQ(0,IQ)
        Y=FIPQ(0,IQ)
        J1=IKAPA
        DO IP=1,LMAX
          S1=FRPQ(IP,IQ)
          S2=FRPQ(IP,-IQ)
          S3=FIPQ(IP,IQ)
          S4=FIPQ(IP,-IQ)
          KAPA=J1*DKAPAR
          CKAPA=COS(KAPA)
          SKAPA=SIN(KAPA)
          X=X+(S1+S2)*CKAPA+(S3+S4)*SKAPA
          Y=Y+(S3-S4)*CKAPA-(S1-S2)*SKAPA
          J1=J1+IKAPA
        ENDDO
        FRQ(IQ)=X
        FIQ(IQ)=Y
      ENDDO
C
C---- Final sum over q
C
      L=0
      IPHI=IPHI1
510   X=.5*FRQ(0)
      J1=IPHI
      DO IQ=1,L2MAX
        PHI=J1*DPHIR
        X=X+FRQ(IQ)*COS(PHI)+FIQ(IQ)*SIN(PHI)
        J1=J1+IPHI
      ENDDO
      IF (X.GT.RMAX) RMAX=X
      RMEAN = RMEAN+X
      RRMS = RRMS+X*X
      L=L+1
      R(L)=X
      IPHI=IPHI+1
      IF (IPHI.LE.IPHI2) GOTO 510
C
      CALL QWRITE(IOUT,R,NPHI)
C
      IB=IB+1
      IKAPA=IKAPA+1
      IF (IKAPA.LE.IKAPA2) GOTO 410
      IOMEG=IOMEG+1
      IF (IOMEG.LE.IOMEG2) GOTO 170
C
      WRITE(6,1010) RMAX
1010  FORMAT(/' Maximum R = ',E15.4/)
      RSCALE=100./RMAX
      RMEAN=RSCALE*RMEAN
      RRMS=RSCALE*SQRT(RRMS/NPOINT)
C
      RWT = 0.0
      WRMS = 0.0
      WRMEAN = 0.0
C
C
C---- Map output
C
      DO IKAPA=IKAPA1,IKAPA2
        KAPA = IKAPA*DKAPA
        WW = 1. - COS(DTR*KAPA)
        IF (NINT(KAPA).EQ.180) WW = .5*WW
        IB=IKAPA-IKAPA1+1
        L=0
        DO IOMEG=IOMEG1,IOMEG2
          OMEG = IOMEG*DOMEG
          WV = WW*SIN(DTR*OMEG)
          IF (NINT(OMEG).EQ.180) WV = .5*WV
          CALL QSEEK(IOUT,IB,1,NPHI)
          CALL QREAD(IOUT,R,NPHI,IEND)
          IF (IEND.NE.0) CALL CCPERR(1, 'stop IOUT')
          IB=IB+NKAPA
          IPHI=IPHI1
          DO J1=1,NPHI
            WU=WV
            IF (IPHI.EQ.0 .OR. NINT(IPHI*DPHI).EQ.180) WU=.5*WU
            AJ2=R(J1)*RSCALE
            WRMEAN = WRMEAN + WU*AJ2
            WRMS = WRMS + AJ2*AJ2*WU
            RWT = RWT + WU
            IF (AJ2.LT.-99.) AJ2=-99
            RMIN=MIN(RMIN,AJ2)
            RROT(J1+L)=AJ2
            IPHI=IPHI+1
          ENDDO
          L=L+NPHI
        ENDDO
        IF (LPRINT.GT.0) CALL PRNSEC(NINT(KAPA),RROT)
C
C Find peaks in section, plot etc - NB MAPSEC calls s/r PEAKPK
C which uses RRMS to select peaks
C
        CALL MAPSEC(NPHI,NOMEG,IKAPA,RROT)
C end of section output loop
      ENDDO
C
C
      WRMEAN = WRMEAN/RWT
      WRMS = WRMS/RWT - WRMEAN**2
      IF (WRMS.GE.0.0) THEN
         WRMS = SQRT(WRMS)
      ELSE
         WRMS = 0.0
      ENDIF
C Search last section for peaks
      CALL MAPSEC(NPHI,NOMEG,-1,RROT)
C Print out peaks
      CALL PPEAKS
C
C Print weighted rms and mean: these also go into map header
      WRITE(6,FMT='(A,F7.3/A,F7.3)') ' Weighted mean of map',WRMEAN,
     1                               ' Weighted rms',WRMS
C
      CALL QCLOSE(IOUT)
      CALL QCLOSE(ITAPE3)
      CALL CCPERR(0,' Normal Termination from POLARRFN')
      END
C
C Subroutine DLMMS calculates the matrix dlmm(omega) (the reduced
C matrix elements of the irreducible representation of the rotation
C group) for a given angular momentum L and a given angle OMEG.
C Various functions of the angle need to be supplied (MDW: this is
C a rough interpretation to help correlate with paper, but not so
C simple): 
C   CB: cos(omega)              -( used in recurrence relations
C   KS: -log(0.5*sin(omega))     (
C   VS: 0.5*sin(omega)           (
C   PCOS: log(cos(0.5*omega)**m)  -( used in analytical expression 
C   PSIN: log(sin(0.5*omega)**m)   ( of starting point
C   FACT: log(m!)                  (
C Output is in DLMM.
C This routine uses Jorge's improved recurrence relations, see
C     Jorge Navaza, Acta Cryst. A46, 619-620 (1990)
C
      SUBROUTINE DLMMS(OMEG,L,CB,KS,VS,FACT,PCOS,PSIN,DLMM)
      IMPLICIT NONE
      INTEGER K0,K1,KS,L,M1,M1S,M2,M2S,MDL,MDL2
      REAL OMEG,DTR,PI
      REAL*8 CB,E0,E1,V0,V1,VS,W0,W1
C
      PARAMETER(MDL=100, MDL2=2*MDL, PI=3.141592654D0, DTR=PI/180.)
C
      INTEGER IEXP(0:MDL,-MDL:MDL)
      REAL*8 DLMM(0:MDL,-MDL:MDL),FACT(0:MDL2),PCOS(0:MDL2),PSIN(0:MDL2)
C
      DATA E0/.2718281828459045D1/, E1/.3678794411714423D0/
C
      DO M1 = 0,L
        DO M2 = -L,L
          IEXP(M1,M2) = 0
          DLMM(M1,M2) = 0D0
        ENDDO
      ENDDO
C
      IF (OMEG.EQ.0.) THEN
        DO M1 = 0,L
          DLMM(M1,M1) = 1D0
        ENDDO
C
      ELSEIF (OMEG.EQ.180.) THEN
        M1S = 1
        DO M1 = 0,L
          DLMM(M1,-M1) = M1S
          M1S = -M1S
        ENDDO
C
      ELSE
        DO M2 = -L,L
          W0 = (FACT(2*L)-FACT(L+M2)-FACT(L-M2))/2+PCOS(L+M2)+PSIN(L-M2)
          K0 = W0
          W0 = MOD(W0,1D0)
C
          IF (W0.GT.0D0) THEN
            K0 = K0+1
            W0 = W0-1
          ENDIF
C
          IEXP(L,M2) = K0
          DLMM(L,M2) = EXP(W0)
        ENDDO
C
        W0 = VS/SQRT(DBLE(2*L))
C
        DO M2 = -L+1,L-1
          K0 = IEXP(L,M2)+KS
          V1 = (L*CB-M2)*W0*DLMM(L,M2)
          V0 = ABS(V1)
C
          IF (V0.GT.1D0) THEN
1           K0 = K0+1
            V0 = E1*V0
            IF (V0.GT.1D0) GOTO 1
          ENDIF
C
          IF (V0.LT.E1) THEN
2           K0 = K0-1
            V0 = E0*V0
            IF (V0.LT.E1) GOTO 2
          ENDIF
C
          IEXP(L-1,M2) = K0
          DLMM(L-1,M2) = SIGN(V0,V1)
        ENDDO
C
        DO M1 = L-1,1,-1
          W0 = VS/SQRT(DBLE((L-M1+1)*(L+M1)))
          W1 = SQRT(DBLE((L+M1+1)*(L-M1))/DBLE((L-M1+1)*(L+M1)))
C
          DO M2 = -M1+1,M1-1
            K1 = IEXP(M1+1,M2)
            V1 = DLMM(M1+1,M2)
            K0 = IEXP(M1,M2)+KS
            V0 = DLMM(M1,M2)
C
            IF (K0.GT.K1) THEN
              V1 = EXP(DBLE(K1-K0))*V1
            ELSEIF (K1.GT.K0) THEN
              V0 = EXP(DBLE(K0-K1))*V0
              K0 = K1
            ENDIF
C
            V1 = (M1*CB-M2)*W0*V0-W1*V1
            V0 = ABS(V1)
C
            IF (V0.GT.1D0) THEN
3             K0 = K0+1
              V0 = E1*V0
              IF (V0.GT.1D0) GOTO 3
            ENDIF
C
            IF (V0.LT.E1) THEN
4             K0 = K0-1
              V0 = E0*V0
              IF (V0.LT.E1) GOTO 4
            ENDIF
C
            IEXP(M1-1,M2) = K0
            DLMM(M1-1,M2) = SIGN(V0,V1)
          ENDDO
        ENDDO
      ENDIF
C
      M1S = -1
      DO M1 = 1,L
        M2S = M1S
        M1S = -M1S
        DO M2 = 0,M1-1
          IEXP(M2,M1) = IEXP(M1,M2)
          DLMM(M2,M1) = M2S*DLMM(M1,M2)
          IEXP(M2,-M1) = IEXP(M1,-M2)
          DLMM(M2,-M1) = DLMM(M1,-M2)
          M2S = -M2S
        ENDDO
      ENDDO
C
      DO M1 = 0,L
        DO M2 = -L,L
          DLMM(M1,M2) = EXP(DBLE(IEXP(M1,M2)))*DLMM(M1,M2)
        ENDDO
      ENDDO
      END
C
C
C
      SUBROUTINE PRNSEC(JKAPA,RROT)
C     =============================
C
C Print out section
C
      DIMENSION RROT(*)
C
      PARAMETER(MCONTR=50)
      COMMON /ALMFRF/ NCRYST,IPHI1,IPHI2,NPHI,DPHI,IOMEG1,IOMEG2,NOMEG,
     . DOMEG,IKAPA1,IKAPA2,NKAPA,DKAPA,ARAD,EPS,RESMIN,RESMAX,SMIN,SMAX,
     . NCODES(2),LSPGPS(2),TFACS(2),FRMINS(2),FRMAXS(2),LPRINT,LMAP,
     . LPLOT,CONTRS(MCONTR),NCONTR,LREAD,LSAVE,PEAK,MAXPEK,LPKRMS,RKAP1,
     . RKAP2,RMIN,RMEAN,RRMS,RWT,NPOINT,LDL
C
      INTEGER OMEGMK(100),PHIMK(100)
      CHARACTER*3 IT,LPRNT(361)
C
      SAVE IFIRST,OMEGMK,PHIMK
      DATA IFIRST/0/
C
C
      IF (IFIRST.EQ.0) THEN
C--Setting arrays PHIMK() and OMEGMK() for marker lines.
        DO IMK=1,NOMEG
          OMEGMK(IMK)=DOMEG*(IOMEG1+IMK-1)
        ENDDO
        DO IMK=1,NPHI
          PHIMK(IMK)=DPHI*(IPHI1+IMK-1)
        ENDDO
        IFIRST=1
      ENDIF
C
C  Write out section
C
      L=NPHI*(NOMEG-1)
      SMAX=-1000.
C
      WRITE(6,1020)JKAPA
1020  FORMAT(//'Rotation angle kappa =',I3/
     & 'Polar coordinates of rotation axes are phi (vertical) and ',
     & 'omega (horizontal)'/2X,'P'/2X,'h'/2X,'i',4X,'O m e g a'/)
      WRITE(6,1030) (OMEGMK(IMK),IMK=1,NOMEG)
1030  FORMAT(4X,40I3)
      WRITE(6,1031) ('  +',IMK=1,NOMEG)
1031  FORMAT(4X,40A)
      DO J1=1,NPHI
        KPR=0
        DO J2=0,L,NPHI
          R=RROT(J1+J2)
          IF (R.GT.SMAX) THEN
            SMAX=R
            J1M=J1
            J2M=J2
          ENDIF
          N9=NINT(R)
          IF (N9.LT.0) THEN
            IT='  -'
          ELSEIF(N9.LT.LPRINT) THEN
            IT='  .'
          ELSE
            WRITE(IT,'(I3)') N9
          ENDIF
          KPR=KPR+1
          LPRNT(KPR)=IT
        ENDDO
        WRITE(6,1057) PHIMK(J1),(LPRNT(J2),J2=1,KPR)
1057    FORMAT(I3,'+',40A:/(4X,40A))
      ENDDO
      WRITE(6,1031) ('  +',IMK=1,NOMEG)
      WRITE(6,1030) (OMEGMK(IMK),IMK=1,NOMEG)
C
      WRITE(6,1035) SMAX,NINT((IOMEG1+J2M/NPHI)*DOMEG),
     &NINT((IPHI1+J1M-1)*DPHI)
1035  FORMAT(/'Maximum on this section = ',F8.1,'   at omega =',I4,
     . '  phi =',I4)
      END
C
C
      SUBROUTINE MAPRED
C     =================
C
C Read back file of rotation function for plotting etc
C Use sections KRKAP1 to KRKAP2
C
      PARAMETER(MCONTR=50)
      COMMON /ALMFRF/ NCRYST,IPHI1,IPHI2,NPHI,DPHI,IOMEG1,IOMEG2,NOMEG,
     . DOMEG,IKAPA1,IKAPA2,NKAPA,DKAPA,ARAD,EPS,RESMIN,RESMAX,SMIN,SMAX,
     . NCODES(2),LSPGPS(2),TFACS(2),FRMINS(2),FRMAXS(2),LPRINT,LMAP,
     . LPLOT,CONTRS(MCONTR),NCONTR,LREAD,LSAVE,PEAK,MAXPEK,LPKRMS,RKAP1,
     . RKAP2,RMIN,RMEAN,RRMS,RWT,NPOINT,LDL
      CHARACTER*80 TITLE,FNAME,QTITLE
      COMMON /TEXTC/ TITLE,FNAME
C
C Maximum section size
      PARAMETER(NSMAP=181*361)
C
      DIMENSION IUVW(3),MXYZ(3),DCELL(6),RROT(NSMAP)
C
C Open map file
      CALL MRDHDR(3,'MAPIN',QTITLE,NSEC,IUVW,MXYZ,IKAPA1,
     .  IPHI1,IPHI2,IOMEG1,IOMEG2,DCELL,LS,LMODE,RMIN,RMAX,RMEAN,RRMS)
C
C Keep input title if specified
      IF (TITLE.EQ.' ') TITLE=QTITLE
C Set parameters from file. See FRFSUM for comments on what is in map
C header
      DPHI=360./MXYZ(1)
      DOMEG=360./MXYZ(2)
      DKAPA=360./MXYZ(3)
      NPHI=IPHI2-IPHI1+1
      NOMEG=IOMEG2-IOMEG1+1
C Check for maximum section size
      NSECSZ=NPHI*NOMEG
      IF (NSECSZ.GT.NSMAP) THEN
        WRITE(6,9050) NSECSZ,NSMAP
9050    FORMAT(//' Section size too large, =',I8,'    maximum=',I8///)
        CALL CCPERR(1,' error on section 9050')
      ENDIF
C
C Read sections KRkap1 to KRkap2
C
C   Corrected next line 3/3/87 PRE
      KRKAP1=NINT(RKAP1/DKAPA)
      CALL MPOSN(3,KRKAP1)
      DO IKAPA=KRKAP1,NINT(RKAP2/DKAPA)
        CALL MGULP(3,RROT,IER)
        IF (IER.NE.0) GOTO 11
C Print section
        JKAPA=NINT(IKAPA*DKAPA)
        IF (LPRINT.GT.0) CALL PRNSEC(JKAPA,RROT)
C Plot and find peaks as required
        CALL MAPSEC(NPHI,NOMEG,IKAPA,RROT)
      ENDDO
C
11    CALL MAPSEC(NPHI,NOMEG,-1,RROT)
      CALL PPEAKS
      END
C
C
      SUBROUTINE MAPSEC(LA,LB,IKAPA,RROT)
C     ===================================
C
C  Deal with section kappa = IKAPA in RROT.
C
C   1. write out map section to map file if LMAP .gt. 0
C
C   2. plot if LPLOT .ne. 0
C
C   3. search for peaks in PREVIOUS section: if IKAPA=-1 search last
C      section
CEJD LA - number of points along Phi
CEJD LB - number of points along Omega - iomeg1,iomeg2
CEJD IKAPA = Kappa section.
C
C
      DIMENSION RROT(LA,LB)
C
      PARAMETER(MCONTR=50)
      COMMON /ALMFRF/ NCRYST,IPHI1,IPHI2,NPHI,DPHI,IOMEG1,IOMEG2,NOMEG,
     . DOMEG,IKAPA1,IKAPA2,NKAPA,DKAPA,ARAD,EPS,RESMIN,RESMAX,SMIN,SMAX,
     . NCODES(2),LSPGPS(2),TFACS(2),FRMINS(2),FRMAXS(2),LPRINT,LMAP,
     . LPLOT,CONTRS(MCONTR),NCONTR,LREAD,LSAVE,PEAK,MAXPEK,LPKRMS,RKAP1,
     . RKAP2,RMIN,RMEAN,RRMS,RWT,NPOINT,LDL
C
      COMMON /RHMOUT/ WRMEAN, WRMS
      SAVE /RHMOUT/, RMAP
      REAL WRMEAN, WRMS
C
C Maximum section size
      PARAMETER(NSMAP=181*361)
C
      DIMENSION PLTMAP(NSMAP),RMAP(3*NSMAP)
C
C
      IF (IKAPA.LT.0) THEN
C Close map file if required
         IF (LMAP.GT.0) CALL MCLOSC(4,RMIN,100.,WRMEAN,WRMS)
C Close plot file if required
         IF (LPLOT.NE.0) CALL PLTSTP
         GOTO 10
      ENDIF
C
C Write map section to map file
      IF (LMAP.GT.0) CALL MSPEW(4,RROT)
C
      IF (LPLOT.NE.0) THEN
C  Plot section
        NLA=NINT(360./DPHI)+1
        NLB=NINT(90./DOMEG)+1
        IF (NLA*NLB.GT.NSMAP) CALL CCPERR(1,'Increase PARAMETER NSMAP')
        CALL PLOTMP(RROT,LA,LB,NINT(IKAPA*DKAPA),PLTMAP,NLA,NLB)
      ENDIF
C
C Pick peaks on PREVIOUS section
10    IF (LA*LB.GT.NSMAP) CALL CCPERR(1,'Increase PARAMETER NSMAP')
      CALL PEAKPK(RROT,LA,LB,IKAPA,RMAP)
      END
C
C
C
      SUBROUTINE PLOTMP(RROT,LA,LB,IG,PLTMAP,NLA,NLB)
C     ===============================================
C
C Plot section as stereogram
C
C First generate array PLTMAP, using symmetry as necessary,
C then plot (this part of program taken from RFPLOT)
C  If LPLOT .lt. 0, don't plot any labels
C
C In the absence of symmetry, each kappa section is plotted as
C two hemispheres omega = 0 to 90, and 90 to 180. Subroutine
C CHKSYM works out the symmeytry of the rotation function, and sets
C flag LSYMFG to indicate whether one or two hemispheres need to
C be plotted. LSYMFG = 2  no symmetry, plot both hemispheres
C                    = 1  symmetry sufficient to reduce asymmetric unit
C                         to one hemisphere
C                    =-2  asymmetric unit omega = 0 to 180, phi = 0 to
C                         180: plot two quarter spheres in same circle
C
C
      DIMENSION RROT(LA,LB),PLTMAP(NLB,NLA)
C
      COMMON /MATRIX/ ORTHOG(3,3,2),FRAC(3,3,2),SYM(4,4,192,2),NSYM(2),
     . MRSYM,JRSYM(3,3,12),LSYMFG,LHALF
C
      PARAMETER(MCONTR=50)
      COMMON /ALMFRF/ NCRYST,IPHI1,IPHI2,NPHI,DPHI,IOMEG1,IOMEG2,NOMEG,
     . DOMEG,IKAPA1,IKAPA2,NKAPA,DKAPA,ARAD,EPS,RESMIN,RESMAX,SMIN,SMAX,
     . NCODES(2),LSPGPS(2),TFACS(2),FRMINS(2),FRMAXS(2),LPRINT,LMAP,
     . LPLOT,CONTRS(MCONTR),NCONTR,LREAD,LSAVE,PEAK,MAXPEK,LPKRMS,RKAP1,
     . RKAP2,RMIN,RMEAN,RRMS,RWT,NPOINT,LDL
      CHARACTER*80 TITLE,FNAME
      COMMON /TEXTC/ TITLE,FNAME
C
      PARAMETER(IPTMAX=200)
      DIMENSION XXX(IPTMAX),YYY(IPTMAX)
C Maximum section size
      PARAMETER(NSMAP=181*361)
      PARAMETER(PI=3.141592654D0, TWOPI=2.*PI, DTR=PI/180.)
C
      LOGICAL BITS(2*NSMAP)
C
      CHARACTER*80 LINE
      SAVE IFIRST
C
C Set up plotting viewport and window
C
C  Relative character sizes for tick marks and titles
      REAL CSZ1, CSZ2
      REAL VWPORT(2,2), WINDOW(2,2)
C    WINDOW  are the corresponding coordinates in user units, which
C           set radius of stereogram = 1
C                   x1    y1    x2   y2
      DATA WINDOW/ -1.3, -1.3, 1.3, 1.3/
C
      DATA CSZ1, CSZ2/0.8,1.1/
C
C    VWPORT  are corners of plotting space in range 0-1 on x & y
C                   x1    y1    x2    y2
      DATA VWPORT/ 0.05, 0.05, 0.95, 0.95/
      DATA IFIRST/0/
C
      CHSZ = 0
C
      IF (IOMEG1.LT.0 .OR. NINT(IOMEG2*DOMEG).GT.180) THEN
        WRITE(6,1001) NINT(IOMEG1*DOMEG),NINT(IOMEG2*DOMEG)
1001    FORMAT(/' Plotting routine cannot cope with omega outside',
     .  ' range 0 to 180, omega limits =',2I5/)
        RETURN
      ENDIF
C
C On first entry, start plotting
C
      IF (IFIRST.EQ.0) THEN
        CALL PLTINI
C Set plotting transform
        CALL PLTWIN(VWPORT, WINDOW)
C Find rotation function symmetry from crystal symmetries
        CALL CHKSYM
      ENDIF
C
C
C If necessary, loop for two halves of kappa section
C   LSYMFG = 1,2 or -2 see above
C Kappa = 180 has symmetry type 1 (180-omega, 180+phi)
      NHALF=ABS(LSYMFG)
      IF (IG.EQ.180) THEN
        NHALF=1
C Add symmetry type 1 if not already present
        IF (NCRYST.EQ.2) THEN
          MRSYM=MRSYM+1
          CALL SETSYM(JRSYM(1,1,MRSYM),1)
        ENDIF
      ENDIF
C
      DO LHALF=1,NHALF
C
C Copy appropriate part of RROT to PLTMAP using symmetry as necessary
C First define limits required on omega(omeg) and phi(phi) in PLTMAP
        IF (LHALF.EQ.1) THEN
          IB1=0
          IB2=90
        ELSE
          IB1=90
          IB2=180
        ENDIF
        IA1=0
        IF (LSYMFG.LT.0 .AND. IG.NE.180) THEN
C LSYMFG = -2, plot 2 half hemispheres in same circle
          MLA=(NLA+1)/2
          IA2=180
          LSPLIT=1
        ELSE
          MLA=NLA
          IA2=360
          LSPLIT=0
        ENDIF
C
        IB1=NINT(IB1/DOMEG)
        IB2=NINT(IB2/DOMEG)
        IA2=NINT(IA2/DPHI)
C
Cejd        CALL ZEROI(PLTMAP,MLA*NLB)
        DO JPLT = 1,NLB
          DO IPLT = 1,MLA
            PLTMAP(JPLT,IPLT) = 0.
          ENDDO
        ENDDO
C
C Loop points in output array PLTMAP
        KB=1
        DO IB=IB1,IB2
          IOMEG=NINT(IB*DOMEG)
          KA=1
          DO IA=IA1,IA2
C Put point IA,IB into calculated area if possible: if not JA=0
            CALL SYMPUT(JA,JB,NINT(IA*DPHI),IOMEG)
            IF (JA.GT.0) PLTMAP(KB,KA)=RROT(JA,JB)
            KA=KA+1
          ENDDO
          KB=KB+1
        ENDDO
C
C Start new picture except for LSYMFG=-2
        IF (LHALF.EQ.1 .OR. LSYMFG.GT.1) THEN
          IF (IFIRST.NE.0) CALL PLTPIC
          IFIRST=1
C
C
C   Now draw the ring around the edge, using a do loop of drawto's
C
          CALL PLTMVU(1.,0.)
          DO K=1,101
            FI=3.6*DTR*(K-1)
            CALL PLTDWU(COS(FI),SIN(FI))
          ENDDO
C
C  Plot tick marks
          TICK=.01
          CALL PLTMVU(1.,0.)
          CALL PLTDBU(TICK,0.)
          IF (LPLOT.GT.0) THEN
            CALL PLTTNF(1.+2.*TICK,0.,X,Y)
            CALL PLTCTX('0.0',X,Y,CSZ1,1,0.0)
          ENDIF
C
          CALL PLTMVU(0.,1.)
          CALL PLTDBU(0.,TICK)
          IF (LPLOT.GT.0) THEN
            CALL PLTTNF(0.,1.+4.*TICK,X,Y)
            IF (LSPLIT.GT.0) THEN
              CALL PLTCTX('90.0, omega 0 to 90',X,Y,CSZ1,1,0.)
            ELSE
              CALL PLTCTX('90.0',X,Y,CSZ1,2,0.)
            ENDIF
          ENDIF
C
          CALL PLTMVU(-1.,0.)
          CALL PLTDBU(-TICK,0.)
          IF (LPLOT.GT.0) THEN
            CALL PLTTNF(-1.-2.*TICK,0.,X,Y)
            CALL PLTCTX('180.0',X,Y,CSZ1,3,0.)
          ENDIF
C
          CALL PLTMVU(0.,-1.)
          CALL PLTDBU(0.,-TICK)
          IF (LPLOT.GT.0) THEN
            CALL PLTTNF(0.,-1.-4.*TICK-CHSZ,X,Y)
            IF (LSPLIT.GT.0) THEN
              CALL PLTCTX('90.0, omega 90 to 180',X,Y,CSZ1,1,0.)
            ELSE
              CALL PLTCTX('-90.0',X,Y,CSZ1,2,0.)
            ENDIF
          ENDIF
C
C Draw line across section
          IF (LSPLIT.GT.0) THEN
            CALL PLTMVU(1.,0.)
            CALL PLTDWU(-1.,0.)
          ENDIF
C  Plot title centred on 0,-1.1
          IF (LPLOT.GT.0) THEN
            CALL PLTTNF(0.,-1.14,X,Y)
            CALL PLTCTX(TITLE,X,Y,CSZ2,2,0.)
            XX=0.
            IF (LSYMFG.EQ.2) XX=-.8
            CALL PLTTNF(XX,-1.24,X,Y)
            WRITE (LINE, 6001) IG
6001        FORMAT('Section kappa =  ',I4)
            L = LENSTR(LINE)
            IF (LSYMFG.EQ.2)
     &      LINE(L+1:) =', omega = 0 to 90, omega = 90 to 180'
            CALL PLTCTX(LINE(:L),X,Y,CSZ2,2,0.)
          ENDIF
        ENDIF
C
C Plot join lines (if any)
        CALL PLJOIN(IG,DPHI,DOMEG)
C
C Set solid lines for contour
        CALL PLTDSH(0,0.,0.,0.)
C
        CALL HPCNTR(PLTMAP,NLB,MLA,BITS,CONTRS,NCONTR,XXX,YYY,IPTMAX)
      ENDDO
C
C
C- - - - -- - - - - - - -    - - - - - - - - - - - - - - - -
C Remove special symmetry for kappa=180
      IF (NCRYST.EQ.2 .AND. IG.EQ.180) MRSYM=MRSYM-1
      END
C
C
      SUBROUTINE PLJOIN(IG,DPHI,DOMEG)
C     =================================
C
C Plot JOIN lines for section kappa=IG
C  DPHI  sampling interval in degrees along phi
C  DOMEG sampling interval in degrees along omega
C
C Lines are drawn between specified points along the great circle
C JOIN stuff
      PARAMETER(MXJOIN=30)
      COMMON /VJOIN/ NJOIN,PEAKJ(3,2,MXJOIN),DASH(2,MXJOIN)
      SAVE /VJOIN/
C PEAKJ(I,J,K) contains peaks
C  I=1,2,3  omega, phi, kappa
C  J=1,2    point 1, point 2  (kappa2=kappa1)
C
      DIMENSION P(3),Q(3),R(3),T(3,3),TINV(3,3),V1(3),V2(3)
C
C Line buffer
      PARAMETER(MAXLIN=200)
      DIMENSION X(MAXLIN),Y(MAXLIN)
      PARAMETER(PI=3.141592654D0, DTR=PI/180.)
C
      DO K=1,NJOIN
C Check all joins, is this one in current kappa section?
      IF (NINT(PEAKJ(3,1,K)).NE.IG) GOTO 1
C
C We have a peak pair K
C  Direction cosines of points 1 & 2
      CALL DRCCOS(V1,PEAKJ(1,1,K))
      CALL DRCCOS(V2,PEAKJ(1,2,K))
C Define coordinate frame pqr with great circle perpendicular to z
C Put p axis along x
      CALL VSET(P,V1)
C r axis orthogonal to V1 and V2
      CALL CROSS(R,V1,V2)
      CALL UNIT(R)
C q axis completes set
      CALL CROSS(Q,R,P)
      CALL UNIT(Q)
C Transformation matrix T takes point in pqr frame to xyz
      CALL VSET(T(1,1),P)
      CALL VSET(T(1,2),Q)
      CALL VSET(T(1,3),R)
C Invert (transpose) T
      CALL TRANSP(TINV,T)
C
C Theta is angle in great circle from p (V1) towards q (v2)
C  Point 1 (V1) is at theta=0
C  Get theta for point 2
      CALL MATVEC(V1,TINV,V2)
      THETA2=ATAN2(V1(2),V1(1))
C Stepsize in radians
      VSTEP=SIGN(0.02,THETA2)
      N=NINT(ABS(THETA2)/VSTEP)
      VSTEP=THETA2/N
      NSTEP=N+1
      IF (NSTEP.GT.MAXLIN) THEN
        WRITE(6,1001) MAXLIN
1001    FORMAT(' More than',I6,'lines in PLJOIN')
        NSTEP=MAXLIN
      ENDIF
C
C Loop to draw line between points
      DO N=1,NSTEP
      THETA=(N-1)*VSTEP
      P(1)=COS(THETA)
      P(2)=SIN(THETA)
      P(3)=0.0
      CALL MATVEC(V1,T,P)
C Store lines, x = omega, y= phi, stored as grid units to match what
C comes out of the contour routine
      X(N)=ACOS(V1(3))/(DTR*DOMEG)
      IF (V1(1).EQ.0 .AND. V1(1).EQ.0.) THEN
        Y(N)=0.
      ELSE
        Y(N)=ATAN2(V1(2),V1(1))/(DTR*DPHI)
      ENDIF
      ENDDO
C
C Set dash parameters
      IF (DASH(1,K).EQ.0.) THEN
C Solid
        M=0
      ELSE
C Dashed
        M=1
      ENDIF
      CALL PLTDSH(M,DASH(1,K),DASH(2,K),0.)
C Plot
      CALL LINES(X,Y,NSTEP)
C
1     CONTINUE
      ENDDO
      END
C
C
C
      SUBROUTINE DRCCOS(V,P)
C     ======================
C
C Set dircetion cosines V of axis along omega = P(1), phi = P(2)
C
      DIMENSION V(3),P(2)
C
      PARAMETER(PI=3.141592654D0, DTR=PI/180.)
C
      OMEGA=DTR*P(1)
      PHI=DTR*P(2)
C
      SOMEGA=SIN(OMEGA)
      V(1)=SOMEGA*COS(PHI)
      V(2)=SOMEGA*SIN(PHI)
      V(3)=COS(OMEGA)
      END
C
C
C
      SUBROUTINE CHKSYM
C     =================
C
C Check symmetry of the two crystals and set up symmetry of rotation
C function
C
C Sets MRSYM  number of symmetry operations
C      JRSYM  rotation function symmetry operators, matrices to multiply
C             (omega   phi  1) in degrees.
C
C Possible operations arise as follows:
C
C 1.  180-omega, 180+phi      self-rotation
C
C 2.  180-omega,  -phi        x dyads in both crystals
C
C 3.  180-omega, 180-phi      y dyads in both crystals
C
C 4.  omega,     180+phi      z dyads in both crystals
C
C Other operations may arise as combinations of these
C
      COMMON /MATRIX/ ORTHOG(3,3,2),FRAC(3,3,2),SYM(4,4,192,2),NSYM(2),
     . MRSYM,JRSYM(3,3,12),LSYMFG,LHALF
C
      PARAMETER(MCONTR=50)
      COMMON /ALMFRF/ NCRYST,IPHI1,IPHI2,NPHI,DPHI,IOMEG1,IOMEG2,NOMEG,
     . DOMEG,IKAPA1,IKAPA2,NKAPA,DKAPA,ARAD,EPS,RESMIN,RESMAX,SMIN,SMAX,
     . NCODES(2),LSPGPS(2),TFACS(2),FRMINS(2),FRMAXS(2),LPRINT,LMAP,
     . LPLOT,CONTRS(MCONTR),NCONTR,LREAD,LSAVE,PEAK,MAXPEK,LPKRMS,RKAP1,
     . RKAP2,RMIN,RMEAN,RRMS,RWT,NPOINT,LDL
C
      DIMENSION JRMAT(3,3),LDYAD(24,2),LPERM(4)
      DATA LPERM/2,3,1,2/
C
C Starting flags for no symmetry
      LSYMFG=2
      MRSYM=0
C
C Is this a self rotation function?
      IF (NCRYST.EQ.1) THEN
            MRSYM=1
            LSYMFG=1
            CALL SETSYM(JRSYM(1,1,1),1)
      ENDIF
C
C Find out which crystal symmetry operations are dyads
      DO K=1,NCRYST
      DO L=1,NSYM(K)
      LDYAD(L,K)=0
C A dyad matrix is diagonal with  Trace = -1
      TRACE=0.
      DO I=1,3
      TRACE=TRACE+SYM(I,I,L,K)
      ENDDO
      IF (ABS(TRACE+1.).GT.0.0001) GOTO 11
C  Trace=-1, check off diagonals
      M=0
      DO I=1,3
      IF (SYM(I,I,L,K).GT.0.) M=I
      DO J=1,3
      IF (I.NE.J) THEN
            IF (ABS(SYM(I,J,L,K)).GT.0.0001) GOTO 11
      ENDIF
      ENDDO
      ENDDO
C Yes we have a dyad along axis M
C  Allow for axis permutation from NCODE = 2 or 3
      IF (NCODES(K).EQ.2) THEN
            M=LPERM(M+1)
      ELSEIF(NCODES(K).EQ.3) THEN
            M=LPERM(M)
      ENDIF
      LDYAD(L,K)=M
C If self-rotation, dyads must be paralle in the two crystals
      IF (NCRYST.EQ.1) THEN
C  so add symmetry to list
            MRSYM=MRSYM+1
            CALL SETSYM(JRSYM(1,1,MRSYM),M+1)
      ENDIF
11    CONTINUE
      ENDDO
      ENDDO
C
C Now check for parallel dyads in the 2 crystals for cross-rotation
      IF (NCRYST.EQ.2) THEN
C  Loop 1st crystal operations
            DO L=1,NSYM(1)
            IF (LDYAD(L,1).GT.0) THEN
C   Check 2nd crystal
                  DO K=1,NSYM(2)
                  IF (LDYAD(K,2).EQ.LDYAD(L,1)) THEN
C    We have parallel dyads, so add symmetry
                        M=LDYAD(L,1)+1
                        MRSYM=MRSYM+1
                        CALL SETSYM(JRSYM(1,1,MRSYM),M)
C    Set asymmetric unit flag
                        IF (M.LE.3) THEN
                              LSYMFG=1
                        ELSEIF(LSYMFG.EQ.2) THEN
                              LSYMFG=-2
                        ENDIF
                  ENDIF
                ENDDO
            ENDIF
          ENDDO
      ENDIF
C
C
C We now have MRSYM symmetry operations: see if combinations generate
C any new ones
      IF (MRSYM.LE.1) RETURN
C Loop pairs of operations: any new ones generated need not be tested
      MRS=MRSYM
      DO K=1,MRS-1
      DO L=K+1,MRS
      CALL MATMLI(JRMAT,JRSYM(1,1,L),JRSYM(1,1,K))
C 'Translations' can have 360 added or subtracted to bring into range
C 0 to 180
      DO I=1,2
      J=JRMAT(I,3)
      IF (J.GT.359) J=J-360
      IF (J.LT.-179) J=J+360
      JRMAT(I,3)=J
C Check if we already have the same matrix
      ENDDO
      DO M=1,MRSYM
      DO I=1,3
      DO J=1,3
      IF (JRMAT(I,J).NE.JRSYM(I,J,M)) GOTO 121
C Same as this matrix M so ignore
      ENDDO
      ENDDO
      GOTO 110
121   CONTINUE
      ENDDO
C
C New one, so add to list
      MRSYM=MRSYM+1
      DO I=1,3
      DO J=1,3
      JRSYM(I,J,MRSYM)=JRMAT(I,J)
      ENDDO
      ENDDO
C
110   CONTINUE
      ENDDO
      ENDDO
      END
C
C
C
      SUBROUTINE SETSYM(JRSYM,M)
C     ==========================
C
C Set symmetry matrix JRSYM to symmetry type M
C
C 1.  180-omega, 180+phi      self-rotation
C
C 2.  180-omega,  -phi        x dyads in both crystals
C
C 3.  180-omega, 180-phi      y dyads in both crystals
C
C 4.  omega,     180+phi      z dyads in both crystals
C
      DIMENSION JRSYM(3,3)
C
      IF (M.LE.0) RETURN
C
      DO I=1,3
      DO J=1,3
      JRSYM(I,J)=0
      ENDDO
      ENDDO
      JRSYM(3,3)=1
C
      IF (M.EQ.1) THEN
C 180-omega, 180+phi
            JRSYM(1,1)=-1
            JRSYM(1,3)=180
            JRSYM(2,2)=+1
            JRSYM(2,3)=180
      ELSEIF(M.EQ.2) THEN
C 180-omega, -phi
            JRSYM(1,1)=-1
            JRSYM(1,3)=180
            JRSYM(2,2)=-1
            JRSYM(2,3)=0
      ELSEIF(M.EQ.3) THEN
C 180-omega, 180-phi
            JRSYM(1,1)=-1
            JRSYM(1,3)=180
            JRSYM(2,2)=-1
            JRSYM(2,3)=180
      ELSEIF(M.EQ.4) THEN
C omega, 180+phi
            JRSYM(1,1)=1
            JRSYM(1,3)=0
            JRSYM(2,2)=1
            JRSYM(2,3)=180
      ENDIF
      END
C
C
C
      SUBROUTINE SYMPUT(JA,JB,IA,IB)
C     ==============================
C
C Put point IA, IB (phi, omega values in degrees) into calculated
C area, using symmetry if required, return array index JA, JB.
C If no corresponding point, return JA=0
C
      COMMON /MATRIX/ ORTHOG(3,3,2),FRAC(3,3,2),SYM(4,4,192,2),NSYM(2),
     . MRSYM,JRSYM(3,3,12),LSYMFG,LHALF
C
      PARAMETER(MCONTR=50)
      COMMON /ALMFRF/ NCRYST,IPHI1,IPHI2,NPHI,DPHI,IOMEG1,IOMEG2,NOMEG,
     . DOMEG,IKAPA1,IKAPA2,NKAPA,DKAPA,ARAD,EPS,RESMIN,RESMAX,SMIN,SMAX,
     . NCODES(2),LSPGPS(2),TFACS(2),FRMINS(2),FRMAXS(2),LPRINT,LMAP,
     . LPLOT,CONTRS(MCONTR),NCONTR,LREAD,LSAVE,PEAK,MAXPEK,LPKRMS,RKAP1,
     . RKAP2,RMIN,RMEAN,RRMS,RWT,NPOINT,LDL
C
C INSIDE is a statement function to decide if a point is inside the
C area calculated
      LOGICAL INSIDE
      INSIDE(KA,KB)=KA.GE.IPHI1 .AND. KA.LE.IPHI2 .AND.
     . KB.GE.IOMEG1 .AND. KB.LE.IOMEG2
C
C
      DO L=0,MRSYM
C
      IF (L.EQ.0) THEN
C Identity
        IAA=IA
        IBB=IB
      ELSE
        IAA=JRSYM(2,2,L)*IA+JRSYM(2,3,L)
        IBB=JRSYM(1,1,L)*IB+JRSYM(1,3,L)
      ENDIF
C
      KA=NINT(IAA/DPHI)
      KB=NINT(IBB/DOMEG)
C
      IF (INSIDE(KA,KB)) GOTO 100
C Check 'translations' on phi
      IF (IAA.GT.0) THEN
        IAA=IAA-360
      ELSE
        IAA=IAA+360
      ENDIF
      KA=NINT(IAA/DPHI)
      IF (INSIDE(KA,KB)) GOTO 100
      ENDDO
C
C Not found
      JA=0
      RETURN
C
C Valid point found, calculate array indices
100   JA=KA-IPHI1+1
      JB=KB-IOMEG1+1
      END
C
C
C
      SUBROUTINE HPCNTR (A,M,N,BITS,CONT,NCONT,X,Y,IPTMAX)
C
C***************************************
C
C                  CONTUR ROUTINE
C
C***************************************
C
C  A IS FUNCTION TO BE CONTOURED, DIMENSIONED A(M,N) OR A(M*N)
C  M = NUMBER OF ROWS OF A
C  N = NUMBER OF COLUMN
C  BITS IS LOGICAL ARRAY DIMENSIONED AT LEAST 2*M*N IN CALLING ROUTINE.
C  CONT  ARRAY OF NCONT CONTOUR LEVELS
C  X,Y  ARRAYS OF DIMENSION IPTMAX USED IN SUBROUTINE TO STORE CONTOUR
C   LINES
C
C  IDIV = NUMBER OF LINE SEGMENTS BETWEEN CONTOUR LEVEL CROSSINGS IN
C  UNIT GRID CELL.  IF >= 1, THEN ROUTINE FINDS IDIV-1 COORDINATES
C  BETWEEN CROSSINGS, BY LINEAR INTERPOLATION ON THE UNIT CELL.
C  CONTUR OUTPUT VIA CALLS TO SUBROUTINE APLOT(X,Y,IPT)
C     X,Y ARE COORDINATES WITH 0<=X<M AND 0<=Y<N.
C  SUBROUTINE CONSTL(CL) IS CALLED TO SET UP PLOTTING STYLE FOR CONTOUR
C  LEVEL CL
C
      DIMENSION IDIR(4), ISIDE(5), IVCT(4), JVCT(4), KVCT(4), LVCT(4),
     *   SIDE(4), NVCT(4)
      LOGICAL BITS
      DIMENSION A(*),BITS(*),C9(4),CONT(NCONT)
      DIMENSION X(*),Y(*)
      DATA  IDIR/3,4,1,2/, ISIDE/1,2,3,4,1/, IVCT/0,-1,0,1/,
     * JVCT/-1,0,1,0/, SIDE/0.,0.,1.0,1.0/, KVCT/4,1,2,3/, LVCT/2,3,4,1/
C
      IDIV=1
      MN=M*N
      MN2=2*MN
C      WRITE(6,1008)M,N,MN,MN2
C 1008 FORMAT (/' M,N,MN,MN2 :',4I5)
      DIV=IDIV
      ICONT=1
C
C LOOP CONTOUR LEVELS CL
1     CL=CONT(ICONT)
      ICONT=ICONT+1
C
C
C
      DO I=1,MN2
        BITS(I)=.FALSE.
      ENDDO
      NVCT(1)=0
      NVCT(2)=MN
      NVCT(3)=M
      NVCT(4)=MN+1
      IPT=1
      MM=M-1
      NN=N-1
C SEARCH FOR CONTOUR CROSSING BETWEEN ADJACENT COLUMN OF ARRAY A(I,J)
      I=0
      J=1
      ISUB=0
      JSUB=0
      IRTN=1
  100 IF (J.GT.N) GOTO 140
  110 IF (I.GE.MM) GOTO 130
      I=I+1
      ISUB=ISUB+1
      JSUB=JSUB+1
      IF (A(ISUB)-CL) 115,600,120
115   IF (A(ISUB+1)-CL) 110,110,125
120   IF (A(ISUB+1)-CL) 125,110,110
125   IF (BITS(JSUB+NVCT(1))) GOTO 110
      XSTART=(CL-A(ISUB))/(A(ISUB+1)-A(ISUB))
      YSTART=0
      GOTO 200
  130 I=0
      ISUB=ISUB+1
      JSUB=JSUB+1
      J=J+1
      GOTO 100
C     SEARCH FOR CONTOUR CROSSING BETWEEN ADJACENT ROWS OF ARRAY A(I,J)
  140 I=0
      J=1
      JSUB=0
      ISUB=0
      IRTN=2
  150 IF (J.GT.NN) GOTO 190
  160 IF (I.GE.M) GOTO 180
      I=I+1
      ISUB=ISUB+1
      JSUB=JSUB+1
      IF (A(ISUB)-CL) 165,160,170
165   IF (A(ISUB+M)-CL) 160,160,175
170   IF (A(ISUB+M)-CL) 175,160,160
175   IF (BITS(JSUB+NVCT(2))) GOTO 160
      YSTART=(CL-A(ISUB))/(A(ISUB+M)-A(ISUB))
      XSTART=0
      GOTO 200
  180 I=0
      J=J+1
      GOTO 150
190   IF (ICONT.GT.NCONT)RETURN
      GOTO 1
C
C
C     BEGIN FOLLOWING CONTOUR LINE... SAVE INDICIES FOR RETURN TO SEARCH
  200 ISAVE=I
      JSAVE=J
      ISUBSV=ISUB
      JSUBSV=JSUB
      XSAVE=XSTART
      YSAVE=YSTART
      X(1)=XSTART+I-1
      Y(1)=YSTART+J-1
      IENT=IRTN
      IRS=0
      GOTO 250
C     DUMP LINE AND FOLLOW CONTOUR LINE ON OPPOSITE SIDE OF STARTING
C     POINT WHEN USED A SECOND TIME THIS ENTRY RETURNS TO SEARCH
  205 IRS=1
  210 IF (IPT.GT.1) CALL  APLOT(X,Y,IPT)
      IPT=1
      I=ISAVE
      J=JSAVE
      ISUB=ISUBSV
      JSUB=JSUBSV
      XSTART=XSAVE
      YSTART=YSAVE
      X(1)=XSTART+I-1
      Y(1)=YSTART+J-1
      IF (IRS.NE.0) GOTO (110,160), IRTN
      IEXIT=IRTN
      IRS=1
      GOTO 240
C     RETURN FROM FOLLOWING CONTOUR LINE THROUGH A CELL
230   IF (BITS(JSUB+NVCT(IEXIT))) GOTO 205
  240 I=I+IVCT(IEXIT)
      J=J+JVCT(IEXIT)
      JSUB=I+(J-1)*M
      ISUB=JSUB
      IENT=IDIR(IEXIT)
250   BITS(JSUB+NVCT(IENT))=.TRUE.
      IF (I.LT.1 .OR. I.GT.MM .OR. J.LT.1 .OR. J.GT.NN) GOTO 210
C     FIND CONTOUR CROSSING IN NEW CELL
      IF (ISUB+1.GT.MN .OR. ISUB+M.GT.MN .OR. ISUB+1+M.GT.MN) GOTO 210
      C9(1)=A(ISUB+1)
      C9(2)=A(ISUB)
      C9(3)=A(ISUB+M)
      C9(4)=A(ISUB+1+M)
      JRTN=1
      ICNT=1
      JCNT=1
      DO IROUND=1,4
        IF (IROUND.EQ.IENT) GOTO 290
        I1=ISIDE(IROUND)
        I2=ISIDE(IROUND+1)
        IF (C9(I1)-CL) 270,285,275
270     IF (C9(I2)-CL) 290,290,280
275     IF (C9(I2)-CL) 280,290,290
280     IEXIT=IROUND
        ICNT=ICNT+1
        GOTO 290
285     JEXIT=IROUND
        JCNT=JCNT+1
290     CONTINUE
      ENDDO
      GOTO (300,310,700,210), JCNT
  300 GOTO (210,320,210,800), ICNT
  310 GOTO (710,320,210,210), ICNT
  320 GOTO (330,340,350,360), IENT
  330 GOTO (210,410,500,410), IEXIT
  340 GOTO (510,210,510,400), IEXIT
  350 GOTO (500,410,210,410), IEXIT
  360 GOTO (510,400,510,210), IEXIT
C     FOLLOW CONTOUR LINE ACROSS A CELL TO A SIDE
  400 XSTART=SIDE(IENT)
  410 XFIN=SIDE(IEXIT)
      XINC=(XFIN-XSTART)/DIV
      XBASE=I-1
      YBASE=J-1
      A1=CL-C9(2)
      A2=C9(1)-C9(2)
      A3=C9(3)-C9(2)
      A4=C9(2)-C9(1)+C9(4)-C9(3)
      DO INTERP=1,IDIV
        XSTART=XSTART+XINC
        YSTART=(A1-A2*XSTART)/(A3+A4*XSTART)
        IF (IPT.LT.IPTMAX) GOTO 430
        CALL APLOT(X, Y, IPT)
        X(1)=X(IPT)
        Y(1)=Y(IPT)
        IPT=1
430     IPT=IPT+1
        X(IPT)=XBASE+XSTART
        Y(IPT)=YBASE+YSTART
      ENDDO
      GOTO (230,210,615,635), JRTN
  500 YSTART=SIDE(IENT)
C     FOLLOW CONTOUR LINE ACROSS A CELL TO A TOP OR BOTTOM
  510 YFIN=SIDE(IEXIT)
      XBASE=I-1
      YINC=(YFIN-YSTART)/DIV
      YBASE=J-1
      A1=CL-C9(2)
      A2=C9(3)-C9(2)
      A3=C9(1)-C9(2)
      A4=C9(2)-C9(1)+C9(4)-C9(3)
      DO INTERP=1,IDIV
        YSTART=YSTART+YINC
        XSTART=(A1-A2*YSTART)/(A3+A4*YSTART)
        IF (IPT.LT.IPTMAX) GOTO 530
        CALL APLOT(X, Y, IPT)
        X(1)=X(IPT)
        Y(1)=Y(IPT)
        IPT=1
530     IPT=IPT+1
        X(IPT)=XBASE+XSTART
        Y(IPT)=YBASE+YSTART
      ENDDO
      GOTO (230,210,615,635), JRTN
C     FOLLOW CONTOUR LINE FROM CORNER TO CORNER
600   IF (ISUB-M.LE.0 .OR. ISUB+1+M.GT.MN) GOTO 640
      K1=ISUB-M
      K2=ISUB+1-M
      K3=ISUB+1
      K4=ISUB+1+M
      K5=ISUB+M
      K6=ISUB-1+M
      K7=ISUB-1
      IF(K1.GT.0) C1=A(K1)
      IF(K2.GT.0) C2=A(K2)
      IF(K3.GT.0) C3=A(K3)
      IF(K4.GT.0) C4=A(K4)
      IF(K5.GT.0) C5=A(K5)
      IF(K6.GT.0) C6=A(K6)
      IF(K7.GT.0) C7=A(K7)
      IF(ISUB.GT.0) C8=A(ISUB)
      X(1)=I-1
      Y(1)=J-1
      IF (J.EQ.1 .OR. J.EQ.M) GOTO 610
      IF (K1.LT.1 .OR. K1.GT.MN) GOTO 610
      IF (K2.LT.1 .OR. K2.GT.MN) GOTO 610
      IF (K3.LT.1 .OR. K3.GT.MN) GOTO 610
      IF (K4.LT.1 .OR. K4.GT.MN) GOTO 610
      IF (K5.LT.1 .OR. K5.GT.MN) GOTO 610
      IF (C3.NE.CL) GOTO 610
      IF (C1.EQ.CL .AND. C2.EQ.CL .AND. C4.EQ.CL .AND. C5.EQ.CL)
     * GOTO 610
      X(2)=X(1)+1.
      Y(2)=Y(1)
      CALL APLOT(X, Y, 2)
      GOTO 620
  610 IF (J.EQ.1) GOTO 620
      IF (K1.LT.1 .OR. K1.GT.MN) GOTO 620
      IF (K2.LT.1 .OR. K2.GT.MN) GOTO 620
      IF (K3.LT.1 .OR. K3.GT.MN) GOTO 620
      IF (C2.NE.CL) GOTO 620
      IF (C1.EQ.CL .OR. C3.EQ.CL) GOTO 620
      IF (C1.GT.CL .AND. C3.GT.CL .OR. C1.LT.CL .AND. C3.LT.CL)
     * GOTO 620
      C9(1)=C2
      C9(2)=C1
      C9(3)=C8
      C9(4)=C3
      J=J-1
      JRTN=3
      IENT=3
      IEXIT=1
      GOTO 500
  615 IF (IPT.GT.1) CALL APLOT(X, Y, IPT)
      IPT=1
      J=J+1
      X(1)=I-1
      Y(1)=J-1
  620  IF (J.EQ.M .OR. I.EQ.1) GOTO 630
      IF (K3.LT.1 .OR. K3.GT.MN) GOTO 630
      IF (K4.LT.1 .OR. K4.GT.MN) GOTO 630
      IF (K5.LT.1 .OR. K5.GT.MN) GOTO 630
      IF (K6.LT.1 .OR. K6.GT.MN) GOTO 630
      IF (K7.LT.1 .OR. K7.GT.MN) GOTO 630
      IF (C5.NE.CL) GOTO 630
      IF (C3.EQ.CL .AND. C4.EQ.CL .AND. C6.EQ.CL .AND. C7.EQ.CL)
     * GOTO 630
      X(2)=X(1)
      Y(2)=Y(1)+1.
      CALL APLOT(X, Y, 2)
      GOTO 640
  630 IF (J.EQ.M) GOTO 640
      IF (K3.LT.1 .OR. K3.GT.MN) GOTO 640
      IF (K4.LT.1 .OR. K4.GT.MN) GOTO 640
      IF (K5.LT.1 .OR. K5.GT.MN) GOTO 640
      IF (C4.NE.CL) GOTO 640
      IF (C3.EQ.CL .OR. C5.EQ.CL) GOTO 640
      IF (C3.GT.CL .AND. C5.GT.CL .OR. C3.LT.CL .AND. C5.LT.CL) GOTO 640
      C9(1)=C3
      C9(2)=C8
      C9(3)=C5
      C9(4)=C4
      JRTN=4
      IENT=1
      IEXIT=3
      GOTO 500
  635 IF (IPT.GT.1) CALL APLOT(X, Y, IPT)
      IPT=1
      X(1)=I-1
      Y(1)=J-1
  640 GOTO (110,160), IRTN
C    FOLLOW CONTOUR LINE FROM SIDE TO CORNER OR CORNERS
  700 JRTN=2
      IOPP=IDIR(IENT)
      I1=ISIDE(IOPP)
      I2=ISIDE(IOPP+1)
      IEXIT=IOPP
      C9(I1)=C9(KVCT(I1))
      C9(I2)=C9(LVCT(I2))
      GOTO 320
  710 JRTN=2
      IEXIT=JEXIT
      GOTO 320
C     FOLLOW CONTOUR LINE THROUGH SADDLE POINT
  800 IOPP=IDIR(IENT)
      I1=ISIDE(IENT)
      C1=C9(I1)
      I2=ISIDE(IENT+1)
      C2=C9(I2)
      I3=ISIDE(IOPP)
      C3=C9(I3)
      I4=ISIDE(IOPP+1)
      C4=C9(I4)
      IF ((C1-CL)/(C1-C2).EQ.(C4-CL)/(C4-C3)) GOTO 820
      IF ((C1-CL)/(C1-C4).GT.(C2-CL)/(C2-C3)) GOTO 810
      IEXIT=I4
      GOTO 320
  810 IEXIT=I2
      GOTO 320
  820 C9(I3)=C9(I2)
      C9(I4)=C9(I1)
      IEXIT=I3
      GOTO 320
      END
C
      SUBROUTINE APLOT(X,Y,IPT)
C     =========================
C
C JOIN IPT COORDINATE PAIRS X,Y OF CONTOUR CROSSINGS, IN GRID UNITS
C FOR M*N GRID , COORDINATES ARE IN RANGE 0,0 TO M,N
C
      DIMENSION X(IPT),Y(IPT)
C
C SAVE 1ST AND LAST POINTS BEFORE TRANSFORMATION
C - - - - - -- - - - - - - - - - - - - - - - - - - - - -
C
C RESET ALL SORTS OF TRANSFORMATIONS AND SCALE ETC TO USE THIS
C SUBROUTINE BLINDLY IN RFPLOT
C
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
      X1=X(1)
      Y1=Y(1)
      XL=X(IPT)
      YL=Y(IPT)
C
C DRAW LINES
C
      ILPPP = 10
      IF (ILPPP.GT.IPT) ILPPP = IPT
C     WRITE(6,1000)(X(INNN),Y(INNN),INNN=1,ILPPP)
C 1000 FORMAT(// ' MADE IT INTO APLOT',15F5.1)
      CALL LINES(X,Y,IPT)
C RESTORE 1ST & LAST POINTS
      X(1)=X1
      Y(1)=Y1
      X(IPT)=XL
      Y(IPT)=YL
      END
C
      SUBROUTINE LINES(X,Y,NPT)
C     =========================
C
C DRAW LINES TO JOIN NPT POINTS X,Y
C
C Note y = phi, x = omega, in grid units
C
      DIMENSION X(NPT),Y(NPT)
C
      COMMON /MATRIX/ ORTHOG(3,3,2),FRAC(3,3,2),SYM(4,4,192,2),NSYM(2),
     . MRSYM,JRSYM(3,3,12),LSYMFG,LHALF
C
      PARAMETER(MCONTR=50)
      COMMON /ALMFRF/ NCRYST,IPHI1,IPHI2,NPHI,DPHI,IOMEG1,IOMEG2,NOMEG,
     . DOMEG,IKAPA1,IKAPA2,NKAPA,DKAPA,ARAD,EPS,RESMIN,RESMAX,SMIN,SMAX,
     . NCODES(2),LSPGPS(2),TFACS(2),FRMINS(2),FRMAXS(2),LPRINT,LMAP,
     . LPLOT,CONTRS(MCONTR),NCONTR,LREAD,LSAVE,PEAK,MAXPEK,LPKRMS,RKAP1,
     . RKAP2,RMIN,RMEAN,RRMS,RWT,NPOINT,LDL
C
      PARAMETER(PI=3.141592654D0, TWOPI=2.*PI, DTR=PI/180.)
      IF (NPT.LE.1) RETURN
C
C  STEREOGRAPH
C
      DO ILLL=1,NPT
C
        OMEGA=X(ILLL)*DOMEG
        IF (LHALF.EQ.2) OMEGA=90.-OMEGA
        OMEGA=DTR*OMEGA
        R=SIN(OMEGA)/(COS(OMEGA)+1.)
C
        PHI=Y(ILLL)*DPHI
        IF (LHALF.EQ.2 .AND. LSYMFG.LT.0) PHI=180.+PHI
        PHI=DTR*PHI
C
        X(ILLL)=R*COS(PHI)
        Y(ILLL)=R*SIN(PHI)
      ENDDO
C
      CALL PLTLIN(X,Y,NPT)
      END
C
C
C
      SUBROUTINE PEAKPK(RROT,LA,LB,IKAPA,RMAP)
C     ========================================
C
C Find peaks in previous section
C   Threshold is PEAK if LPKRMS=0, PEAK*RMS if LPKRMS=1
C
C IKAPA is current kappa section, IKAPA=-1 for search of final section
C
C RMAP is used as a rotating buffer, RMAP(,,I1), RMAP(,,I2), RMAP(,,I3),
C      contain 3 sections, I2 is the one to be searched
C
C
      DIMENSION RROT(LA,LB),RMAP(0:LA+1,0:LB+1,3)
C
C
      PARAMETER(MCONTR=50)
      COMMON /ALMFRF/ NCRYST,IPHI1,IPHI2,NPHI,DPHI,IOMEG1,IOMEG2,NOMEG,
     . DOMEG,IKAPA1,IKAPA2,NKAPA,DKAPA,ARAD,EPS,RESMIN,RESMAX,SMIN,SMAX,
     . NCODES(2),LSPGPS(2),TFACS(2),FRMINS(2),FRMAXS(2),LPRINT,LMAP,
     . LPLOT,CONTRS(MCONTR),NCONTR,LREAD,LSAVE,PEAK,MAXPEK,LPKRMS,RKAP1,
     . RKAP2,RMIN,RMEAN,RRMS,RWT,NPOINT,LDL
C
      PARAMETER(MAXNPK=100)
      COMMON /PEAKS/ NPEAK,PEAKS(MAXNPK),PKPOS(3,MAXNPK),LPEAK,
     .  LPKORD(MAXNPK),LPKOUT
C
      REAL OFFSET(3),OFFMAX,PEAKMX(-1:1,-1:1,-1:1)
      REAL RMAX
      SAVE I1,I2,I3,KKAPA,OFFMAX
C
      DATA I2/0/
C Largest acceptable offset from central grid point
      DATA OFFMAX /.6/
C
      IF (MAXPEK.EQ.0) RETURN
C Set threshold
      THRHLD=PEAK
      IF (LPKRMS.GT.0) THRHLD=THRHLD*RRMS
C
      IF (IKAPA.LT.0) THEN
C  If I1 = -1, this is first & last section (1 section only), so
C   generate dummy section all = -100.
        IF (I1.LT.0) THEN
          I1 = 1
          DO J = 0, LB+1
            DO I = 0, LA+1
              RMAP(I,J,I1) = -100.
            ENDDO
          ENDDO
        ENDIF
C For final section, copy slot 1 to slot 3
        DO J = 0, LB+1
          DO I = 0, LA+1
            RMAP(I,J,I3) =  RMAP(I,J,I1)
          ENDDO
        ENDDO
CEJD        CALL MOVEI(RMAP(0,0,I3), RMAP(0,0,I1), (LA+2)*(LB+2))
        GOTO 10
      ENDIF
C
      IF (I2.EQ.0) THEN
C First section, just store section
        I1=-1
        I2=2
        I3=3
C KKAPA is kappa for section next to be searched (slot I2)
        KKAPA=IKAPA
        DO J = 1, LB
          DO I = 1, LA
            RMAP(I,J,I2)=RROT(I,J)
          ENDDO
        ENDDO
CEJD        CALL MOVEI(RMAP(1,J,I2),RROT(1,J),LA)
C  To handle edge effects, each section is extended by copying the
C  2nd & penultimate rows and columns to give a extra row & column
C  all round. This is equivalent to assuming each edge is a mirror
C  plane, which is unlikely to be true, however it will force all
C  edge peaks to lie on the edge plane. Similarly, sandwich the 1st
C  section, between 2 copies of the 2nd section, and the last section
C  between 2 copies of the penultimate
C
        CALL EXSECT(RMAP(0,0,I2), LA, LB)
C
C  Set up interpolation matrix
        CALL PKSETM
C
        RETURN
      ENDIF
C
C Store new section in slot I3
      DO J = 1, LB
        DO I = 1, LA
          RMAP(I,J,I3)=RROT(I,J)
        ENDDO
      ENDDO
CEJD      CALL MOVEI(RMAP(1,J,I3),RROT(1,J),LA)
      CALL EXSECT(RMAP(0,0,I3), LA, LB)
C
      IF (I1.LT.0) THEN
C  2nd section, copy into slot I1
        I1 = 1
        DO J = 0, LB+1
          DO I = 0, LA+1
            RMAP(I,J,I1) =  RMAP(I,J,I3)
          ENDDO
        ENDDO
CEJD        CALL MOVEI(RMAP(0,0,I1), RMAP(0,0,I3), (LA+2)*(LB+2))
      ENDIF
C
C Now we have a section KKAPA in slot I2, and maybe in slots I1 and I3
C if slot numbers are .gt. 0, so search section I2
C First check for special cases kappa=0
10    IF (MOD(NINT(KKAPA*DKAPA),360).EQ.0) THEN
        NA1=1
        NA2=1
        NB1=1
        NB2=1
        L=0
      ELSE
        NA1=1
        NA2=LA
        NB1=1
        NB2=LB
        L=1
      ENDIF
C
C Search for peaks above threshold THRHLD
      DO I=NA1,NA2
        DO J=NB1,NB2
          R=RMAP(I,J,I2)
C Is this point above THRHLD?
          IF (R.GT.THRHLD) THEN
C Gather 3 x 3 x 3 grid centred on this point
            RMAX = -100.
            DO L2 = -1, 1
              DO L1 = -1, 1
                PEAKMX(L1,L2,-1) = RMAP(I+L1,J+L2,I1)
                PEAKMX(L1,L2, 0) = RMAP(I+L1,J+L2,I2)
                PEAKMX(L1,L2, 1) = RMAP(I+L1,J+L2,I3)
                DO L3 = -1, 1
                  RMAX = MAX (RMAX, PEAKMX(L1,L2,L3))
                ENDDO
              ENDDO
            ENDDO
C
C     Is the central point the highest?
            IF (L.GT.0 .AND. PEAKMX(0,0,0).LT.RMAX) GOTO 21
C  No neighbours higher, so we have a peak
            N=NPEAK+1
            IF (N.GT.MAXPEK) THEN
C  We've already got enough peaks, so find smallest if smaller than the
C  new one
              A=R
              N=0
              DO K=1,NPEAK
                IF (PEAKS(K).LT.A) THEN
                  A=PEAKS(K)
                  N=K
                ENDIF
              ENDDO
              IF (N.EQ.0) GOTO 21
            ELSE
C  Append to list
              NPEAK=N
            ENDIF
C
C Store peak in slot N
            PEAKS(N)=R
C Interpolate position
            CALL PKFITP(PEAKMX, OFFMAX, OFFSET, ISTAT)
C
            PKPOS(1,N)=(IOMEG1+J-1+OFFSET(2))*DOMEG
            PKPOS(2,N)=(IPHI1+I-1+OFFSET(1))*DPHI
            PKPOS(3,N)=(KKAPA+OFFSET(3))*DKAPA
          ENDIF
C
21        CONTINUE
        ENDDO
C
C If omeg limits are 0 or 180, reset omeg limits after first row
C This removes some redundancy, but is by no means foolproof.
C More complicated tests are needed
        IF (I.EQ.1) THEN
          IF (IOMEG1.EQ.0) NB1=NB1+1
          IF (NINT(IOMEG2*DOMEG).EQ.180) NB2=NB2-1
        ENDIF
C
      ENDDO
C
C Set pointers for next section
      I1=I2
      I2=I3
      I3=I2+1
      IF (I3.GT.3) I3=1
      KKAPA=IKAPA
      END
C
C
C
C
C
      SUBROUTINE EXSECT(RMAP, LA, LB)
C     ===============================
C
C Extend section by 1 row & column all round, assuming mirror plane
C on edge. This is likely not to be true, but at least will force
C edge peaks to lie on the edge
C
      INTEGER LA, LB
      REAL RMAP(0:LA+1,0:LB+1)
C
      INTEGER I
C
      DO I = 1, LA
        RMAP(I, 0) = RMAP(I, 2)
        RMAP(I, LB+1) = RMAP(I, LB-1)
      ENDDO
C
      DO I = 0, LB+1
        RMAP(0, I) = RMAP(2, I)
        RMAP(LA+1, I) = RMAP(LA-1, I)
      ENDDO
      END
C
C
C
      SUBROUTINE PKSETM
C     =================
C
C Set up normal matrix etc for 3D paraboloid fit
C
C Observational equation is
C  r(calc) = a x**2 + b y**2 + c z**2 + d xy + e yz + f zx +
C            g x + h y + i z + j
C
C Observation vector
C  xT = ( x**2  y**2  z**2  xy  yz  zx   x   y   z   1 )
C
C Observation matrix
C
C  A  =  (  x(1)T  )          for each point on 3 x 3 x 3 grid
C        (  x(2)T  )
C        ( . . . . )
C        (  x(n)T  )
C
      INTEGER NP, NE, NG
      PARAMETER( NP = 10, NE = 3, NG = NE**3)
      REAL  AT(NP, NG), ATA(NP, NP)
C
      COMMON /PKFIT/ SMAT(NP, NG), A(NG, NP)
      REAL SMAT, A
C
      INTEGER IX, IY, IZ, I, J
      REAL X, Y, Z, DET
      INTEGER W1(NP), W2(NP)
C
      SAVE /PKFIT/
C
C  Loop points on 3 x 3 x 3 grid
      I = 0
      DO IZ = -1, 1
        Z = IZ
        DO IY = -1, 1
          Y = IY
          DO IX = -1, 1
            X = IX
            I = I + 1
C Matrix elements
            A(I, 1) = X**2
            A(I, 2) = Y**2
            A(I, 3) = Z**2
            A(I, 4) = X * Y
            A(I, 5) = Y * Z
            A(I, 6) = Z * X
            A(I, 7) = X
            A(I, 8) = Y
            A(I, 9) = Z
            A(I, 10) = 1.
          ENDDO
        ENDDO
      ENDDO
C
C Form A(transpose)
      DO I = 1, NG
        DO J = 1, NP
          AT(J, I) = A(I, J)
        ENDDO
      ENDDO
C
C A(tranpose) * A
      CALL GMPRD(AT, A, ATA, NP, NG, NP)
C
C
C Invert in place
      CALL MINVN(ATA, NP, DET, W1, W2)
      IF (DET.EQ.0.) CALL CCPERR(1, 'PKSETM: singular matrix')
C
C  SMAT = (AT A)**1 AT
      CALL GMPRD(ATA, AT, SMAT, NP, NP, NG)
      END
C
C
C
      SUBROUTINE PKFITP (PEAKMX, OFFMAX, OFFSET, ISTAT)
C     =================================================
C
C Fit paraboloid to 3 x 3 x 3 grid in PEAKMX (central point assumed to
C be maximum), return position of maximum in OFFSET (grid units)
C Subroutine PKSETM must have been called first to calculate solution
C matrix SMAT
C
C On entry:
C  PEAKMX       values of peak  (3 x 3 x 3 grid)
C  OFFMAX       maximum value of offset
C
C On exit:
C  OFFSET(3)    offsets in grid units
C  ISTAT        = 1 if offset reset to maximum, else = 0
C
C
      REAL PEAKMX(-1:1,-1:1,-1:1), OFFMAX, OFFSET(3)
      INTEGER ISTAT
C
      INTEGER NP, NE, NG
      PARAMETER( NP = 10, NE = 3, NG = NE*NE*NE)
C
      COMMON /PKFIT/ SMAT(NP, NG), A(NG, NP)
      REAL SMAT, A
C
      INTEGER I
      REAL P(NP), B(3,3), G(3), DET
      INTEGER W1(3), W2(3)
C
      SAVE /PKFIT/
C
C
C Solve for parameters P(1)->P(10), = a,b,c,d,e,f,g,h,i,j
C
C    p  = [ (AT A)**-1 AT ]  r
C where  (AT A)**-1 AT = SMAT is calculated in PKSETM
C        r = PEAKMX, observation vector
C Note that in PKSETM, the order of grid points corresponding to the
C rows of matrix A must correspond to the order of values in PEAKMX, ie
C x fast, y medium, z slow
C
      CALL GMPRD (SMAT, PEAKMX, P, NP, NG, 1)
C
C Set up matrix B and vector g for solving position of maximum from the
C paraboloid parameters
C
C           (  2a   d   f  )                    (  -g  )
C     B  =  (   d  2b   e  )              g  =  (  -h  )
C           (   f   e  2c  )                    (  -i  )
C
C   Then position of maximum  OFFSET  =  x  =  B**-1 g
C
      B(1,1) = 2. * P(1)
      B(1,2) = P(4)
      B(2,1) = P(4)
      B(1,3) = P(6)
      B(3,1) = P(6)
      B(2,2) = 2. * P(2)
      B(2,3) = P(5)
      B(3,2) = P(5)
      B(3,3) = 2. * P(3)
C
      G(1) = -P(7)
      G(2) = -P(8)
      G(3) = -P(9)
C
C Invert B
      CALL MINVN(B, 3, DET, W1, W2)
      IF (DET.EQ.0.) CALL CCPERR(1, 'PKFITP: singular matrix')
C
      CALL GMPRD(B, G, OFFSET, 3, 3, 1)
C
      ISTAT = 0
      DO I = 1, 3
        IF (ABS(OFFSET(I)).GT.OFFMAX) THEN
C Reset to maximum, set flag
          ISTAT = 1
          OFFSET(I) = SIGN(OFFMAX, OFFSET(I))
        ENDIF
      ENDDO
      END
C
      SUBROUTINE SORT1(N,A0,A1)
C     =========================
C
C****
C**** N IS THE LENGTH OF THE ARRAY TO BE SORTED I.E. THE NUMBER OF
C**** ITEMS TO BE SORTED
C**** THE ELEMENTS OF ARRAY A0(N) ARE SORTED INTO INCREASING ORDER OF
C**** MAGNITUDE.  ARRAYS A1 MAY BE CARRIED WITH A0 AS REQUIRED
C****
      DIMENSION A0(N),A1(N)
      INTEGER A1
C
      DO I=1,N/2
        C = A0(I)
        D = C
        IH = I
        K = I
        NP1 = N+1-I
        DO J=I+1,NP1
          A = A0(J)
          IF (A.LT.C) THEN
            IH = J
            C = A
          ELSEIF (D.LT.A) THEN
            D = A
            K = J
          ENDIF
        ENDDO
        IF (C.EQ.D) RETURN
        IF (IH.GT.I) THEN
          C = A1(IH)
          A1(IH) = A1(I)
          A1(I) = C
          C = A0(IH)
          A0(IH) = A0(I)
          A0(I) = C
        ENDIF
        IF (K.EQ.I) K=IH
        IF (K.LT.NP1) THEN
          D = A1(K)
          A1(K) = A1(NP1)
          A1(NP1) = D
          D = A0(K)
          A0(K) = A0(NP1)
          A0(NP1) = D
        ENDIF
      ENDDO
      END
C
C
C
      SUBROUTINE PPEAKS
C     =================
C
C Print all peaks found
C
      COMMON /MATRIX/ ORTHOG(3,3,2),FRAC(3,3,2),SYM(4,4,192,2),NSYM(2),
     . MRSYM,JRSYM(3,3,12),LSYMFG,LHALF
      PARAMETER(MCONTR=50)
      COMMON /ALMFRF/ NCRYST,IPHI1,IPHI2,NPHI,DPHI,IOMEG1,IOMEG2,NOMEG,
     . DOMEG,IKAPA1,IKAPA2,NKAPA,DKAPA,ARAD,EPS,RESMIN,RESMAX,SMIN,SMAX,
     . NCODES(2),LSPGPS(2),TFACS(2),FRMINS(2),FRMAXS(2),LPRINT,LMAP,
     . LPLOT,CONTRS(MCONTR),NCONTR,LREAD,LSAVE,PEAK,MAXPEK,LPKRMS,RKAP1,
     . RKAP2,RMIN,RMEAN,RRMS,RWT,NPOINT,LDL
C
      COMMON /PKFILE/ PKFNAM
      CHARACTER*80 PKFNAM
      SAVE /PKFILE/
C
      CHARACTER*80 TITLE,FNAME
      COMMON /TEXTC/ TITLE,FNAME
C
      PARAMETER(MAXNPK=100)
      COMMON /PEAKS/ NPEAK,PEAKS(MAXNPK),PKPOS(3,MAXNPK),LPEAK,
     .  LPKORD(MAXNPK),LPKOUT
C
      DIMENSION AMINV(4,4)
      DIMENSION ARSYM(3,3,24,2),AJUNK4(3,3),TRSYM(3,3),TRMAT(3,3),DC(5),
     & AJUNK2(3,3),AJUNK3(3,3),AJUNK1(3,3)
      REAL EPKPOS(3,MAXNPK),PKHGHT(MAXNPK)
C
      CHARACTER*15  CODES(5)
      EXTERNAL MATMUL
      DATA CODES/'a,c*xa,c*','b,a*xb,a*','c,b*xc,b*',
     . 'a+b,c*x(a+b),c*','a*,cxa*,c'/
C
      IF (NPEAK.LE.0) RETURN
C
      IF (LPEAK.LE.0) THEN
C Sort reflection pointers into ascending order of magnitude
C   Initialize pointers and copy peak height. Note that the sort
C   does not reorder storage arrays PEAKS & PKPOS, but just creates
C   a list of pointers
        DO I = 1, NPEAK
          PKHGHT(I) = PEAKS(I)
          LPKORD(I) = I
        ENDDO
        CALL SORT1(NPEAK, PKHGHT, LPKORD)
C
      ELSE
C  PEAK option, set peak order LPKORD to reverse, so that peaks are
C    printed out in same order as input
        DO I = 1, NPEAK
          LPKORD(I) = NPEAK - I + 1
        ENDDO
      ENDIF
C
      LI=6
C
C Print symmetry
      CALL PRTSYM(LI,1,NSYM(1),SYM(1,1,1,1))
      IF (NCRYST.EQ.2) CALL PRTSYM(LI,2,NSYM(2),SYM(1,1,1,2))
C
      IF (LPEAK.GT.0) THEN
        WRITE(LI,9011) NPEAK
9011    FORMAT('     Positions of ',I4,' peaks'//)
      ELSEIF(LPKRMS.EQ.0) THEN
        WRITE(LI,9012) NPEAK,PEAK
9012    FORMAT('     Positions of ',I4,' peaks above',F8.0//)
      ELSE
        WRITE(LI,9013) NPEAK,PEAK,RRMS
9013    FORMAT('     Positions of ',I4,' peaks above',F8.0,
     .  ' * RMS = ',F9.3//)
      ENDIF
      WRITE(LI,9014)
9014  FORMAT(
     . 24X,' Eulerian angles                        Polar angles'//
     . 20X,'  Alpha   Beta   Gamma      Peak      ',
     . ' Omega    Phi  Kappa          Direction cosines'/
     . ' Symmetry: 1   2'//)
C
      JSYM1=NSYM(1)
      JSYM2=NSYM(2)
C  Set up matrices for crystal 1
      DO ISYM1=1,JSYM1
        DO I=1,3
          DO J=1,3
            AJUNK3(I,J)=SYM(I,J,ISYM1,1)
          ENDDO
        ENDDO
C
        CALL MATMUL(AJUNK4,AJUNK3,FRAC(1,1,1))
        CALL MATMUL(AJUNK3,ORTHOG(1,1,1),AJUNK4)
        DO I=1,3
          DO J=1,3
            ARSYM(I,J,ISYM1,1)=AJUNK3(I,J)
C  ARSYM(,,1 to JSYM1) contains  Orthog . Sym . Frac
          ENDDO
        ENDDO
      ENDDO
C
C  Set up matrices for crystal 2
      DO ISYM2=1,JSYM2
C    inverse symmetry matrix AMINV  (4x4)
        CALL RBRINV(SYM(1,1,ISYM2,2),AMINV)
C    copy to 3x3
        DO I=1,3
          DO J=1,3
            AJUNK3(J,I)=AMINV(J,I)
          ENDDO
        ENDDO
C
      CALL MATMUL(AJUNK4,AJUNK3,FRAC(1,1,2))
      CALL MATMUL(AJUNK3,ORTHOG(1,1,2),AJUNK4)
      DO I=1,3
        DO J=1,3
          ARSYM(I,J,ISYM2,2)=AJUNK3(I,J)
C  ARSYM(,,ISYM2,2) contains  Orthog . Sym**-1 . Frac
        ENDDO
      ENDDO
      ENDDO
C
C  Loop all peaks in list
C
      DO JP=1,NPEAK
C  Get peak number in descending order of size
        J=LPKORD(NPEAK-JP+1)
C
        WRITE(LI,15) JP
15      FORMAT(/1X,' Peak ',I4)
C
C Get rotation matrix corresponding to angles in list
C
        CALL POLMAT(TRMAT,PKPOS(1,J))
C
C  Convert to Eulerian for output file of unique peaks
        CALL EULERA(TRMAT,ALPHA,BETA,GAMMA)
        CALL RNGANG(ALPHA)
        EPKPOS(1,JP)=ALPHA
        CALL RNGANG(BETA)
        EPKPOS(2,JP)=BETA
        CALL RNGANG(GAMMA)
        EPKPOS(3,JP)=GAMMA
C
C
        DO ISYM1=1,JSYM1
          DO I=1,3
            DO JJ=1,3
              AJUNK3(I,JJ)=ARSYM(I,JJ,ISYM1,1)
            ENDDO
          ENDDO
C
          DO ISYM2=1,JSYM2
            DO I=1,3
              DO JJ=1,3
                AJUNK2(I,JJ)=ARSYM(I,JJ,ISYM2,2)
              ENDDO
            ENDDO
            CALL MATMUL(AJUNK1,TRMAT,AJUNK2)
            CALL MATMUL(TRSYM,AJUNK3,AJUNK1)
c
cv    restrict TRSYM to range -1.0 ... +1.0
           DO I=1,3
             DO JJ=1,3
               TRSYM(I,JJ) = MIN(1.0,MAX(-1.0,TRSYM(I,JJ)))
             ENDDO
           ENDDO
cv
C if symmetry-related rotation is identity, print message and 
C move to next peak
            IF (TRSYM(1,1)+TRSYM(2,2)+TRSYM(3,3).GE.2.9999) THEN
              WRITE(LI,116) PEAKS(J),(PKPOS(I,J),I=1,3)
116           FORMAT('  Origin peak   ',28X,F8.1,5X,3F7.1)
              GOTO 100
            ENDIF
C  Get Euler angles
            CALL EULERA(TRSYM,ALPHA,BETA,GAMMA)
C  Get Kappa & direction cosines
            CALL DCOSFD(TRSYM,DC,AKAPPA)
C  Get polar angles
            CALL POLANG(DC,OMEGA,PHI)
C Put all angles in range 0-360 degrees
            CALL RNGANG(ALPHA)
            CALL RNGANG(BETA)
            CALL RNGANG(GAMMA)
            CALL RNGANG(OMEGA)
            CALL RNGANG(PHI)
            CALL RNGANG(AKAPPA)
            WRITE(LI,14) ISYM1,ISYM2,ALPHA,BETA,GAMMA,PEAKS(J),OMEGA,
     1      PHI,AKAPPA,DC(1),DC(2),DC(3)
14          FORMAT(8X,2I4,4X,3F7.1,3X,F8.1,5X,3F7.1,6X,3F8.4)
C
C
            IF (LPEAK.GT.1) THEN
C Output matrices if peaks input and MATRICES keyword given
              CALL MINV3(AJUNK2,TRSYM,D)
              WRITE(LI,114)
     1        ((TRSYM(I,JI),JI=1,3),(AJUNK2(I,K),K=1,3),I=1,3)
114           FORMAT(/'  Rotation matrix and its inverse',
     +        /(3(3F10.5,10X,3F10.5/))/)
            ENDIF
C end of sym2
          ENDDO
C end of sym1
        ENDDO
100     CONTINUE
C back for next peak
      ENDDO
C
      WRITE(LI,9022)
9022  FORMAT(///' The rotation given by the angles of a peak rotates',
     . ' coordinates in the orthogonal frame'/
     . ' of crystal 2 to the orthogonal frame of crystal 1.'/
     . ' Beware of axis permutations introduced by NCODE = 2, 3 or 4'//
     . /' Rotation matrices are defined as follows:'///
     . ' ( l**2+(m**2+n**2)cos k     lm(1-cos k)-nsin k     ',
     .  'nl(1-cos k)+msin k )'/
     . ' ( lm(1-cos k)+nsin k      m**2+(l**2+n**2)cos k    ',
     .  'mn(1-cos k)-lsin k )',5X,'Polar angles'/
     . ' ( nl(1-cos k)-msin k        mn(1-cos k)+lsin k   ',
     .  'n*2+(l**2+m**2)cos k )'//
     . '     where l m n are the direction cosines of the axis about',
     . ' which the rotation k = kappa takes place.'//
     .   31X,'( l )             ( sin omega   cos phi  )'/
     .   31X,'( m )     =       ( sin omega   sin phi  )'/
     .   31X,'( n )             (       cos omega      )'/)
C
      WRITE(LI,9023)
9023  FORMAT(//
     . ' ( cosa cosb cosg - sina sing      -cosa cosb sing - sina cosg',
     .  '      cosa sinb )'/
     . ' ( sina cosb cosg + cosa sing      -sina cosb sing + cosa cosg',
     .  '      sina sinb )',5X,'Eulerian angles Alpha, Beta, Gamma'/
     . ' (         -sinb cosg                         sinb sing       ',
     .  '         cosb   )'/)
C
C
C-----------------------------------------------------------------------
C Write unique peaks out to file
C
      IF (LPKOUT.GT.0) THEN
        L=LENSTR(PKFNAM)
        WRITE(LI, 61001) PKFNAM(:L)
61001   FORMAT(//' Unique peaks written out to file ',A)
        LUN=1
        CALL CCPDPN(LUN, PKFNAM(:L),'NEW','F',0,0)
C
        WRITE (LUN, 61000) TITLE(:LENSTR(TITLE))
61000   FORMAT('!   Peaks from POLARRFN'/'TITLE ',A)
        IF (NCRYST.EQ.1) THEN
          WRITE (LUN, 61002) 'SELF'
        ELSEIF (NCRYST.EQ.1) THEN
          WRITE (LUN, 61002) 'CROSS'
61002     FORMAT(A)
        ENDIF
        DO L=1,NCRYST
          WRITE (LUN, 61003) NCODES(L),CODES(NCODES(L))
61003     FORMAT('CRYSTAL',I5,'       ! Ncode, orthogonalization ',A)
          WRITE (LUN, 61004) L, NSYM(L)
61004     FORMAT('SYMORTHOGONAL',2I5)
C
          DO K=1,NSYM(L)
            WRITE (LUN, 61005) K,((ARSYM(I,J,K,L), J=1,3), I=1,3)
61005       FORMAT('!   Symmetry operation ',I5/2(3F12.5/),3F12.5)
          ENDDO
        ENDDO
C
C Now the peaks
C
        WRITE (LUN, 61006) NPEAK
61006   FORMAT('PEAKS',I8/
     .  '!         Alpha      Beta     Gamma     Height')
        DO L = 1, NPEAK
          K = LPKORD(NPEAK-L+1)
          WRITE (LUN, 61007) L, (EPKPOS(I, L), I = 1, 3), PEAKS(K)
61007     FORMAT(I6,3F10.1,F10.2)
        ENDDO
        CLOSE(LUN)
      ENDIF
      END
C
C
      SUBROUTINE PRTSYM(IOUT,ICRYST,NSYM,ROT)
C     =======================================
C
C Print symmetry matrices
C
      INTEGER IOUT,ICRYST,NSYM
      REAL ROT(4,4,NSYM)
C
      INTEGER J,M,L
C
      WRITE(IOUT,2) ICRYST
2     FORMAT(///' Unpermuted symmetry operations for crystal ',I3,': ')
C
      DO J=1,NSYM
        WRITE(IOUT,4) J,((ROT(L,M,J),M=1,4),L=1,3)
4       FORMAT(/21X,'Symmetry matrix',I5,5X,3F10.5,5X,F10.5/
     .  2(46X,3F10.5,5X,F10.5/))
      ENDDO
      END
C
      SUBROUTINE EULERA(TRSYM,ALPHA,BETA,GAMMA)
C     =========================================
C
C Get Euler angles ALPHA,BETA,GAMMA from matrix TRSYM
C
      REAL TRSYM(3,3),ALPHA,BETA,GAMMA
C
      PARAMETER(PI=3.141592654D0, RTD=180./PI)
C
      IF (ABS(TRSYM(3,3)).GE.0.99999) GOTO 13
C
      BETA=RTD*ACOS(TRSYM(3,3))
C
      R13=TRSYM(1,3)
      R23=TRSYM(2,3)
      R31=TRSYM(3,1)
      R32=TRSYM(3,2)
      GAMMA=RTD*ATAN2(R32,-R31)
      ALPHA=RTD*ATAN2(R23,R13)
      RETURN
C
C
C  BETA = 0 OR 180  CAN ONLY FIND ALPHA + OR - GAMMA.
13    BETA=0.
      IF (TRSYM(3,3).LT.0.) BETA=180.
      GAMMA=RTD*ATAN2(TRSYM(2,1),TRSYM(2,2))
      ALPHA=0.
      END
C
C
C
      SUBROUTINE DCOSFD(ROT,DIRCOS,KAPPA)
C     ===================================
C******************************************************
C     Given a rotation matrix ROTN, expressing a rotation
C     through an angle KAPPA right-handedly about an axis
C     with direction cosines DIRCOS(I),
C     DCOSFD determines DIRCOS() and KAPPA
C     KAPPA is returned in degrees in the range 0 to 180
C
C     Expression for rotation matrix is (see J & J, "Mathl. Phys.",
C     p. 122):-
C
C  cw + n(1)n(1)(1-cw)    n(1)n(2)(1-cw)-n(3)sw  n(1)n(3)(1-cw)+n(3)sw
C  n(1)n(2)(1-cw)+n(3)sw  cw + n(2)n(2)(1-cw)    n(2)n(3)(1-cw)-n(1)sw
C  n(3)n(1)(1-cw)-n(2)sw  n(3)n(2)(1-cw)+n(1)sw  cw + n(3)n(3)(1-cw)
C
C     where cw = cos(KAPPA), sw = sin(KAPPA),
C     & n(1), n(2), n(3) are the direction cosines.
C
C*******************************************************}
C
      REAL ROT(3,3),DIRCOS(3),S(3),P(3),D(3)
      REAL KAPPA
      REAL DIFF,PI,RTD,TRACE,R2,R3,SINKAP,COSKAP,TERM
      INTEGER NS(3),NP(3),NCS(3),ND(3),KNTRL,J,JMX
      PARAMETER(PI=3.141592654D0, RTD=180./PI)
C
C
      DIFF=1E-04
      KNTRL=0
C     Trace = 1 + 2.cos w
      TRACE=ROT(1,1)+ROT(2,2)+ROT(3,3)
C     S(1)=2.n(1).sin w
C     S(2)=2.n(2).sin w
C     S(3)=2.n(3).sin w
      S(1)=ROT(3,2)-ROT(2,3)
      S(2)=ROT(1,3)-ROT(3,1)
      S(3)=ROT(2,1)-ROT(1,2)
      CALL ORDR3(KNTRL,S,NS)
C--Is biggest S() zero (i.e. is sin w = 0)?
      IF (ABS(S(NS(1))).GT.DIFF) THEN
        R2=S(NS(2))/S(NS(1))
        R3=S(NS(3))/S(NS(1))
        DIRCOS(NS(1))=1./SQRT(1. + R2**2 + R3**2)
        DIRCOS(NS(2))=R2*DIRCOS(NS(1))
        DIRCOS(NS(3))=R3*DIRCOS(NS(1))
        GOTO 123
      ENDIF
C--Calculation when sin w = 0 (esp. when w = 180').
C     P(1)=n(2).n(3).{0(W=0)or2(w=180)}
C     P(2)=n(3).n(1).{0(W=0)or2(w=180)}
C     P(3)=n(1).n(2).{0(W=0)or2(w=180)}
      P(1)=ROT(3,2)
      P(2)=ROT(1,3)
      P(3)=ROT(2,1)
      CALL ORDR3(KNTRL,P,NP)
C--   Is biggest P() zero (i.e. are all off-diag. terms zero)?
      IF (ABS(P(NP(1))).LT.DIFF) THEN
        IF (TRACE.GT.0.) THEN
C--Matrix is unit matrix.
          KAPPA=0.0
          DIRCOS(1)=0.
          DIRCOS(2)=0.
          DIRCOS(3)=1.
          RETURN
        ENDIF
C--Trace -ve, so dyad about x,y or z.
        KAPPA=PI
        DO J=1,3
          D(J)=ROT(J,J)+1.
        ENDDO
        CALL ORDR3(KNTRL,D,ND)
        DIRCOS(ND(1))=1.
        DIRCOS(ND(2))=0.
        DIRCOS(ND(3))=0.
        RETURN
      ENDIF
      IF (ABS(P(NP(2))).LT.DIFF) THEN
C--One d.c. is zero.
        DIRCOS(NP(1))=0.
        DIRCOS(NP(2))=SQRT(MAX(.5*(1.+ROT(NP(2),NP(2))),0.))
        DIRCOS(NP(3))=SQRT(MAX(.5*(1.+ROT(NP(3),NP(3))),0.))
        IF (P(NP(1)).LT.0.) DIRCOS(NP(2))=-DIRCOS(NP(2))
        GOTO 123
      ENDIF
C--All d.c's are nonzero.
C     R2=n(NP(1))/n(NP(2))
C     R3=n(NP(1))/n(NP(3))
      R2=P(NP(2))/P(NP(1))
      R3=P(NP(3))/P(NP(1))
      DIRCOS(NP(2))=1./SQRT(1. + R2**2 + (R2/R3)**2)
      DIRCOS(NP(3))=1./SQRT(1. + R3**2 + (R3/R2)**2)
C--Check: don't take sqrt of negative values
      TERM=1.-DIRCOS(NP(2))**2-DIRCOS(NP(3))**2
      IF (TERM.LT.0.) THEN
        IF (ABS(TERM).GT.DIFF)
     +  CALL CCPERR (1,'Negative value in DCOSFD')
        TERM=0.
      ENDIF
      DIRCOS(NP(1))=SQRT(TERM)
C--Adjust signs of DIRCOS().
      JMX=0
      DO J=1,3
        IF (P(J).GT.0.) THEN
          IF (JMX.GT.0) JMX=-1
          IF (JMX.EQ.0) JMX=J
        ENDIF
      ENDDO
      IF (JMX.GT.0) DIRCOS(JMX)=-DIRCOS(JMX)
      IF (JMX.EQ.0) WRITE(6,1358)
1358  FORMAT(' All P()s. are negative or zero')
C--Given d.c's, calculate KAPPA.
123   KNTRL=1
      CALL ORDR3(KNTRL,DIRCOS,NCS)
C     Find KAPPA.
      SINKAP=ROT(NCS(3),NCS(2))-ROT(NCS(2),NCS(3))
      SINKAP=.5*SINKAP/DIRCOS(NCS(1))
      COSKAP=.5*(TRACE-1.)
      KAPPA=RTD*ATAN2(SINKAP,COSKAP)
C  If kappa negative, negate kappa and invert DIRCOS
      IF (KAPPA.LT.0.) THEN
        KAPPA = -KAPPA
        DIRCOS(1) = -DIRCOS(1)
        DIRCOS(2) = -DIRCOS(2)
        DIRCOS(3) = -DIRCOS(3)
      ENDIF
      END
C
C
C
C
C
      SUBROUTINE ORDR3(KNTRL,Q,NQ)
C     ============================
C
C*************************************************************
C     Find NQ(1) so that |Q(NQ(1)| is biggest Q.
C     If KNTRL=1, make NQ(1),NQ(2),NQ(3) a positive permutn.
C     If KNTRL= anything else,
C     find nos. NQ() so that|Q(NQ(1)|>|Q(NQ(2))|>|Q(NQ(3))|
C
C*************************************************************
      REAL Q(3)
      INTEGER NQ(3),KNTRL,J
C
C     Find no. NQ(1) of Q() with largest |Q()|.
      NQ(1)=1
      IF (ABS(Q(2)).GT.ABS(Q(1))) NQ(1)=2
      IF (ABS(Q(3)).GT.ABS(Q(NQ(1)))) NQ(1)=3
C     Find nos. for other two Q()'s & order so |Q(NQ(2))|>|Q(NQ(3))|
      NQ(2)=MOD(NQ(1),3)+1
      NQ(3)=MOD(NQ(2),3)+1
      IF (KNTRL.NE.1 .AND. ABS(Q(NQ(3))).GT.ABS(Q(NQ(2)))) THEN
        J=NQ(3)
        NQ(3)=NQ(2)
        NQ(2)=J
      ENDIF
      END
C
C
C
      SUBROUTINE POLANG(DC,OMEGA,PHI)
C     ===============================
C
C Find polar angles OMEGA & PHI (degrees) from direction cosines DC
C  OMEGA in range 0 to 180, PHI in range 0 to 360
C
      REAL DC(3), OMEGA, PHI
C
      PARAMETER(PI=3.141592654D0, RTD=180./PI)
C
      OMEGA=RTD*ACOS(DC(3))
      IF (DC(1).EQ.0. .AND. DC(2).EQ.0.) THEN
        PHI=0.
      ELSE
        PHI=RTD*ATAN2(DC(2),DC(1))
      ENDIF
      END
C
C
C
      SUBROUTINE RNGANG(A)
C     ====================
C
C Put angle A into range 0-360 degrees
C
      REAL A
C
1     IF (A.LT.0.) THEN
        A = A + 360.
        GOTO 1
      ENDIF
2     IF (A.GE.360.) THEN
        A = A - 360.
        GOTO 2
      ENDIF
      END
C
C
C
      SUBROUTINE POLMAT(ROTN,ANGLES)
C     ==============================
C
C**************************************************************
C       Sets rotation matrix ROTN expressing a rotation through an
C       angle AKAPPA right-handedly about an axis
C       with polar angles OMEGA, PHI (OMEGA with Z-axis, projection
C       giving angle PHI with X-axis).
C       These angles give direction cosines DIRCOS(I).
C***************************************************************
C
      REAL ROTN(3,3),DIRCOS(3),ANGLES(3)
      PARAMETER(PI=3.141592654D0, DTR=PI/180.)
C
      SNOM=SIN(DTR*ANGLES(1))
      CSOM=COS(DTR*ANGLES(1))
      SNPH=SIN(DTR*ANGLES(2))
      CSPH=COS(DTR*ANGLES(2))
      SNKA=SIN(DTR*ANGLES(3))
      CSKA=COS(DTR*ANGLES(3))
      DIRCOS(1)=SNOM*CSPH
      DIRCOS(2)=SNOM*SNPH
      DIRCOS(3)=CSOM
      DO I=1,3
        DO J=1,3
          K1=6-I-J
          K=K1
          IF (K1.LT.1 .OR. K1.GT.3) K=3
          EPSIJK=((I-J)*(J-K)*(K-I))/2
          ROTN(I,J)=DIRCOS(I)*DIRCOS(J)*(1.-CSKA)-EPSIJK*DIRCOS(K)*SNKA
          IF (I.EQ.J) ROTN(I,J)=ROTN(I,J)+CSKA
        ENDDO
      ENDDO
      END
