C
C     PREP_BULK
C     Copyright (C) 2003 A. Fokine
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C ########################################################################
C
C        Service program to prepare INPUT data for program BULKING 
C (bulk solvent correction in translation search in molecular replacement)
C
C#########################################################################
C A. Fokine, G. Capitani, M.G. Grutter, A. Urzhumtsev
C 25.07.02
C###########



	IMPLICIT NONE



        real resolution                ! resolution

        real ksol, Bsol                ! bulk solvent parameters
                                       ! ( see Jiang and Brunger 
                                       !   J.Mol.Biol 1994, 243, 100-115 )


        real rsolv                     ! parameter for solvent mask calculation
                                       ! ( see Jiang and Brunger 
                                       !   J.Mol.Biol 1994, 243, 100-115 )
        

        real xtrans,ytrans,ztrans      !translation vector to put center mass of
                                       !the molecule at the origin (for AmoRe) 

        real theta1,theta2,theta3      !euler angles (around Z, newY, newZ) to
                                       !rotate molecule such that after rotaion the
                                       !inertia axes be parallel to coord. axes  

        character*200  inp_coord_file         ! name of the file with model

        character*200  inp_F_file             ! name of file with str. factors
                                             ! Amore            

 
        character*200  inp_tabling_list_file  ! name of file with results of 
                                             ! TABLING procedure (AmoRe) 


        character*200  out_F_corrected_file   ! File with str. factors corrected
                                             ! by solvent contribution - result 
                                             ! of this program 

                

        character*200  out_file          ! OUTPUT file with 
                                        ! input parameters
                                        ! for solvent correction procedure


 
C_____________________________________________________________________________

      call ccpfyp
      call ccprcs (6, 'PREP_BULK', '$Date: 2006/11/29 11:35:30 $')

      print *
      print *,'###################################################'
      print *
      print *,'              Program prep_bulk                    '
      print *,'    Preparing data for bulk solvent correction     '
      print *
      print *,'###################################################'
      print *


C      Reading input data

       
       CALL READ_INPUT_DATA(inp_coord_file,inp_F_file,
     * inp_tabling_list_file, out_F_corrected_file,
     * ksol,Bsol,rsolv,
     * xtrans,ytrans,ztrans,theta1,theta2,theta3,
     * resolution,out_file)


C      Write file with input data for program BULKING     

       CALL OUT_PUSK_FILE(inp_coord_file,inp_F_file,
     * out_F_corrected_file,
     * ksol,Bsol,rsolv,
     * xtrans,ytrans,ztrans,theta1,theta2,theta3,
     * resolution,out_file)

       CALL CCPERR(0,'*** NORMAL TERMINATION ***')
       END


C############################################################################




C      ######################################################################
C
C       Subroutine READ_INPUT_DATA: 
C
C       inp_coord_file - name of model coor. file 
C       inp_F_file name of model str. factor file (calculated by AmoRe)
C       inp_tabling_list_file - name of the file with listing of TABLING 
C                               procedure (AmoRe)
C       ksol,Bsol - solvent parameters
C       rsolv - solvent radius
C       trans1,trans2,trans3,th1,th2,th3 translation vector and euler angles 
C                                        to transform model coordinates
C       resolution - resolution  for mask calculation
C       
C       a,b,c,alpha,betta,gamma - cell parameters
C       hsup,ksup,lsup - maximal indexes
C
C
C       inp_coord_file, inp_F_file, inp_tabling_list_file, ksol,Bsol,
C       rsolv, resolution  - will be readen from STANDARD INPUT
C
C       trans1,trans2,trans3,th1,th2,th3,  - will be
C       readen from inp_tabling_list_file
C



       SUBROUTINE READ_INPUT_DATA(inp_coord_file,inp_F_file,
     * inp_tabling_list_file, out_F_corrected_file,
     * ksol,Bsol,rsolv,
     * trans1,trans2,trans3,theta1,theta2,theta3,
     * resolution,out_file)



       IMPLICIT NONE

       character*200  inp_coord_file, inp_F_file
       character*200  inp_tabling_list_file, out_F_corrected_file
       character*200  out_file
       real          ksol,Bsol,rsolv
       real          trans1,trans2,trans3,theta1,theta2,theta3
       real          resolution
  
       character*200 errline
       character*40  string_title
       integer       counter

       integer       lenstr
        
        print *
        print *,'Reading parameters from file prep_bulk.inp'
        
        read (5,'(a)',err=10) inp_coord_file
        print *
        print *,'file with model coordinates: '
        print *,inp_coord_file(1:lenstr(inp_coord_file))

        read (5,'(a)') inp_F_file
        print *
        print *,'file with structure factors calculated by AmoRe:'
        print *,inp_F_file(1:lenstr(inp_F_file))

        read (5,'(a)',err=10) inp_tabling_list_file
        print *
        print *,'File produced by tabling procedure (AmoRe):'
        print *,inp_tabling_list_file(1:lenstr(inp_tabling_list_file))


        read (5,'(a)',err=10) out_F_corrected_file
        print *
        print *,'name of file with corrected structure factors '
        print *, out_F_corrected_file(1:lenstr(out_F_corrected_file))


        read (5,'(a)',err=10) out_file
        print *
        print *,'name of file with instruction for bulking procedure'
        print *,out_file(1:lenstr(out_file))


        read *, ksol
        read *, Bsol
        print *
        print *,'Solvent parameters ksol (in e/A3), Bsol (in A2) ', 
     * 'as defined in Jiang and Brunger J.Mol.Biol 1994, 243, 100-115.'
        print *,'ksol= ',ksol
        print *,'Bsol= ',Bsol 
        
        print *
        read *,rsolv
      print *,'Solvent radius - parameter for solvent mask calculation'
        print *,'rsolv= ',rsolv
 
       
        open(unit=1,file=inp_tabling_list_file,
     * status='old', form='formatted')

        print *
        print *,'reading file ',
     +  inp_tabling_list_file(1:lenstr(inp_tabling_list_file))
        print * 

        counter=0
    
        dowhile (.true.)

         read(1,'(a40)',end=20,err=30) string_title

         if(string_title.eq.'                         Center of Mass:')
     * then
    
          backspace 1 
          read(1,'(40X,3F9.2)',end=20,err=30) trans1,trans2,trans3
          print *, 'model center of mass: '
          print *, trans1,trans2,trans3

          trans1=-trans1
          trans2=-trans2
          trans3=-trans3


          counter=counter+1

         else if(string_title.eq.'                         Rotation     
     * :') then        
          backspace 1 
          read(1,'(40X,3F9.2)',end=20,err=30) theta1,theta2,theta3
          print *,'euler angles for model rotation: '
          print *,theta1,theta2,theta3


          counter=counter+1
         
         else if (string_title.eq.'   Resolution Limit for Interpolation
     *  :') then        

          backspace 1 
          read(1,'(40X,F9.2)',end=20,err=30) resolution
          print *
          print *, 'resolution: '
          print *,resolution

          counter=counter+1


          end if

         end do


20       continue
         close(1)

         RETURN

10       CALL CCPERR(1,'ERROR in program input')
 
         
30       write(errline,'(A,A180)') 
     +        'ERROR: reading file ',inp_tabling_list_file 
         CALL CCPERR(1,errline)
 
        END

C      #################################################################





C      #################################################################


       SUBROUTINE OUT_PUSK_FILE(inp_coord_file,inp_F_file,
     * out_F_corrected_file,
     * ksol,Bsol,rsolv,
     * trans1,trans2,trans3,theta1,theta2,theta3,
     * resolution,out_file)

        
       IMPLICIT NONE

       character*200  inp_coord_file, inp_F_file
       character*200  out_F_corrected_file
       character*200  out_file
       real          ksol,Bsol,rsolv
       real          trans1,trans2,trans3,theta1,theta2,theta3
       real          resolution

       integer lenstr

        open(unit=1,file=out_file,
     * status='unknown', form='formatted')
 
C        print *
C        print *,'writing file ',out_file

        write (1,'(a)') inp_coord_file(1:lenstr(inp_coord_file))
        write (1,'(a)') inp_F_file(1:lenstr(inp_F_file))
        write (1,'(a)') 
     +    out_F_corrected_file(1:lenstr(out_F_corrected_file))
        write (1,*)  resolution
        write (1,*)  ksol
        write (1,*)  Bsol
        write (1,*)  rsolv 
        write (1,*)  trans1,trans2,trans3
        write (1,*)  theta1,theta2,theta3

        close(1)
        print *
        print *,'instructions are written in the file:'
        print *, out_file(1:lenstr(out_file))
       
       END

C###################################################################

         

       

       

        


