C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C**************************************************************************
C
C  GFAC2PDB.F  -  Program to generate a PDB-format file of a set of NOE
C                 and/or H-bond restraints, showing violations and
C                 upper-bound excesses for a user-selected range of
C                 models and residues
C
C     Written by Roman Laskowski, University College, London, July 1996.
C
C     Original version was part of v.3.4.3 of the PROCHECK suite of programs.
C     
C     Subsequent amendments will be labelled by CHECK v.m.n--> and
C     CHECK v.m.n<-- where m.n is the version number corresponding to the
C     change
C
C v.3.5.1 - Amendment to statement in REAPDB giving very odd results under
C           some compiler conditions (as used for the CCP4 suite).
C                                            Roman Laskowski (20 Apr 1998)
C
C v.3.5.1a - remove repeat SAVEs
C                                            Clemens Vonrhein (18 Oct 2001)
C
C--------------------------------------------------------------------------
C
C Compilation and linking (on unix)
C -----------------------
C
C f77 -u -c gfac2pdb.f
C f77 -u -c ps.f
C f77 -o gfac2pdb gfac2pdb.o ps.o
C
C Compilation and linking (on VAX VMS)
C -----------------------
C
C FORT GFAC2PDB
C FORT PS
C LINK GFAC2PDB, PS
C
C--------------------------------------------------------------------------
C     
C  Files
C  -----
C     
C  1  <filename>.new - Input .new file holding the structure to be processed
C  3  <filename>.sdh - Input file, created by PROCHECK containing all
C                      the residue-by-residue G-factor values
C  7  <filename>_gfact.pdb - Output PDB-format file, being the same as
C                            the .new file but with the B-factors of the
C                            atoms adjusted to reflect the corresponding
C                            torsional G-factors
C
C--------------------------------------------------------------------------
C     
C     Subroutine calling tree
C     -----------------------
C     
C     MAIN    --> INITS
C             --> GETPDB
C             --> REAPDB  --> LENSTR
C             --> GETSDH
C             --> WRIOUT  --> LENSTR
C
C--------------------------------------------------------------------------
      
      
      PROGRAM VIOL2
      
      INCLUDE 'gfac2pdb.inc'

C---- Initialise variables
      CALL INITS
      
C---- Request the name of the PDB file
      CALL GETPDB
      IF (IFAIL) GO TO 999

C---- Read in the structure from the PDB file
      CALL REAPDB
      IF (IFAIL) GO TO 999

C---- Read in the G-factors and update the corresponding B-values
      CALL GETSDH
      IF (IFAIL) GO TO 999

C---- Write the PDB file out with replaced temperature factors
      CALL WRIOUT

 999  CONTINUE
      IF (IFAIL) THEN
         PRINT*, '**** Program terminated with error'
      ELSE 
         PRINT*, 'Program complete'
      ENDIF
      END
      
C--------------------------------------------------------------------------
C**************************************************************************
C     
C  SUBROUTINE INITS  -  Initialise variables
C     
C----------------------------------------------------------------------+---
      
      SUBROUTINE INITS
      
      INCLUDE 'gfac2pdb.inc'
      
      CHARACTER*3   AMNAME(NAMINO)
      INTEGER       IAMINO, IATOM

      DATA AMNAME /'ALA', 'ARG', 'ASN', 'ASP', 'CYS', 'GLN', 'GLU',
     -             'GLY', 'HIS', 'ILE', 'LEU', 'LYS', 'MET', 'PHE',
     -             'PRO', 'SER', 'THR', 'TRP', 'TYR', 'VAL' /

C---- Initialise variables
      DO 50, IAMINO = 1, NAMINO
          AMINO(IAMINO) = AMNAME(IAMINO)
 50   CONTINUE
      DO 100, IATOM = 1, MXATOM
          BVALUE(IATOM) = 0.0
 100  CONTINUE
      CHAIN = ' '
      FSTRES = 1
      IFAIL = .FALSE.

      RETURN
      END
      
C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE GETPDB  -  Read in the name out the output file and the
C                        chain
C
C----------------------------------------------------------------------+--- 
 
      SUBROUTINE GETPDB
 
      INCLUDE 'gfac2pdb.inc'
 
      INTEGER       IEND, ISTART, LINE
      LOGICAL       IERROR


C---- Initialise variables
      LINE = 0

C---- Accept name of output file
      PRINT*, 'Enter name of input PDB file'
      READ(*,110,ERR=900) PDBFIL
 110  FORMAT(A)
      PRINT*
      PRINT*

C---- Peel off directory path and extension
      CALL GETNAM(PDBFIL,ISTART,IEND,IERROR)
      IF (IERROR) GO TO 990

C---- Form names of other files that will be required in default directory
      FILNEW = PDBFIL(ISTART:IEND) // '.new'
      FILSDH = PDBFIL(ISTART:IEND) // '.sdh'
      FILOUT = PDBFIL(ISTART:IEND) // '_gfact.pdb'

      GO TO 999

C---- Fatal errors
 900  CONTINUE
      PRINT*, '**** ERROR. Data error in entered filename'
      GO TO 990

990   CONTINUE
      IFAIL = .TRUE.
 
999   CONTINUE
      RETURN
      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE REAPDB  -  Read through appropriate .new file and
C                        accumulate coords to calculate mean positions
C
C----------------------------------------------------------------------+---

      SUBROUTINE REAPDB

      INCLUDE 'gfac2pdb.inc'

      CHARACTER*1   INCHN
      CHARACTER*3   RESDUE
      CHARACTER*4   ATNAME
      CHARACTER*5   SEQNO
      CHARACTER*6   IDENT, INSEQ, RSLAST
      CHARACTER*13  ATMKEY
      CHARACTER*80  IREC
      INTEGER       I, IATOM, IERR, LENSTR, LINE, RNO
      LOGICAL       ENDFIL
      REAL          COORDS(3)

C---- Initialise variables
      ENDFIL = .FALSE.
      IFAIL = .FALSE.
      LINE = 0
      IATOM = 0
      NRESID = 0
      RSLAST = ' '

C---- Open the PDB file
      PRINT*, 'Reading in structure from file, ', 
     -    FILNEW(1:LENSTR(FILNEW)), ' ...'
      PRINT*
      OPEN(UNIT=1, FILE=FILNEW, STATUS='OLD', FORM='FORMATTED',
     -     ACCESS='SEQUENTIAL',
CVAX     -     CARRIAGECONTROL = 'LIST', READONLY,
     -     ERR=900)

C---- Loop through the PDB file, picking out all the ATOM and HETATM
C     records
200   CONTINUE
          READ(1,120,END=800,ERR=904) IREC
 120      FORMAT(A)
          LINE = LINE + 1
          IDENT = IREC(1:6)

C----     Determine whether this record is required for processing
          IF (IDENT.EQ.'ATOM  ' .OR. IDENT.EQ.'HETATM') THEN

C----         Pick off atom name, residue name, chain and sequence number
              ATNAME = IREC(13:16)
              RESDUE = IREC(18:20)
              INCHN = IREC(22:22)
              INSEQ = IREC(22:27)
              SEQNO = IREC(23:27)
              READ(INSEQ,240,IOSTAT=IERR) RNO
 240          FORMAT(I5,1X)
              IF (IERR.NE.0) RNO = 0

C----         Retrieve the atomic coordinates
              READ(IREC,280,ERR=902) (COORDS(I), I = 1, 3)
280           FORMAT(30X,3F8.0)

C----         Form the unique, identifying key for this atom
              ATMKEY = INSEQ // RESDUE // ATNAME

C----         Increment atom-store
              IATOM = IATOM + 1
              IF (IATOM.GT.MXATOM) GO TO 906

C----         Save the ATOM ID and coordinates
              ATOMID(IATOM) = ATMKEY
              ATMXYZ(1,IATOM) = COORDS(1)
              ATMXYZ(2,IATOM) = COORDS(2)
              ATMXYZ(3,IATOM) = COORDS(3)
              RESNUM(IATOM) = RNO
CHECK v.3.5.1-->
C              RESNO(IATOM) = INCHN // SEQNO
              RESNO(IATOM)(1:1) = INCHN 
              RESNO(IATOM)(2:) = SEQNO
CHECK v.3.5.1<--

C----         If residue has changes, then store residue details and
C             starting atom number
              IF (RESNO(IATOM).NE.RSLAST) THEN
                  IF (RESDUE.NE.'HOH') THEN
                      NRESID = NRESID + 1
                      IF (NRESID.GT.MXRES) GO TO 908
                      RESDET(NRESID) = RESNO(IATOM)
                      FSTATM(NRESID) = IATOM
                  ENDIF
                  RSLAST = RESNO(IATOM)
              ENDIF

C----     If ENDMDL record, or second MODEL record, encountered then read
C         no more
          ELSE IF (IDENT.EQ.'ENDMDL' .OR.
     -        (IDENT.EQ.'MODEL ' .AND. IATOM.GT.0)) THEN
              ENDFIL = .TRUE.
          ENDIF
      IF (.NOT.ENDFIL) GO TO 200

C---- End of Brookhaven file reached
 800  CONTINUE

C---- Show statistics on the models read in
      NATOMS = IATOM
      IF (NATOMS.EQ.0) GO TO 910
      PRINT*
      PRINT *, 'Total no. of residues stored:    ', NRESID
      PRINT *, 'Total no. of atoms stored:       ', NATOMS
      PRINT*

      GO TO 999

C---- Fatal errors
 900  CONTINUE
      PRINT*, '**** ERROR. Unable to open input PDB file:'
      PRINT*, '****        ', FILNEW(1:LENSTR(FILNEW))
      GO TO 990

 902  CONTINUE
      PRINT*, '**** Error in coords for atom: ', ATNAME, '-', RESDUE,
     -    '-', INSEQ
      GO TO 200

 904  CONTINUE
      PRINT*, '**** Data error reading file:  [',
     -    FILNEW(1:LENSTR(FILNEW)), ']'
      PRINT*, '    Line: ', LINE
      GO TO 990

 906  CONTINUE
      PRINT*, '**** Maximum number of atoms per protein exceeded:',
     -    MXATOM
      GO TO 990

 908  CONTINUE
      PRINT*, '**** Maximum number of residues per protein exceeded:',
     -    MXRES
      GO TO 990

 910  CONTINUE
      PRINT*, '**** No atoms stored from input PDB file'
      GO TO 990

 990  CONTINUE
      IFAIL = .TRUE.

 999  CONTINUE
      RETURN
      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE GETSDH  -  Read through the .sdh G-factors file
C
C----------------------------------------------------------------------+--- 

      SUBROUTINE GETSDH

      INCLUDE 'gfac2pdb.inc'
 
      CHARACTER*1   INCHN
      CHARACTER*2   ATNAME, ATUPD(2,5)
      CHARACTER*3   INRES
      CHARACTER*5   INSEQ
      CHARACTER*6   RESKEY
      CHARACTER*120 IREC
      INTEGER       IATOM, IDISTR, IEND, IRES, IRESID, ISTART, IVALUE,
     -              LENSTR, LINE
      LOGICAL       WANTED
      REAL          BFACT, MAXB, MAXG, MINB, MING, VALUE(NDISTR)

      DATA ATUPD / 'N ', 'CA',
     -             'CB', '*G',
     -             'CB', '  ',
     -             '*D', '*E',
     -             'C ', '  ' /
      DATA MINB, MAXB / 0.0, 80.0 /
      DATA MING, MAXG / -4.0, 1.0 /

C---- Initialise variables
      LINE = 0

C---- Open the file, <filename>.sdh
      PRINT*, 'Reading in G-factors from file, ', 
     -    FILSDH(1:LENSTR(FILSDH)), ' ...'
      PRINT*
      OPEN(UNIT=3, FILE=FILSDH, STATUS='OLD', FORM='FORMATTED',
     -    ACCESS='SEQUENTIAL',
CVAX     -    READONLY,
     -    ERR=900)

C---- Read through the records until get to the final line with mean-values
 100  CONTINUE
          READ(3,120,END=500,ERR=904) IREC
 120      FORMAT(A)
          LINE = LINE + 1

C----     If it is the "Means" record, store the overall stereochemical
C         R-factor
          IF (IREC(1:5).EQ.'Means') THEN

C----         Extract the G-factors
              READ(IREC,140,ERR=904) (GFACTR(IDISTR),
     -            IDISTR = 1, NDISTR)
 140          FORMAT(17X,10F9.4)
              SFACT = GFACTR(NDISTR)

C----     Otherwise read in log-odds values to determine the maximum
          ELSE
              READ(IREC,180,ERR=904) INCHN, INRES, INSEQ,
     -            (VALUE(IVALUE), IVALUE = 1, NDISTR)
 180          FORMAT(6X,1X,A1,1X,A3,A5,10F9.4)

C----         Only process this residue if it belongs to the required
C             chain
              IF (CHAIN.EQ.' ' .OR. INCHN.EQ.CHAIN) THEN

C----             Loop through all the residues until reach the current
C                 one
                  IRESID = 0
                  RESKEY = INCHN // INSEQ
                  DO 200, IRES = 1, NRESID
                      IF (RESDET(IRES).EQ.RESKEY) THEN
                          IRESID = IRES
                      ENDIF
 200              CONTINUE

C----             If have found the appropriate residue, then find
C                 its start- and end-atoms
                  IF (IRESID.GT.0) THEN
                      ISTART = FSTATM(IRESID)
                      IF (IRESID.EQ.NRESID) THEN
                          IEND = NATOMS
                      ELSE
                          IEND = FSTATM(IRESID + 1) - 1
                      ENDIF

C----                 For each given G-factor, search for the corresponding
C                     atom and update its B-value
                      DO 400, IDISTR = 1, 5

C----                     Calculate a pseudo B-value for this G-factor
C                         value
                          IF (VALUE(IDISTR).LT.900.0) THEN
                              BFACT = (VALUE(IDISTR) - MING)
     -                            / (MAXG - MING)
                              IF (BFACT.LT.0.0) BFACT = 0.0
                              IF (BFACT.GT.1.0) BFACT = 1.0
                              BFACT = MINB
     -                            + (1.0 - BFACT) * (MAXB - MINB)

C----                         Loop to find this atom in this residue
                              DO 300, IATOM = ISTART, IEND

C----                             Get the atom's name and determine
C                                 whether it matches the search template
                                  WANTED = .FALSE.
                                  ATNAME = ATOMID(IATOM)(11:12)
                                  IF ((ATNAME(1:1).EQ.
     -                                ATUPD(1,IDISTR)(1:1) .OR.
     -                                ATUPD(1,IDISTR)(1:1).EQ.'*') .AND.
     -                                (ATNAME(2:2).EQ.
     -                                ATUPD(1,IDISTR)(2:2) .OR.
     -                                ATUPD(1,IDISTR)(2:2).EQ.'*')) THEN
                                      WANTED = .TRUE.
                                  ENDIF
                                  IF ((ATNAME(1:1).EQ.
     -                                ATUPD(2,IDISTR)(1:1) .OR.
     -                                ATUPD(2,IDISTR)(1:1).EQ.'*') .AND.
     -                                (ATNAME(2:2).EQ.
     -                                ATUPD(2,IDISTR)(2:2) .OR.
     -                                ATUPD(2,IDISTR)(2:2).EQ.'*')) THEN
                                      WANTED = .TRUE.
                                  ENDIF

C----                             If this atom matches the template, then
C                                 update its B-value
                                  IF (WANTED) THEN
                                      BVALUE(IATOM) = BFACT
                                  ENDIF
 300                          CONTINUE
                          ENDIF
 400                  CONTINUE

                  ELSE
                      PRINT*, '*** Residue not found in PDB file:',
     -                    INCHN, ' ', INRES, INSEQ
                  ENDIF


C----             Update the appropriate residue's atomic B-factors
              ENDIF
          ENDIF

C---- Loop back for next record in file
      GO TO 100

C---- End of file reached
 500  CONTINUE

      GO TO 999
 
C---- Fatal errors
 900  CONTINUE
      PRINT*, '*** ERROR. Unable to open .sdh file'
      GO TO 990
 
 904  CONTINUE
      PRINT*, '*** ERROR. Error reading .sdh file at line:', LINE + 1
      GO TO 990
 
990   CONTINUE
      IFAIL = .TRUE.
 
999   CONTINUE
      RETURN
      END

C--------------------------------------------------------------------------
C**************************************************************************
C
C  SUBROUTINE WRIOUT  -  Read through .new file again, writing out all the
C                        atom records with their altered B-factors
C
C----------------------------------------------------------------------+---

      SUBROUTINE WRIOUT

      INCLUDE 'gfac2pdb.inc'

      CHARACTER*1   INCHN
      CHARACTER*3   RESDUE
      CHARACTER*4   ATNAME
      CHARACTER*5   SEQNO
      CHARACTER*6   IDENT, INSEQ
      CHARACTER*13  ATMKEY
      CHARACTER*80  IREC
      INTEGER       IATOM, LENSTR, LINE
      LOGICAL       ENDFIL

C---- Initialise variables
      ENDFIL = .FALSE.
      IFAIL = .FALSE.
      LINE = 0
      IATOM = 0

C---- Rewind input .new file
      REWIND(1)

C---- Open the output PDB file
      PRINT*, 'Writing modified B-factors to file, ', 
     -    FILOUT(1:LENSTR(FILOUT)), ' ...'
      PRINT*
      OPEN(UNIT=7, FILE=FILOUT, STATUS='UNKNOWN', FORM='FORMATTED',
     -     ACCESS='SEQUENTIAL',
CVAX     -     CARRIAGECONTROL = 'LIST',
     -     ERR=900)

C---- Loop through the PDB file, picking out all the ATOM and HETATM
C     records
200   CONTINUE
          READ(1,120,END=800,ERR=904) IREC
 120      FORMAT(A)
          LINE = LINE + 1
          IDENT = IREC(1:6)

C----     Determine whether this record is required for processing
          IF ((IDENT.EQ.'ATOM  ' .OR. IDENT.EQ.'HETATM') .AND.
     -        .NOT.ENDFIL) THEN

C----         Increment atom-count
              IATOM = IATOM + 1

C----         Pick off atom name, residue name, chain and sequence number
              ATNAME = IREC(13:16)
              RESDUE = IREC(18:20)
              INCHN = IREC(22:22)
              INSEQ = IREC(22:27)
              SEQNO = IREC(23:27)

C----         Form the unique, identifying key for this atom
              ATMKEY = INSEQ // RESDUE // ATNAME

C----         Check that details agree
              IF (ATMKEY.EQ.ATOMID(IATOM)) THEN

C----             Replace existing B-factor with computed B-factor
C                 representing the corresponding G-factor
                  WRITE(IREC(61:66),160) BVALUE(IATOM)
 160              FORMAT(F6.2)

C----             Write out the modified record
                  WRITE(7,120) IREC(1:LENSTR(IREC))

C----         If details don't agree, print error message
              ELSE
                  PRINT*, '*** WARNING. Atom-order in file has changed',
     -                '. Could be program error!'
              ENDIF

C----     If an ENDMDL record, or second MODEL record, encountered then
C         don't process any more ATOM and HETATM records
          ELSE IF (IDENT.EQ.'ENDMDL' .OR.
     -        (IDENT.EQ.'MODEL ' .AND. IATOM.GT.0)) THEN
              ENDFIL = .TRUE.
              WRITE(7,120) IREC(1:LENSTR(IREC))

C----     If not an ATOM or HETATM record, then write out as it stands
          ELSE
              WRITE(7,120) IREC(1:LENSTR(IREC))
          ENDIF

C---- Loop back for next record
      GO TO 200

C---- End of Brookhaven file reached
 800  CONTINUE

      GO TO 999

C---- Fatal errors
 900  CONTINUE
      PRINT*, '**** ERROR. Unable to open input PDB file:'
      PRINT*, '****        ', FILOUT(1:LENSTR(FILOUT))
      GO TO 990

 904  CONTINUE
      PRINT*, '**** Data error reading file:  [',
     -    FILNEW(1:LENSTR(FILNEW)), ']'
      PRINT*, '    Line: ', LINE
      GO TO 990

 990  CONTINUE
      IFAIL = .TRUE.

 999  CONTINUE
      RETURN
      END

C--------------------------------------------------------------------------
