C
C     REBATCH
C     Copyright (C) 1994 Phil Evans
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
c  REBATCH
c
c    Renumber batches in unmerged HKL file
c
      program rebatch
c     ===============
c
c Input commands:
c
c-> BATCH  <batch_range | batch_list | ALL >  
c          ADD <Nadd> | START <Nstart> [STEP <Nstep>]| REJECT
c
c  Define batch number editing for either a range of batches (b1 TO b2),
c     a list of batches ( b1 b2 b3 . . ), or ALL [default]
c  Editing options are
c     ADD <Nadd>        add Nadd to the batch number for all batches specified
c                       Note that Nadd may be negative, but the resulting
c                       batch numbers must all be .gt. 0
c     START <Nstart>    renumber all batches matching the specification
c                       serially from Nstart with step Nstep (default step +1)
c                       eg if the file contains batches
c                          100,  101,  102,  110,  111, then the command
c                       BATCH 1 TO 200 START 1000 will renumber them to
c                         1000, 1001, 1002, 1003, 1004
c        STEP <Nstep>   specify increment for renumbering, default = +1
c     REJECT            reject these batches
c     INCLUDE           include these batches (with their original numbers):
c                       the first INCLUDE command
c                       will reject all other batches
C     MAXWIDTH          to reject refls with ugly width ...
C     PNAME <projectname> XNAME <crystalname> DNAME <datasetname> 
C                       Set or reset project/crystal/dataset names
C                       for these batches. 
C
c
c-> TITLE  new title
c
c-> HISTORY new history record
c
c-> DETECTOR <xmin> <xmax> <ymin> <ymax>
c           limits of detector coordinates XDET, YDET. These are
c           needed for the detector scaling options in Scala
c
c-> CELL   a b c [alpha beta gamma]
c         Unit cell for file & datasets, to override that from batch headers
c-> WAVELENGTH
c         Wavelength for file & datasets, to override that from batch headers
c-> END optional end
c
      implicit none
c
c     ..
c     .. Parameters ..
      integer maxbat,MBLENG,MBLINT,MBLREA
      parameter (maxbat=5000,MBLENG=185,MBLINT=29,MBLREA=156)
      integer maxset
      parameter (maxset=200)
c     ..
c     .. Variables in Common ..
      integer lunout, ncmsym, ncbatc, ncwidt
      character outitl*70, histry*80
      INTEGER NWORDS,NINTGR,NREALS,IORTYP,LCELL,MISFLG,
     .     JUMPAX,NCRYST,LCRFLG,LDTYPE,JSCAXS,NBSCAL,NGONAX,LBMFLG,
     $     NDET,LBSETID,INTPAD
      REAL   CELL,UMATR,PHXYZ,CRYDAT,DATUM,
     $     PHISTT,PHIEND,PHIRANGE,SCANAX,TIME1,TIME2,GONPAD,
     $     BSCALE,BBFAC,SDSCAL,SDBB,BATPAD,E1,E2,E3,
     $     SOURCE,S0,BEMDAT,DX1,THETA1,DETLM1,DX2,THETA2,DETLM2,DETPAD
      CHARACTER BTITLE*70, GONLAB*8
c     ..
c     .. Common Blocks ..
      common /column/ ncmsym, ncbatc, ncwidt
      common /inout/ lunout
      common /outcoc/ outitl, histry
c  Data for each batch in input file
      common /nbtchc/ bcell(6,maxbat),bwavel(maxbat),idsetb(maxbat),
     $   nbtall,ibtall(maxbat),
     +   newbat(maxbat),maxwid(maxbat)
      real bcell, bwavel, maxwid
      integer idsetb, nbtall, ibtall, newbat
c
c  Project/Dataset names for each batch
      character*64 pname(maxbat),xname(maxbat),dname(maxbat)
      common /pnamdnam/ pname, xname, dname
c  List of datasets
      common /datasets/ dcell(6,maxset), dwavel(maxset), 
     $   isetids(maxset),ndatasets, ldatasets
      real dcell, dwavel
      integer isetids, ndatasets
      logical ldatasets
      common /datasetc/ dspname(maxset),dsxname(maxset),dsdname(maxset)
      character*64 dspname, dsxname, dsdname
c
c
      common /detctr/ detlim(2,2), ldtlim
      real detlim
      logical ldtlim
      COMMON /CORIEN/BTITLE,GONLAB(3)
      COMMON /ORIENT/ NWORDS,NINTGR,NREALS,IORTYP,LCELL(6),MISFLG,
     .     JUMPAX,NCRYST,LCRFLG,LDTYPE,JSCAXS,NBSCAL,NGONAX,LBMFLG,
     $     NDET,LBSETID,INTPAD(8),
     +     CELL(6),UMATR(3,3),PHXYZ(3,2),CRYDAT(12),DATUM(3),
     $     PHISTT,PHIEND,SCANAX(3),TIME1,TIME2,
     $     BSCALE,BBFAC,SDSCAL,SDBB,PHIRANGE,BATPAD(11),
     $     E1(3),E2(3),E3(3),
     $     GONPAD(12),SOURCE(3),S0(3),BEMDAT(25),     
     $     DX1,THETA1,DETLM1(2,2),DX2,THETA2,DETLM2(2,2),DETPAD(33)
c     
      call ccpfyp
      call ccp4h_init()
      call ccp4h_header('REBATCH','rebatch',2)
      call ccprcs (6, 'REBATCH', '$Date: 2008/03/03 10:13:45 $')
      call mtzini
      call ccp4h_summary_beg()
      call ccp4h_toc_beg()
      call ccp4h_toc_ent('Command Input','#command')
      call ccp4h_toc_ent('Input MTZ File','#inp')
      call ccp4h_toc_ent('Output File','#out')
      call ccp4h_toc_end()
      call ccp4h_rule()
      call ccp4h_header('Command Input','command',3)
      call ccp4h_link('TITLE    ','rebatch.html#title')
      call ccp4h_link('BATCH ','rebatch.html#batch')
      call ccp4h_link('HISTORY ','rebatch.html#history')
      call ccp4h_link('DETECTOR  ','rebatch.html#detector')
      call ccp4h_link('END','rebatch.html#end')
      call ccp4h_summary_end()

c
      lunout = 6
      
c Open input file & get all batch numbers
      call setupf
c
c Read commands
      call keyin
c
c Open output file
      call opnopf
c Copy all reflections
      call cpyref
c
      call ccperr(0,' *** Normal termination ***')
      end
c
c
c
      subroutine cpyref
c     =================
c
c  Copy all reflections, altering batch numbers as required
c
      implicit none
c
      integer maxcol
      parameter (maxcol=30)
c
c+++ column.h +++
      common /column/ ncmsym, ncbatc, ncwidt
      integer ncmsym, ncbatc, ncwidt
      save /column/
c--- column.h ---
c+++ nbtchc.h +++
c  nbtall         number of batches in input file
c  ibtall         input batch numbers for each batch
c  newbat         output batch number for each batch, = 0 to reject
c  maxwid         for each input batch, maximum width
      integer maxbat
      parameter (maxbat = 5000)
c  Data for each batch in input file
      common /nbtchc/ bcell(6,maxbat),bwavel(maxbat),idsetb(maxbat),
     $   nbtall,ibtall(maxbat),
     +   newbat(maxbat),maxwid(maxbat)
      real bcell, bwavel, maxwid
      integer idsetb, nbtall, ibtall, newbat
      save /nbtchc/
c--- nbtchc.h ---
      common /inout/ lunout
      integer lunout
      save /inout/
c
      real s, data(maxcol)
      logical eof, logmss(maxcol)
c
      integer nref, nrej, nrejmaxw, ibatch, j
c
      integer ccp4_hash_lookup
      external ccp4_hash_lookup, ccperr, lrrefl, lrrefm, lwclos
c
      nref = 0
      nrej = 0
      nrejmaxw = 0
c
 100  call lrrefl(1, s, data, eof)
      if (.not. eof) then
         call lrrefm(1,logmss)
         if (logmss(ncbatc)) then
           call ccperr(1,
     $      ' *** ERROR: batch number was found to be equal to MNF ***')
         else
           ibatch = nint(data(ncbatc))
         endif

         j = ccp4_hash_lookup(ibatch)
         if (ibtall(j) .ne. ibatch) then
            write (lunout, '(/a,i8,a,i8/)')
     $           ' *** Wrong batch! batch number in file is', ibatch,
     $           ', batch number from internal lookup ',ibtall(j)
            call ccperr(1,' *** Crazy batch number ***')
         endif
         if (newbat(j) .ne. 0) then
            data(ncbatc) = newbat(j)
            if (ncwidt.gt.0)then
              if (data(ncwidt).lt.maxwid(j)) then
                 call lwrefl(1, data)
                 nref = nref+1
              else
                 nrejmaxw = nrejmaxw + 1
              endif       
            else
              call lwrefl(1,data)
              nref = nref + 1
            endif
         else
            nrej = nrej + 1
         endif
        go to 100
      endif
c
c  Finished
      call lwclos(1,1)
c
c    
      call ccp4h_summary_beg()
      write (lunout, 6100) nref, nrej, nrejmaxw
 6100 format(/5x, i8,' reflections written to output file'/
     $        5x, i8,' reflections in rejected batches not written'/
     $        5x, i8,' reflections rejected for too big WIDTH'/)
      call ccp4h_summary_end()
      return
      end
C
C
C
      subroutine setupf
C     =================
C
C---- Open main  i/o files,
C     get column assignments from header labels
C
      implicit none
C
C Parameters
C  MAXCOL   maximum number of columns in file
C  NCOLI    maximum number of assigned columns in file
      integer ncoli
      parameter (ncoli=6)
C
C
C     ..
C     .. Local Scalars ..
      integer i,iprint,ifail
C     ..
C     .. Local arrays ..
      integer kcol(ncoli)
      character look(ncoli)*30, cltype(ncoli)*1
C
      external ccp4_hash_setup
C     ..
C     .. Common blocks ..
c+++ column.h +++
      common /column/ ncmsym, ncbatc, ncwidt
      integer ncmsym, ncbatc, ncwidt
      save /column/
c--- column.h ---
c+++ nbtchc.h +++
c  nbtall         number of batches in input file
c  ibtall         input batch numbers for each batch
c  newbat         output batch number for each batch, = 0 to reject
      integer maxbat
      parameter (maxbat = 5000)
c  Data for each batch in input file
      common /nbtchc/ bcell(6,maxbat),bwavel(maxbat),idsetb(maxbat),
     $   nbtall,ibtall(maxbat),
     +   newbat(maxbat),maxwid(maxbat)
      real bcell, bwavel, maxwid
      integer idsetb, nbtall, ibtall, newbat
      save /nbtchc/
c--- nbtchc.h ---
c  List of datasets
      integer maxset
      parameter (maxset=200)
      common /datasets/ dcell(6,maxset), dwavel(maxset), 
     $   isetids(maxset),ndatasets, ldatasets
      real dcell, dwavel
      integer isetids, ndatasets
      logical ldatasets
      common /datasetc/ dspname(maxset),dsxname(maxset),dsdname(maxset)
      character*64 dspname, dsxname, dsdname
      save /datasets/, /datasetc/
C     ..
C     .. Data statements ..
C
C---- Column labels required from file
C
      data look/'H','K','L','M/ISYM','BATCH','WIDTH'/
C
C
C---- Compulsory/optional flags for each column
      data kcol/5*-1,1/
C
C---- Column types
      data cltype/'H','H','H','Y','B','R'/
C     ..
C
      call ccp4h_rule()
      call ccp4h_summary_beg()
      call ccp4h_header('Input MTZ File','inp',3)
      call ccp4h_summary_end()
c
C---- Open input file
C
      iprint = 1
      ifail = 0
C          ******************************
      call lropen(1,'HKLIN',iprint,ifail)
C          ******************************
      if (ifail .ne. 0) then
         call ccperr(1,'*** Failed to open HKLIN file ***')
      endif
C
C---- Read header stuff
C
C          *************
      call lrassn(1,look,ncoli,kcol,cltype)
C          *************
C
C---- Read all batch numbers into array
C          **********************
      call lrbats(1,nbtall,ibtall)
C          **********************
C  Set up lookup table for all batchs
      do 10, i = 1, nbtall
         call ccp4_hash_setup(ibtall(i), i)
 10   continue
C
c Set all batches to include unchanged
      do 1, i=1,nbtall
         newbat(i) = ibtall(i)
c Store dataset ID numbers for each batch
         call lrbsetid(1,ibtall(i),idsetb(i))
 1    continue
C
C---- Store column positions
C
      ncmsym = kcol(4)
      ncbatc = kcol(5)
      ncwidt = kcol(6)

c     Notify space available
      ndatasets = maxset
c---- Get list of all datasets in file
      call lridx(1, dspname, dsxname, dsdname, isetids, 
     +      dcell, dwavel, ndatasets)
c ldatasets = .true. if new dataset information should be written to file
c This will be done if
c  (a) dataset editting information is given explicitly
c      (ie BATCH ... PNAME DNAME or CELL or WAVELENGTH)
c or
c  (b) input information is incomplete (cell, wavelength),
c      & can be completed by copying information from batch headers
c
c  Default don't copy
      ldatasets = .false.
c  Check for incomplete information
      do i = 1, ndatasets
         if (dcell(1,i) .le. 0.0001) then
            ldatasets = .true.
         endif
         if (dwavel(i) .le. 0.0001) then
            ldatasets = .true.
         endif
      enddo
c
      return
      end

c
c
c
      subroutine opnopf
c     =================
c
c Open output file
c
      implicit none

      integer maxset
      parameter (maxset=200)
      integer maxbat
      parameter (maxbat = 5000)
c
c+++ column.h +++
      common /column/ ncmsym, ncbatc, ncwidt
      integer ncmsym, ncbatc, ncwidt
      save /column/
c--- column.h ---
c+++ outcon.h +++
c
c     outitl    title for output file, if non-blank replaces input title
c     histry    new history record to add to file (if non-blank)
c
      common /outcoc/ outitl, histry
      character outitl*70, histry*80
c
      save /outcoc/
c--- outcon.h ---
c
      character*64 pname(maxbat),xname(maxbat),dname(maxbat)
      common /pnamdnam/ pname, xname, dname
      save /pnamdnam/
c
c  Data for each batch in input file
      common /nbtchc/ bcell(6,maxbat),bwavel(maxbat),idsetb(maxbat),
     $   nbtall,ibtall(maxbat),
     +   newbat(maxbat),maxwid(maxbat)
      real bcell, bwavel, maxwid
      integer idsetb, nbtall, ibtall, newbat
      save /nbtchc/
      common /inout/ lunout
      integer lunout
c  List of datasets
      common /datasets/ dcell(6,maxset), dwavel(maxset), 
     $   isetids(maxset),ndatasets, ldatasets
      real dcell, dwavel
      integer isetids, ndatasets
      logical ldatasets
      common /datasetc/ dspname(maxset),dsxname(maxset),dsdname(maxset)
      character*64 dspname, dsxname, dsdname
      save /datasets/, /datasetc/
c Global cell & wavelength
      common /wvlcll/ gcell(6), glambda, lgcell, lglambda
      real gcell, glambda
      logical lgcell, lglambda
      save /wvlcll/
c
      character line*80, date*8, time*8
      integer i
c
c Information about newly-defined datasets
      integer j, k
      integer nnewds, nbtchds(maxset), jbtchds(maxset, maxbat),
     $   joldds(maxset)
      real dscell(6,maxset), dslambda(maxset)
      character*64 newpnam(maxset),newxnam(maxset),newdnam(maxset)
      logical lfail
c
      integer lenstr
      external lenstr
c
c Limits for checks on cell lengths, cell angles, wavelength
      real cllim, calim, wvllim
      data cllim, calim, wvllim/0.005, 0.02, 0.0001/ 
c
c
      call ccp4h_rule()
      call ccp4h_summary_beg()
      call ccp4h_header('Output File','out',3)
      call ccp4h_summary_end()
c
      call lwopen(1, 'HKLOUT')
c
      call ccpdat(date)
      call utime(time)
      write (line, '(a,a,a,a)') 'REBATCH: run at ', time,' on ', date
      call lwhist(1, line, 1)
c
c Optional addition of new history record
      if (lenstr(histry) .gt. 0) then
         call lwhist(1, histry, 1)
      endif
c
c Replace title if given
      if (lenstr(outitl) .gt. 0) then
         call lwtitl(1, outitl, 0)
      endif
c
c Reset cell if given
      if (lgcell) then
         call lwcell(1, gcell)
      endif
c  Copy batch headers, renumbering as appropriate
c  but don't do dataset information until after all datasets are set up
      call cpybth
c
c Write out new dataset information, if it has changed
      if (ldatasets) then
         lfail = .false.
c Collect together all information about datasets
c This is quite complicated because we might have to extract dataset
c cell & wavelength information from the batch headers, in which case
c we need to check that they are consistent
         nnewds = 0
         do i = 1, nbtall
c  Don't bother about omitted batches
            if (newbat(i) .ne. 0) then
c  has this batch been assigned explicitly to a "new" dataset?
               k = 0
               if (pname(i) .eq. ' ' .or. xname(i) .eq. ' ' .or. 
     +                                    dname(i) .eq. ' ') then
c  this batch not explicitly assigned, so check which of the original
c  datasets it is in, & set pname/xname/dname
c
c Find dataset ID for this batch, if known
c (from input file: this may change later)
                  do j = 1, ndatasets
                     if (idsetb(i) .eq. isetids(j)) then
                        pname(i) = dspname(j)
                        xname(i) = dsxname(j)
                        dname(i) = dsdname(j)
                        k = j
                        go to 10
                     endif
                  enddo
c     Not found, no dataset defined for this batch, what to do? Leave blank
                  j = -1
                  idsetb(i) = 0
                  lfail = .true.
                  go to 100
               endif
 10            continue
               if (nnewds .gt. 0) then
c    is this the same as one we've already found?
c    NB check is only on xname/dname since these define the dataset
                  do j = 1, nnewds
                     if (xname(i) .eq. newxnam(j) .and.
     $                  dname(i) .eq. newdnam(j)) then
                        go to 20
                     endif
                  enddo
               endif
c     not yet found
               nnewds = nnewds + 1
               j = nnewds
               newpnam(j) = pname(i)
               newxnam(j) = xname(i)
               newdnam(j) = dname(i)
               nbtchds(j) = 0
c   pointer to old dataset number, = 0 if none
               joldds(j) = k
c
 20            nbtchds(j) = nbtchds(j) + 1
c      list all batches belonging to this dataset
               jbtchds(j,  nbtchds(j)) = i
            endif
 100        continue
         enddo
         if (lfail) then
            write (lunout, '(/a/)')
     $      '**** Some batches are not assigned to a valid dataset ****'
            call ccperr(1,'*** Invalid dataset ***')
         endif
c Now we have a list of datasets, old & new, with a list of batches
c for each set
c We have the names of each set, but we need to add cell & wavelength
c information
c
c-  Cell
         if (lgcell) then
c Global cell set, so store for each dataset
            do j = 1, nnewds
               call ccpmvr(dscell(1,j), gcell, 6)
            enddo
         else
            do j = 1, nnewds
c  Use input dataset cell if present
               if (joldds(j) .gt. 0 .and.
     $            dcell(1,joldds(j)) .gt. 0.05) then
                  call ccpmvr(dscell(1,j), dcell(1,joldds(j)), 6)
               else
c  otherwise we have to extract a suitable cell from all the batches 
c Too complicated to average, so just check
                  call ccpmvr(dscell(1,j), bcell(1,jbtchds(j, 1)), 6)
                  if (nbtchds(j) .gt. 1) then
                     lfail = .false. 
                     do k = 2, nbtchds(j)
                        do i = 1,3
                           if (abs(dscell(i,j) - bcell(i,jbtchds(j, k)))
     $                        .gt. cllim) lfail = .true.
                        enddo
                        do i = 4,6
                           if (abs(dscell(i,j) - bcell(i,jbtchds(j, k)))
     $                        .gt. calim) lfail = .true.
                        enddo
                     enddo
                     if (lfail) then
                        write (lunout, '(/a/a/a/a/)')
     $                '=== WARNING: not all cell dimensions in dataset',
     $                     newxnam(j), newdnam(j),' are the same ==='
                     endif
                  endif
               endif
            enddo
         endif
c-
c-  Wavelength
         if (lglambda) then
c Global wavelength set, so store for each dataset
            do j = 1, nnewds
               dslambda(j) = glambda
            enddo
         else
            do j = 1, nnewds
c  Use input dataset cell if present
               if (joldds(j) .gt. 0 .and.
     $            dwavel(joldds(j)) .gt. 0.01) then
                  dslambda(j) = dwavel(joldds(j))
               else
c  otherwise we have to extract a suitable wavelength from all the batches 
c Too complicated to average, so just check
                  dslambda(j) = bwavel(jbtchds(j, 1))
                  if (nbtchds(j) .gt. 1) then
                     lfail = .false. 
                     do k = 2, nbtchds(j)
                        if (abs(dslambda(j) - bwavel(jbtchds(j, k)))
     $                        .gt. wvllim) lfail = .true.
                     enddo
                     if (lfail) then
                        write (lunout, '(/a/a/a/a/)')
     $                '=== WARNING: not all wavelengths in dataset',
     $                     newxnam(j), newdnam(j),' are the same ==='
                     endif
                  endif
               endif
            enddo
         endif
c-
         do j = 1, nnewds
            call lwidx(1, newpnam(j), newxnam(j), newdnam(j), 
     $               dscell(1,j), dslambda(j))
         enddo
      endif
c
c
c Set crystalname/datasetname for each batch
c Call after lwbat & cpybth
      do i  = 1, nbtall
         if (newbat(i) .gt. 0 . and. 
     +          xname(i).ne.' ' .and. dname(i).ne.' ') then
            call lwbsetidx(1,newbat(i),xname(i),dname(i))
         endif
      enddo

C For multi-record files, columns can contain data from different
C datasets. However, columns need to be assigned to some dataset.
C We choose here to assign them to the dataset associated with the
C first batch (excluding those that have been REJECTed). This also 
C prevents the problem of ghost datasets.
      do i  = 1, nbtall
         if (newbat(i) .gt. 0) then
           call lwidall(1,xname(i),dname(i))
           goto 200
         endif
      enddo
 200  continue

      return
      end
C
C
C
      subroutine cpybth
C     =================
C
C  Copy batch headers from input to output file, for all batches not rejected
C  with batch numbers altered as requested
C
      implicit none
C
c+++ nbtchc.h +++
c  nbtall         number of batches in input file
c  ibtall         input batch numbers for each batch
c  newbat         output batch number for each batch, = 0 to reject
      integer maxbat
      parameter (maxbat = 5000)
c  Data for each batch in input file
      common /nbtchc/ bcell(6,maxbat),bwavel(maxbat),idsetb(maxbat),
     $   nbtall,ibtall(maxbat),
     +   newbat(maxbat),maxwid(maxbat)
      real bcell, bwavel, maxwid
      integer idsetb, nbtall, ibtall, newbat
      save /nbtchc/
c--- nbtchc.h ---
c+++ outcon.h +++
c
c     outitl    title for output file, if non-blank replaces input title
c     histry    new history record to add to file (if non-blank)
c
      common /outcoc/ outitl, histry
      character outitl*70, histry*80
c
      save /outcoc/
c--- outcon.h ---
C+++ orient.h +++
C
C     
C     
C     Orientation block data
C     
C     This contains slots for all information that seems to be essential
C     at present. Each group of parameters is padded at the end for future
C     expansion.
C     
C     Data in the orientation block are referred to the "Cambridge"
C     laboratory axis frame: x along the (idealized) X-ray beam, z along
C     usual rotation axis E1 (omega on 3-axis system). The matrix Q converts
C     a vector in the Madnes frame to the Cambridge frame.  Note that the
C     laboratory frame is essentially defined by the vectors e1,e2,e3 &
C     source. It doesn't really seem necessary to carry through a whole lot
C     of crystal and beam tensors, particularly as we have integrated
C     intensities at this stage, but maybe someone will want to, using the
C     allocated padding
C     
C     The general orientation equation is
C     
C     x  =   R M U B h
C     
C     where x    position in laboratory frame
C     R    goniostat matrix
C     M    missetting angle matrix (if relevant, see MISFLG)
C                 PhiZ PhiY PhiX (PHIXYZ)
C     U    crystal orientation matrix UMAT
C     B    cell orthogonalization matrix, derived from cell dimensions
C     h    reflection indices
C     
C     Note that the description below is NOT is the same order as in the
C     common block, in which all the integers come before all the reals 
C     (flagged as I or R in the description below)
C     
CI    NWORDS       number of words in orientation block
CI    NINTGR       number of integers (first part of block,
C                        includes these counts)
CI    NREALS       number of reals
CI    IORTYP       type of orientation block (for possible future use, now = 0)
CI    INTPAD(8)    padding for future use (integers)
C     
C---  Information for this crystal
C     
CR    CELL(6)      cell dimensions  (A & degrees)
CI    LBCELL(6)    refinement flags for cell dimensions
CR    UMAT(3,3)    orientation matrix U. If MISFLG .gt. 0, U is the
C                   "standard" setting when PhiXYZ ==0 
CI    MISFLG       status of "missetting" angles PHIXYZ
C                   = 0  PHIXYZ not used, all orientation in UMAT
C                   = 1  1 set of missetting angles (PHIXYZ(I,1))
C                   = 2  2 sets PHIXYZ(I,J), J=1,2
CR    PHIXYZ(3,2)  missetting angles at beginning & end of rotation 
CI    JUMPAX       reciprocal axis closest to principle goniostat axis E1
C                       (only used for printing)
CI    NCRYST       crystal number: a crystal may contain several batches
CI    LBSETID      dataset number (this indexes a list of datasets in the
C                  file header)
CI    LCRFLG       type of crystal mosaicity information
C                  (=0 for isotropic, =1 anisotropic)
C     *** CRYDAT(12) equivalenced to following ***
CR    ETAD         reflection width (full width) (degrees)  (if LCRFLG=0)
C      or
CR    ETADH,ETADV  horizontal & vertical reflection width   (if LCRFLG=1)
CR    rest of CRYDAT: padding for crystal information (eg more complicated
C                           mosaicity model)
C     ***
C     
C---  Information for this batch
C     
CI    LDTYPE       type of data
C                   = 1    oscillation data   (2D spots)
C                   = 2    area detector data (3D spots)
C                   = 3    Laue data
CR    DATUM(3)     datum values of goniostat axes, from which Phi is measured
C                      (degrees)
CR    PHISTT,PHIEND start & stop values of Phi (degrees) relative to datum
CR    PHIRANGE     range of Phi values: typically this will be PHIEND-PHISTT,
C                    but storing this explicitly allows a distinction 
C                    eg between a rotation of +160 degrees from a rotation
C                    of -200 degrees
CI    JSCAXS       goniostat scan axis number (=1,2,3, or =0 for
C                        multiple axis scan
CR    SCANAX(3)    rotation axis in laboratory frame (not yet implemented:
C                      only relevant if JSCAXS=0)
CR    TIME1, TIME2 start & stop times in minutes
CI    NBSCAL       number of batch scales & Bfactors plus SD's
C                   (4 at present, BSCALE, BBFAC & sd's)
C                  set = 0 if batch scales unset
CR    BSCALE       batch scale
CR    BBFAC        batch temperature factor
C                   corresponding scale is exp(-2 B (sin theta/lambda)**2)
CR    SDBSCL       sd (Bscale)
CR    SDBFAC       sd (BBfac)
CR    BATPAD(11)   padding for batch information
C     
C---  Crystal goniostat information
C     
CI    NGONAX       number of goniostat axes (normally 1 or 3)
CI    E1(3),E2(3),E3(3) vectors (in "Cambridge" laboratory frame, see below)
C                            defining the NGONAX goniostat axes
CC    GONLAB(3)  names of the three goniostat axes
CR    GONPAD(12) padding for goniostat information
C     
C---  Beam information
C     
CR    SOURCE(3)    Idealized (ie excluding tilts) source vector 
C                   (antiparallel to beam), in "Cambridge" laboratory frame
CR    S0(3)        Source vector (antiparallel ! to beam), in
C                   "Cambridge" laboratory frame, including tilts
CI    LBMFLG       flag for type of beam information following 
C                   = 0 for ALAMBD, DELAMB only (laboratory source)
C                   = 1     ALAMBD,DELAMB,DELCOR,DIVHD,DIVVD (synchrotron) 
C                           (other options could include white beam)
C     *** BEMDAT(25) equivalenced to following ***
CR    ALAMBD       Wavelength in Angstroms
CR    DELAMB       dispersion Deltalambda / lambda.
CR    DELCOR       Correlated component of wavelength dispersion.
CR    DIVHD        Horizontal beam divergence in degrees.
CR    DIVVD        Vertical beam divergence (may be 0.0 for isotropic beam
C                        divergence.
CR    rest of BEMDAT: padding for beam information
C                      (*** How much here for Laue? ***)
C     ***
C     
C---  Detector information
C     
CI    NDET         number of detectors (current maximum 2)
C     -- for each detector
CR    DXn          crystal to detector distance (mm)
CR    THETAn       detector tilt angle (=Madnes:tau2) (degrees)
CR    DETLMn(2,2)  minimum & maximum values of detector coordinates (pixels)
C                     (i,j): i = 1 minimum, = 2 maximum
C                            j = 1 Xdet,    = 2 Ydet
C                  The exact detector frame is not important, but Ydet
C                  should be the axis ~ parallel to the pricipal
C                  rotation axis
CR    DETPAD(34)     padding for detector information
C     
C     
C Lengths: MBLENG is total length of block, MBLINT, MBLREA are numbers
C          of integers & reals
      INTEGER MBLENG,MBLINT,MBLREA
      PARAMETER (MBLENG=185,MBLINT=29,MBLREA=156)
      INTEGER NWORDS,NINTGR,NREALS,IORTYP,LCELL,MISFLG,
     .     JUMPAX,NCRYST,LCRFLG,LDTYPE,JSCAXS,NBSCAL,NGONAX,LBMFLG,
     $     NDET,LBSETID,INTPAD
      REAL   CELL,UMATR,PHXYZ,CRYDAT,ETAD,DATUM,
     $     PHISTT,PHIEND,PHIRANGE,SCANAX,TIME1,TIME2,GONPAD,
     $     BSCALE,BBFAC,SDSCAL,SDBB,BATPAD,E1,E2,E3,
     $     SOURCE,S0,BEMDAT,ALAMBD,DELAMB,DELCOR,DIVHD,DIVVD,
     $     DX1,THETA1,DETLM1,DX2,THETA2,DETLM2,DETPAD
      CHARACTER BTITLE*70, GONLAB*8
C     
      COMMON /CORIEN/BTITLE,GONLAB(3)
C     
      COMMON /ORIENT/ NWORDS,NINTGR,NREALS,IORTYP,LCELL(6),MISFLG,
     .     JUMPAX,NCRYST,LCRFLG,LDTYPE,JSCAXS,NBSCAL,NGONAX,LBMFLG,
     $     NDET,LBSETID,INTPAD(8),
     +     CELL(6),UMATR(3,3),PHXYZ(3,2),CRYDAT(12),DATUM(3),
     $     PHISTT,PHIEND,SCANAX(3),TIME1,TIME2,
     $     BSCALE,BBFAC,SDSCAL,SDBB,PHIRANGE,BATPAD(11),
     $     E1(3),E2(3),E3(3),
     $     GONPAD(12),SOURCE(3),S0(3),BEMDAT(25),
     $     DX1,THETA1,DETLM1(2,2),DX2,THETA2,DETLM2(2,2),DETPAD(33)
      REAL RDUMMY (MBLENG)
      EQUIVALENCE (RDUMMY,NWORDS)
C     
C     *** Note well the following equivalences: these are to allow for
C     alternative definitions of crystal mosaicity, beam parameters, etc
      EQUIVALENCE (ETAD,CRYDAT(1))
      EQUIVALENCE (BEMDAT(1),ALAMBD)
      EQUIVALENCE (BEMDAT(2),DELAMB)
      EQUIVALENCE (BEMDAT(3),DELCOR)
      EQUIVALENCE (BEMDAT(4),DIVHD)
      EQUIVALENCE (BEMDAT(5),DIVVD)
C ***
c
      save /orient/, /corien/
C--- orient.h ---
      real detlim
      logical ldtlim
      common /detctr/ detlim(2,2), ldtlim
      CHARACTER*64 PNAME(maxbat),XNAME(maxbat),DNAME(maxbat)
      common /pnamdnam/ PNAME, XNAME, DNAME
      save /detctr/,/pnamdnam/
C     ..
      integer i, j, ibatch, iprint
C
C
      do 10 i  = 1, nbtall
         ibatch = ibtall(i)
C Position header read to batch IBATCH
         call lrbres(1, ibatch)
         iprint = 0
C     Read orientation block
         call lrbat(1,j,rdummy,btitle,iprint)
C Check right one (I'm not sure this can ever be wrong)
         if (j .ne. ibatch) then
            write (6,'(A,I8,A,I8)') ' !!!! Orientation data for batch',
     $           j,' found when expecting data for batch',ibatch
            call ccperr(1,'*** Invalid orientation data ***')
         endif
c
c Store cell and wavelength
         call ccpmvr(bcell(1,i), cell(1), 6)
         bwavel(i) = alambd
c Reset detector limits if required
         if (ldtlim) then
            detlm1(1,1) = detlim(1,1)
            detlm1(2,1) = detlim(2,1)
            detlm1(1,2) = detlim(1,2)
            detlm1(2,2) = detlim(2,2)
         endif
C     ..
C---- Write out batch headers for selected batches
         if (newbat(i) .gt. 0) then
            call lwbat(1, newbat(i), rdummy, btitle)
         endif
c
C
 10   continue
C
      end
c
c
c
      subroutine keyin
c     ================
c
c  Read all commands
c
      implicit none
c+++ nbtchc.h +++
c  nbtall         number of batches in input file
c  ibtall         input batch numbers for each batch
c  newbat         output batch number for each batch, = 0 to reject
      integer maxbat
      parameter (maxbat = 5000)
c  Data for each batch in input file
      common /nbtchc/ bcell(6,maxbat),bwavel(maxbat),idsetb(maxbat),
     $   nbtall,ibtall(maxbat),
     +   newbat(maxbat),maxwid(maxbat)
      real bcell, bwavel, maxwid
      integer idsetb, nbtall, ibtall, newbat
      save /nbtchc/
c--- nbtchc.h ---
c+++ outcon.h +++
c
c     outitl    title for output file, if non-blank replaces input title
c     histry    new history record to add to file (if non-blank)
c
      common /outcoc/ outitl, histry
      character outitl*70, histry*80
c
      save /outcoc/
c--- outcon.h ---
      common /inout/ lunout
      integer lunout
      common /detctr/ detlim(2,2), ldtlim
      real detlim
      logical ldtlim
      character*64 pname(maxbat),xname(maxbat),dname(maxbat),
     +      projectname,crystalname,datasetname
      common /pnamdnam/ PNAME, XNAME, DNAME
      save /detctr/,/pnamdnam/
c Global cell & wavelength
      common /wvlcll/ gcell(6), glambda, lgcell, lglambda
      real gcell, glambda
      logical lgcell, lglambda
      save /wvlcll/
c  List of datasets
      integer maxset
      parameter (maxset=200)
      common /datasets/ dcell(6,maxset), dwavel(maxset), 
     $   isetids(maxset),ndatasets, ldatasets
      real dcell, dwavel
      integer isetids, ndatasets
      logical ldatasets
      common /datasetc/ dspname(maxset),dsxname(maxset),dsdname(maxset)
      character*64 dspname, dsxname, dsdname
c
C Things for PARSER ----
      integer maxtok
      parameter (maxtok=100)
      character key*4,cvalue(maxtok)*4,line*80
      integer ibeg(maxtok),iend(maxtok),ityp(maxtok),
     .     idec(maxtok),ntok
      real    fvalue(maxtok)
      logical lend
c
c Locals
      integer ierr, ikey, idum, i, j, n, medit, nadd, lflag, nstart,
     $     llist, ibatch, nstep
      real vlist(500),nmaxw
      logical lbatch, lbatchall, lincld, lrejct, lmaxw, ldtsin
c
c Functions
      integer ccp4_hash_lookup, lenstr
      logical ccponl
      external ccp4_hash_lookup, ccponl, lenstr
c
c Commands
      integer nkeys
      parameter (nkeys=7)
      character*16 keys(nkeys)
c
      data keys/
     $   'BATCH','TITLE','HISTORY','DETECTOR','CELL','WAVELENGTH',
     $   'END'/
c
c
      lgcell = .false. 
      lglambda = .false.
      nstart = 1
      nstep = +1
      nadd = 0
      lincld = .false.
      lrejct = .false.
      ldtlim = .false.
      projectname = ' '
      crystalname = ' '
      datasetname = ' '
c
      do i=1,maxbat
         maxwid(i)=50.0
         pname(i)=' '
         xname(i)=' '
         dname(i)=' '
      enddo
c
c  Output title & history
      outitl = ' '
      histry = ' '
c
c  Set input error counter
 9    ierr=0
C
C
C ---- Loop to read control input
C
 10   line=' '
      ntok=maxtok
      call parser(
     .  key,line,ibeg,iend,ityp,fvalue,cvalue,idec,ntok,lend,.true.)
      if(lend) go to 1000
      if(ntok.eq.0) go to 10
      call ccpupc(key)
      call chkkey(key,keys,nkeys,ikey)
      if(ikey.eq.0) then
        write(lunout,fmt='(a)') ' ** Unrecognized keyword **'
        if(ccponl(idum)) then
          goto 10
        else
          ierr=ierr+1
        endif
      else if(ikey.lt.0) then
        write(lunout,fmt='(a)') ' ** Ambiguous keyword **'
        if(ccponl(idum)) then
          goto 10
        else
          ierr=ierr+1
        endif
      endif
c
      do 11, i=1,ntok
         call ccpupc(cvalue(i))
 11   continue
c
      go to (100,200,300,400,500,600,1000), ikey
c------------------------------------------------------------------------
c
c  == BATCH == command
c  
c     [BATCH   | b1 b2 b3 ... | b1 TO b2 | ALL]
c     [ADD   nadd]
c     [START nstart]
c     [REJECT]
c     [INCLUDE]
C     [A.P.: MAXWIDTH]
c
 100  n = 2
      medit = 0
      lbatch = .false.
      lbatchall = .false.
      ldtsin = .false.
c Next keyword N
 109  if (n .gt. 0 .and. n .le. ntok) then
         if (cvalue(n) .eq. 'ADD') then
c     ADD keyword, read number to add (may be negative)
            if (medit .ne. 0) then
               write (lunout, '(/a/)')
     $      ' *** Only one of ADD, START, REJECT or INCLUDE may be'//
     $              ' given on each command ***'
               ierr = ierr+1
               go to 10
            endif
            medit = +1
            call gttint(n+1, nadd, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) then
               go to 990
            endif
            n = n+2
         elseif (cvalue(n) .eq. 'STAR') then
c     START keyword, read number to start at
            if (medit .ne. 0) then
               write (lunout, '(/a/)')
     $      ' *** Only one of ADD, START, REJECT or INCLUDE may be'//
     $              ' given on each command ***'
               ierr = ierr+1
               go to 10
            endif
            medit = +2
            call gttint(n+1, nstart, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) then
               go to 990
            endif
            n = n+2
         elseif (cvalue(n) .eq. 'STEP') then
c     STEP keyword, read number to increment from START
            if (medit .ne. +2) then
               write (lunout, '(/a/)')
     $      ' *** STEP only valid after START ***'
               ierr = ierr+1
               go to 10
            endif
            call gttint(n+1, nstep, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) then
               go to 990
            endif
            n = n+2
         elseif (cvalue(n) .eq. 'REJE') then
c     REJECT keyword
            if (medit .ne. 0) then
               write (lunout, '(/a/)')
     $      ' *** Only one of ADD, START, REJECT or INCLUDE may be'//
     $              ' given on each command ***'
               ierr = ierr+1
               go to 10
            endif
            medit = -1
            lrejct = .true.
            n = n+1
         elseif (cvalue(n) .eq. 'INCL') then
c     INCLUDE keyword
            if (medit .ne. 0) then
               write (lunout, '(/a/)')
     $      ' *** Only one of ADD, START, REJECT or INCLUDE may be'//
     $              ' given on each command ***'
               ierr = ierr+1
               go to 10
            endif
            if (.not.lincld) then
c  If this is the first include command, reset all batches to reject
               if (lrejct) then
                  write (lunout, '(/a,a/a/)')
     $                 ' === WARNING: this INCLUDE command may ',
     $                 'conflict with previous REJCT command(s)',
     $                 ' === Is this what you want? ==='
               endif
               do 105, i=1,nbtall
                  newbat(i) = 0
 105           continue
            endif
            lincld = .true.
c
            medit = +3
            n = n+1
C additions from AP
         elseif (cvalue(n) .eq. 'MAXW') then
c   MAXWIDTH keyword
            if (medit .ne. 0 ) then
               write (lunout, '(/,a,/)')
     $      ' *** Only one of ADD, START, REJECT, INCLUDE '//
     $      ' or MAXWIDTH may be given on each command ***'
               ierr = ierr+1
               go to 10
            endif
            lmaxw = .true.
C
            call gttrea(n+1, nmaxw, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) then
               go to 990
            endif
            medit = +4
            n = n + 2

C subkeyword PNAM
         elseif (cvalue(n) .eq. 'PNAM' .or. cvalue(n) .eq. 'PROJ') then
            if (medit .ne. 0 .and. medit .ne. 5) then
               write (lunout, '(/,a,a,/)')
     $      ' *** Only one edit command can be given for each',
     $      ' BATCH command ***'
               ierr = ierr+1
               go to 10
            endif
            if (n+1.le.ntok) projectname = line(ibeg(n+1):iend(n+1))
            medit = 5
            n = n + 2
            ldatasets = .true.
            ldtsin = .true.
C subkeyword XNAM
         elseif (cvalue(n) .eq. 'XNAM' .or. cvalue(n) .eq. 'CRYS') then
            if (medit .ne. 0 .and. medit .ne. 5) then
               write (lunout, '(/,a,a,/)')
     $      ' *** Only one edit command can be given for each',
     $      ' BATCH command ***'
               ierr = ierr+1
               go to 10
            endif
            if (n+1.le.ntok) crystalname = line(ibeg(n+1):iend(n+1))
            medit = 5
            n = n + 2
            ldatasets = .true.
            ldtsin = .true.
C subkeyword DNAM
         elseif (cvalue(n) .eq. 'DNAM' .or. cvalue(n) .eq. 'DATA') then
            if (medit .ne. 0 .and. medit .ne. 5) then
               write (lunout, '(/,a,a,/)')
     $      ' *** Only one edit command can be given for each',
     $      ' BATCH command ***'
               ierr = ierr+1
               go to 10
            endif
            if (n+1.le.ntok) datasetname = line(ibeg(n+1):iend(n+1))
            medit = 5
            n = n + 2
            ldatasets = .true.
            ldtsin = .true.
         else
            if (lbatch) go to 990
c     BATCH keyword (optional), followed by ALL, batch list or range
            if (cvalue(n) .eq. 'BATC') then
               n = n+1
            endif
            if (cvalue(n) .eq. 'ALL') then
               n = n+1
               lbatch = .false.
               lbatchall = .true.
            else
               call lisran(ntok, n, cvalue, fvalue, ityp, llist, vlist)
               if (llist .lt. 0) then
                  write (lunout, '(A)') ' ** Error in BATCH list **' 
                  go to 999
               endif
               lbatch = .true.
            endif
         endif
         go to 109
      endif
c
      if (medit .eq. 0 .or. medit .gt. 5) then
         write (lunout, '(/a/)')
     $        ' *** No edit command given ***'
         ierr = ierr+1
         go to 10
      endif
      if (.not. lbatch) then
         if (.not. lbatchall) write (lunout, '(/a/)')
     $        ' No batch specification given, ALL assumed'
         llist = 0
      endif
      if (projectname.ne.' '.or.crystalname.ne.' '.or.
     $                                    datasetname.ne.' ') then
        if (.not. (projectname.ne.' '.and.crystalname.ne.' '.and.
     $                                    datasetname.ne.' ')) then
         write (lunout, '(/,a,a,/,a,/)')
     $    ' BATCH command: ',
     $    'some but not all of PNAME/XNAME/DNAME subkeywords given.',
     $    ' Please give all 3, even if unchanged from input. '
         call ccperr(1,'Problem in keyworded input, see log file.')
        endif
      endif
c
      if (llist .eq. 0) then
c     Range
         do 101, i = 1, nbtall
            ibatch = ibtall(i)
            if ((.not.lbatch) .or.
     $         (ibatch .ge. nint(vlist(1)) .and.
     $          ibatch .le. nint(vlist(2)))) then
               call renum(
     $         medit, ibatch, newbat(i), nadd, nstart, nstep)
               if (medit.eq.4) then
                  maxwid(i)=nmaxw
               endif
               if (ldtsin) then
                 pname(i) = projectname
                 xname(i) = crystalname
                 dname(i) = datasetname
               endif
            endif
 101     continue
      else
c     List
         do 102, ibatch = 1, llist
            i = ccp4_hash_lookup(nint(vlist(ibatch)))
            call renum(
     $      medit, ibtall(i), newbat(i), nadd, nstart, nstep)
            if (medit.eq.4) then
               maxwid(i)=nmaxw
            endif
            if (ldtsin) then
              pname(i) = projectname
              xname(i) = crystalname
              dname(i) = datasetname
            endif
 102     continue
      endif
      go to 10
c
c------------------------------------------------------------------------
c
c   == TITLE ==
c Read title to replace input title
 200  outitl = line(ibeg(2):)
      go to 10
c------------------------------------------------------------------------
c
c   == HISTORY ==
c Read new history line to add to file
 300  if (lenstr(histry) .gt. 0) then
         write (lunout, '(a,a)')
     $        ' ==== WARNING: only one new history line may be',
     $        ' added to file ===='
      else
         histry = line(ibeg(2):)
      endif
      go to 10
c------------------------------------------------------------------------
c
c   == DETECTOR ==
c Read detector limits
 400  if (ntok .ne. 5) then
         write (lunout, '(a)')
     $      ' *** DETECTOR must give 4 limits ***'
         call ccperr(1,'*** Illegal DETECTOR command ***')
      endif
      call gttrea(2, detlim(1,1), lflag, ntok,ityp,fvalue)
      if (lflag .ne. 0) then
         go to 990
      endif
      call gttrea(3, detlim(2,1), lflag, ntok,ityp,fvalue)
      if (lflag .ne. 0) then
         go to 990
      endif
      call gttrea(4, detlim(1,2), lflag, ntok,ityp,fvalue)
      if (lflag .ne. 0) then
         go to 990
      endif
      call gttrea(5, detlim(2,2), lflag, ntok,ityp,fvalue)
      if (lflag .ne. 0) then
         go to 990
      endif
      ldtlim = .true.
      write (lunout,'(/a,4f10.3/)') ' Reset detector limits to:',
     $   ((detlim(i,j), i=1,2),j=1,2)
      go to 10
c------------------------------------------------------------------------
c
c   == CELL ==
c Read Cell    
 500  if (ntok .lt. 4) then
         go to 990
      endif
      do i = 4,6
         gcell(i) = 90.0
      enddo 
      do i = 1, ntok-1
         call gttrea(i+1, gcell(i), lflag, ntok,ityp,fvalue)
         if (lflag .ne. 0) then
            go to 990
         endif
      enddo
      lgcell = .true.
      ldatasets = .true.
      go to 10
c------------------------------------------------------------------------
c
c   == WAVELENGTH ==
c Read wavelength
 600  if (ntok .lt. 2) then
         go to 990
      endif
      call gttrea(2, glambda, lflag, ntok,ityp,fvalue)
      if (lflag .ne. 0) then
         go to 990
      endif
      lglambda = .true.
      ldatasets = .true.
      go to 10
c------------------------------------------------------------------------
c------------------------------------------------------------------------
c
 1000 continue
c Check & fail if any errors
      do 1005, j = 1, nbtall
         if (newbat(j) .lt. 0) then
            write (lunout, '(a,i8,a,i8)')
     $           ' *** Batch ', ibtall(j),
     $           ' is renumbered to a illegal non-positive value, ',
     $           newbat(j)
            ierr = ierr+1
         endif
 1005 continue
c
C  Fatal errors
      if (ierr .gt. 0) then
         call ccperr(1,'***** Error in input *****')
      endif
c
c
      call ccp4h_summary_beg()

      write (lunout, 6100)
 6100 format(//
     $     ' List of original batch numbers and new batch numbers,',
     $     ' = 0 if rejected',/)
      write (lunout, 6110)
 6110 format('Reflections with width greater than maximum allowed ',
     $  'width will be rejected',/)

      write (lunout, 6120)
     $ 'Old batch','New batch','Max allowed',
     $ 'New project','New crystal','New dataset'
 6120 FORMAT(/,/,1X,A,3X,A,3X,A,3X,A,3X,A,3X,A)
      write (lunout, 6125)
     $ 'no.','no.','width','name','name','name'
 6125 FORMAT(4X,A,9X,A,9X,A,9X,A,9X,A,9X,A,/)
      do 1020 i= 1, nbtall
        write(lunout,6130) ibtall(i),newbat(i),maxwid(i),
     $        pname(i)(1:10),xname(i)(1:10),dname(i)(1:10)
 1020 continue
 6130 FORMAT(1X,I8,3X,I8,6X,F8.2,7X,A10,4X,A10,4X,A10)
      write (lunout,'(/,/)') 

      if (lenstr(histry) .gt. 0) then
       write (lunout,6140)
 6140  format ('HISTORY : New history record for file HKLIN.',/)
      endif
      if (ldtlim) then
       write (lunout,6150)
 6150  format ('DETECTOR: Reset limits of detector coordinates ',
     $  'XDET, YDET for all batches.',/)
      endif
c
      if (lgcell) then
         write (lunout, 6160) gcell
 6160    format (/' New unit cell for file & datasets headers:',
     $      6f9.3)
      endif
      if (lglambda) then
         write (lunout, 6170) glambda
 6170    format (/' New wavelength for file & datasets headers:',
     $      f9.5)
      endif
c
      call ccp4h_summary_end()
c
      return
c
c
 990  write (lunout,'(A)') ' ** Missing or illegal value **'
 999  ierr = ierr+1
      go to 10

      end
c
c
c
      subroutine lisran(ntok,itok,cvalue,fvalue,ityp,llist,vlist)
c     ===========================================================
c     
c     Parse number list or number range
c     
c     On entry:
c     ntok      number of tokens in line
c     itok      first one to use
c     cvalue    character values from Parser
c     fvalue    number values from Parser
c     ityp      number types from Parser
c     
c     On exit:
c     itok      next token to use, = 0 if all tokens used
c     llist     .lt. 0 error, eg too few tokens for range
c     = 0  number range, returned in vlist(1) & (2)
c     .gt. 0, number list, llist entries in vlist
c     vlist     range or list
c     
      integer ntok,itok,ityp(ntok),llist
      real fvalue(ntok), vlist(*)
      character*(*) cvalue(ntok)
c     
      integer i
c     
      if (itok+2 .le. ntok .and. cvalue(itok+1) .eq. 'TO') then
c     Number range
         if (ityp(itok) .ne. 2 .or. ityp(itok+2) .ne. 2) then
c     Illegal number
            llist = -1
            return
         else
            vlist(1) = fvalue(itok)
            vlist(2) = fvalue(itok+2)
            llist = 0
            itok = itok + 3
         endif
      else
c     Number list
         llist = 0
         do 10, i = itok, ntok
            if (ityp(i) .ne. 1) then
c     Store number
               llist = llist+1
               vlist(llist) = fvalue(i)
            else
c     String found, this terminates list
               itok = i
               if (llist .le. 0) llist = -1
               return
            endif
 10      continue
         itok = 0
      endif
      return
      end
c
c
c
      subroutine renum(medit, ibtold, ibtnew, nadd, nstart, nstep)
c     ============================================================
c
c Generate new batch number
c
c On entry:
c     medit     edit command
c               = -1  reject, ie set batch = 0
c               = +1  add, set ibtnew = ibtold + nadd
c               = +2  start, set ibtnew = nstart and increment nstart
c               = +3  include, set ibtnew = ibtold
c               = +4  just to set max width ...
c     ibtold    original batch number
c     ibtnew    current batch number, or = 0 if already rejected
c     nadd      number to add if medit = +1 (only if not rejected)
c     nstart    new number if medit = +2 (only if not rejected)
c     nstep     step for increment if medit = +2
c
c On exit:
c     ibtnew    new batch number, = -1 if illegal, = 0 if rejected
c               (illegal means negative or zero, unless reject)
c     nstart    incremented + 1 if medit = +2
c
      implicit none
c
      integer medit, ibtold, ibtnew, nadd, nstart, nstep

c
      if (medit .lt. 0) then
c  Reject
         ibtnew = 0
      elseif (medit .eq. +1) then
c  Add
         if (ibtnew .gt. 0) then
            ibtnew = ibtold + nadd
            if (ibtnew .le. 0) ibtnew = -1
         endif 
      elseif (medit .eq. +2) then
c  Start
         if (ibtnew .gt. 0) then
            ibtnew = nstart
            nstart = nstart+nstep
            if (ibtnew .le. 0) ibtnew = -1
         endif
      elseif (medit .eq. +3) then
c  Include
         ibtnew = ibtold
      endif
      return
      end

