C
C     REVISE
C     Copyright (C) 1998 Yao Jia-xing
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
C     **************
      PROGRAM REVISE
C     **************
C
C     REVISE is for MAD data only. The program is used to modify MAD data
C     and then estimate FM, the anomalous contributions from anomalous 
C     scatterers. FM can be used to calculate anomalous Patterson maps or 
C     as input to Direct Methods to locate the positions of anomalous 
C     scatterers.
C
C     The program can handle up to ten wavelength anomalous scattering data
C     and at least two sets of data are needed.
C
C     Input file contains
C                         H        K           L
C                         FP       SIGFP       DP       SIGDP
C                         FPHn(+)  SIGFPHn(+)  FPHn(-)  SIGFPHn(-)
C                         FPHn     SIGFPHn     DPHn     SIGDPHn
C
C     Output file contains input data plus
C
C                         FPHMn(+) SIGFPHMn(+) FPHMn(-) SIGFPHMn(-)
C                         FPHMn    SIGFPHMn    DPHMn    SIGDPHMn
C                         FM       SIGFM
C
C                         n = 1 to 10.
C
C       Name           Item
C
C       H, K, L        Miller indices.
C
C     --------------   Just in case the native data is available.
C
C       FP             Native 'F' value.
C       SIGFP          Sigma(FP).
C       DP             Anomalous difference for native data.
C       SIGDP          Sigma(DP).
C
C     --------------
C
C       FPHn(+)        FPH(h,k,l) for wavelength 'n'.
C       SIGFPHn(+)     Sigma(FPHn(+)).
C       FPHn(-)        FPH(-h,-k,-l) for wavelength 'n'.
C       SIGFPHn(-)     Sigma(FPHn(-)).
C
C     --------------
C
C       FPHn           FPHn = 0.5 * (FPHn(+) + FPHn(-)).
C       SIGFPHn        Sigma(FPHn).
C       DPHn           DPHn = (FPHn(+) - FPHn(-)).
C       SIGDPHn        Sigma(DPHn).
C
C     --------------   Modified values of:
C
C       FPHMn(+)       FPHM(h,k,l) for wavelength 'n'.
C       SIGFPHMn(+)    Sigma(FPHMn(+)).
C       FPHMn(-)       FPHM(-h,-k,-l) for wavelength 'n'.
C       SIGFPHMn(-)    Sigma(FPHMn(-)).
C
C     --------------
C
C       FPHMn          FPHMn = 0.5 * (FPHMn(+) + FPHMn(-)).
C       SIGFPHMn       Sigma(FPHMn).
C       DPHMn          DPHMn = (FPHMn(+) - FPHMn(-)).
C       SIGDPHMn       Sigma(DPHMn).
C
C     --------------   
C
C       FM            Estimates of the anomalous contributions from
C                     anomalous scatterers after applying REVISE.
C       SIGFM         Sigma(FM).
C
C   ----------------------------------------------------------------
C
C       The column types are as follows:
C
C       H       index h,k,l
C       F or G  structure amplitude, F
C       D       anomalous difference
C       Q or L  standard deviation of anything: F,D or other
C
C    ---------------------------------------------------
C
C       I       any other integer
C       R       any other real
C
C               ******************************
C               Description of Key Words Input
C               ******************************
C
C  (i)   TITLE      (optional)
C
C        Replaces title in MTZ file
C
C  (ii)  LABIN      (program_label=file_label)
C
C        Examples:
C           LABI FPH1(+)=F+1 SIGFPH1(+)=SF+1 FPH1(-)=F-1 SIGFPH1(-)=SF-1 -
C                FPH2(+)=F+2 SIGFPH2(+)=SF+2 FPH2(-)=F-2 SIGFPH2(-)=SF-2 -
C                FPH3=FMP3 SIGFPH3=SIGFMP3 DPH3=DMP3 SIGDPH3=SIGDMP3
C
C  (iii) WAVE    (compulsory)
C
C           WAVE <ndata> LAM <wavelength> FPR <f'> FDP <f">
C
C           WAVE <ndata>,  FPR <f'> and FDP <f"> are compulsory and 
C           corresponding the data sets.
C
C        Examples:
C           WAVE 1 LAM 0.9000  FPR -6 FDP 2.6
C        or WAVE 3 FDP 4.8 FPR -3.67
C
C  (iv)  LABOUT   (Allow user assign their own labels)
C
C   Examples:
C   LABO -
C   FPHM1(+)=F+1_mod SIGFPHM1(+)=SF+1_mod FPHM1(-)=F-1_mod SIGFPHM1(-)=SF-1_mod -
C   FPHM2(+)=F+2_mod SIGFPHM2(+)=SF+2_mod FPHM2(-)=F-2_mod SIGFPHM2(-)=SF-2_mod -
C   FPHM3=FMP3_mod SIGFPHM3=SIGFMP3_mod DPHM3=DMP3_mod SIGDPHM3=SIGDMP3_mod -
C   FM=FM_RE SIGFM=SFM_RE
C
C  (v)   EXCLUDE  (optional) 
C
C        EXCL RISO <riso> RANO <rano> SIGM <sigm>
C
C        Data will be excluded
C
C         if the ratio of <|DISO|>/<FPH> great than <riso>.
C                                      Default: RISO = 0.10 (10%).
C
C         if the ratio of |DANO|/FPH great than <rano>.
C                                      Default: RANO = 0.50 (50%).
C
C         if FPH less than SIGM*SIGFPH.
C                                      Default: SIGM = 0.0.
C
C  (vi)  END      (optional)
C
C        Ends keyword input.
C
C     Parameters.
      INTEGER MCOLS,NKEYWD,LSYM,NUMFO,NRMAX,NCMAX,NDALL,NDMIN
      PARAMETER (MCOLS=500,NKEYWD=200,LSYM=192,NUMFO=10,NRMAX=15
     +,NCMAX=10,NDMIN=2)
      INTEGER MSETS
      PARAMETER (MSETS=MCOLS)
C
C     Allow NUMFO sets of Fs.
C     Allow NRMAX resolution ranges for xloggraph. (1 - 10)
C     Allow NCMAX ratio ranges for xloggraph. (-20 - 20)
C
C     Arrays for data.
      REAL ADATA(MCOLS),BDATA(MCOLS),RSYML(4,4,LSYM),
     +     FPR(NUMFO),FDP(NUMFO),LAM(NUMFO),BUF1(NUMFO),BUF2(NUMFO)
      INTEGER LOOKUP(MCOLS),IN(3),NDT(NUMFO)
      CHARACTER*1 CTPRGO(MCOLS),CTPRGI(MCOLS),CTUSRI(MCOLS)
      CHARACTER*30 LSPRGI(MCOLS),LSPRGO(MCOLS),LSUSRI(MCOLS)
      LOGICAL LOGMSS(MCOLS)
C
C     Array for EXCL.
      REAL RISO,RANO,SIGM
C
C     Array for REVISE and FINDFM.
      REAL FPT(NUMFO),SFPT(NUMFO),FNT(NUMFO),SFNT(NUMFO),DT(NUMFO),
     +     SDT(NUMFO),FPRT(NUMFO),FDPT(NUMFO)
C
C     Array for statistics.
      INTEGER NRES1(NRMAX),NRES2(NRMAX),NRAT1(NRMAX,NUMFO,NUMFO),NRSALL1
     +       ,NRAT2(NRMAX,NUMFO,NUMFO),NR1ALL(NUMFO,NUMFO),NRSALL2
     +       ,NR2ALL(NUMFO,NUMFO),NUMAL1(-NCMAX:NCMAX,NRMAX,NUMFO,NUMFO)
     +       ,NTOTA1(-NCMAX:NCMAX,NUMFO,NUMFO)
      REAL SSQ4(NRMAX),RES(NRMAX),RAT1(NRMAX,NUMFO,NUMFO)
     +    ,RAT2(NRMAX,NUMFO,NUMFO),R1ALL(NUMFO,NUMFO),R2ALL(NUMFO,NUMFO)
     +    ,CCD(NRMAX,NUMFO,NUMFO),CCDD(NRMAX,NUMFO,NUMFO),
     +     CCDE(NRMAX,NUMFO,NUMFO)
C
C     Local Scalars.
      REAL SS4,SMAX,SMIN
      INTEGER I,IC,IFAIL,IFIL1,IFLAG,IPRINT,JDO30,JDO35,JDO40,JDO50
      INTEGER JDO60,JDO130,JDO300,LUNOUT,NCOL,NF1,NG1,NLPRGI,NSPGRP
      INTEGER NSYM,NSYMP,NTOK,STARTFP,STARTFO,NLPRGO,NWAVE,NCOUNT
      INTEGER ICHKP1,ICHKP2,ICHKPH1(NUMFO),ICHKPH2(NUMFO),NF2,NG2,JDO45
      INTEGER NCHKPH1(NUMFO),NCHKPH2(NUMFO),NEXCL1(NUMFO),NEXCL2(NUMFO),
     +        NESUM1(NUMFO),NESUM2(NUMFO)
      CHARACTER NAMSPG*10,PGNAM*10,TITLE*80
      LOGICAL ENDFIL,LTITLE
C 
C----Harvesting stuff
C
      INTEGER NDATASETS,ISETS(MSETS),ISET,CSETID(MCOLS),
     +         CSETOUT(MCOLS),SETID,IDUMMY
      REAL DATCELL(6,MSETS),DATWAVE(MSETS)
      CHARACTER*64 PNAME(MSETS),XNAME(MSETS),DNAME(MSETS),   
     +         XNAME_OUT(MCOLS),DNAME_OUT(MCOLS)
C
C     Variables for Parser.
      REAL FVALUE(NKEYWD)
      INTEGER IBEG(NKEYWD),IDEC(NKEYWD),IEND(NKEYWD),ITYPE(NKEYWD)
      CHARACTER KEY*4,CVALUE(NKEYWD)*4,LINE*2000,LTYPE*1,WORD*8,
     +          LABOUT_SAVE*2000
      LOGICAL LEND
C
C     External Routines.
      EXTERNAL CCPERR,CCPFYP,CCPRCS,CENTR,CENTRIC,EQUAL_MAGIC,LKYIN,
     +         LROPEN,LRASSN,LRCLAB,LRCLOS,LRREFM,LRSYMI,LRSYMM,
     +         LWOPEN,LWASSN,LWTITL,LWREFL,MTZINI,PARSER,GTPINT,CCPUPC,
     +         GTPREA,LRRSOL,LKYOUT,LRREFF,LWCLOS,LWHSTL,
     +         LRCLID

C     
C     Data Statements.
      DATA NLPRGI,NLPRGO,NWAVE,NCOUNT/87,0,0,0/
      DATA LTITLE/.FALSE./
      DATA RISO/0.10/,RANO/0.50/,SIGM/0.0/
      DATA LSPRGI/'H','K','L',
     +      'FP',     'SIGFP',     'DP',     'SIGDP',
     +      'FPH1(+)',  'SIGFPH1(+)',  'FPH1(-)',  'SIGFPH1(-)',
     +      'FPH1',   'SIGFPH1',   'DPH1',   'SIGDPH1',
     +      'FPH2(+)',  'SIGFPH2(+)',  'FPH2(-)',  'SIGFPH2(-)',
     +      'FPH2',   'SIGFPH2',   'DPH2',   'SIGDPH2',
     +      'FPH3(+)',  'SIGFPH3(+)',  'FPH3(-)',  'SIGFPH3(-)',
     +      'FPH3',   'SIGFPH3',   'DPH3',   'SIGDPH3',
     +      'FPH4(+)',  'SIGFPH4(+)',  'FPH4(-)',  'SIGFPH4(-)',
     +      'FPH4',   'SIGFPH4',   'DPH4',   'SIGDPH4',
     +      'FPH5(+)',  'SIGFPH5(+)',  'FPH5(-)',  'SIGFPH5(-)',
     +      'FPH5',   'SIGFPH5',   'DPH5',   'SIGDPH5',
     +      'FPH6(+)',  'SIGFPH6(+)',  'FPH6(-)',  'SIGFPH6(-)',
     +      'FPH6',   'SIGFPH6',   'DPH6',   'SIGDPH6',
     +      'FPH7(+)',  'SIGFPH7(+)',  'FPH7(-)',  'SIGFPH7(-)',
     +      'FPH7',   'SIGFPH7',   'DPH7',   'SIGDPH7',
     +      'FPH8(+)',  'SIGFPH8(+)',  'FPH8(-)',  'SIGFPH8(-)',
     +      'FPH8',   'SIGFPH8',   'DPH8',   'SIGDPH8',
     +      'FPH9(+)',  'SIGFPH9(+)',  'FPH9(-)',  'SIGFPH9(-)',
     +      'FPH9',   'SIGFPH9',   'DPH9',   'SIGDPH9',
     +      'FPH10(+)', 'SIGFPH10(+)', 'FPH10(-)', 'SIGFPH10(-)',
     +      'FPH10',  'SIGFPH10',  'DPH10',  'SIGDPH10',
     +       413*' '/
C
      DATA CTPRGI/'H','H','H',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +       413*' '/
C
      DATA LSPRGO/'H','K','L',
     +      'FP',       'SIGFP',       'DP',       'SIGDP',
     +      'FPH1(+)',    'SIGFPH1(+)',    'FPH1(-)',    'SIGFPH1(-)',
     +      'FPHM1(+)',   'SIGFPHM1(+)',   'FPHM1(-)',   'SIGFPHM1(-)',
     +      'FPH1',     'SIGFPH1',     'DPH1',     'SIGDPH1',
     +      'FPHM1',    'SIGFPHM1',    'DPHM1',    'SIGDPHM1',
     +      'FPH2(+)',    'SIGFPH2(+)',    'FPH2(-)',    'SIGFPH2(-)',
     +      'FPHM2(+)',   'SIGFPHM2(+)',   'FPHM2(-)',   'SIGFPHM2(-)',
     +      'FPH2',     'SIGFPH2',     'DPH2',     'SIGDPH2',
     +      'FPHM2',    'SIGFPHM2',    'DPHM2',    'SIGDPHM2',
     +      'FPH3(+)',    'SIGFPH3(+)',    'FPH3(-)',    'SIGFPH3(-)',
     +      'FPHM3(+)',   'SIGFPHM3(+)',   'FPHM3(-)',   'SIGFPHM3(-)',
     +      'FPH3',     'SIGFPH3',     'DPH3',     'SIGDPH3',
     +      'FPHM3',    'SIGFPHM3',    'DPHM3',    'SIGDPHM3',
     +      'FPH4(+)',    'SIGFPH4(+)',    'FPH4(-)',    'SIGFPH4(-)',
     +      'FPHM4(+)',   'SIGFPHM4(+)',   'FPHM4(-)',   'SIGFPHM4(-)',
     +      'FPH4',     'SIGFPH4',     'DPH4',     'SIGDPH4',
     +      'FPHM4',    'SIGFPHM4',    'DPHM4',    'SIGDPHM4',
     +      'FPH5(+)',    'SIGFPH5(+)',    'FPH5(-)',    'SIGFPH5(-)',
     +      'FPHM5(+)',   'SIGFPHM5(+)',   'FPHM5(-)',   'SIGFPHM5(-)',
     +      'FPH5',     'SIGFPH5',     'DPH5',     'SIGDPH5',
     +      'FPHM5',    'SIGFPHM5',    'DPHM5',    'SIGDPHM5',
     +      'FPH6(+)',    'SIGFPH6(+)',    'FPH6(-)',    'SIGFPH6(-)',
     +      'FPHM6(+)',   'SIGFPHM6(+)',   'FPHM6(-)',   'SIGFPHM6(-)',
     +      'FPH6',     'SIGFPH6',     'DPH6',     'SIGDPH6',
     +      'FPHM6',    'SIGFPHM6',    'DPHM6',    'SIGDPHM6',
     +      'FPH7(+)',    'SIGFPH7(+)',    'FPH7(-)',    'SIGFPH7(-)',
     +      'FPHM7(+)',   'SIGFPHM7(+)',   'FPHM7(-)',   'SIGFPHM7(-)',
     +      'FPH7',     'SIGFPH7',     'DPH7',     'SIGDPH7',
     +      'FPHM7',    'SIGFPHM7',    'DPHM7',    'SIGDPHM7',
     +      'FPH8(+)',    'SIGFPH8(+)',    'FPH8(-)',    'SIGFPH8(-)',
     +      'FPHM8(+)',   'SIGFPHM8(+)',   'FPHM8(-)',   'SIGFPHM8(-)',
     +      'FPH8',     'SIGFPH8',     'DPH8',     'SIGDPH8',
     +      'FPHM8',    'SIGFPHM8',    'DPHM8',    'SIGDPHM8',
     +      'FPH9(+)',    'SIGFPH9(+)',    'FPH9(-)',    'SIGFPH9(-)',
     +      'FPHM9(+)',   'SIGFPHM9(+)',   'FPHM9(-)',   'SIGFPHM9(-)',
     +      'FPH9',     'SIGFPH9',     'DPH9',     'SIGDPH9',
     +      'FPHM9',    'SIGFPHM9',    'DPHM9',    'SIGDPHM9',
     +      'FPH10(+)',   'SIGFPH10(+)',   'FPH10(-)',   'SIGFPH10(-)',
     +      'FPHM10(+)',  'SIGFPHM10(+)',  'FPHM10(-)',  'SIGFPHM10(-)',
     +      'FPH10',    'SIGFPH10',    'DPH10',    'SIGDPH10',
     +      'FPHM10',   'SIGFPHM10',   'DPHM10',   'SIGDPHM10',
     +      'FM',       'SIGFM',
     +       331*' '/
C
      DATA CTPRGO/'H','H','H',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'G','L','G','L',
     +      'G','L','G','L',
     +      'F','Q','D','Q',
     +      'F','Q','D','Q',
     +      'F','Q',
     +       331*' '/
C
      DATA LOOKUP/-1,-1,-1,497*0/
      DATA NDT/NUMFO*0/
      DATA LAM/NUMFO*0.0/
      DATA FPR/NUMFO*0.0/
      DATA FDP/NUMFO*0.0/
C 
C     Starting indices of program label blocks
      STARTFP = 3
      STARTFO = STARTFP + 4
      LUNOUT = 6
      IFIL1 = 1
      SMIN = 0
      SMAX = 9.00
      NDATA = 0
      NDALL = 0
      NUMBM = 0
      LABOUT_SAVE = ' '

      do i=1,mcols
        xname_out(i) = ' '
        dname_out(i) = ' '
      enddo

      CALL CCPFYP
      CALL MTZINI
      CALL CCPRCS(LUNOUT,'REVISE','$Date: 2008/05/19 15:26:49 $')
      WRITE (6, 6000)
 6000 FORMAT (/,
     +     ' $TEXT:Reference1: $$ comment $$ ',
     +     ' "New Techniques of Applying Multi-Wavelength Anomalous ',
     +     'Scattering Data.",',/,
     +     ' Fan Hai-fu, Woolfson, M.M.,Yao Jia-xing, (1993). ',
     +     'Proc. R.Soc. Lond.', //,' $$',/,
     +     ' $SUMMARY :Reference1:  $$ Revise: $$',/,
     +     ' :TEXT:Reference1: $$')

C     Read keyword input.
 20   CONTINUE
      KEY  = ' '
      LINE = ' '
C     Reset number of words on line to NKEYWD
      NTOK = NKEYWD
C
C     **********************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYPE,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +     .TRUE.)
C     **********************************************************
C
      IF (LEND .OR. KEY(1:3).EQ.'END') GO TO 200
      IF (KEY.EQ.'TITL') THEN
C      Read new title.
       IF (NTOK.GE.2) THEN
        LTITLE = .TRUE.
        TITLE = LINE(IBEG(2) :IEND(NTOK))
        WRITE(LUNOUT,'(10X,A,/,15X,A80,/)')   
     +       'TITLE OF MTZ FILE REPLACED BY:',TITLE
       ENDIF
       GO TO 20
C==== RESOLUTION (maximum limit only has meaning in revise)
      ELSE IF (KEY.EQ.'RESO') THEN
       CALL RDRESO(2,ITYPE,FVALUE,NTOK,RESMAX,RESMIN,SMIN,SMAX)
       GO TO 20
C
      ELSE IF (KEY.EQ.'WAVE') THEN
C      Read wavelength, f' and f" for a data set.
       NWAVE = NWAVE + 1
       ITOK = 2
C      **********************************
       CALL GTPINT(ITOK,NDATA,NTOK,ITYPE,FVALUE)
C      **********************************
       IF (NDATA.EQ.0)  CALL CCPERR(1,
     +  ' ***Error - Ndata must be greater than zero ***')
       NDT(NDATA) = NDATA
       IF (NDATA.GT.NUMFO)  CALL CCPERR(1,
     +  ' ***Error - Ndata too large - limit is set by NUMFO***')
       ITOK = ITOK + 1
       IF (ITOK.GT.NTOK) GO TO 20
C
   25  CONTINUE
       IF (ITYPE(ITOK).EQ.2) CALL CCPERR(1,
     +  ' ***Error - should have word LAM, FPR or FDP **')
C
C      Next word LAMBDA, FPRIME or FDOUBLEPRIME
C
       WORD = LINE(IBEG(ITOK) :IEND(ITOK))
       CALL CCPUPC(WORD)
       ITOK = ITOK + 1
       IF (ITOK.GT.NTOK) GO TO 20
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' ***Error - should have a number for LAM, FPR or FDP **')
C      **********************************
       CALL GTPREA(ITOK,FFA,NTOK,ITYPE,FVALUE)
C      **********************************
       IF ( WORD(1:3) .EQ.'LAM') THEN
        LAM(NDATA) = FFA
       ELSE IF ( WORD(1:3) .EQ.'FPR') THEN
        FPR(NDATA) = FFA
       ELSE IF ( WORD(1:3) .EQ.'FDP') THEN
        FDP(NDATA) = FFA
        IF (FFA.LT.0.0) CALL CCPERR(1,
     +   ' ***Error - f" should be positive ****')
       ELSE  
        CALL CCPERR(1,' word in WAVE unrecognised')
       END IF 
C
       ITOK = ITOK + 1
       IF (ITOK.GT.NTOK) GO TO 20
       GO TO 25
C
      ELSE IF (KEY.EQ.'LABI') THEN
C      Input labels
C        *****************************************
         CALL LKYIN(IFIL1,LSPRGI,NLPRGI,NTOK,LINE,IBEG,IEND)
C        *****************************************
       IF (NTOK.EQ.1) CALL CCPERR(1,
     +   'No columns assigned. Nothing to do. Bye-bye!')
       WRITE(LUNOUT,'(10X,I3,X,A,/,10X,A,A,/)')   
     +  (NTOK-1)/2, 'column labels have been assigned.',
     +   'NOTE: missing number analysis only performed ',
     +   'for these columns!!'
       GO TO 20
C
      ELSE IF (KEY.EQ.'LABO') THEN
C      Save output labels
       LABOUT_SAVE = LINE
       GO TO 20
C
      ELSE IF (KEY.EQ.'EXCL') THEN
       ITOK = 2
C
   26  CONTINUE
C
       IF (ITYPE(ITOK).EQ.2) CALL CCPERR(1,
     +  ' ***Error - should have word RISO, RANO or SIGM ***')
C
C      Next word RISO, RANO or SIGM.
C
       WORD = LINE(IBEG(ITOK) :IEND(ITOK))
       CALL CCPUPC(WORD)
       ITOK = ITOK + 1
       IF (ITOK.GT.NTOK) GO TO 20
C
       IF ( WORD(1:3) .EQ. 'RIS') THEN
C       Read the ratio of RISO ( = <DISO>/<FPH>).
        IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for RISO ***')
C       **********************************
        CALL GTPREA(ITOK,RISO,NTOK,ITYPE,FVALUE)
C       **********************************
C
        ITOK = ITOK + 1
        IF (ITOK.GT.NTOK) GO TO 20
        GO TO 26
C
       ELSE IF ( WORD(1:3) .EQ. 'RAN') THEN
C       Read the ratio of RANO ( = |DANO|/|FPH|).
        IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for RANO ***')
C       **********************************
        CALL GTPREA(ITOK,RANO,NTOK,ITYPE,FVALUE)
C       **********************************
C
        ITOK = ITOK + 1
        IF (ITOK.GT.NTOK) GO TO 20
        GO TO 26
C
       ELSE IF ( WORD(1:3) .EQ. 'SIG') THEN
C       Read the Sigma cutoff SIGM.
        IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for SIGM ***')
C       **********************************
        CALL GTPREA(ITOK,SIGM,NTOK,ITYPE,FVALUE)
C       **********************************
C
        ITOK = ITOK + 1
        IF (ITOK.GT.NTOK) GO TO 20
        GO TO 26
       ELSE
        CALL CCPERR(1,' word in EXCL unrecognised')
       END IF
C
      ELSE IF (KEY.EQ.'END') THEN
       WRITE(LUNOUT,'(/)')   
       GO TO 200
      ELSE 
       CALL CCPERR(1,'  kEY word unrecognised')
      END IF
C
C     End of keyword input
C
 200  CONTINUE
      IF (NWAVE.LE.1) THEN 
       WRITE(LUNOUT,FMT='(/,A,/)')
     +  'REVISE needs at least two wavelengths input.'
       CALL CCPERR(1,'  Not enough wavelength input.')
      END IF
C
      IPRINT = 2
C     **********************************
      CALL LROPEN(IFIL1,'HKLIN',IPRINT,IFAIL)
C     **********************************
      IF (IFAIL.NE.0) CALL CCPERR(1, ' No input file???????')
C
C     ****************************************
      CALL LRASSN(IFIL1,LSPRGI,NLPRGI,LOOKUP,CTPRGI)
C     *****************************************
C
C     *****************************************
      CALL LRCLAB(IFIL1,LSUSRI,CTUSRI,NCOL)
C     *****************************************

C---- Dataset stuff
C     Notify available space

      do i=1,mcols
        csetout(i) = -999
      enddo
      do i=1,msets
        isets(i) = -998
      enddo
      NDATASETS = MSETS
      CALL LRIDX(IFIL1,PNAME,XNAME,DNAME,ISETS,
     +              DATCELL,DATWAVE,NDATASETS)

C---- Get dataset IDs for columns.
      IF (NDATASETS.GT.0) THEN
        CALL LRCLID(IFIL1,CSETID,IDUMMY)
      ENDIF
C
C     Get resolution range unless it is already set on RESO input
C     *****************************************
      IF(SMAX.EQ.9.0) CALL LRRSOL(IFIL1,SMIN,SMAX)
C     *****************************************
C
C     Initial scalars for statistics.
C
      SDEV = SMAX/REAL(NRMAX)
      NRSALL1 = 0
      NRSALL2 = 0
      DO 202 I = 1, NRMAX
      NRES1(I) = 0
      NRES2(I) = 0
      SSQ4(I) = SDEV*REAL(I)
      RES(I) = SQRT(1.0/SSQ4(I))
      DO 202 J = 1, NUMFO
      DO 202 K = 1, NUMFO
      CCD(I,J,K) = 0.0
      CCDD(I,J,K) = 0.0
      CCDE(I,J,K) = 0.0
      NRAT1(I,J,K) = 0
      NRAT2(I,J,K) = 0
      RAT1(I,J,K) = 0.0
      RAT2(I,J,K) = 0.0
       DO 201 L = -NCMAX, NCMAX
       NUMAL1(L,I,J,K) = 0
       NTOTA1(L,J,K) = 0
  201  CONTINUE
  202 CONTINUE
      DO 203 J = 1, NUMFO
       NESUM1(J) = 0
       NESUM2(J) = 0
      DO 203 K = 1, NUMFO
       NR1ALL(J,K) = 0
       NR2ALL(J,K) = 0
       R1ALL(J,K) = 0.0
       R2ALL(J,K) = 0.0
  203 CONTINUE
C
C     Test correct column types.
C
      Do 205 I = 1, MCOLS
       IF (LOOKUP(I).NE.0) THEN
        NF1 = LOOKUP(I)
        IF (CTUSRI(NF1).EQ.CTPRGI(I) ) GO TO 205
        IF (CTUSRI(NF1).NE.CTPRGI(I) ) THEN
          WRITE (LUNOUT,FMT='(A,I3,3x,A10,A4,10x, A20,A4)')
     +    ' ** Column type does not match assignment:',
     +     I,LSPRGI(I), CTPRGI(I),LSUSRI(NF1),CTUSRI(NF1)
         IF (( CTUSRI(NF1).EQ.'F'.AND.CTPRGI(I).EQ.'G')
     +   .OR.( CTUSRI(NF1).EQ.'Q'.AND.CTPRGI(I).EQ.'L')
     +   .OR.CTUSRI(NF1).EQ.'I'.OR.CTUSRI(NF1).EQ.'R') THEN
          WRITE (LUNOUT,FMT='(/,A,A,/A,/)')
     +     ' Warning: - Input column type is  REAL or INTEGER  or  F',
     +     ' instead of G   or   Q instead of L ',
     +     ' It has been reset to program assignment.'
         ELSE
          CALL CCPERR(1,
     +     ' Column type does not match assignment')
         ENDIF
        END IF 
       END IF
205   CONTINUE
      NLPRGO = 3
      NCOUNT = 3
C
C     Test that SIGFP present if FP specified, etc, etc
C     Test FP present if DP specified.
C
      DO 30 JDO30 = STARTFP,STARTFP+3,4
       ICHKP1 = LOOKUP(JDO30+1)*LOOKUP(JDO30+2)
       ICHKP2 = LOOKUP(JDO30+3)*LOOKUP(JDO30+4)
       IF (LOOKUP(JDO30+1).NE.0 .NEQV. LOOKUP(JDO30+2).NE.0) THEN
        WRITE (LUNOUT,FMT='(/,A,/)')
     +   'Unmatched FP or SIGFP',' assigned.'
        CALL CCPERR(1,' Unmatched FP/SIGFP pair')
       ENDIF
       IF (LOOKUP(JDO30+3).NE.0 .NEQV. LOOKUP(JDO30+4).NE.0) THEN
        WRITE (LUNOUT,FMT='(/,A,/)')
     +   'Unmatched DP or SIGDP',' assigned.'
        CALL CCPERR(1,' Unmatched D/SIGD pair')
       ENDIF
       IF (LOOKUP(JDO30+3).NE.0 .AND. LOOKUP(JDO30+1).EQ.0) THEN
        WRITE (LUNOUT,FMT='(/,A,/)')
     +   'DP assigned but no corresponding FP'
        CALL CCPERR(1,' Unmatched FP/DP pair')
       ENDIF
C
C      Manage output labels for native data.
C      Input and output file contains:
C       FP SIGFP if ICHKP1 > 0.
C       DP SIGDP if ICHKP2 > 0.
C
       IF ( ICHKP1.GT.0) THEN
        NLPRGO = NLPRGO + 1
        NCOUNT = NCOUNT + 1
        LSPRGO(NLPRGO) = LSPRGO(NCOUNT)
        CTPRGO(NLPRGO) = CTPRGO(NCOUNT)
        IF (NDATASETS.GT.0) CSETOUT(NLPRGO)=CSETID(LOOKUP(4))
        NLPRGO = NLPRGO + 1
        NCOUNT = NCOUNT + 1
        LSPRGO(NLPRGO) = LSPRGO(NCOUNT)
        CTPRGO(NLPRGO) = CTPRGO(NCOUNT)
        IF (NDATASETS.GT.0) CSETOUT(NLPRGO)=CSETID(LOOKUP(5))
       ELSE
        NCOUNT = NCOUNT + 2
       ENDIF
       IF ( ICHKP2.GT.0) THEN
        NLPRGO = NLPRGO + 1
        NCOUNT = NCOUNT + 1
        LSPRGO(NLPRGO) = LSPRGO(NCOUNT)
        CTPRGO(NLPRGO) = CTPRGO(NCOUNT)
        IF (NDATASETS.GT.0) CSETOUT(NLPRGO)=CSETID(LOOKUP(6))
        NLPRGO = NLPRGO + 1
        NCOUNT = NCOUNT + 1
        LSPRGO(NLPRGO) = LSPRGO(NCOUNT)
        CTPRGO(NLPRGO) = CTPRGO(NCOUNT)
        IF (NDATASETS.GT.0) CSETOUT(NLPRGO)=CSETID(LOOKUP(7))
       ELSE
        NCOUNT = NCOUNT + 2
       ENDIF
 30   CONTINUE
C
C     Test that SIGF present if F specified.
C     Test F present if D specified.
C     Test that both FPHn+ and FPHn- specified.
C
      NF1 = 0
      DO 35 JDO35 = STARTFO,STARTFO+(NUMFO*8-1),8
       NF1 = NF1 + 1
       IF (LOOKUP(JDO35+1).NE.0 .NEQV. LOOKUP(JDO35+2).NE.0) THEN
        WRITE (LUNOUT,FMT='(/,A,I2,A,I2,A,/)')
     +   'Unmatched F',NF1,'+ or SIGF',NF1,'+ assigned.'
        CALL CCPERR(1,' Unmatched FPHn+/SIGFPHn+ pair')
       ENDIF 
       IF (LOOKUP(JDO35+3).NE.0 .NEQV. LOOKUP(JDO35+4).NE.0) THEN
        WRITE (LUNOUT,FMT='(/,A,I2,A,I2,A,/)')
     +   'Unmatched F',NF1,'- or SIGF',NF1,'- assigned.'
        CALL CCPERR(1,' Unmatched FPHn-/SIGFPHn- pair')
       ENDIF 
       IF (LOOKUP(JDO35+3).NE.0 .NEQV. LOOKUP(JDO35+1).NE.0) THEN
        WRITE (LUNOUT,FMT='(/,A,I2,A,I2,/)')
     +   'Only one of F',NF1,'+ or F',NF1,'- assigned.'
        CALL CCPERR(1,' Unmatched FPHn+/FPHn- pair')
       ENDIF 
       IF (LOOKUP(JDO35+5).NE.0 .NEQV. LOOKUP(JDO35+6).NE.0) THEN
        WRITE (LUNOUT,FMT='(/,A,I2,A,I2,A,/)')
     +   'Unmatched F',NF1,' or SIGF',NF1,' assigned.'
        CALL CCPERR(1,' Unmatched F/SIGF pair')
       ENDIF
       IF (LOOKUP(JDO35+7).NE.0 .NEQV. LOOKUP(JDO35+8).NE.0) THEN
        WRITE (LUNOUT,FMT='(/,A,I2,A,I2,A,/)')
     +   'Unmatched D',NF1,' or SIGD',NF1,' assigned.'
        CALL CCPERR(1,' Unmatched D/SIGD pair')
       ENDIF
       IF (LOOKUP(JDO35+5).NE.0 .AND. LOOKUP(JDO35+7).EQ.0) THEN
        WRITE (LUNOUT,FMT='(/,A,I2,A,I2,A,/)')
     +   'D',NF1,' assigned but no corresponding F',NF1
        CALL CCPERR(1,' Unmatched F/D pair')
       ENDIF
C
C      Manage output labels for MAD data.
C
C      Input file contains:
C       FPHn+  SIGFPHn+  FPHn-  SIGFPHn-        if ICHKPH1 > 0.
C       FPHn   SIGFPHn   DPHn   SIGDPHn         if ICHKPH2 > 0.
C
C      Output file contains:
C       FPHn+  SIGFPHn+  FPHn-  SIGFPHn-   and 
C       FPHMn+ SIGFPHMn+ FPHMn- SIGFPHMn-       if ICHKPH1 > 0.
C       FPHn   SIGFPHn   DPHn   SIGDPHn    and
C       FPHMn  SIGFPHMn  DPHMn  SIGDPHMn        if ICHKPH2 > 0.
C
       ICHK1 =
     +  LOOKUP(JDO35+1)*LOOKUP(JDO35+2)*LOOKUP(JDO35+3)*LOOKUP(JDO35+4)
       ICHK2 =
     +  LOOKUP(JDO35+5)*LOOKUP(JDO35+6)*LOOKUP(JDO35+7)*LOOKUP(JDO35+8)
C      Save the ICHKPH.
       ICHKPH1(NF1) = ICHK1
       ICHKPH2(NF1) = ICHK2
C      Total number of the data sets.
       IF (ICHK1.GT.0.OR.ICHK2.GT.0) NDALL = NDALL + 1
       IF (ICHK1.GT.0) THEN
        DO 32 I = 1, 8
         NLPRGO = NLPRGO + 1
         NCOUNT = NCOUNT + 1
         LSPRGO(NLPRGO) = LSPRGO(NCOUNT)
         CTPRGO(NLPRGO) = CTPRGO(NCOUNT)
         IF (NDATASETS.GT.0) CSETOUT(NLPRGO)=CSETID(LOOKUP(JDO35+1))
   32   CONTINUE
       ELSE
        NCOUNT = NCOUNT + 8
       ENDIF
       IF (ICHK2.GT.0) THEN
        DO 34 I = 1, 8
         NLPRGO = NLPRGO + 1
         NCOUNT = NCOUNT + 1
         LSPRGO(NLPRGO) = LSPRGO(NCOUNT)
         CTPRGO(NLPRGO) = CTPRGO(NCOUNT)
         IF (NDATASETS.GT.0) CSETOUT(NLPRGO)=CSETID(LOOKUP(JDO35+5))
   34   CONTINUE
       ELSE
        NCOUNT = NCOUNT + 8
       ENDIF
 35   CONTINUE
C
C     Manage output labels for FM SIGFM.
C
      DO 36 I = 1, 2
        NLPRGO = NLPRGO + 1
        LSPRGO(NLPRGO) = LSPRGO(167 + I)
        CTPRGO(NLPRGO) = CTPRGO(167 + I)
C I have allowed for different wavelengths being assigned to different
C datasets. But not clear which FM SIGFM should then be assigned to.
C Cop out by using dataset of index H.
        IF (NDATASETS.GT.0) CSETOUT(NLPRGO)=CSETID(LOOKUP(1))
   36 CONTINUE
C
C     Check the number of data sets.
C
      IF (NDALL.LE.1) THEN
       WRITE(LUNOUT,FMT='(/,A,/)')
     +  'REVISE needs at least two data sets.'
       CALL CCPERR(1,'  Not enough data sets.')
      END IF
C
C     Set up information to decide centric zones.
C
      CALL LRSYMI(IFIL1,NSYMP,LTYPE,NSPGRP,NAMSPG,PGNAM)
      CALL LRSYMM(IFIL1,NSYM,RSYML)
      CALL CENTRIC(NSYM,RSYML,IPRINT)
C
C     Check and print the input f' and f".
C
      WRITE(LUNOUT,FMT='(//,A,/)')
     + ' Ndata Wavelengths      Fprime   Fdoublepr'
      DO 38 I = 1, NDALL
       IF (NDT(I).GT.0.AND.ABS(FPR(I)).LT.0.001) THEN
        WRITE(LUNOUT,FMT='(/,A,I4,/)')
     +   'Keyword FPR must be given for wavelength', NDT(I)
         CALL CCPERR(1,' FPR must be given')
       END IF
       IF (NDT(I).GT.0.AND.FDP(I).LT.0.001) THEN
        WRITE(LUNOUT,FMT='(/,A,I4,/)')
     +   'Keyword FDP must be given for wavelength', NDT(I)
         CALL CCPERR(1,' FDP must be given')
       END IF
       IF (NDT(I).GT.0) WRITE(LUNOUT,FMT='(I6,3F12.4)')
     +   NDT(I),LAM(I),FPR(I),FDP(I)
  38  CONTINUE 
C
C     Print the ratio of RISO, RANO and SIGM.
C
      WRITE(LUNOUT,'(//,1X,A,F8.3,/)')
     +   'Data will be excluded if the ratio of (<|DISO|>/<FPH>) > ',
     +  RISO
      WRITE(LUNOUT,'(/,1X,A,F8.3,/)')
     +   'Data will be excluded if the ratio of (|DANO|/FPH) > ',
     +  RANO
      WRITE(LUNOUT,'(/,1X,A,F8.3,A,/)')
     +   'Data will be excluded if FPH < ',SIGM,'*SIGFPH.'
C
      WRITE(LUNOUT,FMT='(//)')
C
C     Open output file.
C
C     **********************
      CALL LWOPEN(IFIL1,'HKLOUT')
C     Parse the "LABOUT" line now.
      LINE = LABOUT_SAVE
      NTOK = MCOLS*2 + 1
C     ******************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYPE,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +     .TRUE.)
      CALL LKYOUT(1,LSPRGO,NLPRGO,NTOK,LINE,IBEG,IEND)
      CALL LWASSN(IFIL1,LSPRGO,NLPRGO,CTPRGO,0)
C     ******************************************************

C Store the column dataset IDs:
      IF (NDATASETS.GT.0) THEN
        DO 210 I = 1,NLPRGO
          SETID = CSETOUT(I)
          DO ISET = 1,NDATASETS
            IF (ISETS(ISET).EQ.SETID) THEN
              XNAME_OUT(I) = XNAME(ISET)
              DNAME_OUT(I) = DNAME(ISET)
            END IF
          ENDDO
 210    CONTINUE

        CALL LWIDASX(IFIL1,NLPRGO,XNAME_OUT,DNAME_OUT,0)
      ENDIF

C
C     Set IFLAG = 0 to replace old title if LTITLE = .TRUE..
C
      IFLAG = 0
      IF (LTITLE) CALL LWTITL(IFIL1,TITLE,IFLAG)
C     Write a history line.
      CALL LWHSTL (IFIL1, 'revise MAD data')
C
C     Loop for reflections.
C
 10   CONTINUE
C
C     Clear all aniso info
      DO II = 1, NDALL
        BUF1(II)=0
        FPRT(II)=0
        FDPT(II)=0
        FPT(II)=0
        SFPT(II)=0
        FNT(II)=0
        SFNT(II)=0
        DT(II)=0
        SDT(II)=0
      END DO 
C
      FM1 = 0.0
      SFM1 = 0.0
      FM2 = 0.0
      SFM2 = 0.0
C     **************************
      CALL LRREFF(IFIL1,SS4,ADATA,ENDFIL)
C     **************************
      IF (ENDFIL) GO TO 1000
      IF(SS4.GT.SMAX) GO TO 10

C     Set LOGMSS(i) as .TRUE. if the data item is missing.
C     **************************
      CALL LRREFM(IFIL1,LOGMSS)
C     **************************
C     Start assigning the whole list of output as "missing" then replace.
      CALL EQUAL_MAGIC(IFIL1,BDATA,NLPRGO)
C     Copy index.
      BDATA(1) = ADATA(1)
      BDATA(2) = ADATA(2)
      BDATA(3) = ADATA(3)
C     Test whether a reflection is centric (ic=1) or not (ic=0).
      IN(1) = NINT(ADATA(1))
      IN(2) = NINT(ADATA(2))
      IN(3) = NINT(ADATA(3))
      NBD = 3
      NSS = INT(SS4/SDEV) + 1
      IF (NSS.GT.NRMAX) NSS = NRMAX
      CALL CENTR(IN,IC)
C 
C     Copy FP, SIGFP, DP, SIGDP.
C
      DO 40 JDO40 = STARTFP,STARTFP+3,4
       IF (ICHKP1.GT.0) THEN
        IF (.NOT.LOGMSS(JDO40+1) .AND..NOT.LOGMSS(JDO40+2)) THEN
         BDATA(NBD+1) = ADATA(JDO40+1)
         BDATA(NBD+2) = ADATA(JDO40+2)
        END IF
        NBD = NBD + 2
       END IF
       IF (ICHKP2.GT.0) THEN
        IF (.NOT.LOGMSS(JDO40+3) .AND..NOT.LOGMSS(JDO40+4)) THEN
         BDATA(NBD+3) = ADATA(JDO40+3)
         BDATA(NBD+4) = ADATA(JDO40+4)
        END IF
C Set centric DANO  = 0
        IF( IC.EQ.1) THEN
         BDATA(NBD+3) = 0.0
         BDATA(NBD+4) = 0.0
        END IF
        NBD = NBD + 2
       END IF
 40   CONTINUE   
C     Check if the data are available and mark the excluded data.
      NF1 = 0
      DO 45 JDO45 = STARTFO, STARTFO+NDALL*8-1, 8
       NF1 = NF1 + 1
       NCHKPH1(NF1) = -NF1
       NCHKPH2(NF1) = -NF1
       NEXCL1(NF1) = NF1
       NEXCL2(NF1) = NF1
C
C    Correct any possible problems for centrics
       IF ( IC.EQ.1) THEN
        IF (ICHKPH1(NF1).GT.0 ) THEN
C   Copy F- SIGi = F+ SIG+ if F- etc missing
          IF(LOGMSS(JDO45+3) .OR. LOGMSS(JDO45+4)) THEN
             LOGMSS(JDO45+3) = LOGMSS(JDO45+1)
             ADATA(JDO45+3)  = ADATA(JDO45+1)
             LOGMSS(JDO45+4) = LOGMSS(JDO45+2)
             ADATA(JDO45+4)  = ADATA(JDO45+2)
          END IF
C   Copy F+ SIG+ = F- SIG- if F+ etc missing
          IF(LOGMSS(JDO45+1) .OR. LOGMSS(JDO45+2)) THEN
             LOGMSS(JDO45+1) = LOGMSS(JDO45+3)
             ADATA(JDO45+1)  = ADATA(JDO45+3)
             LOGMSS(JDO45+2) = LOGMSS(JDO45+4)
             ADATA(JDO45+2)  = ADATA(JDO45+4)
          END IF
C   Average F+ and F-
          IF(.NOT.LOGMSS(JDO45+1) .AND. .NOT.LOGMSS(JDO45+2) .AND.
     +       .NOT.LOGMSS(JDO45+3) .AND. .NOT.LOGMSS(JDO45+4)) THEN
             AVER = 0.5* ( ADATA(JDO45+1) + ADATA(JDO45+3))
              ADATA(JDO45+1) = AVER
              ADATA(JDO45+3) = AVER
             AVER = SQRT( 0.5* ( ADATA(JDO45+2)*ADATA(JDO45+2) 
     +                         + ADATA(JDO45+4)*ADATA(JDO45+2)))
              ADATA(JDO45+2) = AVER
              ADATA(JDO45+4) = AVER
          END IF
        END IF  
C   Set DPH SIGDPH = 0 
        IF (ICHKPH2(NF1).GT.0 ) THEN
             LOGMSS(JDO45+7) = .FALSE.
             ADATA(JDO45+7)  = 0.0
             LOGMSS(JDO45+8) = .FALSE.
             ADATA(JDO45+8)  = 0.0
        END IF  
       END IF 

C   First check F+ F- stats ( ICHKPH1(NF1) > 0)
        IF (ICHKPH1(NF1).GT.0 .AND.
     +     .NOT.LOGMSS(JDO45+1) .AND. .NOT.LOGMSS(JDO45+2) .AND.
     +     .NOT.LOGMSS(JDO45+3) .AND. .NOT.LOGMSS(JDO45+4)) THEN
          IF (ADATA(JDO45+1).GT.1.0.AND.ADATA(JDO45+3).GT.1.0) THEN
            NCHKPH1(NF1) = NF1
            IF (ABS(ADATA(JDO45+1) - ADATA(JDO45+3))
     +         .GT.RANO*0.5*(ADATA(JDO45+1) + ADATA(JDO45+3))
     +         .OR.ADATA(JDO45+1).LT.SIGM*ADATA(JDO45+2)
     +         .OR.ADATA(JDO45+3).LT.SIGM*ADATA(JDO45+4)) THEN
               NEXCL1(NF1) = -NF1
               NESUM1(NF1) = NESUM1(NF1) + 1
            END IF
          END IF
        END IF
C   Now check <F> Dano stats ( ICHKPH2(NF1) > 0)
        IF (ICHKPH2(NF1).GT.0 .AND.
     +      .NOT.LOGMSS(JDO45+5) .AND..NOT.LOGMSS(JDO45+6) .AND.
     +      .NOT.LOGMSS(JDO45+7) .AND..NOT.LOGMSS(JDO45+8)) THEN
C ejd???           IF (ADATA(JDO45+5).GT.0.5*ADATA(JDO45+7)) THEN
           IF (ADATA(JDO45+5).GT.1.0) THEN
             NCHKPH2(NF1) = NF1
             IF (ABS(ADATA(JDO45+7)).GT.RANO*ADATA(JDO45+5)
     +         .OR.ADATA(JDO45+5).LT.SIGM*ADATA(JDO45+6)) THEN
             NEXCL2(NF1) = -NF1
             NESUM2(NF1) = NESUM2(NF1) + 1
             END IF
           END IF
        END IF
   45 CONTINUE
C
C     If the reflection is centric, apply FINDFM only.
C
      IF (IC.EQ.1) THEN
       NT = 0
       NF1 = 0
       DO 50 JDO50 = STARTFO, STARTFO+NDALL*8-1, 8
        NF1 = NF1 + 1
        IF (ICHKPH1(NF1).GT.0) THEN
C
C        Copy input FPHn+ SIGFPHn+ FPHn- SIGFPHn-.
C
         IF (.NOT.LOGMSS(JDO50+1)) BDATA(NBD+1) = ADATA(JDO50+1)
         IF (.NOT.LOGMSS(JDO50+2)) BDATA(NBD+2) = ADATA(JDO50+2)
         IF (.NOT.LOGMSS(JDO50+3)) BDATA(NBD+3) = ADATA(JDO50+3)
         IF (.NOT.LOGMSS(JDO50+4)) BDATA(NBD+4) = ADATA(JDO50+4)
         IF (NCHKPH1(NF1).GT.0) THEN
C
C         Get FPHMn+ SIGFPHMn+ FPHMn- SIGFPHMn-.
C
          BDATA(NBD+5) = 0.5*(ADATA(JDO50+1) + ADATA(JDO50+3))
          BDATA(NBD+6) = ADATA(JDO50+2)
          BDATA(NBD+7) = BDATA(NBD+5)
          BDATA(NBD+8) = ADATA(JDO50+4)
          IF (NEXCL1(NF1).GT.0) THEN
C
C         Prepare data for FINDFM to get FM1 and SIGFM1.
C
           NT = NT + 1
           FPRT(NT) = FPR(NF1)
           FDPT(NT) = FDP(NF1)
           FPT(NT) = BDATA(NBD+5)
           SFPT(NT) = BDATA(NBD+6)
           FNT(NT) = BDATA(NBD+7)
           SFNT(NT) = BDATA(NBD+8)
          END IF
         ELSE
C
C         Copy input FPHn+ SIGFPHn+ FPHn- SIGFPHn- to
C                    FPHMn+ SIGFPHMn+ FPHMn- SIGFPHMn-.
C
          IF (.NOT.LOGMSS(JDO50+1)) BDATA(NBD+5) = ADATA(JDO50+1)
          IF (.NOT.LOGMSS(JDO50+2)) BDATA(NBD+6) = ADATA(JDO50+2)
          IF (.NOT.LOGMSS(JDO50+3)) BDATA(NBD+7) = ADATA(JDO50+3)
          IF (.NOT.LOGMSS(JDO50+4)) BDATA(NBD+8) = ADATA(JDO50+4)
         END IF
         NBD = NBD + 8
        ELSE
         IF (ICHKPH2(NF1).GT.0) THEN
C
C        Copy input FPHn  SIGFPHn  DPHn  SIGDPHn.
C
          IF (.NOT.LOGMSS(JDO50+5)) BDATA(NBD+1) = ADATA(JDO50+5)
          IF (.NOT.LOGMSS(JDO50+6)) BDATA(NBD+2) = ADATA(JDO50+6)
          IF (.NOT.LOGMSS(JDO50+7)) BDATA(NBD+3) = ADATA(JDO50+7)
          IF (.NOT.LOGMSS(JDO50+8)) BDATA(NBD+4) = ADATA(JDO50+8)
          IF (NCHKPH2(NF1).GT.0) THEN
C
C         Get FPHMn  SIGFPHMn  DPHMn  SIGDPHMn.
C
           BDATA(NBD+5) = ADATA(JDO50+5)
           BDATA(NBD+6) = ADATA(JDO50+6)
           BDATA(NBD+7) = 0.0
           BDATA(NBD+8) = ADATA(JDO50+8)
           IF (NEXCL2(NF1).GT.0) THEN
C
C          Prepare data for FINDFM to get FM1 and SIGFM1.
C
            NT=NT + 1
            FPRT(NT) = FPR(NF1)
            FDPT(NT) = FDP(NF1)
            FPT(NT) = BDATA(NBD+5) + 0.5*BDATA(NBD+7)
            SFPT(NT) = SQRT(BDATA(NBD+6)*BDATA(NBD+6) +
     +                         0.25* BDATA(NBD+8)*BDATA(NBD+8))
            FNT(NT) = BDATA(NBD+5) - 0.5*BDATA(NBD+7)
            SFNT(NT) = SFPT(NT)
           END IF
          ELSE
C
C          Copy input FPHn  SIGFPHn  DPHn  SIGDPHn to
C                     FPHMn  SIGFPHMn  DPHMn  SIGDPHMn.
C
           IF (.NOT.LOGMSS(JDO50+5)) BDATA(NBD+5) = ADATA(JDO50+5)
           IF (.NOT.LOGMSS(JDO50+6)) BDATA(NBD+6) = ADATA(JDO50+6)
           IF (.NOT.LOGMSS(JDO50+7)) BDATA(NBD+7) = 0.0
           IF (.NOT.LOGMSS(JDO50+8)) BDATA(NBD+8) = ADATA(JDO50+8)
          END IF
          NBD = NBD + 8
         END IF
        END IF
        IF (ICHKPH1(NF1).GT.0.AND.ICHKPH2(NF1).GT.0) THEN
C
C        Copy input FPHn  SIGFPHn  DPHn  SIGDPHn and
C             to    FPHMn  SIGFPHMn  DPHMn  SIGDPHMn.
C
         IF (.NOT.LOGMSS(JDO50+5)) BDATA(NBD+1) = ADATA(JDO50+5)
         IF (.NOT.LOGMSS(JDO50+6)) BDATA(NBD+2) = ADATA(JDO50+6)
         IF (.NOT.LOGMSS(JDO50+7)) BDATA(NBD+3) = ADATA(JDO50+7)
         IF (.NOT.LOGMSS(JDO50+8)) BDATA(NBD+4) = ADATA(JDO50+8)
         IF (.NOT.LOGMSS(JDO50+5)) BDATA(NBD+5) = ADATA(JDO50+5)
         IF (.NOT.LOGMSS(JDO50+6)) BDATA(NBD+6) = ADATA(JDO50+6)
         IF (.NOT.LOGMSS(JDO50+7)) BDATA(NBD+7) = 0.0
         IF (.NOT.LOGMSS(JDO50+8)) BDATA(NBD+8) = ADATA(JDO50+8)
         NBD = NBD + 8
        END IF
   50  CONTINUE
C
C      Calculate FM1 SIGFM1 FM2 SIGFM2 and statistics.
C
       IF (NT.GE.NDMIN) THEN
        CALL FINDFM(FPT,SFPT,FNT,SFNT,FPRT,FDPT,NT,FM1,SFM1,RISO,NDMIN)
        IF (FM1.GT.0.0001) THEN
         FM2 = FM1
         SFM2 = SFM1
         NRES1(NSS) = NRES1(NSS) + 1
         NRES2(NSS) = NRES2(NSS) + 1
         NRSALL1 = NRSALL1 + 1
         NRSALL2 = NRSALL2 + 1
        ENDIF
       END IF
      END IF
C
C     If the reflection is non-centric, apply REVISE and FINDFM.
C
C     Apply REVISE and FINDFM
      IF (IC.EQ.0) THEN
       NT = 0
       NF1 = 0
       NG1 = 0
       DO 60 JDO60 = STARTFO, STARTFO+NDALL*8-1, 8
        NF1 = NF1 + 1
        BUF1(NF1) = 0.0
        IF (ICHKPH1(NF1).GT.0) THEN
         NG1 = NG1 + 1
         BUF1(NG1) = 0.0
         IF (NCHKPH1(NF1).GT.0.AND.NEXCL1(NF1).GT.0) THEN
C
C         Prepare data for REVISE and FINDFM.
C
          NT=NT + 1
          BUF1(NG1) = ADATA(JDO60+3)**2 - ADATA(JDO60+1)**2
          FPRT(NT) = FPR(NF1)
          FDPT(NT) = FDP(NF1)
          FPT(NT) = ADATA(JDO60+1)
          SFPT(NT) = ADATA(JDO60+2)
          FNT(NT) = ADATA(JDO60+3)
          SFNT(NT) = ADATA(JDO60+4)
          DT(NT) = FPT(NT)-FNT(NT)
          SDT(NT) = SQRT(SFPT(NT)*SFPT(NT) + SFNT(NT)*SFNT(NT))
         END IF
        ELSE
         IF (ICHKPH2(NF1).GT.0) THEN
          NG1 = NG1 + 1
          IF (NCHKPH2(NF1).GT.0.AND.NEXCL2(NF1).GT.0) THEN
C
C          Prepare data for REVISE and FINDFM.
C
           NT = NT + 1
           FPRT(NT) = FPR(NF1)
           FDPT(NT) = FDP(NF1)
           FPT(NT) = ADATA(JDO60+5) + 0.5*ADATA(JDO60+7)
           SFPT(NT) = SQRT(ADATA(JDO60+6)*ADATA(JDO60+6) +
     +                         0.25* ADATA(JDO60+8)*ADATA(JDO60+8))
           FNT(NT) = ADATA(JDO60+5) - 0.5*ADATA(JDO60+7)
           SFNT(NT) = SFPT(NT)
           BUF1(NG1) = FNT(NT)**2 - FPT(NT)**2
           DT(NT) = ADATA(JDO60+7)
           SDT(NT) = ADATA(JDO60+8)
          END IF
         END IF
        END IF
   60  CONTINUE
C    Now get correlation coefficients;
C       ichkk = mod(nrsall1,20000)
       DO 51 JDO51 = 1,ndall
        IF(SDT(JDO51) .GT.0.0001) then
         DO 52 JDO52 = jdo51,ndall
C          IF( ichkk .eq.1) 
C     +     write(6,'(A,5I3,3x,3I3,5(/,4F10.2))') 
C     +  ' NSS,ndall,JDO60,JDO51,JDO52', NSS,ndall,JDO60,JDO51,JDO52,
C     +  IN(1),IN(2),IN(3),(FPT(ii),FNT(ii),DT(ii),SDT(ii), ii=1,ndall)
          IF(ABS(SDT(JDO52)) .GT.0.0001) then
           CCDD(NSS,JDO51,JDO52) = CCDD(NSS,JDO51,JDO52) + 
     +                             DT(JDO51)*DT(JDO51)
           CCDE(NSS,JDO51,JDO52) = CCDE(NSS,JDO51,JDO52) + 1
           CCD(NSS,JDO51,JDO52) = CCD(NSS,JDO51,JDO52) + DT(JDO51)
C
           IF ( JDO52.NE.JDO51) THEN
            CCDD(NSS,JDO52,JDO51) = CCDD(NSS,JDO52,JDO51) +
     +                             DT(JDO52)*DT(JDO52)
            CCDE(NSS,JDO52,JDO51) = CCDE(NSS,JDO52,JDO51) + 
     +                             DT(JDO51)*DT(JDO52)
            CCD(NSS,JDO52,JDO51) = CCD(NSS,JDO52,JDO51) + DT(JDO52)
           END IF 
C
          END IF
 52      CONTINUE
        END IF
 51    CONTINUE
C
C     NDMIN data sets are needed.
C
       IF (NT.LT.NDMIN) NOS = 1
       IF (NT.LT.NDMIN) GOTO 125
C
C      Calculate FM1 SIGFM1 before applying REVISE.
C
       CALL FINDFM(FPT,SFPT,FNT,SFNT,FPRT,FDPT,NT,FM1,SFM1,RISO,NDMIN)
       IF (FM1.GT.0.0001) NRES1(NSS) = NRES1(NSS) + 1
       IF (FM1.GT.0.0001) NRSALL1 = NRSALL1 + 1
       DO 100 J = 1, NDALL
        DO 90 I = 1, NDALL
         IF (ABS(BUF1(I)).GT.0.0001.AND.ABS(BUF1(J)).GT.0.0001) THEN
           TEM = BUF1(I)/BUF1(J)
           NCC = INT(2.0 * TEM + 0.5 * SIGN(1.0, TEM))
           IF (NCC.LT.-NCMAX) NCC = -NCMAX
           IF (NCC.GT.NCMAX) NCC = NCMAX
           NUMAL1(NCC,NSS,I,J) = NUMAL1(NCC,NSS,I,J) + 1
           NRAT1(NSS,I,J) = NRAT1(NSS,I,J) + 1
           RAT1(NSS,I,J) = RAT1(NSS,I,J) + TEM
           NR1ALL(I,J) = NR1ALL(I,J) + 1
           R1ALL(I,J) = R1ALL(I,J) + TEM
         END IF
   90   CONTINUE
  100  CONTINUE
C
C      Start REVISE. NOS = 1 means no modofication has been done.
C
       NC = 0
  105  DO 110 J = 1, NT-1
        TM = FPT(J+1)**2-FDPT(J+1)/FDPT(1)*(FPT(1)**2-FNT(1)**2)
C       TM must be great than zero and shift the data set if not.
        IF (TM.LT.0.0) GOTO 115
  110  CONTINUE
       GOTO 120
  115  NC = NC + 1
C      No solution can be obtained if TM<0.0 after all the shift.
       IF (NC.GE.NT) NOS = 1
       IF (NC.GE.NT) GOTO 125
       CALL SHIFT(FPT,SFPT,FNT,SFNT,FPRT,FDPT,NT)
       GOTO 105
  120  NOS = 0
C      To find the new value of FPHn+ and FPHn-.
       CALL REVS(FPT,SFPT,FNT,SFNT,FDPT,NT,NOS)
  125  IF (NOS.NE.0) THEN
C      No solution can be obtained.
        NF1 = 0
        DO 130 JDO130 = STARTFO, STARTFO+NDALL*8-1, 8
         NF1 = NF1 + 1
         IF (ICHKPH1(NF1).GT.0) THEN
C
C         Copy input FPHn+  SIGFPHn+  FPHn-  SIGFPHn- only.
C
          IF (.NOT.LOGMSS(JDO130+1)) BDATA(NBD+1) = ADATA(JDO130+1)
          IF (.NOT.LOGMSS(JDO130+2)) BDATA(NBD+2) = ADATA(JDO130+2)
          IF (.NOT.LOGMSS(JDO130+3)) BDATA(NBD+3) = ADATA(JDO130+3)
          IF (.NOT.LOGMSS(JDO130+4)) BDATA(NBD+4) = ADATA(JDO130+4)
          NBD = NBD + 8
         END IF
         IF (ICHKPH2(NF1).GT.0) THEN
C
C         Copy FPHn   SIGFPHn   DPHn   SIGDPHn only.
C
          IF (.NOT.LOGMSS(JDO130+5)) BDATA(NBD+1) = ADATA(JDO130+5)
          IF (.NOT.LOGMSS(JDO130+6)) BDATA(NBD+2) = ADATA(JDO130+6)
          IF (.NOT.LOGMSS(JDO130+7)) BDATA(NBD+3) = ADATA(JDO130+7)
          IF (.NOT.LOGMSS(JDO130+8)) BDATA(NBD+4) = ADATA(JDO130+8)
          NBD = NBD + 8
         END IF
  130   CONTINUE
        GOTO 450
       END IF
C      New values are obtained and shift back to original data sets.
       NUMBM = NUMBM + 1
       IF (NC.EQ.0) GOTO 150
       NC = NT-NC
       DO 140 J = 1,NC
       CALL SHIFT(FPT,SFPT,FNT,SFNT,FPRT,FDPT,NT)
  140  CONTINUE
  150  NT = 0
       NF2 = 0
       NG2 = 0
       DO 300 JDO300 = STARTFO, STARTFO+NDALL*8-1, 8
        NF2 = NF2 + 1
        BUF2(NF2) = 0.0
        IF (ICHKPH1(NF2).GT.0) THEN
         NG2 = NG2 + 1
C
C        Copy input FPHn+  SIGFPHn+  FPHn-  SIGFPHn-.
C
         IF (.NOT.LOGMSS(JDO300+1)) BDATA(NBD+1) = ADATA(JDO300+1)
         IF (.NOT.LOGMSS(JDO300+2)) BDATA(NBD+2) = ADATA(JDO300+2)
         IF (.NOT.LOGMSS(JDO300+3)) BDATA(NBD+3) = ADATA(JDO300+3)
         IF (.NOT.LOGMSS(JDO300+4)) BDATA(NBD+4) = ADATA(JDO300+4)
         IF (NCHKPH1(NF2).GT.0.AND.NEXCL1(NF2).GT.0) THEN
C
C         Get modified FPHMn+ SIGFPHMn+ FPHMn- SIGFPHMn-.
C
          NT = NT + 1
          BDATA(NBD+5) = FPT(NT)
          BDATA(NBD+6) = ADATA(JDO300+2)
          BDATA(NBD+7) = FNT(NT)
          BDATA(NBD+8) = ADATA(JDO300+4)
C
C         Prepare data for FINDFM to get FM2 and SIGFM2.
C
          BUF2(NG2) = BDATA(NBD+7)**2 - BDATA(NBD+5)**2
          IF (ICHKPH2(NF2).GT.0) THEN
C
C          Get modified FPHMn  SIGFPHMn  DPHMn  SIGDPHMn.
C
           BDATA(NBD+13) = 0.5*(BDATA(NBD+5) + BDATA(NBD+7))
           BDATA(NBD+14) = ADATA(JDO300+6)
           BDATA(NBD+15) = BDATA(NBD+5) - BDATA(NBD+7)
           BDATA(NBD+16) = ADATA(JDO300+8)
          END IF
         END IF
         IF (ICHKPH2(NF2).GT.0) THEN
C
C         Copy input FPHn   SIGFPHn   DPHn   SIGDPHn.
C
          IF (.NOT.LOGMSS(JDO300+5)) BDATA(NBD+ 9) = ADATA(JDO300+5)
          IF (.NOT.LOGMSS(JDO300+6)) BDATA(NBD+10) = ADATA(JDO300+6)
          IF (.NOT.LOGMSS(JDO300+7)) BDATA(NBD+11) = ADATA(JDO300+7)
          IF (.NOT.LOGMSS(JDO300+8)) BDATA(NBD+12) = ADATA(JDO300+8)
          NBD = NBD + 8
         ENDIF
         NBD = NBD + 8
        ELSE
         IF (ICHKPH2(NF2).GT.0) THEN
          NG2 = NG2 + 1
C
C         Copy input FPHn   SIGFPHn   DPHn   SIGDPHn.
C
          IF (.NOT.LOGMSS(JDO300+5)) BDATA(NBD+1) = ADATA(JDO300+5)
          IF (.NOT.LOGMSS(JDO300+6)) BDATA(NBD+2) = ADATA(JDO300+6)
          IF (.NOT.LOGMSS(JDO300+7)) BDATA(NBD+3) = ADATA(JDO300+7)
          IF (.NOT.LOGMSS(JDO300+8)) BDATA(NBD+4) = ADATA(JDO300+8)
          IF (NCHKPH2(NF2).GT.0.AND.NEXCL2(NF2).GT.0) THEN
C
C          Get modified FPHMn  SIGFPHMn  DPHMn  SIGDPHMn.
C
           NT = NT + 1
           BDATA(NBD+5) = 0.5*(FPT(NT) + FNT(NT))
           BDATA(NBD+6) = ADATA(JDO300+6)
           BDATA(NBD+7) = FPT(NT) - FNT(NT)
           BDATA(NBD+8) = ADATA(JDO300+8)
C
C          Prepare data for FINDFM after applying REVISE.
C
           BUF2(NG2) = FNT(NT)**2 - FPT(NT)**2
          END IF
          NBD = NBD + 8
         END IF
        END IF
  300  CONTINUE
C
C      Calculate FM2 SIGFM2 after applying REVISE.
C
       CALL FINDFM(FPT,SFPT,FNT,SFNT,FPRT,FDPT,NT,FM2,SFM2,RISO,NDMIN)
       NSS = INT(SS4/SDEV) + 1
       IF (NSS.GT.NRMAX) NSS = NRMAX
       IF (FM2.GT.0.0001) NRES2(NSS) = NRES2(NSS) + 1
       IF (FM2.GT.0.0001) NRSALL2 = NRSALL2 + 1
       DO 400 I = 1, NDALL
       DO 400 J = 1, NDALL
        IF (ABS(BUF2(I)).GT.0.0001.AND.ABS(BUF2(J)).GT.0.0001) THEN
         TEM = BUF2(I)/BUF2(J)
         NRAT2(NSS,I,J) = NRAT2(NSS,I,J) + 1
         RAT2(NSS,I,J) = RAT2(NSS,I,J) + TEM
         NR2ALL(I,J) = NR2ALL(I,J) + 1
         R2ALL(I,J) = R2ALL(I,J) + TEM
        END IF
  400  CONTINUE
      END IF
C
C     Output FM SIGFM.
C
  450 IF (FM2.GT.0.0001) BDATA(NBD+1) = FM2
      IF (SFM2.GT.0.0001) BDATA(NBD+2) = SFM2
      NBD = NBD + 2
C
C     Write a reflection to output file and goto next reflection.
C
  500 CALL LWREFL(IFIL1,BDATA)
      GOTO 10
 1000 CONTINUE
C     Print correlations between Danos
C           CCDD(NSS,JDO51,JDO52) = CCDD(NSS,JDO51,JDO52) +
C     +                             DT(JDO51)*DT(JDO51)
C           CCDD(NSS,JDO52,JDO51) = CCDD(NSS,JDO52,JDO51) +
C     +                             DT(JDO52)*DT(JDO52)
C           CCDE(NSS,JDO51,JDO52) = CCDE(NSS,JDO51,JDO52) + 1
C           CCDE(NSS,JDO52,JDO51) = CCDE(NSS,JDO52,JDO51) +
C     +                             DT(JDO51)*DT(JDO52)
C           CCD(NSS,JDO51,JDO52) = CCD(NSS,JDO51,JDO52) + DT(JDO51)
C           CCD(NSS,JDO52,JDO51) = CCD(NSS,JDO52,JDO51) + DT(JDO52)
C  
c       write(6,*)  ' Before CCs - NRMAX NDALL',NRMAX,NDALL
      IF(NDALL.EQ.2) Write(lunout,'(7(/A))')
     + '  $TABLE:Correlations between anomalous differences:',
     + '  $GRAPHS: Correlations :N:1,3:',
     + '  : Number of reflections in shell :N:1,2:', 
     + ' $$ 1/resol^2 ',
     + ' Number_of_reflections_with_Dano1_and_Dano2_observed',
     + ' Correlation_Dano1_and_Dano2 $$',
     + ' 1/resol^2 Nref_1-2 CC_1-2  $$ '
C
      IF(NDALL.EQ.3) Write(lunout,'(14(/A))')
     + '  $TABLE:Correlations between anomalous differences:',
     + '  $GRAPHS: Correlations :N:1,3,5,7:',
     + '  : Number of reflections in shell :N:1,2,4,6:', 
     + ' $$ 1/resol^2 ',
     + ' Number_of_reflections_with_Dano1_and_Dano2_observed',
     + ' Correlation_Dano1_and_Dano2',
     + ' Number_of_reflections_with_Dano1_and_Dano3_observed',
     + ' Correlation_Dano1_and_Dano3',
     + ' Number_of_reflections_with_Dano2_and_Dano3_observed ',
     + ' Correlation_Dano2_and_Dano3 $$',
     + ' 1/resol^2 ',
     + ' Nref_1-2 CC_1-2 Nref_1-3 CC_1-3 Nref_2-3 CC_2-3 $$'
C
      IF(NDALL.EQ.4) Write(lunout,'(21(/A))')
     + '  $TABLE:Correlations between anomalous differences:',
     + '  $GRAPHS: Correlations :N:1,3,5,7,9,11,13:',
     + '  : Number of reflections in shell :N:1,2,4,6,8,10,12:', 
     + ' $$ 1/resol^2 ',
     + ' Number_of_reflections_with_Dano1_and_Dano2_observed',
     + ' Correlation_Dano1_and_Dano2',
     + ' Number_of_reflections_with_Dano1_and_Dano3_observed',
     + ' Correlation_Dano1_and_Dano3',
     + ' Number_of_reflections_with_Dano1_and_Dano4_observed',
     + ' Correlation_Dano1_and_Dano4',
     + ' Number_of_reflections_with_Dano2_and_Dano3_observed',
     + ' Correlation_Dano2_and_Dano3',
     + ' Number_of_reflections_with_Dano2_and_Dano4_observed',
     + ' Correlation_Dano2_and_Dano4',
     + ' Number_of_reflections_with_Dano3_and_Dano4_observed ',
     + ' Correlation_Dano3_and_Dano4 $$',
     + ' 1/resol^2 ',
     + ' N1-2 CC_1-2 N1-3 CC_1-3 N1_4 CC_1_4 ',
     + ' N2-3 CC_2-3 N2-4 CC_2-4 N3-4 CC_3-4 $$'
C
      IF(NDALL.EQ.5) Write(lunout,'(30(/A))')
     + '  $TABLE:Correlations between anomalous differences:',
     + '  $GRAPHS: Correlations for Data set 1 :N:1,3,5,7,9,11,13:',
     + '  : Number of reflections in shell :N:1,2,4,6,8,10,12:', 
     + ' $$ 1/resol^2 ',
     + ' Number_of_reflections_with_Dano1_and_Dano2_observed',
     + ' Correlation_Dano1_and_Dano2',
     + ' Number_of_reflections_with_Dano1_and_Dano3_observed',
     + ' Correlation_Dano1_and_Dano3',
     + ' Number_of_reflections_with_Dano1_and_Dano4_observed',
     + ' Correlation_Dano1_and_Dano4',
     + ' Number_of_reflections_with_Dano1_and_Dano5_observed',
     + ' Correlation_Dano1_and_Dano5 $$',
     + ' Number_of_reflections_with_Dano2_and_Dano3_observed',
     + ' Correlation_Dano2_and_Dano3',
     + ' Number_of_reflections_with_Dano2_and_Dano4_observed',
     + ' Correlation_Dano2_and_Dano4',
     + ' Number_of_reflections_with_Dano2_and_Dano5_observed',
     + ' Correlation_Dano2_and_Dano5',
     + ' Number_of_reflections_with_Dano3_and_Dano4_observed',
     + ' Correlation_Dano3_and_Dano4',
     + ' Number_of_reflections_with_Dano3_and_Dano5_observed',
     + ' Correlation_Dano3_and_Dano5',
     + ' Number_of_reflections_with_Dano4_and_Dano5_observed ',
     + ' Correlation_Dano4_and_Dano5 $$ ',
     + ' 1/resol^2 ',
     + ' N1-2 CC_1-2 N1-3 CC_1-3 N1-4 CC_1-4 N1-5 CC_1-5',
     + ' N2-3 CC_2-3 N2-4 CC_2-4 N2-5 CC_2-5 ',
     + ' N3-4 CC_3-4 N3-5 CC_3-5 ',
     + ' N4-5 CC_4-5 $$ '
      DO NSS = 1, NRMAX
c       WRITE(6,'(A,I4,I4,3(/,I3,3F15.0))') 
c     +   'CCD', NSS,NDALL,(I,(CCD(NSS,J,I),J=1,NDALL),I=1,NDALL)
c       WRITE(6,'(A,2I4,4(/,I3,3x,4(I3,F15.0)))') 'CCDD(J,I)', 
c     +  NSS,NDALL,(I,(J,CCDD(NSS,J,I),J=1,NDALL),I=1,NDALL)
c       WRITE(6,'(A,2I4,4(/,I3,4F15.0))') 
c     +    ' CCDE', NSS,NDALL,(I,(CCDE(NSS,J,I),J=1,NDALL),I=1,NDALL)
       CCTOT=0
       DO J = 1, NDALL-1
        DO I = J+1, NDALL
C Sum total
          CCTOT=CCTOT+CCDE(NSS,J,I)
C         write(6,*)' J I CCDE(NSS,J,I)',J,I,CCDE(NSS,J,I)
         IF(CCDE(NSS,J,I) .GT.0.0) THEN
C  ave(X**2)
           CCDD(NSS,J,I) =  CCDD(NSS,J,I)/CCDE(NSS,J,I)
C  Keep Rms Dano 
           CCDD(NSS,J,J) =  SQRT(ABS(CCDD(NSS,J,I)))
C  ave(y**2)
           CCDD(NSS,I,J) =  CCDD(NSS,I,J)/CCDE(NSS,J,I)
C  ave(X*Y)
           CCDE(NSS,I,J) =  CCDE(NSS,I,J)/CCDE(NSS,J,I)
C  ave(X)
           CCD(NSS,J,I) =  CCD(NSS,J,I)/CCDE(NSS,J,I)
C  ave(Y)
           CCD(NSS,I,J) =  CCD(NSS,I,J)/CCDE(NSS,J,I)
C   Corr Coeff
           CC = (CCDD(NSS,J,I) - CCD(NSS,J,I)*CCD(NSS,J,I))*
     +          (CCDD(NSS,I,J) - CCD(NSS,I,J)*CCD(NSS,I,J))
           CCDD(NSS,J,I) =  0.0
           IF ( ABS(CC).GT.0.00001) 
     +     CCDD(NSS,J,I) = (CCDE(NSS,I,J) - CCD(NSS,J,I)*CCD(NSS,I,J))
     +     /SQRT(ABS(CC))
c         write(6,*)' J I CCDD(NSS,J,I)',J,I,CCDD(NSS,J,I)
         END IF
        END DO 
       END DO 
C  Keep Rms Dano  for J=NDALL; missed above
         IF(CCDE(NSS,NDALL,NDALL) .GT.0.0) CCDD(NSS,NDALL,NDALL) =  
     +   SQRT(ABS(CCDD(NSS,NDALL,NDALL)/CCDE(NSS,NDALL,NDALL)))
       CCDE(NSS,1,1) = CCTOT
c              IF(NINT(CCDE(NSS,1,1)).GT.0) 
c     +     WRITE(6,'(F7.3,2(10(I8,F7.3)/))')SSQ4(NSS), 
c     +    NINT(CCDE(NSS,1,2)),CCDD(NSS,1,2),
c     +    NINT(CCDE(NSS,1,3)),CCDD(NSS,1,3),
c     +    NINT(CCDE(NSS,1,4)),CCDD(NSS,1,4),
c     +    NINT(CCDE(NSS,2,3)),CCDD(NSS,2,3),
c     +    NINT(CCDE(NSS,2,4)),CCDD(NSS,2,4),
c     +    NINT(CCDE(NSS,3,4)),CCDD(NSS,3,4)

              IF(NINT(CCDE(NSS,1,1)).GT.0) 
     +     WRITE(6,'(F7.3,2(10(I8,F7.3)/))')SSQ4(NSS), 
     +   ((NINT(CCDE(NSS,J,I)),CCDD(NSS,J,I),I=J+1,NDALL),J=1,NDALL)
      END DO 
       WRITE(LUNOUT,'(A)')' $$'
C  Write out rmsD for each wavelength
      IF(NDALL.EQ.2) Write(lunout,'(5(/A))')
     + '  $TABLE:Rms anomalous differences v resolution:',
     + '  $GRAPHS: Rms Dano v resolution :N:1,2,3:',
     + ' $$ 1/resol^2 ',
     + ' RmsDano1 RmsDano2 $$',
     + ' 1/resol^2 RmsD1 RmsD2 $$'
C
      IF(NDALL.EQ.3) Write(lunout,'(5(/A))')
     + '  $TABLE:Rms anomalous differences v resolution:',
     + '  $GRAPHS: Rms Dano v resolution :N:1,2,3,4:',
     + ' $$ 1/resol^2 ',
     + ' RmsDano1 RmsDano2 RmsDano3 $$',
     + ' 1/resol^2 RmsD1 RmsD2 RmsD3 $$'
C
      IF(NDALL.EQ.4) Write(lunout,'(5(/A))')
     + '  $TABLE:Rms anomalous differences v resolution:',
     + '  $GRAPHS: Rms Dano v resolution :N:1,2,3,4,5:',
     + ' $$ 1/resol^2 ',
     + ' RmsDano1 RmsDano2 RmsDano3 RmsDano4 $$',
     + ' 1/resol^2 RmsD1 RmsD2 RmsD3 RmsD4 $$'
C
      IF(NDALL.EQ.5) Write(lunout,'(5(/A))')
     + '  $TABLE:Rms anomalous differences v resolution:',
     + '  $GRAPHS: Rms Dano v resolution :N:1,2,3,4:',
     + ' $$ 1/resol^2 ',
     + ' RmsDano1 RmsDano2 RmsDano3 RmsDano4 RmsDano5 $$',
     + ' 1/resol^2 RmsD1 RmsD2 RmsD3 RmsD4 RmsD5 $$'
      DO NSS = 1, NRMAX
       IF(NINT(CCDE(NSS,1,1)).GT.0) 
     +     WRITE(6,'(F7.3,6F7.3)')SSQ4(NSS), 
     +     (CCDD(NSS,J,J),J=1,NDALL)
      END DO 
         WRITE(LUNOUT,FMT='(A,/)') '  $$'

C 1220    FORMAT(/,'  Ratio ',10I7,'  TOTAL $$')
C         DO 1250 NCC = -NCMAX, NCMAX
C          RTT = 0.5 * REAL(NCC)
C          DO 1225 NSS = 1, NRMAX
C           NTOTA1(NCC,I,J) = NTOTA1(NCC,I,J) + NUMAL1(NCC,NSS,I,J)
C 1225     CONTINUE
C           WRITE(LUNOUT,1230) RTT,(NUMAL1(NCC,NSS,I,J),NSS=1,NRMAX)
C     +                       ,NTOTA1(NCC,I,J)
C 1230      FORMAT(F8.2,11I7)
C 1250    CONTINUE
C         WRITE(LUNOUT,FMT='(A,/)') '  $$'
C
C     Print the number of cutoff for each wavelength.
C
      DO 1010 N = 1, NDALL
       IF (NESUM1(N).GT.0) THEN
        WRITE(LUNOUT,FMT='(/,A,I3,A,I6)')
     + ' The number of excluded reflections for data set',N,
     + ' = ',NESUM1(N)
       ELSE
        IF (NESUM2(N).GT.0) WRITE(LUNOUT,FMT='(/,A,I3,A,I6)')
     + ' The number of excluded reflections for data set',N,
     + ' = ',NESUM2(N)
       END IF
 1010 CONTINUE
C
C     Calculate and output the results of statistics.
C
      WRITE(LUNOUT,FMT='(/,A,A,/,A,//36X,A)')
     + ' The ratio of [(FPHn-)**2 - (FPHn+)**2]'
     + ,'/[(FPHm-)**2 - (FPHm+)**2]'
     + ,'     should equal to the ratio of [f"n/f"m].'
     + ,'Average ratio between data set n and m.'
      DO 1100 NSS = 1, NRMAX
       WRITE(LUNOUT,FMT='(/,A,I7,10I10)')
     +  '    N  SSQOL4   RESOL    No. of FM  DATA SET n',(N,N=1,NDALL)
       WRITE(LUNOUT,FMT='(I5,F8.4,F8.2,10X,A,/,A,I9)')
     +  NSS,SSQ4(NSS),RES(NSS),'        m'
     +   ,'       Before REVISE  ',NRES1(NSS)
       DO 1025 J = 1, NDALL
        DO 1020 I = 1, NDALL
         IF (NRAT1(NSS,I,J).GT.0) THEN
          RAT1(NSS,I,J) = RAT1(NSS,I,J) / REAL(NRAT1(NSS,I,J))
         END IF
 1020   CONTINUE
        WRITE(LUNOUT,FMT='(35X,I5,5X,10F10.4)') J,(RAT1(NSS,I,J)
     +        ,I=1,NDALL)
 1025  CONTINUE
       WRITE(LUNOUT,FMT='(A,I10)') '       After REVISE  ',NRES2(NSS)
       DO 1050 J = 1, NDALL
        DO 1030 I = 1, NDALL
         IF (NRAT2(NSS,I,J).GT.0) THEN
          RAT2(NSS,I,J) = RAT2(NSS,I,J) / REAL(NRAT2(NSS,I,J))
         END IF
 1030   CONTINUE
       WRITE(LUNOUT,FMT='(35X,I5,5X,10F10.4)') J,(RAT2(NSS,I,J)
     +       ,I=1,NDALL)
 1050  CONTINUE
 1100 CONTINUE
      WRITE(LUNOUT,1110)
 1110 FORMAT(/,/,50('*'))
      WRITE(LUNOUT,FMT='(/,A,I7,10I10)')
     +  '          TOTAL          No. of FM  DATA SET n',(N,N=1,NDALL)
      WRITE(LUNOUT,FMT='(A,I10,A)') '       Before REVISE ',NRSALL1
     + ,'        m'
      DO 1125 J = 1, NDALL
       DO 1120 I = 1, NDALL
        IF (NR1ALL(I,J).GT.0) THEN
         R1ALL(I,J) = R1ALL(I,J) / REAL(NR1ALL(I,J))
        END IF
 1120  CONTINUE
       WRITE(LUNOUT,FMT='(35X,I5,5X,10F10.4)') J,(R1ALL(I,J)
     +        ,I=1,NDALL)
 1125 CONTINUE
      WRITE(LUNOUT,FMT='(A,I10)') '       After REVISE  ',NRSALL2
      DO 1150 J = 1, NDALL
       DO 1130 I = 1, NDALL
        IF (NR2ALL(I,J).GT.0) THEN
         R2ALL(I,J) = R2ALL(I,J) / REAL(NR2ALL(I,J))
        END IF
 1130  CONTINUE
      WRITE(LUNOUT,FMT='(35X,I5,5X,10F10.4)') J,(R2ALL(I,J)
     +       ,I=1,NDALL)
 1150 CONTINUE
C
C     Output the ratio before REVISE for xloggraph.
C
      WRITE(LUNOUT,1160)
 1160 FORMAT(/,/,/,/,31('*')
     +,/,'*** Ratio between data sets ***',/,31('*'))
      DO 1500 I = 1, NDALL
       DO 1450 J = 1, NDALL
        IF (I.NE.J) THEN
         WRITE(LUNOUT,1200) I, J, I, J, I, J, I, J
 1200    FORMAT(//,2X,40('*'),/,
     +   '  $TABLE:Ratio of data set ',I2,' over ',I2,':',/,
     +   '  $GRAPHS:Ratio at Low Resolutions of data ',I2,' over ',
     +   I2,':N:1,2,3,4,5,6:',/,
     +   '  :Ratio at High Resolutions of data ',I2,' over ',I2,
     +   ':N:1,7,8,9,10,11:',/,
     +   '  :Ratio at Whole Resolutions of data ',I2,' over ',I2,
     +   ':N:1,12:',/,'  $$ Ratio ')
         DO 1210 NSS = 1, NRMAX
          IF(RES(NSS).GE.100.0) THEN
           WRITE(LUNOUT,'(A,F6.2)') '  Resolution_at_range_',RES(NSS)
          ELSE IF(RES(NSS).GE.10.0) THEN
           WRITE(LUNOUT,'(A,F5.2)') '  Resolution_at_range_',RES(NSS)
          ELSE 
           WRITE(LUNOUT,'(A,F4.2)') '  Resolution_at_range_',RES(NSS)
          END IF
 1210    CONTINUE
         WRITE(LUNOUT,FMT='(A)') '  Ratio_at_Whole_Resolution  $$'
         WRITE(LUNOUT,'(/,A,20I6)') '  Ratio ',(NSS,NSS=1,NRMAX)
         WRITE(LUNOUT,'(A)')'  TOTAL $$'
         DO 1250 NCC = -NCMAX, NCMAX
          RTT = 0.5 * REAL(NCC)
          DO 1225 NSS = 1, NRMAX
           NTOTA1(NCC,I,J) = NTOTA1(NCC,I,J) + NUMAL1(NCC,NSS,I,J)
 1225     CONTINUE
           WRITE(LUNOUT,1230) RTT,(NUMAL1(NCC,NSS,I,J),NSS=1,NRMAX)
     +                       ,NTOTA1(NCC,I,J)
 1230      FORMAT(F8.2,20I6)
 1250    CONTINUE
         WRITE(LUNOUT,FMT='(A,/)') '  $$'
        ENDIF
 1450  CONTINUE
 1500 CONTINUE
      WRITE(LUNOUT,FMT='(/,A,I10,/)')
     + 'The number of modified reflections = ',NUMBM
      WRITE(LUNOUT,'(/)')
      WRITE(LUNOUT,FMT='(//)')
      CALL LRCLOS(IFIL1)
      CALL LWCLOS(IFIL1,1)
      CALL CCPERR(0,'Normal termination')
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C                              REVISE                                  C
C                         TO FIND NEW F VALUE.                         C
C                            Version 1997                              C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE REVS(FPT,SFPT,FNT,SFNT,FDPT,NT,NOS)
      DIMENSION FPT(NT),SFPT(NT),XR(11),YR(11),FDPT(NT),
     +          FNT(NT),SFNT(NT),A(11,11),B(11,1)
C     Check FPHn+ or FPHn- not less than 1.0.
      DO 50 I = 1, NT
       IF (FPT(I).LT.1.0.OR.FNT(I).LT.1.0) GOTO 2000
   50 CONTINUE
      NCYC = 0
      DELT = 1.0
C     DELT is a small change of X to calculate dG/dX.
      MT = NT + 1
      DO 10 I = 1, 11
       XR(I) = 0.0
   10 CONTINUE
      DO 15 I = 1, 11
       B(I,1) = 0.0
       DO 15 J = 1, 11
        A(I,J) = 0.0
   15 CONTINUE
   20 NCYC = NCYC + 1
C     Set up the differential equations A*X=B.
      DO 100 I = 1, NT
      DO 100 J = I + 1, MT
       DO 30 K = 1, MT
        YR(K) = XR(K)
   30  CONTINUE
       YR(I) = XR(I) + DELT
       YR(J) = XR(J) + DELT
       CALL GG(RES,NOS,FPT,SFPT,FNT,SFNT,FDPT,YR,NT)
       IF (NOS.EQ.1) GOTO 2000
       Z1 = RES
       YR(I) = XR(I) - DELT
       YR(J) = XR(J) - DELT
       CALL GG(RES,NOS,FPT,SFPT,FNT,SFNT,FDPT,YR,NT)
       IF (NOS.EQ.1) GOTO 2000
       Z2 = RES
       YR(I) = XR(I) + DELT
       YR(J) = XR(J) - DELT
       CALL GG(RES,NOS,FPT,SFPT,FNT,SFNT,FDPT,YR,NT)
       IF (NOS.EQ.1) GOTO 2000
       Z3 = RES
       YR(I) = XR(I) - DELT
       YR(J) = XR(J) + DELT
       CALL GG(RES,NOS,FPT,SFPT,FNT,SFNT,FDPT,YR,NT)
       IF (NOS.EQ.1) GOTO 2000
       Z4 = RES
       A(I,J) = 10.0*(Z1 + Z2 - Z3 - Z4)/(4.0*DELT*DELT)
       A(J,I) = A(I,J)
  100 CONTINUE
      DO 200 I = 1, MT
       DO 150 J = 1, MT
        YR(J) = XR(J)
  150  CONTINUE
      CALL GG(RES,NOS,FPT,SFPT,FNT,SFNT,FDPT,YR,NT)
      IF (NOS.EQ.1) GOTO 2000
      Z1 = RES
      YR(I) = XR(I) + DELT
      CALL GG(RES,NOS,FPT,SFPT,FNT,SFNT,FDPT,YR,NT)
      IF (NOS.EQ.1) GOTO 2000
      Z2 = RES
      YR(I) = XR(I) - DELT
      CALL GG(RES,NOS,FPT,SFPT,FNT,SFNT,FDPT,YR,NT)
      IF (NOS.EQ.1) GOTO 2000
      Z3 = RES
      A(I,I) = 10.0*(Z2 + Z3 - 2.0*Z1)/(DELT*DELT)
  200 CONTINUE
      DO 300 I = 1, MT
       DO 280 J = 1, MT
        YR(J) = XR(J)
  280  CONTINUE
      YR(I) = XR(I) + DELT
      CALL GG(RES,NOS,FPT,SFPT,FNT,SFNT,FDPT,YR,NT)
      IF (NOS.EQ.1) GOTO 2000
      Z1 = RES
      YR(I) = XR(I) - DELT
      CALL GG(RES,NOS,FPT,SFPT,FNT,SFNT,FDPT,YR,NT)
      IF (NOS.EQ.1) GOTO 2000
      Z2 = RES
      B(I,1) = 10.0*(Z2 - Z1)/(2.0*DELT)
  300 CONTINUE
C     Solve the equations by Gauss-Jordan Elimination.
      CALL GAUSSJ(A,MT,11,B,1,1,NOS)
      IF (NOS.EQ.1) GOTO 2000
C     Calculate total modification and stop if it less than 10**-6.
      SUM = 0.0
      DO 400 II = 1, MT
       SUM = SUM + B(II,1)**2
  400 CONTINUE
      IF (SUM.LT.1.0E-6) GOTO 1000
      DO 500 II = 1, MT
       XR(II) = XR(II) + B(II,1)
  500 CONTINUE
      IF (NCYC.GE.100) GOTO 1000
C     Next cycle.
      GOTO 20
C     New values for FPHn+ and FPHn-.
 1000 DO 1100 I = 1, NT
       FPT(I) = FPT(I) + XR(I)
       IF (FPT(I).LT.0.0) GOTO 2000
 1100 CONTINUE
      FNT(1) = FNT(1) + XR(MT)
      IF (FNT(1).LT.0.0) GOTO 2000
      DO 1200 I = 2, NT
        FNT(I) = FPT(I)**2 - FDPT(I)*(FPT(1)**2 - FNT(1)**2)/FDPT(1)
        IF (FNT(I).LT.0.0) GOTO 2000
        FNT(I) = SQRT(FNT(I))
 1200 CONTINUE
      RETURN
 2000 NOS = 1
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C                 SHIFT FPT,SFPT,FNT,SFNT,FPRT,FDPT                    C
C                            Version 1997                              C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C     Shift data set n to n-1 and 1 to n.
      SUBROUTINE SHIFT(FPT,SFPT,FNT,SFNT,FPRT,FDPT,NT)
      DIMENSION FPT(NT),SFPT(NT),FNT(NT),SFNT(NT),FPRT(NT),FDPT(NT)
      T1 = FPT(1)
      T2 = FNT(1)
      T3 = SFPT(1)
      T4 = SFNT(1)
      T5 = FPRT(1)
      T6 = FDPT(1)
      DO 116 J = 1, NT - 1
       FPT(J) = FPT(J+1)
       FNT(J) = FNT(J+1)
       SFPT(J) = SFPT(J+1)
       SFNT(J) = SFNT(J+1)
       FPRT(J) = FPRT(J+1)
       FDPT(J) = FDPT(J+1)
  116 CONTINUE
      FPT(NT) = T1
      FNT(NT) = T2
      SFPT(NT) = T3
      SFNT(NT) = T4
      FPRT(NT) = T5
      FDPT(NT) = T6
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C                Calculate Sum of (Xn/SIGFPHn)**2                      C 
C                            Version 1997                              C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C     See equation 14 in the reference.
C
      SUBROUTINE GG(R,NOS,FPT,SFPT,FNT,SFNT,FDPT,YR,NT)
      DIMENSION FPT(NT),SFPT(NT),FNT(NT),SFNT(NT),YR(NT+1),FDPT(NT)
      NOS = 0
      SUM = 0.0
      DO 10 I = 1, NT - 1
       TM = (FPT(I+1) + YR(I+1))**2 - FDPT(I+1)/FDPT(1)*((FPT(1) + 
     +      YR(1))**2 - (FNT(1) + YR(NT+1))**2)
       IF (TM.LT.0.0) GOTO 1000
       XM = SQRT(TM) - FNT(I+1)
       SUM = SUM + (XM/SFNT(I+1))**2
   10 CONTINUE
      DO 20 I = 1, NT
       SUM = SUM + (YR(I)/SFPT(I))**2 
   20 CONTINUE
      SUM = SUM + (YR(NT+1)/SFNT(1))**2
      R = SUM
      RETURN
 1000 NOS = 1
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C        Linear equation solution by Gauss-Jordan Elimination.         C
C                            Version 1997                              C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE GAUSSJ(A,N,NP,B,M,MP,NOS)
      PARAMETER (NMAX=50)
      DIMENSION A(NP,NP),B(NP,MP),IPIV(NMAX),INDXR(NMAX),INDXC(NMAX)
      DO 11 J = 1, N
       IPIV(J) = 0
   11 CONTINUE
      NOS = 0
      DO 22 I = 1, N
        BIG = 0.
        DO 13 J = 1, N
          IF (IPIV(J).NE.1) THEN
            DO 12 K = 1, N
              IF (IPIV(K).EQ.0) THEN
                IF (ABS(A(J,K)).GE.BIG)THEN
                  BIG = ABS(A(J,K))
                  IROW = J
                  ICOL = K
                ENDIF
              ELSE IF (IPIV(K).GT.1) THEN
                GOTO 1000
              ENDIF
   12       CONTINUE
          ENDIF
   13   CONTINUE
        IPIV(ICOL) = IPIV(ICOL) + 1
        IF (IROW.NE.ICOL) THEN
          DO 14 L = 1, N
            DUM = A(IROW,L)
            A(IROW,L) = A(ICOL,L)
            A(ICOL,L) = DUM
   14     CONTINUE
          DO 15 L = 1, M
            DUM = B(IROW,L)
            B(IROW,L) = B(ICOL,L)
            B(ICOL,L) = DUM
   15     CONTINUE
        ENDIF
        INDXR(I) = IROW
        INDXC(I) = ICOL
        IF (A(ICOL,ICOL).EQ.0.) GOTO 1000
        PIVINV = 1./A(ICOL,ICOL)
        A(ICOL,ICOL) = 1.
        DO 16 L = 1, N
          A(ICOL,L) = A(ICOL,L)*PIVINV
   16   CONTINUE
        DO 17 L = 1, M
          B(ICOL,L) = B(ICOL,L)*PIVINV
   17   CONTINUE
        DO 21 LL = 1, N
          IF (LL.NE.ICOL) THEN
            DUM = A(LL,ICOL)
            A(LL,ICOL) = 0.
            DO 18 L = 1, N
              A(LL,L) = A(LL,L) - A(ICOL,L)*DUM
   18       CONTINUE
            DO 19 L = 1, M
              B(LL,L) = B(LL,L) - B(ICOL,L)*DUM
   19       CONTINUE
          ENDIF
   21   CONTINUE
   22 CONTINUE
      DO 24 L = N, 1, -1
        IF (INDXR(L).NE.INDXC(L))THEN
          DO 23 K = 1, N
            DUM = A(K,INDXR(L))
            A(K,INDXR(L)) = A(K,INDXC(L))
            A(K,INDXC(L)) = DUM
   23     CONTINUE
        ENDIF
   24 CONTINUE
      RETURN
 1000 NOS = 1
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C                       Find the values of FM.                         C
C                            Version 1997                              C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE FINDFM(FPT,SFPT,FNT,SFNT,FPRT,FDPT,NT,FM,SFM,RISO
     +                  ,NDMIN)
      DIMENSION FPT(NT),SFPT(NT),FNT(NT),SFNT(NT),FPRT(NT),FDPT(NT),
     +          FPS(10),FNS(10),SDEL2(10),S2DEL2(10),C2DEL(10),
     +          FAC(10),FMM(10),FPTS(10),SFPTS(10),FNTS(10),SFNTS(10),
     +          FPRTS(10),FDPTS(10)
C     ----------------------------------------------------------
C     Keep original FPT, SFPT, FNT,SFNT,FPRT, FDPT and NT.
      DO 20 I = 1, NT
       FPTS(I) = FPT(I)
       SFPTS(I) = SFPT(I)
       FNTS(I) = FNT(I)
       SFNTS(I) = SFNT(I)
       FPRTS(I) = FPRT(I)
       FDPTS(I) = FDPT(I)
   20 CONTINUE
      NTT = NT
      FCAL = 0.0
      FM = 0.0
      SFM = 0.0
C     Check <DISO> not great than RISO * <FPH>.
      CALL CHECK(FPTS,SFPTS,FNTS,SFNTS,FPRTS,FDPTS,NTT,NTS,RISO)
C     Return if data are not enough.
      IF (NTS.LT.NDMIN) RETURN
C     Prepare data for CALFG subroutine.
      DO 200 N = 1, NTS
       DELT = ATAN2(FDPTS(N),FPRTS(N))
       SDEL2(N) = SIN(DELT)**2
       S2DEL2(N) = SIN(2.0*DELT)**2
       C2DEL(N) = COS(2.0*DELT)
       FAC(N) = FPRTS(N)*FPRTS(N) + FDPTS(N)*FDPTS(N)
       FPS(N) = FPTS(N)*FPTS(N)
       FNS(N) = FNTS(N)*FNTS(N)
  200 CONTINUE
C     Find the best velues of FM**2 for all the wavelengths.
      CALL CALFG(FCAL,FMM,NTS,SDEL2,S2DEL2,C2DEL,FAC,FPS,FNS,
     +           FPTS,FNTS)
C     FMM contains FM**2 and calculate average value of FM**2.
      NFM = 0
      FMMEAN = 0.0
      FM2 = 0.0
      DO 210 I = 1, NTS
       IF (FMM(I).LT.0.0) GOTO 210
       FMMEAN = FMMEAN + FMM(I)
       FM2 = FM2 + FMM(I)*FMM(I)
       NFM = NFM + 1
  210  CONTINUE
      IF (NFM.GT.0) THEN
C      Calculate FM and SIGFM.
       FMMEAN = FMMEAN/REAL(NFM)
       FM2 = FM2/REAL(NFM)
       TSF= FM2 - FMMEAN*FMMEAN
       FM = SQRT(FMMEAN)
       IF (TSF.GT.0.0001) SFM = SQRT(TSF)/(2.0*FM)
       IF (SFM.LT.0.01) SFM = 0.01
      ENDIF
      RETURN
      END
C     -----------------------------------------------------------
C
C     Search for the best FM**2 for all the wavelengths.
C
      SUBROUTINE CALFG(FCAL,FMM,NT,SDEL2,S2DEL2,C2DEL,FAC,FPS,FNS,
     +           FPT,FNT)
      DIMENSION FPT(NT),FNT(NT),FPS(NT),FNS(NT)
     1,SDEL2(NT),S2DEL2(NT),C2DEL(NT),FAC(NT),FMM(NT)
      DIMENSION G2(0:200,10,2)
      FAV = 0.0
      DO 10 I = 1, NT
       FMM(I) = -1.0
       FAV = FAV + FPT(I) + FNT(I)
   10 CONTINUE
      FAV = FAV/REAL(2*NT)
      FMIN = FAV/1.5
      FMAX = FAV*1.5
      DF = (FMAX - FMIN)/200.0
C     Search F in the range from FMIN to FMAX.
      DO 100 I = 0, 200
       F = FMIN + REAL(I)*DF
       DO 50 J = 1, NT
        A = 4.0*SDEL2(J)
        B = -4.0*((FPS(J) + FNS(J) - 2.0*F*F)*SDEL2(J) + F*F*S2DEL2(J))
        C = FPS(J)**2 + FNS(J)**2 - 2.0*FPS(J)*FNS(J)*C2DEL(J)
     +       - 4.0*F*F*(FPS(J) + FNS(J) - F*F)*SDEL2(J)
        G2(I,J,1) = -1.0
        G2(I,J,2) = -1.0
        T = B*B - 4.0*A*C
        IF (T.LT.0.0) GOTO 50
        H = SQRT(T)
        G = -B + H
        IF (G.GT.0.0) G2(I,J,1) = G/FAC(J)/2.0/A
        G = -B - H
        IF (G.GT.0.0) G2(I,J,2) = G/FAC(J)/2.0/A
   50  CONTINUE
  100 CONTINUE
C     Two solutions for FM**2 and only use small one.
      SUMMIN = 1000.0
      DO 200 I = 0, 200
       F = FMIN + REAL(I)*DF
       SUM2 = 0.0
       NSUM = 0
C      Find out minimum difference between all wavelengths.
       DO 150 J = 1, NT - 1
       DO 150 K = J + 1, NT
        IF (G2(I,J,2).LT.0.0.OR.G2(I,K,2).LT.0.0) GOTO 200
        SUM2 = SUM2 + (G2(I,J,2) - G2(I,K,2))**2
        NSUM = NSUM + 1
  150  CONTINUE
       IF (NSUM.GT.0) THEN 
        SUM2 = SQRT(SUM2)
       ELSE 
        SUM2 = 1001.0
       ENDIF
       IF (SUM2.GE.SUMMIN) GOTO 200
C      Store FM**2.
       FCAL = F
       SUMMIN = SUM2
       DO 160 J = 1, NT
  160   FMM(J) = G2(I,J,2)
  200 CONTINUE
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                   C
C                   Check DF-iso and reject large one.              C
C                            Version 1997                           C
C                                                                   C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE CHECK(FPT,SFPT,FNT,SFNT,FPRT,FDPT,NT,NTS,RISO)
      DIMENSION FPT(NT),SFPT(NT),FNT(NT),SFNT(NT),FPRT(NT),FDPT(NT)
     1,FPS(10),SFPS(10),FNS(10),SFNS(10),FPRTS(10),FDPTS(10),FMEAN(10)
      NTT = NT
    5 FMALL = 0.0
C     Store original data.
      DO 10 I = 1, NTT
       FPS(I) = FPT(I)
       SFPS(I) = SFPT(I)
       FNS(I) = FNT(I)
       SFNS(I) = SFNT(I)
       FPRTS(I) = FPRT(I)
       FDPTS(I) = FDPT(I)
       FMEAN(I) = 0.5*(FPT(I) + FNT(I))
       FMALL = FMALL + FMEAN(I)
   10 CONTINUE
C     Calculate <FPH>.
      FMALL = FMALL/REAL(NTT)
   15 NTS = 0
      DMSAV = 0.0
      NSAV = 0
      DO 50 I = 1, NTT
       NFMEAN = 0
       DFMEAN = 0.0
       DO 40 J = 1, NTT
        IF (I.NE.J) THEN
         DFMEAN = DFMEAN + ABS(FMEAN(I) - FMEAN(J))
         NFMEAN = NFMEAN + 1
        ENDIF
   40  CONTINUE
       DFMEAN = DFMEAN/REAL(NFMEAN)
       IF (DFMEAN.GT.DMSAV.AND.DFMEAN.GT.RISO*FMALL) THEN
        DMSAV = DFMEAN
        NSAV = I
       ENDIF
   50 CONTINUE
      IF (NSAV.EQ.0) THEN
       NTS = NTT
       RETURN
      ENDIF
      DO 60 I = 1, NTT
       IF (I.NE.NSAV) THEN
        NTS = NTS + 1
        FPT(NTS) = FPS(I)
        SFPT(NTS) = SFPS(I)
        FNT(NTS) = FNS(I)
        SFNT(NTS) = SFNS(I)
        FPRT(NTS) = FPRTS(I)
        FDPT(NTS) = FDPTS(I)
       ENDIF
   60 CONTINUE
      IF (NTS.LE.1) GOTO 100
      NTT = NTS
      GOTO 5
  100 RETURN
      END
