C
C     RFCORR
C     Copyright (C) 1994 Ian Tickle
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C*************************************************************
C
C
C**************************************************************
C
      PROGRAM RFCORR
C==== Analyse self- and cross-rotation function results for correlations.
C==== This requires a self-rotation function map calculated by ALMN or
C==== AMORE, i.e. the angular variables must be Eulerian, not polar;
C==== and a list of peaks from a cross-rotation function map.
C
      IMPLICIT NONE
      CHARACTER TI*80,TA
      INTEGER I,MODE
      INTEGER LA(1),LUVW(3)
      REAL FL,FR,FS
C
      INTEGER KE
      INTEGER IUVW(3),IXYZ(3),MUVW(3),NUVW(3),NXYZ(3)
      REAL FM
      REAL CC(6)
      COMMON /MAPCOM/ KE,IUVW,IXYZ,MUVW,NUVW,NXYZ,CC,FM
C
      EXTERNAL RFMAIN
      DATA TA/'R'/
C
      KE=0
      CALL CCPRCS (6, 'RFCORR', '$Date: 2005/09/06 11:25:23 $')
      CALL CCPFYP
C
C==== Read and check self-rotation function map header.
C
      CALL MRDHDR(1,'MAPIN',TI,NUVW(3),IUVW,NXYZ,LUVW(3),
     &LUVW(1),MUVW(1),LUVW(2),MUVW(2),CC,I,MODE,FL,FM,FS,FR)
      IF (LUVW(1).NE.0 .OR. LUVW(2).NE.0 .OR. LUVW(3).NE.0) THEN
        WRITE (6,*) '*** ERROR: First point in map must be (0,0,0).'
        KE=KE+1
      ENDIF
C
C==== Inverted axis order and map grid size.
C
      DO 10 I=1,3
        IXYZ(IUVW(I))=I
 10     CC(I)=CC(I)/NXYZ(I)
C
      NUVW(1)=MUVW(1)+1
      NUVW(2)=MUVW(2)+1
      MUVW(3)=NUVW(3)-1
C
      IF (NXYZ(1).NE.NXYZ(3) .OR. NUVW(IXYZ(1)).NE.NUVW(IXYZ(3)) .OR.
     &MOD(NXYZ(1),NUVW(IXYZ(1))).NE.0) THEN
        WRITE (6,*) '*** ERROR: Must be submultiple in Alpha & Gamma.'
        KE=KE+1
      ENDIF
C
      IF (4*MUVW(IXYZ(2)).NE.NXYZ(2) .AND. 2*MUVW(IXYZ(2)).NE.NXYZ(2))
     &THEN
        WRITE (6,*) '*** ERROR: Must be 90 or 180 in Beta.'
        KE=KE+1
      ENDIF
C
C==== Allocate memory for whole map.
C
      LA(1)=NUVW(1)*NUVW(2)*NUVW(3)
      CALL CCPALC(RFMAIN,1,TA,LA)
      CLOSE(UNIT=1,STATUS='KEEP')
      CALL CCPERR(0,'*** NORMAL TERMINATION ***')
      END
C
C
      SUBROUTINE RFMAIN(L,RHO)
      IMPLICIT NONE
      INTEGER MA,MP,MR,MSORT,MSYM,MTOK,NKEY
      PARAMETER (MA=50,MP=50,MR=50,MSORT=12*MR*(MR-1),MSYM=192)
      PARAMETER (MTOK=100,NKEY=9)
      REAL PI,DR
      PARAMETER (PI=3.141592654,DR=PI/180.)
C
      CHARACTER KEY*4,LINE*600,PGN*10,SGN*10
      CHARACTER CV(MTOK)*4,KEYA(NKEY)*4,CS(MSYM)*20
      LOGICAL ALMN,EOF
      LOGICAL LT(3)
      INTEGER I,IP,IR,IS,J,JP,JR,K,KP,KW,L,N,NA,NC,NP,NR,NS,NSG,NTOK
      INTEGER IA(3),IB(MTOK),ID(MTOK),IE(MTOK),IN(3,MSORT),
     &ISORT(MSORT),IT(MTOK),IV(4,4),JA(3),NTOKS(NKEY)
      REAL AR,CHI,DET,PL,TOL,V
      REAL A(3),B(3,3,6),BI(3,3),CA(3),CELL(6),FV(MTOK),R(3,3),
     &RF(3,3,MR),RM(4,4,MSYM),RS(3,3,MSYM),RT(3,3),SA(3),SORT(MSORT),
     &X(3),XS(3,MA)
C
      INTEGER KE,MU,MV,MW
      INTEGER IUVW(3),IXYZ(3),MUVW(3),NUVW(3),NXYZ(3)
      REAL FM
      REAL CC(6)
      COMMON /MAPCOM/ KE,IUVW,IXYZ,MU,MV,MW,NUVW,NXYZ,CC,FM
      EQUIVALENCE (MU,MUVW(1))
C
      REAL RHO(0:MU,0:MV,0:MW)
C
      DATA SGN /' '/
      DATA KEYA /'TITL','ALMN','SPAC','ORTH','PEAK','NUMP','CHI','ANGL',
     &'END'/
      DATA ALMN,LT /4*.FALSE./
      DATA NA,NC,NP,NSG /MA, 1, MP, 0/
      DATA NTOKS /1, -1, -2, -2, -2, -2, 2, 2, -1/
      DATA CELL /3*1., 3*90./
      DATA PL,CHI,TOL /.1, 0., 10./
C
C==== Read the whole self-rotation function map into memory.
C
      DO 10 KW=0,MW
        CALL MGULPR(1,RHO(0,0,KW),I)
 10     IF (I.NE.0) CALL CCPERR(1,'*** ERROR: in MGULPR.')
C
C====   Read control data.
C
 20   LINE=' '
      NTOK=MTOK
      CALL PARSER(KEY,LINE,IB,IE,IT,FV,CV,ID,NTOK,EOF,.TRUE.)
      IF (EOF) GOTO 9
      IF (NTOK.EQ.0) GOTO 20
      DO 30 I=1,NKEY
 30     IF (KEY.EQ.KEYA(I)) GOTO 40
      WRITE (6,*) '*** ERROR: Last line has invalid keyword.'
      KE=KE+1
 40   IF ((NTOKS(I).LT.0 .AND. NTOK.NE.-NTOKS(I)) .OR. NTOK.LT.NTOKS(I))
     &THEN
        WRITE (6,*) '*** ERROR: bad argument count.'
        KE=KE+1
      ENDIF
      GOTO (1,2,3,4,5,6,7,8,9),I
C
C==== Title.
C
 1    GOTO 20
C
C==== ALMN flag.
C
 2    ALMN=.TRUE.
      GOTO 20
C
C==== Space group.
C
 3    IF (IT(2).EQ.1) THEN
        SGN=LINE(IB(2):IE(2))
        CALL CCPUPC(SGN)
        NSG=0
      ELSE
        CALL GTPINT(2,NSG,NTOK,IT,FV)
        SGN=' '
      ENDIF
      GOTO 20
C
C==== Orthogonalisation code.
C
 4    CALL GTPINT(2,NC,NTOK,IT,FV)
      IF (NC.LE.0 .OR. NC.GT.6) THEN
        WRITE (6,*) '*** ERROR: Bad orthogonalisation code.'
        KE=KE+1
      ENDIF
      GOTO 20
C
C==== Peak limit - fraction of maximum self-RF value to store.
C
 5    CALL GTPREA(2,PL,NTOK,IT,FV)
      GOTO 20
C
C==== Maximum number of self-RF peaks to print.
C
 6    CALL GTPINT(2,NP,NTOK,IT,FV)
      GOTO 20
C
C==== Chi value and tolerance.
C
 7    CALL GTPREA(2,CHI,NTOK,IT,FV)
      IF (NTOK.GT.2) CALL GTPREA(3,TOL,NTOK,IT,FV)
      IF ((CHI.NE.0. .AND. CHI.LE.TOL) .OR. CHI.GT.180.) THEN
        WRITE (6,*) '*** ERROR: Bad value for Chi.'
        KE=KE+1
      ENDIF
      GOTO 20
C
C==== Maximum number of self-RF peaks to compute inter-vector angles.
C
 8    CALL GTPINT(2,NA,NTOK,IT,FV)
      NA=MIN(MA,NA)
      GOTO 20
C
 9    CALL MSYMLB(1,NSG,SGN,PGN,NS,I,RM)
      WRITE (6,'(/2A,2I5)') ' Space group:  ',SGN,NSG,NS
      DO 220 IS=1,NS
        DO 220 I=1,3
 220      RM(I,4,IS)=0.
      CALL SYMTR3(NS,RM,CS,0)
      WRITE(6,'(/A/)') ' Rotation function symmetry operators:'
      WRITE(6,'(1X,I2,4X,A)') (IS,CS(IS),IS=1,NS)
      WRITE(6,'(A)')
C
C==== Only the cell angles in trigonal & hexagonal space groups are needed
C==== to orthogonalise the symmetry matrices.
C
      IF (NSG.GT.142 .AND. NSG.LE.194) CELL(6)=120.
C
C==== Form B and B^ (B inverse).
C
      CALL RBFRO1(CELL,V,B)
      CALL MINV3(BI,B(1,1,NC),DET)
C
C==== Store 3x3 rotation symmetry matrices and orthogonalise.
C
      DO 70 IS=1,NS
        DO 50 J=1,3
          DO 50 I=1,3
 50         RS(I,J,IS)=RM(I,J,IS)
C
C==== Form B.S.B^
C
        CALL MXM3(RS(1,1,IS),BI,RT)
        CALL MXM3(B(1,1,NC),RT,RS(1,1,IS))
C
C==== Check symmetry for 2-folds.
C
        DO 60 I=1,3
          J=MOD(I,3)+1
          K=MOD(J,3)+1
          IF (ABS(RS(I,I,IS)-1.).LT..0001 .AND.
     &    ABS(RS(J,J,IS)+1.).LT..0001 .AND.
     &    ABS(RS(K,K,IS)+1.).LT..0001) THEN
            LT(I)=.TRUE.
            GOTO 70
          ENDIF
 60     CONTINUE
 70   CONTINUE
C
      IF (.NOT.(LT(1).OR.LT(2)) .AND. 2*MUVW(IXYZ(2)).NE.NXYZ(2)) THEN
        WRITE (6,*) '*** ERROR: Must be 180 in Beta.'
        KE=KE+1
      ENDIF
      IF (KE.GT.0) CALL CCPERR(1,'*** FATAL ERROR MESSAGE(S) ABOVE ***')
C
C==== Read in cross-RF peak list.
C
      I=0
      CALL CCPDPN(1,'PEAKS','READONLY','F',0,I)
      WRITE (6,'(//A/)') '  Peak     Alpha      Beta     Gamma'
      NR=0
 80   READ (1,'(A)',END=120) LINE
      NTOK=MTOK
      CALL PARSER(KEY,LINE,IB,IE,IT,FV,CV,ID,NTOK,EOF,.FALSE.)
      IF (ALMN) THEN
        IF (NTOK.LT.4 .OR. KEY.NE.'PEAK') GOTO 80
        DO 90 I=1,3
 90      A(I)=FV(1+I)
      ELSE
        IF (NTOK.LT.5 .OR. KEY.NE.'SOLU') GOTO 80
        DO 100 I=1,3
 100      A(I)=FV(2+I)
      ENDIF
C
C==== Reject symmetry related points on the beta=0 or 90 sections.
C
      IF ((LT(3) .AND. A(2).EQ.0. .OR. (LT(1).OR.LT(2)) .AND.
     &A(2).EQ.90.) .AND. A(3).GE.180.) GOTO 80
C
      NR=NR+1
      WRITE (6,'(1X,I5,3F10.2)') NR,A
      DO 110 I=1,3
        AR=DR*A(I)
        CA(I)=COS(AR)
        SA(I)=SIN(AR)
        IF (MOD(A(I),90.).EQ.0.) THEN
          CA(I)=ANINT(CA(I))
          SA(I)=ANINT(SA(I))
        ENDIF
 110  CONTINUE
C
C==== Store Eulerian rotation matrix.
C
      RF(1,1,NR)=CA(1)*CA(2)*CA(3)-SA(1)*SA(3)
      RF(1,2,NR)=-CA(1)*CA(2)*SA(3)-SA(1)*CA(3)
      RF(1,3,NR)=CA(1)*SA(2)
      RF(2,1,NR)=SA(1)*CA(2)*CA(3)+CA(1)*SA(3)
      RF(2,2,NR)=-SA(1)*CA(2)*SA(3)+CA(1)*CA(3)
      RF(2,3,NR)=SA(1)*SA(2)
      RF(3,1,NR)=-SA(2)*CA(3)
      RF(3,2,NR)=SA(2)*SA(3)
      RF(3,3,NR)=CA(2)
      IF (NR.LT.MR) GOTO 80
C
C==== Generate all pairs of cross-RF peaks, including symmetry-related.
C
 120  IF (NR.EQ.0) CALL CCPERR(1,'*** ERROR: No peaks.')
C
      PL=PL*FM
      KP=0
      DO 150 IR=1,NR-1
        DO 150 JR=IR+1,NR
          DO 150 IS=1,NS
C
C==== Form R2.R1~ and S2.R2.R1~
C
            CALL MXMT3(RF(1,1,JR),RF(1,1,IR),RT)
            CALL MXM3(RS(1,1,IS),RT,R)
            CALL GETEUL(R,A)
C
C==== Get array indices and reduce to asymmetric unit of self-RF.
C
            DO 130 I=1,3
 130          IA(I)=NINT(A(I)/CC(I))
            IF (4*IA(2).GT.NXYZ(2)) THEN
              IF (LT(1)) THEN
                IA(1)=NXYZ(1)-IA(1)
                IA(2)=NXYZ(2)/2-IA(2)
                IA(3)=IA(3)+NXYZ(3)/2
              ELSEIF (LT(2)) THEN
                IA(1)=NXYZ(1)/2-IA(1)
                IA(2)=NXYZ(2)/2-IA(2)
                IA(3)=IA(3)+NXYZ(3)/2
              ENDIF
            ENDIF
            DO 135 I=1,3,2
              IA(I)=MOD(IA(I),NUVW(IXYZ(I)))
 135          IF (IA(I).LT.0) IA(I)=IA(I)+NUVW(IXYZ(I))
            DO 140 I=1,3
              JA(I)=IA(IUVW(I))
              IF (JA(I).LT.0 .OR. JA(I).GT.MUVW(I)) THEN
                WRITE(6,'(/1X,3L8/1X,3F8.2/1X,3I8)') LT,A,IA
                CALL CCPERR(1,'*** ERROR: Map array bound check.')
              ENDIF
 140        CONTINUE
C
C====   Store sort key and index pointer.
C
            IF (RHO(JA(1),JA(2),JA(3)).GT.PL) THEN
              IF (KP.EQ.MSORT)
     &        CALL CCPERR(1,'*** ERROR: Sort array bound check.')
              KP=KP+1
              IN(1,KP)=IR
              IN(2,KP)=JR
              IN(3,KP)=IS
              ISORT(KP)=KP
              SORT(KP)=RHO(JA(1),JA(2),JA(3))
            ENDIF
 150  CONTINUE
C
C====   Sort the peaks in descending order of self-RF value.
C
      IF (KP.EQ.0) CALL CCPERR(1,'*** ERROR: No peaks.')
      CALL BTSORT(KP,ISORT,SORT)
C
      WRITE (6,'(///A,4X,A/)')
     &' Serial   #Peak   #Peak(#Symm)','  Theta    Phi    Chi   self-RF'
      N=0
      DO 180 IP=KP,1,-1
        IR=IN(1,ISORT(IP))
        JR=IN(2,ISORT(IP))
        IS=IN(3,ISORT(IP))
C
C==== Form S2.R2.R1~ and get polar angles.
C
        CALL MXMT3(RF(1,1,JR),RF(1,1,IR),RT)
        CALL MXM3(RS(1,1,IS),RT,R)
        CALL GETPOL(R,X,A)
C
C====   Test chi value if requested, and check asymmetric unit in phi.
C
        IF (A(3).GT.TOL .AND. (CHI.EQ.0. .OR. ABS(A(3)-CHI).LE.TOL))
     &  THEN
          IF (LT(1) .AND. ABS(A(2)).GT.90.)
     &    A(2)=MOD(360.-A(2),360.)-180.
          IF (LT(2) .AND. A(2).LT.0.) A(2)=MOD(180.-A(2),360.)-180.
          IF (LT(3) .AND. A(2).LT.0.) A(2)=A(2)+180.
          IF (A(2).LT.0.) A(2)=A(2)+360.
          DO 160 I=1,3
 160        IA(I)=MOD(NINT(A(I)),360)
          N=N+1
          WRITE (6,'(1X,I6,2I8,1X,A,I2,A,6X,3I7,F10.2)')
     &    N,IR,JR,'(',IS,')',IA,SORT(IP)
          IF (N.LE.NA) THEN
            DO 170 I=1,3
 170          XS(I,N)=X(I)
          ENDIF
          IF (N.EQ.NP) GOTO 190
        ENDIF
 180  CONTINUE
C
C==== Print angles between self-RF points.
C
 190  NA=MIN(NA,N)
      IF (NA.GT.1) THEN
        WRITE (6,'(///A/)')
     &  ' Inter-vector angles: Serial[i]  Serial[j] (Symm[j])  Angle,'//
     &  ' in 4 columns.'
        N=0
        DO 210 IP=1,NA-1
          DO 210 JP=IP+1,NA
            DO 210 IS=1,NS
              CALL MXV3(RS(1,1,IS),XS(1,JP),X)
              AR=0.
              DO 200 I=1,3
 200            AR=AR+XS(I,IP)*X(I)
              N=N+1
              IV(1,N)=IP
              IV(2,N)=JP
              IV(3,N)=IS
              IV(4,N)=NINT(ACOS(MIN(MAX(AR,-1.),1.))/DR)
              IF (IV(4,N).GT.90) IV(4,N)=180-IV(4,N)
              IF (N.EQ.4) THEN
                WRITE (6,'(4(1X,2I4,A,I2,A,I5,2X))')
     &          (IV(1,I),IV(2,I),'(',IV(3,I),')',IV(4,I),I=1,N)
                N=0
              ENDIF
 210    CONTINUE
        WRITE (6,'(4(1X,2I4,A,I2,A,I5,2X))')
     &  (IV(1,I),IV(2,I),'(',IV(3,I),')',IV(4,I),I=1,N)
      ENDIF
      WRITE (6,'(/)')
      END
C
C
      SUBROUTINE MXM3(A,B,C)
C==== A.B -> C
      IMPLICIT NONE
      INTEGER I,J,K
      REAL A(3,3),B(3,3),C(3,3)
C
      DO 1 I=1,3
        DO 1 J=1,3
          C(I,J)=0.
          DO 1 K=1,3
 1          C(I,J)=C(I,J)+A(I,K)*B(K,J)
      END
C
C
      SUBROUTINE MXMT3(A,B,C)
C==== A.B~ -> C
      IMPLICIT NONE
      INTEGER I,J,K
      REAL A(3,3),B(3,3),C(3,3)
C
      DO 1 I=1,3
        DO 1 J=1,3
          C(I,J)=0.
          DO 1 K=1,3
 1          C(I,J)=C(I,J)+A(I,K)*B(J,K)
      END
C
C
      SUBROUTINE MXV3(A,X,Y)
C==== A.X -> Y
      IMPLICIT NONE
      INTEGER I,J,K
      REAL A(3,3),X(3),Y(3)
C
      DO 1 I=1,3
        Y(I)=0.
        DO 1 J=1,3
 1        Y(I)=Y(I)+A(I,J)*X(J)
      END
C
C
      SUBROUTINE GETEUL(R,A)
C
C==== Get the Eulerian angles from the rotation matrix R.
C
      IMPLICIT NONE
      REAL PI,RD
      PARAMETER (PI=3.141592654,RD=180./PI)
      REAL A(3),R(3,3)
C
      A(2)=RD*ACOS(MIN(MAX(R(3,3),-1.),1.))
      IF (ABS(R(3,3)).LT..9999) THEN
        A(1)=RD*ATAN2(R(2,3),R(1,3))
        A(3)=RD*ATAN2(R(3,2),-R(3,1))
      ELSE
        A(1)=RD*ATAN2(R(2,1)*R(3,3),R(1,1)*R(3,3))
        A(3)=0.
      ENDIF
      IF (A(1).LT.0.) A(1)=A(1)+360.
      IF (A(3).LT.0.) A(3)=A(3)+360.
      END
C
C
      SUBROUTINE GETPOL(R,X,A)
C
C==== Get the axis vector & polar angles from the rotation matrix R.
C
      IMPLICIT NONE
      REAL PI,RD
      PARAMETER (PI=3.141592654,RD=180./PI)
      INTEGER I,J,K
      REAL D
      REAL A(3),R(3,3),X(3)
C
      A(3)=-1.
      D=0.
      DO 1 I=1,3
        A(3)=A(3)+R(I,I)
        J=MOD(I,3)+1
        K=MOD(J,3)+1
        X(I)=R(K,J)-R(J,K)
        IF (ABS(X(I)).LT.5E-5) X(I)=0.
 1      D=D+X(I)**2
      A(3)=RD*ACOS(MIN(MAX(.5*A(3),-1.),1.))
      IF (A(3).LT.5E-2) THEN
        A(1)=0.
        A(2)=0.
        RETURN
      ENDIF
      IF (D.EQ.0.) THEN
        DO 2 I=1,3
          X(I)=MAX(.5*(R(I,I)+1.),0.)
          D=D+X(I)
 2        X(I)=SQRT(X(I))
        IF (R(1,3).LT.0.) X(1)=-X(1)
        IF (R(2,3).LT.0.) X(2)=-X(2)
      ENDIF
      D=SQRT(D)
      DO 3 I=1,3
        IF (X(3).LT.0.) X(I)=-X(I)
 3      X(I)=X(I)/D
      A(1)=RD*ACOS(MIN(X(3),1.))
      IF (X(1).EQ.0. .AND. X(2).EQ.0.) THEN
        A(2)=0.
      ELSE
        A(2)=RD*ATAN2(X(2),X(1))
        IF (A(2).EQ.180.) A(2)=-180.
      ENDIF
      END
C
C
      SUBROUTINE BTSORT(N,IA,A)
C
C==== Binary tree sort in ascending order.
C==== IA(I) is table index, A(I) is sort key.
C
      IMPLICIT NONE
      INTEGER I,IS,J,L,M,N
      INTEGER IA(N)
      REAL S
      REAL A(N)
C
      L=N/2+1
      M=N
 1    IF (L.GT.1) THEN
        L=L-1
        IS=IA(L)
        S=A(L)
      ELSE
        IS=IA(M)
        IA(M)=IA(1)
        S=A(M)
        A(M)=A(1)
        M=M-1
        IF (M.LE.1) THEN
          IA(1)=IS
          A(1)=S
          RETURN
        ENDIF
      ENDIF
      I=L
      J=L+L
 2    IF (J.LE.M) THEN
        IF (J.LT.M) THEN
          IF (A(J).LT.A(J+1)) J=J+1
        ENDIF
        IF (S.LT.A(J)) THEN
          IA(I)=IA(J)
          A(I)=A(J)
          I=J
          J=J+J
        ELSE
          J=M+1
        ENDIF
        GOTO 2
      ENDIF
      IA(I)=IS
      A(I)=S
      GOTO 1
      END

