C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
************************************************************************
*
*       Package ERRLIB
*       Routines for error control and output
*
*       Contains:
*       s/r errint
*       s/r errmsg
*       s/r errmsi
*       s/r errmsf
*       s/r wrnmsg
*       s/r wrnmsi
*       s/r wrnmsf
************************************************************************
************************************************************************
*
*       Subroutine ERRINT
*
*
*       Function:
*       Initiate error count
*
*       Comment:
*       This routine should be called prior to any off the ERRMS 
*       routines
*       
*       Arguments:
*       Boooom   (i)      maximum allowed error count
*
*       History:
*       Written (!) December 4 1991 S. Knight
************************************************************************
        subroutine errint ( boooom )

        implicit none

*       Arguments
        integer         boooom

*       Error control common
        integer         errcnt
        integer         bomblevel
        common /errctl/ errcnt , bomblevel
        save /errctl/
*-----------------------------------------------------------------------
        errcnt = 0
        bomblevel = boooom
        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine ERRMSG
*
*
*       Function:
*       Output error message, return or stop
*
*       Arguments :
*       Msg     (i)     character string to be output as error message
*       Istop   (i)     stop if istop = 1, otherwise return
*
*       History:
*       December 4 1991 sdk
*       Bomblevel now in common errctl; defined by call to ERRINT
*       November 1991 sdk
*       Introduced bomblevel
*       Written March 1988 S. Knight
************************************************************************
        subroutine errmsg ( msg , istop )

        implicit none

        include         'rspsfil.inc'

*       Arguments
        character*(*)   msg
        integer         istop

*       Error control common
        integer         errcnt
        integer         bomblevel
        common /errctl/ errcnt , bomblevel
        save /errctl/
*-----------------------------------------------------------------------

        errcnt = errcnt + 1
        write ( 6,2000 ) msg
 2000   format(10x,'** ERROR - ',a,' **' )

        if ( istop .eq. 1 .or. errcnt .gt. bomblevel ) goto 9999
        return

 9999   call ttycr( 5 )
        if ( errcnt .gt. bomblevel ) then
        call ttyptl 
     &  ( ' RSPS > Uuhghh hrumph wragh zplt zstrophfstrangsjkj' )
        call ttyptl 
     &  ( ' RSP****** #@$@# $@!!!@  $@#$fzxh@#&%!! *******' )
        call ttyptl 
     &  ( ' RShje...  lllghlll....... zzzzzzzzzz.s......' )
        endif
        call rsinit ( 'ALL' )
        close ( lunpat )
        close ( lunsco )
        close ( lunsgs )
        close ( lunncs )
        close ( lunxin )
        close ( lunxut )
        call ttycr( 5 )
        stop'z'
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine ERRMSI
*
*
*       Function:
*       Output error message + integer, return or stop
*
*       Comment:
*       No test is done that the number of positions requested for the
*       integer value doesn't overflow the format statement
*
*       Arguments :
*       Msg     (i)     character string to be output as error message
*       Int     (i)     integer variable
*       Len     (i)     number of positions to use for integer value
*       Istop   (i)     stop if istop = 1, otherwise return
*
*       History:
*       From ERRMSG December 4 1991 S. Knight
************************************************************************
        subroutine errmsi ( msg , int , len , istop )

        implicit none

        include         'rspsfil.inc'

*       Arguments
        character*(*)   msg
        integer         int
        integer         len
        integer         istop

*       Local variables
        character*40    fmtlin
        data            fmtlin 
     &                  / '(10x,''** ERROR - '',a,i  '' **'' )' /

*       Error control common
        integer         errcnt
        integer         bomblevel
        common /errctl/ errcnt , bomblevel
        save /errctl/
*-----------------------------------------------------------------------

        errcnt = errcnt + 1
        write ( fmtlin(23:24),'(i2)' ) len
        write ( 6,fmtlin ) msg , int

        if ( istop .eq. 1 .or. errcnt .gt. bomblevel ) goto 9999
        return

 9999   call ttycr( 5 )
        if ( errcnt .gt. bomblevel ) then
        call ttyptl 
     &  ( ' RSPS > Uuhghh hrumph wragh zplt zstrophfstrangsjkj' )
        call ttyptl 
     &  ( ' RSP****** #@$@# $@!!!@  $@#$fzxh@#&%!! *******' )
        call ttyptl 
     &  ( ' RShje...  lllghlll....... zzzzzzzzzz.i......' )
        endif
        call rsinit ( 'ALL' )
        close ( lunpat )
        close ( lunsco )
        close ( lunsgs )
        close ( lunncs )
        close ( lunxin )
        close ( lunxut )
        call ttycr( 5 )
        stop'z'
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine ERRMSF
*
*
*       Function:
*       Output error message + real, return or stop
*
*       Comment:
*       No test is done that the number of positions requested for the
*       real value doesn't overflow the format statement
*
*       Arguments :
*       Msg     (i)     character string to be output as error message
*       Rea     (i)     real variable
*       Len     (i)     number of positions to use for real value
*       Dec     (i)     number of decimal positions
*       Istop   (i)     stop if istop = 1, otherwise return
*
*       History:
*       From ERRMSG December 4 1991 S. Knight
************************************************************************
        subroutine errmsf ( msg , rea , len , dec , istop )

        implicit none

        include         'rspsfil.inc'

*       Arguments
        character*(*)   msg
        real            rea
        integer         len
        integer         dec
        integer         istop

*       Local variables
        character*40    fmtlin
        data            fmtlin 
     &                  / '(10x,''** ERROR - '',a,f  .  '' **'' )' /

*       Error control common
        integer         errcnt
        integer         bomblevel
        common /errctl/ errcnt , bomblevel
        save /errctl/
*-----------------------------------------------------------------------

        errcnt = errcnt + 1
        write ( fmtlin(23:24),'(i2)' ) len
        write ( fmtlin(26:27),'(i2)' ) dec
        write ( 6,fmtlin ) msg , rea

        if ( istop .eq. 1 .or. errcnt .gt. bomblevel ) goto 9999
        return

 9999   call ttycr( 5 )
        if ( errcnt .gt. bomblevel ) then
        call ttyptl 
     &  ( ' RSPS > Uuhghh hrumph wragh zplt zstrophfstrangsjkj' )
        call ttyptl 
     &  ( ' RSP****** #@$@# $@!!!@  $@#$fzxh@#&%!! *******' )
        call ttyptl 
     &  ( ' RShje...  lllghlll....... zzzzzzzzzz.f......' )
        endif
        call rsinit ( 'ALL' )
        close ( lunpat )
        close ( lunsco )
        close ( lunsgs )
        close ( lunncs )
        close ( lunxin )
        close ( lunxut )
        call ttycr( 5 )
        stop'z'
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine WNRMSG
*
*
*       Function:
*       Output warning message
*
*       Arguments :
*       Msg     (i)     character string to be output as error message
*
*       History:
*       From ERRMSG December 4 1991 S. Knight
************************************************************************
        subroutine wrnmsg ( msg )

        implicit none

*       Arguments
        character*(*)   msg
*-----------------------------------------------------------------------

        write ( 6,2000 ) msg
 2000   format(10x,'** WARNING - ',a,' **' )
        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine WRNMSI
*
*
*       Function:
*       Output warning message + integer
*
*       Comment:
*       No test is done that the number of positions requested for the
*       integer value doesn't overflow the format statement
*
*       Arguments :
*       Msg     (i)     character string to be output as error message
*       Int     (i)     integer variable
*       Len     (i)     number of positions to use for integer value
*
*       History:
*       From ERRMSG December 4 1991 S. Knight
************************************************************************
        subroutine wrnmsi ( msg , int , len )

        implicit none

*       Arguments
        character*(*)   msg
        integer         int
        integer         len

*       Local variables
        character*40    fmtlin
        data            fmtlin 
     &                  / '(10x,''** WARNING - '',a,i  '' **'' )' /
*-----------------------------------------------------------------------

        write ( fmtlin(25:26),'(i2)' ) len
        write ( 6,fmtlin ) msg , int
        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine WRNMSF
*
*
*       Function:
*       Output warning message + real
*
*       Comment:
*       No test is done that the number of positions requested for the
*       real value doesn't overflow the format statement
*
*       Arguments :
*       Msg     (i)     character string to be output as error message
*       Rea     (i)     real variable
*       Len     (i)     number of positions to use for real value
*       Dec     (i)     number of decimal positions
*
*       History:
*       From ERRMSG December 4 1991 S. Knight
************************************************************************
        subroutine wrnmsf ( msg , rea , len , dec )

        implicit none

*       Arguments
        character*(*)   msg
        real            rea
        integer         len
        integer         dec

*       Local variables
        character*40    fmtlin
        data            fmtlin 
     &                  / '(10x,''** WARNING - '',a,f  .  '' **'' )' /

*       Error control common
        integer         errcnt
        integer         bomblevel
        common /errctl/ errcnt , bomblevel
        save /errctl/
*-----------------------------------------------------------------------

        errcnt = errcnt + 1
        write ( fmtlin(25:26),'(i2)' ) len
        write ( fmtlin(28:29),'(i2)' ) dec
        write ( 6,fmtlin ) msg , rea
        return
        end
*********************************************************************SDK
