C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C Read help text file and output requested parts.
C
C Version 1.0
C
C This procedure uses a number of separately compiled procedures,
C since it can be assumed that it will be used in programs where
C these procedures will be used anyhow.
C
C Note: it is assumed that the calling program has already
C initialized the lexical line analyzer properly.
C
C Questions issued to user by procedure, and their possible answers:
C
C "Topic > "
C     CR  : skip out to end of help file
C     ?   : jump back to start of help file
C     ... : word(s) to output further help text on
C
C "Continue > "
C     CR : output one more screenful of current word help text
C     ?  : jump back to start of help file
C     Q  : quit current help text, and skip to end of help file
C
C "End of help > " and "No help found > "
C     CR  : exit from procedure
C     ?   : jump back to start of help file
C     ... : word(s) to output help on, as if reentering procedure
C
C Uses separately compiled procedures/packages:
C
C STRUTL p  character case control
C STRBLK p  find and eliminate blanks in strings
C LEXANA p  lexical line analyser; find words and delimiters
C TTYIOP p  terminal input/output procedures
C
C Copyright (C) 1988 Per Kraulis
C
C Per Kraulis, Dept Molecular Biology, Uppsala University Sweden
C   26-Feb-1988  first attempts
C    2-Mar-1988  first working version
C    8-Mar-1988  introduced found flag
C
C Modified 95-03-06 SDK: Call to lxline changed to new format
C
C----------------------------------------------------
      SUBROUTINE HLPTXT (LUN, FILNAM, TOPLIN, ERRCOD)

C
      INTEGER       LUN, ERRCOD
      CHARACTER*(*) FILNAM, TOPLIN

C
C LUN     (In)    logical unit number to use
C FILNAM  (In)    name of help text file
C TOPLIN  (InOut) line on topic to output help on; may be modified
C ERRCOD  (Out)   error code
C     0 = no error
C     1 = could not open or read help text file
C     2 = too many topic words
C     3 = input error encountered
C     4 = format error in file; no word or level in title line
C
C Externally defined functions
C
      INTEGER SLNBLK
C
C MAXTOP  max number of topic words
C SCREEN  max number of lines output to screen at one go
C MARKER  title line marker character
C
      INTEGER     MAXTOP, SCREEN
      CHARACTER*1 MARKER
      PARAMETER  (MAXTOP = 9, SCREEN = 20, MARKER = '>')
C
C TOPPOS  positions of topic words in topic line
C TOPTYP  topic word type code
C TOPNUM  total number of words in topic line
C TOPCUR  current topic word
C
      INTEGER      TOPPOS (2, MAXTOP), TOPTYP (MAXTOP)
      INTEGER      TOPNUM, TOPCUR
C
C OUTNUM  number of lines output to screen
C OUTPUT  flag when reading text to output to user
C MATCH   previous topic word matched with title word
C FOUND   flag if requested topic found
C
      INTEGER OUTNUM
      LOGICAL OUTPUT, MATCH, FOUND
C
C Help variables
C
      CHARACTER*80 LINE
      INTEGER      DEPTH, LAST
C
C Open the help text file; to check that it exists
C
cpjx      OPEN (UNIT=LUN, FILE=FILNAM, STATUS='OLD',
cpjx     $      FORM='FORMATTED', READONLY, ERR=900)
      OPEN (UNIT=LUN, FILE=FILNAM, STATUS='OLD',
     $      FORM='FORMATTED', ERR=900)

C Analyze topic line, rewind help text file in case this is a loopback
C
100   CALL LXLINE (TOPLIN, MAXTOP, TOPPOS, TOPTYP, TOPNUM, ERRCOD)
      IF (ERRCOD .EQ. 1) GOTO 910
      REWIND (UNIT=LUN)
C
C Upcase for comparison, init current topic word counter and flags
C If no topic, then output help file start text
C
      IF (TOPNUM .EQ. 0) THEN
        TOPCUR = 0
        OUTNUM = 0
        MATCH = .TRUE.
        FOUND = .TRUE.
        OUTPUT = .TRUE.
      ELSE
        CALL SUPCAS (TOPLIN (: TOPPOS (2, TOPNUM)))
        TOPCUR = 1
        MATCH = .FALSE.
        FOUND = .FALSE.
        OUTPUT = .FALSE.
      END IF
C
C Loop until correct depth and word of help tree has been found
C
200   READ (UNIT=LUN, FMT=210, END=800, ERR=900) LINE
210   FORMAT (A)
C
C Line is a title line; get depth of it
C
      IF (LINE (1:1) .EQ. MARKER) THEN
C
        READ (LINE (2:2), 220, ERR=930) DEPTH
220     FORMAT (I1)
        IF (DEPTH .EQ. 0) GOTO 930
C
C Same title line depth as current topic word
C
        IF (DEPTH .EQ. TOPCUR) THEN
C
C Get word from title; check that there is a word
C
          LINE = LINE (3 :)
          CALL SLESHF (LINE)
          LAST = SLNBLK (LINE)
          IF (LAST .EQ. 0) GOTO 930
          CALL SUPCAS (LINE (: LAST))
C
C Test if what we have of the words match
C
          IF (TOPPOS(2,TOPCUR) - TOPPOS(1,TOPCUR) + 1 .LT. LAST) THEN
            MATCH = TOPLIN (TOPPOS(1,TOPCUR) : TOPPOS(2,TOPCUR)) .EQ.
     $              LINE (: TOPPOS(2,TOPCUR) - TOPPOS(1,TOPCUR) + 1)
          ELSE
            MATCH = TOPLIN (TOPPOS(1,TOPCUR) :
     $                      TOPPOS(1,TOPCUR) + LAST - 1) .EQ. LINE
          END IF
C
          IF (MATCH) THEN
C
C Words match at this level; increment and skip back for next match
C
            IF (TOPNUM .GT. TOPCUR) THEN
              MATCH = .TRUE.
              TOPCUR = TOPCUR + 1
C
C Last topic to match; set flags for text output
C
            ELSE
              OUTNUM = 0
              FOUND = .TRUE.
              OUTPUT = .TRUE.
            END IF
C
C Skip this part if current topic word and title word do not match
C
          ELSE
            OUTPUT = .FALSE.
          END IF
C
C Finished if title line depth value is lower than current topic
C
        ELSE IF (DEPTH .LT. TOPCUR) THEN
          GOTO 800
C
C New choice if prev word matched and depth value is higher than prev
C
        ELSE IF (DEPTH .GT. TOPNUM .AND. MATCH) THEN
C
          CALL TTYGTL ('Topic > ', LINE, ERRCOD)
          IF (ERRCOD .NE. 0) GOTO 920
          CALL SLESHF (LINE)
C
C Exit if no more topic given
C
          IF (LINE .EQ. ' ') THEN
            GOTO 800
C
C Start again if '?' given
C
          ELSE IF (LINE .EQ. '?') THEN
            TOPLIN = ' '
            GOTO 100
C
C Add on topic word(s) to topic line; reanalyze the line
C
          ELSE
            LAST = SLNBLK (LINE)
            CALL SUPCAS (LINE (: LAST))
            CALL SAPPND (TOPLIN, ' ' // LINE (: LAST))
            CALL LXLINE (TOPLIN, MAXTOP, TOPPOS, TOPNUM, TOPTYP, ERRCOD)
            IF (ERRCOD .EQ. 1) GOTO 910
C
C Start looking at added word and current line
C
            FOUND = .FALSE.
            OUTPUT = .FALSE.
            TOPCUR = TOPCUR + 1
            BACKSPACE (UNIT=LUN)
          END IF
C
        END IF
C
C Output help text if flag set
C
      ELSE IF (OUTPUT) THEN
C
        LAST = SLNBLK (LINE)
        IF (LAST .EQ. 0) THEN
          CALL TTYCR (1)
        ELSE
          CALL TTYPTL (LINE (: LAST))
        END IF
C
C Wait for go-ahead from user when screen full
C
        OUTNUM = OUTNUM + 1
        IF (MOD (OUTNUM, SCREEN) .EQ. 0) THEN
          CALL TTYCR (1)
          CALL TTYGTL ('Continue > ', LINE, ERRCOD)
          IF (ERRCOD .NE. 0) GOTO 920
          CALL TTYCR (1)
C
          IF (LINE .EQ. 'q' .OR. LINE .EQ. 'Q') THEN
            GOTO 800
          ELSE IF (LINE .EQ. '?') THEN
            TOPLIN = ' '
            GOTO 100
          END IF
        END IF
C
      END IF
C
C Loop back for next line in help file
C
      GOTO 200
C
C Finish output when nothing more left, or return again
C
800   IF (FOUND) THEN
        CALL TTYGTL ('Topic > ', TOPLIN, ERRCOD)
      ELSE
        CALL TTYPTL ('No help found')
        CALL TTYGTL ('Topic > ', TOPLIN, ERRCOD)
      END IF
      IF (ERRCOD .NE. 0) THEN
        GOTO 920
      ELSE IF (TOPLIN .EQ. '?') THEN
        TOPLIN = ' '
        GOTO 100
      ELSE IF (TOPLIN .NE. ' ') THEN
        GOTO 100
      ELSE
        CLOSE (UNIT=LUN)
        ERRCOD = 0
        RETURN
      END IF
C
900   ERRCOD = 1
      RETURN
C
910   CLOSE (UNIT=LUN)
      ERRCOD = 2
      RETURN
C
920   CLOSE (UNIT=LUN)
      ERRCOD = 3
      RETURN
C
930   CLOSE (UNIT=LUN)
      ERRCOD = 4
      RETURN
C
      END
