C
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
************************************************************************
*
*
*       RSPSSYM.INC for rsps04.
*       Symmetry operators. The matrices and vectors here depend on
*       the file RSPSDIM.INC for their proper dimensioning.
*
*       June 31 1991 SDK
************************************************************************
*       First index points to row, second to column
*
*       Space group symmetry
*
*       NOPSGS          number of space group symmetry operations
*       R_SGS,T_SGS     space group symmetry
*                       these operators operate on the crystallographic
*                       coordinates (fractional or Angstrom), translation
*                       vectors are fractional, generate crystallographic
*                       coordinates (fractional or Angstrom).
*
*       Non crystallographic symmetry
*
*       NOPNCS          number of non crystallographic symmetry operations
*       R_NCS,T_NCS     non crystallographic symmetry 
*                       these operators operate on the crystallographic
*                       coordinates (Angstrom), translation vectors are
*                       fractional, generate crystallographic coordinates 
*                       (Angstrom).
*
*       Combined (full) symmetry
*
*       NOPSYM          total number of symmetry operations
*       R_SYM,T_SYM     "full" symmetry ([sg]*[ncs])
*                       these operators operate on the crystallographic
*                       coordinates (Angstrom), translation vectors are
*                       fractional
*       SGSPNT          pointer from "full" symmetry operator number to
*                       corresponding space group symmetry operator number
*       NCSPNT          pointer from "full" symmetry operator number to
*                       corresponding ncs operator number
*
*       Vector generators from real space to vector space
*
*       NTRANS          number of p'son vectors
*       R_VEC,T_VEC     hold mapping from real space to vector space
*                       operate on fractional crystallographic coordinates
*
*       Laue group (Patterson) symmetry
*
*       R_LGS           holds Laue group symmetry ( 2*nopsgs operators)
*                       these operators operate on the crystallographic
*                       coordinates on grid PATGRD to generate coordinates
*                       on the same grid
*
*       Centering vectors
*
*       ICEN            number of centering translations
*       CENT            holds centering translations (fractional)
*
*
*       Harker plane equations
*       HPLANE          For each Harker plane; normal vector and distance
*                       to origin
*       
*       Miscellaneous
*
*       MULT            multiplicities of p'son vectors
*       VECTYP          type of p'son vector (harker/cross)
*       POLAR           flag for polar axes
*       NPOLX           number of polar axes
*       INVER           inversion matrix 
*       IDENT           identity matrix
*       TNULL           null vector
        integer         nopsgs
        integer         nopncs
        integer         nopsym
        integer         ntrans
        integer         icen
        integer         mult( mxvec )
        integer         vectyp( mxvec )
        integer         polar( 3 )
        integer         npolx
        integer         sgspnt( mxsym )
        integer         ncspnt( mxsym )
        integer         nplanes
        real            r_sgs( 3 , 3 , mxsgs )
        real            t_sgs(     3 , mxsgs )
        real            r_ncs( 3 , 3 , mxncs )
        real            t_ncs(     3 , mxncs )
        real            r_sym( 3 , 3 , mxsym )
        real            t_sym(     3 , mxsym )
        real            r_vec( 3 , 3 , mxvec )
        real            t_vec(     3 , mxvec )
        real            r_lgs( 3 , 3 , 2 * mxsgs )
        real            cent( 3 , 4 )
        real            hplane( 4, mxsgs*(mxsgs-1) )

        common/symm/    nopsgs , r_sgs , t_sgs , 
     &                  nopncs , r_ncs , t_ncs , 
     &                  nopsym , r_sym , t_sym , 
     &                  ntrans , r_vec , t_vec , 
     &                  r_lgs  , icen  , cent  , 
     &                  mult   , vectyp, polar ,
     &                  npolx  , sgspnt, ncspnt,
     &                  nplanes , hplane

        real            r_inv( 3 , 3 ) , r_idt( 3 , 3 ) , t_nul( 3 )
        data            r_inv/-1.,0.,0.,  0.,-1.,0.,  0.,0.,-1./
        data            r_idt/ 1.,0.,0.,  0., 1.,0.,  0.,0., 1./
        data            t_nul/ 0.,0.,0./
************************************************************************





