C
C     SEQWT
C     Copyright (C) 2007 Eleanor Dodson
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
C---------------------------------------------------------------------------
C
C     SEQWT
C
C---------------------------------------------------------------------------
C
C  Run as 
C
C  seqwt SEQUENCE x.seq
C
C  where x.seq is a file containing the protein sequence.
C   Any other "residue" ID such as W or WAT or HOH is ignored
C
C  No other keyworded input (except END)
C
C  The SEQUENCE file can be in these formats:
C  the *.pir type
C x.seq
C > chain 1
C  MTSKIEQPRW ASKDSAAGAA STPDEKIVLE FMDALTSNDA AKLIEYFAED TMYQNMPLPP
C  AYGRDAVEQT LAGLFTVMSI DAVETFHIGS SNGLVYTERV DVLRALPTGK SYNLSILGVF
C  QLTEGKITGW RDYFDLREFE EAVDLPLRG
C 
C > chain 2
C 
C  MTSKIEQPRW ASKDSAAGAA STPDEKIVLE FMDALTSNDA AKLIEYFAED TMYQNMPLPP
C  AYGRDAVEQT LAGLFTVMSI DAVETFHIGS SNGLVYTERV DVLRALPTGK SYNLSILGVF
C QLTEGKITGW RDYFDLREFE EAVDLPLRG
C
C
C  Or just the sequence list..
C x.seq
C  MTSKIEQPRW ASKDSAAGAA STPDEKIVLE FMDALTSNDA AKLIEYFAED TMYQNMPLPP
C  AYGRDAVEQT LAGLFTVMSI DAVETFHIGS SNGLVYTERV DVLRALPTGK SYNLSILGVF
C  QLTEGKITGW RDYFDLREFE EAVDLPLRG

C  Or the SEQRES records in the PDB format
C seqres.seq
CSEQRES   1 A   21  GLY ILE VAL GLU GLN CYS CYS THR SER ILE CYS SER LEU  4INS 170
CSEQRES   2 A   21  TYR GLN LEU GLU ASN TYR CYS ASN                      4INS 171
CSEQRES   1 B   30  PHE VAL ASN GLN HIS LEU CYS GLY SER HIS LEU VAL GLU  4INS 172
CSEQRES   2 B   30  ALA LEU TYR LEU VAL CYS GLY GLU ARG GLY PHE PHE TYR  4INS 173
CSEQRES   3 B   30  THR PRO LYS ALA                                      4INS 174
CSEQRES   1 C   21  GLY ILE VAL GLU GLN CYS CYS THR SER ILE CYS SER LEU  4INS 175
CSEQRES   2 C   21  TYR GLN LEU GLU ASN TYR CYS ASN                      4INS 176
CSEQRES   1 D   30  PHE VAL ASN GLN HIS LEU CYS GLY SER HIS LEU VAL GLU  4INS 177
CSEQRES   2 D   30  ALA LEU TYR LEU VAL CYS GLY GLU ARG GLY PHE PHE TYR  4INS 178
CSEQRES   3 D   30  THR PRO LYS ALA                                      4INS 179
C
      program seqwt
c
      implicit none
c
c Internal flags, counting variables etc
      integer i,j,k,l,len,ifail
c
c Sequence names
      integer maxaa,maxchn
      parameter (maxaa=21,maxchn=50)
C
C Sequence
      integer maxseq,ichk,nrestot
      parameter (maxseq=5000)
      character  line*80, line1*10000, seqid1(maxaa)*1,seqid3(maxaa)*4
      character  chnnow*1, chnid(50)*1,seq3(13)*4
      integer mwseqid(maxaa), nres(maxaa),mwchn(maxchn),mwtot
      real frac
C
C Functions
      integer lenstr
      external lenstr
C
C  Give molecular weight of each type of amino acid
      DATA seqid3/'ALA ','ARG ','ASN ','ASP ','CYS ','GLN ','GLU ',
     +            'GLY ','HIS ','ILE ','LEU ','LYS ','MET ','ORN ',
     +            'PHE ','PRO ','SER ','THR ','TRP ','TYR ','VAL '/

      DATA seqid1/'A','R','N','D','C','Q','E',
     1            'G','H','I','L','K','M','X',
     1            'F','P','S','T','W','Y','V'/
      DATA MWSEQID/ 71 , 157 , 114 , 114 , 102 , 128 , 128,
     1              57 , 138 , 113 , 113 , 129 , 131 , 114,
     1             147 ,  97 ,  87 , 101 , 186 , 163 ,  99  /

C==========================================================================
C  Initializations
C==========================================================================
      call ccpfyp
      call ccp4h_init()
      CALL CCPRCS(6,'SEQWT','$Date: 2007/07/09 14:31:45 $')
c

         ifail = 0
         call ccpdpn(9, 'SEQUENCE', 'READONLY', 'F', 0, ifail)
         if ( ifail.lt.0 ) then
c          Failed to open the file
           call ccperr(1,"Error opening sequence file")
         endif

         chnnow = '>'
         mwtot = 0
          do i = 1,maxchn
            mwchn(i) = 0
          end do 
         k = -999
         nrestot = 0
         do j = 1,maxaa
           nres(j) = 0
         end do 
   1     Read(9,'(A)',end=10,err=20) line

         if(line(1:6).eq.'SEQRES') then 
C   line(12:12) is chnid  - advance chain if necessary
CC  New chain
            If (line(12:12) .NE. chnnow) then
CC  First chain
             If ( chnnow.eq.'>')   k = 1
C  Later chain
             If ( chnnow.ne.'>')  then
C  Write out previous chain if any
               write(6,'(A,A1,/,200(/,6(1x,10a1)))') 
     1          '>chain',chnnow,(line1(i:i) ,i=1,l)
               mwtot = mwtot + mwchn(k)
               nrestot = 0
                do j=1,maxaa
                  nrestot = nrestot + nres(j)
                end do 
               if(nrestot.eq.0) 
     1           write(6,*) ' No protein residues for Chain',k
               if(nrestot.gt.0) then
               Write(6,'(/,A,I3,A,i5,//,a)')
     1        ' Chain', k,' Total Number of Residues',nrestot,
     1        ' Residue Type    Number  Percentage'
                do j=1,maxaa
                   frac = 100.0*nres(j)/nrestot
                    Write(6,'(i3,3x,A3,3x,A1, I7,F13.2)') 
     1                  j,seqid3(j),seqid1(j), nres(j),frac
                end do 
                write(6,'(A,I4,A,I6)') ' Chain',k,' Molecular Weight', 
     1                                    mwchn(k)
                k = k+1
               end if
             end if

             chnnow = line(12:12)
              line1 = '  '
              l = 0
            nrestot = 0
            do j = 1,maxaa
              nres(j) = 0
            end do 
            end If 

C read rest of SEQRES line
             read(line(20:70),'(13A4)')(seq3(i),i=1,13)
            do i = 1,13
             do j = 1,maxaa
              if (seq3(i).eq.seqid3(j)) then
                  l = l+1
                  line1(l:l)=seqid1(j) 
                  if (l.gt.10000) write(6,*)
     1    ' Sequence longer than line1 - increase dimension'
                  ichk = 1
                  nrestot = nrestot + 1
                  nres(j) = nres(j) + 1
                  mwchn(k) = mwchn(k) + mwseqid(j)
              end if 
             end do 
            end do 
           go to 1
         end if
C
C   pir style sequence -  Line beginning ">" means new chain - skip blank lines
         if(line(1:1).eq.'>') then
C write out results for last chain
           if(k.ge.1) then
            mwtot = mwtot + mwchn(k)
            nrestot = 0
             do j=1,maxaa
               nrestot = nrestot + nres(j)
             end do 
               if(nrestot.eq.0) 
     1           write(6,*) ' No protein residues for Chain',k
               if(nrestot.gt.0) then
              Write(6,'(A,I3,A,i5,//,a)')
     1       ' Chain', k,' Total Number of Residues',nrestot,
     1       ' Residue Type    Number  Percentage'
            do j=1,maxaa
               frac = 100.0*nres(j)/nrestot
                Write(6,'(i3,3x,A3,3x,A1, I7,F13.2)') 
     1              j,seqid3(j),seqid1(j), nres(j),frac
            end do 
           write(6,'(A,I4,A,I6,A)') ' Chain',k,' Molecular Weight', 
     1                               mwchn(k), ' (Daltons)'
               end if
           end if
            nrestot = 0
            do j = 1,maxaa
              nres(j) = 0
            end do 
           k = k+1
C  Is this the first chain?
           if (k.lt.0) k = 1
           go to 1
         end if
C Is this a blank line?
         len = lenstr(line)
         if(len.eq.0) go to 1
C  read a sequence line, If no > then k still negative.
           if (k.lt.0) k = 1
C   parse sequence         
           DO 2 i = 1,len
c skip blanks
            if(line(i:i).eq.' ') go to 2
            ichk = 0
            do j = 1,maxaa
             if(line(i:i).eq.seqid1(j)) then
                 ichk = 1
                 nrestot = nrestot + 1
                 nres(j) = nres(j) + 1
                 mwchn(k) = mwchn(k) + mwseqid(j)
             end if
            end do
            if(ichk.eq.0) write(6,*) 
     1      ' Residue type unrecognised:  ', line(i:i)
 2         end do 
           go to 1

 20        continue
C  arrive here on read error
           call ccperr(1,"Error reading from input sequence file")

10         continue
C  results for last chain
            mwtot = mwtot + mwchn(k)
            nrestot = 0
             do j=1,maxaa
               nrestot = nrestot + nres(j)
             end do 
               if(nrestot.eq.0) 
     1           write(6,*) ' No protein residues for Chain',k
               if(nrestot.gt.0) then
              Write(6,'(//,A,I3,A,i5,//,a)')
     1       ' Chain', k,' Total Number of Residues',nrestot,
     1       ' Residue Type    Number  Percentage'
            do j=1,maxaa
               frac = 100.0*nres(j)/nrestot
                Write(6,'(i3,3x,A3,3x,A1, I7,F13.2)') 
     1              j,seqid3(j),seqid1(j), nres(j),frac
             end do 
               end if
           write(6,'(A,I4,A,I6,A,//A,I6,A)') ' Chain',k,
     1              ' Molecular Weight', mwchn(k), ' (Da)',
     2              ' Total     Molecular Weight', mwtot, ' (Da)'
C
           call ccperr(0,'Normal Termination')
           end
C
