/*

     This code is distributed under the terms and conditions of the
     CCP4 Program Suite Licence Agreement as a CCP4 Application.
     A copy of the CCP4 licence can be obtained by writing to the
     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
*/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "solomon.h"
#include "ccp4_general.h"

extern BOOLEAN verbose;




#if defined (PROTOTYPE)
  void ExtractMap(MAP *frm, MAP *to, TRANSFORM *trans, int ntrans)
#else
  void ExtractMap(frm, to, trans, ntrans)
  MAP *frm, *to;
  TRANSFORM *trans;
  int ntrans;
#endif
/* The map 'to' is filled with density values from map 'frm', using the
   transformations defined in 'trans[]' for mapping 'frm' on 'to'. The
   routine is much faster than 'MapFill', but is much less flexible: no
   interpolation is undertaken. Therefore this procedure is only suitable
   if 'trans[]' only defines crystallographic symmetry operations. It is 
   assumed that every gridpoint of map 'to' can be retrieved from map 'frm'.
   If this is not the case, use the routine MapGridFill. */

{ TRANSFORM *ctrans, *tf;
  int i, p[3], t0, t1, t2, itf, tries;
  double *m, tmp[3];
  BOOLEAN success, unit_oper;
#if !defined (PROTOTYPE)
  void InvertTrans();
  void TransAToGrid();
#endif

  ctrans = (TRANSFORM *) malloc(ntrans * sizeof(TRANSFORM));
  for (i = 0; i < ntrans; ++i)
  { InvertTrans(trans + i, ctrans + i);
    TransAToGrid(ctrans + i, ctrans + i, frm->cell, frm->grid);}
  tf = ctrans + (itf = 0);
  unit_oper = ((tf->rotate[0] > 0.9999) && 
	       (tf->rotate[4] > 0.9999) && 
	       (tf->rotate[8] > 0.9999) && 
	       (ABS(tf->translate[0]) < 0.0001) && 
	       (ABS(tf->translate[1]) < 0.0001) && 
	       (ABS(tf->translate[2]) < 0.0001));
  for (t0 = to->ori[0]; t0 < (to->ori[0] + to->ext[0]); ++t0)
    for (t1 = to->ori[1]; t1 < (to->ori[1] + to->ext[1]); ++t1)
      for (t2 = to->ori[2]; t2 < (to->ori[2] + to->ext[2]); ++t2)
      { for (tries=0, success=FALSE; (!success) && (tries < ntrans); ++tries)
	{ m = tf->rotate;
	  if (!unit_oper)
	  { tmp[0] = m[0] * (float) t0 + m[3] * (float) t1 + m[6] * (float) t2;
	    p[0] = NINT(tmp[0] + tf->translate[0]);
	    tmp[1] = m[1] * (float) t0 + m[4] * (float) t1 + m[7] * (float) t2;
	    p[1] = NINT(tmp[1] + tf->translate[1]);
	    tmp[2] = m[2] * (float) t0 + m[5] * (float) t1 + m[8] * (float) t2;
	    p[2] = NINT(tmp[2] + tf->translate[2]);}
	  else
	  { p[0] = t0;
	    p[1] = t1;
	    p[2] = t2;}
	  for (i = 0; i < 3; ++i)
	  { if (p[i] < frm->ori[i])
	      do
		p[i] += frm->grid[i];
	      while (p[i] < frm->ori[i]);
	    else if (p[i] >= (frm->ori[i] + frm->ext[i]))
	      do
		p[i] -= frm->grid[i];
	      while (p[i] >= (frm->ori[i] + frm->ext[i]));}
	  if (VECINMAP(p, frm))
	    success = TRUE;
	  else
	  { tf = ctrans + (++itf % ntrans);
	    unit_oper = ((tf->rotate[0] > 0.9999) && 
			 (tf->rotate[4] > 0.9999) && 
			 (tf->rotate[8] > 0.9999) && 
			 (ABS(tf->translate[0]) < 0.0001) && 
			 (ABS(tf->translate[1]) < 0.0001) && 
			 (ABS(tf->translate[2]) < 0.0001));}}
	if (success)
	  to->map[t0 - to->ori[0]][t1 - to->ori[1]][t2 - to->ori[2]] =
	    frm->map[p[0] - frm->ori[0]][p[1] - frm->ori[1]][p[2] - frm->ori[2]];
	else
	{ printf("Some density could not be retrieved in ExtractMap\n");
	  printf("%d %d %d : (%d %d) (%d %d) (%d %d) \n",
		 t0, t1, t2, frm->ori[0], frm->ext[0],
		 frm->ori[1], frm->ext[1], frm->ori[2], frm->ext[2]); 
	  ccperror(1,"*** Program Terminating ***");}}
  free(ctrans);}



/******************************************************************************/

#if defined (PROTOTYPE)
  void MapGridFill(MAP *frm, MAP *to, MASK *cumm, MASK *mask)
#else
  void MapGridFill(frm, to, cumm, mask)
  MAP *frm, *to;
  MASK *cumm, *mask;
#endif
/* Density values from map 'frm' are added to those of map 'to', using the
   transformations defined in 'cumm->trans' for mapping 'frm' on 'to'. Only
   those points which are set in 'mask' will be added. If the corresponding
   point in 'cumm->msk' is not set, the value from 'frm' replaces the value
   in 'to'; if the point is set in 'cumm->msk', the value from 'frm' is
   added. In either case, 1 is added to the corresponding point in 
   'cumm->msk'. Use this routine only in combination with crystallographic
   symmetry operators: it is assumed that the symmetry operators exactly map
   gridpoints on other gridpoints.  Both 'cumm' and 'frm' should have
   the same ori & ext. 
   Use this routine if 'to' is larger than 'frm' and not all the gridpoints
   of 'to' can be retrieved from 'frm'. Otherwise ExtractMap might be an
   alternative. */

{ TRANSFORM *ctrans, *tf;
  int i, f0, f1, f2, t[3], itf, a, b, c;
  double *m, tmp[3];
#if !defined (PROTOTYPE)
  void TransAToGrid();
#endif

  ctrans = (TRANSFORM *) malloc(cumm->ntrans * sizeof(TRANSFORM));
  for (i = 0; i < cumm->ntrans; ++i)
    TransAToGrid(cumm->trans + i, ctrans + i, frm->cell, frm->grid);
  tf = ctrans + (itf = 0);
  for (f0 = frm->ori[0]; f0 < (frm->ori[0] + frm->ext[0]); ++f0)
    for (f1 = frm->ori[1]; f1 < (frm->ori[1] + frm->ext[1]); ++f1)
      for (f2 = frm->ori[2]; f2 < (frm->ori[2] + frm->ext[2]); ++f2)
	if (mask->msk[f0 - frm->ori[0]][f1 - frm->ori[1]][f2 - frm->ori[2]] != 0)
	  for (tf = ctrans + (itf = 0); itf < cumm->ntrans; tf = ctrans + ++itf)
	  { m = tf->rotate;
	    tmp[0] = m[0] * (float) f0 + m[3] * (float) f1 + m[6] * (float) f2;
	    tmp[1] = m[1] * (float) f0 + m[4] * (float) f1 + m[7] * (float) f2;
	    tmp[2] = m[2] * (float) f0 + m[5] * (float) f1 + m[8] * (float) f2;
	    for (i = 0; i < 3; ++i)
	    { t[i] = NINT(tmp[i] + tf->translate[i]);
	      while (t[i] > to->ori[i])
		t[i] -= to->grid[i];
	      while (t[i] < to->ori[i])
		t[i] += to->grid[i];}
	    for (a = t[0] - to->ori[0]; a < to->ext[0]; a += to->grid[0])
	      for (b = t[1] - to->ori[1]; b < to->ext[1]; b += to->grid[1])
		for (c = t[2] - to->ori[2]; c < to->ext[2]; c += to->grid[2])
		{ if (++cumm->msk[a][b][c] == 1)
		  { ++cumm->nset;
		    to->map[a][b][c] = frm->map[f0 - frm->ori[0]]
		                               [f1 - frm->ori[1]]
					       [f2 - frm->ori[2]];}
		  else
		    to->map[a][b][c] += frm->map[f0 - frm->ori[0]]
		                                [f1 - frm->ori[1]]
					        [f2 - frm->ori[2]];}}
  free(ctrans);}



/******************************************************************************/

#if defined (PROTOTYPE)
  float SplInt(float ***map, double *v)
#else
  float SplInt(map, v)
  float ***map;
  double *v;
#endif
/* 3D Spline interpolation of an electron density map at the position defined
   by 'v' */

{ float plane[4][4], line[4], *vec, v0, v1, v2, a, b, c, d, p0, p2, p3;
  float sqrv1, cubv1; 
  float sqrv2, cubv2; 
  int i, j, x, y, z;

  v0 = v[0] - (x = SMINT(v[0]) - 1) - 1;
  v1 = v[1] - (y = SMINT(v[1]) - 1) - 1;
  v2 = v[2] - (z = SMINT(v[2]) - 1) - 1;
  cubv1 = v1 * (sqrv1 = v1 * v1);
  cubv2 = v2 * (sqrv2 = v2 * v2);

  /* Interpolate, keeping z constant... */
  for (i = 0; i < 4; ++i, ++x)
    for (j = 0; j < 4; ++j)
    { vec = map[x][y + j] + z;
      p0 = vec[0]; p2 = vec[2]; p3 = vec[3];
      a = vec[1];
      c = ((p2 + p0) / 2) - a;
      d = (p3 + 2*p0 -3*a -6*c) / 6;
      b = p2 - a - c - d;
      plane[i][j] = a + b*v2 + c*sqrv2 + d*cubv2;}
  
  /* Interpolate the resulting plane, keeping y constant... */
  for (i = 0; i < 4; ++i)
  { vec = plane[i];
    p0 = vec[0]; p2 = vec[2]; p3 = vec[3];
    a = vec[1];
    c = ((p2 + p0) / 2) - a;
    d = (p3 + 2*p0 -3*a -6*c) / 6;
    b = p2 - a - c - d;
    line[i] = a + b*v1 + c*sqrv1 + d*cubv1;}

  /* Interpolate the resulting line, keeping x constant! */
  p0 = line[0]; p2 = line[2]; p3 = line[3];
  a = line[1];
  c = ((p2 + p0) / 2) - a;
  d = (p3 + 2*p0 -3*a -6*c) / 6;
  b = p2 - a - c - d;
  return(a + b*v0 + c*v0*v0 + d*v0*v0*v0);}



/******************************************************************************/

#if defined (PROTOTYPE)
  float LinInt(float ***map, double *v)
#else
  float  LinInt(map, v)
  float ***map;
  double *v;
#endif
/* Linear interpolation of an electron density map at the position defined
   by 'v' */

{ float v0, v1, v2;
  int x, y, z;

  v0 = v[0] - (x = SMINT(v[0]));
  v1 = v[1] - (y = SMINT(v[1]));
  v2 = v[2] - (z = SMINT(v[2]));
  return((1 - v0) * (1 - v1) * (1 - v2) * map[x  ][y  ][z  ] +
	 (1 - v0) * (1 - v1) * (    v2) * map[x  ][y  ][z+1] +
	 (1 - v0) * (    v1) * (1 - v2) * map[x  ][y+1][z  ] +
	 (1 - v0) * (    v1) * (    v2) * map[x  ][y+1][z+1] +
	 (    v0) * (1 - v1) * (1 - v2) * map[x+1][y  ][z  ] +
	 (    v0) * (1 - v1) * (    v2) * map[x+1][y  ][z+1] +
	 (    v0) * (    v1) * (1 - v2) * map[x+1][y+1][z  ] +
	 (    v0) * (    v1) * (    v2) * map[x+1][y+1][z+1]);}



/******************************************************************************/

#if defined (PROTOTYPE)
  MAP *MapFill(MASK *mask, MAP *map, TRANSFORM *trans, TRANSFORM *csym, int nsym)
#else
  MAP *MapFill(mask, map, trans, csym, nsym)
  MASK *mask; 
  MAP *map; 
  TRANSFORM *trans, *csym; 
  int nsym;
#endif
/* Fills a 'submap' with density taken from 'map' using the transformation
   defined by 'trans'. The crystallographic transformations are passed in
   'csym', the number of crystallographic transformations is passed in 
   'nsym'. The original map should contain all the density required. The
   transformation defines which density the submask would contain if it
   were to be moved into the original map. The submap has the same origin &
   extent as the mask, and only gridpoints set in the mask are filled with
   density. */

{ MAP *submap, *frm;
  int i, i0, i1, i2, maxv[3], ori[3], ext[3];
  double v[3];
#if !defined (PROTOTYPE)
  MAP *SubMap();
  void ExtractMap();
  void TransVector();
  void FreeMap();
#endif

  /* Check if this by any chance 'trans' is the unity operator... */
  if ((trans->rotate[0] > 0.9999) && 
      (trans->rotate[4] > 0.9999) && 
      (trans->rotate[8] > 0.9999) && 
      (ABS(trans->translate[0]) < 0.0001) && 
      (ABS(trans->translate[1]) < 0.0001) && 
      (ABS(trans->translate[2]) < 0.0001))
  { submap = SubMap(map, mask->ori, mask->ext);
    ExtractMap(map, submap, csym, nsym);
    return(submap);}

  /* Extract the bit of the map from which the data will be taken...*/
  for (i = 0; i < 3; ++i)
    maxv[i] = -(ori[i] = 99999);
  for (i0 = 0; i0 < 2; ++i0)
    for(i1 = 0; i1 < 2; ++i1)
      for (i2 = 0; i2 < 2; ++i2)
      { v[0] = i0 * mask->ext[0] + mask->ori[0];
	v[1] = i1 * mask->ext[1] + mask->ori[1];
	v[2] = i2 * mask->ext[2] + mask->ori[2];
	TransVector(v, trans, mask->cell, mask->grid);
	for (i = 0; i < 3; ++i)
	{ maxv[i] = MAX(maxv[i], (int) ceil(v[i]));
	  ori[i] = MIN(ori[i], (int) floor(v[i]));}}
  for (i = 0; i < 3; ++i)
  { ori[i] -= 2;
    maxv[i] += 2;
    ext[i] = maxv[i] - ori[i];}
  frm = SubMap(map, ori, ext);
  ExtractMap(map, frm, csym, nsym);

  /* Now move the data into the submap. */
  submap = SubMap(map, mask->ori, mask->ext);
  for (i0 = 0; i0 < mask->ext[0]; ++i0)
    for (i1 = 0; i1 < mask->ext[1]; ++i1)
      for (i2 = 0; i2 < mask->ext[2]; ++i2)
	if (mask->msk[i0][i1][i2] > 0)
	{ v[0] = i0 + mask->ori[0];
	  v[1] = i1 + mask->ori[1];
	  v[2] = i2 + mask->ori[2];
	  TransVector(v, trans, mask->cell, mask->grid);
	  v[0] -= frm->ori[0];
	  v[1] -= frm->ori[1];
	  v[2] -= frm->ori[2];
	  submap->map[i0][i1][i2] = SplInt(frm->map, v);}
  FreeMap(frm);
  return(submap);}



/*****************************************************************************

#if defined (PROTOTYPE)
  float MapGrid(MAP *map, int *pnt, TRANSFORM *csym, int nsym)
#else
  float MapGrid(map, pnt, csym, nsym)
  MAP *map;
  int *pnt, nsym;
  TRANSFORM *csym;
#endif

 Returns the electron density of 'map' at the gridpoint defined by 'pnt' or
   any of its symmetry mates defined by 'csym' (the crystallographic symmetry
   operators, nsym is the number of crystallographic symmetry operators). 

{ int i, j, sp[3];
  double v[3];
#if !defined (PROTOTYPE)
  void TransVector();
#endif

  for (i = 0; i < nsym; ++i)
  { for (j = 0; j < 3; ++j)
      v[j] = pnt[j];
    TransVector(v, csym + i, map->cell, map->grid);
    for (j = 0; j < 3; ++j)
    { sp[j] = SMINT(v[j] + 0.5);
      while (sp[j] < map->ori[j])
        sp[j] += map->grid[j];
      while ((map->ori[j] + map->ext[j]) <= sp[j])
        sp[j] -= map->grid[j];}
    if (VECINMASK(sp, map))
    { for (j = 0; j < 3; ++j)
	sp[j] -= map->ori[j];
      return(map->map[sp[0]][sp[1]][sp[2]]);}}
  printf("%d %d %d doesn't fall in (%d %d) (%d %d) (%d %d)\n",
	 pnt[0], pnt[1], pnt[2], map->ori[0], map->ext[0], map->ori[1],
	 map->ext[1], map->ori[2], map->ext[2]);
  return(0);}


   
******************************************************************************/

#if defined (PROTOTYPE)
  float SigmaMap(MAP *map, MASK *mask, float *mean)
#else
   float SigmaMap(map, mask, mean)
   MAP *map;
   MASK *mask;
   float *mean;
#endif
/* Returns the standard deviation of the fraction of the map defined by 'mask'.
   If ('mask' == NULL), the s.d. of the whole map is returned. The mean of the
   map is returned in 'mean'. */

{ float *mp = map->map[0][0];
  Byte *msk;
  double sd = 0;
  float dif, nset;
  int i;
#if !defined (PROTOTYPE)
  float MeanMap();
#endif

  *mean = MeanMap(map, mask);
  if (mask == NULL)
  { nset = map->ext[0] * map->ext[1] * map->ext[2];
    for (i = 0; i < nset; ++i, ++mp)
    { dif = *mean - *mp;
      sd += SQR(dif);}}
  else
  { msk = mask->msk[0][0];
    nset = mask->nset;
    for (i = map->ext[0] * map->ext[1] * map->ext[2] - 1; i >= 0; --i, ++msk, ++mp)
      if (*msk == 1)
      { dif = *mean - *mp;
	sd += SQR(dif);}}
  if (nset == 1)
  { printf("Cannot determine standard deviation: only one point set in %s",
	   mask->infile);
    ccperror(1,"*** Program Terminating ***");}
  return(sqrt(sd/(nset - 1)));}



/******************************************************************************/

#if defined (PROTOTYPE)
  float SigmaPosMap(MAP *map, MASK *mask, float *mean)
#else
  float SigmaPosMap(map, mask, mean)
  MAP *map;
  MASK *mask; 
  float *mean;
#endif
/* Returns the standard deviation of the fraction of the positive density of
   the map within 'mask'.
   If ('mask' == NULL), the s.d. of the whole map is returned. The mean of the
   map is returned in 'mean'. */

{ float *mp = map->map[0][0];
  Byte *msk;
  double sd = 0;
  float dif, nset;
  int i;
#if !defined (PROTOTYPE)
  float MeanMap();
#endif

  *mean = MeanMap(map, mask);
  if (mask == NULL)
  { nset = 0;
    for (i = map->ext[0] * map->ext[1] * map->ext[2] - 1; i >= 0; --i, ++msk, ++mp)
      if (*mp > 0)
      { dif = *mean - *mp;
	sd += SQR(dif);
        ++nset;}}
  else
  { msk = mask->msk[0][0];
    nset = 0;
    for (i = map->ext[0] * map->ext[1] * map->ext[2] - 1; i >= 0; --i, ++msk, ++mp)
      if ((*msk == 1) && (*mp > 0))
      { dif = *mean - *mp;
	sd += SQR(dif);
        ++nset;}}
  if (nset <= 1)
  { printf("Cannot determine standard deviation: only one point set in %s",
	   mask->infile);
    ccperror(1,"*** Program Terminating ***");}
  return(sqrt(sd/(nset - 1)));}



/******************************************************************************/

#if defined (PROTOTYPE)
  float SkewnessMap(MAP *map, MASK *mask, float mean, float sd)
#else
  float SkewnessMap(map, mask, mean, sd)
  MAP *map;
  MASK *mask;
  float mean, sd;
#endif
/* Returns the skewness of the fraction of the map defined by 'mask'.
   If ('mask' == NULL), the skew of the whole map is returned. The mean and 
   standard deviation should be passed in 'mean' and 'sd'. */

{ float *mp = map->map[0][0];
  Byte *msk;
  double skew = 0;
  float dif, nset;
  int i;

  if (mask == NULL)
  { nset = map->ext[0] * map->ext[1] * map->ext[2];
    for (i = 0; i < nset; ++i, ++mp)
    { dif = (*mp - mean) / sd;
      skew += CUB(dif);}}
  else
  { msk = mask->msk[0][0];
    nset = mask->nset;
    for (i = map->ext[0] * map->ext[1] * map->ext[2] - 1; i >= 0; --i, ++msk, ++mp)
      if (*msk > 0)
      { dif = (*mp - mean) / sd;
	skew += CUB(dif);}}
  if (nset == 1)
  { printf("Cannot determine standard deviation: only one point set in %s",
	   mask->infile);
    ccperror(1,"*** Program Terminating ***");}
  return((float) (skew / nset));}



/******************************************************************************/

#if defined (PROTOTYPE)
  float MeanMap(MAP *map, MASK *mask)
#else
  float MeanMap(map, mask)
  MAP *map;
  MASK *mask;
#endif
/* Returns the mean of the fraction of the map defined by 'mask'. If 
   ('mask' == NULL), the mean of the whole map is returned. */

{ float *mp = map->map[0][0];
  Byte *msk;
  double tot = 0, nset;
  int i;

  if (mask == NULL)
  { nset = map->ext[0] * map->ext[1] * map->ext[2];
    for (i = 0; i < nset; ++i, ++mp)
      tot += *mp;}
  else
  { msk = mask->msk[0][0];
    nset = mask->nset;
    for (i = map->ext[0] * map->ext[1] * map->ext[2] - 1; i >= 0; --i, ++msk, ++mp)
      if (*msk > 0)
	tot += *mp;}
  if (nset == 0)
  { printf("Division by zero: no points set in %s", mask->infile);
    ccperror(1,"*** Program Terminating ***");}
  return(tot / nset);}



/******************************************************************************/

#if defined (PROTOTYPE)
  void AverageMap(MAP *map, MASK *mask, TRANSFORM *csym, int nsym, float sd)
#else
  void AverageMap(map, mask, csym, nsym, sd)
  MAP map;
  MASK *mask;
  TRANSFORM *csym; 
  int nsym; 
  float sd;
#endif
/* Averages the area within the map defined by 'mask' and the transformations
   associated with 'mask'. The averaged submap is written to disk as mask->infile,
   where the extension is replaced by '.map'. The crystallographic symmetry
   operators are defined by 'csym', and there are 'nsym' crystallographic
   symmetry operators. If (sd >= 0), 'sd' is an indication of the standard
   deviation of the noise of the map, and therefore the averaging can (and will)
   be weighted according to a normal probability distribution with an s.d. of
   sqrt(2)*sd (because the two observations which are compared have a s.d.). This
   probability distribution is normalised so that if both observations are
   identical, the weight for the observation to be added in, is 1. */

{ int i, j;
  MAP *avr, *tst, *weightmap, *old;
  float *avrdata, *tstdata, *weight, *olddata, mean;
  char mapname[256], *dot;
  double rms, dif, wgt, oldmean, tstmean, s1, s2, s3;
  Byte *msk;
#if !defined (PROTOTYPE)
  double *NormalDistr();
  MAP *MapFill();
  float MeanMap();
  MAP *SubMap();
  void WriteMap();
  void FreeMap();
#endif
  double *prob = NormalDistr();

  if ((mask->ntrans > 0) && (strcmp(mask->title, "insert") != 0))
  { strcpy(mapname, mask->infile);
    if ((dot = strrchr(mapname, '.')) != NULL)
      dot[0] = '\0';
    strcat(mapname, ".map");
    if (mask->ntrans == nsym)
    { printf("\nExtracting %s\n", mask->infile);
      avr = MapFill(mask, map, mask->trans, csym, nsym);
      printf("  Standard deviation extracted mask:    %f\n",
	     SigmaMap(avr, mask, &mean));
      WriteMap(avr, mapname);
      FreeMap(avr);}
    else
    { printf("\nAveraging %s\n", mask->infile);
      avr = MapFill(mask, map, mask->trans, csym, nsym);
      avrdata = avr->map[0][0];
      old = MapFill(mask, map, mask->trans, csym, nsym);
      oldmean = MeanMap(old, mask);
      olddata = old->map[0][0];
      msk = mask->msk[0][0];
      weightmap = SubMap(avr, avr->ori, avr->ext);
      weight = weightmap->map[0][0];
      for (i = (avr->ext[0] * avr->ext[1] * avr->ext[2]) - 1; i >= 0; --i)
	weight[i] = 1;
      sd *= 1.4142136;
      for (i = nsym, rms = 0; i < mask->ntrans; i += nsym)
      { tst = MapFill(mask, map, mask->trans + i, csym, nsym);
	tstmean = MeanMap(tst, mask);
	tstdata = tst->map[0][0];
	s1 = s2 = s3 = 0;
	for (j = (avr->ext[0] * avr->ext[1] * avr->ext[2]) - 1; j >= 0; --j)
	  if (msk[j] != 0)
	  { s1 += (olddata[j] - oldmean) * (tstdata[j] - tstmean);
	    s2 += SQR(olddata[j] - oldmean);
	    s3 += SQR(tstdata[j] - tstmean);
	    dif = tstdata[j] - olddata[j];
	    if (sd > 0)
	      wgt = prob[NINT(MIN(3999, ABS(dif / sd) * 1000))];
	    else
	      wgt = 1;
	    rms += SQR(dif);
	    weight[j] += (float) wgt;
	    avrdata[j] += wgt * tstdata[j];}
	printf("    Correlation coefficient operator %d : %f\n", i / nsym,
	                                            s1 / (sqrt(s2) * sqrt(s3)));
	FreeMap(tst);}
      for (j = (avr->ext[0] * avr->ext[1] * avr->ext[2]) - 1; j >= 0; --j)
	if (msk[j] != 0)
	  avrdata[j] /= weight[j];
      printf("  RMS deviation of differences:        %f\n",
	     sqrt(rms / (mask->nset * ((mask->ntrans / nsym) - 1))));
      printf("  Standard deviation averaged mask:    %f\n",
	     SigmaMap(avr, mask, &mean));
      if (sd > 0)
      { sd = SigmaMap(weightmap, mask, &mean);
        printf("  mean weight:                         %f\n", mean);
	printf("  rms weight:                          %f\n", sd);}

      WriteMap(avr, mapname);
      FreeMap(old);
      FreeMap(weightmap);
      FreeMap(avr);}}}



/******************************************************************************/

#if defined (PROTOTYPE)
  void ClearMap(MAP *map)
#else
  void  ClearMap(map)
  MAP *map;
#endif
/* Resets all gridpoints in 'map' */

{ int i;
  float *data = map->map[0][0];

  for (i = (map->ext[0] * map->ext[1] * map->ext[2]) - 1; i >= 0; --i)
    data[i] = 0;}



/******************************************************************************/

#if defined (PROTOTYPE)
  MAP *CombMaps(MAP *map, MASK **mask, int nmask)
#else
  MAP *CombMaps(map, mask, nmask)
  MAP *map;
  MASK **mask;
  int nmask;
#endif
/* Combines maps whose names will be derived from the names of the masks in the 
   array 'mask[]'The density of 'map' is changed. The number of masks is passed
   in 'nmask'. The first mask of the array 'mask[]' defines the crystallographic
   symmetry operators. Returns a pointer to 'map'.*/

{ int i, j, npnts;
  MAP *submap;
  Byte *msk;
  char *dot, filename[256];
  float *mp, mapmean, submapmean;
#if !defined (PROTOTYPE)
  void ClearMask();
  float MeanMap();
  MAP *ReadMap();
  void MapGridFill();
  void FreeMap();
#endif

  ClearMask(mask[0]);
  mapmean = MeanMap(map, NULL);
  for (i = 1; i < nmask; ++i)
  { if ((strcmp(mask[i]->title, "insert") != 0) ||
	(strcmp(mask[i]->title, "monitor") != 0) ||
	(strcmp(mask[i]->title, "weight") != 0))
    { strcpy(filename, mask[i]->infile);
      if ((dot = strrchr(filename, '.')) != NULL)
	dot[0] = '\0';
      strcat(filename, ".map");
      submap = ReadMap(filename);
      MapGridFill(submap, map, mask[0], mask[i]);
      FreeMap(submap);}}
  mp = map->map[0][0];
  msk = mask[0]->msk[0][0];
  npnts = map->ext[0] * map->ext[1] * map->ext[2];
  for (i = 0; i < npnts; ++i)
  { if (msk[i] > 1)
      mp[i] = mp[i] / (float) msk[i];
    msk[i] = 0;}

  for (i = 1; i < nmask; ++i)
  { if ((strcmp(mask[i]->title, "insert") == 0) ||
	(strcmp(mask[i]->title, "monitor") == 0) ||
	(strcmp(mask[i]->title, "weight") == 0))
    { strcpy(filename, mask[i]->infile);
      if ((dot = strrchr(filename, '.')) != NULL)
	dot[0] = '\0';
      strcat(filename, ".map");
      submap = ReadMap(filename);
      if (strcmp(mask[i]->title, "insert") == 0)
      { submapmean = MeanMap(submap, mask[i]);
	npnts = submap->ext[0] * submap->ext[1] * submap->ext[2];
	for (j = 0; j < npnts; ++j)
	  if (mask[i]->msk[0][0][j] >= 1)
	    submap->map[0][0][j] += mapmean - submapmean;}
      MapGridFill(submap, map, mask[0], mask[i]);
      FreeMap(submap);}}

  mp = map->map[0][0];
  msk = mask[0]->msk[0][0];
  npnts = map->ext[0] * map->ext[1] * map->ext[2];
  for (i = 0; i < npnts; ++i)
    if (msk[i] > 1)
      mp[i] = mp[i] / (float) msk[i];
  return(map);}


  

/******************************************************************************/

#if defined (PROTOTYPE)
  MAP *FlatMap(MAP *map, float solv, float radius, float mintrunc, float maxtrunc,
	     MASK **mask, int nmask, float sd, float ptos, float solvmean,
	     float solvmul, char *rmsmapname, float protmul, char *mapmin,
	     float mapminscale, float sdred, char *solmaskname, float *validate,
             BOOLEAN extrude, BOOLEAN keepmean, float sharp) 
#else
  MAP *FlatMap(map, solv, radius, mintrunc, maxtrunc, mask, nmask, sd, ptos, 
             solvmean, solvmul, rmsmapname, protmul, mapmin, mapminscale, sdred, 
             solmaskname, validate, extrude, keepmean, sharp)
  MAP *map;
  float solv, radius, mintrunc, maxtrunc, sd, ptos, solvmean, solvmul, protmul;
  MASK **mask; 
  int nmask;
  char *rmsmapname, *mapmin, *solmaskname;
  float mapminscale, sdred, *validate, sharp;
  BOOLEAN extrude, keepmean;
#endif
/* Flattens 'map', uses a solvent content as defined by 'solv', an integration
   radius for determining the local rms of 'radius' (if 'radius' is positive),
   and truncates a fraction 'mintrunc' of the lowest density values in the 
   protein, and also density values which are higher than a 'maxtrunc' fraction. 
   The array 'mask[]' can define patches which are not to be flattened (masks with
   the title 'weight' or 'monitor') or which are to be flattened (masks with the
   title 'solvent'). The solvent is set to the proper level (according to 
   'ptos', the ratio between the mean protein density and the mean solvent
   density), and attenuated by the factor 'solvmul' (which can be negative!!)
   If there exists a mapfile called `mapmin`, the protein density will be
   multiplied by `protmul`, and at each gridpoint within the protein mask, the
   corresponding gridpoint from mapmin, after scaling by `mapminscale`, will be
   subtracted. If the fraction 'sdred' > 0, the deviation from the mean of the
   protein chain will be reduced by this fraction and the solvent mask will 
   include any excursions into the solvent connected to high density within
   the protein. If 'solvmul' is larger than 900, 1000 will be subtracted, and the
   rms of the solvent will be scaled relative to the rms of the protein.
   The local rms map will be written to disk as 'rmsmapname' if a valid name
   was specified. */

{ int i, npnts = map->ext[0] * map->ext[1] * map->ext[2];
  int nprt, trhigh, trlow;
  MASK *solmsk;
  Byte *msk;
  float *mp, *sm, maxprotdens, minprotdens, solvsd, solvdens, pprotsd;
  float protsd, protdens, mapminmean;
  float mapmean, F000, solvadd, newprotsd;
  float minmean, maxmean;
  double s1, s2, s3;
  FILE *minfile;
  MAP *minmap;
#if !defined (PROTOTYPE)
  MASK *GetSolvMask();
  MASK *NotMask();
  float MinMap();
  float SigmaMap();
  float SigmaPosMap();
  void ValidateMap();
  HISTOGRAM *HistMap();
  HISTOGRAM *DumpHist();
  void FreeHist();
  MASK *ProtSausage();
  MASK *SetMask();
  void WriteMask();
  void FreeMap();
  void FreeMask();
  MAP *ReadMap();
  float MeanMap();
#endif

  solmsk = GetSolvMask(map, mask, nmask, radius, rmsmapname, solvmean);
  solmsk = NotMask(solmsk);
  minprotdens = MinMap(map, solmsk, mintrunc);
  maxprotdens = MinMap(map, solmsk, maxtrunc);
  protsd = SigmaMap(map, solmsk, &protdens);
  pprotsd = SigmaPosMap(map, solmsk, &protdens);
  if (validate[0] != 0)
    ValidateMap(map, mask, solmsk, validate);
/*  MapHistoMax(map, solmsk, mask[0]->trans, mask[0]->ntrans);*/
  if (verbose)
  { printf("\n\nHistogram of the protein density:\n");
    if (verbose == TRUE)
      FreeHist(DumpHist(HistMap(map, solmsk, 100)));}
  solmsk = NotMask(solmsk);
  if (solmsk->nset > 0)
  { solvsd = SigmaMap(map, solmsk, &solvdens);
    if (mintrunc < -1)
    { mapmean = (1 - map->solv) * protdens + map->solv * solvdens;
      F000 = (solvdens - mapmean) * (map->solv + ptos - map->solv * ptos) / 
	     ((1-map->solv) * (1 - ptos));
      minprotdens = protdens - F000;
      printf("\nCalculated F000: %f\n", F000);
      printf("  (assuming a ratio of the protein to solvent density of %f)\n",
	     ptos);}
    printf("\n\nFlattening %f percent of map\n", 100 * (float)solmsk->nset / npnts);
    printf("  sd of solvent before flattening: %f\n", solvsd);
    printf("  mean density of solvent:         %f\n", solvdens);
    printf("  sd of protein:                   %f\n", protsd);
    printf("  mean density of protein:         %f\n", protdens);
    printf("  ratio:                           %f\n\n", solvsd / protsd);
    if (sd > 0)
      printf("Flatness of patch, relative to protein: %f\n", sd / protsd);
    if (sd > 0)
      printf("Flatness of patch, relative to positive protein: %f\n",
	     sd / pprotsd);}
  if (sdred >= 0)
  { solmsk = ProtSausage(map, mask, nmask, solmsk, sdred);
    solmsk = NotMask(solmsk);
    protsd = SigmaMap(map, solmsk, &protdens);
    solmsk = NotMask(solmsk);
    if (solmsk->nset > 0)
    { printf("After modifying the protein region:\n");
      solvsd = SigmaMap(map, solmsk, &solvdens);
      printf("  Flattening %f percent of map\n", 100 * (float)solmsk->nset / npnts);
      printf("  sd of solvent before flattening: %f\n", solvsd);
      printf("  mean density of solvent:         %f\n", solvdens);
      printf("  sd of protein:                   %f\n", protsd);
      printf("  mean density of protein:         %f\n", protdens);
      printf("  ratio:                           %f\n", solvsd / protsd);
      if (sd > 0)
	printf("  Flatness of patch, relative to protein: %f\n", sd / protsd);}}
  for (i = 1; i < nmask; ++i)
    if ((strcmp(mask[i]->title, "monitor") == 0) ||
	(strcmp(mask[i]->title, "weight") == 0))
      solmsk = SetMask(mask[i], solmsk, 2, mask[0]->trans, mask[0]->ntrans);
  if (solmaskname[0] != '\0')
  { strcpy(solmsk->outfile, solmaskname);
    WriteMask(solmsk);}
  msk = solmsk->msk[0][0];
  mp = map->map[0][0];
  if (protmul != 1)
  { for (i = 0; i < npnts; i++)
      if (msk[i] == 0)
	mp[i] *= protmul;
    printf("Scaling protein region by %f\n", protmul);} 
  if (mapmin[0] != '\0')
    { minmap = ReadMap(mapmin);
      if (mapminscale > (HUGE / 10))
      { printf("Autoscaling %s\n", mapmin);
	solmsk = NotMask(solmsk);
	mapminscale = solvsd / SigmaMap(minmap, solmsk, &mapminmean);
	solmsk = NotMask(solmsk);}
      printf("Subtracting %s scaled by %f from protein region\n", mapmin, mapminscale);
      sm = minmap->map[0][0];
      for (i = 0, nprt = 0, s1 = 0, s2 = 0, s3 = 0; i < npnts; ++i)
	if (msk[i] == 0)
	{ s1 += (mp[i]) * (sm[i]);
	  s2 += SQR(mp[i]);
	  s3 += SQR(sm[i]);
	  mp[i] -= sm[i] * mapminscale;
	  ++nprt;}
      printf("    sd of %s: %f\n", mapmin, sqrt(s3 / nprt));
      printf("    Correlation coefficient between the maps: %f\n",
	     s1 / (sqrt(s2) * sqrt(s3)));
      FreeMap(minmap);}

  for (i = 0, trhigh = trlow = minmean = maxmean = 0; i < npnts; ++i)
    if (msk[i] == 0)
    { if (mp[i] >= maxprotdens)
      { maxmean += mp[i];
        ++trhigh;}
      else if (mp[i] <= minprotdens) 
      { minmean += mp[i];
        ++trlow;}}
  if (keepmean == TRUE)
  { minmean /= trlow;
    maxmean /= trhigh;}
  else
  { minmean = minprotdens;
    maxmean = maxprotdens;}
  for (i = 0, solvadd = maxprotdens; i < npnts; ++i)
    if (msk[i] == 0)
    { if (mp[i] >= maxprotdens)
        mp[i] = (mp[i] - maxmean) * sharp + maxmean;
      else if (mp[i] <= minprotdens) 
      { mp[i] = minmean - (minmean - mp[i]) * sharp;
        solvadd = MIN(mp[i], solvadd);}}
  minprotdens = solvadd;
 
  printf("Truncating minimum %f percent of protein\n", 
	 ((float) trlow / (float) (npnts - solmsk->nset)) * 100.0);
  printf("Truncating maximum %f percent of protein\n", 
	 ((float) trhigh / (float) (npnts - solmsk->nset)) * 100.0);
  solmsk = NotMask(solmsk);
  if (solvmul > 900)
  { newprotsd = SigmaMap(map, solmsk, &protdens);
    solvmul = (solvmul - 1000) * SQR(newprotsd / protsd);}
  else
    protdens = MeanMap(map, solmsk);
  for (i = 0; i < npnts; ++i)
    if (msk[i] == 0)
      mp[i] = solvdens + (mp[i] - solvdens) * solvmul;

  printf("Modified variation in solvent density around the mean by %f\n",
	 solvmul);
  if (extrude == TRUE)
  { solvadd = (((protdens - minprotdens) / ptos) + minprotdens) - solvdens;
    printf("\nAdding %f to the solvent\n", solvadd);
    printf("\nnew solvdens: %f\n\n", (1 - map->solv) * 
	   (solvdens + solvadd - protdens));
    for (i = 0; i < npnts; ++i)
      if (msk[i] == 0)
        mp[i] += solvadd;}
  else
  { solvadd = 0;
    printf("\nnew solvdens: %f\n\n", solvdens);}
  FreeMask(solmsk);
  return(map);}




/******************************************************************************/
    
#if defined (PROTOTYPE)
  int AddGrid(MAP *map, int j0, int j1, int j2, MAP *submap, MASK *mask,
	    TRANSFORM *trans, int ntrans)
#else
  int AddGrid(map, j0, j1, j2, submap, mask, trans, ntrans)
  MAP *map, *submap;
  int j0, j1, j2, ntrans;
  MASK *mask;
  TRANSFORM *trans;
#endif
/* Checks if the gridpoint [j0+map->ori[0]] [j1+map->ori[1]] [j2+map->ori[2]]
   or one of its symmetry mates as defined by the symmetries in the array
   'trans[]', is present in 'submap', and is set in 'mask'. If it is, it sets
   the electron density of this gridpoint in 'map' to the value found in
   'submap', and returns a '1'. Otherwise it returns a '0'. */

{ int i, j, sp[3];
  double v[3];
#if !defined (PROTOTYPE)
  void TransVector();
#endif

  for (i = 0; i < ntrans; ++i)
  { v[0] = j0;
    v[1] = j1;
    v[2] = j2;
    TransVector(v, trans + i, map->cell, map->grid);
    for (j = 0; j < 3; ++j)
    { sp[j] = SMINT(v[j] + 0.5);
      while (sp[j] < submap->ori[j])
        sp[j] += submap->grid[j];
      while ((submap->ori[j] + submap->ext[j]) <= sp[j])
        sp[j] -= submap->grid[j];}
    if (VECINMASK(sp, submap))
    { for (j = 0; j < 3; ++j)
	sp[j] -= submap->ori[j];
      if (mask->msk[sp[0]][sp[1]][sp[2]] > 0)
      { map->map[j0 + map->ori[0]] [j1 + map->ori[1]] [j2 + map->ori[2]] =
	  submap->map[sp[0]][sp[1]][sp[2]];
	return(1);}
      else
	return(0);}}
  return(0);}



/******************************************************************************/

#if defined (PROTOTYPE)
  float GridDist(int *p, int *q, MAP *map)
#else
  float GridDist(p, q, map)
  int *p, *q;
  MAP *map;
#endif
/* Returns the distance in Angstroms between gridpoints p[] & q[]. */

{ double dist = 0, d[3];
  int i;

  for (i = 0; i < 3; ++i)
    d[i] = (float) (p[i] - q[i]) * (map->cell[i] / (float) map->grid[i]);
  dist = SQR(d[0]) + SQR(d[1]) + SQR(d[2])
                               + 2 * d[1] * d[2] * cos((double) RADIAN(map->cell[3]))
			       + 2 * d[0] * d[2] * cos((double) RADIAN(map->cell[4]))
			       + 2 * d[0] * d[1] * cos((double) RADIAN(map->cell[5]));
  return((float) sqrt(dist));}



/******************************************************************************/

#if defined (PROTOTYPE)
  HISTOGRAM *HistMap(MAP *map, MASK *mask, int nclasses)
#else
  HISTOGRAM *HistMap(map, mask, nclasses)
  MAP *map;
  MASK *mask;
  int nclasses;
#endif
/* Histograms map into 'nclasses', if 'mask' != NULL, only the gridpoints set in
   'mask' are histogrammed. */

{ int i, npnts = map->ext[0] * map->ext[1] * map->ext[2];
  HISTOGRAM *hst;
  float *mp;
  Byte *msk;

  if (nclasses <= 0)
  { printf("Cannot histogram a map into %d classes!!\n", nclasses);
    ccperror(1,"*** Program Terminating ***");}
  hst = (HISTOGRAM *) malloc(sizeof(HISTOGRAM));
  hst->histo = (int *) calloc(nclasses, sizeof(int));
  hst->nclas = nclasses;
  mp = map->map[0][0];
  if (mask != NULL)
  { msk = mask->msk[0][0];
    for (i = 0; i < npnts; ++i)
    { if (msk[i] != 0)
      { hst->max = hst->min = map->map[0][0][0];
	for ( ; i < npnts; ++i)
	{ hst->min = MIN(hst->min, mp[i]);
	  hst->max = MAX(hst->max, mp[i]);}}}
    hst->csiz = (hst->max - hst->min) / (nclasses - 1);
    for (i = 0, hst->tot = 0; i < npnts; ++i)
      if (msk[i] != 0)
      { ++hst->histo[SMINT((mp[i] - hst->min) / hst->csiz)];
        ++hst->tot;}}
  else
  { hst->max = hst->min = map->map[0][0][0];
    for (i = 0; i < npnts; ++i)
    { hst->min = MIN(hst->min, mp[i]);
      hst->max = MAX(hst->max, mp[i]);}
    hst->csiz = (hst->max - hst->min) / (nclasses - 1);
    for (i = 0, hst->tot = 0; i < npnts; ++i)
    { ++hst->histo[SMINT((mp[i] - hst->min) / hst->csiz)];
      ++hst->tot;}}
  hst->rms = SigmaMap(map, mask, &hst->mean);
  return(hst);}



/******************************************************************************/

#if defined (PROTOTYPE)
  HISTOGRAM *DumpHist(HISTOGRAM *hst)
#else
  HISTOGRAM *DumpHist(hst)
  HISTOGRAM *hst;
#endif
/* Dumps info on 'hst' & returns 'hst'. */

{ int i, j, max, nstr;
  float subtot;
  
  printf("\nMinimum: %f\n", hst->min);
  printf("Maximum: %f\n", hst->max);
  printf("Mean: %f\n", hst->mean);
  printf("Rms: %f\n", hst->rms);
  printf("Observations: %d\n", hst->tot);
  printf("Classes: %d\n", hst->nclas);
  max = hst->histo[0];
  for (i = 0; i < hst->nclas; ++i)
    max = MAX(max, hst->histo[i]);
  for (i = 0, subtot = 0; i < hst->nclas; ++i)
  { subtot += (float) hst->histo[i];
    printf("\n%3d: %+3.2f %+3.2f (%3.1f)", i,
	   hst->min + i * hst->csiz, hst->min + (i+1) * hst->csiz,
	   100.0 * (subtot / hst->tot));
    nstr = NINT(55 * (hst->histo[i]) / (float) max);
    for (j = 0; j < nstr; ++j)
      printf("*");}
  printf("\n\n");
  fflush(stdout);
  return(hst);}



/******************************************************************************/

#if defined (PROTOTYPE)
  float SafeSolv(MAP *map, float safe, float solvcont, int ncl, int n)
#else
  float SafeSolv(map, safe, solvcont, ncl, n)
  MAP *map; 
  float safe, solvcont;
  int ncl, n;
#endif
/* Returns the maximum local rms which can safely be attributed to solvent,
   which, when used for constructing a mask, will not contain more than a
   'safe' fraction of protein. However, not more that 'solvcont' fraction
   of the map will be flattened. */

{ int i, j, max, totsolv, twighlight, lv, mv;
  HISTOGRAM *hst;
  float saferms;
#if !defined (PROTOTYPE)
  HISTOGRAM *HistMap();
  void FreeHist();
#endif

  /* Find first maximum, & determine total solvent by assuming a symmetrical 
     distribution around the first maximum in the histogram. */

  hst = HistMap(map, NULL, ncl);
  for (mv = 0, i = -n; i <= n; ++i)
    mv += hst->histo[n + i];
  for (i = n, max = n; i < (hst->nclas - n); ++i)
  { for (lv = 0, j = -n; j <= n; ++j)
      lv += hst->histo[i + j];
    if (lv > mv)
    { max = i;
      mv = lv;}}

  for (i = 0, totsolv = hst->histo[max]; i < max; ++i)
    totsolv += hst->histo[i] * 2;

  printf("Estimated solvent content : %f\n", totsolv / (float) hst->tot);

  solvcont -= (float)(totsolv + hst->histo[max]) / (2 * hst->tot);
  for (i = 1, twighlight = 0; (i < max) && (i < hst->nclas); ++i)
  { twighlight += hst->histo[max + i] - hst->histo[max - i];
    solvcont -= (float) hst->histo[max + i] / hst->tot;
    if ((((float) twighlight / hst->tot) > safe) || (solvcont < 0))
    { saferms = hst->min + ((max + i - 1) * hst->csiz);
      FreeHist(hst);
      return(saferms);}}
  saferms = hst->min + 2 * max * hst->csiz;
  FreeHist(hst);
  return(saferms);}
    

    
/******************************************************************************/


#if defined (PROTOTYPE)
  float MinMap(MAP *map, MASK *mask, float fraction)
#else
  float MinMap(map, mask, fraction)
  MAP *map;
  MASK *mask;
  float fraction;
#endif
/* Returns the maximum density-cutoff defining the 'fraction' of gridpoints. If
   'mask != NULL', only the points within 'mask' are considered. */

{ int i, cumm;
  float dens;
  HISTOGRAM *hst;
#if !defined (PROTOTYPE)
  HISTOGRAM *HistMap();
  void FreeHist();
#endif

  hst = HistMap(map, mask, 100000);
  fraction = MAX(MIN(fraction, 1), 0);
  for (i = 0, cumm = 0; cumm < (fraction * hst->tot); ++i)
    cumm += hst->histo[i];
  if (i > 0)
    dens = hst->min + (i - 1) * hst->csiz;
  else
    dens = hst->min - 1;
  FreeHist(hst);
  return(dens);}
    

    
/******************************************************************************/

#if defined (PROTOTYPE)
  void FreeHist(HISTOGRAM *hst)
#else
  void FreeHist(hst)
  HISTOGRAM *hst;
#endif
/* Frees space allocated to 'hst' */

{ free(hst->histo);
  free(hst);}



/******************************************************************************/

#if defined (PROTOTYPE)
  MASK *ContourMap(MAP *map, float val)
#else
  MASK *ContourMap(map, val)
  MAP *map;
  float val;
#endif
/* Creates a mask in which all the gridpoints corresponding to values in 'map'
   which are higher than 'val', are set. */

{ MASK *mask;
  float *mp;
  Byte *msk;
  int i, nset = 0;
#if !defined (PROTOTYPE)
  MASK *MallocMask();
#endif

  mask = MallocMask(map->grid, map->ori, map->ext, map->cell);
  msk = mask->msk[0][0];
  mp = map->map[0][0];
  for (i = map->ext[0] * map->ext[1] * map->ext[2] - 1; i >= 0; --i)
    if (mp[i] > val)
    { msk[i] = 1;
      ++nset;}
  mask->nset = nset;
  return(mask);}




/******************************************************************************/


#if defined (PROTOTYPE)
  MASK *SphericalMask(MAP *map, float radius, int *centre)
#else
  MASK *SphericalMask(map, radius, centre)
  MAP *map;
  float radius;
  int *centre;
#endif
/* Returns a spherical mask with the same grid & cell as the map, centered around
   'centre'. */

{ int ori[3], ext[3], i, p[3], c[3];
  MASK *mask;
#if !defined (PROTOTYPE)
  MASK *MallocMask();
  float GridDist();
  MASK *ShrinkMask();
#endif

  for (i = 0; i < 3; ++i)
  { ext[i] = 1 + 6 * SMINT(((float) map->grid[i]) * radius / map->cell[i]);
    ori[i] = centre[i] - (ext[i] - 1) / 2;
    c[i] = (ext[i] - 1 ) / 2;}
  mask = MallocMask(map->grid, ori, ext, map->cell);
  for (p[0] = 0; p[0] < ext[0]; ++p[0])
    for (p[1] = 0; p[1] < ext[1]; ++p[1])
      for (p[2] = 0; p[2] < ext[2]; ++p[2])
	if (GridDist(p, c, map) <= radius)
	{ mask->msk[p[0]][p[1]][p[2]] = 1;
	  ++mask->nset;}
  return(ShrinkMask(mask));}



/******************************************************************************/

#if defined (PROTOTYPE)
  MASK *MinPatch(MAP *map, TRANSFORM *csym, int nsym, float radius)
#else
  MASK *MinPatch(map, csym, nsym, radius)
  MAP *map;
  TRANSFORM *csym;
  int nsym;
  float radius;
#endif
/* Returns a spherical mask with the specified radius, which is centered on 
   the gridpoint with the lowest local rms (using the same radius). */

{ MAP *rmsmap;
  int npnts, centre[3], i0, i1, i2;
  float min;
  MASK *mask;
#if !defined (PROTOTYPE)
  MAP *VarMap();
  void FreeMap();
  MASK *SphericalMask();
#endif

  npnts = map->ext[0] * map->ext[1] * map->ext[2];
  rmsmap = VarMap(map, radius, csym, nsym, 0);
  min = rmsmap->map[0][0][0];
  for (i0 = 0; i0 < rmsmap->ext[0]; ++i0)
    for (i1 = 0; i1 < rmsmap->ext[1]; ++i1)
      for (i2 = 0; i2 < rmsmap->ext[2]; ++i2)
	if (min > rmsmap->map[i0][i1][i2])
	{ min = rmsmap->map[i0][i1][i2];
	  centre[0] = i0;
	  centre[1] = i1;
	  centre[2] = i2;}
  FreeMap(rmsmap);
  mask = SphericalMask(map, radius, centre);
  return(mask);}



/******************************************************************************/

#if defined (PROTOTYPE)
  MAP *VarMap(MAP *map, float radius, TRANSFORM *csym, int nsym, float solvmean)
#else
  MAP *VarMap(map, radius, csym, nsym, solvmean)
  MAP *map; 
  TRANSFORM *csym;
  int nsym;
  float radius, solvmean;
#endif
/* Returns a map which contains the local variation of 'map' within a spherical
   volume defined by 'radius', relative to solvmean. */

{ int i, j, k, m, centre[3], len, i0, i1, **dstr, *ndstr, np, ori[3], ext[3];
  int tox, toy, toz;
  MASK *mask;
  float *mp, *stripe, *buffer, mean, **mpx, *mpxy;
  MAP *stripemap, *rmsmap;
#if !defined (PROTOTYPE)
  MASK *SphericalMask();
  MAP *SubMap();
  void ExtractMap();
  void FreeMap();
#endif

  /* Set up a spherical mask & map */
  centre[0] = centre[1] = centre[2] = 0;
  mask = SphericalMask(map, radius, centre);
  dstr = (int **) calloc(mask->ext[2] + 1, sizeof (int *));
  ndstr = (int *) calloc(mask->ext[2] + 1, sizeof (int));
  for (i = 0; i < mask->ext[0]; ++i)
    for (j = 0; j < mask->ext[1]; ++j)
    { m = 0;
      for (k = 0, np = 0; k < mask->ext[2]; ++k)
        if (mask->msk[i][j][k] > 0)
	{ ++np;
	  m = k;}
      if (ndstr[np] == 0)
        dstr[np] = (int *) malloc(3 * sizeof (int));
      else
	dstr[np] = (int *) realloc(dstr[np], (ndstr[np] + 3) * sizeof (int));
      dstr[np][ndstr[np]] = i - (mask->ext[0] / 2);
      dstr[np][ndstr[np] + 1] = j - (mask->ext[1] / 2);
      dstr[np][ndstr[np] + 2] = m;
      ndstr[np] += 3;}

  ori[0] = 0;
  ori[1] = -mask->ext[1] / 2;
  ori[2] = -mask->ext[2] / 2;
  ext[0] = 1;
  ext[1] = map->ext[1] + mask->ext[1] - 1;
  ext[2] = map->ext[2] + mask->ext[2] - 1;
  stripemap = SubMap(map, ori, ext);
  rmsmap = SubMap(map, map->ori, map->ext);
  stripe = (float *) malloc(sizeof(float) * ext[2]);
  buffer = (float *) malloc(sizeof(float) * ext[2]);
  mean = solvmean;
  for (i0 = -mask->ext[0] / 2; i0 < (map->ext[0] + (mask->ext[0] / 2)); ++i0)
  { stripemap->ori[0] = i0;
    ExtractMap(map, stripemap, csym, nsym);
    for (i1 = 0; i1 < stripemap->ext[1]; ++i1)
    { for (i = 0; i < ext[2]; ++i)
      { buffer[i] = stripemap->map[0][i1][i] - mean;
	stripe[i] = buffer[i] *= buffer[i];}
      for (len = 1; len <= mask->ext[2]; ++len)
	if (ndstr[len] > 0)
	{ if (len > 1)
      	  { for (i = 0, stripe[len - 1] = 0; i < len; ++i)
	      stripe[len - 1] += buffer[i];
	    for (i = len; i < ext[2]; ++i)
	      stripe[i] = stripe[i - 1] + buffer[i] - buffer[i - len];
	  for (i = 0; i < ndstr[len]; i += 3)
	  { tox = i0 + dstr[len][i];
	    if ((tox >= 0) && (tox < rmsmap->ext[0]))
	    { mpx = rmsmap->map[tox];
	      toy = i1 + dstr[len][i + 1] - (int) (mask->ext[1] / 2);
	      if ((toy >= 0) && (toy < rmsmap->ext[1]))
	      { mpxy = mpx[toy];
		j = dstr[len][i + 2];
		for (toz = 0; toz < rmsmap->ext[2]; ++toz)
		  mpxy[toz] += stripe[j++];}}}}}}}
  mp = rmsmap->map[0][0];
  for (i = map->ext[0] * map->ext[1] * map->ext[2] - 1; i >= 0; --i)
    mp[i] = sqrt(mp[i] / (mask->nset - 1));
  FreeMap(stripemap);
  free(stripe);
  free(buffer);
  for (i = 0; i <= mask->ext[2]; ++i)
    if (ndstr[i] > 0)
      free(dstr[i]);
  free(ndstr);
  free(dstr);
  return(rmsmap);}



/******************************************************************************/

#if defined (PROTOTYPE)
  MAP *VarMap2(MAP *map, float radius, TRANSFORM *csym, int nsym)
#else
  MAP *VarMap2(map, radius, csym, nsym)
  MAP *map;
  float radius;
  TRANSFORM *csym;
  int nsym;
#endif
/* Returns a map which contains the local variation of 'map' within a spherical
   volume defined by 'radius' */

{ int i, j, k, m, centre[3], len, i0, i1, **dstr, *ndstr, np, ori[3], ext[3];
  int tox, toy, toz;
  MASK *mask;
  float *mp, *stripe, *buffer, mean, **mpx, *mpxy;
  MAP *stripemap, *rmsmap;
#if !defined (PROTOTYPE)
  MASK *SphericalMask();
  MAP *SubMap();
  float MeanMap();
  void ExtractMap();
  void FreeMap();
#endif

  /* Set up a spherical mask & map */
  centre[0] = centre[1] = centre[2] = 0;
  mask = SphericalMask(map, radius, centre);
  dstr = (int **) calloc(mask->ext[2] + 1, sizeof (int *));
  ndstr = (int *) calloc(mask->ext[2] + 1, sizeof (int));
  for (i = 0; i < mask->ext[0]; ++i)
    for (j = 0; j < mask->ext[1]; ++j)
    { for (k = 0, np = 0; k < mask->ext[2]; ++k)
        if (mask->msk[i][j][k] > 0)
	{ ++np;
	  m = k;}
      if (ndstr[np] == 0)
        dstr[np] = (int *) malloc(3 * sizeof (int));
      else
	dstr[np] = (int *) realloc(dstr[np], (ndstr[np] + 3) * sizeof (int));
      dstr[np][ndstr[np]] = i - (mask->ext[0] / 2);
      dstr[np][ndstr[np] + 1] = j - (mask->ext[1] / 2);
      dstr[np][ndstr[np] + 2] = m;
      ndstr[np] += 3;}

  ori[0] = ori[1] = 0;
  ori[2] = -mask->ext[2] / 2;
  ext[0] = ext[1] = 1;
  ext[2] = map->ext[2] + mask->ext[2] - 1;
  stripemap = SubMap(map, ori, ext);
  rmsmap = SubMap(map, map->ori, map->ext);
  stripe = (float *) malloc(sizeof(float) * ext[2]);
  buffer = (float *) malloc(sizeof(float) * ext[2]);
  mean = MeanMap(map, NULL);
  for (i0 = -mask->ext[0] / 2; i0 < (map->ext[0] + (mask->ext[0] / 2)); ++i0)
    for (i1 = -mask->ext[1] / 2; i1 < (map->ext[1] + (mask->ext[1] / 2)); ++i1)
    { stripemap->ori[0] = i0;
      stripemap->ori[1] = i1;
      ExtractMap(map, stripemap, csym, nsym);
      for (i = 0; i < ext[2]; ++i)
      { buffer[i] = stripemap->map[0][0][i] - mean;
	stripe[i] = buffer[i] *= buffer[i];}

      for (len = 1; len <= mask->ext[2]; ++len)
	if (ndstr[len] > 0)
	{ if (len > 1)
      	  { for (i = 0, stripe[len - 1] = 0; i < len; ++i)
	      stripe[len - 1] += buffer[i];
	    for (i = len; i < ext[2]; ++i)
	      stripe[i] = stripe[i - 1] + buffer[i] - buffer[i - len];
	  for (i = 0; i < ndstr[len]; i += 3)
	  { tox = i0 + dstr[len][i];
	    if ((tox >= 0) && (tox < rmsmap->ext[0]))
	    { mpx = rmsmap->map[tox];
	      toy = i1 + dstr[len][i + 1];
	      if ((toy >= 0) && (toy < rmsmap->ext[1]))
	      { mpxy = mpx[toy];
		j = dstr[len][i + 2];
		for (toz = 0; toz < rmsmap->ext[2]; ++toz)
		  mpxy[toz] += stripe[j++];}}}}}}
  mp = rmsmap->map[0][0];
  for (i = map->ext[0] * map->ext[1] * map->ext[2] - 1; i >= 0; --i)
    mp[i] = sqrt(mp[i] / (mask->nset - 1));
  FreeMap(stripemap);
  free(stripe);
  free(buffer);
  for (i = 0; i <= mask->ext[2]; ++i)
    if (ndstr[i] > 0)
      free(dstr[i]);
  free(ndstr);
  free(dstr);
  return(rmsmap);}



/******************************************************************************/

#if defined (PROTOTYPE)
  MAP *RmsMap(MAP *map, float radius, TRANSFORM *csym, int nsym)
#else
  MAP *RmsMap(map, radius, csym, nsym)
  MAP *map;
  float radius;
  TRANSFORM *csym; 
  int nsym;
#endif
/* Returns a map which contains the local rms of 'map' within a spherical
   volume defined by 'radius' */

{ int i, j, k, m, centre[3], inc, i0, i1, **dstr, *ndstr, np, ori[3], ext[3];
  int tox, toy;
  MASK *mask;
  float *mp, *stripe, *buffer, mean;
  MAP *stripemap, *rmsmap;
#if !defined (PROTOTYPE)
  MASK *SphericalMask();
  MAP *SubMap();
  float MeanMap();
  void ExtractMap();
  void FreeMap();
#endif

  /* Set up a spherical mask & map */
  centre[0] = centre[1] = centre[2] = 0;
  mask = SphericalMask(map, radius, centre);
  dstr = (int **) calloc(mask->ext[2] + 1, sizeof (int *));
  ndstr = (int *) calloc(mask->ext[2] + 1, sizeof (int));
  for (i = 0; i < mask->ext[0]; ++i)
    for (j = 0; j < mask->ext[1]; ++j)
    { for (k = 0, np = 0; k < mask->ext[2]; ++k)
        if (mask->msk[i][j][k] > 0)
	  ++np;
      if (dstr[np] == 0)
      { dstr[np] = (int *) malloc(2 * sizeof (int));
	dstr[np][0] = i - (mask->ext[0] / 2);
	dstr[np][1] = j - (mask->ext[1] / 2);
	ndstr[np] = 2;}
      else
      { dstr[np] = (int *) realloc(dstr[np], (ndstr[np] + 2) * sizeof (int));
	dstr[np][ndstr[np]] = i - (mask->ext[0] / 2);
	dstr[np][ndstr[np] + 1] = j - (mask->ext[1] / 2);
	ndstr[np] += 2;}}


  ori[0] = ori[1] = 0;
  ori[2] = -mask->ext[2] / 2;
  ext[0] = ext[1] = 1;
  ext[2] = map->ext[2] + mask->ext[2] - 1;
  stripemap = SubMap(map, ori, ext);
  rmsmap = SubMap(map, map->ori, map->ext);
  stripe = (float *) malloc(sizeof(float) * ext[2]);
  buffer = (float *) malloc(sizeof(float) * ext[2]);
  mean = MeanMap(map, NULL);
  for (i0 = -mask->ext[0] / 2; i0 < (map->ext[0] + (mask->ext[0] / 2)); ++i0)
    for (i1 = -mask->ext[1] / 2; i1 < (map->ext[1] + (mask->ext[1] / 2)); ++i1)
    { stripemap->ori[0] = i0;
      stripemap->ori[1] = i1;
      ExtractMap(map, stripemap, csym, nsym);
      for (i = 0; i < ext[2]; ++i)
      { buffer[i] = stripemap->map[0][0][i] - mean;
	buffer[i] *= buffer[i];}

      for (i = mask->ext[2] / 2, k = 0; k < map->ext[2]; ++i, ++k) 
	for (j = 0; j < ndstr[1]; j += 2)
	{ tox = i0 + dstr[1][j];
	  if ((tox >= 0) && (tox < rmsmap->ext[0]))
	  { toy = i1 + dstr[1][j + 1];
	    if ((toy >= 0) && (toy < rmsmap->ext[1]))
	      rmsmap->map[tox][toy][k] += buffer[i];}}
      for (i = 3, inc = 1; i <= mask->ext[2]; i += 2, ++inc)
      { for (stripe[inc] = 0, j = 0; j < i; ++j)
	  stripe[inc] += buffer[j];
	for (j = 1 + inc; (j + inc) < ext[2]; ++j)
	  stripe[j] = stripe[j - 1] + buffer[j + inc] - buffer[j - inc];
	for (j = mask->ext[2] / 2, m = 0; m < map->ext[2]; ++j, ++m) 
	  for (k = 0; k < ndstr[i]; k += 2)
	  { tox = i0 + dstr[i][k];
	    if ((tox >= 0) && (tox < rmsmap->ext[0]))
	    { toy = i1 + dstr[i][k + 1];
	      if ((toy >= 0) && (toy < rmsmap->ext[1]))
		rmsmap->map[tox][toy][m] += stripe[j];}}}}
  mp = rmsmap->map[0][0];
  for (i = map->ext[0] * map->ext[1] * map->ext[2] - 1; i >= 0; --i)
    mp[i] = sqrt(mp[i] / (mask->nset - 1));
  FreeMap(stripemap);
  free(stripe);
  free(buffer);
  for (i = 0; i <= mask->ext[2]; ++i)
    if (ndstr[i] > 0)
      free(dstr[i]);
  free(ndstr);
  free(dstr);
  return(rmsmap);}



/******************************************************************************/

#if defined (PROTOTYPE)
  MAP *RmsMap2(MAP *map, float radius, TRANSFORM *csym, int nsym)
#else
  MAP *RmsMap2(map, radius, csym, nsym)
  MAP *map;
  float radius;
  TRANSFORM *csym;
  int nsym;
#endif
/* Returns a map which contains the local rms of 'map' within a spherical
   volume defined by 'radius' */

{ int i, ext[3], ori[3], centre[3], inc[3], i0, i1, i2, npnts, strt[3];
  MASK *mask;
  Byte *msk;
  float *mp, mean, dif, sd;
  double v[3];
  MAP *tst, *minimap, *rmsmap;
#if !defined (PROTOTYPE)
  MASK *SphericalMask();
  MAP *SubMap();
  void ExtractMap();
  void FreeMap();
  void FreeMask();
#endif

  /* Set up a spherical mask & map */
  centre[0] = centre[1] = centre[2] = 0;
  mask = SphericalMask(map, radius, centre);
  tst = SubMap(map, mask->ori, mask->ext);

  /* Set up a minimap in which the local rms will be accumulated. */
  for (i = 0; i < 3; ++i)
  { inc[i] = /*NINT((float) mask->ext[i] / 3);*/ 1;
    ext[i] = 4 + SMINT(map->ext[i] / inc[i]);
    ori[i] = -1;}
  minimap = SubMap(map, ori, ext);

  /* Determine the local rms (but not on every gridpoint) and put it in
     'minimap'. */
  npnts = mask->ext[0] * mask->ext[1] * mask->ext[2];
  mean = map->mean;
  for (i = 0; i < 3; ++i)
    strt[i] = map->ori[i] - (mask->ext[i] - 1) / 2 - inc[i];
  tst->ori[0] = mask->ori[0] = strt[0];
  for (i0 = 0; i0 < ext[0]; ++i0, tst->ori[0] = mask->ori[0] += inc[0])
  { tst->ori[1] = mask->ori[1] = strt[1];
    for (i1 = 0; i1 < ext[1]; ++i1, tst->ori[1] = mask->ori[1] += inc[1])
    { tst->ori[2] = mask->ori[2] = strt[2];
      for (i2 = 0; i2 < ext[2]; ++i2, tst->ori[2] = mask->ori[2] += inc[2])
      { ExtractMap(map, tst, csym, nsym);
	msk = mask->msk[0][0];
	mp = tst->map[0][0];
	for (i = 0, sd = 0; i < npnts; ++i, ++msk, ++mp)
	  if (*msk > 0)
	  { dif = mean - *mp;
	    sd += SQR(dif);}
	minimap->map[i0][i1][i2] = sqrt(sd/(mask->nset - 1));}}}
  FreeMap(tst);
  FreeMask(mask);

  /* Now calculate the local rms in every point of the map by 3D splines */
  rmsmap = SubMap(map, map->ori, map->ext);
  for (i0 = 0, v[0] = 1; i0 < map->ext[0]; ++i0, v[0] += 1 / (double) inc[0])
    for (i1 = 0, v[1] = 1; i1 < map->ext[1]; ++i1, v[1] += 1 / (double) inc[1])
      for (i2 = 0, v[2] = 1; i2 < map->ext[2]; ++i2, v[2] += 1 / (double) inc[2])
        rmsmap->map[i0][i1][i2] = SplInt(minimap->map, v);
  FreeMap(minimap);
  return(rmsmap);}



/******************************************************************************/

#if defined (PROTOTYPE)
  MASK* CarveMask(MASK *frm, MASK *to, TRANSFORM *trans, int ntrans)
#else
  MASK* CarveMask(frm, to, trans, ntrans)
  MASK *frm, *to;
  TRANSFORM *trans;
  int ntrans;
#endif
/* Gridpoints set in both masks 'frm' and 'to', will be removed from
   'to'. Returns 'to'. The transfromations should define crystallographic
   symmetries, ie. transform gridpoints onto gridpoints. */

{ TRANSFORM *ctrans, *tf;
  int i, f0, f1, f2, t[3], itf, a, b, c;
  double *m, tmp[3];
#if !defined (PROTOTYPE)
  void TransAToGrid();
#endif

  ctrans = (TRANSFORM *) malloc(ntrans * sizeof(TRANSFORM));
  for (i = 0; i < ntrans; ++i)
    TransAToGrid(trans + i, ctrans + i, frm->cell, frm->grid);
  for (f0 = frm->ori[0]; f0 < (frm->ori[0] + frm->ext[0]); ++f0)
    for (f1 = frm->ori[1]; f1 < (frm->ori[1] + frm->ext[1]); ++f1)
      for (f2 = frm->ori[2]; f2 < (frm->ori[2] + frm->ext[2]); ++f2)
	if (frm->msk[f0 - frm->ori[0]][f1 - frm->ori[1]][f2 - frm->ori[2]] != 0)
	  for (tf = ctrans, itf = 0; itf < ntrans; tf = ctrans + ++itf)
	  { m = tf->rotate;
	    tmp[0] = m[0] * (float) f0 + m[3] * (float) f1 + m[6] * (float) f2;
	    tmp[1] = m[1] * (float) f0 + m[4] * (float) f1 + m[7] * (float) f2;
	    tmp[2] = m[2] * (float) f0 + m[5] * (float) f1 + m[8] * (float) f2;
	    for (i = 0; i < 3; ++i)
	    { t[i] = NINT(tmp[i] + tf->translate[i]);
	      while (t[i] > to->ori[i])
		t[i] -= to->grid[i];
	      while (t[i] < to->ori[i])
		t[i] += to->grid[i];}
	    for (a = t[0] - to->ori[0]; a < to->ext[0]; a += to->grid[0])
	      for (b = t[1] - to->ori[1]; b < to->ext[1]; b += to->grid[1])
		for (c = t[2] - to->ori[2]; c < to->ext[2]; c += to->grid[2])
		  if (to->msk[a][b][c] > 0)
		  { --to->nset;
		    to->msk[a][b][c] = 0;}}
  free(ctrans);
  return(to);}



/******************************************************************************/

#if defined (PROTOTYPE)
  MASK* SetMask(MASK *frm, MASK *to, int set, TRANSFORM *trans, int ntrans)
#else
  MASK* SetMask(frm, to, set, trans, ntrans)
  MASK *frm, *to;
  int set, ntrans;
  TRANSFORM *trans;
#endif
/* Gridpoints set in masks 'frm', will be set to the value `set` in mask
   'to'. Returns 'to'. The transfromations should define crystallographic
   symmetries, ie. transform gridpoints onto gridpoints. */

{ TRANSFORM *ctrans, *tf;
  int i, f0, f1, f2, t[3], itf, a, b, c;
  double *m, tmp[3];
#if !defined (PROTOTYPE)
  void TransAToGrid();
#endif

  ctrans = (TRANSFORM *) malloc(ntrans * sizeof(TRANSFORM));
  for (i = 0; i < ntrans; ++i)
    TransAToGrid(trans + i, ctrans + i, frm->cell, frm->grid);
  for (f0 = frm->ori[0]; f0 < (frm->ori[0] + frm->ext[0]); ++f0)
    for (f1 = frm->ori[1]; f1 < (frm->ori[1] + frm->ext[1]); ++f1)
      for (f2 = frm->ori[2]; f2 < (frm->ori[2] + frm->ext[2]); ++f2)
	if (frm->msk[f0 - frm->ori[0]][f1 - frm->ori[1]][f2 - frm->ori[2]] != 0)
	  for (tf = ctrans, itf = 0; itf < ntrans; tf = ctrans + ++itf)
	  { m = tf->rotate;
	    tmp[0] = m[0] * (float) f0 + m[3] * (float) f1 + m[6] * (float) f2;
	    tmp[1] = m[1] * (float) f0 + m[4] * (float) f1 + m[7] * (float) f2;
	    tmp[2] = m[2] * (float) f0 + m[5] * (float) f1 + m[8] * (float) f2;
	    for (i = 0; i < 3; ++i)
	    { t[i] = NINT(tmp[i] + tf->translate[i]);
	      while (t[i] > to->ori[i])
		t[i] -= to->grid[i];
	      while (t[i] < to->ori[i])
		t[i] += to->grid[i];}
	    for (a = t[0] - to->ori[0]; a < to->ext[0]; a += to->grid[0])
	      for (b = t[1] - to->ori[1]; b < to->ext[1]; b += to->grid[1])
		for (c = t[2] - to->ori[2]; c < to->ext[2]; c += to->grid[2])
		{ if ((to->msk[a][b][c] > 0) && (set == 0))
		    --to->nset;
		  else if ((to->msk[a][b][c] == 0) && (set > 0))
	            ++to->nset;
		  to->msk[a][b][c] = set;}}
  free(ctrans);
  return(to);}



/******************************************************************************/

#if defined (PROTOTYPE)
  MASK* AddMask(MASK *frm, MASK *to, TRANSFORM *trans, int ntrans)
#else
  MASK* AddMask(frm, to, trans, ntrans)
  MASK *frm, *to;
  TRANSFORM *trans;
  int ntrans;
#endif
/* Gridpoints set in either masks 'frm' and 'to', will be set in 
   'to'. Returns 'to'. The transfromations should define crystallographic
   symmetries, ie. transform gridpoints onto gridpoints. */

{ TRANSFORM *ctrans, *tf;
  int i, f0, f1, f2, t[3], itf, a, b, c;
  double *m, tmp[3];
#if !defined (PROTOTYPE)
  void TransAToGrid();
#endif

  ctrans = (TRANSFORM *) malloc(ntrans * sizeof(TRANSFORM));
  for (i = 0; i < ntrans; ++i)
    TransAToGrid(trans + i, ctrans + i, frm->cell, frm->grid);
  for (f0 = frm->ori[0]; f0 < (frm->ori[0] + frm->ext[0]); ++f0)
    for (f1 = frm->ori[1]; f1 < (frm->ori[1] + frm->ext[1]); ++f1)
      for (f2 = frm->ori[2]; f2 < (frm->ori[2] + frm->ext[2]); ++f2)
	if (frm->msk[f0 - frm->ori[0]][f1 - frm->ori[1]][f2 - frm->ori[2]] != 0)
	  for (tf = ctrans + (itf = 0); itf < ntrans; tf = ctrans + ++itf)
	  { m = tf->rotate;
	    tmp[0] = m[0] * (float) f0 + m[3] * (float) f1 + m[6] * (float) f2;
	    tmp[1] = m[1] * (float) f0 + m[4] * (float) f1 + m[7] * (float) f2;
	    tmp[2] = m[2] * (float) f0 + m[5] * (float) f1 + m[8] * (float) f2;
	    for (i = 0; i < 3; ++i)
	    { t[i] = NINT(tmp[i] + tf->translate[i]);
	      while (t[i] > to->ori[i])
		t[i] -= to->grid[i];
	      while (t[i] < to->ori[i])
		t[i] += to->grid[i];}
	    for (a = t[0] - to->ori[0]; a < to->ext[0]; a += to->grid[0])
	      for (b = t[1] - to->ori[1]; b < to->ext[1]; b += to->grid[1])
		for (c = t[2] - to->ori[2]; c < to->ext[2]; c += to->grid[2])
		  if (to->msk[a][b][c] == 0)
		  { ++to->nset;
		    to->msk[a][b][c] = 1;}}
  free(ctrans);
  return(to);}



/******************************************************************************/

  double *NormalDistr()

/* Returns a pointer to an array which defines a normal probability distribution 
   with a standard deviation of 1 and a mean of 0, with 30000 elements, each
   element specifying an increment of 0.001 in sigma. */

{ static double *prob= NULL;
  int i;

  if (prob == NULL)
  { prob = (double *) malloc(4000 * sizeof(double));
    prob[0] = 0;
    for (i = 1; i < 4000; ++i)
      prob[i] = 0.001 * 0.39894228 * exp(-SQR((float) i * 0.001) / 2);
    for (i = 1; i < 4000; ++i)
      prob[i] += prob[i - 1];
    for (i = 0; i < 4000; ++i)
      prob[i] = 2 * (0.5 - prob[i]);}
  return(prob);}



/******************************************************************************/

#if defined (PROTOTYPE)
  void FillGridp(MAP *map, MASK *mask, int *p, TRANSFORM *trans, int ntrans,
	       float thresh, int depth, MASK *tstmsk, Byte border)
#else
  void FillGridp(map, mask, p, trans, ntrans, thresh, depth, tstmsk, border)
  MAP *map;
  MASK *mask;
  int *p, ntrans, depth;
  TRANSFORM *trans;
  float thresh;
  MASK *tstmsk;
  Byte border;
#endif
/* Recursively fills 'mask': if the gridpoint defined by p[] is lower than 
   'thresh', it will be excluded in the mask 'mask', likewise for its neighbours.
   If the boundaries of the map are overstepped, the neighbours are determined
   by using the crystallographic transformations defined by 'trans'. If 'tstmsk'
   != NULL, points are only set if they are within 'tstmsk'. Gridpoints bordering
   the resulting mask will be set to the value 'border'. */

{ double *m;
  int i, v[3], n = ntrans;
  TRANSFORM *tf;
#if !defined (PROTOTYPE)
  void FillGridp();
#endif

  if (++depth < 2)
  { v[0] = p[0]; 
    v[1] = p[1];
    v[2] = p[2];
    if (VECINMAP(v, map) == FALSE)
      for (i = 0; i < 3; ++i)
      { if (v[i] < map->ori[i])
	  do
	    v[i] += map->grid[i];
	  while (v[i] < map->ori[i]);
      else if (v[i] >= (map->ori[i] + map->ext[i]))
	do
	  v[i] -= map->grid[i];
	while (v[i] >= (map->ori[i] + map->ext[i]));}
    while ((VECINMAP(v, map) == FALSE) && (n >= 0))
    { tf = &trans[n--];
      m = tf->rotate;
      v[0] = (tf->translate[0] / map->cell[0]) * map->grid[0] +
	NINT(m[0] * (float) p[0] + m[3] * (float) p[1] + m[6] * (float) p[2]);
      v[1] = (tf->translate[1] / map->cell[1]) * map->grid[1] +
	NINT(m[1] * (float) p[0] + m[4] * (float) p[1] + m[7] * (float) p[2]);
      v[2] = (tf->translate[2] / map->cell[2]) * map->grid[2] +
	NINT(m[2] * (float) p[0] + m[5] * (float) p[1] + m[8] * (float) p[2]);
      for (i = 0; i < 3; ++i)
      { if (v[i] < map->ori[i])
	  do
	    v[i] += map->grid[i];
	  while (v[i] < map->ori[i]);
      else if (v[i] >= (map->ori[i] + map->ext[i]))
	do
	  v[i] -= map->grid[i];
	while (v[i] >= (map->ori[i] + map->ext[i]));}}
    if (n < 0)
    { printf("Severe error in FillGridp: %s not an asymmetric unit!!\n", map->infile);
      printf("%d %d %d\n", p[0], p[1], p[2]);
      ccperror(1,"*** Program Terminating ***");}
    else
    { if (tstmsk != NULL)
	if (tstmsk->msk[v[0]][v[1]][v[2]] == 0)
	  return;
      if ((map->map[v[0]][v[1]][v[2]] > thresh) &&
	  (mask->msk[v[0]][v[1]][v[2]] == 1))
      { --mask->nset;
	mask->msk[v[0]][v[1]][v[2]] = 0;
	v[0]++;
	FillGridp(map, mask, v, trans, ntrans, thresh, depth, tstmsk, border);
	v[0]--; v[1]++;
	FillGridp(map, mask, v, trans, ntrans, thresh, depth, tstmsk, border);
	v[1]--; v[2]++;
	FillGridp(map, mask, v, trans, ntrans, thresh, depth, tstmsk, border);
	v[2]--; v[0]--;
	FillGridp(map, mask, v, trans, ntrans, thresh, depth, tstmsk, border);
	v[0]++; v[1]--;
	FillGridp(map, mask, v, trans, ntrans, thresh, depth, tstmsk, border);
	v[1]++; v[2]--;
	FillGridp(map, mask, v, trans, ntrans, thresh, depth, tstmsk, border);}
      else if (map->map[v[0]][v[1]][v[2]] <= thresh)
	mask->msk[v[0]][v[1]][v[2]] = border;}}}



/******************************************************************************/

#if defined (PROTOTYPE)
  MASK *GetSolvMask(MAP *map, MASK **mask, int nmask, float radius,
		  char *rmsmapname, float solvmean)
#else
  MASK *GetSolvMask(map, mask, nmask, radius, rmsmapname, solvmean)
  MAP *map;
  MASK **mask;
  int nmask;
  char *rmsmapname;
  float radius, solvmean;
#endif
/* Returns a solvent mask, containing the lowest fraction of the local rms-map
   as defined by the solvent content, using the integration radius defined by
   'radius', to this mask are added any masks with the title 'solvent', and
   masks bearing the title 'weight' or 'monitor' are excluded. The local rms
   map will be written to disk as 'rmsmapname' if a valid name was specified.*/

{ MAP *rmsmap;
  MASK *solmsk;
  float cntrlevel;
  int nset, p[3], i;
#if !defined (PROTOTYPE)
  MAP *VarMap();
  MASK *NotMask();
  float MinMap();
  void FreeMask();
  void WriteMap();
  void FillGridp();
  void FreeHist();
  void FreeMap();
  MASK *MallocMask();
#endif

  if ((map->solv > 0) || (rmsmapname[0] != '\0'))
  { printf("\nDetermining the local rms at %f A integration radius\n", radius);
    if (map->solv > 0)
      printf("in order to determine the solvent mask (%f percent of map).\n",
	     map->solv * 100.0);
    fflush(stdout);
    rmsmap = VarMap(map, radius, mask[0]->trans, mask[0]->ntrans, solvmean);
    solmsk = NotMask(ContourMap(rmsmap, MinMap(rmsmap, NULL, (1 + map->solv) / 2)));
/*    asymu = AsymMask(map, mask[0]->trans, mask[0]->ntrans);
    cntrlevel = MinMap(rmsmap, asymu, map->solv);
    printf("The input map contains %f%% more gridpoints than the asymmetric unit\n",
	   (100.0 * (float)asymu->nset) / ((float)(map->ext[0] + 1) * 
					   (float)(map->ext[1] + 1) *
					   (float)(map->ext[2] + 1)));
    FreeMask(asymu); */
    cntrlevel = MinMap(rmsmap, NULL, map->solv);
    if (rmsmapname != NULL)
      if (rmsmapname[0] != '\0')
      { WriteMap(rmsmap, rmsmapname);
	printf("Contour level of the rms map at the solvent boundary: %f\n", 
	       cntrlevel);}
    for (nset = 0; nset != solmsk->nset; )
    { nset = solmsk->nset; 
      for (p[0] = 1; p[0] < map->ext[0]; ++p[0])
	for (p[1] = 1; p[1] < map->ext[1]; ++p[1])
	  for (p[2] = 1; p[2] < map->ext[2]; ++p[2])
	    if ((solmsk->msk[p[0]][p[1]][p[2]] == 1) &&
		((solmsk->msk[p[0]-1][p[1]][p[2]] == 0) ||
		 (solmsk->msk[p[0]][p[1]-1][p[2]] == 0) ||
		 (solmsk->msk[p[0]][p[1]][p[2]-1] == 0) ||
		 (solmsk->msk[p[0]-1][p[1]-1][p[2]] == 0) ||
		 (solmsk->msk[p[0]-1][p[1]][p[2]-1] == 0) ||
		 (solmsk->msk[p[0]][p[1]-1][p[2]-1] == 0) ||
		 (solmsk->msk[p[0]-1][p[1]-1][p[2]-1] == 0)))
	      FillGridp(rmsmap, solmsk, p, mask[0]->trans, 
			mask[0]->ntrans, cntrlevel, 0, NULL, 1);
      for (p[0] = 0; p[0] < (map->ext[0] - 1); ++p[0])
	for (p[1] = 0; p[1] < (map->ext[1] - 1); ++p[1])
	  for (p[2] = 0; p[2] < (map->ext[2] - 1); ++p[2])
	    if ((solmsk->msk[p[0]][p[1]][p[2]] == 1) &&
		((solmsk->msk[p[0]+1][p[1]][p[2]] == 0) ||
		 (solmsk->msk[p[0]][p[1]+1][p[2]] == 0) ||
		 (solmsk->msk[p[0]][p[1]][p[2]+1] == 0) ||
		 (solmsk->msk[p[0]+1][p[1]+1][p[2]] == 0) ||
		 (solmsk->msk[p[0]+1][p[1]][p[2]+1] == 0) ||
		 (solmsk->msk[p[0]][p[1]+1][p[2]+1] == 0) ||
		 (solmsk->msk[p[0]+1][p[1]+1][p[2]+1] == 0)))
	      FillGridp(rmsmap, solmsk, p, mask[0]->trans, 
			mask[0]->ntrans, cntrlevel, 0, NULL, 1);}
    if (verbose == TRUE)
      FreeHist(DumpHist(HistMap(rmsmap, NULL, 100)));
    FreeMap(rmsmap);}
  else 
    solmsk = MallocMask(map->grid, map->ori, map->ext, map->cell);
  for (i = 1; i < nmask; ++i)
    if ((strcmp(mask[i]->title, "solvent") == 0))
      solmsk = AddMask(mask[i], solmsk, mask[0]->trans, mask[0]->ntrans);
  for (i = 1; i < nmask; ++i)
    if ((strcmp(mask[i]->title, "monitor") == 0) ||
	(strcmp(mask[i]->title, "weight") == 0))
      solmsk = SetMask(mask[i], solmsk, 0, mask[0]->trans, mask[0]->ntrans);
  return(solmsk);}



/******************************************************************************/

#if defined (PROTOTYPE)
  MASK *AsymMask(MAP *map, TRANSFORM *trans, int ntrans)
#else
  MASK *AsymMask(map, trans, ntrans)
  MAP *map;
  TRANSFORM *trans;
  int ntrans;
#endif
/* Returns a mask, the size of 'map', in which all the unique gridpoints have
   been set. Any gridpoints repeated by crystallographic symmetry as defined by
   'trans' and 'ntrans', are marked only once. */

{ MASK *mask;
  TRANSFORM *ctrans;
  int p[3], s, i, j, k, n, q[3];
  double *t, *m;
#if !defined (PROTOTYPE)
  void TransAToGrid();
  MASK *MallocMask();
#endif

  ctrans = (TRANSFORM *) malloc(ntrans * sizeof(TRANSFORM));
  for (i = 0; i < ntrans; ++i)
    TransAToGrid(trans + i, ctrans + i, map->cell, map->grid);
  mask = MallocMask(map->grid, map->ori, map->ext, map->cell);
  for (i = map->ori[0]; i < map->ori[0] + map->ext[0]; ++i)
    for (j = map->ori[1]; j < map->ori[1] + map->ext[1]; ++i)
      for (k = map->ori[2]; k < map->ori[2] + map->ext[2]; ++i)
      { if (mask->msk[i - map->ori[0]][j - map->ori[1]][k - map->ori[2]] == 0)
	{ ++mask->msk[i - map->ori[0]][j - map->ori[1]][k - map->ori[2]];
	  ++mask->nset;
	  for (s = 0; s < ntrans; ++s)
	  { m = trans[s].rotate;
	    t = trans[s].translate;
	    p[0] = m[0] * (float) i + m[3] * (float) j + m[6] * (float) k;
	    p[0] = NINT(p[0] + t[0]);
	    p[1] = m[1] * (float) i + m[4] * (float) j + m[7] * (float) k;
	    p[1] = NINT(p[1] + t[1]);
	    p[2] = m[2] * (float) i + m[5] * (float) j + m[8] * (float) k;
	    p[2] = NINT(p[2] + t[2]);}
	    for (n = 0; n < 3; ++n)
	    { if (p[n] < map->ori[n])
	        do
		  p[n] += map->grid[n];
		while (p[n] < map->ori[n]);
	      else if (p[n] >= (map->ori[n] + map->ext[n]))
		do
		  p[n] -= map->grid[n];
		while (p[n] >= (map->ori[n] - map->grid[n]));}
	    for (q[0] = 0; (p[0] + q[0]) < (map->ori[0] + map->ext[0]); ++q[0])
	      for (q[1] = 0; (p[1] + q[1]) < (map->ori[1] + map->ext[1]); ++q[1])
		for (q[2] = 0; (p[2] + q[2]) < (map->ori[2] + map->ext[2]); ++q[2])
		  if (VECINMASK(q, map))
		    mask->msk[q[0]][q[1]][q[2]] = 1;}}
  return(mask);}


	  

/******************************************************************************/

#if defined (PROTOTYPE)
  void ValidateMap(MAP *map, MASK **mask, MASK *solmask, float *validate)
#else
  void ValidateMap(map, mask, solmask, validate)
  MAP *map;
  MASK **mask, *solmask;
  float *validate;
#endif
/* Determines the top 'validate[0]' fraction of the protein part of the map, 
   and expands this fraction to the cutoff value defined by the protein fraction 
   'validate[0]'. Then it prints some statistics about this area. */

{ MASK *chain, *maxima;
  Byte *msk;
  int nborder = 0, i;
#if !defined (PROTOTYPE)
  MASK *GrowMask();
  MASK *MapMaxima();
  void FreeMask();
#endif

  chain = GrowMask(map, validate[0], validate[1], mask, solmask);
  maxima = MapMaxima(map, chain, mask[0]->trans, mask[0]->ntrans);
  printf("Validate: islands: %f above %f of protein, not connected to %f of protein\n", 
	 (1 - ((float) chain->nset / (float) solmask->nset)) - validate[1],
	 validate[1], validate[0]);
  printf("Validate: %d maxima in this area\n", maxima->nset);
  msk = chain->msk[0][0];
  for (i = (chain->ext[0] * chain->ext[1] * chain->ext[2]) - 1; i > 0; --i)
    if (msk[i] == 2)
      ++nborder;
  printf("Validate: volume to potential surface ratio: %f\n",
	 (float) nborder / ((1 - validate[1]) *
			    (float) ((solmask->ext[0] * 
				      solmask->ext[1] * 
				      solmask->ext[2]) - solmask->nset)));
  FreeMask(chain);
  FreeMask(maxima);}

  
/******************************************************************************/

#if defined (PROTOTYPE)
  MASK *GrowMask(MAP *map, float high, float low, MASK **mask, MASK *solmsk)
#else
  MASK *GrowMask(map, high, low, mask, solmsk)
  MAP *map;
  float high, low;
  MASK **mask, *solmsk;
#endif
/* First sets all gridpoints in a mask corresponding to `map` which fall 
   above the `high` fraction of the intensities. Then it adds gridpoints to the
   mask, provided they are above the `low` fraction and that their neighbour
   gridpoints have already been set. */

{ MASK *outmask;
  float cntrlevel;
  int nset, p[3], i;
#if !defined (PROTOTYPE)
  MASK *ContourMap();
  MASK *NotMask();
  float MinMap();
  void FillGridp();
#endif

  outmask = NotMask(ContourMap(map, MinMap(map, NotMask(solmsk), high)));
  for (i = (map->ext[0] * map->ext[1] * map->ext[2]) - 1; i >= 0; --i)
    if ((solmsk->msk[0][0][i] == 0) && (outmask->msk[0][0][i] == 0))
    { outmask->msk[0][0][i] = 1;
      ++outmask->nset;}
  cntrlevel = MinMap(map, solmsk, low);
  for (nset = 0; nset != outmask->nset; )
  { nset = outmask->nset; 
    for (p[0] = 1; p[0] < map->ext[0]; ++p[0])
      for (p[1] = 1; p[1] < map->ext[1]; ++p[1])
	for (p[2] = 1; p[2] < map->ext[2]; ++p[2])
	  if ((outmask->msk[p[0]][p[1]][p[2]] == 1) &&
	      ((outmask->msk[p[0]-1][p[1]][p[2]] == 0) ||
	       (outmask->msk[p[0]][p[1]-1][p[2]] == 0) ||
	       (outmask->msk[p[0]][p[1]][p[2]-1] == 0) ||
	       (outmask->msk[p[0]-1][p[1]-1][p[2]] == 0) ||
	       (outmask->msk[p[0]-1][p[1]][p[2]-1] == 0) ||
	       (outmask->msk[p[0]][p[1]-1][p[2]-1] == 0) ||
	       (outmask->msk[p[0]-1][p[1]-1][p[2]-1] == 0)))
	    FillGridp(map, outmask, p, mask[0]->trans, 
		      mask[0]->ntrans, cntrlevel, 0, solmsk, 2);
    for (p[0] = 0; p[0] < (map->ext[0] - 1); ++p[0])
      for (p[1] = 0; p[1] < (map->ext[1] - 1); ++p[1])
	for (p[2] = 0; p[2] < (map->ext[2] - 1); ++p[2])
	  if ((outmask->msk[p[0]][p[1]][p[2]] == 1) &&
	      ((outmask->msk[p[0]+1][p[1]][p[2]] == 0) ||
	       (outmask->msk[p[0]][p[1]+1][p[2]] == 0) ||
	       (outmask->msk[p[0]][p[1]][p[2]+1] == 0) ||
	       (outmask->msk[p[0]+1][p[1]+1][p[2]] == 0) ||
	       (outmask->msk[p[0]+1][p[1]][p[2]+1] == 0) ||
	       (outmask->msk[p[0]][p[1]+1][p[2]+1] == 0) ||
	       (outmask->msk[p[0]+1][p[1]+1][p[2]+1] == 0)))
	    FillGridp(map, outmask, p, mask[0]->trans, 
		      mask[0]->ntrans, cntrlevel, 0, solmsk, 2);}
  NotMask(solmsk);
  return(NotMask(outmask));}



/******************************************************************************/

#if defined (PROTOTYPE)
  MASK *ProtSausage(MAP *map, MASK **mask, int nmask, MASK *solmask, float sdred)
#else
  MASK *ProtSausage(map, mask, nmask, solmask, sdred)
  MAP *map;
  MASK **mask, *solmask;
  int nmask;
  float sdred;
#endif
/* Determines connected regions within the protein as defined by `solmask`, and
   reduces the standard deviation of these regions from their mean by a factor
   `sdred`. The standard deviation from the mean of the remaining regions within 
   the solvent mask is similarely reduced. An updated solvent mask is returned 
   (it could be that connected density makes an excursion into the solvent). */

{ MASK *sausage;
  float mean, sd, *m = map->map[0][0];
  Byte *s, *sm;
  int i;
#if !defined (PROTOTYPE)
  MASK *GrowMask();
  MASK *SetMask();
  float SigmaMap();
  void FreeMask();
#endif

  sausage = GrowMask(map, 0.95, 0.8, mask, solmask);
  for (i = 1; i < nmask; ++i)
    if ((strcmp(mask[i]->title, "monitor") == 0) ||
	(strcmp(mask[i]->title, "weight") == 0))
      sausage = SetMask(mask[i], sausage, 0, mask[0]->trans, mask[0]->ntrans);
  s = sausage->msk[0][0];
  sm = solmask->msk[0][0];
  sd = SigmaMap(map, sausage, &mean);
  printf("      Identified %f %% of proteinmask as chain...\n", 
	 100.0 * (float) sausage->nset /
	 ((solmask->ext[0] * solmask->ext[1] * solmask->ext[2]) - 
	  solmask->nset));
  printf("      Mean density of the protein chain:       %f\n", mean);
  printf("      Standard deviation of the protein chain: %f\n", sd);  
  for (i = (map->ext[0] * map->ext[1] * map->ext[2] - 1); i >= 0; --i)
    if (s[i] == 1) 
      m[i] *= sdred;
  for (i = (map->ext[0] * map->ext[1] * map->ext[2]) - 1; i >= 0; --i) 
    if ((sm[i] == 1) && (s[i] == 1)) 
      sm[i] = 0; 
  FreeMask(sausage);
  return solmask; }



/******************************************************************************/

#if defined (PROTOTYPE)
  MASK *MapMaxima(MAP *map, MASK *prot, TRANSFORM *trans, int ntrans)
#else
  MASK *MapMaxima(map, prot, trans, ntrans)
  MAP *map;
  MASK *prot;
  TRANSFORM *trans;
  int ntrans;
#endif
/* Returns a mask where every point within 'prot' which is a local maximum is set
   to 1, the rest of the points are set to 0. If 'prot' == NULL, the entire 
   map is used. */

{ MASK *mask;
  MAP *tstmap;
  int p0, p1, p2, ori[3], ext[3];
  float tst;
#if !defined (PROTOTYPE)
  MASK *MallocMask();
  MAP *SubMap();
  void ExtractMap();
  void FreeMap();
#endif

  mask = MallocMask(map->grid, map->ori, map->ext, map->cell);
  ori[0] = map->ori[0] - 1; ori[1] = map->ori[1] - 1; ori[2] = map->ori[2] - 1; 
  ext[0] = map->ext[0] + 2; ext[1] = map->ext[1] + 2; ext[2] = map->ext[2] + 2; 
  tstmap = SubMap(map, ori, ext);
  ExtractMap(map, tstmap, trans, ntrans);
  if (prot == NULL)
    for (p0 = 1; p0 < ext[0] - 1; ++p0)
      for (p1 = 1; p1 < ext[1] - 1; ++p1)
        for (p2 = 1; p2 < ext[2] - 1; ++p2)
        { tst = tstmap->map[p0][p1][p2];
	  if ((tstmap->map[p0-1][p1-1][p2-1] < tst) &&
	      (tstmap->map[p0-1][p1-1][p2  ] < tst) &&
	      (tstmap->map[p0-1][p1-1][p2+1] < tst) &&
	      (tstmap->map[p0-1][p1  ][p2-1] < tst) &&
	      (tstmap->map[p0-1][p1  ][p2  ] < tst) &&
	      (tstmap->map[p0-1][p1  ][p2+1] < tst) &&
	      (tstmap->map[p0-1][p1+1][p2-1] < tst) &&
	      (tstmap->map[p0-1][p1+1][p2  ] < tst) &&
	      (tstmap->map[p0-1][p1+1][p2+1] < tst) &&
	      (tstmap->map[p0  ][p1-1][p2-1] < tst) &&
	      (tstmap->map[p0  ][p1-1][p2  ] < tst) &&
	      (tstmap->map[p0  ][p1-1][p2+1] < tst) &&
	      (tstmap->map[p0  ][p1  ][p2-1] < tst) &&
	      (tstmap->map[p0  ][p1  ][p2+1] < tst) &&
	      (tstmap->map[p0  ][p1+1][p2-1] < tst) &&
	      (tstmap->map[p0  ][p1+1][p2  ] < tst) &&
	      (tstmap->map[p0  ][p1+1][p2+1] < tst) &&
	      (tstmap->map[p0+1][p1-1][p2-1] < tst) &&
	      (tstmap->map[p0+1][p1-1][p2  ] < tst) &&
	      (tstmap->map[p0+1][p1-1][p2+1] < tst) &&
	      (tstmap->map[p0+1][p1  ][p2-1] < tst) &&
	      (tstmap->map[p0+1][p1  ][p2  ] < tst) &&
	      (tstmap->map[p0+1][p1  ][p2+1] < tst) &&
	      (tstmap->map[p0+1][p1+1][p2-1] < tst) &&
	      (tstmap->map[p0+1][p1+1][p2  ] < tst) &&
	      (tstmap->map[p0+1][p1+1][p2+1] < tst))
	  { mask->msk[p0 - 1][p1 - 1][p2 - 1] = 1;
	    ++mask->nset;}}
  else
    for (p0 = 1; p0 < ext[0] - 1; ++p0)
      for (p1 = 1; p1 < ext[1] - 1; ++p1)
        for (p2 = 1; p2 < ext[2] - 1; ++p2)
        { if (prot->msk[p0 - 1][p1 - 1][p2 - 1] != 0)
	  { tst = tstmap->map[p0][p1][p2];
	    if ((tstmap->map[p0-1][p1-1][p2-1] < tst) &&
		(tstmap->map[p0-1][p1-1][p2  ] < tst) &&
		(tstmap->map[p0-1][p1-1][p2+1] < tst) &&
		(tstmap->map[p0-1][p1  ][p2-1] < tst) &&
		(tstmap->map[p0-1][p1  ][p2  ] < tst) &&
		(tstmap->map[p0-1][p1  ][p2+1] < tst) &&
		(tstmap->map[p0-1][p1+1][p2-1] < tst) &&
		(tstmap->map[p0-1][p1+1][p2  ] < tst) &&
		(tstmap->map[p0-1][p1+1][p2+1] < tst) &&
		(tstmap->map[p0  ][p1-1][p2-1] < tst) &&
		(tstmap->map[p0  ][p1-1][p2  ] < tst) &&
		(tstmap->map[p0  ][p1-1][p2+1] < tst) &&
		(tstmap->map[p0  ][p1  ][p2-1] < tst) &&
		(tstmap->map[p0  ][p1  ][p2+1] < tst) &&
		(tstmap->map[p0  ][p1+1][p2-1] < tst) &&
		(tstmap->map[p0  ][p1+1][p2  ] < tst) &&
		(tstmap->map[p0  ][p1+1][p2+1] < tst) &&
		(tstmap->map[p0+1][p1-1][p2-1] < tst) &&
		(tstmap->map[p0+1][p1-1][p2  ] < tst) &&
		(tstmap->map[p0+1][p1-1][p2+1] < tst) &&
		(tstmap->map[p0+1][p1  ][p2-1] < tst) &&
		(tstmap->map[p0+1][p1  ][p2  ] < tst) &&
		(tstmap->map[p0+1][p1  ][p2+1] < tst) &&
		(tstmap->map[p0+1][p1+1][p2-1] < tst) &&
		(tstmap->map[p0+1][p1+1][p2  ] < tst) &&
		(tstmap->map[p0+1][p1+1][p2+1] < tst))
	    { mask->msk[p0 - 1][p1 - 1][p2 - 1] = 1;
	      ++mask->nset;}}}
  FreeMap(tstmap);
  return(mask);}



/******************************************************************************/

#if defined (PROTOTYPE)
  void MapHistoMax(MAP *map, MASK *mask, TRANSFORM *trans, int ntrans)
#else
  void MapHistoMax(map, mask, trans, ntrans)
  MAP *map;
  MASK *mask;
  TRANSFORM *trans;
  int ntrans;
#endif
/* Dumps a histogram of the maxima of the map which fall within "mask". If
   "mask" == NULL, the histogram describes the entire map. */

{ MASK *maxima;
#if !defined (PROTOTYPE)
  MASK *MapMaxima();
  void FreeHist();
#endif

  maxima = MapMaxima(map, mask, trans, ntrans);
  FreeHist(DumpHist(HistMap(map, maxima, 50)));}
