/*

     This code is distributed under the terms and conditions of the
     CCP4 Program Suite Licence Agreement as a CCP4 Application.
     A copy of the CCP4 licence can be obtained by writing to the
     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "solomon.h"
#include "ccp4_spg.h"
#include "csymlib.h"
#include "cmaplib.h"
#include "ccp4_general.h"
#include "ccp4_parser.h"

extern BOOLEAN verbose;


/*****************************************************************************/

#if defined (PROTOTYPE)
  MAP *MallocMap(int grid[], int ori[], int ext[], float cell[])
#else
  MAP *MallocMap(grid, ori, ext, cell)
  int grid[], ori[], ext[];
  float cell[];
#endif
/* Returns a pointer to an empty map with the requested origin & extent. */

{ int i, j, k;
  MAP *map;
  float *data;

  map = (MAP *) malloc(sizeof(MAP));
  for (i = 0; i < 3; ++i)
  { map->grid[i] = grid[i];
    map->ori[i] = ori[i];
    map->ext[i] = ext[i];
    map->cell[i] = cell[i];
    map->cell[i + 3] = cell[i + 3];}
  map->infile[0] = map->outfile[0] = map->title[0] = '\0';
  map->map = (float ***) malloc((ext[0] * sizeof(float**)) +
                                (ext[0] * ext[1] * sizeof(float*)) +
				(ext[0] * ext[1] * ext[2] * sizeof(float)));
  if (map->map == NULL)
  { printf("Not enough space for a map of size %d %d %d\n",
	   ext[0], ext[1], ext[2]);
    ccperror(1,"*** Program Terminating ***");}
  data = (float *) (map->map + ext[0] * (1 + ext[1]));
  for (i = 0; i < ext[0]; ++i)
  { map->map[i] =  (float **) map->map + ext[0] + i * ext[1];
    for (j = 0; j < ext[1]; ++j, data += ext[2])
    { map->map[i][j] = data;
      for (k = 0; k < ext[2]; ++k)
	data[k] = 0;}}
  map->min = map->max = map->mean = map->rms = 0;
  map->solv = -1;
  return(map);}


/*****************************************************************************/

#if defined (PROTOTYPE)
  void FreeMap(MAP *map)
#else
  void FreeMap(map)
  MAP *map;
#endif
/* Frees memory allocated to 'map'. */

{ free(map->map);
  free(map);}



/*****************************************************************************/

#if defined (PROTOTYPE)
  MAP *SubMap(MAP *map, int ori[], int ext[])
#else
  MAP *SubMap(map, ori, ext)
  MAP *map;
  int ori[], ext[];
#endif
/* Returns a pointer to a new, empty map which inherits most of its 
   characteristics from the input map, but with the specified origin & extent. */

{ MAP *newmap;
  int i;
#if !defined (PROTOTYPE)
  MAP *MallocMap();
#endif

  newmap = MallocMap(map->grid, ori, ext, map->cell);
  for (i = 0; i < 3; ++i)
    newmap->order[i] = map->order[i];
  newmap->mode = map->mode;
  newmap->spcgrp = map->spcgrp;
  newmap->nsyms = map->nsyms;
  newmap->solv = -1;
  for (i = 0; i < (4*4*96); ++i)
    newmap->sym[i] = map->sym[i];
  newmap->mean = newmap->rms = newmap->max = newmap->min = 0;
  return(newmap);}

  
/*****************************************************************************/

#if defined (PROTOTYPE)
  MAP *ReadMap(char *mapname)
#else
  MAP *ReadMap(mapname)
  char *mapname;
#endif
/* Read a CCP4 map. */

{ 
  int i, j, k, slow, medium, fast;
  int order[3], grid[3], ori[3], ext[3];
  int tmp_array[3];
  float cell[6], rms, mean, min, max;
  double dmean, drms;
  float *rot;
  CMMFile *mapfile;
  MAP *map;
  float *data, *plane;
  char buffer[81];
  char *filename=NULL;
#if !defined (PROTOTYPE)
  int *StringCToFrt();
  void ShutUp();
  MAP *MallocMap();
  char *StringFrtToC();
  float MeanMap();
  void PrintMap();
#endif

  /* First read the header: */
  if (!(filename=getenv(mapname))) filename = mapname;
  mapfile = ccp4_cmap_open(filename, O_RDONLY);
  ShutUp(TRUE);
  ccp4_cmap_get_order(mapfile, order);
  fast = order[0] - 1;
  medium = order[1] - 1;
  slow = order[2] - 1;
  ccp4_cmap_get_dim(mapfile, tmp_array);
  ext[slow] = tmp_array[2];
  ext[medium] = tmp_array[1];
  ext[fast] = tmp_array[0];
  ccp4_cmap_get_origin(mapfile, tmp_array);
  ori[slow] = tmp_array[2];
  ori[medium] = tmp_array[1];
  ori[fast] = tmp_array[0];
  ccp4_cmap_get_cell(mapfile, cell);
  ccp4_cmap_get_grid(mapfile, grid);
  map = MallocMap(grid, ori, ext, cell);
  strcpy(map->infile,filename);
  map->spcgrp = ccp4_cmap_get_spacegroup(mapfile);
  map->mode = ccp4_cmap_get_datamode(mapfile);
  ccp4_cmap_get_order(mapfile,map->order);
  ccp4_cmap_get_mapstats(mapfile, &map->min, &map->max, &dmean, &drms);
  map->mean = dmean;
  map->rms = drms;
  strncpy(map->title,ccp4_cmap_get_title(mapfile),81U);
  map->nsyms = ccp4_cmap_num_symop(mapfile);
  rot = map->sym;
  for ( i = 0, rot = map->sym; i != map->nsyms ; ++i , rot += 16) {
    memset(buffer,'\0',81U);
    ccp4_cmap_get_symop(mapfile, buffer);
    symop_to_mat4(buffer,buffer+80,rot);
  }

  /* And read the map... (This can be done in quite a number of ways 
     because of axis permutations) */
  data = (float *) malloc(ext[fast] * ext[medium] *
			  sizeof(float));
  for (i = 0; i < ext[slow] ; ++i)
  { if (!(ccp4_cmap_read_section(mapfile, plane = data)))
    { printf("Error in reading %s", mapname);
      ccperror(1,"*** Program Terminating ***");}
    else if ((fast == 0) && (medium == 1) && (slow == 2))
      for (j = 0; j < ext[medium]; ++j)
	for (k = 0; k < ext[fast]; ++k)
	  map->map[k][j][i] = *(plane++);
    else if ((fast == 0) && (medium == 2) && (slow == 1))
      for (j = 0; j < ext[medium]; ++j)
	for (k = 0; k < ext[fast]; ++k)
	  map->map[k][i][j] = *(plane++);
    else if ((fast == 1) && (medium == 0) && (slow == 2))
      for (j = 0; j < ext[medium]; ++j)
	for (k = 0; k < ext[fast]; ++k)
	  map->map[j][k][i] = *(plane++);
    else if ((fast == 1) && (medium == 2) && (slow == 0))
      for (j = 0; j < ext[medium]; ++j)
	for (k = 0; k < ext[fast]; ++k)
	  map->map[i][k][j] = *(plane++);
    else if ((fast == 2) && (medium == 1) && (slow == 0))
      for (j = 0; j < ext[medium]; ++j)
	for (k = 0; k < ext[fast]; ++k)
	  map->map[i][j][k] = *(plane++);
    else if ((fast == 2) && (medium == 0) && (slow == 1))
      for (j = 0; j < ext[medium]; ++j)
	for (k = 0; k < ext[fast]; ++k)
	  map->map[j][i][k] = *(plane++);}
  ccp4_cmap_close(mapfile);
  map->mean = MeanMap(map, NULL);
  PrintMap(map);
  ShutUp(FALSE);
  free(data);
  return(map);}



/*****************************************************************************/

#if defined (PROTOTYPE)
  char *StringFrtToC(char *c_string, int *frt_string)
#else
  char *StringFrtToC(c_string, frt_string)
  char *c_string;
  int *frt_string;
#endif
/* Translates a string encoded by integers into a character string. Stops
   if 0 is reached and returns a pointer to c_string. */

{ int i;

  for (i = 0; (c_string[i] = (char) frt_string[i]) != '\0'; ++i);

  return(c_string);}



/*****************************************************************************/

#if defined (PROTOTYPE)
  int *StringCToFrt(int *frt_string, char *c_string, int strlen)
#else
  int *StringCToFrt(frt_string, c_string, strlen)
  int *frt_string;
  char *c_string;
  int strlen;
#endif
/* Translates character string into a string encoded by integers. */

{ int i;

  for (i = 0; i < strlen; ++i)
    frt_string[i] = (int) c_string[i];

  return(frt_string);}



/*****************************************************************************/

#if defined (PROTOTYPE)
  void PrintMap(MAP *map)
#else
  void PrintMap(map)
  MAP *map;
#endif
/* Prints relevant information of the structure `map`. */

{ int i;

  printf("\n%s\n\n", map->title);
  printf("From %s\n", map->infile);
  printf("Order: %d %d %d (fast, medium, slow)\n", map->order[0], map->order[1],
	 map->order[2]);
  printf("Mode: %d\n", map->mode);
  printf("Spacegroup: %d\n", map->spcgrp);
  printf("Grid: %d %d %d\n", map->grid[0], map->grid[1], map->grid[2]);
  printf("Origin: %d %d %d\n", map->ori[0], map->ori[1], map->ori[2]);
  printf("Extent: %d %d %d\n", map->ext[0], map->ext[1], map->ext[2]);
  printf("Cell:    %f %f %f %f %f %f  \n", map->cell[0],
          map->cell[1], map->cell[2], map->cell[3], map->cell[4],
          map->cell[5]);
  if (map->nsyms > 0)
  { printf("%d symmetry operators:\n\n", map->nsyms);
    for (i = 0; i < (16 * map->nsyms); i += 16)
    { printf("%1.5f %1.5f %1.5f %1.5f\n",
	     map->sym[i+0], map->sym[i+1], map->sym[i+2], map->sym[i+3]);
      printf("%1.5f %1.5f %1.5f %1.5f\n",
	     map->sym[i+4], map->sym[i+5], map->sym[i+6], map->sym[i+7]);
      printf("%1.5f %1.5f %1.5f %1.5f\n",
	     map->sym[i+8], map->sym[i+9], map->sym[i+10], map->sym[i+11]);
      printf("%1.5f %1.5f %1.5f %1.5f\n\n",
	     map->sym[i+12], map->sym[i+13], map->sym[i+14], map->sym[i+15]);}}
  printf("Min: %f\n", map->min);
  printf("Max: %f\n", map->max);
  printf("Mean: %f\n", map->mean);
  printf("Rms: %f\n", map->rms);
  if (map->solv >= 0)
    printf("Solvent content: %f\n\n", map->solv);}



/*****************************************************************************/

#if defined (PROTOTYPE)
  void WriteMap(MAP *map, char *mapname)
#else
  void WriteMap(map, mapname)
  MAP *map;
  char *mapname;
#endif
/* Writes 'map' as 'mapname'. */

{
  int fast, medium, slow, i, j, k, l;
  char *filename=NULL;
  float *buff, *plane;
  CMMFile *mapfile=NULL;
  int tmp_array[3];
  char buffer[81];
#if !defined (PROTOTYPE)
  void ShutUp();
#endif

  if (mapname[0] == '\0')
    return;
  fast = map->order[0] - 1;
  medium = map->order[1] - 1;
  slow = map->order[2] - 1;

  /* Set up the header of the new file */
  ShutUp(TRUE);
  if (!(filename=getenv(mapname))) filename = mapname;
  mapfile = (CMMFile *) ccp4_cmap_open(filename, O_WRONLY);
  ccp4_cmap_set_datamode(mapfile, map->mode);
  ccp4_cmap_set_cell(mapfile, map->cell);
  ccp4_cmap_set_grid(mapfile, map->grid);
  ccp4_cmap_set_order(mapfile, map->order);
  ccp4_cmap_set_spacegroup(mapfile, map->spcgrp);
  tmp_array[2] = map->ext[slow];
  tmp_array[1] = map->ext[medium];
  tmp_array[0] = map->ext[fast];
  ccp4_cmap_set_dim(mapfile, tmp_array);
  tmp_array[0] = map->ori[fast];
  tmp_array[1] = map->ori[medium];
  tmp_array[2] = map->ori[slow];
  ccp4_cmap_set_origin(mapfile, tmp_array);
  ccp4_cmap_set_title(mapfile, map->title);
  if (map->nsyms > 0) {
    float tran[4][4];
    buff = map->sym;
    for (i=0, l = 0; i != map->nsyms ; ++i) {
      for ( j = 0 ; j != 4 ; ++j )
        for ( k = 0 ; k != 4 ; ++k, ++l)
          tran[j][k] = buff[l];
      memset(buffer,' ',80U);
      buffer[80] = '\0';
      mat4_to_symop(buffer,&buffer[80],tran);
      ccp4_cmap_set_symop(mapfile, buffer);
    }
  }

  /* Write data to the map (many alternatives due to permutations of the
     axes)... */
  buff = (float *) malloc(map->ext[fast] * map->ext[medium] * sizeof(float));
  for (i = 0; i < map->ext[slow]; ++i)
  { plane = buff;
    if ((fast == 0) && (medium == 1) && (slow == 2))
    for (j = 0; j < map->ext[medium]; ++j)
      for (k = 0; k < map->ext[fast]; ++k)
	*(plane++) = map->map[k][j][i];
    else if ((fast == 0) && (medium == 2) && (slow == 1))
    for (j = 0; j < map->ext[medium]; ++j)
      for (k = 0; k < map->ext[fast]; ++k)
	*(plane++) = map->map[k][i][j];
    else if ((fast == 1) && (medium == 0) && (slow == 2))
    for (j = 0; j < map->ext[medium]; ++j)
      for (k = 0; k < map->ext[fast]; ++k)
	*(plane++) = map->map[j][k][i];
    else if ((fast == 1) && (medium == 2) && (slow == 0))
    for (j = 0; j < map->ext[medium]; ++j)
      for (k = 0; k < map->ext[fast]; ++k)
	*(plane++) = map->map[i][k][j];
    else if ((fast == 2) && (medium == 1) && (slow == 0))
    for (j = 0; j < map->ext[medium]; ++j)
      for (k = 0; k < map->ext[fast]; ++k)
	*(plane++) = map->map[i][j][k];
    else
    for (j = 0; j < map->ext[medium]; ++j)
      for (k = 0; k < map->ext[fast]; ++k)
	*(plane++) = map->map[j][i][k];
    ccp4_cmap_write_section(mapfile, buff);}
  free(buff);
  ccp4_cmap_close(mapfile);
  ShutUp(FALSE);
}

/*****************************************************************************/

#if defined (PROTOTYPE)
  void ShutUp(BOOLEAN gag)
#else
  void ShutUp(gag)
  BOOLEAN gag;
#endif
/* Impedes output on stdout if `gag == TRUE`, or allows output on stdout
   if `gag == FALSE`. */

{ static FILE stdout_save, trash;
  static BOOLEAN first = TRUE;

  if (verbose == FALSE)
  { fflush(stdout);
    if (first)
    { stdout_save = *stdout;
#if defined(unix) || defined(__unix) || defined(__unix__) || defined (__APPLE__)
      trash = *fopen("/dev/null", "w"); 
#elif defined(WIN32)
	  trash = *fopen ("NUL", "w");
#else
      trash = *fopen ("NL:", "w");
#endif
      first = FALSE;}
    if (gag)
      *stdout = trash;
    else
      *stdout = stdout_save;}}




