/*

     This code is distributed under the terms and conditions of the
     CCP4 Program Suite Licence Agreement as a CCP4 Application.
     A copy of the CCP4 licence can be obtained by writing to the
     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ccp4_sysdep.h>                   /* CCP4 machine dep. include file */
#include "solomon.h"
#include "cmsk_io.h"
#include "cmaplib.h"
#include "ccp4_parser.h"
#include "ccp4_general.h"

/* clamp the input to the specified range for NEWO_MASK */
#define CLAMP(v,l,h)	((v)<(l) ? (l) : (v) > (h) ? (h) : v)

extern BOOLEAN verbose;

static enum {O_MODE,CCP4_MODE} current_mask_format=CCP4_MODE;

/*****************************************************************************/

#if defined (PROTOTYPE)
  MASK *ReadMask(char *filename)
#else
  MASK *ReadMask(filename)
  char *filename;
#endif
/* Reads an O (new and old) or CCP4 mask-file and returns a pointer to the
   corresponding MASK. */

{ int i,j,k,l,m;
  FILE *file;
  MASK *mask;
  char mtstring[4];
#if !defined (PROTOTYPE)
  MASK *ReadOMask();
  MASK *ReadCCPMask();
#endif

/* Open mask file to read */
  file=fopen(filename,"r");
  if (file == NULL)
  { printf("Cannot open %s\n", filename);
    ccperror(1,"*** Program Terminating ***");}

/* Determine automatically what format mask is, proc. taken from library.c */
#if defined (__alpha) && defined (vms)
     (void) fflush (file);
#endif
  current_mask_format=O_MODE;
  if ((fseek (file, (size_t) (52*sizeof(float)), SEEK_SET) == 0)) {
    if (fread (mtstring, (size_t) sizeof(char), (size_t) 3, file) == 3) {
      mtstring[3] = '\0';
      if (strncasecmp(mtstring,"MAP",3) == 0) 
        current_mask_format=CCP4_MODE;
    }
  }

  if (current_mask_format == O_MODE) {
    printf("Test print OMODE set\n");
    mask=ReadOMask(filename); }
  else {
    printf("Test print CCP4MODE set\n");
    mask=ReadCCP4Mask(filename); }

  l=0,m=0;
  for (i=0;i<mask->ext[2];i++)
    for (j=0;j<mask->ext[1];j++)
      for (k=0;k<mask->ext[0];k++)
	l++,m+=mask->msk[k][j][i];
  printf(" Mask Stats %i %i\n",l,m);
  printf("  Origin %5i %5i %5i\n",mask->ori[0],mask->ori[1],mask->ori[2]);
  printf("  Extent %5i %5i %5i\n",mask->ext[0],mask->ext[1],mask->ext[2]);
  printf("  Grid   %5i %5i %5i\n",mask->grid[0],mask->grid[1],mask->grid[2]);
  printf("  Transformations %6i\n",mask->ntrans);
  printf("  Set grid points %6i\n",mask->nset);

  return (mask);
}


    
/******************************************************************************/

#if defined (PROTOTYPE)
  MASK *ReadOMask(char *filename)
#else
  MASK *ReadOMask(filename)
  char *filename;
#endif
/* Reads an O-style mask-file and returns a pointer to the corresponding
   MASK. */

{ int grid[3], ori[3], ext[3], i, j, k, status, done, type, comp;
  float cell[6];
  MASK *mask;
  FILE *file;
  char buffer[1024], *ch;
#if !defined (PROTOTYPE)
  MASK *MallocMask();
  void Uncompressed();
  void Compressed();
#endif

  file = fopen(filename, "r");
  if (file == NULL) { 
    printf("Cannot open %s\n", filename);
    fclose(file);
    ccperror(1,"*** Program Terminating ***");
  }
  fgets (buffer, 1024, file);
  ch = strtok(buffer, " \n");


/* The LATEST style mask with keywords compressed and uncompressed data 
   and NEW style mask with keywords and uncompressed data */
  if ((strncasecmp(ch,".MASK_INPUT",11)==0) || (strncasecmp(ch,"NEW_MASK",8)==0)) { 
    if (strncasecmp(ch,".NEW_MASK",11)==0)
      type = 1;
    else
      type = 0;
    comp = 0;

    done = 0;
    status = 0;
    while (!done) {
      if (fgets (buffer, 1024, file) == NULL) {
        fclose(file);
        return NULL;
      }

      ch = strtok(buffer, " \n");

      if (strncasecmp(ch, "cell",3) == 0) {
        for (i=0; i<6; i++) {
          cell[i] = atof (strtok(NULL, " "));
        }
        status |= 1;
      }
   
      else if (strncasecmp(ch, "extent",6) == 0) {
        for (i=0; i<3; i++) {
          ext[i] = atoi (strtok(NULL, " "));
        }
        status |= 2;
      }

      else if (strncasecmp(ch, "origin",6) == 0) {
        for (i=0; i<3; i++) {
          ori[i] = atoi (strtok(NULL, " "));
        }
        status |= 4;
      }

      else if (strncasecmp(ch, "grid",4) == 0) {
        for (i=0; i<3; i++) {
          grid[i] = atoi (strtok(NULL, " "));
        }
        status |= 8;
      }

      else {
        if ((strncasecmp(ch,"compressed",10)==0) || 
            (strncasecmp(ch,"explicit",8)==0) || (strncasecmp(ch,"map",3)==0)) {
          if (strncasecmp(ch,"compressed",10) ==0) 
            comp = 1;
          done = 1;
        }
      }
    }

    if (status != 15) {
      fprintf (stderr, "Missing OMASK header information, status = %d\n", 
                   status);
      fclose (file);
      ccperror(1,"*** Program Terminating ***");
    }

    mask = MallocMask(grid, ori, ext, cell);
    if (type == 1 || comp == 0) 
      Uncompressed(mask, file);
    else
      Compressed(mask, file);
  }


/* COMPRESSED style mask with compressed data but with no keywords */
  else if (strncasecmp(ch,"COMPRESSED_MASK",15) == 0) { 
    i = 0;
    j=sscanf(fgets(buffer, 1024, file), " %d %d %d", ori + 0, ori + 1, ori + 2);
    i = i+j;
    printf(" test print ori %d\n",i);
    j=sscanf(fgets(buffer, 1024, file), " %d %d %d", ext + 0, ext + 1, ext + 2);
    i = i+j;
    printf(" test print ext %d\n",i);
    j=sscanf(fgets(buffer, 1024, file), " %d %d %d", grid + 0, grid + 1, 
           grid + 2);
    i = i+j;
    printf(" test print grid %d\n",i);
    j=sscanf(fgets(buffer, 1024, file), " %f %f %f %f %f %f",
           cell + 0, cell + 1, cell + 2, cell + 3, cell + 4, cell + 5);
    i = i+j;
    printf(" test print cell %d\n",i);

    if (i != 15)
    { printf("ERROR: header information corrupted\n");
      fclose(file);
      ccperror(1,"*** Program Terminating ***");
    }
    mask = MallocMask(grid, ori, ext, cell);
    Compressed(mask, file);
  }


/* Old style mask with uncompressed data and no keywords */
  else { 
    i = 0;
    j=sscanf(fgets(buffer, 1024, file), " %d %d %d", ori + 0, ori + 1, ori + 2);
    i = i+j;
    j=sscanf(fgets(buffer, 1024, file), " %d %d %d", ext + 0, ext + 1, ext + 2);
    i = i+j;
    j=sscanf(fgets(buffer, 1024, file), " %d %d %d", grid + 0, grid + 1, 
           grid + 2);
    i = i+j;
    j=sscanf(fgets(buffer, 1024, file), " %f %f %f %f %f %f",
           cell + 0, cell + 1, cell + 2, cell + 3, cell + 4, cell + 5);
    i = i+j;
    if (i != 15)
    { printf("ERROR: header information corrupted");
      fclose(file);
      ccperror(1,"*** Program Terminating ***");
    }
    mask = MallocMask(grid, ori, ext, cell);
    Uncompressed(mask, file);
  }

  fclose(file);
  strcpy(mask->infile, filename);
  printf("test exiting form readomask\n");
  return(mask);
}



/******************************************************************************/

#if defined (PROTOTYPE)
  void Uncompressed(MASK *mask, FILE *file)
#else
  void Uncompressed(mask, file)
  MASK *mask;
  FILE *file;
#endif
/* Reads uncompressed O-style mask data and returns a pointer to the 
   corresponding MASK. */

{ int i, j, k;
  char buffer[1024];

  for (i = 0; i < mask->ext[2]; ++i)
    for (j = 0; j < mask->ext[1]; ++j)
      for (k = 0; k < mask->ext[0]; ++k)
	{ if (getc(file) == '\n')
	    getc(file);
	  mask->msk[k][j][i] = (Byte) (((char) getc(file)) == '1');
	  if (mask->msk[k][j][i] == 1)
	    ++mask->nset;}
  fclose(file);
}


/*****************************************************************************/

#if defined (PROTOTYPE)
  void Compressed (MASK *mask, FILE *file)
#else
  void Compressed(mask, file)
  MASK *mask;
  FILE *file;
#endif
/* Reads a new O-style mask-file and returns a pointer to the corresponding
   MASK. This mask format is called OMASK in MAMA. 960713 mok */

{ 
  int i, j, k, i1, i2;
  char buffer[1024], *ch;
  Byte *cp, *tmpmask;
  int status, size, ct, done;



  /* allocate memory for temporary mask */
  size = mask->ext[0]*mask->ext[1]*mask->ext[2];
  tmpmask = cp = (char *)calloc((size_t)size, sizeof (Byte)); 
  if (!cp) {
    fprintf (stderr, "Could not allocate memory\n");
    fclose(file);
    ccperror(1,"*** Program Terminating ***");
  }

  /* omask is sectioned xyz, and solomon masks zyx. Decode the omask file 
     in a temporary array and reslice it later */

  done = 0;
  ct = 0;
  while (! done) {

    if (fgets (buffer, 1024, file) == NULL) 
      break;

    if (strncasecmp(buffer, "END",3) == 0) {
      done = 1;
    } else {
      sscanf(buffer, "%d %d", &i1, &i2);
      if (i1 > i2) {
	i = i2;
	i2 = i1;
        i1 = i;
      }        
      if (i1 > 0) {
        CLAMP (i1, 1, size);
        CLAMP (i2, 1, size);
        for (i=i1-1; i < i2; i++) {
	  ct++;
          cp[i] = 1;
	}
      }
    }
  }
  fclose(file);

  printf("test print before resectioning\n");

  /* resection omask xyz til solomon zyx */
  cp = tmpmask;
  for (i = 0; i < mask->ext[2]; ++i)
    for (j = 0; j < mask->ext[1]; ++j)
      for (k = 0; k < mask->ext[0]; ++k) { 
	mask->msk[k][j][i] = *cp++;
	if (mask->msk[k][j][i] == 1)
	  ++mask->nset;
      }

  free (tmpmask);
}



/*****************************************************************************/

#if defined (PROTOTYPE)
  MASK *ReadCCP4Mask(char *maskname)
#else
  MASK *ReadCCP4Mask(maskname)
  char *maskname;
#endif
/* Read a CCP4 mask as 'maskname'. */

{ 
  int i, j, k, slow, medium, fast, ierr;
  int order[3], grid[3], ori[3], ext[3], spcgrp, mode;
  float cell[6], rms, mean, min, max;
  MASK *mask;
  Byte *data, *plane;
  FILE *maskfile;
  CMMFile *mapfile=NULL;
  char *filename=NULL;
  int tmp_array[3];
#if !defined (PROTOTYPE)
  void ShutUp();
  MASK *MallocMask();
#endif

  /* First read the header: */
  if (!(filename=getenv(maskname))) filename = maskname;
  mapfile = ccp4_cmap_open(filename, O_RDONLY);
  ShutUp(TRUE);
  ccp4_cmap_get_order(mapfile, order);
  fast = order[0] - 1;
  medium = order[1] - 1;
  slow = order[2] - 1;
  ccp4_cmap_get_dim(mapfile, tmp_array);
  ext[slow] = tmp_array[2];
  ext[medium] = tmp_array[1];
  ext[fast] = tmp_array[0];
  ccp4_cmap_get_origin(mapfile, tmp_array);
  ori[slow] = tmp_array[2];
  ori[medium] = tmp_array[1];
  ori[fast] = tmp_array[0];
  ccp4_cmap_get_cell(mapfile, cell);
  ccp4_cmap_get_grid(mapfile, grid);
  mask = MallocMask(grid, ori, ext, cell);
  strcpy(mask->infile, maskname);
  strncpy(mask->title,ccp4_cmap_get_title(mapfile),80U);  

  /* And read the mask... (This can be done in quite a number of ways 
     because of axis permutations) */
  data = (Byte *) malloc(ext[fast] * ext[medium] *
			  sizeof(Byte));
  for (i = 0; i < ext[slow]; ++i)
  { if (!(ierr = ccp4_cmap_read_section(mapfile, plane=data)))
    { printf("Error in reading %s", maskname);
      ccperror(1,"*** Program Terminating ***");}
    else if ((fast == 0) && (medium == 1) && (slow == 2))
      for (j = 0; j < ext[medium]; ++j)
	for (k = 0; k < ext[fast]; ++k)
	  mask->msk[k][j][i] = *(plane++);
    else if ((fast == 0) && (medium == 2) && (slow == 1))
      for (j = 0; j < ext[medium]; ++j)
	for (k = 0; k < ext[fast]; ++k)
	  mask->msk[k][i][j] = *(plane++);
    else if ((fast == 1) && (medium == 0) && (slow == 2))
      for (j = 0; j < ext[medium]; ++j)
	for (k = 0; k < ext[fast]; ++k)
	  mask->msk[j][k][i] = *(plane++);
    else if ((fast == 1) && (medium == 2) && (slow == 0))
      for (j = 0; j < ext[medium]; ++j)
	for (k = 0; k < ext[fast]; ++k)
	  mask->msk[i][k][j] = *(plane++);
    else if ((fast == 2) && (medium == 1) && (slow == 0))
      for (j = 0; j < ext[medium]; ++j)
	for (k = 0; k < ext[fast]; ++k)
	  mask->msk[i][j][k] = *(plane++);
    else if ((fast == 2) && (medium == 0) && (slow == 1))
      for (j = 0; j < ext[medium]; ++j)
	for (k = 0; k < ext[fast]; ++k)
	  mask->msk[j][i][k] = *(plane++);
    plane = data;
    for (j = 0; j < ext[medium]; ++j)
      for (k = 0; k < ext[fast]; ++k)
	if ( *(plane++) == 1 ) ++mask->nset;}
  ccp4_cmap_close(mapfile);
  ShutUp(FALSE);
  free(data);
  return(mask);}



/*****************************************************************************/

#if defined (PROTOTYPE)
  void WriteMask(MASK *mask)
#else
  void WriteMask(mask)
  MASK *mask;
#endif
/* Writes only CCP4 masks currently, routines have been kept for writing 
   O style masks */

{
#if !defined (PROTOTYPE)
  void WriteCCP4Mask();
#endif

   WriteCCP4Mask(mask);
}

/******************************************************************************/

#if defined (PROTOTYPE)
  void WriteOMask(MASK *mask)
#else
  void WriteOMask(mask)
  MASK *mask;
#endif
/* Writes a mask to disk as an O-type mask-file. */

{ int i, j, k, nc;
  FILE *file;

  file = fopen(mask->outfile, "w");
  if (file == NULL)
  { printf("Cannot open %s for writing!!\n", mask->outfile);
    ccperror(1,"*** Program Terminating ***");}
  fprintf(file, "%5d%5d%5d\n", mask->ori[0], mask->ori[1], mask->ori[2]);
  fprintf(file, "%5d%5d%5d\n", mask->ext[0], mask->ext[1], mask->ext[2]);
  fprintf(file, "%5d%5d%5d\n", mask->grid[0], mask->grid[1], mask->grid[2]);
  fprintf(file, "%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f\n", mask->cell[0],
          mask->cell[1], mask->cell[2], mask->cell[3], mask->cell[4],
          mask->cell[5]);
  for (i = 0, nc = 0; i < mask->ext[2]; ++i)
    for (j = 0; j < mask->ext[1]; ++j)
      for (k = 0; k < mask->ext[0]; ++k)
      { putc(' ', file);
        if (mask->msk[k][j][i] == 0)
          putc('0', file);
        else
          putc('1', file);
        if (++nc == 40)
        { putc('\n', file);
          nc = 0;}}
  if (nc != 0)
    putc('\n', file);}

/*****************************************************************************/

#if defined (PROTOTYPE)
  void WriteNewOMask(MASK *mask)
#else
  void WriteNewOMask(mask)
  MASK *mask;
#endif
/* Writes a mask to disk as a compressed O-type mask-file.  960713 mok */

{
  register int i, x, y, z;
  int i1st, l1, size;
  FILE *file;

  file = fopen(mask->outfile, "w");
  if (file == NULL) { 
    printf("Cannot open %s for writing!!\n", mask->outfile);
    fclose(file);
    ccperror(1,"*** Program Terminating ***");
  }

  fprintf (file, ".MASK_INPUT\n");
  fprintf (file, "! Created by solomon\n");
  fprintf (file, "ORIGIN %d %d %d\n", mask->ori[0], mask->ori[1], mask->ori[2]); 
  fprintf (file, "EXTENT %d %d %d\n", mask->ext[0], mask->ext[1], mask->ext[2]); 
  fprintf (file, "GRID %d %d %d\n", mask->grid[0], mask->grid[1], mask->grid[2]);
  fprintf (file, "CELL %.3f %.3f %.3f %.3f %.3f %.3f\n", 
	   mask->cell[0], mask->cell[1], mask->cell[2],
	   mask->cell[3], mask->cell[4], mask->cell[5]);
  fprintf (file, "COMPRESSED\n");

  l1 = 0;
  i1st = 0;
  i = 0;
  for (z = 0; z < mask->ext[2]; z++) {
    for (y = 0; y < mask->ext[1]; y++) {
      for (x = 0; x < mask->ext[0]; x++) {
	if (l1) {
	  if (!mask->msk[x][y][z]) {
	    fprintf (file, "%d %d\n", i1st, i);
	    l1 = 0;
	  }
	} else {
	  if (mask->msk[x][y][z]) {
	    l1 = 1;
	    i1st = i+1;
	  } 
	}
	i++;
      }
    }
  }

  /* Flush last stretch */
  if (l1)
    fprintf (file, "%d %d\n", i1st, mask->ext[0] * mask->ext[1] * mask->ext[2]);

  fprintf (file, "END\n");
  fclose (file);
}

/*****************************************************************************/

#if defined (PROTOTYPE)
  void WriteCCP4Mask(MASK *mask)
#else
  void WriteCCP4Mask(mask)
  MASK *mask;
#endif
/* Writes 'mask' as CCP4 mask 'maskname'. */

{ 
  int fast, medium, slow, i, j, k, tmp_array[3];
  char *filename=NULL;
  Byte *buff, *plane;
  CMMFile *mapfile;
#if !defined (PROTOTYPE)
  void ShutUp();
#endif

  int spcgrp=1,mode=0,order[3];

  order[0] = 3;
  order[1] = 1;
  order[2] = 2;

  if (mask->outfile[0] == '\0')
    return;
  fast = order[0] - 1;
  medium = order[1] - 1;
  slow = order[2] - 1;

  /* Set up the header of the new file */
  ShutUp(TRUE);
  if (!(filename=getenv(mask->outfile))) filename = mask->outfile;
  mapfile = ccp4_cmap_open(filename, O_WRONLY);
  ccp4_cmap_set_datamode(mapfile, mode);
  ccp4_cmap_set_cell(mapfile, mask->cell);
  ccp4_cmap_set_grid(mapfile, mask->grid);
  ccp4_cmap_set_order(mapfile, order);
  ccp4_cmap_set_spacegroup(mapfile, spcgrp);
  tmp_array[2] = mask->ext[slow];
  tmp_array[1] = mask->ext[medium];
  tmp_array[0] = mask->ext[fast];
  ccp4_cmap_set_dim(mapfile, tmp_array);
  tmp_array[2] = mask->ori[slow];
  tmp_array[1] = mask->ori[medium];
  tmp_array[0] = mask->ori[fast];
  ccp4_cmap_set_origin(mapfile, tmp_array);

  /* Write data to the mask (many alternatives due to permutations of the
     axes)... */
  buff = (Byte *) malloc(mask->ext[fast] * mask->ext[medium] * sizeof(Byte));
  for (i = 0; i < mask->ext[slow]; ++i)
  { plane = buff;
    if ((fast == 0) && (medium == 1) && (slow == 2))
    for (j = 0; j < mask->ext[medium]; ++j)
      for (k = 0; k < mask->ext[fast]; ++k)
	*(plane++) = mask->msk[k][j][i];
    else if ((fast == 0) && (medium == 2) && (slow == 1))
    for (j = 0; j < mask->ext[medium]; ++j)
      for (k = 0; k < mask->ext[fast]; ++k)
	*(plane++) = mask->msk[k][i][j];
    else if ((fast == 1) && (medium == 0) && (slow == 2))
    for (j = 0; j < mask->ext[medium]; ++j)
      for (k = 0; k < mask->ext[fast]; ++k)
	*(plane++) = mask->msk[j][k][i];
    else if ((fast == 1) && (medium == 2) && (slow == 0))
    for (j = 0; j < mask->ext[medium]; ++j)
      for (k = 0; k < mask->ext[fast]; ++k)
	*(plane++) = mask->msk[i][k][j];
    else if ((fast == 2) && (medium == 1) && (slow == 0))
    for (j = 0; j < mask->ext[medium]; ++j)
      for (k = 0; k < mask->ext[fast]; ++k)
	*(plane++) = mask->msk[i][j][k];
    else
    for (j = 0; j < mask->ext[medium]; ++j)
      for (k = 0; k < mask->ext[fast]; ++k)
	*(plane++) = mask->msk[j][i][k];
    ccp4_cmap_write_section(mapfile, buff);}
  free(buff);
  ccp4_cmap_close(mapfile);
  ShutUp(FALSE);}



/******************************************************************************/

#if defined (PROTOTYPE)
  MASK *MallocMask(int grid[], int ori[], int ext[], float cell[])
#else
  MASK *MallocMask(grid, ori, ext, cell)
  int grid[], ori[], ext[];
  float cell[];
#endif
/* Returns a pointer to an empty MASK with the requested origin and extent. */

{ int i, j, k;
  MASK *mask;
  Byte *data;

  mask = (MASK *) malloc(sizeof(MASK));
  for (i = 0; i < 3; ++i)
  { mask->grid[i] = grid[i];
    mask->ori[i] = ori[i];
    mask->ext[i] = ext[i];
    mask->cell[i] = cell[i];
    mask->cell[i + 3] = cell[i + 3];}
  mask->title[0] = '\0';
  mask->ntrans = 0;
  mask->nset = 0; 
  mask->trans = NULL;
  mask->parent = NULL;
  mask->infile[0] = mask->outfile[0] = '\0';
  mask->msk = (Byte ***) malloc((ext[0] * sizeof(Byte**)) +
                                (ext[0] * ext[1] * sizeof(Byte*)) +
				(ext[0] * ext[1] * ext[2] * sizeof(Byte)));
  if (mask->msk == NULL)
  { printf("Not enough space for a mask of size %d %d %d\n",
	   ext[0], ext[1], ext[2]);
    ccperror(1,"*** Program Terminating ***");}
  data = (Byte *) (mask->msk + ext[0] * (1 + ext[1]));
  for (i = 0; i < ext[0]; ++i)
  { mask->msk[i] =  (Byte **) mask->msk + ext[0] + i * ext[1];
    for (j = 0; j < ext[1]; ++j, data += ext[2])
    { mask->msk[i][j] = data;
      for (k = 0; k < ext[2]; ++k)
	data[k] = 0;}}
  return(mask);}
  


/******************************************************************************/

#if defined (PROTOTYPE)
  void FreeMask(MASK *mask)
#else
  void FreeMask(mask)
  MASK *mask;
#endif
/* Frees memory occupied by 'mask'. */

{ int i;

  if (mask->parent != NULL)
  { for (i = 0; mask->parent->trans[i].child != mask; ++i);
    mask->parent->trans[i].child = NULL;}
  free(mask->msk);
  if (mask->trans != NULL)
    free(mask->trans);
  free(mask);}



/******************************************************************************/

#if defined (PROTOTYPE)
  void PrintMask(MASK *mask)
#else
  void PrintMask(mask)
  MASK *mask;
#endif
/* Prints some information on the specified mask. */

{ int i, j, k, nset, mini, minj, mink, maxi, maxj, maxk, nmultiset;

  printf("\n\n*********************************************************\n\n");
  printf("infile:  %s\n", mask->infile);
  printf("outfile: %s\n", mask->outfile);
  printf("origin:  %5d%5d%5d\n", mask->ori[0], mask->ori[1], mask->ori[2]);
  printf("extent:  %5d%5d%5d\n", mask->ext[0], mask->ext[1], mask->ext[2]);
  printf("grid:    %5d%5d%5d\n", mask->grid[0], mask->grid[1], mask->grid[2]);
  printf("cell:    %10.4f%10.4f%10.4f%10.4f%10.4f%10.4f \n", mask->cell[0],
	  mask->cell[1], mask->cell[2], mask->cell[3], mask->cell[4],
	  mask->cell[5]);
  printf("set:     %d\n", mask->nset);
  mini = mask->ext[0];
  minj = mask->ext[1];
  mink = mask->ext[2];
  maxi = maxj = maxk = nset = nmultiset = 0;
  for (i = 0; i < mask->ext[0]; ++i)
    for (j = 0; j < mask->ext[1]; ++j)
      for (k = 0; k < mask->ext[2]; ++k)
	if (mask->msk[i][j][k] > 0)
	{ ++nset;
	  if (mask->msk[i][j][k] > 1)
	    ++nmultiset;
	  mini = MIN(mini, i);
	  minj = MIN(minj, j);
	  mink = MIN(mink, k);
	  maxi = MAX(maxi, i);
	  maxj = MAX(maxj, j);
	  maxk = MAX(maxk, k);}
  if (mask->nset == 0)
    printf("No points set in this mask!\n");
  else
  { printf("Number of points set: %d", nset);
    if (nmultiset > 0)
      printf(", of which %d are set more than once\n", nmultiset);
    else
      printf("\n");
    printf("Points set in mask: slow   %3d to %3d\n", 1 + mini, 1 + maxi);
    printf("                    medium %3d to %3d\n", 1 + minj, 1 + maxj);
    printf("                    fast   %3d to %3d\n", 1 + mink, 1 + maxk);}
  if (mask->ntrans == 1)
    printf("\nThere is 1 associated transformation:\n\n");
  else
    printf("\nThere are %d associated transformations:\n\n", mask->ntrans);
  for (i = 0; i < mask->ntrans; ++i)
  { if (mask->trans[i].outfile[0] != '\0')
      printf("  transform %s\n", mask->trans[i].outfile);
    for (j = 0; j < 9; j += 3)
      printf("    %15.8f%15.8f%15.8f\n", mask->trans[i].rotate[j], 
	     mask->trans[i].rotate[j + 1], mask->trans[i].rotate[j + 2]); 
    printf("       %15.8f%15.8f%15.8f\n\n", mask->trans[i].translate[0], 
	   mask->trans[i].translate[1], mask->trans[i].translate[2]);}}



/******************************************************************************/

#if defined (PROTOTYPE)
  MASK *ShrinkMask(MASK *mask)
#else
  MASK *ShrinkMask(mask)
  MASK *mask;
#endif
/* Shrinks the extent of a mask and updates its origin in order to save some 
   space. */

{ int min[3], max[3], nset, i, j, k;
  MASK *newmask;

  for (i = 0; i < 3; ++i)
  { min[i] = mask->ext[i];
    max[i] = 0;}
  nset = 0;
  for (i = 0; i < mask->ext[0]; ++i)
    for (j = 0; j < mask->ext[1]; ++j)
      for (k = 0; k < mask->ext[2]; ++k)
	if (mask->msk[i][j][k] > 0)
	{ ++nset;
	  min[0] = MIN(min[0], i);
	  min[1] = MIN(min[1], j);
	  min[2] = MIN(min[2], k);
	  max[0] = MAX(max[0], i);
	  max[1] = MAX(max[1], j);
	  max[2] = MAX(max[2], k);}
  if (nset == 0)
  { printf("Mask %s has no gridpoints set!!!\n", mask->outfile);
    printf("This is an indication that something is wrong. Think again!\n");
    ccperror(1,"*** Program Terminating ***");}
  for (i = 0; i < 3; ++i)
  { min[i] = MAX(0, min[i] - 1);
    max[i] = MIN(mask->ext[i] - 1, max[i] + 1);
    mask->ori[i] = mask->ori[i] + min[i];
    mask->ext[i] = 1 + max[i] - min[i];}
  newmask = MallocMask(mask->grid, mask->ori, mask->ext, mask->cell);
  for (i = min[0]; i <= max[0]; ++i)
    for (j = min[1]; j <= max[1]; ++j)
      for (k = min[2]; k <= max[2]; ++k)
        newmask->msk[i - min[0]][j - min[1]][k - min[2]] = mask->msk[i][j][k];
  free(mask->msk);
  mask->msk = newmask->msk;
  free(newmask);
  return(mask);}



/******************************************************************************/

#if defined (PROTOTYPE)
  void DumpMask(MASK *mask)
#else
  void DumpMask(mask)
  MASK *mask;
#endif
/* Dumps the contents of the mask to stdout. */

{ int i0, i1, i2;
  char buff[1024];

  for (i0 = 0; i0 < mask->ext[0]; ++i0)
  { (void) fgets(buff,1024,stdin);
    printf("\n");
    for (i1 = 0; i1 < mask->ext[1]; ++i1)
    { for (i2 = 0; i2 < mask->ext[2]; ++i2)
	printf("%c", mask->msk[i0][i1][i2] + '0');
      printf("\n");}}}



/******************************************************************************/

#if defined (PROTOTYPE)
  void ClearMask(MASK *mask)
#else
  void ClearMask(mask)
  MASK *mask;
#endif
/* Resets all gridpoints in 'mask' */

{ int i;
  Byte *data = mask->msk[0][0];

  for (i = (mask->ext[0] * mask->ext[1] * mask->ext[2]) - 1; i >= 0; --i)
    data[i] = 0;
  mask->nset = 0;}



/******************************************************************************/

#if defined (PROTOTYPE)
  void TestMask(MASK *mask, MASK *un_mask, int nset)
#else
  void TestMask(mask, un_mask, nset)
  MASK *mask, *un_mask;
  int nset;
#endif
/* Checks all the set gridpoints in 'mask' and resets them unless the value
   of the corresponding gridpoint in 'un_mask' is equal or higher than 'nset'. */

{ int *unext, *ori, *unori, *grid, *ext, i0, i1, i2, unit0, unit1, unit2;
  Byte ***mskdata, ***unmsk;

  unext = un_mask->ext;
  grid = un_mask->grid;
  unmsk = un_mask->msk;
  unori = un_mask->ori;
  ori = mask->ori;
  mskdata = mask->msk;
  ext = mask->ext;

  for (i0 = 0; i0 < ext[0]; ++i0)
  { unit0 = PMOD(ori[0] + i0, grid[0]);
    if ((unit0 < unext[0]) && (unit0 >= unori[0]))
    { for (i1 = 0; i1 < ext[1]; ++i1)
      { unit1 = PMOD(ori[1] + i1, grid[1]);
	if ((unit1 < unext[1]) && (unit1 >= unori[1]))
        { for (i2 = 0; i2 < ext[2]; ++i2)
          { unit2 = PMOD(ori[2] + i2, grid[2]);
	    if ((unit2 < unext[2]) && (unit2 >= unori[2]))
	      if ((mskdata[i0][i1][i2] == 1) &&
		  (unmsk[unit0][unit1][unit2] < nset))
	      { mskdata[i0][i1][i2] = 0;
		--mask->nset;}}}}}}}  




/******************************************************************************/

#if defined (PROTOTYPE)
  MASK *NotMask(MASK *mask)
#else
  MASK *NotMask(mask)
  MASK *mask;
#endif
/* Negates 'mask', turning 1's into 0's and v.v. Other values are kept. */

{ int i = mask->ext[0] * mask->ext[1] * mask->ext[2] - 1;
  Byte *msk = mask->msk[0][0];
  
  for ( ; i >= 0; --i)
    if (msk[i] == 0)
      msk[i] = 1;
    else if (msk[i] == 1)
      msk[i] = 0;
  mask->nset = mask->ext[0] * mask->ext[1] * mask->ext[2] - mask->nset;
  return(mask);}

/* Non-ANSI/POSIX procedures: */

#if !defined (VMS) && !defined (__APPLE__) && !defined (_AIX) && !defined (sun)

/* Case-insensitive comparison (BSD) */
#if defined (PROTOTYPE)
  int strncasecmp(const char *s1, const char *s2, size_t n)
#else
  int strncasecmp(s1, s2, n)
  char *s1;
  char *s2;
  size_t n;
#endif
{
  if (n == 0) return 0;
  while (n-- != 0 && ucase(*s1) == ucase(*s2)) {
      if (n == 0 || *s1 == '\0' || *s2 == '\0')
	break;
      s1++;
      s2++; }
  return (int) ucase(*(unsigned char *) s1) - (int) ucase(*(unsigned char *) s2);
}

#endif /* if !defined (VMS) */
