C
C     SORTMTZ
C     Copyright (C) 1977 Peter Daly
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
      PROGRAM SRTMTZ
C     ==============
C
C    Title: Sort a MTZ reflection data file - MTZSORT
C    
C    1. IDENTIFICATION_
C    
C       Program Name : MTZSORT
C    
C       Availability : VMS/UNIX - Fortran 77
C    
C       Orginator    : P. J. Daly
C     
C       Contact      : Daresbury Laboratory
C     
C       Date         : May, 1991
C     
C    2. ABSTRACT_
C    
C       The program  MTZSORT  is used to sort reflection data in a 
C       standard or multirecord type MTZ reflection data file on up
C       to 5 keys in either ascending or descending order. The keys
C       are specified in decreasing order of importance.
C    
C    3. DATA_CONTROL_CARDS
C    
C       When using the program control data is expected from unit 5 
C       in the FIXED order given below:
C    
C       'ASCEND' 'DESCEND'    sort in ascending or descending order.
C         '0'       '1'       You may use 0 (zero) for ascending and
C                             any other number for descending order. 
C                             This flag is OPTIONAL and will default
C                             to ASCEND if it is not given.
C    
C       SORT_KEYS             These are the labels for up to 5 sort 
C                             keys to  be used. Extra keys after the
C                             first five items will be ignored. This
C                             flag is OPTIONAL if HKLIN is assigned
C                             on the input and will default to H K L
C                             in this case. It is COMPULSORY if HKLIN
C                             is not assigned.
C
C       filename              The rest of the input data are the 
C          :                  filenames to be used for input - one 
C          :                  filename per line. If HKLIN is assigned
C          :                  on input then these lines are ignored.
C
C    4. INPUT_AND_OUTPUT_FILES
C 
C       The input files are the reflection data file(s) to be sorted in
C       standard MTZ format. If HKLIN is assigned then only this file
C       is used for input, otherwise the names of the input files are 
C       read as described in section 3.
C 
C       The output file is the sorted reflection data in MTZ format.
C 
C    5. PROGRAM_FUNCTION
C
C       SORTMTZ sort and merges together on or more input files to the
C       output file assigned to HKLOUT. If only one input file is to
C       be used then it may be assigned to HKLIN. Up to FIVE sort keys
C       can be specified in decreasing order of importance.
C
C    6. NOTES_
C 
C       Note_1: On Unix systems the sort procedure is called binsort
C               and is used in a similar manner to the sort/merge 
C               routines under VMS. The default memory size when
C               distributed is 1 Mbyte, you may want to see if this
C               has been increased for your system.
C 
C    7. PRINTER_OUTPUT
C 
C       The printer output gives the following information (not 
C       necessarily in the order given):
C 
C       a) The names of the sort keys
C       b) The sort order (ascending or descending)
C       c) Details from the opening of the input and output reflection 
C          data files
C       d) Details of the header information for each file opened.
C       e) The number of reflections read and written
C 
C    8. ERROR_MESSAGES
C
C       'SORTMTZ cannot find file: ' - You have given a filename
C       that the program could not find. The name of the file is
C       also printed.
C
C       'SORTMTZ cannot find some labels in MTZ file: ' - there is
C       a mismatch between the sort keys you have specified and the 
C       column labels found in the input file. It also displays a
C       list of sort keys and the position found or 'not found'.
C
C       'SORTMTZ has detected a different number of columns in this
C        file compared to the first file' - a mismatch between two
C        input files with different numbers of columns.
C
C       The messages below are errors detected from the return value
C       of the sort routines. They will prevent SORTMTZ from continuing
C       but the error should be sought in the sort routines themselves.
C       The all print the return value which caused the program to stop.
C
C       'SORTMTZ failed to initialise sort' - an error was detected
C       using the sort begin (SRTBEG) routine. A status number is also
C       given.
C
C       'SORTMTZ failed to release record to sort procedure' - an error
C       was detected in the sort release (SRTRLS) routine. 
C
C       'SORTMTZ detected failure in merge phase of sort procedure' - 
C       an error was detected using the sort merge (SRTMRG) routine.
C
C       'SORTMTZ detected error on obtaining record from sort procedure
C       in return phase' - an error was detected using the sort release
C       (SRTRET) routine. 
C 
C    9. FURTHER_INFORMATION
C
C       The program SORTMTZ was developed from the program SORTLCF
C       in a workshop in Cambridge in May, 1991. The move from LCF
C       to MTZ files is part of the cunning CCP4/EACBM master plan
C       to improve the software for protein crystallography.
C
C   10. EXAMPLES_UNDER_UNIX
C 
C       The examples below require the # to be in column 1 of the file
C
C       Example_1:
C
C       #!/bin/csh -f
C       sortmtz HKLOUT test_out.mtz << EOF-sortmtz
C       #
C       # Sort order - this flag is optional
C       #
C       ASCEND
C       # 
C       # Sort keys
C       #
C       H K L M/ISYM
C       # 
C       # Input files
C       #
C       test_in_1.mtz
C       test_in_2.mtz
C       EOF-sortmtz
C
C       Example_2:
C
C       #!/bin/csh -f
C       sortmtz HKLIN test_in.mtz HKLOUT test_out.mtz << EOF-sortmtz
C       # 
C       # Sort keys since default keys are H K L
C       #
C       H K L M/ISYM
C       EOF-sortmtz
C
C   11. EXAMPLES_UNDER_VMS
C
C       $ sortmtz HKLOUT test_out.mtz
C       $ go
C       #
C       # Sort order - this flag is optional
C       #
C       ASCEND
C       # 
C       # Sort keys
C       #
C       H K L M/ISYM
C       # 
C       # Input files
C       #
C       test_in_1.mtz
C       test_in_2.mtz
C       $
C
C       Example_2:
C
C       $ sortmtz HKLIN test_in.mtz HKLOUT test_out.mtz
C       $ go
C       #
C       # Sort keys since default keys are H K L
C       #
C       H K L M/ISYM
C       $
C
C     .. Parameters ..
      INTEGER     ENDSRT,MCOLS,NPARM,MAXKEY,MBATCH,SORTYP
      PARAMETER   (ENDSRT=-1,SORTYP=7,NPARM=200,MAXKEY=5,
     +             MCOLS=500,MBATCH=5000)
      INTEGER MSETS
      PARAMETER (MSETS=MCOLS)
      INTEGER MBLENG,CBLENG
      PARAMETER (MBLENG=185,CBLENG=70+3*8)
C     ..
C     .. Local Scalars ..
      INTEGER     I,INFILE,OUTFIL,ISTAT,J,J1,LASCND,KEYSIZ,LRECL,
     +            LUNIN,LUNOUT,N,NCOL,NCOL1,NFILE,
     +            NKEYS,NREFL,NTOT,IPRINT,NTOK,NBATX,BATNO,
     $            IFAIL, LFLAG, II, JJ, NSYM1, NSYM2
      LOGICAL     FIRST,ONEFIL,LEND,IFOUND,EOF,SETVAL
      CHARACTER   COL*80,FILNAM*255,KEY*4,LINE*400,VERSN*10
      REAL        DUMMY,VAL_MAGIC,VRSET_MAGIC, SRESOL
      REAL        RSYM1(4,4,192), RSYM2(4,4,192)
C     ..
C     .. Local Arrays ..
      INTEGER     IBEG(NPARM),IDEC(NPARM),IEND(NPARM),ITYP(NPARM),
     +            KEYS(MAXKEY),BATCHX(MBATCH)
      INTEGER     KEYBUF(51)
      LOGICAL     LOGMSS(MCOLS)
      CHARACTER   CTYPS(MCOLS),CTYPS1(MCOLS),CLABS(MCOLS)*30,
     +            CLABS1(MCOLS)*30,CVALUE(NPARM),CKEYS(MAXKEY)*30,
     +            CBATCH*(CBLENG)
      REAL        FVALUE(NPARM),ADATA(MCOLS),BDATA(MCOLS),
     +            RANGES(2,MCOLS),RBATCH(MBLENG)
      character*64 PROJECTNAME,CRYSTALNAME,DATASETNAME
      character dna_filename*80, dna_message*160
      INTEGER NDATASETS,ISETS(MSETS),BSETID
      REAL        DCELL(6,MSETS), DWAVEL(MSETS)
      CHARACTER*64 PNAME(MSETS),XNAME(MSETS),DNAME(MSETS)
C     ..
C     .. External Functions ..
      INTEGER     LENSTR,SRTBEG,SRTMRG,SRTRET,SRTRLS,
     +    CCP4SPG_F_EQUAL_OPS_ORDER
      EXTERNAL    LENSTR,SRTBEG,SRTMRG,SRTRET,SRTRLS,
     +    CCP4SPG_F_EQUAL_OPS_ORDER
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPERR,CCPFYP,CCPOPN,CCPUPC,CCPRCS,LRBAT,
     +         LRBATS,LRCLAB,LRCLOS,LRINFO,LROPEN,LRREFL,
     +         LRREFM,LWBAT,LWCLAB,LWCLOS,LWHSTL,LWOPEN,
     +         LWREFL,LWSORT,PARSER,RESET_MAGIC,SET_MAGIC,UGTENV,
     +         CCP4H_INIT,CCP4H_TOC_BEG,CCP4H_TOC_END, 
     +         CCP4h_TOC_ENT,CCP4H_PRE_BEG,CCP4H_PRE_END, 
     +         CCP4h_RULE,CCP4H_LINK,CCP4H_HEADER,
     +         CCP4H_SUMMARY_BEG,CCP4H_SUMMARY_END
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC   MIN
C     ..
C     .. Data statements ..
      DATA        LUNIN/5/,LUNOUT/6/,INFILE/1/,OUTFIL/1/,LEND/.FALSE./,
     +            IPRINT/1/,IFAIL/0/,NCOL/0/,NCOL1/0/,NBATX/0/,
     +            KEYS/MAXKEY*0/,NKEYS/MAXKEY/,KEYSIZ/4/,LASCND/0/,
     +            ONEFIL/.FALSE./,FILNAM/' '/,DUMMY/0.0/,
     +            VRSET_MAGIC/-999999.0/
C
      CALL CCPFYP
      CALL MTZINI

c     switch the DNA things on - based on the DNAOUT command line
c     assignment (the use of 'DNA' is to be phased out)
      call ugtenv('DNAOUT', dna_filename)
      if (dna_filename .eq. ' ') call ugtenv('DNA', dna_filename)
      if(dna_filename .ne. ' ') then
         call dna_start(dna_filename, 'sortmtz')
      else
         call dna_set_no_output
      end if




      call ccp4h_init()
      call ccp4h_header('SORTMTZ','sm',2)
      CALL CCPOPN(LUNIN,'DATA',5,1,0,0)
      CALL CCPOPN(LUNOUT,'PRINTER',6,1,0,0)
      CALL CCPRCS(LUNOUT,'SORTMTZ','$Date: 2005/09/06 11:25:27 $')
      IF (FILNAM.EQ.' ') CALL UGTENV('HKLIN',FILNAM)
      IF (FILNAM.NE.' ') ONEFIL = .TRUE.
C

      call dna_table_start('header_info')
      call dna_list_start('version_information')
      call dna_character_item('version', 
     +     '$Date: 2005/09/06 11:25:27 $')
      call dna_character_item('author', 
     +     'Copyright (C) 1977 Peter Daly')
      call dna_list_end
      call dna_table_end

      call ccp4h_pre_end() 
      call ccp4h_toc_beg()
      call ccp4h_summary_beg()
      call ccp4h_toc_ent('Command Input','#command')
      call ccp4h_toc_ent('Input File Details','#input')
      call ccp4h_toc_ent('Output File Details','#output')
      call ccp4h_summary_end()
      call ccp4h_toc_ent('Header Information for Output MTZ File'
     +         ,'#out')
      call ccp4h_toc_end()
      call ccp4h_rule()
      call ccp4h_summary_beg()
      call ccp4h_header('Command Input','command',3)
      call ccp4h_link('ASCEND/DESCEND','sortmtz.html#ascend')
      call ccp4h_link('SORT KEYS','sortmtz.html#sort_keys')
      call ccp4h_summary_end()
      call ccp4h_pre_beg() 
C
C---- Check for VRESET/ASCEND/DESCEND flag
C
 4420 NTOK = NPARM
      KEY  = ' '
      LINE = ' '
C
C---- Note: PARSER returns zero to FVALUE for strings
C
C     ============================================================
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,
     +            LEND,.TRUE.)
C     ============================================================
C
      IF (LEND) GO TO 4400
      CALL CCPUPC (KEY)
      IF (KEY.EQ.'ASCE' .OR. KEY.EQ.' ') THEN
        LASCND = 0
        GO TO 4420
      ELSE IF (KEY.EQ.'DESC') THEN
        LASCND = 1
        GO TO 4420
      ELSE IF (LGE(KEY(:1),'0') .AND. LLE(KEY(:1),'9')) THEN
        LASCND = NINT(FVALUE(1))
        IF (LASCND.NE.0) LASCND = 1
         GO TO 4420
      ELSE IF (KEY.EQ.'VRSE') THEN
         CALL GTTREA(2,VRSET_MAGIC,LFLAG,NTOK,ITYP,FVALUE)
         IF (LFLAG .NE. 0) THEN
            CALL CCPERR(1,'**** Illegal number on VRSET command ****')
         ENDIF
         GO TO 4420
      ENDIF
C
4400  CONTINUE
      IF (LEND) THEN
C
C---- Set up default sort keys of H K L and parse the line
C
        NTOK = NPARM
        LINE = 'H K L'
        CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,
     +              LEND,.TRUE.)
      ENDIF
      COL = LINE(IBEG(1):IEND(NTOK))
      NKEYS = MIN(NKEYS,NTOK)
      DO 5940 II=1,NKEYS
        CKEYS(II) = LINE(IBEG(II):IEND(II))
5940  CONTINUE        
C
      FIRST = .TRUE.
      SETVAL = .FALSE.
      NFILE = 0
      NTOT = 0
C
C---- Start of loop over files
C
   10 CONTINUE
      IF (ONEFIL) THEN
        CALL UGTENV('HKLIN',FILNAM)
        IF (FILNAM.EQ.' ') FILNAM = 'HKLIN'
      ELSE
        NTOK = NPARM
        KEY  = ' '
        LINE = ' '
C
C       ============================================================
        CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,
     +              LEND,.TRUE.)
C       ============================================================
C
        IF (LEND) GO TO 70
        FILNAM = LINE(IBEG(1):IEND(1))
C
C---- We need to reset the INFILE to 2 for all files other than the
C     first one since the library routines will overwrite the header
C     information and we will lose the history details.
C
        IF (.NOT. FIRST) INFILE = 2
      END IF
C
      call ccp4h_pre_end()
      call ccp4h_rule()
      call ccp4h_summary_beg()
      call ccp4h_header('Input File Details','input',3)
      call ccp4h_summary_end()
      call ccp4h_pre_beg()
C
C---- Open input file 
C
C     =======================================
      CALL LROPEN(INFILE,FILNAM,IPRINT,IFAIL)
C     =======================================
C
      IF (IFAIL.EQ.-1) THEN

 1001   format('cannot find input file ', a)
        write(dna_message, 1001) filnam(1:lenstr(filnam))
        call dna_error(dna_message(1:lenstr(dna_message)))
        call dna_end         

        CALL CCPERR(1, 'Can''t find file: '//FILNAM)
      ENDIF
C
C   sanity check - symmetry operators of the input files must be
C   in the same order, otherwise ISYM is corrupted
      IF (.NOT.ONEFIL) THEN
        IF (FIRST) THEN
          CALL LRSYMM(INFILE, NSYM1, RSYM1)
        ELSE
          CALL LRSYMM(INFILE, NSYM2, RSYM2)
          IF (CCP4SPG_F_EQUAL_OPS_ORDER(NSYM1, RSYM1, NSYM2, RSYM2)
     +           .EQ. 0) THEN
            WRITE(6,'(/,a,I4,a,/,a,/,a,/,a,/,a,/)')
     +        'File ',NFILE+1,' has symmetry operators in a ',
     +        'different order to the first file. The ISYM values ',
     +        'in the output file will therefore be wrong. It is ',
     +        'suggested to run the input files through REINDEX ',
     +        'with the SYMM keyword to get consistent ordering.'
            CALL CCPERR(1,
     +         'Inconsistent operator orders in input files.')
          ENDIF
        ENDIF
      ENDIF
C
C
C     ====================================
      CALL LRCLAB(INFILE,CLABS,CTYPS,NCOL)
C     ====================================
C
C---- Convert record length from items to bytes (assumes 4 byte reals)
C
      LRECL = NCOL*4
C
C---- Check to see if VRSET_MAGIC missing value flag is acceptable
C
C     ===========================================
      CALL LRINFO(INFILE,VERSN,NCOL,NREFL,RANGES)
C     ===========================================
C
      DO 20 I=1,NCOL
        IF (RANGES(1,I) .LT. VRSET_MAGIC) THEN

           call dna_error('data from file too negative')
           call dna_end


          WRITE(LUNOUT,'(A,I6,A,F10.1,/,A,I3,A,G15.4,/,A)')
     $        ' Data from file ',
     +        INFILE,' is more negative than ',VRSET_MAGIC,
     $        ' Minimum value in column ',I, ' is ', RANGES(1,I),
     +    ' You must either scale down the data or change VRSET_MAGIC'
          CALL CCPERR(1,' *** ERROR: Program Terminating')
        ENDIF
  20  CONTINUE
C
C---- Check the input sort keys against the column labels from the file
C
      IFOUND = .FALSE.
      DO 5930 II=1,NKEYS
        DO 5920 JJ=1,NCOL
          IF (CKEYS(II).EQ.CLABS(JJ)) KEYS(II) = JJ
5920    CONTINUE
        IF (KEYS(II).EQ.0) IFOUND = .TRUE.
5930  CONTINUE
C
C---- Write out any non-assigned values and exit
C
      IF (IFOUND) THEN
        WRITE (LUNOUT,5900)
5900    FORMAT (' SORTMTZ cannot find some labels in MTZ file: ',//)
        DO 5910 II=1,NKEYS
          IF (KEYS(II).EQ.0) THEN
            WRITE (LUNOUT,5911) CKEYS(II)
5911        FORMAT (' ',A30,' not found')
          ELSE
            WRITE (LUNOUT,5922) CKEYS(II),KEYS(II) 
5922        FORMAT (' ',A30,' found at ',I4)
          ENDIF
5910    CONTINUE

        call dna_error('bad input labels')
        call dna_end

        call ccperr(1,'Bad input labels')
      ENDIF
C
C---- Increment file count
C
      NFILE = NFILE + 1
C
C---- First file, set up sort
C
      IF (FIRST) THEN
        NCOL1 = NCOL
        DO 30 I = 1,NCOL
          CTYPS1(I) = CTYPS(I)
          CLABS1(I) = CLABS(I)
   30   CONTINUE
        DO 40 I = 1,NKEYS
          J = (I-1)*5 + 1
C
C---- Sort Type is REAL (SORTYP set up in PARAMETER statement)
C
          KEYBUF(J) = SORTYP
C
C---- Sort Order ascending/descending
C
          KEYBUF(J+1) = LASCND
C
C---- position of 1st byte in key (keysiz is the length of sortyp (i.e real))
C
          KEYBUF(J+2) = (KEYS(I)-1)*KEYSIZ
C
C---- keylength in units of SORTYP
C
CZ          KEYBUF(J+3) = KEYSIZ
          KEYBUF(J+3) = 1
C
C---- no mask
	  KEYBUF(J+4) = 0
   40   CONTINUE
        WRITE (LUNOUT,FMT=6006) NKEYS, (KEYS(I),I=1,NKEYS)
 6006   FORMAT (1X,I8,' sort keys, in columns ',30I4,/)
C
C---- Open output file 'HKLOUT' with same titles, etc.
C
        CALL UGTENV ('HKLOUT',FILNAM)
        IF (FILNAM.EQ.' ') FILNAM = 'HKLOUT'
C
C
      call ccp4h_pre_end()
      call ccp4h_rule()
      call ccp4h_summary_beg()
      call ccp4h_header('Output File Details','output',3)
      call ccp4h_summary_end()
      call ccp4h_pre_beg()
C
C       ==========================
        CALL LWOPEN(OUTFIL,FILNAM)
C       ==========================
C
C---- Output the sort keys
C
C       ========================
        CALL LWSORT(OUTFIL,KEYS)
C       ========================
C
C       ======================================
        CALL LWCLAB(OUTFIL,CLABS,NCOL,CTYPS,0)
C       ======================================
C
C
C
C---- Update header info with a line describing the sort
C
C       =================================
        CALL LWHSTL(OUTFIL, ' using keys: ' // COL)
C       =================================
C
C
C---- Initialise sort, set parameters, etc
C
CZ        ISTAT = SRTBEG(NKEYS,KEYBUF,LRECL,0,0)
        ISTAT = SRTBEG(NKEYS,KEYBUF,LRECL,0)
        IF (ISTAT.NE.0) WRITE (LUNOUT,FMT=6008) ISTAT
 6008   FORMAT (' SORTMTZ failed to initialise sort, status = ',I8,/)
C
C       =======================================
        CALL SET_MAGIC(INFILE,VAL_MAGIC,SETVAL)
C       =======================================
C
C---- For 2nd and subsequent files, check that it has same number 
C     of columns as first. Note that no check is done on the identity 
C     of these columns
C
      ELSE 
        IF (NCOL.NE.NCOL1) THEN
          WRITE (LUNOUT,6004)
 6004     FORMAT(' SORTMTZ has detected a different number of columns ',
     +            ' in this file compared to the first file')

          call dna_error('bad input columns - files do not match')
          call dna_end

          call ccperr(1,'Bad input columns')
        ENDIF
        DO 45 I=1,NCOL
          IF (CTYPS1(I) .NE. CTYPS(I)) then
             call dna_error('bad input columns - files do not match')
             call dna_end

             CALL CCPERR(1,
     +            ' *** ERROR: Column types ' //
     +            'do not match between files ***')
          end if
          IF (CLABS1(I) .NE. CLABS(I)) THEN
            J = LENSTR(CLABS(I))
            J1 = LENSTR(CLABS1(I))
            WRITE(6,6018)CLABS(I)(1:J),CLABS1(I)(1:J1)
 6018       FORMAT(/,' Column label from first file ',A,
     +      ' is not the same as that from second ',A,/)
          ENDIF
  45    CONTINUE
      END IF
C
      FIRST = .FALSE.
      N = 0
      IF (.NOT. ONEFIL) THEN

C     Notify available space
        NDATASETS = MSETS
C---- Transfer all datasets to output file
        CALL LRIDX(INFILE,PNAME,XNAME,DNAME,ISETS,
     +                        DCELL,DWAVEL,NDATASETS)
        DO I = 1,NDATASETS
          PROJECTNAME = PNAME(I)
          CRYSTALNAME = XNAME(I)
          DATASETNAME = DNAME(I)
          call lwidx
     +         (OUTFIL,PROJECTNAME,CRYSTALNAME,DATASETNAME,
     +          DCELL(1,I),DWAVEL(I))
        ENDDO

C       ================================
        CALL LRBATS(INFILE,NBATX,BATCHX)
C       ================================
C
        IF (NBATX.GT.0) THEN
          DO 4995 II=1,NBATX
C
C         =========================================
          CALL LRBAT (INFILE,BATNO,RBATCH,CBATCH,0)
C         =========================================
C
C---- Batch header that is transferred to HKLOUT contains old dataset ID
C     which might be out-of-date when merging several datasets
C     together. For each dataset, identify CRYSTALNAME,DATASETNAME
C     and pass these to LWBSETIDX which will write the correct ID.

          CRYSTALNAME = ' '
          DATASETNAME = ' '
C---- Called before LWBAT so BSETID read from input header.
          CALL LRBSETID (INFILE,BATNO,BSETID)
          DO I = 1,NDATASETS
            IF (ISETS(I).EQ.BSETID) THEN
              CRYSTALNAME = XNAME(I)
              DATASETNAME = DNAME(I)
              GOTO 4993
            ENDIF
          ENDDO

 4993     CONTINUE
C
C         =======================================
          CALL LWBAT (OUTFIL,BATNO,RBATCH,CBATCH)
C         =======================================

C---- Called after LWBAT so written to output header.
          IF (CRYSTALNAME.NE.' ')
     +       CALL LWBSETIDX(OUTFIL,BATNO,CRYSTALNAME,DATASETNAME)

4995      CONTINUE
        ENDIF
      ENDIF
   50 CONTINUE
C
C---- Read reflection record
C
C     ====================================
      CALL LRREFL(INFILE,SRESOL,BDATA,EOF)
C     ====================================
C
      IF (EOF) GO TO 60
C
C     ==========================
      CALL LRREFM(INFILE,LOGMSS)
C     ==========================
C
      N = N + 1
C
C---- Convert missing data from original (NaN?) to VRSET_MAGIC
C
C     ===========================================================
      CALL RESET_MAGIC(INFILE,BDATA,ADATA,NCOL,DUMMY,VRSET_MAGIC)
C     ===========================================================
C
C---- Pass keys and record to sort
C
      ISTAT = SRTRLS(ADATA)
      IF (ISTAT.EQ.0) THEN
        GO TO 50
      ELSE
        WRITE (LUNOUT,FMT=6010) ISTAT
 6010   FORMAT (' SORTMTZ failed to release record to sort procedure,',
     +          ' status = ',I9,/)

        call dna_error('failed to release record to sort')
        call dna_end



        call ccperr(1,'Sorting failed')
      END IF
C
C---- End of input file, loop back if required
C
   60 NTOT=NTOT+N
      IF (ONEFIL) GO TO 70
      WRITE (LUNOUT,FMT=6012) N,NFILE
 6012 FORMAT (3X,I8,' records read from file ',I4)
C
C     ===================
      CALL LRCLOS(INFILE)
C     ===================
C
C---- loop back for more input files
C
      GO TO 10
C
C---- All records passed, now sort them
C
   70 WRITE (LUNOUT,FMT=6001) NTOT
 6001 FORMAT (1X,I10,' records passed to sort')
C
C
C
      ISTAT = SRTMRG()
      IF (ISTAT.NE.0) THEN
        WRITE (LUNOUT,FMT=6014) ISTAT
 6014 FORMAT (' SORTMTZ detected failure in merge phase of sort',
     +        ' procedure, status = ',I9,/)

        call dna_error('sorting failed')
        call dna_end


        call ccperr(1,'Sorting failed')
      ELSE
        N = 0
C
C            ==================================
        CALL SET_MAGIC(OUTFIL,VAL_MAGIC,SETVAL)
C            ==================================
C
      call ccp4h_pre_end()
      call ccp4h_rule()
      call ccp4h_header('Header Information For Output MTZ File '
     $      ,'out',3)
      call ccp4h_pre_beg()
C
C
   80   CONTINUE
C
C---- Now pick up records and output
C
        ISTAT = SRTRET(ADATA)
        IF (ISTAT.EQ.ENDSRT) THEN
          GO TO 90
        ELSE IF (ISTAT.EQ.0) THEN
C
C---- Set missing number flag to VRSET_MAGIC then change to that of first 
C     input file before reflections written out
C
C         ==========================================================
          CALL RESET_MAGIC(0,ADATA,BDATA,NCOL,VRSET_MAGIC,VAL_MAGIC)
C         ==========================================================
C
C---- Write out reflection record
C
C         =========================
          CALL LWREFL(OUTFIL,BDATA)
C         =========================
C
          N = N + 1
          GO TO 80
        END IF
        WRITE (LUNOUT,FMT=6016) ISTAT
 6016   FORMAT (' SORTMTZ detected error on obtaining record from sort'
     +          ,' procedure in return phase, status = ',I4,/)

        call dna_error('sorting failed')
        call dna_end


        call ccperr(1, 'Sorting failed')
C
C---- Close sort and end file
C
   90   CONTINUE
C
C---- Close MTZ file but make sure correct MNF is set in header
C
C       ==========================
        CALL LWCLOS(OUTFIL,IPRINT)
C       ==========================
C
        WRITE (LUNOUT,FMT=6002) N
 6002   FORMAT (1X,I10,' records output')

      END IF

      call dna_table_start('status')
      call dna_list_start('message')
      call dna_integer_item('status', 0)
      call dna_character_item('message', 'normal termination')
      call dna_list_end
      call dna_table_end
      call dna_end



      call ccperr(0,'Normal termination')
      call ccp4h_pre_end()
      END











