C
C     STGRID
C     Copyright (C) 2001 Ian Tickle
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
      PROGRAM STGRID
C
C#######################################################################
C####
C#### Generation of Stereographic grid for measuring polar angles on a
C#### stereographic projection.
C####
C#### Ian J. Tickle, March 01.
C####
C#### Installation & usage instructions.
C####
C#### 1. Compile and link with PLOT84 & CCP4 routines, e.g.:
C####    f77 stgrid.f $CLIB/libccp4.a -o stgrid
C####
C#### 2. Run the program:
C####    stgrid
C####
C#### 3. Convert the output file to PostScript (resetting line width):
C####    pltdev -i PLOT -lnw .2
C####
C#### 4. Print the PostScript file plot84.ps on your laser printer.
C####
C#### 5. Copy the stereographic grid onto a transparent sheet.
C####
C#######################################################################
C
      IMPLICIT NONE
      REAL DR,PI
      PARAMETER (PI=3.141592654,DR=PI/180.)
      CHARACTER T*3
      INTEGER IP,IQ,IT
      REAL CQ,SQ,P,P1,P2,Q,R,R1,R2,V(2,2),W(2,2),X1,Y1
      DATA V/.05,.05,.95,.95/
      DATA W/-1.3,-1.3,1.3,1.3/
C
      CALL CCPFYP
C#### Commented out calls are for SIMPLEPLOT.
C      CALL NEWPIC
C      CALL PICSIZ(14.,14.)
C      CALL EQSCAL(-1.,1.,-1.,1.,0)
C      CALL TITLE7('H','C','Stereographic grid.')
C
C#### Plot calls for PLOT84 (CCP4).
      CALL PLTINI
      CALL PLTWIN(V,W)
      CALL PLTTNF(-1.12,0.,X1,Y1)
      CALL PLTCTX('+',X1,Y1,.5,3,0.)
      CALL PLTTNF(1.12,0.,X1,Y1)
      CALL PLTCTX('+',X1,Y1,.5,1,0.)
      CALL PLTTNF(0.,-1.24,X1,Y1)
      CALL PLTCTX('Stereographic grid.',X1,Y1,1.1,2,0.)
C
      DO 1 IQ=0,358,2
        Q=DR*IQ
        CQ=COS(Q)
        SQ=SIN(Q)
        IF (MOD(IQ,90).EQ.0) THEN
          P1=0.
          P2=2.
          IT=2
        ELSEIF (MOD(IQ,30).EQ.0) THEN
          P1=2.
          P2=4.
          IT=2
        ELSEIF (MOD(IQ,10).EQ.0) THEN
          P1=4.
          P2=20.
          IT=2
        ELSE
          P2=20.
          IT=1
        ENDIF
        IF (IT.GT.1) THEN
          P1=DR*P1
          R1=SIN(P1)/(1.+COS(P1))
          CALL MOVETOB(R1*CQ,R1*SQ,.FALSE.,0)
        ENDIF
        P2=DR*P2
        R2=SIN(P2)/(1.+COS(P2))
        CALL MOVETOB(R2*CQ,R2*SQ,.TRUE.,IT-1)
 1      CALL MOVETOB(CQ,SQ,.TRUE.,IT)
C
      DO 2 IP=2,90,2
        P=DR*IP
        R=SIN(P)/(1.+COS(P))
        IF (MOD(IP,10).EQ.0) THEN
          IT=2
        ELSE
          IT=1
        ENDIF
        DO 2 IQ=0,360
          Q=DR*IQ
          CQ=COS(Q)
          SQ=SIN(Q)
 2        CALL MOVETOB(R*CQ,R*SQ,IQ.GT.0,IT)
C
      DO 3 IQ=0,350,10
        Q=DR*IQ
        CALL PLTTNF(1.04*COS(Q),1.04*SIN(Q),X1,Y1)
C        CALL NUMBLB(1.04*COS(Q)-.025,1.04*SIN(Q)-.025,IT)
        WRITE (T,'(I3)') IQ
 3      CALL PLTCTX(T,X1+.01,Y1,.5,3,0.)
C
C      CALL ENDPLT
      CALL PLTSTP
      END
C
C
      SUBROUTINE MOVETOB(X,Y,L,N)
      IMPLICIT NONE
      LOGICAL L
      INTEGER I,N
      REAL D,DX,DY,X,Y,X0,Y0,X1,Y1
      SAVE X0,Y0
      DATA X0,Y0 /2*0./
C
      D=(X-X0)**2+(Y-Y0)**2
      
      IF (.NOT.L .OR. N.LE.0) THEN
C        CALL BREAK
C        CALL JOINPT(X,Y)
        CALL PLTTNF(X,Y,X1,Y1)
        CALL PLTMOV(X1,Y1)
      ELSEIF (N.EQ.1) THEN
C        CALL JOINPT(X,Y)
        CALL PLTTNF(X,Y,X1,Y1)
        CALL PLTDRW(X1,Y1)
      ELSEIF (D.GT.0.) THEN
        D=.001*SQRT(1./D)
        DX=-D*(Y-Y0)
        DY=D*(X-X0)
        DO 1 I=1-N,N-1
C          CALL BREAK
C          CALL JOINPT(X0+I*DX,Y0+I*DY)
C1         CALL JOINPT(X+I*DX,Y+I*DY)
          CALL PLTTNF(X0+I*DX,Y0+I*DY,X1,Y1)
          CALL PLTMOV(X1,Y1)
          CALL PLTTNF(X+I*DX,Y+I*DY,X1,Y1)
1         CALL PLTDRW(X1,Y1)
      ENDIF
      X0=X
      Y0=Y
      END
