C
C     UNIQUE
C     Copyright (C) 1984 Andrew Leslie
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
C
C                       PROGRAM UNIQUE
C
C  Calculates how many unique reflections occur within
C  given resolution limits.
C                                       A.G.W. Leslie Nov 1984
C
C
C*************************************************************
C
C    Program Converted to MTZ from LCF data file format
C
C    Date: Sun Jul 28 22:03:40 BST 1991
C    Revised:   Sun Feb  9 20:59:06 GMT 1992
C
C   The development of the MTZ files and associated software mark 1
C   is part of the masterplan of the ESF/EACBM Working Group 2.1 for
C   better Protein Crystallographic software for Europe.
C
C**************************************************************
C
C     Mon Feb 22 15:03:49 GMT 1993        Phil Evans MRC LMB Cambridge
C     Made it work properly for high symmetry, all symmetry control now
C     from SYMMETRY command, from which Laue group is calculated.
C     Now consistent with other CCP4 programs (calls ASUSET, INASU)
C
C  Converted to MTZ 5/92 H.Terry
C  and keyworded (added subroutine KEYIN)
C  In oscgen option the output columns do not contain S and BATCH
C  has been moved to column 4 so that we get
C           H K L BATCH IC PHI PACK   - IC is the centric flag
C
C     Dave Love, Nov. '95:  FIXME: Comments below are suspect.  The
C     oscgen stuff should now be obsolete and ought to be removed, as it
C     has been in the man page.  Missing data now default to NaNs.
C
C  OUTPUT:
C    An MTZ file containing the unique reflections
C    column headings will be "H,K,L" plus those given on card 2),
C    unless this is left blank, in which case they will be :
C    "H,K,L,A,BATCH,PHI,S,PACK"
C    where PHI,and PACK are set to zero, BATCH is -999, a is zero
C    for centric reflections and 1 for acentric. This is to enable
C    the number of anomalous pairs produced by OSCGEN "UNIQUE" option
C    to be calculated.
C    S is 10000*(4*(sintheta/lambda)**2)
C These are the appropriate columns for use with an MTZ file produced
C    by the "UNIQUE" option of "OSCGEN".
C
C       The MTZ file will have the title given on card1 of ft05 input
C
C
C
C
C         implicit none
C
C
C     .. Parameters ..
      INTEGER NPARM
      PARAMETER (NPARM=200)
      INTEGER MCOLS
      PARAMETER (MCOLS=15)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=96)
      INTEGER NCOUT1
      PARAMETER (NCOUT1=5)
      INTEGER NCOUT2
      PARAMETER (NCOUT2=7)
C     ..
C     .. Scalars in Common ..
      REAL DEFAULT,RESMAX,RESMIN,SMAX,SMIN
      INTEGER LBOTKN,NLAUE,NSPGRP,NSYM,NSYMP
      LOGICAL KYLABO,KYTITL,OSCGEN
      CHARACTER LATTC*1,NAMSPG*10,PGNAME*10,TITOUT*72,LBOLIN*400
C     ..
C     .. Arrays in Common ..
      REAL CELL,RABC,RABCAN,RABCSQ,RANGD,RSYM
      INTEGER LBOBEG,LBOEND
C     ..
C     .. Local Scalars ..
      REAL EPS,SINTHL
      INTEGER HMAX,HMIN,I,ICENT,IH,IK,IL,IPRINT,ISYSAB,J,KMAX,KMIN,
     +        LMAX,LMIN,MTZOUT,NIN,NLPRGO,NOUT,NREJS
      CHARACTER MTZNAM*255
C     ..
C     .. Local Arrays ..
      REAL ADATA(MCOLS)
      INTEGER IHKL(3),ISORT(5)
      CHARACTER CTPRG(MCOLS)*1,CTPRG1(MCOLS)*1,CTPRG2(MCOLS)*1,
     +          LBOUT(MCOLS)*30,LBOUT1(MCOLS)*30,LBOUT2(MCOLS)*30
C     ..
C     .. External Functions ..
      REAL LSTLSQ
      INTEGER LENSTR,INASU
      EXTERNAL LSTLSQ,LENSTR,INASU
C     ..
C     .. External Subroutines ..
      EXTERNAL ASUSET,CCPDPN,CCPERR,CCPFYP,CCPRCS,CENTR,EPSLON,
     +        KEYIN,LKYOUT,LSTRSL,LWASSN,LWCELL,LWCLOS,LWOPEN,
     +         LWREFL,LWSYMM,LWTITL,MTZINI
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC REAL
C     ..
C     .. Common blocks ..
      COMMON /ABSSYM/NSPGRP,NSYM,NSYMP,NLAUE,RSYM(4,4,MAXSYM)
      COMMON /CELPAR/CELL(6),RABCSQ(3),RABCAN(3),RABC(3),RANGD(3)
      COMMON /CHRMTZ/LBOLIN
      COMMON /CHRSYM/PGNAME,NAMSPG,LATTC
      COMMON /CHRTIT/TITOUT
      COMMON /LAB/LBOBEG(NPARM),LBOEND(NPARM),LBOTKN
      COMMON /LOGS/KYTITL,KYLABO,OSCGEN
      COMMON /RESOLN/RESMIN,RESMAX,SMIN,SMAX,DEFAULT
C     ..
C     .. Data statements ..
      DATA (LBOUT1(J),J=1,NCOUT1)/'H','K','L','F','SIGF'/
      DATA (CTPRG1(J),J=1,NCOUT1)/'H','H','H','F','Q'/
      DATA (LBOUT2(J),J=1,NCOUT2)/'H','K','L','BATCH','IC','PHI','PACK'/
      DATA (CTPRG2(J),J=1,NCOUT2)/'H','H','H','B','I','R','I'/
C     ..
C
C
      MTZOUT = 1
      NIN = 0
      NOUT = 0
C
C          ************************************************
      CALL CCPFYP
      CALL MTZINI
      CALL CCPDPN(5,'DATA','READONLY','F',0,0)
      CALL CCPDPN(6,'PRINTER','PRINTER','F',0,0)
      CALL CCPRCS(6,'UNIQUE','$Date: 2007/03/16 14:38:03 $')
C          ************************************************
      CALL KEYIN
C
C     Dave Love, Nov. '95:  FIXME: The following is no longer true.
C     All missing data now get the DEFAULT value (should now be NaN),
C     not zero for all but column 4 as previously.  The oscgen relic
C     stuff should be expurgated.
C     
C----   Set column 4 (F) or 5 (BATCH) to -999 to allow UNIQUE
C       reflections to be distinguished from measured reflections
C       or those produced by OSCGEN, and  to ensure that when UNIQUE
C       MTZ file is merged with any other MTZ file,  after sorting
C       on the first five columns the UNIQUE reflection record
C       always comes before the true data record for any h,k,l
C
      DO 10 I = 1,MCOLS
        ADATA(I) = DEFAULT
   10 CONTINUE
C
C-- Set up No. of output Labels
C
      IF (OSCGEN) THEN
        NLPRGO = NCOUT2
C
C
        DO 20 I = 1,NLPRGO
          LBOUT(I) = LBOUT2(I)
          CTPRG(I) = CTPRG2(I)
   20   CONTINUE
C
C
      ELSE
        NLPRGO = NCOUT1
C
C
        DO 30 I = 1,NLPRGO
          LBOUT(I) = LBOUT1(I)
          CTPRG(I) = CTPRG1(I)
   30   CONTINUE
      END IF
C
C--- Open output reflection file
C
      MTZNAM = 'HKLOUT'
C
C          ***************************************
      CALL LWOPEN(MTZOUT,MTZNAM(1:LENSTR(MTZNAM)))
C          ***************************************
C
      IF (KYTITL) CALL LWTITL(MTZOUT,TITOUT,0)
C
C          *******************
      CALL LWCELL(MTZOUT,CELL)
C          *******************
C
C---- Set up the S calculation
C    and Set up symmetry (RDSYMM or LRSYMM must be called first )
C
C          *************************************************
      CALL LSTRSL(MTZOUT,CELL(1),CELL(2),CELL(3),CELL(4),CELL(5),
     +            CELL(6))
      CALL LWSYMM(MTZOUT,NSYM,NSYMP,RSYM,LATTC,NSPGRP,NAMSPG,PGNAME)
      CALL ASUSET(NAMSPG,NSPGRP,PGNAME,NSYM,RSYM,NSYMP,NLAUE,.FALSE.)
      CALL LKYOUT(MTZOUT,LBOUT,NLPRGO,LBOTKN,LBOLIN,LBOBEG,LBOEND)
      CALL LWASSN(MTZOUT,LBOUT,NLPRGO,CTPRG,0)
C          *************************************************
C
      WRITE (6,FMT=6000) RESMAX,RESMIN
 6000 FORMAT (/1X,'Maximum and minimum Bragg spacings',2F10.3,
     +       ' Angstroms',/)
C
C----  Calculate limits on H,K,L
C
      HMAX = INT(CELL(1)/RESMAX)
      KMAX = INT(CELL(2)/RESMAX)
      LMAX = INT(CELL(3)/RESMAX)
C
C          ******************************************
      CALL HKLRANGE(HMIN,HMAX,KMIN,KMAX,LMIN,LMAX)
C          ******************************************
C
      WRITE (6,FMT=6002) HMIN,HMAX,KMIN,KMAX,LMIN,LMAX
 6002 FORMAT (/1X,'Limits on H,K,L..',3 (I4,' to',I4,3X),/)

C     Write sort order to header.
C     If for any bizarre reason the 70, 60, 50 loops
C     are changed then this will have to be changed too.
      ISORT(1) = 1
      ISORT(2) = 2
      ISORT(3) = 3
      ISORT(4) = 0
      ISORT(5) = 0
      CALL LWSORT(MTZOUT,ISORT)
C
C----  Loop over reflection limits
C
      NREJS = 0
C
C
      DO 70 IH = HMIN,HMAX
        DO 60 IK = KMIN,KMAX
          DO 50 IL = LMIN,LMAX
            NIN = NIN + 1
C
C----  Test for limiting conditions for indices
C
            IHKL(1) = IH
            IHKL(2) = IK
            IHKL(3) = IL
C
C
            IF (IH.EQ.0 .AND. IK.EQ.0 .AND. IL.EQ.0) GO TO 50
C
C   INASU returns +1 if reflection is in bounds
C                *********************
            IF (INASU(IHKL, NLAUE) .LE. 0) GO TO 50
C                *********************
C
C
C---- EPS = multiplicity of this reflection.
C     ISYSAB = 1 - systematic absence
C
            ISYSAB = 0
C
C                ***********************
            CALL EPSLON(IHKL,EPS,ISYSAB)
C                ***********************
C
C---- Test for systematic absences
            IF (ISYSAB .EQ. 1) GO TO 50
C
C----  Calculate resolution
C
C                    **********************
            SINTHL = LSTLSQ(MTZOUT,IH,IK,IL)*4.0
C                    **********************
C
            IF ((SINTHL.LT.SMIN) .OR. (SINTHL.GT.SMAX)) THEN
              NREJS = NREJS + 1
              GO TO 40
            END IF
C
C
            ADATA(1) = REAL(IHKL(1))
            ADATA(2) = REAL(IHKL(2))
            ADATA(3) = REAL(IHKL(3))
C
C
            IF (OSCGEN) THEN
              ADATA(5) = 1.0
C
C                  *****************
              CALL CENTR(IHKL,ICENT)
C                  *****************
C
              IF (ICENT.EQ.1) ADATA(5) = 0.0
            END IF
C
C                ********************
            CALL LWREFL(MTZOUT,ADATA)
C                ********************
C
            NOUT = NOUT + 1
   40       CONTINUE
C
   50     CONTINUE
   60   CONTINUE
   70 CONTINUE
C
C
C----  End reflection loop
C
      IPRINT = 1
C
C          *********************
      CALL LWCLOS(MTZOUT,IPRINT)
C          *********************
C
      WRITE (6,FMT=6004) NIN,NOUT
 6004 FORMAT (//1X,I10,' reflections tested',/,1X,I10,
     $  ' reflections within resolution limits written to output file')
C
C          *******************************
      CALL CCPERR(0, 'Normal Termination')
C          *******************************
C
      END
C
C
      SUBROUTINE KEYIN
C     ================
C
C           implicit none
C
C
C---- Read key-worded input
C
C     .. Parameters ..
      INTEGER NPARM
      PARAMETER (NPARM=200)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=96)
C     ..
C     .. Scalars in Common ..
      REAL DEFAULT,RESMAX,RESMIN,SMAX,SMIN
      INTEGER LBOTKN,NLAUE,NSPGRP,NSYM,NSYMP
      LOGICAL KYLABO,KYTITL,OSCGEN
      CHARACTER LATTC*1,NAMSPG*10,PGNAME*10,TITOUT*72,LBOLIN*400
C     ..
C     .. Arrays in Common ..
      REAL CELL,RABC,RABCAN,RABCSQ,RANGD,RSYM
      INTEGER LBOBEG,LBOEND
C     ..
C     .. Local Scalars ..
      INTEGER IEOL,IPRINT,IWORK,JDO,NTOK
      LOGICAL KYCELL,KYRESO,KYSYMM,LEND
      CHARACTER KEY*4,LINE*400
C     ..
C     .. Local Arrays ..
      REAL FVALUE(NPARM)
      INTEGER IBEG(NPARM),IDEC(NPARM),IEND(NPARM),ITYP(NPARM)
      CHARACTER CVALUE(NPARM)*4
C     ..
C     .. External Subroutines ..
      EXTERNAL ASUSET,CCPERR,CCPUPC,CENTRIC,EPSLN,KEYNUM,PARSER,RDCELL,
     +     RDRESO,RDSYMM, QNAN
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC NINT
C     ..
C     .. Common blocks ..
      COMMON /ABSSYM/NSPGRP,NSYM,NSYMP,NLAUE,RSYM(4,4,MAXSYM)
      COMMON /CELPAR/CELL(6),RABCSQ(3),RABCAN(3),RABC(3),RANGD(3)
      COMMON /CHRMTZ/LBOLIN
      COMMON /CHRSYM/PGNAME,NAMSPG,LATTC
      COMMON /CHRTIT/TITOUT
      COMMON /LAB/LBOBEG(NPARM),LBOEND(NPARM),LBOTKN
      COMMON /LOGS/KYTITL,KYLABO,OSCGEN
      COMMON /RESOLN/RESMIN,RESMAX,SMIN,SMAX,DEFAULT
C     ..
C     .. Save statement ..
      SAVE
C     ..
      CALL QNAN (DEFAULT)
      OSCGEN = .FALSE.
      KYCELL = .FALSE.
      KYRESO = .FALSE.
      KYSYMM = .FALSE.
      KYTITL = .FALSE.
      KYLABO = .FALSE.
   10 CONTINUE
C
C
      NTOK = NPARM
      KEY = ' '
      LINE = ' '
C
C
C          ************************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +            .TRUE.)
C          ************************************************************
C
C---- End of file?
C
      IF (LEND) GO TO 30
C
C---- Get chars of 1st token
C
      KEY(1:4) = LINE(IBEG(1) :IEND(1))
C
C---- Comment !(Extra check also in Parse)
C
      IF (KEY(1:1).EQ.'!') GO TO 10
      IF (KEY(1:1).EQ.'#') GO TO 10
C
C---- Convert to Uppercase
C
C          ***********
      CALL CCPUPC(KEY)
C          ***********
C
C  Possible key-words are : CELL  SYMMETRY  RESOLUTION (all essential)
C                     and   TITLE  LABOUT  DEFAULT RUN (or GO or END) 
C                     and   OSCGEN
C
C     ========
C---- SYMMETRY
C     ========
C
C
      IF (KEY.EQ.'SYMM') THEN
        IWORK = 2
C
C---- we need to set up centric and epsilon tests
C
C            *********************************************************
        CALL RDSYMM(IWORK,LINE,IBEG,IEND,ITYP,FVALUE,NTOK,NAMSPG,NSPGRP,
     +              PGNAME,NSYM,NSYMP,RSYM)
        CALL CENTRIC(NSYM,RSYM,IPRINT)
        CALL EPSLN(NSYM,NSYMP,RSYM,IPRINT)
C            *********************************************************
C
        KYSYMM = .TRUE.
        GO TO 10
C
C     =====
C---- TITLE
C     =====
C
      ELSE IF (KEY.EQ.'TITL') THEN
        IWORK = 2
        IEOL = 72
C
        TITOUT = ' '
        TITOUT = LINE(IBEG(IWORK) :IEOL)
        KYTITL = .TRUE.
C
        GO TO 10
C
C
C     ====
C---- LAUE - No longer used
C     ====
C
      ELSE IF (KEY.EQ.'LAUE') THEN
         WRITE (6, '(/A/)')
     $        ' **** Laue group will be taken from SYMMETRY ****'
C
        GO TO 10
C
C     ======
C---- OSCGEN - To use UNIQUE with the OSCGEN option
C     ======
C
      ELSE IF (KEY.EQ.'OSCG') THEN
        OSCGEN = .TRUE.
C
        GO TO 10
C
C     =====
C---- CELL
C     =====
C
      ELSE IF (KEY.EQ.'CELL') THEN

C
C---- Must be at least 4 tokens on line
C
        IF (NTOK.LT.4) CALL CCPERR(1,
     +                     ' Error for Key_Word CELL, Not enough tokens'
     +                             //' on line')
C
C--- Are angles given - if not default to 90.00 degrees
C
        IF (NTOK.LE.7) THEN
          IWORK = 2
C
C              ***********************************
          CALL RDCELL(IWORK,ITYP,FVALUE,NTOK,CELL)
C              ***********************************
C
C
        END IF
        KYCELL = .TRUE.
        GO TO 10
C
C     ==========
C---- RESOLUTION
C     ==========
C
      ELSE IF (KEY.EQ.'RESO') THEN

C
C---- Must be at least 2 tokens on line
C
        IF (NTOK.LT.2) CALL CCPERR(1,
     +                          ' Error for Key_Word RESOL, Not enough '
     +                             //'tokens on line')
C
C---- To satisfy EJD, we are trying to discourage specifying
C     a low resolution limit.
C
        IF (NTOK.GT.2) WRITE(6,'(/,A,/,A,/)')
     +         '    **** WARNING: it is recommended to specify only',
     +         '                  a high resolution limit.'
        IWORK = 2
C
C            ******************************************************
        CALL RDRESO(IWORK,ITYP,FVALUE,NTOK,RESMIN,RESMAX,SMIN,SMAX)
C            ******************************************************
C
        KYRESO = .TRUE.
        GO TO 10
C
C     ======
C---- LABOUT   - output labels
C     ======
C
      ELSE IF (KEY.EQ.'LABO') THEN
C
        KYLABO = .TRUE.
        LBOLIN = LINE
        LBOTKN = NTOK
C
C
        DO 20 JDO = 1,LBOTKN
          LBOBEG(JDO) = IBEG(JDO)
          LBOEND(JDO) = IEND(JDO)
   20   CONTINUE
        GO TO 10
C
C     ======
C---- DEFAULT F or I 
C     ======
C
      ELSE IF (KEY.EQ.'DEFA') THEN
C
        KEY = LINE(IBEG(2):IEND(2))
        CALL CCPUPC(KEY)
        IF(KEY.EQ.'NAN') THEN
          CALL QNAN (DEFAULT)
          WRITE (6,FMT='(a,F12.3)')
     +    ' **** Default value for I or F set to NaN'
        ELSE
          CALL GTPREA(2,DEFAULT,NTOK,ITYP,FVALUE)
          WRITE (6,FMT='(a,F12.3)')
     +    ' **** Default value for I or F set to ',DEFAULT
        END IF

        GO TO 10
C
C     =============
C---- RUN (GO, END)
C     =============
C
      ELSE IF (KEY(1:3).EQ.'RUN' .OR. KEY(1:2).EQ.'GO' .OR.
     +         KEY(1:3).EQ.'END') THEN
        GO TO 30
      END IF
C
C
   30 CONTINUE
C
C---- Check if compulsory input cards have been read in
C
      IF (KYSYMM) THEN
C
C---- Set up symmetry (RDSYMM must be called first )
C
        LATTC = ' '
C
C          *********************************************************
        CALL ASUSET(NAMSPG,NSPGRP,PGNAME,NSYM,RSYM,NSYMP,NLAUE,.TRUE.)
C          *********************************************************
C
        LATTC = NAMSPG(1:1)
        IF (LATTC.EQ.' ') CALL CCPERR(1,
     +                     ' Cannot get Lattice type from your symmetry'
     +                                //' input')
C
      ELSE
        CALL CCPERR(1,' INPUT DATA ERROR **** No SYMM card ****')
      END IF
C
      IF (.NOT.KYCELL) CALL CCPERR(1,
     +                        ' INPUT DATA ERROR **** No CELL card ****'
     +                             )
C
      IF (.NOT.KYRESO) CALL CCPERR(1,
     +                  ' INPUT DATA ERROR **** No RESOLUTION card ****'
     +                             )
C
C
      RETURN
      END
