C
C     WATERTIDY
C     Copyright (C) 1992 Eleanor Dodson
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C

      PROGRAM WATERTIDY

C   This program reads a distang DISTOUT file and uses that information
C to move the "water " coordinates to the same asymmetric unit as the 
C protein molecule.
C  I attempt to rename the waters sensibly; each water in the first shell
C is given the residue number of its likely protein hydrogen acceptor, 
C a chain ID related to the acceptor chain and a name
C constructed to give information about the acceptor name. 
C  See documentation for details.
C   Example of distang output:
C   Sym TX TY TZ code means the bond is formed between Atom I and 
C   Atom J after application of Symmetry operator 4 ( tabulated somewhere) 
C                               with unit cell shifts  TX*a, TY*b TZ*c.
C      Atom I  Atom J   Dij                Sym TX TY TZ     occ(i)*occ(j)
C  G  1N  C   G  1N  C   1.33135              4 1.-1. 0.          1.0000
C  G  1N  C   G  1CA C   0.76741              4 1.-1. 0.          1.0000
C  G  1N  C   G  1O  C   2.33184              4 1.-1. 0.          1.0000
C  G  1N  C   O104OW0N   2.90837              4 1.-1. 0.          1.0000
C  G  1N  C   O159OW0N   1.78858              4 1.-1. 0.          1.0000


C  The output file would rename 159OW0N as 1W01?  
C  where ? is the requested CHAINID for waters attached to CHAIN C.

C  Input file
C  ATOM      1  N   GLY C   1      20.134   9.748   8.052  1.00 59.38
C  ....
C  ATOM    159  OW0 WAT N 159  0   33.837  24.464   4.700  1.00 36.63   8
C
C  Output file
C  ATOM      1  N   GLY C   1      20.134   9.748   8.052  1.00 59.38
C  ....
C  ATOM    159  W01 WAT ?   1  0   33.837  24.464   4.700  1.00 36.63   8

C  Extend to move Victors atoms back to original asymmetric unit..

C  allow 40 atoms per residue set.
      PARAMETER(Nres2=9999,NAT=80,nres1=9999,nchns=20,nchnw=10)
      PARAMETER (NR2NAT=NRES2*NAT)
      CHARACTER*1 INSCOD,ALTCOD
      CHARACTER*4 atnm1,atnm2,tmpstr
      CHARACTER*4 RESNO,RESNAM,SEGID,ID
      CHARACTER*80 line1,title
      CHARACTER*10 pgname,namspg
      REAL dis(nres2,NAT), sumoc(nchns), bav(5), dissm(5)
      INTEGER iarr(Nres2,NAT), mchn(nchns,2), nres0(nchns),
     +     nwatsch(nchns), JARR(nres1,10), nwch(nchns+1), nbav(5)
      CHARACTER*11 SYMCOD(Nres2,NAT), SYMCD
      CHARACTER*1 chNID(nchns,2),CHNNAM,chn1,chn2,
     +     res2,res1,labw,wchnid(nchnw),acepid(10)
      REAL  b(6),RO(4,4),RF(4,4),RJUNK1(4,4),RJUNK2(4,4),rsym(4,4,192),
     +      rsymO(4,4,192)
C
C     .. Parameters ..
      INTEGER NKEYS,NPAR
      PARAMETER (NKEYS=9,NPAR=200)
C     ..
      CHARACTER LINE*400
      CHARACTER KEY*4,CVALUE(NPAR)*4,STAT*8,TYPE*2
      LOGICAL LEND
      REAL     FVALUE(NPAR)
      INTEGER IBEG(NPAR),IDEC(NPAR),IEND(NPAR),ITYP(NPAR)
      INTEGER IUNDIST,IZ,ixyzin,ixyzout
      REAL biso,EIGHTPI2
      CHARACTER KEYWRD(NKEYS)*4
C     ..
C
C  KEY WORDS
C
      DATA KEYwrd/'SYMM','CHNI','ACCE','WATI','OCCW','TITL',
     1 'SHEL','HBON','END' /
      DATA bav /5*0.0/,nbav/5*0/,dissm/5*0.0/,nhbond/4/
      DATA IDUP /0/, nwch/nchns*0.0,0.0/
      DATA EIGHTPI2 /78.956835/
C
C     initialise symcod - this used to crash with g77
C     DATA SYMCOD/nr2nat*' '/
      DO 950 J=1,NAT
        DO 900 I=1,Nres2
          SYMCOD(I,J) = ' '
          IARR(I,J) = 0
 900    CONTINUE
 950  CONTINUE

      CALL CCPFYP
      CALL CCPRCS(6,'WATERTIDY','$Date: 2006/01/31 17:05:54 $')
      CALL XYZINIT
      write(6,1000)
1000    format(
     1 ' * This program reads a DISTANG DISTOUT file listing distances',
     1 /,' between atoms in well characterised chains and Frodo ',/,
     1 ' output H2os after applying appropriate symmetry operations,',/, 
     1 ' plus the input PDB file used for DISTANG ',/,
     1   ' and uses these to generate a new list of H2os in PDB format',
     1 /,' with names which obey the following conventions ***',//,
     1 ' (Only one water shell can be handled in one pass.)',///)
       write(6,1100)
1100    format(
     1 ' 1) A H2o atom within a set distance of a protein O or N is ',
     1 /,' labelled  nres Oij CHNID  where ',
     1 ' nres is the protein residue number and ',//,
     1 ' CHNID is the single character  protein chain ID.')
       write(6,1001)
1001   format(/,
     1 ' Oij uses the number i (range 0-9)  defined thus - ',/,
     1 '       0 for N , 1 for O - 2 for OG OG1 -  3 for OD1 ND1 -',/,
     1 '       4 for OD2 ND2  - 5 for OE OE1 NE1 - 6 for OE2 NE2 -',/,
     1 '       7 for     NZ -   8 for OH OH1 NH1 - 9 for OH2 NH2.',//,
     1 ' ( Extension for other atom types if requested: ',/,
     1 '       0 for CA, 1 for CB- 2 for CG CG1 -  3 for CD1  -',/,
     1 '       4 for CD2 CD3  - 5 for CE CE1    -  6 for CE2 CE3 -',/,
     1 '       7 for     CZ -   8 for CH CH1     - 9 for CH2 CH3.)',/,
     1 ///,
     1 ' and the number j (range 0-3)  defined thus - ',/,
     1 ' 0 for 1st H2o per atom, 1 for 2nd , 2 for 3rd, 3 for 4th H2o',
     1 //,' 2) A H2o atom within a set distance of first shell water ',
     1 /,' is labelled  nres Oij WATOUTID  where ',
     1 ' nres is the water residue number ',/,
     1 ' WATOUTID is the single character CHNID ',
     1 'matched to the protein chain ID.',
     1 /,' Oi is the same as the first shell water, and j = 4-6',//,
     1  ' For the third shell Oij is similar except j has range 7-9.') 
       write(6,1200)
1200   format(//,
     1 /,'  There is no guarantee that the "j" number for one chain',
     1 /,'  will be the same as that for another.'
     1 //,'  A H2o atom  may be listed several times in the ',
     1 ' output list,',/,
     1 '  once for each  atom it is connected to.',//,
     1 ' The occupancy equals the input value for the closest contact ',
     1 /,' with all other occupancies set to some token value',
     1 /,' - (0.000 causes problems with other programs)'
     1 ,//,'  Any H2o atom not bonded to protein is output as input.'
     1 //,'  This list can then be sorted by the vax sort using the',
     1 ' following call',/,
     1 '  to give a list of H2os with residue numbers in same order',
     1 '  as the protein.'//)
      write(6,1002)
1002   format(
     1 ' $SORT/KEY=(POS:21,SIZE:6)/key=(pos:16,size:1)/ - ',/,
     1 ' key=(pos:15,size:1)  js:natwats2jnk.pdb natwats3.pdb  ')
      write(6,1003)
1003   format(//,
     1 ' ******************** WARNING!!! *****************************',
     1 /,
     1 ' * Post CCP4 V4.2, WATERTIDY _CANNOT_ read the log file',/,
     1 ' * output directly from DISTANG.',/,
     1 ' * Instead the OUTPUT DISTOUT option of DISTANG must be',/,
     1 ' * used, and the resulting DISTOUT file will be formatted',/,
     1 ' * appropriately for input into WATERTIDY.',/,
     1 ' *** Please consult the documentation for more information ***',
     1     //)

      TITLE = ' '
       acepid(1) ='N'
       acepid(2) ='O'
       naccep = 2
       labw='O'
       occw = 0.01
       isum=0
       ish=1
       ishell=0
       ichn=0
       ichn=0
       ichnw=0
       sumoc0=0.
       sumoc2=0.
       sumoc3=0.
       ro(1,1)=0.
       i22 = 0
C
 80   continue
      LINE = ' '
      KEY = ' '
      NTOK = NPAR
C
C          ***********************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +            .TRUE.)
C          ***********************************************************
C
C----  End of file?
C
      IF (LEND)  GO TO 800
C
C----  Loop over possible key-words
C
      DO 81 I = 1,NKEYS
        IF (KEY.EQ.KEYWRD(I)) GO TO 82
 81   CONTINUE
      CALL CCPERR(1, ' *** UNRECOGNISED KEY WORD ***')
C
 82   CONTINUE
      GO TO ( 100, 200, 300, 400, 500, 600, 700, 750, 800) I
C
c      DATA KEYwrd/'SYMM','CHNI','ACCE','WATI','OCCW','TITL',
c     1 'SHEL','HBON','END' /
C
 100  CONTINUE
c  Symmetry...
C
C**** SYMMETRY CARD
C READ SYMMETRY MATRICES
C ------------------------------
C----  SYMM
      ITOK = 2
C
C     ************************************************************
      CALL RDSYMM(ITOK,LINE,IBEG,IEND,ITYP,FVALUE,NTOK,NAMSPG,
     +     NSPGRP,PGNAME,NSYM,NSYMP,RSYM)
C     ************************************************************
C
      GO TO 80
C
C---- CHAIN ids
C
C  READ "HOST" CHAIN LABEL WATOUTID linked LABEL. Range resmin resmax
C
  200   CONTINUE
        ichn = ichn + 1
        if (ichn.gt.nchns)
     1       CALL CCPERR(1,
     +       ' *** Reset parameter for Chain limit - nchns=?? ***')
        chnid(ICHN,1)= LINE(IBEG(2) :ibeg(2))
         ITOK = 2
 201        ITOK = ITOK + 1
            IF(ITOK.GE.NTOK) GO TO 80
           KEY= LINE(IBEG(ITOK) :ibeg(ITOK)+3)
           CALL CCPUPC(KEY)
            ITOK = ITOK + 1
           IF(KEY.EQ.'WATO') 
     1      CHNID(ICHN,2)= LINE(IBEG(ITOK) :IBEG(ITOK))
C
           IF(KEY.EQ.'RANG')  THEN
           CALL GTPINT(ITOK,mchn(ICHN,1),NTOK,ITYP,FVALUE)
            ITOK = ITOK + 1
           CALL GTPINT(ITOK,mchn(ICHN,2),NTOK,ITYP,FVALUE)
            IF(MCHN(ICHN,1).GT.MCHN(ICHN,2) )THEN
            I = MCHN(ICHN,1)
            MCHN(ICHN,1) = MCHN(ICHN,2)
            MCHN(ICHN,2) = I
            END IF
           END IF
        GO TO 201
C
C     Extra Acceptors - single character Atom types Default O N
C
  300   CONTINUE
         DO 301 ITOK = 2,NTOK
         ACEPID(ITOK+1)= LINE(IBEG(ITOK) :IBEG(ITOK))
         naccep = naccep + 1
301      CONTINUE
        GO TO 80
C
C---- Input Water chain IDS
C
  400   CONTINUE
         DO 401 ITOK = 2,NTOK
           ichnw = ichnw + 1
           if (ichnw.gt.nchnw) CALL CCPERR(1,
     +     ' *** Reset parameter for water chain limit - nchnw=?? ***')
           wchnid(ichnw)= LINE(IBEG(ITOK) :ibeg(ITOK))
 401    CONTINUE
        GO TO 80
C
C---- secondary atom occupier  occw
C
  500   CONTINUE
          CALL GTPREA(2,occw,NTOK,ITYP,FVALUE)
        GO TO 80
C
C---- TITLe  read title
C
  600   TITLE = LINE(IBEG(2) :80)
        GO TO 80
C
C---- SHELL  (2 or 3 ) - default = 1
C
  700   continue
          CALL GTPINT(2,ish,NTOK,ITYP,FVALUE)
        if(ish.eq.2)ishell = 4
        if(ish.eq.3)ishell = 7
        GO TO 80
C---- NHBOND - default is 4
C
  750   continue
          CALL GTPINT(2,nhbond,NTOK,ITYP,FVALUE)
        GO TO 80
C
C
  800   continue
        write(6,*)'  ***  Input finished ***'
         write(6,'(//,a,i5)') ' **  Input Spacegroup number is ',nspgrp
         do 801 ism = 1,nsym
         write(6,'(//,a,i3,4(/,4f6.2))')' Symmetry matrix is',ism,
     1  ((rsym(i,j,ism),j=1,4),i=1,4)
801      continue

         NCHN = ICHN
         write(6,'(//,a,i4)')' **  Input number of chains  is ',NCHN
C
         write(6,'(//,a,i4,/,a,10(3x,a))')
     1 ' **  Number of atom acceptors  is ',NACCEP,
     1 ' **  Acceptor IDS:',(ACEPID(I),I=1,NACCEP)
c
         write(6,'(/,a)')
     1 '  protein ID, Output water ID, residue range for protein.'
         do 802 ich = 1,NCHN
         write(6,'(/,10x,a,10x,a,10x,2i6)') (chnid(ich,i),i=1,2),
     1  (mchn(ich,i),i=1,2)
       nres0(ich)=isum 
       nwatsch(ich)=0
       sumoc(ich)=0
       isum=isum + mchn(ich,2) - mchn(ich,1) +1
        if(isum.gt.nres1) 
     1call ccperr(1,' Reparameterise nres1,nres2 - tot no of residues')
  802  continue
c
         if(ichnw.eq.0)
     1 CALL CCPERR(1, ' ** Disaster - no Water chain IDS given ***')
c
         write(6,'(/,a,4(5x,a))')
     1 '  Unassigned water chain IDS: ',(wchnid(ich),ich=1,ichnw)

         If(occw.gt.0.0)write(6,'(//,a,f6.2)')
     1 ' Second occurrence of any atom will have "occupancy" equal  ',
     1 occw

         If(occw.eq.0.0)write(6,'(//,a,f6.2)')
     1 ' Second occurrence of any atom will not be output to PDB file'

         write(6,'(//,a,I6)')
     1 ' You are searching for H2Os for shell :',ish
C
C  cLEAR  jarr used to check how many h2os bonded to each residue.
c   Labels will be On0 On1 On2 On3  protein residue name
c
       do 810 i=1,10
       do 810 j=1,nres1
810   jarr(j,i)=0
       do 811 i=1,NAT
       do 811 j=1,nres2
811    dis(j,i)=10000
c
        write(6,*)
     +  ' Reading distang output - every 100th line monitored'
        write(6,*)
     +  '  And as well every interesting bond is listed...'
      IUNDIST=0
      stat='OLD'
      type='F'
      ifail=0
      CALL CCPDPN(IUNDIST,'DISTOUT',stat,type,LL,ifail)
c  Begin to read DISTang log file
c  search for line ATOM I   ATOM J - beginning of contact list
c
1      CONTINUE
       nwats=0
        ICOUNT = 0
       nres0(NCHN+1)=10000000
c  Skip beginning of DISTANG log file...
c
      NTOK = NPAR
C
      read(iundist,'(a)',end=99)line
      if (line.eq.' ') goto 1
C
C            **********
        CALL CCPUPC(LINE)
C            **********
C
C          ***********************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +            .FALSE.)
C          ***********************************************************
C
       if(line(ibeg(1):ibeg(1)+6).eq.'ATOM I ') go to 2
       go to 1
c  res1  - must be protein residue type.
c  res2  - must be "water" residue type ie. O

c  Example of DISTANG output.
C
C   atnm1_id   atnm2_id   dis                  sym_code          occ1*occ2
c  T208O  A   O 45OW2E   3.12229              2 0. 0. 0.          0.5000
C   atnm1_id = Residue_type,residue_number,atom_name,Chain_id.( A1,I3,A3,A1)
c
2       continue
C
       read(iundist,'(a)',end=10)line
       if (line.eq.' ') goto 2
C
C          ***********************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +            .FALSE.)
C          ***********************************************************
      ICOUNT = ICOUNT + 1
      ICHK = mod(ICOUNT,100)
      IF (NTOK.EQ.0) GO TO 2
      LINE1 = LINE(IBEG(1):IEND(NTOK))
      read(line1,1005,err=2)
     1  res1,ires1,atnm1,chn1,res2,ires2,atnm2,
     1  chn2,dist,symcd,oocc
      if (ichk.eq.1) 
     + write(6, '(1x,a1,i4,a4,a1,3x,a1,i4,a4,a1,f10.5,13x,a11,6x,f6.4)')
     +     res1,ires1,atnm1,chn1,res2,ires2,atnm2,chn2,
     +     dist,symcd,oocc
C  Distang format changed: Mar 16th 1995
 1005           FORMAT
     +   (2(A1,1X,I4,1X,A4,1X,A1,3X),F8.3,10X,A11,2X,F6.3)
C 1005 format(a1,i3,a3,a1,3x,a1,i3,a3,a1,f10.5,13x,a11,6x,f6.4)
c  Skip blank lines
      if (ires1.eq.0 )go to 2
c  Ignore connections to non-waters
      if (res2.ne.'O')go to 2
c  Ignore connections between disordered atoms
      if (oocc .lt. 0.49) go to 2

c  Only interested in connections FROM nominated chains to un-nominated 
c     ones...
c  Is chn2 an unassigned water chain ID?
c
      do 3 ich=1,ichnw
        if (chn2.eq.wchnID(ich)) go to 33
3     continue
      go to 2

33    continue
c
c  Is chn1 one of the nominated ones - if so, remember chain number ?
      do 4 ich=1,NCHN
      ichn = ich
      if(chn1.eq.chnID(ich,1))go to 5
4     continue
      go to 2
 5    continue

        nwats=nwats+1
        iatc=-100
c
c   Give atom number 0 for N, 1 for O - 2 OG etc - 3 for OD1 ND1 - 
c   4 for OD2 ND2  - 5 for OE1 NE1 - 6 for OE2 NE2 -
c   - 7 for OZ NZ - 8 for OH1 NH1 - 9 for OH2 NH2
C   Plus possible others See above..
c   23/4/99: these numbers are now set in the s/r setiatc
C   j= 1 FOR 2ND WATER , j=2 FOR 3RD, 3  FOR 4TH.
c
c  check for wat - wat contacts - resnm1 = O  - atnam = Oij
         if(res1.eq.'O' )then
            call setiatc(res1,atnm1,iatc)
            go to 6
         endif
c
C  Ignore atoms which are not named Acceptors. Default O and N 
        DO 55 I = 1,NACCEP
        if(atnm1(1:1).eq.ACEPID(I)) GO TO 56
 55     CONTINUE
C
c  Not a hydrogen bond  - eg: involves Non acceptor-ow close contact...
c  Do not flag if occupancies le 0.5 

       if( oocc .ge.0.5)then
       write(6,'(/,a,/,a)')
     1  '  ***  No apparent H bond for this close contact ***',
     1  '  Line just read is:'

C1112311231112311123112311123123456781234567890123456789011212345612345678
CG   1 N   A   O  31 OW6 E      2.902           1 0. 0. 0.   1.000   53.10
CG   1 CA  A   O  29 OW2 E      3.008           2 0. 0. 0.   0.500   36.16

        write (6,1006)res1,ires1,atnm1,chn1,
     1   res2,Ires2,atnm2,chn2,dist,SYMCD,oocc
      endif
      go to 2

 56     CONTINUE

C  Call to setiatc will determine the water numbering (iatc=0-9)
       call setiatc(res1,atnm1,iatc)
6      continue
C
C  Any other H bond given iatc = 9...
       if(iatc.eq.-100)then
           write(6,'(a,/,a)')
     1  '  ***  Unconventional atom name for this close contact ***',
     1  '  Line just read is:'
           write (6,1006)res1,ires1,atnm1,chn1,
     1   res2,Ires2,atnm2,chn2,dist,SYMCD,oocc
          iatc=9
       endif
c
C    CHECK RESIDUE NUMBER WITHIN LIMITS
      IF(ires1.LT.mchn(ichn,1) .OR. ires1.GT.mchn(ichn,2)) THEN
       write(6,'(a,/,a)')
     1  '  ***  Residue number outside limits ***',
     1  '  Line just read is:'
        write (6,1006)res1,ires1,atnm1,chn1,
     1   res2,Ires2,atnm2,chn2,dist,SYMCD,oocc
 1006           FORMAT
     +   (5X,2(A1,1X,I4,1X,A4,1X,A1,3X),F8.3,10X,A11,2X,F6.3)

      write(6,'(a,i5)')'  ichn ',ichn
      go to 2
      endif
c
      nwatsch(ichn)=nwatsch(ichn) + 1
      ires1= ires1 -mchn(ichn,1) + nres0(ichn) + 1
C
c  find iatw
c  Check OWi  or W?i
       iatw  = 0
       IF(atnm1(3:3).eq.'0'.or.atnm1(3:3).eq.'1'.or.atnm1(3:3).eq.'2'
     1.or.atnm1(3:3).eq.'3'.or.atnm1(3:3).eq.'4'.or.atnm1(3:3).eq.'5'
     1.or.atnm1(3:3).eq.'6'.or.atnm1(3:3).eq.'7'.or.atnm1(3:3).eq.'8'
     1.or.atnm1(3:3).eq.'9') READ(atnm2,'(2x,i1)')IATW
C       read(atnm2,'(2x,i1)')iatw
       write(6,'(a,2x,a,i7,3x,a,3x,a,i7)')' res2,Ires2,atnm2,chn2,iatw',
     1  res2,Ires2,atnm2,chn2,iatw
c  allow 4 atoms/water
       DO 7 I = 1,NHBOND -1
      if(iarr(ires2,iatw+1).gt.0) iatw = iatw + 10
 7    CONTINUE
      if(ires2.gt.nres2 .or. ires2.lt.1 .or. iatw+1.gt.NAT )then
      write(6,1007)ires2,iatw,ires1,iatc 
1007   format('  problem ires2 iatw ires1 iatc ',2i5,5x,2i5,i6,
     1 /,'  Line just read is:')
        write (6,1006)res1,ires1,atnm1,chn1,
     1   res2,Ires2,atnm2,chn2,dist,SYMCD,oocc
c      stop ' Problem ires2 iatw too big for program parameters'
        call ccperr (1, 'Fatal error')
      endif
c
      if(ires1.gt.nres1 .or.ires1.lt.1 .or.iatc.lt.0)then
      write(6,1008)iatc
1008   format(/,
     1  '  Problem - Crazy residue number or too many Hbonds - nhb =',
     1  i4, /,'  Line just read is:')
        write (6,1006)res1,ires1,atnm1,chn1,
     1   res2,Ires2,atnm2,chn2,dist,SYMCD,oocc
        call ccperr (1, 'Fatal error')
       endif
c
      if( jarr(ires1,iatc+1).gt.NAT/10)then
      write(6,1008)iatc
        write (6,1006)res1,ires1,atnm1,chn1,
     1   res2,Ires2,atnm2,chn2,dist,SYMCD,oocc
        call ccperr (1, 'Fatal error')
       go to 2
       endif
c
      iarr(ires2,iatw+1)=50*ires1 + iatc + 10*jarr(ires1,iatc+1)
      jarr(ires1,iatc+1) = jarr(ires1,iatc+1) + 1
      dis(ires2,iatw+1)=dist
      symcod(ires2,iatw+1)=symcd
       go to 2
c
10    continue
      write(6,1010)nwats,(ichn,nwatsch(ichn),ichn=1,NCHN)
1010   format(////,' Total number of water contacts',i5,
     1 ' ( there may be doubling and non-hydrogen contacts included)',
     1  20(/, ' Hydrogen bond contacts for chain ',i3,i6))
      i14=0
      ifail=0
      ixyzin = 0
      ixyzout = 0
      CALL XYZOPEN('XYZIN','INPUT',' ',IXYZIN,IFAIL)
      CALL XYZOPEN('XYZOUT','OUTPUT',' ',IXYZOUT,IFAIL)
c
      LINE = 'REMARK  '//TITLE(1:72)
      CALL WREMARK(IXYZin,LINE)
c
      CALL RBRORF(RO,RF)
c
      DO 30 NS=1,NSYM
C   GET [RO] [Si] [RF] for each symmetry operator
      n=4
      DO 31 I=1,4
      DO 31 J=1,4
31    RJUNK1(J,I)=rsym(J,I,NS)
      CALL matmln(N,RJUNK2,RJUNK1,RF)
      CALL matmln(N,RJUNK1,RO,RJUNK2)
      DO 32 I=1,4
      DO 32 J=1,4
32    rsymO(J,I,NS)=RJUNK1(J,I)
30    CONTINUE
c
50    CALL XYZADVANCE(IXYZIN,IXYZOUT,0,*99,*99)
      CALL XYZATOM(IXYZIN,ISER,ATNM1,RESNAM,CHNNAM,IRES2,RESNO,
     +        INSCOD,ALTCOD,SEGID,IZ,ID)
C   This is NOT enough - check if CHNID is not equal to 
C    WCHNID?
c  Ignore coordinates already in a nominated chain
        Ikeep = 0
        DO 51 ICH = 1,ICHNW
       if(chnnam.eq.wchnid(ich)) ikeep = 1
51     continue

       IF (IKEEP.EQ.0) THEN
         CALL XYZADVANCE(IXYZOUT,0,0,*99,*99)
         go to 50
       ELSE
C
C   Cannot handle anisotropic Bs at present, converted to isotropic.
C
         CALL XYZCOORD(IXYZIN,'O','U',Xx,Yy,Zz,OCC,BISO,B)
         IF (B(2).NE.0.0 .OR. B(3).NE.0.0) THEN
           IF (XX.EQ.0.0 .AND. YY.EQ.0.0 .AND. ZZ.EQ.0.0) GOTO 50
         ENDIF
       endif

       B(1) = BISO
       DO 45 I=2,6
         B(I) = 0.0
45     CONTINUE

       iatw0 = 0
       IF(atnm1(3:3).eq.'0'.or.atnm1(3:3).eq.'1'.or.atnm1(3:3).eq.'2'
     1.or.atnm1(3:3).eq.'3'.or.atnm1(3:3).eq.'4'.or.atnm1(3:3).eq.'5'
     1.or.atnm1(3:3).eq.'6'.or.atnm1(3:3).eq.'7'.or.atnm1(3:3).eq.'8'
     1.or.atnm1(3:3).eq.'9') READ(atnm1,'(2x,i1)')IATW0
       if(occ.gt.occw)sumoc0=occ+sumoc0
       if(occ.le.occw)idup = idup + 1
       i14=i14+1
       X=XX
       Y=YY
       Z=ZZ
       iw=0
       dist=min(dis(ires2,iatw0+1),dis(ires2,iatw0+11),
     1 dis(ires2,iatw0+21),dis(ires2,iatw0+31))
       dissum=0.
       do 41 i=1,4
       iatw=iatw0 + 10*(i-1)
c
       if(iarr(ires2,iatw+1).eq.0) go to 41
       dissum=dissum + dis(ires2,iatw+1)
       iw=iw+1
       oc=occw
       if(dist.eq.dis(ires2,iatw+1) ) oc=occ
       if(symcod(ires2,iatw+1).eq.' ' ) go to 42
       if(symcod(ires2,iatw+1).eq.' 1 0. 0. 0.' .and. iw.eq.1) go to 42
C     13/09/2002 Trap for new distang output
C     In this case SYMCOD will be a string, not a set of numbers
C     So if there is an error then jump to 9900
       read(symcod(ires2,iatw+1),1015,err=9900)isym,tr1,tr2,tr3
1015    format(i2,3f3.0)
       x1= rsymo(1,1,isym)*x + rsymo(1,2,isym)*y + rsymo(1,3,isym)*z
     1    +rsymo(1,4,isym) 
     1    +ro(1,1)*tr1 +ro(1,2)*tr2 +ro(1,3)*tr3
       y1= rsymo(2,1,isym)*x + rsymo(2,2,isym)*y + rsymo(2,3,isym)*z
     1    +rsymo(2,4,isym)
     1    +ro(2,1)*tr1 +ro(2,2)*tr2 +ro(2,3)*tr3
       z1= rsymo(3,1,isym)*x + rsymo(3,2,isym)*y + rsymo(3,3,isym)*z
     1    +rsymo(3,4,isym) 
     1    +ro(3,1)*tr1 +ro(3,2)*tr2 +ro(3,3)*tr3
       Xx=x1
       Yy=y1
       Zz=z1
42     continue
c
       chn1=chnnam
       ichk=iarr(ires2,iatw+1)
       ires1 = int(0.02*ichk + 0.001)
       iatc = ichk-50*ires1
       iatc2=int(0.1*iatc + 0.001)
       iatc1=iatc-10*iatc2
c
      do 60 ich=1,NCHN
      if(ires1.gt.nres0(ich) .and. ires1.le.nres0(ich+1)) then
       ires1 = ires1 -nres0(ich) + mchn(ich,1) - 1
      chnnam = chnid(ich,2)
      nwch(ich)=nwch(ich) + 1
      if(oc.gt.0.01)sumoc(ich)=sumoc(ich) + oc
      go to 61
      endif
60    continue
C
      CALL CCPERR(2, '  **   NO-good chain id ...***')
61    continue
c
       iatc2 = iatc2 + ishell
       if (oc.gt.0.0) then
         write (tmpstr,fmt='(A1,2I1,A1)') labw,iatc1,iatc2,' '
         CALL XYZATOM(IXYZOUT,ISER,TMPSTR,RESNAM,CHNNAM,IRES1,
     +          RESNO,INSCOD,ALTCOD,SEGID,IZ,ID)
         CALL XYZCOORD(IXYZOUT,'O','U',Xx,Yy,Zz,OC,BISO,B)
         CALL XYZADVANCE(IXYZOUT,0,0,*99,*99)
       endif
       if(oc.eq.0.0) write(6,1024)
     1  iser,labw,iatc1,iatc2,RESNAM,CHNNAM,ires1,xX,yY,zZ,oc,biso
1024  FORMAT(
     1 ' This position is a possible secondary hydrogen contact ',/,
     1 ' ATOM  ',i5,2x,A1,2i1,1x,A4,A1,I4,4X,3F8.3,2F6.2,3i5)
       i22=i22+1
       if(oc.gt.occw)sumoc2=sumoc2+oc
       iarr(ires2,iatw+1) = 0
41    continue
c
c  No match to protein - iw=0
c
        if(iw.eq.0)then
        sumoc3=sumoc3+occ
        if(occ.gt.occw)sumoc2=sumoc2+occ
        CALL XYZATOM(IXYZOUT,ISER,ATNM1,RESNAM,CHNNAM,IRES2,RESNO,
     +               INSCOD,ALTCOD,SEGID,IZ,ID)
        CALL XYZCOORD(IXYZOUT,'O','U',Xx,Yy,Zz,OCC,BISO,B)
        CALL XYZADVANCE(IXYZOUT,0,0,*99,*99)
        i22=i22+1
        nwch(NCHN+1)=nwch(NCHN+1)+1
        endif
c
      nbav(iw+1) = nbav(iw+1) + 1
       bav(iw+1) =  bav(iw+1) + BISO
      dissm(iw+1) = dissm(iw+1) + dissum
      go to 50
c
c  End of input water list
c
99    continue
      write(6,1016)i14,sumoc0,idup
1016  format(//,' *** Input  H2o lines and total occupancy ***',i4,f8.3,
     +  /,      ' ***       Number of duplicate H2os       ***',i5) 
c
      write(6,1017)i22,sumoc2
1017  format(//,' *** Output H2o lines and total occupancy ***',i4,f8.3)
c
       do 70 ich=1,NCHN
      write(6,1018)chnid(ich,2),nwch(ich),sumoc(ich)
1018  format(//,' *** Output H2o lines and occupancy for chain ***',
     1  2x,A1,i4,f8.3) 
70    continue
      write(6,1019)nwch(NCHN+1),sumoc3
1019  format(//,' *** Output H2o lines and  occupancy for nonbonded ***'
     1 ,i4,f8.3,///) 
c
      write(6,*)
     1' ** Average B values and bond length for H2Os ',
     1 ' with 0 1 2 3 4 bonds to protein **'
       do 71 i=1,5
       if(nbav(i).eq.0)go to 71
       i0=i-1
       bav(i)=bav(i)/nbav(i)
       avdis= 0
       avdis= dissm(i)/nbav(i)
       if(i0.gt.0)avdis=avdis/i0
      write(6,1020) i0,nbav(i),bav(i),avdis
1020  format(' Number of bonds per H2O ',i3,' Number of H2Os',i4,
     1 ' Average B',f10.3,'  Average H bond',f10.3)
71    continue

      CALL XYZCLOSE(IXYZIN)
      CALL XYZCLOSE(IXYZOUT)
      CALL CCPERR(0, 'Normal termination')

C     Arrive here if the symcodes from distang cannot be read
 9900 CONTINUE
      WRITE(6,9910)
      CALL CCPERR(1, 'Cannot read symmetry operators from DISTANG!')
 9910 FORMAT(' *** Cannot interpret the symmetry operators written',
     1     ' by DISTANG ***',//,' These should be of the form e.g.',
     1     '  1 0. 0. 0.',/,' If they are of the form e.g.',
     1     ' -X,Y+1/2,-Z+1/2 then you need to use',/,' the OUTPUT',
     1     ' DISTOUT option of DISTANG, and use the file assigned to',/,
     1     ' DISTOUT as input to WATERTIDY.',//,' *** See the',
     1     ' DISTANG/WATERTIDY documentation for more information.',/)

      END

c
c     ==============================
      subroutine setiatc(res1,atnm1,iatc)
c     ==============================
c
c    This subroutine sets the value of iatc according to the
c    host atom name.
c
      implicit none
c
      character*1 res1
      character*4 atnm1
      integer iatc
c
C  Fudge the Water numbering. Only use 0-9
c
c  First check for the existing conventions:
c                          N,CA              = 0
c                          O,C               = 1  (res1.ne.'O') 
c                          OG,OG1,CG,CG1     = 2
c                          OD1,ND1,CD,CD1    = 3
c                          OD2,ND2,CD2,CD3   = 4
c                          OE,OE1,NE1,CE,CE1 = 5
c                          OE2,NE2,CE2,CE3   = 6
c                          NZ,CZ             = 7
c                          OH,OH1,NH1,CH,CH1 = 8
c                          OH2,NH2,CH2,CH2   = 9
c  (see documentation also)
      if(res1.ne.'O' )then
       if(atnm1(1:2).eq.'N ' .or. atnm1(1:2).eq.'CA')then
         iatc=0
       elseif(atnm1(1:2).eq.'O ' .or. atnm1(1:2).EQ.'C ')then
         iatc=1
       elseif(atnm1(2:2).eq.'G'  .or. atnm1(2:2).EQ.'B')then
         iatc=2
       elseif(atnm1(2:3).eq.'D1' .or. atnm1(2:3).EQ.'D ')then
         iatc=3
       elseif(atnm1(2:3).eq.'D2' .or. atnm1(2:3).EQ.'D3')then
         iatc=4
       elseif(atnm1(2:3).eq.'E ' .or. atnm1(2:3).EQ.'E1')then
         iatc=5
       elseif(atnm1(2:3).eq.'E2' .or. atnm1(2:3).EQ.'E3')then
         iatc=6
       elseif(atnm1(2:2).eq.'Z')then
         iatc=7
       elseif(atnm1(2:3).eq.'H1' .or. atnm1(2:3).EQ.'H ')then
         iatc=8
       elseif(atnm1(2:3).eq.'H2' .or. atnm1(2:3).EQ.'H3')then
         iatc=9
       end if  
      end if
c    Convention for waters:
c                          OW  = 0
c                          OWn = n (n is 0-9)
c                          On  = n (" "   " )
c                          Onm = n (n,m are 0-9)
c    Look for OW (no number) first
      if(res1.eq.'O' )then
       if(atnm1(1:3).eq.'OW '.or.atnm1(1:3).eq.'O  ') then
         iatc=0
c    Then OW + number...
       elseif(atnm1(1:2).eq.'OW') then
         read(atnm1,'(2x,i1)',err=9090)iatc
c    Finally for O + number(s)...
c    Set iatc equal to the first integer if there are two digits
       elseif(atnm1(1:1).eq.'O') then
         read(atnm1,'(1x,i1)',err=9090)iatc
c         if(atnm1(3:3).eq.' ')then
c            read(atnm1,'(1x,i1)',err=9090)iatc
c         else
c            read(atnm1,'(2x,i1)',err=9090)iatc
c         end if
       end if  
      end if
c
c   Continuation 9090: errors reading the digits in atom names
c   come here; in this case don't set iatc because it will be
c   picked up later anyway
 9090  continue
c
       return
       end
