C
C     WATPEAK
C     Copyright (C) 1992 Phil Evans
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
c	WATPEAK
c
c	Select peaks output by PEAKMAX which lie close to atoms in
c molecule from XYZIN, write out to file XYZOUT, and contact list to
c printer
c
c Commands:-
c     TITLE 
c  or REMARK
c     anything on these lines is just echoed to the output PDB file.
c     A TITLE line is also echoed to the log file
c
c     DISTANCE  d
c     maximum distance for a peak to be accepted
c
c     BFACTOR  Bfactor [occupancy]
c     values to set Bfactor & occupancy to, default = 30.0, 1.0
c     note the input peak list may have the peak height in the occupancy column
c     (see example). If this command is omitted, the Bfactor & occupancy
c     will be left as input
c
c     CHAIN Chain_id
c     Set chain id for output (default blank)
c
c     HETATOMONLY   
C     toggle switch to only match Oxygen or Nitrogen
c     from the XYZIN file to a new peak in the PEAKS file
c     default = FALSE
c
c     CELL
c     input the cell parameters and the orthogonalisation definition
c     number (default=1). Not needed if PDB file has CRYST and SCALE
c     cards.
c     a b c alpha beta gamma ncode
c
c     SYMM
c     enter the symmetry either as the space group number or symmetry
c     cards
c     symm 19
c     symm x,y,z * -x,y,-z
c
c  Example
c    peakmax  mapin  c40_fo-fcx xyzout c40_peaks.pdb << eof-1
c    100 0.2 -2 1 0     # use NOUT=-2 to write peak height in occupancy slot
c    eof-1
c    
c    watpeak  xyzin a95_40.pdb peaks c40_peaks.pdb xyzout waters_40.pdb <<eof-2
c    title Possible waters from C40 DelF
c    distance 4.0
c    chain W
c    symm 19
c    eof-2
c
      program watpeak
c     ===============
c
C      implicit none
c
c     ..
c     .. Parameters ..
      integer maxatm, maxctc, maxwat
      parameter (maxatm=50000, maxctc=100, maxwat=10000)
c     ..
c     .. Local Arrays ..
      real b(6),cell(6),rf(4,4),ro(4,4),disctc(maxctc)
      real xa(maxatm),ya(maxatm),za(maxatm)
      real xwat(maxwat),ywat(maxwat),zwat(maxwat)
      real qwat(maxwat),bwat(maxwat),xout(3,maxwat)
      integer lcnctc(maxctc), iza(maxatm), mres(maxatm)
      integer iserwat(maxwat)
      character atoms(maxatm)*4, typres(maxatm)*4, idwat(maxwat)*2,
     +          chain(maxatm)*1
      character watrestyp(maxwat)*4
c     ..
c     .. Local Scalars ..
      real biso,x,y,z,bdef,qdef,q
      real dmax,dmaxsq,dissq,dmin,tmp
      integer iser,ires,iz,ncnctc,npkin,npknot,symrej,nlong
      integer i,ifail,j,n,l,m,natoms,npeaks,numsgp,nsymp
      integer pass,nearctc
      character title*80,line80*80,atnam*4,restyp*4,idch*1,resno*4,
     +          wchid*1,segid*4
      character*10 spgnam,pgname
      character*4 id
      character*1 inscod,altcod
      logical lbchng
      integer ixyzin,ixyzout,ipeaks
c     ..
c     .. things for parser ..
      integer maxtok
      parameter (maxtok=100)
      character key*4,cvalue(maxtok)*4,line*80
      integer ibeg(maxtok),iend(maxtok),ityp(maxtok),
     .     idec(maxtok),ntok
      real    fvalue(maxtok)
      logical lend, hetatom
c     ..
c     .. Common Blocks ..
      integer nsym
      real symrot, dminsq
      common /sub/ nsym, dminsq, symrot(4,4,96)

      integer lenstr
      external lenstr,ccperr,ccpfyp,ccprcs,parser,ccpupc,gtprea,
     +         CVFRAC2,rbcell,rbrorf,rdsymm,RBFRAC2,symatm,
     +         WREMARK,XYZADVANCE,XYZATOM,XYZCLOSE,
     +         XYZCOORD,XYZINIT,XYZOPEN

      call ccpfyp
      CALL CCPRCS(6,'WATPEAK','$Date: 2005/09/06 11:25:30 $')
      CALL XYZINIT
c Defaults
      hetatom = .false.
      title = ' '
      line80 = ' '
      dmax = 5.0
      dmin = 0.1
      bdef = 30.0
      qdef = 1.0
      lbchng = .false.
      wchid = ' '
      nsym = 0
      cell(1) = 0.0
      cell(2) = 0.0
      cell(3) = 0.0
      cell(4) = 90.0
      cell(5) = 90.0
      cell(6) = 90.0


1     line=' '
      ntok=maxtok
      call parser(
     .  key,line,ibeg,iend,ityp,fvalue,cvalue,idec,ntok,lend,.true.)
      call ccpupc(key)
      if(lend) go to 10
      if(ntok.eq.0) go to 1
c
      if (key .eq. 'END ') then
         go to 10
      elseif (key .eq. 'TITL') then
         title = line(ibeg(2):)
      elseif (key .eq. 'REMA') then
         l = min(73,lenstr(line(ibeg(2):))) + ibeg(2) - 1
         line80 = 'REMARK '//line(ibeg(2):l)
      elseif (key .eq. 'HETA') then
         hetatom = .true.
         write (6,*)
     + ' Only Hetatoms (O and N) will be matched for distance'
      elseif (key .eq. 'DIST') then
         call gtprea(2,dmax,ntok,ityp,fvalue)
         if(ntok.gt.2)call gtprea(3,dmin,ntok,ityp,fvalue)
         if (dmin.gt.dmax) then
           tmp = dmax
           dmax = dmin
           dmin = tmp
         endif
      elseif (key .eq. 'BFAC') then
         call gtprea(2,bdef,ntok,ityp,fvalue)
         if(ntok.gt.2)call gtprea(3,qdef,ntok,ityp,fvalue)
         lbchng = .true.
      elseif (key .eq. 'CHAI') then
         if (ntok .gt. 1) wchid = line(ibeg(2):ibeg(2))
      elseif (key .eq. 'CELL') then
         if (ntok.lt.4) call ccperr(1,'*** Incorrect cell card ***')
         call gtprea(2,cell(1),ntok,ityp,fvalue)
         call gtprea(3,cell(2),ntok,ityp,fvalue)
         call gtprea(4,cell(3),ntok,ityp,fvalue)
         if (ntok.ge.5) call gtprea(5,cell(4),ntok,ityp,fvalue)
         if (ntok.ge.6) call gtprea(6,cell(5),ntok,ityp,fvalue)
         if (ntok.ge.7) call gtprea(7,cell(6),ntok,ityp,fvalue)
      elseif (key .eq. 'SYMM') then
         call RDSYMM(2,LINE,IBEG,IEND,ITYP,FVALUE,NTOK,
     .                        SPGNAM,NUMSGP,PGNAME,NSYM,NSYMP,SYMROT)
      endif
      go to 1
c
c --- Finished input
c
 10   if(nsym.eq.0)call ccperr(1,'*** SYMMETRY CARD MUST BE GIVEN ***')
      dmaxsq = dmax**2
      dminsq = dmin**2
      write (6,'(/,1x,a,/)')  title
      write (6,'(A,2F8.3)') ' Distance limits: ',DMAX,dmin
c
c --- Read all atoms into arrays
c
      ifail = 0
      IXYZIN = 0
      IXYZOUT = 0
      call XYZOPEN('XYZIN','INPUT',' ',IXYZIN,ifail)
      call XYZOPEN('XYZOUT','OUTPUT','PDB',IXYZOUT,ifail)
      if (line80(1:1) .ne. ' ') call WREMARK(IXYZOUT,line80)
      if (title(1:1) .ne. ' ') then
        l = min(73,lenstr(title))
        line80 = 'REMARK '//title(1:l)
        call WREMARK(IXYZOUT,line80)
      endif
c
      n = 0
c--------------------
 20   call XYZADVANCE(IXYZIN,0,0,*30,*30)
      call XYZATOM(IXYZIN,iser,atnam,restyp,idch,ires,resno,
     .           inscod,altcod,segid,iz,id)
      call XYZCOORD(IXYZIN,'O','U',x,y,z,occ,biso,b)
c
c Skip ANISOU cards
c
      if (b(2).ne.0.0 .or. b(3).ne.0.0) then
        if (x.eq.0.0 .and. y.eq.0.0 .and. z.eq.0.0) goto 20
      endif

       if (hetatom) then
           if (atnam(1:1) .ne. 'O' .and. 
     +         atnam(1:1) .ne. 'N' ) go to 20
       end if
c
c
      n=n+1
      if (n .gt. maxatm) then
         call ccperr(1,'*** Too many protein atoms ***')
      endif
      xa(n) = x
      ya(n) = y
      za(n) = z
      iza(n) = iz
      atoms(n) = atnam
      typres(n) = restyp
      chain(n) = idch
      mres(n) = ires
      go to 20
c
c --- finished atoms
c
 30   call XYZCLOSE(IXYZIN)
      natoms = n
      write (6, '(A,I8)') ' Number of protein atoms:', natoms
c
      ro(1,1) = 0.0
      rf(1,1) = 0.0
      call rbrorf(ro,rf)
c
c --- read peaks
c
      ifail = 0
      IPEAKS = 0
      call XYZOPEN('PEAKS','INPUT','PDB',IPEAKS,ifail)
      npeaks = 0
      npkin  = 0
      npknot = 0
      symrej = 0
      nearctc = 0
      nlong = 0
      do 32 i=1,6
        b(i) = 0.0
32    continue
c
c  Assuming no anisotropic B's in PEAKS file
c
      n = 0
      do 35 i=1,maxwat
       call XYZADVANCE(IPEAKS,0,0,*40,*40)
       call XYZATOM(IPEAKS,iser,atnam,restyp,idch,ires,resno,
     .               inscod,altcod,segid,iz,id)
       call XYZCOORD(IPEAKS,'O','U',x,y,z,q,biso,b)
       n=n+1
       if (n .gt. maxwat) then
          call ccperr(1,'*** Too many possible water peaks ***')
       endif
       iserwat(n) = iser
       xwat(n) = x
       ywat(n) = y
       zwat(n) = z
       qwat(n) = q
       bwat(n) = biso

       idwat(n) = id(1:2)
       watrestyp(n) = restyp
35     continue
c
c --- Calculate orthogonal matrix default ncode = 1
c
40     if(cell(1).gt.1.0 .and. cell(2).gt.1.0 .and. cell(3).gt.1.0)then
         call RBFRAC2(cell(1),cell(2),cell(3),cell(4),cell(5),
     +                    cell(6),1)
       else
         call rbcell(cell,vol)
         if (cell(1).ne.0.0) then
           call rbrorf(ro,rf)
         else
           call ccperr(1,' *** The Cell MUST be specified ***')
         endif
       endif
c
      do 45 m=2,6
        b(m) = 0.0
45    continue
c
      segid = ' '
      npkin = n
      do 57 m=1,npkin
       ncnctc = 0
       call symatm(npeaks,pass,xwat(m),ywat(m),zwat(m),xout)
       if(pass.eq.1)then
         symrej = symrej + 1
         npknot = npknot + 1
         goto 57
       endif
       do 50, i=1,natoms
         dissq = (xwat(m)-xa(i))**2 + (ywat(m)-ya(i))**2 
     .                                    + (zwat(m)-za(i))**2
c If just one contact is less than min reject water
         if (dissq.lt.dminsq) then
           nearctc = nearctc + 1
           npknot = npknot + 1
           goto 57
         endif
         if (dissq.le.dmaxsq .and. dissq.ge.dminsq) then
c store all contacts to this peak
            ncnctc = ncnctc+1
            if (ncnctc .gt. maxctc) then
               write (6,'(a,i6)')
     $              ' *** Too many contacts, .gt. ',maxctc
               go to 57
            endif
            lcnctc(ncnctc) = i
            disctc(ncnctc) = sqrt(dissq)
         endif
 50   continue
c
      if (ncnctc .gt. 0) then
c OK at least one contact found
c  make atomname
         npeaks = npeaks+1
         atnam = 'O'
         ires = npeaks
c
         if (lbchng) then
c If B factor defaults are set, use them
            b(1) = bdef
            q = qdef
         else
            b(1) = bwat(m)
            q = qwat(m)
         endif
         call XYZATOM(IXYZOUT,iserwat(m),atnam,watrestyp(m),wchid,
     .                   ires,resno,inscod,altcod,segid,iz,idwat(m))
         call XYZCOORD(IXYZOUT,'O','U',xwat(m),ywat(m),zwat(m),
     .                   q,biso,b)
         call XYZADVANCE(IXYZOUT,0,0,*60,*60)
c
c Save fractional coordinates of accepted waters for use in symatm
c
         call CVFRAC2(xwat(m),ywat(m),zwat(m),
     .          xout(1,npeaks),xout(2,npeaks),xout(3,npeaks),1)
c
c List all contacts
         write (6, '(/,1x,a,i6,a,f10.3,a,3f8.2)')
     $        ' Peak ', npeaks, '    height ', q,
     $        '   coordinates ',xwat(m),ywat(m),zwat(m)
         do 55, i=1,ncnctc
            j = lcnctc(i)
            write (6,'(7x,a,4x,a4,2x,a4,2x,a1,2x,i4,a,f8.3)') '--->',
     $           atoms(j), typres(j), chain(j), mres(j),
     $           '   --- distance =',disctc(i)
 55      continue
      else
         npknot = npknot+1
         nlong = nlong + 1
      endif
 57   continue
c
c---- Finished
c
 60   call XYZCLOSE(IPEAKS)
      call XYZCLOSE(IXYZOUT)
c
      write (6,'(//,6(A,I8,/))')
     $     ' Number of "water" atoms output       = ',npeaks,
     $     ' Number of peaks read from PEAKS file = ',npkin,
     $     ' Number of peaks not used             = ',npknot,
     $     ' No. of symmetry "waters" rejected    = ',symrej,
     $     ' No. of "waters" with near contacts   = ',nearctc,
     $     ' No. of "waters" too far from protein = ',nlong
      call ccperr (0, 'Normal termination')
      end

c*******************************************************************
c
c This subroutine checks for symm. related peaks to accepted waters
c
c*******************************************************************
      subroutine symatm(npeaks,pass,x,y,z,xout)
c     ..
c     .. Arguments ..
      integer npeaks, pass
      real x, y, z, xout(3,npeaks)
c     ..
c     .. Common blocks ..
      integer nsym
      real symrot, dminsq
      common /sub/ nsym, dminsq, symrot(4,4,96)
c     ..
c     .. Local Scalars ..
      real xfr, yfr, zfr, diff, xor, yor, zor
      integer i,j,n
c     ..
c     .. Local Arrays ..
      real xsym(3,96), xx(3), fr(3)
      equivalence (fr(1), xfr), (fr(2), yfr), (fr(3), zfr)
      external CVFRAC2,matvc4

      pass = 0
c
c Convert othogonal to fractional coord.
c
      call CVFRAC2(x,y,z,xfr,yfr,zfr,1)
c
c Generate symmetry related atoms
c
      do 10 i=1,nsym
       call matvc4(xsym(1,i),symrot(1,1,i),fr)
  10  continue
c
c Compare with accepted peaks; reversed because symm peaks have same height
c
      do 20 i=npeaks,1,-1
       do 20 n=1,nsym
        do 30 j=1,3
         xx(j) = mod(abs(xsym(j,n)-xout(j,i)),1.0)
         if (xx(j).gt.0.5) xx(j) = xx(j)-1.0
  30    continue
        call CVFRAC2(xx(1),xx(2),xx(3),xor,yor,zor,0)
        diff = xor**2 + yor**2 + zor**2
        if (diff.lt.dminsq) then
          pass=1
          goto 40
        endif
  20  continue

  40  return
      end
C
