C
C     WILSON
C     Copyright (C) 1992 Eleanor Dodson, Phil Evans
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
C*************************************************************
C     
C     The development of the MTZ files and associated software mark 1
C     is part of the masterplan of the ESF/EACBM Working Group 2.1 for
C     better Protein Crystallographic software for Europe.
C     
C**************************************************************
C     
      PROGRAM WILSON
C     =================
C     
C     
C Cut down from Wilson plot in Truncate, Phil Evans, October 1991
C
C----------------------------------------------------
C     
C     
C     
C---- DESCRIPTION
C     ===========
C     
C  This program reads a file of amplitudes, and performs a Wilson plot
C  of observed intensities against resolution to determine an absolute
C  scale factor
C
C  If the atoms are randomly distributed in the cell, then 
C
C     <f**2> = Scale * <Fobs**2> * exp (-2B (sin theta/lambda)**2)
C
C  where <f**2> is the mean squared atomic form factor summed over
C     all atoms in the unit cell
C
C  So a plot of ln [ <f**2>/<Fobs**2> ] against 2(sin theta/lambda)**2
C  will have a slope = -B and an intercept = Scale 
C     
C      IMPLICIT NONE
C     
C     .. Parameters ..
      INTEGER NLPRGI
      PARAMETER (NLPRGI=7)
      INTEGER MCOLS
      PARAMETER (MCOLS=500)
      INTEGER MAXHIS
      PARAMETER (MAXHIS=30)
      INTEGER KNBINS
      PARAMETER (KNBINS=50)
      INTEGER MAXSER
      PARAMETER (MAXSER=50)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=96)
      INTEGER NPARM
      PARAMETER (NPARM=200)
C     ..
C     .. Scalars in Common ..
      REAL RANGE
      INTEGER ISOUT,LABFLG,LBOTKN,LENTIT,LUNIN,LUNOUT,MTITOT,
     +     MTZBPR,MTZERR,MTZFLG,MTZIN,MTZOUT,MTZPRT,NBATX,
     +     NHSOUT,NLAUE,NREFLX,NSPGRP,NSYM,NSYMP,
     +     NUMCOL
      LOGICAL KYCELL,KYHIST,KYSYMM,KYTITL,KYRESO,LBOLOG,
     $     MTZEOF
      CHARACTER LATTYP*1,DATEMT*8,LAUNAM*10,NAMSPG*10,PGNAME*10,
     +     VERSNX*10,TITIN*70,TITLE*80
C     ..
C     .. Arrays in Common ..
      REAL CELMTZ,CELOUT,RNGMTZ,RSYM,RSYMT
      INTEGER LBOBEG,LBOEND,MTZLOK
      CHARACTER CTPRGI*1,LSPRGI*30
C     ..
C     .. Local Scalars ..
      REAL DD,FMAX,FMIN,S,SIGMAX,
     +     SIGMIN,SMAX,SMIN,SS,XF,FA,SA
      INTEGER I,IFAIL,J,LDUM, NCF,NCS,NCI,NRANGE,NREF,NT
      REAL TOTIN,TOTFI,TOTFS,TOTFR,AFMEAN,SIGFMEAN
C     ..
C     .. Local Arrays ..
      REAL RECIN(MCOLS)
      LOGICAL LOGMSS(MCOLS)
      INTEGER INHKL(3)
      REAL SUMINS(KNBINS),SUMINSALL(KNBINS),SUMFIS(KNBINS),
     +     SUMFSS(KNBINS),SUMFRS(KNBINS)
C     
C     ..
C     .. External Functions ..
      INTEGER LENSTR
      EXTERNAL LENSTR
C     ..
C     .. External Subroutines ..
      EXTERNAL ASUSET,CCPDAT,CCPFYP,CCPOPN,CCPRCS,CENTRIC,EPSLN,
     +     EPSLON,KEYIN,LRASSN,LRCELL,LRCLOS,LRRSOL,
     +     LRINFO,LROPEN,LRREFF,LRREFM,LRSYMI,LRSYMM,LRTITL
C     Hint to linker for block data: 
      EXTERNAL WILBLK
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC MAX,MIN,NINT,REAL,SQRT
C     ..
C     .. Common blocks ..
C     
      COMMON /AGRSYM/NSPGRP,NSYM,NSYMP,NLAUE,
     +     RSYM(4,4,MAXSYM),RSYMT(4,4,MAXSYM)
      COMMON /CHRMTZ/LSPRGI(MCOLS),CTPRGI(MCOLS),
     +    VERSNX,TITIN,TITLE,DATEMT
      COMMON /CHRSYM/PGNAME,NAMSPG,LAUNAM,LATTYP
      COMMON /INOUT/LUNIN,LUNOUT,ISOUT
      COMMON /KEYRSC/RANGE,NRANGE
      COMMON /WRKMTZ/MTZIN,MTZOUT,MTZERR,MTZBPR,MTZPRT,MTZFLG,MTITOT,
     +     NBATX,NHSOUT,NUMCOL,NREFLX,LENTIT,LABFLG,MTZEOF,
     +     KYTITL,KYHIST,KYCELL,KYSYMM,KYRESO,MTZLOK(MCOLS),
     +     CELMTZ(6),CELOUT(6),RNGMTZ(2,MCOLS),
     +     LBOBEG(NPARM),LBOEND(NPARM),LBOLOG,LBOTKN
      REAL DSS,SI,SN,SR,SW,XY,XYAV,FFS,DSSQ,PI,DELDSS,F000,
     +     VOL,RR,CELLAS,RVOL,FRAC
      INTEGER  NRFUNQ,NRFLCF,NATMS,NELEC,ISMAX
      COMMON /PREWIL/ DSS(100),SI(100),SN(100),SR(100),SW(100),
     +     NRFUNQ(100),NRFLCF(100),XY(2,100),XYAV(2,100),
     +     FFS(500),DSSQ(500),PI,DELDSS,NATMS,NELEC,F000,
     +     ISMAX,VOL,RR(3,3,6),CELLAS(6),RVOL,FRAC
C     ..
C     
      INTEGER NBIN,NMON,IPLFSQ,NK
      REAL RMAXWL,RMINWL,SMINWL,SMAXWL,SMINSC,SMAXSC,RMINSC,RMAXSC,
     $     VPA
      COMMON /WILKEY/ NBIN,NMON,RMAXWL,RMINWL,SMINWL,SMAXWL,SMINSC,
     +     SMAXSC,RMINSC,RMAXSC,VPA,
     +     IPLFSQ,NK
      LOGICAL WILLOG,WILAPY,SCBLOG
      INTEGER IWITCH,NREFWL,MODEQ,KSCRTH
      REAL WILSC(2),WILBT(2)
      COMMON /LOGWIL/ WILLOG,WILAPY,IWITCH,SCBLOG
      CHARACTER NW*2
      COMMON /WILCHR/ NW(8)
      INTEGER NC,NO
      REAL AL,AS,BL,BS,CL
      COMMON /WILFRM/ NC(8),NO(8),AL(8),AS(8),BL(8),BS(8),CL(8)
C     .. Data statements ..
      DATA NREFWL/0/,MODEQ/2/,KSCRTH/60/,WILSC/1.0,1.0/,WILBT/0.0,0.0/
C     
C     
C---- MTZLOK  array containing index from program labels
C     to file labels. On input these should be
C     set to -1 for compulsory labels and 0
C     for optional labels, and exit labels
C     which are not present in the file have
C     their lookup entry set to 0.
C      DATA MTZLOK/5*-1,195*0/
C     
      DO 1 I=1,3
        MTZLOK(I) = -1
 1    CONTINUE 
      DO 2 I=4,MCOLS
        MTZLOK(I) = 0
 2    CONTINUE 
        MTZLOK(4) = 1
        MTZLOK(5) = 1
        MTZLOK(6) = 1
        MTZLOK(7) = 1
C     
      IFAIL = 0
      LUNIN = 5
      LUNOUT = 6
      MTZPRT = 1
      MTZFLG = 0
      MTITOT = 0
      LABFLG = 0
      PI = 3.14159265
C     
C     ****************************************
      CALL CCPFYP
      CALL MTZINI
      CALL CCPDAT(DATEMT)
      CALL CCPOPN(LUNIN,'DATA',5,1,LDUM,IFAIL)
      CALL CCPOPN(LUNOUT,'PRINTER',6,1,LDUM,IFAIL)
      CALL CCPRCS(LUNOUT,'WILSON','$Date: 2005/09/06 11:25:30 $')
C     ****************************************
C     
      NRANGE = 0
C     
C     
      WRITE (LUNOUT,FMT=6000)
 6000 FORMAT (5X,'WILSON PLOT',/,
     +        5X,'===========',////)
C     
C---- Read control cards
C     
C     *****
      CALL KEYIN
C     *****
      SMIN = 1.0
      SMAX = 0.0
      FMIN = 1000000.0
      FMAX = 0.0
      NREFWL = 0
      NREF = 0
      MTZIN = 1
      MTZERR = 0
      DO 5, I=1,KNBINS
         SUMINS(I) = 0.0
         SUMINSALL(I) = 0.0
         SUMFIS(I) = 0.0
         SUMFSS(I) = 0.0
         SUMFRS(I) = 0.0
 5    CONTINUE
C     
C---- Open input MTZ dataset
C     
C     HEADER "string"   Controls printout from reading MTZ HEADER
C     string is one of the following;
C     NONE sets MTZPRT = 0 (default)
C     no header o/p
C     BRIEF sets MTZPRT = 1
C     brief header o/p
C     HIST sets MTZPRT = 2
C     brief + mtz history
C     ALL  sets MTZPRT = 3
C     full header o/p from mtz reads
C     
C     ========================================
      CALL LROPEN(MTZIN,'HKLIN',MTZPRT,MTZERR)
C     ========================================
C     
      IF (MTZERR.EQ.-1) THEN
         WRITE (LUNOUT,FMT=*) ' LROPEN no such file for HKLIN'
         CALL CCPERR(1,' LROPEN no such file for HKLIN')
      END IF
C     
C---- CALL to return information about the MTZ file open for
C     read on index INDEX.
C     
C     ========================================
      CALL LRINFO(MTZIN,VERSNX,NUMCOL,NREFLX,RNGMTZ)
C     ========================================
C     
      WRITE (LUNOUT,FMT=6002) VERSNX,NUMCOL,NREFLX
 6002 FORMAT (/' Reading from HKLIN mtz file:',/,
     +     ' This file written with MTZLIB Version Number : ',A,/,
     +     ' File HKLIN contains a total of ',I6,' Columns',/,
     +     ' and a total of ',I10,' Reflections')
C     
C---- CALL to return the title and it's length from the
C     header of an MTZ file.
C     
C          ==========================
      CALL LRTITL(MTZIN,TITIN,LENTIT)
C          ==========================
C     
      IF (LENTIT.LE.1) TITIN = ' From Wilson on the '//DATEMT
C     
C---- CALL to READ Cell Parameters from the header common block
C     
C          ====================
      CALL LRCELL(MTZIN,CELMTZ)
C          ====================
C     
C---- Call to read resolution limits, if not read from commands
      IF (.NOT.KYRESO) THEN
         CALL LRRSOL(MTZIN,SMINWL,SMAXWL)
         RMINWL = 10000
         RMAXWL = 0.5
         IF (SMINWL.GT.0.0) RMINWL=1./SQRT(SMINWL)
         IF (SMAXWL.GT.0.0) RMAXWL=1./SQRT(SMAXWL)
      ENDIF
C     
C---- Check if user wants to override HKLIN symmetry with
C     Key_Word input symmetry
C     
      IF (KYSYMM) THEN
C     
C---- Set up symmetry (RDSYMM must be called first )
C     
C             *********************************************************
         CALL ASUSET(NAMSPG,NSPGRP,PGNAME,NSYM,RSYM,NSYMP,NLAUE,.TRUE.)
C             *********************************************************
C     
      ELSE
C     
C---- CALL to return symmetry information (other than symmetry
C     operations) from the MTZ header
C     
C             ==============================================
         CALL LRSYMI(MTZIN,NSYMP,LATTYP,NSPGRP,NAMSPG,PGNAME)
C             ==============================================
C     
C     
         IF (NSPGRP.GT.0) THEN
C     
C     
C---- CALL to return symmetry operations from header of MTZ file
C     
C                =======================
            CALL LRSYMM(MTZIN,NSYM,RSYM)
C                =======================
C     
C          ********************************************************
      CALL ASUSET(NAMSPG,NSPGRP,PGNAME,NSYM,RSYM,NSYMP,NLAUE,.TRUE.)
C          ********************************************************
C     
         ELSE
            WRITE (LUNOUT,FMT=6004)
 6004       FORMAT (/' NO SYMMETRY in HKLIN and NO Key_Word input ',
     +           'for SYMMETRY ... STOPPING !!!')
            CALL CCPERR(1,'No symmetry')
         END IF
      END IF
C     
C---- Now I can work out centricity and epsilon classes
C     This will replace icent(...) and Centric_zone
C     
C     
C     *************************
      CALL CENTRIC(NSYM,RSYM,0)
      CALL EPSLN(NSYM,NSYMP,RSYM,0)
C     *************************
C     
C     
C---- CALL to apply the input column label assignments to the
C     MTZ file which is open for read and setup the Lookup pointer array
C     Also note that having an unassigned program label is a
C     fatal error, but this CALL uses LERROR to report
C     this as a warning, so that all the program label assignments
C     can be checked before stopping.
C     
C          =========================================
          CALL LRASSN(MTZIN,LSPRGI,NLPRGI,MTZLOK,CTPRGI)
C          =========================================
C     
      NCF = MTZLOK(4)
      NCS = MTZLOK(5)
      NCI = MTZLOK(6)
      IF(NCS .EQ.0 ) NCS = MTZLOK(7)
          IF(MTZLOK(4).EQ.0 .AND. MTZLOK(6).EQ.0) 
     +       CALL CCPERR(1,' **** Neither FP or IP assigned ****')
          IF(MTZLOK(4).NE.0 .AND. MTZLOK(6).NE.0) 
     +       CALL CCPERR(1,' **** Both FP and IP assigned! ****')
C     
C
C
C---------------------- Set up resolution limits if required
C RMINWL, RMAXWL, SMINWL SMAXWL should have been defined either by
C     RESOLUTION command, or from input file
C Set limits for scaling if not set explicitly
      IF (.NOT.SCBLOG) THEN
C  -- no RSCALE defined, set scaling limits == overall limits
         RMAXSC = RMAXWL
         SMAXSC = SMAXWL
         RMINSC = RMINWL
          IF(RMAXSC.LE.3.5) RMINSC = 4.0
         SMINSC = SMINWL
          IF(RMAXSC.LE.3.5) SMINSC = 0.0625
      ELSE
C  -- force scaling limits to lie within overall resolution limits
         SMINSC = MAX(SMINSC,SMINWL)
         SMAXSC = MIN(SMAXSC,SMAXWL)
         RMINSC = 1./SQRT(SMINSC)
         RMAXSC = 1./SQRT(SMAXSC)
      ENDIF
C Set bin width for Truncate analysis
      IF (RANGE .LT. 0.0) THEN
         RANGE = SMAXWL/NRANGE
      ENDIF
C-----------------------
C
C Set up tables of Atomic form-factors
C                 ***********
      CALL WILPRE(NSYM,CELMTZ)
C                 ***********
C
C---- Return here to read all MTZ data records
C
C---- CALL to read a reflection record from an MTZ file which
C     has been opened for read. This returns the record in lookup order.
C
C          ==========================
  110 CALL LRREFF(MTZIN,S,RECIN,MTZEOF)
      CALL LRREFM(MTZIN,LOGMSS)
C
      IF (MTZEOF) GO TO 230
C
C---- resolution limits
      IF (S.GT.SMAXWL) GO TO 110
C   Summ for all input HKL to flag missing data to outrer resln limit
      NT = INT(S/RANGE) + 1
      NT = MIN(NRANGE,NT)
         SUMINSALL(NT) = SUMINSALL(NT) + 1
C          ==========================
C    Must have FP and SIGFP for analysis to continue
C          ==========================
      IF( NCF.NE.0 .AND. (LOGMSS(4) .OR. LOGMSS(5)) ) GO TO 110
      IF( NCI.NE.0 .AND. (LOGMSS(6) .OR. LOGMSS(7)) ) GO TO 110
C
C
      DO 120, I=1,3
         INHKL(I) = NINT(RECIN(I))
 120  CONTINUE
C
       IF(NCF.NE.0) THEN
         FA = RECIN(4)
         SA = RECIN(5)
         FFA = FA*FA
       END IF
C
       IF(NCI.NE.0) THEN
         FFA = RECIN(6)
         FA = 0.0
          IF( RECIN(6).GE.0.00) THEN
            FA = SQRT(ABS(RECIN(6)))
          END IF
        SA = SQRT(ABS(RECIN(6))+RECIN(7)) - FA
       END IF
C
C---- Sums for sd statistics
      IF (SA .GT. 0.000001) THEN
         SUMINS(NT) = SUMINS(NT) + 1
         SUMFIS(NT) = SUMFIS(NT) + FA
         SUMFSS(NT) = SUMFSS(NT) + SA
         SUMFRS(NT) = SUMFRS(NT) + FA/SA
         NREF = NREF + 1
      ENDIF
C
      IF (FA .LT. FMIN) THEN
         FMIN = FA
         SIGMIN = SA
      ENDIF
      IF (FA .GT. FMAX) THEN
         FMAX = FA
         SIGMAX = SA
      ENDIF
C          *********************************************
C---- resolution limits - dont consider data for wilson plot  where S < SMIN

      IF (S.LT.SMINWL .OR. S.GT.SMAXWL) GO TO 110
C          *********************************************
      CALL WILSTA(INHKL,S,FA,SA,NSYM,NREFWL)
C          *********************************************
C
C
      GO TO 110
C
C---- End of file
C
  230 CONTINUE
C
C---- CALL to close an MTZ file which has been opened for read.
C
C          =============
      CALL LRCLOS(MTZIN)
C          =============
C
C
      WRITE (LUNOUT,FMT=6020) NREF
 6020 FORMAT (////' Number of reflections input = ',I10,/)
C
C---- Check if WILSON requested, if so call WILPLT
C
C          ************************************************
      CALL WILPLT(NSYM,NSYMP,NREFWL,WILSC,WILBT,IWITCH)
C          ************************************************
C
C
C---- Print sd statistics
      TOTIN = 0.0
      TOTFI = 0.0
      TOTFS = 0.0
      TOTFR = 0.0
C
      WRITE(LUNOUT,6400)TITLE(1:40),TITLE(1:40),TITLE(1:40)
 6400 FORMAT(/1X,
     $' $TABLE: Amplitude analysis v resln  ',A,':'/
     $' $GRAPHS:<F> v. resolution ',A,':N:2,6:',/,
     $': <F/sd> v. resolution:N:2,9:'/
     $': Nref_obs Nref_miss ',A,':N:2,4,5:'/
     $' $$'/
     $   ' Range  1/resol^2 Dmin    Nref Nref_miss',
     $   '     <F>     <sd>  <F>/<sd>   <F/sd>    $$'/' $$')
C
      DO 400, I=1,KNBINS
         IF (SUMINS(I) .GT. 0.000001) THEN
            TOTIN = TOTIN + SUMINS(I)
            TOTFI = TOTFI + SUMFIS(I)
            TOTFS = TOTFS + SUMFSS(I)
            TOTFR = TOTFR + SUMFRS(I)
            SS = (FLOAT(I)*RANGE)
            DD = 1./SQRT(SS)
            XF=SUMFRS(I)/SUMINS(I)
            AFMEAN=SUMFIS(I)/SUMINS(I)
            SIGFMEAN=SUMFSS(I)/SUMINS(I)
            J=SUMINS(I)
            JJ=SUMINSALL(I) - SUMINS(I)
            WRITE(LUNOUT,6410) I,SS,DD,J,JJ,
     $           AFMEAN,SIGFMEAN,SUMFIS(I)/SUMFSS(I),XF
 6410       FORMAT(1X,I6,F7.4,F9.2,2I8,4F9.2)
         ENDIF
 400  CONTINUE
C
      WRITE(LUNOUT,'(A)') ' $$'
C
      XF=TOTFR/TOTIN
      AFMEAN=TOTFI/TOTIN
      SIGFMEAN=TOTFS/TOTIN
      J=TOTIN
      WRITE(LUNOUT,6420) J,
     $     AFMEAN,SIGFMEAN,TOTFI/TOTFS,XF
 6420 FORMAT(/1X,'   TOTALS      ',I8,4F9.2)
C
      WRITE (LUNOUT,FMT=6036)
     $     FMIN,SIGMIN,FMAX,SIGMAX
 6036 FORMAT (///' Minimum F = ',F10.3,/,
     +           '   with SD = ',F10.3,/,
     +           ' Maximum F = ',F10.3,/,
     +           '   with SD = ',F10.3,//)
C     
      CALL CCPERR(0,'Normal termination')
      END



C
C
C
      SUBROUTINE ATREC(AL,AS,BL,BS,CL,NK,NC,NW)
C     =========================================
C
C---- Extract scattering factor constants from table
C
      CHARACTER*2 NW(8)
      DIMENSION NC(8)
      DIMENSION AL(8),AS(8),BL(8),BS(8),CL(8)
      DIMENSION ALT(50),AST(50),BLT(50),BST(50),CLT(50)
      CHARACTER*2 NA(50)
C
C
      DATA NA/'H','LI','BE','B','C','N','O','F','NA','MG',
     1 'AL','SI','P','S','CL','K','CA','TI','V','CR',
     2 'MN','FE','CO','NI','CU','ZN','AS','SE','BR','RB',
     3 'SR','ZR','MO','RU','RH','PD','AG','CD','SN','SB',
     4 'I','CS','BA','W','OS','PT','AU','HG','PB','BI'/
C
C---- atomic scattering factors for above atom types
C     f = al * exp(-as * rho) + bl * exp(-bs * rho) + cl
C
      DATA ALT/0.388,1.560,1.261,1.207,2.112,3.188,4.197,5.155,7.488,
     1 7.426,7.276,6.988,6.509,5.967,5.557,9.544,10.25,11.69,12.50,
     2 13.67,14.17,15.01,15.84,16.69,17.88,18.32,18.62,18.48,18.18,
     3 17.27,17.95,18.21,18.38,19.56,20.42,20.02,22.37,23.74,25.20,
     4 25.41,25.51,25.57,26.03,36.53,36.50,36.08,36.22,37.13,38.75,
     5 39.16/
      DATA AST/7.151,3.264,2.620,5.745,7.827,7.341,6.327,5.392,4.821,
     1 3.770,3.143,2.739,2.602,2.753,3.176,7.683,7.176,6.300,5.929,
     2 5.789,5.252,4.958,4.687,4.440,4.357,4.019,3.091,2.815,2.638,
     3 3.382,4.361,4.515,4.490,4.686,4.797,4.398,4.881,4.953,4.631,
     4 4.290,3.653,3.556,4.022,3.505,3.460,3.333,3.360,3.585,3.820,
     5 3.714/
      DATA BLT/0.601,1.059,2.008,2.530,2.462,2.305,2.218,2.172,1.280,
     1 2.267,3.192,4.169,5.107,5.925,6.573,2.843,3.086,3.419,3.482,
     2 3.131,3.554,3.573,3.578,3.558,3.088,3.501,5.729,6.744,7.710,
     3 7.397,5.919,6.578,7.602,7.598,7.360,8.892,6.844,6.153,6.411,
     4 7.297,9.410,10.22,9.230,8.741,9.627,11.22,11.46,10.38,9.179,
     5 9.659/
      DATA BST/30.18,108.3,54.77,38.23,31.65,26.84,22.83,19.61,96.71,
     1 69.72,55.67,43.28,34.46,28.44,24.26,53.75,79.64,62.92,57.74,
     2 46.70,49.62,46.50,43.88,41.79,35.63,38.46,35.40,30.87,27.32,
     3 28.90,44.38,42.30,30.85,28.20,27.65,20.92,26.38,32.06,40.81,
     4 39.07,33.09,31.70,38.86,33.47,29.80,23.06,22.12,25.33,33.99,
     5 35.23/
      DATA CLT/0.008,0.3747,0.7163,1.243,1.412,1.498,1.578,1.668,2.216,
     1 2.291,2.491,2.801,3.343,4.073,4.839,6.494,6.616,6.846,6.966,
     2 7.147,7.226,7.375,7.541,7.712,7.986,8.136,8.601,8.727,9.070,
     3 12.09,13.92,15.06,15.88,16.72,17.11,17.04,17.69,18.02,18.30,
     4 18.22,18.02,18.93,20.42,28.57,29.71,30.56,31.19,32.35,33.92,
     5 34.06/
C
C
      DO 150 I=1,NK
C
C----     check atom type
C
      DO 120 J=1,50
      IF(NW(I).NE.NA(J)) GO TO 120
      AS(I)=AST(J)
      AL(I)=ALT(J)
      BS(I)=BST(J)
      BL(I)=BLT(J)
      CL(I)=CLT(J)
      GOTO 150
  120 CONTINUE
      WRITE(6,*)' *** Atomic scattering factor not found ***'
  150 CONTINUE
      RETURN
      END
C
C     
      BLOCK DATA WILBLK
      INTEGER MCOLS
      PARAMETER (MCOLS=500)
      CHARACTER CTPRGI*1,LSPRGI*30,VERSNX*10,TITIN*70,TITLE*80,DATEMT*8
      COMMON /CHRMTZ/LSPRGI(MCOLS),CTPRGI(MCOLS),
     +    VERSNX,TITIN,TITLE,DATEMT
C---- LSPRGI   array containing the program label strings
C     labels input
C     
      DATA (LSPRGI(J),J=1,200)/'H','K','L','FP','SIGFP','IP','SIGIP',
     +                         193*' '/
C     
C---- CTPRGI   array containing the program label types
C     
      DATA (CTPRGI(J),J=1,200)/'H','H','H','F','Q','J','Q',193*' '/
C     
      END
C   
C     
      SUBROUTINE ITRANS(KINT,KCHAR)
C     ============================
C     
C     
C---- Translate integers into character strings.
C     
C     .. Scalar Arguments ..
      INTEGER           KINT
C     ..
C     .. Array Arguments ..
      CHARACTER         KCHAR(5)*1
C     ..
C     .. Scalars in Common ..
      REAL              CONV
      INTEGER           ISYSR,ISYSW,NCOL,NMON
C     ..
C     .. Local Scalars ..
      INTEGER           I,INT2,J
      LOGICAL           NEG
      CHARACTER         BLANK*1,MINUS*1
C     ..
C     .. Local Arrays ..
      CHARACTER         ITABLE(10)*1
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC         ABS,INT
C     ..
C     .. Common blocks ..
      COMMON            /IO/ISYSR,ISYSW,NCOL,CONV,NMON
C     ..
C     .. Data statements ..
      DATA              BLANK/' '/,MINUS/'-'/
      DATA              ITABLE/'0','1','2','3','4','5','6','7','8','9'/
C     ..
C     
C     
      NEG = .FALSE.
      IF (KINT.LT.0) NEG = .TRUE.
C     
C---- Initialize KCHAR to represent 0, in case it is.
C     
      DO 10 I = 1,4
         KCHAR(I) = BLANK
 10   CONTINUE
      KCHAR(5) = ITABLE(1)
      INT2 = ABS(KINT)
      J = 5
 20   CONTINUE
C     
C---- Assume, since it has already been checked, that int
C     can be printed in 5 characters.
C     
      IF (INT2.NE.0) THEN
         I = INT2 - INT(INT2/10.0)*10 + 1
         KCHAR(J) = ITABLE(I)
         J = J - 1
         INT2 = INT(INT2/10.0)
         GO TO 20
      END IF
      IF (NEG) KCHAR(J) = MINUS
      END
C     
C   
C     
      SUBROUTINE KEYIN
C     ================
C     
C      IMPLICIT NONE
C     
C     
C     
C     .. Parameters ..
      INTEGER NPAR
      PARAMETER (NPAR=200)
      INTEGER MAXSER
      PARAMETER (MAXSER=50)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=96)
      INTEGER NPARM
      PARAMETER (NPARM=200)
      INTEGER NLPRGI
      PARAMETER (NLPRGI=7)
      INTEGER MCOLS
      PARAMETER (MCOLS=500)
      INTEGER MAXHIS
      PARAMETER (MAXHIS=30)
C     ..
C     .. Scalars in Common ..
      REAL RANGE
      INTEGER ISOUT,LABFLG,LBOTKN,LENTIT,LUNIN,LUNOUT,MTITOT,
     +     MTZBPR,MTZERR,MTZFLG,MTZIN,MTZOUT,MTZPRT,NBATX,
     +     NHSOUT,NLAUE,NREFLX,NSPGRP,NSYM,NSYMP,
     +     NUMCOL,NRANGE
      LOGICAL KYCELL,KYHIST,KYSYMM,KYTITL,KYRESO,LBOLOG,
     $     MTZEOF
      CHARACTER LATTYP*1,DATEMT*8,LAUNAM*10,NAMSPG*10,PGNAME*10,
     +     VERSNX*10,TITIN*70,TITLE*80
C     ..
C     .. Arrays in Common ..
      REAL CELMTZ,CELOUT,RNGMTZ,RSYM,RSYMT
      INTEGER LBOBEG,LBOEND,MTZLOK
      CHARACTER CTPRGI*1,LSPRGI*30
C     ..
C     .. Local Scalars ..
      INTEGER IPRINT,J,LXX,NTOK,JTOK,I,IKEY,
     $     IAT,NRES
      LOGICAL LEND
      CHARACTER KEY*4,LINE*400
C     ..
C     .. Local Arrays ..
      REAL FVALUE(NPAR)
      INTEGER IBEG(NPAR),IDEC(NPAR),IEND(NPAR),ITYP(NPAR)
      CHARACTER CVALUE(NPAR)*4
C     ..
C     .. External Functions ..
      INTEGER LENSTR
      EXTERNAL LENSTR
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPUPC,KEYNUM,PARSER,RDSYMM
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC LEN,NINT
C     ..
C     .. Common blocks ..
      COMMON /AGRSYM/NSPGRP,NSYM,NSYMP,NLAUE,
     +     RSYM(4,4,MAXSYM),RSYMT(4,4,MAXSYM)
      COMMON /CHRMTZ/LSPRGI(MCOLS),CTPRGI(MCOLS),
     $     VERSNX,TITIN,TITLE,DATEMT
      COMMON /CHRSYM/PGNAME,NAMSPG,LAUNAM,LATTYP
      COMMON /INOUT/LUNIN,LUNOUT,ISOUT
      COMMON /KEYRSC/RANGE,NRANGE
      COMMON /WRKMTZ/MTZIN,MTZOUT,MTZERR,MTZBPR,MTZPRT,MTZFLG,MTITOT,
     +     NBATX,NHSOUT,NUMCOL,NREFLX,LENTIT,LABFLG,MTZEOF,
     +     KYTITL,KYHIST,KYCELL,KYSYMM,KYRESO,MTZLOK(MCOLS),
     +     CELMTZ(6),CELOUT(6),RNGMTZ(2,MCOLS),
     +     LBOBEG(NPARM),LBOEND(NPARM),LBOLOG,LBOTKN
C     ..
C     
      INTEGER NBIN,NMON,IPLFSQ,NK
      REAL RMAXWL,RMINWL,SMINWL,SMAXWL,SMINSC,SMAXSC,RMINSC,RMAXSC,
     $     VPA
      LOGICAL SCBLOG,LOGCON,LOGRES
      LOGICAL WILLOG,WILAPY
      INTEGER IWITCH
      COMMON /WILKEY/ NBIN,NMON,RMAXWL,RMINWL,SMINWL,SMAXWL,SMINSC,
     +     SMAXSC,RMINSC,RMAXSC,VPA,
     +     IPLFSQ,NK
      COMMON /LOGWIL/ WILLOG,WILAPY,IWITCH,SCBLOG
      CHARACTER NW*2
      COMMON /WILCHR/ NW(8)
      COMMON /WILFRM/ NC(8),NO(8),AL(8),AS(8),BL(8),BS(8),CL(8)
      INTEGER NC,NO
      REAL AL,AS,BL,BS,CL
C     
C     ..
      INTEGER NWKEYS
      PARAMETER (NWKEYS=2)
      CHARACTER*12 WKEYS(NWKEYS)
      DATA WKEYS/'OBSERVED','ALL'/
C     
C     
C     
      LBOLOG = .FALSE.
      KYSYMM = .FALSE.
      KYTITL = .FALSE.
      KYRESO = .FALSE.
      SCBLOG = .FALSE.
      WILAPY = .TRUE.
      WILLOG = .TRUE.
      IWITCH = 1
      LOGCON = .FALSE.
      LOGRES = .FALSE.
      IPRINT = 0
      NRANGE = 30
      RANGE  = -1.0
      NBIN = 30
      VPA = 10.0
      IPLFSQ = 0
      NMON = 1000
      TITLE = ' '
C     
C     
 10   CONTINUE
      LINE = '   '
      KEY = ' '
      NTOK = NPAR
C     
C     ***********************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +     .TRUE.)
C     ***********************************************************
C     
C---- End of file?
C     
      IF (LEND) GO TO 50
C
C---- RANGE n  [optional input]
C     ==========================
C
C    Range is the width of ranges on 4sin**2theta/lambda**2
C    for the intensity analysis, OR the number of ranges
C
      IF (KEY.EQ.'RANG') THEN
C
C
        IF (NTOK.LE.1) THEN
           NRANGE = 30
           WRITE (LUNOUT,FMT=6000)
 6000      FORMAT (//' **** KEYWORDING WARNING ****',/,
     +          ' No argument given on RANGE command,',
     $          ' using default = 30 bins')
        ELSE
C
C              ************************************
          CALL KEYNUM(1,2,LINE,IBEG,IEND,ITYP,NTOK)
C              ************************************
C
          RANGE = FVALUE(2)
          IF (RANGE .GT. 1.0) THEN
             NRANGE = MIN(50,NINT(FVALUE(2)))
C  Set RANGE .lt. 0 as flag that NRANGE has been set
             RANGE = -1.0
          ENDIF
C
        END IF
        GO TO 10
C
C
C
C---- TITLE "string"  [OPTIONAL INPUT]
C     ================================
C
C
C
C     Title defaults to 'FROM TRUNCATE'
C
C     otherwise the input string is used
C     Title (max 80 characters) for the output MTZ file
C
      ELSE IF (KEY.EQ.'TITL') THEN
        LXX = LEN(LINE)
        IF (LXX.GT.80) LXX = 80
        TITLE = LINE(IBEG(2) :LXX)
        KYTITL = .TRUE.
        GO TO 10
C
C
C---- HEADER keywords  [OPTIONAL INPUT]
C     =================================
C     Keywords to set flags for printing MTZ header
C       Subkeywords: NONE, BRIEF, HISTORY, ALL, NOBATCH, BATCH, ORIENTATION
      ELSE IF (KEY.EQ.'HEAD') THEN
         CALL RDHEAD(2,LINE,IBEG,IEND,ITYP,FVALUE,NTOK,
     .    MTZPRT,MTZBPR)
         GO TO 10
C
C---- SYMMETRY
C     ========
C
      ELSE IF (KEY.EQ.'SYMM') THEN
        J = 2
C
C            ******************************************
        CALL RDSYMM(J,LINE,IBEG,IEND,ITYP,FVALUE,NTOK,NAMSPG,NSPGRP,
     +              PGNAME,NSYM,NSYMP,RSYM)
C            ********************************************
C
        KYSYMM = .TRUE.
        GO TO 10
C
C---- WILSON
C      do wilson plot
C
      ELSE IF (KEY.EQ.'WILS') THEN
         JTOK = 2
         IF (NTOK .GE. JTOK) THEN
          DO 110, I=JTOK,NTOK
            KEY = LINE(IBEG(I):IEND(I))
            CALL CCPUPC(KEY)
            CALL CHKKEY(KEY,WKEYS,NWKEYS,IKEY)
            IF (IKEY .LE. 0) THEN
               WRITE(LUNOUT,FMT='(A,A)')
     $      'Unrecognized or ambiguous subkeyword to WILSON: ',KEY
            ELSE
               IF (IKEY .EQ. 1) THEN
C     OBSERVED
                  IWITCH = 1
               ELSEIF (IKEY .EQ. 2) THEN
C     ALL
                  IWITCH = 2
               ENDIF
            ENDIF
 110     CONTINUE
      ENDIF
      GO TO 10
C
C---- RESOlution
C      Resolution limits
C
      ELSE IF (KEY.EQ.'RESO') THEN
         KYRESO = .TRUE.
         CALL RDRESO(
     $        2,ITYP,FVALUE,NTOK,RMINWL,RMAXWL,SMINWL,SMAXWL)
         GO TO 10
C
C---- RSCALE    used for scaling.
C     Resolution limits - either 4sinsq/lambdasq or d
C
      ELSE IF (KEY.EQ.'RSCA') THEN
         SCBLOG = .TRUE.
         CALL RDRESO(
     $        2,ITYP,FVALUE,NTOK,RMINSC,RMAXSC,SMINSC,SMAXSC)
         GO TO 10
C
C---- VPAT   volume per atom - default = 10
C
      ELSE IF (KEY.EQ.'VPAT') THEN
         CALL GTPREA(2,VPA,NTOK,ITYP,FVALUE)
         GOTO 10
C
C---- CONTents  - unit cell content
C
      ELSE IF (KEY.EQ.'CONT') THEN
C
C---- Check if Key_Word NRESidue given first, if so this
C     has priority and ignore keyword CONTents
C
         IF (LOGRES) THEN
            WRITE (LUNOUT,6030)
 6030       FORMAT (' You have already used the Key_Word NRESIDUE',/,
     +           ' IGNORING this CONTENTS line')
            GO TO 10
         END IF
C
C
         LOGCON = .TRUE.
         IAT = 0
C
C
         DO 71 I=2,NTOK,2
            IAT = IAT + 1
            NW(IAT)= LINE( IBEG(I):(IBEG(I)+1) )
C   Very rough figures but better than previous!! 
            CALL CCPUPC(NW(IAT))
            CALL GTPINT(I+1,NC(IAT),NTOK,ITYP,FVALUE)
 71      CONTINUE
C
C
         NK=IAT
         GOTO 10
C
C---- NRESIDUE   Nres
C
C       Nres iNrese number of residues expected in thein thein thein t
C
C          A very approximate atom composition is calculated
C                                                 
C                 add   adne ordered water per amino acid = cad = c
C                 This is then taken as aken as aken as aken as aken as
C          as number of atoms in asymmetric unit.
C
      ELSE IF (KEY.EQ.'NRES') THEN
         IF (NTOK.LE.1) THEN
            WRITE (LUNOUT,6040)
 6040       FORMAT(' ERROR ---- for Key_Word NRESIDUES ',/,
     +           ' NO value given for nres ... Stopping')
            CALL CCPERR(1,'Must give number of residues')
         END IF
C
C
         CALL GTPINT(2,NRES,NTOK,ITYP,FVALUE)
         NW(1) = 'C'
         NC(1) = 5 * NRES
         NW(2) = 'N'
         NC(2) = NINT(1.35 * NRES)
         NW(3) = 'O'
         NC(3) = NINT(1.5 * NRES)
         NW(4) = 'H'
         NC(4) = 8 * NRES
         NK    = 4
         LOGRES = .TRUE.
         GO TO 10
C
      ELSE IF (KEY.EQ.'LABI') THEN
C
C             ***************************************
         CALL LKYIN(1,LSPRGI,NLPRGI,NTOK,LINE,IBEG,IEND)
C             ***************************************
        LXX = LEN(LINE)
        IF( .NOT.KYTITL) TITLE = LINE(IBEG(2) :LXX)
        KYTITL = .TRUE.
C
       ELSE IF (KEY.EQ.'END') THEN
         GO TO 50
      ELSE
        WRITE (LUNOUT,FMT=6022) LINE(1:LENSTR(LINE))
 6022   FORMAT (' Error: Key_Word line NOT Understood:-',/' ',A)
        GO TO 10
      END IF
         GO TO 10
C
C
   50 CONTINUE
C
C
C---- Check if compulsory Key_Word  CONT   or NRES used
C
      IF (.NOT.LOGCON .AND. .NOT.LOGRES) THEN
         WRITE (LUNOUT,6026)
 6026    FORMAT (
     $        ' No CONTENTS or NRESIDUE Key_Word given with WILSON',/,
     +        ' Compulsory Key_Word required... STOPPING')
         CALL CCPERR(1,'CONTENTS or NRESIDUE command compulsory')
      ENDIF
C
C
      END
C     
C   
C     
      SUBROUTINE LABEL(VMIN,VMAX,LOWV,INCREM,NTICK,IPOWER)
C     ====================================================
C     
C     
C---- From range, determine smallest increment that give
C     .le. 15 ticks on axis.
C     
C     .. Scalar Arguments ..
      REAL             VMAX,VMIN
      INTEGER          INCREM,IPOWER,LOWV,NTICK
C     ..
C     .. Local Scalars ..
      REAL             POWER
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC        INT,LOG10
C     ..
C     
C     
      POWER = LOG10(VMAX-VMIN)
C     
C---- Make sure to correct for fact that ifix of negative
C     number is not the same as for positive.
C     
      IF (POWER.GE.0.0) IPOWER = INT(POWER) - 1
      IF (POWER.LT.0.0) IPOWER = INT(POWER) - 2
      INCREM = INT((VMAX-VMIN)/ (10.0**IPOWER*15.0)) + 1
C     
C---- For aesthetic reasons, choose increment of 1,2,5 or 10
C     
      IF (INCREM.GT.2 .AND. INCREM.LT.5) INCREM = 5
      IF (INCREM.GT.5 .AND. INCREM.LT.10) INCREM = 10
      LOWV = (INT(VMIN/ (10.0**IPOWER*INCREM))+1)*INCREM
      IF (VMIN.LT.0.0) LOWV = LOWV - INCREM
      NTICK = INT(VMAX/ (10.0**IPOWER*INCREM)) - LOWV/INCREM + 1
C     
C---- Check for labels too large or small to print with 5
C     characters.  ntick of 0 is flag for this.
C     
      IF (LOWV.LT.-9999 .OR. (LOWV+NTICK*INCREM).GT.99999) NTICK = 0
      END
C     
C   
C     
      SUBROUTINE LSQ(X,Y,NOBS,SIG,MWT,A,B,SIGA,SIGB,CHI2,Q)
C     =====================================================
C     
C     
c     to display graph parameter
c     Calls lsq routine and writes out the numbers associated
c     with prediced lsq fit
c     
c     Author   T J Oldfield
c     
c     ( York  ---   January 1988)
c     very general lsq program
c     From "Numerical recipes"
c     
c     date January 1988
c     Given a set of nobs points x(i), y(i) with SD sig(i), fit
c     them to a straight line y=a+bx by minimising x**2. Returned
c     A,B and their respective probable uncertainties siga and sigb
c     , the chi-square chi2, and the goodness-of-fit probability
c     q. If mwt=0, on input to the routine then the sd are assumed 
c     to be unavailable: q is returned as 1.0 and the nobsrmalisation 
c     of chi2 is to unit sd on all the points
c     
c     implicit none
C     
C     
      integer i,mwt
      real a,b,siga,sigb,chi2,q,ss,wt,sigdat
      real x(nobs),y(nobs),sig(nobs),t
      real sxx,syy,sxy,sx,sy,st2,sxoss
C     
      sxx=0
      syy=0
      sxy=0
      sx=0
      sy=0
      st2=0
      b=0
C     
      if (mwt.eq.0)then
         do 10 i=1,nobs
            sig(i)=1.0
 10      continue
      endif
C     
C     
      ss=0
      do 11 i=1,nobs
         wt=1.0/(sig(i)**2)
         ss=ss+wt
         sxx=sxx+ x(i)*x(i)* wt
         sx=sx+x(i)*wt
         sxy=sxy+x(i)*y(i)*wt
         syy=syy+y(i)*y(i)*wt
         sy=sy+y(i)*wt
 11   continue
C     
c     ----	wt = 1/sig(i)**2 = 1.0 if sig(i) not set
c     ----	sx = sum of x*wt
c     ----  sy = sum of y*wt
c     ----  ss = sum of wt   = nobs if sig(i) not set
c     ---- sxx= sum of x*x*wt
c     ----  syy= sum of y*Y*wt
c     ----  sxy= sum of x*y*wt
C     
c     Find a and b following recipe p 508
C     
      sxoss=sx/ss
C     
C     
      do 13 i=1,nobs
         t=(x(i)-sxoss)/sig(i)
         st2=st2+t*t
         b=b+t*y(i)/sig(i)
 13   continue
C     
C     
      b=b/st2
      a=(sy-sx*b)/ss
      siga=sqrt((1.0+sx*sx/(ss*st2))/ss)
      sigb=sqrt(1.0/st2)
C     
c     ---	Q  is returned as the correlation coefficient	
c     
c     This is TJO original......
C     
      q=(sxy-(sx*sy/ss))/sqrt((sxx-(sx*sx/ss))*(syy-(sy*sy/ss)))
      chi2=0
C     
c     Return Sig(i) as error(y) squared        
C     
      do 16 i=1,nobs
         chi2=chi2+((y(i)-a-b*x(i))/sig(i))**2
         tmp=(y(i)-a-b*x(i))/sig(i)
         sig(i)=abs(tmp)
 16   continue
C     
      sigdat=sqrt(chi2/(ss-2.0))
      chi2=sigdat
      siga=siga*sigdat
      sigb=sigb*sigdat
      q = sigdat
C     
c     q=gammq(0.5*(nobs-2),0.5*chi2)
c     *************  the above is not implemented *****
C     
      return
      end
C     
C   
C     
      SUBROUTINE PLOTR(XY,NXY,ILSQ,CXY,TITLEX,TITLEY,LEGEND)
C     =================================================
C     
C     
C---- Given an array of x,y pairs, this s/r fills in an array which,
C     when printed in "a" format, constitutes a 2-d graph of
C     the points, properly scaled and nicely labelled.  the
C     arguments supplied are, in order, the array of x,y pairs,
C     number of points, the character to represent the points,
C     title(in 20a1 format) to be printed under the x axis, title
C     to be printed along the y axis, and legend to be printed under
C     the x axis title.
C     
C     
C---- Variables for lsq line fit - test which points should
C     be excluded to give a good slope used to estimate
C     coordinate error
C     
C     .. Scalar Arguments ..
      INTEGER          NXY
      CHARACTER        CXY*1
C     ..
C     .. Array Arguments ..
      REAL             XY(2,100)
      CHARACTER        LEGEND*60,TITLEX*20,TITLEY*20
C     ..
C     .. Local Scalars ..
      REAL             XMAX,XMIN,XRANGE,YMAX,YMIN,
     +     YRANGE
      INTEGER          I,I1,I2,INCX,INCY,IPOWRX,IPOWRY,
     +     ISCALE,J,K,KINT,LOWX,LOWY,NOBS,
     +     NTICKX,NTICKY
      CHARACTER        BAR*1,BLANK*1,CORNER*1,DASH*1,LE*1,ONE*1,STAR*1,
     +     TICK*1
C     ..
C     .. Local Arrays ..
      CHARACTER        KCHAR(5)*1,PLOT(129,60)*1
C     ..
C     .. External Subroutines ..
      EXTERNAL         ITRANS,LABEL,LSQ
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC        INT,SQRT
C     ..
C     .. Data statements ..
C     
C---- Choose most suitable characters available on printer.
C     
      DATA             BLANK/' '/,BAR/'|'/,DASH/'-'/,ONE/'1'/,
     +     CORNER/'+'/
      DATA             TICK/'+'/,LE/'E'/,STAR/'*'/
C     ..
C     
C     Try 0 0  - not set  ????????????????????????????????
      LOWX = 0
      LOWY = 0
C     
C     
      NOBS = NXY
      IF (NOBS.GT.50) THEN
         WRITE(6,'(a,i3,a)')
     1 ' **  RESET NOBS TO 50 FOR PLOTTING  ** ',NXY,' OBSERVATIONS **'
         NOBS = 50
      END IF
C     
C     
      XMIN = 1.0E30
      XMAX = -1.0E30
      YMIN = 1.0E30
      YMAX = -1.0E30
C     
C---- Determine outer bounds of x and y values
C     
      IF (NOBS.GE.1) THEN
         DO 50 I = 1,NOBS
            IF (XY(1,I).LT.XMIN) XMIN = XY(1,I)
            IF (XY(1,I).GT.XMAX) XMAX = XY(1,I)
            IF (XY(2,I).LT.YMIN) YMIN = XY(2,I)
            IF (XY(2,I).GT.YMAX) YMAX = XY(2,I)
 50      CONTINUE
         XRANGE = XMAX - XMIN
         YRANGE = YMAX - YMIN
         IF (XRANGE.NE.0.0 .AND. YRANGE.NE.0.0) THEN
C     
C---- Add 5% margin on each edge
C     
            XMAX = 0.05*XRANGE + XMAX
            XMIN = XMIN - 0.05*XRANGE
            YMAX = 0.05*YRANGE + YMAX
            YMIN = YMIN - 0.05*YRANGE
            XRANGE = 1.1*XRANGE
            YRANGE = 1.1*YRANGE
C     
C---- Determine how axes are to be labelled.
C     
C     
C     
            CALL LABEL(XMIN,XMAX,LOWX,INCX,NTICKX,IPOWRX)
            CALL LABEL(YMIN,YMAX,LOWY,INCY,NTICKY,IPOWRY)
C     
C---- Now initialize blanks and axes for character array
C     representing plot of graph
C     
            DO 70 J = 1,60
               DO 60 I = 1,129
                  PLOT(I,J) = BLANK
                  IF (I.EQ.9 .AND. J.LT.55) PLOT(I,J) = BAR
                  IF (J.EQ.55 .AND. I.GT.9) PLOT(I,J) = DASH
 60            CONTINUE
 70         CONTINUE
C     
C     
C     
            PLOT(1,1) = ONE
            PLOT(9,55) = CORNER
            DO 80 I = 1,50
               I1 = I + 40
               I2 = I + 20
               PLOT(I1,58) = ' '
               IF(I2.LE.60)PLOT(2,I2) =  ' '
               PLOT(I+40,60) = LEGEND(I:I)
C     
               IF(I.GT.20) GO TO 80
               PLOT(I1,58) = TITLEX(I:I)
               PLOT(2,I2) = TITLEY(I:I)
 80         CONTINUE
C     
C---- If x axis labels can fit into 5 characters, label x axis.
C     
C     
            IF (NTICKX.NE.0) THEN
               KINT = LOWX - INCX
               DO 100 I = 1,NTICKX
                  KINT = KINT + INCX
                  J = INT((10.0**IPOWRX*KINT-XMIN)/XRANGE*120.0) + 9
                  IF (J.GE.9 .AND. J.LE.125) THEN
                     PLOT(J,55) = TICK
                     CALL ITRANS(KINT,KCHAR)
                     DO 90 I2 = 1,5
                        IF (KCHAR(I2).NE.BLANK) THEN
                           PLOT(J,56) = KCHAR(I2)
                           J = J + 1
                        END IF
 90                  CONTINUE
                  END IF
 100           CONTINUE
C     
C---- If labels have been multiplied by a factor of 10, indicate
C     this factor on the x axis title.
C     
               IF (IPOWRX.NE.0) THEN
                  ISCALE = -IPOWRX
                  CALL ITRANS(ISCALE,KCHAR)
                  PLOT(62,58) = STAR
                  PLOT(63,58) = ONE
                  PLOT(64,58) = '0'
                  PLOT(65,58) = '*'
                  PLOT(66,58) = '*'
                  I = 67
                  DO 110 J = 1,5
                     IF (KCHAR(J).NE.BLANK) THEN
                        PLOT(I,58) = KCHAR(J)
                        I = I + 1
                     END IF
 110              CONTINUE
               END IF
            END IF
C     
C---- Now label y axis
C     
            IF (NTICKY.NE.0) THEN
               KINT = LOWY - INCY
               DO 130 I = 1,NTICKY
                  KINT = KINT + INCY
                  J = INT((YMAX-10.0**IPOWRY*KINT)/YRANGE*55.0)
                  IF (J.GT.0 .AND. J.LE.55) THEN
                     PLOT(9,J) = TICK
                     CALL ITRANS(KINT,KCHAR)
                     DO 120 I2 = 1,5
                        K = I2 + 3
                        PLOT(K,J) = KCHAR(I2)
 120                 CONTINUE
                  END IF
 130           CONTINUE
C     
C---- Indicate multiple of 10.
C     
               IF (IPOWRY.NE.0) THEN
                  ISCALE = -IPOWRY
                  CALL ITRANS(ISCALE,KCHAR)
                  PLOT(2,42) = STAR
                  PLOT(2,43) = ONE
                  PLOT(2,44) = '0'
                  PLOT(2,45) = '*'
                  PLOT(2,46) = '*'
                  I = 47
                  DO 140 J = 1,5
                     IF (KCHAR(J).NE.BLANK) THEN
                        PLOT(2,I) = KCHAR(J)
                        I = I + 1
                     END IF
 140              CONTINUE
               END IF
            END IF
C     
C---- Finally ready to map points into array.
C     
            DO 150 I = 1,NXY
               J = INT((XY(1,I)-XMIN)/XRANGE*120.0) + 9
               K = INT((YMAX-XY(2,I))/YRANGE*55.0)
               IF(J.GE.1 .AND. J.LE.129 .AND.
     1              K.GE.1 .AND. K.LE.60)
     1              PLOT(J,K) = CXY
 150        CONTINUE
C     
C---- Plot is ready for printer.
C     
            WRITE (6,FMT=9020) PLOT
         END IF
      END IF
      RETURN
C     
C---- Format statements
C     
 9020 FORMAT (59 (129A1,/),129A1)
      END
C     
C   
C     
      SUBROUTINE WILPLT(NSYM,NSYMP,NREFWL,WILSC,WILBT,IWITCH)
C     ======================================================
C     
C**********************ERRORvariable CORRN Not defined
C     setting this to 1.0 as I dont know
C     what it means
      INTEGER MCOLS
      PARAMETER (MCOLS=500)
C     .. Scalars in Common ..
      CHARACTER DATEMT*8, VERSNX*10,TITIN*70,TITLE*80
      CHARACTER CTPRGI*1,LSPRGI*30
C     .. Common blocks ..
      COMMON /CHRMTZ/LSPRGI(MCOLS),CTPRGI(MCOLS),
     $     VERSNX,TITIN,TITLE,DATEMT
C     
      INTEGER IWITCH
      REAL WILSC(2),WILBT(2)
      REAL FF1(100),FF2(100),XLSQ(100),YLSQ(100),SIG(100)
      REAL             FFAVV1(100),FFAVV2(100)
      CHARACTER        LEGEND*60,TITLEX*20,TITLEY*20
C     
C     
      COMMON /WILKEY/ NBIN,NMON,RMAX,RMIN,SMIN,SMAX,SMINSC,
     +     SMAXSC,RMINSC,RMAXSC,VPA,
     +     IPLFSQ,NK
      COMMON /PREWIL/ DSS(100),SI(100),SN(100),SR(100),SW(100),
     +     NRFUNQ(100),NRFLCF(100),XY(2,100),XYAV(2,100),
     +     FFS(500),DSSQ(500),PI,DELDSS,NATMS,NELEC,F000,
     +     ISMAX,VOL,RR(3,3,6),CELLAS(6),RVOL,FRAC
      CHARACTER NW*2
      COMMON /WILCHR/ NW(8)
      COMMON /WILFRM/ NC(8),NO(8),AL(8),AS(8),BL(8),BS(8),CL(8)
C     
C---- ?????????????
      CORRN = 1.0
C---- ?????????????
C     
C     
C     
C---- Calculate points for wilson plot
C     
C     Plot log(fp*fp/ff) = log (1/k) -2B*STHSQ
C     = log (1/k) - B*S/2
C     
      JJ= 0
      WRITE(6,'(1X,a)') '  *******  Wilson Plots *******'
      WRITE(6,6543)
 6543 FORMAT(/,
     +'  Nref is the number of observed reflections in a',/,
     +'       hemisphere of reciprocal space.',/,
     +'  N_unq is an estimate of the number of possible reflections',/,
     +'        in an asymmetric unit of reciprocal space ',/,
     +'        ( Nref should be approximately equal to Nsymp*N_unq)',/,
     +'  <ff>         is the expected value of f**2',/,
     +'  <s - resln>  ',
     $     'is the average value of 4(sin theta/lambda)**2 and ',/,
     +'               the corresponding resolution limit.',/,
     +'  <fobssq>     is the average value of Fobs**2')
      WRITE(6,6533)
 6533 FORMAT(//,
     +'  If the reflections which were not measured were all weak,',
     +' then <Fobs**2>'/
     $'   is better estimated using all possible',
     +'  reflections N_unq'/
     $'   (Option WILSON ALL). THIS SHOULD NOT',
     $' NORMALLY BE USED'/
     +'  ln(<(Fo**2)1>)/<ff>  uses the average derived from Nref,',/,
     +'  ln(<(Fo**2)2>)/<ff>  uses the average derived from N_unq.')
C     
C     
      DO 301 I=1,NBIN
         S = DELDSS*I
         NRFUNQ(I) = NINT(0.6667*PI*S**1.5 /( RVOL*NSYM))
         IF(I.EQ.1)NOPT = NRFUNQ(I)
         IF(I.GT.1)NOPT = NRFUNQ(I) - NRFUNQ(I-1)
         IF(SW(I).EQ.0.0 .OR. SN(I) .EQ.0.0) GO TO 301
         JJ = JJ + 1
         SW(I)= SW(I)/SN(I)
         SR(I) = SR(I)/SN(I)
         FFAV1 = SI(I)/SN(I)
         IFFAV1 = NINT(FFAV1)
         RESI = 1.0/SQRT( SR(I) )
         NRFF = NINT(SN(I))
         FF1(JJ) = ALOG(FFAV1/SW(I))
         FFAVV1(JJ) = FFAV1
C     
C---- Now assume all missing Fobs = 0  - ie <Fobs**2> will equal
C     Sig(fobs**2)/ N_unq*nsymp
C     No need to change <ff>  <s**2/l**2> - they won't alter much
C     for each range....
C     
         FFAV2  = SI(I) /(NOPT*NSYMP)
         IFFAV2 = NINT(FFAV2)
         FFAVV2(JJ) = FFAV2
         FF2(JJ) = ALOG(FFAV2/SW(I))
C     
C---- Divide 4sinsq/lsq by 2.0 for lsq  - see equation for b/scale
C     
         XYAV(1,JJ) =   SR(I)
         XY(1,JJ) =   SR(I)
         XLSQ(JJ) = XY(1,JJ)/2.0
         SIG(JJ) = 1.0
C     
C---- First wilson plot uses ff1(...)
C     
         XYAV(2,JJ) = FFAVV1(JJ)
         XY(2,JJ) = FF1(JJ)
         YLSQ(JJ) = XY(2,JJ)
 301  CONTINUE
C
C     
      WRITE (6,9876) NREFWL
 9876 FORMAT (//' A total of ',I9,
     +     ' reflections were included in the Wilson plot',/)
C     
      TITLEX   = ' 4*sinsq/lambdasq'
      TITLEY   = '   mean(FPsq)   '
      LEGEND   = '   Mean Fsq calculated using observed reflns only'
      IF(IPLFSQ.EQ.1)WRITE(6,'(''1'')')
C     
C     
      IF(IPLFSQ.EQ.1)CALL PLOTR(XYAV,JJ,0,'*',TITLEX,TITLEY,LEGEND)
C     
C     
      TITLEX   = '   4*sinsq/lambdasq'
      TITLEY   = '  log(FPsq/<ff>)   '
      LEGEND   = '   WILSON PLOT (observed reflections only)'
      WRITE(6,'(A)') '1'
C     
C     
      CALL PLOTR(XY,JJ,0,'*',TITLEX,TITLEY,LEGEND)
C     
C---- Calculate points for wilson plot 1.
C     Plot log(fp*fp/ff) = log (1/k) -2B*STHSQ
C     = log (1/k) - B*S/2
C     
      CALL LSQ(XLSQ,YLSQ,JJ,SIG,1,A,B,SIGA,SIGB,CHI2,Q)
C     
C     
      SIGRMS = -SIGB*CORRN
      WRITE (6,'(///,a)')'    WILSON PLOT 1 '
      WRITE (6,FMT='(A,F15.6)') '  LSQ Line Gradient =    ',B
      WRITE (6,FMT='(A,E15.4)')
     1     ' Uncertainty in Gradient  = ',SIGB
      WRITE (6,FMT='(A,E15.4)') '  X Intercept          = ',A
      WRITE (6,FMT='(A,E15.4)')
     1     ' Uncertainty in Intercept = ',SIGA
C     
C---- Calculate points for wilson plot
C     
C     Plot log(fp*fp/ff) = log (1/k) -2B*STHSQ
C     = log (1/k) - B*S/2
C     
      WRITE(6,'(///,a,/,a)')
     1     ' For a wilson plot          B          =  -  gradient',
     1     '                          SCALE        = exp( - intercept).'
C     
C     
      WILBT(1) = -  B
      WILSC(1) = EXP(-A)
      WRITE (6,1120) WILBT(1),WILSC(1)
 1120 FORMAT(//,' Least squares straight line gives:   B  =',
     1     F7.3,8X,'SCALE  =',F10.5,
     1     /,'    where  F(absolute)**2 = ',
     1     'SCALE*F(observed)**2*EXP(-B*2*SINTH**2/L**2) ',///)
C     
C---- Now assume all missing Fobs = 0  - ie <Fobs**2> will equal
C     Sig(fobs**2)/ N_unq*nsymp
C     No need to change <ff>  <s**2/l**2> - they won't alter much
C     for each range....
C     
C---- Second wilson plot uses ff2(...)
C     
      IF (IWITCH .EQ. 2) THEN
         DO 401 I=1,JJ
            XYAV(2,I) = FFAVV2(I)
            XY(2,I) = FF2(I)
            YLSQ(I) = XY(2,I)
 401     CONTINUE
C     
C     
         TITLEX = '   4*sinsq/lambdasq'
         TITLEY = '   mean(FPsq)   '
         LEGEND = '   Mean Fsq calculated using n= all possible reflns'
         IF(IPLFSQ.EQ.1)WRITE(6,'(A)') '1'
         IF(IPLFSQ.EQ.1)CALL PLOTR(XYAV,JJ,0,'*',TITLEX,TITLEY,LEGEND)
C     
C     
         TITLEX   = ' 4*sinsq/lambdasq'
         TITLEY   = ' log(FPsq/<ff>)  '
         LEGEND   =
     +        '   WILSON PLOT (all possible reflections)'
         WRITE(6,'(A)') '1'
         CALL PLOTR(XY,JJ,0,'*',TITLEX,TITLEY,LEGEND)
C     
C---- Calculate points for wilson plot
C     Plot log(fp*fp/ff) = log (1/k) -2B*STHSQ
C     = log (1/k) - B*S/2
C     
         CALL LSQ(XLSQ,YLSQ,JJ,SIG,1,A,B,SIGA,SIGB,CHI2,Q)
C     
C     
         SIGRMS = -SIGB*CORRN
         WRITE (6,'(///,a)')'    WILSON PLOT 2 '
         WRITE (6,FMT='(A,F15.6)') '  LSQ Line Gradient =    ',B
         WRITE (6,FMT='(A,E15.4)')
     1        ' Uncertainty in Gradient  = ',SIGB
         WRITE (6,FMT='(A,E15.4)') '  X Intercept          = ',A
         WRITE (6,FMT='(A,E15.4)')
     1        ' Uncertainty in Intercept = ',SIGA
C     
C---- Calculate points for wilson plot
C     
C     Plot log(fp*fp/ff) = log (1/k) -2B*STHSQ
C     = log (1/k) - B*S/2
C     
         WILBT(2) = -  B
         WILSC(2) = EXP(-A)
         WRITE (6,1121) WILBT(2),WILSC(2)
 1121    FORMAT(//,' Least squares straight line gives:   B  =',
     1        F7.3,8X,'SCALE  =',F10.5,
     1        /,'    Where  F(Absolute)**2 = ',
     1        'SCALE*F(observed)**2*EXP(-B*2*SINTH**2/L**2) ',///)
C     
      ENDIF
C
       WRITE(6,'(///A,F5.1,A,/,A,F5.1,A,/,A,/)')
     $     ' $TABLE: Wilson Plot - Suggested Bfactor',WILBT(1),':',
     $     ' $GRAPHS: Wilson plot -Suggested Bfactor',WILBT(1),
     $                                          ':A:5,8,10:','  $$'
      WRITE(6,'(a,a/a/a)')
     1     '   i  nref  N_unq   <ff>  1/resol^2 resln   <(Fo**2)1>',
     1     '  ln<(Fo**2)1>/<ff> <(Fo**2)2>  ln<(Fo**2)2>/<ff> ',
     $     ' $$',' $$'
C
      DO 302 I=1,NBIN
         S = DELDSS*I
         NRFUNQ(I) = NINT(0.6667*PI*S**1.5 /( RVOL*NSYM))
         IF(I.EQ.1)NOPT = NRFUNQ(I)
         IF(I.GT.1)NOPT = NRFUNQ(I) - NRFUNQ(I-1)
         IF(SW(I).EQ.0.0 .OR. SN(I) .EQ.0.0) GO TO 302
         JJ = JJ + 1
         FFAV1 = SI(I)/SN(I)
         IFFAV1 = NINT(FFAV1)
         RESI = 1.0/SQRT( SR(I) )
         NRFF = NINT(SN(I))
         FF1(JJ) = ALOG(FFAV1/SW(I))
         FFAVV1(JJ) = FFAV1
C
C---- Now assume all missing Fobs = 0  - ie <Fobs**2> will equal
C     Sig(fobs**2)/ N_unq*nsymp
C     No need to change <ff>  <s**2/l**2> - they won't alter much
C     for each range....
C
         FFAV2  = SI(I) /(NOPT*NSYMP)
         IFFAV2 = NINT(FFAV2)
         FFAVV2(JJ) = FFAV2
         FF2(JJ) = ALOG(FFAV2/SW(I))
C
         WRITE(6,'(I4,2I6,F11.0,F9.4,F6.2,I10,4X,F9.5,4X,I10,4X,F9.5)')
     1        I,NRFF,NOPT,SW(I),SR(I),RESI,IFFAV1,FF1(JJ),IFFAV2,FF2(JJ)
 302  CONTINUE
C
      WRITE (6,'(A)') ' $$'
C     
      RETURN
      END
C     
C   
C     
      SUBROUTINE WILPRE(NSYM,CELL)
C     ============================
C     
C      IMPLICIT NONE
C     
      INTEGER NSYM
      REAL CELL(6)
C     
C     
      INTEGER NBIN,NMON,IPLFSQ,NK
      REAL RMAX,RMIN,SMIN,SMAX,SMINSC,SMAXSC,RMINSC,RMAXSC,
     $     VPA
      COMMON /WILKEY/ NBIN,NMON,RMAX,RMIN,SMIN,SMAX,SMINSC,
     +     SMAXSC,RMINSC,RMAXSC,VPA,
     +     IPLFSQ,NK
      REAL DSS,SI,SN,SR,SW,XY,XYAV,FFS,DSSQ,PI,DELDSS,F000,
     +     VOL,RR,CELLAS,RVOL,FRAC
      INTEGER  NRFUNQ,NRFLCF,NATMS,NELEC,ISMAX
      COMMON /PREWIL/ DSS(100),SI(100),SN(100),SR(100),SW(100),
     +     NRFUNQ(100),NRFLCF(100),XY(2,100),XYAV(2,100),
     +     FFS(500),DSSQ(500),PI,DELDSS,NATMS,NELEC,F000,
     +     ISMAX,VOL,RR(3,3,6),CELLAS(6),RVOL,FRAC
      CHARACTER NW*2
      COMMON /WILCHR/ NW(8)
      INTEGER NC,NO
      REAL AL,AS,BL,BS,CL
      COMMON /WILFRM/ NC(8),NO(8),AL(8),AS(8),BL(8),BS(8),CL(8)
C
C Locals
      INTEGER LUNOUT,I,J
      REAL FFF,S0,S1,SS,FZ

C
      LUNOUT = 6
C     
C     
      IF (NBIN.GT.50) THEN
         NBIN = 50
         WRITE(LUNOUT,'(A,I5)')
     1        '   Maximum number of Ranges for analysis reset to ',NBIN
      END IF
C     
C     
C     
C     
      WRITE (LUNOUT,FMT=9010) RMIN,RMAX,SMIN,SMAX
 9010 FORMAT (/,
     +     '   Resolution limits in As = ',2F10.2,/,
     +     '            as  4sinsq/lsq = ',2F10.5/)
      WRITE (LUNOUT,FMT=9011) RMINSC,RMAXSC,SMINSC,SMAXSC
 9011 FORMAT (/,
     +     '   Resolution limits used for scaling in As = ',2F10.2,/,
     +     '                             as  4sinsq/lsq = ',2F10.5/)
C     
C     
      CALL ATREC(AL,AS,BL,BS,CL,NK,NC,NW)
C     
C     
C     IF(IPLFSQ.EQ.0)WRITE (LUNOUT,'(/,A)') ' No plot of Fsq '
C     
C---- Grid on 4sinsq/lambdasq
C     
      DELDSS = SMAX/NBIN
C     
C---- How many electrons in asym unit??
C     
      NATMS = 0
      NELEC = 0
C     
C     
      DO 151 I=1,NK
         FFF = AL(I) + BL(I) + CL(I)
         NO(I) = NINT(FFF)
         NELEC=NELEC+ NSYM*NO(I)*NC(I)
         NATMS=NATMS+NSYM*NC(I)
 151  CONTINUE
C     
C     
      F000 = NELEC
C     
C     
      CALL RBFRO1(CELL,VOL,RR)
      CALL RBRCEL(CELLAS,RVOL)
C     
C     
      FRAC = FLOAT(NATMS)*VPA/VOL
C     
C     
      WRITE (LUNOUT,'(///,1X,A)')
     +     '   ****   Volume solvent content etc ***'
      WRITE (LUNOUT,2800)
     +     (NW(I),NC(I),NO(I),AL(I),AS(I),BL(I),BS(I),CL(I),I=1,NK)
 2800 FORMAT(/,
     +     '   Asymmetric Unit Contents            ',5X,
     +     '        Scattering Factor Constants',/,
     +     '   Atom  number in  A.U.  Atomic number',5X,
     +     '(F = AA*EXP(-A*RHO) + BB*EXP(-B*RHO) + CC)',/,
     +     (4X,A4,2I13,10X,5F9.3))
      WRITE (LUNOUT,2820)
     +     VPA,NATMS,VOL,F000,FRAC,RMIN,RMAX,DELDSS
 2820 FORMAT(/,
     1     ' Volume per atom                         = ',F13.1,
     +     ' e/A**3',/,
     1     ' total number of atoms in unit cell      = ',I13,/,
     2     ' unit cell volume                        = ',F13.1,/,
     2     ' F000                                    = ',F13.1,/,
     3     ' fraction of unit cell occupied by atoms = ',F13.3,
     $     '  <====='/,
     4     ' starting resolution                     = ',F13.2,/,
     5     ' finishing resolution                    = ',F13.2,/,
     5     ' resolution increment for plotting       = ',F13.2//)
C     
C---- Number of expected reflections per shell
C     
      S0 = 0.0
      S1 = DELDSS
C      WRITE(LUNOUT,1666)
C 1666 FORMAT(/,
C     1     '     4SS/LL       <4SS/LL>   DSTAR  RESO     ',
C     1     'ff          NREF(unique)')
C     
C     
      DO 160 I = 1,NBIN + 1
         DSS(I) = (I-0.5)*DELDSS
         FFS(I) = 0.0
         SS = DSS(I)
C     
C     
         DO 156 J=1,NK
            FZ = (AL(J)*EXP(-AS(J)*SS/4.0) + BL(J)*EXP(-BS(J)*SS/4.0)
     1           + CL(J))**2
            FFS(I) = FFS(I) + FZ*FLOAT(NSYM*NC(J))
 156     CONTINUE
C     
C     
C         DS     = SQRT(DSS(I))
C         RS     = 1.0/DS
C         NRFUNQ(I) = 0
C         NRFLCF(I) = 0
C         NRFUNQ(I) = NINT(0.6667*PI*DSS(I)**1.5 /( RVOL*NSYM))
C         WRITE(LUNOUT,1667)
C     +        S0,S1,DSS(I),DS,RS,FFS(I),NRFUNQ(I)
C 1667    FORMAT( F9.4,'-',F6.4, F8.4, F10.4, F6.2,F10.0,I10)
C         S0 = S1
C         S1 = S0 + DELDSS
 160  CONTINUE
      RETURN
      END
C     
C   
C     
      SUBROUTINE WILSTA(IH,S,FP,SIGFP,NSYM,NREF)
C     ========================================
C     
C      IMPLICIT NONE
C     
      INTEGER IH(3),NSYM,NREF
      REAL FP,SIGFP
C     
C     
      REAL AMULT
      INTEGER NBIN,NMON,IPLFSQ,NK
      REAL RMAX,RMIN,SMIN,SMAX,SMINSC,SMAXSC,RMINSC,RMAXSC,
     $     VPA
      COMMON /WILKEY/ NBIN,NMON,RMAX,RMIN,SMIN,SMAX,SMINSC,
     +     SMAXSC,RMINSC,RMAXSC,VPA,
     +     IPLFSQ,NK
      REAL DSS,SI,SN,SR,SW,XY,XYAV,FFS,DSSQ,PI,DELDSS,F000,
     +     VOL,RR,CELLAS,RVOL,FRAC
      INTEGER  NRFUNQ,NRFLCF,NATMS,NELEC,ISMAX
      COMMON /PREWIL/ DSS(100),SI(100),SN(100),SR(100),SW(100),
     +     NRFUNQ(100),NRFLCF(100),XY(2,100),XYAV(2,100),
     +     FFS(500),DSSQ(500),PI,DELDSS,NATMS,NELEC,F000,
     +     ISMAX,VOL,RR(3,3,6),CELLAS(6),RVOL,FRAC
      CHARACTER NW*2
      COMMON /WILCHR/ NW(8)
      INTEGER NC,NO
      REAL AL,AS,BL,BS,CL
      COMMON /WILFRM/ NC(8),NO(8),AL(8),AS(8),BL(8),BS(8),CL(8)
C
C Locals
      REAL S,ASYM,EPS,DELIS,FF,AF,BF
      INTEGER ISYSAB,IBIN
C     
      IF(S.LT.SMINSC .OR. S.GT.SMAXSC) RETURN
C     
C---- if sigfp = 0  - NO observation made...
C     
      IF(SIGFP .LT. 0.000001) RETURN
      NREF = NREF + 1
      ASYM = NSYM
C     
C---- Those with epsilon class > 1    have reduced multiplicity.
C     
C     eps = multiplicity of this reflection.
C     the call to epsln/centric done after lwopen
C     
      CALL EPSLON(IH,EPS,ISYSAB)
      AMULT = ASYM/EPS
C     
C---- Look up scattering factor tables generated by ATREC
C     
      IBIN =  INT(S/DELDSS) + 1
      DELIS = S  - FLOAT(IBIN)*DELDSS
C     
C---- Wilson's expected intensity
C     
      IF ( IBIN.LE.1) THEN
         FF = FFS(1)
      ELSE
         BF=0.5*(FFS(IBIN+1) - FFS(IBIN-1))
         AF=BF + FFS(IBIN-1) - FFS(IBIN)
         FF = (AF*DELIS + BF)*DELIS  + FFS(IBIN)
      ENDIF
C     
C     
      IF(IBIN.GT.NBIN)IBIN = NBIN
      SN(IBIN) = SN(IBIN) + AMULT
      SW(IBIN) = SW(IBIN) + AMULT*FF
      SR(IBIN) = SR(IBIN) + AMULT*S
      SI(IBIN) = SI(IBIN) + AMULT*FP*FP
      RETURN
      END

