c     chef - main program
c     
c     Copyright (C) 2008 STFC, Graeme Winter
c     
c     This code is distributed under the BSD license, a copy of which
c     is included in the root directory if this package.
c     
c     15th January, 2008
c     
c     Program CHEF
c     ------------
c
c     A program for analysing scaled, unmerged intensity observations from
c     multiple wavelengths of diffraction data, in terms of radiation damage, 
c     resolution analysis and symmetry operations. The objective of this is
c     to allow rapid hypothesis testing on possible subsets of measurements to 
c     use for substructure determination, phasing and refinement.
c     
c     Usage
c     -----
c     
c     The program follows CCP4 conventions on use:
c     
c     chef hklin1 infl.mtz hklin2 lrem.mtz << eof
c     anomalous on 
c     labin BASE=DOSE
c     print rd rmerge rsym
c     range min 0 max 600 width 10
c     resolution 40 1.7
c     bins 10
c     eof

      program chef

      implicit none

c     included files

      include 'parameters.fh'
      include 'options.fh'
      include 'mtz.fh'

c     external subroutines - CCP4 (-L$CLIB -lccp4f -lccp4c)

      external ccpfyp, ccperr, mtzini, lropen, lrrsol

c     internally defined subroutines from the chef program

      external chef_init, command, chisq, chisq2, meansd, merge, 
     +     rmerge, rmerge2, nref, nref_init, nref1, chef_hklin,
     +     chef_finish, chef_rd, chef_rmerge, chef_rsym,
     +     dsetup, chef_rdc

c     local variables

      integer status
      character message*(200)

c     initialisation

      call ccpfyp
      call mtzini

c     banner
      call ccprcs(6, 'CHEF', '$Date: 2008/03/27 14:13:31 $')

      call chef_init

c     set up hklin files

      call chef_hklin(status, message)

      if (status .eq. 2) call ccperr(1, message)

c     set up the command line

      call command(status, message)

      if (status .eq. 2) call ccperr(1, message)

c     if we have not had the resolution limits assigned, assign these
c     from the input reflection files

      if ((o_dmin .eq. 0.0) .and. (o_dmax .eq. 0.0)) then
         call dsetup
      end if

      if (o_dmax .eq. 0.0) then
         call dsetup2
      end if

c     ok, so now we should be ready to perform some useful calculations...

      if (o_prd) call chef_rd(status, message)
      if (status .eq. 2) call ccperr(1, message)

      if (o_prdc .or. o_pcrdu) call chef_rdc(status, message)
      if (status .eq. 2) call ccperr(1, message)

      if (o_prmg) call chef_rmerge(status, message)
      if (status .eq. 2) call ccperr(1, message)

      if (o_prsym) call chef_rsym(status, message)
      if (status .eq. 2) call ccperr(1, message)

c     then tidy up

      call chef_finish

      call ccperr(0, 'Normal termination')

      stop
      end
      
      
      
