c     chef - variable initialisation
c     
c     Copyright (C) 2008 STFC, Graeme Winter
c     
c     This code is distributed under the BSD license, a copy of which
c     is included in the root directory if this package.
c     
c     15th January, 2008
c
c     This will set the options to sensible default values - should 
c     be called before subroutine command

c     subroutine chef_init - this is essentially a constructor for the 
c     options singleton, as defined in options.fh, and should be called
c     before subroutine commmand and also before chef_hklin.

      subroutine chef_init

      implicit none

      include 'parameters.fh'
      include 'options.fh'
      include 'mtz.fh'

c     local variables 

      integer i, j, k

c     command line options

      anomalous = .false.

      o_dbins = 10
      o_dmin = 0.0
      o_dmax = 0.0
      o_smax = 0.0
      o_smin = 0.0

      o_prd = .true.
      o_pcrd = .true.
      o_prmg = .true.
      o_prsym = .true.
      o_pcomp = .false.

      o_prdc = .false.
      o_pcrdc = .false.
      o_pcrdu = .true.
      o_prmgc = .false.
      o_prsymc = .false.

      o_bmin = 0.0
      o_bmax = 0.0
      o_bwidth = 1.0

      o_wmerge = .false.
      o_isigma = 2.0

c     mtz things

      do i = 1, maxhkl
         hklin(i) = 0
      end do

      nmtz = 0

      mnsymop = 0

      do i = 1, maxsym
         mcsymop(i) = ' '
         do j = 1, 4
            do k = 1, 4
               mrsymop(j, k, i) = 0.0
            end do
         end do
      end do

      end

c     subroutine chef_hklin - open HKLIN files from the command line

      subroutine chef_hklin(status, message)

      implicit none

c     in - none

c     out

      integer status
      character message*(*) 

c     external subroutines

      external ccpexs, lropen, lenstr, lrsymm, lrsymi, symtr4,
     +     lridx

c     types for functions

      logical ccpexs
      integer lenstr

      include 'parameters.fh'
      include 'options.fh'
      include 'mtz.fh'
      
      character chklin*10, lattice*1, sgnam*10, pgnam*10
      integer i, j, mtzerr, mtzprt, nsymop, sgnum

c     local variables to help with lridx
      
      integer mdatasets, ndatasets
      parameter (mdatasets = 10)

      character lpname(mdatasets)*80, lxname(mdatasets)*80
      character ldname(mdatasets)*80

      real dcell(6, mdatasets), dwave(mdatasets)

      integer isets(mdatasets)

c     initialise status

      status = 0
      message = ' '

 1    format('HKLIN', i1)

      do i = 1, maxhkl

         write(chklin, 1) i

         if (ccpexs(chklin)) then

            mtzerr = 0
            mtzprt = 0
            hklin(i) = i

            call lropen(hklin(i), chklin, mtzprt, mtzerr)

c     read the project / crystal / dataset information out and check
c     that theer is exactly one data set in here - when we have the 
c     information, add it to the common area...

c     initialise

            ndatasets = mdatasets

            do j = 1, ndatasets
               lpname(j) = ' '
               lxname(j) = ' '
               ldname(j) = ' '
            end do

            call lridx(hklin(i), lpname, lxname, ldname, isets,
     +           dcell, dwave, ndatasets)

c     if ndatasets != 2 (HKL_base + the real one) raise an exception...

            if (ndatasets .ne. 2) then
               status = 2
               write(message, 4) i
               return               
            end if

c     copy the newly found dataset information out...

            pname(i) = lpname(2)
            xname(i) = lxname(2)
            dname(i) = ldname(2)
            
            write(*, 5) pname(i)(1:lenstr(pname(i))), 
     +           xname(i)(1:lenstr(xname(i))),
     +           dname(i)(1:lenstr(dname(i)))

c     if this is the first reflection file to open, read the symmetry
c     operations out...

            if (mnsymop .eq. 0) then
               nsymop = maxsym
               call lrsymi(hklin(i), nsymop, lattice, sgnum, sgnam, 
     +              pgnam)
               call lrsymm(hklin(i), mnsymop, mrsymop)

c     we only want the primative symmetry operations for this I guess...

               mnsymop = nsymop

               call symtr4(mnsymop, mrsymop, mcsymop)
            end if

            if (mtzerr .ne. 0) then

               hklin(i) = 0

               status = 2
               write(message, 2) chklin(1:lenstr(chklin))
               return

            end if

            nmtz = nmtz + 1

         else

            hklin(i) = 0

            if (nmtz .eq. 0) then
               status = 2
               write(message, 3) 
               return
            end if

            return

         end if

      end do

 2    format('error opening: ', a)
 3    format('no HKLIN files specified')
 4    format('not exactly one dataset in HKLIN', i1)
 5    format('Found: ', a, '/', a, '/', a)

      end

c     subroutine chef_finish - tidy off all of the open file pointers
c     and so on...

      subroutine chef_finish

      implicit none
      
c     external subroutines

      external lrclos

      include 'parameters.fh'
      include 'options.fh'
      include 'mtz.fh'
      
c     local variables

      integer i

      do i = 1, nmtz
         call lrclos(hklin(i))
         hklin(i) = 0
      end do

      end

c     subroutine dsetup - this will read through the reflection files and 
c     assign the resolution limits according to the limits therein.

      subroutine dsetup

      implicit none

c     external subroutines

      external lrrsol

      include 'parameters.fh'
      include 'options.fh'
      include 'mtz.fh'
      
c     local variables

      real smin, smax, sminj, smaxj
      integer m

      smin = 1.0e6
      smax = 0.0

      do m = 1, nmtz

         call lrrsol(hklin(m), sminj, smaxj)

         if (sminj .lt. smin) smin = sminj
         if (smaxj .gt. smax) smax = smaxj

      end do

      o_smin = smin
      o_smax = smax

      o_dmin = 1.0 / sqrt(smax)
      o_dmax = 1.0 / sqrt(smin)
      
      write(*, 1) o_dmin, o_dmax

 1    format('Resolution range: ', 2f6.2)

      end

c     subroutine dsetup2 - this will read through the reflection files and 
c     assign the low resolution limit according to the limits therein.

      subroutine dsetup2

      implicit none

c     external subroutines

      external lrrsol

      include 'parameters.fh'
      include 'options.fh'
      include 'mtz.fh'
      
c     local variables

      real smin, smax, sminj, smaxj
      integer m

      smin = 1.0e6
      smax = 0.0

      do m = 1, nmtz

         call lrrsol(hklin(m), sminj, smaxj)

         if (sminj .lt. smin) smin = sminj
         if (smaxj .gt. smax) smax = smaxj

      end do

      o_smin = smin

      o_dmax = 1.0 / sqrt(smin)
      
      write(*, 1) o_dmax

 1    format('Low resolution limit: ', 2f6.2)

      end
