c     chef - rd radiation damage analysis
c     
c     Copyright (C) 2008 STFC, Graeme Winter
c     
c     This code is distributed under the BSD license, a copy of which
c     is included in the root directory if this package.
c     
c     15th January, 2008

      subroutine chef_rd(status, message)

      implicit none

      integer status
      character message*(*)

c     common blocks containing MTZ information & standard reflection 
c     storage stuff

      include 'parameters.fh'
      include 'options.fh'
      include 'mtz.fh'
      include 'chef_rd.fh'

c     external subroutines which may be used here

      external lrreff, lrrewd, lenstr

c     types for functions
      integer lenstr

c     externalish subroutines - defined within same file

      external calc_rd, accumulate
      integer calc_rd

c     real local variables

      integer i, j, iline(maxbins)
      real line(maxbins), b, ra, rb, smin, smax
      real resol(maxbins), linea(maxbins), lineb(maxbins)
      character loggraph*(200), buffer*(200)

c     initialise array - as R values calculated as sum(a) / sum(b)
c     need to arrays to accumulate measurements

      bmax = 0

      do i = 1, maxbatch
         do j = 1, maxbins
            rda(i, j) = 0.0
            rdb(i, j) = 0.0
            rdc(i, j) = 0
         end do
      end do

      call accumulate(calc_rd, status, message)

c     print the results - in loggraph format

 101  format('$TABLE: ', a, ':')
 102  format('$GRAPHS', $)
 103  format(':', a, ':N:', a, ':')
 107  format('RD resolution ', f6.2, ' to ', f6.2)
 108  format(i2, ',', i2)
 109  format('DOSE  Overall ', 40f8.4)
 110  format('$$')

      write(*, 101) 'RD analysis'
      write(*, 102) 
      write(*, 103) 'RD (overall)', '1,2'

      do i = 1, o_dbins
         smin = o_smin + 
     +        real(i - 1) * (o_smax - o_smin) / real(o_dbins)
         smax = o_smin + 
     +        real(i) * (o_smax - o_smin) / real(o_dbins)

         loggraph = ' '
         buffer = ' '
         write(loggraph, 107) 1.0 / sqrt(smin), 1.0 / sqrt(smax)
         write(buffer, 108) 1, i + 2
         write(*, 103) loggraph(1:lenstr(loggraph)), 
     +        buffer(1:lenstr(buffer))

         resol(i) = 1.0 / sqrt(smax)
      end do

      write(*, 110)
      write(*, 109) (resol(i), i = 1, o_dbins)
      write(*, 110)
      write(*, 110)

      do i = 1, bmax
         
         ra = 0.0
         rb = 0.0

         do j = 1, o_dbins
            if (rdc(i, j) .gt. 0) then
               line(j) = rda(i, j) / rdb(i, j)
               ra = ra + rda(i, j)
               rb = rb + rdb(i, j)
            else
               line(j) = 0.0
            end if
            iline(j) = rdc(i, j)
         end do

         b = o_bmin + (0.5 + real(i - 1)) * o_bwidth

         write(*, 1) b, ra / rb, (line(j), j = 1, o_dbins)
         if (o_pcrd) write(*, 2) b, (iline(j), j = 1, o_dbins)
      end do
      write(*, 110)

 1    format(f6.1, 41f8.2)
 2    format(f6.1, 5x, 40i8)

c     now write out the overall values

      ra = 0.0
      rb = 0.0

      do i = 1, o_dbins
         linea(j) = 0.0
         lineb(j) = 0.0
      end do

      do i = 1, bmax
         do j = 1, o_dbins
            if (rdc(i, j) .gt. 0) then
               linea(j) = linea(j) + rda(i, j)  
               lineb(j) = lineb(j) + rdb(i, j)
               ra = ra + rda(i, j)
               rb = rb + rdb(i, j)
            end if
         end do
      end do

      do j = 1, o_dbins
         if (lineb(j) .ne. 0.0) then
            line(j) = linea(j) / lineb(j)
         else
            line(j) = 0.0
         end if
      end do

      write(*, *) 'Overall:'
      write(*, 1) 0.0, ra / rb, (line(j), j = 1, o_dbins)
      write(*, *) ' '

      end

c     main calculation subroutine

      function calc_rd()

      implicit none

      integer calc_rd

c     common blocks containing MTZ information & standard reflection 
c     storage stuff

      include 'parameters.fh'
      include 'arrays.fh'
      include 'options.fh'
      include 'chef_rd.fh'

c     local variables

      integer i, j, b, d
      real ra, rb, s

      do i = 1, b_npls
         do j = i + 1, b_npls
            ra = abs(b_ipls(i) - b_ipls(j))
            rb = 0.5 * (b_ipls(i) + b_ipls(j))
            b = nint(abs((b_bpls(i) - b_bpls(j)) / o_bwidth))
            s = 1.0 / (b_dpls(i) ** 2)

            d = 1 + int(o_dbins * (s - o_smin) / (o_smax - o_smin))

            rda(b, d) = rda(b, d) + ra
            rdb(b, d) = rdb(b, d) + rb
            rdc(b, d) = rdc(b, d) + 1

            if (b .gt. bmax) bmax = b

         end do
      end do

      do i = 1, b_nmns
         do j = i + 1, b_nmns
            ra = abs(b_imns(i) - b_imns(j))
            rb = 0.5 * (b_imns(i) + b_imns(j))
            b = nint(abs((b_bmns(i) - b_bmns(j)) / o_bwidth))
            s = 1.0 / (b_dmns(i) ** 2)

            d = 1 + int(o_dbins * (s - o_smin) / (o_smax - o_smin))

            rda(b, d) = rda(b, d) + ra
            rdb(b, d) = rdb(b, d) + rb
            rdc(b, d) = rdc(b, d) + 1

            if (b .gt. bmax) bmax = b

         end do
      end do

      calc_rd = 0

      end

