c     chef - rdc cumulative radiation damage analysis
c     
c     Copyright (C) 2008 STFC, Graeme Winter
c     
c     This code is distributed under the BSD license, a copy of which
c     is included in the root directory if this package.
c     
c     15th January, 2008

      subroutine chef_rdc(status, message)

      implicit none

      integer status
      character message*(*)

c     common blocks containing MTZ information & standard reflection 
c     storage stuff

      include 'parameters.fh'
      include 'options.fh'
      include 'mtz.fh'
      include 'chef_rd.fh'
      include 'chef_rdc.fh'

c     external subroutines which may be used here

      external lrreff, lrrewd, lenstr, model

c     types for functions
      integer lenstr
      real model

c     externalish subroutines - defined within same file

      external calc_rdc, accumulate
      integer calc_rdc

c     real local variables

      integer i, j, iline(maxbins)
      real line(maxbins), b, ra, rb, smin, smax, isig, ruse
      real resol(maxbins), mline(maxbins), rlim, nm, nu, weight
      character loggraph*(200), buffer*(200)

c     initialise array - as R values calculated as sum(a) / sum(b)
c     need to arrays to accumulate measurements

      bmax = 0

      do i = 1, maxbatch
         do j = 1, maxbins
            rda(i, j) = 0.0
            rdb(i, j) = 0.0
            rdc(i, j) = 0
            rdo(i, j) = 0
            rdcnu(i, j) = 0
            rdcnm(i, j) = 0
         end do
      end do

      call accumulate(calc_rdc, status, message)

c     print the results - in loggraph format

 101  format('$TABLE: ', a, ':')
 102  format('$GRAPHS', $)
 103  format(':', a, ':N:', a, ':')
 107  format('RD resolution ', f6.2, ' to ', f6.2)
 108  format(i2, ',', i2)
 109  format('DOSE  Overall ', 40f8.4)
 110  format('$$')

      write(*, 101) 'Cumulative RD analysis'
      write(*, 102) 
      write(*, 103) 'CRD (overall)', '1,2'

      do i = 1, o_dbins
         smin = o_smin + 
     +        real(i - 1) * (o_smax - o_smin) / real(o_dbins)
         smax = o_smin + 
     +        real(i) * (o_smax - o_smin) / real(o_dbins)

         loggraph = ' '
         buffer = ' '
         write(loggraph, 107) 1.0 / sqrt(smin), 1.0 / sqrt(smax)
         write(buffer, 108) 1, i + 2
         write(*, 103) loggraph(1:lenstr(loggraph)), 
     +        buffer(1:lenstr(buffer))

         resol(i) = 1.0 / sqrt(smax)
      end do

      write(*, 110)
      write(*, 109) (resol(i), i = 1, o_dbins)
      write(*, 110)
      write(*, 110)

      do i = 1, bmax
         
         ra = 0.0
         rb = 0.0

         do j = 1, o_dbins
            if (rdc(i, j) .gt. 0) then
               line(j) = abs(rda(i, j) / rdb(i, j))
               ra = ra + rda(i, j)
               rb = rb + rdb(i, j)
            else
               line(j) = 0.0
            end if

c     implemented new test based on regression of parameters...

            if (rdcnu(i, j) .ge. 100) then
               nu = real(rdcnu(i, j))
               nm = real(rdcnm(i, j)) / nu
               
c     FIXME should there be a sqrt(nm) weighting term in here, or
c     should it be something like 0.5 * (1.0 * sqrt(nm)) ??
c     note well - nm = mean number of PAIRS not actual multiplicity ->
c     scale should be ~ pow(nm, 0.25)... nm = m (m - 1) so
c     m^2 - m - nm = 0 so find roots for m, which should itself
c     be sqrted.

               weight = sqrt(0.5 + 0.5 * sqrt(1 + 4 * nm))

c     Based on comment in Gonzalez (2003) set the maximum weight 
c     to 2.0 (equivalent to 4 measurements) as that is probably as
c     good as it gets in "real life"

               if (weight .gt. 2.0) weight = 2.0               

               rlim = (1.0 + 2.0 * model((o_isigma / weight), 
     +              nm, nu)) * 1.1284 / (o_isigma / weight)
               
               if (rlim .gt. line(j)) then
                  iline(j) = 1
                  ruse = resol(j)
               else
                  iline(j) = 0
               end if
            else
               iline(j) = -1
            end if

            if (.false.) then

               if (rdo(i, j) .gt. 0) then
                  isig = (1.1284 / o_isigma) + 
     +                 1.0 * (1.1284 / o_isigma)/ 
     +                 (real(rdc(i, j)) / real(rdo(i, j)))
                  if (isig .gt. line(j)) then
                     iline(j) = 1
                     ruse = resol(j)
                  else
                     iline(j) = 0
                  end if
               else
                  iline(j) = -1
               end if

            end if

c     propogate bad regions - just in case...

            if ((j .gt. 1) .and. (iline(j - 1) .eq. 0))
     +           iline(j) = 0
            
c     FIXME - if we do this we need to update the resolution limit
c     bummer!

c     iline(j) = rdc(i, j)
            if (rdo(i, j) .gt. 0) then
               mline(j) = real(rdc(i, j)) / real(rdo(i, j))
            else
               mline(j) = 0.0
            end if

         end do

         b = o_bmin + (0.5 + real(i - 1)) * o_bwidth

         if (o_prdc) 
     +        write(*, 1) b, ra / rb, (line(j), j = 1, o_dbins)
         if (o_pcrdu) 
     +        write(*, 2) b, ruse, (iline(j), j = 1, o_dbins)
         if (o_pcrdc) write(*, 3) b, (mline(j), j = 1, o_dbins)
         
         if (o_pcrdc) then

            do j = 1, o_dbins
               if (rdcnu(i, j) .gt. 0) then
                  mline(j) = real(rdcnm(i, j)) / rdcnu(i, j)
               else
                  mline(j) = 0.0
               end if
            end do
            write(*, 1) b, ra / rb, (line(j), j = 1, o_dbins)
            write(*, 4) b, (real(rdcnu(i, j)), j = 1, o_dbins)
            write(*, 5) b, (mline(j), j = 1, o_dbins)
         end if
            
      end do
      write(*, 110)

 1    format('R', f6.1, 41f8.4)
 2    format(f6.1, f8.2, 40i8)
 3    format(f6.1, 8x, 40f8.2)
 4    format('U', f6.1, 41f10.0)
 5    format('M', f6.1, 41f8.2)
      
      end

c     main calculation subroutine

      function calc_rdc()

      implicit none

      integer calc_rdc

c     common blocks containing MTZ information & standard reflection 
c     storage stuff

      include 'parameters.fh'
      include 'arrays.fh'
      include 'options.fh'
      include 'chef_rd.fh'
      include 'chef_rdc.fh'

c     local variables

      logical upls, umns
      integer i, j, k, b, d, lbmax, lbmin, ld
      real ra, rb, s

      ld = 0

      lbmax = maxbatch
      lbmin = maxbatch

      if (o_bmax .gt. 0) lbmax = nint(o_bmax / o_bwidth)


c     assert: all reflections which are symmetry related have the
c     same resolution (this is clearly right, right?)

      if (b_npls .gt. 0) then
         upls = .true.
      else
         upls = .false.
      end if

      if (b_nmns .gt. 0) then
         umns = .true.
      else
         umns = .false.
      end if

      do i = 1, b_npls
         do j = i + 1, b_npls
            ra = abs(b_ipls(i) - b_ipls(j))
            rb = 0.5 * (b_ipls(i) + b_ipls(j))
            b = max(nint(b_bpls(i) / o_bwidth),
     +           nint(b_bpls(j) / o_bwidth))
            s = 1.0 / (b_dpls(i) ** 2)
            
            d = 1 + int(o_dbins * (s - o_smin) / (o_smax - o_smin))

c     in here accumulate nm, nu - accumulate nu once, nm += 2
c     for each baseline (recalling that it counts both ways in the 
c     analysis...) - accumulate rdcnu at the end of this routine...

            do k = b, lbmax
               rdcnm(k, d) = rdcnm(k, d) + 2
            end do
            
            ld = d
            if (b .lt. lbmin) lbmin = b
            
            do k = b, lbmax
               
               rda(k, d) = rda(k, d) + ra
               rdb(k, d) = rdb(k, d) + rb
               rdc(k, d) = rdc(k, d) + 1
               
            end do
            
            if (b .gt. bmax) bmax = b

         end do
      end do

      do i = 1, b_nmns
         do j = i + 1, b_nmns

            ra = abs(b_imns(i) - b_imns(j))
            rb = 0.5 * (b_imns(i) + b_imns(j))
            b = max(nint(b_bmns(i) / o_bwidth),
     +           nint(b_bmns(j) / o_bwidth))
            s = 1.0 / (b_dmns(i) ** 2)
            
            d = 1 + int(o_dbins * (s - o_smin) / (o_smax - o_smin))
            
            do k = b, lbmax
               rdcnm(k, d) = rdcnm(k, d) + 2
            end do
            
            ld = d
            if (b .lt. lbmin) lbmin = b
            
            do k = b, lbmax
               
               rda(k, d) = rda(k, d) + ra
               rdb(k, d) = rdb(k, d) + rb
               rdc(k, d) = rdc(k, d) + 1
               
            end do
            
            if (b .gt. bmax) bmax = b
         end do
      end do
      
c     populate number of unique observations

      if ((lbmin .lt. maxbatch) .and. (ld .gt. 0)) then
         do k = lbmin, lbmax
            rdo(k, ld) = rdo(k, ld) + 1
         end do
      end if

      if (upls) then
         do k = lbmin, lbmax
            rdcnu(k, ld) = rdcnu(k, ld) + 1
         end do
      end if
            
      if (umns) then
         do k = lbmin, lbmax
            rdcnu(k, ld) = rdcnu(k, ld) + 1
         end do
      end if
            
      calc_rdc = 0

      end

