c     chef - rsym analysis
c     
c     Copyright (C) 2008 STFC, Graeme Winter
c     
c     This code is distributed under the BSD license, a copy of which
c     is included in the root directory if this package.
c     
c     15th January, 2008

      subroutine chef_rsym(status, message)

      implicit none

      integer status
      character message*(*)

c     common blocks containing MTZ information & standard reflection 
c     storage stuff

      include 'parameters.fh'
      include 'options.fh'
      include 'mtz.fh'
      include 'chef_rsym.fh'

c     external subroutines which may be used here

      external lrreff, lrrewd, lenstr

c     function types

      integer lenstr

c     externalish subroutines - defined within same file

      external calc_rsym, accumulate
      integer calc_rsym

c     real local variables

      integer i, j
      real rsym
      character symop*80

c     initialise array - as R values calculated as sum(a) / sum(b)
c     need to arrays to accumulate measurements

      do i = 1, maxsym
         rsyma(i) = 0.0
         rsymb(i) = 0.0
         rsymc(i) = 0
      end do

      nsymop = 0

      call accumulate(calc_rsym, status, message)

c     print the results

      write(*, *) ' '
      write(*, *) 'Analysis by symmetry operation:'

      do i = 1, mnsymop * 2

         if (mod(i, 2) .eq. 1) then
            j = (i + 1) / 2
         else
            j = i / 2
         end if

         if (rsymc(i) .gt. 0) then
            rsym = rsyma(i) / rsymb(i)
         else
            rsym = 0.0
         end if

         symop = mcsymop(j)


         if ((mod(i, 2) .eq. 1) .and. .not. anomalous) then
            write(*, 1) i, rsymc(i), rsym, symop(1:lenstr(symop))
         end if

         if ((mod(i, 2) .eq. 1) .and. anomalous) then
            write(*, 2) i, rsymc(i), rsym, symop(1:lenstr(symop))
         end if

         if ((mod(i, 2) .eq. 0) .and. anomalous) then
            write(*, 3) i, rsymc(i), rsym, symop(1:lenstr(symop))
         end if

      end do

      write(*, *) ' '

 1    format(i3, i7, f5.2, 1x, a)
 2    format(i3, i7, f5.2, 1x, 'I+', 1x, a)
 3    format(i3, i7, f5.2, 1x, 'I-', 1x, a)

      end

c     main calculation subroutine

      function calc_rsym()

      implicit none

      integer calc_rsym

c     common blocks containing MTZ information & standard reflection 
c     storage stuff

      include 'parameters.fh'
      include 'arrays.fh'
      include 'options.fh'
      include 'chef_rsym.fh'

c     external subroutines

      external merge, meansd

c     local variables

      integer i, ns
      real mpls, mmns, sdpls, sdmns

c     compute mean for I+, I- - either weighting the values by 
c     1.0 / sd ** 2 or uniform weighting. 

      if (o_wmerge) then

         call merge(b_ipls, b_spls, b_npls, 0, mpls, sdpls)
         call merge(b_imns, b_smns, b_nmns, 0, mmns, sdmns)

      else

         call meansd(b_ipls, b_npls, 0, mpls, sdpls)
         call meansd(b_imns, b_nmns, 0, mmns, sdmns)

      end if

      do i = 1, b_npls
         ns = b_opls(i)
         if (ns .gt. nsymop) nsymop = ns
         rsyma(ns) = rsyma(ns) + abs(b_ipls(i) - mpls)
         rsymb(ns) = rsymb(ns) + abs(b_ipls(i))
         rsymc(ns) = rsymc(ns) + 1
      end do

      do i = 1, b_nmns
         ns = b_omns(i)
         if (ns .gt. nsymop) nsymop = ns
         rsyma(ns) = rsyma(ns) + abs(b_imns(i) - mmns)
         rsymb(ns) = rsymb(ns) + abs(b_imns(i))
         rsymc(ns) = rsymc(ns) + 1
      end do

      calc_rsym = 0

      end

