c     chef - chi^2 subroutines
c     
c     Copyright (C) 2008 STFC, Graeme Winter
c     
c     This code is distributed under the BSD license, a copy of which
c     is included in the root directory if this package.
c     
c     14th January, 2008
c

c     chisq - compute the chi-squared sum((O-E)**2 / sig(O) ** 2) and return

      subroutine chisq(values, errors, correct, ecorrect, 
     +     ignore, count, chi)

      implicit none

c     in

      real values(*), errors(*), correct, ecorrect, o, e, s
      integer count, ignore

c     out

      real chi

c     local

      integer i

      if (count .le. 1) return

      do i = 1, count

         if (i .eq. ignore) then

            o = values(i)
            e = correct
            s = errors(i)
            
            chi = chi + ((o - e) ** 2) / (s ** 2 + ecorrect ** 2)

         end if

      end do

      end

c     chisq2 - compute chi-square variables but bin the values based
c     on indices

      subroutine chisq2(values, errors, correct, ecorrect, indices, 
     +     ignore, count, chi, chi_count)

      implicit none

c     in

      real values(*), errors(*), correct, ecorrect, o, e, s
      integer count, indices(*), ignore

c     out

      real chi(*)
      integer chi_count(*)

c     local

      integer i, j

      if (count .le. 1) return

      do i = 1, count

         if (i .ne. ignore) then

            o = values(i)
            e = correct
            s = errors(i)
            
            j = indices(i)
            
            chi(j) = chi(j) + ((o - e) ** 2) / (s ** 2 + ecorrect ** 2)
            
            chi_count(j) = chi_count(j) + 1
            
         end if
            
      end do

      end

      
