c     chef - command line parsing and recording
c     
c     Copyright (C) 2008 STFC, Graeme Winter
c     
c     This code is distributed under the BSD license, a copy of which
c     is included in the root directory if this package.
c     
c     15th January, 2008
c     
c     This is a subroutine for parsing the command line options - this
c     must be called after all reflection files are opened, as this will
c     assign column labels and whatnot.

      subroutine command(status, message)

      implicit none

      include 'parameters.fh'
      include 'options.fh'
      include 'mtz.fh'

c     in - none

c     out - status - equal to:
c
c     0 -> all is well
c     1 -> warning
c     2 -> input error
c     
c     message will contain helpful text in event of an error - and 
c     should be allocated in the calling routine to say length 200.

      integer status
      character message*(*)
      
c     external subroutines
      
      external parser, lkyin, lenstr, ccpupc

c     types for functions
      integer lenstr

c     local variables - parsing

      integer ntok, ibeg(maxpar), iend(maxpar), idec(maxpar), 
     +     ityp(maxpar), i
      real fval(maxpar)
      character cval(maxpar)*4, key*4, subkey*4, line*400
      logical eof

c     local variables - reflection file handling

      integer lookup(mcol)
      character colin(mcol)*30, ctypin(mcol)*1

c     set up data for reading input columns

      data colin/'H', 'K', 'L', 'M/ISYM', 'I', 'SIGI',
     +     'BASE', 193 * ' '/
      data ctypin/'H', 'H', 'H', 'Y', 'J', 'Q', 'R', 193 * ' '/
      data lookup/-1, -1, -1, -1, -1, -1, -1, 193 * 0/

c     initialise return status

      status = 0
      message = ' '

c     loop over command line input - return when finished

 1    continue

      line = ' '
      key = ' '
      ntok = maxpar

      call parser(key, line, ibeg, iend, ityp, fval, cval,
     +     idec, ntok, eof, .true.)

      if (eof) goto 2

c     --- keyword END ---

      if (key .eq. 'END') then
         goto 2

c     --- keyword LABIN ---

      else if (key .eq. 'LABI') then
         do i = 1, nmtz
            call lkyin(hklin(i), colin, 7, ntok, line, ibeg, iend)
         end do

c     --- keyword ANOMALOUS ---

      else if (key .eq. 'ANOM') then
         if (ntok .ne. 2) then
            status = 2
            write(message, 101) line(1:lenstr(line))
            return
         end if
         subkey = cval(2)
         call ccpupc(subkey)
         
         if ((subkey .eq. 'ON') .or. (subkey .eq. 'TRUE')) then
            anomalous = .true.
         else
            anomalous = .false.
         end if
         
c     --- keyword RESOLUTION ---

      else if (key .eq. 'RESO') then
         if (ntok .lt. 2) then 
            status = 2
            write(message, 103) 
            return
         end if

c     check that all of the arguments are passed in as numbers
         do i = 2, ntok
            if (ityp(i) .ne. 2) then
               status = 2
               write(message, 104) line(ibeg(i):iend(i))
               return
            end if
         end do

c     check for just the high resolution provided
         if (ntok .eq. 2) then 
            o_dmin = fval(2)
            o_smax = 1.0 / (o_dmin ** 2)
         else if (ntok .eq. 3) then
            if (fval(2) .lt. fval(3)) then
               o_dmin = fval(2)
               o_dmax = fval(3)
            else
               o_dmin = fval(3)
               o_dmax = fval(2)
            end if
            o_smax = 1.0 / (o_dmin ** 2)
            o_smin = 1.0 / (o_dmax ** 2)
         end if

c     --- keyword PRINT ---

      else if (key .eq. 'PRIN') then

c     initialise - print nothing if given 

         o_prd = .false.
         o_pcrd = .false.
         o_prmg = .false.
         o_prsym = .false.

         o_pcrdu = .false.
         o_prdc = .false.
         o_pcrdc = .false.
         o_prmgc = .false.
         o_prsymc = .false.
         
         do i = 2, ntok
            subkey = cval(i)
            call ccpupc(subkey)

            if (subkey .eq. 'RD') then
               o_prd = .true.
            else if (subkey .eq. 'RMER') then
               o_prmg = .true.
            else if (subkey .eq. 'RSYM') then
               o_prsym = .true.
            else if (subkey .eq. 'RDC') then
               o_prdc = .true.
            else if (subkey .eq. 'RDCU') then
               o_pcrdu = .true.
            else if (subkey .eq. 'CRD') then
               o_pcrd = .true.
            else if (subkey .eq. 'CRME') then
               o_prmgc = .true.
            else if (subkey .eq. 'CRSY') then
               o_prsymc = .true.
            else if (subkey .eq. 'CRDC') then
               o_pcrdc = .true.
            else if (subkey .eq. 'COMP') then
               o_pcomp = .true.
            else
               status = 2 
               write(message, 105) line(ibeg(i):iend(i))
               return
            end if
         end do

c     --- keyword RANGE ---

      else if (key .eq. 'RANG') then

         do i = 2, ntok, 2
            subkey = cval(i)
            call ccpupc(subkey)

            if (ityp(i + 1) .ne. 2) then
               status = 2
               write(message, 104) line(ibeg(i + 1):iend(i + 1))
               return
            end if

            if (subkey .eq. 'MIN') then
               o_bmin = fval(i + 1)
            else if (subkey .eq. 'MAX') then
               o_bmax = fval(i + 1)
            else if (subkey .eq. 'WIDT') then
               o_bwidth = fval(i + 1)
            else
               status = 2
               write(message, 105) line(ibeg(i):iend(i))
               return
            end if
         end do
         
c     --- keyword BINS ---

      else if (key .eq. 'BINS') then
         if (ntok .lt. 2) then
            status = 2
            write(message, 106) line(ibeg(1):iend(1))
            return
         end if

         if (ityp(2) .ne. 2) then
            status = 2
            write(message, 104) line(ibeg(2):iend(2))
            return
         end if

         o_dbins = nint(fval(2))

c     --- keyword MERGE ---

      else if (key .eq. 'MERG') then
         
         if (ntok .lt. 2) then
            status = 2
            write(message, 106) line(ibeg(1):iend(1))
            return
         end if

         subkey =  cval(2)
         call ccpupc(subkey)

         if (subkey .eq. 'WEIG') then
            o_wmerge = .true.
         else if (subkey .eq. 'UNIF') then
            o_wmerge = .false.
         else
            status = 2
            write(message, 105) line(ibeg(2):iend(2))
            return
         end if

c     --- ketword ISIGMA ---

      else if (key .eq. 'ISIG') then
         if (ntok .lt. 2) then
            status = 2
            write(message, 106) line(ibeg(1):iend(1))
            return
         end if

         if (ityp(2) .ne. 2) then
            status = 2
            write(message, 104) line(ibeg(2):iend(2))
            return
         end if

         o_isigma = fval(2)

c     --- keyword UNKNOWN ---

      else

         status = 2
         write(message, 102) line(1:lenstr(line))
         return
         
      end if

      goto 1

 2    continue

c     now that we are finished, assign all of the columns to actual
c     reflection files... this allows reading of reflections in logical 
c     rather than file order...

      do i = 1, nmtz
         call lrassn(hklin(i), colin, 7, lookup, ctypin)
      end do

c     format statements

 101  format('error parsing: ', a)
 102  format('unknown command: ', a)
 103  format('no resolution given')
 104  format('numerical argument required: ', a)
 105  format('unknown subkeyword: ', a)
 106  format('argument required: ', a)
      end
      
      
