c     chef - nref calculating subroutines
c     
c     Copyright (C) 2008 STFC, Graeme Winter
c     
c     This code is distributed under the BSD license, a copy of which
c     is included in the root directory if this package.
c     
c     14th January, 2008
c     

c     subroutine nrefinit - set up everything from CCP4 based on a
c     given HKLIN file (unit number) - returns number of symmetry 
c     operations for the spacegroup

c     this may take some debugging - the resulting symmetry operations 
c     counts are a little odd

      subroutine nref_init(hklin, nsymop)

      implicit none

      include 'parameters.fh'

c     in

      integer hklin

c     out

      integer nsymop

c     external

      external lrsymi, lrsymm, asuset, centric, epsln

      integer ns, nsp, sgnum, lgnum
      character lattice*1, sgnam*10, pgnam*10
      real rsym(4, 4, maxsym)

c     read symmetry metadata from mtz file header

      call lrsymi(hklin, ns, lattice, sgnum, sgnam, pgnam)

c     then read the actual symmetry operations

      nsp = maxsym
      call lrsymm(hklin, nsp, rsym)
      
c     record this for future reference - this is a result I want...
      nsymop = nsp

c     then pass this in to configure the tests and whatnot
c     not sure if I really need the asuset but this could 
c     have side-effect behaviour I want...

      call asuset(sgnam, sgnum, pgnam, nsp, rsym, ns, 
     +     lgnum, .false.)

      call centric(nsp, rsym, .false.)

      call epsln(nsp, ns, rsym, 0)

      end

c     subroutine nref - compute the number of reflections (unique)
c     given an input reflection file - this calls the above
c     initialisation subroutine

      subroutine nref(hklin, dmin, dmax, number)

      implicit none

c     in

      real dmin, dmax
      integer hklin

c     out

      integer number

c     external - from CCP4 (-L$CLIB -lccp4f -lccp4c)

      external lrcell, lstlsq, centr, epslon

c     types for functions

      real lstlsq

c     local

      integer h, k, l, hmax, kmax, lmax, nrefl, nsymop, ihkl(3),
     +     mult, sysab, cent
      real s, smin, smax, cell(6), eps

c     initialise

      call nref_init(hklin, nsymop)

c     read unit cell

      call lrcell(hklin, cell)

c     compute maximum possible reflection ranges

      hmax = int(cell(1) / dmin) + 1
      kmax = int(cell(2) / dmin) + 1
      lmax = int(cell(3) / dmin) + 1
      
c     set up resolution ranges

      smin = 1.0 / (dmax ** 2)
      smax = 1.0 / (dmin ** 2)

c     accumulate

      nrefl = 0

      do h = -hmax, hmax
         do k = -kmax, kmax
            do l = -lmax, lmax
               s = 4.0 * lstlsq(hklin, h, k, l)

               if ((s .le. smax) .and. (s .ge. smin)) then

                  ihkl(1) = h
                  ihkl(2) = k
                  ihkl(3) = l

                  mult = 1

c     epslon searches for axial zones

                  call epslon(ihkl, eps, sysab)

c     and centric for centric reflections

                  call centr(ihkl, cent)

c     compose factors for multiplicity for this reflection

                  if (cent .eq. 1) mult = mult * 2

                  mult = nint(mult * eps)

                  if (sysab .ne. 1) nrefl = nrefl + mult

               end if

            end do
         end do
      end do

c     now scale by number of symops * 2 - to get the number of "native"
c     reflections

      nrefl = nrefl / (2 * nsymop)

      number = nrefl

      end

c     subroutine nref1 - compute the number of reflections (unique)
c     given hklin and the number of symmetry operations - this does
c     not call the initialisation subroutine

      subroutine nref1(hklin, nsymop, dmin, dmax, number)

      implicit none

c     in

      real dmin, dmax
      integer hklin, nsymop

c     out

      integer number

c     external - from CCP4 (-L$CLIB -lccp4f -lccp4c)

      external lrcell, lstlsq, centr, epslon

c     types for functions

      real lstlsq

c     local

      integer h, k, l, hmax, kmax, lmax, nrefl, ihkl(3),
     +     mult, sysab, cent
      real s, smin, smax, cell(6), eps

c     read unit cell

      call lrcell(hklin, cell)

c     compute maximum possible reflection ranges

      hmax = int(cell(1) / dmin) + 1
      kmax = int(cell(2) / dmin) + 1
      lmax = int(cell(3) / dmin) + 1
      
c     set up resolution ranges

      smin = 1.0 / (dmax ** 2)
      smax = 1.0 / (dmin ** 2)

c     accumulate

      nrefl = 0

      do h = -hmax, hmax
         do k = -kmax, kmax
            do l = -lmax, lmax
               s = 4.0 * lstlsq(hklin, h, k, l)

               if ((s .le. smax) .and. (s .ge. smin)) then

                  ihkl(1) = h
                  ihkl(2) = k
                  ihkl(3) = l

                  mult = 1

c     epslon searches for axial zones

                  call epslon(ihkl, eps, sysab)

c     and centric for centric reflections

                  call centr(ihkl, cent)

c     compose factors for multiplicity for this reflection

                  if (cent .eq. 1) mult = mult * 2

                  mult = nint(mult * eps)

                  if (sysab .ne. 1) nrefl = nrefl + mult

               end if

            end do
         end do
      end do

c     now scale by number of symops * 2 - to get the number of "native"
c     reflections

      nrefl = nrefl / (2 * nsymop)

      number = nrefl

      end
      

