      program doser

c     a program to add dose and time columns to an MTZ file - 
c     this will add this information on a per-batch basis, with the
c     values coming from commandline input viz:
c
c     batch N time T dose D
c     
c     this information will come from image headers (acumulated dose
c     calculated by xia2) and will be used in the analysis by Scala
c     and Chef.

      implicit none

c     external subroutines - mostly from ccp4

      external ccpfyp, mtzini, lropen, lwopen, ccperr, parser,
     +     ccpupc, lrclab, ccprcs

c     parameters

      integer mcol
      parameter (mcol = 200)

      integer maxbatch
      parameter (maxbatch = 10000)

      integer maxpar
      parameter (maxpar = 200)

c     variables

      integer j, mtzin, mtzerr, mtzprt, ncol, mtzapp, mtzout
      logical mtzeof, eof
      real dose(maxbatch), time(maxbatch), indata(mcol), 
     +     outdata(mcol), s
      
c     variables to support parameter parsing
      integer ntok, ibeg(maxpar), iend(maxpar), idec(maxpar), 
     +     ityp(maxpar), batch
      real fval(maxpar), tval, dval
      character cval(maxpar)*4, line*400, key*4
      
c     variables to support mtz column handling
      character clab(mcol)*30, nlab(2)*30
      character ctyp(mcol)*1, ntyp(2)*1

c     initialise ccp4 subroutines
      call ccpfyp
      call mtzini

c     banner
      call ccprcs(6, 'DOSER', '$Date: 2008/02/01 14:42:36 $')

c     initialise arrays

      do j = 1, maxbatch
         dose(j) = -1.0
         time(j) = -1.0
      end do

      do j = 1, mcol
         indata(j) = 0.0
         outdata(j) = 0.0
      end do

c     open hklin file - on same index open hklout to copy the information 
c     from the header across...

      mtzin = 1
      mtzprt = 0
      mtzerr = 0

      call lropen(mtzin, 'HKLIN', mtzprt, mtzerr)

c     check status

      if (mtzerr .ne. 0) then
         call ccperr(1, 'error opening HKLIN')
      end if

c     open hklout - no error checking, odd

      mtzout = 1
      mtzerr = 0

      call lwopen(mtzout, 'HKLOUT')

c     parse the commandline input - should be just batch B time T dose D
c     repeated endlessly until "end" or EOF.

 101  continue

      ntok = maxpar
      line = ' '
      key = ' '

      call parser(key, line, ibeg, iend, ityp, fval, cval, idec, ntok,
     +     eof, .true.)

      if (eof) goto 102

      if (key .eq. 'END') then
         goto 102

      else if (key .eq. 'BATC') then

c     check the input
         if (ntok .eq. 1) call ccperr(1, 'input error')

c     get and check the batch number
         if (ityp(2) .ne. 2) call ccperr(1, 'input error')

         batch = nint(fval(2))
         if (batch .gt. maxbatch) call ccperr(1, 'batch too large')
         if (batch .lt. 0) call ccperr(1, 'negative batch')

c     parse and store the values
         do j = 3, ntok, 2
            key = cval(j)
            call ccpupc(key)
            if (key .eq. 'TIME') then
               if (ityp(j + 1) .ne. 2) call ccperr(1, 'input error')
               tval = fval(j + 1)
               time(batch) = tval
            else if (key .eq. 'DOSE') then
               if (ityp(j + 1) .ne. 2) call ccperr(1, 'input error')
               dval = fval(j + 1)
               dose(batch) = dval
            else
               call ccperr(1, 'unknown subkey to batch')
            end if
         end do
      else
         call ccperr(1, 'unknown command')
      end if

      goto 101

 102  continue

c     ok, that should be the input read - now add the required 
c     data to the mtz file... first get the number of columns in the
c     mtz file and column labels

      ncol = mcol
      call lrclab(mtzin, clab, ctyp, ncol)
      call lrncol(mtzin, ncol)

c     next assign the new column labels and types
      clab(ncol + 1) = 'DOSE'
      clab(ncol + 2) = 'TIME'
      ctyp(ncol + 1) = 'R'
      ctyp(ncol + 2) = 'R'
      nlab(1) = 'DOSE'
      nlab(2) = 'TIME'
      ntyp(1) = 'R'
      ntyp(2) = 'R'

c     add these to the mtz file
      mtzapp = 1
      call lwassn(mtzout, nlab, 2, ntyp, mtzapp)

c     read and copy reflections from in to out, adding additional
c     data as we go...

 201  continue

      call lrrefl(mtzin, s, indata, mtzeof)

      if (mtzeof) goto 202

      do j = 1, ncol
         outdata(j) = indata(j)
      end do

      batch = nint(indata(5))

      outdata(ncol + 1) = dose(batch)
      outdata(ncol + 2) = time(batch)

      call lwrefl(mtzout, outdata)

      goto 201

 202  continue

      call lrclos(mtzin)

      mtzprt = 0

      call lwclos(mtzout, mtzprt)

      call ccperr(0, 'Normal termination')
      end


      
