c     symop2mat
c     A program to provide access to the CCP4 library function for parsing
c     symmetry operations into symmetry matrices.
c     
c     This will take an operation on the command line as a single token 
c     (e.g. in ' ' or with no spaces) and write out a symmetry matrix
c     and an XDS REIDX record.
c     
c     e.g. 
c     
c     symop2mat OP h,-l,k
c     
c     FIXME 24/AUG/06 The output matrix for REIDX is integer, this
c                     may not be correct, since -1/2 things are
c                     possible. In particular this is a problem
c                     for using the matrix representation as keys...
c
c                     More on this - actually the reindexing matrix
c                     almost certainly has to be integers, so that is
c                     not a problem... however, it would still be
c                     nice to write this out as a single line, so
c                     I'll so that: ONELINE=number,number...

      program symop2matrix

      implicit none

c     the symmetry operation from the command line
      character*80 symop

c     real matrix to hold the operations
      real matrix(4, 4, 1)

c     summy variables
      integer start, ns

      integer i, j

c     the external subroutine from CCP4 symmetry library
      external symfr2

c     initialise
      call ccpfyp

c     get the symmetry operation from the command line
      call ugtenv('OP', symop)

      if (symop .eq. '') then
         call ccperr(1, 'No symop provided')
      end if

      start = 1
      ns = 1

c     parse the symop
      call symfr2(symop, start, ns, matrix)

c     write out the matrices - first in (4,4) format

      write(*, *) 'The matrix [R,T]:'
      do i = 1, 4
         write(*, *) matrix(i, 1, 1), matrix(i, 2, 1), matrix(i, 3, 1),
     +        matrix(i, 4, 1)
      end do

      write(*, *) ' '
      write(*, *) 'In XDS format:'

c     write in XDS REIDX record format
      write(*, *) 'REIDX= ', matrix(1, 1, 1), matrix(1, 2, 1), 
     +     matrix(1, 3, 1), matrix(1, 4, 1),
     +     matrix(2, 1, 1), matrix(2, 2, 1), 
     +     matrix(2, 3, 1), matrix(2, 4, 1),
     +     matrix(3, 1, 1), matrix(3, 2, 1), 
     +     matrix(3, 3, 1), matrix(3, 4, 1)

      call ccperr(0,'normal termination')
      end
