#!/usr/bin/env tclsh

#
# crank - Automated macromolecular structure solution
#
# Copyright (C) 2004 Steven Ness
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

proc showshelx { code crankbin inputxml mtzxml } {
	set status [runcommand [concat $crankbin/xml2shelx.tcl $inputxml $mtzxml]]
}

proc showcrunch2 { code crankbin inputxml mtzxml } {
	puts "newinput"
		set status [runcommand [concat $crankbin/xml2crunch2newinput.tcl $inputxml $mtzxml]]
	puts "pmf.use"
		set status [runcommand [concat $crankbin/xml2pmf.use.tcl $inputxml]]
	puts "crunch2.use"
	set status [runcommand [concat $crankbin/xml2crunch2crunch.use.tcl $inputxml]]
}

proc showbp3 { code crankbin inputxml mtzxml } {
	set status [runcommand [concat $crankbin/xml2bp3com.tcl $inputxml $mtzxml /dev/null]]
}

proc reference_crank { } {
	global XMLParse

	puts "Crank:"
	puts "Ness, S.R., de Graaff, R.A.G., Abrahams, J.P. & Pannu, N.S. (2004)"
	puts "CRANK: New Methods for Automated Macromolecular Crystal Structure"
	puts "Solution. Structure 12, 1753-1761.\n"
}

proc reference_crunch2 { } {
	global XMLParse

	puts "Crunch2:"
	puts "de Graaff, R.A.G., Hilge, M., van der Plas, J.L. & Abrahams, J.P. (2001)"
	puts "Matrix methods for solving protein substructures of chlorine and"
	puts "sulfur from anomalous data. Acta Cryst. D57, 1857-1862.\n"
# 	puts "van der Plas, J. L., de Graaff, R. A. G., and Schenk, H. (1998). On"
# 	puts "the Use of Eigenvalues and Eigenvectors in the Phase Problem. Acta"
# 	puts "Cryst. A54, 262-266."
#	puts ""
# 	puts "van der Plas, J. L., de Graaff, R. A. G., and Schenk,"
# 	puts "H. (1998). Karle-Hauptman Matrices and Eigenvalues: a Practical"
# 	puts "Approach. Acta Cryst. A54, 267-272."
	puts "Drear Suite:"
	puts "Blessing, R.H. & Smith, G.D. (1999)  Difference structure factor normalization"
	puts "for determining heavy-atom or anomalous scattering substructures."
	puts "J. Appl. Cryst. 32, 664-670.\n"
#       puts "Blessing, R.H. (1987). Data Reduction and Error Analysis for Accurate Single"
#       puts "Crystal Diffraction Intensities. Crystallography Reviews, 1, 3-58."
#       puts ""
# 	puts "Blessing, R.H. (1995). An Empirical Correction for Absorption Anisotropy."
# 	puts "Acta Cryst. A51, 33-38."
# 	puts ""
# 	puts "Blessing, R.H., Guo, D.Y. & Langs, D.A. (1996). Statistical Expectation"
# 	puts "Value of the Debye-Waller Factor and |E(hkl)| Values for Macromolecular Crystals."
# 	puts "Acta Cryst. D52, 257- 266."
# 	puts ""
# 	puts "Blessing, R.H. (1997). LOCSCL: a program to statistically optimize local"
# 	puts "scaling of single-isomorphous-replacement and"
# 	puts "single-wavelength-anomalous-scattering data."
# 	puts "J. Appl. Cryst. 30, 176-177."
# 	puts ""
}

proc reference_bp3 { } {
	global XMLParse

	puts "BP3:"
	if { [string compare [string tolower $XMLParse(soap__run__experiment)] "sad"] == 0 } {
		puts "Pannu, N.S. & Read, R.J. (2004). The application of multivariate"
		puts "statistical techniques improves single-wavelength anomalous"
		puts "diffraction phasing. Acta Cryst. D60, 22-27.\n"
	} else {
		puts "Pannu, N.S., McCoy A.J., and Read, R.J. (2003). Application of the"
		puts "complex multivariate normal distribution to crystallographic methods"
		puts "with insights into multiple isomorphous replacement phasing. Acta"
		puts "Cryst. D59, 1801-1808.\n"
	}
}

proc reference_shelx { } {
	global XMLParse

	puts "SHELX:"
	puts "Schneider, T.R. & Sheldrick, G.M. (2002).  Substructure solution with"
	puts "SHELXD. Acta Cryst. D58, 1772-1779.\n"
}

proc reference_dm { } {
	global XMLParse

	puts "DM:"
	puts "Cowtan, K. (1994). Joint CCP4 and ESF-EACBM Newsletter on Protein"
	puts "Crystallography, 31, p34-38.\n"
}

proc reference_refmac { } {

    # General Refmac Reference
    puts "Refmac:"
    puts "Murshudov, G.N., Vagin, A.A. & Dodson, E.J. (1997) Refinement of"
    puts "macromolecular structures by the maximum-likelihood method."
    puts "Acta Cryst. D53, 240-255.\n"

    # SAD function

    if { [info exists XMLParse(soap__arpwarp__phase_restrain)] &&
         [string compare [string toupper $XMLParse(soap__arpwarp__phase_restrain)] "SADSIR"] == 0 } {
        puts "Using likelihood function:"
        puts "Skubak, P., Murshudov, G.N. & Pannu, N.S. (2004) Direct incorporation"
        puts "of experimental phase information in model refinement."
        puts "Acta Cryst. D60, 2196-2201.\n"
    }

    # MLHL function
    if { [info exists XMLParse(soap__arpwarp__phase_restrain)] &&
         [string compare [string toupper $XMLParse(soap__arpwarp__phase_restrain)] "HL"] == 0 } {
        puts "Using likelihood function:"
        puts "Pannu, N.S., Murshudov, G.N., Dodson, E.J. & Read, R.J. (1998)"
        puts "Incorporation of prior phase information strengthens maximum"
        puts "likelihood structure refinement. Acta Cryst. D54, 1285-1294.\n"
    }

}


proc reference_arpwarp { } {
	global XMLParse

	puts "ARP/wARP:"

# 	puts "Morris, R.J., Zwart, P.H., Cohen, S., Fernandez, F.J., Kakaris, M.,"
# 	puts "Kirillova, O., Vonrhein, C., Perrakis, A. & Lamzin, V.S. (2004)" 
# 	puts "Breaking good resolutions with ARP/wARP. J. Synchr. Rad. 11, 56-59."
# 	puts ""
# 	puts "Morris, R.J., Perrakis, A. & Lamzin, V.S. (2002) ARP/wARP's model-building"
# 	puts "algorithms. I. The main chain. Acta Cryst. D58, 968-975"
# 	puts ""
# 	puts "Perrakis, A., Harkiolaki, M., Wilson, K.S. & Lamzin, V.S. (2001)"
# 	puts "ARP/wARP and molecular replacement. Acta Cryst. D57, 1445-1450"
# 	puts ""
# 	puts "Lamzin, V.S., Perrakis, A. & Wilson, K.S. (2001) The ARP/WARP suite"
# 	puts "for automated construction and refinement of protein models."
# 	puts "In Int. Tables for Crystallography. Vol. F: Crystallography of"
# 	puts "biological macromolecules (Rossmann, M.G. & Arnold, E. eds.),"
# 	puts "Dordrecht, Kluwer Academic Publishers, The Netherlands, pp. 720-722"
# 	puts ""
# 	puts "Lamzin, V.S. & Perrakis, A. (2000) Current state of automated"
# 	puts "crystallographic data analysis. Nature Struct. Biol. 7,"
# 	puts "Structural Genomics Supplement, 978-981"
# 	puts ""
	puts "Perrakis, A., Morris, R.M. & Lamzin, V.S. (1999) Automated protein"
	puts "model building combined with iterative structure refinement."
	puts "Nature Struct. Biol. 6, 458-463)"
	puts ""
# 	puts "Perrakis, A., Sixma, T.K., Wilson K.S. & Lamzin, V.S. (1997) wARP:"
# 	puts "improvement and extension of crystallographic phases by weighted"
# 	puts "averaging of multiple refined dummy atomic models. Acta Cryst. D53, 448-455"
# 	puts ""
# 	puts "Lamzin, V.S. & Wilson, K.S. (1997) Automated refinement for protein"
# 	puts "crystallography. In Meth. Enz. (Carter, C. & Sweet, B. eds.) 277, 269-305"
# 	puts ""
# 	puts "Lamzin, V.S. & Wilson, K.S. (1993) Automated refinement of protein"
# 	puts "models. Acta Cryst. D49, 129-147"
# 	puts ""
}

proc show_references { } {
	global XMLParse

    puts "References:"
    puts "----------"
	reference_crank
	for { set i 0 } { [info exists XMLParse(soap__run__commands__job__$i)] } { incr i } {
		set job [string tolower $XMLParse(soap__run__commands__job__$i)]

		if { [StringSame $job crunch2] } {
			reference_crunch2
		} elseif { [StringSame $job shelxd] } {
			reference_shelx
		} elseif { [StringSame $job bp3] } {
			reference_bp3
		} elseif { [StringSame $job dm] } {
			reference_dm
		} elseif { [StringSame $job arpwarp] } {
			reference_arpwarp
			reference_refmac
		}
	}
}

proc show_parameters { } {
}

proc print_banner { } {		  
	global tcl_platform

	puts "-------------------------------------------------------------------------------"
	puts "                         ___   _ __   __ _   _ __   | | __ "
	puts "                        / __| | '__| / _` | | '_ \\  | |/ / "
	puts "                       | (__  | |   | (_| | | | | | |   <  "
	puts "                        \\___| |_|    \\__,_| |_| |_| |_|\\_\\ "
	puts "    "
	puts "                              Crank - Version 0.9"
	puts "      "
	puts "                              Steven Ness (c) - 2004"
	puts "                              s.ness@chem.leidenuniv.nl"
	puts "      "
    puts "                              LIC, Gorlaeus Laboratories"
    puts "                              Leiden University"
    puts "                              PO Box 9502, 2300 RA Leiden"
	puts "                              The Netherlands"
	puts "    "
	puts "                 Crank - Automated Macromolecular Structure Solution"
	puts "\n"
	puts "Please report any bugs or problems to crank@chem.leidenuniv.nl\n"
	puts "Crank is a system to solve the structure of macromolecules using a variety "
	puts "of crystallographic programs.  Please reference both Crank (Ness et al. 2004) "
	puts "and all the individual programs used.\n"
	puts "Below is printed a customized References section based on"
	puts "this specific run of crank.\n"

	show_references

	puts ""
	puts "Tcl Version information: [info patchlevel]"
	parray tcl_platform
	puts ""
}


#
# Print out a very basic summary of where to find the files in the
# Crank job that was just run.
#
# In the future, add a function to summarize all the information from
# the run, perhaps in an XML file as well as a text summary.
#
proc print_info_summary { } {
	global XMLParse
	global orig_pwd

	puts "<a name=\"dmdm\"><h1>dm 2.1</h1></a>"
	puts "<B><FONT COLOR=\"\#FF0000\"><!--SUMMARY_BEGIN-->"
	puts "<pre>"

	puts "CRANK SUMMARY"
	puts "-------------"
	puts ""

	#
	# Program summary
	#
	puts "Crank ran the following programs:"
	for { set i 0 } { [info exists XMLParse(soap__run__commands__job__$i)] } { incr i } {
		set job [string toupper $XMLParse(soap__run__commands__job__$i)]
		puts "   $i) $job"
	}
	puts ""
	
	#
	# Location of run directory
	#
	puts -nonewline "These programs were run in the directory:"
	if { [info exists XMLParse(soap__run__directory)] } {
		set crank_run_dir $XMLParse(soap__run__directory)
	} else {
		set crank_run_dir $orig_pwd
	}
	puts "$crank_run_dir"
	puts ""

	#
	# Location of output .mtz file
	# 
	puts -nonewline "The output MTZ file can be found at:"
	puts $XMLParse(soap__run__mtz_out__filename)
	puts ""

	#
	# Describe the various output directories
	#
	puts "The Crank run directory ($crank_run_dir)"
	puts "contains various subdirectories including both the directories where programs"
	puts "are run and directories for program output.  Here is a short summary of those"
	puts " directories.  For more information, please consult the Crank manual."
	puts ""
	puts "Directory      Contents"
	puts "---------      --------"
	puts "mtz               Input, temporary and output MTZ files from this run"
	puts "sca               Input .mtz files converted to .sca format for Direct Methods programs"
	puts "xml               Input, temporary and output XML files"
	puts "log               Central repository for all logfiles.  These files can also"
	puts "                      be found in the program run directories with the name 'logfile'"
	puts "0-scaling         First step in the Crank run, the scaling of all datasets to one another"
	puts "1-programname     The run directory for the first major crystallographic program to run (e.g. 1-crunch2)"
	puts "2-programname     The run directory for the second major crystallographic program."
	puts "3...-programname  The run directory for the third major crystallographic program."
	puts ""

	#
	# Print footer
	#
	puts "This Crank run is now complete."
	puts ""
	puts "You may need to run the other enantiomorph manually if you have a poor map, but good statistics."
	puts "To run the other enantiomorph, you will need to, depending on the space group, either"
	puts "(1) flip the signs of the atomic coordinates and re-run bp3 and dm, or"
	puts "(2) change the space group and re-run crank, if you have one of the 22 enantiomorphic space groups."
	puts ""
	puts "This will be automated in future versions of crank."
	puts ""
	puts "<!--SUMMARY_END--></FONT></B>"
	puts "</pre>"

}

#
# Output a summary of the data produced by Crank in this run.
#

# This isn't comparison data, where we compare our results to a model,
# rather it is a summary of the data produced by the crystallographic
# programs.
proc print_data_summary { } {
}

