#!/usr/bin/env tclsh

#
# crank - Automated macromolecular structure solution
#
# Copyright (C) 2004 Steven Ness
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

#
# Convert a number of CRUNCH2 trial*.XYZ files to a single CRANK XML coordinate format file
#

#
# Usage: crunch2multitrials2xmlcoords.tcl trial_code crunch2_element trial*.XYZ trial*.XYZ trial*.XYZ
#

#
# The output XML is in the following format:
#
# <coordinate_data>
# 	<code></code>
# 	<atom>
# 		<atom_name></atom_name>
# 		<x></x>
# 		<y></y>
# 		<z></z>
# 		<occupancy></occupancy>
# 	</atom>
# 	<fom></fom>
# </coordinate_data>
#

if { [lindex $argv 2] == "" } {
	puts "Usage:"
	puts "crunch2multitrials2xmlcoords.tcl trial_code crunch2_element trial*.XYZ trial*.XYZ trial*.XYZ"
	puts " "
	puts "Copyright / GPL - Steven Ness 2004"
	puts " "
	exit
}

# Process command line arguments
set trial_code [lindex $argv 0]
set crunch2_element [lindex $argv 1]

puts "<coordinate_data>"


# set curr_trial 2
# while { [file exists [lindex $argv $curr_trial]] } {
#  	puts "argv-$curr_trial=([lindex $argv $curr_trial])"
# 	incr curr_trial
# }
# exit

#
# I'm not really sure why the following stopped working, but I've
# left it in just in case.  It worked fine, but now doesn't.  It might
# have something to do with the new way I'm running these commands
#
# set curr_trial 0
# while { $curr_trial < [llength $argvfiles] } {


set curr_trial 2
while { [file exists [lindex $argv $curr_trial]] } {
	set trial_filename [lindex $argv $curr_trial]

	# Open the .xml file
	set trial_fileid [open $trial_filename r]

	# Output each of the best files to stdout as XML
	puts "\t<code>$trial_code</code>"
	puts "\t<trial id=\"[expr $curr_trial - 2]\">"

	set atomnum 0
	set i 1
	while { [gets $trial_fileid line] >= 0} {
		regexp { *([-0-9.]*) *([-0-9.]*) *([-0-9.]*) *([-0-9.]*)} $line match occ x y z;
		if { $x != "" } {
			puts "\t\t<atom id=\"$atomnum\">"
			puts "\t\t\t<atom_id>$crunch2_element$i</atom_id>"
			puts "\t\t\t<element_name>$crunch2_element</element_name>"
			puts "\t\t\t<x>$x</x>"
			puts "\t\t\t<y>$y</y>"
			puts "\t\t\t<z>$z</z>"
			puts "\t\t\t<occupancy>$occ</occupancy>"
			puts "\t\t</atom>"
			incr atomnum
		} else {
			puts "\t\t<score>$occ</score>"
			puts "\t\t<scoretype>crunch2_fom</scoretype>"
			break
		}
		incr i
	}

	puts "\t</trial>"

	close $trial_fileid

	incr curr_trial
}

puts "</coordinate_data>"





#
# Some old code that might be useful later.
#

# set curr_trial 0
# while { $curr_trial < [llength $argvfiles] } {
# 	puts "$curr_trial [lindex $argvfiles $curr_trial]"
# 	incr curr_trial
# }
# exit


# for { set i 1 } { $i <= 10 } { incr i } {
# 	puts "argv-$i=([lindex $argv $i])"
# }
# exit


# set argvfiles [split [lindex $argv 2]]
