#!/usr/bin/env tclsh

#
# Convert a CRUNCH2 trial*.XYZ file to CRANK XML coordinate format
#
#
# Usage: crunch2trial2xmlcoords.tcl trial_code crunch2_element trial*.XYZ 
#

#
# The output XML is in the following format:
#
# <coordinate_data>
# 	<code></code>
# 	<atom>
# 		<atom_name></atom_name>
# 		<x></x>
# 		<y></y>
# 		<z></z>
# 		<occupancy></occupancy>
# 	</atom>
# 	<fom></fom>
# </coordinate_data>

if { [lindex $argv 2] == "" } {
	puts "Usage:"
	puts "crunch2trial2xmlcoords.tcl trial_code crunch2_element trial*.XYZ"
	puts " "
	puts "Copyright / GPL - Steven Ness 2004"
	puts " "
	exit
}

# Process command line arguments
set trial_code [lindex $argv 0]
set crunch2_element [lindex $argv 1]
set trial_filename [lindex $argv 2]

# Open the .xml file
set trial_fileid [open $trial_filename r]

# Output each of the best files to stdout as XML
puts "<coordinate_data>"
puts "\t<code>$trial_code</code>"

set atomnum 0
set i 1
while { [gets $trial_fileid line] >= 0} {
	regexp { *([-0-9.]*) *([-0-9.]*) *([-0-9.]*) *([-0-9.]*)} $line match occ x y z;
	if { $x != "" } {
		puts "\t<atom id=\"$atomnum\">"
		puts "\t\t<atom_id>$crunch2_element$i</atom_id>"
		puts "\t\t<element_name>$crunch2_element</element_name>"
		puts "\t\t<x>$x</x>"
		puts "\t\t<y>$y</y>"
		puts "\t\t<z>$z</z>"
		puts "\t\t<occupancy>$occ</occupancy>"
		puts "\t</atom>"
		incr atomnum
	} else {
		puts "\t<score>$occ</score>"
		puts "\t<scoretype>crunch2_fom</scoretype>"
		break
	}
	incr i
}

puts "</coordinate_data>"
