#!/usr/bin/env tclsh

#
# crank - Automated macromolecular structure solution
#
# Copyright (C) 2004 Steven Ness
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

#
# runcrunch2xml
#
# Run CRUNCH2 given a number of data elements contained
# in one or a number of XML files.
#
# Usage:
#
#   crunchtrial2xml.tcl
#
set crankbin "$env(CRANK)/bin"

# Convert CRUNCH2 output to XML
set status [runcommand [concat $crankbin/crunch2output2xmlstats.tcl $code output > $code.out.stats.xml]]
if { $status != "SUCCESS" } {
	crank_error "runcrunch2xml::crunch2output2xmlstats.tcl failed"
}

XMLParsefile "$code.out.stats.xml"
set ntrials $XMLParse(crunch2_log__ntrials)

# Convert all trial files to XML
XMLParsefile $userxml
set atomname $XMLParse(dataset_info__atom_name)
for { set i 1 } { $i <= $ntrials } { incr i } {
	set status [runcommand [concat $crankbin/crunch2trial2xmlcoords.tcl $code $atomname trial${i}a.XYZ > $code.trial${i}a.out.coords.xml]]
	if { $status != "SUCCESS" } {
		crank_error "runcrunch2xml::crunch2trial2xmlcoords.tcl failed"
	}
}

set alltrials [glob trial*a.XYZ]

set status [runcommand [concat $crankbin/crunch2multitrials2xmlcoords.tcl $code $atomname $alltrials > $code.out.coords.xml]]
if { $status != "SUCCESS" } {
	crank_error "runcrunch2xml::crunch2multitrials2xmlcoords.tcl failed"
}
