#!/usr/bin/env tclsh

#
# Convert a CCP4 ORTH FRAC (fractional coordinates) file to CRANK XML
# coordinate format.
#
# Usage: frac2xmlcoords.tcl trial_code trial.frac
#

#
# The output XML is in the following format:
#
# <coordinate_data>
# 	<code></code>
# 	<atom>
# 		<atom_name></atom_name>
# 		<x></x>
# 		<y></y>
# 		<z></z>
# 		<occupancy></occupancy>
# 	</atom>
# 	<fom></fom>
# </coordinate_data>

if { [lindex $argv 1] == "" } {
	puts "Usage:"
	puts "frac2xmlcoords.tcl element trial.frac"
	puts " "
	puts "Copyright / GPL - Steven Ness 2004"
	puts " "
	exit
}

# Process command line arguments
set element [lindex $argv 0]
set trial_filename [lindex $argv 1]

# Open the .xml file
set trial_fileid [open $trial_filename r]

# Output each of the best files to stdout as XML
puts "<coordinate_data>"

set atomnum 0
set i 1
gets $trial_fileid line
gets $trial_fileid line
while { [gets $trial_fileid line] >= 0} {
	regexp {ATOM([0-9]*) *Ano *([-0-9.]*) *([-0-9.]*) *([-0-9.]*) *([-0-9.]*).*} $line match atomnum x y z occ
 	if { $x != "" } {
 		puts "\t\t<atom id=\"[expr $i - 1 ]\">"
 		puts "\t\t\t<atom_id>$element$i</atom_id>"
 		puts "\t\t\t<element_name>$element</element_name>"
 		puts "\t\t\t<x>$x</x>"
 		puts "\t\t\t<y>$y</y>"
 		puts "\t\t\t<z>$z</z>"
 		puts "\t\t\t<occupancy>$occ</occupancy>"
 		puts "\t\t</atom>"
 		incr atomnum
 	} else {
 		break
 	}
	incr i
}

puts "</coordinate_data>"
