#!/bin/sh

#
# mtz2drear.sh 
#

# Use the CCP4 program ECALC to convert F(+) and F(-) to EA values.
# With the use of different flags, one can change resolution
# (RESOLUTION), number of reflections in each shell (SHELL).  
# EXCLUDE CENTRIC is tured on by default, as is necessary for use with
# CRUNCH2
#
# Rag's emailed info:
# EXCLUDE CENTRIC, SHELL 500 and RESOLUTION x where x is the resolution of the
#  data in Angstrom + 0.5.
#

#
# Usage:
#
#  mtz2drear.sh input.mtz output.drear logfile f(+) sigf(+) f(-) sigf(-) shell resolution
#
# Where:
# $1 - Input .mtz file
# $2 - Name of output .drear file
# $3 - f(+) column name
# $4 - sigf(+) column name
# $5 - f(-) column name
# $6 - sigf(-) column name
# $7 - SHELL - number of reflections in each resolution shell
# $8 - RESOLUTION - resolution cutoff
#

# Check for correct usage
if [ "$8" = "" ];
    then
    echo "Usage:"
    echo "mtz2drear.sh input.mtz output.drear f(+) sigf(+) f(-) sigf(-) shell resolution"
	echo "Where:"
	echo "input.mtz    - Input .mtz file"
	echo "output.drear - Output .drear file"
	echo "f(+)         - f(+) column name"
	echo "sigf(+)      - sigf(+) column name"
	echo "f(-)         - f(-) column name"
	echo "sigf(-)      - sigf(-) column name"
	echo "shell        - SHELL - number of reflections in each resolution shell"
	echo "resolution   - RESOLUTION - resolution cutoff"
	echo ""
	echo "Steven Ness - (c) 2004"
    exit
fi

#
# Run ecalc to generate E and SIG
#
ecalc hklin $1 hklout tmpdrear.mtz << END
TITLE calculate evalue from f+ f-
LABIN FP=${3} SIGFP=${4} FPH=${5} SIGFPH=${6}
EXCLUDE CENTRIC
SHELL ${7}
RESOLUTION ${8} 
END

#
# Run mtzMADmod to generate DANO and SIGDANO
#
mtzMADmod HKLIN $1 HKLOUT tmpmtzMADmod.mtz << END
TITLE Generate DANO/SIGDANO columns for derivative data
LABIN F1(+)=${3} SIGF1(+)=${4} F1(-)=${5} SIGF1(-)=${6}
LABOUT F1=F SIGF1=SIGF D1=DANO SIGD1=SIGDANO
END

#
# Read both files into sftools and output the 
# DANO and SIGDANO columns 
#
sftools <<END
read tmpdrear.mtz col E SIGE
R
read tmpmtzMADmod.mtz col DANO SIGDANO
write sftools.mtz
exit
END

#
# Output reflections in .drear format
#
mtz2various HKLIN sftools.mtz HKLOUT tmpdrear.drear << END
OUTPUT USER '(3I5,4F9.3)'
LABIN FP=DANO SIGFP=SIGDANO FPH=E SIGFPH=SIGE
END

sortdrear.tcl tmpdrear.drear > $2

#rm tmpdrear.mtz
#rm tmpdrear.drear


