#!/bin/sh

#
# crank - Automated macromolecular structure solution
#
# Copyright (C) 2004 Steven Ness
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# Convert a .mtz file to Scalepack .sca format
#

#
# Usage:
#
#  hkl2mtz.sh <anom> input.mtz output.hkl input_f input_sigf
#
# Where:
# 
#  <anom>     - $1 - "1" - anomalous data - "0" - non-anomalous data
#  input.mtz  - $2 - Input .mtz file
#  output.hkl - $3 - Output SHELX .hkl file
#  input_i    - $4 - Input I(+) column
#  input_sigi - $5 - Input SIGI(+) column
#  input_i    - $6 - Input I(-) column
#  input_sigi - $7 - Input SIGI(-) column
#

# Check for correct usage
if [ "$5" = "" ];
    then
    echo "Usage:"
    echo "mtz2sca.sh 0 input.mtz output.hkl i sigi "
	echo " or "
    echo "mtz2sca.sh 1 input.mtz output.hkl i(+) sigi(+) i(-) sigi(-)"
    exit
fi

if [ "$1" = "0" ]; then

mtz2various HKLIN $2 HKLOUT tmp1.sca <<EOF
LABIN I=$4 SIGI=$5
OUTPUT SCALEPACK
EOF

mtz2various HKLIN $2 HKLOUT tmp2.sca <<EOF
LABIN DUM1=$4 DUM2=$5
OUTPUT USER '(3I4,2F8.1)'
EOF

cat tmp1.sca tmp2.sca > $3

rm tmp1.sca tmp2.sca

else

mtz2various HKLIN $2 HKLOUT $3 <<EOF
LABIN I(+)=$4 SIGI(+)=$5 I(-)=$6 SIGI(-)=$7
OUTPUT SCALEPACK
EOF

fi


