#!/usr/bin/env tclsh

#
# crank - Automated macromolecular structure solution
#
# Copyright (C) 2004 Steven Ness
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# 
# pmfcoordinaten2xmlcoords.tcl
#
# Convert a PMF coordinaten file to CRANK XML coordinate format
#
# Usage: pmfcoordinaten2xml.tcl trial_code crunch2_element coordianten
#

#
# The output XML is in the following format:
#
# <coordinate_data>
# 	<code></code>
# 	<atom>
# 		<atom_name></atom_name>
# 		<x></x>
# 		<y></y>
# 		<z></z>
# 		<occupancy></occupancy>
# 	</atom>
# 	<fom></fom>
# </coordinate_data>
#

if { [lindex $argv 2] == "" } {
	puts "Usage:"
	puts "pmfcoordinaten2xml.tcl trial_code crunch2_element coordianten"
	puts " "
	puts "Copyright / GPL - Steven Ness 2004"
	puts " "
	exit
}

# Process command line arguments
set trial_code [lindex $argv 0]
set crunch2_element [lindex $argv 1]
set trial_filename [lindex $argv 2]

# Open the .xml file
set trial_fileid [open $trial_filename]

# Output each of the best files to stdout as XML
puts "<coordinate_data>"

set fileid [open $trial_filename r]
puts "\t<code>$trial_code</code>"

set trialnum 0
set atomnum 1
while { [gets $trial_fileid line] >= 0} {
	if { [regexp { *([0-9]) *} $line full natoms] } {
		set atomnum 1
		puts "\t<trial id=\"$trialnum\">"
		for {set i 0} {$i < $natoms} {incr i;} {
 			gets $trial_fileid line
 			regexp {([0-9.]*) *([-0-9.]*) *([-0-9.]*) *([-0-9.]*)} $line match occ x y z;
 			if { $occ != "" } {
 				puts "\t\t<atom id=\"[expr $atomnum - 1]\">"
 				puts "\t\t\t<atom_id>$crunch2_element$atomnum</atom_id>"
 				puts "\t\t\t<element_name>$crunch2_element</element_name>"
 				puts "\t\t\t<x>$x</x>"
 				puts "\t\t\t<y>$y</y>"
 				puts "\t\t\t<z>$z</z>"
 				puts "\t\t\t<occupancy>$occ</occupancy>"
 				puts "\t\t</atom>"
 			} else {
 				break
 			}
 			incr atomnum
		}
		puts "\t</trial>"
		incr trialnum
	}
}

puts "</coordinate_data>"
