#!/usr/bin/env tclsh

#
# Scale all the columns of the input mtz file with the CCP4 program
# scaleit.  
#
# Remember to handle native data seperately, in the datasets that have
# native data. 
#
# This program should handle MAD, SIR and SIR(AS) data for now with
# MIR and MIR(AS) to be added later.  
#

#
# Usage:
#    scaleit-all-columns.tcl input.mtz output.mtz column_labels
#
#
set crankbin "$env(CRANK)/bin"
source [file join $crankbin crankutils.tcl]
source [file join $crankbin crankprint.tcl]

if { [lindex $argv 2] == "" } {
	puts "Usage:"
    puts " "
    puts "scaleit-all-columns.tcl inputxml inputmtz output.mtz"
	puts "Where:"
	puts "  inputxml -> Name of inputxml file"
	puts "  output.mtz -> Name of output .mtz file"
	puts ""
	puts "Copyright (c) Steven Ness 2004"
    exit
}

set inputxml [lindex $argv 0]
if { [file exists $inputxml] == 0 } {
	crank_error "mtz2drear::inputxml file does not exist"
}
XMLParsefile $inputxml

set inputmtz [lindex $argv 1]
set outputmtz [lindex $argv 2]

#
# Set variables based on data in inputxml file
#
set n_datasets $XMLParse(soap__run__mtz_in__n_datasets)

# #
# # For all columns without FMEAN, use mtzMADmod.sh to calculate FMEAN
# #
# set status [catch {eval exec add-fmean-to-mtz.tcl $inputxml fmean-tmp.mtz } output ]
# set newout [split $output \n]
# for { set i 0 } { $i < [llength $newout] } { incr i } {
# 	set tmp [lindex $newout $i]
# 	set tmp [split $tmp " "]
# 	set f_column($i) [lindex $tmp 0]
# 	set sf_column($i) [lindex $tmp 1]
# }

#
# Start building the scaleit command
#
set scaleit_command "HKLIN extracted.mtz HKLOUT $outputmtz"

#
# These need to be looked at in more detail.  I'm just copying the 
# defaults from $CCP4_EXAM/scaleit.exam.
#
set scaleit_script "TITLE Data scaled with Crank via scaleit-all-columns.tcl"
#
# sness - This one definitely needs changing.  There should be some
#         way to automatically determine the resolution limits
#         perhaps by running scaleit multiple times.
#
set scaleit_script "$scaleit_script\nRESO 10 2.3"
set scaleit_script "$scaleit_script\nWEIGHT"
set scaleit_script "$scaleit_script\nREFINE ISOTROPIC"
set scaleit_script "$scaleit_script\nCONV ABS 0.0001 TOLR 0.000000001 NCYC 4"
set scaleit_script "$scaleit_script\nEXCLUDE FP SIG 5 FMAX 10000000"
set scaleit_script "$scaleit_script\nLABIN"
for { set i 0 } { $i < $n_datasets } { incr i } {
#	if { $XMLParse(soap__run__mtz_in__data__anomalous__$i) == 0 } {
		set fp $XMLParse(soap__run__mtz_in__data__f__$i)
		set sigfp $XMLParse(soap__run__mtz_in__data__sigf__$i)
		set append ""
# 	} else {
# 		set fp_plus $XMLParse(soap__run__mtz_in__data__f_plus__$i)
# 		set sigfp_plus $XMLParse(soap__run__mtz_in__data__sigf_plus__$i)
# 		set fp_minus $XMLParse(soap__run__mtz_in__data__f_minus__$i)
# 		set sigfp_minus $XMLParse(soap__run__mtz_in__data__sigf_minus__$i)
# 		set append ""
# 	}

	if { $i == 0 } {
		set scaleit_script "$scaleit_script -\nFP=$fp SIGFP=$sigfp"
	} else {
		set scaleit_script "$scaleit_script -\nFPH$i=$fp SIGFPH$i=$sigfp"
	}
}

set scaleit_script "$scaleit_script\nAUTO\nEND"

#
# Run SCALEIT on the input provided
#
puts "scaleit_command=($scaleit_command)"
puts "scaleit_script=($scaleit_script)"

set command "scaleit $scaleit_command << \"$scaleit_script\""
set status [catch {eval exec $command } output ]
puts $output
