#!/usr/bin/env tclsh

#
# crank - Automated macromolecular structure solution
#
# Copyright (C) 2004 Steven Ness
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

#
# shelxcfa2sca.tcl
#
# Tcl program to convert the FA values output of shelxc to 
# scalepack format
#

# Set CRANK path
set crankbin "$env(CRANK)/bin"
source [file join $crankbin crankutils.tcl]

#
# Usage:
#
#   runshelxcxml.tcl input.hkl output.sca
#

# Set CRANK path
set crankbin "$env(CRANK)/bin"
source [file join $crankbin crankutils.tcl]

#
# Main
#

# Check for usage information
if { [lindex $argv 2] == "" } {
	puts "Usage:"
	puts "   shelxcfa2sca.tcl code.mtz.xml input.hkl output.sca"
	puts " "
	puts "Where:"
	puts "  input.hkl -> input FA file from SHELXC"
	puts "  output.sca -> output .sca file"
	puts ""
	puts " Copyright / GPL - Feb 2004 - Steven Ness"
	puts " "
	exit
}

set mtzxml [lindex $argv 0]
if { [file exists $mtzxml] == 0 } {
	crank_error "xml2shelxc.tcl::mtzxml file does not exist"
}
XMLParsefile $mtzxml

set inputfileid [open [lindex $argv 1] r]
set outputfileid [open [lindex $argv 2] w]

#  Output spacegroup
if { [info exists XMLParse(dataset_info__spacegroup__operator__0)] } {
    set operator_0 $XMLParse(dataset_info__spacegroup__operator__0)
} else {
    set operator_0 ""
}

if { [info exists XMLParse(dataset_info__spacegroup__operator__1)] } {
    set operator_1 $XMLParse(dataset_info__spacegroup__operator__1)
} else {
    set operator_1 ""
}

if { [info exists XMLParse(dataset_info__spacegroup__operator__2)] } {
    set operator_2 $XMLParse(dataset_info__spacegroup__operator__2)
} else {
    set operator_2 ""
}
set spacegroup "$XMLParse(dataset_info__spacegroup__lattice)${operator_0}${operator_1}${operator_2}"
set spacegroup [string tolower $spacegroup]

#  Output cell parameters
puts $outputfileid "    1"
puts $outputfileid " -985"

set output [format "%10.3f%10.3f%10.3f%10.3f%10.3f%10.3f %s" \
			  $XMLParse(dataset_info__cell__cell_a) \
			  $XMLParse(dataset_info__cell__cell_b) \
			  $XMLParse(dataset_info__cell__cell_c) \
			  $XMLParse(dataset_info__cell__cell_alpha) \
			  $XMLParse(dataset_info__cell__cell_beta) \
			  $XMLParse(dataset_info__cell__cell_gamma) \
			   $spacegroup ]
puts $outputfileid $output

while { [gets $inputfileid line] >= 0} {
	if { [regexp { *([-0-9]*) *([-0-9]*) *([-0-9]*) *([-0-9.-]*) * ([-0-9.-]*) * ([-0-9]*) *} $line full h k l fa sigfa phase] } {
#		puts $outputfileid "$h $k $l $fa $sigfa"
		set output [format "%4i%4i%4i%8.1f%8.1f" $h $k $l $fa $sigfa]
		puts $outputfileid $output
	}

}
