#!/usr/bin/env tclsh

#
# crank - Automated macromolecular structure solution
#
# Copyright (C) 2004 Steven Ness
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


#
# Convert a SHELXC trial *.hat file to CRANK XML coordinate format
#
#
# Usage: shelxhat2xmlcoords.tcl trial_code crunch2_element trial*.XYZ 
#

#
# The output XML is in the following format:
#
# <coordinate_data>
# 	<code></code>
# 	<atom>
# 		<atom_name></atom_name>
# 		<x></x>
# 		<y></y>
# 		<z></z>
# 		<occupancy></occupancy>
# 	</atom>
# 	<fom></fom>
# </coordinate_data>

if { [lindex $argv 0] == "" } {
	puts "Usage:"
	puts "shelxhat2xmlcoords.tcl file.hat"
	puts " "
	puts "Copyright / GPL - Steven Ness 2004"
	puts " "
	exit
}

# Process command line arguments
set pdb_filename [lindex $argv 0]

# Open the .xml file
set pdb_fileid [open $pdb_filename r]

# Output each of the best files to stdout as XML
puts "<coordinate_data>"

while { [gets $pdb_fileid line] >= 0} {
	if { [regexp {SFAC (.*) *}  $line full element] } {
		break
	}
	if { [regexp {UNIT.*}  $line full] } {
		break
	}
}

set atomnum 0
set i 1
while { [gets $pdb_fileid line] >= 0} {
	regexp {([A-Z0-9a-z]*) *([0-9]*) *([-0-9.]*) *([-0-9.]*) *([-0-9.]*) *([-0-9.]*) *} \
		$line full atomname num x y z occ;
	if { [regexp {HKLF.*}  $line full] } {
		break
	}
	if { $x != "" } {
		puts "\t<atom id=\"$atomnum\">"
		puts "\t\t<atom_id>$element$atomnum</atom_id>"
		puts "\t\t<element_name>$element</element_name>"
		puts "\t\t<x>$x</x>"
		puts "\t\t<y>$y</y>"
		puts "\t\t<z>$z</z>"
		puts "\t\t<occupancy>$occ</occupancy>"
		puts "\t</atom>"
		incr atomnum
		incr i
	}
}

puts "</coordinate_data>"
