#!/usr/bin/env tclsh

#
# crank - Automated macromolecular structure solution
#
# Copyright (C) 2004 Steven Ness
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

#
# splithkl.tcl
# 
# Take a .hkl file as output by mtz2various and split
#  it into a .ins file and a proper SHELX .hkl file
#

#
# Usage:
#
# splithkl.tcl input.hkl output.ins output.hkl
#

if { [lindex $argv 2] == "" } {
	puts "Usage:\n"
	puts "splithkl.tcl input.hkl output.ins output.hkl\n"
	puts "Splits a .hkl file as output by mtz2various into"
	puts "a .ins file and a SHELX .hkl file\n"
	puts "Where:"
	puts "  input.hkl -> input .hkl file from mtz2various"
	puts "  output.ins -> output .ins file"
	puts "  output.hkl -> output .hkl file"
	exit
}

set inputfileid [open [lindex $argv 0] "r"]
set output_ins_id [open [lindex $argv 1] "w"]
set output_hkl_id [open [lindex $argv 2] "w"]

set in_ins 1

while { [gets $inputfileid line] >= 0 } {
	if { $in_ins == 1 } {
		puts $output_ins_id $line
	} else {
		puts $output_hkl_id $line
	}
	if { [regexp {HKLF.*} $line ] } {
		set in_ins 0
	}
}
