#!/usr/bin/env tclsh

#
# crank - Automated macromolecular structure solution
#
# Copyright (C) 2004 Steven Ness
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

#
# Tcl program to take the *.user.xml and *.mtz.xml files,
# the "CRANK XML dataset info" files, and create a CRUNCH2 com file
#

#
# Usage:
#  xml2crunch2com.tcl user.xml mtz.xml
#


# Set CRANK path
set crankbin "$env(CRANK)/bin"
source [file join $crankbin crankutils.tcl]

#
# Output a CRUNCH2 newinput file corresponding to the *.user.xml,
# and *.mtz.xml XML files read in.  
#
proc OutputCRUNCH2comfile { } {
	global XMLParse

	puts "\#!/bin/sh"
	puts " "

	set code $XMLParse(dataset_info__code)

	set ref_in "$code.in.drear"
	set hits_out "crunch2.out.hits"
        set model_in " "
	if { $XMLParse(soap__crunch2__pmf) == 1 } {
          set model_in "MODELIN coordinaten.xyz"
	}

	puts "crunch2 HKLIN $ref_in $model_in HITS $hits_out << END"

	puts "TRY 1 $XMLParse(soap__crunch2__ntrials)"
	puts "NCYC $XMLParse(soap__crunch2__ncycles)"

	if { $XMLParse(soap__crunch2__pmf) == 1 } {
          puts "ICOO 1"
	}

        set cell_a $XMLParse(dataset_info__cell__cell_a)
        set cell_b $XMLParse(dataset_info__cell__cell_b)
        set cell_c $XMLParse(dataset_info__cell__cell_c)
        set cell_alpha $XMLParse(dataset_info__cell__cell_alpha)
        set cell_beta $XMLParse(dataset_info__cell__cell_beta)
        set cell_gamma $XMLParse(dataset_info__cell__cell_gamma)
        puts "CELL $cell_a $cell_b $cell_c $cell_alpha $cell_beta $cell_gamma"

        puts "SYMM $XMLParse(dataset_info__spacegroup__number)"
        puts "NATOM $XMLParse(dataset_info__natoms)"
        puts "SCATT $XMLParse(soap__crunch2__scattering_power)"

	puts "END"
}


#
# Main
#
if { [lindex $argv 1] == "" } {
	puts "Usage:"
	puts "  xml2crunch2com.tcl input.xml mtz.xml "
	puts " "
	puts "Where:"
	puts "  input.xml -> input CRANK XML file with dataset info"
	puts "  mtz.xml -> input CRANK XML file with MTZ info"
	puts " Copyright/GPL Feb 2004 Steven Ness"
	exit
}

#
# Setup variable names
#
set inputxml [lindex $argv 0]
set mtzxml [lindex $argv 1]

#
# Check to make sure files exist
#
if { [file exists $inputxml] == 0 } {
	crank_error "xml2crunch2com.tcl::inputxml file does not exist"
}
if { [file exists $mtzxml] == 0 } {
	crank_error "xml2crunch2com.tcl::mtzxml file does not exist"
}

#
# Read in XML files
#
XMLParsefile $inputxml
XMLParsefile $mtzxml

#
# Generate .com file
#
OutputCRUNCH2comfile



