#!/usr/bin/env tclsh

#
# crank - Automated macromolecular structure solution
#
# Copyright (C) 2004 Steven Ness
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

#
# Tcl program to take the *.user.xml and *.mtz.xml files,
# the "CRANK XML dataset info" files, and one 
# trial.coords.xml file and create a DM com file
#

#
# Usage:
#  xml2dmcom.tcl user.xml mtz.xml trial.coords.xml
#


# Set CRANK path
set crankbin "$env(CRANK)/bin"
source [file join $crankbin crankutils.tcl]

#
# Output a CRUNCH2 newinput file corresponding to the *.user.xml,
# *.mtz.xml and *.coords.xml XML files read in.  
#
proc OutputDMcomfile { hand } {
	global XMLParse

	puts "\#!/bin/sh"
	puts " "

	set code $XMLParse(dataset_info__code)

	set mtz_in "../$code.in.mtz"
	set mtz_out "dm.out.mtz"

	puts "dm HKLIN $mtz_in HKLOUT $mtz_out << END"

	puts "mode $XMLParse(soap__dm__mode)"
	puts "combine $XMLParse(soap__dm__combine)"
	puts "scheme $XMLParse(soap__dm__scheme)"
	puts "ncycles $XMLParse(soap__dm__ncycles)"
	puts "solc $XMLParse(dataset_info__solvent_content)"
	
	if { [info exists XMLParse(dm__ncsmask)] } {
		puts "ncsmask"
	}

	set in_fp $XMLParse(soap__dm__mtz_in__fp)
	set in_sigfp $XMLParse(soap__dm__mtz_in__sigfp)
	if { $hand == 0 } {
		set in_phib $XMLParse(soap__dm__mtz_in__phib)
	} else {
		set in_phib $XMLParse(soap__dm__mtz_in__phiboh)
	}

	set in_fomo $XMLParse(soap__dm__mtz_in__fomo)
	set in_hla $XMLParse(soap__dm__mtz_in__hla)
	set in_hlb $XMLParse(soap__dm__mtz_in__hlb)
	set in_hlc $XMLParse(soap__dm__mtz_in__hlc)
	set in_hld $XMLParse(soap__dm__mtz_in__hld)

	if { $hand == 0 } {
		set out_fdm $XMLParse(soap__dm__mtz_out__fdm)
		set out_phidm $XMLParse(soap__dm__mtz_out__phidm)
		set out_fomdm $XMLParse(soap__dm__mtz_out__fomdm)
	} else {
		set out_fdm $XMLParse(soap__dm__mtz_out__fdmoh)
		set out_phidm $XMLParse(soap__dm__mtz_out__phidmoh)
		set out_fomdm $XMLParse(soap__dm__mtz_out__fomdmoh)
	}	
	
	puts "LABIN  FP=$in_fp SIGFP=$in_sigfp PHIO=$in_phib FOMO=$in_fomo HLA=$in_hla HLB=$in_hlb HLC=$in_hlc HLD=$in_hld"
	puts "LABOUT  FDM=$out_fdm PHIDM=$out_phidm FOMDM=$out_fomdm"

	puts "END"

}


#
# Main
#
if { [lindex $argv 1] == "" } {
	puts "Usage:"
	puts "  xml2bp3com.tcl input.xml <hand> "
	puts " "
	puts "Where:"
	puts "  input.xml -> input CRANK XML file with dataset info"
	puts "  <hand>    -> 0 for normal hand, 1 for other hand" 
	puts " Copyright/GPL Feb 2004 Steven Ness"
	exit
}

#
# Setup variable names
#
set inputxml [lindex $argv 0]
set hand [lindex $argv 1]

#
# Check to make sure files exist
#
if { [file exists $inputxml] == 0 } {
	crank_error "xml2bp3com.tcl::inputxml file does not exist"
}

#
# Read in XML files
#
XMLParsefile $inputxml

#
# Generate .com file
#
OutputDMcomfile $hand



