#!/usr/bin/env tclsh

#
# Tcl program to take the *.user.xml and *.mtz.xml files,
# the "CRANK XML dataset info" files, and one 
# trial.coords.xml file and create a mlphare com file
#

#
# Usage:
#  xml2mlpharecom.tcl user.xml mtz.xml trial.coords.xml
#

# Set CRANK path
set crankbin "$env(CRANK)/bin"
source [file join $crankbin crankutils.tcl]

#
# Output a CRUNCH2 newinput file corresponding to the *.user.xml,
# *.mtz.xml and *.coords.xml XML files read in.  
#
proc Outputmlphare { } {
	global XMLParse

	# Find maximum occupancy, we will scale everything else by it
	# to normalize them.
	set i 0
	set max_occu 0.0
	while { [info exists XMLParse(coordinate_data__trial__atom__x__$i)] } {
		if { $XMLParse(coordinate_data__trial__atom__occupancy__$i) > $max_occu } {
			set max_occu $XMLParse(coordinate_data__trial__atom__occupancy__$i)
		}
		incr i
	}


	puts "#!/bin/sh"

	puts "set -e"

	set mtz_in "../mtz/input.mtz"
 	set mtz_out "mlphare.out.mtz"
 	set logfile "logfile"

 	puts "mlphare HKLIN $mtz_in HKLOUT $mtz_out << END > $logfile"
 	puts " "

	puts "TITLE 9 refining cycles + 1 phasing cycle"
	puts "CYCLE 100"
	puts "THRES 2.5 0.5"
	puts "PRINT AVE AVF"
	puts "ANGLE 10"

	#
	# Build the LABIN line
	#
	# MLPHARE needs to have FP and SIGFP from the native specified
	#
	# - If SAD - Use single dataset as both FP and FPH
	# - If SIR - Use native
	# - If SIRAS - Use native
	# - If MAD
	#     - If it has a native - Use native
	#     - If no native - Use the following, in this order HREM, LREM, INFL, PEAK
	# 

	#
	# sness - working on this
	#
	#puts "LABIN FP=F SIGFP=SIGF FPH1=F SIGFPH1=SIGF DPH1=DANO SIGDPH1=SIGDANO"
	set labin ""
	set i 0

 	if { [string compare [string tolower $XMLParse(soap__run__experiment)] "SAD"] } {
		set f $XMLParse(soap__run__mtz_in__data__f__0)
		set sigf $XMLParse(soap__run__mtz_in__data__sigf__0)
		set d $XMLParse(soap__run__mtz_in__data__d__0)
		set sigd $XMLParse(soap__run__mtz_in__data__sigd__0)
		puts "LABIN FP=$f SIGFP=$sigf FPH1=$f SIGFPH1=$sigf DPH1=$d SIGDPH1=$sigd"
	}  elseif { [string compare [string tolower $XMLParse(soap__run__experiment)] "SIR"] == 0 } {
 		set f_nat $XMLParse(soap__run__mtz_in__data__f__0)
 		set sigf_nat $XMLParse(soap__run__mtz_in__data__sigf__0)
 		set f_der $XMLParse(soap__run__mtz_in__data__f__1)
 		set sigf_der $XMLParse(soap__run__mtz_in__data__sigf__1)
		if { [info exists XMLParse(soap__run__mtz_in__data__d__1)] } {
 			set d $XMLParse(soap__run__mtz_in__data__d__1)
 			set sigd $XMLParse(soap__run__mtz_in__data__sigd__1)
 			puts "LABIN FP=$f SIGFP=$sigf FPH1=$f SIGFPH1=$sigf DPH1=$d SIGDPH1=$sigd"
 		} else {
 			puts "LABIN FP=$f_nat SIGFP=$sigf_nat FPH1=$f_der SIGFPH1=$sigf_der"
 		}
 	} else {
 		crank_error "crank::xml2mlpharecom.tcl::Unknown experiment type $XMLParse(soap__run_experiment)"
 	}



# 	while { [info exists XMLParse(soap__run__mtz_in__data__type__$i)] } {
# 		if { $labin == "" } {
# 			set labin "FP=$f SIGFP=$sigf"
# 		} else {
# 			set labin "$labin FPH1= SIGFPH1= "
# 		}
# 		incr i
# 	}
# 	puts "LABIN $labin"

	puts "LABOUT FP=FPHASED SIGFP=SIGFPHASED FOM=FOM PHIB=PHIB HLA=HLA HLB=HLB HLC=HLC HLD=HLD"
#	puts "LABOUT ALLIN"
	puts "HLOUT"
	puts ""

	set i 0
	set j 0
	while { [info exists XMLParse(soap__run__mtz_in__data__type__$j)] } {
		set type $XMLParse(soap__run__mtz_in__data__type__$i)
		set high_res $XMLParse(soap__run__mtz_in__data__high_res_limit__$i)
		set low_res $XMLParse(soap__run__mtz_in__data__low_res_limit__$i)
		if { [info exists XMLParse(soap__run__mtz_in__data__fprime__$i)] } {
			set atomname $XMLParse(dataset_info__atom_name)
			set fprime $XMLParse(soap__run__mtz_in__data__fprime__$i)
			set fprimeprime $XMLParse(soap__run__mtz_in__data__fprimeprime__$i)
		}

		set wavelength $XMLParse(soap__run__mtz_in__data__wavelength__$i)
		set atom_name $XMLParse(dataset_info__atom_name)


		if { [info exists XMLParse(soap__run__mtz_in__data__fprime__$i)] } {
			set atomname $XMLParse(dataset_info__atom_name)
			set fprime $XMLParse(soap__run__mtz_in__data__fprime__$i)
			set fprimeprime $XMLParse(soap__run__mtz_in__data__fprimeprime__$i)
#			puts " form $atomname FP=$fprime FPP=$fprimeprime"
		}


		puts " DERIV    [subst $type]_[subst $i]"
		puts " WAVE $wavelength FPR $fprime FDP $fprimeprime"
		puts " DCYCLE PHASE ALL REFCYC ALL KBOV ALL"
		puts " RESO                          $high_res $low_res"
		puts " RESO ANO                      $high_res $low_res"

		set j 0
		while { [info exists XMLParse(coordinate_data__trial__atom__x__$j)] } {

			set mlphare_atom $XMLParse(coordinate_data__trial__atom__element_name__$j)
			set mlphare_occu [expr $XMLParse(coordinate_data__trial__atom__occupancy__$j) / $max_occu]
			set mlphare_x $XMLParse(coordinate_data__trial__atom__x__$j)
			set mlphare_y $XMLParse(coordinate_data__trial__atom__y__$j)
			set mlphare_z $XMLParse(coordinate_data__trial__atom__z__$j)

			set output [format "ATOM   %s %-10.3f%-10.3f%-10.3f %-10.3f %-10.3f BFAC %-10.3f" \
							$mlphare_atom $mlphare_x $mlphare_y $mlphare_z $mlphare_occu $mlphare_occu $XMLParse(dataset_info__bfactor)]
			puts $output

			puts "ATREF AX ALL AY ALL AZ ALL AB ALL"
			puts ""
			incr i
		}
		incr j
	}
	puts "END"

}


#
# Main
#
if { [lindex $argv 2] == "" } {
	puts "Usage:"
	puts "  xml2mlpharecom.tcl user.xml mtz.xml trial.coords.xml"
	puts " "
	puts " Copyright/GPL Feb 2004 Steven Ness"
	exit
}

set xmlfile_user [lindex $argv 0]
set xmlfile_mtz [lindex $argv 1]
set xmlfile_coords [lindex $argv 2]

XMLParsefile $xmlfile_user
XMLParsefile $xmlfile_mtz
XMLParsefile $xmlfile_coords

Outputmlphare
