#!/usr/bin/env tclsh

#
# crank - Automated macromolecular structure solution
#
# Copyright (C) 2004 Steven Ness
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

#
# Tcl program to take a "CRANK XML coordinates file" and
# convert it into a format suitable for refmac5D
#


# Set some paths and source some important files
# so that we can run CCP4i utility commands outside of the CCP4i
# interface

set crankbin "$env(CRANK)/bin"
source [file join $crankbin crankutils.tcl]

#
# Output a CRUNCH2 newinput file corresponding to the 
#  *.user.xml and *.mtz.xml XML files read in.
#
# The business end of this program, the rest of the code 
# mainly just reads in and processes the XML.
#
proc OutputCoordinaten { } {
	global XMLParse
	global XMLWork

	# Loop over all trials
	set curr_trial 0

	# Loop over all atoms in trial
	set i 0
	while { [info exists XMLParse(coordinate_data__atom__atom_id__$i)] } {
		set output [format "HEAVY %-10s%10.4f%10.4f%10.4f%10.2f%10.2f" \
						$XMLParse(coordinate_data__atom__element_name__$i) \
						$XMLParse(coordinate_data__atom__x__$i) \
						$XMLParse(coordinate_data__atom__y__$i) \
						$XMLParse(coordinate_data__atom__z__$i) \
						$XMLParse(coordinate_data__atom__occupancy__$i) \
						$XMLParse(coordinate_data__atom__bfactor__$i)]
		puts $output
		incr i
	}
}



#
# Main
#
if { [lindex $argv 0] == "" } {
	puts "Usage:"
	puts "  xmlcoords2refmac5d.tcl coords.xml"
	puts " "
	puts " coords.xml file must contain only one trial"
	puts " "
	puts " Copyright/GPL Feb 2004 Steven Ness"
	exit
}

set xmlfile [lindex $argv 0]

XMLParsefile $xmlfile

# set allvars [array get XMLParse]
# for { set i 0 } { $i < [llength $allvars] } { incr i } {
# 	puts -nonewline "[lindex $allvars $i]\t"
# 	incr i
# 	puts "[lindex $allvars $i] "
#}

OutputCoordinaten

