# Copyright (C) Navraj S. Pannu 2006-2007, Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

proc buccaneer_run { code crankbin executable crankplugins inputxml inccp4 step } {
    global XMLParse
    global env

    set version "121"
    #
    if { [info exists XMLParse([join "crank parameters version" __])] } {
       if { $version != $XMLParse([join "crank parameters version" __]) } {
          crank_error "crank_buccaneer.tcl::plugin and crank XML version do not match"
      }
    } else {
       crank_error "crank_buccaneer.tcl::crank XML version info does not exist"
    }

    crank_run_message "Running $step-buccaneer"
    
    file mkdir $step-buccaneer
    cd $step-buccaneer
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]
    
    if { [file exists [file join $env(CLIBD) reference_structures reference-1ajr.mtz]] } {
	set ref [file join $env(CLIBD) reference_structures] 
    } else {
	crank_error "crank::runbuccaneer:: no reference-1ajr.mtz found in subdirectory reference_structures of $CLIBD"
    } 

    if { [info exists XMLParse([join "crank parameters sequence" __])] } {
	set seq [open input.seq w+]
        regsub -all {[^a-zA-Z]} $XMLParse([join "crank parameters sequence" __]) "" sequence
        for {set i 0 } { $i < [string length $sequence] } { incr i 60 } {
            if { [expr $i + 59] <  [string length $sequence] } {
                puts $seq "[string range $sequence $i [expr $i + 59] ]"
            } else {
                puts $seq "[string range $sequence $i  end]"
            }
        }
	flush $seq
	close $seq
    }
    
    set ex [file join $env(CBIN) cbuccaneer]

    runcommand [concat tclsh [file join $crankplugins buccaneer xml2buccaneercom.tcl] $inputxml $crankbin $ref $ex >& buccaneer-command.com]
    runcommand [concat tclsh [file join $crankplugins buccaneer xml2buccaneertcl.tcl] $inputxml $crankbin $ref $ex >& buccaneer-command.tcl]

    runcommand [concat tclsh buccaneer-command.tcl]

    if { [file exists buccaneer-logfile] } {
	set log [open "buccaneer-logfile" r]
	set output [read $log]
	puts $output
	close $log
	file copy buccaneer-logfile [file join .. log $step-buccaneer-logfile]
    } else {
	crank_error "crank_buccaneer.tcl::buccaneer did not finish successfully"
    }

    if { [file exists temporary_refmac.pdb] } {
	file copy temporary_refmac.pdb [file join .. workdb crank.out.$tag.pdb]
 	file delete temporary_refmac.pdb
    } else {
	crank_error "crank::runbuccaneer:: temporary_refmac.pdb file not present.  Exitting."
    }

     if { [file exists temporary_refmac.mtz] } {
	 runcommand [concat tclsh [file join $crankplugins buccaneer crank-rename-buccaneer-output.tcl] $inputxml $crankbin temporary_refmac.mtz [file join .. workdb crank.out.$tag.mtz] ]
#	file delete temporary_refmac.mtz
     } 

     if { [file exists temporary_buccaneer.pdb] } {
 	file delete temporary_buccaneer.pdb
     } 

     if { [file exists buccaneer.out.pdb] } {
 	file delete buccaneer.out.pdb 
     } 

    cd ..
}

proc buccaneer_reference { } {
    global XMLParse
    
    puts "Buccaneer:"
    puts "Cowtan, K. (2006) The Buccaneer software for automated model building. 1."
    puts "Tracing protein chains. Acta Cryst. D62, 1002-1011.\n"
} 
