#
# Copyright (C) Navraj S. Pannu 2006-2007, Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

proc check_run { code crankbin executable crankplugins inputxml inccp4 step } {
    global XMLParse

    set version "121"
    #
    if { [info exists XMLParse([join "crank parameters version" __])] } {
       if { $version != $XMLParse([join "crank parameters version" __]) } {
          crank_error "crank_check.tcl::plugin and crank XML version do not match"
      }
    } else {
       crank_error "crank_check.tcl::crank XML version info does not exist"
    }
    #
    # Run CHECK for model verification
    #
    crank_run_message "Running $step-check"
    
    file mkdir $step-check
    cd $step-check

    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

    if { [info exists XMLParse([join "crank soap run step=$step check final_pdb" __])] } {
	if { [file exists $XMLParse([join "crank soap run step=$step check final_pdb" __])] } {
	    set final_pdb $XMLParse([join "crank soap run step=$step check final_pdb" __])
	} else {
	    crank_error "Defined final pdb does not exist"
	}
	set finalpdb 1
    } else {
	set finalpdb 0
    }

    if { [info exists XMLParse([join "crank soap run step=$step check no_trans" __])] } {
	set no_trans $XMLParse([join "crank soap run step=$step check no_trans" __])
    } else {
 	set no_trans 0
    }

    for { set istep 1 } { [info exists XMLParse(crank__soap__run__step=$istep)] } { incr istep } {
	if { [info exists XMLParse([join "crank soap run step=$istep name" __])] } {

	    if { $XMLParse([join "crank soap run step=$istep name" __]) == "BP3" } {
		set bp3step $istep
		set bp3tag [string trim $XMLParse([join "crank soap run step=$istep tag" __])]
	    }

	    if { $XMLParse([join "crank soap run step=$istep name" __]) == "REFMAC" } {
		set refmacstep $istep
		set refmactag [string trim $XMLParse([join "crank soap run step=$istep tag" __])]
	    }

	    if { $XMLParse([join "crank soap run step=$istep name" __]) == "SOLOMON" } {
		set solstep $istep
		set soltag [string trim $XMLParse([join "crank soap run step=$istep tag" __])]
	    }
	}
    }

    set inphases 0
    if { [file exists [file join .. workdb crank.in.$tag.mtz] ] } {
	if { [info exists XMLParse([join "crank soap run step=$step input phase_columns f" __])] } {
	    set in_f $XMLParse([join "crank soap run step=$step input phase_columns f" __])
	    set in_sigf $XMLParse([join "crank soap run step=$step input phase_columns sigf" __])
	    set in_phib $XMLParse([join "crank soap run step=$step input phase_columns phib" __])
	    set in_fom $XMLParse([join "crank soap run step=$step input phase_columns fom" __])
	    set inphases 1
        } 
    } 

    if { $inphases && $finalpdb } {
       set sfall_script "mode sfcalc xyzin hklin\nlabin FP = $in_f SIGFP = $in_sigf\n"
       set sfall_script "$sfall_script labout FC=FC PHIC=PHIC\n"
       set command "sfall XYZIN $final_pdb HKLIN [file join .. workdb crank.in.$tag.mtz] HKLOUT final1.mtz << \"$sfall_script\" "
       catch {eval exec $command } output

       set sftools_script "read final1.mtz\nwrite final.mtz col FC PHIC\nquit\n"
       set command "sftools << \"$sftools_script\" "
       catch {eval exec $command } output
       file delete final1.mtz
    }

    if { ([info exists bp3tag] && [file exists final.mtz]) } {
	runcommand [concat tclsh [file join $crankplugins check phasecompare.tcl] "final.mtz" "FC" "PHIC" [file join .. workdb crank.out.$bp3tag.mtz] "${bp3tag}_F" "${bp3tag}_SIGF" "${bp3tag}_PHIB" "${bp3tag}_FOM" "bp3" $no_trans] 
	runcommand [concat sh cphasematch-bp3.com]
	file delete temp.mtz
    }


    if { ([info exists refmactag] && [file exists final.mtz]) } {
 	runcommand [concat tclsh [file join $crankplugins check phasecompare.tcl] "final.mtz" "FC" "PHIC" [file join .. workdb crank.out.$refmactag.mtz] "${refmactag}_F" "${refmactag}_SIGF" "${refmactag}_PHIB" "${refmactag}_FOM" "refmac" $no_trans]
	runcommand [concat sh cphasematch-refmac.com]
	file delete temp.mtz
    }

    if { ([info exists soltag] && [file exists final.mtz]) } {
 	runcommand [concat tclsh [file join $crankplugins check phasecompare.tcl] "final.mtz" "FC" "PHIC" [file join .. workdb crank.out.$soltag.mtz] "${soltag}_F" "${soltag}_SIGF" "${soltag}_PHIB" "${soltag}_FOM" "solomon" $no_trans]
	runcommand [concat sh cphasematch-solomon.com]
	file delete temp.mtz
    }

    if { $inphases && [file exists final.mtz] } {
       runcommand [concat tclsh [file join $crankplugins check phasecompare.tcl] "final.mtz" "FC" "PHIC" [file join .. workdb crank.in.$tag.mtz] $in_f $in_sigf $in_phib $in_fom "final" $no_trans]
       runcommand [concat sh cphasematch-final.com]
       file delete temp.mtz
    }

    # this is for comparing pdb's
    if { [info exists XMLParse([join "crank soap run step=$step input pdb" __])] } {
        set pdb_tag $XMLParse([join "crank soap run step=$step input pdb" __])

        if { [file exists [file join .. workdb crank.out.$pdb_tag.pdb] ] } {
            set crankpdb [file join .. workdb crank.out.$pdb_tag.pdb]
	    set log [open "check-logfile" r]
	    set data [read $log]
	    set data [split $data "\n"]

	    set x ""
	    set y ""
	    set z ""
            set tot 0

	    foreach line $data {
		if { [regexp {.*Change or origin\: uvw = \( *([-0-9.]*), *([-0-9.]*), *([-0-9.]*).*} $line junk x y z] } {
		   incr tot
 		}
	    }
	    close $log

 	    if { $no_trans == 0} {
		if { ($x != "") && ($y != "") && ($z != "") } {
		    set lsqkab_script "ROTAT EULER 0 0 0\nTRAN FRAC $x $y $z\n"
		    set command "lsqkab XYZIN2 $crankpdb XYZOUT rotated.pdb << \"$lsqkab_script\" "
		    catch {eval exec $command } output
		}
	    } else {
		file copy $crankpdb rotated.pdb
	    }

	    if { [file exists rotated.pdb] } { 
		runcommand [concat tclsh [file join $crankplugins check compare-pdbs.tcl] $final_pdb rotated.pdb $crankplugins >> check-logfile]  
	    }
	} 
    }
    
    if { [info exists XMLParse([join "crank soap run step=$step check final_substructure" __])] } {
	if { [file exists $XMLParse([join "crank soap run step=$step check final_substructure" __])] } {
	    set final_sub $XMLParse([join "crank soap run step=$step check final_substructure" __])
	} else {
	    crank_error "Defined final substructure does not exist"
	}
	# see if sitcom exists
#        set command "sitcom -help"
#        catch {eval exec $command } output
#        set err [lindex $::errorCode 0]

        if { [info exists XMLParse([join "crank soap run step=$step input substructure" __])] } {
# &&
#             ([string compare $err "POSIX"] == 1) } {
            set sub_tag $XMLParse([join "crank soap run step=$step input substructure" __])
            set model_sub [file join .. workdb crank.out.$sub_tag.substructure.pdb]

            runcommand [concat sitcom $model_sub $final_sub -sg $XMLParse([join "crank parameters input_spacegroup number" __]) >> check-logfile]
        }
    } 

    if { [file exists final.mtz] } {
	file delete final.mtz
    }

    if { [file exists check-logfile] } {
	file copy check-logfile [file join .. log $step-check-logfile]
	runcommand [concat cat check-logfile]
    } else {
        crank_error "check-logfile not created"
    }

    cd ..
}

proc check_reference { } {
    global XMLParse
    
    puts "CHECK:"
    puts "Cowtan, K: Clipper utilities & Ness, S.R. & Skubak, P: Unpublished"
} 
