#
# Copyright (C) Navraj S. Pannu and Steven Ness 2004-2005, Leiden University
# Copyright (C) Navraj S. Pannu 2006-2007, Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

proc shelxd_run { code crankbin executable crankplugins inputxml inccp4 step } {
    global XMLParse

    set version "121"
    #
    if { [info exists XMLParse([join "crank parameters version" __])] } {
       if { $version != $XMLParse([join "crank parameters version" __]) } {
          crank_error "crank_shelxd.tcl::plugin and crank XML version do not match"
      }
    } else {
       crank_error "crank_shelxd.tcl::crank XML version info does not exist"
    }

    crank_run_message "$step-shelxd"

    file mkdir $step-shelxd
    cd $step-shelxd
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

    set inputmtz [file join .. workdb crank.in.$tag.mtz]

    set executable [file join $crankplugins shelxd $executable]

    set fa $XMLParse([join "crank soap run step=$step input fa_columns fa" __])
    set sigfa $XMLParse([join "crank soap run step=$step input fa_columns sigfa" __])
     set alpha $XMLParse([join "crank soap run step=$step input alpha_columns alpha" __])
    runcommand [concat tclsh [file join $crankplugins shelxd mtz2hkl_fa.tcl] $inputmtz crank_fa.hkl $fa $sigfa $alpha]

    # Run SHELXC to generate SHELXD command file 
    runcommand [concat tclsh [file join $crankplugins shelxd xml2shelxccom.tcl] $inputxml $crankbin >& shelxc-command.com]
    runcommand [concat tclsh [file join $crankplugins shelxd xml2shelxctcl.tcl] $inputxml $crankbin >& shelxc-command.tcl]

    runcommand [concat tclsh shelxc-command.tcl]

    # Delete the crank.hkl file that was just created, since 
    # it could mislead SHELXD
    file delete crank.hkl
    file delete crank_fa.hkl

    set fa $XMLParse([join "crank soap run step=$step input fa_columns fa" __])
    set sigfa $XMLParse([join "crank soap run step=$step input fa_columns sigfa" __])
    set alpha $XMLParse([join "crank soap run step=$step input alpha_columns alpha" __])
    runcommand [concat tclsh [file join $crankplugins shelxd mtz2hkl_fa.tcl] $inputmtz crank_fa.hkl $fa $sigfa $alpha]

    set ccweak_threshold $XMLParse([join "crank soap run step=$step shelx ccweak_threshold" __])

    set min_trials $XMLParse([join "crank soap run step=$step shelx min_trials" __])

    set max_trials $XMLParse([join "crank soap run step=$step shelx ntry" __])

    runcommand [concat tclsh $executable $ccweak_threshold $min_trials $max_trials >& shelxd-logfile]
    
    check_atom crank_fa.pdb crank.out.$tag.substructure.pdb

    file rename crank.out.$tag.substructure.pdb [file join .. workdb crank.out.$tag.substructure.pdb]

    if { [file exists shelxd-logfile] } {
	set log [open "shelxd-logfile" r]
	set output [read $log]
	puts $output
	close $log
	file copy shelxd-logfile [file join .. log $step-shelxd-logfile]
    } else {
	crank_error "crank_shelxd.tcl::shelxd did not finish successfully"
    }

	
    cd ..
}


proc shelxd_reference { } {

	puts "SHELXD:"
	puts "Uson, I. and Sheldrick, G.M. (1999) Advances in direct methods for"
	puts "protein crystallography. Curr. Opin. Struct. Biol. 9(5), 643-648.\n"
}
