#
#     Copyright (C) 1999-2004  Liz Potterton, Peter Briggs
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Library.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
##################################################################
# CCP4Interface  database_browser.tcl
#
#
# create the database interface window 
#
# Liz Potterton Jan97
##################################################################
#----------------------------------------------------------------------
proc db_change_db0 { } {
#----------------------------------------------------------------------

# User has clicked 'secret' key to swap databases (probably F3)
# as defined in browser_utils.tcl
# Flip between regular project database and bugbase

  global archive
  global database
  global directories

  TkBusy . 

  set archive(CURRENT_SELECTION) ""
  if { $archive(CURRENT_DATABASE) == "database" } { 
    set archive(CURRENT_DATABASE) bugbase
    if { [file exists $directories(BUGBASE) ] } {
      set directories(BUGBASE_STATUS) \
         [DbCreateLockFile $directories(BUGBASE)]
      PleaseWait "..reading bugbase file"
      ReadDef $directories(BUGBASE) bugbase CCP4_Bug_Database
      PleaseWait
      Report 2 "Reading bug database $directories(BUGBASE)"
    } else {
      set directories(BUGBASE_STATUS) 0
    }
  } else { 
    if { $directories(BUGBASE_STATUS) == 1 } {
      DeleteFile [DbLockFileName $directories(BUGBASE)]
      set directories(BUGBASE_STATUS) 0
    }
    set archive(CURRENT_DATABASE) database
  }

  DbUpdateList
  DbUpdateOptions
  TkBusy .  1
}

#------------------------------------------------------------------
proc db_open_bug {job_idVar} {
#------------------------------------------------------------------

# Create a new object in the bug database and then open
# window for user to fill out details

  global bugbase
  global directories

  upvar $job_idVar job_id

  if { ![file exists $directories(BUGBASE) ] } {
    WarningMessage "You must open bugbase file before entering bug report"
    return 0
  }
    
  if { [ElementExists bugbase NJOBS] == 0 } {
    set bugbase(NJOBS) 0
  }

  incr bugbase(NJOBS)
  set n $bugbase(NJOBS)

  set bugbase(JOB_ID,$n) $n
  set bugbase(DATE,$n) [GetDate ]
  set bugbase(VERSION,$n) [GetVersion]
  set bugbase(REPORTER,$n) [GetUserId]
  set bugbase(STATUS,$n) "open"
  set bugbase(PRIORITY,$n) "regular"
  set bugbase(MODULE,$n) ""
  set bugbase(KEYWORDS,$n) ""
  set bugbase(DESCRIPTION,$n) ""
  set bugbase(FIX_VERSION,$n) ""
  set bugbase(FIX_DATE,$n) ""
  set bugbase(RESPONSE,$n) ""

  set job_id $n

#  puts "db_open_bug NJOBS $bugbase(NJOBS)"

}

#---------------------------------------------------------------
proc db_edit_bug { mode job_id } {
#---------------------------------------------------------------

# Window for user to report bug or for developer to respond

  global bugbase

  set w .w_bugbase_$job_id

  if { ![OpenWindow $w "Bugbase#$job_id" "Bugbase" \
             -help [SearchPath HELP general  bugbase.html ] ] } { return }

  CreateFrame  $w bugbase

  CreateButton $w dismiss Quit \
	"db_handle_bug_edit $mode quit $job_id
                destroy $w"

  CreateButton $w action "Apply&Exit" \
	"db_handle_bug_edit $mode apply $job_id
                destroy $w"

  if { [ElementExists bugbase [Indxv REPORTER $job_id] ] } {
    set reporter $bugbase(REPORTER,$job_id)
  } else {
    set reporter ""
  }

  CreateLine line \
    label "Bug# $bugbase(JOB_ID,$job_id)  " \
    label "Version: $bugbase(VERSION,$job_id) " \
    label "Reported by: $reporter" \
    label "Date: $bugbase(DATE,$job_id)" 

  CreateLine line \
    message "Set the priority for fixing the bug" \
    label "Priority:" \
    widget PRIORITY,$job_id \
    message "Enter name of task window or general utility" \
    label "In task/utility" \
    widget MODULE,$job_id \
    message "Enter keywords or short description" \
    label "keywords" \
    widget KEYWORDS,$job_id \
	-width 40

  CreateLine line \
    label "Description:"

  CreateText textframe {} -scroll \
       -arguments "-height 15 -width 60 -relief sunken"

  if { $bugbase(DESCRIPTION,$job_id) != "" } {
    $textframe insert end $bugbase(DESCRIPTION,$job_id)
  }

  puts "BUG_ENTRY_FIELD $textframe"
  SetSystemVar BUG_ENTRY_FIELD $textframe
  SetSystemVar BUG_STATUS $bugbase(STATUS,$job_id)

  if { $mode == "edit" } {
    CreateLine line \
      label "Status:" \
      widget STATUS,$job_id \
	-command "db_response_details $job_id" \
      label "on date" \
      widget FIX_DATE,$job_id \
      label "in version" \
      widget FIX_VERSION,$job_id \
	-width 15 

  CreateLine line \
    label "Developer's response:"

    CreateText textframe {} -scroll \
         -arguments "-height 15 -width 60 -relief sunken"

    if { [ElementExists bugbase RESPONSE,$job_id ] &&
                  $bugbase(RESPONSE,$job_id) != "" } {
      $textframe insert end $bugbase(RESPONSE,$job_id)
    }

    puts "BUG_RESPONSE_FIELD $textframe"
    SetSystemVar BUG_RESPONSE_FIELD $textframe
  }


  update_main_scroll $w
}

#----------------------------------------------------------------
proc db_response_details { job_id } {
#----------------------------------------------------------------

  global bugbase

  if { $bugbase(STATUS,$job_id) != [GetSystemVar BUG_STATUS] } {
    set bugbase(FIX_DATE,$job_id) [GetDate]
    if { $bugbase(STATUS,$job_id) == "fixed" } {
      set bugbase(FIX_VERSION,$job_id) [GetVersion]
    }
  }
}


#----------------------------------------------------------------
proc db_handle_bug_edit { edit_mode exit_mode job_id } {
#----------------------------------------------------------------

# Handle user quit or save&exit from bugbase report window
# Get text fromtext imput frame and save to bugbase 

  global directories
  global bugbase

  if { [StringSame $exit_mode "quit" ] } {
    if { [file exists $directories(BUGBASE) ] } {
      ReadDef $directories(BUGBASE) bugbase CCP4_Bug_Database
    }
    return
  }
 
  set entry [GetSystemVar BUG_ENTRY_FIELD]
  set response [GetSystemVar BUG_RESPONSE_FIELD]

  set bugbase(DESCRIPTION,$job_id) [remove_quotes [$entry get 1.0 end] ]

  if {$edit_mode == "edit" } {
    set bugbase(RESPONSE,$job_id) [ remove_quotes [$response get 1.0 end] ]
  }

  db_save_bugbase
  DbUpdateList
}

#-----------------------------------------------------------------------
proc remove_quotes { input } {
#-----------------------------------------------------------------------
  regsub -all {\"} $input ' output
  return $output
}

#-----------------------------------------------------------------
proc db_open_bugbase { } {
#-----------------------------------------------------------------

# Prompt user for name of bug database file 
# If there is currently a file open then save current status
# to that file and then open new file

  global directories

  if { ![SelectFile file -title "Bug Database File" -unknown -filter "*.def" ] } {
  return }

  if { $directories(BUGBASE) != "" && 
       [file exists $directories(BUGBASE) ]  } {
    db_save_bugbase
  }

  set directories(BUGBASE_STATUS) [DbCreateLockFile $file ]
  if { $directories(BUGBASE_STATUS) == 0 } { return 0 }

  if { [file exists [lindex $file 0] ] } {
    ReadDef [lindex $file 0] bugbase CCP4_Bug_Database
  } else { 
    DbCreateNewFile [lindex $file 0] bugbase "CCP4_Bug_Database"
  }
  set directories(BUGBASE) [lindex $file 0]
  SavePreferences directories directories
  DbUpdateList
}

#---------------------------------------------------------------
proc db_save_bugbase { } {
#---------------------------------------------------------------
  global directories

  if { $directories(BUGBASE) == "" ||
       ![file exists $directories(BUGBASE) ]  } {
    if { ![SelectFile file -title "Bug Database File" -unknown -filter "*.def" ] } { return }
    set directories(BUGBASE) [lindex $file 0]
  }

  if { [DbLockFileStatus $directories(BUGBASE) user time] == 0 || \
    $user == [GetUserId] } {
    PleaseWait "..saving bug database"
    SaveArray "CCP4_Bug_Database" $directories(BUGBASE) bugbase \
		-silent -save_types
    PleaseWait
  } else {
    WarningMessage "Nothing saved to $directories(BUGBASE)
File locked by $user"
  }

}


#-------------------------------------------------------------------------
proc ReadDef { filn arrayname taskname args } {
#-------------------------------------------------------------------------
  upvar #0 $arrayname array

# Initialise an array from a def file 
# Create a parameter PARAM_LIST which is list of names of all input 
# parameters - this list usaully used to write out the same file

  if { [catch {open $filn r }  f ] } {

    puts stderr "Could not open initialisation file $filn"
    return 0
  }

  if { ![ElementExists $arrayname PARAM_LIST] } {
    set array(PARAM_LIST) ""
  }

  while { [gets $f line] >= 0 } {
#    puts "line $line"
    if { [string length $line ] > 0 &&
         [string range $line 0 0 ] != "#" } {
       set line [string trimright $line]
       set element [nextword line continue pare]
       set value [nextword line continue pare]
       GetIndx $element root c1 c2

# If the next word begins with an underscore then it is a type for the
# data

       if { [ string index $value  0 ] == "_" } {
          set array(_$element) $value
          set value [nextword line continue pare]
       }

# If nextword returns continue = 1 then it has read opening but not
# closing inverted commas and we need to look whats on the next line 

	while { $continue == 1 } {
          gets $f lineext
          set line ""; append line "\"" "$value" "
" "$lineext"
          set value [nextword line continue pare]
        }

       if { [string length $element] > 0 } {
            set array($element) $value
            if { $c1 == "" || $c1 == 0 } {
              if { [lsearch $array(PARAM_LIST) $element ] < 0 } {
                lappend array(PARAM_LIST) $element
              }
            }
       }
    }
  }
#  puts " $arrayname PARAM_LIST $array(PARAM_LIST) "
  close $f
  return 1
}



#--------------------------------------------------------------------
proc nextword { linein  continueVar {mode " "}} {
#--------------------------------------------------------------------
#d_sum Read the next word from input text -  handle quotes
#d_desc Could this be done more efficiently with lindex?
#d_arg linein Input line of text
#d_arg continueVar Return 0=reached end of line, 1=line continues after this word

  upvar $linein line
  upvar $continueVar continue

  set continue 0
  set len ""
  set linetmp [string trimleft $line]
  if { [string length $linetmp] <= 0 ||
       [regexp {"\" \""|"\"\""} $linetmp ] == 1 } {return ""}

# The next word or phrase starts with quotes - take everything between these
# and the next quotes

  if { [ regsub -- {^\"} $linetmp "" word1 ] } {
    regexp -- {([^\"]*)\"} $word1 word
    set continue [ expr {1 - [regsub -- {\"$} $word1 "" word]} ]
#    if { $continue == 1 } { puts "word1 $word1" }
  } else {
    scan $linetmp %s word
  }

  if { $mode == "pare" } {
    set line [string trimleft \
	     [string range $linetmp [expr {[string length $word] + 1} ] end] ]
  }
    return $word

}
