C
C     GETAX
C     Copyright (C) 1996 Clemens Vonrhein
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C**************************************************************
C
C
C**************************************************************
      program getax
c
c     'getax' is a program to search for your non-crystallographic
c     symmetry if a first map is available. The only knowledge you
c     need is a selfrotation solution ( e.g. POLARRFN ) and a crude
c     knowledge of the size/shape of your molecules.
c
c     GETAX v1.0 22. April 1996  (c) Clemens Vonrhein
c     vonrhein@bio5.chemie.uni-freiburg.de
c
c     v1.1   14. May 1996: removed bug for non-orthogonal cell
c
c     v1.2   28. May 1996: support for two-fold perpendicular to axis
c
c     v1.2.1 28. May 1996: support for o/p CC*(1.0-sd(CC)/CC) MAPOUT
c                          support for XYZIN
c
c     v1.2.2 20. August 1996: just some cleanup
c
c     v1.3.0 08. October 1996: now checking on packing can be done
c                              some more cleanup
c
c v1.4.0 23. October 1996:
c - introduced stick-weighting (popular demand,
c   since experience showed: tubes of connected density in o/p
c   correlation map are a good hint of correct solution)
c - corrected bug for non-orthogonal spacegroups,
c   where c* .ne. c
c - some major cleanup
c - documentation NOT updated yet
c
c v1.5.0 24. October 1996:
c - cleanup
c - dynamic memory allocation via CCPALC
c - updated documentation
c
c v2.0   15. November 1996:
c - cleanup
c - removed bug, where solutions where misset by one grid point
c
c v2.1   06. December 1996
c - removed bug for AX4
c
c v2.2   13. January 1997
c - removed bug(s) for orthogonalization and m2p.f
c
c v2.3   16. Juni 1997
c - minor cleanup for CCP4 distribution
c
c v2.4   17. July 1997
c - added o/p of rotation matrix + translation for highest peaks
c   (can be used directly in DM)
c
c - changed GXYZ from fractional coordinates to orthogonal coordinates
c   after interpolation
c - changed calls to CVFRAC to CVFRAC2
c
c v2.5   1. April 1998
c - changed generation of axis stick: can now handle P1 properly, since
c   generation is independent of grid/orthogonalization
c - small bug in using Dn pointgroup
c================================================================
c Conditions of use:
c - No parts of this software can be incorporated into any other program
c   without my explicit permission.
c - all parts of this software toghether should be kept together
c - any modifications and/or bug fixes should be forwarded to
c   vonrhein@bio5.chemie.uni-freiburg.de
c - a paper has been submitted, so any application of GETAX should
c   please cite:
c
c    Vonrhein, C. & Schulz, G. E. (1999). Locating proper
c    non-crystallographic symmetry in low-resolution electron
c    density maps with program GETAX. Acta Cryst. D55, 225 - 229
c
c================================================================
c
c this is just a small main program to call the real thing
c and have all the arrays dimensioned propperly
c
      implicit none
c
      character prgnam*6,prgdat*28,prgver*5,types*1
      integer ngrid(3),iout,asulim(2,3),length,naloc,secgrid,asugrid
     +  ,arrgrid
c
      parameter (iout=6)
      parameter (prgnam='GETAX')
      parameter (prgdat='$Date: 2005/09/06 11:25:14 $')
      parameter (prgver='2.5')
      parameter (naloc=6)
c
      dimension types(naloc),length(naloc)
c
      external getaxpremain,ccpfyp,ccprcs,ccpalc
c
      common /asymm/ asulim,ngrid
      common /alloc/ asugrid,secgrid
c
      data types /naloc*'R'/
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      call ccpfyp
      call ccprcs(iout,prgnam,prgdat)
      write (iout,10) prgnam,prgver,
     +  ' Vonrhein, C. & Schulz, G. E. (1999).',
     +  ' Locating proper non-crystallographic symmetry in ',
     +  ' low-resolution electron density maps with program GETAX.',
     +  ' Acta Cryst. D55, 225 - 229'
 10   format (1x,a6,1x,a5,//,1x,a,/,3x,a,/,3x,a,/,3x,a,/)
c
c read map header to allocate sufficient memory
c
      call rmaphd(secgrid)
      arrgrid=ngrid(1)*ngrid(2)*ngrid(3)
      asugrid=(asulim(2,1)-asulim(1,1)+1)*(asulim(2,2)-asulim(1,2)+1)
     +  *(asulim(2,3)-asulim(1,3)+1)
c
      write(iout,'(/,a,/,a,i15)') ' Memory allocation for:'
     +  ,' 3D-Map array (unit cell)       = ',arrgrid
      write(iout,'(a,i15)') ' 1D-Section array               = ',secgrid
      write(iout,'(a,i15,//)') ' 3D-Map array (asymmetric unit) = '
     +  ,asugrid
c
c 3D-Map arrays (unit cell)
      length(1)=arrgrid
c change the following line if you want WMAPOUT feature:
c      length(2)=arrgrid
      length(2)=1
c
      length(3)=arrgrid
c 2D-Section array
c change the following line if you want WMAPOUT feature:
c      length(4)=secgrid
      length(4)=1
c
      length(5)=secgrid
c 3D-Map arrays (asymmetric unit)
      length(6)=asugrid
c
      call ccpalc(getaxpremain,naloc,types,length)
c
      end
      subroutine rmaphd(secgrid)
c
      implicit none
c      
      character title*80
      integer lmode,ifail,iprint,nmin(3),nmax(3),ngrid(3),nsec,iuvw(3)
     +  ,mxyz(3),nw1,nu1,nu2,nv1,nv2,lspgrp,asulim(2,3),nsym,iin,iout,i
     +  ,i1,i2,j,secgrid
      real cell(6),rhmin,rhmax,rhmean,rhrms,gtor(3),rtog(3),xyzlim(2,3)
     +  ,rot(4,4,96)
      logical lerr
c
      external mrdhds,setlim,msyget,ccperr,rbrorf
c
      parameter (iin=10)
      parameter (iout=6)
c
      common /mapin/ nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2,lspgrp
      common /cel/ cell
      common /mapinf/ rhmin,rhmax,rhmean,rhrms,gtor,rtog
      common /packit/ nsym,rot
      common /asymm/ asulim,ngrid
c
      write(iout,'(/,a,/,a)') ' s/r RMAPHD:'
     +  ,' read map header to get memory allocation requirements'
c
      ifail=0
      iprint=1
c
      call mrdhds(iin,'MAPIN',title,nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2
     +  ,cell,lspgrp,lmode,rhmin,rhmax,rhmean,rhrms,ifail,iprint)
c
      secgrid=(nu2-nu1+1)*(nv2-nv1+1)
c
      call setlim(lspgrp,xyzlim)
      write(iout,'(a)') ' Asymmetric unit = '
      write(iout,'(a,6f6.3)') '      fractional = ',((xyzlim(i,j),i=1,2)
     +  ,j=1,3)
      do i=1,3
        asulim(1,i)=nint(xyzlim(1,i)*float(mxyz(i)))
        if (asulim(1,i).eq.mxyz(i)) asulim(1,i)=asulim(1,i)-1
        asulim(2,i)=nint(xyzlim(2,i)*float(mxyz(i)))
        if (asulim(2,i).eq.mxyz(i)) asulim(2,i)=asulim(2,i)-1
      enddo
      write(iout,'(a,6i6)') '      grid units = ',((asulim(i,j),i=1,2)
     +  ,j=1,3)
c
      call msyget(99,lspgrp,nsym,rot)
      write(iout,'(/,a,i4)') ' Laue Spacegroup = ',lspgrp
      do i=1,nsym
        write(iout,'(//,a,i2)') ' Symmetry #',i
        write(iout,'(3f10.6)') ((rot(i1,i2,i),i2=1,3),i1=1,3)
        write(iout,'(/,3f10.6)') (rot(i1,4,i),i1=1,3)
      enddo
c 
      if (lmode.ne.2) call ccperr(1,' MAPIN not lmode=2')
      if ((iuvw(1).ne.1).or.(iuvw(2).ne.2).or.(iuvw(3).ne.3)) call
     +  ccperr(1,' change AXIS order to X Y Z')
c
      do i=1,3
        ngrid(i)=mxyz(i)
        gtor(i)=cell(i)/ngrid(i)
        rtog(i)=1./gtor(i)
      enddo
c
      if (iuvw(1).eq.1) then
        nmin(1)=nu1
        nmax(1)=nu2
      endif
      if (iuvw(1).eq.2) then
        nmin(2)=nu1
        nmax(2)=nu2
      endif
      if (iuvw(1).eq.3) then
        nmin(3)=nu1
        nmax(3)=nu2
      endif
      if (iuvw(2).eq.1) then
        nmin(1)=nv1
        nmax(1)=nv2
      endif
      if (iuvw(2).eq.2) then
        nmin(2)=nv1
        nmax(2)=nv2
      endif
      if (iuvw(2).eq.3) then
        nmin(3)=nv1
        nmax(3)=nv2
      endif
      if (iuvw(3).eq.1) then
        nmin(1)=nw1
        nmax(1)=nw1+nsec-1
      endif
      if (iuvw(3).eq.2) then
        nmin(2)=nw1
        nmax(2)=nw1+nsec-1
      endif
      if (iuvw(3).eq.3) then
        nmin(3)=nw1
        nmax(3)=nw1+nsec-1
      endif
      lerr=.false.
      do i=1,3
        if (nmin(i).ne.0) lerr=.true.
        if (nmax(i).ne.(ngrid(i)-1)) lerr=.true.
      enddo
      if (lerr) then
        call ccperr(1,' MAPIN not covering EXACTLY whole unit cell')
      endif
c
      return
      end
      subroutine ortmat(cell,ngrid,ro,rf,rotog,rgtoo,ncode)
c
c  Return orthogonalization matrix 
c
c  On entry:
c     ncode      orthogonalization code
c           = 1  axes along a, c* x a, c*  (Brookhaven standard, default)
c           = 2  axes along b, a* x b, a*
c           = 3  axes along c, b* x c, b*
c           = 4  axes along a+b, c* x (a+b), c*
c           = 5  axes along a*, c x a*, c       ( Rollett )
c           = 6  axes along a, b*, a x b*
c           = 7  axes along a*, b, a* x b   (TNT convention)
c     cell(6)    cell dimensions
c     ngrid(3)   grid dimensions along a,b,c
c
c  On exit:
c     ro(4,4)    orthogonalization matrix
c     rf(4,4)    inverse of ro, fractionalization matrix
c     rotog(4,4) matrix to go from ortogonal coordinates to grid points
c     rgtoo(4,4) matrix to go from grid points to ortogonal coordinates
c
      implicit none
c
      real cell(6),ro(4,4),rf(4,4),rotog(4,4),rgtoo(4,4),rall(3,3,6),vol
     +  ,mat(4,4)
      integer i,j,ncode,ngrid(3)
c
      external rbfro1,rbrinv
c
      call rbfro1(cell,vol,rall)
      do 10, j=1,4
         do 11, i=1,4
            ro(i,j)=0.0
 11      continue
 10   continue
c
      do 210, j=1,3
        do 211, i=1,3
          ro(i,j)=rall(i,j,ncode)
 211    continue
 210  continue
c
      ro(4,4)=1.0
c
c     Invert
      call rbrinv(ro,rf)
c
      do i=1,4
        do j=1,4
          if ((i.eq.j).and.(i.le.3)) then
            mat(i,j)=ngrid(i)
          else
            mat(i,j)=0.
          endif
        enddo
      enddo
      mat(4,4)=1.
c
      do i=1,4
        do j=1,4
          rotog(i,j)=rf(i,1)*mat(1,j)+rf(i,2)*mat(2,j)+rf(i,3)*mat(3,j)
     +      +rf(i,4)*mat(4,j)
        enddo
      enddo
c     Invert
      call rbrinv(rotog,rgtoo)
c
      return
      end
      subroutine getaxpremain(n1,arr1,n2,arr2,n3,arr3,n4,arr4,n5,arr5,n6
     +  ,arr6)
c
      implicit none
c
      integer n1,n2,n3,n4,n5,n6,asulim(2,3),ngrid(3),asugrid,secgrid
      real arr1(n1),arr2(n2),arr3(n3),arr4(n4),arr5(n5),arr6(n6)
c
      common /asymm/ asulim,ngrid
      common /alloc/ asugrid,secgrid
c
      call getaxmain(arr1,arr2,arr3,arr4,arr5,ngrid,asulim,asugrid
     +  ,secgrid,arr6,ngrid(1),ngrid(2),ngrid(3))
c
      return
      end
      subroutine getaxmain(map,wmap,nap,woap,oap,ngrid,asulim,asugrid
     +  ,secgrid,dum1,ngrid1,ngrid2,ngrid3)
c
c this is the real program, called from getax.f
c
      implicit none
c      
      integer ngrid(3),asulim(2,3),dum1,ixyzin,jpeak,secgrid,asugrid
     +  ,ngrid1,ngrid2,ngrid3
      real map,nap,wmap,woap(1),oap(secgrid)
      logical ldebug,lxyzin,lcorr,lpack,lautoskip,lstick
c
      external ccpdpn
c
      parameter (ixyzin=11)
c
      dimension map(ngrid1,ngrid2,ngrid3),wmap(ngrid1,ngrid2
     +  ,ngrid3),nap(ngrid1,ngrid2,ngrid3),dum1(asugrid)
c
      call rcards(ngrid,ldebug,lxyzin,lcorr,lpack,lautoskip,lstick,jpeak
     +  )
c
      if (.not.lxyzin) then
        call mksphe(ngrid,lstick)
      else
        call ccpdpn(ixyzin,'XYZIN','OLD','F',0,0)
        call rdsphe
      endif
c
      call rmapin(ngrid,map,ldebug,oap,secgrid,ngrid1,ngrid2,ngrid3)
c
      call search(ngrid,map,wmap,nap,ldebug,lcorr,lpack,asulim,dum1
     +  ,lautoskip,lxyzin,lstick,jpeak,secgrid,oap,woap,ngrid1,ngrid2
     +  ,ngrid3,asulim(1,1),asulim(1,2),asulim(1,3),asulim(2,1),asulim(2
     +  ,2),asulim(2,3))
c
      return
      end
      subroutine rcards(ngrid,ldebug,lxyzin,lcorr,lpack,lautoskip,lstick
     +  ,jpeak)
c
c parser s/r to get all input parmaters
c
      implicit none
c      
      character key*4,line*500,cvalue*4
      integer maxtok,ibeg,iend,ityp,idec,ntok,xyzl(6),ngrid(3),iskip
     +  ,istep,iweight,iout,i,nrefi,jpeak,ncode
      real radius(2),height,ang(3),fvalue,minden,report,ang2(3),tmp
     +  ,ccmin
      logical lend,lprint,lmapout,lwmapout,lsphere,lslice,lxyzout,ldebug
     +  ,lang2,lsmapout,lxyzin,lcorr,lpack,lgxyzout,lautoskip,lrefine
     +  ,lax1,lax2,lax3,lax4,lstick,lort
c
      external parser,ccpupc,ccperr
c
      parameter(maxtok=100)
      parameter (iout=6)
c
      dimension ibeg(maxtok),iend(maxtok),ityp(maxtok),fvalue(maxtok)
     +  ,cvalue(maxtok),idec(maxtok)
c
      common /sphere/ lsphere,radius,lslice,height,ang,lxyzout,lang2
     +  ,ang2,lgxyzout
      common /wmapou/ lwmapout,iweight,ccmin
      common /mapout/ lmapout,lsmapout
      common /cpusav/ iskip,istep,minden
      common /output/ report
      common /scan/ xyzl
      common /refine/ lrefine,nrefi
      common /axisopt/ lax1,lax2,lax3,lax4
c
      write(iout,'(/,a)') ' s/r RCARDS:'
c
c DEFAULTS
c
      ccmin=-1.
      radius(1)=25.
      radius(2)=0.
      height=15.
      report=1.
      minden=-999.
      jpeak=20
      ncode=1
c
      lprint=.true.
      lslice=.false.
      lsphere=.true.
      ldebug=.false.
      lxyzout=.false.
      lgxyzout=.false.
      lwmapout=.false.
      lmapout=.true.
      lsmapout=.false.
      lang2=.false.
      lxyzin=.false.
      lcorr=.true.
      lpack=.false.
      lautoskip=.true.
      lrefine=.false.
      lax1=.false.
      lax2=.false.
      lax3=.false.
      lax4=.false.
      lstick=.false.
c
      iweight=1
      iskip=500
      istep=1
      xyzl(1)=0
      xyzl(2)=ngrid(1)-1
      xyzl(3)=0
      xyzl(4)=ngrid(2)-1
      xyzl(5)=0
      xyzl(6)=ngrid(3)-1
      nrefi=10
c
c READ CARDS:
c
 10   continue
      line='    '
      key=' '
      ntok=maxtok
      call parser(key,line,ibeg,iend,ityp,fvalue,cvalue,
     +idec,ntok,lend,lprint)
c
      if (.not.lend) then
         call ccpupc(key)
c
c SPHERE <outer-radius> [<inner-radius>]
c
         if (key(1:4).eq.'SPHE') then
           if (ntok.lt.2) call ccperr(1
     +       ,' SPHERE <outer-radius> [<inner-radius>]')
           radius(1)=fvalue(2)
           if (ntok.gt.2) radius(2)=fvalue(3)
c
c SLICE <outer-radius> <height> [<inner-radius>]
c
         else if ((key(1:4).eq.'SLIC').or.(key(1:4).eq.'DISK')) then
           if (ntok.lt.3) call ccperr(1
     +       ,' DISK <outer-radius> <height> [<inner-radius>]')
           radius(1)=fvalue(2)
           height=fvalue(3)
           if (ntok.gt.3) radius(2)=fvalue(4)
           lsphere=.false.
           lslice=.true.
c
c CHECK [[NO]CORR] [[NO]PACK] [[N]AX1/[N]AX2/[N]AX3/[N]AX4]
c
         else if (key(1:4).eq.'CHEC') then
           do i=2,ntok
             if (cvalue(i)(1:4).eq.'CORR') then
               lcorr=.true.
             else if (cvalue(i)(1:4).eq.'NOCO') then
               lcorr=.false.
             else if (cvalue(i)(1:4).eq.'PACK') then
               lpack=.true.
             else if (cvalue(i)(1:4).eq.'NOPA') then
               lpack=.false.
             else if (cvalue(i)(1:3).eq.'AX1') then
               lax1=.true.
               lax2=.false.
               lax3=.false.
               lax4=.false.
               lstick=.true.
             else if (cvalue(i)(1:3).eq.'AX2') then
               lax1=.false.
               lax2=.true.
               lax3=.false.
               lax4=.false.
               lstick=.true.
             else if (cvalue(i)(1:3).eq.'AX3') then
               lax1=.false.
               lax2=.false.
               lax3=.true.
               lax4=.false.
               lstick=.true.
             else if (cvalue(i)(1:3).eq.'AX4') then
               lax1=.false.
               lax2=.false.
               lax3=.false.
               lax4=.true.
               lstick=.true.
             else if (cvalue(i)(1:4).eq.'NAX1') then
               lax1=.false.
             else if (cvalue(i)(1:4).eq.'NAX2') then
               lax2=.false.
             else if (cvalue(i)(1:4).eq.'NAX3') then
               lax3=.false.
             else if (cvalue(i)(1:4).eq.'NAX4') then
               lax4=.false.
             else
               write(iout,'(a,a)') ' ERROR! subkey unknown (ignored) = '
     +           ,cvalue(i)
             endif
           enddo
c
c ORTHO <ncode>
c
         else if (key(1:4).eq.'ORTH') then
           if (ntok.lt.2) call ccperr(1,' ORTHO <ncode>')
           ncode=nint(fvalue(2))
c
c POLAR <omega> <phi> <kappa> [<omega-2> <phi-2> [<kappa-2>]]
c
         else if (key(1:4).eq.'POLA') then
           if (ntok.lt.4) call ccperr(1
     +       ,' POLAR <omega> <phi> <kappa> [<omega-2> <phi-2>]')
           do i=1,3
             ang(i)=fvalue(i+1)
           enddo
           if (ang(1).eq.0.) ang(2)=0.
           if (ntok.ge.6) then
             lang2=.true.
             ang2(3)=180.
             ang2(1)=fvalue(5)
             ang2(2)=fvalue(6)
             if (ntok.ge.7) ang2(3)=fvalue(7)
           endif
           if (lang2) then
             if ((ang(3).ne.180.).and.(ang2(3).ne.180.)) call ccperr(1
     +         ,' at least one KAPPA angle should be 180 degree')
             if (ang2(3).ne.180.) then
               do i=1,3
                 tmp=ang(i)
                 ang(i)=ang2(i)
                 ang2(i)=tmp
               enddo
             endif
             write(iout,'(//,a,/,a,3f8.3,/,a,3f8.3,/)')
     +         ' Rotation angles used:',' Angle-1 [ OMEGA PHI KAPPA ]'
     +         ,ang,' Angle-2 [ OMEGA PHI KAPPA ]',ang2
             call chkang(ang,ang2,lort)
           endif
c
c SKIP [AUTO] <iskip>
c
         else if (key(1:4).eq.'SKIP') then
           if (ntok.lt.2) call ccperr(1,' SKIP [AUTO] <nskip>')
           if (ityp(2).eq.2) then 
             iskip=nint(fvalue(2))
             lautoskip=.false.
           else if (ityp(2).eq.1) then
             iskip=nint(fvalue(3))
             lautoskip=.true.
           endif
c
c STEP <istep>
c
         else if (key(1:4).eq.'STEP') then
           if (ntok.lt.2) call ccperr(1,' STEP <itep>')
           istep=nint(fvalue(2))
c
c WEIGHT <iweight> [<ccmin>]
c
c iweight=1: weight=CC        CC=correlation
c iweight=2: weight=CC*CC
c iweight=3: weight=abs(CC)
c iweight=4: weight=CC*(1/r)
c iweight=5: weight=1-PP      PP=Packing
c
         else if (key(1:4).eq.'WEIG') then
           if (ntok.lt.2) call ccperr(1,' WEIGHT <iweight> [<ccmin>]')
           iweight=nint(fvalue(2))
           if (ntok.gt.2) ccmin=fvalue(3)
c
c MINDEN <minden>
c
         else if (key(1:4).eq.'MIND') then
           if (ntok.lt.2) call ccperr(1,' MINDEN <minden>')
           minden=fvalue(2)
c
c DEBUG
c
         else if (key(1:4).eq.'DEBU') then  
           ldebug=.true.
c
c XYZLIMIT <xmin> <xmax> <ymin> <ymax> <zmin> <zmax>
c
         else if (key(1:4).eq.'XYZL') then
           if (ntok.lt.7) call ccperr(1
     +       ,' XYZLIMIT <xmin> <xmax> <ymin> <ymax> <zmin> <zmax>')
           do i=1,6
             xyzl(i)=nint(fvalue(1+i))
           enddo
c
c OUTPUT [XYZ/GXYZ] [MAP/NOMAP] [SMAP] [WMAP]
c
         else if (key(1:4).eq.'OUTP') then 
           do i=2,ntok
             if (cvalue(i)(1:3).eq.'XYZ') then
               lxyzout=.true.
               lgxyzout=.false.
             elseif (cvalue(i)(1:4).eq.'GXYZ') then
               lgxyzout=.true.
               lxyzout=.false.
             elseif (cvalue(i)(1:3).eq.'MAP') then
               lmapout=.true.
             elseif (cvalue(i)(1:4).eq.'NOMA') then
               lmapout=.false.
             elseif (cvalue(i)(1:4).eq.'WMAP') then
               lwmapout=.true.
             elseif (cvalue(i)(1:4).eq.'SMAP') then
               lsmapout=.true.
               lmapout=.false.
             else
               call ccperr(1
     +           ,' OUTPUT [XYZ/GXYZ] [MAP/NOMAP] [SMAP] [WMAP]')
             endif
           enddo
c
c INPUT [XYZ]
c
         else if (key(1:4).eq.'INPU') then 
           if (ntok.lt.2) call ccperr(1,' INPUT [XYZ]')
           do i=2,ntok
             if (cvalue(i)(1:3).eq.'XYZ') then
               lxyzin=.true.
             else
               call ccperr(1,' INPUT [XYZ]')
             endif
           enddo
c
c REPORT <report> <top>
c
         else if (key(1:4).eq.'REPO') then 
           if (ntok.lt.2) call ccperr(1,' REPORT <report> <top>')
           do i=2,ntok
             if (fvalue(i).lt.1.) then
               report=fvalue(i)
             else
               jpeak=nint(fvalue(i))
             endif
           enddo
c
c REFINE <nrefi>
c
         else if (key(1:4).eq.'REFI') then
           lrefine=.true.
           if (ntok.gt.1) nrefi=nint(fvalue(2))
c
c END
c
         else if (key(1:3).eq.'END') then
           goto 20
         else
           write(iout,'(a,a4,a)') ' !!!! ERROR: Keyword ',key(1:4),
     +       ' not found. Ignoring Keyword !!!!'
         end if
         goto 10
      end if
c
c END READING CARDS
c
 20   continue
cv      if (.not.lxyzin) call ortpol(ang,lang2,ang2,ncode,ngrid)
      call ortpol(ang,lang2,ang2,ncode,ngrid,lxyzin)
      return
      end
      subroutine chkang(ang,ang2,lort)
c
      implicit none
c      
      integer iout,i,j
      real ang(3),ang2(3),so(3),co(3),sp(3),cp(3),l,m,n,mat(3,3),veca(3)
     +  ,vecb(3),vecc(3),error,ck,sk,betb,beta,w,betc
      logical lort
c
      external cross,ccperr
c
      parameter (iout=6)
      parameter (error=5.)
      real dtor,rtod
      parameter (dtor=3.1415926/180.,rtod=180./3.1415926)
c
c direction cosines
c
      so(1)=sin(dtor*ang(1))
      co(1)=cos(dtor*ang(1))
      sp(1)=sin(dtor*ang(2))
      cp(1)=cos(dtor*ang(2))
      so(2)=sin(dtor*ang2(1))
      co(2)=cos(dtor*ang2(1))
      sp(2)=sin(dtor*ang2(2))
      cp(2)=cos(dtor*ang2(2))
      vecb(1)=so(1)*cp(1)
      vecb(2)=so(1)*sp(1)
      vecb(3)=co(1)
      vecc(1)=so(2)*cp(2)
      vecc(2)=so(2)*sp(2)
      vecc(3)=co(2)
c
c veca = vecb x vecc
c
      call cross(veca,vecb,vecc)
      beta=sqrt(veca(1)**2+veca(2)**2+veca(3)**2)
      betb=sqrt(vecb(1)**2+vecb(2)**2+vecb(3)**2)
      betc=sqrt(vecc(1)**2+vecc(2)**2+vecc(3)**2)
      if ((abs(beta).lt.1.).or.(abs(betb).lt.1.).or.(abs(betc).lt.1.))
     +  then
        w=beta/(betb*betc)
        if (w.gt.1.) w=1.
        if (w.lt.-1.) w=-1.
        w=rtod*asin(w)
      else
        w=90.0
      endif
c
      write(iout,'(/,a,f8.3)')
     +  ' Angle between the two rotation axes = ',w
      if (abs(w-90.).gt.error) call ccperr(1
     +  ,' Angle differs more than 5 degrees from 90 degree')
c
      if (abs(w-90.).gt.0.001) then
        lort=.true.
        if (beta.ne.1.) then
          do i=1,3
            veca(i)=(1./beta)*veca(i)
          enddo
        endif
        l=veca(1)
        m=veca(2)
        n=veca(3)
        ck=cos(dtor*(-1.)*(90.-w))
        sk=sin(dtor*(-1.)*(90.-w))
        mat(1,1)=ck+l*l*(1.-ck)
        mat(1,2)=l*m*(1.-ck)-n*sk
        mat(1,3)=n*l*(1.-ck)+m*sk
        mat(2,1)=l*m*(1.-ck)+n*sk
        mat(2,2)=ck+m*m*(1.-ck)
        mat(2,3)=m*n*(1.-ck)-l*sk
        mat(3,1)=n*l*(1.-ck)-m*sk
        mat(3,2)=m*n*(1.-ck)+l*sk
        mat(3,3)=ck+n*n*(1.-ck)
        call vecmat(vecc(1),vecc(2),vecc(3),mat)
        l=vecc(1)
        m=vecc(2)
        n=vecc(3)
        ck=cos(dtor*180.)
        sk=sin(dtor*180.)
        mat(1,1)=ck+l*l*(1.-ck)
        mat(1,2)=l*m*(1.-ck)-n*sk
        mat(1,3)=n*l*(1.-ck)+m*sk
        mat(2,1)=l*m*(1.-ck)+n*sk
        mat(2,2)=ck+m*m*(1.-ck)
        mat(2,3)=m*n*(1.-ck)-l*sk
        mat(3,1)=n*l*(1.-ck)-m*sk
        mat(3,2)=m*n*(1.-ck)+l*sk
        mat(3,3)=ck+n*n*(1.-ck)
c
        write(iout,'(/,a)')
     +    ' changing second angle to become perpendicular : '
        write(iout,'(a,3f13.8)') ' i/p Angles [ OMEGA PHI KAPPA ] = '
     +    ,ang2
        call m2p(mat,ang2)
        write(iout,'(a,3f13.8)') ' o/p Angles [ OMEGA PHI KAPPA ] = '
     +    ,ang2
        write(iout,'(/,a)') ' Matrix:'
        do i=1,3
          write(iout,'(3f15.10)') (mat(i,j),j=1,3)
        enddo
      else
        lort=.false.
      endif
c
      return
c
      end
      subroutine vecmat(x,y,z,mat)
c
      implicit none
c      
      integer i
      real vec(3),mat(3,3),tmp(3),x,y,z
c
      vec(1)=x
      vec(2)=y
      vec(3)=z
      do i=1,3
        tmp(i)=vec(1)*mat(1,i)+vec(2)*mat(2,i)+vec(3)*mat(3,i)
      enddo
      do i=1,3
        vec(i)=tmp(i)
      enddo
      x=vec(1)
      y=vec(2)
      z=vec(3)
c
      return
      end
      subroutine m2p(mat,w)
c
c converts premultiplier matrix to polar angles OMEGA PHI KAPPA
c
c l=sin(omega)*cos(phi)
c m=sin(omega)*sin(phi)
c n=cos(omega)
c
c so=sin(omega)
c sp=sin(phi)
c co=cos(omega)
c cp=cos(phi)
c sk=sin(kappa)
c ck=cos(kappa)
c
c ck+l**2(1-ck)  lm(1-ck)-n*sk  ln(1-ck)+m*sk
c ml(1-ck)+n*sk  ck+m**2(1-ck)  mn(1-ck)-l*sk
c nl(1-ck)-m*sk  nm(1-ck)+l*sk  ck+n**2(1-ck)
c
      implicit none
c      
      integer ip,io,jj,ii,il,ik,in,im,iout
      real mat(3,3),w(3),so,co,sp,cp,sk,ck,tiny,l,ll,m,mm
     +  ,n,nn,c(3,3),xbor,x
c
      real dtor,rtod
      parameter (dtor=3.1415926/180.,rtod=180./3.1415926)
      parameter (iout=6)
      data tiny /0.00001/
c
c
      ck=(mat(1,1)+mat(2,2)+mat(3,3)-1.)/2.
      do ik=1,2
        w(3)=rtod*acos(ck)*((-1.)**(ik+1))
        sk=sin(dtor*w(3))
        do il=1,2
          x=(mat(1,1)-ck)/(1.-ck)
          if (x.ge.0.) then
            l=((-1.)**(il+1))*sqrt(x)
          else
            l=0.
          endif
          do im=1,2
            x=(mat(2,2)-ck)/(1.-ck)
            if (x.ge.0.) then
              m=((-1.)**(im+1))*sqrt(x)
            else
              m=0.
            endif
            do in=1,2
              x=(mat(3,3)-ck)/(1.-ck)
              if (x.ge.0.) then
                n=((-1.)**(in+1))*sqrt(x)
              else
                n=0.
              endif
              do io=1,2
                w(1)=((-1.)**(io+1))*rtod*acos(xbor(n))
                w(1)=((-1.)**(io+1))*rtod*acos(n)
                co=cos(dtor*w(1))
                so=sin(dtor*w(1))
                do ip=1,2
                  if (abs(so).gt.tiny) then
                    w(2)=((-1.)**(ip+1))*rtod*acos(xbor(l/so))
                  else
                    w(2)=(float(ip)-1.)*180.+rtod*atan(m/n)
                  endif
                  sp=sin(dtor*w(2))
                  cp=cos(dtor*w(2))
                  ll=so*cp
                  mm=so*sp
                  nn=co
                  c(1,1)=ck+ll*ll*(1.-ck)
                  c(1,2)=ll*mm*(1.-ck)-nn*sk
                  c(1,3)=nn*ll*(1.-ck)+mm*sk
                  c(2,1)=ll*mm*(1.-ck)+nn*sk
                  c(2,2)=ck+mm*mm*(1.-ck)
                  c(2,3)=mm*nn*(1.-ck)-ll*sk
                  c(3,1)=nn*ll*(1.-ck)-mm*sk
                  c(3,2)=mm*nn*(1.-ck)+ll*sk
                  c(3,3)=ck+nn*nn*(1.-ck)
                  do ii=1,3
                    do jj=1,3
                      if (abs(c(ii,jj)-mat(ii,jj)).gt.tiny) goto 40
                    enddo
                  enddo
                  goto 50
 40               continue
                enddo
              enddo
 30           continue
            enddo
          enddo
        enddo
 10     continue
      enddo
c
      write(iout,'(a)') 'WARNING: s/r M2P = no convergence !!!'
 50   call angasu(w)
      return
      end
      real function xbor(x)
c
      implicit none
c      
      real x
c
      xbor=min(1.,x)
      xbor=max(-1.,xbor)
c 
      return
      end
      subroutine angasu(w)
c
      implicit none
c
      real w(3)
      integer i
c
      do i=1,3
        if (w(i).lt.0.) w(i)=w(i)+360.
        if (w(i).ge.360.) w(i)=w(i)-360.
      enddo
c
      return
      end
      subroutine mksphe(ngrid,lstick)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c Time-stamp: <2004-10-13 09:27:49 vonrhein> 
c
c purpose: - creates sphere or disk/slice for given rotation angles
c          - initial 'atoms' are build on unit-cell grid
c          - after rotation into propper orientation: interpolation
c            onto nearest grid point
c
c (C) Clemens Vonrhein    vonrhein@bio5.chemie.uni-freiburg.de
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      implicit none
c      
      character nampg*8,namspg*7
      integer ipad,nsec,iuvw(3),mxyz(3),nw1,nu1,nu2,nv1,nv2,lspgrp
     +  ,ngrid(3),iout,ixyz,maxfold,maxpts,iwin,jwin,ix,iy,iz,ii,jj,jjj
     +  ,iix,iiy,iiz,npts,i,ibrkfl,ifold,izmin,jpad,ncode,j,nsym,nsymp
     +  ,ist,istep,nstick,i1,istick, iseed, iseed2
      real mat(3,3),mat1(3,3),mat2(3,3),ww(3),dd(3),radius(2)
     +  ,height,ang(3),gtor(3),rtog(3),rhmin,rhmax,rhmean,rhrms,ro(4,4)
     +  ,rf(4,4),cell(6),x,ang2(3),dc(3),ddc(3),win,rx,ry,rz,fx,fy,fz
     +  ,angx,angy,angz,d,rot(4,4,96),rsym(4,4,96),stick
     +  ,xx(3),xxx(3),roo1(4,4),rff1(4,4),roo2(4,4),rff2(4,4),rotmat1(3
     +  ,3),rotmat2(3,3),rxxx(3),mingrid,mat3(3,3),rvec(1)
      logical lsphere,lslice,lxyzout,lang2,llang2,lgxyzout,lstick,ldebug
c
      external ccperr,cvfrac2,ccpdpn,msymlb
c
      parameter (iout=6)
      parameter (ist=9)
      parameter (ixyz=12)
      parameter (maxfold=8)
      parameter (maxpts=20000)
      real dtor,rtod
      parameter (dtor=3.1415926/180.,rtod=180./3.1415926)
c
      dimension x(maxfold,maxpts,3),stick(maxpts,3)
c
      common /mapin/ nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2,lspgrp
      common /cel/ cell
      common /sphere/ lsphere,radius,lslice,height,ang,lxyzout,lang2
     +  ,ang2,lgxyzout
      common /body/ ifold,npts,x,llang2
      common /mapinf/ rhmin,rhmax,rhmean,rhrms,gtor,rtog
      common /orthog/ ro,rf,ncode,ibrkfl
      common /packit/ nsym,rot
      common /axisstick/ nstick,stick
      common /dencode/ roo1,rff1,roo2,rff2
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      ifold=nint(360./ang(3))
      if ((360.-(ifold*ang(3))).gt.0.1) call ccperr(1
     +  ,' KAPPA gives no 2,3,4,5,6,... -fold axis')
      call p2m(ang,rotmat1)
      write(iout,'(//,a,/,a,/,3(3f10.5,/))')
     +  ' s/r MKSPHE:',' Rotation matrix for 1st rotation:'
     +  ,((rotmat1(ii,jj),jj=1,3),ii=1,3)
      write(iout,'(a,3f8.2,/)') ' Polar Angle     = ',ang
      if (lang2) then
        call p2m(ang2,rotmat2)
        write(iout,'(/,a,/,3(3f10.5,/))')
     +    ' Rotation matrix for 2nd rotation:',((rotmat2(ii,jj),jj=1,3)
     +    ,ii=1,3)
        write(iout,'(a,3f8.2,/)') ' Polar Angle     = ',ang2
      endif
      ww(1)=0
      ww(2)=0
      ww(3)=ang(3)
      call p2m(ww,mat)
c
      ipad=5
c
c     what is the necessary padding and minimum grid-spacing?
c
      mingrid=9999.
      do i=1,3
        jpad=nint(((radius(1)*.5)/cell(i))*float(ngrid(i)))
        mingrid=min(mingrid,(cell(i)/float(ngrid(i))))
        if (jpad.gt.ipad) ipad=jpad
      enddo
c
c     what is the maximum box size ?
c
      if (lsphere) then
        iix=nint(radius(1)*rtog(1))+ipad
        iiy=nint(radius(1)*rtog(2))+ipad
        iiz=nint(radius(1)*rtog(3))+ipad
        write(iout,'(/,a,6i4,/,a,f8.2,/,a,f8.2)') 
     +    ' Creating Sphere in box ',-iix,iix,-iiy,iiy,-iiz,iiz
     +    ,' with outer Radius =',radius(1),'      inner Radius ='
     +    ,radius(2)
      else if (lslice) then
        if (height.gt.radius(1)) then
          jpad=nint(((height*.5)/cell(3))*float(ngrid(3)))
          if (jpad.gt.ipad) ipad=jpad
        endif
        iix=nint(radius(1)*rtog(1))+ipad
        iiy=nint(radius(1)*rtog(2))+ipad
        iiz=nint(height*rtog(3))+ipad
        write(iout,'(/,a,6i4,/,a,f8.2,/,a,f8.2,/,a,f8.2)')
     +    ' Creating Slice in box ',-iix,iix,-iiy,iiy,
     +    -iiz,iiz,' with outer Radius =',radius(1)
     +    ,'      inner Radius =',radius(2),'        and Height ='
     +    ,height
      endif
c
      npts=0
c
      izmin=-iiz
      if (lang2) izmin=1
c
      do ix=-iix,iix,1
c
c       get fractional coordinate for X
c
        fx=float(ix)/float(ngrid(1))
        do iy=-iiy,iiy,1
c
c         get fractional coordinate for Y
c
          fy=float(iy)/float(ngrid(2))
          do iz=izmin,iiz,1
c
c           debug condition
c
            ldebug=.false.
c
c           origin?
c
            if ((ix.eq.0).and.(iy.eq.0).and.(iz.eq.0)) goto 10
c
c           get fractional coordinate for Z
c
            fz=float(iz)/float(ngrid(3))
c
c           get orthogonal coordinates
c
            call cvfrac2(fx,fy,fz,rx,ry,rz,0)
            dd(1)=sqrt(rx**2+ry**2)
c
            if (abs(rx).lt.0.0001) rx=0.
            if (abs(ry).lt.0.0001) ry=0.
c
c           get angle to x-axis
c
            if (rx.ge.0.) then
              if (ry.gt.0.0001) then
                win=rtod*acos(abs(rx)/dd(1))
              else
                win=360.-rtod*acos(abs(rx)/dd(1))
              endif
            else
              if (ry.gt.0.0001) then
                win=180.-rtod*acos(abs(rx)/dd(1))
              else
                win=180.+rtod*acos(abs(rx)/dd(1))
              endif
            endif
            if (win.eq.360.) win=0.
c
            iwin=nint((win/ang(3))+0.5)
            jwin=nint((win/(ang(3)/2.))+0.5)
            if (ldebug) write(iout,*) ix,iy,iz,rx,ry,rz,win,iwin
            if (iwin.eq.1) then
              if (lang2.and.(iz.eq.0).and.(jwin.ne.1)) goto 10
              if (lsphere) then
                d=sqrt(rx**2+ry**2+rz**2)
              else if (lslice) then
                d=dd(1)
                if (ldebug) write(iout,*) ix,iy,iz,d,abs(rz),(.5*height)
                if (abs(rz).gt.(.5*height)) goto 10
              endif
              if ((d.gt.radius(1)).or.(d.lt.radius(2))) goto 10
              npts=npts+1
              if (npts.gt.maxpts) then
                write(6,*) 
     . ' ERROR: more than MAXPTS points in sphere - please recompile'
                stop
              endif
              x(1,npts,1)=rx
              x(1,npts,2)=ry
              x(1,npts,3)=rz
c
            endif
 10         continue
          enddo
        enddo
      enddo
c
      write(iout,'(/,a,i10)')
     +  ' number of grid points for each segment = ',npts
      if (.not.lang2) then
        write(iout,'(a,i10)') 
     +    ' number of segments                     = ',ifold
      else
        write(iout,'(a,i10)') 
     +    ' number of segments                     = ',ifold*2
      endif
c
      if (npts.gt.maxpts) then
        write(iout,'(a,/,a,i10,/,a,i10)') ' Current values:'
     +    ,' MAXPTS = ',maxpts,' NPTS = ',npts
        call ccperr(1,' change parameter MAXPTS and recompile')
      endif
c
c     generate other segments
c
      do j=2,ifold
        do jjj=1,npts
          do jj=1,3
            x(j,jjj,jj)=x((j-1),jjj,1)*mat(1,jj)+x((j-1),jjj,2)
     +        *mat(2,jj)+x((j-1),jjj,3)*mat(3,jj)
          enddo
        enddo
      enddo
c
c     now we have all segments, but with the rotation axis oriented
c     along Z
c
      if (lstick) then
c
c       get axis stick
c
        istick=0
        if (lsphere) then
          istick=nint((2.*radius(1))/mingrid)
c
c         make sure we get an uneven number
c
          if (mod(istick,2).eq.0) istick=istick+1
c
c         get new mingrid
c
          mingrid=(2.*radius(1))/float(istick)
        elseif (lslice) then
          istick=nint(height/mingrid)
          if (mod(istick,2).eq.0) istick=istick+1
          mingrid=(height)/float(istick)
        endif
c
        nstick=0
        do iz=-((istick-1)/2),((istick-1)/2),1
          rz=float(iz)*mingrid
cv          call cvfrac2(0.,0.,fz,rx,ry,rz,0)
          nstick=nstick+1
          stick(nstick,1)=0.
          stick(nstick,2)=0.
          stick(nstick,3)=rz
 20       continue
        enddo
      endif
c
      ww(1)=90.
      ww(2)=270.
      ww(3)=ang(1)
      call p2m(ww,mat1)
c
      ww(1)=0.
      ww(2)=0.
      ww(3)=360.-ang(2)
      call p2m(ww,mat2)
c
c     get overall matrix
c
      call matmul(mat3,mat1,mat2,3)
c
      if (lang2) then
        write(iout,'(//,a,/,3(3f10.5,/))')
     +    ' Rotation matrix for orienting first axis:'
     +    ,((mat3(ii,jj),jj=1,3),ii=1,3)
      else
        write(iout,'(//,a,/,3(3f10.5,/))')
     +    ' Rotation matrix for orienting axis:'
     +    ,((mat3(ii,jj),jj=1,3),ii=1,3)
      end if
c
      do j=1,ifold
        do jj=1,npts
          call vecmat(x(j,jj,1),x(j,jj,2),x(j,jj,3),mat3)
        enddo
      enddo
c
      if (lstick) then
        do jj=1,nstick
          call vecmat(stick(jj,1),stick(jj,2),stick(jj,3),mat3)
        enddo
      endif
c
c     get rotation matrix for moving it back
c
      ww(1)=0.
      ww(2)=0.
      ww(3)=ang(2)
      call p2m(ww,mat1)
c
      ww(1)=90.
      ww(2)=90.
      ww(3)=ang(1)
      call p2m(ww,mat2)
c
      call matmul(mat3,mat1,mat2,3)
c
      write(iout,'(/,a,/,3(3f10.5,/))')
     +  ' Rotation matrix for moving oriented axis back to Z:'
     +  ,((mat3(ii,jj),jj=1,3),ii=1,3)
c
c     now we have the segment oriented along the first rotation axis.
c     to get a nicer view for the XYZOUT file:
c     orient the segment borders along the second (2-fold) rotation axis
c
      if (lang2) then
        dc(1)=sin(dtor*ang2(1))*cos(dtor*ang2(2))
        dc(2)=sin(dtor*ang2(1))*sin(dtor*ang2(2))
        dc(3)=cos(dtor*ang2(1))
c
        angx=-1.
        angy=0.
        angz=0.
        call vecmat(angx,angy,angz,mat1)
        call vecmat(angx,angy,angz,mat2)
c
        ddc(1)=sqrt(dc(1)**2+dc(2)**2+dc(3)**2)
        ddc(2)=sqrt(angx**2+angy**2+angz**2)
        ddc(3)=sqrt((angx-dc(1))**2+(angy-dc(2))**2+(angz-dc(3))**2)
        ww(1)=ang(1)
        ww(2)=ang(2)
        ww(3)=(ddc(3)**2-ddc(1)**2-ddc(2)**2)/(-2.*ddc(1)*ddc(2))
        if (ww(3).gt.1.) ww(3)=1.
        if (ww(3).lt.-1.) ww(3)=-1.
        ww(3)=rtod*acos(ww(3))
        ww(3)=-ww(3)
        write(iout,'(a,/,a,f8.3)') ' to have nice segments to look at:',
     +    ' additional Rotation along axis: ',ww(3)
        call p2m(ww,mat1)
        call p2m(ang2,mat2)
c
c       get overall matrix
c
        call matmul(mat3,mat1,mat2,3)
c
        write(iout,'(//,a,/,3(3f10.5,/))')
     +    ' Rotation matrix for orienting others axes:'
     +    ,((mat3(ii,jj),jj=1,3),ii=1,3)
c
        do j=1,ifold
          do jj=1,npts
            call vecmat(x(j,jj,1),x(j,jj,2),x(j,jj,3),mat1)
            do jjj=1,3
              x(j+ifold,jj,jjj)=x(j,jj,jjj)
            enddo
            call vecmat(x(j+ifold,jj,1),x(j+ifold,jj,2),x(j+ifold,jj,3)
     +        ,mat2)
          enddo
        enddo
        ifold=ifold*2
        if (lstick) then
          do j=1,nstick
            jj=nstick+j
            stick(jj,1)=stick(j,1)
            stick(jj,2)=stick(j,2)
            stick(jj,3)=stick(j,3)
            call vecmat(stick(jj,1),stick(jj,2),stick(jj,3),mat1)
            call vecmat(stick(jj,1),stick(jj,2),stick(jj,3),mat2)
          enddo
          nstick=nstick*2
        endif
c
      endif
      if (lstick) then
        write(iout,'(/,a,i4,/)') ' # of points in axis-stick = ',nstick
        if (nstick.eq.0) then
          write(iout,'(a)') ' WARNING: not enough points in stick!'
     +      ,' Stick mode not used'
          lstick=.false.
        endif
      endif
c
      if (lgxyzout) then
        call intpol(ngrid,lstick)
      endif
c
c     write out all segments
c
 30   if (lxyzout.or.lgxyzout) then
        if (npts.gt.9999) then
          istep=nint((float(npts)/9999.)+.5)
          write(iout,'(a,/,a,i10,/,a,i4,a,/,a,i10)')
     +      ' WARNING: more than 9999 points per segment !'
     +      ,' # of points        = ',npts,' Only writing every ',istep
     +      ,'th point',' # of point written = ',nint(float(npts)
     +      /float(istep))
        else
          istep=1
        endif
        call ccpdpn(ixyz,'XYZOUT','UNKNOWN','F',0,0)
        call msymlb(ist,lspgrp,namspg,nampg,nsymp,nsym,rsym)
        write(ixyz,'(a6,3f9.3,3f7.2,1x,a1,1x,a6,3x,i4)') 'CRYST1'
     +    ,(cell(i),i=1,6),namspg(1:1),namspg(2:7),(nsym*ifold)
        do i=1,3
          write(ixyz,'(a5,i1,4x,3f10.6,5x,f10.5)') 'SCALE',i,(rff1(i,ii)
     +      ,ii=1,4)
        enddo
c     CCP4 random number generator: initialise (call rmarin) with
c     variants on the time (ustime)
        call ustime(iseed)
        iseed2 = iseed + 12345
        iseed = mod(iseed,31329)
        iseed2 = mod(iseed2,30082)
        call rmarin(iseed,iseed2)
cv     wraper to C library random number generator:
cv        call randomseed(0)
        do i=1,ifold
          jj=0
          do ii=1,npts,1
c     CCP4 random number:
            call ranmar(rvec,1)
            if ((rvec(1)*istep).le.1.) then
cv     wraper to C library random number generator:
cv            if ((random()*istep).le.1.) then
              jj=jj+1
              do i1=1,3
                xx(i1)=x(i,jj,1)*rff2(i1,1)+x(i,jj,2)*rff2(i1,2)+x(i,jj
     .            ,3)*rff2(i1,3)
              enddo
              do i1=1,3
                xxx(i1)=xx(1)*roo1(i1,1)+xx(2)*roo1(i1,2)+xx(3)*roo1(i1
     .            ,3)
              enddo
c
              if (lxyzout) then
                write(ixyz,100) jj,i,jj,(xxx(j),j=1,3)
              else
                call cvfrac2(xxx(1)/float(ngrid(1)),xxx(2)/float(ngrid(2
     .            )),xxx(3)/float(ngrid(3)),rxxx(1),rxxx(2),rxxx(3),0)
                write(ixyz,100) jj,i,jj,(rxxx(j),j=1,3)
              endif
            endif
c           
          enddo
        enddo
 100    format('ATOM   ',i4,'  C   DUM ',i1,i4,4x,3f8.3
     +    ,'  1.00 10.00')
        if (lstick) then
          i=ifold+1
          do ii=1,nstick
            do i1=1,3
              xx(i1)=stick(ii,1)*rff2(i1,1)+stick(ii,2)*rff2(i1,2)
     +          +stick(ii,3)*rff2(i1,3)
            enddo
            do i1=1,3
              xxx(i1)=xx(1)*roo1(i1,1)+xx(2)*roo1(i1,2)+xx(3)*roo1(i1,3)
            enddo
            if (lxyzout) then
              write(ixyz,100) ii,i,ii,(xxx(j),j=1,3)
            else
              call cvfrac2(xxx(1)/float(ngrid(1)),xxx(2)/float(ngrid(2))
     +          ,xxx(3)/float(ngrid(3)),rxxx(1),rxxx(2),rxxx(3),0)
              write(ixyz,100) ii,i,ii,(rxxx(j),j=1,3)
            endif
          enddo
        endif
        write(ixyz,'(a)') 'END'
        close (ixyz)
      endif
c
      if (.not.lgxyzout) then
        call intpol(ngrid,lstick)
      endif
c
      return
      end
      subroutine p2m(w,mat)
c
c converts polar angles OMEGA PHI KAPPA to premultiplier matrix
c
      implicit none
c      
      real w(3),mat(3,3),l,m,n,ck,sk
      real dtor
      parameter (dtor=3.1415926/180.)
c
      l=sin(dtor*w(1))*cos(dtor*w(2))
      m=sin(dtor*w(1))*sin(dtor*w(2))
      n=cos(dtor*w(1))
      ck=cos(dtor*w(3))
      sk=sin(dtor*w(3))
c
c Int. Tab.:
      mat(1,1)=ck+l*l*(1.-ck)
      mat(1,2)=l*m*(1.-ck)-n*sk
      mat(1,3)=n*l*(1.-ck)+m*sk
      mat(2,1)=l*m*(1.-ck)+n*sk
c Int. Tab.:
      mat(2,2)=ck+m*m*(1.-ck)
      mat(2,3)=m*n*(1.-ck)-l*sk
      mat(3,1)=n*l*(1.-ck)-m*sk
      mat(3,2)=m*n*(1.-ck)+l*sk
c Int. Tab.:
      mat(3,3)=ck+n*n*(1.-ck)
c
      return
      end
      subroutine intpol(ngrid,lstick)
c
c interpolates sphere or disc/slice "atoms" onto grid points
c so it's done one and forever
c
      implicit none
c      
      integer ifold,iout,npts,maxfold,maxpts,ngrid(3),i,j,nstick,k1,k2
     +  ,k3,i1,i2,i3,j1,j2,j3,l1,l2,l3,nover,noverall
      real x,f1,f2,f3,stick,ff1,ff2,ff3,y1,y2,y3,dis,ddis
      logical llang2,lstick
c
      external cvfrac2
c
      parameter (iout=6)
      parameter (maxfold=8)
      parameter (maxpts=20000)
c
      dimension x(maxfold,maxpts,3),stick(maxpts,3)
c
      common /body/ ifold,npts,x,llang2
      common /axisstick/ nstick,stick
c
      write(iout,'(/,a,/,a,/)')
     +  ' s/r INTPOL : '
     +  ,' (interpolates orthogonal points onto map grid points)'
c initialize
      l1=0
      l2=0
      l3=0
c
      do i=1,npts
        do j=1,ifold
          call cvfrac2(x(j,i,1),x(j,i,2),x(j,i,3),f1,f2,f3,1)
          j1=nint(f1*float(ngrid(1)))
          j2=nint(f2*float(ngrid(2)))
          j3=nint(f3*float(ngrid(3)))
          ddis=999.
          do i1=-1,1,1
            k1=j1+i1
            ff1=float(k1)/float(ngrid(1))
            do i2=-1,1,1
              k2=j2+i2
              ff2=float(k2)/float(ngrid(2))
              do i3=-1,1,1
                k3=j3+i3
                ff3=float(k3)/float(ngrid(3))
                call cvfrac2(ff1,ff2,ff3,y1,y2,y3,0)
                dis=sqrt((x(j,i,1)-y1)**2+(x(j,i,2)-y2)**2+(x(j,i,3)-y3)
     +            **2)
                if (dis.lt.ddis) then
                  l1=k1
                  l2=k2
                  l3=k3
                  ddis=dis
                endif
              enddo
            enddo
          enddo
          x(j,i,1)=float(l1)
          x(j,i,2)=float(l2)
          x(j,i,3)=float(l3)
        enddo
      enddo
c
      noverall=0
      do i=1,ifold
        nover=0
        do j1=1,npts
          do j2=j1+1,npts
            if ((x(i,j1,1).eq.x(i,j2,1)).and.(x(i,j1,2).eq.x(i,j2,2))
     +        .and.(x(i,j1,3).eq.x(i,j2,3))) nover=nover+1
          enddo
        enddo
        write(iout,'(a,i2,a,i10,a,f6.3,a)')
     +    ' # of overlaps for segment #',i
     +    ,' in interpolation onto grid points: ',nover,' ( fraction = '
     +    ,float(nover)/float(npts),' )'
        noverall=noverall+nover
      enddo
      write(iout,'(a,i10,a,f6.3,a)')
     +  ' # of overlaps for all segments : ',noverall,' ( fraction = '
     +  ,float(noverall)/float(npts*ifold),' )'
c
      if (lstick) then
        do i=1,nstick
          call cvfrac2(stick(i,1),stick(i,2),stick(i,3),f1,f2,f3,1)
          j1=nint(f1*float(ngrid(1)))
          j2=nint(f2*float(ngrid(2)))
          j3=nint(f3*float(ngrid(3)))
          ddis=999.
          do i1=-1,1,1
            k1=j1+i1
            ff1=float(k1)/float(ngrid(1))
            do i2=-1,1,1
              k2=j2+i2
              ff2=float(k2)/float(ngrid(2))
              do i3=-1,1,1
                k3=j3+i3
                ff3=float(k3)/float(ngrid(3))
                call cvfrac2(ff1,ff2,ff3,y1,y2,y3,0)
                dis=sqrt((stick(i,1)-y1)**2+(stick(i,2)-y2)**2+(stick(i
     +            ,3)-y3)**2)
                if (dis.lt.ddis) then
                  l1=k1
                  l2=k2
                  l3=k3
                  ddis=dis
                endif
              enddo
            enddo
          enddo
          stick(i,1)=float(l1)
          stick(i,2)=float(l2)
          stick(i,3)=float(l3)
        enddo
c
        nover=0
        do j1=1,nstick
          do j2=j1+1,nstick
            if ((stick(j1,1).eq.stick(j2,1)).and.(stick(j1,2).eq
     +        .stick(j2,2)).and.(stick(j1,3).eq.stick(j2,3))) nover
     +        =nover+1
          enddo
        enddo
        write(iout,'(a,a,i10,a,f6.3,a)')
     +    ' # of overlaps for stick'
     +    ,' in interpolation onto grid points: ',nover,' ( fraction = '
     +    ,float(nover)/float(nstick),' )'
      endif
      write(iout,'(//,a,/,a)')
     +  ' If too much overlapping points occured:'
     +  ,' perhaps you should use a finer grid for your i/p map ?!'
c
      return
c
      end
      subroutine rdsphe
c
      implicit none
c      
      character zeile*80,chnold,chnnew
      integer ifold,npts,iout,ixyzin,maxfold,maxpts,i,nptsold
      real x
      logical llang2
c
      external ccperr
c
      parameter (iout=6)
      parameter (ixyzin=11)
      parameter (maxfold=8)
      parameter (maxpts=20000)
c
      dimension x(maxfold,maxpts,3)
c
      common /body/ ifold,npts,x,llang2
c
      ifold=0
      npts=0
      nptsold=maxpts
      write(iout,'(/,a,1x,i2)') ' s/r RDSPHE: reading XYZIN on unit'
     +  ,ixyzin
 10   read(ixyzin,'(a80)',end=100) zeile
c
      if (zeile(1:4).eq.'ATOM') then
        if ((ifold.eq.0).and.(npts.eq.0)) then
          ifold=1
          chnold=zeile(22:22)
          if (chnold.eq.' ') call ccperr(1
     +      ,' XYZIN has invalid CHAIN id ')
        endif
        chnnew=zeile(22:22)
        if (chnnew.ne.chnold) then
          nptsold=npts
          ifold=ifold+1
          npts=0
        endif
        npts=npts+1
        if (npts.gt.nptsold) then
          write(iout,'(a,a,a,i8,/,a,a,a,i8)') 'Chain ',chnold
     +      ,' : #pts = ',nptsold,'Chain ',chnnew,' : #pts = ',npts
          call ccperr(1,' Different # of atoms in XYZIN chains')
        endif
        read(zeile(31:54),'(3f8.3)') (x(ifold,npts,i),i=1,3)
        chnold=chnnew
      endif
      goto 10
c
 100  write(iout,'(//,a,i3,/,a,i5)') ' # of chains read = ',ifold
     +  ,' # of points in each chain = ',npts
c
      return
      end
      subroutine rmapin(ngrid,map,ldebug,nap,secgrid,ngrid1,ngrid2
     +  ,ngrid3)
c
      implicit none
c      
      integer ngrid(3),nsec,iuvw(3),mxyz(3),nw1,nu1,nu2,nv1,nv2
     +  ,lspgrp,iin,iout,ii,i1,i2,i3,ier,jj,jsec,secgrid,ngrid1,ngrid2
     +  ,ngrid3
      real nap,map(ngrid1,ngrid2,ngrid3)
      logical ldebug
c
      external mposn,mgulpr,ccperr,mrclos
c
      parameter (iout=6)
      parameter (iin=10)
c
      dimension nap(secgrid)
c
      common /mapin/ nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2,lspgrp
c
      jj=0
      write(iout,'(//,a,/,a,//,a,3i4)') ' s/r RMAPIN:',
     +  ' (reads in map)',' Grid = ',ngrid
      do i1=1,ngrid(3)
        jsec=nw1+i1-1
        if (ldebug) write(iout,'(a,i4)') ' reading Section #',jsec
        call mposn(iin,jsec)
        call mgulpr(iin,nap,ier)
        if (ier.ne.0) call ccperr(1,' Problems reading MAPIN')
        ii=0
        do i2=1,ngrid(2)
          do i3=1,ngrid(1)
            ii=ii+1
            map(i3,i2,i1)=nap(ii)
          enddo
        enddo
        jj=jj+ii
      enddo
c
      write(iout,'(a,i14)') ' # of values read = ',jj
c
      call mrclos(iin)
c
      return
      end
      subroutine search(ngrid,map,wmap,nap,ldebug,lcorr,lpack,asulim
     +  ,asumap,lautoskip,lxyzin,lstick,jpeak,secgrid,oap,woap,ngrid1
     +  ,ngrid2,ngrid3,asulim11,asulim12,asulim13,asulim21,asulim22
     +  ,asulim23)
c
      implicit none
c      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c 
c s/r which does most of the real work in GETAX
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      integer ngrid(3),j,xyzl(6),iweight,ifold,npts,napnum,asulim(2,3)
     +  ,asumap,maxfold,maxpts,iout,iwmap,i1,i2,i3,i4,i5,i6
     +  ,istep,iskip,num,idone,kdone,nn,i,jdone,nsym,nrefi,jpeak,secgrid
     +  ,iskip1,ngrid1,ngrid2,ngrid3,asulim11,asulim12,asulim13,asulim21
     +  ,asulim22,asulim23
      real sxy,sx,aver,rxy,nap,oap,rxymx,minden,wmap,woap,wmapmax
     +  ,scale,ccmin,x,gtor(3),rtog(3),map,napsum,napave,f1,f2,f3,r1,r2
     +  ,r3,srxy,rhmean,rhmax,rhmin,report,rcut,rhrms,pack,rot(4,4,96)
     +  ,frac,napmean,naprms,rgrid(3),rr1,rr2,rr3
      logical lmapout,lwmapout,ldebug,lsmapout,lang2,lcorr,lpack
     +  ,lpackprint,lautoskip,lxyzin,lrefine,lstick
      double precision sumrho,sumrh2,offstr
c
      external cvfrac2,mspew,mwclose
c
      parameter (maxfold=8)
      parameter (maxpts=20000)
      parameter (iout=6)
      parameter (iwmap=11)
c
c     dimensioning using parameter values defined above
c 
      dimension map(ngrid1,ngrid2,ngrid3),sxy(maxfold
     +  ,maxfold),sx(maxfold),j(maxfold,maxpts,3),aver(maxfold)
     +  ,nap(ngrid1,ngrid2,ngrid3),oap(secgrid),x(maxfold
     +  ,maxpts,3),wmap(ngrid1,ngrid2,ngrid3),woap(1)
     +  ,asumap(asulim11:asulim21,asulim12:asulim22,asulim13:asulim23)
c
c     common blocks
c 
      common /cpusav/ iskip,istep,minden
      common /output/ report
      common /mapout/ lmapout,lsmapout
      common /scan/ xyzl
      common /wmapou/ lwmapout,iweight,ccmin
      common /body/ ifold,npts,x,lang2
      common /mapinf/ rhmin,rhmax,rhmean,rhrms,gtor,rtog
      common /packit/ nsym,rot
      common /refine/ lrefine,nrefi
      common /mohsum/ sumrho,sumrh2,offstr
c
      save /mohsum/
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     initialize
c
      rgrid(1) = float(ngrid(1))
      rgrid(2) = float(ngrid(2))
      rgrid(3) = float(ngrid(3))
      do i=1,secgrid
        oap(i)=0.
      enddo
      lpackprint=.false.
c
c     how many gridpoints do we want to scan?
c
      jdone=((xyzl(2)-xyzl(1)+1)/istep)*((xyzl(4)-xyzl(3)+1)/istep)
     +  *((xyzl(6)-xyzl(5)+1)/istep)
      write(iout,'(//,a)') ' s/r SEARCH:'
c
c     setting up skip
c
      if (lautoskip) then
        iskip1=iskip
        iskip=nint(float(npts)/float(iskip))
        if (iskip.lt.1) iskip=1
        if ((nint(float(npts)/float(iskip))).lt.iskip1) iskip=max(1
     +    ,(iskip-1))
        write(iout,'(/,a,i4)') ' iskip set to : ',iskip
      endif
c
c     echoing some options used:
c
      write(iout,'(/,a)')
     +  ' calculates correlations at grid points of map'
      if (.not.lxyzin) then
        if (.not.lang2) then
          write(iout,'(//,a,i10,a)') ' Order of symmetry axis = ',ifold
     +      ,'-fold'
        else
          write(iout,'(//,a,i10,a)') ' Order of symmetry axis = ',ifold
     +      /2,'-fold'
        endif
      endif
c
      if (istep.gt.1) write(iout,'(a,i4)')
     +  ' Step size in grid units along the axes = ',istep
c
      write(iout,'(a,6i4)')
     +  ' Area of the map to search [ XMIN XMAX YMIN YMAX ZMIN ZMAX ] ='
     +  ,(xyzl(i),i=1,6)
c
      if ((xyzl(1).ne.0).or.(xyzl(2).ne.(ngrid(1)-1)).or.(xyzl(3).ne.0)
     +  .or.(xyzl(4).ne.(ngrid(2)-1)).or.(xyzl(5).ne.0).or.(xyzl(6).ne
     +  .(ngrid(3)-1))) then
        write(iout,'(a,/)')
     +    ' Setting all values outside these limits to the average'
      endif
c
      write(iout,'(a,i20)') ' # of Grid-Points to look at = ',jdone
c
      if (lcorr) then
        write(iout,'(/,a,/,a,i3,a)')
     +    ' At each Grid-point the Correlation for',' each Pair of the '
     +    ,ifold,' Segments will be calculated.'
      endif
c
      write(iout,'(/,a,i10,a)') ' Each segment contains ',npts,' points'
      frac=(float(ifold*npts)/float(jdone))
      write(iout,'(a,f5.3)') ' Corresponding fraction of Unit Cell = '
     +  ,frac
      frac=(float(ifold*npts*nsym)/float(jdone))
      if (frac.gt.1.) then
        write(iout,'(//,a,/,a,i10,/,a,i10,/,a,i10,/,a,i10,/,a,f10.3)')
     +    ' WARNING: more points in segment than in asymmetric unit!'
     +    ,' # points in Unit Cell          = ',jdone
     +    ,' # of asymmetric units          = ',nsym
     +    ,' # of points in asymmetric unit = ',jdone/nsym
     +    ,' # of points in all segments    = ',npts*nsym
     +    ,' percentage of Unit Cell filled = ',frac
      endif
c
      if (iskip.gt.1) write(iout,'(a,i4,a,/,a,i10)') ' Only every '
     +  ,iskip,'th point of these will be used to save CPU'
     +  ,' # of points/segment used for correlation = ',nint(float(npts)
     +  /float(iskip))
c
      if (nint(float(npts)/float(iskip)).lt.500.) write(iout,'(a,/,a)')
     +  ' WARNING: less than 500 points/segment !'
     +  ,' Unreliable results possible.'
c
      if (lcorr) then
        if (report.lt.1.) write(iout,'(/,a,f10.4)')
     +    ' Reporting all Correlations > ',report
      endif
c
      if (minden.gt.-999.) then
        rcut=rhmean+(minden*rhrms)
        write(iout,'(a,a,f5.2,a,f8.5,a,f8.5,a)')
     +    ' Skipping points where the <density> for'
     +    ,' one segment is less than ',minden,' * ',rhrms
     +    ,' below the mean ( = ',rcut,' )'
      else
        rcut=rhmin
      endif
c
      if (lwmapout) then
        write(iout,'(/,a,i2,/)')
     +    ' Writing weighted map WMAPOU with iweight=',iweight
        if (iweight.eq.1) write(iout,'(a)') ' Weight = CC'
        if (iweight.eq.2) write(iout,'(a)') ' Weight = CC*CC'
        if (iweight.eq.3) write(iout,'(a)') ' Weight = abs(CC)'
        if (iweight.eq.4) write(iout,'(a)') ' Weight = CC*(1/r)'
        if (iweight.eq.5) write(iout,'(a)') ' Weight = 1-PP'
        if (ccmin.gt.-1.) write(iout,'(a,f6.3)')
     +    ' Weighting will only be done, if CC.ge.',ccmin
      endif
c
      if (ifold.eq.2) then
        if (lsmapout.and.lcorr) then
          write(iout,'(/,a,a,/)')
     +      ' No MAPOUT with values of '
     +      ,' CC*(1.0-(sd(CC)/CC)) for ifold=2 '
        endif
        lsmapout=.false.
        lmapout=.true.
      endif
      if (lmapout.or.lsmapout) then
        write (iout,'(/,a,/,a)') ' CC = correlation'
     +    ,' PP = 1.0-P (P=fraction overlap)'
      endif
      if (lmapout.and.lpack.and.(.not.lcorr)) then
        write(iout,'(/,a,/)')
     +    ' Writing map MAPOUT with values of PP'
      endif
      if (lsmapout.and.lcorr.and.(.not.lpack)) then
        write(iout,'(/,a,/)')
     +    ' Writing map MAPOUT with values of CC*(1.0-(sd(CC)/CC))'
      endif
      if (lmapout.and.lcorr.and.(.not.lpack)) then
        write(iout,'(/,a,/)')
     +    ' Writing map MAPOUT with values of CC'
      endif
      if (lsmapout.and.lcorr.and.lpack) then
        write(iout,'(/,a,a,/)')
     +    ' Writing map MAPOUT with values of '
     +    ,'(CC*PP)*(1.0-(sd(PP*CC)/(PP*CC)))'
      endif
      if (lmapout.and.lcorr.and.lpack) then
        write(iout,'(/,a,/)')
     +    ' Writing map MAPOUT with values of (CC*PP)'
      endif
c
c     Starting values
c
      napnum=0
      idone=0
      kdone=0
      rxymx=-1.
      scale=100./rhmax
      wmapmax=-999.
      napsum=0.
c
c     clear some arrays
c
      if (lwmapout) then
        do i1=1,ngrid(1)
          do i2=1,ngrid(2)
            do i3=1,ngrid(3)
              wmap(i1,i2,i3)=0.
              nap(i1,i2,i3)=-2.
            enddo
          enddo
        enddo
      endif
c
c     loop over XYZL i1=xmin...xmax
c                    i2=ymin...ymax
c                    i3=zmin...zmax
c
c     default to 1 - ngrid(1), ...
c
      do i1=(xyzl(1)+1),(xyzl(2)+1),istep
        rr1 = float(i1-1)
        write(iout,'(a,3i4)') ' X-Section : ',(i1-1)
        do i2=(xyzl(3)+1),(xyzl(4)+1),istep
          rr2 = float(i2-1)
          do i3=(xyzl(5)+1),(xyzl(6)+1),istep
            rr3 = float(i3-1)
c
            idone=idone+1
            if ((mod(idone,1000).eq.1).and.(idone.gt.1)) write(iout
     +        ,'(a,i20,a,f6.1,a)')' # of Points done so far = ',(idone-1
     +        ),' ( ',((float(idone)-1.)/float(jdone)*100),'% )'
cccccccccccccccccccccccccccc
c
c           get correlation
c
            if (lcorr) then
cccccccccccc
c
c             loop over the different segments
c
              do i5=1,ifold
c
c               get average for this segment together with array j
c               (contains the grid points for each atom)
c
                call average(i1,i2,i3,i5,ngrid,num,aver(i5),map,j,ngrid1
     +            ,ngrid2,ngrid3,rgrid)
c
                if (ldebug) write(iout,'(a,4i4,f10.5)')
     +            ' i1,i2,i3,i5,aver(i5) = ',i1,i2,i3,i5,aver(i5)
c
c               check if <rho>.lt.rcut
c
                if (aver(i5).lt.rcut) then
                  kdone=kdone+1
                  rxy=0.
                  goto 10
                endif
c
c               clear arrays
c
                sx(i5)=0.
                do i6=(i5+1),ifold
                  sxy(i5,i6)=0.
                enddo
              enddo
c
c             end loop over segments
c
cccccccccccc
c
c             get pairwise correlation
c
              do i4=1,npts,iskip
                do i5=1,ifold
                  sx(i5)=sx(i5)+(map(j(i5,i4,1),j(i5,i4,2),j(i5,i4,3))
     +              -aver(i5))**2
                  do i6=(i5+1),ifold
                    sxy(i5,i6)=sxy(i5,i6)+((map(j(i5,i4,1),j(i5,i4,2)
     +                ,j(i5,i4,3))-aver(i5))*(map(j(i6,i4,1),j(i6,i4,2)
     +                ,j(i6,i4,3))-aver(i6)))
                  enddo
                enddo
              enddo
c 
              do i5=1,ifold
                sx(i5)=sqrt((1./(float(num)-1.))*sx(i5))
              enddo
c
              nn=0
              rxy=0.
c
c             correlations 1->2 1->3 1->4 ... 1->ifold
c                               2->3 2->4 ... 2->ifold
c                                    3->4 ... 3->ifold
c                                      .  ... .
c
              do i5=1,ifold
                do i6=(i5+1),ifold
                  nn=nn+1
                  sxy(i5,i6)=(1./(float(num)-1.))*sxy(i5,i6)
                  rxy=rxy+(sxy(i5,i6)/(sx(i5)*sx(i6)))
                enddo
              enddo
c
c             average of all these correlations
c
              rxy=rxy/float(nn)
c
c ignore correlations of 1.0
c
              if (rxy.ge.0.99) then
                rxy = 0.
                srxy = 0.
                goto 10
              end if
c 
c             get standard deviation of this correlation if lsmapout=
c             .true.
c
              if (lsmapout) then
                srxy=0.
                do i5=1,ifold
                  do i6=(i5+1),ifold
                    srxy=srxy+((sxy(i5,i6)/(sx(i5)*sx(i6)))-rxy)**2
                  enddo
                enddo
                srxy=sqrt( (1./(float(nn)-1.))*srxy )
              endif
c
c             is this a new maximum ?
c
              lpackprint=.false.
              if (rxy.gt.rxymx) then
                f1=rr1/rgrid(1)
                f2=rr2/rgrid(2)
                f3=rr3/rgrid(3)
                call cvfrac2(f1,f2,f3,r1,r2,r3,0)
                if (lsmapout) then
                  if (lpack) then
                    write(iout,'(a,3i4,a,3f8.3,a,f7.3,a,f4.3,a,$)'
     +                )' New maximum at ',(i1-1),(i2-1),(i3-1),
     +                ' (',r1,r2,r3,' ) = ',rxy,' (+- ',srxy,')'
                  else
                    write(iout,'(a,3i4,a,3f8.3,a,f7.3,a,f4.3,a)'
     +                )' New maximum at ',(i1-1),(i2-1),(i3-1),
     +                ' (',r1,r2,r3,' ) = ',rxy,' (+- ',srxy,') <=='
                  endif
                else
                  if (lpack) then
                    write(iout,'(a,3i4,a,3f8.3,a,f7.3,$)'
     +                )' New maximum at ',(i1-1),(i2-1),(i3-1),
     +                ' (',r1,r2,r3,' ) = ',rxy
                  else
                    write(iout,'(a,3i4,a,3f8.3,a,f7.3,a)'
     +                )' New maximum at ',(i1-1),(i2-1),(i3-1),
     +                ' (',r1,r2,r3,' ) = ',rxy,' <=='
                  endif
                endif
                lpackprint=.true.
                rxymx=rxy
              else
c 
c               is correlation.gt.report ?
c
                f1=rr1/rgrid(1)
                f2=rr2/rgrid(2)
                f3=rr3/rgrid(3)
                call cvfrac2(f1,f2,f3,r1,r2,r3,0)
                if (rxy.gt.report) then
                  lpackprint=.true.
                  if (lsmapout) then
                    if (lpack) then
                      write(iout,'(a,3i4,a,3f8.3,a,f7.3,a,f4.3,a,$)'
     +                  )' Correlation at ',(i1-1),(i2-1),(i3-1),' (',r1
     +                  ,r2,r3,' ) = ',rxy,' (+- ',srxy,')'
                    else
                      write(iout,'(a,3i4,a,3f8.3,a,f7.3,a,f4.3,a)'
     +                  )' Correlation at ',(i1-1),(i2-1),(i3-1),' (',r1
     +                  ,r2,r3,' ) = ',rxy,' (+- ',srxy,')'
                    endif
                  else
                    if (lpack) then
                      write(iout,'(a,3i4,a,3f8.3,a,f7.3,$)'
     +                  )' Correlation at ',(i1-1),(i2-1),(i3-1),' (',r1
     +                  ,r2,r3,' ) = ',rxy
                    else
                      write(iout,'(a,3i4,a,3f8.3,a,f7.3)'
     +                  )' Correlation at ',(i1-1),(i2-1),(i3-1),' (',r1
     +                  ,r2,r3,' ) = ',rxy
                    endif
                  endif
                endif
              endif
            endif
cccccccccccccccccccccccccccc
c
c           get packing
c
            if (lpack) then
              call chkpack(i1,i2,i3,ngrid,asulim,asumap,pack,ldebug
     +          ,asulim11,asulim12,asulim13,asulim21,asulim22,asulim23)
              if (lpackprint) write(iout,'(a,f6.4)') ' ; PP = ',pack
            endif
c 
c           store correlation in nap
c 
 10         if (lmapout) then
              nap(i1,i2,i3)=rxy
            else if (lsmapout) then
              if (srxy.gt.1.) srxy=1.
              nap(i1,i2,i3)=rxy*(1.-srxy/rxy)
            endif
            if (lpack) nap(i1,i2,i3)=nap(i1,i2,i3)*pack
            napsum=napsum+nap(i1,i2,i3)
            napnum=napnum+1
c
c           do we want a CC-weighted map to be written out
c
            if (lwmapout.and.(rxy.ge.ccmin).and.(lmapout.or.lsmapout))
     +        then
              if (ldebug) write(iout,'(a)') ' calling wmapit'
              call wmapit(i1,i2,i3,iweight,scale,rxy,j,wmap,wmapmax
     +          ,ngrid,map,ngrid1,ngrid2,ngrid3)
              if (ldebug) write(iout,'(a)') ' leaving wmapit'

            else
              if (lwmapout.and.((.not.lmapout).and.(.not.lsmapout)))
     +          write(iout,'(/,a,/)'
     +          )' Sorry, you have to write MAPOUT too'
     +          ,' if you want WMAPOU'
            endif
c
c           end loop over XYZL
c
          enddo
        enddo
      enddo
      napave=napsum/float(napnum)
      write(iout,'(/,a,f6.3,a,i6,a)') ' <Correlation> = ',napave
     +  ,' ( for ',napnum,' points )'
c
c     some output
c
      write(iout,'(a,i20,a,f6.1,a)') ' # of Points done = ',idone
      if (minden.gt.-999.) then
        write(iout,'(a,i20,a,f6.1,a)')
     +    ' # of Points skipped because <density>.lt.(minden*rms) = '
     +    ,kdone
      endif
c
c     do we want to check along stick
c
      if (lstick) then
        write(iout,'(/,a)') ' Weighting o/p map by stick-method'
        call stickit(ngrid,nap,map,rxymx,ngrid1,ngrid2,ngrid3)
      endif
c
c     do we want a CC-map to be written out ?
c
      if (lmapout.or.lsmapout) then
        call wmaphd(iwmap,'MAPOUT',1)
        if (istep.gt.1) write(iout,'(a,/,a,/,a)')
     +    ' Running Interpolation since istep.ne.1'
     +    ,' This is not a recommended use of GETAX !!'
     +    ,' Better to use istep=1 (which will take longer).'
        do i3=1,ngrid(3)
          nn=0
          do i2=1,ngrid(2)
            do i1=1,ngrid(1)
              if ((i1.lt.(xyzl(1)+1)).or.(i1.gt.(xyzl(2)+1)).or.(i2.lt
     +          .(xyzl(3)+1)).or.(i2.gt.(xyzl(4)+1)).or.(i3.lt.(xyzl(5)
     +          +1)).or.(i3.gt.(xyzl(6)+1))) then
                if (abs(nap(i1,i2,i3)).lt.0.00001) nap(i1,i2,i3)=napave
              endif
              nn=nn+1
c
c             do we have to interpolate ?
c
              if ((istep.gt.1).or.lxyzin) then
                call interpol(i1,i2,i3,ngrid,nap,ngrid1,ngrid2,ngrid3)
              endif
c
              oap(nn)=nap(i1,i2,i3)
            enddo
          enddo
c
c         write out MAPOUT
c
          call mspew(iwmap,oap)
        enddo
c
        call mwclose(iwmap)
      endif
c
c     write out WMAPOU
c
      if (lwmapout) then
        if (iweight.eq.2) then
          scale=100./wmapmax
          if (scale.gt.100000.) scale=100000.
        else
          scale=1.
        endif
        call resetmo
        call wmaphd(12,'WMAPOU',2)
        do i3=1,ngrid(3)
          nn=0
          do i2=1,ngrid(2)
            do i1=1,ngrid(1)
              nn=nn+1
              woap(nn)=scale*wmap(i1,i2,i3)
            enddo
          enddo
          call mspew(12,woap)
        enddo
        call mwclose(12)
      endif
c
c     getn mean and rms of o/p map
c
      napmean=sngl(sumrho+offstr)
      if (sumrh2.gt.0.) then
        naprms=sqrt(sngl(sumrh2))
      else
        naprms=0.
      endif
c
c     refine top solutions
c
      if (lrefine) then
        write(iout,'(a)')
     +    ' Sorry, but REFINE option not yet implemented !'
      endif
c
      call sortpk(nap,ngrid,napmean,naprms,jpeak,ngrid1,ngrid2,ngrid3)
c
      return
      end
      subroutine average(ix,iy,iz,jfold,ngrid,num,aver,map,j,ngrid1
     +  ,ngrid2,ngrid3,rgrid)
c 
c i/p:
c ix, iy, iz [integer]    = grid point where body is to be centered
c jfold      [integer]    = # of segment to look at
c ngrid      [integer(3)] = grid sampling
c 
c o/p:
c num        [integer]    = # of grid points in segment used to compute aver
c aver       [real]       = average density in this segment
c j          [integer(*,*,3)] = coordinates in grid points for atoms
c
      implicit none
c      
      integer ifold,npts,iskip,num,j,i,ngrid(3),jfold,istep,maxfold
     +  ,maxpts,ii,ix,iy,iz,ngrid1,ngrid2,ngrid3,fac(3)
      real ssum,map(ngrid1,ngrid2,ngrid3),aver,x,minden,rj(3),rgrid(3)
      logical lang2
c
      parameter (maxfold=8)
      parameter (maxpts=20000)
c
      dimension j(maxfold,maxpts,3),x(maxfold,maxpts,3)
c
      common /body/ ifold,npts,x,lang2
      common /cpusav/ iskip,istep,minden
c
      ssum = 0.
       num = 0
c
      do i=1,npts,iskip
cv
c        j(jfold,i,1)=nint(x(jfold,i,1))+ix
c        j(jfold,i,2)=nint(x(jfold,i,2))+iy
c        j(jfold,i,3)=nint(x(jfold,i,3))+iz
cv
c
        rj(1) = x(jfold,i,1) + ix
        rj(2) = x(jfold,i,2) + iy
        rj(3) = x(jfold,i,3) + iz
c
        do ii=1,3
          do while (rj(ii).le.0.) 
            rj(ii) = rj(ii) + rgrid(ii)
          end do
          do while (rj(ii).gt.rgrid(ii)) 
            rj(ii) = rj(ii) - rgrid(ii)
          end do
          if (rj(ii).eq.0.) rj(ii) = rj(ii) + rgrid(ii)
cv
c          fac(ii)=int(float(abs(j(jfold,i,ii)))/float(ngrid(ii)))
c          if (j(jfold,i,ii).le.0) then
c            j(jfold,i,ii) = j(jfold,i,ii) + (fac(ii)+1) * ngrid(ii)
c          else if (j(jfold,i,ii).gt.ngrid(ii)) then
c            j(jfold,i,ii) = j(jfold,i,ii) -  fac(ii)    * ngrid(ii)
c          endif
cc
cc         just to make sure
cc
c          if (j(jfold,i,ii).eq.0) j(jfold,i,ii)=j(jfold,i,ii)+ngrid(ii)
cv
          j(jfold,i,ii) = int(rj(ii))
        enddo
        ssum=ssum+map(j(jfold,i,1),j(jfold,i,2),j(jfold,i,3))
        num=num+1
      enddo
      aver=ssum/float(num)
c
      return
      end
      subroutine chkpack(ix,iy,iz,ngrid,asulim,asumap,pack
     +  ,ldebug,asulim11,asulim12,asulim13,asulim21,asulim22
     +  ,asulim23)
c 
c i/p:
c ix, iy, iz [integer]    = grid point where body is to be centered
c ngrid      [integer(3)] = grid sampling
c 
c o/p:
c pack       [real]       = packing
c     = 0.0 for complete overlap of all symmetry equivalents
c     = 1.0 for no overlap of all symmetry equivalents
c
      implicit none
c      
      integer asulim(2,3),ix,iy,iz,ngrid(3),maxfold,maxpts,nsym,ifold
     +  ,npts,y(3),cen(3),asumap,iout,i,j,k,l1,l2,jj,i1,i2,i3,mins
     +  ,nfound,mpts,mout,nover,asulim11,asulim12,asulim13,asulim21
     +  ,asulim22,asulim23
      real rot(4,4,96),x,yy(3),pack
      logical lang2,ldebug
c
      parameter (maxfold=8)
      parameter (maxpts=20000)
      parameter (iout=6)
c
      dimension asumap(asulim11:asulim21,asulim12:asulim22
     +  ,asulim13:asulim23),x(maxfold,maxpts,3)
c
      common /packit/ nsym,rot
      common /body/ ifold,npts,x,lang2
c
c
c clear some arrays
c
      do i1=asulim(1,1),asulim(2,1)
        do i2=asulim(1,2),asulim(2,2)
          do i3=asulim(1,3),asulim(2,3)
            asumap(i1,i2,i3)=0
          enddo
        enddo
      enddo
c
      cen(1)=ix-1
      cen(2)=iy-1
      cen(3)=iz-1
c
c loop over segments -> points -> NSym
c
      mpts=0
      mout=0
      mins=0
      nover=0
      do i=1,ifold
        do j=1,npts
          mpts=mpts+1
          do k=1,nsym
c
c generate symmetry related point
c
            do l1=1,3
              if (ldebug) write(iout,*) 'k=',k,'x=',x(i,j,1),x(i,j,2)
     +          ,x(i,j,3),'cen=',cen
              if (k.ne.1) then
                yy(l1)=0.
                do l2=1,3
                  yy(l1)=yy(l1)+((x(i,j,l1)+float(cen(l1)))
     +              /float(ngrid(l1)))*rot(l2,l1,k)
                enddo
                y(l1)=nint((yy(l1)+rot(l1,4,k))*float(ngrid(l1)))
              else
                y(l1)=nint((x(i,j,l1)+float(cen(l1))))
              endif
              if (ldebug) write(iout,'(a,i1,a,i4,$)') 'y(',l1,') = '
     +          ,y(l1)
              do jj=1,2
                if (y(l1).ge.ngrid(l1)) y(l1)=y(l1)-ngrid(l1)
                if (y(l1).lt.0) y(l1)=y(l1)+ngrid(l1)
              enddo
              if (ldebug) write(iout,'(a,i4)') ' --> ',y(l1)
c out of asymmetric unit ?
              if ((y(l1).lt.asulim(1,l1)).or.(y(l1).gt.asulim(2,l1)))
     +          then 
                mout=mout+1
                goto 100
              endif
            enddo
            if (asumap(y(1),y(2),y(3)).eq.1) nover=nover+1
            asumap(y(1),y(2),y(3))=asumap(y(1),y(2),y(3))+1
            mins=mins+1
 100        continue
          enddo
        enddo
      enddo
c
c
      if (ldebug) write(iout,'(a,i10)')
     +  ' # of points in all segments    = ',mpts
c
      nfound=0
      do i1=asulim(1,1),asulim(2,1)
        do i2=asulim(1,2),asulim(2,2)
          do i3=asulim(1,3),asulim(2,3)
            if (asumap(i1,i2,i3).gt.0) nfound=nfound+1
          enddo
        enddo
      enddo
c
      if (ldebug) write(iout,'(a,i10)')
     +  ' # of points in asymmetric unit = ',nfound
      if (ldebug) write(iout,'(a,i10)')
     +  ' # of points outside asymmetric unit = ',mout
      if (ldebug) write(iout,'(a,i10)')
     +  ' # of points inside asymmetric unit = ',mins
      if (ldebug) write(iout,'(a,i10)')
     +  ' # of overlaps in asymmetric unit = ',nover
c
      if (nfound.gt.0) then
        pack=1.0-float(nover)/float(nfound)
      else
        write(iout,'(a,2i10)') 'ERROR: nover,nfound = ',nover,nfound
      end if
c
      return
      end
      subroutine wmapit(ix,iy,iz,iweight,scale,rxy,j,wmap,wmapmax,ngrid
     +  ,map,ngrid1,ngrid2,ngrid3)
c
      implicit none
c      
      integer iweight,j,ngrid(3),ifold,npts,maxfold,maxpts,ix,iy,iz
     +  ,i,ii,ngrid1,ngrid2,ngrid3
      real scale,rxy,wmap(ngrid1,ngrid2,ngrid3),wmapmax,x,weight
     +  ,rr,map(ngrid1,ngrid2,ngrid3),r1,r2,r3,f1,f2,f3
      logical lang2
c
      external cvfrac2
c
      parameter (maxfold=8)
      parameter (maxpts=20000)
c
      dimension x(maxfold,maxpts,3),j(maxfold,maxpts,3)
c
      common /body/ ifold,npts,x,lang2
c
      do i=1,npts
        do ii=1,ifold
          weight=scale*rxy
          if (iweight.eq.2) weight=scale*rxy*rxy
          if (iweight.eq.3) weight=scale*abs(rxy)
          if (iweight.eq.4) then
            f1=float(j(ii,i,1)-ix)/float(ngrid(1))
            f2=float(j(ii,i,2)-iy)/float(ngrid(2))
            f3=float(j(ii,i,3)-iz)/float(ngrid(3))
            call cvfrac2(f1,f2,f3,r1,r2,r3,0)
            rr=sqrt(r1**2+r2**2+r3**2)
            weight=scale*rxy*(1./rr)
          endif
c
          if ((j(ii,i,1).eq.0).or.(j(ii,i,2).eq.0).or.(j(ii,i,3).eq.0))
     +      then
            call ccperr(1,'ERROR in s/r WMAPIT')
          endif
c
          wmap(j(ii,i,1),j(ii,i,2),j(ii,i,3))=wmap(j(ii,i,1),j(ii,i,2)
     +      ,j(ii,i,3))+weight*map(j(ii,i,1),j(ii,i,2),j(ii,i,3))
          if (wmap(j(ii,i,1),j(ii,i,2),j(ii,i,3)).gt.wmapmax) wmapmax
     +      =wmap(j(ii,i,1),j(ii,i,2),j(ii,i,3))
        enddo
      enddo
c
      return
      end
      subroutine stickit(ngrid,nap,map,cmax,ngrid1,ngrid2,ngrid3)
c
      implicit none
c
      integer ngrid(3),nstick,maxpts,xyzl(6),i1,i2,i3,i4,i7,j(3)
     +  ,ngrid1,ngrid2,ngrid3
      real stick,nap,ave,sum,cmax,map,w,wsum,x1,x2,x3,y1,y2,y3,d,f1,f2
     +  ,f3
      logical lax1,lax2,lax3,lax4
c
      external cvfrac2
c
      parameter (maxpts=20000)
c
      dimension stick(maxpts,3),nap(ngrid1,ngrid2,ngrid3),map(ngrid1
     +  ,ngrid2,ngrid3)
c
      common /axisstick/ nstick,stick
      common /axisopt/ lax1,lax2,lax3,lax4
      common /scan/ xyzl
c
      if (cmax.eq.0.) cmax=1.
c
      do i1=(xyzl(1)+1),(xyzl(2)+1),1
        do i2=(xyzl(3)+1),(xyzl(4)+1),1
          do i3=(xyzl(5)+1),(xyzl(6)+1),1
c
            f1=float(i1-1)/float(ngrid(1))
            f2=float(i2-1)/float(ngrid(2))
            f3=float(i3-1)/float(ngrid(3))
            call cvfrac2(f1,f2,f3,x1,x2,x3,0)
            sum=0.
            ave=0.
            wsum=0.
            do i4=1,nstick
              j(1)=nint(stick(i4,1))+i1-1
              j(2)=nint(stick(i4,2))+i2-1
              j(3)=nint(stick(i4,3))+i3-1
              if (lax4) then
                f1=float(j(1))/float(ngrid(1))
                f2=float(j(2))/float(ngrid(2))
                f3=float(j(3))/float(ngrid(3))
                call cvfrac2(f1,f2,f3,y1,y2,y3,0)
                d=sqrt((x1-y1)**2+(x2-y2)**2+(x3-y3)**2)
                if (d.gt.0.) then
                  w=1./d
                else
                  w=1.
                endif
              else
                w=1.
              endif
              do i7=1,3
                if (j(i7).le.0) j(i7)=j(i7)+ngrid(i7)
                if (j(i7).gt.ngrid(i7)) j(i7)=j(i7)-ngrid(i7)
              enddo
              sum=sum+w*nap(j(1),j(2),j(3))
              wsum=wsum+w
            enddo
            if ((wsum.gt.0.).and.(.not.lax4)) then
              ave=sum/wsum
            else
              ave=sum
            endif
c
            if (lax1) then
              map(j(1),j(2),j(3))=ave
            else if (lax2) then
              map(j(1),j(2),j(3))=ave*nap(j(1),j(2),j(3))
            else if (lax3) then
              map(j(1),j(2),j(3))=(ave/cmax)*nap(j(1),j(2),j(3))
            else if (lax4) then
              map(j(1),j(2),j(3))=ave
            endif
c
          enddo
        enddo
      enddo
c
      do i1=(xyzl(1)+1),(xyzl(2)+1),1
        do i2=(xyzl(3)+1),(xyzl(4)+1),1
          do i3=(xyzl(5)+1),(xyzl(6)+1),1
c
            nap(i1,i2,i3)=map(i1,i2,i3)
c
          enddo
        enddo
      enddo

c
      return
      end
      subroutine wmaphd(iunit,filnam,itype)
c
      implicit none
c      
      character title*80,filnam*(*)
      integer nsec,iuvw(3),mxyz(3),nw1,nu1,nu2,nv1,nv2,lspgrp,iunit
     +  ,itype
      real cell(6)
c
      external mwrhdl,msyput
c
      common /mapin/ nsec,iuvw,mxyz,nw1,nu1,nu2,nv1,nv2,lspgrp
      common /cel/ cell
c
      if (itype.eq.1) then
        title='map with correlation coefficients written by GETAX'
      else if (itype.eq.2) then
        title='map with cc-weighted values written by GETAX'
      endif
c
c these maps will always be P1 !!
c
      lspgrp=1
c
      call mwrhdl(iunit,filnam,title,nsec,iuvw,mxyz,nw1,nu1,nu2,nv1
     +  ,nv2,cell,lspgrp,2)
      call msyput(99,lspgrp,iunit)
c
      return
      end
      subroutine interpol(ix,iy,iz,ngrid,nap,ngrid1,ngrid2,ngrid3)
c
      implicit none
c      
      integer iskip,istep,ngrid(3),j1,j2,j3,jj1,jj2,jj3,iix,iiy,iiz,ix
     +  ,iy,iz,iaddx,iaddy,iaddz,iout,ngrid1,ngrid2,ngrid3
      real minden,nap(ngrid1,ngrid2,ngrid3),ss,rr,r1,r2,r3,f1,f2
     +  ,f3
c
      external ccperr,cvfrac2
c
      common /cpusav/ iskip,istep,minden
c
      iout=6
c
c misset for each grid
c
      iix=mod((ix-1),istep)
      if (iix.ne.0) then
        iaddx=istep
      else
        iaddx=0
      endif
      iiy=mod((iy-1),istep)
      if (iiy.ne.0) then
        iaddy=istep
      else
        iaddy=0
      endif
      iiz=mod((iz-1),istep)
      if (iiz.ne.0) then
        iaddz=istep
      else
        iaddz=0
      endif
      if ((iix.ne.0).or.(iiy.ne.0).or.(iiz.ne.0)) then
        if (abs(nap(ix,iy,iz)).gt.1.) then
          write(iout,*) ix,iy,iz,iix,iiy,iiz,iaddx,iaddy,iaddz,nap(ix,iy
     +      ,iz)
          call ccperr(1,' nap(ix,iy,iz) unknown : something wrong ??')
        else
          nap(ix,iy,iz)=0.
        endif
        ss=0.
c 
        do j1=(ix-iix),(ix+iaddx-iix),istep
          jj1=j1
          if (jj1.le.0) jj1=jj1+ngrid(1)
          if (jj1.gt.ngrid(1)) jj1=jj1-ngrid(1)
c 
          do j2=(iy-iiy),(iy+iaddy-iiy),istep
            jj2=j2
            if (jj2.le.0) jj2=jj2+ngrid(2)
            if (jj2.gt.ngrid(2)) jj2=jj2-ngrid(2)
c 
            do j3=(iz-iiz),(iz+iaddz-iiz),istep
              f1=float(ix-j1)/ngrid(1)
              f2=float(iy-j2)/ngrid(2)
              f3=float(iz-j3)/ngrid(3)
              call cvfrac2(f1,f2,f3,r1,r2,r3,0)
              jj3=j3
              if (jj3.le.0) jj3=jj3+ngrid(3)
              if (jj3.gt.ngrid(3)) jj3=jj3-ngrid(3)
              rr=1/(sqrt(r1**2+r2**2+r3**2))
              ss=ss+rr
              nap(ix,iy,iz)=nap(ix,iy,iz)+rr*nap(jj1,jj2,jj3)
            enddo
          enddo
        enddo
        nap(ix,iy,iz)=nap(ix,iy,iz)/ss
      else
      endif
c
      return
      end
      subroutine resetmo
c
      implicit none
c      
      REAL AMAX,AMEAN,AMIN,ARMS
      INTEGER ISPG,ITMHDR,ITMSC1,LSKFLG,MODE,NC,NC1,NCHITM,NLAB,NR,NR1,
     +        NS,NS1,NSYMBT
      REAL CEL,SKWMAT,SKWTRN
      INTEGER JUNK,LABELS,MAPCRS,NXYZ,machst,mapst
      DOUBLE PRECISION SUMRHO, SUMRH2, OFFSTR
c 
      COMMON /MOHDR/NC,NR,NS,MODE,NC1,NR1,NS1,NXYZ(3),CEL(6),MAPCRS(3),
     +  AMIN,AMAX,AMEAN,ISPG,NSYMBT,LSKFLG,SKWMAT(3,3),SKWTRN(3),
     +  JUNK(15),MAPST,MACHST(1),ARMS,NLAB,LABELS(20,10),NCHITM,
     +  ITMHDR,ITMSC1
      COMMON /MOHSUM/  SUMRHO, SUMRH2, OFFSTR
c
      SAVE /MOHDR/,/MOHSUM/
c
c reset rhomin,rhomax,rhomean,rhorms:
c
      amin=0.
      amax=0.
      amean=0.
      arms=0.
c
c reset spacegroup to P1:
c
      ispg=1
c
      return
      end
      subroutine sortpk(nap,ngrid,mean,rms,jpeak,ngrid1,ngrid2,ngrid3)
c
      implicit none
c
      integer ngrid(3),maxpeak,xyzl(6),ipeak,i1,i2,i3,npeak,grid1,i,j,k
     +  ,iout,jpeak,j1,j2,j2end,ii,jj,ngrid1,ngrid2,ngrid3
      real nap,mean,rms,peak1,f1,f2,f3,y1,y2,y3,ang(3),ang2(3),rotmat1(3
     +  ,3),rotmat2(3,3),rotmat3(3,3),o(3),t(3),radius(2),height,w(3)
      logical lang2,lsphere,lslice,lxyzout,lgxyzout
c
      external cvfrac2
c
      parameter (iout=6)
      parameter (maxpeak=100000)
c
      dimension nap(ngrid1,ngrid2,ngrid3),peak1(maxpeak)
     +  ,ipeak(maxpeak),grid1(maxpeak,3)
c
      common /scan/ xyzl
      common /sphere/ lsphere,radius,lslice,height,ang,lxyzout,lang2
     +  ,ang2,lgxyzout
c
      npeak=0
      do i1=(xyzl(1)+1),(xyzl(2)+1),1
        do i2=(xyzl(3)+1),(xyzl(4)+1),1
          do i3=(xyzl(5)+1),(xyzl(6)+1),1
            if (((nap(i1,i2,i3)-mean)/rms).gt.3.) then
              npeak=npeak+1
              if (npeak.gt.maxpeak) then
                write(iout,'(/,a,a,/)')
     +            ' WARNING: parameter MAXPEAK too small!!'
     +            ,' Increase MAXPEAK in s/r SORTPK.'
                npeak=npeak-1
                goto 10
              endif
              peak1(npeak)=nap(i1,i2,i3)
              grid1(npeak,1)=i1-1
              grid1(npeak,2)=i2-1
              grid1(npeak,3)=i3-1
            endif
          enddo
        enddo
      enddo
 10   call qsorti(ipeak,npeak,peak1)
c
      write(iout,'(//,a,/a,//,a,a,a)') ' Correlations sorted by height:'
     +  ,' =============================='
     +  ,'   #    Corr       Grid             Coord(frac)'
     +  ,'             Coord(ortho)    |'
     +  ,'     Rotation matrix             Translation X,Y,Z'
      do i=1,jpeak
        j=npeak-i+1
        f1=float(grid1(ipeak(j),1))/float(ngrid(1))
        f2=float(grid1(ipeak(j),2))/float(ngrid(2))
        f3=float(grid1(ipeak(j),3))/float(ngrid(3))
        call cvfrac2(f1,f2,f3,y1,y2,y3,0)
        o(1)=y1
        o(2)=y2
        o(3)=y3
        do j1=1,nint(360./ang(3))
          if (lang2) then
            j2end=nint(360./ang2(3))
          else
            j2end=1
          endif
          w(1)=ang(1)
          w(2)=ang(2)
          w(3)=(j1-1)*ang(3)
          call p2m(w,rotmat1)
          do j2=1,j2end
            w(1)=ang2(1)
            w(2)=ang2(2)
            w(3)=(j2-1)*ang2(3)
            call p2m(w,rotmat2)
            do ii=1,3
              do jj=1,3
                rotmat3(ii,jj)=rotmat2(ii,1)*rotmat1(1,jj)+rotmat2(ii,2)
     +            *rotmat1(2,jj)+rotmat2(ii,3)*rotmat1(3,jj)
              enddo
            enddo
            call oax2t(rotmat3,o,t)
            if ((j1.eq.2).and.(j2.eq.1)) then
              write(iout,100) i,peak1(ipeak(j)),(grid1(ipeak(j),k),k=1,3
     +          ),f1,f2,f3,y1,y2,y3,(rotmat3(1,ii),ii=1,3),t
     +          ,((rotmat3(jj,ii),ii=1,3),jj=2,3)
            else
              if ((j1.ne.1).or.(j2.ne.1)) then
                write(iout,110) (rotmat3(1,ii),ii=1,3),t,((rotmat3(jj
     +            ,ii),ii=1,3),jj=2,3)
              endif
            endif
          enddo
        enddo
 100    format(i4,2x,f6.3,2x,3i4,2x,3f7.3,2x,3f8.3,1x,'|',1x,3f9.5
     +    ,1x,3f8.2,2(/,76x,'|',1x,3f9.5))
 110    format(76x,'|',1x,3f9.5,1x,3f8.2,2(/,76x,'|',1x,3f9.5))
        write(iout,'(76x,a)') '|'
      enddo
c
      return
      end
C From HDK@psuvm.psu.edu Thu Dec  8 15:27:16 MST 1994
C 
C The following was converted from Algol recursive to Fortran iterative
C by a colleague at Penn State (a long time ago - Fortran 66, please
C excuse the GoTo's). The following code also corrects a bug in the
C Quicksort algorithm published in the ACM (see Algorithm 402, CACM,
C Sept. 1970, pp 563-567; also you younger folks who weren't born at
C that time might find interesting the history of the Quicksort
C algorithm beginning with the original published in CACM, July 1961,
C pp 321-322, Algorithm 64). Note that the following algorithm sorts
C integer data; actual data is not moved but sort is affected by sorting
C a companion index array (see leading comments). The data type being
C sorted can be changed by changing one line; see comments after
C declarations and subsequent one regarding comparisons(Fortran
C 77 takes care of character comparisons of course, so that comment
C is merely historical from the days when we had to write character
C compare subprograms, usually in assembler language for a specific
C mainframe platform at that time). But the following algorithm is
C good, still one of the best available.
      SUBROUTINE QSORTI (ORD,N,B)
C
      IMPLICIT NONE
C
C==============SORTS THE ARRAY A(I),I=1,2,...,N BY PUTTING THE
C   ASCENDING ORDER VECTOR IN ORD.  THAT IS ASCENDING ORDERED A
C   IS A(ORD(I)),I=1,2,...,N; DESCENDING ORDER A IS A(ORD(N-I+1)),
C   I=1,2,...,N .  THIS SORT RUNS IN TIME PROPORTIONAL TO N LOG N .
C
C
C     ACM QUICKSORT - ALGORITHM #402 - IMPLEMENTED IN FORTRAN 66 BY
C                                 WILLIAM H. VERITY, WHV@PSUVM.PSU.EDU
C                                 CENTER FOR ACADEMIC COMPUTING
C                                 THE PENNSYLVANIA STATE UNIVERSITY
C                                 UNIVERSITY PARK, PA.  16802
C
      INTEGER N, ORD
C
      DIMENSION ORD(N),POPLST(2,20)
      INTEGER X,XX,Z,ZZ,Y
C
      INTEGER I, IP, IQ, IX, IZ, L, L1, NDEEP, P, POPLST, Q, U, U1
     .  , YP
C
C     TO SORT DIFFERENT INPUT TYPES, CHANGE THE FOLLOWING
C     SPECIFICATION STATEMENTS; FOR EXAMPLE, FOR FORTRAN CHARACTER
C     USE THE FOLLOWING:  CHARACTER *(*) A(N)
C
      INTEGER A,maxpeak
      REAL B(N)
c
      parameter (maxpeak=100000)
      dimension a(maxpeak)
c
      do i=1,n
        a(i)=nint(1000.*b(i))
      enddo
C
      NDEEP=0
      U1=N
      L1=1
      DO 1  I=1,N
    1 ORD(I)=I
    2 IF (U1.LE.L1) goto 500
C
    3 L=L1
      U=U1
C
C PART
C
    4 P=L
      Q=U
C     FOR CHARACTER SORTS, THE FOLLOWING 3 STATEMENTS WOULD BECOME
C     X = ORD(P)
C     Z = ORD(Q)
C     IF (A(X) .LE. A(Z)) GO TO 2
C
C     WHERE "CLE" IS A LOGICAL FUNCTION WHICH RETURNS "TRUE" IF THE
C     FIRST ARGUMENT IS LESS THAN OR EQUAL TO THE SECOND, BASED ON "LEN"
C     CHARACTERS.
C
      X=A(ORD(P))
      Z=A(ORD(Q))
      IF (X.LE.Z) GO TO 5
      Y=X
      X=Z
      Z=Y
      YP=ORD(P)
      ORD(P)=ORD(Q)
      ORD(Q)=YP
    5 IF (U-L.LE.1) GO TO 15
      XX=X
      IX=P
      ZZ=Z
      IZ=Q
C
C LEFT
C
    6 P=P+1
      IF (P.GE.Q) GO TO 7
      X=A(ORD(P))
      IF (X.GE.XX) GO TO 8
      GO TO 6
    7 P=Q-1
      GO TO 13
C
C RIGHT
C
    8 Q=Q-1
      IF (Q.LE.P) GO TO 9
      Z=A(ORD(Q))
      IF (Z.LE.ZZ) GO TO 10
      GO TO 8
    9 Q=P
      P=P-1
      Z=X
      X=A(ORD(P))
C
C DIST
C
   10 IF (X.LE.Z) GO TO 11
      Y=X
      X=Z
      Z=Y
      IP=ORD(P)
      ORD(P)=ORD(Q)
      ORD(Q)=IP
   11 IF (X.LE.XX) GO TO 12
      XX=X
      IX=P
   12 IF (Z.GE.ZZ) GO TO 6
      ZZ=Z
      IZ=Q
      GO TO 6
C
C OUT
C
   13 CONTINUE
      IF (.NOT.(P.NE.IX.AND.X.NE.XX)) GO TO 14
      IP=ORD(P)
      ORD(P)=ORD(IX)
      ORD(IX)=IP
   14 CONTINUE
      IF (.NOT.(Q.NE.IZ.AND.Z.NE.ZZ)) GO TO 15
      IQ=ORD(Q)
      ORD(Q)=ORD(IZ)
      ORD(IZ)=IQ
   15 CONTINUE
      IF (U-Q.LE.P-L) GO TO 16
      L1=L
      U1=P-1
      L=Q+1
      GO TO 17
   16 U1=U
      L1=Q+1
      U=P-1
   17 CONTINUE
      IF (U1.LE.L1) GO TO 18
C
C START RECURSIVE CALL
C
      NDEEP=NDEEP+1
      POPLST(1,NDEEP)=U
      POPLST(2,NDEEP)=L
      GO TO 3
   18 IF (U.GT.L) GO TO 4
C
C POP BACK UP IN THE RECURSION LIST
C
      IF (NDEEP.EQ.0) GO TO 2
      U=POPLST(1,NDEEP)
      L=POPLST(2,NDEEP)
      NDEEP=NDEEP-1
      GO TO 18
C
C END SORT
C END QSORT
C
 500  continue
      do i=1,n
        b(i)=float(a(i))/1000.
      enddo
c
      return
      END
      subroutine ortpol(ang1,lang2,ang2,nncode,ngrid,lxyzin)
c
c             [Rncode=j] [XOncode=j]  =  [Rncode=i][XOncode=i]
c
c                  and   [XOncode=i] = [ROncode=i] [RFncode=j] [XOncode=j]
c
c        so   [Rncode=j]  = [Rncode=i] [ROncode=i] [RFncode=j]
c
      implicit none
c
      real ang(2,3),ang1(3),ang2(3),mat(3,3),ro(4,4),rf(4,4),cell(6)
     +  ,rotog(4,4),rgtoo(4,4),rff1(4,4),roo1(4,4),rff2(4,4),roo2(4,4)
     +  ,mmat(10,3,3),rff2t(3,3),det,rff21(3
     +  ,3)
      logical lang2,lxyzin
      integer nncode,ncode,i,j,iout,ngrid(3),ibrkfl,ii
c
      parameter (iout=6)
c
      common /cel/ cell
      common /orthog/ ro,rf,ncode,ibrkfl
      common /dencode/ roo1,rff1,roo2,rff2
c
      ncode=nncode
c
      write(iout,'(/,a,/)') ' s/r ORTPOL:'
      if (.not.lxyzin) then
        write(iout,'(a,3f8.2)') ' Polar Angle     = ',ang1
        if (lang2) write(iout,'(a,3f8.2)') ' Polar Angle (2) = ',ang2
      endif
      if (nncode.ne.1) write(iout,'(/,a,i2,a,/)')
     +  ' Transforming polar angles from NCODE=',ncode,' to NCODE=1'
c
      call ortmat(cell,ngrid,ro,rf,rotog,rgtoo,ncode)
      call rbrorf(ro,rf)
c
      write(iout,'(//,a,i1,a)') ' Orthogonalization matrix (NCODE='
     +  ,ncode,'): '
      write(iout,'(4f12.7)') ((ro(i,j),j=1,4),i=1,4)
      write(iout,'(/,a,i1,a)') ' Fractionalization matrix (NCODE=',ncode
     +  ,'): '
      write(iout,'(4f12.7)') ((rf(i,j),j=1,4),i=1,4)
c
      do i=1,4
        do j=1,4
          roo1(i,j)=ro(i,j)
          rff1(i,j)=rf(i,j)
        enddo
      enddo
c
      ncode=1
c
      call ortmat(cell,ngrid,ro,rf,rotog,rgtoo,ncode)
      call rbrorf(ro,rf)
c
      if (nncode.ne.1) then
        write(iout,'(//,a,i1,a)') ' Orthogonalization matrix (NCODE='
     +    ,ncode,'): '
        write(iout,'(4f12.7)') ((ro(i,j),j=1,4),i=1,4)
        write(iout,'(/,a,i1,a)') ' Fractionalization matrix (NCODE='
     +    ,ncode,'): '
        write(iout,'(4f12.7)') ((rf(i,j),j=1,4),i=1,4)
      endif
c
      do i=1,4
        do j=1,4
          roo2(i,j)=ro(i,j)
          rff2(i,j)=rf(i,j)
        enddo
      enddo
      do i=1,3
        do j=1,3
          rff21(i,j)=rf(i,j)
        enddo
      enddo
c
      do i=1,3
        ang(1,i)=ang1(i)
        ang(2,i)=ang2(i)
      enddo
      do ii=1,2
        if ((ii.eq.2.).and.(.not.lang2)) goto 100
        ang1(1)=ang(ii,1)
        ang1(2)=ang(ii,2)
        ang1(3)=ang(ii,3)
c 
        if (.not.lxyzin) then
          call p2m(ang1,mat)
        else
          mat(1,1)=1.
          mat(1,2)=0.
          mat(1,3)=0.
          mat(2,1)=0.
          mat(2,2)=1.
          mat(2,3)=0.
          mat(3,1)=0.
          mat(3,2)=0.
          mat(3,3)=1.
        endif
c
        write(iout,'(//,a,i1,a)') '    Rotation matrix for NCODE='
     +    ,nncode,':'
        write(iout,'(3f12.7)') ((mat(i,j),j=1,3),i=1,3)
        if (nncode.ne.1) then
c 
          do i=1,3
            do j=1,3
              mmat(1,i,j)=roo1(i,1)*rff2(1,j)+roo1(i,2)*rff2(2,j)
     +          +roo1(i,3)*rff2(3,j)
            enddo
          enddo
c 
          do i=1,3
            do j=1,3
              mmat(2,i,j)=mat(i,1)*mmat(1,1,j)+mat(i,2)*mmat(1,2,j)
     +          +mat(i,3)*mmat(1,3,j)
            enddo
          enddo
c 
          do i=1,3
            do j=1,3
              mmat(3,i,j)=rff1(i,1)*mmat(2,1,j)+rff1(i,2)*mmat(2,2,j)
     +          +rff1(i,3)*mmat(2,3,j)
            enddo
          enddo
c 
          call minv3(rff2t,rff21,det)
c 
          do i=1,3
            do j=1,3
              mat(i,j)=rff2t(i,1)*mmat(3,1,j)+rff2t(i,2)*mmat(3,2,j)
     +          +rff2t(i,3)*mmat(3,3,j)
            enddo
          enddo
c 
c
          write(iout,'(/,a)') 'New rotation matrix for NCODE=1:'
          write(iout,'(3f12.7)') ((mat(i,j),j=1,3),i=1,3)
c
          if (.not.lxyzin) then
            call m2p(mat,ang1)
          else
            ang1(1)=0.
            ang1(2)=0.
            ang1(3)=0.
          endif
          ang(ii,1)=ang1(1)
          ang(ii,2)=ang1(2)
          ang(ii,3)=ang1(3)
c
        endif
      enddo
      do i=1,3
        ang1(i)=ang(1,i)
        ang2(i)=ang(2,i)
      enddo
c
 100  if (.not.lxyzin) then 
        write(iout,'(/,a,3f8.2)') ' Polar Angle    = ',ang1
        if (lang2) write(iout,'(a,3f8.2)') ' Polar Angle (2) = ',ang2
      endif
c
      return
      end
      
      subroutine oax2t(r,o,t)
c
c converts rotation/translation of type:
c     (1)     [B] = ([A]-[O])*[R] + [O]
c to rotation/translation of type:
c     (2)     [B] = [A]*[R] + [T]
c
c so: [T] = -[O]*[R] + [O]
c
c (1) = known rotation matrix [R] and origin [O] of rotation axis ( = GETAX
c       definition )
c (2) = known rotation matrix [R] and translation [T] ( = DM/PDBSET
c       /LSQKAB/... defintition )
c
c       | r11 r12 r13 |
c [R] = | r21 r22 r23 |  , not the "O" matrix ( = transformed ) !!!
c       | r31 r32 r33 |
c
c if you want to get [R],[T] from [R],[O] ( = going from GETAX to DM,
c e.g.), you can i/p any value for [A].
c
c (c) 1996 Clemens Vonrhein      vonrhein@bio5.chemie.uni-freiburg.de
c
      implicit none
c
      real r(3,3),o(3),t(3)
c
      t(1)=-o(1)*r(1,1)-o(2)*r(2,1)-o(3)*r(3,1)+o(1)
      t(2)=-o(1)*r(1,2)-o(2)*r(2,2)-o(3)*r(3,2)+o(2)
      t(3)=-o(1)*r(1,3)-o(2)*r(2,3)-o(3)*r(3,3)+o(3)
c
      return
      end
