C     This code is distributed under the terms and conditions of the
C     CCP4 licence agreement as `Part ii)' software.  See the conditions
C     in the CCP4 manual for a copyright statement.
C
C     The CCP4 version of ACORN is for the determination of protein structure
C     using native data at atomic resolution starting from one of follows:
C
C     *** Known phases from any sourse;
C     *** random atoms;
C     *** known heavy atoms;
C     *** Standard alpha helices;
C     *** fragment from protein data bank;
C     *** fragment from AMORE after rotation and/or translational  function.
C
C     If the fragment is not at right orientation and/or position  ACORN will
C     carry out Molecular Replacement by finding the highest correlation
C     coefficient between observed structure factors and calculated one from
C     the fragment.
C
C     A set of initial phases will be obtained from this fragmemnt.
C
C     A real space Sayre equation, Patterson superposition and  Dynamic Density
C     Modification (DDM) will be combined to develop and refine the  initial
C     phases. A figure of merit (CC) will indicate the best solution and
C     phases with weights will be in output MTZ file.
C
C     The details are in the document acorn.html.
C
C     Modified by Yao Jia-xing. 1st of Jan. 2006.
C
C     *******************
      PROGRAM ACORN
C     *******************
C
      INTEGER NW1,NW2,NW3,NW4
C
      CHARACTER TYPE (4)
      INTEGER LENDEF (4)
      DATA TYPE /'I','R','R','R'/
C     External Routines in CCP4.
      EXTERNAL CCPFYP,MTZINI,XYZINIT,CCPRCS,CCPERR,CCPALC
C     External Routines for ACORN.
      EXTERNAL DACORN
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/NSIZE/NUMFO,LIMBUF,LPAR,NURO,INDMAX,CELLB,NWEK,NBIG
     +     ,LBUFT,LPART
C
C
      PI = 4.0*ATAN(1.0)
      TWOPI = 2.0*PI
      DTOR = PI/180.0
      RTOD = 180.0/PI
C
      CALL CCPFYP
      CALL MTZINI
      CALL XYZINIT
      CALL CCPRCS(6,'ACORN','$Date: 2006/12/13 09:26:10 $')
C
C     Read keywords and MTZ infomation.
      CALL RKEYSD
C     Size of arrays.
      WRITE(6,FMT='(/,4(A,I12),/,2(A,I12)/)') ' NUMFO =',
     +     NUMFO,' LIMBUF =',LIMBUF,' LPAR =',LPAR,' NURO =',NURO,
     +     ' LPART =',LPART,'  LBUFT =',LBUFT
      LENDEF(1) = 6*NUMFO
      LENDEF(2) = 8*NUMFO
      LENDEF(3) = 10*NUMFO
C     Memory for PHASE.
      NW1 = 11*NUMFO + 4*LIMBUF
C     Memory for rotation function.
      NW2 = 2*LPAR + 6*NURO
C     Memory for random translational function.
      NW3 = 2*LPAR
C     Memory for translational function.
      NW4 = 2*LPART + 5*NURO + 2*LBUFT
      LENDEF(4) = NW1
      IF (NW2.GT.LENDEF(4)) LENDEF(4) = NW2
      IF (NW3.GT.LENDEF(4)) LENDEF(4) = NW3
      IF (NW4.GT.LENDEF(4)) LENDEF(4) = NW4
C     Call DACORN.
      CALL CCPALC(DACORN,4,TYPE,LENDEF)
C
      WRITE(6,FMT='(//)')
      CALL CCPERR(0,' Normal termination')
      STOP
      END
C
      SUBROUTINE DACORN(NIP,IIP,NAP,AIP,NOU,AOU,NWK,AWK)
C     Determine the size of arrays.
C     COMMON variables.
      COMMON/NSIZE/NUMFO,LIMBUF,LPAR,NURO,INDMAX,CELLB,NWEK,NBIG
     +     ,LBUFT,LPART
      COMMON/IDX/ NI1,NI2,NI3,NI4,NI5,NI6,NA1,NA2,NA3,NA4,NA5,NA6,NA7,NA8
     +     ,NU1,NU2,NU3,NU4,NU5,NU6,NU7,NK1,NK2,NK3,NK4,NK5,NK6,NK7,NK8
     +     ,NK9,NK10
C
      INTEGER NIP,NAP,NOU,NWK
      INTEGER IIP(NIP)
      REAL AIP(NAP),AOU(NOU),AWK(NWK)
C     NIP=6*NUMFO
      NI1 = 1
      NI2 = NUMFO + NI1
      NI3 = NUMFO + NI2
      NI4 = NUMFO + NI3
      NI5 = NUMFO + NI4
      NI6 = NUMFO + NI5
C     NAP=8*NUMFO
      NA1 = 1
      NA2 = NUMFO + NA1
      NA3 = NUMFO + NA2
      NA4 = NUMFO + NA3
      NA5 = NUMFO + NA4
      NA6 = NUMFO + NA5
      NA7 = NUMFO + NA6
      NA8 = NUMFO + NA7
C     NOU=8*NUMFO
      NU1=1
      NU2 = NUMFO + NU1
      NU3 = NUMFO + NU2
      NU4 = NUMFO + NU3
      NU5 = NUMFO + NU4
      NU6 = NUMFO + NU5
      NU7 = NUMFO + NU6
      NU8 = NUMFO + NU7
      NU9 = NUMFO + NU8
      NU10 = NUMFO + NU9
C
      CALL RACORN(IIP(NI1),IIP(NI2),IIP(NI3),IIP(NI4),IIP(NI5),IIP(NI6)
     +     ,AIP(NA1),AIP(NA2),AIP(NA3),AIP(NA4),AIP(NA5),AIP(NA6)
     +     ,AIP(NA7),AIP(NA8),AOU(NU1),AOU(NU2),AOU(NU3),AOU(NU4)
     +     ,AOU(NU5),AOU(NU6),AOU(NU7),AOU(NU8),AOU(NU9),AOU(NU10)
     +     ,NWK,AWK)
C
      RETURN
      END
C     ---------------------------------------------------------------
C     Read in keywords.
C
      SUBROUTINE RKEYSD
C
C     The program can handle up to 500 data items.
C
C     Output file contains input data plus
C
C      EOEXT               The observed and extrended magnitudes.
C      PHIOUT  WTOUT       The phases and weights of the best set
C                          from ACORN
C      ECOUT               The calculate magnitudes of the best set
C                          from ACORN
C
C     The labels are as follows:
C
C    -----------------------------------------------------------
C
C      Name           Item
C
C      H, K, L        Miller indices.
C
C     --------------   Input:
C
C      FP             Native protein data.
C      SIGFP          Sigma of FP.
C      E              Normalized structure factors.
C      PHIN           Known Phases.
C      WTIN           Weights for the known phases.
C      FCIN           Known magnitudes.
C
C     --------------   for calculation of phase errors.
C
C      FT             True structure factors.
C      PHIFT          True phases.
C
C     --------------   Output:
C
C      EOEXT          Observed plus extended E values.
C      PHIOUT         Refined phases for the best set.
C      WTOUT          Weights for the best set.
C      ECOUT          Map coefficient including weight.
C
C   ----------------------------------------------------------------
C
C     The column types are as follows:
C
C      H       index h,k,l.
C      F       structure factor FP
C      E       Normalized structure factor E, ECOUT or EOEXT.
C      D       anomalous difference.
C      Q       standard deviation of anything: J,F,D or other.
C      P       Phase.
C      W       Weight.
C
C    ---------------------------------------------------
C
C      I       any other integer
C      R       any other real
C
C              ******************************
C              Description of Key Words Input
C              ******************************
C
C     Parameters.
      INTEGER MCOLS,NKEYWD,NAMAX,MSETS
      PARAMETER (MCOLS=500,NKEYWD=200,NAMAX=30000,MSETS=100)
C
      REAL TEM1,TEM2,SIGCUT
C     Arrays for data.
      REAL ADATA,BDATA
      INTEGER LOOKUP(MCOLS)
      CHARACTER*1 CTPRGO,CTPRGI(MCOLS),CTUSRI
      CHARACTER*30 LSPRGI(MCOLS),LSPRGO,LSUSRI
      LOGICAL LOGMSS
C     MTZ dataset stuff
      INTEGER NDATASETS,SETID,SET_INDEX,ISETS(MSETS),CSETID(MCOLS)
      REAL DCELL(6,MSETS),DWAVE(MSETS)
      CHARACTER*64 PNAME(MSETS),XNAME(MSETS),DNAME(MSETS),
     +            XNAME_OUT,DNAME_OUT
C     Local Scalars.
      REAL SS4,XYZLIM(2,3)
      INTEGER IC,IFAIL,IFIL1,IFLAG,IPRINT
      INTEGER JDO230,JDO430
      INTEGER NCOL,NLPRGI,NSPGRP,NSYM,NSYMP,NTOK,NLPRGO,REFCOD
      INTEGER ICHKP,STARTFP,NUMKNO
      CHARACTER NAMSPG*10,PGNAM*10,TITLE*80
      LOGICAL ENDFIL,LTITLE
C
C     Variables for Parser.
      REAL FVALUE
      INTEGER IBEG,IDEC,IEND
      CHARACTER KEY*4,CVALUE*4,LINE*2000,LTYPE*1,ELEM*4,
     +     LABOUT_SAVE*2000,WORD*8
      LOGICAL LEND
C
C     External Routines.
      EXTERNAL CCPERR,CCPFYP,CCPRCS,CENTR,CENTRIC,EQUAL_MAGIC,LKYIN,
     +     LROPEN,LRASSN,LRCLAB,LRCLOS,LRREFM,LRSYMI,LRSYMM,INVSYM,
     +     LWOPEN,LWASSN,LWTITL,LWREFL,MTZINI,PARSER,GTPINT,CCPUPC,
     +     GTPREA,LRRSOL,LKYOUT,LRREFL,LWCLOS,LRNREF,LRREWD,LWHSTL,
     +     LRINFO,SETLIM,CALC_ORIG_PS
C
C     Variables for ACORN.
      CHARACTER*4 NW
      CHARACTER*10 VERSNX
      LOGICAL LPAXISX,LPAXISY,LPAXISZ
      INTEGER NORIG
C     Variables for random generater.
      DOUBLE PRECISION IMR,IAR,ICR
C     COMMON variables.
      COMMON/NSIZE/NUMFO,LIMBUF,LPAR,NURO,INDMAX,CELLB,NWEK,NBIG
     +     ,LBUFT,LPART
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/ORIGJY/LPAXISX,LPAXISY,LPAXISZ,NORIG,ORIG(3,96)
      COMMON/ATMFACTOR/NKD,AL(8),AS(8),BL(8),BS(8),CL(8),CS(8)
     +,DL(8),DS(8),EL(8),NW(8),NO(8),NE(8)
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
      COMMON/MTZI/IC,IFIL1,IFLAG,IPRINT,JDO230,JDO430
     +     ,NCOL,NTOK,NLPRGO,REFCOD,NUMKNO
     +     ,IN(3),ICHKP(8),IBEG(NKEYWD),IDEC(NKEYWD),IEND(NKEYWD)
     +     ,ITYPE(NKEYWD)
      COMMON/MTZR/SIGCUT,SS4
     +     ,ADATA(MCOLS),BDATA(MCOLS),FVALUE(NKEYWD)
      COMMON/MTZC/CTPRGO(MCOLS),CTUSRI(MCOLS)
     +     ,LSPRGO(MCOLS),LSUSRI(MCOLS),CVALUE(NKEYWD)
     +     ,NAMSPG,TITLE,KEY,LINE,ELEM,LABOUT_SAVE
     +     ,XNAME_OUT(MCOLS),DNAME_OUT(MCOLS)
      COMMON/MTZL/LOGMSS(MCOLS),ENDFIL,LTITLE,LEND
C     Eulerian angles.
      COMMON/EULER/EU1M,EU2M,EU3M,
     +     EU1(5000),EU2(5000),EU3(5000),EUCC(5000),NSEU(5000)
     +     ,TR1(5000),TR2(5000),TR3(5000),TRCS(5000),NSTR(5000)
C     Translation function.
      COMMON/TRXYZ/MAXHT,MAXKT,MAXLT,NXT,NYT,NZT
     +     ,TRX(5000),TRY(5000),TRZ(5000),TRCC(5000)
      COMMON/XYZIN/NUMA,OC(NAMAX),IZ(NAMAX),XINF(NAMAX),YINF(NAMAX)
     +     ,ZINF(NAMAX),XINO(NAMAX),YINO(NAMAX),ZINO(NAMAX),BFAC(NAMAX)
      COMMON/PDBROF/RMO(4,4),RMF(4,4)
      COMMON/RGEN/IMR,IAR,ICR
C     Local.
      INTEGER NCOLX,ITEMP
      REAL SMIN,SMAX,RANGES(2,200)
C     *************************************************************
      DATA ECUT/5.0/,EMIN/0.0/,EWEK/0.1/,SS4LO/0.0001/,CUTT/3.0/
     +     ,SS4HO/1.0/,GRID/0.3333/,MAXH/0/,MAXK/0/,MAXL/0/,IMSG/0/
     +     ,SS4ROT/1.0/,ASX/1.0/,ASY/1.0/,ASZ/1.0/,SS4LE/0.0001/
     +     ,SS4HE/1.0/
      DATA MAXSET/100/,NEMAX/0/,NEMIN/0/,NSUP/-1/,MODR/5/,MODT/5/
     +     ,NSER/500*-1/,NDDM/500*500/,NRRT/0/,INIT/0/,ISEED/1/,NROT/0/
     +     ,NWEK/0/,NEWEK/0/,NUMKNO/0/,NBIG/0/,NKD/0/,RAD1/4.0/
     +     ,NPOS/0/,NATM/0/,PFIN/0.5/,CFIN/0.0/,STEP/3.0/
     +     ,MKER/0/,NKNO/0/,NTRA/0/,NSROT/1/,TXM/0.0/,TYM/0.0/,TZM/0.0/
     +     ,NSOL/0/,NRTF/0/,NENH/500*1/,NDDMK/500*-1/,NKEXT/1/
     +     ,SOLV/0.50/,CCDDM1/0.12/,CCDDM2/0.20/
      DATA NOX,NOY,NOZ/128,128,128/
     +     ,NTRY/0/,NFRA/0/,NSTA/1/
      DATA IMR,IAR,ICR/1048576,9301,49297/
      DATA NUMFO,LIMBUF,LPAR,NURO/0,0,0,0/
      DATA LBUFT,LPART/0,0/
C     Data Statements.
      DATA NLPRGI,NLPRGO/11,4/
      DATA ICHKP/8*0/
      DATA SIGCUT/0.0/
      DATA LTITLE/.FALSE./
      DATA LSPRGI/'H','K','L','FP', 'SIGFP',
     +     'E',    'PHIN',      'WTIN',
     +     'FT',      'PHIFT',   'FCIN',
     +     489*' '/
C
      DATA CTPRGI/'H','H','H','F','Q',
     +     'F','P','W','F','P','F',
     +     489*' '/
C
      DATA LSPRGO/'EOEXT','PHIOUT','WTOUT','ECOUT',496*' '/
C
      DATA CTPRGO/'F','P','W','F',496*' '/
C
      DATA LOOKUP/-1,-1,-1,497*0/
C     Initial OC,IZ.
      DO N = 1, NAMAX
         OC(N) = 1.0
         IZ(N) = 8
      END DO
C     Starting indices of program label blocks
      STARTFP = 3
      IFIL1 = 1
      LABOUT_SAVE = ' '
C     Set default value.
      NENH(1) = 0
C     Read keyword input.
  20   CONTINUE
      KEY  = ' '
      LINE = ' '
C     Reset number of words on line to NKEYWD
      NTOK = NKEYWD
C
C     **********************************************************
      CALL PARSER (KEY,LINE,IBEG,IEND,ITYPE,FVALUE,CVALUE,IDEC,NTOK
     +     ,LEND,.TRUE.)
C     **********************************************************
C
      IF (LEND) GO TO 100
      IF (KEY.EQ.'TITL') THEN
C     Read in new title.
         IF (NTOK.GE.2) THEN
            LTITLE = .TRUE.
            TITLE = LINE(IBEG(2) :IEND(NTOK))
            WRITE(6,'(10X,A,/,15X,A80,/)')
     +          'TITLE OF MTZ FILE REPLACED BY:',TITLE
         END IF
         GO TO 20
C
      ELSE IF (KEY.EQ.'LABI') THEN
C     Input labels
C     *****************************************
         CALL LKYIN(IFIL1,LSPRGI,NLPRGI,NTOK,LINE,IBEG,IEND)
C     *****************************************
         IF (NTOK.EQ.1) CALL CCPERR(1,
     +       'No columns assigned. Nothing to do. Bye-bye!')
         WRITE(6,'(10X,I3,X,A,/,10X,A,A,/)')
     +       (NTOK-1)/2, 'column labels have been assigned.',
     +       'NOTE: missing number analysis only performed ',
     +       'for these columns!!'
         GO TO 20
C
C     Content of unit cell.
      ELSE IF (KEY.EQ.'CONT') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
 25      CONTINUE
         IF (ITYPE(ITOK).EQ.2) CALL CCPERR(1,
     +       ' *** Error - should have symbol for element  ***')
C
C     Next symbol.
C
         ELEM = LINE(IBEG(ITOK) :IEND(ITOK))
         CALL CCPUPC(ELEM)
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
C     Read the number of element.
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for element. ***')
         NKD = NKD + 1
         NW(NKD) = ELEM
C     **********************************
         CALL GTPINT(ITOK,NO(NKD),NTOK,ITYPE,FVALUE)
C     **********************************
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
         GO TO 25
C
C     Save output labels.
      ELSE IF (KEY.EQ.'LABO') THEN
         LABOUT_SAVE = LINE
         GO TO 20
C
C     Sigma cutoff.
      ELSE IF (KEY.EQ.'EXCL') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for EXCL ***')
C     **********************************
         CALL GTPREA(ITOK,SIGCUT,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Grid point for MAP.
      ELSE IF (KEY.EQ.'GRID') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for GRID ***')
C     **********************************
         CALL GTPREA(ITOK,GRID,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Maximum E value in phase refinement.
      ELSE IF (KEY.EQ.'ECUT') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for ECUT ***')
C     **********************************
         CALL GTPREA(ITOK,ECUT,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Minimum E value for strong E values.
      ELSE IF (KEY.EQ.'ESTR') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for ESTRONG ***')
C     **********************************
         CALL GTPREA(ITOK,EMIN,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Maximum E value for weak reflections.
      ELSE IF (KEY.EQ.'EWEA') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for EWEAK ***')
C     **********************************
         CALL GTPREA(ITOK,EWEK,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Cut top density for DDM Method.
      ELSE IF (KEY.EQ.'CUTD') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for CUTD ***')
C     **********************************
         CALL GTPREA(ITOK,CUTT,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Phase shift to indicates the end of each try.
      ELSE IF (KEY.EQ.'PSFI') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for PSFIN ***')
C     **********************************
         CALL GTPREA(ITOK,PFIN,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     CC's to indicates the end of DDM.
      ELSE IF (KEY.EQ.'CCFI') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have second number for CCFIN ***')
C     **********************************
         CALL GTPREA(ITOK,CFIN,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Solvent region.
      ELSE IF (KEY.EQ.'SOLV') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have second number for SOLVENT ***')
C     **********************************
         CALL GTPREA(ITOK,SOLV,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     CC for introduce DDM1 automatically.
      ELSE IF (KEY.EQ.'CCDD') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have second number for CCDDM1 ***')
C     **********************************
         CALL GTPREA(ITOK,CCDDM1,NTOK,ITYPE,FVALUE)
C     **********************************
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have second number for CCDDM2 ***')
C     **********************************
         CALL GTPREA(ITOK,CCDDM2,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Resolution range of FP or E values.
      ELSE IF (KEY.EQ.'RESO') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a first number for RESO ***')
C     **********************************
         CALL GTPREA(ITOK,TEM1,NTOK,ITYPE,FVALUE)
C     **********************************
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have second number for RESO ***')
C     **********************************
         CALL GTPREA(ITOK,TEM2,NTOK,ITYPE,FVALUE)
C     **********************************
         ACUT1 = AMAX1(TEM1,TEM2)
         ACUT2 = AMIN1(TEM1,TEM2)
         SS4LO = 1.0/(ACUT1*ACUT1)
         SS4HO = 1.0/(ACUT2*ACUT2)
         GO TO 20
C
C     No extended reflections will be used.
      ELSE IF (KEY.EQ.'NOEX') THEN
         NKEXT = 0
         GO TO 20
C     Resolution range of extended reflections.
      ELSE IF (KEY.EQ.'EXTE') THEN
         NKEXT = 1
         SS4LE = 0.0001
         SS4HE = 1.0
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a first number for EXTE ***')
C     **********************************
         CALL GTPREA(ITOK,TEM1,NTOK,ITYPE,FVALUE)
C     **********************************
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have second number for EXTE ***')
C     **********************************
         CALL GTPREA(ITOK,TEM2,NTOK,ITYPE,FVALUE)
C     **********************************
         ACUT1 = AMAX1(TEM1,TEM2)
         ACUT2 = AMIN1(TEM1,TEM2)
         SS4LE = 1.0/(ACUT1*ACUT1)
         SS4HE = 1.0/(ACUT2*ACUT2)
         GO TO 20
C
C     Maximum number of phase sets to be generated.
      ELSE IF (KEY.EQ.'MAXS') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for MAXS ***')
C     **********************************
         CALL GTPINT(ITOK,MAXSET,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Maximum number of sets for random rotate function.
      ELSE IF (KEY.EQ.'RROT') THEN
         NRRT = 50000
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for NRRT ***')
C     **********************************
         CALL GTPINT(ITOK,NRRT,NTOK,ITYPE,FVALUE)
C     **********************************
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for MODR ***')
C     **********************************
         CALL GTPREA(ITOK,TEM1,NTOK,ITYPE,FVALUE)
C     **********************************
         MODR = INT(100.0/TEM1 + 0.5)
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for RESOLUTION ***')
C     **********************************
         CALL GTPREA(ITOK,TEM1,NTOK,ITYPE,FVALUE)
C     **********************************
         SS4ROT = 1.0/(TEM1*TEM1)
         GO TO 20
C
C     The number of sets for Known fragment or atoms.
      ELSE IF (KEY.EQ.'POSI') THEN
         NPOS = 1
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for POSI ***')
C     **********************************
         CALL GTPINT(ITOK,NPOS,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Superposition use of Sum function.
      ELSE IF (KEY.EQ.'SUPP') THEN
         NSUP = 1
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for SUPP ***')
C     **********************************
         CALL GTPINT(ITOK,NSUP,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Rotation function.
      ELSE IF (KEY.EQ.'ROTF') THEN
         NROT = 1
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for STEP ***')
C     **********************************
         CALL GTPREA(ITOK,STEP,NTOK,ITYPE,FVALUE)
C     **********************************
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for MODR ***')
C     **********************************
         CALL GTPREA(ITOK,TEM1,NTOK,ITYPE,FVALUE)
C     **********************************
         MODR = INT(100.0/TEM1 + 0.5)
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for RES2 ***')
C     **********************************
         CALL GTPREA(ITOK,TEM1,NTOK,ITYPE,FVALUE)
C     **********************************
         SS4ROT = 1.0/(TEM1*TEM1)
         GO TO 20
C     Parameter for translational function.
      ELSE IF (KEY.EQ.'TRAN') THEN
         NTRA = 1
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for NSROT ***')
C     **********************************
         CALL GTPINT(ITOK,NSROT,NTOK,ITYPE,FVALUE)
         GOTO 20
C
C     Parameter for random translational function.
      ELSE IF (KEY.EQ.'RTRA') THEN
         NRTF = 50000
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for RTRA ***')
C     **********************************
         CALL GTPINT(ITOK,NRTF,NTOK,ITYPE,FVALUE)
C     **********************************
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for MODT ***')
C     **********************************
         CALL GTPREA(ITOK,TEM1,NTOK,ITYPE,FVALUE)
C     **********************************
         MODT = INT(100.0/TEM1 + 0.5)
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for NSROT ***')
C     **********************************
         CALL GTPINT(ITOK,NSROT,NTOK,ITYPE,FVALUE)
         GOTO 20
C
C     Random number generater seed.
      ELSE IF (KEY.EQ.'SEED') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for SEED ***')
C     **********************************
         CALL GTPINT(ITOK,ISEED,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Radius for GS function to calculate average map and idea envelope.
      ELSE IF (KEY.EQ.'RADI') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for RAD1 ***')
C     **********************************
         CALL GTPREA(ITOK,RAD1,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Number of sets for random atoms.
      ELSE IF (KEY.EQ.'RATM') THEN
         NATM = 50000
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for RATM ***')
C     **********************************
         CALL GTPINT(ITOK,NATM,NTOK,ITYPE,FVALUE)
C     **********************************
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for MODR ***')
C     **********************************
         CALL GTPREA(ITOK,TEM1,NTOK,ITYPE,FVALUE)
C     **********************************
         MODR = INT(100.0/TEM1 + 0.5)
         GO TO 20
C
C     Number of reflections with strong E's.
      ELSE IF (KEY.EQ.'NSTR') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for NSTRONG ***')
C     **********************************
         CALL GTPINT(ITOK,NBIG,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Number of reflections with small E's.
      ELSE IF (KEY.EQ.'NWEA') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for NWEAK ***')
C     **********************************
         CALL GTPINT(ITOK,NWEK,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     The number of try in each phase set.
      ELSE IF (KEY.EQ.'NTRY') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for NTRY ***')
C     **********************************
         CALL GTPINT(ITOK,NTRY,NTOK,ITYPE,FVALUE)
C     **********************************
         IF (NTRY.GT.500) CALL CCPERR(1,
     +       ' *** Error - too large number of try ***')
         GO TO 20
C
C     The number of atoms in a fragment to be used.
      ELSE IF (KEY.EQ.'NAFR') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for NAFRAG ***')
C     **********************************
         CALL GTPINT(ITOK,NFRA,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     The first atom in a fragment.
      ELSE IF (KEY.EQ.'NSTA') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for NSTA ***')
C     **********************************
         CALL GTPINT(ITOK,NSTA,NTOK,ITYPE,FVALUE)
C     **********************************
         GO TO 20
C
C     Number of cycles for Dynamic Density Modification(DDM0).
      ELSE IF (KEY.EQ.'NCDD') THEN
         NC = 0
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
 35      IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for NCDDM ***')
         NC = NC + 1
         IF (NC.GT.500) CALL CCPERR(1,
     +       ' *** Error - too many number of try ***')
C     **********************************
         CALL GTPINT(ITOK,NDDM(NC),NTOK,ITYPE,FVALUE)
C     **********************************
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
         GOTO 35
C
C     Number of cycles for Sayre Equation refinement by FFT(SER).
      ELSE IF (KEY.EQ.'NCSE') THEN
         NC = 0
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
 40      IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for NCSER ***')
         NC = NC + 1
         IF (NC.GT.500) CALL CCPERR(1,
     +       ' *** Error - too many number of try ***')
C     **********************************
         CALL GTPINT(ITOK,NSER(NC),NTOK,ITYPE,FVALUE)
C     **********************************
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
         GOTO 40
C
C     Number of cycles for Enhance refinement(ENHS).
      ELSE IF (KEY.EQ.'ENHS') THEN
         NC = 0
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
 41      IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for ENHS ***')
         NC = NC + 1
         IF (NC.GT.500) CALL CCPERR(1,
     +       ' *** Error - too many number of try ***')
C     **********************************
         CALL GTPINT(ITOK,NENH(NC),NTOK,ITYPE,FVALUE)
C     **********************************
         IF (NENH(NC).GT.1) NENH(NC) = 1
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
         GOTO 41
C
      ELSE IF (KEY.EQ.'DDMK') THEN
         NC = 0
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
C
 42      IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for DDMK ***')
         NC = NC + 1
         IF (NC.GT.500) CALL CCPERR(1,
     +       ' *** Error - too many number of try ***')
C     **********************************
         CALL GTPINT(ITOK,NDDMK(NC),NTOK,ITYPE,FVALUE)
C     **********************************
         IF (NDDMK(NC).GT.2) NDDMK(NC) = 0
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
         GOTO 42
C
C     Previous results from ACORN or input by user.
      ELSE IF (KEY.EQ.'INIT') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
         INIT = INIT + 1
         NSEU(INIT) = INIT
         NC = 0
 45      NC = NC + 1
         IF (NC.GE.8) GO TO 20
         IF (NC.EQ.1) THEN
            IF (ITYPE(ITOK).EQ.2) CALL CCPERR(1,
     +          ' ***Error - should have word **')
            WORD = LINE(IBEG(ITOK) :IEND(ITOK))
            CALL CCPUPC(WORD)
            IF ( WORD(1:4) .EQ.'RATM') THEN
               IMSG = 1
               NATM = INIT
            END IF
            IF ( WORD(1:4) .EQ.'POSI') THEN
               IMSG = 2
               NPOS = INIT
            END IF
            IF ( WORD(1:4) .EQ.'RROT') THEN
               IMSG = 3
               NRRT = INIT
            END IF
            IF ( WORD(1:4) .EQ.'ROTF') THEN
               IMSG = 4
               NROT = INIT
            END IF
            IF ( WORD(1:4) .EQ.'RTRA') THEN
               IMSG = 5
               NRTF = INIT
            END IF
            IF ( WORD(1:4) .EQ.'TRAN') THEN
               IMSG = 6
               NTRA = INIT
            END IF
         ELSE
C
            IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +          ' *** Error - should have a number for INIT ***')
            IF (NC.EQ.2) THEN
C     **********************************
               IF (NPOS.GT.0) THEN
                  CALL GTPINT(ITOK,ITEMP,NTOK,ITYPE,FVALUE)
                  EU1(INIT) = REAL(ITEMP)
                  GO TO 20
               END IF
               CALL GTPREA(ITOK,EUT,NTOK,ITYPE,FVALUE)
            ELSE
               CALL GTPREA(ITOK,EUT,NTOK,ITYPE,FVALUE)
            END IF
C     **********************************
            IF (NC.EQ.2) EU1(INIT) = EUT
            IF (NC.EQ.3) EU2(INIT) = EUT
            IF (NC.EQ.4) EU3(INIT) = EUT
            IF (NC.EQ.4.AND.NATM.GT.0) THEN
               XINF(INIT) = EU1(INIT)
               YINF(INIT) = EU2(INIT)
               ZINF(INIT) = EU3(INIT)
               GO TO 20
            END IF
            IF (NC.EQ.5) TR1(INIT) = EUT
            IF (NC.EQ.6) TR2(INIT) = EUT
            IF (NC.EQ.7) TR3(INIT) = EUT
         END IF
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
         GOTO 45
C
C     Previous solutions from CCP4-AMORE with default setting.
      ELSE IF (KEY.EQ.'SOLU') THEN
         ITOK = 2
         IF (ITOK.GT.NTOK) GO TO 20
         NSOL = NSOL + 1
         IMSG = 7
         NSEU(NSOL) = NSOL
         NC = 0
 46      NC = NC + 1
         IF (NC.GE.8) GO TO 20
C
         IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +       ' *** Error - should have a number for SOLU ***')
C     **********************************
         IF (NC.EQ.1) THEN
            CALL GTPINT(ITOK,ITEMP,NTOK,ITYPE,FVALUE)
         ELSE
            CALL GTPREA(ITOK,EUT,NTOK,ITYPE,FVALUE)
         END IF
C     **********************************
         IF (NC.EQ.2) EU1(NSOL) = EUT
         IF (NC.EQ.3) EU2(NSOL) = EUT
         IF (NC.EQ.4) EU3(NSOL) = EUT
         IF (NC.EQ.5) TR1(NSOL) = EUT
         IF (NC.EQ.6) TR2(NSOL) = EUT
         IF (NC.EQ.7) TR3(NSOL) = EUT
         ITOK = ITOK + 1
         IF (ITOK.GT.NTOK) GO TO 20
         GOTO 46
C     End of Keywords.
      ELSE IF (KEY.EQ.'END') THEN
         WRITE(6,'(/)')
         GO TO 100
      ELSE
         CALL CCPERR(1,' *** Error KEY word unrecognised ***')
      END IF
C
C     End of keyword input
C
 100  CONTINUE
C
      IPRINT = 2
C     **********************************
      CALL LROPEN(IFIL1,'HKLIN',IPRINT,IFAIL)
C     **********************************
      IF (IFAIL.NE.0) CALL CCPERR(1, ' *** Error No input file ***')
C
C     ****************************************
      CALL LRASSN(IFIL1,LSPRGI,NLPRGI,LOOKUP,CTPRGI)
C     *****************************************
C
C     *****************************************
      CALL LRCLAB(IFIL1,LSUSRI,CTUSRI,NCOL)
C     *****************************************
C
C     MTZ infomation.
C     *****************************************
      CALL LRINFO(IFIL1,VERSNX,NCOLX,NUMFO,RANGES)
C     *****************************************
C
C     Cell dimensions.
C     *****************************************
      CALL LRCELL(IFIL1,CELLP)
C     *****************************************
C     Notify available space and Read project and dat set names
      NDATASETS = MSETS
      CALL LRIDX(IFIL1,PNAME,XNAME,DNAME,ISETS,DCELL,DWAVE,NDATASETS)

C---- Get dataset ID for column and match to dataset header info.
      IF (NDATASETS.GT.0) THEN
        CALL LRCLID(IFIL1,CSETID,NCOL)

C     Get setid of FP column and position index of the set
        SETID = 0
        SET_INDEX = 0
        IF (LOOKUP(4).GT.0) THEN
          SETID = CSETID(LOOKUP(4))
          DO ISET = 1,NDATASETS
            IF (ISETS(ISET).EQ.SETID) THEN
              SET_INDEX = ISET
            END IF
          ENDDO
        ENDIF
C     Set output dataset and correct choice of cell
        IF (SET_INDEX.GT.0) THEN
          DO I = 1,NLPRGO
            XNAME_OUT(I) = XNAME(SET_INDEX)
            DNAME_OUT(I) = DNAME(SET_INDEX)
          ENDDO
          DO I=1,6
            CELLP(I) = DCELL(I,SET_INDEX)
          ENDDO
        ENDIF
      ENDIF
C
C     Symmetry information.
C     *****************************************
      CALL LRSYMI(IFIL1,NSYMP,LTYPE,NSPGRP,NAMSPG,PGNAM)
      IF (LTYPE.EQ.' ')
     +     CALL CCPERR(1, ' *** Error No symmetry avaliable.')
C     ****************************************
      CALL LRSYMM(IFIL1,NSYM,RSYML)
C     *****************************************
C     Set Asymmetry unit.
      CALL SETLIM(NSPGRP,XYZLIM)
      IF (XYZLIM(1,1).GT.-0.5) THEN
         ASX = XYZLIM(2,1)
         ASY = XYZLIM(2,2)
         ASZ = XYZLIM(2,3)
      END IF
      IF (NSYMP.EQ.1.AND.NATM.GT.0) NFRA = 1
C
C     Resolution range
C     *****************************************
      CALL LRRSOL(IFIL1,SMIN,SMAX)
C     *****************************************
C     Resolution range for all reflections.
      SS4LO = AMAX1(SMIN,SS4LO)
      SS4HO = AMIN1(SMAX,SS4HO)
      SS4LE = AMAX1(SMIN,SS4LE)
      SS4HE = AMIN1(SMAX,SS4HE)
C     Maximum index.
      RESHO = 1.0/SQRT(SS4HO)
      MAXH = INT(CELLP(1)/RESHO)
      MAXK = INT(CELLP(2)/RESHO)
      MAXL = INT(CELLP(3)/RESHO)
C
      MAXHT = 0
      MAXKT = 0
      MAXLT = 0
      DO I = 1, NSYMP
         LHI = NINT(RSYML(1,1,I))*MAXH
     +      + NINT(RSYML(2,1,I))*MAXK
     +      + NINT(RSYML(3,1,I))*MAXL
         LKI = NINT(RSYML(1,2,I))*MAXH
     +      + NINT(RSYML(2,2,I))*MAXK
     +      + NINT(RSYML(3,2,I))*MAXL
         LLI = NINT(RSYML(1,3,I))*MAXH
     +      + NINT(RSYML(2,3,I))*MAXK
     +      + NINT(RSYML(3,3,I))*MAXL
         DO J = 1, NSYMP
            LHJ = NINT(RSYML(1,1,J))*MAXH
     +         + NINT(RSYML(2,1,J))*MAXK
     +         + NINT(RSYML(3,1,J))*MAXL
            LKJ = NINT(RSYML(1,2,J))*MAXH
     +         + NINT(RSYML(2,2,J))*MAXK
     +         + NINT(RSYML(3,2,J))*MAXL
            LLJ = NINT(RSYML(1,3,J))*MAXH
     +         + NINT(RSYML(2,3,J))*MAXK
     +         + NINT(RSYML(3,3,J))*MAXL
            LH = LHI - LHJ
            LK = LKI - LKJ
            LL = LLI - LLJ
            IF (IABS(LH).GT.MAXHT) MAXHT = IABS(LH)
            IF (IABS(LK).GT.MAXKT) MAXKT = IABS(LK)
            IF (IABS(LL).GT.MAXLT) MAXLT = IABS(LL)
         END DO
      END DO
C     If extend the resolution.
      IF (NKEXT.EQ.1) THEN
         IF (SS4HE.GT.SS4HO) THEN
            RESHE = 1.0/SQRT(SS4HE)
            MAXH = INT(CELLP(1)/RESHE)
            MAXK = INT(CELLP(2)/RESHE)
            MAXL = INT(CELLP(3)/RESHE)
         END IF
      END IF
C     Check data to be used.
C     FP
      ICHKP(1) = LOOKUP(STARTFP+1)
C     SIGFP
      ICHKP(2) = LOOKUP(STARTFP+2)
C     E
      ICHKP(3) = LOOKUP(STARTFP+3)
C     Known phase
      ICHKP(4) = LOOKUP(STARTFP+4)
C     Weight of known phase
      ICHKP(5) = LOOKUP(STARTFP+5)
C     True FT.
      ICHKP(6) = LOOKUP(STARTFP+6)
C     True Phase PHIFT.
      ICHKP(7) = LOOKUP(STARTFP+7)
C     Known calculated magnitudes.
      ICHKP(8) = LOOKUP(STARTFP+8)
C
      IF (ICHKP(1).EQ.0.OR.ICHKP(3).EQ.0) THEN
         CALL CCPERR(1,' *** Error FP and E both must be input. ')
      END IF
      IF (ICHKP(4).GT.0) THEN
         NKNO = NKNO + 1
         IF (ICHKP(5).GT.0) THEN
            NKNO = NKNO + 2
         END IF
         IF (ICHKP(8).GT.0) THEN
            NKNO = NKNO + 4
         END IF
         IF (NKNO.EQ.1)
     +       WRITE (6,FMT='(/,A,/)')
     +       ' ACORN will use known phases with unit weights.'
         IF (NKNO.EQ.3)
     +       WRITE (6,FMT='(/,A,A,/)')
     +       ' ACORN will use known phases with input weights'
         IF (NKNO.GE.5)
     +       WRITE (6,FMT='(/,A,A,/)')
     +       ' ACORN will use known phases and magnitudes to ',
     +       'calculate weights.'
      END IF
C     Calculate phase errors.
      IF (ICHKP(7).GT.0) THEN
         MKER = 1
         WRITE (6,FMT='(/,A,/)')
     +       ' ACORN will calculate phase errors.'
      END IF
C     Check the range for random atom fragments and translational function.
      IF (NATM.GT.0.OR.NTRA.GT.0.OR.NRTF.GT.0.OR.MKER.GT.0) THEN
C     Obtain all possible origins.
         CALL  CALC_ORIG_PS
     +     (NAMSPG,NSYM,RSYML,NORIG,ORIG,LPAXISX,LPAXISY,LPAXISZ)
         IF (LPAXISX) THEN
            TXM = 0.0
         ELSE
            TEM1 = 0.0
            DO N = 1, NORIG
               TEM1 = AMAX1(TEM1,ORIG(1,N))
            END DO
            IF (TEM1.LT.0.001) THEN
               TXM = 1.0
            ELSE
               TXM = TEM1
            END IF
         END IF
         IF (LPAXISY) THEN
            TYM = 0.0
         ELSE
            TEM1 = 0.0
            DO N = 1, NORIG
               TEM1 = AMAX1(TEM1,ORIG(2,N))
            END DO
            IF (TEM1.LT.0.001) THEN
               TYM = 1.0
            ELSE
               TYM = TEM1
            END IF
         END IF
         IF (LPAXISZ) THEN
            TZM = 0.0
         ELSE
            TEM1 = 0.0
            DO N = 1, NORIG
               TEM1 = AMAX1(TEM1,ORIG(3,N))
            END DO
            IF (TEM1.LT.0.001) THEN
               TZM = 1.0
            ELSE
               TZM = TEM1
            END IF
         END IF
         IF (TXM.GT.0.01.AND.TYM.GT.0.01) THEN
            SXY = 0.0
            DO N = 1, NORIG
               SXY = SXY + ABS(ORIG(1,N) - ORIG(2,N))
            END DO
            IF (SXY.LT.0.10.AND.TXM.LT.0.60) TXM = 2.0*TXM
            IF (SXY.LT.0.10.AND.TXM.GT.0.60) TXM = 1.0
            IF (SXY.LT.0.10.AND.TYM.GT.0.60) TYM = 1.0/3.0
         END IF
C     Find values of NOX, NOY, NOZ.
         IF (NSYM.GT.1) THEN
            NOZ = 2
            IF (TZM.LT.0.01) NOZ = 1024
            NOX = 2
            IF (TXM.LT.0.01) NOX = 1024
            IF (TXM.GT.0.60.AND.TXM.LT.0.70) THEN
               IF (NOZ.EQ.2) THEN
                  NOX = 1024
               ELSE
                  NOX = 128
                  NOZ = 128
               END IF
            END IF
            NOY = 2
            IF (TYM.LT.0.01) NOY = 1024
            IF (TYM.GT.0.60.AND.TYM.LT.0.70) THEN
               IF (NOZ.EQ.2) THEN
                  NOY = 1024
               ELSE
                  NOY = 128
                  NOZ = 128
               END IF
            END IF
         END IF
      END IF
      IF (MKER.GT.0) WRITE(6,FMt='(/,A,3I8,/)')
     +     ' Origin shifts on X,Y and Z for phase error calculation'
     +     ,NOX,NOY,NOZ
C
C     Set up information to decide centric zones.
C
      CALL CENTRIC(NSYM,RSYML,IPRINT)
C     Maximum grid points for FFT.
      CALL CKMXYZ(GRID,LIMBUF,CELLP,VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL)
C     Calculate spherical scattering factor coefficients.
      CALL ATMCOEF
C     Calculate F000 and E000.
      CALL CFZERO
C     PDB operations.
      NCODE = 1
      CALL ORTMAT(CELLP,NCODE,RMO,RMF)
C     Check jobs.
      NTEST = 0
      IF (NATM.GT.0) NTEST = NTEST + 1
      IF (NPOS.GT.0) NTEST = NTEST + 1
      IF (NRRT.GT.0) NTEST = NTEST + 1
      IF (NROT.GT.0) NTEST = NTEST + 1
      IF (NKNO.GT.0) NTEST = NTEST + 1
      IF (NSOL.GT.0) NTEST = NTEST + 1
      IF (NTEST.GT.1) CALL CCPERR(1,' *** Error too many jobs. ***')
      IF (NTRA.GT.0) NTEST = NTEST + 1
      IF (NRTF.GT.0) NTEST = NTEST + 1
      IF (NTEST.GT.2) CALL CCPERR(1,' *** Error too many jobs. ***')
      IF (NTEST.EQ.0) CALL CCPERR(1,' *** Error No job specified ***')
C     Input fragment.
      IF (NPOS.GT.0.OR.NRRT.GT.0.OR.NROT.GT.0.OR.NTRA.GT.0
     +     .OR.NRTF.GT.0.OR.NSOL.GT.0) THEN
         CALL PDBIN(NPOS)
         IF (NFRA.GT.NUMA.OR.NFRA.EQ.0) NFRA = NUMA
         WRITE(6,FMT='(/,2(A,I5))')
     +       ' The number of atoms in a fragment = ',NFRA,
     +       ' start from',NSTA
      END IF
      IF (NROT.GT.0.OR.NRRT.GT.0.OR.NTRA.GT.0.OR.NRTF.GT.0
     +     .OR.NSOL.GT.0) THEN
C     Find the dimensions of fragment and the centre of mass,
C     if NSOL=0 shift the centre to the origin.
         CALL BIGCEL(CELLB,NFRA,SS4HO,INDMAX,NSTA,NSOL)
      END IF
C     Find LPAR.
      IF (NSOL.GT.0.OR.INIT.GT.0) THEN
         NURO = 0
         LPAR = 0
         LPART = 0
         IF (IMSG.EQ.3.OR.IMSG.EQ.4.OR.IMSG.EQ.7) THEN
            IF (NROT.GT.0.AND.NSROT.GT.NROT) NSROT = NROT
            IF (NRRT.GT.0.AND.NSROT.GT.NRRT) NSROT = NRRT
            IF (NSOL.GT.0.AND.NSROT.GT.NSOL) NSROT = NSOL
            IF (NTRA.GT.0) THEN
C     Maximum grid points for FFT in TRANSL.
               CALL CKMXYZ(GRID,LBUFT,CELLP,VOLT,NXT,NYT,NZT
     +             ,MAXHT,MAXKT,MAXLT)
               LPAR = (MAXH*2 + 1)*(MAXK*2 + 1)*(MAXL + 1)
               NURO = NUMFO*NSYMP
               LPART = (MAXHT*2 + 1)*(MAXKT*2 + 1)*(MAXLT + 1)
            END IF
            IF (NRTF.GT.0) THEN
               LPAR = (MAXH*2 + 1)*(MAXK*2 + 1)*(MAXL + 1)
            END IF
         END IF
      ELSE
         IF (NROT.GT.0.OR.NRRT.GT.0) THEN
            LPAR = (INDMAX*2 + 1)*(INDMAX*2 + 1)*(INDMAX + 1)
            NURO = NUMFO*NSYMP
         END IF
         IF (NRTF.GT.0) THEN
            LTEM = (MAXH*2 + 1)*(MAXK*2 + 1)*(MAXL + 1)
            IF (LTEM.GT.LPAR) LPAR = LTEM
         END IF
         IF (NTRA.GT.0) THEN
C     Maximum grid points for FFT in TRANSL.
            CALL CKMXYZ(GRID,LBUFT,CELLP,VOLT,NXT,NYT,NZT
     +          ,MAXHT,MAXKT,MAXLT)
            LPART = (MAXHT*2 + 1)*(MAXKT*2 + 1)*(MAXLT + 1)
            LTEM = (MAXH*2 + 1)*(MAXK*2 + 1)*(MAXL + 1)
            IF (LTEM.GT.LPAR) LPAR = LTEM
C     Find NURO.
            NURO = NUMFO*NSYMP
         END IF
      END IF
C
      RETURN
      END
C     --------------------------------------------------------------
C     Read reflections, run jobs  and output new phases.
      SUBROUTINE RACORN(IH,IK,IL,ICODE,IORDE,MKANG
     +     ,SS4V,FP,SIGFP,EV,PHIKNO,WTKNO,FT,PHIFT
     +     ,THETAE,PHIFP,WT,FC,PHIFC,GH,PHIGH,EX,FOM,SGMAA,NWK,AWK)
C
      INTEGER NWK
C     Input & output data.
      INTEGER IH(NUMFO),IK(NUMFO),IL(NUMFO),ICODE(NUMFO)
     +     ,IORDE(NUMFO),MKANG(NUMFO)
      REAL SS4V(NUMFO),FP(NUMFO),SIGFP(NUMFO),EV(NUMFO),PHIKNO(NUMFO)
     +     ,WTKNO(NUMFO),FT(NUMFO),PHIFT(NUMFO),THETAE(NUMFO)
     +     ,PHIFP(NUMFO),WT(NUMFO),FC(NUMFO),PHIFC(NUMFO),GH(NUMFO)
     +     ,PHIGH(NUMFO),EX(NUMFO),FOM(NUMFO),SGMAA(NUMFO),AWK(NWK)
C
C     Parameters.
      INTEGER MCOLS,NKEYWD,NAMAX
      PARAMETER (MCOLS=500,NKEYWD=200,NAMAX=30000)
C
      REAL TEM1,SIGCUT
C     Arrays for data.
      REAL ADATA,BDATA
      CHARACTER*1 CTPRGO,CTUSRI
      CHARACTER*30 LSPRGO,LSUSRI
      LOGICAL LOGMSS
C     Local Scalars.
      REAL SS4
      INTEGER IC,IFIL1,IFLAG
      INTEGER JDO230,JDO430
      INTEGER NCOL,NSYM,NSYMP,NTOK,NLPRGO,REFCOD
      INTEGER ICHKP,NUMKNO
      CHARACTER TITLE*80
      LOGICAL ENDFIL,LTITLE
C
      REAL EUS1(50),EUS2(50),EUS3(50)
C     Variables for Parser.
      REAL FVALUE
      INTEGER IBEG,IDEC,IEND,ITYPE
      CHARACTER KEY*4,CVALUE*4,LINE*2000,
     +     LABOUT_SAVE*2000
      LOGICAL LEND
C
C     External Routines.
      EXTERNAL CCPERR,CCPFYP,CCPRCS,CENTR,CENTRIC,EQUAL_MAGIC,LKYIN,
     +     LROPEN,LRASSN,LRCLAB,LRCLOS,LRREFM,LRSYMI,LRSYMM,INVSYM,
     +     LWOPEN,LWASSN,LWTITL,LWREFL,MTZINI,PARSER,GTPINT,CCPUPC,
     +     GTPREA,LRRSOL,LKYOUT,LRREFL,LWCLOS,LRNREF,LRREWD,LWHSTL,
     +     LRINFO
C
C     Variables for ACORN.
      CHARACTER NAMSPG*10,ELEM*4
      CHARACTER*64 XNAME_OUT,DNAME_OUT
C     COMMON variables.
      COMMON/NSIZE/NUMFO,LIMBUF,LPAR,NURO,INDMAX,CELLB,NWEK,NBIG
     +     ,LBUFT,LPART
      COMMON/IDX/ NI1,NI2,NI3,NI4,NI5,NI6,NA1,NA2,NA3,NA4,NA5,NA6,NA7,NA8
     +     ,NU1,NU2,NU3,NU4,NU5,NU6,NU7,NK1,NK2,NK3,NK4,NK5,NK6,NK7,NK8
     +     ,NK9,NK10
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
      COMMON/MTZI/IC,IFIL1,IFLAG,IPRINT,JDO230,JDO430
     +     ,NCOL,NTOK,NLPRGO,REFCOD,NUMKNO
     +     ,IN(3),ICHKP(8),IBEG(NKEYWD),IDEC(NKEYWD),IEND(NKEYWD)
     +     ,ITYPE(NKEYWD)
      COMMON/MTZR/SIGCUT,SS4
     +     ,ADATA(MCOLS),BDATA(MCOLS),FVALUE(NKEYWD)
      COMMON/MTZC/CTPRGO(MCOLS),CTUSRI(MCOLS)
     +     ,LSPRGO(MCOLS),LSUSRI(MCOLS),CVALUE(NKEYWD)
     +     ,NAMSPG,TITLE,KEY,LINE,ELEM,LABOUT_SAVE
     +     ,XNAME_OUT(MCOLS),DNAME_OUT(MCOLS)
      COMMON/MTZL/LOGMSS(MCOLS),ENDFIL,LTITLE,LEND
C     Eulerian angles.
      COMMON/EULER/EU1M,EU2M,EU3M,
     +     EU1(5000),EU2(5000),EU3(5000),EUCC(5000),NSEU(5000)
     +     ,TR1(5000),TR2(5000),TR3(5000),TRCS(5000),NSTR(5000)
C     *************************************************************
C
C     Initial input data.
      NC = 0
      NREF = 0
      NEMAX = 0
      NEMIN = 0
      NEWEK = 0
      SS4HOT = 0.0
      SS4LOT = 1.0
      SS4HT = AMAX1(SS4HO,SS4HE)
      SS4LT = AMIN1(SS4LO,SS4LE)
      RESHO = 1.0/SQRT(SS4HO)
C
C     Loop for reflections.
 200  CONTINUE
C     **************************
      CALL LRREFL(IFIL1,SS4,ADATA,ENDFIL)
C     **************************
      IF (ENDFIL) GO TO 300
C
C     Set LOGMSS(i) as .TRUE. if the data item is missing.
C     **************************
      CALL LRREFM(IFIL1,LOGMSS)
C     **************************
C
C     Reflection order code.
      CALL LRNREF(IFIL1,REFCOD)
      NOR = REFCOD - 1
      IORDE(NOR) = -1
C
C     Check the resolution range.
      IF (SS4.LT.SS4LT.OR.SS4.GT.SS4HT) GOTO 200
C     Test whether a reflection is centric (ic=1) or not (ic=0).
      IN(1) = NINT(ADATA(1))
      IN(2) = NINT(ADATA(2))
      IN(3) = NINT(ADATA(3))
      CALL CENTR(IN,IC)
      NC = NC + 1
      IH(NC) = IN(1)
      IK(NC) = IN(2)
      IL(NC) = IN(3)
      IORDE(NC) = NOR
      SS4V(NC) = SS4
      FP(NC) = 0.0
      SIGFP(NC) = 0.0
      EV(NC) = 0.0
      PHIKNO(NC) = 0.0
      WTKNO(NC) = 0.0
      FC(NC) = 0.0
      FT(NC) = 0.0
      PHIFT(NC) = 0.0
      EX(NC) = 0.0
C
C     Get FP SIGFP E and FT & PHIFT & PHASE & WEIGHT if any.
C
      TFT = -1.0
      TFP = -1.0
      TSIG = -1.0
      TEVAL = -1.0
      TWEIGHT = 1.0
      TEC = -1.0
      MP = 0
      MT = 0
      DO JDO230 = 1,NCOL
         IF (.NOT.LOGMSS(JDO230)) THEN
            IF (JDO230.EQ.ICHKP(1)) TFP = ADATA(JDO230)
            IF (JDO230.EQ.ICHKP(2)) TSIG = ADATA(JDO230)
            IF (JDO230.EQ.ICHKP(3)) TEVAL = ADATA(JDO230)
            IF (ICHKP(4).GT.0) THEN
               IF (JDO230.EQ.ICHKP(4)) THEN
                  TPHASE = ADATA(JDO230)
                  MP = 1
               END IF
               IF (ICHKP(5).GT.0) THEN
                  IF (JDO230.EQ.ICHKP(5)) TWEIGHT = ADATA(JDO230)
               END IF
               IF (ICHKP(8).GT.0) THEN
                  IF (JDO230.EQ.ICHKP(8)) TEC = ADATA(JDO230)
               END IF
            END IF
            IF (JDO230.EQ.ICHKP(6)) TFT = ADATA(JDO230)
            IF (JDO230.EQ.ICHKP(7)) THEN
               TPHIFT = ADATA(JDO230)
               MT = 1
            END IF
         END IF
      END DO
C     Input phase information.
      IF (MP.GT.0) THEN
         NUMKNO = NUMKNO + 1
         PHIKNO(NC) = TPHASE
         WTKNO(NC) = TWEIGHT
         FC(NC) = TEC
      END IF
C     Input true F and Phase.
      IF (TFT.GT.0.0) FT(NC) = TFT
      IF (MT.EQ.1) PHIFT(NC) = TPHIFT
C     Check observed reflections used.
      IF (SS4.GE.SS4LO.AND.SS4.LE.SS4HO.AND.
     +     TEVAL.GT.0.0.AND.TFP.GE.SIGCUT*TSIG.AND.
     +     TEVAL.LE.ECUT) THEN
         NREF = NREF + 1
         EV(NC) = TEVAL
         EX(NC) = TEVAL
C     Store input data.
         IF (SS4HOT.LT.SS4) SS4HOT = SS4
         IF (SS4LOT.GT.SS4) SS4LOT = SS4
         FP(NC) = TFP
         IF (TSIG.GT.0) SIGFP(NC) = TSIG
      ELSE
         IF (SS4.GE.SS4LE.AND.SS4.LE.SS4HE) THEN
            IF (TFP.GT.0.0) FP(NC) = TFP
            IF (TSIG.GT.0) SIGFP(NC) = TSIG
            EV(NC) = 1.0
            IF (TEVAL.GT.0.0) EV(NC) = TEVAL
            EX(NC) = 0.0
         ELSE
            NC = NC - 1
         END IF
      END IF
C     Next reflection.
      GO TO 200
 300  continue
      NEXT = NC
      SS4LO = SS4LOT
      SS4HO = SS4HOT
      SS4LT = SS4LO
      SS4HT = SS4HO
      RESLO = 1.0/SQRT(SS4LO)
      RESHO = 1.0/SQRT(SS4HO)
      RESLE = 1.0/SQRT(SS4LE)
      RESHE = 1.0/SQRT(SS4HE)
      WRITE (6,FMT='(/,A,F8.2,A,F8.2,A,/)')
     +     ' The resolution range of observed reflections is from'
     +     ,RESLO,' to',RESHO,' A.'
C     Sort on E values.
      CALL SORTR(EX,IH,IK,IL,ICODE,IORDE,SS4V,FP,SIGFP,FT,PHIFT
     +     ,PHIKNO,WTKNO,EV,FC,NEXT)
      IF (NKEXT.EQ.0) NEXT = NREF
C     Set default according resolution.
      IF (EMIN.LT.0.1) EMIN = 0.15 + 1.0/RESHO
      IF (CFIN.LT.0.001) THEN
         IF (RESHO.LE.1.2) THEN
            CFIN = 0.2
         ELSE IF (RESHO.LE.1.5) THEN
            CFIN = 0.15
         ELSE
            CFIN = 0.1
         END IF
      END IF
C     Check if EVAL > EMIN or < EWEK.
      DO N = 1, NREF
         TEVAL = EV(N)
         IF (TEVAL.GT.EMIN) THEN
            NEMAX = NEMAX + 1
         ELSE
            IF (TEVAL.LT.EWEK) THEN
               NEWEK = NEWEK + 1
            ELSE
               NEMIN = NEMIN + 1
            END IF
         END IF
      END DO
C     Extend.
      IF (NKEXT.EQ.1) THEN
         SS4HT = AMAX1(SS4HO,SS4HE)
         SS4LT = AMIN1(SS4LO,SS4LE)
         WRITE (6,FMT='(A,F8.2,A,F8.2,A,/)')
     +       ' The resolution range of extended reflections is from'
     +       ,RESLE,' to',RESHE,' A.'
      END IF
C
C     Open output file.
C
C     **********************
      CALL LWOPEN(IFIL1,'HKLOUT')
C     Parse the "LABOUT" line now.
      LINE = LABOUT_SAVE
C     ******************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYPE,FVALUE,CVALUE,IDEC,NTOK
     +     ,LEND,.TRUE.)
      CALL LKYOUT(1,LSPRGO,NLPRGO,NTOK,LINE,IBEG,IEND)
      IAPPND = 1
      CALL LWASSN(IFIL1,LSPRGO,NLPRGO,CTPRGO,IAPPND)
C     ******************************************************
C
C     Set IFLAG = 0 to replace old title if LTITLE = .TRUE..
C
      IFLAG = 0
      IF (LTITLE) CALL LWTITL(IFIL1,TITLE,IFLAG)
C     Write a history line.
      CALL LWHSTL (IFIL1, 'ACORN')
C     **********************************************************
C
C     Sort on E values.
C     Sigma cutoff.
      IF (SIGCUT.GT.0.0001) WRITE(6,FMT='(/,A,F6.2,A,A)')
     +     ' The reflections with FO < ',SIGCUT,'*SIGFO',
     +     ' will be rejected.'
C     Check the number user gives.
      IF (NBIG.GT.0) THEN
         NEMAX = NBIG
         EMIN = EV(NEMAX)
      END IF
      IF (NWEK.GT.0) THEN
         NEWEK = NWEK
         EWEK = EV(NREF-NEWEK)
      END IF
      NEMIN = NREF - NEWEK
      ECUT = EV(1)
C     Set ICODE.
      NFOM = 0
      DO NF = 1, NEXT
         IF (NF.LE.NEMAX) ICODE(NF) = 1
         IF (NF.GT.NEMAX.AND.NF.LE.NEMIN) ICODE(NF) = 2
         IF (NF.GT.NEMIN.AND.NF.LE.NREF) ICODE(NF) = 3
         IF (NF.GT.NREF) THEN
            ICODE(NF) = 4
            EX(NF) = 1.0
         END IF
      END DO
      NEMED = NEMIN - NEMAX
      NFOM = NFOM + NEMED
      WRITE(6,330) NREF
 330  FORMAT(/,' The total number of reflections',22X,' = ',I8)
      WRITE(6,FMT='(/,A,F8.3,A,F8.3,A,I8)')
     +     ' The Strong reflections with E from',EMIN,' to',ECUT,
     +     ' = ',NEMAX
      WRITE(6,FMT='(A,F8.3,A,F8.3,A,I8)')
     +     ' The medium reflections with E from',EWEK,' to',EMIN,
     +     ' = ',NEMED
      WRITE(6,FMT='(A,F8.3,A,F8.3,A,I8)')
     +     ' The  weak  reflections with E from',0.0,' to',EWEK,
     +     ' = ',NEWEK
      WRITE(6,FMT='(/,A,I8)')
     +     ' The number of reflections used for FOM =',NFOM
      PARTR = 1.0
      NTEM = NEXT - NREF
      IF (NTEM.GT.0) PARTR = SQRT(REAL(NREF)/REAL(NEXT))
      IF (PARTR.GT.1.0) PARTR = 1.0
      IF (PARTR.LT.0.25) PARTR = 0.25
      IF (NTEM.GT.0) WRITE(6,FMT='(/,A,F10.4)')
     +     ' The scale factor for extended reflections =',PARTR
      IF (NTEM.GT.0) WRITE(6,335) NTEM
 335  FORMAT(/,' The number of extended reflections = ',I8,/)
      IF (NUMKNO.GT.0) WRITE(6,340) NUMKNO
 340   FORMAT(/,' The number of input known phases = ',I8,/)
C
C     Calculate f/gV.
      CALL CALGH2(NEXT,SS4V,THETAE,SS4HT)
C     Determine the phase restrictions for all reflections.
      CALL RESTRI(NEXT,IH,IK,IL,MKANG,NSYMP,RSYML)
C     Calculate a rotational ASU for the Euler angles.
      CALL ROTASU(EU1M,EU2M,EU3M,RSYML,NSYMP)
C     **********************************************************
      WRITE(6,FMT='(/,3(A,F10.2))') ' Volume = ',VOLUME
     +     ,' F000 = ',F000,' E000 = ',E000
      IF (SS4ROT.GT.SS4HO) SS4ROT = SS4HO
C     Set default values for NSUP.
      IF (NSUP.LT.0) THEN
         IF (NFRA.GE.50.OR.NFRA.LE.1) THEN
            NSUP = 0
         ELSE
            NSUP = 1
            IF(NKNO.GT.0) NSUP = 0
         END IF
      END IF
C     Starting from input known phases.
      IF (NKNO.GT.0) THEN
         MAXSET = 1
         GOTO 395
      END IF
C     Use solution from AMORE.
      IF (NSOL.GT.0) THEN
         IF (NTRA.GT.0.OR.NRTF.GT.0) GOTO 390
         MAXSET = NSOL
         GOTO 395
      END IF
C     Previous results from ACORN or input by user.
      IF (INIT.GT.0) THEN
         IF (IMSG.LT.5.AND.(NTRA.GT.0.OR.NRTF.GT.0)) GOTO 390
         IF (NATM.GT.0) THEN
            NFRA = 1
            MAXSET = INIT
         ELSE
            MAXSET = INIT
         END IF
         GOTO 395
      ELSE
C     Calculate SF of the large cell for rotation function.
         IF (NROT.GT.0.OR.NRRT.GT.0) THEN
            NK1 = 1
            CALL CALSF(AWK(NK1),NFRA,SS4HO,CELLB,INDMAX,NSTA)
C     Rotation angles.
            WRITE(6,FMT='(/,A,F8.2)')
     +          ' Rotation angle ALPHA from 0.00 to',EU1M
            WRITE(6,FMT='(A,F8.2)')
     +          ' Rotation angle  BETA from 0.00 to',EU2M
            WRITE(6,FMT='(A,F8.2)')
     +          ' Rotation angle GAMMA from 0.00 to',EU3M
C     Rotation function.
            TEM1 = SQRT(1.0/SS4ROT)
            TEM2 = 100.0/REAL(MODR)
            WRITE(6,FMT='(/,A,A,F10.3,A,F10.2,A)')
     +          ' Rotation function using'
     +          ,' data within resolution',TEM1
     +          ,' and',TEM2,' % reflections before RESORT.'
C     Random rotation function.
            IF (NRRT.GT.0) THEN
               IF (NRRT.LT.MAXSET) MAXSET = NRRT
               NK1 = 1
               NK2 = LPAR + NK1
               NK3 = LPAR + NK2
               NK4 = NURO + NK3
               NK5 = NURO + NK4
               NK6 = NURO + NK5
               NK7 = NURO + NK6
               NK8 = NURO + NK7
               CALL RANROT(INDMAX,AWK(NK1),IH,IK,IL,MKANG,SS4V,EV
     +             ,CELLB,AWK(NK3),AWK(NK4),AWK(NK5),AWK(NK6),AWK(NK7)
     +             ,AWK(NK8),NURO)
               GOTO 390
            END IF
C     Rotation function.
            IF (NROT.GT.0) THEN
               NK1 = 1
               NK2 = LPAR + NK1
               NK3 = LPAR + NK2
               NK4 = NURO + NK3
               NK5 = NURO + NK4
               NK6 = NURO + NK5
               NK7 = NURO + NK6
               NK8 = NURO + NK7
               CALL ROTFRA(INDMAX,AWK(NK1),IH,IK,IL,MKANG,SS4V,EV
     +             ,CELLB,AWK(NK3),AWK(NK4),AWK(NK5),AWK(NK6),AWK(NK7)
     +             ,AWK(NK8),NURO)
               GOTO 390
            END IF
         END IF
C     Starting from random atom fragment.
         IF (NATM.GT.0) THEN
            IF (NATM.LT.MAXSET.AND.NFRA.LE.1) MAXSET = NATM
            IF (NFRA.EQ.0) NFRA = 1
            CALL RANATM(IH,IK,IL,MKANG,SS4V,EV,FC)
            GOTO 395
         END IF
         IF (NPOS.GT.0) THEN
            IF (NPOS.LT.MAXSET) MAXSET = NPOS
            NK1 = 1
            NK2 = LIMBUF + NK1
            CALL TRUFRA(IH,IK,IL,MKANG,SS4V,EV,FC
     +          ,NX,NY,NZ,LIMBUF,AWK(NK1),AWK(NK2))
            GOTO 395
         END IF
      END IF
C     Translational function.
 390  IF (NTRA.GT.0.OR.NRTF.GT.0) THEN
         IF (TXM.GT.0.001) WRITE(6,FMT='(/,A,F10.4)')
     +       ' Translation search over X from 0.0 to',TXM
         IF (TYM.GT.0.001) WRITE(6,FMT='(A,F10.4)')
     +       ' Translation search over Y from 0.0 to',TYM
         IF (TZM.GT.0.001) WRITE(6,FMT='(A,F10.4)')
     +       ' Translation search over Z from 0.0 to',TZM
      END IF
C     Translational function.
      IF (NTRA.GT.0) THEN
         NSAV = 0
         NCSAV = 0
         NSTOT = 0
         IF (NFRA.EQ.1) NSROT = 1
         IF (NSROT.GT.50) NSROT = 50
         DO NSR = 1, NSROT
            EUS1(NSR) = EU1(NSR)
            EUS2(NSR) = EU2(NSR)
            EUS3(NSR) = EU3(NSR)
         END DO
         DO NSR = 1, NSROT
            NK1 = 1
            NK2 = LPART + NK1
            NK3 = LPART + NK2
            NK4 = LBUFT + NK3
            NK5 = LBUFT + NK4
            NK6 = NURO + NK5
            NK7 = NURO + NK6
            NK8 = NURO + NK7
            NK9 = NURO + NK8
            CALL TRANSL(LBUFT,NURO,NSAV,NSR,EUS1,EUS2,EUS3
     +          ,IH,IK,IL,MKANG,SS4V,EV,FC,AWK(NK1),AWK(NK2)
     +          ,AWK(NK3),AWK(NK4),AWK(NK5),AWK(NK6),AWK(NK7)
     +          ,AWK(NK8),AWK(NK9))
            IF (NFRA.GT.1) THEN
               NK1 = 1
               NK2 = LPAR + NK1
               CALL TRANCC(NSAV,NCSAV,NSTOT,NSR,EUS1,EUS2,EUS3
     +             ,IH,IK,IL,MKANG,SS4V,EV,FC,AWK(NK1),AWK(NK2))
            END IF
         END DO
      END IF
C     Random translational function.
      IF (NRTF.GT.0) THEN
         IF (NRTF.LT.MAXSET) MAXSET = NRTF
         NK1 = 1
         NK2 = LPAR + NK1
         CALL RANTRA(IH,IK,IL,MKANG,SS4V,EV,FC
     +       ,AWK(NK1),AWK(NK2))
      END IF
C     Phase refinemnet.
 395  IF (MAXSET.GT.0) THEN
         IF (NTRY.LE.0) THEN
            IF (MAXSET.EQ.1) THEN
               NTRY = 10
            ELSE
               NTRY = 1
            END IF
         END IF
C     Set default values for NSER.
         DO N = 1, NTRY
            IF (NSER(N).LT.0) THEN
               IF (NKNO.GT.0) THEN
                  NSER(N) = 0
               ELSE IF (NFRA.GE.50) THEN
                  NSER(N) = 0
               ELSE
                  NSER(N) = 2
               END IF
               IF (N.EQ.1) NSER(N) = 0
               IF (N.GT.4) NSER(N) = 0
            END IF
         END DO
         NK1 = 1
         NK2 = NUMFO + NK1
         NK3 = NUMFO + NK2
         NK4 = NUMFO + NK3
         NK5 = NUMFO + NK4
         NK6 = NUMFO + NK5
         NK7 = NUMFO + NK6
         NK8 = NUMFO + NK7
         NK9 = NUMFO + NK8
         NK10 = NUMFO + NK9
         NK11 = LIMBUF + NK10
         NK12 = LIMBUF + NK11
         NK13 = LIMBUF + NK12
         CALL PHASE(LIMBUF,IH,IK,IL,MKANG,ICODE,SS4V,FP,EV,PHIKNO,WTKNO
     +       ,FT,PHIFT,THETAE,PHIFP,WT,FC,PHIFC,GH,PHIGH,EX,FOM,SGMAA
     +       ,AWK(NK1),AWK(NK2),AWK(NK3),AWK(NK4),AWK(NK5),AWK(NK6)
     +       ,AWK(NK7),AWK(NK8),AWK(NK9),AWK(NK10),AWK(NK11),AWK(NK12)
     +       ,AWK(NK13))
      ELSE
         CALL CCPERR(1,' *** Error No phases have been developed. ***')
      END IF
C     Sort on IORDE.
      CALL SORTI(EV,SS4V,FP,SIGFP,FT,PHIFT,EX,PHIFP,WT,FC,
     +     IH,IK,IL,ICODE,IORDE,NEXT)
C     ----------------------------------------------------------------
C
C     Read the data from beginning.
C
      CALL LRREWD(IFIL1)
C
      NOR = 0
      NC = 1
C
C     Loop for reflections.
C
 400  CONTINUE
C
      CALL LRREFL(IFIL1,SS4,ADATA,ENDFIL)
C
      IF (ENDFIL) GO TO 1000
C
C     Set LOGMSS(i) as .TRUE. if the data item is missing.
C
      CALL LRREFM(IFIL1,LOGMSS)
C
C     Start assigning the whole list of output as "missing" then replace.
      CALL EQUAL_MAGIC(IFIL1,BDATA,MCOLS)
C     Count the number of reflection.
      NOR = NOR + 1
C
C     Copy all the input data.
C
      DO JDO430 = 1,NCOL
         IF (.NOT.LOGMSS(JDO430)) THEN
            BDATA(JDO430) = ADATA(JDO430)
         END IF
      END DO
C
C     Plus New FC PHASE and WEIGHT.
C
      IF (NC.LE.NEXT) THEN
         NORC = IORDE(NC)
         IF (NOR.EQ.NORC) THEN
            IF (WT(NC).GT.0.000001) THEN
               BDATA(NCOL+1) = EX(NC)
               BDATA(NCOL+2) = PHIFP(NC)
               BDATA(NCOL+3) = WT(NC)
               BDATA(NCOL+4) = FC(NC)
            END IF
            NC = NC + 1
         END IF
      END IF
C
C     Write a reflection to output file and goto next reflection.
C
 500  CALL LWREFL(IFIL1,BDATA)
      GOTO 400
 1000 CONTINUE
C
      WRITE(6,FMT='(//)')
      CALL LRCLOS(IFIL1)
      CALL LWCLOS(IFIL1,1)
      RETURN
      END
C     --------------------------------------------------------------
C
      SUBROUTINE SORTR(A,IH,IK,IL,IC,IR,B,C,D,E,F,G,H,P,Q,N)
C     SORT ON A (E value).
      INTEGER N,IH(N),IK(N),IL(N),IC(N),IR(N)
      REAL A(N),B(N),C(N),D(N),E(N),F(N),G(N),H(N),P(N),Q(N)
      INT=2
 10   INT=2*INT
      IF (INT.LT.N) GO TO 10
      INT=MIN0(N,(3*INT)/4-1)
 20   INT=INT/2
      IFIN=N-INT
      DO 70 II=1,IFIN
         I=II
         J=I+INT
         IF (A(I).GE.A(J)) GO TO 70
         T1=A(J)
         T2=B(J)
         T3=C(J)
         T4=D(J)
         T5=E(J)
         T6=F(J)
         T7=G(J)
         T8=H(J)
         T9=P(J)
         T10=Q(J)
         I1=IH(J)
         I2=IK(J)
         I3=IL(J)
         I4=IC(J)
         I5=IR(J)
 40      A(J)=A(I)
         B(J)=B(I)
         C(J)=C(I)
         D(J)=D(I)
         E(J)=E(I)
         F(J)=F(I)
         G(J)=G(I)
         H(J)=H(I)
         P(J)=P(I)
         Q(J)=Q(I)
         IH(J)=IH(I)
         IK(J)=IK(I)
         IL(J)=IL(I)
         IC(J)=IC(I)
         IR(J)=IR(I)
         J=I
         I=I-INT
         IF (I.LE.0) GO TO 60
         IF (A(I).LT.T1) GO TO 40
 60      A(J)=T1
         B(J)=T2
         C(J)=T3
         D(J)=T4
         E(J)=T5
         F(J)=T6
         G(J)=T7
         H(J)=T8
         P(J)=T9
         Q(J)=T10
         IH(J)=I1
         IK(J)=I2
         IL(J)=I3
         IC(J)=I4
         IR(J)=I5
 70   CONTINUE
      IF (INT.GT.1) GO TO 20
      RETURN
      END
C
C-----------------------------------------------------------------------
C     SORT ON IR (integer).
      SUBROUTINE SORTI(A,B,C,D,E,F,G,H,P,Q,IH,IK,IL,IC,IR,N)
      REAL A(N),B(N),C(N),D(N),E(N),F(N),G(N),H(N),P(N),Q(N)
      INTEGER IH(N),IK(N),IL(N),IC(N),IR(N)
      INT=2
 10   INT=2*INT
      IF (INT.LT.N) GO TO 10
      INT=MIN0(N,(3*INT)/4-1)
 20   INT=INT/2
      IFIN=N-INT
      DO 70 II=1,IFIN
         I=II
         J=I+INT
         IF (IR(I).LT.IR(J)) GO TO 70
         T1=A(J)
         T2=B(J)
         T3=C(J)
         T4=D(J)
         T5=E(J)
         T6=F(J)
         T7=G(J)
         T8=H(J)
         T9=P(J)
         T10=Q(J)
         I1=IH(J)
         I2=IK(J)
         I3=IL(J)
         I4=IC(J)
         I5=IR(J)
 40      A(J)=A(I)
         B(J)=B(I)
         C(J)=C(I)
         D(J)=D(I)
         E(J)=E(I)
         F(J)=F(I)
         G(J)=G(I)
         H(J)=H(I)
         P(J)=P(I)
         Q(J)=Q(I)
         IH(J)=IH(I)
         IK(J)=IK(I)
         IL(J)=IL(I)
         IC(J)=IC(I)
         IR(J)=IR(I)
         J=I
         I=I-INT
         IF (I.LE.0) GO TO 60
         IF (IR(I).GE.I5) GO TO 40
 60      A(J)=T1
         B(J)=T2
         C(J)=T3
         D(J)=T4
         E(J)=T5
         F(J)=T6
         G(J)=T7
         H(J)=T8
         P(J)=T9
         Q(J)=T10
         IH(J)=I1
         IK(J)=I2
         IL(J)=I3
         IC(J)=I4
         IR(J)=I5
 70   CONTINUE
      IF (INT.GT.1) GO TO 20
      RETURN
      END
C-----------------------------------------------------------------------
C     Calculate a rotational ASU for the Euler angles. (Kevin Cowtan)
      SUBROUTINE ROTASU(ALPHA,BETA,GAMMA,RSYM,NSYMP)
      INTEGER NSYMP
      REAL ALPHA,BETA,GAMMA,RSYM(4,4,NSYMP)
C
      REAL M2(4,4),M3(4,4),M4(4,4),M6(4,4)
      REAL MA(4,4),MB(4,4),MU(4,4),MV(4,4)
C
      INTEGER I
      REAL DROT
      EXTERNAL DROT
C
      DATA M2/-1, 0, 0, 0, 0,-1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1/
      DATA M3/ 0, 1, 0, 0,-1,-1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1/
      DATA M4/ 0, 1, 0, 0,-1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1/
      DATA M6/ 0,-1, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1/
      DATA MA/ 1, 0, 0, 0, 0,-1, 0, 0, 0, 0,-1, 0, 0, 0, 0, 1/
      DATA MB/-1, 0, 0, 0, 0, 1, 0, 0, 0, 0,-1, 0, 0, 0, 0, 1/
      DATA MU/ 0, 1, 0, 0, 1, 0, 0, 0, 0, 0,-1, 0, 0, 0, 0, 1/
      DATA MV/ 0,-1, 0, 0,-1, 0, 0, 0, 0, 0,-1, 0, 0, 0, 0, 1/
C
      ALPHA = 360.0
      BETA = 180.0
      GAMMA = 360.0
      DO I = 1, NSYMP
C     Rotations about c
         IF (DROT(RSYM(1,1,I),M2).LT.0.01) ALPHA = MIN(ALPHA,180.0)
         IF (DROT(RSYM(1,1,I),M3).LT.0.01) ALPHA = MIN(ALPHA,120.0)
         IF (DROT(RSYM(1,1,I),M4).LT.0.01) ALPHA = MIN(ALPHA,90.0)
         IF (DROT(RSYM(1,1,I),M6).LT.0.01) ALPHA = MIN(ALPHA,60.0)
C     Rotations prependicular to c
         IF (DROT(RSYM(1,1,I),MA).LT.0.01) BETA = 90.0
         IF (DROT(RSYM(1,1,I),MB).LT.0.01) BETA = 90.0
         IF (DROT(RSYM(1,1,I),MU).LT.0.01) BETA = 90.0
         IF (DROT(RSYM(1,1,I),MV).LT.0.01) BETA = 90.0
      END DO
C
      RETURN
      END
C
      REAL FUNCTION DROT(MAT1,MAT2)
C     Difference between two rotations
      REAL MAT1(4,4),MAT2(4,4),R
      INTEGER I,J
      R = 0.0
      DO I = 1, 3
         DO J = 1, 3
            R = R + (MAT1(I,J) - MAT2(I,J))**2
         END DO
      END DO
      DROT = SQRT(0.5*R)
      RETURN
      END
C-----------------------------------------------------------------------
C     A RANDOM NUMBER GENERATER.
      FUNCTION RANG2 (IR)
      INTEGER IR
      DOUBLE PRECISION IMR,IAR,ICR
C
      COMMON/RGEN/IMR,IAR,ICR
C
      IR = MOD(IAR*IR+ICR,IMR)
      RANG2 = REAL(IR)/REAL(IMR)
      RETURN
      END
C-----------------------------------------------------------------
C     Check maximum value of grid NX,NY and NZ.
      SUBROUTINE CKMXYZ(GRID,LBUF,CELLP,VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL)
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      INTEGER MXYZ(3)
      REAL CELLP(6)
C     Calculate grid points for FFT.
      MT1 = INT(CELLP(1)/GRID + 0.5)
      MT1 = 2*(MT1/2 + 1)
      MT2 = 2*(MAXH + 1)
      MXYZ(1) = MAX0(MT1,MT2)
      MT1 = INT(CELLP(2)/GRID + 0.5)
      MT1 = 2*(MT1/2 + 1)
      MT2 = 2*(MAXK + 1)
      MXYZ(2) = MAX0(MT1,MT2)
      MT1 = INT(CELLP(3)/GRID + 0.5)
      MT1 = 2*(MT1/2 + 1)
      MT2 = 2*(MAXL + 1)
      MXYZ(3) = MAX0(MT1,MT2)
      NEST = 14
      IMAX = 19
      DO 500 I = 1, 3
 20      N = MXYZ(I)
         NF = 2
         NP = 0
         NQ = 0
 100     IF (N.LE.1) GO TO 500
         DO 200 J = NF, IMAX
            IF (N.EQ.(N/J)*J) GO TO 300
 200     CONTINUE
         MXYZ(I) = MXYZ(I) + 2
         GOTO 20
 300     NEV =  2*NP + NQ
         IF (NEV.GE.NEST) GO TO 460
         NF = J
         N = N/NF
         IF (N.EQ.(N/NF)*NF) GO TO 400
         NQ = NQ + 1
         GO TO 100
 400     N = N/NF
         NP = NP + 1
         GO TO 100
 460     MXYZ(I) = MXYZ(I) + 2
         GOTO 20
 500  CONTINUE
      NX = MXYZ(1)
      NY = MXYZ(2)
      NZ = MXYZ(3)
      LBUF = NX*NY*(NZ + 2)
C     Calculate volume of cell.
      A = CELLP(1)
      B = CELLP(2)
      C = CELLP(3)
      ALP = DTOR*CELLP(4)
      BET = DTOR*CELLP(5)
      GAM = DTOR*CELLP(6)
      SUM = 0.5*(ALP + BET + GAM)
      VOLUME = 2.0*A*B*C*
     +     SQRT(SIN(SUM)*SIN(SUM-ALP)*SIN(SUM-BET)*SIN(SUM-GAM))
      RETURN
      END
C     -----------------------------------------------------------
      SUBROUTINE AVEMAP(NX,NY,NZ,YMAP,ROMIN,ROMAX,AVE,SIGRO,RO3)
C
C     Find minimum, maximum and average of a map.
C
      INTEGER NX,NY,NZ
      REAL YMAP(NX,NY,NZ)
      REAL ROMIN,ROMAX,AVE,SIGRO,RO3
C     Local.
      INTEGER NAVE
C
      SIGRO = 0.0
      RO3 = 0.0
      NAVE = 0
      AVE = 0.0
      ROMIN = 100000.0
      ROMAX = -100000.0
      DO K = 1, NZ
         DO J = 1, NY
            DO I = 1, NX
               TEM = YMAP(I,J,K)
               SIGRO = SIGRO + TEM*TEM
               RO3 = RO3 + TEM*TEM*TEM
               AVE = AVE + TEM
               NAVE = NAVE + 1
               IF (TEM.LT.ROMIN) ROMIN = TEM
               IF (TEM.GT.ROMAX) ROMAX = TEM
            END DO
         END DO
      END DO
      AVE = AVE/REAL(NAVE)
      SIGRO = SIGRO/REAL(NAVE)
      SIGRO = SQRT(SIGRO - AVE*AVE)
      DO K = 1, NZ
         DO J = 1, NY
            DO I = 1, NX
               YMAP(I,J,K) = YMAP(I,J,K)/SIGRO
            END DO
         END DO
      END DO
      AVE = AVE/SIGRO
      ROMIN = ROMIN/SIGRO
      ROMAX = ROMAX/SIGRO
      RO3 = RO3/(SIGRO*SIGRO*SIGRO)
      SIGRO = 1.0
      RETURN
      END
C
C     -----------------------------------------------------------
      SUBROUTINE SQMAP(NX,NY,NZ,YMAP)
C
C     Square map.
C
      INTEGER NX,NY,NZ
      REAL YMAP(NX,NY,NZ)
      DO K = 1, NZ
         DO J = 1, NY
            DO I = 1, NX
               YMAP(I,J,K) = YMAP(I,J,K)**2
            END DO
         END DO
      END DO
      RETURN
      END
C
C     -----------------------------------------------------------
      SUBROUTINE SAVMAP(NX,NY,NZ,XMAP,YMAP)
C
C     Save YMAP to XMAP.
C
      INTEGER NX,NY,NZ
      REAL XMAP(NX,NY,NZ),YMAP(NX,NY,NZ)
      DO K = 1, NZ
         DO J = 1, NY
            DO I = 1, NX
               XMAP(I,J,K) = YMAP(I,J,K)
            END DO
         END DO
      END DO
      RETURN
      END
C
C     -----------------------------------------------------------
      SUBROUTINE MODMAP(NX,NY,NZ,YMAP,ZMAP,ROMAX,CUTT,MESG,NSET
     +     ,NCYC,LPTSK,SIGRO,NCT,NTRYS,NCENH)
C
C     Modify map.
C
      CHARACTER*4 MESG
      INTEGER NX,NY,NZ,NSET,NCYC,LPTSK,NCT,NTRYS
      REAL YMAP(NX,NY,NZ),ZMAP(NX,NY,NZ)
      REAL ROMAX,CUTT,SIGRO
C     Local.
      REAL CUTOP
C
      IF (NCENH.EQ.1) THEN
         IF (NCT.LE.5) THEN
            CUTOP = CUTT*SIGRO*REAL(NCT)
         ELSE
            CUTOP = 5.0*CUTT*SIGRO
         END IF
      ELSE
         IF (NTRYS.EQ.1.AND.NCT.LE.5) THEN
            CUTOP = CUTT*SIGRO*REAL(NCT)
         ELSE
            CUTOP = 5.0*CUTT*SIGRO
         END IF
      END IF
      IF (CUTOP.LT.0.1*ROMAX*REAL(NCT)) CUTOP = 0.1*ROMAX*REAL(NCT)
      IF (CUTOP.GT.0.8*ROMAX) CUTOP = 0.8*ROMAX
      IF (LPTSK.EQ.1) THEN
         WRITE(6,FMT='(/,A,F9.4,A,F9.5,A,F9.2,A,1X,A,3I5)')
     +       ' ROmax =',ROMAX,' Cut top ratio =',CUTOP/ROMAX
     +       ,' Cut top at',CUTOP/SIGRO,'*SIGRO'
     +       ,MESG,NSET,NTRYS,NCYC
      END IF
      DO K = 1, NZ
         DO J = 1, NY
            DO I = 1, NX
               TEM = YMAP(I,J,K)
               IF (TEM.LT.0.0) THEN
                  YMAP(I,J,K) = 0.0
               ELSE
                  IF (NCENH.EQ.1) THEN
                     YMAP(I,J,K) = SQRT(TEM)*ZMAP(I,J,K)
                     IF (YMAP(I,J,K).GT.CUTOP) YMAP(I,J,K) = CUTOP
                  ELSE
                     TEM1 = 0.20*(TEM/SIGRO)**1.5
                     YMAP(I,J,K) = TEM*TANH(TEM1)
                     IF (YMAP(I,J,K).GT.CUTOP) YMAP(I,J,K) = CUTOP
                  END IF
               END IF
            END DO
         END DO
      END DO
      RETURN
      END
C
C     -----------------------------------------------------------
      SUBROUTINE MULMAP(NX,NY,NZ,XMAP,YMAP)
C
C     Product of two map.
C
      INTEGER NX,NY,NZ
      REAL XMAP(NX,NY,NZ),YMAP(NX,NY,NZ)
      DO K = 1, NZ
         DO J = 1, NY
            DO I = 1, NX
               YMAP(I,J,K) = XMAP(I,J,K)*YMAP(I,J,K)
            END DO
         END DO
      END DO
      RETURN
      END
C
C     ---------------------------------------------------------
C     Calculate R-factor and CC vs resolution for loggraph.
      SUBROUTINE RFACTOR(NEXT,NREF,FO,FC,SS4V,SS4H,NSS4,NSET,NCYC
     +     ,NCSL,CCSL)
C
      PARAMETER (NSSA=100)
      INTEGER NREF,NSS4,NSET,NCYC
      REAL FO(NEXT),FC(NEXT),SS4V(NEXT),CCSL(NSSA)
C     Local variables.
      INTEGER NDET,KS,NSUM(NSSA)
      REAL SUMFT,SUMFC,RFAC,SS4DEV,SCALE
     +     ,SUMFC2,SUMFT2,SUMFTC,CCALL,FTMAX,CCDIV
      REAL SFT(NSSA),SFC(NSSA),SK(NSSA),RF(NSSA),
     +     SFT2(NSSA),SFC2(NSSA),SFTC(NSSA)
C
      NDET = 0
      NSS = 0
C     Resolution range.
      SS4DEV = SS4H/REAL(NSS4)
      DO I = 1, NSSA
         SFT(I) = 0.0
         SFC(I) = 0.0
         SK(I) = 0.0
         RF(I) = 0.0
         NSUM(I) = 0
         SFT2(I) = 0.0
         SFC2(I) = 0.0
         SFTC(I) = 0.0
         CCSL(I) = 0.0
      END DO
      SUMFT = 0.0
      SUMFC = 0.0
      RFAC = 0.0
      SUMFT2 = 0.0
      SUMFC2 = 0.0
      SUMFTC = 0.0
      CCALL = 0.0
      FTMAX = 0.0
C     Scale between FT and FC and sum F's..
      DO N = 1, NEXT
         IF (FO(N).GT.0.0.AND.FC(N).GT.0.0) THEN
            NDET = NDET + 1
            IF (FO(N).GT.FTMAX) FTMAX = FO(N)
            SUMFT = SUMFT + FO(N)
            SUMFC = SUMFC + FC(N)
            SUMFT2 = SUMFT2 + FO(N)*FO(N)
            SUMFC2 = SUMFC2 + FC(N)*FC(N)
            SUMFTC = SUMFTC + FO(N)*FC(N)
            KS = INT(SS4V(N)/SS4DEV) + 1
            IF (KS.GT.NSSA) KS = NSSA
            IF (KS.GT.NSS) NSS = KS
            SFT(KS) = SFT(KS) + FO(N)
            SFC(KS) = SFC(KS) + FC(N)
            SFT2(KS) = SFT2(KS) + FO(N)*FO(N)
            SFC2(KS) = SFC2(KS) + FC(N)*FC(N)
            SFTC(KS) = SFTC(KS) + FO(N)*FC(N)
            NSUM(KS) = NSUM(KS) + 1
            END IF
      END DO
      SCALE = SUMFT/SUMFC
      DO KS = 1, NSS
         IF (NSUM(KS).GT.0) SK(KS) = SFT(KS)/SFC(KS)
      END DO
      DO N = 1, NEXT
         RFAC = RFAC + ABS(FO(N) - SCALE*FC(N))
         KS = INT(SS4V(N)/SS4DEV) + 1
         IF (KS.GT.NSSA) KS = NSSA
         RF(KS) = RF(KS) + ABS(FO(N) - SK(KS)*FC(N))
      END DO
      RFAC = RFAC/SUMFT
      SNUM = REAL(NDET)
      CCALL = (SNUM*SUMFTC-SUMFT*SUMFC)/
     +     (SQRT((SNUM*SUMFT2 - SUMFT**2)*(SNUM*SUMFC2 - SUMFC**2)))
      DO KS = 1, NSS
         IF (NSUM(KS).GT.0) THEN
            RF(KS) = RF(KS)/SFT(KS)
            SNUM = REAL(NSUM(KS))
            IF (NSUM(KS).EQ.1) THEN
               CCSL(KS) = 1.0
            ELSE
               CCDIV = (SNUM*SFT2(KS) - SFT(KS)**2)*
     +             (SNUM*SFC2(KS) - SFC(KS)**2)
               IF (CCDIV.GT.0.0)
     +             CCSL(KS) = (SNUM*SFTC(KS) - SFT(KS)*SFC(KS))/
     +             SQRT(CCDIV)
            END IF
         END IF
      END DO
      IF (NCSL.GT.0) RETURN
C
C     Print the results for XLOGGRAPH.
C
      WRITE(6,500)
 500  FORMAT(/,/,/,/,1X,44('*')
     +     ,/,' *** R-factor and Correlation Coefficient ***'
     +     ,/,1X,44('*'))
      WRITE(6,510) NSET,NCYC,NSET,NCYC
 510  FORMAT(//,2X,40('*'),/,
     +     ' $TABLE:R-Factor analysis against 4(sin theta/lambda)**2 :'
     +     ,/,' $GRAPHS: R & Corr vs Resln. for set',I6
     +     ,' cycle',I6,':A:2,7,8: $$'
     +     ,/,' $GRAPHS: Refls. vs Resln. for set',I6
     +     ,' cycle',I6,':N:2,4: $$')
      WRITE(6,520)
 520  FORMAT(' Bin',/
     +     ,' 1/resol^2',/
     +     ,' Reso_in_Angstroms',/
     +     ,' Refls_per_bin',/
     +     ,' Mean_AbsEobs',/
     +     ,' Mean_AbsEcalc',/
     +     ,' R_factor',/
     +     ,' Linear_Correlation',/
     +     ,' $$',/
     +     ,' Bin 4(S/L)**2    d       N    <AbsEobs> <AbsEcalc>'
     +     ,'     R      Corr $$'
     +     ,/)
      DO KS = 1, NSS
            SS4 = SS4DEV*REAL(KS)
            RES = SQRT(1.0/SS4)
            AVSFT = 0.0
            AVSFC = 0.0
         IF (NSUM(KS).GT.0) THEN
            AVSFT = SFT(KS)/REAL(NSUM(KS))
            AVSFC = SFC(KS)/REAL(NSUM(KS))
            WRITE(6,550) KS,SS4,RES,NSUM(KS),AVSFT,AVSFC,RF(KS),CCSL(KS)
 550       FORMAT(I4,F9.4,F8.2,I8,2F10.2,2F10.5)
         END IF
      END DO
      SUMFT = SUMFT/REAL(NDET)
      SUMFC = SUMFC/REAL(NDET)
      WRITE(6,610) NDET,SUMFT,SUMFC,RFAC,CCALL
 610  FORMAT(' $$',/,' Overall Totals:',5X,I8,2F10.2,2F10.5)
C     ------
C     Calculate R-Factor vs FO.
      NDEV = NREF/NSSA
      DO I = 1, NSSA
         SFT(I) = 0.0
         SFC(I) = 0.0
         SK(I) = 0.0
         RF(I) = 0.0
         NSUM(I) = 0
         SFT2(I) = 0.0
         SFC2(I) = 0.0
         SFTC(I) = 0.0
         CCSL(I) = 0.0
      END DO
C     Scale between FO and FC and sum F's..
      DO N = 1, NREF
         IF (FO(N).GT.0.0.AND.FC(N).GT.0.0) THEN
            KS = N/NDEV + 1
            IF (KS.GT.NSSA) KS = NSSA
            SFT(KS) = SFT(KS) + FO(N)
            SFC(KS) = SFC(KS) + FC(N)
            SFT2(KS) = SFT2(KS) + FO(N)*FO(N)
            SFC2(KS) = SFC2(KS) + FC(N)*FC(N)
            SFTC(KS) = SFTC(KS) + FO(N)*FC(N)
            NSUM(KS) = NSUM(KS) + 1
         END IF
      END DO
      DO KS = 1, NSSA
         IF (NSUM(KS).GT.0) SK(KS) = SFT(KS)/SFC(KS)
      END DO
      DO N = 1, NREF
         RFAC = RFAC + ABS(FO(N) - SCALE*FC(N))
         KS = N/NDEV + 1
         IF (KS.GT.NSSA) KS = NSSA
         RF(KS) = RF(KS) + ABS(FO(N) - SK(KS)*FC(N))
      END DO
      DO KS = 1, NSSA
         IF (NSUM(KS).GT.0) THEN
            RF(KS) = RF(KS)/SFT(KS)
            SNUM = REAL(NSUM(KS))
            IF (NSUM(KS).EQ.1) THEN
               CCSL(KS) = 1.0
            ELSE
               CCDIV = (SNUM*SFT2(KS) - SFT(KS)**2)*
     +             (SNUM*SFC2(KS) - SFC(KS)**2)
               IF (CCDIV.GT.0.0)
     +             CCSL(KS) = (SNUM*SFTC(KS) - SFT(KS)*SFC(KS))/
     +             SQRT(CCDIV)
            END IF
         END IF
      END DO
C
C     Print the results for XLOGGRAPH.
C
      WRITE(6,1510) NSET,NCYC,NSET,NCYC,NSET,NCYC
 1510 FORMAT(//,2X,40('*'),/,
     +     ' $TABLE:R-Factor analysis against Eo :'
     +     ,/,' $GRAPHS: R & Corr vs Eo for set',I6
     +     ,' cycle',I6,':A:1,6,7: $$'
     +     ,/,' $GRAPHS: R & Corr vs Eo for set',I6
     +     ,' cycle',I6,':A:2,6,7: $$'
     +     ,/,' $GRAPHS: Refls. vs Eo for set',I6
     +     ,' cycle',I6,':N:1,3: $$')
      WRITE(6,1520)
 1520 FORMAT(' Bin',/
     +     ,' Eo',/
     +     ,' Refls_per_bin',/
     +     ,' Mean_AbsEobs',/
     +     ,' Mean_AbsEcalc',/
     +     ,' R_factor',/
     +     ,' Linear_Correlation',/
     +     ,' $$',/
     +     ,' Bin     Eo       N    <AbsEobs> <AbsEcalc>'
     +     ,'     R      Corr $$'
     +     ,/)
      DO KS = 1, NSSA
         NTEM = KS*NDEV
         IF (NTEM.GT.NREF) NTEM = NREF
         FTT = FO(NTEM)
         AVSFT = 0.0
         AVSFC = 0.0
         IF (NSUM(KS).GT.0) THEN
            AVSFT = SFT(KS)/REAL(NSUM(KS))
            AVSFC = SFC(KS)/REAL(NSUM(KS))
            WRITE(6,1550) KS,FTT,NSUM(KS),AVSFT,AVSFC,RF(KS),CCSL(KS)
 1550      FORMAT(I4,F9.2,I8,2F10.2,2F10.5)
         END IF
      END DO
      WRITE(6,1610)
 1610 FORMAT(' $$',/)
      RETURN
      END
C     ---------------------------------------------------------
C     Calculate weighted phase error vs resolution for loggraph.
C     No origin checking.
      SUBROUTINE PHIERR(NEXT,NREF,EV,PHIFT,PHIFC,WT,SS4V,SS4H,NSS4
     +     ,NSET,NCYC)
C
      INTEGER NREF,NSS4,NSET
      PARAMETER (NSSA=100)
      REAL EV(NEXT),PHIFT(NEXT),PHIFC(NEXT),WT(NEXT),SS4V(NEXT)
C     Local variables.
      INTEGER KS,NSUM(NSSA),NDET,NCYC
      REAL ERRAL,ERRWT,ERREV,ERRWF,SS4DEV,SUMWT,SUMEV,SUMWF
      REAL EAL(NSSA),EWT(NSSA),EEV(NSSA),EWF(NSSA),SWT(NSSA)
     +     ,SEV(NSSA),SWF(NSSA)
C
      NDET = 0
      NSS = 0
      ERRAL = 0.0
      ERRWT = 0.0
      ERREV = 0.0
      ERRWF = 0.0
      SUMWT = 0.0
      SUMEV = 0.0
      SUMWF = 0.0
      EVMAX = 0.0
C     Resolution range.
      SS4DEV = SS4H/REAL(NSS4)
      DO I = 1, NSSA
         EAL(I) = 0.0
         EWT(I) = 0.0
         EEV(I) = 0.0
         EWF(I) = 0.0
         SWT(I) = 0.0
         SEV(I) = 0.0
         SWF(I) = 0.0
         NSUM(I) = 0
      END DO
C     Phase error.
      DO N = 1, NEXT
C     Total.
         NDET = NDET + 1
         IF (EV(N).GT.EVMAX) EVMAX = EV(N)
         ERROR = ABS(PHIFT(N) - PHIFC(N))
         ERROR = AMOD(ERROR + 720.0, 360.0)
         IF (ERROR.GT.180.0) ERROR = 360.0 - ERROR
         ERRAL = ERRAL + ERROR
         ERRWT = ERRWT + WT(N)*ERROR
         SUMWT = SUMWT + WT(N)
         ERREV = ERREV + EV(N)*ERROR
         SUMEV = SUMEV + EV(N)
         ERRWF = ERRWF + WT(N)*EV(N)*ERROR
         SUMWF = SUMWF + WT(N)*EV(N)
C     Error vs resolution.
         KS = INT(SS4V(N)/SS4DEV) + 1
         IF (KS.GT.NSSA) KS = NSSA
         IF (KS.GT.NSS) NSS = KS
         EAL(KS) = EAL(KS) + ERROR
         NSUM(KS) = NSUM(KS) + 1
         EWT(KS) = EWT(KS) + WT(N)*ERROR
         SWT(KS) = SWT(KS) + WT(N)
         EEV(KS) = EEV(KS) + EV(N)*ERROR
         SEV(KS) = SEV(KS) + EV(N)
         EWF(KS) = EWF(KS) + WT(N)*EV(N)*ERROR
         SWF(KS) = SWF(KS) + WT(N)*EV(N)
      END DO
      ERRAL = ERRAL/REAL(NDET)
      ERRWT = ERRWT/SUMWT
      ERREV = ERREV/SUMEV
      ERRWF = ERRWF/SUMWF
      DO KS = 1, NSS
         IF (NSUM(KS).GT.0) THEN
            EAL(KS) = EAL(KS)/REAL(NSUM(KS))
            EWT(KS) = EWT(KS)/SWT(KS)
            EEV(KS) = EEV(KS)/SEV(KS)
            EWF(KS) = EWF(KS)/SWF(KS)
         END IF
      END DO
C
C     Print the results for XLOGGRAPH.
C
      WRITE(6,500)
 500  FORMAT(/,/,/,/,1X,31('*')
     +     ,/,' ***      Phase error       *** '
     +     ,/1X,31('*'))
      WRITE(6,510) NSET,NCYC,NSET,NCYC
 510  FORMAT(//,2X,40('*'),/,
     +     ' $TABLE:Phase error against 4(sin theta/lambda)**2 :'
     +     ,/,' $GRAPHS: Phase error vs Resln. for set',I6
     +     ,' cycle',I6,':N:2,5,6,7,8: $$'
     +     ,/,' $GRAPHS: Refls. vs Resln. for set',I6
     +     ,' cycle',I6,':N:2,4: $$')
      WRITE(6,520)
 520  FORMAT(' Bin',/
     +     ,' 1/resol^2',/
     +     ,' Reso_in_Angstroms',/
     +     ,' Refls_per_bin',/
     +     ,' No_weight',/
     +     ,' Weighted_by_Wt ',/
     +     ,' Weighted_by_Eo',/
     +     ,' Weighted_by_Wt*Eo',/
     +     ,' $$',/
     +     ,' Bin 4(S/L)**2    d      N     <DPHI> weighted by WT'
     +     ,'     Eo    Wt*Eo $$',/)
      DO KS = 1, NSS
         SS4 = SS4DEV*REAL(KS)
         RES = SQRT(1.0/SS4)
         WRITE(6,550) KS,SS4,RES,NSUM(KS),EAL(KS),EWT(KS),EEV(KS)
     +       ,EWF(KS)
 550     FORMAT(I4,F9.4,F8.2,I8,F8.2,8X,3F8.2)
      END DO
      WRITE(6,610) NDET,ERRAL,ERRWT,ERREV,ERRWF
 610  FORMAT(' $$',/,' Overall Totals:',5X,I8,F8.2,8X,3F8.2)
C
C     Calculate phase error vs EV.
      NDEV = NREF/NSSA
      DO I = 1, NSSA
         EAL(I) = 0.0
         EWT(I) = 0.0
         EEV(I) = 0.0
         EWF(I) = 0.0
         SWT(I) = 0.0
         SEV(I) = 0.0
         SWF(I) = 0.0
         NSUM(I) = 0
      END DO
C     Phase error.
      DO N = 1, NREF
         KS = N/NDEV + 1
         IF (KS.GT.NSSA) KS = NSSA
         ERROR = ABS(PHIFT(N) - PHIFC(N))
         ERROR = AMOD(ERROR + 720.0, 360.0)
         IF (ERROR.GT.180.0) ERROR = 360.0 - ERROR
         EAL(KS) = EAL(KS) + ERROR
         NSUM(KS) = NSUM(KS) + 1
         EWT(KS) = EWT(KS) + WT(N)*ERROR
         SWT(KS) = SWT(KS) + WT(N)
         EEV(KS) = EEV(KS) + EV(N)*ERROR
         SEV(KS) = SEV(KS) + EV(N)
         EWF(KS) = EWF(KS) + WT(N)*EV(N)*ERROR
         SWF(KS) = SWF(KS) + WT(N)*EV(N)
      END DO
      DO KS = 1, NSSA
         IF (NSUM(KS).GT.0) THEN
            EAL(KS) = EAL(KS)/REAL(NSUM(KS))
            EWT(KS) = EWT(KS)/SWT(KS)
            EEV(KS) = EEV(KS)/SEV(KS)
            EWF(KS) = EWF(KS)/SWF(KS)
         END IF
      END DO
C
C     Print the results for XLOGGRAPH.
C
      WRITE(6,1510) NSET,NCYC,NSET,NCYC,NSET,NCYC
 1510 FORMAT(//,2X,40('*'),/,
     +     ' $TABLE:Phase error against Eo :'
     +     ,/,' $GRAPHS: Phase error vs Eo for set',I6
     +     ,' cycle',I6,':N:1,4,5,6,7: $$'
     +     ,/,' $GRAPHS: Phase error vs Eo for set',I6
     +     ,' cycle',I6,':N:2,4,5,6,7: $$'
     +     ,/,' $GRAPHS: Refls. vs Eo for set',I6
     +     ,' cycle',I6,':N:1,3: $$')
      WRITE(6,1520)
 1520 FORMAT(' Bin',/
     +     ' Eo',/
     +     ,' Refls_per_bin',/
     +     ,' No_weight',/
     +     ,' Weighted_by_Wt ',/
     +     ,' Weighted_by_Eo',/
     +     ,' Weighted_by_Wt*Eo',/
     +     ,' $$',/
     +     ,' Bin     Eo      N    <DPHI> weighted by WT'
     +     ,'     Eo    Wt*Eo $$',/)
      DO KS = 1, NSSA
         NTEM = KS*NDEV
         IF (NTEM.GT.NREF) NTEM = NREF
         EVT = EV(NTEM)
         IF (NSUM(KS).GT.0)
     +       WRITE(6,1550) KS,EVT,NSUM(KS),EAL(KS),EWT(KS),EEV(KS)
     +       ,EWF(KS)
 1550    FORMAT(I4,F9.2,I8,F8.2,8X,3F8.2)
      END DO
      WRITE(6,FMT='(A)') ' $$'
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE ATMCOEF
C            CALCULATE SPHERICAL SCATTERING FACTOR COEFFICIENTS
C
C           F = AL * EXP(-AS * RHO) + BL * EXP(-BS * RHO)
C            + CL * EXP(-CS * RHO) + DL * EXP(-DS * RHO) + EL
C
      CHARACTER*4 NW,N1(4)
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/ATMFACTOR/NKD,AL(8),AS(8),BL(8),BS(8),CL(8),CS(8)
     +  ,DL(8),DS(8),EL(8),NW(8),NO(8),NE(8)
C
      LOGICAL OP
      INTEGER NG,IELEC,Ifail
      REAL A(8),B(8),C,CU(2),MO(2)
C
      EXTERNAL SFREAD2,CCPERR,CCPDPN
C
      DATA N1/'C','N','O','S'/
C     No CONT input.
      IF (NKD.EQ.0) THEN
         NKD = 4
         NW(1) = N1(1)
         NE(1) = 6
         NO(1) = INT(VOLUME/63.3 + 0.5)
         NW(2) = N1(2)
         NE(2) = 7
         NO(2) = INT(VOLUME/221.6 + 0.5)
         NW(3) = N1(3)
         NE(3) = 8
         NO(3) = INT(VOLUME/105.9 + 0.5)
         NW(4) = N1(4)
         NE(4) = 16
         NO(4) = INT(VOLUME/17812.7 + 0.5)
      END IF
      NG = 5
C---- Check to open file
C
      INQUIRE (UNIT=45, OPENED=OP, IOSTAT=IOS)
      IF (IOS .NE. 0) CALL CCPERR(1,'Error opening ATOMSF file')

      IF (.NOT.OP) THEN
        Ifail  = 1
        CALL CCPDPN(45,'ATOMSF','READONLY','F',0,Ifail)
        IF (Ifail.LT.0) CALL CCPERR(1,'Error opening library file')
      ELSE
        REWIND 45
      END IF
C
      DO I = 1, NKD
         CALL SFREAD2(NW(I),NG,A,B,C,IWT,IELEC,CU,MO,Ifail)
         IF (Ifail.EQ.-1)
     +       CALL CCPERR(1,' No match for atom label(ATSORT)')
         NE(I) = IELEC
         AL(I) = A(1)
         AS(I) = B(1)
         BL(I) = A(2)
         BS(I) = B(2)
         CL(I) = A(3)
         CS(I) = B(3)
         DL(I) = A(4)
         DS(I) = B(4)
         EL(I) = C
      END DO
      RETURN
      END
C     -----------------------------------------------------------------
C     Calculate F000 and E000.
      SUBROUTINE CFZERO
      CHARACTER*4 NW
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/ATMFACTOR/NKD,AL(8),AS(8),BL(8),BS(8),CL(8),CS(8)
     +  ,DL(8),DS(8),EL(8),NW(8),NO(8),NE(8)
C
      F000 = 0.0
      SFJ2 = 0.0
C     Calculate F000 and E000.
      DO K = 1, NKD
         FZ = AL(K) + BL(K) + CL(K) + DL(K) + EL(K)
         F000 = F000 + FZ*REAL(NO(K))
         SFJ2 = SFJ2 + FZ*FZ*REAL(NO(K))
      END DO
      E000 = F000/SQRT(SFJ2)
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C      CALCULATE  G(S) = PI/N*(4*SMAX**3/3 - S*SMAX**2 + S**3/12)     C
C                    S = 2*SIN(THETA)/LAMDA                           C
C                        CCP4  VERSON   1998                          C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE CALGH2(NEXT,SS4V,THETAE,SS4E)
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      REAL THETAE(NEXT),SS4V(NEXT),SS4E
C     -------------------------------------------------------------
      SLM = SQRT(SS4E)
      FH = 1.0/E000
      DO N = 1, NEXT
         SL = SQRT(SS4V(N))
         T1 = 4.0*SLM*SLM*SLM/3.0 - SL*SLM*SLM + SL*SL*SL/12.0
         GH = PI*T1/(E000*E000)
         THETAE(N) = FH/(GH*VOLUME)
      END DO
      RETURN
      END
C     -------------------------------------------------------------
      SUBROUTINE WTSIM(NEXT,FP,FC,WT,PARTR,ICODE)
C     Sim weights.
      INTEGER NEXT,ICODE(NEXT)
      REAL FP(NEXT),FC(NEXT),WT(NEXT)
C
      DO N = 1, NEXT
         WT(N) = 0.0
         X = FC(N)*FP(N)
         IF (ICODE(N).GT.3) X = X*PARTR
         WT(N) = TANH(0.5*X)
      END DO
      RETURN
      END
C     -------------------------------------------------------------
C     Obtain new phases.
      SUBROUTINE PHINEW(NEXT,PHIFP,FC,PHIFC,NNEW
     +     ,SNEW,MKANG,ICODE)
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
C
      INTEGER NEXT,NNEW,MKANG(NEXT),ICODE(NEXT)
      REAL PHIFP(NEXT),FC(NEXT),PHIFC(NEXT)
C
      NNEW = 0
      SNEW = 0.0
      DO N = 1, NEXT
         IF (MKANG(N).GT.0) THEN
            ID = IABS(MKANG(N)) - 1
            IF (ID.GT.0) THEN
               TOP = FC(N)*SIN(DTOR*PHIFC(N))
               BOT = FC(N)*COS(DTOR*PHIFC(N))
               SID = DTOR*REAL(ID)
               T2 = TOP*SIN(SID) + BOT*COS(SID)
               TOP = T2*SIN(SID)
               BOT = T2*COS(SID)
               FC(N) = SQRT(TOP*TOP + BOT*BOT)
               PHIFC(N) = 0.0
               IF (FC(N).GT.0.0) PHIFC(N) = ATAN2(TOP,BOT)*RTOD
            END IF
            IF (ICODE(N).LE.1) THEN
               NNEW = NNEW + 1
               TNEW = ABS(PHIFP(N) - PHIFC(N) + 720.0)
               TNEW = AMOD(TNEW,360.0)
               IF (TNEW.GT.180) TNEW = 360.0 - TNEW
               SNEW = SNEW + TNEW
            END IF
            PHIFP(N) = PHIFC(N)
         END IF
      END DO
      IF (NNEW.GT.0) SNEW = SNEW/REAL(NNEW)
      RETURN
      END
C     -------------------------------------------------------------
      SUBROUTINE RFNEW(NEXT,FO,FC,NSET,NTRYS,NCYC,CC,MKLPT
     +     ,MESG,ICODE)
C     R-factor and CC for reflection groups.
      CHARACTER*4 MESG
      INTEGER NEXT,NSET,NTRYS,NCYC,ICODE(NEXT)
      REAL FO(NEXT),FC(NEXT)
C
      INTEGER NSUM(4)
      REAL SFT(4),SFC(4),SK(4),RF(4),CCL(4),
     +     SFT2(4),SFC2(4),SFTC(4),CC(3),CCDIV
C
      DO I = 1, 4
         SFT(I) = 0.0
         SFC(I) = 0.0
         SK(I) = 0.0
         RF(I) = 0.0
         NSUM(I) = 0
         SFT2(I) = 0.0
         SFC2(I) = 0.0
         SFTC(I) = 0.0
      END DO
C
      DO N = 1, NEXT
         KS = ICODE(N)
         NSUM(KS) = NSUM(KS) + 1
         SFC(KS) = SFC(KS) + FC(N)
         SFC2(KS) = SFC2(KS) + FC(N)*FC(N)
         SFT(KS) = SFT(KS) + FO(N)
         SFT2(KS) = SFT2(KS) + FO(N)*FO(N)
         SFTC(KS) = SFTC(KS) + FO(N)*FC(N)
      END DO
      DO KS = 1, 4
         IF (NSUM(KS).GT.0) SK(KS) = SFT(KS)/SFC(KS)
      END DO
      DO N = 1, NEXT
         KS = ICODE(N)
            RF(KS) = RF(KS) + ABS(FO(N) - SK(KS)*FC(N))
      END DO
      DO KS = 1, 4
         IF (NSUM(KS).GT.0) THEN
            RF(KS) = RF(KS)/SFT(KS)
            SNUM = REAL(NSUM(KS))
            CCDIV = (SNUM*SFT2(KS) - SFT(KS)**2)*
     +          (SNUM*SFC2(KS) - SFC(KS)**2)
            IF (CCDIV.GT.0.0)
     +          CCL(KS) = (SNUM*SFTC(KS) - SFT(KS)*SFC(KS))/
     +          SQRT(CCDIV)
            IF (KS.LE.3) CC(KS) = CCL(KS)
            SFT(KS) = SFT(KS)/SNUM
            SFC(KS) = SFC(KS)/SNUM
         END IF
      END DO
      IF (MKLPT.EQ.0) RETURN
C     Print the results.
      WRITE(6,600)
 600  FORMAT(/,30X,'     N      <Eo>      <Ec>    R-Factor     CC',
     +     '   MESG NSET NTRY NCYC')
      WRITE(6,610) NSUM(1),SFT(1),SFC(1),RF(1),CCL(1),MESG,NSET,NTRYS
     +     ,NCYC
 610  FORMAT(' R-factor & Corr for strong E ',I8,2F10.2,2F10.5
     +     ,1X,A,3I5)
      WRITE(6,620) NSUM(2),SFT(2),SFC(2),RF(2),CCL(2),MESG,NSET,NTRYS
     +     ,NCYC
 620  FORMAT(' R-factor & Corr for medium E ',I8,2F10.2,2F10.5
     +     ,1X,A,3I5)
      WRITE(6,630) NSUM(3),SFT(3),SFC(3),RF(3),CCL(3),MESG,NSET,NTRYS
     +     ,NCYC
 630  FORMAT(' R-factor & Corr for weak   E ',I8,2F10.2,2F10.5
     +     ,1X,A,3I5)
      IF (NSUM(4).EQ.0) RETURN
      WRITE(6,640) NSUM(4),SFT(4),SFC(4),RF(4),CCL(4),MESG,NSET,NTRYS
     +     ,NCYC
 640  FORMAT(' R-factor & Corr for extstr E ',I8,2F10.2,2F10.5
     +     ,1X,A,3I5)
      RETURN
      END
C     -----------------------------------------------------------
      SUBROUTINE FRENEW(NEXT,ICODE,PHIFT,EV,FC,PHIFC,WT,NSET,NTRYS
     +     ,NCYC,MESG)
C     Weighted phase errors for reflection groups.
      CHARACTER*4 MESG
      INTEGER NEXT,NSET,NTRYS,NCYC,ICODE(NEXT)
      REAL PHIFT(NEXT),EV(NEXT),FC(NEXT),PHIFC(NEXT),WT(NEXT)
C
C     Local variables.
      INTEGER KS,NSUM(4)
      REAL EAL(4),EWT(4),EFC(4),EWF(4),SWT(4),SFC(4),SWF(4)
     +     ,EWE(4),SWE(4)
C
      DO I = 1, 4
         EAL(I) = 0.0
         EWT(I) = 0.0
         EFC(I) = 0.0
         EWF(I) = 0.0
         SWT(I) = 0.0
         SFC(I) = 0.0
         SWF(I) = 0.0
         EWE(I) = 0.0
         SWE(I) = 0.0
         NSUM(I) = 0
      END DO
C     Phase error.
      DO N = 1, NEXT
         ERROR = ABS(PHIFT(N) - PHIFC(N))
         ERROR = AMOD(ERROR + 720.0, 360.0)
         IF (ERROR.GT.180.0) ERROR = 360.0 - ERROR
         KS = ICODE(N)
         EAL(KS) = EAL(KS) + ERROR
         NSUM(KS) = NSUM(KS) + 1
         EWT(KS) = EWT(KS) + WT(N)*ERROR
         SWT(KS) = SWT(KS) + WT(N)
         EFC(KS) = EFC(KS) + FC(N)*ERROR
         SFC(KS) = SFC(KS) + FC(N)
         EWF(KS) = EWF(KS) + WT(N)*FC(N)*ERROR
         SWF(KS) = SWF(KS) + WT(N)*FC(N)
         EWE(KS) = EWE(KS) + WT(N)*EV(N)*ERROR
         SWE(KS) = SWE(KS) + WT(N)*EV(N)
      END DO
      DO KS = 1, 4
         IF (NSUM(KS).GT.0) THEN
            EAL(KS) = EAL(KS)/REAL(NSUM(KS))
            EWT(KS) = EWT(KS)/SWT(KS)
            EFC(KS) = EFC(KS)/SFC(KS)
            EWF(KS) = EWF(KS)/SWF(KS)
            EWE(KS) = EWE(KS)/SWE(KS)
         END IF
      END DO
C
C     Print the results.
      WRITE(6,600)
 600  FORMAT(/,' Weighting Scheme',15X,
     +     'N   No weight      Wt      Wt*Eo     Wt*Ec')
      WRITE(6,610) NSUM(1),EAL(1),EWT(1),EWE(1),EWF(1),MESG,NSET,NTRYS
     +     ,NCYC
 610  FORMAT(' Phase error for strong E ',I8,4F10.2,1X
     +     ,A,3I5)
      WRITE(6,620) NSUM(2),EAL(2),EWT(2),EWE(2),EWF(2),MESG,NSET,NTRYS
     +     ,NCYC
 620  FORMAT(' Phase error for medium E ',I8,4F10.2,1X
     +     ,A,3I5)
      WRITE(6,630) NSUM(3),EAL(3),EWT(3),EWE(3),EWF(3),MESG,NSET,NTRYS
     +     ,NCYC
 630  FORMAT(' Phase error for weak   E ',I8,4F10.2,1X
     +     ,A,3I5)
      IF (NSUM(4).EQ.0) RETURN
      WRITE(6,640) NSUM(4),EAL(4),EWT(4),EWE(4),EWF(4),MESG,NSET,NTRYS
     +     ,NCYC
 640  FORMAT(' Phase error for extend E ',I8,4F10.2,1X
     +     ,A,3I5)
      RETURN
      END
C     -----------------------------------------------------------
C     Scale Ec to <Ec**2>=<Eo**2> for reflection groups.
      SUBROUTINE SCNEW(NEXT,EV,EC,SS4V,SS4H,NSS4,ICODE)
C
      PARAMETER (NSSA=100)
      INTEGER NEXT,NSS4,ICODE(NEXT)
      REAL SS4H,EV(NEXT),EC(NEXT),SS4V(NEXT)
C     Local variables.
      INTEGER KS,NSUM(NSSA,4)
      REAL SEV2(NSSA,4),SEC2(NSSA,4),SK(NSSA,4)
C     Resolution range.
      NJ = 3
      NSS = 0
      SS4DEV = SS4H/REAL(NSS4)
      DO I = 1, NSSA
         DO J = 1, 4
            NSUM(I,J) = 0
            SEV2(I,J) = 0.0
            SEC2(I,J) = 0.0
            SK(I,J) = 0.0
         END DO
      END DO
C     Sum EC**2.
      DO N = 1, NEXT
         IF (EV(N).GT.0.0.AND.EC(N).GT.0.0) THEN
            KS = INT(SS4V(N)/SS4DEV) + 1
            IF (KS.GT.NSSA) KS = NSSA
            IF (KS.GT.NSS) NSS = KS
            J = ICODE(N)
            IF (ICODE(N).GE.2) J = 2
            IF (ICODE(N).EQ.4) J = 3
            NSUM(KS,J) = NSUM(KS,J) + 1
            SEC2(KS,J) = SEC2(KS,J) + EC(N)*EC(N)
            IF (J.LE.2) THEN
               SEV2(KS,J) = SEV2(KS,J) + EV(N)*EV(N)
            ELSE IF (J.GE.3) THEN
               SEV2(KS,J) = SEV2(KS,J) + 1.0
            END IF
         END IF
      END DO
C     Scale factors.
      DO KS = 1, NSS
         DO J = 1, NJ
            IF (NSUM(KS,J).GT.0) THEN
               SK(KS,J) = SQRT(SEV2(KS,J)/SEC2(KS,J))
            END IF
         END DO
      END DO
C     Scale Ecal values.
      DO N = 1, NEXT
         KS = INT(SS4V(N)/SS4DEV) + 1
         IF (KS.GT.NSSA) KS = NSSA
         J = ICODE(N)
         IF (ICODE(N).GE.2) J = 2
         IF (ICODE(N).EQ.4) J = 3
         EC(N) = SK(KS,J)*EC(N)
      END DO
      RETURN
      END
C     -----------------------------------------------------------
C     Scale Ec2 to Eo2 for rotation function.
      SUBROUTINE ROTSC(NRALL,RFO,RFC,S4VR,SS4H,NSS4)
C
      INTEGER NRALL,NSS4
      REAL SS4H,RFO(NRALL),RFC(NRALL),S4VR(NRALL)
C     Local variables.
      INTEGER KS,NSUM(50)
      REAL SEV2(50),SEC2(50),SK(50)
C     Resolution range.
      SS4DEV = SS4H/REAL(NSS4)
      DO I = 1, NSS4
         NSUM(I) = 0
         SEV2(I) = 0.0
         SEC2(I) = 0.0
         SK(I) = 0.0
      END DO
C     Sum EC**2.
      DO N = 1, NRALL
         IF (RFO(N).GT.0.0.AND.RFC(N).GT.0.0) THEN
            KS = INT(S4VR(N)/SS4DEV) + 1
            IF (KS.GT.NSS4) KS = NSS4
            NSUM(KS) = NSUM(KS) + 1
            SEC2(KS) = SEC2(KS) + RFC(N)*RFC(N)
            SEV2(KS) = SEV2(KS) + RFO(N)*RFO(N)
         END IF
      END DO
C     Scale factors.
      DO KS = 1, NSS4
         IF (NSUM(KS).GT.0) SK(KS) = SQRT(SEV2(KS)/SEC2(KS))
      END DO
C     Scale Ecal values.
      DO N = 1, NRALL
         KS = INT(S4VR(N)/SS4DEV) + 1
         IF (KS.GT.NSS4) KS = NSS4
         RFC(N) = SK(KS)*RFC(N)
      END DO
      RETURN
      END
C     -------------------------------------------------------------
      SUBROUTINE ROTRF(NRALL,RFO,RFC,NSET,CC,MKLPT,MESG)
C     R-factor and CC for reflection groups.
      CHARACTER*4 MESG
      INTEGER NRALL,NSET,MKLPT
      REAL RFO(NRALL),RFC(NRALL)
C
      REAL SFT,SFC,SK,RF,SNUM,
     +     SFT2,SFC2,SFTC,CC
C     Initial.
      SFT = 0.0
      SFC = 0.0
      SK = 0.0
      RF = 0.0
      SFT2 = 0.0
      SFC2 = 0.0
      SFTC = 0.0
      NUMB = 0
C
      DO N = 1, NRALL
         NUMB = NUMB + 1
         SFT = SFT + RFO(N)
         SFC = SFC + RFC(N)
         SFT2 = SFT2 + RFO(N)*RFO(N)
         SFC2 = SFC2 + RFC(N)*RFC(N)
         SFTC = SFTC + RFO(N)*RFC(N)
      END DO
         SK = SFT/SFC
      DO N = 1, NRALL
         RF = RF + ABS(RFO(N) - SK*RFC(N))
      END DO
      RF = RF/SFT
      SNUM = REAL(NUMB)
      CC = (SNUM*SFTC - SFT*SFC)/
     +     (SQRT((SNUM*SFT2 - SFT**2)*
     +     (SNUM*SFC2 - SFC**2)))
      SFT = SFT/SNUM
      SFC = SFC/SNUM
      IF (MKLPT.EQ.0) RETURN
C     Print the results.
      WRITE(6,600)
 600  FORMAT(/,30X,'     N      <Eo>      <Ec>    R-Factor     CC')
      WRITE(6,610) NUMB,SFT,SFC,RF,CC,MESG,NSET
 610  FORMAT(' R-factor & Corr for large  E ',I8,2F10.2,2F10.5
     +     ,1X,A,I5)
      RETURN
      END
C     -----------------------------------------------------------
C     Scale Ec**2 to 1.0.
      SUBROUTINE SCALE1(NREF,MKANG,EC,SS4V,SS4H,NSS4)
C
      PARAMETER (NSSA=100)
      INTEGER NREF,NSS4,MKANG(NREF)
      REAL SS4H,EC(NREF),SS4V(NREF)
C     Local variables.
      INTEGER KS,NSUM(NSSA)
      REAL SEC2(NSSA),SK(NSSA)
C     Resolution range.
      SS4DEV = SS4H/REAL(NSS4)
      DO I = 1, NSSA
         NSUM(I) = 0
         SEC2(I) = 0.0
         SK(I) = 0.0
      END DO
C     Sum EC**2.
      DO N = 1, NREF
         IF (MKANG(N).GE.0) THEN
            KS = INT(SS4V(N)/SS4DEV) + 1
            IF (KS.GT.NSSA) KS = NSSA
            NSUM(KS) = NSUM(KS) + 1
            SEC2(KS) = SEC2(KS) + EC(N)*EC(N)
         END IF
      END DO
C     Scale factors.
      DO KS = 1, NSSA
         IF (NSUM(KS).GT.0) THEN
            SEC2(KS) = SEC2(KS)/REAL(NSUM(KS))
            SK(KS) = SQRT(1.0/SEC2(KS))
         END IF
      END DO
C     Scale Ecal values.
      DO N = 1, NREF
         IF (MKANG(N).GE.0) THEN
            KS = INT(SS4V(N)/SS4DEV) + 1
            IF (KS.GT.NSSA) KS = NSSA
            EC(N) = SK(KS)*EC(N)
         END IF
      END DO
      RETURN
      END
C     -----------------------------------------------------------
C     Scale Ec**2 to Eo**2.
      SUBROUTINE FASTSC(NREF,MKANG,EV,EC,SS4V,SS4H,NSS4)
C
      INTEGER NREF,NSS4,MKANG(NREF)
      REAL SS4H,EV(NREF),EC(NREF),SS4V(NREF)
C     Local variables.
      INTEGER KS,NSUM(50)
      REAL SEV2(50),SEC2(50),SK(50)
C     Resolution range.
      SS4DEV = SS4H/REAL(NSS4)
      DO I = 1, 50
         NSUM(I) = 0
         SEV2(I) = 0.0
         SEC2(I) = 0.0
         SK(I) = 0.0
      END DO
C     Sum EC**2.
      DO N = 1, NREF
         IF (MKANG(N).GE.0) THEN
            KS = INT(SS4V(N)/SS4DEV) + 1
            IF (KS.GT.50) KS = 50
            NSUM(KS) = NSUM(KS) + 1
            SEC2(KS) = SEC2(KS) + EC(N)*EC(N)
            SEV2(KS) = SEV2(KS) + EV(N)*EV(N)
         END IF
      END DO
C     Scale factors.
      DO KS = 1, 50
         IF (NSUM(KS).GT.0) SK(KS) = SQRT(SEV2(KS)/SEC2(KS))
      END DO
C     Scale Ecal values.
      DO N = 1, NREF
         IF (MKANG(N).GE.0) THEN
            KS = INT(SS4V(N)/SS4DEV) + 1
            IF (KS.GT.50) KS = 50
            EC(N) = SK(KS)*EC(N)
         END IF
      END DO
      RETURN
      END
C     -------------------------------------------------------------
      SUBROUTINE FASTRF(NREF,MKANG,FO,FC,CC)
C     R-factor and CC for all reflections.
      INTEGER NREF,MKANG(NREF)
      REAL FO(NREF),FC(NREF)
C
      REAL SFT,SFC,SK,RF,SNUM,
     +     SFT2,SFC2,SFTC,CC
C     Initial.
      SFT = 0.0
      SFC = 0.0
      SK = 0.0
      RF = 0.0
      SFT2 = 0.0
      SFC2 = 0.0
      SFTC = 0.0
      NUMB = 0
C
      DO N = 1, NREF
         IF (MKANG(N).GE.0) THEN
            NUMB = NUMB + 1
            SFT = SFT + FO(N)
            SFC = SFC + FC(N)
            SFT2 = SFT2 + FO(N)*FO(N)
            SFC2 = SFC2 + FC(N)*FC(N)
            SFTC = SFTC + FO(N)*FC(N)
         END IF
      END DO
         SK = SFT/SFC
      DO N = 1, NREF
         IF (MKANG(N).GE.0) THEN
         RF = RF + ABS(FO(N) - SK*FC(N))
         END IF
      END DO
      RF = RF/SFT
      SNUM = REAL(NUMB)
      CC = (SNUM*SFTC - SFT*SFC)/
     +     (SQRT((SNUM*SFT2 - SFT**2)*
     +     (SNUM*SFC2 - SFC**2)))
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                  C
C              Calculate structure factors from a map              C
C                   using CCP4 symmetry operation.                 C
C                     Version 1998  Yao Jiaxing                    C
C                                                                  C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     ****************                           18 March 1993
C     ***  FFTSF1  ***                           V.Yu.Lunin
C     ****************
      SUBROUTINE FFTSF1(NREF,IH,IK,IL,FMOD,PHI,LBUF,BUF,NX,NY,NZ,RO,
     +     MES,SCALE,F000)
C
      INTEGER IH(NREF),IK(NREF),IL(NREF)
      REAL FMOD(NREF),PHI(NREF),BUF(LBUF),RO(NX,NY,NZ)
      REAL SCALE,F000
      INTEGER NREF,LBUF,NX,NY,NZ,MES
C     Local veriables.
      INTEGER LH,LK,LL,ISI
C
C
C     ATTENTION !
C     A    - phases values are calculated in DEGREES;
C     A    - the length LBUF of BUF-array must be not less then
C     A      NX * NY * (NZ+2);
C     A    - NZ must be even;
C     A    - NX,NY,NZ must be products not more then 14 prime
C     A      factors; maximum prime factor must not exceed 19;
C     A    - if !h!>=NX/2 or !k!>=NY/2 or l<0 or l>=NZ/2, then
C     A      modules and phases values for calculated structure
C     A      factors will be set equal to 0. ;
C     A    - the subroutine returns RO-array unchanged;
C     ATTENTION !
C
C     INPUT DATA
C     RO(.,.,.) - three-dimensional array with NX*NY*NZ
C     dimensions containing ro-function values;
C     NX,NY,NZ - dimensions of RO-array;
C
C     NREF - number of reflections for which structure
C     factors will be calculated;
C         and the size of dimension of IH, IK, IL, FMOD and PHI
C     IH(.),IK(.),IL(.) - their indexes;
C
C     SCALE - the structure factors values are calculated as
C
C     FEXP(I*PHI)=
C
C       SCALE                                H*R   K*S   L*T
C     =  ------- *SUMMA(RO(r,s,t)*EXP(2*PI*i*(--- + --- + ---)))
C       NX*NY*NZ R=0,NX-1                     NX    NY    NZ
C                S=0,NY-1
C                t=0,NZ-1
C
C     (if RO-values are in absolute scale the factor
C     SCALE must be set equal to the unit cell volume to
C     produce structure factors values in absolute scale);
C
C     MES - the number of messages file; if MES < 0 ,then
C     all messages will be suppressed;
C     LBUF - the size of work array BUF(.); it must be not
C     less then NX*NY*(NZ+2); Check it before CALL.
C     BUF(.) - work array;
C
C     OUTPUT DATA
C     FMOD(.),PHI(.) - modules and phases of calculated
C     structure factors
C
C     Degrees to radians.
      DTOR = ATAN(1.0)/45.0
      RTOD = 1.0/DTOR
C
      MAXH = (NX + 1)/2 - 1
      MAXK = (NY + 1)/2 - 1
      MAXL = NZ/2 - 1
      K1 = 2
      K2 = 2*(MAXL + 1)
      K3 = 2*(MAXL + 1)*NX
      NLARGE = NX*NY*(NZ + 2)
C
      CALL SF1SU1(NX,NY,NZ,RO,BUF)
      CALL SF1(BUF,NX,NY,NZ,MAXH,MAXK,MAXL,SCALE,NLARGE)
C
      IOUT = 0
C     Loop to get structure factors.
      DO N = 1, NREF
         FMOD(N) = 0.0
         PHI(N) = 0.0
         LH = IH(N)
         LK = IK(N)
         LL = IL(N)
C     ISI=-1 IF INDEX L < 0.
         ISI = 1
         IF (LL.LT.0) ISI = -1
         IHW = LH*ISI
         IKW = LK*ISI
         ILW = LL*ISI
C     Check the maximum.
         IF (IHW.LT.-MAXH .OR. IHW.GT.MAXH .OR.
     +       IKW.LT.-MAXK .OR. IKW.GT.MAXK .OR.
     +       ILW.LT.0 .OR. ILW.GT.MAXL) THEN
            IOUT = IOUT + 1
         ELSE
            IF(IHW.LT.0) IHW = NX + IHW
            IF(IKW.LT.0) IKW = NY + IKW
            IND = K1*ILW + K2*IHW + K3*IKW + 1
            WRE = BUF(IND)
            WIM = BUF(IND + 1)*REAL(ISI)
            FMOD(N) = SQRT(WRE*WRE + WIM*WIM)
            PHI(N) = 0.0
            IF (FMOD(N).GT.0.0) PHI(N) = RTOD*ATAN2(WIM,WRE)
         END IF
      END DO
      F000 = BUF(1)
      IF (IOUT.GT.0 .AND. MES.GT.0) WRITE(MES,1001) IOUT
      RETURN
 1001 FORMAT(1X,'*** FFTSF1 ***'/
     +     1X,I10,' Structure factors are undefined due to too',
     +     ' large indexes')
      END
C
      SUBROUTINE SF1SU1(NX,NY,NZ,X,Y)
      DIMENSION X(NX,NY,NZ),Y(NZ,NX,NY)
      DO JX = 1, NX
         DO JY = 1, NY
            DO JZ = 1, NZ
               Y(JZ,JX,JY) = X(JX,JY,JZ)
            ENDDO
         ENDDO
      ENDDO
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                  C
C              Calculate a map from structure factors.             C
C                   using CCP4 symmetry operation.                 C
C                     Version 1998  Yao Jiaxing                    C
C                                                                  C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  ****************                           18 March 1993
C  ***  FFTFT1  ***                           V.Yu.Lunin
C  ****************
      SUBROUTINE FFTFT1(NREF,IH,IK,IL,FMOD,PHI,LBUF,BUF,NX,NY,NZ,RO,
     +     NSYM,RSYM,IPAT,MES,SCALE,F000)
C
      INTEGER IH(NREF),IK(NREF),IL(NREF)
      REAL FMOD(NREF),PHI(NREF),BUF(LBUF),RO(NX,NY,NZ),RSYM(4,4,NSYM)
      REAL SCALE,F000
      INTEGER NREF,LBUF,NX,NY,NZ,NSYM,IPAT,MES
C     Local veriables.
      INTEGER LH,LK,LL
C
C
C  ATTENTION !
C  A    - phases values must be set  in DEGREES;
C  A    - the length LBUF of BUF-array must be not less then
C  A      NX * NY * (NZ+2); Check it before CALL.
C  A    - NZ must be even;
C  A    - NX,NY,NZ must be products not more then 14 prime
C  A      factors; maximum prime factor must not exceed 19;
C  A    - the structure factors with
C  A      !h!>=NX/2 or !k!>=NY/2 or l<0 or l>=NZ/2
C  A      are not used for synthesis calculation;
C  ATTENTION !
C
C  INPUT DATA
C      NX,NY,NZ - dimensions of RO-array;
C
C      NREF - number of reflections designed to calculate synthesis
C             AND size of dimension of IH, IK, IL, FMOD and PHI
C      IH(.),IK(.),IL(.) - their indexes;
C      FMOD(.),PHI(.) - modules and phases of structure factors;
C
C      SCALE - the synthesis is calculated
C         for R=0,NX-1; S=0,NY-1; T=0,NZ-1 as
C         RO(R/NX,S/NY,T/NZ)=
C                                                 H*R   K*S   L*T
C        = SCALE *SUMMA(F*EXP(i*PHI)*EXP(-2*PI*i*(--- + --- + ---)))
C                 H,K,L                            NX    NY    NZ
C
C         (if F-values are in absolute scale the factor
C         SCALE must be set equal to 1./Vcell   to
C         produce electron density values in absolute scale);
C
C      MES - the number of messages file; if MES < 0 ,then
C         all messages will be suppressed;
C      LBUF - the size of work array BUF(.); it must be not
C         less then NX*NY*(NZ+2);
C      BUF(.) - work array;
C      IPAT - 0 for electron density map.
C             1 for patterson map.
C
C  OUTPUT DATA
C      RO(.,.,.) - three-dimensional array with NX*NY*NZ
C         dimensions containing RO-function values;
C     Degrees to radians.
      DTOR = ATAN(1.0)/45.0
C     Maximum index.
      MAXH = (NX + 1)/2 - 1
      MAXK = (NY + 1)/2 - 1
      MAXL = NZ/2 - 1
      MINX = 0
      MAXX = NX - 1
      MINY = 0
      MAXY = NY - 1
      K1 = 2
      K2 = 2*(MAXL + 1)
      K3 = 2*(MAXL + 1)*NX
      NLARGE = NX*NY*(NZ + 2)
C     Clear the BUF.
      DO J = 1, NLARGE
         BUF(J) = 0.
      ENDDO
C
      IOUT = 0
C     Loop for reflections.
      DO N = 1, NREF
C     Equivalent reflections.
         DO J = 1, NSYM
            LH = NINT(RSYM(1,1,J)*IH(N) + RSYM(2,1,J)*IK(N) +
     +          RSYM(3,1,J)*IL(N))
            LK = NINT(RSYM(1,2,J)*IH(N) + RSYM(2,2,J)*IK(N) +
     +          RSYM(3,2,J)*IL(N))
            LL = NINT(RSYM(1,3,J)*IH(N) + RSYM(2,3,J)*IK(N) +
     +          RSYM(3,3,J)*IL(N))
            JS = 1
            IF (LL.LT.0) JS = -1
            SHIFT = 0.0
            IF (IPAT.EQ.0) THEN
               SHIFT = -360.0*(IH(N)*RSYM(1,4,J) +
     +             IK(N)*RSYM(2,4,J) + IL(N)*RSYM(3,4,J))
               SHIFT = AMOD(SHIFT,360.0)
            END IF
C     Fill the BUF.
            IHW = LH*JS
            IKW = LK*JS
            ILW = LL*JS
            FW = FMOD(N)
            PW = DTOR*(PHI(N)+SHIFT)*REAL(JS)
            IF (IHW.GE.-MAXH .AND. IHW.LE.MAXH .AND.
     +          IKW.GE.-MAXK .AND. IKW.LE.MAXK .AND.
     +          ILW.GE.0 .AND. ILW.LE.MAXL) THEN
               WRE = FW*COS(PW)
               WIM = FW*SIN(PW)
               IF (IHW.LT.0) IHW = NX + IHW
               IF (IKW.LT.0) IKW = NY + IKW
               IND = K1*ILW + K2*IHW + K3*IKW + 1
               BUF(IND) = WRE
               BUF(IND + 1) = WIM
C     All values are needed for L=0 plane.
               IF (ILW.EQ.0) THEN
                  IHW = -IHW
                  IKW = -IKW
                  PW = -PW
                  WRE = FW*COS(PW)
                  WIM = FW*SIN(PW)
                  IF (IHW.LT.0) IHW = NX + IHW
                  IF (IKW.LT.0) IKW = NY + IKW
                  IND = K1*ILW + K2*IHW + K3*IKW + 1
                  BUF(IND) = WRE
                  BUF(IND + 1) = WIM
               END IF
            ELSE
               IOUT = IOUT + 1
            END IF
         END DO
      END DO
C     Include (000)
      BUF(1) = F000
C
      IF(IOUT.GT.0 .AND. MES.GT.0) WRITE(MES,1001) IOUT
C
      CALL FT1(BUF,NX,NY,NZ,MAXH,MAXL,
     +     MINX,MAXX,MINY,MAXY,NLARGE)
      CALL FT1SU1(NX,NY,NZ,BUF,RO,SCALE)
C
      RETURN
C
 1000 FORMAT(1X,'*** FFTFT1 ***'/
     +     1X,'BUF-array must be extended for',I8,' at least')
 1001 FORMAT(1x,'*** FFTFT1 ***'/
     +     1X,I10,' reflections were not used due to too',
     +     ' large indexes')
C
      END
C
C
      SUBROUTINE FT1SU1(NX,NY,NZ,Y,X,SCALE)
      DIMENSION X(NX,NY,NZ),Y(NZ,NX,NY)
C
      DO JX = 1, NX
         DO JY = 1, NY
            DO JZ = 1, NZ
               X(JX,JY,JZ) = Y(JZ,JX,JY)*SCALE
            ENDDO
         ENDDO
      ENDDO
C
      RETURN
      END
C*******************************************************************
C*******************************************************************
C
C                                                         29.03.85
C        FT1                                             V.Yu.Lunin
C
C
C        The subroutine performs the Fourier transform
C
C
C                                                H*R   K*S   L*T
C        RO(R,S,T)=SUMMA   F(H,K,L)*EXP(-2*PI*I*(--- + --- + ---))
C                  H=0,NX-1                       NX    NY    NZ
C                  K=0,NY-1
C                  L=0,NZ-1
C
C        for  MINX<=R<=MAXX,MINY<=S<=MAXY,MINZ<=T<=MAXZ
C        suppousing RO(.,.,.) be a real-value function.
C
C
C        INPUT DATA
C        X - data to be transformed;
C            this array is considered as three-dimensional COMPLEX
C            array with dimensions (maxl+1) Nx Ny;
C            elements X(l+1,h+1,k+1) contain values of structure
C            factors (Fourier coefficients) F(h,k,l)  for
C                  -MAXH<=H<=MAXH,-MAXK<=K<=MAXK,0<=L<=MAXL
C                  (it is assumed that F(-H,K,L)=F(NX-H,K,L) ,
C                                 F(H,-K,L)=F(H,NY-K ,L) )
C            and zero values for other indexes.
C         Nx,Ny,Nz - numbers of divisions along unit cell axes;
C         maxh,maxk,maxl - restrict the indexes of nonzero structure
C            factors,
C         minx,maxx,miny,maxy,minz,maxz - restrict the part of the
C            unit cell there transform will be calculated;
C         mes - messages file
C
C         OUTPUT DATA
C         X - Fourier synthesis Ro;
C             this array is considered as three-dimensional REAL array
C             with dimensions Nz  Nx  Ny;
C             elements X(t+1,r+1,s+1) contain values Ro(r,s,t);
C             the only defined Ro-values are ones with
C             minx<=r<=maxx, miny<=s<=maxy, minz<=t<=maxz;
C
C         ATTENTION
C           - X-array musr be described as [huge]
C           - the total length X-array (as real one) must be
C             Nx*Ny*Nz at least;
C           - the indexes order in input data  l h k
C                               in output data z x y;
C
C           -    NZ - must be even;
C                NX >= 2*MAXH+1
C                NY>=2*MAXK+1
C                NZ>=2*MAXL+2
C                0<=MINX<=MAXX<=NX-1
C                0<=MINY<=MAXY<=NY-1
C                0<=MINZ<=MAXZ<=NZ-1
C           - the Hermitian  symmetry
C                                        *                            *
C                     F(H,K,0)=F(-H,-K,0)   ,F(H,K,NZ/2)=F(-H,-K,NZ/2)
C              must be present in input data.
C
C
C         SUBROUTINE USED
C            one-dimensional Fast Fourier Transform package of
C            L.F.Ten Eyck;
C            FT1A
C
C
      SUBROUTINE FT1(X,NX,NY,NZ,MAXH,MAXL,MINX,MAXX,MINY,MAXY,
     +              NLARGE)
C
      DIMENSION X(NLARGE),K(5)
C
C     External Routines.
      EXTERNAL CMPLFT,HERMFT
C
C
      K(1)=2*(MAXL+1)*NX*NY
      K(2)=2*(MAXL+1)*NX
      K(3)=2*(MAXL+1)*NX*NY
      K(4)=2*(MAXL+1)*(MAXH+1)
      K(5)=2
      CALL CMPLFT(X(1),X(2),NY,K)
C
      K(4)=2*(MAXL+1)*MAXH
      JA=(NX-MAXH)*2*(MAXL+1)+1
      CALL CMPLFT(X(JA),X(JA+1),NY,K)
C
C
C
      JA=MINY*2*(MAXL+1)*NX+1
      K(1)=2*(MAXL+1)*NX*(MAXY-MINY+1)
      K(2)=2*(MAXL+1)
      K(3)=2*(MAXL+1)*NX
      K(4)=2*(MAXL+1)
      K(5)=2
      CALL CMPLFT(X(JA),X(JA+1),NX,K)
C
C
C
      N3=2*(MAXL+1)
      IF(N3.EQ.NZ) GOTO 50
      CALL FT1A(X,N3,NX,NY,X,NZ,NX,NY)
50    CONTINUE
C
C
      JA=    NZ*NX*MINY+    NZ*MINX+1
      K(1)=    NZ*NX*(MAXY-MINY+1)
      K(2)=2
      K(3)=    NZ*NX
      K(4)=    NZ*(MAXX-MINX+1)
      K(5)=NZ
C
C
C
      K1=K(1)
      K3=K(3)
      K5=K(5)
C
C
      IF(N3.EQ.NZ) GOTO 220
      DO 210 J1=JA,K1,K3
      K4=K(4)+J1-1
      DO 200 J=J1,K4,K5
      K6=J+N3
      K7=J+NZ-1
      DO 190 J2=K6,K7
      X(J2)=0.
190   CONTINUE
200   CONTINUE
210   CONTINUE
C
C
220   CONTINUE
C
C
C
      CALL HERMFT(X(JA),X(JA+1),NZ/2,K)
C
      RETURN
      END
C
C     The  decompressing the X-array into Y-array.
C
C     ATTENTION !
C     It is suppoused n1<=k1, n2<=k2, n3<=k3.
C
      SUBROUTINE FT1A(X,n1,n2,n3,Y,k1,k2,k3)
C
      dimension X(n1,n2,n3), Y(k1,k2,k3)
C
      do 50 i3=1,n3
        m3=n3-i3+1
        do 40 i2=1,n2
          m2=n2-i2+1
          do 30 i1=1,n1
            m1=n1-i1+1
            Y(m1,m2,m3)=X(m1,m2,m3)
30       continue
40      continue
50    continue
C
C
      return
      end
C
C      SF1                                              28.03.85
C                                                   V.Yu.Lunin
C
C       The program calculates inverse Fourier transform
C
C         F(H,K,L)=(SCALE/(NX*NY*NZ))*
C                                                  H*R K*S L*T
C                  * SUMMA  (RO(R,S,T)*EXP(2*PI*I*(---+---+---)) )
C                    R=0,NX-1                      NX  NY   NZ
C                    S=0,NY-1
C                    T=0,NZ-1
C
C       for  -MAXH<=H<=MAXH,-MAXK<=K<=MAXK,0<=L<=MAXL
C
C       INPUT DATA
C       X - containes RO(r,s,t) values; X-array is considered as
C           three-dimensional with dimensions Nz Nx Ny;
C           the value RO(r,s,t) is in X(t+1,r+1,s+1);
C       Nx,Ny,Nz - grid numbers (dimensions of RO-array);
C       maxh,maxk,maxl - define the set of calculating F-values;
C	 SCALE - scale-value;
C       mes - messages file
C
C       OUTPUT DATA
C       X - containes calculated F(h,k,l)-values; it is considered
C           as three-dimensional array with dimensions
C           (maxl+1) Nx Ny; the value F(h,k,l) is in X(l+1,h+1,k+1)
C                   (assuming that  F(-H,K,L)=F(NX-H,K,L),
C                                   F(H,-K,L)=F(H,NY-K,L)  )
C           the only defined elements are ones with
C           -maxh<=h<=maxh, -maxk<=k<=maxk, 0<=l<=maxl.
C
C       ATTENTION !
C          - the real size of X-array must be (Nz+2)*Nx*Ny at least !
C          - must be
C                             2*MAXH+1<=NX
C                             2*MAXK+1<=NY
C                             2*MAXL+2<=NZ
C                             NZ - even;
C          - the axes order : input data z x y ,
C                             ouput data l h k ;
C          - pay your attention to F-data placing in X-array
C            (especially for negative h and k indexes)
C          - X-array must be described as [huge].
C
C       SUBROUTINES USED
C          One-dimensional Fast Fourier Transform package of
C          L.F.Ten Eyck;
C          SF1A, SF1B.
C
      SUBROUTINE SF1(X,NX,NY,NZ,MAXH,MAXK,MAXL,SCALE,NLARGE)
C
      DIMENSION X(NLARGE),K(5)
C
C     External Routines.
      EXTERNAL CMPLFT,REALFT
C	 decompressing X-array; The first dimension will be Nz+2
C
      CALL SF1A(X,NZ,NX,NY,X,NZ+2,NX,NY)
C
      NTOT=(NZ+2)*NX*NY
      K(1)=NTOT
      K(2)=2
      K(3)=NTOT
      K(4)=NTOT
      K(5)=NZ+2
      CALL REALFT(X(1),X(2),NZ/2,K)
C
C         compressing  X - array;
C         the first dimension will be equal to  2*(MAXL+1)
C
      N3=2*(MAXL+1)
      CALL SF1B(X,NZ+2,NX,NY,X,N3,NX,NY)
C
C
C
      K(1)=2*(MAXL+1)*NX*NY
      K(2)=2*(MAXL+1)
      K(3)=2*(MAXL+1)*NX
      K(4)=2*(MAXL+1)
      K(5)=2
      CALL CMPLFT(X(1),X(2),NX,K)
C
C
C
      K(1)=2*(MAXL+1)*NX*NY
      K(2)=2*(MAXL+1)*NX
      K(3)=2*(MAXL+1)*NX*NY
      K(4)=2*(MAXL+1)*(MAXH+1)
      K(5)=2
      CALL CMPLFT(X(1),X(2),NY,K)
C
      K(4)=2*(MAXL+1)*MAXH
      JA=(NX-MAXH)*2*(MAXL+1)+1
      CALL CMPLFT(X(JA),X(JA+1),NY,K)
C
C
C         scaling and transition to complex conjugate
C
      CONST=SCALE/(NX*NY*NZ)
      N1=2*(MAXL+1)*NX*(MAXK+1)
      W=-CONST
      DO 10 J=1,N1
      W=-W
      X(J)=W*X(J)
10    CONTINUE
C
      N2=2*(MAXL+1)*NX*(NY-MAXK)+1
      N3=2*(MAXL+1)*NX*NY
      W=-CONST
      DO 20 J=N2,N3
      W=-W
      X(J)=W*X(J)
20    CONTINUE
C
C
C
      RETURN
      END
C
      SUBROUTINE SF1A(X,n1,n2,n3,Y,k1,k2,k3)
C
C	Decompressing X-array into Y-array;
C      must be n1<=k1, n2<=k2, n3<=k3
C
      dimension X(n1,n2,n3), Y(k1,k2,k3)
C
      do 50 i3=1,n3
        m3=n3-i3+1
        do 40 i2=1,n2
          m2=n2-i2+1
          do 30 i1=1,n1
            m1=n1-i1+1
            Y(m1,m2,m3)=X(m1,m2,m3)
30       continue
40      continue
50    continue
C
C
      return
      end
C     ------------------------------------------------------------
C
      SUBROUTINE SF1B(X,n1,n2,n3,Y,k1,k2,k3)
      dimension X(n1,n2,n3), Y(k1,k2,k3)
C
C      compressing array X into Y begining from the first element
C      must be n1>=k1, n2>=k2, n3>=k3
C
      do 50 i3=1,k3
        do 40 i2=1,k2
          do 30 i1=1,k1
            Y(i1,i2,i3)=X(i1,i2,i3)
30       continue
40      continue
50    continue
C
      return
      end
C     ------------------------------------------------------------
C     CAlculate phase errors.
      SUBROUTINE ERRNEW(NREF,IH,IKKT,IL,NOX,NOY,NOZ,PHIFT,PHIFP
     +     ,WT,MDIS,KP,NSYMP,NSET,NTRYS,NCYC,MESG,ICODE,ERROR)
C
      CHARACTER*4 MESG
      INTEGER NREF,NOX,NOY,NOZ,MDIS,KP,NSYMP,NSET,NTRYS,NCYC
      REAL ERROR
      INTEGER IH(NREF),IKKT(NREF),IL(NREF),ICODE(NREF)
      REAL PHIFT(NREF),PHIFP(NREF),WT(NREF)
C     Local variables.
      COMPLEX X(2097152)
C
      CALL FERNEW(NREF,ERROR,X,NOX,NOY,NOZ,IH,IKKT,IL,PHIFT
     +     ,PHIFP,WT,MDIS,KP,NSYMP,NSET,NTRYS,NCYC,MESG,ICODE)
      RETURN
      END
C
C-----------------------------------------------------------
      SUBROUTINE FERNEW(NREF,ERROR,X,NOX,NOY,NOZ,IH,IKKT,IL,PHIFT
     +     ,PHIFP,WT,MDIS,KP,NSYMP,NSET,NTRYS,NCYC,MESG,ICODE)
C
      CHARACTER*4 MESG
      INTEGER NREF,NOX,NOY,NOZ,MDIS,KP,NSYMP,NSET,NTRYS,NCYC
      INTEGER IH(NREF),IKKT(NREF),IL(NREF),ICODE(NREF)
      REAL ERROR
      REAL PHIFT(NREF),PHIFP(NREF),WT(NREF)
C
      COMPLEX X(NOX,NOY,NOZ)
C     Local variables.
      INTEGER NDIS(50),NDET,LXS,LYS,LZS
      REAL DIS(50),XYZP(3)
      DTOR = ATAN(1.0)/45.0
      DO 1050 I = 1, 2
         IF (I.EQ.1) NDET = 0
         DO 300 KS = 1, 50
            NDIS(KS) = 0
            DIS(KS) = 0.0
 300     CONTINUE
         RMAX = 0.
         DO 310 LX = 1, NOX
            DO 310 LY = 1, NOY
               DO 310 LZ = 1, NOZ
 310     X(LX,LY,LZ) = (0., 0.)
         DO 320 LL = 1, NREF
            IF (ICODE(LL).LE.(KP+1)) THEN
               IF (WT(LL).GT.0.0001) THEN
                  IF (I.EQ.1) NDET = NDET + 1
                  SM = 1.0
                  IF (I.EQ.2) SM = -1.0
                  FF = (PHIFT(LL) - SM*PHIFP(LL))*DTOR
                  AA = COS(FF)
                  BB = -SIN(FF)
                  JH = IH(LL)
                  JK = IKKT(LL)
                  JL = IL(LL)
                  LX = MOD(JH,NOX) + 1
                  IF(LX.LE.0) LX = LX + NOX
                  LY = MOD(JK,NOY) + 1
                  IF(LY.LE.0) LY = LY + NOY
                  LZ = MOD(JL,NOZ) + 1
                  IF(LZ.LE.0) LZ = LZ + NOZ
                  X(LX,LY,LZ) = X(LX,LY,LZ) + CMPLX(AA,BB)
               END IF
            END IF
 320     CONTINUE
         CALL ABC(X,NOX,NOY,NOZ,-1)
         DO 350 LX = 1, NOX
            DO 350 LY = 1, NOY
               DO 350 LZ = 1, NOZ
                  R = REAL(X(LX,LY,LZ))
                  IF(R.LE.RMAX)GO TO 350
                  RMAX = R
                  DXM = (REAL(LX) - 1.)/REAL(NOX)
                  DYM = (REAL(LY) - 1.)/REAL(NOY)
                  DZM = (REAL(LZ) - 1.)/REAL(NOZ)
                  IF (NSYMP.EQ.1) THEN
                     LXS = LX
                     LYS = LY
                     LZS = LZ
                  END IF
 350     CONTINUE
         IF (NSYMP.EQ.1) THEN
            CALL PEAK(X,NOX,NOY,NOZ,LXS,LYS,LZS,XYZP)
            DXM = DXM + XYZP(1)
            DYM = DYM + XYZP(2)
            DZM = DZM + XYZP(3)
         END IF
         ERM = 0.0
         DO 200 LL = 1, NREF
            IF (ICODE(LL).LE.(KP+1)) THEN
               IF (WT(LL).GT.0.0001) THEN
                  JH = IH(LL)
                  JK = IKKT(LL)
                  JL = IL(LL)
                  DEL = 360.*(JH*DXM + JK*DYM + JL*DZM) + PHIFT(LL)
     +                - SM*PHIFP(LL)
                  DEL = ABS(AMOD(DEL + 1800.0, 360.))
                  IF (DEL.GT.180.) DEL = 360. - DEL
                  ERM = ERM + DEL
                  KS = INT(WT(LL)*10.0) + 1
                  IF (KS.GT.MDIS) KS = MDIS
                  NDIS(KS) = NDIS(KS) + 1
                  DIS(KS) = DIS(KS) + DEL
               END IF
            END IF
 200     CONTINUE
         ERM = ERM/REAL(NDET)
         IF (I.EQ.1) ERROR = ERM
         IF (I.EQ.2) THEN
            IF (ERM.LT.ERROR) ERROR = ERM
            GOTO 900
         END IF
         ERM1 = ERM
         DXM1 = DXM
         DYM1 = DYM
         DZM1 = DZM
         SM1 = SM
         GOTO 950
 900     ERM2 = ERM
 950     DO 1000 KS = 1, MDIS
            IF(NDIS(KS).NE.0) DIS(KS) = DIS(KS)/REAL(NDIS(KS))
 1000    CONTINUE
         IF (I.EQ.1) WRITE(6,FMT='(/,A,3F10.5,/)')
     +       ' Phase error vs weight with origin shift',DXM,DYM,DZM
         IF (I.EQ.2) WRITE(6,FMT='(/,A,3F10.5,/)')
     +       ' Enantiomorph with origin shift',DXM,DYM,DZM
         WRITE(6,6) (0.1*REAL(KS), KS = 1, MDIS),MESG,NSET,NTRYS,NCYC
         WRITE(6,7) (NDIS(KS), KS = 1, MDIS),NDET,MESG,NSET,NTRYS,NCYC
         WRITE(6,8) (DIS(KS), KS = 1, MDIS),ERM,MESG,NSET,NTRYS,NCYC
  6      FORMAT(' Weight',2X,10F6.1,' Total',1X,A,3I5)
  7      FORMAT(' Number',2X,11I6,1X,A,3I5)
  8      FORMAT(' Errors',2X,11F6.1,1X,A,3I5)
 1050 CONTINUE
      IF (KP.EQ.0) RETURN
C     Change the phases to new origin.
      IF (ERM2.LT.ERM1) GOTO 1100
      DXM = DXM1
      DYM = DYM1
      DZM = DZM1
      SM = SM1
 1100 DO 1200 LL = 1, NREF
         IF (WT(LL).GT.0.0001) THEN
            JH = IH(LL)
            JK = IKKT(LL)
            JL = IL(LL)
            DEL = AMOD(360.*(JH*DXM + JK*DYM + JL*DZM), 360.0)
            IF (DEL.LT.0.0) DEL = 360.0 + DEL
            DEL = SM*PHIFP(LL) - DEL
            PHIFP(LL) = AMOD(DEL, 360.0)
            IF (PHIFP(LL).LE.0) PHIFP(LL) = 360.0 + PHIFP(LL)
         END IF
 1200 CONTINUE
      RETURN
      END
C
C-----------------------------------------------------------------------
      SUBROUTINE ABC(X,K,L,M,ISIGN)
      COMPLEX X(2097152),FX,FY,V,W,F
      DIMENSION IM(3),IN(3),ICA(3)
      PI=4.*ATAN(1.0)
      IM(1)=K
      IM(2)=L
      IM(3)=M
      DO 15 J=1,3
         IA=1
         IB=0
 11      IC=IM(J)-IA
         IF(IC)12,15,10
 10      IA=IA+IA
         IB=IB+1
         GO TO 11
 15   IM(J)=IB
      IF(ISIGN*ISIGN)12,12,17
 12   PRINT 100,K,L,M,ISIGN
 100  FORMAT(' * ERROR ABC * ',4I11)
      RETURN
 17   CALL REV(X,K,L,M)
      IN(1)=1
      IN(2)=2
      IN(3)=3
      DO 75 ID=1,2
         IA=IN(1)
         IB=IN(2)
         IF(IM(IB)-IM(IA))76,77,77
 76      IN(1)=IB
         IN(2)=IA
 77      IB=IN(2)
         IC=IN(3)
         IF(IM(IC)-IM(IB))78,75,75
 78      IN(2)=IC
         IN(3)=IB
 75   CONTINUE
      II=1
      II2=2
      ICC=ISIGN
      KL=K*L
      KLM=KL*M
      IK=K
      IK2=K+K
      IKL=KL
      IKL2=KL+KL
      DO 80 J=1,3
         ICA(J)=0
         IF(IM(J))80,80,81
 81      ICA(J)=1
 80   CONTINUE
      IA=0
      DO 85 J=1,3
         IB=IN(J)
         IMIN=IM(IB)-IA
         IF(IMIN) 16,16,21
 21      DO 20 IG=1,IMIN
            F=(0.0,1.0)
            F=F*(PI/ICC)
            W=CEXP(F)
            IF(ICA(1))24,24,23
 23        V=(1.0,0.0)
            DO 25 IA=1,II
               DO 30 IB=IA,K,II2
                  DO 30 IC=IB,KL,K
                     DO 30 ID=IC,KLM,KL
                        IE=ID+II
                        FX=X(ID)
                        FY=X(IE)*V
                        X(ID)=FX+FY
 30           X(IE)=FX-FY
 25        V=V*W
 24        IF(ICA(2))27,27,26
 26        V=(1.0,0.0)
            DO 35 IA=K,IK,K
               DO 40 IB=1,K
                  IF=IA+IB-K
                  DO 40 IC=IF,KL,IK2
                     DO 40 ID=IC,KLM,KL
                        IE=ID+IK
                        FX=X(ID)
                        FY=X(IE)*V
                        X(ID)=FX+FY
 40           X(IE)=FX-FY
 35        V=V*W
 27        IF(ICA(3))22,22,28
 28        V=(1.0,0.0)
            DO 45 IA=KL,IKL,KL
               DO 50 IB=1,K
                  IF=IA+IB-KL
                  DO 50 IC=IF,IA,K
                     DO 50 ID=IC,KLM,IKL2
                        IE=ID+IKL
                        FX=X(ID)
                        FY=X(IE)*V
                        X(ID)=FX+FY
 50           X(IE)=FX-FY
 45        V=V*W
 22        II=II+II
            II2=II2+II2
            IK=IK+IK
            IK2=IK2+IK2
            IKL=IKL+IKL
            IKL2=IKL2+IKL2
 20      ICC=ICC+ICC
 16      IB=IN(J)
         IA=IM(IB)
 85   ICA(IB)=0
      IF(ISIGN)19,19,29
 29   C=1.0/KLM
      DO 90 IA=1,KLM
 90   X(IA)=X(IA)*C
 19   RETURN
      END
C----------------------------------------------------------------------
      SUBROUTINE REV(X,K,L,M)
      COMPLEX X(K,L,M),F
      DO 10 MA=1,M
         MM=MIR(MA-1,M)+1
         IF(MM-MA)10,11,21
 21      DO 20 LA=1,L
            LL=MIR(LA-1,L)+1
            DO 20 KA=1,K
               KK=MIR(KA-1,K)+1
               F=X(KA,LA,MA)
               X(KA,LA,MA)=X(KK,LL,MM)
 20      X(KK,LL,MM)=F
         GO TO 10
 11      DO 50 LA=1,L
            LL=MIR(LA-1,L)+1
            IF(LL-LA)50,12,16
 16        DO 15 KA=1,K
               KK=MIR(KA-1,K)+1
               F=X(KA,LA,MA)
               X(KA,LA,MA)=X(KK,LL,MM)
 15        X(KK,LL,MM)=F
            GO TO 50
 12        DO 60 KA=1,K
               KK=MIR(KA-1,K)+1
               IF(KK-KA)60,60,13
 13           F=X(KA,LA,MA)
               X(KA,LA,MA)=X(KK,LL,MM)
               X(KK,LL,MM)=F
 60        CONTINUE
 50      CONTINUE
 10   CONTINUE
      RETURN
      END
C----------------------------------------------------------------------
      FUNCTION MIR(I,K)
      KA = K
      IA = I
      MIR = 0
    1 IF ((KA - 1)*IA) 3, 3, 2
    2 KA = KA/2
      IB = IA/2
      MIR = MIR + KA*(IA - IB - IB)
      IA = IB
      GO TO 1
    3 RETURN
      END
C     -----------------------------------------------------------------
      SUBROUTINE PEAK(RO,LM,MM,NM,L,M,N,XYZP)
C
C     The density map RO with (LM,MM,NM) elements in the (X,Y,Z)  
C     directions
C     has the highest value at grid point (L,M,N). The array BB contains
C     the coordinates of the maximum on exit.
C
      INTEGER LM,MM,NM,L,M,N
      COMPLEX RO(LM,MM,NM)
      REAL XYZP(3)
C     Local.
      REAL A(10,10),B(10,1),C(27),D(27,10),AA(3,3),BB(3,1)
C
      DO 10 I = -1, 1
         DO 10 J = -1, 1
            DO 10 K = -1, 1
               II = I + 2 + 3*(J + 1) + 9*(K + 1)
               IL = MOD(I + L + LM, LM)
               JM = MOD(J + M + MM, MM)
               KN = MOD(K + N + NM, NM)
               IF (IL.EQ.0) IL = LM
               IF (JM.EQ.0) JM = MM
               IF (KN.EQ.0) KN = NM
               C(II) = REAL(RO(IL,JM,KN))
               D(II,1) = I*I
               D(II,2) = J*J
               D(II,3) = K*K
               D(II,4) = I*J
               D(II,5) = I*K
               D(II,6) = J*K
               D(II,7) = I
               D(II,8) = J
               D(II,9) = K
               D(II,10) = 1
 10   CONTINUE
C
      DO 30 I = 1, 10
         DO 30 J = 1, 10
            A(I,J) = 0.0
            DO 20 K = 1, 27
               A(I,J) = A(I,J) + D(K,I)*D(K,J)
 20        CONTINUE
 30   CONTINUE
      DO 50 I = 1, 10
         B(I,1) = 0.0
         DO 40 K = 1, 27
            B(I,1) = B(I,1) + C(K)*D(K,I)
 40      CONTINUE
 50   CONTINUE
      CALL GAUSSJ(A,10,10,B,1,1,NOS)
      IF (NOS.EQ.1) CALL CCPERR(1,
     +       ' *** Error - No solution from GAUSSJ-1 ***')
C     Coefficients of Polynomial Function in B.
      AA(1,1) = 2.0*B(1,1)
      AA(1,2) = B(4,1)
      AA(1,3) = B(5,1)
      AA(2,1) = B(4,1)
      AA(2,2) = 2.0*B(2,1)
      AA(2,3) = B(6,1)
      AA(3,1) = B(5,1)
      AA(3,2) = B(6,1)
      AA(3,3) = 2.0*B(3,1)
      BB(1,1) = -B(7,1)
      BB(2,1) = -B(8,1)
      BB(3,1) = -B(9,1)
      CALL GAUSSJ(AA,3,3,BB,1,1,NOS)
      IF (NOS.EQ.1) CALL CCPERR(1,
     +       ' *** Error - No solution from GAUSSJ-2 ***')
      XYZP(1) = BB(1,1)/REAL(LM)
      XYZP(2) = BB(2,1)/REAL(MM)
      XYZP(3) = BB(3,1)/REAL(NM)
      RETURN
      END
C     -----------------------------------------------------------
C     Linear equation solution by Gauss-Jordan Elimination.
      SUBROUTINE GAUSSJ(A,N,NP,B,M,MP,NOS)
      PARAMETER (NMAX=50)
      DIMENSION A(NP,NP),B(NP,MP),IPIV(NMAX),INDXR(NMAX),INDXC(NMAX)
      DO 11 J = 1, N
         IPIV(J) = 0
 11   CONTINUE
      NOS = 0
      DO 22 I = 1, N
         BIG = 0.
         DO 13 J = 1, N
            IF(IPIV(J).NE.1) THEN
               DO 12 K = 1, N
                  IF (IPIV(K).EQ.0) THEN
                     IF (ABS(A(J,K)).GE.BIG)THEN
                        BIG = ABS(A(J,K))
                        IROW = J
                        ICOL = K
                     END IF
                  ELSE IF (IPIV(K).GT.1) THEN
                     GOTO 1000
                  END IF
 12           CONTINUE
            END IF
 13      CONTINUE
         IPIV(ICOL) = IPIV(ICOL)+1
         IF (IROW.NE.ICOL) THEN
            DO 14 L = 1, N
               DUM = A(IROW,L)
               A(IROW,L) = A(ICOL,L)
               A(ICOL,L) = DUM
 14        CONTINUE
            DO 15 L = 1, M
               DUM = B(IROW,L)
               B(IROW,L) = B(ICOL,L)
               B(ICOL,L) = DUM
 15        CONTINUE
         END IF
         INDXR(I) = IROW
         INDXC(I) = ICOL
         IF (A(ICOL,ICOL).EQ.0.) GOTO 1000
         PIVINV = 1./A(ICOL,ICOL)
         A(ICOL,ICOL) = 1.
         DO 16 L = 1, N
            A(ICOL,L) = A(ICOL,L)*PIVINV
 16      CONTINUE
         DO 17 L = 1, M
            B(ICOL,L) = B(ICOL,L)*PIVINV
 17      CONTINUE
         DO 21 LL = 1, N
            IF (LL.NE.ICOL) THEN
               DUM = A(LL,ICOL)
               A(LL,ICOL) = 0.
               DO 18 L = 1, N
                  A(LL,L) = A(LL,L)-A(ICOL,L)*DUM
 18           CONTINUE
               DO 19 L = 1, M
                  B(LL,L) = B(LL,L)-B(ICOL,L)*DUM
 19           CONTINUE
            END IF
 21      CONTINUE
 22   CONTINUE
      DO 24 L = N, 1, -1
         IF (INDXR(L).NE.INDXC(L))THEN
            DO 23 K = 1, N
               DUM = A(K,INDXR(L))
               A(K,INDXR(L)) = A(K,INDXC(L))
               A(K,INDXC(L)) = DUM
 23        CONTINUE
         END IF
 24   CONTINUE
      RETURN
 1000 NOS = 1
      RETURN
      END
C-----------------------------------------------------------------------
C     Determine the phase restrictions for the reflection..
      SUBROUTINE RESTRI(NREF,IH,IK,IL,MKANG,NSYMP,RSYML)
C
      INTEGER IH(NREF),IK(NREF),IL(NREF),MKANG(NREF),NSYMP
      REAL RSYML(4,4,192)
C     Local.
      INTEGER IND(3),KHL(4)
C     MKANG = 1 means general reflection.
      DO 5000 N = 1, NREF
         MKANG(N) = 1
         IND(1) = IH(N)
         IND(2) = IK(N)
         IND(3) = IL(N)
C     Generate symmetry related reflections.
         DO 2180 I = 1, NSYMP
            KHL(1) = NINT(RSYML(1,1,I))*IND(1)
     +          + NINT(RSYML(2,1,I))*IND(2)
     +          + NINT(RSYML(3,1,I))*IND(3)
            KHL(2) = NINT(RSYML(1,2,I))*IND(1)
     +          + NINT(RSYML(2,2,I))*IND(2)
     +          + NINT(RSYML(3,2,I))*IND(3)
            KHL(3) = NINT(RSYML(1,3,I))*IND(1)
     +          + NINT(RSYML(2,3,I))*IND(2)
     +          + NINT(RSYML(3,3,I))*IND(3)
            KHL(4) = 0
            DO 2140 K=1,3
               KHL(4) = KHL(4) + IND(K)*NINT(RSYML(K,4,I)*12.0)
 2140      CONTINUE
C     Has friedel opposite been generated?
            DO 2160 K=1,3
               IF (IND(K) + KHL(K)) 2180,2160,2180
 2160      CONTINUE
C     Yes - phase must be restricted to MKANG-1 and MKANG-1+180.
            MKANG(N) = MOD(KHL(4), 12)*15 + 1
            IF (MKANG(N) .LE. 1) MKANG(N) = MKANG(N) + 180
            GO TO 5000
 2180    CONTINUE
 5000 CONTINUE
      RETURN
      END
C     --------------------------------------------------------
C     Calculate S1=Ro**3.
      SUBROUTINE CALS1(NNN,IH,IK,IL,EV,PHIFP,WTFP,PHIGH,
     +     LBUF,BUF,YMAP,S1,S2,D,LPTSK,MESG,NSET,NTRYS,NCYC)
C     COMMON variables.
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     Data.
      CHARACTER*4 MESG
      INTEGER LBUF,LPTSK,NSET,NTRYS,NCYC
      REAL S1,S2,D
      INTEGER IH(NREF),IK(NREF),IL(NREF)
      REAL EV(NREF),PHIFP(NREF),WTFP(NREF),PHIGH(NREF)
C     FFT variables.
      REAL BUF(LBUF),YMAP(NX,NY,NZ)
C     Local.
      INTEGER IPAT
      REAL SCF
C
      IPAT = 0
      SCF = 1.0/VOLUME
C     Calculate S1.
      DO N = 1, NNN
         WTFP(N) = EV(N)
         PHIGH(N) = AMOD(2.0*PHIFP(N) + 720.0, 360.0)
      END DO
C     Calculate a map.(YMAP -- FFT of E with 2*phase)
      CALL FFTFT1(NNN,IH,IK,IL,WTFP,PHIGH,LBUF,BUF,NX,NY,NZ
     +     ,YMAP,NSYMP,RSYML,IPAT,6,SCF,0.0)
C     Minimum, maximum and average value of map.
      CALL AVEMAP(NX,NY,NZ,YMAP,ROMIN,ROMAX,AVE,SIGRO,RO3)
      S1 = RO3
      D = 0.5*(S1 + S2)/S2
      IF (LPTSK.NE.0) THEN
         WRITE(6,FMT='(/,A,f12.6,1X,A,3I5)') ' Value D = '
     +       ,D,MESG,NSET,NTRYS,NCYC
      END IF
      RETURN
      END
C     ------------------------------------------------------------
C     Calculate S2.
      SUBROUTINE CALS2(NNN,IH,IK,IL,EV,WTFP,PHIGH,
     +     LBUF,BUF,YMAP,S2)
C     COMMON variables.
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     Data.
      INTEGER NNN,LBUF
      INTEGER IH(NREF),IK(NREF),IL(NREF)
      REAL EV(NREF),WTFP(NREF),PHIGH(NREF)
C     FFT variables.
      REAL BUF(LBUF),YMAP(NX,NY,NZ)
C     Local.
      INTEGER IPAT
      REAL SCF
C
      IPAT = 1
      SCF = 1.0/VOLUME
C     Calculate S2.
      DO N = 1, NNN
         WTFP(N) = EV(N)
         PHIGH(N) = 0.0
      END DO
C     Calculate a map.(YMAP -- FFT of E with 2*phase)
      CALL FFTFT1(NNN,IH,IK,IL,WTFP,PHIGH,LBUF,BUF,NX,NY,NZ
     +     ,YMAP,NSYMP,RSYML,IPAT,6,SCF,0.0)
C     Minimum, maximum and average value of map.
      CALL AVEMAP(NX,NY,NZ,YMAP,ROMIN,ROMAX,AVE,SIGRO,RO3)
      S2 = RO3
      RETURN
      END
C     --------------------------------------------------------
C     Dynamic Density Modification Method.
      SUBROUTINE DDMM(IH,IK,IL,SS4V,EV,PHIFP,WT,WTFP,FC,PHIFC,ICODE
     +     ,MKANG,PHIFT,EX,PHIGH,FOM,SGMAA,LBUF,BUF,YMAP,LPTSK
     +     ,NSET,NCYC,CC,ERROR,S1,S2,D,NNEW,SNEW,NCT,NTRYS,NCENH
     +     ,KDDM,CCTRY,XMAP,ZMAP)
C
      INTEGER NAMAX
      PARAMETER (NAMAX=30000)
C     COMMON variables.
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     Data.
      INTEGER LBUF,LPTSK,NSET,NCYC,NNEW,NCT,NTRYS
      INTEGER IH(NEXT),IK(NEXT),IL(NEXT),MKANG(NEXT),ICODE(NEXT)
      REAL CC(3),S1,S2,D,SNEW,CCTRY
      REAL SS4V(NEXT),EV(NEXT),PHIFP(NEXT),PHIFT(NEXT),EX(NEXT)
     +     ,PHIGH(NEXT),WT(NEXT),WTFP(NEXT),FC(NEXT)
     +     ,PHIFC(NEXT),FOM(NEXT),SGMAA(NEXT)
C     FFT variables.
      REAL BUF(LBUF),YMAP(NX,NY,NZ),XMAP(NX,NY,NZ),ZMAP(NX,NY,NZ)
C     Local.
      CHARACTER*4 MDDM
      INTEGER IPAT
      REAL SCF,SCR,F0C
C
      MDDM = 'DDM0'
      PARTR = PARTRS
      IF (NCENH.EQ.1) THEN
         MDDM = 'ENHS'
         PARTR = 1.0
      ELSE IF(KDDM.EQ.2) THEN
         MDDM = 'DDM2'
      ELSE IF (KDDM.EQ.1) THEN
         MDDM = 'DDM1'
      END IF
C
      IPAT = 0
      SCR = VOLUME
      SCF = 1.0/VOLUME
C     One cycle DDM.
      DO N = 1, NEXT
         WTFP(N) = 0.0
         IF (ICODE(N).EQ.1.OR.ICODE(N).EQ.4) THEN
            WTFP(N) = WT(N)*EX(N)
            IF (NCENH.EQ.1.OR.KDDM.EQ.1)
     +          WTFP(N) = WT(N)*(2.0*EX(N) - FC(N))
            IF (KDDM.EQ.2)
     +          WTFP(N) = 2.0*FOM(N)*EX(N) - SGMAA(N)*FC(N)
         END IF
      END DO
C     Calculate a map.
      CALL FFTFT1(NEXT,IH,IK,IL,WTFP,PHIFP,LBUF,BUF,NX,NY,NZ,YMAP
     +     ,NSYMP,RSYML,IPAT,6,SCF,0.0)
C     Minimum, maximum and average value of map.
      CALL AVEMAP(NX,NY,NZ,YMAP,ROMIN,ROMAX,AVE,SIGRO,RO3)
C     Locale average.
      IF (NCENH.EQ.1) THEN
         CALL LOCVAR(NX,NY,NZ,CELLP,YMAP,XMAP,ZMAP)
      END IF
C     Modify map by DDM method.
      IF (MAXSET.EQ.1.OR.LPTSK.EQ.1) THEN
         CALL MODMAP(NX,NY,NZ,YMAP,ZMAP,ROMAX,CUTT,MDDM,NSET
     +       ,NCYC,1,SIGRO,NCT,NTRYS,NCENH)
      ELSE
         CALL MODMAP(NX,NY,NZ,YMAP,ZMAP,ROMAX,CUTT,MDDM,NSET
     +       ,NCYC,0,SIGRO,NCT,NTRYS,NCENH)
      END IF
C     Calculate Structure factors from a map.
      CALL FFTSF1(NEXT,IH,IK,IL,FC,PHIFC,LBUF,BUF,NX,NY,NZ,YMAP
     +     ,6,SCR,F0C)
C     Scale FC.
      CALL SCNEW(NEXT,EX,FC,SS4V,SS4HO,20,ICODE)
C     Sigmaa weights and Sim weights.
      CALL WTSGA(NEXT,PARTR,SS4V,EX,FC,FOM,SGMAA,WT
     +     ,MKANG,SS4HO,ICODE)
C     Calculate weights and obtain new phases.
      CALL PHINEW(NEXT,PHIFP,FC,PHIFC,
     +     NNEW,SNEW,MKANG,ICODE)
C     Free R-Factor as figure of merit without print out.
      CALL RFNEW(NEXT,EX,FC,NSET,NTRYS,NCYC,CC,0,MDDM,ICODE)
      MKDDM = 0
      IF (NCT.GE.5.AND.NTRYS.LT.NTRY.AND.CC(2).LT.CCTRY) MKDDM = 1
C     Phase error for each cycle.
      IF (MAXSET.EQ.1) THEN
         MKDDM = 1
C     Write out phase shift.
         WRITE(6,FMT='(/,A,f10.4,I8,1X,A,3I5)') ' The phase shift ='
     +       ,SNEW,NNEW,MDDM,NSET,NTRYS,NCYC
C     Free R-Factor as figure of merit.
         CALL RFNEW(NEXT,EX,FC,NSET,NTRYS,NCYC,CC,1,MDDM,ICODE)
         IF (SNEW.LT.PFIN.OR.LPTSK.EQ.1.OR.MKDDM.EQ.1) THEN
C     Calculate S1 and D.
            CALL CALS1(NEMAX,IH,IK,IL,EV,PHIFP,WTFP,PHIGH,
     +          LBUF,BUF,YMAP,S1,S2,D,1,MDDM,NSET,NTRYS,NCYC)
C     Phase error.
            IF (KDDM.EQ.2.AND.MKER.EQ.1)
     +          CALL ERRNEW(NEXT,IH,IK,IL,NOX,NOY,NOZ,PHIFT,PHIFP
     +          ,FOM,10,0,NSYMP,NSET,NTRYS,NCYC,MDDM,ICODE,ERROR)
            IF (KDDM.LE.1.AND.MKER.EQ.1)
     +          CALL ERRNEW(NEXT,IH,IK,IL,NOX,NOY,NOZ,PHIFT,PHIFP
     +          ,WT,10,0,NSYMP,NSET,NTRYS,NCYC,MDDM,ICODE,ERROR)
         END IF
      ELSE IF (SNEW.LT.PFIN.OR.LPTSK.EQ.1.OR.MKDDM.EQ.1) THEN
C     Write out phase shift.
         WRITE(6,FMT='(/,A,f10.4,I8,1X,A,3I5)') ' The phase shift ='
     +       ,SNEW,NNEW,MDDM,NSET,NTRYS,NCYC
C     Free R-Factor as figure of merit.
         CALL RFNEW(NEXT,EX,FC,NSET,NTRYS,NCYC,CC,1,MDDM,ICODE)
C     Calculate S1 and D.
         CALL CALS1(NEMAX,IH,IK,IL,EV,PHIFP,WTFP,PHIGH,
     +       LBUF,BUF,YMAP,S1,S2,D,1,MDDM,NSET,NTRYS,NCYC)
C     Phase error
            IF (KDDM.EQ.2.AND.MKER.EQ.1)
     +          CALL ERRNEW(NEXT,IH,IK,IL,NOX,NOY,NOZ,PHIFT,PHIFP
     +          ,FOM,10,0,NSYMP,NSET,NTRYS,NCYC,MDDM,ICODE,ERROR)
            IF (KDDM.LE.1.AND.MKER.EQ.1)
     +          CALL ERRNEW(NEXT,IH,IK,IL,NOX,NOY,NOZ,PHIFT,PHIFP
     +          ,WT,10,0,NSYMP,NSET,NTRYS,NCYC,MDDM,ICODE,ERROR)
      END IF
      RETURN
      END
C     --------------------------------------------------------
C     Real Space Sayre Equation Refinement by FFT.
      SUBROUTINE RSSER(IH,IK,IL,SS4V,EV,PHIFP,WTFP,FC,PHIFC,MKANG
     +     ,PHIFT,WT,THETAE,GH,PHIGH,X1,PHIX1,X2,PHIX2,X3,PHIX3
     +     ,LBUF,BUF,XMAP,YMAP,NCYC,ERROR,SUMES,NSET,NTRYS
     +     ,LPTSK,S1,S2,D,CC,ICODE,EX)
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     Data.
      INTEGER LBUF,NCYC,NSET,LPTSK
      INTEGER IH(NEXT),IK(NEXT),IL(NEXT),MKANG(NEXT),ICODE(NEXT)
      REAL SUMES,S1,S2,D,CC(3)
      REAL SS4V(NEXT),EV(NEXT),PHIFP(NEXT)
     +     ,WTFP(NEXT),FC(NEXT),PHIFC(NEXT),PHIFT(NEXT)
     +     ,WT(NEXT),THETAE(NEXT),EX(NEXT),GH(NEXT),PHIGH(NEXT)
     +     ,X1(NEXT),PHIX1(NEXT),X2(NEXT),PHIX2(NEXT),X3(NEXT)
     +     ,PHIX3(NEXT)
C     FFT variables.
      REAL BUF(LBUF),XMAP(NX,NY,NZ),YMAP(NX,NY,NZ)
C     Local variables.
      CHARACTER*4 MSER
      INTEGER IPAT,NNN
      REAL SCR,SCF,SUMX1,SUMX2,SUMX3,SUMEV,SUMEC
C
      DATA MSER/'RSER'/
C
      NNN = 0
      IPAT = 0
      SCR = VOLUME
      SCF = 1.0/VOLUME
C
      SUMX1 = 0.0
      SUMX2 = 0.0
      SUMX3 = 0.0
      SUMEV = 0.0
      SUMEC = 0.0
      SUMES = 0.0
      DO N = 1, NEXT
         WTFP(N) = 0.0
         IF (ICODE(N).EQ.1.OR.ICODE(N).EQ.3) WTFP(N) = WT(N)*EX(N)
         IF (ICODE(N).EQ.4) WTFP(N) = WT(N)*EX(N)
         IF (ICODE(N).EQ.1.OR.ICODE(N).EQ.4) NNN = NNN + 1
      END DO
C     Calculate a map.(YMAP -- FFT of E)
      CALL FFTFT1(NEXT,IH,IK,IL,WTFP,PHIFP,LBUF,BUF,NX,NY,NZ,YMAP
     +     ,NSYMP,RSYML,IPAT,6,SCF,0.0)
C     Save RO map in XMAP.
      CALL SAVMAP(NX,NY,NZ,XMAP,YMAP)
C     Minimum, maximum and average value of map.
      CALL AVEMAP(NX,NY,NZ,YMAP,ROMIN,ROMAX,AVE,SIGRO,RO3)
C     Square map.(YMAP**2)
      CALL SQMAP(NX,NY,NZ,YMAP)
C     Calculate Structure factors from a map. (GH)
      CALL FFTSF1(NEXT,IH,IK,IL,GH,PHIGH,LBUF,BUF,NX,NY,NZ,YMAP
     +     ,6,SCR,F0C)
C     Calculate X1 and PHIX1.
      DO N = 1, NEXT
         X1(N) = VOLUME*THETAE(N)*GH(N)
         PHIX1(N) = PHIGH(N)
         IF (ICODE(N).EQ.1.OR.ICODE(N).EQ.4) SUMX1 = SUMX1 + X1(N)
      END DO
C     Calculate a map.(YMAP -- FFT of C*E)
      DO N = 1, NEXT
         WTFP(N) = 0.0
         IF (ICODE(N).EQ.1.OR.ICODE(N).EQ.3)
     +       WTFP(N) = THETAE(N)*WT(N)*EX(N)
         IF (ICODE(N).EQ.4) WTFP(N) = THETAE(N)*WT(N)*EX(N)
      END DO
      CALL FFTFT1(NEXT,IH,IK,IL,WTFP,PHIFP,LBUF,BUF,NX,NY,NZ,YMAP
     +     ,NSYMP,RSYML,IPAT,6,SCF,0.0)
C     Minimum, maximum and average value of map.
      CALL AVEMAP(NX,NY,NZ,YMAP,ROMIN,ROMAX,AVE,SIGRO,RO3)
C     Multiply map.(XMAP*YMAP)
      CALL MULMAP(NX,NY,NZ,XMAP,YMAP)
C     Calculate Structure factors from a map.
      CALL FFTSF1(NEXT,IH,IK,IL,FC,PHIFC,LBUF,BUF,NX,NY,NZ,YMAP
     +     ,6,SCR,F0C)
C     Calculate X2 and PHIX2.
      DO N = 1, NEXT
         X2(N) = VOLUME*FC(N)
         PHIX2(N) = PHIFC(N)
         IF (ICODE(N).EQ.1.OR.ICODE(N).EQ.4) SUMX2 = SUMX2 + X2(N)
      END DO
C     Calculate a map.(YMAP -- FFT of C*C*GH)
      DO N = 1, NEXT
         WTFP(N) = 0.0
         IF (ICODE(N).EQ.1.OR.ICODE(N).EQ.3)
     +       WTFP(N) = THETAE(N)*THETAE(N)*WT(N)*GH(N)
         IF (ICODE(N).GE.4) WTFP(N) = THETAE(N)*THETAE(N)*WT(N)*GH(N)
      END DO
      CALL FFTFT1(NEXT,IH,IK,IL,WTFP,PHIGH,LBUF,BUF,NX,NY,NZ,YMAP
     +     ,NSYMP,RSYML,IPAT,6,SCF,0.0)
C     Minimum, maximum and average value of map.
      CALL AVEMAP(NX,NY,NZ,YMAP,ROMIN,ROMAX,AVE,SIGRO,RO3)
C     Multiply map.(XMAP*YMAP)
      CALL MULMAP(NX,NY,NZ,XMAP,YMAP)
C     Calculate Structure factors from a map.
      CALL FFTSF1(NEXT,IH,IK,IL,FC,PHIFC,LBUF,BUF,NX,NY,NZ,YMAP
     +     ,6,SCR,F0C)
C     Calculate X3 and PHIX3.
      DO N = 1, NEXT
         X3(N) = VOLUME*VOLUME*FC(N)
         PHIX3(N) = PHIFC(N)
         IF (ICODE(N).EQ.1.OR.ICODE(N).EQ.4) SUMX3 = SUMX3 + X3(N)
      END DO
C     Scale for X2 & X3.
      SCX3 = 1.0
      TEST = SUMX1/SUMX3
      IF (TEST.LT.1.5) SCX3 = TEST/1.5
      SCX2 = 1.0
      TEST = SUMX1/SUMX2
      IF (TEST.LT.5.0) SCX2 = TEST/5.0
C     Final phase.
      DO N = 1, NEXT
            SCVG = X1(N)*SIN(DTOR*PHIX1(N))
            CCVG = X1(N)*COS(DTOR*PHIX1(N))
            TOP =  SCVG
     +          + X2(N)*SIN(DTOR*PHIX2(N))*SCX2
     +          - X3(N)*SIN(DTOR*PHIX3(N))*SCX3
            BOT =  CCVG
     +          + X2(N)*COS(DTOR*PHIX2(N))*SCX2
     +          - X3(N)*COS(DTOR*PHIX3(N))*SCX3
            FC(N) = SQRT(TOP*TOP + BOT*BOT)
            PHIFC(N) = 0.0
            IF (FC(N).GT.0.0) PHIFC(N) = RTOD*ATAN2(TOP,BOT)
         IF (ICODE(N).EQ.1.OR.ICODE(N).EQ.4) THEN
            SUMEV = SUMEV + EX(N)
            SUMEC = SUMEC + FC(N)
            SUMES = SUMES + (EX(N)*COS(DTOR*PHIFC(N)) - CCVG)**2
     +          + (EX(N)*SIN(DTOR*PHIFC(N)) - SCVG)**2
         END IF
      END DO
C
      SUMES = SUMES/REAL(NNN)
C     Scale FC.
      CALL SCNEW(NEXT,EX,FC,SS4V,SS4HO,20,ICODE)
C     Sim weights.
      CALL WTSIM(NEXT,EX,FC,WT,PARTR,ICODE)
C     Calculate weights and obtain new phases.
      CALL PHINEW(NEXT,PHIFP,FC,PHIFC,
     +     NNEW,SNEW,MKANG,ICODE)
C     Calculate value S1 and D.
      CALL CALS1(NEMAX,IH,IK,IL,EV,PHIFP,WTFP,PHIGH,
     +     LBUF,BUF,YMAP,S1,S2,D,LPTSK,MSER,NSET,NTRYS,NCYC)
C     End if D > 0.6.
      IF (D.GT.0.6.AND.LPTSK.EQ.0) THEN
         LPTSK = 1
         WRITE(6,FMT='(/,A,F12.6,1X,A,3I5)') ' Value D = '
     +       ,D,MSER,NSET,NTRYS,NCYC
      END IF
C     Write out phase shift.
      IF (LPTSK.NE.0) THEN
         WRITE(6,FMT='(/,A,F10.4,I8,1X,A,3I5)') ' The phase shift ='
     +       ,SNEW,NNEW,MSER,NSET,NTRYS,NCYC
C     Free R-Factor as figure of merit.
         CALL RFNEW(NEXT,EX,FC,NSET,NTRYS,NCYC,CC,1,MSER,ICODE)
C     Phase error for each cycle..
         IF (MKER.EQ.1) CALL ERRNEW(NEXT,IH,IK,IL,NOX,NOY,NOZ,
     +       PHIFT,PHIFP,WT,10,0,NSYMP,NSET,NTRYS,NCYC,MSER
     +       ,ICODE,ERROR)
      END IF
      RETURN
      END
C     --------------------------------------------------------
C     Obtain various initial phases.
      SUBROUTINE INIPHI(IH,IK,IL,MKANG,SS4V,PHIFP,WT,EX,FOM,SGMAA
     +     ,FC,PHIFC,PHIFT,NSET,NTRYS,NCYC,CC,ERROR,ICODE,
     +     NX,NY,NZ,YMAP,LBUF,BUF,FT)
C
      INTEGER NAMAX
      PARAMETER (NAMAX=30000)
C     COMMON variables.
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     Eulerian angles.
      COMMON/EULER/EU1M,EU2M,EU3M,
     +     EU1(5000),EU2(5000),EU3(5000),EUCC(5000),NSEU(5000)
     +     ,TR1(5000),TR2(5000),TR3(5000),TRCS(5000),NSTR(5000)
C     Translation function.
      COMMON/XYZIN/NUMA,OC(NAMAX),IZ(NAMAX),XINF(NAMAX),YINF(NAMAX)
     +     ,ZINF(NAMAX),XINO(NAMAX),YINO(NAMAX),ZINO(NAMAX),BFAC(NAMAX)
C     External Routines.
      EXTERNAL CCPTIM
C     Data.
      INTEGER NSET,NCYC
      INTEGER IH(NEXT),IK(NEXT),IL(NEXT),MKANG(NEXT),ICODE(NEXT)
      REAL PHIFP(NEXT),SS4V(NEXT),PHIFT(NEXT)
     +     ,WT(NEXT),EX(NEXT),FOM(NEXT),SGMAA(NEXT),FC(NEXT)
     +     ,PHIFC(NEXT),FT(NEXT)
      REAL CC(3),ERROR,CCSL(100)
C     Local.
      CHARACTER*4 MESG,MPOS,MATM,MROT,MRRT,MTRA,MRTF,MSOL
      INTEGER NALL,NSS4,IZT(NAMAX)
      REAL XR(NAMAX),YR(NAMAX),ZR(NAMAX),XN(NAMAX),YN(NAMAX),ZN(NAMAX)
     +     ,OCT(NAMAX),BF(NAMAX)
      REAL YMAP(NX,NY,NZ),BUF(LBUF)
C
      DATA MPOS/'POSI'/,MATM/'RATM'/,
     +     MROT/'ROTF'/,MRRT/'RROT'/,MTRA/'TRAN'/,MRTF/'RTRA'/,
     +     MSOL/'NSOL'/
C
      NSS4 = 20
C     ACORN-MR Method to obtain a set of initial phases.
      IF (IMSG.GE.3) THEN
C     Initial values.
         IF (IMSG.EQ.3) MESG = MRRT
         IF (IMSG.EQ.4) MESG = MROT
         IF (IMSG.EQ.5) MESG = MRTF
         IF (IMSG.EQ.6) MESG = MTRA
         IF (NSOL.GT.0) MESG = MSOL
         EU1S = EU1(NSET)
         EU2S = EU2(NSET)
         EU3S = EU3(NSET)
         TXS = TR1(NSET)
         TYS = TR2(NSET)
         TZS = TR3(NSET)
C     Pick up atoms.
         NALL = 0
         NEND = NSTA + NFRA - 1
         IF (NEND.GT.NUMA) NEND = NUMA
         DO NA = NSTA, NEND
            NALL = NALL + 1
            OCT(NALL) = OC(NA)
            IZT(NALL) = IZ(NA)
            XN(NALL) = XINO(NA)
            YN(NALL) = YINO(NA)
            ZN(NALL) = ZINO(NA)
            BF(NALL) = BFAC(NA)
         END DO
C     New XYZ after rotated.
         CALL ROTXYZ(NALL,XN,YN,ZN,XR,YR,ZR,EU1S,EU2S,EU3S
     +       ,TXS,TYS,TZS)
C     Calculate SF.
         IF (NALL.LT.100) THEN
            CALL CALFP(NALL,OCT,IZT,BF,XR,YR,ZR,NEXT,MKANG,IH,IK,IL
     +          ,SS4V,FC,PHIFC)
         ELSE
            CALL CALEC(NALL,OCT,IZT,BF,XR,YR,ZR,NEXT,MKANG,IH,IK,IL
     +          ,SS4V,FC,PHIFC,YMAP,LBUF,BUF)
         END IF
C     Scale FC.
         CALL SCALE1(NEXT,MKANG,FC,SS4V,SS4HO,20)
C     Sigmaa weights and Sim weights.
         CALL WTSGA(NEXT,PARTR,SS4V,EX,FC,FOM,SGMAA,WT
     +       ,MKANG,SS4HO,ICODE)
         CALL PHINEW(NEXT,PHIFP,FC,PHIFC,
     +       NNEW,SNEW,MKANG,ICODE)
C     Free R-Factor as figure of merit with print out.
         CALL RFNEW(NEXT,EX,FC,NSET,NTRYS,NCYC,CC,1,MESG,ICODE)
C     Phase error
         IF (MKER.EQ.1) CALL ERRNEW(NEXT,IH,IK,IL,NOX,NOY,NOZ,
     +       PHIFT,PHIFP,WT,10,2,NSYMP,NSET,NTRYS,NCYC,MESG
     +       ,ICODE,ERROR)
C     R-factor for xloggraph.
         IF (MAXSET.EQ.1) THEN
            CALL RFACTOR(NEXT,NREF,EX,FC,SS4V,SS4HO,NSS4,NSET,NCYC
     +          ,NCSL,CCSL)
C     Phase errors for xloggraph.
            IF (MKER.EQ.1) CALL PHIERR(NEXT,NREF,EX,PHIFT,PHIFP,WT
     +          ,SS4V,SS4HO,NSS4,NSET,NCYC)
         END IF
      ELSE
C     Pick atoms according to CC for a right frangment (NPOS>0).
         IF (IMSG.EQ.2) THEN
            MESG = MPOS
            NALL = 0
            NS = INT(EU1(NSET) + 0.5)
            NT = NS + NFRA - 1
            IF (NT.GT.NUMA) NT = NUMA
            WRITE(6,FMT='(/,20X,A,A)')
     +          'No.        XF        YF        ZF    OC  BFAC',
     +          '  ZJ   MESG NSET NCYC'
            WRITE(6,FMT='(A,I8,3F10.5,2F6.2,I4,3X,A,2I5)')
     +          ' The first atom'
     +          ,NS,XINF(NS),YINF(NS),ZINF(NS),OC(NS),BFAC(NS)
     +          ,IZ(NS),MESG,NSET,NCYC
            IF (NFRA.GT.1)
     +          WRITE(6,FMT='(A,I8,3F10.5,2F6.2,I4,3X,A,2I5)')
     +          ' The last  atom'
     +          ,NT,XINF(NT),YINF(NT),ZINF(NT),OC(NT),BFAC(NT)
     +          ,IZ(NT),MESG,NSET,NCYC
            DO NA = NS, NT
               NALL = NALL + 1
               OCT(NALL) = OC(NA)
               IZT(NALL) = IZ(NA)
               XR(NALL) = XINF(NA)
               YR(NALL) = YINF(NA)
               ZR(NALL) = ZINF(NA)
               BF(NALL) = BFAC(NA)
            END DO
C     Calculate SF.
            IF (NALL.LT.100) THEN
               CALL CALFP(NALL,OCT,IZT,BF,XR,YR,ZR,NEXT,MKANG,IH,IK,IL
     +             ,SS4V,FC,PHIFC)
            ELSE
               CALL CALEC(NALL,OCT,IZT,BF,XR,YR,ZR,NEXT,MKANG,IH,IK,IL
     +             ,SS4V,FC,PHIFC,YMAP,LBUF,BUF)
            END IF
         END IF
C     Pick atoms according to CC for random atoms (NATM>0).
         IF (IMSG.EQ.1) THEN
            MESG = MATM
            NALL = 1
            IF (NSYMP.EQ.1) NALL = 2
            OCT(2) = OC(NSET)
            IZT(2) = IZ(NSET)
            XR(2) = 0.0
            YR(2) = 0.0
            ZR(2) = 0.0
            BF(2) = 0.0
            WRITE(6,FMT='(/,11X,A,A)')
     +          '  XF        YF        ZF    OC  BFAC  ZJ',
     +          '   MESG NSET NCYC'
            OCT(1) = OC(NSET)
            IZT(1) = IZ(NSET)
            XR(1) = XINF(NSET)
            YR(1) = YINF(NSET)
            ZR(1) = ZINF(NSET)
            BF(1) = BFAC(NSET)
            WRITE(6,FMT='(A,3F10.5,2F6.2,I4,3X,A,2I5)')
     +          ' Atom',XR(1),YR(1)
     +          ,ZR(1),OCT(1),BF(1),IZT(1),MESG,NSET,NCYC
C     Calculate SF.
            CALL CALFP(NALL,OCT,IZT,BF,XR,YR,ZR,NEXT,MKANG,IH,IK,IL
     +          ,SS4V,FC,PHIFC)
         END IF
C     Scale FC.
         CALL SCALE1(NEXT,MKANG,FC,SS4V,SS4HO,20)
C     Sigmaa weights and Sim weights.
         CALL WTSGA(NEXT,PARTR,SS4V,EX,FC,FOM,SGMAA,WT
     +       ,MKANG,SS4HO,ICODE)
C     Obtain new phases.
         CALL PHINEW(NEXT,PHIFP,FC,PHIFC,
     +          NNEW,SNEW,MKANG,ICODE)
C     Free R-Factor as figure of merit with print out.
         CALL RFNEW(NEXT,EX,FC,NSET,NTRYS,NCYC,CC,1,MESG,ICODE)
C     Phase error
         IF (MKER.EQ.1) CALL ERRNEW(NEXT,IH,IK,IL,NOX,NOY,NOZ,
     +       PHIFT,PHIFP,WT,10,2,NSYMP,NSET,NTRYS,NCYC,MESG
     +       ,ICODE,ERROR)
C     R-factor for xloggraph.
         IF (MAXSET.EQ.1) THEN
            CALL RFACTOR(NEXT,NREF,EX,FC,SS4V,SS4HO,NSS4,NSET,NCYC
     +       ,NCSL,CCSL)
C     Phase errors for xloggraph.
            IF (MKER.EQ.1) CALL PHIERR(NEXT,NREF,EX,PHIFT,PHIFP,WT
     +          ,SS4V,SS4HO,NSS4,NSET,NCYC)
         END IF
      END IF
      RETURN
      END
C     ----------------------------------------------------------
C     Obtain new XYZ after rotate.
      SUBROUTINE ROTXYZ(NALL,XN,YN,ZN,XR,YR,ZR,EU1S,EU2S,EU3S
     +     ,TXS,TYS,TZS)
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
C     Data.
      INTEGER NALL
      REAL EU1S,EU2S,EU3S,TXS,TYS,TZS
      REAL XN(NALL),YN(NALL),ZN(NALL),XR(NALL),YR(NALL),ZR(NALL)
C     Local.
      REAL ROTX(3,3),ROTC(3,3)
C     Matrix for rotation function.
      CALL MATXCP4(EU1S,EU2S,EU3S,DTOR,ROTX)
C     Matrix to real unit cell.
      CALL MATB(ROTC,ROTX)
C     New X Y Z.
      DO NA = 1, NALL
         CALL ROTXH(XN(NA),YN(NA),ZN(NA)
     +       ,XR(NA),YR(NA),ZR(NA),ROTC)
         XR(NA) = XR(NA) + TXS
         YR(NA) = YR(NA) + TYS
         ZR(NA) = ZR(NA) + TZS
      END DO
      RETURN
      END
C     ---------------------------------------------------------
C     Calculate |FC| directly from XYZ.
      SUBROUTINE CALFC(NALL,OCT,IZT,XR,YR,ZR,NNN,MKANG,IH,IK,IL,FC)
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
C
      INTEGER NALL,NNN
      INTEGER IH(NNN),IK(NNN),IL(NNN),MKANG(NNN),IZT(NALL)
      REAL OCT(NALL),XR(NALL),YR(NALL),ZR(NALL),FC(NNN)
C     Local.
      REAL X,Y,Z
C
      DO NF = 1, NNN
         TOP = 0.0
         BOT = 0.0
         IF (MKANG(NF).GT.0) THEN
C     Cal SF.
            DO N = 1, NALL
               DO NS = 1, NSYMP
                  X = RSYML(1,1,NS)*XR(N) + RSYML(1,2,NS)*YR(N) +
     +                RSYML(1,3,NS)*ZR(N) + RSYML(1,4,NS)
                  Y = RSYML(2,1,NS)*XR(N) + RSYML(2,2,NS)*YR(N) +
     +                RSYML(2,3,NS)*ZR(N) + RSYML(2,4,NS)
                  Z = RSYML(3,1,NS)*XR(N) + RSYML(3,2,NS)*YR(N) +
     +                RSYML(3,3,NS)*ZR(N) + RSYML(3,4,NS)
                  TEM = REAL(IH(NF))*X + REAL(IK(NF))*Y
     +                + REAL(IL(NF))*Z
                  TOP = TOP + OCT(N)*REAL(IZT(N))*SIN(TWOPI*TEM)
                  BOT = BOT + OCT(N)*REAL(IZT(N))*COS(TWOPI*TEM)
               END DO
            END DO
            FC(NF) = SQRT(TOP*TOP + BOT*BOT)
         END IF
      END DO
      RETURN
      END
C     ---------------------------------------------------------
C     Calculate SF directly from XYZ.
      SUBROUTINE CALFP(NALL,OCT,IZT,BF,XR,YR,ZR,NNN,MKANG,IH,IK,IL
     +     ,SS4V,FC,PHIFC)
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
C
      INTEGER NALL,NNN
      INTEGER IH(NNN),IK(NNN),IL(NNN),MKANG(NNN),IZT(NALL)
      REAL OCT(NALL),BF(NALL),XR(NALL),YR(NALL),ZR(NALL)
     +     ,SS4V(NNN),FC(NNN),PHIFC(NNN)
C     Local.
      REAL X,Y,Z
C
      DO NF = 1, NNN
         TOP = 0.0
         BOT = 0.0
         IF (MKANG(NF).GT.0) THEN
            SS4T = 0.25*SS4V(NF)
C     Cal SF.
            DO N = 1, NALL
               DO NS = 1, NSYMP
                  X = RSYML(1,1,NS)*XR(N) + RSYML(1,2,NS)*YR(N) +
     +                RSYML(1,3,NS)*ZR(N) + RSYML(1,4,NS)
                  Y = RSYML(2,1,NS)*XR(N) + RSYML(2,2,NS)*YR(N) +
     +                RSYML(2,3,NS)*ZR(N) + RSYML(2,4,NS)
                  Z = RSYML(3,1,NS)*XR(N) + RSYML(3,2,NS)*YR(N) +
     +                RSYML(3,3,NS)*ZR(N) + RSYML(3,4,NS)
                  TEM = REAL(IH(NF))*X + REAL(IK(NF))*Y
     +                + REAL(IL(NF))*Z
                  TOP = TOP + OCT(N)*REAL(IZT(N))*EXP(-BF(N)*SS4T)
     +                *SIN(TWOPI*TEM)
                  BOT = BOT + OCT(N)*REAL(IZT(N))*EXP(-BF(N)*SS4T)
     +                *COS(TWOPI*TEM)
               END DO
            END DO
            FC(NF) = SQRT(TOP*TOP + BOT*BOT)
            PHIFC(N) = 0.0
            IF (FC(NF).GT.0.0)
     +          PHIFC(NF) = ATAN2(TOP,BOT)*RTOD
         END IF
      END DO
      RETURN
      END
C     ---------------------------------------------------------
C     Calculate SF directly from XYZ by FFT.
      SUBROUTINE CALEC(NALL,OCT,IZT,BF,XR,YR,ZR,NNN,MKANG,IH,IK,IL
     +     ,SS4V,FC,PHIFC,YMAP,LBUF,BUF)
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/ATMFACTOR/NKD,AL(8),AS(8),BL(8),BS(8),CL(8),CS(8)
     +,DL(8),DS(8),EL(8),NW(8),NO(8),NE(8)
C
      INTEGER NALL,NNN
      INTEGER IH(NNN),IK(NNN),IL(NNN),MKANG(NNN),IZT(NALL)
      REAL OCT(NALL),BF(NALL),XR(NALL),YR(NALL),ZR(NALL)
     +     ,SS4V(NNN),FC(NNN),PHIFC(NNN)
      REAL YMAP(NX,NY,NZ),BUF(LBUF)
C     Local.
      REAL X,Y,Z
C
      IPAT = 0
      SCR = VOLUME
      SCF = 1.0/VOLUME
      A = CELLP(1)
      B = CELLP(2)
      C = CELLP(3)
      CSA = COS(DTOR*CELLP(4))
      CSB = COS(DTOR*CELLP(5))
      CSC = COS(DTOR*CELLP(6))
      MIJK = 8
C
      DO I = 1, NX
         DO J = 1, NY
            DO K = 1, NZ
               YMAP(I,J,K) = 0.0
            END DO
         END DO
      END DO
C     Make a map.
      DO N = 1, NALL
         DO NK = 1, NKD
            IF (IZT(N).EQ.NE(NK)) THEN
               ALT = AL(NK)
               AST = AS(NK)
               BLT = BL(NK)
               BST = BS(NK)
               CLT = CL(NK)
               CST = CS(NK)
               DLT = DL(NK)
               DST = DS(NK)
               ELT = EL(NK)
               EST = 0.0
               T11 = OCT(N)*ALT*(4.0*PI/(AST+BF(N)))**1.5
               T21 = OCT(N)*BLT*(4.0*PI/(BST+BF(N)))**1.5
               T31 = OCT(N)*CLT*(4.0*PI/(CST+BF(N)))**1.5
               T41 = OCT(N)*DLT*(4.0*PI/(DST+BF(N)))**1.5
               T51 = OCT(N)*ELT*(4.0*PI/(EST+BF(N)))**1.5
               GOTO 10
            END IF
         END DO
         CALL CCPERR(1,
     +       'No such atom type assigned in CONTENT, Bye-bye!')
  10      DO NS = 1, NSYMP
            X = RSYML(1,1,NS)*XR(N) + RSYML(1,2,NS)*YR(N) +
     +          RSYML(1,3,NS)*ZR(N) + RSYML(1,4,NS)
            Y = RSYML(2,1,NS)*XR(N) + RSYML(2,2,NS)*YR(N) +
     +          RSYML(2,3,NS)*ZR(N) + RSYML(2,4,NS)
            Z = RSYML(3,1,NS)*XR(N) + RSYML(3,2,NS)*YR(N) +
     +          RSYML(3,3,NS)*ZR(N) + RSYML(3,4,NS)
            X = AMOD(X+10.0,1.0)
            Y = AMOD(Y+10.0,1.0)
            Z = AMOD(Z+10.0,1.0)
            MX = NINT(X*REAL(NX))
            MY = NINT(Y*REAL(NY))
            MZ = NINT(Z*REAL(NZ))
            DO I = -MIJK, MIJK
               MXI = MX+I
               TX = REAL(MXI)/REAL(NX)
               DO J = -MIJK, MIJK
                  MYJ = MY+J
                  TY = REAL(MYJ)/REAL(NY)
                  DO K = -MIJK, MIJK
                     MZK = MZ+K
                     TZ = REAL(MZK)/REAL(NZ)
                     CALL DISTAN(DIS,X,Y,Z,TX,TY,TZ,A,B,C
     +                   ,CSA,CSB,CSC)
                     T12 = (4.0*PI*PI*DIS*DIS)/(AST+BF(N))
                     T22 = (4.0*PI*PI*DIS*DIS)/(BST+BF(N))
                     T32 = (4.0*PI*PI*DIS*DIS)/(CST+BF(N))
                     T42 = (4.0*PI*PI*DIS*DIS)/(DST+BF(N))
                     T52 = (4.0*PI*PI*DIS*DIS)/(EST+BF(N))
                     ROT = T11*EXP(-T12) + T21*EXP(-T22) +
     +                    T31*EXP(-T32) + T41*EXP(-T42) +
     +                    T51*EXP(-T52)
                     NXI = MOD(MXI+1+NX,NX)
                     IF (NXI.EQ.0) NXI = NX
                     NYJ = MOD(MYJ+1+NY,NY)
                     IF (NYJ.EQ.0) NYJ = NY
                     NZK = MOD(MZK+1+NZ,NZ)
                     IF (NZK.EQ.0) NZK = NZ
                     YMAP(NXI,NYJ,NZK) = YMAP(NXI,NYJ,NZK) +
     +                   ROT
                  END DO
               END DO
            END DO
         END DO
      END DO
C     Calculate Structure factors from a map.
      CALL FFTSF1(NNN,IH,IK,IL,FC,PHIFC,LBUF,BUF,NX,NY,NZ,YMAP
     +     ,6,SCR,F0C)
      RETURN
      END
C     --------------------------------------------------------
C     Patterson superposition use of Sum function.
      SUBROUTINE SUPSUM(IH,IK,IL,SS4V,EV,EX,PHIFP,WT,WTFP,FC
     +     ,PHIFC,ICODE,MKANG,PHIFT,PHIGH,LBUF,BUF,YMAP,XMAP
     +     ,NSET,NTRYS,NCYC,CC,S1,S2,D,NCENH,ERROR)
C
C     COMMON variables.
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     Data.
      INTEGER LBUF,NSET,NCYC
      INTEGER IH(NEXT),IK(NEXT),IL(NEXT),MKANG(NEXT),ICODE(NEXT)
      REAL S1,S2,D
      REAL EV(NEXT),EX(NEXT),PHIFP(NEXT),SS4V(NEXT),PHIFT(NEXT)
     +     ,PHIGH(NEXT),WT(NEXT),WTFP(NEXT),FC(NEXT),PHIFC(NEXT),CC(3)
C     FFT variables.
      REAL BUF(LBUF),YMAP(NX,NY,NZ),XMAP(NX,NY,NZ)
C     Local.
      CHARACTER*4 MSUP
      INTEGER IPAT,NSS4
      REAL SCF,SCR,F0C
C
      DATA MSUP/'SUPP'/
C
      NSS4 = 20
      IPAT = 0
      SCR = VOLUME
      SCF = 1.0/VOLUME
      DO N = 1, NREF
         WTFP(N) = 0.0
         IF (ICODE(N).LE.1) THEN
            WTFP(N) = EV(N)*EV(N)*FC(N)
         END IF
      END DO
C     Superposition map.
      CALL FFTFT1(NREF,IH,IK,IL,WTFP,PHIFP,LBUF,BUF,NX,NY,NZ,YMAP
     +     ,NSYMP,RSYML,IPAT,6,SCF,0.0)
C     Minimum, maximum and average value of map.
      CALL AVEMAP(NX,NY,NZ,YMAP,ROMIN,ROMAX,AVE,SIGRO,RO3)
C     Modify map by DDM method.
      CALL MODMAP(NX,NY,NZ,YMAP,XMAP,ROMAX,CUTT,MSUP,NSET
     +     ,NCYC,1,SIGRO,1,1,NCENH)
C     Calculate Structure factors from a map.
      CALL FFTSF1(NEXT,IH,IK,IL,FC,PHIFC,LBUF,BUF,NX,NY,NZ,YMAP
     +     ,6,SCR,F0C)
C     Scale FC.
      CALL SCNEW(NEXT,EX,FC,SS4V,SS4HO,NSS4,ICODE)
C     Sim weights.
      CALL WTSIM(NEXT,EX,FC,WT,PARTR,ICODE)
C     Calculate weights and obtain new phases.
      CALL PHINEW(NEXT,PHIFP,FC,PHIFC,
     +     NNEW,SNEW,MKANG,ICODE)
C     Calculate value S1 and D.
      CALL CALS1(NEMAX,IH,IK,IL,EV,PHIFC,WTFP,PHIGH,
     +     LBUF,BUF,YMAP,S1,S2,D,1,MSUP,NSET,NTRYS,NCYC)
C     Write out phase shift.
         WRITE(6,FMT='(/,A,f10.4,I8,1X,A,3I5)') ' The phase shift ='
     +       ,SNEW,NNEW,MSUP,NSET,NTRYS,NCYC
C     Free R-Factor as figure of merit.
      CALL RFNEW(NEXT,EX,FC,NSET,NTRYS,NCYC,CC,1,MSUP,ICODE)
C     Phase error for each cycle.
         IF (MKER.EQ.1) CALL ERRNEW(NEXT,IH,IK,IL,NOX,NOY,NOZ,
     +       PHIFT,PHIFP,WT,10,0,NSYMP,NSET,NTRYS,NCYC,MSUP
     +     ,ICODE,ERROR)
      RETURN
      END
C     --------------------------------------------------------
C     Calculate Structure factors for the large cell.
      SUBROUTINE CALSF(APAR,NFRA,SS4H,CELLB,INDMAX,NSTA)
C
      INTEGER NAMAX
      PARAMETER (NAMAX=30000)
C
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/XYZIN/NUMA,OC(NAMAX),IZ(NAMAX),XINF(NAMAX),YINF(NAMAX)
     +     ,ZINF(NAMAX),XINO(NAMAX),YINO(NAMAX),ZINO(NAMAX),BFAC(NAMAX)
C
      INTEGER INDMAX,NFRA,NSTA
      REAL SS4H,CELLB
      REAL APAR(-INDMAX:INDMAX,-INDMAX:INDMAX,0:INDMAX)
C     Local.
      INTEGER LH,LK,LL,NUMB,NEND
      REAL TOP,BOT
      NEND = NSTA + NFRA - 1
C     Fractional coordinates.
      DO N = NSTA, NEND
         XINO(N) = XINO(N)/CELLB
         YINO(N) = YINO(N)/CELLB
         ZINO(N) = ZINO(N)/CELLB
      END DO
C     Cal. SF.
      NUMB = 0
      DO LL = 0, INDMAX
         DO LK = -INDMAX, INDMAX
            DO LH = -INDMAX, INDMAX
               APAR(LH,LK,LL) = 0.0
               SS4 = REAL(LH*LH + LK*LK + LL*LL)/(CELLB*CELLB)
               IF (SS4.LE.SS4H) THEN
                  NUMB = NUMB + 1
                  TOP = 0.0
                  BOT = 0.0
                  DO N = NSTA, NEND
                     TEM = REAL(LH)*XINO(N) + REAL(LK)*YINO(N)
     +                   + REAL(LL)*ZINO(N)
                     TOP = TOP + OC(N)*REAL(IZ(N))*SIN(TWOPI*TEM)
                     BOT = BOT + OC(N)*REAL(IZ(N))*COS(TWOPI*TEM)
                  END DO
                  APAR(LH,LK,LL) = SQRT(BOT*BOT + TOP*TOP)
               END IF
            END DO
         END DO
      END DO
      WRITE(6,FMT='(/,A,I10)')
     +     ' The number of calculated structure factors = ',NUMB
C     Restore to PDB format.
      DO N = NSTA, NEND
         XINO(N) = XINO(N)*CELLB
         YINO(N) = YINO(N)*CELLB
         ZINO(N) = ZINO(N)*CELLB
      END DO
      RETURN
      END
C     --------------------------------------------------------
C     Find the dimensions of fragment, set to the centre of mass
C     and put it in a large unit cell.
      SUBROUTINE BIGCEL(CELLB,NFRA,SS4H,INDMAX,NSTA,NSOL)
      INTEGER NFRA,INDMAX,NSTA,NSOL
      REAL CELLB,SS4H
C
      INTEGER NAMAX
      PARAMETER (NAMAX=30000)
C
      COMMON/XYZIN/NUMA,OC(NAMAX),IZ(NAMAX),XINF(NAMAX),YINF(NAMAX)
     +     ,ZINF(NAMAX),XINO(NAMAX),YINO(NAMAX),ZINO(NAMAX),BFAC(NAMAX)
C     Local.
      INTEGER NEND
      REAL DX,DY,DZ,XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX,DMAX
C
      NEND = NSTA + NFRA - 1
      DMAX = 0.0
      XMIN = 1000.0
      XMAX = -1000.0
      YMIN = 1000.0
      YMAX = -1000.0
      ZMIN = 1000.0
      ZMAX = -1000.0
      DO N = NSTA, NEND
         IF (XINO(N).LT.XMIN) XMIN = XINO(N)
         IF (XINO(N).GT.XMAX) XMAX = XINO(N)
         IF (YINO(N).LT.YMIN) YMIN = YINO(N)
         IF (YINO(N).GT.YMAX) YMAX = YINO(N)
         IF (ZINO(N).LT.ZMIN) ZMIN = ZINO(N)
         IF (ZINO(N).GT.ZMAX) ZMAX = ZINO(N)
      END DO
      DX = XMAX - XMIN
      DY = YMAX - YMIN
      DZ = ZMAX - ZMIN
C     Maximum dimension of fragment.
      IF (DX.GT.DMAX) DMAX = DX
      IF (DY.GT.DMAX) DMAX = DY
      IF (DZ.GT.DMAX) DMAX = DZ
C     Centre of fragment.
      DX = 0.5*(XMAX + XMIN)
      DY = 0.5*(YMAX + YMIN)
      DZ = 0.5*(ZMAX + ZMIN)
C     A large unit cell.
      CELLB = 3.0*DMAX
      IF (CELLB.LT.20.0) CELLB = 20.0
C     Maximum index for the large unit cell.
      INDMAX = INT(SQRT(SS4H)*CELLB + 0.5) + 10
C     Information of the fragment and a large unit cell.
      WRITE(6,FMT='(/,A,3F8.2,/,A,6F8.2,/,A,I6/)')
     +     ' The centre of mass of the fragment = ',DX,DY,DZ,
     +     ' The large cell dimension = ',CELLB
     +     ,CELLB,CELLB,90.0,90.0,90.0,
     +     ' The maximum index for the large cell = ',INDMAX
C     Put the fragment at origin of cell.
      IF (NSOL.EQ.0) THEN
         DO N = NSTA, NEND
            XINO(N) = (XINO(N) - DX)
            YINO(N) = (YINO(N) - DY)
            ZINO(N) = (ZINO(N) - DZ)
         END DO
         WRITE(6,FMT='(/,A,A)') ' The centre of mass of the fragment',
     +       ' has been placed on the origin of the large cell.'
      ELSE
         WRITE(6,FMT='(/,A)') ' No shift for the fragment.'
      END IF
      RETURN
      END
C     --------------------------------------------------------
C     Random rotation function.
      SUBROUTINE RANROT(INDMAX,APAR,IH,IK,IL,MKANG,SS4V,EV
     +          ,CELLB,WH,WK,WL,S4VR,RFO,RFC,NURO)
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     Eulerian angles: EU1=ALPHA, EU2=BETA, EU3=GAMMA.
      COMMON/EULER/EU1M,EU2M,EU3M,
     +     EU1(5000),EU2(5000),EU3(5000),EUCC(5000),NSEU(5000)
     +     ,TR1(5000),TR2(5000),TR3(5000),TRCS(5000),NSTR(5000)
C
      INTEGER INDMAX,NURO,IH(NREF),IK(NREF),IL(NREF),MKANG(NREF)
      REAL CELLB
      REAL APAR(-INDMAX:INDMAX,-INDMAX:INDMAX,0:INDMAX),
     +     SS4V(NREF),EV(NREF),
     +     WH(NURO),WK(NURO),WL(NURO),S4VR(NURO),RFO(NURO),RFC(NURO)
C     Local.
      CHARACTER*4 MRRT
      INTEGER NSET,NSAV,NCC,NLPT,NRALL
      INTEGER IR3
      REAL ROTH(3,3),RH,RK,RL,TH,TK,TL,TFC,
     +     CCM,CCMAX
C
      DATA MRRT/'RROT'/
      IMSG = 3
      IR3 = ISEED
      NSAV = 0
      NCC = 4000
      CCMAX = -1.0
      NUMROT =0
C     Use 1/MODR reflections.
      IF (MODR.GT.1) THEN
         DO NF = 1, NREF
            IF (MOD(NF,MODR).NE.0) MKANG(NF) = -IABS(MKANG(NF))
         END DO
      END IF
C     Transfer h,k,l to H,K,L in large cell.
      CALL GETHKL(NREF,WH,WK,WL,S4VR,RFO,IH,IK,IL,MKANG,SS4V,EV
     +     ,CELLB,NURO,NRALL,SS4ROT)
      NUMROT = NRALL
C     New random set.
      DO NSET = 1, NRRT
C     Three random EULER angles.
         EU1T = EU1M*RANG2(IR3)
         EU2T = EU2M*RANG2(IR3)
         EU3T = EU3M*RANG2(IR3)
         IF (NSET.EQ.1) THEN
            EU1T = 0.0
            EU2T = 0.0
            EU3T = 0.0
         END IF
C     Matrix for index.
         CALL MATHCP4(EU1T,EU2T,EU3T,DTOR,ROTH)
C     New reflection.
         DO NF = 1, NRALL
            TH = WH(NF)
            TK = WK(NF)
            TL = WL(NF)
C     New index RH, RK and RL after rotate.
            CALL ROTXH(TH,TK,TL,RH,RK,RL,ROTH)
C     Find SF.
            CALL FINDFC(RH,RK,RL,TFC,INDMAX,APAR)
            RFC(NF) = TFC
         END DO
C     Scale FC.
         CALL ROTSC(NRALL,RFO,RFC,S4VR,SS4ROT,20)
C     Free R-Factor as figure of merit.
         CALL ROTRF(NRALL,RFO,RFC,NSET,CCM,0,MRRT)
         IF (CCM.GT.CCMAX) THEN
            NSAV = NSAV + 1
            NSEU(NSAV) = NSET
            EU1(NSAV) = EU1T
            EU2(NSAV) = EU2T
            EU3(NSAV) = EU3T
            EUCC(NSAV) = CCM
            IF (NSAV.EQ.5000) THEN
               CALL SORTCC(EUCC,EU1,EU2,EU3,NSEU,NSAV)
               CCMAX = EUCC(NCC)
               NSAV = NCC
            END IF
         END IF
      END DO
C     Sort on CC.
 600  CALL SORTCC(EUCC,EU1,EU2,EU3,NSEU,NSAV)
      IF (NSAV.GT.NCC) NSAV = NCC
C     Use all reflections (MKANG>0).
      IF (MODR.GT.1.OR.SS4ROT.LT.SS4HO) THEN
         DO NF = 1, NREF
            MKANG(NF) = IABS(MKANG(NF))
         END DO
C     Resort on CC for all reflections.
         CALL RESORT(INDMAX,APAR,IH,IK,IL,MKANG,SS4V,EV,CELLB
     +       ,WH,WK,WL,S4VR,RFO,RFC,NURO,NSAV,MRRT,NUMROT)
      END IF
C
      IF (MAXSET.GT.NSAV) MAXSET = NSAV
      NLPT = 200
      IF (NLPT.LT.MAXSET) NLPT = MAXSET
      IF (NLPT.GT.NSAV) NLPT = NSAV
      WRITE(6,FMT='(/,A,I10,//17X,A,A)')
     +     ' The total number of sets = ',NRRT,
     +     'ALPHA     BETA    GAMMA      TX       TY       TZ',
     +     '       CC    NREF    NSET   ORDER'
      DO N = 1, NLPT
         WRITE(6,FMT='(A,A,3F9.2,3F9.5,F10.5,3I8)')
     +       ' INITIAL ',MRRT,EU1(N),EU2(N),EU3(N),0.0,0.0,0.0
     +       ,EUCC(N),NUMROT,N,NSEU(N)
      END DO
      RETURN
      END
C     --------------------------------------------------------
C     Rotation function.
      SUBROUTINE ROTFRA(INDMAX,APAR,IH,IK,IL,MKANG,SS4V,EV
     +          ,CELLB,WH,WK,WL,S4VR,RFO,RFC,NURO)
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     Eulerian angles: EU1=ALPHA, EU2=BETA, EU3=GAMMA.
      COMMON/EULER/EU1M,EU2M,EU3M,
     +     EU1(5000),EU2(5000),EU3(5000),EUCC(5000),NSEU(5000)
     +     ,TR1(5000),TR2(5000),TR3(5000),TRCS(5000),NSTR(5000)
C
      INTEGER INDMAX,IH(NREF),IK(NREF),IL(NREF),MKANG(NREF)
      REAL CELLB
      REAL APAR(-INDMAX:INDMAX,-INDMAX:INDMAX,0:INDMAX),
     +     SS4V(NREF),EV(NREF)
     +     ,WH(NURO),WK(NURO),WL(NURO),S4VR(NURO),RFO(NURO),RFC(NURO)
C     Local.
      CHARACTER*4 MROT
      INTEGER NSET,NSAV,NCC,NP1M,NP2M,NP3M,NLPT
      REAL ROTH(3,3),RH,RK,RL,TH,TK,TL,
     +     CCM,CCMAX
C
      DATA MROT/'ROTF'/
      IMSG = 4
C     Number for EU1.
      NP1M = INT(EU1M/STEP + 0.5)
C     Number for EU2.
      NP2M = INT(EU2M/STEP + 0.5)
C     Number for EU3.
      NP3M = INT(EU3M/STEP + 0.5)
      NSAV = 0
      NCC = 4000
      CCMAX = -1.0
      NUMROT = 0
C     Use 1/MODR reflections.
      IF (MODR.GT.1) THEN
         DO NF = 1, NREF
            IF (MOD(NF,MODR).NE.0) MKANG(NF) = -IABS(MKANG(NF))
         END DO
      END IF
C     Transfer h,k,l to H,K,L,FO,SS4 in large cell.
      CALL GETHKL(NREF,WH,WK,WL,S4VR,RFO,IH,IK,IL,MKANG,SS4V,EV
     +     ,CELLB,NURO,NRALL,SS4ROT)
      NUMROT = NRALL
C     Searching three angles.
      NSET = 0
      DO NP1 = 1, NP1M
         EU1T = REAL(NP1 - 1)*STEP
         DO NP2 = 1, NP2M
            EU2T = REAL(NP2 - 1)*STEP
            DO NP3 = 1, NP3M
               EU3T = REAL(NP3 - 1)*STEP
               NSET = NSET + 1
C     Matrix for index.
               CALL MATHCP4(EU1T,EU2T,EU3T,DTOR,ROTH)
C     New reflection.
               DO NF = 1, NRALL
                  TH = WH(NF)
                  TK = WK(NF)
                  TL = WL(NF)
C     New index RH, RK and RL after rotate.
                  CALL ROTXH(TH,TK,TL,RH,RK,RL,ROTH)
C     Find SF.
                  CALL FINDFC(RH,RK,RL,TFC,INDMAX,APAR)
                  RFC(NF) = TFC
               END DO
C     Scale FC.
               CALL ROTSC(NRALL,RFO,RFC,S4VR,SS4ROT,20)
C     Free R-Factor as figure of merit.
               CALL ROTRF(NRALL,RFO,RFC,NSET,CCM,0,MROT)
               IF (CCM.GT.CCMAX) THEN
                  NSAV = NSAV + 1
                  NSEU(NSAV) = NSET
                  EU1(NSAV) = EU1T
                  EU2(NSAV) = EU2T
                  EU3(NSAV) = EU3T
                  EUCC(NSAV) = CCM
                  IF (NSAV.EQ.5000) THEN
                     CALL SORTCC(EUCC,EU1,EU2,EU3,NSEU,NSAV)
                     CCMAX = EUCC(NCC)
                     NSAV = NCC
                  END IF
               END IF
            END DO
         END DO
      END DO
C     Sort on CC.
      CALL SORTCC(EUCC,EU1,EU2,EU3,NSEU,NSAV)
      IF (NSAV.GT.NCC) NSAV = NCC
C     Use all reflections (MKANG>0).
      IF (MODR.GT.1.OR.SS4ROT.LT.SS4HO) THEN
         DO NF = 1, NREF
            MKANG(NF) = IABS(MKANG(NF))
         END DO
C     Resort on CC for all reflections.
         CALL RESORT(INDMAX,APAR,IH,IK,IL,MKANG,SS4V,EV,CELLB
     +       ,WH,WK,WL,S4VR,RFO,RFC,NURO,NSAV,MROT,NUMROT)
      END IF
C
      IF (MAXSET.GT.NSAV) MAXSET = NSAV
      NLPT = 200
      IF (NLPT.LT.MAXSET) NLPT = MAXSET
      IF (NLPT.GT.NSAV) NLPT = NSAV
      WRITE(6,FMT='(/,A,I10,//17X,A,A)')
     +     ' The total number of sets = ',NSET,
     +     'ALPHA     BETA    GAMMA      TX       TY       TZ',
     +     '       CC    NREF    NSET   ORDER'
      DO N = 1, NLPT
         WRITE(6,FMT='(A,A,3F9.2,3F9.5,F10.5,3I8)')
     +       ' INITIAL ',MROT,EU1(N),EU2(N),EU3(N),0.0,0.0,0.0
     +       ,EUCC(N),NUMROT,N,NSEU(N)
      END DO
      RETURN
      END
C     --------------------------------------------------------
C     Transfer h,k,l to H,K,L in large cell.
      SUBROUTINE GETHKL(NREF,WH,WK,WL,S4VR,RFO,IH,IK,IL,MKANG
     +     ,SS4V,EV,CELLB,NURO,NRALL,SS4ROT)
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
C
      INTEGER NREF,NURO,NRALL,IH(NREF),IK(NREF),IL(NREF)
     +     ,MKANG(NREF)
      REAL WH(NURO),WK(NURO),WL(NURO),S4VR(NURO),RFO(NURO)
      REAL CELLB,SS4ROT,SS4V(NREF),EV(NREF)
C     Local.
      INTEGER NF,LH,LK,LL
      REAL A,B,C,CAL,CBE,SGA,CGA,T1,T2,T3,T4,T5,T6,Q,P,R,SX,SY,SZ
C
      A = CELLP(1)
      B = CELLP(2)
      C = CELLP(3)
      CAL = COS(DTOR*CELLP(4))
      CBE = COS(DTOR*CELLP(5))
      CGA = COS(DTOR*CELLP(6))
      SGA = SIN(DTOR*CELLP(6))
      P = CBE
      Q = (CAL - CGA*CBE)/SGA
      R = SQRT(1.0 - CAL*CAL - CBE*CBE - CGA*CGA + 2.0*CAL*CBE*CGA)/SGA
      T1 = B*C*R*SGA/VOLUME
      T2 = -B*C*R*CGA/VOLUME
      T3 = A*C*R/VOLUME
      T4 = B*C*(Q*CGA - P*SGA)/VOLUME
      T5 = -A*C*Q/VOLUME
      T6 = A*B*SGA/VOLUME
C
      NRALL = 0
      DO 100 NF = 1, NREF
         IF (SS4V(NF).GT.SS4ROT.OR.MKANG(NF).LT.0) GOTO 100
         DO I = 1, NSYMP
            NRALL = NRALL + 1
            LH = NINT(RSYML(1,1,I))*IH(NF) +
     +          NINT(RSYML(2,1,I))*IK(NF) +
     +          NINT(RSYML(3,1,I))*IL(NF)
            LK = NINT(RSYML(1,2,I))*IH(NF) +
     +          NINT(RSYML(2,2,I))*IK(NF) +
     +          NINT(RSYML(3,2,I))*IL(NF)
            LL = NINT(RSYML(1,3,I))*IH(NF) +
     +          NINT(RSYML(2,3,I))*IK(NF) +
     +          NINT(RSYML(3,3,I))*IL(NF)
            SX = REAL(LH)*T1
            SY = REAL(LH)*T2 + REAL(LK)*T3
            SZ = REAL(LH)*T4 + REAL(LK)*T5 + REAL(LL)*T6
            WH(NRALL) = CELLB*SX
            WK(NRALL) = CELLB*SY
            WL(NRALL) = CELLB*SZ
            S4VR(NRALL) = SS4V(NF)
            RFO(NRALL) = EV(NF)
      END DO
 100  CONTINUE
      RETURN
      END
C     --------------------------------------------------------
      SUBROUTINE FINDFC(RH,RK,RL,TFC,INDMAX,APAR)
C     Calculate |FC| by interpolation.
      INTEGER INDMAX
      REAL RH,RK,RL,TFC
      REAL APAR(-INDMAX:INDMAX,-INDMAX:INDMAX,0:INDMAX)
C     Local.
      INTEGER LH,LK,LL
      REAL A1,A2,A3,SFC(2,2,2)
C
      IF (RL.LT.0.0) THEN
         RH = -RH
         RK = -RK
         RL = -RL
      END IF
      IF (RH.GE.0.0) THEN
         LH = INT(RH)
         A1 = RH - REAL(LH)
      ELSE
         LH = INT(RH) - 1
         A1 = RH - REAL(LH)
      END IF
      IF (RK.GE.0.0) THEN
         LK = INT(RK)
         A2 = RK - REAL(LK)
      ELSE
         LK = INT(RK) - 1
         A2 = RK - REAL(LK)
      END IF
      LL = INT(RL)
      A3 = RL - REAL(LL)
C
      DO I = 1, 2
         DO J = 1, 2
            DO K = 1, 2
               SFC(I,J,K) = APAR(LH+I-1,LK+J-1,LL+K-1)
            END DO
         END DO
      END DO
C
      CALL PO(SFC,A1,A2,A3,TFC)
      RETURN
      END
C     --------------------------------------------------------
C     SORT ON A (CC).
      SUBROUTINE SORTCC(A,B,C,D,IC,N)
      REAL A(N),B(N),C(N),D(N)
      INTEGER N,IC(N)
C     Local.
      INTEGER INT,IFIN,I1
      REAL T1,T2,T3,T4
      INT = 2
 10   INT = 2*INT
      IF (INT.LT.N) GO TO 10
      INT = MIN0(N,(3*INT)/4-1)
 20   INT = INT/2
      IFIN = N - INT
      DO 70 II = 1, IFIN
         I = II
         J = I + INT
         IF (A(I).GE.A(J)) GO TO 70
         T1 = A(J)
         T2 = B(J)
         T3 = C(J)
         T4 = D(J)
         I1 = IC(J)
 40      A(J) = A(I)
         B(J) = B(I)
         C(J) = C(I)
         D(J) = D(I)
         IC(J) = IC(I)
         J = I
         I = I - INT
         IF (I.LE.0) GO TO 60
         IF (A(I).LT.T1) GO TO 40
 60      A(J) = T1
         B(J) = T2
         C(J) = T3
         D(J) = T4
         IC(J) = I1
 70   CONTINUE
      IF (INT.GT.1) GO TO 20
      RETURN
      END
C     --------------------------------------------------------
C     SORT ON A (CC).
      SUBROUTINE SORTC6(A,B,C,D,E,F,G,IC,N)
      REAL A(N),B(N),C(N),D(N),E(N),F(N),G(N)
      INTEGER N,IC(N)
C     Local.
      INTEGER INT,IFIN,I1
      REAL T1,T2,T3,T4
      INT = 2
 10   INT = 2*INT
      IF (INT.LT.N) GO TO 10
      INT = MIN0(N,(3*INT)/4-1)
 20   INT = INT/2
      IFIN = N - INT
      DO 70 II = 1, IFIN
         I = II
         J = I + INT
         IF (A(I).GE.A(J)) GO TO 70
         T1 = A(J)
         T2 = B(J)
         T3 = C(J)
         T4 = D(J)
         T5 = E(J)
         T6 = F(J)
         T7 = G(J)
         I1 = IC(J)
 40      A(J) = A(I)
         B(J) = B(I)
         C(J) = C(I)
         D(J) = D(I)
         E(J) = E(I)
         F(J) = F(I)
         G(J) = G(I)
         IC(J) = IC(I)
         J = I
         I = I - INT
         IF (I.LE.0) GO TO 60
         IF (A(I).LT.T1) GO TO 40
 60      A(J) = T1
         B(J) = T2
         C(J) = T3
         D(J) = T4
         E(J) = T5
         F(J) = T6
         G(J) = T7
         IC(J) = I1
 70   CONTINUE
      IF (INT.GT.1) GO TO 20
      RETURN
      END
C     ---------------------------------------------------------------
C     Resort on CC for all reflections.
      SUBROUTINE RESORT(INDMAX,APAR,IH,IK,IL,MKANG,SS4V,EV,CELLB
     +     ,WH,WK,WL,S4VR,RFO,RFC,NURO,NSAV,MESG,NUMROT)
C
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     Eulerian angles: EU1=ALPHA, EU2=BETA, EU3=GAMMA.
      COMMON/EULER/EU1M,EU2M,EU3M,
     +     EU1(5000),EU2(5000),EU3(5000),EUCC(5000),NSEU(5000)
     +     ,TR1(5000),TR2(5000),TR3(5000),TRCS(5000),NSTR(5000)
C     Data.
      CHARACTER*4 MESG
      INTEGER NSAV,NUMROT,NURO,INDMAX
      INTEGER IH(NREF),IK(NREF),IL(NREF),MKANG(NREF)
      REAL CELLB
      REAL EV(NREF),SS4V(NREF)
      REAL WH(NURO),WK(NURO),WL(NURO),S4VR(NURO),RFO(NURO),RFC(NURO)
C     Structure Calculation..
      REAL APAR(-INDMAX:INDMAX,-INDMAX:INDMAX,0:INDMAX)
C     Local.
      INTEGER NSS4
      REAL ROTH(3,3),TFC,CCM
C
      NUMROT = 0
      NSS4 = 20
C     Transfer h,k,l to H,K,L in large cell.
      CALL GETHKL(NREF,WH,WK,WL,S4VR,RFO,IH,IK,IL,MKANG,SS4V,EV
     +     ,CELLB,NURO,NRALL,SS4HO)
C     Initial values.
      DO NS = 1, NSAV
         EU1S = EU1(NS)
         EU2S = EU2(NS)
         EU3S = EU3(NS)
C     Matrix for index.
         CALL MATHCP4(EU1S,EU2S,EU3S,DTOR,ROTH)
         DO NF = 1, NRALL
            IF (NS.EQ.1) NUMROT = NUMROT + 1
            TH = WH(NF)
            TK = WK(NF)
            TL = WL(NF)
C     New index RH, RK and RL after rotate.
            CALL ROTXH(TH,TK,TL,RH,RK,RL,ROTH)
C     Find SF.
            CALL FINDFC(RH,RK,RL,TFC,INDMAX,APAR)
            RFC(NF) = TFC
         END DO
C     Scale FC.
         CALL ROTSC(NRALL,RFO,RFC,S4VR,SS4HO,NSS4)
C     Free R-Factor as figure of merit.
         CALL ROTRF(NRALL,RFO,RFC,NS,CCM,0,MESG)
C     Store new CC.
         EUCC(NS) = CCM
         NSEU(NS) = NS
      END DO
C     Sort on New CC.
      CALL SORTCC(EUCC,EU1,EU2,EU3,NSEU,NSAV)
      RETURN
      END
C     ----------------------------------------------------
C
      SUBROUTINE ORTMAT(CELL,NCODE,RO,RF)
C     Return orthogonalization matrix
C
C     On entry:
C     CELL(6)    cell dimensions
c     NCODE      orthogonalization code
C       = 1  axes along a, c* x a, c*  (Brookhaven standard, default)
C       = 2  axes along b, a* x b, a*
C       = 3  axes along c, b* x c, b*
C       = 4  axes along a+b, c* x (a+b), c*
C       = 5  axes along a*, c x a*, c       ( Rollett )
C       = 6  axes along a, b*, a x b*
C       = 7  axes along a*, b, a* x b   (TNT)
C
C     On exit:
C     RO(4,4)    orthogonalization matrix
C     RF(4,4)    inverse of ro, fractionalization matrix
C
      IMPLICIT NONE
C
      REAL CELL(6),RO(4,4),RF(4,4)
      INTEGER NCODE
c
      REAL RALL(3,3,6),VOL
      INTEGER I,J,K
C
      EXTERNAL RBFRO1,RBRINV,CCPERR
C
      CALL RBFRO1(CELL,VOL,RALL)
      DO J = 1, 4
         DO I = 1, 4
            RO(I,J) = 0.0
         END DO
      END DO
C
      IF (NCODE.LE.6) THEN
         DO J = 1, 3
            DO I = 1, 3
               RO(I,J) = RALL(I,J,NCODE)
            END DO
         END DO
      ELSE IF (NCODE.EQ.7) THEN
C     ncode 7 for TNT, permute ncode 2, rows of matrix are 3,1,2
         DO J = 1, 3
            DO I = 1, 3
               K = I - 1
               IF (K.LE.0) K = K + 3
               RO(I,J) = RALL(K,J,2)
            END DO
         END DO
      ELSE
         WRITE (6, '(/A,I6,A/)')
     +       ' *** Illegal NCODE: ',NCODE,' ***'
         CALL CCPERR(1,' *** Illegal NCODE ***')
      END IF
C
      RO(4,4) = 1.0
C
C     Invert
      CALL RBRINV(RO,RF)
C
      RETURN
      END
C     --------------------------------------------------------
C     PROGRAM TO READ BROOKHAVEN FILE in PDB format.
C
      SUBROUTINE PDBIN(NPOS)
C
      INTEGER NAMAX,NPOS
      PARAMETER (NAMAX=30000)
      COMMON/XYZIN/NUMA,OC(NAMAX),IZ(NAMAX),XINF(NAMAX),YINF(NAMAX)
     +     ,ZINF(NAMAX),XINO(NAMAX),YINO(NAMAX),ZINO(NAMAX),BFAC(NAMAX)
      COMMON/PDBROF/RMO(4,4),RMF(4,4)
      CHARACTER*4 NW,N1(4)
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/ATMFACTOR/NKD,AL(8),AS(8),BL(8),BS(8),CL(8),CS(8)
     +  ,DL(8),DS(8),EL(8),NW(8),NO(8),NE(8)
C
      LOGICAL OP
      INTEGER NG,IELEC
      REAL A(4),B(4),C,CU(2),MO(2)
C
      EXTERNAL SFREAD2,CCPERR,CCPDPN
C     Local.
      REAL X1,Y1,Z1,X2,Y2,Z2,OCC,BISO,U(6)
      INTEGER IFAIL,ISER,IRS,IXYZIN,IZT
      CHARACTER*4 ATNAM,RESNAM,RESNO,SEGID,ID
      CHARACTER*1 CHNNAM,ALTCOD,INSCOD
C
      EXTERNAL RBCELL,XYZADVANCE,XYZATOM,XYZCLOSE,XYZCOORD,
     +        XYZINIT,XYZOPEN
C
C     OPEN FILE
C
      IFAIL = 0
      IXYZIN = 0
      CALL XYZOPEN('XYZIN','INPUT',' ',IXYZIN,IFAIL)
C
C     READ COORDINATE RECORDS
C
      NUMA = 0
 10   CONTINUE
      IOUT =0
      ITER = 0
      BISO = 0.0
      CALL XYZADVANCE(IXYZIN,IOUT,ITER,*10,*100)
      CALL XYZATOM(IXYZIN,ISER,ATNAM,RESNAM,CHNNAM,IRS,
     +     RESNO,INSCOD,ALTCOD,SEGID,IZT,ID)
C     Check the element.
      DO N = 1, NKD
         IF (NE(N).EQ.IZT) GOTO 20
      END DO
      NKD = NKD + 1
      NW(NKD) = ID
      IF (ID(1:1).EQ.' ') NW(NKD) = ID(2:2)
      NE(NKD) = IZT
      NO(NKD) = 1
      CALL SFREAD2(NW(NKD),NG,A,B,C,IWT,IELEC,CU,MO,Ifail)
      IF (Ifail.EQ.-1)
     +     CALL CCPERR(1,' No match for atom label(ATSORT)')
      AL(NKD) = A(1)
      AS(NKD) = B(1)
      BL(NKD) = A(2)
      BS(NKD) = B(2)
      CL(NKD) = A(3)
      CS(NKD) = B(3)
      DL(NKD) = A(4)
      DS(NKD) = B(4)
      EL(NKD) = C
 20   CALL XYZCOORD(IXYZIN,'O','U',X1,Y1,Z1,OCC,BISO,U)
      IF (NPOS.GT.0) CALL XYZCOORD(IXYZIN,'F','U',X2,Y2,Z2,OCC,BISO,U)
C
C     THIS PROGRAM CANNOT HANDLE ANISOTROPIC TEMPERATURE FACTORS
C     SO SKIP ANY ANISOU CARDS FOUND
C
      IF (U(2).NE.0.0 .OR. U(3).NE.0.0) THEN
         IF (X1.EQ.0.0 .AND. Y1.EQ.0.0 .AND. Z1.EQ.0.0) GOTO 10
      END IF
      NUMA = NUMA + 1
      IF (NUMA.GT.NAMAX) CALL CCPERR(1,
     +     ' *** Error - The number of atoms is too large. ***')
      OC(NUMA) = OCC
      IZ(NUMA) = IZT
      XINO(NUMA) = X1
      YINO(NUMA) = Y1
      ZINO(NUMA) = Z1
      BFAC(NUMA) = BISO
      IF (NPOS.EQ.0) THEN
         X2 = RMF(1,1)*X1 + RMF(1,2)*Y1 + RMF(1,3)*Z1
         Y2 = RMF(2,1)*X1 + RMF(2,2)*Y1 + RMF(2,3)*Z1
         Z2 = RMF(3,1)*X1 + RMF(3,2)*Y1 + RMF(3,3)*Z1
      END IF
      XINF(NUMA) = X2
      YINF(NUMA) = Y2
      ZINF(NUMA) = Z2
      GO TO 10
C
C     END OF FILE REACHED
C
 100  CALL XYZCLOSE(IXYZIN)
      IF (NPOS.EQ.0) THEN
         WRITE(6,FMT='(/,A,I8,A,//,A,A)') ' There are ',NUMA,
     +       ' atoms in whole fragment.',
     +       '    Atom        XO        YO        ZO      OC    BISO',
     +       '  ZJ'
         DO I = 1, NUMA
            IF (I.EQ.1.OR.I.EQ.NUMA)
     +          WRITE(6,FMT='(I8,3F10.3,2F8.2,I4)')
     +          I,XINO(I),YINO(I),ZINO(I),OC(I),BFAC(I),IZ(I)
         END DO
      ELSE
         WRITE(6,FMT='(/,A,I8,A,//,A,A,A)') ' There are ',NUMA,
     +       ' atoms in whole fragment.',
     +       '    Atom        XO        YO        ZO',
     +       '             XF        YF        ZF      OC    BISO',
     +       '  ZJ'
         DO I = 1, NUMA
            IF (I.EQ.1.OR.I.EQ.NUMA)
     +          WRITE(6,FMT='(I8,3F10.3,5X,3F10.5,2F8.2,I4)')
     +          I,XINO(I),YINO(I),ZINO(I),
     +          XINF(I),YINF(I),ZINF(I),OC(I),BFAC(I),IZ(I)
         END DO
      END IF
      RETURN
      END
C     --------------------------------------------------------
C     Calculate SF from true atom positions.
      SUBROUTINE TRUFRA(IH,IK,IL,MKANG,SS4V,EV,FC
     +     ,NX,NY,NZ,LBUF,BUF,YMAP)
C
      INTEGER NAMAX
      PARAMETER (NAMAX=30000)
C     COMMON variables.
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     EU1 --- the first atom of a fragment.
      COMMON/EULER/EU1M,EU2M,EU3M,
     +     EU1(5000),EU2(5000),EU3(5000),EUCC(5000),NSEU(5000)
     +     ,TR1(5000),TR2(5000),TR3(5000),TRCS(5000),NSTR(5000)
      COMMON/XYZIN/NUMA,OC(NAMAX),IZ(NAMAX),XINF(NAMAX),YINF(NAMAX)
     +     ,ZINF(NAMAX),XINO(NAMAX),YINO(NAMAX),ZINO(NAMAX),BFAC(NAMAX)
C
      INTEGER IR3,NX,NY,NZ,LBUF
      INTEGER IH(NREF),IK(NREF),IL(NREF),MKANG(NREF)
      REAL SS4V(NREF),EV(NREF),FC(NREF)
      REAL YMAP(NX,NY,NZ),BUF(LBUF)
C     Local.
      CHARACTER*4 MPOS
      INTEGER NUMR,NALL,NSAV,NS,NT,NCC,NLPT,IZT(NAMAX)
      REAL CCMAX,CCM
      REAL XR(NAMAX),YR(NAMAX),ZR(NAMAX),OCT(NAMAX),BF(NAMAX)
C
      DATA MPOS/'POSI'/
      IMSG = 2
      NSAV = 0
      NCC = 1000
      CCMAX = -1.0
      IR3 = ISEED
      NTEST = 100*NPOS
C     Random pick up atoms.
      NUMR = NUMA - NFRA + 1
      DO 500 NSET = 1, NTEST
         NALL = 0
         NS = INT(RANG2(IR3)*REAL(NUMR) + 0.5) + 1
         IF (NS.GT.NUMR) NS = NUMR
         IF (NSET.EQ.1) NS = NSTA
         NT = NS + NFRA - 1
C     Check if it is the same set.
         IF (NSAV.GT.0) THEN
            DO NCK = 1, NSAV
               NTEM = INT(EU1(NCK) + 0.5)
               IF (NS.EQ.NTEM) GOTO 500
            END DO
         END IF
         IF (NPOS.EQ.1) THEN
            NSEU(1) = NSET
            EU1(1) = REAL(NS)
            RETURN
         END IF
         DO NA = NS, NT
            NALL = NALL + 1
            OCT(NALL) = OC(NA)
            IZT(NALL) = IZ(NA)
            XR(NALL) = XINF(NA)
            YR(NALL) = YINF(NA)
            ZR(NALL) = ZINF(NA)
            BF(NALL) = BFAC(NA)
         END DO
C     Calculate SF.
         IF (NALL.LT.100) THEN
            CALL CALFP(NALL,OCT,IZT,BF,XR,YR,ZR,NEXT,MKANG,IH,IK,IL
     +          ,SS4V,FC,PHIFC)
         ELSE
            CALL CALEC(NALL,OCT,IZT,BF,XR,YR,ZR,NEXT,MKANG,IH,IK,IL
     +          ,SS4V,FC,PHIFC,YMAP,LBUF,BUF)
         END IF
C     Scale FC.
         CALL FASTSC(NREF,MKANG,EV,FC,SS4V,SS4HO,20)
C     Free R-Factor as figure of merit.
         CALL FASTRF(NREF,MKANG,EV,FC,CCM)
         IF (CCM.GT.CCMAX) THEN
            NSAV = NSAV + 1
            NSEU(NSAV) = NSET
            EU1(NSAV) = REAL(NS)
            EUCC(NSAV) = CCM
            IF (NSAV.EQ.2000) THEN
C     Sort on CC.
               CALL SORTCC(EUCC,EU1,EU2,EU3,NSEU,NSAV)
               CCMAX = EUCC(NCC)
               NSAV = NCC
            END IF
            IF (NSAV.EQ.NPOS) GOTO 600
         END IF
 500  CONTINUE
C     Sort on CC.
 600  CALL SORTCC(EUCC,EU1,EU2,EU3,NSEU,NSAV)
C
      IF (NSAV.GT.NCC) NSAV = NCC
      IF (MAXSET.GT.NSAV) MAXSET = NSAV
      NLPT = 200
      IF (NLPT.LT.MAXSET) NLPT = MAXSET
      IF (NLPT.GT.NSAV) NLPT = NSAV
      WRITE(6,FMT='(/,A,I10,//18X,A)')
     +     ' The total number of sets = ',NPOS,
     +     'NSTA       CC    NREF    NSET   ORDER'
      DO N = 1, NLPT
         NST = NINT(EU1(N))
         WRITE(6,FMT='(A,A,I9,F10.5,3I8)') ' INITIAL '
     +       ,MPOS,NST,EUCC(N),NREF,N,NSEU(N)
      END DO
      RETURN
      END
C     --------------------------------------------------------
C     Calculate SF random aton.
      SUBROUTINE RANATM(IH,IK,IL,MKANG,SS4V,EV,FC)
C
      INTEGER NAMAX
      PARAMETER (NAMAX=30000)
      LOGICAL LPAXISX,LPAXISY,LPAXISZ
      INTEGER NORIG
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/ORIGJY/LPAXISX,LPAXISY,LPAXISZ,NORIG,ORIG(3,96)
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     EU1 --- seed of random number generater.
      COMMON/EULER/EU1M,EU2M,EU3M,
     +     EU1(5000),EU2(5000),EU3(5000),EUCC(5000),NSEU(5000)
     +     ,TR1(5000),TR2(5000),TR3(5000),TRCS(5000),NSTR(5000)
      COMMON/XYZIN/NUMA,OC(NAMAX),IZ(NAMAX),XINF(NAMAX),YINF(NAMAX)
     +     ,ZINF(NAMAX),XINO(NAMAX),YINO(NAMAX),ZINO(NAMAX),BFAC(NAMAX)
C
      INTEGER IR3
      INTEGER IH(NREF),IK(NREF),IL(NREF),MKANG(NREF)
      REAL SS4V(NREF),EV(NREF),FC(NREF)
C     Local.
      CHARACTER*4 MATM
      INTEGER NALL,NSAV,NCC,NLPT,IR3S,IZT(1000)
      REAL CCMAX,CCM
      REAL XR(1000),YR(1000),ZR(1000),OCT(1000)
C
      DATA MATM/'RATM'/
      NNN = NREF
C     Use 1/MODR reflections.
      IF (MODR.GT.1) THEN
         DO NF = 1, NREF
            IF (MOD(NF,MODR).NE.0) MKANG(NF) = -IABS(MKANG(NF))
         END DO
      END IF
      TEMP = 100.0/REAL(MODR)
      WRITE(6,FMT='(/,A,F10.2,A)')
     +     ' Random single atom searching using'
     +     ,TEMP,' % reflections before RESORT.'
C
      IMSG = 1
      NSAV = 0
      NCC = 4000
      CCMAX = -1.0
      IR3 = ISEED
      A = CELLP(1)
      B = CELLP(2)
      C = CELLP(3)
      CSA = COS(DTOR*CELLP(4))
      CSB = COS(DTOR*CELLP(5))
      CSC = COS(DTOR*CELLP(6))
      NALL = 1
      IF (NSYMP.EQ.1) THEN
         TSX = ASX
         TSY = ASY
         TSZ = ASZ
         NALL = 2
         OCT(NALL) = 1.0
         IZT(NALL) = 8
         XR(NALL) = 0.0
         YR(NALL) = 0.0
         ZR(NALL) = 0.0
      ELSE
         TSX = TXM*ASX
         TSY = TYM*ASY
         TSZ = TZM*ASZ
      END IF
C     Random atoms.
      DO NSET = 1, NATM
         IR3S = IR3
C     Check if it is the same set.
         IF (NSAV.GT.0) THEN
            DO NCK = 1, NSAV
               IF (IR3S.EQ.NSEU(NCK)) GOTO 100
            END DO
         END IF
         OCT(1) = 1.0
         IZT(1) = 8
         XR(1) = RANG2(IR3)*TSX
         YR(1) = RANG2(IR3)*TSY
         ZR(1) = RANG2(IR3)*TSZ
C     Calculate SF.
         CALL CALFC(NALL,OCT,IZT,XR,YR,ZR,NNN,MKANG,IH,IK,IL,FC)
C     Scale FC.
         CALL FASTSC(NNN,MKANG,EV,FC,SS4V,SS4HO,20)
C     Free CC as figure of merit.
         CALL FASTRF(NNN,MKANG,EV,FC,CCM)
         IF (CCM.GT.CCMAX) THEN
            NSAV = NSAV + 1
            NSEU(NSAV) = IR3S
            EU1(NSAV) = XR(1)
            EU2(NSAV) = YR(1)
            EU3(NSAV) = ZR(1)
            EUCC(NSAV) = CCM
            IF (NSAV.EQ.5000) THEN
C     Sort on CC.
               CALL SORTCC(EUCC,EU1,EU2,EU3,NSEU,NSAV)
               CCMAX = EUCC(NCC)
               NSAV = NCC
            END IF
         END IF
      END DO
C     Sort on CC.
 100  CALL SORTCC(EUCC,EU1,EU2,EU3,NSEU,NSAV)
C
      IF (NSAV.GT.NCC) NSAV = NCC
C     Use all reflections for 1000 sets.
      IF (MODR.GT.1) THEN
         DO NF = 1, NREF
            IF (MOD(NF,MODR).NE.0) MKANG(NF) = IABS(MKANG(NF))
         END DO
      END IF
      NALL = 1
      IF (NSYMP.EQ.1) THEN
         NALL = 2
         OCT(NALL) = 1.0
         IZT(NALL) = 8
         XR(NALL) = 0.0
         YR(NALL) = 0.0
         ZR(NALL) = 0.0
      END IF
      DO NSET = 1, NSAV
         OCT(NALL) = 1.0
         IZT(NALL) = 8
         XR(1) = EU1(NSET)
         YR(1) = EU2(NSET)
         ZR(1) = EU3(NSET)
C     Calculate SF.
         CALL CALFC(NALL,OCT,IZT,XR,YR,ZR,NNN,MKANG,IH,IK,IL,FC)
C     Scale FC.
         CALL FASTSC(NNN,MKANG,EV,FC,SS4V,SS4HO,20)
C     Free CC as figure of merit.
         CALL FASTRF(NNN,MKANG,EV,FC,CCM)
         EUCC(NSET) = CCM
         NSEU(NSET) = NSET
      END DO
C     Sort on CC.
      CALL SORTCC(EUCC,EU1,EU2,EU3,NSEU,NSAV)
C     Check if they are close to origin.
      NSET = 0
      DO N = 1, NSAV
         CALL DISTA(DIS,EU1(N),EU2(N),EU3(N)
     +       ,0.0,0.0,0.0,A,B,C,CSA,CSB,CSC)
         IF (DIS.LT.1.5) THEN
            EUCC(N) = -ABS(EUCC(N))
            NSET = NSET + 1
         END IF
      END DO
C     Check peaks if they are close to each other.
      DO N = 2, NSAV
         IF (EUCC(N).GT.0.0) THEN
            DO NT = 1, N-1
               IF (EUCC(NT).GT.0.0) THEN
                  CALL DISTA(DIS,EU1(N),EU2(N),EU3(N)
     +                ,EU1(NT),EU2(NT),EU3(NT),A,B,C,CSA,CSB,CSC)
                  IF (DIS.LT.0.5) THEN
                     EUCC(N) = -ABS(EUCC(N))
                     NSET = NSET + 1
                     GOTO 200
                  END IF
               END IF
            END DO
         END IF
 200     CONTINUE
      END DO
C     Sort on CC.
      CALL SORTCC(EUCC,EU1,EU2,EU3,NSEU,NSAV)
      NSAV = NSAV - NSET
      IF (MAXSET.GT.NSAV) MAXSET = NSAV
C     Parameter shift to get maximum CC.
      NSX = 1
      NSY = 1
      NSZ = 1
      IF (LPAXISX) NSX = 0
      IF (LPAXISY) NSY = 0
      IF (LPAXISZ) NSZ = 0
      XSF = 0.2*GRID/CELLP(1)
      YSF = 0.2*GRID/CELLP(2)
      ZSF = 0.2*GRID/CELLP(3)
      DO N = 1, MAXSET
 300     CCMAX = -1.0
         DO N1 = -NSX, NSX
            XR(1) = EU1(N) + XSF*REAL(N1)
            DO N2 = -NSY, NSY
               YR(1) = EU2(N) + YSF*REAL(N2)
               DO N3 = -NSZ, NSZ
                  ZR(1) = EU3(N) + ZSF*REAL(N3)
C     Calculate SF.
                  CALL CALFC(NALL,OCT,IZT,XR,YR,ZR,NNN,MKANG
     +                ,IH,IK,IL,FC)
C     Scale FC.
                  CALL FASTSC(NNN,MKANG,EV,FC,SS4V,SS4HO,20)
C     Free CC as figure of merit.
                  CALL FASTRF(NNN,MKANG,EV,FC,CCM)
                  IF (CCM.GT.CCMAX) THEN
                     XSAV = XR(1)
                     YSAV = YR(1)
                     ZSAV = ZR(1)
                     CCMAX = CCM
                  END IF
               END DO
            END DO
         END DO
         IF (CCMAX.GT.EUCC(N)) THEN
            EU1(N) = XSAV
            EU2(N) = YSAV
            EU3(N) = ZSAV
            EUCC(N) = CCMAX
            GOTO 300
         END IF
      END DO
C     Sort on CC.
      CALL SORTCC(EUCC,EU1,EU2,EU3,NSEU,NSAV)
C     Check peaks if they are close to each other.
      DO N = 2, MAXSET
         IF (EUCC(N).GT.0.0) THEN
            DO NT = 1, N-1
               IF (EUCC(NT).GT.0.0) THEN
                  CALL DISTA(DIS,EU1(N),EU2(N),EU3(N)
     +                ,EU1(NT),EU2(NT),EU3(NT),A,B,C,CSA,CSB,CSC)
                  IF (DIS.LT.0.5) THEN
                     EUCC(N) = -ABS(EUCC(N))
                     NSET = NSET + 1
                     GOTO 400
                  END IF
               END IF
            END DO
         END IF
 400     CONTINUE
      END DO
C     Sort on CC.
      CALL SORTCC(EUCC,EU1,EU2,EU3,NSEU,NSAV)
      NLPT = 200
      IF (NLPT.LT.MAXSET) NLPT = MAXSET
      IF (NLPT.GT.NSAV) NLPT = NSAV
      WRITE(6,FMT='(/,A,I10,//21X,A)')
     +     ' The total number of sets = ',NATM,
     +     'XF       YF        ZF        CC    NREF    NSET     ORDER'
      DO N = 1, NLPT
         XINF(N) = EU1(N)
         YINF(N) = EU2(N)
         ZINF(N) = EU3(N)
         WRITE(6,FMT='(A,A,4F10.5,2I8,I10)') ' INITIAL '
     +       ,MATM,EU1(N),EU2(N),EU3(N),EUCC(N),NNN,N,NSEU(N)
         NSEU(N) = N
      END DO
      RETURN
      END
C     --------------------------------------------------------
      SUBROUTINE DISTA(DIS,X1,Y1,Z1,X2,Y2,Z2,A,B,C,CA,CB,CC)
C     Distance between two atoms with symmetry.
      LOGICAL LPAXISX,LPAXISY,LPAXISZ
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/ORIGJY/LPAXISX,LPAXISY,LPAXISZ,NORIG,ORIG(3,96)
C
      REAL DIS,X1,Y1,Z1,X2,Y2,Z2,A,B,C,CA,CB,CC
C     Local.
      REAL DX,DY,DZ,T1,T2,T3
C
      DIS = 100.0
      DO NS = 1, NSYMP
         XT = RSYML(1,1,NS)*X2 + RSYML(1,2,NS)*Y2 +
     +       RSYML(1,3,NS)*Z2 + RSYML(1,4,NS)
         YT = RSYML(2,1,NS)*X2 + RSYML(2,2,NS)*Y2 +
     +       RSYML(2,3,NS)*Z2 + RSYML(2,4,NS)
         ZT = RSYML(3,1,NS)*X2 + RSYML(3,2,NS)*Y2 +
     +       RSYML(3,3,NS)*Z2 + RSYML(3,4,NS)
         DO NO = 1, NORIG
            XTT = AMOD(XT + ORIG(1,NO) + 10.0,1.0)
            YTT = AMOD(YT + ORIG(2,NO) + 10.0,1.0)
            ZTT = AMOD(ZT + ORIG(3,NO) + 10.0,1.0)
            DX = XTT - X1
            DY = YTT - Y1
            DZ = ZTT - Z1
            T1 = 2.0*DX*DY*A*B*CC
            T2 = 2.0*DZ*DX*C*A*CB
            T3 = 2.0*DY*DZ*B*C*CA
            DIST = DX*DX*A*A + DY*DY*B*B + DZ*DZ*C*C
            DIST = SQRT(DIST + T1 + T2 + T3)
            IF (DIST.LT.DIS) DIS = DIST
         END DO
      END DO
      RETURN
      END
C     --------------------------------------------------------
C     Distance between two atoms without symmetry.
      SUBROUTINE DISTAN(DIS,X1,Y1,Z1,X2,Y2,Z2,A,B,C,CA,CB,CC)
C
      REAL DIS,X1,Y1,Z1,X2,Y2,Z2,A,B,C,CA,CB,CC
C     Local.
      REAL DX,DY,DZ,T1,T2,T3
      X3 = X2
      Y3 = Y2
      Z3 = Z2
      IF (X3-X1.GT.0.5) THEN
         X3 = X3 - 1.0
      ELSE IF (X3-X1.LT.-0.5) THEN
         X3 = X3 + 1.0
      END IF
      IF (Y3-Y1.GT.0.5) THEN
         Y3 = Y3 - 1.0
      ELSE IF (Y3-Y1.LT.-0.5) THEN
         Y3 = Y3 + 1.0
      END IF
      IF (Z3-Z1.GT.0.5) THEN
         Z3 = Z3 - 1.0
      ELSE IF (Z3-Z1.LT.-0.5) THEN
         Z3 = Z3 + 1.0
      END IF
C
      DX = X3 - X1
      DY = Y3 - Y1
      DZ = Z3 - Z1
      T1 = 2.0*DX*DY*A*B*CC
      T2 = 2.0*DZ*DX*C*A*CB
      T3 = 2.0*DY*DZ*B*C*CA
      DIST = DX*DX*A*A + DY*DY*B*B + DZ*DZ*C*C
      DIS = SQRT(DIST + T1 + T2 + T3)
      RETURN
      END
C     --------------------------------------------------------
      SUBROUTINE PO(PA,A1,A2,A3,A)
C     Interpolate SF.
      REAL PA(2,2,2),A1,A2,A3,A
C
      A = A1*A2*A3*PA(2,2,2) + (1.0 - A1)*A2*A3*PA(1,2,2)
     +     + A1*(1.0 - A2)*A3*PA(2,1,2) + A1*A2*(1.0 - A3)*PA(2,2,1)
     +     + (1.0 - A1)*(1.0 - A2)*A3*PA(1,1,2)
     +     + (1.0 - A1)*A2*(1.0 - A3)*PA(1,2,1)
     +     + A1*(1.0 - A2)*(1.0 - A3)*PA(2,1,1)
     +     + (1.0 - A1)*(1.0 - A2)*(1.0 - A3)*PA(1,1,1)
      RETURN
      END
C     --------------------------------------------------------
C     Matrix from PDB to Real unit cell..
      SUBROUTINE MATB(ROTC,ROTX)
C     COMMON variables.
      COMMON/PDBROF/RMO(4,4),RMF(4,4)
C
      REAL ROTC(3,3),ROTX(3,3)
      ROTC(1,1) = RMF(1,1)*ROTX(1,1) + RMF(1,2)*ROTX(2,1)
     +     + RMF(1,3)*ROTX(3,1)
      ROTC(1,2) = RMF(1,1)*ROTX(1,2) + RMF(1,2)*ROTX(2,2)
     +     + RMF(1,3)*ROTX(3,2)
      ROTC(1,3) = RMF(1,1)*ROTX(1,3) + RMF(1,2)*ROTX(2,3)
     +     + RMF(1,3)*ROTX(3,3)
      ROTC(2,1) = RMF(2,1)*ROTX(1,1) + RMF(2,2)*ROTX(2,1)
     +     + RMF(2,3)*ROTX(3,1)
      ROTC(2,2) = RMF(2,1)*ROTX(1,2) + RMF(2,2)*ROTX(2,2)
     +     + RMF(2,3)*ROTX(3,2)
      ROTC(2,3) = RMF(2,1)*ROTX(1,3) + RMF(2,2)*ROTX(2,3)
     +     + RMF(2,3)*ROTX(3,3)
      ROTC(3,1) = RMF(3,1)*ROTX(1,1) + RMF(3,2)*ROTX(2,1)
     +     + RMF(3,3)*ROTX(3,1)
      ROTC(3,2) = RMF(3,1)*ROTX(1,2) + RMF(3,2)*ROTX(2,2)
     +     + RMF(3,3)*ROTX(3,2)
      ROTC(3,3) = RMF(3,1)*ROTX(1,3) + RMF(3,2)*ROTX(2,3)
     +     + RMF(3,3)*ROTX(3,3)
      RETURN
      END
C     --------------------------------------------------------
C     Matrix by giving angles A1, A2 and A3 for X' = R*X.
      SUBROUTINE MATXCP4(A1,A2,A3,DTOR,ROTX)
C
      REAL A1,A2,A3,DTOR,ROTX(3,3)
      S1 = SIN(DTOR*A1)
      C1 = COS(DTOR*A1)
      S2 = SIN(DTOR*A2)
      C2 = COS(DTOR*A2)
      S3 = SIN(DTOR*A3)
      C3 = COS(DTOR*A3)
      ROTX(1,1) = C1*C2*C3 - S1*S3
      ROTX(1,2) = -C1*C2*S3 - S1*C3
      ROTX(1,3) = C1*S2
      ROTX(2,1) = S1*C2*C3 + C1*S3
      ROTX(2,2) = -S1*C2*S3  + C1*C3
      ROTX(2,3) = S1*S2
      ROTX(3,1) = -S2*C3
      ROTX(3,2) = S2*S3
      ROTX(3,3) = C2
      RETURN
      END
C     --------------------------------------------------------
C     Matrix by giving angles A1, A2 and A3 for H' = R*H.
      SUBROUTINE MATHCP4(A1,A2,A3,DTOR,ROTH)
C
      REAL A1,A2,A3,DTOR,ROTH(3,3)
      S1 = SIN(DTOR*A1)
      C1 = COS(DTOR*A1)
      S2 = SIN(DTOR*A2)
      C2 = COS(DTOR*A2)
      S3 = SIN(DTOR*A3)
      C3 = COS(DTOR*A3)
      ROTH(1,1) = C1*C2*C3 - S1*S3
      ROTH(1,2) = S1*C2*C3 + C1*S3
      ROTH(1,3) = -S2*C3
      ROTH(2,1) = -C1*C2*S3 - S1*C3
      ROTH(2,2) = -S1*C2*S3  + C1*C3
      ROTH(2,3) = S2*S3
      ROTH(3,1) = C1*S2
      ROTH(3,2) = S1*S2
      ROTH(3,3) = C2
      RETURN
      END
C     --------------------------------------------------------
C     X' = R*X or H' = R*H.
      SUBROUTINE ROTXH(XI,YI,ZI,XO,YO,ZO,ROT)
C
      REAL XI,YI,ZI,XO,YO,ZO,ROT(3,3)
      XO = ROT(1,1)*XI + ROT(1,2)*YI + ROT(1,3)*ZI
      YO = ROT(2,1)*XI + ROT(2,2)*YI + ROT(2,3)*ZI
      ZO = ROT(3,1)*XI + ROT(3,2)*YI + ROT(3,3)*ZI
      RETURN
      END
C     --------------------------------------------------------
C     Translational function.
      SUBROUTINE TRANSL(LBUFT,NURO,NSAV,NSR,EUS1,EUS2,EUS3
     +     ,IH,IK,IL,MKANG,SS4V,EV,FC,APAR,BPAR,BUFT,TMAP
     +     ,MH,MK,ML,TM,PM)
C
      INTEGER NAMAX
      PARAMETER (NAMAX=30000)
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     Eulerian angles: EU1=ALPHA, EU2=BETA, EU3=GAMMA.
      COMMON/EULER/EU1M,EU2M,EU3M,
     +     EU1(5000),EU2(5000),EU3(5000),EUCC(5000),NSEU(5000)
     +     ,TR1(5000),TR2(5000),TR3(5000),TRCS(5000),NSTR(5000)
C     Translation function.
      COMMON/TRXYZ/MAXHT,MAXKT,MAXLT,NXT,NYT,NZT
     +     ,TRX(5000),TRY(5000),TRZ(5000),TRCC(5000)
      COMMON/XYZIN/NUMA,OC(NAMAX),IZ(NAMAX),XINF(NAMAX),YINF(NAMAX)
     +     ,ZINF(NAMAX),XINO(NAMAX),YINO(NAMAX),ZINO(NAMAX),BFAC(NAMAX)
C
      INTEGER IH(NREF),IK(NREF),IL(NREF),MKANG(NREF)
      REAL APAR(-MAXHT:MAXHT,-MAXKT:MAXKT,0:MAXLT),
     +     BPAR(-MAXHT:MAXHT,-MAXKT:MAXKT,0:MAXLT),
     +     SS4V(NREF),EV(NREF),FC(NREF)
C
      INTEGER LBUFT,NURO
      REAL BUFT(LBUFT),TMAP(NXT,NYT,NZT)
      INTEGER MH(NURO),MK(NURO),ML(NURO)
      REAL TM(NURO),PM(NURO)
C     Local.
      CHARACTER*4 MTRA
      INTEGER NSET,NSAV,NCC,NLPT,IZT(NAMAX)
      REAL CCM
      REAL XR(NAMAX),YR(NAMAX),ZR(NAMAX),XN(NAMAX),YN(NAMAX),ZN(NAMAX)
     +     ,OCT(NAMAX)
      REAL EUS1(NSROT),EUS2(NSROT),EUS3(NSROT)
C
      DATA MTRA/'TRAN'/
      IMSG = 6
      IF (NSYMP.EQ.1) RETURN
      NNN = NREF
      IF (SS4H.LT.0.25.AND.NFRA.EQ.1) NNN = NEMAX
      IPAT = 0
      SCF = 1.0/VOLUME
      TXS = 0.0
      TYS = 0.0
      TZS = 0.0
      IF (NFRA.EQ.1) THEN
         NTX = NINT(ASX*TXM*REAL(NXT))
         NTY = NINT(ASY*TYM*REAL(NYT))
         NTZ = NINT(ASZ*TZM*REAL(NZT))
      ELSE
         NTX = NINT(TXM*REAL(NXT))
         NTY = NINT(TYM*REAL(NYT))
         NTZ = NINT(TZM*REAL(NZT))
      END IF
      NCC = 2000
      NCCS = 4000
      NSAV = 0
      A = CELLP(1)
      B = CELLP(2)
      C = CELLP(3)
      CSA = COS(DTOR*CELLP(4))
      CSB = COS(DTOR*CELLP(5))
      CSC = COS(DTOR*CELLP(6))
C     Pick up atoms.
      NALL = 0
      NEND = NSTA + NFRA - 1
      IF (NEND.GT.NUMA) NEND = NUMA
      DO NA = NSTA, NEND
         NALL = NALL + 1
         OCT(NALL) = OC(NA)
         IZT(NALL) = IZ(NA)
         XN(NALL) = XINO(NA)
         YN(NALL) = YINO(NA)
         ZN(NALL) = ZINO(NA)
      END DO
C     Rotate XYZ into real cell.
      CCMIN = -10000.0
      NSET = 0
      EU1S = EUS1(NSR)
      EU2S = EUS2(NSR)
      EU3S = EUS3(NSR)
C     New XYZ after rotated.
      CALL ROTXYZ(NALL,XN,YN,ZN,XR,YR,ZR,EU1S,EU2S,EU3S
     +     ,TXS,TYS,TZS)
      DO N = 1, NNN
         DO J = 1, NSYMP
            LH = NINT(RSYML(1,1,J)*IH(N) + RSYML(2,1,J)*IK(N) +
     +          RSYML(3,1,J)*IL(N))
            LK = NINT(RSYML(1,2,J)*IH(N) + RSYML(2,2,J)*IK(N) +
     +          RSYML(3,2,J)*IL(N))
            LL = NINT(RSYML(1,3,J)*IH(N) + RSYML(2,3,J)*IK(N) +
     +          RSYML(3,3,J)*IL(N))
            SHIFT = AMOD(IH(N)*RSYML(1,4,J) + IK(N)*RSYML(2,4,J) +
     +          IL(N)*RSYML(3,4,J)+10000.0,1.0)
            SHIFT = TWOPI*SHIFT
            TOP = 0.0
            BOT = 0.0
            DO NA = 1, NALL
               TEM = TWOPI*(REAL(LH)*XR(NA) + REAL(LK)*YR(NA)
     +             + REAL(LL)*ZR(NA)) + SHIFT
               TOP = TOP + OCT(NA)*REAL(IZT(NA))*SIN(TEM)
               BOT = BOT + OCT(NA)*REAL(IZT(NA))*COS(TEM)
            END DO
            NFT = NSYMP*(N-1) + J
            MH(NFT) = LH
            MK(NFT) = LK
            ML(NFT) = LL
            TM(NFT) = EV(N)*SQRT(TOP*TOP + BOT*BOT)
            PM(NFT) = ATAN2(TOP,BOT)
         END DO
      END DO
      NSALL = 0
      DO K = 0, MAXLT
         DO J = -MAXKT, MAXKT
            DO I = -MAXHT, MAXHT
               APAR(I,J,K) = 0.0
               BPAR(I,J,K) = 0.0
            END DO
         END DO
      END DO
      DO N = 1, NNN
         DO I = 1, NSYMP
            NFTI = NSYMP*(N-1) + I
            LHI = MH(NFTI)
            LKI = MK(NFTI)
            LLI = ML(NFTI)
            DO J = 1, NSYMP
               IF (J.NE.I) THEN
                  NFTJ = NSYMP*(N-1) + J
                  LHJ = MH(NFTJ)
                  LKJ = MK(NFTJ)
                  LLJ = ML(NFTJ)
                  LH = LHI - LHJ
                  LK = LKI - LKJ
                  LL = LLI - LLJ
                  NSALL = NSALL + 1
                  SJ = 1.0
                  IF (LL.LT.0) THEN
                     SJ = -1.0
                     LH = -LH
                     LK = -LK
                     LL = -LL
                  END IF
                  PHI = SJ*(PM(NFTI) - PM(NFTJ))
                  APAR(LH,LK,LL) = APAR(LH,LK,LL) +
     +                TM(NFTI)*TM(NFTJ)*COS(PHI)
                  BPAR(LH,LK,LL) = BPAR(LH,LK,LL) -
     +                TM(NFTI)*TM(NFTJ)*SIN(PHI)
               END IF
            END DO
         END DO
      END DO
      NFALL = 0
      DO LL = 0, MAXLT
         DO LK = -MAXKT, MAXKT
            DO LH = -MAXHT, MAXHT
               TEM = APAR(LH,LK,LL)*APAR(LH,LK,LL) +
     +             BPAR(LH,LK,LL)*BPAR(LH,LK,LL)
               IF (TEM.GT.0.0001) THEN
                  NFALL = NFALL + 1
                  MH(NFALL) = LH
                  MK(NFALL) = LK
                  ML(NFALL) = LL
                  TM(NFALL) = SQRT(TEM)
                  PM(NFALL) =
     +                ATAN2(BPAR(LH,LK,LL),APAR(LH,LK,LL))*RTOD
               END IF
            END DO
         END DO
      END DO
C     Calculate a map.
      CALL FFTFT1(NFALL,MH,MK,ML,TM,PM,LBUFT,BUFT,NXT,NYT,NZT
     +     ,TMAP,1,RSYML,IPAT,6,SCF,0.0)
C     Minimum, maximum and average value of map.
      CALL AVEMAP(NXT,NYT,NZT,TMAP,ROMIN,ROMAX,AVE,SIGRO,RO3)
C     find peaks.
      DO MZ = 0, NTZ
         TZ = REAL(MZ)/REAL(NZT)
         LZ = MZ + 1
         IF (LZ.GT.NZT) LZ = LZ - NZT
         DO MY = 0, NTY
            TY = REAL(MY)/REAL(NYT)
            LY = MY + 1
            IF (LY.GT.NYT) LY = LY - NYT
            DO MX = 0, NTX
               TX = REAL(MX)/REAL(NXT)
               LX = MX + 1
               IF (LX.GT.NXT) LX = LX - NXT
               NSET = NSET + 1
               CCM = TMAP(LX,LY,LZ)/ROMAX
               IF (CCM.GT.CCMIN) THEN
                  NSAV = NSAV + 1
                  NSEU(NSAV) = NSET
                  TRX(NSAV) = TX
                  TRY(NSAV) = TY
                  TRZ(NSAV) = TZ
                  TRCC(NSAV) = CCM
                  IF (NSAV.EQ.NCCS) THEN
                     CALL SORTCC(TRCC,TRX,TRY,TRZ,NSEU,NSAV)
                     CCMIN = TRCC(NCC)
                     NSAV = NCC
                  END IF
               END IF
            END DO
         END DO
      END DO
C     Sort on CC.
      CALL SORTCC(TRCC,TRX,TRY,TRZ,NSEU,NSAV)
      IF (NSAV.GT.NCC) NSAV = NCC
      NREJ = 0
      DO N = 1, NSAV-1
         IF (NFRA.EQ.1) THEN
            CALL DISTA(DIS,TRX(N),TRY(N),TRZ(N)
     +          ,0.0,0.0,0.0,A,B,C,CSA,CSB,CSC)
            IF (DIS.LT.1.5) THEN
               TRCC(N) = -ABS(TRCC(N))
               NREJ = NREJ + 1
               GOTO 200
            END IF
         END IF
         IF (TRCC(N).GT.0.0) THEN
            DO NT = N+1, NSAV
               IF (TRCC(NT).GT.0.0) THEN
                  IF (NFRA.EQ.1) THEN
                     CALL DISTA(DIS,TRX(N),TRY(N),TRZ(N)
     +                   ,TRX(NT),TRY(NT),TRZ(NT),A,B,C,CSA,CSB,CSC)
                  ELSE
                     CALL DISTAN(DIS,TRX(N),TRY(N),TRZ(N)
     +                   ,TRX(NT),TRY(NT),TRZ(NT),A,B,C,CSA,CSB,CSC)
                  END IF
                  IF (DIS.LT.0.5) THEN
                     TRCC(NT) = -ABS(TRCC(NT))
                     NREJ = NREJ + 1
                  END IF
               END IF
            END DO
         END IF
 200     CONTINUE
      END DO
C     Sort on CC.
      CALL SORTCC(TRCC,TRX,TRY,TRZ,NSEU,NSAV)
      NSAV = NSAV - NREJ
      IF (NFRA.EQ.1) THEN
C     Remove central ones.
         NREJ = 0
         DO N = 1, NSAV-1
            IF (TRCC(N).GT.0.0) THEN
               DO NT = N+1, NSAV
                  IF (TRCC(NT).GT.0.0) THEN
                     TEM = ABS(TRCC(N)-TRCC(NT))
                     IF (TEM.LT.0.0001) THEN
                        TRCC(NT) = -ABS(TRCC(NT))
                        NREJ = NREJ + 1
                     END IF
                  END IF
               END DO
            END IF
         END DO
C     Sort on CC.
         CALL SORTCC(TRCC,TRX,TRY,TRZ,NSEU,NSAV)
         NSAV = NSAV - NREJ
         DO N = 1, NSAV
            EU1(N) = EU1S
            EU2(N) = EU2S
            EU3(N) = EU3S
            TR1(N) = TRX(N)
            TR2(N) = TRY(N)
            TR3(N) = TRZ(N)
            NSEU(N) = N
C     Calculate SF.
            CALL CALFC(NALL,OCT,IZT,TR1(N),TR2(N),TR3(N)
     +          ,NREF,MKANG,IH,IK,IL,FC)
C     Scale FC.
            CALL FASTSC(NREF,MKANG,EV,FC,SS4V,SS4HO,20)
C     Free R-Factor as figure of merit.
            CALL FASTRF(NREF,MKANG,EV,FC,CCM)
            TRCC(N) = CCM
         END DO
C     Sort all sets.
         CALL SORTCC(TRCC,TR1,TR2,TR3,NSEU,NSAV)
         IF (MAXSET.GT.NSAV) MAXSET = NSAV
         NLPT = 200
         IF (NLPT.LT.MAXSET) NLPT = MAXSET
         IF (NLPT.GT.NSAV) NLPT = NSAV
         WRITE(6,FMT='(/,A,I10,//17X,A,A)')
     +       ' The total number of sets = ',NSAV,
     +       'ALPHA     BETA    GAMMA       TX       TY       TZ',
     +       '        CC    NREF    NSET   ORDER'
         DO N = 1, NLPT
            WRITE(6,FMT='(A,A,3F9.2,3F9.5,F10.5,3I8)') ' INITIAL '
     +          ,MTRA,EU1(N),EU2(N),EU3(N),TR1(N),TR2(N),TR3(N)
     +          ,TRCC(N),NREF,N,NSEU(N)
         END DO
      END IF
      RETURN
      END
C     --------------------------------------------------------
C     Translational function.
      SUBROUTINE TRANCC(NSAV,NCSAV,NSTOT,NSR,EUS1,EUS2,EUS3
     +     ,IH,IK,IL,MKANG,SS4V,EV,FC,APAR,BPAR)
C
      INTEGER NAMAX
      PARAMETER (NAMAX=30000)
C     COMMON variables.
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     Eulerian angles: EU1=ALPHA, EU2=BETA, EU3=GAMMA.
      COMMON/EULER/EU1M,EU2M,EU3M,
     +     EU1(5000),EU2(5000),EU3(5000),EUCC(5000),NSEU(5000)
     +     ,TR1(5000),TR2(5000),TR3(5000),TRCS(5000),NSTR(5000)
C     Translation function.
      COMMON/TRXYZ/MAXHT,MAXKT,MAXLT,NXT,NYT,NZT
     +     ,TRX(5000),TRY(5000),TRZ(5000),TRCC(5000)
      COMMON/XYZIN/NUMA,OC(NAMAX),IZ(NAMAX),XINF(NAMAX),YINF(NAMAX)
     +     ,ZINF(NAMAX),XINO(NAMAX),YINO(NAMAX),ZINO(NAMAX),BFAC(NAMAX)
C
      INTEGER IH(NREF),IK(NREF),IL(NREF),MKANG(NREF)
      REAL APAR(-MAXH:MAXH,-MAXK:MAXK,0:MAXL),
     +     BPAR(-MAXH:MAXH,-MAXK:MAXK,0:MAXL),
     +     SS4V(NREF),EV(NREF),FC(NREF)
      REAL EUS1(NSROT),EUS2(NSROT),EUS3(NSROT)
C     Local.
      CHARACTER*4 MTRA
      INTEGER NSET,NSAV,NCC,NLPT,NUMTRA,IZT(NAMAX)
      REAL CCM
      REAL XR(NAMAX),YR(NAMAX),ZR(NAMAX),XN(NAMAX),YN(NAMAX),ZN(NAMAX)
     +     ,OCT(NAMAX)
C
      DATA MTRA/'TRAN'/
      IF (NSYMP.EQ.1) RETURN
      IMSG = 6
      NSS4 = 20
      TXS = 0.0
      TYS = 0.0
      TZS = 0.0
      NCC = 1000
C     Pick up atoms.
      NALL = 0
      NEND = NSTA + NFRA - 1
      IF (NEND.GT.NUMA) NEND = NUMA
      DO NA = NSTA, NEND
         NALL = NALL + 1
         OCT(NALL) = OC(NA)
         IZT(NALL) = IZ(NA)
         XN(NALL) = XINO(NA)
         YN(NALL) = YINO(NA)
         ZN(NALL) = ZINO(NA)
      END DO
C     Rotate XYZ into real cell.
      NSET = 0
      EU1S = EUS1(NSR)
      EU2S = EUS2(NSR)
      EU3S = EUS3(NSR)
C     New XYZ after rotated.
      CALL ROTXYZ(NALL,XN,YN,ZN,XR,YR,ZR,EU1S,EU2S,EU3S
     +     ,TXS,TYS,TZS)
C     Calculate SF for hemisphere.
      CALL EXTHEMI(NALL,OCT,IZT,XR,YR,ZR,NREF,IH,IK,IL,MKANG
     +     ,APAR,BPAR)
C     Add phase shift by translation.
      DO N = 1, NSAV
         TX = TRX(N)
         TY = TRY(N)
         TZ = TRZ(N)
         NSEU(N) = N
         NSET = NSET + 1
         CALL TRAFC(NREF,IH,IK,IL,MKANG,FC,APAR,BPAR
     +       ,TX,TY,TZ,NUMTRA)
C     Scale FC.
         CALL FASTSC(NREF,MKANG,EV,FC,SS4V,SS4HO,NSS4)
C     Free R-Factor as figure of merit.
         CALL FASTRF(NREF,MKANG,EV,FC,CCM)
         TRCC(N) = CCM
      END DO
C
      DO N = 1, NSAV
         NCSAV = NCSAV + 1
         NSTOT = NSTOT + 1
         EU1(NCSAV) = EU1S
         EU2(NCSAV) = EU2S
         EU3(NCSAV) = EU3S
         TR1(NCSAV) = TRX(N)
         TR2(NCSAV) = TRY(N)
         TR3(NCSAV) = TRZ(N)
         TRCS(NCSAV) = TRCC(N)
         NSTR(NCSAV) = NSTOT
      END DO
C     Sort all sets.
      CALL SORTC6(TRCS,EU1,EU2,EU3,TR1,TR2,TR3,NSTR,NCSAV)
      IF (NCSAV.GT.NCC) NCSAV = NCC
      IF (NSR.LT.NSROT) RETURN
      IF (MAXSET.GT.NSAV) MAXSET = NCSAV
      NLPT = 200
      IF (NLPT.LT.MAXSET) NLPT = MAXSET
      IF (NLPT.GT.NCSAV) NLPT = NCSAV
      WRITE(6,FMT='(/,A,I10,//17X,A,A)')
     +     ' The total number of sets = ',NCSAV,
     +     'ALPHA     BETA    GAMMA      TX       TY       TZ',
     +     '       CC    NREF    NSET   ORDER'
      DO N = 1, NLPT
         WRITE(6,FMT='(A,A,3F9.2,3F9.5,F10.5,3I8)') ' INITIAL '
     +       ,MTRA,EU1(N),EU2(N),EU3(N),TR1(N),TR2(N),TR3(N)
     +       ,TRCS(N),NREF,N,NSTR(N)
      END DO
      RETURN
      END
C     --------------------------------------------------------
C     Random Translational function.
      SUBROUTINE RANTRA(IH,IK,IL,MKANG,SS4V,EV,FC,APAR,BPAR)
C
      INTEGER NAMAX
      PARAMETER (NAMAX=30000)
C     COMMON variables.
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     Eulerian angles: EU1=ALPHA, EU2=BETA, EU3=GAMMA.
      COMMON/EULER/EU1M,EU2M,EU3M,
     +     EU1(5000),EU2(5000),EU3(5000),EUCC(5000),NSEU(5000)
     +     ,TR1(5000),TR2(5000),TR3(5000),TRCS(5000),NSTR(5000)
C     Translation function.
      COMMON/TRXYZ/MAXHT,MAXKT,MAXLT,NXT,NYT,NZT
     +     ,TRX(5000),TRY(5000),TRZ(5000),TRCC(5000)
      COMMON/XYZIN/NUMA,OC(NAMAX),IZ(NAMAX),XINF(NAMAX),YINF(NAMAX)
     +     ,ZINF(NAMAX),XINO(NAMAX),YINO(NAMAX),ZINO(NAMAX),BFAC(NAMAX)
C
      INTEGER IH(NREF),IK(NREF),IL(NREF),MKANG(NREF)
      REAL APAR(-MAXH:MAXH,-MAXK:MAXK,0:MAXL),
     +     BPAR(-MAXH:MAXH,-MAXK:MAXK,0:MAXL),
     +     SS4V(NREF),EV(NREF),FC(NREF)
C     Local.
      CHARACTER*4 MRTF
      INTEGER NSET,NSAV,NCC,NLPT,NUMTRA,IZT(NAMAX)
      REAL CCM,CCMAX
      REAL XR(NAMAX),YR(NAMAX),ZR(NAMAX),XN(NAMAX),YN(NAMAX),ZN(NAMAX)
     +     ,OCT(NAMAX)
C
      DATA MRTF/'RTRA'/
      IF (NSYMP.EQ.1) RETURN
      IMSG = 5
      IR3 = ISEED
      NSS4 = 20
      TXS = 0.0
      TYS = 0.0
      TZS = 0.0
      NSAV = 0
      NCC = 4000
C     Pick up atoms.
      NALL = 0
      NEND = NSTA + NFRA - 1
      IF (NEND.GT.NUMA) NEND = NUMA
      DO NA = NSTA, NEND
         NALL = NALL + 1
         OCT(NALL) = OC(NA)
         IZT(NALL) = IZ(NA)
         XN(NALL) = XINO(NA)
         YN(NALL) = YINO(NA)
         ZN(NALL) = ZINO(NA)
      END DO
C     Rotate XYZ into real cell.
      CCMAX = -1.0
      NSET = 0
      EU1S = EU1(NSROT)
      EU2S = EU2(NSROT)
      EU3S = EU3(NSROT)
C     New XYZ after rotated.
      CALL ROTXYZ(NALL,XN,YN,ZN,XR,YR,ZR,EU1S,EU2S,EU3S
     +     ,TXS,TYS,TZS)
C     Use 1/MODTth reflections.
      IF (MODT.GT.1) THEN
         DO NF = 1, NREF
            IF (MOD(NF,MODT).NE.0) MKANG(NF) = -IABS(MKANG(NF))
         END DO
      END IF
C     Calculate SF for hemisphere for 1/MODTth E's.
      CALL EXTHEMI(NALL,OCT,IZT,XR,YR,ZR,NREF,IH,IK,IL,MKANG
     +     ,APAR,BPAR)
C     Add phase shift by translation.
      DO NSET = 1, NRTF
         TX = TXM*RANG2(IR3)
         TY = TYM*RANG2(IR3)
         TZ = TZM*RANG2(IR3)
         CALL TRAFC(NREF,IH,IK,IL,MKANG,FC,APAR,BPAR
     +       ,TX,TY,TZ,NUMTRA)
C     Scale FC.
         CALL FASTSC(NREF,MKANG,EV,FC,SS4V,SS4HO,NSS4)
C     Free R-Factor as figure of merit.
         CALL FASTRF(NREF,MKANG,EV,FC,CCM)
         IF (CCM.GT.CCMAX) THEN
            NSAV = NSAV + 1
            NSEU(NSAV) = NSET
            TRX(NSAV) = TX
            TRY(NSAV) = TY
            TRZ(NSAV) = TZ
            TRCC(NSAV) = CCM
            IF (NSAV.EQ.5000) THEN
               CALL SORTCC(TRCC,TRX,TRY,TRZ,NSEU,NSAV)
               CCMAX = TRCC(NCC)
               NSAV = NCC
            END IF
         END IF
      END DO
C     Sort on CC.
      CALL SORTCC(TRCC,TRX,TRY,TRZ,NSEU,NSAV)
      IF (NSAV.GT.NCC) NSAV = NCC
C     Use all reflections (MKANG>0).
      IF (MODT.GT.1) THEN
         DO NF = 1, NREF
            MKANG(NF) = IABS(MKANG(NF))
         END DO
C     Calculate SF for hemisphere for all E's.
         CALL EXTHEMI(NALL,OCT,IZT,XR,YR,ZR,NREF,IH,IK,IL,MKANG
     +       ,APAR,BPAR)
C     Add phase shift by translation.
         DO NS = 1, NSAV
            TX = TRX(NS)
            TY = TRY(NS)
            TZ = TRZ(NS)
            CALL TRAFC(NREF,IH,IK,IL,MKANG,FC,APAR,BPAR,TX,TY,TZ
     +          ,NUMTRA)
C     Scale FC.
            CALL FASTSC(NREF,MKANG,EV,FC,SS4V,SS4HO,NSS4)
C     Free R-Factor as figure of merit.
            CALL FASTRF(NREF,MKANG,EV,FC,CCM)
            TRCC(NS) = CCM
            NSEU(NS) = NS
         END DO
C     Sort on CC.
         CALL SORTCC(TRCC,TRX,TRY,TRZ,NSEU,NSAV)
      END IF
C
      IF (MAXSET.GT.NSAV) MAXSET = NSAV
      NLPT = 200
      IF (NLPT.LT.MAXSET) NLPT = MAXSET
      IF (NLPT.GT.NSAV) NLPT = NSAV
      WRITE(6,FMT='(/,A,I10,//17X,A,A)')
     +     ' The total number of sets = ',NRTF,
     +     'ALPHA     BETA    GAMMA      TX       TY       TZ',
     +     '       CC    NREF    NSET   ORDER'
      DO N = 1, NLPT
         EU1(N) = EU1S
         EU2(N) = EU2S
         EU3(N) = EU3S
         TR1(N) = TRX(N)
         TR2(N) = TRY(N)
         TR3(N) = TRZ(N)
         WRITE(6,FMT='(A,A,3F9.2,3F9.5,F10.5,3I8)') ' INITIAL '
     +       ,MRTF,EU1(N),EU2(N),EU3(N),TR1(N),TR2(N),TR3(N)
     +       ,TRCC(N),NUMTRA,N,NSEU(N)
      END DO
      RETURN
      END
C     ------------------------------------------------------------
C     Cal. SF for hemisphere.
      SUBROUTINE EXTHEMI(NALL,OCT,IZT,XR,YR,ZR,NREF,IH,IK,IL,MKANG
     +     ,APAR,BPAR)
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
C
      INTEGER NALL,MAXH,MAXK,MAXL
      INTEGER IH(NREF),IK(NREF),IL(NREF),MKANG(NREF),IZT(NALL)
      REAL XR(NALL),YR(NALL),ZR(NALL),OCT(NALL)
      REAL APAR(-MAXH:MAXH,-MAXK:MAXK,0:MAXL),
     +     BPAR(-MAXH:MAXH,-MAXK:MAXK,0:MAXL)
C     Local.
      INTEGER LH,LK,LL
C
      DO NF = 1, NREF
         IF (MKANG(NF).GT.0) THEN
C     Symmetry.
            DO J = 1, NSYMP
               LH = NINT(RSYML(1,1,J))*IH(NF) +
     +             NINT(RSYML(2,1,J))*IK(NF) +
     +             NINT(RSYML(3,1,J))*IL(NF)
               LK = NINT(RSYML(1,2,J))*IH(NF) +
     +             NINT(RSYML(2,2,J))*IK(NF) +
     +             NINT(RSYML(3,2,J))*IL(NF)
               LL = NINT(RSYML(1,3,J))*IH(NF) +
     +             NINT(RSYML(2,3,J))*IK(NF) +
     +             NINT(RSYML(3,3,J))*IL(NF)
               ISN = 1
               IF (LL.LT.0) THEN
                  ISN = -1
                  LH = -LH
                  LK = -LK
                  LL = -LL
               END IF
               TOP = 0.0
               BOT = 0.0
C     Cal SF.
               DO N = 1, NALL
                  TEM = REAL(ISN*LH)*XR(N) + REAL(ISN*LK)*YR(N)
     +                + REAL(ISN*LL)*ZR(N)
                  TOP = TOP + OCT(N)*REAL(IZT(N))*SIN(TWOPI*TEM)
                  BOT = BOT + OCT(N)*REAL(IZT(N))*COS(TWOPI*TEM)
               END DO
               APAR(LH,LK,LL) = SQRT(TOP*TOP + BOT*BOT)
               BPAR(LH,LK,LL) = 0.0
               IF (APAR(LH,LK,LL).GT.0.0)
     +             BPAR(LH,LK,LL) = ATAN2(TOP,BOT)
            END DO
         END IF
      END DO
      RETURN
      END
C     -------------------------------------------------------
C     Translational function by TX,TY,TZ.
      SUBROUTINE TRAFC(NREF,IH,IK,IL,MKANG,FC,APAR,BPAR
     +     ,TX,TY,TZ,NUMB)
C     COMMON variables.
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
C
      INTEGER NUMB
      INTEGER IH(NREF),IK(NREF),IL(NREF),MKANG(NREF)
      REAL TX,TY,TZ
      REAL FC(NREF)
      REAL APAR(-MAXH:MAXH,-MAXK:MAXK,0:MAXL),
     +     BPAR(-MAXH:MAXH,-MAXK:MAXK,0:MAXL)
C     Local.
      INTEGER LH,LK,LL
      NUMB = 0
      DO NF = 1, NREF
         IF (MKANG(NF).GT.0) THEN
            TOP = 0.0
            BOT = 0.0
            NUMB = NUMB + 1
            DO J = 1, NSYMP
               LH = NINT(RSYML(1,1,J))*IH(NF) +
     +             NINT(RSYML(2,1,J))*IK(NF) +
     +             NINT(RSYML(3,1,J))*IL(NF)
               LK = NINT(RSYML(1,2,J))*IH(NF) +
     +             NINT(RSYML(2,2,J))*IK(NF) +
     +             NINT(RSYML(3,2,J))*IL(NF)
               LL = NINT(RSYML(1,3,J))*IH(NF) +
     +             NINT(RSYML(2,3,J))*IK(NF) +
     +             NINT(RSYML(3,3,J))*IL(NF)
               IF (LL.LT.0) THEN
                  LH = -LH
                  LK = -LK
                  LL = -LL
               END IF
               SFTX = RSYML(1,1,J)*TX + RSYML(1,2,J)*TY +
     +             RSYML(1,3,J)*TZ
               SFTY = RSYML(2,1,J)*TX + RSYML(2,2,J)*TY +
     +             RSYML(2,3,J)*TZ
               SFTZ = RSYML(3,1,J)*TX + RSYML(3,2,J)*TY +
     +             RSYML(3,3,J)*TZ
               SFTX = IH(NF)*(SFTX + RSYML(1,4,J))
               SFTY = IK(NF)*(SFTY + RSYML(2,4,J))
               SFTZ = IL(NF)*(SFTZ + RSYML(3,4,J))
               SHIFT = AMOD(SFTX + SFTY + SFTZ + 10000.0,1.0)
               PHI = BPAR(LH,LK,LL) + TWOPI*SHIFT
               TOP = TOP + APAR(LH,LK,LL)*SIN(PHI)
               BOT = BOT + APAR(LH,LK,LL)*COS(PHI)
            END DO
            FC(NF) = SQRT(TOP*TOP + BOT*BOT)
         END IF
      END DO
      RETURN
      END
C     --------------------------------------------------------
C     Main routine of phase refinement.
      SUBROUTINE PHASE (LBUF,IH,IK,IL,MKANG,ICODE,SS4V,FP,EV,PHIKNO,WTKNO
     +     ,FT,PHIFT,THETAE,PHIFP,WT,FC,PHIFC,GH,PHIGH,EX,FOM,SGMAA,WTFP
     +     ,VH,PHIVH,VK,PHIVK,VL,PHIVL,FCSAV,PHISAV,BUF,XMAP,YMAP
     +     ,ZMAP)
C     COMMON variables.
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,CCDDM2,NKEXT
C     Data.
      INTEGER LBUF
      INTEGER IH(NEXT),IK(NEXT),IL(NEXT),MKANG(NEXT),ICODE(NEXT)
      REAL SS4V(NEXT),FP(NEXT),EV(NEXT),PHIFP(NEXT),FOM(NEXT)
     +     ,SGMAA(NEXT),WTFP(NEXT),FC(NEXT),PHIFC(NEXT)
     +     ,PHIFT(NEXT),WT(NEXT),THETAE(NEXT),GH(NEXT),PHIGH(NEXT)
     +     ,VH(NEXT),PHIVH(NEXT),VK(NEXT),PHIVK(NEXT),VL(NEXT)
     +     ,PHIVL(NEXT),PHIKNO(NEXT),WTKNO(NEXT),FCSAV(NEXT)
     +     ,PHISAV(NEXT),EX(NEXT),FT(NEXT)
C     FFT variables.
      REAL BUF(LBUF),XMAP(NX,NY,NZ),YMAP(NX,NY,NZ),ZMAP(NX,NY,NZ)
C     Local variables.
      CHARACTER*4 MKNO,MFIN,MINI
      REAL CCSAV(0:5000),CCNCY(0:5000),CC(3),CCMAX,S1,S2,D,CCTRY
     +     ,ERSAV(0:5000),ERNCY(0:5000),CCSL(100)
      INTEGER NSS4,NSSAV,NTT,LPTSK,NCYCS,KDDM
      DATA MKNO/'KNOW'/,MFIN/' FIN'/,MINI/'INIT'/
      PARTRS = PARTR
C     Scale for FFT.
      NSS4 = 20
      NSET = 0
      NTRYS = 0
      LPTSK = 0
      NSWEK = NREF - NEWEK + 1
      NCENH = 0
C     Calculate S2.
      CALL CALS2(NEMAX,IH,IK,IL,EV,WTFP,PHIGH,
     +     LBUF,BUF,YMAP,S2)
      CCMAX = -1.0
C     Loop for a phase set.
      DO 800 NSET = 1, MAXSET
         NCYC = 0
         NTRYS = 0
         DO 30 N = 1, NEXT
            WT(N) = 0.0
            PHIFP(N) = 0.0
            WTFP(N) = 0.0
 30      CONTINUE
C     Known phases and weights.
         IF (NKNO.GT.0) THEN
            CCNCY(NCYC) = 0.0
            IF (NKNO.EQ.1) THEN
               NUMKNO = 0
               DO N = 1, NREF
                  NUMKNO = NUMKNO + 1
                  WT(N) = 1.0
                  PHIFP(N) = PHIKNO(N)
               END DO
            ELSE IF (NKNO.EQ.3) THEN
               NUMKNO = 0
               DO N = 1, NREF
                  WT(N) = 0.0
                  NUMKNO = NUMKNO + 1
                  IF (WTKNO(N).GT.0.0) WT(N) = WTKNO(N)
                  PHIFP(N) = PHIKNO(N)
               END DO
            ELSE IF (NKNO.GE.5) THEN
               NUMKNO = 0
               DO N = 1, NREF
                  NUMKNO = NUMKNO + 1
                  PHIFP(N) = PHIKNO(N)
               END DO
C     Scale FC.
               CALL SCNEW(NEXT,EX,FC,SS4V,SS4HO,20,ICODE)
C     Sigmaa weights and Sim weights.
               CALL WTSGA(NEXT,PARTR,SS4V,EX,FC,FOM,SGMAA,WT
     +             ,MKANG,SS4HO,ICODE)
C     Free R-Factor as figure of merit without print out.
               CALL RFNEW(NEXT,EX,FC,NSET,NTRYS,NCYC,CC,0,MKNO,ICODE)
               CCNCY(NCYC) = CC(2)
            END IF
C     Phase error.
            IF (MKER.EQ.1) THEN
               CALL ERRNEW(NEXT,IH,IK,IL,NOX,NOY,NOZ,
     +             PHIFT,PHIFP,WT,10,2,NSYMP,NSET,NTRYS,NCYC,MKNO
     +             ,ICODE,ERROR)
            END IF
C     Phase error.
            IF (MKER.EQ.1) CALL FRENEW(NREF,ICODE,PHIFT,EX,FC,PHIFP,WT
     +          ,NSET,NTRYS,NCYC,MKNO)
C     Phase errors for xloggraph.
            IF (MKER.EQ.1) CALL PHIERR(NREF,NEMAX,EV,PHIFT,PHIFP,WT
     +          ,SS4V,SS4HO,NSS4,NSET,NCYC)
            IF (NCYC.LE.5000) THEN
               IF (MKER.EQ.1) ERNCY(NCYC) = ERROR
            END IF
C     Calculate S1 and D.
            CALL CALS1(NEMAX,IH,IK,IL,EV,PHIFP,WTFP,PHIGH,
     +          LBUF,BUF,YMAP,S1,S2,D,1,MKNO,NSET,NTRYS,NCYC)
         ELSE
C     Obtain initial phases.
            CALL INIPHI(IH,IK,IL,MKANG,SS4V,PHIFP,WT,EX,FOM,SGMAA
     +          ,FC,PHIFC,PHIFT,NSET,NTRYS,NCYC,CC,ERROR,ICODE,
     +          NX,NY,NZ,YMAP,LBUF,BUF,FT)
            IF (NCYC.LE.5000) THEN
               CCNCY(NCYC) = CC(2)
               IF (MKER.EQ.1) ERNCY(NCYC) = ERROR
            END IF
C     Calculate S1 and D.
            CALL CALS1(NEMAX,IH,IK,IL,EV,PHIFP,WTFP,PHIGH,
     +          LBUF,BUF,YMAP,S1,S2,D,1,MINI,NSET,NTRYS,NCYC)
C     Patterson superposition to give a set of initial phases.
            IF (NSUP.EQ.1) THEN
               NCYC = NCYC + 1
               CALL NOENVL(NX,NY,NZ,ZMAP)
               CALL SUPSUM(IH,IK,IL,SS4V,EV,EX,PHIFP,WT,WTFP
     +          ,FC,PHIFC,ICODE,MKANG,PHIFT,PHIGH,LBUF,BUF,YMAP,ZMAP
     +          ,NSET,NTRYS,NCYC,CC,S1,S2,D,NCENH,ERROR)
               IF (NCYC.LE.5000) THEN
                  CCNCY(NCYC) = CC(2)
                  IF (MKER.EQ.1) ERNCY(NCYC) = ERROR
               END IF
            END IF
         END IF
         NTRYS = 0
C     New try.
 100     IF (NTRY.EQ.0) GOTO 360
         NTRYS = NTRYS + 1
         NTT = NSER(NTRYS) + NDDM(NTRYS)
         IF (NTT.EQ.0) GOTO 360
C     Real Space Sayre Equation Refinement.
         IF (NSER(NTRYS).GT.0) THEN
            NCS = NSER(NTRYS)
            DO 200 NCT = 1, NCS
               NCYC = NCYC + 1
               LPTSK = 0
               IF (NCT.EQ.1.OR.NCT.EQ.NCS) LPTSK = 1
               CALL RSSER(IH,IK,IL,SS4V,EV,PHIFP,WTFP,FC,PHIFC
     +             ,MKANG,PHIFT,WT,THETAE,GH,PHIGH,VH,PHIVH,VK,PHIVK
     +             ,VL,PHIVL,LBUF,BUF,XMAP,YMAP,NCYC,ERROR
     +             ,SUMES,NSET,NTRYS,LPTSK,S1,S2,D,CC,ICODE,EX)
               IF (NCYC.LE.5000) THEN
                  CCNCY(NCYC) = CC(2)
                  IF (MKER.EQ.1) ERNCY(NCYC) = ERROR
               END IF
               IF (D.GT.0.6) GOTO 250
C     Next cycle.
 200       CONTINUE
         END IF
 250     CONTINUE
C     DDM phase refinement.
         IF (NDDM(NTRYS).GT.0) THEN
            NCL = NDDM(NTRYS)
            NCENH = 0
            CCTRY = -1.0
            DO 300 NCT = 1, NCL
               NCENH = 0
               IF (NENH(NTRYS).EQ.1.AND.NCT.EQ.1) NCENH = 1
               IF (NSER(NTRYS).NE.0) NCENH = 0
               KDDM = NDDMK(NTRYS)
               IF (KDDM.LT.0) THEN
                  IF (CCNCY(NCYC).GT.CCDDM2) THEN
                     KDDM = 2
                  ELSE IF (CCNCY(NCYC).GT.CCDDM1) THEN
                     KDDM = 1
                  ELSE
                     KDDM = 0
                  END IF
               END IF
               NCYC = NCYC + 1
               LPTSK = 0
               IF (NCT.EQ.1.OR.NCT.EQ.NCL) LPTSK = 1
C     Make envelop.
               IF (NCENH.EQ.1) THEN
C     Make envelop ZMAP from GS.
                  CALL RENVLP(IH,IK,IL,SS4V,PHIFP,WT,WTFP
     +                ,LBUF,BUF,ZMAP,NCYC,FP)
               ELSE
                  CALL NOENVL(NX,NY,NZ,ZMAP)
               END IF
C     DDM.
               CALL DDMM(IH,IK,IL,SS4V,EV,PHIFP,WT,WTFP,FC,PHIFC,ICODE
     +             ,MKANG,PHIFT,EX,PHIGH,FOM,SGMAA,LBUF,BUF,YMAP
     +             ,LPTSK,NSET,NCYC,CC,ERROR,S1,S2,D,NNEW,SNEW
     +             ,NCT,NTRYS,NCENH,KDDM,CCTRY,XMAP,ZMAP)
               IF (NCYC.LE.5000) THEN
                  CCNCY(NCYC) = CC(2)
                  IF (MKER.EQ.1) ERNCY(NCYC) = ERROR
               END IF
               IF (NCT.LT.5) THEN
                  CCTRY = CC(2)
               ELSE
                  IF (NTRYS.LT.NTRY.AND.CC(2).LT.CCTRY) THEN
                     GOTO 350
                  ELSE
                     CCTRY = CC(2)
                  END IF
               END IF
               IF (SNEW.LT.PFIN) GOTO 350
 300       CONTINUE
         END IF
 350     IF (NTRYS.LT.NTRY.AND.CC(2).LT.CFIN) GOTO 100
C     Save the best set of phases according the CC(2).
 360     IF (CC(2).GT.CCMAX) THEN
            CCMAX = CC(2)
            NSSAV = NSET
            NCYCS = NCYC
            DO N = 1, NEXT
               FCSAV(N) = FC(N)
               PHISAV(N) = PHIFC(N)
            END DO
            DO N = 0, NCYC
               IF (NCYC.LE.5000) THEN
                  CCSAV(N) = CCNCY(N)
                  IF (MKER.EQ.1) ERSAV(N) = ERNCY(N)
               END IF
            END DO
         END IF
C     End if CC(2).GT.CFIN.
         IF (CC(2).GE.CFIN) GOTO 810
C     Next set of starting phases.
 800  CONTINUE
 810  NSET = NSSAV
      NCYC = NCYCS
C     Restore FC and PHIFC.
      IF (NCYC.GE.1) THEN
         DO N = 1, NEXT
            FC(N) = FCSAV(N)
            PHIFC(N) = PHISAV(N)
         END DO
      END IF
      WRITE(6,FMT='(/,A,I10)')
     +     ' Output phase set ',NSSAV
C     Scale FC.
      CALL SCNEW(NEXT,EX,FC,SS4V,SS4HO,20,ICODE)
C     Sigmaa weights and Sim weights.
      PARTR = 1.0
      CALL WTSGA(NEXT,PARTR,SS4V,EX,FC,FOM,SGMAA,WT
     +     ,MKANG,SS4HO,ICODE)
      CALL PHINEW(NEXT,PHIFP,FC,PHIFC,
     +     NNEW,SNEW,MKANG,ICODE)
      IF (KDDM.EQ.2) THEN
C     Phase error with origin shift.
         IF (MKER.EQ.1) CALL ERRNEW(NEXT,IH,IK,IL,NOX,NOY,NOZ,
     +       PHIFT,PHIFP,FOM,10,2,NSYMP,NSET,NTRYS,NCYC,MFIN
     +       ,ICODE,ERROR)
C     Phase error.
         IF (MKER.EQ.1) CALL FRENEW(NEXT,ICODE,PHIFT,EX,FC,PHIFP,FOM
     +       ,NSET,NTRYS,NCYC,MFIN)
      ELSE
C     Phase error with origin shift.
         IF (MKER.EQ.1) CALL ERRNEW(NEXT,IH,IK,IL,NOX,NOY,NOZ,
     +       PHIFT,PHIFP,WT,10,2,NSYMP,NSET,NTRYS,NCYC,MFIN
     +       ,ICODE,ERROR)
C     Phase error.
         IF (MKER.EQ.1) CALL FRENEW(NEXT,ICODE,PHIFT,EX,FC,PHIFP,WT
     +       ,NSET,NTRYS,NCYC,MFIN)
      END IF
C     RF, ERROR vs the number of cycles.
      CALL RFCYCLE(NSET,NCYC,MKER,CCSAV,ERSAV)
C     R-factor for xloggraph.
      NCSL = 0
      CALL RFACTOR(NEXT,NREF,EX,FC,SS4V,SS4HO,NSS4,NSET,NCYC,NCSL,CCSL)
C     Phase errors for xloggraph.
      IF (MKER.EQ.1) THEN
         IF (KDDM.EQ.2) THEN
            CALL PHIERR(NEXT,NREF,EX,PHIFT,PHIFP,FOM,SS4V
     +          ,SS4HO,NSS4,NSET,NCYC)
         ELSE
            CALL PHIERR(NEXT,NREF,EX,PHIFT,PHIFP,WT,SS4V
     +          ,SS4HO,NSS4,NSET,NCYC)
         END IF
      END IF
C     Output Eo plus Ex.
      DO N = 1, NEXT
         IF (KDDM.EQ.2) THEN
            FC(N) = 2.0*FOM(N)*EX(N) - SGMAA(N)*FC(N)
            WT(N) = FOM(N)
         ELSE IF (KDDM.EQ.1) THEN
            FC(N) = WT(N)*(2.0*EX(N) - FC(N))
         ELSE
            FC(N) = WT(N)*EX(N)
         END IF
      END DO
      RETURN
      END
C     --------------------------------------------------------
C     Calculate real envelope from FC & PHIFC with GS.
      SUBROUTINE RENVLP(IH,IK,IL,SS4V,PHIFP,WT,WTFP
     +     ,LBUF,BUF,XMAP,NCYC,FP)
C     COMMON variables.
      COMMON/CELL/CELLP(6),VOLUME,NX,NY,NZ,MAXH,MAXK,MAXL,F000,E000
      COMMON/SYMMETRY/RSYML(4,4,192),NSYM,NSYMP,NSPGRP
      COMMON/KEYW/MAXSET,NTRY,NFRA,NSTA,IMSG,NEXT,RAD1,NDDMK(500)
     +     ,NREF,NEMAX,NEMIN,NEWEK,NSER(500),NDDM(500),NENH(500)
     +     ,ECUT,EMIN,EWEK,SS4LO,SS4HO,SS4LE,SS4HE,NRRT,INIT,ISEED,NROT
     +     ,NSUP,MODR,NPOS,CUTT,NATM,PFIN,SS4ROT,MODT,CFIN,STEP,GRID,
     +     SS4LT,SS4HT,ASX,ASY,ASZ,NKNO,NTRA,NSROT,TXM,TYM,TZM,NSOL,NRTF
     +     ,NOX,NOY,NOZ,MKER,PARTR,PARTRS,SOLV,CCDDM1,NKEXT
C     Data.
      INTEGER LBUF,NCYC,NPT
      INTEGER IH(NEXT),IK(NEXT),IL(NEXT)
      REAL SS4V(NEXT),PHIFP(NEXT)
     +     ,WT(NEXT),WTFP(NEXT),FP(NEXT)
C     FFT variables.
      REAL BUF(LBUF),XMAP(NX,NY,NZ)
C     Local.
      INTEGER IPAT,NHIS(-100:100)
      REAL SCF,CUTS,RHIS(-100:100)
C
      IPAT = 0
      SCF = 1.0/VOLUME
C     Cal. GS.
      CALL CALGS(NREF,RAD1,SS4V,WTFP)
C     Average Map.
      DO N = 1, NREF
         WTFP(N) = WTFP(N)*WT(N)*FP(N)
      END DO
C     Calculate average map.(XMAP)
      CALL FFTFT1(NREF,IH,IK,IL,WTFP,PHIFP,LBUF,BUF,NX,NY,NZ,XMAP
     +     ,NSYMP,RSYML,IPAT,6,SCF,0.0)
C     Minimum, maximum and average value of map.
      CALL AVEMAP(NX,NY,NZ,XMAP,ROMIN,ROMAX,AVE,SIGRO,RO3)
C     Histogram of grid points vs RO/SIGRO.
      CALL STAAVE(NX,NY,NZ,XMAP,SIGRO,NHIS,RHIS,CUTS,SOLV)
C     Obtain envelope in XMAP.
      CALL MODCUV(NX,NY,NZ,XMAP,NCYC,SIGRO,CUTS,NPT)
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C      CALCULATE  G(s) = FT[W(r)] = Y(x) - Z(x)                       C
C                    S = 2*SIN(THETA)/LAMDA                           C
C                    u = 2*PI*s                                       C
C                    R = RADIUS                                       C
C                    x = uR                                           C
C                 Y(x) = 3*(sin(x)-x*cos(x))/(x**3)                   C
C                 Z(x) = 3*(2x*sin(x)-(x*x-2)*cos(x)-2)/(x**4)        C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE CALGS(NNN,RAD1,SS4V,GS)
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
      INTEGER NNN
      REAL RAD1,GS(NNN),SS4V(NNN)
C     -------------------------------------------------------------
      T = TWOPI*RAD1
      DO 100 N = 1, NNN
         SL = SQRT(SS4V(N))
         X = T*SL
         X2 = X*X
         Y = 3.0*(SIN(X) - X*COS(X))/(X*X2)
         Z = 3.0*(2.0*X*SIN(X) - (X2 - 2.0)*COS(X) - 2.0)/(X2*X2)
         GS(N) = Y - Z
 100  CONTINUE
      RETURN
      END
C     -----------------------------------------------------------
      SUBROUTINE STAAVE(NX,NY,NZ,XMAP,SIGRO,NHIS,RHIS,CUTS,SOLV)
C
C     Histogram of average map.
C
      INTEGER NX,NY,NZ,NHIS(-100:100)
      REAL XMAP(NX,NY,NZ),RHIS(-100:100)
      REAL SIGRO
C     Local.
      INTEGER N,I,J,K,KS
      REAL DEV,TEM
C
      NPA = NX*NY*NZ
      NCUT = NINT(SOLV*REAL(NPA))
      DEV = 0.1
      DO N = -100, 100
         RHIS(N) = 0.1*REAL(N)
         NHIS(N) = 0
      END DO
C
      DO I = 1, NX
         DO J = 1, NY
            DO K = 1, NZ
C     Histogram RO/SIGRO.
               TEM = XMAP(I,J,K)/SIGRO
               KS = NINT(TEM/DEV)
               IF (KS.GT.100) KS = 100
               IF (KS.LT.-100) KS = -100
               NHIS(KS) = NHIS(KS) + 1
            END DO
         END DO
      END DO
      DO N = -99, 100
         NHIS(N) = NHIS(N) + NHIS(N-1)
         IF (NCUT.GE.NHIS(N-1).AND.NCUT.LT.NHIS(N)) THEN
            T1 = RHIS(N) - RHIS(N-1)
            T2 = REAL(NCUT-NHIS(N-1))/REAL(NHIS(N)-NHIS(N-1))
            CUTS = RHIS(N-1) + T1*T2
         END IF
      END DO
      WRITE(6,FMT='(A,2F10.4,I9)') ' Solvent input =',SOLV,CUTS,NCUT
      RETURN
      END
C     -----------------------------------------------------------
C     Obtain envelope in XMAP.
      SUBROUTINE MODCUV(NX,NY,NZ,XMAP,NCYC,SIGRO,CUTS,NPT)
C
      INTEGER NX,NY,NZ,NCYC,NPT
      REAL XMAP(NX,NY,NZ)
      REAL SIGRO,CUTS
C
      NPT = 0
      NPA = NX*NY*NZ
      DO 150 K = 1, NZ
         DO 145 J = 1, NY
            DO 140 I = 1, NX
               TEM = XMAP(I,J,K)/SIGRO
               IF (TEM.GT.CUTS) THEN
                  XMAP(I,J,K) = 1.0
                  NPT = NPT + 1
               ELSE
                  XMAP(I,J,K) = 0.0
               END IF
 140       CONTINUE
 145     CONTINUE
 150  CONTINUE
      RATIO = REAL(NPA-NPT)/REAL(NPA)
      RETURN
      END
C     --------------------------------------------------------
      SUBROUTINE NOENVL(NX,NY,NZ,ZMAP)
C
      INTEGER NX,NY,NZ
      REAL ZMAP(NX,NY,NZ)
C
      DO K = 1, NZ
         DO J = 1, NY
            DO I = 1, NX
               ZMAP(I,J,K) = 1.0
            END DO
         END DO
      END DO
      RETURN
      END
C     --------------------------------------------------------
      SUBROUTINE RFCYCLE(NSET,NCYC,MKER,CCSAV,ERSAV)
C
      INTEGER NSET,NCYC,MKER
      REAL CCSAV(0:5000),ERSAV(0:5000)
C
      IF (MKER.EQ.1) THEN
         WRITE(6,300)
 300     FORMAT(/,/,/,/,1X,44('*')
     +       ,/,' *** CC  &Phase Error vs number of cycles ***'
     +       ,/,1X,44('*'))
         WRITE(6,310) NSET
 310     FORMAT(//,2X,40('*'),/,
     +       ' $TABLE:CC & Phase Error vs number of cycles :'
     +       ,/,' $GRAPHS: CC & Phase Error vs CYCLES for set'
     +       ,I6,':N:2,3,4: $$')
         WRITE(6,320)
 320     FORMAT(' Bin',/
     +       ,' Cycle_number',/
     +       ,' CC_%',/
     +       ,' Phase_Error',/
     +       ,' $$',/
     +       ,' Bin Cycle_number   CC_% Phase_Error $$'
     +       ,/)
         DO N = 0, NCYC
            WRITE(6,FMT='(2I6,2F10.5)') N+1,N,100.0*CCSAV(N),ERSAV(N)
         END DO
      ELSE
         WRITE(6,400)
 400     FORMAT(/,/,/,/,1X,44('*')
     +       ,/,' *** Correlation Coefficient vs number of cycles ***'
     +       ,/,1X,44('*'))
         WRITE(6,410) NSET
 410     FORMAT(//,2X,40('*'),/,
     +       ' $TABLE:Correlation Coefficient vs number of cycles :'
     +       ,/,' $GRAPHS: CC vs CYCLES for set',I6,':N:2,3: $$')
         WRITE(6,420)
 420     FORMAT(' Bin',/
     +       ,' Cycle_number',/
     +       ,' CC',/
     +       ,' $$',/
     +       ,' Bin Cycle_number   CC  $$'
     +       ,/)
         DO N = 0, NCYC
            WRITE(6,FMT='(2I6,F10.5)') N+1,N,CCSAV(N)
         END DO
      END IF
      WRITE(6,FMT='(A)') ' $$'
      RETURN
      END
C     ----------------------------------------------------------------
      SUBROUTINE LOCVAR(NX,NY,NZ,CELLP,YMAP,XMAP,ZMAP)
C     Local average of the map.
      INTEGER NX,NY,NZ
      REAL CELLP(6)
      REAL YMAP(NX,NY,NZ),XMAP(NX,NY,NZ),ZMAP(NX,NY,NZ)
C
      COMMON/CONS/DTOR,RTOD,PI,TWOPI
C     Local.
      INTEGER MASK(-6:6,-6:6,-6:6)
C
      A = CELLP(1)
      B = CELLP(2)
      C = CELLP(3)
      CSA = COS(DTOR*CELLP(4))
      CSB = COS(DTOR*CELLP(5))
      CSC = COS(DTOR*CELLP(6))
      DA = 1.0/REAL(NX)
      DB = 1.0/REAL(NY)
      DC = 1.0/REAL(NZ)
      X1 = 0.0
      Y1 = 0.0
      Z1 = 0.0
      DO I = -6, 6
         DO J = -6, 6
            DO K = -6, 6
               MASK(I,J,K) = 0
               IF (I.EQ.0.AND.J.EQ.0.AND.K.EQ.0) THEN
                  MASK(I,J,K) = 1
               ELSE
                  X2 = REAL(I)*DA
                  Y2 = REAL(J)*DB
                  Z2 = REAL(K)*DC
                  CALL DISTAN(DIS,X1,Y1,Z1,X2,Y2,Z2,A,B,C,CSA,CSB,CSC)
                  IF (DIS.LT.2.00) MASK(I,J,K) = 1
               END IF
            END DO
         END DO
      END DO
C
      NPT = 0
      DO N1 = 1, NX
         DO N2 = 1, NY
            DO N3 = 1,NZ
               XMAP(N1,N2,N3) = 0.0
               ROAVEL = 0.0
               NAVE = 0
               ROMAXL = -100000.0
               ROSIGL = 0.0
               DO I = -6, 6
                  DO J = -6, 6
                     DO K = -6, 6
                        IF (MASK(I,J,K).EQ.1) THEN
                           N1T = MOD(N1+I+NX,NX)
                           IF (N1T.EQ.0) N1T = NX
                           N2T = MOD(N2+J+NY,NY)
                           IF (N2T.EQ.0) N2T = NY
                           N3T = MOD(N3+K+NZ,NZ)
                           IF (N3T.EQ.0) N3T = NZ
                           TEM = YMAP(N1T,N2T,N3T)
                           IF (TEM.GT.ROMAXL) ROMAXL = TEM
                           IF (TEM.GT.0.0) THEN
                              NAVE = NAVE + 1
                              ROAVEL = ROAVEL + TEM
                              ROSIGL = ROSIGL + TEM*TEM
                          END IF
                        END IF
                     END DO
                  END DO
               END DO
 100          CONTINUE
               IF (NAVE.GT.0) THEN
                  ROAVEL = ROAVEL/REAL(NAVE)
                  ROSIGL = ROSIGL/REAL(NAVE)
                  ROSIGL = SQRT(ROSIGL-ROAVEL*ROAVEL)
               END IF
               XMAP(N1,N2,N3) = ROAVEL
            END DO
         END DO
      END DO
C     Minimum, maximum and average value of map.
      CALL AVEMAP(NX,NY,NZ,XMAP,ROMIN,ROMAX,AVE,SIGRO,RO3)
      NPT = 0
      DO N1 = 1, NX
         DO N2 = 1, NY
            DO N3 = 1,NZ
               IF (XMAP(N1,N2,N3).LT.0.0) THEN
                  NPT = NPT + 1
                  XMAP(N1,N2,N3) = 0.0
               ELSE
                  XMAP(N1,N2,N3) = XMAP(N1,N2,N3)/ROMAX
               END IF
               ZMAP(N1,N2,N3) = ZMAP(N1,N2,N3)*XMAP(N1,N2,N3)
            END DO
         END DO
      END DO
      RETURN
      END
C     -------------------------------------------------------------
      SUBROUTINE WTSGA(NEXT,PARTR,SS4V,FP,FC,FOM,SGMAA,WT
     +     ,MKANG,SS4H,ICODE)
C     Sim weights.
      INTEGER NEXT,MKANG(NEXT),ICODE(NEXT)
      REAL SS4H,CCSL(100)
      REAL FP(NEXT),FC(NEXT),SS4V(NEXT)
     +     ,FOM(NEXT),SGMAA(NEXT),WT(NEXT)
C     Local.
      INTEGER KS
C     I1/I0 (BESSEL FUNCTIONS)
      VEC(U)=U*(U+0.4807)/((U+0.8636)*U+1.3943)
C     CC for sigmaa.
      NSS4 = 20
      NSSA = 100
      CALL RFCC(NEXT,ICODE,FP,FC,SS4V,SS4H,NSS4,CCSL)
C
      DEV = SS4H/REAL(NSS4)
      DO N = 1, NEXT
         WT(N) = 0.0
         X1 = FC(N)*FP(N)
         IF (ICODE(N).GT.3) X1 = X1*PARTR
         FOM(N) = 0.0
         SGMAA(N) = 0.0
         KS = INT(SS4V(N)/DEV) + 1
         IF (KS.GT.NSSA) KS = NSSA
         SGMA = 0.0
         IF (CCSL(KS).GT.0.0) SGMA = SQRT(CCSL(KS))
         IF (SGMA.GT.1.0) SGMA = 1.0
         SGMAA(N) = SGMA
         X2 = 2.0*SGMA*FC(N)*FP(N)/(1.0-SGMA*SGMA)
         IF (MKANG(N).GT.1) THEN
            WT(N) = TANH(0.5*X1)
            FOM(N) = TANH(0.5*X2)
         ELSE
            WT(N) = TANH(0.5*X1)
            FOM(N)  = VEC(X2)
         END IF
      END DO
      RETURN
      END
C     ---------------------------------------------------------
C     Calculate R-factor and Correlation Coefficient vs resolution.
      SUBROUTINE RFCC(NEXT,ICODE,FO,FC,SS4V,SS4H,NSS4,CCSL)
C
      PARAMETER (NSSA=100)
      INTEGER NEXT,NSS4,ICODE(NEXT)
      REAL SS4H,CCSL(NSSA)
      REAL FO(NEXT),FC(NEXT),SS4V(NEXT)
C     Local variables.
      INTEGER NDET,KS,NSUM(NSSA)
      REAL SUMFT,SUMFC,RFAC,SS4DEV,SCALE
     +     ,SUMFC2,SUMFT2,SUMFTC,CCALL,CCDIV
      REAL SFT(NSSA),SFC(NSSA),SK(NSSA),RF(NSSA),
     +     SFT2(NSSA),SFC2(NSSA),SFTC(NSSA)
C
      NDET = 0
      NSS = 0
C     Resolution range.
      SS4DEV = SS4H/REAL(NSS4)
      DO I = 1, NSSA
         SFT(I) = 0.0
         SFC(I) = 0.0
         SK(I) = 0.0
         RF(I) = 0.0
         NSUM(I) = 0
         SFT2(I) = 0.0
         SFC2(I) = 0.0
         SFTC(I) = 0.0
         CCSL(I) = 0.0
      END DO
      SUMFT = 0.0
      SUMFC = 0.0
      RFAC = 0.0
      SUMFT2 = 0.0
      SUMFC2 = 0.0
      SUMFTC = 0.0
      CCALL = 0.0
C     Scale between FT and FC and sum F's..
      DO N = 1, NEXT
         KS = INT(SS4V(N)/SS4DEV) + 1
         IF (KS.GT.NSS) NSS = KS
         IF (ICODE(N).EQ.2) THEN
            NDET = NDET + 1
            SUMFT = SUMFT + FO(N)
            SUMFC = SUMFC + FC(N)
            SUMFT2 = SUMFT2 + FO(N)*FO(N)
            SUMFC2 = SUMFC2 + FC(N)*FC(N)
            SUMFTC = SUMFTC + FO(N)*FC(N)
            IF (KS.GT.NSS4) KS = NSS4
            SFT(KS) = SFT(KS) + FO(N)
            SFC(KS) = SFC(KS) + FC(N)
            SFT2(KS) = SFT2(KS) + FO(N)*FO(N)
            SFC2(KS) = SFC2(KS) + FC(N)*FC(N)
            SFTC(KS) = SFTC(KS) + FO(N)*FC(N)
            NSUM(KS) = NSUM(KS) + 1
         END IF
      END DO
      SCALE = SUMFT/SUMFC
      DO KS = 1, NSS4
         IF (NSUM(KS).GT.0) SK(KS) = SFT(KS)/SFC(KS)
      END DO
      DO N = 1, NEXT
         IF (ICODE(N).EQ.2) THEN
            RFAC = RFAC + ABS(FO(N) - SCALE*FC(N))
            KS = INT(SS4V(N)/SS4DEV) + 1
            IF (KS.GT.NSS4) KS = NSS4
            RF(KS) = RF(KS) + ABS(FO(N) - SK(KS)*FC(N))
         END IF
      END DO
      RFAC = RFAC/SUMFT
      SNUM = REAL(NDET)
      CCALL = (SNUM*SUMFTC-SUMFT*SUMFC)/
     +     (SQRT((SNUM*SUMFT2 - SUMFT**2)*(SNUM*SUMFC2 - SUMFC**2)))
      DO KS = 1, NSS4
         IF (NSUM(KS).GT.0) THEN
            RF(KS) = RF(KS)/SFT(KS)
            SNUM = REAL(NSUM(KS))
            IF (NSUM(KS).EQ.1) THEN
               CCSL(KS) = 1.0
            ELSE
               CCDIV = (SNUM*SFT2(KS) - SFT(KS)**2)*
     +             (SNUM*SFC2(KS) - SFC(KS)**2)
               IF (CCDIV.GT.0.0)
     +             CCSL(KS) = (SNUM*SFTC(KS) - SFT(KS)*SFC(KS))/
     +             SQRT(CCDIV)
            END IF
         END IF
         CCSL(KS) = CCSL(KS) + 0.10
      END DO
      DO KS = NSS4+1, NSS
         CCSL(KS) = CCALL
         CCSL(KS) = CCSL(KS) + 0.10
      END DO
      RETURN
      END
