C
C     ALMN
C     Copyright (C) 1991 Tony Crowther
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C*************************************************************
C
C
C**************************************************************
C
C    Date:  Wed Jul 31 08:01:48 BST 1991
C    This version :     1993/07/09 16:13:47
C
C   The development of the MTZ files and associated software mark 1
C   is part of the masterplan of the ESF/EACBM Working Group 2.1 for
C   better Protein Crystallographic software for Europe.
C
C**************************************************************
C
C
         PROGRAM ALMN_MAIN
C        =================
C
C
C-- ALMN        ALMN.FOR   24/04/85    PAM
C
C Fast rotation function program by Eleanor Dodson
C
C     CALCULATION OF SPHERICAL HARMONIC COEFFICIENTS  ALMN
C     =====================================================
C
C
C CCP4 VERSION IN FORTRAN 77
C
C
C   Date: Thurs,  2 MAY 91
C   Program now runs on the Alliant at the LMB 
C   Problem with dividing indices by 2**16. 
C   Not fixed but avoided by setting largest index to  +- 100...
C
C   On the fx2800 this so far gives a Bus error Page violation
C   when compiled with    FCOMP = fortran -c  -Og
C           o.k.  with    NOOPTFCOMP = fortran -c  -save -ieee -g
C           o.k.  with    SPEC_FCOMP = fortran -c  -Og -save -ieee
C
C
C  Ian Tickle's correction to Bessel arithmetic incorporated. 
C  (See comments labelled IJT )
C
C
C
C Date: Tue,  6 FEB 90 11:03:59 GMT
C From: PRE@UK.AC.CAMBRIDGE.MRC-MOLECULAR-BIOLOGY
C       Subject:ALMN modified by Peter Daly, 
C       parser error fixed, new DCOSFD routine
C
C
C
C This file contains the routines:
C
C  ahvsor    almn_main   almnbd   almnmn  angfx2
C  chkeql    chkxno      dcosfd   eulera
C  exsect    frfs        gtpint   gtprea
C  maprnt    matmul      maxval   mdcft2  mdhft
C  msymlb    peakpk      pkfitp   pkintp
C  pksetm    polang      rbfror   rcards  rfmod
C  rmat      rngang      rorder   sort1   sphbes
C  symort
C
C
C
C---- This version may be used for both order 30 and 60
C     if the dimensions are set to allow for the 60
C     bessel function option.
C
C
C---- Messages will be printed if the program limits are
C     exceeded and execution terminated.
C
C Changes:
C  14-Jun-2000    Peter Briggs        CCP4
C      Removed s/rs DETMAT, GMPRD (now in libraries)
C      Removed s/r MATINV; replaced with calls to identical
C      library routine MINVN
C
C  15-Sep-1993    Phil Evans          MRC LMB
C      Restored Ian Tickle's weighting for the mean & RMS deviations
C
C   2-Feb-1990    Phil Evans          MRC LMB
C      Changed subroutine DCOSFD for one which is more robust
C      Altered by Peter Daly for new Parser
C
C  21-Nov-1989    Phil Evans          MRC LMB
C      More changes: s/r RORDER checks that matrix is pure rotation
C      Format of symmetry input changed
C      CRYSTAL, CELL, SYMMETRY commands can have crystal number
C                              specified as "# N", N =  or 2
C
C  26-Jun-1989    Phil Evans          MRC LMB
C      Tests on intensity limits now done on F's after application of
C      B-factor, before squaring
C
C  15-Jun-1989    Phil Evans          MRC LMB
C      Various changes (mostly minor):
C         Much of the control input and the printer output changed
C         Number of reflections 10000
C         Map file has Title from input
C         s/r PEAKPK interpolates peak positions & sorts them properly
C             into descending order: PEAKPK extensively rewritten
C         all angles output in range 0 - 360 degrees
C         NSYMZO (rotational symmetry around Zortho)
C         calculated by default
C         Peaks written optionally to output file
C
C  June 1987
C  EJD  - extensive tidying up to include Calling Cards in the style of
C  Polarrfn.  More sensible output I hope.
C  Peak search incorporated in this version.
C  Map output in form suitable for symmetry generation.
C
C------------------------------------------------------------------
C   Example Self-rotation function input for unix
C
C    #!/bin/csh -f
C    #
C    # almn
C    #
C    #      A complete self rotation calculation
C    #
C    setenv LCForMTZ mtz
C    #
C    almn  hklin  ../mtz/fvb_f_np.mtz \
C          mapout ../map/fvb_almnself.map \
C          << 'EOF-almn_self'
C    TITLE fvb self-rotation function, resolution 15-6, radius 0 30
C    SELF 3 30  
C    RESOLUTION  15 6
C    CRYSTAL NUMBER 1 1 -20 1000
C    LABIN F=fvb_F 
C    SYMMETRY NUMBER 1 P2122
C    LIMITS 0 90 5
C    PRINT
C    FIND  2 30 RMS OUTPUT fvb_self_peaks.pek
C    'EOF-almn_self'
C    #
C    #  Now read back map to search at different limits
C    #
C    almn  hklin  ../mtz/fvb_f_np.mtz \
C          mapout ../map/fvb_almnself.map \
C          << 'EOF-almn_mapread'
C    TITLE fvb  self-rotation function, resolution 15-6, radius 0 30
C    SELF 3 30  
C    RESOLUTION  15 6
C    CRYSTAL NUMBER 1 1 -20 1000000
C    LABIN F=fvb_F 
C    SYMMETRY NUMBER 1 P2221
C    LIMITS 0 90 5
C    NOPRINT
C    READ
C    FIND  30 30 OUTPUT fvb_self_peaks.pek
C    'EOF-almn_mapread'
C    #
C
C---------------------------------------------------------------------
C
C
C     ALMN is a version of Crowther's FFT rotation  function  program  
C which was obtained from Eleanor Dodson (York University). The program  
C calculates rotation function overlap values using FFT  techniques  
C (ref.  T.  Crowther "The Molecular Replacement Method" ed. M. Rossmann). 
C The  original  three programs have been merged into the one here by 
C Eleanor Dodson. She has also extended the output to detail symmetry 
C related Eulerian  angles,  spherical polars and direction cosines for 
C each peak in the output  Eulerian  angle map. 
C
C    This is a fast rotation function which works in Eulerian angles, an 
C extension of the Crowther Rotation program by Eleanor Dodson.
C     The position of peaks is given for all symmetry related positions 
C in  both Eulerian and polar angles.
C    The  axis  conventions  have  been generalised and it is possible 
C to deal with all space groups.
C
C    Options are provided in the program for various restart points.
C
C  1.   The results of the first stage of the calculation (ALMN) may also 
C       be saved for subsequent rotation function summations on different 
C       grids.
C       Beware however these spherical harmonics  change  for  different
C       resolution and integration sphere cut  off  values.
C
C  2.   The calculated rotation map is written to a standard CCP4 map
C       file, which can be used as input for the programs EXTEND, if a 
C       set of "spacegroup" symmetry operations can be derived, or can be 
C       plotted with a standard package (eg PLUTO). The program will 
C       reread this file to do a peak search. (see description of the card 
C       FIND for details of symmetry) 
C
C  3.  A list of peaks may be read in, and all symmetry related peaks 
C      generated (option PEAK)
C
C
C
C
C     .. Scalars in Common ..
      INTEGER LCR1,LCR2,LCO1,LCO2,NCRYST,NREF1,NREF2
C     ..
C     .. Arrays in Common ..
      INTEGER LSPGRP,NCODES,NSYMZO
      REAL CELCRY,F2MAX,F2MIN,F2MINS,RFCRYS,ROCRYS,SCALFO,TFAC,VOLCR
C     ..
C     .. Local Scalars ..
      INTEGER IFAIL,LDUM
C     ..
C     .. External Subroutines ..
      EXTERNAL ALMNMN,CCPDPN
C     ..
C     .. Common Blocks ..
      COMMON /INALM/ LCR1,LCR2,LCO1,LCO2
      COMMON /CRYSTS/NCRYST,ROCRYS(4,4,2),RFCRYS(4,4,2),CELCRY(6,2),
     +       NCODES(2),LSPGRP(2),VOLCR(2),TFAC(2),NSYMZO(2),F2MIN(2),
     +       SCALFO(2),F2MAX(2),F2MINS(2),NREF1,NREF2
C
C---- Open main read and write files.
C
      IFAIL = 0
C
C          ********************************************
      CALL MTZINI
      CALL CCPFYP
      CALL CCPDPN(5,'DATA','READONLY','F',LDUM,IFAIL)
      CALL CCPDPN(6,'PRINTER','PRINTER','F',LDUM,IFAIL)
      CALL CCPRCS(6,'ALMN', '$Date: 2005/09/06 11:25:07 $')
C          *********************************************
C
C---- Read control data
C
C          ******
      CALL ALMNMN
C          ******
C
      CLOSE(UNIT=LCO1)
      IF (NCRYST .EQ. 2) CLOSE(UNIT=LCO2)
      CALL CCPERR(0,' Normal Termination of almn')
C
C
      END
C
C
C
C
      SUBROUTINE RCARDS(AMIN,GMIN)
C     ============================
C
C
C---- Read in the control data                         
C
C  31-May-1989  Phil Evans    MRC LMB
C        changed some input around to match POLARRFN                   
C                                                          
C---- Maximum section size  PARAMETER (NSMAP=32851)  
C
C     .. Parameters ..
      INTEGER NL                          
      PARAMETER (NL=15)
      INTEGER NLOC,MaxSets                          
C      PARAMETER (NLOC=50)
      PARAMETER (NLOC=200,MaxSets=50)
      INTEGER NPARM
      PARAMETER (NPARM=200)
      INTEGER MAXNPK
      PARAMETER (MAXNPK=100)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)             
      INTEGER NKEY
      PARAMETER (NKEY=21)             
      INTEGER NREFMX
      PARAMETER (NREFMX=10000)
      DOUBLE PRECISION PI
      PARAMETER (PI=3.141592653589793D00)
C     ..
C     ..
C     .. Scalars in Common ..               
      REAL ASTEP,BSTEP,BTMAX,BTMIN,GSTEP,PEAK,RESMAX,RESMIN,SMAX,SMIN,
     +     SPHMAX,SPHMIN
      INTEGER IBRKFL,IRSYGP,LCO1,
     +        LCO2,LCR1,LCR2,LMAP,LPEAK,LPKOUT,LPKRMS,LPLOT,LPRINT,
     +        LRDAL,LRDMP,LSAVE,LSEC1,LSEC2,MAXPEK,NALPHA,NBES,
     +        NBETA,NBETAG,NCODE,NCRYST,NGAMMA,NPEAK,NREF1,NREF2,
     +        LREFMX
      CHARACTER PKFNAM*80,TITLE*80
      LOGICAL OutputXML

C     ..
C     .. Arrays in Common ..
      REAL ARSYM,CELCRY,F2MAX,F2MIN,F2MINS,PEAKS,
     +     PKPOS,RF,RFCRYS,RO,ROCRYS,RSYM,RSYMT,SCALFO,TFAC,VOLCR
      INTEGER JSYM,JSYMP,LPKORD,LSPGRP,NCODES,NSYMZO
C     ..
C     ..         
C     .. Local Scalars ..
      REAL AMAX,C,C1,C2,GMAX,RAT2,RESTEM,RHMAX,RHMEAN,RHMIN,
     +     RMS,STEP,VOLAS1,VOLAS2,VOLSPH
      INTEGER I,IA1,IA2,IB1,IG1,IG2,IMAP,IS,IXTL,J,JDO10,JDO20,
     +        JDO220,JDO30,JDO330,JDO340,JDO350,JDO360,JDO370,JDO390,
     +        JDO40,JDO400,JDO410,JDO420,JDO430,JDO440,JDO450,JDO460,
     +        JDO470,JDO60,JS,JSS,KS,LP,M,MODE,NCOL,NCONTR,NCR,NERR,
     +        NORDER,NSPGRP,NSYM,NTOK,NREFTOT,JJ,LMAXR
      CHARACTER NAMSGP*10,PGNAME*10,LINE*600,KEY*4,LTYPE*1,
     +          LABIN_SAVE(2)*600
      LOGICAL LEND, NOTCAL,LSPHERE
C
C     ..
C     .. Local Scalars ...new
      REAL VOL
      INTEGER IERR,IPRINT,NLPRGI, JDO325
      CHARACTER DUMMY*10,FILIN(2)*70,ERRLIN*100
C                
C
C     .. Local Arrays ..
      REAL CONTRS(50),
     +     FVALUE(NPARM)
      INTEGER IUVW(3),NXYZ(3),IBEG(NPARM),IEND(NPARM),
     +        ITYP(NPARM),IDEC(NPARM)
      CHARACTER KEYS(NKEY)*4,NAMSGS(2)*10,CODES(5)*15,CVALUE(NPARM)*4
C     ..
C     .. Local Arrays ..new
      REAL RANGES(2,100) 
      INTEGER LOOKUP(NLOC)
      CHARACTER CTPRGI(NLOC)*1,LSPRGI(NLOC,2)*30
      CHARACTER PNAME(MaxSets)*64,XNAME(MaxSets)*64,DNAME(MaxSets)*64
      REAL DATCELL(6,MaxSets),DATWAVE(MaxSets)
      INTEGER ISETS(MaxSets),CSETID(NLOC),SETID,SET_INDEX
      REAL CEL(6),RR(3,3,6)
C
C
C     .. External Functions ..
      INTEGER LENSTR
      EXTERNAL LENSTR
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPUPC,GTPINT,GTPREA,INVSYM,MRCLOS,MRDHDR,
     +         PARSER,RBRINV,RORDER,SYMORT,LKYIN,
     +         LRASSN,LRINFO,LROPEN,LRSYMM
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC MAX,MIN,NINT,REAL,SQRT
C     ..
C     .. Common blocks ..
      COMMON /INALM/ LCR1,LCR2,LCO1,LCO2
      COMMON /BESSPH/SPHMIN,SPHMAX,NBES,NBETAG
      COMMON /CRYSTS/NCRYST,ROCRYS(4,4,2),RFCRYS(4,4,2),CELCRY(6,2),
     +       NCODES(2),LSPGRP(2),VOLCR(2),TFAC(2),NSYMZO(2),F2MIN(2),
     +       SCALFO(2),F2MAX(2),F2MINS(2),NREF1,NREF2
      COMMON /ORTHOG/RO(4,4),RF(4,4),NCODE,IBRKFL
      COMMON /PARAMS/LPRINT,LMAP,LPLOT,LRDMP,LRDAL,PEAK,MAXPEK,LSAVE,
     +       LPKRMS,LREFMX(2)
      COMMON /PEKPK/NPEAK,PEAKS(MAXNPK),PKPOS(3,MAXNPK),LPEAK,
     +       LPKORD(MAXNPK),LPKOUT
      COMMON /PKNAME/PKFNAM
      COMMON /RANGE/BTMIN,BTMAX,ASTEP,BSTEP,GSTEP,NALPHA,NBETA,NGAMMA,
     +       IRSYGP,LSEC1,LSEC2
C      COMMON /RBRKZZ/CEL(6),RR(3,3,6),VOL,CELLAS(6)
      COMMON /RES/RESMAX,RESMIN,SMIN,SMAX
      COMMON /SYMTR/JSYMP(2),RSYM(4,4,MAXSYM,2),RSYMT(4,4,MAXSYM,2),
     +       ARSYM(3,3,MAXSYM,2)
      COMMON /TITL/TITLE
      COMMON /XMLINFO/OutputXML
      DIMENSION JSYM(2)
C     ..
C     .. Save statement ..
      SAVE 
C     ..
C     .. Data statements ..
c      DATA KEYS/'TITL','SELF','CROS','RESO','CRYS','LABI','CELL','SYMM',
c     +     'LIMI','PRIN','NOPR','MAP ','PLOT','FIND','READ','SAVE',
c     +     'SUM ','PEAK','ROTS','ORTH','END ','MAXR'/
      DATA KEYS/'TITL','SELF','CROS','RESO','CRYS','LABI',
     +     'LIMI','PRIN','NOPR','MAP ','PLOT','FIND','READ','SAVE',
     +     'XMLO','SUM ','PEAK','ROTS','END ','MAXR','LSPH'/
      DATA CODES/'a,c*xa,c*','b,a*xb,a*','c,b*xc,b*','a+b,c*x(a+b),c*',
     +     'a*,cxa*,c'/
      DATA LP/6/
C     ..                                   
C     .. Data statements ..new
C
C---- NLPRGI  =  number of input labels
C                          
      DATA NLPRGI,(LSPRGI(JJ,1),JJ=1,200)
     +               /5,'H','K','L','F','SIGF',195*' '/
      DATA (LSPRGI(JJ,2),JJ=1,200)
     +               /'H','K','L','F','SIGF',195*' '/
C        
C
C---- This code signs which input columns are essential (LOOKUP)
C
      DATA CTPRGI/'H','H','H','F','Q',195*' '/
      DATA FILIN/'HKLIN','HKLIN2'/
C     ..
C
C---- Set defaults
C
C  Sphere radius reset if volume too great.
      LSPHERE   = .TRUE.
      LREFMX(1) = NREFMX
      LREFMX(2) = NREFMX
      LTYPE = ' '
      TITLE = ' '
      NCRYST = 1
      SPHMAX = 0.0
      RESMIN = 10000.0
      RESMAX = 2.0
      CEL(1) = 0.0
      CEL(2) = 0.0
      CEL(3) = 0.0
C
C---- Default cell angles to 90.0
C
      CEL(4) = 90.0
      CEL(5) = 90.0
      CEL(6) = 90.0
C
C
      DO 10 JDO10 = 1,2
        NCODES(JDO10) = 1
        NSYMZO(JDO10) = -1
        LSPGRP(JDO10) = 1
        JSYM(JDO10) = 0
        JSYMP(JDO10) = 0
        TFAC(JDO10) = 0.0
        F2MIN(JDO10) = 0.0
        CELCRY(1,JDO10) =  0.0
        CELCRY(2,JDO10) =  0.0
        CELCRY(3,JDO10) =  0.0
        CELCRY(4,JDO10) = 90.0
        CELCRY(5,JDO10) = 90.0
        CELCRY(6,JDO10) = 90.0
   10 CONTINUE
C
C
      DO 40 JDO40 = 1,2
        DO 30 JDO30 = 1,4
          DO 20 JDO20 = 1,4
            RSYM(JDO20,JDO30,1,JDO40) = 0.0
   20     CONTINUE
   30   CONTINUE
C
C
        RSYM(1,1,1,JDO40) = 1.0
        RSYM(2,2,1,JDO40) = 1.0
        RSYM(3,3,1,JDO40) = 1.0
        RSYM(4,4,1,JDO40) = 1.0
   40 CONTINUE
C
C
      LPRINT = -100
      LMAP = 1
      LPLOT = 0
      NCONTR = 0
      NPEAK = 0
      PEAK = 0.0
      LPKRMS = 0
      LPKOUT = 0
      MAXPEK = 10
      LRDMP = 0
      LRDAL = 0
      LPEAK = 0
      LSAVE = 0
      OutputXML = .FALSE.
C
C---- if you know the rotation symmetry make an entry
C     into the symop,.dat file with 'spacegroup number ' irsygp
C     read it in on the range card
C
      IRSYGP = 1
      NERR = 0
      IXTL = 0
      NSYM = 0
   50 CONTINUE
C
C
      NTOK=NPARM
      LINE = '    '
C
C          ***********************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +            .TRUE.)
C          ***********************************************************
C
      IF (.NOT.LEND) THEN
C
C
C  reset this default - you will otherwise default Crystal 2 value to that 
C   of crystal 1.
        NCODE = 1
        DO 60 JDO60 = 1,NKEY
          IF (KEY.EQ.KEYS(JDO60)) GO TO 70
   60   CONTINUE                                             
C
C
        WRITE (6,FMT=6000)
 6000 FORMAT (/' Unrecognized keyword',/)
        NERR = NERR + 1
        GO TO 50
C
C
c      DATA KEYS/'TITL','SELF','CROS','RESO','CRYS','LABI',
c     +     'LIMI','PRIN','NOPR','MAP ','PLOT','FIND','READ','SAVE',
c     +     'XMLO','SUM ','PEAK','ROTS','END ','MAXR','LSPH'/
C
   70   GO TO (80,90,100,110,120,125,
     +         150,160,170,180,190,210,240,250,255,
     +         260,270,290, 320,299,85) JDO60
C
C---- TITLe  read title
C     =====
C
   80   TITLE = LINE(IBEG(2) :)
        GO TO 50
C
C---- LSPHERE - do not reset sphere radius if it is too large.
C     =====
C
   85   LSPHERE = .FALSE.
        GO TO 50
C
C---- MAXREF maxnumber of reflections to be used
C
299        CONTINUE
C         CALL GTPINT(2,LREFMX,NTOK,ITYP,FVALUE)
           GO TO 50
C
C---- SELF,
C     =====
C      set flag for self rotation,
C      read integration radius sphmax,
C      smoothing
C
   90   NCRYST = 1
        GO TO 310
C
C---- CROSS,
C     ======
C      set flag for cross rotation,
C      read integration radii sphmin sphmax.
C
  100   NCRYST = 2
        GO TO 310
C
C---- RESOlution
C     ==========
C     read resolution limits in A, if only one treat as high
C     resolution limit
C
  110   CONTINUE
C
C            *********************************
C     read resolution limits in A, if only one treat as high
C     resolution limit
        ITOK = 2
C            ********************************************************
        CALL RDRESO(ITOK,ITYP,FVALUE,NTOK,RESMIN,RESMAX,SMIN,SMAX)
C            ********************************************************
        GO TO 50
C
C                                 
c
C
C---- CRYSTAL   - subsiduary keywords; FILE or NUMBER 
C                                      SYMM  ORTH  BFAC FLIMITS SCALE
C
C---- Check command line for
C---- subsidiary keyword FILE  crystal number
C---- subsidiary keyword ORTH  - orthogonalisation code NCODE, 
C---- subsidiary keyword BFAC  - temperature factor B  
C                            Fused = Finput * exp(-Bssq/lsq)
C---- subsidiary keyword FLIMITS  - minimum F and maximum F(order unimportant)
C---- subsidiary keyword SCALE  - scalfo(ncr) to divide Fsquared.
C---- subsidiary keyword CELL  - a b c ( alpha beta  gamma - default = 90.0)
C                      rarely needed - just for peak analysis
C---- subsiduary keyword MAXREF max number of reflections to be used for this
C                               crystal. 
C
C
120    CONTINUE
C
      NOTCAL = .FALSE.
C
      FL1 = 0
      FL2 = 100000000
      SC  = 200000
      LMAXR  = 0
C
      ITOK = 1
121          ITOK = ITOK + 1
            IF (ITOK.GT.NTOK) THEN
              NCODES(IXTL) = NCODE
              TFAC(IXTL) = BB
              F2MIN(IXTL) = FL1
              F2MAX(IXTL) = FL2
              SCALFO(IXTL) = SC
              IF(LMAXR.NE.0)LREFMX(IXTL) = LMAXR
              GO TO 50
            END IF
              ICEND = MIN(IBEG(ITOK)+3,IEND(ITOK))
              KEY = LINE(IBEG(ITOK) :ICEND)
              CALL CCPUPC(KEY)
              IF (KEY.EQ.'FILE' .OR. KEY.EQ.'NUMB') THEN
                ITOK = ITOK + 1
C            ********************************
                CALL GTPINT(ITOK,IXTL,NTOK,ITYP,FVALUE)
C            ********************************
C
C---- Check if allowed range of files ... 1 to 2 (is this enough ???)
C
                IF (IXTL.LE.0 .OR. IXTL.GT.2) THEN
                  WRITE (LP,FMT='(a,/,a,i4,a,a,/,a)')
     +      ' Key_Word Error, For CRYSTAL FILE_NUMBER line:-',
     +      '  File_Number Given = ',IXTL,
     +      ' Is invalid only files 1 to 2',
     +      ' Allowed, Ignoring this line -',LINE(1:LENSTR(LINE))
                END IF
                GO TO 121
              END IF 
C
C
              IF (KEY.EQ.'ORTH') THEN
                ITOK = ITOK + 1
                CALL GTPINT(ITOK,NCODE,NTOK,ITYP,FVALUE)
                GO TO 121
              END IF 
C
              IF (KEY.EQ.'MAXR') THEN
                ITOK = ITOK + 1
                CALL GTPINT(ITOK,LMAXR,NTOK,ITYP,FVALUE)
                GO TO 121
              END IF 
C
              IF (KEY.EQ.'BFAC') THEN
                ITOK = ITOK + 1
                CALL GTPREA(ITOK,BB,NTOK,ITYP,FVALUE)
                GO TO 121
              END IF 
C
              IF (KEY.EQ.'SCAL') THEN
                ITOK = ITOK + 1
                CALL GTPREA(ITOK,SC,NTOK,ITYP,FVALUE)
                GO TO 121
              END IF 
C
              IF (KEY.EQ.'FLIM') THEN
                ITOK = ITOK + 1
                CALL GTPREA(ITOK,FL1,NTOK,ITYP,FVALUE)
                ITOK = ITOK + 1
                CALL GTPREA(ITOK,FL2,NTOK,ITYP,FVALUE)
                IF(FL1.LT.FL2) GO TO 121
                FL3 = FL1
                FL1 = FL2
                FL2 = FL3
                GO TO 121
              END IF 
C
              IF (KEY.EQ.'SYMM') THEN
C SYMMetry    read crystal number or  space group 
                ITOK = ITOK + 1
C 
C   Only nterested in checking next token
              NTOKS = ITOK
C
C          ************************************************************
              CALL RDSYMM(ITOK,LINE,IBEG,IEND,ITYP,FVALUE,NTOKS,
     +            NAMSGP,NSPGRP,PGNAME,NSYM,NSYMP,RSYM(1,1,1,IXTL))
C          ************************************************************
C
C   Only need primitive symmetry operators
                JSYM(IXTL) = NSYM
                JSYMP(IXTL) = NSYMP
                LSPGRP(IXTL) = NSPGRP
                NAMSGS(IXTL) = NAMSGP
                GO TO 121
              END IF 
C
C CELL    read cell dimensions - not usually needed.
              IF (KEY.EQ.'CELL') THEN
                DO 122 I=4,6
122              CELCRY(I,IXTL)=90.
                ITOK = ITOK + 1
                CALL GTPREA(ITOK,CELCRY(1,IXTL),NTOK,ITYP,FVALUE)
                ITOK = ITOK + 1
                CALL GTPREA(ITOK,CELCRY(2,IXTL),NTOK,ITYP,FVALUE)
                ITOK = ITOK + 1
                CALL GTPREA(ITOK,CELCRY(3,IXTL),NTOK,ITYP,FVALUE)
                   IF(ITOK.LT.NTOK)THEN
                   ITOK = ITOK + 1
                   CALL GTPREA(ITOK,CELCRY(4,IXTL),NTOK,ITYP,FVALUE)
                   ITOK = ITOK + 1
                   CALL GTPREA(ITOK,CELCRY(5,IXTL),NTOK,ITYP,FVALUE)
                   ITOK = ITOK + 1
                   CALL GTPREA(ITOK,CELCRY(6,IXTL),NTOK,ITYP,FVALUE)
                   END IF
C
                GO TO 121
              END IF 
C

C---- LABIN   (File_Number)  "a=b c=d ...."
C     ==========
C
 125  CONTINUE
              ITOK = 2
              KEY = LINE(IBEG(ITOK) :IEND(ITOK))
C
C          *************
      CALL CCPUPC(KEY)
C          *************
C
      IF (KEY(1:1).EQ.'F') THEN
C
C---- second arguement = 'FILE_NUMBER'
C     test if third argument is integer
C
        IF (ITYP(3).NE.2) THEN
          WRITE (LP,FMT='(A,A,/,A,A,A,/,A)')
     +      ' Key_Word Error, Wrong type of input for  LABIN',' line:-',
     +      ' If Sub_KeyWord = FILE_NUMBER  then ',
     +      'third  Arguement MUST be an Integer  Ignoring this',
     +      ' line -',LINE(1:LENSTR(LINE))
          GO TO 50
        END IF
C
C---- For an input file being given
C     Get File_Number and check if valid
C
C            ********************************
        CALL GTPINT(3,NFILE,NTOK,ITYP,FVALUE)
C            ********************************
C
        NFILES_IN = MAX(NFILES_IN,NFILE)
C
C---- Check if allowed range of files ... 1 to 5 (is this enough ???)
C
        IF (NFILE.LE.0 .OR. NFILE.GT.2) THEN
          WRITE (LP,FMT='(a,/,a,i4,a,a,/,a)')
     +      ' Key_Word Error, For LABIN FILE_NUMBER line:-',
     +      '  File_Number Given = ',NFILE,
     +      ' Is invalid only files 1 to 2',
     +      ' Allowed, Ignoring this line -',LINE(1:LENSTR(LINE))
          GO TO 50
        END IF
      END IF
C
C---- Now save labels for this input file
C
      LABIN_SAVE(NFILE) = 'LABIN  '
      LABIN_SAVE(NFILE)(8:) = LINE(IBEG(4) :IEND(NTOK))
C         write(6,*) ' labin_save',LABIN_SAVE(NFILE) 
cc      GOT_LABIN(NFILE)  = .TRUE.
C
C
      GO TO 50
C
C
C            
C           
C
C---- LIMIT
C     =====
C     read btmin btmax bstep
C
  150   CONTINUE
C
C            ********************************
        CALL GTPREA(2,BTMIN,NTOK,ITYP,FVALUE)
        CALL GTPREA(3,BTMAX,NTOK,ITYP,FVALUE)
        CALL GTPREA(4,BSTEP,NTOK,ITYP,FVALUE)
C            ********************************
C
        GO TO 50
C
C---- PRINT
C     =====
C      read print threshold
C
  160   CONTINUE
        LPRINT = 0
C
C            ********************************
        CALL GTPINT(2,LPRINT,NTOK,ITYP,FVALUE)
C            ********************************
C
        LMAP = 1
        LPRINT = MAX(-99,LPRINT)
        GO TO 50
C
C---- NOPRint
C     =======
C     switch off map print
C
  170   LPRINT = -100
        GO TO 50
C
C---- MAP
C     ====
C     switch to write map to map file
C
  180   LMAP = 1
        GO TO 50
C
C---- PLOT
C     ====
C     read contour start, interval , NOTItle flag
C
C                 NOT IMPLEMENTED !!!
C
C
  190   LPLOT = 1
        C1 = 10.0
        C2 = 100.0
        C = 10.0
C
C            *****************************
        CALL GTPREA(2,C1,NTOK,ITYP,FVALUE)
        CALL GTPREA(3,C,NTOK,ITYP,FVALUE)
C            *****************************
C
        IF (C1.GT.C2 .OR. C.LE.0) THEN
          WRITE (6,FMT=6004) C1,C2,C
 6004 FORMAT (/' Unreasonable contour limits:',3F10.2,/)
          NERR = NERR + 1
        ELSE
          NCONTR = 1
          CONTRS(1) = C1
  200     CONTINUE
          CONTRS(NCONTR+1) = CONTRS(NCONTR) + C
C
C
          IF (CONTRS(NCONTR+1).LE.C2) THEN
            NCONTR = NCONTR + 1
            GO TO 200
          END IF
        END IF
C
C
        GO TO 50
C
C---- FIND
C     =====
C     read peak threshold and maximum number of peaks
C  also
C    optional keyword "RMS" to indicate that threshold is in
C       multiples of RMS density rather than absolute
C    optional keyword OUTPUT filename to output unique
C    peak list to file
C
C    NB RMS must precede OUTPUT if both are present
C
  210   CONTINUE
C
C---- must have a map to search!!
C
        LMAP = 1
C
C            ********************************
        CALL GTPREA(2,PEAK,NTOK,ITYP,FVALUE)
        CALL GTPINT(3,MAXPEK,NTOK,ITYP,FVALUE)
C            ********************************
C
        MAXPEK = MIN(MAXPEK,MAXNPK)
C
C---- Process rest of line
C
        LPKRMS = 0
        LPKOUT = 0
C
C
        IF (NTOK.GE.4) THEN
          DO 220 JDO220 = 4,NTOK
            IF (CVALUE(JDO220).EQ.'RMS') THEN
              LPKRMS = 1
            ELSE IF (CVALUE(JDO220).EQ.'OUTP') THEN
              LPKOUT = 1
C
C---- Store rest of line as filename if given
C
              IF (NTOK.GT.JDO220) THEN
                GO TO 230
              ELSE
                PKFNAM = 'PEAKS'
              END IF
            END IF
  220     CONTINUE                        
          GO TO 50
  230     PKFNAM = LINE(IBEG(JDO220+1) :)
        END IF
        GO TO 50
C
C---- READ
C     =====
C        read map
C
  240   LRDMP = 1
C
C---- no save, no map output
C
        LRDAL = 0
        LMAP = 0
        IMAP = 1
        MODE = 2
C
C            **********************************************************
        CALL MRDHDR(IMAP,'MAPOUT',TITLE,NBETA,IUVW,NXYZ,IB1,IA1,IA2,IG1,
     +              IG2,CEL,IRSYGP,MODE,RHMIN,RHMAX,RHMEAN,RMS)
C            **********************************************************
C
        NALPHA = IA2 - IA1 + 1
        NGAMMA = IG2 - IG1 + 1
C
C            ************
        CALL MRCLOS(IMAP)
C            ************
C
        NBETAG = NXYZ(2)/2
        ASTEP = CEL(1)/NXYZ(1)
        BSTEP = CEL(2)/NXYZ(2)
        GSTEP = CEL(3)/NXYZ(3)
        STEP = ASTEP
        AMIN = IA1*ASTEP
        AMAX = IA2*ASTEP
        GMIN = IG1*GSTEP
        GMAX = IG2*GSTEP
        BTMIN = IB1*BSTEP
        BTMAX = (IB1+NBETA-1)*BSTEP
C
C            ********************************
        CALL GTPREA(2,BTMIN,NTOK,ITYP,FVALUE)
        CALL GTPREA(3,BTMAX,NTOK,ITYP,FVALUE)
C            ********************************
C
        LSEC1 = NINT(BTMIN/BSTEP)
        LSEC2 = NINT(BTMAX/BSTEP)
        WRITE (6,FMT=6020) AMIN,AMAX,BTMIN,BTMAX,GMIN,GMAX,ASTEP,
     +    BSTEP,GSTEP
        GO TO 50
C
C---- SAVE
C     ====
C      read file name for saving ALMN output
C    
  250   CONTINUE
        LSAVE = 1
C
        GO TO 50
C
C---- XMLO
C     ====
C      output an XML file
  255   CONTINUE
        OutputXML = .TRUE.
        GO TO 50
C
C---- SUM
C     ===
C    set flag for reading coeffs1  (and coeffs2) which
C    have been calculated previuosly.
C
  260   LRDAL = 1
        GO TO 50
C
C---- PEAK
C     ====
C     read list of peaks alpha,beta,gamma from following cards,
C     terminated by blank line or 'END'
C
C  if Peak set, then there can be no map input or coeffs input
C
  270   LPEAK = 1
        LRDAL = 0
        LRDMP = 0
        LMAP = 0
        NPEAK = 0
  280   CONTINUE
        NTOK=NPARM
        LINE = ' '
C
C            *******************************************************
        CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,
     +              LEND,.TRUE.)
C            *******************************************************
C
        IF (LEND) THEN
          GO TO 320
        ELSE IF (NTOK.LE.0 .OR. KEY.EQ.'END') THEN
          GO TO 50
        ELSE
          NPEAK = NPEAK + 1
C
C
          IF (NPEAK.LT.MAXNPK) THEN
C
C                *****************************************
            CALL GTPREA(1,PKPOS(1,NPEAK),NTOK,ITYP,FVALUE)
            CALL GTPREA(2,PKPOS(2,NPEAK),NTOK,ITYP,FVALUE)
            CALL GTPREA(3,PKPOS(3,NPEAK),NTOK,ITYP,FVALUE)
C                *****************************************
C
            LPKORD(NPEAK) = NPEAK
            GO TO 280
          END IF
        END IF
C
C
        WRITE (6,FMT=6006) MAXNPK
 6006 FORMAT (/' Too many peaks given, reset to maximum =',I4,/)
        NPEAK = MAXNPK
        GO TO 50
C
C---- ROTSym
C     ======
C      set special 'rotation' space-group number
C
C            *********************************
  290   CALL GTPINT(2,IRSYGP,NTOK,ITYP,FVALUE)
C            *********************************
C
        GO TO 50
C               
C
  310   CONTINUE
C
C            *********************************
        CALL GTPREA(2,SPHMIN,NTOK,ITYP,FVALUE)
        CALL GTPREA(3,SPHMAX,NTOK,ITYP,FVALUE)
C            *********************************
C
        IF (SPHMIN.GT.SPHMAX) THEN
          RESTEM = SPHMIN
          SPHMIN = SPHMAX
          SPHMAX = RESTEM
        END IF
C
C
        IF (SPHMIN.LE.0.0) SPHMIN = 0.1
        GO TO 50
      END IF
C
C---- End of input
C
C---- Check for errors
C
  320 IF (NERR.GT.0) THEN
        WRITE (6,FMT=6008) NERR
 6008 FORMAT (/1X,I6,' errors in input, see above !!!!',/)
        CALL CCPERR(1,' input error')
      ELSE
C
C---- Check that enough crystal cards were given and MTZ files opened
C     unless READ  options used
C
        IF (IXTL.EQ.0 .AND. LRDMP.EQ.0 .AND. LRDAL.GE.0 .AND.
     +      LPEAK.EQ.0) WRITE (6,FMT=6010) IXTL,NCRYST
 6010 FORMAT (/'  CRYSTAL card missing !!!',/2I6)
C
C---- move the calcultation of rfcrys etc to after end card read
C
C---- this generates the various orthogonalising matrices
C     ' NCODE =1 -  ORTHOG AXES ARE DEFINED TO HAVE'
C                   A PARALLEL TO XO   CSTAR PARALLEL TO ZO'
C     ' NCODE =2 -  ORTHOG AXES ARE DEFINED TO HAVE'
C     '               B PARALLEL TO XO   ASTAR PARALLEL TO ZO'
C     ' NCODE =3 -  ORTHOG AXES ARE DEFINED TO HAVE'
C     '               C PARALLEL TO XO   BSTAR PARALLEL TO ZO'
C     ' NCODE =4 -  ORTHOG AXES ARE DEFINED TO HAVE'
C     '         HEX A+B PARALLEL TO XO   CSTAR PARALLEL TO ZO'
C     ' NCODE =5 -  ORTHOG AXES ARE DEFINED TO HAVE'
C     '           ASTAR PARALLEL TO XO       C PARALLEL TO ZO'
C   SET UP MATRICES TO ORTHOGONALISE H K L AND X Y Z FOR THIS CELL.
C
C
        DO 380 NCR = 1,NCRYST
C
C---- Check NCODE defined
C
        IF (NCODES(NCR) .LT. 1 .OR. NCODES(NCR) .GT. 5) THEN
           WRITE (6, 6101) NCR
6101       FORMAT(' !!!! Illegal or undefined NCODE for crystal ',I4/)
           CALL CCPERR(1,' NCODE error')
        ENDIF
C
C   Open MTZ files
C---- Open input file for Read; print header...
C
            IPRINT = 1
            IERR = -1
C
C           ==================================
            CALL LROPEN(NCR,FILIN(NCR),IPRINT,IERR)
C           ==================================
C
            ERRLIN = 'ERROR:HEADER NOT THERE, ILLEGAL FILE'
C
C                          =========================================
            IF (IERR.EQ.3) CALL LERROR(2,1,ERRLIN(1:LENSTR(ERRLIN)))
C                         =========================================
C
         LINE= LABIN_SAVE(NCR) 
         DO 325 JDO325=1,4
          LOOKUP(JDO325) = -1
  325    CONTINUE
         LOOKUP(5) = 0
C
C             ======================================================
         CALL PARSER(KEY,LINE,IBEG,IEND,ITYP,FVALUE,CVALUE,IDEC,NTOK,
     +               LEND,.TRUE.)
         CALL LKYIN(NCR,LSPRGI(1,NCR),NLPRGI,NTOK,LINE,IBEG,IEND)

         CALL LRASSN(NCR,LSPRGI(1,NCR),NLPRGI,LOOKUP,CTPRGI)
C             ======================================================
C
C           ======================
            CALL LRCELL(NCR,CEL)
C           ======================
C---- Get dataset info using LRIDX:
C     Notify available space
      NDATASETS = MaxSets
      CALL LRIDX(NCR,PNAME,XNAME,DNAME,ISETS,
     +                     DATCELL,DATWAVE,NDATASETS)
C
C---- We can only check the datasets if there is dataset
C     information in the mtz file
C
      IF (NDATASETS.GT.0) THEN
C
C---- Get the complete list of labels and types and the

C     datasets to which they belong
C
        NCOL = NLOC
C       ****************************************
        CALL LRCLID(NCR,CSETID,NCOL)
C       ****************************************
C     Get setid of FP column and use that DATCELL as master
        SET_INDEX = 0
        SETID = 0
        IF (LOOKUP(4).GT.0) THEN
          SETID = CSETID(LOOKUP(4))
          DO ISET = 1,NDATASETS
            IF (ISETS(ISET).EQ.SETID) THEN
              SET_INDEX = ISET
            END IF
          ENDDO
        ENDIF
C     Set output dataset and correct choice of cell
        IF (SET_INDEX.GT.0) THEN
          DO I=1,6
            CEL(I) = DATCELL(I,SET_INDEX)
          ENDDO
        ENDIF
      ENDIF
C      check cell
            IF(CELCRY(1,NCR).NE.0.0) THEN
              DO 351 II = 1,6
              ACHK = ACHK +
     +               ABS(CEL(II) - CELCRY(II,NCR)) /
     +                  (CEL(II) + CELCRY(II,NCR))
              IF (ACHK.GT.0.02)CALL CCPERR(1,
     +        ' Large Difference in CELL DIMENSIONS')
              IF (ACHK.GT.0.01) WRITE(LP,*)
     +         ' Small Difference in CELL DIMENSION'
351           CONTINUE
            ELSE
              DO 352 II = 1,6
              CELCRY(II,NCR) =  CEL(II)
352           CONTINUE
            END IF

C---- Find out how many columns in input file?
C
C           ========================================
            CALL LRINFO(NCR,DUMMY,NCOL,NREFTOT,RANGES)
C           ========================================
C             IF(F2MIN(NCR).LE.0.0001)
C     +          F2MIN(NCR) = 0.5*RANGES(2,LOOKUP(4))
C
            IF(NCR.EQ.1)NREF1=NREFTOT
            IF(NCR.EQ.2)NREF2=NREFTOT
C    Read symmetry and check
        CALL LRSYMI(NCR,NSYMP,LTYPE,NSPGRP,NAMSGP,PGNAME)
        CALL LRSYMM(NCR,NSM,RSYM(1,1,1,NCR))
        IF(JSYMP(NCR).GT.0 .AND. JSYMP(NCR).NE.NSYMP)THEN
           WRITE(LP,'(a)') ' Awful symmetry error??'
c        a mess...
        END IF
                JSYM(NCR) = NSM
                JSYMP(NCR) = NSYMP
                LSPGRP(NCR) = NSPGRP
                NAMSGS(NCR) = NAMSGP
C  
C
C---- Check NCODE defined
C
         IF (NCODES(NCR) .LT. 1 .OR. NCODES(NCR) .GT. 6) THEN
           WRITE (LP, 6101) J
           CALL CCPERR(1,' NCODE error')
         ENDIF
C


C
C Check that crystal cards were given and LCF files opened
C unless READ or SUM options used
      IF(NOTCAL.AND.LRDMP.EQ.0.AND.
     .              LSAVE.GE.0.AND.LPEAK.EQ.0) THEN
            WRITE(LP,9902)
9902      FORMAT(/' No CRYSTAL card given !!!'/)
            CALL CCPERR(1,' input error')
      ENDIF
C
C If 1 crystal, copy symmetry etc to crystal 2
      IF(NCRYST.EQ.1) THEN
            DO 9903 IS =1,JSYMP(1)
            DO 9903 I =1,4
            DO 9903 J =1,4
9903        RSYM(I,J,IS,2) = RSYM(I,J,IS,1)
            JSYM(2)=JSYM(1)
            JSYMP(2)=JSYMP(1)
            NCODES(2)=NCODES(1)
                LSPGRP(NCR) = NSPGRP
            LSPGRP(2)=LSPGRP(1)
CEJD            CALL MOVEI(ORTHOG(1,1,2),ORTHOG(1,1,1),9)
CEJD            CALL MOVEI(FRAC(1,1,2),FRAC(1,1,1),9)
      ENDIF
C
C  Default title
C
      IF(LENSTR(TITLE).EQ.0) THEN
            IF(NCRYST.EQ.1) THEN
                  TITLE='Self-rotation function'
            ELSE
                  TITLE='Cross-rotation function'
            ENDIF
      ENDIF
C
C Print control data






C---- Set symmetry
C
          DO 350 JDO350 = 1,JSYMP(NCR)
            DO 340 JDO340 = 1,4
              DO 330 JDO330 = 1,4
C
C                    ********************************************
                CALL INVSYM(RSYM(1,1,JDO350,NCR),RSYMT(1,1,JDO350,NCR))
C                    ********************************************
C
                RSYM(JDO330,JDO340,JDO350,NCR) = RSYM(JDO330,JDO340,
     +            JDO350,NCR)
                RSYMT(JDO330,JDO340,JDO350,NCR) = RSYMT(JDO330,JDO340,
     +            JDO350,NCR)
  330         CONTINUE
  340       CONTINUE
  350     CONTINUE
C
C
          CEL(1) = CELCRY(1,NCR)
          CEL(2) = CELCRY(2,NCR)
          CEL(3) = CELCRY(3,NCR)
          CEL(4) = CELCRY(4,NCR)
          CEL(5) = CELCRY(5,NCR)
          CEL(6) = CELCRY(6,NCR)
C
C              ******
C  Set up fractional to orthogonal matrix RR with appropriate NCODE.
      CALL RBFRO1(CEL,VOL,RR)
C              ******
C
          VOLCR(NCR) = ABS(VOL)
          RO(4,4) = 1.0
C
C
C
          DO 370 JDO370 = 1,3
            DO 360 JDO360 = 1,3
              RO(4,JDO360) = 0
              RO(JDO360,4) = 0
              RO(JDO360,JDO370) = RR(JDO360,JDO370,NCODES(NCR))
  360       CONTINUE
  370     CONTINUE
C
C---- invert ro...
C
C              *************
          CALL RBRINV(RO,RF)
C              *************
C
          RFCRYS(1,1,NCR) = RF(1,1)
          RFCRYS(1,2,NCR) = RF(1,2)
          RFCRYS(1,3,NCR) = RF(1,3)
          RFCRYS(2,1,NCR) = RF(2,1)
          RFCRYS(2,2,NCR) = RF(2,2)
          RFCRYS(2,3,NCR) = RF(2,3)
          RFCRYS(3,1,NCR) = RF(3,1)
          RFCRYS(3,2,NCR) = RF(3,2)
          RFCRYS(3,3,NCR) = RF(3,3)
          ROCRYS(1,1,NCR) = RO(1,1)
          ROCRYS(1,2,NCR) = RO(1,2)
          ROCRYS(1,3,NCR) = RO(1,3)
          ROCRYS(2,1,NCR) = RO(2,1)
          ROCRYS(2,2,NCR) = RO(2,2)
          ROCRYS(2,3,NCR) = RO(2,3)
          ROCRYS(3,1,NCR) = RO(3,1)
          ROCRYS(3,2,NCR) = RO(3,2)
          ROCRYS(3,3,NCR) = RO(3,3)
  380   CONTINUE
C         
C---- If 1 crystal, copy symmetry etc to crystal 2
C
        IF (NCRYST.EQ.1) THEN
C
C---- If single hkl set set 2nd set of symm operations equal to first
C
          JSYM(2) = JSYM(1)
          JSYMP(2) = JSYMP(1)
C
C
          DO 410 JDO410 = 1,JSYMP(1)
            DO 400 JDO400 = 1,4
              DO 390 JDO390 = 1,4
                RSYM(JDO390,JDO400,JDO410,2) = RSYM(JDO390,JDO400,
     +            JDO410,1)
                RSYMT(JDO390,JDO400,JDO410,2) = RSYMT(JDO390,JDO400,
     +            JDO410,1)
  390         CONTINUE
  400       CONTINUE
  410     CONTINUE
C
C
          DO 430 JDO430 = 1,4
            DO 420 JDO420 = 1,4
              RFCRYS(JDO420,JDO430,2) = RFCRYS(JDO420,JDO430,1)
              ROCRYS(JDO420,JDO430,2) = ROCRYS(JDO420,JDO430,1)
  420       CONTINUE
  430     CONTINUE
C
C
          VOLCR(2) = VOLCR(1)
          NSYMZO(2) = NSYMZO(1)
          NCODES(2) = NCODES(1)
          LSPGRP(2) = LSPGRP(1)
        END IF
C
C---- Calculate orthogonal-space symmetry operations
C
C            ******
        CALL SYMORT
C            ******
C
        DO 450 JDO450 = 1,NCRYST
          IF (NSYMZO(JDO450).LT.0) THEN
C
C---- Work out rotational symmetry around Zortho if NSYMZO
C     has not been input
C
            M = 0
C
C
            DO 440 JDO440 = 1,JSYMP(JDO450)
C
C                  *****************************************
              CALL RORDER(ARSYM(1,1,JDO440,JDO450),3,NORDER)
C                  *****************************************
C
              M = MAX(M,NORDER)
  440       CONTINUE
C
C
            WRITE (6,FMT=6012) M,JDO450
 6012 FORMAT (//'  Rotational symmetry about Zortho set to ',I4,'  for',
     +       ' crystal ',I3,/)
            NSYMZO(JDO450) = M
          END IF
  450   CONTINUE
C
C
        IF (NCRYST.EQ.1) NSYMZO(2) = NSYMZO(1)
C
C---- Default title
C
        IF (LENSTR(TITLE).EQ.0 .AND. LPEAK.EQ.0) THEN
          IF (NCRYST.EQ.1) THEN
            TITLE = 'Self-rotation function'
          ELSE
            TITLE = 'Cross-rotation function'
          END IF
        END IF
C
C---- Print control data
C
C---- no input map  - We must calculate something...
C     Set print default if not set explicitly
C
        IF (LRDMP.EQ.0 .AND. LPEAK.EQ.0) THEN
          IF (NCRYST.EQ.1) THEN
            WRITE (6,FMT=6014) 'Self',TITLE(1:LENSTR(TITLE))
          ELSE
            WRITE (6,FMT=6014) 'Cross',TITLE(1:LENSTR(TITLE))
 6014 FORMAT ('1',20X,'Eulerian angle rotation function:  ',A,'-rotati',
     +       'on function',/21X,'=====================================',
     +       '=====================',//' Title:  ',A,//)
          END IF
C
C---- check spherical harmonic dimensions
C     We must know these dimensions for both
C     almnmn(called when lrdal=0)
C     and frfs subroutines.
C
          RAT2 = SPHMAX/RESMAX
          IF (RAT2.GT.12.0) THEN
            WRITE (6,FMT=6016) RESMAX,SPHMAX,SPHMAX/12.0
 6016 FORMAT (/'  !!! Warning !!! maximum resolution reset from',F10.3,
     +    /,' to integration radius ',F10.3,'/ 12.0: ie:',F10.3)
               CALL CCPERR(2,'Resolution limit reset')
C
C---- Reset maximum resolution to integration radius/12.0
C
            RESMAX = SPHMAX/12.0
          END IF
C
C---- 30 bessel functions sufficient if rat2 lt 5.83
C     5 degree beta search..
C
          IF (RAT2.LE.5.83) THEN
            NBES = 30
          ELSE
            NBES = 60
          END IF
C
C
          IF (NBES.EQ.60 .OR. ABS(BSTEP).LE.3.) THEN
            NBETAG = 72
          ELSE
            NBETAG = 36
          ENDIF
C
C
          STEP = 180.0/NBETAG
          ASTEP = STEP
          GSTEP = STEP
          WRITE (6,FMT=6018) SPHMAX,RESMIN,RESMAX,NBES
 6018 FORMAT (//'            Integration radius = ',F9.2,'A',/,
     +          '            Resolution limits  = ',2F9.2,/,
     +          ' Bessel function version used. = ',I5)
C
C---- step along alpha and gamma axes - 5deg for 30 bessel
C                                     - 2.5deg for 60 bessel
C
C---- Set alpha and gamma limits NALPHA AND NGAMMA SET IN RCARDSALMN
C
          AMIN = 0.0
          AMAX = 360.0/REAL(NSYMZO(1)) - STEP
          NALPHA = NINT(360.0/ (REAL(NSYMZO(1))*STEP))
          GMIN = 0.0
          GMAX = 360.0/REAL(NSYMZO(2)) - STEP
          NGAMMA = NINT(360.0/ (REAL(NSYMZO(2))*STEP))
C
C
          WRITE (6,FMT=6020) AMIN,AMAX,BTMIN,BTMAX,GMIN,GMAX,ASTEP,
     +      BSTEP,GSTEP
 6020 FORMAT (9X,'Range of alpha  ',2F10.2,/9X,'Range of Beta   ',
     +       2F10.2,/9X,'Range of Gamma  ',2F10.2,/'  Step Sizes ',
     +       3F10.2,'  Degrees',/)
        END IF
C
C---- Must have some information about spacegroup etc
C
        DO 460 JDO460 = 1,NCRYST
          WRITE (6,FMT=6021) JDO460,NCODES(JDO460),
     +      CODES(NCODES(JDO460)) 
 6021 FORMAT (/' Crystal',I3,': orthogonalisation code (NCODE) =',I2,
     +       '   x y z axes along ',A)
          WRITE (6,FMT=6022) ((RFCRYS(IS,JS,JDO460),JS=1,4),
     +      (ROCRYS(IS,JSS,JDO460),JSS=1,4),IS=1,4)
 6022 FORMAT (4 (/4F10.4,5X,4F10.4),/)
          WRITE (6,FMT=6023)
     +      LSPGRP(JDO460),NAMSGS(JDO460),JSYMP(JDO460)
 6023 FORMAT (' Space group ',I4,',   ',A,
     +       ' ,   ',I4,' primitive symmetry operations')
          WRITE (6,FMT=6024)
     +      (KS, ((RSYM(IS,JS,KS,JDO460),JS=1,4),IS=1,4),
     +      KS=1,JSYMP(JDO460))
 6024 FORMAT (/48 (/' Symmetry',I3,4 (/4F8.3),/))
  460   CONTINUE
C
C
        IF (LRDAL.EQ.0 .AND. LRDMP.EQ.0 .AND. LPEAK.EQ.0) THEN
C
C
          DO 470 JDO470 = 1,NCRYST
            WRITE (6,FMT=6025) JDO470,TFAC(JDO470),F2MIN(JDO470)
 6025 FORMAT (/' Crystal',I3,6X,'Temperature factor = ',F10.3,5X,'mini',
     +       'mum F =',F10.0,/)
  470     CONTINUE
C
C---- if almn calculations to be done check all this
C
          IF ((SPHMAX.GT. (CELCRY(1,1)-SPHMIN)) .OR.
     +        (SPHMAX.GT. (CELCRY(2,1)-SPHMIN)) .OR.
     +        (SPHMAX.GT. (CELCRY(3,1)-SPHMIN))) THEN
            CALL CCPERR(1,
     +        '  Sphere Radius too large - start again !!')
          ELSE IF (NCRYST.EQ.2 .AND. ((SPHMAX.GT. (CELCRY(1,
     +             2)-SPHMIN)).OR. (SPHMAX.GT. (CELCRY(2,
     +             2)-SPHMIN)).OR. (SPHMAX.GT. (CELCRY(3,
     +             2)-SPHMIN)))) THEN
            CALL CCPERR(1,
     +        '  Sphere Radius too large - start again !!')
          ELSE
C
C---- check sphere size against cell dimensions
C
            IF ((SPHMAX.GT. (CELCRY(1,1)/2.0)) .OR.
     +          (SPHMAX.GT. (CELCRY(2,1)/2.0)) .OR.
     +          (SPHMAX.GT. (CELCRY(3,1)/2.0))) WRITE (6,
     +          FMT=*) '  Sphere Radius > 0.5*cell edge. OK??'
            IF (NCRYST.EQ.2 .AND. ((SPHMAX.GT. (CELCRY(1,
     +          2)/2.0)).OR. (SPHMAX.GT. (CELCRY(2,
     +          2)/2.0)).OR. (SPHMAX.GT. (CELCRY(3,
     +          2)/2.0)))) WRITE (6,FMT=*
     +          ) '  Sphere Radius > 0.5*cell edge. OK??'
C
C---- check sphere volume against crystal assymetric unit volume.
C
            VOLSPH =  ( REAL(1.33333D00*PI) )*SPHMAX*SPHMAX*SPHMAX
            VOLAS1 = VOLCR(1)/JSYM(1)
            VOLAS2 = VOLCR(2)/JSYM(2)
             IF (NCRYST.EQ.1) WRITE (6,FMT=6026) VOLSPH,VOLAS1
 6026 FORMAT (' Sphere Volume',F9.0,/'  Asymmetric volume of Crystal 1',
     +       F9.0)
            IF (NCRYST.EQ.2) WRITE (6,FMT=6028) VOLSPH,VOLAS1,
     +          VOLAS2
 6028 FORMAT (' Sphere Volume',F9.0,/'  Asymmetric volume of Crystal 1',
     +       F9.0,/'  Asymmetric volume of Crystal 2',F9.0)
C
            IF (VOLSPH.GT.VOLAS1 .OR. VOLSPH.GT.VOLAS2) THEN
               WRITE (6, FMT=6030)
 6030 FORMAT (' Sphere Volume > Asymmetric volume of Crystal - OK??')
C  Rest sphare radius to cover 1/2 asymmetric unit unless logical LSPHERE set.
             IF(LSPHERE)  THEN
               SPHMAX = 
     +         (0.5*MIN(VOLAS1,VOLAS2) /(1.33333D00*PI))**0.3333
               WRITE (6, '(//,A,F9.2)')
     +         '  Sphere radius reset to:',SPHMAX
               CALL CCPERR(2,'Sphere radius reset')
             END IF
            END IF 
C
          END IF
        END IF
C
C
        IF (LRDAL.GT.0) WRITE (6,FMT=6032)
 6032 FORMAT (/' Output from ALMN stage will be read from file(s) :  a',
     +       'ssigned to coeffs1 (and coeffs2)')
C
C---- 1.  Check map reading correctly called.
C
        IF (LRDMP.GT.0) THEN
C
C---- is lplot > 0 or peak > 0
C
          IF (LPRINT.LT.-99 .AND. LPLOT.EQ.0 .AND. PEAK.EQ.0.0) THEN
            WRITE (6,FMT=6034)
 6034 FORMAT (/' No Reason to read map given !!!',/' Do you want to PR',
     +       'INT or PLOT or FIND peaks??')
            CALL CCPERR(1,' input error')
          ELSE
C
C---- READ map option
C
            WRITE (6,FMT=6036) TITLE(1:LENSTR(TITLE))
 6036 FORMAT (/' Previously calculated map will be read from MAPOUT',
     +       //' Title: ',A,/)
          END IF
        END IF
C
C---- FIND PEAK option
C
        IF (PEAK.NE.0.0) THEN
          WRITE (6,FMT=6038) MAXPEK
 6038 FORMAT (///'    ROTATION FUNCTION PEAK CALCULATION',//' All symm',
     +       'etry related peaks and matrices will be calculated for ',
     +       I4,'  peaks found in map ',/)
          IF (LPKRMS.EQ.0) THEN
            WRITE (6,FMT=6040) PEAK
 6040 FORMAT (' Peak threshold = ',F8.2,//)
          ELSE
            WRITE (6,FMT=6042) PEAK
 6042 FORMAT (' Peak threshold = ',F8.2,' * RMS density',//)
          END IF
        END IF
C
C---- INPUT PEAK option
C
        IF (LPEAK.GT.0) THEN
          WRITE (6,FMT=6044) NPEAK
 6044 FORMAT ('1    Rotation function peak calculation',///' All symme',
     +       'try related peaks and matrices will be calculated for ',
     +       I4,' input peaks',//)
          IF (JSYMP(1).EQ.0 .OR. JSYMP(2).EQ.0) THEN
            CALL CCPERR(1,
     +    '  **  symmetry card must be given AFTER peak card**')
          ELSE IF (CELCRY(1,1).EQ.0.0) THEN
            CALL CCPERR(1,
     +        '  **  cell card must be given AFTER peak card**')
          END IF
        END IF
C
C---- PRINT MAP option
C
        IF (LPRINT.GT.-100) WRITE (6,FMT=6046) LPRINT
 6046 FORMAT (///'    Map points higher than',I5,' printed.')
C
      END IF
C
C
      END

      BLOCK DATA ALMNBD
C     ==================
C
C
C     .. Arrays in Common ..
      REAL BZ60
C     ..
C     .. Common blocks ..
      COMMON /ALMN60/BZ60(60,30)
      COMMON /INALM/ LCR1,LCR2,LCO1,LCO2
       save
C     .. Data statements ..
      DATA LCR1,LCR2,LCO1,LCO2 /1,2,11,12/
C
      DATA (bz60(ii,1),ii=1,60)/
     +     4.4934,5.7635,6.9879,8.1826,9.3558,10.5128,11.6570,
     +     12.7908,13.9158,15.0335,16.1447,17.2505,18.3513,19.4477,
     +     20.5402,21.6292,22.7150,23.7978,24.8780,25.9557,27.0311,
     +     28.1043,29.1756,30.2450,31.3127,32.3787,33.4432,34.5063,
     +     35.5680,36.6284,37.6875,38.7455,39.8024,40.8583,41.9131,
     +     42.9669,44.0199,45.0719,46.1231,47.1735,48.2231,49.2719,
     +     50.3201,51.3682,52.4142,53.4603,54.5063,55.5507,56.5950,
     +     57.6394,58.6819,59.7243,60.7668,61.8082,62.8496,63.8900,
     +     64.9304,65.9708,67.0093,68.0478/
      DATA (bz60(ii,2),ii=1,60)/
     +     7.7253,9.0950,10.4171,
     +     11.7049,12.9665,14.2074,15.4313,16.6410,17.8386,19.0257,
     +     20.2038,21.3738,22.5367,23.6931,24.8437,25.9889,27.1293,
     +     28.2653,29.3971,30.5251,31.6496,32.7707,33.8888,35.0040,
     +     36.1164,37.2263,38.3338,39.4390,40.5420,41.6430,
     +     42.7420,43.8391,44.9345,46.0282,47.1202,48.2107,49.2997,
     +     50.3873,51.4735,52.5584,53.6420,54.7245,55.8057,56.8858,
     +     57.9648,59.0427,60.1196,61.1956,62.2706,63.3446,64.4177,
     +     65.4900,66.5614,67.6320,68.7019,69.7709,70.8392,71.9067,
     +     72.9735,74.0397/
      DATA (bz60(ii,3),ii=1,60)/
     +     10.9041,12.3229,13.6980,15.0395,16.3546,
     +     17.6479,18.9229,20.1824,21.4284,22.6626,23.8865,25.1010,
     +     26.3071,27.5057,28.6974,29.8828,31.0623,32.2365,33.4058,
     +     34.5704,35.7308,36.8871,38.0396,39.1886,40.3342,41.4766,
     +     42.6161,43.7527,44.8867,46.0180,47.1470,48.2736,49.3979,
     +     50.5202,51.6404,52.7587,53.8751,54.9896,56.1025,57.2137,
     +     58.3233,59.4313,60.5379,61.6430,62.7467,63.8491,64.9502,
     +     66.0500,67.1486,68.2460,69.3422,70.4373,71.5314,72.6244,
     +     73.7164,74.8074,75.8974,76.9865,78.0746,79.1619/
      DATA (bz60(ii,4),ii=1,60)/
     +     14.0662,15.5146,16.9236,18.3012,19.6531,20.9834,22.2953,
     +     23.5912,24.8732,26.1427,27.4012,28.6498,29.8893,31.1206,
     +     32.3444,33.5612,34.7717,35.9763,37.1753,38.3692,39.5582,
     +     40.7428,41.9230,43.0993,44.2718,45.4407,46.6062,47.7685,
     +     48.9277,50.0839,51.2374,52.3882,53.5365,54.6823,55.8257,
     +     56.9670,58.1060,59.2430,60.3779,61.5110,62.6422,63.7715,
     +     64.8992,66.0252,67.1495,68.2723,69.3935,70.5133,71.6317,
     +     72.7487,73.8643,74.9786,76.0917,77.2035,78.3142,79.4236,
     +     80.5319,81.6392,82.7453,83.8504/
      DATA (bz60(ii,5),ii=1,60)/
     +     17.2207,18.6890,20.1218,
     +     21.5254,22.9045,24.2627,25.6028,26.9270,28.2371,29.5346,
     +     30.8208,32.0967,33.3632,34.6211,35.8711,37.1139,38.3498,
     +     39.5795,40.8033,42.0215,43.2346,44.4428,45.6465,46.8457,
     +     48.0409,49.2322,50.4198,51.6039,52.7846,53.9621,
     +     55.1365,56.3080,57.4767,58.6428,59.8062,60.9671,62.1257,
     +     63.2819,64.4360,65.5879,66.7377,67.8856,69.0315,70.1755,
     +     71.3178,72.4583,73.5971,74.7343,75.8699,77.0039,78.1364,
     +     79.2675,80.3971,81.5254,82.6523,83.7779,84.9023,86.0254,
     +     87.1473,88.2680/
      DATA (bz60(ii,6),ii=1,60)/
     +     20.3713,21.8539,23.3042,24.7276,26.1277,
     +     27.5079,28.8704,30.2172,31.5502,32.8705,34.1795,35.4780,
     +     36.7670,38.0472,39.3193,40.5839,41.8415,43.0925,44.3374,
     +     45.5765,46.8102,48.0388,49.2626,50.4818,51.6966,52.9074,
     +     54.1141,55.3172,56.5167,57.7127,58.9055,60.0951,61.2818,
     +     62.4655,63.6465,64.8248,66.0006,67.1739,68.3448,69.5133,
     +     70.6797,71.8439,73.0060,74.1661,75.3242,76.4805,77.6349,
     +     78.7875,79.9384,81.0876,82.2352,83.3812,84.5257,85.6686,
     +     86.8101,87.9501,89.0888,90.2261,91.3621,92.4968/
      DATA (bz60(ii,7),ii=1,60)/
     +     23.5195,25.0128,26.4768,27.9156,29.3326,30.7304,32.1112,
     +     33.4768,34.8287,36.1681,37.4963,38.8140,40.1221,41.4214,
     +     42.7124,43.9958,45.2721,46.5416,47.8049,49.0622,50.3139,
     +     51.5603,52.8018,54.0384,55.2706,56.4985,57.7222,58.9421,
     +     60.1582,61.3707,62.5798,63.7856,64.9882,66.1878,67.3845,
     +     68.5783,69.7695,70.9580,72.1440,73.3275,74.5087,75.6876,
     +     76.8643,78.0388,79.2113,80.3817,81.5502,82.7168,83.8816,
     +     85.0445,86.2057,87.3652,88.5231,89.6793,90.8340,91.9871,
     +     93.1388,94.2890,95.4378,96.5852/
      DATA (bz60(ii,8),ii=1,60)/
     +     26.6661,28.1678,29.6426,
     +     31.0939,32.5247,33.9371,35.3332,36.7145,38.0825,39.4382,
     +     40.7827,42.1170,43.4416,44.7574,46.0650,47.3648,48.6574,
     +     49.9432,51.2226,52.4960,53.7636,55.0259,56.2830,57.5353,
     +     58.7829,60.0261,61.2650,62.4999,63.7309,64.9582,
     +     66.1820,67.4023,68.6194,69.8333,71.0441,72.2520,73.4571,
     +     74.6594,75.8591,77.0562,78.2509,79.4432,80.6331,81.8208,
     +     83.0063,84.1897,85.3710,86.5503,87.7277,88.9032,90.0769,
     +     91.2488,92.4189,93.5873,94.7541,95.9192,97.0828,98.2448,
     +     99.4054,100.5645/
      DATA (bz60(ii,9),ii=1,60)/
     +     29.8116,31.3201,32.8037,34.2654,35.7076,
     +     37.1323,38.5414,39.9361,41.3179,42.6876,44.0464,45.3950,
     +     46.7341,48.0644,49.3865,50.7008,52.0079,53.3082,54.6020,
     +     55.8897,57.1716,58.4480,59.7193,60.9855,62.2470,63.5040,
     +     64.7567,66.0052,67.2497,68.4904,69.7275,70.9611,72.1912,
     +     73.4181,74.6419,75.8626,77.0804,78.2953,79.5076,80.7171,
     +     81.9241,83.1286,84.3307,85.5305,86.7280,87.9233,89.1164,
     +     90.3075,91.4965,92.6836,93.8687,95.0520,96.2334,97.4131,
     +     98.5910,99.7673,100.9419,102.1148,103.2863,104.4561/
      DATA (bz60(ii,10),ii=1,60)/
     +     32.9564,34.4705,35.9614,37.4317,38.8836,40.3189,41.7391,
     +     43.1454,44.5391,45.9212,47.2925,48.6537,50.0056,51.3488,
     +     52.6837,54.0110,55.3311,56.6443,57.9510,59.2516,60.5463,
     +     61.8356,63.1195,64.3985,65.6726,66.9421,68.2073,69.4682,
     +     70.7250,71.9780,73.2273,74.4729,75.7151,76.9539,78.1895,
     +     79.4220,80.6515,81.8781,83.1018,84.3229,85.5412,86.7571,
     +     87.9704,89.1813,90.3899,91.5962,92.8002,94.0021,95.2020,
     +     96.3997,97.5955,98.7894,99.9813,101.1714,102.3598,103.5463,
     +     104.7312,105.9143,107.0959,108.2758/
      DATA (bz60(ii,11),ii=1,60)/
     +     36.1006,37.6194,39.1165,
     +     40.5942,42.0544,43.4988,44.9286,46.3451,47.7493,49.1422,
     +     50.5245,51.8970,53.2603,54.6149,55.9615,57.3004,58.6321,
     +     59.9570,61.2754,62.5877,63.8941,65.1950,66.4905,67.7810,
     +     69.0667,70.3477,71.6242,72.8965,74.1647,75.4289,
     +     76.6894,77.9462,79.1994,80.4492,81.6958,82.9392,84.1795,
     +     85.4168,86.6512,87.8829,89.1118,90.3381,91.5619,92.7831,
     +     94.0020,95.2185,96.4328,97.6448,98.8547,100.0624,101.2681,
     +     102.4719,103.6736,104.8735,106.0715,107.2677,108.4621,
     +     109.6548,110.8458,112.0352/
      DATA (bz60(ii,12),ii=1,60)/
     +     39.2444,40.7671,42.2695,43.7536,
     +     45.2211,46.6733,48.1117,49.5371,50.9507,52.3532,53.7453,
     +     55.1279,56.5014,57.8664,59.2233,60.5728,61.9150,63.2505,
     +     64.5796,65.9025,67.2196,68.5311,69.8373,71.1384,72.4346,
     +     73.7261,75.0132,76.2960,77.5746,78.8492,80.1200,81.3871,
     +     82.6506,83.9106,85.1673,86.4208,87.6711,88.9184,90.1628,
     +     91.4043,92.6431,93.8792,95.1126,96.3436,97.5721,98.7982,
     +     100.0219,101.2434,102.4627,103.6798,104.8948,106.1077,
     +     107.3187,108.5277,109.7348,110.9401,112.1435,113.3451,
     +     114.5450,115.7432/
      DATA (bz60(ii,13),ii=1,60)/
     +     42.3879,43.9140,45.4210,46.9106,48.3844,49.8437,51.2895,
     +     52.7229,54.1448,55.5559,56.9569,58.3485,59.7312,61.1056,
     +     62.4720,63.8310,65.1829,66.5281,67.8668,69.1995,70.5263,
     +     71.8476,73.1636,74.4745,75.7804,77.0817,78.3785,79.6710,
     +     80.9593,82.2435,83.5239,84.8006,86.0736,87.3431,88.6093,
     +     89.8721,91.1319,92.3885,93.6421,94.8929,96.1409,97.3861,
     +     98.6287,99.8687,101.1062,102.3412,103.5739,104.8042,106.0323,
     +     107.2582,108.4820,109.7036,110.9233,112.1409,113.3565,
     +     114.5703,115.7822,116.9923,118.2006,119.4071/
      DATA (bz60(ii,14),ii=1,60)/
     +     45.5311,47.0601,
     +     48.5711,50.0657,51.5451,53.0105,54.4630,55.9036,57.3329,
     +     58.7517,60.1608,61.5606,62.9517,64.3346,65.7097,67.0774,
     +     68.4381,69.7921,71.1398,72.4815,73.8173,75.1476,76.4726,
     +     77.7925,79.1075,80.4179,81.7237,83.0251,84.3224,85.6156,
     +     86.9050,88.1905,89.4725,90.7508,92.0258,93.2975,94.5660,
     +     95.8313,97.0937,98.3531,99.6097,100.8635,102.1146,103.3632,
     +     104.6091,105.8526,107.0937,108.3324,109.5688,110.8030,
     +     112.0350,113.2648,114.4926,115.7183,116.9421,118.1639,
     +     119.3838,120.6019,121.8181,123.0325/
      DATA (bz60(ii,15),ii=1,60)/
     +     48.6741,50.2057,51.7202,
     +     53.2191,54.7035,56.1745,57.6330,59.0800,60.5160,61.9419,
     +     63.3582,64.7655,66.1642,67.5549,68.9379,70.3136,71.6825,
     +     73.0447,74.4006,75.7505,77.0947,78.4334,79.7667,81.0950,
     +     82.4185,83.7372,85.0514,86.3613,87.6669,88.9685,90.2662,
     +     91.5601,92.8504,94.1371,95.4203,96.7003,97.9770,99.2505,
     +     100.5211,101.7886,103.0533,104.3152,105.5744,106.8309,
     +     108.0849,109.3363,110.5853,111.8319,113.0762,114.3182,
     +     115.5580,116.7956,118.0311,119.2645,120.4959,121.7254,
     +     122.9529,124.1785,125.4022,126.6242/
      DATA (bz60(ii,16),ii=1,60)/
     +     51.8170,53.3508,54.8685,56.3712,57.8601,59.3360,60.8000,
     +     62.2527,63.6949,65.1272,66.5501,67.9642,69.3700,70.7678,
     +     72.1581,73.5412,74.9175,76.2873,77.6509,79.0085,80.3604,
     +     81.7068,83.0480,84.3841,85.7153,87.0419,88.3640,89.6817,
     +     90.9952,92.3047,93.6102,94.9119,96.2100,97.5045,98.7955,
     +     100.0832,101.3677,102.6490,103.9272,105.2024,106.4748,
     +     107.7443,109.0111,110.2752,111.5367,112.7957,114.0522,
     +     115.3063,116.5580,117.8074,119.0546,120.2995,121.5424,
     +     122.7831,124.0218,125.2584,126.4931,127.7259,128.9568,
     +     130.1859/
      DATA (bz60(ii,17),ii=1,60)/
     +     54.9597,56.4956,58.0160,59.5222,61.0151,62.4956,
     +     63.9645,65.4225,66.8702,68.3084,69.7374,71.1577,72.5699,
     +     73.9744,75.3714,76.7613,78.1446,79.5214,80.8920,82.2568,
     +     83.6159,84.9695,86.3180,87.6614,88.9999,90.3339,91.6633,
     +     92.9884,94.3092,95.6261,
     +     96.9389,98.2480,99.5535,100.8553,102.1537,103.4487,
     +     104.7405,106.0291,107.3146,108.5971,109.8767,111.1534,
     +     112.4274,113.6987,114.9674,116.2335,117.4971,118.7582,
     +     120.0170,121.2734,122.5276,123.7796,125.0294,126.2770,
     +     127.5226,128.7662,130.0078,131.2474,132.4851,133.7209/
      DATA (bz60(ii,18),ii=1,60)/
     +     58.1023,59.6400,61.1629,62.6722,64.1688,65.6534,67.1267,
     +     68.5896,70.0424,71.4859,72.9205,74.3467,75.7648,77.1754,
     +     78.5786,79.9749,81.3646,82.7479,84.1252,85.4966,86.8624,
     +     88.2228,89.5780,90.9283,92.2737,93.6145,94.9509,96.2829,
     +     97.6107,98.9344,100.2543,101.5703,102.8827,104.1914,105.4968,
     +     106.7987,108.0974,109.3929,110.6853,111.9746,113.2611,
     +     114.5447,115.8255,117.1036,118.3791,119.6519,120.9223,
     +     122.1902,123.4557,124.7188,125.9797,127.2383,128.4947,
     +     129.7490,131.0012,132.2513,133.4994,134.7455,135.9898,
     +     137.2321/
      DATA (bz60(ii,19),ii=1,60)/
     +     61.2447,62.7841,64.3093,65.8215,67.3213,68.8097,70.2871,
     +     71.7544,73.2120,74.6604,76.1002,77.5317,78.9553,80.3715,
     +     81.7805,83.1828,84.5784,85.9678,87.3513,88.7289,90.1010,
     +     91.4678,92.8294,94.1861,95.5380,96.8852,98.2281,99.5666,
     +     100.9009,102.2312,103.5576,104.8802,106.1991,107.5145,
     +     108.8263,110.1348,111.4401,112.7421,114.0410,115.3369,
     +     116.6299,117.9200,119.2073,120.4919,121.7738,123.0531,
     +     124.3299,125.6042,126.8761,128.1456,129.4129,130.6778,
     +     131.9406,133.2011,134.4596,135.7160,136.9704,138.2227,
     +     139.4732,140.7217/
      DATA (bz60(ii,20),ii=1,60)/
     +     64.3871,65.9279,67.4553,68.9700,70.4729,
     +     71.9647,73.4459,74.9172,76.3791,77.8322,79.2767,80.7132,
     +     82.1419,83.5634,84.9778,86.3855,87.7868,89.1819,90.5711,
     +     91.9546,93.3326,94.7053,96.0729,97.4357,98.7936,100.1470,
     +     101.4960,102.8406,104.1812,105.5176,
     +     106.8502,108.1790,109.5042,110.8257,112.1438,113.4585,
     +     114.7700,116.0782,117.3834,118.6855,119.9846,121.2809,
     +     122.5744,123.8652,125.1532,126.4387,127.7216,129.0021,
     +     130.2801,131.5557,132.8290,134.1000,135.3688,136.6354,
     +     137.8999,139.1623,140.4226,141.6809,142.9373,144.1917/
      DATA (bz60(ii,21),ii=1,60)/
     +     67.5294,69.0716,70.6008,72.1179,73.6236,75.1185,76.6032,
     +     78.0783,79.5443,81.0015,82.4505,83.8916,85.3251,86.7515,
     +     88.1710,89.5838,90.9904,92.3908,93.7854,95.1744,96.5579,
     +     97.9363,99.3095,100.6780,102.0417,103.4008,104.7556,106.1061,
     +     107.4524,108.7948,110.1332,111.4679,112.7989,114.1264,
     +     115.4504,116.7710,118.0884,119.4025,120.7136,122.0216,
     +     123.3267,124.6289,125.9282,127.2249,128.5188,129.8102,
     +     131.0989,132.3852,133.6690,134.9505,136.2296,137.5064,
     +     138.7810,140.0534,141.3236,142.5917,143.8578,145.1219,
     +     146.3839,147.6440/
      DATA (bz60(ii,22),ii=1,60)/
     +     70.6710,72.2151,73.7461,75.2653,76.7736,78.2714,79.7593,
     +     81.2379,82.7076,84.1688,85.6219,87.0673,88.5053,89.9362,
     +     91.3604,92.7781,94.1896,95.5950,96.9947,98.3889,99.7777,
     +     101.1613,102.5399,103.9137,105.2828,106.6474,108.0077,
     +     109.3637,110.7156,112.0635,113.4075,114.7478,116.0844,
     +     117.4175,118.7471,120.0734,121.3964,122.7162,124.0328,
     +     125.3465,126.6572,127.9650,129.2700,130.5723,131.8718,
     +     133.1688,134.4631,135.7550,137.0444,138.3314,139.6161,
     +     140.8985,142.1786,143.4565,144.7323,146.0059,147.2775,
     +     148.5470,149.8145,151.0801/
      DATA (bz60(ii,23),ii=1,60)/
     +     73.8139,75.3579,76.8910,78.4123,
     +     79.9229,81.4234,82.9143,84.3961,85.8692,87.3341,88.7911,
     +     90.2405,91.6827,93.1180,94.5466,95.9688,97.3849,98.7951,
     +     100.1996,101.5986,102.9923,104.3809,105.7646,107.1435,
     +     108.5178,109.8876,111.2530,112.6142,113.9714,115.3246,
     +     116.6739,118.0195,119.3615,120.6999,122.0349,123.3665,
     +     124.6949,126.0200,127.3421,128.6611,129.9772,131.2904,
     +     132.6008,133.9084,135.2133,136.5156,137.8154,139.1126,
     +     140.4074,141.6997,142.9897,144.2774,145.5629,146.8461,
     +     148.1272,149.4061,150.6829,151.9577,153.2305,154.5013/
      DATA (bz60(ii,24),ii=1,60)/
     +     76.9555,78.5016,80.0356,81.5588,83.0716,84.5746,86.0683,
     +     87.5531,89.0295,90.4978,91.9584,93.4116,94.8577,96.2970,
     +     97.7298,99.1563,100.5767,101.9914,103.4004,104.8040,106.2024,
     +     107.5958,108.9842,110.3679,111.7471,113.1218,114.4922,
     +     115.8584,117.2205,118.5788,119.9332,121.2838,122.6309,
     +     123.9744,125.3145,126.6512,127.9847,129.3150,130.6422,
     +     131.9664,133.2876,134.6059,135.9215,137.2342,138.5443,
     +     139.8518,141.1567,142.4590,143.7589,145.0564,146.3516,
     +     147.6444,148.9349,150.2232,151.5094,152.7934,154.0753,
     +     155.3552,156.6330,157.9089/
      DATA (bz60(ii,25),ii=1,60)/ 
     +     80.0981,81.6443,83.1801,84.7049,86.2198,87.7251,89.2214,
     +     90.7090,92.1884,93.6600,95.1239,96.5807,98.0305,99.4736,
     +     100.9103,102.3408,103.7654,105.1843,106.5976,108.0056,
     +     109.4084,110.8062,112.1992,113.5875,114.9713,116.3507,
     +     117.7258,119.0967,120.4636,121.8266,123.1858,124.5413,
     +     125.8932,127.2416,128.5866,129.9282,131.2666,132.6018,
     +     133.9339,135.2630,136.5892,137.9125,139.2329,140.5507,
     +     141.8657,143.1781,144.4879,145.7952,147.1000,148.4024,
     +     149.7025,151.0002,152.2957,153.5889,154.8799,156.1688,
     +     157.4556,158.7404,160.0231,161.3038/
      DATA (bz60(ii,26),ii=1,60)/ 
     +     83.2397,84.7876,86.3234,
     +     87.8507,89.3675,90.8750,92.3737,93.8640,95.3462,96.8208,
     +     98.2879,99.7480,101.2013,102.6480,104.0884,105.5227,106.9512,
     +     108.3741,109.7915,111.2036,112.6106,114.0127,115.4100,
     +     116.8026,118.1908,119.5746,120.9542,122.3297,123.7011,
     +     125.0687,
     +     126.4325,127.7926,129.1492,130.5022,131.8518,133.1982,
     +     134.5412,135.8812,137.2180,138.5518,139.8827,141.2107,
     +     142.5359,143.8584,145.1781,146.4953,147.8098,149.1218,
     +     150.4314,151.7386,153.0434,154.3458,155.6460,156.9440,
     +     158.2397,159.5333,160.8248,162.1143,163.4017,164.6871/
      DATA (bz60(ii,27),ii=1,60)/ 
     +     86.3822,87.9301,89.4683,90.9956,92.5148,94.0244,95.5253,
     +     97.0181,98.5030,99.9804,101.4505,102.9137,104.3703,105.8204,
     +     107.2643,108.7023,110.1344,111.5611,112.9823,114.3984,
     +     115.8094,117.2155,118.6169,120.0137,121.4061,122.7941,
     +     124.1780,125.5577,126.9335,128.3055,129.6737,131.0382,
     +     132.3992,133.7567,135.1108,136.4616,137.8092,139.1536,
     +     140.4950,141.8333,143.1687,144.5013,145.8310,147.1580,
     +     148.4823,149.8040,151.1232,152.4398,153.7539,155.0656,
     +     156.3749,157.6820,158.9867,160.2892,161.5895,162.8877,
     +     164.1837,165.4777,166.7696,168.0596/
      DATA (bz60(ii,28),ii=1,60)/ 
     +     89.5238,91.0732,92.6115,94.1415,95.6617,97.1732,98.6763,
     +     100.1714,101.6588,103.1389,104.6118,106.0780,107.5377,
     +     108.9910,110.4382,111.8796,113.3153,114.7455,116.1704,
     +     117.5902,119.0050,120.4149,121.8203,123.2210,124.6174,
     +     126.0095,127.3974,128.7813,130.1613,131.5374,132.9098,
     +     134.2785,135.6437,137.0055,138.3639,139.7190,141.0709,
     +     142.4197,143.7654,145.1081,146.4479,147.7848,149.1189,
     +     150.4503,151.7790,153.1051,154.4286,155.7496,157.0681,
     +     158.3842,159.6979,161.0093,162.3185,163.6253,164.9300,
     +     166.2326,167.5330,168.8314,170.1277,171.4220/
      DATA (bz60(ii,29),ii=1,60)/ 
     +     92.6654,94.2159,
     +     95.7559,97.2860,98.8083,100.3216,101.8267,103.3240,104.8137,
     +     106.2963,107.7720,109.2410,110.7035,112.1599,113.6103,
     +     115.0549,116.4939,117.9275,119.3559,120.7793,122.1977,
     +     123.6113,125.0204,126.4249,127.8251,129.2211,130.6129,
     +     132.0007,133.3846,134.7648,
     +     136.1412,137.5140,138.8833,140.2491,141.6116,142.9709,
     +     144.3269,145.6798,147.0297,148.3766,149.7206,151.0617,
     +     152.4000,153.7356,155.0685,156.3988,157.7266,159.0518,
     +     160.3746,161.6949,163.0129,164.3285,165.6419,166.9530,
     +     168.2620,169.5687,170.8734,172.1760,173.4765,174.7750/
      DATA (bz60(ii,30),ii=1,60)/ 
     +     95.8081,97.3587,98.8986,100.4314,101.9537,103.4695,104.9765,
     +     106.4759,107.9679,109.4529,110.9311,112.4027,113.8681,
     +     115.3273,116.7807,118.2284,119.6706,121.1074,122.5391,
     +     123.9659,125.3877,126.8049,128.2175,129.6256,131.0295,
     +     132.4291,133.8247,135.2163,136.6040,137.9880,139.3683,
     +     140.7450,142.1182,143.4879,144.8544,146.2176,147.5776,
     +     148.9346,150.2884,151.6393,152.9874,154.3325,155.6749,
     +     157.0146,158.3516,159.6859,161.0177,162.3470,163.6739,
     +     164.9983,166.3204,167.6401,168.9576,170.2728,171.5858,
     +     172.8967,174.2055,175.5121,176.8168,178.1194/
C     ..                                            
C
C
      END                                                     
      SUBROUTINE AHVSOR(IA,KEYS,ITEM,ILIM,IDIM)
C     =========================================
C
C
C---- ahvsor taken from cadlcf
C     returns ia(...) in ascending order
C
C
C     .. Scalar Arguments ..
      INTEGER IDIM,ILIM,ITEM,KEYS
C     ..
C     .. Array Arguments ..
      INTEGER IA(IDIM)
C     ..
C     .. Local Scalars ..
      INTEGER BOTTOM,CHUNK,CUT,CUTOFF,DOWN,HIGH,I,IP1,ITEMP,LOW,M,
     +        MIDDLE,MM,MMM,N,NMIN,STACK,TOP,UP
C     ..
C     .. Local Arrays ..
      INTEGER IP(10),SAVEIT(2,16)
C     ..
C     .. Data statements ..
      DATA CUTOFF/20/
C     ..               
C
C
C
      DO 10 I = 1,ITEM
        IP(I) = (I-1)*ILIM
   10 CONTINUE
C
      STACK = 0
      LOW = 1
      HIGH = KEYS
   20 CONTINUE
      CHUNK = HIGH - LOW
      IF (CHUNK.GT.CUTOFF) THEN
        MIDDLE = (LOW+HIGH)/2
        IF (IA(LOW).GT.IA(MIDDLE)) THEN
          DO 30 I = 1,ITEM
            IP1 = IP(I)
            ITEMP = IA(LOW+IP1)
            IA(LOW+IP1) = IA(MIDDLE+IP1)
            IA(MIDDLE+IP1) = ITEMP
   30     CONTINUE
        END IF
        IF (IA(MIDDLE).GT.IA(HIGH)) THEN
          DO 40 I = 1,ITEM
            IP1 = IP(I)
            ITEMP = IA(MIDDLE+IP1)
            IA(MIDDLE+IP1) = IA(HIGH+IP1)
            IA(HIGH+IP1) = ITEMP
   40     CONTINUE
          IF (IA(LOW).GT.IA(MIDDLE)) THEN
            DO 50 I = 1,ITEM
              IP1 = IP(I)
              ITEMP = IA(LOW+IP1)
              IA(LOW+IP1) = IA(MIDDLE+IP1)
              IA(MIDDLE+IP1) = ITEMP
   50       CONTINUE
          END IF
        END IF
        CUT = IA(MIDDLE)
        UP = LOW
        DOWN = HIGH
   60   CONTINUE
        DOWN = DOWN - 1
        IF (CUT.LT.IA(DOWN)) THEN
          GO TO 60
        ELSE
   70     CONTINUE
          UP = UP + 1
          IF (CUT.GT.IA(UP)) GO TO 70
          IF ((UP-DOWN).NE.0) THEN
            IF ((UP-DOWN).GT.0) THEN
              GO TO 90
            ELSE
              DO 80 I = 1,ITEM
                IP1 = IP(I)
                ITEMP = IA(UP+IP1)
                IA(UP+IP1) = IA(DOWN+IP1)
                IA(DOWN+IP1) = ITEMP
   80         CONTINUE
              GO TO 60
            END IF
          END IF
        END IF
        DOWN = DOWN - 1
        UP = UP + 1
   90   STACK = STACK + 1
        IF (STACK.LE.16) THEN
          BOTTOM = DOWN - LOW
          TOP = HIGH - UP
          IF (BOTTOM.GT.TOP) THEN
            SAVEIT(1,STACK) = LOW
            SAVEIT(2,STACK) = DOWN
            LOW = UP
          ELSE
            SAVEIT(1,STACK) = UP
            SAVEIT(2,STACK) = HIGH
            HIGH = DOWN
          END IF
          GO TO 20
        END IF
      ELSE
        IF (CHUNK.GE.2) THEN
          MM = HIGH - 1
          DO 120 M = LOW,MM
            NMIN = M
            MMM = M + 1
            DO 100 N = MMM,HIGH
              IF (IA(NMIN).GT.IA(N)) NMIN = N
  100       CONTINUE
            IF (NMIN.NE.M) THEN
              DO 110 I = 1,ITEM
                IP1 = IP(I)
                ITEMP = IA(M+IP1)
                IA(M+IP1) = IA(NMIN+IP1)
                IA(NMIN+IP1) = ITEMP
  110         CONTINUE
            END IF
  120     CONTINUE
        END IF
        IF (STACK.EQ.0) THEN
          RETURN
        ELSE
          LOW = SAVEIT(1,STACK)
          HIGH = SAVEIT(2,STACK)
          STACK = STACK - 1
          GO TO 20
        END IF
      END IF
      WRITE (6,FMT=6000) STACK
      CALL CCPERR(1,' STACK error')
 6000 FORMAT (' AHVSOR ERROR **** STACK ',I10,' EXCEEDS 16')
C
C
      END
C
C                                
C
      SUBROUTINE ALMNMN
C     =================
C
C     .. Parameters ..
      INTEGER NPARM
      PARAMETER (NPARM=200)
      INTEGER NREFMX,NBESMX,NBTGMX
      PARAMETER (NREFMX=10000,NBESMX=60,NBTGMX=72)
      INTEGER NCLIM,NSLIM
      PARAMETER (NCLIM=80000,NSLIM=80000)
      INTEGER MAXNPK
      PARAMETER (MAXNPK=100)
cc
cc     dble - convert to double precision
cc Converts a number to a real*8 real number.
cc dble-val.as.r8 = dble (number.as.irc)
cc For an integer argument, the result is the same value (a fraction of 0.0
cc is appended). For a real argument, the result is the same.  For a complex
cc argument, the imaginary part is dropped.
cc A real number that is not fully represented in binary in single precision
cc may lose precision during a conversion to double precision. The converted
cc double precision number fills out the mantissa with zeros while the true
cc double precision number fills out the mantissa with significant bits.
cc
cc     *** standardization note ***
cc     The specific function names listed below are extensions.
cc          Specific    Arg Type    Result Type Generic
cc          dble        real*4      real*8      dble
cc          dreal       complex*16  real*8      dble
cc
cc  *** extension ***
cc  The use of 8-byte integer or logical values affects the behavior of many
cc  generic intrinsic functions. See Appendix I for details.
cc
cc     real - convert to real
cc     Converts a number to a real*4 real number.
cc     real-val.as.r4 = real (number.as.irc)
cc For an integer argument, the result is the same value (a fraction of 0.0
cc is appended). For a real argument, the result is the same for single pre-
cc cision and is rounded for double precision. For a complex argument, the
cc imaginary part is dropped.
cc
cc     *** standardization note ***
cc The specific function names floati and floatj, listed below, are exten-
cc sions. The specific function name real has a different meaning than in
cc Fortran-77. Fortran-77 lists as specific names under real: real (integer
cc to real), float (integer to real), and sngl (double precision to real).
cc          Specific    Arg Type    Result Type Generic
cc          floati      integer*2   real*4      real
cc          floatj      integer*4   real*4      real
cc          sngl        real*8      real*4      real
cc          real        complex*8   real*4      real
cc
cc     *** extension ***
cc  The use of 8-byte integer or logical values affects the behavior of many
cc  generic intrinsic functions. See Appendix I for details.
cc
      DOUBLE PRECISION PI
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)             
      PARAMETER (PI=3.141592653589793D00)
      REAL TWOPI
cc    This intrinsic function is illegal in Parameter statement
cc      parameter (twopi= real(2.0d00*pi) )
cc
C     ..
C     .. Scalars in Common ..
      REAL ASTEP,BSTEP,BTMAX,BTMIN,GSTEP,PEAK,RESMAX,RESMIN,SMAX,SMIN,
     +     SPHMAX,SPHMIN
      INTEGER IRSYGP,LCO1,LCO2,LCR1,LCR2,LMAP,LPEAK,LPKOUT,
     +        LPKRMS,LPLOT,LPRINT,LRDAL,LRDMP,LSAVE,LSEC1,LSEC2,MAXPEK,
     +        NALPHA,NBES,NBETA,NBETAG,NCRYST,NGAMMA,NPEAK,NREF1,NREF2,
     +        lrefmx
      CHARACTER TITLE*80
C     ..                             
C     .. Arrays in Common ..
      REAL ARSYM,BZ60,CELCRY,F2MAX,
     +     F2MIN,F2MINS,PEAKS,PKPOS,
     +     RFCRYS,ROCRYS,RSYM,RSYMT,SCALFO,TFAC,VOLCR
      INTEGER JSYMP,LPKORD,LSPGRP,NCODES,
     +        NSYMZO
C     ..
C     .. Local Scalars ..
      REAL A2PI,AM2PI,ARGMAX,BMZ,BZ,CPH,DR1,DR2,EFACT,
     +     EXPON,FAC,FAC1,FAC2,FAC3,FAC3BZ,FAC4,FF,FMAC4,PHASE,
     +     PIBY2,PIBY2L,RANG1,RANG2,RCBMAX,RIMAX,RIMIN,RMAX,RMMAX,
     +     RSQ,SCALIO,SPH,STEP,TEMP,XH,XING,XK,XL,XX,XYSQ,YING,YY,ZZ
      real amin,gmin,sqrtl2
      INTEGER ICB,IFAIL,IH,IK,IKL,IL,INDEXY,IPASS,J,JDO10,JDO110,
     +        JDO120,JDO130,JDO140,JDO160,JDO180,JDO20,JDO210,JDO220,
     +        JDO230,JDO240,JDO250,JDO260,JDO280,JDO30,JDO320,JDO330,
     +        JDO340,JDO350,JDO360,JDO370,JDO40,JDO50,JDO70,JDO80,JDO90,
     +        L,L1,L21,LDUM,LMAX,LMBLK2,LMBLOK,LMMAX,LNBLK1,NNCR,
     +        LNBLK2,M,M1,MMAX,MREF,MULT,N,NANG,NB1,NBASA,NBASB,NBASC,
     +        NBASD,NBES2,NBESB2,NCH,NCR,NEWH,NEWK,NEWL,NMN,NN,NREF,
     +        NREFIP,NSLM4,NSYMM,NTMAX,nchck
      LOGICAL EOF
      CHARACTER STATUS*8
C     ..
C     .. Local Arrays ..
      COMPLEX FMM(NBTGMX,2*NBTGMX)
      REAL RI2(4*NBTGMX*NBTGMX),RMAP(2* (NBTGMX+2),2* (NBTGMX+2),3)
      REAL ADATA(100)
      LOGICAL LOGMSS(100)
C     ..
C     .. External Subroutines ..
      REAL FSBES
      EXTERNAL ANGFX2,CCPDPN,FRFS,MAPRNT,MAXVAL,PEAKPK,RCARDS,
     +         LRREFF,LRREWD,SPHBES,FSBES
C
C---- Block data external
C
      EXTERNAL ALMNBD
C     ..                                                   
C     .. Intrinsic Functions ..
      INTRINSIC ATAN2,COS,EXP,INT,LOG,MOD,REAL,SIN,SQRT
C     ..
C     .. Common blocks ..
      COMMON /ALMN60/BZ60(60,30)
      COMMON /INALM/ LCR1,LCR2,LCO1,LCO2
      COMMON /BESSPH/SPHMIN,SPHMAX,NBES,NBETAG
      COMMON /CRYSTS/NCRYST,ROCRYS(4,4,2),RFCRYS(4,4,2),CELCRY(6,2),
     +       NCODES(2),LSPGRP(2),VOLCR(2),TFAC(2),NSYMZO(2),F2MIN(2),
     +       SCALFO(2),F2MAX(2),F2MINS(2),NREF1,NREF2
      COMMON /PARAMS/LPRINT,LMAP,LPLOT,LRDMP,LRDAL,PEAK,MAXPEK,LSAVE,
     +       LPKRMS,LREFMX(2)
      COMMON /PEKPK/NPEAK,PEAKS(MAXNPK),PKPOS(3,MAXNPK),LPEAK,
     +       LPKORD(MAXNPK),LPKOUT
      COMMON /RANGE/BTMIN,BTMAX,ASTEP,BSTEP,GSTEP,NALPHA,NBETA,NGAMMA,
     +       IRSYGP,LSEC1,LSEC2
      COMMON /RES/RESMAX,RESMIN,SMIN,SMAX
cc
cc--- trash scratch common
cc
      REAL   F2MOD(NREFMX), F2BES(NREFMX), RHSQ(NREFMX),
     +       SINTH(NREFMX), COTTH(NREFMX), PHI(NREFMX),
     +       PLM1(NREFMX),  PLM2(NREFMX),  
     +       FACT(2*NBESMX+1), ALMN(NBESMX+1,NBESMX/2),
     +       BLMN(NBESMX+1,NBESMX/2),BZEROS(0:NBESMX,NBESMX/2),
     +       F2MBES(NREFMX),  FKP(15)
      INTEGER ITEMP(NREFMX), NMAX(0:NBESMX), NMMAX(0:NBESMX),
     +        JKP(15),       NRF(20),KK(72),IKK(72)
      REAL SBES
      COMMON /COMBES/ SBES(0:NBESMX,10*NBESMX)
c
c
      COMMON /SYMTR/JSYMP(2),RSYM(4,4,MAXSYM,2),RSYMT(4,4,MAXSYM,2),
     +       ARSYM(3,3,MAXSYM,2)
      COMMON /TITL/TITLE
C     ..
      SAVE
c
c
      TWOPI= REAL(2.0D00*PI)
C
C---- Crystal intensities
C
      LCR1 = 1
      LCR2 = 2
C
C---- Files 11 and 12 will contain series coeffs almn
C
      LCO1 = 11
      LCO2 = 12
C             
C---- Read all control input
C
C          *****************
      CALL RCARDS(AMIN,GMIN)
C          *****************
C             
C---- If LRDMP set - read a pre-calculated map
C     If LPEAK SET - just call angfx to generate
C                    symmetry equivalents of input peaks
C
      NANG = NALPHA*NGAMMA
      IF ((LRDMP.GT.0.AND.PEAK.NE.0.0) .OR. LPEAK.GT.0) THEN
C
C
        IF (LRDMP.GT.0) THEN
C
C              ***********************************
          CALL PEAKPK(RI2,NANG,RMAP,NALPHA,NGAMMA)
C              ***********************************
C
        END IF
C
C            ******
        CALL ANGFX2
C            ******
C
      END IF
C
C
      IF (LRDMP.GT.0 .AND. LPRINT.GE.-99) THEN
C
C            *********************************************
        CALL MAPRNT(RI2,NANG,RMAP,NALPHA,NGAMMA,AMIN,GMIN)
C            *********************************************
C
      END IF
C
C
      IF (LRDMP.GT.0 .OR. LPEAK.GT.0) CALL CCPERR(1,
     +   'Normal termination')
C
C---- Otherwise
C     We are going to calculate something - either in almnmn or frfs
C
      IF (LRDAL.EQ.0) THEN
         IF (LSAVE.EQ.0) THEN
           STATUS = 'SCRATCH'
         ELSE
           STATUS = 'NEW'
         END IF
      ELSE
        STATUS = 'READONLY'
      ENDIF
C
C          ********************************************
      CALL CCPDPN(LCO1,'COEFFS1',STATUS,'U',LDUM,IFAIL)
C          ********************************************
C
C                      *************************************************
      IF (NCRYST.EQ.2) CALL CCPDPN(LCO2,'COEFFS2',STATUS,'U',LDUM,IFAIL)
C                      *************************************************
C
C---- if(lrdal.gt.0)   go directly to frfs...
C
      IF (LRDAL.GT.0) GO TO 390
C
C---- set dimensions  for frfs arrays  and check there is enough space.
C
      NSLM4 = NSLIM/4
      NSLM4 = NSLIM/4
      NBESB2 = 2*NBES
      NBES2 = NBES/2
c     Factor to scale FMIN up or down to get sensible number of reflections
      FMINFC = 1.2
C
C---- Set up tables for required order of bessel functions
C
      DO 10 JDO10 = 1,NBES2
        BZEROS(0,JDO10) = REAL(JDO10*PI)
C
C
        DO 20 JDO20 = 1,NBES
          BZEROS(JDO20,JDO10) = BZ60(JDO20,JDO10)
20      CONTINUE
   10   CONTINUE
C
C
C
      RIMAX = 1.0/RESMAX
      RCBMAX = RIMAX**3
      RIMIN = 1.0/RESMIN
      AM2PI = TWOPI*SPHMIN
      RMMAX = AM2PI/RESMAX
      XING = SPHMAX/SPHMIN
      YING = SQRT(SPHMIN/SPHMAX)
      A2PI = TWOPI*SPHMAX
      PIBY2 = REAL(0.5D00 * PI)
      RMAX = A2PI/RESMAX
      FACT(1) = 0.0
C
C
      DO 30 JDO30 = 1,NBESB2
        FACT(JDO30+1) = LOG(REAL(JDO30)) + FACT(JDO30)
   30 CONTINUE
C
C---- Find max. order l, no. of zeros for each l
C
      LMAX = 0
C
C
      DO 50 JDO50 = 0,NBES,2
        IF (BZEROS(JDO50,1).GT.RMAX) GO TO 60
        LMAX = JDO50
C
C
        DO 40 JDO40 = 1,NBES2
          IF (BZEROS(JDO50,JDO40).GT.RMAX) GO TO 50
          NMAX(JDO50) = JDO40
   40   CONTINUE
   50 CONTINUE
C
C
   60 WRITE (6,FMT=6000) LMAX
      IF (LMAX.EQ.0) WRITE (6,FMT=6004)
 6004 FORMAT (
     + ' Small RMAX cf Bessel functions - LMAX ZERO - BAD NEWS!!')
      WRITE (6,FMT=6002) (NMAX(L),L=0,LMAX,2)
C
C
      DO 70 JDO70 = 0,NBES
        NMMAX(JDO70) = 0
   70 CONTINUE
C
C
      LMMAX = 0
C
C
      DO 90 JDO90 = 0,NBES,2
        IF (BZEROS(JDO90,1).GT.RMMAX) GO TO 100
        LMMAX = JDO90
C
C
        DO 80 JDO80 = 1,NBES2
          IF (BZEROS(JDO90,JDO80).GT.RMMAX) GO TO 90
          NMMAX(JDO90) = JDO80
   80   CONTINUE
   90 CONTINUE
C
C
  100 WRITE (6,FMT=6003) LMMAX
 6000 FORMAT (////' Maximum BESSEL Order = ',I5)
 6003 FORMAT (////' Maximum BESSEL Order for origin removal = ',I5)
      IF (LMMAX.EQ.0) WRITE (6,FMT=6006)
 6006 FORMAT (' Small RMMAX cf Bessel functions - LMMAX ZERO - does it',
     +       'matter??')
      WRITE (6,FMT=6002) (NMMAX(L),L=0,LMMAX,2)
 6002 FORMAT (' Cut-Off in N for Even L',/15I5)
      NBASA = 0
      NBASB = 0
      NBASC = 0
      NBASD = 0
C
C
      DO 110 JDO110 = 2,LMAX,2
        LMBLK2 = (2*JDO110+1)* (JDO110+1)
        L1 = JDO110/NSYMZO(1) + 1
        L21 = (JDO110/NSYMZO(2))*2 + 1
        NTMAX = NMAX(JDO110)
        LMBLOK = L1*L21
        LNBLK1 = (JDO110/NSYMZO(1)+1)*NTMAX
        LNBLK2 = (JDO110/NSYMZO(2)+1)*NTMAX
        NBASA = NBASA + LNBLK1
        NBASB = NBASB + LNBLK2
        NBASC = NBASC + LMBLOK
        NBASD = NBASD + LMBLK2
  110 CONTINUE
C
C
      IF (NBASA.GT.NSLM4) THEN
        WRITE (6,FMT=6008) NBASA,NSLM4
 6008 FORMAT (/' Value of ALMN,BLMN arrays(NBASA) for XL1 is ',I6,6X,
     +       'Program limit is ',I6)
        CALL CCPERR(1,
     +  ' ** LIMITS WRONG - INCREASE NSLIM PARAMETER**')
      END IF
C
C
      IF (NBASB.GT.NSLM4) THEN
        WRITE (6,FMT=6010) NBASB,NSLM4
 6010 FORMAT (/' Value of ALMN,BLMN arrays(NBASB) for XL2 is ',I6,6X,
     +       'Program limit is ',I6)
        CALL CCPERR(1, ' ** LIMITS WRONG - INCREASE NSLIM PARAMETER**')
      END IF
C
C
      IF (NBASC.GT.NCLIM) THEN
        WRITE (6,FMT=6012) NBASC,NCLIM
 6012 FORMAT (/' Value of CLMN array(NBASC) is',I6,7X,'Program limit i',
     +       's ',I6)
        CALL CCPERR(1, ' ** LIMITS WRONG - INCREASE NCLIM PARAMETER**')
      END IF
C
C
      IF (NBASD.GT.NSLIM) THEN
        WRITE (6,FMT=6014) NBASD,NSLIM
 6014 FORMAT (/' Value of SLMN array(NBASD) is',I6,7X,'Program limit i',
     +       's ',I6)
        CALL CCPERR(1, ' ** LIMITS WRONG - INCREASE NSLIM PARAMETER**')
      END IF
C
C---- Find spherical bessel function coefficients.
C     also max arg for bess
C
      DO 130 JDO130 = 0,LMAX,2
        NN = NMAX(JDO130)
C
C
        DO 120 JDO120 = 1,NN
          BZEROS(JDO130,JDO120) = BZEROS(JDO130,JDO120)/SPHMAX
  120   CONTINUE
  130 CONTINUE                 
C
C
      ARGMAX = A2PI/RESMAX + 1.0
C
C---- Set up table of spherical bessels for required range of args
C
C          *******************************
      CALL SPHBES(SBES,NBESMX,LMAX,ARGMAX)
C          *******************************
C
C---- Initialise for first hkl file
C
      DO 380 NCR = 1,NCRYST
C
C---- Main loop over datasets starts here
C
      WRITE(6,6001) NCR
 6001 FORMAT(////' Listing of data for crystal',I5)
      IF(NCR.EQ.1)WRITE(6,*)
     +    ' Total Number of REFL. for XTAL 1=',NREF1
      IF(NCR.EQ.2)WRITE(6,*)
     +    ' Total Number of REFL. for XTAL 2=',NREF2
C
C---- Read in intensities and convert indices to polar coordinates
C
        DO 140 JDO140 = 1,20
          NRF(JDO140) = 0
  140   CONTINUE
C
C
        SCALIO = SCALFO(NCR)
        NREFIP = 0
        J = 0
        NREF = 1 
        IUP = 0
        IDOWN = 0
C                
C---- Main loop for reflections starts here
C
  150   CONTINUE
C
C
        IF (NCR.EQ.1) THEN
C
C              ======================
          CALL LRREFF(LCR1,SSS,ADATA,EOF)
          CALL LRREFM(LCR1,LOGMSS)
C              ======================
C
        ELSE IF (NCR.EQ.2) THEN
C
C              ======================
          CALL LRREFF(LCR2,SSS,ADATA,EOF)
          CALL LRREFM(LCR2,LOGMSS)
C              ======================
C
        END IF
C
C
          IF (EOF) GOTO 200
C
C
C  Do not use a "Missing " entry for FP
        IF (LOGMSS(4)) GO TO 190
        IH = NINT(ADATA(1))
        IK = NINT(ADATA(2))
        IL = NINT(ADATA(3))
        FF = ADATA(4)
        IF (FF.LT.0.1) GO TO 190
C
Cejd - problems with MIPS Fortran which goes astray with2**16
C      replace al 128 etcwith 100
C      index = (ik+128)*256 + ih*65536 + il + 128
C
        INDEXY = (IK+100)*200 + IH*40000 + IL + 100
C
C---- Calculate a cartesian grid      
C
        XH = ADATA(1)
        XK = ADATA(2)
        XL = ADATA(3)
C
C
        XX = RFCRYS(1,1,NCR)*XH + 
     +       RFCRYS(2,1,NCR)*XK +
     +       RFCRYS(3,1,NCR)*XL
        YY = RFCRYS(1,2,NCR)*XH + 
     +       RFCRYS(2,2,NCR)*XK +
     +       RFCRYS(3,2,NCR)*XL
        ZZ = RFCRYS(1,3,NCR)*XH + 
     +       RFCRYS(2,3,NCR)*XK +
     +       RFCRYS(3,3,NCR)*XL
C
C
        RSQ = XX*XX + YY*YY + ZZ*ZZ
        RH = SQRT(RSQ)
        IF (RH.GT.RIMAX .OR. RH.LT.RIMIN) GO TO 190
C
C---- Count the number of reflections between resolution limits
C
        NREFIP = NREFIP + 1
C
C---- Compute temperature factor  if necessary
C
        IF (TFAC(NCR).NE.0.0) THEN
          EXPON = -TFAC(NCR)*RSQ/4.0
          IF (EXPON.LT.-38.0) EXPON = -38.0
          EXPON = EXP(EXPON)
          FF = FF*EXPON
        END IF
C
C
        IF (FF.LT.F2MIN(NCR) .OR. FF.GT.F2MAX(NCR)) GO TO 190
C
C---- Square amplitude and scale
C
        FF = FF*FF/SCALIO
C
C---- Correct for multiplicities prior to expansion
C
        MULT = 1
        KK(1) = INDEXY
        IKK(1) = 40200 - KK(1)
        IF (JSYMP(NCR).EQ.1) GO TO 170
C
C
        DO 160 JDO160 = 2,JSYMP(NCR)
          NEWH = RSYMT(1,1,JDO160,NCR)*IH + 
     +           RSYMT(2,1,JDO160,NCR)*IK +
     +           RSYMT(3,1,JDO160,NCR)*IL
          NEWK = RSYMT(1,2,JDO160,NCR)*IH + 
     +           RSYMT(2,2,JDO160,NCR)*IK +
     +           RSYMT(3,2,JDO160,NCR)*IL
          NEWL = RSYMT(1,3,JDO160,NCR)*IH + 
     +           RSYMT(2,3,JDO160,NCR)*IK +
     +           RSYMT(3,3,JDO160,NCR)*IL
cejd          kk(jdo160) = (newk+128)*256 + 65536*newh + newl + 128
cejd          ikk(jdo160) = 65792 - kk(jdo160)
          KK(JDO160) = (NEWK+100)*200 + 40000*NEWH + NEWL + 100
          IKK(JDO160) = 40200 - KK(JDO160)
          IF (KK(JDO160).EQ.KK(1) .OR. IKK(JDO160).EQ.
     +        KK(1)) MULT = MULT + 1
  160   CONTINUE
C
C
        FF = 2.0*FF/REAL(MULT)
  170   ICB = INT(20.0*RSQ*RH/RCBMAX+1.0)
        IF (ICB.GT.20) ICB = 20
        NRF(ICB) = NRF(ICB) + 1
        J = J + 1
C
C
Cejd        IF (J.GT.NREFMX) THEN
        IF (J.GT.LREFMX(NCR)) THEN
          IUP = IUP + 1
          IF(ABS(F2MIN(NCR)) .LT. 0.0000001) F2MIN(NCR)= 100
          IF(IDOWN.NE.0) FMINFC = (FMINFC +1.0)/2.0
          F2MIN(NCR) = F2MIN(NCR)*FMINFC
          WRITE (6,FMT=6016) F2MIN(NCR)
 6016 FORMAT ('  More than 10000 reflections, program reset Fmin to ',
     +       F12.0)
C
C                       =================
          IF (NCR.EQ.1) CALL LRREWD(LCR1)
          IF (NCR.EQ.2) CALL LRREWD(LCR2)
C                       =================
C
          J = 0
          NREFIP=0
C
C
          DO 180 JDO180 = 1,20
            NRF(JDO180) = 0
  180     CONTINUE                  
C
C
          GO TO 150
        END IF
C
C             
        F2MOD(J) = FF
        RHSQ(J) = RSQ 
        ITEMP(J) = INDEXY           
  190   CONTINUE
        GO TO 150
C              
C---- NREF is the number of reflections accepted
C                  
  200   NREF = J
C
        WRITE(6,'(i10,a)') NREFIP,' Refl. in the resol. limits'
        WRITE(6,'(i10,a)') NREF  ,' Refl. accepted'
C
C---- if NREF less than LREFMX refl.
C
C  1) Check how many reflections in whole file
C
         NCHCK = NINT(0.9*NREFIP)
C
C  2) Is F2MIN already = 0.0?
C  3) Has there already been an "UP" step 
C   211  is "out..."
C
         IF (NREF.GE. NCHCK .OR. 
     +       ABS(F2MIN(NCR)) .LE. 0.000001 )  GO TO 211
         IF(NREF.GT.500 .AND. IUP .GT. 0) GO TO 211
C
C----   If nrefip le lrefmx Set F2MIN = 0  ie accept everything 
C
         IF (NREFIP .LE. LREFMX(NCR) )  F2MIN(NCR) = 0.0
C
C----  Now reduce F2MIN and read MTZ file again.
C
         F2MIN(NCR) = F2MIN(NCR)/FMINFC
         IDOWN = IDOWN + 1
C
C---- Now try again
C
C                         =================
            IF (NCR.EQ.1) CALL LRREWD(LCR1)
            IF (NCR.EQ.2) CALL LRREWD(LCR2)
C                         =================
C
            DO 210 JDO210 = 1,20
              NRF(JDO210) = 0
  210       CONTINUE
C
C
            WRITE (6,FMT=6020) NREF,NREFIP,F2MIN(NCR)
 6020 FORMAT (' Only ',I5,'reflections used from ',I8,/,
     +        '  program resets Fmin to ',F12.0)
            NREFIP=0  
            NREF = 1                           
            J = 0
            GO TO 150
 211        CONTINUE
          IF(NREF.EQ.0) CALL CCPERR(1,
     1   ' Problems with number of reflections - reset FMIN')
       MREF = M
C
C
        WRITE (6,FMT=6036) NCR,NREF
 6036 FORMAT (' Number of reflections for crystal',I3,'  is',I6)
        RANG1 = 0.0
        STEP = RCBMAX/20.0
        WRITE (6,FMT=6022)
 6022 FORMAT (/,
     +' Number of reflections in 20 (2sin theta/lambda)**3 ranges ',
     + //6X,'(2sin theta/lambda)**3   Number          d (A)',/)
C
C
        DO 220 JDO220 = 1,20
          RANG2 = RANG1 + STEP
          DR1 = (RANG1)**0.3333334
          IF (DR1.NE.0.0) DR1 = 1.0/DR1
           IF (ABS(DR1).LT.0.00000001) DR1 = 1000.0
           DR2 = (RANG2)**0.3333334
          IF (DR2.NE.0.0) DR2 = 1.0/DR2
           IF (ABS(DR2).LT.0.0000001) DR2 = 1000.0
           WRITE (6,FMT=6024) JDO220,RANG1,RANG2,NRF(JDO220),DR1,DR2
 6024 FORMAT (I5,F8.5,' - ',F8.5,I10,5X,F8.3,' - ',F8.3)
          RANG1 = RANG1 + STEP
          NRF(JDO220) = 0
  220   CONTINUE
C
C---- Scaling of intensities to avoid overflow
C
        WRITE (6,FMT=6026) SCALIO
 6026 FORMAT (//1X,'Intensities are divided by ',F15.5)
C
C            **************************
        CALL MAXVAL(F2MOD,NREF,FKP,JKP)
C            **************************
C
        WRITE (6,FMT=6028)
 6028 FORMAT (//'  Largest Reflections are ')
        WRITE (6,FMT=6032)
 6032 FORMAT ('    h    k    l     Rsq             Fsq     Mult')
C
C
        DO 230 JDO230 = 1,15
          J = JKP(JDO230)
          F2MOD(J) = FKP(JDO230)
          IKL = ITEMP(J)
          IH = IKL/40000
          IF (IKL.LT.0) IH = IH - 1
          IKL = IKL - IH*40000
          IK = IKL/200 - 100
          IKL = IKL - (IK+100)*200
          IL = IKL - 100
          WRITE (6,FMT=6034) IH,IK,IL,RHSQ(J),F2MOD(J),MULT
 6034 FORMAT (3I5,F10.4,F15.2,I5)
          JKP(JDO230) = 0
          FKP(JDO230) = 0.0
  230   CONTINUE
C
C---- Main summation loops
C
        IPASS = 0
        NSYMM = NSYMZO(NCR)
        WRITE (6,FMT=6038) NCR
 6038 FORMAT (' Spherical harmonic coefficients for crystal',I5)
C
C
        IF (NCR.EQ.1) THEN
           NNCR = LCO1
        ELSE IF (NCR.EQ.2) THEN
           NNCR = LCO2
        ELSE
           CALL CCPERR(1, ' Invalid crystal number ')
        END IF
C
C
        WRITE (NNCR) TITLE
        WRITE (NNCR) LMAX
C
C---- begin loop about bessel functions
C
        DO 370 JDO370 = 0,LMAX,2
          PIBY2L = JDO370*PIBY2
          MMAX = JDO370 + 1
          NMN = NMMAX(JDO370)
          NN = NMAX(JDO370)
          WRITE (NNCR) JDO370,NN
          FAC = FACT(2*JDO370+1)*0.5 - FACT(JDO370+1)
          SQRTL2 = SQRT(JDO370+.5)
          NB1 = NBES + 1
C
C---- Set almn blmn arrays to 0
C
          DO 250 JDO250 = 1,NB1
            DO 240 JDO240 = 1,NBES2
              ALMN(JDO250,JDO240) = 0.0
              BLMN(JDO250,JDO240) = 0.0
  240       CONTINUE
  250     CONTINUE
C
C---- Summ over patterson symmetry elements
C
          DO 350 JDO350 = 1,JSYMP(NCR)
C
C
            DO 260 JDO260 = 1,NREF
C
C---- Unpack reflexion indices and convert to polar coords
C
              IKL = ITEMP(JDO260)
              IH = IKL/40000
              IF (IKL.LT.0) IH = IH - 1
              IKL = IKL - 40000*IH
              IK = IKL/200 - 100
              IKL = IKL - (IK+100)*200
              IL = IKL - 100
C
C
              NEWH = NINT  (RSYMT(1,1,JDO350,NCR)*IH +
     +               RSYMT(2,1,JDO350,NCR)*IK + 
     +               RSYMT(3,1,JDO350,NCR)*IL  )
              NEWK = NINT  (RSYMT(1,2,JDO350,NCR)*IH +
     +               RSYMT(2,2,JDO350,NCR)*IK + 
     +               RSYMT(3,2,JDO350,NCR)*IL  )
              NEWL = NINT  (RSYMT(1,3,JDO350,NCR)*IH +
     +               RSYMT(2,3,JDO350,NCR)*IK + 
     +               RSYMT(3,3,JDO350,NCR)*IL  )
C
C
              XH = REAL(NEWH)
              XK = REAL(NEWK)
              XL = REAL(NEWL)
C
C
              XX = RFCRYS(1,1,NCR)*XH + 
     +             RFCRYS(2,1,NCR)*XK +
     +             RFCRYS(3,1,NCR)*XL
              YY = RFCRYS(1,2,NCR)*XH + 
     +             RFCRYS(2,2,NCR)*XK +
     +             RFCRYS(3,2,NCR)*XL
              ZZ = RFCRYS(1,3,NCR)*XH + 
     +             RFCRYS(2,3,NCR)*XK +
     +             RFCRYS(3,3,NCR)*XL
C
C
              XYSQ = XX*XX + YY*YY
              SINTH(JDO260) = SQRT(XYSQ/RHSQ(JDO260))
              COTTH(JDO260) = 0.0
              PHI(JDO260) = 0.0
C
C
              IF ((ABS(YY).LT.0.0000001) .AND. 
     +            (ABS(XX).LT.0.0000001)) GO TO 260
              COTTH(JDO260) = ZZ/SQRT(XYSQ)
              PHI(JDO260) = ATAN2(YY,XX)
  260       CONTINUE
C
C
            DO 280 JDO280 = 1,NREF
C
C---- Start calculation - function of symmetry and l
C
              PLM1(JDO280) = 0.0
              PLM2(JDO280) = 0.0
              EFACT = 0.0
C
C
              IF (SINTH(JDO280).NE.0.0) THEN
                EXPON = LOG(SINTH(JDO280)/2.0)*REAL(JDO370) + FAC
                IF (EXPON.GT.-60.0) EFACT = EXP(EXPON)
              END IF
C
C
              PLM2(JDO280) = SQRT(REAL(JDO370)+0.5)*EFACT
C
C---- Apply inner cutoff to patterson density -
C                        function of l- not symm
C
              IF (JDO350.GT.1) GO TO 280
              IF (NMN.gt.0) then
C
C---- IJT  - His function FSBES does this.
C
C                              **************************************
               BESARG = AM2PI * SQRT( RHSQ(JDO280) )
              F2MBES(JDO280) = FSBES(JDO370,BESARG)
     +                        *F2MOD(JDO280)
              end if
c
c
              BESARG = A2PI*SQRT(RHSQ(JDO280))
              F2BES(JDO280) = FSBES(JDO370,BESARG)
     +                        *F2MOD(JDO280)
C                             ***************************************
C
  280       CONTINUE
C
C
            DO 340 JDO340 = 1,MMAX
              M = MMAX - JDO340
              M1 = M + 1
              IF (JDO340.EQ.1) GO TO 290
              FAC1 = REAL(M1)*2.0/
     +               SQRT(REAL((JDO370-M1+1)* (JDO370+M1)))
              FAC2 = SQRT(REAL((JDO370+M1+1)* (JDO370-M1))/
     +               REAL((JDO370-M1+1)* (JDO370+M1)))
  290         CONTINUE
C
C
              DO 330 JDO330 = 1,NREF
                IF (JDO340.EQ.1) GO TO 300
                TEMP = PLM1(JDO330)
                PLM1(JDO330) = PLM2(JDO330)
                PLM2(JDO330) = COTTH(JDO330)*FAC1*PLM1(JDO330) -
     +                         FAC2*TEMP
                IF ((ABS(SINTH(JDO330)).LT.0.000001) .AND.
     +              (M.EQ.0)) PLM2(JDO330) = SQRTL2
  300           IF (MOD(M,NSYMM).NE.0) GO TO 330
                PHASE = PIBY2L - PHI(JDO330)*M
                CPH = COS(PHASE)
                SPH = SIN(PHASE)
                FAC3 = TWOPI*TWOPI*RHSQ(JDO330)
C
C---- 2-5-91 Ian points out problems when FAC3BZ is small. 
C            Here is a FIX  Remember FAC3 and BZ*BZ are 
C            both large and small diff is unreliable. 
C            approximate FAC4 equals deriv(Numerator)/deriv(denom) 
C            when this happens.
C
                DO 320 JDO320 = 1,NN
                  BZ = BZEROS(JDO370,JDO320)
                  FAC3BZ = FAC3 - BZ**2
c
c
             IF (ABS(FAC3BZ) .LT.( .05/(JDO320*SQRTL2))*FAC3) THEN
                    FAC4 = .5*SPHMAX*PLM2(JDO330)*
     &              FSBES(JDO370-1,SPHMAX*BZ)*F2MOD(JDO330)
                  ELSE
                    FAC4 = (BZ*PLM2(JDO330)*F2BES(JDO330) )/FAC3BZ
                  ENDIF
C
C---- End OF Ian TIckle MOD 1
C
                  IF (NMN.LT.JDO320) GO TO 310
                  BMZ = BZ*XING
C
C---- Start OF Ian TIckle MOD 2
C
                  FAC3BZ = FAC3 - BMZ**2
                  IF (ABS(FAC3BZ) .LT. 0.001*FAC3) THEN
                    FMAC4 = .5*SPHMIN*FSBES(JDO370-1,SPHMIN*BMZ)*
     &              F2MOD(JDO330)
                  ELSE
                    FMAC4 = BMZ*F2MBES(JDO330)/FAC3BZ
                  ENDIF
                  FAC4 = FAC4 - FMAC4*PLM2(JDO330)*YING
C
  310             ALMN(M1,JDO320) = ALMN(M1,JDO320) + FAC4*CPH
                  IF (M1.EQ.1) GO TO 320
                  BLMN(M1,JDO320) = BLMN(M1,JDO320) + FAC4*SPH
  320           CONTINUE
  330         CONTINUE
  340       CONTINUE
  350     CONTINUE
C
C
C
          DO 360 JDO360 = 1,MMAX,NSYMM
            M = JDO360 - 1
            IF (JDO370.EQ.8) WRITE (6,FMT=6040) JDO370,M
 6040 FORMAT (2I10)
            IF (JDO370.EQ.8) WRITE (6,FMT=6042) (ALMN(JDO360,N),
     +          BLMN(JDO360,N),N=1,NN)
 6042 FORMAT (5X,2E13.4)
            WRITE (NNCR) (ALMN(JDO360,N),BLMN(JDO360,N),N=1,NN)
  360     CONTINUE
  370   CONTINUE
C
C
  380 CONTINUE
C
cc??????
        NCR = NCR + 1
        NCH = NCH + 1
        IF (NCR.EQ.1) REWIND LCO1
        IF (NCR.EQ.2) REWIND LCO2
cc???????
  390 CONTINUE
C
C
      WRITE (6,FMT=6030) NBETAG
 6030 FORMAT (/' End of ALMN calculation ',4I5)
C
C          ****************
      CALL FRFS(NBETAG,FMM)
C          ****************
C
      CALL CCPERR(0,' Normal termination of ALMN ')
C
C
      END
C                                                          
C
C
      SUBROUTINE ANGFX2
C     =================
C
C
C---- This s/r works out various rotation angles corresponding
C     to a given rotation matrix.
C     It could easily be extended to give other sets.
C
C
C
C     .. Parameters ..
      INTEGER MAXNPK
      PARAMETER (MAXNPK=100)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)             
      DOUBLE PRECISION PI
      PARAMETER (PI=3.141592653589793D00)
C     ..
C     .. Scalars in Common ..
      INTEGER LCO1,LCO2,LCR1,LCR2,LPEAK,LPKOUT,NCRYST,NPEAK
      CHARACTER PKFNAM*80,TITLE*80
      LOGICAL OutputXML
C     ..
C     .. Arrays in Common ..
      REAL ARSYM,CELCRY,F2MAX,F2MIN,F2MINS,PEAKS,PKPOS,RFCRYS,ROCRYS,
     +     RSYM,RSYMT,SCALFO,TFAC,VOLCR
      INTEGER JSYMP,LPKORD,LSPGRP,NCODES,NSYMZO
C     ..
C     .. Local Scalars ..
      REAL AKAPPA,ALPHA,BETA,CONV,COSA,COSB,COSG,GAMMA,OMEGA,PHI,SINA,
     +     SINB,SING,TRACE
      INTEGER I,ISYM1,ISYM2,J,JJ,JP,K,L,LI,LUN,ngamma2,nkappa,nomega,
     +     nphi, ifail, XMLFileUnit
      LOGICAL flagHKL,flagHbarKbarL, flagKHLbar,flagKbarHbarLbar
C     ..
C     .. Local Arrays ..
      REAL AJUNKF(3,3),AJUNKO(3,3),AJUNKS(3,3),DC(5),PKHGHT(MAXNPK),
     +     TRMAT(3,3),TRSYM(3,3)
      CHARACTER CODES(5)*15
C     ..
C     .. External Functions ..
      INTEGER LENSTR
      EXTERNAL LENSTR
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPDPN,DCOSFD,EULERA,MATMUL,POLANG,RNGANG,SORT1
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ATAN,COS,SIN
C     ..
C     .. Common blocks ..
      COMMON /INALM/ LCR1,LCR2,LCO1,LCO2
      COMMON /CRYSTS/NCRYST,ROCRYS(4,4,2),RFCRYS(4,4,2),CELCRY(6,2),
     +       NCODES(2),LSPGRP(2),VOLCR(2),TFAC(2),NSYMZO(2),F2MIN(2),
     +       SCALFO(2),F2MAX(2),F2MINS(2),NREF1,NREF2
      COMMON /PEKPK/NPEAK,PEAKS(MAXNPK),PKPOS(3,MAXNPK),LPEAK,
     +       LPKORD(MAXNPK),LPKOUT
      COMMON /PKNAME/PKFNAM
      COMMON /SYMTR/JSYMP(2),RSYM(4,4,MAXSYM,2),RSYMT(4,4,MAXSYM,2),
     +       ARSYM(3,3,MAXSYM,2)
      COMMON /TITL/TITLE
      COMMON /XMLINFO/OutputXML
C     ..    
      SAVE
C     .. Data statements ..
      DATA CODES/'a,c*xa,c*','b,a*xb,a*','c,b*xc,b*','a+b,c*x(a+b),c*',
     +     'a*,cxa*,c'/
C     ..
C           
C
       flagHKL = .FALSE.
       flagHbarKbarL = .FALSE.
       flagKHLbar = .FALSE.
       flagKbarHbarLbar = .FALSE.
      LI = 6
      CONV = REAL(180.0D00/PI)
      ifail = 0
      XMLFileUnit = 0
C 
       IF (LPEAK.EQ.0) THEN 
C
C---- Sort reflection pointers into ascending order of magnitude 
C Initialize pointers and copy peak height. Note that the sort 
C does not reorder storage arrays PEAKS & PKPOS, but just creates 
C a list of pointers 
C 
         DO 10 I = 1,NPEAK 
           PKHGHT(I) = PEAKS(I) 
           LPKORD(I) = I 
10       CONTINUE 
C 
C           ************************** 
       CALL SORT1(NPEAK,PKHGHT,LPKORD) 
C           ************************** 
C 
            ELSE 
C
C---- PEAK option, set pointers to keep peaks in input order 
C 
        DO 20 I = 1,NPEAK 
          LPKORD(I) = NPEAK - I + 1 
20       CONTINUE 
C 
C 
         END IF 
C 
C 
           WRITE (6,FMT=6000) 
6000  FORMAT (///25X, 
     + 'Eulerian angles                       Polar angles',//22X, 
     + 'Alpha   Beta  Gamma       Peak       ',
     $          'Omega    Phi  Kappa', 
     + '           Direction cosines',/, 
     + ' PkNo Symm: 1   2',//) 
C 
         DO 100 JP = 1,NPEAK 
C
C---- J is pointer to peaks in order of decreasing height 
C 
            J = LPKORD(NPEAK-JP+1) 
            WRITE (6,FMT=6002) JP 
6002        FORMAT (/2X,'Peak',I4) 
            BETA = PKPOS(2,J)/CONV 
            COSB = COS(BETA) 
            SINB = SIN(BETA) 
            ALPHA = PKPOS(1,J)/CONV 
            COSA = COS(ALPHA) 
            SINA = SIN(ALPHA) 
            GAMMA = PKPOS(3,J)/CONV 
            COSG = COS(GAMMA) 
            SING = SIN(GAMMA) 
            TRMAT(1,1) = COSG*COSB*COSA - SING*SINA 
            TRMAT(2,1) = COSG*COSB*SINA + SING*COSA
            TRMAT(3,1) = -COSG*SINB 
            TRMAT(1,2) = -SING*COSB*COSA - COSG*SINA
            TRMAT(2,2) = -SING*COSB*SINA + COSG*COSA 
            TRMAT(3,2) = SING*SINB
            TRMAT(1,3) = SINB*COSA 
            TRMAT(2,3) = SINB*SINA 
            TRMAT(3,3) = COSB 
C 
C 
               DO 80 ISYM1 = 1,JSYMP(1) 
                 DO 40 I = 1,3 
                   DO 30 JJ = 1,3 
                    AJUNKS(I,JJ) = ARSYM(I,JJ,ISYM1,1) 
30                 CONTINUE 
40               CONTINUE 
C 
               DO 70 ISYM2 = 1,JSYMP(2)
C 
C---- use ajunkf and ajunk o arrays as temporary store...  
C 
                 DO 60 I = 1,3 
                     DO 50 JJ = 1,3 
                      AJUNKF(I,JJ) = ARSYM(I,JJ,ISYM2,2) 
50                   CONTINUE 
60                CONTINUE 
C 
C               *************************** 
           CALL MATMUL(AJUNKO,TRMAT,AJUNKF) 
           CALL MATMUL(TRSYM,AJUNKS,AJUNKO) 
C               *************************** 
C 
             TRACE = TRSYM(1,1) + TRSYM(2,2) + TRSYM(3,3) 
C 
C 
              IF (TRACE.GE.2.9999 .AND. NCRYST .EQ.1) THEN 
                   GO TO 90 

                ELSE 
C 
C---- Get Euler angles 
C 
C                ****************************** 
            CALL EULERA(TRSYM,ALPHA,BETA,GAMMA) 
C                ****************************** 
C
C----Get Kappa & direction cosines 
C 
C                *********************** 
            CALL DCOSFD(TRSYM,DC,AKAPPA) 
C                *********************** 
C 
C---- Get polar angles 
C 
C                  ******************** 
              CALL POLANG(DC,OMEGA,PHI) 
C                  ******************** 
C 
C---- Put all angles in range 0-360 degrees 
C 
C               ************* 
           CALL RNGANG(ALPHA) 
           CALL RNGANG(BETA) 
           CALL RNGANG(GAMMA)
           CALL RNGANG(OMEGA) 
           CALL RNGANG(PHI) 
           CALL RNGANG(AKAPPA) 
C               ************** 
C 
               WRITE (6,FMT=6006) JP,ISYM1,ISYM2,ALPHA,BETA,GAMMA,
     +               PEAKS(J),OMEGA,PHI,AKAPPA,DC(1),DC(2),DC(3) 
6006     FORMAT(4X,3I4,4X,3F7.1,3X,F8.1,5X,3F7.1,6X,3F8.4) 
C  Consider reindexing..
C#
C# Run rotation function between two sets of reflection data
C#
C# If they are both on the same convention
C# the first peak will have at least one symmetry pair with kappa = 0
C#
C#  All other reindexing conventions require a  symmetry pair with kappa = 180
C# If the reindexing is -h,-k,l  omega = 0 or 180
C# if the reindexing is k,h,-l or -k,-h,-l omega = 90
C#
C#  For  k, h,-l  phi =  gamma/2
C#  For -k,-h,-l  phi = -gamma/2 or gamma/2 + 90
C#
            IF(JP.EQ.1) THEN
              IF(NCRYST.EQ.2) THEN
                ngamma2 = nint(0.5*celcry(6,2))
                nkappa = 15*nint(akappa/15.0)
                nomega = 15*nint(omega/15.0)
                nphi = 15*nint(phi/15.0)
C  No rotation - Kappa = 0.0
                  if(nkappa.eq.0)  flagHKL=.TRUE.
C  Kappa = 180; can have reindexing as -h, -k,l
                  if(nkappa.eq.180 .and.
     +              (nomega.eq.0 .or.nomega.eq.180) )  
     +                             flagHbarKbarL=.TRUE.
C  Omega = 90; can rotation k,h,-l or -k,-h,-l
                  if(nomega.eq.90) then
                    if(nphi.eq.ngamma2) flagKHLbar = .TRUE.
                    if(nphi.eq.-ngamma2 .or.nphi.eq.ngamma2+90)
     +            flagKbarHbarLbar = .TRUE.
                  end if
              end if
            END IF
          END IF 
70         CONTINUE 
80         CONTINUE 
C 
C 
             GO TO 100 
90          WRITE (6,FMT=6004) PEAKS(J), (PKPOS(I,J),I=1,3) 
6004         FORMAT (' Origin peak',31X,F8.1,5X,3F7.1) 
100         CONTINUE 

         IF (OutputXML) CALL XMLOpen (XMLFileUnit,'almn_run', ifail)

         CALL ccp4h_summary_beg()

         IF (flagHKL) THEN
           write(6,'(//,a,//)')
     +   ' REINDEX HKL h,k,l ie:  Orthogonal Crystal axes are aligned'
           IF (OutputXML) CALL XMLWriteAttribute 
     +       (XMLFileUnit, 'reindex_operator', 'required', 'no', ifail)
         ELSE IF (flagHbarKbarL) THEN 
           write(6,'(//,a,//)')
     +   ' REINDEX HKL -h,-k,l to align Orthogonal Crystal axes'
           IF (OutputXML) THEN
             CALL XMLWriteAttribute (XMLFileUnit, 
     +            'reindex_operator', 'required', 'yes', ifail)
             CALL XMLWriteAttribute (XMLFileUnit,
     +            'reindex_operator', 'operator', '-h,-k,l', ifail)
           ENDIF
         ELSE IF (flagKHLbar) THEN 
           write(6,'(//,a,//)')
     +   ' REINDEX HKL k,h,-l  to align Orthogonal Crystal axes'
           IF (OutputXML) THEN
             CALL XMLWriteAttribute (XMLFileUnit, 
     +            'reindex_operator', 'required', 'yes', ifail)
             CALL XMLWriteAttribute (XMLFileUnit,
     +            'reindex_operator', 'operator', 'k,h,-l', ifail)
           ENDIF
         ELSE IF (flagKbarHbarLbar) THEN 
           write(6,'(//,a,//)')
     +   ' REINDEX HKL -k,-h,-l to align Orthogonal Crystal axes'
           IF (OutputXML) THEN
             CALL XMLWriteAttribute (XMLFileUnit, 
     +            'reindex_operator', 'required', 'yes', ifail)
             CALL XMLWriteAttribute (XMLFileUnit,
     +            'reindex_operator', 'operator', '-k,-h,-l', ifail)
           ENDIF
         ENDIF

         CALL ccp4h_summary_end()

         IF (OutputXML) CALL XMLCloseElement 
     +        (XMLFileUnit,'almn_run', ifail)
C 
C 
             WRITE (LI,FMT=6008) 
6008       FORMAT (///, 
     +' The rotation given by the angles of a peak rotates ',/, 
     +' coordinates in the orthogonal frame of crystal 2 to ',/, 
     +' the orthogonal frame of crystal 1.',/, 
     +' Beware of axis permutations introduced by NCODE = 2, 3 or 4', 
     +///, 
     +' Rotation matrices are defined as follows:',///, 
     +' ( l**2+(m**2+n**2)cos k lm(1-cos k)-nsin k nl(1-', 
     +'cos k)+msin k )',/,
     +' ( lm(1-cos k)+nsin k m**2+(l**2+n**2)cos k mn(', 
     +'1-cos k)-lsin k )',5X,'Polar angles',/, 
     +' ( nl(1-cos k)-msin k mn(1-cos k)+lsin k n*2+(', 
     +'l**2+m**2)cos k )',//, 
     +' where l m n are the direction cosines of the axis',/, 
     +' about which the rotation k = kappa takes place.',//31X, 
     +'( l ) ( sin omega cos phi )',/31X,'( m ) ', 
     +' = ( sin omega sin phi )',/31X,'( n ) ', 
     +' ( cos omega )',/) 
        WRITE (LI,FMT=6010) 
6010     FORMAT (//, 
     +' ( cosa cosb cosg - sina sing -cosa cosb sing - ', 
     + 'sina cosg cosa sinb )',/' ( sina cosb cosg + cosa s', 
     + 'ing -sina cosb sing + cosa cosg sina sinb )',5X, 
     + 'Eulerian angles Alpha, Beta, Gamma',/' ( -sinb c', 
     + 'osg sinb sing cos', 
     + 'b )',/) 
        WRITE (6,FMT=6012) 
6012    FORMAT (///, 
     +' Remember this',/18X, 
     + 'when Beta=0 can only define Alpha + Gamma .',/18X, 
     + 'when Beta = 180 can only define Gamma - Alpha.')  
C 
C---- Write unique peaks out to file 
C 
       IF (LPKOUT.GT.0) THEN 
          L = LENSTR(PKFNAM) 
          WRITE (6,FMT=6014) PKFNAM(1:L) 
6014     FORMAT (//' Unique peaks written out to file ',A) 
            LUN = 1 
C 
C               ************************************* 
           CALL CCPDPN(LUN,PKFNAM(1:L),'NEW','F',0,0) 
C               ************************************* 
C 
             L = LENSTR(TITLE) 
               WRITE (LUN,FMT=6016) TITLE(1:L) 
6016       FORMAT ('!  Peaks from ALMN',/'TITLE ',A) 
C 
C 
             IF (NCRYST.EQ.1) THEN 
             WRITE (LUN,FMT=6018) 'SELF' 
              ELSE IF (NCRYST.EQ.1) THEN 
             WRITE (LUN,FMT=6018) 'CROSS' 
6018         FORMAT (A) 
              END IF
C 
C 
               DO 120 L = 1,NCRYST 
                  WRITE (LUN,FMT=6020) NCODES(L),CODES(NCODES(L)) 
6020          FORMAT ('CRYSTAL',I5,' ! Ncode, orthogonalization ',A) 
              WRITE (LUN,FMT=6022) L,JSYMP(L) 
6022          FORMAT ('SYMORTHOGONAL',2I5) 
C 
C 
                DO 110 K = 1,JSYMP(L) 
                  WRITE (LUN,FMT=6024) K, ((ARSYM(I,J,K,L),J=1,3),I=1,3)
6024        FORMAT ('!  Symmetry operation ',I5,/2 (3F12.5,/),3F12.5) 
110        CONTINUE 
120        CONTINUE 
C 
C---- Now the peaks 
C 
           WRITE (LUN,FMT=6026) NPEAK 
6026       FORMAT ('PEAKS',I8,/'!  Alpha Beta Gamma Height') 
C
C 
C 
           DO 130 L = 1,NPEAK 
             K = LPKORD(NPEAK-L+1)
               WRITE (LUN,FMT=6028) L, (PKPOS(I,K),I=1,3),PEAKS(K) 
6028      FORMAT(I6,3F10.1,F10.2) 
130       CONTINUE 
C 
C 
         CLOSE(UNIT=LUN,STATUS='KEEP')
       END IF 
C 
C 
       END 
C 
C 
C
      LOGICAL FUNCTION CHKEQL(PKPOS,PKPOS2,ABGSTP)
C     ============================================
C
C---- Returns .True. if two peaks PKPOS & PKPOS and the same within
C     the limits ABGSTP
C
C
C
C     .. Array Arguments ..
      REAL ABGSTP(3),PKPOS(3),PKPOS2(3)
C     ..
C     .. Local Scalars ..
      INTEGER I
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS
C     ..
C
C
      DO 10 I = 1,3
        IF (ABS(PKPOS(I)-PKPOS2(I)).GT.ABGSTP(I)) GO TO 20
   10 CONTINUE
      CHKEQL = .TRUE.
      RETURN
   20 CHKEQL = .FALSE.
C
C
      END
C
C
C
C
      SUBROUTINE DCOSFD(RSYM,DIRCOS,KAPPA)
C     ===================================
C******************************************************
C       Given a rotation matrix ROTN, expressing a rotation
C       through an angle KAPPA right-handedly about an axis
C       with direction cosines DIRCOS(I),
C       DCOSFD determines DIRCOS() and KAPPA
C       KAPPA is returned in degrees in the range 0 to 180
C
C       Expression for rotation matrix is (see J & J, "Mathl.
C               Phys.", p. 122):-
C
C  cw + n(1)n(1)(1-cw)    n(1)n(2)(1-cw)-n(3)sw  n(1)n(3)(1-cw)+n(3)sw
C  n(1)n(2)(1-cw)+n(3)sw  cw + n(2)n(2)(1-cw)    n(2)n(3)(1-cw)-n(1)sw
C  n(3)n(1)(1-cw)-n(2)sw  n(3)n(2)(1-cw)+n(1)sw  cw + n(3)n(3)(1-cw)
C
C       where cw = cos(KAPPA), sw = sin(KAPPA),
C       & n(1), n(2), n(3) are the direction cosines.
C
C*******************************************************}
C
      DOUBLE PRECISION PI
      PARAMETER (PI=3.141592653589793D00)
        REAL RSYM(3,3),DIRCOS(3),S(3),P(3),D(3)
        REAL KAPPA
      REAL DIFF,TRACE,R2,R3,SINKAP,COSKAP
        INTEGER NS(3),NP(3),NCS(3),ND(3),KNTRL,J,JMX
C
C
          DIFF=1.0E-04
          KNTRL=0
C       Trace = 1 + 2.cos w
          TRACE=RSYM(1,1)+RSYM(2,2)+RSYM(3,3)
C       S(1)=2.n(1).sin w
C       S(2)=2.n(2).sin w
C       S(3)=2.n(3).sin w
          S(1)=RSYM(3,2)-RSYM(2,3)
          S(2)=RSYM(1,3)-RSYM(3,1)
          S(3)=RSYM(2,1)-RSYM(1,2)
          CALL ORDR3(KNTRL,S,NS)
C--Is biggest S() zero (i.e. is sin w = 0)?
          IF (ABS(S(NS(1))).GT.DIFF) THEN
            R2=S(NS(2))/S(NS(1))
            R3=S(NS(3))/S(NS(1))
            DIRCOS(NS(1))=1.0/SQRT(1 + R2**2 + R3**2)
            DIRCOS(NS(2))=R2*DIRCOS(NS(1))
            DIRCOS(NS(3))=R3*DIRCOS(NS(1))
            GO TO 123
          END IF
C--Calculation when sin w = 0 (esp. when w = 180').
C       P(1)=n(2).n(3).{0(W=0)or2(w=180)}
C       P(2)=n(3).n(1).{0(W=0)or2(w=180)}
C       P(3)=n(1).n(2).{0(W=0)or2(w=180)}
          P(1)=RSYM(3,2)
          P(2)=RSYM(1,3)
          P(3)=RSYM(2,1)
          CALL ORDR3(KNTRL,P,NP)
C--Is biggest P() zero (i.e. are all off-diag. terms zero)?
          IF (ABS(P(NP(1))).LT.DIFF) THEN
            IF (TRACE.GT.0.0) THEN
C--Matrix is unit matrix.
                KAPPA=0.0
                DIRCOS(1)=0.0
                DIRCOS(2)=0.0
                DIRCOS(3)=1.0
                RETURN
            END IF
C
C---- Trace -ve, so dyad about x,y or z.
C           
            KAPPA=180.0
C
C
            DO 40 J=1,3
 40           D(J)=RSYM(J,J)+1.0
            CALL ORDR3(KNTRL,D,ND)
            DIRCOS(ND(1))=1.0
            DIRCOS(ND(2))=0.0
            DIRCOS(ND(3))=0.0
            RETURN
          END IF
        IF (ABS(P(NP(2))).LT.DIFF) THEN
C--One d.c. is zero.
          DIRCOS(NP(1))=0.0
          DIRCOS(NP(2))=SQRT(MAX(.5+.5*RSYM(NP(2),NP(2)),0.))
          DIRCOS(NP(3))=SQRT(MAX(.5+.5*RSYM(NP(3),NP(3)),0.))
          IF (P(NP(1)).LT.0.0) DIRCOS(NP(2))=-DIRCOS(NP(2))
          GO TO 123
        END IF
C--All d.c's are nonzero.
C       R2=n(NP(1))/n(NP(2))
C       R3=n(NP(1))/n(NP(3))
          R2=P(NP(2))/P(NP(1))
          R3=P(NP(3))/P(NP(1))
          DIRCOS(NP(2))=1.0/SQRT(1 + R2**2 + (R2/R3)**2)
          DIRCOS(NP(3))=1.0/SQRT(1 + R3**2 + (R3/R2)**2)
          DIRCOS(NP(1))=SQRT(1.0-DIRCOS(NP(2))**2-DIRCOS(NP(3))**2)
C--Adjust signs of DIRCOS().
          JMX=0
          DO 20 J=1,3
            IF (P(J).GT.0.0) THEN
              IF (JMX.GT.0) JMX=-1
              IF (JMX.EQ.0) JMX=J
            END IF
 20       CONTINUE
          IF (JMX.GT.0) DIRCOS(JMX)=-DIRCOS(JMX)
          IF (JMX.EQ.0) WRITE(*,1358)
 1358     FORMAT(' All P()s. are negative or zero')
C
C---- Given d.c's, calculate KAPPA.
C
123       KNTRL=1
          CALL ORDR3(KNTRL,DIRCOS,NCS)
C
C----  Find KAPPA.
C
          SINKAP=RSYM(NCS(3),NCS(2))-RSYM(NCS(2),NCS(3))
          SINKAP=0.5*SINKAP/DIRCOS(NCS(1))
          COSKAP=0.5*(TRACE-1.0)
          KAPPA=ATAN2(SINKAP,COSKAP)
          KAPPA=KAPPA*( REAL (180.0D00/PI) )
C
C----  If kappa negative, negate kappa and invert DIRCOS
C
          IF (KAPPA .LT. 0.0) THEN
             KAPPA = -KAPPA
             DIRCOS(1) = -DIRCOS(1)
             DIRCOS(2) = -DIRCOS(2)
             DIRCOS(3) = -DIRCOS(3)
          ENDIF
        RETURN
        END
C
C
      SUBROUTINE EULERA(TRSYM,ALPHA,BETA,GAMMA)
C     =========================================
C
C---- Get Euler angles ALPHA,BETA,GAMMA from matrix TRSYM
C
C
C     .. Scalar Arguments ..
      REAL ALPHA,BETA,GAMMA
C     ..
C     .. Array Arguments ..
      REAL TRSYM(3,3)
C     ..
C     .. Local Scalars ..
      REAL CONV,R13,R23,R31,R32
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,ACOS,ATAN,ATAN2
C     ..
C
C
      CONV = 45.0/ATAN(1.0)
      IF (ABS(TRSYM(3,3)).GE.0.99999) THEN
C
C---- BETA = 0 or 180  can only find alpha + or - gamma.
C
        BETA = 0.0
        IF (TRSYM(3,3).LT.0.0) BETA = 180.0
        GAMMA = ATAN2(TRSYM(2,1),TRSYM(2,2))*CONV
        ALPHA = 0.0
      ELSE
C
        BETA = ACOS(TRSYM(3,3))*CONV
C
        R13 = TRSYM(1,3)
        R23 = TRSYM(2,3)
        R31 = TRSYM(3,1)
        R32 = TRSYM(3,2)
        GAMMA = ATAN2(R32,-R31)*CONV
        ALPHA = ATAN2(R23,R13)*CONV
      END IF
C
C
      END
C
C
C
      SUBROUTINE EXSECT(RMAP,LA,LB)
C     ===============================
C
C---- Extend section by 1 row & column all round, assuming rotation 
C
C
C
C     .. Scalar Arguments ..
      INTEGER LA,LB
C     ..
C     .. Array Arguments ..
      REAL RMAP(0:LA+1,0:LB+1)
C     ..
C     .. Local Scalars ..
      INTEGER I
C     ..
C
C
      DO 10 I = 1,LA
        RMAP(I,0) = RMAP(I,LB)
        RMAP(I,LB+1) = RMAP(I,1)
c        RMAP(I,0) = RMAP(I,2)
c        RMAP(I,LB+1) = RMAP(I,LB-1)
   10 CONTINUE
C
      DO 20 I = 0,LB + 1
        RMAP(0,I) = RMAP(LA,I)
        RMAP(LA+1,I) = RMAP(1,I)
c        RMAP(0,I) = RMAP(2,I)
c        RMAP(LA+1,I) = RMAP(LA-1,I)
   20 CONTINUE
C
C
      END
C
C
C
      SUBROUTINE FRFS(NBETAG,FMM)
C     ===========================
C
C
C---- Fast rotation function summation
C
C
C  problems over funny beta limits..
C  set up this array to try to save time
C    BDONE(75)
C
C
C
C
C     .. Parameters ..
      INTEGER NBESMX,NBTGMX
      PARAMETER (NBESMX=60,NBTGMX=72)
      INTEGER NCLIM,NSLIM
      PARAMETER (NCLIM=80000,NSLIM=80000)
      INTEGER MAXNPK
      PARAMETER (MAXNPK=100)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)             
      DOUBLE PRECISION PI
      PARAMETER (PI=3.141592653589793D00)
C     ..
C     .. Scalar Arguments ..
      INTEGER NBETAG
C     ..
C     .. Array Arguments ..
      COMPLEX FMM(NBETAG,2*NBETAG)
C     ..
C     .. Scalars in Common ..
      DOUBLE PRECISION RTD
      REAL ASTEP,BETA,BSTEP,BTMAX,BTMIN,GSTEP,PEAK
      INTEGER IRSYGP,LCO1,LCO2,LCR1,LCR2,LMAP,LMAX,LMAX2,
     +        LPEAK,LPKOUT,LPKRMS,LPLOT,LPRINT,LRDAL,LRDMP,LSAVE,LSEC1,
     +        LSEC2,MAXPEK,NALPHA,NBETA,NCRYST,NGAMMA,NPEAK,
     +        LREFMX
      CHARACTER TITLE*80
C     ..
C     .. Arrays in Common ..
      DOUBLE PRECISION FACT
      REAL ARSYM,CELCRY,F2MAX,F2MIN,F2MINS,PEAKS,PKPOS,RFCRYS,ROCRYS,
     +     RSYM,RSYMT,SCALFO,TFAC,VOLCR
      INTEGER JSYMP,LPKORD,LSPGRP,NCODES,NSYMZO
C     ..
C     .. Local Scalars ..
      COMPLEX CA,CB
      REAL AMAX,AMIN,GMAX,GMIN,SN,STEP,TEMP
      INTEGER I,I1,I2,ICH,IND,IND1,IND2,INDA,INDA0,INDB,INDB0,INDC,INDD,
     +        INDD0,ISGN,JDO10,JDO100,JDO150,JDO170,JDO180,JDO200,
     +        JDO210,JDO230,JDO240,JDO270,JDO280,JDO30,JDO300,JDO40,
     +        JDO70,JDO80,L1,L2,L21,LBASE,LL,LMAX1,
     +        MD1,MDASH1,MM1,MPRIME,MSTEP1,MSTEP2,MSTP11,N,NANG,NBASA,
     +        NBASB,NBASC,NBASD,NBDONE,NCODE1,NCODE2,NMAX,NSLM2,
     +        NSLM34,NSLM4,NBCHK
C     ..
C     .. Local Arrays ..
      REAL BDONE(75),RI2(4*NBTGMX*NBTGMX),
     +     RMAP(2* (NBTGMX+2),2* (NBTGMX+2),3)
      INTEGER D(3),LS(0:60),NN(0:60)
C     ..
C     .. External Subroutines ..
      EXTERNAL ANGFX2,MDHFT,PEAKPK,RFMOD,RMAT
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,CMPLX,CONJG,INT,MOD,NINT,REAL,DLOG10
C     ..
C     .. Common blocks ..
      COMMON /INALM/ LCR1,LCR2,LCO1,LCO2
      COMMON /CRYSTS/NCRYST,ROCRYS(4,4,2),RFCRYS(4,4,2),CELCRY(6,2),
     +       NCODES(2),LSPGRP(2),VOLCR(2),TFAC(2),NSYMZO(2),F2MIN(2),
     +       SCALFO(2),F2MAX(2),F2MINS(2),NREF1,NREF2
      COMMON /PARAMS/LPRINT,LMAP,LPLOT,LRDMP,LRDAL,PEAK,MAXPEK,LSAVE,
     +       LPKRMS,LREFMX(2)
      COMMON /PEKPK/NPEAK,PEAKS(MAXNPK),PKPOS(3,MAXNPK),LPEAK,
     +       LPKORD(MAXNPK),LPKOUT
      COMMON /RANGE/BTMIN,BTMAX,ASTEP,BSTEP,GSTEP,NALPHA,NBETA,NGAMMA,
     +       IRSYGP,LSEC1,LSEC2
      COMMON /RDLM/RTD,FACT(2*NBESMX+1),BETA,LMAX,LMAX2
cc
      COMPLEX CLMM
      REAL    SLMM
       COMMON /COMCSL/SLMM(NSLIM),CLMM(NCLIM)
cc
      COMMON /SYMTR/JSYMP(2),RSYM(4,4,MAXSYM,2),RSYMT(4,4,MAXSYM,2),
     +       ARSYM(3,3,MAXSYM,2)
      COMMON /TITL/TITLE
C     ..
      save
      DATA   LS /61*-1/
C
C
      NSLM4 = NSLIM/4
      NSLM2 = NSLIM/2
      NSLM34 = 3*NSLIM/4
C
C
C---- Read in input cards and adjust limits
C
      IF (NCRYST.EQ.1) THEN
        NSYMZO(2) = NSYMZO(1)
        NCODES(2) = NCODES(1)
      END IF
      NCODE2 = NCODES(2)
      NCODE1 = NCODES(1)
C
C---- step along alpha and gamma axes - 5deg for 30 bessel
C                                     - 2.5deg for 60 bessel
C
      STEP = 180.0/NBETAG
C
C---- astep=step
C     gstep=step
C
C---- Set alpha and gamma limits NALPHA AND NGAMMA SET IN RCARDSALMN
C
      AMIN = 0.0
      AMAX = 360.0/REAL(NSYMZO(1)) - STEP
      GMIN = 0.0
      GMAX = 360.0/REAL(NSYMZO(2)) - STEP
C
      MSTEP1 = NINT(180.0/STEP)
      MSTEP2 = NINT(360.0/STEP)
      MSTP11 = MSTEP1 + 1
C
C---- number of beta sections
C
      NBETA = INT((BTMAX-BTMIN)/BSTEP+0.5) + 1
C
      IF (NBETA.GT.MSTEP2+1) THEN
        WRITE (6,FMT=6010)
        CALL CCPERR(1,' NBETA error')
      END IF
C
C---- Set beta section controls
C
      DO 10 JDO10 = 1,75
        BDONE(JDO10) = 0.0
   10 CONTINUE
C
C---- replace all this with bdone(..) check - probably wrongly!!
C
C
C---- Read in input coefficients
C
      REWIND LCO1
      READ (LCO1) TITLE
      READ (LCO1) LMAX
      I2 = 0
C
C
      DO 40 JDO40 = 0,LMAX,2
        READ (LCO1,END=40) LL,N
        LS(JDO40) = LL
        NN(LL) = N
        L1 = LL + 1
C
C
        DO 30 JDO30 = 1,L1,NSYMZO(1)
          I1 = I2 + 1
          I2 = I2 + N
30        READ (LCO1) (SLMM(I),SLMM(NSLM4+I),I=I1,I2)
        IF (LL.EQ.0) I2 = 0
   40 CONTINUE
C
      IF (NCRYST.EQ.1) GO TO 90
C
      REWIND LCO2
      READ (LCO2) TITLE
      READ (LCO2) LMAX1
      IF (LMAX.NE.LMAX1)
     +       CALL CCPERR(1, 'ERROR - WRONG LMAX IN FILE 2')
      I2 = 0
C
C
      DO 80 JDO80 = 0,LMAX,2
        READ (LCO2,END=80) LL,N
        IF (LL.NE.LS(JDO80)) 
     +      CALL CCPERR(1, 'ERROR - WRONG L IN FILE 2')
        IF (N.NE.NN(LL)) 
     +      CALL CCPERR(1, 'ERROR - WRONG N IN FILE 2')
        L1 = LL + 1
C
C
        DO 70 JDO70 = 1,L1,NSYMZO(2)
          I1 = I2 + 1
          I2 = I2 + N
70        READ (LCO2) (SLMM(NSLM2+I),SLMM(NSLM34+I),I=I1,I2)
        IF (LL.EQ.0) I2 = 0
   80 CONTINUE
C
C---- Set up constants for rotation matrices
C
   90 RTD = PI/360.0D00
      LMAX2 = 2*LMAX
      FACT(1) = 0.0D00
C
cc     log10 - common logarithm
cc     Returns the common logarithm of a real number. The argument must be
cc     greater than 0.0.
cc     log10-val.as.r = log10 (number.as.r)
cc          Specific    Arg Type    Result Type Generic
cc          alog10      real*4      real*4      log10
cc          dlog10      real*8      real*8      log10
cc     *** extension ***
cc  The use of -r8 on the fortran command line causes alog10 to be treated as
cc  log10 in an expression and as dlog10 when passed as an actual argument.
C
      DO 100 JDO100 = 1,LMAX2
        FACT(JDO100+1) = DLOG10(DBLE(JDO100)) + FACT(JDO100)
  100 CONTINUE
C
C---- Sum over n to form clmm'
C
      NBASA = 0
      NBASB = 0
      NBASC = 0
C
C
      DO 180 JDO180 = 2,LMAX,2
        L1 = JDO180/NSYMZO(1) + 1
        L2 = JDO180/NSYMZO(2) + 1
        L21 = (JDO180/NSYMZO(2))*2 + 1
        NMAX = NN(JDO180)
C
C
        DO 170 JDO170 = 1,L1
C
C
          DO 160 MDASH1 = 1,L21
            MPRIME = MDASH1 - L2
            ISGN = MOD(ABS(MPRIME)*NSYMZO(2),2)
            INDC = (JDO170-1)*L21 + NBASC + MDASH1
            CLMM(INDC) = (0.0,0.0)
            INDA0 = (JDO170-1)*NMAX + NBASA
            INDB0 = ABS(MPRIME)*NMAX + NBASB
C
C
            DO 150 JDO150 = 1,NMAX
              INDA = INDA0 + JDO150
              INDB = INDB0 + JDO150
              CA = CMPLX(SLMM(INDA),-SLMM(NSLM4+INDA))
              IF (NCRYST.EQ.2) GO TO 120
              CB = CMPLX(SLMM(INDB),SLMM(NSLM4+INDB))
              GO TO 130
  120         CB = CMPLX(SLMM(NSLM2+INDB),SLMM(NSLM34+INDB))
  130         IF (MPRIME.GE.0) GO TO 140
              CB = CONJG(CB)
              IF (ISGN.EQ.1) CB = -CB
  140         CLMM(INDC) = CLMM(INDC) + CA*CB
  150       CONTINUE
  160     CONTINUE
  170   CONTINUE
C
C
        NBASA = NBASA + (JDO180/NSYMZO(1)+1)*NMAX
        NBASB = NBASB + (JDO180/NSYMZO(2)+1)*NMAX
        NBASC = NBASC + L1*L21
  180 CONTINUE
C
C---- Sum over l to form fmm' for each beta
C  7-4-87
C
C---- RMAT expects values of beta between 0 and 90
C     if necessary call rmat with beta lt 90 but not rfmod
C     and GO STRAIGHT ON TO REORDER SLMN
C
C---- use bdone to check whether a beta section has been done or not..
C
      DO 300 JDO300 = 1,NBETA
        ICH = 0
        BETA = REAL(JDO300-1)*BSTEP + BTMIN
        IF (ABS(BDONE(JDO300)).GE.0.000001) GO TO 290
        NBCHK = NINT(BETA)
        IF (NBCHK.GT.90) THEN
          ICH = 1
          BETA = 180.0 - BETA
        END IF
C
C---- Calculate slmm for this beta
C
        CALL RMAT
        IF (ICH.EQ.1) GO TO 250
  190   CONTINUE
C
C---- Clear fmm
C
        DO 210 JDO210 = 1,MSTEP1
          DO 200 JDO200 = 1,MSTEP2
            FMM(JDO210,JDO200) = (0.0,0.0)
  200     CONTINUE
  210   CONTINUE
C
        IND = 1
        NBASD = 0
C
C
        DO 240 JDO240 = 2,LMAX,2
          L1 = JDO240/NSYMZO(1) + 1
          L2 = JDO240/NSYMZO(2) + 1
          L21 = (JDO240/NSYMZO(2))*2 + 1
C
C
          DO 230 JDO230 = 1,L1
            INDD0 = (JDO230-1)*NSYMZO(1)* (2*JDO240+1) + NBASD +
     +              JDO240 + 1
C
C
            DO 220 MDASH1 = 1,L21
              MD1 = (MDASH1-L2)*NSYMZO(2)
              INDD = INDD0 + MD1
              MD1 = MD1 + 1
              IF (MD1.LE.0) MD1 = MSTEP2 + MD1
              MM1 = (JDO230-1)*NSYMZO(1) + 1
              FMM(MM1,MD1) = CLMM(IND)*SLMM(INDD) + FMM(MM1,MD1)
              IND = IND + 1
  220       CONTINUE
  230     CONTINUE
C
C
          NBASD = (2*JDO240+1)* (JDO240+1) + NBASD
  240   CONTINUE
C
C---- M and m' summations using 2-d fft
C
        D(1) = MSTEP1
        D(2) = MSTEP2
        D(3) = 0
C
        CALL MDHFT(FMM,D)
C
        MSTP11 = MSTEP1 + 1
        CALL RFMOD(FMM,BETA,STEP,MSTEP1,MSTEP2,MSTP11,GMIN,AMIN)
C
C---- set bdone(..)
C
        NBDONE = NINT((BETA-BTMIN)/BSTEP) + 1
        BDONE(NBDONE) = 1.0
C
        IF (ICH.EQ.1) GO TO 290
  250   CONTINUE
C
C---- Come here if there is no beta < 90 section to
C      match beta > 90 section.
C
        BETA = 180.0 - BETA
        IF (INT(BETA+0.5).EQ.90) GO TO 290
cc        if ((beta.lt.btmin) .or. (beta.gt.btmax)) go to 290
        IF ((BETA-BTMIN) .LT. -0.001 .OR. 
     +      (BETA-BTMAX) .GT.  0.001) GO TO 290
C
C---- Set up slmm(180.-beta) by reordering slmm(beta) with appropriate
C     sign changes
C
        ICH = 1
        LBASE = 0
C
C
        DO 280 JDO280 = 2,LMAX,2
          L1 = JDO280 + 1
          L21 = 2*JDO280 + 1
C
C
          DO 270 JDO270 = 1,L1
            IND = (JDO270-1)*L21 + LBASE
C
C
            DO 260 MDASH1 = 1,L1
              IND1 = IND + MDASH1
              IND2 = IND + L21 - MDASH1 + 1
              SN = 1.0
              IF (MOD(JDO270,2).EQ.0) SN = -1.0
              TEMP = SLMM(IND1)
              SLMM(IND1) = SLMM(IND2)*SN
              SLMM(IND2) = SN*TEMP
  260       CONTINUE
  270     CONTINUE
C
C
          LBASE = L1*L21 + LBASE
  280   CONTINUE
        GO TO 190
  290   CONTINUE
  300 CONTINUE
C
C
C---- If searching for peaks:-
C
      IF (PEAK.GT.0) THEN
        NANG = NALPHA*NGAMMA
        CALL PEAKPK(RI2,NANG,RMAP,NALPHA,NGAMMA)
C
C
        CALL ANGFX2
      END IF
C
      CALL CCPERR(0,' Normal termination of ALMN')
C
C---- Format statements
C
 6010 FORMAT (' **** MAXIMUM NO. OF BETA SECTIONS EXCEEDED')
C
C
      END
C
C
C
      REAL FUNCTION FSBES(L,ARG)
C     ..
C     .. Parameters ..
      INTEGER NREFMX,NBESMX
      PARAMETER (NREFMX=10000,NBESMX=60)
C     ..
C     .. Scalar Arguments ..
      INTEGER L
      REAL    ARG
C     ..
C     .. Common blocks ..
      REAL SBES
      COMMON /COMBES/ SBES(0:NBESMX,10*NBESMX)
C     .. Local Scalars ..
      INTEGER I,LA
      REAL    X
      save
c
c
C
      LA = IABS(L)
      X = 4.*ARG + 1.
      I = INT(X)
      X = X - REAL(I)
      FSBES = (1.-X)*SBES(LA,I) + X*SBES(LA,I+1)
      IF (L.LT.0 .AND. MOD(L,2).NE.0) FSBES = -FSBES
      END
C
C
      SUBROUTINE MAPRNT(RI2,NANG,RMAP,NALPH,NGAMM,AMIN,GMIN)
C     ======================================================
C
C
C
C
C
C
C     .. Parameters ..
      INTEGER NBTGMX
      PARAMETER (NBTGMX=72)
C     ..
C     .. Scalar Arguments ..
      INTEGER NALPH,NANG,NGAMM
      REAL AMIN,GMIN
C     ..
C     .. Array Arguments ..
      REAL RI2(NANG),RMAP(NALPH,NGAMM)
C     ..
C     .. Scalars in Common ..
      REAL ASTEP,BSTEP,BTMAX,BTMIN,GSTEP,PEAK
      INTEGER IRSYGP,LCO1,LCO2,LCR1,LCR2,LMAP,LPKRMS,LPLOT,
     +        LPRINT,LRDAL,LRDMP,LSAVE,LSEC1,LSEC2,MAXPEK,NALPHA,
     +        NBETA,NGAMMA,LREFMX
C     ..
C     .. Local Scalars ..
      REAL AT,BETA,RHMAX,RHMEAN,RHMIN,RMS
      INTEGER I,I1,I2,IA1,IA2,IB1,IER,IG1,IG2,IM,IMAP,INDJ,IUP,J,
     +        MODE,NA1,NA2,NB1,NG1,NG2,NP,NPAGE,NPP
      CHARACTER MAPTIT*80
C     ..
C     .. Local Arrays ..
      REAL ANGLA(2*NBTGMX),ANGLG(2*NBTGMX),CELLM(6)
      INTEGER IRF(NBTGMX),IUVW(3),NXYZ(3)
      CHARACTER CHRF(NBTGMX)*3
C     ..
C     .. External Subroutines ..
      EXTERNAL MGULPR,MPOSN,MRDHDR
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC MIN,MOD,NINT,REAL
C     ..
C     .. Common blocks ..
      COMMON /INALM/ LCR1,LCR2,LCO1,LCO2
      COMMON /PARAMS/LPRINT,LMAP,LPLOT,LRDMP,LRDAL,PEAK,MAXPEK,LSAVE,
     +       LPKRMS,LREFMX(2)
      COMMON /RANGE/BTMIN,BTMAX,ASTEP,BSTEP,GSTEP,NALPHA,NBETA,NGAMMA,
     +       IRSYGP,LSEC1,LSEC2
C     ..
      save
C     .. Data statements ..
      DATA CELLM/6*0.0/
C     ..
C
C
C---- Initialise arrays
C
      IUP = 4
      IF (NALPHA.LE.72) IUP = 2
      IF (NALPHA.LE.36) IUP = 1
C
C####### error amin and gmin not defined values now come from almnmn
C
      DO 10 I = 1,NBTGMX
        AT = AMIN + 360.0 + REAL(I-1)*ASTEP
        ANGLA(I) = MOD(AT,360.0)
        AT = GMIN + 360.0 + REAL(I-1)*GSTEP
        ANGLG(I) = MOD(AT,360.0)
   10 CONTINUE
C
C---- reopen map
C
      WRITE (6,FMT=6000)
      IMAP = 1
      MODE = 2
      CALL MRDHDR(IMAP,'MAPOUT',MAPTIT,NBETA,IUVW,NXYZ,IB1,IA1,IA2,IG1,
     +            IG2,CELLM,IRSYGP,MODE,RHMIN,RHMAX,RHMEAN,RMS)
C
C---- astep bstep gstep set in rcardsalmn
C     cell(1 2 3 ) set = 360.
C     nxyz(1 2 3) = grid
C
      NA1 = 1
      NA2 = IA2 - IA1 + 1
      NG1 = 1
      NG2 = IG2 - IG1 + 1
C
      CALL MPOSN(IMAP,LSEC1)
C
      DO 80 NB1 = LSEC1,LSEC2
        CALL MGULPR(IMAP,RI2,IER)
        IF (IER.GT.0) THEN
          GO TO 90
        ELSE
          DO 30 J = NG1,NG2
            INDJ = (J-1)*NA2
            DO 20 I = NA1,NA2
              RMAP(I,J) = RI2(I+INDJ)
   20       CONTINUE
   30     CONTINUE
C
C---- Output lineprinter sections
C
          BETA = NB1*BSTEP
          DO 70 NPAGE = 1,IUP
            WRITE (6,FMT=6006) BETA
            NP = (NPAGE-1)*36
            I1 = NP + 1
            NPP = MIN(36,NALPHA-NP)
            DO 40 I = 1,NPP
              IRF(I) = NINT(ANGLA(I+NP))
   40       CONTINUE
            I2 = NP + NPP
            WRITE (6,FMT=6008) (IRF(I),I=1,NPP)
            DO 60 J = 1,NGAMMA
              DO 50 I = 1,NPP
                IRF(I) = NINT(RMAP(I+NP,J))
                CHRF(I) = '   '
                IF (IRF(I).GT.LPRINT) WRITE (CHRF(I),FMT=6002) IRF(I)
C
C---- set values lt -99 to  *
C
                IF (IRF(I).LT.-99 .AND. LPRINT.LE.-99) CHRF(I) = ' * '
   50         CONTINUE
              WRITE (6,FMT=6010) ANGLG(J), (CHRF(IM),IM=1,NPP)
   60       CONTINUE
   70     CONTINUE
        END IF
   80 CONTINUE
C
C
      RETURN
   90 CALL CCPERR(1, ' ****  Error in map read ***')
C
C---- Format statements
C
 6000 FORMAT ('1')
 6002 FORMAT (I3)
 6006 FORMAT (////'   ALPHA HORIZ, GAMMA VERTICAL, BETA=',F5.1,/)
 6008 FORMAT (8X,36I3,/)
 6010 FORMAT (1X,F5.1,2X,36A3)
C
C
      END
C
C
C
      SUBROUTINE MAXVAL(F2MOD,NREF,FKP,JKP)
C     =====================================
C
C
C
C     .. Scalar Arguments ..
      INTEGER NREF
C     ..
C     .. Array Arguments ..
      REAL F2MOD(NREF),FKP(15)
      INTEGER JKP(15)
C     ..
C     .. Local Scalars ..
      REAL FMAX
      INTEGER JDO10,JDO20,JMAX
C     ..
C
C
      DO 20 JDO20 = 1,15
        FMAX = 0
        JMAX = 0
C
C
        DO 10 JDO10 = 1,NREF
          IF (FMAX.LT.F2MOD(JDO10)) THEN
            JMAX = JDO10
            FMAX = F2MOD(JDO10)
          END IF
   10   CONTINUE
C
        FKP(JDO20) = F2MOD(JMAX)
        JKP(JDO20) = JMAX
        F2MOD(JMAX) = 0.0
   20 CONTINUE
C
C
      END
C
C
C
      SUBROUTINE MDCFT2(X,D)
C     ======================
C
C
C
C---- Multi-dimensional complex fourier transform for data stored as
C     fortran complex numbers
C
C     X is an array of complex numbers stored by columns (i.e. in normal
C       fortran order).
C     D is an integer array which contains the
C       dimensioning information for X.
C       The absolute value of D(I) is the range of subscript I.
C       If D(I) is negative, the Ith dimension is not transformed.
C       Thus one may calculate, for example, 7 16 by 32
C       point transforms simultaneously by setting d to 16, 32, -7, 0.
C       the last element of d must be 0.
C
C
C     .. Array Arguments ..
      REAL X(*)
      INTEGER D(*)
C     ..
C     .. Scalars in Common ..
      INTEGER LCO1,LCO2,LCR1,LCR2
C     ..
C     .. Local Scalars ..
      INTEGER I,INC,NT,PMAX,PSYM,PTS,SPACE,TWOGRP
      LOGICAL ERROR
C     ..
C     .. Local Arrays ..
      INTEGER DIM(5),FACTOR(15),SYM(15),UNSYM(15)
C     ..
C     .. External Subroutines ..
      EXTERNAL DIPRP,MDFTKD,SRFP
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS
C     ..
C     .. Common blocks ..
      COMMON /INALM/ LCR1,LCR2,LCO1,LCO2
C     ..
      save
C
C
      IF (D(1).NE.0) THEN
C
C
        PMAX = 3
        TWOGRP = 4
        I = 2
        NT = D(1)
   10   CONTINUE
        IF (D(I).NE.0) THEN
          NT = D(I)*NT
          I = I + 1
          GO TO 10
        END IF
C
        NT = ABS(2*NT)
        DIM(1) = NT
        SPACE = NT
        INC = NT/D(I-1)
   20   CONTINUE
C
        I = I - 1
        IF (I.EQ.0) THEN
          RETURN
        ELSE
C
C
          IF (D(I).GT.1) THEN
            PTS = D(I)
C
C                *************************************************
            CALL SRFP(PTS,PMAX,TWOGRP,FACTOR,SYM,PSYM,UNSYM,ERROR)
C                *************************************************
C
            IF (ERROR) THEN
              GO TO 30
            ELSE
              IF (I.EQ.1) THEN
                DIM(2) = 2
                DIM(3) = NT
                DIM(4) = NT
                DIM(5) = 2*PTS
              ELSE
                DIM(2) = ABS(INC)
                DIM(3) = ABS(SPACE)
                DIM(4) = DIM(2)
                DIM(5) = 2
              END IF
C
C                  ***************************************
              CALL MDFTKD(PTS,FACTOR,DIM,X(1),X(2))
              CALL DIPRP(PTS,SYM,PSYM,UNSYM,DIM,X(1),X(2))
C                  ***************************************
C
            END IF
          END IF
C
          IF (I.GT.2) THEN
            INC = INC/D(I-1)
            SPACE = SPACE/D(I)
          END IF
          GO TO 20
        END IF
C
   30   CONTINUE
        WRITE (6,FMT=6000)
      END IF
C
C---- Format statements
C
 6000 FORMAT (' Factoring error in MDCFT',//)
C
C
      END
C
C
C
      SUBROUTINE MDHFT(X,DD)
C     ========================
C
C
C---- Multidimensional hermitian symmetric transform
C     it is assumed that the first dimension has only half of the
C     fourier coefficients stored in the complex array x.
C
C
      DOUBLE PRECISION PI
      PARAMETER (PI=3.141592653589793D00)
      REAL TWOPI
cc    This intrinsic function is illegal in Parameter statement
cc      parameter (twopi=real(2.0d00*pi))
C
C     .. Array Arguments ..
      REAL X(*)
      INTEGER DD(*)
C     ..
C     .. Scalars in Common ..
      INTEGER LCO1,LCO2,LCR1,LCR2
C     ..
C     .. Local Scalars ..
      REAL A,ANGLE,B,C,CO,D,E,F,SI,TWON
      INTEGER I,J,K,KK,LL,ND,NT,PTS
C     ..
C     .. Local Arrays ..
      INTEGER DIM(10)
C     ..
C     .. External Subroutines ..
      EXTERNAL MDCFT2
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,COS,REAL,SIN
C     ..
C     .. Common blocks ..
      COMMON /INALM/ LCR1,LCR2,LCO1,LCO2
C     ..
      save
c
c
      TWOPI=REAL(2.0D00*PI)
C
C
      NT = 2
      DO 10 I = 1,10
        DIM(I) = DD(I)
        IF (DIM(I).EQ.0) THEN
          GO TO 20
        ELSE
          NT = DIM(I)*NT
        END IF
   10 CONTINUE
C
      WRITE (6,FMT=6000)
      RETURN
C
   20 CONTINUE
      ND = I
      NT = ABS(NT)
      DIM(1) = -ABS(DIM(1))
      CALL MDCFT2(X,DIM)
C
      PTS = -DIM(1)
      DIM(1) = PTS
      I = 2*PTS
      TWON = REAL(I)
      DO 30 J = 1,NT,I
        X(J+1) = X(J)
   30 CONTINUE
C
      PTS = PTS + 1
      DO 50 J = 3,PTS,2
        ANGLE = REAL(J/2)*TWOPI/TWON
        CO = COS(ANGLE)
        SI = SIN(ANGLE)
        K = (PTS-J)*2
        DO 40 KK = J,NT,I
          LL = KK + K
          A = X(KK) + X(LL)
          B = X(KK) - X(LL)
          C = X(KK+1) + X(LL+1)
          D = X(KK+1) - X(LL+1)
          E = B*CO + C*SI
          F = B*SI - C*CO
          X(KK) = A + F
          X(LL) = A - F
          X(KK+1) = E + D
          X(LL+1) = E - D
   40   CONTINUE
   50 CONTINUE
C
      DO 60 J = 2,ND
        DIM(J) = -ABS(DIM(J))
   60 CONTINUE
C
      CALL MDCFT2(X,DIM)
C
C---- Format statements
C
 6000 FORMAT (' MDHFT more than nine dimensions',//)
C
C
      END
C
C
      SUBROUTINE ORDR3(KNTRL,Q,NQ)
C     ============================
C
C*************************************************************
C       Find NQ(1) so that |Q(NQ(1)| is biggest Q.
C       If KNTRL=1, make NQ(1),NQ(2),NQ(3) a positive permutn.
C       If KNTRL= anything else,
C       find nos. NQ() so that|Q(NQ(1)|>|Q(NQ(2))|>|Q(NQ(3))|
C
C*************************************************************
        REAL Q(3)
        INTEGER NQ(3),KNTRL,J
C
C---- Find no. NQ(1) of Q() with largest |Q()|.
C
          NQ(1)=1
          IF (ABS(Q(2)).GT.ABS(Q(1))) NQ(1)=2
          IF (ABS(Q(3)).GT.ABS(Q(NQ(1)))) NQ(1)=3
C
C---- Find nos. for other two Q()'s & order so |Q(NQ(2))|>|Q(NQ(3))|
C
          NQ(2)=MOD(NQ(1),3)+1
          NQ(3)=MOD(NQ(2),3)+1
          IF (KNTRL.EQ.1) GO TO 100
C
C
            IF (ABS(Q(NQ(3))).GT.ABS(Q(NQ(2)))) THEN
                J=NQ(3)
                NQ(3)=NQ(2)
                NQ(2)=J
            END IF
C
C
 100    RETURN
C
C
        END
C
C
C
      SUBROUTINE PEAKPK(RI2,NANG,RMAP,NALPH,NGAMM)
C     ============================================
C
C---- Find peaks in previous section
C
C
C rmap is used as a rotating buffer, rmap(,,I1), rmap(,,I2), rmap(,,I3),
C      contain 3 sections, I2 is the one to be searched
C
C---- For this routine, equivalence A/B/GSTEP == ABGSTP
C
C
C
C
C
C
C
C     .. Parameters ..
      INTEGER MAXNPK
      PARAMETER (MAXNPK=100)
C     ..
C     .. Scalar Arguments ..
      INTEGER NALPH,NANG,NGAMM
C     ..
C     .. Array Arguments ..
      REAL RI2(NANG),RMAP(0:NALPH+1,0:NGAMM+1,3)
C     ..
C     .. Scalars in Common ..
      REAL ASTEP,BSTEP,BTMAX,BTMIN,GSTEP,PEAK,SPHMAX,SPHMIN
      INTEGER IRSYGP,LCO1,LCO2,LCR1,LCR2,LMAP,LPEAK,LPKOUT,
     +        LPKRMS,LPLOT,LPRINT,LRDAL,LRDMP,LSAVE,LSEC1,LSEC2,MAXPEK,
     +        NALPHA,NBES,NBETA,NBETAG,NGAMMA,NPEAK,LREFMX
C     ..
C     .. Arrays in Common ..
      REAL PEAKS,PKPOS
      INTEGER LPKORD
C     ..
C     .. Local Scalars ..
      REAL A,BCHCK,OFFMAX,PKTHRD,R,RHMAX,RHMEAN,RHMIN,RMAX,RMS
      INTEGER I,I1,I2,I3,IA1,IA2,IB1,IBEG,IDIM,IEND,IER,IG1,IG2,IMAP,
     +        INDJ,IPEAK,ISTAT,ITEM,J,K,KEYS,L1,L2,L3,MAXREF,MODE,N,NA1,
     +        NA2,NB,NG1,NG2
      CHARACTER MAPTIT*80
C     ..
C     .. Local Arrays ..
      REAL ABGSTP(3),CELLM(6),OFFSET(3),PEAKMX(-1:+1,-1:+1,-1:+1)
      INTEGER IUVW(3),NXYZ(3)
C     ..
C     .. External Functions ..
      LOGICAL CHKEQL
      EXTERNAL CHKEQL
C     ..
C     .. External Subroutines ..
      EXTERNAL EXSECT,MGULPR,MPOSN,MRDHDR,PKFITP,PKSETM
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MAX
C     ..
C     .. Common blocks ..
      COMMON /INALM/ LCR1,LCR2,LCO1,LCO2
      COMMON /BESSPH/SPHMIN,SPHMAX,NBES,NBETAG
      COMMON /PARAMS/LPRINT,LMAP,LPLOT,LRDMP,LRDAL,PEAK,MAXPEK,LSAVE,
     +       LPKRMS,LREFMX(2)
      COMMON /PEKPK/NPEAK,PEAKS(MAXNPK),PKPOS(3,MAXNPK),LPEAK,
     +       LPKORD(MAXNPK),LPKOUT
      COMMON /RANGE/BTMIN,BTMAX,ASTEP,BSTEP,GSTEP,NALPHA,NBETA,NGAMMA,
     +       IRSYGP,LSEC1,LSEC2
C     ..
C     .. Equivalences ..
      EQUIVALENCE (ASTEP,ABGSTP(1)), (BSTEP,ABGSTP(2)),
     +            (GSTEP,ABGSTP(3))
C     ..
C     .. Save statement ..
      SAVE 
C     ..
C     .. Data statements ..
C
C---- OFFMAX is maximum allowed offset from grid position
C
      DATA OFFMAX/0.6/, N /0/
C     ..
C
C---- search 3*3*3 box for peak
C
      KEYS = 27
      MAXREF = 27
      ITEM = 1
      IDIM = MAXREF*ITEM
C
C---- set up paraboloid fit
C
      CALL PKSETM
C
      DO 10 I = 1,MAXNPK
        PEAKS(I) = 0.0
   10 CONTINUE
      IEND = 1
      IMAP = 1
      MODE = 2
C
      WRITE (6,FMT=6000)
C
C---- reopen map to search for peaks
C
      CALL MRDHDR(IMAP,'MAPOUT',MAPTIT,NBETA,IUVW,NXYZ,IB1,IA1,IA2,IG1,
     +            IG2,CELLM,IRSYGP,MODE,RHMIN,RHMAX,RHMEAN,RMS)
C
C---- set astep bstep gstep if you are just
C     reading map to search for peaks
C
C  cell(1 2 3 ) set = 360.
C  nxyz(1 2 3) = grid
C
      IF (LRDMP.GT.0) THEN
        ASTEP = CELLM(1)/NXYZ(1)
        BSTEP = CELLM(2)/NXYZ(2)
        GSTEP = CELLM(3)/NXYZ(3)
      END IF
      NA1 = 1
      NA2 = IA2 - IA1 + 1
      NG1 = 1
      NG2 = IG2 - IG1 + 1
C
      IF (LPKRMS.EQ.0) THEN
C
C---- Peak threshold is absolute
C
        PKTHRD = PEAK
        WRITE (6,FMT=6002) PKTHRD
      ELSE
        PKTHRD = PEAK*RMS
        WRITE (6,FMT=6004) PEAK,RMS
      END IF
C
C---- To handle edge effects, each section is extended by copying the
C     2nd & penultimate rows and columns to give a extra row & column
C     all round. This is equivalent to assuming each edge is a mirror
C     plane, which is unlikely to be true, however it will force all
C     edge peaks to lie on the edge plane. Similarly, sandwich the 1st
C     section, between 2 copies of the 2nd section, and the last section
C     between 2 copies of the penultimate
C
      IBEG = -1
      IEND = 1
      I1 = 1
      I2 = 2
      I3 = 3
      CALL MPOSN(IMAP,LSEC1)
      CALL MGULPR(IMAP,RI2,IER)
      IF (IER.LE.0) THEN
        DO 30 J = NG1,NG2
          INDJ = (J-1)*NA2
          DO 20 I = NA1,NA2
            RMAP(I,J,I2) = RI2(I+INDJ)
   20     CONTINUE
   30   CONTINUE
C
C---- Extend section around edges
C
        CALL EXSECT(RMAP(0,0,I2),NALPH,NGAMM)
C
        DO 220 NB = LSEC1 + 1,LSEC2 + 1
C
C---- read next section  - except for last section flagged by i3=-1
C
          IF (IEND.GT.0) THEN
            CALL MGULPR(IMAP,RI2,IER)
            IF (IER.GT.0) THEN
              GO TO 230
            ELSE
C
              DO 50 J = NG1,NG2
                INDJ = (J-1)*NA2
                DO 40 I = NA1,NA2
                  RMAP(I,J,I3) = RI2(I+INDJ)
   40           CONTINUE
   50         CONTINUE
C
C---- Extend section around edges
C
              CALL EXSECT(RMAP(0,0,I3),NALPH,NGAMM)
            END IF
          END IF
C
C---- Special for 2nd & penultimate sections
C
          IF (NB.EQ.LSEC1+1) THEN
C
C---- 2nd section, copy slot 3 to 1
C
            DO 70 J = NG1 - 1,NG2 + 1
              DO 60 I = NA1 - 1,NA2 + 1
                RMAP(I,J,I1) = RMAP(I,J,I3)
   60         CONTINUE
   70       CONTINUE
          ELSE IF (NB.EQ.LSEC2+1) THEN
C
C---- Last section, copy slot 1 to 3
C
            DO 90 J = NG1 - 1,NG2 + 1
              DO 80 I = NA1 - 1,NA2 + 1
C
C---- If only one section, fill with dummy values
C
                IF (LSEC1.EQ.LSEC2) RMAP(I,J,I1) = -100.0
                RMAP(I,J,I3) = RMAP(I,J,I1)
   80         CONTINUE
   90       CONTINUE
          END IF
C
C---- Now we have a section  in slot I2,
C     and maybe in slots I1 and I3 if slot
C     numbers are .gt. 0, so search section I2
C
C---- Search for peaks above threshold PKTHRD
C
          DO 210 I = NA1,NA2
            DO 200 J = NG1,NG2
              R = RMAP(I,J,I2)
C
C---- Is this point above PKTHRD?
C
              IF (R.GT.PKTHRD) THEN
C
C---- Yes, are there any larger neighbours?
C     Build array PEAKMX(-1:+1,-1:+1,-1:+1) centred on RMAP(I,J,I2);
C      ie PEAKMX(0,0,0)=RMAP(I,J,I2)
C
                RMAX = -1000.0
                DO 120 L2 = -1,1
                  DO 110 L1 = -1,1
                    PEAKMX(L1,L2,-1) = RMAP(I+L1,J+L2,I1)
                    PEAKMX(L1,L2,0) = RMAP(I+L1,J+L2,I2)
                    PEAKMX(L1,L2,+1) = RMAP(I+L1,J+L2,I3)
                    DO 100 L3 = -1,1
                      RMAX = MAX(RMAX,PEAKMX(L1,L2,L3))
  100               CONTINUE
  110             CONTINUE
  120           CONTINUE
C
C---- Is central point the highest?
C
                IF (RMAX.LE.R) THEN
C
C---- No neighbours higher, so we have a peak
C
                  N = N + 1
                  IPEAK = N
                  IF (N.GT.MAXPEK) THEN
C
C---- We've already got enough peaks,
C     so find smallest if smaller than the new one
C
                    N = N - 1
                    A = R
                    DO 130 K = 1,MAXPEK
                      IF (PEAKS(K).LT.A) THEN
                        IPEAK = K
                        A = PEAKS(K)
                      END IF
  130               CONTINUE
                    IF (IPEAK.GT.N) GO TO 200
                  END IF
C
C---- Append to list
C
C---- Store peak in slot ipeak
C
                  PEAKS(IPEAK) = R
C
C---- Interpolate point
C
                  CALL PKFITP(PEAKMX,OFFMAX,OFFSET,ISTAT)
C
C---- Store angles
C
                  PKPOS(IUVW(1),IPEAK) = I - 1 + IA1 + OFFSET(1)
                  PKPOS(IUVW(2),IPEAK) = J - 1 + IG1 + OFFSET(2)
                  PKPOS(IUVW(3),IPEAK) = NB - 1 + OFFSET(3)
C
C---- Convert coordinates to angles (degrees)
C     multiply peak positions by angular step
C
                  PKPOS(1,IPEAK) = PKPOS(1,IPEAK)*ASTEP
                  PKPOS(2,IPEAK) = PKPOS(2,IPEAK)*BSTEP
                  PKPOS(3,IPEAK) = PKPOS(3,IPEAK)*GSTEP
C
C---- check section beta = 0
C
                  BCHCK = PKPOS(2,IPEAK)
                  IF (ABS(BCHCK).LE.0.001) THEN
C
C---- set pkpos(1,..)= Alpha + gamma, pkpos(3,..)=0.
C
                    PKPOS(1,IPEAK) = PKPOS(1,IPEAK) + PKPOS(3,IPEAK)
                    PKPOS(3,IPEAK) = 0.0
C
C---- is this a second peak?
C
                    IF (N.GT.1) THEN
                      DO 140 K = 1,N - 1
C
C---- has it equal angles within step limits NB ABGSTP === A/B/GSTEP
C
                        IF (CHKEQL(PKPOS(1,K),PKPOS(1,IPEAK),
     +                      ABGSTP)) GO TO 150
  140                 CONTINUE
C
                      GO TO 160
  150                 N = N - 1
                    END IF
C
C---- bangl ne 0
C
  160               CONTINUE
                  END IF
C
C---- check beta = 180.
C
                  IF (ABS(BCHCK-180.0).LT.0.001) THEN
C
C---- set pkpos(3,..)= Gamma - alpha, pkpos(1,..)=0.
C
                    PKPOS(3,IPEAK) = PKPOS(3,IPEAK) - PKPOS(1,IPEAK)
                    PKPOS(1,IPEAK) = 0.0
C
C---- is this a second peak?
C
                    IF (N.GT.1) THEN
                      DO 170 K = 1,N - 1
C
C---- has it equal angles within step limits NB ABGSTP === A/B/GSTEP
C
                        IF (CHKEQL(PKPOS(1,K),PKPOS(1,IPEAK),
     +                      ABGSTP)) GO TO 180
  170                 CONTINUE
C
                      GO TO 190
  180                 N = N - 1
                    END IF
C
C---- bchck ne 0 or 180
C
  190               CONTINUE
                  END IF
                END IF
              END IF
  200       CONTINUE
  210     CONTINUE
C
C
C
C---- Set pointers for next section
C     we have passed the first section
C
          IBEG = 1
          I1 = I2
          I2 = I3
          I3 = I2 + 1
          IF (I3.GT.3) I3 = 1
C
C---- dont reset i3 here - build array peakmx then reset bits = -1000
C     if this is the last section.
C
          IF (NB.EQ. (LSEC2)) IEND = -1
  220   CONTINUE
        NPEAK = N
C
        RETURN
      END IF
C
  230 CALL CCPERR(1, '  error in map input')
C
C---- Format statements
C
 6000 FORMAT ('1',21X,'P E A K   S E A R C H',//)
 6002 FORMAT ('      Peaks must be greater than ',F8.2)
 6004 FORMAT (//'   Peaks must be greater than ',F8.2,'  times RMS den',
     +       'sity',F12.4)
C
C
      END
C
C
C
      SUBROUTINE PKFITP(PEAKMX,OFFMAX,OFFSET,ISTAT)
C     =================================================
C
C---- Fit paraboloid to 3 x 3 x 3 grid in PEAKMX (central point
C     assumed to be maximum), return position of
C     maximum in OFFSET (grid units)
C
C Subroutine PKSETM must have been called first to calculate solution
C matrix SMAT
C
C On entry:
C  PEAKMX       values of peak  (3 x 3 x 3 grid)
C  OFFMAX       maximum value of offset
C
C On exit:
C  OFFSET(3)    offsets in grid units
C  ISTAT        = 1 if offset reset to maximum, else = 0
C
C
C
C
C
C
C
C
C Solve for parameters P(1)->P(10), = a,b,c,d,e,f,g,h,i,j
C
C    p  = [ (AT A)**-1 AT ]  r
C where  (AT A)**-1 AT = SMAT is calculated in PKSETM
C        r = PEAKMX, observation vector
C
C  Note that in PKSETM,
C   the order of grid points corresponding to the rows
C   of matrix A must correspond to the order of values
C   in PEAKMX, ie x fast, y medium, z slow
C
C     .. Parameters ..
      INTEGER NP,NE,NG
      PARAMETER (NP=10,NE=3,NG=NE*NE*NE)
C     ..
C     .. Scalar Arguments ..
      REAL OFFMAX
      INTEGER ISTAT
C     ..
C     .. Array Arguments ..
      REAL OFFSET(3),PEAKMX(-1:+1,-1:+1,-1:+1)
C     ..
C     .. Arrays in Common ..
      REAL A,SMAT
C     ..
C     .. Local Scalars ..
      REAL DET
      INTEGER I
C     ..
C     .. Local Arrays ..
      REAL B(3,3),G(3),P(NP)
      INTEGER W1(3),W2(3)
C     ..
C     .. External Subroutines ..
      EXTERNAL GMPRD,MINVN
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,SIGN
C     ..
C     .. Common blocks ..
      COMMON /PKFIT/SMAT(NP,NG),A(NG,NP)
C     ..
C     .. Save statement ..
      SAVE 
C     ..
C
C
      CALL GMPRD(SMAT,PEAKMX,P,NP,NG,1)
C
C Set up matrix B and vector g for solving position of maximum from the
C paraboloid parameters
C
C           (  2a   d   f  )                    (  -g  )
C     B  =  (   d  2b   e  )              g  =  (  -h  )
C           (   f   e  2c  )                    (  -i  )
C
C   Then position of maximum  OFFSET  =  x  =  B**-1 g
C
      B(1,1) = P(1)*2.0
      B(1,2) = P(4)
      B(2,1) = P(4)
      B(1,3) = P(6)
      B(3,1) = P(6)
      B(2,2) = P(2)*2.0
      B(2,3) = P(5)
      B(3,2) = P(5)
      B(3,3) = P(3)*2.0
C
      G(1) = -P(7)
      G(2) = -P(8)
      G(3) = -P(9)
C
C---- Invert B
C
      CALL MINVN(B,3,DET,W1,W2)
      IF (ABS(DET).LT.0.00000001) THEN
        CALL CCPERR(1, 'PKFITP: singular matrix')
      ELSE
C
        CALL GMPRD(B,G,OFFSET,3,3,1)
C
        ISTAT = 0
        DO 10 I = 1,3
          IF (ABS(OFFSET(I)).GT.OFFMAX) THEN
C
C---- Reset to maximum, set flag
C
            ISTAT = 1
            OFFSET(I) = SIGN(OFFMAX,OFFSET(I))
          END IF
   10   CONTINUE
C
      END IF
      END
C
C
C
      SUBROUTINE PKINTP(PEAKMX,OFFSET,QLIM)
C     =======================================
C
C---- Interpolate peak position by 3 1-dimensional 3-point parabolas
C
C On entry:
C       PEAKMX(3,3,3)  part of grid around peak. Central point is
C                       assumed to be maximum. In PEAKMX(I,J,K),
C                       I corresponds to 1st axis, J to 2nd, K to 3rd
C       QLIM            value to test for edge points: points with
C                       PEAKMX .lt. QLIM will be ignored
C
C On exit:
C       OFFSET(3)       offset from central point in grid units, along
C                       the 3 axes
C
C
C
C     .. Scalar Arguments ..
      REAL QLIM
C     ..
C     .. Array Arguments ..
      REAL OFFSET(3),PEAKMX(3,3,3)
C     ..
C     .. Local Scalars ..
      REAL R,R1,R2,RMINUS,RPLUS
      INTEGER I,L
C     ..
C     .. Local Arrays ..
      INTEGER LMINUS(3),LPLUS(3)
C     ..
C
C
      R = PEAKMX(2,2,2)
C
C---- Loop 3 axes
C
      DO 20 L = 1,3
C
C---- Set up array indices
C
        DO 10 I = 1,3
          IF (I.EQ.L) THEN
            LPLUS(I) = 3
            LMINUS(I) = 1
          ELSE
            LPLUS(I) = 2
            LMINUS(I) = 2
          END IF
   10   CONTINUE
C
        RPLUS = PEAKMX(LPLUS(1),LPLUS(2),LPLUS(3))
        RMINUS = PEAKMX(LMINUS(1),LMINUS(2),LMINUS(3))
C
C---- Check for junk points
C
        IF (RMINUS.LE.QLIM .OR. RPLUS.LE.QLIM) THEN
C
C---- Set equal to central point
C
          OFFSET(L) = 0.0
C
C---- Check which of the flanking points is nearer
C     in value to central point
C
        ELSE IF (RPLUS.GT.RMINUS) THEN
          R2 = RPLUS - RMINUS
          R1 = R - RPLUS
          OFFSET(L) = R2/ (2.0*R2+4.0*R1)
        ELSE
          R2 = RPLUS - RMINUS
          R1 = R - RMINUS
          OFFSET(L) = -R2/ (2.0*R2-4.0*R1)
        END IF
   20 CONTINUE
C
C
      END
C
C
C
      SUBROUTINE PKSETM
C     =================
C
C---- Set up normal matrix etc for 3D paraboloid fit
C
C Observational equation is
C  r(calc) = a x**2 + b y**2 + c z**2 + d xy + e yz + f zx +
C            g x + h y + i z + j
C
C Observation vector
C  xT = ( x**2  y**2  z**2  xy  yz  zx   x   y   z   1 )
C
C Observation matrix
C
C  A  =  (  x(1)T  )          for each point on 3 x 3 x 3 grid
C        (  x(2)T  )
C        ( . . . . )
C        (  x(n)T  )
C
C
C
C
C
C---- Loop points on 3 x 3 x 3 grid
C
C     .. Parameters ..
      INTEGER NP,NE,NG
      PARAMETER (NP=10,NE=3,NG=NE*NE*NE)
C     ..
C     .. Arrays in Common ..
      REAL A,SMAT
C     ..
C     .. Local Scalars ..
      REAL DET,X,Y,Z
      INTEGER I,IX,IY,IZ,J
C     ..
C     .. Local Arrays ..
      REAL AT(NP,NG),ATA(NP,NP)
      INTEGER W1(NP),W2(NP)
C     ..
C     .. External Subroutines ..
      EXTERNAL GMPRD,MINVN
C     ..
C     .. Common blocks ..
      COMMON /PKFIT/SMAT(NP,NG),A(NG,NP)
C     ..
C     .. Save statement ..
      SAVE 
C     ..
C
C
      I = 0
      DO 30 IZ = -1,+1
        Z = IZ
        DO 20 IY = -1,+1
          Y = IY
          DO 10 IX = -1,+1
            X = IX
            I = I + 1
C
C---- Matrix elements
C
            A(I,1) = X**2
            A(I,2) = Y**2
            A(I,3) = Z**2
            A(I,4) = X*Y
            A(I,5) = Y*Z
            A(I,6) = Z*X
            A(I,7) = X
            A(I,8) = Y
            A(I,9) = Z
            A(I,10) = 1.0
   10     CONTINUE
   20   CONTINUE
   30 CONTINUE
C
C---- Form A(transpose)
C
      DO 50 I = 1,NG
        DO 40 J = 1,NP
          AT(J,I) = A(I,J)
   40   CONTINUE
   50 CONTINUE
C
C---- A(tranpose) * A
C
      CALL GMPRD(AT,A,ATA,NP,NG,NP)
C
C---- Invert in place
C
      CALL MINVN(ATA,NP,DET,W1,W2)
      IF (ABS(DET).LT.0.0000001) THEN
        CALL CCPERR(1, 'PKSETM: singular matrix')
      ELSE
C
C
        CALL GMPRD(ATA,AT,SMAT,NP,NP,NG)
C
      END IF
C
C
      END
C
C
C
      SUBROUTINE POLANG(DC,OMEGA,PHI)
C     ===============================
C
C---- Find polar angles OMEGA & PHI (degrees)
C     from direction cosines DC
C
C     OMEGA in range 0 to 180, PHI in range 0 to 360
C
C
C
C     .. Scalar Arguments ..
      REAL OMEGA,PHI
C     ..
C     .. Array Arguments ..
      REAL DC(3)
C     ..
C     .. Local Scalars ..
      REAL CONV,SOMEGA
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,ATAN,ATAN2,SQRT
C     ..
C
C
      CONV = 45.0/ATAN(1.0)
C
      SOMEGA = SQRT(ABS(1.0-DC(3)*DC(3)))
      OMEGA = ATAN2(SOMEGA,DC(3))*CONV
      IF (ABS(DC(1)).LE.0.000001 .AND. ABS(DC(2)).LE.0.000001) THEN
        PHI = 0.0
      ELSE
        PHI = ATAN2(DC(2),DC(1))*CONV
      END IF
C
C
      END
C
C
C                        
C
      SUBROUTINE RFMOD(FMM,BETA,STEP,MSTEP1,MSTEP2,MSTP11,GMIN,AMIN)
C     ==============================================================
C
C
C---- ejd this subroutine deals with output of rotation function map
C     or printout of sections
C
C
C
C
C
C
C     .. Parameters ..
      INTEGER NBTGMX
      PARAMETER (NBTGMX=72)
      INTEGER MAXNPK
      PARAMETER (MAXNPK=100)
C     ..
C     .. Scalar Arguments ..
      REAL BETA,STEP,AMIN,GMIN
      INTEGER MSTEP1,MSTEP2,MSTP11
C     ..
C     .. Array Arguments ..
      REAL FMM(MSTEP2,MSTEP2)
C     ..
C     .. Scalars in Common ..
      REAL ASTEP,BSTEP,BTMAX,BTMIN,GSTEP,PEAK
      INTEGER IRSYGP,LCO1,LCO2,LCR1,LCR2,LMAP,LPEAK,LPKOUT,
     +        LPKRMS,LPLOT,LPRINT,LRDAL,LRDMP,LSAVE,LSEC1,LSEC2,MAXPEK,
     +        NALPHA,NBETA,NGAMMA,NPEAK,LREFMX
      CHARACTER TITLE*80
C     ..
C     .. Arrays in Common ..
      REAL PEAKS,PKPOS
      INTEGER LPKORD
C     ..
C     .. Local Scalars ..
      REAL AT,RHMAX,RHMEAN,RHMEN1,RHMIN,RMAX,RMIN,RMSMN,
     +     RMSMN1,SCALE,RHWT,RBETA
      INTEGER I,I1,I2,IA1,IA2,IB1,IG1,IG2,IM,IMAP,IUP,J,N,NBSEC,NP,
     +        NPAGE,NPP,NSECT
      LOGICAL FIRST
C     ..
C     .. Local Arrays ..
      REAL ANGLA(2*NBTGMX),ANGLG(2*NBTGMX),CELLM(6)
      INTEGER IRF(NBTGMX),IUVW(3),NXYZ(3)
      CHARACTER CHRF(NBTGMX)*3
C     ..
C     .. External Subroutines ..
      EXTERNAL MCLOSC,MPOSNW,MWRHDR,MWRSEC
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC INT,MIN,MOD,NINT,REAL,SQRT
C     ..
C     .. Common blocks ..
      COMMON /INALM/ LCR1,LCR2,LCO1,LCO2
      COMMON /PARAMS/LPRINT,LMAP,LPLOT,LRDMP,LRDAL,PEAK,MAXPEK,LSAVE,
     +       LPKRMS,LREFMX(2)
      COMMON /PEKPK/NPEAK,PEAKS(MAXNPK),PKPOS(3,MAXNPK),LPEAK,
     +       LPKORD(MAXNPK),LPKOUT
      COMMON /RANGE/BTMIN,BTMAX,ASTEP,BSTEP,GSTEP,NALPHA,NBETA,NGAMMA,
     +       IRSYGP,LSEC1,LSEC2
      COMMON /TITL/TITLE
C     ..
      SAVE 
C
C     .. Data statements ..
      DATA FIRST/.TRUE./
      DATA CELLM/6*0.0/
C     ..
C
C  Beta in radians
      RBETA = BETA * ATAN(1.0)/45.0
C
C---- Initialise arrays
C
      IF (FIRST) THEN
        IUP = 4
        IF (NALPHA.LE.72) IUP = 2
        IF (NALPHA.LE.36) IUP = 1
        NSECT = 0
C
C##### error amin gmin not defined values now come from FRFS
C
        DO 10 I = 1,MSTEP2
          AT = AMIN + 360.0 + REAL(I-1)*STEP
          ANGLA(I) = MOD(AT,360.0)
          AT = GMIN + 360.0 + REAL(I-1)*STEP
          ANGLG(I) = MOD(AT,360.0)
   10   CONTINUE
      END IF
C
C---- Find maximum of section and store
C
      RMAX = -1.0E10
      RMIN = -RMAX
      NSECT = NSECT + 1
      PKPOS(2,NSECT) = BETA
      DO 30 I = 1,NALPHA
        DO 20 J = 1,NGAMMA
          IF (FMM(I,J).GT.RMAX) THEN
            RMAX = FMM(I,J)
            PKPOS(1,NSECT) = ANGLA(I)
            PKPOS(3,NSECT) = ANGLG(J)
            PEAKS(NSECT) = FMM(I,J)
          END IF
          IF (FMM(I,J).LT.RMIN) RMIN = FMM(I,J)
   20   CONTINUE
   30 CONTINUE
      IF (ABS(PEAK).LE.0.000001) NPEAK = NSECT
C
C---- Output scale factor
C
      IF (FIRST) THEN
        IF (RMAX.GT.1) SCALE = 50.0/RMAX
        IF (RMAX.LE.1) SCALE = 50.0/ (-RMIN)
        FIRST = .FALSE.
C
C---- Scale output etc
C
        RHMIN = 10.0E20
        RHMAX = -10.0E20
        RMSMN = 0.0
        RHMEAN = 0.0
        RHWT = 0.0
        IF (LMAP.GT.0) THEN
C
C---- set 'grid' to 360/angle step -
C     I think all axes will be the same.
C     then it should be easier to work out 'symmetry'
C
C  eg.  180-alpha should equal 1/2-x
C
          NXYZ(1) = INT(360.0/STEP+0.5)
          NXYZ(2) = INT(360.0/BSTEP+0.5)
          NXYZ(3) = NXYZ(1)
C
C---- alpha fastest- gamma second - beta sections.
C
          IUVW(1) = 1
          IUVW(2) = 3
          IUVW(3) = 2
          CELLM(1) = 360.0
          CELLM(3) = 360.0
          CELLM(2) = 360.0
          CELLM(4) = 90.0
          CELLM(5) = 90.0
          CELLM(6) = 90.0
          IA1 = 0
          IA2 = NALPHA - 1
          IG1 = 0
          IG2 = NGAMMA - 1
          IB1 = NINT(BTMIN/BSTEP)
          LSEC1 = IB1
          LSEC2 = LSEC1 + NBETA - 1
          IMAP = 1
C
C---- open file with logical name MAPOUT
C
          CALL MWRHDR(IMAP,TITLE,NBETA,IUVW,
     +                NXYZ,IB1,IA1,IA2,IG1,IG2,
     +                CELLM,IRSYGP,2)
        END IF
        WRITE (6,FMT=6006) SCALE
C
        WRITE (6,FMT=6000)
C
C
      END IF
C
C---- scale maximum
C
      PEAKS(NSECT) = PEAKS(NSECT)*SCALE
CC      nbsec = nint((beta-btmin)/bstep)
        NBSEC = NINT(BETA/BSTEP)
      WRITE (6,FMT=6002) NBSEC,PKPOS(1,NSECT),PKPOS(2,NSECT),
     +  PKPOS(3,NSECT),PEAKS(NSECT)
      IF (LMAP.NE.0 .OR. LPRINT.NE.-100) THEN
C
C---- otherwise out put sections
C     output sections
C
        N = 0
        DO 50 I = 1,NGAMMA
          DO 40 J = 1,NALPHA
            N = N + 1
            FMM(J,I) = (FMM(J,I)*SCALE)
            IF (FMM(J,I).LT.RHMIN) RHMIN = FMM(J,I)
            IF (FMM(J,I).GT.RHMAX) RHMAX = FMM(J,I)
C Mean & RMS need to be weighted by sin beta, to allow for distortions
c     in Eulerian space.
c     On the last section, if beta = 90, the weight is 1/2 because of
c     symmetry. Here we only need to test for last section, because
c     (a) if last section = 180deg, the weight is zero anyway
c     (b) if the last section is not 90 or 180, the rms is not valid anyway
            WEIGHT = SIN(RBETA)
            IF (NSECT .EQ. NBETA) WEIGHT = WEIGHT/2.0
            RMSMN =WEIGHT*FMM(J,I)*FMM(J,I) + RMSMN
            RHMEAN = WEIGHT*FMM(J,I) + RHMEAN
            RHWT = RHWT + WEIGHT
   40     CONTINUE
   50   CONTINUE
C
C
        IF (LMAP.NE.0) THEN
          CALL MPOSNW(IMAP,NBSEC)
          CALL MWRSEC(IMAP,FMM,MSTEP2,MSTEP2,1,NALPHA,1,NGAMMA)
          IF (NSECT.EQ.NBETA) THEN
C  Weighted mean & RMS deviation from 0 for output file (mean should = 0)
             RHMEAN = RHMEAN/RHWT
             RMSMN = RMSMN/RHWT - RHMEAN**2
             IF (RMSMN .GT. 0.0) THEN
                RMSMN = SQRT(RMSMN)
             ELSE
                RMSMN = 0.0
             ENDIF
             CALL MCLOSC(IMAP,RHMIN,RHMAX,RHMEAN,RMSMN)
          END IF
        END IF
C
C---- Output lineprinter sections
C
        IF (LPRINT.NE.-100) THEN
          DO 90 NPAGE = 1,IUP
            WRITE (6,FMT=6008) BETA
            NP = (NPAGE-1)*36
            I1 = NP + 1
            NPP = MIN(36,NALPHA-NP)
            DO 60 I = 1,NPP
              IRF(I) = NINT(ANGLA(I+NP))
   60       CONTINUE
            I2 = NP + NPP
            WRITE (6,FMT=6010) (IRF(I),I=1,NPP)
            DO 80 J = 1,NGAMMA
              DO 70 I = 1,NPP
                IRF(I) = INT(FMM(I+NP,J)+0.5)
                CHRF(I) = '   '
                IF (IRF(I).GT.LPRINT) WRITE (CHRF(I),FMT=6004) IRF(I)
   70         CONTINUE
              WRITE (6,FMT=6012) ANGLG(J), (CHRF(IM),IM=1,NPP)
   80       CONTINUE
   90     CONTINUE
          RMSMN1 = SQRT(RMSMN1/N)
          RHMEN1 = RHMEN1/N
C
          WRITE (6,FMT=6014)
C
C
        END IF
      END IF
C
C---- Format statements
C
 6000 FORMAT (//'  List of maxima on each Beta section ',//'  Section ',
     +       'number     Alpha   Beta    Gamma  Peak height',/)
 6002 FORMAT (10X,I5,5X,3F7.1,3X,F8.1)
 6004 FORMAT (I3)
 6006 FORMAT (///' Multiplying Scale Factor used on RF Output= ',E12.4)
 6008 FORMAT (//'   ALPHA HORIZ, GAMMA VERTICAL, BETA=',F5.1,/)
 6010 FORMAT (8X,36I3,/)
 6012 FORMAT (1X,F5.1,2X,36A3)
 6014 FORMAT (///)
C
C
      END
C
C
C
      SUBROUTINE RMAT
C     ===============
C
C---- Fast rotation function
C     dlmm from recurrence relations  altmann phil. trans.
C
C---- Special case beta
C
C     .. Parameters ..
      INTEGER NBESMX
      PARAMETER (NBESMX=60)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)             
      INTEGER NCLIM,NSLIM
      PARAMETER (NCLIM=80000,NSLIM=80000)
C     ..
C     .. Scalars in Common ..
      DOUBLE PRECISION RTD
      REAL BETA
      INTEGER LCO1,LCO2,LCR1,LCR2,LMAX,LMAX2
C     ..
C     .. Arrays in Common ..
      DOUBLE PRECISION FACT
      REAL ARSYM,RSYM,RSYMT
      INTEGER JSYMP
C     ..
C     .. Local Scalars ..
      DOUBLE PRECISION ANG,CONV,CTB2,D,E,SIND,V1,V2,V3
      INTEGER I1,I2,IADD,IARG,IDIF,ILW,IP,ISUB,J1,J2,K1,K2,L,L1,L21,M,
     +        M1,M2,M221,MI1,MI2
C     ..
C     .. Local Arrays ..
      DOUBLE PRECISION IEXP(NBESMX+1,2*NBESMX+1)
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,DCOS,DBLE,EXP,INT,LOG,DLOG10,MOD,REAL,SIGN,
     +          DSIN,SQRT
C     ..
C     .. Common blocks ..
      COMMON /INALM/ LCR1,LCR2,LCO1,LCO2
      COMMON /RDLM/RTD,FACT(2*NBESMX+1),BETA,LMAX,LMAX2
      COMPLEX CLMM
      REAL    SLMM
       COMMON /COMCSL/SLMM(NSLIM),CLMM(NCLIM)
       DOUBLE PRECISION DLMM(NBESMX+1,2*NBESMX+1),
     +       PCOT(2*NBESMX+1),PSIN(2*NBESMX+1)
      COMMON /SYMTR/JSYMP(2),RSYM(4,4,MAXSYM,2),RSYMT(4,4,MAXSYM,2),
     +       ARSYM(3,3,MAXSYM,2)
C     ..
      save
C
cc     log - natural logarithm
cc     Returns the natural logarithm of a real or complex number.
cc     log-val.as.rc = log (number.as.rc)
cc  An argument that is a real number must be greater than 0.0. An argument
cc  that is a complex number must not equal (0,0). If the real part of a com-
cc  plex argument is less than 0.0 and the imaginary part is 0.0, the result
cc  is pi. The imaginary part of the result of a complex argument is always in
cc  the range -pi, exclusive, through +pi, inclusive.
cc     *** standardization note ***
cc     The specific function name cdlog, listed below, is an extension.
cc          Specific    Arg Type    Result Type Generic
cc          alog        real*4      real*4      log
cc          dlog        real*8      real*8      log
cc          clog        complex*8   complex*8   log
cc          cdlog       complex*16  complex*16  log
cc     *** extension ***
cc The use of -r8 on the fortran command line causes alog to be treated as
cc log in an expression and as dlog when passed as an actual argument; and
cc causes clog to be treated as log in an expression and as cdlog when passed
cc as an actual argument.
C
cc     sin - sine in radians
cc     Returns the sine of a number in radians.
cc     sin-val.as.rc = sin (number.as.rc)
cc The argument must be between -2**17 and 2**17, inclusive, in single preci-
cc sion, and between -2**31-2**12+1 and 2**31+2**12-1, inclusive, in double
cc precision, to be in range.
cc     *** standardization note ***
cc     The specific function name cdsin, listed below, is an extension.
cc          Specific    Arg Type    Result Type Generic
cc          sin         real*4      real*4      sin
cc          dsin        real*8      real*8      sin
cc          csin        complex*8   complex*8   sin
cc          cdsin       complex*16  complex*16  sin
cc     *** extension ***
cc The use of -r8 on the fortran command line causes sin to be treated as
cc dsin when passed as an actual argument; and causes csin to be treated as
cc sin in an expression and as cdsin when passed as an actual argument.
cc
      IF (BETA.NE.0.0) THEN
        CONV = DLOG(10.0D00)
        ANG = DBLE(BETA)*RTD
        SIND = DSIN(ANG)
        CTB2 = DCOS(ANG)/SIND
        PCOT(1) = 0.0D00
        PSIN(1) = 0.0D00
C
C
        DO 10 IP = 1,LMAX2
          PCOT(IP+1) = PCOT(IP) + DLOG10(CTB2) 
          PSIN(IP+1) = PSIN(IP) + DLOG10(SIND) 
   10   CONTINUE
      END IF
C
C---- Recurrence relations
C
      ILW = 0
C
C
      DO 160 L = 2,LMAX,2
        L1 = L + 1
        L21 = 2*L + 1
C
C
        DO 30 I1 = 1,L1
          DO 20 I2 = 1,L21
            IEXP(I1,I2) = 0.0D00
            DLMM(I1,I2) = 0.0D00
   20     CONTINUE
   30   CONTINUE
C
C
        IF (ABS(BETA).LE.0.000001) THEN
C
C---- Special case beta=0.
C
          DO 40 M1 = 1,L1
            M2 = M1 + L
            IEXP(M1,M2) = 0.0D00
            DLMM(M1,M2) = 1.0D00
   40     CONTINUE
        ELSE
C
          DO 50 M1 = 1,L21
            M = M1 - L - 1
            D = (FACT(2*L+1)-FACT(L+M+1)-FACT(L-M+1))*0.5D00 +
     +          PCOT(L+M+1) + PSIN(2*L+1) 
cc
cc            iexp(l1,m1) = int(d)  !!!! is this a serious error
cc
            IEXP(L1,M1) = D
cc     mod - remainder
cc     Returns the remainder of a division between two numbers.
cc     mod-val.as.ir = mod (dividend.as.ir, divisor.as.ir)
cc     The result is as follows:
cc          dividend-(int(dividend/divisor)*divisor)
cc     The result is undefined if divisor is 0.
cc     *** standardization note ***
cc The specific function names imod and jmod, listed below, are extensions.
cc          Specific    Arg Type    Result Type Generic
cc          imod        integer*2   integer*2   mod
cc          jmod        integer*4   integer*4   mod
cc          mod         integer*4   integer*4   mod
cc          amod        real*4      real*4      mod
cc          dmod        real*8      real*8      mod
cc     *** extension ***
cc Mixed integer*2, integer*4, and integer*8 arguments are permitted; a warn-
cc ing message is issued when mixing occurs.
cc     *** extension ***
cc The use of 8-byte integer or logical values affects the behavior of many
cc generic intrinsic functions. See Appendix I for details.
cc     *** extension ***
cc The use of -r8 on the fortran command line causes amod to be treated as
cc mod in an expression and as dmod when passed as an actual argument.
cc
cc     abs - absolute value
cc     Returns the absolute value of a number.
cc     abs-val.as.ir = abs (number.as.irc)
cc If number is real, the result is the square root of the argument squared.
cc If number is complex, the result is the square root of the sum of the real
cc part squared and the imaginary part squared (the result is a real number).
cc     *** standardization note ***
cc  The specific function names iiabs, jiabs, and cdabs, listed below, are
cc     extensions.
cc          Specific    Arg Type    Result Type Generic
cc          iiabs       integer*2   integer*2   abs
cc          jiabs       integer*4   integer*4   abs
cc          iabs        integer*4   integer     abs
cc          abs         real*4      real*4      abs
cc          dabs        real*8      real*8      abs
cc          cabs        complex*8   real*4      abs
cc          cdabs       complex*16  real*8      abs
cc     *** extension ***
cc The use of -r8 on the fortran command line causes abs to be treated as
cc dabs when passed as an actual argument; and causes cabs to be treated as
cc abs in an expression and as cdabs when passed as an actual argument.
cc     *** extension ***
cc The use of 8-byte integer or logical values affects the behavior of many
cc generic intrinsic functions. See Appendix I for details.
cc
cc     sign - sign transfer
cc     Transfers the sign of one number to another.
cc     sign-val.as.ir = sign (number.as.ir, sign.as.ir)
cc The result is the value of number with the sign of sign.  If number is 0,
cc the result is neither positive or negative.  If sign is 0, the result is
cc positive.
cc     *** standardization note ***
cc The specific function names iisign and jisign, listed below, are exten-
cc  sions.
cc          Specific    Arg Type    Result Type Generic
cc          iisign      integer*2   integer*2   sign
cc          jisign      integer*4   integer*4   sign
cc          isign       integer     integer*2   sign
cc          sign        real*4      real*4      sign
cc          dsign       real*8      real*8      sign
cc     *** extension ***
cc Mixed integer*2, integer*4, and integer*8 arguments are permitted; a warn-
cc ing message is issued when mixing occurs.
cc     *** extension ***
cc The use of 8-byte integer or logical values affects the behavior of many
cc generic intrinsic functions. See Appendix I for details.
cc     *** extension ***
cc The use of -r8 on the fortran command line causes sign to be treated as
cc dsign when passed as an actual argument.
cc
            E = DMOD(DABS(D),1.0D00)
            D = DSIGN(E,D)
cc
cc     exp - exponential
cc  Returns the exponential of a real or complex number. The result is the
cc  value of e raised to the power of the argument.
cc     exp-val.as.rc = exp (number.as.rc)
cc     *** standardization note ***
cc     The specific function name cdexp, listed below, is an extension.
cc          Specific    Arg Type    Result Type Generic
cc          exp         real*4      real*4      exp
cc          dexp        real*8      real*8      exp
cc          cexp        complex*8   complex*8   exp
cc          cdexp       complex*16  complex*16  exp
cc
cc     *** extension ***
cc  The use of -r8 on the fortran command line causes exp to be treated as
cc  dexp when passed as an actual argument; and causes cexp to be treated as
cc  exp in an expression and as cdexp when passed as an actual argument.
cc
            DLMM(L1,M1) = DEXP(D*CONV)
   50     CONTINUE
C
C
          DO 90 I2 = 1,L
            M2 = L - I2
            MI2 = M2 + 1
            M221 = 2*M2 + 1
C
C
            DO 80 I1 = 1,M221
              M1 = I1 - M2 - 1
              MI1 = M1 + L1
              V1 = DLMM(MI2+1,MI1)
cc
cc      int - truncate to integer
cc     Converts a number to an integer by truncating the fraction if any.
cc     int-val.as.i = int (number.as.irc)
cc For an integer argument, the result is the same. For a real argument, the
cc result is the largest integer not exceeding the magnitude of the argument
cc and whose sign is the same as the argument; if the absolute value of the
cc argument is less than 1.0, the result is 0. For a complex argument, the
cc imaginary part is dropped.
cc     *** standardization note ***
cc The specific function names listed below are extensions. Fortran-77 lists
cc int, ifix, and idint as specific function names under int.
cc          Specific    Arg Type    Result Type Generic
cc          iint        real*4      integer*2   int
cc          iidint      real*8      integer*2   int
cc          jint        real*4      integer*4   int
cc          jidint      real*8      integer*4   int
cc
cc     *** extension ***
cc The result of the generic intrinsic int is the default integer type:
cc integer*2 if the -i2 option is in effect, integer*8 if the -i8 option is
cc in effect, or integer*4 otherwise.
cc     *** extension ***
cc The use of -r8 on the fortran command line causes int to be treated as
cc dint when passed as an actual argument.
cc
cc     *** extension ***
cc The use of 8-byte integer or logical values affects the behavior of many
cc generic intrinsic functions. See Appendix I for details.
cc
              K1 = DINT ( IEXP(MI2+1,MI1) )
              V2 = DLMM(MI2+1,MI1-1)
              K2 = DINT ( IEXP(MI2+1,MI1-1) )
              IDIF = ABS(K1-K2)
C
C
              IF (IDIF.NE.0) THEN
                IF (K1.LT.K2) THEN
                  K2 = K2 - IDIF
                  V2 = 10.0D00**DBLE(IDIF)*V2
                ELSE
                  K1 = K1 - IDIF
                  V1 = 10.0D00**DBLE(IDIF)*V1
                END IF
              END IF
C
C
cc     sqrt - square root
cc Returns the square root of a number. A real argument must be greater than
cc or equal to 0.0.
cc     sqrt-val.as.rc = sqrt (number.as.rc)
cc     *** standardization note ***
cc     The specific function name cdsqrt, listed below, is an extension.
cc          Specific    Arg Type    Result Type Generic
cc          sqrt        real*4      real*4      sqrt
cc          dsqrt       real*8      real*8      sqrt
cc          csqrt       complex*8   complex*8   sqrt
cc          cdsqrt      complex*16  complex*16  sqrt
cc     *** extension ***
cc The use of -r8 on the fortran command line causes csqrt to be treated as
cc sqrt in an expression and as cdsqrt when passed as an actual argument.
cc
              V3 = (DBLE(M2-M1+1)*CTB2*V1-
     +             DSQRT(DBLE((L-M1+1)* (L+M1)))*V2)/
     +             DSQRT(DBLE((L+M2+1)* (L-M2)))
              IADD = 0             
              ISUB = 0
C
C
              IF (DABS(V3).LE.0.000001D00) THEN
C
C----  V3 exactly = 0.0, don't try to scale it
C
                DLMM(MI2,MI1) = 0.0D00
                IEXP(MI2,MI1) = 0.0D00
              ELSEIF (V3.GT.10.0D00) THEN
   60           CONTINUE
                V3 = V3/10.0D00
                IADD = IADD + 1
                IF (DABS(V3).GE.10.0D00) GO TO 60
                DLMM(MI2,MI1) = V3
                IEXP(MI2,MI1) = DBLE ( K2 + IADD )
                IADD = 0
              ELSE IF (V3.LT.0.1D00) THEN
   70           CONTINUE
                V3 = V3*10.0D00
                ISUB = ISUB + 1
                IF (DABS(V3).LE.0.1D00) GO TO 70
                DLMM(MI2,MI1) = V3
                IEXP(MI2,MI1) = DBLE ( K2 - ISUB )
                ISUB = 0
              ELSE
                IEXP(MI2,MI1) = DBLE ( K2 )
                DLMM(MI2,MI1) = V3
              END IF
   80       CONTINUE
   90     CONTINUE
C
C---- Extend by symmetry
C
          DO 110 J2 = 2,L1
            M2 = J2 - 1
            DO 100 J1 = 1,M2
              M1 = 1 - J1
              I1 = L1 + M1
              IEXP(1-M1,L1-M2) = IEXP(J2,I1)
              DLMM(1-M1,L1-M2) = DLMM(J2,I1)
  100       CONTINUE
  110     CONTINUE
C
          DO 130 J2 = 2,L1
            M2 = J2 - 1
            DO 120 J1 = 1,M2
              M1 = J1 - 1
              IEXP(J1,L1+M2) = IEXP(J2,L1+M1)
              DLMM(J1,L1+M2) = DLMM(J2,L1+M1)
              IF (MOD(M1+M2,2).NE.0) DLMM(J1,L1+M2) = -DLMM(J1,L1+M2)
  120       CONTINUE
  130     CONTINUE
        END IF
C
C
C---- Output to printer if following  statements used
C
        DO 150 M1 = 1,L1
          DO 140 M2 = 1,L21
            IARG = (M1-1)*L21 + M2 + ILW
            D = DLMM(M1,M2)
            K1 = DINT ( IEXP(M1,M2) )
            K1 = K1 + 20
            IF (K1.GE.-38) THEN
              IF (K1.NE.-38 .OR. DABS(D).GT.0.14D00) THEN
                D = 10.0D00**DBLE(K1)*D
                SLMM(IARG) = REAL(D)
                GO TO 140
              END IF
            END IF
            SLMM(IARG) = 0.0
  140     CONTINUE
  150   CONTINUE
        ILW = L1*L21 + ILW
  160 CONTINUE
C
C---- Check precision all norms should be 1.0
C
      END
C
C
C
      SUBROUTINE RNGANG(A)
C     ====================
C
C---- Put angle A into range 0-360 degrees
C
C
C     .. Scalar Arguments ..
      REAL A
C     ..
C
C
   10 CONTINUE
      IF (A.LT.0.0) THEN
        A = A + 360.0
        GO TO 10
      END IF
   20 CONTINUE
      IF (A.GE.360.0) THEN
        A = A - 360.0
        GO TO 20
      END IF
C
C
      END
C
C
C
      SUBROUTINE RORDER(SYMMAT,NAX,NORDER)
C     ====================================
C
C---- Determine rotational order of matrix SYMMAT about
C     the NAX'th axis
C
C On entry:
C  SYMMAT(3,3) symmetry matrix acting on orthogonal coordinates
C  NAX         axis to test
C
C On exit:
C  NORDER      order of axis if valid (2,3,4,6 only),
C              else = 1
C
C
C
C     .. Scalar Arguments ..
      INTEGER NAX,NORDER
C     ..
C     .. Array Arguments ..
      REAL SYMMAT(3,3)
C     ..
C     .. Local Scalars ..
      REAL COSKAP,DET,KAPPA,ORDER,RTODEG,TRACE
      INTEGER I
C     ..
C     .. External Subroutines ..
      EXTERNAL DETMAT
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,ACOS,ATAN,MAX,MIN,NINT
C     ..
C
C
      RTODEG = 45.0/ATAN(1.0)
C
C---- First check that determinant = +1
C
      CALL DETMAT(SYMMAT,DET)
      IF (DET.GE.0.999) THEN
C
C---- Check for pure rotation (ie rotation leaves value of NAX'th
C     axis unchanged). Check NAX'th row of matrix
C
        TRACE = 0.0
        DO 10 I = 1,3
          IF (I.EQ.NAX) THEN
            IF (ABS(SYMMAT(NAX,I)-1.0).GT.0.001) GO TO 30
          ELSE IF (ABS(SYMMAT(NAX,I)).GT.0.001) THEN
            GO TO 20
          END IF
          TRACE = SYMMAT(I,I) + TRACE
   10   CONTINUE
C
C---- Success, this is a rotation about NAX
C     Get rotation angle Kappa:  Trace = 1 + 2 cos Kappa
C
        COSKAP = MIN(MAX((TRACE-1.0)*0.5,-1.0),+1.0)
        KAPPA = ACOS(COSKAP)*RTODEG
        IF (ABS(KAPPA).LT.0.01) THEN
          GO TO 40
        ELSE
          ORDER = 360.0/KAPPA
C
          NORDER = NINT(ORDER)
C
          IF (ABS(ORDER-NORDER).GT.0.001) THEN
C
C---- Not close enough to integral
C
            GO TO 40
C
          ELSE IF (NORDER.NE.2 .AND. NORDER.NE.3 .AND. NORDER.NE.4 .AND.
     +             NORDER.NE.6) THEN
C
C---- Not acceptable order
C
            GO TO 40
          ELSE
C
C---- Order .gt. 0 found
C
            RETURN
          END IF
        END IF
C
C---- Failed
C
   20   CONTINUE
        GO TO 40
C
C---- Failed
C
   30   CONTINUE
      END IF
C
C---- Failed to find acceptable order
C
   40 NORDER = 1
C
C
      END
C
C
C
      SUBROUTINE SORT1(N,A0,A1)
C     =========================
C
C
C---- N is the length of the array to be sorted i.e. the number of
C       items to be sorted
C
C---- The elements of array a0(n) are sorted into increasing order of
C     magnitude.  arrays a1 may be carried with a0 as required
C
C
C
C     .. Scalar Arguments ..
      INTEGER N
C     ..
C     .. Array Arguments ..
      REAL A0(N)
      INTEGER A1(N)
C     ..
C     .. Local Scalars ..
      REAL A,C,D
      INTEGER I,IH,IP1,J,K,LD,NO2,NP1
C     ..
C
C
      NO2 = N/2
      DO 20 I = 1,NO2
        C = A0(I)
        D = C
        IH = I
        K = I
        IP1 = I + 1
        NP1 = N + 1 - I
        DO 10 J = IP1,NP1
          A = A0(J)
          IF (A.LT.C) THEN
            IH = J
            C = A
          ELSE IF (D.LT.A) THEN
            D = A
            K = J
          END IF
   10   CONTINUE
        IF (C.EQ.D) THEN
          GO TO 30
        ELSE
          IF (IH.NE.I) THEN
            LD = A1(IH)
            A1(IH) = A1(I)
            A1(I) = LD
            C = A0(IH)
            A0(IH) = A0(I)
            A0(I) = C
          END IF
          IF (K.EQ.I) K = IH
          IF (K.NE.NP1) THEN
            LD = A1(K)
            A1(K) = A1(NP1)
            A1(NP1) = LD
            D = A0(K)
            A0(K) = A0(NP1)
            A0(NP1) = D
          END IF
        END IF
   20 CONTINUE
   30 RETURN
C
C
      END
C
C
C
      SUBROUTINE SPHBES(SBES,NBESMX,NMAX,ARGMAX)
C     ==========================================
C
C
C
C
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)             
C
C     .. Scalar Arguments ..
      REAL ARGMAX
      INTEGER NBESMX,NMAX
C     ..
C     .. Array Arguments ..
      REAL SBES(0:NBESMX,10*NBESMX)
C     ..
C     .. Arrays in Common ..
      REAL ARSYM,RSYM,RSYMT
      INTEGER JSYMP
C     ..
C     .. Local Scalars ..
      DOUBLE PRECISION ARG,DELARG,V1,V2,V3
      INTEGER JDO10,JDO20,JDO30,JDO40,MAXARG
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,COS,SIN
C     ..
C     .. Common blocks ..
      COMMON /SYMTR/JSYMP(2),RSYM(4,4,MAXSYM,2),RSYMT(4,4,MAXSYM,2),
     +       ARSYM(3,3,MAXSYM,2)
C     ..
      save
C
C
      DELARG = 0.25D00
      MAXARG = INT( ARGMAX/  ( REAL (DELARG) )  ) + 2
C
C
      DO 20 JDO20 = 0,NMAX
        DO 10 JDO10 = 1,MAXARG
          SBES(JDO20,JDO10) = 0.0
   10   CONTINUE
   20 CONTINUE
      SBES(0,1) = 1.
C
      ARG = 0.0D00
C
C
      DO 40 JDO40 = 2,MAXARG
        ARG = ARG + DELARG
        V1 = DSIN(ARG)/ARG
        V2 = (V1 - DCOS(ARG))/ARG
        SBES(0,JDO40) = REAL(V1)
        SBES(1,JDO40) = REAL(V2)
C
C
        DO 30 JDO30 = 2,NMAX
          V3 = DBLE(2*JDO30-1)*V2/ARG - V1
          SBES(JDO30,JDO40) = REAL(V3)
C
C
          IF (DABS(V3).LE.1.0D-6) THEN
            GO TO 40
          ELSE
            V1 = V2
            V2 = V3
          END IF
C
C
   30   CONTINUE
   40 CONTINUE
C
C
      END
C
C
C
      SUBROUTINE SYMORT
C     =================
C
C---- Generate symmetry operations ARSYM which
C     operate on orthogonal coordinates
C
C On entry:
C  RSYM                 symmetry operations
C  RFCRYS       orthogonal to fractional matrices
C  ROCRYS       fractional to orthogonal matrices
C
C On exit:
C  ARSYM        =  ROCRYS * RSYM * RFCRYS
C
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)             
C
C
C
C     .. Scalars in Common ..
      INTEGER NCRYST
C     ..
C     .. Arrays in Common ..
      REAL ARSYM,CELCRY,F2MAX,F2MIN,F2MINS,RFCRYS,ROCRYS,RSYM,RSYMT,
     +     SCALFO,TFAC,VOLCR
      INTEGER JSYMP,LSPGRP,NCODES,NSYMZO
C     ..
C     .. Local Scalars ..
      INTEGER I,IR,ISYM1,IXL,J,JR,K,NCR
C     ..
C     .. Local Arrays ..
      REAL AJUNK4(3,3),AJUNKF(3,3),AJUNKO(3,3),AJUNKS(3,3)
C     ..
C     .. External Subroutines ..
      EXTERNAL MATMUL
C     ..
C     .. Common blocks ..
      COMMON /CRYSTS/NCRYST,ROCRYS(4,4,2),RFCRYS(4,4,2),CELCRY(6,2),
     +       NCODES(2),LSPGRP(2),VOLCR(2),TFAC(2),NSYMZO(2),F2MIN(2),
     +       SCALFO(2),F2MAX(2),F2MINS(2),NREF1,NREF2
      COMMON /SYMTR/JSYMP(2),RSYM(4,4,MAXSYM,2),RSYMT(4,4,MAXSYM,2),
     +       ARSYM(3,3,MAXSYM,2)
C     ..
        save
C
C
        DO 40 IXL = 1,2
        DO 30 K = 1,MAXSYM
          DO 20 J = 1,3
            DO 10 I = 1,3
C
              ARSYM(I,J,K,IXL) = RSYM(I,J,K,IXL)
   10       CONTINUE
   20     CONTINUE
   30   CONTINUE
   40 CONTINUE
C
C---- rationalise symmetry - here we are multiplying real space
C     symmetry ops by orthogonalising and fractionalising matrices...
C
C     arsym(4,4,isym,ixl) = ro(ixl) s(isym,ixl) rf(ixl)
C
C     remember s(j)**(-1) will equal some s(i) -
C     they form a symmetry GROUP!
C
C---- information pointed out by Ian Tickle...
C
      DO 120 IXL = 1,2
        IF (JSYMP(IXL).NE.1) THEN
          DO 60 IR = 1,3
            DO 50 JR = 1,3
              AJUNKF(JR,IR) = RFCRYS(JR,IR,IXL)
              AJUNKO(JR,IR) = ROCRYS(JR,IR,IXL)
   50       CONTINUE
   60     CONTINUE
C
          NCR = 1
          DO 110 ISYM1 = 2,JSYMP(IXL)
            DO 80 I = 1,3
              DO 70 J = 1,3
                AJUNKS(I,J) = RSYM(I,J,ISYM1,IXL)
   70         CONTINUE
   80       CONTINUE
C
            CALL MATMUL(AJUNK4,AJUNKS,AJUNKF)
            CALL MATMUL(AJUNKS,AJUNKO,AJUNK4)
            DO 100 I = 1,3
              DO 90 J = 1,3
                ARSYM(I,J,ISYM1,IXL) = AJUNKS(I,J)
   90         CONTINUE
  100       CONTINUE
  110     CONTINUE
        END IF
  120 CONTINUE
C
C
      END
