C
C     NPO
C     Copyright (C) 1993 Sandra McLaughlin, Kim Henrick, Phil Evans
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C

C
C
C     ==============
      PROGRAM AANPO
C     ==============
C
C
C----- Contains routines:   (main  aanpo.f)
C
C akwinit.f             akwmapdat.f              akwprotin.f
C arccos.f              atmoverlapchk.f          atomin.f
C bondin.f              bondoverlapchk.f         brickcontour.f
C checkatmbondoverlap.f chrread.f                conic2plotter.f
C contourstyle.f        drawcircles.f            drawellipsoids.f
C drawgrid.f            drawlines.f              drawlineset.f
C drawsolidatoms.f      drawvisiblelines.f       dscan.f
C generateellipse.f     getatmlabelposn.f        getatomconnections.f
C getatomid.f           getbondtype.f            getbvalues.f
C getcellmatrices.f     getcontours.f            geteigenvalues.f
C gethbondconnections.f getmapsection.f          getplotxy.f
C getqmatrix.f          getunitvectors.f         joincoordpairs.f
C matrixmultply.f       matrixsolve.f
C matrxbymatrx.f        matrxbyvector.f          mint.f
C                       normvectproduct.f        openmaps.f
C ortbond.f             ortdraw.f                ortview.f
C plotbondlength.f      plotcontour.f            plotmaps.f
C plotmolecules.f       pltmapaxes.f             primer.f
C project2dto3d.f       radset.f                 setboundarylimits.f
C setdashparams.f       setplotstyle.f           setrad.f
C setupbondoverlap.f    setupeigenvals.f         storeoverlap.f
C symfrn.f              testmatrix.f             transposematrix.f
C vectbymatrxbyvector.f vectbyvecttranspose.f    vectordifference.f
C vectorproduct.f       vectorstore.f            vecttimesmatrx.f
C
C zadvanceframe.f       zdrawby.f                zdrawto.f
C zendplotting.f        zinitplotter.f           zmoveby.f
C zmoveto.f             zplotstring.f            zsetcharactersize.f
C zsetfont.f            zsetpaperorientation.f   zsetpencolour.f
C zsetpenthickness.f    zsettextangle.f          zsettextposition.f
C
C
C----- History
C      =======
C
C  5/7/90 In KWMAPDAT, extend bound 0.05A beyond map
C         to enclose atoms on edge  PRE
C
C 21/6/90 increase symmetry arrays to 96 operations  PRE
C
C 27/3/90 increase max map size 400000-->800000
C         (MAPSIZE, common/scrat/)
C
C 31/1/90 fixed bug in "" option in S/R FREDA; RS(.)
C         in S/R KWPROTIN
C
C 24/1/90 increase max map size 280000-->400000
C         (MAPSIZE, common/scrat/)
C
C 11/1/90 PRE added CENtre command, to set picture centre centre in
C         plot frame, so that you can do a series of plots with
C         the same centre
C
C 26/7/89 PRE Flag IJOIN says whether JOIN command has been given,
C        don't link up atoms in maps if not
C
C  2/6/89 PRE Correction to line in s/r atomin to allow for rotated
C             maps
C
C 28/4/88 PRE  changed S/R FREDA so that a field enclosed
C         in "" quotes is forced to be treated as a
C         literal (string)
C
C 8/4/88  SMM/HCT Plot calls changed -
C         all calls to plotting packages
C         now in set of subroutines at end,
C         therefore easy to use with
C         any new plotting packages (!)
C
C 23/6/87 PRE bug in plotmaps fixed, so that MODE DASHED works,
C         also MAP . . . NOBOX option to suppress box around map
C
C  3/10/86 PRE added MODE FIRST option to have
C          alternative contouring style
C          in the first section of a stack
C          Added LMap_ModeFirst to /com_Maps/
C
C 1/9/86 PRE Removed from FREDA line that ignores "'" characters
C
C 27/5/86 3D contouring option added PRE
C               see also additions to /com_Maps/
C                           (,LMap3D_Flag,LSetView_Flag)
C                            and to /com_CONTOURS/  (WN,W0)
C               also altered for F77 MSUBS (KWMAPDAT, openmaps)
C
C     UPDATED 5 NOV 1984
C     MODIFIED FOR VARIED PLOTTING STYLE & MAP PLOTTING
C     OCTOBER 1979 PRE
C     New map format version  September 1981
C
C Date:           Mon, 18 Apr 88 19:41:08
C From:           Sandra M. McLaughlin <SANDRA@EARN.DHHEMBL5>
C -- PLUTO   -   a plot-package independent PLUTO    18/4/88   SMM
C
C
C  Most of the features introduced lately into the PLUTO program
C  have been implemented in the version of the program at the
C  MRC LMB by Phil Evans.  This version of PLUTO calls the plot
C  package PLOT84. The version of PLUTO which calls the GHOST
C  subroutine library is from Daresbury Lab, from 1985.
C  An attempt has been made to merge these two versions so that
C  those groups using GHOST can benefit from the improvements made
C  at Cambridge.
C
C  It was found to be easier to define a set of subroutine calls
C  for standard plotting operations
C   (eg DRAWTO, MOVEBY, INITIALISE_PLOTTER etc.),
C  which could be easily implemented for any plotting package,
C  thus making the main PLUTO program independent of which system
C  was used.  So far these subroutines have been written for GHOST
C  and PLOT84.
C
C   THE SUBROUTINES  FOR PLOTTING
C
C  The subroutines called by PLUTO, which should be implemented
C  for any new plot subroutine library are listed below,
C  along with their purpose and arguments.
C
C    zadvanceframe.f
C    zdrawby.f
C    zdrawto.f
C    zendplotting.f
C    zinitplotter.f
C    zmoveby.f
C    zmoveto.f
C    zplotstring.f
C    zsetcharactersize.f
C    zsetfont.f
C    zsetpaperorientation.f
C    zsetpencolour.f
C    zsetpenthickness.f
C    zsettextangle.f
C    zsettextposition.f
C
C ---- zinitplotter.f
C
C  This subroutine should initialise the plotting system.
C  It should use the CCP4 logical name PLOT , which may
C  be set in the input file, as a name for any plot file output.
C  Also it should set the default values of the two
C  integer variables in the COMMON BLOCK COLOUR  -
C
C  where        INTEGER Max_Colours
C               PARAMETER (Max_Colours=10)
C is the maximum number of colours available
C    and  NDefault_Colour is the default line colour
C
C ---- zendplotting.f
C
C This subroutine should stop the plotting system gracefully,
C closing all files etc.
C
C
C ---- zadvanceframe.f
C
C This subroutine should advance the plot to the
C next picture or frame.
C
C
C ---- zdrawto.f
C
C Arguments: X,Y : REAL : co-ordinates of new position
C
C This subroutine draws a line from the current plotting
C position to the position (X,Y).
C
C
C ---- zmoveto.f
C
C Arguments: X,Y : REAL : co-ordinates of new position
C
C This subroutine moves the current plotting position to (X,Y).
C
C
C ---- zdrawby.f
C
C Arguments: X,Y : REAL : displacement in X and Y
C
C This subroutine draws a line from the current plotting
C position to a position (X,Y) away from it.
C
C
C ---- zmoveby.f
C
C Arguments: X,Y : REAL : displacement in X and Y
C
C This subroutine moves the current plotting position
C by X,Y relative to the old position.
C
C
C ---- zsettextposition.f
C
C Arguments: X,Y : REAL : co-ordinates of new position
C
C This subroutine sets the current position before outputing text.
C
C
C ---- zsetcharactersize.f
C
C Argument : P : REAL : character size in mm
C
C This subroutine sets the character size.
C
C
C ---- zsettextangle.f
C
C Argument : ANG : REAL : angle in degrees
C
C Subroutine to set the angle of text output relative to the X-axis.
C
C
C ---- zsetpencolour.f
C
C  Argument : ICOLR : INTEGER : pen (or colour) number
C
C Subroutine to select a pen (or colour).
C ICOLR should be less than the variable Max_Colours set in
C SUBROUTINE zinitplotter.f
C
C
C ---- zsetfont.f
C
C  Argument : NFont_Number : INTEGER : font number
C
C Subroutine to select the font in which text is output.
C In both GHOST and PLOT84 font number 0-3 are used.
C
C
C ---- zsetpenthickness.f
C
C Argument : NPen_Thickness : INTEGER : line thickness
C
C Subroutine to select the thickness of lines output.
C In both GHOST and PLOT84 versions the range of 1-9 is used.
C
C
C ---- zplotstring.f
C
C Arguments : MODE  :
C            INTEGER : =1 text string starts at current position
C                      =2 text string is centred on current position
C                      =3 text string is right justified
C
C        ID    : CHARACTER*80 : character string containing text
C        I1,I2 : INTEGER : output ID from character I1 to I2
C
C  Subroutine to output a text string.
C
C
C ---- zsetpaperorientation.f
C
C Arguments : XMAX,YMAX : REAL : Maximum X and Y on plot
C             LPaper_Setting : INTEGER
C                    : =0 X-axis is across paper roll (MRC LMB)
C                      =1 Y-axis is across paper roll (EMBLHH, DL)
C             ISTER : INTEGER  : =0 not stereo plot
C                                =1 stereo plot
C             PAPWID : REAL    : width of paper in mm,
C                                ie across roll
C             PAPLEN : REAL    : length of paper in mm,
C                                ie down roll
C
C
C This subroutine sets up the orientation of the plot on
C the paper. It is a bit of a mess. Probably all these
C parameters are not needed. Basically the rule is do
C everything here that you need to do to set
C up the plot, that isn't done in zinitplotter.f.
C The other thing to note is that PLTOPN is only called once, but
C if you have more than one picture then ORISET is called every
C time. So anything which needs to be re-done for each picture
C should go here.
C
C---- INSTALLATION CHECKS
C     ===================
C
C(a) Default Orthogonalisation -
C    variable is LOrthogonalisation_Code,
C                                =1 is Brookhaven (EMBLHH default)
C                                =2 is Rollett    (MRC LMB default)
C
C    This variable is set in the PLUTO main program
C
C (b) Paper size - variables PAPWID and PAPLEN,
C            the defaults for these are set in subroutine primer
C            For PLOT84 the defaults are 328. and 508.
C            and for gridfile output
C            from GHOST the defaults are 253. and 320.
C
C(c) Default paper setup - variable is LPaper_Setting,
C     default is 0 for MRC LMB
C     (X-axis across the paper roll)
C            and 1 for EMBLHH & DL
C     (Y-axis across the paper roll).
C     This variable is set in subroutine primer
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Charcter_Size,
     +     CurrentCharcter_Size,D001,D010,D100,Dist_to_Boundary,
     +     Drawing_Size,Ellipse_ScaleFact,Eye_Distance,
     +     ROR(4,4),RFR(4,4),
     +     Overlap_Margin,Paper_Length,Paper_Width,Perspective_Distance,
     +     Picture_Scale,Plotter_Step,Retrace_Displmnt,S,Rmax_Save,
     +     Xmax_Save,Xmin_Save,Ymax_Save,Ymin_Save,Zmax_Save,Zmin_Save,
     +     Scal_pre_Proj,SCL,Shadelight_X,Shadelight_Y,Stereo_Separation
      INTEGER IERR,Invert_Map_Flag,Iuse_persp_Labs,KSEC1,KSEC2,
     +        Label_atoms_Flag,Lcurrent_Colour,Ldraw_cell_Flag,
     +        LimitsSet_Flag,Ljoin_Flag,Lmap3d_Flag,Lmap_ModeFirst,
     +        Lorthogonalisation_Code,Lpack_Flag,Lpaper_Setting,
     +        Lperspective_Flag,Lplot_colour_Stereo,Lposition_Flag,
     +        Lprint_Flag,Lsaved_Paper_Setting,LSetView_Flag,
     +        LShade_Flag,LSolid_Flag,LStereo_Flag,LSuperimpose_Flag,
     +        LTaper_Flag,LTrace_Flag,Map_Counter,Map_Flag_Counter,
     +        Map_pts_U,Map_pts_V,Msecst,MU,MV,MW,NAT,NATO,
     +        Natoms_plusCell,NC,NC1,Ncopies,NDrawing_Number,
     +        NFirst_Section,NLast_Section,Npicture,Npts_U_compressed,
     +        Npts_V_compressed,Nsections_in_Group,Nspecified_Bonds,
     +        NstpMapGrpSect,Num_Moles
      LOGICAL Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks,
     +        CELL_GIVEN
C     ..
C     .. Arrays in Common ..
      REAL AID,AQ,Atom_Radii,Contour_Levels,Contour_Threshold,
     +     Drawing_Origin,Ellipse_Resolution,Fractional_XYZ,OrtCell,
     +     OrtConts,Orthogonal_XYZ,Plotting_XYZ,RP,Title_Rotation,
     +     ViewMatrix,XABC,XCLANG,XO
      INTEGER IAtom_Connectivity,IAtom_Type,ISpecified_Bond_Params,
     +        LAtom_Inc_Flag,LAtom_Label_Flag,Map_def_Restrictions,
     +        Map_LineTypes,Map_UVW_Limits,Map_XYZ_Limits,MOL,NSECC,
     +        Num_Contours
      CHARACTER Map_Title*80
C     ..
C     .. Local Scalars ..
      REAL ACELL,ALF,BCELL,BET,CCELL,GAM
      INTEGER JDO,NUM_FILES,LPERM
C     ..
C     .. External Subroutines ..
      EXTERNAL akwinit,atomin,CCPFYP,CCPRCS,plotmolecules,primer,
     +         XYZINIT
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC MAX,MIN
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
      COMMON /com_SAVESCAL/Xmin_Save,Ymin_Save,Zmin_Save,Xmax_Save,
     +       Ymax_Save,Zmax_Save,Rmax_Save,Lsaved_Paper_Setting
      COMMON /com_SECTIONS/NFirst_Section,NLast_Section,
     +       Nsections_in_Group,NstpMapGrpSect,KSEC1,KSEC2,Msecst,
     +       NSECC(2)
C     ..
C
C          ****************************
      CALL CCPFYP
      CALL CCPRCS(6,'N_PLUTO','$Date: 2005/11/08 14:04:16 $')
      CALL XYZINIT
      CALL primer('primer')
C          ****************************
C
      WRITE (6,FMT=6000)
 6000 FORMAT (30X,'PLUTO',/30X,'=====')
C
C

      CELL_GIVEN = .FALSE.
      LCODE = 0
C
C          ***********************************
      CALL CELLPRESENT(XABC,XCLANG,CELL_GIVEN,LCODE)
C          ***********************************
C
      IF ( .NOT. CELL_GIVEN) THEN
         WRITE(6,8888)
8888     FORMAT(' NO CELL in XYZIN or MAPIN expecting CELL KEYWORD')
      END IF
C
C
      ACELL = XABC(1)
      BCELL = XABC(2)
      CCELL = XABC(3)
      ALF = XCLANG(1)
      BET = XCLANG(2)
      GAM = XCLANG(3)
      Lorthogonalisation_Code = LCODE
C
      UseOrtep = .FALSE.
C
      NUM_FILES = 0
C
   10 CONTINUE
C
C---- Read the input cards, and return on PLOT card
C
C          *****************************
   20 CALL akwinit(CELL_GIVEN,NUM_FILES)
C          *****************************
C
      IF (IERR.GT.0) GO TO 20
      IF (NAT.GT.Max_Atoms) WRITE (6,FMT=6002) Max_Atoms
 6002 FORMAT (' ***Too many atoms in basic  list. max = ',I5)
C
      WRITE (6,FMT=6004)
 6004 FORMAT (' ***Begin Plotting***')
C
C----     loop pictures
C
      KSEC1 = NFirst_Section
      Npicture = MAX(Npicture,1)
C
C
      DO 30 JDO = 1,Npicture
        KSEC2 = KSEC1 + Nsections_in_Group - 1
        KSEC2 = MIN(KSEC2,Map_UVW_Limits(6))
C
C            ******
        CALL atomin
C            ******
C
C----     This routine does the actual plotting
C
C            *************
        CALL plotmolecules
C            *************
C
        KSEC1 = KSEC1 + NstpMapGrpSect
   30 CONTINUE
C
C----     Go back for more input
C
      GO TO 10
      END
C
C
C
C     ========================================
      SUBROUTINE akwinit(CELL_GIVEN,NUM_FILES)
C     ========================================
C
C
C
C   This subroutine reads in the input cards, and returns when it
C   reaches the PLOT command.
C   If it reaches end of input file instead
C   it STOPs - ie in practise the end of the program is here.
C
C    Summary Of Cards
C    ================
C
C  Cards  are  indicated  in  the  table  as  compulsory   (C),
C  forbidden  (F) or optional (O) for plotting of molecules and
C  maps.
C
C            Molecule   Map           Comments
C
C Group 1, initial settings (read by akwinit)
C
C        DATA    O       O       If present must be 1st
C        TITLE   O       O       Must be 1st after DATA
C        CELL    C       C
C          Not quite but this is the only way to
C          change orthogonalisation..
C        SYMM    C       C
C        PRINT   O       O       May appear in group 4
C
C Group 2, map control (MAP read by akwinit Folowing by akwmapdat)
C
C        MAP     C               Must be 1st in group
C
C            CONTRS  C               Must be 2nd in group
C            LIMITS  O
C            MODE    O
C            SECTNS  C
C            SLAB    O               Must be after SECTNS
C            GRID    O
C
C Group 3, atom selection
C    (INPUT read by akwinit Folowing by akwprotin)
C
C         INPUT   C               Must be 1st in group
C
C            FILE    O
C            BASE    O
C            LABEL   O
C            BOND    O
C            RESIDU  O
C            BOUNDS  O       F
C            BOX     O       F
C            SPHERE  O       F
C            STYLE   O
C            LINK    O
C            SKEW  ) O
C            TRANS ) O
C
C  Group 4, plotting style etc (read by akwinit)
C
C            OMIT    O
C              F ???? serves no function as used in akwprotin ???
C            ELSTYLE
C            ELLIPSOID
C            JOIN    C       O
C            VIEW  ) C       (F)
C            MATRIX)
C            SOLID or STICK  O       O
C            RADII   O       O       Must not precede JOIN
C            DCELL   O       O
C            STEREO or MONO  O       O
C            FONT    O       O
C            THICK   O       O
C            SIZE    O       O
C            COLOUR  O       O
C
C
C  Group 5, plot
C        PLOT    C       C       Must be last
C
C
C
C---- NEllipsoidStyle packed as
C
C   ==>      NPLANE + NDOT*10 + NLINE*100 +
C             NDASH*1000 + AIN0*10000 + AIN1*1000000
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_BndLabels
      PARAMETER (Max_BndLabels=10)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Conics
      PARAMETER (Max_Conics=20)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER MAX_LABELS
      PARAMETER (Max_Labels=100)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER Max_Quadrangles
      PARAMETER (Max_Quadrangles=30)
      INTEGER Max_Radii
      PARAMETER (Max_Radii=10)
      INTEGER Max_Segments
      PARAMETER (Max_Segments=50)
      INTEGER Max_Symmetry
      PARAMETER (Max_Symmetry=192)
      INTEGER Max_Tokens
      PARAMETER (Max_Tokens=200)
      INTEGER NpackAtmConn
      PARAMETER (NpackAtmConn=5000000)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Cent_Xmax,Cent_Xmin,
     +     Cent_Ymax,Cent_Ymin,Cent_Zmax,Cent_Zmin,Charcter_Size,
     +     CurrentCharcter_Size,D001,D010,D100,Def_Bnd_Radius,
     +     Def_Char_Size,Def_Paper_Wdth,Def_Plt_Radius,Def_Plt_Size,
     +     Dist_to_Boundary,Drawing_Size,Ellipse_ScaleFact,Eye_Distance,
     +     ROR(4,4),RFR(4,4),
     +     Overlap_Margin,Paper_Length,Paper_Width,
     +     Perspective_Distance,Picture_Scale,Plotter_Step,
     +     Retrace_Displmnt,S,Scal_pre_Proj,SCL,Shadelight_X,
     +     Shadelight_Y,Stereo_Separation
      INTEGER IERR,Invert_Map_Flag,Iuse_persp_Labs,KCHOSE,
     +        Label_atoms_Flag,Lcurrent_Colour,Ldef_Bnd_Lines,
     +        Ldraw_cell_Flag,LimitsSet_Flag,Ljoin_Flag,Lmap3d_Flag,
     +        Lmap_ModeFirst,Lorthogonalisation_Code,Lpack_Flag,
     +        Lpaper_Setting,Lperspective_Flag,Lplot_colour_Stereo,
     +        Lposition_Flag,Lprint_Flag,LSetView_Flag,LShade_Flag,
     +        LSolid_Flag,LStereo_Flag,LSuperimpose_Flag,LTaper_Flag,
     +        LTrace_Flag,Map_Counter,Map_Flag_Counter,Map_pts_U,
     +        Map_pts_V,MU,MV,MW,NAT,NATO,Natoms_plusCell,NBondLabels,
     +        NC,NC1,Ncopies,NDrawing_Number,Nellipise,NFont_Number,
     +        NPen_Thickness,Npicture,Npts_U_compressed,
     +        Npts_V_compressed,NQuads_Overlaped,Nspecified_Bonds,
     +        NSymmetry_Ops,NumHbondRadii,NumLinkRadii,NumPlotRadii,
     +        Num_Bnd_Quadrangles,Num_Conics,Num_Moles,Num_Overlaps
      LOGICAL LCent_Flag,Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
      CHARACTER Iden_Title*80
C     ..
C     .. Arrays in Common ..
      REAL AID,AQ,Atm_Hbond_Radii,Atm_Link_Radii,Atm_Plot_Radii,
     +     Atom_Radii,Contour_Levels,Contour_Threshold,Drawing_Origin,
     +     Ellipse_Conics,Ellipse_Resolution,Fractional_XYZ,OrtCell,
     +     OrtConts,Orthogonal_XYZ,Overlap_Conics,Parr_Offset,
     +     Perp_Offset,Plotting_XYZ,Quadrangles,Quads_Overlaped,
     +     Rotsym_Matrx,RP,Segments,Title_Rotation,TransSym_Vect,
     +     ViewMatrix,XABC,XCLANG,XO
      INTEGER IATID,IAtom_Connectivity,IAtom_Type,
     +        ISpecified_Bond_Params,KC,KEllipseTypes,KQ,LAtom_Inc_Flag,
     +        LAtom_Label_Flag,LHbond_Flag,LRadi_Atm_typ,
     +        LRadi_Hbond_typ,LRadi_Plt_typ,Map_def_Restrictions,
     +        Map_LineTypes,Map_UVW_Limits,Map_XYZ_Limits,MOL,
     +        NEllipsoidStyle,Num_Contours
      CHARACTER Atm_Lab_Type*2,Atm_Identifier*10,Atm_Label*10,
     +          Atm_Lab_Txt*10,BondtoLabel*10,Map_Title*80
C     ..
C     .. Local Scalars ..
      REAL ACELL,AIN0,AIN1,ALF,ANG,BCELL,BET,CA,CCELL,GAM,R1MOD,R2MOD,
     +     RAD,RB,SA,TEMP,V3MOD,XX,YY,ZZ
      INTEGER I,IA,IDASH,IDO,ILINE,INSTR,INTER,IUNIQ,IWORK,J,J1,J2,J3,
     +        JA,JAT,JC,JDASH,JDO,JDOT,JHBOND,JMODE,JREPET,JRES,K,KAT,
     +        KMODE,KSTART,KWSTART,KWWANT,L,LJOINSAVE,LPRINTSAVE,M,N1,
     +        N2,N3,NC2,NED,NEH,NEL,NELLY,NEP,NOMITWRK,NTOKX,NUM_FILES,
     +        JERR
      LOGICAL CELL_GIVEN,INPUT_GIVEN,JOIN_GIVEN,LJoinRadRes,LENDX,
     +        LOGPARSER,MAP_GIVEN,OMIT_GIVEN,RADII_GIVEN,SYMM_GIVEN,
     +        VIEW_GIVEN
      CHARACTER XYZSET*1,CWORK*4,KEYX*4,FWORK*10,SYMM_LINE*80,LINEX*600
C     ..
C     .. Local Arrays ..
      REAL A(3,3),B(3,3),FVALUEX(Max_Tokens),PLINE(3),R1(3),R2(3),V1(3),
     +     V2(3),V3(3),CELL1(6),CELL2(6)
      INTEGER IBEGX(Max_Tokens),ICCELL(12),IDECX(Max_Tokens),
     +        IENDX(Max_Tokens),ITYPX(Max_Tokens),JCELL(8)
      CHARACTER CVALUEX(Max_Tokens)*4, OABC(8)*1
C     ..
C     .. External Functions ..
      INTEGER LENSTR,LPERM
      EXTERNAL LENSTR
C     ..
C     .. External Subroutines ..
      EXTERNAL akwmapdat,akwprotin,CCPERR,CCPUPC,dscan,
     +         getatomconnections,getatomid,getcellmatrices,
     +         gethbondconnections,KEYNUM,normvectproduct,ortview,
     +         PARSER,primer,setrad,setupeigenvals,symfrn,testmatrix,
     +         zendplotting
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,COS,MOD,NINT,REAL,SIN,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_BNDLAB/NBondLabels,Perp_Offset(Max_BndLabels),
     +       Parr_Offset(Max_BndLabels)
C      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
C     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
C     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CENTRE/LCent_Flag,Cent_Xmax,Cent_Ymax,Cent_Zmax,
     +       Cent_Xmin,Cent_Ymin,Cent_Zmin
      COMMON /com_CHOOSE/IATID(8),KCHOSE
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_DEFAULTS/Ldef_Bnd_Lines,Def_Bnd_Radius,Def_Plt_Radius,
     +       Def_Char_Size,Def_Plt_Size,Def_Paper_Wdth
      COMMON /com_ELLIPSESTYLE/NEllipsoidStyle(Max_Atoms),Nellipise,
     +       KEllipseTypes(6)
      COMMON /com_FONT/NFont_Number,NPen_Thickness
      COMMON /com_LABBND/BondtoLabel(Max_BndLabels*2)
      COMMON /com_LABCHR/Atm_Lab_Txt(Max_Labels),
     +       Atm_Lab_Type(Max_Labels),Atm_Label(Max_Labels),
     +       Atm_Identifier(Max_Atoms),Iden_Title
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
      COMMON /com_OVERLAP/Num_Conics,Num_Overlaps,NQuads_Overlaped,
     +       Num_Bnd_Quadrangles,KC(Max_Conics),KQ(Max_Quadrangles),
     +       Ellipse_Conics(7,Max_Atoms),Overlap_Conics(6,Max_Conics),
     +       Quads_Overlaped(3,4,Max_Quadrangles),
     +       Quadrangles(9,2*Max_Atoms),Segments(Max_Segments,2)
      COMMON /com_RADII/NumLinkRadii,NumPlotRadii,NumHbondRadii,
     +       Atm_Link_Radii(Max_Radii),Atm_Plot_Radii(Max_Radii),
     +       Atm_Hbond_Radii(Max_Radii),LRadi_Atm_typ(Max_Radii),
     +       LRadi_Plt_typ(Max_Radii),LRadi_Hbond_typ(Max_Radii),
     +       LHbond_Flag(2)
      COMMON /com_SYMMETRY/NSymmetry_Ops,Rotsym_Matrx(3,3,Max_Symmetry),
     +       TransSym_Vect(3,Max_Symmetry)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Data statements ..
C
      DATA OABC /'O','A',' ','B','C',' ',' ',' '/
C
C
C
      DATA JCELL/0,100,110,10,1,101,111,11/
      DATA ICCELL/2002,4003,6004,8001,2005,10006,12007,14008,16005,8008,
     +     14003,4006/
C     ..
C
C
      NBondLabels = 0
C
C
      DO 10 JDO = 1,Max_BndLabels
        Perp_Offset(JDO) = 0.0
        Parr_Offset(JDO) = 0.0
   10 CONTINUE
C
C
      LOGPARSER = .FALSE.
C
C
      MAP_GIVEN = .FALSE.
C
C                         *********************
C  Done in call to CELLPRESENT.
C      IF (CELL_GIVEN)     CALL getcellmatrices
C                         *********************
C
      SYMM_GIVEN = .FALSE.
      INPUT_GIVEN = .FALSE.
      JOIN_GIVEN = .FALSE.
      VIEW_GIVEN = .FALSE.
      OMIT_GIVEN = .FALSE.
      RADII_GIVEN = .FALSE.
      LJoinRadRes = .TRUE.
C
C
      NAT = 0
C
C
      ACELL = XABC(1)
      BCELL = XABC(2)
      CCELL = XABC(3)
      ALF = XCLANG(1)
      BET = XCLANG(2)
      GAM = XCLANG(3)
      IF(Lorthogonalisation_Code.EQ.0) Lorthogonalisation_Code = 1
C
C          ****************
      CALL primer('ATMINI')
C          ****************
C
      RAD = 57.2957795
      IERR = 0
      NC1 = 0
C
C---- set cell corner coordinates,
C     also include/omit code for KWPROTIN
C
      DO 20 JDO = 1,8
        IATID(JDO) = 0
        LAtom_Label_Flag(JDO) = 0
        LAtom_Inc_Flag(JDO) = 0
        IAtom_Type(JDO) = 0
        Atom_Radii(JDO) = 0.0
        Fractional_XYZ(JDO,1) = REAL(JCELL(JDO)/100)
        Fractional_XYZ(JDO,2) = MOD(JCELL(JDO)/10,10)
        Fractional_XYZ(JDO,3) = MOD(JCELL(JDO),10)
        Atm_Identifier(JDO) = OABC(JDO)
   20 CONTINUE
C
C---- set up cell connectivity here.
C
      DO 30 JDO = 1,12
        IAtom_Connectivity(JDO) = ICCELL(JDO)
   30 CONTINUE
C
C---- first time through make sure all
C     parameters have been initialised
C
C                   *********************
      IF (NAT.LE.0) CALL primer('FIRSTG')
C                   *********************
C
C----  all data cards read here
C
   40 CONTINUE
C
C
      LINEX = '    '
      NTOKX = Max_Tokens
   50 CONTINUE
C
C          **********************************************************
      CALL PARSER(KEYX,LINEX,IBEGX,IENDX,ITYPX,FVALUEX,CVALUEX,IDECX,
     +            NTOKX,LENDX,LOGPARSER)
C          **********************************************************
C
      WRITE(6,55)LINEX(1:100)
55    FORMAT(' Data Line---- ',A100)
      IF (LENDX .OR. KEYX.EQ.'END') GO TO 740
C
C
C     ========
C---- UseOrtep
C     ========
C
      IF (KEYX.EQ.'USEO') THEN
        UseOrtep = .TRUE.
        GO TO 40
C
C
C     ========
C---- MAP CARD
C     ========
C
C---- MAP card, subsidiary cards
C        contrs,mode,sectns read in s/r KWMAPDAT
C
      ELSE IF (KEYX(1:3).EQ.'MAP') THEN
C
C            ********************************************************
        CALL akwmapdat(LINEX,IBEGX,IENDX,ITYPX,FVALUEX,CVALUEX,IDECX,
     +                 NTOKX,LOGPARSER)
C            ********************************************************
C
        MAP_GIVEN = .TRUE.
        GO TO 50
C
C     =============================
C---- DATA data identification card
C     =============================
C
C  DATA  This card clears all arrays and flags
C        (  but  not  the  view direction).
C        It is only required to separate two independent
C        drawings (or superimposed drawings)
C
      ELSE IF (KEYX.EQ.'DATA') THEN
        IF (LSuperimpose_Flag.EQ.1) LSuperimpose_Flag =
     +      LSuperimpose_Flag + 1
C
C
        IF (NTOKX.GT.1) THEN
          Iden_Title = LINEX(IBEGX(2) :IENDX(NTOKX))
          NAT = 8
          Ljoin_Flag = 0
          NC = 12
          NSymmetry_Ops = 0
        END IF
C
C            ****************
        CALL primer('RESETS')
C            ****************
C
        GO TO 40
C
C      =====================================================
C----  CELL a   b   c   alpha   beta   gamma Lorthogonalisation_Code
C      =====================================================
C
C
C
C---- cell constants and orthogonalisation switch
C      if only one number use as orthogonalisation code
C
C     Cell dimensions in Angstrom and degrees.  Angles default  to
C     90.   The  cell  dimensions  are  used for the conversion of
C     orthogonal coordinates to fractional, and  vice  versa.   If
C     cell  dimensions of 1 1 1 are given , then the conversion to
C     fractional coordinates will have no effect, which will work,
C     provided  that  no  map is being plotted, and no translation
C     (from  symmetry  or  BOUNDS)  is  required.
C     Lorthogonalisation_Code=1   for
C     Brookhaven   orthogonalisation   convention,   =2  (default)
C     Rollett convention (see above).  (*) The cell dimensions may
C     be  omitted  if  a  map file is read:  cell dimensions given
C     here override those on the map file.
C
      ELSE IF (KEYX.EQ.'CELL') THEN
C
C
        IF (NTOKX.EQ.1) THEN
          Ldraw_cell_Flag = 2
          GO TO 40
        ELSE IF (NTOKX.GE.2) THEN
          CWORK = CVALUEX(2)
C
C              *************
          CALL CCPUPC(CWORK)
C              *************
C
          IF (CWORK(1:2).EQ.'NO') THEN
            Ldraw_cell_Flag = 0
            GO TO 40
          END IF
        END IF
C
C---- default orthogonalisation is
C      brookhaven, Lorthogonalisation_Code=1
C
        IWORK = 0
        IF (NTOKX.EQ.8) IWORK = 8
        IF (NTOKX.EQ.2) IWORK = 2
        IF(Lorthogonalisation_Code.EQ.0) Lorthogonalisation_Code = 1
CCC        Lorthogonalisation_Code = 2
C
C---- Correction 5/3/87 PRE
C
        IF (IWORK.NE.0) THEN
          KWWANT = 1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C

          Lorthogonalisation_Code = NINT(FVALUEX(IWORK))
        END IF
C
C    
        IF (Lorthogonalisation_Code.LT.1 .OR.
     +      Lorthogonalisation_Code.GT.3)
C
C            **********************************************************
     +  CALL CCPERR(1,' CELL Key_WORD  Lorthogonalisation_Code Wrong ')
C            **********************************************************
C
        IF (NTOKX.LT.4) GO TO 40
        IF (NTOKX.GE.7) GO TO 70
        J = NTOKX + 1
C
C
        DO 60 JDO = J,7
          FVALUEX(JDO) = 90.0
   60   CONTINUE
C
C
   70   ACELL = FVALUEX(2)
        BCELL = FVALUEX(3)
        CCELL = FVALUEX(4)
        ALF = FVALUEX(5)
        BET = FVALUEX(6)
        GAM = FVALUEX(7)
C  Check for cell inconsistency
         IF(XABC(1) .GT.0.1) THEN
           CELL1(1)=ACELL
           CELL1(2)=BCELL
           CELL1(3)=CCELL
           CELL1(4)=ALF
           CELL1(5)=BET
           CELL1(6)=GAM
           CELL2(1)=XABC(1)
           CELL2(2)=XABC(2)
           CELL2(3)=XABC(3)
           CELL2(4)=XCLANG(1)
           CELL2(5)=XCLANG(2)
           CELL2(6)=XCLANG(3)
            ERRFRC = 0.02
            JERR = 0
            CALL CELLCHK(CELL1,CELL2,ERRFRC,JERR)
            IF (JERR.EQ.1)
     +        CALL CCPERR(2,'* Possible error from CELLCHK')
         END IF
C--- replace cell
        XABC(1) = ACELL
        XABC(2) = BCELL
        XABC(3) = CCELL
        XCLANG(1) = ALF
        XCLANG(2) = BET
        XCLANG(3) = GAM
        CELL_GIVEN = .TRUE.
C
C---- calculate orthogonalisation matrices
C
C            ***************
        CALL getcellmatrices
C            ***************
C
        GO TO 40
C
C     ==================================
C---- ELLIPSOID TYPE n  (Atom_Type ) ...
C     =================================
C
C     These must be group 4 keywords, since atom types are
C     not defined until after INPUT.

      ELSE IF (KEYX.EQ.'ELLI') THEN
        IF (Map_Flag_Counter.GT.0) THEN
          WRITE (6,FMT=6000)
 6000     FORMAT (' ELLIPSOID not allowed with MAPs')
          GO TO 40
        END IF
C
C
        IWORK = 2
        IF (IWORK.GT.NTOKX)
C
C                **********************************************
     +      CALL CCPERR(1,' No tokens given on ELLIPSOID line')
C                **********************************************
C
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
        IF (CWORK.NE.'TYPE')
C
C                ******************************************************
     +      CALL CCPERR(1,' wrong subkeyword given for ELLIPSOID line')
C                ******************************************************
C
        IWORK = IWORK + 1
        KWWANT = 1
        KWSTART = IWORK
C
C            ****************************************************
        CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C            ****************************************************
C
        Nellipise = NINT(FVALUEX(IWORK))
        IF (Nellipise.GT.700) THEN
C
C---- use a standard value
C
          Nellipise = Nellipise - 700 + 1
        END IF
C
C
        IF (Nellipise.LT.1 .OR. Nellipise.GT.6) 
C
C                *********************************************
     +      CALL CCPERR(1,' Ellipsoid type not allowed value')
C                *********************************************
C
        IWORK = IWORK + 1
        IF (IWORK.GT.NTOKX) 
C
C              ******************************************************
     +    CALL CCPERR(1,' not enough tokens given on ELLIPSOID  line')
C              ******************************************************
C
        DO 90 JDO = IWORK,NTOKX
          FWORK = LINEX(IBEGX(JDO) :IENDX(JDO))
C
C              ********************
          CALL getatomid(KAT,FWORK)
C              ********************
C
          IF (KAT.EQ.0) 
C
C              ********************************************************
     +    CALL CCPERR(1,'on ELLIPSOID  (Atom_Type n) AtomName unknown')
C              ********************************************************
C
          IF (KAT.LT.0) THEN
C
C---- Atom_Type Given, e.g. CA, C, etc.
C
            DO 80 IDO = 1,NAT
              IF (IAtom_Type(IDO).EQ.KAT) NEllipsoidStyle(IDO)
     +            = KEllipseTypes(Nellipise)
   80       CONTINUE
C
C---- Specifically named atom
C
          ELSE
            NEllipsoidStyle(KAT) = KEllipseTypes(Nellipise)
          END IF
C
C
   90   CONTINUE
C
C
        GO TO 40
C
C
C     ================
C---- ESCALE n
C     ================
C
C see page ??in ortep manual default = 1.54
C
      ELSE IF (KEYX.EQ.'ESCA') THEN
        KWWANT = 1
        KWSTART = 2
C
C            ****************************************************
        CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C            ****************************************************
C
        Ellipse_ScaleFact = FVALUEX(2)
        GO TO 40

C
C
C     ===============================================================
C---- ELSTYLE TYPE 6 NPLANES n NDOTS n NLINES n NDASH n ZERO n RATE n
C     ===============================================================
C
C
      ELSE IF (KEYX.EQ.'ELST') THEN
        IF (NTOKX.LT.3) 
C
C            *******************************************************
     +  CALL CCPERR(1,' NOT enough values given for ELSTYLE KeyWord')
C            *******************************************************
C
        NELLY = NINT(FVALUEX(3))
        IWORK = 3
        IF (NELLY.LT.0 .OR. NELLY.GT.6) 
C
C               ******************************************
     +     CALL CCPERR(1,' Only 6 ellipse styles allowed')
C               ******************************************
C
  100   IWORK = IWORK + 1
        IF (IWORK.GT.NTOKX) GO TO 110
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
        IF (CWORK.EQ.'NPLA') THEN
          IWORK = IWORK + 1
          NEP = NINT(FVALUEX(IWORK))
          IF (NEP.GT.4 .OR. NEP.LT.0) NEP = 4
        ELSE IF (CWORK.EQ.'NDOT') THEN
          IWORK = IWORK + 1
          NED = NINT(FVALUEX(IWORK))
          IF (NED.LT.-6 .OR. NED.GT.6) NED = 0
        ELSE IF (CWORK.EQ.'NLIN') THEN
          IWORK = IWORK + 1
          NEL = NINT(FVALUEX(IWORK))
          IF (NEL.LT.0 .OR. NEL.GT.9) NEL = 8
        ELSE IF (CWORK.EQ.'NDAS') THEN
          IWORK = IWORK + 1
          NEH = NINT(FVALUEX(IWORK))
          IF (NEH.LT.0 .OR. NEH.GT.9) NEH = 0
        ELSE IF (CWORK.EQ.'ZERO') THEN
          IWORK = IWORK + 1
          AIN0 = FVALUEX(IWORK)
          IF (AIN0.LT.0.0 .OR. AIN0.GT.0.9) AIN0 = 0.40
        ELSE IF (CWORK.EQ.'RATE') THEN
          IWORK = IWORK + 1
          AIN1 = FVALUEX(IWORK)
          IF (AIN1.LT.0.0 .OR. AIN1.GT.0.1) AIN1 = 0.08
        ELSE
          WRITE (6,FMT=6002) CWORK
 6002     FORMAT (' SubKeyWord for ELSTYLE not understood')
          IWORK = IWORK + 1
        END IF
        GO TO 100
C
C
  110   CONTINUE
C
C---- NEllipsoidStyle packed as
C
C   ==>      NPLANE + NDOT*10 + NLINE*100 +
C             NDASH*1000 + AIN0*10000 + AIN1*1000000
C
C   if NSOLID/NDOT negative whole number negated
C
        KEllipseTypes(NELLY) =  ABS(NED)*10 + NEP + NEL*100 + 
     +                          NEH*1000  + NINT(AIN0*10000) + 
     +                          NINT(AIN1*1000000.0)
        IF (NED.LT.0) KEllipseTypes(NELLY) = -KEllipseTypes(NELLY)
C
C      ==========================
C----  SYMM    symmetry operators
C      ==========================
C
C     Symmetry operations may be given in two ways:  either in the
C     style  of  International  Tables,  with  different operators
C     separated by * or on different cards
C
C     eg SYMM X,Y,Z * -X,1/2+Y,-Z
C
C     Translations may be given as 1/2 or 0.5 etc.  Alternatively,
C     the  operation  may  be  given as 12 numbers, a matrix S and
C     translation vector t, in the order S11, S12, S13,  t1,  S21,
C     S22,   etc.   Note  that  the  symmetry  operations  act  on
C     fractional coordinates x' = Sx + t .   (370)  At  least  the
C     identity must normally be given (*) Symmetry operations will
C     be read from the first map file, if any, but  symmetry  data
C     given  here on SYMM cards overrides that in the file.  If no
C     symmetry is given in either place, the identity is used.
C
C
      ELSE IF (KEYX.EQ.'SYMM') THEN
        NSymmetry_Ops = NSymmetry_Ops + 1
C
C
        IF (NSymmetry_Ops.GT.Max_Symmetry) 
C
C              ****************************************
     +    CALL CCPERR(1,'Too Many symmetry Operations')
C              ****************************************
C
        IF (NTOKX.NE.13) GO TO 150
C
C   ITYP(I)      =0  null field
C                =1  character string
C                =2  number
C
        DO 120 JDO = 2,NTOKX
          IF (ITYPX(JDO).EQ.1) GO TO 150
  120   CONTINUE
C
C---- this is a numerically coded symmetry operator.
C
        IWORK = 1
C
C
        DO 140 IDO = 1,3
          DO 130 JDO = 1,3
            IWORK = IWORK + 1
            KWWANT = 1
            KWSTART = IWORK
C
C                ****************************************************
            CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                ****************************************************
C
            Rotsym_Matrx(IDO,JDO,NSymmetry_Ops) = FVALUEX(IWORK)
  130     CONTINUE
C
C
          IWORK = IWORK + 1
          KWWANT = 1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          TransSym_Vect(IDO,NSymmetry_Ops) = FVALUEX(IWORK)
  140   CONTINUE
C
C
        GO TO 40
C
C---- this is an alphanumeric symmetry card, like internat. tabs.
C
  150   CONTINUE
        SYMM_LINE = LINEX(IBEGX(2) :IENDX(NTOKX))
        IWORK = 1
C
C            **************************************************
        CALL symfrn(SYMM_LINE,IWORK,NSymmetry_Ops,Rotsym_Matrx,
     +              TransSym_Vect)
C            **************************************************
C
        GO TO 40
C
C     =================================
C---- INPUT   optional keywords, LSuperimpose_Flag
C     =================================
C
C
C     This card introduces atom input.  Optional keywords are:
C
C     LIST list coordinates (not relevant if plotting into map)
C     CAS select alpha-carbon (CA) atoms only
C     SAVED for plotting into map, the scratch  file  has  already
C               been created in a previous run of the program, and
C               coordinates will not be read from stream 1.
C
C         If a number .gt.1 is given on the  card
C         (LSuperimpose_Flag),  this
C     drawing  is superimposed on the previous one (ie it is drawn
C     at the  same  scale  and  position,  without  advancing  the
C     paper).   This  provides  a  way of evading the limit of 900
C     (2000) atoms in a drawing.
C
C
      ELSE IF (KEYX.EQ.'INPU') THEN
C
C---- read protein data file,
C When you get the INPUT card, the protein data file is read in
C in KWPROTIN, and also some more input cards
C
C The next call is for ORTEP routines and MUST
C have a CELL from somewhere - fix this (ccx)
C
C                             **************
        IF (CELL_GIVEN ) THEN
          IF (.NOT. MAP_GIVEN) CALL setupeigenvals
        END IF
C                             **************
C
C            ********************************************************
        CALL akwprotin(LINEX,IBEGX,IENDX,ITYPX,FVALUEX,CVALUEX,IDECX,
     +                 NTOKX,LOGPARSER,CELL_GIVEN,NUM_FILES)
C            ********************************************************
C
C---- restore cell connections
C
        DO 160 JDO = 1,12
          IAtom_Connectivity(JDO) = ICCELL(JDO)
  160   CONTINUE
C
C
        GO TO 50
C
C       =============
C----   JOIN keywords
C       =============
C
C    This card controls the way in which atoms are linked, either
C    by explicit connection, or usually by search for atoms lying
C    within the sum of their bonding radii  (keyword  RADII).   A
C    list  of  bonding  radii  is then given.  The keyword RESIDU
C    causes the distance search to be done only within  residues,
C    and this is usually preferable and faster than a search over
C    all atoms.  Residues are then linked by atom name (see  LINK
C    card).
C
C    eg   JOIN RADII RESIDU CA 0.9 C 0.9 O 0.9 N 0.9 S 1.1
C
C    Note that CA is a different type from C.
C
C   Hydrogen  bonds  may  be  generated  with  the
C    keyword HBOND :
C
C    eg   JOIN RADII HBOND 3 4 O 1.6 N 1.6
C
C    Search for connections between atoms of type O  and  type  N
C    with  radii  of  1.6  A.  The first two numbers are the bond
C    types (qv BOND)  for  mainchain-mainchain  bonds  and  bonds
C    involving  sidechains respectively.  The bond types may be 0
C    , in which case the bond style will be taken from the  RADII
C    BONDS  card  or  from  the  default,  or  they may be -1, to
C    suppress  one  type   of   bond   (ie   1   -1   will   find
C    mainchain-mainchain  bonds only).  Mainchain-mainchain bonds
C    are excluded within the same or  adjacent  residues  in  the
C    same  molecule,  and  sidechain-sidechain  bonds in the same
C    residue.
C
C    The  H-bond  search  must  be  done  before  other  bond
C    searches (JOIN RADII ....  ).
C
C        Explicit connectivities may be given as a list  of  atom
C    names,  with  breaks  indicated  by  end of card or by *.  A
C    number on a card of explicit connections is used as  a  bond
C    type, referring back to a BOND card.
C
C    eg   JOIN 3 O243 N222 * N243 O251
C
C
C                  If maps are  plotted,  the  atoms  are  always
C    joined  by residue, and a JOIN card is only needed to change
C    the radii from the default 0.9 A for  all  atoms.   Explicit
C    connections are not allowed.
C
C
      ELSE IF (KEYX.EQ.'JOIN') THEN
        Ljoin_Flag = 1
        IWORK = 1
        IWORK = IWORK + 1
        IF (IWORK.GT.NTOKX) GO TO 40
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
C      ===========
C---- JOIN EXCLUDE
C      ===========
C
        IF (CWORK.EQ.'EXCL') THEN
C
C----  exclude some bonds
C
C    Connections may also be explicitly removed from a previously
C    generated   list,  (either  connections  between  particular
C    atoms, or less usefully all bonds between two  atom  types).
C    Only one bond or bond type may be excluded on each card.
C
C    eg   JOIN  EXCLUD CA124 CA125
C
C         JOIN  EXCLUD O N
C
C
          IWORK = IWORK + 2
C
C
          IF (IWORK.GT.NTOKX) 
C
C             *******************************************************
     +   CALL CCPERR(1,'JOIN EXCLUDE n m Key_Word, Not Enough tokens')
C             *******************************************************
C
          FWORK = LINEX(IBEGX(IWORK-1) :IENDX(IWORK-1))
C
C              ********************
          CALL getatomid(KAT,FWORK)
C              ********************
C
          FWORK = LINEX(IBEGX(IWORK) :IENDX(IWORK))
C
C              ********************
          CALL getatomid(JAT,FWORK)
C              ********************
C
          NC2 = 13
C
C
          DO 190 JDO = 13,NC
            K = MOD(IAtom_Connectivity(JDO),NpackAtmConn)
            IA = K/Max_Atoms
            JA = K - Max_Atoms*IA
            I = JA
            IF (IAtom_Type(IA).EQ.KAT .OR. IA.EQ.KAT) GO TO 170
            I = IA
            IF (IAtom_Type(JA).NE.KAT .AND. JA.NE.KAT) GO TO 180
C
C---- atom KAT found , test other atom against jat
C
  170       IF (IAtom_Type(I).EQ.JAT .OR. I.EQ.JAT) GO TO 190
  180       IAtom_Connectivity(NC2) = IAtom_Connectivity(JDO)
            NC2 = NC2 + 1
  190     CONTINUE
C
C
          NC = NC2 - 1
C
C
          DO 200 JDO = NC2,Max_Bonds
            IAtom_Connectivity(JDO) = 0
  200     CONTINUE
C
C---- End of JOIN Exclude
C
          GO TO 40
        END IF
C
C
        IF (Map_Flag_Counter.LE.0) THEN
C
C
          DO 210 JDO = 9,NAT
            Atom_Radii(JDO) = 0.0
  210     CONTINUE
        END IF
C
C    eg   JOIN RADII RESIDU CA 0.9 C 0.9 O 0.9 N 0.9 S 1.1
C    eg   JOIN RADII HBOND 3 4 O 1.6 N 1.6
C    eg   JOIN BOND 3 O243 N222 * N243 O251
C    eg   JOIN NONE
C    eg   JOIN BOND 3 O243 TO N222 N243 N251
C
C---- JOIN NONE
C
        IF (CWORK.EQ.'NONE') THEN
          NC = 12
          GO TO 40
        END IF
C
C---- JOIN RADII
C
        IF (CWORK.EQ.'RADI') THEN
          IUNIQ = 0
          INTER = 0
          JRES = 0
          JHBOND = 0
  230     IWORK = IWORK + 1
          IF (IWORK.GT.NTOKX) GO TO 340
          CWORK = CVALUEX(IWORK)
C
C              *************
          CALL CCPUPC(CWORK)
C              *************
C
C---- JOIN RADII UNIQUE
C
          IF (CWORK.EQ.'UNIQ') THEN
            IUNIQ = 1
            GO TO 230
C
C---- JOIN RADII PRINT
C
          ELSE IF (CWORK.EQ.'PRIN') THEN
            Lprint_Flag = 1
            LOGPARSER = .TRUE.
            GO TO 230
C
C---- JOIN RADII INTER
C
          ELSE IF (CWORK.EQ.'INTE') THEN
            INTER = 1
            GO TO 230
C
C---- JOIN RADII RESIDUE
C
          ELSE IF (CWORK.EQ.'RESI' .AND. LJoinRadRes) THEN
            LJoinRadRes = .FALSE.
            JRES = 1
            IWORK = IWORK + 1
C
C---- JOIN RADII RESIDUE (Atom_Type n) ...
C
            KWSTART = IWORK - 1
            DO 250 JDO = IWORK,NTOKX,2
              FWORK = LINEX(IBEGX(JDO) :IENDX(JDO))
C
C                  ********************
              CALL getatomid(KAT,FWORK)
C                  ********************
C
              IF (KAT.EQ.0) 
     + CALL CCPERR(1,'JOIN RADII RESIDUE  AtomName unknown')
C
              KWWANT = 1
C Original version barfs under irix64 MIPSpro Compilers: Version 7.3.1.1m with -O2
C              KWSTART = JDO + 1
              KWSTART = KWSTART + 2
C
C                  ****************************************************
              CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                  ****************************************************
C
              IF (KAT.LT.0) THEN
C
C---- Atom_Type Given JOIN
C
                IF (Map_Flag_Counter.GT.0) THEN
                  NumLinkRadii = NumLinkRadii + 1
                  Atm_Link_Radii(NumLinkRadii) = FVALUEX(JDO+1)
                  LRadi_Atm_typ(NumLinkRadii) = KAT
                ELSE
                  DO 240 IDO = 1,NAT
                    IF (IAtom_Type(IDO).EQ.KAT) Atom_Radii(IDO)
     +                  = FVALUEX(JDO+1)
  240             CONTINUE
                END IF
C
C---- Specifically named atom
C
              ELSE
                IF (Map_Flag_Counter.GT.0) 
C
C             ***************************************************
     +   CALL CCPERR(1,' Cant have JOIN RADII AtomName with MAP')
C             ***************************************************
C
                Atom_Radii(KAT) = FVALUEX(JDO+1)
              END IF
C
C
  250       CONTINUE
C
C
            GO TO 340
C
C---- JOIN RADII HBOND
C
          ELSE IF (CWORK.EQ.'HBON') THEN
            JHBOND = 1
            IF (IWORK+2.GE.NTOKX) THEN
              IF (ITYPX(IWORK+1).EQ.2 .AND. ITYPX(IWORK+2).EQ.2) THEN
                LHbond_Flag(1) = NINT(FVALUEX(IWORK+1))
                LHbond_Flag(2) = NINT(FVALUEX(IWORK+2))
                IWORK = IWORK + 3
              ELSE
                CALL CCPERR(1, ' ERROR IN JOIN RADII HBOND')
              ENDIF
            ENDIF
C
C
            DO 270 K = IWORK,NTOKX,2
              FWORK = LINEX(IBEGX(K) :IENDX(K))
C
C                  ********************
              CALL getatomid(KAT,FWORK)
C                  ********************
C
              IF (KAT.EQ.0) 
C
C           ***********************************************
     + CALL CCPERR(1,'JOIN RADII HBOND , AtomName unknown')
C           ***********************************************
C
              KWWANT = 1
              KWSTART = K + 1
C
C                  ****************************************************
              CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                  ****************************************************
C
              IF (KAT.LT.0) THEN
C
C---- Atom_Type Given
C
                IF (Map_Flag_Counter.GT.0) THEN
                  NumHbondRadii = NumHbondRadii + 1
                  Atm_Hbond_Radii(NumHbondRadii) = FVALUEX(K+1)
                  LRadi_Hbond_typ(NumHbondRadii) = KAT
                ELSE
                  DO 260 JDO = 1,NAT
                    IF (IAtom_Type(JDO).EQ.KAT) Atom_Radii(JDO)
     +                  = FVALUEX(K+1)
  260             CONTINUE
                END IF
C
C---- Specifically named atom
C
              ELSE
                IF (Map_Flag_Counter.GT.0) 
C
C            *********************************************************
     +  CALL CCPERR(1,' Cant have JOIN RADII HBOND AtomName with MAP')
C            *********************************************************
C
                Atom_Radii(KAT) = FVALUEX(K+1)
              END IF
C
C
  270       CONTINUE
C
C
            GO TO 340
C
C---- JOIN RADII ALL n
C
          ELSE IF (CWORK(1:3).EQ.'ALL') THEN
            IWORK = IWORK + 1
            KWWANT = 1
            KWSTART = IWORK
C
C                ****************************************************
            CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                ****************************************************
C
            Atm_Link_Radii(1) = FVALUEX(IWORK)
C
C
            IF (Map_Flag_Counter.LE.0) THEN
              DO 280 JDO = 1,NAT
                Atom_Radii(JDO) = FVALUEX(IWORK)
  280         CONTINUE
            END IF
C
C
            GO TO 340
C
C---- End of JOIN radii
C
          ELSE
            IF (LJoinRadRes) THEN
C
C                  *************************************************
              CALL CCPERR(1,' Error on JOIN RADII SubKyWrd Unknown')
C                  *************************************************
C
            ELSE
C
C                  ************************************************
              CALL CCPERR(1,' Only one JOIN RADII RESID permitted')
C                  ************************************************
C
            ENDIF
          END IF
        END IF
C
C---- JOIN BOND
C
        IF (CWORK.EQ.'BOND') THEN
          IWORK = IWORK + 1
          IF (IWORK.GT.NTOKX) 
C
C            ********************************************************
     +  CALL CCPERR(1,' ERROR not enough tokens on JOIN BOND KeyWord')
C            ********************************************************
C
          JC = 0
C
C---- JOIN BOND bondtype
C
          KWWANT = 1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          JC = NINT(FVALUEX(IWORK))
C
C---- test for iwork+2 equal to 'TO'
C
          IF (IWORK+2.LE.NTOKX) THEN
            CWORK = CVALUEX(IWORK+2)
C
C                *************
            CALL CCPUPC(CWORK)
C                *************
C
            IF (CWORK(1:2).EQ.'TO') GO TO 320
          END IF
C
C
          IWORK = IWORK + 1
          KAT = 0
C
C
          DO 310 K = IWORK,NTOKX
            CWORK = CVALUEX(K)
            IF (CWORK(1:1).EQ.'*') GO TO 290
            FWORK = LINEX(IBEGX(K) :IENDX(K))
C
C                ********************
            CALL getatomid(JAT,FWORK)
C                ********************
C
            IF (JAT.LE.0) THEN
              WRITE (6,FMT=6004) FWORK
 6004         FORMAT (
     +  ' ERROR on JOIN BOND KeyWord',/,
     +' atomname < ',A,' > is Unknown')
C
C                  *******************************
              CALL CCPERR(1,'ERROR on JOIN BOND')
C                  *******************************
C
            END IF
C
C
            IF (KAT.EQ.0) GO TO 300
            NC = NC + 1
            IAtom_Connectivity(NC) = Max_Atoms*KAT + JAT +
     +                               JC*NpackAtmConn
            GO TO 300
  290       JAT = 0
  300       KAT = JAT
  310     CONTINUE
          GO TO 40
C
C---- JOIN BOND n Atom1  TO list of atoms
C
  320     CONTINUE
          IWORK = IWORK + 1
          FWORK = LINEX(IBEGX(IWORK) :IENDX(IWORK))
C
C              ********************
          CALL getatomid(KAT,FWORK)
C              ********************
C
          IWORK = IWORK + 2
C
C
          IF (IWORK.GT.NTOKX) 
C
C            **************************************************
     +  CALL CCPERR(1,'JOIN BOND n Atom1 TO NOT enough tokens')
C            **************************************************
C
          DO 330 K = IWORK,NTOKX
            FWORK = LINEX(IBEGX(K) :IENDX(K))
C
C                ********************
            CALL getatomid(JAT,FWORK)
C                ********************
C
            IF (KAT.LE.0 .OR. JAT.LE.0) 
C
C             *********************************************
     +   CALL CCPERR(1,' ERROR on JOIN BOND n atom TO ...')
C             *********************************************
C
            NC = NC + 1
            IAtom_Connectivity(NC) = Max_Atoms*KAT + JAT +
     +                               JC*NpackAtmConn
  330     CONTINUE
          GO TO 40
C
C---- end of JOIN
C
        ELSE
C
C              ***************************
          CALL CCPERR(1,'ERROR on JOIN ')
C              ***************************
C
        END IF
C
C
  340   CONTINUE
C
C
        IF (JHBOND.NE.0) THEN
C
C              ********************************
          CALL gethbondconnections(LHbond_Flag)
C              *******************************
C
          JHBOND = 0
          GO TO 40
        END IF
C
C
        IF (JRES.EQ.0  .AND. Map_Flag_Counter.LE.0) THEN
          KMODE = 2
          KSTART = 1
C
C---- dscan kmode 2 - used by the join instruction
C     django --- distances and angles
C
C              ***************************************
          CALL dscan(KMODE,IUNIQ,INTER,FVALUEX,KSTART)
C              ***************************************
C
        ELSE
C
C---- join residue by residue
C called for e.g.
C  JOIN RADII RESIDU CA 0.9 C 0.9 O 0.9 N 0.9 S 1.0 P 1.2
C
C              *******************************
          CALL getatomconnections(Lprint_Flag)
C              *******************************
C
          JRES = 0
        END IF
C
        GO TO 40
C
C     ===========
C---- TITLE title
C     ===========
C
C
C     If present, this card must be first (after DATA).  The title
C     is plotted at the head of the drawing.
C
C      clear flags controlling plotting
C
      ELSE IF (KEYX.EQ.'TITL') THEN
        Iden_Title = LINEX(7 : 87)
        Log_Draw_Title = .TRUE.
C
C            ****************
        CALL primer('RESETS')
C            ****************
C
        GO TO 40
C
C
C---- ORTEPVIEW, ORIGIN,OTRANSLATE,OROTATION
C
      ELSE IF (KEYX.EQ.'ORIG') THEN
        INSTR = 501
C
C            ******************************************************
        CALL ortview(INSTR,LINEX,IBEGX,IENDX,ITYPX,FVALUEX,CVALUEX,
     +               IDECX,NTOKX,LOGPARSER)
C            ******************************************************
C
        GO TO 40
      ELSE IF (KEYX.EQ.'OTRA') THEN
        INSTR = 504
C
C            ******************************************************
        CALL ortview(INSTR,LINEX,IBEGX,IENDX,ITYPX,FVALUEX,CVALUEX,
     +               IDECX,NTOKX,LOGPARSER)
C            ******************************************************
C
        GO TO 40
      ELSE IF (KEYX.EQ.'OROT') THEN
        INSTR = 502
C
C            ******************************************************
        CALL ortview(INSTR,LINEX,IBEGX,IENDX,ITYPX,FVALUEX,CVALUEX,
     +               IDECX,NTOKX,LOGPARSER)
C            ******************************************************
C
        GO TO 40
C
C     ====================
C---- VIEW view definition
C     ====================
C
C    This card defines the viewpoint for the  drawing.   For  map
C    plotting,  the  view is calculated from the map orientation,
C    so a VIEW card should not  normally  be  given,  unless  the
C    atoms are not in the same fractional coordinate frame as the
C    map (eg for skewed maps).  However it may be present  before
C    the MAP card to define the view direction (particularly with
C    the 3D option on map stacks).  The information on this  card
C    is  used  to  construct  a  matrix  to rotate the orthogonal
C    molecule coordinates Orthogonal_XYZ into plotter coordinates
C
C            Xp , Xp = R    Orthogonal_XYZ .
C
C    Several options are available.
C
C    (a) View relative to axes.
C
C    X                                            ( Yo along Xp
C    Y                View along orthogonal axes  ( Zo along Xp
C    Z                towards origin              ( Xo along Xp
C
C    AFACE            View from (1,1/2,1/2)
C    BFACE                      (1/2,1,1/2)
C    CFACE                      (1/2,1/2,1) towards origin
C
C    LINE x y z       view from fractional point xyz towards
C    origin
C
C    (b) View relative to named atoms
C
C    LINE n1 n2 view direction from atom n1 to n2.  The vector v1
C                    = (n1-n2) x (0,1,0) is along Xp.
C    BISECT n1 n2 n3 view along bisector of  n1n2n3  towards  n2,
C                    perpendicular along Yp
C    PERP n1 n2 n3 view perpendicular to plane n1n2n3 ,  bisector
C                    along Xp
C
C    (c) Rotation
C                  After the matrix has been defined  by  one  of
C    the above options, it may be rotated about the plotter axes.
C    A positive rotation is clockwise when looking along the axis
C    towards the origin.  Several rotations may be concatenated.
C
C    eg   VIEW PERP O139 N36 CB72 X -30
C
C    rotate  by  -30  degrees  about  Xp   from   the   specified
C    orientation
C
C    The keyword MATRIX causes the current matrix to be modified.
C    VIEW   MATRIX   alone  preserves  the  current  matrix  (for
C    superimposing drawings).
C
C         VIEW MATRIX Z 90
C
C
      ELSE IF (KEYX.EQ.'VIEW') THEN
C    If there is a map the VIEW has to correspond to that
C    Input line ignored
C
       IF ( MAP_GIVEN ) THEN
         WRITE(6,'(2A,//,A)') 
     + '   ****  NO VIEW can be set when MAP plotting;',
     + ' it is defined by the map axis order ***',
     + '   ****  Input line ignored ****'
         GO TO 40
       END IF 

        XYZSET = ' '
        IWORK = 1
        IWORK = IWORK + 1
C
C
        IF (IWORK.GT.NTOKX) 
C
C            **************************************************
     +  CALL CCPERR(1,' ERROR on VIEW KeyWord NO Tokens given')
C            **************************************************
C
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
C---- VIEW MATRIX
C
        IF (CWORK.EQ.'MATR') THEN
          IWORK = IWORK + 1
          IF (IWORK+1.GT.NTOKX) THEN
            LSetView_Flag = 1
C
C                **********************
            CALL testmatrix(ViewMatrix)
C                **********************
C
            GO TO 40
          END IF
C
C---- VIEW MATRIX [X,Y,Z] n
C
          XYZSET = CVALUEX(IWORK) (1:1)
C
C              **************
          CALL CCPUPC(XYZSET)
C              **************
C
          IWORK = IWORK + 1
          KWWANT = 1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          ANG = FVALUEX(IWORK)/RAD
          GO TO 460
        END IF
C
C---- NO MATRIX
C
C      keywords defining a rotation matrix to be calculated.
C
        DO 360 I = 1,3
          PLINE(I) = 0.5
          DO 350 J = 1,3
            ViewMatrix(I,J) = 0.0
  350     CONTINUE
  360   CONTINUE
C
C---- VIEW [X,Y,Z]
C
        IF (CWORK(1:1).EQ.'X' .OR. CWORK(1:1).EQ.'Y' .OR.
     +      CWORK(1:1).EQ.'Z') THEN
          XYZSET = CWORK(1:1)
C
C
          IF (XYZSET.EQ.'X') THEN
            ViewMatrix(1,2) = 1.0
            ViewMatrix(2,3) = 1.0
            ViewMatrix(3,1) = 1.0
          ELSE IF (XYZSET.EQ.'Y') THEN
            ViewMatrix(1,3) = 1.0
            ViewMatrix(2,1) = 1.0
            ViewMatrix(3,2) = 1.0
          ELSE IF (XYZSET.EQ.'Z') THEN
            ViewMatrix(1,1) = 1.0
            ViewMatrix(2,2) = 1.0
            ViewMatrix(3,3) = 1.0
          END IF
C
C
          IWORK = IWORK + 1
          IF (IWORK.GT.NTOKX) THEN
            LSetView_Flag = 1
C
C                **********************
            CALL testmatrix(ViewMatrix)
C                **********************
C
            GO TO 40
          END IF
C
C---- VIEW [X,Y,Z] n
C
          KWWANT = 1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          ANG = FVALUEX(IWORK)/RAD
          GO TO 460
        END IF
C
C---- VIEW [AFACE, BFACE, CFACE]
C
        IF (CWORK.EQ.'AFAC' .OR. CWORK.EQ.'BFAC' .OR.
     +      CWORK.EQ.'CFAC') THEN
C
C
          IF (CWORK.EQ.'AFAC') THEN
            ILINE = 1
            PLINE(1) = 1.0
          ELSE IF (CWORK.EQ.'BFAC') THEN
            ILINE = 1
            PLINE(2) = 1.0
          ELSE IF (CWORK.EQ.'CFAC') THEN
            ILINE = 1
            PLINE(3) = 1.0
          END IF
C
C
          GO TO 390
        END IF
C
C---- VIEW LINE x y z
C
        IF (CWORK.EQ.'LINE') THEN
          ILINE = 1
          IWORK = IWORK + 1
          IF (IWORK.GT.NTOKX) 
C
C            *******************************************************
     +  CALL CCPERR(1,' ERROR on VIEW LINE not enough tokens given')
C            *******************************************************
C
C   ITYP(I)      =0  null field
C                =1  character string
C                =2  number
C
          IF (ITYPX(IWORK).EQ.1) THEN
C
C---- line is defined by two atom names
C
            FWORK = LINEX(IBEGX(IWORK) :IENDX(IWORK))
C
C                ********************
            CALL getatomid(KAT,FWORK)
C                ********************
C
            FWORK = LINEX(IBEGX(IWORK+1) :IENDX(IWORK+1))
C
C                ********************
            CALL getatomid(JAT,FWORK)
C                ********************
C
            IF (KAT.LE.0 .OR. JAT.LE.0) 
C
C            **********************************************
     +  CALL CCPERR(1,' View error atom id not recognised')
C            **********************************************
C
C
            DO 370 JDO = 1,3
              PLINE(JDO) = Fractional_XYZ(KAT,JDO) -
     +                     Fractional_XYZ(JAT,JDO)
  370       CONTINUE
C
C
          ELSE
C
C---- line defined by given point
C      (fvaluex(2),fvaluex(3),fvaluex(4))
C       from point to origin.
C
            DO 380 JDO = 1,3
              KWWANT = 1
              KWSTART = IWORK + JDO - 1
C
C                  ****************************************************
              CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                  ****************************************************
C
              PLINE(JDO) = FVALUEX(KWSTART)
  380       CONTINUE
C
C
          END IF
C
C
          GO TO 390
        END IF
C
C---- VIEW PERP n1 n2 n3
C
        IF (CWORK.EQ.'PERP') THEN
          ILINE = 2
          IWORK = IWORK + 1
          IF (IWORK+2.GT.NTOKX) 
C
C             *****************************************************
     +   CALL CCPERR(1,' VIEW PERP n1 n2 n3 ... NOT enough tokens')
C             *****************************************************
C
          FWORK = LINEX(IBEGX(IWORK) :IENDX(IWORK))
C
C              *******************
          CALL getatomid(J1,FWORK)
C              *******************
C
          FWORK = LINEX(IBEGX(IWORK+1) :IENDX(IWORK+1))
C
C              *******************
          CALL getatomid(J2,FWORK)
C              *******************
C
          FWORK = LINEX(IBEGX(IWORK+2) :IENDX(IWORK+2))
C
C              *******************
          CALL getatomid(J3,FWORK)
C              *******************
C
          IF (J1.LE.0 .OR. J2.LE.0 .OR. J3.LE.0) 
C
C            **********************************************
     +  CALL CCPERR(1,' View error atom id not recognised')
C            **********************************************
C
          GO TO 410
        END IF
C
C---- VIEW BISECT n1 n2 n3
C
        IF (CWORK.EQ.'BISE') THEN
          ILINE = 3
          IWORK = IWORK + 1
          IF (IWORK+2.GT.NTOKX) 
C
C               ******************************************************
     +     CALL CCPERR(1,'VIEW BISECT n1 n2 n3 ... NOT enough tokens')
C               ******************************************************
C
          FWORK = LINEX(IBEGX(IWORK) :IENDX(IWORK))
C
C              *******************
          CALL getatomid(J1,FWORK)
C              *******************
C
          FWORK = LINEX(IBEGX(IWORK+1) :IENDX(IWORK+1))
C
C              *******************
          CALL getatomid(J2,FWORK)
C              *******************
C
          FWORK = LINEX(IBEGX(IWORK+2) :IENDX(IWORK+2))
C
C              *******************
          CALL getatomid(J3,FWORK)
C              *******************
C
          IF (J1.LE.0 .OR. J2.LE.0 .OR. J3.LE.0) 
C
C              **********************************************
     +    CALL CCPERR(1,' View error atom id not recognised')
C              **********************************************
C
          GO TO 410
        END IF
C
C---- VIEW unknown sub key word
C
C            **********************************************
        CALL CCPERR(1,' ERROR UNKNOWN SubKeyWord for VIEW')
C            **********************************************
C
C
  390   XX = PLINE(1)
        YY = PLINE(2)
        ZZ = PLINE(3)
c        V3(1) = OR1*XX + OR2*YY + OR3*ZZ
c        V3(2) = OR4*XX + OR5*YY + OR6*ZZ
c        V3(3) = OR7*XX + OR8*YY + OR9*ZZ
        V3(1) = ROR(1,1)*XX + ROR(1,2)*YY + ROR(1,3)*ZZ
        V3(2) = ROR(2,1)*XX + ROR(2,2)*YY + ROR(2,3)*ZZ
        V3(3) = ROR(3,1)*XX + ROR(3,2)*YY + ROR(3,3)*ZZ
C
C---- calc r matrix for line view
C
        V3MOD = SQRT(V3(1)**2+V3(2)**2+V3(3)**2)
C
C
        DO 400 JDO = 1,3
          V3(JDO) = V3(JDO)/V3MOD
  400   CONTINUE
C
C
        R2(1) = 0.0
        R2(2) = 1.0
        R2(3) = 0.0
C
C            *************************
        CALL normvectproduct(R2,V3,V1)
        CALL normvectproduct(V3,V1,V2)
C            *************************
C
        GO TO 450
C
C---- calc r matrix for perp and bisect views
C
  410   CONTINUE
C
C
        DO 420 JDO = 1,3
          V3(JDO) = (Orthogonal_XYZ(J1,JDO)+Orthogonal_XYZ(J3,JDO))*
     +              0.5 - Orthogonal_XYZ(J2,JDO)
          R1(JDO) = Orthogonal_XYZ(J1,JDO) - Orthogonal_XYZ(J2,JDO)
          R2(JDO) = Orthogonal_XYZ(J3,JDO) - Orthogonal_XYZ(J2,JDO)
  420   CONTINUE
C
C
        V3MOD = SQRT(V3(1)**2+V3(2)**2+V3(3)**2)
        R1MOD = SQRT(R1(1)**2+R1(2)**2+R1(3)**2)
        R2MOD = SQRT(R2(1)**2+R2(2)**2+R2(3)**2)
C
C
        DO 430 JDO = 1,3
          V3(JDO) = V3(JDO)/V3MOD
          R1(JDO) = R1(JDO)/R1MOD
          R2(JDO) = R2(JDO)/R2MOD
  430   CONTINUE
C
C---- now get v2=v3xr2  and then v1=v2xv3
C
C            *************************
        CALL normvectproduct(V3,R2,V2)
        CALL normvectproduct(V2,V3,V1)
C            *************************
C
        IF (ILINE.EQ.3) GO TO 450
C
C---- for perp  just swap axes
C
        DO 440 JDO = 1,3
          TEMP = V3(JDO)
          V3(JDO) = V2(JDO)
          V2(JDO) = V1(JDO)
          V1(JDO) = TEMP
  440   CONTINUE
C
C---- set r matrix
C
  450   ViewMatrix(1,1) = V1(1)
        ViewMatrix(1,2) = V1(2)
        ViewMatrix(1,3) = V1(3)
        ViewMatrix(2,1) = V2(1)
        ViewMatrix(2,2) = V2(2)
        ViewMatrix(2,3) = V2(3)
        ViewMatrix(3,1) = V3(1)
        ViewMatrix(3,2) = V3(2)
        ViewMatrix(3,3) = V3(3)
C
C---- kewords giving rotations to be applied
C     to the current r matrix
C
        LSetView_Flag = 1
C
C            **********************
        CALL testmatrix(ViewMatrix)
C            **********************
C
        GO TO 40
C
C
  460   CONTINUE
C
C
        DO 480 I = 1,3
          DO 470 J = 1,3
            A(I,J) = 0.0
  470     CONTINUE
  480   CONTINUE
C
C
        CA = COS(ANG)
        SA = SIN(ANG)
C
C
        IF (XYZSET.EQ.'X') THEN
C
C---- x rotn
C
          A(1,1) = 1.0
          A(2,2) = CA
          A(2,3) = SA
          A(3,2) = -SA
          A(3,3) = CA
        ELSE IF (XYZSET.EQ.'Y') THEN
C
C---- y rotn
C
          A(1,1) = CA
          A(1,3) = -SA
          A(2,2) = 1.0
          A(3,1) = SA
          A(3,3) = CA
        ELSE IF (XYZSET.EQ.'Z') THEN
C
C---- z rotn
C
          A(1,1) = CA
          A(1,2) = SA
          A(2,1) = -SA
          A(2,2) = CA
          A(3,3) = 1.0
        ELSE
C
C              *******************************************************
          CALL CCPERR(1,'on VIEW line XYZSET test under wrong condit')
C              *******************************************************
C
        END IF
C
C
        DO 500 I = 1,3
          DO 490 J = 1,3
            B(I,J) = A(I,1)*ViewMatrix(1,J) + A(I,2)*ViewMatrix(2,J) +
     +               A(I,3)*ViewMatrix(3,J)
  490     CONTINUE
  500   CONTINUE
C
C
        DO 520 I = 1,3
          DO 510 J = 1,3
            ViewMatrix(I,J) = B(I,J)
  510     CONTINUE
  520   CONTINUE
C
C
        LSetView_Flag = 1
C
C            **********************
        CALL testmatrix(ViewMatrix)
C            **********************
C
        GO TO 40
C
C      =====================================
C----  SOLID     NOHID SHADE a1 a2 Plotter_Step s
C      =====================================
C
C     Solid drawing ,  ie  ball  and  spoke.   The  keyword  NOHID
C     suppresses  the hidden line removal, making the program much
C     faster.  This should normally be  done  when  plotting  into
C     maps.   The  keyword  SHADE causes shading of the atom as if
C     illuminated by a light source whose position is given by the
C     amgles  a1 & a2.  Zero angles give the light coming directly
C     towards the viewer  along  the  Zp  axis.   a1  and  a2  are
C     rotations  about  Yp  and Zp (suitable values 120,-45).  The
C     SHADE option is useful for space-filling drawings (see RADII
C     ATOMS).   The  Plotter_Step  keyword
C     (see  also  the DEVICE card)
C     allows the plotter step size for circle drawing to be  reset
C     from  its  default  of  0.25  mm to finer step eg 0.1 .  The
C     smaller step gives smoother  circles  but  uses  more  time,
C     particularly if the hidden lines option is used.
C
C
      ELSE IF (KEYX.EQ.'SOLI') THEN
        LSolid_Flag = 1
C
C
        DO 530 JDO = 9,NAT
          IF(Atom_Radii(JDO) .LT. 0.0001)Atom_Radii(JDO) = 0.3
  530   CONTINUE
C
C
C keywords  shade  a1 a2
C nohid,   no hidden line removal
C  Plotter_Step p    reset Plotter_Step
C
        IF (NTOKX.LE.1) THEN
          WRITE (6,FMT=6006)
 6006     FORMAT (' Default SOLID sub parameters will be used')
          GO TO 40
        END IF
C
C
        IWORK = 1
  540   IWORK = IWORK + 1
        IF (IWORK.GT.NTOKX) GO TO 40
        CWORK = CVALUEX(IWORK)

C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
        IF (CWORK.EQ.'NOHI') THEN
          LSolid_Flag = 2
          GO TO 540
        ELSE IF (CWORK.EQ.'SHAD') THEN
C
C---- shade
C
          LShade_Flag = 1
          IWORK = IWORK + 1
          IF (IWORK.GT.NTOKX) GO TO 40
          KWWANT = 2
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Shadelight_X = FVALUEX(IWORK)
          Shadelight_Y = FVALUEX(IWORK+1)
          IWORK = IWORK + 1
          GO TO 540
C
C
        ELSE IF (CWORK.EQ.'PLOT') THEN
C
C---- Plotter_Step, reset value
C
          IWORK = IWORK + 1
          KWWANT = 1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Plotter_Step = FVALUEX(IWORK)
        ELSE
          WRITE (6,FMT=6008) CWORK
 6008     FORMAT (' ERROR for KeyWord SOLID, Sub_Key_Word < ',A,' >',
     +           /'    Not Understood')
          GO TO 540
        END IF
C
C
C      ===========================================================
C----  STICK     Stick model drawing (one line / bond, no circles)
C      ===========================================================
C
C              (default)
C
      ELSE IF (KEYX.EQ.'STIC') THEN
        LSolid_Flag = 0
        GO TO 40
C
C     ==========
C---- RADII card
C     ==========
C
C
C RADII CARD.
C
C this sets radii for atoms and bonds. there are several
C options for bond specification, atom type, intermolecular,
C to list, specific atom pair.  we also enter number of lines
C for bond. tapering of bonds may be controlled also.
C
C   RADII ATOMS n1 r1 n2 r2 .....
C
C    This card sets the plotting radii for  atom  names  or  atom
C    types  n1,  n2 etc to r1, r2 Angstrom .  The default setting
C    for all atoms is 0.3 A.  Radii may be  set  to  aproximately
C    Van  der Waals radii to give a space-filling drawing.  Radii
C    may not be set for individual atoms when map  plotting.   If
C    the  radius  is  negative,  the  atoms  are  drawn  with two
C    concentric circles,  the  inner  one  with  half  the  given
C    radius.   This  is  particularly  useful  for  atoms such as
C    waters which are not joined to other atoms.
C
C    eg   RADII ATOMS CA 0.15 C 0.15 N 0.2 O 0.25 S 0.3 WA -0.2
C
C         RADII ATOMS ALL 0.5
C
C    RADII BONDS keywords, r, n
C
C     This card may be used to set the bond radii, in a  different
C     way to the BOND card.  The bond style is given by a radius r
C     in A and a number of lines n.  Alternatively, the style  may
C     be  given  as DASHED and three numbers (see MODE card).  The
C     default values are 0.04 and 8.
C
C     The keyword ALL resets the default values.
C
C     eg   RADII BONDS ALL 0.05 5
C
C     The keyword TO sets the  bond  style  for  all  bonds  to  a
C     particular atom or atom type.
C
C     eg   RADII BONDS TO FE 0 1
C
C     The bond style may also be given for a particular bond  (not
C     when map plotting)
C
C     eg   RADII BOND P2402 07402 0.08 6
C
C     The keyword TAPER applies  exaggerated  perspective  to  all
C     bonds.   A  factor  m  is  given and is used to multiply the
C     ratio of the radii of the end points of the bonds.
C
C     eg   RADII BONDS TAPER 25
C
C
      ELSE IF (KEYX.EQ.'RADI') THEN
        IDASH = 0
        IWORK = 1
        IWORK = IWORK + 1
        IF (IWORK.GT.NTOKX) GO TO 40
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
C---- RADII BOND
C
        IF (CWORK.EQ.'BOND') THEN
C
C----   reset Def_Bnd_Radius Ldef_Bnd_Lines after dashed line
C
          IF (IDASH.GT.0) Def_Bnd_Radius = 0.04
          IF (IDASH.GT.0) Ldef_Bnd_Lines = 5
          IDASH = 0
  550     IWORK = IWORK + 1
          IF (IWORK.GT.NTOKX) GO TO 40
          CWORK = CVALUEX(IWORK)
C
C              **************
          CALL CCPUPC(CWORK)
C              **************
C
C---- RADII BONDS DASHED l m n
C
          IF (CWORK.EQ.'DASH') THEN
            IDASH = 1
            IWORK = IWORK + 1
            IF (IWORK+2.GT.NTOKX) 
C
C            *******************************************************
     +  CALL CCPERR(1,'RADII BONDS DASHED l n m  NOT enough tokens')
C            *******************************************************
C
            KWWANT = 3
            KWSTART = IWORK
C
C                ****************************************************
            CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                ****************************************************
C
            JREPET = NINT(FVALUEX(IWORK)*10.0)
            JDASH = NINT(FVALUEX(IWORK+1)*10.0)
            JDOT = NINT(FVALUEX(IWORK+2)*10.0)
            JMODE = 3
            IF (JDOT.GT.0) JMODE = 4
            IF (JREPET.EQ.0) JREPET = 2
            IF (JDASH.EQ.0) JDASH = 1
C
C
            IDASH = 50000*JDOT + 500*JDASH + 5*JREPET + JMODE
            IWORK = IWORK + 2
            GO TO 550
C
C---- RADII BONDS ALL n m
C
          ELSE IF (CWORK(1:3).EQ.'ALL') THEN
C
C---- change default settings for radius and lines
C
            IWORK = IWORK + 1
            KWWANT = 2
            KWSTART = IWORK
C
C                ****************************************************
            CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                ****************************************************
C
            Def_Bnd_Radius = FVALUEX(IWORK)
            Ldef_Bnd_Lines = NINT(FVALUEX(IWORK+1))
            IWORK = IWORK + 1
            IF (IDASH.EQ.0) GO TO 550
            Def_Bnd_Radius = -1.0
            Ldef_Bnd_Lines = IDASH
            GO TO 550
C
C---- RADII BONDS TAPER n
C
          ELSE IF (CWORK.EQ.'TAPE') THEN
            IWORK = IWORK + 1
            KWWANT = 1
            KWSTART = IWORK
C
C                ****************************************************
            CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                ****************************************************
C
            LTaper_Flag = NINT(FVALUEX(IWORK))
            GO TO 550
C
C---- RADII BONDS TO atom_ID m n
C
          ELSE IF (CWORK(1:2).EQ.'TO') THEN
C
C---- keyword to  followed by atom type
C
            IWORK = IWORK + 1
C
C
            IF (IWORK.GT.NTOKX) 
C
C            *********************************************************
     +  CALL CCPERR(1,'RADII BONDS TO atom_ID m n, NOT enough tokens')
C            *********************************************************
C
            FWORK = LINEX(IBEGX(IWORK) :IENDX(IWORK))
C
C                ******************
            CALL getatomid(M,FWORK)
C                ******************
C
            N1 = 0
            N2 = M
            IWORK = IWORK + 1
C
C---- store bond radius and number of lines.
C
            IF (IDASH.EQ.0) THEN
              IF (IWORK.LT.NTOKX) THEN
                KWWANT = 1
                KWSTART = IWORK
C
C                    **********************************************
                CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,
     +                      NTOKX)
C                    **********************************************
C
                RB = FVALUEX(IWORK)
                IWORK = IWORK + 1
C
C
                IF (IWORK.GT.NTOKX) THEN
                  L = Ldef_Bnd_Lines
                ELSE
                  KWWANT = 1
                  KWSTART = IWORK
C
C                      **********************************************
                  CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,
     +                        NTOKX)
C                      **********************************************
C
                  L = NINT(FVALUEX(IWORK))
                END IF
              END IF
C
C
            END IF
C
C
            N3 = NINT((RB+L)*1000.0)
            IF (IDASH.NE.0) N3 = -IDASH
            Nspecified_Bonds = Nspecified_Bonds + 1
            ISpecified_Bond_Params(Nspecified_Bonds,1) = N1
            ISpecified_Bond_Params(Nspecified_Bonds,2) = N2
            ISpecified_Bond_Params(Nspecified_Bonds,3) = N3
            RB = Def_Bnd_Radius
            L = Ldef_Bnd_Lines
            GO TO 40
C
C---- RADII BONDS INTER
C
          ELSE IF (CWORK.EQ.'INTE') THEN
C
C---- keyword inter      for all intermolecular bonds...
C
            N1 = -1
            N2 = 0
            GO TO 550
C
C---- RADII BONDS atom_1 atom_2 m n
C
          ELSE
C            IWORK = IWORK + 1
C
C
            IF (IWORK+3.GT.NTOKX) 
C
C            *********************************************
     +  CALL CCPERR(1,'not enough tokens on RADII BONDS ')
C            *********************************************
C
            FWORK = LINEX(IBEGX(IWORK) :IENDX(IWORK))
C
C                ********************
            CALL getatomid(KAT,FWORK)
C                ********************
C
            IWORK = IWORK + 1
            FWORK = LINEX(IBEGX(IWORK) :IENDX(IWORK))
C
C                ********************
            CALL getatomid(JAT,FWORK)
C                ********************
C
C---- See if atom type asked for
C     i.e. CB CA   (means radii bonds C-beta TO C-alpha)
C
            IF (KAT.LE.0 .OR. JAT.LE.0) 
C
C            *****************************************************
     +  CALL CCPERR(1,' BOND Specifications error on RADII bonds')
C            *****************************************************
C
C---- specific bond named here
C
            N1 = KAT
            N2 = JAT
C
C---- store bond radius and number of lines.
C
            IF (IDASH.EQ.0) THEN
              IWORK = IWORK + 1
              IF (IWORK.LT.NTOKX) THEN
                KWWANT = 1
                KWSTART = IWORK
C
C                    **********************************************
                CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,
     +                      NTOKX)
C                    **********************************************
C
                RB = FVALUEX(IWORK)
                IWORK = IWORK + 1
                IF (IWORK.GT.NTOKX) THEN
                  L = Ldef_Bnd_Lines
                ELSE
                  KWWANT = 1
                  KWSTART = IWORK
C
C                      ***********************************************
                  CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,
     +                        NTOKX)
C                      ***********************************************
C
                  L = NINT(FVALUEX(IWORK))
                END IF
              END IF
            END IF
            N3 = NINT((RB+L)*1000.0)
            IF (IDASH.NE.0) N3 = -IDASH
            Nspecified_Bonds = Nspecified_Bonds + 1
            ISpecified_Bond_Params(Nspecified_Bonds,1) = N1
            ISpecified_Bond_Params(Nspecified_Bonds,2) = N2
            ISpecified_Bond_Params(Nspecified_Bonds,3) = N3
            RB = Def_Bnd_Radius
            L = Ldef_Bnd_Lines
            GO TO 40
          END IF
C
C---- end of RADII BONDS
C
        END IF
C
C---- start of RADII ATOMS
C
        IF (CWORK.EQ.'ATOM') THEN
C
C---- atom radius specification here...
C
          IWORK = IWORK + 1
          IF (IWORK.GT.NTOKX) GO TO 40
          CWORK = CVALUEX(IWORK)
C
C              *************
          CALL CCPUPC(CWORK)
C              *************
C
C---- set all atom radii to the value following
C
C---- RADII ATOMS ALL n
C
          IF (CWORK(1:3).EQ.'ALL') THEN
            IWORK = IWORK + 1
C
C
            IF (IWORK.GT.NTOKX) 
C
C            **********************************************************
     +  CALL CCPERR(1,' Error on RADII ATOMS ALL n, NOT enough tokens')
C            **********************************************************
C
            KWWANT = 1
            KWSTART = IWORK
C
C                ****************************************************
            CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                ****************************************************
C
            IF (Map_Flag_Counter.NE.0) THEN
              Atm_Plot_Radii(1) = FVALUEX(IWORK)
              GO TO 40
            ELSE
C
C
              DO 570 JDO = 9,NAT
C
C                    **************************
                CALL setrad(JDO,FVALUEX(IWORK))
C                    **************************
C
  570         CONTINUE
              GO TO 40
            END IF
C
C---- RADII ATOMS CA n C m ......
C
          ELSE
C
C
            DO 590 JDO = IWORK,NTOKX,2
              FWORK = LINEX(IBEGX(JDO) :IENDX(JDO))
C
C                  ********************
              CALL getatomid(KAT,FWORK)
C                  ********************
C

              IF (KAT.EQ.0) THEN
                WRITE (6,FMT=6010) FWORK
 6010           FORMAT (' ERROR for RADII ATOMS atom_name radii',/,
     +                  ' atom_name < ',A,' > ',/,
     +                  ' NOT recognised')
              ELSE IF (KAT.LT.0 .AND. Map_Flag_Counter.EQ.0) THEN
C
C---- Atom_type given
C
                M = KAT
C
C
                DO 580 K = 9,NAT
C
C                                         *****************************
                  IF (IAtom_Type(K).EQ.M) CALL setrad(K,FVALUEX(JDO+1))
C                                         *****************************
C
  580           CONTINUE
C
              ELSE IF (KAT.GT.0 .AND. Map_Flag_Counter.EQ.0) THEN
C
C---- Atom_name given
C     store this radius
C
                NumPlotRadii = NumPlotRadii + 1
                KWWANT = 1
                KWSTART = JDO + 1
C
C                    **********************************************
                CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,
     +                      NTOKX)
C                    **********************************************
C
                Atm_Plot_Radii(NumPlotRadii) = FVALUEX(JDO+1)
                LRadi_Plt_typ(NumPlotRadii) = KAT
              ELSE
C
C---- RADII ATOMS atom_name radii
C     individual atom radii not allowed with map plotting, ignore
C
                IF (Map_Flag_Counter.GT.0) GO TO 590
                KWWANT = 1
                KWSTART = JDO + 1
C
C                    *********************************************
                CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,
     +                      NTOKX)
                CALL setrad(KAT,FVALUEX(JDO+1))
C                    *********************************************
C
              END IF
  590       CONTINUE
          END IF
C
C---- End of RADII ATOMS
C
        END IF
C
C
        GO TO 40
C
C

C     =====
C---- PACK   Not Documented ????
C     =====
C
C     packing diagram control
C
      ELSE IF (KEYX.EQ.'PACK') THEN
        Lpack_Flag = 1
        IWORK = 2
C
C---- pack  range     x1,x2,y1,y2,z1,z2
C
C    Used by pack range instruction
C    find molecules having their centre wthin the range given
C
        LPRINTSAVE = Lprint_Flag
        LJOINSAVE = Ljoin_Flag
        Lprint_Flag = 0
        Ljoin_Flag = 0
C
         IDUMMY = 0
         KDUMMY = 0
C            ***********************************
        CALL dscan(Lpack_Flag,IDUMMY,KDUMMY,FVALUEX,IWORK)
C            ***********************************
C
        Lprint_Flag = LPRINTSAVE
        Ljoin_Flag = LJOINSAVE
        GO TO 40
C
C     ==========================
C---- MOL   ???? Not documented
C     ==========================
C
C          moles to be plotted
C
      ELSE IF (KEYX.EQ.'MOLE') THEN
        IF (CWORK.EQ.'NONE') THEN
          Num_Moles = 0
          GO TO 40
        END IF
C
C
        IF (NTOKX.LT.5) 
C
C            *****************************************************
     +  CALL CCPERR(1,' Error on MOLE Key_Word NOT enough tokens')
C            *****************************************************
C
        DO 600 JDO = 2,NTOKX,4
          Num_Moles = Num_Moles + 1
C
C
          IF (Num_Moles.GT.Max_Mols) 
C
C            ***************************************************
     +  CALL CCPERR(1,' MOLE Key_Word, Max Number of MOLE = 50')
C            ***************************************************
C
          KWWANT = 4
          KWSTART = JDO
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          MOL(Num_Moles,1) = NINT(FVALUEX(JDO))
          MOL(Num_Moles,2) = NINT(FVALUEX(JDO+1))
          MOL(Num_Moles,3) = NINT(FVALUEX(JDO+2))
          MOL(Num_Moles,4) = NINT(FVALUEX(JDO+3))
  600   CONTINUE
C
C
        GO TO 40
C
C     ================
C---- INCLUDE/OMIT
C     ================
C
C  OMIT atoms list or atom type list
C
C     Omit from plotting all  specified  atoms  or  all  atoms  of
C     specified types.  Up to eight atoms or types may be given on
C     each OMIT card.  This card may also appear in group 1 before
C     INPUT  to omit all atoms of one type from the atom selection
C     (one OMIT card only).
C
C
      ELSE IF (KEYX.EQ.'INCL' .OR. KEYX.EQ.'OMIT') THEN
        NOMITWRK = 0
        IF (NAT.LT.9) GO TO 40
        IF (KEYX.EQ.'INCL') THEN
          NOMITWRK = 1
C
C
          DO 610 JDO = 9,NAT
            LAtom_Inc_Flag(JDO) = 0
  610     CONTINUE
C
C
        END IF
C
C
        IWORK = 0
C
C
        DO 630 JDO = 2,NTOKX
          FWORK = LINEX(IBEGX(JDO) :IENDX(JDO))
C
C              ******************
          CALL getatomid(M,FWORK)
C              ******************
C
          IATID(JDO-1) = M
C
C
          IF (M.EQ.0) THEN
            WRITE (6,FMT=6012) FWORK
 6012       FORMAT (' ERROR atomname < ',A,' > on KeyWord OMIT/INCLUDE',
     +             ' NOT recognised')
          ELSE IF (M.GT.0) THEN
            IWORK = IWORK + 1
            LAtom_Inc_Flag(M) = NOMITWRK
          ELSE
            IWORK = IWORK + 1
C
C
            DO 620 I = 9,NAT
              IF (IAtom_Type(I).EQ.M) LAtom_Inc_Flag(I) = NOMITWRK
  620       CONTINUE
C
C
          END IF
  630   CONTINUE
C
C
        KCHOSE = IWORK
        IF (KEYX.EQ.'OMIT') KCHOSE = -KCHOSE
        GO TO 40
C
C       =============
C----   DCELL
C       =============
C
C  DCELL Draw unit cell as a box, with the origin and axes labelled.
C        If a  BOUNDS  CELL  card  is  present,  the cell plotted is
C        the  bounds limits.
C
      ELSE IF (KEYX.EQ.'DCEL') THEN
        Ldraw_cell_Flag = 2
        LAtom_Label_Flag(1)=1
        LAtom_Label_Flag(2)=1
        LAtom_Label_Flag(4)=1
        LAtom_Label_Flag(5)=1
        GO TO 40
C
C       =========================
C----   STEREO     COLOUR SEP s d
C       =========================
C
C     Causes a stereo pair to be drawn, as seen by an observer  at
C     a  distance  of  d  mm  (default  600mm) from the top of the
C     plotting coordinates, with an eye separation of  60mm.   The
C     keyword  COLOUR  selects red/green stereo, otherwise the two
C     images  are  side  by  side.   The  keyword  SEP  sets   the
C     separation  of the two images to s mm.  This separation will
C     be altered if necessary by the program to a minimum value of
C     the  width  of the drawing (default), and a maximum value of
C     (paper width - drawing width) (see SIZE and DEVICE cards for
C     paper  width).   If the drawing is wider than half the paper
C     width, the stereo images will overlap.
C
C
C   MONO    d
C
C     Draw perspective drawing, as seen from a distance d (default
C     600mm).   If  neither MONO nor STEREO cards are present, the
C     mono drawing is without perspective.
C
      ELSE IF (KEYX.EQ.'STER' .OR. KEYX.EQ.'MONO') THEN
C
C----  stereo
C
        LStereo_Flag = 0
        IF (KEYX.EQ.'STER') LStereo_Flag = 1
        IF (KEYX.EQ.'MONO') Lperspective_Flag = 1
        IWORK = 1
  640   CONTINUE
        IWORK = IWORK + 1
        IF (IWORK.GT.NTOKX) GO TO 40
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
        IF (CWORK.EQ.'COLO') THEN
          Lplot_colour_Stereo = -1
          GO TO 640
C
C---- MONO [STEREO] NOBOX
C
        ELSE IF (CWORK.EQ.'NOBO') THEN
          Use_Box = .FALSE.
          GO TO 640
C
C---- STEREO NOTICKS
C
        ELSE IF (CWORK.EQ.'NOTI') THEN
          Use_Ticks = .FALSE.
          GO TO 640
C
C---- keyword sep, read stereo pair separation in mm
C
        ELSE IF (CWORK(1:3).EQ.'SEP') THEN
          IWORK = IWORK + 1
          KWWANT = 1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Stereo_Separation = FVALUEX(IWORK)
          GO TO 640
        ELSE
          KWWANT = 1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Perspective_Distance = FVALUEX(IWORK)
          GO TO 640
        END IF
C
C      ==================================
C----  SIZE   s1 CHAR s2 SCALE p PAPER w
C      ==================================
C
C     This card controls the scaling of the drawing.  If the SCALE
C     keyword  is  given, p is the scale in mm/A .  If no scale is
C     given  (see  also  the  MAP  card),  the  scale  factor   is
C     calculated  to fit the drawing into size s1 mm.  The scaling
C     is normally done along the plotter Xp axis, but if the  PLOT
C     Y option is used, scaling is done along the plotter Yp axis.
C     The default size is 250mm, 125mm for side-by-side stereo, or
C     244mm for PLOT Y.
C                    The keyword CHAR sets the  size  for  labels:
C     the characters are spaced by s2 mm (default = 2mm).
C                    The keyword PAPER (see also the DEVICE  card)
C     sets  the  paper width for scaling and calculation of stereo
C     separation.
C                    Note that a value for s1 must be given:  this
C     may  be  0  if an explicit SCALE is given, either here or on
C     the MAP card.
C
C     eg   SIZE 0 CHAR 1.5
C
C
C size control
C size of box enclosing drawing, scale a/mm, characters.
C
C  SIZE   s1 CHAR s2 SCALE p PAPER w
C
      ELSE IF (KEYX.EQ.'SIZE') THEN
        IF (NTOKX.LE.1)
C
C            ********************************************** 
     +  CALL CCPERR(1,'SIZE NO scale given as first value')
C            ********************************************** 
C
        KWWANT = 1
        KWSTART = 2
C
C            ****************************************************
        CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C            ****************************************************
C
        Drawing_Size = FVALUEX(2)
        IWORK = 2
  650   CONTINUE
        IWORK = IWORK + 1
        IF (IWORK.GT.NTOKX) GO TO 40
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
C
C
        IF (CWORK.EQ.'PAPE') THEN
          IWORK = IWORK + 1
          KWWANT = 1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Paper_Width = FVALUEX(IWORK)
          GO TO 650
        ELSE IF (CWORK.EQ.'SCAL') THEN
          IWORK = IWORK + 1
          KWWANT = 1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Picture_Scale = FVALUEX(IWORK)
          GO TO 650
        ELSE IF (CWORK.EQ.'CHAR') THEN
          IWORK = IWORK + 1
          KWWANT = 1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Charcter_Size = fvaluex(iwork)
          GO TO 650
        END IF
        GO TO 40
C
C      ============================================
C----  Matrix   r11,r12,r13,r21,r22,r23,r31,r32,r33
C      ============================================
C
C
C     Read view matrix R, Xp = R Xo .   Alternative  to  the  VIEW
C     card.
C
C
      ELSE IF (KEYX.EQ.'MATR') THEN
        IF (NTOKX.LT.10) 
C
C           **************************************************
     + CALL CCPERR(1,'MATRIX KeyWord NOT enough tokens given')
C           **************************************************
C
C
C
        IWORK = 1
C
C
        DO 670 I = 1,3
          DO 660 J = 1,3
            IWORK = IWORK + 1
            KWWANT = 1
            KWSTART = IWORK
C
C                ****************************************************
            CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                ****************************************************
C
            ViewMatrix(I,J) = FVALUEX(IWORK)
  660     CONTINUE
  670   CONTINUE
C
C
        LSetView_Flag = 1
C
C            **********************
        CALL testmatrix(ViewMatrix)
C            **********************
C
        GO TO 40
C
C      ==============
C----  Colour keyword
C      ==============
C
C     Keywords BLACK, RED, GREEN, BLUE, CYAN, ORANGE, MAGENTA,
C     YELLOW  or  PURPLE
C     set the pen colour (where not overridden by other commands).
C     Default is black.
C
      ELSE IF (KEYX.EQ.'COLO') THEN
        IF (NTOKX.LT.2) GO TO 40
        CWORK = CVALUEX(2)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
        Lplot_colour_Stereo = 1
C
C
        IF (CWORK.EQ.'BLAC') THEN
          Lplot_colour_Stereo = 1
        ELSE IF (CWORK.EQ.'CYAN') THEN
          Lplot_colour_Stereo = 2
        ELSE IF (CWORK(1:3).EQ.'RED') THEN
          Lplot_colour_Stereo = 3
        ELSE IF (CWORK.EQ.'GREE') THEN
          Lplot_colour_Stereo = 4
        ELSE IF (CWORK.EQ.'BLUE') THEN
          Lplot_colour_Stereo = 5
        ELSE IF (CWORK.EQ.'MAGE') THEN
          Lplot_colour_Stereo = 6
        ELSE IF (CWORK.EQ.'YELL') THEN
          Lplot_colour_Stereo = 7
        ELSE IF (CWORK.EQ.'PURP') THEN
          Lplot_colour_Stereo = 8
        ELSE IF (CWORK.EQ.'ORAN') THEN
          Lplot_colour_Stereo = 9
        ELSE IF (CWORK.EQ.'BLAC') THEN
          Lplot_colour_Stereo = 10
        END IF
        GO TO 40
C
C     ===========
C---- PRINT
C     ===========
C
C
C     This card may appear with the  group  4  cards,  and  causes
C     extensive  diagnostic  printing  in  the  atom  input,  atom
C     linking, and reading of the scratch file.  Its main  use  is
C     for debugging.
C
C
C printing control...
C
      ELSE IF (KEYX.EQ.'PRIN') THEN
        Lprint_Flag = 1
        LOGPARSER = .TRUE.
        IF (NTOKX.GT.1) THEN
          CWORK = CVALUEX(2)
C
C              *************
          CALL CCPUPC(CWORK)
C              *************
C
          IF (CWORK(1:1).EQ.'N') Lprint_Flag = 0
        END IF
        GO TO 40
C
C
      ELSE IF (KEYX.EQ.'BNLA') THEN
        NBondLabels = NBondLabels + 1
        IF (NBondLabels.GT.Max_BndLabels) THEN
          WRITE (6,FMT=6014)
 6014     FORMAT (' No more bond labels allowed max = 10')
          NBondLabels = NBondLabels - 1
          GO TO 40
        END IF
C
C
        IWORK = 1
        IWORK = IWORK + 1
        IF (IWORK+1.GT.NTOKX) 
C
C            *************************************************
     +  CALL CCPERR(1,' Not enough tokens on bond label line')
C            *************************************************
C
        BondtoLabel(NBondLabels*2-1) = LINEX(IBEGX(IWORK) :IENDX(IWORK))
        BondtoLabel(NBondLabels*2) = LINEX(IBEGX(IWORK+1) :
     +                               IENDX(IWORK+1))
        IWORK = IWORK + 2
        IF (IWORK.LE.NTOKX) Perp_Offset(NBondLabels) = FVALUEX(IWORK)
        IF (IWORK+1.LE.NTOKX) Parr_Offset(NBondLabels) = FVALUEX(IWORK+
     +      1)
        GO TO 40
C
C     ======
C---- PLOT Y
C     ======
C
C
C     This must be the last control card for a drawing, and starts
C     the  drawing  process.   Normally  the  plotter x axis Xp is
C     across the paper , and the Yp axis along the roll,  but  the
C     keyword  Y  rotates the whole drawing through 90 degrees, to
C     put Xp along the roll.
C
C
C
      ELSE IF (KEYX.EQ.'PLOT') THEN
C
C PLOT .... this card may contain the number of traces
C           and number of copies of the drawing to be made.
C
C  yacross   draw y axis across paper (Lpaper_Setting=1)
C            THE DEFAULT AT emblhh & dl
C            xacross, Lpaper_Setting=0
C            THE DEFAULT USING plot84
C
        Lpaper_Setting = 0
C
C---- Width and length now set in plotmolecules
C
        Paper_Width = Paper_Width - 1.0
        Paper_Length = Paper_Length - 1.0
        IWORK = 1
  680   CONTINUE
        IWORK = IWORK + 1
        IF (IWORK.GT.NTOKX) GO TO 690
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
        IF (CWORK(1:1).EQ.'X') THEN
          Lpaper_Setting = 0
          GO TO 680
        ELSE IF (CWORK(1:1).EQ.'Y') THEN
          Lpaper_Setting = 1
          GO TO 680
        ELSE IF (CWORK.EQ.'TRAC') THEN
          KWWANT = 1
          KWSTART = IWORK + 1
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          LTrace_Flag = NINT(FVALUEX(IWORK+1))
          IWORK = IWORK + 1
          GO TO 680
        ELSE IF (CWORK.EQ.'COPY') THEN
          KWWANT = 1
          KWSTART = IWORK + 1
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Ncopies = NINT(FVALUEX(IWORK+1))
          IWORK = IWORK + 1
          GO TO 680
        ELSE
          GO TO 680
        END IF
C
  690   IF (Lpaper_Setting.NE.1) THEN
C
C---- xacross set size
C
          IF (Drawing_Size.GT.0.001) GO TO 710
          Drawing_Size = Paper_Width
          IF (LStereo_Flag.GT.0 .AND. Lplot_colour_Stereo.NE.
     +        -1) Drawing_Size = Paper_Width/2.0
        ELSE
C
C---- yacross
C
          IF (Drawing_Size.LT.0.001) Drawing_Size = Paper_Width -
     +        Map_Flag_Counter*2.0*Charcter_Size - 6.0
        END IF
C
C---- eliminate all previously stored global overlap information
C
        Num_Overlaps = 0
        NQuads_Overlaped = 0
        Num_Conics = 8
        Num_Bnd_Quadrangles = 0
C
C
        GO TO 710
C
C     ============================
C---- SET  - reset all parameters
C     ============================
C
      ELSE IF (KEYX(1:3).EQ.'SET') THEN
C
C            ****************
        CALL primer('RESETS')
C            ****************
C
        GO TO 40
C
C     ==============
C---- Font     NFont_Number
C     ==============
C
C      Set font number for different style characters.
C
C             = 0  simple capitals
C             = 1  block letters (default)
C             = 2  italic
C             = 3  script
C
      ELSE IF (KEYX.EQ.'FONT') THEN
C
C  For GHOST NFont_Number = 0 not used in GHOST,
C                      so used here as flag for italics
C                    1 default Roman font
C                    2 Greek
C                    3 Cyrillic
C
        IF (NTOKX.GE.2) THEN
          KWWANT = 1
          KWSTART = 2
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          NFont_Number = NINT(FVALUEX(2))
        END IF
C
C
        IF (NFont_Number.LT.0 .OR. NFont_Number.GT.3) NFont_Number = 1
        GO TO 40
C
C     ================
C---- Thick     NPen_Thickness
C     ================
C
C
C     Set line-thickness for Trilog, default =1 .
C     If  .gt.   1  (.le.9) the lines are thickened
C     NPen_Thickness times.
C     This is useful  for large-scale plots.
C
      ELSE IF (KEYX.EQ.'THIC') THEN
        IF (NTOKX.GE.2) THEN
          KWWANT = 1
          KWSTART = 2
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          NPen_Thickness = NINT(FVALUEX(2))
        END IF
C
C
        IF (NPen_Thickness.LE.0 .OR. NPen_Thickness.GE.
     +      10) NPen_Thickness = 1
        GO TO 40
C
C     ===============
C---- DEVICE keywords
C     ===============
C
C
C      The DEVICE card sets the paper size (and plotter step  size)
C      appropriate  for  a given device.  The parameters may be set
C      individually by a keyword/value pair:
C            WIDTH      paper width in mm
C            LENGTH     paper length in mm
C            STEP       plotter step in mm (default = 0.25)
C      The paper size is used for scaling if a scale factor is  not
C      given,  and  for  clipping  the picture if a scale factor is
C      given.  The STEP keyword (see also the  SOLID  card)  allows
C      the  plotter  step  size for circle drawing to be reset from
C      its default of 0.25 mm to finer step eg 0.1 .   The  smaller
C      step gives smoother circles but uses more time, particularly
C      if the hidden lines option is used.
C
C      Alternatively, appropriate values may be  set  for  standard
C      devices:
C                             Keyword    Width    Length    Step
C                   (default)  TRILOG     330      508     default
C                              BIG        330     2032     default
C      for Hewlett-Packard     HP         275      350       0.1
C      eg       DEVICE HP  LENGTH 300 STEP 0.25
C                Set values for HP plotter, then change length
C                to 300mm, step to 0.25
C
      ELSE IF (KEYX.EQ.'DEVI') THEN
C
C   Keywords  WIDTH    read paper width in mm  Paper_Width
C             LENGTH   read paper length in mm Paper_Length
C             STEP     read plotter step in mm Plotter_Step
C                      (default = 0.25)
C
C           GRID (X and Y are swapped here !)
C                      set width = 253.  length=320.
C             TRILOG   set width = 328., length=508.
C             BIG      set width = 328., length=2032.
C             HP       set width = 275., length=350., step =0.1mm
C                      (for Hewlett-Packard plotter)
C
        IWORK = 1
  700   CONTINUE
        IWORK = IWORK + 1
        IF (IWORK.GT.NTOKX) GO TO 40
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
        IF (CWORK.EQ.'TRIL') THEN
C
C---- Trilog
C
          Paper_Width = 328.0
          Paper_Length = 508.0
          GO TO 700
        ELSE IF (CWORK(1:3).EQ.'BIG') THEN
C
C---- Big
C
          Paper_Width = 328.0
          Paper_Length = 2032.0
          GO TO 700
        ELSE IF (CWORK(1:2).EQ.'HP') THEN
C
C----  hp
C
          Paper_Width = 275.0
          Paper_Length = 350.0
          Plotter_Step = 0.1
          GO TO 700
        ELSE IF (CWORK.EQ.'GRID') THEN
C
C---- grid file (GHOST)
C
          Paper_Width = 253.0
          Paper_Length = 320.0
          GO TO 700
        ELSE IF (CWORK.EQ.'WIDT') THEN
C
C---- width
C
          KWWANT = 1
          KWSTART = IWORK + 1
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Paper_Width = FVALUEX(IWORK+1)
          IWORK = IWORK + 1
          GO TO 700
        ELSE IF (CWORK.EQ.'LENG') THEN
C
C---- length
C
          KWWANT = 1
          KWSTART = IWORK + 1
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Paper_Length = FVALUEX(IWORK+1)
          IWORK = IWORK + 1
          GO TO 700
        ELSE IF (CWORK.EQ.'STEP') THEN
C
C---- step
C
          KWWANT = 1
          KWSTART = IWORK + 1
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Plotter_Step = FVALUEX(IWORK+1)
          IWORK = IWORK + 1
          GO TO 700
        ELSE
C
C              ************************************
          CALL CCPERR(1,' ERROR on KeyWord DEVICE')
C              ************************************
C
        END IF
C
C
        GO TO 40
C
C     ======
C---- Notitl
C     ======
C
C  Suppresses all titles, and other text around the plot
C
C
      ELSE IF (KEYX.EQ.'NOTI') THEN
        Log_Draw_Title = .FALSE.
        GO TO 40
C
C      ===============
C----  CENtre  command
C      ===============
C
C  read min and max of coords (as printed out by previous run)
C  for produceing a series of pictures on same origin
C  Xmin,Xmax,Ymin,Ymax,Zmin,Zmax are in plot frame in A
C
C
      ELSE IF (KEYX.EQ.'CENT') THEN
C
C
        IF (NTOKX.LT.7) 
C
C            *************************************************
     +  CALL CCPERR(1,' Not enough Tokens on CENTRE Key_Word')
C            *************************************************
C
        KWWANT = 6
        KWSTART = 2
C
C            ****************************************************
        CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C            ****************************************************
C
        Cent_Xmax = FVALUEX(2)
        Cent_Ymax = FVALUEX(3)
        Cent_Zmax = FVALUEX(4)
        Cent_Xmin = FVALUEX(5)
        Cent_Ymin = FVALUEX(6)
        Cent_Zmin = FVALUEX(7)
        LCent_Flag = .TRUE.
        GO TO 40
C
C     ================
C---- ???????????????
C     ================
C
      ELSE
C
C----  ignore this card.
C
        WRITE (6,FMT=6016) LINEX(1:LENSTR(LINEX))
 6016   FORMAT (' IGNORED >> ',A)
        GO TO 40
C
C
      END IF
C
C       ===================
C------ End of all Keywords
C       ===================
C
C
C--- Jump here at end of PLOT card,
C    try to exit, check for errors first
C
  710 CONTINUE
      IF (LJoinRadRes .AND. Map_Flag_Counter.LE.0) THEN
        CALL getatomconnections(Lprint_Flag)
      ENDIF
C
C
      WRITE (6,FMT=6018) Drawing_Size,Picture_Scale,Charcter_Size,
     +  Perspective_Distance
 6018 FORMAT (
     +' Drawing_Size         = ',F10.4,/,
     +' Picture_Scale        = ',F10.4,/,
     +' Charcter_Size        = ',F10.4,/,
     +' Perspective_Distance = ',F10.4)
C
C---- set symmetry to identity opr by default.
C
      IF (NSymmetry_Ops.LE.0) THEN
C
C
        DO 730 I = 1,3
          DO 720 J = 1,3
            Rotsym_Matrx(I,J,1) = 0.0
  720     CONTINUE
          TransSym_Vect(I,1) = 0.0
          Rotsym_Matrx(I,I,1) = 1.0
  730   CONTINUE
C
C
        NSymmetry_Ops = 1
      END IF
C
C
      IF (LSetView_Flag.EQ.0) 
C
C            *****************************************************
     +  CALL CCPERR(1,' ***ERROR---View direction not defined***')
C            *****************************************************
C
C---- Here is the return
C
      RETURN
C
C-----DL MOD - If end of input file reached,
C              close graphics properly & stop
C
C          *****************************************
  740 CALL zendplotting
      CALL CLOSEIT(NUM_FILES)
      CALL CCPERR(0,' Normal Termination of npluto')
C          *****************************************
C
C
      END
C
C
C
C     =============================================================
      SUBROUTINE akwmapdat(LINEX,IBEGX,IENDX,ITYPX,FVALUEX,CVALUEX,
     +                     IDECX,NTOKX,LOGPARSER)
C     ==============================================================
C
C
C
C---- Sets in COMMON:
C
C        Picture_Scale
C        Invert_Map_Flag
C        Lmap3d_Flag
C        Map_NoBox
C
C
C read control cards for map plotting
C 1st keywords map,contrs,mode,sectns
C
C Group 2, map control
C              MAP          C   Must be 1st in group
C              CONTRS        C   Must be 2nd in group
C              LIMITS*       O
C              MODE          O
C              SECTNS        C
C              SLAB          O   Must be after SECTNS
C              GRID          O
C
C    MAP     map order and sampling SCALE  s INVERT 3D NOBOX
C
C     This card introduces map plotting.  The three letters X,  Y,
C     and Z must be given in the appropriate order to indicate the
C     fast, medium and slow (section) axes of the map (u,  v,  and
C     w).  They are accompanied by three numbers giving the number
C     of  sampling  intervals  along  the  whole  cell   in   each
C     direction.   The  optional  keyword SCALE gives the scale in
C     mm/A (this is an alternative to  the  SIZE  card).   If  the
C     keyword  3D is present, the map is contoured in three planes
C     (one  contour  level  only  recommended):   this   is   only
C     appropriate  for  map  stacks in stereo.  (*) The axis order
C     and sampling are not needed, but  if  present  override  the
C     data  in the file.  If the keyword NOBOX is present, the map
C     is not surrounded by a box.
C
C     eg  MAP   Y 104  X 152  Z 76  SCALE 2.5
C
C         Map has z sections in 1/76 ths with  y  running  fastest
C     (in 1/104) and x next (in 1/152).  The scale is 2.5 mm/A.
C
C         The default orientation of the map  on  the  plotter  is
C     with  the  fast  axis  u  along the plotter Xp (to right), v
C     along -Yp (down the paper) and w along +/-  Zp  (towards  or
C     away  depending  on  the  permutation of axes).  The keyword
C     INVERT on the MAP card interchanges u  &  v  and  inverts  w
C     relative to the plotter axes, ie it puts v along Xp, u along
C     -Yp, and w along-/+ Zp.  Alternatively, a viewpoint  may  be
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER Max_Symmetry
      PARAMETER (Max_Symmetry=192)
      INTEGER Max_Tokens
      PARAMETER (Max_Tokens=200)
C     ..
C     .. Scalar Arguments ..
      INTEGER NTOKX
      LOGICAL LOGPARSER
      CHARACTER LINEX*600
C     ..
C     .. Array Arguments ..
      REAL FVALUEX(Max_Tokens)
      INTEGER IBEGX(Max_Tokens),IDECX(Max_Tokens),IENDX(Max_Tokens),
     +        ITYPX(Max_Tokens)
      CHARACTER CVALUEX(Max_Tokens)*4
C     ..
C     .. Scalars in Common ..
      REAL Charcter_Size,D001,D010,D100,Drawing_Size,ROR(4,4),RFR(4,4),
     +     Perspective_Distance,Picture_Scale,S,Shadelight_X,
     +     Shadelight_Y,Stereo_Separation,Ugrid_Offset,Ugrid_Spacing,
     +     Vgrid_Offset,Vgrid_Spacing,StpMapGrpSect,
     +     First_Section,Last_Section,Sections_in_Group
      INTEGER Invert_Map_Flag,KSEC1,KSEC2,Lcurrent_Colour,
     +        Ldraw_cell_Flag,LGRID,LimitsSet_Flag,Lmap3d_Flag,
     +        Lmap_ModeFirst,Lorthogonalisation_Code,Lpaper_Setting,
     +        Lperspective_Flag,Lplot_colour_Stereo,Lposition_Flag,
     +        Lprint_Flag,LSetView_Flag,LShade_Flag,LSolid_Flag,
     +        LStereo_Flag,LSuperimpose_Flag,LTaper_Flag,LTrace_Flag,
     +        Map_Counter,Map_Flag_Counter,Map_Nobox,Map_pts_U,
     +        Map_pts_V,Msecst,MU,MV,MW,Natoms_plusCell,Ncopies,
     +        NFirst_Section,NLast_Section,Npicture,Npts_U_compressed,
     +        Npts_V_compressed,Nsections_in_Group,Nspecified_Bonds,
     +        NstpMapGrpSect,NSymmetry_Ops,Num_Moles
      LOGICAL Log_Draw_Title
C     ..
C     .. Arrays in Common ..
      REAL AID,AQ,Bound_Edge,Bound_Limits,Contour_Levels,
     +     Contour_Threshold,OrtCell,OrtConts,Plotting_XYZ,Rotsym_Matrx,
     +     RP,TransSym_Vect,ViewMatrix,XABC,XCLANG
      INTEGER ISpecified_Bond_Params,Major_Grid,MapGrdCol,
     +        Map_def_Restrictions,Map_LineTypes,Map_UVW_Limits,
     +        Map_XYZ_Limits,MOL,NSECC,Num_Contours
      CHARACTER Map_Title*80
C     ..
C     .. Local Scalars ..
      REAL CEND,CSTART,CSTEP,CT1,CT1STR,CT2,CT3,EDGE,Q3,RAD,ST1STR,ST2,
     +     ST3,T,U,V
      REAL RHMAX,RHMEAN,RHMIN,RHRMS
      INTEGER I,IWORK,J,J1,J2,JDO,JWORK,K,KCOUNT_UVW,
     +        KLOOP,KWSTART,KWWANT,LPERM,LSLAB,M,MAPCOLOURS_WRK,
     +        MAPDASH_WRK,MAPFLAG_WRK,MAPMODE_WRK,MP,MQ,MR,NCOUNT,NSEC1,
     +        NSEC2,NUMGRID_CARDS
      LOGICAL LENDX,FRACSEC
      CHARACTER CWORK*4,CWORK1*4,KEYX*4
C     ..
C     .. Local Arrays ..
      REAL A(3,3),B(3,3),PT(3,3),Q(3,3),RTSYM(4,4,Max_Symmetry),CELL(6),
     +  MAP_SECfrac_first,MAP_SECfrac_last,MAP_Secfrac_range 
      INTEGER KEYWRD_UVW(3),LIMTMP(6),MAP_UVW(3),MAP_XYZ(3),
     +        NDIM1(5),NDIM2(5)
      CHARACTER LAXIS(3)*1
      LOGICAL NEG_FLG(Max_Maps),RMS_FLG(Max_Maps),MAX_FLG(Max_Maps)
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPERR,CCPUPC,getcellmatrices,KEYNUM,matrixmultply,
     +         MSYMOP,openmaps,PARSER
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,COS,NINT,REAL,SIN,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_GRID/LGRID,Map_Nobox,Ugrid_Spacing,Vgrid_Spacing,
     +       Ugrid_Offset,Vgrid_Offset,Major_Grid(Max_Maps),
     +       MapGrdCol(4,Max_Maps,2)
      COMMON /com_LIMITS/Bound_Limits(3),Bound_Edge(3)
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
      COMMON /com_SECTIONS/NFirst_Section,NLast_Section,
     +       Nsections_in_Group,NstpMapGrpSect,KSEC1,KSEC2,Msecst,
     +       NSECC(2)
      COMMON /com_SYMMETRY/NSymmetry_Ops,Rotsym_Matrx(3,3,Max_Symmetry),
     +       TransSym_Vect(3,Max_Symmetry)
C     ..
C     .. Equivalences ..
      EQUIVALENCE (MU,KEYWRD_UVW(1)), (MV,KEYWRD_UVW(2)),
     +            (MW,KEYWRD_UVW(3))
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Data statements ..
C
C
      DATA LAXIS/'X','Y','Z'/,RAD/57.2957795/
C     ..
C
C---- initialise
C
C   Put CELL as MAP cell - already been picked up..
      cell(1) = XABC(1)
      cell(2) = XABC(2)
      cell(3) = XABC(3)
      cell(4) = XCLANG(1)
      cell(5) = XCLANG(2)
      cell(6) = XCLANG(3)
      Invert_Map_Flag = +1
      Lmap3d_Flag = 0
      Map_Nobox = 0
      Lmap_ModeFirst = 0
      NUMGRID_CARDS = 1
C
C---- set flag to scale on cell
C
      Ldraw_cell_Flag = 1
C
C
      DO 20 I = 1,Max_Maps
        Num_Contours(I) = 8
        Contour_Threshold(I) = -1.0E10
        Major_Grid(I) = 1
        DO 10 J = 1,2
          MapGrdCol(1,I,J) = 0
          Map_LineTypes(1,I,J) = 0
   10   CONTINUE
   20 CONTINUE
C
C
      Ugrid_Spacing = 0.1
      Vgrid_Spacing = Ugrid_Spacing
      Ugrid_Offset = 0.0
      Vgrid_Offset = 0.0
      LGRID = 0
      LSLAB = 0
      LimitsSet_Flag = 0
      NDIM1(1) = -10000
      First_Section = 0
      NFirst_Section = 0
      Last_Section = 500
      NLast_Section = 500
      sections_in_Group = 1
      Nsections_in_Group = 1
      stpMapGrpSect = 1
      NstpMapGrpSect = 1
      Msecst = 1
      DO 15 J=1,2
         DO 15, M = 1,2
            MapGrdCol(1,M,J) = 0
 15   CONTINUE
C
C       ========================================================
C----    MAP     map order and sampling SCALE  s INVERT 3D NOBOX
C       ========================================================
C
C    This card introduces map plotting.  The three letters X,  Y,
C    and Z must be given in the appropriate order to indicate the
C    fast, medium and slow (section) axes of the map (u,  v,  and
C    w).  They are accompanied by three numbers giving the number
C    of  sampling  intervals  along  the  whole  cell   in   each
C    direction.   The  optional  keyword SCALE gives the scale in
C    mm/A (this is an alternative to  the  SIZE  card).   If  the
C    keyword  3D is present, the map is contoured in three planes
C    (one  contour  level  only  recommended):   this   is   only
C    appropriate  for  map  stacks in stereo.  (*) The axis order
C    and sampling are not needed, but  if  present  override  the
C    data  in the file.  If the keyword NOBOX is present, the map
C    is not surrounded by a box.
C
C    eg  MAP   Y 104  X 152  Z 76  SCALE 2.5
C
C        Map has z sections in 1/76 ths with  y  running  fastest
C    (in 1/104) and x next (in 1/152).  The scale is 2.5 mm/A.
C
C        The default orientation of the map  on  the  plotter  is
C    with  the  fast  axis  u  along the plotter Xp (to right), v
C    along -Yp (down the paper) and w along +/-  Zp  (towards  or
C    away  depending  on  the  permutation of axes).  The keyword
C    INVERT on the MAP card interchanges u  &  v  and  inverts  w
C    relative to the plotter axes, ie it puts v along Xp, u along
C    -Yp, and w along-/+ Zp.  Alternatively, a viewpoint  may  be
C
C
C---- interpret map card
C
C KEYWORDS:
C   INVERT (optional) to put v axis along xp (default u along xp)
C   MAP AXIS ORDER MU,MV,MW, letters X,Y,Z, to give fast,medium,slow
C      directions in map (optional, otherwise picked up from map)
C   3 Numbers giving sampling intervals in these directions
C   SCALE (optional) scale in mm/a
C
C
C     #### Sub_Key_Words of "MAP" ####
C     ================================
C
      KEYWRD_UVW(1) = -1
      KCOUNT_UVW = 0
      IWORK = 1
   30 CONTINUE
      IWORK = IWORK + 1
      IF (IWORK.GT.NTOKX) GO TO 40
      CWORK = CVALUEX(IWORK)
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
C---- keyword scale, read scale in mm/a
C
      IF (CWORK.EQ.'SCAL') THEN
        IWORK = IWORK + 1
        KWWANT = 1
        KWSTART = IWORK
C
C            ****************************************************
        CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C            ****************************************************
C

        Picture_Scale = FVALUEX(IWORK)
        GO TO 30
C
C---- keyword invert, put v-axis along xp
C
      ELSE IF (CWORK.EQ.'INVE') THEN
        Invert_Map_Flag = -1
        GO TO 30
C
C---- Keyword '3D', set 3D contouring flag
C
      ELSE IF (CWORK(1:2).EQ.'3D') THEN
        Lmap3d_Flag = 1
        WRITE (6,FMT=6000)
 6000   FORMAT (' Contours will be in three planes',/)
        GO TO 30
C
C---- Keyword NOBOX, set flag
C
      ELSE IF (CWORK.EQ.'NOBO') THEN
        Map_Nobox = 1
        GO TO 30
C
C---- check for keyword x,y,z
C
      ELSE IF (CWORK(1:1).EQ.'X') THEN
        JWORK = 1
        KCOUNT_UVW = KCOUNT_UVW + 1
        KEYWRD_UVW(KCOUNT_UVW) = JWORK
        IWORK = IWORK + 1
        KWWANT = 1
        KWSTART = IWORK
C
C            ****************************************************
        CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C            ****************************************************
C
        Map_XYZ_Limits(JWORK) = NINT(FVALUEX(IWORK))
        GO TO 30
      ELSE IF (CWORK(1:1).EQ.'Y') THEN
        JWORK = 2
        KCOUNT_UVW = KCOUNT_UVW + 1
        KEYWRD_UVW(KCOUNT_UVW) = JWORK
        IWORK = IWORK + 1
        KWWANT = 1
        KWSTART = IWORK
C
C            ****************************************************
        CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C            ****************************************************
C
        Map_XYZ_Limits(JWORK) = NINT(FVALUEX(IWORK))
        GO TO 30
      ELSE IF (CWORK(1:1).EQ.'Z') THEN
        JWORK = 3
        KCOUNT_UVW = KCOUNT_UVW + 1
        KEYWRD_UVW(KCOUNT_UVW) = JWORK
        IWORK = IWORK + 1
        KWWANT = 1
        KWSTART = IWORK
C
C            ****************************************************
        CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C            ****************************************************
C
        Map_XYZ_Limits(JWORK) = NINT(FVALUEX(IWORK))
        GO TO 30
      ELSE
        WRITE (6,FMT=6002)
 6002   FORMAT (' Error, for Key_Word MAP Sub_Keyword not recognised')
        GO TO 30
      END IF
C
C-----  read next control card
C
   40 CONTINUE
C
C
      NTOKX = Max_Tokens
      LINEX = '    '
C
C          **********************************************************
      CALL PARSER(KEYX,LINEX,IBEGX,IENDX,ITYPX,FVALUEX,CVALUEX,IDECX,
     +            NTOKX,LENDX,LOGPARSER)
C          **********************************************************
C
      WRITE(6,45)LINEX(1:100)
45    FORMAT(' Data Line---- ',A100)
      IF (LENDX) GO TO 310
C
C       ========================
C----   CONTRS    contour levels This Key_Word also opens the maps
C       =========================
C
C     Contour levels for map.  If two maps are to be plotted, this
C     card will appear twice.  The contours may either be given as
C     a list, or as a lower & upper limits and interval.
C
C     eg   CONTRS    200 400 600 800
C          CONTRS 200 TO 800 BY 200
C
C     Equivalent positive  and  negative  contour  levels  may  be
C     specified  by  the  keyword  NEG before the specification of
C     range and interval.
C
C     eg   CONTRS NEG 20 TO 60 BY 20
C          will give contour levels 20 40 60 -20 -40 -60
C
C     Contours can be given as fractions of the maximum or by sigma
C     levels.
C
C     eg   CONTRS SIG 1.0 2.0 3.0
C          CONTRS NEG MAX 0.5 0.6 0.7
C
C       count maps to be plotted
C       read contour levels as list , or as start,stop,step
C
C
C     #### Sub_Key_Words of "CONTOURS" ####
C     =====================================
C
      IF (KEYX.EQ.'CONT') THEN
        Map_Flag_Counter = Map_Flag_Counter + 1
        NEG_FLG(Map_Flag_Counter) = .FALSE.
        RMS_FLG(Map_Flag_Counter) = .FALSE.
        MAX_FLG(Map_Flag_Counter) = .FALSE.
        IWORK = 2
C
C
        IF (Map_Flag_Counter.GT.Max_Maps) THEN
          WRITE (6,FMT=6004) Max_Maps
 6004     FORMAT (' Error - Only ',I2,' CONTOUR Key_Words allowed',
     +           /' The CONTOUR Key_Word also opens the MAPS',/' and i',
     +           's used as a Map counter')
          GO TO 40
        END IF
C
C
        CWORK = CVALUEX(IWORK)
        CWORK1 = CVALUEX(IWORK+1)
C
C            *************
        CALL CCPUPC(CWORK)
        CALL CCPUPC(CWORK1)
C            *************
C
        IF (CWORK(1:3).EQ.'NEG') THEN
          NEG_FLG(Map_Flag_Counter) = .TRUE.
          IWORK = IWORK + 1
          IF (CWORK1(1:3).EQ.'SIG') THEN
            RMS_FLG(Map_Flag_Counter) = .TRUE.
            IWORK = IWORK + 1
          ELSE IF (CWORK1(1:3).EQ.'MAX') THEN
            MAX_FLG(Map_Flag_Counter) = .TRUE.
            IWORK = IWORK + 1
          ENDIF
        ELSE
          IF (CWORK(1:3).EQ.'SIG') THEN
            RMS_FLG(Map_Flag_Counter) = .TRUE.
            IWORK = IWORK + 1
          ELSE IF (CWORK(1:3).EQ.'MAX') THEN
            MAX_FLG(Map_Flag_Counter) = .TRUE.
            IWORK = IWORK + 1
          ENDIF
        ENDIF
C
C---- keyword 'to', range & step given
C
        IF (IWORK+4.LE.NTOKX) THEN
          CWORK = CVALUEX(IWORK+1)
          CWORK1 = CVALUEX(IWORK+3)
C
C              **************
          CALL CCPUPC(CWORK)
          CALL CCPUPC(CWORK1)
C              **************
C
          IF (CWORK(1:2).EQ.'TO' .AND. CWORK1(1:2).EQ.'BY') THEN
C
C----          contrs 200 to 800 by 200
C              contrs neg 20 to 60 by 20
C
            CSTART = FVALUEX(IWORK)
            CEND = FVALUEX(IWORK+2)
            CSTEP = FVALUEX(IWORK+4)
            NCOUNT = 1
            Contour_Levels(1,Map_Flag_Counter) = CSTART
   50       Contour_Levels(NCOUNT+1,Map_Flag_Counter)
     +        = Contour_Levels(NCOUNT,Map_Flag_Counter) + CSTEP
            IF (Contour_Levels(NCOUNT+1,Map_Flag_Counter).GT.
     +          CEND) GO TO 70
            NCOUNT = NCOUNT + 1
            IF (NCOUNT.LE.Max_Contours) GO TO 50
            WRITE (6,FMT=6006) Max_Contours
 6006       FORMAT (//' !!! Maximum number of contour levels ',I5,/'  ',
     +             '   exceeded, rest ignored !!!',//)
            NCOUNT = NCOUNT - 1
            GO TO 70
C
C---- end of TO BY
C
          END IF
        END IF
C
C---- list of contour levels
C
        Num_Contours(Map_Flag_Counter) = NTOKX - IWORK + 1
C
C----     eg   contrs    200 400 600 800
C
        DO 60 JDO = 1,Num_Contours(Map_Flag_Counter)
          KWWANT = 1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Contour_Levels(JDO,Map_Flag_Counter) = FVALUEX(IWORK)
          IWORK = IWORK + 1
   60   CONTINUE
C
C
        NCOUNT = Num_Contours(Map_Flag_Counter)
C
C
   70   IF (NEG_FLG(Map_Flag_Counter)) THEN
C
C---- keyword 'negative', generate  negative contours
C              equivalent to contour levels  1 to n
C
          M = NCOUNT
C
C
          DO 80 JDO = 1,M
            NCOUNT = NCOUNT + 1
            IF (NCOUNT.GT.Max_Contours) GO TO 90
            Contour_Levels(NCOUNT,Map_Flag_Counter)
     +        = -Contour_Levels(JDO,Map_Flag_Counter)
   80     CONTINUE
C
C
          GO TO 100
C
C---- Too many levels
C
   90     WRITE (6,FMT=6006) Max_Contours
          NCOUNT = NCOUNT - 1
        END IF
C
C---- Open map file and read headers etc
C
  100   IF (Map_Flag_Counter.EQ.Max_Maps) GO TO 170
C
C---- 1st map
C       Check whether likely to be Patterson ( spacegrp centric) or not
        Patt_Flag = 0
C
C            **********************************************************
        CALL openmaps(Map_Flag_Counter,NDIM1,
     +                Map_Title(Map_Flag_Counter),NSEC1,MAP_UVW,MAP_XYZ,
     +                CELL,RHMAX,RHMEAN,RHMIN,RHRMS,Patt_Flag)
C            **********************************************************
C
        IF (RMS_FLG(Map_Flag_Counter)) THEN
          DO 6003 KLOOP = 1,NCOUNT
           Contour_Levels(KLOOP,Map_Flag_Counter) =
     +        Contour_Levels(KLOOP,Map_Flag_Counter)*RHRMS
 6003     CONTINUE
        ELSE IF (MAX_FLG(Map_Flag_Counter)) THEN
          IF (NEG_FLG(Map_FLag_Counter)) THEN
            DO 6005 KLOOP = 1,NCOUNT/2
              Contour_Levels(KLOOP,Map_Flag_Counter) =
     +          Contour_Levels(KLOOP,Map_Flag_Counter)*RHMAX
 6005       CONTINUE
            DO 6007 KLOOP = NCOUNT/2+1,NCOUNT
              Contour_Levels(KLOOP,Map_Flag_Counter) =
     +          Contour_Levels(KLOOP,Map_Flag_Counter)*RHMIN
 6007       CONTINUE
          ELSE
            DO 6009 KLOOP = 1,NCOUNT
             Contour_Levels(KLOOP,Map_Flag_Counter) =
     +         Contour_Levels(KLOOP,Map_Flag_Counter)*RHMAX
 6009       CONTINUE
          ENDIF
        ENDIF
C
        Num_Contours(Map_Flag_Counter) = NCOUNT
        WRITE (6,FMT=6008) Map_Flag_Counter,
     +    (Contour_Levels(KLOOP,Map_Flag_Counter),KLOOP=1,NCOUNT)
 6008   FORMAT (/' Contour levels for map number ',I5,
     +         //3 (7X,12F10.3,/))
C
C
C  May as well set all this here..
      NSECC(1) = NDIM1(1)
      NSECC(2) = NDIM1(1)
      Map_pts_U = NDIM1(3) - NDIM1(2) + 1
      Map_pts_V = NDIM1(5) - NDIM1(4) + 1
C
C
      DO 350 JDO = 1,4
        Map_UVW_Limits(JDO) = NDIM1(JDO+1)
  350 CONTINUE
C
C
      Map_UVW_Limits(5) = NDIM1(1)
      Map_UVW_Limits(6) = NDIM1(1) + NSEC1 - 1
C
C---- If cell dimensions were not given on a CELL card,
C     use cell from map file
C
        IF (XABC(1).LE.0.000001) THEN
C
C
          DO 110 JDO = 1,3
            XABC(JDO) = CELL(JDO)
            XCLANG(JDO) = CELL(JDO+3)
  110     CONTINUE
C
C              ***************
          CALL getcellmatrices
C              ***************
C
        END IF
C
C--- Set up permutations and sampling etc
C
C---- If map dimensions not given on MAP card, use those from file
C
        IF (KEYWRD_UVW(1).LE.0) THEN
C
C
          DO 120 JDO = 1,3
            KEYWRD_UVW(JDO) = MAP_UVW(JDO)
            Map_XYZ_Limits(JDO) = MAP_XYZ(JDO)
  120     CONTINUE
C
C
        END IF
C
C
        WRITE (6,FMT=6010) (LAXIS(KEYWRD_UVW(KLOOP)),
     +    Map_XYZ_Limits(KEYWRD_UVW(KLOOP)),KLOOP=1,3)
 6010   FORMAT (/'  Axis   Sampling interval',/'  Fast    ',A1,8X,I5,
     +         /'  Medium  ',A1,8X,I5,/'  Section ',A1,8X,I5)
C
C
        IF (KEYWRD_UVW(1)+KEYWRD_UVW(2)+KEYWRD_UVW(3).NE.6) THEN
C
C              *************************************
          CALL CCPERR(1,'  Illegal axis order !!!!')
C              *************************************
C
        END IF
C
C---- set up orientation matrix to put u axis along xp, & v axis
C     in plane of xp,yp, or vice versa if 'invert' keyword given
C     (Invert_Map_Flag=-1)
C
C     first,set up inverse permutation matrix pt
C
        DO 140 I = 1,3
          DO 130 J = 1,3
            A(I,J) = 0.0
            Q(I,J) = 0.0
            PT(I,J) = 0.0
  130     CONTINUE
  140   CONTINUE
C
C
        PT(1,MU) = 1.0
        PT(2,MV) = 1.0
        PT(3,MW) = 1.0
C
C
        IF (LSetView_Flag.GT.0) THEN
C
C---- A view matrix has already been given,
C     set map matrix AQ = R. OR. P
C     where R = view matrix,
C     OR = fractional to orthogonal matrix (= FR **-1)
C     P = PT transpose
C
c          B(1,1) = PT(1,1)*OR1 + PT(1,2)*OR2 + PT(1,3)*OR3
c          B(1,2) = PT(2,1)*OR1 + PT(2,2)*OR2 + PT(2,3)*OR3
c          B(1,3) = PT(3,1)*OR1 + PT(3,2)*OR2 + PT(3,3)*OR3
c          B(2,1) = PT(1,1)*OR4 + PT(1,2)*OR5 + PT(1,3)*OR6
c          B(2,2) = PT(2,1)*OR4 + PT(2,2)*OR5 + PT(2,3)*OR6
c          B(2,3) = PT(3,1)*OR4 + PT(3,2)*OR5 + PT(3,3)*OR6
c          B(3,1) = PT(1,1)*OR7 + PT(1,2)*OR8 + PT(1,3)*OR9
c          B(3,2) = PT(2,1)*OR7 + PT(2,2)*OR8 + PT(2,3)*OR9
c          B(3,3) = PT(3,1)*OR7 + PT(3,2)*OR8 + PT(3,3)*OR9
        DO I = 1,3
         Do J = 1,3
          B(I,J) = PT(J,1)*ROR(I,1) +PT(J,2)*ROR(I,2) + PT(J,3)*ROR(I,3)
         END DO
        END DO
C
C              ******************************
          CALL matrixmultply(AQ,ViewMatrix,B)
C              ******************************
C
          LSetView_Flag = 2
          WRITE (6,FMT=6012)
 6012     FORMAT (' Map matrix is calculated from previously input vie',
     +           'w matrix',/)
          GO TO 160
        END IF
C
C---- pt has trace=1 if permutation is non-cyclic, set lperm=-1
C
        LPERM = +1
        T = PT(1,1) + PT(2,2) + PT(3,3)
        IF (ABS(T-1.0).LT.0.2) LPERM = -1
C
C---- b=pt*fr, fr is orthogonal to fractional matrix
C
c        B(1,1) = PT(1,1)*FR1 + PT(1,2)*FR4 + PT(1,3)*FR7
c        B(1,2) = PT(1,1)*FR2 + PT(1,2)*FR5 + PT(1,3)*FR8
c        B(1,3) = PT(1,1)*FR3 + PT(1,2)*FR6 + PT(1,3)*FR9
c        B(2,1) = PT(2,1)*FR1 + PT(2,2)*FR4 + PT(2,3)*FR7
c        B(2,2) = PT(2,1)*FR2 + PT(2,2)*FR5 + PT(2,3)*FR8
c        B(2,3) = PT(2,1)*FR3 + PT(2,2)*FR6 + PT(2,3)*FR9
c        B(3,1) = PT(3,1)*FR1 + PT(3,2)*FR4 + PT(3,3)*FR7
c        B(3,2) = PT(3,1)*FR2 + PT(3,2)*FR5 + PT(3,3)*FR8
c        B(3,3) = PT(3,1)*FR3 + PT(3,2)*FR6 + PT(3,3)*FR9
        DO I = 1,3
         Do J = 1,3
          B(I,J) = PT(I,1)*RFR(1,J) +PT(I,2)*RFR(2,J) + PT(I,3)*RFR(3,J)
         END DO
        END DO
C
C---- set up permutation from uvw to pqr
C      p axis along xp, q along yp, r along zp
C
        MP = MU
        MQ = MV
        MR = MW
C
C
        IF (Invert_Map_Flag.LE.0) THEN
C
C---- invert, v-axis along xp
C
          MP = MV
          MQ = MU
          Q(2,1) = 1.0
          Q(1,2) = 1.0
        ELSE
          Q(1,1) = 1.0
          Q(2,2) = 1.0
        END IF
C
C
        Q(3,3) = 1.0
C
C---- matrix a converts u to xp (in angstroms)
C
        CT1 = COS(XCLANG(MP)/RAD)
        ST2 = SIN(XCLANG(MQ)/RAD)
        CT2 = COS(XCLANG(MQ)/RAD)
        ST3 = SIN(XCLANG(MR)/RAD)
        CT3 = COS(XCLANG(MR)/RAD)
        CT1STR = (CT2*CT3-CT1)/ (ST2*ST3)
        ST1STR = SQRT(1.0-CT1STR*CT1STR)
C
C
        A(1,1) = XABC(MP)
        A(1,2) = XABC(MQ)*CT3
        A(1,3) = XABC(MR)*CT2
        A(2,2) = -XABC(MQ)*ST3
        A(2,3) = +XABC(MR)*ST2*CT1STR
        A(3,3) = -XABC(MR)*ST2*ST1STR
C
C---- keep matrix aq=a*q for contouring,  xp=aq.u
C
C            *********************
        CALL matrixmultply(AQ,A,Q)
C            *********************
C
        IF (Invert_Map_Flag*LPERM.LE.0) THEN
C
C---- Direction of z determined by Invert_Map_Flag
C     (keyword 'INVERT')
C     and hand of permutation P. Invert z
C
          DO 150 JDO = 1,3
            AQ(3,JDO) = -AQ(3,JDO)
  150     CONTINUE
        END IF
C
C---- orientation matrix r=a*q*pt*fr
C
C            ******************************
        CALL matrixmultply(ViewMatrix,AQ,B)
C            ******************************
C
        LSetView_Flag = 1
C
C
  160   WRITE (6,FMT=6014) ((AQ(I,J),J=1,3),I=1,3),
     +    ((ViewMatrix(I,J),J=1,3),I=1,3)
 6014   FORMAT (/' Matrix to convert uvw to plotter axes:',
     +         /3 (10X,3F12.5,/),/' Orientation matrix for atoms :',
     +         /3 (10X,3F12.5,/))
C
C
        GO TO 40
C
C---- 2nd map , check same as first
C
C            ***********************************************************
  170   CALL openmaps(Map_Flag_Counter,NDIM2,
     +                Map_Title(Map_Flag_Counter),NSEC2,MAP_UVW,MAP_XYZ,
     +                CELL,RHMAX,RHMEAN,RHMIN,RHRMS,Patt_Flag2)
C            ***********************************************************
C
        IF(Patt_Flag.NE.Patt_Flag2) call ccperr
     +       (2, ' One map is a Patterson, the other is not')
C
        IF (RMS_FLG(Map_Flag_Counter)) THEN
          DO 6105 KLOOP = 1,NCOUNT
           Contour_Levels(KLOOP,Map_Flag_Counter) =
     +        Contour_Levels(KLOOP,Map_Flag_Counter)*RHRMS
 6105     CONTINUE
        ELSE IF (MAX_FLG(Map_Flag_Counter)) THEN
          IF (NEG_FLG(Map_FLag_Counter)) THEN
            DO 6106 KLOOP = 1,NCOUNT/2
              Contour_Levels(KLOOP,Map_Flag_Counter) =
     +          Contour_Levels(KLOOP,Map_Flag_Counter)*RHMAX
 6106       CONTINUE
            DO 6107 KLOOP = NCOUNT/2+1,NCOUNT
              Contour_Levels(KLOOP,Map_Flag_Counter) =
     +          Contour_Levels(KLOOP,Map_Flag_Counter)*RHMIN
 6107       CONTINUE
          ELSE
            DO 6109 KLOOP = 1,NCOUNT
             Contour_Levels(KLOOP,Map_Flag_Counter) =
     +         Contour_Levels(KLOOP,Map_Flag_Counter)*RHMAX
 6109       CONTINUE
          ENDIF
        ENDIF
C
        Num_Contours(Map_Flag_Counter) = NCOUNT
        WRITE (6,FMT=6008) Map_Flag_Counter,
     +    (Contour_Levels(KLOOP,Map_Flag_Counter),KLOOP=1,NCOUNT)
C
C
        DO 180 JDO = 2,5
          IF (NDIM1(JDO).NE.NDIM2(JDO)) THEN
            WRITE (6,FMT=6010) (LAXIS(MAP_UVW(KLOOP)),
     +        MAP_XYZ(MAP_UVW(KLOOP)),KLOOP=1,3)
C
C                *****************************************************
            CALL CCPERR(1,
     +           ' Map axes/sampling 2nd map does not match 1st map ')
C                *****************************************************
C
          END IF
  180   CONTINUE
C
C
        DO 190 JDO = 1,3
          IF (KEYWRD_UVW(JDO).NE.MAP_UVW(JDO)) THEN
            WRITE (6,FMT=6010) (LAXIS(MAP_UVW(KLOOP)),
     +        MAP_XYZ(MAP_UVW(KLOOP)),KLOOP=1,3)
C
C                *****************************************************
            CALL CCPERR(1,
     +    ' Map axes or sampling in 2nd map does not match 1st map ')
C                *****************************************************
C
          END IF
C
C
          IF (Map_XYZ_Limits(JDO).NE.MAP_XYZ(JDO)) THEN
            WRITE (6,FMT=6010) (LAXIS(MAP_UVW(KLOOP)),
     +        MAP_XYZ(MAP_UVW(KLOOP)),KLOOP=1,3)
C
C                *****************************************************
            CALL CCPERR(1,
     +    ' Map axes or sampling in 2nd map does not match 1st map ')
C                *****************************************************
C
          END IF
  190   CONTINUE
C
C
        GO TO 40
C
C
C     =====================
C---- MODE CARD   keywords:
C     =====================
C
C
C   1) colour and/or dashed (optional, default black)
C   2) 'below' Contour_Threshold, threshold for contour levels
C   3) colour and/or dashed, mode for levels below Contour_Threshold
C   4) FIRST to set parameters for 1st section in a stack
C      DASHED keyword followed by 3 numbers, repeat,dash,dot
C
C
C
C     Mode for plotting contours.  If two maps are  plotted,  each
C     may  have  a  MODE card, which must follow its corresponding
C     CONTRS card.  The mode is BLACK, RED, GREEN,  BLUE,  YELLOW,
C     ORANGE,  PURPLE,  or  DASHED  :   DASHED  may  appear with a
C     colour.  If the keyword BELOW is present, a second  mode  is
C     given  for contour levels below the specified threshold.  If
C     the keyword FIRST is present, this  mode  is  used  for  the
C     first section in each map stack.  This may be useful to plot
C     sections in pairs, (SECTNS n1 n2 2 1) with one dashed or  in
C     a different colour.  If no mode is given, the default colour
C     is used (usually black, unless a COLOUR  card  is  present).
C     The  keyword  DASHED is followed by 3 numbers, repeat, dash,
C     dot.
C
C               repeat repeat length in mm
C               dash   dash length in mm (gap=repeat-dash)
C               dot    length  of  dot  for   chain   lines   (not
C                        available in Cambridge)
C
C     eg   MODE RED BELOW 0 RED DASHED 4 2 0
C          All red, negative contours dashed.
C
C          MODE BELOW 0 GREEN
C          Positive contours black
C
C          MODE DASHED 1 5 3.5 0
C          All dashed
C
C default mode unset=0
C
C
C     #### Sub_Key_Words of "MODE" ####
C     ==================================
C
C
      ELSE IF (KEYX.EQ.'MODE') THEN
        MAPCOLOURS_WRK = 0
        MAPMODE_WRK = 1
        MAPDASH_WRK = 0
        MAPFLAG_WRK = Map_Flag_Counter
        IF (MAPFLAG_WRK .EQ. 0)
C
C              ***********************************************
     +    CALL CCPERR(1,' CONTRS keyword must be before MODE')
C              ***********************************************
C
        IWORK = 1
  200   IWORK = IWORK + 1
        IF (IWORK.GT.NTOKX) GO TO 210
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
C          MODE BELOW 0 GREEN
C
        IF (CWORK.EQ.'BELO') THEN
C
C---- keyword below, store mode for higher levels & threshold
C     eg   mode red below 0 red dashed 4 2 0
C
          IWORK = IWORK + 1
          KWWANT = 1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Contour_Threshold(MAPFLAG_WRK) = FVALUEX(IWORK)
          MAPMODE_WRK = 2
          GO TO 200
C
C          mode dashed 1 5 3.5 0
C
        ELSE IF (CWORK.EQ.'DASH') THEN
C
C---- keyword dashed, store dash parameters, set mode negative
C
          MAPDASH_WRK = -2
          IWORK = IWORK + 1
C
C
          IF (IWORK+2.LE.NTOKX) THEN
            MAPDASH_WRK = -3
            KWWANT = 1
            KWSTART = IWORK + 2
C
C                ****************************************************
            CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                ****************************************************
C
            Map_LineTypes(4,MAPMODE_WRK,MAPFLAG_WRK)
     +        = NINT(FVALUEX(IWORK+2)*10.0)
          END IF
C
C
          KWWANT = 2
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          I = NINT(FVALUEX(IWORK)*10.0)
          J = NINT(FVALUEX(IWORK+1)*10.0)
          IF (I.LE.0) I = 30
          IF (J.LE.0) J = 15
          Map_LineTypes(2,MAPMODE_WRK,MAPFLAG_WRK) = I
          Map_LineTypes(3,MAPMODE_WRK,MAPFLAG_WRK) = J
          IWORK = IWORK + 3
          IF (MAPDASH_WRK.EQ.-3) IWORK = IWORK - 1
          GO TO 200
C
C----  keyword FIRST
C
        ELSE IF (CWORK.EQ.'FIRS') THEN
C
C                     
          IF (Map_Flag_Counter.GT.1) 
C
C            **************************************************
     +  CALL CCPERR(1,'  MODE FIRST not allowed with two maps')
C            **************************************************
C
C----  Store parameters in 2nd map slot, set flag
C
          MAPFLAG_WRK = 2
          Lmap_ModeFirst = 1
          GO TO 200
C
C---- keywords black,red,green,blue,yellow,orange,purple
C
        ELSE IF (CWORK.EQ.'BLAC') THEN
          MAPCOLOURS_WRK = 1
        ELSE IF (CWORK.EQ.'CYAN') THEN
          MAPCOLOURS_WRK = 2
        ELSE IF (CWORK(1:3).EQ.'RED') THEN
          MAPCOLOURS_WRK = 3
        ELSE IF (CWORK.EQ.'GREE') THEN
          MAPCOLOURS_WRK = 4
        ELSE IF (CWORK.EQ.'BLUE') THEN
          MAPCOLOURS_WRK = 5
        ELSE IF (CWORK.EQ.'MAGE') THEN
          MAPCOLOURS_WRK = 6
        ELSE IF (CWORK.EQ.'YELL') THEN
          MAPCOLOURS_WRK = 7
        ELSE IF (CWORK.EQ.'PURP') THEN
          MAPCOLOURS_WRK = 8
        ELSE IF (CWORK.EQ.'ORAN') THEN
          MAPCOLOURS_WRK = 9
        ELSE IF (CWORK.EQ.'BLAC') THEN
          MAPCOLOURS_WRK = 10
        ELSE
          WRITE (6,FMT=6016) CWORK
 6016     FORMAT ('  Error: SubKeyWord for MODE (Map) KeyWord',/'     ',
     +           '     NOT Understood: < ',A,' >')
          GO TO 200
        END IF
        GO TO 200
  210   CONTINUE
C
C---- MODE
C
        IF (MAPDASH_WRK.NE.0) MAPCOLOURS_WRK = MAPDASH_WRK -
     +      MAPCOLOURS_WRK*4
        Map_LineTypes(1,MAPMODE_WRK,MAPFLAG_WRK) = MAPCOLOURS_WRK
        MAPMODE_WRK = 2
        MAPCOLOURS_WRK = 0
        MAPDASH_WRK = 0
        GO TO 40
C
C     ============
C---- SECTNS card
C     ============
C
C
C  read 1st & last section number NFirst_Section, NLast_Section
C  for start of group
C number of sections in group nsecgr, (default=1),
C and step between groups NstpMapGrpSect
C       (default=nsecgr)
C and step between sections
C
C
C   SECTNS   (FRAC) NFirst_Section, NLast_Section, Nsections_in_Group,
C                          NstpMapGrpSect
C
C          Select sections for map plotting.
C
C  NFirst_Section,NLast_Section First and last sections.
C                 If section stacks  are being  plotted,
C      NLast_Section is the first section in the last stack.
C                 If NLast_Section  is  omitted,  only
C                    section NFirst_Section is plotted.
C      Nsections_in_Group Number of  sections  to  be  plotted
C                         in  each  stack
C                     (default=1).   If  .gt.1,  the highest number
C                     section to be plotted is
C                    NLast_Section + Nsections_in_Group - 1.
C      NstpMapGrpSect
C            Step in section number between succesive  map  stacks
C                     (default=Nsections_in_Group).
C                      This  can be used to plot
C                     overlapping map stacks.
C
C      eg   SECTNS 3 14 plot sections 3 to 14 separately
C
C           SECTNS 3 11 4 plot 3 stacks of 4 sections :
C           3,4,5,6 ; 7,8,9,10 ; 11,12,13,14
C
C           SECTNS 3 11 4 2
C           plot 6 overlapping stacks of 4 sections
C           3,4,5,6 ; 5,6,7,8 ; 7,8,9,10 ; 9,10,11,12 ; etc
C
C
C     #### Sub_Key_Words of "SECTIONS" ####
C     =====================================
C
C
      ELSE IF (KEYX.EQ.'SECT') THEN

        FRACSEC = .FALSE.
        IWORK = 2
C   Check if next word is 'FRAC' 
        IF(ITYPX(2).EQ.1) THEN
         IF(CVALUEX(2)(1:1).EQ.'F' .or. CVALUEX(2)(1:1).EQ.'f') 
     +      FRACSEC =.TRUE.
        IWORK = 3
        END IF
C
C
        IF (IWORK.GT.NTOKX) 
C
C             ********************************************************
     +   CALL CCPERR(1,' Error for Key_Word SECTIONS No tokens given')
C             ********************************************************
C
C---- expect 1,2,3 or 4 numbers
C
        KWWANT = 1
        KWSTART = IWORK
C
C            ****************************************************
        CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C            ****************************************************
        First_Section = FVALUEX(IWORK)
C
C
        IF (NTOKX.LE.IWORK) THEN
          Last_Section = First_Section
        ELSE
          KWWANT = 1
          IWORK = IWORK+1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
        Last_Section = FVALUEX(IWORK)
        END IF
C
C
        IF (NTOKX.GT.IWORK) THEN
          KWWANT = 1
          IWORK = IWORK+1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          sections_in_Group = FVALUEX(IWORK)
        END IF
C
C
        stpMapGrpSect = sections_in_Group
C
C
        IF (NTOKX.GT.IWORK) THEN
          KWWANT = 1
          IWORK = IWORK+1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
C
        StpMapGrpSect = FVALUEX(IWORK)
        END IF
C
C
C
        write(6,*)' First_Section,Last_Section,Sections_in_Group',
     + First_Section,Last_Section,Sections_in_Group
        GO TO 40
C
C     ===========
C---- SLAB CARD
C     ===========
C
C
C  set sections limits for selection of atoms .
C  atoms will be stored on file
C  if they lie between the sections specified + 1
C  section thickness at each end
C
C
C SLAB    section1, section2
C
C     Only useful for large plots.  If  present,  this  card  must
C     follow  the  SECTNS  card.  Normally atoms are stored in the
C     scratch file if they lie between the  sections
C     NFirst_Section-1  and
C     NLast_Section+Nsections_in_Group.
C     However, in some cases plotting a map has to
C     be split into several jobs, and it may be better  to  select
C     atoms  for  the  whole map, and to save the scratch file for
C     the subsequent jobs (see  the  SAVED  option  on  the  INPUT
C     card).    In  this  case,  SLAB  gives  section  limits  for
C     selection of atoms for the scratch file:   atom  limits  are
C     section1-1  to  section2+1,  where section1 and section2 are
C     the first and last sections of a series of plots.
C
C     eg    SLAB 0 20
C
C
C     #### Sub_Key_Words of "SLAB" ####
C     ==================================
C
C
      ELSE IF (KEYX.EQ.'SLAB') THEN
C
C                       *******************************************
        IF (NTOKX.LT.3) CALL CCPERR(1,' Error on SLAB control card')
C                       *******************************************
C
        KWWANT = 2
        KWSTART = 2
C
C            ****************************************************
        CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C            ****************************************************
C
        Bound_Limits(MW) = FVALUEX(2) - 1.0
        Bound_Edge(MW) = FVALUEX(3) + 1.0
        LSLAB = 1
        GO TO 40
C
C     ===========
C---- GRID CARD
C     ===========
C
C
C
C   read grid spacing along u & v in angstrom
C    (default Ugrid_Spacing=10,Vgrid_Spacing=Ugrid_Spacing)
C  & grid offset from map origin along u & v in angstrom (default=0)
C
C  GRID    Ugrid_Spacing,Vgrid_Spacing,Ugrid_Offset,Vgrid_Offset
C
C    Draw grid on map, at spacings of
C     Ugrid_Spacing Angstrom along u  and
C     Vgrid_Spacing  along v
C
C          (default Ugrid_Spacing = 10 A,
C                   Vgrid_Spacing = Ugrid_Spacing).
C
C    Normally, the grid will pass through the map origin,
C    but  it  may  be offset   by
C        Ugrid_Offset  Angstrom  along  u  and
C        Vgrid_Offset  along  v
C          (default=0),
C
C    ie the grid passes  through  the  point
C     (Ugrid_Offset, Vgrid_Offset).
C    On map stacks, the grid is plotted only on the first
C    section (with the lowest section number).
C
C    eg   GRID 15
C    Draw grid at 15 A spacing in both directions
C
C         GRID
C    Draw grid at 10 A spacing.
C
C  GRID U or V style for minor lines EVERY n style for major lines
C
C    Set style for plotting grid lines  (default  full  lines  in
C    current  colour).   If the first keyword is U or V, the card
C    refers to grid lines across the u or v axes:  otherwise  the
C    style  refers to both directions.  Two classes of grid lines
C    may be defined, major and minor.  The  keyword  EVERY  gives
C    the  spacing  of major grid lines, every n'th grid line from
C    the origin (default=1, ie all grid lines major).  The  style
C    specification  may  be  a colour keyword (BLACK, RED, GREEN,
C    BLUE,  YELLOW,  ORANGE,  PURPLE)  and/or  DASHED   or   FULL
C    (default).  DASHED is followed by 3 numbers (see MODE card).
C    If the colour is not set, the current default colour is used
C    (usually  black).   Up  to three GRID cards may be given, to
C    set the spacing and the style along u and v.  Any  one  GRID
C    card switches on grid plotting, so a style card may be given
C    without a spacing card if the  default  spacing  of  10A  is
C    wanted.
C
C    eg   GRID RED DASHED 10 5 0 EVERY 5 FULL
C
C         Grid at default spacing 10A, full line every 50A.
C
C         GRID 10 25
C         GRID U RED EVERY 5 BLACK
C         GRID V RED EVERY 2 BLACK
C
C         GRID RED
C
C         Grid at 10A spacing in red.
C
C
C     #### Sub_Key_Words of "GRID" ####
C     ==================================
C
      ELSE IF (KEYX.EQ.'GRID') THEN
        LGRID = 1
C
C optional keywords in following order
C 1) u or v for lines across u or v axis
C 2) style for minor grid lines
C    (black,red,green,blue,yellow,dashed,full)
C 3) 'every' Major_Grid() interval for major grid lines
C 4) style for major grid lines
C
C---- style for grid along which axis (or both)
C
C   itypx(I)      =0  null field
C                 =1  character string
C                 =2  number
C
        J1 = 1
        J2 = 2
C
C
        IF (NTOKX.LE.1 .OR. ITYPX(2).EQ.2) THEN
C
C---- any keywords ? if none read grid spacing parameters
C
          Ugrid_Spacing = 0.1
          Ugrid_Offset = 0.0
          Vgrid_Offset = 0.0
          Vgrid_Spacing = Ugrid_Spacing
C
C
          IF (NTOKX.GT.1) THEN
            IF (NTOKX.GE.2) Ugrid_Spacing = FVALUEX(2)
            IF (NTOKX.GE.3) Vgrid_Spacing = FVALUEX(3)
            IF (NTOKX.GE.4) Ugrid_Offset = FVALUEX(4)
            IF (NTOKX.GE.5) Vgrid_Offset = FVALUEX(5)
          END IF
          GO TO 40
        END IF
C
        MAPCOLOURS_WRK = 0
        MAPDASH_WRK = 0
        M = 1
C           
        IWORK = 1
  220   IWORK = IWORK + 1
        IF (IWORK.GT.NTOKX) GO TO 260
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
C         GRID 10 25
C         GRID U RED EVERY 5 BLACK
C         GRID V RED EVERY 2 BLACK
C         GRID RED
C
        IF (CWORK(1:1).EQ.'U') THEN
          J2 = 1
        ELSE IF (CWORK(1:1).EQ.'V') THEN
          J1 = 2
C
C---- process style keywords
C
        ELSE IF (CWORK.EQ.'DASH') THEN
C
C---- keyword dashed
C    eg   GRID RED DASHED 10 5 0 EVERY 5 FULL
C
          MAPDASH_WRK = -2
C
C
          JWORK = IWORK
          DO 240 J = J1,J2
            IWORK = JWORK
            DO 230 I = 1,3
              IWORK = IWORK + 1
              KWWANT = 1
              KWSTART = IWORK
C
C                  ****************************************************
              CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                  ****************************************************
C
              MapGrdCol(I+1,M,J) = NINT(FVALUEX(IWORK)*10.0)
  230       CONTINUE
C
C
            IF (MapGrdCol(2,M,J).LE.0) MapGrdCol(2,M,J) = 30
            IF (MapGrdCol(3,M,J).LE.0) MapGrdCol(3,M,J) = 15
            IF (MapGrdCol(4,M,J).GT.0) MAPDASH_WRK = -3
  240     CONTINUE
C
        ELSE IF (CWORK.EQ.'EVER') THEN
C
C---- keyword every, store style for minor lines
C
           IF (MAPDASH_WRK.NE.0) MAPCOLOURS_WRK = MAPDASH_WRK -
     +          MAPCOLOURS_WRK*4
           DO 249, J = J1,J2
              MapGrdCol(1,1,J) = MAPCOLOURS_WRK
 249       CONTINUE 
           M = 2
           MAPCOLOURS_WRK = 0
C
C
           IWORK = IWORK + 1
           KWWANT = 1
           KWSTART = IWORK
C
C                ****************************************************
           CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                ****************************************************
C
           DO 250 J = J1,J2
              Major_Grid(J) = NINT(FVALUEX(IWORK))
 250       CONTINUE
C
        ELSE IF (CWORK.EQ.'FULL') THEN
          MAPDASH_WRK = 0
        ELSE IF (CWORK.EQ.'BLAC') THEN
          MAPCOLOURS_WRK = 1
        ELSE IF (CWORK.EQ.'CYAN') THEN
          MAPCOLOURS_WRK = 2
        ELSE IF (CWORK(1:3).EQ.'RED') THEN
          MAPCOLOURS_WRK = 3
        ELSE IF (CWORK.EQ.'GREE') THEN
          MAPCOLOURS_WRK = 4
        ELSE IF (CWORK.EQ.'BLUE') THEN
          MAPCOLOURS_WRK = 5
        ELSE IF (CWORK.EQ.'MAGE') THEN
          MAPCOLOURS_WRK = 6
        ELSE IF (CWORK.EQ.'YELL') THEN
          MAPCOLOURS_WRK = 7
        ELSE IF (CWORK.EQ.'PURP') THEN
          MAPCOLOURS_WRK = 8
        ELSE IF (CWORK.EQ.'ORAN') THEN
          MAPCOLOURS_WRK = 9
        ELSE IF (CWORK.EQ.'BLAC') THEN
          MAPCOLOURS_WRK = 10
        ELSE
          WRITE (6,FMT=6018) CWORK
 6018     FORMAT (' Error Sub_KeyWord for GRID Key_Word',/' NOT Unders',
     +           'tood - < ',A,' >')
        END IF
        GO TO 220
C
C
  260   IF (MAPDASH_WRK.NE.0) MAPCOLOURS_WRK = MAPDASH_WRK -
     +      MAPCOLOURS_WRK*4
C
C
        DO 270 JDO = J1,J2
          MapGrdCol(1,M,JDO) = MAPCOLOURS_WRK
  270   CONTINUE
C
C
        MAPCOLOURS_WRK = 0
        MAPDASH_WRK = 0
C
C---- if no minor lines, copy minor style to major
C
        IF (M.EQ.2) GO TO 40
C
C
        DO 290 J = J1,J2
          DO 280 I = 1,4
            MapGrdCol(I,2,J) = MapGrdCol(I,1,J)
  280     CONTINUE
  290   CONTINUE
C
C
        GO TO 40
C
C     ========================================================
C---- LIMITS card, xmin,xmax,ymin,ymax,zmin,zmax in grid units
C     ========================================================
C
C
C  LIMITS    xmin, xmax, ymin, ymax, zmin, zmax
C
C     Limit the part of the map to be plotted to the given volume,
C     in  grid  points.   A  SECTNS  card must also be given:  the
C     limits given here on the section axis are not used.
C
C
C     #### Sub_Key_Words of "LIMITS" ####
C     ====================================
C
C
      ELSE IF (KEYX.EQ.'LIMI') THEN
C
C                       *****************************************
        IF (NTOKX.LT.7) CALL CCPERR(1,' Error on LIMITS card')
C                       *****************************************
C
        DO 300 JDO = 1,6
          KWWANT = 1
          KWSTART = JDO + 1
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          LIMTMP(JDO) = NINT(FVALUEX(JDO+1))
  300   CONTINUE
C
C
        LimitsSet_Flag = 1
        GO TO 40
C
C
      ELSE
C
C     ===============================================
C---- card not recognized, finished map control cards
C     ===============================================
C
        GO TO 310
      END IF
C
C---- control cards finished
C
C
  310 IF (Map_Flag_Counter.LE.0) 
C
C             *********************************************
     +   CALL CCPERR(1,' ERROR NO CONTOURS Key_Word given')
C             *********************************************
C
        IF( FRACSEC ) THEN
           NFirst_Section = NINT(First_Section*MAP_XYZ_limits(MW))
           NLast_Section =  NINT(Last_Section*MAP_XYZ_limits(MW))
        END IF
C
        IF( .NOT. FRACSEC ) THEN
           NFirst_Section = NINT(First_Section)
           First_Section = First_Section/MAP_XYZ_limits(MW)
           NLast_Section = NINT(Last_Section)
           Last_Section =  Last_Section/MAP_XYZ_limits(MW)
        END IF
C
        MAP_SECfrac_first = float(MAP_UVW_limits(5))/MAP_XYZ_limits(MW)
        MAP_SECfrac_last  = float(MAP_UVW_limits(6))/MAP_XYZ_limits(MW)
        MAP_Secfrac_range = MAP_SECfrac_last - MAP_SECfrac_first
C
        write(6,'(A,/,3I5,/,6I4/,3F10.5)')
     + ' MAP_XYZ_limits,MAP_UVW_limits Map range',
     +              MAP_XYZ_limits,MAP_UVW_limits,
     +   MAP_SECfrac_first,MAP_SECfrac_last,MAP_SECfrac_range
C
C  Check section range:
C If NFirst_Section lt MAP_UVW_limits(5) - reset it
C If NLast_Section  gt MAP_UVW_limits(6) - reset it
C  Check if requested sections are outside range; reset..
        IF(NFirst_Section .GT.MAP_UVW_limits(6) .OR.
     +     NLast_Section .LT.MAP_UVW_limits(5))  THEN
           CALL CCPERR(2,' All Requested sections outside map limits')
           CALL CCPERR(0,' Finishing here - nothing to do')
        END IF
C
        IF(NFirst_Section .LT.MAP_UVW_limits(5)) THEN
           First_Section = MAP_UVW_limits(5)/MAP_XYZ_limits(MW)
           NFirst_Section = MAP_UVW_limits(5)
          CALL CCPERR(2,
     +   ' First Requested sections below map limit - resetting it')
        END IF

        IF(NLast_Section .GT.MAP_UVW_limits(6)) THEN
           Last_Section = MAP_UVW_limits(6)/MAP_XYZ_limits(MW)
           NLast_Section = MAP_UVW_limits(6)
           CALL CCPERR(2,
     +   ' Last Requested sections above map limit - resetting it')
       END IF

C
  312   CONTINUE

        IF(Sections_in_Group .LT.1) 
     +  Nsections_in_Group =   NINT(Sections_in_Group*MAP_XYZ(MW))
         IF(Sections_in_Group.GE.1) 
     +     Nsections_in_Group = NINT(Sections_in_Group)
C
C
        NstpMapGrpSect = Nsections_in_Group
C
        IF(StpMapGrpSect .LT.1)
     +      NstpMapGrpSect =  NINT(StpMapGrpSect*MAP_XYZ(MW))
        IF(StpMapGrpSect .GE.1) NstpMapGrpSect = NINT(StpMapGrpSect)
C
C
        IF (LSLAB.EQ.0) THEN
C
C---- set limits for atoms in scratch file, 1/2 section thickness on
C     either side of slab + 2 angstrom
C
c        IF (MW.EQ.1) Q3 = OR1
c        IF (MW.EQ.2) Q3 = OR5
c        IF (MW.EQ.3) Q3 = OR9
C   Take care! The interval is DelX = A/NX * unit(A*) = Cell(1)/NX * sin Gamma sin Beta*
C                           or DelY = B/NY * unit(B*) = Cell(2)/NY * sin Alpha sin Gamma*
C                           or DelZ = C/NZ * unit(C*) = Cell(3)/NZ * sin Beta  sin Alpha*
C   Q3 = 2/DelX or 2/DelY or 2/DelZ
C        IF (MW.EQ.1) Q3 = ROR(1,1)
C        IF (MW.EQ.2) Q3 = ROR(2,2)
C        IF (MW.EQ.3) Q3 = ROR(3,3)
      CONV = 3.14159/180.0
      ALPH = CELL(4)*CONV
      BET  = CELL(5)*CONV
      GAMM = CELL(6)*CONV
      SINA = SIN(ALPH)
      COSA = COS(ALPH)
      SINB = SIN(BET)
      COSB = COS(BET)
      SING = SIN(GAMM)
      COSG = COS(GAMM)
      COSAS = (COSG*COSB-COSA)/ (SINB*SING)
      SINAS = SQRT(1.0-COSAS*COSAS)
      COSBS = (COSA*COSG-COSB)/ (SINA*SING)
      SINBS = SQRT(1.0-COSBS*COSBS)
      COSGS = (COSA*COSB-COSG)/ (SINA*SINB)
      SINGS = SQRT(1.0-COSGS*COSGS)
         IF (MW.EQ.1) Q3 = CELL(1) *SING*SINBS
         IF (MW.EQ.2) Q3 = CELL(2) *SINA*SINGS
         IF (MW.EQ.3) Q3 = CELL(3) *SINB*SINAS
         Q3 = (2.0/Q3)*Map_XYZ_Limits(MW)
        Bound_Limits(MW) = NFirst_Section - 0.5 - Q3
        Bound_Edge(MW) = NLast_Section + Nsections_in_Group - 0.5 + Q3
        END IF 
C
C Assume if grid spacing is .lt. 1.0 it is fractional:
C     offset is then also fractional
      IF (Ugrid_Spacing .LT. 1.0) THEN
         Ugrid_Spacing = Ugrid_Spacing * XABC(MU)
         Ugrid_Offset  = Ugrid_Offset  * XABC(MU)
      ENDIF
      IF (Vgrid_Spacing .LT. 1.0) THEN
         Vgrid_Spacing = Vgrid_Spacing * XABC(MV)
         Vgrid_Offset  = Vgrid_Offset  * XABC(MV)
      ENDIF
C---- convert to map grid units
C
      U = REAL(Map_XYZ_Limits(MU))/XABC(MU)
      Ugrid_Spacing = Ugrid_Spacing*U
      Ugrid_Offset = Ugrid_Offset*U
      V = REAL(Map_XYZ_Limits(MV))/XABC(MV)
      Vgrid_Spacing = Vgrid_Spacing*V
      Vgrid_Offset = Vgrid_Offset*V
C
C--- If no SYMM cards present, take symmetry from 1st map file
C
      IF (NSymmetry_Ops.EQ.0) THEN
C
C            ****************************
        CALL MSYMOP(3,NSymmetry_Ops,RTSYM)
C            ****************************
C
        DO 340 K = 1,NSymmetry_Ops
          DO 330 I = 1,3
            TransSym_Vect(I,K) = RTSYM(I,4,K)
            DO 320 J = 1,3
              Rotsym_Matrx(I,J,K) = RTSYM(I,J,K)
  320       CONTINUE
  330     CONTINUE
  340   CONTINUE
      END IF
C
C---- Check for running off beginning or end of map
C
       write(6,*)
     + ' line 5547 NDIM1(1),NFirst_Section,NLast_Section,NSEC1',
     + NDIM1(1),NFirst_Section,NLast_Section,NSEC1

      IF (NFirst_Section.LT.NDIM1(1)) NFirst_Section = NDIM1(1)
      IF (NLast_Section.LT.NFirst_Section)
     +    NLast_Section = NFirst_Section
      IF (NLast_Section.GT.(NDIM1(1) + NSEC1 - 1)) 
     +    NLast_Section =   NDIM1(1) + NSEC1 - 1
C
       write(6,*)
     + ' line 5550 NDIM1(1),NFirst_Section,NLast_Section,NSEC1',
     + NDIM1(1),NFirst_Section,NLast_Section,NSEC1
C---- store 1st section number as current section, and dimensions
C
C---- Npicture  is number of pictures
C
      if (NstpMapGrpSect .eq. 0)
     +  call CCPERR(1, 'Sections to be plotted not set')
C
      Npicture = (NLast_Section-NFirst_Section)/NstpMapGrpSect + 1
C
C---- Set up default restricted limits
C
      DO 360 JDO = 1,4
        Map_def_Restrictions(JDO) = Map_UVW_Limits(JDO)
  360 CONTINUE
C
C---- If LIMITS are set, permute to uv(w)
C
      IF (LimitsSet_Flag.EQ.0) GO TO 370
      Map_def_Restrictions(1) = LIMTMP(2*MU-1)
      Map_def_Restrictions(2) = LIMTMP(2*MU)
      Map_def_Restrictions(3) = LIMTMP(2*MV-1)
      Map_def_Restrictions(4) = LIMTMP(2*MV)
C
C---- Number of points in compressed section
C
  370 Npts_U_compressed = Map_def_Restrictions(2) -
     +                    Map_def_Restrictions(1) + 1
      Npts_V_compressed = Map_def_Restrictions(4) -
     +                    Map_def_Restrictions(3) + 1
C
C
      IF (LimitsSet_Flag.NE.0) WRITE (6,FMT=6020) Map_def_Restrictions
 6020 FORMAT (/11X,'Limits for plotting , on u and v :',4I7,/)
C
C---- Check valid limits
C
      DO 380 JDO = 1,3,2
        IF (Map_def_Restrictions(JDO).LT.Map_UVW_Limits(JDO) .OR.
     +      Map_def_Restrictions(JDO+1).GT.Map_UVW_Limits(JDO+1)) THEN
C
C              ******************************************************
          CALL CCPERR(1,'!!!! Plotting limits outside map area !!!!')
C              ******************************************************
C
        END IF
  380 CONTINUE
C
C---- OK
C
      GO TO 390
C
C---- set bounds for s/r KWPROTIN to map limits
C      (fractional coordinates)
C      limits on w already set
C
  390 Bound_Limits(MU) = Map_def_Restrictions(1)
      Bound_Edge(MU) = Map_def_Restrictions(2)
      Bound_Limits(MV) = Map_def_Restrictions(3)
      Bound_Edge(MV) = Map_def_Restrictions(4)
C
C--- to fractional
C *** PRE July 1990, inflate bounds very slightly
C                    to include atoms on
C                    edge, by EDGE Angstrom
C
      EDGE = 0.05
C
C
      DO 400 JDO = 1,3
        S = Map_XYZ_Limits(JDO)
        Bound_Limits(JDO) = (Bound_Limits(JDO)/S) - EDGE/XABC(JDO)
        Bound_Edge(JDO) = (Bound_Edge(JDO)/S) + EDGE/XABC(JDO)
  400 CONTINUE
C
      RETURN
      END
C
C
C
C     =============================================================
      SUBROUTINE akwprotin(LINEX,IBEGX,IENDX,ITYPX,FVALUEX,CVALUEX,
     +                   IDECX,NTOKX,LOGPARSER,CELL_GIVEN,NUM_FILES)
C     ==============================================================
C
C
C
C---- Sets in COMMON the following:
C
C   NAtoms_Input                      LSuperimpose_Flag
C
C   Orthogonal_XYZ(Max_Atoms,3)       Fractional_XYZ(Max_Atoms,3)
C
C   IAtom_Type(Max_Atoms)       = MTYPE
C                           where MTYPE is
C                     MTYPE = -ABS (ICHAR(Atom_Name(1:1))**2 +
C                                         ICHAR(Atom_Name(2:2)**3)
C
C    LAtom_Inc_Flag(Max_Atoms)   =
C                      ISideChain_Code + 2 + ISYM*10 + ICOLR*1000 +
C                                              IRes_Number*10000
C
C    LAtom_Label_Flag(Max_Atoms) = LLAB
C                           LLAB = 0 nolabel
C                                = LabPackedFlags(i) TEXT label
C                                  >> Num_LabTxt_Input*100
C                                = 1 LABEL ALL
C
C   Atm_Lab_Txt(Num_LabTxt_Input) = LABEL TEXT string
C
C   Atm_Lab_Type(Nlab_types) = LABEL FIX
C
C   Atm_Identifier(Max_Atoms) =
C              For OW and C-alpha
C                 Residue_Name(1:Len_Residue_Name) //
C                   Residue_Number(Kstart_ResNum:4) //
C                     Chain_Name
C              For others
C                 Atom_Name(1:Len_Atom_Name) //
C                    Residue_Number(Kstart_ResNum:4) //
C                      Chain_Name
C
C   Link_Atm_Names(2,2)
C
C   IResid_BonTyp(Max_Atoms) =
C                            =  -1  default
C                            =  KMain_Bond*1000 + KSide_Bond
C
C   ISym_BndType(Max_Symmetry) =
C                                 = -1 default
C                                 =  IPack_Bond_Types
C             where  IPack_Bond_Types = IMain_Bond*1000 + ISide_Bond
C
C   IConn_BndType(Max_BondType) =
C                    = 0 (default)
C                    = -(50000*JDOT + 500*JDASH + 5*JREPET + JMODE)
C
C   Num_Posit_Lables     Num_LabTxt_Input
C
C   Positioned_Labels(1,Num_Posit_Lables) , From LABEL TEXT
C   Positioned_Labels(2,Num_Posit_Lables)
C
C   Bound_Limits(3), Bound_Edge(3)  from BOUNDS, BOX or SPHERE
C                 defaults -10000, 10000
C
C
C
C   INPUT   optional keywords, OVERPLOT
C
C
C    This card introduces atom input.  Optional keywords are:
C
C    CA   select alpha-carbon (CA) atoms only
C    SAVED for plotting into map, the scratch  file  has  already
C              been created in a previous run of the program, and
C              coordinates will not be read from stream 1.
C
C    If OVERPLOT is given on the  card  (LSuperimpose_Flag), this
C    indicates that the next drawing  is superimposed on the
C    previous  one (ie it is drawn at the  same  scale  and
C    position,  without advancing  the paper). This  provides
C    a  way of evading the limit  of (Max_Atoms) atoms in
C    a drawing.
C
C Group 3, atom selection
C             INPUT   C         Must be 1st in group
C             FILE*   O
C             LABEL   O
C             BOND    O
C             BTYPE   O
C             RESIDU  O
C             MAIN    O
C             BOUNDS  O     F
C             BOX*    O     F
C             SPHERE* O     F
C             STYLE   O
C             LINK    O
C             SKEW  )       O
C             TRANS )       O
C
C
C
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_BondType
      PARAMETER (Max_BondType=20)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Chains
      PARAMETER (Max_Chains=8)
      INTEGER Max_Colours
      PARAMETER (Max_Colours=10)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER Max_Labels
      PARAMETER (Max_Labels=100)
      INTEGER Max_Main
      PARAMETER (Max_Main=20)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER Max_Residues
      PARAMETER (Max_Residues=1000)
      INTEGER Max_Symmetry
      PARAMETER (Max_Symmetry=192)
      INTEGER Max_Tokens
      PARAMETER (Max_Tokens=200)
      INTEGER Max_Files
      PARAMETER (Max_Files=100)
C     ..
C     .. Scalar Arguments ..
      INTEGER NTOKX,NUM_FILES
      LOGICAL LOGPARSER,CELL_GIVEN
      CHARACTER LINEX*600, FILE_TEST*80
C     ..
C     .. Array Arguments ..
      REAL FVALUEX(Max_Tokens)
      INTEGER IBEGX(Max_Tokens),IDECX(Max_Tokens),IENDX(Max_Tokens),
     +        ITYPX(Max_Tokens)
      CHARACTER CVALUEX(Max_Tokens)*4
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Charcter_Size,
     +     CurrentCharcter_Size,D001,D010,D100,Def_Bnd_Radius,
     +     Def_Char_Size,Def_Paper_Wdth,Def_Plt_Radius,Def_Plt_Size,
     +     Dist_to_Boundary,Drawing_Size,Ellipse_ScaleFact,Eye_Distance,
     +     ROR(4,4),RFR(4,4),
     +     Overlap_Margin,Paper_Length,Paper_Width,
     +     Perspective_Distance,Picture_Scale,Plotter_Step,
     +     Retrace_Displmnt,S,Scal_pre_Proj,SCL,Shadelight_X,
     +     Shadelight_Y,Stereo_Separation
      INTEGER IERR,Invert_Map_Flag,ITEMP,Iuse_persp_Labs,KCHOSE,
     +        Label_atoms_Flag,Lcurrent_Colour,Ldef_Bnd_Lines,
     +        Ldraw_cell_Flag,LimitsSet_Flag,Ljoin_Flag,Lmap3d_Flag,
     +        Lmap_ModeFirst,Lorthogonalisation_Code,Lpack_Flag,
     +        Lpaper_Setting,Lperspective_Flag,Lplot_colour_Stereo,
     +        Lposition_Flag,Lprint_Flag,LSetView_Flag,LShade_Flag,
     +        LSolid_Flag,LStereo_Flag,LSuperimpose_Flag,LTaper_Flag,
     +        LTrace_Flag,Map_Counter,Map_Flag_Counter,Map_pts_U,
     +        Map_pts_V,MU,MV,MW,NAT,NATO,Natoms_plusCell,NC,NC1,
     +        Ncopies,NDrawing_Number,Npicture,Npts_U_compressed,
     +        Npts_V_compressed,Nspecified_Bonds,NSymmetry_Ops,
     +        Num_Labtxt_input,Num_LinkSets,Num_Moles,Num_Posit_Labels
      LOGICAL Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
      CHARACTER Iden_Title*80
C     ..
C     .. Arrays in Common ..
      REAL AID,AQ,ATM_AXIS_RMS,Atom_Radii,Bound_Edge,Bound_Limits,
     +     Contour_Levels,Contour_Threshold,Def_LnkBond,Drawing_Origin,
     +     Ellipse_Resolution,Fractional_XYZ,OrtCell,OrtConts,
     +     Orthogonal_XYZ,Plotting_XYZ,Positioned_Labels,Rotsym_Matrx,
     +     RP,Temp_Factors,Title_Rotation,TransSym_Vect,ViewMatrix,XABC,
     +     XCLANG,XLABEL,XO,YLABEL
      INTEGER IATID,IAtom_Connectivity,IAtom_Type,IConn_BndType,
     +        IResid_BndTyp,ISpecified_Bond_Params,ISym_BndType,IXYZIN,
     +        LAtom_Inc_Flag,LAtom_Label_Flag,Map_def_Restrictions,
     +        Map_LineTypes,Map_UVW_Limits,Map_XYZ_Limits,MOL,
     +        Num_Contours
      CHARACTER Atm_Lab_Type*2,Link_Atm_Names*2,Atm_Identifier*10,
     +          Atm_Label*10,Atm_Lab_Txt*10,Map_Title*80
C     ..
C     .. Local Scalars ..
      REAL ACELL,ALF,ASYM,BCELL,BET,BISO,CCELL,GAM,XDV,XX,YDV,YY,ZDV,ZZ
      INTEGER EndChain,I,I1,I2,IASYM,IAT,IBON,IBOND_NUMBER,IBOUNDS_SET,
     +        ICALPHA,ICALPHA_ONLY,ICOLR,ICOL_STYLE,IDO,IFAIL,IMAIN,
     +        IMAIN_BOND,INC,INCLUSIONTEST_FLAG,IPACK_BOND_TYPES,
     +        IRESCOUNTER,IRES_NUMBER,ISIDE,ISIDECHAIN_CODE,ISIDE_BOND,
     +        ISM,ISYM,IWORK,J,JDASH,JDO,JDOT,JMODE,JREPET,JSEL,JTEXT,
     +        JWORK,K,KCH,KDO,KEND_EXC,KEND_INC,KEND_XP,KKK,
     +        KMAIN_BOND,KRESSTORE,KSIDE_BOND,KSTART_EXC,KSTART_INC,
     +        KSTART_RESNUM,KSTART_XS,KWSTART,KWWANT,
     +        LABELS_SETFLAG,LDUM,LEN_ATOM_NAME,LEN_RESIDUE_NAME,LINDEX,
     +        LKIM,LLAB,LSAVED_FLAG,LSKEW_FLAG,LTRANS_COLOUR,
     +        LTRANS_FLAG,MAIN_WORK,MTYPE,N,NAtm_Label,NBTYPE,
     +        NCHAIN_NUMBER,NKIM,NLAB_TYPES,NNN,NOUT,NUM_MAINCHNTYPS,
     +        StChain
      LOGICAL BONDSETFLAG,EXCLUDE_SET,
     +        INDIVID_EXCSET,INDIVID_RESSET,LENDX,SELECT_SET
      CHARACTER CCHAIN_EXC*1,CCHAIN_INC*1,CHAIN_NAME*1,ELEMENT_SYMBOL*2,
     +          RESIDUE_NAME*4,CWORK*4,KEYX*4,RESIDUE_NUMBER*4,ID*4,
     +          ATOM_NAME*4,BWORK*5,NAME*10,FILE_NAME*10,INSCOD*1,
     +          ALTCOD*1,SEGID*4
C     ..
C     .. Local Arrays ..
      REAL AXISRMSWORK(3,Max_Atoms),BB(6),BFACT_WRK(3,3,Max_Atoms),
     +     RWORK(3),SKEW_MATRIX(3,3),TRANS_VECTOR(3),
     +     XB(Max_Symmetry,4),Y(3)
      INTEGER IATM_TYP_WRK(Max_Atoms),INCREMENT_LABSFLGS(Max_Labels),
     +        IRES_BONTYPWRK(Max_Atoms,Max_Chains),
     +        IRES_COLS(Max_Atoms,Max_Chains),
     +        IRES_STORE(Max_Residues),
     +        ISYM_COLS(Max_Symmetry),IUSE_MAIN(Max_Atoms,Max_Chains),
     +        IUSE_SIDE(Max_Atoms,Max_Chains),
     +        LABPACKEDFLAGS(Max_Labels),LATM_INC_WRK(Max_Atoms),
     +        LATM_LAB_FLG_WRK(Max_Atoms),LPEN_NUMS(Max_Colours)
      CHARACTER CHAIN_STORE(Max_Chains)*1,
     +          CNUM(10)*1,MAINCHAINTYPES(Max_Main)*3,
     +          ALLOWED_COLS(Max_Colours)*4,ATM_ID_WRK(Max_Atoms)*10
C     ..
C     .. External Functions ..
      INTEGER LENSTR,MSKGET,LPERM
      EXTERNAL LENSTR,MSKGET
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPDPN,CCPERR,CCPLWC,CCPUPC,chrread,getatomid,getbvalues,
     +         KEYNUM,PARSER,setboundarylimits,testmatrix, UGTENV,
     +         XYZADVANCE,XYZATOM,XYZCOORD,XYZOPEN,
     +         XYZREWD
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,ICHAR,INDEX,MOD,NINT,REAL
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_BFAC/Temp_Factors(3,3,Max_Atoms),
     +       ATM_AXIS_RMS(3,Max_Atoms)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CHOOSE/IATID(8),KCHOSE
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_DEFAULTS/Ldef_Bnd_Lines,Def_Bnd_Radius,Def_Plt_Radius,
     +       Def_Char_Size,Def_Plt_Size,Def_Paper_Wdth
      COMMON /com_LABCHR/Atm_Lab_Txt(Max_Labels),
     +       Atm_Lab_Type(Max_Labels),Atm_Label(Max_Labels),
     +       Atm_Identifier(Max_Atoms),Iden_Title
      COMMON /com_LABELS/Num_Posit_Labels,Num_Labtxt_input,
     +       XLABEL(Max_Atoms),YLABEL(Max_Atoms),
     +       Positioned_Labels(2,Max_Labels)
      COMMON /com_LIMITS/Bound_Limits(3),Bound_Edge(3)
      COMMON /com_LINK/Num_LinkSets,IResid_BndTyp(Max_Atoms),
     +       ISym_BndType(Max_Symmetry),IConn_BndType(Max_BondType),
     +       Def_LnkBond(2)
      COMMON /com_LNKCHR/Link_Atm_Names(2,2)
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
      COMMON /com_SYMMETRY/NSymmetry_Ops,Rotsym_Matrx(3,3,Max_Symmetry),
     +       TransSym_Vect(3,Max_Symmetry)
      COMMON /UNITS/ ITEMP,IXYZIN(Max_Files)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Data statements ..
C
C
      DATA CNUM/'0','1','2','3','4','5','6','7','8','9'/
      DATA ALLOWED_COLS/'BLAC','CYAN','RED ','GREE','BLUE','MAGE',
     +     'YELL','PURP','ORAN','BLAC'/
      DATA LPEN_NUMS/1,2,3,4,5,6,7,8,9,10/
C     ..
C
C
      NBTYPE = 7
C
C
      ACELL = XABC(1)
      BCELL = XABC(2)
      CCELL = XABC(3)
      ALF = XCLANG(1)
      BET = XCLANG(2)
      GAM = XCLANG(3)
C
C iside=1 include sidechain, =0 exclude
C imain=1 include m/c, =0 exclude
C jmain=-1 ca only, =+1 all main chain atoms
C
      ISIDE = 1
      IMAIN = 1
      ICALPHA = 0
      NCHAIN_NUMBER = 0
C
C--- Default protein main-chain atom names
C
      NUM_MAINCHNTYPS = 4
      MAINCHAINTYPES(1) = 'CA '
      MAINCHAINTYPES(2) = 'C  '
      MAINCHAINTYPES(3) = 'N  '
      MAINCHAINTYPES(4) = 'O  '
C
C---- set flag for atom plotting
C
      Label_atoms_Flag = 1
C
C---- set default link atoms
C
      Link_Atm_Names(1,1) = 'N '
      Link_Atm_Names(2,1) = 'C '
      Num_LinkSets = 0
C
C---- default link bond length
C
      Def_LnkBond(1) = 1.8
C
C
      DO 20 KKK = 1,Max_Chains
        CHAIN_STORE(KKK) = ' '
        DO 10 JDO = 1,Max_Atoms
          IUSE_SIDE(JDO,KKK) = 0
          IUSE_MAIN(JDO,KKK) = 0
          IRES_COLS(JDO,KKK) = 0
          IResid_BndTyp(JDO) = -1
          IRES_BONTYPWRK(JDO,KKK) = -1
   10   CONTINUE
   20 CONTINUE
C
C
      DO 30 JDO = 1,Max_Symmetry
        ISym_BndType(JDO) = -1
        ISYM_COLS(JDO) = 0
   30 CONTINUE
C
C
      DO 40 JDO = 1,Max_BondType
        IConn_BndType(JDO) = 0
   40 CONTINUE
C
C---- Set symmetry to identity by default
C
      IF (NSymmetry_Ops.EQ.0) THEN
        NSymmetry_Ops = 1
C
C
        DO 60 I = 1,3
          DO 50 J = 1,3
            Rotsym_Matrx(I,J,1) = 0.0
   50     CONTINUE
          TransSym_Vect(I,1) = 0.0
          Rotsym_Matrx(I,I,1) = 1.0
   60   CONTINUE
C
C
      END IF
C
C
      LSKEW_FLAG = 0
C
C
      DO 80 I = 1,3
        TRANS_VECTOR(I) = 0.0
        DO 70 J = 1,3
          SKEW_MATRIX(I,J) = 0.0
   70   CONTINUE
        SKEW_MATRIX(I,I) = 1.0
   80 CONTINUE
C
C
      LTRANS_FLAG = 0
      LABELS_SETFLAG = 0
C
C
      DO 90 JDO = 1,Max_Labels
        INCREMENT_LABSFLGS(JDO) = 1
   90 CONTINUE
C
C
      NLAB_TYPES = 0
      NAtm_Label = 0
      Num_Posit_Labels = 0
      Num_Labtxt_input = 0
      ICALPHA_ONLY = 0
      IBOUNDS_SET = 0
      NAT = 8
      INCLUSIONTEST_FLAG = 0
      LSAVED_FLAG = 0
C
C---- default bounds
C
      IF (Map_Flag_Counter.GT.0) THEN
C
C---- bounds already set
C
        IBOUNDS_SET = 1
        NOUT = 0
      ELSE
C
C
        DO 100 JDO = 1,3
          Bound_Limits(JDO) = -10000.0
          Bound_Edge(JDO) = 10000.0
  100   CONTINUE
      END IF
C
C      =========================
C----  INCLUDE  [overplot]
C      =========================
C
C----  LSuperimpose_Flag=1  first set of coordinates.
C      LSuperimpose_Flag gt 1
C                  more coordinates to plot on same picture
C
      LSuperimpose_Flag = 1
      IWORK = 1
C
C
  110 CONTINUE
      IWORK = IWORK + 1
      IF (IWORK.GT.NTOKX) GO TO 120
      CWORK = CVALUEX(IWORK)
C
C          *************
      CALL CCPUPC(CWORK)
C          *************
C
C----  recognises keywords on input card - saved,cas
C
C     #### Sub_Key_Words of "INPUT" ####
C     ==================================
C
      IF (CWORK(1:2).EQ.'CA') THEN
        ICALPHA_ONLY = 1
      ELSE IF (CWORK.EQ.'SAVE') THEN
        LSAVED_FLAG = 1
      ELSE IF (CWORK.EQ.'OVER') THEN
        LSuperimpose_Flag = LSuperimpose_Flag + 1
      END IF
C
C
      GO TO 110
C
C---- Now Get next line - if unrecognised
C     return the line to subroutine KEYWORDS
C
  120 CONTINUE
C
C
      NTOKX = Max_Tokens
      LINEX = '    '
C
C   itypx(I)      =0  null field
C                =1  character string
C                =2  number
C
C          **********************************************************
      CALL PARSER(KEYX,LINEX,IBEGX,IENDX,ITYPX,FVALUEX,CVALUEX,IDECX,
     +            NTOKX,LENDX,LOGPARSER)
C          **********************************************************
C
      WRITE(6,125)LINEX(1:100)
125   FORMAT(' Data Line---- ',A100)
      IF (LENDX) GO TO 530
C
C     =================
C---- LABEL Instruction
C     =================
C
C LABEL     keywords    n1 n2 n3 etc
C
C    Label the given atoms or atom types n1, n2 etc.  If a number
C    is  given  with  an atom type, this label is only plotted is
C    the residue number is an integral multiple of  this  number.
C    The  program  by  default  tries  to  find  the  'best' of 8
C    possible positions for each label.  If two numbers are given
C    with  an  atom  name,  they  are used as the position of the
C    start of the label, relative to the atom position, in mm.
C
C                        Optional keywords:
C
C    FIXED plot all labels to the right of the atoms.  This saves
C              some  time.  FIXED must precede other keywords and
C              atom types.
C    ALL label all atoms (default if no atoms specified)
C    TEXT This is followed by a label (of up to 6 characters)  to
C              be plotted on the following atom.
C
C          eg   LABEL FIXED CA 10
C               label every 10th CA atom
C
C           LABEL TEXT IRON FE150 12 -5
C           Plot the label 'IRON' displaced by +12mm on  plotter  x
C      and -5mm on y from the atom with name FE150 .
C
C
      IF (KEYX.EQ.'LABE') THEN
C
C     #### Sub_Key_Words of "LABEL" ####
C     ==================================
C
        LABELS_SETFLAG = 0
        Lposition_Flag = 0
        JTEXT = 0
C
C---- No tokens given LABEL ALL
C
        IF (NTOKX.LT.2) THEN
          LABELS_SETFLAG = 1
          GO TO 120
        END IF
C
C
        IWORK = 2
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
        IF (CWORK(1:3).NE.'TEX' .AND.
     +        CWORK(1:3).NE.'NON' .AND. CWORK(1:3).NE.'ALL') THEN
C
C---- Expecting line of:-
C
C           LABEL FIX atom_name [increment number, default = 1]
C        or LABEL atom_name
C
C---- label every j'th residue
C
          Lposition_Flag = 1
          NLAB_TYPES = NLAB_TYPES + 1
C
C
          IF(CWORK(1:3).EQ.'FIX')THEN
            IWORK = IWORK + 1
            IF (IWORK.GT.NTOKX) 
C
C                  ************************************************
     +        CALL CCPERR(1,' Not enough tokens on LABEL FIX line')
C                  ************************************************
C
            CWORK = CVALUEX(IWORK)
C
C                *************
            CALL CCPUPC(CWORK)
C                *************
C
            IF (CWORK .EQ. 'ALL') THEN
              LABELS_SETFLAG = 1
              GOTO 120
            ENDIF
          ENDIF
          Atm_Lab_Type(NLAB_TYPES) = CVALUEX(IWORK) (1:2)
          IWORK = IWORK + 1
          IF (IWORK.GT.NTOKX) GO TO 120
          KWWANT = 1
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          INCREMENT_LABSFLGS(NLAB_TYPES) = NINT(FVALUEX(KWSTART))
          GO TO 120
C
C
        ELSE IF (CWORK.EQ.'ALL') THEN
C
C---- expecting Line of :-
C
C              LABEL ALL
C
          LABELS_SETFLAG = 1
          GO TO 120
C
C
        ELSE IF (CWORK.EQ.'NONE') THEN
C
C---- expecting line of :-
C
C            LABEL NONE
C
          LABELS_SETFLAG = -1
          GO TO 120
C
C
        ELSE IF (CWORK.EQ.'TEXT') THEN
C
C---- Expecting line of:-
C
C        LABEL TEXT string ATOM_ID [dx, dy]
C
          LABELS_SETFLAG = 0
          IWORK = IWORK + 1
          IF (IWORK.GT.NTOKX) GO TO 120
          Num_Labtxt_input = Num_Labtxt_input + 1
C
C                        
          IF (Num_Labtxt_input.GT.Max_Labels) 
C
C             *******************************************
     +   CALL CCPERR(1,' Too many labels, maximum = 100')
C             *******************************************
C
          Atm_Lab_Txt(Num_Labtxt_input) = LINEX(IBEGX(IWORK) :
     +      IENDX(IWORK))
          IWORK = IWORK + 1
          JTEXT = 2
C
C---- next string is atom id CBnumber e.g.
C     STORE LABEL
C     - ie atom plus residue number input, only label this atom
C
          NAtm_Label = NAtm_Label + 1
C
C                      
          IF (NAtm_Label.GT.Max_Labels) 
C
C             *******************************************
     +   CALL CCPERR(1,' Too many labels, maximum = 100')
C             *******************************************
C
          IF (IWORK.GT.NTOKX) 
C
C            ******************************************************
     +  CALL CCPERR(1,
     + ' Not enough tokens LABEL TEXT string Atom_Id [dx,dy] line')
C            ******************************************************
C
          Atm_Label(NAtm_Label) = LINEX(IBEGX(IWORK) :IENDX(IWORK))
          JWORK = 1
          IWORK = IWORK + 1
C
C
          IF (IWORK+1.LE.NTOKX) THEN
C
C---- if 2 numbers on card, use as positions for this label relative
C     to atom position, in mm
C
            Num_Posit_Labels = Num_Posit_Labels + 1
C
C                     
            IF (Num_Posit_Labels.GT.Max_Labels) 
C
C            *******************************************
     +  CALL CCPERR(1,' Too many labels, maximum = 100')
C            *******************************************
C
            JWORK = Num_Posit_Labels + 1
            KWWANT = 2
            KWSTART = IWORK
C
C                ****************************************************
            CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                ****************************************************
C
            Positioned_Labels(1,Num_Posit_Labels) = FVALUEX(IWORK)
            Positioned_Labels(2,Num_Posit_Labels) = FVALUEX(IWORK+1)
          END IF
C
C---- pack into LabPackedFlags text number, if present
C
          IF (JTEXT.NE.0) JTEXT = Num_Labtxt_input*100
          JWORK = JWORK + JTEXT
          LABPACKEDFLAGS(NAtm_Label) = JWORK
          GO TO 120
        END IF
        GO TO 120
C
C
C     ================================================
C---- BTYPE - type of anisotropic temperature factors
C     ================================================
C
C
C  default type  NBType = 7  (using values from 'RADII ATOMS'
C
C  for  b(11) b(22) b(33) b(12) b(13) b(23)
C
C with (Base)^{_D (b11*h**2  + b22*k**2  + b33*l**2 +
C                  C*b12*h*k + C*b13*h*l + C*b23*k*l)}
C
C        NBType   Base   C     D
C         0       e      2     1
C         1       e      1     1
C         2       2      2     1
C         3       2      1     1
C        10       e      2     2pi**2
C
C or the coefficients U(ij)
C
C with  exp{ - D (a'**2 * U11 * h**2 +
C                 b'**2 * U22 * k**2 +
C                 c'**2 * U33 * l**2 +
C                 C * a' * b' * U22 * h * k +
C                 C * a' * c' * U13 * h * l +
C                 C * b' * c' * U23 * k * l ) }
C
C where   a',  b',  c'   are reciprocal cell dimensions
C
C        NBType   C    D
C         4       2    1/4
C         5       1    1/4
C         8       2    2 * pi**2
C         9       1    2 * pi**2
C
C   NBType = 6 allows for the Debye-Waller isotropic temperature
C              factor B which is used as follows:
C
C       exp ( -B sin^2(theta/lambda**2)
C
C       B is related to mean-square displacement mu^2 of the atom
C         from its mean position by the relation
C
C         B = 8 pi**2 <rms**2>
C
C   the length of the ellipse principal axes == rms
C
C  NBType = 7 allows for radii to be used
C
C
      ELSE IF (KEYX.EQ.'BTYP') THEN
        IF (NTOKX.LE.1) GO TO 120
        NBTYPE = NINT(FVALUEX(2))
        IF (NBTYPE.LT.0 .OR. NBTYPE.GT.10) 
C
C            *****************************************************
     +  CALL CCPERR(1,' Input BFactor type outside range 0 to 10')
C            *****************************************************
C
        GO TO 120
C
C      ====================================================
C----  BOUNDS   xmin, xmax, ymin, ymax, zmin, zmax,   CELL
C      ====================================================
C
C     Atom limits in fractional coordinates.   Only  atoms  within
C     these  limits  are  plotted.   Unit  cell  translations  are
C     applied if necessary to put the atom into this  volume.   If
C     maps  are plotted, the bounds are set automatically.  If the
C     keyword CELL is present, the bounds are also stored  as  the
C     cell  edges,  so  that automatic scaling will be done on the
C     bounds.  Otherwise automatic scaling will be done  on  atoms
C     only.
C
C
      ELSE IF (KEYX.EQ.'BOUN') THEN
C
C     #### Sub_Key_Words of "BOUNDS" ####
C     ==================================
C
        IF (NTOKX.LT.7) 
C
C            *******************************************************
     +  CALL CCPERR(1,' Error on BOUNDS Key_Word Not Enough Tokens')
C            *******************************************************
C
        IBOUNDS_SET = 1
C
C
        DO 130 JDO = 1,3
          KSTART_XS = 2*JDO
          KEND_XP = 2*JDO + 1
          KWWANT = 1
          KWSTART = KSTART_XS
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Bound_Limits(JDO) = FVALUEX(KSTART_XS)
          KWWANT = 1
          KWSTART = KEND_XP
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Bound_Edge(JDO) = FVALUEX(KEND_XP)
  130   CONTINUE
C
C---- if keyword 'cell' absent, do not reset cell edges
C     to bounds values, so auto scaling is done on real atoms only
C
        IF (NTOKX.LT.8) GO TO 120
        CWORK = CVALUEX(8)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
        IF (CWORK.EQ.'CELL') THEN
C
C----  reset cell edges to bounds values
C
C              *********************************************************
          CALL setboundarylimits(Fractional_XYZ,Bound_Limits,Bound_Edge)
C              *********************************************************
C
C---- set plot cell flag
C
          Ldraw_cell_Flag = 1
        END IF
C
C
        GO TO 120
C
C     ================
C---- RESIDU  keywords
C     ================
C
C
C     word2  .(essential )  exclud, includ ,select, black, red,
C                          green, blue, yellow, or bond
C     3 (optional)  side ,main, or ca
C     4   (optional)  to, all
C   last  (optional) black,red,green,blue,yellow,bond
C
C
C
C      Select residues  for  plotting,  and  plotting  style.   The
C      second  keyword  may  be  a selection word SELECT, INCLUD or
C      EXCLUD, or a style keyword BLACK, RED, GREEN, BLUE,  YELLOW,
C      ORANGE,  PURPLE,  or  BOND.  The third keyword is optionally
C      MAIN ,SIDE, or CA , to choose main chain, side chain, or  Ca
C      atoms  only  (main  chain  atoms  are CA,C ,O ,N ).  Residue
C      numbers may be given as a list, as a range n1 TO n2, or  all
C      residues  by  the  keyword ALL.  A style keyword may also be
C      included as the last keyword on the card.  If a  residue  is
C      SELECTed,  but  not  EXCLUDed,  all  symmetry operations are
C      applied to its atoms.  If a BOUNDS card is present  or  maps
C      are  plotted  , the coordinates are tested against them, and
C      unit cell translations are applied to try to bring them into
C      the specified volume.  Note that each symmetry operation can
C      generate no more than one  atom.   INCLUDed  atoms  have  no
C      symmetry operations applied, and no bounds are tested.
C                    The BOND keyword is followed by  two  numbers,
C      which give the bond types (specified on BOND cards) for main
C      and side chain atoms (only one number is  expected  if  MAIN
C      ,SIDE , or CA are specified).
C                    Several RESIDU cards may be given,  and  their
C      effect is cumulative.  If no cards are given, the default is
C      to SELECT all residues, but if any INCLUD or SELECT card  is
C      present, all other residues are excluded.
C
C      Some examples:
C
C      RESIDU SELECT 100 TO 123 BOND 1 2
C
C      RESIDU BOND 1 2 ALL
C      RESIDU BOND 3 4 50 TO 56 RED
C
C        Draw residues 50 to 56 with bond types  3&4  in  red,  all
C        others with bond types 1&2
C
C      RESIDU GREEN ALL
C      RESIDU EXCLUD SIDE 50 53 67
C
C
C
      ELSE IF (KEYX.EQ.'RESI') THEN
C
C     #### Sub_Key_Words of "RESIDIE" ####
C     ====================================
C
        IRESCOUNTER = 0
        INDIVID_RESSET = .FALSE.
        INDIVID_EXCSET = .FALSE.
        SELECT_SET = .FALSE.
        EXCLUDE_SET = .FALSE.
        BONDSETFLAG = .FALSE.
        KSIDE_BOND = 1
        KMAIN_BOND = 1
        CCHAIN_INC = ' '
        CCHAIN_EXC = ' '
        NNN = 0
C
C iside=1 include sidechain, =0 exclude
C imain=1 include m/c, =0 exclude
C jmain=-1 ca only, =+1 all main chain atoms
C
        ISIDE = 1
        IMAIN = 1
        ICALPHA = 0
        ICOLR = 0
        IBON = 0
        JSEL = 0
C
C
        IWORK = 1
  140   CONTINUE
        IWORK = IWORK + 1
        IF (IWORK.GT.NTOKX) GO TO 320
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
C---- Try colour first    RESIDUE COLOUR_string
C
C
C
        DO 150 JDO = 1,Max_Colours
          IF (CWORK.EQ.ALLOWED_COLS(JDO)) THEN
            ICOLR = LPEN_NUMS(JDO)
            GO TO 140
          END IF
  150   CONTINUE
C
C---- Expecting line BOND main side
C
        IF (CWORK.EQ.'BOND') THEN
          IWORK = IWORK + 1
C
C
          IF (IWORK+1.GT.NTOKX) 
C
C            *****************************************************
     +  CALL CCPERR(1,' Not enough tokens RESIDUE BOND main side')
C            *****************************************************
C
          KWWANT = 2
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          KMAIN_BOND = NINT(FVALUEX(IWORK))
          KSIDE_BOND = NINT(FVALUEX(IWORK+1))
          IWORK = IWORK + 1
          BONDSETFLAG = .TRUE.
          GO TO 140
C
C---- RESIDUE MAIN, SIDE, CA
C
C iside=1 include sidechain, =0 exclude
C imain=1 include m/c, =0 exclude
C jmain=-1 ca only, =+1 all main chain atoms
C
C
        ELSE IF (CWORK.EQ.'MAIN') THEN
          ISIDE = 0
          IMAIN = 1
          GO TO 140
        ELSE IF (CWORK.EQ.'SIDE') THEN
          ISIDE = 1
          IMAIN = 0
          GO TO 140
        ELSE IF (CWORK(1:2).EQ.'CA') THEN
          ISIDE = 0
          IMAIN = 1
          ICALPHA = -2
          GO TO 140
C
C---- RESIDUE INCLUDE or SELECT
C
        ELSE IF (CWORK.EQ.'SELE' .OR. CWORK.EQ.'INCL') THEN
C
C---- includ or select:
C               select tests bounds,
C               includ does not
C
          JSEL = 0
C
C
          IF (INCLUSIONTEST_FLAG.EQ.0) THEN
C
C---- default all residues to exclud before 1st inclusion
C
            DO 170 KKK = 1,Max_Chains
              DO 160 JDO = 1,Max_Atoms
                IUSE_MAIN(JDO,KKK) = -1
                IUSE_SIDE(JDO,KKK) = -1
  160         CONTINUE
  170       CONTINUE
          END IF
C
C---- set flags, =0 select, =+1 includ
C
          IF (CWORK.EQ.'INCL') JSEL = 1
          IF (CWORK.EQ.'SELE') JSEL = 0
C
C---- RESIDUE SELECT[INCLUDE] n1 TO n2
C           RESIDUE SELECT 529 TO 601 BOND 1 1 RED
C
          IF ((IWORK+3).LE.NTOKX .AND. ITYPX(IWORK+2).EQ.1) THEN
            CWORK = CVALUEX(IWORK+2)
C
C                *************
            CALL CCPUPC(CWORK)
C                *************
C
            IF (CWORK(1:2).EQ.'TO') THEN
C
C---- range of residues KSTART to KFINISH
C
              BWORK = LINEX(IBEGX(IWORK+1) :IENDX(IWORK+1))
C
C                  ************************************
              CALL chrread(KSTART_INC,CCHAIN_INC,BWORK)
C                  ************************************
C
              IF (NCHAIN_NUMBER.EQ.0) GO TO 190
              DO 180 KKK = 1,NCHAIN_NUMBER
                IF (CCHAIN_INC.EQ.CHAIN_STORE(KKK)) GO TO 200
  180         CONTINUE
  190         NCHAIN_NUMBER = NCHAIN_NUMBER + 1
              IF (NCHAIN_NUMBER.GT.Max_Chains) 
C
C           **************************************
     + CALL CCPERR(1,' Too many chains specified')
C           **************************************
C
              CHAIN_STORE(NCHAIN_NUMBER) = CCHAIN_INC
              KKK = NCHAIN_NUMBER
  200         CONTINUE
              NNN = KKK
C
C
              BWORK = LINEX(IBEGX(IWORK+3) :IENDX(IWORK+3))
C
C                  **********************************
              CALL chrread(KEND_INC,CCHAIN_INC,BWORK)
C                  **********************************
C
              SELECT_SET = .TRUE.
              IWORK = IWORK + 3
              GO TO 320
            END IF
          END IF
C
C
          IF ((IWORK+1).LE.NTOKX .AND. ITYPX(IWORK+1).EQ.1) THEN
            CWORK = CVALUEX(IWORK+1)
C
C                *************
            CALL CCPUPC(CWORK)
C                *************
C
            IF (CWORK(1:3).EQ.'ALL') THEN
C
C---- Residue Select/Include All
C
              SELECT_SET = .TRUE.
              KSTART_INC = 1
              KEND_INC = Max_Atoms
              IWORK = IWORK + 1
              GO TO 320
            END IF
          END IF
C
C---- individual residue numbers
C
  210     CONTINUE
          IWORK = IWORK + 1
          IF (IWORK.GT.NTOKX) GO TO 320
          IRESCOUNTER = IRESCOUNTER + 1
C
C      
          IF (IRESCOUNTER.GT.Max_Residues) 
C
C           ****************************************************
     + CALL CCPERR(1,
     + ' Too many individual residues selected on RESIDUE line')
C           ****************************************************
C
          BWORK = LINEX(IBEGX(IWORK) :IENDX(IWORK))
C
C              ***********************************
          CALL chrread(KRESSTORE,CCHAIN_INC,BWORK)
C              ***********************************
C
          IF (NCHAIN_NUMBER.EQ.0) GO TO 230
          DO 220 KKK = 1,NCHAIN_NUMBER
            IF (CCHAIN_INC.EQ.CHAIN_STORE(KKK)) THEN
              IRES_STORE(IRESCOUNTER) = KRESSTORE + 10000*KKK
              GO TO 240
            END IF
  220     CONTINUE
  230     NCHAIN_NUMBER = NCHAIN_NUMBER + 1
          IF (NCHAIN_NUMBER.GT.Max_Chains) 
C
C            **************************************
     +  CALL CCPERR(1,' Too many chains specified')
C            **************************************
C
          CHAIN_STORE(NCHAIN_NUMBER) = CCHAIN_INC
          IRES_STORE(IRESCOUNTER) = KRESSTORE + 10000*NCHAIN_NUMBER
          KKK = NCHAIN_NUMBER
  240     CONTINUE
          NNN = KKK
C
C
          INDIVID_RESSET = .TRUE.
          GO TO 210
C
C---- RESIDUE EXCLUDE n1 TO n2
C
        ELSE IF (CWORK.EQ.'EXCL') THEN
C
C
          IF ((IWORK+3).LE.NTOKX .AND. ITYPX(IWORK+2).EQ.1) THEN
            CWORK = CVALUEX(IWORK+2)
C
C                *************
            CALL CCPUPC(CWORK)
C                *************
C
            IF (CWORK(1:2).EQ.'TO') THEN
              BWORK = LINEX(IBEGX(IWORK+1) :IENDX(IWORK+1))
C
C                  ************************************
              CALL chrread(KSTART_EXC,CCHAIN_EXC,BWORK)
C                  ************************************
C
              IF (NCHAIN_NUMBER.EQ.0) GO TO 260
              DO 250 JDO = 1,NCHAIN_NUMBER
                IF (CCHAIN_EXC.EQ.CHAIN_STORE(JDO)) GO TO 270
  250         CONTINUE
  260         NCHAIN_NUMBER = NCHAIN_NUMBER + 1
              IF (NCHAIN_NUMBER.GT.Max_Chains) 
C
C            **************************************
     +  CALL CCPERR(1,' Too many chains specified')
C            **************************************
C
              CHAIN_STORE(NCHAIN_NUMBER) = CCHAIN_EXC
              JDO = NCHAIN_NUMBER
  270         CONTINUE
              NNN = JDO
C
C
              BWORK = LINEX(IBEGX(IWORK+3) :IENDX(IWORK+3))
C
C                  **********************************
              CALL chrread(KEND_EXC,CCHAIN_EXC,BWORK)
C                  **********************************
C
              EXCLUDE_SET = .TRUE.
              GO TO 320
            END IF
          END IF
C
C---- range individual numbers for residue exclude
C
  280     CONTINUE
          IWORK = IWORK + 1
          IF (IWORK.GT.NTOKX) GO TO 320
          IRESCOUNTER = IRESCOUNTER + 1
C
C         
          IF (IRESCOUNTER.GT.Max_Residues) 
C
C           *******************************************************
     + CALL CCPERR(1,
     +    ' Too many individual residues selected on RESIDUE line')
C           *******************************************************
C
          BWORK = LINEX(IBEGX(IWORK) :IENDX(IWORK))
C
C              ***********************************
          CALL chrread(KRESSTORE,CCHAIN_EXC,BWORK)
C              ***********************************
C
          IF (NCHAIN_NUMBER.EQ.0) GO TO 300
          DO 290 KKK = 1,NCHAIN_NUMBER
            IF (CCHAIN_EXC.EQ.CHAIN_STORE(KKK)) THEN
              IRES_STORE(IRESCOUNTER) = KRESSTORE + 10000*KKK
              GO TO 310
            END IF
  290     CONTINUE
  300     NCHAIN_NUMBER = NCHAIN_NUMBER + 1
          IF (NCHAIN_NUMBER.GT.Max_Chains) 
C
C            **************************************
     +  CALL CCPERR(1,' Too many chains specified')
C            **************************************
C
          CHAIN_STORE(NCHAIN_NUMBER) = CCHAIN_EXC
          IRES_STORE(IRESCOUNTER) = KRESSTORE + 10000*NCHAIN_NUMBER
          KKK = NCHAIN_NUMBER
  310     CONTINUE
C
          NNN = KKK
          INDIVID_EXCSET = .TRUE.
          GO TO 280
C
C
        ELSE
          WRITE (6,FMT=6000)
 6000     FORMAT (' For Key_Word RESIDUE Line Not Understood')
          GO TO 120
        END IF
C
C
  320   CONTINUE
C
C---- pack stored BOND info
C
        IF (BONDSETFLAG) THEN
          KMAIN_BOND = KMAIN_BOND*IMAIN
          KSIDE_BOND = KSIDE_BOND*ISIDE
C
C---- pack main type*1000 + side type
C
          IF (KSIDE_BOND.EQ.0) KSIDE_BOND = KMAIN_BOND
          IF (KMAIN_BOND.EQ.0) KMAIN_BOND = KSIDE_BOND
          IBON = KMAIN_BOND*1000 + KSIDE_BOND
        END IF
C
C
        IF (INDIVID_RESSET) THEN
C
C
          INCLUSIONTEST_FLAG = 1
C
          DO 330 JDO = 1,IRESCOUNTER
            I = MOD(IRES_STORE(JDO),10000)
            KKK = IRES_STORE(JDO)/10000
C
C iside=1 include sidechain, =0 exclude
C imain=1 include m/c, =0 exclude
C jmain=-1 ca only, =+1 all main chain atoms
C
            IF (ISIDE.EQ.1) IUSE_SIDE(I,KKK) = JSEL
            IF (IMAIN.EQ.1) IUSE_MAIN(I,KKK) = JSEL - ICALPHA
            IF (ICOLR.GT.0) IRES_COLS(I,KKK) = ICOLR
            IF (IBON.GT.0) IRES_BONTYPWRK(I,KKK) = IBON
  330     CONTINUE
C
C
        ELSE IF (SELECT_SET) THEN
          INCLUSIONTEST_FLAG = 1
C
C---- If SELECT ALL then apply to all chains ELSE to just one chain.
C
          IF (NNN .EQ. 0) THEN
            StChain = 1
            EndChain = Max_Chains
          ELSE
            StChain = NNN
            EndChain = NNN
          ENDIF
C
          DO 340 IDO = StChain,EndChain
            DO 340 JDO = KSTART_INC,KEND_INC
              K = JDO
C
C                     
              IF (K.LE.0 .OR. K.GT.Max_Atoms) 
C
C               *********************************************
     +     CALL CCPERR(1,' Error in RESIDUE INCLUDE/SELECT ')
C               *********************************************
C
              IF (ISIDE.EQ.1) IUSE_SIDE(K,IDO) = JSEL
              IF (IMAIN.EQ.1) IUSE_MAIN(K,IDO) = JSEL - ICALPHA
              IF (ICOLR.GT.0) IRES_COLS(K,IDO) = ICOLR
              IF (IBON.GT.0) IRES_BONTYPWRK(K,IDO) = IBON
  340     CONTINUE
C
C
        ELSE IF (EXCLUDE_SET) THEN
C
C
          INCLUSIONTEST_FLAG = 1
C
C
          KKK = NNN
C
C
          DO 350 JDO = KSTART_EXC,KEND_EXC
            K = JDO
C
C                     
            IF (K.LE.0 .OR. K.GT.Max_Atoms) 
C
C            ************************************
     +  CALL CCPERR(1,'ERROR in RESIDUE EXCLUDE')
C            ************************************
C
            IF (ISIDE.EQ.1) IUSE_SIDE(K,KKK) = -1
            IF (IMAIN.EQ.1) IUSE_MAIN(K,KKK) = -1
  350     CONTINUE
C
C
        ELSE IF (INDIVID_EXCSET) THEN
          INCLUSIONTEST_FLAG = 1
C
C
          DO 360 JDO = 1,IRESCOUNTER
            I = MOD(IRES_STORE(JDO),10000)
            KKK = IRES_STORE(JDO)/10000
            IF (ISIDE.EQ.1) IUSE_SIDE(I,KKK) = -1
            IF (IMAIN.EQ.1) IUSE_MAIN(I,KKK) = -1
  360     CONTINUE
C
C
        END IF
C
C
        GO TO 120
C
C     ================
C---- STYLE CARD
C     ================
C
C
C     read bond type or colour for symmetry numbers
C     2nd keyword black,red,green,blue,yellow, or bond
C     optional 3rd keyword trans: colour for all translated atoms
C     otherwise list of symmetry numbers
C  STYLE    keyword, symmetry numbers or TRANS
C
C     Read plotting style for molecules with  the  given  symmetry
C     numbers.   Style  keywords  are  BLACK,  RED,  GREEN,  BLUE,
C     YELLOW, ORANGE,  PURPLE  and  BOND.   The  BOND  keyword  is
C     followed  by  two  numbers giving the bond type for main and
C     side  chains.   Colours  given  here  take  precedence  over
C     colours  given  on  RESIDU  cards.   If the keyword TRANS is
C     given with a colour, this colour is used for all atoms which
C     have been translated to get them into the BOUNDS or the map,
C     irrespective of symmetry  operation.   This  takes  priority
C     over a symmetry colour.
C
C     eg   STYLE RED 2 3
C
C          STYLE BOND 1 2 2
C
C          Bond types 1 & 2 for symmetry number 2
C
C          STYLE GREEN TRANS
C
      ELSE IF (KEYX.EQ.'STYL') THEN
C
C     #### Sub_Key_Words of "STYLE" ####
C     ==================================
C
        JWORK = -2
        IWORK = 2
C
C                      
        IF (IWORK.GT.NTOKX) 
C
C             ************************************************
     +   CALL CCPERR(1,' Not Enough tokens on STYLE Key_Word')
C             ************************************************
C
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
C---- Try first    STYLE COLOUR string
C
        DO 380 JDO = 1,Max_Colours
          IF (CWORK.EQ.ALLOWED_COLS(JDO)) THEN
            ICOL_STYLE = LPEN_NUMS(JDO)
C
C---- colour, read symmetry numbers
C
            IWORK = IWORK + 1
C
C
            IF (IWORK.GT.NTOKX) 
C
C            **************************************************
     +  CALL CCPERR(1,' Not enough tokens STYLE COLOUR string')
C            **************************************************
C
            CWORK = CVALUEX(IWORK)
C
C                *************
            CALL CCPUPC(CWORK)
C                *************
C
            IF (CWORK.EQ.'TRAN') THEN
C
C---- STYLE COLOUR string TRANS
C                store colour fort translated atoms
C
              LTRANS_FLAG = ICOL_STYLE
            ELSE
C
C---- STYLE COLOUR string sym_numbers
C
              I = NTOKX - IWORK + 1
C
C
              IF (I.GT.Max_Symmetry) 
C
C           *******************************************************
     + CALL CCPERR(1,'STYLE COLOUR sym_numbers, NOT Enough tokens')
C           *******************************************************
C
              DO 370 KDO = IWORK,NTOKX
                KWWANT = 1
                KWSTART = KDO
C
C                    **********************************************
                CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,
     +                      NTOKX)
C                    **********************************************
C
                I = NINT(FVALUEX(KDO))
                ISYM_COLS(I) = ICOL_STYLE
  370         CONTINUE
C
C
            END IF
            GO TO 120
          END IF
  380   CONTINUE
C
C---- STYLE BOND main side sym_numbers
C

        IF (CWORK.EQ.'BOND') THEN
C
C---- bond type
C     first 2 numbers are bond types for main and side chains
C
          IWORK = IWORK + 1
C
C
          IF (IWORK+2.GT.NTOKX)
C
C           *********************************************************
     + CALL CCPERR(1,'Not enough tokens of STYLE BOND main side symm')
C           *********************************************************
C
          KWWANT = 2
          KWSTART = IWORK
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          IMAIN_BOND = NINT(FVALUEX(IWORK))
          ISIDE_BOND = NINT(FVALUEX(IWORK+1))
          IPACK_BOND_TYPES = IMAIN_BOND*1000 + ISIDE_BOND
C
C---- remaining numbers are symmetry numbers
C
          DO 390 JDO = IWORK + 2,NTOKX
            KWWANT = 1
            KWSTART = JDO
C
C                ****************************************************
            CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                ****************************************************
C
            J = NINT(FVALUEX(JDO))
            ISym_BndType(J) = IPACK_BOND_TYPES
  390     CONTINUE
C
C
          GO TO 120
        END IF
C
C
        WRITE (6,FMT=6002)
 6002   FORMAT (' Warning STYLE card NOT Understood')
        GO TO 120
C
C     ==============
C---- BOND CARD
C     ==============
C
C
C     read bond type, bond radius, number of lines
C     for dashed bond, keyword 'dash' followed by
C     repeat,dash,dot
C
C  BOND   bond type, bond radius, number of lines
C
C
C      Defines a bond type, referred to on RESIDU and STYLE  cards.
C      The  bond  is  defined  by  a  cylindrical radius in A and a
C      number of lines.  Dashed bonds (one line only) are specified
C      by  the  keyword  DASHED  and three numbers (see MODE card).
C      Bond styles may also be defined by RADII BONDS card  (qv)  .
C      The default style is radius 0.04, 8 lines.
C
C          eg   BOND 1 0.04 5
C               BOND 3 DASHED 4 1.5 0
C
      ELSE IF (KEYX.EQ.'BOND') THEN
C
C     #### Sub_Key_Words of "BOND" ####
C     ==================================
C
C
        IF (NTOKX.LT.4) 
C
C            ***************************************************
     +  CALL CCPERR(1,' BOND Key_Word Error -- No Tokens given')
C            ***************************************************
C
        KWWANT = 1
        KWSTART = 2
C
C            ****************************************************
        CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C            ****************************************************
C
        IBOND_NUMBER = NINT(FVALUEX(2))
C
C                    
        IF (IBOND_NUMBER.GT.Max_BondType) 
C
C           **************************************************
     + CALL CCPERR(1,' Bond number greater than Max_BondType')
C           **************************************************
C
        IWORK = 2
        IF (ITYPX(IWORK+1).EQ.1) THEN
          CWORK = CVALUEX(IWORK+1)
C
C              *************
          CALL CCPUPC(CWORK)
C              *************
C
          IF (CWORK.EQ.'DASH') THEN
C
C---- dashed line
C
            IWORK = IWORK + 2
C
C
            IF (IWORK.GT.NTOKX) THEN
C
C--- use default 0.04 8 0
C
              JMODE = 3
              JREPET = 4
              JDASH = 80
              JDOT = 0
            ELSE
              IF (IWORK.LE.NTOKX) JREPET = NINT(FVALUEX(IWORK)*10.0)
              IF (IWORK+1.LE.NTOKX) JDASH = NINT(FVALUEX(IWORK+1)*10.0)
              IF (IWORK+2.LE.NTOKX) JDOT = NINT(FVALUEX(IWORK+2)*10.0)
              IF (IWORK+2.LE.NTOKX) JMODE = 4
            END IF
C
C
            IF (JMODE.EQ.2) JMODE = 3
            IF (JREPET.EQ.0) JREPET = 2
            IF (JDASH.EQ.0) JDASH = 1
            J = 50000*JDOT + 500*JDASH + 5*JREPET + JMODE
            J = -J
            IConn_BndType(IBOND_NUMBER) = J
            GO TO 120
C
C
          ELSE
C
C                *************************************
            CALL CCPERR(1,' BOND number DASHED error')
C                *************************************
C
          END IF
        END IF
C
C--- BOND  number thick lines
C    Store bond type
C
C                            *******************************************
        IF (NTOKX.LT.4) CALL CCPERR(1,' Not enough tokens on BOND line')
C                            *******************************************
C
        KWWANT = 2
        KWSTART = 3
C
C            ****************************************************
        CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C            ****************************************************
C
        J = NINT(FVALUEX(4))
        J = NINT((FVALUEX(3)+REAL(J))*1000.0)
        IConn_BndType(IBOND_NUMBER) = J
        GO TO 120
C
C     ==============
C---- LINK CARD
C     ==============
C
C
C  LINK link atoms
C
C   If the JOIN RESIDU option is used  (qv,  automatic  for  map
C   plotting),  residues  are linked by recognizing the names of
C   link atoms, by default N and C .  These names may be changed
C   on  a LINK card.  A number may be given for the maximum bond
C   length  between  these  atoms  for  a  bond  to   be   drawn
C   (default=1.8).
C
C          eg   LINK N C (=default, bondlength =1.8)
C
C               LINK CA CA 4.1        for CA plots
C
C   Two different LINK atom pairs may be given.  They must  both
C   be  given  (no  defaults)  and must be in ascending order of
C   bond distance.  This may be useful for mixed main-chain  and
C   CA only drawings.
C
C          eg       LINK N C 1.8
C                   LINK CA CA 4.1
C                   LINK P O3'
C
      ELSE IF (KEYX.EQ.'LINK') THEN
C
C     #### Sub_Key_Words of "LINK" ####
C     ==================================
C
        Num_LinkSets = Num_LinkSets + 1
C
C                     
        IF (Num_LinkSets.GT.2) 
C
C              *******************************************
     +    CALL CCPERR(1,'TOO many LINK Key_Words, max 2')
C              *******************************************
C
        IF (NTOKX.LT.3)
C
C              *******************************************
     +    CALL CCPERR(1,' not enough fields on LINK line')
C              *******************************************
C
        DO 400 JDO = 2,3
C
C---- link atom name
C
          Link_Atm_Names(JDO-1,Num_LinkSets) = CVALUEX(JDO) (1:
     +      LENSTR(CVALUEX(JDO)))
  400   CONTINUE
C
C---- link bond length
C
        IF (NTOKX.GE.4) THEN
          KWWANT = 1
          KWSTART = 4
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Def_LnkBond(Num_LinkSets) = FVALUEX(4)
        END IF
C
C
        WRITE (6,FMT=6004) Num_LinkSets,
     +    (Link_Atm_Names(I,Num_LinkSets),I=1,2),
     +    Def_LnkBond(Num_LinkSets)
 6004   FORMAT (/' Link atoms',I3,' : ',A,4X,A,10X,
     +         'Link bond length = ',F10.5,/)
        GO TO 120
C
C     ===============
C---- SKEW CARD
C     ===============
C
C
C     read rotation matrix to rotate orthogonal atom
C     coordinates to orthogonal map frame:
C
C          xormap=Skew_Matrix*(xor-tran)
C
C     SKEW   S11,S12,S13,S21,S22,S23,S31,S32,S33
C
C    Read matrix S to rotate  orthogonal  atom  coordinates  into
C    orthogonal  map  frame:  Xo(map) = S * (Xo(atom) - t), where
C    the  vector  t  comes  from  a  TRANS   card   (qv).    This
C    transformation  will  usually be required for a skew map, or
C    other cases where the map and the molecule are  not  in  the
C    same  frame.   The program then works in the map frame (both
C    orthogonal  and  fractional),  which  means  that   symmetry
C    operations  (if  any)  must also be transformed into the map
C    frame.  A blank SKEW card  causes  the  skew  transformation
C    (rotation  and  translation)  to  be  picked up from the map
C    header, if it is present there.
C
C
      ELSE IF (KEYX.EQ.'SKEW') THEN
C
C     #### Sub_Key_Words of "SKEW" ####
C     ==================================
C
        IF (NTOKX.EQ.1) THEN
C
C---- If blank SKEW card, pick up skew matrix from map if present
C
C                      ********************************
          LSKEW_FLAG = MSKGET(SKEW_MATRIX,TRANS_VECTOR)
C                      ********************************
C
          GO TO 120
        END IF
C
C
        IF (NTOKX.LT.10) 
C
C             ************************************************
     +   CALL CCPERR(1,'ERROR not enough tokens on SKEW line')
C             ************************************************
C
        LSKEW_FLAG = 1
        K = 1
C
C
        DO 420 I = 1,3
          DO 410 J = 1,3
            K = K + 1
            KWWANT = 1
            KWSTART = K
C
C                ****************************************************
            CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C                ****************************************************
C
            SKEW_MATRIX(I,J) = FVALUEX(K)
  410     CONTINUE
  420   CONTINUE
C
C            ***********************
        CALL testmatrix(SKEW_MATRIX)
C            ***********************
C
        GO TO 120
C
C     ==================
C---- TRANS CARD
C     ==================
C
C
C     read translation vector to shift orthogonal
C     atom coordinates to map frame  xormap=Skew_Matrix*(xor-tran)
C
C  TRANS    tx, ty, tz
C
C    Translation vector for skew transformation.  The vector t is
C    the position of the map origin in the molecule frame.
C
C
      ELSE IF (KEYX.EQ.'TRAN') THEN
C
C     #### Sub_Key_Words of "TRANS" ####
C     ==================================
C
C
        IF (NTOKX.LT.4) 
C
C             ********************************************
     +   CALL CCPERR(1,' Not enough tokens on TRANS line')
C             ********************************************
C
        LSKEW_FLAG = 1
C
C
        DO 430 JDO = 1,3
          KWWANT = 1
          KWSTART = JDO + 1
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          TRANS_VECTOR(JDO) = FVALUEX(JDO+1)
  430   CONTINUE
C
C
        GO TO 120
C
C      =========================================
C----  BOX    x   y   z    hx   hy  hz     CELL
C      =========================================
C
C BOX card, read coordinates of centre of box (A), and half-edges
C  (either 1 number for all directions or 3 numbers)
C
C
C    Select atoms in box centred at x,y,z (orthogonal coordinates
C    in  A),  with  half-edges  hx,hy,hz  A.   If  hy  and hz are
C    omitted, hx is used for  all  directions  (default  hx=10A).
C    The  limits  are  converted to fractional coordinates before
C    use, so in a skew cell the  box  will  be  skew.   The  CELL
C    keyword is the same as on the BOUNDS card.
C
      ELSE IF (KEYX(1:3).EQ.'BOX') THEN
C
C     #### Sub_Key_Words of "BOX" ####
C     ==================================
C
        IBOUNDS_SET = 1
        IWORK = 8
C
C---- If 6 numbers , separate half-edges for each direction
C
        IF (NTOKX.LT.7) THEN
C
C                     
          IF (NTOKX.LT.4) 
C
C            ******************************************
     +  CALL CCPERR(1,' NOT Enough tokens on BOX line')
C            ******************************************
C
          IF (NTOKX.EQ.4) THEN
C
C---- If 4 numbers, 4th is half-edge, default = 10A
C
            IWORK = 5
            FVALUEX(5) = 10.0
          END IF
C
C
          DO 460 JDO = 1,3
            Bound_Edge(JDO) = FVALUEX(5)
  460     CONTINUE
C
C
          GO TO 480
        END IF
C
C
        DO 470 JDO = 1,3
          Bound_Edge(JDO) = FVALUEX(JDO+4)
  470   CONTINUE
C
C
  480   CONTINUE
C
C
        DO 490 JDO = 1,3
C
C---- Start and finish points in A
C
          Bound_Limits(JDO) = FVALUEX(JDO+1) - Bound_Edge(JDO)
          Bound_Edge(JDO) = FVALUEX(JDO+1) + Bound_Edge(JDO)
  490   CONTINUE
C
C---- Convert to fractional
C
c        Y(1) = Bound_Limits(1)*FR1 + Bound_Limits(2)*FR2 +
c     +         Bound_Limits(3)*FR3
c        Y(2) = Bound_Limits(1)*FR4 + Bound_Limits(2)*FR5 +
c     +         Bound_Limits(3)*FR6
c        Y(3) = Bound_Limits(1)*FR7 + Bound_Limits(2)*FR8 +
c     +         Bound_Limits(3)*FR9
c        RWORK(1) = Bound_Edge(1)*FR1 + Bound_Edge(2)*FR2 +
c     +             Bound_Edge(3)*FR3
c        RWORK(2) = Bound_Edge(1)*FR4 + Bound_Edge(2)*FR5 +
c     +             Bound_Edge(3)*FR6
c        RWORK(3) = Bound_Edge(1)*FR7 + Bound_Edge(2)*FR8 +
c     +             Bound_Edge(3)*FR9
        Y(1) = Bound_Limits(1)*RFR(1,1) + Bound_Limits(2)*RFR(1,2) +
     +         Bound_Limits(3)*RFR(1,3)
        Y(2) = Bound_Limits(1)*RFR(2,1) + Bound_Limits(2)*RFR(2,2) +
     +         Bound_Limits(3)*RFR(2,3)
        Y(3) = Bound_Limits(1)*RFR(3,1) + Bound_Limits(2)*RFR(3,2) +
     +         Bound_Limits(3)*RFR(3,3)
        RWORK(1) = Bound_Edge(1)*RFR(1,1) + Bound_Edge(2)*RFR(1,2) +
     +             Bound_Edge(3)*RFR(1,3)
        RWORK(2) = Bound_Edge(1)*RFR(2,1) + Bound_Edge(2)*RFR(2,2) +
     +             Bound_Edge(3)*RFR(2,3)
        RWORK(3) = Bound_Edge(1)*RFR(3,1) + Bound_Edge(2)*RFR(3,2) +
     +             Bound_Edge(3)*RFR(3,3)
C
C
        DO 500 JDO = 1,3
          Bound_Limits(JDO) = Y(JDO)
          Bound_Edge(JDO) = RWORK(JDO)
  500   CONTINUE
C
C---- Look for CELL keyword
C
        CWORK = CVALUEX(IWORK)
C
C            *************
        CALL CCPUPC(CWORK)
C            *************
C
        IF (CWORK.EQ.'CELL') THEN
C
C----  reset cell edges to bounds values
C
C              *********************************************************
          CALL setboundarylimits(Fractional_XYZ,Bound_Limits,Bound_Edge)
C              *********************************************************
C
C---- set plot cell flag
C
          Ldraw_cell_Flag = 1
        END IF
C
C
        GO TO 120
C
C    ================================
C--- SPHERE card, default radius = 10
C    ================================
C
C   SPHERE   x   y   z    r
C
C    Select atoms in sphere centred at x,y,z A, radius r (default
C          r=10A).
C
      ELSE IF (KEYX.EQ.'SPHE') THEN
C
C     #### Sub_Key_Words of "SPHERE" ####
C     ==================================
C
        IBOUNDS_SET = -1
C
C                      
        IF (NTOKX.LT.4) 
C
C             *********************************************
     +   CALL CCPERR(1,' NOT ENOUGH tokens on SPHERE line')
C             *********************************************
C

        DO 510 JDO = 1,3
          KWWANT = 1
          KWSTART = JDO + 1
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Bound_Limits(JDO) = FVALUEX(JDO+1)
  510   CONTINUE
C
C
        Bound_Edge(1) = 10.0
C
C
        IF (NTOKX.GE.5) THEN
          KWWANT = 1
          KWSTART = 5
C
C              ****************************************************
          CALL KEYNUM(KWWANT,KWSTART,LINEX,IBEGX,IENDX,ITYPX,NTOKX)
C              ****************************************************
C
          Bound_Edge(1) = FVALUEX(5)
        END IF
C
C
        GO TO 120
C
C     ================================================
C---- MAIN card, read list of atom names in main-chain
C     ================================================
C
C   MAIN        main-chain atoms
C
C    Define atoms to be  classified  as  main-chain  atoms.   The
C    first  atom  in  the list is considered as the "CA" atom for
C    selection by RESIDU SELECT CA and INPUT CAS.  The default is
C    MAIN CA C N O, for proteins.
C    eg  MAIN    C1' C2' C3' C4' C5' O4' O5' P O1P O2P O3' N9 N1
C
C
      ELSE IF (KEYX.EQ.'MAIN') THEN
C
C     #### Sub_Key_Words of "MAIN" ####
C     ==================================
C
        IF (NTOKX.LT.2) GO TO 120
        NUM_MAINCHNTYPS = NTOKX - 1
        IWORK = 1
C
C                      
        IF (NUM_MAINCHNTYPS.GT.Max_Main) 
C
C             ************************************************
     +   CALL CCPERR(1,' Too many MAIN chain atoms requested')
C             ************************************************
C
        DO 520 JDO = 1,NUM_MAINCHNTYPS
          IWORK = IWORK + 1
          CWORK = LINEX(IBEGX(IWORK) :IENDX(IWORK))
          MAINCHAINTYPES(JDO) = CWORK(1:3)
  520   CONTINUE
C
C
        WRITE (6,FMT=6008) NUM_MAINCHNTYPS,
     +    (MAINCHAINTYPES(I),I=1,NUM_MAINCHNTYPS)
 6008   FORMAT (/,1X,I6,' main chain atoms defined:',/,20 (1X,A))
C
C
        GO TO 120
C
C
      ELSE
C
C     ====================================================
C---- Un-recognised return this line to calling subroutine
C     ====================================================
C
C
C     unrecognized card, all KWPROTIN control cards finished
C     saved keyword on input card means that scratch file
C     already exists
C
        IF (LSAVED_FLAG.NE.0) RETURN
        GO TO 530
      END IF
C
C
  530 CONTINUE
      IF (Num_LinkSets.NE.2) Num_LinkSets = 1
C
C
      IFAIL = 0
      LDUM = 80
      NUM_FILES = NUM_FILES + 1
C
C----  Open atom file , Logical name is XYZINn, stored in File_Name
C
      IF (NUM_FILES .GT. 9) THEN
        WRITE(FILE_NAME,FMT='(A,I2)') 'XYZIN',NUM_FILES
      ELSE
        WRITE(FILE_NAME,FMT='(A,I1)') 'XYZIN',NUM_FILES
      ENDIF

      WRITE(6,6005) FILE_NAME
 6005 FORMAT(/,' Opening coordinate file ',A)
C
      IXYZIN(NUM_FILES) = 0
      IF (NUM_FILES.EQ.1) THEN
        FILE_TEST=' '
        CALL UGTENV('XYZIN', FILE_TEST)
        IF (FILE_TEST.NE.' ') THEN
          FILE_NAME='XYZIN'
          WRITE (6, *) 'MSG: Using XYZIN and not XYZIN1'
        ENDIF
      ENDIF
C          *****************************************************
      CALL XYZOPEN(FILE_NAME,'INPUT',' ',IXYZIN(NUM_FILES),IFAIL)
C          *****************************************************
C
C----  Open scratch file if required, Logical name is TEMP
C
      IF (Map_Flag_Counter.GT.0) THEN
        ITEMP = 0
C
C            *********************************************
        CALL CCPDPN(ITEMP,'TEMP','SCRATCH','U',LDUM,IFAIL)
C            *********************************************
C
      ENDIF
      WRITE (6,FMT=6010) LSuperimpose_Flag
 6010 FORMAT (' Protein input file - number = ',I5)
      IF (IBOUNDS_SET.GT.0) WRITE (6,FMT=6012) (Bound_Limits(I),
     +    Bound_Edge(I),I=1,3)
 6012 FORMAT (//,' Boundary for input coordinates and for cell edge ',
     +       /,3(5X,2F12.4))
      IF (IBOUNDS_SET.LT.0) WRITE (6,FMT=6014) Bound_Limits,
     +    Bound_Edge(1)
 6014 FORMAT (/,'  Select coordinates inside a sphere centre ',3F10.3,
     +       /,'   radius = ',F10.3,/)
C
C
      IF (IBOUNDS_SET.LT.0) Bound_Edge(1) = Bound_Edge(1)*Bound_Edge(1)
C
C
      IF (LSKEW_FLAG.NE.0) THEN
        WRITE (6,FMT=6016) ((SKEW_MATRIX(I,J),J=1,3),I=1,3),
     +    TRANS_VECTOR
 6016   FORMAT (/,'  Matrix to rotate atoms to orthogonal map frame : ',
     +         /,3(/,10X,3F10.5),//,'  Translation : ',/,3F10.5)
      END IF
C
C                            *****************************************
      IF ( .NOT. CELL_GIVEN) CALL CCPERR(1,' ERROR: no cell given')
C                            *****************************************
C
C---- Input Brookhaven format
C
C          *********************************************************
  540 CALL XYZADVANCE(IXYZIN(NUM_FILES),0,0,*920,*920)
      CALL XYZATOM(IXYZIN(NUM_FILES),ISER,ATOM_NAME,RESIDUE_NAME,
     +          CHAIN_NAME,IRES_NUMBER,RESIDUE_NUMBER,INSCOD,ALTCOD,
     +                                                   SEGID,IZ,ID)
      CALL XYZCOORD(IXYZIN(NUM_FILES),'O','B',XX,YY,ZZ,OCC,BISO,BB)
C          *********************************************************
      write(6,'(I6,A4,A4,1x,A1,I4,A6,3A,I6,A4,/,5F8.2)')
     +          ISER,ATOM_NAME,RESIDUE_NAME,
     +          CHAIN_NAME,IRES_NUMBER,RESIDUE_NUMBER,INSCOD,ALTCOD,
     +                                               SEGID,IZ,ID,
     +          XX,YY,ZZ,OCC,BISO
C
C--- Reset B=0.0 to something..
      IF (BB(1).EQ.0.0 .AND. BB(2).EQ.0.0 .AND. 
     +                                   BB(3).EQ.0.0) BB(1)=1.0
C      IF (BB(1).EQ.0.0 .AND. BB(2).EQ.0.0 .AND. 
C     +                                   BB(3).EQ.0.0) GOTO 540

      LEN_RESIDUE_NAME = LENSTR(RESIDUE_NAME)
      LEN_ATOM_NAME = LENSTR(ATOM_NAME)
C
C
      IF (NCHAIN_NUMBER.EQ.0) THEN
        KKK = 1
        GO TO 570
      END IF
C
C---- seems this checks RESIDUE RED BOND 1 1 SELECT 1A TO 120A
C
      DO 560 NNN = 1,NCHAIN_NUMBER
        IF (CHAIN_STORE(NNN).EQ.CHAIN_NAME) THEN
          KKK = NNN
          GO TO 570
        END IF
  560 CONTINUE
      GO TO 540
  570 CONTINUE
C
C
      DO 580 JDO = 1,4
        IF (RESIDUE_NUMBER(JDO:JDO).NE.' ') GO TO 590
  580 CONTINUE
C
C
      KSTART_RESNUM = 1
      GO TO 600
  590 KSTART_RESNUM = JDO
  600 CONTINUE
C
C
  620 CONTINUE
C
C
C----  Put element symbol
C
C   classify atoms as they are read in
C   from the coordinate data file. Note that the subroutine presumes
C   that anything beginning with C, N, O, or S is Carbon, Nitrogen,
C   Oxygen or Sulphur resp.,with the exceptions of, CA which is
C   alpha carbon which gets special treatment in KWPROTIN,
C   and OW - water.
C   Added Copper, Cu to this list of exceptions,  Feb.88, SMM/EMBLHH
C
C
C
C---- Check whether C, N, O or S, additional checks on CA and water (OW)
C
      IF (ID(1:1) .EQ. ' ') THEN
        IF (ATOM_NAME(1:2).EQ.'CA' .OR. ATOM_NAME(1:2).EQ.'OW') THEN
          ELEMENT_SYMBOL = ATOM_NAME(1:2)
        ELSE
          ELEMENT_SYMBOL = ATOM_NAME(1:1)//' '
        ENDIF
      ELSE

        LINDEX = 0
        DO 630 JDO = 1,10
          LINDEX = INDEX(ATOM_NAME(2:2),CNUM(JDO))
          IF (LINDEX.GT.0) GOTO 635
 630    CONTINUE

 635    IF (LINDEX .EQ. 0) THEN
          CWORK = ATOM_NAME(2:2)
C
C              *************
          CALL CCPLWC(CWORK)
C              *************
C
          ELEMENT_SYMBOL = ATOM_NAME(1:1)//CWORK(1:1)
        ELSE
          ELEMENT_SYMBOL = ATOM_NAME(1:1)//' '
        ENDIF
      END IF
C
C
  640 CONTINUE
C
C
C---- the following test doesnt count input residue numbers
C     just says a residue number cannot be lt 0
C     or greater than Max_Atoms ?
C
      IF (IRES_NUMBER.LE.0 .OR. IRES_NUMBER.GT.Max_Atoms) GO TO 540
C
C---- Get side-chain code
C      ISideChain_Code=-1 CA, =0 main, =1 side-chain
C
C      Compare atom label Atom_Name with Num_MainChnTyps
C      main-chain labels
C
C        ISideChain_Code = -1  if matches 1st label (CA)
C                        =  0  if matches any other (main-chain)
C                        =  1  if no match (side-chain)
C
      ISIDECHAIN_CODE = 0
C
C
      DO 650 JDO = 1,NUM_MAINCHNTYPS
        IF (ATOM_NAME(1:3).NE.MAINCHAINTYPES(JDO)) GO TO 650
        IF (JDO.EQ.1) ISIDECHAIN_CODE = -1
        GO TO 660
  650 CONTINUE
      ISIDECHAIN_CODE = 1
  660 CONTINUE
C
C--- plot cas only ?
C
      IF (ICALPHA_ONLY.EQ.1 .AND. ISIDECHAIN_CODE.NE.-1) GO TO 540
C
C
      IF (ELEMENT_SYMBOL.EQ.'CA' .OR.
     +    ELEMENT_SYMBOL.EQ.'OW') THEN
C
C----  ca atoms, set 1st 3 characters of atom name=res name
C      if not blank
C---- atom type wa, copy residue name (=water number)
C     to label to replace  residue number
C
        NAME = RESIDUE_NAME(1:LEN_RESIDUE_NAME)//
     +         RESIDUE_NUMBER(KSTART_RESNUM:4)//CHAIN_NAME
C
C
        IF (ELEMENT_SYMBOL.EQ.'CA') THEN
          I1 = ICHAR('C')
          I2 = ICHAR('A')
          MTYPE = -ABS(I1**2+I2**3)
        ELSE IF (ELEMENT_SYMBOL.EQ.'OW') THEN
          I1 = ICHAR('O')
          I2 = ICHAR('W')
          MTYPE = -ABS(I1**2+I2**3)
        END IF
C
C
      ELSE
        NAME = ATOM_NAME(1:LEN_ATOM_NAME)//
     +         RESIDUE_NUMBER(KSTART_RESNUM:4)//CHAIN_NAME
        I1 = ICHAR(ELEMENT_SYMBOL(1:1))
        I2 = ICHAR(ELEMENT_SYMBOL(2:2))
        MTYPE = -ABS(I1**2+I2**3)
      END IF
C
C---- Check for ANISOU cards
C
      IF (BB(2).NE.0.0 .OR. BB(3).NE.0.0) THEN
        IF (XX.EQ.0.0 .AND. YY.EQ.0.0 .AND. ZZ.EQ.0.0) THEN
          DO 665 I=NAT,1,-1
            IF (NAME .EQ. ATM_ID_WRK(I)) THEN
C
C                  *********************************************
              CALL getbvalues(BB,I,NBTYPE,BFACT_WRK,AXISRMSWORK)
C                  *********************************************
C
            ENDIF
  665     CONTINUE
          GOTO 540
        ENDIF
      ENDIF
C
C---- skew coordinates to map frame
C
      IF (LSKEW_FLAG.NE.0) THEN
        XDV = XX - TRANS_VECTOR(1)
        YDV = YY - TRANS_VECTOR(2)
        ZDV = ZZ - TRANS_VECTOR(3)
        XX = SKEW_MATRIX(1,1)*XDV + SKEW_MATRIX(1,2)*YDV +
     +       SKEW_MATRIX(1,3)*ZDV
        YY = SKEW_MATRIX(2,1)*XDV + SKEW_MATRIX(2,2)*YDV +
     +       SKEW_MATRIX(2,3)*ZDV
        ZZ = SKEW_MATRIX(3,1)*XDV + SKEW_MATRIX(3,2)*YDV +
     +       SKEW_MATRIX(3,3)*ZDV
      END IF
C
C---- calculate fractional coordinates
C
c      XDV = FR1*XX + FR2*YY + FR3*ZZ
c      YDV = FR4*XX + FR5*YY + FR6*ZZ
c      ZDV = FR7*XX + FR8*YY + FR9*ZZ
      XDV = RFR(1,1)*XX + RFR(1,2)*YY + RFR(1,3)*ZZ
      YDV = RFR(2,1)*XX + RFR(2,2)*YY + RFR(2,3)*ZZ
      ZDV = RFR(3,1)*XX + RFR(3,2)*YY + RFR(3,3)*ZZ
C
C---- any inclusion tests ?
C
      IF (INCLUSIONTEST_FLAG.EQ.0) GO TO 700
C
C---- Test on exclude/include on chains
C
C---- test for residue to be included or excluded
C    'INCL'  JSEL = 1
C    'SELE'  JSEL = 0
C
C
C     IUse_Main = -1  exclude
C               =  0  Select
C               =  1  Include
C But If C-alpha only then
C               = -1 exclude
C               =  2 select
C               =  3 include
C
      MAIN_WORK = IUSE_MAIN(IRES_NUMBER,KKK)
C
C        ISideChain_Code = -1  if matches 1st label (CA)
C                        =  0  if matches any other (main-chain)
C                        =  1  if no match (side-chain)
C
      IF (ISIDECHAIN_CODE.EQ.0) GO TO 670
      IF (ISIDECHAIN_CODE.GT.0) GO TO 690
C
C--- ca atom
C
      IF (MAIN_WORK.GT.1) MAIN_WORK = MAIN_WORK - 2
C
C---- main Reject if
C                      Resid_Chaintype(IRes_Number)
C                and
C                      IUse_Main(IRes_Number,KKK)
C                match excluded criteria
C
  670 IF (MAIN_WORK.LT.0) GO TO 540
C
C---- main chain
C
      IF (MAIN_WORK.EQ.0) GO TO 700
C
C---- Included no bounds tests
C
      IF (MAIN_WORK.GT.0) THEN
        IASYM = 1
        XB(1,1) = XDV
        XB(1,2) = YDV
        XB(1,3) = ZDV
        XB(1,4) = 1
        GO TO 790
      END IF
C
C
      GO TO 540
C
C---- Side Reject if
C                      Resid_Chaintype(IRes_Number,KKK)
C                and
C                      IUse_Side(IRes_Number,KKK)
C                match excluded criteria
C
  690 IF (IUSE_SIDE(IRES_NUMBER,KKK).LT.0) GO TO 540
C
C---- Included no bounds tests
C
      IF (IUSE_SIDE(IRES_NUMBER,KKK).GT.0) THEN
        IASYM = 1
        XB(1,1) = XDV
        XB(1,2) = YDV
        XB(1,3) = ZDV
        XB(1,4) = 1
        GO TO 790
      END IF
C
C
      IF (IUSE_SIDE(IRES_NUMBER,KKK).NE.0) GO TO 540
C
C---- Selected do bounds tests
C
  700 IASYM = 0
      LTRANS_COLOUR = 0
C
C
      DO 780 ISYM = 1,NSymmetry_Ops
        DO 710 J = 1,3
          Y(J) = Rotsym_Matrx(J,1,ISYM)*XDV +
     +           Rotsym_Matrx(J,2,ISYM)*YDV +
     +           Rotsym_Matrx(J,3,ISYM)*ZDV + TransSym_Vect(J,ISYM)
  710   CONTINUE
C
C----  check bounds only if given
C
        IF (IBOUNDS_SET.EQ.0) GO TO 770
        IF (IBOUNDS_SET.LT.0) THEN
C
C---- SPHERE check on orthogonal coordinates
C
c          XX = Y(1)*OR1 + Y(2)*OR2 + Y(3)*OR3
c          YY = Y(1)*OR4 + Y(2)*OR5 + Y(3)*OR6
c          ZZ = Y(1)*OR7 + Y(2)*OR8 + Y(3)*OR9
          XX = Y(1)*ROR(1,1) + Y(2)*ROR(1,2) + Y(3)*ROR(1,3)
          YY = Y(1)*ROR(2,1) + Y(2)*ROR(2,2) + Y(3)*ROR(2,3)
          ZZ = Y(1)*ROR(3,1) + Y(2)*ROR(3,2) + Y(3)*ROR(3,3)
          XX = (XX-Bound_Limits(1))**2 + (YY-Bound_Limits(2))**2 +
     +         (ZZ-Bound_Limits(3))**2
          IF (XX.GT.Bound_Edge(1)) GO TO 780
          GO TO 770
        END IF
C
C
        CONTINUE
C
C
        DO 760 JDO = 1,3
  730     IF (Y(JDO).GE.Bound_Limits(JDO)) GO TO 740
          Y(JDO) = Y(JDO) + 1.0
          LTRANS_COLOUR = LTRANS_FLAG
          GO TO 730
  740     IF (Y(JDO).LT.Bound_Edge(JDO)) GO TO 750
          Y(JDO) = Y(JDO) - 1.0
          LTRANS_COLOUR = LTRANS_FLAG
          GO TO 740
  750     IF (Y(JDO).LT.Bound_Limits(JDO) .OR.
     +        Y(JDO).GT.Bound_Edge(JDO)) GO TO 780
  760   CONTINUE
C
C
  770   IASYM = IASYM + 1
        XB(IASYM,1) = Y(1)
        XB(IASYM,2) = Y(2)
        XB(IASYM,3) = Y(3)
        XB(IASYM,4) = ISYM
  780 CONTINUE
C
C
      IF (IASYM.LE.0) GO TO 540
  790 CONTINUE
C
C
      IF (KCHOSE.NE.0) THEN
        N = 1
        KCH = KCHOSE
C
C
        IF (KCHOSE.LT.0) THEN
          N = 0
          KCH = -KCHOSE
        END IF
C
        INC = 0
C
C
        DO 800 JDO = 1,KCH
          IF (MTYPE.EQ.IATID(JDO)) INC = 1
  800   CONTINUE
C
C
        IF (N.EQ.0 .AND. INC.EQ.1) GO TO 540
        IF (N.EQ.1 .AND. INC.EQ.0) GO TO 540
      END IF
C
C
      LLAB = 0
C
C---- check if this atom should be labelled
C
      IF (LABELS_SETFLAG.EQ.-1) GO TO 880
      IF (LABELS_SETFLAG.GT.0) GO TO 870
C
C---- is this atom specifically labelled ?
C      i.e. LABEL TEXT
C
      IF (NAtm_Label.NE.0) THEN
C
C---- Search through Atm_Label array for this atom
C
C
        DO 830 JDO = 1,NAtm_Label
          IF (NAME.NE.Atm_Label(JDO)) GO TO 830
C
C---- FOUND - so set label for flag
C
          LLAB = LABPACKEDFLAGS(JDO)
          GO TO 880
  830   CONTINUE
C
C
      END IF
C
C---- check atom type labels
C     i.e. LABEL FIX
C
      DO 850 JDO = 1,NLAB_TYPES
C
C            **********************************
        CALL getatomid(IWORK,Atm_Lab_Type(JDO))
C            **********************************
C
        IF (IWORK.EQ.MTYPE) GO TO 860
  850 CONTINUE
C
C---- no label
C
      GO TO 880
C
C---- check residue number For Increment number
C
  860 IF (MOD(IRES_NUMBER,INCREMENT_LABSFLGS(JDO)).NE.0) GO TO 880
C
C---- LABEL THIS ATOM - set flag, jump here if LABEL ALL specified
C
  870 LLAB = 1
C
  880 CONTINUE
      IF (Map_Flag_Counter.GT.0) NAT = 0
C
C
      DO 900 JDO = 1,IASYM
        NAT = NAT + 1
        IF (NAT.GT.Max_Atoms) GO TO 910
C
C----  store atom names
C
        ATM_ID_WRK(NAT) = NAME
        IResid_BndTyp(NAT) = IRES_BONTYPWRK(IRES_NUMBER,KKK)
C
C---- find atom colour, symmetry colour takes precedence
C
        ICOLR = IRES_COLS(IRES_NUMBER,KKK)
        ISYM = NINT(XB(JDO,4))
        IF (ISYM_COLS(ISYM).GT.0) ICOLR = ISYM_COLS(ISYM)
C
C---- colour for translated atoms
C
        IF (LTRANS_COLOUR.NE.0) ICOLR = LTRANS_COLOUR
C
C---- pack sidechain code,
C            symmetry number,
C              colour,
C               residue number
C
C     into LAtm_Inc_Wrk ( eventually to LAtom_Inc_Flag)
C    LAtom_Inc_Flag(Max_Atoms)   =
C                      ISideChain_Code + 2 + ISYM*10 + ICOLR*1000 +
C                       +      IRes_Number*10000
C
      LATM_INC_WRK(NAT) = ISIDECHAIN_CODE + 2 + ISYM*10 + ICOLR*1000 +
     +                      IRES_NUMBER*10000
C
C---- added later  + jexcl*NPackAtmId (BUT only for maps)
c      INTEGER NPackAtmId
C
        IF (Map_Flag_Counter.EQ.0) GO TO 890
C
C---- write out atom to scratch file
C
        NOUT = NOUT + 1
        WRITE (ITEMP) (XB(JDO,K),K=1,3),ATM_ID_WRK(NAT),ELEMENT_SYMBOL,
     +    LATM_INC_WRK(NAT),LLAB
        IF (Lprint_Flag.GT.0) WRITE (6,FMT=6024) (XB(JDO,K),K=1,3),
     +      ATM_ID_WRK(NAT),ELEMENT_SYMBOL,LATM_INC_WRK(NAT),LLAB
 6024   FORMAT (' File',3F10.4,3X,A,1X,A,1X,3I12)
        GO TO 900
C
  890   Orthogonal_XYZ(NAT,1) = XB(JDO,1)
        Orthogonal_XYZ(NAT,2) = XB(JDO,2)
        Orthogonal_XYZ(NAT,3) = XB(JDO,3)
        Atom_Radii(NAT) = XB(JDO,4)
        IATM_TYP_WRK(NAT) = MTYPE
        LATM_LAB_FLG_WRK(NAT) = LLAB
C
C            ***********************************************
        CALL getbvalues(BB,NAT,NBTYPE,BFACT_WRK,AXISRMSWORK)
C            ***********************************************
C
  900 CONTINUE
C
      GO TO 540
  910 WRITE (6,FMT=6026) Max_Atoms
 6026 FORMAT (' More than ',I6,'  Atoms,  rest ignored')
      NAT = Max_Atoms
C
  920 CALL XYZREWD(IXYZIN(NUM_FILES))
      IF (Map_Flag_Counter.LE.0) GO TO 930
      WRITE (6,FMT=6028) NOUT
 6028 FORMAT (/,1X,I8,' atoms written to scratch file',/)
      RETURN
C
C----   reorder atoms here
C
  930 I = 9
      ISM = 0
C
C
      DO 970 ISYM = 1,NSymmetry_Ops
        ISM = ISM + 1
        ASYM = ISM
C
C
        DO 960 IAT = 9,NAT
          IF (Atom_Radii(IAT).NE.ASYM) GO TO 960
C
C
          Fractional_XYZ(I,1) = Orthogonal_XYZ(IAT,1)
          Fractional_XYZ(I,2) = Orthogonal_XYZ(IAT,2)
          Fractional_XYZ(I,3) = Orthogonal_XYZ(IAT,3)
C
C
          DO 950 NKIM = 1,3
            ATM_AXIS_RMS(NKIM,I) = AXISRMSWORK(NKIM,IAT)
            DO 940 LKIM = 1,3
              Temp_Factors(NKIM,LKIM,I) = BFACT_WRK(NKIM,LKIM,IAT)
  940       CONTINUE
  950     CONTINUE
C
C
          Atm_Identifier(I) = ATM_ID_WRK(IAT)
          IAtom_Type(I) = IATM_TYP_WRK(IAT)
          LAtom_Inc_Flag(I) = LATM_INC_WRK(IAT)
          LAtom_Label_Flag(I) = LATM_LAB_FLG_WRK(IAT)
          write(6,'(i3,2x,A10,3i15)')I,Atm_Identifier(I),
     +  IAtom_Type(I),LAtom_Inc_Flag(I),LAtom_Label_Flag(I)
          I = I + 1
  960   CONTINUE
  970 CONTINUE
C
C
      DO 980 IAT = 9,NAT
C
C---- orthonormal coordinates calculated
C
c        Orthogonal_XYZ(IAT,1) = Fractional_XYZ(IAT,1)*OR1 +
c     +                          Fractional_XYZ(IAT,2)*OR2 +
c     +                          Fractional_XYZ(IAT,3)*OR3
c        Orthogonal_XYZ(IAT,2) = Fractional_XYZ(IAT,1)*OR4 +
c     +                          Fractional_XYZ(IAT,2)*OR5 +
c     +                          Fractional_XYZ(IAT,3)*OR6
c        Orthogonal_XYZ(IAT,3) = Fractional_XYZ(IAT,1)*OR7 +
c     +                          Fractional_XYZ(IAT,2)*OR8 +
c     +                          Fractional_XYZ(IAT,3)*OR9
        Orthogonal_XYZ(IAT,1) = Fractional_XYZ(IAT,1)*ROR(1,1) +
     +                          Fractional_XYZ(IAT,2)*ROR(1,2) +
     +                          Fractional_XYZ(IAT,3)*ROR(1,3)
        Orthogonal_XYZ(IAT,2) = Fractional_XYZ(IAT,1)*ROR(2,1) +
     +                          Fractional_XYZ(IAT,2)*ROR(2,2) +
     +                          Fractional_XYZ(IAT,3)*ROR(2,3)
        Orthogonal_XYZ(IAT,3) = Fractional_XYZ(IAT,1)*ROR(3,1) +
     +                          Fractional_XYZ(IAT,2)*ROR(3,2) +
     +                          Fractional_XYZ(IAT,3)*ROR(3,3)
  980 CONTINUE
C
C
      DO 990 I = 1,Max_Atoms
        IATM_TYP_WRK(I) = 0
        Atom_Radii(I) = 0.3
  990 CONTINUE
C
C
      DO 1000 I = 1,8
        Atom_Radii(I) = 0.0
 1000 CONTINUE
C
C----    reset 1 symmetry operation = identity
C
      NSymmetry_Ops = 1
C
C
      DO 1020 I = 1,3
        DO 1010 J = 1,3
          Rotsym_Matrx(I,J,1) = 0.0
 1010   CONTINUE
        Rotsym_Matrx(I,I,1) = 1.0
 1020 CONTINUE
C
C
      WRITE (6,FMT=6030) NAT
 6030 FORMAT ('  Number of atoms to PLOT = ',I6)
      NATO = NAT
      RETURN
      END
C
C     =======================
      REAL FUNCTION arccos(XX)
C     =======================
C
C     arccos(X) IN DEGREES
C
C
C     .. Scalar Arguments ..
      REAL XX,X
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,ATAN,SIGN,SQRT
C     ..
C
C
      X = XX
C
      IF ( 1.0-ABS(X) .LT. 0.0)    X = SIGN(1.0,X)
C
C
      IF (X .LT. 0.0) THEN
        arccos = ATAN(SQRT(1.0-X*X)/X)*57.29577951 + 180.0
        GO TO 60
      ELSE IF (X .EQ. 0.0) THEN
        arccos = 90.0
        GO TO 60
      ELSE
        arccos = ATAN(SQRT(1.0-X*X)/X)*57.29577951
      END IF
C
C
   60 RETURN
C
C
      END
C
C
C
C     ===========================================
      SUBROUTINE atmoverlapchk(NATOM,ICQ,LDOUBLE)
C     ===========================================
C
Cccx lap700
C---- in general, double precision is not required
C     but in case of trouble, activate the cd comment cards
C     real*8 aov3,aov3sq,bov3,deter,pi,phi,pov3,pov3cu,qa,qc,qov2,qov2sq
C     real*8 root,td
C
ccx      REAL AOV3,AOV3SQ,BOV3,PHI,PI,POV3,POV3CU,QOV2,QOV2SQ,ROOT,TD
ccx      REAL DETER(2),QA(3,3,2),QC(3,3,2)
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Conics
      PARAMETER (Max_Conics=20)
      INTEGER Max_Quadrangles
      PARAMETER (Max_Quadrangles=30)
      INTEGER Max_Segments
      PARAMETER (Max_Segments=50)
C     ..
C     .. Scalar Arguments ..
      INTEGER ICQ,LDOUBLE,NATOM
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,CurrentCharcter_Size,
     +     Dist_to_Boundary,Ellipse_ScaleFact,Eye_Distance,
     +     Overlap_Margin,Paper_Length,Paper_Width,Plotter_Step,
     +     Retrace_Displmnt,Scal_pre_Proj,SCL
      INTEGER Iuse_persp_Labs,Lpack_Flag,NDrawing_Number,
     +        NQuads_Overlaped,Num_Bnd_Quadrangles,Num_Conics,
     +        Num_Overlaps
      LOGICAL UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL Drawing_Origin,Ellipse_Conics,Ellipse_Resolution,
     +     Overlap_Conics,Quadrangles,Quads_Overlaped,Segments,
     +     Title_Rotation,XO
      INTEGER KC,KQ
C     ..
C     .. Local Scalars ..
      DOUBLE PRECISION AOV3,AOV3SQ,BOV3,PHI,PI,POV3,POV3CU,QOV2,QOV2SQ,
     +                 ROOT,TD
      REAL T1,T2,T6,T7,T8,T9,TID
      INTEGER I,IA,IJ,IQ,IQQ,ITY,ITYPE,J,J1,J2,K,K1,K2,KA,KATOM,KB,L,
     +        LATOM
C     ..
C     .. Local Arrays ..
      DOUBLE PRECISION DETER(2),QA(3,3,2),QC(3,3,2)
      REAL DA(3,3),OVMR(2),V12(3,2),YMAX(2),YMIN(2)
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPERR,checkatmbondoverlap,conic2plotter
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ATAN,COS,DBLE,MAX,MIN,MOD,NINT,REAL,SIGN,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_OVERLAP/Num_Conics,Num_Overlaps,NQuads_Overlaped,
     +       Num_Bnd_Quadrangles,KC(Max_Conics),KQ(Max_Quadrangles),
     +       Ellipse_Conics(7,Max_Atoms),Overlap_Conics(6,Max_Conics),
     +       Quads_Overlaped(3,4,Max_Quadrangles),
     +       Quadrangles(9,2*Max_Atoms),Segments(Max_Segments,2)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      PI = 3.1415926535897932
C
C
ccx      PI = 3.141593
      ICQ = 0
      OVMR(1) = Overlap_Margin
      OVMR(2) = 0.0
C
C
      IF (NATOM.GT.Num_Conics) RETURN
C
C
      DO 10 J = 1,2
        YMIN(J) = Ellipse_Conics(2*J-1,NATOM)
        YMAX(J) = Ellipse_Conics(2*J,NATOM)
   10 CONTINUE
C
C---- set up arrays for NAtom
C
C          *****************************************
      CALL conic2plotter(Ellipse_Conics(1,NATOM),DA,
     +                   V12(1,1),OVMR(1))
C          *****************************************
C
      DO 30 J = 1,3
        DO 20 K = 1,3
          QA(J,K,1) = DBLE(DA(J,K))
   20   CONTINUE
   30 CONTINUE
C
C---- calculate cofactors and determinants
C
      DETER(1) = 0.0D0
C
C
      DO 50 J = 1,3
        J1 = MOD(J+3,3) + 1
        J2 = MOD(J+1,3) + 1
C
C
        DO 40 K = 1,3
          K1 = MOD(K+3,3) + 1
          K2 = MOD(K+1,3) + 1
          TD = QA(J1,K1,1)*QA(J2,K2,1) - QA(J1,K2,1)*QA(J2,K1,1)
          DETER(1) = QA(J,K,1)*TD + DETER(1)
          QC(J,K,1) = TD
   40   CONTINUE
   50 CONTINUE
C
C
      L = 2
C
C
      DO 310 IA = 9,Num_Conics
        IF (IA.EQ.NATOM) GO TO 310
C
C---- exact check for overlapping atoms
C
C            ***************************************
        CALL conic2plotter(Ellipse_Conics(1,IA),DA,
     +                     V12(1,2),OVMR(2))
C            ***************************************
C
        DO 70 J = 1,3
          DO 60 K = 1,3
            QA(J,K,2) = DBLE(DA(J,K))
   60     CONTINUE
   70   CONTINUE
C
C---- calculate cofactors and determinants
C
        DETER(2) = 0.0D0
C
C
        DO 90 J = 1,3
          J1 = MOD(J+3,3) + 1
          J2 = MOD(J+1,3) + 1
C
C
          DO 80 K = 1,3
            K1 = MOD(K+3,3) + 1
            K2 = MOD(K+1,3) + 1
            TD = QA(J1,K1,2)*QA(J2,K2,2) - QA(J1,K2,2)*QA(J2,K1,2)
            DETER(2) = QA(J,K,2)*TD + DETER(2)
            QC(J,K,2) = TD
   80     CONTINUE
   90   CONTINUE
C
C---- deter(l) is the determinant times 3
C     form characteristic equation and examine its roots
C
        AOV3 = 0.0
        BOV3 = 0.0
C
C
        DO 110 J = 1,3
          DO 100 K = 1,3
            AOV3 = QC(J,K,2)*QA(J,K,1) + AOV3
            BOV3 = QC(J,K,1)*QA(J,K,2) + BOV3
  100     CONTINUE
  110   CONTINUE
C
C
        AOV3 = AOV3/DETER(2)
        AOV3SQ = AOV3**2
        BOV3 = BOV3/DETER(2)
        POV3 = BOV3 - AOV3SQ
C
cd    qov2=aov3*(aov3sq-bov3*1.5d0)+deter(1)/(deter(2)*2.0d0)
C
        QOV2 = (AOV3SQ-BOV3*1.5D0)*AOV3 + DETER(1)/ (DETER(2)*2.0D0)
C
C---- check discriminant of characteristic cubic equation
C
        ITYPE = 0
        POV3CU = POV3**3
        QOV2SQ = QOV2**2
C
C
        IF ((POV3CU+QOV2SQ).GT.0.0) THEN
C
C
          IF ((POV3CU*1.00001+QOV2SQ).LE.0.0) THEN
            GO TO 150
          ELSE
            GO TO 280
          END IF
C
C
        ELSE IF ((POV3CU+QOV2SQ).LT.0.0) THEN
          IF ((POV3CU+1.00001*QOV2SQ).GE.0.0) GO TO 150
C
C---- three real roots, all different
C
          ITYPE = 1
C
C---- no intersection if a/3 and b/3 invariants are negative
C
          IF (AOV3.GE.0.0) GO TO 120
          IF (BOV3.LT.0.0) GO TO 310
C
C---- calculate one root of characteristic cubic equation
C
  120     IF (QOV2.NE.0.0) GO TO 130
C
cd--- phi=pi/2.0d0
C
          PHI = PI/2.0D0
          GO TO 140
C
cd--- 110 phi=datan(-dsqrt(-pov3cu-qov2sq)/qov2)
C
  130     PHI = ATAN(-SQRT(-POV3CU-QOV2SQ)/QOV2)
          IF (PHI.GE.0.0D0) GO TO 140
          PHI = PHI + PI
C
cd--- 130 root=2.0d0*dsqrt(-pov3)*dcos(phi/3.0d0)-aov3
C
  140     ROOT = SQRT(-POV3)*2.0D0*COS(PHI/3.0D0) - AOV3
          GO TO 170
        END IF
C
C---- three real roots, at least two are equal
C
  150   ITYPE = 2
C
C---- check signs of invariants a/3 and b/3
C
        IF (AOV3.GE.0.0) GO TO 160
        IF (BOV3.LT.0.0) GO TO 310
C
C---- calculate repeated root of cubic equation
C
cd---  150 root=dsign(dsqrt(-pov3),qov2)-aov3
C
  160   ROOT = SIGN(SQRT(-POV3),QOV2) - AOV3
C
C---- form degenerate conic (line pair which may be coincident)
C
  170   CONTINUE
C
C
        DO 190 J = 1,3
          DO 180 K = 1,3
            DA(J,K) = REAL(QA(J,K,2)*ROOT+QA(J,K,1))
  180     CONTINUE
  190   CONTINUE
C
C---- examine invariants of the degenerate conic
C
        T6 = DA(1,1)*DA(2,2)
        T7 = DA(1,2)**2
C
C---- negative denotes real intersecting line pair
C     positive denotes imaginary lines intersecting at real point
C
        IF ((T6-T7).EQ.0.0) GO TO 210
        IF ((T6-T7).GT.0.0) GO TO 200
C
C
        IF ((T6*1.0001-T7).LT.0.0) THEN
          GO TO 280
        ELSE
          GO TO 210
        END IF
C
C
  200   IF ((T6-1.0001*T7).GT.0.0) GO TO 240
  210   T8 = (DA(1,1)+DA(2,2))*DA(3,3)
        T9 = DA(1,3)**2 + DA(2,3)**2
C
C---- negative denotes real parallel line pair
C     positive denotes imaginary parallels
C     zero denotes one real line (coincident parallels)
C
        IF ((T8-T9).EQ.0.0) THEN
          GO TO 230
        ELSE IF ((T8-T9).GT.0.0) THEN
          GO TO 220
        END IF
C
C
        IF ((T8*1.0001-T9).LT.0.0) THEN
          GO TO 280
        ELSE
          GO TO 230
        END IF
C
C
  220   IF ((T8-1.0001*T9).GT.0.0) GO TO 240
C
C---- coincident line pair found for the repeated root
C
  230   ITYPE = 3
C
C---- compare areas of conics
C
  240   KA = 1
        KB = 2
        IF ((QC(3,3,KA)-QC(3,3,KB)).GE.0.0D0) GO TO 250
        KA = 2
        KB = 1
C
C---- see if one conic is inside the other conic
C
  250   T1 = 0.0
C
C
        DO 270 J = 1,3
          T2 = REAL(QA(J,3,KB))
          DO 260 K = 1,2
            T2 = REAL(QA(J,K,KB)*V12(K,KA)+T2)
  260     CONTINUE
          T1 = V12(J,KA)*T2 + T1
  270   CONTINUE
C
C---- discard if ka is outside kb
C
        IF (T1.GT.0.0) GO TO 310
        IF ((KA-1).GT.0) GO TO 280
C
C---- the overlapping atom hides the original atom
C
        ICQ = -1
        RETURN
C
C---- store overlapping atom
C
  280   ICQ = ICQ + 1
C
C                     
        IF (Num_Overlaps.GT.Max_Conics) 
C
C            ********************************************
     +  CALL CCPERR(1,' Too many overlaps gt Max_Conics')
C            ********************************************
C
        Num_Overlaps = Num_Overlaps + 1
        IJ = 1
C
C
        DO 300 I = 1,3
          DO 290 J = I,3
            Overlap_Conics(IJ,Num_Overlaps) = REAL(QA(I,J,2))
            IJ = IJ + 1
  290     CONTINUE
  300   CONTINUE
C
C
        KC(Num_Overlaps) = IA
  310 CONTINUE
C
C---- second part of subroutine checks for bonds over the atom
C
      IF (Num_Bnd_Quadrangles.LE.0) GO TO 350
      ITY = 0
C
C---- rough check for overlapping bonds
C
      DO 340 IQ = 1,Num_Bnd_Quadrangles
        TID = Quadrangles(9,IQ)
        KATOM = NINT(TID/REAL(Max_Atoms))
        LATOM = NINT(MOD(TID,REAL(Max_Atoms)))
        IF (NATOM.EQ.LATOM) GO TO 340
C
C
        DO 320 J = 1,2
          IF ((YMAX(J)-MIN(Quadrangles(J,IQ),Quadrangles(J+2,IQ),
     +        Quadrangles(J+4,IQ),Quadrangles(J+6,IQ))).LE.
     +        0.0) GO TO 340
          IF ((YMIN(J)-MAX(Quadrangles(J,IQ),Quadrangles(J+2,IQ),
     +        Quadrangles(J+4,IQ),Quadrangles(J+6,IQ))).GE.
     +        0.0) GO TO 340
  320   CONTINUE
C
C---- exact check for overlapping bonds
C
        ITY = ITY - 1
        IQQ = 0
C
C            ************************************
        CALL checkatmbondoverlap(IQ,NATOM,IQQ,ITY)
C            ************************************
C
        IF (IQQ.GE.0) GO TO 330
        ICQ = -1
        RETURN
  330   ICQ = ICQ + 1
        IF (NQuads_Overlaped.GT.Max_Quadrangles) GO TO 350
  340 CONTINUE
  350 RETURN
      END
C
C
C
C     =================
      SUBROUTINE atomin
C     =================
C
C
C
C---- set up atom arrays for plotting
C     if plotting into map, read atoms from scratch file 10
C     section numbers for this group of sections are ksec1 to ksec2
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_BondType
      PARAMETER (Max_BondType=20)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER Max_Labels
      PARAMETER (Max_Labels=100)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER Max_Radii
      PARAMETER (Max_Radii=10)
      INTEGER Max_Symmetry
      PARAMETER (Max_Symmetry=192)
      INTEGER NPackAtmID
      PARAMETER (NPackAtmID=10000000)
      INTEGER Max_Files
      PARAMETER (Max_Files=100)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Charcter_Size,
     +     CurrentCharcter_Size,D001,D010,D100,Dist_to_Boundary,
     +     Drawing_Size,Ellipse_ScaleFact,Eye_Distance,
     +     ROR(4,4),RFR(4,4),
     +     Overlap_Margin,Paper_Length,Paper_Width,Perspective_Distance,
     +     Picture_Scale,Plotter_Step,Retrace_Displmnt,S,Scal_pre_Proj,
     +     SCL,Shadelight_X,Shadelight_Y,Stereo_Separation
      INTEGER IERR,Invert_Map_Flag,ITEMP,Iuse_persp_Labs,KSEC1,KSEC2,
     +        Label_atoms_Flag,Lcurrent_Colour,Ldraw_cell_Flag,
     +        LimitsSet_Flag,Ljoin_Flag,Lmap3d_Flag,Lmap_ModeFirst,
     +        Lorthogonalisation_Code,Lpack_Flag,Lpaper_Setting,
     +        Lperspective_Flag,Lplot_colour_Stereo,Lposition_Flag,
     +        Lprint_Flag,LSetView_Flag,LShade_Flag,LSolid_Flag,
     +        LStereo_Flag,LSuperimpose_Flag,LTaper_Flag,LTrace_Flag,
     +        Map_Counter,Map_Flag_Counter,Map_pts_U,Map_pts_V,Msecst,
     +        MU,MV,MW,NAT,NATO,Natoms_plusCell,NC,NC1,Ncopies,
     +        NDrawing_Number,NFirst_Section,NLast_Section,Npicture,
     +        Npts_U_compressed,Npts_V_compressed,Nsections_in_Group,
     +        Nspecified_Bonds,NstpMapGrpSect,NSymmetry_Ops,
     +        NumHbondRadii,NumLinkRadii,NumPlotRadii,Num_Labtxt_input,
     +        Num_LinkSets,Num_Moles,Num_Posit_Labels
      LOGICAL Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
      CHARACTER Iden_Title*80
C     ..
C     .. Arrays in Common ..
      REAL AID,AQ,Atm_Hbond_Radii,Atm_Link_Radii,Atm_Plot_Radii,
     +     Atom_Radii,Bound_Edge,Bound_Limits,Contour_Levels,
     +     Contour_Threshold,Def_LnkBond,Drawing_Origin,
     +     Ellipse_Resolution,Fractional_XYZ,OrtCell,OrtConts,
     +     Orthogonal_XYZ,Plotting_XYZ,Positioned_Labels,Rotsym_Matrx,
     +     RP,Title_Rotation,TransSym_Vect,ViewMatrix,XABC,XCLANG,
     +     XLABEL,XO,YLABEL
      INTEGER IAtom_Connectivity,IAtom_Type,IConn_BndType,IResid_BndTyp,
     +        ISpecified_Bond_Params,ISym_BndType,LAtom_Inc_Flag,
     +        IXYZIN,LAtom_Label_Flag,LHbond_Flag,LRadi_Atm_typ,
     +        LRadi_Hbond_typ,LRadi_Plt_typ,Map_def_Restrictions,
     +        Map_LineTypes,Map_UVW_Limits,Map_XYZ_Limits,MOL,NSECC,
     +        Num_Contours
      CHARACTER Atm_Lab_Type*2,Link_Atm_Names*2,Atm_Identifier*10,
     +          Atm_Label*10,Atm_Lab_Txt*10,Map_Title*80
C     ..
C     .. Local Scalars ..
      REAL ACELL,ALF,BCELL,BET,CCELL,CENSLB,DDX,DX,GAM,RL,W,WN,XX,Y,Z
      INTEGER I,IEND,IINCL,ISYM,ITYPCA,IXX,J,JDO,JDO210,JDO300,JLOOP,
     +        JSORT,JXX,K,KIM,LCTYP1,LCTYP2,LINCL,LL,LLAB,LRES,MRES,NIN,
     +        NRES,NXX,LPERM
      CHARACTER LCWORK*1,EL_ITYP*2,NAMEBB*10
C     ..
C     .. Local Arrays ..
      REAL XB(3)
      INTEGER IATM_TYP_WRK(Max_Atoms),JSYM(Max_Atoms),
     +        LATM_INC_WRK(Max_Atoms),LATM_LAB_FLG_WRK(Max_Atoms)
      LOGICAL LINKCA(2,2)
      CHARACTER ATM_ID_WRK(Max_Atoms)*10
C     ..
C     .. External Subroutines ..
      EXTERNAL getatomconnections,getatomid,gethbondconnections,radset,
     +         setboundarylimits
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,ICHAR,MAX,MOD,REAL
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_LABCHR/Atm_Lab_Txt(Max_Labels),
     +       Atm_Lab_Type(Max_Labels),Atm_Label(Max_Labels),
     +       Atm_Identifier(Max_Atoms),Iden_Title
      COMMON /com_LABELS/Num_Posit_Labels,Num_Labtxt_input,
     +       XLABEL(Max_Atoms),YLABEL(Max_Atoms),
     +       Positioned_Labels(2,Max_Labels)
      COMMON /com_LIMITS/Bound_Limits(3),Bound_Edge(3)
      COMMON /com_LINK/Num_LinkSets,IResid_BndTyp(Max_Atoms),
     +       ISym_BndType(Max_Symmetry),IConn_BndType(Max_BondType),
     +       Def_LnkBond(2)
      COMMON /com_LNKCHR/Link_Atm_Names(2,2)
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
      COMMON /com_RADII/NumLinkRadii,NumPlotRadii,NumHbondRadii,
     +       Atm_Link_Radii(Max_Radii),Atm_Plot_Radii(Max_Radii),
     +       Atm_Hbond_Radii(Max_Radii),LRadi_Atm_typ(Max_Radii),
     +       LRadi_Plt_typ(Max_Radii),LRadi_Hbond_typ(Max_Radii),
     +       LHbond_Flag(2)
      COMMON /com_SECTIONS/NFirst_Section,NLast_Section,
     +       Nsections_in_Group,NstpMapGrpSect,KSEC1,KSEC2,Msecst,
     +       NSECC(2)
      COMMON /com_SYMMETRY/NSymmetry_Ops,Rotsym_Matrx(3,3,Max_Symmetry),
     +       TransSym_Vect(3,Max_Symmetry)
      COMMON /UNITS/ ITEMP,IXYZIN(Max_Files)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      ACELL = XABC(1)
      BCELL = XABC(2)
      CCELL = XABC(3)
      ALF = XCLANG(1)
      BET = XCLANG(2)
      GAM = XCLANG(3)
C
C---- if no map, most arrays are already set up
C
      IF (Map_Flag_Counter.LE.0) GO TO 320
C
C---- Set LINKCA .true. if this atom is CA
C
      DO 20 I = 1,2
        DO 10 J = 1,Num_LinkSets
          LINKCA(I,J) = .TRUE.
          IF (Link_Atm_Names(I,J).NE.'CA') LINKCA(I,J) = .FALSE.
   10   CONTINUE
   20 CONTINUE
C
C---- Get atom type for CA
C
      LCTYP1 = ICHAR('C')
      LCTYP2 = ICHAR('A')
      ITYPCA = -ABS(LCTYP1**2+LCTYP2**3)
C
C---- set up tests along section axis in fractional coordinates
C
      WN = Map_XYZ_Limits(MW)
C
C---- centre of map slab
C
      CENSLB = REAL(KSEC1+KSEC2)*0.5/WN
C
C---- half-thickness of slab along cell axis
C
      W = ABS((REAL(KSEC2-KSEC1)*0.5+0.5)/WN)
      Bound_Limits(MW) = CENSLB - W
      Bound_Edge(MW) = CENSLB + W
C
C---- set cell corners to bounds values as 1st 8 atoms
C
C          *********************************************************
      CALL setboundarylimits(Fractional_XYZ,Bound_Limits,Bound_Edge)
C          *********************************************************
C
C---- return if no atoms to be plotted
C
      IF (Label_atoms_Flag.EQ.0) RETURN
C
C---- get maximum link radius
C
      RL = 0.0
C
C
      DO 30 JDO = 1,NumLinkRadii
        RL = MAX(RL,Atm_Link_Radii(JDO))
   30 CONTINUE
C
C---- allow for link atom radius Atm_Link_Radii
C
      DO 40 JDO = 1,Num_LinkSets
        RL = MAX(RL,Def_LnkBond(JDO)*0.5)
   40 CONTINUE
C
C---- correct for skew axes and convert to fractional
C   2/6/89 : divide by 33 element of matrix R**-1 AQ
C
C      RL = RL/ABS(ViewMatrix(1,3)*AQ(1,3)+ViewMatrix(2,3)*AQ(2,3)+
C    +     ViewMatrix(3,3)*AQ(3,3))
      RL = RL/ABS(AQ(3,3))
C
C---- 1/2 slab thickness + twice radius
C
      RL = RL + RL + W
      LRES = 9
      NAT = 8
      MRES = -1000
      REWIND ITEMP
      IEND = 0
C
C---- read atom
C in akwprotin the write is
C
C
   50 READ (ITEMP,END=240) (XB(JLOOP),JLOOP=1,3),NAMEBB,EL_ITYP,IINCL,
     +  LLAB
C
C---- unpack residue number
C
      IF (IINCL.EQ.0) GO TO 50
      NRES = IINCL/400
      IF (MRES.EQ.-1000) GO TO 60
C
C---- same residue ?
C
      IF (NRES.NE.MRES) GO TO 80
C
C---- store this atom in temporary arrays
C
   60 NAT = NAT + 1
      IF (NAT.GT.Max_Atoms) GO TO 250
C
C
      DO 70 JDO = 1,3
        Orthogonal_XYZ(NAT,JDO) = XB(JDO)
   70 CONTINUE
C
C
      ATM_ID_WRK(NAT) = NAMEBB
      LATM_INC_WRK(NAT) = IINCL
      LATM_LAB_FLG_WRK(NAT) = LLAB
C
C          **********************
      CALL getatomid(KIM,EL_ITYP)
C          **********************
C
      IATM_TYP_WRK(NAT) = KIM
      MRES = NRES
      GO TO 50
C
C---- process last residue, include whole residue if any atom is
C     within bounds
C
   80 LINCL = -1
C
C
      DO 110 JDO = LRES,NAT
C
C---- check if atom lies within map slab (included in plotting)
C     or within bonding distance of slab (excluded from plotting,
C     but kept if this residue  has at least one atom in slab)
C     no need to check u or v directions
C
        DDX = Orthogonal_XYZ(JDO,MW) - CENSLB
        DX = ABS(DDX)
        IF (DX.GT.RL) GO TO 90
        IF (DX.GT.W) GO TO 100
C
C---- this atom included, set flag
C
        LINCL = +1
        GO TO 110
C
C---- omit atom, too far from slab
C
   90   LATM_INC_WRK(JDO) = 0
        GO TO 110
C
C---- exclude atom
C
  100   J = 0
        IF (DDX.GT.0.0) J = 1
C
C---- j (=jexcl) flags which margin the excluded atom is in
C
        LATM_INC_WRK(JDO) = LATM_INC_WRK(JDO) + J*NPackAtmID
        LATM_INC_WRK(JDO) = -LATM_INC_WRK(JDO)
        IF (LINCL.LE.0) LINCL = 0
  110 CONTINUE
C
C---- residue to be included ?
C
      IF (LINCL.LT.0) GO TO 230
C
C---- include residue, but remove any atoms more than bonding
C     distance from slab
C          (flagged with LAtm_Inc_Wrk=0)
C
      JSORT = LRES - 1
C
C
      DO 210 JDO210 = LRES,NAT
        IF (LATM_INC_WRK(JDO210).EQ.0) GO TO 210
C
C---- if no atoms within map slab, keep only link atoms
C
        IF (LINCL.GT.0) GO TO 190
C
C
        DO 180 K = 1,2
          DO 170 LL = 1,Num_LinkSets
            IF (LINKCA(K,LL)) GO TO 160
C
C---- 1st character
C
            IF (Link_Atm_Names(K,LL) (1:1).NE.
     +          ATM_ID_WRK(JDO210) (1:1)) GO TO 170
C
C---- if 2nd character of link atom name is blank,
C     can match with digit
C
            IF (Link_Atm_Names(K,LL) (1:1).NE.' ') GO TO 150
C
C
            DO 120 JDO = 1,10
              WRITE (LCWORK,FMT=6000) JDO
 6000         FORMAT (I1)
              IF (Atm_Identifier(JDO210) (2:2).EQ.LCWORK) GO TO 130
  120       CONTINUE
C
C
            LCWORK = ATM_ID_WRK(JDO210) (2:2)
            GO TO 140
  130       LCWORK = ' '
  140       CONTINUE
  150       IF (LCWORK.EQ.Link_Atm_Names(K,LL) (2:2)) GO TO 190
            GO TO 170
C
C--- This link atom is CA, test atom type
C
  160       IF (IATM_TYP_WRK(JDO210).EQ.ITYPCA) GO TO 190
C
  170     CONTINUE
  180   CONTINUE
C
C---- not link atom, reject
C
        GO TO 210
C
  190   JSORT = JSORT + 1
C
C
        DO 200 K = 1,3
          Orthogonal_XYZ(JSORT,K) = Orthogonal_XYZ(JDO210,K)
  200   CONTINUE
C
C
        ATM_ID_WRK(JSORT) = ATM_ID_WRK(JDO210)
        LATM_INC_WRK(JSORT) = LATM_INC_WRK(JDO210)
        LATM_LAB_FLG_WRK(JSORT) = LATM_LAB_FLG_WRK(JDO210)
        IATM_TYP_WRK(JSORT) = IATM_TYP_WRK(JDO210)
C
C---- unpack symmetry number into jsym
C    LAtom_Inc_Flag(Max_Atoms)   =
C                      ISideChain_Code + 2 + ISYM*10 + ICOLR*1000 +
C                       +      IRes_Number*10000
C
C
        IXX = ABS(LATM_INC_WRK(JSORT))
        JXX = IXX/NPackAtmID
        IXX = IXX - JXX*NPackAtmID
        NXX = IXX/10000
        IXX = IXX - NXX*10000
        JXX = IXX/1000
        IXX = IXX - JXX*1000
        JSYM(JSORT) = IXX/10
  210 CONTINUE
C
C
      NAT = JSORT
      LRES = NAT + 1
  220 CONTINUE
C
C
      IF (IEND.LE.0) THEN
        GO TO 60
      ELSE
        GO TO 260
      END IF
C
C---- exclude
C
  230 NAT = LRES - 1
      GO TO 220
C
C---- end of file
C
  240 IEND = 1
      GO TO 80
C
C---- array full
C
  250 WRITE (6,FMT=6002) Max_Atoms,KSEC1
 6002 FORMAT (' More than ',I6,' atoms in section ',I5,', rest ignored')
      NAT = Max_Atoms
      IEND = 1
      GO TO 80
C
C---- reorder atoms into groups of same symmetry number
C
  260 IF (NAT.LT.9) GO TO 350
      I = 8
C
C
      DO 290 ISYM = 1,NSymmetry_Ops
        DO 280 J = 9,NAT
          IF (JSYM(J).NE.ISYM) GO TO 280
          I = I + 1
C
C
          DO 270 K = 1,3
            Fractional_XYZ(I,K) = Orthogonal_XYZ(J,K)
  270     CONTINUE
C
C
          IAtom_Type(I) = IATM_TYP_WRK(J)
          LAtom_Inc_Flag(I) = LATM_INC_WRK(J)
          LAtom_Label_Flag(I) = LATM_LAB_FLG_WRK(J)
          Atm_Identifier(I) = ATM_ID_WRK(J)
  280   CONTINUE
  290 CONTINUE
C
      IF (Lprint_Flag.NE.0) WRITE (6,FMT=6004)
 6004 FORMAT (' Number       Fractional',20X,'Orthogonal',17X,'Name',5X,
     +       'ITYP',8X,'LAtom_Inc_Flag',8X,'LAtom_Label_Flag',/)
C
C---- convert to orthogonal axes
C
      NIN = 0
C
C
      DO 300 JDO300 = 1,NAT
        IF (LAtom_Inc_Flag(JDO300).GT.0) NIN = NIN + 1
        XX = Fractional_XYZ(JDO300,1)
        Y = Fractional_XYZ(JDO300,2)
        Z = Fractional_XYZ(JDO300,3)
c        Orthogonal_XYZ(JDO300,1) = OR1*XX + OR2*Y + OR3*Z
c        Orthogonal_XYZ(JDO300,2) = OR4*XX + OR5*Y + OR6*Z
c        Orthogonal_XYZ(JDO300,3) = OR7*XX + OR8*Y + OR9*Z
        Orthogonal_XYZ(JDO300,1) =ROR(1,1)*XX +ROR(1,2)*Y +ROR(1,3)*Z
        Orthogonal_XYZ(JDO300,2) =ROR(2,1)*XX +ROR(2,2)*Y +ROR(2,3)*Z
        Orthogonal_XYZ(JDO300,3) =ROR(3,1)*XX +ROR(3,2)*Y +ROR(3,3)*Z
C
C
        IF (Lprint_Flag.NE.0) WRITE (6,FMT=6006) JDO300,XX,Y,Z,
     +      (Orthogonal_XYZ(JDO300,K),K=1,3),Atm_Identifier(JDO300),
     +      IAtom_Type(JDO300),LAtom_Inc_Flag(JDO300),
     +      LAtom_Label_Flag(JDO300)
 6006   FORMAT (1X,I6,3F10.4,3F10.2,3X,A,3I12)
  300 CONTINUE
C
      NC = 12
C
      WRITE (6,FMT=6008) NAT,KSEC1,KSEC2,NIN
 6008 FORMAT (5X,I8,' atoms selected in sections',I6,' to ',I6,/8X,I8,
     +       ' of these atoms included in plotting')
C
C---- hbonds ?
C
      IF (LHbond_Flag(1).GE.-1) THEN
C
C            *****************************************************
        CALL radset(NumHbondRadii,Atm_Hbond_Radii,LRadi_Hbond_typ)
c Atom_Radii set then used
        CALL gethbondconnections(LHbond_Flag)
C            *****************************************************
C 
      END IF
C
C---- set link radii
C
C          ************************************************
      CALL radset(NumLinkRadii,Atm_Link_Radii,LRadi_Atm_typ)
C          ************************************************
C
C  common Atom_Radii reset then used in getatomconnections
C
C---- join atoms
C
C                          ************************************
      IF (Ljoin_Flag.GT.0) CALL getatomconnections(Lprint_Flag)
C                          ***********************************
C
C---- set plotting radii
C
C          *************************************************
      CALL radset(NumPlotRadii,Atm_Plot_Radii,LRadi_Plt_typ)
C  common Atom_Radii reset
C          *************************************************
C
C---- pick up any specified label positions
C
  320 IF (Num_Posit_Labels.LE.0) GO TO 340
C
C
      DO 330 I = 9,NAT
        K = MOD(LAtom_Label_Flag(I),100)
        IF (K.LE.1) GO TO 330
C
C---- this atom has a specified label position
C
        J = K - 1
        XLABEL(I) = Positioned_Labels(1,J)
        YLABEL(I) = Positioned_Labels(2,J)
  330 CONTINUE
C
  340 CONTINUE
      RETURN
C
  350 WRITE (6,FMT=6010) KSEC1,KSEC2
 6010 FORMAT (' No atoms in sections',2I5,/)
      NAT = 8
      RETURN
      END
C
C
C
C     ====================================================
      SUBROUTINE bondin(IATOM,JATOM,Plotting_XYZ,RP,NOBSC,LTaper_Flag,
     +                  RBO,NLI,IOBSC)
C     ====================================================
C
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER MAX_HIDDEN_LINES
      PARAMETER (MAX_HIDDEN_LINES=2000)
C     ..
C     .. Scalar Arguments ..
      REAL RBO
      INTEGER IATOM,JATOM,LTaper_Flag,NLI,NOBSC
C     ..
C     .. Array Arguments ..
      REAL Plotting_XYZ(3,Max_Atoms),RP(*)
      INTEGER IOBSC(MAX_HIDDEN_LINES)
C     ..
C     .. Local Scalars ..
      REAL BL,BM,COST,CX,CY,D,D1,D2,D3,DELT,DELX,DELY,DELZ,DINC,DL,DPL,
     +     DX,DXK,DXL,DXY,DXYZ,DY,DYK,DYL,DZ,E,Perspective_Distance,
     +     PLINC,Plotter_Step,RB1,RB2,RI,RISQ,RK,RN,RNSQ,SINT,T,X,XA,
     +     XEND,XI,XN,XSTART,Y,YA,YEND,YI,YN,YSTART,ZI,ZN,ZSTART
      INTEGER I,IDOWN,IDRAW,K,L,N,NBLINE,NLIN
C     ..
C     .. Local Arrays ..
      REAL DC(3),P(3),Q(3),U1(3),U2(3),U3(3),ZAX(3)
C     ..
C     .. External Subroutines ..
      EXTERNAL drawlines,normvectproduct
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,COS,SIN,SQRT
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      IF (NLI.LE.0) GO TO 160
C
C
      Plotter_Step = 0.1
      Perspective_Distance = 600.0
      DPL = Perspective_Distance
      PLINC = Plotter_Step
      I = IATOM
      N = JATOM
C
C
      XI = Plotting_XYZ(1,I)
      YI = Plotting_XYZ(2,I)
      ZI = Plotting_XYZ(3,I)
      XN = Plotting_XYZ(1,N)
      YN = Plotting_XYZ(2,N)
      ZN = Plotting_XYZ(3,N)
C
C
      RI = RP(I)
      RN = RP(N)
      IDOWN = 0
      NBLINE = ABS(NLI)
      DX = XN - XI
      DY = YN - YI
      DZ = ZN - ZI
      DXYZ = SQRT(DX*DX+DY*DY+DZ*DZ)
      IF (ABS(DXYZ).LT.0.00001) RETURN
      RISQ = RI*RI
      RNSQ = RN*RN
C
C
      IF (NLI.LE.1 .OR. RBO.LT.0.001) GO TO 40
C
C---- Solid bonds are to be drawn.
C         get bond centre line direction-cosines.
C
      U3(1) = DX/DXYZ
      U3(2) = DY/DXYZ
      U3(3) = DZ/DXYZ
C
C
      IF (ABS(U3(3)).GT.0.9999) GO TO 160
      ZAX(1) = 0.0
      ZAX(2) = 0.0
      ZAX(3) = 1.0
C
C---- set up coord system for the semi-circle cross-section of bond.
C
C          **************************
      CALL normvectproduct(ZAX,U3,U1)
      CALL normvectproduct(U3,U1,U2)
C          *************************
C
C---- get bond radius at atoms i and n.
C
      RB1 = RBO*DPL/ (DPL-ZI)
      RB2 = RBO*DPL/ (DPL-ZN)
      IF (LTaper_Flag.GT.0) RB2 = RB1*LTaper_Flag* (DPL-ZI)/ (DPL-ZN)
      IF (RB1.GT.RI) RB1 = RI - 0.001
      IF (RB2.GT.RN) RB2 = RN - 0.001
      IF (RB1.LE.Plotter_Step) RB1 = Plotter_Step
      IF (RB2.LE.Plotter_Step) RB2 = Plotter_Step
      NLIN = 0
      T = 0
C
C---- set up points on semicircle at centres of atoms.
C
   10 COST = COS(T)
      SINT = SIN(T)
      CX = RB1*COST
      CY = RB1*SINT
      P(1) = U1(1)*CX + U2(1)*CY
      P(2) = U1(2)*CX + U2(2)*CY
      P(3) = U1(3)*CX + U2(3)*CY
      P(1) = P(1) + XI
      P(2) = P(2) + YI
      P(3) = P(3) + ZI
C
C---- and again for point q.
C
      CX = RB2*COST
      CY = RB2*SINT
      Q(1) = U1(1)*CX + U2(1)*CY
      Q(2) = U1(2)*CX + U2(2)*CY
      Q(3) = U1(3)*CX + U2(3)*CY
      Q(1) = Q(1) + XN
      Q(2) = Q(2) + YN
      Q(3) = Q(3) + ZN
C
C---- get direction cosines for this line pq.
C
      D1 = Q(1) - P(1)
      D2 = Q(2) - P(2)
      D3 = Q(3) - P(3)
      D = SQRT(D1*D1+D2*D2+D3*D3)
      DC(1) = D1/D
      DC(2) = D2/D
      DC(3) = D3/D
      DELX = DC(1)*PLINC
      DELY = DC(2)*PLINC
      DELZ = DC(3)*PLINC
C
C----  now find point where line emerges from srhere of aton i.
C
      E = RISQ - RB1*RB1
      IF (E.LT.0.0) E = 0.0
      E = SQRT(E)
      XSTART = DC(1)*E + P(1)
      YSTART = DC(2)*E + P(2)
      ZSTART = DC(3)*E + P(3)
C
C---- test if this point is obscured by circle i.
C
      IF (ZSTART.GT.ZI) GO TO 30
C
C---- obscured therefore move along line to
C     circumferencs of circle i.
C
   20 XSTART = XSTART + DELX
      YSTART = YSTART + DELY
      ZSTART = ZSTART + DELZ
      DX = XSTART - XI
      DY = YSTART - YI
      IF (DX*DX+DY*DY.LT.RISQ) GO TO 20
C
C---- now get the end point of the line.
C      ( intersection with sphere n.)
C
   30 E = RNSQ - RB2*RB2
      IF (E.LT.0.0) E = 0.0
      E = SQRT(E)
      XEND = Q(1) - DC(1)*E
      YEND = Q(2) - DC(2)*E
      GO TO 60
C
C---- single line bond to be drawn......
C
   40 NLIN = 0
C
C---- get direction cosines of bond in xy plane  bl,bm
C
      DXY = SQRT(DX*DX+DY*DY)
      BL = 0.0
      BM = 0.0
      IF (ABS(DXY).LT.0.000001) GO TO 50
      BL = DX/DXY
      BM = DY/DXY
C
C---- get the starting point which is on the
C     circumference of circle i
C
   50 XSTART = BL*RI + XI
      YSTART = BM*RI + YI
C
C---- now get the end point for the bond
C
      XEND = XN - RN*DX/DXYZ
      YEND = YN - RN*DY/DXYZ
      DELX = BL*PLINC
      DELY = BM*PLINC
C
C---- start plotting the line xstart-xend...........
C
   60 NLIN = NLIN + 1
C
C---- we go to 300 if the bond is completely obscured by the atom i
C
      IF ((XEND-XI)**2+ (YEND-YI)**2.LE.RISQ) GO TO 150
      DXL = XEND - XSTART
      DYL = YEND - YSTART
      DL = SQRT(DXL*DXL+DYL*DYL)
      DINC = SQRT(DELX*DELX+DELY*DELY)
      X = XSTART
      Y = YSTART
      IF (NOBSC.EQ.0) GO TO 140
C
C---- start of testing for obscuring by atoms k
C
   70 IF (NOBSC.EQ.0) GO TO 90
C
C
      DO 80 L = 1,NOBSC
        K = IOBSC(L)
        RK = RP(K)
        DXK = X - Plotting_XYZ(1,K)
        IF (ABS(DXK).GT.RK) GO TO 80
        DYK = Y - Plotting_XYZ(2,K)
        IF (ABS(DYK).GT.RK) GO TO 80
        IF (DXK*DXK+DYK*DYK.LT.RK*RK) GO TO 100
   80 CONTINUE
C
C----  no obscuring .  go ahead with this step
C      IDRAW signals whether to draw the line at
C            the current poin  0/1 for no
C      IDOWN records the apparent status
C            of the pen = 0/1 for up/down
C
   90 IDRAW = 1
      GO TO 110
C
C---- lift pen when obscuring found.
C
  100 IDRAW = 0
  110 IF (DL.LE.0.0) IDRAW = 0
      IF (IDRAW.EQ.0) GO TO 120
      IF (IDOWN.EQ.1) GO TO 130
C
C---- note the starting point of a
C     line segment at this point (xa,ya).
C
      XA = X
      YA = Y
      IDOWN = 1
      GO TO 130
C
C---- obscuring occurs at this point.
C     draw line segment (xa,ya) to ( x,y )
C     then lift the pen.
C
  120 IF (IDOWN.EQ.0) GO TO 130
C
C          ********************
      CALL drawlines(XA,YA,X,Y)
C          ********************
C
      IDOWN = 0
C
C---- test for end of bond line.
C
  130 IF (DL.LE.0.0) GO TO 150
      X = X + DELX
      Y = Y + DELY
      DL = DL - DINC
      GO TO 70
C
C---- simple case of there being no obscuring atoms.
C     draw line i to n
C
C          **********************************
  140 CALL drawlines(XSTART,YSTART,XEND,YEND)
C          **********************************
C
  150 CONTINUE
      IF (NLI.LE.1 .OR. RBO.LE.0.001) GO TO 160
      IF (NLIN.GE.NBLINE) GO TO 160
C
C---- draw next line.
C
      DELT = 3.14159/ (NBLINE-1)
      T = T + DELT
      GO TO 10
  160 RETURN
      END
C
C
C
C     ===================================================
      SUBROUTINE bondoverlapchk(KATOM,LATOM,ICQ,DP,INSTR)
C     ===================================================
C
C---- subroutine checks for atoms and bonds overlapping a bond
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Conics
      PARAMETER (Max_Conics=20)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER Max_Quadrangles
      PARAMETER (Max_Quadrangles=30)
      INTEGER Max_Segments
      PARAMETER (Max_Segments=50)
C     ..
C     .. Scalar Arguments ..
      INTEGER ICQ,INSTR,KATOM,LATOM
C     ..
C     .. Array Arguments ..
      REAL DP(2,130)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Charcter_Size,
     +     CurrentCharcter_Size,Dist_to_Boundary,Drawing_Size,
     +     Ellipse_ScaleFact,Eye_Distance,Overlap_Margin,Paper_Length,
     +     Paper_Width,Perspective_Distance,Picture_Scale,Plotter_Step,
     +     Retrace_Displmnt,S,Scal_pre_Proj,SCL,Shadelight_X,
     +     Shadelight_Y,Stereo_Separation
      INTEGER Iuse_persp_Labs,Lcurrent_Colour,Ldraw_cell_Flag,
     +        Lpack_Flag,Lpaper_Setting,Lperspective_Flag,
     +        Lplot_colour_Stereo,Lposition_Flag,Lprint_Flag,
     +        LShade_Flag,LSolid_Flag,LStereo_Flag,LSuperimpose_Flag,
     +        LTaper_Flag,LTrace_Flag,Natoms_plusCell,Ncopies,
     +        NDrawing_Number,NQuads_Overlaped,Nspecified_Bonds,
     +        Num_Bnd_Quadrangles,Num_Conics,Num_Moles,Num_Overlaps
      LOGICAL Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL Base_Vect_Refsys,Base_Vect_Wrksys,BBB,Drawing_Origin,
     +     Ellipse_Conics,Ellipse_Resolution,G_Metric_Tensor,
     +     Overlap_Conics,P1_to_Refsys,P1_to_Wrksys,Plotting_XYZ,
     +     Principal_axis_rms,Prin_Angstrom_Vectors,Prin_Axis_Col_Vects,
     +     Q,Quadrangles,Quads_Overlaped,Recip_Metric_Tensor,RP,
     +     Segments,Sorted_xyz,Title_Rotation,Title_RotnTrans,
     +     ViewMatrix,XO
      INTEGER ISpecified_Bond_Params,KC,KQ,KSortPointer,MOL
C     ..
C     .. Local Scalars ..
      REAL T1,T2,T3,T4,T5,T6,TID1,TID2,TOL
      INTEGER IA,IB,IQ,IQQ,ITY,ITYPE,J,JDO,K,K0,K1,KWORK,L1,LDO,LWORK,
     +        MATOM,MWORK,NATOM
C     ..
C     .. Local Arrays ..
      REAL FL(4,4),QUA(3,4),V1(4),V2(3),V3(3),V4(3),V5(3),VUE(3),Y1(2),
     +     Y2(2),YMAX(2),YMIN(2)
C     ..
C     .. External Functions ..
      REAL vectbyvecttranspose
      EXTERNAL vectbyvecttranspose
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPERR,checkatmbondoverlap,getunitvectors,
     +         vectordifference,vectorproduct
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MAX,MIN,MOD,NINT,REAL,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_OVERLAP/Num_Conics,Num_Overlaps,NQuads_Overlaped,
     +       Num_Bnd_Quadrangles,KC(Max_Conics),KQ(Max_Quadrangles),
     +       Ellipse_Conics(7,Max_Atoms),Overlap_Conics(6,Max_Conics),
     +       Quads_Overlaped(3,4,Max_Quadrangles),
     +       Quadrangles(9,2*Max_Atoms),Segments(Max_Segments,2)
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      IQ = 0
      ICQ = 0
C
C
      IF (KATOM*LATOM.LE.0) RETURN
      TID1 = REAL(KATOM*Max_Atoms) + REAL(LATOM)
      IF (Num_Conics.LE.8) RETURN
C
C
      IF (INSTR.EQ.822) THEN
C
C---- part 1, called from bond, stores bond outline information
C
        IF (Num_Bnd_Quadrangles.GT. (2*Max_Atoms-1)) 
C
C            ***************************************
     +  CALL CCPERR(1,' Too many bond Quadrangles ')
C            ***************************************
C
        Num_Bnd_Quadrangles = Num_Bnd_Quadrangles + 1
C
C---- calculate overlap margin for bond quadrangle
C
        T1 = 0.0
        T2 = 0.0
C
C
        DO 10 JDO = 1,2
          Y1(JDO) = DP(JDO,1) - DP(JDO,65)
          Y2(JDO) = DP(JDO,2) - DP(JDO,66)
          T1 = Y1(JDO)**2 + T1
          T2 = Y2(JDO)**2 + T2
   10   CONTINUE
C
C
        IF (T1*T2.GT.0.0) GO TO 20
        T1 = 0.0
        T2 = 0.0
        GO TO 30
   20   T1 = Overlap_Margin/SQRT(T1)
        T2 = Overlap_Margin/SQRT(T2)
C
C---- store bond quadrangle
C
   30   CONTINUE
C
C
        DO 40 JDO = 1,2
          Y1(JDO) = Y1(JDO)*T1
          Y2(JDO) = Y2(JDO)*T2
C
C
          Quadrangles(JDO,Num_Bnd_Quadrangles) = DP(JDO,1) + Y1(JDO)
          Quadrangles(JDO+2,Num_Bnd_Quadrangles) = DP(JDO,2) + Y2(JDO)
          Quadrangles(JDO+4,Num_Bnd_Quadrangles) = DP(JDO,66) - Y2(JDO)
          Quadrangles(JDO+6,Num_Bnd_Quadrangles) = DP(JDO,65) - Y1(JDO)
   40   CONTINUE
C
C
        Quadrangles(9,Num_Bnd_Quadrangles) = TID1
        RETURN
      END IF
C
C---- part 2, called from bond, overlap check for bond KAtom-LAtom
C
      TOL = 1.0E-5
C
C---- save quadrangle temporarily
C
      IQ = Num_Bnd_Quadrangles + 1
C
C
      DO 50 JDO = 1,2
        Quadrangles(JDO,IQ) = DP(JDO,1)
        Quadrangles(JDO+2,IQ) = DP(JDO,2)
        Quadrangles(JDO+4,IQ) = DP(JDO,66)
        Quadrangles(JDO+6,IQ) = DP(JDO,65)
   50 CONTINUE
C
C
      Quadrangles(9,IQ) = TID1
C
C---- fit rectangle around quadrangle
C
      DO 60 JDO = 1,2
        YMIN(JDO) = MIN(DP(JDO,1),DP(JDO,2),DP(JDO,66),DP(JDO,65))
        YMAX(JDO) = MAX(DP(JDO,1),DP(JDO,2),DP(JDO,66),DP(JDO,65))
   60 CONTINUE
C
C---- rough check for atom-over-bond overlap
C
      ITY = 0
C
C
      DO 80 IA = 9,Num_Conics
        DO 70 JDO = 1,2
          IF (IA.EQ.LATOM) GO TO 80
          IF ((YMAX(JDO)-Ellipse_Conics(2*JDO-1,IA)).LE.0.0) GO TO 80
          IF ((YMIN(JDO)-Ellipse_Conics(2*JDO,IA)).GE.0.0) GO TO 80
   70   CONTINUE
C
C---- check for true atom-over-bond overlap
C
        ITY = ITY + 1
C
C            *********************************
        CALL checkatmbondoverlap(IQ,IA,IQQ,ITY)
C            *********************************
C
        IF (IQQ.LT.0) THEN
C
C---- hidden bond
C
          ICQ = -1
          RETURN
        ELSE IF (IQQ.GT.0) THEN
          ICQ = ICQ + 1
          IF (Num_Overlaps.GT.Max_Conics) GO TO 90
        END IF
C
C
   80 CONTINUE
   90 IF (Num_Bnd_Quadrangles.LE.0) RETURN
C
C---- rough check for bond-over-bond overlap
C
C          ************************************
      CALL vectordifference(Plotting_XYZ(1,LATOM),
     +                      Plotting_XYZ(1,KATOM),
     +                      V1)
      CALL getunitvectors(V1,V1,1)
C          ************************************
C
      VUE(1) = Plotting_XYZ(1,KATOM)
      VUE(2) = Plotting_XYZ(2,KATOM)
      VUE(3) = Plotting_XYZ(3,KATOM) - Eye_Distance
C
C
      DO 320 IB = 1,Num_Bnd_Quadrangles
        TID2 = Quadrangles(9,IB)
        IF (TID1.EQ.TID2) GO TO 320
        NATOM = NINT(MOD(TID2,REAL(Max_Atoms)))
        MATOM = NINT(TID2/REAL(Max_Atoms))
C
C
        DO 100 JDO = 1,2
          IF ((YMAX(JDO)-MIN(Quadrangles(JDO,IB),Quadrangles(JDO+2,IB),
     +        Quadrangles(JDO+4,IB),Quadrangles(JDO+6,IB))).LE.
     +        0.0) GO TO 320
          IF ((YMIN(JDO)-MAX(Quadrangles(JDO,IB),Quadrangles(JDO+2,IB),
     +        Quadrangles(JDO+4,IB),Quadrangles(JDO+6,IB))).GE.
     +        0.0) GO TO 320
  100   CONTINUE
C
C---- set up linear forms for edges of quadrangle
C
        DO 120 LDO = 1,4
          K = 2*LDO
          K1 = MOD(K,8) + 2
          QUA(1,LDO) = Quadrangles(K,IB) - Quadrangles(K1,IB)
          QUA(2,LDO) = Quadrangles(K1-1,IB) - Quadrangles(K-1,IB)
          QUA(3,LDO) = Quadrangles(K-1,IB)*Quadrangles(K1,IB) -
     +                 Quadrangles(K1-1,IB)*Quadrangles(K,IB)
C
C---- normalize line equation coefficients
C
          T1 = SQRT(QUA(1,LDO)**2+QUA(2,LDO)**2)
          IF (T1.LE.0.0) GO TO 320
C
C
          DO 110 JDO = 1,3
            QUA(JDO,LDO) = QUA(JDO,LDO)/T1
  110     CONTINUE
  120   CONTINUE
C
C---- evaluate linear forms and signatures for quadrangle
C
        T3 = 3.0
C
C
        DO 150 K = 1,4
          T2 = 3.0
          J = K*2
C
C
          DO 140 LDO = 1,4
            T1 = Quadrangles(J-1,IQ)*QUA(1,LDO) +
     +           Quadrangles(J,IQ)*QUA(2,LDO) + QUA(3,LDO)
            IF (T1.GE.0.0) GO TO 130
            T2 = T2 - 1.0
  130       FL(LDO,K) = T1
  140     CONTINUE
          IF (T2.GE.0.0) GO TO 150
          T3 = T3 - 1.0
  150   CONTINUE
C
C---- check for 4 points inside quadrangle
C
        IF (T3.GE.0.0) GO TO 160
        ITYPE = -1
        GO TO 210
C
C---- check for 1 to 3 points inside quadrangle
C
  160   IF ((T3-3.0).GE.0.0) GO TO 170
        ITYPE = 0
        GO TO 210
C
C---- determine which edges are crossed by the 4 line Segments
C
  170   CONTINUE
C
C
        DO 190 LDO = 1,4
          L1 = MOD(LDO,4) + 1
C
C---- line segment l from point y1 to point y2
C
          Y1(1) = Quadrangles(LDO*2-1,IQ)
          Y1(2) = Quadrangles(LDO*2,IQ)
          Y2(1) = Quadrangles(L1*2-1,IQ)
          Y2(2) = Quadrangles(L1*2,IQ)
C
C
          DO 180 K = 1,4
            T1 = FL(K,LDO)
            T2 = FL(K,L1)
            T3 = T1 - T2
C
C---- t1 and t2 must have opposite signs for intersection to occur
C
            IF (T1*T2.GT.0.0) GO TO 180
C
C---- component of segment l perpendicular to edge k of ib is t3
C
            IF ((ABS(T3)-1.0E-5).LE.0.0) GO TO 180
C
C---- calculate coordinates of intersection
C
            T4 = (Y2(1)*T1-Y1(1)*T2)/T3
            T5 = (Y2(2)*T1-Y1(2)*T2)/T3
            K0 = 2*K
            K1 = (MOD(K,4)+1)*2
C
C---- is intersection within quadrangle iq
C
            T6 = (T4-Quadrangles(K0-1,IB))* (Quadrangles(K1-1,IB)-T4) +
     +           (T5-Quadrangles(K0,IB))* (Quadrangles(K1,IB)-T5)
            IF ((ABS(T6)-1.0E-4).LE.0.0) GO TO 200
            IF (T6.GE.0.0) GO TO 200
  180     CONTINUE
  190   CONTINUE
        GO TO 320
  200   ITYPE = 1
C
C---- check over/under ambiguity
C
  210   IF (KATOM.EQ.MATOM .OR. LATOM.EQ.NATOM .OR.
     +      LATOM.EQ.MATOM) GO TO 220
        GO TO 230
  220   CONTINUE
C
C---- bonds share an atom
C
        KWORK = KSortPointer(KATOM) + KSortPointer(LATOM)
        LWORK = KSortPointer(MATOM) + KSortPointer(NATOM)
        MWORK = KWORK - LWORK
        IF (MWORK.LT.0) GO TO 280
        GO TO 320
C
C            *************************************
  230   CALL vectordifference(Plotting_XYZ(1,NATOM),
     +                        Plotting_XYZ(1,MATOM),V2)
        CALL vectordifference(Plotting_XYZ(1,MATOM),
     +                        Plotting_XYZ(1,KATOM),V4)
        CALL getunitvectors(V2,V2,1)
        CALL getunitvectors(V4,V4,1)
        CALL vectorproduct(V1,V2,V3,1)
C            *************************************
C
C            **************************
        IF ((vectbyvecttranspose(V3,V3)-TOL).GT.0.0) GO TO 240
C            **************************
C
C---- parallel bonds, recalculate v3
C
C            *************************
        CALL vectorproduct(V1,V4,V5,1)
        CALL vectorproduct(V5,V1,V3,1)
C            *************************
C
C---- check for collinear bonds
C
C            **************************
        IF ((vectbyvecttranspose(V3,V3)-TOL).LE.0.0) THEN
C            **************************
C
          GO TO 280
        ELSE
          GO TO 260
        END IF
C
C            *************************
  240   IF ((vectbyvecttranspose(V3,V4)+TOL).GE.0.0) GO TO 260
C            *************************
C
        DO 250 J = 1,3
          V3(J) = -V3(J)
  250   CONTINUE
C
C---- v3 is normal to bonds iq and ib going from iq toward ib
C
  260   IF (Eye_Distance.GT.0.0) GO TO 270
C
C
        IF (V3(3).LE.0.0) THEN
          GO TO 320
        ELSE
          GO TO 280
        END IF
C
C           ***************************
  270   IF (vectbyvecttranspose(VUE,V3).GE.0.0) GO TO 320
C           ***************************
C
C---- overlapping bond found
C
  280   ICQ = ICQ + 1
        IF (ITYPE.GE.0) GO TO 290
C
C---- hidden bond
C
        ICQ = -1
        RETURN
C
C---- store interfering quadrangle
C
  290   IF (NQuads_Overlaped.GT.Max_Quadrangles) 
C
C            *********************************************
     +  CALL CCPERR(1,' Too many Quadrangles overlapped ')
C            *********************************************
C
        NQuads_Overlaped = NQuads_Overlaped + 1
C
C
        DO 310 K = 1,4
          DO 300 J = 1,3
            Quads_Overlaped(J,K,NQuads_Overlaped) = QUA(J,K)
  300     CONTINUE
  310   CONTINUE
C
C
        KQ(NQuads_Overlaped) = IB
  320 CONTINUE
      RETURN
      END
C
C
C
C     ==========================================================
      SUBROUTINE brickcontour(RHO,FLAGS,CONLEV,MACS,IBRICK_SIZE,LLIM,
     +                        ORIGIN,VECTOR,PSBITS,LIST,MAX_LST)
C     =========================================================
C
C
C----  contouring program for a single brick.
C
C       the algorithm used is described in
C       computational crystallography
C       d. sayre editor, o.u.p. (1982) p266.  r. diamond
C
C  PLUTO version: differences:-
C      (a) LOGICAL*1 variables
C      (b) when vector list full, calls plotcontour to plot,
C          then continues
C
C       Variables
C       =========
C
C Input:
C
C   RHO(*):- Real densities in a one dimensional array
C            stored as if dimensioned by
C            dimension rho(macs(1),macs(2),macs(3))
C
C   FLAGS(4,*):- Logical array. calling routine must arrange for *
C                to be at least equal to the number of grid points
C                on the largest brick face. there is no internal
C                check for over-running this limit.
C
C   CONLEV:- Real scalar contour level.
C
C   MACS(3):- Integer array giving effective dimensions of rho
C             treated as a 3d array.
C             macs(1) relates to fastest subscript.
C
C   IBrick_Size(3):- Integer array giving dimensions of
C                     the region to be contoured.
C             IBrick_Size(i).le.macs(i), equality being usual.
C             IBrick_Size contains numbers of grid points,
C                         and is 1 greater than the brick
C                         sizes used by bilder which used the
C                         number of intervals on the brick edge.
C
C   LLIM(3):- Integer array giving grid coords of bottom corner
C             of contoured region relative to a global origin.
C
C   ORIGIN:- Integer scalar being the subscript to rho of the bottom
C            corner of the contoured region.
C
C   Max_Lst:- Integer scalar supplied by calling routine, being the
C             max value for list and the true dimension of vector
C             and psbits.
C
C Output:
C
C   VECTOR(3,*):- Real array of output vectors in grid coords (uvw)
C                 relative to the origin implied by llim.
C
C   PSBITS(*):- Logical array. psbits(i)=.false. means that
C               vector( ,i) is a move-to.
C               psbits(i)=.true. means draw-to.
C
C   LIST:- Integer scalar declaring the number of entries in vector.
C
C
C
C
C       Example
C       =======
C
C
C               -----------------------------------
C               !                                 !
C               !  1   2   3   4   5   6   7   8  !
C               !                                 !
C               !        -----------------        !
C               !        !               !        !
C               !  9  10 !11  12  13  14 !15  16  !
C               !        !               !        !
C               !        !               !        !
C               !        !               !        !
C               ! 17  18 !19  20  21  22 !23  24  !
C               !        !               !        !
C               !        !               !        !
C               !        !               !        !
C               ! 25  26 !27  28  29  30 !31  32  !
C               !        !               !        !
C               !        -----------------        !
C               !                                 !
C               ! 33  34  35  36  37  38  39  40  !
C               !                                 !
C               -----------------------------------
C
C
C       Assuming the global origin to be on grid point 1
C       with the inner rectangle to be contoured, the above
C       diagram corresponds to
C
C                       MACS(1)=8
C                       MACS(2)=5
C                       MACS(3)=SIMILAR
C
C                       IBrick_Size(1)=4
C                       IBrick_Size(2)=3
C                       IBrick_Size(3)=SIMILAR
C
C                       ORIGIN=11
C
C                       LLIM(1)=2
C                       LLIM(2)=1
C                       LLIM(3)=SIMILAR
C
C       For each pair of boundary planes the routine contours
C       only the lower one. this is so that contiguous bricks
C       may be prepared without double contours on the interfaces.
C       note, too, that if contiguous bricks are wanted their
C       interfacial plane must be presented as the top plane of
C       one contouring operation and as the bottom plane of
C       another, otherwise one or more uncontoured intervals
C       will appear between them.
C       the region defined by origin and
C       ISpecified_Bond_Params must lie entirely
C       within the region defined by macs. there is no internal
C       check that it does.
C
C
C     .. Scalar Arguments ..
      REAL CONLEV
      INTEGER LIST,MAX_LST,ORIGIN
C     ..
C     .. Array Arguments ..
      REAL RHO(*),VECTOR(3,*)
      INTEGER IBRICK_SIZE(3),LLIM(3),MACS(3)
      LOGICAL*1 FLAGS(4,*),PSBITS(*)
C     ..
C     .. Local Scalars ..
      REAL X
      INTEGER I,II,J,JJ,JU,JV,K,KU,KUT,KV,L,LL,LLU,LLV,M,MM,MODE,N,NN,
     +        NODE,ORIG,ULU,ULV,UU,VV,WW
      LOGICAL*1 FIN,FLAG,PSBIT,RIGHT
C     ..
C     .. Local Arrays ..
      INTEGER STEP(3)
C     ..
C     .. External Subroutines ..
      EXTERNAL plotcontour
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC MOD,REAL
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      STEP(1) = 1
      STEP(2) = MACS(1)
      STEP(3) = MACS(1)*MACS(2)
      LIST = 0
      KUT = 0
C
C---- next sectioning direction
C
   10 KUT = KUT + 1
      IF (KUT.GE.4) RETURN
      II = KUT + 1
      IF (II.GE.4) II = 1
      JJ = II + 1
      IF (JJ.GE.4) JJ = 1
      ORIG = ORIGIN - STEP(KUT)
      WW = 0
C
C----  next plane
C
   20 WW = WW + 1
      IF (WW.GE.IBRICK_SIZE(KUT)) GO TO 10
      ORIG = STEP(KUT) + ORIG
      FIN = .FALSE.
      N = IBRICK_SIZE(II)*IBRICK_SIZE(JJ)
C
C
      DO 40 I = 1,N
        DO 30 J = 1,4
          FLAGS(J,I) = .FALSE.
   30   CONTINUE
   40 CONTINUE
C
C
      UU = 1
      VV = 1
      ULU = IBRICK_SIZE(II)
      ULV = IBRICK_SIZE(JJ)
      LLU = 1
      LLV = 2
      KU = 1
      KV = 0
      MODE = 1
      M = ORIG
      MM = 1
C
C---- search path
C
      I = 0
      IF (RHO(M).GE.CONLEV) I = 1
   50 L = M
      LL = MM
      GO TO (60,70,80,90) MODE
C
C---- turn corners
C       finish if an about turn is required
C
   60 IF (UU.LT.ULU) GO TO 100
      IF (FIN) GO TO 20
      FIN = .TRUE.
      ULU = ULU - 1
      KU = 0
      KV = 1
      MODE = 2
   70 IF (VV.LT.ULV) GO TO 110
      IF (FIN) GO TO 20
      FIN = .TRUE.
      ULV = ULV - 1
      KU = -1
      KV = 0
      MODE = 3
   80 IF (UU.GT.LLU) GO TO 120
      IF (FIN) GO TO 20
      FIN = .TRUE.
      LLU = LLU + 1
      KU = 0
      KV = -1
      MODE = 4
   90 IF (VV.GT.LLV) GO TO 130
      IF (FIN) GO TO 20
      FIN = .TRUE.
      LLV = LLV + 1
      KU = 1
      KV = 0
      MODE = 1
      GO TO 60
C
C---- straight steps
C
  100 M = STEP(II) + L
      MM = LL + 1
      UU = UU + 1
      GO TO 140
  110 M = STEP(JJ) + L
      MM = IBRICK_SIZE(II) + LL
      VV = VV + 1
      GO TO 140
  120 M = L - STEP(II)
      MM = LL - 1
      UU = UU - 1
      GO TO 140
  130 M = L - STEP(JJ)
      MM = LL - IBRICK_SIZE(II)
      VV = VV - 1
  140 FIN = .FALSE.
C
C---- test for contour
C
  150 J = 0
      IF (RHO(M).GE.CONLEV) J = 1
      IF (I.EQ.J) GO TO 50
C
C----   contour crossed
C
      I = J
C
C----   have we been here before?
C
      IF (FLAGS(MODE,MM)) GO TO 50
C
C----   no, so record state of search path
C       and initiate chase path
C
      N = M
      NN = MM
      JU = UU
      JV = VV
      K = J
      PSBIT = .FALSE.
C
C----   select even parity path
C
      IF (MOD(UU+VV+MODE+J,2).NE.0) GO TO 160
      NODE = MODE
      RIGHT = .FALSE.
      GO TO 260
  160 NODE = MODE + 2
      IF (NODE.GE.5) NODE = NODE - 4
      RIGHT = .TRUE.
      I = M
      M = L
      L = I
      I = MM
      MM = LL
      LL = I
      I = 1 - J
      UU = UU - KU
      VV = VV - KV
      GO TO 260
C
C----  turn corners
C
  170 RIGHT = .NOT. RIGHT
  180 IF (RIGHT) GO TO 190
C
C----  turn left
C
      NODE = NODE - 1
      IF (NODE.LT.1) NODE = 4
      GO TO 200
C
C---- turn right
C
  190 NODE = NODE + 1
      IF (NODE.GE.5) NODE = 1
C
C---- step and test for boundary
C
  200 L = M
      LL = MM
      GO TO (210,220,230,240) NODE
  210 UU = UU + 1
      IF (UU.GT.IBRICK_SIZE(II)) GO TO 320
      M = STEP(II) + L
      MM = LL + 1
      GO TO 250
  220 VV = VV + 1
      IF (VV.GT.IBRICK_SIZE(JJ)) GO TO 320
      M = STEP(JJ) + L
      MM = IBRICK_SIZE(II) + LL
      GO TO 250
  230 UU = UU - 1
      IF (UU.LT.1) GO TO 320
      M = L - STEP(II)
      MM = LL - 1
      GO TO 250
  240 VV = VV - 1
      IF (VV.LT.1) GO TO 320
      M = L - STEP(JJ)
      MM = LL - IBRICK_SIZE(II)
C
C----  test for contours
C
  250 J = 0
      IF (RHO(M).GE.CONLEV) J = 1
      IF (I.EQ.J) GO TO 180
C
C----  contour crossed so
C
      I = J
  260 IF (LIST.GE.MAX_LST) THEN
C
C            *******************************
        CALL plotcontour(VECTOR,PSBITS,LIST)
C            *******************************
C
        LIST = 0
      END IF
C
C
      LIST = LIST + 1
      X = (RHO(M)-CONLEV)/ (RHO(M)-RHO(L))
      VECTOR(II,LIST) = REAL(UU-1+LLIM(II))
      VECTOR(JJ,LIST) = REAL(VV-1+LLIM(JJ))
      VECTOR(KUT,LIST) = REAL(WW-1+LLIM(KUT))
      PSBITS(LIST) = PSBIT
      PSBIT = .TRUE.
      GO TO (270,280,290,300) NODE
  270 VECTOR(II,LIST) = VECTOR(II,LIST) - X
      GO TO 310
  280 VECTOR(JJ,LIST) = VECTOR(JJ,LIST) - X
      GO TO 310
  290 VECTOR(II,LIST) = VECTOR(II,LIST) + X
      GO TO 310
  300 VECTOR(JJ,LIST) = VECTOR(JJ,LIST) + X
  310 FLAG = FLAGS(NODE,MM)
C
C----   record our visit
C
      FLAGS(NODE,MM) = .TRUE.
      FLAGS(MOD(NODE+1,4)+1,LL) = .TRUE.
C
C----  have we finished this contour?
C
      IF (.NOT.FLAG) GO TO 170
C
C----   contour closed or boundary reached
C       therefore resume search path
C
  320 M = N
      MM = NN
      UU = JU
      VV = JV
      I = K
      GO TO 150
      END
C
C
C
C     ===========================================
      SUBROUTINE checkatmbondoverlap(IQ,IA,ICQ,ITY)
C     ============================================
C
C---- subroutine checks for overlap between atoms and bonds
C
C---- called by subroutines lap700 and lap800
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Conics
      PARAMETER (Max_Conics=20)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER Max_Quadrangles
      PARAMETER (Max_Quadrangles=30)
      INTEGER Max_Segments
      PARAMETER (Max_Segments=50)
C     ..
C     .. Scalar Arguments ..
      INTEGER IA,ICQ,IQ,ITY
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Charcter_Size,
     +     CurrentCharcter_Size,Dist_to_Boundary,Drawing_Size,
     +     Ellipse_ScaleFact,Eye_Distance,Overlap_Margin,Paper_Length,
     +     Paper_Width,Perspective_Distance,Picture_Scale,Plotter_Step,
     +     Retrace_Displmnt,S,Scal_pre_Proj,SCL,Shadelight_X,
     +     Shadelight_Y,Stereo_Separation
      INTEGER Iuse_persp_Labs,Lcurrent_Colour,Ldraw_cell_Flag,
     +        Lpack_Flag,Lpaper_Setting,Lperspective_Flag,
     +        Lplot_colour_Stereo,Lposition_Flag,Lprint_Flag,
     +        LShade_Flag,LSolid_Flag,LStereo_Flag,LSuperimpose_Flag,
     +        LTaper_Flag,LTrace_Flag,Natoms_plusCell,Ncopies,
     +        NDrawing_Number,NQuads_Overlaped,Nspecified_Bonds,
     +        Num_Bnd_Quadrangles,Num_Conics,Num_Moles,Num_Overlaps
      LOGICAL Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL Base_Vect_Refsys,Base_Vect_Wrksys,BBB,Drawing_Origin,
     +     Ellipse_Conics,Ellipse_Resolution,G_Metric_Tensor,
     +     Overlap_Conics,P1_to_Refsys,P1_to_Wrksys,Plotting_XYZ,
     +     Principal_axis_rms,Prin_Angstrom_Vectors,Prin_Axis_Col_Vects,
     +     Q,Quadrangles,Quads_Overlaped,Recip_Metric_Tensor,RP,
     +     Segments,Sorted_xyz,Title_Rotation,Title_RotnTrans,
     +     ViewMatrix,XO
      INTEGER ISpecified_Bond_Params,KC,KQ,KSortPointer,MOL
C     ..
C     .. Local Scalars ..
      REAL T1,T2,T3,T4,T5,TID
      INTEGER I,IJ,ITYPE,J,JDO,K,K1,L1,LDO,NA1,NA2
C     ..
C     .. Local Arrays ..
      REAL BF(4),CON(3,3),QF(5),QUA(3,4),V1(4),V2(3),V3(3),V4(3),V5(3)
C     ..
C     .. External Functions ..
      REAL vectbyvecttranspose
      EXTERNAL vectbyvecttranspose
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPERR,conic2plotter,getunitvectors,vectordifference,
     +         vectorproduct
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MOD,NINT,REAL,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_OVERLAP/Num_Conics,Num_Overlaps,NQuads_Overlaped,
     +       Num_Bnd_Quadrangles,KC(Max_Conics),KQ(Max_Quadrangles),
     +       Ellipse_Conics(7,Max_Atoms),Overlap_Conics(6,Max_Conics),
     +       Quads_Overlaped(3,4,Max_Quadrangles),
     +       Quadrangles(9,2*Max_Atoms),Segments(Max_Segments,2)
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      TID = Quadrangles(9,IQ)
      NA1 = NINT(TID/REAL(Max_Atoms))
      NA2 = NINT(MOD(TID,REAL(Max_Atoms)))
C
C---- ity.gt.0, check for atoms over a bond
C     ity.lt.0, check for bonds over an atom
C
      ICQ = 0
C
C
      IF (ITY.EQ.0) THEN
        RETURN
      ELSE IF (ITY.GT.0) THEN
C
C            **********************************************
        CALL conic2plotter(Ellipse_Conics(1,IA),CON,V1,0.0)
C            **********************************************
C
        IF ((ITY-2).GE.0) GO TO 50
      END IF
C
C
      IF ((ITY+2).LE.0) GO TO 10
C
C          *********************************************************
      CALL conic2plotter(Ellipse_Conics(1,IA),CON,V1,Overlap_Margin)
C          *********************************************************
C
C---- set up linear forms for edges of quadrangle
C
   10 CONTINUE
C
C
      DO 40 LDO = 1,4
        K = 2*LDO
        K1 = MOD(K,8) + 2
        QUA(1,LDO) = Quadrangles(K,IQ) - Quadrangles(K1,IQ)
        QUA(2,LDO) = Quadrangles(K1-1,IQ) - Quadrangles(K-1,IQ)
        QUA(3,LDO) = Quadrangles(K-1,IQ)*Quadrangles(K1,IQ) -
     +               Quadrangles(K1-1,IQ)*Quadrangles(K,IQ)
        T1 = SQRT(QUA(1,LDO)**2+QUA(2,LDO)**2)
        IF (T1.GT.0.0) GO TO 20
        ITY = 0
        ICQ = 0
        RETURN
C
C---- transform coefficients for edges to normal form
C
   20   DO 30 JDO = 1,3
          QUA(JDO,LDO) = QUA(JDO,LDO)/T1
   30   CONTINUE
   40 CONTINUE
C
C---- evaluate 4 quadratic and 4 bilinear forms
C
   50 V2(3) = 1.0
      V3(3) = 1.0
      T2 = 3.0
C
C
      DO 90 LDO = 1,4
        L1 = (MOD(LDO,4)+1)*2
        V2(1) = Quadrangles(2*LDO-1,IQ)
        V2(2) = Quadrangles(2*LDO,IQ)
        V3(1) = Quadrangles(L1-1,IQ)
        V3(2) = Quadrangles(L1,IQ)
        QF(LDO) = 0.0
        BF(LDO) = 0.0
C
C
        DO 70 K = 1,3
          T1 = CON(3,K)
          DO 60 JDO = 1,2
            T1 = V2(JDO)*CON(JDO,K) + T1
   60     CONTINUE
          QF(LDO) = V2(K)*T1 + QF(LDO)
          BF(LDO) = V3(K)*T1 + BF(LDO)
   70   CONTINUE
C
C
        IF (QF(LDO).LT.0.0) THEN
          GO TO 80
        ELSE IF (QF(LDO).GT.0.0) THEN
          GO TO 90
        END IF
C
C
        T2 = T2 - 0.8
        GO TO 90
   80   T2 = T2 - 1.0
   90 CONTINUE
C
C
      QF(5) = QF(1)
C
C---- check for 4 points of quadrangle inside or on ellipse
C
      IF (T2.GE.0.0) GO TO 100
      ITYPE = -1
      GO TO 160
C
C---- check for 1 to 3 points of quadrangle inside the ellipse
C
  100 IF ((T2-2.2).GE.0.0) GO TO 120
  110 ITYPE = 0
C
C
ccx??    IF (NA2 .EQ. IA) GO TO 300
      IF (NA2.EQ.IA) GO TO 230
      GO TO 170
C
C---- check for quadrangle-ellipse intersection
C
  120 CONTINUE
C
C
      DO 130 K = 1,4
C
C---- evaluate discriminant
C
        T1 = BF(K)**2 - QF(K+1)*QF(K)
        IF (T1.LE.0.0) GO TO 130
        T1 = SQRT(T1)
C
C---- is intersection within bounds of quadrangle
C
        T3 = QF(K) - BF(K)
        T4 = QF(K+1) + T3 - BF(K)
        IF ((ABS(T4)-1.0E-5).LE.0.0) GO TO 130
        T5 = (T3-T1)/T4
C
C
        IF (T5.LT.0.0) THEN
          GO TO 130
        ELSE IF (T5.EQ.0.0) THEN
          GO TO 110
        END IF
C
C
        IF ((1.0-T5).GT.0.0) GO TO 110
  130 CONTINUE
C
C---- no valid intersection found
C     check for center of ellipse within the quadrangle
C
      T3 = 3.0
C
C
      DO 150 K = 1,4
        T1 = QUA(3,K)
        DO 140 J = 1,2
          T1 = V1(J)*QUA(J,K) + T1
  140   CONTINUE
        IF (T1.GE.0.0) GO TO 150
        T3 = T3 - 1.0
  150 CONTINUE
C
C
      IF (T3.GE.0.0) GO TO 220
      ITYPE = 1
C
C---- check over/under ambiguity
C
ccx??  280 IF (NA2.EQ.IA)  GO TO 370
  160 CONTINUE
  170 IF (NA1.EQ.IA) GO TO 230
C
C          ************************************************************
      CALL vectordifference(Plotting_XYZ(1,NA2),Plotting_XYZ(1,NA1),V2)
      CALL vectordifference(Plotting_XYZ(1,IA),Plotting_XYZ(1,NA1),V3)
      CALL getunitvectors(V2,V2,1)
      CALL getunitvectors(V3,V3,1)
      CALL vectorproduct(V2,V3,V4,1)
C          ************************************************************
C
C          **************************
      IF ((vectbyvecttranspose(V4,V4)-1.0E-5).GT.0.0) GO TO 190
C          **************************
C
C---- center of atom iq is on the bond line
C
      IF (ITY.LE.0) THEN
        GO TO 220
      ELSE
        GO TO 240
      END IF
C
C---- center of atom iq is not on the bond line
C
C          *************************
  190 CALL vectorproduct(V4,V2,V5,1)
C          *************************
C
      T1 = -V5(3)
      IF (Eye_Distance.LE.0.0) GO TO 210
      T1 = (Plotting_XYZ(3,IA)-Eye_Distance)*V5(3)
C
C
      DO 200 J = 1,2
        T1 = Plotting_XYZ(J,IA)*V5(J) + T1
  200 CONTINUE
C
C
  210 IF ((T1*REAL(ITY)).LE.0.0) GO TO 230
C
C---- no interference found
C
  220 ICQ = 0
      RETURN
C
C---- itype=1 enclosed ellipse / itype=-1 enclosed quadrangle
C
  230 IF (ITYPE*ITY.GE.0) GO TO 240
C
C---- hidden atom or hidden bond
C
      ICQ = -1
      RETURN
  240 ICQ = 1
      IF (ITY.LT.0) GO TO 270
C
C---- store interfering ellipse
C
      IF (Num_Overlaps.GT.Max_Conics) 
C
C             *************************************
     +   CALL CCPERR(1,' LAPAB too many overlaps ')
C             *************************************
C
      Num_Overlaps = Num_Overlaps + 1
      IJ = 1
C
C
      DO 260 I = 1,3
        DO 250 J = I,3
          Overlap_Conics(IJ,Num_Overlaps) = CON(I,J)
          IJ = IJ + 1
  250   CONTINUE
  260 CONTINUE
C
C
      KC(Num_Overlaps) = IA
      RETURN
C
C---- store interfering quadrangle
C
  270 IF (NQuads_Overlaped.GT.Max_Quadrangles) 
C
C            **************************************************
     +  CALL CCPERR(1,' LAPAB too many Quadrangles overlapped')
C            **************************************************
C
      NQuads_Overlaped = NQuads_Overlaped + 1
C
C
      DO 290 K = 1,4
        DO 280 J = 1,3
          Quads_Overlaped(J,K,NQuads_Overlaped) = QUA(J,K)
  280   CONTINUE
  290 CONTINUE
C
C
      KQ(NQuads_Overlaped) = IQ
      RETURN
      END
C
C
C
C     ====================================
      SUBROUTINE chrread(IRES,CHAIN,BWORK)
C     ====================================
C
C
C
C
C
C
C     .. Scalar Arguments ..
      INTEGER IRES
      CHARACTER CHAIN*1,BWORK* (*)
C     ..
C     .. Local Scalars ..
      INTEGER JDO,K,LINDEX,LLEN
      CHARACTER DWORK*1,CWORK*10
C     ..
C     .. Local Arrays ..
      CHARACTER CNUM(10)*1
C     ..
C     .. External Functions ..
      INTEGER LENSTR
      EXTERNAL LENSTR
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPERR
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC INDEX
C     ..
C     .. Data statements ..
C
C
      DATA CNUM/'0','1','2','3','4','5','6','7','8','9'/
C     ..
C
C
      CWORK = ' '
      LLEN = LENSTR(BWORK)
      K = LLEN
C
C                 ********************************************
      IF (K.GT.5) CALL CCPERR(1,' chrread LENGTH STRING GT 5')
C                 ********************************************
C
      DO 10 JDO = 1,10
        IF (BWORK(1:1) .EQ. CNUM(JDO)) GOTO 20
   10 CONTINUE
C
C     **********************************************************
      CALL CCPERR(1,' There is an ERROR in the above data line')
C     **********************************************************
C
   20 CHAIN = ' '
      CWORK = BWORK(1:LLEN)
      DWORK = CWORK(K:K)
C
C
      DO 30 JDO = 1,10
        LINDEX = INDEX(DWORK,CNUM(JDO))
        IF (LINDEX.NE.0) GO TO 40
   30 CONTINUE
C
C
      CHAIN = DWORK
      K = LLEN - 1
C
C
   40 CONTINUE
C
C---- Have to read integer from bwork(1:LLEN)
C   LLEN = 1,2,3 or 4
C
      GO TO (50,60,70,80) K
   50 READ (CWORK(1:1),FMT=6000) IRES
 6000 FORMAT (I1)
      GO TO 90
   60 READ (CWORK(1:2),FMT=6002) IRES
 6002 FORMAT (I2)
      GO TO 90
   70 READ (CWORK(1:3),FMT=6004) IRES
 6004 FORMAT (I3)
      GO TO 90
   80 READ (CWORK(1:4),FMT=6006) IRES
 6006 FORMAT (I4)
   90 CONTINUE
C
C
      END
C
C
C
C     =========================================
      SUBROUTINE conic2plotter(CON1,CON,Y,OVMR)
C     =========================================
C
C---- transform conic to plotter homogeneous coordinate system
C     called by subroutines lap700 and lapab
C
C
C     .. Scalar Arguments ..
      REAL OVMR
C     ..
C     .. Array Arguments ..
      REAL CON(3,3),CON1(7),Y(3)
C     ..
C     .. Local Scalars ..
      REAL T1
      INTEGER J,K
C     ..
C
C
      Y(1) = (CON1(1)+CON1(2))*0.5
      Y(2) = (CON1(3)+CON1(4))*0.5
      Y(3) = 1.0
      CON(1,1) = CON1(5)
      CON(1,2) = CON1(6)
      CON(2,1) = CON1(6)
      CON(2,2) = CON1(7)
      T1 = (CON1(2)-CON1(1)+CON1(4)-CON1(3))*0.25
      CON(3,3) = - ((T1-OVMR)/T1)**2
C
C
      DO 20 K = 1,2
        CON(K,3) = 0.0
        DO 10 J = 1,2
          CON(K,3) = CON(K,3) - Y(J)*CON(J,K)
   10   CONTINUE
        CON(3,K) = CON(K,3)
        CON(3,3) = CON(3,3) - CON(3,K)*Y(K)
   20 CONTINUE
      RETURN
      END
C
C
C
C     ===========================
      SUBROUTINE contourstyle(CL)
C     ===========================
C
C---- Set up plotting style for contour level cl
C
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
C     ..
C     .. Scalar Arguments ..
      REAL CL
C     ..
C     .. Scalars in Common ..
      INTEGER Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,Lmap_ModeFirst,
     +        LSetView_Flag,Map_Counter,Map_Flag_Counter,Map_pts_U,
     +        Map_pts_V,MU,MV,MW,Npicture,Npts_U_compressed,
     +        Npts_V_compressed,LPERM
C     ..
C     .. Arrays in Common ..
      REAL AQ,Contour_Levels,Contour_Threshold
      INTEGER Map_def_Restrictions,Map_LineTypes,Map_UVW_Limits,
     +        Map_XYZ_Limits,Num_Contours
      CHARACTER Map_Title*80
C     ..
C     .. Local Scalars ..
      INTEGER K,M
C     ..
C     .. External Subroutines ..
      EXTERNAL setplotstyle
C     ..
C     .. Common blocks ..
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C---- above or below threshold ?
C
      K = 1
      IF (CL.LT.Contour_Threshold(Map_Counter)) K = 2
C
C
      IF (Lmap_ModeFirst.LE.0) THEN
C
C---- Normal, style for map 1 or 2
C
        M = Map_Counter
      ELSE
C
C---- MODE FIRST set,
C       Lmap_ModeFirst=2 for first sectio in stack, else 1
C
        M = Lmap_ModeFirst
      END IF
C
C          **********************************
      CALL setplotstyle(Map_LineTypes(1,K,M))
C          **********************************
C
      RETURN
      END
C
C
C
C     ====================================================
      SUBROUTINE drawcircles(IATOM,Plotting_XYZ,RP,NOBSC,LShade_Flag,
     +                       Shadelight_X,Shadelight_Y,IOBSC)
C     =====================================================
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Hidden_Lines
      PARAMETER (Max_Hidden_Lines=2000)
C     ..
C     .. Scalar Arguments ..
      REAL Shadelight_X,Shadelight_Y
      INTEGER IATOM,LShade_Flag,NOBSC
C     ..
C     .. Array Arguments ..
      REAL Plotting_XYZ(3,Max_Atoms),RP(*)
      INTEGER IOBSC(Max_Hidden_Lines)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,CurrentCharcter_Size,
     +     Dist_to_Boundary,Ellipse_ScaleFact,Eye_Distance,
     +     Overlap_Margin,Paper_Length,Paper_Width,Plotter_Step,
     +     Retrace_Displmnt,Scal_pre_Proj,SCL
      INTEGER Iuse_persp_Labs,Lpack_Flag,NDrawing_Number
      LOGICAL UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL Drawing_Origin,Ellipse_Resolution,Title_Rotation,XO
C     ..
C     .. Local Scalars ..
      REAL A,B,CA,CB,DIJ,DIN,DSECT,DSQ,DTH,DX,DY,DZ,PI,R1,R11,R12,R13,
     +     R21,R22,R23,R31,R32,R33,RADIAN,RADIUS,RI,RIN,RINSQ,RJ,RJSQ,
     +     RSQ,SA,SB,THETA,X,X1,XB,XCENT,XX,Y,Y1,YB,YCENT,YS,YSTART,YY,
     +     Z,Z1,ZB,ZCENT,ZZ
      INTEGER I,INS,IP,IUP,J,JIN,M,N,N1,N2,N3,NN,NPP,NSECT
C     ..
C     .. Local Arrays ..
      REAL CX(600),CY(600),U1(3),U2(3),U3(3),ZAX(3)
C     ..
C     .. External Subroutines ..
      EXTERNAL normvectproduct,zdrawto,zmoveto
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,COS,NINT,SIN,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      PI = 3.14159
      DSECT = 0.75
      ZAX(1) = 0.0
      ZAX(2) = 0.0
      ZAX(3) = 1.0
C
C---- draw the circle representing the sphere (iatom)
C     pjojected on xy plane.
C
      XCENT = Plotting_XYZ(1,IATOM)
      YCENT = Plotting_XYZ(2,IATOM)
      ZCENT = Plotting_XYZ(3,IATOM)
      RADIUS = RP(IATOM)
C
C
      IF (RADIUS.LE.Plotter_Step) RETURN
C
C
      DTH = 3.0*Plotter_Step/RADIUS
      N = NINT(2.0*PI/DTH)
      M = (N/4)*4
C
C---- minimum number of lines /circle =8
C
      IF (M.LT.8) M = 8
   10 DTH = 2.0*PI/M
      N1 = M/4
      N2 = M/2
      N3 = 3*M/4
      THETA = 0.0
      IF (M.LE.600) GO TO 20
      WRITE (6,FMT=6000) RADIUS
 6000 FORMAT (' Beeg circle needing more than 600 points. radius= ',
     +       F8.1,' mm')
      M = 596
      GO TO 10
C
C---- set up points for circle
C
   20 CONTINUE
C
C
      DO 30 I = 1,N1
        A = COS(THETA)
        B = SIN(THETA)
        CX(I) = A
        CY(I) = B
        CX(I+N1) = -B
        CY(I+N1) = A
        CX(I+N2) = -A
        CY(I+N2) = -B
        CX(I+N3) = B
        CY(I+N3) = -A
        THETA = THETA + DTH
   30 CONTINUE
C
C
      NPP = M + 1
      CX(M+1) = CX(1)
      CY(M+1) = CY(1)
C
C---- now plot the circle as the sequence of npp points.
C
      INS = 0
      JIN = 0
      IUP = 1
C
C
      DO 80 I = 1,NPP
        X = CX(I)*RADIUS + XCENT
        Y = CY(I)*RADIUS + YCENT
        IF (NOBSC.EQ.0) GO TO 60
        Z = Plotting_XYZ(3,IATOM)
C
C---- this section tests for the obscuring of the
C     point (x,y,z) by any of the atoms given in the array iobsc.
C     if there is intersection of sphere
C     the negative is given of the atom serial number.
C
C
C
        DO 40 N = 1,NOBSC
          J = IOBSC(N)
          J = ABS(J)
          DX = X - Plotting_XYZ(1,J)
          RJ = RP(J)
          IF (ABS(DX).GT.RJ) GO TO 40
          DY = Y - Plotting_XYZ(2,J)
          IF (ABS(DY).GT.RJ) GO TO 40
          DSQ = DX*DX + DY*DY
          RJSQ = RJ*RJ
C
C---- point is obscured when within envelope circle j and zj.gt.z
C
          IF (DSQ.LT.RJSQ .AND. Plotting_XYZ(3,J).GT.Z) GO TO 50
          IF (IOBSC(N).GE.0) GO TO 40
C
C---- point is obscured if within sphere j.
C
          IF (J.EQ.JIN) GO TO 40
          DZ = Z - Plotting_XYZ(3,J)
          DSQ = DZ*DZ + DSQ
          IF (DSQ.LT.RJSQ) GO TO 50
   40   CONTINUE
C
C----  no obscuring so go ahead with plotting of this step.
C
        GO TO 60
C
C---- an obscuring atom has been found .  lift the pen.
C
   50   IUP = 1
        GO TO 70
C
C---- pen down and plot this step.
C
   60   CONTINUE
C
C                     ****************
        IF (IUP.EQ.1) CALL zmoveto(X,Y)
C                     ****************
C
C            ***********
        CALL zdrawto(X,Y)
C            ***********
C
        IUP = 0
   70   CONTINUE
C
C
        IF (INS.LT.0) THEN
          GO TO 240
        ELSE IF (INS.GT.0) THEN
          GO TO 150
        END IF
   80 CONTINUE
C
C---- end of circle
C
      IUP = 1
C
C---- draw circles showing the section of overlap of spheres.
C
      INS = 1
      IF (NOBSC.EQ.0) GO TO 170
C
C     The control flow was illegal with this DO:
CCC      DO 160 NN = 1,NOBSC
      NN = 1
 85   NN = NN + 1
      IF (NN.GT.NOBSC) GOTO 165
        IF (IOBSC(NN).GE.0) GO TO 160
        J = -IOBSC(NN)
C
C---- tests to avoid drawing each intersection more than once
C
        DZ = Plotting_XYZ(3,J) - Plotting_XYZ(3,IATOM)
        IF (DZ.LT.0.00) THEN
          GO TO 160
        ELSE IF (DZ.GT.0.00) THEN
          GO TO 90
        END IF
C
C
        IF (J.LE.IATOM) GO TO 160
   90   DX = Plotting_XYZ(1,J) - Plotting_XYZ(1,IATOM)
        DY = Plotting_XYZ(2,J) - Plotting_XYZ(2,IATOM)
C
C---- get the distance along line ij of the
C     plane of the intersection.
C     this is calculated by simple cosine formula.
C
        DSQ = DX*DX + DY*DY + DZ*DZ
        DIJ = SQRT(DSQ)
        RI = RP(IATOM)
        RJ = RP(J)
        DIN = (RI*RI+DSQ-RJ*RJ)/ (2.0*DIJ)
        RINSQ = RI*RI - DIN*DIN
        RIN = 0.0
        IF (RINSQ.GT.0.0) RIN = SQRT(RINSQ)
C
C---- direction cosines of vector ij
C
        U3(1) = DX/DIJ
        U3(2) = DY/DIJ
        U3(3) = DZ/DIJ
        XB = U3(1)*DIN + Plotting_XYZ(1,IATOM)
        YB = U3(2)*DIN + Plotting_XYZ(2,IATOM)
        ZB = U3(3)*DIN + Plotting_XYZ(3,IATOM)
C
C---- the intersection circle has centre at (xb,yb,zb)
C     and radius rin.
C
        IF (ABS(U3(3)).GT.0.9999) U3(1) = 0.0001
C
C---- set up axial system with u3 along direction ij.
C     the circle is generate  as coordinates  (xx,yy,0)
C
C            **************************
        CALL normvectproduct(ZAX,U3,U1)
        CALL normvectproduct(U3,U1,U2)
C            **************************
C
C---- calculate points on circle using theta from 0 to pi.
C
        JIN = J
C
C
        DO 140 IP = 1,NPP
          XX = CX(IP)*RIN
          YY = CY(IP)*RIN
          X = U1(1)*XX + U2(1)*YY + XB
          Y = U1(2)*XX + U2(2)*YY + YB
          Z = U1(3)*XX + U2(3)*YY + ZB
C
C---- now plot this point testing for obscuring as before.
C
          DO 100 N = 1,NOBSC
            J = IOBSC(N)
            J = ABS(J)
            DX = X - Plotting_XYZ(1,J)
            RJ = RP(J)
            IF (ABS(DX).GT.RJ) GO TO 100
            DY = Y - Plotting_XYZ(2,J)
            IF (ABS(DY).GT.RJ) GO TO 100
            DSQ = DX*DX + DY*DY
            RJSQ = RJ*RJ
C
C---- point is obscured when within envelope circle j and zj.gt.z
C
            IF (DSQ.LT.RJSQ .AND. Plotting_XYZ(3,J).GT.Z) GO TO 110
            IF (IOBSC(N).GE.0) GO TO 100
C
C---- point is obscured if within sphere j.
C
            IF (J.EQ.JIN) GO TO 100
            DZ = Z - Plotting_XYZ(3,J)
            DSQ = DZ*DZ + DSQ
            IF (DSQ.LT.RJSQ) GO TO 110
  100     CONTINUE
C
C----  no obscuring so go ahead with plotting of this step.
C
          GO TO 120
C
C---- an obscuring atom has been found .  lift the pen.
C
  110     IUP = 1
          GO TO 130
C
C---- pen down and plot this step.
C
  120     CONTINUE
C
C                       ****************
          IF (IUP.EQ.1) CALL zmoveto(X,Y)
C                       ****************
C
C              ***********
          CALL zdrawto(X,Y)
C              ***********
C
          IUP = 0
  130     CONTINUE
C
C
          GO TO 150
C
C---- after plotting add increment to theta.
C
  140   CONTINUE
  150   CONTINUE
C
C---- end of circle
C
        IUP = 1
 160    GOTO 85
 165    CONTINUE
C
C---- draw shade lines on surface of sphere.
C     light source at yrot,zrot
C     given by (Shadelight_X,Shadelight_Y)
C
  170 IF (LShade_Flag.EQ.0) GO TO 270
      INS = -1
      JIN = 0
      NPP = NPP/2
      N = NINT(RADIUS/DSECT)
      NSECT = 2*N + 1
      YSTART = -N*DSECT
C
C---- rotation matrix r.  rotate first about y, then about z
C     yrot=a  zrot=b
C
      RADIAN = 180.0/PI
      A = Shadelight_X/RADIAN
      B = Shadelight_Y/RADIAN
      CA = COS(A)
      SA = SIN(A)
      CB = COS(B)
      SB = SIN(B)
      R11 = CB*CA
      R12 = SB
      R13 = -CB*SA
      R21 = -SB*CA
      R22 = CB
      R23 = SB*SA
      R31 = SA
      R32 = 0.0
      R33 = CA
C
C----  rotate points on circlular section
C
      YS = YSTART
C
C     The control flow was illegal with this DO:
CCC      DO 260 I = 1,NSECT
      I = 0
 175  I = I + 1
      IF (I.GT.NSECT) GOTO 265
        RSQ = RADIUS*RADIUS - YS*YS
        IF (RSQ.LT.0.0) GO TO 250
        R1 = SQRT(RSQ)
        IUP = 1
C
C
        DO 230 IP = 1,NPP
          XX = CX(IP)*R1
          YY = YS
          ZZ = CY(IP)*R1
          Z1 = R31*XX + R32*YY + R33*ZZ
          IF (Z1.LT.0.0) GO TO 220
          X1 = R11*XX + R12*YY + R13*ZZ
          Y1 = R21*XX + R22*YY + R23*ZZ
          X = X1 + XCENT
          Y = Y1 + YCENT
          Z = Z1 + ZCENT
C
C---- plot this point testing for obscuring as usual...
C
          IF (NOBSC.EQ.0) GO TO 200
C
C
          DO 180 N = 1,NOBSC
            J = IOBSC(N)
            J = ABS(J)
            DX = X - Plotting_XYZ(1,J)
            RJ = RP(J)
            IF (ABS(DX).GT.RJ) GO TO 180
            DY = Y - Plotting_XYZ(2,J)
            IF (ABS(DY).GT.RJ) GO TO 180
            DSQ = DX*DX + DY*DY
            RJSQ = RJ*RJ
C
C---- point is obscured when within envelope circle j and zj.gt.z
C
            IF (DSQ.LT.RJSQ .AND. Plotting_XYZ(3,J).GT.Z) GO TO 190
            IF (IOBSC(N).GE.0) GO TO 180
C
C---- point is obscured if within sphere j.
C
            IF (J.EQ.JIN) GO TO 180
            DZ = Z - Plotting_XYZ(3,J)
            DSQ = DZ*DZ + DSQ
            IF (DSQ.LT.RJSQ) GO TO 190
  180     CONTINUE
C
C----  no obscuring so go ahead with plotting of this step.
C
          GO TO 200
C
C---- an obscuring atom has been found .  lift the pen.
C
  190     IUP = 1
          GO TO 210
C
C---- pen down and plot this step.
C
  200     CONTINUE
C
C                       ****************
          IF (IUP.EQ.1) CALL zmoveto(X,Y)
C                       ****************
C
C              ***********
          CALL zdrawto(X,Y)
C              ***********
C
          IUP = 0
  210     CONTINUE
C
C
          GO TO 240
  220     IUP = 1
  230   CONTINUE
  240   CONTINUE
  250   YS = YS + DSECT
CCC 260    CONTINUE 
        GOTO 175
 265    CONTINUE
  270 RETURN
      END
C
C
C
C     ======================================
      SUBROUTINE drawellipsoids(ITOM,LDOUBLE)
C     =======================================
C
C----       subroutine to draw ellipsoids
C
C
C---- NEllipsoidStyle packed as
C
C   ==>      NPLANE + NDOT*10 + NLINE*100 +
C             NDASH*1000 + AIN0*10000 + AIN1*1000000
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Conics
      PARAMETER (Max_Conics=20)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER Max_Quadrangles
      PARAMETER (Max_Quadrangles=30)
      INTEGER Max_Segments
      PARAMETER (Max_Segments=50)
C     ..
C     .. Scalar Arguments ..
      INTEGER ITOM,LDOUBLE
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Charcter_Size,
     +     CurrentCharcter_Size,D001,D010,D100,Dist_to_Boundary,
     +     Drawing_Size,Ellipse_ScaleFact,Eye_Distance,
     +     RFR(4,4),ROR(4,4),
     +     Overlap_Margin,Paper_Length,Paper_Width,Perspective_Distance,
     +     Picture_Scale,Plotter_Step,Retrace_Displmnt,S,Scal_pre_Proj,
     +     SCL,Shadelight_X,Shadelight_Y,Stereo_Separation
      INTEGER IERR,Iuse_persp_Labs,Label_atoms_Flag,Lcurrent_Colour,
     +        Ldraw_cell_Flag,Ljoin_Flag,Lorthogonalisation_Code,
     +        Lpack_Flag,Lpaper_Setting,Lperspective_Flag,
     +        Lplot_colour_Stereo,Lposition_Flag,Lprint_Flag,
     +        LShade_Flag,LSolid_Flag,LStereo_Flag,LSuperimpose_Flag,
     +        LTaper_Flag,LTrace_Flag,NAT,NATO,Natoms_plusCell,NC,NC1,
     +        Ncopies,NDrawing_Number,Nellipise,NQuads_Overlaped,
     +        Nspecified_Bonds,Num_Bnd_Quadrangles,Num_Conics,Num_Moles,
     +        Num_Overlaps
      LOGICAL Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL AID,Atom_Radii,Base_Vect_Refsys,Base_Vect_Wrksys,BBB,
     +     Drawing_Origin,Ellipse_Conics,Ellipse_Resolution,
     +     Fractional_XYZ,G_Metric_Tensor,OrtCell,OrtConts,
     +     Orthogonal_XYZ,Overlap_Conics,P1_to_Refsys,P1_to_Wrksys,
     +     Plotting_XYZ,Principal_axis_rms,Prin_Angstrom_Vectors,
     +     Prin_Axis_Col_Vects,Q,Quadrangles,Quads_Overlaped,
     +     Recip_Metric_Tensor,RP,Segments,Sorted_xyz,Title_Rotation,
     +     Title_RotnTrans,ViewMatrix,XABC,XCLANG,XO
      INTEGER IAtom_Connectivity,IAtom_Type,ISpecified_Bond_Params,KC,
     +        KEllipseTypes,KQ,KSortPointer,LAtom_Inc_Flag,
     +        LAtom_Label_Flag,MOL,NEllipsoidStyle
C     ..
C     .. Local Scalars ..
      REAL AIN0,AIN1,BORDER_WIDTH,T1,T2,T3,T4,T5
      INTEGER I,ICQ,IES,II,II0,II1,II2,J,JDO,K,KSORTED,KSYMNUM,KWORK,L,
     +        LINES,LNS,M,N,NBIS,NCYCLE,NDASH,NDOT,NFIRST,NG,NPLANE,
     +        NRES1,NRESOL,NSOLID,NUMSUBDIVI,NWORK
C     ..
C     .. Local Arrays ..
      REAL CIRCLELINES_2D(2,130),CIRCLESPOKES(3,130),
     +     CONJUGATEVECTORS(3,3),EYE(3),QMATRIX(3,3),V1(4),V2(3),V3(3),
     +     V4(3),V5(3),V6(3),V7(3),VIEWV(3),XTT(3)
C     ..
C     .. External Functions ..
      REAL vectbymatrxbyvector,vectbyvecttranspose
      EXTERNAL vectbymatrxbyvector,vectbyvecttranspose
C     ..
C     .. External Subroutines ..
      EXTERNAL atmoverlapchk,generateellipse,getqmatrix,getunitvectors,
     +         matrxbyvector,ortdraw,project2dto3d,vectordifference,
     +         vectorproduct,vecttimesmatrx
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,INT,MOD,NINT,REAL,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_ELLIPSESTYLE/NEllipsoidStyle(Max_Atoms),Nellipise,
     +       KEllipseTypes(6)
      COMMON /com_OVERLAP/Num_Conics,Num_Overlaps,NQuads_Overlaped,
     +       Num_Bnd_Quadrangles,KC(Max_Conics),KQ(Max_Quadrangles),
     +       Ellipse_Conics(7,Max_Atoms),Overlap_Conics(6,Max_Conics),
     +       Quads_Overlaped(3,4,Max_Quadrangles),
     +       Quadrangles(9,2*Max_Atoms),Segments(Max_Segments,2)
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      BORDER_WIDTH = 1.0
C
C
      Iuse_persp_Labs = 0
      NG = 0
C
C---- establish reference point of view
C
      T1 = 1.0E6
C
C
      IF (Eye_Distance.GT.0.0) T1 = Eye_Distance/Scal_pre_Proj
C
C
      DO 10 JDO = 1,3
        EYE(JDO) = ViewMatrix(JDO,3)*T1 + Drawing_Origin(JDO)
   10 CONTINUE
C
C
      LNS = -1
C
C---- set ellipsoid graphic details
C
      NFIRST = 1
C
C---- NEllipsoidStyle packed as
C
C   ==>      NPLANE + NDOT*10 + NLINE*100 +
C             NDASH*1000 + AIN0*10000 + AIN1*1000000
C
C   if NSOLID/NDOT negative whole number negated
C
C
C
      IES = ABS(NEllipsoidStyle(ITOM))
      NWORK = IES/100000
      AIN1 = REAL(NWORK)/100.0
      KWORK = IES - NWORK*100000
      AIN0 = REAL(KWORK)/10000.0
      NWORK = KWORK - INT(AIN0)*10000
      AIN0 = AIN0/10.0
      NDASH = NWORK/1000
      KWORK = NWORK - NDASH*1000
      LINES = KWORK/100
      NWORK = KWORK - LINES*100
      NSOLID = NWORK/10
      NDOT = 64/2** (ABS(NSOLID))
      KWORK = NWORK - NSOLID*10
      NPLANE = KWORK
      IF (NEllipsoidStyle(ITOM).LT.0) NSOLID = -NSOLID
C
C
      IF ((NPLANE-1).EQ.0) THEN
        NFIRST = 4
        NPLANE = 4
      END IF
C
C
      KSORTED = ITOM
      XTT(1) = Plotting_XYZ(1,KSORTED)
      XTT(2) = Plotting_XYZ(2,KSORTED)
      XTT(3) = Plotting_XYZ(3,KSORTED)
C
C---- call overlap routine
C
      ICQ = 0
C
C          **********************************
      CALL atmoverlapchk(KSORTED,ICQ,LDOUBLE)
C          **********************************
C
ccx             IF (LDOUBLE .GT. 0) RETURN
C
C---- omit hidden atom
C
      IF (ICQ.EQ.-1) RETURN
C
C
      IF (NPLANE.LE.0) GO TO 370
C
C---- ellipsoid princ vectors toward viewer
C
      KSYMNUM = 1
C
C          ************************************
      CALL getqmatrix(2,QMATRIX,KSYMNUM,KSORTED)
      CALL vectordifference(EYE,XTT,VIEWV)
C          ************************************
C
      V7(1) = VIEWV(1)
      V7(2) = VIEWV(2)
      V7(3) = VIEWV(3)
C
C          ************************************
      CALL getunitvectors(V7,VIEWV,1)
      CALL vecttimesmatrx(VIEWV,ViewMatrix,V2)
C          ************************************
C
      DO 40 I = 1,3
        IF ((vectbyvecttranspose(V2,Prin_Angstrom_Vectors(1,I))).LT.
     +      0.0) THEN
C
C
          DO 30 J = 1,3
            Prin_Axis_Col_Vects(J,I) = -Prin_Axis_Col_Vects(J,I)
            Prin_Angstrom_Vectors(J,I) = -Prin_Angstrom_Vectors(J,I)
   30     CONTINUE
        END IF
C
C
   40 CONTINUE
C
C
      DO 50 J = 1,3
        Prin_Axis_Col_Vects(J,4) = Prin_Axis_Col_Vects(J,1)
        Prin_Axis_Col_Vects(J,5) = Prin_Axis_Col_Vects(J,2)
   50 CONTINUE
C
C---- v4 = vector normal to polar plane
C
C          ***********************************
      CALL vecttimesmatrx(VIEWV,ViewMatrix,V6)
C          ***********************************
C
      V7(1) = V6(1)
      V7(2) = V6(2)
      V7(3) = V6(3)
C
C          ****************************
      CALL getunitvectors(V7,V6,1)
      CALL matrxbyvector(QMATRIX,V6,V4)
C          ****************************
C
      V7(1) = V4(1)
      V7(2) = V4(2)
      V7(3) = V4(3)
C
C          ***********************
      CALL getunitvectors(V7,V4,1)
C          ***********************
C
C---- set plotting resolution for ellipsoid
C
      T3 = Principal_axis_rms(3)*SCL
      NRESOL = 1
      NBIS = 5
C
C
      DO 60 J = 1,3
        IF ((T3-Ellipse_Resolution(J)).GE.0.0) GO TO 70
        NBIS = NBIS - 1
        NRESOL = NRESOL*2
   60 CONTINUE
C
C
   70 NRES1 = NRESOL + 1
C
C---- loop through princ and polar planes
C
      DO 360 II = NFIRST,NPLANE
        II0 = MOD(II+2,3) + 1
        II1 = MOD(II,3) + 1
        II2 = MOD(II+1,3) + 1
C
C---- generate conjugate diameters
C
        IF ((0.99938-ABS(vectbyvecttranspose(V4,Prin_Axis_Col_Vects(1,
     +      II2)))).LE.0.0) THEN
          T1 = Principal_axis_rms(II0)*SCL
          T2 = Principal_axis_rms(II1)*SCL
C
C
          DO 80 J = 1,3
            CONJUGATEVECTORS(J,1) = Prin_Axis_Col_Vects(J,II0)*T1
            CONJUGATEVECTORS(J,2) = Prin_Axis_Col_Vects(J,II1)*T2
   80     CONTINUE
C
C
          GO TO 100
        END IF
C
C            ***************************************
        CALL vectorproduct(Prin_Axis_Col_Vects(1,II0),
     +                     Prin_Axis_Col_Vects(1,II1),V1,1)
        CALL vectorproduct(V1,V4,V2,1)
        CALL getunitvectors(V2,V2,1)
        CALL matrxbyvector(QMATRIX,V2,V3)
C            ***************************************
C
        IF ((II-4).LT.0) THEN
C
C              *************************
          CALL vectorproduct(V3,V1,V5,1)
C              *************************
        ELSE
C
C              *************************
          CALL vectorproduct(V3,V4,V5,1)
C              *************************
C
        END IF
C
C            ***********************
        CALL getunitvectors(V5,V5,1)
C            ***********************
C
        T1 = SCL/SQRT(vectbymatrxbyvector(V2,QMATRIX,V2))
        T2 = SCL/SQRT(vectbymatrxbyvector(V5,QMATRIX,V5))
C
C
        DO 90 J = 1,3
          CONJUGATEVECTORS(J,1) = V2(J)*T1
          CONJUGATEVECTORS(J,2) = V5(J)*T2
   90   CONTINUE
C
C---- generate ellipse
C
  100   NUMSUBDIVI = NBIS
C
C            *********************************************************
        CALL generateellipse(NUMSUBDIVI,CIRCLESPOKES,CONJUGATEVECTORS)
C            *********************************************************
C
        IF ((II-4).LT.0) GO TO 170
C
C
        IF (NSOLID.GT.0) THEN
C
C---- plot dotted boundary ellipse
C
          IF ((NDOT-NRESOL).LT.0) THEN
            NUMSUBDIVI = NSOLID - 1
C
C                ****************************************
            CALL generateellipse(NUMSUBDIVI,CIRCLESPOKES,
     +                           CONJUGATEVECTORS)
C                ****************************************
C
          END IF
C
C              *************************************************
          CALL project2dto3d(CIRCLESPOKES,CIRCLELINES_2D,XTT,XO,
     +                       Eye_Distance,1,129,NDOT)
C              *************************************************
C
C
          DO 130 J = 1,129,NDOT
C
C                *******************************************
            CALL ortdraw(CIRCLELINES_2D(1,J),Retrace_Displmnt,
     +                   Retrace_Displmnt,3)
C                *******************************************
C
            DO 120 I = 1,3,2
              T1 = REAL(I-2)*Retrace_Displmnt
              DO 110 K = 1,3,2
                T2 = REAL(K-2)*Retrace_Displmnt
C
C                    ************************************
                CALL ortdraw(CIRCLELINES_2D(1,J),T1,T2,2)
C                    ************************************
C
                IF (Retrace_Displmnt.LE.0.0) GO TO 130
  110         CONTINUE
  120       CONTINUE
  130     CONTINUE
C
C
          GO TO 360
        END IF
C
C---- plot solid boundary ellipse
C
C            ******************************************
        CALL project2dto3d(CIRCLESPOKES,CIRCLELINES_2D,XTT,XO,
     +                     Eye_Distance,1,129,NRESOL)
        CALL ortdraw(CIRCLELINES_2D,0.0,0.0,3)
C            ******************************************
C
        DO 140 J = NRES1,129,NRESOL
C
C              ************************************
          CALL ortdraw(CIRCLELINES_2D(1,J),0.0,0.0,2)
C              ************************************
C
  140   CONTINUE
C
C
        IF (Retrace_Displmnt.LE.0.0) GO TO 360
C
C---- boundary annulus as a linear function of height
C
C
C            **************************************
        CALL vectordifference(XTT,Drawing_Origin,V1)
C            **************************************
C
        T5 = vectbyvecttranspose(V1,ViewMatrix(1,3))*Scal_pre_Proj
        NCYCLE = NINT((AIN1*T5+AIN0)/Retrace_Displmnt+0.5)
C
C
        IF (NCYCLE.LE.0) GO TO 360
        T3 = (2.0*Retrace_Displmnt)/ (T1+T2)
C
C---- increase annular thickness
C
        DO 160 I = 1,NCYCLE
          T4 = REAL(I)*T3
          DO 150 J = 1,129,NRESOL
C
C                *******************************
            CALL ortdraw(CIRCLELINES_2D(1,J),
     +                   CIRCLESPOKES(1,J)*T4,
     +                   CIRCLESPOKES(2,J)*T4,2)
C                  *******************************
C
  150     CONTINUE
  160   CONTINUE
C
C
        GO TO 360
C
C            ******************************************
  170   CALL project2dto3d(CIRCLESPOKES,CIRCLELINES_2D,XTT,XO,
     +                     Eye_Distance,1,65,NRESOL)
C            ******************************************
C
C---- plot half an ellipse
C
C            *********************************
        CALL ortdraw(CIRCLELINES_2D,0.0,0.0,3)
C            *********************************
C
        DO 180 J = NRES1,65,NRESOL
C
C              **************************************
          CALL ortdraw(CIRCLELINES_2D(1,J),0.0,0.0,2)
C              **************************************
C
  180   CONTINUE
C
C
        IF (Retrace_Displmnt.GT.0.0) THEN
C
C---- accentuate front half
C
          DO 210 I = 1,3,2
            T2 = REAL(I-2)*Retrace_Displmnt
C
C
            DO 190 J = 1,65,NRESOL
              K = 66 - J
C
C                  **************************************************
              CALL ortdraw(CIRCLELINES_2D(1,K),Retrace_Displmnt,T2,2)
C                  **************************************************
C
  190       CONTINUE
C
C
            DO 200 K = 1,65,NRESOL
C
C                  ****************************************************
              CALL ortdraw(CIRCLELINES_2D(1,K),-Retrace_Displmnt,-T2,2)
C                  ****************************************************
C
  200       CONTINUE
  210     CONTINUE
C
C
        END IF
C
C
        IF (NSOLID.GT.0) THEN
          L = NDOT
          IF ((NDOT-NRESOL).LT.0) THEN
            NUMSUBDIVI = NSOLID - 1
C
C                ****************************************
            CALL generateellipse(NUMSUBDIVI,CIRCLESPOKES,
     +                           CONJUGATEVECTORS)
C                ***************************************
C
          ELSE IF (NDOT-NRESOL.GT.0) THEN
            L = NRESOL
          END IF
        ELSE IF (NSOLID.LT.0) THEN
          L = NRESOL
        ELSE
          GO TO 260
        END IF
C
C            *****************************************
        CALL project2dto3d(CIRCLESPOKES(1,65),
     +                     CIRCLELINES_2D(1,65),XTT,
     +                     XO,Eye_Distance,1,65,L)
C            ******************************************
C
        IF (NSOLID.GT.0) THEN
C
C---- dotted line on reverse side
C
          DO 240 J = 65,129,NDOT
C
C                *******************************************
            CALL ortdraw(CIRCLELINES_2D(1,J),Retrace_Displmnt,
     +                   Retrace_Displmnt,3)
C                *******************************************
C
            DO 230 I = 1,3,2
              T1 = REAL(I-2)*Retrace_Displmnt
              DO 220 K = 1,3,2
                T2 = REAL(K-2)*Retrace_Displmnt
C
C                    ************************************
                CALL ortdraw(CIRCLELINES_2D(1,J),T1,T2,2)
C                    ************************************
C
                IF (Retrace_Displmnt.LE.0.0) GO TO 240
  220         CONTINUE
  230       CONTINUE
  240     CONTINUE
C
C
        ELSE IF (NSOLID.LT.0) THEN
C
C---- single line on reverse side
C
          DO 250 J = 65,129,NRESOL
C
C                **************************************
            CALL ortdraw(CIRCLELINES_2D(1,J),0.0,0.0,2)
C                **************************************
C
  250     CONTINUE
C
C
        END IF
C
C---- detail interior features
C
  260   T2 = NDASH*2
C
C
        DO 270 J = 1,3
          T1 = Prin_Axis_Col_Vects(J,II0)*Principal_axis_rms(II0)*SCL
          CONJUGATEVECTORS(J,1) = T1
          CONJUGATEVECTORS(J,2) = Prin_Axis_Col_Vects(J,II1)*
     +                            Principal_axis_rms(II1)*SCL
          CONJUGATEVECTORS(J,3) = 0.0
C
C
          IF (NDASH.GT.0) THEN
            V1(J) = -T1
            V2(J) = T1/T2
          END IF
C
C
  270   CONTINUE
C
C
        IF (NDASH.GT.0) THEN
C
C---- dashed line for reverse axis
C
          DO 300 J = 1,NDASH
            DO 290 K = 1,2
              L = 4 - K
C
C                  *********************************
              CALL project2dto3d(V1,CIRCLELINES_2D,XTT,
     +                           XO,Eye_Distance,1,1,1)
              CALL ortdraw(CIRCLELINES_2D,0.0,0.0,L)
C                  *********************************
C
              DO 280 I = 1,3
                V1(I) = V1(I) + V2(I)
  280         CONTINUE
  290       CONTINUE
  300     CONTINUE
C
C
        END IF
C
C---- solid line for forward axis
C
        IF (LINES.LE.0) GO TO 360
C
C            ****************************************************
        CALL project2dto3d(CONJUGATEVECTORS,CIRCLELINES_2D,XTT,XO,
     +                     Eye_Distance,1,3,1)
C            ****************************************************
C
        T1 = Retrace_Displmnt*0.5
C
C
        DO 310 I = 1,3,2
          T2 = REAL(2-I)*T1
C
C              ************************************
          CALL ortdraw(CIRCLELINES_2D,T1,T2,3)
          CALL ortdraw(CIRCLELINES_2D(1,3),T1,T2,2)
C              ************************************
C
          IF (Retrace_Displmnt.LE.0.0) GO TO 320
C
C              *************************************
          CALL ortdraw(CIRCLELINES_2D(1,3),-T1,T2,2)
          CALL ortdraw(CIRCLELINES_2D,-T1,T2,2)
C              **************************************
C
  310   CONTINUE
C
C---- shade quadrant between two principal axes
C
  320   L = LINES - 1
C
C
        IF (L.GT.0) THEN
          T2 = LINES
C
C
          DO 340 I = 1,L
            T1 = REAL(I)/T2
            T3 = SQRT(1.0-T1*T1)
C
C
            IF (MOD(I,2).NE.0) THEN
              M = I*2
              N = M - 1
            ELSE
              N = I*2
              M = N - 1
            END IF
C
C
            DO 330 J = 1,3
              T4 = CONJUGATEVECTORS(J,1)*T1
              CIRCLESPOKES(J,M) = T4
              CIRCLESPOKES(J,N) = CONJUGATEVECTORS(J,2)*T3 + T4
  330       CONTINUE
  340     CONTINUE
C
C
          L = L*2
C
C              ******************************************
          CALL project2dto3d(CIRCLESPOKES,CIRCLELINES_2D,XTT,XO,
     +                       Eye_Distance,1,L,1)
C              ******************************************
C
          DO 350 I = 2,L,2
C
C                ****************************************
            CALL ortdraw(CIRCLELINES_2D(1,I-1),0.0,0.0,3)
            CALL ortdraw(CIRCLELINES_2D(1,I),0.0,0.0,2)
C                ****************************************
C
  350     CONTINUE
C
C
        END IF
C
C
  360 CONTINUE
C
C
  370 CONTINUE
      RETURN
      END
C
C
C
C     ====================
      SUBROUTINE drawgrid
C     ====================
C
C---- draw grid at intervals of
C      Ugrid_Spacing,Vgrid_Spacing map grid units
C
C     grid origin at
C      Ugrid_Offset,Vgrid_Offset relative to map origin
C
C     major grid lines at intervals of
C             Major_Grid,
C
C     grid style in MapGrdCol
C
C
C
C
C
C
C---- get map limits in units of number of grid spacings
C     NB function mint gives next higher integer
C        if 2nd argument positive,
C        next lower if 2nd argument negative
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
C     ..
C     .. Scalars in Common ..
      REAL Ugrid_Offset,Ugrid_Spacing,Vgrid_Offset,Vgrid_Spacing
      INTEGER Invert_Map_Flag,LGRID,LimitsSet_Flag,Lmap3d_Flag,
     +        Lmap_ModeFirst,LSetView_Flag,Map_Counter,Map_Flag_Counter,
     +        Map_Nobox,Map_pts_U,Map_pts_V,MU,MV,MW,Npicture,
     +        Npts_U_compressed,Npts_V_compressed,LPERM
C     ..
C     .. Arrays in Common ..
      REAL AQ,Contour_Levels,Contour_Threshold
      INTEGER Major_Grid,MapGrdCol,Map_def_Restrictions,Map_LineTypes,
     +        Map_UVW_Limits,Map_XYZ_Limits,Num_Contours
      CHARACTER Map_Title*80
C     ..
C     .. Local Scalars ..
      REAL T,X1,X2,Y1,Y2
      INTEGER I,K,M,NU,NU1,NU2,NV,NV1,NV2
C     ..
C     .. Local Arrays ..
      REAL X(2),Y(2)
C     ..
C     .. External Functions ..
      INTEGER mint
      EXTERNAL mint
C     ..
C     .. External Subroutines ..
      EXTERNAL joincoordpairs,setplotstyle
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MOD,REAL
C     ..
C     .. Common blocks ..
      COMMON /com_GRID/LGRID,Map_Nobox,Ugrid_Spacing,Vgrid_Spacing,
     +       Ugrid_Offset,Vgrid_Offset,Major_Grid(Max_Maps),
     +       MapGrdCol(4,Max_Maps,2)
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      NU1 = mint(((Map_def_Restrictions(1)-Ugrid_Offset) / 
     +            Ugrid_Spacing),  1)
      NU2 = mint(((Map_def_Restrictions(2)-Ugrid_Offset) / 
     +            Ugrid_Spacing), -1)
      NV1 = mint(((Map_def_Restrictions(3)-Vgrid_Offset) / 
     +            Vgrid_Spacing),  1)
      NV2 = mint(((Map_def_Restrictions(4)-Vgrid_Offset) / 
     +            Vgrid_Spacing), -1)
C
C---- number of lines
C
      NU = NU2 - NU1 + 1
      NV = NV2 - NV1 + 1
C
C---- draw lines parallel to v
C
      Y1 = 0.0
      Y2 = Npts_V_compressed - 1
C
C---- start point
C
      X1 = REAL(NU1)*Ugrid_Spacing - REAL(Map_def_Restrictions(1)) +
     +     Ugrid_Offset
C
C
      DO 10 I = 1,NU
C
C---- style for this line
C
        K = 1
C
C---- m is grid line number from grid origin
C
        M = ABS(I+NU1-1)
C
C---- major grid line if multiple of major grid interval
C
        IF (MOD(M,Major_Grid(1)).EQ.0) K = 2
C
C---- set plotting style for this line
C
C            ******************************
        CALL setplotstyle(MapGrdCol(1,K,1))
C            ******************************
C
        Y(1) = Y1
        Y(2) = Y2
        X(1) = X1
        X(2) = X(1)
C
C            *********************
        CALL joincoordpairs(X,Y,2)
C            *********************
C
C---- interchange y coordinates
C
        T = Y1
        Y1 = Y2
        Y2 = T
        X1 = X1 + Ugrid_Spacing
   10 CONTINUE
C
C---- draw lines along u
C
      X1 = 0.0
      X2 = Npts_U_compressed - 1
      Y1 = REAL(NV1)*Vgrid_Spacing - 
     +     REAL(Map_def_Restrictions(3)) + Vgrid_Offset
C
C
      DO 20 I = 1,NV
C
C---- style
C
        K = 1
        M = ABS(I+NV1-1)
        IF (MOD(M,Major_Grid(Max_Maps)).EQ.0) K = 2
C
C            ******************************
        CALL setplotstyle(MapGrdCol(1,K,2))
C            ******************************
C
        X(1) = X1
        X(2) = X2
        Y(1) = Y1
        Y(2) = Y(1)
C
C            *********************
        CALL joincoordpairs(X,Y,2)
C            *********************
C
        T = X1
        X1 = X2
        X2 = T
        Y1 = Y1 + Vgrid_Spacing
   20 CONTINUE
C
      RETURN
      END
C
C
C
C     =================================
      SUBROUTINE drawlines(X1,Y1,X2,Y2)
C     =================================
C
C---- Draw line from x1,y1 to x2,y2
C
C
C
C     .. Scalar Arguments ..
      REAL X1,X2,Y1,Y2
C     ..
C     .. Scalars in Common ..
      REAL A1,B1,DASH,DOT,REPEAT
      INTEGER LPT,MODE
C     ..
C     .. Local Arrays ..
      REAL X(2),Y(2)
C     ..
C     .. External Subroutines ..
      EXTERNAL drawlineset,zdrawto,zmoveto
C     ..
C     .. Common blocks ..
      COMMON /com_DASH/LPT,MODE,REPEAT,DASH,DOT,A1,B1
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      IF (MODE.GT.0) THEN
C
C---- dashed
C
        X(1) = X1
        Y(1) = Y1
        X(2) = X2
        Y(2) = Y2
C
C            ******************
        CALL drawlineset(X,Y,2)
C            ******************
C
      ELSE
C
C---- solid line
C
C            *************
        CALL zmoveto(X1,Y1)
        CALL zdrawto(X2,Y2)
C            *************
C
      END IF
C
C
      END
C
C
C
C     ===============================
      SUBROUTINE drawlineset(X,Y,NPT)
C     ===============================
C
C
C
C---- draw lines to join npt points x,y
C
C
C     .. Scalar Arguments ..
      INTEGER NPT
C     ..
C     .. Array Arguments ..
      REAL X(NPT),Y(NPT)
C     ..
C     .. Scalars in Common ..
      REAL A1,B1,DASH,DOT,REPEAT
      INTEGER LPT,MODE
C     ..
C     .. Local Scalars ..
      REAL A,B,D,DX,DY,GAP,R,SMALL,X1,X2,XD,XG,Y1,Y2,YD,YG
      INTEGER I,J
C     ..
C     .. External Subroutines ..
      EXTERNAL zdrawby,zdrawto,zmoveby,zmoveto
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_DASH/LPT,MODE,REPEAT,DASH,DOT,A1,B1
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Data statements ..
      DATA SMALL/0.001/
C     ..
C
C---- mode =0 for solid lines
C
      IF (NPT.LE.1) RETURN
      IF (MODE.GT.0) GO TO 20
C
C---- solid
C
C          *****************
      CALL zmoveto(X(1),Y(1))
C          *****************
C
      DO 10 I = 2,NPT
C
C            *****************
        CALL zdrawto(X(I),Y(I))
C            *****************
C
   10 CONTINUE
      RETURN
C
C---- dashed lines, repeat is repeat length, dash is dash length
C      dot is dummy (no chain lines)
C
   20 GAP = REPEAT - DASH
      A = DASH
      B = GAP
C
C---- lpt=0 if 1st call since call to s/r dashed,
C     otherwise keep dashes in phase
C
      IF (LPT.EQ.0) GO TO 30
      A = A1
      B = B1
C
   30 LPT = 1
      X1 = X(1)
      Y1 = Y(1)
      I = 2
C
C          *************
      CALL zmoveto(X1,Y1)
C          *************
C
C
C---- come here for new line
C
   40 X2 = X(I)
      Y2 = Y(I)
      DX = X2 - X1
      DY = Y2 - Y1
      D = SQRT(DX*DX+DY*DY)
      IF (D.LT.SMALL) GO TO 110
      DX = DX/D
      DY = DY/D
      R = D
      A1 = 0.0
      B1 = 0.0
      J = -1
C
C---- r is remaining line length
C     come here for each dash repeat
C
   50 IF (R.LT. (A+B)) GO TO 60
      IF (J.LE.0) THEN
        GO TO 90
      ELSE
        GO TO 100
      END IF
C
C---- last bit, get remaining dash length
C
   60 IF (R.LT.A) GO TO 70
C
C---- last part is in gap
C
      A1 = 0.0
      B1 = B - R + A
      B = R - A
      GO TO 80
C
C---- last part is in dash
C
   70 A1 = A - R
      B1 = GAP
      A = R
      B = 0.0
   80 J = -2
C
   90 J = J + 1
      XD = A*DX
      YD = A*DY
      XG = B*DX
      YG = B*DY
C
C                     ******************
  100 IF (A.GT.SMALL) CALL zdrawby(XD,YD)
      IF (B.GT.SMALL) CALL zmoveby(XG,YG)
C                     ******************
C
      R = R - A - B
      IF (J.LT.0) THEN
        GO TO 110
      ELSE IF (J.GT.0) THEN
        GO TO 50
      END IF
C
C---- reset dash length after 1st repeat
C
      A = DASH
      B = GAP
      J = 0
      GO TO 50
C
C---- end of line, restore unused dash length
C
  110 I = I + 1
      IF (I.GT.NPT) RETURN
      X1 = X2
      Y1 = Y2
      A = A1
      B = B1
      GO TO 40
C
      END
C
C
C
C     =========================
      SUBROUTINE drawsolidatoms
C     =========================
C
C
C
C
C
C---- NEllipsoidStyle packed as
C
C   ==>      NPLANE + NDOT*10 + NLINE*100 +
C             NDASH*1000 + AIN0*10000 + AIN1*1000000
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_BondType
      PARAMETER (Max_BondType=20)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Colours
      PARAMETER (Max_Colours=10)
      INTEGER Max_Conics
      PARAMETER (Max_Conics=20)
      INTEGER Max_Hidden_Lines
      PARAMETER (Max_Hidden_Lines=2000)
      INTEGER Max_Labels
      PARAMETER (Max_Labels=100)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER Max_Quadrangles
      PARAMETER (Max_Quadrangles=30)
      INTEGER Max_Segments
      PARAMETER (Max_Segments=50)
      INTEGER Max_Symmetry
      PARAMETER (Max_Symmetry=192)
      INTEGER NDefault_Colour
      PARAMETER (NDefault_Colour=1)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Charcter_Size,
     +     CurrentCharcter_Size,Def_Bnd_Radius,Def_Char_Size,
     +     Def_Paper_Wdth,Def_Plt_Radius,Def_Plt_Size,Dist_to_Boundary,
     +     Drawing_Size,Ellipse_ScaleFact,Eye_Distance,Overlap_Margin,
     +     Paper_Length,Paper_Width,Perspective_Distance,Picture_Scale,
     +     Plotter_Step,Retrace_Displmnt,S,Scal_pre_Proj,SCL,
     +     Shadelight_X,Shadelight_Y,Stereo_Separation
      INTEGER IERR,Iuse_persp_Labs,Label_atoms_Flag,Lcurrent_Colour,
     +        Ldef_Bnd_Lines,Ldraw_cell_Flag,Ljoin_Flag,Lpack_Flag,
     +        Lpaper_Setting,Lperspective_Flag,Lplot_colour_Stereo,
     +        Lposition_Flag,Lprint_Flag,LShade_Flag,LSolid_Flag,
     +        LStereo_Flag,LSuperimpose_Flag,LTaper_Flag,LTrace_Flag,
     +        NAT,NATO,Natoms_plusCell,NC,NC1,Ncopies,NDrawing_Number,
     +        Nellipise,NQuads_Overlaped,Nspecified_Bonds,
     +        Num_Bnd_Quadrangles,Num_Conics,Num_Labtxt_input,
     +        Num_LinkSets,Num_Moles,Num_Overlaps,Num_Posit_Labels
      LOGICAL Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
      CHARACTER Iden_Title*80
C     ..
C     .. Arrays in Common ..
      REAL Atom_Radii,Base_Vect_Refsys,Base_Vect_Wrksys,BBB,Def_LnkBond,
     +     Drawing_Origin,Ellipse_Conics,Ellipse_Resolution,
     +     Fractional_XYZ,G_Metric_Tensor,Orthogonal_XYZ,Overlap_Conics,
     +     P1_to_Refsys,P1_to_Wrksys,Plotting_XYZ,Positioned_Labels,
     +     Principal_axis_rms,Prin_Angstrom_Vectors,Prin_Axis_Col_Vects,
     +     Q,Quadrangles,Quads_Overlaped,Recip_Metric_Tensor,RP,
     +     Segments,Sorted_xyz,Title_Rotation,Title_RotnTrans,
     +     ViewMatrix,XLABEL,XO,YLABEL
      INTEGER IAtom_Connectivity,IAtom_Type,IConn_BndType,IResid_BndTyp,
     +        ISpecified_Bond_Params,ISym_BndType,KC,KEllipseTypes,KQ,
     +        KSortPointer,LAtom_Inc_Flag,LAtom_Label_Flag,MOL,
     +        NEllipsoidStyle
      CHARACTER Atm_Lab_Type*2,Link_Atm_Names*2,Atm_Identifier*10,
     +          Atm_Label*10,Atm_Lab_Txt*10
C     ..
C     .. Local Scalars ..
      REAL ADOTB,ASQ,AX,AY,BSQ,BTHICK,BX,BY,DASH,DD,DIJSQ,DOT,DX,DY,DZ,
     +     PERPSQ,RBO,REPEAT,RI,RIJ,RIJSQ,RK,X1,X2,XHI,XI,XJ,XK,XLABL,
     +     XLO,XN,Y1,Y2,YHI,YI,YJ,YK,YLABL,YLO,YN,ZI,ZK,ZN
      INTEGER I,I1,I2,IB,IDASH,II,INCI,INCJ,INSTR,IO,J,JCOLR,JDASH,JDOT,
     +        JMODE,JO,JREPET,K,KO,L,LCOLR,LDOUBL,M,MM,N,N1,N2,N3,NA1,
     +        NA2,NAB,NJ,NLI,NOBSC
      CHARACTER LSTRNG*80
C     ..
C     .. Local Arrays ..
      INTEGER IOBSC(Max_Hidden_Lines)
C     ..
C     .. External Subroutines ..
      EXTERNAL bondin,drawcircles,drawellipsoids,ortbond,setdashparams,
     +         zdrawto,zmoveto,zplotstring,zsetpencolour,
     +         zsettextposition
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MAX,MIN,MOD
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_DEFAULTS/Ldef_Bnd_Lines,Def_Bnd_Radius,Def_Plt_Radius,
     +       Def_Char_Size,Def_Plt_Size,Def_Paper_Wdth
      COMMON /com_ELLIPSESTYLE/NEllipsoidStyle(Max_Atoms),Nellipise,
     +       KEllipseTypes(6)
      COMMON /com_LABCHR/Atm_Lab_Txt(Max_Labels),
     +       Atm_Lab_Type(Max_Labels),Atm_Label(Max_Labels),
     +       Atm_Identifier(Max_Atoms),Iden_Title
      COMMON /com_LABELS/Num_Posit_Labels,Num_Labtxt_input,
     +       XLABEL(Max_Atoms),YLABEL(Max_Atoms),
     +       Positioned_Labels(2,Max_Labels)
      COMMON /com_LINK/Num_LinkSets,IResid_BndTyp(Max_Atoms),
     +       ISym_BndType(Max_Symmetry),IConn_BndType(Max_BondType),
     +       Def_LnkBond(2)
      COMMON /com_LNKCHR/Link_Atm_Names(2,2)
      COMMON /com_OVERLAP/Num_Conics,Num_Overlaps,NQuads_Overlaped,
     +       Num_Bnd_Quadrangles,KC(Max_Conics),KQ(Max_Quadrangles),
     +       Ellipse_Conics(7,Max_Atoms),Overlap_Conics(6,Max_Conics),
     +       Quads_Overlaped(3,4,Max_Quadrangles),
     +       Quadrangles(9,2*Max_Atoms),Segments(Max_Segments,2)
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      NAB = NAT - 8
C
C---- set current colour=default colour
C
      LCOLR = Lcurrent_Colour
C
C---- plot circles....
C
      DO 110 I = 9,Natoms_plusCell
        IO = MOD(I-9,NAB) + 9
        IF (LAtom_Inc_Flag(IO).LE.0) GO TO 110
C
C---- atom colour (nb LAtom_Inc_Flag reset by
C     s/r getatomconnections to jcolr+1)
C
        IF (LAtom_Inc_Flag(I).GT.100) THEN
          LDOUBL = 1
          L = MOD(LAtom_Inc_Flag(I),100) - 1
        ELSE
          LDOUBL = 0
          L = LAtom_Inc_Flag(I) - 1
        END IF
C
C
        IF (L.GT.Max_Colours) L = NDefault_Colour
        IF (L.LE.0 .AND. LCOLR.EQ.Lcurrent_Colour) GO TO 20
C
C---- same colour ?
C
        IF (L.EQ.LCOLR) GO TO 20
C
C            ****************
        CALL zsetpencolour(L)
C            ****************
C
        LCOLR = L
   20   IF (LAtom_Label_Flag(IO).EQ.0) GO TO 50
        XLABL = XLABEL(IO)
        YLABL = YLABEL(IO)
C
C            *****************************
        CALL zsettextposition(XLABL,YLABL)
C            *****************************
C
        IF (LAtom_Label_Flag(IO).LT.100) GO TO 30
C
C---- get label text from array Atm_Lab_Txt
C
        M = LAtom_Label_Flag(IO)/100
        LSTRNG = Atm_Lab_Txt(M)
        GO TO 40
   30   LSTRNG = Atm_Identifier(I)
C
C
   40     KZMODE = 1
          KLT = LENSTR(LSTRNG)
C
C            ********************************
        CALL zplotstring(KZMODE,LSTRNG,1,KLT)
C            ********************************
C
   50   IF (LSolid_Flag.EQ.0) GO TO 110
        L = 1
        XI = Plotting_XYZ(1,I)
        YI = Plotting_XYZ(2,I)
        ZI = Plotting_XYZ(3,I)
        RI = RP(I)
C
C----  no hidden lines removal if LSolid_Flag=2
C
        NOBSC = 0
        IF (LSolid_Flag.EQ.2) GO TO 80
C
C---- get list of atoms j which obscure atom i
C     this allows for intersection of spheres
C     in van der waals models.
C
        DO 60 J = 9,Natoms_plusCell
          JO = MOD(J-9,NAB) + 9
          IF (LAtom_Inc_Flag(JO).LE.0) GO TO 60
          IF (J.EQ.I) GO TO 60
          IF (Plotting_XYZ(3,J)+RP(J).LT.ZI) GO TO 60
          RIJ = RP(J) + RI
          XJ = Plotting_XYZ(1,J)
          DX = XJ - XI
          IF (ABS(DX).GT.RIJ) GO TO 60
          YJ = Plotting_XYZ(2,J)
          DY = YJ - YI
          IF (ABS(DY).GT.RIJ) GO TO 60
          DIJSQ = DX*DX + DY*DY
          RIJSQ = RIJ*RIJ
          IF (DIJSQ.GT.RIJSQ) GO TO 60
C
C---- envelope circles intersect in projn down zp -
C      test now for sphere  intersection.
C
          NJ = J
          DZ = Plotting_XYZ(3,J) - Plotting_XYZ(3,I)
          DIJSQ = DZ*DZ + DIJSQ
C
C---- remove atoms which have identical coords to atom i.
C     intersecting spheres are marked with negative j.
C
          IF (DIJSQ.LT.0.001) GO TO 60
          IF (DIJSQ.LT.RIJSQ) NJ = -J
C
C---- enter this atom j in the obscuring list.
C
          IOBSC(L) = NJ
          L = L + 1
          IF (L.GT.Max_Atoms) GO TO 70
   60   CONTINUE
   70   IF (L.GT.Max_Hidden_Lines) WRITE (6,
     +      FMT=6000) Atm_Identifier(I),Max_Hidden_Lines
 6000   FORMAT (' Help*** More than ',I6,' atoms obscure',A)
        NOBSC = L - 1
C
C---- now we can draw the circle for atom i
C
C          if ldoubl ne 0, plot two concentric circles
C
   80   L = 0
C
C
        IF (UseOrtep) THEN
          IF (I.LT.9) GO TO 90
C
C             ************************
          CALL zsetpencolour(LCOLR)
          CALL drawellipsoids(I,LDOUBL)
C             ************************
C
ccx           IF (LDOUBL.GT.0) GO TO 100
          GO TO 110
        END IF
C
C
   90   CONTINUE
C
C            ***********************************************
        CALL drawcircles(I,Plotting_XYZ,RP,NOBSC,LShade_Flag,
     +                   Shadelight_X,Shadelight_Y,IOBSC)
C            ***********************************************
C
        IF (LDOUBL.LE.0) GO TO 110
        L = L + 1
        IF (L.GT.1) GO TO 100
C
C---- halve radius for 2nd circle
C
        RP(I) = RP(I)*0.5
        GO TO 90
C
C---- restore radius
C
  100   RP(I) = RP(I)*2.0
C
  110 CONTINUE
C
C
C---- plotting of bonds...
C
      IF (NC.LE.12 .AND. Ldraw_cell_Flag.EQ.0) GO TO 310
C
C---- reset to initial colour
C
      LCOLR = Lcurrent_Colour
C
C          ********************
      CALL zsetpencolour(LCOLR)
C          ********************
C
      DO 300 M = 1,Num_Moles
        MM = (M-1)* (NAT-8)
        I1 = 13
        IF (M.EQ.1 .AND. Ldraw_cell_Flag.EQ.2) I1 = 1
        I2 = NC
C
C---- loop through the list of connections
C     stored in array IAtom_Connectivity
C
        DO 290 II = I1,I2
C
C---- unpack bond type l and atom numbers
C
          JO = MOD(IAtom_Connectivity(II),Max_Atoms)
          L = IAtom_Connectivity(II)/Max_Atoms
          IO = MOD(L,Max_Bonds)
          L = L/Max_Bonds
          INCI = LAtom_Inc_Flag(IO)
          INCJ = LAtom_Inc_Flag(JO)
C
C---- omit intramolecular bonds when either atom is omitted,
C        LAtom_Inc_Flag=0
C
          IF (IO.LT.9 .OR. IO.GT.NATO .OR. JO.LT.9 .OR.
     +        JO.GT.NATO) GO TO 120
C
C---- include bonds unless both atoms are excluded & in same margin
C
          IF (INCI.GT.0 .OR. INCJ.GT.0) GO TO 120
          I = -INCI/5
          J = -INCJ/5
          IF (I+J.NE.1) GO TO 290
  120     I = IO + MM
          J = JO + MM
          IDASH = 0
C
C---- bond colour from 2nd atom if set
C
          JCOLR = MOD(ABS(INCI),11) - 1
          IF (JCOLR.LE.0) JCOLR = MOD(ABS(INCJ),11) - 1
          IF (JCOLR.LE.0) JCOLR = Lcurrent_Colour
          IF (JCOLR.GT.Max_Colours) JCOLR = NDefault_Colour
          IF (JCOLR.EQ.LCOLR) GO TO 130
C
C              ********************
          CALL zsetpencolour(JCOLR)
C              ********************
C
          LCOLR = JCOLR
C
  130     IF (LSolid_Flag.EQ.0) GO TO 280
          IF (Plotting_XYZ(3,I).GT.Plotting_XYZ(3,J)) GO TO 140
          K = I
          I = J
          J = K
C
C---- allocate bond radius and number of lines.
C
  140     CONTINUE
          IF (I.LT.9) GO TO 200
          IF (L.EQ.0) GO TO 150
C
C---- bond type from connectivity list
C
          L = IConn_BndType(L)
          GO TO 180
C
C---- get default bond type
C
  150     RBO = Def_Bnd_Radius
          NLI = Ldef_Bnd_Lines
          IF (RBO.GT.0.0) GO TO 160
C
C---- default type is dashed
C
          IDASH = 1
          L = -NLI
C
  160     IF (Nspecified_Bonds.EQ.0) GO TO 210
          L = 0
C
C
          DO 170 K = 1,Nspecified_Bonds
            N1 = ISpecified_Bond_Params(K,1)
            N2 = ISpecified_Bond_Params(K,2)
            N3 = ISpecified_Bond_Params(K,3)
C
C
            IF (N1.LT.0) THEN
C
C---- intermolecular
C
              IF (IO.GT.NATO .OR. JO.GT.NATO) L = N3
C
C---- atom type
C
            ELSE IF (N1.EQ.0) THEN
              IF (IAtom_Type(IO).EQ.N2 .OR. IAtom_Type(JO).EQ.N2) L = N3
C
C---- specific bond
C
            ELSE IF (N1.GT.0) THEN
              IF ((N1.EQ.IO.AND.N2.EQ.JO) .OR.
     +            (N1.EQ.JO.AND.N2.EQ.IO)) L = N3
            END IF
C
C
  170     CONTINUE
          IF (L.EQ.0 .AND. IDASH.EQ.0) GO TO 210
C
C----  L contains bond parameters
C
  180     IF (L.GT.0) GO TO 190
C
C---- dashed line
C
          L = -L
          JDOT = L/50000
          DOT = 0.1*JDOT
          L = L - 50000*JDOT
          JDASH = L/500
          DASH = 0.1*JDASH
          L = L - 500*JDASH
          JREPET = L/5
          REPEAT = 0.1*JREPET
          JMODE = L - 5*JREPET - 2
          L = -1000
          IDASH = 1
C
C              ************************************
          CALL setdashparams(JMODE,REPEAT,DASH,DOT)
C              ************************************
C
          GO TO 200
C
  190     RBO = MOD(L,1000)*0.001
          NLI = L/1000
          GO TO 210
  200     RBO = 0.0
          NLI = 1
  210     IB = J
C
C---- scale bond radius to mm. i j bond call ortbond
C
          IF (UseOrtep) THEN
            NA1 = I
            NA2 = J
            INSTR = 812
            IF (I.LT.9 .OR. J.LT.9) GO TO 220
            IF (L.LT.0) GO TO 220
            IF (NLI.GT.5) NLI = 5
C
C                ******************************
            CALL ortbond(INSTR,NA1,NA2,RBO,NLI)
C                ******************************
C
            GO TO 290
          END IF

          RBO = RBO*S
          NOBSC = 0
          IF (LSolid_Flag.EQ.2) GO TO 270
C
C---- search begins for atoms k which obscure this bond (i-n)
C
  220     BTHICK = RBO
          XI = Plotting_XYZ(1,I)
          YI = Plotting_XYZ(2,I)
          ZI = Plotting_XYZ(3,I)
          RI = RP(I)
          XN = Plotting_XYZ(1,IB)
          YN = Plotting_XYZ(2,IB)
          ZN = Plotting_XYZ(3,IB)
          AX = XN - XI
          AY = YN - YI
          ASQ = AX*AX + AY*AY
          IF (ASQ.LT.0.01) GO TO 290
          L = 1
          X1 = MIN(XI,XN)
          X2 = MAX(XI,XN)
          Y1 = MIN(YI,YN)
          Y2 = MAX(YI,YN)
C
C
          DO 250 K = 9,Natoms_plusCell
            KO = MOD(K-9,NAB) + 9
            IF (LAtom_Inc_Flag(KO).EQ.0) GO TO 250
            IF (K.EQ.I .OR. K.EQ.IB) GO TO 250
            IF (Plotting_XYZ(3,K).LT.ZN) GO TO 250
            ZK = Plotting_XYZ(3,K)
            RK = RP(K)
            XK = Plotting_XYZ(1,K)
            XLO = X1 - RK
            XHI = X2 + RK
            IF (XK.LT.XLO .OR. XK.GT.XHI) GO TO 250
            YK = Plotting_XYZ(2,K)
            YLO = Y1 - RK
            YHI = Y2 + RK
            IF (YK.LT.YLO .OR. YK.GT.YHI) GO TO 250
C
C---- if we get this far we apply the exact test for intersection.
C
            BX = XK - XI
            BY = YK - YI
            BSQ = BX*BX + BY*BY
            ADOTB = AX*BX + AY*BY
            PERPSQ = BSQ - ADOTB*ADOTB/ASQ
            DD = RK + BTHICK
            IF (PERPSQ.GT.DD*DD) GO TO 250
C
C---- ignore the atom for obscuring list
C     if within 0.1 mm of atom i or n
C
            IF (ABS(XK-XI).GT.0.1) GO TO 230
            IF (ABS(YK-YI).GT.0.1) GO TO 230
            IF (ABS(ZK-ZI).GT.0.1) GO TO 230
            GO TO 250
  230       IF (ABS(XK-XN).GT.0.1) GO TO 240
            IF (ABS(YK-YN).GT.0.1) GO TO 240
            IF (ABS(ZK-ZN).GT.0.1) GO TO 240
            GO TO 250
C
C----  this atom intersects the bond line i to n.
C
  240       IOBSC(L) = K
            L = L + 1
            IF (L.GT.Max_Hidden_Lines) GO TO 260
  250     CONTINUE
  260     IF (L.GT.Max_Hidden_Lines) WRITE (6,FMT=6002) I,IB,
     +        Max_Hidden_Lines
 6002     FORMAT (//' ***WARNING*** ',/'    More than ',I6,'  obscurin',
     +           'g atoms for bond ',2I5)
          NOBSC = L - 1
C
C---- now we can draw the bond i to n.
C
  270     N = IB
          IF (Lprint_Flag.NE.0) WRITE (6,FMT=6004) I,N,IO,JO,
     +        LAtom_Inc_Flag(IO),LAtom_Inc_Flag(JO),RBO,NLI,IDASH
 6004     FORMAT (' DRAW BOND:I,J,IO,JO,LAtom_Inc_Flag(IO), LAtom_Inc_',
     +           'Flag(J0),RBO,NLI,IDASH',6I5,F8.2,2I5)
C
C              ****************************************************
          CALL bondin(I,N,Plotting_XYZ,RP,NOBSC,LTaper_Flag,RBO,NLI,
     +                IOBSC)
C              ****************************************************
C
C                         *********************************
          IF (IDASH.EQ.1) CALL setdashparams(0,0.0,0.0,0.0)
C                         *********************************
C
          IDASH = 0
          GO TO 290
C
C---- stick bonds only---no circles - no obscuring...
C
C              ********************************************
  280     CALL zmoveto(Plotting_XYZ(1,I),Plotting_XYZ(2,I))
          CALL zdrawto(Plotting_XYZ(1,J),Plotting_XYZ(2,J))
C              ********************************************
C
  290   CONTINUE
  300 CONTINUE
C
C---- eliminate local overlap information before returning
C
      Num_Overlaps = 0
      NQuads_Overlaped = 0
C
C
C---- reset to initial colour
C
C          ******************************
  310 CALL zsetpencolour(Lcurrent_Colour)
C          ******************************
C
      RETURN
      END
C
C
C
C     =======================================
      SUBROUTINE drawvisiblelines(Y,NPEN,NCQ)
C     =======================================
C
C---- subroutine eliminates hidden lines and draws visible lines
C
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Conics
      PARAMETER (Max_Conics=20)
      INTEGER Max_Quadrangles
      PARAMETER (Max_Quadrangles=30)
      INTEGER Max_Segments
      PARAMETER (Max_Segments=50)
C     ..
C     .. Scalar Arguments ..
      INTEGER NCQ,NPEN
C     ..
C     .. Array Arguments ..
      REAL Y(3)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,CurrentCharcter_Size,
     +     D001,D010,D100,Dist_to_Boundary,Ellipse_ScaleFact,
     +     Eye_Distance,
     +     RFR(4,4),ROR(4,4),Overlap_Margin,Paper_Length,
     +     Paper_Width,Plotter_Step,Retrace_Displmnt,Scal_pre_Proj,SCL
      INTEGER Iuse_persp_Labs,Lorthogonalisation_Code,Lpack_Flag,
     +        NDrawing_Number,NQuads_Overlaped,Num_Bnd_Quadrangles,
     +        Num_Conics,Num_Overlaps
      LOGICAL UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL AID,Base_Vect_Refsys,Base_Vect_Wrksys,BBB,Drawing_Origin,
     +     Ellipse_Conics,Ellipse_Resolution,G_Metric_Tensor,OrtCell,
     +     OrtConts,Overlap_Conics,P1_to_Refsys,P1_to_Wrksys,
     +     Principal_axis_rms,Prin_Angstrom_Vectors,Prin_Axis_Col_Vects,
     +     Q,Quadrangles,Quads_Overlaped,Recip_Metric_Tensor,Segments,
     +     Sorted_xyz,Title_Rotation,Title_RotnTrans,XABC,XCLANG,XO
      INTEGER KC,KQ,KSortPointer
C     ..
C     .. Local Scalars ..
      REAL P0,P1,T1,T2,T3,T4,T5,T6
      INTEGER I,I12,IM,IQ,J,J1,K,KCQ,L,M,NINTCOUNT,NPM3,NPN,NPO
C     ..
C     .. Local Arrays ..
      REAL CB(20),CQ(50,2),QL(4,30,2),SEG(2),YN(3),YO(3),Z(3)
C     ..
C     .. External Subroutines ..
      EXTERNAL zdrawto,zmoveto
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MAX,MOD,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_OVERLAP/Num_Conics,Num_Overlaps,NQuads_Overlaped,
     +       Num_Bnd_Quadrangles,KC(Max_Conics),KQ(Max_Quadrangles),
     +       Ellipse_Conics(7,Max_Atoms),Overlap_Conics(6,Max_Conics),
     +       Quads_Overlaped(3,4,Max_Quadrangles),
     +       Quadrangles(9,2*Max_Atoms),Segments(Max_Segments,2)
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      NCQ = Num_Overlaps + NQuads_Overlaped
      IF (NCQ.LE.0) RETURN
C
C---- check all overlapping atoms and bonds
C
      NPM3 = NPEN - 3
      IF (NPM3.GE.0) GO TO 40
C
C---- save information from last point if pen is down
C
      YO(1) = YN(1)
      YO(2) = YN(2)
      YO(3) = 1.0
      NPO = NPN
C
C
      DO 10 K = 1,NCQ
        CQ(K,1) = CQ(K,2)
   10 CONTINUE
C
C
      IF (NQuads_Overlaped.GT.0) THEN
        DO 30 K = 1,NQuads_Overlaped
          DO 20 J = 1,4
            QL(J,K,1) = QL(J,K,2)
   20     CONTINUE
   30   CONTINUE
      END IF
C
C---- evaluate conic quadratic forms at new point yn
C
   40 YN(1) = Y(1)
      YN(2) = Y(2)
      YN(3) = 1.0
      NPN = NPEN
C
C
      IF (Num_Overlaps.GT.0) THEN
        DO 50 K = 1,Num_Overlaps
          Z(1) = YN(1)*Overlap_Conics(1,K) + YN(2)*Overlap_Conics(2,K) +
     +           Overlap_Conics(3,K)
          Z(2) = YN(1)*Overlap_Conics(2,K) + YN(2)*Overlap_Conics(4,K) +
     +           Overlap_Conics(5,K)
          Z(3) = YN(1)*Overlap_Conics(3,K) + YN(2)*Overlap_Conics(5,K) +
     +           Overlap_Conics(6,K)
          CQ(K,2) = Z(1)*YN(1) + Z(2)*YN(2) + Z(3)
C
C---- evaluate conic bilinear form if pen is down
C
          IF (NPM3.LT.0) CB(K) = Z(1)*YO(1) + Z(2)*YO(2) + Z(3)
   50   CONTINUE
      END IF
C
C---- evaluate linear forms and signature for quadrangle
C
      IF (NQuads_Overlaped.GT.0) THEN
        KCQ = Num_Overlaps
C
C
        DO 80 K = 1,NQuads_Overlaped
          T2 = 3.0
          DO 70 J = 1,4
            T1 = YN(1)*Quads_Overlaped(1,J,K) +
     +           YN(2)*Quads_Overlaped(2,J,K) + Quads_Overlaped(3,J,K)
            IF (T1.GE.0.0) GO TO 60
            T2 = T2 - 1.0
   60       QL(J,K,2) = T1
   70     CONTINUE
C
C
          KCQ = KCQ + 1
C
C---- t2=-1 inside, =0 across any edge, =1 across any vertex
C
          CQ(KCQ,2) = T2
   80   CONTINUE
      END IF
C
C---- if pen is up, omit all subsequent checking
C
      IF (NPM3.GE.0) THEN
        NPN = 3
C
C            ********************
        CALL zmoveto(YN(1),YN(2))
C            ********************
C
        RETURN
      END IF
C
C---- check for hidden segment
C
      DO 90 K = 1,NCQ
        IF (CQ(K,1).GE.0.0) GO TO 90
        IF (CQ(K,2).LT.0.0) THEN
C
C              ********************
          CALL zmoveto(YN(1),YN(2))
C              ********************
C
          RETURN
        END IF
   90 CONTINUE
C
C---- find entry and exit points on each conic
C
      NINTCOUNT = 0
C
C
      IF (Num_Overlaps.GT.0) THEN
        DO 100 K = 1,Num_Overlaps
C
C---- evaluate discriminant
C
          T1 = CB(K)**2 - CQ(K,1)*CQ(K,2)
          IF (T1.LE.0.0) GO TO 100
          T1 = SQRT(T1)
C
C---- solve quadratic eqation
C
          T2 = CQ(K,1) - CB(K)
          T3 = CQ(K,2) + T2 - CB(K)
          IF ((ABS(T3)-1.0E-5).LE.0.0) GO TO 100
          T4 = (T2-T1)/T3
          T5 = (T2+T1)/T3
C
C---- valid intersection if t4.lt.1 and t5.gt.0
C
          IF ((T4-1.0).GE.0.0) GO TO 100
          IF (T5.LE.0.0) GO TO 100
C
C---- save valid conic intersections
C
          NINTCOUNT = NINTCOUNT + 1
          Segments(NINTCOUNT,1) = T4
          Segments(NINTCOUNT,2) = T5
  100   CONTINUE
      END IF
C
C
      IF (NQuads_Overlaped.LE.0) GO TO 200
C
C---- find entry and exit points for each quadrangle
C
      DO 190 K = 1,NQuads_Overlaped
        I12 = 0
        KCQ = Num_Overlaps + K
C
C---- check for single inside point
C
        SEG(1) = CQ(KCQ,1)
        IF (SEG(1).LT.0.0) GO TO 110
        SEG(1) = 1.0 - CQ(KCQ,2)
        IF ((SEG(1)-1.0).LE.0.0) GO TO 120
C
C---- inside point found, only one intersection possible
C
  110   I12 = 1
C
C---- find which edges are crossed by the segment
C
  120   DO 180 J = 1,4
          T1 = QL(J,K,1)
          T2 = QL(J,K,2)
          T3 = T1 - T2
          IF (T1*T2.GT.0.0) GO TO 180
C
C---- check for segment on an edge
C
          IF ((ABS(T3)-1.0E-5).LE.0.0) GO TO 190
C
C---- calculate coordinates of intersection
C
          T4 = (YN(1)*T1-YO(1)*T2)/T3
          T5 = (YN(2)*T1-YO(2)*T2)/T3
          J1 = (MOD(J,4)+1)*2
          IQ = KQ(K)
C
C---- is intersection within limits of quadrangle
C
          T6 = (T4-Quadrangles(2*J-1,IQ))* (Quadrangles(J1-1,IQ)-T4) +
     +         (T5-Quadrangles(2*J,IQ))* (Quadrangles(J1,IQ)-T5)
          IF ((ABS(T6)-1.0E-4).LE.0.0) GO TO 130
          IF (T6.LT.0.0) GO TO 180
C
C---- calculate fraction parameter and store it
C
  130     T1 = T1/T3
          IF ((I12-1).EQ.0) THEN
            GO TO 140
          ELSE IF ((I12-1).GT.0) THEN
            GO TO 160
          END IF
C
C---- store first intersection
C
          I12 = 1
          GO TO 150
C
C---- store second intersection
C
  140     I12 = 2
          IF ((T1-SEG(1)).GE.0.0) GO TO 170
          SEG(2) = SEG(1)
  150     SEG(1) = T1
          GO TO 180
C
C---- more than two intersections (i.e.,quadrangle diagonal)
C
  160     IF ((T1-SEG(1)).LT.0.0) THEN
            GO TO 150
          ELSE IF ((T1-SEG(1)).EQ.0.0) THEN
            GO TO 180
          END IF
C
C
          IF ((T1-SEG(2)).LE.0.0) GO TO 180
  170     SEG(2) = T1
  180   CONTINUE
        IF ((I12-1).LE.0) GO TO 190
C
C---- store fraction parameters
C
        NINTCOUNT = NINTCOUNT + 1
        Segments(NINTCOUNT,1) = SEG(1)
        Segments(NINTCOUNT,2) = SEG(2)
  190 CONTINUE
C
C---- end of entry-and-exit-point calculations
C
  200 IF ((NINTCOUNT-1).EQ.0) THEN
        GO TO 290
      ELSE IF ((NINTCOUNT-1).GT.0) THEN
        GO TO 210
      END IF
C
C---- no interference found, draw entire segment
C
C          ********************
      CALL zdrawto(YN(1),YN(2))
C          ********************
C
      RETURN
C
C---- sort segment intersection list
C     sorting procedure by shell,d.l. comm. acm 2,30-32 (1959)
C
  210 M = NINTCOUNT
  220 M = M/2
      IF (M.LE.0) GO TO 290
      K = NINTCOUNT - M
      J = 1
  230 I = J
  240 IM = I + M
      IF (Segments(I,1).GE.0.0) GO TO 250
      IF (Segments(IM,1).GT.0.0) GO TO 280
C
C
      IF ((Segments(I,2)-Segments(IM,2)).LE.0.0) THEN
        GO TO 280
      ELSE
        GO TO 260
      END IF
  250 IF ((Segments(I,1)-Segments(IM,1)).LE.0.0) GO TO 280
  260 DO 270 L = 1,2
        T1 = Segments(I,L)
        Segments(I,L) = Segments(IM,L)
        Segments(IM,L) = T1
  270 CONTINUE
C
C
      I = I - M
      IF (I.GT.0) GO TO 240
  280 J = J + 1
      IF ((J-K).LE.0) THEN
        GO TO 230
      ELSE
        GO TO 220
      END IF
C
C---- find starting point p0 and end point p1
C
  290 P0 = 0.0
      K = 0
  300 K = K + 1
      IF ((K-NINTCOUNT).GT.0) GO TO 320
      P1 = Segments(K,1)
      IF (P1.LT.0.0) GO TO 310
      IF ((P1-P0).GT.0.0) GO TO 330
  310 P0 = MAX(P0,Segments(K,2))
C
C
      IF ((P0-1.0).LT.0.0) THEN
        GO TO 300
      ELSE IF ((P0-1.0).EQ.0.0) THEN
        GO TO 350
      ELSE
        GO TO 340
      END IF
  320 P1 = 1.0
C
C---- draw segment from p0 to p1
C
  330 IF (P0.LE.0.0) THEN
        GO TO 360
      ELSE
        GO TO 350
      END IF
C
C
  340 P0 = 1.0
  350 Z(1) = (1.0-P0)*YO(1) + YN(1)*P0
      Z(2) = (1.0-P0)*YO(2) + YN(2)*P0
      NPN = 3
C
C          ******************
      CALL zmoveto(Z(1),Z(2))
C          ******************
C
      IF ((P0-1.0).GE.0.0) GO TO 370
  360 Z(1) = (1.0-P1)*YO(1) + YN(1)*P1
      Z(2) = (1.0-P1)*YO(2) + YN(2)*P1
      NPN = 2
C
C          ******************
      CALL zdrawto(Z(1),Z(2))
C          ******************
C
      IF ((P1-1.0).LT.0.0) GO TO 310
  370 RETURN
      END
C
C
C
C     ================================================
      SUBROUTINE dscan(MODE,IUNIQ,INTER,FVALUEX,KWORK)
C     ================================================
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Labels
      PARAMETER (Max_Labels=100)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER Max_Symmetry
      PARAMETER (Max_Symmetry=192)
      INTEGER Max_Tokens
      PARAMETER (Max_Tokens=200)
      INTEGER NPackAtmID
      PARAMETER (NPackAtmID=10000000)
C     ..
C     .. Scalar Arguments ..
      INTEGER INTER,IUNIQ,KWORK,MODE
C     ..
C     .. Array Arguments ..
      REAL FVALUEX(Max_Tokens)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Charcter_Size,
     +     CurrentCharcter_Size,D001,D010,D100,Dist_to_Boundary,
     +     Drawing_Size,Ellipse_ScaleFact,Eye_Distance,
     +     RFR(4,4),ROR(4,4),
     +     Overlap_Margin,Paper_Length,Paper_Width,Perspective_Distance,
     +     Picture_Scale,Plotter_Step,Retrace_Displmnt,S,Scal_pre_Proj,
     +     SCL,Shadelight_X,Shadelight_Y,Stereo_Separation
      INTEGER IERR,Iuse_persp_Labs,Label_atoms_Flag,Lcurrent_Colour,
     +        Ldraw_cell_Flag,Ljoin_Flag,Lorthogonalisation_Code,
     +        Lpack_Flag,Lpaper_Setting,Lperspective_Flag,
     +        Lplot_colour_Stereo,Lposition_Flag,Lprint_Flag,
     +        LShade_Flag,LSolid_Flag,LStereo_Flag,LSuperimpose_Flag,
     +        LTaper_Flag,LTrace_Flag,NAT,NATO,Natoms_plusCell,NC,NC1,
     +        Ncopies,NDrawing_Number,Nspecified_Bonds,NSymmetry_Ops,
     +        Num_Moles
      LOGICAL Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
      CHARACTER Iden_Title*80
C     ..
C     .. Arrays in Common ..
      REAL AID,Atom_Radii,Drawing_Origin,Ellipse_Resolution,
     +     Fractional_XYZ,OrtCell,OrtConts,Orthogonal_XYZ,Plotting_XYZ,
     +     Rotsym_Matrx,RP,Title_Rotation,TransSym_Vect,ViewMatrix,XABC,
     +     XCLANG,XO
      INTEGER IAtom_Connectivity,IAtom_Type,ISpecified_Bond_Params,
     +        LAtom_Inc_Flag,LAtom_Label_Flag,MOL
      CHARACTER Atm_Lab_Type*2,Atm_Identifier*10,Atm_Label*10,
     +          Atm_Lab_Txt*10
C     ..
C     .. Local Scalars ..
      REAL ACELL,ALF,ANGLE,AXHI,AXLO,BCELL,BET,C,C1,C2,CCELL,COSANG,
     +     DANGL,DIJ,DIJSQ,DMAX,DMAXSQ,DSQ,DX,DXO,DY,DYO,DZ,DZO,GAM,XF,
     +     XJST,XX,YF,YJST,YY,ZF,ZJST,ZZ
      INTEGER I,IL,IT,ITX,ITY,ITZ,IXX,J,J1,J2,JB1,JB2,JCOLR,JXX,K,KAT,
     +        KX,KY,KZ,MM,MN,N,N1,N2,NATC,NB,NBONDS,NFOUND,NN,NSG,NXX
      CHARACTER LMOL*1
C     ..
C     .. Local Arrays ..
      REAL AXIS(3),CENT(3),CL(Max_Atoms),CM(Max_Atoms),CMAX(3),CMIN(3),
     +     CN(Max_Atoms),D(Max_Atoms),DLIM(3),SEP(3),T(3),XI(3),XJS(3)
      INTEGER IDONE(Max_Atoms),IFIX(Max_Atoms),INEXT(Max_Atoms),
     +        JBOND(Max_Atoms),KEY(3)
      CHARACTER IALPH(26)*1
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,ACOS,MAX,MIN,NINT,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_LABCHR/Atm_Lab_Txt(Max_Labels),
     +       Atm_Lab_Type(Max_Labels),Atm_Label(Max_Labels),
     +       Atm_Identifier(Max_Atoms),Iden_Title
      COMMON /com_SYMMETRY/NSymmetry_Ops,Rotsym_Matrx(3,3,Max_Symmetry),
     +       TransSym_Vect(3,Max_Symmetry)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Data statements ..
      DATA IALPH/'A','B','C','D','E','F','G','H','I','J','K','L','M',
     +     'N','O','P','Q','R','S','T','U','V','W','X','Y','Z'/
C     ..
C
C
      ACELL = XABC(1)
      BCELL = XABC(2)
      CCELL = XABC(3)
      ALF = XCLANG(1)
      BET = XCLANG(2)
      GAM = XCLANG(3)
C
C
      IF (MODE.NE.1) GO TO 100
C
C---- MODE 1 - Used by pack range instruction
C              find molecules having their centre
C              wthin the range given
C
C Calculate the centre of the coordinate set -
C the definition of centre
C here is (xmin+xmax)/2.0        ( not centre of gravity )
C
      CMIN(1) = FVALUEX(KWORK)
      CMAX(1) = FVALUEX(KWORK+1)
      CMIN(2) = FVALUEX(KWORK+2)
      CMAX(2) = FVALUEX(KWORK+3)
      CMIN(3) = FVALUEX(KWORK+4)
      CMAX(3) = FVALUEX(KWORK+5)
      Num_Moles = 0
C
C
      DO 20 K = 1,3
        C1 = 1.0E10
        C2 = -C1
C
C
        DO 10 I = 9,NATO
          C = Fractional_XYZ(I,K)
          C1 = MIN(C1,C)
          C2 = MAX(C2,C)
   10   CONTINUE
        CENT(K) = (C1+C2)/2.0
   20 CONTINUE
C
C
      WRITE (6,FMT=6000) CENT
 6000 FORMAT (21X,'Molecular Centre is at ',3F8.4)
C
C---- scan through the symm opr list and all possible
C     translations tx,ty,tz to test for molecular centres
C     within the coordinate range given.
C
      DO 90 N = 1,NSymmetry_Ops
        XX = CENT(1)
        YY = CENT(2)
        ZZ = CENT(3)
C
C
        DO 30 K = 1,3
          XI(K) = Rotsym_Matrx(K,1,N)*XX + Rotsym_Matrx(K,2,N)*YY +
     +            Rotsym_Matrx(K,3,N)*ZZ + TransSym_Vect(K,N)
   30   CONTINUE
C
C
        T(1) = 0.0
        T(2) = 0.0
        T(3) = 0.0
        K = 1
   40   IF (XI(K).LT.CMIN(K)) GO TO 50
        XI(K) = XI(K) - 1.0
        T(K) = T(K) - 1.0
        GO TO 40
   50   XI(K) = XI(K) + 1.0
        T(K) = T(K) + 1.0
        IF (XI(K).GT.CMAX(K)) GO TO 80
        K = K + 1
        IF (K.LE.3) GO TO 40
C
C---- the current position of the centre ( xi )
C     is found to be within range
C     test for overlap with a centre already in the list
C
        IF (Num_Moles.LE.0) GO TO 70
C
C
        DO 60 NN = 1,Num_Moles
          IF (ABS(XI(1)-CL(NN)).GT.0.001) GO TO 60
          IF (ABS(XI(2)-CM(NN)).GT.0.001) GO TO 60
          IF (ABS(XI(3)-CN(NN)).GT.0.001) GO TO 60
C
C---- overlaps with centre nn - do not duplicate in mol list
C
          K = K - 1
          GO TO 50
   60   CONTINUE
C
C---- no overlap - enter in the list of molecules to be plotted.
C
   70   WRITE (6,FMT=6002) N,T,XI
 6002   FORMAT (' Molecule Centre in Range ',I3,3F3.0,' Coords',3F8.4)
        Num_Moles = Num_Moles + 1
        MOL(Num_Moles,1) = N
        MOL(Num_Moles,2) = NINT(T(1))
        MOL(Num_Moles,3) = NINT(T(2))
        MOL(Num_Moles,4) = NINT(T(3))
        CL(Num_Moles) = XI(1)
        CM(Num_Moles) = XI(2)
        CN(Num_Moles) = XI(3)
        K = K - 1
        GO TO 50
   80   K = K - 1
        IF (K.GT.0) GO TO 50
   90 CONTINUE
      GO TO 590
C
C---- mode 2 - used by the join instruction
C
C    django **********************distances and angles**
C
C
  100 MOL(1,1) = 1
      MOL(1,2) = 0
      MOL(1,3) = 0
      MOL(1,4) = 0
      Num_Moles = 1
      NAT = NATO
      IF (IUNIQ.EQ.1) INTER = 1
      IF (IUNIQ.EQ.1 .AND. Lprint_Flag.EQ.1) WRITE (6,FMT=6004)
 6004 FORMAT (' Unique molecular coordinates to be assembled',/' - the',
     +       ' order of connections works outward from 1st atom')
C
C---- decide order of axes in descending axial length.
C
      AXIS(1) = ACELL
      AXIS(2) = BCELL
      AXIS(3) = CCELL
      IF (ACELL.EQ.BCELL .OR. ACELL.EQ.CCELL) AXIS(1) = ACELL + 0.0002
      IF (BCELL.EQ.CCELL) AXIS(2) = BCELL + 0.0001
      AXHI = MAX(AXIS(1),AXIS(2),AXIS(3))
      AXLO = MIN(AXIS(1),AXIS(2),AXIS(3))
C
C
      DO 130 K = 1,3
        IF (AXIS(K).EQ.AXHI) GO TO 110
        IF (AXIS(K).EQ.AXLO) GO TO 120
        KEY(K) = 2
        GO TO 130
  110   KEY(K) = 1
        GO TO 130
  120   KEY(K) = 3
  130 CONTINUE
C
C
      KX = KEY(1)
      KY = KEY(2)
      KZ = KEY(3)
C
C
      DO 140 I = 9,NAT
        IDONE(I) = 0
        IFIX(I) = 0
        INEXT(I) = 0
  140 CONTINUE
C
C
      NSG = 0
      I = 9
      IFIX(I) = 1
C
C---- start loop on atom i
C
  150 XI(KX) = Fractional_XYZ(I,1)
      XI(KY) = Fractional_XYZ(I,2)
      XI(KZ) = Fractional_XYZ(I,3)
      NFOUND = 0
      NB = 1
C
C---- start loop on atoms j
C
      DO 350 J = 9,NAT
        DMAX = Atom_Radii(I) + Atom_Radii(J)
        DMAXSQ = DMAX*DMAX
        DANGL = DMAX
        IF (INTER.EQ.1) DANGL = 0
        IF (IUNIQ.EQ.1) DANGL = DMAX
C
C---- set up the d limits for each axis
C
        DLIM(KX) = DMAX/D100
        DLIM(KY) = DMAX/D010
        DLIM(KZ) = DMAX/D001
C
C---- start loop on symmetry operators
C
        DO 340 N = 1,NSymmetry_Ops
          IF (INTER.EQ.0 .AND. N.GT.1) GO TO 350
          XX = Fractional_XYZ(J,1)
          YY = Fractional_XYZ(J,2)
          ZZ = Fractional_XYZ(J,3)
          XF = Rotsym_Matrx(1,1,N)*XX + Rotsym_Matrx(1,2,N)*YY +
     +         Rotsym_Matrx(1,3,N)*ZZ + TransSym_Vect(1,N)
          YF = Rotsym_Matrx(2,1,N)*XX + Rotsym_Matrx(2,2,N)*YY +
     +         Rotsym_Matrx(2,3,N)*ZZ + TransSym_Vect(2,N)
          ZF = Rotsym_Matrx(3,1,N)*XX + Rotsym_Matrx(3,2,N)*YY +
     +         Rotsym_Matrx(3,3,N)*ZZ + TransSym_Vect(3,N)
          XJS(KX) = XF
          XJS(KY) = YF
          XJS(KZ) = ZF
C
C----  begin translational search routine.
C
          T(1) = 0.0
          T(2) = 0.0
          T(3) = 0.0
          K = 1
          IF (INTER.EQ.1) GO TO 170
C
C---- no calculation of intermolecular distances.
C
          IF (I.EQ.J) GO TO 340
C
C
          DO 160 K = 1,3
            SEP(K) = XI(K) - XJS(K)
            IF (ABS(SEP(K)).GT.DLIM(K)) GO TO 340
  160     CONTINUE
C
C
          GO TO 210
C
C---- translate xjs on axis k to a point below lower limit.
C
  170     IF (XI(K)-XJS(K).GT.DLIM(K)) GO TO 180
          XJS(K) = XJS(K) - 1.0
          T(K) = T(K) - 1.0
          GO TO 170
C
C---- advance +1.0 on axis k.
C
  180     XJS(K) = XJS(K) + 1.0
          T(K) = T(K) + 1.0
          SEP(K) = XJS(K) - XI(K)
          IF (SEP(K).LT.-DLIM(K)) GO TO 180
C
C---- test if xjs-xi less than or equal to dlim(k)
C
          IF (SEP(K).GT.DLIM(K)) GO TO 190
          GO TO 200
C
C---- test gives atom out of range. reduce axis k by 1.
C
  190     K = K - 1
          IF (K.LE.0) THEN
            GO TO 330
          ELSE
            GO TO 180
          END IF
C
C---- k=0 signals the end of the search and we leave through 50.
C     if xjs was within range on axis k then we advance k by 1
C
  200     K = K + 1
          IF (K.LE.3) GO TO 170
C
C---- atom in range for all 3 axes. now calculate exact distance.
C
          IT = NINT(ABS(T(KX))+ABS(T(KY))+ABS(T(KZ)))
C
C---- eliminate distance of atom from itself.
C
          IF (N.EQ.1 .AND. IT.EQ.0 .AND. I.EQ.J) GO TO 220
          IF (IUNIQ.EQ.1) GO TO 210
          IF (INTER.EQ.0) GO TO 210
C
C---- eliminate distances intramolecular
C     ( inter.eq.1 means intermol only)
C
          IF (N.EQ.1 .AND. IT.EQ.0) GO TO 220
c  210     DXO = SEP(KX)*OR1 + SEP(KY)*OR2 + SEP(KZ)*OR3
c          DYO = SEP(KX)*OR4 + SEP(KY)*OR5 + SEP(KZ)*OR6
c          DZO = SEP(KX)*OR7 + SEP(KY)*OR8 + SEP(KZ)*OR9
  210     DXO = SEP(KX)*ROR(1,1) + SEP(KY)*ROR(1,2) + SEP(KZ)*ROR(1,3)
          DYO = SEP(KX)*ROR(2,1) + SEP(KY)*ROR(2,2) + SEP(KZ)*ROR(2,3)
          DZO = SEP(KX)*ROR(3,1) + SEP(KY)*ROR(3,2) + SEP(KZ)*ROR(3,3)
          DIJSQ = DXO*DXO + DYO*DYO + DZO*DZO
          IF (DIJSQ.LE.DMAXSQ) GO TO 230
  220     K = K - 1
          IF (INTER.EQ.0) GO TO 340
          GO TO 180
C
C---- we have a distance less tha or equal to dmax.
C     print and store.
C
  230     DIJ = SQRT(DIJSQ)
          XJST = XJS(KX)
          YJST = XJS(KY)
          ZJST = XJS(KZ)
          IF (Lprint_Flag.EQ.0) GO TO 240
          WRITE (6,FMT=6006) Atm_Identifier(I),Atm_Identifier(J),DIJ,N,
     +      T(KX),T(KY),T(KZ)
  240     NFOUND = NFOUND + 1
          IF (DIJ.GT.DANGL) GO TO 260
          IF (DIJ.GT.0.05) GO TO 250
          IF (I.NE.J) WRITE (6,FMT=6008) Atm_Identifier(I),
     +        Atm_Identifier(J)
          IF (I.EQ.J) WRITE (6,FMT=6010) Atm_Identifier(I)
 6006     FORMAT (5X,A,2X,A,F10.5,10X,I5,3F3.0,10X,2 (2X,3F8.5))
 6008     FORMAT (' Note:',6X,A,2X,A,' are equivalent by space group s',
     +           'ymmetry')
 6010     FORMAT (' Note:',6X,A,' Lies on a special position')
          IF (I.EQ.J) GO TO 220
          Fractional_XYZ(J,1) = Fractional_XYZ(I,1)
          Fractional_XYZ(J,2) = Fractional_XYZ(I,2)
          Fractional_XYZ(J,3) = Fractional_XYZ(I,3)
          IFIX(J) = 1
          IDONE(J) = 1
          GO TO 220
C
C---- save direction cosines for angle calculation
C
  250     CL(NB) = DXO/DIJ
          CM(NB) = DYO/DIJ
          CN(NB) = DZO/DIJ
          D(NB) = DIJ
          JBOND(NB) = J
          NB = NB + 1
C
C---- decide what to do with this connection.
C
  260     KAT = J
          IF (INTER.EQ.0) GO TO 320
          IF (IUNIQ.EQ.0) GO TO 280
C
C---- unique mode only............
C
          IF (IDONE(J).EQ.0) INEXT(J) = 1
          IF (IFIX(J).EQ.1) GO TO 270
C
C---- move atom j
C
          Fractional_XYZ(J,1) = XJST
          Fractional_XYZ(J,2) = YJST
          Fractional_XYZ(J,3) = ZJST
          IFIX(J) = 1
          GO TO 310
C
C---- test for symmetry generated atom j'
C
  270     IF (ABS(Fractional_XYZ(J,1)-XJST).GT.0.0001) GO TO 280
          IF (ABS(Fractional_XYZ(J,2)-YJST).GT.0.0001) GO TO 280
          IF (ABS(Fractional_XYZ(J,3)-ZJST).GT.0.0001) GO TO 280
          GO TO 310
C
C---- this is an atom connected by symmetry to the basic fragment
C     the symmetry operation is noted and saved
C     for generating the unique
C     set of molecular coordinates later.
C     all inter-molecular connections are set up here also.
C
  280     IF (Ljoin_Flag.EQ.0) GO TO 290
          NSG = NSG + 1
          KAT = NATO + NSG
C
C---- add this atom to the list -
C     it has the suffix * added to the name
C
          Fractional_XYZ(KAT,1) = XJST
          Fractional_XYZ(KAT,2) = YJST
          Fractional_XYZ(KAT,3) = ZJST
          Atm_Identifier(KAT) (1:5) = Atm_Identifier(J) (1:5)
          Atm_Identifier(KAT) (6:6) = '*'
          LAtom_Inc_Flag(KAT) = 0
          IAtom_Type(KAT) = IAtom_Type(J)
          LAtom_Label_Flag(KAT) = LAtom_Label_Flag(J)
          Atom_Radii(KAT) = Atom_Radii(J)

C
C
          IF (INTER.EQ.1 .AND. IUNIQ.EQ.0) WRITE (6,
     +        FMT=6012) Atm_Identifier(I),Atm_Identifier(KAT),DIJ,N,
     +        T(KX),T(KY),T(KZ)
 6012     FORMAT (' Intermolecular Bond ',A,1X,A,F8.3,I4,3F3.0)
          IF (IUNIQ.EQ.1) WRITE (6,FMT=6014) Atm_Identifier(KAT)
 6014     FORMAT (' Symmetry generated atom added to list ',A)
C
C---- add molecule to list...
C
  290     ITX = NINT(T(KX))
          ITY = NINT(T(KY))
          ITZ = NINT(T(KZ))
C
C
          DO 300 MM = 1,Num_Moles
            IF (MOL(MM,1).EQ.N .AND. MOL(MM,2).EQ.ITX .AND.
     +          MOL(MM,3).EQ.ITY .AND. MOL(MM,4).EQ.ITZ) GO TO 320
  300     CONTINUE
C
C
          Num_Moles = Num_Moles + 1
          MOL(Num_Moles,1) = N
          MOL(Num_Moles,2) = ITX
          MOL(Num_Moles,3) = ITY
          MOL(Num_Moles,4) = ITZ
  310     CONTINUE
C
C---- add connection to list...
C
  320     IF (Ljoin_Flag.EQ.0) GO TO 220
          IF (I.GT.KAT) GO TO 220
          NC = NC + 1
          IAtom_Connectivity(NC) = Max_Atoms*I + KAT
          GO TO 220
C
C---- end of translational search.....
C
  330     CONTINUE
C
C----  end loop on symmetry oprs.
C
  340   CONTINUE
C
C---- end of loop on atoms j
C
  350 CONTINUE
C
C---- do angle calculation for atom i
C
      IF (Lprint_Flag.EQ.0) GO TO 380
      NBONDS = NB - 1
      IF (NFOUND.EQ.0) WRITE (6,FMT=6016) Atm_Identifier(I)
 6016 FORMAT (5X,A,' ** NO Connections **')
      IF (NBONDS.LE.1) GO TO 380
      NN = NBONDS - 1
C
C
      DO 370 J1 = 1,NN
        MM = J1 + 1
        DO 360 J2 = MM,NBONDS
          COSANG = CL(J1)*CL(J2) + CM(J1)*CM(J2) + CN(J1)*CN(J2)
          IF (COSANG.GT.1.000000) COSANG = 1.000000
          IF (COSANG.LT.-1.00000) COSANG = -1.000000
          ANGLE = ACOS(COSANG)*57.29578
          JB1 = JBOND(J1)
          JB2 = JBOND(J2)
          WRITE (6,FMT=6018) Atm_Identifier(JB1),Atm_Identifier(I),
     +      Atm_Identifier(JB2),ANGLE
 6018     FORMAT (3 (4X,A),F10.2,F6.2)
  360   CONTINUE
  370 CONTINUE
C
C----  now decide what to do about the next atom i
C
  380 IF (IUNIQ.EQ.0) GO TO 430
      IDONE(I) = 1
      INEXT(I) = 0
C
C
      DO 390 IL = 9,NAT
        IF (INEXT(IL).EQ.1) GO TO 410
  390 CONTINUE
C
C----  no connected atom is left to go to therefore we begin a new
C      residue
C      ............... unless all atoms have been done
C
      DO 400 IL = 9,NAT
        IF (IDONE(IL).EQ.0) GO TO 420
  400 CONTINUE
C
C
      GO TO 440
C
C---- set the next atom i for calculation
C
  410 I = IL
      GO TO 150
C
C---- set i for the first atom of a new residue
C
  420 WRITE (6,FMT=6020)
 6020 FORMAT (///' *** Begin new residue ***')
      I = IL
      IFIX(I) = 1
      GO TO 150
C
C---- advance i to i+1 if not unique mode
C
  430 I = I + 1
      IF (I.GT.NAT) GO TO 440
      GO TO 150
C
C---- end of loop on i
C
  440 IF (IUNIQ.EQ.1) GO TO 450
      IF (INTER.EQ.1) NAT = NATO + NSG
      GO TO 550
C
C---- now expand the coordinate list to give
C     the unique molecular residue(s)
C
  450 IF (NSG.EQ.0) GO TO 550
      NAT = NATO
      IF (Num_Moles.LT.2) GO TO 510
C
C
      DO 500 MM = 2,Num_Moles
        N = MOL(MM,1)
        ITX = MOL(MM,2)
        ITY = MOL(MM,3)
        ITZ = MOL(MM,4)
        MN = MM - 1
        IF (MN.LE.26) LMOL = IALPH(MN)
        IF (MN.GT.26) LMOL = '*'
        IF (N.EQ.1 .AND. ITX.EQ.0 .AND. ITY.EQ.0 .AND.
     +      ITZ.EQ.0) GO TO 500
        NATC = NAT
C
C---- run through the current atom list generating new atoms by the
C     symm operation defined for the mol(mm,k)
C
        DO 490 I = 9,NATC
          XX = Fractional_XYZ(I,1)
          YY = Fractional_XYZ(I,2)
          ZZ = Fractional_XYZ(I,3)
C
C
          XF = Rotsym_Matrx(1,1,N)*XX + Rotsym_Matrx(1,2,N)*YY +
     +         Rotsym_Matrx(1,3,N)*ZZ + TransSym_Vect(1,N)
          YF = Rotsym_Matrx(2,1,N)*XX + Rotsym_Matrx(2,2,N)*YY +
     +         Rotsym_Matrx(2,3,N)*ZZ + TransSym_Vect(2,N)
          ZF = Rotsym_Matrx(3,1,N)*XX + Rotsym_Matrx(3,2,N)*YY +
     +         Rotsym_Matrx(3,3,N)*ZZ + TransSym_Vect(3,N)
C
C
          XF = XF + ITX
          YF = YF + ITY
          ZF = ZF + ITZ
C
C---- test for this symm generated atom overlapping
C     an atom already in the  list
C
          DO 460 J = 9,NATC
            IF (ABS(Fractional_XYZ(J,1)-XF).GT.0.0001) GO TO 460
            IF (ABS(Fractional_XYZ(J,2)-YF).GT.0.0001) GO TO 460
            IF (ABS(Fractional_XYZ(J,3)-ZF).GT.0.0001) GO TO 460
C
C---- this atom overlaps atom j in list - do not add to list
C
            GO TO 490
  460     CONTINUE
C
C---- the atom (xf,yf,zf) does not overlap any in list -
C       add to the list.
C
          IF (NAT.GE.Max_Atoms) GO TO 580
          NAT = NAT + 1
          Fractional_XYZ(NAT,1) = XF
          Fractional_XYZ(NAT,2) = YF
          Fractional_XYZ(NAT,3) = ZF
C
C---- copy the atom name label with an alphabetic character added
C     to identify the symm operation which generated it.
C
          Atm_Identifier(NAT) (1:5) = Atm_Identifier(I) (1:5)
C
C
          DO 470 K = 3,5
            IF (Atm_Identifier(NAT) (K:K).EQ.' ') GO TO 480
  470     CONTINUE
C
C
          K = 6
  480     Atm_Identifier(NAT) (K:K) = LMOL
C
C---- copy the other atom flags and radius
C
          Atom_Radii(NAT) = Atom_Radii(I)
          IAtom_Type(NAT) = IAtom_Type(I)
          LAtom_Label_Flag(NAT) = LAtom_Label_Flag(I)
          LAtom_Inc_Flag(NAT) = LAtom_Inc_Flag(I)
C
C---- end of loop on  atoms i
C
  490   CONTINUE
        NATC = NAT
        WRITE (6,FMT=6022) N,ITX,ITY,ITZ,NATC
 6022   FORMAT (' After Symm operation (',4I2,') Number of atoms = ',I4)
C
C---- end of loop on symm oprs mm
C
  500 CONTINUE
C
C---- this is now the unique molecule set of coordinates
C     - generate the connectivity on the basis of this list.
C     This connection list overwrites whatever was generated earlier
C
  510 NC = 12
      NATO = NAT
      N = NAT - 8
      WRITE (6,FMT=6024) N
 6024 FORMAT (/' The Unique molecule coordinate list contains ',I4,' A',
     +       'toms')
      WRITE (6,FMT=6026) NAT
 6026 FORMAT (' The Total number of atomic positions, including 8 cell',
     +       '  corner points =',I4)
C
C
      DO 520 I = 9,NATO
        WRITE (6,FMT=6028) Atm_Identifier(I),
     +    (Fractional_XYZ(I,K),K=1,3)
  520 CONTINUE
C
C
 6028 FORMAT (5X,A,4X,3F10.5)
C
C---- do an intra-mol distance calculation now
C
      N1 = NATO - 1
C
C
      DO 540 I = 9,N1
        N2 = I + 1
        DO 530 J = N2,NATO
          DMAX = Atom_Radii(I) + Atom_Radii(J)
          DX = Fractional_XYZ(I,1) - Fractional_XYZ(J,1)
          IF (ABS(DX).GT.DMAX/D100) GO TO 530
          DY = Fractional_XYZ(I,2) - Fractional_XYZ(J,2)
          IF (ABS(DY).GT.DMAX/D010) GO TO 530
          DZ = Fractional_XYZ(I,3) - Fractional_XYZ(J,3)
          IF (ABS(DZ).GT.DMAX/D001) GO TO 530
c          DXO = OR1*DX + OR2*DY + OR3*DZ
c          DYO = OR4*DX + OR5*DY + OR6*DZ
c          DZO = OR7*DX + OR8*DY + OR9*DZ
          DXO = ROR(1,1)*DX + ROR(1,2)*DY + ROR(1,3)*DZ
          DYO = ROR(2,1)*DX + ROR(2,2)*DY + ROR(2,3)*DZ
          DZO = ROR(3,1)*DX + ROR(3,2)*DY + ROR(3,3)*DZ
          DSQ = DXO*DXO + DYO*DYO + DZO*DZO
          IF (DSQ.GT.DMAX*DMAX) GO TO 530
          NC = NC + 1
          IAtom_Connectivity(NC) = Max_Atoms*I + J
  530   CONTINUE
  540 CONTINUE
C
C---- reset the number of moles in the list to the first one (x,y,z)
C
      Num_Moles = 1
      GO TO 550
C
C---- change LAtom_Inc_Flag array to jcolr+1 or 0,
C     as in s/r getatomconnections
C
  550 CONTINUE
C
C
      DO 570 I = 9,NAT
        JCOLR = 0
        IF (LAtom_Inc_Flag(I).EQ.0) GO TO 560
        IXX = ABS(LAtom_Inc_Flag(I))
        JXX = IXX/NPackAtmID
        IXX = IXX - JXX*NPackAtmID
        NXX = IXX/10000
        IXX = IXX - NXX*10000
        JCOLR = IXX/1000 + 1
        IF (LAtom_Inc_Flag(I).LT.0) JCOLR = -JCOLR
  560   LAtom_Inc_Flag(I) = JCOLR
  570 CONTINUE
C
C
      RETURN
C
C
  580 WRITE (6,FMT=6030) Max_Atoms
 6030 FORMAT (' *** Too many atoms in basic list',/'     - extra symme',
     +       'try generated atoms ',/'     - max atom total = ',I4)
  590 RETURN
      END
C
C
C
C     ====================================================
      SUBROUTINE generateellipse(NUMSUBDIVI,CIRCLESPOKES,
     +                           CONJUGATEVECTORS)
C     ===================================================
C
C---- generate ellipse from two conjugate vectors
C     orthonormal vectors produce 8-128 spoked circle
C     NumSubDivi denotes number of subdivisions (1 to 5)
C     radial
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
C     ..
C     .. Scalar Arguments ..
      INTEGER NUMSUBDIVI
C     ..
C     .. Array Arguments ..
      REAL CIRCLESPOKES(3,130),CONJUGATEVECTORS(3,3)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,CurrentCharcter_Size,
     +     D001,D010,D100,Dist_to_Boundary,Ellipse_ScaleFact,
     +     Eye_Distance,RFR(4,4),ROR(4,4),
     +     Overlap_Margin,Paper_Length,
     +     Paper_Width,Plotter_Step,Retrace_Displmnt,Scal_pre_Proj,SCL
      INTEGER Iuse_persp_Labs,Lorthogonalisation_Code,Lpack_Flag,
     +        NDrawing_Number
      LOGICAL UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL AID,Drawing_Origin,Ellipse_Resolution,OrtCell,OrtConts,
     +     Title_Rotation,XABC,XCLANG,XO
C     ..
C     .. Local Scalars ..
      REAL RWORK,SWORK
      INTEGER J,JDO,KDEL,KDEL1,KDEL2,KDO,LDO,M,NDO
C     ..
C     .. Common blocks ..
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      DO 10 JDO = 1,3
        RWORK = CONJUGATEVECTORS(JDO,1)
        CIRCLESPOKES(JDO,1) = RWORK
        CIRCLESPOKES(JDO,129) = RWORK
        CIRCLESPOKES(JDO,65) = -RWORK
        RWORK = CONJUGATEVECTORS(JDO,2)
        CIRCLESPOKES(JDO,33) = RWORK
        CIRCLESPOKES(JDO,97) = -RWORK
   10 CONTINUE
C
C
      DO 40 KDO = 1,NUMSUBDIVI
        RWORK = OrtConts(KDO)
        KDEL = 2** (6-KDO)
        KDEL1 = KDEL + 1
        KDEL2 = KDEL/2
C
C
        DO 30 LDO = KDEL1,65,KDEL
          J = LDO - KDEL
          M = LDO - KDEL2
C
C
          DO 20 NDO = 1,3
            SWORK = (CIRCLESPOKES(NDO,LDO)+CIRCLESPOKES(NDO,J))*RWORK
            CIRCLESPOKES(NDO,M) = SWORK
            CIRCLESPOKES(NDO,M+64) = -SWORK
   20     CONTINUE
   30   CONTINUE
   40 CONTINUE
C
C
      RETURN
      END
C
C
C
C     =============================
      SUBROUTINE getatmlabelposn(D)
C     =============================
C
C---- This routine finds positions for atom labels
C     ( minimum overlap)
C     the coord of bottom l.h. corner of label are xlabl,ylabl.
C     if Lposition_Flag ne. 0 , all labels are put to right of atom
C      minor change for plot84 7 sep 1984
C
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Labels
      PARAMETER (Max_Labels=100)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER NpackAtmConn
      PARAMETER (NpackAtmConn=5000000)
C     ..
C     .. Scalar Arguments ..
      REAL D
C     ..
C     .. Scalars in Common ..
      REAL Charcter_Size,Drawing_Size,Perspective_Distance,
     +     Picture_Scale,S,Shadelight_X,Shadelight_Y,Stereo_Separation
      INTEGER IERR,Label_atoms_Flag,Lcurrent_Colour,Ldraw_cell_Flag,
     +        Ljoin_Flag,Lpaper_Setting,Lperspective_Flag,
     +        Lplot_colour_Stereo,Lposition_Flag,Lprint_Flag,
     +        LShade_Flag,LSolid_Flag,LStereo_Flag,LSuperimpose_Flag,
     +        LTaper_Flag,LTrace_Flag,NAT,NATO,Natoms_plusCell,NC,NC1,
     +        Ncopies,Nspecified_Bonds,Num_Labtxt_input,Num_Moles,
     +        Num_Posit_Labels
      LOGICAL Log_Draw_Title
      CHARACTER Iden_Title*80
C     ..
C     .. Arrays in Common ..
      REAL Atom_Radii,Fractional_XYZ,Orthogonal_XYZ,Plotting_XYZ,
     +     Positioned_Labels,RP,ViewMatrix,XLABEL,YLABEL
      INTEGER IAtom_Connectivity,IAtom_Type,ISpecified_Bond_Params,
     +        LAtom_Inc_Flag,LAtom_Label_Flag,MOL
      CHARACTER Atm_Lab_Type*2,Atm_Identifier*10,Atm_Label*10,
     +          Atm_Lab_Txt*10
C     ..
C     .. Local Scalars ..
      REAL DIJ,DIN,DSQ,DX,DY,EDGE,H2,H4,HEIGHT,R2,RI,RJ,W,W2,WIDTH,X1,
     +     X2,XI,XJ,XLABL,Y1,Y2,YI,YJ,YLABL
      INTEGER I,IO,J,JO,K,M,MINA,MINL,NAB,NATL,NMAX,NRAT
C     ..
C     .. Local Arrays ..
      REAL RAT(Max_Atoms),RPS(Max_Atoms),XLAB(8),XPS(Max_Atoms),
     +     XRAT(Max_Atoms),YLAB(8),YPS(Max_Atoms),YRAT(Max_Atoms)
      INTEGER NSUM(8,2)
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC MIN,MOD,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_LABCHR/Atm_Lab_Txt(Max_Labels),
     +       Atm_Lab_Type(Max_Labels),Atm_Label(Max_Labels),
     +       Atm_Identifier(Max_Atoms),Iden_Title
      COMMON /com_LABELS/Num_Posit_Labels,Num_Labtxt_input,
     +       XLABEL(Max_Atoms),YLABEL(Max_Atoms),
     +       Positioned_Labels(2,Max_Labels)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      NMAX = 400
      NAB = NAT - 8
      NATL = Natoms_plusCell
      HEIGHT = Charcter_Size
      H4 = 0.25*HEIGHT
      EDGE = Drawing_Size/2.0
      WIDTH = Charcter_Size
C
C
      DO 220 I = 1,NAT
        IF (MOD(LAtom_Label_Flag(I),100)-1) 220,20,10
C
C---- position of label specified
C
   10   XLABEL(I) = XLABEL(I) + Plotting_XYZ(1,I)
        YLABEL(I) = YLABEL(I) + Plotting_XYZ(2,I)
        GO TO 220
   20   IF (I.LT.9 .AND. Ldraw_cell_Flag.EQ.0) GO TO 220
        XI = Plotting_XYZ(1,I)
        YI = Plotting_XYZ(2,I)
        RI = RP(I)
C
C---- add 0.5mm to the atom radius to allow a small gap.
C
        RI = RI + 0.5
        IF (Lposition_Flag.NE.0) GO TO 200
C
C---- set a distance to scan for atoms which
C     may intersect with label.
C     find width of label
C
        DO 30 K = 1,6
          IF (Atm_Identifier(I) (K:K).EQ.' ') GO TO 40
   30   CONTINUE
C
C
        K = 7
   40   W = (K-1)*WIDTH
C
C---- W is the width of the label in mm. we now set a distance
C       for scannig for possible intersecting atoms with this label.
C       this distance over rides the value od d supplied
C       to getatmlabelposn.
C
        D = RI + W + WIDTH
        W2 = 0.5*W
        H2 = 0.5*HEIGHT
        R2 = (RI+H4)/1.414
C
C---- set up the coords of the centres of label positions.
C
        XLAB(1) = RI + W2 + H4
        YLAB(1) = 0.0
        XLAB(2) = R2 + W2
        YLAB(2) = R2 + H2
        XLAB(3) = 0.0
        YLAB(3) = RI + H2 + H4
        XLAB(4) = -XLAB(2)
        YLAB(4) = YLAB(2)
        XLAB(5) = -XLAB(1)
        YLAB(5) = -YLAB(1)
        XLAB(6) = -XLAB(2)
        YLAB(6) = -YLAB(2)
        XLAB(7) = -XLAB(3)
        YLAB(7) = -YLAB(3)
        XLAB(8) = -XLAB(4)
        YLAB(8) = -YLAB(4)
C
C
        DO 50 K = 1,8
          XLAB(K) = XLAB(K) + XI
          YLAB(K) = YLAB(K) + YI
   50   CONTINUE
C
C---- make a list of atoms lying within the range (x1...x2, y1...y2)
C     this defines a square  around the atom from -d to +d (mm)
C
        X1 = XI - D
        X2 = XI + D
        Y1 = YI - D
        Y2 = YI + D
        NRAT = 0
C
C
        DO 80 J = 1,NATL
          IF (J.GT.Natoms_plusCell) GO TO 60
          IF (I.EQ.J) GO TO 80
          JO = MOD(J-9,NAB) + 9
          IF (LAtom_Inc_Flag(JO).EQ.0) GO TO 80
          XJ = Plotting_XYZ(1,J)
          YJ = Plotting_XYZ(2,J)
C
C---- negative radius for labels
C
          RJ = -RP(J)
          GO TO 70
   60     K = J - Natoms_plusCell
          XJ = XPS(K)
          YJ = YPS(K)
          RJ = RPS(K)
   70     IF (XJ.GT.X2+RJ) GO TO 80
          IF (XJ.LT.X1-RJ) GO TO 80
          IF (YJ.GT.Y2+RJ) GO TO 80
          IF (YJ.LT.Y1-RJ) GO TO 80
          DX = XJ - XI
          DY = YJ - YI
          IF (DX*DX+DY*DY.LT.0.01) GO TO 80
          NRAT = NRAT + 1
          XRAT(NRAT) = XJ
          YRAT(NRAT) = YJ
          RAT(NRAT) = RJ
   80   CONTINUE
C
C
        IF (NRAT.GT.NMAX) GO TO 190
C
C---- add dummy 'atom centres' at points where bonds
C     to atom i  intersect ci
C
        DIN = RI
C
C
        DO 110 K = 1,NC
          M = MOD(IAtom_Connectivity(K),NpackAtmConn)
          IO = M/Max_Atoms
          JO = MOD(M,Max_Atoms)
          IF (IO.EQ.I) GO TO 100
          IF (JO.EQ.I) GO TO 90
          GO TO 110
   90     M = JO
          JO = IO
          IO = M
  100     CONTINUE
          IF (LAtom_Inc_Flag(JO).EQ.0) GO TO 110
          DX = Plotting_XYZ(1,JO) - Plotting_XYZ(1,IO)
          DY = Plotting_XYZ(2,JO) - Plotting_XYZ(2,IO)
          DSQ = DX*DX + DY*DY
          IF (DSQ.LT.0.001) GO TO 110
          DIJ = SQRT(DSQ)
          NRAT = NRAT + 1
          XRAT(NRAT) = DIN*DX/DIJ + XI
          YRAT(NRAT) = DIN*DY/DIJ + YI
          RAT(NRAT) = H4
  110   CONTINUE
C
C
        IF (NRAT.GT.NMAX) GO TO 190
        DO 130 K = 1,8
          DO 120 J = 1,2
            NSUM(K,J) = 0
  120     CONTINUE
  130   CONTINUE
        MINL = 10000
C
C----  count intersections of label with atoms and other labels
C
        DO 170 M = 1,8
C
C---- test for intersection
C
          X1 = XLAB(M) - W2
          X2 = XLAB(M) + W2
          Y1 = YLAB(M) - H2
          Y2 = YLAB(M) + H2
C
C---- check that label is within picture area on x
C
          IF (X1.GT.-EDGE .AND. X2.LT.EDGE) GO TO 140
          NSUM(M,1) = 10000
          NSUM(M,2) = 10000
          GO TO 170
  140     CONTINUE
C
C
          DO 160 J = 1,NRAT
            XJ = XRAT(J)
            YJ = YRAT(J)
            RJ = RAT(J)
            K = 1
            IF (RJ.GT.0.0) GO TO 150
            K = 2
            RJ = -RJ
  150       IF (XJ.GT.X2+RJ) GO TO 160
            IF (XJ.LT.X1-RJ) GO TO 160
            IF (YJ.GT.Y2+RJ) GO TO 160
            IF (YJ.LT.Y1-RJ) GO TO 160
            NSUM(M,K) = NSUM(M,K) + 1
  160     CONTINUE
C
C---- record minimum number of label/label intersections
C
          MINL = MIN(MINL,NSUM(M,2))
  170   CONTINUE
C
C---- now find label position with minl l/l intersections
C     which has minimum l/atom intersections
C
        MINA = 10000
C
C
        DO 180 K = 1,8
          IF (NSUM(K,2).GT.MINL) GO TO 180
          IF (NSUM(K,1).GT.MINA) GO TO 180
          MINA = NSUM(K,1)
          M = K
  180   CONTINUE
C
C
        XLABL = XLAB(M) - W2
        YLABL = YLAB(M) - H2
C
C---- add this label coordinates to the atom list
C
        NATL = NATL + 1
        K = NATL - Natoms_plusCell
        XPS(K) = XLAB(M)
        YPS(K) = YLAB(M)
        RPS(K) = W2
        GO TO 210
  190   WRITE (6,FMT=6000) NMAX,Atm_Identifier(I)
 6000   FORMAT (' *** More than ',I4,' Atoms in label position calc li',
     +         'st...',A)
  200   XLABL = XI + RI + H4
        YLABL = YI
  210   XLABEL(I) = XLABL
        YLABEL(I) = YLABL
  220 CONTINUE
      RETURN
      END
C
C
C
C     =========================================
      SUBROUTINE getatomconnections(Lprint_Flag)
C     ==========================================
C
C
C
C----  set up connectivity list IAtom_Connectivity by distance
C      search within each residue
C      linkage between residues is done by atom Atm_Identifier
C
C   Lprint_Flag .ne.0   print diagnostics and connections
C
C
C  Orthogonal_XYZ(I,3)   orthogonal coordinates
C  Atom_Radii      bond radius
C
C  LAtom_Inc_Flag      on entry:
c      INTEGER NPackAtmID
C     =ISideChain_Code+2+isym*10+jcolr*1000+nres*10000+
C                 jexcl*NPackAtmID
C
C                negated if atom excluded
C            where ISideChain_Code
C               sidechain code =-1 ca,=0 m/c, =1 s/c
C                 isym   symmetry number (1-24)
C                    jcolr  colour code (0-3)
C                         =0 undefined
C                          1 black
C                          2 red
C                          3 green
C                          4 blue
C                          5 yellow
C                  nres   residue number
C                  jexcl  exclude code,
C                        =0 lower margin,=1 upper margin
C  no link generated if LAtom_Inc_Flag=0 for either atom
C
C
C       on exit: LAtom_Inc_Flag=
C         jcolr+1 or =-(jexcl*11+jcolr+1) if atom excluded
C
C  Atm_Identifier(i,6) label
C
C IN COMMON /com_Link/   :
C
C  Link_Atm_Names(i,2) 2 character label for link atoms i=1,2, for
C                Num_LinkSets (.le.2) link sets k
C  IResid_BndTyp(i)   bond types for residue i,
C              = 1000 * m/c type + s/c type
C              =-1 if unset
C  ISym_BndType(j)   bond types for symmetry number j
C              =-1 if unset
C Def_LnkBond(k)   bond length for link bond, for link set k
C
C  Num_LinkSets       number of link sets
C
C on exit, array IAtom_Connectivity
c      INTEGER NpackAtmConn
c      PARAMETER (NpackAtmConn = 5000000)
C          contains jtyp*NpackAtmConn+io*Max_Atoms+jo
C  bond of type jtyp joining atom io to atom jo
C
C
C
C
C
C
C---- initialise current residue number NCurrentResid, pointer
C     to start of residue KStart_Previous_Res
C     number of atoms in residue NumAtoms_in_Resid,
C     pointer to link atom in previous  residue ln2
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_BondType
      PARAMETER (Max_BondType=20)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Labels
      PARAMETER (Max_Labels=100)
      INTEGER Max_Symmetry
      PARAMETER (Max_Symmetry=192)
      INTEGER NpackAtmConn
      PARAMETER (NpackAtmConn=5000000)
      INTEGER NPackAtmID
      PARAMETER (NPackAtmID=10000000)
C     ..
C     .. Scalar Arguments ..
      INTEGER Lprint_Flag
C     ..
C     .. Scalars in Common ..
      INTEGER IERR,Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,
     +        Num_LinkSets
      CHARACTER Iden_Title*80
C     ..
C     .. Arrays in Common ..
      REAL Atom_Radii,Def_LnkBond,Fractional_XYZ,Orthogonal_XYZ
      INTEGER IAtom_Connectivity,IAtom_Type,IConn_BndType,IResid_BndTyp,
     +        ISym_BndType,LAtom_Inc_Flag,LAtom_Label_Flag
      CHARACTER Atm_Lab_Type*2,Link_Atm_Names*2,Atm_Identifier*10,
     +          Atm_Label*10,Atm_Lab_Txt*10
C     ..
C     .. Local Scalars ..
      REAL D,R,R2
      INTEGER I,IEND,IEX,ISCC,ISD,ISIDECHAIN_CODE,J,JCOLR,JDO200,JDO70,
     +        JEXCL,JTYP,K,KCurrentATM_NUM,KEND_CurrentRES,
     +        KSTART_PREVIOUS_RES,KWORKING_ATM,L,L1,L2,LL1,LL2,LN1,LN2,
     +        NCurrentRESID,NCurrentSYMM,NLAST_RESID,NLAST_SYMM,
     +        NUMATOMS_IN_RESID
      CHARACTER CLWORK*1,KLWORK*1
C     ..
C     .. Local Arrays ..
      INTEGER LSDCH(Max_Atoms)
      LOGICAL LINKCA(2,2)
C     ..
C     .. External Functions ..
      INTEGER getbondtype
      EXTERNAL getbondtype
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MAX
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_LABCHR/Atm_Lab_Txt(Max_Labels),
     +       Atm_Lab_Type(Max_Labels),Atm_Label(Max_Labels),
     +       Atm_Identifier(Max_Atoms),Iden_Title
      COMMON /com_LINK/Num_LinkSets,IResid_BndTyp(Max_Atoms),
     +       ISym_BndType(Max_Symmetry),IConn_BndType(Max_BondType),
     +       Def_LnkBond(2)
      COMMON /com_LNKCHR/Link_Atm_Names(2,2)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      NCurrentRESID = -1000
      KSTART_PREVIOUS_RES = 9
      NUMATOMS_IN_RESID = 0
      LN1 = -1
C
C---- if link atoms are ca, set linkca false
C
      DO 20 I = 1,2
        DO 10 J = 1,Num_LinkSets
          LINKCA(I,J) = .TRUE.
          IF (Link_Atm_Names(I,J).NE.'CA') GO TO 10
          LINKCA(I,J) = .FALSE.
   10   CONTINUE
   20 CONTINUE
C
C
      IF (Lprint_Flag.NE.0) THEN
        WRITE (6,FMT=6000) ((Link_Atm_Names(I,K),I=1,2),K=1,
     +    Num_LinkSets)
        WRITE (6,FMT=6002) IResid_BndTyp
        WRITE (6,FMT=6004) ISym_BndType,IConn_BndType
 6000   FORMAT (' Link_Atm_Names : ',A,2X,A)
 6002   FORMAT (' IResid_BndTyp',/ (1X,12I10,/))
 6004   FORMAT (' ISym_BndType',11I10,/1X,13I10,/' IConn_BndType',10I10,
     +         /1X,10I10,/)
      END IF
C
C---- loop atoms
C
      IEND = 0
      KCurrentATM_NUM = 8
C
C
   30 KCurrentATM_NUM = KCurrentATM_NUM + 1
C
C
      IF (KCurrentATM_NUM.GT.NAT) THEN
C
C---- end atom loop
C            set end flag
C
        IEND = 1
        GO TO 40
      END IF
C
C---- unpack LAtom_Inc_Flag, le. 0 for excluded atoms
C
C    LAtom_Inc_Flag(Max_Atoms)   =
C               ISideChain_Code + 2 + ISYM*10 + ICOLR*1000 +
C                IRes_Number*10000 + jexcl*NPackAtmID
C
C         jcolr+1 or =-(jexcl*11+jcolr+1) if atom excluded
C
      IEX = LAtom_Inc_Flag(KCurrentATM_NUM)
C
C
      IF (IEX.EQ.0) THEN
C
C---- same residue , store flags
C
        NUMATOMS_IN_RESID = NUMATOMS_IN_RESID + 1
        LSDCH(NUMATOMS_IN_RESID) = ISIDECHAIN_CODE
        IF (IEX.EQ.0) GO TO 30
C
C---- reset LAtom_Inc_Flag to  jcolr+1 or or -(jexcl*11+jcolr+1)
C
        LAtom_Inc_Flag(KCurrentATM_NUM) = JCOLR + 1
        IF (IEX.LT.0) LAtom_Inc_Flag(KCurrentATM_NUM) = - (JEXCL*11+
     +      JCOLR+1)
        GO TO 30
      END IF
C
C    LAtom_Inc_Flag(Max_Atoms)   =
C              ISideChain_Code + 2 + ISYM*10 + ICOLR*1000 +
C               IRes_Number*10000 + jexcl*NPackAtmID
C
      I = ABS(IEX)
      JEXCL = I/NPackAtmID
      I = I - JEXCL*NPackAtmID
      NLAST_RESID = I/10000
      I = I - NLAST_RESID*10000
      JCOLR = I/1000
      I = I - JCOLR*1000
      NLAST_SYMM = I/10
      ISIDECHAIN_CODE = I - NLAST_SYMM*10 - 2
C
C---- is this type same residue ?
C
      IF (NCurrentRESID.LT.-999) THEN
C
C---- finished with this residue, store flags for new one
C
        IF (IEND.NE.0) THEN
          WRITE (6,FMT=6006) NC
 6006     FORMAT (1X,I8,' bonds found')
          RETURN
        END IF
C
C
        NCurrentRESID = NLAST_RESID
        NCurrentSYMM = NLAST_SYMM
        KSTART_PREVIOUS_RES = KCurrentATM_NUM
        LN2 = LN1
        LL2 = LL1
        NUMATOMS_IN_RESID = 0
C
C---- store this atom
C     same residue , store flags
C
        NUMATOMS_IN_RESID = NUMATOMS_IN_RESID + 1
        LSDCH(NUMATOMS_IN_RESID) = ISIDECHAIN_CODE
        IF (IEX.EQ.0) GO TO 30
C
C---- reset LAtom_Inc_Flag to  jcolr+1 or or -(jexcl*11+jcolr+1)
C
        LAtom_Inc_Flag(KCurrentATM_NUM) = JCOLR + 1
        IF (IEX.LT.0) LAtom_Inc_Flag(KCurrentATM_NUM) = - (JEXCL*11+
     +      JCOLR+1)
        GO TO 30
      END IF
C
C
      IF (NLAST_RESID.EQ.NCurrentRESID) THEN
C
C---- same residue , store flags
C
        NUMATOMS_IN_RESID = NUMATOMS_IN_RESID + 1
        LSDCH(NUMATOMS_IN_RESID) = ISIDECHAIN_CODE
        IF (IEX.EQ.0) GO TO 30
C
C---- reset LAtom_Inc_Flag to  jcolr+1 or or -(jexcl*11+jcolr+1)
C
        LAtom_Inc_Flag(KCurrentATM_NUM) = JCOLR + 1
        IF (IEX.LT.0) LAtom_Inc_Flag(KCurrentATM_NUM) = - (JEXCL*11+
     +      JCOLR+1)
        GO TO 30
      END IF
C
C---- new residue, link up previous one
C     atoms KStart_Previous_Res to KEnd_CurrentRes
C
   40 KEND_CurrentRES = KSTART_PREVIOUS_RES + NUMATOMS_IN_RESID - 1
      L1 = 1
      L2 = Num_LinkSets
C
C---- loop atom 1
C
      DO 150 JDO200 = KSTART_PREVIOUS_RES,KEND_CurrentRES
C
C---- no bond if LAtom_Inc_Flag=0
C
        J = JDO200 - KSTART_PREVIOUS_RES + 1
        ISCC = LSDCH(J)
C
C---- is this a link atom ?
C
        DO 90 I = 1,2
          DO 80 L = L1,L2
C
C---- If this link atom is CA, test mc/sc type
C
            IF (LINKCA(I,L) .OR. ISCC.NE.-1) GO TO 50
            LN1 = JDO200
            LL1 = L
            GO TO 100
C
C---- first character
C
   50       IF (Link_Atm_Names(I,L) (1:1).NE.
     +          Atm_Identifier(JDO200) (1:1)) GO TO 80
C
C---- if second character is link Atm_Identifier is blank,
C     this can match  with a digit
C
            KLWORK = Atm_Identifier(JDO200) (2:2)
            IF (Link_Atm_Names(I,L) (2:2).NE.' ') GO TO 70
C
C
            DO 60 JDO70 = 0,9
              WRITE (CLWORK,FMT=6008) JDO70
 6008         FORMAT (I1)
              IF (Atm_Identifier(JDO200) (2:2).EQ.CLWORK) KLWORK = ' '
   60       CONTINUE
C
C
   70       IF (Link_Atm_Names(I,L) (2:2).EQ.KLWORK) GO TO 100
   80     CONTINUE
   90   CONTINUE
C
C---- not link atom
C
        GO TO 120
C
C---- link atom
C
  100   L1 = L
        L2 = L
        IF (I.EQ.2) GO TO 110
C
C---- link atom 1, link back to previous residue
C
        IF (LN2.LT.0) GO TO 120
C
C---- bond only if distance ok
C
        R = Def_LnkBond(MAX(L,LL2))
C
C
        D = (Orthogonal_XYZ(LN2,1)-Orthogonal_XYZ(JDO200,1))**2 +
     +      (Orthogonal_XYZ(LN2,2)-Orthogonal_XYZ(JDO200,2))**2 +
     +      (Orthogonal_XYZ(LN2,3)-Orthogonal_XYZ(JDO200,3))**2
C
C
        IF (D.GT.R*R) GO TO 120
C
C---- get main chain bond type
C
C              ******************************************************
        JTYP = getbondtype(NCurrentRESID,NCurrentSYMM,0,JDO200,LN2)
C              ******************************************************
C
C---- store bond
C
        NC = NC + 1
        IAtom_Connectivity(NC) = JTYP*NpackAtmConn + LN2*Max_Atoms +
     +                           JDO200
        IF (Lprint_Flag.NE.0) WRITE (6,FMT=6010) JTYP,LN2,JDO200,
     +      NCurrentRESID,NCurrentSYMM
 6010   FORMAT (' LINK:JTYP,Jdo200,JO,NLast_Resid,NLast_Symm',5I10)
        GO TO 120
C
C---- link atom 2, store pointer
C
  110   LN1 = JDO200
        LL1 = L
C
C---- pick up position of atom 1
C
  120   IF (NUMATOMS_IN_RESID.EQ.1) GO TO 150
C
C---- inner loop over other atoms to find bonds
C
        DO 140 KWORKING_ATM = (JDO200+1),KEND_CurrentRES
          IF (LAtom_Inc_Flag(KWORKING_ATM).EQ.0) GO TO 140
          R = Atom_Radii(KWORKING_ATM) + Atom_Radii(JDO200)
          R2 = R**2
C
C
          D = (Orthogonal_XYZ(JDO200,1)-
     +        Orthogonal_XYZ(KWORKING_ATM,1))**2 +
     +        (Orthogonal_XYZ(JDO200,2)-Orthogonal_XYZ(KWORKING_ATM,
     +        2))**2 + (Orthogonal_XYZ(JDO200,3)-
     +        Orthogonal_XYZ(KWORKING_ATM,3))**2
          IF (D.GT.R2) GO TO 140
C
C---- we have a bond, get type
C      bond between m/c and s/c atoms is s/c bond
C
          ISD = 1
          IF (LSDCH(KWORKING_ATM-KSTART_PREVIOUS_RES+1).GT.
     +        0) GO TO 130
          IF (ISCC.LE.0) ISD = 0
C
C                ***************************************************
  130     JTYP = getbondtype(NCurrentRESID,NCurrentSYMM,ISD,JDO200,
     +                       KWORKING_ATM)
C                ***************************************************
C
C---- store bond
C
          NC = NC + 1
          IAtom_Connectivity(NC) = JTYP*NpackAtmConn +
     +                             JDO200*Max_Atoms + KWORKING_ATM
          IF (Lprint_Flag.NE.0) WRITE (6,FMT=6010) JTYP,JDO200,
     +        KWORKING_ATM,NCurrentRESID,NCurrentSYMM
  140   CONTINUE
  150 CONTINUE
C
C---- finished with this residue, store flags for new one
C
      IF (IEND.NE.0) THEN
        WRITE (6,FMT=6006) NC
        RETURN
      END IF
C
C
      NCurrentRESID = NLAST_RESID
      NCurrentSYMM = NLAST_SYMM
      KSTART_PREVIOUS_RES = KCurrentATM_NUM
      LN2 = LN1
      LL2 = LL1
      NUMATOMS_IN_RESID = 0
C
C---- store this atom
C     same residue , store flags
C
      NUMATOMS_IN_RESID = NUMATOMS_IN_RESID + 1
      LSDCH(NUMATOMS_IN_RESID) = ISIDECHAIN_CODE
      IF (IEX.EQ.0) GO TO 30
C
C---- reset LAtom_Inc_Flag to  jcolr+1 or or -(jexcl*11+jcolr+1)
C
      LAtom_Inc_Flag(KCurrentATM_NUM) = JCOLR + 1
      IF (IEX.LT.0) LAtom_Inc_Flag(KCurrentATM_NUM) = - (JEXCL*11+
     +    JCOLR+1)
      GO TO 30
C
C
      END
C
C
C
C     ===================================
      SUBROUTINE getatomid(IRETURN,FWORK)
C     ===================================
C
C
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Labels
      PARAMETER (Max_Labels=100)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
C     ..
C     .. Scalar Arguments ..
      INTEGER IRETURN
      CHARACTER FWORK* (*)
C     ..
C     .. Scalars in Common ..
      REAL Charcter_Size,Drawing_Size,Perspective_Distance,
     +     Picture_Scale,S,Shadelight_X,Shadelight_Y,Stereo_Separation
      INTEGER IERR,Label_atoms_Flag,Lcurrent_Colour,Ldraw_cell_Flag,
     +        Ljoin_Flag,Lpaper_Setting,Lperspective_Flag,
     +        Lplot_colour_Stereo,Lposition_Flag,Lprint_Flag,
     +        LShade_Flag,LSolid_Flag,LStereo_Flag,LSuperimpose_Flag,
     +        LTaper_Flag,LTrace_Flag,NAT,NATO,Natoms_plusCell,NC,NC1,
     +        Ncopies,Nspecified_Bonds,Num_Moles
      LOGICAL Log_Draw_Title
      CHARACTER Iden_Title*80
C     ..
C     .. Arrays in Common ..
      REAL Atom_Radii,Fractional_XYZ,Orthogonal_XYZ,Plotting_XYZ,RP,
     +     ViewMatrix
      INTEGER IAtom_Connectivity,IAtom_Type,ISpecified_Bond_Params,
     +        LAtom_Inc_Flag,LAtom_Label_Flag,MOL
      CHARACTER Atm_Lab_Type*2,Atm_Identifier*10,Atm_Label*10,
     +          Atm_Lab_Txt*10
C     ..
C     .. Local Scalars ..
      INTEGER I,I1,I2,I3,JDO,LINDEX,LLX
      CHARACTER C1*1,C2*1
C     ..
C     .. Local Arrays ..
      CHARACTER CNUM(10)*1
C     ..
C     .. External Functions ..
      INTEGER LENSTR
      EXTERNAL LENSTR
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,ICHAR,INDEX
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_LABCHR/Atm_Lab_Txt(Max_Labels),
     +       Atm_Lab_Type(Max_Labels),Atm_Label(Max_Labels),
     +       Atm_Identifier(Max_Atoms),Iden_Title
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Data statements ..
      DATA CNUM/'0','1','2','3','4','5','6','7','8','9'/
C     ..
C
C
      DO 10 JDO = 1,10
        LINDEX = INDEX(FWORK,CNUM(JDO))
        IF (LINDEX.NE.0) GO TO 20
   10 CONTINUE
C
C---- no number attached to atom element symbol.
C     this causes a characteristic number to be generated from the
C     literal values of the symbol. this will be used for
C     identification of atom type.
C
      C1 = FWORK(1:1)
      C2 = FWORK(2:2)
      I1 = ICHAR(C1)
      I2 = ICHAR(C2)
      I3 = I1**2 + I2**3
      IRETURN = -ABS(I3)
C
      GO TO 40
C
C---- search list of atoms for this name
C
   20 CONTINUE
C
C
      LLX = LENSTR(FWORK)
C
C
      DO 30 I = 1,NAT
        IF (Atm_Identifier(I) (1:LLX).NE.FWORK) GO TO 30
        IRETURN = I
        GO TO 40
   30 CONTINUE
C
C---- Un-identified atom name
C
      IRETURN = 0
   40 RETURN
      END
C
C
C
C     ======================================================
      INTEGER FUNCTION getbondtype(NRES,ISYM,ISIDECHAIN_CODE,NWORK,
     +                             NLOOP)
C     =======================================================
C
C
C---- get bond type getbondtype for atom of
C     sidechain code ISideChain_Code,
C     residue nres,  symmetry number isym
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_BondType
      PARAMETER (Max_BondType=20)
      INTEGER Max_Symmetry
      PARAMETER (Max_Symmetry=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER ISIDECHAIN_CODE,ISYM,NLOOP,NRES,NWORK
C     ..
C     .. Scalars in Common ..
      INTEGER Num_LinkSets
C     ..
C     .. Arrays in Common ..
      REAL Def_LnkBond
      INTEGER IConn_BndType,IResid_BndTyp,ISym_BndType
      CHARACTER Link_Atm_Names*2
C     ..
C     .. Local Scalars ..
      INTEGER I
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC MOD
C     ..
C     .. Common blocks ..
      COMMON /com_LINK/Num_LinkSets,IResid_BndTyp(Max_Atoms),
     +       ISym_BndType(Max_Symmetry),IConn_BndType(Max_BondType),
     +       Def_LnkBond(2)
      COMMON /com_LNKCHR/Link_Atm_Names(2,2)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C---- symmetry type takes precedence over residue type
C
      I = ISym_BndType(ISYM)
      IF (I.GT.0) GO TO 10
      I = IResid_BndTyp(NWORK)
      IF (I.GT.0) GO TO 10
C
C--- bond type unspecified, set to 0
C
      getbondtype = 0
      RETURN
C
C---- types for mainchain/sidechain
C
   10 IF (ISIDECHAIN_CODE.GT.0) GO TO 20
C
C---- mainchain
C
      getbondtype = I/1000
      RETURN
C
C---- sidechain
C
   20 getbondtype = MOD(I,1000)
      RETURN
      END
C
C
C
C     ============================================================
      SUBROUTINE getbvalues(BB,NATOMNUMBER,KBTYPE,Temp_Factors,
     +                                               ATM_AXIS_RMS)
C     ============================================================
C
C---- thermal factor routine
C
C
C
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
C     ..
C     .. Scalar Arguments ..
      INTEGER KBTYPE,NATOMNUMBER
C     ..
C     .. Array Arguments ..
      REAL BB(6),ATM_AXIS_RMS(3,Max_Atoms),Temp_Factors(3,3,Max_Atoms)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,CurrentCharcter_Size,
     +     D001,D010,D100,Dist_to_Boundary,Ellipse_ScaleFact,
     +     Eye_Distance,RFR(4,4),ROR(4,4),
     +     Overlap_Margin,Paper_Length,
     +     Paper_Width,Plotter_Step,Retrace_Displmnt,Scal_pre_Proj,SCL
      INTEGER Iuse_persp_Labs,Lorthogonalisation_Code,Lpack_Flag,
     +        NDrawing_Number
      LOGICAL UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL AID,Base_Vect_Refsys,Base_Vect_Wrksys,BBB,Drawing_Origin,
     +     Ellipse_Resolution,G_Metric_Tensor,OrtCell,OrtConts,
     +     P1_to_Refsys,P1_to_Wrksys,Principal_axis_rms,
     +     Prin_Angstrom_Vectors,Prin_Axis_Col_Vects,Q,
     +     Recip_Metric_Tensor,Sorted_xyz,Title_Rotation,
     +     Title_RotnTrans,XABC,XCLANG,XO
      INTEGER KSortPointer
C     ..
C     .. Local Scalars ..
      REAL BWORK,DEBYE_WALLER_FACTOR,PI,RESOL_FUDGE_FACTOR,T1,T2,T3,
     +     TEMPFACT_BASE
      INTEGER IERROR,J,K,KBFLAG,L,N,NBTYPE
C     ..
C     .. Local Arrays ..
      REAL DA(3,3),V1(4),VWORK(3,3)
C     ..
C     .. External Functions ..
      REAL vectbymatrxbyvector
      EXTERNAL vectbymatrxbyvector
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPERR,geteigenvalues,getunitvectors,matrxbymatrx,PARSER,
     +         vectorstore
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MOD,SIGN,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Data statements ..
      DATA PI/3.1415926/
C
ccx 8 * pi * pi  -
C
      DATA DEBYE_WALLER_FACTOR/78.956835/
      DATA RESOL_FUDGE_FACTOR/12.00/
C     ..
C
C---- thermal parameters
C
      T1 = 0.0
C
C
      IF (BB(2).NE.0.0 .OR. BB(3).NE.0.0) THEN
        Temp_Factors(1,1,NATOMNUMBER) = BB(1)
        Temp_Factors(2,1,NATOMNUMBER) = BB(2)
        Temp_Factors(3,1,NATOMNUMBER) = BB(3)
        Temp_Factors(1,2,NATOMNUMBER) = BB(4)
        Temp_Factors(2,2,NATOMNUMBER) = BB(5)
        Temp_Factors(3,2,NATOMNUMBER) = BB(6)
        NBTYPE = KBTYPE
        KBFLAG = 0
      ELSE
C
C----    NBType = 7 ! debye-waller
C
        BWORK = SQRT(BB(1)/DEBYE_WALLER_FACTOR)/RESOL_FUDGE_FACTOR
        IF (BWORK.LT.0.025) BWORK = 0.025
        Temp_Factors(1,1,NATOMNUMBER) = BWORK
        Temp_Factors(2,1,NATOMNUMBER) = BWORK
        Temp_Factors(3,1,NATOMNUMBER) = BWORK
        Temp_Factors(1,2,NATOMNUMBER) = 0.0
        Temp_Factors(2,2,NATOMNUMBER) = 0.0
        Temp_Factors(3,2,NATOMNUMBER) = 0.0
        NBTYPE = 1
        KBFLAG = -1
      END IF
C
C---- convert temp factor coef to standard type zero
C
      T1 = Temp_Factors(1,1,NATOMNUMBER)
      TEMPFACT_BASE = 0.0506605918
C
C
      IF (NBTYPE.EQ.7) GO TO 170
      IF (NBTYPE.EQ.8) GO TO 180
C
C
      IF (NBTYPE.EQ.2 .OR. NBTYPE.EQ.6 .OR. NBTYPE.EQ.10) THEN
C
C---- type 1 bij
C        -D(b11h^2 + b22k^2 + b33l^2 + cb12hk + cb13hl + cb23kl)
C         Base= e c=1 d= 1
C
        TEMPFACT_BASE = 0.0506605918
C
C
        DO 10 J = 1,3
          Temp_Factors(J,2,NATOMNUMBER) = Temp_Factors(J,2,NATOMNUMBER)*
     +      0.5
   10   CONTINUE
C
C
      ELSE IF (NBTYPE.EQ.4) THEN
C
C---- type 3 bij
C               -D(b11h^2 + b22k^2 + b33l^2 + cb12hk + cb13hl + cb23kl)
C         Base= 2 c=2 d= 1
C
        TEMPFACT_BASE = 0.351152464
C
C
        DO 20 J = 1,3
          Temp_Factors(J,2,NATOMNUMBER) = Temp_Factors(J,2,NATOMNUMBER)*
     +      0.5
   20   CONTINUE
C
C---- type 2 bij
C               -D(b11h^2 + b22k^2 + b33l^2 + cb12hk + cb13hl + cb23kl)
C         Base= 2 c=2 d= 1
C
      ELSE IF (NBTYPE.EQ.3) THEN
        TEMPFACT_BASE = 0.351152464
      END IF
C
C---- types 0 through 5
C
C      test if isotropic
C
      IF (ABS(Temp_Factors(2,1,NATOMNUMBER)) .LT. 1.0E-6) GO TO 170
C
C
      DO 60 J = 1,3
        DO 50 L = J,3
          T2 = TEMPFACT_BASE
          IF ((NBTYPE-5).LT.0) GO TO 40
          IF ((NBTYPE-6).GT.0) GO TO 30
C
C---- types 4 and 5
C
          T2 = BBB(J)*BBB(L)*T2*0.25
          GO TO 40
C
C---- types 8 and 9 (u(i,j) tensor systems)
C
   30     T2 = BBB(J)*BBB(L)
          IF ((K-11).LT.0) GO TO 40
C
C---- type 10, (cartesian tensor system)
C
          T2 = 1.0
   40     IF (J.EQ.L) THEN
            VWORK(J,J) = Temp_Factors(J,1,NATOMNUMBER)*T2
            GO TO 50
          END IF
          VWORK(J,L) = Temp_Factors(J,L,NATOMNUMBER)*T2
          VWORK(L,J) = VWORK(J,L)
   50   CONTINUE
   60 CONTINUE
C
C---- find principal axes
C
      IF (K-11.GE.0) THEN
C
C            ********************************************************
        CALL matrxbymatrx(VWORK,Q,Prin_Axis_Col_Vects)
        CALL matrxbymatrx(Base_Vect_Refsys,Prin_Axis_Col_Vects,VWORK)
C            ********************************************************
C
      END IF
C
C
      IERROR = 0
C
C          ***********************************************************
      CALL matrxbymatrx(VWORK,G_Metric_Tensor,DA)
      CALL geteigenvalues(DA,Principal_axis_rms,Prin_Angstrom_Vectors,
     +                    IERROR)
C          ***********************************************************
C
C---- are eigenvalues positive
C
      IF (Principal_axis_rms(1).GT.0.0) THEN
        IF (IERROR.LT.0) THEN
          GO TO 100
        ELSE IF (IERROR.EQ.0) THEN
          GO TO 150
        END IF
      END IF
C
C---- 3 equal eigenvalues, use reference vectors
C
   70 T3 = SIGN(SQRT(ABS(Principal_axis_rms(1)+Principal_axis_rms(2)+
     +     Principal_axis_rms(3))/3.0),Principal_axis_rms(1))
C
C
      DO 90 J = 1,3
        DO 80 K = 1,3
          Temp_Factors(J,K,NATOMNUMBER) = Base_Vect_Refsys(J,K)
   80   CONTINUE
C
C
        ATM_AXIS_RMS(J,NATOMNUMBER) = T3
   90 CONTINUE
C
C
      GO TO 200
  100 IF ((IERROR+6).LE.0) GO TO 70
C
C---- two equal eigenvalues
C
      N = IERROR + 5
C
C          ************************************************
      CALL getunitvectors(Prin_Angstrom_Vectors(1,N),V1,-1)
C          ************************************************
C
      DO 110 K = 1,3
        IF ((ABS(vectbymatrxbyvector(V1,G_Metric_Tensor,
     +      Base_Vect_Refsys(1,K)))-0.58).LT.0.0) GO TO 120
  110 CONTINUE
C
C          ************************************************
  120 CALL matrxbymatrx(G_Metric_Tensor,DA,Title_RotnTrans)
      CALL vectorstore(V1,Base_Vect_Refsys(1,K),DA,-1)
C          ************************************************
C
      DO 140 K = 1,3
        L = MOD(N+K-2,3) + 1
        DO 130 J = 1,3
          Temp_Factors(J,L,NATOMNUMBER) = DA(J,K)
  130   CONTINUE
C
C
        ATM_AXIS_RMS(L,NATOMNUMBER) = SIGN(SQRT(ABS(vectbymatrxbyvector
     +                                (DA(1,K),Title_RotnTrans,DA(1,
     +                                K)))),Principal_axis_rms(L))
  140 CONTINUE
      GO TO 200
C
C---- make eigenvectors 1 angstrom long
C
C          **********************************************
  150 CALL vectorstore(Prin_Angstrom_Vectors(1,1),
     +                 Prin_Angstrom_Vectors(1,3),
     +                 Temp_Factors(1,1,NATOMNUMBER),-1)
C          **********************************************
C
      IERROR = 0
C
C---- sqrt eigenvalue = rms displacement
C
      DO 160 J = 1,3
        T2 = Principal_axis_rms(J)
        ATM_AXIS_RMS(J,NATOMNUMBER) = SIGN(SQRT(ABS(T2)),T2)
  160 CONTINUE
      GO TO 200
C
C---- type 7 (isotropic temp factor)
C
  170 T1 = SQRT(T1/ (8.0*PI*PI))*8.0
C
C---- type 7 (dummy sphere)
C
  180 DO 190 J = 1,3
        ATM_AXIS_RMS(J,NATOMNUMBER) = T1
  190 CONTINUE
C
C
  200 IERROR = 0
C
C
      IF (KBFLAG.LE.0) THEN
        ATM_AXIS_RMS(2,NATOMNUMBER) = ATM_AXIS_RMS(1,NATOMNUMBER)
        ATM_AXIS_RMS(3,NATOMNUMBER) = ATM_AXIS_RMS(1,NATOMNUMBER)
      END IF
C
C
      RETURN

      END
C
C
C
C     ==========================
      SUBROUTINE getcellmatrices
C     ==========================
C
C
C  EJD extend to Lorthogonalisation_Code 3  -..
C Lorthogonalisation_Code 1  has axes a aXc* c*
C      - suitable for sectionning down Z
C Lorthogonalisation_Code 2  has axes a* a*Xc c
C      - suitable for sectionning down X
C Lorthogonalisation_Code 3  has axes a b* aXb*
C      - suitable for sectionning down Y
C
C---- calculate orthogonalisation matrices from cell dimensions
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
C     ..
C     .. Scalars in Common ..
      REAL RFR(4,4),ROR(4,4)
      INTEGER Lorthogonalisation_Code
C     ..
C     .. Arrays in Common ..
      REAL AID,OrtCell,OrtConts,XABC,XCLANG
C     ..
C     .. Local Scalars ..
      REAL ACELL,ALF,BCELL,BET,CA,CASTAR,CB,CCELL,CG,CGSTAR,DET,GAM,RAD,
     +     SA,SASTAR,SB,SG,SGSTAR,VOL
C     ..
C     .. Local Scalar Arrays ..
      REAL   CELLG(6),RRR(3,3,6)
C     .. Intrinsic Functions ..
      INTRINSIC COS,SIN,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Data statements ..
      DATA RAD/57.2957795/
C     ..
C
C
      CELLG(1)= XABC(1)
      CELLG(2)= XABC(2)
      CELLG(3)= XABC(3)
      CELLG(4)= XCLANG(1)
      CELLG(5)= XCLANG(2)
      CELLG(6)= XCLANG(3)
        CALL RBFRO1(CELLG,VOL,RRR)
        ROR(4,4)=1.0
        DO I = 1,3
        ROR(I,4)=0.0
        ROR(4,I)=0.0
         DO J = 1,3
          ROR(I,J)=RRR(I,J,Lorthogonalisation_Code)
         END DO 
        END DO 
       CALL RBRINV(ROR,RFR)
c      D100 = VOL/ (BCELL*CCELL*SA)
c      D010 = VOL/ (ACELL*CCELL*SB)
c      D001 = VOL/ (ACELL*BCELL*SG)
      D100 = VOL/ (XABC(2)*XABC(3)*SIN(XCLANG(1)/RAD))
      D010 = VOL/ (XABC(1)*XABC(3)*SIN(XCLANG(2)/RAD))
      D001 = VOL/ (XABC(1)*XABC(2)*SIN(XCLANG(3)/RAD))
c      ACELL = XABC(1)
c      BCELL = XABC(2)
c      CCELL = XABC(3)
c      ALF = XCLANG(1)
c      BET = XCLANG(2)
c      GAM = XCLANG(3)
C
C---- Ref. rollett computing methods in crystallography p.23
C
c      SA = SIN(ALF/RAD)
c      SB = SIN(BET/RAD)
c      SG = SIN(GAM/RAD)
c      CA = COS(ALF/RAD)
c      CB = COS(BET/RAD)
c      CG = COS(GAM/RAD)
c      CASTAR = (CB*CG-CA)/ (SB*SG)
c      SASTAR = SQRT(1.0-CASTAR**2)
c      CGSTAR = (CA*CB-CG)/ (SA*SB)
c      SGSTAR = SQRT(1.0-CGSTAR**2)
C
C     GO TO (10,20,30) Lorthogonalisation_Code
C
C---- brookhaven convention
C     Lorthogonalisation_Code 1  has axes a aXc* c*
C        - suitable for sectionning down Z
C
c  10 OR1 = ACELL
c     OR2 = BCELL*CG
c     OR3 = CCELL*CB
c     OR4 = 0.0
c     OR5 = BCELL*SG
c     OR6 = (CA-CB*CG)*CCELL/SG
c     OR7 = 0.0
c     OR8 = 0.0
c     OR9 = SQRT(1.0-CA**2-CB**2-CG**2+2.0*CA*CB*CG)*CCELL/SG
c
c---- orthogonal to fractional matrix
c
c     DET = OR1*OR5*OR9
c     FR1 = 1.0/OR1
c     FR2 = -OR2/ (OR1*OR5)
c     FR3 = (OR2*OR6-OR3*OR5)/DET
c     FR4 = 0.0
c     FR5 = 1.0/OR5
c     FR6 = -OR6*OR1/DET
c     FR7 = 0.0
c     FR8 = 0.0
c     FR9 = 1.0/OR9
c     GO TO 40
c
c---- orthogonalisation for rollett convention
c     Lorthogonalisation_Code 2  has axes a* a*Xc c
c       - suitable for sectionning down X
c
c  20 OR1 = ACELL*SB*SGSTAR
c     OR4 = -ACELL*SB*CGSTAR
c     OR5 = BCELL*SA
c     OR7 = ACELL*CB
c     OR8 = BCELL*CA
c     OR9 = CCELL
c     OR2 = 0.0
c     OR3 = 0.0
c     OR6 = 0.0
c
c---- calculate matrix to convert orthogonal coordinates
c     to fractional
c
c     DET = OR1*OR5*OR9
c     FR1 = OR5*OR9/DET
c     FR4 = -OR4*OR9/DET
c     FR5 = OR1*OR9/DET
c     FR7 = (OR4*OR8-OR7*OR5)/DET
c     FR8 = -OR1*OR8/DET
c     FR9 = OR1*OR5/DET
c     FR2 = 0.0
c     FR3 = 0.0
c     FR6 = 0.0
c     GO TO 40
c
c---- ncode 6 convention
c     Lorthogonalisation_Code 3  has axes a b* aXb*
c       - suitable for sectionning down Y
c     EJD extend to Lorthogonalisation_Code 3  -..
c
c  30 OR1 = ACELL
c     OR2 = BCELL*CG
c     OR3 = CCELL*CB
c     OR4 = 0.0
c     OR5 = BCELL*SG*SASTAR
c     OR6 = 0.0
c     OR7 = 0.0
c     OR8 = -BCELL*SG*CASTAR
c     OR9 = CCELL*SB
c
c---- orthogonal to fractional matrix
c
c     DET = OR1*OR5*OR9
c     FR1 = 1.0/OR1
c     FR2 = (OR3*OR8-OR2*OR9)/DET
c     FR3 = -OR3*OR5/DET
c     FR4 = 0.0
c     FR5 = 1.0/OR5
c     FR6 = 0.0
c     FR7 = 0.0
c     FR8 = -OR1*OR8/DET
c     FR9 = 1.0/OR9
c     GO TO 40
c
c----  set up limits for testing if atom in
c      range for distance calculatio
c
c  40 VOL = ACELL*BCELL*CCELL*SQRT(1.0-CA*CA-CB*CB-CG*CG+2.0*CA*CB*CG)
c     D100 = VOL/ (BCELL*CCELL*SA)
c     D010 = VOL/ (ACELL*CCELL*SB)
c     D001 = VOL/ (ACELL*BCELL*SG)
C
      RETURN
      END
C
C
C
C     =================================================
      SUBROUTINE getcontours(FARRAY,M,N,BITS,CONT,NCONT,X,Y,IPTMAX)
C     =================================================
C
C
C                  Contur routine
C
C  Farray is function to be contoured, dimensioned a(m,n) or a(m*n)
C  M = number of rows of a
C  N = number of column
C  BITS is logical*1 array dimensioned at least
C        2*m*n in calling routine.
C  CONT  array of ncont contour levels
C  X,Y  arrays of dimension iptmax used in subroutine to store
C       contour lines
C
C  NLine_Segments = number of line Segments
C                   between contour level crossings
C                   in unit grid
C  CELL.  if >= 1, then routine finds NLine_Segments-1 coordinates
C          between crossings, by linear interpolation on
C          the unit cell.
C
C  CONTUR output via calls to subroutine joincoordpairs (x,y,ipt)
C     x,y are coordinates with 0<=x<m and 0<=y<n.
C
C  SUBROUTINE contourstyle(CurrentContour)
C  is called to set up plotting style for
C   contour level CurrentContour
C
C
C
C
C     .. Scalar Arguments ..
      INTEGER IPTMAX,M,N,NCONT
C     ..
C     .. Array Arguments ..
      REAL CONT(NCONT),FARRAY(*),X(*),Y(*)
      LOGICAL*1 BITS(*)
C     ..
C     .. Local Scalars ..
      REAL A1,A2,A3,A4,C1,C2,C3,C4,C5,C6,C7,C8,CurrentCONTOUR,DIV,
     +     XBASE,XFIN,XINC,XSAVE,XSTART,YBASE,YFIN,YINC,YSAVE,YSTART
      INTEGER I,I1,I2,I3,I4,ICNT,IENT,IEXIT,INTERP,IOPP,IROUND,IRS,IRTN,
     +        ISAVE,ISUB,ISUBSV,J,JCNT,JEXIT,JRTN,JSAVE,JSUB,JSUBSV,K1,
     +        K2,K3,K4,K5,K6,K7,KPOINT,MM,NContour_Levels,
     +        NLINE_Segments,NN,NROWS_BY_COLUMNS,NROWS_BY_COLUMNS2
C     ..
C     .. Local Arrays ..
      REAL C(4),SIDE(4)
      INTEGER IDIR(4),ISIDE(5),IVCT(4),JVCT(4),KVCT(4),LVCT(4),NVCT(4)
C     ..
C     .. External Subroutines ..
      EXTERNAL contourstyle,joincoordpairs
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC REAL
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Data statements ..
      DATA IDIR/3,4,1,2/,ISIDE/1,2,3,4,1/,IVCT/0,-1,0,1/,JVCT/-1,0,1,0/,
     +     SIDE/0.0,0.0,1.0,1.0/,KVCT/4,1,2,3/,LVCT/2,3,4,1/
C     ..
C
C
      NLINE_Segments = 1
      NROWS_BY_COLUMNS = M*N
      NROWS_BY_COLUMNS2 = 2*NROWS_BY_COLUMNS
      DIV = REAL(NLINE_Segments)
      NContour_Levels = 1
C
C---- loop contour levels CurrentContour
C
   10 CurrentCONTOUR = CONT(NContour_Levels)
      NContour_Levels = NContour_Levels + 1
C
C---- set up plotting style for this contour level
C
C          *****************************
      CALL contourstyle(CurrentCONTOUR)
C          *****************************
C
      DO 20 I = 1,NROWS_BY_COLUMNS2
        BITS(I) = .FALSE.
   20 CONTINUE
C
C
      NVCT(1) = 0
      NVCT(2) = NROWS_BY_COLUMNS
      NVCT(3) = M
      NVCT(4) = NROWS_BY_COLUMNS + 1
      KPOINT = 1
      MM = M - 1
      NN = N - 1
C
C---- search for contour crossing between adjacent
C     column of array farrya(i,j)
C
      I = 0
      J = 1
      ISUB = 0
      JSUB = 0
      IRTN = 1
   30 IF (J.GT.N) GO TO 90
   40 IF (I.GE.MM) GO TO 80
      I = I + 1
      ISUB = ISUB + 1
      JSUB = JSUB + 1
      IF (FARRAY(ISUB)-CurrentCONTOUR) 50,430,60
   50 IF (FARRAY(ISUB+1)-CurrentCONTOUR) 40,40,70
   60 IF (FARRAY(ISUB+1)-CurrentCONTOUR) 70,40,40
   70 IF (BITS(JSUB+NVCT(1))) GO TO 40
      XSTART = (CurrentCONTOUR-FARRAY(ISUB))/
     +         (FARRAY(ISUB+1)-FARRAY(ISUB))
      YSTART = 0
      GO TO 170
   80 I = 0
      ISUB = ISUB + 1
      JSUB = JSUB + 1
      J = J + 1
      GO TO 30
C
C---- search for contour crossing between adjacent rows
C     of array Farray(i,j)
C
   90 I = 0
      J = 1
      JSUB = 0
      ISUB = 0
      IRTN = 2
  100 IF (J.GT.NN) GO TO 160
  110 IF (I.GE.M) GO TO 150
      I = I + 1
      ISUB = ISUB + 1
      JSUB = JSUB + 1
      IF (FARRAY(ISUB)-CurrentCONTOUR) 120,110,130
  120 IF (FARRAY(ISUB+M)-CurrentCONTOUR) 110,110,140
  130 IF (FARRAY(ISUB+M)-CurrentCONTOUR) 140,110,110
  140 IF (BITS(JSUB+NVCT(2))) GO TO 110
      YSTART = (CurrentCONTOUR-FARRAY(ISUB))/
     +         (FARRAY(ISUB+M)-FARRAY(ISUB))
      XSTART = 0
      GO TO 170
  150 I = 0
      J = J + 1
      GO TO 100
  160 IF (NContour_Levels.GT.NCONT) RETURN
      GO TO 10
C
C----  begin following contour line...
C      save indicies for return to search
C
  170 ISAVE = I
      JSAVE = J
      ISUBSV = ISUB
      JSUBSV = JSUB
      XSAVE = XSTART
      YSAVE = YSTART
      X(1) = REAL(I-1) + XSTART
      Y(1) = REAL(J-1) + YSTART
      IENT = IRTN
      IRS = 0
      GO TO 220
C
C---- dump line and follow contour line on opposite
C     side of starting piont when used a second time
C     this entry returns to search
C
  180 IRS = 1
C
C                      *******************************
  190 IF (KPOINT.GT.1) CALL joincoordpairs(X,Y,KPOINT)
C                      *******************************
C
      KPOINT = 1
      I = ISAVE
      J = JSAVE
      ISUB = ISUBSV
      JSUB = JSUBSV
      XSTART = XSAVE
      YSTART = YSAVE
      X(1) = REAL(I-1) + XSTART
      Y(1) = REAL(J-1) + YSTART
      IF (IRS.NE.0) GO TO (40,110) IRTN
      IEXIT = IRTN
      IRS = 1
      GO TO 210
C
C---- return from following contour line through a cell
C
  200 IF (BITS(JSUB+NVCT(IEXIT))) GO TO 180
  210 I = IVCT(IEXIT) + I
      J = JVCT(IEXIT) + J
      JSUB = (J-1)*M + I
      ISUB = JSUB
      IENT = IDIR(IEXIT)
  220 BITS(JSUB+NVCT(IENT)) = .TRUE.
      IF (I.LT.1 .OR. I.GT.MM .OR. J.LT.1 .OR. J.GT.NN) GO TO 190
C
C----     find contour crossing in new cell
C
      IF (ISUB+1.GT.NROWS_BY_COLUMNS .OR.
     +    ISUB+M.GT.NROWS_BY_COLUMNS .OR.
     +    ISUB+1+M.GT.NROWS_BY_COLUMNS) GO TO 190
      C(1) = FARRAY(ISUB+1)
      C(2) = FARRAY(ISUB)
      C(3) = FARRAY(ISUB+M)
      C(4) = FARRAY(ISUB+1+M)
      JRTN = 1
      ICNT = 1
      JCNT = 1
C
C
      DO 270 IROUND = 1,4
        IF (IROUND.EQ.IENT) GO TO 270
        I1 = ISIDE(IROUND)
        I2 = ISIDE(IROUND+1)
        IF (C(I1)-CurrentCONTOUR) 230,260,240
  230   IF (C(I2)-CurrentCONTOUR) 270,270,250
  240   IF (C(I2)-CurrentCONTOUR) 250,270,270
  250   IEXIT = IROUND
        ICNT = ICNT + 1
        GO TO 270
  260   JEXIT = IROUND
        JCNT = JCNT + 1
  270 CONTINUE
C
C
      GO TO (280,290,500,190) JCNT
  280 GO TO (190,300,190,520) ICNT
  290 GO TO (510,300,190,190) ICNT
  300 GO TO (310,320,330,340) IENT
  310 GO TO (190,360,390,360) IEXIT
  320 GO TO (400,190,400,350) IEXIT
  330 GO TO (390,360,190,360) IEXIT
  340 GO TO (400,350,400,190) IEXIT
C
C---- follow contour line across a cell to a side
C
  350 XSTART = SIDE(IENT)
  360 XFIN = SIDE(IEXIT)
      XINC = (XFIN-XSTART)/DIV
      XBASE = REAL(I-1)
      YBASE = REAL(J-1)
      A1 = CurrentCONTOUR - C(2)
      A2 = C(1) - C(2)
      A3 = C(3) - C(2)
      A4 = C(2) - C(1) + C(4) - C(3)
C
C
      DO 380 INTERP = 1,NLINE_Segments
        XSTART = XSTART + XINC
        YSTART = (A1-A2*XSTART)/ (A4*XSTART+A3)
        IF (KPOINT.LT.IPTMAX) GO TO 370
C
C            **************************
        CALL joincoordpairs(X,Y,KPOINT)
C            **************************
C
        X(1) = X(KPOINT)
        Y(1) = Y(KPOINT)
        KPOINT = 1
  370   KPOINT = KPOINT + 1
        X(KPOINT) = XBASE + XSTART
        Y(KPOINT) = YBASE + YSTART
  380 CONTINUE
C
C
      GO TO (200,190,450,480) JRTN
  390 YSTART = SIDE(IENT)
C
C---- follow contour line across a cell to a top or bottom
C
  400 YFIN = SIDE(IEXIT)
      XBASE = REAL(I-1)
      YINC = (YFIN-YSTART)/DIV
      YBASE = REAL(J-1)
      A1 = CurrentCONTOUR - C(2)
      A2 = C(3) - C(2)
      A3 = C(1) - C(2)
      A4 = C(2) - C(1) + C(4) - C(3)
C
C
      DO 420 INTERP = 1,NLINE_Segments
        YSTART = YSTART + YINC
        XSTART = (A1-A2*YSTART)/ (A4*YSTART+A3)
        IF (KPOINT.LT.IPTMAX) GO TO 410
C
C            **************************
        CALL joincoordpairs(X,Y,KPOINT)
C            **************************
C
        X(1) = X(KPOINT)
        Y(1) = Y(KPOINT)
        KPOINT = 1
  410   KPOINT = KPOINT + 1
        X(KPOINT) = XBASE + XSTART
        Y(KPOINT) = YBASE + YSTART
  420 CONTINUE
C
C
      GO TO (200,190,450,480) JRTN
C
C---- follow contour line from corner to corner
C
  430 K1 = ISUB - M
      K2 = ISUB + 1 - M
      K3 = ISUB + 1
      K4 = ISUB + 1 + M
      K5 = ISUB + M
      K6 = ISUB - 1 + M
      K7 = ISUB - 1
      C1 = FARRAY(K1)
      C2 = FARRAY(K2)
      C3 = FARRAY(K3)
      C4 = FARRAY(K4)
      C5 = FARRAY(K5)
      C6 = FARRAY(K6)
      C7 = FARRAY(K7)
      C8 = FARRAY(ISUB)
      IF (ISUB.LT.1 .OR. ISUB.GT.NROWS_BY_COLUMNS) GO TO 490
      X(1) = REAL(I-1)
      Y(1) = REAL(J-1)
      IF (J.EQ.1 .OR. J.EQ.M) GO TO 440
      IF (K1.LT.1 .OR. K1.GT.NROWS_BY_COLUMNS) GO TO 440
      IF (K2.LT.1 .OR. K2.GT.NROWS_BY_COLUMNS) GO TO 440
      IF (K3.LT.1 .OR. K3.GT.NROWS_BY_COLUMNS) GO TO 440
      IF (K4.LT.1 .OR. K4.GT.NROWS_BY_COLUMNS) GO TO 440
      IF (K5.LT.1 .OR. K5.GT.NROWS_BY_COLUMNS) GO TO 440
      IF (C3.NE.CurrentCONTOUR) GO TO 440
C
C
      IF (C1.EQ.CurrentCONTOUR .AND. 
     +    C2.EQ.CurrentCONTOUR .AND.
     +    C4.EQ.CurrentCONTOUR .AND. 
     +    C5.EQ.CurrentCONTOUR) GO TO 440
      X(2) = X(1) + 1.0
      Y(2) = Y(1)
C
C          *********************
      CALL joincoordpairs(X,Y,2)
C          *********************
C
      GO TO 460
  440 IF (J.EQ.1) GO TO 460
      IF (K1.LT.1 .OR. K1.GT.NROWS_BY_COLUMNS) GO TO 460
      IF (K2.LT.1 .OR. K2.GT.NROWS_BY_COLUMNS) GO TO 460
      IF (K3.LT.1 .OR. K3.GT.NROWS_BY_COLUMNS) GO TO 460
      IF (C2.NE.CurrentCONTOUR) GO TO 460
      IF (C1.EQ.CurrentCONTOUR .OR. C3.EQ.CurrentCONTOUR) GO TO 460
C
C
      IF (C1.GT.CurrentCONTOUR .AND. 
     +    C3.GT.CurrentCONTOUR .OR.
     +    C1.LT.CurrentCONTOUR .AND. 
     +    C3.LT.CurrentCONTOUR) GO TO 460
      C(1) = C2
      C(2) = C1
      C(3) = C8
      C(4) = C3
      J = J - 1
      JRTN = 3
      IENT = 3
      IEXIT = 1
      GO TO 390
C
C                      *******************************
  450 IF (KPOINT.GT.1) CALL joincoordpairs(X,Y,KPOINT)
C                      *******************************
C
      KPOINT = 1
      J = J + 1
      X(1) = REAL(I-1)
      Y(1) = REAL(J-1)
  460 IF (J.EQ.M .OR. I.EQ.1) GO TO 470
      IF (K3.LT.1 .OR. K3.GT.NROWS_BY_COLUMNS) GO TO 470
      IF (K4.LT.1 .OR. K4.GT.NROWS_BY_COLUMNS) GO TO 470
      IF (K5.LT.1 .OR. K5.GT.NROWS_BY_COLUMNS) GO TO 470
      IF (K6.LT.1 .OR. K6.GT.NROWS_BY_COLUMNS) GO TO 470
      IF (K7.LT.1 .OR. K7.GT.NROWS_BY_COLUMNS) GO TO 470
      IF (C5.NE.CurrentCONTOUR) GO TO 470
C
C
      IF (C3.EQ.CurrentCONTOUR .AND. 
     +    C4.EQ.CurrentCONTOUR .AND.
     +    C6.EQ.CurrentCONTOUR .AND. 
     +    C7.EQ.CurrentCONTOUR) GO TO 470
      X(2) = X(1)
      Y(2) = Y(1) + 1.0
C
C          *********************
      CALL joincoordpairs(X,Y,2)
C          *********************
C
      GO TO 490
  470 IF (J.EQ.M) GO TO 490
      IF (K3.LT.1 .OR. K3.GT.NROWS_BY_COLUMNS) GO TO 490
      IF (K4.LT.1 .OR. K4.GT.NROWS_BY_COLUMNS) GO TO 490
      IF (K5.LT.1 .OR. K5.GT.NROWS_BY_COLUMNS) GO TO 490
      IF (C4.NE.CurrentCONTOUR) GO TO 490
      IF (C3.EQ.CurrentCONTOUR .OR. C5.EQ.CurrentCONTOUR) GO TO 490
C
C
      IF (C3.GT.CurrentCONTOUR .AND. 
     +    C5.GT.CurrentCONTOUR .OR.
     +    C3.LT.CurrentCONTOUR .AND. 
     +    C5.LT.CurrentCONTOUR) GO TO 490
      C(1) = C3
      C(2) = C8
      C(3) = C5
      C(4) = C4
      JRTN = 4
      IENT = 1
      IEXIT = 3
      GO TO 390
C
C                      *******************************
  480 IF (KPOINT.GT.1) CALL joincoordpairs(X,Y,KPOINT)
C                      *******************************
C
      KPOINT = 1
      X(1) = REAL(I-1)
      Y(1) = REAL(J-1)
  490 GO TO (40,110) IRTN
C
C----  follow contour line from side to corner or corners
C
  500 JRTN = 2
      IOPP = IDIR(IENT)
      I1 = ISIDE(IOPP)
      I2 = ISIDE(IOPP+1)
      IEXIT = IOPP
      C(I1) = C(KVCT(I1))
      C(I2) = C(LVCT(I2))
      GO TO 300
  510 JRTN = 2
      IEXIT = JEXIT
      GO TO 300
C
C----     follow contour line through saddle point
C
  520 IOPP = IDIR(IENT)
      I1 = ISIDE(IENT)
      C1 = C(I1)
      I2 = ISIDE(IENT+1)
      C2 = C(I2)
      I3 = ISIDE(IOPP)
      C3 = C(I3)
      I4 = ISIDE(IOPP+1)
      C4 = C(I4)
C
C
      IF ((C1-CurrentCONTOUR)/ (C1-C2).EQ.
     +    (C4-CurrentCONTOUR)/ (C4-C3)) GO TO 540
      IF ((C1-CurrentCONTOUR)/ (C1-C4).GT.
     +    (C2-CurrentCONTOUR)/ (C2-C3)) GO TO 530
      IEXIT = I4
      GO TO 300
  530 IEXIT = I2
      GO TO 300
  540 C(I3) = C(I2)
      C(I4) = C(I1)
      IEXIT = I3
      GO TO 300
      END
C
C
C
C     ==============================================================
      SUBROUTINE geteigenvalues(VALUE_ON_ENTRY,RTN_ARRAY,VECTORARRAY,
     +                          IERROR)
C     ==============================================================
C
C       eigenvalues and eigenvectors of 3x3 matrix
C
C
C
C
C
C     .. Scalar Arguments ..
      INTEGER IERROR
C     ..
C     .. Array Arguments ..
      REAL RTN_ARRAY(3),VALUE_ON_ENTRY(3,3),VECTORARRAY(3,3)
C     ..
C     .. Local Scalars ..
      REAL B0,B1,B2,C0,C1,ERRND,SIGMA,SMAX,T1,VNEW,VOLD,VTEMP,X,Y,Z
      INTEGER I,I1,II,III,IMAX,J,L,LLL
C     ..
C     .. Local Arrays ..
      REAL A(3,3),B(3,3),U(3),V(3),Vin(3,1),Vout(3,1)
C     ..
C     .. External Functions ..
      REAL vectbymatrxbyvector
      EXTERNAL vectbymatrxbyvector
C     ..
C     .. External Subroutines ..
      EXTERNAL getunitvectors,matrixsolve
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MAX,SQRT
C     ..
C     .. Statement Functions ..
      REAL PHIF
C     ..
C     .. Statement Function definitions ..
      PHIF(Z) = ((B2-Z)*Z+B1)*Z + B0
C     ..
C
C
      ERRND = 5.0E-7
      SIGMA = 0.0
C
C
      DO 20 J = 1,3
        DO 10 I = 1,3
          VTEMP = VALUE_ON_ENTRY(I,J)
          A(I,J) = VTEMP
          SIGMA = VTEMP*VTEMP + SIGMA
   10   CONTINUE
          Vin(J,1) = 0.0
          Vout(J,1) = 0.0
   20 CONTINUE
C
C---- check for null matrix
C
      IF (SIGMA.LE.0.0) THEN
        WRITE (6,FMT=6000)
 6000   FORMAT (' EIGEN Error null matrix')
        IERROR = 6
        RETURN
      END IF
C
C
      SIGMA = SQRT(SIGMA)
C
C---- form characteristic equation
C
      B2 = A(1,1) + A(2,2) + A(3,3)
C
C
      B1 = -A(1,1)*A(2,2) - A(1,1)*A(3,3) - A(2,2)*A(3,3) +
     +     A(1,3)*A(3,1) + A(2,3)*A(3,2) + A(1,2)*A(2,1)
C
C
      B0 = A(1,1)*A(2,2)*A(3,3) + A(1,2)*A(2,3)*A(3,1) +
     +     A(1,3)*A(3,2)*A(2,1) - A(1,3)*A(3,1)*A(2,2) -
     +     A(1,1)*A(2,3)*A(3,2) - A(1,2)*A(2,1)*A(3,3)
C
C---- first root by bisection
C
      X = 0.0
      Y = SIGMA
C
C             ***********
      VTEMP = PHIF(SIGMA)
C             ***********
C
      VNEW = 0.0
C
C
      IF (B0.EQ.0.0) GO TO 50
C
C
      IF (B0.LE.0.0) THEN
        IF (VTEMP.LE.0.0) THEN
          Y = -Y
          GO TO 30
        ELSE
          GO TO 30
        END IF
      ELSE
        Y = 0.0
        X = SIGMA
        IF (VTEMP.LE.0.0) GO TO 30
        X = -X
      END IF
C
C---- now phif(x).lt.0.and.phif(y).gt.0.
C
   30 VNEW = (X+Y)*0.5
C
C
      DO 40 I = 1,40
C
C           **********
        IF (PHIF(VNEW).EQ.0.0) THEN
C           **********
C
          GO TO 50
C
C                **********
        ELSE IF (PHIF(VNEW).LT.0.0) THEN
C                **********
C
          X = VNEW
        ELSE
          Y = VNEW
        END IF
C
C
        VOLD = VNEW
        VNEW = (X+Y)*0.5
        VTEMP = ABS(VOLD-VNEW)
        IF ((VTEMP-ERRND).LE.0.0) GO TO 50
C
C
        IF (VOLD.NE.0.0) THEN
          IF ((ABS(VTEMP/VOLD)-ERRND).LE.0.0) GO TO 50
        END IF
C
C
   40 CONTINUE
C
C---- did not converge, set error indicator
C
      IERROR = 6
      WRITE (6,FMT=6002)
 6002 FORMAT ('  EIGEN no convergence')
      RETURN
C
C---- store first root
C
   50 U(3) = VNEW
C
C---- deflate
C
      C1 = B2 - VNEW
      C0 = C1*VNEW + B1
C
C---- solve quadratic
C
      VTEMP = C1*C1 + 4.0*C0
C
C
      IF (VTEMP.LT.0.0) THEN
C
C---- ignore imaginary component of complex root
C
        VTEMP = 0.0
      ELSE IF (VTEMP.GT.0.0) THEN
        VTEMP = SQRT(VTEMP)
      END IF
C
C
      U(1) = (C1-VTEMP)*0.5
      U(2) = (C1+VTEMP)*0.5
C
C---- sort roots
C
      DO 60 J = 1,2
        IF ((U(J)-U(3)).LE.0.0) GO TO 60
        VTEMP = U(J)
        U(J) = U(3)
        U(3) = VTEMP
   60 CONTINUE
C
C
      LLL = -2
C
C
      DO 160 III = 1,2
C
C---- check for multiple roots
C
        VTEMP = ERRND*100.0
        L = 1
C
C
        DO 80 I = 1,2
          IF ((U(I+1)-U(I)-VTEMP).LE.0.0) GO TO 70
          IF (U(I).EQ.0.0) GO TO 80
          IF ((ABS((U(I+1)-U(I))/U(I))-VTEMP).GT.0.0) GO TO 80
   70     L = L - 1
          IERROR = IERROR - 2*I
   80   CONTINUE
C
C
        IF ((LLL-L).GE.0) RETURN
        LLL = L
C
C---- eigenvector routine
C
        DO 150 II = 1,3
          T1 = U(II)
C
C
          IF (L.EQ.0) THEN
C
C---- two vectors null for double root
C
            IF ((IERROR+5-II).EQ.0) GO TO 100
          ELSE IF (L.GT.0) THEN
            GO TO 100
          END IF
C
C---- all vectors null for triple root
C
          DO 90 J = 1,3
            VECTORARRAY(J,II) = 0.0
   90     CONTINUE
C
C
          GO TO 150
  100     CONTINUE
C
C
          DO 110 J = 1,3
            A(J,J) = VALUE_ON_ENTRY(J,J) - T1
  110     CONTINUE
C
C
          SMAX = 0.0
C
C
          DO 120 I = 1,3
            I1 = 1
            IF ((I-2).LE.0) I1 = I + 1
C
C
            B(I,1) = A(I,2)*A(I1,3) - A(I,3)*A(I1,2)
            B(I,2) = A(I,3)*A(I1,1) - A(I,1)*A(I1,3)
            B(I,3) = A(I,1)*A(I1,2) - A(I,2)*A(I1,1)
C
C
            VTEMP = B(I,1)**2 + B(I,2)**2 + B(I,3)**2
            IF ((VTEMP-SMAX).LE.0.0) GO TO 120
            SMAX = VTEMP
            IMAX = I
  120     CONTINUE
C
C
          IF (SMAX.LE.0.0) THEN
            IERROR = 7
          ELSE
            SMAX = SQRT(SMAX)
C
C
            DO 130 J = 1,3
              V(J) = B(IMAX,J)/SMAX
              Vin(J,1) = V(J)
  130       CONTINUE
C
C---- refine eigenvector
C
C                *************************
            CALL matrixsolve(A,Vout,Vin,1)
C                *************************
C
            VTEMP = MAX(ABS(V(1)),ABS(V(2)),ABS(V(3)))
C
C
            DO 140 J = 1,3
              V(J) = Vout(J,1)/VTEMP
  140       CONTINUE
C
C                *************************************
            CALL getunitvectors(V,VECTORARRAY(1,II),1)
C                *************************************
C
C---- refine eigenvalue
C
C                **************************************
            T1 = vectbymatrxbyvector(VECTORARRAY(1,II),VALUE_ON_ENTRY,
     +           VECTORARRAY(1,II))
C                **************************************
C
            U(II) = T1
          END IF
C
C
          RTN_ARRAY(II) = T1
  150   CONTINUE
  160 CONTINUE
C
C
      RETURN
      END
C
C
C
C     =====================================
      SUBROUTINE gethbondconnections(KBTYP)
C     =====================================
C
C
C
C
C---- set up connection list for h-bonds,
C     excluding mainchain-mainchain bonds
C     within residues or adjacent residues,
C     and sidechain-sidechain bond within
C     same residue.
C
C    must be called before getatomconnections or dscan  !!!!
C
C Lprint_Flag.ne.0 print bonds
C KBTYP bond types for m/c-m/c & other bonds
C       =-1 for no bonds of this type
C
C IN COMMON /com_Atoms/
C    Orthogonal_XYZ(i,3)  orthogonal coordinates
C   Atom_Radii link radii
C
c      INTEGER NPackAtmID
c      PARAMETER (NPackAtmID = 2300000)
C
C   LAtom_Inc_Flag=
C      ISideChain_Code+2+isym*10+jcolr*1000+nres*10000+jexcl*NPackAtmID
C        ISideChain_Code=-1 ca, 0 mainchain, 1 sidechain
C        nres  residue number
C
C
C on exit: IAtom_Connectivity=NpackAtmConn*jtyp+i*Max_Atoms+j
C     for bond of type jtyp between atoms i&j
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Labels
      PARAMETER (Max_Labels=100)
      INTEGER NpackAtmConn
      PARAMETER (NpackAtmConn=5000000)
      INTEGER NPackAtmID
      PARAMETER (NPackAtmID=10000000)
C     ..
C     .. Array Arguments ..
      INTEGER KBTYP(2)
C     ..
C     .. Scalars in Common ..
      INTEGER IERR,Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1
      CHARACTER Iden_Title*80
C     ..
C     .. Arrays in Common ..
      REAL Atom_Radii,Fractional_XYZ,Orthogonal_XYZ
      INTEGER IAtom_Connectivity,IAtom_Type,LAtom_Inc_Flag,
     +        LAtom_Label_Flag
      CHARACTER Atm_Lab_Type*2,Atm_Identifier*10,Atm_Label*10,
     +          Atm_Lab_Txt*10
C     ..
C     .. Local Scalars ..
      REAL D,D2,R,RI
      INTEGER I,I2,IN,IRES,ISD,ISYM,J,J1,JCOLR,JRES,JSD,JSYM,JTYP,K
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MOD,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_LABCHR/Atm_Lab_Txt(Max_Labels),
     +       Atm_Lab_Type(Max_Labels),Atm_Label(Max_Labels),
     +       Atm_Identifier(Max_Atoms),Iden_Title
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      WRITE (6,FMT=6000) KBTYP
 6000 FORMAT (' H-bond search, bond types for mainchain-mainchain bond',
     +       's,and sidechain bonds ',2I8)
C
C---- outer loop for atom i
C
      I2 = NAT - 1
C
C
      DO 50 I = 8,I2
        RI = Atom_Radii(I)
        IF (RI.LT.0.00001) GO TO 50
C
C---- unpack sidechain code,symmetry & residue number
C
        IN = ABS(LAtom_Inc_Flag(I))
        IF (IN.EQ.0) GO TO 50
        IN = MOD(IN,NPackAtmID)
        IRES = IN/10000
        IN = IN - IRES*10000
        JCOLR = IN/1000
        IN = IN - JCOLR*1000
        ISYM = IN/10
        ISD = IN - ISYM*10 - 2
C
C---- kbtyp(2)=-1, mainchain-mainchain bonds only
C
        IF (KBTYP(2).LT.0 .AND. ISD.GT.0) GO TO 50
C
C---- inner loop for atom j
C
        J1 = I + 1
        DO 40 J = J1,NAT
          IF (Atom_Radii(J).LT.0.00001) GO TO 40
C
C---- sidechain code & residue number
C
          IN = ABS(LAtom_Inc_Flag(J))
          IF (IN.EQ.0) GO TO 40
          IN = MOD(IN,NPackAtmID)
          JRES = IN/10000
          IN = IN - JRES*10000
          JCOLR = IN/1000
          IN = IN - JCOLR*1000
          JSYM = IN/10
          JSD = IN - JSYM*10 - 2
          JTYP = 2
C
C---- exclude m/c-m/c bonds between same or adjacent residues
C
          IF (ISYM.NE.JSYM) GO TO 20
          IF (ISD.GT.0) GO TO 10
          IF (JSD.GT.0) GO TO 20
          IF (ABS(IRES-JRES).LE.1) GO TO 40
          IF (KBTYP(1).LT.0) GO TO 40
          JTYP = 1
          GO TO 20
   10     IF (JSD.LE.0) GO TO 20
C
C---- s/c-s/c , exclude if same residue
C
          IF (IRES.EQ.JRES) GO TO 40
   20     R = Atom_Radii(J) + RI
          D2 = 0.0
C
C
          DO 30 K = 1,3
            D = Orthogonal_XYZ(I,K) - Orthogonal_XYZ(J,K)
            IF (ABS(D).GT.R) GO TO 40
            D2 = D*D + D2
   30     CONTINUE
C
C
          IF (D2.GT.R*R) GO TO 40
C
C---- we have a bond, jtyp gives type =1 m/c-m/c, =2 other
C
          NC = NC + 1
          IAtom_Connectivity(NC) = KBTYP(JTYP)*NpackAtmConn +
     +                             I*Max_Atoms + J
          D = SQRT(D2)
          WRITE (6,FMT=6002) I,Atm_Identifier(I),J,Atm_Identifier(J),D,
     +      KBTYP(JTYP)
 6002     FORMAT (' H-bond between atoms ',I6,3X,A,'  and',I6,3X,A,'  ',
     +           ' Length',F10.3,'   Type',I4)
   40   CONTINUE
C
   50 CONTINUE
      RETURN
      END
C
C
C
C     =========================================================
      SUBROUTINE getmapsection(MAP,ISEC,RHO,NUMMAP_POINTS,IEND)
C     ========================================================
C
C---- find & read section isec of map map
C     compress map section if limits given
C       returns iend=1 if end of file found
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
C     ..
C     .. Scalar Arguments ..
      INTEGER IEND,ISEC,MAP,NUMMAP_POINTS
C     ..
C     .. Array Arguments ..
      REAL RHO(NUMMAP_POINTS)
C     ..
C     .. Scalars in Common ..
      INTEGER Invert_Map_Flag,KSEC1,KSEC2,LimitsSet_Flag,Lmap3d_Flag,
     +        Lmap_ModeFirst,LSetView_Flag,Map_Counter,Map_Flag_Counter,
     +        Map_pts_U,Map_pts_V,Msecst,MU,MV,MW,NFirst_Section,
     +        NLast_Section,Npicture,Npts_U_compressed,
     +        Npts_V_compressed,Nsections_in_Group,NstpMapGrpSect
C     ..
C     .. Arrays in Common ..
      REAL AQ,Contour_Levels,Contour_Threshold
      INTEGER Map_def_Restrictions,Map_LineTypes,Map_UVW_Limits,
     +        Map_XYZ_Limits,NSECC,Num_Contours
      CHARACTER Map_Title*80
C     ..
C     .. Local Scalars ..
      INTEGER I,IER,IST,J,K,NB,LPERM
C     ..
C     .. External Subroutines ..
      EXTERNAL MGULPR,CCPMVI,MPOSN
C     ..
C     .. Common blocks ..
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
      COMMON /com_SECTIONS/NFirst_Section,NLast_Section,
     +       Nsections_in_Group,NstpMapGrpSect,KSEC1,KSEC2,Msecst,
     +       NSECC(2)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      IEND = 0
      IST = MAP + 2
      IER = 0
C
C---- find section
C
C          ********************
      CALL MPOSN(IST,ISEC)
      CALL MGULPR(IST,RHO,IER)
C          ********************
C
      IF (IER.NE.0) GO TO 20
      NSECC(MAP) = ISEC + 1
C
C---- Compress map section if required
C
      IF (LimitsSet_Flag.EQ.0) RETURN
C
C---- Start address in RHO array of 1st point required
C
      J = (Map_def_Restrictions(3)-Map_UVW_Limits(3))*Map_pts_U +
     +    Map_def_Restrictions(1) - Map_UVW_Limits(1) + 1
C
C---- 1st address to copy to
C
      K = 1
C
C---- Number of bytes (NB!!!!) to move
C
Corig      NB = Npts_U_compressed*4
      NB = Npts_U_compressed
C
C---- Loop lines of map
C
      DO 10 I = 1,Npts_V_compressed
C
C---- Fast move from RHO(J) to RHO(K)
C
C            ****************************
         call ccpmvi(rho(k),rho(j),nb)
C            ****************************
C
C---- Increment addresses
C
        J = J + Map_pts_U
        K = K + Npts_U_compressed
   10 CONTINUE
C
C
      RETURN
C
C
   20 IEND = 1
      WRITE (6,FMT=6000) ISEC,MAP
 6000 FORMAT (' End of map file found when searching for section',I5,
     +       ' on map',I5,' , plotting continues')
      REWIND IST
      RETURN
      END
C
C
C
C     =========================
      SUBROUTINE getplotxy(X,Y)
C     =========================
C
C---- plot coord. and closest edge after projection
C
C
C
C
C
C
C          #############################
C
C     .. Array Arguments ..
      REAL X(3),Y(2)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,CurrentCharcter_Size,
     +     Dist_to_Boundary,Ellipse_ScaleFact,Eye_Distance,
     +     Overlap_Margin,Paper_Length,Paper_Width,Plotter_Step,
     +     Retrace_Displmnt,Scal_pre_Proj,SCL
      INTEGER Iuse_persp_Labs,Lpack_Flag,NDrawing_Number
      LOGICAL UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL Drawing_Origin,Ellipse_Resolution,Title_Rotation,XO
C     ..
C     .. Local Scalars ..
      REAL T1,T2,T4
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MIN
C     ..
C     .. Common blocks ..
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      T4 = 1.0
      T1 = 1.0
C
C
      IF (Eye_Distance.GT.0.0) THEN
        T4 = Eye_Distance - X(3)
C
C
        IF (T4.LE.0.0) THEN
          Y(1) = -99.0
          Y(2) = -99.0
          Dist_to_Boundary = -99.0
          RETURN
        ELSE
          T1 = Eye_Distance/T4
        END IF
      END IF
C
C
ccx       Y(1) = X(1)*T1 + XO(1)
ccx      Y(2) = X(2)*T1 + XO(2)
      Y(1) = X(1)*T1
      Y(2) = X(2)*T1
      T1 = Paper_Width - ABS(Y(1)*2.0-Paper_Width)
      T2 = Paper_Length - ABS(Y(2)*2.0-Paper_Length)
      Dist_to_Boundary = MIN(T1,T2)*0.5
      IF ((T4-Eye_Distance*0.5).LT.0.0) Dist_to_Boundary = -99.0
      RETURN
      END
C
C
C
C     =======================================================
      SUBROUTINE getqmatrix(ITYPE,CONTRAEIGVECTS,KSYMNUM,KAT)
C     =======================================================
C
C---- itype .lt.0 for covariance matrix in q
C     itype .gt.0 for ellipsoid quadratic form in q
C     xabsf(itype)=1 based on triclinic coordinate system
C     =2 or 3 for working or reference cartesian systems
C     contravariant eigenvectors for q in columns of
C     Prin_Axis_Col_Vects
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Symmetry
      PARAMETER (Max_Symmetry=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER ITYPE,KAT,KSYMNUM
C     ..
C     .. Array Arguments ..
      REAL CONTRAEIGVECTS(3,3)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,CurrentCharcter_Size,
     +     D001,D010,D100,Dist_to_Boundary,Ellipse_ScaleFact,
     +     Eye_Distance,RFR(4,4),ROR(4,4),
     +     Overlap_Margin,Paper_Length,
     +     Paper_Width,Plotter_Step,Retrace_Displmnt,Scal_pre_Proj,SCL
      INTEGER Iuse_persp_Labs,Lorthogonalisation_Code,Lpack_Flag,
     +        NDrawing_Number,NSymmetry_Ops
      LOGICAL UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL AID,ATM_AXIS_RMS,Base_Vect_Refsys,Base_Vect_Wrksys,BBB,
     +     Drawing_Origin,Ellipse_Resolution,G_Metric_Tensor,OrtCell,
     +     OrtConts,P1_to_Refsys,P1_to_Wrksys,Principal_axis_rms,
     +     Prin_Angstrom_Vectors,Prin_Axis_Col_Vects,Q,
     +     Recip_Metric_Tensor,Rotsym_Matrx,Sorted_xyz,Temp_Factors,
     +     Title_Rotation,Title_RotnTrans,TransSym_Vect,XABC,XCLANG,XO
      INTEGER KSortPointer
C     ..
C     .. Local Scalars ..
      REAL T1
      INTEGER I,IT,J,K
C     ..
C     .. Local Arrays ..
      REAL XOR(3)
C     ..
C     .. External Subroutines ..
      EXTERNAL matrxbymatrx,transposematrix
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS
C     ..
C     .. Common blocks ..
      COMMON /com_BFAC/Temp_Factors(3,3,Max_Atoms),
     +       ATM_AXIS_RMS(3,Max_Atoms)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_SYMMETRY/NSymmetry_Ops,Rotsym_Matrx(3,3,Max_Symmetry),
     +       TransSym_Vect(3,Max_Symmetry)
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      IT = ABS(ITYPE) - 1
C
C---- crystallographic symmetry rotation
C
C          ****************************************
      CALL transposematrix(Temp_Factors(1,1,KAT),
     +                     Rotsym_Matrx(1,1,KSYMNUM),
     +                     Prin_Angstrom_Vectors)
C          ****************************************
C
      IF ((IT-1).EQ.0) THEN
C
C---- transform to cartesian systems
C
C            **************************************************
        CALL transposematrix(Prin_Angstrom_Vectors,P1_to_Wrksys,
     +                       Prin_Axis_Col_Vects)
C            **************************************************
C
        GO TO 20
      ELSE IF ((IT-1).GT.0) THEN
C
C            **************************************************
        CALL transposematrix(Prin_Angstrom_Vectors,P1_to_Refsys,
     +                       Prin_Axis_Col_Vects)
C            **************************************************
C
        GO TO 20
      END IF
C
C
      IF (ITYPE.EQ.0) THEN
C
C            **************************************************
        CALL transposematrix(Prin_Angstrom_Vectors,P1_to_Refsys,
     +                       Prin_Axis_Col_Vects)
C            **************************************************
C
        GO TO 20
      ELSE IF (ITYPE.LT.0) THEN
C
C---- transform to triclinic system
Cccx 1-9 xxxx???
        DO 10 J = 1,9
          Prin_Axis_Col_Vects(J,1) = Prin_Angstrom_Vectors(J,1)
   10   CONTINUE
C
C
        GO TO 20
      ELSE
C
C            **************************************************
        CALL matrxbymatrx(G_Metric_Tensor,Prin_Angstrom_Vectors,
     +                    Prin_Axis_Col_Vects)
C            **************************************************
C
      END IF
C
C---- form diagonal matrix or its inverse
C
   20 CONTINUE
C
C
      DO 30 J = 1,3
        T1 = ATM_AXIS_RMS(J,KAT)
C
C
        IF (ITYPE.GT.0) THEN
          XOR(J) = 1.0/ (T1*T1)
        ELSE
          XOR(J) = T1*T1
        END IF
C
C
        Principal_axis_rms(J) = T1
   30 CONTINUE
C
C---- form quadratic form
C
      DO 60 I = 1,3
        DO 50 J = I,3
          T1 = 0.0
          DO 40 K = 1,3
            T1 = Prin_Axis_Col_Vects(I,K)*Prin_Axis_Col_Vects(J,K)*
     +           XOR(K) + T1
   40     CONTINUE
C
C
          CONTRAEIGVECTS(J,I) = T1
          CONTRAEIGVECTS(I,J) = T1
   50   CONTINUE
   60 CONTINUE
C
C
      RETURN
      END
C
C
C
C     ===================================
      SUBROUTINE getunitvectors(X,Z,ITYPE)
C     ====================================
C
C
C
C
C---- Unit vectors are subset of base vectors - in orthogonal axes
C          A = xi + yj + zk
C           |A|**2 = x^2 + y^2 + z^2
C
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
C     ..
C     .. Scalar Arguments ..
      INTEGER ITYPE
C     ..
C     .. Array Arguments ..
      REAL X(3),Z(3)
C     ..
C     .. Arrays in Common ..
      REAL Base_Vect_Refsys,Base_Vect_Wrksys,BBB,G_Metric_Tensor,
     +     P1_to_Refsys,P1_to_Wrksys,Principal_axis_rms,
     +     Prin_Angstrom_Vectors,Prin_Axis_Col_Vects,Q,
     +     Recip_Metric_Tensor,Sorted_xyz,Title_RotnTrans
      INTEGER KSortPointer
C     ..
C     .. Local Scalars ..
      REAL T1
C     ..
C     .. Local Arrays ..
      REAL Y(3)
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPERR
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC SQRT
C     ..
C     .. Common blocks ..
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      Y(1) = X(1)
      Y(2) = X(2)
      Y(3) = X(3)
C
C
      IF (ITYPE.GT.0) THEN
        T1 = SQRT(Y(1)*Y(1)+Y(2)*Y(2)+Y(3)*Y(3))
      ELSE
        T1 = SQRT(((G_Metric_Tensor(1,2)+G_Metric_Tensor(2,
     +       1))*Y(2)+Y(1)*G_Metric_Tensor(1,1)+ (G_Metric_Tensor(1,
     +       3)+G_Metric_Tensor(3,1))*Y(3))*Y(1)+
     +       ((G_Metric_Tensor(2,3)+G_Metric_Tensor(3,
     +       2))*Y(3)+Y(2)*G_Metric_Tensor(2,2))*Y(2)+
     +       Y(3)*Y(3)*G_Metric_Tensor(3,3))
      END IF
C
C                    ***********************************************
      IF (T1.EQ.0.0) CALL CCPERR(1,' Error in getunitvectors le. 0')
C                    ***********************************************
C
      Z(1) = Y(1)/T1
      Z(2) = Y(2)/T1
      Z(3) = Y(3)/T1
      RETURN
      END
C
C
C
C     ==================================
      SUBROUTINE joincoordpairs(X,Y,IPT)
C     ==================================
C
C---  join ipt coordinate pairs x,y of contour crossings,
C     in grid units for m*n grid ,
C     coordinates are in range 0,0 to m,n
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
C     ..
C     .. Scalar Arguments ..
      INTEGER IPT
C     ..
C     .. Array Arguments ..
      REAL X(IPT),Y(IPT)
C     ..
C     .. Scalars in Common ..
      REAL Charcter_Size,DEY,DEYC,DPL,Drawing_Size,
     +     Perspective_Distance,Picture_Scale,QPERS,QX,QY,S,
     +     Shadelight_X,Shadelight_Y,Stereo_Separation,U0,UN,V0,VN,W,W0,
     +     WN,XC,YC,ZMAX,ZPL
      INTEGER Invert_Map_Flag,Lcurrent_Colour,Ldraw_cell_Flag,
     +        LimitsSet_Flag,Lmap3d_Flag,Lmap_ModeFirst,Lpaper_Setting,
     +        Lperspective_Flag,Lplot_colour_Stereo,Lposition_Flag,
     +        Lprint_Flag,LSetView_Flag,LShade_Flag,LSolid_Flag,
     +        LStereo_Flag,LSuperimpose_Flag,LTaper_Flag,LTrace_Flag,
     +        Map_Counter,Map_Flag_Counter,Map_pts_U,Map_pts_V,MU,MV,MW,
     +        Natoms_plusCell,Ncopies,Npicture,Npts_U_compressed,
     +        Npts_V_compressed,Nspecified_Bonds,Num_Moles
      LOGICAL Log_Draw_Title
C     ..
C     .. Arrays in Common ..
      REAL AQ,Contour_Levels,Contour_Threshold,Plotting_XYZ,RP,
     +     ViewMatrix
      INTEGER ISpecified_Bond_Params,Map_def_Restrictions,Map_LineTypes,
     +        Map_UVW_Limits,Map_XYZ_Limits,MOL,Num_Contours
      CHARACTER Map_Title*80
C     ..
C     .. Local Scalars ..
      REAL AQ11,AQ12,AQ13,AQ21,AQ22,AQ23,AQ31,AQ32,AQ33,U,V,X1,XL,XXP,
     +     Y1,YL,YYP,ZZP
      INTEGER I,LPERM
C     ..
C     .. External Subroutines ..
      EXTERNAL drawlineset
C     ..
C     .. Common blocks ..
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_CONTOURS/UN,VN,WN,U0,V0,W0,W,ZPL,QPERS,DPL,DEY,DEYC,
     +       QX,QY,XC,YC,ZMAX
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      AQ11 = AQ(1,1)
      AQ21 = AQ(2,1)
      AQ31 = AQ(3,1)
      AQ12 = AQ(1,2)
      AQ22 = AQ(2,2)
      AQ32 = AQ(3,2)
      AQ13 = AQ(1,3)
      AQ23 = AQ(2,3)
      AQ33 = AQ(3,3)
C
C---- save 1st and last points before transformation
C
      X1 = X(1)
      Y1 = Y(1)
      XL = X(IPT)
      YL = Y(IPT)
C
C---- transform coordinates
C
      DO 30 I = 1,IPT
C
C---- fractional coordinates
C
        U = (X(I)+U0)/UN
        V = (Y(I)+V0)/VN
C
C---- plotter coordinates
C
        XXP = AQ11*U + AQ12*V + AQ13*W
        YYP = AQ21*U + AQ22*V + AQ23*W
        ZZP = AQ31*U + AQ32*V + AQ33*W
C
C---- shift to centre
C
        XXP = (XXP-XC)*S
        YYP = (YYP-YC)*S
        ZZP = (ZZP-ZMAX)*S
        IF (LStereo_Flag.GT.0) GO TO 10
C
C---- mono
C
        IF (Lperspective_Flag.EQ.0) GO TO 20
C
C---- perspective
C
        QPERS = DPL/ (DPL-ZZP)
        XXP = XXP*QPERS
        YYP = YYP*QPERS
        GO TO 20
C
C--- stereo
C
   10   QPERS = DPL/ (DPL-ZZP)
        XXP = (XXP+DEYC)*QPERS - DEYC
        YYP = YYP*QPERS
   20   X(I) = XXP + QX
        Y(I) = YYP + QY
   30 CONTINUE
C
C---- draw lines
C
C          ********************
      CALL drawlineset(X,Y,IPT)
C          ********************
C
C---- restore 1st & last points
C
      X(1) = X1
      Y(1) = Y1
      X(IPT) = XL
      Y(IPT) = YL
      RETURN
      END
C
C
C
C     ===============================
      SUBROUTINE matrixmultply(A,B,C)
C     ===============================
C
C     matrix multiply a=b*c
C
C     .. Array Arguments ..
      REAL A(3,3),B(3,3),C(3,3)
C     ..
C     .. Local Scalars ..
      REAL D
      INTEGER I,J,K
C     ..
C
C
      DO 30 I = 1,3
        DO 20 J = 1,3
          D = 0.0
          DO 10 K = 1,3
            D = B(I,K)*C(K,J) + D
   10     CONTINUE
          A(I,J) = D
   20   CONTINUE
   30 CONTINUE
C
C
      RETURN
      END
C
C
C
C     ===================================
      SUBROUTINE matrixsolve(A1,X,B1,JJJ)
C     ===================================
C
C----  Solution of matrix equation ax=b for x
C       uses method of triangular elimination
C       b and x have dimensions (3,jjj),a is always (3,3)
C       to invert a make b 3 by 3 idenity matrix
C
C
C     .. Scalar Arguments ..
      INTEGER JJJ
C     ..
C     .. Array Arguments ..
      REAL A1(3,3),B1(3,JJJ),X(3,JJJ)
C     ..
C     .. Local Scalars ..
      REAL R,S,TEM
      INTEGER I,IPO,J,K,L,M,N,NV
C     ..
C     .. Local Arrays ..
      REAL A(3,3),B(3,3)
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MAX
C     ..
C
C
      NV = JJJ
C
C---- transfer data
C
      DO 20 I = 1,3
        DO 10 J = 1,3
          IF ((NV-J).GE.0) B(I,J) = B1(I,J)
          A(I,J) = A1(I,J)
   10   CONTINUE
   20 CONTINUE
C
C---- triangularize matrix a
C
      DO 100 I = 1,2
        S = 0.0
C
C
        DO 30 J = I,3
          R = ABS(A(J,I))
C
C
          IF ((R-S).GE.0.0) THEN
            S = R
            L = J
          END IF
C
C
   30   CONTINUE
C
C
        IF ((L-I).EQ.0) GO TO 60
C
C
        DO 40 J = I,3
          S = A(I,J)
          A(I,J) = A(L,J)
          A(L,J) = S
   40   CONTINUE
C
C
        DO 50 J = 1,NV
          S = B(I,J)
          B(I,J) = B(L,J)
          B(L,J) = S
   50   CONTINUE
C
C
   60   TEM = A(I,I)
        IF (TEM.EQ.0.0) GO TO 100
        IPO = I + 1
C
C
        DO 90 J = IPO,3
          IF (A(J,I).EQ.0.0) GO TO 90
          S = A(J,I)/TEM
          A(J,I) = 0.0
C
C
          DO 70 K = IPO,3
            A(J,K) = A(J,K) - A(I,K)*S
   70     CONTINUE
C
C
          DO 80 K = 1,NV
            B(J,K) = B(J,K) - B(I,K)*S
   80     CONTINUE
   90   CONTINUE
C
C
  100 CONTINUE
C
C---- modify singular matrix
C
      DO 110 I = 1,3
        IF (ABS(A(I,I)).LT.1.0E-15) A(I,I) = MAX(1.0E-15,
     +                            MAX(A(1,1),A(2,2),A(3,3))*1.0E-15)
  110 CONTINUE
C
C
      DO 140 K = 1,NV
        DO 130 I = 1,3
          N = 4 - I
          M = N + 1
          TEM = B(N,K)
C
C
          IF ((3-M).GE.0) THEN
C
C
            DO 120 J = M,3
              TEM = TEM - A(N,J)*B(J,K)
  120       CONTINUE
C
C
          END IF
C
C
          B(N,K) = TEM/A(N,N)
          X(N,K) = B(N,K)
  130   CONTINUE
  140 CONTINUE
C
C
      RETURN
      END
C
C
C
C     ==============================
      SUBROUTINE matrxbymatrx(X,Y,Z)
C     ==============================
C
C---- multiply two matrices
C     Z(3,3)=X(3,3)*Y(3,3)
C
C
C     .. Array Arguments ..
      REAL X(3,3),Y(3,3),Z(3,3)
C     ..
C     .. Local Scalars ..
      INTEGER I,J,K
C     ..
C
C
      DO 30 I = 1,3
        DO 20 K = 1,3
          Z(I,K) = 0.0
          DO 10 J = 1,3
            Z(I,K) = X(I,J)*Y(J,K) + Z(I,K)
   10     CONTINUE
   20   CONTINUE
   30 CONTINUE
C
C
      RETURN
      END
C
C
C
C     ===============================
      SUBROUTINE matrxbyvector(X,Y,Z)
C     ===============================
C
C---- matrix * vector
C     Z(3)=X(3,3)*Y(3)
C
C
C     .. Array Arguments ..
      REAL X(3,3),Y(3),Z(3)
C     ..
C     .. Local Scalars ..
      INTEGER I,J
C     ..
C
C
      DO 20 I = 1,3
        Z(I) = 0.0
        DO 10 J = 1,3
          Z(I) = X(I,J)*Y(J) + Z(I)
   10   CONTINUE
   20 CONTINUE
C
C
      RETURN
      END
C
C
C
C     ==========================
      INTEGER FUNCTION mint(A,N)
C     ==========================
C
C
C---- next higher integer if n.ge.0, next lower if n.lt.0
C
C
C     .. Scalar Arguments ..
      REAL A
      INTEGER N
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC INT
C     ..
C
C
      mint = INT(A)
C
C
      IF (N.LT.0) THEN
        IF (A.LT.0.0) mint = mint - 1
      ELSE
        IF (A.GT.0.0) mint = mint + 1
      END IF
C
C
      END
C
C
C
C     =================================
      SUBROUTINE normvectproduct(A,B,C)
C     =================================
C
C
C---- will be the vector product of a x b  and will be normalised
C     to unit length
C
C
C
C
C     .. Array Arguments ..
      REAL A(3),B(3),C(3)
C     ..
C     .. Local Scalars ..
      REAL D
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,SQRT
C     ..
C
C
      C(1) = A(2)*B(3) - A(3)*B(2)
      C(2) = A(3)*B(1) - A(1)*B(3)
      C(3) = A(1)*B(2) - A(2)*B(1)
C
C
      D = SQRT(C(1)*C(1)+C(2)*C(2)+C(3)*C(3))
C
C
      IF (D.LT.0.000001) GO TO 20
   10 C(1) = C(1)/D
      C(2) = C(2)/D
      C(3) = C(3)/D
      RETURN
C
C---- vectors a and b are co-linear -
C     give message and return a vector
C     perpendicular to a
C
   20 WRITE (6,FMT=6000) A,B
 6000 FORMAT (' *WARNING* ',/'    vector product of co-linear vectors ',
     +       'a=',3F8.5,'   b=',3F8.5)
      IF (ABS(A(2)).LT.0.000001 .AND. ABS(A(3)).LT.0.000001) GO TO 30
      C(1) = 0.0
      C(2) = -B(3)
      C(3) = B(2)
      GO TO 40
   30 C(1) = B(3)
      C(2) = 0.0
      C(3) = B(1)
C
   40 D = SQRT(C(1)*C(1)+C(2)*C(2)+C(3)*C(3))
      IF (D.GT.0.000001) GO TO 10
      RETURN
      END
C
C
C
C     ===============================================================
      SUBROUTINE openmaps(MAP,NDIM,Map_Title,NSEC,MUVW,Map_XYZ_Limits,
     +                    CELL,RHMAX,RHMEAN,RHMIN,RHRMS,Patt_Flag)
C     ===============================================================
C
C
C
C read Map_Title & 1st data block from map to  get dimensions
C map on stream map+2 (ie 3 or 4)
C
C
C     .. Scalar Arguments ..
      REAL CELL(6)
      INTEGER MAP,NSEC
      CHARACTER Map_Title*80
C     ..
C     .. Array Arguments ..
      INTEGER Map_XYZ_Limits(3),MUVW(3),NDIM(5)
C     ..
C     .. Local Scalars ..
      REAL RHMAX,RHMEAN,RHMIN,RHRMS
      INTEGER IST,LMODE,LSPGRP,LPATSGS(24)
      CHARACTER TITLE*80
C     ..
C     .. Local Arrays ..
      CHARACTER MAPNAM(2)*6
C     ..
C     .. External Functions ..
      INTEGER LENSTR
      EXTERNAL LENSTR
C     ..
C     .. External Subroutines ..
      EXTERNAL MRDHDR
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Data statements ..
      DATA MAPNAM/'MAPIN ','MAPIN2'/
C     ..
      DATA LPATSGS /
     $    2,  10,  12,  47,  65, 69,  71,  83,  87,  123, 139, 147,
     $  148, 162, 164, 166, 175, 191, 200, 202, 204, 221, 225, 229/
C
C
      IST = MAP + 2
C
C---- Open map input file, named MAPIN or MAPIN2 and read header
C
C          **********************************************************
      CALL MRDHDR(IST,MAPNAM(MAP),TITLE,NSEC,MUVW,Map_XYZ_Limits,
     +            NDIM(1),NDIM(2),NDIM(3),NDIM(4),NDIM(5),CELL,LSPGRP,
     +            LMODE,RHMIN,RHMAX,RHMEAN,RHRMS)
C          ***********************************************************
C
      READ (TITLE,FMT=6000) Map_Title
 6000 FORMAT (A)
      WRITE (6,FMT=6002) MAP,Map_Title(1:LENSTR(Map_Title)),NDIM
 6002 FORMAT (/' Title from map',I5,' : ',a,/' 1st section number and ',
     +       'limits on u & v :',5I6)
      DO II = 1,24
       IF(LSPGRP.EQ.LPATSGS(II)) Patt_Flag = 1
      END DO
      RETURN
      END
C
C
C
C     =========================================
      SUBROUTINE ortbond(INSTR,NA1,NA2,RBO,NLI)
C     =========================================
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Labels
      PARAMETER (Max_Labels=100)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
C     ..
C     .. Scalar Arguments ..
      REAL RBO
      INTEGER INSTR,NA1,NA2,NLI
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Charcter_Size,
     +     CurrentCHARACTER_SIZE,D001,D010,D100,Dist_to_Boundary,
     +     Drawing_Size,Ellipse_ScaleFact,Eye_Distance,
     +     RFR(4,4),ROR(4,4),
     +     Overlap_Margin,Paper_Length,Paper_Width,Perspective_Distance,
     +     Picture_Scale,Plotter_Step,Retrace_Displmnt,S,Scal_pre_Proj,
     +     SCL,Shadelight_X,Shadelight_Y,Stereo_Separation
      INTEGER IERR,Iuse_persp_Labs,Label_atoms_Flag,Lcurrent_Colour,
     +        Ldraw_cell_Flag,Ljoin_Flag,Lorthogonalisation_Code,
     +        Lpack_Flag,Lpaper_Setting,Lperspective_Flag,
     +        Lplot_colour_Stereo,Lposition_Flag,Lprint_Flag,
     +        LShade_Flag,LSolid_Flag,LStereo_Flag,LSuperimpose_Flag,
     +        LTaper_Flag,LTrace_Flag,NAT,NATO,Natoms_plusCell,NC,NC1,
     +        Ncopies,NDrawing_Number,Nspecified_Bonds,Num_Moles
      LOGICAL Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
      CHARACTER Iden_Title*80
C     ..
C     .. Arrays in Common ..
      REAL AID,Atom_Radii,Base_Vect_Refsys,Base_Vect_Wrksys,BBB,
     +     Drawing_Origin,Ellipse_Resolution,Fractional_XYZ,
     +     G_Metric_Tensor,OrtCell,OrtConts,Orthogonal_XYZ,P1_to_Refsys,
     +     P1_to_Wrksys,Plotting_XYZ,Principal_axis_rms,
     +     Prin_Angstrom_Vectors,Prin_Axis_Col_Vects,Q,
     +     Recip_Metric_Tensor,RP,Sorted_xyz,Title_Rotation,
     +     Title_RotnTrans,ViewMatrix,XABC,XCLANG,XO
      INTEGER IAtom_Connectivity,IAtom_Type,ISpecified_Bond_Params,
     +        KSortPointer,LAtom_Inc_Flag,LAtom_Label_Flag,MOL
      CHARACTER Atm_Lab_Type*2,Atm_Identifier*10,Atm_Label*10,
     +          Atm_Lab_Txt*10
C     ..
C     .. Local Scalars ..
      REAL BORDER_WIDTH,RADIUS,T1,T2,T3,T4,T5,T6,VJ1,VJ2
      INTEGER I,IBND,ICQ,II,J,J1,J2,K,K1,K2,KKK,KL,KODE,KSTP,KSYMNUM,L,
     +        M,N,NBND,NUMSUBDIVI
C     ..
C     .. Local Arrays ..
      REAL B(3,3),CIRCLELINES_2D(2,130),CIRCLESPOKES(3,130),
     +     CONJUGATEVECTORS(3,3),E(3,3),QMATRIX(3,3),SSS(3,3),U(3,3),
     +     V1(4),V2(3),V3(3),V4(3),V5(3),V6(3),VUE(3),WW(3,2),WWW(3,2),
     +     WX(3,2),WXX(2)
      INTEGER NATTA(2)
C     ..
C     .. External Functions ..
      REAL vectbymatrxbyvector,vectbyvecttranspose
      EXTERNAL vectbymatrxbyvector,vectbyvecttranspose
C     ..
C     .. External Subroutines ..
      EXTERNAL bondoverlapchk,generateellipse,getplotxy,getqmatrix,
     +         getunitvectors,matrxbyvector,ortdraw,project2dto3d,
     +         vectordifference,vectorstore
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MOD,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,Picture_Scale,
     +       Perspective_Distance,Shadelight_X,Shadelight_Y,S,
     +       Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),ViewMatrix(3,3),
     +       MOL(Max_Mols,4),ISpecified_Bond_Params(30,3)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCHARACTER_SIZE,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_LABCHR/Atm_Lab_Txt(Max_Labels),
     +       Atm_Lab_Type(Max_Labels),Atm_Label(Max_Labels),
     +       Atm_Identifier(Max_Atoms),Iden_Title
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      BORDER_WIDTH = 1.0
C
C
      DO 10 J = 1,3
        WW(J,1) = Plotting_XYZ(J,NA1)
        WW(J,2) = Plotting_XYZ(J,NA2)
   10 CONTINUE
C
C
      DO 20 J = 1,2
C
C---- adjusted plotting   2 d array returned
C
C            **************************
        CALL getplotxy(WW(1,J),WWW(1,J))
C            **************************
C
ccx         IF ((Dist_to_Boundary-Border_Width*0.25).LT.0.0)
ccx     +   CALL CCPERR(1,' BOND ERROR out of bounds')
   20 CONTINUE
C
C
      KODE = NLI
C
C
      IF (KODE.EQ.0) NBND = 0
      IF (KODE.LT.0) THEN
        KODE = -KODE
        NBND = 128/2**KODE
      END IF
      IF (KODE.GT.0) THEN
        NBND = 128/2**KODE
      END IF
C
C---- find uppermost atom put in position one
C
      IF (Eye_Distance.EQ.0.0) THEN
        WX(3,1) = 1.0
        WX(3,2) = 1.0
C
C
        IF ((WW(3,1)-WW(3,2)).LT.0.0) THEN
          GO TO 50
        ELSE
          GO TO 70
        END IF
      END IF
C
C---- vector from atom to viewpoint
C
      DO 40 I = 1,2
        DO 30 J = 1,3
          WX(J,I) = -WW(J,I)
   30   CONTINUE
C
C
        WX(3,I) = WX(3,I) + Eye_Distance
C
C---- distance squared to viewpoint
C
C                ************************************
        WXX(I) = vectbyvecttranspose(WX(1,I),WX(1,I))
C                ************************************
C
   40 CONTINUE
C
C
      IF ((WXX(2)-WXX(1)).GE.0.0) GO TO 70
C
C---- switch atoms
C
   50 CONTINUE
C
C
      DO 60 J = 1,3
        T1 = WW(J,1)
        WW(J,1) = WW(J,2)
        WW(J,2) = T1
        T1 = WWW(J,1)
        WWW(J,1) = WWW(J,2)
        WWW(J,2) = T1
        T1 = WX(J,1)
        WX(J,1) = WX(J,2)
        WX(J,2) = T1
   60 CONTINUE
      T1 = WXX(1)
      WXX(1) = WXX(2)
      WXX(2) = T1
C
C---- form idemfactor matrix
C
   70 CONTINUE
C
C
      DO 90 KKK = 1,3
        DO 80 J = 1,3
          E(J,KKK) = 0.0
   80   CONTINUE
        E(KKK,KKK) = 1.0
   90 CONTINUE
C
C---- form vector set radial to bond
C
C          *******************************************************
      CALL vectordifference(WW(1,2),WW(1,1),CONJUGATEVECTORS(1,3))
      CALL getunitvectors(CONJUGATEVECTORS(1,3),V3,1)
C          *******************************************************
C
C---- unit vector from bond midpoint to reference viewpoint
C
      DO 110 I = 1,3
        V2(I) = 0.0
C
C
        DO 100 J = 1,3
          V2(I) = ViewMatrix(J,I) + V2(I)
  100   CONTINUE
C
C
        IF (Eye_Distance.GT.0.0) V2(I) = V2(I)*Eye_Distance -
     +      0.5* (WW(I,1)+WW(I,2))
  110 CONTINUE
C
C          *******************************
      CALL getunitvectors(V2,V2,1)
      T6 = ABS(vectbyvecttranspose(V3,V2))
C          *******************************
C
C
C
      IF ((0.9994-T6).LE.0.0) THEN
C
C---- alternate calc if bond is along reference view direction
C
        DO 120 J = 1,3
          V2(J) = WX(J,1) + WX(J,2)
  120   CONTINUE
C
C            *******************************
        CALL getunitvectors(V2,V2,1)
        T6 = ABS(vectbyvecttranspose(V3,V2))
C            *******************************
C
        IF ((0.9994-T6).LE.0.0) THEN
          IF (Lprint_Flag.GT.0) WRITE (6,FMT=6000) Atm_Identifier(NA1),
     +        Atm_Identifier(NA2)
 6000     FORMAT (' HIDDEN end on BOND between atoms ',A,' ',A)
          RETURN
        END IF
      END IF
C
C          **********************
      CALL vectorstore(V3,V2,B,1)
C          **********************
C

      T1 = RBO/Ellipse_ScaleFact
C
C
      DO 130 J = 1,3
        CONJUGATEVECTORS(J,1) = -B(J,2)*T1
        CONJUGATEVECTORS(J,2) = -B(J,3)*T1
  130 CONTINUE
C
C
      IF (NBND.LE.0) GO TO 360
      NUMSUBDIVI = 3
C
C          *********************************************************
      CALL generateellipse(NUMSUBDIVI,CIRCLESPOKES,CONJUGATEVECTORS)
C          *********************************************************
C
C---- derive quadrics for each atom
C
      NATTA(1) = NA1
      NATTA(2) = NA2
C
C
      DO 300 II = 1,2
        KSYMNUM = 1
C
C            *****************************************
        CALL getqmatrix(2,QMATRIX,KSYMNUM,NATTA(II))
C            *****************************************
C
C---- does bond go to ellipsoid or to envelope
C
        T1 = 3 - II*2
C
C
        DO 140 J = 1,3
          V3(J) = V3(J)*T1
          VUE(J) = 0.0
  140   CONTINUE
C
C
        IF (NLI.GT.0) THEN
C
C             **********************************
          IF (vectbymatrxbyvector(V3,QMATRIX,WX(1,II)).LT.0) THEN
C             **********************************
C
            IBND = 0
            IF (Eye_Distance.GT.0.0) THEN
C
C---- derive tangent cone directly without rotating coordinates
C
              T2 = - (Principal_axis_rms(1)*Ellipse_ScaleFact*
     +             Principal_axis_rms(2)*Principal_axis_rms(3))**2
C
C
              DO 170 J = 1,3
                V1(J) = -WX(J,II)/Scal_pre_Proj
                VUE(J) = V1(J)/Ellipse_ScaleFact
C
C---- invert ellipsoid matrix
C
                DO 160 K = J,3
                  T1 = 0.0
                  DO 150 I = 1,3
                    T1 = Prin_Axis_Col_Vects(J,I)*
     +                   Prin_Axis_Col_Vects(K,I)*
     +                   Principal_axis_rms(I)**2 + T1
  150             CONTINUE
C
C
                  U(J,K) = T1
                  U(K,J) = T1
  160           CONTINUE
  170         CONTINUE
C
C---- add polarized cofactor matrix to ellipsoid matrix
C
              DO 190 J = 1,3
                J1 = MOD(J,3) + 1
                VJ1 = V1(J1)
                J2 = MOD(J+1,3) + 1
                VJ2 = V1(J2)
                DO 180 K = J,3
                  K1 = MOD(K,3) + 1
                  K2 = MOD(K+1,3) + 1
                  SSS(J,K) = ((U(J1,K1)*V1(K2)-U(J1,K2)*V1(K1))*VJ2+
     +                       (U(J2,K2)*V1(K1)-U(J2,K1)*V1(K2))*VJ1) +
     +                       QMATRIX(J,K)*T2
                  SSS(K,J) = SSS(J,K)
  180           CONTINUE
  190         CONTINUE
C
C
              T5 = 0.0
              GO TO 240
            END IF
C
C---- derive tangent cylinder with axis along z
C
            T1 = -1.0/QMATRIX(3,3)
C
C
            DO 210 J = 1,2
              DO 200 K = 1,2
                SSS(K,J) = QMATRIX(K,3)*QMATRIX(J,3)*T1 + QMATRIX(K,J)
  200         CONTINUE
C
C
              SSS(3,J) = 0.0
              SSS(J,3) = 0.0
  210       CONTINUE
C
C
            SSS(3,3) = 0.0
            GO TO 230
          END IF
        END IF
C
C---- transfer ellipsoid
C
        DO 220 J = 1,9
          SSS(J,1) = QMATRIX(J,1)
  220   CONTINUE
C
C
        IBND = II
  230   T5 = 1.0
C
C---- check for bond taper
C
  240   CONTINUE
C
C
        IF ((II-2).LT.0) THEN
          RADIUS = T6*Bond_Taper + 1.0
        ELSE
          RADIUS = 1.0 - T6*Bond_Taper
        END IF
C
C            **************************
        CALL matrxbyvector(SSS,V3,V4)
        T2 = vectbyvecttranspose(V3,V4)
C            **************************
C
C---- compute bond intersection
C
        KL = 5 - II - II
        KSTP = 4
C
C
        IF (INSTR.GE.822) KSTP = 32
C
C
        DO 270 K = 1,65,KSTP
          DO 250 J = 1,3
            V6(J) = CIRCLESPOKES(J,K)*RADIUS
            V5(J) = V6(J) + VUE(J)
  250     CONTINUE
C
C              ********************************************
          T3 = vectbyvecttranspose(V5,V4)
          T4 = T3*T3 - (vectbymatrxbyvector(V5,SSS,V5)-T5)*T2
C              ********************************************
C
          IF (T4.LT.0.0) THEN
            IF (Lprint_Flag.GT.0) WRITE (6,
     +          FMT=6002) Atm_Identifier(NA1),Atm_Identifier(NA2)
 6002       FORMAT (' BOND larger than atom, Bond from atom ',A,' to a',
     +             'tom ',A)
            RETURN
          END IF
C
C
          T4 = SQRT(T4)
C
          T1 = (T4-T3)/T2
          T3 = (-T4-T3)/T2
          L = K + KL - 1
C
C
          DO 260 J = 1,3
            CIRCLESPOKES(J,L) = (V3(J)*T1+V6(J))*SCL
            CIRCLESPOKES(J,L+1) = (-V6(J)-V3(J)*T3)*SCL
  260     CONTINUE
  270   CONTINUE
C
C
ccx        IF (INSTR.EQ.822) THEN
        IF (NLI.GT.0) THEN
C
C---- for overlap, make bond quadrangle tangent to enveloping cone
C
C                **********************************************
          T3 = vectbyvecttranspose(VUE,V4)
          T4 = T3**2 - (vectbymatrxbyvector(VUE,SSS,VUE)-T5)*T2
C                **********************************************
C
          IF (T4.LT.0.0) THEN
            IF (Lprint_Flag.GT.0) WRITE (6,
     +          FMT=6004) Atm_Identifier(NA1),Atm_Identifier(NA2)
 6004       FORMAT (' Imaginary bond intersection Bond from atom ',A,
     +             ' to atom ',A)
            RETURN
          END IF
C
C
          T1 = (SQRT(T4)-T3)/T2
C
C
          DO 280 J = 1,3
            T4 = (V3(J)*T1*SCL- (CIRCLESPOKES(J,KL+64)+CIRCLESPOKES(J,
     +           KL))*0.5)*1.001
            CIRCLESPOKES(J,KL) = CIRCLESPOKES(J,KL) + T4
            CIRCLESPOKES(J,KL+64) = CIRCLESPOKES(J,KL+64) + T4
  280     CONTINUE
C
C
        END IF
ccx        END IF
C
C            ***********************************************
        CALL project2dto3d(CIRCLESPOKES(1,KL),CIRCLELINES_2D(1,II),
     +                     WW(1,II),XO,Eye_Distance,1,65,KSTP)
C            ***********************************************
C
        IF ((IBND-1).EQ.0) THEN
C
C              **********************************************
          CALL project2dto3d(CIRCLESPOKES(1,KL+5),
     +                       CIRCLELINES_2D(1,II+68),WW(1,II),XO,
     +                       Eye_Distance,1,61,KSTP)
C              **********************************************
C
          GO TO 300
        END IF
C
C---- retrace top half
C
        DO 290 K = 4,64,4
          L = K + II
          M = L + 64
          N = 66 - L
          CIRCLELINES_2D(1,M) = CIRCLELINES_2D(1,N)
          CIRCLELINES_2D(2,M) = CIRCLELINES_2D(2,N)
  290   CONTINUE
  300 CONTINUE
C
C---- check for overlap or hidden bond
C
      DO 320 K = 1,65,32
        T1 = 0.0
        T2 = 0.0
        DO 310 J = 1,2
          T1 = (CIRCLELINES_2D(J,K)-WWW(J,1))**2 + T1
          T2 = (CIRCLELINES_2D(J,K+1)-WWW(J,1))**2 + T2
  310   CONTINUE
        IF ((T2-T1).LE.0.0) THEN
          IF (Lprint_Flag.GT.0) WRITE (6,FMT=6006) Atm_Identifier(NA1),
     +        Atm_Identifier(NA2)
 6006     FORMAT (' End on Bond, Bond from atom ',A,' to atom ',A)
          RETURN
        END IF
  320 CONTINUE
C
C---- call overlap routine
C
      ICQ = 0
C
C          ************************************************
      CALL bondoverlapchk(NA1,NA2,ICQ,CIRCLELINES_2D,INSTR)
C          ************************************************
C
      IF (INSTR.EQ.822) RETURN
      IF (ICQ.LT.0) THEN
        IF (Lprint_Flag.GT.0) WRITE (6,FMT=6008) Atm_Identifier(NA1),
     +      Atm_Identifier(NA2)
 6008   FORMAT (' hidden end on bond, Bond from atom ',A,' to atom ',A)
        RETURN
      END IF
C
C---- draw bond outline
C
C          **************************************
      CALL ortdraw(CIRCLELINES_2D(1,1),0.0,0.0,3)
C          **************************************
C
      DO 330 K = 5,129,4
C
C            **************************************
        CALL ortdraw(CIRCLELINES_2D(1,K),0.0,0.0,2)
C            **************************************
C
  330 CONTINUE
C
C
      DO 340 K = 2,66,4
C
C            **************************************
        CALL ortdraw(CIRCLELINES_2D(1,K),0.0,0.0,2)
C            **************************************
C
  340 CONTINUE
C
C          **************************************
      CALL ortdraw(CIRCLELINES_2D(1,65),0.0,0.0,2)
C          **************************************
C
C---- draw bond detail
C
      K = 65
  350 K = K - NBND
      IF ((K-1).LE.0) GO TO 360
C
C          **************************************
      CALL ortdraw(CIRCLELINES_2D(1,K),0.0,0.0,3)
      CALL ortdraw(CIRCLELINES_2D(1,K+1),0.0,0.0,2)
C          **************************************
C
      K = K - NBND
      IF ((K-1).LE.0) GO TO 360
C
C          **************************************
      CALL ortdraw(CIRCLELINES_2D(1,K+1),0.0,0.0,3)
      CALL ortdraw(CIRCLELINES_2D(1,K),0.0,0.0,2)
C          **************************************
C
      GO TO 350
  360 CONTINUE
      RETURN
      END
C
C
C
C     ================================
      SUBROUTINE ortdraw(W,DX,DY,NPEN)
C     ================================
C
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
C     ..
C     .. Scalar Arguments ..
      REAL DX,DY
      INTEGER NPEN
C     ..
C     .. Array Arguments ..
      REAL W(2)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,CurrentCharcter_Size,
     +     D001,D010,D100,Dist_to_Boundary,Ellipse_ScaleFact,
     +     Eye_Distance,RFR(4,4),ROR(4,4),
     +     Overlap_Margin,Paper_Length,
     +     Paper_Width,Plotter_Step,Retrace_Displmnt,Scal_pre_Proj,SCL
      INTEGER Iuse_persp_Labs,Lorthogonalisation_Code,Lpack_Flag,
     +        NDrawing_Number
      LOGICAL UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL AID,Base_Vect_Refsys,Base_Vect_Wrksys,BBB,Drawing_Origin,
     +     Ellipse_Resolution,G_Metric_Tensor,OrtCell,OrtConts,
     +     P1_to_Refsys,P1_to_Wrksys,Principal_axis_rms,
     +     Prin_Angstrom_Vectors,Prin_Axis_Col_Vects,Q,
     +     Recip_Metric_Tensor,Sorted_xyz,Title_Rotation,
     +     Title_RotnTrans,XABC,XCLANG,XO
      INTEGER KSortPointer
C     ..
C     .. Local Scalars ..
      INTEGER I,NCQ
C     ..
C     .. Local Arrays ..
      REAL X(3),Y(3),Z(3)
C     ..
C     .. External Subroutines ..
      EXTERNAL drawvisiblelines,getplotxy,zdrawto,zmoveto
C     ..
C     .. Common blocks ..
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      Iuse_persp_Labs = 0
C
      Y(1) = W(1) + DX
      Y(2) = W(2) + DY
C
C
      IF (Iuse_persp_Labs.NE.0) THEN
C
C---- rotate for perspective title
C
        Y(3) = XO(3)
C
C
        DO 10 I = 1,3
          Z(I) = Y(I) - Title_RotnTrans(I,4)
   10   CONTINUE
C
C
        DO 20 I = 1,3
          X(I) = Title_RotnTrans(I,1)*Z(1) + Title_RotnTrans(I,2)*Z(2) +
     +           Title_RotnTrans(I,3)*Z(3) + Title_RotnTrans(I,4)
   20   CONTINUE
C
C            **************
        CALL getplotxy(X,Y)
C            **************
C
      END IF
C
C---- check boundry
C
      IF ((Y(1)-Paper_Width+0.1).GT.0.0) Y(1) = Paper_Width - 0.1
      IF ((Y(1)-0.1).LT.0.0) Y(1) = 0.1
C
C
      IF ((Y(2)-Paper_Length+0.1).GT.0.0) Y(2) = Paper_Length - 0.1
      IF ((Y(2)-0.1).LT.0.0) Y(2) = 0.1
C
C---- check for overlap
C
      NCQ = 0
C
C          ****************************
      CALL drawvisiblelines(Y,NPEN,NCQ)
C          ***************************
C
      IF (NCQ.GT.0) RETURN
C
C---- call plotting routine if no overlapping elements are stored
C
C                    ***********************
      IF (NPEN.EQ.2) CALL zdrawto(Y(1),Y(2))
      IF (NPEN.EQ.3) CALL zmoveto(Y(1),Y(2))
C                    ***********************
C
      RETURN
      END
C
C
C
C     =========================================================
      SUBROUTINE ortview(INSTR,LINEX,IBEGX,IENDX,ITYPX,FVALUEX,CVALUEX,
     +                   IDECX,NTOKX,LOGPARSER)
C     ============================================================
C
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER Max_Tokens
      PARAMETER (Max_Tokens=200)
C     ..
C     .. Scalar Arguments ..
      INTEGER INSTR,NTOKX
      LOGICAL LOGPARSER
      CHARACTER LINEX*600
C     ..
C     .. Array Arguments ..
      REAL FVALUEX(Max_Tokens)
      INTEGER IBEGX(Max_Tokens),IDECX(Max_Tokens),IENDX(Max_Tokens),
     +        ITYPX(Max_Tokens)
      CHARACTER CVALUEX(Max_Tokens)*4
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Charcter_Size,
     +     CurrentCharcter_Size,Dist_to_Boundary,Drawing_Size,
     +     Ellipse_ScaleFact,Eye_Distance,Overlap_Margin,Paper_Length,
     +     Paper_Width,Perspective_Distance,Picture_Scale,Plotter_Step,
     +     Retrace_Displmnt,S,Scal_pre_Proj,SCL,Shadelight_X,
     +     Shadelight_Y,Stereo_Separation
      INTEGER IERR,Iuse_persp_Labs,Label_atoms_Flag,Lcurrent_Colour,
     +        Ldraw_cell_Flag,Ljoin_Flag,Lpack_Flag,Lpaper_Setting,
     +        Lperspective_Flag,Lplot_colour_Stereo,Lposition_Flag,
     +        Lprint_Flag,LShade_Flag,LSolid_Flag,LStereo_Flag,
     +        LSuperimpose_Flag,LTaper_Flag,LTrace_Flag,NAT,NATO,
     +        Natoms_plusCell,NC,NC1,Ncopies,NDrawing_Number,
     +        Nspecified_Bonds,Num_Moles
      LOGICAL Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL Atom_Radii,Base_Vect_Refsys,Base_Vect_Wrksys,BBB,
     +     Drawing_Origin,Ellipse_Resolution,Fractional_XYZ,
     +     G_Metric_Tensor,Orthogonal_XYZ,P1_to_Refsys,P1_to_Wrksys,
     +     Plotting_XYZ,Principal_axis_rms,Prin_Angstrom_Vectors,
     +     Prin_Axis_Col_Vects,Q,Recip_Metric_Tensor,RP,Sorted_xyz,
     +     Title_Rotation,Title_RotnTrans,ViewMatrix,XO
      INTEGER IAtom_Connectivity,IAtom_Type,ISpecified_Bond_Params,
     +        KSortPointer,LAtom_Inc_Flag,LAtom_Label_Flag,MOL
C     ..
C     .. Local Scalars ..
      REAL T1,T2,XOR
      INTEGER I,I1,I2,I3,IND,IWORK,J,JDO,K,KAT,KK
      CHARACTER FWORK*10
C     ..
C     .. Local Arrays ..
      REAL RM(3,3),V(3,4),V1(4),V2(4),WW(3)
C     ..
C     .. External Subroutines ..
      EXTERNAL CCPERR,getatomid,matrxbymatrx,vectorstore
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC COS,MOD,NINT,SIN
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,Picture_Scale,
     +       Perspective_Distance,Shadelight_X,Shadelight_Y,S,
     +       Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),ViewMatrix(3,3),
     +       MOL(Max_Mols,4),ISpecified_Bond_Params(30,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      IWORK = 2
C
C
      IF (INSTR.EQ.501) THEN
        FWORK = LINEX(IBEGX(IWORK) :IENDX(IWORK))
C
C           ********************
        CALL getatomid(KAT,FWORK)
C           ********************
C
        IF (KAT.LE.0) 
C
C           **************************************************
     + CALL CCPERR(1,' Ortep view ORIGIN atom not recognised')
C           **************************************************
C
        DO 10 JDO = 1,3
          Drawing_Origin(JDO) = Orthogonal_XYZ(KAT,JDO)
   10   CONTINUE
C
C
        IWORK = IWORK + 1
C
C
        DO 30 K = 1,4
          FWORK = LINEX(IBEGX(K+2) :IENDX(K+2))
C
C              ********************
          CALL getatomid(KAT,FWORK)
C              ********************
C
          IF (KAT.LE.0) 
C
C            *******************************************************
     +  CALL CCPERR(1,' Ortep view Vectors --  atom not recognised')
C            *******************************************************
C
          DO 20 JDO = 1,3
            V(JDO,K) = Orthogonal_XYZ(KAT,JDO)
   20     CONTINUE
C
C
   30   CONTINUE
C
C
        DO 40 J = 1,3
          V1(J) = V(J,2) - V(J,1)
          V2(J) = V(J,4) - V(J,3)
   40   CONTINUE
C
C
        IND = -1
        IF (NTOKX.EQ.7) IND = -2
C
C            ***************************************
        CALL vectorstore(V1,V2,Base_Vect_Refsys,IND)
C            ***************************************
C
        DO 60 J = 1,3
          DO 50 K = 1,3
            ViewMatrix(J,K) = Base_Vect_Refsys(J,K)
   50     CONTINUE
   60   CONTINUE
        RETURN
      END IF
C
C
      IF (INSTR.EQ.504) THEN
C
C---- shift origin for projection axis (in inches ??? ccx)
C
        DO 70 JDO = 1,3
          WW(JDO) = FVALUEX(JDO+1)
   70   CONTINUE
C
C
        DO 90 J = 1,3
          DO 80 K = 1,3
            Drawing_Origin(J) = ViewMatrix(J,K)*WW(K)/Scal_pre_Proj +
     +                          Drawing_Origin(J)
   80     CONTINUE
          XO(J) = XO(J) + WW(J)
   90   CONTINUE
C
C
        DO 110 J = 1,3
          DO 100 K = 1,3
            ViewMatrix(J,K) = Base_Vect_Refsys(J,K)
  100     CONTINUE
  110   CONTINUE
        RETURN
      END IF
C
C---- form rotation matrix 502/503(stereo)
C
      IF (INSTR.EQ.502 .OR. INSTR.EQ.503) THEN
        I = NINT(FVALUEX(2))
        IF (I.GT.0) THEN
          XOR = FVALUEX(3)*1.7453293E-2
          T1 = COS(XOR)
          T2 = SIN(XOR)
          I3 = MOD(I+2,3) + 1
          I1 = MOD(I3,3) + 1
          I2 = MOD(I1,3) + 1
          RM(I1,I1) = T1
          RM(I1,I2) = T2
          RM(I1,I3) = 0.0
          RM(I2,I1) = -T2
          RM(I2,I2) = T1
          RM(I2,I3) = 0.0
          RM(I3,I1) = 0.0
          RM(I3,I2) = 0.0
          RM(I3,I3) = 1.0
C
C              *****************************
          CALL matrxbymatrx(ViewMatrix,RM,V)
C              *****************************
C
          DO 130 J = 1,3
            DO 120 KK = 1,3
              ViewMatrix(J,K) = V(J,K)
  120       CONTINUE
  130     CONTINUE
C
C
        ELSE IF (I.LT.0) THEN
          IF (INSTR.EQ.502) THEN
            I = MOD(-I,3)
C
C
            DO 150 J = 1,I
              DO 140 K = 1,3
                T1 = ViewMatrix(K,3)
                ViewMatrix(K,3) = ViewMatrix(K,2)
                ViewMatrix(K,2) = ViewMatrix(K,1)
                ViewMatrix(K,1) = T1
  140         CONTINUE
  150       CONTINUE
          END IF
        END IF
      END IF
C
C
      IF (INSTR.EQ.503) RETURN
C
C
      RETURN
      END
C
C
C
C     =========================
      SUBROUTINE plotbondlength
C     =========================
C
C---- labeling function subroutine
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_BndLabels
      PARAMETER (Max_BndLabels=10)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Charcter_Size,
     +     CurrentCharcter_Size,D001,D010,D100,Dist_to_Boundary,
     +     Drawing_Size,Ellipse_ScaleFact,Eye_Distance,
     +     RFR(4,4),ROR(4,4),
     +     Overlap_Margin,Paper_Length,Paper_Width,Perspective_Distance,
     +     Picture_Scale,Plotter_Step,Retrace_Displmnt,S,Scal_pre_Proj,
     +     SCL,Shadelight_X,Shadelight_Y,Stereo_Separation
      INTEGER IERR,Iuse_persp_Labs,Label_atoms_Flag,Lcurrent_Colour,
     +        Ldraw_cell_Flag,Ljoin_Flag,Lorthogonalisation_Code,
     +        Lpack_Flag,Lpaper_Setting,Lperspective_Flag,
     +        Lplot_colour_Stereo,Lposition_Flag,Lprint_Flag,
     +        LShade_Flag,LSolid_Flag,LStereo_Flag,LSuperimpose_Flag,
     +        LTaper_Flag,LTrace_Flag,NAT,NATO,Natoms_plusCell,
     +        NBondLabels,NC,NC1,Ncopies,NDrawing_Number,
     +        Nspecified_Bonds,Num_Moles
      LOGICAL Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL AID,Atom_Radii,Drawing_Origin,Ellipse_Resolution,
     +     Fractional_XYZ,OrtCell,OrtConts,Orthogonal_XYZ,Parr_Offset,
     +     Perp_Offset,Plotting_XYZ,RP,Title_Rotation,ViewMatrix,XABC,
     +     XCLANG,XO
      INTEGER IAtom_Connectivity,IAtom_Type,ISpecified_Bond_Params,
     +        LAtom_Inc_Flag,LAtom_Label_Flag,MOL
      CHARACTER BondtoLabel*10
C     ..
C     .. Local Scalars ..
      REAL COSTH,D,HGT,RAD,SCAL,SINTH,T1,T2,T3,TH,XCENT,YCENT,ZCENT
      INTEGER JAT,JDO,KAT
      CHARACTER CWORK*5,FWORK*10,GWORK*10
C     ..
C     .. Local Arrays ..
      REAL V1(3),V2(3),XWORK(3),Y(3)
C     ..
C     .. External Subroutines ..
      EXTERNAL getatomid,zplotstring,zsetcharactersize,zsettextangle,
     +         zsettextposition
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ACOS,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,Picture_Scale,
     +       Perspective_Distance,Shadelight_X,Shadelight_Y,S,
     +       Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),ViewMatrix(3,3),
     +       MOL(Max_Mols,4),ISpecified_Bond_Params(30,3)
      COMMON /com_BNDLAB/NBondLabels,Perp_Offset(Max_BndLabels),
     +       Parr_Offset(Max_BndLabels)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_LABBND/BondtoLabel(Max_BndLabels*2)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      RAD = 57.2957795
C
C---- label bond from atom1 to atom2
C
      DO 10 JDO = 1,NBondLabels
        FWORK = BondtoLabel(JDO*2-1)
        GWORK = BondtoLabel(JDO*2)
C
C            ********************
        CALL getatomid(KAT,FWORK)
        CALL getatomid(JAT,GWORK)
C            ********************
C
        IF (KAT.LE.0 .OR. JAT.LE.0) THEN
          WRITE (6,FMT=6000) FWORK,GWORK
 6000     FORMAT (' For bond label betwen atoms :',/'  ',A,' and ',A,
     +           ' atom names not found')
          GO TO 10
        END IF
C
C
        D = (Orthogonal_XYZ(JAT,1)-Orthogonal_XYZ(KAT,1))**2 +
     +      (Orthogonal_XYZ(JAT,2)-Orthogonal_XYZ(KAT,2))**2 +
     +      (Orthogonal_XYZ(JAT,3)-Orthogonal_XYZ(KAT,3))**2
C
C
        D = SQRT(D)
        WRITE (CWORK,FMT=6002) D
 6002   FORMAT (F5.2)
C
C---- find mean reference point
C
        XCENT = (Plotting_XYZ(1,KAT)+Plotting_XYZ(1,JAT))/2.0
        YCENT = (Plotting_XYZ(2,KAT)+Plotting_XYZ(2,JAT))/2.0
        ZCENT = (Plotting_XYZ(3,KAT)+Plotting_XYZ(3,JAT))/2.0
C
C
        XWORK(1) = Plotting_XYZ(1,KAT) - Plotting_XYZ(1,JAT)
        XWORK(2) = Plotting_XYZ(2,KAT) - Plotting_XYZ(2,JAT)
        XWORK(3) = Plotting_XYZ(3,KAT) - Plotting_XYZ(3,JAT)
C
C---- perspective scaling factor
C
        SCAL = 1.0
        IF (Eye_Distance.GT.0.0) SCAL = Eye_Distance/
     +      (Eye_Distance-ZCENT)
ccx      HGT = Charcter_Size * SCAL
        HGT = Charcter_Size
C
C---- projected vector baseline
C
        V1(1) = Plotting_XYZ(1,KAT)
        V1(2) = Plotting_XYZ(2,KAT)
        V1(3) = Plotting_XYZ(3,KAT)
C
C
        V2(1) = Plotting_XYZ(1,JAT)
        V2(2) = Plotting_XYZ(2,JAT)
        V2(3) = Plotting_XYZ(3,JAT)
C
C
        T1 = V2(1) - V1(1)
        T2 = V2(2) - V1(2)
        T3 = SQRT(T1*T1+T2*T2)
        COSTH = T1/T3
        SINTH = T2/T3
        TH = ACOS(COSTH)
        IF (SINTH.LT.0.0) TH = -TH
        TH = TH*RAD
C
C---- find center of projected label
C
        Y(1) = (Parr_Offset(JDO)*COSTH+XCENT-Perp_Offset(JDO)*SINTH)*
     +         SCAL
        Y(2) = (Parr_Offset(JDO)*SINTH+YCENT+Perp_Offset(JDO)*COSTH)*
     +         SCAL
        Y(3) = 0.0
C
C---- set parameters for individual functions
C       true perspective labels
C
        Y(2) = Perp_Offset(JDO) + YCENT - 22*HGT/7.0
C
C---- perspective bond labels
C
        Y(1) = Parr_Offset(JDO) + XCENT - HGT/2.0
C
C---- plot bond distance labels
C
C            ***************************
        CALL zsetcharactersize(HGT)
        CALL zsettextposition(Y(1),Y(2))
        CALL zsettextangle(TH)
          KZMODE = 1
        CALL zplotstring(KZMODE,CWORK,1,5)
C            ***************************
C
   10 CONTINUE
C
C            *********************************
      CALL zsettextangle(0.0)
      CALL zsetcharactersize(Charcter_Size)
C          *********************************
C
      RETURN
      END
C
C
C
C     ==========================================
      SUBROUTINE plotcontour(VECTOR,PSBITS,LIST)
C     ==========================================
C
C
C  Variables may be used before set in module plotcontour:
C        X1                 Y1
C
C Plot vectors in grid units (starting at 0,0,0)
C    VECTOR(3,LIST)  list of coordinates
C    PSBITS(LIST)    =.false. move, =.true. draw
C    LIST            number of vectors
C
C
C
C
C     .. Parameters ..
      INTEGER MAXLIN
      PARAMETER (MAXLIN=300)
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
C     ..
C     .. Scalar Arguments ..
      INTEGER LIST
C     ..
C     .. Array Arguments ..
      REAL VECTOR(3,LIST)
      LOGICAL*1 PSBITS(LIST)
C     ..
C     .. Scalars in Common ..
      REAL Charcter_Size,DEY,DEYC,DPL,Drawing_Size,Perspective_Distance,
     +     Picture_Scale,QPERS,QX,QY,S,Shadelight_X,Shadelight_Y,
     +     Stereo_Separation,U0,UN,V0,VN,W,W0,WN,XC,YC,ZMAX,ZPL
      INTEGER Invert_Map_Flag,Lcurrent_Colour,Ldraw_cell_Flag,
     +        LimitsSet_Flag,Lmap3d_Flag,Lmap_ModeFirst,Lpaper_Setting,
     +        Lperspective_Flag,Lplot_colour_Stereo,Lposition_Flag,
     +        Lprint_Flag,LSetView_Flag,LShade_Flag,LSolid_Flag,
     +        LStereo_Flag,LSuperimpose_Flag,LTaper_Flag,LTrace_Flag,
     +        Map_Counter,Map_Flag_Counter,Map_pts_U,Map_pts_V,MU,MV,MW,
     +        Natoms_plusCell,Ncopies,Npicture,Npts_U_compressed,
     +        Npts_V_compressed,Nspecified_Bonds,Num_Moles
      LOGICAL Log_Draw_Title
C     ..
C     .. Arrays in Common ..
      REAL AQ,Contour_Levels,Contour_Threshold,Plotting_XYZ,RP,
     +     ViewMatrix
      INTEGER ISpecified_Bond_Params,Map_def_Restrictions,Map_LineTypes,
     +        Map_UVW_Limits,Map_XYZ_Limits,MOL,Num_Contours
      CHARACTER Map_Title*80
C     ..
C     .. Local Scalars ..
      REAL AQ11,AQ12,AQ13,AQ21,AQ22,AQ23,AQ31,AQ32,AQ33,U,V,X1,XXP,Y1,
     +     YYP,ZZP
      INTEGER IFIRST,J,K,LPERM
C     ..
C     .. Local Arrays ..
      REAL X(MAXLIN),Y(MAXLIN)
C     ..
C     .. External Subroutines ..
      EXTERNAL drawlineset
C     ..
C     .. Common blocks ..
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,Picture_Scale,
     +       Perspective_Distance,Shadelight_X,Shadelight_Y,S,
     +       Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),ViewMatrix(3,3),
     +       MOL(Max_Mols,4),ISpecified_Bond_Params(30,3)
      COMMON /com_CONTOURS/UN,VN,WN,U0,V0,W0,W,ZPL,QPERS,DPL,DEY,DEYC,
     +       QX,QY,XC,YC,ZMAX
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Data statements ..
      DATA IFIRST/0/
C     ..
C
C
      AQ11 = AQ(1,1)
      AQ21 = AQ(2,1)
      AQ31 = AQ(3,1)
      AQ12 = AQ(1,2)
      AQ22 = AQ(2,2)
      AQ32 = AQ(3,2)
      AQ13 = AQ(1,3)
      AQ23 = AQ(2,3)
      AQ33 = AQ(3,3)
C
C
      J = 0
      K = 0
C
C---- Is 1st vector a draw?
C
      IF (PSBITS(1)) THEN
C
C----   Yes so pick up saved last point unless 1st entry
C
        IF (IFIRST.NE.0) THEN
          J = J + 1
          X(J) = X1
          Y(J) = Y1
        END IF
      END IF
C
C
      IFIRST = 1
C
C---- Transform vectors to plotter coordinates
C
   10 K = K + 1
      U = (VECTOR(1,K)+U0)/UN
      V = (VECTOR(2,K)+V0)/VN
      W = (VECTOR(3,K)+W0)/WN
      XXP = AQ11*U + AQ12*V + AQ13*W
      YYP = AQ21*U + AQ22*V + AQ23*W
      ZZP = AQ31*U + AQ32*V + AQ33*W
C
C---- shift to centre
C
      XXP = (XXP-XC)*S
      YYP = (YYP-YC)*S
      ZZP = (ZZP-ZMAX)*S
C
C
      IF (LStereo_Flag.GT.0) THEN
C
C---- stereo
C
        QPERS = DPL/ (DPL-ZZP)
        XXP = (XXP+DEYC)*QPERS - DEYC
        YYP = YYP*QPERS
C
C---- mono
C
      ELSE IF (Lperspective_Flag.NE.0) THEN
C
C---- perspective
C
        QPERS = DPL/ (DPL-ZZP)
        XXP = XXP*QPERS
        YYP = YYP*QPERS
      END IF
C
C
      J = J + 1
C
C
      IF (J.GT.MAXLIN) THEN
        J = J - 1
C
C            ******************
        CALL drawlineset(X,Y,J)
C            ******************
C
        X(1) = X(J)
        Y(1) = Y(J)
        J = 2
      END IF
C
C
      X(J) = XXP + QX
      Y(J) = YYP + QY
C
C---- Is next vector a draw?
C
      IF (K.LT.LIST) THEN
        IF (PSBITS(K+1)) GO TO 10
      END IF
C
C---- Yes or end-of-list, plot connected lines
C
C          ******************
      CALL drawlineset(X,Y,J)
C          ******************
C
      IF (K.LT.LIST) THEN
        J = 0
        GO TO 10
      END IF
C
C---- Save last vector
C
      X1 = X(J)
      Y1 = Y(J)
      RETURN
      END
C
C
C
C     ==================================
      SUBROUTINE plotmaps(MAP,XMIN,XMAX)
C     ==================================
C
C
C
C     plot map number map (1 or 2)
C     section numbers ksec1 to ksec2
C
C
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Map_Size2
      PARAMETER (Map_Size2=2*Map_Size)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER MAX_FACE
      PARAMETER (MAX_FACE=50000)
      INTEGER MAX_LST
      PARAMETER (MAX_LST=1000)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
C     ..
C     .. Scalar Arguments ..
      REAL XMAX,XMIN
      INTEGER MAP
C     ..
C     .. Scalars in Common ..
      REAL Charcter_Size,DEY,DEYC,DPL,Drawing_Size,Perspective_Distance,
     +     Picture_Scale,QPERS,QX,QY,S,Shadelight_X,Shadelight_Y,
     +     Stereo_Separation,U0,Ugrid_Offset,Ugrid_Spacing,UN,V0,
     +     Vgrid_Offset,Vgrid_Spacing,VN,W,W0,WN,XC,YC,ZMAX,ZPL
      INTEGER Invert_Map_Flag,KSEC1,KSEC2,Lcurrent_Colour,
     +        Ldraw_cell_Flag,LGRID,LimitsSet_Flag,Lmap3d_Flag,
     +        Lmap_ModeFirst,Lpaper_Setting,Lperspective_Flag,
     +        Lplot_colour_Stereo,Lposition_Flag,Lprint_Flag,
     +        LSetView_Flag,LShade_Flag,LSolid_Flag,LStereo_Flag,
     +        LSuperimpose_Flag,LTaper_Flag,LTrace_Flag,Map_Counter,
     +        Map_Flag_Counter,Map_Nobox,Map_pts_U,Map_pts_V,Msecst,MU,
     +        MV,MW,Natoms_plusCell,Ncopies,NFirst_Section,
     +        NLast_Section,Npicture,Npts_U_compressed,
     +        Npts_V_compressed,Nsections_in_Group,Nspecified_Bonds,
     +        NstpMapGrpSect,Num_Moles
      LOGICAL Log_Draw_Title
C     ..
C     .. Arrays in Common ..
      REAL AQ,Contour_Levels,Contour_Threshold,Plotting_XYZ,RP,
     +     ViewMatrix
      INTEGER ISpecified_Bond_Params,Major_Grid,MapGrdCol,
     +        Map_def_Restrictions,Map_LineTypes,Map_UVW_Limits,
     +        Map_XYZ_Limits,MOL,NSECC,Num_Contours
      CHARACTER Map_Title*80
C     ..
C     .. Local Scalars ..
      REAL CHHT,XX,YY,ZZ
      INTEGER I,I1,IEND,IPTMAX,ISEC,J,K,LIST,LORIGN,MUVPNT,MWPNT,N,
     +        NUMMAP_POINTS,N1,N2,N3,NSECT
      CHARACTER LSTRNG*80,DOTS*3
C     ..
C     .. Local Arrays ..
      REAL RHO(Map_Size),VECTOR(3,MAX_LST),X(200),Y(200)
      INTEGER LLIM(3),MACS(3),LPERM
      LOGICAL*1 BITS(Map_Size2),FLAGS(4,MAX_FACE),PSBITS(MAX_LST)
C     ..
C     .. External Subroutines ..
      EXTERNAL brickcontour,CCPERR,contourstyle,drawgrid,getcontours,
     +         getmapsection,joincoordpairs,plotcontour,pltmapaxes,
     +         setdashparams,zplotstring,zsetpencolour,zsettextposition
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC REAL
C     ..
C     .. Common blocks ..
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,Picture_Scale,
     +       Perspective_Distance,Shadelight_X,Shadelight_Y,S,
     +       Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),ViewMatrix(3,3),
     +       MOL(Max_Mols,4),ISpecified_Bond_Params(30,3)
      COMMON /com_CONTOURS/UN,VN,WN,U0,V0,W0,W,ZPL,QPERS,DPL,DEY,DEYC,
     +       QX,QY,XC,YC,ZMAX
      COMMON /com_GRID/LGRID,Map_Nobox,Ugrid_Spacing,Vgrid_Spacing,
     +       Ugrid_Offset,Vgrid_Offset,Major_Grid(Max_Maps),
     +       MapGrdCol(4,Max_Maps,2)
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
      COMMON /com_SECTIONS/NFirst_Section,NLast_Section,
     +       Nsections_in_Group,NstpMapGrpSect,KSEC1,KSEC2,Msecst,
     +       NSECC(2)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C---- iptmax is dimension of arrays x,y used in contour routine
C
      IPTMAX = 200
      DOTS = ' ..'
C
C---- check map array size
C
      IF (Lmap3d_Flag.GT.0) GO TO 20
      NUMMAP_POINTS = Map_pts_U*Map_pts_V
      IF (NUMMAP_POINTS.LE.Map_Size) GO TO 30
   10 WRITE (6,FMT=6000) NUMMAP_POINTS,Map_Size
 6000 FORMAT (' Map too large for array : ',2I8,/)
C
C          **************************************
      CALL CCPERR(1,' Map too large for array ')
C          **************************************
C
   20 MWPNT = KSEC2 - KSEC1 + 1
      MUVPNT = Map_pts_U*Map_pts_V
      NUMMAP_POINTS = Npts_U_compressed*Npts_V_compressed*MWPNT
      IF (NUMMAP_POINTS.GT.Map_Size) GO TO 10
C
C---- Check face sizes
C
      IF (Npts_U_compressed*Npts_V_compressed.GT.MAX_FACE .OR.
     +    Npts_V_compressed*MWPNT.GT.MAX_FACE .OR.
     +    MWPNT*Npts_U_compressed.GT.MAX_FACE) THEN
        WRITE (6,FMT=6002) MAX_FACE
 6002   FORMAT (/' "Brick" face larger than',I8,/)
C
C            ***********************************
        CALL CCPERR(1,' "Brick" face TOO LARGE')
C            ***********************************
C
      END IF
C
C---- set up parameters for coordinate transformation
C     from grid coordinates to plotter coordinates
C     in subroutine joincoordpairs
C
   30 UN = Map_XYZ_Limits(MU)
      VN = Map_XYZ_Limits(MV)
      WN = Map_XYZ_Limits(MW)
      U0 = Map_def_Restrictions(1)
      V0 = Map_def_Restrictions(3)
C
C---- map number
C
      Map_Counter = MAP
      W0 = KSEC1
      QPERS = 1.0
C
C
      IF (LStereo_Flag-1.LT.0) THEN
        GO TO 50
      ELSE IF (LStereo_Flag-1.GT.0) THEN
        GO TO 40
      END IF
C
C---- stereo left
C
      DEYC = DEY
      GO TO 50
C
C---- stereo right
C
   40 DEYC = -DEY
   50 IF (KSEC2 .GT. KSEC1) WRITE(6,6003)KSEC1,KSEC2,Msecst-1
 6003 FORMAT(' First section',I4,' to last section',I4,/,7X,' with',
     . I4,' sections excluded before next one is plotted in stack')
      IF (Lmap3d_Flag.GT.0) GO TO 90
C
C
      DO 80 ISEC = KSEC1,KSEC2,Msecst
C
C---- If MODE FIRST set,
C        set Lmap_ModeFirst = 2 for first section else 1
C
        IF (Lmap_ModeFirst.GT.0) THEN
          Lmap_ModeFirst = 1
          IF (ISEC.EQ.KSEC1) Lmap_ModeFirst = 2
        END IF
C
C---- z coordinate of section
C
        W = REAL(ISEC)/WN
C
C---- position map file to start of section, read it
C     & compress if necessary
C
C            **********************************************
        CALL getmapsection(MAP,ISEC,RHO,NUMMAP_POINTS,IEND)
C            **********************************************
C
        IF (IEND.NE.0) GO TO 190
C
C---- perspective or stereo parameters
C
        IF (Map_Nobox.EQ.0) THEN
C
C---- set up corners
C
          DO 60 I = 1,5
            X(I) = 0.0
            Y(I) = 0.0
   60     CONTINUE
          DO 70 I = 1,2
            X(I+2) = Npts_U_compressed - 1
            Y(I+1) = Npts_V_compressed - 1
   70     CONTINUE
C
C---- plot box round sections
C
C              *********************
          CALL joincoordpairs(X,Y,5)
C              *********************
C
        END IF
C
C---- plot grid if required
C
C                                           *************
        IF (LGRID.NE.0 .AND. ISEC.EQ.KSEC1) CALL drawgrid
C                                           *************
C
C
C---- plot contours
C
C    RHO(Npts_U_compressed,Npts_V_compressed)
C       Contains map for contouring,
C    BITS is scratch array,
C    Contour_Levels Contains Num_Contours()
C    CONTOUR LEVELS, X(IPTMAX),Y(IPTMAX) are used
C        to store end points of lines
C
C            ************************************************
        CALL getcontours(RHO,Npts_U_compressed,Npts_V_compressed,BITS,
     +                   Contour_Levels(1,MAP),Num_Contours(MAP),X,Y,
     +                   IPTMAX)
C            ************************************************
C
C---- restore plotting style
C
C            ******************
        CALL setdashparams(0,0.0,0.0,0.0)
        CALL zsetpencolour(Lcurrent_Colour)
C            ******************
C
   80 CONTINUE
C
C
      GO TO 190
C
C---- 3D contouring
C
C       Read in whole slab of sections
C
   90 CONTINUE
C
C
      DO 100 ISEC = KSEC1,KSEC2,Msecst
        J = (ISEC-KSEC1)*Npts_U_compressed*Npts_V_compressed + 1
C
C            ******************************************
        CALL getmapsection(MAP,ISEC,RHO(J),MUVPNT,IEND)
C            *******************************************
C
        IF (IEND.NE.0) GO TO 110
  100 CONTINUE
C
C
      GO TO 120
  110 KSEC2 = ISEC
      MWPNT = KSEC2 - KSEC1 + 1
C
C---- Set up things for brickcontour
C
  120 MACS(1) = Npts_U_compressed
      MACS(2) = Npts_V_compressed
      MACS(3) = MWPNT
      LLIM(1) = 0
      LLIM(2) = 0
      LLIM(3) = 0
      LORIGN = 1
C
C---- Loop over contour levels
C
      DO 130 N = 1,Num_Contours(MAP)
C
C            ***********************************
        CALL contourstyle(Contour_Levels(N,MAP))
C            ***********************************
C
C---- Contour whle map as one brick
C
C            ***********************************************
        CALL brickcontour(RHO,FLAGS,Contour_Levels(N,MAP),MACS,MACS,
     +                    LLIM,LORIGN,VECTOR,PSBITS,LIST,MAX_LST)
C            ***********************************************
C
C---- Plot last bit
C
C                      ************************************
        IF (LIST.GT.0) CALL plotcontour(VECTOR,PSBITS,LIST)
C                      ************************************
C
  130 CONTINUE
C
C---- restore plotting style
C
C          ******************************
      CALL setdashparams(0,0.0,0.0,0.0)
      CALL zsetpencolour(Lcurrent_Colour)
C          *******************************
C
      IF (Map_Nobox.EQ.0) THEN
C
C---- Plot corners of box
C
        I = 0
        ZZ = 0.0
C
C
        DO 160 K = 1,2
          I1 = (K-1)*5 + 1
          DO 140 I = I1,I1 + 4
            VECTOR(1,I) = 0.0
            VECTOR(2,I) = 0.0
            VECTOR(3,I) = ZZ
            PSBITS(I) = .TRUE.
  140     CONTINUE
C
C
          PSBITS(I1) = .FALSE.
C
C
          DO 150 I = I1,I1 + 1
            VECTOR(1,I+2) = Npts_U_compressed - 1
            VECTOR(2,I+1) = Npts_V_compressed - 1
  150     CONTINUE
C
C
          ZZ = MWPNT - 1
  160   CONTINUE
        J = 11
C
C
        DO 180 K = 1,4
          DO 170 I = 1,3
            VECTOR(I,J) = VECTOR(I,K)
            VECTOR(I,J+1) = VECTOR(I,K+5)
  170     CONTINUE
C
C
          PSBITS(J) = .FALSE.
          PSBITS(J+1) = .TRUE.
          J = J + 2
  180   CONTINUE
C
C            *****************************
        CALL plotcontour(VECTOR,PSBITS,18)
C            *****************************
C
      END IF
C
C---- plot scale and section numbers if 1st map
C     no titles on 2nd red/green stereo stack
C     annotate axes
C
  190 CONTINUE
C
C---- No axis labelling if non-"standard" view
C       (too complicated to work out)
C
C                               ****************
        IF (LSetView_Flag.LT.2) CALL pltmapaxes
C                               ****************
C
      IF (Log_Draw_Title) THEN
        W = KSEC2/WN
        IF (Lplot_colour_Stereo.EQ.-1 .AND. LStereo_Flag.EQ.2) RETURN
        IF (MAP.EQ.2) GO TO 210
        CHHT = Charcter_Size*2.0
        YY = Map_Flag_Counter*CHHT + Charcter_Size
        XX = (XMAX+XMIN)/2.0
C
C            ***********************
        CALL zsettextposition(XX,YY)
C            ***********************
C

        IF (Nsections_in_Group.GT.1) GO TO 200
C keywrd_uvw(3) equivalent to MW in SUBROUTINE akwmapdat
C        FRCSEC = KSEC1/MAP_XYZ_limits(keywrd_uvw(3))
        FRCSEC = FLOAT(KSEC1)/FLOAT(MAP_XYZ_limits(MW))
        WRITE (LSTRNG,FMT=6004) S,KSEC1,FRCSEC
 6004   FORMAT (' Scale =',F12.4,' mm/A',10X,'Section',I5,'(',F6.2,')')
C
         KZMODE = 2
C            **************************
        CALL zplotstring(KZMODE,LSTRNG,1,59)
C            **************************
C
        GO TO 210
C
C
  200   IF (Msecst .LE. 1) THEN
          WRITE (LSTRNG,FMT=6006) S,KSEC1,KSEC2
        ELSE
          NSECT = (KSEC2-KSEC1)/Msecst + 1
          IF (NSECT.EQ.1) THEN
            WRITE (LSTRNG,FMT=6004) S,KSEC1
          ELSE
            IF (NSECT.EQ.2) THEN
              N2 = KSEC1 + Msecst
              WRITE (LSTRNG,FMT=6008) S,KSEC1,N2
            ELSE
              N1 = KSEC1 + Msecst
              N2 = N1 + Msecst
              N3 = KSEC1 + Msecst*(NSECT-1)
              IF (N3 .EQ. N2) THEN
                WRITE (LSTRNG,FMT=6010) S,KSEC1,N1,N2
              ELSE
                WRITE (LSTRNG,FMT=6010) S,KSEC1,N1,N2,DOTS,N3
              ENDIF
            ENDIF
          ENDIF
        ENDIF
C
          KZMODE = 2
C            **************************
        CALL zplotstring(KZMODE,LSTRNG,1,59)
C            **************************
C
 6006   FORMAT (' Scale    = ',F12.4,' mm/A Sections = ',I5,'  to ',I5)
 6008   FORMAT (' Scale    = ',F12.4,' mm/A Sections = ',I5,' and ',I5)
 6010   FORMAT (' Scale = ',F10.2,' mm/A Sections = ',I5,',',I4,',',
     .                                 I4,A,I5)
  210   YY = (Map_Flag_Counter-MAP)*CHHT + Charcter_Size
C
C            ***********************
        CALL zsettextposition(XX,YY)
C            ***********************
C
        LSTRNG = Map_Title(MAP) (1:76)
C
          KZMODE = 2
C            **************************
        CALL zplotstring(KZMODE,LSTRNG,1,76)
C            **************************
C
      END IF
C
      RETURN
      END
C
C
C
C     ========================
      SUBROUTINE plotmolecules
C     ========================
C
C
C     Plot calls changed April 1988
C
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_BndLabels
      PARAMETER (Max_BndLabels=10)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Colours
      PARAMETER (Max_Colours=10)
      INTEGER Max_Conics
      PARAMETER (Max_Conics=20)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER Max_Labels
      PARAMETER (Max_Labels=100)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER Max_Quadrangles
      PARAMETER (Max_Quadrangles=30)
      INTEGER Max_Segments
      PARAMETER (Max_Segments=50)
      INTEGER Max_Symmetry
      PARAMETER (Max_Symmetry=192)
      INTEGER NDefault_Colour
      PARAMETER (NDefault_Colour=1)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Cent_Xmax,Cent_Xmin,
     +     Cent_Ymax,Cent_Ymin,Cent_Zmax,Cent_Zmin,Charcter_Size,
     +     CurrentCharcter_Size,D001,D010,D100,DEY,DEYC,
     +     Dist_to_Boundary,DPL,Drawing_Size,Ellipse_ScaleFact,
     +     Eye_Distance,RFR(4,4),ROR(4,4),
     +     Overlap_Margin,Paper_Length,
     +     Paper_Width,Perspective_Distance,Picture_Scale,Plotter_Step,
     +     QPERS,QX,QY,Retrace_Displmnt,Rmax_Save,S,Scal_pre_Proj,SCL,
     +     Shadelight_X,Shadelight_Y,Stereo_Separation,U0,UN,V0,VN,W,W0,
     +     WN,XC,Xmax_Save,Xmin_Save,YC,Ymax_Save,Ymin_Save,ZMAX,
     +     Zmax_Save,Zmin_Save,ZPL
      INTEGER IERR,Invert_Map_Flag,Iuse_persp_Labs,Label_atoms_Flag,
     +        Lcurrent_Colour,Ldraw_cell_Flag,LimitsSet_Flag,Ljoin_Flag,
     +        Lmap3d_Flag,Lmap_ModeFirst,Lorthogonalisation_Code,
     +        Lpack_Flag,Lpaper_Setting,Lperspective_Flag,
     +        Lplot_colour_Stereo,Lposition_Flag,Lprint_Flag,
     +        Lsaved_Paper_Setting,LSetView_Flag,LShade_Flag,
     +        LSolid_Flag,LStereo_Flag,LSuperimpose_Flag,LTaper_Flag,
     +        LTrace_Flag,Map_Counter,Map_Flag_Counter,Map_pts_U,
     +        Map_pts_V,MU,MV,MW,NAT,NATO,Natoms_plusCell,NBondLabels,
     +        NC,NC1,Ncopies,NDrawing_Number,NFont_Number,
     +        NPen_Thickness,Npicture,Npts_U_compressed,
     +        Npts_V_compressed,NQuads_Overlaped,Nspecified_Bonds,
     +        NSymmetry_Ops,Num_Bnd_Quadrangles,Num_Conics,
     +        Num_Labtxt_input,Num_Moles,Num_Overlaps,Num_Posit_Labels
      LOGICAL LCent_Flag,Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
      CHARACTER Iden_Title*80
C     ..
C     .. Arrays in Common ..
      REAL AID,AQ,Atom_Radii,Base_Vect_Refsys,Base_Vect_Wrksys,BBB,
     +     Contour_Levels,Contour_Threshold,Drawing_Origin,
     +     Ellipse_Conics,Ellipse_Resolution,Fractional_XYZ,
     +     G_Metric_Tensor,OrtCell,OrtConts,Orthogonal_XYZ,
     +     Overlap_Conics,P1_to_Refsys,P1_to_Wrksys,Parr_Offset,
     +     Perp_Offset,Plotting_XYZ,Positioned_Labels,
     +     Principal_axis_rms,Prin_Angstrom_Vectors,Prin_Axis_Col_Vects,
     +     Q,Quadrangles,Quads_Overlaped,Recip_Metric_Tensor,
     +     Rotsym_Matrx,RP,Segments,Sorted_xyz,Title_Rotation,
     +     Title_RotnTrans,TransSym_Vect,ViewMatrix,XABC,XCLANG,XLABEL,
     +     XO,YLABEL
      INTEGER IAtom_Connectivity,IAtom_Type,ISpecified_Bond_Params,KC,
     +        KQ,KSortPointer,LAtom_Inc_Flag,LAtom_Label_Flag,
     +        Map_def_Restrictions,Map_LineTypes,Map_UVW_Limits,
     +        Map_XYZ_Limits,MOL,Num_Contours
      CHARACTER Atm_Lab_Type*2,Atm_Identifier*10,Atm_Label*10,
     +          Atm_Lab_Txt*10,BondtoLabel*10,Map_Title*80
C     ..
C     .. Local Scalars ..
      REAL ACELL,ALEN,ALF,BCELL,BET,CCELL,D,GAM,PMAX,PMIN,QXQ,R11,R12,
     +     R13,R21,R22,R23,R31,R32,R33,RMAX,SIZEA,SIZEB,SIZEN,SMUL2,
     +     SMULT,T1,T2,TEMPSORT,TX,TY,TZ,XF,XLABL,XMAX,XMIN,XOR,XR,XX,
     +     YF,YLABL,YMAX,YMIN,YOR,YR,YSTART,YY,ZC,ZF,ZMIN,ZOR,ZR,ZZ
      INTEGER I,I1,IBOX,IMSORT,ISORT,J,JDO,JSORT,KKKSORT,KSORT,LSORT,M,
     +        MAP,MSORT,N,NAP,NCOPY_Counter,NTOT,NTRACE_Counter,LPERM
      CHARACTER LSTRNG*80
C     ..
C     .. Local Arrays ..
      REAL SCAL(2),V3(3)
      CHARACTER LRLAB(2)*1
C     ..
C     .. External Functions ..
      REAL vectbyvecttranspose
      EXTERNAL vectbyvecttranspose
C     ..
C     .. External Subroutines ..
      EXTERNAL drawsolidatoms,getatmlabelposn,plotbondlength,plotmaps,
     +         setdashparams,setupbondoverlap,storeoverlap,
     +         zadvanceframe,zdrawto,zinitplotter,zmoveto,zplotstring,
     +         zsetcharactersize,zsetfont,zsetpaperorientation,
     +         zsetpencolour,zsetpenthickness,zsettextposition
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC AINT,MAX,MIN,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,Picture_Scale,
     +       Perspective_Distance,Shadelight_X,Shadelight_Y,S,
     +       Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),ViewMatrix(3,3),
     +       MOL(Max_Mols,4),ISpecified_Bond_Params(30,3)
      COMMON /com_BNDLAB/NBondLabels,Perp_Offset(Max_BndLabels),
     +       Parr_Offset(Max_BndLabels)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CENTRE/LCent_Flag,Cent_Xmax,Cent_Ymax,Cent_Zmax,
     +       Cent_Xmin,Cent_Ymin,Cent_Zmin
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_CONTOURS/UN,VN,WN,U0,V0,W0,W,ZPL,QPERS,DPL,DEY,DEYC,
     +       QX,QY,XC,YC,ZMAX
      COMMON /com_FONT/NFont_Number,NPen_Thickness
      COMMON /com_LABBND/BondtoLabel(Max_BndLabels*2)
      COMMON /com_LABCHR/Atm_Lab_Txt(Max_Labels),
     +       Atm_Lab_Type(Max_Labels),Atm_Label(Max_Labels),
     +       Atm_Identifier(Max_Atoms),Iden_Title
      COMMON /com_LABELS/Num_Posit_Labels,Num_Labtxt_input,
     +       XLABEL(Max_Atoms),YLABEL(Max_Atoms),
     +       Positioned_Labels(2,Max_Labels)
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
      COMMON /com_OVERLAP/Num_Conics,Num_Overlaps,NQuads_Overlaped,
     +       Num_Bnd_Quadrangles,KC(Max_Conics),KQ(Max_Quadrangles),
     +       Ellipse_Conics(7,Max_Atoms),Overlap_Conics(6,Max_Conics),
     +       Quads_Overlaped(3,4,Max_Quadrangles),
     +       Quadrangles(9,2*Max_Atoms),Segments(Max_Segments,2)
      COMMON /com_SAVESCAL/Xmin_Save,Ymin_Save,Zmin_Save,Xmax_Save,
     +       Ymax_Save,Zmax_Save,Rmax_Save,Lsaved_Paper_Setting
      COMMON /com_SYMMETRY/NSymmetry_Ops,Rotsym_Matrx(3,3,Max_Symmetry),
     +       TransSym_Vect(3,Max_Symmetry)
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Data statements ..
      DATA LRLAB/'L','R'/
C     ..
C
C
      ACELL = XABC(1)
      BCELL = XABC(2)
      CCELL = XABC(3)
      ALF = XCLANG(1)
      BET = XCLANG(2)
      GAM = XCLANG(3)
C
C
      NDrawing_Number = NDrawing_Number + 1
      NTRACE_Counter = 0
      NCOPY_Counter = 0
   10 NAP = 0
      PMAX = -1.0E10
      PMIN = 1.0E10
      XMAX = PMAX
      YMAX = PMAX
      XMIN = PMIN
      YMIN = PMIN
      ZMAX = -1000.0
      ZMIN = 1000.0
      IF (NDrawing_Number.LE.1) WRITE (6,
     +    FMT=6000) ((ViewMatrix(I,J),J=1,3),I=1,3)
 6000 FORMAT (' View Rotation Matrix',3 (/10X,3F10.5))
      R11 = ViewMatrix(1,1)
      R12 = ViewMatrix(1,2)
      R13 = ViewMatrix(1,3)
      R21 = ViewMatrix(2,1)
      R22 = ViewMatrix(2,2)
      R23 = ViewMatrix(2,3)
      R31 = ViewMatrix(3,1)
      R32 = ViewMatrix(3,2)
      R33 = ViewMatrix(3,3)
      NAP = 0
      NATO = NAT
C
C
      IF (Num_Moles.LE.0) THEN
        MOL(1,1) = 1
        MOL(1,2) = 0
        MOL(1,3) = 0
        MOL(1,4) = 0
        Num_Moles = 1
      END IF
C
C
      DO 60 M = 1,Num_Moles
        N = MOL(M,1)
        TX = MOL(M,2)
        TY = MOL(M,3)
        TZ = MOL(M,4)
        I1 = 9
        IF (M.EQ.1) I1 = 1
C
C
        DO 50 I = I1,NAT
C
C---- generate  fractional coords first   xf,yf,zf
C
          IF (I.LT.9) THEN
            XF = Fractional_XYZ(I,1)
            YF = Fractional_XYZ(I,2)
            ZF = Fractional_XYZ(I,3)
            GO TO 20
          END IF
C
C---- if default molecule,
C     we already have orthogonal coordinates in Orthogonal_XYZ
C
          IF (M.LE.1) THEN
            XOR = Orthogonal_XYZ(I,1)
            YOR = Orthogonal_XYZ(I,2)
            ZOR = Orthogonal_XYZ(I,3)
            GO TO 30
          END IF
C
C
          XX = Fractional_XYZ(I,1)
          YY = Fractional_XYZ(I,2)
          ZZ = Fractional_XYZ(I,3)
C
C
          ZF = Rotsym_Matrx(3,1,N)*XX + Rotsym_Matrx(3,2,N)*YY +
     +         Rotsym_Matrx(3,3,N)*ZZ + TransSym_Vect(3,N) + TZ
          YF = Rotsym_Matrx(2,1,N)*XX + Rotsym_Matrx(2,2,N)*YY +
     +         Rotsym_Matrx(2,3,N)*ZZ + TransSym_Vect(2,N) + TY
          XF = Rotsym_Matrx(1,1,N)*XX + Rotsym_Matrx(1,2,N)*YY +
     +         Rotsym_Matrx(1,3,N)*ZZ + TransSym_Vect(1,N) + TX
C
C---- make orthonormal
C
c   20     XOR = OR1*XF + OR2*YF + OR3*ZF
c          YOR = OR4*XF + OR5*YF + OR6*ZF
c          ZOR = OR7*XF + OR8*YF + OR9*ZF
   20     XOR = ROR(1,1)*XF + ROR(1,2)*YF + ROR(1,3)*ZF
          YOR = ROR(2,1)*XF + ROR(2,2)*YF + ROR(2,3)*ZF
          ZOR = ROR(3,1)*XF + ROR(3,2)*YF + ROR(3,3)*ZF
C
C---- rotate the orthonormal  coords
C
   30     XR = R11*XOR + R12*YOR + R13*ZOR
          YR = R21*XOR + R22*YOR + R23*ZOR
          ZR = R31*XOR + R32*YOR + R33*ZOR
          IF (LSuperimpose_Flag.GT.1) GO TO 40
          IF (Ldraw_cell_Flag.EQ.0 .AND. I.LT.9) GO TO 40
          IF (I.GT.9 .AND. LAtom_Inc_Flag(I).LE.0) GO TO 40
          XMAX = MAX(XR,XMAX)
          YMAX = MAX(YR,YMAX)
          ZMAX = MAX(ZR,ZMAX)
          XMIN = MIN(XR,XMIN)
          YMIN = MIN(YR,YMIN)
          ZMIN = MIN(ZR,ZMIN)
   40     NAP = NAP + 1
C
C
          IF (NAP.GT.Max_Atoms) THEN
            WRITE (6,FMT=6002) Max_Atoms
 6002       FORMAT (' ***TOO Many atom coordinates  for packing diagra',
     +             'm.',/'         maximum = ',I5,' atoms')
            RETURN
          END IF
C
C
          Plotting_XYZ(1,NAP) = XR
          Plotting_XYZ(2,NAP) = YR
          Plotting_XYZ(3,NAP) = ZR
          RP(NAP) = Atom_Radii(I)
   50   CONTINUE
   60 CONTINUE
C
C
      IF (LCent_Flag) THEN
C
C----  Limits have been read in as CENtre command,
C      reset XMIN,...,ZMAX
C
        XMAX = Cent_Xmax
        YMAX = Cent_Ymax
        ZMAX = Cent_Zmax
        XMIN = Cent_Xmin
        YMIN = Cent_Ymin
        ZMIN = Cent_Zmin
      END IF
C
C
      IF (LSuperimpose_Flag.LE.1) THEN
        Xmax_Save = XMAX
        Ymax_Save = YMAX
        Zmax_Save = ZMAX
        Xmin_Save = XMIN
        Ymin_Save = YMIN
        Zmin_Save = ZMIN
      END IF
C
C
      Natoms_plusCell = NAP
      NTOT = Natoms_plusCell - 8
      IF (NDrawing_Number.LE.1) WRITE (6,FMT=6004) NTOT
 6004 FORMAT (' Total number of atoms to be plotted = ',I5)
C
C----  get all coords referred to centre of the plot as origin.
C
      IF (LSuperimpose_Flag.GT.1) THEN
        XMAX = Xmax_Save
        YMAX = Ymax_Save
        ZMAX = Zmax_Save
        XMIN = Xmin_Save
        YMIN = Ymin_Save
        ZMIN = Zmin_Save
      END IF
C
C---- calculate centre
C
      XC = (XMIN+XMAX)/2.0
      YC = (YMIN+YMAX)/2.0
      ZC = (ZMIN+ZMAX)/2.0
C
C---- debug only
C
      IF (Lprint_Flag.GT.0) THEN
        WRITE (6,FMT=6006) LSuperimpose_Flag,XMAX,YMAX,ZMAX,XMIN,YMIN,
     +    ZMIN
 6006   FORMAT (' LSuperimpose_Flag,XMAX,YMAX,ZMAX,XMIN,YMIN,ZMIN',/,1X,
     +         I4,1X,F20.5,1X,F20.5,/,6X,F20.5,1X,F20.5,/6X,F20.5,1X,
     +         F20.5)
        WRITE (6,FMT=6008) LSuperimpose_Flag,XC,YC,ZC,ZMAX
 6008   FORMAT (' LSuperimpose_Flag XC YC ZC ZMAX ',/,' ',I5,F10.5,1X,
     +         F10.5,/,' ',F10.5,1X,F10.5,///)
      END IF
C
C
      XMAX = XMAX - XC
      XMIN = XMIN - XC
      YMAX = YMAX - YC
      YMIN = YMIN - YC
C
C
      DO 70 I = 1,Natoms_plusCell
        Plotting_XYZ(1,I) = Plotting_XYZ(1,I) - XC
        Plotting_XYZ(2,I) = Plotting_XYZ(2,I) - YC
        Plotting_XYZ(3,I) = Plotting_XYZ(3,I) - ZMAX
   70 CONTINUE
C
C---- get max and min for x boundary across page
C     allowing a safty margin.
C
C----  if LSuperimpose_Flag gt 1
C      do not reset s Drawing_Size pmax pmin etc.
C
      RMAX = Rmax_Save
C
C
      IF (LSuperimpose_Flag.LE.1) THEN
        RMAX = 0.0
C
C
        DO 80 I = 1,NAT
          IF (LAtom_Inc_Flag(I).EQ.0 .AND. I.GE.9 .AND.
     +        I.LE.NATO) LAtom_Label_Flag(I) = 0
          IF (LAtom_Inc_Flag(I).EQ.0) GO TO 80
          RMAX = MAX(RMAX,Atom_Radii(I))
   80   CONTINUE
C
C
        Rmax_Save = RMAX
      END IF
C
C
      SIZEA = XMAX - XMIN + 2.0*RMAX
      SIZEB = YMAX - YMIN + 2.0*RMAX
C
C
      IF (Picture_Scale.GE.0.001) THEN
C
C---- apply scale factor given .  ( Picture_Scale mm/angstrom)
C
        Drawing_Size = SIZEA*Picture_Scale
        SIZEN = SIZEB*Picture_Scale
        S = Picture_Scale
        IBOX = 0
        GO TO 90
      END IF
C
C---- calculate scale factor to fit
C     drawing along x or y side of Drawing_Size mm.
C
      IBOX = 1
      S = Drawing_Size/SIZEA
      SIZEN = SIZEB*S
C
C
      IF (Lpaper_Setting.NE.0) THEN
C
C----  yacross  scale on y
C
        S = Drawing_Size/SIZEB
        SIZEN = Drawing_Size
        Drawing_Size = S*SIZEA
      END IF
C
C---- scale down by factor 0.9 to leave margin if no map
C
      IF (Map_Flag_Counter.LE.0) S = S*0.9
   90 DPL = Perspective_Distance
      IF (LStereo_Flag.EQ.0 .AND. Lperspective_Flag.EQ.0) DPL = 10000.0
      DEY = Eye_Distance
C
C---- set stereo separation
C
      IF (Stereo_Separation.LT.1.0) Stereo_Separation = Drawing_Size
      IF (Lplot_colour_Stereo.EQ.-1 .OR.
     +    LStereo_Flag.EQ.0) Stereo_Separation = 0.0
C
C---- Calculate paper length for each picture in mm,
C     allowing for stereo pair  if Yacross
C
      IF (Lpaper_Setting.NE.0) THEN
C
C---- y across
C
        ALEN = Drawing_Size + 24.0
        IF (LStereo_Flag.GT.0) ALEN = ALEN + Stereo_Separation
        GO TO 100
      END IF
C
C---- x across
C
      ALEN = SIZEN + 28.0
  100 IF (Picture_Scale.GT.0.0001) GO TO 110
C
C---- Reduce scale if necessary and not explicitly given
C
      IF (ALEN.GE.Paper_Length) THEN
        QXQ = Paper_Length/ALEN
        S = S*QXQ
        Drawing_Size = Drawing_Size*QXQ
        SIZEN = SIZEN*QXQ
        Stereo_Separation = Stereo_Separation*QXQ
        ALEN = Paper_Length
      END IF
C
C---- save scale to stop rescaling next time
C
  110 Picture_Scale = S
C
C---- allocate space on y for titles, etc, 4mm each
C
      IF (Map_Flag_Counter.GT.0) IBOX = 0
      YSTART = 2.0
      IF (Map_Flag_Counter.GT.0) YSTART = (Map_Flag_Counter+1)*2.0*
     +    Charcter_Size + Charcter_Size
      YMIN = YSTART
      YMAX = SIZEN + YSTART
      IF (NDrawing_Number.LE.1) WRITE (6,FMT=6010) SIZEA,Drawing_Size,
     +    S,DPL
 6010 FORMAT (' ***Length of plotted X side = ',F8.2,' Angstroms',/'  ',
     +       '                             ',F8.2,' mm.',/'           ',
     +       '     Scale Factor = ',F8.4,' mm/A',/'        Perspective',
     +       ' Distance = ',F10.1,' mm')
      Lcurrent_Colour = Lplot_colour_Stereo
      IF (Lcurrent_Colour.LT.1 .OR. Lcurrent_Colour.GT.
     +    Max_Colours) Lcurrent_Colour = NDefault_Colour
C
C----  clear dash parameters
C
C          ****************************
      CALL setdashparams(0,0.0,0.0,0.0)
C          ****************************
C
C---- convert coords to mm in range 0 - Drawing_Size
C
      DO 120 I = 1,Natoms_plusCell
        Plotting_XYZ(1,I) = Plotting_XYZ(1,I)*S
        Plotting_XYZ(2,I) = Plotting_XYZ(2,I)*S
        Plotting_XYZ(3,I) = Plotting_XYZ(3,I)*S
        RP(I) = RP(I)*S*DPL/ (DPL-Plotting_XYZ(3,I))
        IF (LSolid_Flag.EQ.0) RP(I) = 0.0
  120 CONTINUE
C
C---- get coords for atom labels here.
C
      D = 2.5*S
C
C                   ***********************
      IF (NAT.GT.8) CALL getatmlabelposn(D)
C                   ***********************
C
C---- Plotting sequence begins here
C
  130 CONTINUE
      IF (Lplot_colour_Stereo.EQ.-1 .AND.
     +    LStereo_Flag.EQ.1) Lcurrent_Colour = 4
      IF (Lplot_colour_Stereo.EQ.-1 .AND.
     +    LStereo_Flag.EQ.2) Lcurrent_Colour = 3
C
C
      IF (LStereo_Flag.EQ.0 .AND. Lperspective_Flag.EQ.0) GO TO 170
C
C---- if stereo or perspective required adjust co-ordinates.
C
      DO 160 I = 1,Natoms_plusCell
        QXQ = DPL/ (DPL-Plotting_XYZ(3,I))
        IF (LStereo_Flag.EQ.0) GO TO 150
C
C
        IF (LStereo_Flag.EQ.2) GO TO 140
C
C---- stereo coords for left eye view
C
        Plotting_XYZ(1,I) = (Plotting_XYZ(1,I)+DEY)*QXQ - DEY
        Plotting_XYZ(2,I) = Plotting_XYZ(2,I)*QXQ
        XLABEL(I) = (XLABEL(I)+DEY)*QXQ - DEY
        YLABEL(I) = YLABEL(I)*QXQ
        GO TO 160
C
C---- stereo coords for right eye view
C
  140   Plotting_XYZ(1,I) = (Plotting_XYZ(1,I)-TX+DEY)/QXQ - DEY
        Plotting_XYZ(1,I) = (Plotting_XYZ(1,I)-DEY)*QXQ + DEY
        XLABEL(I) = (XLABEL(I)-TX+DEY)/QXQ - DEY
        XLABEL(I) = (XLABEL(I)-DEY)*QXQ + DEY
        GO TO 160
C
C---- perspective only
C
  150   Plotting_XYZ(1,I) = Plotting_XYZ(1,I)*QXQ
        Plotting_XYZ(2,I) = Plotting_XYZ(2,I)*QXQ
        XLABEL(I) = XLABEL(I)*QXQ
        YLABEL(I) = YLABEL(I)*QXQ
  160 CONTINUE
C
C---- adjust coordinates by origin shift so
C     thar all are in range 0 - Drawing_Size
C
  170 QXQ = Drawing_Size/2.0 + 5.0
      IF (Lpaper_Setting.EQ.0) QXQ = (Paper_Width-Stereo_Separation)/
     +                               2.0 + 1.0
      IF (LStereo_Flag-1.GT.0) GO TO 190
C
C---- For Xacross, centre on page
C
      TX = QXQ
      TY = SIZEN/2.0 + YSTART
      QX = TX
      QY = TY
      GO TO 200
C
  190 TX = QXQ + Stereo_Separation
      TY = 0.0
      QX = TX
      QY = SIZEN/2.0 + YSTART
C
C
  200 CONTINUE
      IF (Lprint_Flag.GT.0) THEN
        WRITE (6,FMT=6012) LSuperimpose_Flag,TX,TY
 6012   FORMAT (' LSuperimpose_Flag TX TY ',I5,2F10.5)
      END IF
C
C
      DO 210 JDO = 1,Natoms_plusCell
        Plotting_XYZ(1,JDO) = Plotting_XYZ(1,JDO) + TX
        Plotting_XYZ(2,JDO) = Plotting_XYZ(2,JDO) + TY
        XLABEL(JDO) = XLABEL(JDO) + TX
        YLABEL(JDO) = YLABEL(JDO) + TY
  210 CONTINUE
C
C
      XMAX = Drawing_Size/2.0 + TX
      XMIN = -Drawing_Size/2.0 + TX
C
C
      IF (UseOrtep) THEN
C
C---- constant for overlap margin (white margin at overlap)
C
        IF (Overlap_Margin.GT.0.0) THEN
C
C---- set overlap margin width directly in inches
C
C     AINT - truncate to integer (real to real)
C     Converts a real number to a real number with a fraction
C     of 0.0 by truncating the fraction.
C    The result is the largest integer value not exceeding 
C    the magnitude of number and whose sign is the same as number; 
C    if the absolute value of number is less than 1.0, 
C    the result is 0.0.
C
C
          Overlap_Margin = Overlap_Margin - AINT(Overlap_Margin)
C
C----  default option, overlap margin
C      width as a function of Scal_pre_Proj
C
        ELSE
          Overlap_Margin = MAX(SQRT(Scal_pre_Proj)*0.75,0.625)
        END IF
C
C---- sort atoms list by -viewdistance or by z parameter
C
        IF (Eye_Distance.GT.0.0) THEN
C
C---- calculate viewdistances**2 *(-1) if view.gt.zero
C
          DO 220 JDO = 1,NAT
            V3(1) = Plotting_XYZ(1,JDO)
            V3(2) = Plotting_XYZ(2,JDO)
            V3(3) = Plotting_XYZ(3,JDO)
            V3(3) = V3(3) - Eye_Distance
            Sorted_xyz(1,JDO) = Plotting_XYZ(1,JDO)
            Sorted_xyz(2,JDO) = Plotting_XYZ(2,JDO)
C
C                                **************************
            Sorted_xyz(3,JDO) = -vectbyvecttranspose(V3,V3)
C                                **************************
C
  220     CONTINUE
C
C---- store cartesian coordinates if view.eq.zero
C
        ELSE
          DO 230 JDO = 1,NAT
            Sorted_xyz(1,JDO) = Plotting_XYZ(1,JDO)
            Sorted_xyz(2,JDO) = Plotting_XYZ(2,JDO)
            Sorted_xyz(3,JDO) = Plotting_XYZ(3,JDO)
  230     CONTINUE
        END IF
C
C---- sorting procedure by shell, comm acm 2,30 (1959)
C
        MSORT = NAT
  240   MSORT = MSORT/2
        IF (MSORT.LE.0) GO TO 290
        KSORT = NAT - MSORT
        JSORT = 1
  250   ISORT = JSORT
  260   IMSORT = ISORT + MSORT
        IF ((Sorted_xyz(3,ISORT)-Sorted_xyz(3,IMSORT)).LE.
     +      0.0) GO TO 280
C
C
        DO 270 LSORT = 1,3
          TEMPSORT = Sorted_xyz(LSORT,ISORT)
          Sorted_xyz(LSORT,ISORT) = Sorted_xyz(LSORT,IMSORT)
          Sorted_xyz(LSORT,IMSORT) = TEMPSORT
  270   CONTINUE
C
C---- KSortPointer (J) holds the position in
C     Plotter_XYZ(1,KSortPointer(J)) that corresponds
C     to sort order
C  i.e. sorted position 1 is Plotter_XYZ(1,KSortPointer(1))
C
        KKKSORT = KSortPointer(ISORT)
        KSortPointer(ISORT) = KSortPointer(IMSORT)
        KSortPointer(IMSORT) = KKKSORT
C
C
        ISORT = ISORT - MSORT
        IF (ISORT.GT.0) GO TO 260
  280   JSORT = JSORT + 1
        IF ((JSORT-KSORT).LE.0) THEN
          GO TO 250
        ELSE
          GO TO 240
        END IF
C
C
  290   CONTINUE
C
C---- scaling and centering functions
C     604 instruction
C     set ellipsoid scale factor
C      automatic origin and/or scale
C
        XO(1) = Paper_Width*0.5
        XO(2) = Paper_Length*0.5
        Scal_pre_Proj = 1.0
C
C---- fit box around set of atoms
C        km=top atom
C
        SMULT = 1.0
C
C---- check view distance
C
        IF (Eye_Distance.LE.0.0) THEN
          Scal_pre_Proj = Scal_pre_Proj*SMULT
          SCL = Scal_pre_Proj*Ellipse_ScaleFact
          GO TO 300
        ELSE
          T1 = ZMAX*SMULT
        END IF
C
C---- increase view distance
C
        IF ((Eye_Distance*0.5-T1).LT.0.0) Eye_Distance = 2.0*T1
C
C---- find perspective projection limits
C
        T2 = Eye_Distance/ (Eye_Distance-ZMAX)
C
C---- automatic scale and position
C
        SCAL(1) = Paper_Width/ (XMAX-XMIN)
        SCAL(2) = Paper_Length/ (YMAX-YMIN)
        SMUL2 = MIN(SCAL(1),SCAL(2))
        XO(1) = (Paper_Width- (XMAX+XMIN)*SMUL2)*0.5
        XO(2) = (Paper_Length- (YMAX+YMIN)*SMUL2)*0.5
        SMULT = SMULT*SMUL2
        Eye_Distance = Eye_Distance*SMUL2
        Scal_pre_Proj = Scal_pre_Proj*SMULT
        SCL = Scal_pre_Proj*Ellipse_ScaleFact
  300   CONTINUE
C
C           ****************
        CALL storeoverlap
        CALL setupbondoverlap
C           ****************
C
      END IF
C
C--- End of ORTEP section
C
      IF (NDrawing_Number.EQ.1 .AND. LStereo_Flag.LE.1) GO TO 310
      IF (NTRACE_Counter.GT.0) GO TO 330
      IF (LSuperimpose_Flag.GT.1) GO TO 330
      IF (LStereo_Flag.GE.2) GO TO 330
C
C--- Advance Frame
C
C          *************
      CALL zadvanceframe
C          *************
C
      GO TO 320
C
C--- First picture - initialise plotting system
C
C          ************
  310 CALL zinitplotter
C          ************
C
C--- Set orientation of plot on paper
C
C          ************************************************
  320 CALL zsetpaperorientation(Lpaper_Setting,Paper_Width)
C          ************************************************
C
C--- Set up character font and line thickness
C
C          ********************************
  330 CALL zsetfont(NFont_Number)
      CALL zsetpenthickness(NPen_Thickness)
C          ********************************
C
C--- Set basic pen colour
C
C          ******************************
      CALL zsetpencolour(Lcurrent_Colour)
C          ******************************
C
C----  plot title.
C      set character size for title=Charcter_Size,
C      current size=CurrentCharcter_Size
C
      QXQ = Charcter_Size/CurrentCharcter_Size
      CurrentCharcter_Size = Charcter_Size
C
C          *********************************
      CALL zsetcharactersize(Charcter_Size)
C          *********************************
C
      YY = YMAX + Charcter_Size + 1.0
      XLABL = (XMIN+XMAX)*0.5
C
C
      IF (Log_Draw_Title) THEN
C
C            **************************
        CALL zsettextposition(XLABL,YY)
C            **************************
C
C---- Title centred on given position
C
        KLT = LENSTR(Iden_Title)
        WRITE (LSTRNG,FMT=6014) Iden_Title(1:KLT)
 6014   FORMAT (A)
C
          KZMODE = 2
C            **************************
        CALL zplotstring(KZMODE,LSTRNG,1,KLT)
C            **************************
C
      END IF
C
C---- label stereo images l or r
C
      IF (LStereo_Flag.LT.1) GO TO 340
      IF (.NOT.Use_Ticks) GO TO 340
      XLABL = CurrentCharcter_Size*1.2 + XMIN
      YLABL = YMAX - 1.2*CurrentCharcter_Size
C
C          *****************************
      CALL zsettextposition(XLABL,YLABL)
C          *****************************
C
      LSTRNG = LRLAB(LStereo_Flag)
C
          KZMODE = 1
C          *************************
      CALL zplotstring(KZMODE,LSTRNG,1,1)
C          *************************
C
C---- set character size for labels =Charcter_Size
C
  340 QXQ = Charcter_Size/CurrentCharcter_Size
      CurrentCharcter_Size = Charcter_Size
C
C          *********************************
      CALL zsetcharactersize(Charcter_Size)
C          *********************************
C
      IF (IBOX.EQ.0) GO TO 350
      IF (.NOT.Use_Box) GO TO 350
C
C---- draw box defining the limit of plotting coords
C
C          *****************
      CALL zmoveto(XMIN,YMIN)
      CALL zdrawto(XMIN,YMAX)
      CALL zdrawto(XMAX,YMAX)
      CALL zdrawto(XMAX,YMIN)
      CALL zdrawto(XMIN,YMIN)
C          *****************
C
  350 IF (Map_Flag_Counter.LE.0) GO TO 370
C
C---- plot maps
C
      DO 360 MAP = 1,Map_Flag_Counter
C
C            *********************
        CALL plotmaps(MAP,XMIN,XMAX)
C            *********************
C
  360 CONTINUE
C
C---- plot molecule
C
C                   *******************
  370 IF (NAT.GT.8) CALL drawsolidatoms
C                   *******************
C
C----   transferred labels to solid
C       labels plotted here...
C
      IF (Ldraw_cell_Flag.NE.2) GO TO 390
C
C
      DO 380 I = 1,8
        XLABL = XLABEL(I)
        YLABL = YLABEL(I)
C
C            *****************************
        CALL zsettextposition(XLABL,YLABL)
C            *****************************
C
        LSTRNG = Atm_Identifier(I)
C
          KZMODE = 1
C            *************************
        CALL zplotstring(KZMODE,LSTRNG,1,6)
C            *************************
C
  380 CONTINUE
C
C---- test if stereo pair is to be drawn.
C
  390 CONTINUE
      IF (LStereo_Flag.EQ.0) GO TO 400
      LStereo_Flag = LStereo_Flag + 1
      IF (LStereo_Flag.LE.2) GO TO 130
      LStereo_Flag = 1
C
C---- control of number of tracings and number of copies.
C
  400 NTRACE_Counter = NTRACE_Counter + 1
      IF (NTRACE_Counter.LT.LTrace_Flag) GO TO 10
      NCOPY_Counter = NCOPY_Counter + 1
      IF (NDrawing_Number.LE.1) WRITE (6,FMT=6016) NDrawing_Number,
     +    NCOPY_Counter,NTRACE_Counter
 6016 FORMAT (' *** Completed Drawing no.',I3,' - Copy no.',I3,' - Tra',
     +       'ce no.',I3)
      NTRACE_Counter = 0
      IF (NCOPY_Counter.LT.Ncopies) GO TO 10
C
C                             *******************
      IF (NBondLabels.GT.0) CALL plotbondlength
C                             *******************
C
      RETURN
      END
C
C
C
C     =====================
      SUBROUTINE pltmapaxes
C     =====================
C
C---- annotate map axes
C
C
C
C
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
C     ..
C     .. Scalars in Common ..
      REAL Charcter_Size,D001,D010,D100,DEY,DEYC,DPL,Drawing_Size,
     +     RFR(4,4),ROR(4,4),
     +     Perspective_Distance,Picture_Scale,QPERS,QX,QY,S,
     +     Shadelight_X,Shadelight_Y,Stereo_Separation,U0,UN,V0,VN,W,W0,
     +     WN,XC,YC,ZMAX,ZPL
      INTEGER Invert_Map_Flag,Lcurrent_Colour,Ldraw_cell_Flag,
     +        LimitsSet_Flag,Lmap3d_Flag,Lmap_ModeFirst,
     +        Lorthogonalisation_Code,Lpaper_Setting,Lperspective_Flag,
     +        Lplot_colour_Stereo,Lposition_Flag,Lprint_Flag,
     +        LSetView_Flag,LShade_Flag,LSolid_Flag,LStereo_Flag,
     +        LSuperimpose_Flag,LTaper_Flag,LTrace_Flag,Map_Counter,
     +        Map_Flag_Counter,Map_pts_U,Map_pts_V,MU,MV,MW,
     +        Natoms_plusCell,Ncopies,Npicture,Npts_U_compressed,
     +        Npts_V_compressed,Nspecified_Bonds,Num_Moles,Lperm
      LOGICAL Log_Draw_Title
C     ..
C     .. Arrays in Common ..
      REAL AID,AQ,Contour_Levels,Contour_Threshold,OrtCell,OrtConts,
     +     Plotting_XYZ,RP,ViewMatrix,XABC,XCLANG
      INTEGER ISpecified_Bond_Params,Map_def_Restrictions,Map_LineTypes,
     +        Map_UVW_Limits,Map_XYZ_Limits,MOL,Num_Contours
      CHARACTER Map_Title*80
C     ..
C     .. Local Scalars ..
      REAL ANG,AQ11,AQ12,AQ13,AQ21,AQ22,AQ23,AQ31,AQ32,AQ33,PI,ROT,U,V,
     +     X,XXP,Y,YYP,ZZP
      INTEGER LH,LV
      CHARACTER LSTRNG*80
C     ..
C     .. Local Arrays ..
      CHARACTER LXYZ(3)*1, LUVW(3)*1
C     ..
C     .. External Subroutines ..
      EXTERNAL zplotstring,zsettextangle,zsettextposition
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ATAN2,COS,SIN
C     ..
C     .. Common blocks ..
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,Picture_Scale,
     +       Perspective_Distance,Shadelight_X,Shadelight_Y,S,
     +       Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),ViewMatrix(3,3),
     +       MOL(Max_Mols,4),ISpecified_Bond_Params(30,3)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CONTOURS/UN,VN,WN,U0,V0,W0,W,ZPL,QPERS,DPL,DEY,DEYC,
     +       QX,QY,XC,YC,ZMAX
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Data statements ..
      DATA LXYZ/'X','Y','Z'/
      DATA LUVW/'U','V','W'/
C     ..
C
C
      AQ11 = AQ(1,1)
      AQ21 = AQ(2,1)
      AQ31 = AQ(3,1)
      AQ12 = AQ(1,2)
      AQ22 = AQ(2,2)
      AQ32 = AQ(3,2)
      AQ13 = AQ(1,3)
      AQ23 = AQ(2,3)
      AQ33 = AQ(3,3)
C
        write(6,*)'pltmapaxes'
         write(6,'(3(/,3f10.5))')((aq(i,j),j=1,3),i=1,3)
C
      PI = ATAN2(1.0,1.0)*4.0
      ROT = 180.0/PI
C
C---- transform coordinates of origin
C     fractional coordinates
C
      U = U0/UN
      V = V0/VN
C
C---- plotter coordinates
C
      XXP = AQ11*U + AQ12*V + AQ13*W
      YYP = AQ21*U + AQ22*V + AQ23*W
      ZZP = AQ31*U + AQ32*V + AQ33*W
C
C---- shift to centre
C
      XXP = (XXP-XC)*S
      YYP = (YYP-YC)*S
      ZZP = (ZZP-ZMAX)*S
         write(6,*)' xxp yyp zzp',xxp,yyp,zzp
         write(6,*)'LStereo_Flag,Lperspective_Flag',
     + LStereo_Flag,Lperspective_Flag
      IF (LStereo_Flag.GT.0) GO TO 10
C
C---- mono
C
      IF (Lperspective_Flag.EQ.0) GO TO 20
C
C---- perspective
C
      QPERS = DPL/ (DPL-ZZP)
      XXP = XXP*QPERS
      YYP = YYP*QPERS
      GO TO 20
C
C---- stereo
C
   10 QPERS = DPL/ (DPL-ZZP)
      XXP = (XXP+DEYC)*QPERS - DEYC
      YYP = YYP*QPERS
C
C---- back off origin
C
   20 X = XXP + QX - Charcter_Size
      Y = YYP + QY + 0.5*Charcter_Size
C
C---- horizontal and vertical axes lh,lv
C
      LH = MU
      LV = MV
C
      IF (Invert_Map_Flag.GT.0) GO TO 30
      LH = MV
      LV = MU
C---- annotate horizontal axis
C
C          *********************
   30 CONTINUE
         write(6,*)' Invert_Map_Flag,mu,mv,lh,lv',
     + Invert_Map_Flag,mu,mv,lh,lv
       CALL zsettextposition(X,Y)
C          *********************
C
      IF(Patt_Flag.EQ.0) LSTRNG = '  ----> '//LXYZ(LH)
      IF(Patt_Flag.EQ.1) LSTRNG = '  ----> '//LUVW(LH)
C
C add
      CALL zsettextangle(0.0)
C
          KZMODE = 1
C          *************************
      CALL zplotstring(KZMODE,LSTRNG,1,9)
C          *************************
C
C---- annotate vertical axis
C      Rotate character frame to angle of axis
C
c      ANG = 180.0 - XCLANG(MW) + 90.0
       ANG = 180.0 - XCLANG(MW) 
C
C---- Start point of string is 16mm from origin for
C     default character size = 2
C
      X = X - 8.0*Charcter_Size*COS(ANG/ROT)
      Y = Y - 8.0*Charcter_Size*SIN(ANG/ROT) 
C
C          *********************
      CALL zsettextposition(X,Y)
C          *********************
C
C---- rotate
C
C          ******************
      CALL zsettextangle(ANG)
C          ******************
C
      IF(Patt_Flag.EQ.0) LSTRNG = LXYZ(LV)//' <----'
      IF(Patt_Flag.EQ.1) LSTRNG = LUVW(LV)//' <----'
C
          KZMODE = 1
C          *************************
      CALL zplotstring(KZMODE,LSTRNG,1,7)
C          *************************
C
C---- Restore frame
C
C          ******************
      CALL zsettextangle(0.0)
C          ******************
C
      RETURN
      END
C
C
C
C     =========================
      SUBROUTINE primer(ACTION)
C     =========================
C
C
C
C---- NEllipsoidStyle packed as
C
C   ==>      NPLANE + NDOT*10 + NLINE*100 +
C             NDASH*1000 + AIN0*10000 + AIN1*1000000
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Conics
      PARAMETER (Max_Conics=20)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER Max_Labels
      PARAMETER (Max_Labels=100)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER Max_Quadrangles
      PARAMETER (Max_Quadrangles=30)
      INTEGER Max_Radii
      PARAMETER (Max_Radii=10)
      INTEGER Max_Segments
      PARAMETER (Max_Segments=50)
      INTEGER Max_Symmetry
      PARAMETER (Max_Symmetry=192)
      INTEGER NDefault_Colour
      PARAMETER (NDefault_Colour=1)
C     ..
C     .. Scalar Arguments ..
      CHARACTER ACTION*6
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Cent_Xmax,Cent_Xmin,
     +     Cent_Ymax,Cent_Ymin,Cent_Zmax,Cent_Zmin,Charcter_Size,
     +     CurrentCharcter_Size,D001,D010,D100,Def_Bnd_Radius,
     +     Def_Char_Size,Def_Paper_Wdth,Def_Plt_Radius,Def_Plt_Size,
     +     Dist_to_Boundary,Drawing_Size,Ellipse_ScaleFact,Eye_Distance,
     +     RFR(4,4),ROR(4,4),
     +     Overlap_Margin,Paper_Length,Paper_Width,
     +     Perspective_Distance,Picture_Scale,Plotter_Step,
     +     Retrace_Displmnt,Rmax_Save,S,Scal_pre_Proj,SCL,Shadelight_X,
     +     Shadelight_Y,Stereo_Separation,Xmax_Save,Xmin_Save,Ymax_Save,
     +     Ymin_Save,Zmax_Save,Zmin_Save
      INTEGER IERR,Invert_Map_Flag,Iuse_persp_Labs,KCHOSE,
     +        Label_atoms_Flag,Lcurrent_Colour,Ldef_Bnd_Lines,
     +        Ldraw_cell_Flag,LimitsSet_Flag,Ljoin_Flag,Lmap3d_Flag,
     +        Lmap_ModeFirst,Lorthogonalisation_Code,Lpack_Flag,
     +        Lpaper_Setting,Lperspective_Flag,Lplot_colour_Stereo,
     +        Lposition_Flag,Lprint_Flag,Lsaved_Paper_Setting,
     +        LSetView_Flag,LShade_Flag,LSolid_Flag,LStereo_Flag,
     +        LSuperimpose_Flag,LTaper_Flag,LTrace_Flag,Map_Counter,
     +        Map_Flag_Counter,Map_pts_U,Map_pts_V,MU,MV,MW,NAT,NATO,
     +        Natoms_plusCell,NC,NC1,Ncopies,NDrawing_Number,Nellipise,
     +        NFont_Number,NPen_Thickness,Npicture,Npts_U_compressed,
     +        Npts_V_compressed,NQuads_Overlaped,Nspecified_Bonds,
     +        NSymmetry_Ops,NumHbondRadii,NumLinkRadii,NumPlotRadii,
     +        Num_Bnd_Quadrangles,Num_Conics,Num_Moles,Num_Overlaps
      LOGICAL LCent_Flag,Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
      CHARACTER Iden_Title*80
C     ..
C     .. Arrays in Common ..
      REAL AID,AQ,Atm_Hbond_Radii,Atm_Link_Radii,Atm_Plot_Radii,
     +     Atom_Radii,Base_Vect_Refsys,Base_Vect_Wrksys,BBB,
     +     Contour_Levels,Contour_Threshold,Drawing_Origin,
     +     Ellipse_Conics,Ellipse_Resolution,Fractional_XYZ,
     +     G_Metric_Tensor,OrtCell,OrtConts,Orthogonal_XYZ,
     +     Overlap_Conics,P1_to_Refsys,P1_to_Wrksys,Plotting_XYZ,
     +     Principal_axis_rms,Prin_Angstrom_Vectors,Prin_Axis_Col_Vects,
     +     Q,Quadrangles,Quads_Overlaped,Recip_Metric_Tensor,
     +     Rotsym_Matrx,RP,Segments,Sorted_xyz,Title_Rotation,
     +     Title_RotnTrans,TransSym_Vect,ViewMatrix,XABC,XCLANG,XO
      INTEGER IATID,IAtom_Connectivity,IAtom_Type,
     +        ISpecified_Bond_Params,KC,KEllipseTypes,KQ,KSortPointer,
     +        LAtom_Inc_Flag,LAtom_Label_Flag,LHbond_Flag,LRadi_Atm_typ,
     +        LRadi_Hbond_typ,LRadi_Plt_typ,Map_def_Restrictions,
     +        Map_LineTypes,Map_UVW_Limits,Map_XYZ_Limits,MOL,
     +        NEllipsoidStyle,Num_Contours
      CHARACTER Atm_Lab_Type*2,Atm_Identifier*10,Atm_Label*10,
     +          Atm_Lab_Txt*10,Map_Title*80
C     ..
C     .. Local Scalars ..
      REAL ACELL,AIN0,AIN1
      INTEGER I,J,JDO,NED,NEH,NEL,NEP,LPERM
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC COS,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,Picture_Scale,
     +       Perspective_Distance,Shadelight_X,Shadelight_Y,S,
     +       Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),ViewMatrix(3,3),
     +       MOL(Max_Mols,4),ISpecified_Bond_Params(30,3)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CENTRE/LCent_Flag,Cent_Xmax,Cent_Ymax,Cent_Zmax,
     +       Cent_Xmin,Cent_Ymin,Cent_Zmin
      COMMON /com_CHOOSE/IATID(8),KCHOSE
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_DEFAULTS/Ldef_Bnd_Lines,Def_Bnd_Radius,Def_Plt_Radius,
     +       Def_Char_Size,Def_Plt_Size,Def_Paper_Wdth
      COMMON /com_ELLIPSESTYLE/NEllipsoidStyle(Max_Atoms),Nellipise,
     +       KEllipseTypes(6)
      COMMON /com_FONT/NFont_Number,NPen_Thickness
      COMMON /com_LABCHR/Atm_Lab_Txt(Max_Labels),
     +       Atm_Lab_Type(Max_Labels),Atm_Label(Max_Labels),
     +       Atm_Identifier(Max_Atoms),Iden_Title
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
      COMMON /com_OVERLAP/Num_Conics,Num_Overlaps,NQuads_Overlaped,
     +       Num_Bnd_Quadrangles,KC(Max_Conics),KQ(Max_Quadrangles),
     +       Ellipse_Conics(7,Max_Atoms),Overlap_Conics(6,Max_Conics),
     +       Quads_Overlaped(3,4,Max_Quadrangles),
     +       Quadrangles(9,2*Max_Atoms),Segments(Max_Segments,2)
      COMMON /com_RADII/NumLinkRadii,NumPlotRadii,NumHbondRadii,
     +       Atm_Link_Radii(Max_Radii),Atm_Plot_Radii(Max_Radii),
     +       Atm_Hbond_Radii(Max_Radii),LRadi_Atm_typ(Max_Radii),
     +       LRadi_Plt_typ(Max_Radii),LRadi_Hbond_typ(Max_Radii),
     +       LHbond_Flag(2)
      COMMON /com_SAVESCAL/Xmin_Save,Ymin_Save,Zmin_Save,Xmax_Save,
     +       Ymax_Save,Zmax_Save,Rmax_Save,Lsaved_Paper_Setting
      COMMON /com_SYMMETRY/NSymmetry_Ops,Rotsym_Matrx(3,3,Max_Symmetry),
     +       TransSym_Vect(3,Max_Symmetry)
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      IF (ACTION.EQ.'primer') THEN
C
C----  Default orthogonalisation Rollett
C
        Lorthogonalisation_Code = 1
        NAT = -1
        Lsaved_Paper_Setting = -1
        NDrawing_Number = 0
        Ncopies = 1
        LTrace_Flag = 1
        ACELL = -10.0
        CurrentCharcter_Size = 3.0
        Lposition_Flag = -1
        LSuperimpose_Flag = 0
        Overlap_Margin = 0.0
ccx
C
C---- set various constant in mm.
C
        Plotter_Step = 0.25
        Eye_Distance = 30.0
        Use_Box = .TRUE.
        Use_Ticks = .TRUE.
C
C---- calculate constants
C
        DO 10 I = 1,5
          OrtConts(I) = SQRT(1.0/ ((COS(3.141593/2.0**I)+1.0)*2.0))
   10   CONTINUE
C
C
ccx      Retrace_Displmnt = 0.005
        Retrace_Displmnt = 0.00
        Bond_Lab_Ang = 0.866
        Iuse_persp_Labs = 0
        NATO = 0
        Ellipse_Resolution(1) = 1.25
        Ellipse_Resolution(2) = 0.5
        Ellipse_Resolution(3) = 0.2
        Scal_pre_Proj = 1.0
        Ellipse_ScaleFact = 1.54
        SCL = 1.54
C
C
        DO 30 J = 1,3
          DO 20 I = 1,3
            Title_Rotation(I,J) = 0.0
   20     CONTINUE
          Title_Rotation(J,J) = 1.0
   30   CONTINUE

C
C inches/mm ccx
        Bond_Taper = 0.375
        Ang_CharPlt = 0.0
        Eye_Distance = 30.0
        XO(1) = Paper_Width/2.0
        XO(2) = Paper_Length/2.0
C
C
        DO 40 J = 1,3
          Drawing_Origin(J) = 0.0
   40   CONTINUE
C
C
        KEllipseTypes(1) = 845101
        KEllipseTypes(2) = 840804
        KEllipseTypes(3) = 840004
        KEllipseTypes(4) = -845153
        KEllipseTypes(5) = 840101
        KEllipseTypes(6) = 845101
C
C
        NEP = 4
        NED = 0
        NEL = 8
        NEH = 0
        AIN0 = 0.40
        AIN1 = 0.08
C
C---- default is ortep 704 ellipse type
C
        DO 50 JDO = 1,Max_Atoms
          NEllipsoidStyle(JDO) = KEllipseTypes(5)
          KSortPointer(JDO) = JDO
   50   CONTINUE
C
C---- initiate overlap routines
C     eliminate all previously stored global overlap information
C
        Num_Overlaps = 0
        NQuads_Overlaped = 0
        Num_Conics = 8
        Num_Bnd_Quadrangles = 0
C
C
      END IF
C
C
      IF (ACTION.EQ.'ATMINI') THEN
        Ldef_Bnd_Lines = 5
        Def_Bnd_Radius = 0.04
        Def_Plt_Radius = 0.3
        Def_Char_Size = 2
        Def_Plt_Size = 600
        KCHOSE = 0
        LCent_Flag = .FALSE.
        Map_Flag_Counter = 0
C
C---- set default linking & plotting radii in
C     Atm_Link_Radii(1), Atm_Plot_Radii(1)
C
        NumLinkRadii = 1
        Atm_Link_Radii(1) = 0.9
        LRadi_Atm_typ(1) = 0
        NumPlotRadii = 1
        Atm_Plot_Radii(1) = Def_Plt_Radius
        LRadi_Plt_typ(1) = 0
C
C---- default h-bond radii
C
        NumHbondRadii = 1
        Atm_Hbond_Radii(1) = 0.0
        LRadi_Hbond_typ(1) = 0
C
        Stereo_Separation = 0.0
C
C---- Paper_Width is width of paper in mm ie across roll
C     Paper_Length is length of paper in mm ie down roll
C     Def_Paper_Wdth is not used
C
        Paper_Width = 328.0
        Paper_Length = 508.0
        Def_Paper_Wdth = Paper_Width
        LTaper_Flag = 0
C
C
        Atm_Identifier(1) = 'O'
        Atm_Identifier(2) = 'A'
        Atm_Identifier(4) = 'B'
        Atm_Identifier(5) = 'C'
C
C
        LAtom_Label_Flag(1) = 1
        LAtom_Label_Flag(2) = 1
        LAtom_Label_Flag(4) = 1
        LAtom_Label_Flag(5) = 1
C
C---- default is ortep 704 ellipse type
C
        DO 60 JDO = 1,Max_Atoms
          NEllipsoidStyle(JDO) = KEllipseTypes(5)
          KSortPointer(JDO) = JDO
   60   CONTINUE
C
C---- initiate overlap routines
C     eliminate all previously stored global overlap information
C
        Num_Overlaps = 0
        NQuads_Overlaped = 0
        Num_Conics = 8
        Num_Bnd_Quadrangles = 0
C

      END IF
C
C
      IF (ACTION.EQ.'FIRSTG') THEN
        IF (LSuperimpose_Flag.EQ.1) LSuperimpose_Flag =
     +      LSuperimpose_Flag + 1
        NAT = 8
        NC = 12
        NSymmetry_Ops = 0
        IERR = 0
        NPen_Thickness = 1
        NFont_Number = 1
        LHbond_Flag(1) = -2
        Ljoin_Flag = 0
        Map_Flag_Counter = 0
        Label_atoms_Flag = 0
        LSetView_Flag = 0
        LSolid_Flag = 0
        LStereo_Flag = 0
        Lperspective_Flag = 0
        Lpack_Flag = 0
        Nspecified_Bonds = 0
        Num_Moles = 0
        Ldraw_cell_Flag = 0
        Picture_Scale = 0.0
        LTrace_Flag = 0
        Lplot_colour_Stereo = NDefault_Colour
        Lprint_Flag = 0
        LShade_Flag = 0
        Shadelight_X = 120
        Shadelight_Y = -45
        Eye_Distance = 30.0
        Drawing_Size = 0.0
        Charcter_Size = Def_Char_Size
        Perspective_Distance = Def_Plt_Size
C
C
        DO 70 JDO = 9,Max_Atoms
          Atom_Radii(JDO) = Def_Plt_Radius
          LAtom_Label_Flag(JDO) = 0
          LAtom_Inc_Flag(JDO) = 1
   70   CONTINUE
C
C
        DO 80 JDO = 1,Max_Atoms
          KSortPointer(JDO) = JDO
C
C---- default is ortep 704 ellipse type
C
          NEllipsoidStyle(JDO) = KEllipseTypes(5)
   80   CONTINUE
C
C---- initiate overlap routines
C     eliminate all previously stored global overlap information
C
        Num_Overlaps = 0
        NQuads_Overlaped = 0
        Num_Conics = 8
        Num_Bnd_Quadrangles = 0
C
C
      END IF
C
C
      IF (ACTION.EQ.'RESETS') THEN
C
C---- reset parameters
C
        IERR = 0
        NPen_Thickness = 1
        NFont_Number = 1
        LHbond_Flag(1) = -2
        Map_Flag_Counter = 0
        Label_atoms_Flag = 0
        LSetView_Flag = 0
        LSolid_Flag = 0
        LStereo_Flag = 0
        Lperspective_Flag = 0
        Lpack_Flag = 0
        Nspecified_Bonds = 0
        Num_Moles = 0
        Ldraw_cell_Flag = 0
        Picture_Scale = 0.0
        LTrace_Flag = 0
        Lplot_colour_Stereo = NDefault_Colour
        Charcter_Size = Def_Char_Size
        Perspective_Distance = Def_Plt_Size
        Lprint_Flag = 0
        LShade_Flag = 0
        Shadelight_X = 120
        Shadelight_Y = -45
        Eye_Distance = 30.0
        Drawing_Size = 0.0
C
C
        DO 90 JDO = 9,Max_Atoms
          Atom_Radii(JDO) = Def_Plt_Radius
          LAtom_Label_Flag(JDO) = 0
          LAtom_Inc_Flag(JDO) = 1
   90   CONTINUE
C
C
        DO 100 JDO = 1,Max_Atoms
C
C---- default is ortep 704 ellipse type
C
          NEllipsoidStyle(JDO) = KEllipseTypes(5)
C
          KSortPointer(JDO) = JDO
  100   CONTINUE
C
C---- initiate overlap routines
C     eliminate all previously stored global overlap information
C
        Num_Overlaps = 0
        NQuads_Overlaped = 0
        Num_Conics = 8
        Num_Bnd_Quadrangles = 0
C
C
      END IF
C
C
      RETURN
      END
C
C
C
C     =========================================================
      SUBROUTINE project2dto3d(CIRCLESPOKES,CIRCLELINES_2D,X,XO,
     +                         Eye_Distance,ISTART,IEND,NRESOLUTION)
C     ==========================================================
C
C---- 3d cartesian to 2d plotter coordinates
C
C
C
C
C
C
C     .. Scalar Arguments ..
      REAL Eye_Distance
      INTEGER IEND,ISTART,NRESOLUTION
C     ..
C     .. Array Arguments ..
      REAL CIRCLELINES_2D(2,ISTART:IEND),CIRCLESPOKES(3,ISTART:IEND),
     +     X(3),XO(3)
C     ..
C     .. Local Scalars ..
      REAL T1,T2,T3,T4
      INTEGER IDO
C     ..
C
C
      T3 = Eye_Distance - X(3)
C
C
      DO 10 IDO = ISTART,IEND,NRESOLUTION
        T1 = CIRCLESPOKES(1,IDO) + X(1)
        T2 = CIRCLESPOKES(2,IDO) + X(2)
C
C
        IF (Eye_Distance.GT.0.0) THEN
          T4 = Eye_Distance/ (T3-CIRCLESPOKES(3,IDO))
          T1 = T1*T4
          T2 = T2*T4
        END IF
C
C
ccx        CircleLines_2D(1,Ido) = XO(1) + T1
ccx        CircleLines_2D(2,Ido) = XO(2) + T2
        CIRCLELINES_2D(1,IDO) = T1
        CIRCLELINES_2D(2,IDO) = T2
   10 CONTINUE
C
C
      RETURN
      END
C
C
C
C     =================================
      SUBROUTINE radset(NRAD,RAD,ITYPR)
C     =================================
C
C---- set radii into array radi
C     nrad number of radius types
C     rad  radii ( rad(1) is default value)
C     itypr atom types for this radius
C     if radius negative,
C         set flag into LAtom_Inc_Flag to plot 2 circles
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Labels
      PARAMETER (Max_Labels=100)
C     ..
C     .. Scalar Arguments ..
      INTEGER NRAD
C     ..
C     .. Array Arguments ..
      REAL RAD(NRAD)
      INTEGER ITYPR(NRAD)
C     ..
C     .. Scalars in Common ..
      INTEGER IERR,Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1
      CHARACTER Iden_Title*80
C     ..
C     .. Arrays in Common ..
      REAL Atom_Radii,Fractional_XYZ,Orthogonal_XYZ
      INTEGER IAtom_Connectivity,IAtom_Type,LAtom_Inc_Flag,
     +        LAtom_Label_Flag
      CHARACTER Atm_Lab_Type*2,Atm_Identifier*10,Atm_Label*10,
     +          Atm_Lab_Txt*10
C     ..
C     .. Local Scalars ..
      REAL R,RD
      INTEGER I,ITYP,J,JD,K
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_LABCHR/Atm_Lab_Txt(Max_Labels),
     +       Atm_Lab_Type(Max_Labels),Atm_Label(Max_Labels),
     +       Atm_Identifier(Max_Atoms),Iden_Title
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      RD = ABS(RAD(1))
      JD = 0
      IF (RAD(1).LT.0.0) JD = 100
C
C
      DO 30 I = 9,NAT
        R = RD
        K = JD
        IF (NRAD.LE.1) GO TO 20
        ITYP = IAtom_Type(I)
C
C
        DO 10 J = 2,NRAD
          IF (ITYP.NE.ITYPR(J)) GO TO 10
C
C---- set radius for this type
C
          R = RAD(J)
          K = 0
          IF (R.LT.0.0) K = 100
          R = ABS(R)
          GO TO 20
   10   CONTINUE
C
C
   20   Atom_Radii(I) = R
        IF (LAtom_Inc_Flag(I).GT.0) LAtom_Inc_Flag(I)
     +      = LAtom_Inc_Flag(I) + K
   30 CONTINUE
C
C
      RETURN
      END
C
C
C
C     =====================================================
      SUBROUTINE setboundarylimits(Fractional_XYZ,Bound_Limits,
     +                             Bound_Edge)
C     =====================================================
C
C
C
C---- set cell corners to limits Bound_Limits,Bound_Edge
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
C     ..
C     .. Array Arguments ..
      REAL Bound_Edge(3),Bound_Limits(3),Fractional_XYZ(Max_Atoms,3)
C     ..
C     .. Local Scalars ..
      INTEGER I,J
C     ..
C
C
      DO 20 I = 1,4
        DO 10 J = 1,3
          Fractional_XYZ(I,J) = Bound_Limits(J)
          Fractional_XYZ(I+4,J) = Bound_Edge(J)
   10   CONTINUE
   20 CONTINUE
C
C
      DO 30 I = 1,2
        Fractional_XYZ(I+2,2) = Bound_Edge(2)
        Fractional_XYZ(I+4,2) = Bound_Limits(2)
        Fractional_XYZ(I+1,1) = Bound_Edge(1)
   30 CONTINUE
C
C
      Fractional_XYZ(5,1) = Bound_Limits(1)
      Fractional_XYZ(8,1) = Bound_Limits(1)
C
C
      RETURN
      END
C
C
C
C     =================================
      SUBROUTINE setdashparams(M,R,D,T)
C     =================================
C
C
C
C   MODE =0 solid line
C   MODE =1 dashed line (dot dummy parameter)
C   MODE =2 chained line
C
C  Repeat dash dot  are in current units  default is mms.
C
C
C     .. Scalar Arguments ..
      REAL D,R,T
      INTEGER M
C     ..
C     .. Scalars in Common ..
      REAL A1,B1,DASH,DOT,REPEAT
      INTEGER LPT,MODE
C     ..
C     .. Common blocks ..
      COMMON /com_DASH/LPT,MODE,REPEAT,DASH,DOT,A1,B1
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      MODE = M
      REPEAT = R
      DASH = D
      DOT = T
      LPT = 0
C
C
      RETURN
      END
C
C
C
C     =============================
      SUBROUTINE setplotstyle(MSTL)
C     =============================
C
C
C---- set plotting style from array mstl
C
C MSTL(1)  if .gt. 0, colour
C          (1=black, 2=red, 3=green, 4=blue, 5=yellow)
C              =0     colour unset
C             .lt. 0, dashed line, =-mode-1-colour*4
C
C MSTL(2-4) dashed parameters
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Colours
      PARAMETER (Max_Colours=10)
      INTEGER Max_Labels
      PARAMETER (Max_Labels=100)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER NDefault_Colour
      PARAMETER (NDefault_Colour=1)
C     ..
C     .. Array Arguments ..
      INTEGER MSTL(4)
C     ..
C     .. Scalars in Common ..
      REAL Charcter_Size,Drawing_Size,Perspective_Distance,
     +     Picture_Scale,S,Shadelight_X,Shadelight_Y,Stereo_Separation
      INTEGER Lcurrent_Colour,Ldraw_cell_Flag,Lpaper_Setting,
     +        Lperspective_Flag,Lplot_colour_Stereo,Lposition_Flag,
     +        Lprint_Flag,LShade_Flag,LSolid_Flag,LStereo_Flag,
     +        LSuperimpose_Flag,LTaper_Flag,LTrace_Flag,Natoms_plusCell,
     +        Ncopies,Nspecified_Bonds,Num_Moles
      LOGICAL Log_Draw_Title
      CHARACTER Iden_Title*80
C     ..
C     .. Arrays in Common ..
      REAL Plotting_XYZ,RP,ViewMatrix
      INTEGER ISpecified_Bond_Params,MOL
      CHARACTER Atm_Lab_Type*2,Atm_Identifier*10,Atm_Label*10,
     +          Atm_Lab_Txt*10
C     ..
C     .. Local Scalars ..
      REAL D,RPT,T
      INTEGER M,MD
C     ..
C     .. External Subroutines ..
      EXTERNAL setdashparams,zsetpencolour
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC MOD,REAL
C     ..
C     .. Common blocks ..
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_LABCHR/Atm_Lab_Txt(Max_Labels),
     +       Atm_Lab_Type(Max_Labels),Atm_Label(Max_Labels),
     +       Atm_Identifier(Max_Atoms),Iden_Title
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      M = MSTL(1)
      IF (M.LT.0) GO TO 20
C
C---- colour
C
C          ****************************
      CALL setdashparams(0,0.0,0.0,0.0)
C          ****************************
C
C---- colour=0, use default set elsewhere
C
   10 IF (M.LE.0) M = Lcurrent_Colour
      IF (M.GT.Max_Colours .OR. M.LE.0) M = NDefault_Colour
C
C          ****************
      CALL zsetpencolour(M)
C          ****************
C
      RETURN
C
C---- dashed
C
   20 M = -M
      MD = MOD(M,4) - 1
      M = M/4
      RPT = REAL(MSTL(2))*0.1
      D = REAL(MSTL(3))*0.1
      T = REAL(MSTL(4))*0.1
C
C          *************************
      CALL setdashparams(MD,RPT,D,T)
C          *************************
C
      GO TO 10
      END
C
C
C
C     ======================
      SUBROUTINE setrad(I,R)
C     ======================
C
C---- set radius r into array radi(i)
C     if radius negative, put flag into LAtom_Inc_Flag(i)
C     todraw two concentric circles
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
C     ..
C     .. Scalar Arguments ..
      REAL R
      INTEGER I
C     ..
C     .. Scalars in Common ..
      INTEGER IERR,Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1
C     ..
C     .. Arrays in Common ..
      REAL Atom_Radii,Fractional_XYZ,Orthogonal_XYZ
      INTEGER IAtom_Connectivity,IAtom_Type,LAtom_Inc_Flag,
     +        LAtom_Label_Flag
C     ..
C     .. Local Scalars ..
      INTEGER K
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      K = 0
      IF (R.LT.0.0) K = 100
      Atom_Radii(I) = ABS(R)
      IF (LAtom_Inc_Flag(I).GT.0) LAtom_Inc_Flag(I) = LAtom_Inc_Flag(I)
     +     + K
C
C
      RETURN
      END
C
C
C
C        ===========================
      SUBROUTINE setupbondoverlap
C        ===========================
C
C
C---- store bond Quadrangles if search codes are given
C      generate pseudo-instruction 822 to calculate bonds
C      loop through atoms array connection array
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_BondType
      PARAMETER (Max_BondType=20)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Contours
      PARAMETER (Max_Contours=200)
      INTEGER Max_Maps
      PARAMETER (Max_Maps=2)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER Max_Symmetry
      PARAMETER (Max_Symmetry=192)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Charcter_Size,
     +     CurrentCharcter_Size,D001,D010,D100,Def_Bnd_Radius,
     +     Def_Char_Size,Def_Paper_Wdth,Def_Plt_Radius,Def_Plt_Size,
     +     Dist_to_Boundary,Drawing_Size,Ellipse_ScaleFact,Eye_Distance,
     +     RFR(4,4),ROR(4,4),
     +     Overlap_Margin,Paper_Length,Paper_Width,
     +     Perspective_Distance,Picture_Scale,Plotter_Step,
     +     Retrace_Displmnt,S,Rmax_Save,Xmax_Save,Xmin_Save,Ymax_Save,
     +     Ymin_Save,Zmax_Save,Zmin_Save,Scal_pre_Proj,SCL,Shadelight_X,
     +     Shadelight_Y,Stereo_Separation
      INTEGER IERR,Invert_Map_Flag,Iuse_persp_Labs,KSEC1,KSEC2,
     +        Label_atoms_Flag,Lcurrent_Colour,Ldef_Bnd_Lines,
     +        Ldraw_cell_Flag,LimitsSet_Flag,Ljoin_Flag,Lmap3d_Flag,
     +        Lmap_ModeFirst,Lorthogonalisation_Code,Lpack_Flag,
     +        Lpaper_Setting,Lperspective_Flag,Lplot_colour_Stereo,
     +        Lposition_Flag,Lprint_Flag,Lsaved_Paper_Setting,
     +        LSetView_Flag,LShade_Flag,LSolid_Flag,LStereo_Flag,
     +        LSuperimpose_Flag,LTaper_Flag,LTrace_Flag,Map_Counter,
     +        Map_Flag_Counter,Map_pts_U,Map_pts_V,Msecst,MU,MV,MW,NAT,
     +        NATO,Natoms_plusCell,NC,NC1,Ncopies,NDrawing_Number,
     +        NFirst_Section,NLast_Section,Npicture,Npts_U_compressed,
     +        Npts_V_compressed,Nsections_in_Group,Nspecified_Bonds,
     +        NstpMapGrpSect,Num_LinkSets,Num_Moles
      LOGICAL Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL AID,AQ,Atom_Radii,Base_Vect_Refsys,Base_Vect_Wrksys,BBB,
     +     Contour_Levels,Contour_Threshold,Def_LnkBond,Drawing_Origin,
     +     Ellipse_Resolution,Fractional_XYZ,G_Metric_Tensor,OrtCell,
     +     OrtConts,Orthogonal_XYZ,P1_to_Refsys,P1_to_Wrksys,
     +     Plotting_XYZ,Principal_axis_rms,Prin_Angstrom_Vectors,
     +     Prin_Axis_Col_Vects,Q,Recip_Metric_Tensor,RP,Sorted_xyz,
     +     Title_Rotation,Title_RotnTrans,ViewMatrix,XABC,XCLANG,XO
      INTEGER IAtom_Connectivity,IAtom_Type,IConn_BndType,IResid_BndTyp,
     +        ISpecified_Bond_Params,ISym_BndType,KSortPointer,
     +        LAtom_Inc_Flag,LAtom_Label_Flag,Map_def_Restrictions,
     +        Map_LineTypes,Map_UVW_Limits,Map_XYZ_Limits,MOL,NSECC,
     +        Num_Contours
      CHARACTER Link_Atm_Names*2,Map_Title*80
C     ..
C     .. Local Scalars ..
      REAL DASH,DOT,RBO,REPEAT
      INTEGER I,I1,I2,IB,IDASH,II,INCI,INCJ,INSTR,IO,J,JDASH,JDOT,JMODE,
     +        JO,JREPET,K,L,M,MM,N,N1,N2,N3,NA1,NA2,NLI,NOBSC,LPERM
C     ..
C     .. External Subroutines ..
      EXTERNAL ortbond
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC MOD
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_DEFAULTS/Ldef_Bnd_Lines,Def_Bnd_Radius,Def_Plt_Radius,
     +       Def_Char_Size,Def_Plt_Size,Def_Paper_Wdth
      COMMON /com_LINK/Num_LinkSets,IResid_BndTyp(Max_Atoms),
     +       ISym_BndType(Max_Symmetry),IConn_BndType(Max_BondType),
     +       Def_LnkBond(2)
      COMMON /com_LNKCHR/Link_Atm_Names(2,2)
      COMMON /com_MAPCHR/Map_Title(Max_Maps)
      COMMON /com_MAPS/Map_Flag_Counter,Npicture,Map_Counter,
     +       Invert_Map_Flag,LimitsSet_Flag,Lmap3d_Flag,LSetView_Flag,
     +       Lmap_ModeFirst,Npts_U_compressed,Npts_V_compressed,
     +       Map_pts_U,Map_pts_V,MU,MV,MW,Map_XYZ_Limits(3),
     +       Map_UVW_Limits(6),Map_LineTypes(4,Max_Maps,2),
     +       Num_Contours(Max_Maps),Map_def_Restrictions(4),
     +       Contour_Levels(Max_Contours,Max_Maps),
     +       Contour_Threshold(Max_Maps),AQ(3,3),Lperm,Patt_Flag
      COMMON /com_SAVESCAL/Xmin_Save,Ymin_Save,Zmin_Save,Xmax_Save,
     +       Ymax_Save,Zmax_Save,Rmax_Save,Lsaved_Paper_Setting
      COMMON /com_SECTIONS/NFirst_Section,NLast_Section,
     +       Nsections_in_Group,NstpMapGrpSect,KSEC1,KSEC2,Msecst,
     +       NSECC(2)
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      DO 130 M = 1,Num_Moles
        MM = (M-1)* (NAT-8)
        I1 = 13
        IF (M.EQ.1 .AND. Ldraw_cell_Flag.EQ.2) I1 = 1
        I2 = NC
C
C---- loop through the list of connections
C     stored in array IAtom_Connectivity
C
        DO 120 II = I1,I2
C
C---- unpack bond type l and atom numbers
C
          JO = MOD(IAtom_Connectivity(II),Max_Atoms)
          L = IAtom_Connectivity(II)/Max_Atoms
          IO = MOD(L,Max_Bonds)
          L = L/Max_Bonds
          INCI = LAtom_Inc_Flag(IO)
          INCJ = LAtom_Inc_Flag(JO)
C
C---- omit intramolecular bonds when either atom is omitted,
C        LAtom_Inc_Flag=0
C
          IF (IO.LT.9 .OR. IO.GT.NATO .OR. JO.LT.9 .OR.
     +        JO.GT.NATO) GO TO 10
C
C---- include bonds unless both atoms are excluded & in same margin
C
          IF (INCI.GT.0 .OR. INCJ.GT.0) GO TO 10
          I = -INCI/5
          J = -INCJ/5
          IF (I+J.NE.1) GO TO 120
   10     I = IO + MM
          J = JO + MM
          IDASH = 0
          IF (LSolid_Flag.EQ.0) GO TO 110
          IF (Plotting_XYZ(3,I).GT.Plotting_XYZ(3,J)) GO TO 20
          K = I
          I = J
          J = K
C
C---- allocate bond radius and number of lines.
C
   20     CONTINUE
          IF (I.LT.9) GO TO 80
          IF (L.EQ.0) GO TO 30
C
C---- bond type from connectivity list
C
          L = IConn_BndType(L)
          GO TO 60
C
C---- get default bond type
C
   30     RBO = Def_Bnd_Radius
          NLI = Ldef_Bnd_Lines
          IF (RBO.GT.0.0) GO TO 40
C
C---- default type is dashed
C
          IDASH = 1
          L = -NLI
C
   40     IF (Nspecified_Bonds.EQ.0) GO TO 90
          L = 0
C
C
          DO 50 K = 1,Nspecified_Bonds
            N1 = ISpecified_Bond_Params(K,1)
            N2 = ISpecified_Bond_Params(K,2)
            N3 = ISpecified_Bond_Params(K,3)
C
C
            IF (N1.LT.0) THEN
C
C---- intermolecular
C
              IF (IO.GT.NATO .OR. JO.GT.NATO) L = N3
C
C---- atom type
C
            ELSE IF (N1.EQ.0) THEN
              IF (IAtom_Type(IO).EQ.N2 .OR. IAtom_Type(JO).EQ.N2) L = N3
C
C---- specific bond
C
            ELSE IF (N1.GT.0) THEN
              IF ((N1.EQ.IO.AND.N2.EQ.JO) .OR.
     +            (N1.EQ.JO.AND.N2.EQ.IO)) L = N3
            END IF
C
C
   50     CONTINUE
          IF (L.EQ.0 .AND. IDASH.EQ.0) GO TO 90
C
C----  L contains bond parameters
C
   60     IF (L.GT.0) GO TO 70
C
C---- dashed line
C
          L = -L
          JDOT = L/50000
          DOT = 0.1*JDOT
          L = L - 50000*JDOT
          JDASH = L/500
          DASH = 0.1*JDASH
          L = L - 500*JDASH
          JREPET = L/5
          REPEAT = 0.1*JREPET
          JMODE = L - 5*JREPET - 2
          L = -1000
          IDASH = 1
          GO TO 80
C
   70     RBO = MOD(L,1000)*0.001
          NLI = L/1000
          GO TO 90
   80     RBO = 0.0
          NLI = 1
   90     IB = J
C
C---- scale bond radius to mm. i j bond call ortbond
C
          RBO = RBO*S
          NOBSC = 0
          IF (LSolid_Flag.EQ.2) GO TO 100
C
C---- now we can draw the bond i to n.
C
  100     N = IB
C
C---- prepare bond overlap
C     get na1 and na2 atom numbers to connect in plotting_array
C
C
          INSTR = 822
          NA1 = I
          NA2 = N
          IF (NLI.GT.5) NLI = 5
C
C              ******************************
          CALL ortbond(INSTR,NA1,NA2,RBO,NLI)
C              ******************************
C
C---- stick bonds only---no circles - no obscuring...
C
  110     CONTINUE
  120   CONTINUE
  130 CONTINUE
C
C
      RETURN
      END
C
C
C
C     =========================
      SUBROUTINE setupeigenvals
C     =========================
C
C---- set up ortep style matrices
C
C
C
C
C---- store idemfactor matrix
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Charcter_Size,
     +     CurrentCharcter_Size,D001,D010,D100,Dist_to_Boundary,
     +     Drawing_Size,Ellipse_ScaleFact,Eye_Distance,
     +     RFR(4,4),ROR(4,4),
     +     Overlap_Margin,Paper_Length,Paper_Width,Perspective_Distance,
     +     Picture_Scale,Plotter_Step,Retrace_Displmnt,S,Scal_pre_Proj,
     +     SCL,Shadelight_X,Shadelight_Y,Stereo_Separation
      INTEGER Iuse_persp_Labs,Lcurrent_Colour,Ldraw_cell_Flag,
     +        Lorthogonalisation_Code,Lpack_Flag,Lpaper_Setting,
     +        Lperspective_Flag,Lplot_colour_Stereo,Lposition_Flag,
     +        Lprint_Flag,LShade_Flag,LSolid_Flag,LStereo_Flag,
     +        LSuperimpose_Flag,LTaper_Flag,LTrace_Flag,Natoms_plusCell,
     +        Ncopies,NDrawing_Number,Nspecified_Bonds,Num_Moles
      LOGICAL Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL AID,Base_Vect_Refsys,Base_Vect_Wrksys,BBB,Drawing_Origin,
     +     Ellipse_Resolution,G_Metric_Tensor,OrtCell,OrtConts,
     +     P1_to_Refsys,P1_to_Wrksys,Plotting_XYZ,Principal_axis_rms,
     +     Prin_Angstrom_Vectors,Prin_Axis_Col_Vects,Q,
     +     Recip_Metric_Tensor,RP,Sorted_xyz,Title_Rotation,
     +     Title_RotnTrans,ViewMatrix,XABC,XCLANG,XO
      INTEGER ISpecified_Bond_Params,KSortPointer,MOL
C     ..
C     .. Local Scalars ..
      INTEGER I,J,K
C     ..
C     .. External Functions ..
      REAL arccos
      EXTERNAL arccos
C     ..
C     .. External Subroutines ..
      EXTERNAL matrixsolve,matrxbymatrx,vectorstore
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC COS,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_CELL/Lorthogonalisation_Code,ROR,RFR,D100,
     +       D010,D001,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
c      COMMON /com_CELL/Lorthogonalisation_Code,OR1,OR2,OR3,OR4,OR5,OR6,
c     +       OR7,OR8,OR9,D100,D010,D001,FR1,FR2,FR3,FR4,FR5,FR6,FR7,FR8,
c     +       FR9,XABC(3),XCLANG(3),OrtConts(9),OrtCell(9),AID(3,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      DO 20 J = 1,3
        DO 10 K = 1,3
          AID(J,K) = 0.0
          AID(K,K) = 1.0
   10   CONTINUE
   20 CONTINUE
C
C---- cell dimensions from pluto /com_cell
C
      DO 30 J = 1,3
        OrtCell(J) = XABC(J)
        OrtCell(J+6) = XCLANG(J)
        OrtCell(J+3) = COS(XCLANG(J)*1.745329E-2)
C
C---- store metric tensor a*a b*b c*c
C     tensor - the ratio of lengthening of a vector
C
C     Metric Coefficients - any point in a set of
C                           cartesian axes P(y1,y2,y3)
C         Position vector  r = I1Y1 + I2Y2 + I3Y3
C               (I1,I2,I3) unit base vectors
C
        G_Metric_Tensor(J,J) = OrtCell(J)**2
   30 CONTINUE
C
C
      G_Metric_Tensor(1,2) = OrtCell(1)*OrtCell(2)*OrtCell(6)
      G_Metric_Tensor(1,3) = OrtCell(1)*OrtCell(3)*OrtCell(5)
      G_Metric_Tensor(2,3) = OrtCell(2)*OrtCell(3)*OrtCell(4)
      G_Metric_Tensor(2,1) = G_Metric_Tensor(1,2)
      G_Metric_Tensor(3,1) = G_Metric_Tensor(1,3)
      G_Metric_Tensor(3,2) = G_Metric_Tensor(2,3)
C
C---- invert metric tensor
C
C          ******************************************************
      CALL matrixsolve(G_Metric_Tensor,Recip_Metric_Tensor,AID,3)
C          ******************************************************
C
C---- calculate reciprocal cell parameters
C
      DO 40 J = 1,3
        BBB(J) = SQRT(Recip_Metric_Tensor(J,J))
   40 CONTINUE
C
C
      BBB(6) = Recip_Metric_Tensor(1,2)/ (BBB(1)*BBB(2))
      BBB(5) = Recip_Metric_Tensor(1,3)/ (BBB(1)*BBB(3))
      BBB(4) = Recip_Metric_Tensor(2,3)/ (BBB(2)*BBB(3))
C
C
      DO 50 J = 1,3
ccx  !!?? radians
        BBB(J+6) = arccos(BBB(J+3))
   50 CONTINUE
C
C---- store standard vectors
C     base vector - the diagonal of a parallelepiped with edges
C                   xa, yb, zc
C            V = xa + yb + zc
C                               a,b,c are base vectors
C                               x,y,z are measure numbers
C
C          *************************************
      CALL vectorstore(AID,AID(1,2),Base_Vect_Refsys,0)
      CALL matrxbymatrx(G_Metric_Tensor,Base_Vect_Refsys,P1_to_Refsys)
C          *************************************
C
      DO 70 I = 1,3
        DO 60 J = 1,3
          P1_to_Wrksys(J,I) = P1_to_Refsys(J,I)
          Q(J,I) = Base_Vect_Refsys(I,J)
          ViewMatrix(J,I) = Base_Vect_Refsys(J,I)
   60   CONTINUE
   70 CONTINUE
C
      RETURN
      END
C
C
C
C     =======================
      SUBROUTINE storeoverlap
C     =======================
C
C---- store projected atom conics and bond Quadrangles
C
C
C
C---- double precision for very small atoms and short wordlength *
C     in general, double precision is not required
C     real*8 qc,qd,vd1,vd2,td1,td2,td3
C     just remove the preceeding real*8 card to make single prec
C
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
      INTEGER Max_Bonds
      PARAMETER (Max_Bonds=2500)
      INTEGER Max_Conics
      PARAMETER (Max_Conics=20)
      INTEGER Max_Mols
      PARAMETER (Max_Mols=50)
      INTEGER Max_Quadrangles
      PARAMETER (Max_Quadrangles=30)
      INTEGER Max_Segments
      PARAMETER (Max_Segments=50)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,Charcter_Size,
     +     CurrentCharcter_Size,Dist_to_Boundary,Drawing_Size,
     +     Ellipse_ScaleFact,Eye_Distance,Overlap_Margin,Paper_Length,
     +     Paper_Width,Perspective_Distance,Picture_Scale,Plotter_Step,
     +     Retrace_Displmnt,S,Scal_pre_Proj,SCL,Shadelight_X,
     +     Shadelight_Y,Stereo_Separation
      INTEGER IERR,Iuse_persp_Labs,Label_atoms_Flag,Lcurrent_Colour,
     +        Ldraw_cell_Flag,Ljoin_Flag,Lpack_Flag,Lpaper_Setting,
     +        Lperspective_Flag,Lplot_colour_Stereo,Lposition_Flag,
     +        Lprint_Flag,LShade_Flag,LSolid_Flag,LStereo_Flag,
     +        LSuperimpose_Flag,LTaper_Flag,LTrace_Flag,NAT,NATO,
     +        Natoms_plusCell,NC,NC1,Ncopies,NDrawing_Number,
     +        NQuads_Overlaped,Nspecified_Bonds,Num_Bnd_Quadrangles,
     +        Num_Conics,Num_Moles,Num_Overlaps
      LOGICAL Log_Draw_Title,UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL Atom_Radii,Base_Vect_Refsys,Base_Vect_Wrksys,BBB,
     +     Drawing_Origin,Ellipse_Conics,Ellipse_Resolution,
     +     Fractional_XYZ,G_Metric_Tensor,Orthogonal_XYZ,Overlap_Conics,
     +     P1_to_Refsys,P1_to_Wrksys,Plotting_XYZ,Principal_axis_rms,
     +     Prin_Angstrom_Vectors,Prin_Axis_Col_Vects,Q,Quadrangles,
     +     Quads_Overlaped,Recip_Metric_Tensor,RP,Segments,Sorted_xyz,
     +     Title_Rotation,Title_RotnTrans,ViewMatrix,XO
      INTEGER IAtom_Connectivity,IAtom_Type,ISpecified_Bond_Params,KC,
     +        KQ,KSortPointer,LAtom_Inc_Flag,LAtom_Label_Flag,MOL
C     ..
C     .. Local Scalars ..
      REAL T1,TD1,TD2,TD3
      INTEGER IA,J,J1,J2,K,K1,K2,NDG
C     ..
C     .. Local Arrays ..
      REAL QC(3,3),QD(3,3),V1(4),V5(3),V6(3),VD1(3),VD2(3)
C     ..
C     .. External Functions ..
      REAL vectbymatrxbyvector
      EXTERNAL vectbymatrxbyvector
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC MOD,SQRT
C     ..
C     .. Common blocks ..
      COMMON /com_ATOMS/Fractional_XYZ(Max_Atoms,3),
     +       Orthogonal_XYZ(Max_Atoms,3),Atom_Radii(Max_Atoms),
     +       LAtom_Label_Flag(Max_Atoms),IAtom_Type(Max_Atoms),
     +       IAtom_Connectivity(Max_Bonds),LAtom_Inc_Flag(Max_Atoms),
     +       Label_atoms_Flag,Ljoin_Flag,NAT,NATO,NC,NC1,IERR
      COMMON /com_B/Log_Draw_Title,Num_Moles,Nspecified_Bonds,Ncopies,
     +       Natoms_plusCell,Ldraw_cell_Flag,LStereo_Flag,
     +       Lperspective_Flag,LSolid_Flag,LTaper_Flag,LTrace_Flag,
     +       Lplot_colour_Stereo,LShade_Flag,Lprint_Flag,Lpaper_Setting,
     +       Lcurrent_Colour,Lposition_Flag,LSuperimpose_Flag,
     +       Charcter_Size,Stereo_Separation,Drawing_Size,
     +       Picture_Scale,Perspective_Distance,Shadelight_X,
     +       Shadelight_Y,S,Plotting_XYZ(3,Max_Atoms),RP(Max_Atoms),
     +       ViewMatrix(3,3),MOL(Max_Mols,4),
     +       ISpecified_Bond_Params(30,3)
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
      COMMON /com_OVERLAP/Num_Conics,Num_Overlaps,NQuads_Overlaped,
     +       Num_Bnd_Quadrangles,KC(Max_Conics),KQ(Max_Quadrangles),
     +       Ellipse_Conics(7,Max_Atoms),Overlap_Conics(6,Max_Conics),
     +       Quads_Overlaped(3,4,Max_Quadrangles),
     +       Quadrangles(9,2*Max_Atoms),Segments(Max_Segments,2)
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      DO 250 IA = 9,NAT
C
C
        DO 20 J = 1,3
          V1(J) = Plotting_XYZ(J,IA)
          VD1(J) = V1(J)
          DO 10 K = 1,3
            QD(J,K) = ViewMatrix(J,K)
   10     CONTINUE
   20   CONTINUE
C
C
        IF (Eye_Distance.LE.0.0) GO TO 110
C
C---- calculate enveloping cone with origin at viewpoint
C
        V1(3) = V1(3) - Eye_Distance
        VD1(3) = V1(3)
C
C---- form cofactor matrix
C
        DO 40 J = 1,3
          J1 = MOD(J,3) + 1
          J2 = MOD(J+1,3) + 1
C
C
          DO 30 K = J,3
            K1 = MOD(K,3) + 1
            K2 = MOD(K+1,3) + 1
            QC(J,K) = QD(J1,K1)*QD(J2,K2) - QD(J1,K2)*QD(J2,K1)
            QC(K,J) = QC(J,K)
   30     CONTINUE
   40   CONTINUE
C
C---- form polarized cofactor matrix and add to ellipsoid matrix
C
        TD2 = -SCL**2
C
C---- td1 is an arbitrary scaling factor
C
C             ****************************
        TD1 = vectbymatrxbyvector(V1,ViewMatrix,V1)
C             ****************************
C
C
        DO 60 J = 1,3
          J1 = MOD(J,3) + 1
          J2 = MOD(J+1,3) + 1
C
C
          DO 50 K = J,3
            K1 = MOD(K,3) + 1
            K2 = MOD(K+1,3) + 1
            QD(J,K) = (((QC(J1,K1)*VD1(K2)-QC(J1,
     +                K2)*VD1(K1))*VD1(J2)+ (VD1(K1)*QC(J2,
     +                K2)-VD1(K2)*QC(J2,K1))*VD1(J1))+QD(J,K)*TD2)/TD1
            QD(K,J) = QD(J,K)
   50     CONTINUE
C
C---- projected ellipse in homogeneous coord of working system
C
          QD(J,3) = -QD(J,3)*Eye_Distance
          QD(3,J) = -QD(3,J)*Eye_Distance
   60   CONTINUE
C
C---- project center of atom onto projection plane
C
        TD1 = -Eye_Distance/VD1(3)
        VD2(1) = VD1(1)*TD1
        VD2(2) = VD1(2)*TD1
C
C---- transform to new origin to improve condition of matrix q
C
        DO 80 J = 1,3
          DO 70 K = 1,2
            QD(J,3) = QD(J,K)*VD2(K) + QD(J,3)
   70     CONTINUE
   80   CONTINUE
C
C
        DO 100 J = 1,3
          DO 90 K = 1,2
            QD(3,J) = VD2(K)*QD(K,J) + QD(3,J)
   90     CONTINUE
  100   CONTINUE
C
C
        V6(1) = XO(1) + VD2(1)
        V6(2) = XO(2) + VD2(2)
        GO TO 150
C
C---- calculate enveloping cylinder along z of working system
C
  110   CONTINUE
C
C
        DO 130 J = 1,2
          DO 120 K = 1,2
            QD(J,K) = QD(J,K) - QD(J,3)*QD(K,3)/QD(3,3)
  120     CONTINUE
  130   CONTINUE
C
C
        DO 140 J = 1,2
          QD(J,3) = 0.0
          QD(3,J) = 0.0
          V6(J) = Plotting_XYZ(J,IA) + XO(J)
  140   CONTINUE
C
C---- projected ellipse in homogeneous coord about center of atom
C
        QD(3,3) = -SCL**2
C
C---- fit rectangle around ellipse allowing overlap margin
C     form matrix of cofactors
C
  150   CONTINUE
C
C
        DO 170 J = 1,3
          J1 = MOD(J,3) + 1
          J2 = MOD(J+1,3) + 1
          DO 160 K = J,3
            K1 = MOD(K,3) + 1
            K2 = MOD(K+1,3) + 1
            K1 = MOD(K,3) + 1
            K2 = MOD(K+1,3) + 1
            QC(J,K) = QD(J1,K1)*QD(J2,K2) - QD(J1,K2)*QD(J2,K1)
  160     CONTINUE
  170   CONTINUE
C
C---- rescale matrix of cofactors so that qc(3,3)=1.0
C
        DO 190 J = 1,3
          DO 180 K = J,3
            QC(J,K) = QC(J,K)/QC(3,3)
            QC(K,J) = QC(J,K)
  180     CONTINUE
  190   CONTINUE
C
C
        TD2 = QD(3,3)
        NDG = 0
C
C
        DO 220 J = 1,2
C
C---- solve quadratic equation
C
          T1 = QC(3,J)**2 - QC(J,J)
          IF (T1.GT.0.0) GO TO 200
C
C---- roundoff problems, reset limits in x or y
C
          NDG = 1
          V5(J) = 0.001 + Overlap_Margin
          GO TO 210
  200     V5(J) = SQRT(T1) + Overlap_Margin
          V6(J) = V6(J) + QC(3,J)
          TD2 = QD(3,J)*QC(3,J) + TD2
  210     Ellipse_Conics(2*J-1,IA) = V6(J) - V5(J)
          Ellipse_Conics(2*J,IA) = V6(J) + V5(J)
  220   CONTINUE
C
C
        IF (NDG.GT.0) GO TO 230
        IF (TD2.LT.0.0) GO TO 240
C
C---- ellipse imaginary due to roundoff, reset to real value
C
  230   Ellipse_Conics(5,IA) = 1.0/ ((Ellipse_Conics(2,
     +                         IA)-Ellipse_Conics(1,IA))*0.5)**2
        Ellipse_Conics(6,IA) = 0.0
        Ellipse_Conics(7,IA) = 1.0/ ((Ellipse_Conics(4,
     +                         IA)-Ellipse_Conics(3,IA))*0.5)**2
        GO TO 250
C
C---- store normalized quadratic coefficients for ellipse
C     scaled by overlap margin parameter
C
  240   TD3 = - (1.0-2.0*Overlap_Margin/ (V5(1)+V5(2)))**2/TD2
        Ellipse_Conics(5,IA) = QD(1,1)*TD3
        Ellipse_Conics(6,IA) = QD(1,2)*TD3
        Ellipse_Conics(7,IA) = QD(2,2)*TD3
  250 CONTINUE
C
C
      Num_Conics = NAT
      Scal_pre_Proj = 1.0
C
      END
C
C
C
C     ====================================
      SUBROUTINE symfrn(ICOL,I1,NS,ROT,TR)
C     ====================================
C
C
C
C Read and interpret symmetry operations
C
C On entry, ICOL contains line of 80 characters (80A1),
C   I1 is the first column to look at (say after keyword 'SYM')
C
C NS is the number of the first symmetry operation to be read,
C    & returns  with the number of the last one read.
C
C On exit, ROT(3,3,I) contains the real-space symmetry matrices, in
C                     standard convention, ie
C                     x'(I)=Sum(J=1,3)[ROT(I,J,)*x(J) + TR(I,)]
C          TR(3,I)    contains the fractional translations
C
C
C
C
C     .. Scalar Arguments ..
      INTEGER I1,NS
      CHARACTER ICOL*80
C     ..
C     .. Array Arguments ..
      REAL ROT(3,3,1),TR(3,1)
C     ..
C     .. Local Scalars ..
      REAL A,S,T
      INTEGER I,IFOUND,IMAX,IP,ISL,J,K,NOP,NP
      CHARACTER ICH*1,IWORK*1
C     ..
C
C
      IMAX = 80
      I = I1 - 1
      NS = NS - 1
   10 NS = NS + 1
      NOP = 1
C
C
      DO 30 J = 1,3
        TR(J,NS) = 0.0
        DO 20 K = 1,3
          ROT(J,K,NS) = 0.0
   20   CONTINUE
   30 CONTINUE
C
C
   40 S = 1.0
      T = 0.0
      IP = 0
      NP = 0
      ISL = 0
      IFOUND = 0
   50 I = I + 1
      IF (I.GT.IMAX) GO TO 180
      ICH = ICOL(I:I)
      IF (ICH.EQ.' ') GO TO 50
      IF (ICH.EQ.',' .OR. ICH.EQ.'*') GO TO 180
      IFOUND = 1
      IF (ICH.EQ.'X' .OR. ICH.EQ.'x') GO TO 100
      IF (ICH.EQ.'Y' .OR. ICH.EQ.'y') GO TO 110
      IF (ICH.EQ.'Z' .OR. ICH.EQ.'z') GO TO 120
      IF (ICH.EQ.'+') GO TO 140
      IF (ICH.EQ.'-') GO TO 150
      IF (ICH.EQ.'/') GO TO 160
      IF (ICH.EQ.'.') GO TO 170
C
C
      DO 60 K = 0,9
        WRITE (IWORK,FMT=6000) K
 6000   FORMAT (I1)
        IF (ICH.EQ.IWORK) GO TO 70
   60 CONTINUE
C
C
      WRITE (6,FMT=6002) ICH
 6002 FORMAT (' ***ERROR*** Invalid character...',a)
      GO TO 50
   70 A = K
      IF (ISL.EQ.1) GO TO 80
      IF (IP.EQ.1) GO TO 90
      T = A*S
      GO TO 50
   80 T = T/A
      GO TO 50
   90 NP = NP + 1
      T = A/10**NP + T
      GO TO 50
  100 J = 1
      GO TO 130
  110 J = 2
      GO TO 130
  120 J = 3
  130 ROT(NOP,J,NS) = S
C
      S = 1.0
      GO TO 50
  140 S = 1.0
      GO TO 50
  150 S = -1.0
      GO TO 50
  160 ISL = 1
      GO TO 50
  170 IP = 1
      GO TO 50
  180 TR(NOP,NS) = T
      IF (IFOUND.EQ.0 .AND. I.LE.IMAX) WRITE (6,FMT=6004)
 6004 FORMAT (' ***ERROR*** Blank operator field')
      IF (I.GT.IMAX) GO TO 190
      NOP = NOP + 1
      IF (NOP.LE.3) GO TO 40
      GO TO 10
  190 IF (NOP.EQ.1 .AND. IFOUND.EQ.0) GO TO 200
      IF (NOP.EQ.3 .AND. IFOUND.EQ.1) GO TO 210
      WRITE (6,FMT=6006)
 6006 FORMAT (' ***ERROR*** Last general position is incomplete')
  200 NS = NS - 1
  210 CONTINUE
      RETURN
      END
C
C
C
C     ========================
      SUBROUTINE testmatrix(R)
C     ========================
C
C---- test matrix for determinant of 1.0
C
C
C     .. Array Arguments ..
      REAL R(3,3)
C     ..
C     .. Local Scalars ..
      REAL DET
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS
C     ..
C
C
      DET = (R(2,2)*R(3,3)-R(2,3)*R(3,2))*R(1,1) -
     +      (R(2,1)*R(3,3)-R(2,3)*R(3,1))*R(1,2) +
     +      (R(2,1)*R(3,2)-R(2,2)*R(3,1))*R(1,3)
C
C
      IF (ABS(DET-1.000).LT.0.001) RETURN
      WRITE (6,FMT=6000) DET
 6000 FORMAT (
     + ' *WARNING* ',/,
     + '    Did you know that the determinant of your view',/,
     + '    MATRIX = ',F10.5,/,
     + '    It should be +1.000 for a proper rotation')
C
C
      IF (DET.LE.-0.999) WRITE (6,FMT=6002)
 6002 FORMAT (
     + ' *WARNING* ',/,
     + '     The determinant represents an inversion. ',/,
     + '     Absolute configuration is changed. ',/,
     + '      you have been warned...')
C
C
      RETURN
      END
C
C
C
C     =================================
      SUBROUTINE transposematrix(X,Y,Z)
C     =================================
C
C---- transpose(transpose(x)*y)=z
C     X,Y,Z ARE 3X3 MATRICES
C
C     .. Array Arguments ..
      REAL X(3,3),Y(3,3),Z(3,3)
C     ..
C     .. Local Scalars ..
      INTEGER I,K
C     ..
C
C
      DO 20 I = 1,3
        DO 10 K = 1,3
          Z(K,I) = X(1,I)*Y(1,K) + X(2,I)*Y(2,K) + X(3,I)*Y(3,K)
   10   CONTINUE
   20 CONTINUE
C
C
      RETURN
      END
C
C
C
C     ==========================================
      REAL FUNCTION vectbymatrxbyvector(X1,Q,X2)
C     ==========================================
C
C---- transposed vector * matrix * vector
C     vectbymatrxbyvector=X1(3)*Q(3,3)*X2(3)
C
C to  evaluate quadratic or bilinear form
C
C
C
C     .. Array Arguments ..
      REAL Q(3,3),X1(3),X2(3)
C     ..
C     .. Local Scalars ..
      REAL T1
      INTEGER J
C     ..
C
C
      T1 = 0.0
C
C
      DO 10 J = 1,3
        T1 = (X2(1)*Q(J,1)+X2(2)*Q(J,2)+X2(3)*Q(J,3))*X1(J) + T1
   10 CONTINUE
C
C     *******************
      vectbymatrxbyvector = T1
C     *******************
C
      RETURN
      END
C
C
C
C     ======================================
      REAL FUNCTION vectbyvecttranspose(X,Y)
C     ======================================
C
C---- transposed vector * vector
C     vectbyvecttranspose=X(3)*Y(3)
C
C     .. Array Arguments ..
      REAL X(3),Y(3)
C     ..
C
C
C     *******************
      vectbyvecttranspose = X(1)*Y(1) + X(2)*Y(2) + X(3)*Y(3)
C     *******************
C
      RETURN
      END
C
C
C
C     ==================================
      SUBROUTINE vectordifference(X,Y,Z)
C     ==================================
C
C     vector - vector
C     z(3)=x(3)-y(3)
C
C
C     .. Array Arguments ..
      REAL X(3),Y(3),Z(3)
C     ..
C     .. Local Scalars ..
      INTEGER I
C     ..
C
C
      DO 10 I = 1,3
        Z(I) = X(I) - Y(I)
   10 CONTINUE
C
C
      RETURN
      END
C
C
C
C     =====================================
      SUBROUTINE vectorproduct(X,Y,Z,ITYPE)
C     =====================================
C
C---- vector product  z=x*y
C     itype .gt.0 for cartesian,.le.0 for triclinic
C
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
      INTEGER Max_Atoms
      PARAMETER (Max_Atoms=2000)
C     ..
C     .. Scalar Arguments ..
      INTEGER ITYPE
C     ..
C     .. Array Arguments ..
      REAL X(3),Y(3),Z(3)
C     ..
C     .. Arrays in Common ..
      REAL Base_Vect_Refsys,Base_Vect_Wrksys,BBB,G_Metric_Tensor,
     +     P1_to_Refsys,P1_to_Wrksys,Principal_axis_rms,
     +     Prin_Angstrom_Vectors,Prin_Axis_Col_Vects,Q,
     +     Recip_Metric_Tensor,Sorted_xyz,Title_RotnTrans
      INTEGER KSortPointer
C     ..
C     .. Local Scalars ..
      REAL T1
      INTEGER I,I1,I2
C     ..
C     .. Local Arrays ..
      REAL Z1(3)
C     ..
C     .. External Subroutines ..
      EXTERNAL matrxbyvector
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC MOD
C     ..
C     .. Common blocks ..
      COMMON /ORT_MATRIX/Base_Vect_Refsys(3,3),Base_Vect_Wrksys(3,3),
     +       G_Metric_Tensor(3,3),Principal_axis_rms(5),
     +       Prin_Axis_Col_Vects(3,5),Prin_Angstrom_Vectors(3,3),
     +       P1_to_Refsys(3,3),P1_to_Wrksys(3,3),
     +       Recip_Metric_Tensor(3,3),Q(3,3),Title_RotnTrans(3,4),
     +       BBB(9),Sorted_xyz(3,Max_Atoms),KSortPointer(Max_Atoms)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      DO 10 I = 1,3
        I1 = MOD(I+3,3) + 1
        I2 = MOD(I+1,3) + 1
        T1 = X(I1)*Y(I2) - X(I2)*Y(I1)
C
C
        IF (ITYPE.GT.0) THEN
          Z(I) = T1
        ELSE
          Z1(I) = T1
        END IF
   10 CONTINUE
C
C                     ********************************************
      IF (ITYPE.LE.0) CALL matrxbyvector(Recip_Metric_Tensor,Z1,Z)
C                     ********************************************
C
      RETURN
      END
C
C
C
C     ===================================
      SUBROUTINE vectorstore(U,V,X,ITYPE)
C     ===================================
C
C----  store three orthogonal vectors each 1 angstrom long
C       itype .gt.0 for cartesian,.le.0 for triclinic
C      iabs(itype)=1 w(1)=u,w(2)=(uxv),w(3)=ux(uxv)
C      iabs(itype)=2 w(1)=u,w(2)=(uxv)xu,w(3)=(uxv)
C       itype=0 w(1)=a,w(2)=(axb)xa,w(3)=(axb), abc=cell vectors ***
C
C
C     .. Scalar Arguments ..
      INTEGER ITYPE
C     ..
C     .. Array Arguments ..
      REAL U(3),V(3),X(3,3)
C     ..
C     .. Local Scalars ..
      INTEGER I,IC,IT,J
C     ..
C     .. Local Arrays ..
      REAL W(3,3)
C     ..
C     .. External Subroutines ..
      EXTERNAL getunitvectors,vectorproduct
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS
C     ..
C
C
      IT = ITYPE
C
C
      IF (IT.EQ.0) THEN
        U(1) = 1.0
        U(2) = 0.0
        U(3) = 0.0
        V(1) = 0.0
        V(2) = 1.0
        V(3) = 0.0
      END IF
C
C
      DO 10 J = 1,3
        W(J,1) = U(J)
   10 CONTINUE
C
C
      IF ((ABS(IT)-1).EQ.0) THEN
C
C            *********************************
        CALL vectorproduct(U,V,W(1,2),IT)
        CALL vectorproduct(U,W(1,2),W(1,3),IT)
      ELSE
        CALL vectorproduct(U,V,W(1,3),IT)
        CALL vectorproduct(W(1,3),U,W(1,2),IT)
C            *********************************
C
      END IF
C
C
      DO 20 I = 1,3
C
C
        IF (IT.LE.0) THEN
          IC = -1
        ELSE
          IC = 1
        END IF
C
C            ********************************
        CALL getunitvectors(W(1,I),X(1,I),IC)
C            ********************************
C
   20 CONTINUE
C
C
      RETURN
      END
C
C
C
C     ================================
      SUBROUTINE vecttimesmatrx(X,Y,Z)
C     ================================
C
C---- transposed vector times matrix
C     Z(3)=X(3)*Y(3,3)
C
C     .. Array Arguments ..
      REAL X(3),Y(3,3),Z(3)
C     ..
C     .. Local Scalars ..
      INTEGER I,J
C     ..
C
C
      DO 20 J = 1,3
        Z(J) = 0.0
        DO 10 I = 1,3
          Z(J) = X(I)*Y(I,J) + Z(J)
   10   CONTINUE
   20 CONTINUE
C
C
      RETURN
      END
C
C
C
C     ========================
      SUBROUTINE zadvanceframe
C     ========================
C
C
C
C --- Subroutine to advance to next picture or frame
C
C
C     .. External Subroutines ..
      EXTERNAL GSENDP,GSPICT
C     ..
C
C          ******
      CALL GSENDP
      CALL GSPICT
C          ******
C
      END
C
C
C
C     =======================
      SUBROUTINE zdrawby(X,Y)
C     =======================
C
C
C---- Subroutine to draw a line from current position
C          to X,Y away from it
C
C     .. Scalar Arguments ..
      REAL X,Y
C     ..
C     .. External Subroutines ..
      EXTERNAL GSDWBY
C     ..
C
C          ***********
      CALL GSDWBY(X,Y)
C          ***********
C
      END
C
C
C
C     =======================
      SUBROUTINE zdrawto(X,Y)
C     ========================
C
C
C---- Subroutine to draw a line from current position
C           to new position X,Y
C
C     .. Scalar Arguments ..
      REAL X,Y
C     ..
C     .. External Subroutines ..
      EXTERNAL GSDWTO
C     ..
C
C          ***********
      CALL GSDWTO(X,Y)
C          ***********
C
      END
C
C
C
C     =======================
      SUBROUTINE zendplotting
C     =======================
C
C---- Subroutine to stop the plotting system
C
C
C     .. External Subroutines ..
      EXTERNAL GSSTOP
C     ..
C
C          ******
      CALL GSSTOP
C          ******
C
      END
C
C
C
C     =======================
      SUBROUTINE zinitplotter
C     =======================
C
C
C---- Subroutine to initialise the plotting system
C
C     .. Parameters ..
      INTEGER Map_Size
      PARAMETER (Map_Size=800000)
C     ..
C     .. Scalars in Common ..
      REAL Ang_CharPlt,Bond_Lab_Ang,Bond_Taper,CurrentCharcter_Size,
     +     Dist_to_Boundary,Ellipse_ScaleFact,Eye_Distance,
     +     Overlap_Margin,Paper_Length,Paper_Width,Plotter_Step,
     +     Retrace_Displmnt,Scal_pre_Proj,SCL
      INTEGER Iuse_persp_Labs,Lpack_Flag,NDrawing_Number
      LOGICAL UseOrtep,Use_Box,Use_Ticks
C     ..
C     .. Arrays in Common ..
      REAL Drawing_Origin,Ellipse_Resolution,Title_Rotation,XO
C     ..
C     .. Local Scalars ..
      REAL DOTSMM
      INTEGER MIXCLR
C     ..
C     .. External Subroutines ..
      EXTERNAL GSBSIZ,GSDTRN,GSINIT,GSMIXC,GSPICT
C     ..
C     .. Common blocks ..
      COMMON /com_CONSTANTS/Use_Box,Use_Ticks,Iuse_persp_Labs,
     +       NDrawing_Number,Lpack_Flag,UseOrtep,Ang_CharPlt,Bond_Taper,
     +       Bond_Lab_Ang,CurrentCharcter_Size,Dist_to_Boundary,
     +       Ellipse_ScaleFact,Eye_Distance,Plotter_Step,Paper_Width,
     +       Paper_Length,Overlap_Margin,Retrace_Displmnt,Scal_pre_Proj,
     +       SCL,Drawing_Origin(3),Ellipse_Resolution(4),
     +       Title_Rotation(3,3),XO(3)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C---- Initialise PLOT84
C
C          **************
      CALL GSINIT('PLOT')
C          **************
C
C---- Set multiple colour mode
C
      MIXCLR = 1
C
C          **************
      CALL GSMIXC(MIXCLR)
C          **************
C
C---- Set Plot box size
C
C          *********************************************
      CALL GSBSIZ((Paper_Width+1.0), (Paper_Length+1.0))
C          *********************************************
C
C---- Set Dots per mm
C
      DOTSMM = 10.0
C
C          *********************
      CALL GSDTRN(DOTSMM,DOTSMM)
C          *********************
C
C---- Start new picture
C
C          ******
      CALL GSPICT
C          ******
C
      RETURN
      END
C
C
C
C     =======================
      SUBROUTINE zmoveby(X,Y)
C     =======================
C
C     .. Scalar Arguments ..
      REAL X,Y
C     ..
C     .. External Subroutines ..
      EXTERNAL GSMVBY
C     ..
C
C          ***********
      CALL GSMVBY(X,Y)
C          ***********
C
      END
C
C
C
C     =======================
      SUBROUTINE zmoveto(X,Y)
C     =======================
C
C---- Subroutine to move current plotting position to X,Y
C
C     .. Scalar Arguments ..
      REAL X,Y
C     ..
C     .. External Subroutines ..
      EXTERNAL GSMVTO
C     ..
C
C          ***********
      CALL GSMVTO(X,Y)
C          ***********
C
      END
C
C
C
C     =====================================
      SUBROUTINE zplotstring(MODE,ID,I1,I2)
C     =====================================
C
C
C
C---- Subroutine to output a text string
C     Parameters : MODE : =1 text string starts at current position
C                         =2 text string centred on current position
C                         =3 text string right justified
C                  Text string in CHARACTER array ID from I1 to I2
C
C
C     .. Scalar Arguments ..
      INTEGER I1,I2,MODE
      CHARACTER ID*(*)
C     ..
C     .. External Subroutines ..
      EXTERNAL GSCETX
C     ..
C
C
C     Avoid "!!!GSCETX ERROR: PLOTTING BLANK OR  ZERO LENGTH STRING"
      IF (ID(I1:I2).EQ.' ') RETURN
      IF ((MODE.LT.1) .OR. (MODE.GT.3)) MODE = 1
C
C          **********************
      CALL GSCETX(ID(I1:I2),MODE)
C          **********************
C
      END
C
C
C
C     ===============================
      SUBROUTINE zsetcharactersize(P)
C     ===============================
C
C---- Subroutine to set the character size
C
C
C     .. Scalar Arguments ..
      REAL P
C     ..
C     .. External Subroutines ..
      EXTERNAL GSSCLC
C     ..
C
C          ***********
      CALL GSSCLC(P,P)
C          ***********
C
      END
C
C
C
C     =================================
      SUBROUTINE zsetfont(NFont_Number)
C     =================================
C
C
C---- Subroutine to set the font
C
C
C     .. Scalar Arguments ..
      INTEGER NFont_Number
C     ..
C     .. External Subroutines ..
      EXTERNAL GSFONT
C     ..
C
C          ********************
      CALL GSFONT(NFont_Number)
C          ********************
C
      END
C
C
C
C     ===========================================================
      SUBROUTINE zsetpaperorientation(Lpaper_Setting,Paper_Width)
C     ===========================================================
C
C
C---- Subroutine to set up orientation of plot on paper
C
C
C
C     REVISED 5 NOV 1984 A.D.MCLACHLAN
C  Extra unnecessary parameters added
C  for compatibility with GHOST version
C     SMM 11/4/88
C
C     Set up orientation of plot on paper
C     default  Lpaper_Setting=0   xp across paper
C                   =1   yp across
C     Lsaved_Paper_Setting is current setting
C
C
C
C     .. Scalar Arguments ..
      REAL Paper_Width
      INTEGER Lpaper_Setting
C     ..
C     .. Scalars in Common ..
      REAL Rmax_Save,Xmax_Save,Xmin_Save,Ymax_Save,Ymin_Save,Zmax_Save,
     +     Zmin_Save
      INTEGER Lsaved_Paper_Setting
C     ..
C     .. Local Scalars ..
      REAL PI,PI2
C     ..
C     .. External Subroutines ..
      EXTERNAL GSORGD,GSUROT
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ATAN2
C     ..
C     .. Common blocks ..
      COMMON /com_SAVESCAL/Xmin_Save,Ymin_Save,Zmin_Save,Xmax_Save,
     +       Ymax_Save,Zmax_Save,Rmax_Save,Lsaved_Paper_Setting
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C
      PI = ATAN2(1.0,1.0)*4.0
      PI2 = PI/2.0
C
C---- in plot84 set the origin in absolute terms at start of every
C     picture
C
        write(6,*)' Lpaper_Setting',Lpaper_Setting
      IF (Lpaper_Setting.EQ.1) GO TO 10
C
C---- settings for xacross (Lpaper_Setting=0)
C
C          ***************
      CALL GSORGD(0.0,0.0)
      CALL GSUROT(0.0,PI2)
C          ***************
C
      GO TO 20
C
C---- settings for yacross (Lpaper_Setting=1)
C     set origin at maximum x (=Paper_Width) and axes rotated by 90
C
C          **********************
   10 CALL GSORGD(Paper_Width,0.0)
      CALL GSUROT(PI2,PI)
C          **********************
C
   20 CONTINUE
      Lsaved_Paper_Setting = Lpaper_Setting
      RETURN
      END
C
C
C
C     ===========================
      SUBROUTINE zsetpencolour(I)
C     ===========================
C
C
C---- Subroutine to select a pen
C
C     .. Scalar Arguments ..
      INTEGER I
C     ..
C     .. External Subroutines ..
      EXTERNAL GSCOLR
C     ..
C
C          *********
      CALL GSCOLR(I)
C          *********
C
      END
C
C
C
C     ===========================================
      SUBROUTINE zsetpenthickness(NPen_Thickness)
C     ===========================================
C
C
C---- Subroutine to set the line thickness
C
C
C     .. Scalar Arguments ..
      INTEGER NPen_Thickness
C     ..
C     .. External Subroutines ..
      EXTERNAL GSLNWT
C     ..
C
C          **********************
      CALL GSLNWT(NPen_Thickness)
C          **********************
C
      END
C
C
C
C     =============================
      SUBROUTINE zsettextangle(ANG)
C     =============================
C
C---- Subroutine to set text at angle relative to x axis,
C     ANG in degrees
C
C
C     .. Scalar Arguments ..
      REAL ANG
C     ..
C     .. Local Scalars ..
      REAL PI,RADIAN,ROT
C     ..
C     .. External Subroutines ..
      EXTERNAL GSCROT
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ATAN2
C     ..
C
C
      PI = ATAN2(1.0,1.0)*4.0
C
C---- Convert ANG to Radians
C
      ROT = 180.0/PI
      RADIAN = ANG/ROT
C
C          ****************************
      CALL GSCROT(RADIAN,RADIAN+PI/2.0)
C          ****************************
C
      RETURN
      END
C
C
C
C     ===============================
      SUBROUTINE zsettextposition(X,Y)
C     ================================
C
C---- Subroutine to set current position before outputing text
C
C     .. Scalar Arguments ..
      REAL X,Y
C     ..
C     .. External Subroutines ..
      EXTERNAL GSANCU
C     ..
C
C          ***********
      CALL GSANCU(X,Y)
C          ***********
C
      END
C
C     =============================
      SUBROUTINE CLOSEIT(NUM_FILES)
C     =============================
C
C     .. Parameters ..
      INTEGER Max_Files
      PARAMETER (Max_Files=100)
C     ..
C     .. Arguments ..
      INTEGER NUM_FILES
C     ..
C     .. Variables in Common ..
      INTEGER ITEMP,IXYZIN
C     ..
C     .. External Routines ..
      EXTERNAL XYZCLOSE
C     ..
C     .. Common Blocks ..
      COMMON /UNITS/ ITEMP,IXYZIN(Max_Files)
C
C---- get rid of the scratch file
C
      IF (ITEMP.GT.0) CLOSE (UNIT=ITEMP)

      DO 10 I=1,NUM_FILES
        CALL XYZCLOSE(IXYZIN(I))
   10 CONTINUE

      RETURN
      END
C
C     ==============================================
      SUBROUTINE CELLPRESENT(XABC,XCLANG,CELL_GIVEN,LCODE)
C     ==============================================
C
C
C
      INTEGER IFAIL,IUN,JERR
      REAL XABC(3),XCLANG(3),CELLMAP(6),CELL(6)
      LOGICAL CELL_GIVEN
      INTEGER Map_XYZ_Limits(3),MUVW(3),NDIM(5)
      CHARACTER MAP*255,TITLE*80,PDBIN*80,PDBIN1*80
        COMMON /CELLSTUFF/ ROO(4,4),RFF(4,4)
C
C
        CELL_GIVEN = .FALSE.
        CELL(1) = -999.0
        ROO(1,1) = 0.0
C
C
        IFAIL = 0
        IUN = 0
        PDBIN1=' '
        PDBIN=' '
C
C    now checks for xyzin1 being present
C    awa 971009
        CALL UGTENV('XYZIN',PDBIN)
        CALL UGTENV('XYZIN1',PDBIN1)
        IF (PDBIN .NE. ' ') THEN
C            ***************************************
          CALL XYZOPEN('XYZIN','INPUT',' ',IUN,IFAIL)
C            ***************************************
        ELSEIF (PDBIN1 .NE. ' ') THEN
C            ***************************************
          CALL XYZOPEN('XYZIN1','INPUT',' ',IUN,IFAIL)
C            ***************************************
        ELSE 
         IFAIL = -1
        ENDIF
C
        IF (IFAIL .LT. 0) GOTO 50
C
C            *******************
C  Obtain PDB cell if available
        CALL RBCELL(CELL,VOL)
C
        IF (CELL(1) .LE. 0.0) GO TO 50
         XABC(1) = CELL(1)
         XABC(2) = CELL(2)
         XABC(3) = CELL(3)
         XCLANG(1) = CELL(4)
         XCLANG(2) = CELL(5)
         XCLANG(3) = CELL(6)
         CELL_GIVEN = .TRUE.
C         RETURN
C
C
50       CONTINUE
C
        CALL RBRORF2(ROO,RFF,LCODE)
c         write(6,'(A,4(/,4f10.5))')' roo in cellpresent',
c     +                           ((roo(i,j),j=1,4),i=1,4)
c         write(6,'(A,4(/,4f10.5))')' rff in cellpresent',
c     +                           ((rff(i,j),j=1,4),i=1,4)
c         write(6,'(A,I5)') ' Lcode',lcode
        CALL XYZCLOSE(IUN)
C            *******************
C
C   Now check MAP cell
         MAP = ' '
C
C            *******************
        CALL UGTENV('MAPIN',MAP)
C            *******************
C
        IF (MAP .EQ. ' ') GO TO 100
C
C
        CELLMAP(1) = -999.0
C
C          *******************************************************
      CALL MRDHDR(1,'MAPIN',TITLE,NSEC,MUVW,Map_XYZ_Limits,
     +            NDIM(1),NDIM(2),NDIM(3),NDIM(4),NDIM(5),CELLMAP,
     +            LSPGRP,LMODE,RHMIN,RHMAX,RHMEAN,RMS)
C          *******************************************************
C
C
         CALL MRCLOS(1)
C
C
         IF (CELLMAP(1) .LT. 0.0) GO TO 100
            ERRFRC = 0.01
            IF(CELL(1) .GT.0.1) THEN
               JERR = 0
               CALL CELLCHK(CELL,CELLMAP,ERRFRC,JERR)
               IF (JERR.EQ.1)
     +           CALL CCPERR(2,'* Possible error from CELLCHK')
            END IF
         XABC(1) = CELLMAP(1)
         XABC(2) = CELLMAP(2)
         XABC(3) = CELLMAP(3)
         XCLANG(1) = CELLMAP(4)
         XCLANG(2) = CELLMAP(5)
         XCLANG(3) = CELLMAP(6)
         CELL_GIVEN = .TRUE.


C
C
100      CONTINUE
         IF(XABC(1) .GT.0.1) CALL getcellmatrices
         RETURN
C
C             ***************************************
200      CALL CCPERR(1,' *** XYZIN1 is empty !! ***')
C             ***************************************
C
         RETURN
         END
C
C--- PJB: 5/02/1999: COMMENTED OUT S/R CELLCHECK WHICH CLASHES
C    WITH VERSION IN MTZLIB
C
C         SUBROUTINE  CELLCHK(CELL1,CELL2,ERRFRC)
C           REAL CELL1(6),CELL2(6),RRR(3,3,6)
C           CHARACTER WORD*5
CC
CC----      check cell
C           WORD = '  '
CC
C            CALL RBFRO1(CELL1,VOL1,RRR)
C            CALL RBFRO1(CELL2,VOL2,RRR)
C             ACHK = ABS(0.5*(Vol1 - Vol2))/(Vol1 + Vol2)
C             IF(ACHK .GT. ERRFRC) GO TO 10
CC  Check individual dimensions
C                ACHK = 0.00
C                DO 550 II = 1,6
C                  ACHK = ABS(0.5*(Cell2(II)-Cell1(II)))/
C     +                   (Cell2(II)+Cell1(II)) + ACHK
C                  IF (ACHK.GT.3*ERRFRC) WORD='Large'
C                  IF (ACHK.GT.  ERRFRC) WORD='Small'
C 550            CONTINUE
C           IF(WORD.EQ.'  ') RETURN
CC
C  10       CONTINUE
C           WRITE(6,'(/,3A)')'-----',
C     +                 WORD,'Difference in cell parameters detected'
C           WRITE(6,'(A,6F8.3,F14.2)')' ----- First  Cell and Volume :',
C     +                               CELL1,VOL1
C           WRITE(6,'(A,6F8.3,F14.2)')' ----- Second Cell and Volume :',
C     +                               CELL2,VOL2
CC
C           RETURN
C           END
C--- END OF COMMENTED OUT SUBROUTINE CELLCHK





