c
c
c     ===========================
      PROGRAM r500
c     ===========================
c
c
c see end of this file for sample PDB and sample mmcif output files
c
c   to be run after dohlc as LINK records
c   are eliminated from close contact records
c   has to have atoms and residues in remediation form
c   if not run checknames first
c
c    checking compile with
c    ifort r500.f -o r500 -C -check all -g
c    setenv NLSPATH /opt/intel_fc_80/lib/ifcore_msg.cat
c
c    on production
c    ifort r500.f -o r500 -static
c    or
c    f77 r500.f -o r500
c
c    needs
c     setenv STANDATA standard_geometry.cif
c     setenv SYMOP    symop.lib (the ccp4 library)
c
c
c   simply run as
c   r500 1dcode
c   expects pdb4tst.ent to exist
c   gives   RMSD (an overall guide to quality)
c            rem500 PDB REMARK 500 format
c            rem500.cif  more detail in mmcif
c            cispep      if CISPEP found
c
c    r500 idcode -d
c keeps the scratch file for debugging
c
c Bond breaks list at run time e.g.
c  r500 2j0k
cbond break: 0 PRO A 362  THR A 394 Ca(-1)--Ca = 25.996 C(-1)--N = 26.701
cbond break: 0 GLU A 403  THR A 412 Ca(-1)--Ca = 16.095 C(-1)--N = 13.564
cbond break: 0 LEU A 567  LYS A 583 Ca(-1)--Ca = 13.546 C(-1)--N = 13.078
cbond break: 0 PRO B 362  GLY B 383 Ca(-1)--Ca = 21.518 C(-1)--N = 19.418
cbond break: 0 THR B 388  ASP B 395 Ca(-1)--Ca = 16.654 C(-1)--N = 17.263
cbond break: 0 ARG B 569  LEU B 584 Ca(-1)--Ca = 11.745 C(-1)--N = 12.929
c
c
c       note: linking angles in the PDB format are given with the
c             residue name of the central atom
c
c Sample RMSD file when no individual class of bond/angle is poor
c        contains just an overall rmsd
c     1.608  all_angle_rmsd
c     0.016  all_bond_rmsd
c see end of this file for example where thre is poor geometry
c
c
c   REMARK 500, Geometry and Stereochemistry - order of output to be
c   Subtopic: Close Contacts In Same Asymmetric Unit
c   Subtopic: Close Contacts (Symmetry related)
c   Subtopic: Covalent Bond Lengths
c   Subtopic: Covalent Bond Angles
c   Subtopic: Torsion Angles
c   Subtopic: NonCis NonTrans
c   Subtopic: Side Chain Planarity
c   Subtopic: Main Chain Planarity
c   Subtopic: Chiral Centres
c
c
c CLOWNEY ET AL 1996
c Clowney et al. (Geometric Parameters in Nucleic Acids: Nitrogenous
c Bases. Lester Clowney, Shri C. Jain, A. R. Srinivasan, John Westbrook,
c Wilma K. Olson, and Helen M. Berman. (1996), J. Am. Chem. Soc., 118,
c 509-518.)
c
c ENGH AND HUBER, 1999 (is this 2001 or 2006 or 1999 ????)
c Structure quality and target parameters R. A. Engh and
c R. Huber. International Tables for Crystallography (2006). Vol. F,
c ch. 18.3, pp. 382-392
c
c Engh, R.A. & Huber, R. (2001). International Tables for
c Crystallography, Vol F. edited by M.G.Rossmann & E. Arnold, 
c pp 382-392. Dordrecht: Kluwer Academic Publishers
c
c Torsion angles from
c GJ KLEYWEGT AND TA JONES (1996). PHI/PSI-CHOLOGY: 
c RAMACHANDRAN REVISITED. STRUCTURE 4, 1395 - 1400
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,M37,MaxA,MaxSize,MaxSym,MaxLinks,MaxCys
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (M37            = 37)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxA           = 250000)
      PARAMETER (MaxSym         = 230)
      PARAMETER (MaxSize        = 100000000)
      PARAMETER (MaxLinks       = 2000)
      PARAMETER (MaxCys         = 200)
c
c
c  expect
cMaxStandard    37
cNaa            24
cNna            37
cNumBondLinker   4
cNumAngleLinker 11
cMaxAngleTarget 56
cMaxBondTarget  36
cMaxPlanesAtoms 12
cMaxBondLinker   4
cMaxAngleLinker 11
c
cclasses bonds  - 260  aa 1-81, na 101-1998 (used to 212)
cclasses angles - 270  aa 1-99, na 100-235 (used to 255)
c
      INTEGER Nlinks,Nhy,Fpdb,iun,ifail
      CHARACTER NAMFIL*255
      CHARACTER LINKS(MaxLinks)*80
      LOGICAL debug,Qexist,Ldump
      DOUBLE PRECISION Xmax,Xmin,Ymax,Ymin,Zmax,Zmin,
     +                 Hxyz(3,MaxAtoms)
      INTEGER Idnt,Kall,Sall,Msym,Ctran,Ias,Qnum,Snum,Qseq,Qsym
      DOUBLE PRECISION Rsym,S,TR,Qxyz,Sxyz
      REAL    Qocc
      CHARACTER Qalt*1,Qchain*1,Qins*1,Qres*3,Qname*4,Qele*2
      DOUBLE PRECISION Dlim,PlaneCutoff,MainChainPlaneCutoff
      DOUBLE PRECISION cell(6),Rf(4,4),Ro(4,4),Dss
      INTEGER Bmax,NX,NY,NZ,Ncode,L1,L2,L3,Bbad,Abad,Ncys,
     +        Wcys,Wpro,Whis,LLz
      LOGICAL Neutron,NMR
      INTEGER Boxes(MaxSize),
     +        ASclass(2,MaxAclass+MaxAngleLinker),
     +        BSclass(2,MaxBclass+MaxBondLinker)
      INTEGER RamaCore (m37,m37)
      INTEGER Wserial,Wseq,Wseq1,Wseq0,Wmodel,Watoms1,Watoms0,
     +        Nres,Juse,Iphi,Ipsi,Ephipsi,Cdo,J1,K1,
     +        NumArgs,Jdo,Kdo,U61,U31,Wcurrent1,Wcurrent0
      DOUBLE PRECISION X,Y,Z,Wocc,Wb,rmsdCutOff,R
      CHARACTER Wname*4,Walt*1,Wres*3,Wres1*3,Wres0*3,
     +          Wchain*1,Wchain1*1,Wchain0*1,
     +          Wins*1,Wins1*1,Wins0*1,Wele*2,
     +          Wcharge*2,line*80,Ores*3,Fname*255,
     +          Odescriptor*10,Wdescriptor*10,
     +          IDcode*4,Wdes1*10,
     +          Wdes0*10,Wspg*11,Awork*2,Fwork*255
      DOUBLE PRECISION xyz1(3,MaxAtoms),xyz0(3,MaxAtoms)
      CHARACTER Wname1(MaxAtoms)*4,Wname0(MaxAtoms)*4,
     +          Walt1(MaxAtoms)*1,Walt0(MaxAtoms)*1
      DOUBLE PRECISION BondClassEsds(MaxBclass+MaxBondLinker),
     +                 AngleClassEsds(MaxAclass+MaxAngleLinker)
      LOGICAL LastAtm,Dpuck,SSdone(MaxCys)
      LOGICAL Bdone(MaxBondTarget),Adone(MaxAngleTarget)
      INTEGER iargc,CisTrans
      INTEGER QQstan,Naa,Nna
      INTEGER  BAATotalType(MaxBclass+MaxBondLinker),
     +         AAATotalType(MaxAclass+MaxAngleLinker),
     +         ANATotalType(MaxAclass+MaxAngleLinker),
     +         BNATotalType(MaxBclass+MaxBondLinker)
      DOUBLE PRECISION BAArmsdClass(MaxBclass+MaxBondLinker),
     +                 AAArmsdClass(MaxAclass+MaxAngleLinker),
     +                 ANArmsdClass(MaxAclass+MaxAngleLinker),
     +                 BNArmsdClass(MaxBclass+MaxBondLinker)
      DOUBLE PRECISION all_angle_rmsd,all_bond_rmsd,
     +                 Wphi1,Wpsi0,Wphi0,Wpsi1
      INTEGER Num_all_bonds,Num_all_angles
      DOUBLE PRECISION
     +  RMSD(MaxAclass+MaxBclass+MaxAngleLinker+MaxBondLinker)
c
c
      INTEGER NNtype,N1N9,Natoms,NumAtmStanPlane,
     +        NumBondTarget,BondTargetClass,NumAngleTarget,
     +        Ntype1,Ntype0,
     +        AngleTargetClass,NumBondLinker,AAflag,
     +        NumAngleLinker
      CHARACTER StanRes3*3,stanNewNam*4,
     +          PlaneNames*4,
     +          BondPairs*4,AngleTriples*4,
     +          BondLinkerClass*12,BondLinkAtom*4,
     +          AngleLinkerClass*12,
     +          AngleLinkAtom*4,BondDescr*10,
     +          AngleDescr*10
      DOUBLE PRECISION BondTargetValues,BondTargetEsds,
     +                 AngleTargetValues,AngleTargetEsds,
     +                 BondLinkerValues,BondLinkerEsds,
     +                 AngleLinkerValues,AngleLinkerEsds
      COMMON /Cstan/ StanRes3(MaxStandard),
     +       stanNewNam(MaxStandard,MaxAtoms),
     +       PlaneNames(MaxStandard,MaxPlanesAtoms),
     +       BondPairs(2,MaxStandard,MaxBondTarget),
     +       AngleTriples(3,MaxStandard,MaxAngleTarget),
     +       BondLinkerClass(MaxBondLinker),
     +       BondLinkAtom(2,MaxBondLinker),
     +       AngleLinkerClass(MaxAngleLinker),
     +       AngleLinkAtom(3,MaxAngleLinker),
     +       BondDescr(MaxStandard,MaxBondTarget),
     +       AngleDescr(MaxStandard,MaxAngleTarget)
      COMMON /Dstan/
     +       BondTargetClass(MaxStandard,MaxBondTarget),
     +       BondTargetValues(MaxStandard,MaxBondTarget),
     +       BondTargetEsds(MaxStandard,MaxBondTarget),
     +       AngleTargetClass(MaxStandard,MaxAngleTarget),
     +       AngleTargetValues(MaxStandard,MaxAngleTarget),
     +       AngleTargetEsds(MaxStandard,MaxAngleTarget),
     +       BondLinkerValues(MaxBondLinker),
     +       BondLinkerEsds(MaxBondLinker),
     +       AngleLinkerValues(MaxAngleLinker),
     +       AngleLinkerEsds(MaxAngleLinker)
      COMMON /Istan/ NNtype(MaxStandard),
     +       N1N9(MaxStandard),
     +       Natoms(MaxStandard),
     +       NumAtmStanPlane(MaxStandard),
     +       NumBondTarget(MaxStandard),
     +       NumAngleTarget(MaxStandard),
     +       AAflag(MaxStandard),
     +       NumBondLinker,NumAngleLinker
c
c
      INTEGER Catoms,Cseq
      COMMON /Icys/ Catoms(MaxCys),Cseq(MaxCys)
      CHARACTER Cchain*1,Cins*1,Cname*4,Calt*1
      COMMON /Ccys/ Cchain(MaxCys),Cins(MaxCys),
     +              Cname(MaxCys,MaxAtoms),
     +              Calt(MaxCys,MaxAtoms)
      DOUBLE PRECISION Cxyz
      COMMON /Dcys/ Cxyz(MaxCys,3,MaxAtoms)
c
c
      COMMON /W0/Xmin,Xmax,Ymin,Ymax,Zmin,Zmax
      COMMON /W1/S(4,4,MaxSym),Rsym(4,4,MaxSym),Sxyz(3,2*MaxA)
      COMMON /W11/Snum(2*MaxA),Ias(2*MaxA),Qsym(2*MaxA),Sall,Msym,
     +       Idnt
      COMMON /W3/Qseq(MaxA)
      COMMON /W4/Qnum(MaxA),Kall
      COMMON /W41/Qxyz(3,MaxA)
      COMMON /W7/Ctran(2*MaxA)
      COMMON /W71/TR(4,4,MaxSym)
      COMMON /W9/Qins(MaxA),Qalt(MaxA),Qname(MaxA),Qres(MaxA),
     +           Qchain(MaxA),Qele(MaxA)
      COMMON /W31/Qocc(MaxA)
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      INTEGER r500_lenstr
c
c values from
c http://xray.bmc.uu.se/gerard/supmat/ramarev.html
c
       DATA ((RamaCore(Kdo,Jdo),Kdo=1,M37),Jdo=1,M37)/
     +  0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0,
     +  1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1,
     +  1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     +  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
     +  1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1,
     +  1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1,
     +  1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
     +  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
     +  1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1,
     +  1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1,
     +  1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
     +  1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     +  1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
     +  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     +  1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1,
     +  1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     +  0, 0, 0, 0/
c
c
c
       Iscr   = 21
       Ipdb   = 22
       Irmsd  = 23
       Irout  = 24
       Icif   = 25
       Istan  = 26
       Icis   = 27
       Idump  = 28
c
c
      Ncys    = 0
      Nlinks  = 0
      Dlim    = 2.2D0
      Xmin    = 1000.0D0
      Xmax    = -1000.0D0
      Ymin    = 1000.0D0
      Ymax    = -1000.0D0
      Zmin    = 1000.0D0
      Zmax    = -1000.0D0
      Kall    = 0
      Neutron = .false.
      debug   = .false.
      Ldump   = .false.
c
c

CCC CCP4:
CCC Set up the CCP4 environment
      CALL CCPFYP
      CALL CCPRCS (6, 'R500', '$Date: 2008/07/22 15:20:58 $')

      CALL XYZINIT()
      iun=0
      ifail=0

      CALL XYZOPEN( 'XYZIN', 'INPUT',' ',   iun, ifail)

      CALL UGTENV('XYZIN',NAMFIL)

      NumArgs = iargc()
      IF (numargs .le. 0) THEN
      write(6,'(16(a,/))') 'Run as ',
     + ' r500 idcode ',
     + ' expects file pdbIDCODE.ent to exist',
     + ' gives files rem500 in PDB format and rem500.cif',
     + ' Note: must have atom names and residue names as 1stJuly 2007',
     + '       needs STANDATA as standard_geometry.cif env',
     + '       needs SYMOP as symop.lib env (use the CCP4 one)',
     + ' Optional flags  -d sets debug true and keps scratch file',
     + '                 -u sets dump true and writes out E&H values',
     + ' Aternative way of running not using pdbIDCODE.ent ',
     + '   run as r500 -i my_file_name (may use -d & -u )',
     + '   here o/p is my_file_namerem500 & my_file_namerem500.cif',
     + '   The file must be close to a true PDB file as the swich to',
     + '    symmetry contacts is based on CRYST1 which is Mandatory'
      STOP
       END IF
      CALL getarg(1,IDcode)

      Numargs=1
      debug = .false.
      Ldump = .false.
c   
c

      IF (IDcode(1:2) .eq. '-i' .or.
     +    IDcode(1:2) .eq. '-I') THEN
        CALL getarg(2,Fwork)
        Qexist = .false.
        INQUIRE (FILE=Fwork,EXIST=Qexist)
        IF (.not. Qexist) THEN
         write(6,'(a)') 'Error cant find file ',
     +      Fwork(1:r500_lenstr(Fwork))
         STOP
        END IF
        Fpdb = 1
        OPEN (UNIT=Ipdb,FILE=Fwork,STATUS='OLD')
        IF (NumArgs .gt.2 ) THEN
         CALL getarg(3,Awork)
         IF (Awork(1:2) .eq. '-d' .or. Awork(1:2) .eq. '-D')
     +     debug = .true.
         IF (Awork(1:2) .eq. '-u' .or. Awork(1:2) .eq. '-U')
     +     Ldump = .true.
        END IF
        IF (NumArgs .gt.3 ) THEN
         CALL getarg(4,Awork)
         IF (Awork(1:2) .eq. '-d' .or. Awork(1:2) .eq. '-D')
     +    debug = .true.
         IF (Awork(1:2) .eq. '-u' .or. Awork(1:2) .eq. '-U')
     +    Ldump = .true.
        END IF
        Fname = './RMSD'
        OPEN (UNIT=Irmsd,FILE=Fname,STATUS='UNKNOWN')
        Fname = Fwork(1:r500_lenstr(Fwork))//'.scratch'
        OPEN (UNIT = Iscr,FILE=Fname,STATUS='UNKNOWN')
        IDcode = Fwork(1:4)
      ELSE
c        Fname = './pdb'//IDcode//'.ent'
CCC CCP4:
        Fname = NAMFIL
        Qexist = .false.
        INQUIRE (FILE=Fname,EXIST=Qexist)
        IF (.not. Qexist) THEN
         write(6,'(a)') 'Error cant find file ',
     +      Fname(1:r500_lenstr(Fname))
         STOP
        END IF
        Fpdb = 0
        OPEN (UNIT=Ipdb,FILE=Fname,STATUS='OLD')
        IF (NumArgs .gt.1 ) THEN
         CALL getarg(2,Awork)
         IF (Awork(1:2) .eq. '-d' .or. Awork(1:2) .eq. '-D')
     +     debug = .true.
         IF (Awork(1:2) .eq. '-u' .or. Awork(1:2) .eq. '-U')
     +     Ldump = .true.
        END IF
        IF (NumArgs .gt.2 ) THEN
         CALL getarg(3,Awork)
         IF (Awork(1:2) .eq. '-d' .or. Awork(1:2) .eq. '-D')
     +    debug = .true.
         IF (Awork(1:2) .eq. '-u' .or. Awork(1:2) .eq. '-U')
     +    Ldump = .true.
        END IF
        Fname = './RMSD'
        OPEN (UNIT=Irmsd,FILE=Fname,STATUS='UNKNOWN')
        Fname = IDcode//'.scratch'
        OPEN (UNIT = Iscr,FILE=Fname,STATUS='UNKNOWN')
      END IF

c
c
      CALL StandardRead(QQstan,Naa,Nna,Ldump,Wcys,Wpro,Whis)
c
c
      DO Jdo=1,MaxCys
       SSdone(Jdo) = .false.
      ENDDO
      DO Jdo=1,MaxBclass+MaxBondLinker
       BondClassEsds(Jdo) = 0.0d0
      ENDDO
      DO Jdo=1,MaxAclass+MaxAngleLinker
       AngleClassEsds(Jdo) = 0.0d0
      ENDDO
      DO Jdo = 1,MaxStandard
       DO Kdo=1,MaxBondTarget
        Juse = BondTargetClass(Jdo,Kdo)
        IF (Juse .gt. 0) THEN
         BondClassEsds(Juse) = BondTargetEsds(Jdo,Kdo)
         BSclass(1,Juse) = Jdo
         BSclass(2,Juse) = Kdo
        END IF
       ENDDO
      ENDDO
       DO Jdo=1,MaxBondLinker
        BondClassEsds(Jdo+MaxBclass) = BondLinkerEsds(Jdo)
       ENDDO
       DO Jdo=1,MaxAngleLinker
        AngleClassEsds(Jdo+MaxAclass) = AngleLinkerEsds(Jdo)
       ENDDO
c
c
      DO Jdo = 1,MaxStandard
       DO Kdo=1,MaxAngleTarget
        Juse = AngleTargetClass(Jdo,Kdo)
        IF (Juse .gt. 0) THEN
         AngleClassEsds(Juse) = AngleTargetEsds(Jdo,Kdo)
         ASclass(1,Juse) = Jdo
         ASclass(2,Juse) = Kdo
        END IF
       ENDDO
      ENDDO
      DO Jdo=1,MaxBclass+MaxBondLinker
       BAArmsdClass(Jdo) = 0.0d0
       BAATotalType(Jdo) = 0
       BNArmsdClass(Jdo) = 0.0d0
       BNATotalType(Jdo) = 0
      ENDDO
      DO Jdo = 1,MaxAclass+MaxAngleLinker
       AAArmsdClass(Jdo) = 0.0d0
       AAATotalType(Jdo) = 0
       ANArmsdClass(Jdo) = 0.0d0
       ANATotalType(Jdo) = 0
      ENDDO
c
c
      Dss          = 2.30d0*2.30d0
      rmsdCutOff   = 6.0d0
      PlaneCutoff  = 0.02d0
      PlaneCutoff  = PlaneCutoff*rmsdCutOff
      MainChainPlaneCutoff = 10.0d0
      U61          = 0
      U31          = 0
      Dpuck        = .false.
      Wmodel       = 0
      ODescriptor  = '@'
      Watoms1      = 0
      Watoms0      = 0
      Nhy          = 0
      Nres         = 0
      LastAtm      = .false.
c
c
      Wphi1 = MissingValue
      Wpsi0 = MissingValue
      Wphi0 = MissingValue
      Wpsi1 = MissingValue
c
c
      NMR = .false.
c
c
   20 read (Ipdb,fmt='(a)',END=9999) line
c
c
      IF (line(1:6) .eq. 'EXPDTA') THEN
       IF (index(line,'NEUTRON').ne.0) Neutron=.true.
       IF (index(line,'X-RAY').ne.0) NMR=.false.
       IF (index(line,'NMR').ne.0) NMR=.true.
       GO TO 20
      ELSE IF (line(1:6) .eq. 'LINK  ') THEN
c
cLINK         C   CME A 297                 N   MET A 298     1555   1555
c123456789012345678901234567890123456789012345678901234567890123456789012
cxxxxxxxxxxxx123456789012345xxxxxxxxxxxxxxx123456789012345
c
       Nlinks = Nlinks + 1
       IF (Nlinks .gt. MaxLinks) STOP 'max links exceeded'
       LINKS(Nlinks) = line
       GO TO 20
      ELSE IF (line(1:6) .eq. 'CRYST1') THEN
        Wspg = line(56:66)
        READ (line,fmt='(6X,3F9.3,3f7.2)') cell
        CALL GetSymm(cell,Wspg)
        Ncode = 1
        CALL OrtMat(cell,Ncode,Ro,Rf)
        LLz = nint(cell(1) + cell(2) + cell(3))
        IF (LLz .eq. 3) NMR = .true.
        GO TO 20
      ELSE IF (line(1:6) .eq. 'END   ' .or.
     +    line(1:6) .eq. 'MASTER' .or.
     +    line(1:6) .eq. 'CONECT' ) THEN
       IF (LastAtm .and. Watoms1 .gt. 0) THEN
         Wphi1 = MissingValue
         Wpsi0 = MissingValue
         CALL Linker(Wmodel,Ntype1,Watoms1,Wseq1,
     +                xyz1,Wname1,Walt1,
     +                Wins1,Wchain1,Wres1,Wdes1,
     +                Ntype0,Watoms0,Wseq0,
     +                xyz0,Wname0,Walt0,
     +                Wins0,Wchain0,Wres0,Wdes0,
     +                Wphi1,Wpsi0,
     +                BAATotalType,AAATotalType,
     +                ANATotalType,BNATotalType,
     +                BAArmsdClass,AAArmsdClass,
     +                ANArmsdClass,BNArmsdClass,
     +                MainChainPlaneCutoff,
     +                PlaneCutoff,CisTrans,debug)
c
c---- only do PRO after deciding if it is cis/trans
c
       IF (Wres1 .eq. 'PRO') THEN
          CALL Proline(Wpro,Watoms1,Wname1,xyz1,Walt1,
     +                   BAArmsdClass,BAATotalType,
     +                   AAArmsdClass,AAATotalType,
     +                   Wres1,Wchain1,Wseq1,Wins1,
     +                   Wmodel,CisTrans,Bdone,Adone)
          CALL GetChiral(Watoms1,Wres1,Wseq1,Wins1,Wchain1,
     +                   Wmodel,Walt1,xyz1,Wname1)
        END IF
c
c
        Ephipsi = 0
        IF (Wphi0 .gt. MissingValue + 1.0d0 .and.
     +      Wpsi0 .gt. MissingValue + 1.0d0) THEN
        IF (Wres0 .ne. 'GLY') THEN
         IF (Wphi0 .gt. MissingValue + 1.0d0 .and.
     +       Wpsi0 .gt. MissingValue + 1.0d0) THEN
          Iphi = int ( (180.0d0+Wphi0) / 10.0d0 ) + 1
          Ipsi = int ( (180.0d0+Wpsi0) / 10.0d0 ) + 1
          IF (Iphi .ge. 1 .and. Iphi .le. 37 .and.
     +        Ipsi .ge. 1 .and. Ipsi .le. 37) THEN
           IF (RamaCore(Iphi,Ipsi) .ne. 1) Ephipsi = 1
          END IF
         END IF
        END IF
        IF (Ephipsi .eq. 1 .or. debug) THEN
c
c---- bad phi/psi for last residue
c
          WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'RAM',Juse,Wmodel,Wpsi0,Wphi0,Ephipsi,
     +      Wres0,Wchain0,Wseq0,Wins0,'    ',' ',
     +      Wres0,Wchain0,Wseq0,Wins0,'    ',' ',
     +      Wres0,Wchain0,Wseq0,Wins0,'    ','  '
        END IF
        END IF
       END IF
       GOTO 9999
c
c
      ELSE IF (line(1:6) .eq. 'MODEL ') THEN
       read (line,fmt='(10X,i4)') Wmodel
       IF (LastAtm .and. Watoms1 .gt. 0) THEN
         Wphi1 = MissingValue
         Wpsi0 = MissingValue
         CALL Linker(Wmodel,Ntype1,Watoms1,Wseq1,
     +                xyz1,Wname1,Walt1,
     +                Wins1,Wchain1,Wres1,Wdes1,
     +                Ntype0,Watoms0,Wseq0,
     +                xyz0,Wname0,Walt0,
     +                Wins0,Wchain0,Wres0,Wdes0,
     +                Wphi1,Wpsi0,
     +                BAATotalType,AAATotalType,
     +                ANATotalType,BNATotalType,
     +                BAArmsdClass,AAArmsdClass,
     +                ANArmsdClass,BNArmsdClass,
     +                MainChainPlaneCutoff,
     +                PlaneCutoff,CisTrans,debug)
c
c---- only do PRO after deciding if it is cis/trans
c
       IF (Wres1 .eq. 'PRO') THEN
          CALL Proline(Wpro,Watoms1,Wname1,xyz1,Walt1,
     +                   BAArmsdClass,BAATotalType,
     +                   AAArmsdClass,AAATotalType,
     +                   Wres1,Wchain1,Wseq1,Wins1,
     +                   Wmodel,CisTrans,Bdone,Adone)
          CALL GetChiral(Watoms1,Wres1,Wseq1,Wins1,Wchain1,
     +                   Wmodel,Walt1,xyz1,Wname1)
        END IF
c
c
        Ephipsi = 0
        IF (Wphi0 .gt. MissingValue + 1.0d0 .and.
     +      Wpsi0 .gt. MissingValue + 1.0d0) THEN
        IF (Wres0 .ne. 'GLY') THEN
         IF (Wphi0 .gt. MissingValue + 1.0d0 .and.
     +       Wpsi0 .gt. MissingValue + 1.0d0) THEN
          Iphi = int ( (180.0d0+Wphi0) / 10.0d0 ) + 1
          Ipsi = int ( (180.0d0+Wpsi0) / 10.0d0 ) + 1
          IF (Iphi .ge. 1 .and. Iphi .le. 37 .and.
     +        Ipsi .ge. 1 .and. Ipsi .le. 37) THEN
           IF (RamaCore(Iphi,Ipsi) .ne. 1) Ephipsi = 1
          END IF
         END IF
        END IF
        IF (Ephipsi .eq. 1 .or. debug) THEN
c
c---- bad phi/psi for last residue
c
          WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'RAM',Juse,Wmodel,Wpsi0,Wphi0,Ephipsi,
     +      Wres0,Wchain0,Wseq0,Wins0,'    ',' ',
     +      Wres0,Wchain0,Wseq0,Wins0,'    ',' ',
     +      Wres0,Wchain0,Wseq0,Wins0,'    ','  '
        END IF
        END IF
       END IF
c
c
       Wphi1       = MissingValue
       Wpsi0       = MissingValue
       Wphi0       = MissingValue
       Wpsi1       = MissingValue
       Watoms1     = 0
       Watoms0     = 0
       Nhy         = 0
       LastAtm     = .false.
       ODescriptor = '@'
       Ores        = '@'
       GO TO 20
c
c
      ELSE IF (line(1:6) .eq. 'TER   ') THEN
       IF (Watoms1 .gt. 0) THEN
c
c---- cant do CYS-CYS untl all atoms read to look for
c     di-suphide - here only doing one by one
c
          IF (Wres1 .eq. 'CYS' .and. Wmodel .le. 1) THEN
           Ncys = Ncys + 1
           CALL CysStore(Ncys,Watoms1,Wname1,xyz1,Walt1,
     +                   Wres1,Wchain1,Wseq1,Wins1,Wmodel)
          ELSE IF (Wres1 .eq. 'HIS') THEN
           CALL Histidine(Whis,Watoms1,Wname1,xyz1,Walt1,
     +                BAArmsdClass,BAATotalType,
     +                AAArmsdClass,AAATotalType,
     +                Wres1,Wchain1,Wseq1,Wins1,
     +                Wmodel,CisTrans,
     +                Bdone,Adone,Nhy,Hxyz)
          ELSE IF (Wres1 .ne. 'PRO') THEN
           CALL Process(Wcurrent1,
     +         Watoms1,Wname1,xyz1,Walt1,
     +         Wres1,Wchain1,Wseq1,Wins1,Wmodel,
     +         Ntype1,U31,
     +         BAATotalType,AAATotalType,
     +         ANATotalType,BNATotalType,
     +         BAArmsdClass,AAArmsdClass,
     +         ANArmsdClass,BNArmsdClass,
     +         N1N9,StanRes3,IDcode,CisTrans,
     +         Bdone,Adone,Fpdb,Fwork)
           END IF
c
c
          CALL GetPlanes(Wcurrent1,NumAtmStanPlane,Wseq1,
     +                   Wchain1,Wmodel,Wdes1,Watoms1,Wname1,
     +                   PlaneNames,xyz1,Walt1,PlaneCutoff,
     +                   debug)
       END IF
       IF (LastAtm) THEN
         Wphi1 = MissingValue
         Wpsi0 = MissingValue
         CALL Linker(Wmodel,Ntype1,Watoms1,Wseq1,
     +                xyz1,Wname1,Walt1,
     +                Wins1,Wchain1,Wres1,Wdes1,
     +                Ntype0,Watoms0,Wseq0,
     +                xyz0,Wname0,Walt0,
     +                Wins0,Wchain0,Wres0,Wdes0,
     +                Wphi1,Wpsi0,
     +                BAATotalType,AAATotalType,
     +                ANATotalType,BNATotalType,
     +                BAArmsdClass,AAArmsdClass,
     +                ANArmsdClass,BNArmsdClass,
     +                MainChainPlaneCutoff,
     +                PlaneCutoff,CisTrans,debug)
c
c---- only do PRO after deciding if it is cis/trans
c
       IF (Wres1 .eq. 'PRO') THEN
          CALL Proline(Wpro,Watoms1,Wname1,xyz1,Walt1,
     +                   BAArmsdClass,BAATotalType,
     +                   AAArmsdClass,AAATotalType,
     +                   Wres1,Wchain1,Wseq1,Wins1,
     +                   Wmodel,CisTrans,Bdone,Adone)
          CALL GetChiral(Watoms1,Wres1,Wseq1,Wins1,Wchain1,
     +                   Wmodel,Walt1,xyz1,Wname1)
        END IF
c
c
        Ephipsi = 0
        IF (Wpsi0 .gt. MissingValue + 1.0d0 .and.
     +      Wphi0 .gt. MissingValue + 1.0d0) THEN
        IF (Wres0 .ne. 'GLY') THEN
         IF (Wphi0 .gt. MissingValue + 1.0d0 .and.
     +       Wpsi0 .gt. MissingValue + 1.0d0) THEN
          Iphi = int ( (180.0d0+Wphi0) / 10.0d0 ) + 1
          Ipsi = int ( (180.0d0+Wpsi0) / 10.0d0 ) + 1
          IF (Iphi .ge. 1 .and. Iphi .le. 37 .and.
     +        Ipsi .ge. 1 .and. Ipsi .le. 37) THEN
           IF (RamaCore(Iphi,Ipsi) .ne. 1) Ephipsi = 1
          END IF
         END IF
        END IF
        IF (Ephipsi .eq. 1 .or. debug) THEN
c
c---- bad phi/psi for last residue
c
          WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'RAM',Juse,Wmodel,Wpsi0,Wphi0,Ephipsi,
     +      Wres0,Wchain0,Wseq0,Wins0,'    ',' ',
     +      Wres0,Wchain0,Wseq0,Wins0,'    ',' ',
     +      Wres0,Wchain0,Wseq0,Wins0,'    ','  '
        END IF
        END IF
       END IF
c
c
       Wphi1       = MissingValue
       Wpsi0       = MissingValue
       Wphi0       = MissingValue
       Wpsi1       = MissingValue
       Watoms1     = 0
       Watoms0     = 0
       Nhy         = 0
       LastAtm     = .false.
       ODescriptor = '@'
       Ores        = '@'
       GO TO 20
c
c
      ELSE IF (line(1:6) .eq. 'ATOM  ' .or.
     +         line(1:6) .eq. 'HETATM') THEN
       read (line,fmt='(6X,i5,1X,a4,a1,a3,1X,a1,i4,
     +        a1,3X,3f8.3,2f6.2,10X,a2,a2)') Wserial,
     +        Wname,Walt,Wres,Wchain,Wseq,Wins,
     +        X,Y,Z,Wocc,Wb,Wele,Wcharge
c
c--- prepare for contacts
c
        IF (Wmodel .gt. 1) GO TO 25
        IF (Wocc.lt.1.00D0) GO TO 25
        IF (index(Wname,'Q') .ne.0) GO TO 25
        Kall = Kall + 1
        IF (Kall.gt.MaxA) STOP 'Max atoms exceeded'
        Qnum(Kall) = Kall
        IF (X.lt.Xmin) Xmin = X
        IF (X.gt.Xmax) Xmax = X
        IF (Y.lt.Ymin) Ymin = Y
        IF (Y.gt.Ymax) Ymax = Y
        IF (Z.lt.Zmin) Zmin = Z
        IF (Z.gt.Zmax) Zmax = Z
        Qname(Kall)  = Wname
        Qres(Kall)   = Wres
        Qchain(Kall) = Wchain
        Qseq(Kall)   = Wseq
        Qxyz(1,Kall) = X
        Qxyz(2,Kall) = Y
        Qxyz(3,Kall) = Z
        Qins(Kall)   = Wins
        Qalt(Kall)   = Walt
        Qocc(Kall)   = Wocc
        Qele(Kall)   = Wele
c
c
25      CONTINUE
       Wdescriptor = line(18:27)
       DO Jdo=1,QQstan
        IF (Wres .eq. StanRes3(Jdo)) GO TO 35
       ENDDO
c
c---- skip non standard residues
c
       GO TO 20
35     CONTINUE
c
c---- is HIS protonated
c
       IF (Wres .eq. 'HIS') THEN
        IF (Wele .eq. 'H' .or. Wele .eq. ' D') THEN
         Nhy         = Nhy + 1
         Hxyz(1,Nhy) = X
         Hxyz(2,Nhy) = Y
         Hxyz(3,Nhy) = Z
        END IF
       END IF
c
c---- skip hydrogens and deuterium
c
       IF (Wele .eq. 'H' .or. Wele .eq. ' D') GO TO 20
c
       IF (Wdescriptor .eq. Odescriptor) THEN
        Watoms1         = Watoms1 + 1
        xyz1(1,Watoms1) = X
        xyz1(2,Watoms1) = Y
        xyz1(3,Watoms1) = Z
        Wname1(Watoms1) = Wname
        Walt1(Watoms1)  = Walt
        GO TO 20
       END IF
c
c
       IF (Wdescriptor.ne.Odescriptor) THEN
        IF (Watoms1 .eq. 0) THEN
         DO Jdo=1,QQstan
          IF (Wres .eq. StanRes3(Jdo)) THEN
           Wcurrent1 = Jdo
           GO TO 45
          END IF
         ENDDO
         GO TO 20
45       CONTINUE
         Ntype1          = AAflag(Wcurrent1)
         Watoms1         = 1
         xyz1(1,Watoms1) = X
         xyz1(2,Watoms1) = Y
         xyz1(3,Watoms1) = Z
         Wname1(Watoms1) = Wname
         Walt1(Watoms1)  = Walt
         Wins1           = Wins
         Wchain1         = Wchain
         Wseq1           = Wseq
         Wres1           = Wres
         Wdes1           = Wdescriptor
         Odescriptor     = Wdescriptor
         Wphi0           = Wphi1
         GO TO 20
       ELSE IF (Watoms1.ne.0) THEN
          IF (Wres1 .eq. 'CYS' .and. Wmodel .le. 1) THEN
           Ncys = Ncys + 1
           CALL CysStore(Ncys,Watoms1,Wname1,xyz1,Walt1,
     +                   Wres1,Wchain1,Wseq1,Wins1,Wmodel)
          ELSE IF (Wres1 .eq. 'HIS') THEN
           CALL Histidine(Whis,Watoms1,Wname1,xyz1,Walt1,
     +                BAArmsdClass,BAATotalType,
     +                AAArmsdClass,AAATotalType,
     +                Wres1,Wchain1,Wseq1,Wins1,
     +                Wmodel,CisTrans,
     +                Bdone,Adone,Nhy,Hxyz)
          ELSE IF (Wres1 .ne. 'PRO') THEN
           CALL Process(Wcurrent1,
     +         Watoms1,Wname1,xyz1,Walt1,
     +         Wres1,Wchain1,Wseq1,Wins1,Wmodel,
     +         Ntype1,U31,
     +         BAATotalType,AAATotalType,
     +         ANATotalType,BNATotalType,
     +         BAArmsdClass,AAArmsdClass,
     +         ANArmsdClass,BNArmsdClass,
     +         N1N9,StanRes3,IDcode,CisTrans,
     +         Bdone,Adone,Fpdb,Fwork)
           END IF
c
c
        CALL GetPlanes(Wcurrent1,NumAtmStanPlane,Wseq1,Wchain1,
     +                   Wmodel,Wdes1,Watoms1,Wname1,
     +                   PlaneNames,xyz1,Walt1,PlaneCutoff,
     +                   debug)
        IF (LastAtm) THEN
         Wphi1 = MissingValue
         Wpsi0 = MissingValue
         CALL Linker(Wmodel,Ntype1,Watoms1,Wseq1,
     +                xyz1,Wname1,Walt1,
     +                Wins1,Wchain1,Wres1,Wdes1,
     +                Ntype0,Watoms0,Wseq0,
     +                xyz0,Wname0,Walt0,
     +                Wins0,Wchain0,Wres0,Wdes0,
     +                Wphi1,Wpsi0,
     +                BAATotalType,AAATotalType,
     +                ANATotalType,BNATotalType,
     +                BAArmsdClass,AAArmsdClass,
     +                ANArmsdClass,BNArmsdClass,
     +                MainChainPlaneCutoff,
     +                PlaneCutoff,CisTrans,debug)
c
c---- only do PRO after deciding if it is cis/trans
c
       IF (Wres1 .eq. 'PRO') THEN
          CALL Proline(Wpro,Watoms1,Wname1,xyz1,Walt1,
     +                   BAArmsdClass,BAATotalType,
     +                   AAArmsdClass,AAATotalType,
     +                   Wres1,Wchain1,Wseq1,Wins1,
     +                   Wmodel,CisTrans,Bdone,Adone)
          CALL GetChiral(Watoms1,Wres1,Wseq1,Wins1,Wchain1,
     +                   Wmodel,Walt1,xyz1,Wname1)
        END IF
c
c
        Ephipsi = 0
        IF (Wpsi0 .gt. MissingValue + 1.0d0 .and.
     +      Wphi0 .gt. MissingValue + 1.0d0) THEN
        IF (Wres0 .ne. 'GLY') THEN
         IF (Wphi0 .gt. MissingValue + 1.0d0 .and.
     +       Wpsi0 .gt. MissingValue + 1.0d0) THEN
          Iphi = int ( (180.0d0+Wphi0) / 10.0d0 ) + 1
          Ipsi = int ( (180.0d0+Wpsi0) / 10.0d0 ) + 1
          IF (Iphi .ge. 1 .and. Iphi .le. 37 .and.
     +        Ipsi .ge. 1 .and. Ipsi .le. 37) THEN
           IF (RamaCore(Iphi,Ipsi) .ne. 1) Ephipsi = 1
          END IF
         END IF
        END IF
        IF (Ephipsi .eq. 1 .or. debug) THEN
c
c---- bad phi/psi for last residue
c
          WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'RAM',0,Wmodel,Wpsi0,Wphi0,Ephipsi,
     +      Wres0,Wchain0,Wseq0,Wins0,'    ',' ',
     +      Wres0,Wchain0,Wseq0,Wins0,'    ',' ',
     +      Wres0,Wchain0,Wseq0,Wins0,'    ','  '
        END IF
        END IF
       END IF
c
c--- move current residue to last residue
c
        DO Jdo=1,Watoms1
         xyz0(1,Jdo) = xyz1(1,Jdo)
         xyz0(2,Jdo) = xyz1(2,Jdo)
         xyz0(3,Jdo) = xyz1(3,Jdo)
         Wname0(Jdo) = Wname1(Jdo)
         Walt0(Jdo)  = Walt1(Jdo)
        ENDDO
         Watoms0         = Watoms1
         Wins0           = Wins1
         Wchain0         = Wchain1
         Wseq0           = Wseq1
         Wres0           = Wres1
         Wdes0           = Wdes1
         Wcurrent0       = Wcurrent1
         Ntype0          = Ntype1
         LastAtm         = .true.
         Odescriptor     = Wdescriptor
         Wphi0           = Wphi1
         DO Jdo=1,QQstan
          IF (Wres .eq. StanRes3(Jdo)) THEN
           Wcurrent1 = Jdo
           GO TO 145
          END IF
         ENDDO
         GO TO 20
145      CONTINUE
         Ntype1          = AAflag(Wcurrent1)
         Watoms1         = 1
         xyz1(1,Watoms1) = X
         xyz1(2,Watoms1) = Y
         xyz1(3,Watoms1) = Z
         Wname1(Watoms1) = Wname
         Walt1(Watoms1)  = Walt
         Wins1           = Wins
         Wchain1         = Wchain
         Wseq1           = Wseq
         Wres1           = Wres
         Wdes1           = Wdescriptor
         GO TO 20
        END IF
       END IF
       END IF
       GO TO 20
c
c---- finished PDB read
c
9999    CONTINUE
        CLOSE (UNIT=Ipdb)
c
c---- do contacts no-symmetry
c     boxes 6 x 6 x 6 Angstr.
c
      IF (Xmin.lt.0.0D0) THEN
        NX = nint((Xmax-Xmin+6.1D0)/6.0D0) + 3
      ELSE
        NX = INT((Xmax-Xmin)/6.0D0) + 3
      END IF
      IF (Ymin.lt.0.0D0) THEN
        NY = nint((Ymax-Ymin+6.1D0)/6.0D0) + 3
      ELSE
        NY = INT((Ymax-Ymin)/6.0D0) + 3
      END IF
      IF (Zmin.lt.0.0D0) THEN
        NZ = nint((Zmax-Zmin+6.1D0)/6.0D0) + 3
      ELSE
        NZ = INT((Zmax-Zmin)/6.0D0) + 3
      END IF
      Bmax = MaxSize/51 - 1
      IF (NX*NY*NZ.gt.Bmax) STOP 'Too many boxes'
      CALL SetNet(NX,NY,NZ,Boxes,Dlim)
      CALL GetCons(Xmin,Ymin,Zmin,NX,NY,NZ,Boxes,Dlim,Neutron)
      IF (Ncys .gt. 0) 
     + CALL GetDisulphide(Xmin,Ymin,Zmin,NX,NY,NZ,
     +                   Boxes,Ncys,Wcys,Dss,SSdone,
     +                   BAArmsdClass,BAATotalType,
     +                   AAArmsdClass,AAATotalType,
     +                   BondLinkerValues,Bdone,Adone)
c
c
      IF (.not. NMR) THEN
c
c----   symmetry related atoms generated here
c
       CALL SymMat(Ro,Rf)
       CALL SymGen(Dlim)
       CALL SetNet2(Xmin,Ymin,Zmin,NX,NY,NZ,Snum,Sxyz,Sall,Boxes)
       CALL GetCons2(Xmin,Ymin,Zmin,NX,NY,NZ,Boxes,Dlim)
       IF (Ncys .gt. 0) 
     +  CALL SymmDisulphide(Xmin,Ymin,Zmin,NX,NY,NZ,
     +                      Boxes,Ncys,Wcys,Dss,SSdone,
     +                      BAArmsdClass,BAATotalType,
     +                      AAArmsdClass,AAATotalType,
     +                      BondLinkerValues,Bdone,Adone)
      END IF
c
c
      IF (Ncys .gt. 0) THEN
c
c---- do the non di-sulphide CYS
c
       DO Cdo=1,Ncys
        IF (.not. SSdone(Cdo)) THEN
         Wchain1 = Cchain(Cdo)
         Wins1   = Cins(Cdo)
         Wseq1   = Cseq(Cdo)
         Wmodel  = 0
         Watoms1 = Catoms(Cdo)
         DO K1=1,MaxAtoms
          DO J1=1,3
           xyz1(J1,K1) = Cxyz(Cdo,J1,K1)
          ENDDO
          Wname1(K1) = Cname(Cdo,K1) 
          Walt1(K1)  = Calt(Cdo,K1)
         ENDDO
         CisTrans = -100
         CALL Cystine(Wcys,Watoms1,Wname1,xyz1,Walt1,
     +                BAArmsdClass,BAATotalType,
     +                AAArmsdClass,AAATotalType,
     +                Wres1,Wchain1,Wseq1,Wins1,
     +                Wmodel,CisTrans,Bdone,Adone)
         SSdone(Cdo) = .true. 
        END IF
       ENDDO
      END IF
c
c---- calculate rmsd per type for diffs in this structure
c
      all_angle_rmsd = 0.0d0
      all_bond_rmsd  = 0.0d0
      Num_all_bonds  = 0
      Num_all_angles = 0
c
c
      DO Jdo = 1,MaxBclass+MaxBondLinker
        RMSD(Jdo) = 0.0d0
        IF (BAATotalType(Jdo).gt.0)  THEN
         RMSD(Jdo) = sqrt(BAArmsdClass(Jdo)/BAATotalType(Jdo))
          all_bond_rmsd = all_bond_rmsd + BAArmsdClass(jdo)
          Num_all_bonds = Num_all_bonds + BAATotalType(jdo)
        END IF
        IF (BNATotalType(Jdo).gt.0)  THEN
         RMSD(Jdo) = sqrt(BNArmsdClass(Jdo)/BNATotalType(Jdo))
          all_bond_rmsd = all_bond_rmsd + BNArmsdClass(jdo)
          Num_all_bonds = Num_all_bonds + BNATotalType(jdo)
        END IF
      ENDDO
      DO Jdo=1,MaxAclass+MaxAngleLinker
        IF (AAATotalType(Jdo).gt.0)  THEN
         RMSD(Jdo+MaxBclass+MaxBondLinker) =
     +        sqrt(AAArmsdClass(Jdo)/AAATotalType(Jdo))
          all_angle_rmsd = all_angle_rmsd + AAArmsdClass(jdo)
          Num_all_angles = Num_all_angles + AAATotalType(jdo)
        END IF
        IF (ANATotalType(Jdo).gt.0)  THEN
         RMSD(Jdo+MaxBclass+MaxBondLinker) =
     +        sqrt(ANArmsdClass(Jdo)/ANATotalType(Jdo))
          all_angle_rmsd = all_angle_rmsd + ANArmsdClass(jdo)
          Num_all_angles = Num_all_angles + ANATotalType(jdo)
        END IF
      ENDDO
c
c---   this block is just for the old system
c
         IF (Num_all_angles .gt. 0) THEN
          all_angle_rmsd =sqrt(all_angle_rmsd/Num_all_angles)
         else
          all_angle_rmsd = 0.0d0
         END IF
         IF (Num_all_bonds .gt. 0) THEN
          all_bond_rmsd =sqrt(all_bond_rmsd/Num_all_bonds)
         else
          all_bond_rmsd = 0.0d0
         END IF
c
c--- this is the internal structure rmsd only not that based on e&h
c
         write(Irmsd,8877) all_angle_rmsd,all_bond_rmsd
 8877    format(f10.3,'  all_angle_rmsd ',/,
     +          f10.3,'  all_bond_rmsd  ')
c
c
      CALL Rwrite(BondClassEsds,AngleClassEsds,
     +            U31,U61,rmsdCutOff,
     +            Nlinks,LINKS,debug,PlaneCutoff,
     +            PlaneNames,IDcode,
     +            MainChainPlaneCutoff,Fpdb,Fwork)
c
c
       Bbad = 0
       Abad = 0
c
c
       DO Jdo = 1,MaxBclass+MaxBondLinker
        IF (BAATotalType(Jdo).gt.0)  THEN
         R = sqrt(BAArmsdClass(Jdo)/BAATotalType(Jdo))
         IF (R .gt. rmsdCutOff*BondClassEsds(Jdo)) THEN
          IF (Bbad .eq. 0) write(Irmsd,'(a)')
     +' The following bond classes show an overall poor rmsd'//
     +' - check your dictionary',
     +'  Total  Observed   Expected    E&H99     Example    Example',
     +'  Class    RMSD       RMSD      Value     Bond       Residue'
          IF (Jdo.le. MaxBclass) THEN
           L1 = BSclass(1,Jdo)
           L2 = BSclass(2,Jdo)
           write(Irmsd,
     + fmt='(i7,3x,f5.3,6x,f5.3,6x,f5.3,4x,a4,a1,a4,3x,a)')
     +      BAATotalType(Jdo),
     +      R,  BondClassEsds(Jdo),
     +      BondTargetValues(L1,L2),
     +      BondPairs(1,L1,L2),'-',
     +      BondPairs(2,L1,L2),
     +      StanRes3(L1)
           ELSE
            L3 = Jdo-MaxBclass
            write(Irmsd,
     + fmt='(i7,3x,f5.3,6x,f5.3,6x,f5.3,4x,a4,a1,a4,3x,a)')
     +       BAATotalType(Jdo),
     +       R,  BondLinkerEsds(L3),
     +       BondLinkerValues(L3),
     +       BondLinkAtom(1,L3),'-',
     +       BondLinkAtom(2,L3),
     +       BondLinkerClass(L3)
           END IF
           Bbad = 1
          END IF
         END IF
c
c
        IF (BNATotalType(Jdo).gt.0)  THEN
         R = sqrt(BNArmsdClass(Jdo)/BNATotalType(Jdo))
         IF (R .gt. rmsdCutOff*BondClassEsds(Jdo)) THEN
          IF (Bbad .eq. 0) write(Irmsd,'(a)')
     +' The following bond classes show an overall poor rmsd'//
     +' - check your dictionary',
     +'  Total  Observed   Expected    E&H99     Example    Example',
     +'  Class    RMSD       RMSD      Value     Bond       Residue'
          IF (Jdo.le. MaxBclass) THEN
           L1 = BSclass(1,Jdo)
           L2 = BSclass(2,Jdo)
           write(Irmsd,
     + fmt='(i7,3x,f5.3,6x,f5.3,6x,f5.3,4x,a4,a1,a4,3x,a)')
     +      BNATotalType(Jdo),
     +      R,  BondClassEsds(Jdo),
     +      BondTargetValues(L1,L2),
     +      BondPairs(1,L1,L2),'-',
     +      BondPairs(2,L1,L2),StanRes3(L1)
           ELSE
            L3 = Jdo-MaxBclass
            write(Irmsd,
     + fmt='(i7,3x,f5.3,6x,f5.3,6x,f5.3,4x,a4,a1,a4,3x,a)')
     +       BNATotalType(Jdo),
     +       R,  BondLinkerEsds(L3),
     +       BondLinkerValues(L3),
     +       BondLinkAtom(1,L3),'-',
     +       BondLinkAtom(2,L3),
     +       BondLinkerClass(L3)
          END IF
          Bbad = 1
         END IF
        END IF
       ENDDO
c
c
       DO Jdo=1,MaxAclass+MaxAngleLinker
        IF (AAATotalType(Jdo).gt.0)  THEN
         R = sqrt(AAArmsdClass(Jdo)/AAATotalType(Jdo))
         IF (R .gt. rmsdCutOff*AngleClassEsds(Jdo)) THEN
          IF (Abad .eq. 0) write(Irmsd,'(a)')
     +' The following angle classes show an overall poor rmsd'//
     +' - check your dictionary',
     +'  Total  Observed  Expected   E&H99       Example   Example',
     +'  Class    RMSD      RMSD     Value       Angle     Residue'
          IF (Jdo.le. MaxAclass) THEN
           L1 = ASclass(1,Jdo)
           L2 = ASclass(2,Jdo)
           write(Irmsd,
     + fmt='(i7,3x,f5.2,5x,f5.2,3x,f7.2,2x,a4,a1,a4,a1,a4,2x,a)')
     +      AAATotalType(Jdo),
     +      R,  AngleTargetEsds(L1,L2),
     +      AngleTargetValues(L1,L2),
     +      AngleTriples(1,L1,L2),'-',
     +      AngleTriples(2,L1,L2),'-',
     +      AngleTriples(3,L1,L2),
     +      StanRes3(L1)
           ELSE
            L3 = Jdo-MaxAclass
            write(Irmsd,
     + fmt='(i7,3x,f5.2,5x,f5.2,3x,f7.2,2x,a4,a1,a4,a1,a4,2x,a)')
     +       AAATotalType(Jdo),
     +       R,  AngleLinkerEsds(L3),
     +       AngleLinkerValues(L3),
     +       AngleLinkAtom(1,L3),'-',
     +       AngleLinkAtom(2,L3),'-',
     +       AngleLinkAtom(3,L3),
     +       AngleLinkerClass(L3)
           END IF
           Abad = 1
          END IF
        END IF
c
c
        IF (ANATotalType(Jdo).gt.0)  THEN
         R = sqrt(ANArmsdClass(Jdo)/ANATotalType(Jdo))
         IF (R .gt. rmsdCutOff*AngleClassEsds(Jdo)) THEN
          IF (Abad .eq. 0) write(Irmsd,'(a)')
     +' The following angle classes show an overall poor rmsd'//
     +' - check your dictionary',
     +'  Total  Observed  Expected   E&H99       Example   Example',
     +'  Class    RMSD      RMSD     Value       Angle     Residue'
          IF (Jdo.le. MaxAclass) THEN
           L1 = ASclass(1,Jdo)
           L2 = ASclass(2,Jdo)
           write(Irmsd,
     + fmt='(i7,3x,f5.2,5x,f5.2,3x,f7.2,2x,a4,a1,a4,a1,a4,2x,a)')
     +      ANATotalType(Jdo),
     +      R,  AngleTargetEsds(L1,L2),
     +      AngleTargetValues(L1,L2),
     +      AngleTriples(1,L1,L2),'-',
     +      AngleTriples(2,L1,L2),'-',
     +      AngleTriples(3,L1,L2),
     +      StanRes3(L1)
           ELSE
            L3 = Jdo-MaxAclass
            write(Irmsd,
     + fmt='(i7,3x,f5.2,5x,f5.2,3x,f7.2,2x,a4,a1,a4,a1,a4,2x,a)')
     +       ANATotalType(Jdo),
     +       R,  AngleLinkerEsds(L3),
     +       AngleLinkerValues(L3),
     +       AngleLinkAtom(1,L3),'-',
     +       AngleLinkAtom(2,L3),'-',
     +       AngleLinkAtom(3,L3),
     +       AngleLinkerClass(L3)
           END IF
           Abad = 1
          END IF
        END IF
       ENDDO
c
c
      CLOSE (UNIT=Irmsd)
      CALL CCPERR(0,'Normal Termination')
      STOP
      END
c
c
c
c
c
c     ========================================
      SUBROUTINE StandardRead(QQstan,Naa,Nna,Ldump,Wcys,Wpro,Whis)
c     ========================================
c
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
      INTEGER Jdo,I2,I3,I4,I5,Rcurrent,Acurrent,
     +        i1,LLx,Kdo,Wcys,Wpro,Whis
      DOUBLE PRECISION v1,e1
      CHARACTER c0*3,c1*4,c2*4,c3*4,c5*2,c6*1,c14*14,
     +          line*255,STANDATA*255,Fname*255
      LOGICAL Qexist,Ldump
      INTEGER QQstan,Naa,Nna
      LOGICAL   Ldone(MaxStandard*MaxAngleTarget)
c
c
      INTEGER NNtype,N1N9,Natoms,NumAtmStanPlane,
     +        NumBondTarget,BondTargetClass,NumAngleTarget,
     +        AngleTargetClass,NumBondLinker,AAflag,
     +        NumAngleLinker
      CHARACTER StanRes3*3,stanNewNam*4,
     +          PlaneNames*4,
     +          BondPairs*4,AngleTriples*4,
     +          BondLinkerClass*12,BondLinkAtom*4,
     +          AngleLinkerClass*12,
     +          AngleLinkAtom*4,BondDescr*10,
     +          AngleDescr*10
      DOUBLE PRECISION BondTargetValues,BondTargetEsds,
     +                 AngleTargetValues,AngleTargetEsds,
     +                 BondLinkerValues,BondLinkerEsds,
     +                 AngleLinkerValues,AngleLinkerEsds
      COMMON /Cstan/ StanRes3(MaxStandard),
     +       stanNewNam(MaxStandard,MaxAtoms),
     +       PlaneNames(MaxStandard,MaxPlanesAtoms),
     +       BondPairs(2,MaxStandard,MaxBondTarget),
     +       AngleTriples(3,MaxStandard,MaxAngleTarget),
     +       BondLinkerClass(MaxBondLinker),
     +       BondLinkAtom(2,MaxBondLinker),
     +       AngleLinkerClass(MaxAngleLinker),
     +       AngleLinkAtom(3,MaxAngleLinker),
     +       BondDescr(MaxStandard,MaxBondTarget),
     +       AngleDescr(MaxStandard,MaxAngleTarget)
      COMMON /Dstan/
     +       BondTargetClass(MaxStandard,MaxBondTarget),
     +       BondTargetValues(MaxStandard,MaxBondTarget),
     +       BondTargetEsds(MaxStandard,MaxBondTarget),
     +       AngleTargetClass(MaxStandard,MaxAngleTarget),
     +       AngleTargetValues(MaxStandard,MaxAngleTarget),
     +       AngleTargetEsds(MaxStandard,MaxAngleTarget),
     +       BondLinkerValues(MaxBondLinker),
     +       BondLinkerEsds(MaxBondLinker),
     +       AngleLinkerValues(MaxAngleLinker),
     +       AngleLinkerEsds(MaxAngleLinker)
      COMMON /Istan/ NNtype(MaxStandard),
     +       N1N9(MaxStandard),
     +       Natoms(MaxStandard),
     +       NumAtmStanPlane(MaxStandard),
     +       NumBondTarget(MaxStandard),
     +       NumAngleTarget(MaxStandard),
     +       AAflag(MaxStandard),
     +       NumBondLinker,NumAngleLinker
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
      INTEGER r500_lenstr
c
c
      Wcys   = 0
      Wpro   = 0
      Whis   = 0
      QQstan = 0
      Naa    = 0
      Nna    = 0
      Qexist = .false.
c
c
      CALL getenv('STANDATA',STANDATA)
      INQUIRE (FILE=STANDATA,EXIST=Qexist)
      IF (.not. Qexist) THEN
       write(6,'(a)') 'ERROR cant find data file for env STANDATA'
       STOP
      END IF
c
c
      OPEN (UNIT=Istan,FILE=STANDATA,STATUS='OLD')
c
c order of data
c_chem_comp.comp_id
c_chem_comp_atom.comp_id
c_chem_comp_Datom.comp_id  ignore
c_chem_comp_Qatom.comp_id  ignore
c_chem_link_bond.atom_id_1
c_chem_comp_bond.comp_id
c_chem_link_angle.atom_id_1
c_chem_comp_angle.comp_id
c_chem_comp_tor.comp_id      currently ignore ?
c_chem_comp_chir.comp_id     currently ignore ?
c_chem_comp_plane_atom.comp_id
c
 100  Read(Istan,'(a)',END=999) line
       IF (r500_lenstr(line).le.0) GO TO 100
       IF (line(1:1) .eq. '#') THEN
           GO TO 100
       ELSE IF (line(1:5) .eq. 'loop_') THEN
           GO TO 100
c
cloop_
c_chem_comp.comp_id
c_chem_comp.one_letter_code
c_chem_comp.type
c_chem_comp.number_atoms_all
c_chem_comp.number_atoms_nh
c_chem_comp.number_Q_atoms
c_chem_comp.number_D_atoms
c_chem_comp.aa_flag        AAflag
c_chem_comp.nuc_acid_type  NNtype
c_chem_comp.N1N9           N1N9
c_chem_comp.asa_side
c_chem_comp.asa_main
c_chem_comp.asa_non-polar
c_chem_comp.asa_all-polar
c  ALA  A   1  16  10   1   4   1   0   0
c           1  2   3    4   5   6   7   8
c
       ELSE IF (index(line,
     +          '_chem_comp.comp_id').ne.0) THEN
         DO Jdo=1,MaxStandard
          AAflag(Jdo) = 0
          NNtype(Jdo) = 0
          N1N9(Jdo)   = 0
         ENDDO
         DO Jdo=1,13
          READ(Istan,fmt='(a)') line
         ENDDO
110      READ (Istan,fmt='(a)') line
         IF (line(1:1) .eq. '#') GO TO 100
         QQstan = QQstan + 1
         IF (QQstan .gt. MaxStandard)
     +       STOP 'QQstan gt MaxStandard'
         READ(line,fmt='(2x,a3,2x,a1,8i4)')
     +       StanRes3(QQstan), c6,
     +       I1,I2,I3,I4,I5,
     +       AAflag(QQstan),
     +       NNtype(QQstan),
     +       N1N9(QQstan)
         IF (StanRes3(QQstan) .eq. 'UNK') THEN
             Naa  = QQstan
         ELSE IF (StanRes3(QQstan) .eq. '  N') THEN
             Nna  = QQstan
         ELSE IF (StanRes3(QQstan) .eq. 'CYS') THEN
             Wcys = QQstan
         ELSE IF (StanRes3(QQstan) .eq. 'PRO') THEN
             Wpro = QQstan
         ELSE IF (StanRes3(QQstan) .eq. 'HIS') THEN
             Whis = QQstan
         END IF
         GO TO 110
c
cloop_
c_chem_comp_atom.comp_id
c_chem_comp_atom.atom_id
c_chem_comp_atom.alt_atom_id
c_chem_comp_atom.type_symbol
c_chem_comp_atom.charge
c_chem_comp_atom.pdbx_align
c_chem_comp_atom.pdbx_aromatic_flag
c_chem_comp_atom.pdbx_leaving_atom_flag
c_chem_comp_atom.pdbx_stereo_config
c_chem_comp_atom.ccp4_type_energy
c_chem_comp_atom.ccp4_partial_charge
c_chem_comp_atom.pdbx_model_Cartn_x_ideal
c_chem_comp_atom.pdbx_model_Cartn_y_ideal
c_chem_comp_atom.pdbx_model_Cartn_z_ideal
c_chem_comp_atom.pdbx_ordinal
c_chem_comp_atom.ref_radii_asa_probe
c_chem_comp_atom.descriptor
c
cALA   HB1   1HB   H 0 1 N N N
c
       ELSE IF (index(line,
     +          '_chem_comp_atom.').ne.0) THEN
         DO Jdo=1,16
          READ(Istan,fmt='(a)') line
         ENDDO
         DO Jdo=1,MaxStandard
          Natoms(Jdo)= 0
         ENDDO
 120     READ (Istan,fmt='(a)') line
         IF (line(1:1) .eq. '#') GO TO 120
         IF (line(1:5) .eq. 'loop_') GO TO 100
          DO Jdo=1,QQstan
           IF (line(1:3) .eq. StanRes3(Jdo)) THEN
            Rcurrent=Jdo
            GO TO 130
           END IF
          ENDDO
          write(6,'(a)') line(1:r500_lenstr(line))
          STOP 'residue name not found _chem_comp_atom.'
 130      READ(line,fmt='(5x,a4,3x,a4,1x,a2)') c1,c2,c5
          IF (c5 .ne. ' H') THEN
           Natoms(Rcurrent)        = Natoms(Rcurrent)+1
           Acurrent                = Natoms(Rcurrent)
           IF (Acurrent .gt. MaxAtoms)
     +       STOP 'MaxAtoms exceeded 2'
           stanNewNam(Rcurrent,Acurrent) = c1
          END IF
          GO TO 120
c
cloop_
c_chem_comp_Datom.comp_id
c_chem_comp_Datom.atom_id
c_chem_comp_Datom.alt_atom_id
c_chem_comp_Datom.type_symbol
c
       ELSE IF (index(line,
     +          '_chem_comp_Datom.').ne.0) THEN
         DO Jdo=1,3
          READ(Istan,fmt='(a)') line
         ENDDO
 140     READ (Istan,fmt='(a)') line
         IF (line(1:1) .eq. '#') GO TO 140
         IF (line(1:5) .eq. 'loop_') GO TO 100
         READ(line,fmt='(a)') c1
         GO TO 140
c
cloop_
c_chem_comp_Qatom.comp_id
c_chem_comp_Qatom.atom_id
c_chem_comp_Qatom.alt_atom_id
c_chem_comp_Qatom.type_symbol
c_chem_comp_Qatom.descriptor
c
       ELSE IF (index(line,
     +          '_chem_comp_Qatom.').ne.0) THEN
         DO Jdo=1,4
          READ(Istan,fmt='(a)') line
         ENDDO
 150     READ (Istan,fmt='(a)') line
         IF (line(1:1) .eq. '#') GO TO 150
         IF (line(1:5) .eq. 'loop_') GO TO 100
         READ(line,fmt='(a)') c1
         GO TO 150
c
cloop_
c_chem_comp_plane_atom.comp_id
c_chem_comp_plane_atom.plane_id
c_chem_comp_plane_atom.atom_id
c_chem_comp_plane_atom.dist_esd
c
       ELSE IF (index(line,
     +          '_chem_comp_plane_atom.').ne.0) THEN
         DO Jdo=1,MaxStandard
          NumAtmStanPlane(Jdo) = 0
         ENDDO
         DO Jdo=1,3
          Read(Istan,fmt='(a)') line
         ENDDO
 160     Read(Istan,fmt='(a)',END=999) line
         IF (line(1:1) .eq. '#') GO TO 160
         IF (line(1:5) .eq. 'loop_') GO TO 100
c
c ARG      1      CD        0.02
c TRP      1      HZ3       0.02
c#
c TYR      1      CB        0.02
c12345678901234561234xxx1234xxx1234
c      12345     1234
c
         READ(line,fmt='(1x,a3,12x,a4)') c0,c1
         DO Jdo=1,QQstan
          IF (c0 .eq. StanRes3(Jdo)) THEN
           Rcurrent = Jdo
           GO TO 170
          END IF
         ENDDO
         write(6,'(a)') line(1:r500_lenstr(line))
         STOP 'residue name not found in plane definition'
 170     CONTINUE
         NumAtmStanPlane(Rcurrent) =
     +        NumAtmStanPlane(Rcurrent) + 1
         IF (NumAtmStanPlane(Rcurrent) .gt.
     +       MaxPlanesAtoms) THEN
          write(6,'(a,2x,i8)')
     +    'ERROR too many atoms in Plane '//StanRes3(Rcurrent)//
     +    ' dimensioned ',MaxPlanesAtoms
          STOP
         END IF
         PlaneNames(Rcurrent,NumAtmStanPlane(Rcurrent))=c1
         GO TO 160
c
cloop_
c_chem_comp_bond.comp_id
c_chem_comp_bond.atom_id_1
c_chem_comp_bond.atom_id_2
c_chem_comp_bond.value_order
c_chem_comp_bond.pdbx_aromatic_flag
c_chem_comp_bond.pdbx_stereo_config
c_chem_comp_bond.pdbx_ordinal
c_chem_comp_bond.value_dist
c_chem_comp_bond.value_dist_esd
c_chem_comp_bond.exception_class
c_chem_comp_bond.description
cALA    CA    CB     SING N N  5    1.521    0.033   61
cALA    CA    C      SING N N  4    1.525    0.026   65
cALA    C     O      DOUB N N  7    1.229    0.019    4
c DA    C1'   N9     SING N N 22    1.468    0.014  170  C2'-endo
c DA    C1'   N9     SING N N 22    1.488    0.013  175  C3'-endo
c DA    C2    N3     DOUB Y N 36    1.331    0.009  111  s
c DA    O5'   C5'    SING N N  7    1.418    0.025  153  (H)C2'-end
c DA    O5'   C5'    SING N N  7    1.423    0.011  157  (H)C3'-end
c DA    O5'   C5'    SING N N  7    1.440    0.016  163  (P)O5'-C5
caaaxxxaaaaxxaaaaxxxxxxxxxxxxxxxxx1234567xx123456712345xx1234567890
c                12345678901234567
c
       ELSE IF (index(line,
     +          '_chem_comp_bond.comp_id').ne.0) THEN
        DO Jdo=1,MaxStandard
         NumBondTarget(Jdo) = 0
         DO Kdo=1,MaxBondTarget
          BondDescr(Jdo,Kdo)        = '.'
          BondTargetValues(Jdo,Kdo) = MissingValue
          BondTargetEsds(Jdo,Kdo)   = MissingValue
          BondTargetClass(Jdo,Kdo)  = 0
          BondPairs(1,Jdo,Kdo)      = '?'
          BondPairs(2,Jdo,Kdo)      = '?'
         ENDDO
        ENDDO
        DO Jdo=1,10
         Read(Istan,fmt='(a)') line
        ENDDO
 180    Read(Istan,fmt='(a)',END=999) line
        IF (line(1:1) .eq. '#') GO TO 180
        IF (line(1:5) .eq. 'loop_') GO TO 100
        READ(line,
     +   fmt='(a3,3x,a4,2x,a4,17x,f7.3,2x,f7.3,i5,2x,a)')
     +         c0,   c1,   c2,    v1,     e1,  i1,   c14
c
c---- skip hydrogens
c
        IF (i1 .eq. 0) GO TO 180
c
c
        DO Jdo=1,QQstan
         IF (c0 .eq. StanRes3(Jdo) ) THEN
          Rcurrent = Jdo
          GO TO 190
         END IF
        ENDDO
        write(6,'(a)') line(1:r500_lenstr(line))
        STOP 'resid not found in bonds '
 190    CONTINUE
        NumBondTarget(Rcurrent) = NumBondTarget(Rcurrent) + 1
        IF (NumBondTarget(Rcurrent) .gt. MaxBondTarget) THEN
         write(6,'(a,2x,i8)')
     +   'ERROR too many Bonds for '//StanRes3(Rcurrent)//
     +   ' dimensioned ',MaxBondTarget
         STOP
        END IF
        IF (c14(1:1) .eq. '"') c14=c14(2:)
        LLx = index(c14,'"')
        IF (LLx .ne.0) c14(LLx:LLx)=' '
        IF (index(c14,'.') .ne.0) c14 = '.'
        IF (r500_lenstr(c14) .le.0) c14 = '.'
c
c
        Acurrent =  NumBondTarget(Rcurrent)
        BondTargetValues(Rcurrent,Acurrent) = v1
        BondTargetEsds(Rcurrent,Acurrent)   = e1
        BondPairs(1,Rcurrent,Acurrent)      = c1
        BondPairs(2,Rcurrent,Acurrent)      = c2
        BondTargetClass(Rcurrent,Acurrent)  = i1
        BondDescr(Rcurrent,Acurrent)        = c14
        GO TO 180
c
cloop_
c_chem_comp_angle.comp_id
c_chem_comp_angle.atom_id_1
c_chem_comp_angle.atom_id_2
c_chem_comp_angle.atom_id_3
c_chem_comp_angle.value_angle
c_chem_comp_angle.value_angle_esd
c_chem_comp_angle.exception_class
c_chem_comp_angle.description
c ALA      CA     C      O       120.1      2.1   62
c   I      C1'    C2'    O2'    109.47      3.0  133 s
c   I      H3'    C3'    C2'    108.34      3.0    0
c  DA      C1'    O4'    C4'     110.1      1.0  137 C2'-endo
c  DA      C1'    O4'    C4'     110.3      0.7  138 C3'-endo
c  DA      O5'    C5'    C4'     109.4      0.8  130 (P)O5'-C5'
c  DA      O5'    C5'    C4'     110.9      1.7  143 C2'-endo
c  DA      O5'    C5'    C4'     111.0      2.5  144 C3'-endo
c  DA      O5'    P      OP1     105.7      0.9  113 small
c  DA      O5'    P      OP1     110.7      1.2  141 large
cxaaaxxxxxaaaaxxxaaaaxxxaaaaxx12345678xx1234567xxiiixaaaaa
c aa class  1 - 79
c na class 10-213
c
       ELSE IF (index(line,
     +          '_chem_comp_angle.comp_id').ne.0) THEN
        DO Jdo=1,7
         Read(Istan,fmt='(a)') line
        ENDDO
        DO Jdo=1,MaxStandard
         NumAngleTarget(Jdo) = 0
         DO Kdo=1,MaxAngleTarget
          AngleDescr(Jdo,Kdo)          = '.'
          AngleTargetValues(Jdo,Kdo)   = MissingValue
          AngleTargetEsds(Jdo,Kdo)     = MissingValue
          AngleTargetClass(Jdo,Kdo)    = 0
          AngleTriples(1,Jdo,Kdo)      = '?'
          AngleTriples(2,Jdo,Kdo)      = '?'
          AngleTriples(3,Jdo,Kdo)      = '?'
         ENDDO
        ENDDO
 200    Read(Istan,fmt='(a)',END=999) line
        IF (line(1:1) .eq. '#') GO TO 200
        IF (line(1:5) .eq. 'loop_') GO TO 100
c
c   I      C1'    C2'    O2'    109.47      3.0  133 s
c  DA      O5'    C5'    C4'     109.4      0.8  130 (P)O5'-C5'
cxaaa12345aaaaxxxaaaaxxxaaaax123456789x12345678xiii1x123456789012
c ALA      CA     C      O       120.1      2.1   62
c
        READ(line,
     +  fmt='(1x,a3,5x,a4,3x,a4,3x,a4,1x,f9.2,1x,f8.1,2x,i3,1x,
     +        a)')
     +            c0,c1,c2,c3,v1,e1,i1,c14
c
c---- skip hydrogens
c
        IF (i1 .eq. 0) GO TO 200
c
c
        DO Jdo=1,QQstan
         IF (c0 .eq. StanRes3(Jdo) ) THEN
          Rcurrent = Jdo
          GO TO 210
         END IF
        ENDDO
        write(6,'(a)') line(1:r500_lenstr(line))
        STOP 'resid not found in bonds '
 210    CONTINUE
        NumAngleTarget(Rcurrent) = NumAngleTarget(Rcurrent) + 1
        IF (NumAngleTarget(Rcurrent) .gt. MaxAngleTarget) THEN
         write(6,'(a,2x,i8)')
     +   'ERROR too many Angles for '//StanRes3(Rcurrent)//
     +   ' dimensioned ',MaxAngleTarget
         STOP
        END IF
        LLx = index(C14,'"')
        IF (LLx.ne.0) c14 = c14(LLx+1:)
        LLx = index(c14,'"')
        IF (LLx .ne.0) c14(LLx:LLx)=' '
        IF (index(c14,'.') .ne.0) c14 = '.'
        IF (r500_lenstr(c14) .le.0) c14 = '.'
        Acurrent =  NumAngleTarget(Rcurrent)
        AngleTargetValues(Rcurrent,Acurrent) = v1
        AngleTargetEsds(Rcurrent,Acurrent)   = e1
        AngleTargetClass(Rcurrent,Acurrent)  = i1
        AngleTriples(1,Rcurrent,Acurrent)    = c1
        AngleTriples(2,Rcurrent,Acurrent)    = c2
        AngleTriples(3,Rcurrent,Acurrent)    = c3
        AngleDescr(Rcurrent,Acurrent)        = c14
        GO TO 200
c
cloop_
c_chem_link_bond.atom_id_1
c_chem_link_bond.atom_id_2
c_chem_link_bond.value_dist
c_chem_link_bond.value_dist_esd
c_chem_link_bond.link_id
c C    N        1.336    0.023    "NOT GLY PRO"
c C    N        1.338    0.019    "PRO"
c C    N        1.326    0.018    "GLY"
c O3'  P        1.607    0.012    "NUC-ACID-ALL"
caaaaxaaaaxxx12345678xx1234567xxxxx123456789012
c
       ELSE IF (index(line,
     +          '_chem_link_bond.atom_id_1').ne.0) THEN
        NumBondLinker = 0
        DO Jdo=1,4
         Read(Istan,fmt='(a)',end=999) line
        ENDDO
 220    Read(Istan,fmt='(a)',END=999) line
        IF (line(1:1) .eq. '#') GO TO 220
        IF (line(1:5) .eq. 'loop_') GO TO 100
        READ(line,
     +  fmt='(a4,1x,a4,3x,f8.3,2x,f7.3,5x,a)')
     +            c1,c2,v1,e1,c14
        NumBondLinker = NumBondLinker + 1
        IF (NumBondLinker .gt. MaxBondLinker) THEN
         write(6,'(a,2x,i8)')
     +   'ERROR too many Bond Linkers'
         STOP
        END IF
        IF (c14(1:1) .eq. '"') c14=c14(2:)
        LLx = index(c14,'"')
        IF (LLx .ne.0) c14(LLx:LLx)=' '
        BondLinkerValues(NumBondLinker) = v1
        BondLinkerEsds(NumBondLinker)   = e1
        BondLinkerClass(NumBondLinker)  = c14
        BondLinkAtom(1,NumBondLinker)   = c1
        BondLinkAtom(2,NumBondLinker)   = c2
        GO TO 220
c
cloop_
c_chem_link_angle.atom_id_1
c_chem_link_angle.atom_id_2
c_chem_link_angle.atom_id_3
c_chem_link_angle.value_angle
c_chem_link_angle.value_angle_esd
c_chem_link_angle.link_id
c     CA   C    N      117.2      2.2    "NOT GLY PRO"
c     CA   C    N      117.1      2.8    PRO
c     CA   C    N      116.2      2.0    GLY
c     O    C    N      122.7      1.6    "NOT PRO GLY"
c     O    C    N      121.1      1.9    PRO
c     O    C    N      123.2      1.7    GLY
c     C    N    CA     121.7      2.5    "NOT GLY PRO"
c     C    N    CA     122.0      4.2    PRO-ALL
c     C    N    CA     127.0      2.4    PRO-CIS
c     C    N    CA     119.3      1.5    PRO-TRANS
c     C    N    CA     122.3      2.1    GLY
cxxxxaaaaxaaaaxaaaax12345678xx1234567xxxx123456789012
c
c CB SG SG  104.3 2.3  not tested
c
       ELSE IF (index(line,
     +          '_chem_link_angle.atom_id_1').ne.0) THEN
        NumAngleLinker = 0
        DO Jdo=1,5
         Read(Istan,fmt='(a)') line
        ENDDO
 230    Read(Istan,fmt='(a)',END=999) line
        IF (line(1:1) .eq. '#') GO TO 230
        IF (line(1:5) .eq. 'loop_') GO TO 100
        READ(line,
     +  fmt='(4x,a4,1x,a4,1x,a4,1x,f8.1,2x,f7.1,4x,a)')
     +           c1,c2,c3,v1,e1,c14
        NumAngleLinker = NumAngleLinker + 1
        IF (NumAngleLinker .gt. MaxAngleLinker) THEN
         write(6,'(a,2x,i8)')
     +   'ERROR too many Angle Linkers'
         STOP
        END IF
        IF (c14(1:1) .eq. '"') c14=c14(2:)
        LLx = index(c14,'"')
        IF (LLx .ne.0) c14(LLx:LLx)=' '
        AngleLinkerValues(NumAngleLinker) = v1
        AngleLinkerEsds(NumAngleLinker)   = e1
        AngleLinkerClass(NumAngleLinker)  = c14
        AngleLinkAtom(1,NumAngleLinker)   = c1
        AngleLinkAtom(2,NumAngleLinker)   = c2
        AngleLinkAtom(3,NumAngleLinker)   = c3
        GO TO 230
c
cloop_
c_chem_comp_tor.comp_id
c_chem_comp_tor.id
c_chem_comp_tor.atom_id_1
c_chem_comp_tor.atom_id_2
c_chem_comp_tor.atom_id_3
c_chem_comp_tor.atom_id_4
c_chem_comp_tor.value_angle
c_chem_comp_tor.value_angle_esd
c_chem_comp_tor.period
c ALA      hh1      N      CA     CB     HB3       60.0     15.0     3
c
       ELSE IF (index(line,
     +          '_chem_comp_tor.comp_id').ne.0) THEN
        DO Jdo=1,8
         READ(Istan,fmt='(a)') line
        ENDDO
 240    READ (Istan,fmt='(a)') line
        IF (line(1:1) .eq. '#') GO TO 240
        IF (line(1:5) .eq. 'loop_') GO TO 100
        READ(line,fmt='(a)') c1
        GO TO 240
c
cloop_
c_chem_comp_chir.comp_id
c_chem_comp_chir.id
c_chem_comp_chir.atom_id_centre
c_chem_comp_chir.atom_id_1
c_chem_comp_chir.atom_id_2
c_chem_comp_chir.atom_id_3
c_chem_comp_chir.volume_sign
c ALA      1  CA     N      CB     C         negativ
c
       ELSE IF (index(line,
     +          '_chem_comp_chir.comp_id').ne.0) THEN
        DO Jdo=1,6
         READ(Istan,fmt='(a)') line
        ENDDO
250     READ (Istan,fmt='(a)') line
        IF (line(1:1) .eq. '#') GO TO 250
        IF (line(1:5) .eq. 'loop_') GO TO 100
        READ(line,fmt='(a)') c1
        GO TO 250
       END IF
c
c
      GO TO 100
 999  CONTINUE
c
      CLOSE (UNIT=Istan)
c
c
      IF (Ldump) THEN
      Fname = './e_h_values'
      OPEN (UNIT=Idump,FILE=Fname,STATUS='UNKNOWN')
        DO Jdo=1,MaxStandard*MaxAngleTarget
          Ldone(Jdo) = .false.
        ENDDO
       DO Jdo=1,QQstan
        write(Idump,'(a3,2x,3i8)') StanRes3(Jdo),AAflag(Jdo),
     +                         NNtype(Jdo),N1N9(Jdo)
       ENDDO
        DO Jdo=1,MaxStandard
         DO Kdo=1,MaxBondTarget
          IF (BondTargetValues(Jdo,Kdo) .gt. MissingValue) THEN
          IF (.not. Ldone(BondTargetClass(Jdo,Kdo))) THEN
          write(Idump,
     +     fmt='(i5,2x,2(a4,2x),f8.3,2x,f8.3,2x,a,2x,a)')
     +          BondTargetClass(Jdo,Kdo),
     +     BondPairs(1,Jdo,Kdo),
     +     BondPairs(2,Jdo,Kdo),
     +     BondTargetValues(Jdo,Kdo),
     +     BondTargetEsds(Jdo,Kdo),
     +     Stanres3(Jdo),BondDescr(Jdo,Kdo)
          Ldone(BondTargetClass(Jdo,Kdo))=.true.
          END IF
          END IF
         ENDDO
        ENDDO
        DO Jdo=1,MaxStandard*MaxAngleTarget
          Ldone(Jdo) = .false.
        ENDDO
        DO Jdo=1,MaxStandard
         DO Kdo=1,MaxAngleTarget
          IF (AngleTargetValues(Jdo,Kdo) .gt. MissingValue) THEN
           IF (.not. Ldone(AngleTargetClass(Jdo,Kdo))) THEN
          write(Idump,
     +     fmt='(i5,2x,3(a4,2x),f8.3,2x,f8.3,2x,a,2x,a)')
     +     AngleTargetClass(Jdo,Kdo),
     +     AngleTriples(1,Jdo,Kdo),
     +     AngleTriples(2,Jdo,Kdo),
     +     AngleTriples(3,Jdo,Kdo),
     +     AngleTargetValues(Jdo,Kdo),
     +     AngleTargetEsds(Jdo,Kdo),
     +     Stanres3(Jdo),AngleDescr(Jdo,Kdo)
           Ldone(AngleTargetClass(Jdo,Kdo))= .true.
           END IF
          END IF
         ENDDO
        ENDDO
      END IF
      RETURN
      END
c
c
c     =====================================================
       SUBROUTINE Process(Wcurrent,
     +         Watoms,Wname,xyz,Walt,
     +         Wres,Wchain,Wseq,Wins,Wmodel,
     +         Ntype,U31,
     +         BAATotalType,AAATotalType,
     +         ANATotalType,BNATotalType,
     +         BAArmsdClass,AAArmsdClass,
     +         ANArmsdClass,BNArmsdClass,
     +         N1N9,StanRes3,IDcode,CisTrans,Bdone,Adone, 
     +         Fpdb,Fwork)
c     ======================================================
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
      LOGICAL Bdone(MaxBondTarget),Adone(MaxAngleTarget)
      INTEGER  BAATotalType(MaxBclass+MaxBondLinker),
     +         AAATotalType(MaxAclass+MaxAngleLinker),
     +         ANATotalType(MaxAclass+MaxAngleLinker),
     +         BNATotalType(MaxBclass+MaxBondLinker)
      DOUBLE PRECISION BAArmsdClass(MaxBclass+MaxBondLinker),
     +                 AAArmsdClass(MaxAclass+MaxAngleLinker),
     +                 ANArmsdClass(MaxAclass+MaxAngleLinker),
     +                 BNArmsdClass(MaxBclass+MaxBondLinker)
      INTEGER  N1N9(MaxStandard)
      CHARACTER StanRes3(MaxStandard)*4,IDcode*4
c
c
      LOGICAL Dpuck
      INTEGER Wseq,Wmodel,Watoms,Wcurrent,RnaDna,U31,Ntype, 
     +        CisTrans,Fpdb
      DOUBLE PRECISION PuckerAngle
      CHARACTER Wres*3,Wchain*1,Wins*1,PuckerDescription*14
      DOUBLE PRECISION xyz(3,MaxAtoms)
      CHARACTER Wname(MaxAtoms)*4,Walt(MaxAtoms)*1,
     +          Nconnect*4,Fwork*255
c
c---- amino acid
c
         IF (Ntype .eq. 1) THEN
          CALL AminoAcid(Wcurrent,Watoms,Wname,xyz,Walt,
     +                    BAArmsdClass,BAATotalType,
     +                    AAArmsdClass,AAATotalType,
     +                    Wres,Wchain,Wseq,Wins,Wmodel,CisTrans)
          CALL GetChiral(Watoms,Wres,Wseq,Wins,Wchain,Wmodel,
     +                   Walt,xyz,Wname)
c
c---- nucleic acid
c
         ELSE IF (Ntype .eq. 0) THEN
          IF (StanRes3(Wcurrent)(1:2) .eq. '  ') THEN
             RnaDna = 1
          ELSE IF (StanRes3(Wcurrent)(1:2) .eq. ' D') THEN
             RnaDna = 2
          END IF
c
c---- which atom is connected to ribose
c
          IF (N1N9(Wcurrent) .eq. 9) THEN
           Nconnect = ' N9 '
          ELSE IF (N1N9(Wcurrent) .eq. 1) THEN
           Nconnect = ' N1 '
          END IF
          Dpuck=.true.
          CALL GetPuckerAngle(Nconnect,Wname,
     +                Wres,Wseq,Wchain,Walt,Wmodel,Wins,Wcurrent,
     +                xyz,Watoms,PuckerDescription,
     +                PuckerAngle,RnaDna,U31,IDcode,Fpdb,Fwork)
          CALL NucAcid(Wcurrent,Watoms,Wname,xyz,Walt,RnaDna,
     +                   Wmodel,Wins,Wseq,Wres,Wchain,
     +                   BNArmsdClass,BNATotalType,
     +                   ANArmsdClass,ANATotalType,
     +                   PuckerDescription,Bdone,Adone)
         END IF
c
c
         RETURN
         END
c
c      ========================================================
       SUBROUTINE AminoAcid(Wcurrent,Watoms,Wname1,xyz1,Walt1,
     +                       BAArmsdClass,BAATotalType,
     +                       AAArmsdClass,AAATotalType,
     +                       Wres,Wchain,Wseq,Wins,
     +                       Wmodel,CisTrans)
c     =========================================================
c
c  expect
cMaxStandard    37
cNaa            24
cNna            37
cNumBondLinker   4
cNumAngleLinker 11
cMaxAngleTarget 56
cMaxBondTarget  36
cMaxPlanesAtoms 12
cMaxBondLinker   4
cMaxAngleLinker 11
c
cclasses bonds  - 200  aa 1-73, na 100-194
cclasses angles - 250  aa 1-79, na 10-213
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
c
c
       INTEGER Wcurrent,Watoms,Wseq,Wmodel,Klinker,CisTrans
       CHARACTER Wname1(MaxAtoms)*4,Walt1(MaxAtoms)*1,Wres*3,
     +           Wchain*1,Wins*1
       DOUBLE PRECISION xyz1(3,MaxAtoms)
      INTEGER  BAATotalType(MaxBclass+MaxBondLinker),
     +         AAATotalType(MaxAclass+MaxAngleLinker)
      DOUBLE PRECISION BAArmsdClass(MaxBclass+MaxBondLinker),
     +                 AAArmsdClass(MaxAclass+MaxAngleLinker)
c
c
      INTEGER NNtype,N1N9,Natoms,NumAtmStanPlane,
     +        NumBondTarget,BondTargetClass,NumAngleTarget,
     +        AngleTargetClass,NumBondLinker,AAflag,
     +        NumAngleLinker
      CHARACTER StanRes3*3,stanNewNam*4,
     +          PlaneNames*4,
     +          BondPairs*4,AngleTriples*4,
     +          BondLinkerClass*12,BondLinkAtom*4,
     +          AngleLinkerClass*12,
     +          AngleLinkAtom*4,BondDescr*10,
     +          AngleDescr*10
      DOUBLE PRECISION BondTargetValues,BondTargetEsds,
     +                 AngleTargetValues,AngleTargetEsds,
     +                 BondLinkerValues,BondLinkerEsds,
     +                 AngleLinkerValues,AngleLinkerEsds
      COMMON /Cstan/ StanRes3(MaxStandard),
     +       stanNewNam(MaxStandard,MaxAtoms),
     +       PlaneNames(MaxStandard,MaxPlanesAtoms),
     +       BondPairs(2,MaxStandard,MaxBondTarget),
     +       AngleTriples(3,MaxStandard,MaxAngleTarget),
     +       BondLinkerClass(MaxBondLinker),
     +       BondLinkAtom(2,MaxBondLinker),
     +       AngleLinkerClass(MaxAngleLinker),
     +       AngleLinkAtom(3,MaxAngleLinker),
     +       BondDescr(MaxStandard,MaxBondTarget),
     +       AngleDescr(MaxStandard,MaxAngleTarget)
      COMMON /Dstan/
     +       BondTargetClass(MaxStandard,MaxBondTarget),
     +       BondTargetValues(MaxStandard,MaxBondTarget),
     +       BondTargetEsds(MaxStandard,MaxBondTarget),
     +       AngleTargetClass(MaxStandard,MaxAngleTarget),
     +       AngleTargetValues(MaxStandard,MaxAngleTarget),
     +       AngleTargetEsds(MaxStandard,MaxAngleTarget),
     +       BondLinkerValues(MaxBondLinker),
     +       BondLinkerEsds(MaxBondLinker),
     +       AngleLinkerValues(MaxAngleLinker),
     +       AngleLinkerEsds(MaxAngleLinker)
      COMMON /Istan/ NNtype(MaxStandard),
     +       N1N9(MaxStandard),
     +       Natoms(MaxStandard),
     +       NumAtmStanPlane(MaxStandard),
     +       NumBondTarget(MaxStandard),
     +       NumAngleTarget(MaxStandard),
     +       AAflag(MaxStandard),
     +       NumBondLinker,NumAngleLinker
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
       INTEGER Jdo,Kdo,Ldo,Ndo,Juse,Qdo
       DOUBLE PRECISION Dist,Dvalue,Wangle,Dist1,Dist2,
     +                  Dist3,Dif1,Dif2,Dif3,COSANG,RADDEG,PI
c
c
c
       Klinker = 1
       PI     = ATAN(1.0d0)*4.0d0
       RADDEG = 180.0d0/PI
c
c---- bonds
c
      DO Jdo=1,NumBondTarget(Wcurrent)
       DO Kdo=1,Watoms
        IF (Wname1(Kdo) .eq. BondPairs(1,Wcurrent,Jdo))THEN
         IF (xyz1(1,Kdo) .gt. MissingValue) THEN
          DO Ldo=1,Watoms
           IF (Ldo.ne.Kdo) THEN
            IF(Wname1(Ldo) .eq. BondPairs(2,Wcurrent,Jdo))THEN
             IF (Walt1(Ldo) .eq. Walt1(Kdo) ) THEN
              IF (xyz1(1,Ldo) .gt. MissingValue) THEN
               Dist = 0.0d0
               DO Ndo = 1,3
                Dist = ( xyz1(Ndo,Kdo) - xyz1(Ndo,Ldo) )**2 + Dist
               ENDDO
               Dist   = sqrt (Dist)
               Juse   = BondTargetClass(Wcurrent,Jdo)
               Dvalue = Dist - BondTargetValues(Wcurrent,Jdo)
               BAArmsdClass(Juse) = BAArmsdClass(Juse) + Dvalue*Dvalue
               BAATotalType(Juse) = BAATotalType(Juse) + 1
      WRITE (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +   'BAA',Juse,Wmodel,Dist,Dvalue,Klinker,
     +   Wres,Wchain,Wseq,Wins,Wname1(Kdo),Walt1(Kdo),
     +   Wres,Wchain,Wseq,Wins,Wname1(Ldo),Walt1(Ldo),
     +   '   ',' ',0,' ','    ',' '
               GO TO 110
              END IF
             END IF
            END IF
           END IF
          ENDDO
         ENDIF
        END IF
       ENDDO
 110   CONTINUE
      ENDDO
c
c---- angles
c
c
      DO Jdo=1,NumAngleTarget(Wcurrent)
       DO Kdo=1,Watoms
        IF (Wname1(Kdo) .eq. AngleTriples(1,Wcurrent,Jdo))THEN
         IF (xyz1(1,Kdo) .gt. MissingValue) THEN
          DO Ldo=1,Watoms
           IF (Ldo.ne.Kdo) THEN
            IF(Wname1(Ldo) .eq. AngleTriples(2,Wcurrent,Jdo))THEN
             IF (Walt1(Ldo) .eq. Walt1(Kdo) ) THEN
              IF (xyz1(1,Ldo) .gt. MissingValue) THEN
               DO Qdo=1,Watoms
                IF (Qdo.ne.Ldo .and. Qdo.ne.Kdo) THEN
                 IF (Wname1(Qdo) .eq. AngleTriples(3,Wcurrent,Jdo))THEN
                  IF (Walt1(Qdo) .eq. Walt1(Kdo) ) THEN
                   Wangle = MissingValue
                   Dist1 = 0.0d0
                   Dist2 = 0.0d0
                   Dist3 = 0.0d0
                   DO Ndo = 1,3
                    Dif1 = xyz1(Ndo,Ldo) - xyz1(Ndo,Kdo)
                    Dif2 = xyz1(Ndo,Qdo) - xyz1(Ndo,Ldo)
                    Dif3 = xyz1(Ndo,Kdo) - xyz1(Ndo,Qdo)
                    Dist1 = Dif1*Dif1 + Dist1
                    Dist2 = Dif2*Dif2 + Dist2
                    Dist3 = Dif3*Dif3 + Dist3
                   ENDDO
                   IF (Dist1.gt.0.0d0 .and.
     +                 Dist2.gt.0.0d0 .and.
     +                 Dist3.ge.0.0d0) THEN
                    Dist1 = sqrt(Dist1)
                    Dist2 = sqrt(Dist2)
                    Dist3 = sqrt(Dist3)
                    COSANG = (Dist3*Dist3-
     +                        Dist1*Dist1-
     +                        Dist2*Dist2)/
     +                       (-2.0d0*Dist1*Dist2)
                    IF (ABS(COSANG).le.1.0d0)
     +               Wangle = ACOS(COSANG)*RADDEG
                    IF (Wangle .gt. MissingValue+1.0d0) THEN
              Juse   = AngleTargetClass(Wcurrent,Jdo)
              Dvalue = Wangle - AngleTargetValues(Wcurrent,Jdo)
              AAArmsdClass(Juse) = AAArmsdClass(Juse) + Dvalue*Dvalue
              AAATotalType(Juse) = AAATotalType(Juse) + 1
      WRITE (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +   'AAA',Juse,Wmodel,Wangle,Dvalue,Klinker,
     +   Wres,Wchain,Wseq,Wins,Wname1(Kdo),Walt1(Kdo),
     +   Wres,Wchain,Wseq,Wins,Wname1(Ldo),Walt1(Ldo),
     +   Wres,Wchain,Wseq,Wins,Wname1(Qdo),Walt1(Qdo)
                  END IF
                  END IF
                  GO TO 120
                  END IF
                 END IF
                END IF
               ENDDO
              END IF
             END IF
            END IF
           END IF
          ENDDO
         END IF
        END IF
       ENDDO
 120   CONTINUE
      ENDDO
c
c
      RETURN
      END
c
c
c      ========================================================
       SUBROUTINE NucAcid(Wcurrent,Watoms,Wname1,xyz1,Walt1,
     +                   RnaDna,Wmodel,Wins,Wseq,Wres,Wchain,
     +                   BNArmsdClass,BNATotalType,
     +                   ANArmsdClass,ANATotalType,
     +                   PuckerDescription,Bdone,Adone)
c     =========================================================
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
c
c
       INTEGER Wcurrent,Watoms,RnaDna,Klinker,Wmodel,Wseq
       CHARACTER Wname1(MaxAtoms)*4,Walt1(MaxAtoms)*1,
     +            PuckerDescription*14,Wins*1,Wres*3,Wchain*1
       DOUBLE PRECISION xyz1(3,MaxAtoms)
      INTEGER  ANATotalType(MaxAclass+MaxAngleLinker),
     +         BNATotalType(MaxBclass+MaxBondLinker)
      DOUBLE PRECISION ANArmsdClass(MaxAclass+MaxAngleLinker),
     +                 BNArmsdClass(MaxBclass+MaxBondLinker)
c
c
       INTEGER Jdo,Kdo,Ldo,Ndo,Juse,Qdo
       LOGICAL Bdone(MaxBondTarget),Adone(MaxAngleTarget)
       DOUBLE PRECISION Dist,Dvalue,Wangle,Dist1,Dist2,
     +                  Dist3,Dif1,Dif2,Dif3,COSANG,RADDEG,
     +                  D1,D2,D3,XM,PI
c
c
      INTEGER NNtype,N1N9,Natoms,NumAtmStanPlane,
     +        NumBondTarget,BondTargetClass,NumAngleTarget,
     +        AngleTargetClass,NumBondLinker,AAflag,
     +        NumAngleLinker
      CHARACTER StanRes3*3,stanNewNam*4,
     +          PlaneNames*4,
     +          BondPairs*4,AngleTriples*4,
     +          BondLinkerClass*12,BondLinkAtom*4,
     +          AngleLinkerClass*12,
     +          AngleLinkAtom*4,BondDescr*10,
     +          AngleDescr*10
      DOUBLE PRECISION BondTargetValues,BondTargetEsds,
     +                 AngleTargetValues,AngleTargetEsds,
     +                 BondLinkerValues,BondLinkerEsds,
     +                 AngleLinkerValues,AngleLinkerEsds
      COMMON /Cstan/ StanRes3(MaxStandard),
     +       stanNewNam(MaxStandard,MaxAtoms),
     +       PlaneNames(MaxStandard,MaxPlanesAtoms),
     +       BondPairs(2,MaxStandard,MaxBondTarget),
     +       AngleTriples(3,MaxStandard,MaxAngleTarget),
     +       BondLinkerClass(MaxBondLinker),
     +       BondLinkAtom(2,MaxBondLinker),
     +       AngleLinkerClass(MaxAngleLinker),
     +       AngleLinkAtom(3,MaxAngleLinker),
     +       BondDescr(MaxStandard,MaxBondTarget),
     +       AngleDescr(MaxStandard,MaxAngleTarget)
      COMMON /Dstan/
     +       BondTargetClass(MaxStandard,MaxBondTarget),
     +       BondTargetValues(MaxStandard,MaxBondTarget),
     +       BondTargetEsds(MaxStandard,MaxBondTarget),
     +       AngleTargetClass(MaxStandard,MaxAngleTarget),
     +       AngleTargetValues(MaxStandard,MaxAngleTarget),
     +       AngleTargetEsds(MaxStandard,MaxAngleTarget),
     +       BondLinkerValues(MaxBondLinker),
     +       BondLinkerEsds(MaxBondLinker),
     +       AngleLinkerValues(MaxAngleLinker),
     +       AngleLinkerEsds(MaxAngleLinker)
      COMMON /Istan/ NNtype(MaxStandard),
     +       N1N9(MaxStandard),
     +       Natoms(MaxStandard),
     +       NumAtmStanPlane(MaxStandard),
     +       NumBondTarget(MaxStandard),
     +       NumAngleTarget(MaxStandard),
     +       AAflag(MaxStandard),
     +       NumBondLinker,NumAngleLinker
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
        PI     = ATAN(1.0d0)*4.0d0
        RADDEG = 180.0d0/PI
         Klinker = 1
         DO Jdo=1,MaxBondTarget
          Bdone(Jdo) = .false.
         ENDDO
         DO Jdo=1,MaxAngleTarget
          Adone(Jdo)= .false.
         ENDDO
c
c---- bonds
c
       DO Jdo=1,NumBondTarget(Wcurrent)
        IF (.not. Bdone(Jdo)) THEN
         Dvalue = MissingValue
         DO Kdo=1,Watoms
          IF (Wname1(Kdo) .eq. BondPairs(1,Wcurrent,Jdo))THEN
           IF (xyz1(1,Kdo) .gt. MissingValue) THEN
            DO Ldo=1,Watoms
             IF (Ldo.ne.Kdo) THEN
              IF(Wname1(Ldo) .eq. BondPairs(2,Wcurrent,Jdo))THEN
               IF (Walt1(Ldo) .eq. Walt1(Kdo) ) THEN
                IF (xyz1(1,Ldo) .gt. MissingValue) THEN
                 Dist = 0.0d0
                 DO Ndo = 1,3
                  Dist = ( xyz1(Ndo,Kdo) - xyz1(Ndo,Ldo) )**2 + Dist
                 ENDDO
                 Dist = sqrt (Dist)
c
c
                 IF (BondDescr(Wcurrent,Jdo)(1:1) .eq. 's') THEN
                  Juse   = BondTargetClass(Wcurrent,Jdo)
                  Dvalue = Dist - BondTargetValues(Wcurrent,Jdo)
                  Bdone(Jdo) = .true.
                 ELSE IF (BondDescr(Wcurrent,Jdo) .eq. 
     +                    PuckerDescription(1:10)) THEN
                  Juse   = BondTargetClass(Wcurrent,Jdo)
                  Dvalue = Dist - BondTargetValues(Wcurrent,Jdo)
                  Bdone(Jdo) = .true.
c
c always a pair
c DA    C3'   C2'    SING N N 16    1.516    0.008  181  C2'-endo
c DA    C3'   C2'    SING N N 16    1.518    0.012  183  C3'-endo
c
                 ELSE IF (
     +    index(BondDescr(Wcurrent,Jdo),'endo').ne.0) THEN
                  D1 = ABS(Dist - BondTargetValues(Wcurrent,Jdo))
                  D2 = ABS(Dist - BondTargetValues(Wcurrent,Jdo+1))
                  IF (D1.gt.D2) THEN
                   Juse = BondTargetClass(Wcurrent,Jdo+1)
                   Dvalue = Dist - BondTargetValues(Wcurrent,Jdo+1)
                  ELSE
                   Juse = BondTargetClass(Wcurrent,Jdo)
                   Dvalue = Dist - BondTargetValues(Wcurrent,Jdo)
                  END IF
                  Bdone(Jdo) = .true.
                  Bdone(Jdo+1) = .true.
c
c  always a triple
c DA    O5'   C5'    SING N N  7    1.418    0.025  153  (H)C2'-end
c DA    O5'   C5'    SING N N  7    1.423    0.011  157  (H)C3'-end
c DA    O5'   C5'    SING N N  7    1.440    0.016  163  (P)O5'-C5
c
                 ELSE IF (index(BondDescr(Wcurrent,Jdo),
     +                   '(').ne.0) THEN
                  D1 = ABS(Dist - BondTargetValues(Wcurrent,Jdo))
                  D2 = ABS(Dist - BondTargetValues(Wcurrent,Jdo+1))
                  D3 = ABS(Dist - BondTargetValues(Wcurrent,Jdo+2))
                  XM = min(D1,D2,D3)
        IF (XM .lt. D1+0.0001d0 .and. XM .gt. D1-0.0001d0) THEN
                   Juse = BondTargetClass(Wcurrent,Jdo)
                   Dvalue = Dist - BondTargetValues(Wcurrent,Jdo)
        ELSE IF (XM .lt. D2+0.0001d0 .and. XM .gt. D2-0.0001d0) THEN
                   Juse = BondTargetClass(Wcurrent,Jdo+1)
                   Dvalue = Dist - BondTargetValues(Wcurrent,Jdo+1)
        ELSE IF (XM .lt. D3+0.0001d0 .and. XM .gt. D3-0.0001d0) THEN
                   Juse = BondTargetClass(Wcurrent,Jdo+2)
                   Dvalue = Dist - BondTargetValues(Wcurrent,Jdo+2)
                  END IF
                  Bdone(Jdo)   = .true.
                  Bdone(jdo+1) = .true.
                  Bdone(Jdo+2) = .true.
                 ELSE
                  Juse   = BondTargetClass(Wcurrent,Jdo)
                  Dvalue = Dist - BondTargetValues(Wcurrent,Jdo)
                  Bdone(Jdo) = .true.
                 END IF
c
c
                 IF (Dvalue .gt. MissingValue + 1.0d0) THEN
                  BNArmsdClass(Juse) = BNArmsdClass(Juse) +
     +                                 Dvalue*Dvalue
                  BNATotalType(Juse) = BNATotalType(Juse) + 1
                  WRITE (Iscr,
     +            fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +                 3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +             'BNA',Juse,Wmodel,Dist,Dvalue,Klinker,
     +             Wres,Wchain,Wseq,Wins,Wname1(Kdo),Walt1(Kdo),
     +             Wres,Wchain,Wseq,Wins,Wname1(Ldo),Walt1(Ldo),
     +             '   ',' ',0,' ','    ',' '
                 END IF
c
c
                 GO TO 10
                END IF
               END IF
              END IF
             END IF
            ENDDO
           ENDIF
          END IF
         ENDDO
 10      CONTINUE
        END IF
       ENDDO
c
c---- angles
c
      DO Jdo=1,NumAngleTarget(Wcurrent)
       IF (.not. Adone(Jdo) ) THEN
        Dvalue = MissingValue
        DO Kdo=1,Watoms
         IF (Wname1(Kdo) .eq. AngleTriples(1,Wcurrent,Jdo))THEN
          IF (xyz1(1,Kdo) .gt. MissingValue) THEN
           DO Ldo=1,Watoms
            IF (Ldo.ne.Kdo) THEN
             IF(Wname1(Ldo) .eq. AngleTriples(2,Wcurrent,Jdo))THEN
              IF (Walt1(Ldo) .eq. Walt1(Kdo) ) THEN
               IF (xyz1(1,Ldo) .gt. MissingValue) THEN
                DO Qdo=1,Watoms
                 IF (Qdo.ne.Ldo .and. Qdo.ne.Kdo) THEN
                  IF (Wname1(Qdo) .eq. AngleTriples(3,Wcurrent,Jdo))THEN
                   IF (Walt1(Qdo) .eq. Walt1(Kdo) ) THEN
                    Wangle = MissingValue
                    Dist1 = 0.0d0
                    Dist2 = 0.0d0
                    Dist3 = 0.0d0
                    DO Ndo = 1,3
                     Dif1 = xyz1(Ndo,Ldo) - xyz1(Ndo,Kdo)
                     Dif2 = xyz1(Ndo,Qdo) - xyz1(Ndo,Ldo)
                     Dif3 = xyz1(Ndo,Kdo) - xyz1(Ndo,Qdo)
                     Dist1 = Dif1*Dif1 + Dist1
                     Dist2 = Dif2*Dif2 + Dist2
                     Dist3 = Dif3*Dif3 + Dist3
                    ENDDO
                    IF (Dist1.gt.0.0d0 .and.
     +                  Dist2.gt.0.0d0 .and.
     +                  Dist3.ge.0.0d0) THEN
                     Dist1 = sqrt(Dist1)
                     Dist2 = sqrt(Dist2)
                     Dist3 = sqrt(Dist3)
                     COSANG = (Dist3*Dist3-Dist1*Dist1-Dist2*Dist2)/
     +                             (-2.0d0*Dist1*Dist2)
                     IF (ABS(COSANG).le.1.0d0)
     +                Wangle = ACOS(COSANG)*RADDEG
                    END IF
                    IF (Wangle .gt. MissingValue+1.0d0) THEN
c
c
                     IF (AngleDescr(Wcurrent,Jdo)(1:2) .eq. 's ') THEN
                      Juse   = AngleTargetClass(Wcurrent,Jdo)
                      Dvalue = Wangle - AngleTargetValues(Wcurrent,Jdo)
                      Adone(Jdo) = .true.
c
c   G      O5'    C5'    C4'     109.4      0.8  130 "(P)O5'-C5'"
c   G      O5'    C5'    C4'     111.5      1.6  145 "C3'-endo"
c   G      O5'    C5'    C4'     111.7      1.9  146 "C2'-endo"
c
                 ELSE IF (index(AngleDescr(Wcurrent,Jdo),
     +                   '(').ne.0) THEN
                  D1 = ABS( Wangle - AngleTargetValues(Wcurrent,Jdo))
                  D2 = ABS( Wangle - AngleTargetValues(Wcurrent,Jdo+1))
                  D3 = ABS( Wangle - AngleTargetValues(Wcurrent,Jdo+2))
                  XM = min(D1,D2,D3)
        IF (XM .lt. D1+0.001d0 .and. XM .gt. D1-0.001d0) THEN
                   Juse = AngleTargetClass(Wcurrent,Jdo)
                   Dvalue = Wangle - AngleTargetValues(Wcurrent,Jdo)
        ELSE IF (XM .lt. D2+0.001d0 .and. XM .gt. D2-0.001d0) THEN
                   Juse = AngleTargetClass(Wcurrent,Jdo+1)
                   Dvalue = Wangle - AngleTargetValues(Wcurrent,Jdo+1)
        ELSE IF (XM .lt. D3+0.001d0 .and. XM .gt. D3-0.001d0) THEN
                   Juse = AngleTargetClass(Wcurrent,Jdo+2)
                   Dvalue = Wangle - AngleTargetValues(Wcurrent,Jdo+2)
        END IF
                  Adone(Jdo)   = .true.
                  Adone(jdo+1) = .true.
                  Adone(Jdo+2) = .true.
c
c   G      C5'    C4'    C3'     115.2      1.4  161 "C2'-endo"
c   G      C5'    C4'    C3'     116.0      1.6  163 "C3'-endo"
c
                     ELSE IF (
     +        index(AngleDescr(Wcurrent,Jdo),'endo').ne.0) THEN
                      D1 = Wangle - AngleTargetValues(Wcurrent,Jdo)
                      D2 = Wangle - AngleTargetValues(Wcurrent,Jdo+1)
                      IF (D1 .lt. D2) THEN
                       Juse = AngleTargetClass(Wcurrent,Jdo)
                       Dvalue = Wangle - AngleTargetValues(Wcurrent,Jdo)
                       ELSE
                       Juse = AngleTargetClass(Wcurrent,Jdo+1)
                       Dvalue = Wangle -
     +                  AngleTargetValues(Wcurrent,Jdo+1)
                      END IF
                      Adone(Jdo) = .true.
                      Adone(Jdo+1) = .true.
c
c   G      O5'    P      OP1     105.7      0.9  113 "small"
c   G      O5'    P      OP1     110.7      1.2  141 "large"
c
                     ELSE IF (AngleDescr(Wcurrent,Jdo)(1:5) .eq. 
     +                'small' ) THEN
                      D1 = Wangle - AngleTargetValues(Wcurrent,Jdo)
                      D2 = Wangle - AngleTargetValues(Wcurrent,Jdo+1)
                      IF (D1. lt. D2) THEN
                       Juse   = AngleTargetClass(Wcurrent,Jdo)
                       Dvalue = Wangle - AngleTargetValues(Wcurrent,Jdo)
                       Adone(Jdo) = .true.
                       Adone(Jdo+1) = .true.
                      ELSE
                       Juse   = AngleTargetClass(Wcurrent,Jdo+1)
                    Dvalue = Wangle - AngleTargetValues(Wcurrent,Jdo+1)
                       Adone(Jdo) = .true.
                       Adone(Jdo+1) = .true.
                      END IF
c
c
                     ELSE IF (AngleDescr(Wcurrent,Jdo) .eq. 
     +                PuckerDescription(1:10)) THEN
                      Juse   = AngleTargetClass(Wcurrent,Jdo)
                      Dvalue = Wangle - AngleTargetValues(Wcurrent,Jdo)
                      Adone(Jdo) = .true.
c
c
                     END IF
c
c
                     IF (Dvalue .gt. MissingValue + 1.0d0) THEN
                      ANArmsdClass(Juse) = ANArmsdClass(Juse) +
     +                                     Dvalue*Dvalue
                      ANATotalType(Juse) = ANATotalType(Juse) + 1
                      WRITE (Iscr,
     +                 fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +                 3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +                 'ANA',Juse,Wmodel,Wangle,Dvalue,Klinker,
     +                 Wres,Wchain,Wseq,Wins,Wname1(Kdo),Walt1(Kdo),
     +                 Wres,Wchain,Wseq,Wins,Wname1(Ldo),Walt1(Ldo),
     +                 Wres,Wchain,Wseq,Wins,Wname1(Qdo),Walt1(Qdo)
                     END IF
                      end if
                     GO TO 20
                    END IF
                   END IF
                  END IF
                 ENDDO
                END IF
               END IF
              END IF
             END IF
            ENDDO
           END IF
          END IF
         ENDDO
        END IF
20      CONTINUE
       ENDDO
c
c
      RETURN
      END
c
c
c         ========================================================
          SUBROUTINE GetChiral(Watoms,Wres,Wseq,Wins,Wchain,Wmodel,
     +                         Walt,xyz,Wname)
c         ========================================================
c
c  IMPROPER   HA  N   C   CB   chirality CA
c  IMPROPER   HB1 HB2 CA  CG   stereo CB
c
c
c     as this number approaches (+) or (-) 180.0, then the
c     error in predicting the true chirality of the center increases.
c
c  Improper dihedrals are a measure of the chirality/planarity of the
c  structure at a specific atom. Values around -35 or +35 are expected
c  for chiral atoms, and values around 0 for planar atoms.
c
c
c   HERE improper C---N----CA---CB done
c    expected answer is around -120 mean -122.52
c
c    D-amino acid is +120.0
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
      CHARACTER Wres*3,Wins*1,Wchain*1
      INTEGER   Wseq,Ma,L,Wmodel,I,J,Kdo,Lx,Watoms,K,Mdo
      CHARACTER ChiralAtoms(10)*4, ResChiral(4)*3,ResPlanar(5)*3,
     +          PlaneAtm(4,2,5)*4
      DOUBLE PRECISION Txyz(3,20), Tvalue,T1,T2
      INTEGER  SchainPnt(16), ChiralSignFlags(4)
      DOUBLE PRECISION xyz(3,MaxAtoms)
      CHARACTER Wname(MaxAtoms)*4,Walt(MaxAtoms)*1
c
c
      DATA ChiralAtoms/' CA ',' CB ',' CG1',' CG2',' OG1',' CG ',
     +     ' CD1',' CD2',' C  ',' N  '/
      DATA ResChiral/'VAL','THR','ILE','LEU'/
      DATA ResPlanar/'ASP','GLU','ARG','PHE','TYR'/
      DATA SchainPnt/1,3,2,4,1,5,2,4,1,3,2,4,2,7,6,8/
      DATA ChiralSignFlags/-1,1,1,-1/
      DATA (PlaneAtm(L,1,1),L=1,4)/' CA ',' CB ',' CG ',' OD1'/
      DATA (PlaneAtm(L,2,1),L=1,4)/' CA ',' CB ',' CG ',' OD2'/
      DATA (PlaneAtm(L,1,2),L=1,4)/' CB ',' CG ',' CD ',' OE1'/
      DATA (PlaneAtm(L,2,2),L=1,4)/' CB ',' CG ',' CD ',' OE2'/
      DATA (PlaneAtm(L,1,3),L=1,4)/' CG ',' NE ',' CZ ',' NH1'/
      DATA (PlaneAtm(L,2,3),L=1,4)/' CG ',' NE ',' CZ ',' NH2'/
      DATA (PlaneAtm(L,1,4),L=1,4)/' CA ',' CB ',' CG ',' CD1'/
      DATA (PlaneAtm(L,2,4),L=1,4)/' CA ',' CB ',' CG ',' CD2'/
      DATA (PlaneAtm(L,1,5),L=1,4)/' CA ',' CB ',' CG ',' CD1'/
      DATA (PlaneAtm(L,2,5),L=1,4)/' CA ',' CB ',' CG ',' CD2'/
c
c---- is it GLYCINE
c
      IF (Wres .eq. 'GLY') RETURN
c
c---- loop over number of altcode sets
c     allowing space + other to be considered one complete residue
c
C
C----- get chirality on Calpha
C
      Ma = 0
      DO Kdo = 1,Watoms
       IF (Walt(Kdo) .eq. 'A'.or.Walt(Kdo) .eq. ' ') THEN
        IF (Wname(Kdo) .eq. ' C  ') THEN
         Ma = Ma + 1
         Txyz(1,1) = xyz(1,Kdo)
         Txyz(2,1) = xyz(2,Kdo)
         Txyz(3,1) = xyz(3,Kdo)
        ELSE IF (Wname(Kdo) .eq. ' N  ') THEN
         Ma = Ma + 1
         Txyz(1,2) = xyz(1,Kdo)
         Txyz(2,2) = xyz(2,Kdo)
         Txyz(3,2) = xyz(3,Kdo)
        ELSE IF (Wname(Kdo) .eq. ' CA ') THEN
         Ma = Ma + 1
         Txyz(1,3) = xyz(1,Kdo)
         Txyz(2,3) = xyz(2,Kdo)
         Txyz(3,3) = xyz(3,Kdo)
        ELSE IF (Wname(Kdo) .eq. ' CB ') THEN
         Ma = Ma + 1
         Txyz(1,4) = xyz(1,Kdo)
         Txyz(2,4) = xyz(2,Kdo)
         Txyz(3,4) = xyz(3,Kdo)
        END IF
       END IF
      ENDDO
c
c
      IF (Ma .eq. 4) THEN
c
c----  improper   C---N---CA---CB
c
        CALL Torsion(1,2,3,4,Txyz,Tvalue)
ccx        write(6,'(a,2x,f10.4)') 'chiral',Tvalue
        IF (Tvalue .gt. 0.0d0)
     +    WRITE (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +   'CHI',0,Wmodel,Tvalue,Tvalue,0,
     +   Wres,Wchain,Wseq,Wins,'    ',Walt(1),
     +   Wres,Wchain,Wseq,Wins,'    ',Walt(1),
     +   Wres,Wchain,Wseq,Wins,'    ',Walt(1)
      END IF
c
c---- the rest here is not required as r500.f doesnt write out the
c     pdb file
c
c
c
      RETURN
c
c
      DO Kdo = 1,4
       IF (Wres .eq. ResChiral(Kdo)) THEN
        Ma = 0
        DO I = 1,Watoms
         IF (Walt(I) .eq. 'A'.or.Walt(I) .eq. ' ') THEN
         DO J = 1,4
          Lx = SchainPnt((Kdo-1)*4+J)
          IF (Wname(I) .eq. ChiralAtoms(Lx)) THEN
           Ma = Ma + 1
           Txyz(1,Ma) = xyz(1,I)
           Txyz(2,Ma) = xyz(2,I)
           Txyz(3,Ma) = xyz(3,I)
           GO TO 140
          END IF
         ENDDO
  140    CONTINUE
        END IF
       ENDDO
c
c   ChiralSignFlags       / -1,    1,    1,    -1/
c
       IF (Ma.ge.4) THEN
        CALL Torsion(1,2,3,4,Txyz,Tvalue)
        Lx = nint(real(ChiralSignFlags(Kdo))*real(Tvalue))
        IF (Lx.gt.0) THEN
         DO I=1,Watoms
          IF (Walt(I) .eq. 'A' .or.Walt(I) .eq. ' ') THEN
           IF( Wres .eq. 'VAL') THEN
            IF (Wname(I) .eq. ' CG1') THEN
             Wname(I) =' CG2'
            ELSE IF (Wname(I) .eq. ' CG2')THEN
             Wname(I) =' CG1'
            END IF
           ELSE IF(Wres .eq. 'LEU') THEN
            IF(Wname(I) .eq. ' CD1')THEN
             Wname(I)=' CD2'
            ELSE IF (Wname(I) .eq. ' CD2') THEN
             Wname(I)=' CD1'
            END IF
           END IF
          END IF
         ENDDO
        END IF
        GO TO 170
       END IF
      END IF
      ENDDO
c
c
  170 Ma = 0
      DO J = 1,5
       IF (Wres .eq. ResPlanar(J)) THEN
        DO I = 1,Watoms
         IF (Walt(I) .eq. 'A' .or.Walt(I) .eq. ' ') THEN
          DO K = 1,2
           DO Mdo = 1,4
            IF (Wname(I) .eq. PlaneAtm(Mdo,K,J)) THEN
             Ma = Ma + 1
             Txyz(1,Ma) = xyz(1,I)
             Txyz(2,Ma) = xyz(2,I)
             Txyz(3,Ma) = xyz(3,I)
             GO TO 200
            END IF
           ENDDO
          ENDDO
  200     CONTINUE
         END IF
        ENDDO
        IF (Ma.ge.8) THEN
c
c-why? this 1-4 and 5-8 ??
c
         CALL Torsion(1,2,3,4,Txyz,T1)
         CALL Torsion(5,6,7,8,Txyz,T2)
         IF (ABS(T1).gt.abs(T2)) THEN
          DO I=1,Watoms
           IF (Walt(I) .eq. 'A' .or.Walt(I) .eq. ' ') THEN
            IF( Wres .eq. 'ARG') THEN
             IF (Wname(I) .eq. ' NH1') THEN
              Wname(I) =' NH2'
             ELSE IF (Wname(I) .eq. ' NH2')THEN
              Wname(I) =' NH1'
             END IF
            ELSE IF(Wres .eq. 'ASP') THEN
             IF(Wname(I) .eq. ' OD1')THEN
              Wname(I)=' OD2'
             ELSE IF (Wname(I) .eq. ' OD2') THEN
              Wname(I)=' OD1'
             END IF
            ELSE IF(Wres .eq. 'GLU') THEN
             IF(Wname(I) .eq. ' OE1')THEN
              Wname(I)=' OE2'
             ELSE IF (Wname(I) .eq. ' OE2') THEN
              Wname(I)=' OE1'
             END IF
            ELSE IF(Wres .eq. 'PHE' .or.Wres .eq. 'TYR') THEN
             IF(Wname(I) .eq. ' CD1')THEN
              Wname(I)=' CD2'
             ELSE IF (Wname(I) .eq. ' CD2') THEN
              Wname(I)=' CD1'
             ELSE IF (Wname(I) .eq. ' CE1')THEN
              Wname(I)=' CE2'
             ELSE IF (Wname(I) .eq. ' CE2') THEN
              Wname(I)=' CE1'
             END IF
            END IF
           END IF
          ENDDO
c
c
         END IF
         END IF
         RETURN
         END IF
         ENDDO
         RETURN
       END
c
c
c        ==================================
         SUBROUTINE Linker(Wmodel,Ntype1,Watoms1,Wseq1,
     +                xyz1,Wname1,Walt1,
     +                Wins1,Wchain1,Wres1,Wdes1,
     +                Ntype0,Watoms0,Wseq0,
     +                xyz0,Wname0,Walt0,
     +                Wins0,Wchain0,Wres0,Wdes0,
     +                Wphi1,Wpsi0,
     +                BAATotalType,AAATotalType,
     +                ANATotalType,BNATotalType,
     +                BAArmsdClass,AAArmsdClass,
     +                ANArmsdClass,BNArmsdClass,
     +                MainChainPlaneCutoff,
     +                PlaneCutoff,CisTrans,debug)
c        ===============================================
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
      LOGICAL  debug
      INTEGER  BAATotalType(MaxBclass+MaxBondLinker),
     +         AAATotalType(MaxAclass+MaxAngleLinker),
     +         ANATotalType(MaxAclass+MaxAngleLinker),
     +         BNATotalType(MaxBclass+MaxBondLinker)
      DOUBLE PRECISION BAArmsdClass(MaxBclass+MaxBondLinker),
     +                 AAArmsdClass(MaxAclass+MaxAngleLinker),
     +                 ANArmsdClass(MaxAclass+MaxAngleLinker),
     +                 BNArmsdClass(MaxBclass+MaxBondLinker)
c
c
      INTEGER NNtype,N1N9,Natoms,NumAtmStanPlane,
     +        NumBondTarget,BondTargetClass,NumAngleTarget,
     +        AngleTargetClass,NumBondLinker,AAflag,
     +        NumAngleLinker
      CHARACTER StanRes3*3,stanNewNam*4,
     +          PlaneNames*4,
     +          BondPairs*4,AngleTriples*4,
     +          BondLinkerClass*12,BondLinkAtom*4,
     +          AngleLinkerClass*12,
     +          AngleLinkAtom*4,BondDescr*10,
     +          AngleDescr*10
      DOUBLE PRECISION BondTargetValues,BondTargetEsds,
     +                 AngleTargetValues,AngleTargetEsds,
     +                 BondLinkerValues,BondLinkerEsds,
     +                 AngleLinkerValues,AngleLinkerEsds
c
c
      COMMON /Cstan/ StanRes3(MaxStandard),
     +       stanNewNam(MaxStandard,MaxAtoms),
     +       PlaneNames(MaxStandard,MaxPlanesAtoms),
     +       BondPairs(2,MaxStandard,MaxBondTarget),
     +       AngleTriples(3,MaxStandard,MaxAngleTarget),
     +       BondLinkerClass(MaxBondLinker),
     +       BondLinkAtom(2,MaxBondLinker),
     +       AngleLinkerClass(MaxAngleLinker),
     +       AngleLinkAtom(3,MaxAngleLinker),
     +       BondDescr(MaxStandard,MaxBondTarget),
     +       AngleDescr(MaxStandard,MaxAngleTarget)
      COMMON /Dstan/
     +       BondTargetClass(MaxStandard,MaxBondTarget),
     +       BondTargetValues(MaxStandard,MaxBondTarget),
     +       BondTargetEsds(MaxStandard,MaxBondTarget),
     +       AngleTargetClass(MaxStandard,MaxAngleTarget),
     +       AngleTargetValues(MaxStandard,MaxAngleTarget),
     +       AngleTargetEsds(MaxStandard,MaxAngleTarget),
     +       BondLinkerValues(MaxBondLinker),
     +       BondLinkerEsds(MaxBondLinker),
     +       AngleLinkerValues(MaxAngleLinker),
     +       AngleLinkerEsds(MaxAngleLinker)
      COMMON /Istan/ NNtype(MaxStandard),
     +       N1N9(MaxStandard),
     +       Natoms(MaxStandard),
     +       NumAtmStanPlane(MaxStandard),
     +       NumBondTarget(MaxStandard),
     +       NumAngleTarget(MaxStandard),
     +       AAflag(MaxStandard),
     +       NumBondLinker,NumAngleLinker
c
c
      LOGICAL Ifail
      INTEGER Wmodel,Ntype1,Watoms1,Wseq1,
     +        Ntype0,Watoms0,Wseq0,
     +        Ovec(3),Iplane,Patoms,
     +        Jdo,Ncur,CaCur,CaLast,Clast,CisTrans,
     +        Klinker,Olast,Pcur,Juse,C3last,OP1,OP2,O5cur,
     +        Ccur,Nlast,Ido
      DOUBLE PRECISION Centre(3),SPxyz(3,MaxPlanesAtoms),
     +                 Evec(3,3),Normal(3),Pxyz(3,MaxPlanesAtoms)
      DOUBLE PRECISION xyz1(3,MaxAtoms),xyz0(3,MaxAtoms),
     +                 MainChainPlaneCutoff,Wplane,
     +                 rmsDIF,
     +                 Prms(MaxPlanesAtoms),PlaneCutoff,
     +                 Calpha,PeptideBnd,NucBnd,Dist,
     +                 Omega,Dvalue,Wangle,
     +                 Dist1,Dist2,Dist3,Dif1,Dif2,Dif3,
     +                 COSANG,RADDEG,PI,D1,D2,Wphi1,Wpsi0
      CHARACTER Wname1(MaxAtoms)*4,Walt1(MaxAtoms)*1,Wins1*1,
     +          Wchain1*1,Wres1*3,Wdes1*10,
     +          Wname0(MaxAtoms)*4,Walt0(MaxAtoms)*1,Wins0*1,
     +          Wchain0*1,Wres0*3,Wdes0*10
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
       Klinker    = 2
       Calpha     = 4.0d0*4.0d0
       PeptideBnd = 1.65d0*1.65d0
       NucBnd     = 1.85d0*1.85d0
       CisTrans   = 0
       CaCur      = -999
       CaLast     = -999
       Ccur       = -999
       Clast      = -999
       Ncur       = -999
       Nlast      = -999
       Olast      = -999
       PI     = ATAN(1.0d0)*4.0d0
       RADDEG = 180.0d0/PI
c
c---- is there a previous residue - DO spanners
c
       IF (Ntype1 .eq. 1) THEN
        DO Jdo=1,Watoms1
         IF (Wname1(Jdo) .eq. ' CA ') THEN
          CaCur = Jdo
          GO TO 10
         END IF
        ENDDO
c
c---- no C-alpha in residue i
c
        RETURN
10      CONTINUE
c
c
        DO Jdo=1,Watoms1
         IF (Wname1(Jdo) .eq. ' C  ' .and.
     +       Walt1(CaCur) .eq. Walt1(Jdo)) THEN
          Ccur = Jdo
         ELSE IF (Wname1(Jdo) .eq. ' N  ' .and.
     +       Walt1(CaCur) .eq. Walt1(Jdo)) THEN
          Ncur = Jdo
         END IF
        ENDDO
        DO Jdo=1,Watoms0
         IF (Wname0(Jdo) .eq. ' N  ' .and.
     +       Walt0(Jdo) .eq. Walt1(CaCur)) THEN
          Nlast = Jdo
         ELSE IF (Wname0(Jdo) .eq. ' C  ' .and.
     +       Walt0(Jdo) .eq. Walt1(CaCur)) THEN
          Clast = Jdo
         ELSE IF (Wname0(Jdo) .eq. ' CA ' .and.
     +       Walt0(Jdo) .eq. Walt1(CaCur)) THEN
          CaLast = Jdo
         ELSE IF (Wname0(Jdo) .eq. ' O  ' .and.
     +            Walt0(Jdo) .eq. Walt1(CaCur)) THEN
          Olast = Jdo
         END IF
        ENDDO
c
c
       IF (CaCur .ne. -999 .and.
     +     CaLast .ne. -999) THEN
        Dist = (xyz1(1,CaCur) - xyz0(1,CaLast))**2 +
     +         (xyz1(2,CaCur) - xyz0(2,CaLast))**2 +
     +         (xyz1(3,CaCur) - xyz0(3,CaLast))**2
c
c---- is it a peptide bond?
c
        IF (Dist .gt. Calpha) THEN
         D1 = sqrt(Dist)
c
c---- try  C(-1)---N
c
         IF (Clast .ne. -999 .and.
     +    Ncur .ne. -999) THEN
          Dist = (xyz1(1,Ncur) - xyz0(1,Clast))**2 +
     +           (xyz1(2,Ncur) - xyz0(2,Clast))**2 +
     +           (xyz1(3,Ncur) - xyz0(3,Clast))**2
          IF (Dist .gt. PeptideBnd ) THEN
           D2 = sqrt(Dist)
           write(6,'(a,2x,i3,2x,a,2x,a,2x,a,f8.3,2x,a,f8.3)')
     +       'bond break:',Wmodel,Wdes0,Wdes1,
     +       'Ca(-1)--Ca = ',D1,' C(-1)--N = ',D2
           RETURN
          ELSE
           GO TO 45
          END IF
         END IF
         RETURN
        END IF
       ELSE
        RETURN
       END IF
 45    CONTINUE
c
c
cPLANARITY = C(i-1) - CA(i-1) - N(i) - O(i-1) ==> planar < 5
c
c---- Calculate Planarity (pseudo) torsion
c
c
         IF (Clast  .gt. 0 .and.
     +       CAlast .gt. 0 .and.
     +       Ncur   .gt. 0 .and.
     +       Olast  .gt. 0 ) THEN
c
c
       DO Jdo=1,MaxPlanesAtoms
        Pxyz(1,Jdo) = MissingValue
       ENDDO
c
c
          DO Jdo=1,3
           Pxyz(Jdo,1) = xyz0(Jdo,Clast)
           Pxyz(Jdo,2) = xyz0(Jdo,CAlast)
           Pxyz(Jdo,3) = xyz1(Jdo,Ncur)
           Pxyz(Jdo,4) = xyz0(Jdo,Olast)
          ENDDO
          CALL Torsion(1,2,3,4,Pxyz,Wplane)
      WRITE (Iscr,
     +fmt='(a3,2x,i6,2x,i4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x),
     +     3x,25(f12.3,2x))')
     +     'PLM',0,Wmodel,Wplane,Wplane,0,
     +     Wres0,Wchain0,Wseq0,Wins0,'    ',' ',
     +     Wres1,Wchain1,Wseq1,Wins1,'    ',' ',
     +     Wres0,Wchain0,Wseq0,Wins0,'    ',' '
         END IF
c
c
c---- Calculate PHI of Wres1
c
         IF (Clast .gt. 0 .and.
     +       Ncur  .gt. 0 .and.
     +       CaCur .gt. 0 .and.
     +       Ccur  .gt. 0 ) THEN
c
c
       DO Jdo=1,MaxPlanesAtoms
        Pxyz(1,Jdo) = MissingValue
       ENDDO
c
c
          DO Jdo=1,3
           Pxyz(Jdo,1) = xyz0(Jdo,Clast)
           Pxyz(Jdo,2) = xyz1(Jdo,Ncur)
           Pxyz(Jdo,3) = xyz1(Jdo,CaCur)
           Pxyz(Jdo,4) = xyz1(Jdo,Ccur)
          ENDDO
          CALL Torsion(1,2,3,4,Pxyz,Wphi1)
         END IF
c
c---- Calculate PSI of Wres0
c
         IF (Nlast   .gt. 0 .and.
     +       CaLast  .gt. 0 .and.
     +       Clast   .gt. 0 .and.
     +       Ncur    .gt. 0 ) THEN
c
c
       DO Jdo=1,MaxPlanesAtoms
        Pxyz(1,Jdo) = MissingValue
       ENDDO
c
c
          DO Jdo=1,3
           Pxyz(Jdo,1) = xyz0(Jdo,Nlast)
           Pxyz(Jdo,2) = xyz0(Jdo,CaLast)
           Pxyz(Jdo,3) = xyz0(Jdo,Clast)
           Pxyz(Jdo,4) = xyz1(Jdo,Ncur)
          ENDDO
          CALL Torsion(1,2,3,4,Pxyz,Wpsi0)
         END IF
c
c
        IF (Ncur   .gt. 0 .and.
     +      CaCur  .gt. 0 .and.
     +      CaLast .gt. 0 .and.
     +      Clast  .gt. 0) THEN
c
c
       DO Jdo=1,MaxPlanesAtoms
        Pxyz(1,Jdo) = MissingValue
       ENDDO
c
c        cis or trans
c---- look for CIS  Ca(-1)----C(-1)----N----Ca
c
          DO Jdo=1,3
           Pxyz(Jdo,1) = xyz0(Jdo,CaLast)
           Pxyz(Jdo,2) = xyz0(Jdo,Clast)
           Pxyz(Jdo,3) = xyz1(Jdo,Ncur)
           Pxyz(Jdo,4) = xyz1(Jdo,CaCur)
          ENDDO
          CALL Torsion(1,2,3,4,Pxyz,Omega)
c
c
          IF (abs(Omega) .le. 30.0d0) THEN
           CisTrans = 1
      WRITE (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +   'CIS',0,Wmodel,omega,omega,0,
     +   Wres1,Wchain1,Wseq1,Wins1,'    ',Walt1(CaCur),
     +   Wres0,Wchain0,Wseq0,Wins0,'    ',Walt1(CaCur),
     +   Wres0,Wchain0,Wseq0,Wins0,'    ',Walt1(CaCur)
          ELSE IF (abs(Omega) .gt. 30.0d0 .and.
     +             abs(Omega) .lt. 150.0d0) THEN
      WRITE (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +   'OME',0,Wmodel,omega,omega,0,
     +   Wres1,Wchain1,Wseq1,Wins1,'    ',Walt1(CaCur),
     +   Wres0,Wchain0,Wseq0,Wins0,'    ',Walt1(CaCur),
     +   Wres0,Wchain0,Wseq0,Wins0,'    ',Walt1(CaCur)
          END IF
c
c---- main chain plane here
c
          Patoms = 4
          DO Jdo=1,3
           DO Ido=1,MaxPlanesAtoms
            SPxyz(Jdo,Ido) = Pxyz(Jdo,Ido)
           ENDDO
          ENDDO
c
c
          CALL Plane(Pxyz,Patoms,Centre,Normal,Evec,
     +               Ovec,Ifail)
c
c
          IF (.not.Ifail) THEN
c
c---- Calculate RMS distance of all points from best-fit
c       plane
c
cOME       0     0    179.3454    179.3454      0  ILE  A      2  GLY  A      1
cMPL       0     0         nan         nan      0  ILE  A      2  GLY  A      1
c
           CALL GetPlaneRMS(Pxyz,Patoms,Normal,rmsDIF,
     +                      Prms,Iplane,PlaneCutoff)
      WRITE (Iscr,
     +fmt='(a3,2x,i6,2x,i4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x),
     +     3x,25(f12.3,2x))')
     +     'MPL',0,Wmodel,rmsDIF,rmsDIF,Iplane,
     +     Wres1,Wchain1,Wseq1,Wins1,'    ',' ',
     +     Wres0,Wchain0,Wseq0,Wins0,'    ',' ',
     +     Wres0,Wchain0,Wseq0,Wins0,'    ',' ',
     +     (Prms(Jdo),Jdo=1,4)
         ELSE
c
c
      IF (debug) WRITE (6,fmt=6000) Wres1,Wchain1,Wseq1,Wins1
 6000 FORMAT (a3,2x,a1,2x,i5,2x,a1, 'Failed Jacobi')
      CALL Praxin(SPxyz,Patoms,Centre,Dvalue,
     +            Iplane,PlaneCutoff,rmsDIF,Prms,debug)
      WRITE (Iscr,
     +fmt='(a3,2x,i6,2x,i4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x),
     +     3x,25(f12.3,2x))')
     +     'MPL',0,Wmodel,Dvalue,rmsDIF,Iplane,
     +     Wres1,Wchain1,Wseq1,Wins1,'    ',' ',
     +     Wres0,Wchain0,Wseq0,Wins0,'    ',' ',
     +     Wres0,Wchain0,Wseq0,Wins0,'    ',' ',
     +     (Prms(Jdo),Jdo=1,4)
         END IF
         END IF
c
c--- fixed problem here for linker betwen altcode residues
c    was doing wrong partner
c
       IF (Clast .ne. -999 .and. Ncur .ne. -999) THEN

c
c
       Dist = (xyz0(1,Clast) - xyz1(1,Ncur))**2 +
     +        (xyz0(2,Clast) - xyz1(2,Ncur))**2 +
     +        (xyz0(3,Clast) - xyz1(3,Ncur))**2
       Dist = sqrt(Dist)
       Dvalue = MissingValue
c
c1 C    N        1.336    0.023    "NOT GLY PRO"
c2 C    N        1.338    0.019    "PRO"
c3 C    N        1.326    0.018    "GLY"
c
         IF (Wres1 .eq. 'PRO') THEN
           Juse   =  MaxBclass + 2
           Dvalue = Dist - BondLinkerValues(2)
         ELSE IF (Wres1 .eq. 'GLY' ) THEN
           Juse   =  MaxBclass + 3
           Dvalue = Dist - BondLinkerValues(1)
         ELSE
           Juse   =  MaxBclass + 1
           Dvalue = Dist - BondLinkerValues(3)
         END IF
         IF (Dvalue .gt. MissingValue + 1.0d0) THEN
          BAArmsdClass(Juse) = BAArmsdClass(Juse) + Dvalue*Dvalue
          BAATotalType(Juse) = BAATotalType(Juse) + 1
          WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +         3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'BAA',Juse,Wmodel,Dist,Dvalue,Klinker,
     +      Wres0,Wchain0,Wseq0,Wins0,' C  ',Walt0(Clast),
     +      Wres1,Wchain1,Wseq1,Wins1,' N  ',Walt1(Ncur),
     +      Wres1,Wchain1,Wseq1,Wins1,'    ','  '
          END IF
        END IF
c
c----- angles
c
c
c1     CA   C    N      117.2      2.2    "NOT GLY PRO"
c2     CA   C    N      117.1      2.8    PRO
c3     CA   C    N      116.2      2.0    GLY
c
        Dvalue = MissingValue
        IF (Ncur   .gt. 0 .and.
     +      CaLast .gt. 0 .and.
     +      Clast  .gt. 0) THEN
          Wangle = 0.0d0
          Dist1 = 0.0d0
          Dist2 = 0.0d0
          Dist3 = 0.0d0
          DO Jdo = 1,3
           Dif1 = xyz0(Jdo,Clast)  - xyz0(Jdo,CaLast)
           Dif2 = xyz1(Jdo,Ncur)   - xyz0(Jdo,Clast)
           Dif3 = xyz0(Jdo,CaLast) - xyz1(Jdo,Ncur)
           Dist1 = Dif1*Dif1 + Dist1
           Dist2 = Dif2*Dif2 + Dist2
           Dist3 = Dif3*Dif3 + Dist3
          ENDDO
          IF (Dist1.gt.0.0d0 .and.
     +        Dist2.gt.0.0d0 .and.
     +        Dist3.ge.0.0d0) THEN
           Dist1 = sqrt(Dist1)
           Dist2 = sqrt(Dist2)
           Dist3 = sqrt(Dist3)
           COSANG = (Dist3*Dist3 - Dist1*Dist1 -
     +               Dist2*Dist2)/(-2.0d0*Dist1*Dist2)
           IF (ABS(COSANG).le.1.0d0)
     +       Wangle = ACOS(COSANG)*RADDEG
          END IF
          IF (Wres1 .eq. 'PRO') THEN
           Juse   =  MaxAclass + 2
           Dvalue = Wangle - AngleLinkerValues(2)
          ELSE IF (Wres1 .eq. 'GLY') THEN
           Juse   =  MaxAclass + 3
           Dvalue = Wangle - AngleLinkerValues(3)
          ELSE
           Juse   =  MaxAclass + 1
           Dvalue = Wangle - AngleLinkerValues(1)
          END IF
          IF (Dvalue .gt. MissingValue + 1.0d0) THEN
           AAArmsdClass(Juse) = AAArmsdClass(Juse) + Dvalue*Dvalue
           AAATotalType(Juse) = AAATotalType(Juse) + 1
           WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'AAA',Juse,Wmodel,Wangle,Dvalue,Klinker,
     +     Wres0,Wchain0,Wseq0,Wins0,Wname0(CaLast),Walt0(CaLast),
     +     Wres0,Wchain0,Wseq0,Wins0,Wname0(Clast),Walt0(Clast),
     +     Wres1,Wchain1,Wseq1,Wins1,Wname1(Ncur),Walt1(Ncur)
           END IF
          END IF
c
c4     O    C    N      122.7      1.6    "NOT PRO GLY"
c5     O    C    N      121.1      1.9    PRO
c6     O    C    N      123.2      1.7    GLY
c
        Dvalue = MissingValue
        IF (Ncur   .gt. 0 .and.
     +      Olast  .gt. 0 .and.
     +      Clast  .gt. 0) THEN
          Wangle = 0.0d0
          Dist1 = 0.0d0
          Dist2 = 0.0d0
          Dist3 = 0.0d0
          DO Jdo = 1,3
           Dif1 = xyz0(Jdo,Clast)  - xyz0(Jdo,Olast)
           Dif2 = xyz1(Jdo,Ncur)   - xyz0(Jdo,Clast)
           Dif3 = xyz0(Jdo,Olast) - xyz1(Jdo,Ncur)
           Dist1 = Dif1*Dif1 + Dist1
           Dist2 = Dif2*Dif2 + Dist2
           Dist3 = Dif3*Dif3 + Dist3
          ENDDO
          IF (Dist1.gt.0.0d0 .and.
     +        Dist2.gt.0.0d0 .and.
     +        Dist3.ge.0.0d0) THEN
           Dist1 = sqrt(Dist1)
           Dist2 = sqrt(Dist2)
           Dist3 = sqrt(Dist3)
           COSANG = (Dist3*Dist3 - Dist1*Dist1 -
     +               Dist2*Dist2)/(-2.0d0*Dist1*Dist2)
           IF (ABS(COSANG).le.1.0d0)
     +       Wangle = ACOS(COSANG)*RADDEG
          END IF
          IF (Wres1 .eq. 'PRO') THEN
           Juse   =  MaxAclass + 5
           Dvalue = Wangle - AngleLinkerValues(5)
          ELSE IF (Wres1 .eq. 'GLY') THEN
           Juse   =  MaxAclass + 6
           Dvalue = Wangle - AngleLinkerValues(6)
          ELSE
           Juse   =  MaxAclass + 4
           Dvalue = Wangle - AngleLinkerValues(4)
          END IF
          IF (Dvalue .gt. MissingValue + 1.0d0) THEN
           AAArmsdClass(Juse) = AAArmsdClass(Juse) + Dvalue*Dvalue
           AAATotalType(Juse) = AAATotalType(Juse) + 1
           WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'AAA',Juse,Wmodel,Wangle,Dvalue,Klinker,
     +     Wres0,Wchain0,Wseq0,Wins0,Wname0(Olast),Walt0(Olast),
     +     Wres0,Wchain0,Wseq0,Wins0,Wname0(Clast),Walt0(Clast),
     +     Wres1,Wchain1,Wseq1,Wins1,Wname1(Ncur),Walt1(Ncur)
           END IF
          END IF
c
c7      C    N    CA     121.7      2.5    "NOT GLY PRO"
c8      C    N    CA     122.0      4.2    PRO-ALL
c9      C    N    CA     127.0      2.4    PRO-CIS
c10     C    N    CA     119.3      1.5    PRO-TRANS
c11     C    N    CA     122.3      2.1    GLY
c
        Dvalue = MissingValue
        IF (Ncur   .gt. 0 .and.
     +      CaCur  .gt. 0 .and.
     +      Clast  .gt. 0) THEN
          Wangle = 0.0d0
          Dist1 = 0.0d0
          Dist2 = 0.0d0
          Dist3 = 0.0d0
          DO Jdo = 1,3
           Dif1 = xyz1(Jdo,Ncur)  - xyz0(Jdo,Clast)
           Dif2 = xyz1(Jdo,CaCur) - xyz1(Jdo,Ncur)
           Dif3 = xyz0(Jdo,Clast) - xyz1(Jdo,CaCur)
           Dist1 = Dif1*Dif1 + Dist1
           Dist2 = Dif2*Dif2 + Dist2
           Dist3 = Dif3*Dif3 + Dist3
          ENDDO
          IF (Dist1.gt.0.0d0 .and.
     +        Dist2.gt.0.0d0 .and.
     +        Dist3.ge.0.0d0) THEN
           Dist1 = sqrt(Dist1)
           Dist2 = sqrt(Dist2)
           Dist3 = sqrt(Dist3)
           COSANG = (Dist3*Dist3 - Dist1*Dist1 -
     +               Dist2*Dist2)/(-2.0d0*Dist1*Dist2)
           IF (ABS(COSANG).le.1.0d0)
     +       Wangle = ACOS(COSANG)*RADDEG
          END IF
          IF (Wres1 .eq. 'PRO') THEN
           IF (CisTrans .eq. 1) THEN
            Juse   =  MaxAclass + 9
            Dvalue = Wangle - AngleLinkerValues(9)
           ELSE
            Juse   =  MaxAclass + 10
            Dvalue = Wangle - AngleLinkerValues(10)
           END IF
          ELSE IF (Wres1 .eq. 'GLY') THEN
           Juse   =  MaxAclass + 11
           Dvalue = Wangle - AngleLinkerValues(11)
          ELSE
           Juse   =  MaxAclass + 7
           Dvalue = Wangle - AngleLinkerValues(7)
          END IF
          IF (Dvalue .gt. MissingValue + 1.0d0) THEN
           AAArmsdClass(Juse) = AAArmsdClass(Juse) + Dvalue*Dvalue
           AAATotalType(Juse) = AAATotalType(Juse) + 1
           WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'AAA',Juse,Wmodel,Wangle,Dvalue,Klinker,
     +     Wres0,Wchain0,Wseq0,Wins0,Wname0(Clast),Walt0(Clast),
     +     Wres1,Wchain1,Wseq1,Wins1,Wname1(Ncur),Walt1(Ncur),
     +     Wres1,Wchain1,Wseq1,Wins1,Wname1(CaCur),Walt1(CaCur)
           END IF
          END IF
c
c--- nucleic acid
c
       ELSE IF (Ntype1 .eq. 0) THEN
c
c4 O3'  P        1.607    0.012    "NUC-ACID-ALL"
c
c
c    P---O
c
        Pcur   = -999
        Olast  = -999
        DO Jdo=1,Watoms1
         IF (Pcur .eq. -999) THEN
          IF (Wname1(Jdo) .eq. ' P  ') THEN
           Pcur = Jdo
           GOTO 50
          END IF
         END IF
        ENDDO
        RETURN
50      CONTINUE
        DO Jdo=1,Watoms0
         IF (Wname0(Jdo) .eq. ' O3''' .and.
     +        Walt0(Jdo) .eq. Walt1(Pcur)) THEN
          Olast = Jdo
         END IF
        ENDDO
c
c
        IF (Pcur   .gt. 0 .and. Olast  .gt. 0 ) THEN
        Dist = (xyz0(1,Olast) - xyz1(1,Pcur))**2 +
     +         (xyz0(2,Olast) - xyz1(2,Pcur))**2 +
     +         (xyz0(3,Olast) - xyz1(3,Pcur))**2
        IF (Dist .le. NucBnd ) THEN
        Dist = sqrt(Dist)
        Dvalue = MissingValue
        Juse   =  MaxBclass + 4
        Dvalue = Dist - BondLinkerValues(4)
        IF (Dvalue .gt. MissingValue + 1.0d0) THEN
          BNArmsdClass(Juse) = BNArmsdClass(Juse) + Dvalue*Dvalue
          BNATotalType(Juse) = BNATotalType(Juse) + 1
          WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'BNA',Juse,Wmodel,Dist,Dvalue,Klinker,
     +      Wres0,Wchain0,Wseq0,Wins0,Wname0(Olast),Walt0(Olast),
     +      Wres1,Wchain1,Wseq1,Wins1,Wname1(Pcur),Walt1(Pcur),
     +      '   ',' ',0,' ','    ','  '
          END IF
         END IF
c
c O3'  P        1.607    0.012    "NUC-ACID-ALL"
c--- find angle atoms
c
        O5cur = -999
        C3last = -999
        OP1 = -999
        OP2  = -999
        DO Jdo=1,Watoms1
          IF (Wname1(Jdo) .eq. ' OP1' .and.
     +        Walt1(Jdo) .eq. Walt1(Pcur))  OP1   = Jdo
          IF (Wname1(Jdo) .eq. ' OP2' .and.
     +        Walt1(Jdo) .eq. Walt1(Pcur))  OP2   = Jdo
          IF (Wname1(Jdo) .eq. ' O5''' .and.
     +        Walt1(Jdo) .eq. Walt1(Pcur)) O5cur = Jdo
        ENDDO
        DO Jdo=1,Watoms0
         IF (Wname0(Jdo) .eq. ' C3''' .and.
     +        Walt0(Jdo) .eq. Walt1(Pcur)) C3last = Jdo
        ENDDO
c
c12     O3'  P    O5'    104.0      1.9    "NA s"
c
        IF (O5cur .ne. -999 ) THEN
          Dvalue = MissingValue
          Wangle = 0.0d0
          Dist1 = 0.0d0
          Dist2 = 0.0d0
          Dist3 = 0.0d0
c
c     O3'  P    O5'    104.0      1.9    "NA s"
c     OP1  P    O3'    105.2      2.2    "NA small"
c     OP1  P    O3'    110.5      1.1    "NA large"
c     OP2  P    O3'    105.2      2.2    "NA small"
c     OP2  P    O3'    110.5      1.1    "NA large"
c     P    O3'  C3'    119.7      1.2    "NA s"
c
c    Olast --- Pcur --- O5cur
c    O3'       P        O5'
c
          DO Jdo = 1,3
           Dif1 = xyz1(Jdo,Pcur)  - xyz0(Jdo,Olast)
           Dif2 = xyz1(Jdo,O5cur) - xyz1(Jdo,Pcur)
           Dif3 = xyz0(Jdo,Olast) - xyz1(Jdo,O5cur)
           Dist1 = Dif1*Dif1 + Dist1
           Dist2 = Dif2*Dif2 + Dist2
           Dist3 = Dif3*Dif3 + Dist3
          ENDDO
          IF (Dist1.gt.0.0d0 .and.
     +        Dist2.gt.0.0d0 .and.
     +        Dist3.ge.0.0d0) THEN
           Dist1 = sqrt(Dist1)
           Dist2 = sqrt(Dist2)
           Dist3 = sqrt(Dist3)
           COSANG = (Dist3*Dist3 - Dist1*Dist1 -
     +               Dist2*Dist2)/(-2.0d0*Dist1*Dist2)
           IF (ABS(COSANG).le.1.0d0)
     +       Wangle = ACOS(COSANG)*RADDEG
           Juse   =  MaxAclass + 12
           Dvalue = Wangle - AngleLinkerValues(12)
          END IF
          IF (Dvalue .gt. MissingValue + 1.0d0) THEN
           ANArmsdClass(Juse) = ANArmsdClass(Juse) + Dvalue*Dvalue
           ANATotalType(Juse) = ANATotalType(Juse) + 1
           WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'ANA',Juse,Wmodel,Wangle,Dvalue,Klinker,
     +     Wres0,Wchain0,Wseq0,Wins0,Wname0(Olast),Walt0(Olast),
     +     Wres1,Wchain1,Wseq1,Wins1,Wname1(Pcur),Walt1(Pcur),
     +     Wres1,Wchain1,Wseq1,Wins1,Wname1(O5cur),Walt1(O5cur)
          END IF
         END IF
c
c13     O3'  P    OP1    105.2      2.2    "NA small"
c14     O3'  P    OP1    110.5      1.1    "NA large"
c
        IF (OP1 .ne. -999 ) THEN
          Dvalue = MissingValue
          Wangle = 0.0d0
          Dist1 = 0.0d0
          Dist2 = 0.0d0
          Dist3 = 0.0d0
          DO Jdo = 1,3
           Dif1 = xyz1(Jdo,Pcur)  - xyz0(Jdo,Olast)
           Dif2 = xyz1(Jdo,OP1) - xyz1(Jdo,Pcur)
           Dif3 = xyz0(Jdo,Olast) - xyz1(Jdo,OP1)
           Dist1 = Dif1*Dif1 + Dist1
           Dist2 = Dif2*Dif2 + Dist2
           Dist3 = Dif3*Dif3 + Dist3
          ENDDO
          IF (Dist1.gt.0.0d0 .and.
     +        Dist2.gt.0.0d0 .and.
     +        Dist3.ge.0.0d0) THEN
           Dist1 = sqrt(Dist1)
           Dist2 = sqrt(Dist2)
           Dist3 = sqrt(Dist3)
           COSANG = (Dist3*Dist3 - Dist1*Dist1 -
     +               Dist2*Dist2)/(-2.0d0*Dist1*Dist2)
           IF (ABS(COSANG).le.1.0d0)
     +       Wangle = ACOS(COSANG)*RADDEG
           D1 = Wangle - AngleLinkerValues(13)
           D2 = Wangle - AngleLinkerValues(14)
c
c
           IF (D1 .lt. D2) THEN
            Juse   =  MaxAclass + 13
            Dvalue = Wangle - AngleLinkerValues(13)
           ELSE
            Juse   =  MaxAclass + 14
            Dvalue = Wangle - AngleLinkerValues(14)
           END IF
c
c
          END IF
c
c
          IF (Dvalue .gt. MissingValue + 1.0d0) THEN
           ANArmsdClass(Juse) = ANArmsdClass(Juse) + Dvalue*Dvalue
           ANATotalType(Juse) = ANATotalType(Juse) + 1
           WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'ANA',Juse,Wmodel,Wangle,Dvalue,Klinker,
     +     Wres0,Wchain0,Wseq0,Wins0,Wname0(Olast),Walt0(Olast),
     +     Wres1,Wchain1,Wseq1,Wins1,Wname1(Pcur),Walt1(Pcur),
     +     Wres1,Wchain1,Wseq1,Wins1,Wname1(OP1),Walt1(OP1)
          END IF
         END IF
c
c15     O3'  P    OP2    105.2      2.2    "NA small"
c16     O3'  P    OP2    110.5      1.1    "NA large"
c
        IF (OP2 .ne. -999 ) THEN
          Dvalue = MissingValue
          Wangle = 0.0d0
          Dist1 = 0.0d0
          Dist2 = 0.0d0
          Dist3 = 0.0d0
          DO Jdo = 1,3
           Dif1 = xyz1(Jdo,Pcur)  - xyz0(Jdo,Olast)
           Dif2 = xyz1(Jdo,OP1) - xyz1(Jdo,Pcur)
           Dif3 = xyz0(Jdo,Olast) - xyz1(Jdo,OP1)
           Dist1 = Dif1*Dif1 + Dist1
           Dist2 = Dif2*Dif2 + Dist2
           Dist3 = Dif3*Dif3 + Dist3
          ENDDO
          IF (Dist1.gt.0.0d0 .and.
     +        Dist2.gt.0.0d0 .and.
     +        Dist3.ge.0.0d0) THEN
           Dist1 = sqrt(Dist1)
           Dist2 = sqrt(Dist2)
           Dist3 = sqrt(Dist3)
           COSANG = (Dist3*Dist3 - Dist1*Dist1 -
     +               Dist2*Dist2)/(-2.0d0*Dist1*Dist2)
           IF (ABS(COSANG).le.1.0d0)
     +       Wangle = ACOS(COSANG)*RADDEG
           D1 = Wangle - AngleLinkerValues(15)
           D2 = Wangle - AngleLinkerValues(16)
           IF (D1 .lt. D2) THEN
            Juse   =  MaxAclass + 15
            Dvalue = Wangle - AngleLinkerValues(15)
           ELSE
            Juse   =  MaxAclass + 16
            Dvalue = Wangle - AngleLinkerValues(16)
           END IF
          END IF
          IF (Dvalue .gt. MissingValue + 1.0d0) THEN
           ANArmsdClass(Juse) = ANArmsdClass(Juse) + Dvalue*Dvalue
           ANATotalType(Juse) = ANATotalType(Juse) + 1
           WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'ANA',Juse,Wmodel,Wangle,Dvalue,Klinker,
     +     Wres0,Wchain0,Wseq0,Wins0,Wname0(Olast),Walt0(Olast),
     +     Wres1,Wchain1,Wseq1,Wins1,Wname1(Pcur),Walt1(Pcur),
     +     Wres1,Wchain1,Wseq1,Wins1,Wname1(OP2),Walt1(OP2)
          END IF
         END IF
c
c17     C3'  O3'  P      119.7      1.2    "NA s"
c
        IF (C3last .ne. -999 ) THEN
          Dvalue = MissingValue
          Wangle = 0.0d0
          Dist1 = 0.0d0
          Dist2 = 0.0d0
          Dist3 = 0.0d0
          DO Jdo = 1,3
           Dif1 = xyz0(Jdo,Olast)  - xyz0(Jdo,C3last)
           Dif2 = xyz1(Jdo,Pcur) - xyz0(Jdo,Olast)
           Dif3 = xyz0(Jdo,C3last) - xyz1(Jdo,Pcur)
           Dist1 = Dif1*Dif1 + Dist1
           Dist2 = Dif2*Dif2 + Dist2
           Dist3 = Dif3*Dif3 + Dist3
          ENDDO
          IF (Dist1.gt.0.0d0 .and.
     +        Dist2.gt.0.0d0 .and.
     +        Dist3.ge.0.0d0) THEN
           Dist1 = sqrt(Dist1)
           Dist2 = sqrt(Dist2)
           Dist3 = sqrt(Dist3)
           COSANG = (Dist3*Dist3 - Dist1*Dist1 -
     +               Dist2*Dist2)/(-2.0d0*Dist1*Dist2)
           IF (ABS(COSANG).le.1.0d0)
     +       Wangle = ACOS(COSANG)*RADDEG
            Juse   =  MaxAclass + 17
            Dvalue = Wangle - AngleLinkerValues(17)
          END IF
          IF (Dvalue .gt. MissingValue + 1.0d0) THEN
           ANArmsdClass(Juse) = ANArmsdClass(Juse) + Dvalue*Dvalue
           ANATotalType(Juse) = ANATotalType(Juse) + 1
           WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'ANA',Juse,Wmodel,Wangle,Dvalue,Klinker,
     +     Wres0,Wchain0,Wseq0,Wins0,Wname0(C3last),Walt0(C3last),
     +     Wres1,Wchain1,Wseq1,Wins1,Wname0(Olast),Walt0(Olast),
     +     Wres1,Wchain1,Wseq1,Wins1,Wname1(Pcur),Walt1(Pcur)
          END IF
         END IF
        END IF
c
         END IF
         RETURN
         END
c
c
c     ==================================================
      SUBROUTINE Rwrite(BondClassEsds,AngleClassEsds,
     +                  U31,U61,rmsdCutOff,
     +                  Nlinks,LINKS,debug,
     +                  PlaneCutoff,
     +                  PlaneNames,IDcode,
     +                  MainChainPlaneCutoff,Fpdb,Fwork)
c     =================================================
c
c  expect
cMaxStandard    37
cNaa            24
cNna            37
cNumBondLinker   4
cNumAngleLinker 11
cMaxAngleTarget 56
cMaxBondTarget  36
cMaxPlanesAtoms 12
cMaxBondLinker   4
cMaxAngleLinker 11
c
cclasses bonds  - 200  aa 1-73, na 100-194
cclasses angles - 250  aa 1-79, na 10-213
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,MaxLinks,MaxCon
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxLinks       = 2000 )
      PARAMETER (MaxCon         = 1000)
c
c
c
      INTEGER Nlinks,Jdo,Nmain,MaxOut,Fpdb
      LOGICAL debug,Adone
      CHARACTER LINKS(MaxLinks)*80,
     +          Wdes1*15,Wdes2*15,Wdes3*15,Wdes4*15,
     +          ConDone(2,MaxCon)*15,IDcode*4,
     +          Lflag(2)*1,Fname*255,Fwork*255,
     +          PlaneNames(MaxStandard,MaxPlanesAtoms)*4
      CHARACTER Qswitch*3,Wn1*4,Wn2*4,Wn3*4,
     +                    Wa1*1,Wa2*1,Wa3*1,
     +                    Wc1*1,Wc2*1,Wc3*3,
     +                    Wi1*1,Wi2*1,Wi3*1,
     +                    Wr1*3,Wr2*3,Wr3*3
      INTEGER Ws1,Ws2,Ws3,Wmodel,Klinker,Kcis,KBadCalphaChiral,
     +        KBadAngle,KBadPlane,KBadBond,Komega,Nplane,
     +        Juse,U31,U61,U71,Kbadlink,NbadRama,Nclose,Nsymm
      DOUBLE PRECISION Dvalue,Value,rmsdCutOff,PlaneCutoff,
     +     Prms(MaxPlanesAtoms),MainChainPlaneCutoff
      CHARACTER OrtSym(125)*3,MainChain(4)*6,Pwork(4)*10
      DOUBLE PRECISION BondClassEsds(MaxBclass+MaxBondLinker),
     +                 AngleClassEsds(MaxAclass+MaxAngleLinker)
c
c
      CHARACTER Cwork*1000
ccx      INTEGER lenstr,system,flush,LLx
ccx there is no flush ? on gfortran ??
c
      INTEGER r500_lenstr,system,LLx
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
      DATA OrtSym/
     +     '333','334','335','336','337','343','344','345','346','347',
     +     '353','354','355','356','357','363','364','365','366','367',
     +     '373','374','375','376','377','433','434','435','436','437',
     +     '443','444','445','446','447','453','454','455','456','457',
     +     '463','464','465','466','467','473','474','475','476','477',
     +     '533','534','535','536','537','543','544','545','546','547',
     +     '553','554','555','556','557','563','564','565','566','567',
     +     '573','574','575','576','577','633','634','635','636','637',
     +     '643','644','645','646','647','653','654','655','656','657',
     +     '663','664','665','666','667','673','674','675','676','677',
     +     '733','734','735','736','737','743','744','745','746','747',
     +     '753','754','755','756','757','763','764','765','766','767',
     +     '773','774','775','776','777'/
c
c
      DATA MainChain/'CA(-1)', ' C(-1)', ' N    ', ' CA   '/
      DATA Pwork/'SIDE CHAIN','MAIN CHAIN',
     +           'SIDE-CHAIN','MAIN-CHAIN'/
c
c
      REWIND Iscr
      KBadAngle = 0
      KBadPlane = 0
      KBadBond  = 0
      Wmodel    = 0
      Kbadlink  = 0
      Lflag(1)  = 'N'
      Lflag(2)  = 'Y'
      Kcis      = 0
      Komega    = 0
      U71       = 0
      KBadCalphaChiral = 0
      Nclose    = 0
      Nsymm     = 0
      Nplane    = 0
      Nmain     = 0
      MaxOut    = 50
c
c
      Adone = .false.
c----------------------------------------------------------------------
c
c---- close contacts error here
c
      Nclose     = 0
c
c
  140 READ (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))'
     +       ,end=150)
     +   Qswitch,Juse,Wmodel,Value,Dvalue,Klinker,
     +   Wr1,Wc1,Ws1,Wi1,Wn1,Wa1,
     +   Wr2,Wc2,Ws2,Wi2,Wn2,Wa2,
     +   Wr3,Wc3,Ws3,Wi3,Wn3,Wa3
c
c
      IF (Qswitch .eq. 'CON') THEN
c
c  NOTE SSBOND's are excluded in the search procedure
c
          WRITE(Wdes1,'(a4,a1,a3,1x,a1,i4,a1)')
     +            Wn1,Wa1,Wr1,Wc1,Ws1,Wi1
          WRITE(Wdes2,'(a4,a1,a3,1x,a1,i4,a1)')
     +            Wn2,Wa2,Wr2,Wc2,Ws2,Wi2

c
c  only do one of these
c  1     1   O    B  VAL     36  .  .     NH2  B  ARG    108  .  .         2.16
c  2     1   NH2  B  ARG    108  .  .     O    B  VAL     36  .  .         2.16
c
c
         IF (Nlinks .gt. 0 ) THEN
          DO Jdo =1,Nlinks
            READ(LINKS(Jdo),fmt='(12x,a15,15x,a15)') Wdes3,Wdes4
            IF (Wdes1 .eq. Wdes3 .or. Wdes1 .eq. Wdes4) THEN
             IF (Wdes2 .eq. Wdes3 .or. Wdes2 .eq. Wdes4) THEN
c
c---- it is a LINK ignore
c
              GO TO 140
            END IF
           END IF
          ENDDO
         END IF
c
c--- only output unique set
c
       IF (Nclose .gt. 0) THEN
        DO Jdo=1,Nclose
ccx         write(6, '(4(a,2x))') Wdes1,Wdes2,ConDone(1,Jdo),ConDone(2,Jdo)
         IF (Wdes1 .eq. ConDone(2,Jdo)) THEN
           IF (Wdes2 .eq. ConDone(1,Jdo)) THEN
           GO TO 140
           END IF
          END IF
         IF (Wdes1 .eq. ConDone(1,Jdo)) THEN
           IF (Wdes2 .eq. ConDone(2,Jdo)) THEN
           GO TO 140
           END IF
          END IF
         ENDDO
        END IF
c
c
       Nclose = Nclose + 1
       IF (Nclose .gt. MaxCon) GO TO 150
       ConDone(1,Nclose) = Wdes1
       ConDone(2,Nclose) = Wdes2

       IF (Nclose .eq. 1) THEN

         IF (U61 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          END IF
         END IF

          WRITE (Irout,fmt=6021)
 6021 FORMAT (
     +'REMARK 500',/,
     +'REMARK 500 GEOMETRY AND STEREOCHEMISTRY',/,
     +'REMARK 500 SUBTOPIC: CLOSE CONTACTS IN SAME ASYMMETRIC UNIT',/,
     +'REMARK 500',/,
     +'REMARK 500 THE FOLLOWING ATOMS ARE IN CLOSE CONTACT.',/,
     +'REMARK 500',/,
     +'REMARK 500  ATM1  RES C  SSEQI   ATM2  RES C  SSEQI           ',
     +     'DISTANCE')

         IF (U31 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          END IF
         END IF

         write(Icif,fmt=6022)
6022  FORMAT(
     +  '#',/,'loop_',/,
     +  '_pdbx_validate_close_contact.id',/,
     +  '_pdbx_validate_close_contact.PDB_model_num',/,
     +  '_pdbx_validate_close_contact.auth_atom_id_1',/,
     +  '_pdbx_validate_close_contact.auth_asym_id_1',/,
     +  '_pdbx_validate_close_contact.auth_comp_id_1',/,
     +  '_pdbx_validate_close_contact.auth_seq_id_1',/,
     +  '_pdbx_validate_close_contact.PDB_ins_code_1',/,
     +  '_pdbx_validate_close_contact.label_alt_id_1',/,
     +  '_pdbx_validate_close_contact.auth_atom_id_2',/,
     +  '_pdbx_validate_close_contact.auth_asym_id_2',/,
     +  '_pdbx_validate_close_contact.auth_comp_id_2',/,
     +  '_pdbx_validate_close_contact.auth_seq_id_2',/,
     +  '_pdbx_validate_close_contact.PDB_ins_code_2',/,
     +  '_pdbx_validate_close_contact.label_alt_id_2',/,
     +  '_pdbx_validate_close_contact.dist')
       END IF

        IF (Nclose .le. MaxOut)
     +   WRITE (Irout,fmt=6023)
     +          Wn1,Wa1,Wr1,Wc1,Ws1,Wi1,
     +          Wn2,Wa2,Wr2,Wc2,Ws2,Wi2, Value
 6023 FORMAT ('REMARK 500',2X,a4,a1,1X,a3,1X,a1,
     +        2X,i4,a1,' - ',a4,a1,1X,a3,
     +        1X,a1,2X,i4,a1,10X,f7.2)

ccx       Wmodel = 0
       IF (Wc1 .eq. ' ') Wc1 = '.'
       IF (Wi1 .eq. ' ') Wi1 = '.'
       IF (Wa1 .eq. ' ') Wa1 = '.'
       IF (Wc2 .eq. ' ') Wc2 = '.'
       IF (Wi2 .eq. ' ') Wi2 = '.'
       IF (Wa2 .eq. ' ') Wa2 = '.'
       WRITE(Icif,
     +     fmt='(i4,2x,i4,
     + 2(2x,a4,2x,a1,2x,a3,2x,i5,2x,a1,2x,a1,2x),4x,f7.2)')
     +     Nclose,Wmodel,Wn1,Wc1,Wr1,Ws1,Wi1,Wa1,
     +                   Wn2,Wc2,Wr2,Ws2,Wi2,Wa2,
     +                 Value
      END IF
      GO TO 140
  150 IF (Nclose.gt.0) THEN
       IF (Nclose .gt. MaxOut) WRITE(Irout,6105) Nclose
 6105 FORMAT ('REMARK 500',/,
     +'REMARK 500 THIS ENTRY HAS ',i7,' CLOSE CONTACTS')
       WRITE (Irout,fmt=6005)
      END IF
c
c
      REWIND Iscr
c------------------------------------------------------------------
c
c
c---- symmetry contacts error here
c
      Nsymm     = 0
  160 READ (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))'
     +       ,end=170)
     +   Qswitch,Juse,Wmodel,Value,Dvalue,Klinker,
     +   Wr1,Wc1,Ws1,Wi1,Wn1,Wa1,
     +   Wr2,Wc2,Ws2,Wi2,Wn2,Wa2,
     +   Wr3,Wc3,Ws3,Wi3,Wn3,Wa3
      IF (Qswitch .eq. 'SYM') THEN
c
c---- warning LINK records  dont have symmetry ?
c     No check done on unqiue set here ???
c
         Nsymm = Nsymm + 1
       IF (Nsymm .eq. 1) THEN

         IF (U61 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          END IF
         END IF

          WRITE (Irout,fmt=6024)
 6024 FORMAT (
     +'REMARK 500',/,
     +'REMARK 500 GEOMETRY AND STEREOCHEMISTRY',/,
     +'REMARK 500 SUBTOPIC: CLOSE CONTACTS',/,
     +'REMARK 500',/,
     +'REMARK 500 THE FOLLOWING ATOMS THAT ARE RELATED BY ',
     +    'CRYSTALLOGRAPHIC',/,
     +'REMARK 500 SYMMETRY ARE IN CLOSE CONTACT.  AN ATOM LOCATED ',
     +    'WITHIN 0.15 ',/,
     +'REMARK 500 ANGSTROMS OF A SYMMETRY RELATED ATOM IS ASSUMED ',
     +    'TO BE ON A ',/,
     +'REMARK 500 SPECIAL POSITION AND IS, THEREFORE, LISTED IN ',
     +    'REMARK 375 ',/,
     +'REMARK 500 INSTEAD OF REMARK 500.  ATOMS WITH ',
     +       'NON-BLANK ALTERNATE',/,
     +'REMARK 500 LOCATION INDICATORS ARE NOT INCLUDED IN THE ',
     +     'CALCULATIONS.',/,
     + 'REMARK 500',/,
     + 'REMARK 500 DISTANCE CUTOFF: ',/,
     + 'REMARK 500 2.2 ANGSTROMS FOR CONTACTS NOT ',
     +             'INVOLVING HYDROGEN ATOMS',/,
     + 'REMARK 500 1.6 ANGSTROMS FOR CONTACTS ',
     +              'INVOLVING HYDROGEN ATOMS',/,
     + 'REMARK 500',/,
     + 'REMARK 500  ATM1  RES C  SSEQI   ATM2  RES C  ',
     +             'SSEQI  SSYMOP   DISTANCE',/,'REMARK 500')
         IF (U31 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          END IF
         END IF
         write(Icif,fmt=6025)
6025  FORMAT(
     +  '#',/,'loop_',/,
     +  '_pdbx_validate_symm_contact.id',/,
     +  '_pdbx_validate_symm_contact.PDB_model_num',/,
     +  '_pdbx_validate_symm_contact.auth_atom_id_1',/,
     +  '_pdbx_validate_symm_contact.auth_asym_id_1',/,
     +  '_pdbx_validate_symm_contact.auth_comp_id_1',/,
     +  '_pdbx_validate_symm_contact.auth_seq_id_1',/,
     +  '_pdbx_validate_symm_contact.PDB_ins_code_1',/,
     +  '_pdbx_validate_symm_contact.label_alt_id_1',/,
     +  '_pdbx_validate_symm_contact.site_symmetry_1',/,
     +  '_pdbx_validate_symm_contact.auth_atom_id_2',/,
     +  '_pdbx_validate_symm_contact.auth_asym_id_2',/,
     +  '_pdbx_validate_symm_contact.auth_comp_id_2',/,
     +  '_pdbx_validate_symm_contact.auth_seq_id_2',/,
     +  '_pdbx_validate_symm_contact.PDB_ins_code_2',/,
     +  '_pdbx_validate_symm_contact.label_alt_id_2',/,
     +  '_pdbx_validate_symm_contact.site_symmetry_2',/,
     +  '_pdbx_validate_symm_contact.dist')
       END IF
        IF (Nsymm .le. MaxOut)
     +   WRITE (Irout,fmt=6026)
     +          Wn1,Wa1,Wr1,Wc1,Ws1,Wi1,
     +          Wn2,Wa2,Wr2,Wc2,Ws2,Wi2,
     +          Klinker, OrtSym(Juse),Value
 6026 FORMAT ('REMARK 500',2X,a4,a1,1X,a3,1X,a1,2X,i4,a1,
     +                     3X,a4,a1,1X,a3,1X,a1,2X,i4,a1,
     +                     2X,i3,a3,4X,f6.2)
ccx       Wmodel = 0
       IF (Wc1 .eq. ' ') Wc1 = '.'
       IF (Wi1 .eq. ' ') Wi1 = '.'
       IF (Wa1 .eq. ' ') Wa1 = '.'
       IF (Wc2 .eq. ' ') Wc2 = '.'
       IF (Wi2 .eq. ' ') Wi2 = '.'
       IF (Wa2 .eq. ' ') Wa2 = '.'
       WRITE(Icif,
     +     fmt='(i4,2x,i4,
     +     2(2x,a4,2x,a1,2x,a3,2x,i5,2x,a1,2x,a1,2x,i3,a3),4x,f7.2)')
     +     Nsymm,Wmodel,
     +     Wn1,Wc1,Wr1,Ws1,Wi1,Wa1, 1,'555',
     +     Wn2,Wc2,Wr2,Ws2,Wi2,Wa2, Klinker, OrtSym(Juse),
     +     Value
      END IF
      GO TO 160
  170 IF (Nsymm.gt.0) THEN
       IF (Nsymm .gt. MaxOut) WRITE(Irout,6205) Nsymm
 6205 FORMAT ('REMARK 500',/,
     +'REMARK 500 THIS ENTRY HAS ',i7,' SYMMETRY CONTACTS')
       WRITE (Irout,fmt=6005)
      END IF
c
c
      REWIND Iscr
c------------------------------------------------------------------
c
c---- bond lengths this pass
c
   40 READ (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))'
     +       ,end=50)
     +   Qswitch,Juse,Wmodel,Value,Dvalue,Klinker,
     +   Wr1,Wc1,Ws1,Wi1,Wn1,Wa1,
     +   Wr2,Wc2,Ws2,Wi2,Wn2,Wa2,
     +   Wr3,Wc3,Ws3,Wi3,Wn3,Wa3
      IF (Qswitch .eq. 'BAA' .or. Qswitch .eq. 'BNA') THEN
         IF (ABS(Dvalue).gt.rmsdCutOff*BondClassEsds(Juse)) THEN
        KBadBond = KBadBond + 1
        IF (KBadBond .eq. 1) THEN

         IF (U61 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          END IF
         END IF

         WRITE (Irout,fmt=6006) 
 6006 FORMAT (
     +'REMARK 500',/,
     +'REMARK 500 GEOMETRY AND STEREOCHEMISTRY',/,
     +'REMARK 500 SUBTOPIC: COVALENT BOND LENGTHS',/,
     +'REMARK 500',/,
     +'REMARK 500 THE STEREOCHEMICAL PARAMETERS OF ',
     +     'THE FOLLOWING RESIDUES',/,
     +'REMARK 500 HAVE VALUES WHICH DEVIATE FROM ',
     +     'EXPECTED VALUES BY MORE',/,
     +'REMARK 500 THAN 6*RMSD (M=MODEL NUMBER; ',
     +     'RES=RESIDUE NAME; C=CHAIN',/,
     +'REMARK 500 IDENTIFIER; SSEQ=SEQUENCE NUMBER; ',
     +     'I=INSERTION CODE).',/,
     +'REMARK 500', /,'REMARK 500 STANDARD TABLE:', /,
     +'REMARK 500 FORMAT: (10X,I3,1X,A3,1X,A1,I4,A1,1X,2(A4,A1,3X)',
     +      ',12X,F5.3)',/,
     +'REMARK 500', /,
     +'REMARK 500 EXPECTED VALUES PROTEIN: ENGH AND HUBER,',
     +    ' 1999', /,
     +'REMARK 500 EXPECTED VALUES NUCELIC ACID: CLOWNEY ET AL',
     +    ' 1996', /,
     +'REMARK 500', /,
     +'REMARK 500  M RES CSSEQI ATM1   RES CSSEQI ATM2   DEVIATION')
         IF (U31 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          END IF
         END IF
          WRITE(Icif,6007)
 6007  FORMAT(//,'#',/,'loop_',/,
     +'_pdbx_validate_rmsd_bond.id',/,
     +'_pdbx_validate_rmsd_bond.PDB_model_num',/,
     +'_pdbx_validate_rmsd_bond.auth_asym_id_1',/,
     +'_pdbx_validate_rmsd_bond.auth_atom_id_1',/,
     +'_pdbx_validate_rmsd_bond.auth_comp_id_1',/,
     +'_pdbx_validate_rmsd_bond.auth_seq_id_1',/,
     +'_pdbx_validate_rmsd_bond.auth_PDB_ins_code_1',/,
     +'_pdbx_validate_rmsd_bond.label_alt_id_1',/,
     +'_pdbx_validate_rmsd_bond.auth_asym_id_2',/,
     +'_pdbx_validate_rmsd_bond.auth_atom_id_2',/,
     +'_pdbx_validate_rmsd_bond.auth_comp_id_2',/,
     +'_pdbx_validate_rmsd_bond.auth_seq_id_2',/,
     +'_pdbx_validate_rmsd_bond.auth_PDB_ins_code_2',/,
     +'_pdbx_validate_rmsd_bond.label_alt_id_2',/,
     +'_pdbx_validate_rmsd_bond.bond_deviation',/,
     +'_pdbx_validate_rmsd_bond.bond_value',/,
     +'_pdbx_validate_rmsd_bond.linker_flag')
        END IF
c
c
         IF (KBadBond .le. MaxOut) THEN
c
c
         IF (Wmodel .ne. 0) THEN
          WRITE (Irout,fmt=6008) Wmodel,
     +     Wr1,Wc1,Ws1,Wi1,Wn1,Wa1,
     +     Wr2,Wc2,Ws2,Wi2,Wn2,Wa2,
     +     Dvalue
 6008 FORMAT (
     +'REMARK 500',I3,1X,A3,1X,A1,i4,a1,1x,A4,a1,2X,
     +                   A3,1X,A1,i4,a1,1x,A4,a1,3X,
     +                   f6.3)
         ELSE
          WRITE (Irout,fmt=6009)
     +     Wr1,Wc1,Ws1,Wi1,Wn1,Wa1,
     +     Wr2,Wc2,Ws2,Wi2,Wn2,Wa2,
     +     Dvalue
 6009 FORMAT (
     +'REMARK 500',4X,   A3,1X,A1,i4,a1,1x,A4,a1,2X,
     +                   A3,1X,A1,i4,a1,1x,A4,a1,3X,
     +                   f6.3)
         END IF
c
c
        END IF
c
c
        IF (Wc1 .eq. ' ') Wc1 = '.'
        IF (Wc2 .eq. ' ') Wc2 = '.'
        IF (Wi1 .eq. ' ') wi1 = '.'
        IF (Wi2 .eq. ' ') Wi2 = '.'
        IF (Wa1 .eq. ' ') Wa1 = '.'
        IF (Wa2 .eq. ' ') Wa2 = '.'
c
c---- cif both types spanner and per residue
c
        WRITE(Icif,
     +  fmt='(1x,i8,2x,i3,2(2x,a1,2x,a4,2x,a3,2x,i5,2x,a1,2x,a1,2x),
     +         2x,f8.3,2x,f8.3,2x,a1)')
     +      KBadBond,Wmodel,
     +      Wc1, Wn1, Wr1, Ws1, Wi1, Wa1,
     +      Wc2, Wn2, Wr2, Ws2, Wi2, Wa2,
     +     Dvalue,Value,Lflag(Klinker)
       END IF
      END IF
      GO TO 40
   50 IF (Nclose .gt. MaxOut) WRITE(Irout,6305) KBadBond
 6305 FORMAT ('REMARK 500',/,
     +'REMARK 500 THIS ENTRY HAS ',i7,' BOND DEVIATIONS')
      IF (KBadBond.gt.0) WRITE (Irout,fmt=6005)
c
c
      REWIND Iscr
c-------------------------------------------------------------------------


   20 READ (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))'
     +       ,end=30)
     +   Qswitch,Juse,Wmodel,Value,Dvalue,Klinker,
     +   Wr1,Wc1,Ws1,Wi1,Wn1,Wa1,
     +   Wr2,Wc2,Ws2,Wi2,Wn2,Wa2,
     +   Wr3,Wc3,Ws3,Wi3,Wn3,Wa3
c
c---- angles this pass
c
      IF (Qswitch .eq. 'AAA' .or. Qswitch .eq. 'ANA') THEN

        IF (ABS(Dvalue).gt.rmsdCutOff*AngleClassEsds(Juse)) THEN

        IF (Klinker .eq. 2) THEN
             Kbadlink = Kbadlink + 1
        ELSE
             KBadAngle = KBadAngle + 1
        END IF

        IF (KBadAngle+Kbadlink .eq. 1) THEN

         IF (U61 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          END IF
         END IF

          IF(.not.Adone) THEN
            WRITE (Irout,fmt=6001) NINT(rmsdCutOff)
6001        FORMAT (
     +'REMARK 500',/,
     +'REMARK 500 GEOMETRY AND STEREOCHEMISTRY',/,
     +'REMARK 500 SUBTOPIC: COVALENT BOND ANGLES',/,
     +'REMARK 500',/,
     +'REMARK 500 THE STEREOCHEMICAL PARAMETERS OF THE FOLLOWING',
     +      ' RESIDUES',/,
     +'REMARK 500 HAVE VALUES WHICH DEVIATE FROM EXPECTED ',
     +      'VALUES BY MORE',/,
     +'REMARK 500 THAN ',i1,'*RMSD (M=MODEL NUMBER; RES=',
     +      'RESIDUE NAME; C=CHAIN',/,
     +'REMARK 500 IDENTIFIER; SSEQ=SEQUENCE NUMBER; ',
     +      'I=INSERTION CODE).',/,
     +'REMARK 500',/,
     +'REMARK 500 STANDARD TABLE:',/,
     +'REMARK 500 FORMAT: (10X,I3,1X,A3,1X,A1,I4,A1,3(1X,A4,2X),',
     +      '12X,F5.1)',/,
     +'REMARK 500',/,
     +'REMARK 500 EXPECTED VALUES: ENGH AND HUBER, 1999',/,
     +'REMARK 500 EXPECTED VALUES NUCELIC ACID: CLOWNEY ET AL',
     +    ' 1996', /,
     +'REMARK 500',/,'REMARK 500  M RES CSSEQI ATM1   ATM2   ATM3')
            Adone = .true.
          END IF

        END IF

        IF (KBadAngle +Kbadlink .eq. 1) THEN
         IF (U31 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          END IF
         END IF

         WRITE(Icif,6002)
 6002 FORMAT(//,'#',/,'loop_',/,
     +'_pdbx_validate_rmsd_angle.id',/,
     +'_pdbx_validate_rmsd_angle.PDB_model_num',/,
     +'_pdbx_validate_rmsd_angle.auth_asym_id_1',/,
     +'_pdbx_validate_rmsd_angle.auth_atom_id_1',/,
     +'_pdbx_validate_rmsd_angle.auth_comp_id_1',/,
     +'_pdbx_validate_rmsd_angle.auth_seq_id_1',/,
     +'_pdbx_validate_rmsd_angle.PDB_ins_code_1',/,
     +'_pdbx_validate_rmsd_angle.label_alt_id_1',/,
     +'_pdbx_validate_rmsd_angle.auth_asym_id_2',/,
     +'_pdbx_validate_rmsd_angle.auth_atom_id_2',/,
     +'_pdbx_validate_rmsd_angle.auth_comp_id_2',/,
     +'_pdbx_validate_rmsd_angle.auth_seq_id_2',/,
     +'_pdbx_validate_rmsd_angle.PDB_ins_code_2',/,
     +'_pdbx_validate_rmsd_angle.label_alt_id_2',/,
     +'_pdbx_validate_rmsd_angle.auth_asym_id_3',/,
     +'_pdbx_validate_rmsd_angle.auth_atom_id_3',/,
     +'_pdbx_validate_rmsd_angle.auth_comp_id_3',/,
     +'_pdbx_validate_rmsd_angle.auth_seq_id_3',/,
     +'_pdbx_validate_rmsd_angle.PDB_ins_code_3',/,
     +'_pdbx_validate_rmsd_angle.label_alt_id_3',/,
     +'_pdbx_validate_rmsd_angle.angle_deviation',/,
     +'_pdbx_validate_rmsd_angle.angle_value',/,
     +'_pdbx_validate_rmsd_angle.linker_flag')

        END IF
c
c--- where write out Wins, Walt3,Walt4 ??
c
c now write out linker angles to pdb format
c
         IF (KBadAngle +Kbadlink .le. MaxOut) THEN
c
c
         IF (Klinker .eq. 1) THEN
         IF (Wmodel.gt.0) THEN
          WRITE (Irout,fmt=6003)
     +    Wmodel,Wr1,Wc1,Ws1,Wi1,Wn1,Wn2,Wn3,Dvalue
 6003     FORMAT ('REMARK 500',I3,1X,A3,1X,A1,I4,a1,1X,A4,
     +    ' - ',A4,' - ',A4,' ANGL. DEV. = ',F5.1,' DEGREES')
         ELSE
          WRITE (Irout,fmt=6004)
     +    Wr1,Wc1,Ws1,Wi1,Wn1,Wn2,Wn3,Dvalue
 6004     FORMAT ('REMARK 500',4X,A3,1X,A1,I4,a1,1X,A4,
     +    ' - ',A4,' - ',A4,' ANGL. DEV. = ',F5.1,' DEGREES')
         END IF

          else
c
c--- if linker use central atom as pdb format doesnt allow
c    two residue names
c
         IF (Wmodel.gt.0) THEN
          WRITE (Irout,fmt=6003)
     +    Wmodel,Wr2,Wc2,Ws2,Wi2,Wn1,Wn2,Wn3,Dvalue
         ELSE
          WRITE (Irout,fmt=6004)
     +    Wr2,Wc2,Ws2,Wi2,Wn1,Wn2,Wn3,Dvalue
         END IF
         END IF
c
c
         END IF
c
c
         IF (Wc1 .eq. ' ') Wc1 = '.'
         IF (Wc2 .eq. ' ') Wc2 = '.'
         IF (Wc3 .eq. ' ') Wc3 = '.'
         IF (Wi1 .eq. ' ') Wi1 = '.'
         IF (Wi2 .eq. ' ') Wi2 = '.'
         IF (Wi3 .eq. ' ') Wi3 = '.'
         IF (Wa1 .eq. ' ') Wa1 = '.'
         IF (Wa2 .eq. ' ') Wa2 = '.'
         IF (Wa3 .eq. ' ') Wa3 = '.'
         WRITE(Icif,
     + fmt='(2x,i8,2x,i3,1x,3(a1,1x,a4,1x,a3,1x,i4,1x,a1,1x,a1,2x),
     +         2x,f7.2,2x,f7.2,2x,a1)')
     +      (KBadAngle+KBadLink),Wmodel,
     +      Wc1,Wn1,Wr1,Ws1,Wi1,Wa1,
     +      Wc2,Wn2,Wr2,Ws2,Wi2,Wa2,
     +      Wc2,Wn3,Wr3,Ws3,Wi3,Wa3,
     +      Dvalue,Value,Lflag(Klinker)
       END IF
      END IF
      GO TO 20
   30 IF (KBadAngle +Kbadlink .gt. MaxOut) 
     +        WRITE(Irout,6405) KBadAngle +Kbadlink
 6405 FORMAT ('REMARK 500',/,
     +'REMARK 500 THIS ENTRY HAS ',i7,' ANGLE DEVIATIONS')
      IF (KBadAngle.gt.0)
     +  WRITE (Irout,fmt=6005)
 6005 FORMAT ('REMARK 500',/,'REMARK 500 REMARK: NULL')
c
c
      REWIND Iscr
c----------------------------------------------------------------
c
c---- torsion error here
c
      NbadRama     = 0
   60 READ (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))'
     +       ,end=70)
     +   Qswitch,Juse,Wmodel,Value,Dvalue,Klinker,
     +   Wr1,Wc1,Ws1,Wi1,Wn1,Wa1,
     +   Wr2,Wc2,Ws2,Wi2,Wn2,Wa2,
     +   Wr3,Wc3,Ws3,Wi3,Wn3,Wa3
      IF (Qswitch .eq. 'RAM' .and. Klinker .eq. 1) THEN
         NbadRama = NbadRama + 1
       IF (NbadRama .eq. 1) THEN
         IF (U61 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          END IF
         END IF
          WRITE (Irout,fmt=6010)
 6010  FORMAT (
     +'REMARK 500',/,
     +'REMARK 500 GEOMETRY AND STEREOCHEMISTRY',/,
     +'REMARK 500 SUBTOPIC: TORSION ANGLES',/,
     +'REMARK 500',/,
     +'REMARK 500 TORSION ANGLES OUTSIDE THE EXPECTED ',
     +'RAMACHANDRAN REGIONS:',/,
     +'REMARK 500 (M=MODEL NUMBER; RES=RESIDUE NAME; ',
     +'C=CHAIN IDENTIFIER;',/,
     +'REMARK 500 SSEQ=SEQUENCE NUMBER; I=INSERTION CODE).',/,
     +'REMARK 500 STANDARD TABLE:',/,
     +'REMARK 500 FORMAT:(10X,I3,1X,A3,1X,A1,I4,A1,4X,F7.2,',
     +      '3X,F7.2)',/,
     +'REMARK 500',/,
     +'REMARK 500 EXPECTED VALUES: GJ KLEYWEGT AND TA JONES ',
     +   '(1996). PHI/PSI-',/,
     +'REMARK 500 CHOLOGY: RAMACHANDRAN REVISITED. STRUCTURE ',
     +    '4, 1395 - 1400',/,
     +'REMARK 500',/,
     +'REMARK 500  M RES CSSEQI        PSI       PHI')
         IF (U31 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          END IF
         END IF
         write(Icif,
     +    '(''#'',/,''loop_'',/,
     +    ''_pdbx_validate_torsion.id'',/,
     +    ''_pdbx_validate_torsion.PDB_model_num'',/,
     +    ''_pdbx_validate_torsion.auth_comp_id'',/,
     +    ''_pdbx_validate_torsion.auth_asym_id'',/,
     +    ''_pdbx_validate_torsion.auth_seq_id'',/,
     +    ''_pdbx_validate_torsion.PDB_ins_code'',/,
     +    ''_pdbx_validate_torsion.phi'',/,
     +    ''_pdbx_validate_torsion.psi'')')
       END IF
       IF (Wmodel .gt. 0) THEN
        WRITE (Irout,fmt=6011)
     +          Wmodel,Wr1,Wc1,Ws1,Wi1,Value,Dvalue
 6011   FORMAT ('REMARK 500',i3,1X,a3,1X,a1,i4,a1,4X,f7.2,3X,f7.2)
       ELSE
        WRITE (Irout,fmt=6012)
     +          Wr1,Wc1,Ws1,Wi1,Value,Dvalue
 6012   FORMAT ('REMARK 500',4x,a3,1X,a1,i4,a1,4X,f7.2,3X,f7.2)
       END IF
       IF (Wc1 .eq. ' ') Wc1 = '.'
       IF (Wi1 .eq. ' ') Wi1 = '.'
       WRITE(Icif,
     +     fmt='(i4,2x,i4,2x,a3,2x,a1,2x,i5,2x,a1,4x,f7.2,2x,f7.2)')
     +          NbadRama,Wmodel,Wr1,Wc1,Ws1,Wi1,Dvalue,Value
      END IF
      GO TO 60
  70  IF (NbadRama.gt.0) WRITE (Irout,fmt=6005)
c
c
      REWIND Iscr
c-----------------------------------------------------------------------
c
c---- omega this pass
c
  100 READ (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))'
     +       ,end=110)
     +   Qswitch,Juse,Wmodel,Value,Dvalue,Klinker,
     +   Wr1,Wc1,Ws1,Wi1,Wn1,Wa1,
     +   Wr2,Wc2,Ws2,Wi2,Wn2,Wa2,
     +   Wr3,Wc3,Ws3,Wi3,Wn3,Wa3
      IF (Qswitch .eq. 'OME') THEN
       Komega = Komega + 1
       IF (Komega .eq. 1) THEN
         IF (U61 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          END IF
         END IF
         WRITE (Irout,fmt=6015)
 6015 format(
     +'REMARK 500',/,
     +'REMARK 500 GEOMETRY AND STEREOCHEMISTRY',/,
     +'REMARK 500 SUBTOPIC: NON-CIS, NON-TRANS',/,
     +'REMARK 500',/,
     +'REMARK 500 THE FOLLOWING PEPTIDE BONDS DEVIATE ',
     +      'SIGNIFICANTLY FROM BOTH',/,
     +'REMARK 500 CIS AND TRANS CONFORMATION.  ',
     +      'CIS BONDS, IF ANY, ARE LISTED',/,
     +'REMARK 500 ON CISPEP RECORDS.  ',
     +'TRANS IS DEFINED AS 180 +/- 30 AND',/,
     +'REMARK 500 CIS IS DEFINED AS 0 +/- 30 DEGREES.',/,
     +'REMARK 500                                 MODEL     OMEGA')
c
c
         IF (U31 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          END IF
         END IF
         WRITE(Icif,6016)
6016    FORMAT(
     +   '#',/,'loop_',/,
     +   '_pdbx_validate_peptide_omega.id',/,
     +   '_pdbx_validate_peptide_omega.PDB_model_num',/,
     +   '_pdbx_validate_peptide_omega.auth_comp_id_1',/,
     +   '_pdbx_validate_peptide_omega.auth_asym_id_1',/,
     +   '_pdbx_validate_peptide_omega.auth_seq_id_1',/,
     +   '_pdbx_validate_peptide_omega.PDB_ins_code_1',/,
     +   '_pdbx_validate_peptide_omega.label_alt_id_1',/,
     +   '_pdbx_validate_peptide_omega.auth_comp_id_2',/,
     +   '_pdbx_validate_peptide_omega.auth_asym_id_2',/,
     +   '_pdbx_validate_peptide_omega.auth_seq_id_2',/,
     +   '_pdbx_validate_peptide_omega.PDB_ins_code_2',/,
     +   '_pdbx_validate_peptide_omega.label_alt_id_2',/,
     +   '_pdbx_validate_peptide_omega.omega')
       END IF
        WRITE (Irout,fmt=6017) Wr2,Wc2,Ws2,Wi2,
     +                           Wr1,Wc1,Ws1,Wi1,
     +                           Wmodel,Value
 6017   FORMAT ('REMARK 500 ',a3,1X,a1,i5,a1,4X,
     +                        a3,1X,a1,i5,a1,4x,
     +                        i6,6x,f7.2)
         IF (Wc1 .eq. ' ') Wc1 = '.'
         IF (Wc2 .eq. ' ') Wc2 = '.'
         IF (Wi1 .eq. ' ') Wi1 = '.'
         IF (Wi2 .eq. ' ') Wi2 = '.'
         IF (Wa1 .eq. ' ') Wa1 = '.'
         IF (Wa2 .eq. ' ') Wa2 = '.'
         WRITE(Icif,
     + fmt='(1x,i3,2x,i3,1x,
     +       2(a3,1x,a1,1x,i4,1x,a1,1x,a1,2x),
     +        2x,f7.2)')
     +      Komega,Wmodel,
     +      Wr1,Wc1,Ws1,Wi1,Wa1,
     +      Wr2,Wc2,Ws2,Wi2,Wa2,
     +      Value
      END IF
      GO TO 100
  110 IF (Komega.gt.0) WRITE (Irout,fmt=6005)
c
c
      REWIND Iscr
c
c--------------------------------------------------------
c
c---- planes error here sidechain
c
  180 READ (Iscr,
     +fmt='(a3,2x,i6,2x,i4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x),
     +     3x,25(f12.3,2x))',end=190)
     +     Qswitch,Juse,Wmodel,Value,Dvalue,Klinker,
     +     Wr1,Wc1,Ws1,Wi1,Wn1,Wa1,
     +     Wr2,Wc2,Ws2,Wi2,Wn2,Wa2,
     +     Wr3,Wc3,Ws3,Wi3,Wn3,Wa3,
     +     (Prms(Jdo),Jdo=1,MaxPlanesAtoms)
ccx      IF (Qswitch .eq. 'PLN' .or. Qswitch .eq. 'MPL') THEN
      IF (Qswitch .eq. 'PLN' ) THEN
        IF (Dvalue .gt. PlaneCutoff .or.
     +      Klinker .ne. 0) THEN
         Nplane = Nplane + 1
c
c  flag for 1 atom outside PlaneCutoff is Klinker
c  plane number is Juse
c
       IF (Nplane .eq. 1) THEN
         IF (U61 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          END IF
         END IF
          WRITE (Irout,fmt=6027)
 6027 FORMAT ('REMARK 500',/,
     +'REMARK 500 GEOMETRY AND STEREOCHEMISTRY',/,
     +'REMARK 500 SUBTOPIC: PLANAR GROUPS',/,
     +'REMARK 500',/,
     +'REMARK 500 PLANAR GROUPS IN THE FOLLOWING RESIDUES ',
     +    'HAVE A TOTAL',/,
     +'REMARK 500 RMS DISTANCE OF ALL ATOMS FROM THE ',
     +    'BEST-FIT PLANE',/,
     +'REMARK 500 BY MORE THAN AN EXPECTED VALUE OF 6*RMSD,',
     +      ' WITH AN',/,
     +'REMARK 500 RMSD 0.02 ANGSTROMS, OR AT LEAST ONE ATOM HAS',/,
     +'REMARK 500 AN RMSD GREATER THAN THIS VALUE',/,
     +'REMARK 500 (M=MODEL NUMBER; RES=RESIDUE NAME; ',
     +   'C=CHAIN IDENTIFIER;',/,
     +'REMARK 500 SSEQ=SEQUENCE NUMBER; I=INSERTION CODE).',/,
     +'REMARK 500',/,
     +'REMARK 500  M RES CSSEQI        RMS     TYPE')
         IF (U31 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          END IF
         END IF
         write(Icif,fmt=6028)
6028  FORMAT(
     +  '#',/,'loop_',/,
     +  '_pdbx_validate_planes.id',/,
     +  '_pdbx_validate_planes.PDB_model_num',/,
     +  '_pdbx_validate_planes.auth_comp_id',/,
     +  '_pdbx_validate_planes.auth_asym_id',/,
     +  '_pdbx_validate_planes.auth_seq_id',/,
     +  '_pdbx_validate_planes.PDB_ins_code',/,
     +  '_pdbx_validate_planes.rmsd',/,
     +  '_pdbx_validate_planes.type')
       END IF
c
cREMARK 500  M RES CSSEQI        RMS     TYPE
cREMARK 500    PHE A  26         0.15    SIDE CHAIN
cREMARK 500    PHE A  26         0.15    MAIN CHAIN
c123456789012345678901234567890123456xxxx(A4,1x,f5.2)
c
        IF (Qswitch .eq. 'PLN') THEN
          Juse = 1
        ELSE IF (Qswitch .eq. 'MPL') THEN
          Juse = 2
        END IF
c
c
        IF (Wmodel .le.0) THEN
        WRITE (Irout,fmt=6229)
     +          Wr1,Wc1,Ws1,Wi1,Value,
     +          Pwork(Juse)
 6229 FORMAT ('REMARK 500',3x,1x,a3,1x,a1,i4,a1,5x,
     +                    f7.2,4x,a)
        else
        WRITE (Irout,fmt=6029)
     +          Wmodel,Wr1,Wc1,Ws1,Wi1,Value,
     +          Pwork(Juse)
 6029 FORMAT ('REMARK 500',i3,1x,a3,1x,a1,i4,a1,5x,
     +                    f7.2,4x,a)
        end if

c
c    deviations per atoms are stored in Prms corresponding
c    to atom names NumAtmStanPlane(Juse), PlaneNames(Juse,Kdo))
c
       IF (Wc1 .eq. ' ') Wc1 = '.'
       IF (Wi1 .eq. ' ') Wi1 = '.'
       WRITE(Icif,
     +     fmt='(i4,2x,i4,2x,a3,2x,a1,2x,i5,2x,a1,
     +           2x,f7.3,2x,a)')
     +        Nplane,Wmodel,Wr1,Wc1,Ws1,Wi1,Value,
     +        Pwork(Juse+2)
        END IF
      END IF
      GO TO 180
  190 IF (Nplane.gt.0) WRITE (Irout,fmt=6005)
      REWIND Iscr
c--------------------------------------------------------------
c
c---- planes error here main chain
c
c if using least squares for main chain skip this torsion
c
ccx      GO TO 215
c
c
 200  READ (Iscr,
     +fmt='(a3,2x,i6,2x,i4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x),
     +     3x,25(f12.3,2x))',end=210)
     +     Qswitch,Juse,Wmodel,Value,Dvalue,Klinker,
     +     Wr1,Wc1,Ws1,Wi1,Wn1,Wa1,
     +     Wr2,Wc2,Ws2,Wi2,Wn2,Wa2,
     +     Wr3,Wc3,Ws3,Wi3,Wn3,Wa3,
     +     (Prms(Jdo),Jdo=1,4)
      IF (Qswitch .eq. 'PLM') THEN
        IF (abs(Value) .gt. MainChainPlaneCutoff ) THEN
         Nmain = Nmain + 1
       IF (Nmain .eq. 1) THEN
         IF (U61 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          END IF
         END IF
          WRITE (Irout,fmt=6127)
 6127 FORMAT ('REMARK 500',/,
     +'REMARK 500 GEOMETRY AND STEREOCHEMISTRY',/,
     +'REMARK 500 SUBTOPIC: MAIN CHAIN PLANARITY',/,
     +'REMARK 500',/,
     +'REMARK 500 THE FOLLOWING RESIDUES HAVE A PSEUDO PLANARITY',/,
     +'REMARK 500 TORSION, C(I) - CA(I) - N(I+1) - O(I), GREATER',/,
     +'REMARK 500 10.0 DEGREES. (M=MODEL NUMBER; RES=RESIDUE NAME;',/,
     +'REMARK 500 C=CHAIN IDENTIFIER; SSEQ=SEQUENCE NUMBER; ',/,
     +'REMARK 500 I=INSERTION CODE).',/,
     +'REMARK 500',/,
     +'REMARK 500  M RES CSSEQI        ANGLE')
         IF (U31 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          END IF
         END IF
         write(Icif,fmt=6128)
6128  FORMAT(
     +  '#',/,'loop_',/,
     +  '_pdbx_validate_main_chain_plane.id',/,
     +  '_pdbx_validate_main_chain_plane.PDB_model_num',/,
     +  '_pdbx_validate_main_chain_plane.auth_comp_id',/,
     +  '_pdbx_validate_main_chain_plane.auth_asym_id',/,
     +  '_pdbx_validate_main_chain_plane.auth_seq_id',/,
     +  '_pdbx_validate_main_chain_plane.PDB_ins_code',/,
     +  '_pdbx_validate_main_chain_plane.improper_torsion_angle')
       END IF
        WRITE (Irout,fmt=6129) Wmodel,Wr1,Wc1,Ws1,Wi1,Value
 6129 FORMAT ('REMARK 500',i3,1x,a3,1x,a1,i4,a1,6x,f7.2)
       IF (Wc1 .eq. ' ') Wc1 = '.'
       IF (Wi1 .eq. ' ') Wi1 = '.'
       WRITE(Icif,
     +     fmt='(i4,2x,i4,2x,a3,2x,a1,2x,i5,2x,a1,
     +           3x,f7.3)')
     +        Nmain,Wmodel,Wr1,Wc1,Ws1,Wi1,Value
        END IF
      END IF
      GO TO 200
 210  IF (Nmain.gt.0) WRITE (Irout,fmt=6005)
c
c
 215  CONTINUE
c
c
      Nplane = 0
      REWIND Iscr
c
c---- planes atom error here side chain
c
  220 READ (Iscr,
     +fmt='(a3,2x,i6,2x,i4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x),
     +     3x,25(f12.3,2x))',end=230)
     +     Qswitch,Juse,Wmodel,Value,Dvalue,Klinker,
     +     Wr1,Wc1,Ws1,Wi1,Wn1,Wa1,
     +     Wr2,Wc2,Ws2,Wi2,Wn2,Wa2,
     +     Wr3,Wc3,Ws3,Wi3,Wn3,Wa3,
     +     (Prms(Jdo),Jdo=1,MaxPlanesAtoms)
ccx      IF (Qswitch .eq. 'PLN' .or. Qswitch .eq. 'MPL') THEN
c
c
      IF (Qswitch .eq. 'PLN') THEN
        IF (Dvalue .gt. PlaneCutoff .or.
     +      Klinker .ne. 0) THEN
         Nplane = Nplane + 1
c
c  flag for 1 atom outside PlaneCutoff is Klinker
c  plane number is Juse
c
       IF (Nplane .eq. 1) THEN
         IF (U31 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          END IF
         END IF
         write(Icif,fmt=6030)
 6030    FORMAT(
     +  '#',/,'loop_',/,
     +  '_pdbx_validate_planes_atom.plane_id',/,
     +  '_pdbx_validate_planes_atom.id',/,
     +  '_pdbx_validate_planes_atom.PDB_model_num',/,
     +  '_pdbx_validate_planes_atom.auth_asym_id',/,
     +  '_pdbx_validate_planes_atom.auth_comp_id',/,
     +  '_pdbx_validate_planes_atom.auth_seq_id',/,
     +  '_pdbx_validate_planes_atom.PDB_ins_code',/,
     +  '_pdbx_validate_planes_atom.auth_atom_id',/,
     +  '_pdbx_validate_planes_atom.atom_deviation')
       END IF
c
c    deviations per atoms are stored in Prms corresponding
c    to atom names NumAtmStanPlane(Juse), PlaneNames(Juse,Kdo))
c-999999.9D0
c
       IF (Wc1 .eq. ' ') Wc1 = '.'
       IF (Wi1 .eq. ' ') Wi1 = '.'
c
c
        IF (Qswitch .eq. 'PLN') THEN
         DO Jdo=1,MaxPlanesAtoms
          IF (Prms(Jdo) .gt. 0.000001d0) THEN
           WRITE(Icif,
     +     fmt='(i4,2x,i4,2x,i4,2x,a3,2x,a1,2x,i5,
     +          2x,a1,2x,a4,2x,f12.3)')
     +        Nplane,Jdo,Wmodel,Wr1,Wc1,Ws1,Wi1,
     +        PlaneNames(Juse,Jdo),Prms(Jdo)
          END IF
         ENDDO
        ELSE IF (Qswitch .eq. 'MPL') THEN
         DO Jdo=1,4
          IF (Prms(Jdo) .gt. 0.000001d0) THEN
           WRITE(Icif,
     +     fmt='(i4,2x,i4,2x,i4,2x,a3,2x,a1,2x,i5,
     +          2x,a1,2x,a4,2x,f12.3)')
     +        Nplane,Jdo,Wmodel,Wr1,Wc1,Ws1,Wi1,
     +        MainChain(Jdo),Prms(Jdo)
          END IF
         ENDDO
        END IF
c
c
       END IF
      END IF
      GO TO 220
 230  REWIND Iscr
c---------------------------------------------------------------
c
c---- cis this pass
c
  80  READ (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))'
     +       ,end=90)
     +   Qswitch,Juse,Wmodel,Value,Dvalue,Klinker,
     +   Wr1,Wc1,Ws1,Wi1,Wn1,Wa1,
     +   Wr2,Wc2,Ws2,Wi2,Wn2,Wa2,
     +   Wr3,Wc3,Ws3,Wi3,Wn3,Wa3
      IF (Qswitch .eq. 'CIS') THEN
       Kcis = Kcis + 1
       IF (Kcis .eq. 1) THEN
        IF (U71 .eq. 0) THEN
         Fname = 'cispep'
         OPEN (UNIT=Icis,FILE=Fname,STATUS='UNKNOWN')
         U71=1
        END IF
        IF (U31 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          END IF
        END IF
        WRITE(Icif,6013)
6013    FORMAT(
     +  '#',/,'loop_',/,
     +  '_struct_mon_prot_cis.pdbx_id',/,
     +  '_struct_mon_prot_cis.pdbx_PDB_model_num',/,
     +  '_struct_mon_prot_cis.label_comp_id',/,
     +  '_struct_mon_prot_cis.label_asym_id',/,
     +  '_struct_mon_prot_cis.label_seq_id',/,
     +  '_struct_mon_prot_cis.pdbx_PDB_ins_code',/,
     +  '_struct_mon_prot_cis.label_alt_id',/,
     +  '_struct_mon_prot_cis.pdbx_label_comp_id_2',/,
     +  '_struct_mon_prot_cis.pdbx_label_asym_id_2',/,
     +  '_struct_mon_prot_cis.pdbx_label_seq_id_2',/,
     +  '_struct_mon_prot_cis.pdbx_PDB_ins_code_2',/,
     +  '_struct_mon_prot_cis.label_alt_id_2',/,
     +  '_struct_mon_prot_cis.pdbx_omega_angle')
       END IF
c
c
c 1 -  6       Record name      "CISPEP"
c 8 - 10       Integer          serNum        Record serial number.
c12 - 14       LString(3)       pep1          Residue name.
c16            Character        chainID1      Chain identifier.
c18 - 21       Integer          seqNum1       Residue sequence number.
c22            AChar            icode1        Insertion code.
c26 - 28       LString(3)       pep2          Residue name.
c30            Character        chainID2      Chain identifier.
c32 - 35       Integer          seqNum2       Residue sequence number.
c36            AChar            icode2        Insertion code.
c44 - 46       Integer          modNum        Identifies the specific model.
c54 - 59       Real(6.2)        measure       Measure of the angle in degrees.
c
c
       WRITE (Icis,fmt=6014) Kcis, Wr2,Wc2,Ws2,Wi2,
     +                             Wr1,Wc1,Ws1,Wi1,
     +                             Wmodel,Value
 6014  FORMAT ('CISPEP',1X,i3,1X,a3,1X,a1,i5,a1,3X,
     +                           a3,1X,a1,i5,a1,7X,
     +                           i3,6X,f7.2)
         IF (Wc1 .eq. ' ') Wc1 = '.'
         IF (Wc2 .eq. ' ') Wc2 = '.'
         IF (Wi1 .eq. ' ') Wi1 = '.'
         IF (Wi2 .eq. ' ') Wi2 = '.'
         IF (Wa1 .eq. ' ') Wa1 = '.'
         IF (Wa2 .eq. ' ') Wa2 = '.'
         WRITE(Icif,
     + fmt='(1x,i3,2x,i3,1x,
     +       2(a3,1x,a1,1x,i4,1x,a1,1x,a1,2x),
     +        2x,f7.2)')
     +      Kcis,Wmodel,
     +      Wr1,Wc1,Ws1,Wi1,Wa1,
     +      Wr2,Wc2,Ws2,Wi2,Wa2,
     +      Value
      END IF
      GO TO 80
  90  REWIND Iscr
c
c---- CalphaChiral this pass
c
  120 READ (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))'
     +       ,end=130)
     +   Qswitch,Juse,Wmodel,Value,Dvalue,Klinker,
     +   Wr1,Wc1,Ws1,Wi1,Wn1,Wa1,
     +   Wr2,Wc2,Ws2,Wi2,Wn2,Wa2,
     +   Wr3,Wc3,Ws3,Wi3,Wn3,Wa3
      IF (Qswitch .eq. 'CHI') THEN
       KBadCalphaChiral = KBadCalphaChiral +1
       IF (KBadCalphaChiral .eq. 1) THEN
         IF (U61 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500'
            OPEN (UNIT=Irout,FILE=Fname,STATUS='UNKNOWN')
            U61=1
          END IF
         END IF
         WRITE (Irout,fmt=6018)
 6018  FORMAT (
     +'REMARK 500',/,
     +'REMARK 500 GEOMETRY AND STEREOCHEMISTRY',/,
     +'REMARK 500 SUBTOPIC: CHIRAL CENTERS',/,
     +'REMARK 500',/,
     +'REMARK 500 UNEXPECTED CONFIGURATI',
     +               'ON OF THE FOLLOWING CHIRAL',/,
     +'REMARK 500 CENTER(S) (M=MODEL NUMBER; ',
     +       'RES=RESIDUE NAME; C=CHAIN',/,
     + 'REMARK 500 IDENTIFIER; SSEQ=SEQUENCE ',
     +      'NUMBER; I=INSERTION CODE).',/,
     +'REMARK 500',/,
     +'REMARK 500 STANDARD TABLE:',/,
     +'REMARK 500 FORMAT: (10X,I3,1X,A3,1X,A1,I4,A1,6X,A12)',/,
     +'REMARK 500',/,
     +'REMARK 500  M RES CSSEQI                    OMEGA')
         IF (U31 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          END IF
         END IF
         WRITE(Icif,6019)
6019    FORMAT(
     +   '#',/,'loop_',/,
     +   '_pdbx_validate_chiral.id',/,
     +   '_pdbx_validate_chiral.PDB_model_num',/,
     +   '_pdbx_validate_chiral.auth_comp_id',/,
     +   '_pdbx_validate_chiral.auth_asym_id',/,
     +   '_pdbx_validate_chiral.auth_seq_id',/,
     +   '_pdbx_validate_chiral.PDB_ins_code',/,
     +   '_pdbx_validate_chiral.details',/,
     +   '_pdbx_validate_chiral.angle')
       END IF
cREMARK 500  M RES CSSEQI                    OMEGA
cREMARK 500  0 SER A 315       ALPHA-CARBON 154.98 A
      IF (Wmodel .le.0) THEN
      WRITE (Irout,fmt=6120) Wr1,Wc1,Ws1,Wi1,Value
 6120 FORMAT (
     +'REMARK 500',3x,1X,A3,1X,A1,I4,A1,'      ALPHA-CARBON ',
     +      f6.2,' ',a1)
      ELSE
      WRITE (Irout,fmt=6020) Wmodel,Wr1,Wc1,Ws1,Wi1,Value
 6020 FORMAT (
     +'REMARK 500',I3,1X,A3,1X,A1,I4,A1,'      ALPHA-CARBON ',
     +      f6.2,' ',a1)
        endif
         IF (Wc1 .eq. ' ') Wc1 = '.'
         IF (Wi1 .eq. ' ') Wi1 = '.'
         IF (Wa1 .eq. ' ') Wa1 = '.'
         WRITE(Icif,
     + fmt='(1x,i3,2x,i3,1x,a1,1x,a3,1x,i5,1x,a1,2x,a,2x,f7.2)')
     +      KBadCalphaChiral,Wmodel,
     +      Wc1,Wr1,Ws1,Wi1,'ALPHA-CARBON',
     +      Value
      END IF
      GO TO 120
  130 IF (KBadCalphaChiral.gt.0) WRITE (Irout,fmt=6005)
c
c
      REWIND Iscr
c
c---- Close any open files
c
      CLOSE (Ipdb)
      IF (Kcis .gt. 0) CLOSE (Icis)
c
cthis is to try with gfortran
c
      IF (.not. debug) THEN
        IF (Fpdb .eq. 0) THEN
        Fname = IDcode//'.scratch'
        Cwork = '/bin/rm -rf '//Fname(1:r500_lenstr(Fname))
        LLx=system(Cwork)
ccx        LLx=flush(6)
ccx        LLx=flush(0)
c
c this is ok in gnu f77
ccx       CLOSE (Iscr,DISPOSE='DELETE')
c
        ELSE
          Fname = Fwork(1:r500_lenstr(Fwork))//'.scratch'
          Cwork = '/bin/rm -rf '//Fname(1:r500_lenstr(Fname))
          LLx=system(Cwork)
        END IF
      ELSE
        CLOSE (Iscr)
      END IF
c
c
      IF (U61 .eq. 1) CLOSE (UNIT=Irout)
      IF (U31 .eq. 1) CLOSE(UNIT=Icif)
c
c
      RETURN
      END
c
c     =================================
      SUBROUTINE GetSymfr(line,I1,NS,Rrot)
c     =================================
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,M37,MaxA,MaxSize,MaxSym
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (M37            = 37)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxA           = 250000)
      PARAMETER (MaxSym         = 230)
      PARAMETER (MaxSize        = 100000000)
c
c
      INTEGER I1,NS
      CHARACTER line* (*)
      DOUBLE PRECISION Rrot(4,4,*)
      REAL A,RECIP,RRR,S,T
      INTEGER I,I11,ICOMST,IERR,Ifound,Imax,IP,ISL,J,K,NOP,NP,Nsym
      CHARACTER ICH*1
      INTEGER NUM(10)
      CHARACTER INUM(10)*1
      INTEGER r500_lenstr
      DATA NUM/1,2,3,4,5,6,7,8,9,0/
      DATA INUM/'1','2','3','4','5','6','7','8','9','0'/
c
      Imax = r500_lenstr(line)
      IERR = 0
c
c---- Search for first blank to skip flag sym symtr symmetry
c     or whatever
c
      I11 = I1
      I = I11 - 1
      NS = NS - 1
   10 CONTINUE
      NS = NS + 1
      RRR = 0.0
      RECIP = 0.0
      NOP = 1
c
      DO J = 1,4
        DO K = 1,4
          Rrot(J,K,NS) = 0.0D0
        ENDDO
      ENDDO
      Rrot(4,4,NS) = 1.0D0
   40 CONTINUE
c
      S = 1.0
c
c---- Set j=4 for translation vector
c
      J = 4
      T = 0.0
      IP = 0
      NP = 0
      ISL = 0
      Ifound = 0
      ICOMST = 0
   50 CONTINUE
      I = I + 1
c
      IF (I.le.Imax) THEN
        ICH = line(I:I)
        IF (ICH .eq. ' ') THEN
          GO TO 50
        ELSE IF (ICH.ne.',' .and. ICH.ne.'*') THEN
          Ifound = 1
          IF (ICH .eq. 'X' .or. ICH .eq. 'x' .or. ICH .eq. 'H' .or.
     +        ICH .eq. 'h') THEN
            IF (ICH .eq. 'X' .or. ICH .eq. 'x') RRR = RRR + 1.0
            IF (ICH .eq. 'H' .or. ICH .eq. 'h') RECIP = RECIP + 1.0
            J = 1
            IF (T .eq. 0.0) T = S
            GO TO 50
          ELSE IF (ICH .eq. 'Y' .or. ICH .eq. 'y' .or. ICH .eq. 'K' .or.
     +             ICH .eq. 'k') THEN
            IF (ICH .eq. 'Y' .or. ICH .eq. 'y') RRR = RRR + 1.0
            IF (ICH .eq. 'K' .or. ICH .eq. 'k') RECIP = RECIP + 1.0
            J = 2
            IF (T .eq. 0.0) T = S
            GO TO 50
          ELSE IF (ICH .eq. 'Z' .or. ICH .eq. 'z' .or. ICH .eq. 'L' .or.
     +             ICH .eq. 'l') THEN
            IF (ICH .eq. 'Z' .or. ICH .eq. 'z') RRR = RRR + 1.0
            IF (ICH .eq. 'L' .or. ICH .eq. 'l') RECIP = RECIP + 1.0
            J = 3
            IF (T .eq. 0.0) T = S
            GO TO 50
          ELSE IF (ICH .eq. '+') THEN
            S = 1.0
            IF (T .eq. 0.0 .and. J .eq. 4) THEN
              GO TO 50
            ELSE
              GO TO 80
            END IF
          ELSE IF (ICH .eq. '-') THEN
            S = -1.0
            IF (T .eq. 0.0 .and. J .eq. 4) THEN
              GO TO 50
            ELSE
              GO TO 80
            END IF
          ELSE IF (ICH .eq. '/') THEN
            ISL = 1
            GO TO 50
          ELSE IF (ICH .eq. '.') THEN
            IP = 1
            GO TO 50
          ELSE
            DO 60 K = 1,10
              IF (ICH .eq. INUM(K)) GO TO 70
   60       CONTINUE
            IERR = 1
            GO TO 50
   70       A = NUM(K)
            IF (ISL .eq. 1) THEN
              T = T/A
            ELSE IF (IP .eq. 1) THEN
              NP = NP + 1
              T = S*A/10**NP + T
            ELSE
              T = 10.0*T + A*S
            END IF
            GO TO 50
          END IF
        END IF
      END IF
      IF (T .eq. 0.0 .and. J .eq. 4) THEN
        GO TO 90
      ELSE
        ICOMST = 1
      END IF
   80 Rrot(NOP,J,NS) = dble(T)
      J = 4
      T = 0.0
      IP = 0
      NP = 0
      ISL = 0
c
      IF (ICOMST .eq. 0) GO TO 50
c
      IF (Ifound .eq. 0 .and. I.le.Imax) IERR = 1
c
      IF (I.le.Imax) THEN
        NOP = NOP + 1
        IF (NOP.le.3) THEN
          GO TO 40
        ELSE
          GO TO 10
        END IF
      ELSE
        GO TO 100
      END IF
   90 IERR = 1
      GO TO 120
  100 IF (NOP.ne.1 .or. Ifound.ne.0) THEN
        IF (NOP .eq. 3 .and. Ifound .eq. 1) THEN
          GO TO 110
        ELSE
          IERR = 1
        END IF
      END IF
      NS = NS - 1
  110 Nsym = NS
      IF (RRR.lt.3.0 .and. RECIP.lt.3.0) IERR = 1
      IF (RECIP.ge.3.0) Nsym = -Nsym
      IF (IERR.ne.1) RETURN
  120 STOP 'SYMMETRY OPERATOR ERROR'
c
c
      END
c
c
c     ==============================
      SUBROUTINE GetSymm(cell,Wspg)
c     ==============================
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,M37,MaxA,MaxSize,MaxSym
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (M37            = 37)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxA           = 250000)
      PARAMETER (MaxSym         = 230)
      PARAMETER (MaxSize        = 100000000)
c
c
      DOUBLE PRECISION cell(6)
      INTEGER Idnt,Sall,Msym
      DOUBLE PRECISION Rsym,S,TR,Sxyz
      INTEGER Ctran,Ias,Snum,Qsym
      CHARACTER Qalt*1,Qchain*1,Qins*1,Qname*4,Qres*3,Qele*2
      INTEGER Iflag,Jdo,L,LL,Mdo,Lspg,Nspg,Nsym,NsymP,Kspg
      CHARACTER PGname*10,SpgNam*11,Wspg*11
      CHARACTER N2O(3,2)*11,Rhomb(2,2)*11
      INTEGER r500_lenstr
c
c
      COMMON /W1/S(4,4,MaxSym),Rsym(4,4,MaxSym),Sxyz(3,2*MaxA)
      COMMON /W11/Snum(2*MaxA),Ias(2*MaxA),Qsym(2*MaxA),Sall,Msym,
     +       Idnt
      COMMON /W7/Ctran(2*MaxA)
      COMMON /W71/TR(4,4,MaxSym)
      COMMON /W9/Qins(MaxA),Qalt(MaxA),Qname(MaxA),Qres(MaxA),
     +           Qchain(MaxA),Qele(MaxA)
c
c
      DATA N2O(1,1),N2O(1,2)/'P 21       ','P 1 21 1   '/
      DATA N2O(2,1),N2O(2,2)/'P 2        ','P 1 2 1    '/
      DATA N2O(3,1),N2O(3,2)/'C 2        ','C 1 2 1    '/
      DATA Rhomb(1,1),Rhomb(1,2)/'R 3    ','H 3        '/
      DATA Rhomb(2,1),Rhomb(2,2)/'R 3 2  ','H 3 2      '/
c
c
c3 2 2 P2 PG2 MONOCLINIC 'P 1 2 1'
c X,Y,Z
c -X,Y,-Z
c3 2 2 P121 PG2 MONOCLINIC 'P 1 2 1'
c X,Y,Z
c -X,Y,-Z
c4 2 2 P21 PG2 MONOCLINIC 'P 1 21 1'
c X,Y,Z
c -X,Y+1/2,-Z
c4 2 2 P1211 PG2 MONOCLINIC 'P 1 21 1'
c X,Y,Z
c -X,Y+1/2,-Z
c1155 6 6 R32 PG32 TRIGONAL 'R 3 2'
cX,Y,Z
cZ,X,Y
cY,Z,X
c-Y,-X,-Z
c-Z,-Y,-X
c-X,-Z,-Y
c1146 3 3 R3 PG3 TRIGONAL 'R 3'
cX,Y,Z
cZ,X,Y
cY,Z,X
c
c---- defaults
c
      Idnt = 0
      Msym = 0
      Nsym = 0
      IF (Wspg(1:1).ne.' ') THEN
       DO Jdo = 1,3
        IF (Wspg .eq. N2O(Jdo,2)) Wspg = N2O(Jdo,1)
       ENDDO
       DO Jdo = 1,2
        IF (Wspg .eq. Rhomb(Jdo,1)) THEN
         IF (cell(4).lt.cell(5)+0.002D0 .and.
     +       cell(4).gt.cell(5)-0.002D0 .and.
     +       cell(4).lt.90.0D0+0.002D0 .and.
     +       cell(4).gt.90.0D0-0.002D0 .and.
     +       cell(6).lt.120.0D0+0.002D0 .and.
     +       cell(6).gt.120.0D0-0.002D0) THEN
          Wspg = Rhomb(Jdo,2)
          GO TO 50
         END IF
        END IF
        IF (Wspg .eq. Rhomb(Jdo,2)) THEN
         IF (cell(1).lt.cell(2)+0.002D0 .and.
     +       cell(1).gt.cell(2)-0.002D0 .and.
     +       cell(1).lt.cell(3)+0.002D0 .and.
     +       cell(1).gt.cell(3)-0.002D0) THEN
          Wspg = Rhomb(Jdo,1)
         END IF
         GO TO 50
        END IF
       ENDDO
   50   Lspg = r500_lenstr(Wspg)
        Nspg = 1
        SpgNam = ' '
        DO Jdo = 1,Lspg
         IF (Wspg(Jdo:Jdo).ne.' ') THEN
          SpgNam(Nspg:Nspg) = Wspg(Jdo:Jdo)
          Nspg = Nspg + 1
         END IF
        ENDDO
        Kspg = -99
        CALL GetSymop(Kspg,SpgNam,PGname,NsymP,Nsym,Rsym)
        IF (Kspg .eq. -99) STOP 'error in symmetry'
        IF (Nsym.gt.1) THEN
          Nsym = Nsym - 1
          DO Jdo = 2,Nsym + 1
            Mdo = Jdo - 1
            DO L = 1,4
              DO LL = 1,4
                TR(LL,L,Mdo) = Rsym(L,LL,Jdo)
              ENDDO
            ENDDO
            Iflag = -99
            CALL sident(TR(1,1,Mdo),Iflag)
            IF (Iflag .eq. 1) THEN
              IF (Idnt.ne.0) STOP 'identity error'
              Idnt = Mdo
            END IF
          ENDDO
          Msym = Nsym
        ELSE
          Msym = -1
          Idnt = 63
          GO TO 100
        END IF
        GO TO 100
      END IF
  100 CONTINUE
c
c
      IF (Msym .eq. 0) STOP 'no SYMMETRY '
c
c---- NumSymm is a flag
c
      IF (Msym .eq. 0) Msym = 1
      IF (Msym .eq. -1) Msym = 0
      IF (Idnt .eq. 0) Idnt = Msym + 1
      CALL TRsym(Msym,TR)
      RETURN
      END
c
c
c     ===============================================================
       SUBROUTINE GetPlaneCenter(Pxyz,Patoms,Centre)
c     ===============================================================
c
c  Calculate centre of mass of supplied data points
c
c
c  expect
cMaxStandard    37
cNaa            24
cNna            37
cNumBondLinker   4
cNumAngleLinker 11
cMaxAngleTarget 56
cMaxBondTarget  36
cMaxPlanesAtoms 12
cMaxBondLinker   4
cMaxAngleLinker 11
c
cclasses bonds  - 200  aa 1-73, na 100-194
cclasses angles - 250  aa 1-79, na 10-213
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
      INTEGER Patoms,Jdo
      DOUBLE PRECISION Centre(3),Pxyz(3,*),
     +                 Xsum,Ysum,Zsum
c
c
      INTRINSIC dble
c
c
c
      Xsum = 0.0d0
      Ysum = 0.0d0
      Zsum = 0.0d0
c
c---- Loop through the data points
c
      DO Jdo = 1,MaxPlanesAtoms
       IF (Pxyz(1,Jdo) .gt. MissingValue) THEN
        Xsum = Pxyz(1,Jdo) + Xsum
        Ysum = Pxyz(2,Jdo) + Ysum
        Zsum = Pxyz(3,Jdo) + Zsum
       END IF
      ENDDO
c
c---- Calculate centre of mass and adjust all coordinates accordingly
c
      Centre(1) = Xsum/dble(Patoms)
      Centre(2) = Ysum/dble(Patoms)
      Centre(3) = Zsum/dble(Patoms)
c
c---- shift atoms in plane to CG 0,0,0
c
      DO Jdo = 1,MaxPlanesAtoms
       IF (Pxyz(1,Jdo).gt. MissingValue) THEN
        Pxyz(1,Jdo) = Pxyz(1,Jdo) - Centre(1)
        Pxyz(2,Jdo) = Pxyz(2,Jdo) - Centre(2)
        Pxyz(3,Jdo) = Pxyz(3,Jdo) - Centre(3)
       END IF
      ENDDO
c
c
      RETURN
      END
c
c
c     ===============================================
      SUBROUTINE GetCons(Xmin,Ymin,Zmin,NX,NY,NZ,Boxes,Dlim,Neutron)
c     ===============================================
c
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,M37,MaxA,MaxSize,MaxSym
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (M37            = 37)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxA           = 250000)
      PARAMETER (MaxSym         = 230)
      PARAMETER (MaxSize        = 100000000)
c
c
      DOUBLE PRECISION Dlim,Xmin,Ymin,Zmin
      INTEGER NX,NY,NZ
      LOGICAL Neutron
      INTEGER Boxes(51,NX,NY,NZ)
      INTEGER Kall
      DOUBLE PRECISION Qxyz
      INTEGER Qnum,Qseq
      CHARACTER Qalt*1,Qchain*1,Qins*1,Qres*3,Qname*4,Qele*2
      DOUBLE PRECISION D,Dlim2,X,Y,Z,DH,DHclose,DD
      INTEGER Ide,IX,IY,IZ,Jdo,Katl,Iab,Kbtl,Kctl,Kta,Msa,
     +        Nab
      DOUBLE PRECISION Dx(3)
      REAL Qocc
c
c
      COMMON /W41/Qxyz(3,MaxA)
      COMMON /W4/Qnum(MaxA),Kall
      COMMON /W3/Qseq(MaxA)
      COMMON /W31/Qocc(MaxA)
      COMMON /W9/Qins(MaxA),Qalt(MaxA),Qname(MaxA),Qres(MaxA),
     +           Qchain(MaxA),Qele(MaxA)
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
      Dlim2   = Dlim*Dlim
      DH      = 1.6d0*1.6d0
      DD      = 1.44d0*1.44d0
      DHclose = 1.35d0*1.35d0
c
c---- neutron diffraction finds H and C and O at bette positions
c     i.e. O---H longer and O---H...O shorter
c      e.g.
c          O----H....O
c          1.07   1.42
c
c      C----O----D....OD2
c             1.03  1.49
c
c  The positions of Hydrogens determined by X-ray diffraction are
c  different from those determined from neutron diffraction studies.
c  In X-ray diffraction th H atom is apparently closer to the
c  heavier atom to which it is covalently bonded. Typically the
c  difference for anO--H distance is about 0.15A
c  (neutron 1.0A and X-ray 0.85A), the effect which is smaller for
c  heavier atoms, appears to arise from the asphericity of the electron
c  distribution due to chemical bonding. This affects the X-ray scattering
c  of the atom (which depens on the orbital electrons) but not the
c  neutron scattering, which, for a diamagnetic atom, is purely nuclear.
c
      IF (Neutron) THEN
        DH = 1.415D0*1.415D0
        DD = DH
      END IF
      DO 80 Msa = 1,Kall
       X = Qxyz(1,Msa) - Xmin
       Y = Qxyz(2,Msa) - Ymin
       Z = Qxyz(3,Msa) - Zmin
       IF (Xmin.lt.0.0D0) THEN
        IX = nint((X+6.1D0)/6.0D0) + 1
       ELSE
        IX = INT(X/6.0D0) + 2
       END IF
       IF (Ymin.lt.0.0D0) THEN
        IY = nint((Y+6.1D0)/6.0D0) + 1
       ELSE
        IY = INT(Y/6.0D0) + 2
       END IF
       IF (Zmin.lt.0.0D0) THEN
        IZ = nint((Z+6.1D0)/6.0D0) + 1
       ELSE
        IZ = INT(Z/6.0D0) + 2
       END IF
       DO 70 Katl = IX - 1,IX + 1
        IF (Katl.le.0 .or. Katl.gt.NX) GO TO 70
        DO 60 Kbtl = IY - 1,IY + 1
         IF (Kbtl.le.0 .or. Kbtl.gt.NY) GO TO 60
         DO 50 Kctl = IZ - 1,IZ + 1
          IF (Kctl.le.0 .or. Kctl.gt.NZ) GO TO 50
          Nab = Boxes(1,Katl,Kbtl,Kctl)
          IF (Nab.ne.0) THEN
           DO 40 Iab = 2,Nab + 1
            Kta = Boxes(Iab,Katl,Kbtl,Kctl)
            IF (Kta.ne.Msa) THEN
c
c---- if both water reject unless water numbers increasing
c
             IF ((Qres(Msa) .eq. 'HOH' .and.Qres(Kta) .eq. 'HOH') .or.
     +           (Qres(Msa) .eq. 'DOD' .and.Qres(Kta) .eq. 'DOD') .or.
     +           (Qres(Msa) .eq. 'HOH' .and.Qres(Kta) .eq. 'DOD') .or.
     +           (Qres(Msa) .eq. 'DOD' .and.Qres(Kta) .eq. 'HOH') ) THEN
              IF (Qseq(Msa).gt.Qseq(Kta)) GO TO 40
             END IF
c
c---- reject if di-sulphide
c
             IF (Qname(Msa) .eq. ' SG ' .and. Qname(Kta) .eq. ' SG ')
     +          GO TO 40
c
c---- if diff chains skipp next 2 tests
c
             IF (Qchain(Msa) .eq. Qchain(Kta)) THEN
c
c--- reject if same residue number
c
              IF (Qseq(Msa) .eq. Qseq(Kta)) GO TO 40
c
c---- reject if delta residue seq numbers less than 2
c
              Ide = abs(Qseq(Msa)-Qseq(Kta))
              IF (Ide.lt.2) GO TO 40
             END IF
c
c---- altcode
c
             IF (Qalt(Msa) .ne. ' ' .or. Qalt(Kta) .ne. ' ') THEN
              IF (Qalt(Msa) .ne. ' ' .and. Qalt(Kta) .ne. ' ')THEN
               IF (Qalt(Msa) .ne. Qalt(Kta)) GO TO 40
              END IF
c-why?      IF (abs(Qocc(Msa)-Qocc(Kta)) .gt. 0.5) GO TO 40
             END IF
             DO Jdo = 1,3
              Dx(Jdo) = abs(Qxyz(Jdo,Kta)-Qxyz(Jdo,Msa))
             ENDDO
             D = Dx(1)*Dx(1) + Dx(2)*Dx(2) + Dx(3)*Dx(3)
             IF (D.le.Dlim2) THEN
              IF (Qele(Msa) .eq. ' H' .or. Qele(Kta) .eq. ' H') THEN
               IF (D.gt. DH .or. D .lt. DHclose) GO TO 40
              END IF
              IF (Qele(Msa) .eq. ' D' .or. Qele(Kta) .eq. ' D') THEN
               IF (D.gt. DD .or. D .lt. DHclose) GO TO 40
              END IF
              D = sqrt(D)
c
c--- eliminate peptide bonds for numbering schemes not consectutive
c
              IF ((Qname(Msa) .eq. ' C  '.and.
     +             Qname(Kta) .eq. ' N  ') .or.
     +            (Qname(Msa) .eq. ' N  '.and.
     +             Qname(Kta) .eq. ' C  ')) THEN
               IF (D .le. 1.6d0) GO TO 40
              END  IF
c
          WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'CON',0,0,D,D,-1,
     +      Qres(Msa),Qchain(Msa),Qseq(Msa),Qins(Msa),
     +                                   Qname(Msa),Qalt(Msa),
     +      Qres(Kta),Qchain(Kta),Qseq(Kta),Qins(Kta),
     +                                   Qname(Kta),Qalt(Kta),
     +      '   ',' ',0,' ','    ',' '
             END IF
            END IF
   40      CONTINUE
          END IF
   50    CONTINUE
   60   CONTINUE
   70  CONTINUE
   80 CONTINUE
      RETURN
      END
c
c     ===============================================
      SUBROUTINE GetCons2(Xmin,Ymin,Zmin,NX,NY,NZ,Boxes,Dlim)
c     ===============================================
c
      implicit none
c
c
      INTEGER MaxA,MaxSym
      PARAMETER (MaxA=250000,MaxSym=230)
c
c
      DOUBLE PRECISION Dlim,Xmin,Ymin,Zmin
      INTEGER NX,NY,NZ
      INTEGER Boxes(51,NX,NY,NZ)
      INTEGER Idnt,Kall,Sall,Msym
      DOUBLE PRECISION Rsym,S,TR,Qxyz,Sxyz
      INTEGER Ctran,Ias,Qnum,Snum,Qseq,Qsym
      CHARACTER Qalt*1,Qchain*1,Qins*1,Qname*4,Qres*3,Qele*2
      DOUBLE PRECISION D,D2,X,Y,Z
      INTEGER Idel,Ipr,IX,IY,IZ,Jdo,Kab,Kat,Kbt,Kct,Kst,Ktan,Ktt,Msa,
     +        Nab,Nkm,Qtar,Wmodel
      DOUBLE PRECISION Dx(3)
      REAL Qocc
c
c
      COMMON /W1/S(4,4,MaxSym),Rsym(4,4,MaxSym),Sxyz(3,2*MaxA)
      COMMON /W11/Snum(2*MaxA),Ias(2*MaxA),Qsym(2*MaxA),Sall,Msym,
     +       Idnt
      COMMON /W3/Qseq(MaxA)
      COMMON /W4/Qnum(MaxA),Kall
      COMMON /W41/Qxyz(3,MaxA)
      COMMON /W7/Ctran(2*MaxA)
      COMMON /W71/TR(4,4,MaxSym)
      COMMON /W9/Qins(MaxA),Qalt(MaxA),Qname(MaxA),Qres(MaxA),
     +           Qchain(MaxA),Qele(MaxA)
c
c--- check use of qele and qocc
c
      COMMON /W31/Qocc(MaxA)
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
      D2 = Dlim*Dlim
      Wmodel = 0
c
c
      DO 100 Msa = 1,Kall
       Ipr = 1
       X = Qxyz(1,Msa) - Xmin
       Y = Qxyz(2,Msa) - Ymin
       Z = Qxyz(3,Msa) - Zmin
       IF (Xmin.lt.0.0D0) THEN
        IX = nint((X+6.1D0)/6.0D0) + 1
       ELSE
        IX = INT(X/6.0D0) + 2
       END IF
       IF (Ymin.lt.0.0) THEN
        IY = nint((Y+6.1D0)/6.0D0) + 1
       ELSE
        IY = INT(Y/6.0D0) + 2
       END IF
       IF (Zmin.lt.0.0D0) THEN
        IZ = nint((Z+6.1D0)/6.0D0) + 1
       ELSE
        IZ = INT(Z/6.0D0) + 2
       END IF
c
c
       DO 90 Kat = IX - 1,IX + 1
        IF (Kat.le.0 .or. Kat.gt.NX) GO TO 90
         DO 80 Kbt = IY - 1,IY + 1
          IF (Kbt.le.0 .or. Kbt.gt.NY) GO TO 80
          DO 70 Kct = IZ - 1,IZ + 1
           IF (Kct.le.0 .or. Kct.gt.NZ) GO TO 70
           Nab = Boxes(1,Kat,Kbt,Kct)
           IF (Nab.ne.0) THEN
            DO 60 Kab = 2,Nab + 1
             Ktan = Boxes(Kab,Kat,Kbt,Kct)
             Kst = Qsym(Ktan)
             Ktt = Ctran(Ktan)
             Qtar = Ias(Ktan)
             IF (Qname(Msa).ne.Qname(Qtar) .or. Kst.ne.Idnt) THEN
              IF (Qseq(Msa) .eq. Qseq(Qtar) .and. 
     +            Kst .eq. Idnt) GO TO 40
              IF (Kst .eq. Idnt) THEN
               Idel = abs(Qseq(Msa)-Qseq(Qtar))
               IF (Idel.lt.2) GO TO 40
              END IF
              DO 10 Jdo = 1,3
               Dx(Jdo) = abs(Sxyz(Jdo,Ktan)-Qxyz(Jdo,Msa))
   10         CONTINUE
              D = Dx(1)*Dx(1) + Dx(2)*Dx(2) + Dx(3)*Dx(3)
c
c
              IF (D.le.D2) THEN
               D = sqrt(D)
               Kst = Qsym(Ktan)
               Ktt = Ctran(Ktan)
c
c
               IF (Ipr .eq. 1) THEN
                Nkm = 1
          IF (Kst.le.Msym.and. Msym.gt.1) Nkm = Kst+1
          IF ((Qres(Msa) .eq. 'HOH' .and. Qres(Qtar) .eq. 'HOH') .or.
     +       (Qres(Msa) .eq. 'DOD' .and. Qres(Qtar) .eq. 'DOD') .or.
     +       (Qres(Msa) .eq. 'DOD' .and. Qres(Qtar) .eq. 'HOH') .or.
     +       (Qres(Msa) .eq. 'HOH' .and. Qres(Qtar) .eq. 'DOD'))THEN
           IF (Qseq(Msa).gt.Qseq(Qtar)) GO TO 20
          END IF
c-why?                IF (Msym .eq. 1) Ktt=Ktt-1
c
          WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'SYM',Ktt,Wmodel,D,D,Nkm,
     +      Qres(Msa),Qchain(Msa),Qseq(Msa),Qins(Msa),
     +                                   Qname(Msa),Qalt(Msa),
     +      Qres(Qtar),Qchain(Qtar),Qseq(Qtar),Qins(Qtar),
     +                                   Qname(Qtar),Qalt(Qtar),
     +      '   ',' ',0,' ','    ',' '
   20           CONTINUE
                Ipr = 0
               ELSE
                Nkm = 1
          IF (Kst.le.Msym.and. Msym.gt.1) Nkm = Kst+1
          IF ((Qres(Msa) .eq. 'HOH' .and. Qres(Qtar) .eq. 'HOH') .or.
     +       (Qres(Msa) .eq. 'DOD' .and. Qres(Qtar) .eq. 'DOD') .or.
     +       (Qres(Msa) .eq. 'DOD' .and. Qres(Qtar) .eq. 'HOH') .or.
     +       (Qres(Msa) .eq. 'HOH' .and. Qres(Qtar) .eq. 'DOD'))THEN
           IF (Qseq(Msa).gt.Qseq(Qtar)) GO TO 30
          END IF
c-why?                IF (Msym .eq. 1) Ktt=Ktt-1
c
          WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'SYM',Ktt,Wmodel,D,D,Nkm,
     +      Qres(Msa),Qchain(Msa),Qseq(Msa),Qins(Msa),
     +                                   Qname(Msa),Qalt(Msa),
     +      Qres(Qtar),Qchain(Qtar),Qseq(Qtar),Qins(Qtar),
     +                                   Qname(Qtar),Qalt(Qtar),
     +      '   ',' ',0,' ','    ',' '
   30           CONTINUE
               END IF
              END IF
             END IF
   40       CONTINUE
   60     CONTINUE
         END IF
   70    CONTINUE
   80   CONTINUE
   90  CONTINUE
  100 CONTINUE
c
c
      RETURN
      END
c
c
c     =========================================================
      SUBROUTINE GetSymop(LspgRP,NamSpg,PGname,NsymP,Nsym,ROT)
c     =========================================================
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,M37,MaxA,MaxSize,MaxSym,MaxT
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (M37            = 37)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxA           = 250000)
      PARAMETER (MaxSym         = 230)
      PARAMETER (MaxSize        = 100000000)
      PARAMETER (MaxT           = 200)
c
c
      INTEGER LspgRP,Nsym,NsymP
      CHARACTER PGname* (*),NamSpg* (*)
      DOUBLE PRECISION ROT(4,4,*)
      INTEGER I,ISG,IST,Nlin,Nlines,Ntok
      LOGICAL Hexists
      CHARACTER Fname*255,line*400
      REAL Fv(MaxT)
      INTEGER Ibeg(MaxT),Idec(MaxT),Iend(MaxT),Ityp(MaxT)
      CHARACTER Cv(MaxT)*4
c
c1 1 1 P1 PG1 TRICLINIC 'P 1'
c X,Y,Z
c2 2 2 P-1 PG1bar TRICLINIC 'P -1'
c X,Y,Z
c -X,-Y,-Z
c
c
      IST = 55
      CALL getenv('SYMOP',Fname)
      INQUIRE (FILE=Fname,EXIST=Hexists)
      IF (.NOT.Hexists) STOP ' no symmop set'
      OPEN (UNIT=IST,FILE=Fname,STATUS='OLD')
      Ntok = 0
      Nsym = 0
   10 CONTINUE
      READ (IST,fmt='(A)',ERR=60,end=60) line
      CALL r500_ccpupc(line)
      Ntok = -MaxT
      CALL r500_parse(line,Ibeg,Iend,Ityp,Fv,Cv,Idec,Ntok)
      IF (Ityp(1).ne.2 .or. Ityp(2).ne.2 .or.
     +    Ityp(3).ne.2) STOP 'error in SYMOP'
      ISG = nint(Fv(1))
      Nlin = nint(Fv(2))
      Nlines = nint(Fv(3))
      IF (LspgRP.gt.0) THEN
        IF (LspgRP .eq. ISG) GO TO 30
      ELSE IF (NamSpg .eq. line(Ibeg(4) :Iend(4))) THEN
        GO TO 30
      END IF
      DO 20 I = 1,Nlin
        READ (IST,fmt=*)
   20 CONTINUE
      GO TO 10
   30 CONTINUE
      LspgRP = ISG
      NamSpg = line(Ibeg(4) :Iend(4))
      PGname = line(Ibeg(5) :Iend(5))
cc      write(6,7777) NamSpg
 7777 format('[',a,']')
      DO 40 I = 1,Nlines
        READ (IST,fmt='(A)') line
        Nsym = Nsym + 1
        CALL GetSymfr(line,1,Nsym,ROT)
   40 CONTINUE
      NsymP = Nsym
      IF (Nlin.gt.Nlines) THEN
        DO 50 I = Nlines + 1,Nlin
          READ (IST,fmt='(A)') line
          Nsym = Nsym + 1
          CALL GetSymfr(line,1,Nsym,ROT)
   50   CONTINUE
      END IF
      CLOSE (UNIT=IST)
      RETURN
   60 CONTINUE
      LspgRP = -99
      END
c
c     ===============================
      SUBROUTINE GetMats(cell,Volume,RR)
c     ===============================
c
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,M37,MaxA,MaxSize,MaxSym
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (M37            = 37)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxA           = 250000)
      PARAMETER (MaxSym         = 230)
      PARAMETER (MaxSize        = 100000000)
c
c
      DOUBLE PRECISION Volume
      DOUBLE PRECISION cell(6),RR(3,3,6)
      DOUBLE PRECISION A,Alpha,B,Beta,C,conv,
     +                 cosA,cosAS,cosB,cosBS,cosG,cosGS,
     +                 Gamma,PI,
     +                 sinA,sinAS,sinB,sinBS,sinG,sinGS,
     +                 Sum,V
      INTEGER I,J,N,Ncode
c
c
      PI = Acos(-1.0D0)
      conv = PI/180.0D0
      Alpha = cell(4)*conv
      Beta = cell(5)*conv
      Gamma = cell(6)*conv
      Sum = (Alpha+Beta+Gamma)*0.5D0
      V = sqrt(sin(Sum-Alpha)*sin(Sum-Beta)*sin(Sum-Gamma)*sin(Sum))
      Volume = cell(1)*2.0D0*cell(2)*cell(3)*V
      sinA = sin(Alpha)
      cosA = cos(Alpha)
      sinB = sin(Beta)
      cosB = cos(Beta)
      sinG = sin(Gamma)
      cosG = cos(Gamma)
      cosBS = (cosA*cosG-cosB)/ (sinA*sinG)
      sinBS = sqrt(1.0D0-cosBS*cosBS)
      cosAS = (cosG*cosB-cosA)/ (sinB*sinG)
      sinAS = sqrt(1.0D0-cosAS*cosAS)
      cosGS = (cosA*cosB-cosG)/ (sinA*sinB)
      sinGS = sqrt(1.0D0-cosGS*cosGS)
      A = cell(1)
      B = cell(2)
      C = cell(3)
c
c---- Zero matrices
c
      DO N = 1,5
        DO I = 1,3
          DO J = 1,3
            RR(I,J,N) = 0.0D0
          ENDDO
        ENDDO
      ENDDO
c
c---- Calculate matrices
c
c---- XO along a  Zo along c*
c
      Ncode = 1
      RR(1,1,Ncode) = A
      RR(1,2,Ncode) = B*cosG
      RR(1,3,Ncode) = C*cosB
      RR(2,2,Ncode) = B*sinG
      RR(2,3,Ncode) = -C*sinB*cosAS
      RR(3,3,Ncode) = C*sinB*sinAS
c
c---- XO along b  Zo along a*
c
      Ncode = 2
      RR(3,1,Ncode) = A*sinG*sinBS
      RR(1,1,Ncode) = A*cosG
      RR(1,2,Ncode) = B
      RR(1,3,Ncode) = C*cosA
      RR(2,1,Ncode) = -A*sinG*cosBS
      RR(2,3,Ncode) = C*sinA
c
c---- XO along c  Zo along b*
c
      Ncode = 3
      RR(2,1,Ncode) = A*sinB
      RR(2,2,Ncode) = -B*sinA*cosGS
      RR(3,2,Ncode) = B*sinA*sinGS
      RR(1,1,Ncode) = A*cosB
      RR(1,2,Ncode) = B*cosA
      RR(1,3,Ncode) = C
c
c---- trigonal only - XO along a+b  YO alon a-b  Zo along c*
c
      Ncode = 4
      RR(3,3,Ncode) = C
      RR(1,1,Ncode) = A/2.0D0
      RR(1,2,Ncode) = A/2.0D0
      RR(2,1,Ncode) = -A*sinG
      RR(2,2,Ncode) = A*sinG
c
c---- XO along a*   ZO along c
c
      Ncode = 5
      RR(1,1,Ncode) = A*sinB*sinGS
      RR(3,1,Ncode) = A*cosB
      RR(3,2,Ncode) = B*cosA
      RR(3,3,Ncode) = C
      RR(2,1,Ncode) = -A*sinB*cosGS
      RR(2,2,Ncode) = B*sinA
c
c---- Grr*! to  Gerard Bricogne - his setting for P1 in SKEW.
c     XO along a  Yo along b*
c
      Ncode = 6
      RR(1,1,Ncode) = A
      RR(1,2,Ncode) = B*cosG
      RR(1,3,Ncode) = C*cosB
      RR(2,2,Ncode) = B*sinG*sinAS
      RR(3,2,Ncode) = -B*sinG*cosAS
      RR(3,3,Ncode) = C*sinB
c
c
c
      END
c
c
c       ================================================
        SUBROUTINE GetPlanes(Wcurrent,NumAtmStanPlane,Wseq,Wchain,
     +                       Wmodel,Wdes,Watoms,Wname,
     +                       PlaneNames,xyz,Walt,PlaneCutoff,
     +                       debug)
c       =================================================
c
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
c
      LOGICAL debug
      CHARACTER PlaneNames(MaxStandard,MaxPlanesAtoms)*4
      INTEGER   NumAtmStanPlane(MaxStandard)
      INTEGER Wseq,Wmodel,Jdo,Patom,Wcurrent,Kdo,Watoms,Ndo,
     +        Ovec(3),Juse,Klinker,Iplane,Ido
      DOUBLE PRECISION Pxyz(3,MaxPlanesAtoms),Dvalue,
     +                 SPxyz(3,MaxPlanesAtoms),
     +                 xyz(3,MaxAtoms),
     +                 Centre(3),Normal(3),PlaneCutoff,
     +                 rmsDIF,evec(3,3),Prms(MaxPlanesAtoms)
      CHARACTER Wres*3,Wchain*1,Wins*1,Wdes*10,Palt*1,
     +          Wname(MaxAtoms)*4,Walt(MaxAtoms)*1
      LOGICAL Qskip,Ifail
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
c---- DO planes
c
        Wres = Wdes(1:3)
        Wins = Wdes(10:10)
        DO Jdo = 1,MaxPlanesAtoms
          Pxyz(1,Jdo) = MissingValue
          Prms(Jdo)   = MissingValue
        ENDDO
        Patom  = 0
        IF (NumAtmStanPlane(Wcurrent) .gt. 0) THEN
         DO Kdo=1,NumAtmStanPlane(Wcurrent)
          DO Jdo=1,Watoms
           IF (Wname(Jdo) .eq. PlaneNames(Wcurrent,Kdo)) THEN
            IF (xyz(1,Jdo).gt. MissingValue) THEN
             Patom = Patom + 1
             IF (Patom .eq. 1) Palt=Walt(Jdo)
             IF (Patom .gt. 1) THEN
              Qskip = .false.
              IF (Walt(Jdo) .eq. Palt .or. Walt(Jdo) .eq. ' ')
     +         Qskip = .true.
              IF (.not. Qskip) GOTO 10
             END IF
             DO Ndo=1,3
              Pxyz(Ndo,Patom) = xyz(Ndo,Jdo)
             ENDDO
            END IF
           END IF
10         CONTINUE
          ENDDO
         ENDDO
        END IF
c
c
         IF (Patom .gt.3) THEN
          DO Jdo=1,3
           DO Ido=1,MaxPlanesAtoms
            SPxyz(Jdo,Ido) = Pxyz(Jdo,Ido)
           ENDDO
          ENDDO
c
c---- Calculate best-fit plane
c
          CALL Plane(Pxyz,Patom,Centre,Normal,
     +               Evec,Ovec,Ifail)
c
c
          IF (.not.Ifail) THEN
c
c---- Calculate RMS distance of all points from best-fit
c       plane
c
           CALL GetPlaneRMS(Pxyz,Patom,Normal,rmsDIF,Prms,Iplane,
     +                 PlaneCutoff)
c
c---- IF have all the planar atoms for this residue, then
c     calculate a best-fit plane
c
      IF (Patom .eq. NumAtmStanPlane(Wcurrent)) THEN
      Klinker = Iplane
      Juse = Wcurrent
      WRITE (Iscr,
     +fmt='(a3,2x,i6,2x,i4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x),
     +     3x,25(f12.3,2x))')
     +     'PLN',Juse,Wmodel,rmsDIF,rmsDIF,Klinker,
     +     Wres,Wchain,Wseq,Wins,'    ',' ',
     +     Wres,Wchain,Wseq,Wins,'    ',' ',
     +     Wres,Wchain,Wseq,Wins,'    ',' ',
     +     (Prms(Jdo),Jdo=1,MaxPlanesAtoms)
           END IF
c
c--- try alternative to Jacobi
c
          ELSE
c
c
      IF (debug) WRITE (6,fmt=6000) Wres,Wchain,Wseq,Wins
 6000 FORMAT (a3,2x,a1,2x,i5,2x,a1, 'Failed Jacobi2')
      CALL Praxin(SPxyz,Patom,Centre,Dvalue,
     +             Iplane,PlaneCutoff,rmsDIF,Prms,debug)
      Klinker = Iplane
      Juse = Wcurrent
      WRITE (Iscr,
     +fmt='(a3,2x,i6,2x,i4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x),
     +     3x,25(f12.3,2x))')
     +     'PLN',Juse,Wmodel,Dvalue,rmsDIF,Klinker,
     +     Wres,Wchain,Wseq,Wins,'    ',' ',
     +     Wres,Wchain,Wseq,Wins,'    ',' ',
     +     Wres,Wchain,Wseq,Wins,'    ',' ',
     +     (Prms(Jdo),Jdo=1,MaxPlanesAtoms)
          END IF
         END IF
c
c
         RETURN
         END
c
c
c     ==============================================================
      SUBROUTINE GetPuckerAngle(Nconnect,Wname,
     +                Wres,Wseq,Wchain,Walt,Wmodel,Wins,Wcurrent,
     +                xyz,Watom,PuckerDescription,
     +                PuckerAngle,RnaDna,U31,IDcode,Fpdb,Fwork)
c     =============================================================
c
c  calculates ribose pseudo angle
c  subroutine assumes ribose atoms are either ' or *
c
c  expect
cMaxStandard    37
cNaa            24
cNna            37
cNumBondLinker   4
cNumAngleLinker 11
cMaxAngleTarget 56
cMaxBondTarget  36
cMaxPlanesAtoms 12
cMaxBondLinker   4
cMaxAngleLinker 11
c
cclasses bonds  - 200  aa 1-73, na 100-194
cclasses angles - 250  aa 1-79, na 10-213
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
c
      CHARACTER Wname(MaxAtoms)*4,IDcode*4
      DOUBLE PRECISION PuckerAngle,xyz(3,MaxAtoms),A,Amp,Tangle,B,CP,
     +                 DG2RD,PHA,PIVALU,RD2DG,SP,UNDEFN,TAU(5)
      INTEGER Wmodel,U31,I,IC,IC1P,IC2P,IC3P,IC4P,Wcurrent,
     +        IN,IO4P,Jdo,NC,ITA0(4),ITA1(4),ITA2(4),
     +        ITA3(4),ITA4(4),Wseq,Watom,RnaDna,Fpdb
      CHARACTER Wres*4,Wchain*1,Walt*1,Nconnect*4,
     +          PuckerDescription*14,Wins*1,
     +          PP*1,SS*1,NAME*4,Fname*255,
     +          SUGT(10)*14,Fwork*255,Wtmp*1
      LOGICAL FIRSTNUC
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
      INTEGER r500_lenstr
c
c
      SAVE
c
c
      DATA PP/''''/
      DATA SS/'*'/
      DATA FIRSTNUC/.true./
      DATA SUGT/'C3''-endo','C4''-exo ','O1''-endo','C1''-exo ',
     +     'C2''-endo','(H)C3''-endo','C4''-endo','O1''-exo ',
     +     'C1''-endo','(H)C2''-endo'/
c
c---- conversion constants between degrees and radians
c
      UNDEFN = 0.123456d0
      PIVALU = ACOS(-1.0d0)
      RD2DG = 180.0d0/PIVALU
      DG2RD = PIVALU/180.0d0
c
c find first and last atm in Rname
c   send in NFound and savxyz()
c
c find atoms for pseudo angle calculation
c
      IC1P = 0
      IC2P = 0
      IC3P = 0
      IC4P = 0
      IO4P = 0
      NC = 0
c
c
      DO Jdo = 1,Watom
       IF (Wname(Jdo) .eq. ' C1''') THEN
        IC1P = Jdo
        NC = NC + 1
       ELSE IF (Wname(Jdo) .eq. ' C2''') THEN
        IC2P = Jdo
        NC = NC + 1
       ELSE IF (Wname(Jdo) .eq. ' C3''') THEN
        IC3P = Jdo
        NC = NC + 1
       ELSE IF (Wname(Jdo) .eq. ' C4''') THEN
        IC4P = Jdo
        NC = NC + 1
       ELSE IF (Wname(Jdo) .eq. ' O4''') THEN
        IO4P = Jdo
        NC = NC + 1
       END IF
      ENDDO
      IF (NC.ne.5) THEN
       IF (FIRSTNUC) THEN
        FIRSTNUC = .false.
        IF (U31 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          END IF
        END IF
        WRITE (Icif,fmt=6112)
 6112   FORMAT('#',/,
     +'# Sanger Principle of Nucleic Acid...',/,
     +'# ANTI 90.0 -> 180.0 & -180.0 -> -90.0',/,
     +'# SYN -90.0 ->   0.0 &    0.0 ->  90.0',/,
     +'# HIGH ANTI  -60.0 -> -90.0',/,
     +'# HIGH SYN   90.0 -> 60.0  ',/,'#',/)
        WRITE (Icif,fmt=6002)
 6002   FORMAT (
     +'loop_',/,
     +'_struct_mon_nucl.model_id',/,
     +'_struct_mon_nucl.auth_comp_id',/,
     +'_struct_mon_nucl.auth_seq_id',/,
     +'_struct_mon_nucl.auth_asym_id',/,
     +'_struct_mon_nucl.auth_insert_code',/,
     +'_struct_mon_nucl.tau0',/,
     +'_struct_mon_nucl.tau1',/,
     +'_struct_mon_nucl.tau2',/,
     +'_struct_mon_nucl.tau3',/,
     +'_struct_mon_nucl.tau4',/,
     +'_struct_mon_nucl.taum',/,
     +'_struct_mon_nucl.P                            ',/,
     +'_struct_mon_nucl.glycosyl_angle_value         ',/,
     +'_struct_mon_nucl.glycosyl_angle               ',/,
     +'_struct_mon_nucl.glycosyl_angle_conformation  ',/,
     +'_struct_mon_nucl.ribose_conformation          ')
       END IF
       Wtmp = Wins
       IF (Wins .eq. ' ') Wtmp = '.'
       WRITE (Icif,fmt=6018) Wmodel,Wres,Wseq,Wchain,Wtmp
 6018  FORMAT (i4,1x,a3,1x,i5,1x,a1,1x,a1,1x,11(' ? '))
       RETURN
      END IF
c
c_struct_mon_nucl.taum
c The maximum amplitide of puckering. It is derived from the
c the pseudorotation value, P, and the torsion angles in the ribose
c ring.
c  Tau2= Taum cosP
c  Tau3= Taum cos(P+144)
c  Tau4= Taum cos(P+288)
c  Tau0= Taum cos(P+ 72)
c  Tau1= Taum cos(P+216)
c
c
c
c_struct_mon_nucl.zeta
c The value in degrees of the backbone torsion angle zeta
c              c3'_o3'_p_o5'.
c
c
c
c tau0 :(c2'-c1'-o4'-c4')   C4'_O4'_C1'_C2'
c
      ITA0(1) = IC2P
      ITA0(2) = IC1P
      ITA0(3) = IO4P
      ITA0(4) = IC4P
c
c tau1 :(c3'-c2'-c1'-o4')   O4'_C1'_C2'_C3'
c
      ITA1(1) = IC3P
      ITA1(2) = IC2P
      ITA1(3) = IC1P
      ITA1(4) = IO4P
c
c tau2 :(c4'-c3'-c2'-c1')  C1'_C2'_C3'_C4'
c
      ITA2(1) = IC4P
      ITA2(2) = IC3P
      ITA2(3) = IC2P
      ITA2(4) = IC1P
c
c tau3 :(o4'-c4'-c3'-c2')   C2'_C3'_C4'_O4'
c
      ITA3(1) = IO4P
      ITA3(2) = IC4P
      ITA3(3) = IC3P
      ITA3(4) = IC2P
c
c tau4 :(c1'-o4'-c4'-c3')  C3'_C4'_O4'_C1'
c
      ITA4(1) = IC1P
      ITA4(2) = IO4P
      ITA4(3) = IC4P
      ITA4(4) = IC3P
c
c---- calculate angles
c  order  2   3   4  0  1
c
      CALL Torsion(ITA2(1),ITA2(2),ITA2(3),ITA2(4),xyz,TAU(1))
      CALL Torsion(ITA3(1),ITA3(2),ITA3(3),ITA3(4),xyz,TAU(2))
      CALL Torsion(ITA4(1),ITA4(2),ITA4(3),ITA4(4),xyz,TAU(3))
      CALL Torsion(ITA0(1),ITA0(2),ITA0(3),ITA0(4),xyz,TAU(4))
      CALL Torsion(ITA1(1),ITA1(2),ITA1(3),ITA1(4),xyz,TAU(5))
c
c--- open output cif file
c
      IF (FIRSTNUC) THEN
       FIRSTNUC = .false.
       IF (U31 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          END IF
       END IF
       WRITE (Icif,fmt=6112)
       WRITE (Icif,fmt=6002)
      END IF
c
c---- sugar pucker
c     calculation  methods taken from CUREVS
c
c__struct_mon_nucl.P
c  P is the phase angle of pseudorotation for five membered rings.
c  This formulation is used for ribo and deoxyribo sugars in nucleic
c   acids.
c                          (tau4 +tau1)-(tau3+tau0)
c               P = ATAN (-------------------------)
c                           2tau2 (sin 36+sin 72)
c
c               IF tau2 is <0, THEN P=P+180
c
c  This formulation is by Altona and Sundaralingam (1972), J.A.C.S.,
c   94, 8205-8212.
c
      A = 0.0d0
      B = 0.0d0
c
c
      DO I = 1,5
        A = COS(((I-1)*144.0d0)*DG2RD)*TAU(I) + A
        B = SIN(((I-1)*144.0d0)*DG2RD)*TAU(I) + B
      ENDDO
c
c
      A = A*2.0d0/5.0d0
      B = -B*2.0d0/5.0d0
      Amp = sqrt(A*A+B*B)
c
c
      IF (Amp.gt.0.0d0) THEN
        CP = A/Amp
        SP = B/Amp
        IF (ABS(CP).gt.1.0d0) CP = SIGN(1.0d0,CP)
        PHA = ACOS(CP)*RD2DG
        IF (SP.lt.0.0d0) PHA = 360.0d0 - PHA
      ELSE
        PHA = 0.0d0
      END IF
c
c
      PuckerDescription = SUGT(INT(PHA/36.0d0)+1)
c
c----  glycosyl angle
c
      IN = 0
      IC = 0
c
c
      DO Jdo = 1,Watom
          IF (Wname(Jdo) .eq. Nconnect) THEN
            IN = Jdo
          ELSE IF (Nconnect .eq. ' N9 ' .and.
     +             Wname(Jdo) .eq. ' C4 ') THEN
            IC = Jdo
          ELSE IF (Nconnect .eq. ' N1 ' .and.
     +             Wname(Jdo) .eq. ' C2 ') THEN
            IC = Jdo
          END IF
        IF (IN.ne.0 .and. IC.ne.0) GO TO 50
      ENDDO
   50 CONTINUE
c
c
      IF (IN .eq. 0 .or. IC .eq. 0) THEN
      IF (FIRSTNUC) THEN
        FIRSTNUC = .false.
        IF (U31 .eq. 0) THEN
          IF (Fpdb .eq. 0) THEN
            Fname = 'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          ELSE IF (Fpdb.eq. 1) THEN
            Fname = Fwork(1:r500_lenstr(Fwork))//'rem500.cif'
            OPEN (UNIT=Icif,FILE=Fname,STATUS='UNKNOWN')
            U31=1
            WRITE(Icif,'(a,/,a,/,a)') '#','data_'//IDcode,'#'
          END IF
        END IF
        WRITE (Icif,fmt=6112)
        WRITE (Icif,fmt=6002)
      END IF
c
c
      Wtmp = Wins
      IF (Wtmp .eq. ' ') Wtmp = '.'
      WRITE (Icif,fmt=6108) Wmodel,Wres,Wseq,Wchain,Wtmp,
     +       TAU(4),TAU(5),TAU(1),TAU(2),TAU(3),Amp,PHA,
     +       PuckerDescription(1:r500_lenstr(PuckerDescription))
 6108 FORMAT (i4,1x,a3,1x,i5,1x,a1,1x,a1,7(1x,f6.1),
     +        '  ?   ?   ? ','"',a,'"')
        RETURN
      END IF
c
c
c tau0 :(c2'-c1'-o4'-c4')   C4'_O4'_C1'_C2'
c tau1 :(c3'-c2'-c1'-o4')   O4'_C1'_C2'_C3'
c tau2 :(c4'-c3'-c2'-c1')   C1'_C2'_C3'_C4'
c tau3 :(o4'-c4'-c3'-c2')   C2'_C3'_C4'_O4'
c tau4 :(c1'-o4'-c4'-c3')   C3'_C4'_O4'_C1'
c
c
c ????? why is there tau and nuc
c_struct_mon_nucl.nu0   The value in degrees of the sugar torsion angle nu0
c                       c4'_o4'_c1'_c2'
c_struct_mon_nucl.tau0  The value in degrees of the sugar torsion angle tau0
c                       C4'_O4'_C1'_C2'
c
c_struct_mon_nucl.nu0  c4'_o4'_c1'_c2'
c_struct_mon_nucl.nu1  o4'_c1'_c2'_c3'
c_struct_mon_nucl.nu2  c1'_c2'_c3'_c4'
c_struct_mon_nucl.nu3  c2'_c3'_c4'_o4'
c_struct_mon_nucl.nu4  c3'_c4'_o4'_c1'
c_struct_mon_nucl.tau0    T0   TAU(4)
c_struct_mon_nucl.tau1    T1   TAU(5)
c_struct_mon_nucl.tau2    T2   TAU(1)
c_struct_mon_nucl.tau3    T3   TAU(2)
c_struct_mon_nucl.tau4    T4   TAU(3)
c
c
c Bond Angle (degrees)
c  C4* -  C3* -  C2* -  C1*   40.5   tau2
c  O4* -  C4* -  C3* -  C2*  -42.0
c  C2* -  C1* -  O4* -  C4*   27.3   tau0
c  C3* -  C2* -  C1* -  O4*   -1.4   tau1
c
c Amplitude=  43.7 Phase=  20.0  C3'-endo
c
c  Glycosyl Angle (degrees)
c  O4* -  C1* -  N1  -  C2  -164.5  ANTI
c
c
      CALL Torsion(IO4P,IC1P,IN,IC,xyz,Tangle)
c
c
      NAME = '?'
      IF (Tangle.ge.90.0d0 .and.
     +    Tangle.le.180.0d0) NAME = 'ANTI'
      IF (Tangle.ge.-180.0d0 .and.
     +    Tangle.le.-90.0d0) NAME = 'ANTI'
      IF (Tangle.ge.-90.0d0 .and.
     +    Tangle.le.0.0d0) NAME = 'SYN'
      IF (Tangle.ge.0.0d0 .and.
     +    Tangle.le.90.0d0) NAME = 'SYN'
      Wtmp=Wins
      IF (Wtmp .eq. ' ') Wtmp = '.'
      WRITE (Icif,fmt=6008) Wmodel,Wres,Wseq,Wchain,Wtmp,
     +       TAU(4),TAU(5),TAU(1),TAU(2),
     +       TAU(3),Amp,PHA,Tangle,
     +       Wname(IO4P),Wname(IC1P),
     +       Wname(IN),Wname(IC),
     +       NAME,
     +       PuckerDescription(1:r500_lenstr(PuckerDescription))
 6008 FORMAT (i4,1x,a3,1x,i5,1x,a1,1x,a1,8(1x,f6.1),1x,
     +       '"',a4,'-',a4,'-',a4,'-',a4,'"',1x,
     +       a,1x,'"',a,'"')
c
c
      RETURN
      END
c
c
c     ===============================================================
      SUBROUTINE GetPlaneRMS(Pxyz,Patoms,Normal,rmsDIF,
     +                       Prms,Iplane,Planecutoff)
c     ===============================================================
c
c  Calculate the RMS distance of the atoms from the
c       best-fit plane
c
c  expect
cMaxStandard    37
cNaa            24
cNna            37
cNumBondLinker   4
cNumAngleLinker 11
cMaxAngleTarget 56
cMaxBondTarget  36
cMaxPlanesAtoms 12
cMaxBondLinker   4
cMaxAngleLinker 11
c
cclasses bonds  - 200  aa 1-73, na 100-194
cclasses angles - 250  aa 1-79, na 10-213
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
      INTEGER Jdo,Patoms,Iplane
      DOUBLE PRECISION rmsDIF,Pxyz(3,*),
     +                 Normal(3),DOTPRD,VECTPQ(3),
     +                 Prms(MaxPlanesAtoms),PlaneCutoff
      INTRINSIC dble,sqrt
c
c
      rmsDIF = 0.0d0
      Iplane = 0
c
c---- Loop through all the data points
c
      DO Jdo = 1,MaxPlanesAtoms
        Prms(Jdo) = MissingValue
c
c---- Calculate distance of point from plane
c   Form vector between point and an arbitrary point in the plane
c       (here the CofG is used as the arbitrary point)
c
       IF (Pxyz(1,Jdo).gt. MissingValue) THEN
        VECTPQ(1) = Pxyz(1,Jdo)
        VECTPQ(2) = Pxyz(2,Jdo)
        VECTPQ(3) = Pxyz(3,Jdo)
c
c---- Calculate dot product PQ.n
c
        DOTPRD = VECTPQ(1)*Normal(1) + VECTPQ(2)*Normal(2) +
     +           VECTPQ(3)*Normal(3)
c
c---- Add distance squared to RMS-diff summation
c
        IF (abs(DOTPRD) .gt. PlaneCutoff) Iplane = Jdo
        Prms(Jdo) = abs(DOTPRD)
        rmsDIF = DOTPRD*DOTPRD + rmsDIF
       END IF
      ENDDO
c
c---- Calculate and print RMS difference
c
      rmsDIF = sqrt(rmsDIF/dble(Patoms))
c
c
      RETURN
      END
c
c
c
c     ===============================================================
      SUBROUTINE PLANE(Pxyz,Patoms,Centre,Normal,Evec,
     +                 Ovec,Ifail)
c     ===============================================================
c
c
c  SUBROUTINE PLANE  -  Calculate a best-fit plane through the supplied
c       data points
c
c
c---- Calculate centre of mass of points and adjust coords such that
c     CofM is at the origin
c
c  expect
cMaxStandard    37
cNaa            24
cNna            37
cNumBondLinker   4
cNumAngleLinker 11
cMaxAngleTarget 56
cMaxBondTarget  36
cMaxPlanesAtoms 12
cMaxBondLinker   4
cMaxAngleLinker 11
c
cclasses bonds  - 200  aa 1-73, na 100-194
cclasses angles - 250  aa 1-79, na 10-213
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
      INTEGER Patoms,Ovec(3),Nrot
      LOGICAL Ifail
      DOUBLE PRECISION Centre(3),Pxyz(3,*),
     +                 Evec(3,3),Normal(3),
     +                 D,Eval(3),Mat(3,3)
c
c
      Ifail = .false.
      CALL GetPlaneCenter(Pxyz,Patoms,Centre)
c
c---- Calculate scalar products matrix
c
      CALL GetScalarProduct(Pxyz,Patoms,Mat)
c
c---- Calculate eigen values and eigenvectors of matrix
c
      CALL Jacobi(Mat,3,3,Eval,Evec,Ifail,Nrot)
      IF (Ifail) THEN
       Ifail = .true.
       RETURN
      END IF
c
c---- Sort the eigen vectors into descending order of eigen value
c
      CALL EIGSRT(Eval,Ovec)
c
c---- Compute parameters of best-fit plane
c
      Normal(1) = Evec(1,Ovec(3))
      Normal(2) = Evec(2,Ovec(3))
      Normal(3) = Evec(3,Ovec(3))
      D = Normal(1)*Centre(1) +
     +    Normal(2)*Centre(2) +
     +    Normal(3)*Centre(3)
      RETURN
      END
c
c     =======================================================
       SUBROUTINE SetNet(NX,NY,NZ,Boxes,Dlim)
c     =======================================================
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,M37,MaxA,MaxSize,MaxSym
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (M37            = 37)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxA           = 250000)
      PARAMETER (MaxSym         = 230)
      PARAMETER (MaxSize        = 100000000)
c
c
      DOUBLE PRECISION Dlim
      INTEGER NX,NY,NZ
      INTEGER Boxes(51,NX,NY,NZ)
      DOUBLE PRECISION Xmax,Xmin,Ymax,Ymin,Zmax,Zmin
      INTEGER Kall
      DOUBLE PRECISION Qxyz
      REAL Qocc
      INTEGER Qnum,Qseq
      DOUBLE PRECISION X,Y,Z
      INTEGER I,IN,Imax,IX,IY,IZ,J,K
c
c
      COMMON /W0/Xmin,Xmax,Ymin,Ymax,Zmin,Zmax
      COMMON /W41/Qxyz(3,MaxA)
      COMMON /W4/Qnum(MaxA),Kall
      COMMON /W3/Qseq(MaxA)
      COMMON /W31/Qocc(MaxA)
c
c
      Imax = -1
      DO I = 1,NX
        DO J = 1,NY
          DO K = 1,NZ
            Boxes(1,I,J,K) = 0
          ENDDO
         ENDDO
        ENDDO
      DO I = 1,Kall
        X = Qxyz(1,I) - Xmin
        Y = Qxyz(2,I) - Ymin
        Z = Qxyz(3,I) - Zmin
        IF (Xmin.lt.0.0D0) THEN
          IX = nint((X+6.1D0)/6.0D0) + 1
        ELSE
          IX = INT(X/6.0D0) + 2
        END IF
        IF (Ymin.lt.0.0D0) THEN
          IY = nint((Y+6.1D0)/6.0D0) + 1
        ELSE
          IY = INT(Y/6.0D0) + 2
        END IF
        IF (Zmin.lt.0.0D0) THEN
          IZ = nint((Z+6.1D0)/6.0D0) + 1
        ELSE
          IZ = INT(Z/6.0D0) + 2
        END IF
        Boxes(1,IX,IY,IZ) = Boxes(1,IX,IY,IZ) + 1
        IN = Boxes(1,IX,IY,IZ)
        IF (Imax.lt.IN) Imax = IN
        IF (IN+1.gt.51) THEN
          Boxes(1,IX,IY,IZ) = Boxes(1,IX,IY,IZ) - 1
          Imax = Imax - 1
        ELSE
          Boxes(IN+1,IX,IY,IZ) = Qnum(I)
        END IF
      ENDDO
      RETURN
      END
c
c
c
c     =======================================================
      SUBROUTINE SetNet2(Xmin,Ymin,Zmin,NX,NY,NZ,NU,WXYZ,Mall,Boxes)
c     =======================================================
c
c
c
      implicit none
c
c
      DOUBLE PRECISION Xmin,Ymin,Zmin
      INTEGER Mall,NX,NY,NZ
      DOUBLE PRECISION WXYZ(3,*)
      INTEGER Boxes(51,NX,NY,NZ),NU(*)
      DOUBLE PRECISION X,Y,Z
      INTEGER I,IN,INmax,IX,IY,IZ,J,K
c
c
      INmax = -1
      DO I = 1,NX
        DO J = 1,NY
          DO K = 1,NZ
            Boxes(1,I,J,K) = 0
          ENDDO
        ENDDO
      ENDDO
c
c
      DO I = 1,Mall
        X = WXYZ(1,I) - Xmin
        Y = WXYZ(2,I) - Ymin
        Z = WXYZ(3,I) - Zmin
        IF (Xmin.lt.0.0D0) THEN
          IX = nint((X+6.1D0)/6.0D0) + 1
        ELSE
          IX = INT(X/6.0D0) + 2
        END IF
        IF (Ymin.lt.0.0D0) THEN
          IY = nint((Y+6.1D0)/6.0D0) + 1
        ELSE
          IY = INT(Y/6.0D0) + 2
        END IF
        IF (Zmin.lt.0.0D0) THEN
          IZ = nint((Z+6.1D0)/6.0D0) + 1
        ELSE
          IZ = INT(Z/6.0D0) + 2
        END IF
        Boxes(1,IX,IY,IZ) = Boxes(1,IX,IY,IZ) + 1
        IN = Boxes(1,IX,IY,IZ)
        IF (INmax.lt.IN) INmax = IN
        IF (IN+1.gt.51) THEN
          Boxes(1,IX,IY,IZ) = Boxes(1,IX,IY,IZ) - 1
          INmax = INmax - 1
        ELSE
          Boxes(IN+1,IX,IY,IZ) = NU(I)
        END IF
      ENDDO
      RETURN
      END
c
c
c     =================
      SUBROUTINE SymGen(Dlim)
c     =================
c
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,M37,MaxA,MaxSize,MaxSym
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (M37            = 37)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxA           = 250000)
      PARAMETER (MaxSym         = 230)
      PARAMETER (MaxSize        = 100000000)
c
c
      DOUBLE PRECISION Dlim
      INTEGER Idnt,Kall,Sall,Msym
      DOUBLE PRECISION Rsym,S,TR,Qxyz,Bxyz,Sxyz
      INTEGER Ctran,Ias,Qnum,Snum,Qsym
      CHARACTER Qalt*1,Qchain*1,Qins*1,Qname*4,Qres*3,Qele*2
      INTEGER I,J,K,L,LT
      REAL Qocc
      DOUBLE PRECISION Lxyz(2,3),X(3)
c
c
      COMMON /W0/Bxyz(2,3)
      COMMON /W1/S(4,4,MaxSym),Rsym(4,4,MaxSym),Sxyz(3,2*MaxA)
      COMMON /W11/Snum(2*MaxA),Ias(2*MaxA),Qsym(2*MaxA),Sall,Msym,
     +       Idnt
      COMMON /W4/Qnum(MaxA),Kall
      COMMON /W41/Qxyz(3,MaxA)
      COMMON /W7/Ctran(2*MaxA)
      COMMON /W71/TR(4,4,MaxSym)
      COMMON /W9/Qins(MaxA),Qalt(MaxA),Qname(MaxA),Qres(MaxA),
     +           Qchain(MaxA),Qele(MaxA)
      COMMON /W31/Qocc(MaxA)
c
c
      Sall = 0
      DO K = 1,3
        Lxyz(1,K) = Bxyz(1,K) - Dlim
        Lxyz(2,K) = Bxyz(2,K) + Dlim
      ENDDO
      DO 110 I = 1,Kall
        DO 40 LT = Msym + 1,Msym + 125
c
c---- skip identity
c
          IF (LT-Msym .eq. 63) GO TO 40
          DO 20 K = 1,3
            X(K) = Qxyz(K,I) + S(4,K,LT)
            IF (X(K).lt.Lxyz(1,K) .or. X(K).gt.Lxyz(2,K)) GO TO 40
   20     CONTINUE
          Sall = Sall + 1
          IF (Sall.gt.MaxA) STOP 'max atoms exceeded'
          Ias(Sall) = Qnum(I)
          Snum(Sall) = Sall
          DO L = 1,3
            Sxyz(L,Sall) = X(L)
          ENDDO
          Qsym(Sall) = Msym + 1
          Ctran(Sall) = LT - Msym
   40   CONTINUE
c
c
        IF (Msym .gt.0) THEN
        DO 100 J = 1,Msym
          IF (J .eq. Idnt) THEN
            DO K = 1,3
              X(K) = Qxyz(1,I)*S(1,K,J) + Qxyz(2,I)*S(2,K,J) +
     +               Qxyz(3,I)*S(3,K,J) + S(4,K,J)
            ENDDO
            Sall = Sall + 1
            IF (Sall.gt.MaxA) STOP 'max atoms exceeded'
            Ias(Sall) = Qnum(I)
            Snum(Sall) = Sall
            DO 60 L = 1,3
              Sxyz(L,Sall) = X(L)
   60       CONTINUE
            Qsym(Sall) = J
c
c---- lattice translation code
c
            Ctran(Sall) = 63
          ELSE
c
c----  other symmetries
c
            IF (Msym .eq. Idnt) GO TO 95
            DO 90 LT = Msym + 1,Msym + 125
              DO 70 K = 1,3
                X(K) = Qxyz(1,I)*S(1,K,J) + Qxyz(2,I)*S(2,K,J) +
     +                 Qxyz(3,I)*S(3,K,J) + S(4,K,J) + S(4,K,LT)
                IF (X(K).lt.Lxyz(1,K) .or.
     +              X(K).gt.Lxyz(2,K)) GO TO 90
   70         CONTINUE
              Sall = Sall + 1
              IF (Sall.gt.MaxA) STOP 'max atoms exceeded'
              Ias(Sall) = Qnum(I)
              Snum(Sall) = Sall
              DO L = 1,3
                Sxyz(L,Sall) = X(L)
              ENDDO
              Qsym(Sall) = J
c
c---- lattice translation code
c
              Ctran(Sall) = LT - Msym
   90       CONTINUE
          END IF
 95       CONTINUE
  100   CONTINUE
        end if
  110 CONTINUE
      RETURN
      END
c
c     =================
      SUBROUTINE SymMat(RO,RF)
c     =================
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,M37,MaxA,MaxSize,MaxSym
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (M37            = 37)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxA           = 250000)
      PARAMETER (MaxSym         = 230)
      PARAMETER (MaxSize        = 100000000)
c
c
      DOUBLE PRECISION RF(4,4),RO(4,4)
      INTEGER Idnt,Sall,Msym
      DOUBLE PRECISION Rsym,S,TR,Sxyz
      INTEGER Ctran,Ias,Snum,Qsym
      CHARACTER Qalt*1,Qchain*1,Qins*1,Qname*4,Qres*3,Qele*2
      INTEGER I,J,K
      DOUBLE PRECISION Umat(4,4),Wmat(4,4),Xmat(4,4),Ymat(4,4),Zmat(4,4)
      COMMON /W1/S(4,4,MaxSym),Rsym(4,4,MaxSym),Sxyz(3,2*MaxA)
      COMMON /W11/Snum(2*MaxA),Ias(2*MaxA),Qsym(2*MaxA),Sall,Msym,
     +       Idnt
      COMMON /W7/Ctran(2*MaxA)
      COMMON /W71/TR(4,4,MaxSym)
      COMMON /W9/Qins(MaxA),Qalt(MaxA),Qname(MaxA),Qres(MaxA),
     +       Qchain(MaxA),Qele(MaxA)
c
c
      DO I = 1,MaxSym
        S(4,4,I) = 1.0D0
        TR(4,4,I) = 1.0D0
      ENDDO
      Xmat(4,4) = 1.0D0
      DO I = 1,4
        DO J = 1,4
          Xmat(J,I) = RF(I,J)
          Ymat(J,I) = RO(I,J)
        ENDDO
      ENDDO
c
c----   matrix multiplications:  [S] = [Xmat] * [TR] * [Ymat]
c
      DO I = 1,Msym + 125
        DO J = 1,4
          DO K = 1,4
            Wmat(K,J) = TR(K,J,I)
          ENDDO
        ENDDO
        CALL matbymat(Zmat,Xmat,Wmat)
        CALL matbymat(Umat,Zmat,Ymat)
        DO  J = 1,4
          DO  K = 1,4
            S(K,J,I) = Umat(K,J)
          ENDDO
        ENDDO
      ENDDO
      RETURN
      END
c
c     ===============================================================
      SUBROUTINE GetScalarProduct(Pxyz,Patoms,Mat)
c     ===============================================================
c
c
c---- Calculate scalar products matrix from supplied coordinates
c
c
c  expect
cMaxStandard    37
cNaa            24
cNna            37
cNumBondLinker   4
cNumAngleLinker 11
cMaxAngleTarget 56
cMaxBondTarget  36
cMaxPlanesAtoms 12
cMaxBondLinker   4
cMaxAngleLinker 11
c
cclasses bonds  - 200  aa 1-73, na 100-194
cclasses angles - 250  aa 1-79, na 10-213
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
      INTEGER Patoms,Jdo
      DOUBLE PRECISION Pxyz(3,*),Mat(3,3),
     +                 XXsum,XYsum,XZsum,YYsum,YZsum,ZZsum
c
c
      XXsum = 0.0d0
      XYsum = 0.0d0
      XZsum = 0.0d0
      YYsum = 0.0d0
      YZsum = 0.0d0
      ZZsum = 0.0d0
c
c---- Loop through all the data points
c
      DO Jdo = 1,MaxPlanesAtoms
       IF (Pxyz(1,Jdo) .gt. MissingValue) THEN
        XXsum = Pxyz(1,Jdo)*Pxyz(1,Jdo) + XXsum
        XYsum = Pxyz(1,Jdo)*Pxyz(2,Jdo) + XYsum
        XZsum = Pxyz(1,Jdo)*Pxyz(3,Jdo) + XZsum
        YYsum = Pxyz(2,Jdo)*Pxyz(2,Jdo) + YYsum
        YZsum = Pxyz(2,Jdo)*Pxyz(3,Jdo) + YZsum
        ZZsum = Pxyz(3,Jdo)*Pxyz(3,Jdo) + ZZsum
       END IF
      ENDDO
c
c---- Form the scalar products matrix
c
      Mat(1,1) = XXsum
      Mat(1,2) = XYsum
      Mat(1,3) = XZsum
      Mat(2,1) = XYsum
      Mat(2,2) = YYsum
      Mat(2,3) = YZsum
      Mat(3,1) = XZsum
      Mat(3,2) = YZsum
      Mat(3,3) = ZZsum
      RETURN
      END
c
c
c     =================================================
      SUBROUTINE OrtMat(cell,Ncode,RO,RF)
c     =================================================
c
c
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,M37,MaxA,MaxSize,MaxSym
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (M37            = 37)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxA           = 250000)
      PARAMETER (MaxSym         = 230)
      PARAMETER (MaxSize        = 100000000)
c
c
      INTEGER Ncode
      DOUBLE PRECISION cell(6),RF(4,4),RO(4,4)
      DOUBLE PRECISION Volume
      INTEGER I,J,K
      DOUBLE PRECISION Rall(3,3,6)
c
c
      CALL GetMats(cell,Volume,Rall)
      DO J = 1,4
        DO I = 1,4
          RO(I,J) = 0.0D0
        ENDDO
      ENDDO
      IF (Ncode.le.6) THEN
        DO J = 1,3
          DO I = 1,3
            RO(I,J) = Rall(I,J,Ncode)
          ENDDO
        ENDDO
      ELSE IF (Ncode .eq. 7) THEN
c
c---- Ncode 7 for TNT, permute Ncode 2, rows of matrix are 3,1,2
c
        DO J = 1,3
          DO I = 1,3
            K = I - 1
            IF (K.le.0) K = K + 3
            RO(I,J) = Rall(K,J,2)
          ENDDO
        ENDDO
      ELSE
        WRITE (6,fmt=6000) Ncode
 6000   FORMAT (' *** Illegal Ncode: ',i3)
        STOP
      END IF
c
      RO(4,4) = 1.0D0
c
c---- Invert
c
      CALL InvMat(RO,RF)
c
      RETURN
      END
c
c
c     ====================================
      SUBROUTINE Torsion(IA,IB,IC,ID,xyz,Angle)
c     ====================================
c
c      Calculate torsion angle Angle defined by the 4 atoms
c      X1,X2,X3,X4.  Angle is in degrees.
c
c  expect
cMaxStandard    37
cNaa            24
cNna            37
cNumBondLinker   4
cNumAngleLinker 11
cMaxAngleTarget 56
cMaxBondTarget  36
cMaxPlanesAtoms 12
cMaxBondLinker   4
cMaxAngleLinker 11
c
cclasses bonds  - 200  aa 1-73, na 100-194
cclasses angles - 250  aa 1-79, na 10-213
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
c
c
      DOUBLE PRECISION Angle,xyz(3,*),RTOD,S,T,Wmag,
     +                 A(3),B(3),C(3),U(3),V(3),W(3),X1(3),X2(3),
     +                 X3(3),X4(3)
      INTEGER IA,IB,IC,ID
      DOUBLE PRECISION DOTDP,VLEN
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
      EXTERNAL DOTDP,VLEN
      EXTERNAL CROSSDP,VDIFDP
      INTRINSIC ATAN2
      DATA RTOD/57.29577951d0/
c
c
        Angle = MissingValue
c
c
       IF (IA .gt. 0 .and.
     +     IB .gt. 0 .and.
     +     IC .gt. 0 .and.
     +     ID .gt. 0) THEN
        IF (xyz(1,IA).gt. MissingValue .and.
     +      xyz(1,IB).gt. MissingValue .and.
     +      xyz(1,IC).gt. MissingValue .and.
     +      xyz(1,ID).gt. MissingValue) THEN
         X1(1) = xyz(1,IA)
         X1(2) = xyz(2,IA)
         X1(3) = xyz(3,IA)
         X2(1) = xyz(1,IB)
         X2(2) = xyz(2,IB)
         X2(3) = xyz(3,IB)
         X3(1) = xyz(1,IC)
         X3(2) = xyz(2,IC)
         X3(3) = xyz(3,IC)
         X4(1) = xyz(1,ID)
         X4(2) = xyz(2,ID)
         X4(3) = xyz(3,ID)
         CALL VDIFDP(U,X1,X2)
         CALL VDIFDP(V,X4,X3)
         CALL VDIFDP(W,X3,X2)
         CALL CROSSDP(A,U,W)
         CALL CROSSDP(B,V,W)
c
c---- Now find !W!
c
         Wmag = VLEN(W)
c
c---- (A.CROSSDP.B).DOTDP.W = !A!*!B!*!W!*SIN(Angle)
c
         CALL CROSSDP(C,A,B)
         S = DOTDP(C,W)
c
c---- (A.DOTDP.B)*!W! = !A!*!B!*!W!*SIN(Angle)
c
         T = DOTDP(A,B)*Wmag
         IF (S .eq. 0.0d0 .and. T .eq. 0.0d0) THEN
          Angle = 0.0d0
         ELSE
          Angle = ATAN2(S,T)*RTOD
         END IF
        END IF
       END IF
c
c
      RETURN
      END
c
c
c     ==========================
      SUBROUTINE TRsym(Msym,TR)
c     ==========================
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,M37,MaxA,MaxSize,MaxSym
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (M37            = 37)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxA           = 250000)
      PARAMETER (MaxSym         = 230)
      PARAMETER (MaxSize        = 100000000)
c
c
      INTEGER Msym
      DOUBLE PRECISION TR(4,4,MaxSym)
      INTEGER I,J,K,N
      DOUBLE PRECISION Tmat(4,4)
      DATA Tmat/1.0D0, 0.0D0, 0.0D0, 0.0D0,
     +          0.0D0, 1.0D0, 0.0D0, 0.0D0,
     +          0.0D0, 0.0D0, 1.0D0, 0.0D0,
     +          0.0D0, 0.0D0, 0.0D0, 1.0D0/
c
      N = Msym
      DO I = N + 1,N + 125
        DO J = 1,4
          DO K = 1,4
            TR(K,J,I) = Tmat(K,J)
          ENDDO
        ENDDO
      ENDDO
      DO I = 1,5
        DO J = 1,5
          DO K = 1,5
            N = N + 1
            IF (N.gt.MaxSym) STOP 'Maxsymmetry exceeded'
            TR(4,1,N) = DBLE(I-3)
            TR(4,2,N) = DBLE(J-3)
            TR(4,3,N) = DBLE(K-3)
          ENDDO
        ENDDO
      ENDDO
      END
c
c
c     =============================
      SUBROUTINE sident(Xmat,Iflag)
c     =============================
c
c
      implicit none
c
c
      INTEGER Iflag
      DOUBLE PRECISION Xmat(12)
      DOUBLE PRECISION Xsum
      INTEGER I
c
c
      Iflag = 0
      IF (Xmat(1) .eq. 1.0D0 .and. Xmat(6) .eq. 1.0D0 .and.
     +    Xmat(11) .eq. 1.0D0) THEN
        Xsum = 0.0D0
        DO I = 1,12
          Xsum = Xmat(I) + Xsum
        ENDDO
        IF (Xsum .eq. 3.0D0) Iflag = 1
      END IF
      END
c
c
c
c     =====================
      DOUBLE PRECISION FUNCTION VLEN(A)
c     =====================
c
c---- Length of vector A
c
c
      implicit none
c
c
      DOUBLE PRECISION A(3),V
c
c
      INTRINSIC sqrt
c
c
      V = A(1)**2 + A(2)**2 + A(3)**2
      IF (V .eq. 0.0d0) THEN
        VLEN = 0.0d0
      ELSE
        VLEN = sqrt(V)
      END IF
      END
c
c
c     =========================
      SUBROUTINE r500_ccpupc(String)
c     =========================
c
c
      implicit none
c
c
      CHARACTER String* (*)
      INTEGER K,L,LL
      CHARACTER LC*26,UC*26
      INTRINSIC index,len
      DATA LC/'abcdefghijklmnopqrstuvwxyz'/
      DATA UC/'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
c
c
      LL = len(String)
      IF (LL.gt.0) THEN
        DO L = 1,LL
          K = index(LC,String(L:L))
          IF (K. ne. 0) String(L:L) = UC(K:K)
        ENDDO
      END IF
      RETURN
      END
c
c
c     =========================
      SUBROUTINE matbymat(A,B,C)
c     =========================
c
      DOUBLE PRECISION A(4,4),B(4,4),C(4,4)
      DOUBLE PRECISION S
      INTEGER I,J,K
c
c
      DO I = 1,4
       DO J = 1,4
        S = 0.0D0
        DO K = 1,4
         S = B(I,K)*C(K,J) + S
        ENDDO
        A(I,J) = S
       ENDDO
      ENDDO
      RETURN
      END
c
c
c     =======================
      SUBROUTINE CROSSDP(A,B,C)
c     =======================
c
c
c
      implicit none
c
c
c
      DOUBLE PRECISION A(3),B(3),C(3)
c
c
      A(1)=B(2)*C(3)-C(2)*B(3)
      A(2)=B(3)*C(1)-C(3)*B(1)
      A(3)=B(1)*C(2)-C(1)*B(2)
      RETURN
      END
c
c     =======================
      SUBROUTINE InvMat(A,AI)
c     =======================
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,M37,MaxA,MaxSize,MaxSym
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (M37            = 37)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxA           = 250000)
      PARAMETER (MaxSym         = 230)
      PARAMETER (MaxSize        = 100000000)
c
c
      DOUBLE PRECISION A(4,4),AI(4,4)
      DOUBLE PRECISION AM,D
      INTEGER I,I1,II,J,J1,JJ
      DOUBLE PRECISION C(4,4),X(3,3)
c
c
      DO II = 1,4
        DO JJ = 1,4
          I = 0
          DO  I1 = 1,4
            IF (I1.ne.II) THEN
            I = I + 1
            J = 0
            DO J1 = 1,4
              IF (J1.ne.JJ) THEN
               J = J + 1
               X(I,J) = A(I1,J1)
              END IF
            ENDDO
           END IF
          ENDDO
          AM = X(1,1)*X(2,2)*X(3,3) - X(1,1)*X(2,3)*X(3,2) +
     +         X(1,2)*X(2,3)*X(3,1) - X(1,2)*X(2,1)*X(3,3) +
     +         X(1,3)*X(2,1)*X(3,2) - X(1,3)*X(2,2)*X(3,1)
          C(II,JJ) = (-1)** (II+JJ)*AM
         ENDDO
       ENDDO
c
c---- Calculate determinant
c
      D = 0.0D0
      DO I = 1,4
        D = A(I,1)*C(I,1) + D
      ENDDO
c
c---- Get inverse matrix
c
      DO I = 1,4
        DO J = 1,4
          AI(I,J) = C(J,I)/D
        ENDDO
      ENDDO
      RETURN
      END
c
c
c     =================
      DOUBLE PRECISION FUNCTION DOTDP(A,B)
c     ================
c
c---- dot product of two vectors
c
      implicit none
c
c
      DOUBLE PRECISION A(3),B(3)
c
c
      DOTDP=A(1)*B(1)+A(2)*B(2)+A(3)*B(3)
      RETURN
      END
c
c
c     ===============================================================
      SUBROUTINE Jacobi(Mat,N,NP,D,V,Ifail,Nrot)
c     ===============================================================
c
c
c--- computes all eigenvalues and vectors of a real
c     symmetric matrix a using the jacobi method (from
c     NUMERICAL RECIPES - PRESS ET AL.)
c
c
c     A = real symmetric matrix, which is of size n by n, stored in
c         a physical np by np array. on exit, elements of a above
c         the diagonal are destroyed.
c     N = on entry, order of matrix a for which computations are
c         done. unchanged on exit.
c     NP = physical array subscripts of matrix a. unchanged on exit.
c     D  = real array of physical dimension np which, on exit,
c          contains eigenvalues of a in its first n elements.
c     V = real matrix with the same logical and physical dimensions
c         as a whose columns contain, on exit, the normalized
c         eigenvectors of A.
c     Ifail = failure indicator. this is set to 0 for normal return, and
c       to 1 when more than 50 * n(n-1)/2 rotations are needed.
c     Nrot  = counter which, on exit, contains the number of rotations.
c
c     DURING COMPUTATION:
c
c     Z    = vector which accumulates terms of form TA(PQ)
c     B    = vector which contains current eigenvalues of A
c
c     OPERATION COUNT:
c
c     TYPICAL MATRICES REQUIRE 6 TO 10 SWEEPS OF N*(N-1)/2 Jacobi
c     ROTATIONS TO ACHIEVE CONVERGENCE, OR 3*N**2 TO 5*N**2
c     ROTATIONS. EACH ROTATION REQUIRES OF ORDER 6*N OPERATIONS, SO
c     THE TOTAL LABOR IS OF THE ORDER 18*N**3 TO 30*N**3.
c
c
c
c
      implicit none
c
c
      DOUBLE PRECISION EMIN
      INTEGER NMAX
      PARAMETER (EMIN=1.0E-8)
      PARAMETER (NMAX=100)
c
c
      INTEGER N,NP,Nrot,I,IP,IQ,J
      LOGICAL Ifail
      DOUBLE PRECISION Mat(NP,NP),D(NP),V(NP,NP),
     +                 C,G,H,S,SM,T,TAU,THETA,TRESH,
     +                 B(NMAX),Z(NMAX)
c
c
      INTRINSIC ABS,sqrt
c
c
      Ifail = .false.
c
c----   jacobi method not most efficient method for matrices
c       of order greater than 10 ; maximum allowed preset by nmax
c
      IF (N.gt.NMAX) THEN
        WRITE (6,fmt=6000)
 6000   FORMAT ('ERROR  MAXIMUM NUMBER OF ELEMENTS EXCEEDED IN SUBROUT',
     +         'INE Jacobi; CHANGE NMAX - RUN TERMINATED')
        STOP
      END IF
c
c----   initialise v to the identity matrix
c
      DO IP = 1,N
        DO IQ = 1,N
          V(IP,IQ) = 0.0d0
        ENDDO
        V(IP,IP) = 1.0d0
      ENDDO
c
c----   initialise b and d to the diagonal of a and z to 0.0
c
      DO IP = 1,N
        B(IP) = Mat(IP,IP)
        D(IP) = B(IP)
        Z(IP) = 0.0d0
      ENDDO
c
c----   start of sweeps
c
      Nrot = 0
c
c
      DO I = 1,50
c
c---- sum off-diagonal elements of matrix a
c
        SM = 0.0d0
        DO IP = 1,N - 1
          DO IQ = IP + 1,N
            SM = ABS(Mat(IP,IQ)) + SM
          ENDDO
        ENDDO
c
c---- normal return on quadratic convergence to machine underflow
c
        IF (abs(SM) .lt. 0.000001d0 ) RETURN
c
c---- set treshold on the first three sweeps
c
        IF (I.lt.4) THEN
          TRESH = 0.2d0*SM/N**2
        ELSE
c
c---- thereafter
c
          TRESH = 0.0d0
        END IF
c
c---- perform pq rotations
c
        DO IP = 1,N - 1
          DO IQ = IP + 1,N
            G = ABS(Mat(IP,IQ))*100.0d0
c
c---- after 4 sweeps,skip rotation IF the off-diagonal element is small
c
            IF ((I.gt.4) .and. (ABS(D(IP))+G .eq. ABS(D(IP))) .and.
     +          (ABS(D(IQ))+G .eq. ABS(D(IQ)))) THEN
              Mat(IP,IQ) = 0.0d0
c
c---- otherwise rotate dependent on the treshold
c
            ELSE IF (ABS(Mat(IP,IQ)).gt.TRESH) THEN
              H = D(IQ) - D(IP)
              IF (ABS(H).lt.EMIN) H = 0.0d0
              IF (ABS(H)+G .eq. ABS(H)) THEN
c
c   T=1/(2THETA)
c
                T = Mat(IP,IQ)/H
              ELSE
c
c   T=SIGN(THETA) / (ABS(THETA)+ sqrt(THETA**2 + 1))
c
                THETA = 0.5d0*H/Mat(IP,IQ)
                T = 1.0d0/ (sqrt(THETA**2+1.0)+ABS(THETA))
                IF (THETA.lt.0.0) T = -T
              END IF
              C = 1.0d0/sqrt(T**2+1)
              IF (ABS(C).lt.EMIN) C = 0.0d0
              S = T*C
              IF (ABS(S).lt.EMIN) S = 0.0d0
              TAU = S/ (1.0d0+C)
              IF (ABS(TAU).lt.EMIN) TAU = 0.0d0
              H = Mat(IP,IQ)*T
              IF (ABS(H).lt.EMIN) H = 0.0d0
              Z(IP) = Z(IP) - H
              IF (ABS(Z(IP)).lt.EMIN) Z(IP) = 0.0d0
              Z(IQ) = Z(IQ) + H
              IF (ABS(Z(IQ)).lt.EMIN) Z(IQ) = 0.0d0
              D(IP) = D(IP) - H
              IF (ABS(D(IP)).lt.EMIN) D(IP) = 0.0d0
              D(IQ) = D(IQ) + H
              IF (ABS(D(IQ)).lt.EMIN) D(IQ) = 0.0d0
              Mat(IP,IQ) = 0.0d0
c
c----   CASE OF ROTATIONS 1.le.J.lt.P
c
              DO J = 1,IP - 1
                G = Mat(J,IP)
                IF (ABS(G).lt.EMIN) G = 0.0d0
                H = Mat(J,IQ)
                IF (ABS(H).lt.EMIN) H = 0.0d0
                Mat(J,IP) = G - (G*TAU+H)*S
                IF (ABS(Mat(J,IP)).lt.EMIN) Mat(J,IP) = 0.0d0
                Mat(J,IQ) = (G-H*TAU)*S + H
                IF (ABS(Mat(J,IQ)).lt.EMIN) Mat(J,IQ) = 0.0d0
              ENDDO
c
c----   CASE OF ROTATIONS P.lt.J.lt.Q
c
              DO J = IP + 1,IQ - 1
                G = Mat(IP,J)
                H = Mat(J,IQ)
                Mat(IP,J) = G - (G*TAU+H)*S
                IF (ABS(Mat(IP,J)).lt.EMIN) Mat(IP,J) = 0.0d0
                Mat(J,IQ) = (G-H*TAU)*S + H
                IF (ABS(Mat(J,IQ)).lt.EMIN) Mat(J,IQ) = 0.0d0
              ENDDO
c
c----   CASE OF ROTATIONS Q.lt.J.le.N
c
              DO J = IQ + 1,N
                G = Mat(IP,J)
                H = Mat(IQ,J)
                Mat(IP,J) = G - (G*TAU+H)*S
                IF (ABS(Mat(IP,J)).lt.EMIN) Mat(IP,J) = 0.0d0
                Mat(IQ,J) = (G-H*TAU)*S + H
                IF (ABS(Mat(IQ,J)).lt.EMIN) Mat(IQ,J) = 0.0d0
              ENDDO
c
c
              DO J = 1,N
                G = V(J,IP)
                H = V(J,IQ)
                V(J,IP) = G - (G*TAU+H)*S
                IF (ABS(V(J,IP)).lt.EMIN) V(J,IP) = 0.0d0
                V(J,IQ) = (G-H*TAU)*S + H
                IF (ABS(V(J,IQ)).lt.EMIN) V(J,IQ) = 0.0d0
              ENDDO
c
c
              Nrot = Nrot + 1
            END IF
         ENDDO
        ENDDO
c
c---- end of pq rotations
c   update d with the sum of ta(pq) and reinitialize z to 0.0
c
        DO  IP = 1,N
          B(IP) = B(IP) + Z(IP)
          D(IP) = B(IP)
          Z(IP) = 0.0d0
       ENDDO
      ENDDO
c
c----   end of sweeps
c   abnormal return; 50 sweeps in jacobi should never happen
c
      Ifail = .true.
      RETURN
      END
c
c     ===============================================================
      SUBROUTINE EIGSRT(Eval,Vorder)
c     ===============================================================
c
c
c---- Sort the eigen vectors into descending order
c     of eigen value
c
c
      implicit none
c
c
      DOUBLE PRECISION Eval(3)
      INTEGER Vorder(3),I,Iswap,J,K
c
c
c---- Bubble sort the eigenvalues
c
      Vorder(1) = 1
      Vorder(2) = 2
      Vorder(3) = 3
c
c---- Repeate the sort twice
c
      DO K = 1,2
        DO J = 1,2
          DO I = J + 1,3
            IF (Eval(Vorder(I)).gt.Eval(Vorder(J))) THEN
              Iswap = Vorder(I)
              Vorder(I) = Vorder(J)
              Vorder(J) = Iswap
            END IF
          ENDDO
        ENDDO
      ENDDO
c
      RETURN
      END
c
c     ===============================
      INTEGER FUNCTION lefttrim(string)
c     ===============================
c
c---- Returns first non space char in a string
c
c
      CHARACTER string* (*)
      INTEGER Jdo
      INTRINSIC len
c
c
      lefttrim = 1
c
c
   10 CONTINUE
      IF (len(string) .ne. 0) THEN
        DO Jdo = 1,len(string)
          IF (string(Jdo:Jdo) .ne. ' ') THEN
            lefttrim = Jdo
            RETURN
          END IF
        ENDDO
      END IF
c
c
      RETURN
      END
c
c
c
c     ==========================================================
      SUBROUTINE r500_parse(line,Ibeg,Iend,Ityp,Fv,Cv,Idec,N)
c     ==========================================================
c
      implicit none
c
c
      INTEGER MaxDlm
      PARAMETER (MaxDlm=20)
c
c
      INTEGER N
      CHARACTER line* (*)
      REAL Fv(*)
      INTEGER Ibeg(*),Idec(*),Iend(*),Ityp(*)
      CHARACTER Cv(*)*4
      LOGICAL SKIP
      REAL F10,SIGN,SIGN0,VALUE,VALUE0
      INTEGER I,IdoT,INTLEN,J,L,LENG,LINLEN,NCHK,NDDELM,NDELM,NDIGS,
     +        NDONE,NDSDLM,NITEM,NPLACE,NSPDLM,OPER
      LOGICAL COMMNT,NULL,NUMBER,OPRATR,QUOTE,TOKEN,TQUOTE
      CHARACTER BLANK*1,DBLQT*1,ICOMM1*1,ICOMM2*1,LETQT*1,OLDQUT*1,
     +          TAB*1
      INTEGER ISGN(2)
      CHARACTER DDELIM(MaxDlm)*1,DELIM(MaxDlm)*1,DIGS(18)*1
      INTEGER r500_lenstr
      EXTERNAL r500_lenstr
      INTRINSIC CHAR
      SAVE DELIM,NDELM,NSPDLM,DDELIM,NDDELM,NDSDLM
c Delimiters
c---- DELIM  array of NDELM delimiters
c---- DDELIM  default array of NDDELM delimiters
c---- NSPDLM (NDSDLM default) is number of special delimiters which
c            cannot delimit a null field these are
c            at the beginning of the delimiter array
c
c-- Note that delimiters may be changed by a call to PARSDL (entry point)
c
      DATA LETQT,DBLQT/'''','"'/,BLANK/' '/,ICOMM1,ICOMM2/'#','!'/
      DATA DIGS/'0','1','2','3','4','5','6','7','8','9','+','-','*','/',
     +     'E','.','e',' '/
      DATA ISGN/1,-1/
      DATA NDIGS/17/
      DATA DDELIM/' ',' ','=',',',16*' '/
      DATA NDDELM/4/,NDSDLM/3/,NDELM/-1/
c
c---- Setup delimiters if not done
c
      IF (NDELM .lt. 0) THEN
        NDELM = NDDELM
        NSPDLM = NDSDLM
        DO I = 1,MaxDlm
          DELIM(I) = DDELIM(I)
        ENDDO
c
c---- Set tab (assumes ASCII)
c
        TAB = CHAR(9)
        DELIM(2) = TAB
      END IF
c
c---- First call  - N = - NITEM ; NDONE = 0
c
      IF (N .lt. 0) THEN
        NITEM = -N
        NDONE = 0
c
c---- Continuation line: N = number already read
c
      ELSE IF (N.GE.0) THEN
        NDONE = N
        NITEM = N
      END IF
      N = 1
      TOKEN = .FALSE.
      VALUE = 0.0
      OPRATR = .TRUE.
      IdoT = 0
      INTLEN = 0
      SIGN = 1.0
      OPER = 0
      OLDQUT = BLANK
      QUOTE = .FALSE.
      TQUOTE = .FALSE.
      NUMBER = .FALSE.
      COMMNT = .FALSE.
      LINLEN = r500_lenstr(line)
      IF (LINLEN .le. 0) THEN
        N = 0
        RETURN
      END IF
c
c---- Main loop over character buffer.  The loop goes one past the end,
c     but we're careful not to index this character.
c
      DO I = 1,LINLEN + 1
c
c----     check for comment character (not in string)
c
        IF (.not. QUOTE  .and.
     +     (line(I:I) .eq. ICOMM1 .or. line(I:I) .eq. ICOMM2)) THEN
          COMMNT = .TRUE.
c
c---- special case; comment line:
c
          IF (N .eq. 1) THEN
            N = 0
            RETURN
          END IF
        END IF
c
c---- Look for quotation marks
c
        IF (I .le. LINLEN) THEN
          IF (line(I:I) .eq. LETQT  .or.  line(I:I) .eq. DBLQT) THEN
c
c---- 1st quote must come at beginning of string, otherwise
c     treat as normal
c
            IF (OLDQUT .eq. BLANK  .and.  .not. TOKEN) THEN
c
c---- Start of quoted string
c
              OLDQUT = LETQT
              QUOTE = .TRUE.
            ELSE IF (OLDQUT .eq. LETQT) THEN
c
c---- End of quoted string
c
              OLDQUT = BLANK
              QUOTE = .FALSE.
            END IF
            GO TO 100
          END IF
        ELSE
          QUOTE = .FALSE.
        END IF
c
c---- Check for delimiting characters
c
        SKIP = .false.
        IF (I .le. LINLEN) THEN
          DO J = 1,NDELM
            IF (line(I:I) .eq. DELIM(J)) THEN
             SKIP = .true.
             GO TO 80
            END IF
          ENDDO
80        CONTINUE
        END IF
        IF (.not. SKIP) J = NDELM + 1
c
c
   30   CONTINUE
c
c
        IF ((.not. QUOTE .and.
     +      (J .le. NDELM .or. I .gt. LINLEN))  .or. COMMNT) THEN
c
c---- Have found a delimiter
c
          NULL = .FALSE.
          IF (.not. TOKEN  .and.  .not. COMMNT  .and.
     +         J .gt. NSPDLM) THEN
c
c---- Allow delimiters other than
c     <space> & <tab> to delimit null fields
c
            Ibeg(N) = I
            Ityp(N) = 0
            Iend(N) = I
            NULL = .TRUE.
          END IF
          IF (TOKEN) THEN
c
c---- End of token
c
            Iend(N) = I - 1
c               Exclude quote from token
            IF (TQUOTE  .and.  OLDQUT .eq. BLANK) Iend(N) = I - 2
c
c---- Store first 4 characters in Cv for all types
c
            LENG = Iend(N) - Ibeg(N) + 1
            IF (LENG .gt. 4) LENG = 4
            L = Ibeg(N)
            Cv(N) = line(L:L+LENG-1)
c
c---- Token is a number
c
            IF (NUMBER) THEN
              Ityp(N) = 2
              Fv(N) = VALUE*SIGN
              IF (OPER .eq. 1) THEN
c
c---- unary +
c
                Fv(N) = Fv(N) + SIGN0*VALUE0
              ELSE IF (OPER .eq. 2) THEN
c
c----  unary -
c
                Fv(N) = Fv(N) - SIGN0*VALUE0
              ELSE IF (OPER .eq. 5) THEN
c
c---- exponent
c
                Fv(N) = SIGN0*VALUE0*10.0**Fv(N)
              END IF
              IF (IdoT .eq. 1) THEN
                Idec(N) = 100*INTLEN + NPLACE
              ELSE
                Idec(N) = INTLEN
              END IF
            ELSE
c
c---- Token is Alphameric
c
              Ityp(N) = 1
              Idec(N) = LENG
            END IF
          END IF
          IF (TOKEN  .or.  NULL) THEN
            N = N + 1
            NCHK = N + NDONE
            TOKEN = .FALSE.
            VALUE = 0.0
            OPRATR = .TRUE.
            IdoT = 0
            INTLEN = 0
            SIGN = 1.0
            OPER = 0
            TQUOTE = .FALSE.
            NUMBER = .FALSE.
c
c---- Check number of items.
c
            IF (NCHK .gt. NITEM) THEN
              N = N - 1
              RETURN
            END IF
          END IF
c
c----  there's nothing else to do with a comment
c
          IF (COMMNT) THEN
            N = N - 1
            RETURN
          END IF
c
c---- If delimiter was "+" or "-", also treat it as part of the
c     next token
c
          IF (DELIM(J) .eq. '+'  .or.  DELIM(J) .eq. '-') THEN
            J = NDELM + 1
            GO TO 30
          END IF
          GO TO 100
        END IF
c
c---- Not a delimiter so must be a token -- suspect numeric token
c
        IF (.not. TQUOTE  .and.  (.not. TOKEN .or. NUMBER)) THEN
          SKIP = .false.
          IF (.not. QUOTE) THEN
            DO J = 1,NDIGS
              IF (line(I:I) .eq. DIGS(J)) THEN
                SKIP = .true.
                GO TO 85
               END IF
            ENDDO
85          CONTINUE
            IF (.not. SKIP) J = NDIGS + 1
c
c----  Change "e" to "E"
c
            IF (J .eq. 17) J = 15
            IF (J .le. NDIGS) THEN
c
c---- May be number
c
              NUMBER = .TRUE.
              IF (J .le. 10) THEN
c
c----  Have a digit 0-9
c
                IF (IdoT .eq. 0) THEN
                  INTLEN = INTLEN + 1
                  VALUE = (J-1) + VALUE*10
                END IF
                IF (IdoT .eq. 1) THEN
c
c----  Before decimal point
c
                  VALUE = (J-1)*F10 + VALUE
                  F10 = F10*0.1
c
c----  After decimal point
c
                  NPLACE = NPLACE + 1
                END IF
                OPRATR = .FALSE.
              ELSE IF (OPRATR  .and.
     +              (J .eq. 11 .or. J .eq. 12)) THEN
c
c---- Find + or - as signs not operators
c
                OPRATR = .FALSE.
c
c---- Set sign of number
c
                SIGN = ISGN(J-10)
              ELSE IF (J .eq. 15) THEN
c
c---- Do not allow 2 operators
c
                IF (OPRATR) NUMBER = .FALSE.
                VALUE0 = VALUE
                SIGN0 = SIGN
                OPER = J - 10
                VALUE = 0.0
                SIGN = 1.0
                IdoT = 0
                OPRATR = .TRUE.
c
c---- Find a decimal point
c       decimal point
c
              ELSE IF (J .eq. 16) THEN
                IdoT = IdoT + 1
                NPLACE = 0
                F10 = 0.1
c
c----  A valid number has one point
c
                IF (IdoT .eq. 2) NUMBER = .FALSE.
                OPRATR = .FALSE.
              END IF
            ELSE
c
c---- Token is not number
c
              NUMBER = .FALSE.
            END IF
          END IF
c
c---- Start a new token
c
          IF (.not. TOKEN) THEN
c
c---- Of any type
c
            TOKEN = .TRUE.
            Ibeg(N) = I
c
c---- Start quoted string
c
            IF (QUOTE) THEN
              TQUOTE = .TRUE.
              NUMBER = .FALSE.
            END IF
          END IF
        END IF
100     CONTINUE
      ENDDO
      N = N - 1
      RETURN
      END
c
c
c
c
c     ===============================
      INTEGER FUNCTION r500_lenstr(String)
c     ===============================
c
c---- Returns significant string length excluding trailing spaces
c
c  string (I)   CHARACTER*(*): Input string
c
      implicit none
c
c
      CHARACTER String* (*)
      INTRINSIC ICHAR,LEN
c
c
      r500_lenstr = LEN(String)
   10 CONTINUE
      IF (r500_lenstr.ne.0) THEN
        IF (String(r500_lenstr:r500_lenstr) .eq. ' ' .or.
     +      ICHAR(String(r500_lenstr:r500_lenstr)) .eq. 0) THEN
          r500_lenstr = r500_lenstr - 1
          GO TO 10
        END IF
      END IF
c
c
      END
c
c
c     ======================
      SUBROUTINE VDIFDP(A,B,C)
c     =====================
c
c
c     SUBTRACT TWO VECTORS
c
c
      implicit none
c
c
c
      integer I
      DOUBLE PRECISION A(3),B(3),C(3)
c
c
      DO I=1,3
       A(I)=B(I)-C(I)
      ENDDO
      RETURN
      END
c
c
c     =======================================================
      SUBROUTINE Praxin(Pxyz,Patoms,Centre,Dvalue,
     +                  Iplane,PlaneCutoff,rmsDIF,Prms,debug)
c     =======================================================
c
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
      DOUBLE PRECISION EigenVec(6),Vwork(3,3),LatentRoot(3),
     +                 DE,Det,TransposeMatrix(3,3),
     +                 DCx(3),DCy(3),DCz(3),
     +                 Qmat(3,3),Pxyz(3,MaxPlanesAtoms),
     +                 Xin(3),Centre(3),
     +                 Delta,Dsq,S1,Ssq,Value,Dvalue,
     +                 Cx,Cy,Cz,P,Prms(*),
     +                 PlaneCutoff,rmsDIF,
     +                 Xsum,Ysum,Zsum,XX,YY,ZZ,
     +                 XSQ,YSQ,ZSQ
      INTEGER Nmat,Jdo,Ido,Patoms,Iplane,Kdo
      LOGICAL debug
      INTRINSIC dble,sqrt
c
c
      Iplane = 0
      rmsDIF = 0.0d0
c
c
      Xsum = 0.0d0
      Ysum = 0.0d0
      Zsum = 0.0d0
c
c---- Loop through the data points
c
      DO Jdo = 1,MaxPlanesAtoms
       IF (Pxyz(1,Jdo) .gt. MissingValue) THEN
        Xsum = Pxyz(1,Jdo) + Xsum
        Ysum = Pxyz(2,Jdo) + Ysum
        Zsum = Pxyz(3,Jdo) + Zsum
       END IF
      ENDDO
c
c---- Calculate centre of mass and adjust all coordinates accordingly
c
      Centre(1) = Xsum/dble(Patoms)
      Centre(2) = Ysum/dble(Patoms)
      Centre(3) = Zsum/dble(Patoms)
c
c---- set up the inertial matrix q
c
      DO Ido=1,3
       DO Jdo=1,3
        Qmat(Ido,Jdo) = 0.0d0
       ENDDO
      ENDDO
c
c
      DO Jdo = 1,MaxPlanesAtoms
       IF (Pxyz(1,Jdo) .gt. MissingValue) THEN
          XX  = Pxyz(1,Jdo) - Centre(1)
          YY  = Pxyz(2,Jdo) - Centre(2)
          ZZ  = Pxyz(3,Jdo) - Centre(3)
          XSQ = XX*XX
          YSQ = YY*YY
          ZSQ = ZZ*ZZ
          Qmat(1,1) = Qmat(1,1) + (YSQ+ZSQ)
          Qmat(1,2) = Qmat(1,2) - XX*YY
          Qmat(1,3) = Qmat(1,3) - XX*ZZ
          Qmat(2,2) = Qmat(2,2) + (ZSQ+XSQ)
          Qmat(2,3) = Qmat(2,3) - YY*ZZ
          Qmat(3,3) = Qmat(3,3) + (XSQ + YSQ)
       END IF
      ENDDO
c
c
      Qmat(2,1) = Qmat(1,2)
      Qmat(3,1) = Qmat(1,3)
      Qmat(3,2) = Qmat(2,3)
c
c---- now get the latent roots and latent vectors of Qmat
c     store matrix Qmat in compact mode for eigen
c
      EigenVec(1) = Qmat(1,1)
      EigenVec(2) = Qmat(1,2)
      EigenVec(3) = Qmat(2,2)
      EigenVec(4) = Qmat(1,3)
      EigenVec(5) = Qmat(2,3)
      EigenVec(6) = Qmat(3,3)
c
c
      Nmat = 3
c
c---- this is a sledge-hammer
c
      CALL  EIGEN(EigenVec,Vwork,Nmat,0)
c
c---- the latent roots are now the diagonal  elements of A
c
      LatentRoot(1) = EigenVec(1)
      LatentRoot(2) = EigenVec(3)
      LatentRoot(3) = EigenVec(6)
c
c---- normalise the latent vectors to unit length
c
      DO Jdo=1,3
        DE = Vwork(1,Jdo)**2 +
     +       Vwork(2,Jdo)**2 +
     +       Vwork(3,Jdo)**2
        DE = sqrt(DE)
        Vwork(1,Jdo) = Vwork(1,Jdo)/DE
        Vwork(2,Jdo) = Vwork(2,Jdo)/DE
        Vwork(3,Jdo) = Vwork(3,Jdo)/DE
      ENDDO
c
c---- the rotation matrix TransposeMatrix is the transpose of Vwork
c
      DO Ido=1,3
       DO Jdo=1,3
        TransposeMatrix(Ido,Jdo) = Vwork(Jdo,Ido)
       ENDDO
      ENDDO
c
c
      Det =  TransposeMatrix(1,1)*
     +      (TransposeMatrix(2,2)*TransposeMatrix(3,3) -
     +       TransposeMatrix(2,3)*TransposeMatrix(3,2)) -
     +       TransposeMatrix(1,2)*
     +      (TransposeMatrix(2,1)*TransposeMatrix(3,3) -
     +       TransposeMatrix(2,3)*TransposeMatrix(3,1) ) +
     +       TransposeMatrix(1,3)*
     +      (TransposeMatrix(2,1)*TransposeMatrix(3,2) -
     +       TransposeMatrix(2,2)*TransposeMatrix(3,1) )
c
c
      IF ( ABS(Det - 1.000d0)  .ge. 0.0001d0)  THEN
c
c---- not proper matrix of rotation
c
      IF (debug) WRITE(6,6000) Det
 6000 FORMAT(
     + ' Least Squares Plane Vectors do not form a proper ',
     +     'rotation matrix',/,
     + ' Determinant = ',F10.5,/,
     + ' Direction of ZI axis changed in order to preserve',
     + ' RightHand Axial system')
      DO Kdo=1,3
       TransposeMatrix(3,Kdo)=-TransposeMatrix(3,Kdo)
      ENDDO
      END IF
c
c
      DO Jdo=1,3
       DCx(Jdo)=TransposeMatrix(1,Jdo)
       DCy(Jdo)=TransposeMatrix(2,Jdo)
       DCz(Jdo)=TransposeMatrix(3,Jdo)
      ENDDO
c
c--- transform coords xin = TransposeMatrix(Pxyz - Centre)
c     where TransposeMatrix is Vwork transpose
c
      Ido = 0
      S1  = 0d0
      Ssq = 0.0d0
      DO Jdo=1,MaxPlanesAtoms
       IF (Pxyz(1,Jdo) .gt. MissingValue) THEN
        Ido = Ido + 1
        XX = Pxyz(1,Jdo) - Centre(1)
        YY = Pxyz(2,Jdo) - Centre(2)
        ZZ = Pxyz(3,Jdo) - Centre(3)
        DO Kdo=1,3
         Xin(Kdo) = TransposeMatrix(Kdo,1)*XX +
     +              TransposeMatrix(Kdo,2)*YY +
     +              TransposeMatrix(Kdo,3)*ZZ
        ENDDO
        Delta  = Xin(1)
        Dsq    = Delta*Delta
        S1     = S1  + Dsq
        Ssq    = Ssq + Dsq
        END IF
       ENDDO
c
c---- Distance from plane
c
      DO Jdo=1,MaxPlanesAtoms
       IF (Pxyz(1,Jdo) .gt. MissingValue) THEN
        XX = Pxyz(1,Jdo) - Centre(1)
        YY = Pxyz(2,Jdo) - Centre(2)
        ZZ = Pxyz(3,Jdo) - Centre(3)
        Value = TransposeMatrix(1,1)*XX +
     +          TransposeMatrix(1,2)*YY +
     +          TransposeMatrix(1,3)*ZZ
        IF (Value .gt. PlaneCutoff) Iplane = Jdo
        rmsDIF = Value*Value + rmsDIF
        Prms(Jdo) = abs(Value)
       END IF
      ENDDO
c
c---- equation of the least squares plane
c
      Cx = Vwork(1,1)
      Cy = Vwork(2,1)
      Cz = Vwork(3,1)
      P  = Cx*Centre(1) + Cy*Centre(2) + Cz*centre(3)
c
c---- deviatons from plane
c
      Dvalue = sqrt(Ssq/Ido)
      rmsDIF = sqrt(rmsDIF/dble(Patoms))
c
c
      IF (debug) WRITE(6,6005) Cx,Cy,Cz,P,Ssq,Dvalue
 6005  FORMAT(' Equation of plane:',/,
     + 5x,'(',F8.5,').X +(',F8.5,').Y +(',F8.5,').Z =',F10.5,/,
     + 5x,' SUM Delta**2   = ',F10.4,10X,'R.M.S. Delta = ',F10.4)
      RETURN
      END
c
c     ==========================
      SUBROUTINE EIGEN(Amat,IdentMat,Npoints,Qswitch)
c     ==========================
c
c
c Author -  W.H.Press
c           B.P.Flannery
c           S.A.Teukolsky
c           W.T.Vetterling
c
c Cambridge Press
c This is a ccp4 program adaptation from a general release
c eigen function routine
c
c  COMPUTE EIGENVALUES AND EIGENVECTORS OF A REAL SYMMETRIC
c  MATRIX
c  DESCRIPTION OF PARAMETERS
c      A - ORIGINAL MATRIX (SYMMETRIC), DESTROYED IN COMPUTATION.
c          RESULTANT EIGENVALUES ARE DEVELOPED IN DIAGONAL OF
c          MATRIX A IN DESCENDING ORDER.
c      R - RESULTANT MATRIX OF EIGENVECTORS (STORED COLUMNWISE,
c          IN SAME SEQUENCE AS EIGENVALUES)
c      N - ORDER OF MATRICES A AND R
c      Qswitch- INPUT CODE
c              0   COMPUTE EIGENVALUES AND EIGENVECTORS
c              1   COMPUTE EIGENVALUES ONLY (R NEED NOT BE
c                  DIMENSIONED BUT MUST STILL APPEAR IN CALLING
c                  SEQUENCE)
c
c   REMARKS
c    ORIGINAL MATRIX A MUST BE REAL SYMMETRIC (STORAGE MODE=1)
c    MATRIX A CANNOT BE IN THE SAME LOCATION AS MATRIX R
c   METHOD
c     DIAGONALIZATION METHOD ORIGINATED BY JACOBI AND ADAPTED
c     BY VON NEUMANN FOR LARGE COMPUTERS AS FOUND IN 'MATHEMATICAL
c     METHODS FOR DIGITAL COMPUTERS', EDITED BY A. RALSTON AND
c     H.S. WILF, JOHN WILEY AND SONS, NEW YORK, 1962, CHAPTER 7
c
c
      implicit none
c
c
      DOUBLE PRECISION Anorm,Anrmx,Threshold,Y,
     +       Xsin,Xsin2,Xcos,Xcos2,CSsin,Range,
     +       Amat(*),IdentMat(*),Work
      INTEGER Qswitch,Jdo,Ido,Mdo,IQ,JQ,Npoints,IJ,IA,
     +        Kindex,L,M,MQ,LQ,LM,ILQ,IMQ,MM,LL,ILR,IMR,
     +        IM,IL
c
c---- generate identity matrix
c
    5 Range = 1.0d-12
c
c
      IF(Qswitch-1 .ne. 0) THEN
       IQ = -Npoints
       DO Jdo=1,Npoints
        IQ = IQ + Npoints
        DO Ido=1,Npoints
         IJ = IQ + Ido
         IdentMat(IJ) = 0.0d0
         IF (Ido .eq. Jdo) IdentMat(IJ) = 1.0d0
        ENDDO
       ENDDO
      END IF
c
c---- compute initial and final norms (anorm and anormx)
c
      Anorm=0.0
c
c
      DO Ido=1,Npoints
       DO Jdo=Ido,Npoints
        IF(Ido .ne. Jdo) THEN
         IA    = Ido + (Jdo*Jdo-Jdo)/2
         Anorm = Anorm + Amat(IA)*Amat(IA)
        END IF
       ENDDO
      ENDDO
c
c
      IF (Anorm .gt. 0.0d0) THEN
       Anorm = 1.414*sqrt(Anorm)
       Anrmx = Anorm*Range/dble(Npoints)
c
c---- initialize indicators and compute threshold, thr
c
       Kindex    = 0
       Threshold = Anorm
   10  Threshold = Threshold/dble(Npoints)
   20  L         = 1
   30  M         = L + 1
   40  MQ        = (M*M-M)/2
       LQ        = (L*L-L)/2
       LM        = L + MQ
       IF (abs(Amat(LM))-Threshold .ge. 0.0d0) THEN
        Kindex = 1
        LL     = L + LQ
        MM     = M + MQ
        Work   = 0.5d0*(Amat(LL)-Amat(MM))
C---- Protect against rounding error. C. Flensburg 20080307.
        Y      = -Amat(LM)/ sqrt(Amat(LM)*Amat(LM) + Work*Work)
        IF (abs(Y) .gt. 1.0d0) Y = sign(1.0d0, Y)
        IF (Work .lt. 0.0d0) Y = -Y
        Xsin  = Y/ sqrt(2.0d0*(1.0d0+(sqrt(1.0d0 - Y*Y))))
        Xsin2 = Xsin*Xsin
        Xcos  = sqrt(1.0d0-Xsin2)
        Xcos2 = Xcos*Xcos
        CSsin = Xsin*Xcos
c
c---- rotate l and m columns
c
        ILQ = Npoints*(L-1)
        IMQ = Npoints*(M-1)
c
c
        DO Ido=1,Npoints
         IQ = (Ido*Ido-Ido)/2
         IF (Ido-L .ne. 0) THEN
          IF (Ido-M .ne. 0) THEN
           IF (Ido-M .lt. 0) THEN
            IM = Ido + MQ
           ELSE IF (Ido-M .gt. 0) THEN
            IM = M + IQ
           END IF
           IF (Ido-L .lt. 0) THEN
            IL = Ido + LQ
           ELSE
            IL = L + IQ
           END IF
           Work     = Amat(IL)*Xcos - Amat(IM)*Xsin
           Amat(IM) = Amat(IL)*Xsin + Amat(IM)*Xcos
           Amat(IL) = Work
          END IF
         END IF
         IF (Qswitch-1 .ne. 0) THEN
          ILR = ILQ + Ido
          IMR = IMQ + Ido
          Work          = IdentMat(ILR)*Xcos - IdentMat(IMR)*Xsin
          IdentMat(IMR) = IdentMat(ILR)*Xsin + IdentMat(IMR)*Xcos
          IdentMat(ILR) = Work
         END IF
        ENDDO
        Work     = 2.0d0*Amat(LM)*CSsin
        Y        = Amat(LL)*Xcos2 + Amat(MM)*Xsin2 - Work
        Work     = Amat(LL)*Xsin2 + Amat(MM)*Xcos2 + Work
        Amat(LM) = (Amat(LL) - Amat(MM))*CSsin +
     +                  Amat(LM)*(Xcos2-Xsin2)
        Amat(LL) = Y
        Amat(MM) = Work
c
c---- tests for completion
c     test for m = last column
c
      END IF
      IF (M-Npoints .ne. 0) THEN
       M = M + 1
       GO TO 40
      END IF
c
c----  test for l = second from last column
c
      IF (L-(Npoints-1) .ne. 0) THEN
       L = L + 1
       GO TO 30
      END IF
      IF(Kindex-1 .eq. 0) THEN
       Kindex = 0
       GO TO 20
      END IF
c
c---- compare threshold with final norm
c
      IF (Threshold-Anrmx .gt. 0.0d0) GO TO 10
c
c---- sort eigenvalues and eigenvectors
c
      END IF
c
c
      IQ = -Npoints
      DO Ido=1,Npoints
       IQ = IQ + Npoints
       LL = Ido + (Ido*Ido - Ido)/2
       JQ = Npoints*(Ido-2)
       DO Jdo=Ido,Npoints
        JQ = JQ + Npoints
        MM = Jdo + (Jdo*Jdo - Jdo)/2
        IF (Amat(LL) - Amat(MM) .lt. 0.0d0) THEN
         Work     = Amat(LL)
         Amat(LL) = Amat(MM)
         Amat(MM) = Work
         IF (Qswitch-1 .ne. 0) THEN
          DO Mdo=1,Npoints
           Work             = IdentMat(IQ+Mdo)
           IdentMat(IQ+Mdo) = IdentMat(JQ+Mdo)
           IdentMat(JQ+Mdo) = Work
          ENDDO
         END IF
        END IF
       ENDDO
      ENDDO
c
c
      RETURN
      END
c
c
c     ======================================================
      SUBROUTINE CysStore(Ncys,Watoms,Wname,xyz,Walt,
     +                    Wres,Wchain,Wseq,Wins,Wmodel)
c     ======================================================
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,MaxCys
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxCys         = 200)
c
c
      INTEGER Ncys,Watoms,Wseq,Wmodel,Jdo,Kdo
      CHARACTER Wres*3,Wchain*1,Wins*1
      DOUBLE PRECISION xyz(3,MaxAtoms)
      CHARACTER Wname(MaxAtoms)*4,Walt(MaxAtoms)*1
c
c
      INTEGER Catoms,Cseq
      COMMON /Icys/ Catoms(MaxCys),Cseq(MaxCys)
      CHARACTER Cchain*1,Cins*1,Cname*4,Calt*1
      COMMON /Ccys/ Cchain(MaxCys),Cins(MaxCys),
     +              Cname(MaxCys,MaxAtoms),
     +              Calt(MaxCys,MaxAtoms)
      DOUBLE PRECISION Cxyz
      COMMON /Dcys/ Cxyz(MaxCys,3,MaxAtoms)
c
c
      IF (Ncys .gt. MaxCys) STOP 'Max CYS exceeded'
      Cchain(Ncys) = Wchain
      Cins(Ncys)   = Wins
      Cseq(Ncys)   = Wseq
      Catoms(Ncys) = Watoms
      DO Jdo=1,MaxAtoms
       DO Kdo=1,3
        Cxyz(Ncys,Kdo,Jdo) = xyz(Kdo,Jdo)
       ENDDO
       Cname(Ncys,Jdo) = Wname(Jdo)
       Calt(Ncys,Jdo)  = Walt(Jdo)
      ENDDO
c
c
      RETURN
      END
c
c
c     ===============================================
      SUBROUTINE GetDisulphide(Xmin,Ymin,Zmin,NX,NY,NZ,
     +                         Boxes,Ncys,Wcys,Dss,SSdone,
     +                         BAArmsdClass,BAATotalType,
     +                         AAArmsdClass,AAATotalType,
     +                         BondLinkerValues,
     +                         Bdone,Adone)
c     ===============================================
c
c
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,M37,MaxA,MaxSize,MaxSym,MaxCys
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (M37            = 37)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxA           = 250000)
      PARAMETER (MaxSym         = 230)
      PARAMETER (MaxSize        = 100000000)
      PARAMETER (MaxCys         = 200)
c
c
      LOGICAL Bdone(MaxBondTarget),Adone(MaxAngleTarget)
      INTEGER  BAATotalType(MaxBclass+MaxBondLinker),
     +         AAATotalType(MaxAclass+MaxAngleLinker)
      DOUBLE PRECISION BAArmsdClass(MaxBclass+MaxBondLinker),
     +                 AAArmsdClass(MaxAclass+MaxAngleLinker),
     +                 BondLinkerValues(MaxBondLinker) 
c
c
      DOUBLE PRECISION Xmin,Ymin,Zmin
      INTEGER NX,NY,NZ
      INTEGER Boxes(51,NX,NY,NZ)
      INTEGER Kall,Wcys
      DOUBLE PRECISION Qxyz
      INTEGER Qnum,Qseq
      CHARACTER Qalt*1,Qchain*1,Qins*1,Qres*3,Qname*4,Qele*2
      CHARACTER Wres*3,Wchain*1,Wins*1
      CHARACTER Wname(MaxAtoms)*4,Walt(MaxAtoms)*1
      DOUBLE PRECISION D,Dss,X,Y,Z,Dx(3),Dvalue
      DOUBLE PRECISION xyz(3,MaxAtoms)
      INTEGER Ide,IX,IY,IZ,Jdo,Katl,Iab,Kbtl,Kctl,
     +        Mtarget,Msource,Nab,Juse,Klinker,Ndone,
     +        Ncys,Watoms,Wseq,Wmodel,Cdo,K1,J1,Mdo
      REAL Qocc
      LOGICAL SSdone(MaxCys)
c
c
      INTEGER Catoms,Cseq,Disulphide
      COMMON /Icys/ Catoms(MaxCys),Cseq(MaxCys)
      CHARACTER Cchain*1,Cins*1,Cname*4,Calt*1
      COMMON /Ccys/ Cchain(MaxCys),Cins(MaxCys),
     +              Cname(MaxCys,MaxAtoms),
     +              Calt(MaxCys,MaxAtoms)
      DOUBLE PRECISION Cxyz
      COMMON /Dcys/ Cxyz(MaxCys,3,MaxAtoms)
c
c
      COMMON /W41/Qxyz(3,MaxA)
      COMMON /W4/Qnum(MaxA),Kall
      COMMON /W3/Qseq(MaxA)
      COMMON /W31/Qocc(MaxA)
      COMMON /W9/Qins(MaxA),Qalt(MaxA),Qname(MaxA),Qres(MaxA),
     +           Qchain(MaxA),Qele(MaxA)
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
      Disulphide = 0
      Ndone = 0
c
c
      DO 80 Msource = 1,Kall
       X = Qxyz(1,Msource) - Xmin
       Y = Qxyz(2,Msource) - Ymin
       Z = Qxyz(3,Msource) - Zmin
       IF (Xmin.lt.0.0D0) THEN
        IX = nint((X+6.1D0)/6.0D0) + 1
       ELSE
        IX = INT(X/6.0D0) + 2
       END IF
       IF (Ymin.lt.0.0D0) THEN
        IY = nint((Y+6.1D0)/6.0D0) + 1
       ELSE
        IY = INT(Y/6.0D0) + 2
       END IF
       IF (Zmin.lt.0.0D0) THEN
        IZ = nint((Z+6.1D0)/6.0D0) + 1
       ELSE
        IZ = INT(Z/6.0D0) + 2
       END IF
       DO 70 Katl = IX - 1,IX + 1
        IF (Katl.le.0 .or. Katl.gt.NX) GO TO 70
        DO 60 Kbtl = IY - 1,IY + 1
         IF (Kbtl.le.0 .or. Kbtl.gt.NY) GO TO 60
         DO 50 Kctl = IZ - 1,IZ + 1
          IF (Kctl.le.0 .or. Kctl.gt.NZ) GO TO 50
          Nab = Boxes(1,Katl,Kbtl,Kctl)
          IF (Nab.ne.0) THEN
           DO 40 Iab = 2,Nab + 1
            Mtarget = Boxes(Iab,Katl,Kbtl,Kctl)
            IF (Mtarget.ne.Msource) THEN
             IF (Qres(Msource) .eq. 'CYS' .and. 
     +           Qres(Mtarget) .eq. 'CYS') THEN
             IF (Qname(Msource) .eq. ' SG ' .and. 
     +           Qname(Mtarget) .eq. ' SG ') THEN
             IF (Qchain(Msource) .eq. Qchain(Mtarget)) THEN
              IF (Qseq(Msource) .eq. Qseq(Mtarget)) GO TO 40
              Ide = abs(Qseq(Msource)-Qseq(Mtarget))
              IF (Ide.lt.2) GO TO 40
             END IF
             IF (Qalt(Msource) .ne. ' ' .or. 
     +           Qalt(Mtarget) .ne. ' ') THEN
              IF (Qalt(Msource) .ne. ' ' .and. 
     +            Qalt(Mtarget) .ne. ' ')THEN
               IF (Qalt(Msource) .ne. Qalt(Mtarget)) GO TO 40
              END IF
             END IF
             DO Jdo = 1,3
              Dx(Jdo) = abs(Qxyz(Jdo,Mtarget)-Qxyz(Jdo,Msource))
             ENDDO
             D = Dx(1)*Dx(1) + Dx(2)*Dx(2) + Dx(3)*Dx(3)
c
c
             IF (D.le.Dss) THEN
              D = sqrt(D)
c
c---- which CYS is this, use Disulphide as switch for di-sulphide
c
            DO Cdo=1,Ncys
             IF (.not. SSdone(Cdo)) THEN
c
c
               IF (Qchain(Msource) .eq. Cchain(Cdo)) THEN
                IF (Qseq(Msource) .eq. Cseq(Cdo)) THEN
                 IF (Qins(Msource) .eq. Cins(Cdo)) THEN
                  Wchain = Cchain(Cdo)
                  Wins   = Cins(Cdo)
                  Wseq   = Cseq(Cdo)
c
c---- is it model 0 or model 1 ??
c
                  Wmodel = 0
                  Watoms = Catoms(Cdo)
                  DO K1=1,MaxAtoms
                   DO J1=1,3
                    xyz(J1,K1) = Cxyz(Cdo,J1,K1)
                   ENDDO
                   Wname(K1) = Cname(Cdo,K1) 
                   Walt(K1)  = Calt(Cdo,K1)
                  ENDDO
                  Disulphide = 100
c
c
                  CALL Cystine(Wcys,Watoms,Wname,xyz,Walt,
     +                   BAArmsdClass,BAATotalType,
     +                   AAArmsdClass,AAATotalType,
     +                   Wres,Wchain,Wseq,Wins,
     +                   Wmodel,Disulphide,Bdone,Adone)
                    SSdone(Cdo) = .true. 
                    Ndone = Ndone + 1
c
c---- check the rms for the SG--SG here
c
cSSBOND   1 CYS A    6    CYS A   11 -- how to stop getting 2nd one
cSSBOND   2 CYS A    7    CYS B    7 
cSSBOND   3 CYS A   20    CYS B   19 
cSSBOND   4 CYS C    6    CYS C   11 
cSSBOND   5 CYS C    7    CYS D    7 
cSSBOND   6 CYS C   20    CYS D   19 
cSSBOND   7 CYS E    6    CYS E   11 
cSSBOND   8 CYS E    7    CYS F    7 
cSSBOND   9 CYS E   20    CYS F   19 
cREMARK 500    CYS A   6   SG    CYS A  11   SG     -0.123
cREMARK 500    CYS A  11   SG    CYS A   6   SG     -0.123 x dont want this x
c
c---- check if partner already done
c
               IF (Ndone .gt.0) THEN
                DO Mdo = 1,Ncys
                 IF (Qchain(Mtarget) .eq. Cchain(Mdo)) THEN
                  IF (Qseq(Mtarget) .eq. Cseq(Mdo)) THEN
                   IF (Qins(Mtarget) .eq. Cins(Mdo)) THEN
                    IF (SSdone(Mdo)) GO TO 800
                   END IF
                  END IF
                 END IF
                ENDDO
               END IF
c
c
           Juse    = MaxBclass + 5
           Dvalue  = D - BondLinkerValues(5)
           Klinker = 2
          BAArmsdClass(Juse) = BAArmsdClass(Juse) + Dvalue*Dvalue
          BAATotalType(Juse) = BAATotalType(Juse) + 1
c
c
          WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'BAA',Juse,Wmodel,D,Dvalue,Klinker,
     +      'CYS',Wchain,Wseq,Wins,' SG ',Qalt(Msource),
     +      'CYS',Qchain(Mtarget),Qseq(Mtarget),Qins(Mtarget),
     +          ' SG ',Qalt(Mtarget),
     +      '   ',' ',0,' ','    ','  '
                 GO TO 800
                 END IF
                END IF
               END IF
c
c
               IF (Qchain(Mtarget) .eq. Cchain(Cdo)) THEN
                IF (Qseq(Mtarget) .eq. Cseq(Jdo)) THEN
                 IF (Qins(Mtarget) .eq. Cins(Jdo)) THEN
                  Wchain = Cchain(Cdo)
                  Wins   = Cins(Cdo)
                  Wseq   = Cseq(Cdo)
                  Wmodel = 0
                  Watoms = Catoms(Cdo)
                  DO K1=1,MaxAtoms
                   DO J1=1,3
                    xyz(J1,K1) = Cxyz(Cdo,J1,K1)
                   ENDDO
                   Wname(K1) = Cname(Cdo,K1) 
                   Walt(K1)  = Calt(Cdo,K1)
                  ENDDO
                  Disulphide = 100
                  CALL Cystine(Wcys,Watoms,Wname,xyz,Walt,
     +                   BAArmsdClass,BAATotalType,
     +                   AAArmsdClass,AAATotalType,
     +                   Wres,Wchain,Wseq,Wins,
     +                   Wmodel,Disulphide,Bdone,Adone)
                  SSdone(Cdo) = .true. 
                  Ndone = Ndone + 1
                 END IF
                END IF
               END IF
c
c
             END IF
 800         CONTINUE
            ENDDO
c
c
               END IF
              END IF
             END IF
            END IF
   40      CONTINUE
          END IF
   50    CONTINUE
   60   CONTINUE
   70  CONTINUE
   80 CONTINUE
      RETURN
      END
c
c     ===============================================
      SUBROUTINE SymmDisulphide(Xmin,Ymin,Zmin,NX,NY,NZ,
     +                      Boxes,Ncys,Wcys,Dss,SSdone,
     +                      BAArmsdClass,BAATotalType,
     +                      AAArmsdClass,AAATotalType,
     +                      BondLinkerValues,   
     +                      Bdone,Adone)
c     ===============================================
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass,M37,MaxA,MaxSize,MaxSym,MaxCys
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (M37            = 37)
      PARAMETER (MissingValue   = -999999.9D0)
      PARAMETER (MaxA           = 250000)
      PARAMETER (MaxSym         = 230)
      PARAMETER (MaxSize        = 100000000)
      PARAMETER (MaxCys         = 200)
c
c
      LOGICAL Bdone(MaxBondTarget),Adone(MaxAngleTarget)
      INTEGER  BAATotalType(MaxBclass+MaxBondLinker),
     +         AAATotalType(MaxAclass+MaxAngleLinker)
      DOUBLE PRECISION BAArmsdClass(MaxBclass+MaxBondLinker),
     +                 AAArmsdClass(MaxAclass+MaxAngleLinker),
     +                 BondLinkerValues(MaxBondLinker)
      LOGICAL SSdone(MaxCys)
      INTEGER Ncys,Watoms,Wseq,Jdo,Cdo,K1,J1,Wcys,Klinker,Juse
      CHARACTER Wres*3,Wchain*1,Wins*1
      DOUBLE PRECISION xyz(3,MaxAtoms),Dvalue
      CHARACTER Wname(MaxAtoms)*4,Walt(MaxAtoms)*1
c
c
      INTEGER Catoms,Cseq,Disulphide
      COMMON /Icys/ Catoms(MaxCys),Cseq(MaxCys)
      CHARACTER Cchain*1,Cins*1,Cname*4,Calt*1
      COMMON /Ccys/ Cchain(MaxCys),Cins(MaxCys),
     +              Cname(MaxCys,MaxAtoms),
     +              Calt(MaxCys,MaxAtoms)
      DOUBLE PRECISION Cxyz
      COMMON /Dcys/ Cxyz(MaxCys,3,MaxAtoms)
c
c
      DOUBLE PRECISION Dss,Xmin,Ymin,Zmin
      INTEGER NX,NY,NZ
      INTEGER Boxes(51,NX,NY,NZ)
      INTEGER Idnt,Kall,Sall,Msym
      DOUBLE PRECISION Rsym,S,TR,Qxyz,Sxyz
      INTEGER Ctran,Ias,Qnum,Snum,Qseq,Qsym
      CHARACTER Qalt*1,Qchain*1,Qins*1,Qname*4,Qres*3,Qele*2
      DOUBLE PRECISION D,X,Y,Z
      INTEGER Idel,Ipr,IX,IY,IZ,Kab,Kat,Kbt,Kct,Kst,
     +        Ktan,Ktt,Msource,Nab,Mtarget,Wmodel
      DOUBLE PRECISION Dx(3)
      REAL Qocc
c
c
      COMMON /W1/S(4,4,MaxSym),Rsym(4,4,MaxSym),Sxyz(3,2*MaxA)
      COMMON /W11/Snum(2*MaxA),Ias(2*MaxA),Qsym(2*MaxA),Sall,Msym,
     +       Idnt
      COMMON /W3/Qseq(MaxA)
      COMMON /W4/Qnum(MaxA),Kall
      COMMON /W41/Qxyz(3,MaxA)
      COMMON /W7/Ctran(2*MaxA)
      COMMON /W71/TR(4,4,MaxSym)
      COMMON /W9/Qins(MaxA),Qalt(MaxA),Qname(MaxA),Qres(MaxA),
     +           Qchain(MaxA),Qele(MaxA)
c
c--- check use of qele and qocc
c
      COMMON /W31/Qocc(MaxA)
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
      Wmodel = 0
c
c
      DO 100 Msource = 1,Kall
       Ipr = 1
       X = Qxyz(1,Msource) - Xmin
       Y = Qxyz(2,Msource) - Ymin
       Z = Qxyz(3,Msource) - Zmin
       IF (Xmin.lt.0.0D0) THEN
        IX = nint((X+6.1D0)/6.0D0) + 1
       ELSE
        IX = INT(X/6.0D0) + 2
       END IF
       IF (Ymin.lt.0.0) THEN
        IY = nint((Y+6.1D0)/6.0D0) + 1
       ELSE
        IY = INT(Y/6.0D0) + 2
       END IF
       IF (Zmin.lt.0.0D0) THEN
        IZ = nint((Z+6.1D0)/6.0D0) + 1
       ELSE
        IZ = INT(Z/6.0D0) + 2
       END IF
c
c
       DO 90 Kat = IX - 1,IX + 1
        IF (Kat.le.0 .or. Kat.gt.NX) GO TO 90
         DO 80 Kbt = IY - 1,IY + 1
          IF (Kbt.le.0 .or. Kbt.gt.NY) GO TO 80
          DO 70 Kct = IZ - 1,IZ + 1
           IF (Kct.le.0 .or. Kct.gt.NZ) GO TO 70
           Nab = Boxes(1,Kat,Kbt,Kct)
           IF (Nab.ne.0) THEN
            DO 60 Kab = 2,Nab + 1
             Ktan = Boxes(Kab,Kat,Kbt,Kct)
             Kst = Qsym(Ktan)
             Ktt = Ctran(Ktan)
             Mtarget = Ias(Ktan)

             IF (Qres(Msource) .eq. 'CYS' .and. 
     +           Qres(Mtarget) .eq. 'CYS') THEN
             IF (Qname(Msource) .eq. ' SG ' .and.
     +           Qname(Mtarget) .eq. ' SG ') THEN
             IF (Kst.ne.Idnt) THEN
              IF (Qseq(Msource) .eq. Qseq(Mtarget) .and. 
     +             Kst .eq. Idnt) GO TO 40
              IF (Kst .eq. Idnt) THEN
               Idel = abs(Qseq(Msource)-Qseq(Mtarget))
               IF (Idel.lt.2) GO TO 40
              END IF
              DO Jdo = 1,3
               Dx(Jdo) = abs(Sxyz(Jdo,Ktan)-Qxyz(Jdo,Msource))
              ENDDO
              D = Dx(1)*Dx(1) + Dx(2)*Dx(2) + Dx(3)*Dx(3)
              IF (D.le.Dss) THEN
               D = sqrt(D)
c
c---- which CYS is this, use Disulphide as switch for di-sulphide
c
            DO Cdo=1,Ncys
             IF (.not. SSdone(Cdo)) THEN
               IF (Qchain(Msource) .eq. Cchain(Cdo)) THEN
                IF (Qseq(Msource) .eq. Cseq(Cdo)) THEN
                 IF (Qins(Msource) .eq. Cins(Cdo)) THEN
                  Wchain = Cchain(Cdo)
                  Wins   = Cins(Cdo)
                  Wseq   = Cseq(Cdo)
c
c---- is it model 0 or model 1 ??
c
                  Wmodel = 0
                  Watoms = Catoms(Cdo)
                  DO K1=1,MaxAtoms
                   DO J1=1,3
                    xyz(J1,K1) = Cxyz(Cdo,J1,K1)
                   ENDDO
                   Wname(K1) = Cname(Cdo,K1) 
                   Walt(K1)  = Calt(Cdo,K1)
                  ENDDO
                  Disulphide = 100
                  CALL Cystine(Wcys,Watoms,Wname,xyz,Walt,
     +                   BAArmsdClass,BAATotalType,
     +                   AAArmsdClass,AAATotalType,
     +                   Wres,Wchain,Wseq,Wins,
     +                   Wmodel,Disulphide,Bdone,Adone)
                  SSdone(Cdo) = .true. 
c
c---- check the rms for the SG--SG here
c
           Juse    = MaxBclass + 5
           Dvalue  = D - BondLinkerValues(5)
           Klinker = 2
          BAArmsdClass(Juse) = BAArmsdClass(Juse) + Dvalue*Dvalue
          BAATotalType(Juse) = BAATotalType(Juse) + 1
          WRITE (Iscr,
     +     fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +     3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +     'BAA',Juse,Wmodel,D,Dvalue,Klinker,
     +      'CYS',Wchain,Wseq,Wins,' SG ',Qalt(Msource),
     +      'CYS',Qchain(Mtarget),Qseq(Mtarget),Qins(Mtarget),
     +          ' SG ',Qalt(Mtarget),
     +      '   ',' ',0,' ','    ','  '
                 END IF
                END IF
               END IF
               IF (Qchain(Mtarget) .eq. Cchain(Cdo)) THEN
                IF (Qseq(Mtarget) .eq. Cseq(Cdo)) THEN
                 IF (Qins(Mtarget) .eq. Cins(Cdo)) THEN
                  Wchain = Cchain(Cdo)
                  Wins   = Cins(Cdo)
                  Wseq   = Cseq(Cdo)
                  Wmodel = 0
                  Watoms = Catoms(Cdo)
                  DO K1=1,MaxAtoms
                   DO J1=1,3
                    xyz(J1,K1) = Cxyz(Cdo,J1,K1)
                   ENDDO
                   Wname(K1) = Cname(Cdo,K1) 
                   Walt(K1)  = Calt(Cdo,K1)
                  ENDDO
                  Disulphide = 100
                  CALL Cystine(Wcys,Watoms,Wname,xyz,Walt,
     +                   BAArmsdClass,BAATotalType,
     +                   AAArmsdClass,AAATotalType,
     +                   Wres,Wchain,Wseq,Wins,
     +                   Wmodel,Disulphide,Bdone,Adone)
                  SSdone(Cdo) = .true. 
                 END IF
                END IF
               END IF
             END IF
            ENDDO
c
c
              END IF
             END IF
             END IF
             END IF
   40       CONTINUE
   60     CONTINUE
         END IF
   70    CONTINUE
   80   CONTINUE
   90  CONTINUE
  100 CONTINUE
c
c
      RETURN
      END
c
c      ========================================================
       SUBROUTINE Cystine(Wcys,Watoms,Wname,xyz,Walt,
     +                    BAArmsdClass,BAATotalType,
     +                    AAArmsdClass,AAATotalType,
     +                    Wres,Wchain,Wseq,Wins,
     +                    Wmodel,Disulphide,Bdone,Adone)
c     =========================================================
c
c  expect
cMaxStandard    37
cNaa            24
cNna            37
cNumBondLinker   4
cNumAngleLinker 11
cMaxAngleTarget 56
cMaxBondTarget  36
cMaxPlanesAtoms 12
cMaxBondLinker   4
cMaxAngleLinker 11
c
cclasses bonds  - 200  aa 1-73, na 100-194
cclasses angles - 250  aa 1-79, na 10-213
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
c
c
      INTEGER Wcys,Watoms,Wseq,Wmodel,Klinker,Disulphide
      CHARACTER Wname(MaxAtoms)*4,Walt(MaxAtoms)*1,Wres*3,
     +           Wchain*1,Wins*1
      INTEGER  BAATotalType(MaxBclass+MaxBondLinker),
     +         AAATotalType(MaxAclass+MaxAngleLinker)
      DOUBLE PRECISION xyz(3,MaxAtoms)
      DOUBLE PRECISION BAArmsdClass(MaxBclass+MaxBondLinker),
     +                 AAArmsdClass(MaxAclass+MaxAngleLinker)
c
c
      INTEGER NNtype,N1N9,Natoms,NumAtmStanPlane,
     +        NumBondTarget,BondTargetClass,NumAngleTarget,
     +        AngleTargetClass,NumBondLinker,AAflag,
     +        NumAngleLinker
      CHARACTER StanRes3*3,stanNewNam*4,
     +          PlaneNames*4,
     +          BondPairs*4,AngleTriples*4,
     +          BondLinkerClass*12,BondLinkAtom*4,
     +          AngleLinkerClass*12,
     +          AngleLinkAtom*4,BondDescr*10,
     +          AngleDescr*10
      DOUBLE PRECISION BondTargetValues,BondTargetEsds,
     +                 AngleTargetValues,AngleTargetEsds,
     +                 BondLinkerValues,BondLinkerEsds,
     +                 AngleLinkerValues,AngleLinkerEsds
      COMMON /Cstan/ StanRes3(MaxStandard),
     +       stanNewNam(MaxStandard,MaxAtoms),
     +       PlaneNames(MaxStandard,MaxPlanesAtoms),
     +       BondPairs(2,MaxStandard,MaxBondTarget),
     +       AngleTriples(3,MaxStandard,MaxAngleTarget),
     +       BondLinkerClass(MaxBondLinker),
     +       BondLinkAtom(2,MaxBondLinker),
     +       AngleLinkerClass(MaxAngleLinker),
     +       AngleLinkAtom(3,MaxAngleLinker),
     +       BondDescr(MaxStandard,MaxBondTarget),
     +       AngleDescr(MaxStandard,MaxAngleTarget)
      COMMON /Dstan/
     +       BondTargetClass(MaxStandard,MaxBondTarget),
     +       BondTargetValues(MaxStandard,MaxBondTarget),
     +       BondTargetEsds(MaxStandard,MaxBondTarget),
     +       AngleTargetClass(MaxStandard,MaxAngleTarget),
     +       AngleTargetValues(MaxStandard,MaxAngleTarget),
     +       AngleTargetEsds(MaxStandard,MaxAngleTarget),
     +       BondLinkerValues(MaxBondLinker),
     +       BondLinkerEsds(MaxBondLinker),
     +       AngleLinkerValues(MaxAngleLinker),
     +       AngleLinkerEsds(MaxAngleLinker)
      COMMON /Istan/ NNtype(MaxStandard),
     +       N1N9(MaxStandard),
     +       Natoms(MaxStandard),
     +       NumAtmStanPlane(MaxStandard),
     +       NumBondTarget(MaxStandard),
     +       NumAngleTarget(MaxStandard),
     +       AAflag(MaxStandard),
     +       NumBondLinker,NumAngleLinker
c
c
      LOGICAL Bdone(MaxBondTarget),Adone(MaxAngleTarget)
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
       INTEGER Jdo,Kdo,Ldo,Ndo,Juse,Qdo
       DOUBLE PRECISION Dist,Dvalue,Wangle,Dist1,Dist2,
     +                  Dist3,Dif1,Dif2,Dif3,COSANG,RADDEG,PI
c
c
c
       Klinker = 1
       PI     = ATAN(1.0d0)*4.0d0
       RADDEG = 180.0d0/PI
         DO Jdo=1,MaxBondTarget
          Bdone(Jdo) = .false.
         ENDDO
         DO Jdo=1,MaxAngleTarget
          Adone(Jdo)= .false.
         ENDDO
c
c---- bonds -- check if di-sulhide
c              Disulphide = 100 if sg-sg
c
      DO Jdo=1,NumBondTarget(Wcys)
        IF (.not. Bdone(Jdo)) THEN
       DO Kdo=1,Watoms
        IF (Wname(Kdo) .eq. BondPairs(1,Wcys,Jdo))THEN
         IF (xyz(1,Kdo) .gt. MissingValue) THEN
          DO Ldo=1,Watoms
           IF (Ldo.ne.Kdo) THEN
            IF(Wname(Ldo) .eq. BondPairs(2,Wcys,Jdo))THEN
             IF (Walt(Ldo) .eq. Walt(Kdo) ) THEN
              IF (xyz(1,Ldo) .gt. MissingValue) THEN
               Dist = 0.0d0
               DO Ndo = 1,3
                Dist = ( xyz(Ndo,Kdo) - xyz(Ndo,Ldo) )**2 + Dist
               ENDDO
               Dist   = sqrt (Dist)
c
c always a pair
cCYS    CA    CB     SING N N  5    1.526    0.013    7 "SH"
cCYS    CA    CB     SING N N  5    1.535    0.022    ? "SS"
c
               IF (BondDescr(Wcys,Jdo)(1:1) .eq. '.') THEN
                Juse       = BondTargetClass(Wcys,Jdo)
                Dvalue     = Dist - BondTargetValues(Wcys,Jdo)
                Bdone(Jdo) = .true.
               ELSE IF (BondDescr(Wcys,Jdo)(1:2) .eq. 'SH' .and.
     +           Disulphide .eq. 100) THEN
                 Juse         = BondTargetClass(Wcys,Jdo+1)
                 Dvalue       = Dist - BondTargetValues(Wcys,Jdo+1)
                 Bdone(Jdo)   = .true.
                 Bdone(Jdo+1) = .true.
               ELSE IF (BondDescr(Wcys,Jdo)(1:2) .eq. 'SH' .and.
     +           Disulphide .ne. 100) THEN
                 Juse         = BondTargetClass(Wcys,Jdo)
                 Dvalue       = Dist - BondTargetValues(Wcys,Jdo)
                 Bdone(Jdo)   = .true.
                 Bdone(Jdo+1) = .true.
               END IF
          BAArmsdClass(Juse) = BAArmsdClass(Juse) + Dvalue*Dvalue
          BAATotalType(Juse) = BAATotalType(Juse) + 1
      WRITE (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +   'BAA',Juse,Wmodel,Dist,Dvalue,Klinker,
     +   'CYS',Wchain,Wseq,Wins,Wname(Kdo),Walt(Kdo),
     +   'CYS',Wchain,Wseq,Wins,Wname(Ldo),Walt(Ldo),
     +   '   ',' ',0,' ','    ',' '
               GO TO 110
              END IF
             END IF
            END IF
           END IF
          ENDDO
         ENDIF
        END IF
       ENDDO
       END IF
 110   CONTINUE
      ENDDO
c
c---- angles CYS/S-S angles
c
      DO Jdo=1,NumAngleTarget(Wcys)
        IF (.not. Adone(Jdo)) THEN
       DO Kdo=1,Watoms
        IF (Wname(Kdo) .eq. AngleTriples(1,Wcys,Jdo))THEN
         IF (xyz(1,Kdo) .gt. MissingValue) THEN
          DO Ldo=1,Watoms
           IF (Ldo.ne.Kdo) THEN
            IF(Wname(Ldo) .eq. AngleTriples(2,Wcys,Jdo))THEN
             IF (Walt(Ldo) .eq. Walt(Kdo) ) THEN
              IF (xyz(1,Ldo) .gt. MissingValue) THEN
               DO Qdo=1,Watoms
                IF (Qdo.ne.Ldo .and. Qdo.ne.Kdo) THEN
                 IF (Wname(Qdo) .eq. AngleTriples(3,Wcys,Jdo))THEN
                  IF (Walt(Qdo) .eq. Walt(Kdo) ) THEN
                   Wangle = MissingValue
                   Dist1 = 0.0d0
                   Dist2 = 0.0d0
                   Dist3 = 0.0d0
                   DO Ndo = 1,3
                    Dif1 = xyz(Ndo,Ldo) - xyz(Ndo,Kdo)
                    Dif2 = xyz(Ndo,Qdo) - xyz(Ndo,Ldo)
                    Dif3 = xyz(Ndo,Kdo) - xyz(Ndo,Qdo)
                    Dist1 = Dif1*Dif1 + Dist1
                    Dist2 = Dif2*Dif2 + Dist2
                    Dist3 = Dif3*Dif3 + Dist3
                   ENDDO
                   IF (Dist1.gt.0.0d0 .and.
     +                 Dist2.gt.0.0d0 .and.
     +                 Dist3.ge.0.0d0) THEN
                    Dist1 = sqrt(Dist1)
                    Dist2 = sqrt(Dist2)
                    Dist3 = sqrt(Dist3)
                    COSANG = (Dist3*Dist3-
     +                        Dist1*Dist1-
     +                        Dist2*Dist2)/
     +                       (-2.0d0*Dist1*Dist2)
                    IF (ABS(COSANG).le.1.0d0)
     +               Wangle = ACOS(COSANG)*RADDEG
c
c
                    IF (Wangle .gt. MissingValue+1.0d0) THEN
c
c CYS      CA     CB     SG      114.2      1.1   51  "SH"
c CYS      CA     CB     SG      114.0      1.8  240  "SS"
c
           IF (AngleDescr(Wcys,Jdo)(1:1) .eq. '.') THEN
            Juse       = AngleTargetClass(Wcys,Jdo)
            Dvalue     = Wangle - AngleTargetValues(Wcys,Jdo)
            Adone(Jdo) = .true.
           ELSE IF (AngleDescr(Wcys,Jdo)(1:2) .eq. 'SH' .and.
     +              Disulphide .eq. 100) THEN
             Juse         = AngleTargetClass(Wcys,Jdo+1)
             Dvalue       = Wangle - AngleTargetValues(Wcys,Jdo+1)
             Adone(Jdo)   = .true.
             Adone(Jdo+1) = .true.
           ELSE IF (AngleDescr(Wcys,Jdo)(1:2) .eq. 'SH' .and.
     +              Disulphide .ne. 100) THEN
             Juse         = AngleTargetClass(Wcys,Jdo)
             Dvalue       = Wangle - AngleTargetValues(Wcys,Jdo)
             Adone(Jdo)   = .true.
             Adone(Jdo+1) = .true.
           END IF
           AAArmsdClass(Juse) = AAArmsdClass(Juse) + Dvalue*Dvalue
           AAATotalType(Juse) = AAATotalType(Juse) + 1
      WRITE (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +   'AAA',Juse,Wmodel,Wangle,Dvalue,Klinker,
     +   'CYS',Wchain,Wseq,Wins,Wname(Kdo),Walt(Kdo),
     +   'CYS',Wchain,Wseq,Wins,Wname(Ldo),Walt(Ldo),
     +   'CYS',Wchain,Wseq,Wins,Wname(Qdo),Walt(Qdo)
                  END IF
                  END IF
                  GO TO 120
                  END IF
                 END IF
                END IF
               ENDDO
              END IF
             END IF
            END IF
           END IF
          ENDDO
         END IF
        END IF
       ENDDO
 120   CONTINUE
       END IF
      ENDDO
c
c
      RETURN
      END
c
c      ========================================================
       SUBROUTINE Proline(Wpro,Watoms,Wname1,xyz1,Walt1,
     +                       BAArmsdClass,BAATotalType,
     +                       AAArmsdClass,AAATotalType,
     +                       Wres,Wchain,Wseq,Wins,
     +                       Wmodel,CisTrans,Bdone,Adone)
c     =========================================================
c
c  expect
cMaxStandard    37
cNaa            24
cNna            37
cNumBondLinker   4
cNumAngleLinker 11
cMaxAngleTarget 56
cMaxBondTarget  36
cMaxPlanesAtoms 12
cMaxBondLinker   4
cMaxAngleLinker 11
c
cclasses bonds  - 200  aa 1-73, na 100-194
cclasses angles - 250  aa 1-79, na 10-213
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
       LOGICAL Bdone(MaxBondTarget),Adone(MaxAngleTarget)
       INTEGER Wpro,Watoms,Wseq,Wmodel,Klinker,CisTrans
       CHARACTER Wname1(MaxAtoms)*4,Walt1(MaxAtoms)*1,Wres*3,
     +           Wchain*1,Wins*1
       DOUBLE PRECISION xyz1(3,MaxAtoms)
      INTEGER  BAATotalType(MaxBclass+MaxBondLinker),
     +         AAATotalType(MaxAclass+MaxAngleLinker)
      DOUBLE PRECISION BAArmsdClass(MaxBclass+MaxBondLinker),
     +                 AAArmsdClass(MaxAclass+MaxAngleLinker)
c
c
      INTEGER NNtype,N1N9,Natoms,NumAtmStanPlane,
     +        NumBondTarget,BondTargetClass,NumAngleTarget,
     +        AngleTargetClass,NumBondLinker,AAflag,
     +        NumAngleLinker
      CHARACTER StanRes3*3,stanNewNam*4,
     +          PlaneNames*4,
     +          BondPairs*4,AngleTriples*4,
     +          BondLinkerClass*12,BondLinkAtom*4,
     +          AngleLinkerClass*12,
     +          AngleLinkAtom*4,BondDescr*10,
     +          AngleDescr*10
      DOUBLE PRECISION BondTargetValues,BondTargetEsds,
     +                 AngleTargetValues,AngleTargetEsds,
     +                 BondLinkerValues,BondLinkerEsds,
     +                 AngleLinkerValues,AngleLinkerEsds
      COMMON /Cstan/ StanRes3(MaxStandard),
     +       stanNewNam(MaxStandard,MaxAtoms),
     +       PlaneNames(MaxStandard,MaxPlanesAtoms),
     +       BondPairs(2,MaxStandard,MaxBondTarget),
     +       AngleTriples(3,MaxStandard,MaxAngleTarget),
     +       BondLinkerClass(MaxBondLinker),
     +       BondLinkAtom(2,MaxBondLinker),
     +       AngleLinkerClass(MaxAngleLinker),
     +       AngleLinkAtom(3,MaxAngleLinker),
     +       BondDescr(MaxStandard,MaxBondTarget),
     +       AngleDescr(MaxStandard,MaxAngleTarget)
      COMMON /Dstan/
     +       BondTargetClass(MaxStandard,MaxBondTarget),
     +       BondTargetValues(MaxStandard,MaxBondTarget),
     +       BondTargetEsds(MaxStandard,MaxBondTarget),
     +       AngleTargetClass(MaxStandard,MaxAngleTarget),
     +       AngleTargetValues(MaxStandard,MaxAngleTarget),
     +       AngleTargetEsds(MaxStandard,MaxAngleTarget),
     +       BondLinkerValues(MaxBondLinker),
     +       BondLinkerEsds(MaxBondLinker),
     +       AngleLinkerValues(MaxAngleLinker),
     +       AngleLinkerEsds(MaxAngleLinker)
      COMMON /Istan/ NNtype(MaxStandard),
     +       N1N9(MaxStandard),
     +       Natoms(MaxStandard),
     +       NumAtmStanPlane(MaxStandard),
     +       NumBondTarget(MaxStandard),
     +       NumAngleTarget(MaxStandard),
     +       AAflag(MaxStandard),
     +       NumBondLinker,NumAngleLinker
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
       INTEGER Jdo,Kdo,Ldo,Ndo,Juse,Qdo
       DOUBLE PRECISION Dist,Dvalue,Wangle,Dist1,Dist2,
     +                  Dist3,Dif1,Dif2,Dif3,COSANG,RADDEG,PI
c
c
c
       Klinker = 1
       PI     = ATAN(1.0d0)*4.0d0
       RADDEG = 180.0d0/PI
c
c
         DO Jdo=1,MaxBondTarget
          Bdone(Jdo) = .false.
         ENDDO
         DO Jdo=1,MaxAngleTarget
          Adone(Jdo)= .false.
         ENDDO
c
c---- bonds -- check if cis/trans
c                CisTrans = 1 if cis
c
      DO Jdo=1,NumBondTarget(Wpro)
       IF ( .not. Bdone(Jdo)) THEN
       DO Kdo=1,Watoms
        IF (Wname1(Kdo) .eq. BondPairs(1,Wpro,Jdo))THEN
         IF (xyz1(1,Kdo) .gt. MissingValue) THEN
          DO Ldo=1,Watoms
           IF (Ldo.ne.Kdo) THEN
            IF(Wname1(Ldo) .eq. BondPairs(2,Wpro,Jdo))THEN
             IF (Walt1(Ldo) .eq. Walt1(Kdo) ) THEN
              IF (xyz1(1,Ldo) .gt. MissingValue) THEN
               Dist = 0.0d0
               DO Ndo = 1,3
                Dist = ( xyz1(Ndo,Kdo) - xyz1(Ndo,Ldo) )**2 + Dist
               ENDDO
               Dist   = sqrt (Dist)
c
cPRO    CA    CB     SING N N  5    1.531    0.020    2 "trans"
cPRO    CA    CB     SING N N  5    1.533    0.018  100 "cis"
c
               IF (BondDescr(Wpro,Jdo)(1:1) .eq. '.') THEN
                Juse       = BondTargetClass(Wpro,Jdo)
                Dvalue     = Dist - BondTargetValues(Wpro,Jdo)
                Bdone(Jdo) = .true.
               ELSE IF (BondDescr(Wpro,Jdo)(1:5) .eq. 'trans' .and.
     +           CisTrans .eq. 1) THEN
                 Juse       = BondTargetClass(Wpro,Jdo+1)
                 Dvalue     = Dist - BondTargetValues(Wpro,Jdo+1)
                 Bdone(Jdo) = .true.
                 Bdone(Jdo+1) = .true.
               ELSE IF (BondDescr(Wpro,Jdo)(1:5) .eq. 'trans' .and.
     +           CisTrans .ne. 1) THEN
                 Juse         = BondTargetClass(Wpro,Jdo)
                 Dvalue       = Dist - BondTargetValues(Wpro,Jdo)
                 Bdone(Jdo)   = .true.
                 Bdone(Jdo+1) = .true.
               END IF
               BAArmsdClass(Juse) = BAArmsdClass(Juse) + Dvalue*Dvalue
               BAATotalType(Juse) = BAATotalType(Juse) + 1
      WRITE (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +   'BAA',Juse,Wmodel,Dist,Dvalue,Klinker,
     +   Wres,Wchain,Wseq,Wins,Wname1(Kdo),Walt1(Kdo),
     +   Wres,Wchain,Wseq,Wins,Wname1(Ldo),Walt1(Ldo),
     +   '   ',' ',0,' ','    ',' '
               GO TO 110
              END IF
             END IF
            END IF
           END IF
          ENDDO
         ENDIF
        END IF
       ENDDO
 110   CONTINUE
       END IF
      ENDDO
c
c---- angles
c
c
      DO Jdo=1,NumAngleTarget(Wpro)
       IF (.not. Adone(Jdo)) THEN
       DO Kdo=1,Watoms
        IF (Wname1(Kdo) .eq. AngleTriples(1,Wpro,Jdo))THEN
         IF (xyz1(1,Kdo) .gt. MissingValue) THEN
          DO Ldo=1,Watoms
           IF (Ldo.ne.Kdo) THEN
            IF(Wname1(Ldo) .eq. AngleTriples(2,Wpro,Jdo))THEN
             IF (Walt1(Ldo) .eq. Walt1(Kdo) ) THEN
              IF (xyz1(1,Ldo) .gt. MissingValue) THEN
               DO Qdo=1,Watoms
                IF (Qdo.ne.Ldo .and. Qdo.ne.Kdo) THEN
                 IF (Wname1(Qdo) .eq. AngleTriples(3,Wpro,Jdo))THEN
                  IF (Walt1(Qdo) .eq. Walt1(Kdo) ) THEN
                   Wangle = MissingValue
                   Dist1 = 0.0d0
                   Dist2 = 0.0d0
                   Dist3 = 0.0d0
                   DO Ndo = 1,3
                    Dif1 = xyz1(Ndo,Ldo) - xyz1(Ndo,Kdo)
                    Dif2 = xyz1(Ndo,Qdo) - xyz1(Ndo,Ldo)
                    Dif3 = xyz1(Ndo,Kdo) - xyz1(Ndo,Qdo)
                    Dist1 = Dif1*Dif1 + Dist1
                    Dist2 = Dif2*Dif2 + Dist2
                    Dist3 = Dif3*Dif3 + Dist3
                   ENDDO
                   IF (Dist1.gt.0.0d0 .and.
     +                 Dist2.gt.0.0d0 .and.
     +                 Dist3.ge.0.0d0) THEN
                    Dist1 = sqrt(Dist1)
                    Dist2 = sqrt(Dist2)
                    Dist3 = sqrt(Dist3)
                    COSANG = (Dist3*Dist3-
     +                        Dist1*Dist1-
     +                        Dist2*Dist2)/
     +                       (-2.0d0*Dist1*Dist2)
                    IF (ABS(COSANG).le.1.0d0)
     +               Wangle = ACOS(COSANG)*RADDEG
                    IF (Wangle .gt. MissingValue+1.0d0) THEN
c
c PRO      CA     CB     CG      104.8      1.9    5  "trans"
c PRO      CA     CB     CG      104.0      1.9   20  "cis"
c
               IF (AngleDescr(Wpro,Jdo)(1:1) .eq. '.') THEN
                Juse       = AngleTargetClass(Wpro,Jdo)
                Dvalue     = Wangle - AngleTargetValues(Wpro,Jdo)
                Adone(Jdo) = .true.
               ELSE IF (AngleDescr(Wpro,Jdo)(1:5) .eq. 'trans' .and.
     +                  CisTrans .eq. 1) THEN
                 Juse         = AngleTargetClass(Wpro,Jdo+1)
                 Dvalue       = Wangle - AngleTargetValues(Wpro,Jdo+1)
                 Adone(Jdo)   = .true.
                 Adone(Jdo+1) = .true.
               ELSE IF (AngleDescr(Wpro,Jdo)(1:3) .eq. 'trans' .and.
     +                  CisTrans .ne. 1) THEN
                 Juse         = AngleTargetClass(Wpro,Jdo)
                 Dvalue       = Wangle - AngleTargetValues(Wpro,Jdo)
                 Adone(Jdo)   = .true.
                 Adone(Jdo+1) = .true.
               END IF
              AAArmsdClass(Juse) = AAArmsdClass(Juse) + Dvalue*Dvalue
              AAATotalType(Juse) = AAATotalType(Juse) + 1
      WRITE (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +   'AAA',Juse,Wmodel,Wangle,Dvalue,Klinker,
     +   Wres,Wchain,Wseq,Wins,Wname1(Kdo),Walt1(Kdo),
     +   Wres,Wchain,Wseq,Wins,Wname1(Ldo),Walt1(Ldo),
     +   Wres,Wchain,Wseq,Wins,Wname1(Qdo),Walt1(Qdo)
                  END IF
                  END IF
                  GO TO 120
                  END IF
                 END IF
                END IF
               ENDDO
              END IF
             END IF
            END IF
           END IF
          ENDDO
         END IF
        END IF
       ENDDO
       END IF
 120   CONTINUE
      ENDDO
c
c
      RETURN
      END
c
c      ========================================================
       SUBROUTINE Histidine(Whis,Watoms,Wname1,xyz1,Walt1,
     +                       BAArmsdClass,BAATotalType,
     +                       AAArmsdClass,AAATotalType,
     +                       Wres,Wchain,Wseq,Wins,
     +                       Wmodel,Idummy,Bdone,Adone,
     +                       Nhy,Hxyz)
c     =========================================================
c
      implicit none
c
c
      INTEGER MaxBondTarget,MaxAngleTarget,MaxStandard,
     +        MaxAlt,MaxAtoms,MaxPlanesAtoms,
     +        MaxBondLinker,MaxAngleLinker,MaxBclass,
     +        MaxAclass
      DOUBLE PRECISION MissingValue
      PARAMETER (MaxStandard    =  40)
      PARAMETER (MaxBondTarget  =  50)
      PARAMETER (MaxAngleTarget =  60)
      PARAMETER (MaxAlt         =   4)
      PARAMETER (MaxAtoms       =  60)
      PARAMETER (MaxPlanesAtoms =  25)
      PARAMETER (MaxBondLinker  =   5)
      PARAMETER (MaxAngleLinker =  18)
      PARAMETER (MaxBclass      = 260)
      PARAMETER (MaxAclass      = 270)
      PARAMETER (MissingValue   = -999999.9D0)
c
c
      LOGICAL Bdone(MaxBondTarget),Adone(MaxAngleTarget)
      INTEGER Whis,Watoms,Wseq,Wmodel,Klinker,Idummy,Nhy,
     +         Kne,Knd,BNE,BND,Khis
      CHARACTER Wname1(MaxAtoms)*4,Walt1(MaxAtoms)*1,Wres*3,
     +           Wchain*1,Wins*1
      DOUBLE PRECISION xyz1(3,MaxAtoms),Hxyz(3,MaxAtoms),
     +                 DHlim
      INTEGER  BAATotalType(MaxBclass+MaxBondLinker),
     +         AAATotalType(MaxAclass+MaxAngleLinker)
      DOUBLE PRECISION BAArmsdClass(MaxBclass+MaxBondLinker),
     +                 AAArmsdClass(MaxAclass+MaxAngleLinker)
c
c
      INTEGER NNtype,N1N9,Natoms,NumAtmStanPlane,
     +        NumBondTarget,BondTargetClass,NumAngleTarget,
     +        AngleTargetClass,NumBondLinker,AAflag,
     +        NumAngleLinker
      CHARACTER StanRes3*3,stanNewNam*4,
     +          PlaneNames*4,
     +          BondPairs*4,AngleTriples*4,
     +          BondLinkerClass*12,BondLinkAtom*4,
     +          AngleLinkerClass*12,
     +          AngleLinkAtom*4,BondDescr*10,
     +          AngleDescr*10
      DOUBLE PRECISION BondTargetValues,BondTargetEsds,
     +                 AngleTargetValues,AngleTargetEsds,
     +                 BondLinkerValues,BondLinkerEsds,
     +                 AngleLinkerValues,AngleLinkerEsds
      COMMON /Cstan/ StanRes3(MaxStandard),
     +       stanNewNam(MaxStandard,MaxAtoms),
     +       PlaneNames(MaxStandard,MaxPlanesAtoms),
     +       BondPairs(2,MaxStandard,MaxBondTarget),
     +       AngleTriples(3,MaxStandard,MaxAngleTarget),
     +       BondLinkerClass(MaxBondLinker),
     +       BondLinkAtom(2,MaxBondLinker),
     +       AngleLinkerClass(MaxAngleLinker),
     +       AngleLinkAtom(3,MaxAngleLinker),
     +       BondDescr(MaxStandard,MaxBondTarget),
     +       AngleDescr(MaxStandard,MaxAngleTarget)
      COMMON /Dstan/
     +       BondTargetClass(MaxStandard,MaxBondTarget),
     +       BondTargetValues(MaxStandard,MaxBondTarget),
     +       BondTargetEsds(MaxStandard,MaxBondTarget),
     +       AngleTargetClass(MaxStandard,MaxAngleTarget),
     +       AngleTargetValues(MaxStandard,MaxAngleTarget),
     +       AngleTargetEsds(MaxStandard,MaxAngleTarget),
     +       BondLinkerValues(MaxBondLinker),
     +       BondLinkerEsds(MaxBondLinker),
     +       AngleLinkerValues(MaxAngleLinker),
     +       AngleLinkerEsds(MaxAngleLinker)
      COMMON /Istan/ NNtype(MaxStandard),
     +       N1N9(MaxStandard),
     +       Natoms(MaxStandard),
     +       NumAtmStanPlane(MaxStandard),
     +       NumBondTarget(MaxStandard),
     +       NumAngleTarget(MaxStandard),
     +       AAflag(MaxStandard),
     +       NumBondLinker,NumAngleLinker
c
c
      INTEGER Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
      COMMON /Cio/Istan,Iscr,Ipdb,Irmsd,Irout,Icif,Icis,Idump
c
c
       INTEGER Jdo,Kdo,Ldo,Ndo,Juse,Qdo
       DOUBLE PRECISION Dist,Dvalue,Wangle,Dist1,Dist2,
     +                  Dist3,Dif1,Dif2,Dif3,COSANG,RADDEG,PI
c
c
c
       Klinker = 1
       PI      = ATAN(1.0d0)*4.0d0
       RADDEG  = 180.0d0/PI
       DHlim   = 1.1d0*1.1d0
c
c
         DO Jdo=1,MaxBondTarget
          Bdone(Jdo) = .false.
         ENDDO
         DO Jdo=1,MaxAngleTarget
          Adone(Jdo)= .false.
         ENDDO
c
c---- need to determine if HIS is protonated
c     NE and ND ? - if any hydrogens
c                   or if the nearest ?
c
        Kne = 0
        Knd = 0
        BNE = 0
        BND = 0
c
c
        IF (Nhy .gt. 0) THEN
         DO Jdo=1,Watoms
          IF (Wname1(Jdo) .eq. ' NE2') THEN
             Kne = Jdo
          ELSE IF (Wname1(Jdo) .eq. ' ND1') THEN
             Knd = Jdo
          END IF
         ENDDO
         IF (Kne .ne. 0) THEN
          DO Jdo=1,Nhy
           Dist = 0.0d0
           DO Ndo=1,3
            Dist = (xyz1(Ndo,Kne) - Hxyz(Ndo,Ldo) )**2 + Dist
           ENDDO
           IF (Dist .lt. DHlim) BNE= BNE + 1
          ENDDO
         END IF
         IF (Knd .ne. 0) THEN
          DO Jdo=1,Nhy
           Dist = 0.0d0
           DO Ndo=1,3
            Dist = (xyz1(Ndo,Knd) - Hxyz(Ndo,Ldo) )**2 + Dist
           ENDDO
           IF (Dist .lt. DHlim) BND= BND + 1
          ENDDO
         END IF
        END IF
c
c
        Khis = 1
        IF (BND .gt. 0 .and. BNE .gt. 0) THEN
           Khis = 3
        ELSE IF (BND .gt. 0) THEN
           Khis = 2
        END IF
c
c---- bonds -- check which HIS
c
      DO Jdo=1,NumBondTarget(Whis)
       IF ( .not. Bdone(Jdo)) THEN
       DO Kdo=1,Watoms
        IF (Wname1(Kdo) .eq. BondPairs(1,Whis,Jdo))THEN
         IF (xyz1(1,Kdo) .gt. MissingValue) THEN
          DO Ldo=1,Watoms
           IF (Ldo.ne.Kdo) THEN
            IF(Wname1(Ldo) .eq. BondPairs(2,Whis,Jdo))THEN
             IF (Walt1(Ldo) .eq. Walt1(Kdo) ) THEN
              IF (xyz1(1,Ldo) .gt. MissingValue) THEN
               Dist = 0.0d0
               DO Ndo = 1,3
                Dist = (xyz1(Ndo,Kdo) - xyz1(Ndo,Ldo) )**2 + Dist
               ENDDO
               Dist   = sqrt (Dist)
c
cHIS    ND1   CE1    DOUB Y N 14    1.323    0.015   10 "HISE"
cHIS    ND1   CE1    DOUB Y N 14    1.343    0.025  211 "HISD"
cHIS    ND1   CE1    DOUB Y N 14    1.326    0.010  212 "HISH"
c
               IF (BondDescr(Whis,Jdo)(1:1) .eq. '.') THEN
                Juse       = BondTargetClass(Whis,Jdo)
                Dvalue     = Dist - BondTargetValues(Whis,Jdo)
                Bdone(Jdo) = .true.
               ELSE IF (BondDescr(Whis,Jdo)(1:4) .eq. 'HISE' .and.
     +           Khis .eq. 1) THEN
                 Juse         = BondTargetClass(Whis,Jdo+1)
                 Dvalue       = Dist - BondTargetValues(Whis,Jdo+1)
                 Bdone(Jdo)   = .true.
                 Bdone(Jdo+1) = .true.
                 Bdone(Jdo+2) = .true.
               ELSE IF (BondDescr(Whis,Jdo)(1:4) .eq. 'HISE' .and.
     +           Khis .eq. 2) THEN
                 Juse         = BondTargetClass(Whis,Jdo+1)
                 Dvalue       = Dist - BondTargetValues(Whis,Jdo+1)
                 Bdone(Jdo)   = .true.
                 Bdone(Jdo+1) = .true.
                 Bdone(Jdo+2) = .true.
               ELSE IF (BondDescr(Whis,Jdo)(1:4) .eq. 'HISE' .and.
     +           Khis .eq. 3) THEN
                 Juse         = BondTargetClass(Whis,Jdo+2)
                 Dvalue       = Dist - BondTargetValues(Whis,Jdo+2)
                 Bdone(Jdo)   = .true.
                 Bdone(Jdo+1) = .true.
                 Bdone(Jdo+2) = .true.
               END IF
               BAArmsdClass(Juse) = BAArmsdClass(Juse) + Dvalue*Dvalue
               BAATotalType(Juse) = BAATotalType(Juse) + 1
      WRITE (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +   'BAA',Juse,Wmodel,Dist,Dvalue,Klinker,
     +   Wres,Wchain,Wseq,Wins,Wname1(Kdo),Walt1(Kdo),
     +   Wres,Wchain,Wseq,Wins,Wname1(Ldo),Walt1(Ldo),
     +   '   ',' ',0,' ','    ',' '
               GO TO 110
              END IF
             END IF
            END IF
           END IF
          ENDDO
         ENDIF
        END IF
       ENDDO
 110   CONTINUE
      END IF
      ENDDO
c
c---- angles
c
c
      DO Jdo=1,NumAngleTarget(Whis)
       IF ( .not. Adone(Jdo)) THEN
       DO Kdo=1,Watoms
        IF (Wname1(Kdo) .eq. AngleTriples(1,Whis,Jdo))THEN
         IF (xyz1(1,Kdo) .gt. MissingValue) THEN
          DO Ldo=1,Watoms
           IF (Ldo.ne.Kdo) THEN
            IF(Wname1(Ldo) .eq. AngleTriples(2,Whis,Jdo))THEN
             IF (Walt1(Ldo) .eq. Walt1(Kdo) ) THEN
              IF (xyz1(1,Ldo) .gt. MissingValue) THEN
               DO Qdo=1,Watoms
                IF (Qdo.ne.Ldo .and. Qdo.ne.Kdo) THEN
                 IF (Wname1(Qdo) .eq. AngleTriples(3,Whis,Jdo))THEN
                  IF (Walt1(Qdo) .eq. Walt1(Kdo) ) THEN
                   Wangle = MissingValue
                   Dist1 = 0.0d0
                   Dist2 = 0.0d0
                   Dist3 = 0.0d0
                   DO Ndo = 1,3
                    Dif1 = xyz1(Ndo,Ldo) - xyz1(Ndo,Kdo)
                    Dif2 = xyz1(Ndo,Qdo) - xyz1(Ndo,Ldo)
                    Dif3 = xyz1(Ndo,Kdo) - xyz1(Ndo,Qdo)
                    Dist1 = Dif1*Dif1 + Dist1
                    Dist2 = Dif2*Dif2 + Dist2
                    Dist3 = Dif3*Dif3 + Dist3
                   ENDDO
                   IF (Dist1.gt.0.0d0 .and.
     +                 Dist2.gt.0.0d0 .and.
     +                 Dist3.ge.0.0d0) THEN
                    Dist1 = sqrt(Dist1)
                    Dist2 = sqrt(Dist2)
                    Dist3 = sqrt(Dist3)
                    COSANG = (Dist3*Dist3-
     +                        Dist1*Dist1-
     +                        Dist2*Dist2)/
     +                       (-2.0d0*Dist1*Dist2)
                    IF (ABS(COSANG).le.1.0d0)
     +               Wangle = ACOS(COSANG)*RADDEG
                    IF (Wangle .gt. MissingValue+1.0d0) THEN
c
c HIS      CA     CB     CG      113.6      1.7   47  "HISE"
c HIS      CA     CB     CG      113.6      1.7   47  "HISD"
c HIS      CA     CB     CG      113.6      1.6  241  "HISH"
c
               IF (AngleDescr(Whis,Jdo)(1:1) .eq. '.') THEN
                Juse       = AngleTargetClass(Whis,Jdo)
                Dvalue     = Wangle - AngleTargetValues(Whis,Jdo)
                Adone(Jdo) = .true.
               ELSE IF (AngleDescr(Whis,Jdo)(1:4) .eq. 'HISE' .and.
     +                  Khis .eq. 1) THEN
                 Juse         = AngleTargetClass(Whis,Jdo)
                 Dvalue       = Wangle - AngleTargetValues(Whis,Jdo)
                 Adone(Jdo)   = .true.
                 Adone(Jdo+1) = .true.
                 Adone(Jdo+2) = .true.
               ELSE IF (AngleDescr(Whis,Jdo)(1:4) .eq. 'HISE' .and.
     +                  Khis .eq. 2) THEN
                 Juse         = AngleTargetClass(Whis,Jdo+1)
                 Dvalue       = Wangle - AngleTargetValues(Whis,Jdo+1)
                 Adone(Jdo)   = .true.
                 Adone(Jdo+1) = .true.
                 Adone(Jdo+2) = .true.
               ELSE IF (AngleDescr(Whis,Jdo)(1:4) .eq. 'HISE' .and.
     +                  Khis .eq. 3) THEN
                 Juse         = AngleTargetClass(Whis,Jdo+2)
                 Dvalue       = Wangle - AngleTargetValues(Whis,Jdo+2)
                 Adone(Jdo)   = .true.
                 Adone(Jdo+1) = .true.
                 Adone(Jdo+2) = .true.
               END IF
              AAArmsdClass(Juse) = AAArmsdClass(Juse) + Dvalue*Dvalue
              AAATotalType(Juse) = AAATotalType(Juse) + 1
      WRITE (Iscr,
     + fmt='(a3,2x,I6,2x,I4,2x,f10.4,2x,f10.4,2x,i5,2x,
     +       3(a3,2x,a1,2x,i5,2x,a1,2x,a4,2x,a1,2x))')
     +   'AAA',Juse,Wmodel,Wangle,Dvalue,Klinker,
     +   Wres,Wchain,Wseq,Wins,Wname1(Kdo),Walt1(Kdo),
     +   Wres,Wchain,Wseq,Wins,Wname1(Ldo),Walt1(Ldo),
     +   Wres,Wchain,Wseq,Wins,Wname1(Qdo),Walt1(Qdo)
                  END IF
                  END IF
                  GO TO 120
                  END IF
                 END IF
                END IF
               ENDDO
              END IF
             END IF
            END IF
           END IF
          ENDDO
         END IF
        END IF
       ENDDO
 120   CONTINUE
       END IF
      ENDDO
c
c
      RETURN
      END
c
c---- Sample cispep file for 2j0k
c
c CISPEP   1 THR A  284    ASP A  285          0        15.64
c CISPEP   2 ASP A  285    LYS A  286          0       -17.48
c CISPEP   3 ASN A  493    PRO A  494          0        -6.46
c CISPEP   4 ASP B  285    LYS B  286          0       -14.41
c CISPEP   5 LYS B  286    GLY B  287          0         8.65
c CISPEP   6 GLY B  383    VAL B  384          0        -5.75
c CISPEP   7 ASN B  493    PRO B  494          0         4.93
c
c---- Sample PDB output
c
c   Template
c
c            1         2         3         4         5         6         7
c   1234567890123456789012345678901234567890123456789012345678901234567890
c   REMARK 500
c   REMARK 500 GEOMETRY AND STEREOCHEMISTRY
c   REMARK 500 SUBTOPIC:
c   REMARK 500
c   REMARK 500 FREE TEXT GOES HERE.
c
c
c----------------------------------------------------------------
c
c REMARK 500
c REMARK 500 GEOMETRY AND STEREOCHEMISTRY
c REMARK 500 SUBTOPIC: CLOSE CONTACTS IN SAME ASYMMETRIC UNIT
c REMARK 500
c REMARK 500 THE FOLLOWING ATOMS ARE IN CLOSE CONTACT.
c REMARK 500
c REMARK 500  ATM1  RES C  SSEQI   ATM2  RES C  SSEQI           DISTANCE
c REMARK 500   N    PHE 1     8  -  OD2  ASP 1    31              2.17
c REMARK 500   OD2  ASP 1    31  -  N    PHE 1     8              2.17
c REMARK 500
c REMARK 500 THIS ENTRY HAS     104 CLOSE CONTACTS
c REMARK 500
c REMARK 500 REMARK: NULL
c
c----------------------------------------------------------------
c
c REMARK 500
c REMARK 500 GEOMETRY AND STEREOCHEMISTRY
c REMARK 500 SUBTOPIC: CLOSE CONTACTS
c REMARK 500
c REMARK 500 THE FOLLOWING ATOMS THAT ARE RELATED BY CRYSTALLOGRAPHIC
c REMARK 500 SYMMETRY ARE IN CLOSE CONTACT.  AN ATOM LOCATED WITHIN 0.15 
c REMARK 500 ANGSTROMS OF A SYMMETRY RELATED ATOM IS ASSUMED TO BE ON A 
c REMARK 500 SPECIAL POSITION AND IS, THEREFORE, LISTED IN REMARK 375 
c REMARK 500 INSTEAD OF REMARK 500.  ATOMS WITH NON-BLANK ALTERNATE
c REMARK 500 LOCATION INDICATORS ARE NOT INCLUDED IN THE CALCULATIONS.
c REMARK 500
c REMARK 500 DISTANCE CUTOFF: 
c REMARK 500 2.2 ANGSTROMS FOR CONTACTS NOT INVOLVING HYDROGEN ATOMS
c REMARK 500 1.6 ANGSTROMS FOR CONTACTS INVOLVING HYDROGEN ATOMS
c REMARK 500
c REMARK 500  ATM1  RES C  SSEQI   ATM2  RES C  SSEQI  SSYMOP   DISTANCE
c REMARK 500
c REMARK 500   OH   TYR 1    90     O    HOH     343     1566      2.09
c REMARK 500   OE1  GLU 1   134     CB   LYS 2   135     1556      2.18
c REMARK 500
c REMARK 500 THIS ENTRY HAS      64 SYMMETRY CONTACTS
c REMARK 500
c REMARK 500 REMARK: NULL
c
c----------------------------------------------------------------
c
c REMARK 500
c REMARK 500 GEOMETRY AND STEREOCHEMISTRY
c REMARK 500 SUBTOPIC: COVALENT BOND LENGTHS
c REMARK 500
c REMARK 500 THE STEREOCHEMICAL PARAMETERS OF THE FOLLOWING RESIDUES
c REMARK 500 HAVE VALUES WHICH DEVIATE FROM EXPECTED VALUES BY MORE
c REMARK 500 THAN 6*RMSD (M=MODEL NUMBER; RES=RESIDUE NAME; C=CHAIN
c REMARK 500 IDENTIFIER; SSEQ=SEQUENCE NUMBER; I=INSERTION CODE).
c REMARK 500
c REMARK 500 STANDARD TABLE:
c REMARK 500 FORMAT: (10X,I3,1X,A3,1X,A1,I4,A1,1X,2(A4,A1,3X),12X,F5.3)
c REMARK 500
c REMARK 500 EXPECTED VALUES PROTEIN: ENGH AND HUBER, 1999
c REMARK 500 EXPECTED VALUES NUCELIC ACID: CLOWNEY ET AL 1996
c REMARK 500
c REMARK 500  M RES CSSEQI ATM1   RES CSSEQI ATM2   DEVIATION
c REMARK 500    ASP B 117   CB    ASP B 117   CG     -0.129
c REMARK 500    CYS J  29   CB    CYS J  29   SG     -0.111
c REMARK 500
c REMARK 500 REMARK: NULL
c
c----------------------------------------------------------------
c
c REMARK 500
c REMARK 500 GEOMETRY AND STEREOCHEMISTRY
c REMARK 500 SUBTOPIC: COVALENT BOND ANGLES
c REMARK 500
c REMARK 500 THE STEREOCHEMICAL PARAMETERS OF THE FOLLOWING RESIDUES
c REMARK 500 HAVE VALUES WHICH DEVIATE FROM EXPECTED VALUES BY MORE
c REMARK 500 THAN 6*RMSD (M=MODEL NUMBER; RES=RESIDUE NAME; C=CHAIN
c REMARK 500 IDENTIFIER; SSEQ=SEQUENCE NUMBER; I=INSERTION CODE).
c REMARK 500
c REMARK 500 STANDARD TABLE:
c REMARK 500 FORMAT: (10X,I3,1X,A3,1X,A1,I4,A1,3(1X,A4,2X),12X,F5.1)
c REMARK 500
c REMARK 500 EXPECTED VALUES: ENGH AND HUBER, 1999
c REMARK 500 EXPECTED VALUES NUCELIC ACID: CLOWNEY ET AL 1996
c REMARK 500
c REMARK 500  M RES CSSEQI ATM1   ATM2   ATM3
c REMARK 500    VAL A 124   CB  -  CA  -  C   ANGL. DEV. = -12.0 DEGREES
c REMARK 500    ARG B  70   NE  -  CZ  -  NH1 ANGL. DEV. =  -3.0 DEGREES
c REMARK 500
c REMARK 500 REMARK: NULL
c
c----------------------------------------------------------------
c
c REMARK 500
c REMARK 500 GEOMETRY AND STEREOCHEMISTRY
c REMARK 500 SUBTOPIC: TORSION ANGLES
c REMARK 500
c REMARK 500 TORSION ANGLES OUTSIDE THE EXPECTED RAMACHANDRAN REGIONS:
c REMARK 500 (M=MODEL NUMBER; RES=RESIDUE NAME; C=CHAIN IDENTIFIER;
c REMARK 500 SSEQ=SEQUENCE NUMBER; I=INSERTION CODE).
c REMARK 500 STANDARD TABLE:
c REMARK 500 FORMAT:(10X,I3,1X,A3,1X,A1,I4,A1,4X,F7.2,3X,F7.2)
c REMARK 500
c REMARK 500  M RES CSSEQI        PSI       PHI
c REMARK 500    ASN A 100     -110.87   -163.72
c REMARK 500    ILE A 166      -28.81    -31.64
c REMARK 500
c REMARK 500 REMARK: NULL
c
c----------------------------------------------------------------
c
c REMARK 500
c REMARK 500 GEOMETRY AND STEREOCHEMISTRY
c REMARK 500 SUBTOPIC: NON-CIS, NON-TRANS
c REMARK 500
c REMARK 500 THE FOLLOWING PEPTIDE BONDS DEVIATE SIGNIFICANTLY FROM BOTH
c REMARK 500 CIS AND TRANS CONFORMATION.  CIS BONDS, IF ANY, ARE LISTED
c REMARK 500 ON CISPEP RECORDS.  TRANS IS DEFINED AS 180 +/- 30 AND
c REMARK 500 CIS IS DEFINED AS 0 +/- 30 DEGREES.
c REMARK 500                                 MODEL     OMEGA
c REMARK 500 ARG A  413     ASP A  414          0       147.84
c REMARK 500 ALA B  288     ASN B  289          0       -39.12
c REMARK 500
c REMARK 500 REMARK: NULL
c
c----------------------------------------------------------------
c
c REMARK 500
c REMARK 500 GEOMETRY AND STEREOCHEMISTRY
c REMARK 500 SUBTOPIC: PLANAR GROUPS
c REMARK 500
c REMARK 500 PLANAR GROUPS IN THE FOLLOWING RESIDUES HAVE A TOTAL
c REMARK 500 RMS DISTANCE OF ALL ATOMS FROM THE BEST-FIT PLANE
c REMARK 500 BY MORE THAN AN EXPECTED VALUE OF 6*RMSD, WITH AN
c REMARK 500 RMSD 0.02 ANGSTROMS, OR AT LEAST ONE ATOM HAS
c REMARK 500 AN RMSD GREATER THAN THIS VALUE
c REMARK 500 (M=MODEL NUMBER; RES=RESIDUE NAME; C=CHAIN IDENTIFIER;
c REMARK 500 SSEQ=SEQUENCE NUMBER; I=INSERTION CODE).
c REMARK 500
c REMARK 500  M RES CSSEQI        RMS     TYPE
c REMARK 500    TYR A  36         0.08    SIDE CHAIN
c REMARK 500    TYR A 104         0.08    SIDE CHAIN
c REMARK 500
c REMARK 500 REMARK: NULL
c
c----------------------------------------------------------------
c
c REMARK 500
c REMARK 500 GEOMETRY AND STEREOCHEMISTRY
c REMARK 500 SUBTOPIC: MAIN CHAIN PLANARITY
c REMARK 500
c REMARK 500 THE FOLLOWING RESIDUES HAVE A PSEUDO PLANARITY
c REMARK 500 TORSION, C(I) - CA(I) - N(I+1) - O(I), GREATER
c REMARK 500 10.0 DEGREES. (M=MODEL NUMBER; RES=RESIDUE NAME;
c REMARK 500 C=CHAIN IDENTIFIER; SSEQ=SEQUENCE NUMBER; 
c REMARK 500 I=INSERTION CODE).
c REMARK 500
c REMARK 500  M RES CSSEQI        ANGLE
c REMARK 500
c REMARK 500 REMARK: NULL
c
c----------------------------------------------------------------
c
c REMARK 500
c REMARK 500 GEOMETRY AND STEREOCHEMISTRY
c REMARK 500 SUBTOPIC: CHIRAL CENTERS
c REMARK 500
c REMARK 500 UNEXPECTED CONFIGURATION OF THE FOLLOWING CHIRAL
c REMARK 500 CENTER(S) (M=MODEL NUMBER; RES=RESIDUE NAME; C=CHAIN
c REMARK 500 IDENTIFIER; SSEQ=SEQUENCE NUMBER; I=INSERTION CODE).
c REMARK 500
c REMARK 500 STANDARD TABLE:
c REMARK 500 FORMAT: (10X,I3,1X,A3,1X,A1,I4,A1,6X,A12)
c REMARK 500
c REMARK 500  M RES CSSEQI                    OMEGA
c REMARK 500    ASP B 405       ALPHA-CARBON 150.48 
c REMARK 500
c REMARK 500 REMARK: NULL
c
c----------------------------------------------------------------
c
c
c---- Sample mmCIF output
c
c
c #
c data_2j0k
c #
c #
c loop_
c _pdbx_validate_close_contact.id
c _pdbx_validate_close_contact.model_id
c _pdbx_validate_close_contact.auth_atom_id_1
c _pdbx_validate_close_contact.auth_asym_id_1
c _pdbx_validate_close_contact.auth_comp_id_1
c _pdbx_validate_close_contact.auth_seq_id_1
c _pdbx_validate_close_contact.insert_id_1
c _pdbx_validate_close_contact.alt_id_1
c _pdbx_validate_close_contact.auth_atom_id_2
c _pdbx_validate_close_contact.auth_asym_id_2
c _pdbx_validate_close_contact.auth_comp_id_2
c _pdbx_validate_close_contact.auth_seq_id_2
c _pdbx_validate_close_contact.insert_id_2
c _pdbx_validate_close_contact.alt_id_2
c _pdbx_validate_close_contact.dist
c    1     1     .  B  VAL     36  .  .       .  B  ARG    108  .  .         2.16
c    2     1     .  B  ARG    108  .  .       .  B  VAL     36  .  .         2.16
c  
c loop_
c _pdbx_validate_symm_contact.id
c _pdbx_validate_symm_contact.model_id
c _pdbx_validate_symm_contact.auth_atom_id_1
c _pdbx_validate_symm_contact.auth_asym_id_1
c _pdbx_validate_symm_contact.auth_comp_id_1
c _pdbx_validate_symm_contact.auth_seq_id_1
c _pdbx_validate_symm_contact.insert_id_1
c _pdbx_validate_symm_contact.alt_id_1
c _pdbx_validate_symm_contact.site_symmetry_1
c _pdbx_validate_symm_contact.auth_atom_id_2
c _pdbx_validate_symm_contact.auth_asym_id_2
c _pdbx_validate_symm_contact.auth_comp_id_2
c _pdbx_validate_symm_contact.auth_seq_id_2
c _pdbx_validate_symm_contact.insert_id_2
c _pdbx_validate_symm_contact.alt_id_2
c _pdbx_validate_symm_contact.site_symmetry_2
c _pdbx_validate_symm_contact.dist
c   1     1   O    .  HOH     70  .  .    1555   O    .  HOH     70  .  .    7555       2.05
c 
c 
c loop_
c _pdbx_validate_rmsd_bond.model_number
c _pdbx_validate_rmsd_bond.atom_site_auth_asym_id_1
c _pdbx_validate_rmsd_bond.atom_site_auth_atom_id_1
c _pdbx_validate_rmsd_bond.atom_site_auth_comp_id_1
c _pdbx_validate_rmsd_bond.atom_site_auth_seq_id_1
c _pdbx_validate_rmsd_bond.atom_site_auth_insert_id_1
c _pdbx_validate_rmsd_bond.atom_site_auth_altcode_1
c _pdbx_validate_rmsd_bond.atom_site_auth_asym_id_2
c _pdbx_validate_rmsd_bond.atom_site_auth_atom_id_2
c _pdbx_validate_rmsd_bond.atom_site_auth_comp_id_2
c _pdbx_validate_rmsd_bond.atom_site_auth_seq_id_2
c _pdbx_validate_rmsd_bond.atom_site_auth_insert_id_2
c _pdbx_validate_rmsd_bond.atom_site_auth_altcode_2
c _pdbx_validate_rmsd_bond.bond_deviation
c _pdbx_validate_rmsd_bond.bond_value
c _pdbx_validate_rmsd_bond.linker_flag
c    0  A   CD   LYS    152  .  .    A   CE   LYS    152  .  .      -0.372     1.136  N
c    0  A   CG   GLU    166  .  .    A   CD   GLU    166  .  .      -0.622     0.893  N
c 
c
c loop_
c _pdbx_validate_rmsd_angle.model_number
c _pdbx_validate_rmsd_angle.atom_site_auth_asym_id_1
c _pdbx_validate_rmsd_angle.atom_site_auth_atom_id_1
c _pdbx_validate_rmsd_angle.atom_site_auth_comp_id_1
c _pdbx_validate_rmsd_angle.atom_site_auth_seq_id_1
c _pdbx_validate_rmsd_angle.atom_site_auth_insert_id_1
c _pdbx_validate_rmsd_angle.atom_site_auth_altcode_1
c _pdbx_validate_rmsd_angle.atom_site_auth_asym_id_2
c _pdbx_validate_rmsd_angle.atom_site_auth_atom_id_2
c _pdbx_validate_rmsd_angle.atom_site_auth_comp_id_2
c _pdbx_validate_rmsd_angle.atom_site_auth_seq_id_2
c _pdbx_validate_rmsd_angle.atom_site_auth_insert_id_2
c _pdbx_validate_rmsd_angle.atom_site_auth_altcode_2
c _pdbx_validate_rmsd_angle.atom_site_auth_asym_id_3
c _pdbx_validate_rmsd_angle.atom_site_auth_atom_id_3
c _pdbx_validate_rmsd_angle.atom_site_auth_comp_id_3
c _pdbx_validate_rmsd_angle.atom_site_auth_seq_id_3
c _pdbx_validate_rmsd_angle.atom_site_auth_insert_id_3
c _pdbx_validate_rmsd_angle.atom_site_auth_altcode_3
c _pdbx_validate_rmsd_angle.angle_deviation
c _pdbx_validate_rmsd_angle.angle_value
c _pdbx_validate_rmsd_angle.linker_flag
c    0 A  NE  ARG   35 . .  A  CZ  ARG   35 . .  A  NH2 ARG   35 . .      -3.14   117.16  N
c    0 A  CB  GLU  166 . .  A  CG  GLU  166 . .  A  CD  GLU  166 . .      34.68   148.88  N
c
c
c loop_
c _pdbx_validate_torsion.id
c _pdbx_validate_torsion.model_id
c _pdbx_validate_torsion.auth_comp_id
c _pdbx_validate_torsion.auth_asym_id
c _pdbx_validate_torsion.auth_seq_id
c _pdbx_validate_torsion.insert_id
c _pdbx_validate_torsion.phi
c _pdbx_validate_torsion.psi
c    1     0  SER  A  .     -64.75     2.02
c    2     0  THR  A  .    -116.30    61.44
c
c loop_
c _pdbx_validate_peptide_omega.id
c _pdbx_validate_peptide_omega.model_id
c _pdbx_validate_peptide_omega.auth_comp_id_1
c _pdbx_validate_peptide_omega.auth_asym_id_1
c _pdbx_validate_peptide_omega.auth_seq_id_1
c _pdbx_validate_peptide_omega.insert_id_1
c _pdbx_validate_peptide_omega.label_alt_id_1
c _pdbx_validate_peptide_omega.auth_comp_id_2
c _pdbx_validate_peptide_omega.auth_asym_id_2
c _pdbx_validate_peptide_omega.auth_seq_id_2
c _pdbx_validate_peptide_omega.insert_id_2
c _pdbx_validate_peptide_omega.label_alt_id_2
c _pdbx_validate_peptide_omega.omega
c    1    0 A ASP  414 . .  A ARG  413 . .     147.84
c    2    0 B ASN  289 . .  B ALA  288 . .     -39.12
c
c
c loop_
c _pdbx_validate_planes.id
c _pdbx_validate_planes.model_id
c _pdbx_validate_planes.auth_comp_id
c _pdbx_validate_planes.auth_asym_id
c _pdbx_validate_planes.auth_seq_id
c _pdbx_validate_planes.insert_id
c _pdbx_validate_planes.rmsd
c _pdbx_validate_planes.type
c   1     1   DG  A      3  .    0.068  SIDE-CHAIN
c   2     1   DT  A      4  .    0.198  SIDE-CHAIN
c   3     1   DC  A      8  .    0.090  SIDE-CHAIN
c 
c loop_
c _pdbx_validate_planes_atom.plane_id
c _pdbx_validate_planes_atom.id
c _pdbx_validate_planes_atom.model_id
c _pdbx_validate_planes_atom.auth_asym_id
c _pdbx_validate_planes_atom.auth_comp_id
c _pdbx_validate_planes_atom.auth_seq_id
c _pdbx_validate_planes_atom.insert_id
c _pdbx_validate_planes_atom.auth_atom_id
c _pdbx_validate_planes_atom.atom_deviation
c   1     1     1   DG  A      3  .   N1          0.003
c   1     2     1   DG  A      3  .   C2          0.011
c   1     3     1   DG  A      3  .   N2          0.074
c   1     4     1   DG  A      3  .   N3          0.005
c   1     5     1   DG  A      3  .   C4          0.010
c   1     6     1   DG  A      3  .   C5          0.029
c   1     7     1   DG  A      3  .   C6          0.039
c   1     8     1   DG  A      3  .   O6          0.074
c   1     9     1   DG  A      3  .   N7          0.050
c   1    10     1   DG  A      3  .   C8          0.129
c   1    11     1   DG  A      3  .   N9          0.033
c   1    12     1   DG  A      3  .   C1'         0.147
c   2     1     1   DT  A      4  .   N1          0.069
c
c
c loop_
c _pdbx_validate_main_chain_plane.id
c _pdbx_validate_main_chain_plane.model_id
c _pdbx_validate_main_chain_plane.auth_comp_id
c _pdbx_validate_main_chain_plane.auth_asym_id
c _pdbx_validate_main_chain_plane.auth_seq_id
c _pdbx_validate_main_chain_plane.insert_id
c _pdbx_validate_main_chain_plane.pseudo_torsion_plane
c    1     0  TRP  G     20  .    29.901
c    2     0  TRP  G     21  .   -42.450
c 
c 
c loop_
c _pdbx_validate_chiral.id
c _pdbx_validate_chiral.model_id
c _pdbx_validate_chiral.auth_comp_id
c _pdbx_validate_chiral.auth_asym_id
c _pdbx_validate_chiral.auth_seq_id
c _pdbx_validate_chiral.insert_id
c _pdbx_validate_chiral.details
c _pdbx_validate_chiral.angle
c   1    0 B ASP   405 .  ALPHA-CARBON   150.48
c
c
c loop_
c _struct_mon_prot_cis.pdbx_id
c _struct_mon_prot_cis.pdbx_PDB_model_num
c _struct_mon_prot_cis.label_comp_id
c _struct_mon_prot_cis.label_asym_id
c _struct_mon_prot_cis.label_seq_id
c _struct_mon_prot_cis.pdbx_PDB_ins_code
c _struct_mon_prot_cis.label_alt_id
c _struct_mon_prot_cis.pdbx_label_comp_id_2
c _struct_mon_prot_cis.pdbx_label_asym_id_2
c _struct_mon_prot_cis.pdbx_label_seq_id_2
c _struct_mon_prot_cis.pdbx_PDB_ins_code_2
c _struct_mon_prot_cis.label_alt_id_2
c _struct_mon_prot_cis.pdbx_omega_angle
c   1    0 A ASP  285 . .  A THR  284 . .      15.64
c   2    0 A LYS  286 . .  A ASP  285 . .     -17.48
c 
c---------------------------------------------------------------
c  IF NUCLEIC ACID ALSO GET
c
c
c #
c data_203d
c #
c #
c # Sanger Principle of Nucleic Acid...
c # ANTI 90.0 -> 180.0 & -180.0 -> -90.0
c # SYN -90.0 ->   0.0 &    0.0 ->  90.0
c # HIGH ANTI  -60.0 -> -90.0
c # HIGH SYN   90.0 -> 60.0  
c #
c 
c loop_
c _struct_mon_nucl.model_id
c _struct_mon_nucl.auth_comp_id
c _struct_mon_nucl.auth_seq_id
c _struct_mon_nucl.auth_asym_id
c _struct_mon_nucl.auth_insert_code
c _struct_mon_nucl.tau0
c _struct_mon_nucl.tau1
c _struct_mon_nucl.tau2
c _struct_mon_nucl.tau3
c _struct_mon_nucl.tau4
c _struct_mon_nucl.taum
c _struct_mon_nucl.P                            
c _struct_mon_nucl.glycosyl_angle_value         
c _struct_mon_nucl.glycosyl_angle               
c _struct_mon_nucl.glycosyl_angle_conformation  
c _struct_mon_nucl.ribose_conformation          
c    1  DG     1 A      1.2   20.4  -32.2   33.1  -22.2   34.9  199.9 -136.2 " O4'- C1'- N9 - C4 " ANTI "(H)C3'-endo"
c    1  DC     2 A    -13.6   30.7  -35.2   27.8   -9.2   35.9  176.2  -95.9 " O4'- C1'- N1 - C2 " ANTI "C2'-endo"
c
c
c---------------------------------------------------------------
c
c Sample RMSD file for bad geometry
c
c      3.573  all_angle_rmsd 
c      0.018  all_bond_rmsd  
c  The following bond classes show an overall poor rmsd - check your dictionary
c   Total  Observed   Expected    E&H99     Example    Example
c   Class    RMSD       RMSD      Value     Bond       Residue
c      80   0.137      0.007      1.339     C5 - C6      T
c      80   0.081      0.007      1.378     N1 - C6      T
c      80   0.056      0.009      1.445     C4 - C5      T
c  The following angle classes show an overall poor rmsd - check your dictionary
c   Total  Observed  Expected   E&H99       Example   Example
c   Class    RMSD      RMSD     Value       Angle     Residue
c     274    3.18      0.50    106.80   O4'- C1'- C2'   DT
c     366    4.40      0.70    108.00   O4'- C1'- N1    DT
c     640    4.41      0.30    108.30   O4'- C1'- N1    DT
c      80    4.81      0.50    117.00   C4 - C5 - C6     A
c     240    7.32      0.50    117.40   C4 - C5 - C6     C
c      80    6.83      0.50    117.70   C5 - C6 - N1     A
c     240    7.88      0.60    118.80   C4 - C5 - C6     G
c      80    5.67      0.60    119.00   C4 - C5 - C7     T
c     560   11.23      1.50    119.60   OP1- P  - OP2    T
c     240    6.51      0.70    120.20   C5 - C4 - N4     C
c     240    9.33      0.50    121.00   C5 - C6 - N1     C
c      80   11.51      0.60    122.90   C6 - C5 - C7     T
c      80    4.24      0.60    123.70   C5 - C6 - N1     T
c
c---------------------------------------------------------------
c
c end of r500.f
c 
