C
C     RANTAN
C     Copyright (C) 1998 Yao Jia-xing
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
C     *******************
      PROGRAM RANTAN
C     *******************
C
C     The CCP4 version of RANTAN is for the determination of heavy atoms 
C     in a macro-molecule structure or to determine a small molecule 
C     structure from E values calculated by ECALC. The phases and weights are
C     obtained from refining random phases by the tangent formula. Any known
C     phases can be input and combined with random phases as a starting set.
C     
C     The program can handle up to 200 data items.
C
C     Output file contains input data plus
C
C       PHIn  WTn      The refined phases and weights of the best sets
C                      according to the combine figure of merit.
C                      n is from 1 to 5.
C
C     The labels are as follows:
C
C    -----------------------------------------------------------
C
C       Name           Item
C
C       H, K, L        Miller indices.
C 
C     --------------   Input:
C
C       EVAL           Normalized structure factors.
C       PHI            Known Phases.
C       WT             Weights for the known phases.
C
C     --------------   Output:
C
C       PHI1           Refined phases and
C       WT1            weights        for set 1.
C       PHI2           Refined phases and
C       WT2            weights        for set 2.
C       PHI3           Refined phases and
C       WT3            weights        for set 3.
C       PHI4           Refined phases and
C       WT4            weights        for set 4.
C       PHI5           Refined phases and
C       WT5            weights        for set 5.
C
C   ----------------------------------------------------------------
C
C     The column types are as follows:
C
C       H       index h,k,l.
C       F       structure factor FP, FPH, FA, FM and EVAL.
C       D       anomalous difference.
C       Q       standard deviation of anything: J,F,D or other.
C       P       Phase.
C       W       Weight.
C
C    ---------------------------------------------------
C
C       I       any other integer
C       R       any other real
C
C               ******************************
C               Description of Key Words Input
C               ******************************
C
C     See the document rantan.html
C
C     Parameters.
      INTEGER MCOLS,NKEYWD,LSYM,NUMFO,NUMDI,NOMAX
      PARAMETER (MCOLS=500,NKEYWD=200,LSYM=192,NUMFO=2000,NUMDI=2002)
      PARAMETER (NOMAX=5)
C
C     Arrays for data.
      REAL ADATA(MCOLS),BDATA(MCOLS),RSYML(4,4,LSYM)
      INTEGER LOOKUP(MCOLS),IN(3),NEVAL,NPHASE,NWEIGHT
      CHARACTER*1 CTPRGO(MCOLS),CTPRGI(MCOLS),CTUSRI(MCOLS)
     +           ,CTUSRO(MCOLS)
      CHARACTER*30 LSPRGI(MCOLS),LSPRGO(MCOLS),LSUSRI(MCOLS)
     +            ,LSUSRO(MCOLS)
      LOGICAL LOGMSS(MCOLS)
C
C     Local Scalars.
      REAL SS4,SMAX,SMIN
      INTEGER IC,IFAIL,IFIL1,IFLAG,IPRINT,JDO120,JDO230,JDO320,JDO350
      INTEGER JDO430,JDO450,NCOL,NLPRGI,NSPGRP,NSYM,NSYMP,NTOK,NLPRGO
      INTEGER IDTYPE,ICHKP(2),STARTEV,NLABO,NNSET
      CHARACTER NAMSPG*10,PGNAM*10,TITLE*80
      LOGICAL ENDFIL,LTITLE
C
C     Variables for Parser.
      REAL FVALUE(NKEYWD)
      INTEGER IBEG(NKEYWD),IDEC(NKEYWD),IEND(NKEYWD),ITYPE(NKEYWD)
      CHARACTER KEY*4,CVALUE(NKEYWD)*4,LINE*2000,LTYPE*1,WORD*8,
     +          LABOUT_SAVE*2000
      LOGICAL LEND
C
C     External Routines.
      EXTERNAL CCPERR,CCPFYP,CCPRCS,CENTR,CENTRIC,EQUAL_MAGIC,LKYIN,
     +         LROPEN,LRASSN,LRCLAB,LRCLOS,LRREFM,LRSYMI,LRSYMM,
     +         LWOPEN,LWASSN,LWTITL,LWREFL,MTZINI,PARSER,GTPINT,CCPUPC,
     +         GTPREA,LRRSOL,LKYOUT,LRREFL,LWCLOS,LRNREF,LRREWD,LWHSTL
C
C     KNOWN PHASES FROM CCP4.
      COMMON /PHIWT/IMKC,KDSC(2000),MKGC(2000),IPKC(2000),WTSC(2000)
C
C     Variables for RANTAN.
      COMMON/SYMTRY/IS(2,3,24),TS(3,24),NSYMP,LATT,ICENT,NORI
      COMMON/KEYW/KAPMIN,KAPMAX,MAXSET,NRANDOM,NUMREF,NUMZRO,NSKIP
     +  ,IWMINI,WFOM(3),EMAX,EMIN,EPSI,SS4L,SS4H,LISTC,KSWT,NOUT
     +,WTRIP,NSET(5)
      INTEGER IH(NUMDI),IK(NUMDI),IL(NUMDI),ICODE(NUMDI),REFNUM
      REAL EVAL(NUMDI),RPHASE(NUMDI),WEIGHT(NUMDI),TEM1,TEM2
      REAL PHIOUT(NUMDI,NOMAX),WTOUT(NUMDI,NOMAX)
C     *************************************************************
      DATA WFOM/0.1,0.4,0.5/,EMAX/5.0/,EMIN/1.2/,EPSI/0.3/,SS4L/0.0/
     +    ,SS4H/1.3/,WTRIP/0.1/
      DATA KAPMIN/60/,KAPMAX/5000/,MAXSET/500/,NRANDOM/250/,NUMREF/600/
     +    ,NUMZRO/100/,NSET/5*0/,NSKIP/0/,IWMINI/25/,ICENT/0/
     +    ,LISTC/0/,KSWT/0/
C
C     Data Statements.
      DATA NLPRGI,NLPRGO,IDTYPE,NOUT,NLABO,NNSET/6,0,0,3,0,0/
      DATA ICHKP/2*0/
      DATA LTITLE/.FALSE./
      DATA LSPRGI/'H','K','L',
     +      'EVAL',    'PHI',       'WT',
     +       494*' '/
C
      DATA CTPRGI/'H','H','H',
     +      'E','P','W',
     +       494*' '/
C
      DATA LSPRGO/'H','K','L',
     +       'PHI1',       'WT1',
     +       'PHI2',       'WT2',
     +       'PHI3',       'WT3',
     +       'PHI4',       'WT4',
     +       'PHI5',       'WT5',
     +       487*' '/
C
      DATA CTPRGO/'H','H','H',
     +      'P','W',
     +      'P','W',
     +      'P','W',
     +      'P','W',
     +      'P','W',
     +      487*' '/
C
      DATA LOOKUP/-1,-1,-1,497*0/
C 
C     Starting indices of program label blocks
      STARTEV = 3
      IFIL1 = 1
      LABOUT_SAVE = ' '

      CALL CCPFYP
      CALL MTZINI
      CALL CCPRCS(6,'RANTAN','$Date: 2005/09/06 11:25:22 $')
      WRITE (6, 6000)
 6000 FORMAT (/,
     +     ' $TEXT:Reference1: $$ comment $$ ',
     +     ' "On the Application of Phase Relationships to ',
     +     'Complex Structure. XVIII. RANTAN - Random MULTAN",',/,
     +     ' Yao Jia-xing, (1983). Acta. Cryst.A39, 35-37.',//,' $$',/,
     +     ' $SUMMARY :Reference1:  $$ Rantan: $$',/,
     +     ' :TEXT:Reference1: $$')

C     Read keyword input.
 20   CONTINUE
      KEY  = ' '
      LINE = ' '
C     Reset number of words on line to NKEYWD
      NTOK = NKEYWD
C
C     **********************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYPE,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +     .TRUE.)
C     **********************************************************
C
      IF (LEND) GO TO 100
      IF (KEY.EQ.'TITL') THEN
C      Read new title.
       IF (NTOK.GE.2) THEN
        LTITLE = .TRUE.
        TITLE = LINE(IBEG(2) :IEND(NTOK))
        WRITE(6,'(10X,A,/,15X,A80,/)')   
     +       'TITLE OF MTZ FILE REPLACED BY:',TITLE
       ENDIF
       GO TO 20
C
      ELSE IF (KEY.EQ.'LABI') THEN
C      Input labels
C        *****************************************
         CALL LKYIN(IFIL1,LSPRGI,NLPRGI,NTOK,LINE,IBEG,IEND)
C        *****************************************
       IF (NTOK.EQ.1) CALL CCPERR(1,
     +   'No columns assigned. Nothing to do. Bye-bye!')
       WRITE(6,'(10X,I3,X,A,/,10X,A,A,/)')   
     +  (NTOK-1)/2, 'column labels have been assigned.',
     +   'NOTE: missing number analysis only performed ',
     +   'for these columns!!'
       GO TO 20
C
      ELSE IF (KEY.EQ.'LABO') THEN
C      Save output labels
       LABOUT_SAVE = LINE
       NLABO = (NTOK - 1)/4
       IF (NLABO.GT.NNSET.AND.NNSET.GT.0) CALL CCPERR(1,
     + ' *** Error - too many labels in LABOUT ***')
       IF (NLABO.GT.NOUT) NOUT = NLABO
       GO TO 20
C
C     Weights of three figures of merits.
      ELSE IF (KEY.EQ.'WFOM') THEN
       ITOK = 2
C
   30  CONTINUE
C
       IF (ITYPE(ITOK).EQ.2) CALL CCPERR(1,
     +  ' ***Error - should have word ABS, PSI or RES ***')
C
C      Next word ABS, PSI or RES.
C
       WORD = LINE(IBEG(ITOK) :IEND(ITOK))
       CALL CCPUPC(WORD)
       ITOK = ITOK + 1
       IF (ITOK.GT.NTOK) GO TO 20
C
       IF ( WORD(1:3) .EQ. 'ABS') THEN
C       Read the weight for ABSFOM.
        IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for ABS ***')
C       **********************************
        CALL GTPREA(ITOK,WFOM(1),NTOK,ITYPE,FVALUE)
C       **********************************
C
        ITOK = ITOK + 1
        IF (ITOK.GT.NTOK) GO TO 20
        GO TO 30
C
       ELSE IF ( WORD(1:3) .EQ. 'PSI') THEN
C       Read the weight for PSIZERO.
        IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for PSI ***')
C       **********************************
        CALL GTPREA(ITOK,WFOM(2),NTOK,ITYPE,FVALUE)
C       **********************************
C
        ITOK = ITOK + 1
        IF (ITOK.GT.NTOK) GO TO 20
        GO TO 30
C
       ELSE IF ( WORD(1:3) .EQ. 'RES') THEN
C       Read the weight for RESIDURE.
        IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for RES ***')
C       **********************************
        CALL GTPREA(ITOK,WFOM(3),NTOK,ITYPE,FVALUE)
C       **********************************
C
        ITOK = ITOK + 1
        IF (ITOK.GT.NTOK) GO TO 20
        GO TO 30
       ELSE
        CALL CCPERR(1,' word in WFOM unrecognised')
       END IF
C
C     Maximum E value in phase development.
      ELSE IF (KEY.EQ.'EMAX') THEN
       ITOK = 2
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for EMAX ***')
C      **********************************
       CALL GTPREA(ITOK,EMAX,NTOK,ITYPE,FVALUE)
C      **********************************
       GO TO 20
C
C     Minimum E value in phase development.
      ELSE IF (KEY.EQ.'EMIN') THEN
       ITOK = 2
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for EMIN ***')
C      **********************************
       CALL GTPREA(ITOK,EMIN,NTOK,ITYPE,FVALUE)
C      **********************************
       GO TO 20
C
C     Maximum E value for PSIZERO figure of merit.
      ELSE IF (KEY.EQ.'EPSI') THEN
       ITOK = 2
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for EPSI ***')
C      **********************************
       CALL GTPREA(ITOK,EPSI,NTOK,ITYPE,FVALUE)
C      **********************************
       GO TO 20
C
C     Minimum KAPPA value of relationships.
      ELSE IF (KEY.EQ.'KMIN') THEN
       ITOK = 2
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for KMIN ***')
C      **********************************
       CALL GTPREA(ITOK,TEM1,NTOK,ITYPE,FVALUE)
C      **********************************
       KAPMIN = INT(100.0*TEM1 + 0.5)
       GO TO 20
C
C     Maximum KAPPA value of relationships.
      ELSE IF (KEY.EQ.'KMAX') THEN
       ITOK = 2
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for KMAX ***')
C      **********************************
       CALL GTPREA(ITOK,TEM1,NTOK,ITYPE,FVALUE)
C      **********************************
       KAPMAX = INT(100.0*TEM1 + 0.5)
       GO TO 20
C
C     Resolution range of E values.
      ELSE IF (KEY.EQ.'RESO') THEN
       ITOK = 2
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a first number for RESO ***')
C      **********************************
       CALL GTPREA(ITOK,TEM1,NTOK,ITYPE,FVALUE)
C      **********************************
       ITOK = ITOK + 1
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a second number for RESO ***')
C      **********************************
       CALL GTPREA(ITOK,TEM2,NTOK,ITYPE,FVALUE)
C      **********************************
       ACUT1 = AMAX1(TEM1,TEM2)
       ACUT2 = AMIN1(TEM1,TEM2)
       SS4L = 1.0/(ACUT1*ACUT1)
       SS4H = 1.0/(ACUT2*ACUT2)
       GO TO 20
C
C     Maximum number of phase sets to be generated.
      ELSE IF (KEY.EQ.'MAXS') THEN
       ITOK = 2
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for MAXS ***')
C      **********************************
       CALL GTPINT(ITOK,MAXSET,NTOK,ITYPE,FVALUE)
C      **********************************
       GO TO 20
C
C     Information print out control.
C     Default --- print out normally enough information.
C        LIST --- print out full information.
C
      ELSE IF (KEY.EQ.'LIST') THEN
       LISTC = 1
       GO TO 20
C
C     Number of reflections in starting set.
      ELSE IF (KEY.EQ.'NRAN') THEN
       ITOK = 2
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for NRAN ***')
C      **********************************
       CALL GTPINT(ITOK,NRANDOM,NTOK,ITYPE,FVALUE)
C      **********************************
       GO TO 20
C
C     Number of large E's.
      ELSE IF (KEY.EQ.'NREF') THEN
       ITOK = 2
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for NREF ***')
C      **********************************
       CALL GTPINT(ITOK,NUMREF,NTOK,ITYPE,FVALUE)
C      **********************************
       GO TO 20
C
C     Number of PSIZERO reflections.
      ELSE IF (KEY.EQ.'NZRO') THEN
       ITOK = 2
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for NZRO ***')
C      **********************************
       CALL GTPINT(ITOK,NUMZRO,NTOK,ITYPE,FVALUE)
C      **********************************
       GO TO 20
C
C     The particular set numbers for phase development.
      ELSE IF (KEY.EQ.'NSET') THEN
       ITOK = 2
       NNSET = 0
C
   40  IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for NSET ***')
C      **********************************
       CALL GTPINT(ITOK,NSET(ITOK-1),NTOK,ITYPE,FVALUE)
C      **********************************
       ITOK = ITOK + 1
       NNSET = NNSET + 1
       IF (ITOK.GT.NTOK) THEN
        NOUT = NNSET
        IF (NNSET.LT.NLABO) CALL CCPERR(1,
     +  ' *** Error - too many labels in LABOUT ***')
        GO TO 20
       ENDIF
       GO TO 40
C
C     The number of sets of phases in output file.
      ELSE IF (KEY.EQ.'NOUT') THEN
       ITOK = 2
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for NOUT ***')
C      **********************************
       CALL GTPINT(ITOK,NOUT,NTOK,ITYPE,FVALUE)
C      **********************************
       IF (NOUT.LT.NLABO.AND.NLABO.GT.0) NOUT = NLABO
       IF (NNSET.GT.0) NOUT = NNSET
       GO TO 20
C
C     Skip the first n sets in phase development.
      ELSE IF (KEY.EQ.'SKIP') THEN
       ITOK = 2
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for SKIP ***')
C      **********************************
       CALL GTPINT(ITOK,NSKIP,NTOK,ITYPE,FVALUE)
C      **********************************
       GO TO 20
C
C     Initial weight for the random starting phases.
      ELSE IF (KEY.EQ.'WMIN') THEN
       ITOK = 2
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for WMIN ***')
C      **********************************
       CALL GTPREA(ITOK,TEM1,NTOK,ITYPE,FVALUE)
C      **********************************
       IWMINI = INT(100.0*TEM1 + 0.5)
       GO TO 20
C
C     Weight for triplets.
      ELSE IF (KEY.EQ.'WTRI') THEN
       ITOK = 2
C
       IF (ITYPE(ITOK).EQ.1) CALL CCPERR(1,
     +  ' *** Error - should have a number for WTRI ***')
C      **********************************
       CALL GTPREA(ITOK,WTRIP,NTOK,ITYPE,FVALUE)
C      **********************************
       GO TO 20
C
C     Statistics Weighted Tangent-formula Refinement.
      ELSE IF (KEY.EQ.'SWTR') THEN
       KSWT = 1
       GO TO 20
C
      ELSE IF (KEY.EQ.'END') THEN
       WRITE(6,'(/)')   
       GO TO 100
      ELSE 
       CALL CCPERR(1,'  KEY word unrecognised')
      END IF
C
C     End of keyword input
C
 100  CONTINUE
C
C Check the number of Es is <= 2000
      IF( NUMREF+NUMZRO.GE.1999) THEN
         ITOK = 1999 -NUMZRO
         Write(6,'(A,i4,a,i4,A)')' NREF reset from',NUMREF, ' to ',
     +   ITOK, ' to keep Total Sum of strong and weak Es <= 1999'
         NUMREF = ITOK
         CALL CCPERR(2,
     + ' NREF rest to keep Total Sum of strong and weak Es <= 1999')
      END IF
C
       IPRINT = 2
C     **********************************
      CALL LROPEN(IFIL1,'HKLIN',IPRINT,IFAIL)
C     **********************************
      IF (IFAIL.NE.0) CALL CCPERR(1, ' No input file???????')
C
C     ****************************************
      CALL LRASSN(IFIL1,LSPRGI,NLPRGI,LOOKUP,CTPRGI)
C     *****************************************
C
C     *****************************************
      CALL LRCLAB(IFIL1,LSUSRI,CTUSRI,NCOL)
C     *****************************************
C
C     Cell dimensions.
C     *****************************************
C     LRCELL(IFIL1,CELLP)
C     *****************************************
C
C     Symmetry information.
C     *****************************************
      CALL LRSYMI(IFIL1,NSYMP,LTYPE,NSPGRP,NAMSPG,PGNAM)
      IF (LTYPE.EQ.' ') THEN 
       CALL CCPERR(1, ' No symmetry information avaliable.')
      ELSE IF (LTYPE.EQ.'P') THEN
       LATT = 1
      ELSE IF (LTYPE.EQ.'A') THEN
       LATT = 2
      ELSE IF (LTYPE.EQ.'B') THEN
       LATT = 3
      ELSE IF (LTYPE.EQ.'C') THEN
       LATT = 4
      ELSE IF (LTYPE.EQ.'I') THEN
       LATT = 5
      ELSE IF (LTYPE.EQ.'F') THEN
       LATT = 6
      ELSE IF (LTYPE.EQ.'R') THEN
       LATT = 7
      ELSE
      END IF
C     ****************************************
      CALL LRSYMM(IFIL1,NSYM,RSYML)
      IF (NSYM.GT.0) THEN
       CALL TRANSYM(RSYML,NSYM,NSYMP)
      ELSE
       CALL CCPERR(1, ' No symmetry operation avaliable.')
      END IF
C     *****************************************
C
C     Resolution range
C     *****************************************
      CALL LRRSOL(IFIL1,SMIN,SMAX)
C     *****************************************
C     Resolution range for E values.
      SS4L = AMAX1(SMIN,SS4L)
      SS4H = AMIN1(SMAX,SS4H)
      RESL = 1.0/SQRT(SS4L)
      RESH = 1.0/SQRT(SS4H)
      WRITE (6,FMT='(/,A,F8.2,A,F8.2,/)') 
     + ' The resolution range of E values is from'
     + ,RESL,' to',RESH
C
C     Check what kind of data to be used.
C     IDTYPE = 1 use of |EVAL| only.
C     IDTYPE = 2 use of |EVAL|, PHASE and WEIGHT.
C
      DO 120 JDO120 = STARTEV,STARTEV+2,3
       ICHKP(1) = LOOKUP(JDO120+1)
       NEVAL = LOOKUP(JDO120+1)
       NPHASE = LOOKUP(JDO120+2)
       NWEIGHT = LOOKUP(JDO120+3)
       ICHKP(1) = NEVAL
       ICHKP(2) = NPHASE*NWEIGHT
       IF (LOOKUP(JDO120+2).NE.0 .NEQV. LOOKUP(JDO120+3).NE.0) THEN
        WRITE (6,FMT='(/,A,/)')
     +   'Unmatched PHASE or WEIGHT',' assigned.'
        CALL CCPERR(1,' Unmatched PHASE/WEIGHT pair')
       ENDIF
  120 CONTINUE
      IF (ICHKP(1).EQ.0) THEN 
       CALL CCPERR(1,' EVAL must be input. ')
      ELSE
       IDTYPE = 1
      END IF
      IF (ICHKP(2).NE.0) THEN
       IDTYPE = 2
       WRITE (6,FMT='(/,A,/)') 
     +   ' RANTAN will use known phases with weights in the input file.'
      END IF
C
C     Set up information to decide centric zones.
C
      CALL CENTRIC(NSYM,RSYML,IPRINT)
C     Initial input data.
      LIM = NUMFO
      NC = 0
      NS = 0
      NL = 0
C
C     Loop for reflections.
C
 200  CONTINUE
C     **************************
      CALL LRREFL(IFIL1,SS4,ADATA,ENDFIL)
C     **************************
      IF (ENDFIL) GO TO 300
C
C     Set LOGMSS(i) as .TRUE. if the data item is missing.
C     **************************
      CALL LRREFM(IFIL1,LOGMSS)
C     **************************
C
C     Reflection number.
      CALL LRNREF(IFIL1,REFNUM)
C
C     Check the resolution range.
      IF (SS4.LT.SS4L.OR.SS4.GT.SS4H) GOTO 200
C     Test whether a reflection is centric (ic=1) or not (ic=0).
      IN(1) = NINT(ADATA(1))
      IN(2) = NINT(ADATA(2))
      IN(3) = NINT(ADATA(3))
      CALL CENTR(IN,IC)
C     Checking systematic extintion.
      IEXT=0
      CALL REJECT(IEXT,IN)
      IF (IEXT.EQ.1) GO TO 200
C 
C     Get EVAL and PHASE & WEIGHT if any.
C
      MKEV = 0
      DO 230 JDO230 = 1,NCOL
       IF (.NOT.LOGMSS(JDO230)) THEN
        IF (JDO230.EQ.NEVAL) THEN
         MKEV = 1
         TEVAL = ADATA(JDO230)
        END IF
        IF (IDTYPE.EQ.2) THEN
         IF (JDO230.EQ.NPHASE) TPHASE = ADATA(JDO230)
         IF (JDO230.EQ.NWEIGHT) TWEIGHT = ADATA(JDO230)
        END IF
       END IF
  230 CONTINUE
      IF (MKEV.EQ.1) THEN
       NL = NL + 1
       IF (TEVAL.GT.EMIN.AND.TEVAL.LT.EMAX) GOTO 235
       NL = NL - 1
       IF (TEVAL.GT.EPSI.OR.TEVAL.LT.0.01) GO TO 200
       NS = NS + 1
  235  NC = NC + 1
       ICODE(NC) = REFNUM - 1
       IH(NC) = IN(1)
       IK(NC) = IN(2)
       IL(NC) = IN(3)
       EVAL(NC) = TEVAL
C      Input phase information.
       IF (IDTYPE.EQ.2) THEN
        ITEM1 = INT(100.0*TWEIGHT + 0.5)
        IF (ITEM1.GT.IWMINI) THEN
         RPHASE(NC) = TPHASE
         WEIGHT(NC) = TWEIGHT
        END IF
       END IF
C      Sort data.
       IF (NC.NE.LIM) GO TO 200
       CALL SORT(EVAL,RPHASE,WEIGHT,IH,IK,IL,ICODE,NUMFO)
       IF (NUMZRO.GT.0) EPSI = AMIN1(EPSI,EVAL(NUMFO+1-NUMZRO))
       EMIN = AMAX1(EMIN,EVAL(NUMREF))
       NC = MIN0(NUMREF,NL)
       LIM = NUMFO - NUMZRO
      END IF
      GO TO 200
  300 MR = NUMFO
      IF (LIM.EQ.NUMFO) MR = NC
      CALL SORT(EVAL,RPHASE,WEIGHT,IH,IK,IL,ICODE,MR)
      NUMREF = MIN0(NUMREF,NL)
      NUMZRO = MIN0(NUMZRO,NS)
      IF (LIM.EQ.NUMFO) THEN
       NSTART = NC - NUMZRO + 1
      ELSE
       NSTART = NUMFO - NUMZRO + 1
      END IF
      IMKC = 0
      DO 310 N=1,NUMREF
       IF (IDTYPE.EQ.2.AND.WEIGHT(N).GT.0.85) IMKC = IMKC + 1
  310 CONTINUE
C
C     Transfer parameters from CCP4 to RANTAN.
      CALL TRANPARA(NNSET)
C
C
C     Copy all the input labels to output.
C
      DO 320 JDO320 = 1, NCOL
       LSUSRO(JDO320) = LSUSRI(JDO320)
       CTUSRO(JDO320) = CTUSRI(JDO320)
  320 CONTINUE
C
C     Manage output label for PHASE and WEIGHT.
C
      NLPRGO = NCOL
      NC2 = 3
      DO 350 JDO350 = 1, NOUT
C      Phase.
       NLPRGO = NLPRGO + 1
       NC2 = NC2 + 1
       LSUSRO(NLPRGO) = LSPRGO(NC2)
       CTUSRO(NLPRGO) = CTPRGO(NC2)
C      Weight.
       NLPRGO = NLPRGO + 1
       NC2 = NC2 + 1
       LSUSRO(NLPRGO) = LSPRGO(NC2)
       CTUSRO(NLPRGO) = CTPRGO(NC2)
  350 CONTINUE
C
C     Open output file.
C
C     **********************
      CALL LWOPEN(IFIL1,'HKLOUT')
C     Parse the "LABOUT" line now.
      LINE = LABOUT_SAVE
      NTOK = NKEYWD
C     ******************************************************
      CALL PARSER(KEY,LINE,IBEG,IEND,ITYPE,FVALUE,CVALUE,IDEC,NTOK,LEND,
     +     .FALSE.)
      CALL LKYOUT(1,LSUSRO,NLPRGO,NTOK,LINE,IBEG,IEND)
      CALL LWASSN(IFIL1,LSUSRO,NLPRGO,CTUSRO,0)
C     ******************************************************
C
C     Set IFLAG = 0 to replace old title if LTITLE = .TRUE..
C
      IFLAG = 0
      IF (LTITLE) CALL LWTITL(IFIL1,TITLE,IFLAG)
C     Write a history line.
      CALL LWHSTL (IFIL1, 'Random MULTAN')
C     Transfer data to RANTAN format.
      CALL TRANDATA(IH,IK,IL,ICODE,EVAL,RPHASE,WEIGHT
     +,             NUMDI,NUMREF,NUMZRO,NSTART)
C
C
C      Start RANTAN. 
C
      CALL RTCCP4(IH,IK,IL,EVAL,RPHASE,WEIGHT,PHIOUT,WTOUT,NOMAX
     +,                NUMREF,NUMDI,IDTYPE)
C
C
C     Read the data from beginning.
C
      CALL LRREWD(IFIL1)
C
      NC = 0
C
C     Loop for reflections.
C
  400 CONTINUE
C     **************************
      CALL LRREFL(IFIL1,SS4,ADATA,ENDFIL)
C     **************************
      IF (ENDFIL) GO TO 1000
C     Count the number of reflection.
      NC = NC + 1
C
C     Set LOGMSS(i) as .TRUE. if the data item is missing.
C     **************************
      CALL LRREFM(IFIL1,LOGMSS)
C     **************************
C     Start assigning the whole list of output as "missing" then replace.
      CALL EQUAL_MAGIC(IFIL1,BDATA,NLPRGO)
C
C     Copy all the input data.
C
      DO 430 JDO430 = 1,NCOL
       IF (.NOT.LOGMSS(JDO430)) THEN
        BDATA(JDO430) = ADATA(JDO430)
       END IF
  430 CONTINUE
C
C     Plus New PHASE and WEIGHT.
C
      DO 450 JDO450 = 1, NUMREF
       IF (NC.EQ.ICODE(JDO450)) THEN
        DO 440 JDO440 = 1, NOUT
         BDATA(NCOL+2*JDO440-1) = PHIOUT(JDO450,JDO440)
         BDATA(NCOL+2*JDO440) = WTOUT(JDO450,JDO440)
  440   CONTINUE
        GO TO 500
       END IF
  450 CONTINUE
C
C     Write a reflection to output file and goto next reflection.
C
  500 CALL LWREFL(IFIL1,BDATA)
      GOTO 400
 1000 CONTINUE
C
      WRITE(6,FMT='(//)')
      CALL LRCLOS(IFIL1)
      CALL LWCLOS(IFIL1,1)
      CALL CCPERR(0,'Normal termination')
      END
C
C     ----------------------------------------------------------
C     Subroutines for RANTAN.
C     ----------------------------------------------------------
C
C     Transfer symmetry from CCP4 to RANTAN.
      SUBROUTINE TRANSYM(RSYML,NSYM,NSYMP)
      DIMENSION RSYML(4,4,NSYM)
      COMMON/SYMTRY/IS(2,3,24),TS(3,24),NSYMR,LATT,ICENT,NORI
      NSYMR = NSYMP
      DO 150 I = 1, NSYMP
       DO 140 J = 1, 4
        IF (J.EQ.4) GOTO 140
        NC = 0
        DO 130 K = 1, 4
         IF (K.EQ.4) THEN
          TS(J,I) = RSYML(J,K,I)
         ELSE
          IF (ABS(RSYML(J,K,I)).GT.0.5) THEN
           NC=NC+1
           IS(NC,J,I) = K
           IF (RSYML(J,K,I).LT.0.0) IS(NC,J,I) = -K
          END IF
         END IF
  130   CONTINUE
  140  CONTINUE
  150 CONTINUE
      RETURN
      END
C     ----------------------------------------------------
C     SORT ON A.
      SUBROUTINE SORT(A,B,C,IH,IK,IL,IC,N)
      DIMENSION A(N),B(N),C(N),IH(N),IK(N),IL(N),IC(N)
      INT=2
   10 INT=2*INT
      IF (INT.LT.N) GO TO 10
      INT=MIN0(N,(3*INT)/4-1)
   20 INT=INT/2
      IFIN=N-INT
      DO 70 II=1,IFIN
       I=II
       J=I+INT
       IF (A(I).GE.A(J)) GO TO 70
       T1=A(J)
       T2=B(J)
       T3=C(J)
       I1=IH(J)
       I2=IK(J)
       I3=IL(J)
       I4=IC(J)
   40  A(J)=A(I)
       B(J)=B(I)
       C(J)=C(I)
       IH(J)=IH(I)
       IK(J)=IK(I)
       IL(J)=IL(I)
       IC(J)=IC(I)
       J=I
       I=I-INT
       IF (I.LE.0) GO TO 60
       IF (A(I).LT.T1) GO TO 40
   60  A(J)=T1
       B(J)=T2
       C(J)=T3
       IH(J)=I1
       IK(J)=I2
       IL(J)=I3
       IC(J)=I4
   70 CONTINUE
      IF (INT.GT.1) GO TO 20
      RETURN
      END
C     --------------------------------------------------------
C     THIS PROGRAM IS USE FOR CHECKING AND REJECTING THE REFLEXIONS 
C     WITH SYSTEMATIC EXTINTION .
      SUBROUTINE REJECT(IEXT,I1)
      COMMON/SYMTRY/IS(2,3,24),TS(3,24),NSYM,LATT,ICENT,NORI
      DIMENSION I1(3),T(3)
      EXTI=0.0
      DO 151 I=1,24
      DO 150 J=1,3
       EXTI=EXTI+ABS(TS(J,I))
  150 CONTINUE
  151 CONTINUE
      IHKL=I1(1)+I1(2)
C     LATT. CENTERED (P:500 A:400 B:200 C:450 I:300 F:350 R:100) 
      GO TO (500,400,200,450,300,350,100) LATT
 100  IF (MOD((IHKL+I1(1)+I1(3)),3).EQ.0)  GO TO 500
      GO TO 1000
 200  IHKL=I1(1)+I1(3)
      GO TO 450
 300  IHKL=IHKL+I1(3)
      GO TO 450
 350  IF (MOD(IHKL,2).EQ.0) GO TO 400
      GO TO 1000
 400  IHKL=I1(2)+I1(3)
 450  IF (MOD(IHKL,2).EQ.0) GO TO 500
      GO TO 1000
 500  IF (EXTI.LT.0.01) GO TO 1100
      TEMP1=-999999.0
      RECY=0.0
      DO 980 J=1,NSYM
       DO 800 L=1,3
        T(L)=0.0
 800    CONTINUE
       DO 901 L=1,3
       DO 900 K=1,2
        M=IABS(IS(K,L,J))
        IF (M.GT.0) T(M)=T(M)+I1(L)*ISIGN(1,IS(K,L,J))
 900   CONTINUE
 901   CONTINUE
       IF (49.5-AMAX1(ABS(T(1)),ABS(T(2)))) 1100,1100,950
 950   TEMP2=T(1)+100.00*(T(2)+100.0*T(3))
       IF (ICENT.EQ.1) TEMP2=ABS(TEMP2)
       IF (TEMP2-TEMP1+0.01) 980,960,960
 960   ANG=6.283185*(I1(1)*TS(1,J)+I1(2)*TS(2,J)+I1(3)*TS(3,J))
       SUM=COS(ANG)
C      FOR THE ACENTRIC CASE.
       IF (ICENT.EQ.0) SUM=SUM+100.0*SIN(ANG)
       ANG=SUM
       RECY=RECY+ANG
       IF (TEMP2-TEMP1-0.05) 980,970,970
 970   RECY=ANG
       TEMP1=TEMP2
 980  CONTINUE
      IF (ABS(RECY)-0.01) 1000,1100,1100
C     FOR THE CASE OF SYSTEMATIC EXTINCTION.
 1000 IEXT=1
      RETURN
C     FOR THE CASE OF NON-SYSTEMATIC EXTINCTION.
 1100 IEXT=0
      RETURN
      END      
C-----------------------------------------------------------------------
C     Transfer parameters from CCP4 to RANTAN.
      SUBROUTINE TRANPARA(NNSET)
C     CCP4 COMMON.
      COMMON/KEYW/KAPMIN,KAPMAX,MAXSET,NRANDOM,NUMREF,NUMZRO,NSKIP
     +,IWMINI,WFOM(3),EMAX,EMIN,EPSI,SS4L,SS4H,LISTC,KSWT,NOUT
     +,WTRIP,NSET(5)
C     KNOWN PHASES FROM CCP4.
      COMMON /PHIWT/IMKC,KDSC(2000),MKGC(2000),IPKC(2000),WTSC(2000)
C
C     RANTAN COMMON.
      COMMON /USER/ MS,LIST,NSREQ,PROB,NSPEC,NGEN,NANY,KMIN,
     1 IMK,ITAN,IPUB,ISKIP,WTFOM(3),KMAX,IWMIN,NRAN,IOFR,NSOUT,IPSTOR(5)
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRREC,NREC,PARA(6),NDET
C
C     Initial value for RANTAN.
C
      DATA PROB/1.01/,NSPEC/0/,NGEN/0/,NANY/0/,IOFR/0/,NDET/0/,IPUB/0/
C
C     Values from CCP4.
      IMK = IMKC
      KMIN = KAPMIN
      KMAX = KAPMAX
      NRAN = NRANDOM
      NUMB = NUMREF
      IZRO = NUMZRO
      DO 10 I=1, NNSET
       IPSTOR(I) = NSET(I)
       IPUB = IPUB + 1
   10 CONTINUE
      IF (NNSET.GT.0) THEN
       NSKIP = NSET(1) - 1
       MAXSET = NSET(NNSET) - NSKIP
      END IF
   20 ISKIP = NSKIP
      IF (IMK.GT.NUMB*8/10) THEN
       NOUT = 1
       MAXSET = 10
       WRITE(6,FMT='(/,A,/,A,A,/,A,/,A,/)')
     +  '      **** Warning about LABOUT ****'
     + ,' Since the input known phases are more than 80%'
     + ,' of the number of large E,'
     + ,' one set of refined phases will be output.'
     + ,' Only label PHI1 and WT1 can be assign by LABOUT.'
      END IF
      IF (MAXSET.LT.NOUT) MAXSET = NOUT
      NSREQ = MAXSET
      IWMIN = IWMINI
      WTFOM(1) = WFOM(1)
      WTFOM(2) = WFOM(2)
      WTFOM(3) = WFOM(3)
      LIST = LISTC
      ITAN = KSWT
      NSOUT = NOUT
      SIGMA = 200.0*WTRIP
      RETURN
      END
C---------------------------------------------------------------------
C     Transfer data to RANTAN format.
      SUBROUTINE  TRANDATA(IH,IK,IL,ICODE,EVAL,RPHASE,WEIGHT
     +,             NUMDI,NUMREF,NUMZRO,NSTART)
      DIMENSION IH(NUMDI),IK(NUMDI),IL(NUMDI),ICODE(NUMDI)
     1,         EVAL(NUMDI),RPHASE(NUMDI),WEIGHT(NUMDI)
      DIMENSION LH(5000),LK(5000),LL(5000),LCODE(5000)
     1,         EV(5000)
C
C     Copy all the data.
      DO 50 I = 1, NUMDI
       LH(I) = IH(I)
       LK(I) = IK(I)
       LL(I) = IL(I)
       LCODE(I) = ICODE(I)
       EV(I) = EVAL(I)
   50 CONTINUE
C     Transfer large E's.
      NC = NUMREF + 1
      IH(NC) = 0
      IK(NC) = 0
      IL(NC) = 0
      ICODE(NC) = 0
      EVAL(NC) = -1.0
      RPHASE(NC) = 0.0
      WEIGHT(NC) = 0.0
C     Transfer small E's.
      NZ = NSTART + NUMZRO - 1
      DO 100 I = 1, NUMZRO
       NC = NC + 1
       NT = NZ - I + 1
       IH(NC) = LH(NT)
       IK(NC) = LK(NT)
       IL(NC) = LL(NT)
       ICODE(NC) = LCODE(NT)
       EVAL(NC) = EV(NT)
       RPHASE(NC) = 0.0
       WEIGHT(NC) = 0.0
  100 CONTINUE
      NC = NC + 1
      IH(NC) = 0
      IK(NC) = 0
      IL(NC) = 0
      EVAL(NC) = -1.0
      RPHASE(NC) = 0.0
      WEIGHT(NC) = 0.0
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C   RRRRRRRR       A      NN     NN  TTTTTTTTT      A      NN     NN   C
C   RRRRRRRRR     AAA     NN     NN  TTTTTTTTT     AAA     NN     NN   C
C   RR     RR    AA AA    NNN    NN     TTT       AA AA    NNN    NN   C
C   RR     RR   AA   AA   NNNN   NN     TTT      AA   AA   NNNN   NN   C
C   RRRRRRRRR  AA     AA  NN NN  NN     TTT     AA     AA  NN NN  NN   C
C   RRRRRRRR   AA     AA  NN  NN NN     TTT     AA     AA  NN  NN NN   C
C   RR  RR     AAAAAAAAA  NN   NNNN     TTT     AAAAAAAAA  NN   NNNN   C
C   RR   RR    AAAAAAAAA  NN    NNN     TTT     AAAAAAAAA  NN    NNN   C
C   RR    RR   AA     AA  NN     NN     TTT     AA     AA  NN     NN   C
C   RR     RR  AA     AA  NN     NN     TTT     AA     AA  NN     NN   C
C                                                                      C
C                        CCP4   VERSION   1997                         C
C                                                                      C
C           ***  AN EXTENSIVE MODIFICATION OF RANTAN-81  ***           C
C            (YAO JIA-XING, ACTA CRYST. A37, 642-644, 1981)            C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE RTCCP4(JH,JK,JL,EVAL,RPHASE,WEIGHT,PHIOUT,WTOUT,NOMAX
     +,                      NUMREF,NUMDI,IDTYPE)
      COMMON /USER/ MS,LIST,NSREQ,PROB,NSPEC,NGEN,NANY,KMIN,
     1 IMK,ITAN,IPUB,ISKIP,WTFOM(3),KMAX,IWMIN,NRAN,IOFR,NSOUT,IPSTOR(5)
      COMMON /CONST/ KUSER1, KUSER2,
     1 KUSER3,KUSER4,KUSER5,KUSER6,DTOR,RTOD,STABLE(450)
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B4/ X(1000),Y(1000),Z(1000),NZ(1000),IZ(2000),IH(2000),
     1 E(2000),LIM(2101),MKANG(2000)
C     Data from CCP4.
      INTEGER IDTYPE
      INTEGER JH(NUMDI),JK(NUMDI),JL(NUMDI)
      REAL EVAL(NUMDI),RPHASE(NUMDI),WEIGHT(NUMDI)
      REAL PHIOUT(NUMDI,NOMAX),WTOUT(NUMDI,NOMAX)
C
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
C     PRELIMINARY SECTION        DEFINE ALL NECESSARY PROGRAM PARAMETERS
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
      WRITE (6,50)
   50 FORMAT(//)
      DTOR = ATAN(1.0) / 45.0
      RTOD = 1.0 / DTOR
C     SET UP SIN/COS TABLE
      DO 100 I=1,450
      STABLE(I)=SIN(DTOR*REAL(I-1))
  100 CONTINUE
      STABLE(181)=0.0
      STABLE(361)=0.0
      LIM(1) = 0
C     PROGRAM DIMENSION PARAMETERS  -  SEE WRITE UP FOR EQUIVALENCES
C     KUSER1 = NUMBER OF SIGMA2 + PSI ZERO RELATIONSHIPS
      KUSER1 = 32000
C     KUSER2 = DIMENSION OF IOR, LOC  STORE HEMISPHERE OF REFLEXIONS
      KUSER2 = 18000
C     KUSER3 = MAXIMUM NUMBER OF INPUT REFLEXIONS
C     DIMENSION OF MKG, MKANG, PALF, IZ, IH, E, IPHAZ, WT, ALPHA, IORDE
C     TWICE DIMENSION OF KDSTOR,  WTSTOR,  X, Y ,Z, NZ
      KUSER3 = 2000
C     KUSER4 = DIMENSION OF EEE,IEE AND IPH IN SORT2
C     KUSER4 = 36000
      KUSER4 = KUSER1 + 2*KUSER3
C     KUSER5 = MAX. NUMBER OF STRONG + PSI ZERO REFLEXIONS
      KUSER5 = 2100
C     DIMENSION OF LIM = KUSER5 + 1
C     KUSER6 = DIMENSION OF KNOWN
C     * * * * *   KUSER6 .GE. NUMB * NSYM  * * * * *
      KUSER6 = 8000
C     READ PARAMETERS FROM CONTROL FILE GENERATED BY PREPAR PROGRAM
      call ccpdpn(2, 'RANTMP2','SCRATCH','U',0,0)
      call ccpdpn(3, 'RANTMP3','SCRATCH','U',0,0)
      PARA(1) = 0.0
      CALL RELATN(JH,JK,JL,EVAL,RPHASE,WEIGHT
     +,           NUMREF,NUMDI,IDTYPE)
 1070 CALL PHASER
      CALL OUTPHI(PHIOUT,WTOUT,NUMREF,NUMDI,NSOUT)
      CLOSE (UNIT=2)
      CLOSE (UNIT=3)
 1090 CONTINUE
      RETURN
      END
C-----------------------------------------------------------------------
C     Unpack indices.
      SUBROUTINE UNPACK(INDEX,IH,IK,IL)
      IH = INDEX / 262144
      IS = INDEX - 262144 * IH
      IK = IS / 512 - 256
      IL = IS - 512 * (IK + 256) - 256
      RETURN
      END
C-----------------------------------------------------------------------
C     Stop RANTAN.
      SUBROUTINE ERROR
      STOP'                             --- Error in RANTAN ---'
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C         RRRRRR   EEEEEEE  L           A     TTTTTTT  N     N         C
C         R     R  E        L          A A       T     NN    N         C
C         R     R  E        L         A   A      T     N N   N         C
C         RRRRRR   EEEEEE   L        A     A     T     N  N  N         C
C         R   R    E        L        AAAAAAA     T     N   N N         C
C         R    R   E        L        A     A     T     N    NN         C
C         R     R  EEEEEEE  LLLLLLL  A     A     T     N     N         C
C                                                                      C
C                           RANTAN  PART  1                            C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE RELATN(JH,JK,JL,EVAL,RPHASE,WEIGHT
     +,           NUMREF,NUMDI,IDTYPE)
      COMMON /USER/ MS,LIST,NSREQ,PROB,NSPEC,NGEN,NANY,KMIN,
     1 IMK,ITAN,IPUB,ISKIP,WTFOM(3),KMAX,IWMIN,NRAN,IOFR,NSOUT,IPSTOR(5)
      COMMON /CONST/ KUSER1, KUSER2,
     1 KUSER3,KUSER4,KUSER5,KUSER6,DTOR,RTOD,STABLE(450)
      COMMON /SYMTRY/ IS(2,3,24), TS(3,24), NSYM, LATT, ICENT, NORI
      COMMON /KFRAG/ SIGMAQ
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ IEE(18000),IOR(18000)
      COMMON /B2/ IPH(18000),LOC(18000)
      COMMON /B4/ X(1000),Y(1000),Z(1000),NZ(1000),IZ(2000),IH(2000),
     1 E(2000),LIM(2101),MKANG(2000)
C     Data from CCP4.
      INTEGER IDTYPE
      INTEGER JH(NUMDI),JK(NUMDI),JL(NUMDI)
      REAL EVAL(NUMDI),RPHASE(NUMDI),WEIGHT(NUMDI)
C
      EXTERNAL ERROR
C
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
      WRITE (6,400)('+',i=1,79)
  400 FORMAT(/1X,79A,//)
      WRITE (6,'(2a)')
     + ' RANTAN PART 1     SET UP SIGMA2 RELATIONSHIPS     ',
     + 'VERSION  OCT. 1997'
      WRITE(6,FMT='(/1X,A,F8.3,A,F8.3)')
     +  'THE LARGE E VALUE FROM',EVAL(1),' TO',EVAL(NUMREF)
        IF(EVAL(NUMREF).GT.3.0) WRITE(6,'(2A)')
     + '  You are only including VERY large Evalues: ',
     + 'perhaps you should increase NREF or decrease EMAX'
        IF(EVAL(NUMREF).GT.3.0) CALL CCPERR(2,
     + 'Only VERY large Evalues used: increase NREF or decrease EMAX?')
      WRITE(6,FMT='(/1X,A,F8.3,A,F8.3)')
     +  'THE SMALL E VALUE FROM',EVAL(NUMREF+2),' TO'
     +  ,EVAL(NUMREF+IZRO+1)
C     INPUT REFLEXION DATA  AND  GENERATE  SYMMETRY  RELATED  REFLEXIONS
  800 CALL INPUT1(NIOR,LIST,JH,JK,JL,EVAL,RPHASE,WEIGHT
     +,           NUMREF,NUMDI,IDTYPE)
      IF (NUMB * NSYM .LE. KUSER6) GO TO 820
      WRITE (6,810) NUMB,NSYM,KUSER6
  810 FORMAT(/' NUMB*NSYM > KUSER6, NUMB =',I5,
     1 '  NSYM =',I5,'  KUSER6 =',I6)
      CALL ERROR
  820 CONTINUE
C     SORT PACKED INDICES KEEPING TRACK OF ADDRESS IN LOC
      CALL SORT1(IOR,LOC,NIOR)
C     SET UP SIGMA2 RELATIONSHIPS
      CALL SIGMA2(NIOR,1,NSRTOT,JH,JK,JL,EVAL,NUMREF,NUMDI)
      WRITE (6,900) NUMB,NSRTOT
  900 FORMAT(/1X,'THE NUMBER OF REFLECTIONS IS',I7,/
     +,   /' THE TOTAL NUMBER OF PHASE RELATIONSHIPS IS',I7)
      IF(NSRTOT/NUMB .LT.10) CALL CCPERR(2,
     + 'Best to have 10*No_refs phase relationships - increase NREF')
      SIGMA=0.005*SIGMA
      SIGMAQ=-REAL(ICENT)
      ICENT=MAX0(ICENT,0)
C     STORE IOR,LOC FOR PSIZERO RELATIONSHIPS LATER
      WRITE (2) IOR,LOC
      DO 1000 I=1,4
      NANT(I) = 0
 1000 CONTINUE
C     READ BACK STRONG RELATIONSHIPS AND SORT THEM
      CALL SORT2(NSRTOT)
      E3MAX=0.01*KMAX
      WRITE (6,1050) NSRTOT,E3MIN,E3MAX
 1050 FORMAT(/' THE NUMBER OF PHASE RELATIONSHIPS SAVED IS',
     1  I7,//,' THE MINIMUM VALUE OF KAPPA IS',F7.2
     2  //,' THE MAXIMUM VALUE OF KAPPA IS',F7.2)
      WRITE(6,1055) SIGMA
 1055 FORMAT(/1X,'THE WEIGHTS FOR PHASE RELATIONSHIPS IS',F7.2)
C     DETERMINE STRUCTURE SEMINVARIANTS FOR ORIGIN DEFINITION AND SIGMA1
C     DETERMINE ALSO ALL PHASE RESTRICTIONS
      CALL GROUP
      IF (IOFR.NE.0) NORI=0
C     SET CONVERGE PARAMETERS
      IF (ICENT .EQ. 1) NANY = NANY + NGEN
      IF (ICENT .EQ. 1) NGEN = 0
      IF (NSPEC+NGEN+NANY.NE.0) WRITE (6,1080) NSPEC,NGEN,NANY
 1080 FORMAT(/' REFLEXIONS REQUESTED FOR STARTING SET ARE  ',
     1 '  NSPEC =',I3,'  NGEN =',I3,'  NANY =',I3)
C     IS ENANTIOMORPH DEFINED BY THE INVARIANTS
      IF (SIGMAQ .GE. 0.25) ICENT = -1
C     APPLY SIGMA1 FORMULA
      CALL SIGMA1(PROB)
C     INPUT USER CHOSEN REFLEXIONS TO THE CONVERGENCE PROCEDURE
      NORIN = 0
C     NANY = -1 DEFINES DEFAULT PARAMETERS
      IF ( NSPEC + NGEN + NANY .EQ. 0) NANY = -1
      IF (NSREQ .GT. 0) WRITE (6,1090) NSREQ
 1090 FORMAT(/' MAXIMUM NUMBER OF SETS REQUIRED =',I5)
      IF (NSREQ.EQ.0) WRITE (6,1095)
 1095 FORMAT(/' MAXIMUM NUMBER OF SETS IS DETERMINED BY PROGRAM')
C     CONVERGENCE METHOD FOR ORIGIN  AND  STARTING  POINT  DETERMINATION
      CALL CONVEG(NORIN)
C     OUTPUT CONVERGENCE RESULTS AND STORE SIGMA2S
      NSRTOT=LIM(NUMB+1)
      CALL OUTPT1(NSRTOT,LIST)
C     READ BACK IOR,LOC AND SET UP PSI-ZEROS
      SIGMA=200.0*SIGMA
      READ (2) IOR,LOC
      CALL SIGMA2(NIOR,2,NSRPSI,JH,JK,JL,EVAL,NUMREF,NUMDI)
      IF (NSRPSI.GT.0) WRITE (6,1700) IZRO,NSRPSI
 1700 FORMAT (//'---------------------------------------------',
     + '------------------------------------------------',
     + //' SET UP PSI ZERO RELATIONSHIPS',
     1//' THE ',I4,' WEAKEST REFLEXIONS TAKE PART IN',I6,
     2 ' RELATIONSHIPS')
      IF (NSRPSI.GT.0) CALL SORT2(NSRPSI)
C     SET UP RELATIONSHIP ARRAYS AND WRITE TAPE
      SIGMA=0.005*SIGMA
 1800 CALL OUTPT2(NSRTOT,NSRPSI)
      RETURN
      END
C-----------------------------------------------------------------------
C     INPUT REFLEXION DATA  AND  GENERATE  SYMMETRY  RELATED  REFLEXIONS
      SUBROUTINE INPUT1(NIOR,LIST,JH,JK,JL,EVAL,RPHASE,WEIGHT
     +,           NUMREF,NUMDI,IDTYPE)
C     DIMENSION STATEMENT PARTICULAR TO SUBROUTINE INPUT1
      DIMENSION LINE(6,4), ENIL(6)
      COMMON /LOCAL/ KL1(24), KL2(24), KL3(24), I1(3), I2(3),BLK1(22)
      COMMON /CONST/ KUSER1, KUSER2,
     1 KUSER3,KUSER4,KUSER5,KUSER6,DTOR,RTOD,STABLE(450)
      COMMON /SYMTRY/ IS(2,3,24), TS(3,24), NSYM, LATT, ICENT, NORI
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ IEE(18000),IOR(18000)
      COMMON /B2/ IPH(18000),LOC(18000)
      COMMON /B4/ X(1000),Y(1000),Z(1000),NZ(1000),IZ(2000),IH(2000),
     1 E(2000),LIM(2101),MKANG(2000)
C     KNOWN PHASES FROM CCP4.
      COMMON /PHIWT/IMKC,KDSC(2000),MKGC(2000),IPKC(2000),WTSC(2000)
C     Data from CCP4.
      INTEGER IDTYPE
      INTEGER JH(NUMDI),JK(NUMDI),JL(NUMDI)
      REAL EVAL(NUMDI),RPHASE(NUMDI),WEIGHT(NUMDI)
      EXTERNAL UNPACK, ERROR
C
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
C     EQUIVALENCE STATEMENT PARTICULAR TO SUBROUTINE INPUT1
      EQUIVALENCE (KL1(1),LINE(1,1)), (KL2(1),ENIL(1))
      DO 52 I=1,3
      MAXH(I) = - 999
   52 CONTINUE
      NUMB=0
      NIOR=0
C     LIMITE = MAXIMUM NUMBER OF REFLEXIONS  (LIMITED BY PACKING)
      LIMITE = MIN0(KUSER3 , 2047)
C     READ REFLEXION DATA  H K L E and PHASE, WEIGHT IF ANY.
C     CODE NUMBERS ARE ASSIGNED TO REFLEXIONS IN ORDER OF INPUT
      IMKC = 0
      DO 480 N=1,NUMREF
      I1(1) = JH(N)
      I1(2) = JK(N)
      I1(3) = JL(N)
      A = EVAL(N)
      IF (NUMB.LE.LIMITE .AND. NIOR .LE. KUSER2-NSYM) GO TO 100
      WRITE (6,70) NUMB
   70 FORMAT(/' * ** *** ** *',/' THERE ARE MORE THAN',I4,
     1' INPUT REFLEXIONS   -   PROGRAM TERMINATED',/,
     2' * ** *** ** *',//)
      CALL ERROR
C     GENERATE SYMMETRY RELATED REFLEXIONS
  100 NUMB = NUMB+1
      MAXI=1
      DO 250 J=1,NSYM
      KL1(J)=24000
      DO 200 I=1,3
      I2(I)=0
      KL1(J)=KL1(J)-I1(I)*INT(TS(I,J)*24.0+0.1)
  200 CONTINUE
      KL1(J)=MOD(KL1(J),24)
      DO 220 I=1,3
      DO 210 K=1,2
      M=IABS(IS(K,I,J))
      IF (M.GT.0) I2(M)=I2(M)+I1(I)*ISIGN(1,IS(K,I,J))
  210 CONTINUE
  220 CONTINUE
      IND=262144*I2(1)+512*I2(2)+I2(3)
      KL2(J)=ISIGN(1,IND)
      KL1(J)=KL1(J)*ISIGN(1,IND)
      KL3(J)=131328+IABS(IND)
      DO 230 I=1,3
      IF (IABS(I2(I)).GT.MAXH(I)) MAXH(I)=IABS(I2(I))
  230 CONTINUE
      IF (J.EQ.1) GO TO 250
      JM1=J-1
      DO 240 I=1,JM1
      IF (KL3(I).EQ.KL3(J)) KL3(J)=0
  240 CONTINUE
      IF (KL3(J).GT.KL3(MAXI)) MAXI=J
  250 CONTINUE
C     EXPRESS GENERATED REFLEXIONS IN TERMS OF STANDARD REFLEXION
      KLS = KL2(MAXI)
      L = KL1(MAXI) * KLS
C     STORE STANDARD REFLEXIONS
      IH(NUMB)=KL3(MAXI)
      E(NUMB)=A
C     Input known phases and weights.
      IF (IDTYPE.EQ.2.AND.WEIGHT(N).GT.0.85) THEN
       IMKC = IMKC + 1
       KDSC(IMKC) = N
       MKGC(IMKC) = 1
       IF (WEIGHT(N).GT.0.98) MKGC(IMKC) = -1
       IPKC(IMKC) = INT(RPHASE(N) + 360.5)
       IPKC(IMKC) = MOD(IPKC(IMKC) + L * 15 + 360, 360)
       IF (IPKC(IMKC).EQ.0) IPKC(IMKC) = 360
       WTSC(IMKC) = WEIGHT(N)
      END IF
      DO 450 J=1,NSYM
      IF (KL3(J).EQ.0) GO TO 450
      KL1(J) = MOD(KL1(J) - KL2(J) * L,24)
      IF (KL1(J).LT.0) KL1(J)=24+KL1(J)
      KL2(J)=KL2(J) * KLS
      IF (KL2(J).EQ.(-1)) KL2(J)=0
C     STORE PACKED INDICES IN IOR AND CODE IN LOC
      NIOR=NIOR+1
      IOR(NIOR)=KL3(J)
      LOC(NIOR)=48*NUMB+2*KL1(J)+KL2(J)
  450 CONTINUE
  480 CONTINUE
      IF (IMKC.GT.0) IPKC(IMKC+1) = 555
      IF (LIST.GE.1) WRITE (6,720) NUMB
  720 FORMAT (/1X,I5,' STANDARD REFLEXIONS',
     1//3(' CODE  H  K  L   E  '))
C     PRINT STANDARD REFLEXIONS IN ORDER OF SORTED INDICES
C     SORT REFLEXIONS ON INDICES KEEPING TRACK OF CODE NUMBERS IN LIM
      DO 730 I=1,NUMB
      LIM(I) = I
  730 CONTINUE
      CALL SORT1(IH,LIM,NUMB)
      K = 0
      DO 800 J=1,NUMB
      K = K + 1
      I = NUMB + 1 - J
      LINE(K,1) = LIM(I)
      CALL UNPACK(IH(I),LINE(K,2),LINE(K,3),LINE(K,4))
      IF (I.NE.1.AND.IH(I).EQ.IH(I-1)) WRITE (6,750) (LINE(K,L),L=2,4)
  750 FORMAT(' WARNING - REFLEXION',3I5,'  OCCURS TWICE IN INPUT DATA'/)
      LIM(I) = -LIM(I)
      I = -LIM(I)
      ENIL(K) = E(I)
      IF (K.LT.6) GO TO 800
      IF (LIST.GE.1) WRITE (6,760) ((LINE(L,M),M=1,4),ENIL(L),L=1,6)
  760 FORMAT (3(I5,3I3,F6.3))
      K = 0
  800 CONTINUE
      IF (K.NE.0.AND.LIST.GE.1) WRITE (6,760) ((LINE(L,M),M=1,4),
     1 ENIL(L),L=1,K)
C     RESORT REFLEXIONS TO INITIAL ORDER
      CALL SORT1(LIM,IH,NUMB)
      RETURN
      END
C-----------------------------------------------------------------------
C     GENERAL PURPOSE SORT ON TWO INTEGER ARRAYS
      SUBROUTINE SORT1(IOR,LOC,MOST)
      DIMENSION IOR(MOST), LOC(MOST)
      INT = 2
 1000 INT = 2*INT
      IF (INT .LT. MOST) GO TO 1000
      INT = MIN0(MOST,(3*INT)/4-1)
 1020 INT = INT/2
      IFIN = MOST - INT
      DO 1120 II=1,IFIN
      I = II
      J = I + INT
      IF (IOR(I).GE.IOR(J)) GO TO 1120
      L = IOR(J)
      M = LOC(J)
 1060 IOR(J) = IOR(I)
      LOC(J) = LOC(I)
      J = I
      I = I - INT
      IF (I.LE.0) GO TO 1100
      IF (IOR(I).LT.L) GO TO 1060
 1100 IOR(J) = L
      LOC(J) = M
 1120 CONTINUE
      IF (INT .GT. 1) GO TO 1020
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C       SSSSS    IIIII    GGGGG   M     M     A            22222       C
C      S     S     I     G     G  MM   MM    A A          2     2      C
C      S           I     G        M M M M   A   A               2      C
C       SSSSS      I     G  GGGG  M  M  M  A     A  ====    2222       C
C            S     I     G     G  M     M  AAAAAAA         2           C
C      S     S     I     G     G  M     M  A     A        2            C
C       SSSSS    IIIII    GGGGG   M     M  A     A        2222222      C
C                                                                      C
C                      Set up SIGMA2 relationships.                    C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     JUMP=2 : PSI ZERO'S ;  JUMP=1 : SIGMA2'S
      SUBROUTINE SIGMA2(NIOR,JUMP,NSR,JH,JK,JL,EVAL,NUMREF,NUMDI)
C     DIMENSION STATEMENT PARTICULAR TO SUBROUTINE SIGMA2
      DIMENSION NKEY(2000)
      COMMON/LOCAL/I1(3),I2(3),I3(3),I4(3),I5(3),BLK1(85)
      COMMON /CONST/ KUSER1, KUSER2,
     1 KUSER3,KUSER4,KUSER5,KUSER6,DTOR,RTOD,STABLE(450)
      COMMON /KFRAG/ SIGMAQ
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ IEE(18000),IOR(18000)
      COMMON /B2/ IPH(18000),LOC(18000)
      COMMON /B3/ KNOWN(8000)
      COMMON /B4/ X(1000),Y(1000),Z(1000),NZ(1000),IZ(2000),IH(2000),
     1 E(2000),LIM(2101),MKANG(2000)
C     Data from CCP4.
      INTEGER JH(NUMDI),JK(NUMDI),JL(NUMDI)
      REAL EVAL(NUMDI)
      EXTERNAL UNPACK
C
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
C     EQUIVALENCE STATEMENT PARTICULAR TO SUBROUTINE SIGMA2
      EQUIVALENCE (NKEY(1),KNOWN(1))
C     ARRAY LIM CONTAINS ADDRESS OF H-H' IN ARRAY IOR
      REWIND 2
      LIMIT=KUSER4-KUSER1
      KONST=512*(256-MAXH(2))
      KONH=KUSER5/(MAXH(1)+1)
      KONK=512*(2*MAXH(2)+1)/KONH+1
      DO 500 I=1,KUSER5
      LIM(I+1) = 0
  500 CONTINUE
      N=KONH*MAXH(1)
      M=262144*MAXH(1)
      DO 540 I=1,NIOR
  510 IF (IOR(I).GT.M) GO TO 520
      M=M-262144
      N=N-KONH
      GO TO 510
  520 J=(IOR(I)-M-KONST)/KONK+N
      LIM(J+1)=I
  540 CONTINUE
      KU5M1=KUSER5-1
      DO 570 I=1,KU5M1
      J = KUSER5 - I
      IF (LIM(J).EQ.0) LIM(J) = LIM(J+1)
  570 CONTINUE
      IP = 47
      NSRT=0
      NSR = 0
      I = 1
      IF (JUMP .EQ. 2) GO TO 600
      NREC=0
  580 IF (I .LE. NUMB) GO TO 590
      GO TO 800
C     I1 CONTAINS INDICES OF VECTOR H
  590 CALL UNPACK(IH(I),I1(1),I1(2),I1(3))
      IP = IP+48
      EVA = E(I)
      GO TO 610
C     INPUT REFLEXIONS FOR PSI ZERO
  600 IF (NUMB+I.GT.KUSER5) GO TO 603
      I1(1) = JH(NUMREF+1+I)
      I1(2) = JK(NUMREF+1+I)
      I1(3) = JL(NUMREF+1+I)
      EVA = EVAL(NUMREF+1+I)
      IP=100000
      IF (EVA .GE. 0.0) GO TO 605
  603 IZRO = I - 1
      IF (NSR.EQ.0) IZRO=0
      IF (IZRO.NE.0) GO TO 800
      RETURN
  605 EVA = 1.0
  610 NP=0
      DO 740 K=1,NIOR
      IF (LOC(K).GT.IP) GO TO 740
      CALL UNPACK(IOR(K),I2(1),I2(2),I2(3))
      J=LOC(K)/48
      IF (JUMP.EQ.2.AND.NKEY(J).GT.NDET) GO TO 740
      IFAZ2=LOC(K)-48*J
      ISI2=(2*MOD(IFAZ2,2)-1)*J
      IFAZ2=IFAZ2/2
      DO 730 M=1,3,2
      L=M-2
      DO 620 N=1,3
      I2(N) = - I2(N)
  620 CONTINUE
C     I3 CONTAINS INDICES OF VECTOR H-H'
      DO 640 N=1,3
      I3(N)=I1(N) + I2(N)
      IF (IABS(I3(N)).GT.MAXH(N)) GO TO 730
  640 CONTINUE
      INDEX=262144*I3(1)+512*I3(2)+I3(3)
      MARK=-ISIGN(1,INDEX)
      INDEX=131328+IABS(INDEX)
      II1=IABS(I3(1))
      KUT=(INDEX-262144*II1-KONST)/KONK+II1*KONH+1
      LIM1=LIM(KUT+1)
      LIM2=LIM(KUT)
      IF (LIM1.GE.LIM2.OR.INDEX.LT.IOR(NIOR)) GO TO 730
C     LOOK FOR H-H' IN ARRAY IOR BETWEEN LIMITS LIM1 AND LIM2
      N=LIM2-LIM1+1
  690 N=(N+1)/2
      IR=LIM1+N
      IF (IOR(IR)-INDEX) 696,700,692
  692 LIM1=IR
  696 IF (N.LE.1) GO TO 730
      GO TO 690
  700 K3=LOC(IR)/48
      IF (JUMP.EQ.2.AND.NKEY(K3).GT.NDET) GO TO 730
      IF (I.EQ.J.AND.JUMP.EQ.1) GO TO 710
      IF (K3.GE.J) GO TO 730
  710 IFAZ3=LOC(IR)-48*K3
      ISI3=2*MOD(IFAZ3,2)-1
      IFAZ3=IFAZ3/2
C     ELIMINATE DUPLICATIONS
      IND=2048*J+K3
      IF (NP.LE.0) GO TO 718
      DO 715 IR=1,NP
      IF (IND.EQ.MKANG(IR)) GO TO 730
  715 CONTINUE
  718 IF (I .NE. J .OR. JUMP .EQ. 2) GO TO 720
C     POSSIBLE SIGMA1 RELATIONSHIP  -  STORE IN SEPARATE ARRAY
      NREC = NREC + 1
      IF (NREC.GT.KUSER3) GO TO 725
      ISI = ISIGN(1,MARK * ISI3)
      IFAZE = MOD(-L * IFAZ2 * ISI - ISI3*IFAZ3+240, 24)
      IZ(NREC)=ISIGN(131072*K3+32*(I*ISI+2048)+IFAZE,L*ISI2*ISI)
      GO TO 725
  720 NSR = NSR + 1
      SIGMAQ=SIGMA
      E3=SIGMAQ*EVA*E(J)*E(K3)
      NSRT=NSRT+1
      IF (NSRT.LE. LIMIT) GO TO 722
C     STORE ON TAPE FOR USE IN SORT2
      WRITE (2) (IPH(II),IEE(II),II=1,LIMIT)
      NSRT=1
C     PACK RELATIONSHIPS INTO IPH AND IEE
  722 IFAZE=MOD(-IFAZ2*L-MARK*IFAZ3+240,24)
      IF (IFAZE .LT. 0) IFAZE = IFAZE + 24
      IPH(NSRT)=131072*(2048-L*ISI2)+32*(2048-ISIGN(K3,ISI3)*MARK)+IFAZE
      IEE(NSRT)=4096*INT(E3+0.5)+2048-I
  725 NP = NP + 1
      MKANG(NP) = IND
  730 CONTINUE
  740 CONTINUE
      IF (NREC.GT.KUSER3) NREC=KUSER3
      I = I + 1
      GO TO (580,600), JUMP
  800 WRITE (2) (IPH(II),IEE(II),II=1,LIMIT)
      RETURN
      END
C-----------------------------------------------------------------------
C     READ BACK RELATIONSHIPS AND SORT FOR CONVERGE
      SUBROUTINE SORT2(NSR)
C     DIMENSION STATEMENT PARTICULAR TO SUBROUTINE SORT2
      DIMENSION IEE(36000)
      COMMON /USER/ MS,LIST,NSREQ,PROB,NSPEC,NGEN,NANY,KMIN,
     1 IMK,ITAN,IPUB,ISKIP,WTFOM(3),KMAX,IWMIN,NRAN,IOFR,NSOUT,IPSTOR(5)
      COMMON /CONST/ KUSER1, KUSER2,
     1 KUSER3,KUSER4,KUSER5,KUSER6,DTOR,RTOD,STABLE(450)
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ EEE(36000)
      COMMON /B2/ IPH(36000)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),IH(2000),E(2000),
     1 LIM(2101),MKANG(2000)
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
C     EQUIVALENCE STATEMENT PARTICULAR TO SUBROUTINE SORT2
      EQUIVALENCE (EEE(1),IEE(1))
      REWIND 2
      DO 50 I=1,KUSER5
      LIM(I+1)=0
   50 CONTINUE
      LIMIT=KUSER4-KUSER1
      MAX=KUSER1/LIMIT
      MAX=MAX*LIMIT
      LF=0
      L=0
C     READ BACK AND SORT RELATIONSHIPS
  100 LS=LF+1
      LF=LF+LIMIT
      L=L+LIMIT
      IF (L.GT.NSR) LF=LF-L+NSR
      READ (2) (IPH(I),IEE(I),I=LS,LF)
      IF (L.LT.NSR.AND.LF.LE.MAX-LIMIT) GO TO 100
      CALL SORT1(IEE,IPH,LF)
      IF (NSR.LT.KUSER1) GO TO 130
      LF=KUSER1
      IF (L.LT.NSR) GO TO 100
      NSR=LF
C     GET MAXIMUM KAPPA
 130  NDD=0
      DO 131 I=1,NSR
      IE3=IEE(I)/4096
      IF (IE3.LE.KMAX) GO TO 132
      NDD=NDD+1
 131  CONTINUE
 132  KMAX=IE3
      IF (NDD.EQ.0) GO TO 138
      NDD1 = NDD+1
      DO 133 I=NDD1,NSR
      IEE(I-NDD)=IEE(I)
      IPH(I-NDD)=IPH(I)
 133  CONTINUE
 135  NSR=NSR-NDD
C     GET MINIMUM KAPPA AND SORT RELATIONSHIPS IN FORM
C     SUITABLE FOR CONVERGE
  138 DO 140 I=1,NSR
      IE3=IEE(I)/4096
      IF (IE3.LT.KMIN) GO TO 150
      J=IEE(I)-4096*IE3
      IEE(I)=524288*J+IE3
  140 CONTINUE
      I = NSR + 1
  150 NSR = I - 1
      IE3 = IEE(NSR)-524288*(IEE(NSR)/524288)
      E3MIN = 0.01 * IE3
      CALL SORT1(IEE,IPH,NSR)
      LIM(1)=0
      DO 200 I=1,NSR
      J=IEE(I)/524288
      EEE(I)=0.01*REAL(IEE(I)-524288*J)
      J=2048-J
      LIM(J+1)=I
  200 CONTINUE
      DO 250 I=1,KUSER5
      LIM(I+1)=MAX0(LIM(I+1),LIM(I))
  250 CONTINUE
C     SET KMIN = 0 FOR PSIZROS ON SECOND CALL OF SORT2
      KMIN = 0
      RETURN
      END
C-----------------------------------------------------------------------
C     DETERMINE SPACE GROUP TYPE ACCORDING TO THE CATEGORIES OF HAUPTMAN
C     & KARLE,  ACTA CRYST. (1956) 9, 45  AND  ACTA CRYST. (1959) 12, 93
C     AND KARLE & HAUPTMAN,  ACTA CRYST. (1961) 14, 217,  GIVEN ONLY THE
C     GENERAL EQUIVALENT POSITIONS AND LATTICE TYPE  -  GIVES  NECESSARY
C     INFORMATION ON NUMBER AND  TYPE  OF  REFLEXIONS  USED  FOR  ORIGIN
C     DEFINITION, SEMINVARIANT MODULUS & REDUCED INDICES FOR  REFLEXIONS
C     ROUTINE WILL HANDLE 230 SPACE GROUPS IN THE STANDARD  ORIENTATIONS
C     INCLUDING ALTERNATIVE SETTINGS AS IN INTERNATIONAL TABLES VOLUME 1
C     DETERMINE  ALSO  THE  PHASE   RESTRICTIONS  FOR   ALL   REFLEXIONS
C     TEST  WHETHER  THE  ENANTIOMORPH  IS  FIXED  BY  THE  SPACE  GROUP
      SUBROUTINE GROUP
      COMMON/LOCAL/ IND(3),KHL(4),BLK1(93)
      COMMON /SYMTRY/ IS(2,3,24), TS(3,24), NSYM, LATT, ICENT, NORI
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ EEE(32000),ALPHA(2000),WT(2000)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),IH(2000),E(2000),
     1 LIM(2101),MKANG(2000)
      EXTERNAL UNPACK
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
C     INITIALIZE PARAMETERS
      NRAL = 0
      DO 1000 I=1,3
      MODUL(I) = 2
 1000 CONTINUE
C     IT = 3  TESTS FOR Z IN 1ST COORD. OF GENERAL EQUIVALENT  POSITIONS
C     -  GIVES RHOMBOHEDRAL OR CUBIC
C     IT = 2  TESTS FOR Y IN 1ST COORD. - GIVES TETRAGONAL, HEXAGONAL OR
C     TRIGONAL
      DO 1060 II=1,2
      IT = 4 - II
      DO 1040 J=1,NSYM
      IF (IABS(IS(1,1,J)) - IT) 1020,1500,1020
 1020 IF (IABS(IS(2,1,J)) - IT) 1040,1500,1040
 1040 CONTINUE
 1060 CONTINUE
C     SPACE GROUP MUST BE TRICLINIC,  MONOCLINIC  OR  ORTHORHOMBIC
C     THREE REFLEXIONS ARE NORMALLY REQUIRED FOR ORIGIN DEFINITION
      NORI = 3
      IF (ICENT) 1080,1080,1160
C     DETERMINE  SEMINVARIANT  MODULUS VECTOR (MODUL) AND NUMBER OF AXES
C     ALONG  WHICH THE  ORIGIN  POSITION IS INFINITELY  VARIABLE  (NRAL)
 1080 NRAL = 3
      DO 1140 I=1,3
      MODUL(I) = 1000
      DO 1100 J=1,NSYM
      IF (IS(1,I,J)) 1120,1100,1100
 1100 CONTINUE
      GO TO 1140
 1120 MODUL(I) = 2
      NRAL = NRAL - 1
 1140 CONTINUE
C     CHECK FOR CENTRED LATTICE
 1160 GO TO (1380,1260,1180,1220,1380,1320,1380), LATT
C     B CENTRED LATTICE  -  TEST FOR B2  OR  BM, BB
 1180 IF (NRAL.LE.1) GO TO 1380
 1200 II= 2
      MODUL(3) = 1000
      GO TO 1280
C     C CENTRED LATTICE  -  TEST FOR C2  OR  CM, CC
 1220 IF (NRAL.LE.1) GO TO 1380
 1240 MODUL(2) = 1000
 1260 II = 3
C     A CENTRED LATTICES, BM, BB, CM, CC
C     DETERMINE REDUCED INDICES FOR ALL REFLEXIONS
 1280 DO 1300 I=1,NUMB
      CALL UNPACK(IH(I),IND(1),IND(2),IND(3))
      MKG(I)=512*(MOD(IND(1),MODUL(2))+256)+MOD(IND(II),MODUL(3))+256
 1300 CONTINUE
      GO TO 2000
C     F CENTRED LATTICE - TEST POINT GROUP SYMMETRY AROUND (1/4,1/4,1/4)
C     TO DETECT F222 IF SPACE GROUP IS NON-CENTROSYMMETRIC
 1320 IF (ICENT.GT.0) GO TO 1980
      CALL TEST(1,1,1,4,MM)
      IF (MM.EQ.0) GO TO 1960
C     TRICLINIC, MONOCLINIC & ORTHORHOMBIC EXCEPT A CENTRED, BM,BB,CM,CC
C     DETERMINE REDUCED INDICES FOR ALL REFLEXIONS
 1380 DO 1400 I=1,NUMB
      CALL UNPACK(IH(I),IND(1),IND(2),IND(3))
      MKG(I)=262144*MOD(IND(1),MODUL(1))+512*(MOD(IND(2),MODUL(2)
     1  ) + 256) + MOD(IND(3),MODUL(3)) + 256
 1400 CONTINUE
      GO TO 2000
C     SPACE GROUP MUST BE TETRAGONAL, TRIGONAL, HEXAGONAL OR CUBIC
 1500 IF (ICENT) 1520,1520,1640
 1520 IF (LATT - 1) 1600,1540,1600
C     TEST WHETHER ENANTIOMORPH IS FIXED BY THE SPACE GROUP
 1540 DO 1560 I=1,NSYM
      IT = INT(24.0 * TS(3,I) + 0.1)
      IF (MOD(IT,12)) 1580,1560,1580
 1560 CONTINUE
      GO TO 1600
C     ENANTIOMORPH FIXED BY SPACE GROUP  -  SET FLAG
 1580 ICENT = -1
C     IS ORIGIN POSITION INFINITELY VARIABLE ALONG Z
C     SET FLAG (NRAL) AND 3RD COMPONENT OF SEMINVARIANT  MODULUS  VECTOR
 1600 DO 1620 I=1,NSYM
      IF (IS(1,3,I)) 1640,1620,1620
 1620 CONTINUE
C     SPACE GROUP MUST BE 2P20, 3P30, 3P(1)0 OR 3P(2)0
      NRAL = 1
      MODUL(3) = 1000
C     BRANCH FOR RHOMBOHEDRAL AND CUBIC
 1640 GO TO (1940,1660), II
C     TEST POINT GROUP SYMMETRY AROUND (1/2,1/2,0) TO DETECT  TETRAGONAL
 1660 CALL TEST(1,1,0,2,MM)
      IF (MM) 1820,1680,1820
C     SPACE GROUP MUST BE TETRAGONAL
C     TWO REFLEXIONS ARE NORMALLY REQUIRED FOR ORIGIN DEFINITION
 1680 NORI = 2
      IF (LATT.NE.5) GO TO 1780
C     I CENTRED  TETRAGONAL    -    TEST  POINT  GROUP  SYMMETRY  AROUND
C     (0,1/2,1/4)  TO  DETECT  BAR 4  AXIS
      CALL TEST(0,2,1,4,MM)
      IF (MM.NE.0) GO TO 1780
C     SPACE GROUP MUST BE 3P(3)4
      MODUL(3) = 4
C     DETERMINE REDUCED INDICES FOR ALL REFLEXIONS
      DO 1740 I=1,NUMB
      CALL UNPACK(IH(I),IND(1),IND(2),IND(3))
      MKG(I) = MOD(2 * IND(2) + IND(3), MODUL(3)) + 131328
 1740 CONTINUE
      GO TO 2000
C     SPACE GROUP MUST BE 3P(3)0 OR 3P(3)2
C     ENTER HERE FOR PRIMITIVE TETRAGONAL
C     DETERMINE REDUCED INDICES FOR ALL REFLEXIONS
 1780 DO 1800 I=1,NUMB
      CALL UNPACK(IH(I),IND(1),IND(2),IND(3))
      MKG(I) = 512*(MOD(IND(1) + IND(2), MODUL(2)) + 256) + MOD(IND(3),
     1  MODUL(3)) + 256
 1800 CONTINUE
      GO TO 2000
C     SPACE GROUP MUST BE TRIGONAL OR HEXAGONAL
 1820 IF (ICENT) 1840,1840,1900
C     TEST POINT GROUP SYMMETRY AROUND (1/3,2/3,0) TO DETECT 3P30 & 3P32
 1840 IF (LATT - 7) 1850,1900,1900
 1850 CALL TEST(1,2,0,3,MM)
      IF (MM) 1900,1860,1900
C     SPACE GROUP MUST BE 3P30 OR 3P32
 1860 MODUL(2) = 3
C     DETECT 6P6
      IF (MODUL(3).EQ.2) GO TO 1890
C     TWO REFLEXIONS ARE NORMALLY REQUIRED FOR ORIGIN DEFINITION
      NORI = 2
C     DETERMINE REDUCED INDICES FOR ALL REFLEXIONS
      DO 1880 I=1,NUMB
      CALL UNPACK(IH(I),IND(1),IND(2),IND(3))
      MKG(I) = 512*(MOD(IND(1) - IND(2), MODUL(2)) + 256) + MOD(IND(3),
     1  MODUL(3)) + 256
 1880 CONTINUE
      GO TO 2000
C     ONE REFLEXION NORMALLY REQUIRED FOR ORIGIN DEFINITION
 1890 NORI=1
      MODUL(3)=6
C     DETERMINE REDUCED REFLEXIONS FOR ALL REFLEXIONS
      DO 1895 I=1,NUMB
      CALL UNPACK(IH(I),IND(1),IND(2),IND(3))
      MKG(I)=MOD(2*IND(1)+4*IND(2)+3*IND(3),MODUL(3))
 1895 CONTINUE
      GO TO 2000
C     SPACE GROUP MUST BE 3P(1)0, 3P(1)2 OR RHOMBOHEDRAL USING HEXAGONAL
C     AXES
C     ONE REFLEXION IS NORMALLY REQUIRED FOR ORIGIN DEFINITION
 1900 NORI = 1
C     DETERMINE REDUCED INDICES FOR ALL REFLEXIONS
      DO 1920 I=1,NUMB
      CALL UNPACK(IH(I),IND(1),IND(2),IND(3))
      MKG(I) = MOD(IND(3),MODUL(3)) + 131328
 1920 CONTINUE
      GO TO 2000
C     SPACE GROUP MUST BE 3P(2)0, 3P(2)2, 3P(2)4 OR 4P111
C     TEST POINT GROUP SYMMETRY AROUND  (1/4,1/4,1/4)  TO DETECT  3P(2)4
 1940 IF (ICENT) 1950,1950,1980
 1950 IF (LATT - 7) 1960,1980,1980
 1960 CALL TEST(1,1,1,4,MM)
      IF (MM) 1980,1970,1980
 1970 MODUL(3) = 4
C     ONE REFLEXION IS NORMALLY REQUIRED FOR ORIGIN DEFINITION
 1980 NORI = 1
C     DETERMINE REDUCED INDICES FOR ALL REFLEXIONS
      DO 1990 I=1,NUMB
      CALL UNPACK(IH(I),IND(1),IND(2),IND(3))
      MKG(I) = MOD(IND(1) + IND(2) + IND(3), MODUL(3)) + 131328
 1990 CONTINUE
C     CALCULATE PHASE RESTRICTIONS ON ALL REFLEXIONS
 2000 DO 2040 I=1,NUMB
      MKANG(I) = 13
      ALPHA(I) = 0.0
      PALF(I) = 0.0
 2040 CONTINUE
      IF (ICENT .GT . 0) GO TO 2240
      DO 2200 I=1,NUMB
      MKANG(I) = 1
      CALL UNPACK(IH(I),IND(1),IND(2),IND(3))
C     GENERATE SYMMETRY RELATED REFLEXIONS
      DO 2180 J=1,NSYM
      DO 2080 K=1,4
      KHL(K) = 0
 2080 CONTINUE
      DO 2140 K=1,3
      KHL(4) = KHL(4) + IND(K) * INT(TS(K,J) * 12.0 + 0.1)
      DO 2120 L=1,2
      M = IABS(IS(L,K,J))
      IF (M .NE. 0) KHL(M) = KHL(M) + IND(K) * ISIGN(1,IS(L,K,J))
 2120 CONTINUE
 2140 CONTINUE
C     HAS FRIEDEL OPPOSITE BEEN GENERATED
      DO 2160 K=1,3
      IF (IND(K) + KHL(K)) 2180,2160,2180
 2160 CONTINUE
C     YES - PHASE MUST BE RESTRICTED TO  PI*KHL(4)/24 OR PI*KHL(4)/24+PI
      MKANG(I) = MOD(KHL(4), 12) + 1
      IF (MKANG(I) .LE. 1) MKANG(I) = MKANG(I) + 12
      GO TO 2200
 2180 CONTINUE
 2200 CONTINUE
      DO 2220 I=1,3
      MODUL(I) = MOD(MODUL(I),1000)
 2220 CONTINUE
C     ADJUST NUMBER OF ORIGIN  DEFINING  REFLEXIONS FOR CENTRED LATTICES
 2240 GO TO (2280,2260,2260,2260,2260,2300,2280), LATT
C     A, B, C AND I CENTRED LATTICES
 2260 NORI = NORI - 1
 2280 RETURN
C     F CENTRED LATTICES
 2300 NORI = IABS(NORI - 2)
      RETURN
      END
C-----------------------------------------------------------------------
C     TEST TO SEE IF THE POINT (M1/M4,M2/M4,M3/M4) IS A POSSIBLE  ORIGIN
C     POSITION     MM .EQ. 0  IF YES     MM .NE. 0  IF NO
C     THIS ROUTINE IS WRITTEN TO TEST  NON-CENTROSYMMETRIC  SPACE GROUPS
C     WITH P, I AND F LATTICES ONLY
      SUBROUTINE TEST(M1,M2,M3,M4,KNT)
      DIMENSION MATRIX(3,3)
      COMMON /SYMTRY/ IS(2,3,24), TS(3,24), NSYM, LATT, ICENT, NORI
      DO 1220 I=1,NSYM
      DO 1020 J=1,3
      DO 1000 K=1,3
      MATRIX(J,K) = 0
 1000 CONTINUE
      MATRIX(J,J) = -1
 1020 CONTINUE
      DO 1080 J=1,2
      DO 1060 K=1,3
      M = IABS(IS(J,K,I))
      IF (M) 1060,1060,1040
 1040 MATRIX(K,M) = MATRIX(K,M) + ISIGN(1,IS(J,K,I))
 1060 CONTINUE
 1080 CONTINUE
      KNT = 0
      DO 1140 J=1,3
      MM = IABS(MOD(MATRIX(J,1) * M1 + MATRIX(J,2) * M2 + MATRIX(J,3) *
     1  M3, M4))
      IF (MM) 1140,1140,1100
 1100 IF (2 * MM - M4) 1260,1120,1260
 1120 KNT = KNT + 1
 1140 CONTINUE
      IF (KNT) 1220,1220,1160
 1160 GO TO (1240,1240,1240,1240,1180,1200,1240), LATT
 1180 IF (KNT - 3) 1240,1220,1240
 1200 IF (KNT - 2) 1240,1220,1240
 1220 CONTINUE
      KNT = 0
 1240 RETURN
 1260 KNT = 1
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C          CCCCC    OOOOO   N     N  V     V  EEEEEEE  RRRRRR          C
C         C     C  O     O  NN    N  V     V  E        R     R         C
C         C        O     O  N N   N  V     V  E        R     R         C
C         C        O     O  N  N  N  V     V  EEEEEE   RRRRRR          C
C         C        O     O  N   N N   V   V   E        R   R           C
C         C     C  O     O  N    NN    V V    E        R    R          C
C          CCCCC    OOOOO   N     N     V     EEEEEEE  R     R         C
C                                                                      C
C               Convergence map to select the refletions               C
C                     fix origin and enantiomorph                      C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE CONVEG(NORIN)
C     DIMENSION STATEMENT PARTICULAR TO SUBROUTINE CONVEG
      DIMENSION IEE(32000),LINE(5,6)
      CHARACTER*1 ISK,NOT,ich(6)
      COMMON /USER/ MS,LIST,NSREQ,PROB,NSPEC,NGEN,NANY,KMIN,
     1 IMK,ITAN,IPUB,ISKIP,WTFOM(3),KMAX,IWMIN,NRAN,IOFR,NSOUT,IPSTOR(5)
      COMMON /CONST/ KUSER1, KUSER2,
     1 KUSER3,KUSER4,KUSER5,KUSER6,DTOR,RTOD,STABLE(450)
      COMMON /SYMTRY/ IS(2,3,24), TS(3,24), NSYM, LATT, ICENT, NORI
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ EEE(32000),ALPHA(2000),BUFR(400),IBUFR(4,400)
      COMMON /B2/ IPH(32000),ALFST(2000),IORDE(2000)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),IH(2000),E(2000),
     1 LIM(2101),MKANG(2000)
      EXTERNAL UNPACK
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
C     EQUIVALENCE STATEMENT PARTICULAR TO SUBROUTINE CONVEG
      EQUIVALENCE (EEE(1),IEE(1))
C     I1/I0 (BESSEL FUNCTION)
      VEC(U)=U*(U+0.4807)/((U+0.8636)*U+1.3943)
C     SET DEFAULTS
       ISK = '*'
       NOT=' '
      ILIST = -1
      NSR = 0
      NGR = 0
      MARK = -1
      IF (NANY .EQ. (-1)) NSPEC = 10
      IF (NANY .EQ. (-1)) NGEN = 10
      ITT = NSREQ
      NSREQ = 100
C     ALL ORIGIN FIXING REFLEXIONS MUST APPEAR IN THE BOTTOM  25  PLACES
C     OF THE CONVERGENCE MAP
      LKGIN = NORI + NSPEC + NGEN + NANY + 25
      IF (NANY .EQ. (-1)) LKGIN = LKGIN-12
C     REFLEXIONS IN BOTTOM 1/3 OF CONVERGENCE MAP MAY GO IN STARTING SET
      LKSTA = NUMB / 3
      CALL CALPHA(LARN,NORIN,LIST,LKGIN,NSPEC)
      NSRTOT=LIM(NUMB+1)
C     CALCULATE NUMBER OF PHASES TO BE DETERMINED
      NDET = MIN0(NDET,NUMB)
      IF (NDET.EQ.0) NDET=NUMB-MIN0(KUSER3-NUMB,50)
      IF (IMK.GT.50) NDET = NUMB
C     INITIALISE COUNTER OF SPACE FOR PSIZERO RELATIONSHIPS
      NREJ=KUSER1-NSRTOT
      IF (NSPEC+NGEN+NANY.EQ.0) GO TO 1770
C     INITIALISE ARRAYS FOR NEXT PASS OF CONVERGE
 1200 KNT = NUMB
      DO 1210 I=1,NUMB
      IF (IZ(I) .LT. (-10)) IZ(I) = 0
      ALPHA(I) = PALF(I)
 1210 CONTINUE
 1220 DO 1230 I=1,NSRTOT
      IF (KNT.LT.NUMB.AND.EEE(I).LT.0.0) GO TO 1230
      IF (IPH(I).LT.0) IPH(I)=-IPH(I)
      IF (EEE(I).LT.0.0) EEE(I)=-EEE(I)
 1230 CONTINUE
C     LOOK FOR SMALLEST ESTIMATED ALPHA
 1240 ALFMIN=1.0E10
      DO 1270 I=1,NUMB
      IF (IZ(I).NE.0.OR.ALPHA(I).GE.ALFMIN) GO TO 1270
      MIN=I
      ALFMIN=ALPHA(I)
 1270 CONTINUE
      IF (ALFMIN.GT.0.9E10) GO TO 1560
      IZ(MIN)=-100
      IF (ALFMIN .LT. 0.3) ALPHA(MIN) = 0.0
C     THIS REFLEXION IS ELIMINATED
      IF (KNT.GT.ILIST) GO TO 1435
      IF (KNT.EQ.ILIST) WRITE (6,1400)
 1400 FORMAT (/' CODE   H  K  L  ALFEST        CONTRIBUTORS'/)
      CALL UNPACK(IH(MIN),IJ,IK,IL)
      WRITE (6,1405) MIN,IJ,IK,IL,ALFMIN
 1405 FORMAT(/1X,2I4,2I3,F6.2)
 1435 LAST = MIN
C     RECORD ORDER OF PHASE DETERMINATION
      IORDE(KNT) = MIN
      IF (KNT.NE.LKSTA.OR.ILIST.GE.0) GO TO 1445
      DO 1440 I=1,NUMB
      ALFST(I)=ALPHA(I)
 1440 CONTINUE
C     REDUCE STRONG REFS TO MAKE SPACE FOR PSI-ZEROS IF NECESSARY
 1445 IF (ILIST.GE.0.AND.KNT.EQ.NDET.AND.NREJ.LT.2000) NDET=NDET-1
C     UPDATE ESTIMATED ALPHAS
      N=0
      DO 1500 IL=MIN,NUMB
      IF (IL.NE.MIN.AND.IZ(IL).LT.(-10)) GO TO 1500
      LI=LIM(IL)+1
      LS=LIM(IL+1)
      IF (LI.GT.LS) GO TO 1500
      DO 1480 I=LI,LS
      IF (IPH(I).LT.0) GO TO 1480
      IJ=IPH(I)/131072-2048
      IJA=IABS(IJ)
      IF (IL.NE.MIN.AND.IJA.LT.MIN) GO TO 1480
      IK=(IPH(I)-131072*(IJ+2048))/32-2048
      IKA=IABS(IK)
      IF (IL.NE.MIN.AND.IJA.NE.MIN.AND.IKA.NE.MIN) GO TO 1480
      VVEC=EEE(I)*VEC(EEE(I))
      IF (IZ(IL).EQ.0) ALPHA(IL)=ALPHA(IL)-VVEC
      IF (IZ(IJA).EQ.0) ALPHA(IJA)=ALPHA(IJA)-VVEC
      IF (IZ(IKA).EQ.0) ALPHA(IKA)=ALPHA(IKA)-VVEC
      IPH(I)=-IPH(I)
      IF (KNT.GT.LKSTA) EEE(I)=-EEE(I)
      IF (ILIST.LT.0.OR.N.GE.100) GO TO 1480
      N=N+1
      B = ABS(EEE(I))
      IB3 = -IPH(I)-131072*(IJ+2048)-32*(IK+2048)
      IF (IL.NE.MIN) GO TO 1450
      IB1 = IJ
      IB2 = IK
      GO TO 1470
 1450 IX=ISIGN(1,IJ)
      IF (MIN.EQ.IKA) IX=ISIGN(1,IK)
      IB1 = IL*IX
      IB2 = -IK*IX
      IF (MIN.EQ.IKA) IB2 = -IJ*IX
      IF (IX.EQ.1) IB3 = MOD(24-IB3,24)
 1470 IPH(I)=-131072*(2048+IB1)-32*(2048+IB2)-IB3
      IEE(I)=524288*(2048-MIN)+INT(100.0*B+0.5)
      IF (KNT.GT.ILIST) GO TO 1475
      BUFR(N) = B
      IBUFR(1,N) = IB1
      IBUFR(2,N) = IB2
      IBUFR(3,N) = IB3
 1475 IF (KNT.LE.NDET) GO TO 1480
C     MARK REJECTED RELATIONSHIPS
      IEE(I)=0
      NREJ=NREJ+1
 1480 CONTINUE
 1500 CONTINUE
C     OUTPUT WARNING IF WEAK LINK OCCURS ABOVE LISTING OF MAP
      IF (ILIST.GT.0.AND.KNT.GT.ILIST.AND.N.EQ.0)
     1                             WRITE (6,1510) KNT,MIN
 1510 FORMAT (/' **** WARNING - WEAK LINK IN POSITION',I5,
     1 ' WITH REFLEXION',I5,' ****')
      KNT = KNT - 1
      IF (KNT.GE.ILIST) GO TO 1240
      IF (N.EQ.0) WRITE (6,1520)
 1520 FORMAT(/1X,'**** WARNING  -  WEAK LINK  -  PLEASE EXAMINE'
     1,' CONVERGENCE MAP CAREFULLY ****')
      M=0
 1530 IF (M.GE.N) GO TO 1240
      MP=M+1
      M=MIN0(N,M+5)
      WRITE (6,1550) ((IBUFR(J,I),J=1,3),BUFR(I),I=MP,M)
 1550 FORMAT(23X,2I4,I3,F6.2,5X,2I4,I3,F6.2)
      GO TO 1530
C     THIS PASS OF CONVERGENCE IS NOW FINISHED
 1560 KNT=KNT+1
      MARK = MARK + 1
C     AFTER THE FIRST PASS OF CONVERGENCE, DEFINE THE ORIGIN
C     AND PICK A GENERAL REFLEXION (IF DEFAULTS)
      IF (MARK.NE.0) GO TO 1625
C     FIND SUITABLE ORIGIN FIXING REFLEXIONS
C     ASCEND CONVERGENCE MAP UNTIL THE ORIGIN CAN BE DEFINED
      IF (NORIN.EQ.NORI) GO TO 1610
 1570 DO 1580 I=KNT,LKGIN
      MIN=IORDE(I)
      IZ(MIN)=0
      CALL ORIGIN_YAO(JUMP,NORIN,0,LARN,MIN)
      IF (JUMP.NE.0) GO TO 1580
      IF (ALPHA(MIN).GT.0.3.OR.I.EQ.NUMB) GO TO 1610
      LAST=IORDE(I+1)
C     PREFER REFLEXION WITH HIGHEST ESTIMATED ALPHA
      IF (PALF(LAST).LE.PALF(MIN)) GO TO 1610
C     ALWAYS USE A SPECIAL REFLEXION IF POSSIBLE
      IF (MKANG(MIN).GT.1.AND.MKANG(LAST).EQ.1) GO TO 1610
      IZ(LAST)=0
      IZ(MIN)=-100
      CALL ORIGIN_YAO(JUMP,NORIN,0,LARN,LAST)
      IF (JUMP.EQ.0) GO TO 1610
      IZ(LAST)=-100
      IZ(MIN)=0
      GO TO 1610
 1580 CONTINUE
      IF (NRAL.EQ.NORI.OR.ICENT.EQ.1) GO TO 1590
C     ALLOW EXTRA GENERAL REFLEXIONS IN ORIGIN DEFINITION
      NRAL=NRAL+1
      GO TO 1570
C     ORIGIN CANNOT BE DEFINED FROM BOTTOM LKGIN REFLEXIONS
C     ALLOW ALL REFLEXIONS TO BE CONSIDERED
 1590 LKGIN=NUMB
      IF (LIST.GE.1) WRITE (6,1600)
 1600 FORMAT(  /' * ** *** ** *',//' CONVERGENCE HAS DIFFICULTY IN'
     1,' FINDING GOOD ORIGIN DEFINING REFLEXIONS  -  PLEASE LOOK CRI'
     2,'TICALLY AT THE CONVERGENCE RESULTS'/' * ** *** ** *'//)
      GO TO 1570
C     ALLOCATE PHASES TO ORIGIN DEFINING REFLEXIONS
 1610 CALL ORIGIN_YAO(JUMP,NORIN,1,LARN,0)
      DO 1615 I=1,NUMB
      IF (IZ(I).EQ.0) IZ(I)=-100
 1615 CONTINUE
C     ENSURE A GENERAL REFLEXION IS IN THE STARTING SET IF POSSIBLE
      IF (ILIST.GE.0) GO TO 1820
      IF (ICENT.NE.0.OR.NANY.GE.0) GO TO 1725
      ALFMIN = 1.0E10
      IREC = 0
      DO 1620 I=KNT,LKSTA
      MIN = IORDE(I)
      IF (IZ(MIN).NE.(-100).OR.ALPHA(MIN).GT.ALFMIN) GO TO 1620
      IF (MKANG(MIN).NE.1) GO TO 1620
      IREC = MIN
 1620 CONTINUE
      IF (IREC.EQ.0) GO TO 1725
      IZ(IREC) = -1
      NGR = 1
      GO TO 1725
C     PUT REFLEXIONS THAT WERE UNDEFINED (ZERO ESTIMATED ALPHA)
C     ON ELIMINATION INTO STARTING SET IF POSSIBLE. IF NOT,
C     USE REFLEXION WITH LOWEST ESTIMATED ALPHA.
 1625 IF (ILIST.GE.0) GO TO 1820
 1627 ALFMIN=1.0E10
      IREC=0
      DO 1680 I=KNT,LKSTA
      MIN=IORDE(I)
      IF (IZ(MIN).GE.(-10)) GO TO 1680
      IAL=1
C     LOOK FOR LOWEST ESTIMATED ALPHA
      IF (ALPHA(MIN).GT.ALFMIN) GO TO 1680
      IF (ALPHA(MIN).GT.0.3) GO TO 1630
      IAL=0
      LAST=IORDE(I+1)
      IF (IZ(LAST).GE.(-10)) GO TO 1680
      IF (PALF(LAST).LE.PALF(MIN).OR.ALPHA(LAST).LT.0.3) GO TO 1630
      ALPHA(MIN)=ALPHA(LAST)
      MIN=LAST
 1630 IF (MKANG(MIN).LE.1) GO TO 1640
      IF (NSPEC.LE.0) GO TO 1650
      IF (IAL.EQ.0) NSPEC=NSPEC-1
      GO TO 1660
 1640 IF (NGEN.LE.0) GO TO 1650
      IF (IAL.EQ.0) NGEN=NGEN-1
      GO TO 1660
 1650 IF (NANY.LE.0) GO TO 1680
      IF (IAL.EQ.0) NANY=NANY-1
 1660 IF (IAL.EQ.0) GO TO 1670
      ALFMIN=ALPHA(MIN)
      IREC=MIN
      GO TO 1680
 1670 IF (MKANG(MIN).EQ.1) NGR=NGR+1
      IF (MKANG(MIN).NE.1) NSR=NSR+1
      IZ(MIN)=-1
 1680 CONTINUE
      MARK=MARK+1
      IF (MARK.EQ.2) GO TO 1627
      IF (IREC.LE.0) GO TO 1770
C     WORST REFLEXION AT END OF CONVERGENCE GOES INTO STARTING SET
      IZ(IREC) = -1
      IF (MKANG(IREC).LE.1) GO TO 1690
      NSR = NSR+1
      IF (NSPEC.LE.0) GO TO 1700
      NSPEC = NSPEC - 1
      GO TO 1720
 1690 NGR = NGR+1
      IF (NGEN.LE.0) GO TO 1700
      NGEN = NGEN - 1
      GO TO 1720
 1700 NANY = NANY - 1
 1720 IF (NANY.LT.0) GO TO 1770
C     REPEAT CONVERGENCE FROM LKSTA
 1725 DO 1730 I=KNT,LKSTA
      J=IORDE(I)
      ALPHA(J)=ALFST(J)
      IF (IZ(J).LT.(-10)) IZ(J)=0
 1730 CONTINUE
      KNT=LKSTA
C     ARE ANY MORE REFLEXIONS TO BE CHOSEN
      IF (NSPEC+NGEN+NANY.GT.0) GO TO 1220
C     IS LISTING OF CONVERGENCE MAP REQUIRED
 1770 IF (LIST) 1780,1780,1790
C     NO LIST
 1780 ILIST = 0
      GO TO 1200
C     PARTIAL LIST
 1790 ILIST=MIN0(60,NUMB)
      IF (ILIST .LT. NUMB) WRITE (6,1795) ILIST
 1795 FORMAT(/' PARTIAL CONVERGENCE LISTING - FINAL STAGES ONLY -',
     1 I5,' REFLEXIONS'/)
      GO TO 1200
 1820 KNT = 0
C     FIND RELATIONSHIPS AMONGST THE STARTING SET REFLEXIONS
      DO 1860 I=1,NUMB
      IF (IZ(I).LT.(-10)) GO TO 1860
      KNT = KNT + 1
      IORDE(KNT) = I
      LI=LIM(I)+1
      LS=LIM(I+1)
      IF (LI.GT.LS) GO TO 1860
      DO 1840 J=LI,LS
      IF (IPH(J).LE.0) GO TO 1840
      IL=IPH(J)/131072-2048
      IJ=(IPH(J)-131072*(IL+2048))/32-2048
      IK=IPH(J)-131072*(IL+2048)-32*(IJ+2048)
      IPH(J)=-131072*(2048+IL)-32*(2048+IJ)-IK
      IEE(J)=524288*(2048-I)+INT(100.0*EEE(J)+0.5)
 1840 CONTINUE
 1860 CONTINUE
C     SORT RELATIONSHIPS IN CONVERGENCE MAP ORDER
      CALL SORT1(IEE,IPH,NSRTOT)
      DO 1880 I=1,KUSER5
      LIM(I+1)=0
 1880 CONTINUE
      DO 1900 I=1,NSRTOT
      IPH(I)=-IPH(I)
      J=IEE(I)/524288
      IF (J.EQ.0) GO TO 1910
      EEE(I)=0.01*REAL(IEE(I)-524288*J)
      J=2048-J
      LIM(J+1)=I
 1900 CONTINUE
 1910 DO 1950 I=1,KUSER5
      LIM(I+1)=MAX0(LIM(I+1),LIM(I))
 1950 CONTINUE
C     FIX ENANTIOMORPH
      CALL START
      NREJ=NUMB-NDET
      IF (NREJ.EQ.0) GO TO 2500
C     OUTPUT REJECTED REFLEXIONS
      WRITE (6,2000) NREJ
 2000 FORMAT(/' CONVERGENCE HAS REJECTED',
     1 I4,' REFLEXIONS FROM THE PHASE DETERMINATION PROCESS')
      IF (LIST.LE.0) GO TO 2250
      WRITE (6,2010)
 2010 FORMAT(/3('  CODE   H  K  L MK '))
      K=0
      NDETP1=NDET+1
      DO 2200 II=NDETP1,NUMB
      LL=IORDE(II)
      K=K+1
      LINE(1,K)=LL
      CALL UNPACK(IH(LL),LINE(2,K),LINE(3,K),LINE(4,K))
      LINE(5,K)=MKANG(LL)
      ICH(K)=NOT
      IF (MKG(LL).EQ.131328) ICH(K)=ISK
      IF (K.LT.6) GO TO 2200
      WRITE (6,2100)((LINE(I,L),I=1,5),ICH(L),L=1,6)
 2100 FORMAT(3(I6,I4,3I3,A1))
      K=0
 2200 CONTINUE
      IF (K.NE.0) WRITE (6,2100)((LINE(I,L),I=1,5),ICH(L),L=1,K)
 2250 NSRTOT=LIM(NUMB+1)
      WRITE (6,2300) NSRTOT
 2300 FORMAT(/' THE NUMBER OF RELATIONSHIPS ACCEPTED IS',I6)
 2500 NSREQ = ITT
      RETURN
      END
C-----------------------------------------------------------------------
C     SUM ALPHA'S AND CHECK ORIGIN DEFINITION
      SUBROUTINE CALPHA(LARN,NORIN,LIST,LKGIN,NSPEC)
      character*1 isk,not,ich(8)
      COMMON /LOCAL/ Ijunk(8), LNE(2,8), ENL(8),BLK1(68)
      COMMON /SYMTRY/ IS(2,3,24), TS(3,24), NSYM, LATT, ICENT, NORI
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ EEE(32000),ALPHA(2000),WT(2000)
      COMMON /B2/ IPH(32000),IPHAZ(2000),IORDE(2000)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),IH(2000),E(2000),
     1 LIM(2101),MKANG(2000)
      EXTERNAL ERROR
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
      DATA ISK/'*'/, NOT/' '/
C     I1/I0 (BESSEL FUNCTION)
      VEC(U)=U*(U+0.4807)/((U+0.8636)*U+1.3943)
      ALFRAN = 0.0
      NUMS = 0
      K = 0
      IF (LIST.GE.1) WRITE (6,1000)
 1000 FORMAT(/' TABLE OF ESTIMATED ALPHAS'//
     1 4(' CODE ALPHA MK '))
      DO 1042 I=1,NUMB
      LI = LIM(I) + 1
      LS = LIM(I+1)
      IF (LI.GT.LS) GO TO 1042
      IF (MKANG(I).GT.1) NUMS=NUMS+1
      DO 1040 J=LI,LS
C     COMPUTE QUANTITIES FOR ESTIMATED AND RANDOM ALPHA
      IL=IPH(J)/131072-2048
      IR=(IPH(J)-131072*(IL+2048))/32-2048
      IL=IABS(IL)
      IR=IABS(IR)
      EX=EEE(J)*VEC(EEE(J))
      PALF(I)=PALF(I)+EX
      PALF(IL)=PALF(IL)+EX
      PALF(IR)=PALF(IR)+EX
      EX=EEE(J)*EEE(J)
      ALPHA(I)=ALPHA(I)+EX
      ALPHA(IL)=ALPHA(IL)+EX
      ALPHA(IR)=ALPHA(IR)+EX
 1040 CONTINUE
 1042 CONTINUE
      DO 1100 I=1,NUMB
C     COMPUTE TOTAL RANDOM ALPHA
      ALFRAN = ALFRAN + SQRT(ALPHA(I))
C     IS THIS PHASE ALREADY DETERMINED BY SIGMA1
      IF (IZ(I).NE.0) LKGIN=LKGIN+1
      IF (LIST.LE.0) GO TO 1100
      K = K + 1
      LNE(1,K)=I
      LNE(2,K)=MKANG(I)
      ENL(K)=PALF(I)
      ICH(K)=NOT
      IF (MKG(I).EQ.131328) ICH(K)=ISK
      IF (K.LT.8) GO TO 1100
      WRITE (6,1055) (LNE(1,L),ENL(L),LNE(2,L),ICH(L),L=1,8)
 1055 FORMAT(4(I5,F6.1,I3,A1))
      K = 0
 1100 CONTINUE
      NSPEC = MIN0(NSPEC, NUMS)
      IF (LIST.GE.1.AND.K.GT.0) WRITE (6,1055) (LNE(1,L),ENL(L),
     1 LNE(2,L),ICH(L),L=1,K)
C     CHECK THAT ORIGIN MAY BE DEFINED AT OUTSET
      LARN = -NRAL
 1200 CALL ORIGIN_YAO(JUMP,NORIN,0,LARN,0)
      IF (JUMP.EQ.0) GO TO 1230
      IF (NRAL.GE.NORI) GO TO 1220
      NRAL = NRAL + 1
      GO TO 1200
 1220 IF (NORIN .LE. 0) WRITE (6,1225)
      IF (NORIN .GT. 0) WRITE (6,1227)
      CALL ERROR
 1225 FORMAT (/' * ** *** ** *',/'  ORIGIN CAN NOT BE PROPER'
     1,'LY DEFINED'/' * ** *** ** *')
 1227 FORMAT (/' * ** *** ** *',/'  ORIGIN FIXING REFLEXIONS'
     1,' CHOSEN BY THE USER WILL NOT FIX THE ORIGIN'/,
     2' * ** *** ** *')
 1230 IF (LIST .GE. 1) WRITE (6,1235) 
 1235 FORMAT(/' ---------------------------------------',
     + '-----------------------------------------------',
     +' CONVERGENCE MAPPING'/)
      RETURN
      END
C-----------------------------------------------------------------------
C     FIND ANY NORI REFLEXIONS FROM THOSE AVAILABLE TO DEFINE THE ORIGIN
C     ROUTINE WILL HANDLE 230 SPACE GROUPS IN THE STANDARD  ORIENTATIONS
C     INCLUDING ALTERNATIVE SETTINGS AS IN INTERNATIONAL TABLES VOLUME 1
      SUBROUTINE ORIGIN_YAO(JUMP,NORIN,IAL,LARN,MIN)
      COMMON /SYMTRY/ IS(2,3,24), TS(3,24), NSYM, LATT, ICENT, NORI
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),IH(2000),E(2000),
     1 LIM(2101),MKANG(2000)
      EXTERNAL UNPACK
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
      JUMP=0
C     ORIGIN IS DEFINED BY SPACE GROUP
      IF (NORI.EQ.0) RETURN
C     REFLEXION MIN MUST BE INCLUDED IN ORIGIN
      MUST=0
      IF (MIN.EQ.0) MUST=-1
      DO 900 I=1,NUMB
C     TEST IF REFLEXION MIN IS IN ORIGIN DEFINING SET
      IF (MUST.EQ.1) GO TO 950
      IF (I.EQ.MIN) MUST=1
C     ELIMINATED REFLEXION
      IF (IZ(I).LT.0) GO TO 900
C     REFLEXION WITH NO SIGMA-2 CONTRIBUTORS
      IF (PALF(I).LT.0.2) GO TO 900
C     SEMINVARIANT REFLEXION
      IF (MKG(I).EQ.131328) GO TO 900
      NRAL1=0
C     COUNT GENERAL REFLEXIONS
      IF (MKANG(I).EQ.1) NRAL1=1
      IF (NRAL1.GT.NRAL) GO TO 900
      N1=0
C     COUNT USER REFLEXIONS REQUESTED
      IF (MOD(IZ(I),1000).EQ.200) N1=1
      IF (NORIN-N1.GE.3) GO TO 900
      CALL UNPACK(MKG(I),J1,K1,L1)
      IF (NORI.GT.1) GO TO 100
C     IS MIN INCLUDED
      IF (MUST.EQ.0) GO TO 900
C     TEST WITH ORIGIN RULES IF ONE REFLEXION IS NEEDED TO DEFINE ORIGIN
C     MODULUS = (2), (3), (4), (6) OR (0)
      IF (L1.EQ.1) GO TO 1000
      IF (L1.EQ.MODUL(3)-1) GO TO 1000
      GO TO 900
  100 IPO=I+1
      IF (IPO.GT.NUMB) GO TO 900
      DO 800 J=IPO,NUMB
      IF (MUST.EQ.2) GO TO 900
      IF (J.EQ.MIN) MUST=2
      IF (IZ(J).LT.0) GO TO 800
      IF (PALF(J).LT.0.2) GO TO 800
      IF (MKG(J).EQ.131328) GO TO 800
      NRAL2=NRAL1
      IF (MKANG(J).EQ.1) NRAL2=NRAL2+1
      IF (NRAL2.GT.NRAL) GO TO 800
      N2=N1
      IF (MOD(IZ(J),1000).EQ.200) N2=N2+1
      IF (NORIN-N2.GE.2) GO TO 800
      CALL UNPACK(MKG(J),J2,K2,L2)
      IF (NORI.GT.2) GO TO 200
      IF (MUST.EQ.0) GO TO 800
C     TEST WITH ORIGIN RULES IF TWO REFLEXIONS ARE NEEDED
      MDET=K1*L2-K2*L1
      MAXM=MAX0(MODUL(2),MODUL(3))
C     MODULUS = (0,0) OR (2,2)
      IF (IABS(MDET).EQ.1) GO TO 1000
      IF (MAXM.EQ.0) GO TO 800
C     MODULUS = (0,2), (2,0) OR (3,0)
      IF (MOD(MDET,MAXM).EQ.0) GO TO 800
C     MODULUS = (0,2)
      IF (MODUL(2).EQ.0.AND.JFAC(K1,K2,-1000).NE.1) GO TO 800
C     MODULUS = (2,0) OR (3,0)
      IF (MODUL(3).EQ.0.AND.JFAC(L1,L2,-1000).NE.1) GO TO 800
      GO TO 1000
  200 JPO=J+1
      IF (JPO.GT.NUMB) GO TO 800
      DO 700 K=JPO,NUMB
      IF (MUST.EQ.3) GO TO 800
      IF (K.EQ.MIN) MUST=3
      IF (IZ(K).LT.0) GO TO 700
      IF (PALF(K).LT.0.2) GO TO 700
      IF (MKG(K).EQ.131328) GO TO 700
      NRAL3=NRAL2
      IF (MKANG(K).EQ.1) NRAL3=NRAL3+1
      IF (NRAL3.GT.NRAL) GO TO 700
      N3=N2
      IF (MOD(IZ(K),1000).EQ.200) N3=N3+1
      IF (NORIN-N3.GE.1) GO TO 700
      CALL UNPACK(MKG(K),J3,K3,L3)
      IF (MUST.EQ.0) GO TO 700
C     TEST WITH ORIGIN RULES IF THREE REFLEXIONS ARE NEEDED
      MDET=J1*(K2*L3-K3*L2)-J2*(K1*L3-K3*L1)+J3*(K1*L2-K2*L1)
      MAXM=MODUL(1)+MODUL(2)+MODUL(3)
C     MODULUS = (0,0,0) OR (2,2,2)
      IF (IABS(MDET).EQ.1) GO TO 1000
      IF (MAXM.EQ.0) GO TO 700
      IF (MAXM.EQ.2) GO TO 600
C     MODULUS = (0,2,2), (2,0,2) OR (2,2,0)
      IF (MOD(MDET,2).EQ.0) GO TO 700
C     MODULUS = (0,2,2)
      IF (MODUL(1).EQ.0.AND.JFAC(J1,J2,J3).NE.1) GO TO 700
C     MODULUS = (2,0,2)
      IF (MODUL(2).EQ.0.AND.JFAC(K1,K2,K3).NE.1) GO TO 700
C     MODULUS = (2,2,0)
      IF (MODUL(3).EQ.0.AND.JFAC(L1,L2,L3).NE.1) GO TO 700
      GO TO 1000
  600 IF (MODUL(2).NE.2) GO TO 650
C     MODULUS = (0,2,0)
      IF (K1.EQ.1.AND.IABS(J2*L3-J3*L2).EQ.1) GO TO 1000
      IF (K2.EQ.1.AND.IABS(J3*L1-J1*L3).EQ.1) GO TO 1000
      IF (K3.EQ.1.AND.IABS(J1*L2-J2*L1).EQ.1) GO TO 1000
      GO TO 700
C     MODULUS = (0,0,2)
  650 IF (L1.EQ.1.AND.IABS(J2*K3-J3*K2).EQ.1) GO TO 1000
      IF (L2.EQ.1.AND.IABS(J3*K1-J1*K3).EQ.1) GO TO 1000
      IF (L3.EQ.1.AND.IABS(J1*K2-J2*K1).EQ.1) GO TO 1000
  700 CONTINUE
  800 CONTINUE
  900 CONTINUE
  950 JUMP=-1
      RETURN
 1000 IF (IAL.EQ.0) RETURN
      L=0
C     COUNT GENERAL REFLEXIONS AND ASSIGN PHASES
 1050 IF (MKANG(I).GT.1) GO TO 1100
      LARN=LARN+1
C     IS THIS A POSSIBLE ENANTIOMORPH DEFINING REFLEXION
      IF (LARN.LE.0) GO TO 1100
      IF (NANT(LARN).GT.0) LARN=LARN+1
      NANT(LARN)=I
      IZ(I)=45200
      GO TO 1200
 1100 M=MKANG(I)-1
      IF (MOD(M,12).EQ.0) M=24
      IZ(I)=15000*M+200
 1200 L=L+1
      IF (L.EQ.NORI) RETURN
      IF (L.EQ.1) I=J
      IF (L.EQ.2) I=K
      GO TO 1050
      END
C-----------------------------------------------------------------------
C     TEST FOR COMMON FACTOR (.GT.1) AMONGST K,L,N
      FUNCTION JFAC(K,L,N)
      JFAC=1
      M=MIN0(K,L,IABS(N))
      IF (M.EQ.1) RETURN
      DO 10 I=2,M
      IF (MOD(K,I).NE.0) GO TO 10
      IF (MOD(L,I).NE.0) GO TO 10
      IF (N.GT.0.AND.MOD(N,I).NE.0) GO TO 10
      JFAC=I
      GO TO 20
   10 CONTINUE
   20 RETURN
      END
C-----------------------------------------------------------------------
C     DETERMINE   THE   ENANTIOMORPH   FIXING   REFLEXION
      SUBROUTINE START
      COMMON /SYMTRY/ IS(2,3,24), TS(3,24), NSYM, LATT, ICENT, NORI
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B2/ IPH(32000),IPHAZ(2000),IORDE(2000)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),IH(2000),E(2000),
     1 LIM(2101),MKANG(2000)
      EXTERNAL UNPACK
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
C     HAS ENANTIOMORPH BEEN FIXED     NANT(1) .EQ. 0 IF NOT
 1580 IF (NANT(1)) 2160,1590,1584
C     DOES THE ENANTIOMORPH  FIXING  REFLEXION  ALSO  DEFINE  THE ORIGIN
 1584 MAX = NANT(1)
      ALFMAX = ABS(PALF(MAX))
      IF (IZ(MAX).GE.0) GO TO 2160
C     DOES THE  ENANTIOMORPH  NEED TO BE FIXED   -  ICENT .NE. 0  IF NOT
      IF (ICENT) 2105,1595,2105
 1590 IF (ICENT.NE.0) GO TO 2160
C     ENANTIOMORPH TO BE FIXED  -  INITIALISE PALF
 1595 DO 1597 I=1,NUMB
      PALF(I) = ABS(PALF(I))
 1597 CONTINUE
      JUMP = 0
      NAN = 1
C     TEST FOR PRIMITIVE SPACE GROUP WITH SEMINVARIANT  MODULUS  (2,2,2)
      IF (LATT.NE.1) GO TO 1605
      J = 4 - NORI
      DO 1603 I=J,3
      IF (MODUL(I).NE.2) GO TO 1605
 1603 CONTINUE
      JUMP = 1
 1605 IF (NANT(1).NE.0) GO TO 1625
C     SELECT ENANTIOMORPH REFLEXION FROM BOTTOM OF THE CONVERGENCE MAP
C     IS THERE A PERMUTABLE REFLEXION IN THE STARTING SET?
      DO 1607 I=1,NUMB
      IF (IZ(I).EQ.(-1)) GO TO 1608
 1607 CONTINUE
      GO TO 2120
C     SELECT LOWEST REFLEXION USED IN CONVERGENCE MAP
 1608 MAX = 0
      ALFMAX = 0.0
      DO 1615 I=1,NUMB
      MIN = IORDE(I)
      IF (IZ(MIN).NE.(-100)) GO TO 1615
      LI = LIM(MIN)+1
      LF = LIM(MIN+1)
      IF (LI.GT.LF) GO TO 1615
      DO 1610 JJ=LI,LF
      IL = IPH(JJ)/131072-2048
      ILA = IABS(IL)
      IF (IZ(ILA).NE.(-1)) GO TO 1609
      IF (PALF(ILA).LE.ALFMAX) GO TO 1609
      ALFMAX = PALF(ILA)
      MAX = ILA
 1609 IR = (IPH(JJ)-131072*(IL+2048))/32-2048
      IRA = IABS(IR)
      IF (IZ(IRA).NE.(-1)) GO TO 1610
      IF (PALF(IRA).LE.ALFMAX) GO TO 1610
      ALFMAX = PALF(IRA)
      MAX = IRA
 1610 CONTINUE
      IF (MAX.NE.0) GO TO 1625
 1615 CONTINUE
      GO TO 2120
 1625 IF (JUMP.LE.0) GO TO 1860
C     TO DETERMINE  WHETHER  ENANTIOMORPH  FIXING  REFLEXION  MUST  HAVE
C     POSITIVE REAL (NAN = -1) OR IMAGINARY (NAN = 1) PART,  FIND LINEAR
C     DEPENDENCE OF ENANTIOMORPH FIXING REFLEXION ON  ORIGIN  REFLEXIONS
 1635 LOG1 = 0
      CALL UNPACK(MKG(MAX),M1,M2,M3)
      CALL UNPACK(MKG(MAX),L1,L2,L3)
      NORF=0
      DO 1800 J=1,NUMB
      IF (MOD(IZ(J),1000).NE.200) GO TO 1800
      NORF=NORF+1
      CALL UNPACK(MKG(J),J1,J2,J3)
      LOG2 = MOD(M1 + J1, 2) + MOD(M2 + J2, 2) + MOD(M3 + J3, 2)
      IF (LOG2.GT.0) GO TO 1660
C     REFLEXION IS LINEARLY DEPENDENT ON ONE ORIGIN REFLEXION
      IF (IABS(MKANG(J) - 7) - 3) 1840,1820,1820
 1660 JP=J+1
      IF (NORF.EQ.NORI) JP=1
      DO 1670 K=JP,NUMB
      IF (MOD(IZ(K),1000).EQ.200) GO TO 1675
 1670 CONTINUE
 1675 CALL UNPACK(MKG(K),K1,K2,K3)
      LOG2 = MOD(M1+J1+K1,2) + MOD(M2+J2+K2,2) + MOD(M3+J3+K3,2)
      IF (LOG2.GT.0) GO TO 1700
C     REFLEXION IS LINEARLY DEPENDENT ON TWO ORIGIN REFLEXIONS
      IF (IABS(MOD(MKANG(J) + MKANG(K) - 2, 12) - 6) - 3) 1840,1820,1820
C     DETERMINE LINEAR DEPENDENCE ON ALL THREE ORIGIN REFLEXIONS
 1700 LOG1 = LOG1 + MKANG(J) - 1
      L1 = L1 + J1
      L2 = L2 + J2
      L3 = L3 + J3
 1800 CONTINUE
      IF (MOD(L1,2) + MOD(L2,2) + MOD(L3,2)) 2100,1810,1820
C     REFLEXION IS LINEARLY DEPENDENT ON THREE ORIGIN REFLEXIONS
 1810 IF (IABS(MOD(LOG1, 12) - 6) - 3) 1840,1820,1820
C     IMAGINARY PART MUST BE POSITIVE TO FIX ENANTIOMORPH
 1820 NAN = 1
      GO TO 1860
C     REAL PART MUST BE POSITIVE TO FIX ENANTIOMORPH
 1840 NAN = -1
 1860 IF (MKANG(MAX).LE.1) GO TO 2060
      IF (IABS(MKANG(MAX) - 7) - 3) 2020,2040,2040
 2020 IF (NAN) 2100,2100,2080
 2040 IF (NAN) 2080,2100,2100
 2060 IF (NAN) 2080,2100,2080
 2080 NANT(1) = NAN * MAX
      GO TO 2160
 2100 PALF(MAX) = -PALF(MAX)
      IF (NANT(1).EQ.0) GO TO 1605
C     OUTPUT ERROR MESSAGE
 2105 WRITE (6,2110)
 2110 FORMAT(/' * ** *** ** *'/' ENANTIOMORPH FIXING REFLEXIO'
     1,'N CHOSEN BY THE USER DOES NOT FIX THE ENANTIOMORPH'
     2/' * ** *** ** *'//)
C     REMOVE REFLEXION AND TRY AGAIN
      DO 2115 I=1,3
      NANT(I) = NANT(I+1)
 2115 CONTINUE
      GO TO 1580
 2120 WRITE (6,2140)
 2140 FORMAT(/' * ** *** ** *',/' THE REFLEXIONS IN THE STAR'
     1,'TING SET WILL NOT DEFINE THE ENANTIOMORPH'/
     2,' * ** *** ** *'//)
 2160 DO 2180 I=1,NUMB
      PALF(I) = ABS(PALF(I))
 2180 CONTINUE
      RETURN
      END
C-----------------------------------------------------------------------
C     OUTPUT CONVERGENCE RESULTS
      SUBROUTINE OUTPT1(NSRTOT,LIST)
      DIMENSION LINE3(250,8)
      COMMON /LOCAL/ LINE1(3,6),BLK1(82)
      COMMON /SYMTRY/ IS(2,3,24), TS(3,24), NSYM, LATT, ICENT, NORI
      COMMON /KFRAG/ SIGMAQ
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ EEE(32000),BLK2(4000)
      COMMON /B2/ IPH(32000),LINE2(400,5),IORDE(2000)
      COMMON /B3/ NKEY(2000),MKSTOR(2000),KNOWN(2000),BLK3(2000)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),IH(2000),E(2000),
     1 LIM(2101),MKANG(2000)
      EXTERNAL UNPACK
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
      NS=0
      WRITE (6,480) NORI
  480 FORMAT(/'------------------------------------------------',
     1 '-----------------------------------------------------',
     1//' CONVERGENCE  RESULTS',
     1 //' THE NUMBER OF ORIGIN DEFINING REFLEXIONS IS',I6)
      IF (ICENT .LT. 0 .AND. SIGMAQ .LT. 0.25) WRITE (6,620)
  620 FORMAT(/' THE ENANTIOMORPH IS FIXED BY THE SPACE GROUP')
      IF (SIGMAQ .GE. 0.25) WRITE (6,630)
  630 FORMAT(/' THE ENANTIOMORPH IS FIXED BY THE INVARIANTS')
  700 J = 0
      K = 0
      L = 0
      MARK = 0
C     ACCUMULATE NUMBER OF UNKNOWN GENERAL REFLEXIONS IN STARTING SET
      NRAL = 0
      DO 1200 I=1,NUMB
      IF (IZ(I)+1) 1000,1080,1020
C     ELIMINATED REFLEXION
 1000 IZ(I) = 0
      GO TO 1200
 1020 IWT=IZ(I)-1000*(IZ(I)/1000)
      IF (IWT-100) 1120,1120,1040
C     ORIGIN FIXING REFLEXION
 1040 J = J + 1
      LINE1(J,1) = I
      CALL UNPACK(IH(I),LINE1(J,2),LINE1(J,3),LINE1(J,4))
      LINE1(J,5)=IZ(I)/1000
      LINE1(J,6) = MKANG(I)
C     DOES REFLEXION ALSO FIX ENANTIOMORPH
      IF (IABS(NANT(1)).EQ.I) NS = NS +1
      GO TO 1200
C     STARTING SET REFLEXIONS
 1080 K = K + 1
      LINE2(K,1) = I
      CALL UNPACK(IH(I),LINE2(K,2),LINE2(K,3),LINE2(K,4))
      LINE2(K,5) = MKANG(I)
      IF (MKANG(I) .NE. 1) IZ(I) = -15000*MOD(MKANG(I)-1, 12) - 360000
      IF (MKANG(I) .NE. 1) NS=NS+1
      IF (MKANG(I) .EQ. 1) NRAL = NRAL + 1
C     NOTE CODE NUMBER OF FIRST SPECIAL REFLEXION IN LIST
      IF (MARK.NE.0) GO TO 1200
      IF (MKANG(I) .NE. 1) MARK = I
      GO TO 1200
C     KNOWN REFLEXION
 1120 L = L + 1
      LINE3(L,1) = I
      CALL UNPACK(IH(I),LINE3(L,2),LINE3(L,3),LINE3(L,4))
      LINE3(L,5)=IZ(I)/1000
      LINE3(L,6)=IWT
 1200 CONTINUE
C     IF NRAL=0, SUBTRACT INCREMENT FROM FIRST UNKNOWN SPECIAL PHASE
      IF (NRAL .EQ. 0 .AND. MARK .GT. 0) IZ(MARK) = IZ(MARK) + 180000
      IF (L .NE. 0) WRITE (6,1240) ((LINE3 (I,M),M=1,6),I=1,L)
 1240 FORMAT(/' KNOWN PHASES',//
     1,2('    CODE    H    K    L   PHI 10 10*WT    ')
     2,/2(I8,3I5,2I6,5X))
      IF (J.LE.0) GO TO 1400
      WRITE (6,1300) ((LINE1(I,M),M=1,6),I=1,J)
 1300 FORMAT (/'  ORIGIN FIXING REFLEXIONS',/
     1/3(' CODE   H   K   L  PHI MK ')/2(I5,3I4,I5,I3,1X))
      IF (NANT(2).LE.0) GO TO 1400
      NN = NANT(2)
      CALL UNPACK(IH(NN),I1,I2,I3)
      WRITE (6,1340) I1,I2,I3
 1340 FORMAT(/' THE PHASE OF THE',3I4,'  REFLEXION CAN HAVE THE',
     1' TWO VALUES 45 AND 315 DEGREES')
      NS=NS+1
      IF (NANT(3).LE.0) GO TO 1400
      NN = NANT(3)
      CALL UNPACK(IH(NN),I1,I2,I3)
      WRITE (6,1380) I1,I2,I3
 1380 FORMAT(/' HAND OF THE',3I4,'  REFLEXION')
      NS=NS+1
 1400 IF (K.NE.0.AND.LIST.GE.1)
     1          WRITE (6,1440) ((LINE2(I,M),M=1,5),I=1,K)
 1440 FORMAT (/' OTHER REFLEXIONS IN BOTTOM OF CONVERGENCE MAP',
     1 //3(' CODE   H   K   L MK ')/3(I5,3I4,I3,1X))
      IF (NANT(1).EQ.0) GO TO 1520
      NN = IABS(NANT(1))
      CALL UNPACK(IH(NN),I1,I2,I3)
      IF (ICENT.EQ.0) WRITE (6,1500) I1,I2,I3
 1500 FORMAT(/' THE ENANTIOMORPH IS FIXED BY THE',3I4,'  REFLEXION')
 1520 CONTINUE
C     PRESERVE IORDE IN FORM SUITABLE FOR WEAK RELATIONSHIPS
C     AND MKANG IN MKSTOR
      DO 1820 I=1,NUMB
      MKSTOR(I)=MKANG(I)
      J=IORDE(I)
      NKEY(J)=I
 1820 CONTINUE
C     STORE SIGMA2S ON TAPE WHILE SETTING UP PSIZEROS
      REWIND 3
      WRITE (3) (IPH(I),EEE(I),I=1,NSRTOT),(LIM(I+1),I=1,NUMB)
      RETURN
      END
C-----------------------------------------------------------------------
C     RATIONALISE WEAK REFLEXIONS AND SET UP FILE FOR TANGENT FORMULA
      SUBROUTINE OUTPT2(NSRTOT,NSRPSI)
      COMMON /CONST/ KUSER1, KUSER2,
     1 KUSER3,KUSER4,KUSER5,KUSER6,DTOR,RTOD,STABLE(450)
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ EEE(32000),ALPHA(2000),WT(2000)
      COMMON /B2/ IPH(32000),IPHAZ(2000),IORDE(2000)
      COMMON /B3/ NKEY(2000),MKSTOR(2000),LIMM(2000),ISPZRO(100),
     1 ISP2(200),ISP3(200),BLK3(1500)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),IH(2000),E(2000),
     1 LIM(2101),MKANG(2000)
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
      NSRP=NSRPSI
C     RESTORE MKANG AND IORDE TO THEIR ORIGINAL FORM
      DO 100 I=1,NUMB
      MKANG(I)=MKSTOR(I)
      J=NKEY(I)
      IORDE(J)=I
  100 CONTINUE
      IF (IZRO.EQ.0) GO TO 1350
C     REDUCE WEAK RELATIONSHIPS. NREF AND NREF2 ARE THE
C     NUMBER OF REFLEXIONS WITH .GE.,.LT. 10 CONTRIBUTORS
C     IN THE BOTTOM OF THE CONVERGENCE MAP RESPECTIVELY
      NREF=0
      NREF2=0
      DO 700 I=1,IZRO
      LIMM(I)=0
      NC1=0
      NC2=0
      LI=LIM(I)+1
      LS=LIM(I+1)
      IF (LI.GT.LS) GO TO 700
      DO 300 JJ=LI,LS
      IL=IPH(JJ)/131072-2048
      ILA=IABS(IL)
      IF (NKEY(ILA).GT.100) GO TO 200
      IR=(IPH(JJ)-131072*(IL+2048))/32-2048
      IRA=IABS(IR)
      IF (NKEY(IRA).GT.100) GO TO 200
      NC1=NC1+1
  200 NC2=NC2+1
  300 CONTINUE
      IF (NC1.LT.10.OR.NREF.GE.100) GO TO 400
C     RECORD WEAK REFS FOR EARLY F.O.M.
      NREF=NREF+1
      ISPZRO(NREF)=I+NUMB
      LIMM(I)=NC2
C     ALLOW THESE REFS TO BE REJECTED LAST
      NREF2=NREF2+1
      ISP2(NREF2)=I
      ISP3(NREF2)=-1000-NC1
      GO TO 700
  400 IF (NC2.LT.15.OR.NREF2.GT.200) GO TO 500
C     RECORD REFS + CONTRIBS FOR FINAL F.O.M. & REJECTION
      NREF2=NREF2+1
      ISP2(NREF2)=I
      ISP3(NREF2)=-NC2
      LIMM(I)=NC2
      GO TO 700
C     REMOVE ALL RELATIONSHIPS FOR REJECTED REFLEXION
  500 DO 600 JJ=LI,LS
      EEE(JJ)=0.0
      NSRP=NSRP-1
  600 CONTINUE
  700 CONTINUE
      IF (NSRP.LE.KUSER1-NSRTOT) GO TO 1000
C     REDUCE NO OF WEAK REFS UNTIL RELATIONSHIPS WILL FIT
      CALL SORT1(ISP3,ISP2,NREF2)
      DO 900 I=1,NREF2
      LL=ISP2(I)
      LI=LIM(LL)+1
      LS=LIM(LL+1)
      DO 800 JJ=LI,LS
      EEE(JJ)=0.0
      NSRP=NSRP-1
  800 CONTINUE
      LIMM(LL)=0
      IF (ISP3(I).GT.(-1000)) GO TO 850
C     REDUCE ISPZRO ARRAY IF AN EFOM REFLEXION IS ELIMINATED
      NREF=NREF-1
      KK=LL+NUMB
      DO 840 J=1,NREF
      IF (ISPZRO(J).NE.KK) GO TO 840
      ISPZRO(J)=ISPZRO(J+1)
      ISPZRO(J+1)=KK
  840 CONTINUE
  850 IF (NSRP.LE.KUSER1-NSRTOT) GO TO 1000
  900 CONTINUE
C     MOVE WEAK RELS TO CORRECT PART OF IPH,EEE, & REFS IN LIM
C     CLOSE UP WEAK RELATIONSHIPS
 1000 K = 0
      DO 1050 J=1,NSRPSI
      IF (EEE(J).EQ.0.0) GO TO 1050
      K=K+1
      EEE(K)=EEE(J)
      IPH(K)=IPH(J)
 1050 CONTINUE
C     MOVE RELATIONSHIPS TO CORRECT PLACES
      DO 1100 J=1,NSRP
      K = NSRTOT + J
      IPH(K)=IPH(J)
      EEE(K)=EEE(J)
 1100 CONTINUE
C     MOVE REFS TO TOP OF LIM & COUNT THE USED REFS
      K=NSRTOT
      KNT=0
      DO 1200 I=1,IZRO
      IF (LIMM(I).GT.0) KNT=KNT+1
      K=K+LIMM(I)
      ITEMP = I+NUMB+1
      LIM(ITEMP)=K
 1200 CONTINUE
      WRITE (6,1300) KNT,NSRP
 1300 FORMAT (/' PSI ZERO FIGURE OF MERIT TO BE CALCULATED USING',
     1 I4,' REFLEXIONS AND',I5,' RELATIONSHIPS')
C     MARK IZRO FOR PSIZERO TEST IF NO RELATIONSHIPS TO BE USED
      IF (NSRP.EQ.0) IZRO = 0
C     READ SIGMA2S BACK & SET UP TAPE FOR TANGENT FORMULA
 1350 REWIND 3
      READ (3) (IPH(I),EEE(I),I=1,NSRTOT),(LIM(I+1),I=1,NUMB)
      RETURN
      END
C-----------------------------------------------------------------------
C     No SIGMA1 used but does other stuff.
      SUBROUTINE SIGMA1(PROB)
      COMMON /LOCAL/ LINE(3,6), ENIL(3,2),BLK1(76)
      COMMON /CONST/ KUSER1, KUSER2,
     1 KUSER3,KUSER4,KUSER5,KUSER6,DTOR,RTOD,STABLE(450)
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ EEE(32000),ALPHA(2000),ZR(2000)
      COMMON /B2/ IPH(32000),IPHAZ(2000),ZI(2000)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),IH(2000),E(2000),
     1 LIM(2101),MKANG(2000)
      EXTERNAL UNPACK
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
      NXSIG = 0
      DO 1000 I=1,NUMB
      ZI(I) = 0.0
      ZR(I) = 0.0
      IPHAZ(I) = 0
      IF (I.GT.NREC) IZ(I)=0
 1000 CONTINUE
      IF (NREC .EQ. 0) GO TO 1150
      DO 1120 J=1,NREC
      IZJ=IZ(J)
      IZ(J)=0
      ISAVE = IABS(IZJ)
      IND1 = ISAVE / 131072
C     TEST FOR STRUCTURE INVARIANT WITH RESTRICTED PHASE
      IF (MKG(IND1).NE.131328.OR.MKANG(IND1).EQ.1) GO TO 1120
      NXSIG = 1
C     UNPACK SIGMA1 RELATIONSHIP
      ISAVE = ISAVE - 131072 * IND1
      IND2 = ISAVE / 32 - 2048
      IFAZ = (ISAVE - 32 * (IND2 + 2048)) * 15
      IND2A = IABS(IND2)
      IF (ISIGN(1,IND2) .NE. ISIGN(1,IZJ)) GO TO 1100
      IF (MKANG(IND2A) .EQ. 1) GO TO 1120
      IFAZ = IFAZ + ISIGN(30 * (MKANG(IND2A) - 1), IND2)
C     ACCUMULATE REAL AND IMAGINARY PARTS
 1100 ZI(IND1) = ZI(IND1) - (E(IND2A)**2 - 1.0) * SIN(DTOR *REAL(IFAZ))
      ZR(IND1) = ZR(IND1) + (E(IND2A)**2 - 1.0) * COS(DTOR *REAL(IFAZ))
C     ACCUMULATE NUMBER OF CONTRIBUTORS TO EACH SIGMA1 INDICATION
      IPHAZ(IND1) = IPHAZ(IND1) + 1
 1120 CONTINUE
      IF (NXSIG .EQ. 1) GO TO 1200
 1150 WRITE (6,1160)
 1160 FORMAT(/' * * *  NO SIGMA1 REFLEXIONS  * * *')
      RETURN
 1200 IF (PROB.LT.1.0) WRITE (6,1220) PROB
 1220 FORMAT(/,/,' * * *   SIGMA1  RESULTS   * * *'/' PHASE ACCE',
     1'PTED IF PROBABILITY GREATER THAN',F5.2,' AND NUMBER OF CONTR',
     2'IBUTORS IS GREATER THAN 3',/,/,
     3 2('H   CODE   H   K   L   E  PHI  PROB NC   '))
      K = 0
      DO 1320 I=1,NUMB
      IF (MKANG(I).EQ.1.OR.MKG(I).NE.131328) GO TO 1320
      ARG = 15 * MKANG(I) - 14
      T2 = 0.5 * SIGMA*E(I)* (ZI(I)*SIN(DTOR*ARG) + ZR(I)*COS(DTOR*ARG))
      K = K + 1
      LINE(K,1) = I
      CALL UNPACK(IH(I),LINE(K,2),LINE(K,3),LINE(K,4))
      ENIL(K,1) = E(I)
      LINE(K,5) = 15 * (MKANG(I) - 1)
      IF (T2 .LT. 0.0) LINE(K,5) = LINE(K,5) + 180
      ENIL(K,2) = 0.5 + 0.5 * TANH(ABS(T2))
      LINE(K,6) = IPHAZ(I)
      IF (ENIL(K,2).LT.PROB.OR.IPHAZ(I).LE.3) GO TO 1240
C     SIGMA1 PHASE ACCEPTED
      IZ(I) = 1000 * LINE(K,5) + INT(200.0 * ENIL(K,2) - 100.0)
 1240 IF (K .LT. 3) GO TO 1320
      IF (PROB.LT.1.0) WRITE (6,1300) ((LINE(L,J),J=1,4),ENIL(L,1),
     1  LINE(L,5),ENIL(L,2),LINE(L,6), L=1,3)
 1300 FORMAT (2(I7,3I4,F5.2,I4,F6.3,I3,3X))
      K = 0
 1320 CONTINUE
      IF (PROB.LT.1.0.AND.K.NE.0) WRITE (6,1300) ((LINE(L,J),J=1,4),
     1  ENIL(L,1),LINE(L,5),ENIL(L,2),LINE(L,6), L=1,K)
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C         PPPPPP   H     H     A      SSSSS   EEEEEEE  RRRRRR          C
C         P     P  H     H    A A    S     S  E        R     R         C
C         P     P  H     H   A   A   S        E        R     R         C
C         PPPPPP   HHHHHHH  A     A   SSSSS   EEEEEE   RRRRRR          C
C         P        H     H  AAAAAAA        S  E        R   R           C
C         P        H     H  A     A  S     S  E        R    R          C
C         P        H     H  A     A   SSSSS   EEEEEEE  R     R         C
C                                                                      C
C                           RANTAN  PART 2                             C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE PHASER
C     DIMENSION STATEMENT PARTICULAR TO SUBROUTINE PHASER
      DIMENSION KDSTOR(2000),WTSTOR(2000)
      COMMON /LOCAL/ AMX(3),AMN(3),LINE(20),LINEX(20),BLK1(54)
      COMMON /USER/ MS,LIST,NSREQ,PROB,NSPEC,NGEN,NANY,KMIN,
     1 IMK,ITAN,IPUB,ISKIP,WTFOM(3),KMAX,IWMIN,NRAN,IOFR,NSOUT,IPSTOR(5)
      COMMON /SYMTRY/ IS(2,3,24), TS(3,24), NSYM, LATT, ICENT, NORI
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ EEE(32000),ALPHA(2000),WT(2000)
      COMMON /B2/ IPH(32000),IPHAZ(2000),IORDE(2000)
      COMMON /B3/ EALF(2000),TALF(2000),RALF(2000),ISPZRO(100),FOM(400)
     1,           BLK3(1500)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),IH(2000),E(2000),
     1 LIM(2101),MKANG(2000)
      COMMON /B6/ IXRAN,IYRAN,PSIMIN,RMIN
C     KNOWN PHASES FROM CCP4.
      COMMON /PHIWT/IMKC,KDSC(2000),MKGC(2000),IPKC(2000),WTSC(2000)
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
C     
      PSIMIN=5.0
      RMIN=50.0
      IXRAN = 1
      IYRAN = 1
      MS=0
      NSREQ = NSREQ + ISKIP
      WRITE (6,1438)
 1438 FORMAT(/' +++++++++++++++++++++++++++++++++++++++++++++++',
     + '++++++++++++++++++++++++++++++++++++++++++++')
      WRITE (6,1439) 
 1439 FORMAT( ' RANTAN PART 2  TANGENT FORMULA'
     1,' PHASE DETERMINATION     VERSION  OCT. 1997')
      WMIN=0.01*IWMIN
      NNN=MIN0(NDET,NRAN)
      IF (IMK.GT.50) NNN=NDET
      IF (IMK.GT.50) NRAN=NDET
      IF (ITAN.EQ.0) WRITE (6,1444)
 1444 FORMAT (/' CONVENTIONAL WEIGHTED TANGENT FORMULA')
      IF (ITAN.EQ.1) WRITE (6,1445)
 1445 FORMAT (/' STATISTICALLY WEIGHTED TANGENT FORMULA')
      WRITE (6,1441) NNN
 1441 FORMAT (/' NUMBER OF RANDOM PHASES = ',I6)
      WRITE (6,1442) WMIN
 1442 FORMAT (/' WEIGHT FOR RANDOM PHASES = ',F10.3)
      REWIND 2
      MARK = 0
      DO 1475 I=1,NUMB
      EALF(I)=E(I)
 1475 CONTINUE
      WRITE (2) MARK
      WRITE (2) ICENT, LATT, NSYM,
     1 ((TS(I,J),(IS(K,I,J),K=1,2),I=1,3),J=1,NSYM), MAXH,PARA,NUMB,
     2 (IH(I),EALF(I),I=1,NUMB)
      IF (ISKIP.GT.0) WRITE (6,1500) ISKIP
 1500 FORMAT (/' NUMBER OF SKIPPED PHASE SETS IS', I5)
      MARK = ISKIP
      IF (IMK.GT.0) GO TO 1800
C     * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C     STARTING SETS DETERMINED BY PROGRAM
 1501 CONTINUE
C     STORE IZ IN MKG AND NANT IN NTSTOR IN CASE NO SETS DEVELOPED
 1580 DO 1590 I=1,NUMB
      MKG(I)=IZ(I)
 1590 CONTINUE
      NSETS = 0
      AMN(2) = 100.0
      AMN(3) = 100.0
      ISETX = 0
      J = 0
      NUMSET = 0
C     READ SPECIFIED SET NUMBERS
      IF (IPUB.GT.0) THEN
       WRITE(6,FMT='(/1X,A,5I5)') 
     +    'THE SET NUMBER IS GIVEN BY USER = ',
     +    (IPSTOR(I), I=1,IPUB)
      ENDIF
 1600 NSETS = NSETS + 1
      IF (IPUB.GT.0.AND.NSETS.GT.IPUB) GO TO 2040
      IF (IPUB.GT.0) ISETX = IPSTOR(NSETS)
 1620 NUMSET = NUMSET + 1
C     PERMUTE STARTING PHASES
      CALL ASIGN(MARK,ISKIP,ISETX)
      IF (MARK.EQ.0) GO TO 2040
      IF (NUMSET.LE.ISKIP.OR.NUMSET.LT.ISETX) GO TO 1620
C     TANGENT FORMULA PHASE DETERMINATION
      IREJ=0
      IF (ITAN.NE.1) CALL FASTAN(IREJ)
      IF (ITAN.EQ.1) CALL SWTR(IREJ)
      IF (IREJ.EQ.1) NSETS=NSETS-1
      IF (IREJ.NE.(-1)) GO TO 1600
      NSETS=NSETS+1
      NSREQ=NUMSET
      WRITE (6,1700) NUMSET
 1700 FORMAT (//' SET ',I3,' ACCEPTED AS TRUE SOLUTION')
      GO TO 2040
C     * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C     STARTING PHASES INPUT FROM USER.
 1800 NSETS = 0
C     Phases and weights from CCP4.
      DO 1810 I = 1, IMK
       KDSTOR(I) = KDSC(I)
       MKG(I) = MKGC(I)
       WTSTOR(I) = WTSC(I)
       IF (MKGC(I).LT.0) WTSTOR(I)=-WTSTOR(I)
       MKG(I) = IPKC(I)
 1810 CONTINUE
      NUMSET = IPKC(IMK+1)
      IF (NUMSET.EQ.555)
     + WRITE(6,FMT='(/1X,A,I6,A)')
     + 'COMBINE RANDOM PHASES AND ',IMK
     +,' KNOWN PHASES AS A STARTING SET.'
      NSETS=NSETS+1
      IF (NUMSET .EQ. 0) GO TO 2040
      DO 1910 I=1,NUMB
      ALPHA(I) = 0.0
      WT(I) = 0.0
      IPHAZ(I) = 0
 1910 CONTINUE
C     COMBINE RANDOM PHASES AND KNOWN PHASES AS A STARTING SET.
 1950 NANT(1) = 0
      DO 1951 I=1,NUMB
      IZ(I)=0
      MKANG(I)=IABS(MKANG(I))
 1951 CONTINUE
      DO 1960 I=1,IMK
      IF (ABS(WTSTOR(I)).GT.0.85) THEN
       IND = KDSTOR(I)
        DO 1954 J=1,NNN
         JND = IORDE(J)
         IF (IND.EQ.JND) GO TO 1955
 1954   CONTINUE
        GO TO 1960
 1955   MKANG(IND) = INT(SIGN(1.0,WTSTOR(I)))*MKANG(IND)
        MKG(I)=MOD(MKG(I)+360,360)
        IF (MKG(I) .EQ. 0) MKG(I) = 360
        IZ(IND) = MKG(I)*1000+INT((2.0*ABS(WTSTOR(I))-1.0)*100.0)
       ENDIF
 1960 CONTINUE
      GO TO 1501
C     * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C     PHASE REFINEMENT COMPLETE
 2040 NUMSET = NSETS - 1
C     CHECK FOR IDENTICAL PHASE SETS
 3000 CALL COMPAR
      RETURN
      END
C-----------------------------------------------------------------------
C     GENERATE STARTING SETS OF PHASES FROM CONVERGENCE RESULTS
      SUBROUTINE ASIGN(MARK,ISKIP,ISETX)
      COMMON /USER/ MS,LIST,NSREQ,PROB,NSPEC,NGEN,NANY,KMIN,
     1 IMK,ITAN,IPUB,IIIII,WTFOM(3),KMAX,IWMIN,NRAN,IOFR,NSOUT,IPSTOR(5)
      COMMON /SYMTRY/ IS(2,3,24), TS(3,24), NSYM, LATT, ICENT, NORI
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ EEE(32000),ALPHA(2000),WT(2000)
      COMMON /B2/ IPH(32000),IPHAZ(2000),IORDE(2000)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),IH(2000),E(2000),
     1 LIM(2101),MKANG(2000)
      COMMON /B6/ IXRAN,IYRAN,PSIMIN,RMIN
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
      NNN=MIN0(NDET,NRAN)
      NAN = IABS(NANT(1))
 1000 MARK = 0
 1005 IF (NUMSET.GT.NSREQ) RETURN
C     GENERATE NEXT SET OF PHASES FOR STARTING POINT
 1006 MARK=1
      IF (MS.EQ.1) GO TO 1040
 1010 DO 1035 I=1,NUMB
      IF (MKG(I)) 1020,1020,1030
 1020 IF (IABS(MKANG(I)).EQ.1) GO TO 1025
      MPH=15*(IABS(MKANG(I))-1)
      IRA=INT(360.0*RANGEN(IXRAN,IYRAN)+0.5)
      IF (IRA.GT.180) MPH=MPH+180
      IZ(I)=MPH*1000+IWMIN
      IF (I.EQ.NAN) IZ(I)=MPH*1000+99
      GO TO 1035
 1025 MPH=INT(360.0*RANGEN(IXRAN,IYRAN)+0.5)
      IZ(I)=MPH*1000+IWMIN
 1026 IF (I.EQ.NAN) IZ(I)=MPH*1000+85
      GO TO 1035
 1030 IZ(I)=MKG(I)
 1035 CONTINUE
      MS=1
      GO TO 1070
 1040 J=2+ICENT
      DO 1045 I=J,4
      IF (NANT(I)) 1045,1060,1050
 1045 CONTINUE
      GO TO 1060
 1050 DO 1055 K=J,I
      NANT(K)=-NANT(K)
      NN=IABS(NANT(K))
      IZ(NN)=-(IZ(NN)-200)+360200
 1055 CONTINUE
      GO TO 1070
 1060 DO 1065 I=J,4
      NANT(I)=IABS(NANT(I))
 1065 CONTINUE
      MS=0
      GO TO 1010
 1070 DO 1200 LL=1,NUMB
      I=IORDE(LL)
      IF (LL.GT.NNN) GO TO 1081
      IWT=IZ(I)-1000*(IZ(I)/1000)
      ALPHA(I)=0.0025*IWT*IWT
      IF (IWT-100) 1080,1100,1100
 1080 WT(I)=-0.01*REAL(IWT)
      GO TO 1180
 1081 WT(I)=0.0
      GO TO 1200
 1100 WT(I)=1.0
      MKANG(I)=-IABS(MKANG(I))
 1180 IPHAZ(I)=MOD(IABS(IZ(I)/1000)+360,360)
 1190 IF (IPHAZ(I) .EQ. 0) IPHAZ(I) = 360
 1200 CONTINUE
      IF (NUMSET .LE. ISKIP .OR. NUMSET .LT. ISETX) RETURN
 1360 MARK=1
      RETURN
      END
C-----------------------------------------------------------------------
C     A FUNCTION FOR RANDOM NUMBER
      FUNCTION RANGEN (IX,IY)
      IX = MOD(251*IX,1048576)
      IY = MOD(179*IY,1048576)
      RANGEN = REAL (MOD(IX+IY,1048576))/1048576.0
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C         FFFFFFF     A      SSSSS   TTTTTTT     A     N     N         C
C         F          A A    S     S     T       A A    NN    N         C
C         F         A   A   S           T      A   A   N N   N         C
C         FFFFFF   A     A   SSSSS      T     A     A  N  N  N         C
C         F        AAAAAAA        S     T     AAAAAAA  N   N N         C
C         F        A     A  S     S     T     A     A  N    NN         C
C         F        A     A   SSSSS      T     A     A  N     N         C
C                                                                      C
C                        FAST TANGENT FORMULA                          C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE FASTAN(IREJ)
      LOGICAL TEST
C     DIMENSION STATEMENT PARTICULAR TO SUBROUTINE FASTAN
      DIMENSION CTABLE(360)
      COMMON /USER/ MS,LIST,NSREQ,PROB,NSPEC,NGEN,NANY,KMIN,
     1 IMK,ITAN,IPUB,ISKIP,WTFOM(3),KMAX,IWMIN,NRAN,IOFR,NSOUT,IPSTOR(5)
      COMMON /LOCAL/ AMX(3),AMN(3),LINE(20),LINEX(20),BLK1(54)
      COMMON /CONST/ KUSER1, KUSER2,
     1 KUSER3,KUSER4,KUSER5,KUSER6,DTOR,RTOD,STABLE(450)
      COMMON /SYMTRY/ IS(2,3,24), TS(3,24), NSYM, LATT, ICENT, NORI
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ EEE(32000),ALPHA(2000),WT(2000)
      COMMON /B2/ IPH(32000),IPHAZ(2000),IORDE(2000)
      COMMON /B3/ EALF(2000),TALF(2000),RALF(2000),ISPZRO(100),FOM(400)
     1,           BLK3(1500)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),SUMNUM(2000),
     1 SUMDEN(2000),LIM(2101),MKANG(2000)
      COMMON /B6/ IXRAN,IYRAN,PSIMIN,RMIN
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
C     EQUIVALENCE STATEMENT PARTICULAR TO SUBROUTINE FASTAN
      EQUIVALENCE (STABLE(91),CTABLE(1))
C     I1/I0 (BESSEL FUNCTION)
      VEC(U)=U*(U+0.4807)/((U+0.8636)*U+1.3943)
      NNN=MIN0(NDET,NRAN)
      IF (IMK.GT.50) NNN=NDET
      NCYCLE = 0
  900 MARK=0
      SALF = 0.0
C     CLEAR ARRAYS AT START OF DETERMINATION
      DO 990 I=1,NUMB
      EALF(I)=0.0
      TALF(I)=0.0
      PALF(I)=0.0
      SUMNUM(I)=0.0
      SUMDEN(I)=0.0
  990 CONTINUE
C     START OF NEXT CYCLE
 1000 SUMALF=SALF
      NCYCLE=NCYCLE+1
      SALF=0.0
C     TEST IS .TRUE. IF EXPECTED ALPHA IS TO BE CALCULATED
C     FINAL 2 CYCLES
      TEST=(MARK.GT.0)
      DO 1500 I=1,NNN
      LL=IORDE(I)
      LI=LIM(LL)+1
      LS=LIM(LL+1)
      IF (LI.GT.LS) GO TO 1200
      DO 1100 JJ=LI,LS
      IL=IPH(JJ)/131072-2048
      ILA=IABS(IL)
      IF (ABS(WT(ILA)).LT.0.1) GO TO 1100
      IR=(IPH(JJ)-131072*(IL+2048))/32-2048
      IRA=IABS(IR)
 1040 IF (ABS(WT(IRA)).LT.0.1) GO TO 1100
      EE=EEE(JJ)*ABS(WT(ILA)*WT(IRA))
      IF (TEST) VECX=EE*VEC(EEE(JJ))
      IF (TEST) EALF(LL)=EALF(LL)+VECX
      IF (TEST) TALF(LL)=TALF(LL)+EE*EE-VECX*VECX
      IP=15*(IPH(JJ)-32*(IPH(JJ)/32))
      IF (ICENT.EQ.1) GO TO 1050
      IFL=ISIGN(IPHAZ(ILA),IL)
      IFR=ISIGN(IPHAZ(IRA),IR)
      IARG=MOD(IFL+IFR+IP+1440,360)+1
      SUMNUM(LL)=SUMNUM(LL)+EE*STABLE(IARG)
      SUMDEN(LL)=SUMDEN(LL)+EE*CTABLE(IARG)
      GO TO 1100
 1050 IF (MOD(IPHAZ(ILA)+IPHAZ(IRA)+IP,360).NE.0) EE=-EE
      SUMNUM(LL)=SUMNUM(LL)+EE
 1100 CONTINUE
 1200 ID=15*IABS(MKANG(LL))-14
      IF (ID.EQ.1.OR.ICENT.EQ.1) GO TO 1300
      T2=SUMNUM(LL)*STABLE(ID)+SUMDEN(LL)*CTABLE(ID)
      SUMNUM(LL)=T2*STABLE(ID)
      SUMDEN(LL)=T2*CTABLE(ID)
 1300 ALFA=SUMNUM(LL)**2+SUMDEN(LL)**2
      IF (ALFA.EQ.0.0) GO TO 1500
      IF (MKANG(LL).LE.0) GO TO 1320
      SALF=SALF+ALFA
      WATE=1.0
      IF (ALFA.LT.25.0) WATE=0.2*SQRT(ALFA)
      IF (WT(LL).LT.0.0.AND.WATE.LT.(-WT(LL))) GO TO 1320
      WT(LL)=AMAX1(WATE,0.15)
      IF (ICENT.NE.1) PHAZ=RTOD*ATAN2(SUMNUM(LL),SUMDEN(LL))
      IF (ICENT.EQ.1) PHAZ=90.0-SIGN(90.0,SUMNUM(LL))
      IF (PHAZ.LE.0.0) PHAZ=PHAZ+360.0
      IPHAZ(LL)=INT(PHAZ+0.5)
 1320 ALPHA(LL)=ALFA
      IF (TEST) PALF(LL)=EALF(LL)*EALF(LL)+TALF(LL)
 1340 IF (LI.GT.LS) GO TO 1490
      DO 1450 JJ=LI,LS
      IL=IPH(JJ)/131072-2048
      ILA=IABS(IL)
      IR=(IPH(JJ)-131072*(IL+2048))/32-2048
      IRA=IABS(IR)
 1342 IP=15*(IPH(JJ)-32*(IPH(JJ)/32))
      IF (ABS(WT(IRA)).LT.0.1) GO TO 1350
      EE=EEE(JJ)*ABS(WT(IRA)*WT(LL))
      IF (TEST) VECX=EE*VEC(EEE(JJ))
      IF (TEST) EALF(ILA)=EALF(ILA)+VECX
      IF (TEST) TALF(ILA)=TALF(ILA)+EE*EE-VECX*VECX
      IF (ICENT.EQ.1) GO TO 1345
      IFR=ISIGN(IPHAZ(IRA),IR)
      IARG=MOD(1440-(IFR-IPHAZ(LL)+IP)*ISIGN(1,IL),360)+1
      SUMNUM(ILA)=SUMNUM(ILA)+EE*STABLE(IARG)
      SUMDEN(ILA)=SUMDEN(ILA)+EE*CTABLE(IARG)
      GO TO 1350
 1345 IF (MOD(IPHAZ(IRA)+IPHAZ(LL)+IP,360).NE.0) EE=-EE
      SUMNUM(ILA)=SUMNUM(ILA)+EE
 1350 IF (ABS(WT(ILA)).LT.0.1) GO TO 1450
      EE=EEE(JJ)*ABS(WT(ILA)*WT(LL))
      IF (TEST) VECX=EE*VEC(EEE(JJ))
      IF (TEST) EALF(IRA)=EALF(IRA)+VECX
      IF (TEST) TALF(IRA)=TALF(IRA)+EE*EE-VECX*VECX
      IF (ICENT.EQ.1) GO TO 1440
      IFL=ISIGN(IPHAZ(ILA),IL)
      IARG=MOD(1440-(IFL-IPHAZ(LL)+IP)*ISIGN(1,IR),360)+1
      SUMNUM(IRA)=SUMNUM(IRA)+EE*STABLE(IARG)
      SUMDEN(IRA)=SUMDEN(IRA)+EE*CTABLE(IARG)
      GO TO 1450
 1440 IF (MOD(IPHAZ(ILA)+IPHAZ(LL)+IP,360).NE.0) EE=-EE
      SUMNUM(IRA)=SUMNUM(IRA)+EE
 1450 CONTINUE
 1490 SUMNUM(LL)=0.0
      SUMDEN(LL)=0.0
      EALF(LL)=0.0
      TALF(LL)=0.0
 1500 CONTINUE
C     FASTAN CONTROL STATEMENTS
      IF (NCYCLE.LT.6) GO TO 1000
 1510 IF (NNN.EQ.NDET) GO TO 1550
 1520 NNN=NDET
      GO TO 1000
 1550 IF ((SALF-SUMALF)/SALF.GT.0.02.AND.MARK.EQ.0) GO TO 1000
      IF (MARK.NE.0) GO TO 1700
      DO 1600 LL=1,NUMB
      MKANG(LL)=IABS(MKANG(LL))
 1600 CONTINUE
 1700 MARK=MARK+1
C     2 CYCLES OF REFINING ALL PHASES AT END
      IF (MARK.LE.2) GO TO 1000
C     CALCULATE FINAL F.O.M.S AND OUTPUT RESULTS
 4700 RESID = 0.0
      SUMEO = 0.0
      SUMALF = 0.0
      NUNDET = NDET-NUMB
      ALFEST=0.0
      DO 4920 LL=1,NUMB
      IF (INT(100.0*WT(LL)) .EQ.(-IWMIN)) NUNDET = NUNDET + 1
      IF (ABS(WT(LL)).LT.0.05) NUNDET=NUNDET+1
      IF (ABS(WT(LL)).LT.0.05) GO TO 4920
      ALPHA(LL) = SQRT(ALPHA(LL))
      PALF(LL) = SQRT(PALF(LL))
      TTT = ALPHA(LL)/PALF(LL)
      WT(LL) = AMIN1(1.0,TTT)
      ALFEST = ALFEST + PALF(LL)
      SUMALF = SUMALF + ALPHA(LL)
      IF (IPHAZ(LL) .LE. 0) IPHAZ(LL) = IPHAZ(LL) + 360
      SUMEO = SUMEO + PALF(LL)
 4920 CONTINUE
C     CALCULATE ABSOLUTE FIGURE OF MERIT
      ABSFOM = (SUMALF - ALFRAN) / (ALFEST - ALFRAN)
C     CALCULATE FINAL PSI ZERO FIGURE OF MERIT
      CALL EFOM(IREJ,PSIZRO)
      PSIZRO = PSIZRO / AMIN1(1.3,ABSFOM)
C     CALCULATE A SCALED RESIDUAL
      SC = 1.0
      DO 4930 LL=1,NUMB
      IF (ABS(WT(LL)).LT.0.05) GO TO 4930
      RESID=RESID+ABS(SC*PALF(LL)-ALPHA(LL))
 4930 CONTINUE
      IF (SUMEO.LE.0.1) RESID = 100.0
      IF (SUMEO.GT.0.1) RESID = 100.0 * RESID / SUMEO
      IF (PSIMIN.LT.PSIZRO.OR.RMIN.LT.RESID) GO TO 4955
      PSIMIN=PSIZRO
      RMIN=RESID
C     REJECT SET IF TOO FEW PHASES DETERMINED
 4955 IF (NUNDET.GE.NDET/3) IREJ=1
      IF (NUNDET.GE.NDET/3) RETURN
      WRITE (2) NUMSET,ABSFOM,PSIZRO,RESID,
     1         (IPHAZ(I),WT(I),I=1,NUMB),NUNDET
C     TEST FOR ACCEPTABLE SOLUTION
      IF (PSIZRO.LT.AMN(2)) AMN(2) = PSIZRO + 0.01
      IF (RESID.LT.AMN(3)) AMN(3) = RESID + 0.1
      IF (PSIZRO.GT.1.30.OR.PSIZRO.GT.AMN(2)) RETURN
      IF (RESID.GT.21.0.OR.RESID.GT.AMN(3)) RETURN
      IREJ = -1
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C                   SSSSS   W     W  TTTTTTT  RRRRRR                   C
C                  S     S  W     W     T     R     R                  C
C                  S        W     W     T     R     R                  C
C                   SSSSS   W  W  W     T     RRRRRR                   C
C                        S  W W W W     T     R   R                    C
C                  S     S  WW   WW     T     R    R                   C
C                   SSSSS   W     W     T     R     R                  C
C                                                                      C
C                 STATISTICAL WEIGHTED TANGENT FORMULA                 C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE SWTR(IREJ)
C     DIMENSION STATEMENT PARTICULAR TO SUBROUTINE SWTR
      DIMENSION CTABLE(360)
      COMMON /USER/ MS,LIST,NSREQ,PROB,NSPEC,NGEN,NANY,KMIN,
     1 IMK,ITAN,IPUB,ISKIP,WTFOM(3),KMAX,IWMIN,NRAN,IOFR,NSOUT,IPSTOR(5)
      COMMON /LOCAL/ AMX(3),AMN(3),LINE(20),LINEX(20),BLK1(54)
      COMMON /CONST/ KUSER1, KUSER2,
     1 KUSER3,KUSER4,KUSER5,KUSER6,DTOR,RTOD,STABLE(450)
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ EEE(32000),ALPHA(2000),WT(2000)
      COMMON /B2/ IPH(32000),IPHAZ(2000),IORDE(2000)
      COMMON /B3/ EALF(2000),TALF(2000),RALF(2000),KNOWN(2000)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),SUMNUM(2000),
     1 SUMDEN(2000),LIM(2101),MKANG(2000)
      COMMON /B6/ IXRAN,IYRAN,PSIMIN,RMIN
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
C     EQUIVALENCE STATEMENT PARTICULAR TO SUBROUTINE SWTR
      EQUIVALENCE (STABLE(91),CTABLE(1))
C     I1/I0 (BESSEL FUNCTION)
      VEC(U)=U*(U+0.4807)/((U+0.8636)*U+1.3943)
      SCALE=SIGMA/0.09
      MARK = 0
      NNN=MIN0(NDET,NRAN)
      CUT=12.5
C     DO NOT REFINE STARTING PHASES IF HEAVY ATOM IS IN STRUCTURE
      SALF = 0.0
      IF (IMK.GT.50) NNN=NDET
C     INITIALISE ARRAYS AT START OF EACH CYCLE
 1000 DO 1010 I=1,NUMB
      SUMNUM(I)=0.0
      SUMDEN(I)=0.0
      EALF(I)=0.0
      TALF(I)=0.0
      RALF(I)=0.0
      IF (ABS(WT(I)).LT.0.1) IPHAZ(I)=-1
 1010 CONTINUE
      SUMALF=SALF
      SALF=0.0
C     CALCULATE TOP & BOTTOM OF TANGENT FORMULA FOR EACH REFLEXION
      DO 1250 I=1,NNN
      LL=IORDE(I)
      LI=LIM(LL)+1
      LS=LIM(LL+1)
      IF (LI.GT.LS) GO TO 1250
      DO 1100 JJ=LI,LS
      KN=7
      IF (IPHAZ(LL).GE.0) KN=KN-1
      IL=IPH(JJ)/131072-2048
      ILA=IABS(IL)
      IF (IPHAZ(ILA).GE.0) KN=KN-2
      IF (KN.EQ.7) GO TO 1100
      IR=(IPH(JJ)-131072*(IL+2048))/32-2048
      IRA=IABS(IR)
      IF (IPHAZ(IRA).GE.0) KN=KN-3
      IF (KN.GT.4) GO TO 1100
 1030 IP=15*(IPH(JJ)-32*(IPH(JJ)/32))
      VECE=VEC(EEE(JJ))
      GO TO (1040,1040,1060,1050),KN
 1040 EE=EEE(JJ)*ABS(WT(ILA)*WT(IRA))
      VECX=EE*VECE
      EALF(LL)=EALF(LL)+VECX
      TALF(LL)=TALF(LL)+EE*EE-VECX*VECX
      RALF(LL)=RALF(LL)+EE*EE
      IFL=ISIGN(IPHAZ(ILA),IL)
      IFR=ISIGN(IPHAZ(IRA),IR)
      IARG=MOD(IFL+IFR+IP+1440,360)+1
      SUMNUM(LL)=SUMNUM(LL)+EE*STABLE(IARG)
      SUMDEN(LL)=SUMDEN(LL)+EE*CTABLE(IARG)
      IF (KN.NE.1) GO TO 1100
 1050 EE=EEE(JJ)*ABS(WT(ILA)*WT(LL))
      VECX=EE*VECE
      EALF(IRA)=EALF(IRA)+VECX
      TALF(IRA)=TALF(IRA)+EE*EE-VECX*VECX
      RALF(IRA)=RALF(IRA)+EE*EE
      IF (KN.NE.1) IFL=ISIGN(IPHAZ(ILA),IL)
      IARG=MOD(1440-(IFL-IPHAZ(LL)+IP)*ISIGN(1,IR),360)+1
      SUMNUM(IRA)=SUMNUM(IRA)+EE*STABLE(IARG)
      SUMDEN(IRA)=SUMDEN(IRA)+EE*CTABLE(IARG)
      IF (KN.NE.1) GO TO 1100
 1060 EE=EEE(JJ)*ABS(WT(IRA)*WT(LL))
      VECX=EE*VECE
      EALF(ILA)=EALF(ILA)+VECX
      TALF(ILA)=TALF(ILA)+EE*EE-VECX*VECX
      RALF(ILA)=RALF(ILA)+EE*EE
      IF (KN.NE.1) IFR=ISIGN(IPHAZ(IRA),IR)
      IARG=MOD(1440-(IFR-IPHAZ(LL)+IP)*ISIGN(1,IL),360)+1
      SUMNUM(ILA)=SUMNUM(ILA)+EE*STABLE(IARG)
      SUMDEN(ILA)=SUMDEN(ILA)+EE*CTABLE(IARG)
 1100 CONTINUE
 1250 CONTINUE
C     UPDATE PHASES AT END OF CYCLE
      DO 1500 I=1,NNN
      LL=IORDE(I)
      IF (MKANG(LL).LE.0) GO TO 1500
      ID=15*MKANG(LL)-14
      IF (ID.EQ.1) GO TO 1300
      T2=SUMNUM(LL)*STABLE(ID)+SUMDEN(LL)*CTABLE(ID)
      SUMNUM(LL)=T2*STABLE(ID)
      SUMDEN(LL)=T2*CTABLE(ID)
 1300 ALFA=SUMNUM(LL)**2+SUMDEN(LL)**2
      IF (ALFA.EQ.0.0) GO TO 1500
      WATE=1.0
      IF (ALFA.LT.25.0) WATE=0.2*SQRT(ALFA)
      PALF(LL)=TALF(LL)+EALF(LL)*EALF(LL)
      WX1 = ALFA / PALF(LL)
      WX = 0
      IF (WX1.GT.1.03) WX=1.03-(WX1-1.03)*SCALE
      IF (WX1.GT.1.03) WX=1.03-(WX1-1.03)*SCALE
      IF (WX1.GT.1.03) WX=WX/((1.2178*WX-1.0698)*WX+0.858)
      IF (WX.LT.WATE.AND.WX1.GT.1.03) WATE=WX
      IF (WT(LL).LT.0.0.AND.WATE.LT.(-WT(LL))) GO TO 1500
      SALF = SALF + WATE
      WT(LL)=AMAX1(WATE,0.15)
      PHAZ=RTOD*ATAN2(SUMNUM(LL),SUMDEN(LL))
      IF (PHAZ.LT.0.5) PHAZ=PHAZ+360.0
      IPHAZ(LL)=INT(PHAZ+0.5)
      ALPHA(LL)=ALFA
 1500 CONTINUE
C     SWTR CONTROL STATEMENTS
 1540 CUT=AMAX1(0.5*CUT,0.05)
      IF (CUT.GT.0.1) GO TO 1000
 1545 IF (NNN.EQ.NDET) GO TO 1550
      NNN=MIN0(NNN+NNN/2,NDET)
      GO TO 1000
 1550 IF ((SALF-SUMALF)/SALF.GT.0.02.AND.MARK.EQ.0) GO TO 1000
      IF (MARK.EQ.1) GO TO 1620
      DO 1600 LL=1,NUMB
      MKANG(LL)=IABS(MKANG(LL))
 1600 CONTINUE
      MARK = 1
      GO TO 1000
C     CALCULATE AND OUTPUT FINAL FIGURES OF MERIT
 1620 ALFEST=0.0
      ALFRAN=0.0
      SALF=0.0
      SUMEO=0.0
      RESID=0.0
      NUNDET=NDET-NUMB
      DO 1700 I=1,NUMB
      IF (ABS(WT(I)).LT.0.05.OR.INT(100.0*WT(I)).EQ.(-IWMIN))
     1                    NUNDET=NUNDET+1
      IF (ABS(WT(I)).LT.0.05) GO TO 1700
      ALPHA(I)=SQRT(ALPHA(I))
      PALF(I)=SQRT(PALF(I))
      TTT = ALPHA(I)/PALF(I)
      WT(I) = AMIN1(1.0,TTT)
      ALFEST=ALFEST+PALF(I)
      ALFRAN=ALFRAN+SQRT(RALF(I))
      SALF=SALF+ALPHA(I)
      SUMEO=SUMEO+PALF(I)
 1700 CONTINUE
C     CALCULATE ABSOLUTE FIGURE OF MERIT
      ABSFOM = (SALF-ALFRAN)/(ALFEST-ALFRAN)
C     CALCULATE FINAL PSIZERO FIGURE OF MERIT
      CALL EFOM(IREJ,PSIZRO)
      PSIZRO = PSIZRO / AMIN1(1.3,ABSFOM)
C     CALCULATE A SCALED RESIDUAL
      SC = 1.0
      DO 1720 LL=1,NUMB
      IF (ABS(WT(LL)).LT.0.05) GO TO 1720
      RESID=RESID+ABS(SC*PALF(LL)-ALPHA(LL))
 1720 CONTINUE
      IF (SUMEO.LE.0.1) RESID = 100.0
      IF (SUMEO.GT.0.1) RESID = 100.0 * RESID / SUMEO
      IF (PSIMIN.LT.PSIZRO.OR.RMIN.LT.RESID) GO TO 1885
      PSIMIN=PSIZRO
      RMIN=RESID
C     REJECT SET IF TOO FEW PHASES DETERMINED
 1885 IF (NUNDET.GE.NDET/3) GO TO 2000
C     TEST FOR ACCEPTABLE SOLUTION
      IF (PSIZRO.LT.AMN(2)) AMN(2) = PSIZRO + 0.01
      IF (RESID.LT.AMN(3)) AMN(3) = RESID + 0.1
      IF (PSIZRO.GT.1.30.OR.PSIZRO.GT.AMN(2)) GO TO 1890
      IF (RESID.GT.21.0.OR.RESID.GT.AMN(3)) GO TO 1890
      IREJ = -1
 1890 DO 1900 I=1,NUMB
 1900 CONTINUE
      WRITE (2) NUMSET,ABSFOM,PSIZRO,RESID,
     1         (IPHAZ(I),WT(I),I=1,NUMB),NUNDET
      RETURN
 2000 IREJ=1
      RETURN
      END
C-----------------------------------------------------------------------
C     CALCULATE FINAL PSI ZERO.
      SUBROUTINE EFOM(IREJ,VALUE)
C     DIMENSION STATEMENT PARTICULAR TO SUBROUTINE EFOM
      DIMENSION CTABLE(360)
      COMMON /CONST/ KUSER1, KUSER2,
     1 KUSER3,KUSER4,KUSER5,KUSER6,DTOR,RTOD,STABLE(450)
      COMMON /SYMTRY/ IS(2,3,24), TS(3,24), NSYM, LATT, ICENT, NORI
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ EEE(32000),ALPHA(2000),WT(2000)
      COMMON /B2/ IPH(32000),IPHAZ(2000),IORDE(2000)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),IH(2000),E(2000),
     1 LIM(2101),MKANG(2000)
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
C     EQUIVALENCE STATEMENT PARTICULAR TO SUBROUTINE EFOM
      EQUIVALENCE (CTABLE(1),STABLE(91))
      IREJ=0
      PSIZRO=0.0
      RANPSI=0.0
      VALUE=2.0
      IF (IZRO.EQ.0) RETURN
C     CALCULATE A PSI ZERO FIGURE OF MERIT
      DO 1500 I=1,IZRO
      LL=I+NUMB
      LI=LIM(LL)+1
      LS=LIM(LL+1)
      IF (LI.GT.LS) GO TO 1500
      TOP=0.0
      BOT=0.0
      RAN=0.0
      DO 1100 JJ=LI,LS
      IL=IPH(JJ)/131072-2048
      ILA=IABS(IL)
      IF (ABS(WT(ILA)).LT.0.1) GO TO 1100
      IR=(IPH(JJ)-131072*(IL+2048))/32-2048
      IRA=IABS(IR)
      IF (ABS(WT(IRA)).LT.0.1) GO TO 1100
      EE=EEE(JJ)
      RAN=RAN+EE*EE
      IP=15*(IPH(JJ)-32*(IPH(JJ)/32))
      IF (ICENT.EQ.1) GO TO 1050
      IFL=ISIGN(IPHAZ(ILA),IL)
      IFR=ISIGN(IPHAZ(IRA),IR)
      IARG=MOD(IFL+IFR+IP+1440,360)+1
      TOP=TOP+EE*STABLE(IARG)
      BOT=BOT+EE*CTABLE(IARG)
      GO TO 1100
 1050 IF (MOD(IPHAZ(ILA)+IPHAZ(IRA)+IP,360).NE.0) EE=-EE
      TOP=TOP+EE
 1100 CONTINUE
      PSIZRO=PSIZRO+SQRT(TOP*TOP+BOT*BOT)
      RANPSI=RANPSI+SQRT(RAN)
 1500 CONTINUE
      IF (RANPSI.GT.0.1) VALUE=PSIZRO/RANPSI
      RETURN
      END
C-----------------------------------------------------------------------
C     CHECK FOR EQUIVALENT PHASE SETS - TO AVOID DUPLICATION IN FFT
      SUBROUTINE COMPAR
C     DIMENSION STATEMENT PARTICULAR TO SUBROUTINE COMPAR
      DIMENSION CTABLE(360),IPTEM(2000),AVER(2000),IEQ(2000),IUND(2000)
     +         ,NAMEI(2000)
      COMMON /LOCAL/ AMX(3), AMN(3), LINE(20), LINEX(20),BLK1(54)
      COMMON /USER/ MS,LIST,NSREQ,PROB,NSPEC,NGEN,NANY,KMIN,
     1 IMK,ITAN,IPUB,ISKIP,WTFOM(3),KMAX,IWMIN,NRAN,IOFR,NSOUT,IPSTOR(5)
      COMMON /CONST/ KUSER1, KUSER2,
     1 KUSER3,KUSER4,KUSER5,KUSER6,DTOR,RTOD,STABLE(450)
      COMMON /SYMTRY/ IS(2,3,24), TS(3,24), NSYM, LATT, ICENT, NORI
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B1/ FM(2000,4),BLK2(28000)
      COMMON /B2/ IPH(32000),IPHAZ(2000),IORDE(2000)
      COMMON /B3/ KNOWN(2000),WT(2000),WTT(2000),BLK3(2000)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),IH(2000),E(2000),
     1 LIM(2101),MKANG(2000)
      COMMON /BESET/NBEST(5),CBEST(5,4),IPBEST(2000,5),WTBEST(2000,5)
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
C     EQUIVALENCE STATEMENT PARTICULAR TO SUBROUTINE COMPAR
      EQUIVALENCE (STABLE(91),CTABLE(1))
      DO 60 J=1, NSOUT
       NBEST(J) = 0
       CBEST(J,4) = -1.0
       DO 50 I = 1, 2000
        NAMEI(I) = 0
        IEQ(I) = 0
        IPBEST(I,J) = 0
        WTBEST(I,J) = 0.0
   50  CONTINUE   
   60 CONTINUE   
      FM(1,4)=3.0
C     COMPUTE COMBINED FIGURES OF MERIT AND OUTPUT SUMMARY OF RESULTS
      REWIND 2
      READ (2)
      READ (2)
C     INITIALISE AMX,AMN AND NORMALISE CFOM TO 1.0
      IF (IZRO.EQ.0) WTFOM(2) = 0.0
      FACTOR = 1.0 / (WTFOM(1)+WTFOM(2)+WTFOM(3))
      DO 80 I=1,3
      AMX(I)=-1.0E20
      AMN(I)=1.0E20
      WTFOM(I)=WTFOM(I)*FACTOR
   80 CONTINUE
      DO 120 I=1,NUMSET
      READ (2)  J,(FM(I,J),J=1,3)
      DO 100 J=1,3
      AMX(J) = AMAX1(AMX(J), FM(I,J))
      AMN(J) = AMIN1(AMN(J), FM(I,J))
  100 CONTINUE
  120 CONTINUE
      WRITE (6,140) AMX,AMN,WTFOM
  140 FORMAT(//' ------------------------------------------',
     1 '---------------------------------------------------',
     1//' SUMMARY OF FIGURES'
     1,' OF MERIT OUTPUT BY TANGENT FORMULA'//,
     1  26X,'ABS FOM           PSI ZERO           RESID',//,
     1 ' MAXIMUM VALUE',F19.4,F18.3,F17.2//
     3,' MINIMUM VALUE',F19.4,F18.3,F17.2
     4,//' WEIGHTS OF COMBINED FOM',F7.2,F19.2,F18.2,///,
     4   ' NUMSET  UNDETER  ABS FOM    PSI ZERO    RESIDUAL  ',
     5   'COMBINED FOM    REMARK')
      IF (AMX(1) .EQ. AMN(1)) AMN(1) = 0.0
      IF (AMX(2) .EQ. AMN(2)) AMX(2) = 100.0
      IF (AMX(3) .EQ. AMN(3)) AMX(3) = 100.0
      DO 180 I=1,NUMSET
      FMABS=FM(I,1)
      IF (FMABS.GT.1.0) FMABS=2.0-FMABS
      IF (FMABS.LT.0.0) FMABS=AMN(1)
      FM(I,4) = WTFOM(1) * (FMABS - AMN(1)) / (AMX(1) - AMN(1)) +
     1          WTFOM(2) * (AMX(2) - FM(I,2)) / (AMX(2) - AMN(2)) +
     2          WTFOM(3) * (AMX(3) - FM(I,3)) / (AMX(3) - AMN(3))
  180 CONTINUE
      IREC = 100
      ISET = 0
      JSET = 0
C     ALLOW AVERAGE DIFFERENCE OF ARCSIN(0.034)
      TEST = 0.034*REAL(LIM(NUMB+1))
      REWIND 2
      READ (2)
      DO 1000 I=1,NUMSET
      AVER(I) = 0.0
      CALL RTAPE(IPHAZ,WT,IREC,ISET,I,NUMB,NAMEI(I),IUND(I))
      IF (ICENT.EQ.0) CALL TRIPLE(AVER(I))
      IF (I.EQ.NUMSET) GO TO 910
      NS = I+1
      DO 900 J=NS,NUMSET
      IF (IEQ(J).GT.0) GO TO 900
C     COMPARE SETS WITH SIMILAR FOMS
      IF (ABS(FM(J,1)-FM(I,1)) .GT. 0.05) GO TO 900
      IF (ABS(FM(J,2)-FM(I,2)) .GT. 0.05) GO TO 900
      IF (ABS(FM(J,3)-FM(I,3)) .GT. 0.4) GO TO 900
      IF (J .NE. JSET) CALL RTAPE(IPTEM,WTT,IREC,JSET,J,NUMB,NAMJ,JA)
      TESTI = 0.0
      TESTE = 0.0
      DO 600 K=1,NUMB
      LI = LIM(K)+1
      LS = LIM(K+1)
      IF (LI .GT. LS) GO TO 600
      DO 500 JJ=LI,LS
      IL = IPH(JJ)/131072-2048
      ILA = IABS(IL)
      IR=(IPH(JJ)-131072*(IL+2048))/32-2048
      IRA = IABS(IR)
      IP = 15*(IPH(JJ)-32*(IPH(JJ)/32))
      IPI = IPHAZ(K)-ISIGN(IPHAZ(ILA),IL)-ISIGN(IPHAZ(IRA),IR)-IP
      IPJ = IPTEM(K)-ISIGN(IPTEM(ILA),IL)-ISIGN(IPTEM(IRA),IR)-IP
      IARG = MOD(IPI-IPJ+3600,360)+1
      TESTI = TESTI+1.0-CTABLE(IARG)
      IARG = MOD(IPI+IPJ+3600,360)+1
      TESTE = TESTE+1.0-CTABLE(IARG)
      IF (TESTI .GT. TEST .AND. TESTE .GT. TEST) GO TO 900
  500 CONTINUE
  600 CONTINUE
      IEQ(J) = NAMEI(I)
  900 CONTINUE
  910 IF (LIST.LE.0.AND.FM(I,4).GT.0.6) THEN
       IF (IEQ(I).EQ.0) THEN
        IF (AVER(I).GT.15.0) THEN
         WRITE(6,920) NAMEI(I),IUND(I),(FM(I,J),J=1,4)
  920    FORMAT(1X,I6,I6,F12.4,F11.3,F12.2,F12.4)
        ELSE
         WRITE(6,930) NAMEI(I),IUND(I),(FM(I,J),J=1,4)
  930    FORMAT(1X,I6,I6,F12.4,F11.3,F12.2,F12.4,4X,'PSEUDO-CENTRIC')
        END IF
       ELSE
        WRITE(6,940) NAMEI(I),IUND(I),(FM(I,J),J=1,4),IEQ(I)
  940   FORMAT(1X,I6,I6,F12.4,F11.3,F12.2,F12.4,4X,'= SET ',I5)
       END IF
      END IF
      IF (LIST.GE.1) THEN
       IF (IEQ(I).EQ.0) THEN
        WRITE(6,950) NAMEI(I),IUND(I),(FM(I,J),J=1,4),AVER(I)
  950 FORMAT(1X,I6,I6,F12.4,F11.3,F12.2,F12.4,4X,F8.2)
       ELSE
        WRITE(6,940) NAMEI(I),IUND(I),(FM(I,J),J=1,4),IEQ(I)
       END IF
      END IF
 1000 CONTINUE
      IF (NUMSET.EQ.1) THEN
       CBEST(1,1) = FM(1,1)
       CBEST(1,2) = FM(1,2)
       CBEST(1,3) = FM(1,3)
       CBEST(1,4) = FM(1,4)
       NBEST(1) = 1
       GO TO 2100
      ENDIF
      ISET01=ISKIP+1
      WRITE (6,1010) ISET01,NSREQ
 1010 FORMAT(/' PHASE DEVELOPED FROM SET NUMBER',I5,' TO',I5,/)
      NSC = 0
      DO 2000 I=1,NUMSET
       IF (IEQ(I).GT.0) GO TO 2000
       NSC = NSC + 1
       DO 1100 J=1, NSOUT
        IF (FM(I,4).GT.CBEST(J,4)) THEN
         DO 1050 K=J+1, NSOUT
          KT = NSOUT - K + J
          CBEST(KT+1,1) = CBEST(KT,1)
          CBEST(KT+1,2) = CBEST(KT,2)
          CBEST(KT+1,3) = CBEST(KT,3)
          CBEST(KT+1,4) = CBEST(KT,4)
          NBEST(KT+1) = NBEST(KT)
 1050    CONTINUE
        CBEST(J,1) = FM(I,1)
        CBEST(J,2) = FM(I,2)
        CBEST(J,3) = FM(I,3)
        CBEST(J,4) = FM(I,4)
        NBEST(J) = I
        GO TO 2000
        END IF
 1100  CONTINUE
 2000 CONTINUE
      IF (NSC.LT.NSOUT) THEN
       DO 2010 I=NSC+1, NSOUT
        CBEST(I,1) = FM(I,1)
        CBEST(I,2) = FM(I,2)
        CBEST(I,3) = FM(I,3)
        CBEST(I,4) = FM(I,4)
        NBEST(I) = I
 2010  CONTINUE
      ENDIF
 2100 IF (NUMSET.GT.1) THEN
       WRITE(6,2105) NSOUT
       DO 2104 J=1, NSOUT
        N = NBEST(J)
        WRITE(6,2110) NAMEI(N),(CBEST(J,K),K=1,4)
 2104  CONTINUE
 2105  FORMAT(/' THE NUMBER OF THE BEST SETS IN OUTPUT FILE = ',I5
     +,  //' NUMSET  ABS FOM   PSI ZERO    RESIDUAL  COMBINED FOM')
 2110  FORMAT(1X,I6,F9.4,F10.3,F12.2,F12.4)
      ENDIF
      DO 2300 J = 1, NSOUT
       JSET = NBEST(J)
       CALL RTAPE(IPHAZ,WT,IREC,ISET,JSET,NUMB,NAMI,JA)
       DO 2200 I = 1, NUMB
        IPBEST(I,J) = IPHAZ(I)
        WTBEST(I,J) = WT(I)
 2200  CONTINUE
 2300 CONTINUE
      RETURN
      END
C-----------------------------------------------------------------------
C     POSITION TAPE FOR READING SET I
      SUBROUTINE RTAPE(IA,B,IREC,ISET,I,NUMB,NAME,NUND)
      DIMENSION IA(NUMB),B(NUMB)
      ISET = I
      IF (I .GE. IREC) GO TO 10
      IREC = -1
      REWIND 2
   10 IREC = IREC+1
      IF (IREC .GT. I) GO TO 20
      READ (2)
      GO TO 10
   20 READ (2) NAME,A,A,A,(IA(K),B(K),K=1,NUMB),NUND
      RETURN
      END
C-----------------------------------------------------------------------
C     CHECK FOR ENANTIOMORPHOUS AMBIGUITY
      SUBROUTINE TRIPLE(AVER)
      COMMON /PARAM/ E3MIN,SIGMA,MAXH(3),NUMB,IZRO,NUMSET,NANT(4),
     1 NRAL,MODUL(3),NREF,ALFRAN,NREC,PARA(6),NDET
      COMMON /B2/ IPH(32000),IPHAZ(2000),IORDE(2000)
      COMMON /B4/ MKG(2000),PALF(2000),IZ(2000),IH(2000),E(2000),
     1 LIM(2101),MKANG(2000)
C     * * * * * * * * * * * * * * * * ** * * * * * * * * * * * * * * * *
      NREL=0
      ISUMP3=0
      DO 1500 I=1,NDET
      LL=IORDE(I)
      LI=LIM(LL)+1
      LS=LIM(LL+1)
      IF (LI.GT.LS) GO TO 1500
      DO 1100 JJ=LI,LS
      IL=IPH(JJ)/131072-2048
      ILA=IABS(IL)
      IR=(IPH(JJ)-131072*(IL+2048))/32-2048
      IRA=IABS(IR)
      NREL=NREL+1
      IP=15*(IPH(JJ)-32*(IPH(JJ)/32))
      IFL=ISIGN(IPHAZ(ILA),IL)
      IFR=ISIGN(IPHAZ(IRA),IR)
      IARG=MOD(IFL+IFR+IP-IPHAZ(LL)+1440,180)
      IF (IARG.GT.90) IARG=180-IARG
      ISUMP3=ISUMP3+IARG
 1100 CONTINUE
 1500 CONTINUE
      AVER=REAL(ISUMP3)/REAL(NREL)
      RETURN
      END
C     ------------------------------------------------------------
C     Transfer the new phases and weights to CCP4.
      SUBROUTINE OUTPHI(PHIOUT,WTOUT,NUMREF,NUMDI,NSOUT)
      COMMON /BESET/NBEST(5),CBEST(5,4),IPBEST(2000,5),WTBEST(2000,5)
      REAL PHIOUT(NUMDI,NSOUT),WTOUT(NUMDI,NSOUT)
      DO 100 I=1, NUMREF
       DO 50 J=1, NSOUT
        PHIOUT(I,J) = REAL(IPBEST(I,J))
        IF (PHIOUT(I,J).GT.180.0) PHIOUT(I,J) = PHIOUT(I,J) - 360.0
        WTOUT(I,J) = WTBEST(I,J)
   50  CONTINUE
  100 CONTINUE
      RETURN
      END
