C
C     RSPS Real Space Patterson Search
C     Copyright (C) 1991-2000 Stefan Knight
C
C     This code is distributed under the terms and conditions of the
C     CCP4 Program Suite Licence Agreement as a CCP4 Application.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
************************************************************************
************************************************************************
************************************************************************
*                       Program RSPS, version 4.2
*                         
*
*
*      -----------------Real Space Patterson Search-----------------
*
*                  Copyright (C) 1991-2000 Stefan Knight
*                          All Rights Reserved
*
*
************************************************************************
************************************************************************
************************************************************************
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Updates, modifications and bugfixes
*-----------------------------------------------------------------------
*       First version (0.0) sep 1987 sdk
*
*       Feb 18 1988 - sdk
*       Introduced different levels of printing to help save paper.
*
*       March 1 1988 - sdk
*       First attempts with version 0.1
*
*       March 7 1988 - sdk
*       Keyword input
*       Extensive rearrangements of commons and subroutine arguments
*
*       March 13 1988 - sdk
*       Compute transformations in subroutine trnsf instead
*       of in main program
*       Search cell scan modularized (hscan,hvlist)
*       Vlist option added
*
*       March 21 1988 - sdk
*       Pick option added
*
*       March 29 1988 - sdk
*       Cross mode added
*
*       April 14 1988 - sdk
*       Getsets option added
*
*       October 14 1988 - sdk
*       New arguments in call to trnsf : sext, indet, vectyp
*
*       December 1988 - sdk
*       Min n scoring function added
*       Better selection of unique transformations
*       Local symmetry for cross scan added
*
*       October 1989 - sdk
*       Added three pick suboptions:
*       Pick level
*       Pick limits
*       Pick exclude
*
*       July 1 1991 - sdk 
*       Moved dimensioning parameters and common block data
*       structures to include files:
*          rspsdim.inc   - dimensions
*          rspssym.inc   - symmetry
*          rspsgrd.inc   - grids
*          rspsctl.inc   - flags, parameters (control varibles)
*       Standardization + cosmetic changes:
*       * Remove tabs
*       * Lower case code except in quotes
*       * Change 'C' comment indicator to '*'
*       * Change all continuation indicators to '&'
*       * Change 'real*4' to 'real'
*       * Change 'integer*4' to 'integer'
*       * Change 'logical*1' to 'logical'
*
*       July 4 1991 - sdk
*       All variables explicitly declared except in subroutines
*       getr0,getr1,getr2,getr5 and ccphdr
*
*       September 1991 - sdk
*       New simplified map reading routine. Statistics and scaling
*       factored out to separate routines
*
*       October 2 1991 - sdk
*       Removed packing of coordinates from PICK routines, s/r SORT
*       changed accordingly.
*
*-----------------------------------------------------------------------
*       November 1991 - february 1992 - sdk
*       Version 3.0
*       Major rewrite to give version 3.0:
*       Program structure changed, new data structures, a number of new
*       subroutines and functions introduced
*       Program now fully interactive
*       Internal representation of coordinates changed to fractional
*       Several new commands
*
*       March 12 1992 - sdk
*       Discovered and fixed bug in several concatenation statements:
*       passed length variables not allowed except in character constant
*       assignement.
*
*       March 26 1992 - sdk
*       Read and write positions in PDB format.
*
*       March 31 1992 - sdk
*       Space group specified with name as well as number. Subroutines
*       SYMMIN and TEST now obsolete; corresponding functions performed
*       in GETSGS and GETNCS; old routines removed.
*
*       June 17 1992 - sdk
*       Fixed bug giving out of bounds pointer to array PATMAP for some
*       points in some grids.
*
*       July 1992 - sdk
*       First (very slow) version of rigid body translation routine
*       written and incorporated.
*
*       August 10 1992 - sdk
*       Fixed bug in HVSCOR: for special positions with no Harker
*       vectors except the origin NVEC = 0 and the call to RSSCOR
*       must not be made or an adjustable array dimension error will
*       occur.
*
*       August 13 1992 - sdk
*       Space group specific reduction to asymmetric unit for
*       trigonal and hexagonal space groups added.
*
*       August 18 1992 - sdk
*       Rigid body translation rewritten to use pre-defined 
*       transformations (from s/r trnsf) - computation time down 
*       by a factor of 100. Tested OK. Command OLDXYZ renamed 
*       FIXXYZ. Command MODE is now used to select how the vectors 
*       are computed (HARKER, CROSS or TRANSLATE).
*       
*       August 20 1992 - sdk
*       Two new commands added: TOLERANCE and ROTATE. TRANSLATE
*       command renamed POLARSCAN.
*
*-----------------------------------------------------------------------
*       August-september 1992 - sdk
*       Version 3.2 completed.
*       Continuation input command lines allowed. New PATFILE, 
*       SCORFILE, PICK and SCAN commands with a number of old 
*       commands replaced by subcommands to these. DEFINE and 
*       GO commands removed. VLIST now always calls s/r VLIST
*       with predefined transformations. CROSS VLIST now gives
*       vectors between VLIST positions and FIXXYZ positions.
*       New GETPAT routine incorporates scaling, truncation and 
*       resetting of peaks. Several organizational modifications
*       to s/r RSCMND. New and faster PICK routine incorporated.
*
*       November 13 1992 - sdk
*       Changed Angstrom coordinates in printer output file from 
*       crystallographic to orthogonal.
*
*       November 17 1992 - sdk
*       Changed score statistics output to show stats for map instead
*       of for points measured. Automatic pick level assignment in s/r 
*       PICK changed to PLEVEL = RMIN + (RMAX-RMIN)/10.
*
*       January 27 1993 - sdk
*       Changed check of indeterminate axis to be done before scan
*       irrespective of mode.
*
*       February 4 1993 - sdk
*       Fixed macro reading bug that caused exit from macro after
*       execution of delayed action command.
*
*       February 5 1993 - sdk
*       Fixed check for homomorphous positions in space groups
*       with indeterminate axis so that it is properly? and
*       generally? handled for all cases.
*
*       February 23 1993 - sdk
*       Pick level and increment assignement modified (s/r PICK).
*       S/r REDUCE: Initial reduction to unit cell modified to
*       avoid sampling outside map volume due to rounding errors.
*
*       March 28 1993 - sdk
*       Changed site assignement AGAIN: february 5 fix was not working
*       as it should. Now site refers simply to positions with the
*       same set of Harker vectors.
*       Fixed s/r GETRHO to finally get rid of out-of-bounds errors for
*       vectors close to map edge.
*
*       April 13 1993 Pete Dunten
*       Fixed bug for Laue groups P6/mmm & Pm-3 
*
*-----------------------------------------------------------------------
*       April 26 1993 - sdk
*       Version 3.2.1
*       Call to WRNMSI on line 3060 fixed - was called with extra
*       argument. == CCP4 Version 3.2.1
*
*       August 17 1993 - sdk
*       Minor mods to STATUS command output: now also list statistics
*       for Patterson and score maps. Sym ops now listed as in Int.
*       Tables as well as in matrix form.
*
*-----------------------------------------------------------------------
*       November 16 1993 - sdk
*       Version 3.2.2
*       Fixed bug in call to ccpdpn in s/r rswpdb: error flag was
*       incorrectly tested after call leading to an incorrect write 
*       fail message. This is version 3.2.2 sent to CCP4 931116
*
*       November 21 1993 - sdk
*       A bug in the assignement of pick level for picking Patterson
*       map was fixed by removing assignement of pick level to rms
*       of (possibly scaled and truncated) Patterson map in s/r PIKPAT.
*
*       December 1 1993 - sdk
*       Added two LIST subcommands SYMOPS and VECTORS to list symmetry
*       operations and vector transformations.
*
*-----------------------------------------------------------------------
*       December 3 1993 - sdk
*       Version 3.2.3
*       New TRUNCATE argument to specify value for truncated high
*       densities: TRUNCATE from (to) (S).
*       Mode FIND to look for ncs axes parallel to crystallographic
*       axes. This is a primitive version of a FIND command that
*       will eventually allow you to look for the position of ncs 
*       axes one at a time at the same time incorporating information
*       from already positioned axes.
*
*       December 6 1993 - sdk
*       Determination of polar and indeterminate axis now separate in
*       s/r's INDAXS and POLAXS. GETTNF calls INDAXS to find out what
*       directions we need to scan. POLAXS returns polar axes 
*       directions from sgs.
*
*       December 9 1993 - sdk
*       Normalization of multiplicities. Low-count is now multiplied
*       by the multiplicity thus penalizing missing high-multiplicity
*       peaks.
*
*       December 10 1993 - sdk
*       In CROSS mode also generate and list Harker score when listing
*       positions.
*
*       December 20 1993 - sdk
*       Changed low-count again; now is only weighted if peaks are.
*
*       February 10 1994 - sdk
*       Changed specification of RESET coordinates from grid to 
*       fractional. Can also ask for origin to be reset using RESET
*       subcommand ORIGIN (will reset all origin + centering peaks). 
*       Changed specification of PICK LIMITS from grid to fractional.
*
*       February 14 1994 - sdk
*       Changed mode assignement from HARKER, CROSS, TRANSLATE,
*       FIND to six new modes. The new modes are partitioned in two
*       basic sets, ATOMS and MOLECULES that deal with finding
*       atom positions or positiones of molecules with ncs. The
*       new modes are
*       SINGLE ATOMS (old HARKER mode)
*       MORE ATOMS (old CROSS mode)
*       TRANSLATE ATOMS (old TRANSLATE mode)
*       SINGLE MOLECULES (old FIND mode)
*       MORE MOLECULES (new)
*       TRANSLATE MOLECULES (new)
*
*       February 15 1994 - sdk
*       Fixed bug in s/r PICKSEC that resulted in errouneous behaviour
*       when there was only one section in the map.
*
*       February 22 1994 - sdk
*       Action switches now properly reset after failure to execute.
*       PICK SCOREMAP command now determines mode used to generate
*       scoremap from scoremap title and resets the current mode
*       accordingly.
*
*       February 23 1994 - sdk
*       MORE MOLECULES mode debugged and tested.
*
*       February 28 1994 - sdk
*       Fixed broken POLARSCAN - symmetry was not applied to the
*       fixed positions prior to generating transformations.
*       TRANSLATE MOLECULES mode debugged and tested.
*
*       March 1 1994 - sdk
*       Changed recomputation of Patterson map statistics so that it is
*       only done if the map is rescaled; not after truncation/reset. 
*       S/N ratio now output in Patterson peak lists in addition to
*       absolute peak values. Patterson pick modified to pick stored 
*       Patterson instead of map file.
*
*       March 2 1994 - sdk
*       Patterson peak list is now cleared when a new Patterson map
*       is read.
*
*       March 3 1994 - sdk
*       Check for clashing molecules improved. Now looks at all
*       equivalent positions. Bump distance in MOLECULES modes
*       can now be user specified but is restricted to be larger
*       than a default (df_molbmp).
*       NCSROT command now recognizes the construct
*       NCSROT n-FOLD ALONG < X | Y | Z | l m n > (THROUGH t1 t2 t3)
*       where n must be one of 2, 3, 4, 6; X,Y, or Z specify the 
*       parallell axis, l,m,n are the direction cosines of
*       the n-fold axis and t1,t2,t3 specifies a point through
*       which the axis passes.
*
*       October 26 1994 - sdk
*       Fixed persisting bug in function GETRHO that sometimes gave
*       unpredictable results due to out-of-bounds errors in the
*       map array pointers. Also inserted test that a density value
*       has been taken from the map so as to be notified of any such
*       errors in the future.
*
*       February 22 1995 
*       Fixed an awful bug in s/r CONNECT resulting in GETSETS
*       missing solutions if all positions in two sets had the
*       same site numbers. Now determine equivalence by comparing
*       vector sets (logical function EQUSET). Also changed getsets 
*       data structure: Number of set members now in array SETSIZ;
*       pointers to members in array SETPOS.
*       Getests sets are now sorted on scores.
*       Cosmetic change to output: text changed from upper case to
*       lower case throughout for increased readability.
*       Map picking procedures now use Booth interpolation which makes
*       reported fit of predicted vectors to peak list slightly better.
*       Changed distance cutoff for peak search (VLIST) to 2.5 x
*       maximum grid spacing.
*
*       February 24 1995
*       Improved WRITE command: can now select to write positions, 
*       peaks, fixed positions, or sets.
*
*       March 1-6 1995 - sdk
*       Simplified s/r CONNECT interface; argument list reduced. 
*       Renamed nmem, cset, nset = setsiz,setpos,settot in common 
*       /sets/ in rspspos.inc
*       Score table now also lists overall score/position
*
*       March 10 1995 - sdk
*       Simple online help reinstalled
*
*-----------------------------------------------------------------------
*       October 31 1995 - sdk
*       Started work on version 4.0
*       Plans:
*       Delete fixxyz with selection - done
*       Simple vector algebraic operation - done
*
*       October 30 1995 - sdk
*       Changed nomenclature: modes now called "vector sets"; MODE 
*       command consequently changed to VECTORSET, mode parameter 
*       renamed vecset, df_mode renamed df_vecset
*
*       November 2 1995 - sdk
*       Added rudimentary add & subtract operations to add/subtract
*       vectors to/from fixxyz coordinates.
*
*       February - march 1996 - sdk
*       Added commands to force a specified vector:
*       SCAN REQUIRE PEAK n: only keeps positions generating
*       the specified peak.
*       Removed SAVE command - have never used it for years.
*       RSCMND code somewhat simplified by introduction of logical
*       functions TOPCMD, SUBCMD, NEXTWD for identifying commands, 
*       subcommands, words.
*       Improved PICK PATTERSON output: Harker peaks now flagged 'H'.
*       PICK PATTERSON HARKER command: pick Harker sections only.
*       "Matching index" a la LSQMAN now reported in VLIST option.
*       MySYMTRN was wrong for some translation components - fixed.
*       DELETE FIXXYZ with selection.
*
*       February 14 1997 - sdk
*       Fixed nasty bug for P1 in GETSETS
*
*       June 10 1997 - sdk
*       Harker plane equations were incorrect for non-orthogonal
*       space groups; fixed.
************************************************************************
        Program rsps

*       Data declarations and common blocks

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'
        include         'rspspos.inc'
        include         'rspsmap.inc'
        include         'rspsctl.inc'
        include         'rspsfil.inc'


*       Map title
        character*132   titin

*       Vector set label from scoremap title
        character*20    vstype

*       Run date
        character       date*8

*       Version label
        character       versn*6

*       Axis labels X,Y and Z
        character       label( 3 ) * 1

*       Getsets connect matrix
        logical         conmtx(mxpick,mxpick)

*       Extent, axis order, grid and limits of picked score map
        integer         pekext( 3 )
        integer         pekaxo( 3 )
        integer         pekgrd( 3 )
        integer         peklim( 6 )

*       Number of sections in scoremap
        integer         nsec2

*       Unit cell parameters from score map
        real            cellin( 6 )

*       Orthogonalization matrix
        real            toort( 3,3 )

*       Temporary coordinate store
        real            xut(3)

*       Scratch coordinate array
        real            xscr(3,mxpick)

*       Error flags
        integer         errcod

*       Maximum error count online
        integer         bomblevel
        parameter       ( bomblevel = 1000 )

*       Loop control variables
        integer         i, j

*       On/off switch for s/r timer
        integer         iflag

*       Extra score variable for hv-score
        real            hvsc

*       Scratch variables
        integer         idummy(10)
        real            rdummy(10)

*       External functions
        logical         ccponl,ldum
        integer         sfnblk
        integer         slnblk

*       Data statements
        data            versn      / '   4.2' /
        data            label      / 'X' , 'Y' , 'Z' /
        data            patfil     / 'patterson' /
        data            scrfil     / 'rsps.map' /
        data            xinfil     / 'rsps.pdb' /
        data            xutfil     / 'positions.pdb' /
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

        title = ' '
        titin = ' '
*       Process command line arguments
        call ccpfyp

*       Set error handling flag
        stop_on_error = .false.
Cdoesn't work on alphas        if ( .not. ccponl(ldum) ) stop_on_error = .true.
*       Initiate error count and set bomblevel
        call errint ( bomblevel )

*       Initiate variables, set defaults
        call rsinit ( 'ALL' )

*       Write header with program name, version number and run date
*       Report some current array limits
        call ccpdat ( date )
        write(6,2000)versn,date
        call ttyptl ( ' RSPS >> Current limitations' )
        call ttypti ( ' RSPS >> Max number of points in Patterson ' //
     &  'map =     ' , mxpatp , 8 , errcod )
        call ttypti ( ' RSPS >> Max number of points/section in ' //
     &  'input map = ' , mxisct , 8 , errcod )
        call ttypti ( ' RSPS >> Max number of peaks for PICK and ' //
     &  'GETSETS =  ' , mxpick , 8 , errcod )
        call ttypti ( ' RSPS >> Max number of members of GETSETS ' //
     &  'sets =     ' , mxgmem , 8 , errcod )
        call ttypti ( ' RSPS >> Max number of GESETS sets =      ' //
     &  '           ' , mxgset , 8 , errcod )
        call ttypti ( ' RSPS >> Max number of SGS operations =   ' //
     &  '           ' , mxsgs , 8 , errcod )
        call ttypti ( ' RSPS >> Max number of NCS operations =   ' //
     &  '           ' , mxncs , 8 , errcod )
        call ttypti ( ' RSPS >> Max number of combined symops =  ' //
     &  '           ' , mxsym , 8 , errcod )


*-----------------------------------------------------------------------
*       Read and interpret commands
 10     call rscmnd

*-----------------------------------------------------------------------
*       Get out ?
        if ( exit ) then
          write(6,5000)
          goto 9999
        endif

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Do whatever.
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*-----------------------------------------------------------------------
*=======search scan=====================================================
*-----------------------------------------------------------------------
        if ( doscan ) then
          iflag = 0
          call timer ( iflag , ' ' )
          if ( vecset .eq. 'SINGLE ATOMS' ) then
            call ttycr ( 1 )
            call ttyptl ( '   RSPS SINGLE ATOM SCAN >>' )
          else if ( vecset .eq. 'MORE ATOMS' ) then
            call ttycr ( 1 )
            call ttyptl ( '   RSPS MORE ATOMS SCAN >>' )
            write(6,2330)nxyz2
            do i = 1 , nxyz2
              write(6,2335) i,( xyz2(j,i),j=1,3 )
            end do
          else if ( vecset .eq. 'TRANSLATE ATOMS' ) then
            call ttycr ( 1 )
            call ttyptl ( '   RSPS TRANSLATE ATOMS SCAN >>' )
            write(6,2330)nxyz2
            do i = 1 , nxyz2
              write(6,2335) i,( xyz2(j,i),j=1,3 )
            end do
          else if ( vecset .eq. 'SINGLE MOLECULES' ) then
            call ttycr ( 1 )
            call ttyptl ( '   RSPS SINGLE MOLECULE SCAN >>' )
          else if ( vecset .eq. 'MORE MOLECULES' ) then
            call ttycr ( 1 )
            call ttyptl ( '   RSPS MORE MOLECULES SCAN >>' )
            write(6,2330)nxyz2
            do i = 1 , nxyz2
              write(6,2335) i,( xyz2(j,i),j=1,3 )
            end do
          else if ( vecset .eq. 'TRANSLATE MOLECULES' ) then
            call ttycr ( 1 )
            call ttyptl ( '   RSPS TRANSLATE MOLECULES SCAN >>' )
            write(6,2330)nxyz2
            do i = 1 , nxyz2
              write(6,2335) i,( xyz2(j,i),j=1,3 )
            end do
          end if

          if ( iprt .gt. 0 ) then
            write(6,2340)titut
            write(6,2350)scrgrd
            write(6,2360) ( nint ( scrlim( 2*i-1 ) * scrgrd( i ) ) ,
     &      nint ( scrlim( 2*i ) * scrgrd( i ) ) , i = 1 , 3 )
            write(6,2370)label(scraxo(3)),label(scraxo(1)),
     &      label(scraxo(2))
            if ( special ) write(6,2380)
            write(6,2390)reject,rej,maxlow,reject
            if ( index ( vecset , 'SINGLE' ) .ne. 0 ) write(6,2400)bump
            if ( index ( vecset , 'MORE' ) .ne. 0 ) write(6,2401)bump
            if ( index ( vecset , 'TRANSLATE' ) .ne. 0 ) 
     &      write(6,2402)bump
            if ( usemult ) then
              if ( scrtyp .eq. 1 ) then
                write(6,2411)
                write(6,2420)
              else if ( scrtyp .eq. 2 ) then
                write(6,2412)
                write(6,2420)
              else if ( scrtyp .eq. 3 ) then
                write(6,2413)
                write(6,2420)
              endif
            else
              if ( scrtyp .eq. 1 ) then
                write(6,2421)
                write(6,2430)
              else if ( scrtyp .eq. 2 ) then
                write(6,2422)
                write(6,2430)
              else if ( scrtyp .eq. 3 ) then
                write(6,2423)
                write(6,2430)
              endif
            endif
            if ( minfun ) write(6,2440)nuse
          endif

          call sscan ( lunsco , patmap , patfas , patslo , patsec ,
     &    patsig , scomin , scomax , scoave , scosig , errcod )

          iflag = 1
          call timer ( iflag , 'SCAN' )
          doscan = .false.
          goto 10
        endif


*-----------------------------------------------------------------------
*=======vlist===========================================================
*-----------------------------------------------------------------------
        if ( dovlis ) then
          iflag = 0
          call timer ( iflag , ' ' )
          if ( vecset .eq. 'SINGLE ATOMS' ) then
            call ttycr ( 1 )
            call ttyptl ( '   RSPS SINGLE ATOMS VLIST >>' )
          else if ( vecset .eq. 'MORE ATOMS' ) then
            call ttycr ( 1 )
            call ttyptl ( '   RSPS MORE ATOMS VLIST >>' )
          else if ( vecset .eq. 'TRANSLATE ATOMS' ) then
            call ttycr ( 1 )
            call ttyptl ( '   RSPS TRANSLATE ATOMS VLIST >>' )
          else if ( vecset .eq. 'SINGLE MOLECULES' ) then
            call ttycr ( 1 )
            call ttyptl ( '   RSPS SINGLE MOLECULES VLIST >>' )
          else if ( vecset .eq. 'MORE MOLECULES' ) then
            call ttycr ( 1 )
            call ttyptl ( '   RSPS MORE MOLECULES VLIST >>' )
          else if ( vecset .eq. 'TRANSLATE MOLECULES' ) then
            call ttycr ( 1 )
            call ttyptl ( '   RSPS TRANSLATE MOLECULES VLIST >>' )
          endif

          if ( iprt .gt. 0 ) then
            write(6,2460)reject,rej
            if ( usemult ) then
              if ( scrtyp .eq. 1 ) then
                write(6,2411)
                write(6,2420)
              else if ( scrtyp .eq. 2 ) then
                write(6,2412)
                write(6,2420)
              else if ( scrtyp .eq. 3 ) then
                write(6,2413)
                write(6,2420)
              endif
            else
              if ( scrtyp .eq. 1 ) then
                write(6,2421)
                write(6,2430)
              else if ( scrtyp .eq. 2 ) then
                write(6,2422)
                write(6,2430)
              else if ( scrtyp .eq. 3 ) then
                write(6,2423)
                write(6,2430)
              endif
            endif
            if ( minfun ) write(6,2440)nuse
          endif


          do i = 1 , nxyz1
            call vlist ( patmap , patfas , patslo , patsec , 
     &      ppnum , patsig , xyz1(1,i) , errcod )
            if ( errcod .ne. 0 ) then
              call errmsg ( 'Error in VLIST' , 0 )
            endif
          end do

          iflag=1
          call timer ( iflag , 'VLIST' )

          dovlis = .false.
          goto 10
        endif


*-----------------------------------------------------------------------
*=======polarscan=======================================================
*-----------------------------------------------------------------------
        if ( dotran ) then
          iflag = 0
          call timer ( iflag , ' ' )
          call ttycr ( 1 )
          call ttyptl ( '   RSPS POLARSCAN >>' )
          write(6,2330)nxyz2
          do i = 1 , nxyz2
            write(6,2335) i,( xyz2(j,i),j=1,3 )
          end do
          write(6,2450)ntrnps,label(scndir)
          if ( iprt .gt. 0 ) then
            write(6,2390)reject,rej,maxlow,reject
            if ( usemult ) then
              if ( scrtyp .eq. 1 ) then
                write(6,2411)
                write(6,2420)
              else if ( scrtyp .eq. 2 ) then
                write(6,2412)
                write(6,2420)
              else if ( scrtyp .eq. 3 ) then
                write(6,2413)
                write(6,2420)
              endif
            else
              if ( scrtyp .eq. 1 ) then
                write(6,2421)
                write(6,2430)
              else if ( scrtyp .eq. 2 ) then
                write(6,2422)
                write(6,2430)
              else if ( scrtyp .eq. 3 ) then
                write(6,2423)
                write(6,2430)
              endif
            endif
            if ( minfun ) write(6,2440)nuse
          endif

          call xtran ( errcod )
          if ( errcod .ne. 0 ) then
            if ( stop_on_error ) goto 9999
            goto 10
          endif
          iflag = 1
          call timer ( iflag , 'POLARSCAN' )
          dotran = .false.
        endif


*-----------------------------------------------------------------------
*=======pick patterson==================================================
*-----------------------------------------------------------------------
        if ( dopipa ) then
          iflag = 0
          call timer ( iflag , ' ' )
          call ttycr ( 1 )
          call ttyptl ( '   RSPS PICK >>' )
          call pikpat ( errcod )
          if ( errcod .ne. 0 ) then
            if ( stop_on_error ) goto 9999
            goto 10
          endif
          iflag = 1
          call timer ( iflag , 'PICK' )
          dopipa = .false.
        endif


*-----------------------------------------------------------------------
*=======pick scoremap===================================================
*-----------------------------------------------------------------------
        if( dopisc ) then
          iflag = 0
          call timer ( iflag , ' ' )
          call ttycr ( 1 )
          call ttyptl ( '   RSPS PICK >>' )

*       Get map parameters
          call rdhdr ( 'CCP4' , lunsco , scrfil , nsec2 , pekext , 
     &    pekaxo , pekgrd , peklim , cellin , titin , errcod )
          call mrclos ( lunsco )
          if ( errcod .ne. 0 ) then
            if ( stop_on_error ) goto 9999
            goto 10
          endif
          title(1:80) = titin(1:80)
          i = index ( title , '>' )
          if ( i .eq. 0 ) then
            i = slnblk ( title )
            write(6,2530)npick,title(sfnblk(title):i)
          else
            write(6,2535)npick,title(sfnblk(title):i),
     &      title(i+2:slnblk(title))
          endif

*       Figure out type of scoremap
          if ( title(1:6) .ne. ' RSPS ' ) then
            call errmsg ( 'Illegal header > ' // title(1:6) , 0 )
            call errmsg ( 'This is not an RSPS scorefile' , 0 )
            if ( stop_on_error ) goto 9999
            goto 10
          else if ( index ( title , 'SINGLE ATOMS' ) .ne. 0 ) then
            vstype = 'SINGLE ATOMS'
          else if ( index ( title , 'MORE ATOMS' ) .ne. 0 ) then
            vstype = 'MORE ATOMS'
          else if ( index ( title , 'TRANSLATE ATOMS' ) .ne. 0 ) then
            vstype = 'TRANSLATE ATOMS'
          else if ( index ( title , 'SINGLE MOLECULES' ) .ne. 0 ) then
            vstype = 'SINGLE MOLECULES'
          else if ( index ( title , 'MORE MOLECULES' ) .ne. 0 ) then
            vstype = 'MORE MOLECULES'
          else if ( index ( title , 'TRANSLATE MOLECULES' ) .ne. 
     &    0 ) then
            vstype = 'TRANSLATE MOLECULES'
          endif
          write(6,2540)vstype
          if ( vecset .ne. vstype ) write(6,2545)
          vecset = vstype


*       If no Patterson has been read copy cellin to patgrd,
*       and get cos of cell angles.
*       This info is needed in s/r equsit to find equivalent
*       positions
*       Also copy cellin into cell
          if ( patnotin ) then
            call fveci ( cellin , patgrd , 3 )
            call cosabg ( cellin( 4 ) , cosang )
            call matcop ( cellin , cell   , 6 , 1 )
          endif

*       Pick. Peaks are returned as fractional coordinates.
          call pickfile ( 'CCP4',scrfil,lunsco,npick,pick_level,
     &    pkltfr,nxcl,xclpnt,pkxcld,xpick,value,errcod )
          if ( errcod .ne. 0 ) then
            if ( stop_on_error ) goto 9999
            goto 10
          endif

*       Total number of positions stored
          xintot = npick

          if ( npick .ne. 0 ) then

*       Find and flag equivalent positions (these have the same
*       set of Harker vectors)
            call equsit ( npick , xpick , sittot , sitbeg , sitend ,
     &      sitpnt , errcod )

*       Set matrix for converting fractional crystallographic
*       coordinates to Angstrom orthogonal in PDB standard frame
            call ortho ( cellin , 1 , 'TOORT' , toort , errcod )
            if ( errcod .ne. 0 ) then
              call errmsg ( 'Failed getting orthogonalization matrix' , 
     &        0 )
              if ( stop_on_error ) goto 9999
              goto 10
            endif

*       Write out result
            if ( vecset .eq. 'MORE ATOMS' .and. patisin ) then
              write(6,2560)npick
            else
              write(6,2565)npick
            endif   
            do 50 i = 1 , npick
              call vmtply ( toort , xpick( 1,i ) , xut )
              if ( vecset .eq. 'MORE ATOMS' .and. patisin ) then
*       When using MORE ATOMS vector set also get Harker vector score
                call hvscor ( patmap , patfas , patslo , patsec , 
     &          patsig , r_sym , t_sym , nopsym , xpick( 1,i ) , 
     &          hvsc , idummy(1) , idummy(2) , idummy(3) , 
     &          rdummy(1) , rdummy(2) , errcod )
                if ( errcod .ne. 0 ) then
                  call errmsg ( 'Error computing Harker score' , 0 )
                  write(6,2585) i , ( xpick( j,i ) , j = 1 , 3 )  ,
     &            xut , value( i ) , sitpnt( i )
                  goto 50
                endif
                write(6,2580) i , ( xpick( j,i ) , j = 1 , 3 )  ,
     &          xut , value( i ) , hvsc , sitpnt( i )
              else
                write(6,2585) i , ( xpick( j,i ) , j = 1 , 3 )  ,
     &          xut , value( i ) , sitpnt( i )
              endif
 50         continue

          else

            write(6,2590)

          endif

          iflag = 1
          call timer ( iflag , 'PICK' )
          dopisc = .false.
          goto 10
        endif


*-----------------------------------------------------------------------
*=======getsets=========================================================
*-----------------------------------------------------------------------
        if ( dogets ) then
          call ttycr ( 1 )
          call ttyptl ( '   RSPS GETSETS >>' )
          write(6,2650)npguse

          if ( iprt .gt. 0 ) then
            write(6,2395)reject,rej,maxlow,reject
            write(6,2610)bump
            if ( usemult ) then
              if ( scrtyp .eq. 1 ) then
                write(6,2411)
                write(6,2420)
              else if ( scrtyp .eq. 2 ) then
                write(6,2412)
                write(6,2420)
              else if ( scrtyp .eq. 3 ) then
                write(6,2413)
                write(6,2420)
              endif
            else
              if ( scrtyp .eq. 1 ) then
                write(6,2421)
                write(6,2430)
              else if ( scrtyp .eq. 2 ) then
                write(6,2422)
                write(6,2430)
              else if ( scrtyp .eq. 3 ) then
                write(6,2423)
                write(6,2430)
              endif
            endif
            if ( minfun ) write(6,2440)nuse
          endif

          if ( npguse .ne. 0 ) then

*       Set up connect matrix
            iflag = 0
            call timer ( iflag , ' ' )
            call setcon ( patmap,patfas,patslo,patsec,r_sym,t_sym,
     &      nopsym,xpick,sitpnt,npguse,polar,conmtx,errcod )
            iflag = 1
            call timer ( iflag , 'SETCON' )

*       Extract connected sets
            iflag = 0
            call timer ( iflag , ' ' )
            settot = 0
            call connect ( conmtx,npguse,minset )
            iflag = 1
            call timer ( iflag , 'CONNECT' )

            if(settot.eq.0)then
              write(6,2660)
            else
*       Write score table for each set
              write(6,2665)settot
              do 800 i = 1 , settot
                do j = 1 , setsiz(i)
                  call rmatcop ( xpick(1,setpos(j,i)) , xscr(1,j) , 
     &            3 , 1 )
                end do
                call scrtab ( i , setsiz(i) , setpos(1,i) , xscr , 1 , 
     &          errcod )
 800          continue
            endif

          endif
          dogets = .false.
        endif


*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        goto 10
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements


*       Output formats

 2000   format(/20x,'*************************************************',
     &  '*************',/20x,'*******',22x,'RSPS',22x,'*******',/20x,
     &  '*******  REAL SPACE PATTERSON SEARCH - VERSION ',a6,
     &  '  *******',/20x,'*******',48x,'*******',/20x,'*******',17x,
     &  'Stefan Knight',18x,'*******',/20x,'*******',11x,'Dept. of ',
     &  'Molecular Biology',11x,'*******',/20x,'*******',5x,'Swedish ',
     &  'Univ. of Agricultural Sciences',5x,'*******',/20x,'*******',
     &  16x,'Uppsala, Sweden',17x,'*******',/20x,'*******************',
     &  '*******************************************',//,47x,a8,///)
 2030   format(/10x,'Rescaling >> density = ',f10.3,' * density + ',
     &  f10.1)
 2040   format(/10x,'Density statistics on rescaled patterson map:',
     &         /10x,'Minimum density    =',f12.4,
     &         /10x,'Maximum density    =',f12.4,
     &         /10x,'Average density    =',f12.4,
     &         /10x,'Std Dev of density =',f12.4)
 2330   format(/10x,'Number of fixed positions = ',i3)
 2335   format(10x,'Position ',i3,': ',3f8.4)
 2340   format(///10x,'The output map will have the title: ',5x,a80)
 2350   format(/10x,'Search grid along X,Y,Z (grid 2): ',3i8)
 2360   format(/10x,'The output map requested includes grid points ',
     &  3(//30x,'from',i5,' to',i5),//20x,' along the search axes')
 2370   format(/10x,'The output map will have ',a1,
     &  '-sections with ',a1,' varying most rapidly and ',a1,
     &  ' most slowly')
 2380   format(/10x,'Special positions will be included in the search')
 2390   format(/10x,'Minimum accepted vector density = ',f8.2,
     &  ' (',f5.1,' sigma above the mean)',/,
     &  10x,'A maximum of',i5,' vectors with density smaller than ',
     &  f8.2,' are allowed')
 2395   format(/10x,'Minimum accepted vector density = ',f8.2,
     &  ' (',f5.1,' sigma above the mean)',/,
     &  10x,'For each pair of positions, a maximum of',i5,
     &  ' cross vectors with density smaller than ',f8.2,
     &  ' are allowed')
 2400   format(/10x,'Minimum distance between symmetry ',
     &  'related positions = ',f8.2,' Angstrom')
 2401   format(/10x,'Minimum distance between a fixed',
     &  ' and a new position = ',f8.2,' Angstrom')
 2402   format(/10x,'Minimum distance between positions = ',f8.2,
     &  ' Angstrom')
 2411   format(/10x,'Scores are computed as Sum(Weight*Rho/Sigma)',
     &  '/Nvec where')
 2412   format(/10x,'Scores are computed as Product(Weight*Rho/Sigma)',
     &  '/Nvec where')
 2413   format(/10x,'Scores are computed as ',
     &  '1./Sum(1./(Weight*Rho/Sigma))/Nvec where')
 2420   format(/15x,'Weight is the multiplicity of the vector',/,
     &  15x,'Rho    is the density at a vector position',/,
     &  15x,'Sigma  is the rms deviation from the mean of the map',/,
     &  15x,'Nvec   is the number of vectors contributing to the sum')
 2421   format(/10x,'Scores are computed as Sum(Rho/Sigma)',
     &  '/Nvec where')
 2422   format(/10x,'Scores are computed as Product(Rho/Sigma)',
     &  '/Nvec where')
 2423   format(/10x,'Scores are computed as ',
     &  '1./Sum(1./(Rho/Sigma))/Nvec where')
 2430   format(/15x,'Rho    is the density at a vector position',/,
     &  15x,'Sigma  is the rms deviation from the mean of the map',/,
     &  15x,'Nvec   is the number of vectors contributing to the sum')
 2440   format(/10x,'The minimum N = ',i8,' vector densities ',/,
     &  'will be used')
 2450   format(/10x,'Number of positions to translate = ',i3,/10x,
     &  'Scan direction = ',a)
 2460   format(/10x,'Vectors with density less than ',f8.2,' (',f5.1,
     &  ' sigma above mean) are counted as low')
 2530   format(10x,'PICK SCOREMAP >>',/10x,'The',i5,' highest peaks ',
     &  'will be selected from score map with title:',/10x,a)
 2535   format(10x,'PICK SCOREMAP >>',/10x,'The',i5,' highest peaks ',
     &  'will be selected from score map with title:',/10x,a,/10x,a)
 2540   format(10x,'Type of scoremap: ',a,/)
 2545   format(10x,'Vectorset has been reset to match type of ',
     &  'score map' )
 2550   format(//10x,'No density on score map; could not pick',/10x,
     &  'Peak picking aborted')
 2560   format(10x,'PICK >> ',i5,' peaks found; these are ',
     &  'listed in descending order',//12x,'PosnN     ',
     &  ' Fractional coordinates     Angstrom coordinates',
     &  '     CVScore  HVScore   Site',/12x,'-----     ',
     &  '------------------------  ------------------------   ',
     &  '-------  -------   ----')
 2565   format(/10x,'PICK >> ',i5,' peaks found; these are ',
     &  'listed in descending order',//12x,'PosnN     ',
     &  ' Fractional coordinates     Angstrom coordinates',
     &  '      Score    Site',/12x,'-----     ',
     &  '------------------------  ------------------------  ',
     &  '---------  ----')
 2580   format(10x,i5,6x,3f8.4,2x,3f8.2,2x,f8.2,1x,f8.2,3x,i4)
 2585   format(10x,i5,6x,3f8.4,2x,3f8.2,2x,f8.2,4x,i4)
 2590   format(/10x,'PICK >> no peaks found')
 2610   format(/10x,'Minimum distance between positions = ',f8.2,
     &  ' Angstrom')
 2650   format(/10x,'Getsets will use ',i4,' stored positions' )
 2655   format(//10x,'The following ',i4,' symmetry operations',
     &  ' will be used: ',/)
 2660   format(/10x,'GETSETS >> no sets found')
 2665   format(/10x,'GETSETS >> ',i3,' sets found')

*       Error messages
 9300   format(/10x,'** ERROR - Too many symmetry operations generated',
     &  ', max = ',i5,' found = ',i5,' **')

*       hejhej
 5000   format(/28x,'****************Wat er Woffla****************')
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

 9999   continue
        close ( lunpat )
        close ( lunsco )
        close ( lunsgs )
        close ( lunncs )
        close ( lunxin )
        close ( lunxut )
        close ( lunmac )
        end
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
************************************************************************
*********************************************************************sdk
************************************************************************
*       Subroutines
************************************************************************
************************************************************************
*
*       Package RSPSIO
*       Routines for terminal i/o and command interpretation in RSPS
*
*       Contains:
*       s/r rscmnd
*       s/r rcomnt
*       s/r rsinit
*       s/r keyin
************************************************************************
************************************************************************
*       Subroutine RSCMND
*
*       Function:
*       Read and interpret commands for rsps
*
*       Comments:
*       Uses separately compiled packages/procedures in package FORLIB
*       (copyright (c) Per Kraulis)
*
*       Arguments:
*       None
*
*       History:
*       February 24 1995 - sdk
*       WRITEX and READX commands renamed WRITE and READ
*       WRITE command now accepts arguments POSITIONS, PEAKS,
*       FIXXYZ, and SET to write out corresponding coordinates
*       February 14 1994 - sdk
*       New MODEs
*       September 5 1992 - sdk
*       RSCMND for RSPS version 3.2 now functional
*       December 1991 - sdk
*       RSCMND for RSPS version 3.0 now functional
*       March 1990 - sdk
*       First attempts on interactive version
*       March 1988 first version S. Knight
************************************************************************
        subroutine rscmnd

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'
        include         'rspsmap.inc'
        include         'rspsgrd.inc'
        include         'rspspos.inc'
        include         'rspssym.inc'
        include         'rspsfil.inc'
        include         'rspsdef.inc'



*       Local variables
        integer         i , j , k , l , ict , indx
        integer         ib , ie , ipi , ipf , iri , irf
        integer         iuvw
        integer         imlt
        integer         nmem , imem
        integer         usrset( mxgmem )
        integer         istop
        integer         errcod
        integer         ifail
        integer         inext
        integer         iposarg
        integer         irotarg
        integer         iputarg
        integer         nposarg
        integer         nrotarg
        integer         iput
        integer         npos
        integer         nrot
        integer         nfold
        integer         idummy( 10 )
        integer         iset
        integer         nx
        real            oldscl( 2 )
        real            toort( 3,3 ) , deort( 3,3 )
        real            rtmp( 3,3 ) , ttmp( 3 )
        real            xtmp1( 3 ) , xtmp2( 3 ) , xtmp3( 3 )
        real            tpos( 3 )
        real            xut( 3 )
        real            rotang , polang( 3 ) , dircos( 3 )
        real            hvsc
        real            rdummy( 10 )
        real            xscr(3,mxpick) , vscr(mxpick)
        integer         pscr(mxpick)
        character       xlabel(3)*1 , secfl*3 , chr*5
        character       item*12
        character       vecnam(3)*6
        character*3     output
        character*3     possel , rotsel
        character*8     scrlab(3)
        character*40    symchr
        character*80    macfil
        character*80    hlpfil
        character*132   junkln
        character*132   tit
        character*1     symbol( 2 )
        logical         macro
        logical         echo

*       Command line read/parse variables
        integer         lstchr
        logical         newlin
        integer         maxword
*       If maxword is changed it must be changed in fn nextwd also 
        parameter       ( maxword = 40 )
        integer         ibeg( maxword ) , iend( maxword )
        integer         dtyp( maxword ) , idec( maxword )
        real            fvalue( maxword )
        character*4     cvalue( maxword )
        character*132   linein
        character*500   line


*       ==command words==
*       NTOK    number of tokens (words) on command line
*       MAXCMND is maximum number of commands on each line
*       CMND    command words from the latest line
*       CMNDPOS oridnal number for each command word on a line
*       CMNDNUM position in CMNDLST for each command word on a line
*       CMNDCNT number of commands found
*       NUMARG  for each command, the number of arguments
*       ICMND   command interpreter loop variable
*       ICUR    Pointer to next command
        integer         ntok 
        integer         maxcmnd
        parameter       ( maxcmnd = 20 )
        character*12    cmnd( maxcmnd ) , subcmnd( maxcmnd )
        integer         cmndpos( maxcmnd ) , subcmndpos( maxcmnd )
        integer         cmndnum( maxcmnd ) , subcmndnum( maxcmnd )
        integer         cmndcnt , subcmndcnt
        integer         numarg , numsubarg
        integer         icmnd , jcmnd
        integer         icur

*       CMNDTOT is number of command words
*       CMNDLST contains the list of valid commands
*       CMNDIN is set .true. for an executed command
        integer         cmndtot
        parameter       ( cmndtot = 34 )
        character*12    cmndlst( cmndtot )
        logical         cmndin( cmndtot )
        data            cmndlst/
     &                  '@'           ,
     &                  'ADD'         ,
     &                  'BUMP'        ,
     &                  'CELL'        ,
     &                  'DELETE'      ,
     &                  'EXIT'        ,
     &                  'FIXXYZ'      ,
     &                  'GETSETS'     ,
     &                  'HELP'        ,
     &                  'LIST'        ,
     &                  'LOW'         ,
     &                  'NCSROT'      ,
     &                  'NCSYMM'      ,
     &                  'PATFILE'     ,
     &                  'PICK'        ,
     &                  'POLARSCAN'   ,
     &                  'PRINT'       ,
     &                  'QUIT'        ,
     &                  'READ'        ,
     &                  'REJECT'      ,
     &                  'ROTATE'      ,
     &                  'SCAN'        ,
     &                  'SCORE'       ,
     &                  'SCORFILE'    ,
     &                  'SGSYMM'      ,
     &                  'SPACEGROUP'  ,
     &                  'STATUS'      ,
     &                  'SUBTRACT'    ,
     &                  'TABLE'       ,
     &                  'TOLERANCE'   ,
     &                  'VECTORSET'   ,
     &                  'VLIST'       ,
     &                  'WEIGHT'      ,
     &                  'WRITE'       /


*       Subcommands to PATFILE command
        integer         patcmndtot
        parameter       ( patcmndtot = 4 )
        character*12    patcmndlst( patcmndtot )
        data            patcmndlst/
     &                  'RESET'       ,
     &                  'SCALE'       ,
     &                  'TRUNCATE'    ,
     &                  'TYPE'        /

*       Subcommands to SCORFILE command
        integer         scrcmndtot
        parameter       ( scrcmndtot = 2 )
        character*12    scrcmndlst( scrcmndtot )
        data            scrcmndlst/
     &                  'SECT'        ,
     &                  'TITLE'       /

*       Subcommands to PICK command
        integer         pikcmndtot
        parameter       ( pikcmndtot = 5 )
        character*12    pikcmndlst( pikcmndtot )
        data            pikcmndlst/
     &                  'EXCLUDE'     ,
     &                  'LEVEL'       ,
     &                  'LIMITS'      ,
     &                  'PATTERSON'   ,
     &                  'SCOREMAP'    /


*       Subcommands to SCAN command
        integer         scncmndtot
        parameter       ( scncmndtot = 4 )
        character*12    scncmndlst( scncmndtot )
        data            scncmndlst/
     &                  'AU'          ,
     &                  'GRID'        ,
     &                  'LIMITS'      ,
     &                  'REQUIRE'     /


*       External functions
        integer         sfnblk
        integer         slnblk
        integer         keynm
        logical         ccponl,ldum
        logical         ccpexs
        logical         nextwd
        logical         topcmd
        logical         subcmd

*       Data statements
        data            oldscl      / 1.0 , 0.0 /
        data            scrlab      / 'SUM' , 'PRODUCT' , 'HARMONIC' /
        data            secfl       / 'XYZ' /
        data            xlabel      / 'X' , 'Y' , 'Z' /
        data            vecnam      / 'Harker' , 'Cross ' , 'Any   ' /
        data            item        / ' ' /
        data            echo        / .false. /
        data            macro       / .false. /
        data            newlin      / .true. /
        data            imlt        / 1 /
        data            symbol      / 'H' , ' ' /
        data            icur        / 0 /

*       Data in common for functions nextwd, topcmd, subcmd 
        common /cmddel/ inext , ibeg , iend
        common /cmdlin/ line
        common /cmdnum/ icmnd , jcmnd
        common /cmdwds/ cmnd , subcmnd

*       Save statements
        save            macro
        save            dtyp
        save            ntok 
        save            cmndnum
        save            cmndpos
        save            cmndcnt
        save            icur
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Reset some variables to user-defined values
        special = usrspc
        usencs  = usrncs
        bump    = usrbmp

*       If there are remaining commands (this may happen after
*       carrying out "delayed-action commands" then re-enter command
*       interpetation loop
        if ( icur .gt. 1 .and. icur .le. cmndcnt ) goto 55

*
*       Initiations
        call rsinit ( 'ACTIONS' )
        dftgrd = .true.
*        if ( .not. ccponl(ldum) ) echo = .true.       
*       Set integer error control flag
        istop = 0
        if (stop_on_error) istop = 1
*       Initiate pointer to next command
        icur = 1
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*
*
*       Start of command line read loop
*
*
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Init error code, output prompt, get command line
 50     errcod = 0
        if ( .not. macro ) then

          if ( ccponl(ldum) ) then
*       Prompt for commands 
            call ttycr ( 1 )
            call ttygtl ( ' RSPS > ', linein , errcod )
          else
*       Read commands from standard input without prompting
            read(5,'(a)',iostat=errcod)linein
            if ( errcod .ne. 0 ) then
              linein = 'EXIT'
              errcod = 0
            endif
          endif  

          if ( errcod .eq. 2 ) then
*       End of file; insert EXIT command in empty line
            linein = 'EXIT'
          else if ( errcod.ne.0 ) then
*       Could not read line; abort
            call errmsg ( 'Error reading command line' , 1 )
          endif

        else 

*       Read commands from macro file
          read ( lunmac , '(a)' , iostat = errcod ) linein
          if ( errcod .lt. 0 ) then
*       End-of-file; switch of macro flag, close macro file, loop
*       back to read from standard input
            macro = .false.
            close ( lunmac )
            goto 50
          else if ( errcod .ne. 0 ) then
*       Could not read line; abort
            call errmsg ( 'Error reading command line' , 1 )
          endif

        endif

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Find, reflect and remove comments
        call rcomnt ( linein )

*       If nothing left on line, loop back
        if ( linein .eq. ' ' ) goto 50

*       Question ?
        if ( linein(sfnblk(linein):sfnblk(linein)) .eq. '?' ) then
          call ttyptl ( ' Valid commands :' )
          write ( 6 , '( 4 ( 2x , a ) )' ) cmndlst
          call ttycr ( 1 )
          call ttyptl ( ' For help on a particular ' //
     &    'command enter HELP (command)' )
          goto 50
        endif

*       Look for continuation mark
        lstchr = slnblk ( linein )
        if ( linein( lstchr:lstchr ) .eq. '-' .or.
     &  linein( lstchr:lstchr ) .eq. '&' ) then
          if ( lstchr .le. 1 ) goto 50
          if ( newlin ) then
            line = linein( :lstchr-1 )
            newlin = .false.
          else
            if ( slnblk ( line ) + lstchr .gt. len ( line ) ) then
              call errmsg ( 'Input line too long' , istop )
              newlin = .true.
              goto 50
            endif
            line = line( :slnblk ( line ) ) // 
     &      ' ' // linein( :lstchr-1 )
          endif
          goto 50
        else
          if ( newlin ) then
            line = linein( :lstchr )
          else
            if ( slnblk ( line ) + lstchr + 1 .gt. len ( line ) ) then
              call errmsg ( 'Input line too long' , istop )
              newlin = .true.
              goto 50
            endif
            line = line( :slnblk ( line ) ) // 
     &      ' ' // linein( :lstchr )
            newlin = .true.
          endif
        endif

*       Echo input line
        if ( echo ) call ttyptl ( ' RSPS > ' // line( :
     &  slnblk ( line ) ) )

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Lexical analysis
        ntok = - maxword
        call parse ( line , ibeg , iend , dtyp , 
     &  fvalue , cvalue , idec , ntok )
        if ( ntok .le. 0 ) goto 50

*       Get expanded command words
        call keyin ( line , ibeg , iend , dtyp , ntok , cmndlst , 
     &  cmndtot , maxcmnd ,';' , cmnd , cmndnum , cmndpos , cmndcnt )


*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Interpret command stream
 55     do 7500 icmnd = icur , cmndcnt

*       Compute number of (possible) data tokens present for this
*       command word
          if ( icmnd .eq. cmndcnt ) then
            numarg = ntok - cmndpos( icmnd )
          else
            numarg = cmndpos( icmnd+1 ) - cmndpos( icmnd ) - 1
          endif
*       Don't count command delimiter
          if ( line( ibeg( cmndpos( icmnd ) + numarg ):
     &               iend( cmndpos( icmnd ) + numarg ) ) .eq. ';' ) 
     &    numarg = numarg - 1

*-----------------------------------------------------------------------
*-------EXIT------------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'EXIT' ) .or. topcmd( 'QUIT' ) ) then
             exit = .true.
             return
          endif

*-----------------------------------------------------------------------
*-------@ (MACRO)-------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( '@' ) .and. numarg .ge. 1 ) then

            if ( macro ) then
              call errmsg ( 'Nesting of macros not allowed' , istop )
              macro = .false.
              close ( lunmac )
              goto 50
            endif

            macro = .true.
            call ttyptl ( ' RSPS > Reading commands from macro file' )
            macfil = line( ibeg( cmndpos( icmnd ) + 1 ) : 
     &                     iend( cmndpos( icmnd ) + 1 ) ) 
            close ( lunmac )
            ifail = 1
            call ccpdpn ( -lunmac,macfil,'READONLY','F',0,ifail )
            if ( ifail .ne. 1 ) then
              call errmsg ( 'Couldn''t open macro file' , istop )
              macro = .false.
            endif
            goto 50

          endif


*-----------------------------------------------------------------------
*-------ADD-------------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'ADD' ) ) then

*       Quit if no FIXXYZ sites stored
            if ( nxyz2 .eq. 0 ) then
              call wrnmsg ( 'RSPS ADD > currently vectors can ' //
     &        'only be added to FIXXYZ area' )
              call errmsg ( 'No FIXXYZ sites stored' , istop )
              goto 50
            endif

            if ( numarg .ge. 3 ) then

              inext = cmndpos( icmnd ) + 1
              if ( .not. nextwd( 'FIXXYZ' ) ) then
                call errmsg ( 'RSPS ADD > currently vectors can ' //
     &          'only be added to FIXXYZ area' , istop )
                goto 50
              endif

              inext = inext + 1
              ib = ibeg( inext )
              ie = iend( inext )
              if ( nextwd( 'POSITION' ) ) then
                inext = inext + 1
                if ( dtyp( inext ) .eq. 2 ) then
                  ipi =  fvalue( inext )
                  if ( ipi .le. 0 .or. ipi .gt. xintot ) then
                    call errmsg ( 'POSITION number out of range: ' //
     &              line( ib:iend( inext ) ) , istop )
                    goto 50
                  endif
*       Add position ipi to FIXXYZ coords
                  write(6,2800)ipi,(xpick(j,ipi),j=1,3)
                  do j = 1 , nxyz2
                    call rvecadd ( xyz2( 1,j ) , xpick( 1,ipi ) , 
     &              xtmp1 , 3 )
                    call rvecmod ( xtmp1 , xtmp1 , 1.0 , 3 )
                    write(6,2805)(xyz2(k,j),k=1,3),(xpick(k,ipi),k=1,3),
     &              xtmp1
                    call rmatcop ( xtmp1 , xyz2( 1,j ) , 3 , 1 )
                  end do
                else
                  call errmsg ( 'Illegal argument: ' //
     &            line( ib:iend( inext ) ) , istop )
                  goto 50
                endif

              else if ( nextwd( 'VECTOR' ) ) then
                if ( numarg .lt. 5 ) then
                  call errmsg ( 'Missing arguments: ' // 
     &            line( ib:slnblk( line ) ) , istop ) 
                  goto 50
                endif
                do j = 1 , 3
                  inext = inext + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    xtmp1( j ) = fvalue( inext )
                  else
                    call errmsg ( 'Illegal argument: ' //
     &              line( ib:iend( inext ) ) , istop )
                    goto 50
                  endif
                end do
*       Add vector to FIXXYZ coords
                write(6,2810)(xtmp1(j),j=1,3)
                do j = 1 , nxyz2
                  call rvecadd ( xyz2( 1,j ) , xtmp1 , xtmp2 , 3 )
                  call rvecmod ( xtmp2 , xtmp2 , 1.0 , 3 )
                  write(6,2805)(xyz2(k,j),k=1,3),xtmp1,xtmp2
                  call rmatcop ( xtmp2 , xyz2( 1,j ) , 3 , 1)
                end do

              else
                call errmsg ( 'Strange ADD argument > ' //
     &          line( ib:ie ) , istop ) 
                call ttyptl ( ' RSPS ADD > possible arguments ' //
     &          'are FIXXYZ POSITION n or FIXXYZ VECTOR u v w' )
                goto 50
              endif

              cmndin( cmndnum( icmnd ) ) = .true.

            else
              call ttyptl ( ' RSPS ADD > possible arguments ' //
     &        'are FIXXYZ POSITION n or FIXXYZ VECTOR u v w' )
            endif
            goto 7000
          endif



*-----------------------------------------------------------------------
*-------BUMP------------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd ( 'BUMP' ) ) then
            if ( numarg .ge. 1 ) then
              inext = cmndpos( icmnd ) + 1 
              if ( dtyp( inext ) .eq. 2 ) then
                bump = fvalue( inext )
                if ( bump .le. 0 ) then
                  special = .true.
                else
                  special = .false.
                endif
                bump = abs( bump )
                usrspc  = special
                usrbmp  = bump
              else
                call errmsg ( 'Bad BUMP argument' , istop )
                goto 50
              endif
            endif
            if ( special ) then
              write(6,2140)bump
            else
              write(6,2145)bump
            endif
            cmndin ( cmndnum( icmnd ) ) = .true.
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------CELL------------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'CELL' ) ) then
            if ( numarg .gt. 0 .and. numarg .lt. 3 ) then
              call errmsg ( 'You must give at least a,b,c' , istop )
              goto 50
            else if ( numarg .ge. 3 ) then
              inext = cmndpos( icmnd )
              do j = 1 , 3
                inext = inext + 1
                if ( dtyp( inext ) .eq. 2 ) then
                  cell( j ) = fvalue( inext )
                else
                  call errmsg ( 'Bad CELL parameters' , istop )
                  call matclr ( cell , 6 , 1 )
                  goto 50
                endif
              end do
              if ( numarg .ge. 6 ) then
                do j = 4 , 6
                  inext = inext + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    cell( j ) = fvalue( inext )
                  else
                    call errmsg ( 'Bad CELL parameters' , istop )
                    call matclr ( cell , 6 , 1 )
                    goto 50
                  endif
                end do
              else
                cell( 4 ) = 90.0
                cell( 5 ) = 90.0
                cell( 6 ) = 90.0
              end if
              call cosabg ( cell( 4 ) , cosang )
            endif
            cmndin( cmndnum( icmnd ) ) = .true.
            write(6,2380)cell
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------DELETE----------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'DELETE' ) ) then

            if ( numarg .ge. 1 ) then

              inext = cmndpos( icmnd ) + 1 
              ib = ibeg( inext )
              ie = iend( inext )
              call supcas ( line( ib:ie ) )
              if ( line( ib:ie ) .eq. 'FIXXYZ' ) then
                if ( nxyz2 .eq. 0 ) then
                  call rsinit ( 'FIXXYZ' )
                  write(6,2322)
                  goto 7000
                endif
                if ( numarg .ge. 2 ) then
*       Get pointers
                  call getpnt ( ipi , ipf , nxyz2 , dtyp , fvalue , 
     &            cmndpos( icmnd ) + 2 , numarg - 1 , errcod )
                  if ( errcod .ne. 0 ) goto 50
*                  inext = inext + 1
*                  if ( dtyp( inext ) .eq. 2 ) then
*                    ipi = fvalue( inext )
*                  else
*                    call errmsg ( 'Bad DELETE FIXXYZ argument' , 
*     &              istop )
*                    goto 50
*                  endif
*                  if ( ipi .le. 0 .or. ipi .gt. nxyz2 ) then
*                    call errmsg ( 'FIXXYZ number out of range: ' //
*     &              line( ib:iend( inext ) ) , istop )
*                    goto 50
*                  endif
                  do j = ipi , ipf
                    call matclr ( xyz2( 1,j ) , 3 , 1 )
                    write(6,2320) j
                  end do
                  do j  = 1 , nxyz2 - ipf
                    call matcop ( xyz2( 1,ipf+j ) , xyz2( 1,ipi-1+j ) , 
     &              3 , 1 ) 
                    call matclr ( xyz2( 1,ipf+j ) , 3 , 1 )
                  end do
                  nxyz2 = nxyz2 - ( ipf - ipi + 1 )
                else
                  call rsinit ( 'FIXXYZ' )
                  write(6,2322)
                endif
              else if ( line( ib:ie ) .eq. 'PEAKS' ) then
                call rsinit ( 'PEAKS' )
                write(6,2325)
              else if ( line( ib:ie ) .eq. 'NCS' ) then
                call rsinit ( 'NCS' )
                write(6,2326)
              else if ( line( ib:ie ) .eq. 'SETS' ) then
                call rsinit ( 'SETS' )
                write(6,2327)
              else
                call errmsg ( 'Strange DELETE argument > ' //
     &          line( ib:ie ) , istop ) 
                call ttyptl ( ' RSPS DELETE > possible arguments ' //
     &          'are FIXXYZ or PEAKS or SETS or NCS' )
                goto 50
              endif
              cmndin( cmndnum( icmnd ) ) = .true.


            else

              call ttyptl ( ' RSPS DELETE > possible arguments are ' //
     &        'FIXXYZ or PEAKS or SETS or NCS' )

            endif


            goto 7000

          endif


*-----------------------------------------------------------------------
*-------FIXXYZ----------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd ( 'FIXXYZ' ) ) then

            if ( numarg .eq. 0 ) then
              write(6,2265)nxyz2
              do j = 1 , nxyz2
                write(6,2260)(xyz2(k,j),k=1,3)
              end do
              goto 7000
            else if ( numarg .eq. 1 ) then
              inext = cmndpos( icmnd ) + 1
              if ( nextwd( 'SITES'     ) .or.
     &             nextwd( 'POSITIONS' ) .or.
     &             nextwd( 'SETS'      ) .or.
     &             nextwd( 'PEAKS'     ) ) then
                call wrnmsg ( 'Missing argument; no coordinates ' //
     &          'selected' )
                goto 50
              else 
                ib = ibeg( inext )
                ie = iend( inext )
                call errmsg ( 'Strange FIXXYZ argument > ' // 
     &          line(ib:ie) , istop )
                call ttyptl ( ' RSPS FIXXYZ > possible arguments ' //
     &          'are SITES or SETS or POSITIONS or PEAKS or x y z' )
                goto 50
              endif
            endif


*=======================================================================
            if ( numarg .ge. 2 ) then
*       At least two arguments; attempt interpretation as subcommand +
*       value
*=======================================================================

*       Are there any free slots in fixxyz array?
              if ( nxyz2 .eq. mxipsn ) then
                call errmsg ( 'FIXXYZ array full' , istop )
                goto 50
              endif

*       Check first argument is uniquely interpretable
              inext = cmndpos( icmnd ) + 1
              ib = ibeg( inext )
              ie = iend( inext )
              call supcas ( line( ib:ie ) )
              if ( line( ib:ie ) .eq. 'S' .or. line( ib:ie ) .eq. 'P' )
     &        then
                call errmsg ( 'Ambiguous FIXXYZ argument > ' // 
     &          line( ib:ie ) , istop )
                call ttyptl ( ' RSPS FIXXYZ > possible arguments ' //
     &          'are SITES or SETS or POSITIONS or PEAKS or x y z' )
                goto 50
              endif
              

*       Look for subcommands >
*-----------------------------------------------------------------------
              if ( nextwd( 'SITES' ) ) then
*       Get fixxyz positions from stored positions by specifying 
*       site number
*-----------------------------------------------------------------------
*       Quit if no sites stored
                if ( sittot .eq. 0 ) then
                  call errmsg ( ' RSPS >> FIXXYZ SITES >> ' //
     &            'No sites stored' , istop )
                  goto 50
                endif

*       Get pointers
                ipi = 1
                ipf = 1
                call getpnt ( ipi , ipf , sittot , dtyp , fvalue , 
     &          cmndpos( icmnd ) + 2 , numarg - 1 , errcod )
                if ( errcod .ne. 0 ) goto 50

*       Copy coordinates from XPICK array
                do j = ipi , ipf
                  nxyz2 = nxyz2 + 1
                  if ( nxyz2 .le. mxipsn ) then
                    call matcop ( xpick( 1,sitbeg(j) ) , 
     &              xyz2( 1,nxyz2 ) , 3 , 1 )
                    write(6,2261)(xyz2(k,nxyz2),k=1,3),j,sitbeg(j)
                  endif
                end do


*-----------------------------------------------------------------------
              else if ( nextwd( 'POSITIONS' ) ) then
*       Get fixxyz positions from stored positions by specifying 
*       position number
*-----------------------------------------------------------------------
*       Quit if no positions stored
                if ( xintot .eq. 0 ) then
                  call errmsg ( ' RSPS >> FIXXYZ POSITIONS >> ' //
     &            'No positions stored' , istop )
                  goto 50
                endif

*       Get pointers
                ipi = 1
                ipf = 1
                call getpnt ( ipi , ipf , xintot , dtyp , fvalue ,
     &          cmndpos( icmnd ) + 2 , numarg - 1 , errcod )
                if ( errcod .ne. 0 ) goto 50

*       Copy coordinates from XPICK array
                do j = ipi , ipf
                  nxyz2 = nxyz2 + 1
                  if ( nxyz2 .le. mxipsn ) then
                    call matcop ( xpick( 1,j ) , xyz2( 1,nxyz2 ) , 
     &              3 , 1 )
                    write(6,2261)(xyz2(k,nxyz2),k=1,3),sitpnt(j),j
                  endif
                end do


*-----------------------------------------------------------------------
              else if ( nextwd( 'SETS' ) ) then
*       Get fixxyz positions from stored positions by specifying 
*       set number
*-----------------------------------------------------------------------
*       Quit if no sets stored
                if ( settot .eq. 0 ) then
                  call errmsg ( ' RSPS >> FIXXYZ SETS >> ' //
     &            'No sets stored' , istop )
                  goto 50
                endif

*       Get pointers
                ipi = 1
                ipf = 1
                call getpnt ( ipi , ipf , settot , dtyp , fvalue ,
     &          cmndpos( icmnd ) + 2 , numarg - 1 , errcod )
                if ( errcod .ne. 0 ) goto 50

*       Copy coordinates from XPICK array
                do j = ipi , ipf
                  do k = 1 , setsiz( j )
                    nxyz2 = nxyz2 + 1
                    imem = setpos( k,j )
                    if ( nxyz2 .le. mxipsn ) then
                      call matcop ( xpick( 1,imem ) , xyz2( 1,nxyz2 ) , 
     &                3 , 1 )
                      write(6,2261)(xyz2(l,nxyz2),l=1,3),j,imem
                    endif
                  end do
                end do


*-----------------------------------------------------------------------
              else if ( nextwd( 'PEAKS' ) ) then
*       Get fixxyz positions from stored peaks by specifying 
*       peak number
*-----------------------------------------------------------------------
*       Quit if no peaks stored
                if ( ppnum .eq. 0 ) then
                  call errmsg ( ' RSPS >> FIXXYZ PEAKS >> ' //
     &            'No peaks stored' , istop )
                  goto 50
                endif

*       Get pointers
                ipi = 1
                ipf = 1
                call getpnt ( ipi , ipf , ppnum , dtyp , fvalue ,
     &          cmndpos( icmnd ) + 2 , numarg - 1 , errcod )
                if ( errcod .ne. 0 ) goto 50

*       Copy coordinates from PATPEK array
                do j = ipi , ipf
                  nxyz2 = nxyz2 + 1
                  if ( nxyz2 .le. mxipsn ) then
                    call matcop ( ppuvw( 1,j ) , xyz2( 1,nxyz2 ) , 
     &              3 , 1 )
                    write(6,2262)(xyz2(k,nxyz2),k=1,3),j
                  endif
                end do


*-----------------------------------------------------------------------
              else if ( numarg .ge. 3 ) then
*       At least three arguments and failure to interpret first arg as
*       subcommand; attempt interpretation as coordinates
*-----------------------------------------------------------------------

*       Get fixxyz positions from line
                if ( nxyz2 .lt. mxipsn ) then
                  nxyz2 = nxyz2 + 1
                  inext = cmndpos( icmnd )
                  do j = 1 , 3
                  inext = inext + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    xyz2( j , nxyz2 ) = fvalue( inext )
                  else
                    call errmsg ( 'Bad FIXXYZ argument' , istop )
                    nxyz2 = nxyz2 - 1
                    goto 50
                  endif
                  end do
                  write(6,2260)(xyz2(l,nxyz2),l=1,3)
                else
                  call wrnmsg ( 'FIXXYZ array full - not stored' )
                  goto 50
                endif

*-----------------------------------------------------------------------
              else
*       Couldn't interpret arguments
*-----------------------------------------------------------------------
                call errmsg ( 'Bad FIXXYZ arguments', istop )
                goto 50

*-----------------------------------------------------------------------
              endif
*-----------------------------------------------------------------------

*=======================================================================
            endif
*=======================================================================


*       If legal input we end up here >

*       Check and report trapped overflow attempt
            if ( nxyz2 .gt. mxipsn ) then
              nxyz2 = mxipsn
              call wrnmsg ( 'FIXXYZ array full' )
              call wrnmsi ( 'last stored position > ' ,
     &        ipi + nxyz2 - 1 , 4 )
            endif

            write(6,2265)nxyz2
            cmndin( cmndnum( icmnd ) ) = .true.
            goto 7000

          endif



*-----------------------------------------------------------------------
*-------GETSETS---------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd ( 'GETSETS' ) ) then

            call rsinit ( 'GETSET' )
            if ( xintot .eq. 0 ) then
              call errmsg ( 'GETSETS: No positions stored' , istop )
              goto 50
            endif
            if ( numarg .ge. 1 ) then
              inext = cmndpos( icmnd ) + 1 
              if ( dtyp( inext ) .eq. 2 ) then
                minset = fvalue( inext )
              else
                call errmsg ( 'Bad GETSETS argument' , istop )
                goto 50
              endif
            endif
            if ( numarg .ge. 2 ) then
              inext = cmndpos( icmnd ) + 2
              if ( dtyp( inext ) .eq. 2 ) then
                npguse = fvalue( inext )
                if ( npguse .gt. xintot ) then
                  call wrnmsg ( 'Number of positions ' //
     &            'requested exceeds number of positions stored' )
                  call wrnmsi ( 'GETSETS will use all stored ' //
     &            'positions = ' , xintot , 4 )
                  npguse = xintot
                endif
              else
                call errmsg ( 'Bad GETSETS argument' , istop )
                goto 50
              endif
            else
              npguse = xintot
            endif

            dogets = .true.
            cmndin( cmndnum( icmnd ) ) = .true.
            write(6,2270)minset,npguse
            goto 8000
          endif


*-----------------------------------------------------------------------
*-------HELP------------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'HELP' ) ) then


            if ( numarg .eq. 0 ) then
              call ttyptl ( ' Valid commands :' )
              write ( 6 , '( 4 ( 2x , a ) )' ) cmndlst
              if ( ccpexs ( 'rspshlp4' ) ) then 
                call ttycr ( 1 )
                call ttyptl ( ' For help on a particular ' //
     &          'command enter HELP command' )
              endif
            else
              if ( ccpexs ( 'rspshlp4' ) ) then
                inext = cmndpos( icmnd ) + 1
                ib = ibeg( inext )
                ie = iend( inext )
                item = line( ib:ie )
                call ugtenv ( 'rspshlp4' , hlpfil )
                call hlptxt ( lunhlp , hlpfil , item , errcod )
                if ( errcod .eq. 1 ) then
                  call errmsg ( 'Could not open or read help ' //
     &            'text file ' // hlpfil( :slnblk(hlpfil) ), 0 )
                  goto 7500
                else if ( errcod .ge. 2 ) then
                  call errmsg ( 'Help error', 0 )
                endif
              else
                call ttycr ( 1 )
                call ttyptl ( ' Help not installed' )
                call ttyptl ( ' Can''t find help file' )
              endif
            endif

            cmndin( cmndnum( icmnd ) ) = .true.
            goto 7000

          endif


*-----------------------------------------------------------------------
*-------LIST------------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'LIST' ) ) then

            if ( numarg .ge. 1 ) then

              inext = cmndpos( icmnd ) + 1 
              ib = ibeg( inext )
              ie = iend( inext )
              call supcas ( line( ib:ie ) )
              if ( line( ib:ie ) .eq. 'P' .or. 
     &             line( ib:ie ) .eq. 'S' ) then
                call errmsg ( 'Ambiguous LIST argument > ' // 
     &          line( ib:ie ) , istop )
                call ttyptl ( ' RSPS LIST > possible arguments are ' //
     &          'POSITIONS or PEAKS or SETS or SYMOPS or VECTORS' )
                goto 50
              endif


*-----------------------------------------------------------------------
              if ( nextwd( 'POSITIONS' ) ) then
*-----------------------------------------------------------------------
*       Quit if no positions stored
                if ( xintot .eq. 0 ) then
                  call ttyptl ( ' RSPS >> LIST POSITIONS >> ' //
     &            'No positions stored' )
                  goto 7000
                endif

*       Set matrix for converting fractional crystallographic
*       coordinates to Angstrom orthogonal in PDB standard frame
                call ortho ( cell , 1 , 'TOORT' , toort , errcod )
                if ( errcod .ne. 0 ) then
                  call errmsg ( 'Failed getting orthogonalization ' //
     &            'matrix' , istop )
                  goto 50
                endif

*       Get pointers
                ipi = 1
                ipf = 1
                call getpnt ( ipi , ipf , xintot , dtyp , fvalue ,
     &          cmndpos( icmnd ) + 2 , numarg - 1 , errcod )
                if ( errcod .ne. 0 ) goto 50

                ict = 0
                if ( vecset .eq. 'MORE ATOMS' .and. patisin ) then
                  write(6,2330) ipi , ipf
                else
                  write(6,2335) ipi , ipf
                endif
                do j = ipi , ipf
                  ict = ict + 1
                  call vmtply ( toort , xpick( 1,j ) , xut )
                  if ( vecset .eq. 'MORE ATOMS' .and. patisin ) then
*       When using vectorset MORE ATOMS also get Harker vector score

                    call hvscor ( patmap , patfas , patslo , patsec , 
     &              patsig , r_sym , t_sym , nopsym , xpick( 1,j ) , 
     &              hvsc , idummy(1) , idummy(2) , idummy(3) , 
     &              rdummy(1) , rdummy(2) , errcod )

                    write(6,2337) j , ( xpick( k,j ) , k = 1 , 3 )  ,
     &              xut , value( j ) , hvsc , sitpnt( j )

                  else
                    write(6,2338) j , ( xpick( k,j ) , k = 1 , 3 )  ,
     &              xut , value( j ) , sitpnt( j )
                  endif

                  if ( ict .eq. 40 .and. j .lt. ipf ) then
                    call ttycr ( 2 )
                    call ttygtl ( ' Hit RETURN to continue, ' //
     &              'enter q to quit > ' , junkln , errcod )
                    call ttycr ( 2 )
                    ict = 0
                    call supcas ( junkln )
                    if ( index( junkln , 'Q' ) .ne. 0 ) then
                      cmndin( cmndnum( icmnd ) ) = .true.
                      goto 7000
                    endif
                  endif
                end do

*-----------------------------------------------------------------------
              else if ( nextwd( 'PEAKS' ) ) then
*-----------------------------------------------------------------------
*       Quit if no peaks stored
                if ( ppnum .eq. 0 ) then
                  call ttyptl ( ' RSPS >> LIST PEAKS >> ' //
     &            'No peaks stored' )
                  goto 7000
                endif

*       Set matrix for converting fractional crystallographic
*       coordinates to Angstrom orthogonal in PDB standard frame
                call ortho ( cell , 1 , 'TOORT' , toort , errcod )
                if ( errcod .ne. 0 ) then
                  call errmsg ( 'Failed getting orthogonalization ' //
     &            'matrix' , istop )
                  goto 50
                endif

*       Get pointers
                ipi = 1
                ipf = 1
                call getpnt ( ipi , ipf , ppnum , dtyp , fvalue ,
     &          cmndpos( icmnd ) + 2 , numarg - 1 , errcod )
                if ( errcod .ne. 0 ) goto 50

                ict = 0
                write(6,2340) ipi , ipf
                do j = ipi , ipf
                  ict = ict + 1
                  call vmtply ( toort , ppuvw( 1,j ) , xut )
                  write ( 6,2345 ) j , symbol( pptyp( j ) ) , 
     &            ( ppuvw( k,j ) , k = 1 , 3 ) , xut , 
     &            ( nint ( ppuvw( k,j ) * patgrd( k ) ) , k = 1 , 3 ) ,
     &            ppval( j ) , ppval( j )/patsig
                  if ( ict .eq. 40 .and. j .lt. ipf ) then
                    call ttycr ( 2 )
                    call ttygtl ( ' Hit RETURN to continue, ' //
     &              'enter q to quit > ' , junkln , errcod )
                    ict = 0
                    call ttycr ( 2 )
                    call supcas ( junkln )
                    if ( index( junkln , 'Q' ) .ne. 0 ) then
                      cmndin( cmndnum( icmnd ) ) = .true.
                      goto 7000
                    endif
                  endif
                end do

*-----------------------------------------------------------------------
              else if ( nextwd( 'SETS' ) ) then
*-----------------------------------------------------------------------
*       Quit if no sets stored
                if ( settot .eq. 0 ) then
                  call ttyptl ( ' RSPS >> LIST SETS >> ' //
     &            'No sets stored' )
                  goto 7000
                endif

*       Get pointers
                ipi = 1
                ipf = 1
                call getpnt ( ipi , ipf , settot , dtyp , fvalue ,
     &          cmndpos( icmnd ) + 2 , numarg - 1 , errcod )
                if ( errcod .ne. 0 ) goto 50

                ict = 0
                write(6,2350)
                do j = ipi , ipf
                  ict = ict + 1
                  nmem = setsiz( j )
*                  write(6,2355)j,setscr(j),nmem,
*     &            (sitpnt( setpos(k,j) ),setpos(k,j),k=1,nmem)
                  write(6,2355)j,setscr(j),nmem,
     &            (setpos(k,j),k=1,nmem)
                  if ( ict .eq. 40 .and. j .lt. ipf ) then
                    call ttycr ( 2 )
                    call ttygtl ( ' Hit RETURN to continue, ' //
     &              'enter q to quit > ' , junkln , errcod )
                    ict = 0
                    call ttycr ( 2 )
                    call supcas ( junkln )
                    if ( index( junkln , 'Q' ) .ne. 0 ) then
                      cmndin( cmndnum( icmnd ) ) = .true.
                      goto 7000
                    endif
                  endif
                end do

                cmndin( cmndnum( icmnd ) ) = .true.
                goto 7000

*-----------------------------------------------------------------------
              else if ( nextwd( 'SYMOPS' ) ) then
*-----------------------------------------------------------------------
*       List crystallographic symmetry operators
                write(6,2360) spgnam
                do i = 1 , nopsgs
                  call mysymtrn ( r_sgs(1,1,i) , t_sgs(1,i) , symchr )
                  write(6,2361) i , symchr( :slnblk(symchr) )
                end do
                if ( icen .gt. 1 ) then
                  write(6,2362) ( ( cent(j,i),j=1,3),i=2,icen )
                else
                  write(6,*)
                endif

*       List NCS operators if present
                if ( nopncs .gt. 1 ) then
                  write(6,2363)nopncs
                  do i = 1 , nopncs
                    write(6,2364)i,((r_ncs(k,j,i),j=1,3),
     &              t_ncs(k,i),k=1,3)
                  end do
                  call ttycr ( 1 )
                else
                  write(6,2365)
                endif

                cmndin( cmndnum( icmnd ) ) = .true.
                goto 7000

*-----------------------------------------------------------------------
              else if ( nextwd( 'VECTORS' ) ) then
*-----------------------------------------------------------------------
*       List vectors
                if ( index ( vecset , 'MORE' ) .ne. 0 .and. 
     &          nxyz2 .lt. 1 ) then
                  call errmsg ( vecset // ' VECTORS > must have ' //
     &            'at least one fixed position (FIXXYZ)' , istop )
                  goto 50
                endif
                if ( index ( vecset , 'TRANSLATE' ) .ne. 0 .and. 
     &          nxyz2 .lt. 2 ) then
                  call errmsg ( vecset // ' VECTORS > must have at ' //
     &            'least two fixed positions (FIXXYZ)' , istop )
                  goto 50
                endif
                if ( index ( vecset , 'MOLECULES' ) .ne. 0 ) then
                  if ( nopncs .le. 1 ) then
                    call errmsg ( vecset // ' VECTORS > non ' //
     &              'crystallographic symmetry must be defined' , 
     &              istop )
                    goto 50
                  endif
                  usrncs = usencs
                  usencs = .true.
                endif
                call lodsym ( .true. , errcod )
                usencs = usrncs
                if ( errcod .ne. 0 ) then
                  call errmsg ( 'LODSYM failed: symmetry error' , 
     &            istop )
                  goto 50
                endif
                write(6,2367)ntrans
                do i = 1 , ntrans
                  write(6,2368)i , mult( i ) , vecnam( vectyp( i ) ) ,
     &            ( ( r_vec( k,j,i ) , j = 1 , 3 ) , 
     &            t_vec( k,i ) , k = 1 , 3 )
                end do
 
                cmndin( cmndnum( icmnd ) ) = .true.
                goto 7000

*-----------------------------------------------------------------------
              else
*-----------------------------------------------------------------------
                call errmsg ( 'Strange LIST argument > ' //
     &          line( ib:ie ) , istop ) 
                call ttyptl ( ' RSPS LIST > possible arguments are ' //
     &          'POSITIONS or PEAKS or SETS or SYMOPS or VECTORS' )
                goto 50

*-----------------------------------------------------------------------
              endif
*-----------------------------------------------------------------------

            else

              call ttyptl ( ' RSPS LIST > possible arguments are ' //
     &        'POSITIONS or PEAKS or SETS or SYMOPS or VECTORS' )
              goto 50

            endif

            cmndin( cmndnum( icmnd ) ) = .true.
            goto 7000

          endif


*-----------------------------------------------------------------------
*-------LOW-------------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'LOW' ) ) then
            if ( numarg .ge. 1 ) then
              inext = cmndpos( icmnd ) + 1 
              if ( dtyp( inext ) .eq. 2 ) then
                maxlow = fvalue( inext )
              else
                call errmsg ( 'Bad LOW argument' , istop )
                goto 50
              endif
            endif
            cmndin ( cmndnum( icmnd ) ) = .true.
            write(6,2130)maxlow
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------NCSROT----------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'NCSROT' ) ) then

            if ( numarg .eq. 0 ) then
              write(6,2242)nopncs
              do j = 1 , nopncs
                write(6,2243)j
                do k = 1 , 3
                  write(6,2244)(r_ncs(k,l,j),l=1,3)
                end do
                write(6,2244)(t_ncs(l,j),l=1,3)
              end do
              goto 7000
            endif

            inext = cmndpos( icmnd ) + 1 

*-----------------------------------------------------------------------
            if ( nextwd( 'MATRIX' ) ) then
*-----------------------------------------------------------------------
*       Check number of arguments
              if ( numarg .lt. 13 ) then
                call errmsg ( 'NCSROT MATRIX: too few arguments' ,
     &          istop )
                goto 50
              endif

*       We must know cell parameters
              if ( patnotin .and. .not. 
     &        cmndin( keynm( 'CELL' , cmndlst , cmndtot ) ) ) then
                call errmsg ( 'You must define the unit cell ' //
     &          'parameters' , istop )
                goto 50
              endif

*       Read rotation matrix row by row
              inext = cmndpos( icmnd ) + 1 
              do k = 1 , 3
                do j = 1 , 3
                  inext = inext + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    rtmp( k,j ) = fvalue( inext )
                  else
                    call errmsg ( 'Bad NCSROT MATRIX argument' , 
     &              istop )
                    goto 50
                  endif
                end do
                if ( iprt .gt. 2 ) write(6,2244)(rtmp( k,j ) , 
     &          j = 1 , 3 )
              end do

*       Read translation components
              do j = 1 , 3
                inext = inext + 1
                if ( dtyp( inext ) .eq. 2 ) then
                  ttmp( j ) = fvalue( inext )
                else
                  call errmsg ( 'Bad NCSROT MATRIX argument' , 
     &            istop )
                  goto 50
                endif
              end do
              if ( iprt .gt. 2 ) write(6,2244)ttmp

*       Store ncs 
              call getncs ( rtmp , ttmp , cell , ortcod , 
     &        iprt , errcod )
              if ( errcod .ne. 0 ) then
                call errmsg ( 'GETNCS failure' , istop )
                goto 50
              endif

*-----------------------------------------------------------------------
            else if ( nextwd( 'POLAR' ) ) then
*-----------------------------------------------------------------------
*       Check number of arguments
              if ( numarg .lt. 7 ) then
                call errmsg ( 'NCSROT POLAR: too few arguments' ,
     &          istop )
                goto 50
              endif

*       We must know cell parameters
              if ( patnotin .and. .not. 
     &        cmndin( keynm( 'CELL' , cmndlst , cmndtot ) ) ) then
                call errmsg ( 'You must define the unit cell ' // 
     &          'parameters' , istop )
                goto 50
              endif

*       Read polar angles
              inext = cmndpos( icmnd ) + 1 
              do j = 1 , 3
                inext = inext + 1
                if ( dtyp( inext ) .eq. 2 ) then
                  polang( j ) = fvalue( inext )
                else
                  call errmsg ( 'Bad NCSROT POLAR argument' , 
     &            istop )
                  goto 50
                endif
              end do

*       Read translation components
              do j = 1 , 3
                inext = inext + 1
                if ( dtyp( inext ) .eq. 2 ) then
                  ttmp( j ) = fvalue( inext )
                else
                  call errmsg ( 'Bad NCSROT POLAR argument' , 
     &            istop )
                  goto 50
                endif
              end do
              write(6,2245)polang,ttmp

*       Convert polar angles to matrix
              call polar2mtx ( polang(1) , polang(2) , polang(3) ,
     &        rtmp )

              if ( iprt .gt. 2 ) then
                do k = 1 , 3
                  write(6,2244)(rtmp( k,j ) , j = 1 , 3 )
                end do
              endif

*       Store ncs 
              call getncs ( rtmp , ttmp , cell , ortcod , 
     &        iprt , errcod )
              if ( errcod .ne. 0 ) then
                call errmsg ( 'GETNCS failure' , istop )
                goto 50
              endif

*-----------------------------------------------------------------------
            else if ( nextwd( '2-FOLD' ) .or.
     &                nextwd( '3-FOLD' ) .or.
     &                nextwd( '4-FOLD' ) .or.
     &                nextwd( '6-FOLD' ) ) then
*-----------------------------------------------------------------------
*       Check that axis asked for is one of 2-,3-,4-,6-fold
              if ( nextwd( '2-FOLD' ) ) then
                nfold = 2
              else if ( nextwd( '3-FOLD' ) ) then
                nfold = 3
              else if ( nextwd( '4-FOLD' ) ) then
                nfold = 4
              else if ( nextwd( '6-FOLD' ) ) then
                nfold = 6
              else
                ib = ibeg( inext )
                ie = iend( inext )
                call errmsg ( 'Illegal n-fold axis > ' // 
     &          line( ib:ie ) , istop )
                goto 50
              endif

*       Initiate angle and direction cosines
              rotang = 0.0
              dircos( 1 ) = 0.0
              dircos( 2 ) = 0.0
              dircos( 3 ) = 0.0

*       We must know cell parameters
              if ( patnotin .and. .not. 
     &        cmndin( keynm( 'CELL' , cmndlst , cmndtot ) ) ) then
                call errmsg ( 'You must define the unit cell ' // 
     &          'parameters' , istop )
                goto 50
              endif

*       Check number of arguments
              if ( numarg .lt. 3 ) then
                call errmsg ( 'NCSROT n-FOLD: too few arguments' , 
     &          istop )
                goto 50
              endif
              inext = inext + 1 
              if ( nextwd( 'ALONG' ) ) then
*       Get direction cosines or parallell axis
                if ( dtyp( inext + 1 ) .eq. 1 ) then
                  inext = inext + 1
                  if ( nextwd( 'X' ) ) then
                    dircos( 1 ) = 1.0
                  else if ( nextwd( 'Y' ) ) then
                    dircos( 2 ) = 1.0
                  else if ( nextwd( 'Z' ) ) then
                    dircos( 3 ) = 1.0
                  else
                    call errmsg ( 'Unknown value following NCSROT ' //
     &              'n-FOLD ALONG - must be one of X,Y,Z or ' //
     &              'direction cosines l,m,n' , istop )
                    goto 50
                  endif
                else
                  do j = 1 , 3
                    inext = inext + 1
                    if ( dtyp( inext ) .eq. 2 ) then
                      dircos( j ) = fvalue( inext )
                    else
                      call errmsg ( 'Bad NCSROT n-FOLD argument' , 
     &                istop )
                      goto 50
                    endif
                  end do
                end if
              else
                call errmsg ( 'Illegal NCSROT n-FOLD syntax ' , istop )
                call ttyptl ( ' RSPS NCSROT n-FOLD > ' //
     &          'Legal construct is' )
                call ttyptl ( ' NCSROT n-FOLD ALONG < X | Y | ' //
     &          'Z | l m n > (THROUGH t1 t2 t3)' )
                goto 50
              endif

              if ( numarg + 1 - inext .gt. 0 .and. 
     &             numarg + 1 - inext .lt. 4 ) then
                call errmsg ( 'NCSROT n-FOLD: too few arguments' ,
     &          istop )
                goto 50
              else if ( numarg + 1 - inext .ge. 4 ) then
                inext = inext + 1 
                if ( nextwd( 'THROUGH' ) ) then
*       Read axis position
                  do j = 1 , 3
                    inext = inext + 1
                    if ( dtyp( inext ) .eq. 2 ) then
                      tpos( j ) = fvalue( inext ) * cell( j )
                    else
                      call errmsg ( 'Bad NCSROT n-FOLD argument' , 
     &                istop )
                      goto 50
                    endif
                  end do
                else
                  call errmsg ( 'Illegal NCSROT n-FOLD syntax ' , 
     &            istop )
                  call ttyptl ( ' RSPS NCSROT n-FOLD > ' //
     &            'Legal construct is' )
                  call ttyptl ( ' NCSROT n-FOLD ALONG < X | Y | ' //
     &            'Z | l m n > (THROUGH t1 t2 t3)' )
                  goto 50
                endif
              endif

*       Construct rotation matrices and translation vectors
              rotang = 0.0
              do j = 2 , nfold
*        Rotation matrix
                rotang = rotang + 360.0 / nfold
                call nfold2mtx ( dircos , rotang , rtmp )
*        Translation vector = (I-R)*tpos
                call vmtply ( rtmp , tpos , ttmp )
                call vecdif ( tpos , ttmp , ttmp , 3 )
                do k = 1 , 3
                  ttmp( k ) = mod ( ttmp( k ) / cell( k ) , 1.0 )
                  if ( abs( ttmp( k ) ) .lt. 0.0001 ) ttmp( k ) = 0.0
                  if ( ttmp( k ) .lt. 0. ) ttmp( k ) = ttmp( k ) + 1.0
                end do

                if ( iprt .gt. 2 ) then
                  do k = 1 , 3
                    write(6,2244)(rtmp( k,l ) , l = 1 , 3 )
                  end do
                  write(6,2244)ttmp
                endif

*       Store ncs 
                call getncs ( rtmp , ttmp , cell , ortcod , 
     &          iprt , errcod )
                if ( errcod .ne. 0 ) then
                  call errmsg ( 'GETNCS failure' , istop )
                  goto 50
                endif

              end do

*-----------------------------------------------------------------------
            else if ( nextwd( 'ORTHO' ) ) then
*-----------------------------------------------------------------------
*       Check number of arguments
              if ( numarg .lt. 2 ) then
                write(6,2246)ortcod
                goto 50
              endif

*       Read orthogonalization code
              inext = cmndpos( icmnd ) + 2
              if ( dtyp( inext ) .eq. 2 ) then
                ortcod = fvalue( inext )
              else
                call errmsg ( 'Bad NCSROT ORTHO argument' , 
     &          istop )
                goto 50
              endif
              if ( ortcod .lt. 1 .or. ortcod .gt. 5 ) then
                call errmsg ( 'Illegal orthogonalization code,' //
     &          'must be 1,2,3,4 or 5' , istop )
                call rsinit ( 'ORTHO' )
                goto 50
              endif

              write(6,2246)ortcod
*-----------------------------------------------------------------------
            else
*-----------------------------------------------------------------------

              ib = ibeg( inext )
              ie = iend( inext )
              call errmsg ('Strange NCSROT argument > ' // 
     &        line(ib:ie) , istop )
              call ttyptl ( ' RSPS NCSROT > possible arguments are ' 
     &        // 'MATRIX or POLAR or n-FOLD or ORTHO' )
              goto 50

*-----------------------------------------------------------------------
            endif
*-----------------------------------------------------------------------

            cmndin ( cmndnum( icmnd ) ) = .true.
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------NCSYMM - non-crystallographic symmetry switch-------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'NCSYMM' ) ) then
            if ( numarg .ge. 1 ) then
              inext = cmndpos( icmnd ) + 1 
              if ( nextwd( 'ON' ) ) then
                usencs = .true.
              else if ( nextwd( 'OFF' ) ) then
                usencs = .false.
              else
                call errmsg ( 'Illegal NCSYMM command' , istop )
                goto 50
              endif
              usrncs = usencs
            endif
            if ( usencs ) then
              write(6,2240)
            else
              write(6,2241)
            endif
            cmndin ( cmndnum( icmnd ) ) = .true.
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------PATFILE---------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'PATFILE' ) ) then

*       No arguments - just report
            if ( numarg .eq. 0 ) then
              chr = ' '
              if ( dotrnc .eq. 2 ) chr = 'sigma'
              write(6,2020)patfil( :slnblk ( patfil ) )
              write(6,2025)pattyp
              write(6,2030)patscl
              if ( dotrnc .ne. 0 ) write(6,2035)dens_max,trunclev,chr
              do j = 1 , rstnum
              write(6,2040)(rstuvw(k,j),k=1,3),
     &        rstrad(j),rstlev(j)
              end do
              goto 7000
            endif

*       At least one argument
*       Init variables
            call rsinit ( 'MAPFORMAT' )
            call rsinit ( 'SCALE' )
            call rsinit ( 'DENS_MAX' )
            call rsinit ( 'RESET' )

*       First argument interpreted as file name
            inext = cmndpos( icmnd ) + 1
            ib = ibeg( inext )
            ie = iend( inext )
            patfil = line ( ib:ie )

            if ( numarg .ge. 2 ) then
*       Additional subcomands present - find location in string

*       First get expanded subcommands
            inext = inext + 1
            call keyin ( line , ibeg( inext ) , iend( inext ) ,
     &      dtyp( inext ) , numarg-1 , patcmndlst , patcmndtot ,
     &      maxcmnd , 'NONE' , subcmnd , subcmndnum , subcmndpos , 
     &      subcmndcnt )

            if ( subcmndcnt .eq. 0 ) then
              call errmsg ( 'Illegal PATFILE command > ' //
     &        line( ibeg(cmndpos(icmnd)):
     &              iend(cmndpos(icmnd)+numarg) ) , istop )
              call ttyptl ( ' RSPS PATFILE > legal construct is' )
              call ttyptl ( ' PATFILE filename (TYPE ' //
     &        '<[CCP4] | PROTEIN>) (SCALE s1 (s2)) ' //
     &        '(TRUNCATE rhomax (level) (S) ) ' //
     &        '({RESET <ORIGIN | u v w> radius level})' )
              goto 50
            endif

*=======================================================================
*       Interpret subcommand stream
            do jcmnd = 1 , subcmndcnt
*=======================================================================

*       Compute number of (possible) data tokens present for this
*       command word
              if ( jcmnd .eq. subcmndcnt ) then
                numsubarg = numarg - subcmndpos(jcmnd) - 1
              else
                numsubarg = subcmndpos(jcmnd+1) - subcmndpos(jcmnd) - 1
              endif

*       Interpret
*-----------------------------------------------------------------------
              if ( subcmd( 'RESET' ) ) then
*-----------------------------------------------------------------------
                if ( numsubarg .lt. 2 ) then
                  call errmsg ( 'Missing RESET argument(s)' , istop )
                  goto 50
                endif

                if ( rstnum .ge. mxreset ) then
                  call errmsg ( 'Maximum number of RESET commands ' //
     &            'exceeded' , istop )
                  goto 50
                endif

                ict = rstnum
                inext = cmndpos(icmnd) + subcmndpos(jcmnd) + 2

*       If less than 4 arguments must have character string in first
*       argument. Check to see if first word can be interpreted as 
*       valid text string.
                if ( numsubarg .le. 3 ) then
                  if ( nextwd( 'ORIGIN' ) ) then
                    do k = 1 , icen
                      ict = ict + 1
                      rstuvw( 1,ict ) = cent( 1,k )
                      rstuvw( 2,ict ) = cent( 2,k )
                      rstuvw( 3,ict ) = cent( 3,k )
                    end do
                  else
                    ib = ibeg( inext )
                    ie = iend( inext )
                    call errmsg ( 'Invalid RESET argument > ' //
     &              line( ib:ie ) , istop )
                    goto 50
                  endif
                  inext = inext + 1
                else
*       Reset coordinates (fractional)
                  ict = ict + 1
                  do k = 1 , 3
                    if ( dtyp( inext ) .eq. 2 ) then
                      rstuvw( k,ict ) = mod ( fvalue( inext ) , 1.0 )
                    else
                      call errmsg ( 'Bad RESET argument' , istop )
                      goto 50
                    endif
                    inext = inext + 1
                  end do
                endif

*       Reset radius
                if ( dtyp( inext ) .eq. 2 ) then
                  do k = rstnum + 1 , ict
                    rstrad( k) = fvalue( inext )
                  end do
                else
                  call errmsg ( 'Bad RESET argument' , istop )
                  goto 50
                endif

*       Reset level
                if ( numsubarg .ge. 5 ) then
                  inext = inext + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    do k = rstnum + 1 , ict
                      rstlev( k ) = fvalue( inext )
                    end do
                  else
                    call errmsg ( 'Bad RESET argument' , istop )
                    goto 50
                  endif
                endif

*       Set reset counter
                rstnum = ict

*-----------------------------------------------------------------------
              else if ( subcmd( 'SCALE' ) ) then
*-----------------------------------------------------------------------

                if ( numsubarg .lt. 1 ) then
                  call errmsg ( 'Missing SCALE argument' , istop )
                  goto 50
                endif

*       First arg is scale; second is term to add
                inext = cmndpos(icmnd) + subcmndpos(jcmnd) + 2
                if ( dtyp( inext ) .ne. 2 ) then
                  call errmsg ( 'Bad SCALE argument' , istop )
                  goto 50
                else
                  patscl( 1 ) = fvalue( inext )
                  patscl( 2 ) = 0.0
                endif
                if ( numsubarg .ge. 2 ) then
                  inext = inext + 1
                  if ( dtyp( inext ) .ne. 2 ) then
                    call errmsg ( 'Bad SCALE argument' , istop )
                    goto 50
                  else
                    patscl( 2 ) = fvalue( inext )
                  endif
                endif
                doscal = .true.

*-----------------------------------------------------------------------
              else if ( subcmd( 'TRUNCATE' ) ) then
*-----------------------------------------------------------------------

                if ( numsubarg .lt. 1 ) then
                  call errmsg ( 'Missing TRUNCATE argument' , istop )
                  goto 50
                endif

                chr = ' '
                inext = cmndpos(icmnd) + subcmndpos(jcmnd) + 2
                if ( dtyp( inext ) .eq. 2 ) then
                  dens_max = fvalue( inext )
                  trunclev = dens_max
                  dotrnc = 1
                else
                  call errmsg ( 'Bad TRUNCATE "from" argument' , istop )
                  goto 50
                endif
                if ( numsubarg .ge. 2 ) then
                  inext = inext + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    trunclev = fvalue( inext )
                  else
                    call errmsg ( 'Bad TRUNCATE "to" argument' , 
     &              istop )
                    goto 50
                  endif
                endif
                if ( numsubarg .ge. 3 ) then
                  inext = inext + 1
                  if ( nextwd( 'SIGMA' ) ) then
                    dotrnc = 2
                    chr = 'sigma'
                  else
                    call errmsg ( 'Unknown flag following TRUNCATE ' //
     &              'argument - valid : s(igma)' , istop )
                    call rsinit ( 'DENS_MAX' )
                    goto 50
                  endif
                endif

*-----------------------------------------------------------------------
              else if ( subcmd( 'TYPE' ) ) then
*-----------------------------------------------------------------------

                if ( numsubarg .lt. 1 ) then
                  call errmsg ( 'Missing TYPE argument' , istop )
                  goto 50
                endif
                inext = cmndpos(icmnd) + subcmndpos(jcmnd) + 2 
                ib = ibeg( inext )
                ie = iend( inext )
                if ( nextwd( 'CCP4' ) ) then
                  pattyp = 'CCP4'
                else if ( nextwd( 'PROTEIN' ) ) then
                  pattyp = 'PROTEIN'
                else
                  call errmsg ( 'Illegal Patterson format > ' //
     &            line( ib:ie ) , istop )
                  goto 50
                endif

*-----------------------------------------------------------------------
              endif
*-----------------------------------------------------------------------
*=======================================================================
            end do
*=======================================================================
            endif

*       Report
            chr = ' '
            if ( dotrnc .eq. 2 ) chr = 'sigma'
            write(6,2020)patfil( :slnblk ( patfil ) )
            write(6,2025)pattyp
            write(6,2030)patscl
            if ( dotrnc .ne. 0 ) write(6,2035)dens_max,trunclev,chr
            do j = 1 , rstnum
            write(6,2040)(rstuvw(k,j),k=1,3),
     &      rstrad(j),rstlev(j)
            end do

*       Read, truncate, reset and scale map
            call getpat ( errcod )
            if ( errcod .ne. 0 ) then
              call errmsg ( 'GETPAT failed' , istop )
              goto 50
            endif
            patnotin = .false.
            patisin = .true.
            dftgrd = .true.
            cmndin( cmndnum( icmnd ) ) = .true.
            goto 7000

          endif            


*-----------------------------------------------------------------------
*-------PICK------------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'PICK' ) ) then

*       No arguments
            if ( numarg .eq. 0 ) then
              call ttyptl ( ' RSPS PICK > legal construct is' )
              call ttyptl ( 
     &        ' PICK < SCOREMAP (n) | PATTERSON (n) (HARKER) > ' )
              call ttyptl ( ' (LEVEL level) ' )
              call ttyptl ( ' (LIMITS xi xf yi yf zi zf) ' )
              call ttyptl ( ' ({EXCLUDE {<X|Y|Z>} value}) ' )
              goto 50
            endif

*       At least one argument

*       Initiate PICK variables
            call rsinit ( 'PICK' )
            call rsinit ( 'PKLIMT' )
            call rsinit ( 'PKXCLD' )
            call rsinit ( 'PICKLEVEL' )

*       Subcomands present - find location in string
*       First get expanded subcommands
            inext = cmndpos(icmnd) + 1
            call keyin ( line , ibeg( inext ) , iend( inext ) ,
     &      dtyp( inext ) , numarg , pikcmndlst , pikcmndtot ,
     &      maxcmnd , 'NONE' , subcmnd , subcmndnum , subcmndpos , 
     &      subcmndcnt )

            if ( subcmndcnt .eq. 0 ) then
              call errmsg ( 'Illegal PICK command > ' //
     &        line( ibeg(cmndpos(icmnd)):
     &              iend(cmndpos(icmnd)+numarg) ) , istop )
              call ttyptl ( ' RSPS PICK > legal construct is' )
              call ttyptl ( 
     &        ' PICK < SCOREMAP (n) | PATTERSON (n) (HARKER) > ' )
              call ttyptl ( ' (LEVEL level) ' )
              call ttyptl ( ' (LIMITS xi xf yi yf zi zf) ' )
              call ttyptl ( ' ({EXCLUDE {<X|Y|Z>} value}) ' )
              goto 50
            endif

*=======================================================================
*       Interpret subcommand stream
            do jcmnd = 1 , subcmndcnt
*=======================================================================

*       First subcommand must be SCOREMAP or PATTERSON
              if ( jcmnd .eq. 2 .and. .not.
     &        dopisc .and. .not. dopipa ) then
                call errmsg ( 'Illegal PICK command > PICK ' //
     &          line ( ibeg(cmndpos(icmnd)+1):
     &                 iend(cmndpos(icmnd)+numarg) ) , istop )
                call ttyptl ( ' RSPS PICK > legal construct is' )
                call ttyptl ( 
     &          ' PICK < SCOREMAP (n) | PATTERSON (n) (HARKER) > ' )
                call ttyptl ( ' (LEVEL level) ' )
                call ttyptl ( ' (LIMITS xi xf yi yf zi zf) ' )
                call ttyptl ( ' ({EXCLUDE {<X|Y|Z>} value}) ' )
                goto 50
              endif

*       Compute number of (possible) data tokens present for this
*       command word
              if ( jcmnd .eq. subcmndcnt ) then
                numsubarg = numarg - subcmndpos(jcmnd)
              else
                numsubarg = subcmndpos(jcmnd+1) - subcmndpos(jcmnd) - 1
              endif

*-----------------------------------------------------------------------
              if ( subcmd( 'SCOREMAP' ) ) then
*-----------------------------------------------------------------------

                if ( numsubarg .ge. 1 ) then
                  inext = cmndpos(icmnd) + subcmndpos(jcmnd) + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    npick = abs ( fvalue( inext ) )
                    if ( npick .gt. mxpick ) npick = mxpick
                  else
                    call errmsg ( 'Bad PICK SCOREMAP argument' , istop )
                    goto 50
                  endif
                endif

                dopisc = .true.

*-----------------------------------------------------------------------
              else if ( subcmd( 'PATTERSON' ) ) then
*-----------------------------------------------------------------------

                hrkonl = .false.
                if ( numsubarg .ge. 1 ) then
                  inext = cmndpos(icmnd) + subcmndpos(jcmnd) + 1
                  if ( nextwd( 'HARKER' ) ) then
                    hrkonl = .true.
                  else if ( dtyp( inext ) .eq. 2 ) then
                    npick = fvalue( inext )
                    if ( npick .gt. mxpick ) npick = mxpick
                  else
                    call errmsg ( 'Bad PICK PATTERSON argument' , 
     &              istop )
                    goto 50
                  endif
                endif
                if ( numsubarg .ge. 2 ) then
                  inext = inext + 1
                  if ( nextwd( 'HARKER' ) ) then
                    hrkonl = .true.
                  else 
                    call errmsg ('Bad PICK PATTERSON argument' ,
     &              istop )
                    goto 50
                  endif
                endif

                dopipa = .true.

*-----------------------------------------------------------------------
              else if ( subcmd( 'EXCLUDE' ) ) then
*-----------------------------------------------------------------------

                if ( numsubarg .lt. 2 ) then
                  call errmsg ( 'PICK EXCLUDE: too few arguments' ,
     &            istop )
                  goto 50
                endif

                inext = cmndpos(icmnd) + subcmndpos(jcmnd) + 1
                if ( dtyp( inext ) .eq. 1 ) then
*       First argument should be 1-3 character string {<X|Y|Z>}
                  ib = ibeg( inext )
                  ie = iend( inext )
                  call supcas ( line( ib:ie ) )
                else
                  call errmsg ( 'Illegal PICK EXCLUDE argument >' //
     &            line( ib:ie ) , istop )
                  goto 50
                endif

*       Second argument should be value
                if ( dtyp( inext + 1 ) .ne. 2 ) then
                  call errmsg ( 'Illegal PICK EXCLUDE argument' //
     &            '(non-numeric where numeric expected)', istop )
                  goto 50
                endif
 
*       Arguments have correct type
*       Step through three first characters of string and find if 
*       current character is X, Y or Z
                do k = 1 , min ( 3 , ie - ib + 1 )
                  indx = index ( 'XYZ' , line( ib-1+k:ib-1+k ) )
                  if ( indx .le. 0 ) then
                    call errmsg ( 'Illegal PICK EXCLUDE argument > ' 
     &              // line( ib:ie ) , istop )
                    goto 50
                  endif
                  nxcl = nxcl + 1
                  if ( nxcl .gt. mxexcl ) then
                    call errmsg ( 'Too many exclude zones generated' , 
     &              istop )
                    nxcl = mxexcl
                    goto 50
                  endif
                  xclpnt( nxcl ) = indx
                  pkxcld( nxcl ) = fvalue( inext + 1 )
                end do

*-----------------------------------------------------------------------
              else if ( subcmd( 'LIMITS' ) ) then
*-----------------------------------------------------------------------

                if ( numsubarg .lt. 6 ) then
                  call errmsg ( 'PICK LIMITS: too few arguments' ,
     &            istop )
                  goto 50
                endif

                inext = cmndpos(icmnd) + subcmndpos(jcmnd)
                do k = 1 , 6
                  inext = inext + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    pkltfr( k ) = fvalue( inext )
                  else
                    call errmsg ( 'Bad PICK LIMITS argument' , istop )
                    call rsinit ( 'PKLIMT' )
                    goto 50
                  endif
                end do

*-----------------------------------------------------------------------
              else if ( subcmd( 'LEVEL' ) ) then
*-----------------------------------------------------------------------

                if ( numsubarg .lt. 1 ) then
                  call errmsg ( 'PICK LEVEL: too few arguments' ,
     &            istop )
                  goto 50
                endif

                inext = cmndpos(icmnd) + subcmndpos(jcmnd) + 1
                if ( dtyp( inext ) .eq. 2 ) then
                  pick_level = fvalue( inext )
                else
                  call errmsg ( 'Bad PICK LEVEL arguments' , istop )
                  goto 50
                endif

*-----------------------------------------------------------------------
              endif
*-----------------------------------------------------------------------
*=======================================================================
            end do
*=======================================================================

            if ( dopisc ) then
              write(6,2250)npick
            else if ( dopipa ) then
              write(6,2251)npick
            else
              call errmsg ( 'Illegal PICK command > PICK ' //
     &        line ( ibeg(cmndpos(icmnd)+1):
     &               iend(cmndpos(icmnd)+numarg) ) , istop )
              goto 50
            endif
            if ( nxcl .gt. 0 ) then
              write(6,2252)nxcl
              do j = 1 , nxcl
                write(6,2253) xlabel( xclpnt(j) ) , pkxcld(j)
              end do
            endif
            if ( pkltfr(1) .gt. pkltfr(2) .and.
     &      pkltfr(3) .gt. pkltfr(4) .and.
     &      pkltfr(5) .gt. pkltfr(6) ) then
              write(6,2254)
            else
              write(6,2255)pkltfr
            endif
            if ( pick_level .ne. 0 ) write(6,2256)pick_level
            cmndin( cmndnum( icmnd ) ) = .true.
            goto 8000

          endif


*-----------------------------------------------------------------------
*-------POLARSCAN-------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'POLARSCAN' ) ) then

            if ( numarg .le. 1 ) then
              call errmsg ( 'Missing or illegal POLARSCAN arguments > '
     &        // line( ibeg(cmndpos(icmnd)):
     &                 iend(cmndpos(icmnd)+numarg) ) , istop )
              call ttyptl ( ' RSPS POLARSCAN > legal construct is' )
              call ttyptl ( ' POALRSCAN <POSITIONS n1 (n2) | x y z>' )
              goto 50
            endif

*       Init
            call rsinit ( 'POLARSCAN' )

            inext = cmndpos( icmnd ) + 1 

*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
            if ( nextwd( 'POSITIONS' ) ) then
*       Get positions to translate from stored positions by specifying 
*       position number
*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*       Quit if no positions stored
              if ( xintot .eq. 0 ) then
                call errmsg ( ' RSPS >> POLARSCAN POSITIONS >> ' //
     &          'No positions stored' , istop )
                goto 50
              endif

*       Check space in POLARSCAN array
              if ( ntrnps .eq. mxpick ) then
                call errmsg ( 'POLARSCAN array full' , istop )
                goto 50
              endif

*       Get pointers
              ipi = 1
              ipf = 1
              call getpnt ( ipi , ipf , xintot , dtyp , fvalue ,
     &        cmndpos( icmnd ) + 2 , numarg - 1 , errcod )
              if ( errcod .ne. 0 ) goto 50

*       Copy coordinates from XPICK array
              do j = ipi , ipf
                ntrnps = ntrnps + 1
                if ( ntrnps .le. mxpick ) then
                  call matcop ( xpick( 1,j ) , xscan( 1,ntrnps) , 
     &            3 , 1 )
                endif
              end do


*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
            else if ( numarg .ge. 3 ) then
*       At least three arguments and failure to interpret first arg as
*       subcommand; attempt interpretation as coordinates
*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
              if ( ntrnps .lt. mxpick ) then
                ntrnps = ntrnps + 1
                inext = cmndpos( icmnd )
                do j = 1 , 3
                  inext = inext + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    xscan( j,ntrnps ) = fvalue( inext )
                  else
                    call errmsg ( 'Bad POLARSCAN argument' , istop )
                    goto 50
                  endif
                end do
              else
                call wrnmsg ( 'POLARSCAN array full - not stored' )
                goto 50
              endif

*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
            else
*       Couldn't interpret arguments
*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
              ib = ibeg( cmndpos( icmnd ) + 1 )
              ie = iend( cmndpos( icmnd ) + numarg )
              call supcas ( line( ib:ie ) )
              call errmsg ( 'Strange POLARSCAN argument > ' // 
     &        line(ib:ie) , istop )
              call ttyptl ( ' RSPS POLARSCAN > possible ' //
     &        'arguments are <POSITIONS n1 (n2) | x y z>' )
              goto 50

*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
            endif
*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

*       If legal input we end up here >

*       Check we have fixed position(s)
            if ( nxyz2 .eq. 0 ) then
              call errmsg ( 'No fixed positions stored (use FIXXYZ' //
     &        ' to select fixed positions)' , istop )
              goto 50
            endif

*       Check and report trapped overflow attempt
            if ( ntrnps .gt. mxpick ) then
              ntrnps = mxipsn
              call wrnmsg ( 'POLARSCAN array full' )
              call wrnmsi ( 'Last stored position > ' ,
     &        ipi + nxyz1 - 1 , 4 )
            endif

            dotran = .true.
            write(6,2267)ntrnps
            cmndin( cmndnum( icmnd ) ) = .true.
            goto 8000

          endif


*-----------------------------------------------------------------------
*-------PRINT-----------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'PRINT' ) ) then
            if ( numarg .ge. 1 ) then
            inext = cmndpos( icmnd ) + 1 
            if ( dtyp( inext ) .eq. 2 ) then
              iprt = fvalue( inext )
            else
              call errmsg ( 'Bad PRINT argument' , istop )
              goto 50
            endif
            endif
            if ( iprt .lt. 0 .or. iprt .gt. 3 ) iprt = df_iprt
            cmndin( cmndnum( icmnd ) ) = .true.
            write(6,2210)iprt
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------READ------------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'READ' ) ) then

*       Get file name
            if ( numarg .ge. 1 ) then
              inext = cmndpos( icmnd ) + 1 
              ib = ibeg( inext )
              ie = iend( inext )
              xinfil = line ( ib:ie )
            endif

*       Report
            write(6,2280)xinfil( :slnblk ( xinfil ) )

*       Set combined symmetry operators
            call lodsym ( .false. , errcod )
            if ( errcod .ne. 0 ) then
              call errmsg ( 'LODSYM failed: symmetry error' , istop )
              goto 50
            endif

*       Read coordinates; assign site numbers
            call getcrd ( errcod )
            if ( errcod .ne. 0 ) then
              call errmsg ( 'Couldn''t read coordinates' , istop )
              goto 50
            endif

            cmndin( cmndnum( icmnd ) ) = .true.
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------REJECT----------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'REJECT' ) ) then
            if ( numarg .ge. 1 ) then
              inext = cmndpos( icmnd ) + 1 
              if ( dtyp( inext ) .eq. 2 ) then
                rej = fvalue( inext )
              else
                call errmsg ('Bad REJECT argument' , istop )
                goto 50
              endif
            endif
            write(6,2110)rej
            reject = rej * patsig + patave
            cmndin ( cmndnum( icmnd ) ) = .true.
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------ROTATE----------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'ROTATE' ) ) then

            if ( numarg .eq. 0 ) then
              call ttyptl ( ' RSPS ROTATE > legal construct is ' )
              call ttyptl ( ' ROTATE <POSITIONS n1 (n2) | FIXXYZ ' //
     &        'n1 (n2)> <SGS n1 (n2) | NCS n1 (n2) | MATRIX ' //
     &        'r11...r33 (t1 t2 t3)> (FIXXYZ (n1))' )
              goto 7000
            endif

            if ( numarg .lt. 4 ) then
*       Not enough arguments
              call errmsg ( 'Invalid ROTATE command > ROTATE ' //
     &        line( ibeg(cmndpos(icmnd)+1):
     &              iend(cmndpos(icmnd)+numarg) ) , 0 )
              call errmsg ( 'Missing arguments' , istop )
              goto 50
            endif


*       Locate subcommands
            iposarg  = 0
            irotarg  = 0
            iputarg  = 0
            nposarg  = 0
            nrotarg  = 0
            possel   = ' '
            rotsel   = ' '
            iput     = 0
            npos     = 0
            nrot     = 0
            inext    = cmndpos( icmnd )
            do jcmnd = 1 , numarg

              inext = inext + 1
              if ( nextwd( 'POSITIONS' ) ) then

                if ( possel .eq. ' ' ) then
                  iposarg = jcmnd
                  possel = 'POS'
                  npos = xintot
                else
                  call errmsg ( 'Invalid ROTATE command > ROTATE ' //
     &            line( ibeg(cmndpos(icmnd)+1):
     &                  iend(cmndpos(icmnd)+numarg) ) , istop )
                  goto 50
                endif

              else if ( nextwd( 'FIXXYZ' ) ) then

                if ( possel .eq. ' ' ) then
                  iposarg = jcmnd
                  possel = 'FIX'
                  npos = nxyz2
                else if ( iputarg .eq. 0 ) then
                  iputarg = jcmnd
                else
                  call errmsg ( 'Invalid ROTATE command > ROTATE ' //
     &            line( ibeg(cmndpos(icmnd)+1):
     &                  iend(cmndpos(icmnd)+numarg) ) , istop )
                  goto 50
                endif

              else if ( nextwd( 'SGS' ) ) then

                if ( rotsel .eq. ' ' ) then
                  irotarg = jcmnd
                  rotsel = 'SGS'
                  nrot = nopsgs
                else
                  call errmsg ( 'Invalid ROTATE command > ROTATE ' //
     &            line( ibeg(cmndpos(icmnd)+1):
     &                  iend(cmndpos(icmnd)+numarg) ) , istop )
                  goto 50
                endif

              else if ( nextwd( 'NCS' ) ) then

                if ( rotsel .eq. ' ' ) then
                  irotarg = jcmnd
                  rotsel = 'NCS'
                  nrot = nopncs
                else
                  call errmsg ( 'Invalid ROTATE command > ROTATE ' //
     &            line( ibeg(cmndpos(icmnd)+1):
     &                  iend(cmndpos(icmnd)+numarg) ) , istop )
                  goto 50
                endif

              else if ( nextwd( 'MATRIX' ) ) then

                if ( rotsel .eq. ' ' ) then
                  irotarg = jcmnd
                  rotsel = 'MTX'
                  nrot = 1
                else
                  call errmsg ( 'Invalid ROTATE command > ROTATE ' //
     &            line( ibeg(cmndpos(icmnd)+1):
     &                  iend(cmndpos(icmnd)+numarg) ) , istop )
                  goto 50
                endif

              endif

            end do


*       Check that valid subcommands have been given
            if ( possel .eq. ' ' ) then
              call errmsg ( 'Invalid ROTATE command > ROTATE ' //
     &        line( ibeg(cmndpos(icmnd)+1):
     &              iend(cmndpos(icmnd)+numarg) ) , 0 )
              call errmsg ( 'Missing position specification' , istop )
              goto 50
            endif
            if ( rotsel .eq. ' ' ) then
              call errmsg ( 'Invalid ROTATE command > ROTATE ' //
     &        line( ibeg(cmndpos(icmnd)+1):
     &              iend(cmndpos(icmnd)+numarg) ) , 0 )
              call errmsg ( 'Missing symmetry specification' , istop )
              goto 50
            endif

*       Get number of arguments to position subcommand
            nposarg = irotarg - iposarg - 1
            if ( nposarg .le. 0 ) then
              call errmsg ( 'Invalid ROTATE command > ROTATE ' //
     &        line( ibeg(cmndpos(icmnd)+1):
     &              iend(cmndpos(icmnd)+numarg) ) , istop )
              goto 50
            endif

*       Get number of arguments to symmetry subcommand
            if ( iputarg .ne. 0 ) then
              nrotarg = iputarg - irotarg - 1
            else
              nrotarg = numarg - irotarg
            endif
            if ( rotsel .ne. 'MTX' .and. nrotarg .le. 0 ) then
              call errmsg ( 'Invalid ROTATE command > ROTATE ' //
     &        line( ibeg(cmndpos(icmnd)+1):
     &              iend(cmndpos(icmnd)+numarg) ) , istop )
              goto 50
            else if ( rotsel .eq. 'MTX' .and. nrotarg .lt. 9 ) then
              call errmsg ( 'Invalid ROTATE command > ROTATE ' //
     &        line( ibeg(cmndpos(icmnd)+1):
     &              iend(cmndpos(icmnd)+numarg) ) , istop )
              goto 50
            endif


*       Get position pointers
            ipi = 1
            ipf = 1
            call getpnt ( ipi , ipf , npos , dtyp , fvalue ,
     &      cmndpos( icmnd ) + iposarg + 1 , nposarg , errcod )
            if ( errcod .ne. 0 ) goto 50

*       Get symmetry pointers or read matrix
            if ( rotsel .ne. 'MTX' ) then
              iri = 1
              irf = 1
              call getpnt ( iri , irf , nrot , dtyp , fvalue ,
     &        cmndpos( icmnd ) + irotarg + 1 , nrotarg , errcod )
              if ( errcod .ne. 0 ) goto 50
            else
*       Read rotation matrix row by row
              inext = cmndpos( icmnd ) + irotarg
              call matclr ( rtmp , 3 , 3 )
              call matclr ( ttmp , 3 , 1 )
              do k = 1 , 3
                do j = 1 , 3
                  inext = inext + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    rtmp( k,j ) = fvalue( inext )
                  else
                    call errmsg ( 'Bad ROTATE MATRIX argument' , 
     &              istop )
                    goto 50
                  endif
                end do
              end do
*       Read translation components
              if ( nrotarg .ge. 12 ) then
                do j = 1 , 3
                  inext = inext + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    ttmp( j ) = fvalue( inext )
                  else
                    call errmsg ( 'Bad ROTATE MATRIX argument' , 
     &              istop )
                    goto 50
                  endif
                end do
              endif
            endif

*       Get pointer to start of storage location in FIXXYZ array
            if ( iputarg .ne. 0 ) then
              iput = nxyz2 + 1
              if ( numarg .gt. iputarg ) then
                inext = cmndpos( icmnd ) + iputarg + 1
                if ( dtyp( inext ) .eq. 2 ) then
                  iput = fvalue( inext )
                  if ( iput .le. nxyz2 ) nxyz2 = iput - 1
                else
                  call errmsg ( 'Bad ROTATE argument' , istop )
                  goto 50
                endif
              endif 
            endif


*       Rotate point(s)
            do j = ipi , ipf
              if ( possel .eq. 'POS' ) then
                call matcop ( xpick( 1,j ) , xtmp1 , 3 , 1 )
              else if ( possel .eq. 'FIX' ) then
                call matcop ( xyz2( 1,j ) , xtmp1 , 3 , 1 )
              endif
              if ( rotsel .ne. 'MTX' ) then
                do k = iri , irf
                  if ( possel .eq. 'POS' .and. rotsel .eq. 'SGS' ) then
                    write(6,2500)j,k
                  endif
                  if ( possel .eq. 'POS' .and. rotsel .eq. 'NCS' ) then
                    write(6,2510)j,k
                  endif
                  if ( possel .eq. 'FIX' .and. rotsel .eq. 'SGS' ) then
                    write(6,2520)j,k
                  endif
                  if ( possel .eq. 'FIX' .and. rotsel .eq. 'NCS' ) then
                    write(6,2530)j,k
                  endif
                  if ( rotsel .eq. 'SGS' ) then
                    call matcop ( r_sgs( 1,1,k ) , rtmp , 3 , 3 )
                    call matcop ( t_sgs( 1,k ) , ttmp , 3 , 1 )
                  else if ( rotsel .eq. 'NCS' ) then
                    call matcop ( r_ncs( 1,1,k ) , rtmp , 3 , 3 )
                    call matcop ( t_ncs( 1,k ) , ttmp , 3 , 1 )
                  endif
                  call vmtply ( rtmp , xtmp1 , xtmp2 )
                  call vecadd ( xtmp2 , ttmp , xtmp3 , 3 )
                  do l = 1 , 3
                    xtmp3( l ) = mod ( xtmp3( l ) , 1.0 )
                    if ( xtmp3( l ) .lt. 0.0 ) xtmp3( l ) = 
     &              xtmp3( l ) + 1.0
                  end do
                  if ( iputarg .ne. 0 ) then
                    if ( iput .gt. mxipsn ) then
                      call errmsg ( 'FIXXYZ array full, ' //
     &                'position not stored' , istop )
                    else
                      call matcop ( xtmp3 , xyz2( 1,iput ) , 3 , 1 )
                      nxyz2 = nxyz2 + 1
                      iput = iput + 1
                    endif
                  endif
                  if ( iputarg .eq. 0 ) then
                    write(6,2540)xtmp3
                  else
                    write(6,2550)xtmp3,iput - 1
                  endif
                end do
              else
                if ( possel .eq. 'POS' ) write(6,2560)j
                if ( possel .eq. 'FIX' ) write(6,2570)j
                call vmtply ( rtmp , xtmp1 , xtmp2 )
                call vecadd ( xtmp2 , ttmp , xtmp3 , 3 )
                do l = 1 , 3
                  xtmp3( l ) = mod ( xtmp3( l ) , 1.0 )
                  if ( xtmp3( l ) .lt. 0.0 ) xtmp3( l ) = 
     &            xtmp3( l ) + 1.0
                end do
                if ( iputarg .ne. 0 ) then
                  if ( iput .gt. mxipsn ) then
                    call errmsg ( 'FIXXYZ array full, ' //
     &              'position not stored' , istop )
                  else
                    call matcop ( xtmp3 , xyz2( 1,iput ) , 3 , 1 )
                    nxyz2 = nxyz2 + 1
                    iput = iput + 1
                  endif
                endif
                if ( iputarg .eq. 0 ) then
                  write(6,2540)xtmp3
                else
                  write(6,2550)xtmp3,iput - 1
                endif
              endif
            end do

            cmndin( cmndnum( icmnd ) ) = .true.
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------SCAN------------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'SCAN' ) ) then

*       Init variables
            call matclr ( scrlim , 6 , 1 )
            dftgrd = .true.
            call rsinit ( 'FORCE' )
            if ( ispgrp .gt. 0 ) then
              call stlim ( ispgrp , scrlim )
              if ( scrlim(1) .lt. 0 ) then
                call errmsg ( 'Space group not found' , istop )
                goto 50
              endif
            else
              call errmsg ( 'Space group not defined' , istop )
              goto 50
            endif

            if ( numarg .gt. 0 ) then
*       At least one argument
*       First get expanded subcommands
            inext = cmndpos(icmnd) + 1
            call keyin ( line , ibeg( inext ) , iend( inext ) ,
     &      dtyp( inext ) , numarg , scncmndlst , scncmndtot ,
     &      maxcmnd , 'NONE' , subcmnd , subcmndnum , subcmndpos , 
     &      subcmndcnt )

            if ( subcmndcnt .eq. 0 ) then
              call errmsg ( 'Illegal SCAN command > ' //
     &        line( ibeg(cmndpos(icmnd)):
     &              iend(cmndpos(icmnd)+numarg) ) , istop )
              call ttyptl ( ' RSPS SCAN > legal construct is' )
              call ttyptl ( ' SCAN <AU | LIMITS xi xf yi yf ' //
     &        'zi zf> (GRID nx ny nz) (REQUIRE PEAK nr)' )
              goto 50
            endif

*=======================================================================
*       Interpret subcommand stream
            do jcmnd = 1 , subcmndcnt
*=======================================================================

*       Compute number of (possible) data tokens present for this
*       command word
              if ( jcmnd .eq. subcmndcnt ) then
                numsubarg = numarg - subcmndpos(jcmnd)
              else
                numsubarg = subcmndpos(jcmnd+1) - subcmndpos(jcmnd) - 1
              endif

*       Interpret
*-----------------------------------------------------------------------
              if ( subcmd( 'AU' ) ) then
*-----------------------------------------------------------------------

                if ( ispgrp .gt. 0 ) then
                  call stlim ( ispgrp , scrlim )
                  if ( scrlim(1) .lt. 0 ) then
                    call errmsg ( 'Space group not found' , istop )
                    goto 50
                  endif
                else
                  call errmsg ( 'Space group not defined' , istop )
                  goto 50
                endif

*-----------------------------------------------------------------------
              else if ( subcmd( 'GRID' ) ) then
*-----------------------------------------------------------------------

                if ( numsubarg .lt. 3 ) then
                  call errmsg ( 'Missing GRID arguments' , istop )
                  goto 50
                endif

                inext = cmndpos(icmnd) + subcmndpos(jcmnd)
                do k=1,3
                  inext = inext + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    scrgrd( k ) = fvalue( inext )
                  else
                    call errmsg ( 'Bad GRID argument' , istop )
                    call imatclr ( scrgrd , 3 , 1 )
                    goto 50
                  endif
                end do
                dftgrd = .false.


*-----------------------------------------------------------------------
              else if ( subcmd( 'LIMITS' ) ) then
*-----------------------------------------------------------------------

                if ( numsubarg .lt. 6 ) then
                  call errmsg ( 'Missing LIMITS arguments' , istop )
                  goto 50
                endif

                inext = cmndpos(icmnd) + subcmndpos(jcmnd)
                do k=1,6
                  inext = inext + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    scrlim( k ) = fvalue( inext )
                  else
                    call errmsg ( 'Bad LIMITS arguments' , istop )
                    call matclr ( scrlim , 6 , 1 )
                    goto 50
                  endif
                end do

                do k = 1 , 3
                  if ( scrlim( 2*k ) .lt. scrlim( 2*k-1 ) ) then
                    call errmsg ( 'Strange SCAN limits' , istop )
                    call matclr ( scrlim , 6 , 1 )
                    goto 50 
                  endif
                  if ( abs ( scrlim( 2*k ) - scrlim( 2*k-1 ) ) 
     &            .gt. 2.01 ) then
                    call errmsg ( 'Strange SCAN limits' , istop )
                    call ttyptl 
     &              ( ' That''s a huge search cell! I don''t ' //
     &              'belive you want that!' )
                     call matclr ( scrlim , 6 , 1 )
                    goto 50
                  endif
                end do

*-----------------------------------------------------------------------
              else if ( subcmd( 'REQUIRE' ) ) then
*-----------------------------------------------------------------------

                if ( numsubarg .lt. 2  ) then
                  call errmsg ( 'Missing REQUIRE arguments' , istop )
                  goto 50
                endif

*       Attempt interpretation as PEAK nr
                if ( numsubarg .eq. 2 ) then
                  inext = cmndpos(icmnd) + subcmndpos(jcmnd) + 1
                  ib = ibeg( inext )
                  ie = iend( inext )
                  if ( nextwd( 'PEAK' ) ) then
*       Quit if no peaks stored
                    if ( ppnum .eq. 0 ) then
                      call errmsg ( ' RSPS >> SCAN REQUIRE PEAK ' //
     &                '>> No peaks stored' , istop )
                      goto 50
                    endif
*       Get peak pointer
                    inext = inext + 1
                    if ( dtyp( inext ) .eq. 2 ) then
                      ipi =  fvalue( inext )
                      if ( ipi .le. 0 .or. ipi .gt. ppnum ) then
                        call errmsg ( 'PEAK number out of range: ' //
     &                  line( ib:iend( inext ) ) , istop )
                        goto 50
                      endif
                    else
                      call errmsg ( 'Illegal argument: ' //
     &                line( ib:iend( inext ) ) , istop )
                      goto 50
                    endif
*       Set forced-vector flag and store peak coordinates
                    force = .true.
                    call rmatcop ( ppuvw( 1,ipi ) , frcpek , 3 , 1 )

                  else

                    call errmsg ( 'Illegal argument: ' //
     &              line( ib:iend( inext ) ) , istop )
                    goto 50

                  endif
                else
                  call errmsg ( 'Illegal argument: ' //
     &            line( ib:iend( inext ) ) , istop )
                  goto 50
                endif


*-----------------------------------------------------------------------
              endif
*-----------------------------------------------------------------------
*=======================================================================
            end do
*=======================================================================
            endif

            doscan = .true.
            cmndin( cmndnum( icmnd ) ) = .true.
            if ( force ) write(6,2080)frcpek
            write(6,2090)scrlim
            if ( .not. dftgrd ) write(6,2095)scrgrd
            goto 8000
          endif


*-----------------------------------------------------------------------
*-------SCORE-----------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'SCORE' ) ) then

            if ( numarg .ge. 1 ) then
*       Look for subcommands

              inext = cmndpos( icmnd ) + 1 
*-----------------------------------------------------------------------
              if ( nextwd( 'SUM' ) ) then
*-----------------------------------------------------------------------
                minfun = .false.
                scrtyp = 1
                if ( numarg .ge. 2 ) then
                inext = inext + 1
                if ( dtyp( inext ) .eq. 2 ) then
                  nuse = fvalue( inext )
                  minfun = .true.
                else
                  call errmsg ( 'Bad SCORE SUM argument' , istop )
                  goto 50
                endif
                endif
                if ( minfun ) write(6,2175)nuse
                write(6,2160)


*-----------------------------------------------------------------------
              else if ( nextwd( 'PRODUCT' ) ) then
*-----------------------------------------------------------------------
                minfun = .false.
                scrtyp = 2
                if ( numarg .ge. 2 ) then
                inext = inext + 1
                if ( dtyp( inext ) .eq. 2 ) then
                  nuse = fvalue( inext )
                  minfun = .true.
                else
                  call errmsg ( 'Bad SCORE PRODUCT argument' , istop )
                  goto 50
                endif
                endif
                if ( minfun ) write(6,2175)nuse
                write(6,2165)

*-----------------------------------------------------------------------
              else if ( nextwd( 'HARMONIC' ) ) then
*-----------------------------------------------------------------------
                minfun = .false.
                scrtyp = 3
                if ( numarg .ge. 2 ) then
                inext = inext + 1
                if ( dtyp( inext ) .eq. 2 ) then
                  nuse = fvalue( inext )
                  minfun = .true.
                else
                  call errmsg ( 'Bad SCORE HARMONIC argument' , istop )
                  goto 50
                endif
                endif
                if ( minfun ) write(6,2175)nuse
                write(6,2170)

*-----------------------------------------------------------------------
              else if ( nextwd( 'MINIMUM' ) ) then
*-----------------------------------------------------------------------
                minfun = .true.
                if ( numarg .ge. 2 ) then
                inext = inext + 1
                if ( dtyp( inext ) .eq. 2 ) then
                  nuse = fvalue( inext )
                else
                  call errmsg ( 'Bad SCORE MINIMUM argument' , istop )
                  goto 50
                endif
                endif
                write(6,2175)nuse

*-----------------------------------------------------------------------
              else
*-----------------------------------------------------------------------
                ib = ibeg( inext )
                ie = iend( inext )
                call errmsg ( 'Strange SCORE argument > ' // 
     &          line(ib:ie) , istop )
                call ttyptl ( ' RSPS SCORE > possible arguments ' //
     &          'are SUM or PRODUCT or HARMONIC or MINIMUM (n)' )
                goto 50

*-----------------------------------------------------------------------
              endif
*-----------------------------------------------------------------------
            
            else

              if ( minfun ) then
                write(6,2180)scrlab( scrtyp ),nuse
              else
                write(6,2185)scrlab( scrtyp ) 
              endif

            endif

            cmndin ( cmndnum( icmnd ) ) = .true.
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------SCORFILE--------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'SCORFILE' ) ) then

*       No arguments - just report
            if ( numarg .eq. 0 ) then
              write(6,2050)scrfil( :slnblk ( scrfil ) )
              write(6,2060)(xlabel(scraxo(j)),j=1,3)
              write(6,2065)titut( :slnblk ( titut ) ) 
              goto 7000
            endif

*       At least one argument
*       Init variables
            call rsinit ( 'TITLE' )
            call rsinit ( 'IUVW2' )

*       First argument interpreted as file name
            inext = cmndpos( icmnd ) + 1
            ib = ibeg( inext )
            ie = iend( inext )
            scrfil = line ( ib:ie )

            if ( numarg .ge. 2 ) then
*       Additional subcomands present - find location in string

*       First get expanded subcommands
            inext = inext + 1
            call keyin ( line , ibeg( inext ) , iend( inext ) ,
     &      dtyp( inext ) , numarg-1 , scrcmndlst , scrcmndtot ,
     &      maxcmnd , 'NONE' , subcmnd , subcmndnum , subcmndpos , 
     &      subcmndcnt )

            if ( subcmndcnt .eq. 0 ) then
              call errmsg ( 'Illegal SCORFILE command > ' //
     &        line( ibeg(cmndpos(icmnd)):
     &              iend(cmndpos(icmnd)+numarg) ) , istop )
              call ttyptl ( ' RSPS SCORFILE > legal construct is' )
              call ttyptl ( ' SCORFILE filename (SECT fast slow ' //
     &        'sect) (TITLE string)' )
              goto 50
            endif

*=======================================================================
*       Interpret subcommand stream
            do jcmnd = 1 , subcmndcnt
*=======================================================================

*       Compute number of (possible) data tokens present for this
*       command word
              if ( jcmnd .eq. subcmndcnt ) then
                numsubarg = numarg - subcmndpos(jcmnd) - 1
              else
                numsubarg = subcmndpos(jcmnd+1) - subcmndpos(jcmnd) - 1
              endif

*       Interpret
*-----------------------------------------------------------------------
              if ( subcmd( 'SECT' ) ) then
*-----------------------------------------------------------------------

                if ( numsubarg .lt. 3 ) then
                  call errmsg ( 'Missing SECT arguments' , istop )
                  goto 50
                endif

                inext = cmndpos(icmnd) + subcmndpos(jcmnd) + 1
                do k = 1 , 3
                  inext = inext + 1
                  ib = ibeg( inext )
                  ie = iend( inext )
                  call supcas ( line( ib:ie ) )
                  iuvw = index ( secfl , line( ib:ie ) )
                  if ( iuvw .ge. 1 .and. iuvw .le. 3 ) then
                    scraxo( k ) = iuvw
                  else
                    call errmsg ( 'Bad SECT argument' , istop )
                    call rsinit ( 'IUVW2' )
                    goto 50
                  endif
                end do
                if ( scraxo( 1 ) .eq. scraxo( 2 ) .or.
     &          scraxo( 1 ) .eq. scraxo( 3 ) .or.
     &          scraxo( 2 ) .eq. scraxo( 3 ) ) then
                  call errmsg ( 'SECT flags must all be different' , 
     &            istop )
                  call rsinit ( 'IUVW2' )
                  goto 50
                endif

*-----------------------------------------------------------------------
              else if ( subcmd( 'TITLE' ) ) then
*-----------------------------------------------------------------------

                if ( numsubarg .lt. 1 ) then
                  call wrnmsg ( 'Missing TITLE argument - ' //
     &            'title not changed' )
                else
                  inext = cmndpos(icmnd) + subcmndpos(jcmnd) + 2
                  ib = ibeg( inext )
                  ie = iend( inext + numsubarg - 1 )
                  titut = line( ib:ie )
                endif

*-----------------------------------------------------------------------
              endif
*-----------------------------------------------------------------------
*=======================================================================
            end do
*=======================================================================
            endif

            write(6,2050)scrfil( :slnblk ( scrfil ) )
            write(6,2060)(xlabel(scraxo(j)),j=1,3)
            write(6,2065)titut( :slnblk ( titut ) ) 
            cmndin( cmndnum( icmnd ) ) = .true.
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------SGSYMM - space group symmetry-----------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'SGSYMM' ) ) then
            if ( numarg .ge. 1 ) then
              inext = cmndpos( icmnd ) + 1 
              if ( nextwd( 'ON' ) ) then
                usesgs = .true.
              else if ( nextwd( 'OFF' ) ) then
                usesgs = .false.
              else
                call errmsg ( 'Illegal SGSYMM command' , istop )
                goto 50
              endif
            endif
            if ( usesgs ) then
              write(6,2230)
            else
              write(6,2235)
            endif
            cmndin ( cmndnum( icmnd ) ) = .true.
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------SPACEGROUP------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'SPACEGROUP' ) ) then

            if ( numarg .eq. 0 ) then
              write(6,2201)spgnam( :slnblk ( spgnam ) )
              goto 7000
            endif

            inext = cmndpos( icmnd ) + 1 
            if ( dtyp( inext ) .eq. 2 ) then
              ispgrp = fvalue( inext )
              spgnam = ' '
              if ( ispgrp .le. 0 ) then
                call errmsg ( 'Illegal space group number' , istop )
                ispgrp = 0
                goto 50
              endif
            else
              ib = ibeg( inext )
              ie = iend( inext )
              spgnam = line( ib:ie )
              call supcas ( spgnam )
              ispgrp = 0
            endif
            if ( ispgrp .ne. 0 ) then
              write(6,2200)ispgrp
            else
              write(6,2201)spgnam( :slnblk ( spgnam ) )
            endif
*      Load space group symmetry
            call getsgs ( errcod )
            if ( errcod .ne. 0 ) then
              call errmsg ( 'GETSGS failed' , istop )
              goto 50
            endif

            cmndin ( cmndnum( icmnd ) ) = .true.
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------STATUS----------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'STATUS' ) ) then

            write(6,2390)
            write(6,*)
            write(6,'(a)')
     &      ' RSPS >> Patterson map                 >  ' // 
     &      patfil( :slnblk ( patfil ) )
            write(6,'(a)') 
     &      ' RSPS >> Patterson map format          >  ' // pattyp
            write(6,'(a,2f8.2)')
     &      ' RSPS >> Patterson map scale           > ' , patscl
            write(6,'(a,3i4)')
     &      ' RSPS >> Patterson map grid            > ' , patgrd
            if ( dotrnc .ne. 0 ) then
            write(6,'(a,2f6.2)')
     &      ' RSPS >> Patterson map truncation      > ' , 
     &      dens_max , trunclev
            endif
            write(6,'(a,i2)')
     &      ' RSPS >> Number of modified peaks      > ' , rstnum
            write(6,'(a,f12.4)')
     &      ' RSPS >> Minimum density               > ' , patmin
            write(6,'(a,f12.4)')
     &      ' RSPS >> Maximum density               > ' , patmax
            write(6,'(a,f12.4)')
     &      ' RSPS >> Average density               > ' , patave
            write(6,'(a,f12.4)')
     &      ' RSPS >> RMS of density                > ' , patsig

            call ttycr ( 2 )
            call ttygtl ( ' Hit RETURN to continue > ' ,
     &      junkln , errcod )
            call ttycr ( 2 )

            write(6,*)
            write(6,'(a)') 
     &      ' RSPS >> Space group                   > ' // spgnam
            write(6,'(a,6f8.2)') 
     &      ' RSPS >> Unit cell parameters          > ' , cell
            if ( usesgs ) then
            write(6,'(a)')
     &      ' RSPS >> Space group symmetry          >  on'
            else
            write(6,'(a)')
     &      ' RSPS >> Space group symmetry          >  off'
            endif
            if ( usencs ) then
            write(6,'(a)')
     &      ' RSPS >> Non crystallographic symmetry >  on'
            else
            write(6,'(a)')
     &      ' RSPS >> Non crystallographic symmetry >  off'
            endif
            write(6,'(a)')
     &      ' RSPS >> Vectorset                     >  ' // vecset

            call ttycr ( 2 )
            call ttygtl ( ' Hit RETURN to continue > ' ,
     &      junkln , errcod )
            call ttycr ( 2 )

            write(6,*)
            write(6,'(a)')
     &      ' RSPS >> Score map                     >  ' // 
     &      scrfil( :slnblk ( scrfil ) )
            write(6,'(a)')
     &      ' RSPS >> Score map title               >  ' //
     &      titut( :slnblk ( titut ) )
            write(6,'(a,3i4)')
     &      ' RSPS >> Score map grid                > ' , scrgrd
            write(6,'(a)')
     &      ' RSPS >> Score map sectioning          >  ' //
     &      xlabel( scraxo(1) ) // xlabel( scraxo(2) ) //
     &      xlabel( scraxo(3) )
            if ( minfun ) then
              write(6,'(a,i2)')
     &        ' RSPS >> Scoring function              >  ' //
     &        scrlab( scrtyp ) // ' MINIMUM ' , nuse
            else
              write(6,'(a)')
     &        ' RSPS >> Scoring function              >  ' //
     &        scrlab( scrtyp )
            endif
            write(6,'(a,f12.4)')
     &      ' RSPS >> Minimum density               > ' , scomin
            write(6,'(a,f12.4)')
     &      ' RSPS >> Maximum density               > ' , scomax
            write(6,'(a,f12.4)')
     &      ' RSPS >> Average density               > ' , scoave
            write(6,'(a,f12.4)')
     &      ' RSPS >> RMS of density                > ' , scosig

            call ttycr ( 2 )
            call ttygtl ( ' Hit RETURN to continue > ' ,
     &      junkln , errcod )
            call ttycr ( 2 )

            write(6,'(a,i3)')
     &      ' RSPS >> Tolerance                     > ' , pektol
            write(6,'(a,f5.1)')
     &      ' RSPS >> Bump                          > ' , bump
            if ( special ) then
              write(6,'(a)')' RSPS >> Special positions allowed'
            else
              write(6,'(a)')' RSPS >> Special positions not allowed'
            endif
            write(6,'(a,f5.1,a)')
     &      ' RSPS >> Reject                        > ' , rej , 
     &      ' sigma'
            write(6,'(a,i3)')
     &      ' RSPS >> Low                           > ' , maxlow
            write(6,'(a,i3)')
     &      ' RSPS >> Weight                        > ' , imlt

            write(6,*)
            write(6,'(a)')
     &      ' RSPS >> Coordinate output file        >  ' // 
     &      xutfil( :slnblk ( xutfil ) )
            write(6,'(a)')
     &      ' RSPS >> Coordinate input file         >  ' // 
     &      xinfil( :slnblk ( xinfil ) )

            write(6,*)
            write(6,'(a,i4)')
     &      ' RSPS >> Number of stored peaks        > ' , ppnum
            write(6,'(a,i4)')
     &      ' RSPS >> Number of stored positions    > ' , xintot
            write(6,'(a,i4)')
     &      ' RSPS >> Number of stored sites        > ' , sittot
            write(6,'(a,i4)')
     &      ' RSPS >> Number of stored sets         > ' , settot
            write(6,'(a,i4)')
     &      ' RSPS >> Number of FIXXYZ positions    > ' , nxyz2
            write(6,'(a,i4)')
     &      ' RSPS >> Number of SGS operations      > ' , nopsgs
            write(6,'(a,i4)')
     &      ' RSPS >> Number of NCS operations      > ' , nopncs


            cmndin( cmndnum( icmnd ) ) = .true.
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------SUBTRACT--------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'SUBTRACT' ) ) then

*       Quit if no FIXXYZ sites stored
            if ( nxyz2 .eq. 0 ) then
              call wrnmsg ( 'RSPS SUBTRACT > currently vectors can ' //
     &        'only be subtracted from FIXXYZ positions' )
              call errmsg ( 'No FIXXYZ sites stored' , istop )
              goto 50
            endif

            if ( numarg .ge. 3 ) then

              inext = cmndpos( icmnd ) + 1 
              ib = ibeg( inext )
              ie = iend( inext )
              call supcas ( line( ib:ie ) )
              if ( index ( 'FIXXYZ' , line( ib:ie ) ) .ne. 1 ) then
                call errmsg ( 'RSPS SUBTRACT > currently vectors ' //
     &          'can only be subtracted from FIXXYZ positions' , istop )
                goto 50
              endif

              inext = inext + 1
              ib = ibeg( inext )
              ie = iend( inext )
              if ( nextwd( 'POSITION' ) ) then
                inext = inext + 1
                if ( dtyp( inext ) .eq. 2 ) then
                  ipi =  fvalue( inext )
                  if ( ipi .le. 0 .or. ipi .gt. xintot ) then
                    call errmsg ( 'POSITION number out of range: ' //
     &              line( ib:iend( inext ) ) , istop )
                    goto 50
                  endif
*       Subtract position ipi from FIXXYZ coords
                  write(6,2820)ipi,(xpick(j,ipi),j=1,3)
                  do j = 1 , nxyz2
                    call rvecdif ( xyz2( 1,j ) , xpick( 1,ipi ) , 
     &              xtmp1 , 3 )
                    call rvecmod ( xtmp1 , xtmp1 , 1.0 , 3 )
                    write(6,2825)(xyz2(k,j),k=1,3),(xpick(k,ipi),k=1,3),
     &              xtmp1
                    call rmatcop ( xtmp1 , xyz2( 1,j ) , 3 , 1 )
                  end do
                else
                  call errmsg ( 'Illegal argument: ' //
     &            line( ib:iend( inext ) ) , istop )
                  goto 50
                endif

              else if ( nextwd( 'VECTOR' ) ) then
                if ( numarg .lt. 5 ) then
                  call errmsg ( 'Missing arguments: ' // 
     &            line( ib:slnblk( line ) ) , istop ) 
                  goto 50
                endif
                do j = 1 , 3
                  inext = inext + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    xtmp1( j ) = fvalue( inext )
                  else
                    call errmsg ( 'Illegal argument: ' //
     &              line( ib:iend( inext ) ) , istop )
                    goto 50
                  endif
                end do
*       Subtract vector from FIXXYZ coords
                write(6,2830)(xtmp1(j),j=1,3)
                do j = 1 , nxyz2
                  call rvecdif ( xyz2( 1,j ) , xtmp1 , xtmp2 , 3 )
                  call rvecmod ( xtmp2 , xtmp2 , 1.0 , 3 )
                  write(6,2825)(xyz2(k,j),k=1,3),xtmp1,xtmp2
                  call rmatcop ( xtmp2 , xyz2( 1,j ) , 3 , 1)
                end do

              else
                call errmsg ( 'Strange SUBTRACT argument > ' //
     &          line( ib:ie ) , istop ) 
                call ttyptl ( ' RSPS SUBTRACT > possible arguments ' //
     &          'are FIXXYZ POSITION n or FIXXYZ VECTOR u v w' )
                goto 50
              endif

              cmndin( cmndnum( icmnd ) ) = .true.

            else
              call ttyptl ( ' RSPS SUBTRACT > possible arguments ' //
     &        'are FIXXYZ POSITION n or FIXXYZ VECTOR u v w' )
            endif
            goto 7000
          endif



*-----------------------------------------------------------------------
*-------TABLE-----------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'TABLE' ) ) then

            if ( numarg .ge. 1 ) then
              inext = cmndpos( icmnd ) + 1 

*-----------------------------------------------------------------------
              if ( nextwd( 'SETS' ) ) then
*-----------------------------------------------------------------------
*       Quit if no sets stored
                if ( settot .eq. 0 ) then
                  call ttyptl ( ' RSPS >> TABLE SETS >> ' //
     &            'No sets stored' )
                  goto 7000
                endif

*       Get pointers
                if ( numarg .ge. 2 ) then
                  call getpnt ( ipi , ipf , settot , dtyp , fvalue ,
     &            cmndpos( icmnd ) + 2 , numarg - 1 , errcod )
                  if ( errcod .ne. 0 ) goto 50
                else
                  ipi = 1
                  ipf = 1
                endif

*       Write header
                write(6,2370)

*       Get missing stuff
                call lodsym ( .false. , errcod )
                if ( errcod .ne. 0 ) then
                  call errmsg ( 'LODSYM failed: symmetry error' , 
     &            istop )
                  goto 50
                endif

*       Loop over sets and cumpute and list score table
                do j = ipi , ipf
                  nmem = setsiz( j )
                  do k = 1 , nmem
                    call rmatcop ( xpick(1,setpos(k,j)) , xscr(1,k) ,
     &              3 , 1 )
                  end do
                  call scrtab ( j , nmem , setpos(1,j) , xscr , 1 , 
     &            errcod )
                end do
                cmndin( cmndnum( icmnd ) ) = .true.


*-----------------------------------------------------------------------
              else if ( nextwd( 'FIXXYZ' ) ) then
*-----------------------------------------------------------------------
*       Quit if no fixed positions stored
                if ( nxyz2 .eq. 0 ) then
                  call ttyptl ( ' RSPS >> TABLE FIXXYZ >> ' //
     &            'No fixed positions stored' )
                  goto 7000
                endif

*       Write header
                write(6,2372)

*       Get missing stuff
                call lodsym ( .false. , errcod )
                if ( errcod .ne. 0 ) then
                  call errmsg ( 'LODSYM failed: symmetry error' , 
     &            istop )
                  goto 50
                endif

*       Get score table
                call rmatcop ( xyz2 , xscr , 3 , nxyz2 )
                do j = 1 , nxyz2
                  pscr(j) = j
                end do
                call scrtab ( 0 , nxyz2 , pscr , xscr , 1 , errcod )
                cmndin( cmndnum( icmnd ) ) = .true.


*-----------------------------------------------------------------------
              else if ( nextwd( 'POSITIONS' ) ) then
*-----------------------------------------------------------------------
                if ( xintot .eq. 0 ) then
                  call ttyptl ( ' RSPS >> TABLE POSITIONS >> ' //
     &            'No positions stored' )
                  goto 7000
                endif

                nmem = 0
                do j = 2 , numarg
                  inext = inext + 1
                  if ( dtyp( inext ) .eq. 2 ) then
                    nmem = nmem + 1
                    if ( nmem .le. mxgmem ) then
                      ipi = fvalue( inext )
                      if ( ipi .le. 0 ) then
                        call errmsg ( 'Strange TABLE POSITIONS number', 
     &                  istop )
                        goto 50
                      endif
                      if ( ipi .gt. xintot ) then
                        call errmsg ( 'TABLE POSITIONS number out ' //
     &                  'of range' , istop )
                        goto 50
                      endif
                      usrset( nmem ) = ipi
                    else
                      call wrnmsg ( 'TABLE array full > list ' // 
     &                'truncated' )
                    endif
                  else
                    call errmsg ( 'Bad TABLE argument' , istop )
                    goto 50
                  endif
                end do

                if ( nmem .le. 0 ) then
                  call errmsg ( 'Missing TABLE POSITIONS argument' ,
     &            istop )
                  goto 50
                endif

*       Write header
                write(6,2375)

*       Get missing stuff
                call lodsym ( .false. , errcod )
                if ( errcod .ne. 0 ) then
                  call errmsg ( 'LODSYM failed: symmetry error' , 
     &            istop )
                  goto 50
                endif

*       Compute and list score table
                do j = 1 , nmem
                  call rmatcop ( xpick(1,usrset(j)) , xscr(1,j) ,
     &            3 , 1 )
                end do
                call scrtab ( 0 , nmem , usrset , xscr , 1 , errcod )
                cmndin( cmndnum( icmnd ) ) = .true.

*-----------------------------------------------------------------------
              else
*-----------------------------------------------------------------------
                inext = cmndpos( icmnd ) + 1 
                ib = ibeg( inext )
                ie = iend( inext )
                call errmsg ( 'Strange TABLE argument > ' //
     &          line( ib:ie ) , istop ) 
                call ttyptl ( ' RSPS TABLE > possible arguments ' //
     &          'are SETS or POSITIONS or FIXXYZ' )
                goto 50

*-----------------------------------------------------------------------
              endif
*-----------------------------------------------------------------------

            else

              call ttyptl ( ' RSPS TABLE > possible arguments are ' //
     &        'SETS or POSITIONS or FIXXYZ' )
              goto 50

            endif

            goto 7000

          endif


*-----------------------------------------------------------------------
*-------TOLERANCE-------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'TOLERANCE' ) ) then
            if ( numarg .ge. 1 ) then
              inext = cmndpos( icmnd ) + 1 
              if ( dtyp( inext ) .eq. 2 ) then
                pektol = fvalue( inext )
              else
                call errmsg ( 'Bad TOLERANCE argument' , istop )
                goto 50
              endif
            endif
            cmndin ( cmndnum( icmnd ) ) = .true.
            write(6,2120)pektol
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------VECTORSET-------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'VECTORSET' ) ) then

            if ( numarg .eq. 1 ) then

              inext = cmndpos( icmnd ) + 1 
              if ( nextwd( '?' ) ) then
                call ttyptl ( ' RSPS > Possible vectorsets are ' )
              else
                call errmsg ( 'Illegal or incomplete vectorset > ' // 
     &          line( ibeg(cmndpos(icmnd)+1):
     &                iend(cmndpos(icmnd)+1) ) , istop )
                call ttyptl ( ' RSPS > Possible vectorsets are ' )
              endif
              call ttyptl ( ' RSPS > VECTORSET SINGLE    ATOMS ' )
              call ttyptl ( ' RSPS > VECTORSET MORE      ATOMS ' )
              call ttyptl ( ' RSPS > VECTORSET TRANSLATE ATOMS ' )
              call ttyptl ( ' RSPS > VECTORSET SINGLE    MOLECULES ' )
              call ttyptl ( ' RSPS > VECTORSET MORE      MOLECULES ' )
              call ttyptl ( ' RSPS > VECTORSET TRANSLATE MOLECULES ' )
              goto 7000

            else if ( numarg .ge. 2 ) then
              inext = cmndpos( icmnd ) + 1 
              if ( nextwd( 'SINGLE' ) ) then
                inext = inext + 1 
*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                if ( nextwd( 'ATOMS' ) ) then
*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                  if ( vecset .ne. 'SINGLE ATOMS' ) newset = .true.
                  vecset = 'SINGLE ATOMS'
                  cmndin( cmndnum( icmnd ) ) = .true.
                  write(6,2220)vecset
                  goto 7000
                else if ( nextwd( 'MOLECULES' ) ) then
                  if ( vecset .ne. 'SINGLE MOLECULES' ) newset = .true.
                  vecset = 'SINGLE MOLECULES'
                  cmndin( cmndnum( icmnd ) ) = .true.
                  write(6,2220)vecset
                  goto 7000
                else
                  call errmsg ( 'Illegal vectorset > ' // 
     &            line( ibeg(cmndpos(icmnd)+1):
     &                  iend(cmndpos(icmnd)+2) ) , istop )
                  call ttyptl ( ' RSPS > Possible vectorsets are ' )
                  call ttyptl ( ' RSPS > VECTORSET SINGLE    ATOMS ' )
                  call ttyptl ( ' RSPS > VECTORSET MORE      ATOMS ' )
                  call ttyptl ( ' RSPS > VECTORSET TRANSLATE ATOMS ' )
                  call ttyptl ( ' RSPS > VECTORSET SINGLE    ' //
     &            'MOLECULES ' )
                  call ttyptl ( ' RSPS > VECTORSET MORE      ' //
     &            'MOLECULES ' )
                  call ttyptl ( ' RSPS > VECTORSET TRANSLATE ' //
     &            'MOLECULES ' )
                  goto 50
                endif

*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
              else if ( nextwd( 'MORE' ) ) then
*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                inext = inext + 1 
                if ( nextwd( 'ATOMS' ) ) then
                  if ( vecset .ne. 'MORE ATOMS' ) newset = .true.
                  vecset = 'MORE ATOMS'
                  cmndin( cmndnum( icmnd ) ) = .true.
                  write(6,2220)vecset
                  goto 7000
                else if ( nextwd( 'MOLECULES' ) ) then
                  if ( vecset .ne. 'MORE MOLECULES' ) newset = .true.
                  vecset = 'MORE MOLECULES'
                  cmndin( cmndnum( icmnd ) ) = .true.
                  write(6,2220)vecset
                  goto 7000
                else
                  call errmsg ( 'Illegal vectorset > ' // 
     &            line( ibeg(cmndpos(icmnd)+1):
     &                  iend(cmndpos(icmnd)+2) ) , istop )
                  call ttyptl ( ' RSPS > Possible vectorsets are ' )
                  call ttyptl ( ' RSPS > VECTORSET SINGLE    ATOMS ' )
                  call ttyptl ( ' RSPS > VECTORSET MORE      ATOMS ' )
                  call ttyptl ( ' RSPS > VECTORSET TRANSLATE ATOMS ' )
                  call ttyptl ( ' RSPS > VECTORSET SINGLE    ' //
     &            'MOLECULES ' )
                  call ttyptl ( ' RSPS > VECTORSET MORE      ' //
     &            'MOLECULES ' )
                  call ttyptl ( ' RSPS > VECTORSET TRANSLATE ' //
     &            'MOLECULES ' )
                  goto 50
                endif

*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
              else if ( nextwd( 'TRANSLATE' ) ) then
*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                inext = inext + 1 
                if ( nextwd( 'ATOMS' ) ) then
                  if ( vecset .ne. 'TRANSLATE ATOMS' ) newset = .true.
                  vecset = 'TRANSLATE ATOMS'
                  cmndin( cmndnum( icmnd ) ) = .true.
                  write(6,2220)vecset
                  goto 7000
                else if ( nextwd( 'MOLECULES' ) ) then
                  if ( vecset .ne. 'TRANSLATE MOLECULES' ) newset = 
     &            .true.
                  vecset = 'TRANSLATE MOLECULES'
                  cmndin( cmndnum( icmnd ) ) = .true.
                  write(6,2220)vecset
                  goto 7000
                else
                  call errmsg ( 'Illegal vectorset > ' // 
     &            line( ibeg(cmndpos(icmnd)+1):
     &                  iend(cmndpos(icmnd)+2) ) , istop )
                  call ttyptl ( ' RSPS > Possible vectorsets are ' )
                  call ttyptl ( ' RSPS > VECTORSET SINGLE    ATOMS ' )
                  call ttyptl ( ' RSPS > VECTORSET MORE      ATOMS ' )
                  call ttyptl ( ' RSPS > VECTORSET TRANSLATE ATOMS ' )
                  call ttyptl ( ' RSPS > VECTORSET SINGLE    ' //
     &            'MOLECULES ' )
                  call ttyptl ( ' RSPS > VECTORSET MORE      ' //
     &            'MOLECULES ' )
                  call ttyptl ( ' RSPS > VECTORSET TRANSLATE ' //
     &            'MOLECULES ' )
                  goto 50
                endif

*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
              else
*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                call errmsg ( 'Illegal vectorset > ' // 
     &          line( ibeg(cmndpos(icmnd)+1):
     &                iend(cmndpos(icmnd)+2) ) , istop )
                call ttyptl ( ' RSPS > Possible vectorsets are ' )
                call ttyptl ( ' RSPS > VECTORSET SINGLE    ATOMS ' )
                call ttyptl ( ' RSPS > VECTORSET MORE      ATOMS ' )
                call ttyptl ( ' RSPS > VECTORSET TRANSLATE ATOMS ' )
                call ttyptl ( ' RSPS > VECTORSET SINGLE    MOLECULES ' )
                call ttyptl ( ' RSPS > VECTORSET MORE      MOLECULES ' )
                call ttyptl ( ' RSPS > VECTORSET TRANSLATE MOLECULES ' )

*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
              endif
*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

            else
              write(6,2220)vecset
              goto 7000
            endif
          endif


*-----------------------------------------------------------------------
*-------VLIST-----------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'VLIST' ) ) then

            if ( numarg .eq. 0 ) then
              call ttyptl ( ' RSPS VLIST > possible arguments ' //
     &        'are SITES or SETS or POSITIONS or FIXXYZ or x y z' )
              goto 50
            endif

*       Init position pointers and vlist variables
            ipi = 1
            ipf = 1
            call rsinit ( 'VLIST' )

*       Are there any free slots in vlist array?
            if ( nxyz1 .eq. mxipsn ) then
              call errmsg ( 'VLIST array full' , istop )
              goto 50
            endif

*       Check first argument is uniquely interpretable
            inext = cmndpos( icmnd ) + 1 
            ib = ibeg( inext )
            ie = iend( inext )
            call supcas ( line( ib:ie ) )
            if ( line( ib:ie ) .eq. 'S' ) then
              call errmsg ( 'Ambiguous VLIST argument > S', istop )
              call ttyptl ( ' RSPS VLIST > possible arguments ' //
     &        'are SITES or SETS or POSITIONS or FIXXYZ or x y z' )
              goto 50
            endif
              


*       Look for subcommands >
*-----------------------------------------------------------------------
            if ( nextwd( 'SITES' ) ) then
*       Get vlist positions from stored positions by specifying 
*       site number
*-----------------------------------------------------------------------
*       Quit if no sites stored
              if ( sittot .eq. 0 ) then
                call ttyptl ( ' RSPS >> VLIST SITES >> ' //
     &          'No sites stored' )
                goto 7000
              endif

*       Get pointers
              call getpnt ( ipi , ipf , sittot , dtyp , fvalue , 
     &        cmndpos( icmnd ) + 2 , numarg - 1 , errcod )
              if ( errcod .ne. 0 ) goto 50

*       Copy coordinates from XPICK array
              do 70 j = ipi , ipf
                nxyz1 = nxyz1 + 1
                if ( nxyz1 .le. mxipsn ) then
                  call matcop ( xpick( 1,sitbeg(j) ) , 
     &            xyz1( 1,nxyz1 ) , 3 , 1 )
                  write(6,2191)(xyz1(k,nxyz1),k=1,3),sitbeg(j)
                endif
 70           continue


*-----------------------------------------------------------------------
            else if ( nextwd( 'POSITIONS' ) ) then
*       Get vlist positions from stored positions by specifying 
*       position number
*-----------------------------------------------------------------------
*       Quit if no positions stored
              if ( xintot .eq. 0 ) then
                call ttyptl ( ' RSPS >> VLIST POSITIONS >> ' //
     &          'No positions stored' )
                goto 7000
              endif

*       Get pointers
              call getpnt ( ipi , ipf , xintot , dtyp , fvalue ,
     &        cmndpos( icmnd ) + 2 , numarg - 1 , errcod )
              if ( errcod .ne. 0 ) goto 50

*       Copy coordinates from XPICK array
              do 80 j = ipi , ipf
                nxyz1 = nxyz1 + 1
                if ( nxyz1 .le. mxipsn ) then
                  call matcop ( xpick( 1,j ) , xyz1( 1,nxyz1 ) , 
     &            3 , 1 )
                  write(6,2191)(xyz1(k,nxyz1),k=1,3),j
                endif
 80           continue


*-----------------------------------------------------------------------
            else if ( nextwd( 'SETS' ) ) then
*       Get vlist positions from stored positions by specifying 
*       set number
*-----------------------------------------------------------------------
*       Quit if no sets stored
              if ( settot .eq. 0 ) then
                call ttyptl ( ' RSPS >> VLIST SETS >> ' //
     &          'No sets stored' )
                goto 7000
              endif

*       Get pointers
              call getpnt ( ipi , ipf , settot , dtyp , fvalue ,
     &        cmndpos( icmnd ) + 2 , numarg - 1 , errcod )
              if ( errcod .ne. 0 ) goto 50

*       Copy coordinates from XPICK array
              do 90 j = ipi , ipf
                do k = 1 , setsiz( j )
                nxyz1 = nxyz1 + 1
                imem = setpos( k,j )
                if ( nxyz1 .le. mxipsn ) then
                  call matcop ( xpick( 1,imem ) , xyz1( 1,nxyz1 ) , 
     &            3 , 1 )
                  write(6,2191)(xyz1(l,nxyz1),l=1,3),imem
                endif
                end do
 90           continue


*-----------------------------------------------------------------------
            else if ( nextwd( 'FIXXYZ' ) ) then
*       Get vlist positions from FIXXYZ positions 
*-----------------------------------------------------------------------
*       Quit if no fixxyz positions stored
              if ( nxyz2 .eq. 0 ) then
                call ttyptl ( ' RSPS >> VLIST FIXXYZ >> ' //
     &          'No FIXXYZ positions stored' )
                goto 50
              endif

*       Get pointers
              ipi = 1
              ipf = nxyz2
              if ( numarg .gt. 1 ) then
                call getpnt ( ipi , ipf , nxyz2 , dtyp , fvalue ,
     &          cmndpos( icmnd ) + 2 , numarg - 1 , errcod )
                if ( errcod .ne. 0 ) goto 50
              endif

*       Copy coordinates from FIXXYZ array
              do 100 j = ipi , ipf
                nxyz1 = nxyz1 + 1
                if ( nxyz1 .le. mxipsn ) then
                  call matcop ( xyz2( 1,j ) , xyz1( 1,nxyz1 ) , 
     &            3 , 1 )
                  write(6,2192)(xyz1(k,nxyz1),k=1,3),j
                endif
 100          continue


*-----------------------------------------------------------------------
            else if ( numarg .ge. 3 ) then
*       At least three arguments and failure to interpret first arg as
*       subcommand; attempt interpretation as coordinates
*-----------------------------------------------------------------------

*       Get vlist positions from line
              if ( nxyz1 .lt. mxipsn ) then
                nxyz1 = nxyz1 + 1
                inext = cmndpos( icmnd )
                do j = 1 , 3
                inext = inext + 1
                if ( dtyp( inext ) .eq. 2 ) then
                  xyz1( j , nxyz1 ) = fvalue( inext )
                else
                  call errmsg ( 'Bad VLIST coordinates' , istop )
                  nxyz1 = nxyz1 - 1
                  goto 50
                endif
                end do
                write(6,2190)(xyz1(l,nxyz1),l=1,3)
              else
                call wrnmsg ( 'VLIST array full - not stored' )
                goto 50
              endif

*-----------------------------------------------------------------------
            else
*       Couldn't interpret arguments
*-----------------------------------------------------------------------
              call errmsg ( 'Bad VLIST arguments', istop )
              goto 50

*-----------------------------------------------------------------------
            endif
*-----------------------------------------------------------------------
*       If legal input we end up here >

*       Check and report trapped overflow attempt
            if ( nxyz1 .gt. mxipsn ) then
              nxyz1 = mxipsn
              call wrnmsg ( 'VLIST array full' )
              call wrnmsi ( 'Last stored position > ' ,
     &        ipi + nxyz1 - 1 , 4 )
            endif

            dovlis = .true.
            cmndin( cmndnum( icmnd ) ) = .true.
            goto 8000
          endif


*-----------------------------------------------------------------------
*-------WEIGHT----------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'WEIGHT' ) ) then
            if ( numarg .ge. 1 ) then
              inext = cmndpos( icmnd ) + 1 
              if ( dtyp( inext ) .eq. 2 ) then
                imlt = fvalue( inext )
                if ( imlt .lt. 1 .or. imlt .gt. 2 ) imlt = 1
                usemult = ( imlt .eq. 2 )
              else
                call errmsg ( 'Bad WEIGHT argument' , istop )
                goto 50
              endif
            endif
            cmndin( cmndnum( icmnd ) ) = .true.
            write(6,2150)imlt
            goto 7000
          endif


*-----------------------------------------------------------------------
*-------WRITE-----------------------------------------------------------
*-----------------------------------------------------------------------
          if ( topcmd( 'WRITE' ) ) then

*       Need unit cell for orthogonalization matrix
        if ( nint ( cell( 1 ) ) .eq. 0 ) then
           call ttyptl (' RSPS WRITE > Please enter unit cell first')
           goto 50
        endif


*       Find out what to write
            if ( numarg .ge. 1 ) then
              inext = cmndpos( icmnd ) + 1 
              ib = ibeg( inext )
              ie = iend( inext )
              call supcas ( line( ib:ie ) )
              if ( line( ib:ie ) .eq. 'P' ) then
                call errmsg ( 'Ambiguous WRITE argument > ' // 
     &          line( ib:ie ) , istop )
                call ttyptl ( ' RSPS WRITE > possible arguments ' //
     &          'are POSITIONS or PEAKS or SET or FIXXYZ' )
                goto 50
              endif

*-----------------------------------------------------------------------
              if ( nextwd( 'POSITIONS' ) ) then
*-----------------------------------------------------------------------
*       Do we have any positions?
                if ( xintot .eq. 0 ) then
                  call ttyptl ( ' RSPS >> WRITE POSITIONS >> No ' //
     &            'positions stored' )
                  goto 7000
                endif
*       Set output type flag
                output = 'POS'
*       Set default file name
                xutfil = 'positions.pdb'
*       Set title
                tit = title
*       Set number of positions
                nx = xintot
*       Copy coordinates to output array
                call rmatcop ( xpick , xscr , 3 , xintot )
*       Copy "value" (score) array
                call rmatcop ( value , vscr , 1 , xintot )
*       Copy site pointer array
                call imatcop ( sitpnt , pscr , 1 , xintot )

*-----------------------------------------------------------------------
              else if ( nextwd( 'PEAKS' ) ) then
*-----------------------------------------------------------------------
*       Do we have any peaks?
                if ( ppnum .eq. 0 ) then
                  call ttyptl ( ' RSPS >> WRITE PEAKS >> No peaks ' //
     &            'stored' )
                  goto 7000
                endif

*       Set output type flag
                output = 'PEA'
*       Set default file name
                xutfil = 'peaks.pdb'
*       Set title
                tit = 'RSPS >> peaks from Patterson map ' // 
     &          patfil( 1:slnblk(patfil))
*       Set number of positions
                nx = ppnum
*       Copy coordinates to output array
                call rmatcop ( ppuvw , xscr , 3 , ppnum )
*       Copy "value" (peak height) array
                call rmatcop ( ppval , vscr , 1 , ppnum )
*       Set site numbers to ordinal
                do j = 1 , ppnum
                  pscr( j ) = j
                end do


*-----------------------------------------------------------------------
              else if ( nextwd( 'SET' ) ) then
*-----------------------------------------------------------------------
*       Do we have any sets?
                if ( settot .eq. 0 ) then
                  call ttyptl ( ' RSPS >> WRITE SET >> No sets stored' )
                  goto 7000
                endif

*       If more arguments, interpret as set number, otherwise set set = 1
                if ( numarg .ge. 2 ) then
                  inext = cmndpos( icmnd ) + 2
                  if ( dtyp( inext ) .eq. 2 ) then
                    iset = fvalue( inext )
                    if ( iset .gt. settot ) then
                      call errmsg ( 'No such set' , istop )
                      goto 50
                    endif
                  else
                    call errmsg ( 'Bad WRITE SET argument' , istop )
                    goto 50
                  endif
                else
                  iset = 1
                end if

*       Set output type flag
                output = 'SET'
*       Set default file name
                xutfil = 'set.pdb'
*       Set title
                tit = 'RSPS >> getsets set'
*       Set number of positions
                nx = setsiz( iset )
*       Copy coordinates to output array
                do j = 1 , setsiz( iset )
                  call rmatcop ( xpick(1,setpos(j,iset)) , 
     &            xscr(1,j) , 3 , 1 )
*       Copy "value" (use set score) array
                  vscr( j ) = setscr( iset )
*       Set site pointer array
                  pscr( j ) = sitpnt(setpos(j,iset))
                end do

*-----------------------------------------------------------------------
              else if ( nextwd( 'FIXXYZ' ) ) then
*-----------------------------------------------------------------------
*       Do we have any fixxyz positions?
                if ( nxyz2 .eq. 0 ) then
                  call ttyptl ( ' RSPS >> WRITE FIXXYZ >> No fixed ' //
     &            'positions stored' )
                  goto 7000
                endif

*       Set output type flag
                output = 'FIX'
*       Set default file name
                xutfil = 'fixxyz.pdb'
*       Set title
                tit = 'RSPS >> fixed positions'
*       Set number of positions
                nx = nxyz2
*       Copy coordinates to output array
                call rmatcop ( xyz2 , xscr , 3 , nxyz2 )
*       Set "value" array arbitrarily to 1.0
                call rmatset ( vscr , 1 , nxyz2 , 1.0 )
*       Set site numbers to ordinal
                do j = 1 , nxyz2
                  pscr( j ) = j
                end do


*-----------------------------------------------------------------------
              else
*-----------------------------------------------------------------------
                call errmsg ( 'Bad WRITE argument', istop )
                goto 50

*-----------------------------------------------------------------------
              endif
*-----------------------------------------------------------------------

*       User specified file name   
              if ( numarg .gt. inext - cmndpos( icmnd ) ) then
                inext = inext + 1
                ib = ibeg( inext )
                ie = iend( inext )
                xutfil = line( ib:ie )
              endif

            else

*       No argument; write position coordinates
*       Do we have any positions?
              if ( xintot .eq. 0 ) then
                call ttyptl ( ' RSPS >> WRITE POSITIONS >> No ' //
     &          'positions stored' )
                goto 7000
              endif
*       Set output type flag
              output = 'POS'
*       Set file name
              xutfil = 'positions.pdb'
*       Set title
              tit = title
*       Set number of positions
              nx = xintot
*       Copy coordinates to output array
              call rmatcop ( xpick , xscr , 3 , xintot )
*       Copy "value" (score) array
              call rmatcop ( value , vscr , 1 , xintot )
*       Copy site pointer array
              call imatcop ( sitpnt , pscr , 1 , xintot )

            endif

*       Report
            if ( output .eq. 'POS' ) then
              write(6,2285)xutfil( :slnblk ( xutfil ) )
            else if ( output .eq. 'PEA' ) then
              write(6,2286)xutfil( :slnblk ( xutfil ) )
            else if ( output .eq. 'SET' ) then
              write(6,2287)iset,xutfil( :slnblk ( xutfil ) )
            else if ( output .eq. 'FIX' ) then
              write(6,2288)xutfil( :slnblk ( xutfil ) )
            endif

*       Write output file
            call rswpdb ( lunxut , xutfil( :slnblk ( xutfil ) ) , 
     &      nx , xscr , vscr , pscr , cell , tit , output , errcod )
            if ( errcod .ne. 0 ) then
              call errmsg ( 'Couldn''t write output pdb file' , 
     &        istop )
              goto 50
            endif

            cmndin( cmndnum( icmnd ) ) = .true.
            goto 7000
          endif



 7000     continue
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Next command in stream
 7500   continue
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       No more commands; read next command line
        goto 50
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=


*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*
*       Go > execute delayed action command
 8000   continue
*
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Write header
        if ( dopipa ) then
          junkln = 'RSPS PICK >> PATTERSON'
        else if ( dopisc ) then
          junkln = 'RSPS PICK >> SCOREMAP'
        else if ( dotran ) then
          junkln = 'RSPS POLARSCAN >>'
        else if ( dogets ) then
          junkln = 'RSPS GETSETS >>'
        else if ( vecset .eq. 'SINGLE ATOMS' ) then
          if ( dovlis ) junkln = 'RSPS SINGLE ATOM VLIST >>'
          if ( doscan ) junkln = 'RSPS SINGLE ATOM SCAN >>'
        else if ( vecset .eq. 'MORE ATOMS' ) then
          if ( dovlis ) junkln = 'RSPS MORE ATOMS VLIST >>'
          if ( doscan ) junkln = 'RSPS MORE ATOMS SCAN >>'
        else if ( vecset .eq. 'TRANSLATE ATOMS' ) then
          if ( dovlis ) junkln = 'RSPS TRANSLATE ATOMS VLIST >>'
          if ( doscan ) junkln = 'RSPS TRANSLATE ATOMS SCAN >>'
        else if ( vecset .eq. 'SINGLE MOLECULES' ) then
          if ( dovlis ) junkln = 'RSPS SINGLE MOLECULE VLIST >>'
          if ( doscan ) junkln = 'RSPS SINGLE MOLECULE SCAN >>'
        else if ( vecset .eq. 'MORE MOLECULES' ) then
          if ( dovlis ) junkln = 'RSPS MORE MOLECULES VLIST >>'
          if ( doscan ) junkln = 'RSPS MORE MOLECULES SCAN >>'
        else if ( vecset .eq. 'TRANSLATE MOLECULES' ) then
          if ( dovlis ) junkln = 'RSPS TRANSLATE MOLECULES VLIST >>'
          if ( doscan ) junkln = 'RSPS TRANSLATE MOLECULES SCAN >>'
        else
          call errmsg ( 'HOW THE FUCK DID I END UP HERE???', 1 )
        endif
        write(6,2000) junkln( :slnblk ( junkln ) )
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*
*       Check we know enough to carry out whatever.
*       Set some flags and switches.
*       Execute preliminaries.
*
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        if ( index ( vecset , 'MORE' ) .eq. 1 .and. nxyz2 .lt. 1 ) then
*       No fixed positions stored
          if ( doscan ) then
            call errmsg ( 'MORE ATOMS/MOLECULES SCAN > must have ' //
     &      'at least one fixed position (FIXXYZ)' , istop )
            call rsinit ( 'ACTIONS' )
            goto 50
          else if ( dovlis ) then
            call errmsg ( 'MORE ATOMS/MOLECULES VLIST > must have ' //
     &      'at least one fixed position (FIXXYZ)' , istop )
            call rsinit ( 'ACTIONS' )
            goto 50
          endif
        endif

        if ( index ( vecset , 'TRANSLATE' ) .eq. 1 .and. 
     &  nxyz2 .lt. 2 ) then
*       No fixed group stored
          if ( doscan ) then
            call errmsg ( 'TRANSLATE ATOMS/MOLECULES SCAN > ' //
     &      'must have at least two fixed positions (FIXXYZ)' , istop )
            call rsinit ( 'ACTIONS' )
            goto 50
          else if ( dovlis ) then
            call errmsg ( 'TRANSLATE ATOMS/MOLECULES VLIST > ' //
     &      'must have at least two fixed positions (FIXXYZ)' , istop )
            call rsinit ( 'ACTIONS' )
            goto 50
          endif
        endif

        if ( index ( vecset , 'MOLECULES' ) .ne. 0 ) then
*       Must have ncs stored and turned on for scan and vlist
          if ( ( doscan .or. dovlis ) ) then
            if ( nopncs .le. 1 ) then
              call errmsg ( vecset // ' VLIST/SCAN > non ' //
     &        'crystallographic symmetry must be defined' , istop )
              call rsinit ( 'ACTIONS' )
              goto 50
            endif
            usrncs  = usencs
            usencs  = .true.
*       Use large bump distance when locating molecules
*       Always allow special positions since it is fairly common
*       to have ncs molecules there
            if ( doscan ) then
              usrbmp  = bump
              usrspc  = special
              if ( bump .lt. df_molbmp ) then
                bump = df_molbmp
                write(6,2600)vecset,usrbmp,bump
              endif
              special = .true.
            endif
          endif
        endif

*       Check we have Patterson map stored
        if ( .not. dopisc ) then
          if ( patnotin ) then
            call errmsg ( 'Patterson map has not been read' , istop )
            call rsinit ( 'ACTIONS' )
            goto 50
          else
*       Check limits in case space group changed
            call chklim ( ispgrp , errcod )
            if ( errcod .ne. 0 ) then
              call errmsg ( 'Illegal Patterson map limits' , istop )
              call rsinit ( 'ACTIONS' )
              goto 50
            endif
          endif
        endif

*       Set default scan grid if required
        if ( dftgrd .and. .not. dopipa ) call stgrd

*       Load combined symmetry (and real to vector space transformations)
        if ( doscan .or. dotran .or. dovlis ) then
          call lodsym ( .true. , errcod )
          if ( errcod .ne. 0 ) then
            call errmsg ( 'LODSYM failed: symmetry error' , istop )
            call rsinit ( 'ACTIONS' )
            goto 50
          endif
        else if ( .not. dopipa ) then
          call lodsym ( .false. , errcod )
          if ( errcod .ne. 0 ) then
            call errmsg ( 'LODSYM failed: symmetry error' , istop )
            call rsinit ( 'ACTIONS' )
            goto 50
          endif
        endif

*       Check polar axes
        if ( .not. dopipa ) then
          call polaxs ( nopsym , r_sym , npolx , polar )
        endif

*       For POLARSCAN, set scan direction = polar axis
        if ( dotran ) then
          if ( npolx .ne. 1 ) then
            call errmsg ( 'POLARSCAN > symmetry not polar' , istop )
            call rsinit ( 'ACTIONS' )
            goto 50
          else
            do j = 1 , 3
              if ( polar( j ) .eq. 0 ) scndir = j
            end do
          endif
        endif


*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Set pointer to next command and
*       return to main for execution of delayed action commands
        icur = icmnd + 1
        return
*
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
 1000   format(a)
 2000   format(1x,'RSPS >> GO              >> ' , a )
 2020   format(1x,'RSPS >> PATFILE         >> Name       = ',a )
 2025   format(1x,'RSPS >> PATFILE         >> Type       = ',a )
 2030   format(1x,'RSPS >> PATFILE         >> Scale      = ',2f8.2)
 2035   format(1x,'RSPS >> PATFILE         >> Truncate   = ',2f8.2,a)
 2040   format(1x,'RSPS >> PATFILE         >> Reset uvw  = ',3f8.4,
     &  ' radius = ' ,f10.2,' level = ' , f10.2 )
 2050   format(1x,'RSPS >> SCORFILE        >> Name       = ',a )
 2060   format(1x,'RSPS >> SCOREFILE       >> Sectioning = ',
     &  a1,1x,a1,1x,a1)
 2065   format(1x,'RSPS >> SCOREFILE       >> Title      = ',a )
 2080   format(1x,'RSPS >> SCAN REQUIRE    >> Vector at (',3f8.4,' ) ',
     &  'will be required in scan' )
 2090   format(1x,'RSPS >> SCAN            >> limits = ',6f8.3)
 2095   format(1x,'RSPS >> SCAN            >> Grid       = ',3i5)
 2110   format(1x,'RSPS >> REJECT          >> ',f8.2,1x,' sigma')
 2120   format(1x,'RSPS >> TOLERANCE       >> ',i5)
 2130   format(1x,'RSPS >> LOW             >> ',i5)
 2140   format(1x,'RSPS >> BUMP            >> ',f8.2,
     &  '; special positions allowed')
 2145   format(1x,'RSPS >> BUMP            >> ',f8.2,
     &  '; special positions not allowed')
 2150   format(1x,'RSPS >> WEIGHT          >> ',i5)
 2160   format(1x,'RSPS >> SCORE SUM       >> sum',
     &  ' function selected' )
 2165   format(1x,'RSPS >> SCORE PRODUCT   >> product',
     &  ' function selected' )
 2170   format(1x,'RSPS >> SCORE HARMONIC  >> harmonic mean',
     &  ' function selected' )
 2175   format(1x,'RSPS >> SCORE MINIMUM   >> the minimum ',i2,
     &  ' peaks will be used in scoring' )
 2180   format(1x,'RSPS >> SCORE           >> ' , a , ' MINIMUM ',i2)
 2185   format(1x,'RSPS >> SCORE           >> ' , a )
 2190   format(1x,'RSPS >> VLIST           >> ',3f8.4)
 2191   format(1x,'RSPS >> VLIST           >> ',3f8.4,
     &  '       (position ',i3,')')
 2192   format(1x,'RSPS >> VLIST           >> ',3f8.4,
     &  '       (FIXXYZ position ',i3,')')
 2195   format(1x,'RSPS >> VLIST           >> ',i5,' positions stored' )
 2200   format(1x,'RSPS >> SPACEGROUP      >> ',i5)
 2201   format(1x,'RSPS >> SPACEGROUP      >> ',a)
 2210   format(1x,'RSPS >> PRINT           >> ',i5)
 2220   format(1x,'RSPS >> VECTORSET       >> vecset = ',a)
 2230   format(1x,'RSPS >> SGSYMM ON       >> ',
     &  'space group symmetry ON' )
 2235   format(1x,'RSPS >> SGSYMM OFF      >> ',
     &  'space group symmetry OFF' )
 2240   format(1x,'RSPS >> NCSYMM ON       >> local symmetry ON' )
 2241   format(1x,'RSPS >> NCSYMM OFF      >> local symmetry OFF' )
 2242   format(1x,'RSPS >> NCSROT          >> number of ncs ',
     &  'operations stored = ',i5)
 2243   format(/,1x,'RSPS >> NCSROT          >> symmetry operation # ',
     &  i5)
 2244   format(1x,'RSPS >> NCSROT MATRIX   >> ',3f8.4)
 2245   format(1x,'RSPS >> NCSROT POLAR    >> omega = ',f6.1,
     &  ' phi = ',f6.1,' kappa = ',f6.1,' translation = ',3f8.4)
 2246   format(1x,'RSPS >> NCSROT ORTHO    >> orthogonalization = ',
     &  i3)
 2250   format(1x,'RSPS >> PICK SCOREMAP   >> ',i5)
 2251   format(1x,'RSPS >> PICK PATTERSON  >> ',i5)
 2252   format(1x,'RSPS >> PICK EXCLUDE    >> ',i5,' zones defined')
 2253   format(1x,'RSPS >> PICK EXCLUDE    >> exclude ',a,' = ',f5.2)
 2254   format(1x,'RSPS >> PICK LIMITS     >> whole map' )
 2255   format(1x,'RSPS >> PICK LIMITS     >> ',6f8.4)
 2256   format(1x,'RSPS >> PICK LEVEL      >> ',f8.2)
 2260   format(1x,'RSPS >> FIXXYZ          >> ',3f8.4)
 2261   format(1x,'RSPS >> FIXXYZ          >> ',3f8.4,
     &  ' (site ',i3,' position ',i3,')')
 2262   format(1x,'RSPS >> FIXXYZ          >> ',3f8.4,
     &  ' (Patterson peak ',i3,')')
 2265   format(1x,'RSPS >> FIXXYZ          >> ',i5,' positions stored' )
 2267   format(1x,'RSPS >> POLARSCAN       >> number of positions ',
     &  'to translate = ',i5)
 2270   format(1x,'RSPS >> GETSETS         >> minimum set = ',i5,
     &  ',   positions = ',i5)
 2280   format(1x,'RSPS >> READ            >> ',
     &  'reading coordinates from file ' , a )
 2285   format(1x,'RSPS >> WRITE           >> ',
     &  'writing coordinates to file ' , a )
 2286   format(1x,'RSPS >> WRITE           >> ',
     &  'writing peaks to file ' , a )
 2287   format(1x,'RSPS >> WRITE           >> ',
     &  'writing set ',i2,' to file ' , a )
 2288   format(1x,'RSPS >> WRITE           >> ',
     &  'writing fixed positions to file ' , a )
 2310   format(1x,'RSPS >> DELETE VLIST    >> VLIST positions deleted' )
 2320   format(1x,'RSPS >> DELETE FIXXYZ   >> FIXXYZ position #',i3,
     &  ' deleted' )
 2322   format(1x,'RSPS >> DELETE FIXXYZ   >> FIXXYZ positions',
     &  'deleted' )
 2325   format(1x,'RSPS >> DELETE PEAKS    >> peaks deleted' )
 2326   format(1x,'RSPS >> DELETE NCS      >> ncs operators deleted' )
 2327   format(1x,'RSPS >> DELETE SETS     >> sets deleted' )
 2330   format(1x,'RSPS >> LIST POSITIONS  >> ',i5,' to ',i5,//12x,
     &  'PosnN      Fractional coordinates     Angstrom coordinates',
     &  '     CVScore  HVScore   Site',/12x,'-----     ',
     &  '------------------------  ------------------------   ',
     &  '-------  -------   ----')
 2335   format(1x,'RSPS >> LIST POSITIONS  >> ',i5,' to ',i5,//12x,
     &  'PosnN      Fractional coordinates     Angstrom coordinates',
     &  '      Score    Site',/12x,'-----     ',
     &  '------------------------  ------------------------  ',
     &  '---------  ----')
 2337   format(10x,i5,6x,3f8.4,2x,3f8.2,2x,f8.2,1x,f8.2,3x,i4)
 2338   format(10x,i5,6x,3f8.4,2x,3f8.2,2x,f8.2,4x,i4)
 2340   format(1x,'RSPS >> LIST PEAKS      >> ',i5,' to ',i5,//10x,
     &  'Peak       Fractional coordinates     Angstrom coordinates',
     &  '         Grid coordinates       Value      S/N',/10x,
     &  '----      ------------------------  ------------------------',
     &  '    ----------------------  ---------  -------')
 2345   format(8x,i5,1x,a1,4x,3f8.4,2x,3f8.2,2x,3i8,4x,f8.2,f8.1)
 2350   format(1x,'RSPS >> LIST SET        >> ' )
* 2355   format(10x,'Set ',i3,', score > ',f8.2,';',x,i2,' members > ',
*     &  20(2x,i3,'(',i3,')'))
 2355   format(10x,'Set ',i3,', score > ',f8.2,';',1x,i2,' members > ',
     &  20(2x,i3))
 2360   format(1x,'RSPS >> LIST SYMOPS     >> Space group: ' , a )
 2361   format(1x,'RSPS >> Symmetry operation #',
     &  i5,' > ',5x,a)
 2362   format(1x,'RSPS >> + Centering (', 3f6.2, ')' )
 2363   format(/,1x,'RSPS >> LIST SYMOPS    >> Number of ncs ',
     &  'operations: ', i3)
 2364   format(/,1x,'RSPS >> NCS operation #',i5,
     &  //(10x,f10.7,2x,f10.7,2x,f10.7,10x,f12.5))
 2365   format(1x,'RSPS >> LIST SYMOPS     >> No NCS operators stored' )
 2367   format(1x,'RSPS >> LIST VECTORS    >> ',i4,' vectors stored' )
 2368   format(/,1x,'RSPS >> Vector #',i5,10x,'Multiplicity :',i5,
     &  10x,'Type : ',a,//(10x,f10.7,2x,f10.7,2x,f10.7,10x,f12.5))
 2370   format(1x,'RSPS >> TABLE SET       >> ' )
 2372   format(1x,'RSPS >> TABLE FIXXYZ    >> ' )
 2375   format(1x,'RSPS >> TABLE POSITIONS >> ' )
 2380   format(1x,'RSPS >> CELL            >> ',6f8.2)
 2390   format(1x,'RSPS >> STATUS          >> Current settings >> ' )
 2400   format(1x,'RSPS >> SAVE            >> Saving current session ')
 2410   format(1x,'RSPS >> Savefile ' , a , ' written ' )
 2500   format(1x,'RSPS >> ROTATE          >> Rotate position ',i3,
     &  ' by SGS ',i3)
 2510   format(1x,'RSPS >> ROTATE          >> Rotate position ',i3,
     &  ' by NCS ',i3)
 2520   format(1x,'RSPS >> ROTATE          >> Rotate FIXXYZ position ',
     &  i3,' by SGS ',i3)
 2530   format(1x,'RSPS >> ROTATE          >> Rotate FIXXYZ position ',
     &  i3,' by NCS ',i3)
 2540   format(1x,'RSPS >> ROTATE          >> Rotated position = ',
     &  3f8.4)
 2550   format(1x,'RSPS >> ROTATE          >> Rotated position = ',
     &  3f8.4,' stored as FIXXYZ position ',i3)
 2560   format(1x,'RSPS >> ROTATE          >> Rotate position ',i3,
     &  ' by input matrix')
 2570   format(1x,'RSPS >> ROTATE          >> Rotate FIXXYZ position ',
     &  i3,' by input matrix')
 2600   format(1x,'RSPS >> MOLECULES vectorset  >> ',a,
     &  /,1x,'RSPS >> Unreasonably low BUMP distance discovered > ' , 
     &  f8.2,/,1x,'RSPS >> Reset to > ',f8.2)
 2800   format(1x, 'RSPS >> ADD > Adding position ',i3,' (',3f8.4,
     &  ' ) to FIXXYZ positions')
 2805   format(1x,'(',3f8.4,' ) + (',3f8.4,' ) = (',3f8.4,' )')
 2810   format(1x, 'RSPS >> ADD > Adding vector (',3f8.4,' ) to ',
     &  'FIXXYZ positions')
 2820   format(1x, 'RSPS >> SUBTRACT > Subtracting position ',i3,
     &  ' (',3f8.4,' ) from FIXXYZ positions')
 2825   format(1x,'(',3f8.4,' ) - (',3f8.4,' ) = (',3f8.4,' )')
 2830   format(1x, 'RSPS >> SUBTRACT > Subtracting vector (',
     &  3f8.4,' ) from FIXXYZ positions')
 3000   format(1x,'Peaks from ' ,a)
 3010   format(1x,i5,5x,3f12.8,5x,f8.2,3x,i4)

*-----------------------------------------------------------------------
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine RCOMNT
*
*       Function:
*       Find and reflect comments. Comments are truncated to 80 char's.
*
*       Arguments:
*       Line     (i)     line to search for comment
*
*       History:
*       Written November 1991 S. Knight
************************************************************************
        subroutine rcomnt ( line )

        implicit none

*       Arguments
        character*(*)   line

*       Local variables
        character*80    comment
        integer         combeg
        logical         incomm
        logical         starln

*       External functions
        integer         sfnblk
        integer         slnblk

*       Data
        data            incomm / .false. /
*-----------------------------------------------------------------------

        comment = ' '
        starln = .false.
        if ( index ( line , '!' ) .ne. 0 ) then
          combeg = index ( line , '!' )
          combeg = combeg + sfnblk ( line( combeg: ) )
          comment = line( combeg: )
          line( index ( line , '!' ): ) = ' '
          if ( .not. incomm ) starln = .true.
          incomm = .true.
        else if ( index ( line , '#' ) .ne. 0 ) then
          combeg = index ( line , '#' )
          combeg = combeg + sfnblk ( line( combeg: ) )
          comment = line( combeg: )
          line( index ( line , '#' ): ) = ' '
          if ( .not. incomm ) starln = .true.
          incomm = .true.
        else
          if ( incomm ) then
            call ttyptl ( ' RSPS COMMENT > ' // 
     &      '***********************************' //
     &      '***********************************'  )
            call ttycr ( 1 )
          endif
          incomm = .false.
        endif

        if ( incomm ) then
          if ( starln ) then
            call ttycr ( 1 )
            call ttyptl ( ' RSPS COMMENT > ' // 
     &      '***********************************' //
     &      '***********************************'  )
          endif
          call ttyptl ( ' RSPS COMMENT > ' // 
     &    comment( 1:max(1,slnblk(comment)) ) )
        endif



        return

*-----------------------------------------------------------------------
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine RSINIT
*
*       Function:
*       Set rsps defaults and reset some arrays
*
*       Comment:
*       Defaults are defined in rspsdef.inc
*
*       Arguments:
*       Variable  (i)     initiation control keyword
*
*       History:
*       Written 1991 S. Knight
************************************************************************
        subroutine rsinit ( variable )

        implicit none

        include         'rspsdim.inc'
        include         'rspsdef.inc'
        include         'rspsctl.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'
        include         'rspsmap.inc'
        include         'rspspos.inc'

*       Arguments
        character*(*)   variable

*       Local variables       
        integer         i , j

*-----------------------------------------------------------------------
*       Init control structure
        if ( variable .eq. 'ALL' ) then
          patnotin = .true.
          cmbnotin = .true.
          tnfnotin = .true.        
          patisin  = .false.
          cmbisin  = .false.
          tnfisin  = .false.        
          newset = .false.
          usesgs = .true.
          usencs = .false.
          usrncs = usencs
          dftgrd = .true.
        endif

*       Initiate action switches
        if ( variable .eq. 'ALL' .or. variable .eq. 'ACTIONS' ) then
          doscan = .false.
          dotran = .false.
          dovlis = .false.
          dogets = .false.
          dopisc = .false.
          dopipa = .false.
          dorigd = .false.
          doscal = .false.
cpjx Linux/g77 won't allow this assignment, since dotrnc is type int
cpjx          dotrnc = .false.
          dotrnc = 0
        endif

*       Initiate grids and frames
*       Grids are arbitrarily set here since they are used
*       in some tests that fail if = 0
        if ( variable .eq. 'ALL' ) then
          patgrd(1) = 100
          patgrd(2) = 100
          patgrd(3) = 100
          scrgrd(1) = 100
          scrgrd(2) = 100
          scrgrd(3) = 100
        endif

*       Space group symmetry
        if ( variable .eq. 'ALL' .or. variable .eq. 'SGS' ) then
          nopsgs = 1
          ispgrp = 0
          spgnam = 'UNDEFINED'
          call matclr ( r_sgs , 9 , mxsgs )
          call matclr ( t_sgs , 3 , mxsgs )
          call matcop ( r_idt , r_sgs , 3 , 3 )
        endif
          
*       Non crystallographic symmetry
        if ( variable .eq. 'ALL' .or. variable .eq. 'NCS' ) then
          nopncs = 1
          call matclr ( r_ncs , 9 , mxsgs )
          call matclr ( t_ncs , 3 , mxsgs )
          call matcop ( r_idt , r_ncs , 3 , 3 )
        endif

*       Title for scoremap
        if ( variable .eq. 'ALL' .or. variable .eq. 'TITLE' ) then
          titut = df_titut
        endif

*       Patterson map format
        if ( variable .eq. 'ALL' .or. variable .eq. 'MAPFORMAT' ) then
          pattyp = df_pattyp
        endif

*       Vectorset
        if ( variable .eq. 'ALL' .or. variable .eq. 'VECTORSET' ) then
          vecset = df_vecset
        endif

*       Patterson map scale
        if ( variable .eq. 'ALL' .or. variable .eq. 'SCALE' ) then
          doscal = .false.
          patscl( 1 ) = df_patscl1
          patscl( 2 ) = df_patscl2
        endif

*       Peak modification (RESET)
        if ( variable .eq. 'ALL' .or. variable .eq. 'RESET' ) then
          rstnum = 0
          call matclr ( rstuvw , 3 , mxreset )
          call matclr ( rstrad , 1 , mxreset )
          call matclr ( rstlev , 1 , mxreset )
        endif

*       Sectioning of output map
        if ( variable .eq. 'ALL' .or. variable .eq. 'IUVW2' ) then
          scraxo( 1 ) = df_scraxo( 1 ) 
          scraxo( 2 ) = df_scraxo( 2 ) 
          scraxo( 3 ) = df_scraxo( 3 ) 
        endif

*       Rejection level
        if ( variable .eq. 'ALL' .or. variable .eq. 'REJECT' ) then
          rej = df_rej
        endif

*       Maximum number of low peaks
        if ( variable .eq. 'ALL' .or. variable .eq. 'MAXLOW' ) then
          maxlow = df_maxlow
        endif

*       Minimum distance between symmetry related positions (harker)
*       or independent positions (cross)
        if ( variable .eq. 'ALL' .or. variable .eq. 'BUMP' ) then
          bump = df_bump
          usrbmp = bump
        endif

*       Scoring function
        if ( variable .eq. 'ALL' .or. variable .eq. 'SCORE' ) then
          scrtyp = df_scrtyp
        endif

*       Minimum function defaults
        if ( variable .eq. 'ALL' .or. variable .eq. 'MINFUN' ) then
          minfun = df_minfun
          nuse = df_nuse
        endif

*       Forced-vector default
        if ( variable .eq. 'ALL' .or. variable .eq. 'FORCE' ) then
          force = .false.
          frcpek( 1 ) = 0.0
          frcpek( 2 ) = 0.0
          frcpek( 3 ) = 0.0
        endif

*       Number of grid points added on each side of vector when
*       looking for peak max in scoring
        if ( variable .eq. 'ALL' .or. variable .eq. 'TOLERANCE' ) then
          pektol = df_pektol
        endif

*       Printing level
        if ( variable .eq. 'ALL' .or. variable .eq. 'PRINT' ) then
          iprt = df_iprt
        endif

*       Special positions switch
        if ( variable .eq. 'ALL' .or. variable .eq. 'SPEC' ) then
          special = df_spec
          usrspc = special
        endif

*       Peak weighting switch
        if ( variable .eq. 'ALL' .or. variable .eq. 'WEIGHT' ) then
          usemult = df_usemult
        endif

*       Cross scan fixed positions array
        if ( variable .eq. 'ALL' .or. variable .eq. 'FIXXYZ' ) then
          nxyz2 = 0
          do i = 1 , mxipsn
            xyz2( 1 , i ) = 0.0
            xyz2( 2 , i ) = 0.0
            xyz2( 3 , i ) = 0.0
          end do
        endif

*       Vlist array
        if ( variable .eq. 'ALL' .or. variable .eq. 'VLIST' ) then
          nxyz1 = 0
          do i = 1 , mxipsn
            xyz1( 1 , i ) = 0.0
            xyz1( 2 , i ) = 0.0
            xyz1( 3 , i ) = 0.0
          end do
        endif

*       Translation scan array
        if ( variable .eq. 'ALL' .or. variable .eq. 'POLARSCAN' ) then
          ntrnps = 0
          do i = 1 , mxpick
            xscan( 1,i ) = 0.0
            xscan( 2,i ) = 0.0
            xscan( 3,i ) = 0.0
          end do
        endif

*       Patterson peak array
        if ( variable .eq. 'ALL' .or. variable .eq. 'PEAKS' ) then
          ppnum = 0
          do i = 1 , mxpick
            ppuvw( 1,i ) = 0.0
            ppuvw( 2,i ) = 0.0
            ppuvw( 3,i ) = 0.0
            ppval( i )   = 0.0
          end do
        endif


*       Default number of peaks to pick
        if ( variable .eq. 'ALL' .or. variable .eq. 'PICK' ) then
          npick = df_npick
        endif

*       Limits for pick (values flag for whole map)
        if ( variable .eq. 'ALL' .or. variable .eq. 'PKLIMT' ) then
          pkltfr(1)     =       1.0
          pkltfr(2)     =      -1.0
          pkltfr(3)     =       1.0
          pkltfr(4)     =      -1.0
          pkltfr(5)     =       1.0
          pkltfr(6)     =      -1.0
        endif

*       Number of and limits for pick exclude regions
        if ( variable .eq. 'ALL' .or. variable .eq. 'PKXCLD' ) then
          nxcl          =       0
          do i = 1,mxexcl
          xclpnt(i)     =       0
          pkxcld(i)     =   999.0
          end do
        endif

*       Pick level (if 0 level is set internally)
        if ( variable .eq. 'ALL' .or. variable .eq. 'PICKLEVEL' ) then
          pick_level = df_pick_level
        endif

*       Minimum number of positions/getset set 
        if ( variable .eq. 'ALL' .or. variable .eq. 'GETSET' ) then
          minset = df_minset
        endif

*       Getsets sets
        if ( variable .eq. 'ALL' .or. variable .eq. 'SETS' ) then
          settot = 0
          do i = 1 , mxgset
            setscr( i ) = 0.0
            setsiz( i ) = 0
            do j = 1 , mxgmem
              setpos( j,i ) = 0
            end do
          end do
        endif

*       Truncate level
        if ( variable .eq. 'ALL' .or. variable .eq. 'DENS_MAX' ) then
          dotrnc = df_dotrnc
          dens_max = df_dens_max
          trunclev = dens_max
        endif

*       Orthogonalization code
        if ( variable .eq. 'ALL' .or. variable .eq. 'ORTHO' ) then
           ortcod = df_ortcod
        endif

        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine KEYIN
*
*       Function:
*       Extract and expand commands from input command line. Command
*       delimiter passed as argument. If delimiter = 'NONE' no special
*       command delimiter is used and trapping of unknown words is
*       skipped.
*
*       Arguments :
*       Line    (i)     character string to be searched for command words
*       Ibeg    (i)     start position of words in line
*       Iend    (i)     end position of words in line
*       Wdtyp   (i)     word type code:
*                       0  null field
*                       1  character string
*                       2  number
*       Numwd   (i)     number of words in line
*       Cmndlst (i)     lexically sorted list of command words
*       Cmndtot (i)     number of command words
*       Mxcmnd  (i)     maximum number of command words in a line
*       Delim   (i)     command delimiter; 'NONE' if none
*       Cmnd    (o)     list of expanded commands
*       Cmndnum (o)     list of command word ordinal numbers
*       Cmndpos (o)     ordinal position of command words in line
*       Ncmnd   (o)     number of command words found
*
*       History:
*       August 31 1992 - sdk
*       Command delimiter passed as argument
*       August 1991 - sdk 
*       Command word expansion added
*       Written March 1988 S. Knight
************************************************************************
        subroutine keyin(line,ibeg,iend,wdtyp,numwd,cmndlst,cmndtot,
     &                  mxcmnd,delim,cmnd,cmndnum,cmndpos,ncmnd)

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'


*       Arguments
        integer         numwd , cmndtot , mxcmnd , ncmnd
        integer         ibeg( numwd ) , iend( numwd ) , wdtyp( numwd )
        integer         cmndpos( mxcmnd ) , cmndnum( mxcmnd )
        character*(*)   line
        character*(*)   delim
        character*(*)   cmndlst( cmndtot ) , cmnd( mxcmnd )


*       Local variables
        character*20    word , string , catlin*80
        integer         i , j , istop
        integer         errcod
        logical         incmnd

*       External functions
        integer         slnblk
*-----------------------------------------------------------------------
*       Init
        istop = 0
        if (stop_on_error) istop = 1
        ncmnd = 0
        incmnd = .false.
        do 10 i = 1 , mxcmnd 
          cmnd( i )    = ' '
          cmndpos( i ) = 0
          cmndnum( i )  = 0
 10     continue


*       Attempt to expand each word into command word. Count and
*       store command words.
        do 30 i = 1 , numwd

*       Skip any non-character strings
          if ( wdtyp( i ) .ne. 1 ) goto 30

*       Skip quoted strings
*       The CCP4 parser strips quotes so we need this ugly test
          if ( ibeg( i ) .gt. 1 ) then
            if ( line( ibeg(i)-1:ibeg(i)-1 ) .eq. '''' .or.
     &      line( ibeg(i)-1:ibeg(i)-1 ) .eq. '"' ) goto 30
          endif

*       Copy and upcase word
          string = line( ibeg( i ):iend( i ) )
          call supcas ( string )

*       End of command word sequence ?
          if ( delim .ne. 'NONE' .and. string .eq. delim ) then
            incmnd = .false.
            goto 30
          endif

*       Skip word if in command word sequence
          if ( incmnd ) goto 30

*       Try to expand
          call sexpnd ( word , string( :slnblk( string ) ) , 
     &    cmndlst , cmndtot , errcod )

          if ( errcod .eq. 0 ) then

            if ( ncmnd .eq. mxcmnd ) then
              call errmsg ( 'TOO MANY COMMANDS ON A LINE',istop )
              catlin = ' LAST COMMAND STORED > ' // cmnd(ncmnd) 
              call ttyptl ( catlin )
              return
            endif

*       Set command start flag
            if ( delim .ne. 'NONE' ) incmnd = .true.
*       Count command words
            ncmnd = ncmnd + 1
*       Store expanded command word
            cmnd( ncmnd ) = word 
*       Store positional number in line
            cmndpos( ncmnd ) = i
*       Get and store command word number
            do 20 j = 1 , cmndtot
              if ( word .eq. cmndlst( j ) ) then
                cmndnum( ncmnd ) = j
                goto 30
              endif
 20         continue


          endif

          if ( delim .eq. 'NONE' .and. errcod .eq. 1 ) errcod = 0
          if ( errcod .eq. 1 ) then

            call ttycr ( 1 )
            call wrnmsg ( 'Unknown command > ' // string )

          else if ( errcod .eq. 2 ) then

            call ttycr ( 1 )
            call wrnmsg ( 'Ambiguous command > ' // string )

          else if ( errcod .eq. 3 ) then

            call ttycr ( 1 )
            call wrnmsg ( 'Too long word > ' // string )

          endif
          if ( errcod .ne. 0 ) then
            call errmsg ( 'Couldn''t interpret command' , istop )
            call ttyptl ( ' Valid commands >' )
            write ( 6 , '( 4 ( 2x , a ) )' ) cmndlst
            return
          endif

 30     continue

        return
        end
*********************************************************************SDK
************************************************************************
*       Integer function keyword
*       Return keyword number for keyword key
************************************************************************
        integer function keynm ( key , keywrd , nwrd )
        integer          nwrd
        character*(*)    key , keywrd(nwrd)
        integer          i

        keynm = 0
        do 10 i = 1 , nwrd
          if ( key .eq. keywrd( i ) ) then
            keynm = i
            return
          endif
 10     end do
        return
        end
*********************************************************************SDK
************************************************************************
*       Logical function TOPCMD
*       Return true if characters in WORD match current item in array
*       CMND
************************************************************************
        logical function topcmd( word )
        character*(*)    word
        integer          icmnd , jcmnd
        integer          maxcmnd
        parameter        ( maxcmnd = 20 )
        character*12     cmnd( maxcmnd ) , subcmnd( maxcmnd )
        common /cmdnum/  icmnd , jcmnd
        common /cmdwds/  cmnd , subcmnd

        topcmd = .false.
        if ( cmnd( icmnd ) .eq. word ) topcmd = .true.
        return
        end
*********************************************************************SDK
************************************************************************
*       Logical function SUBCMD
*       Return true if characters in WORD match current item in array
*       SUBCMND
************************************************************************
        logical function subcmd( word )
        character*(*)    word
        integer          icmnd , jcmnd
        integer          maxcmnd
        parameter        ( maxcmnd = 20 )
        character*12     cmnd( maxcmnd ) , subcmnd( maxcmnd )
        common /cmdnum/  icmnd , jcmnd
        common /cmdwds/  cmnd , subcmnd

        subcmd = .false.
        if ( subcmnd( jcmnd ) .eq. word ) subcmd = .true.
        return
        end
*********************************************************************SDK
************************************************************************
*       Logical function nextwd
*       Return true if characters in WORD match current item in LINE
************************************************************************
        logical function nextwd( word )
        character*(*)    word
        integer          inext , ib , ie
        integer          maxword
        parameter        ( maxword = 40 )
        integer          ibeg( maxword ) , iend( maxword )
        character*500    line
        common /cmddel/  inext, ibeg , iend
        common /cmdlin/  line

        nextwd = .false.
        ib = ibeg( inext )
        ie = iend( inext )
        call supcas ( line( ib:ie ) ) 
        if ( index ( word , line( ib:ie ) ) .eq. 1 ) nextwd = .true.
        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine GETPNT
*
*       Function:
*       Get and check pointers to array
*       
*       Arguments:
*       Pos1      (o)     pointer to first slot
*       Pos2      (o)     pointer to final slot
*       Ntot      (i)     total number of slots in array
*       Dtyp      (i)     item data type flag array (idtf)
*       Fval      (i)     item numerical value array (inva)
*       Start     (i)     slot in inva to look for first pointer
*       Numpnt    (i)     number of pointer arguments (including start)
*       Errcod    (o)     error code
*                         errcod = 0 - no error
*                         errcod = 1 - error in pointer definition
*       
*       
*       
*       
*       History:
*       Written Dec 1991 S. Knight
************************************************************************
        subroutine getpnt ( pos1 , pos2 , ntot , dtyp , fval ,
     &  start , numpnt , errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'

*       Arguments
        integer         pos1 , pos2
        integer         ntot
        integer         start
        integer         numpnt
        integer         errcod
        integer         dtyp( start + numpnt )
        real            fval( start + numpnt )

*       Local variables
        integer         istop
        integer         ib , ie
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initialize error code
        errcod = 0

*       Check that there are slots to point at
        if ( ntot .eq. 0 ) then
          istop = 0
          if ( stop_on_error ) istop = 1
          call errmsg ( 'GetPnt: Array is empty' , istop )
          errcod = 1
          return
        endif

*       Check pointer to inva OK
        if ( start .le. 0 ) then
          istop = 0
          if ( stop_on_error ) istop = 1
          call errmsg ( 'GetPnt: Argument pointer out of bounds' , 
     &    istop )
          errcod = 1
          return
        endif

*       Set default pointers
        ib = 1
        ie = ntot

*       Get pointer to first position
        if ( numpnt .ge. 1 ) then
          if ( dtyp( start ) .eq. 2 ) then
            ib = fval ( start )
          else
            istop = 0
            if (stop_on_error) istop = 1
            call errmsg ( 'GetPnt: Bad argument; non-numeric', istop )
            errcod = 1
            return
          endif
        endif

*       Get pointer to final position or set = first position
        if ( numpnt .ge. 2 ) then
          if ( dtyp( start + 1 ) .eq. 2 ) then
            ie = fval ( start + 1 )
          else
            istop = 0
            if (stop_on_error) istop = 1
            call errmsg ( 'GetPnt: Bad argument; non-numeric' , 
     &      istop )
            errcod = 1
            return
          endif
        else if ( numpnt .eq. 1 ) then
          ie = ib
        endif

*       Check pointers are reasonable
        if ( ib .le. 0 .or. ie .le. 0 .or. ie .lt. ib ) then
          istop = 0
          if (stop_on_error) istop = 1
          call errmsg ( 'GetPnt: Strange argument; illegal range' , 
     &    istop )
          errcod = 1
          return
        endif

*       Check pointers are in range
        if ( ib .gt. ntot ) then
          istop = 0
          if (stop_on_error) istop = 1
          call errmsg ( 'GetPnt: First number out of range' , istop )
          errcod = 1
          return
        endif
        if ( ie .gt. ntot ) then
          ie = ntot
          call ttyptl ( ' Warning - final number out of range' )
          call ttyptl ( ' Warning - final position reset' )
        endif


        pos1 = ib
        pos2 = ie
        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Rsps GET package
*
*       Routines to handle preliminary activities like reading
*       symmetry, patterson map etc.
*
*       Contains:
*       s/r GETSGS     read and check SGS
*       s/r MYSYMTRN   translate symmetry matrix to characters
*       s/r GETLGS     generate LGS (called by GETSGS)
*       s/r GETNCS     read and check NCS
*       s/r GETPAT     read, truncate, reset & scale Patterson map
*       s/r GETCMB     combine symmetry
*       s/r GETTNF     get transformations from real to vector space
*       s/r GETCRD     read coordinates from file ; assign site numbers
*       s/r RSRPDB     read positions from RSPS truncated PDB file
*       s/r RSWPDB     write positions to RSPS truncated PDB file
*       s/r LODSYM     load symmetry
*       s/r HRKPLN     compute Harker plane equations
*       s/r CENTR1     check for centering
*       fn  CHECKID    check if matrix is identity

*
*
*       November 1991 S. Knight
************************************************************************
************************************************************************
*        Subroutine MySYMTRN ( Rmatrix,Tvector,Rsymbol )
*
*        Function:
*        Translate symmetry matrix into International Tables character
*        string.
*
*        Arguments:
*        Rmatrix   (i)    3 x 3 rotation matrix
*        Tvector   (i)    3 x 1 translation vector (fractional)
*        Rsymbol   (o)    symmetry character string
*
*        History:
*        December 1 1993 - sdk
*        Modified to accept 3x3 rotation matrix + 3x1 translation
*        vector as argumnet.
*        Adapted from CCP4 s/r SYMTRN August 17 1993 - sdk
************************************************************************
        subroutine MySYMTRN ( Rmatrix,Tvector,Rsymbol )

        implicit none

*       Arguments      
        real            Rmatrix(3,3)
        real            Tvector(3)
        character*(*)   Rsymbol

*       Local variables
        integer         i, j, ich, ist, itr, i1, i2
        integer         npntr1(10), npntr2(10)
        character       axiscr(3)*1, numb(9)*1, symchs*80

*       External functions
        integer         slnblk

*       Data statements
        data            axiscr/'X','Y','Z'/
        data            numb/'1','2','3','4','5','6','7','8','9'/
        data            npntr1/0,1,1,1,0,1,0,2,3,5/
        data            npntr2/0,6,4,3,0,2,0,3,4,6/
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Clear symchs
        symchs = ' '
        ich = 1

        do 20 j = 1,3

*       Ist is flag for first character of operator
          ist = 0

          do 10 i = 1,3

            if ( Rmatrix(j,i) .ne. 0 ) then

              if ( Rmatrix(j,i) .gt. 0 .and. ist .gt. 0 ) then
                symchs(ich:ich) = '+'
                ich = ich + 1
              end if

              if ( Rmatrix(j,i) .lt. 0 ) then
                symchs(ich:ich) = '-'
                ist = 1
                ich = ich + 1
              end if

              if ( ist .eq. 0 ) ich = ich + 1
              symchs(ich:ich) = axiscr(i)
              ist = 1
              ich = ich + 1

            end if
   10     continue

*       Translation
          if ( Tvector(j) .ne. 0 ) then
            if ( Tvector(j) .lt. 0 ) then
              symchs(ich:ich+1) = '-'
            else
              symchs(ich:ich+1) = '+'
            endif
            ich = ich + 1
            itr = nint( abs( Tvector(j)*12.0 ) )
            i1  = npntr1( itr ) 
            i2  = npntr2( itr )
            if ( i1 .ne. 0 .and. i2 .ne. 0 ) then
              symchs(ich:ich+2) = numb( i1 ) // '/' // numb( i2 )
              ich = ich + 3
            endif
          end if

*       Add comma space
          if ( j .ne. 3 ) then
            ich = ich + j*12 - slnblk(symchs)
            symchs(ich:ich+2) = ',  '
            ich = ich + 3
          end if
   20   continue

        Rsymbol = symchs
        return
        end
************************************************************************
************************************************************************
*
*       Subroutine GETSGS
*
*       Function:
*       Read Space Group Symmetry.
*       Results are kept in common /symm/.
*       NOPSGS symmetry operators are stored in R_SGS,T_SGS.
*       ICEN centering translations are stored in CENT.
*
*       Arguments:
*       Errcod    (o)     error code
*                         errcod = 0 - no error
*                         errcod = 1 - not enough space
*                         errcod = 2 - bad vector or matrix element
*                         errcod = 3 - determinant .ne. 1.0
*
*       History
*       March 31 1992 - sdk
*       Call to SYMMIN removed. All testing done here.
*       Space group may now be specified by name as well as number.
*       November 11 1991 - sdk
*       Generation of Patterson symmetry factored out to GETPSY.
*       November 5 1991 - sdk
*       Modified for fractional translations and centering.
*       Written October 30 1991 S. Knight
************************************************************************
        subroutine getsgs ( errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'
        include         'rspsctl.inc'
        include         'rspsfil.inc'

*       Arguments
        integer         errcod

*       Local variables
        integer         i , j , k
        integer         nsymp
        real            sgrot(4,4,mxsgs)
        character*8     pgnam
        character*40    symchr

*       External functions
        real            det
        integer         slnblk
        logical         checkid
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Write s/r id
        call ttycr ( 1 )
        call ttyptl ( '   RSPS GETSGS >>' )

*       Initiate error code
        errcod = 0

*       Get symmetry operations from library file
        close ( lunsgs )
        nopsgs = 0
        call msymlb ( lunsgs , ispgrp , spgnam , pgnam , 
     &  nsymp , nopsgs , sgrot )

*       Check space
        if ( nopsgs .gt. mxsgs ) then
          call errmsi ( 'Too many space group symmetry ' //
     &    'operations, max = ' , mxsgs , 3 , 0 )
          errcod = 1
          return
        endif

*       Copy symmetry operations from 4 x 4 matrix to 3 x 3 matrix
*       + 3 x 1 vector.
        do 300 i = 1 , nopsgs

          do 200 j = 1 , 3
*       Check translation vector for obvious errors
            if ( abs ( sgrot( j,4,i ) ) .gt. 1.001 ) then
              call errmsg ( 'Bad translation component (>1.0) in SGS' ,
     &         0 )
              errcod = 2
              return
            endif
*       Copy
            t_sgs( j,i ) = sgrot( j,4,i )

            do 100 k = 1 , 3
*       Check matrix for obvious errors
              if ( abs ( sgrot( k,j,i ) ) .gt. 1.001 ) then
                call errmsg ( 'Bad matrix element (>1.0) in SGS' , 0 ) 
                errcod = 2
                return
              endif
*        Copy
              r_sgs( k,j,i ) = sgrot( k,j,i )

 100        continue
 200      continue

*       Check determinant
          if ( abs ( det( r_sgs( 1,1,i ) ) - 1. ) .gt. 0.001 ) then
            call errmsg ( 'Matrix does not represent rigid body ' //
     &      'rotation (det .ne. 1.0)' , 0 )
            call errmsi ( 'Illegal operation in matrix number ' , 
     &      i , 3 , 0 )
            errcod = 3
            return
          endif
 300    continue

*       Check that first operation is identity
        if ( .not. checkid ( r_sgs(1,1,1),t_sgs(1,1) ) ) then
          call errmsg ( 'First symmetry card must be identity - ' //
     &    'check your symmetry library' , 1 )
        endif

*       Check for centering; store translations in cent if present
        call centr1 ( r_sgs , t_sgs , nopsgs , cent , icen )

*       Generate Laue group (Patterson) symmetry
        call getlgs

*       Set apropriate default cell parameters if cell not properly
*       defined for space group
*       The cell needs to set for some tests in e.g. s/r TRNSF but
*       the exact values are not important
        call setcel ( ispgrp , cell )

*       Write out space group symmetry
        write(6,2270)ispgrp,spgnam(:slnblk(spgnam)),nopsgs
        if ( iprt .lt. 2 ) then
          do 10 i = 1 , nopsgs
            call mysymtrn ( r_sgs(1,1,i) , t_sgs(1,i) , symchr )
            write(6,2280) i , symchr( :slnblk(symchr) )
 10       continue
          if(icen.gt.1)then
            write(6,2300) ( ( cent(j,i),j=1,3),i=1,icen )
          else
            write(6,*)
          endif
        else if ( iprt .ge. 2 ) then
          do 20 i = 1 , nopsgs
            call mysymtrn ( r_sgs(1,1,i) , t_sgs(1,i) , symchr )
            write(6,2290) i , symchr( :slnblk(symchr) ) , 
     &      ( (r_sgs(k,j,i),j=1,3) , t_sgs(k,i),k=1,3 )
 20       continue
          if(icen.gt.1)then
            write(6,2300) ( ( cent(j,i),j=1,3),i=1,icen )
          else
            write(6,*)
          endif
        endif

        return


*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
 2270   format(10x,'Space group symmetry operations read from ',
     &  'library entry #',i4,' (space group ',a,')',/10x,
     &  'Number of operations stored = ',i2,/)
 2280   format(10x,'Symmetry operation #',i5,': ',5x,a)
 2290   format(//10x,'Symmetry operation #',i5,': ',5x,a,
     &  //(10x,f10.7,2x,f10.7,2x,f10.7,10x,f12.5))
 2300   format(/, (18x,'+ (',3f6.2,' )') )

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine GETLGS
*
*       Function:
*       Generate Patterson (Laue group) symmetry.
*       Results are kept in common /symm/.
*       2 * NOPSGS Patterson symmetry operators are stored in R_LGS.
*
*       Arguments:
*       None.
*
*       History:
*       Written November 11 1991 (Factored out from GETSGS) S. Knight
************************************************************************
        subroutine getlgs

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'

*       Local variables
        integer         i
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Generate patterson symmetry operators
        do 100 i = 1 , nopsgs

          call mmtply ( r_idt , r_sgs(1,1,i) , r_lgs(1,1,2*i-1) )
          call mmtply ( r_inv , r_sgs(1,1,i) , r_lgs(1,1,2*i  ) )

 100    continue

        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine GETNCS
*
*       Function:
*       Check and store non crystallographic symmetry.
*       Results are kept in common /symm/.
*       NOPNCS symmetry operators are stored in R_NCS,T_NCS.
*
*       Comment:
*       Input matrices are expected to operate on an orthogonal frame as
*       specified by the orthogonalization code:
*                        conv = 1 >        a  // xo   c* // zo
*                        conv = 2 >        b  // xo   a* // zo
*                        conv = 3 >        c  // xo   b* // zo
*                        conv = 4 >  hex a+b  // xo   c* // zo
*                        conv = 5 >        a* // xo   c  // zo
*                        conv = 6 >        a  // xo   b* // yo
*       The stored matrices operate on fractional coordinates in the
*       crystallographic frame.
*
*
*       Arguments:
*       Rtmp      (i)     rotation matrix for orthogonal frame
*       Ttmp      (i)     fractional translations to be applied
*                         after rotation
*       Cell      (i)     unit cell parameters
*       Code      (i)     orthogonalization code
*       Iprint    (i)     print level flag
*       Errcod    (o)     error code
*                         errcod = 0 - no error
*                         errcod = 1 - no space left
*                         errcod = 2 - bad matrix or translation
*                         errcod = 3 - determinant .ne. 1
*                         errcod = 4 - orthogonalization error
*
*       History:
*       August 18 1992 - sdk
*       Init argument removed. Counter and matrices now initiated by
*       call to rsinit instead.
*       February 1992 - sdk
*       Modified for orthogonal input matrices. Call to SYMMIN removed.
*       All testing done here.
*       November 5 1991 - sdk
*       Modified for fractional translations.
*       Written October 30 1991 S. Knight
************************************************************************
        subroutine getncs ( rtmp , ttmp , cell , code , iprint ,
     &  errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'

*       Arguments
        real            rtmp( 3,3 ) , ttmp( 3 ) , cell( 6 )
        integer         code
        integer         iprint
        integer         errcod

*       Local variables
        integer         i , j , k
        real            deort( 3,3 ) , toort( 3,3 ) , tmat( 3,3 )
        character*12    jnklin

*       External functions
        real            det
        logical         checkid
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Write s/r id
        call ttycr ( 1 )
        call ttyptl ( '   RSPS GETNCS >>' )

*       Init error code
        errcod = 0

*       Skip if identity matrix (already stored)
        if ( checkid( rtmp , ttmp ) ) then
          write(6,2000)nopncs
          if ( iprint .ge. 1 ) then
            write(6,2100)nopncs,((r_ncs(k,j,nopncs),j=1,3),
     &      t_ncs(k,nopncs),k=1,3)
            call ttycr ( 1 )
          endif
          return
        endif

*       Check space
        if ( nopncs .ge. mxncs ) then
          call errmsi ( 'Too many non crystallographic symmetry ' //
     &    'operations, max = ' , mxncs , 3 , 0 )
          errcod = 1
          return
        endif

*       Check symmetry operators for obvious errors
        do i = 1 , 3
          if ( abs ( ttmp( i ) ) .gt. 1.001 ) then
            call errmsg ( 'Bad translation component (>1.0) in NCS' , 
     &      0 )
            errcod = 2
            return
          endif
          do j = 1 , 3
          if ( abs ( rtmp( j,i ) ) .gt. 1.001 ) then
            call errmsg ( 'Bad matrix element (>1.0) in NCS' , 0 )
            errcod = 2
            return
          endif
          end do
        end do

*       Check determinant
        if ( abs ( det( rtmp( 1,1 ) ) - 1. ) .gt. 0.001 ) then
          call errmsg ( 'Matrix does not represent rigid body ' //
     &    'rotation (det .ne. 1.0)' , 0 )
          write(jnklin,'(f12.4)') det( rtmp( 1,1 ) )
          call errmsg ( 'Determinant = ' // jnklin , 0 )
          errcod = 3
          return
        endif

*       Set orthogonalization matrices
        call ortho ( cell , code , 'TOORT' , toort , errcod )
        call ortho ( cell , code , 'DEORT' , deort , errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'Failed getting orthogonalization matrix' , 0 )
          errcod = 4
          return        
        endif

*       Form rotation operating on and giving fractional coordinates
*       in crystallographic frame
        call mmtply ( rtmp , toort , tmat )
        call mmtply ( deort , tmat , rtmp )

*       Increase counter and store symmetry
        nopncs = nopncs + 1
        call matcop ( rtmp , r_ncs( 1,1,nopncs ) , 3 , 3 )
        call matcop ( ttmp , t_ncs( 1,nopncs ) , 3 , 1 )

*       Write it out 
        write(6,2000)nopncs
        if ( iprint .ge. 1 ) then
          write(6,2100)nopncs,((r_ncs(k,j,nopncs),j=1,3),
     &    t_ncs(k,nopncs),k=1,3)
          call ttycr ( 1 )
        endif

        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
 2000   format(10x,'GETNCS > ',i3,' operations stored',/)
 2100   format(/10x,'Symmetry operation #',i5,
     &  //(10x,f10.7,2x,f10.7,2x,f10.7,10x,f12.5))
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine GETPAT
*
*       Function:
*       Read and store Patterson map. Also get statistics (and 
*       truncate high densities). S/r also returns cos of cell angles
*       and score map grid (if not explicitly set) in common /ucell/.
*
*       Arguments:
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - error reading CCP4 format map
*                       errcod = 2 - error reading PROTEIN format map
*                       errcod = 3 - illegal Patterson map format
*                       errcod = 4 - map too large
*                       errcod = 5 - MAPSTAT error
*                       errcod = 6 - illegal space group or map limits
*                       errcod = 7 - error reading map header
*
*       History:
*       March 1 1994 - sdk
*       Only save new map statistics if map has been rescaled; not
*       after truncate/reset.
*       Written October 30 1991 S. Knight
************************************************************************
        subroutine getpat ( errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspsgrd.inc'
        include         'rspsmap.inc'
        include         'rspsctl.inc'
        include         'rspsfil.inc'

*       Argument
        integer         errcod

*       Local variables
        integer         iflag
        real            cutlev
        real            pmin , pmax , pave , psig
        character       label( 3 ) * 1
        data            label  / 'X' , 'Y' , 'Z' /
        character*80    pattit
       
        integer lenstr, ilen
        external lenstr

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Write s/r id
        call ttycr ( 1 )
        call ttyptl ( '   RSPS GETPAT >>' )

*       Initiate error code
        errcod = 0

*       Initiate timer
        iflag = 0
        call timer ( iflag , ' ' )

        pattit = ' '

*       Open patterson map and get relevant header info
        call rdhdr ( pattyp , lunpat , patfil , patsec , patext , 
     &  pataxo , patgrd , patlim , cell , pattit , errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'Error reading map header' , 0 )
          call mrclos ( lunpat )
          errcod = 7
          return
        endif
        ilen = lenstr(pattit)
        write(6,2010)pattyp,pattit(1:ilen)


*       Get number of points along fast, slow axis
*       Check that we have enough space to store patterson map
        patfas = patext( pataxo( 1 ) )
        patslo = patext( pataxo( 2 ) )
        pattot = patfas * patslo * patsec
        if ( pattot .gt. mxpatp ) then
          write(6,9200)pattot,mxpatp
          call errmsg ( 'Input map too large for array patmap' , 0 )
          call mrclos ( lunpat )
          errcod = 4
          return
        endif

*       Check validity of asymmetric unit as read from map
        call chklim ( ispgrp , errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'Map not stored' , 0 )
          call mrclos ( lunpat )
          errcod = 6
          return
        endif

*       Write out info on patterson layout
        if ( iprt .gt. 0 ) then
          write(6,2020)cell,label(pataxo(3)),label(pataxo(1)),
     &    label(pataxo(2)),patgrd,patlim
        endif

*       Read patterson map into core
        call rdmap ( patmap , patfas , patslo , patsec , pattyp ,
     &  lunpat , errcod )
        if ( errcod .eq. 1 ) then
          call errmsg ( 'Error reading CCP4 format map' , 0 )
          return
        else if ( errcod .eq. 2 ) then
          call errmsg ( 'Error reading PROTEIN format map' , 0 )
          return
        else if ( errcod .eq. 3 ) then
          call errmsg ( 'Illegal Patterson map format' , 0 )
          return
        endif

*       Get (and print) map statistics from input map
        call mapstat ( patmap , pattot , patmin , patmax ,
     &  patave , patsig , errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'Couldn''t get map statistics' , 0 )
          errcod = 5
          return
        endif
        if ( iprt .gt. 0 ) write(6,2040)patmin,patmax,patave,patsig

*       Truncate
        if ( dotrnc .gt. 0 ) then
          if ( dotrnc .eq. 2 ) then
            cutlev = dens_max * patsig
            trunclev = trunclev * patsig
          else
            cutlev = dens_max
          endif
          write(6,2050)cutlev, trunclev
          call maptrun ( patmap , pattot , cutlev , trunclev )
        endif

*       Reset peaks
        if ( rstnum .gt. 0 ) then
          write(6,2060)rstnum
          call maprset ( patmap , patfas , patslo , patsec , 
     &    rstnum , rstuvw , rstrad , rstlev , pataxo , patgrd , cell )
        endif

*       Scale (if scale is ca 1.,0. don't bother)
        if ( doscal ) then
          if ( abs ( patscl(1) - 1. ) .gt. 0.01 .or.
     &    abs ( patscl(2) ) .gt. 0.01 ) then
            write(6,2070)patscl
            call mapscal ( patmap , pattot , patscl )
          else
            doscal = .false.
          endif
        endif

*       Get (and print) map statistics from modified map
        if ( dotrnc .ne. 0 .or. rstnum .ne. 0 .or. doscal ) then
          call mapstat ( patmap , pattot , pmin , pmax ,
     &    pave , psig , errcod )
          if ( errcod .ne. 0 ) then
            call errmsg ( 'Couldn''t get map statistics' , 0 )
            errcod = 5
            return
          endif
          if ( doscal ) then
            patmin = pmin
            patmax = pmax
            patave = pave
            patsig = psig
            write(6,2080)patmin,patmax,patave,patsig
          else
            write(6,2090)pmin,pmax
          endif
        endif

*       Cosine of cell angles are stored in common /ucell/
        call cosabg ( cell( 4 ) , cosang )

*       Set default score map grid if not explicitly set
        if ( dftgrd ) call stgrd

*       Reset rejection level
        reject = rej * patsig + patave

*       Set flags
        patnotin = .false.
        patisin  = .true.

*       Clear Patterson peak array
        call rsinit ( 'PEAKS' )

*       Report time
        iflag = 1
        call timer ( iflag , 'GETPAT' )


        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       format statements
 2010   format(10x,a,' format map opened',/10x,'Title: ',a)
 2020   format(/10x,'Unit cell parameters: ',6f10.2,//,
     &  10x,'The input map has ',a1,'-sections with ',
     &  a1,' varying most rapidly and ',a1,' most slowly',/,
     &  10x,'Grid   along X,Y,Z        : ',3i8,/,
     &  10x,'Limits along X,Y,Z        : ',6i8)
 2040   format(/10x,'Density statistics on Patterson map: ',
     &  /10x,'Minimum density    =',f12.4,
     &  /10x,'Maximum density    =',f12.4,
     &  /10x,'Average density    =',f12.4,
     &  /10x,'Std Dev of density =',f12.4)
 2050   format(/10x,'Truncating >> Densities above ',f8.1,' will be ',
     &  'truncated to ',f8.1)
 2060   format(/10x,'Resetting  >> ',i3,' peaks will be reset')
 2070   format(/10x,'Rescaling  >> Density = ',f10.3,' * density + ',
     &  f10.1)
 2080   format(/10x,'Density statistics on modified Patterson map:',
     &         /10x,'Minimum density    =',f12.4,
     &         /10x,'Maximum density    =',f12.4,
     &         /10x,'Average density    =',f12.4,
     &         /10x,'Std Dev of density =',f12.4)
 2090   format(/10x,'Min and max densities in modified Patterson map >',
     &         /10x,'Minimum density    =',f12.4,
     &         /10x,'Maximum density    =',f12.4)


 9200   format(/10x,'** ERROR - input map too large for array ',
     &  'PATMAP **',/10x,'** Number of points in map = ',i10,' **',
     &  /10x,'** Maximum number          = ',i10,' **',/10x,
     &  '** Increase parameter MXPATP in rspsdim.inc **')

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine GETCMB
*
*       Function:
*       Load symmetry operators with sgs, ncs or both
*       
*       Comments:
*       
*       Arguments:
*       Errcod    (o)      error code
*                          errcod = 0 - no error
*                          errcod = 1 - too many sym ops
*       
*       
*       History:
*       Written November 27 1991 S. Knight
************************************************************************
        subroutine getcmb ( errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'
        include         'rspssym.inc'

*       Arguments
        integer         errcod

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Write s/r id
        call ttycr ( 1 )
        call ttyptl ( '   RSPS GETCMB >>' )

*       Initiate error code
        errcod = 0


*-----------------------------------------------------------------------
        if ( usesgs .and. usencs ) then
*       Use both sgs and ncs
*-----------------------------------------------------------------------

*       Total number of sym ops
          nopsym = nopsgs * nopncs

*       Space ?
          if(nopsym.gt.mxsym)then
            write(6,9000)mxsym,nopsym
            errcod = 1
            call errmsg ( 'Too many symmetry operations ' //
     &      'generated in GETCMB' , 0 )
            return
          endif
*       Get combined symmetry
          call cmbsym ( r_sgs , t_sgs , nopsgs , 
     &                  r_ncs , t_ncs , nopncs )
*       Set flags
          cmbnotin = .false.
          cmbisin  = .true.


*-----------------------------------------------------------------------
        else if ( usesgs ) then
*       Use only sgs
*-----------------------------------------------------------------------

*       Total number of symops
          nopsym = nopsgs

*       Get combined symmetry
          call cmbsym ( r_sgs , t_sgs , nopsgs , 
     &                  r_idt , t_nul , 1 )

*       Set flags
          cmbnotin = .false.
          cmbisin  = .true.


*-----------------------------------------------------------------------
        else if ( usencs ) then
*       Use only ncs
*-----------------------------------------------------------------------

*       Total number of symops
          nopsym = nopncs

*       Get combined symmetry
          call cmbsym ( r_idt , t_nul , 1 ,
     &                  r_ncs , t_ncs , nopncs ) 

*       Set flags
          cmbnotin = .false.
          cmbisin  = .true.

*-----------------------------------------------------------------------
        endif
*-----------------------------------------------------------------------

        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
 9000   format(/10x,'** ERROR - Too many symmetry operations generated',
     &  ', max = ',i5,' found = ',i5,' **')
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine GETTNF
*
*       Function:
*       Get transformations from real space to vector space via call
*       to s/r trnsf.
*       In MORE ATOMS and DoTran modes symmetry is applied to fixed 
*       positions. Transformations are then computed in s/r TRNSF.
*       Indeterminate axes are flagged (s/r INDAXS) and scan limits
*       reset along these directions. Also reset section flags to have 
*       section along indeterminate axis. Finally, the generated 
*       transformations are reported.
*
*       Comment:
*       Combined symmetry must be stored in r_sym,t_sym
*
*       Arguments:
*       Errcod   (o)    error code
*                       errcod = 0 - no error
*                       errcod = 1 - SYMAPP error
*                       errcod = 2 - TRNSF error
*                       errcod = 3 - all axes indeterminate 
*
*
*       Variables in commons used (i) or modified (o) by this routine
*       Name       i/o  Common   .incfile  
*       ----------------------------------
*       Vecset     (i)  chrpar    rspsctl
*       Dotran     (i)  flags     rspsctl
*       Doscan     (i)  flags     rspsctl
*       Nopsym     (i)  symm      rspssym
*       R_sym      (i)  symm      rspssym
*       T_sym      (i)  symm      rspssym
*       Nxyz2      (i)  crpsn     rspspos
*       Xyz2       (i)  crpsn     rspspos
*       Iprt       (i)  params    rspsctl
*       TnfNotIn   (o)  flags     rspsctl
*       TnfIsIn    (o)  flags     rspsctl
*       Ntrans     (o)  symm      rspssym
*       R_vec      (o)  symm      rspssym
*       T_vec      (o)  symm      rspssym
*       Mult       (o)  symm      rspssym
*       Vecnam     (o)  symm      rspssym
*       Scrlim     (o)  gridut    rspsgrd 
*       Scraxo     (o)  gridut    rspsgrd

*       History:
*       February 23 1995 - sdk
*       Section flags reset to have section along indeterminate axis
*       February 14 1994 - sdk
*       New MODEs
*       December 6 1993 - sdk
*       Check for indeterminate axes moved here (test is now on
*       vectors used) - sdk
*       Written October 31 1991 S. Knight
************************************************************************
        subroutine gettnf ( errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'
        include         'rspspos.inc'
        include         'rspsctl.inc'

*       Arguments
        integer         errcod         

*       Local variables
        integer         i , j , k , ict , iax , isec
        integer         iflag
        integer         nxs
        integer         indet(3)
        real            xsym( 3,mxtpsn )
        character       label( 3 ) * 1
        data            label  / 'X' , 'Y' , 'Z' /
        character*20    modus
        character       vecnam( 3 ) * 6
        data            vecnam/ 'Harker' , 'Cross ' , 'Any   ' /
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Write s/r id
        call ttycr ( 1 )
        call ttyptl ( '   RSPS GETTNF >>' )

*       Initiate error code
        errcod = 0

*       Initiate timer
        iflag = 0
        call timer ( iflag , ' ' )

*       Vectorset
        modus = vecset
        if ( dotran ) modus = 'MORE ATOMS'

*       Generate mapping from real to vector space
        if ( modus .eq. 'MORE ATOMS' ) then
*       Generate all symmetry related positions from fixed positions
          nxs = nopsym * nxyz2
          call symapp ( nopsym , r_sym , t_sym , nxyz2 , xyz2 , nxs ,
     &    xsym , errcod )
          if ( errcod .ne. 0 ) then
            errcod = 1
            return
          endif
        else if ( index ( modus , 'SINGLE' ) .eq. 0 .and. 
     &            index ( modus , 'SUPERPOSE' ) .eq. 0 ) then
*       Copy rigid fragment positions
          nxs = nxyz2
          call matcop ( xyz2 , xsym , 3 , nxyz2 )
        else 
*       When no fixed positions are used set dummy nxs = 1
          nxs = 1
        endif

*       Get transformations mapping from real space grid 2 to
*       Patterson grid 1
        call trnsf ( xsym , nxs , modus , errcod )
        if ( errcod .ne. 0 ) then
          errcod = 2
          return
        endif
        if ( ntrans .eq. 0 ) then
          iflag = 1
          call timer ( iflag , 'GETTNF' )
          tnfnotin = .true.
          tnfisin  = .false.
          return
        endif

*       Flag indeterminate axes
        call indaxs ( ntrans , r_vec , indet )

*       Reset scan limits if any axis is indeterminate
*       Also reset section flags to have section along indeterminate axis
        if ( doscan ) then
          ict = 0
          do i = 1 , 3
            if ( indet( i ) .eq. 0 ) then
              ict = ict + 1
              scrlim( 2*i-1 ) = 0.0
              scrlim( 2*i ) = 0.0
              write(6,2305)label( i )
              isec = i
            endif
          end do
          if ( ict .eq. 1 .and. scraxo( 3 ) .ne. isec ) then
            iax = scraxo( 3 )
            scraxo( 3 ) = isec
            if ( scraxo( 1 ) .eq. isec ) then
              scraxo( 1 ) = iax
            else
              scraxo( 2 ) = iax
            endif
          endif
          if ( ict .eq. 3 ) then
            write(6,2306)
            errcod = 3
            call errmsg ( 'All coordinates indeterminate' , 0 )
            return
          endif
        endif

*       Write out transformations
        if ( iprt .ge. 3 ) then
          write(6,2310)ntrans
          do i = 1 , ntrans
          write(6,2320)i , mult( i ) , vecnam( vectyp( i ) ) ,
     &    ( ( r_vec( k,j,i ) , j = 1 , 3 ) , 
     &    t_vec( k,i ) , k = 1 , 3 )
          end do
        else
          write(6,2322)ntrans
        endif


*       Report times
        iflag = 1
        call timer ( iflag , 'GETTNF' )

*       Set flags
        tnfnotin = .false.
        tnfisin  = .true.

        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
 2305   format(/10x,'Coordinates along ',a,' are indeterminate;',
     &  ' limits along this axis set to 0 0')
 2306   format(/10x,'Coordinates indeterminate along all axes - ',
     &  ' scan aborted')
 2310   format(/10x,'The following',i8,' transformations will be ',
     &  'used for mapping from real to vector space: ',/)
 2320   format(//10x,'Transformation #',i5,10x,'Multiplicity: ',i5,
     &  10x,'Type: ',a,//(10x,f10.7,2x,f10.7,2x,f10.7,10x,f12.5))
 2322   format(/10x,i8,' unique transformations have been generated')
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine GETCRD
*
*       Function:
*       Read coordinates from external file to RSPS positions array
*       Assign site numbers
*       
*       Arguments:
*       Errcod    (o)     error code
*                         errcod = 0 - no error
*                         errcod = 1 - file not defined
*                         errcod = 2 - file read error
*                         errcod = 3 - site number assignement error
*                         errcod = 4 - orhogonalization error
*       
*       
*       History:
*       March 27 1992 S. Knight
*       Modified to read PDB file using call to s/r RSRPDB.
*       Written November 28 1991 S. Knight
************************************************************************
        subroutine getcrd ( errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'
        include         'rspsgrd.inc'
        include         'rspspos.inc'
        include         'rspsfil.inc'

*       Arguments
        integer         errcod

*       Local variables
        logical         insite
        integer         i , j
        integer         iflag
        real            toort( 3,3 )
        real            xut( 3 )

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Write s/r id
        call ttycr ( 1 )
        call ttyptl ( '   RSPS READ >>' )

*       Initiate error code
        errcod = 0

*       Initiate timer
        iflag = 0
        call timer ( iflag , ' ' )

*       Check we have a file name
        if ( xinfil .eq. ' ' ) then
          errcod = 1
          call errmsg ( 'File not defined' , 0 )
          return
        endif

*       Read coordinates
        xintot = mxpick
        call rsrpdb ( lunxin , xinfil , xintot , sittot , xpick , 
     &  value , sitpnt , cell , title , errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'Couldn''t read coordinate file' , 0 )
          errcod = 2
          xintot = 0
          return
        endif

*       File read; report
        call ttypti ( '   RSPS READ >> Number of positions read > ' ,
     &  xintot , 4 , errcod )

*       Got any xyz ?
        if ( xintot .gt. 0 ) then

          if ( sittot .le. 0 ) then
*       Assign site numbers if not from file

            call ttyptl ( '   RSPS READ >> Assigning site numbers' )

*       If no Patterson has been read then set patgrd approx =
*       unit cell. This info is needed in s/r equsit to find 
*       equivalent positions, but the actual values don't matter.
            if ( patnotin ) call fveci ( cell , patgrd , 3 )

*       Assign site numbers
            call equsit ( xintot , xpick , sittot , sitbeg , sitend , 
     &      sitpnt , errcod )
            if ( errcod .ne. 0 ) then
              call errmsg ( 'Couldn''t assign site numbers' , 0 )
              errcod = 3
              return
            endif

          else
*       Site numbers from file; set pointers to start and end of site
*       Note that since positions are not sorted on site number site limits
*       may overlap. However as long as only the start of a site is ever used 
*       this is not be a problem.

*       Loop over sites
            do 100 i = 1, sittot
*       Set site start flag
              insite = .false.
*       Loop over positions
              do 50 j = 1 , xintot
*       Set pointer to start of site on first occurance
                if ( .not. insite .and. sitpnt( j ) .eq. i ) then
                  sitbeg( i ) = j
                  insite = .true.
                endif
*       Increase pointer to end of site when found
                if ( sitpnt( j ) .eq. i ) sitend( i ) = j
 50           continue
 100        continue

          endif


*       Set matrix for converting fractional crystallographic
*       coordinates to Angstrom orthogonal in PDB standard frame
          call ortho ( cell , 1 , 'TOORT' , toort , errcod )
          if ( errcod .ne. 0 ) then
            call errmsg ( 'Failed getting orthogonalization matrix' , 
     &      0 )
            errcod = 4
            return
          endif

*       List coordinates
          write(6,2000)
          do i = 1 , xintot
            call vmtply ( toort , xpick( 1,i ) , xut )
            write(6,2100) i , (xpick(j,i),j=1,3) ,
     &      xut , value( i ) , sitpnt( i )
          end do

        endif


*       Close coordinate file
        close ( lunxin )


*       Report times
        iflag = 1
        call timer ( iflag , 'READ' )



        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
 2000   format(/10x,'Positions read from file > ',//12x,'PosnN     ',
     &  ' Fractional coordinates     Angstrom coordinates',
     &  '      Score    Site',/12x,'-----     ',
     &  '------------------------  ------------------------  ',
     &  '---------  ----')
 2100   format(10x,i5,6x,3f8.4,2x,3f8.2,2x,f8.2,4x,i4)
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       RSPS PDB file read and write routines
*       
*       s/r RSWPDB        write positions to PDB file
*       s/r RSRPDB        read positions from PDB file
*
*       Written March 26-27 1992 S. Knight
************************************************************************
************************************************************************
*
*       Subroutine RsWpdb
*
*       Function:
*       Write RSPS positions to PDB format file
*
*       Arguments:
*       Lunpdb    (i)     logical unit to use
*       Filnam    (i)     file name
*       Xtot      (i)     number of positions to write
*       X         (i)     fractional crystallographic coordinates
*       V         (i)     scores
*       S         (i)     site numbers
*       Cell      (i)     unit cell parameters
*       Title     (i)     title
*       Ouput     (i)     output type flag
*                         POS - positions
*                         PEA - peaks
*                         SET - getset set
*                         FIX - fixxyz positions
*       Errcod    (o)     error code
*                         errcod = 0 - no error
*                         errcod = 1 - file open error
*                         errcod = 2 - orhogonalization error
*
*
*       History:
*       93 11 16 - sdk
*       Fortran open changed to ccpdpn call 
*       Written March 26 1992 S. Knight
************************************************************************
        subroutine rswpdb ( lunpdb , filnam , xtot , x , v , s , cell ,
     &  title , output , errcod )

        implicit none

*       Arguments
        integer         lunpdb
        character*(*)   filnam
        integer         xtot
        real            x( 3,xtot ) , v( xtot ) 
        integer         s( xtot )
        real            cell( 6 )
        character*(*)   title
        character*(*)   output
        integer         errcod , ifail

*       Local variables
        integer         i
        real            toort( 3,3 ) , deort( 3,3 )
        real            xut( 3 ) , bfac
        character       atyp*2 , rtyp*3 , date*8

*       External functions
        integer         slnblk
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Initiate atom and residue type
        if ( output .eq. 'PEA' ) then
          atyp = ' O'
          rtyp = 'OHH'
        else
          atyp = 'HG'
          rtyp = 'MTL'
        endif

*       Dummy B-factor
        bfac = 20.0

*       Open output file
        ifail= 1
        call ccpdpn ( -lunpdb , filnam , 'UNKNOWN' , 'F' , 0 , ifail )
        if ( ifail .ne. 1 ) then
          call errmsg ( 'Couldn''t open xutfile file' , 0 )
          call wrnmsg ( 'Coordinate file not written' )
          errcod = 1
          return
        endif


*       Set matrices for converting fractional crystallographic
*       coordinates to Angstrom orthogonal in PDB standard frame
*       and vice verse
        call ortho ( cell , 1 , 'TOORT' , toort , errcod )
        if ( errcod .ne. 0 ) then
          errcod = 2
          return
        endif
        call ortho ( cell , 1 , 'DEORT' , deort , errcod )
        if ( errcod .ne. 0 ) then
          errcod = 2
          return
        endif

*       Write header card(s)
        write ( lunpdb,2010 ) 'HEADER' , title( :slnblk( title ) )

*       Write stamp
        call ccpdat ( date )
        write ( lunpdb,2010 ) 'REMARK' , 'File written by RSPS ' //
     &  'on ' // date

*       Write cell card
        write ( lunpdb,2020 ) 'CRYST ' , cell

*       Write scale cards
        write ( lunpdb,2030 ) 'SCALE1' , ( deort( 1,i ) , i = 1 , 3 )
        write ( lunpdb,2030 ) 'SCALE2' , ( deort( 2,i ) , i = 1 , 3 )
        write ( lunpdb,2030 ) 'SCALE3' , ( deort( 3,i ) , i = 1 , 3 )

*       Write column descriptor remark
        if ( output .eq. 'POS' ) then
          write ( lunpdb,2040 )
        else if ( output .eq. 'PEA' ) then
          write ( lunpdb,2041 )
        else if ( output .eq. 'SET' ) then
          write ( lunpdb,2042 )
        else if ( output .eq. 'FIX' ) then
          write ( lunpdb,2043 )
        endif

*       Write atom cards
        do 100 i = 1 , xtot
          call vmtply ( toort , x( 1,i ) , xut )
          write ( lunpdb,2050 ) 'ATOM  ' , i , atyp , rtyp , 
     &    s( i ) , xut , v( i ) , bfac
 100    continue

*       Write END record
        write ( lunpdb,'(a3)' ) 'END'
*       Close file
        close ( lunpdb )

        return
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
 2010   format(a6,1x,a)
 2020   format(a6,3f9.3,3f7.2)
 2030   format(a6,4x,3f10.5)
 2040   format('REMARK  POSNN   TYPE   SITE       X       Y       Z',
     &  '    SCORE')
 2041   format('REMARK  PEAKN   TYPE   SITE       X       Y       Z',
     &  '    HEIGHT')
 2042   format('REMARK  POSNN   TYPE   SITE       X       Y       Z',
     &  '    SCORE')
 2043   format('REMARK  POSNN   TYPE   SITE       X       Y       Z')
 2050   format(a6,i5,2x,a2,2x,a3,2x,i4,4x,3f8.3,2f6.2)
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine RsRpdb
*
*       Function:
*       Read RSPS positions from PDB format file
*
*       Arguments:
*       Lunpdb    (i)     logical unit to use
*       Filnam    (i)     file name
*       Xtot      (i/o)   on input : max number of positions to read
*                         on output: number of positions read
*       Stot      (o)     number of sites (0 if sites not defined)
*       X         (o)     fractional crystallographic coordinates
*       V         (o)     scores
*       S         (o)     site numbers
*       Cell      (o)     unit cell parameters
*       Title     (o)     title
*       Errcod    (o)     error code
*                         errcod = 0 - no error
*                         errcod = 1 - file open error
*                         errcod = 2 - file read error
*                         errcod = 3 - bad CRYST card in PDB file
*                         errcod = 4 - bad SCALE card in PDB file
*                         errcod = 5 - missing CRYST or SCALE card
*                         errcod = 6 - bad ATOM record
*
*
*       History:
*       Written March 26 1992 S. Knight
************************************************************************
        subroutine rsrpdb ( lunpdb , filnam , xtot , stot , x , v , s , 
     &  cell , title , errcod )

        implicit none

*       Arguments
        integer         lunpdb
        character*(*)   filnam
        integer         stot
        integer         xtot
        real            x( 3,xtot ) , v( xtot ) 
        integer         s( xtot )
        real            cell( 6 )
        character*(*)   title
        integer         errcod

*       Local variables
        logical         nositn
        logical         cdef
        integer         sdef
        integer         i , j
        integer         ict
        integer         ifail
        integer         s1
        real            x1( 3 ) , v1
        real            deort( 3,3 )
        character*80    line

*       External functions
        integer         slnblk
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Open file
        ifail = 1
        call ccpdpn ( -lunpdb , filnam , 'READONLY' , 'F' , 0 , ifail )
        if ( ifail .ne. 1 ) then
          line = 'File not found > ' // filnam( :
     &    max ( 1,slnblk ( filnam ) ) ) 
          call errmsg ( line , 0 )
          errcod = 1
          return
        endif

*       Initiate site number indicator
        nositn = .false.

*       Initiate site and position counters
        stot = 0
        ict  = 0

*       Initiate cryst and scale card indicators
        cdef = .false.
        sdef = 0


*       Read records
 500    continue

          read ( lunpdb , '(a)' , iostat = errcod , end = 600 ) line
          if ( errcod .ne. 0 ) then
            close ( lunpdb )
            errcod = 2
            call errmsg ( 'Couldn''t read file' , 0 )
            return
          endif

          if ( line( 1:6 ) .eq. 'HEADER' ) then
*       Title record
            call ttyptl ( '   RSPS READ >> Title > ' //
     &      line( 7:max ( 7,slnblk ( line ) ) ) )
            title = line( 7:slnblk( line ) )

          else if ( line( 1:5 ) .eq. 'CRYST' ) then
*       Unit cell parameters
            read ( line ( 7: ) , '(3f9.3,3f7.2)' , iostat = 
     &      errcod ) cell
            if ( errcod .ne. 0 ) then
              close ( lunpdb )
              call errmsg ( 'Couldn''t interpret CRYST card' , 0 )
              errcod = 3
              return
            endif
            call ttyptl ( '   RSPS READ >> Cell > ' //
     &      line( 7:max ( 7,slnblk ( line ) ) ) )
            cdef = .true.

          else if ( line( 1:5 ) .eq. 'SCALE' ) then
*       Transformation from stored orthogonal coordinates to fractional
*       crystallographic coordinates
            call schtoi ( i , line ( 6:6 ) , errcod )
            if ( errcod .ne. 0 ) then
              call errmsg ( 'Illegal SCALE card > ' // line ( 1:6 ) ,
     &        0 )
              close ( lunpdb )
              errcod = 4
              return
            endif
            if ( i .ge. 1 .and. i .le. 3 ) then
              read ( line , '(t11,3f10.5)' , iostat = errcod )
     &        ( deort( i , j ) , j = 1 , 3 )
              if ( errcod .ne. 0 ) then
                call errmsg ( 'Couldn''t interpret SCALE card' , 0 )
                close ( lunpdb )
                errcod = 4
                return
              endif
              call ttyptl ( '   RSPS READ >> Deortho > ' //
     &        line( 7:max ( 7,slnblk ( line ) ) ) )
              sdef = sdef + 1
            else
              call errmsg ( 'Illegal SCALE card : ' // line ( 1:6 ) , 
     &        0 )
              close ( lunpdb )
              errcod = 4
              return
            endif


          else if ( line( 1:4 ) .eq. 'ATOM' ) then
*       Atom records

*       Check cell and orthogonalization matrix have been read
            if ( .not. cdef ) then
              call errmsg ( 'Missing CRYST card' , 0 )
              close ( lunpdb )
              errcod = 5
              return
            endif
            if ( sdef .ne. 3 ) then
              call errmsg ( 'Missing SCALE card(s)' , 0 )
              close ( lunpdb )
              errcod = 5
              return
            endif

*       Transfer site number, coordinates and score value
            read ( line , '(6x,i5,11x,i4,4x,3f8.3,f6.2)' , 
     &      iostat = errcod ) i , s1 , x1 , v1
            call ttyptl ( '   RSPS READ >> Position > ' //
     &      line( 5:max ( 5,slnblk ( line ) ) ) )
            if ( errcod .ne. 0 ) then
              call errmsg ( 'Couldn''t interpret ATOM records' , 0 )
              close ( lunpdb )
              errcod = 6
              return
            endif

*       Count and store
            if ( ict .lt. xtot ) then

              ict = ict + 1
              call vmtply ( deort , x1 , x( 1,ict ) )
              v( ict ) = v1
              s( ict ) = s1
              stot = max ( stot , s1 )
              if ( s1 .eq. 0 ) nositn = .true.

            else

              call errmsg ( 'Positions array full' , 0 )
              call errmsi ( 'Positions list truncated, ' // 
     &        'last stored = ' , ict , 4 , 0 )
              goto 600

            endif

          endif

*       Next record
        goto 500


*       File read; set position (and site) counter and return
 600    xtot = ict
        if ( nositn ) stot = 0
        close ( lunpdb )
        errcod = 0
        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine LODSYM
*
*       Function:
*       Load symmetry information data structures
*       
*       Arguments:
*       Gttran   (i)      Switch for computation of vector transformations
*       Errcod   (o)      error code
*                         errcod = 0 - no error
*                         errcod = 1 - Space group not defined
*                         errcod = 2 - failed loading sgs
*                         errcod = 3 - failed loading ncs
*                         errcod = 4 - failed combining symmetry
*                         errcod = 5 - failed loading transformations
*       
*       
*       History:
*       March 18 1992 - sdk
*       Added logical argument GTTRAN. Vector transformations now
*       computed only if GTTRAN is .true.
*       Written February 1992 S. Knight
************************************************************************
        subroutine lodsym ( gttran , errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'
        include         'rspssym.inc'


*       Arguments
        logical         gttran
        integer         errcod

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Set default symmetry flags
        if ( .not. usesgs .and. .not. usencs ) then
          call wrnmsg ( 'Symmetry was not set - ' //
     &    'space group symmetry switched ON' )
          usesgs = .true.
        endif

*       Check space group is known
        if ( usesgs .and. ispgrp .le. 0 ) then
*       Space group not defined
          call errmsg ( 'Requested action requires symmetry: ' //
     &    'SPACE GROUP NOT DEFINED' , 0 )
          errcod = 1
          return
        endif

*       Combine symmetry
        call getcmb ( errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'GETCMB failed' , 0 )
          errcod = 4
          return
        endif

*       Return if vector transformations not requested
        if ( .not. gttran ) return

*       Get real to vector space transformations
        call gettnf ( errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'GETTNF failed' , 0 )
          errcod = 5
          return
        endif

        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine HRKPLN
*
*       Function:
*       Determine Harker plane equations
*       
*       Arguments:
*       Errcod   (o)      error code
*                         errcod = 0 - no error
*                         errcod = 1 - ORTHO failure
*                         errcod = 2 - GETCMB failure
*                         errcod = 3 - GETTNF failure
*                         errcod = 4 - Couldn't solve for equations
*       
*       Result:
*       Harker plane equations are stored as normal vector & distance
*       to origin in array hplane, common /symm/.
*
*       History:
*       Written February 1996 S. Knight
************************************************************************
        subroutine hrkpln ( errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'


*       Arguments
        integer         errcod

*       Local variables
        logical         savsgs , savncs
        integer         i , j , k , ict , seed
        integer         np
        parameter       ( np = 3 )
        real            xyz( 3 ) , tmp1( 3 )
        real            uvw( 3,np ) , toort( 3,3 )
        character*20    savvec

*       External functions
        real            random
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Set matrices for converting fractional crystallographic
*       coordinates to orthogonal and back
        call ortho ( cell , 1 , 'TOORT' , toort , errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'Failed getting orthogonalization matrix' , 0 )
          errcod = 1
          return
        endif

*       Set sgs on, ncs off, vecset = 'SINGLE ATOMS'
        savsgs = usesgs
        savncs = usencs
        savvec = vecset
        usesgs = .true.
        usencs = .false.
        vecset = 'SINGLE ATOMS'

*       Combine symmetry
        call getcmb ( errcod )
        if ( errcod .ne. 0 ) then
          call errmsg (  'GETCMB failed' , 0 )
          usesgs = savsgs
          usencs = savncs
          vecset = savvec
          errcod = 2
          return
        endif

*       Get Harker vector transformations
        call gettnf ( errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'GETTNF failed' , 0 )
          usesgs = savsgs
          usencs = savncs        
          vecset = savvec
          errcod = 3
          return
        endif

*       Loop over Harker vector operators & generate plane equation
        nplanes = ntrans
        call matclr ( hplane , 4 , nplanes )
        do i = 1 , nplanes
          ict = 0
 10       seed = -1
*       Generate np random points on each Harker plane
          do j = 1 , np
            do k = 1 , 3 
              xyz( k ) = random(seed)
            end do
            call vmtply ( r_vec( 1,1,i ) , xyz , tmp1 )
            call vecadd ( t_vec( 1,i ) , tmp1 , uvw( 1,j ) , 3 )
          end do

*       Get plane equation
          call pnt2pln ( uvw , hplane( 1,i ) , errcod )
          if ( errcod .eq. 1 ) then
*       Ill conditioned; try with new set of points
            ict = ict + 1
            if ( ict .le. 5 ) then
              goto 10
            else
*       Failed five times; give up
              usesgs = savsgs
              usencs = savncs
              vecset = savvec
              errcod = 4
              return
            endif
          endif
        end do

        usesgs = savsgs
        usencs = savncs
        vecset = savvec
        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine CENTR1
*
*       Function:
*       Test if any symmetry operations in rot and transl differ in 
*       translation components only. If so store translations in cent.
*       
*
*       Arguments:
*       Rot       (i)      3 x 3 rotation matrix
*       Transl    (i)      3 x 1 translation vector
*       Nsym      (i)      number of symmetry operations
*       Cent      (o)      centering translations
*       Icen      (o)      number of centering translations
*
*
*       History:
*       November 5 1991 - sdk
*       CELL argument removed. Fractional translations expected.
*       March 18 1988 - sdk
*       Centering in cent
*       Written March 13 1988 S. Knight
************************************************************************
        subroutine centr1 (rot,transl,nsym,cent,icen)

        implicit none

        include         'rspsdim.inc'

*       Arguments
        integer         nsym , icen
        real            rot(3,3,nsym) , transl(3,nsym)
        real            cent(3,4)

*       Local variables
        logical         extra(mxsgs)
        integer         i , j , k , ict , match
        real            rcop(3,3,mxsgs) , tcop(3,mxsgs)
        real            tvec(3)
        real            tol

        data            tol/0.001/
*-----------------------------------------------------------------------

        if(nsym.gt.mxsgs)
     &  stop' ERROR - ARRAY BOUNDS EXCEEDED IN S/R CENTR1'

        ict = 0
        do i=1,nsym
        extra(i)=.false.
        end do


        icen=1
        do i=1,3
        cent(i,1)=0.0
        end do
        do 20 i=1,nsym-1
        if(.not.extra(i))then
          ict=ict+1
          call matcop ( rot(1,1,i),rcop(1,1,ict),3,3 )
          call matcop ( transl(1,i),tcop(1,ict),3,1 )
        else
          goto 20
        endif
        do 10 j=i+1,nsym
        if(.not.extra(j))then
          call matcmp ( rot(1,1,i),rot(1,1,j),3,3,match,tol )
          if(match.eq.9)then
            extra(j)=.true.
            do k=1,3
              tvec(k)=transl(k,j)-transl(k,i)
              tvec(k)=mod(tvec(k),1.0)
              if(tvec(k).lt.0)tvec(k)=tvec(k)+1.0
           end do
           do k=1,icen
             call matcmp ( tvec,cent(1,k),3,1,match,tol )
             if(match.eq.3)goto 10
           end do
           icen=icen+1
           if (icen.gt.4) call errmsg ('MORE THAN THREE CENTERING' //
     & 'TRANSLATIONS FOUND IN S/R CENTR1',1)
           call matcop ( tvec,cent(1,icen),3,1 )
          endif
        endif
 10     continue
 20     continue
        if(.not.extra(nsym))then
          ict=ict+1
          call matcop ( rot(1,1,i),rcop(1,1,ict),3,3 )
          call matcop ( transl(1,i),tcop(1,ict),3,1 )
        endif
        do i=1,nsym
          call matclr ( rot(1,1,i),3,3 )
          call matclr ( transl(1,i),3,1 )
        end do
        do i=1,ict
          call matcop ( rcop(1,1,i),rot(1,1,i),3,3 )
          call matcop ( tcop(1,i),transl(1,i),3,1 )
        end do
        nsym=ict
        return
        end
*********************************************************************SDK
************************************************************************
*
*       Logical function CHECKID
*
*       Function:
*       Test if input 3 x 3 matrix a is identity and 3 x 1 vector t is
*       null vector. Return CHEKID = .false. if test fails
*
*       Arguments:
*       a         (i)     3 x 3 matrix
*       t         (i)     3 x 1 vector
*
* 
*       History:
*       Written October 1987 S. Knight
************************************************************************
        logical function checkid ( a , t )

        implicit none

*       Arguments
        real            a(3,3) , t(3)

*       Local variables
        integer         i
        integer         match
        real            ident(3,3)
        real            tol

        data            ident/ 1.,0.,0.,  0., 1.,0.,  0.,0., 1./
        data            tol/0.00001/
*-----------------------------------------------------------------------
        checkid = .false.
        do i = 1 , 3
        if ( abs ( t(i) ) .gt. 0.0001 ) return
        end do
        call matcmp ( a , ident , 3 , 3 , match , tol )
        if ( match .ne. 9 ) return
        checkid = .true.
        return
        end
*********************************************************************SDK
************************************************************************
*       Rsps coordinate transforming routines
*       Contains modules trnsf,vxtrct,symapp,cmbsym,cvcalc,
*       hvcalc,maxu,polaxs,indaxs
*
*       Trnsf   compute transformations mapping from search space to
*               vector space
*
*       Vxtrct  extract unique transformations
*
*       Symapp  apply symmetry to input positions; extract unique set
*
*       Cmbsym  Combine space group symmetry and non crystallographic
*               symmetry
*
*       Cvcalc  compute all unique vectors between positions in two 
*               lists (cross vectors)
*
*       Hvcalc  compute all unique vectors between positions in one list
*               (harker vectors)
*
*       Maxu    determine maximum length of a vector given a 
*               transformation and extreme limits
*
*       Polaxs  find and flag polar axes
*
*       Indaxs  find and flag indeterminate axes
************************************************************************
************************************************************************
*       Subroutine TRNSF
*
*
*       Function:
*       Generate transformations from real to vector space. The 
*       symmetry operations used for generating the transformations
*       should be stored in R_sym, t_sym in common /symm/ 
*       (rspssym.inc). The symops are expected to operate on/generate
*       fractional coordinates and the generated transformations
*       likewise.
*       
*
*       Arguments :
*       Xyz          (i)  Fixed postions for use in MORE and TRANSLATE
*                         vectorsets (dummy in SINGLE vectorsets)
*       Nxyz         (i)  number of fixed positions
*       What         (i)  vectorset 
*                         Allowed vectorsets are
*                         SINGLE ATOMS
*                         MORE ATOMS (includes polarscan)
*                         TRANSLATE ATOMS
*                         SINGLE MOLECULES
*                         MORE MOLECULES
*                         TRANSLATE MOLECULES
*       Errcod       (o)  error code
*                         errcod = 0 - no error
*                         errcod = 1 - VXTRCT error (trnsf ovflw)
*                         errcod = 2 - call with unknown vectorset
*
*       History:
*       February 16 1994 - sdk
*       The method for throwing out null matrices, and for selecting 
*       structure-invariant vectors changed.
*
*       February 14 1994 - sdk
*       New MODEs (SINGLE/MORE/TRANSLATE ATOMS/MOLECULES)
*
*       December 9 1993 - sdk
*       Multiplicity normalization
*
*       December 3 1993 - sdk
*       FIND mode added
*
*       August 14 1992 - sdk
*       Transformations for rigid fragment translation search added.
*
*       February 11 1992 - sdk
*       Determination of polar (indeterminate) axes moved to s/r
*       polaxs.
*
*       October 31 1991 - sdk
*       INDET output to common /symm/ instead of as argument.
*
*       August 24 1991 - sdk
*       Clear transformation and multiplicity arrays on each entry.
*
*       October 14 1988 - sdk
*       Argument sext added to allow calculation of maximum separation
*       between vectors generated by two different transformations for
*       use in selection of the same.
*       Add check for indeterminate axes (indet).
*       Flag for harker or cross vector (vectyp).
*       Modified to set up transformations for both harker and 
*       cross scan.
*
*       March 22 1988 - sdk
*       Grid1 and cell as arguments; common/maphdr1/ deleted.
*
*       March 10 1988 - sdk
*       Grid2 argument added so that g1 and g2 can be computed here
*       instead of in main program.
*
*       February 13 1988 - sdk
*       Patterson symmetry computed in main program, stored in common
*       /symm/. Grids and cell in common/gridin/,/gridut/,/ucell/.
*
*       Written Oct 1987 S. Knight
************************************************************************
        subroutine trnsf ( xyz,nxyz,what,errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'


*       Arguments
        integer         nxyz
        real            xyz(3,nxyz)
        character*20    what
        integer         errcod

*       Local variables
        integer         i , j , k , l , m , ipos
        integer         nr
        integer         minmul
        real            tol0
        real            tol1
        real            tstlen
        real            tstvec(3)
        real            aulim(6)
        real            u(3)
        real            tmat1(3,3) , tmat2(3,3)
        real            tvec1(3) , tvec2(3) , tvec3(3)
        real            xrot(3)

*       External functions
        real            vlen2
        integer         lenstr

*       Data statements
*       Minimum accepted vector length generated by a transformation
*       (Patterson grid units)
*       tol0 is for throwing out null matrices
*       tol1 is for selecting structure-invariant vectors
*       The apropriate value for tol1 will depend primarily on the angle
*       between the ncs axis and the crystallographic axis.
*       If a significant translation peak is to arise from ncs a 
*       significant number of vectors must cluster within a reasonable
*       volume. The question is what is significant and what is reasonable.
*       A possible treatment is to require all vectors from some fraction
*       of the volume of the au to cluster to within, say 5 A (resolution
*       dependent). Actually, the cutoff should probably be on a per-cell-
*       axis basis since the cell edges could vary considerably in length.
*       For the time being a crude estimate of tol1 based on spinach
*       Rubisco is used - a value of xx for tol1 was then needed in
*       order to generate the pseudo F-centering peak.
        data            tol0/5.5/
        data            tol1/18.0/
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Init transformation and multiplicity arrays
        ntrans = 0
        call imatclr ( mult , mxvec , 1 )
        call imatclr ( vectyp , mxvec , 1 )
        call matclr ( t_vec , 3 , mxvec )
        call matclr ( r_vec , 9 , mxvec )


*       Get limits of asymmetric unit (may actually be more than
*       one au for high-symmetry space groups).
*       Use "remote corner" of au to set points for testing if
*       a matrix is "close" to the null matrix or not.
        call stlim ( ispgrp , aulim )
        do i = 1 , 3
          tstvec( i ) = aulim( 2*i )
        end do


        if ( what .eq. 'SINGLE ATOMS' ) then

*       Loop over symmetry operators and generate all transformations
*       (Rij,tij) mapping a point in real space onto its Harker vectors:
*
*       u(x) = Ri * x + ti - (Rj * x + tj)
*       u(x) = (Ri - Rj) * x + ( ti - tj )
*
******************************************
*                                        *
*       u(x) = Rij * x + tij             *
*                                        *
******************************************


        do 100 i = 1 , nopsym - 1
        do 50  j = i + 1 , nopsym

          do k = 1 , 3
            tvec1( k ) = t_sym( k,i ) - t_sym( k,j ) 
            tvec1( k ) = mod ( tvec1( k ) , 1.0 )
            if ( abs ( tvec1( k ) ) .lt. 0.0001 ) tvec1( k ) = 0.0
            if ( tvec1( k ) .lt. 0.0 ) tvec1( k ) = tvec1( k ) + 1.0
            do l = 1 , 3
              tmat1( l,k ) = r_sym( l,k,i ) - r_sym( l,k,j )
            end do
          end do

*       Compute maximum length of vector generated by rotational part.
*       The test point in tstvec is at upper limits of au; normally 
*       this will be the point furthest from the origin in a real search 
*       scan. If the vector is too close to the origin (tolerance tol0)
*       this transformation is skipped since it will only add a (almost)
*       constant term to the score, independent of the search position x.
*       If the matrix is not a null matrix (within tolerance tol0)
*       compare with previously found transformations to see if this
*       is new.

          call vmtply ( tmat1 , tstvec , u )
          tstlen = vlen2 ( u , cell , cosang )
          if ( tstlen*patgrd(1)/cell(1) .ge. tol0 .or. 
     &         tstlen*patgrd(2)/cell(2) .ge. tol0 .or.
     &         tstlen*patgrd(3)/cell(3) .ge. tol0     ) then

            call vxtrct ( tmat1,tvec1,nr,errcod )
            if ( errcod .ne. 0 ) return

            if ( nr .eq. 0 ) then
              mult( ntrans ) = 1
              vectyp( ntrans ) = 1
              if ( usencs ) then
                if ( ncspnt(i) .ne. ncspnt(j) ) then
                  vectyp( ntrans ) = 2
                else
                  vectyp( ntrans ) = 1
                endif
              endif
            else
              mult( nr ) = mult( nr ) + 1
              if ( ncspnt(i) .eq. ncspnt(j) ) vectyp( nr ) = 1
            endif

          endif

 50     continue
 100    continue

        else if ( what .eq. 'MORE ATOMS' .or. dotran ) then

*       Loop over symmetry operators and generate all transformations
*       (Rij,tij) mapping a point in real space onto its cross vectors 
*       with the input positions:
*
*       u(x) = Ri * x + ti - (Rj * xfix + tj)
*       u(x) = Ri * x + ( ti - (Rj * xfix + tj) )
*
******************************************
*                                        *
*       u(x) = Ri * x + tijk             *
*                                        *
******************************************
*
*       Observe that no symmetry is applied to the input positions here;
*       so that effectively Rj = I and tj = 0. However, symmetry may be
*       applied (and usually is) in the calling routine.

        do 250 i = 1 , nopsym
          do 200 j = 1 , nxyz

            do k = 1 , 3
              tvec1( k ) = t_sym( k,i ) - xyz( k,j )
              tvec1( k ) = mod ( tvec1( k ) , 1.0 )
              if ( abs ( tvec1( k ) ) .lt. 0.0001 ) tvec1( k ) = 0.0
              if ( tvec1( k ) .lt. 0.0 ) tvec1( k ) = tvec1( k ) + 1.0
            end do

*       Possible new vector; compare with previously found
*       transformations to see if it is
            call vxtrct ( r_sym( 1,1,i ),tvec1,nr,errcod )

            if ( errcod .ne. 0 ) return
            if ( nr .eq. 0 ) then
              mult( ntrans ) = 1
              vectyp( ntrans ) = 2
            else
              mult( nr ) = mult( nr ) + 1
            endif

 200      continue
 250    continue


        else if ( what .eq. 'TRANSLATE ATOMS' ) then

*       Find position of a group of atoms
*
*       Loop over symmetry operators and generate all transformations
*       (Rijkl,tijkl) giving the Harker and cross vectors associated
*       with translation tx of the rigid fragment x
*
*       u(tx) = Ri * ( xk + tx ) + ti - ( Rj * ( xl + tx ) + tj )
*
*******************************************************************
*                                                                 *
*       u(tx) = (Ri - Rj) * tx + (ti - tj) + (Ri * xl - Rj * xm)  *
*                                                                 *
*******************************************************************


        do 400 i = 1 , nopsym - 1
          do 350 j = i + 1 , nopsym

*       First form the difference matrices and position-independent
*       difference vectors
            do k = 1 , 3
              tvec1( k ) = t_sym( k,i ) - t_sym( k,j ) 
              do l = 1 , 3
                tmat1( l,k ) = r_sym( l,k,i ) - r_sym( l,k,j )
              end do
            end do

*       Compute maximum length of vector generated by rotational part.
*       The test point in tstvec is at upper limits of au; normally 
*       this will be the point furthest from the origin in a real search 
*       scan. If the vector is too close to the origin (tolerance tol0)
*       this transformation is skipped since it will only add a (almost)
*       constant term to the score, independent of the search position x.
*       If the matrix is not a null matrix (within tolerance tol0)
*       compare with previously found transformations to see if this
*       is new.

            call vmtply ( tmat1 , tstvec , u )
            tstlen = vlen2 ( u , cell , cosang )
            if ( tstlen*patgrd(1)/cell(1) .lt. tol0 .and. 
     &           tstlen*patgrd(2)/cell(2) .lt. tol0 .and.
     &           tstlen*patgrd(3)/cell(3) .lt. tol0     ) goto 350

*       Now loop over positions in fragment and form complete
*       difference vectors by adding the position dependent term
            do 320 k = 1 , nxyz
              call vmtply ( r_sym( 1,1,i ) , xyz( 1,k ) , xrot )
              call vecadd ( tvec1 , xrot , tvec2 , 3 )
              do 310 l = 1 , nxyz
                call vmtply ( r_sym( 1,1,j ) , xyz( 1,l ) , xrot )
                call vecdif ( tvec2 , xrot , tvec3 , 3 )
                do m = 1 , 3
                  tvec3( m ) = mod ( tvec3( m ) , 1.0 )
                  if ( abs ( tvec3( m ) ) .lt. 0.0001 ) 
     &            tvec3( m ) = 0.0
                  if ( tvec3( m ) .lt. 0.0 ) tvec3( m ) = 
     &            tvec3( m ) + 1.0
                end do

*       Test transformation
                call vxtrct ( tmat1,tvec3,nr,errcod )
                if ( errcod .ne. 0 ) return

                if ( nr .eq. 0 ) then
                  mult( ntrans ) = 1
                  if ( k .ne. l ) then
                    vectyp( ntrans ) = 2
                  else
                    vectyp( ntrans ) = 1
                    if ( usencs ) then
                      if ( ncspnt(i) .ne. ncspnt(j) ) then
                       vectyp( ntrans ) = 2
                      else
                        vectyp( ntrans ) = 1
                      endif
                    endif
                  endif
                else
                  mult( nr ) = mult( nr ) + 1
                  if ( ncspnt(i) .eq. ncspnt(j) .and. 
     &            k .eq. l ) vectyp( nr ) = 1
                endif


 310          continue
 320        continue
 350      continue
 400    continue
 
        else if ( what .eq. 'SINGLE MOLECULES' ) then

*       Find position of molecule with ncs axis (parallel to 
*       crystallographic) axis using structure invariant vectors
*
*       Loop over symmetry operators and generate all transformations
*       where Ri(sg) * Rk(nc) = Rj(sg) * Rl(nc) (the corresponding
*       vectors are then independent of x and only depend on the 
*       position of the ncs molecule):
*
*       u = (Ri(sg)*Rk(nc) - Rj(sg)*Rl(nc)) * x + 
*            Ri(sg)*tk(nc) -
*            Rj(sg)*tl(nc) +
*            ti(sg) - tj(sg)
*       Then
*       u =  Ri(sg)*tk(nc) - Rj(sg)*tl(nc) + ti(sg) - tj(sg)
*
*       Now, tk(nc) = (I - Rk(nc)) * tx where tx is the position of
*       the ncs axis
*  
*       So
*
*       u = Ri(sg)*(I - Rk(nc))*tx - Rj(sg)*(I - Rl(nc))*tx + ti(sg) - tj(sg)
*       u = (Ri(sg) - Rj(sg) - 
*            Ri(sg)*Rk(nc) + Rj(sg)*Rl(nc))*tx + ti(sg) - tj(sg)
*
*******************************************************
*                                                     *
*       u = (Ri(sg) - Rj(sg))*tx + ti(sg) - tj(sg)    *
*                                                     *
*******************************************************

        do 500 i = 1 , nopsym - 1
          do 450 j = i + 1 , nopsym

            do k = 1 , 3
              do l = 1 , 3
                tmat1( l,k ) = r_sym( l,k,i ) - r_sym( l,k,j )
              end do
            end do


*       Compute maximum length of structure-dependent vector generated by 
*       difference matrix. The test point in tstvec is at upper limits of 
*       the au; normally this will be the point furthest from the origin 
*       in a real search scan. 
*       Here we are only interested in structure invariant vectors so
*       we want this difference matrix to be close to the null matrix.
*       I am not sure what the optimal cutoff here would be; selecting
*       only those cases were the difference matrix is exactly the
*       null matrix would exclude all cases were the ncs is not
*       exactly parallell to a crystallographic axis. A value of 18.0 for
*       tol1 was needed in the case of spinach Rubisco in order to
*       generate the pseudo F-centering peaks; this is the currently used
*       value.

            call vmtply ( tmat1 , tstvec , u )
            tstlen = vlen2 ( u , cell , cosang )

            if ( tstlen*patgrd(1)/cell(1) .ge. tol1 .or. 
     &           tstlen*patgrd(2)/cell(2) .ge. tol1 .or.
     &           tstlen*patgrd(3)/cell(3) .ge. tol1     ) goto 450


*       Null matrix; continue
*       Construct Harker vector from sgs embedded in operators
*       i and j

            do k = 1 , 3
              tvec1( k ) = t_sgs( k,sgspnt(i) ) - t_sgs( k,sgspnt(j) ) 
              tvec1( k ) = mod ( tvec1( k ) , 1.0 )
              if ( abs ( tvec1( k ) ) .lt. 0.0001 ) tvec1( k ) = 0.0
              if ( tvec1( k ) .lt. 0.0 ) tvec1( k ) = tvec1( k ) + 1.0
              do l = 1 , 3
                tmat1( l,k ) = r_sgs( l,k,sgspnt(i) ) - 
     &          r_sgs( l,k,sgspnt(j) )
              end do
            end do

*       Null matrix test again - now we don't want this matrix
*       to be the null matrix since this is what we are going to
*       use in the search.
            call vmtply ( tmat1 , tstvec , u )
            tstlen = vlen2 ( u , cell , cosang )
            if ( tstlen*patgrd(1)/cell(1) .lt. tol0 .and. 
     &           tstlen*patgrd(2)/cell(2) .lt. tol0 .and.
     &           tstlen*patgrd(3)/cell(3) .lt. tol0     ) goto 450

            call vxtrct ( tmat1,tvec1,nr,errcod )
            if ( errcod .ne. 0 ) return

            if ( nr .eq. 0 ) then
              mult( ntrans ) = 1
              vectyp( ntrans ) = 1
            else
              mult( nr ) = mult( nr ) + 1
            endif

 450      continue
 500    continue


        else if ( what .eq. 'MORE MOLECULES' ) then

*       Find position of molecule with ncs axis (parallel to 
*       crystallographic) axis using structure invariant vectors
*       to one or more fixed identical molecule(s).
*
*       Loop over symmetry operators and generate all transformations
*       where Ri(sg) * Rk(nc) = Rj(sg) * Rl(fix) (the corresponding
*       vectors are then independent of x and only depend on the 
*       position of the ncs molecule(s)):
*
*       Positions within the molecules:
*       xk = Rk(nc)* (x1-t(nc )) + t(nc ) = Rk(nc )*x1 + (I-Rk(nc ))*t(nc )
*       xl = Rl(fix)*(x2-t(fix)) + t(fix) = Rl(fix)*x2 + (I-Rl(fix))*t(fix)
*
*       Set x1 = x2 + t(sep) = x2 + t(nc ) - t(fix)
*
*       Applying sgs:
*       xik = Ri(sg)*xk + ti(sg) =
*
*             Ri(sg)*Rk(nc)*x1 + Ri(sg)*(I-Rk(nc))*t(nc) + ti(sg) =
*
*             Ri(sg)*Rk(nc)*(x2 + t(nc) - t(fix)) + 
*             Ri(sg)*(I-Rk(nc))*t(nc) + ti(sg) =
*
*             Ri(sg)*Rk(nc)*(x2 + t(fix)) + Ri(sg)*t(nc) + ti(sg)
*
*       xjl = Rj(sg)*xl + tj(sg) =
*
*             Rj(sg)*Rl(fix)*x2 + Rj(sg)*(I-Rl(fix))*t(fix) + tj(sg)
*
*
*       u = xik - xjl
*       u = (Ri(sg)*Rk(nc) - Rj(sg)*Rl(fix)) * x2 + 
*           (Ri(sg)*Rk(nc) - Rj(sg)*Rl(fix)) * t(fix) +
*           Ri(sg)*t(nc) - Rj(sg)*t(fix) + ti(sg) - tj(sg)
*
*       With Ri(sg)*Rk(nc) = Rj(sg)*Rl(fix) 
*****************************************************************
*                                                               *
*       u = Ri(sg)*t(nc) - Rj(sg)*t(fix) + ti(sg) - tj(sg)      *
*                                                               *
*****************************************************************

        do 600 i = 1 , nopsym
          do 550 j = 1 , nopsym

            do k = 1 , 3
              do l = 1 , 3
                tmat1( l,k ) = r_sym( l,k,i ) - r_sym( l,k,j )
              end do
            end do

*       Compute maximum length of structure-dependent vector generated by 
*       difference matrix. The test point in tstvec is at upper limits of 
*       the au; normally this will be the point furthest from the origin 
*       in a real search scan. 
*       Here we are only interested in structure invariant vectors so
*       we want this difference matrix to be close to the null matrix.
*       I am not sure what the optimal cutoff here would be; selecting
*       only those cases were the difference matrix is exactly the
*       null matrix would exclude all cases were the ncs is not
*       exactly parallell to a crystallographic axis.

            call vmtply ( tmat1 , tstvec , u )
            tstlen = vlen2 ( u , cell , cosang )
            if ( tstlen*patgrd(1)/cell(1) .ge. tol1 .or. 
     &           tstlen*patgrd(2)/cell(2) .ge. tol1 .or.
     &           tstlen*patgrd(3)/cell(3) .ge. tol1     ) goto 550


*       Null matrix; continue
*       Copy sgs embedded in symop i to tmat1
*       Copy sgs embedded in symop j to tmat2
            call matcop (  r_sgs( 1,1,sgspnt(i) ) , tmat1 , 3 , 3 )
            call matcop (  r_sgs( 1,1,sgspnt(j) ) , tmat2 , 3 , 3 )


*       Loop over fixed positions and generate translational part
            do 520 ipos = 1 , nxyz
              call vmtply ( tmat2 , xyz( 1,ipos ) , tvec1 )
              do k = 1 , 3
                tvec1( k ) = tvec1( k ) + t_sgs( k,sgspnt(i) ) - 
     &          t_sgs( k,sgspnt(j) )
                tvec1( k ) = mod ( tvec1( k ) , 1.0 )
                if ( abs ( tvec1( k ) ) .lt. 0.0001 ) tvec1( k ) = 0.0
                if ( tvec1( k ) .lt. 0.0 ) tvec1( k ) = tvec1( k ) + 1.0
              end do

              call vxtrct ( tmat1,tvec1,nr,errcod )
              if ( errcod .ne. 0 ) return

              if ( nr .eq. 0 ) then
                mult( ntrans ) = 1
                vectyp( ntrans ) = 2
              else
                mult( nr ) = mult( nr ) + 1
              endif
 520        continue
 550      continue
 600    continue


        else if ( what .eq. 'TRANSLATE MOLECULES' ) then

*       Find position of a pair of molecules with ncs axis (parallel to 
*       crystallographic) axis using structure invariant vectors
*       and fixed separation between molecules (given as two initial
*       positions for the molecules)
*
*       Loop over symmetry operators and generate all transformations
*       where Ri(sg) * Rk(nc) = Rj(sg) * Rl(nc) (the corresponding
*       vectors are then independent of x and only depend on the 
*       position of the ncs molecules):
*
*       Two molecules m = 1,2 in the asymmetric unit, initially centred at
*       tm(initial). Define the separation vector tmn(sep) = tm - tn.
*
*       All the positions of atoms in the molecules in the unit cell are then
*       defined by
*
*       xikm = Ri(sg)*Rkm(nc)*(xm - tm) + Ri(sg)*tm + ti(sg)
*
*       Taking all differences (i,j = 1,nsgs; k,l = 1,nncs; m,n = 1,2)
*
*       u =  Ri(sg)*Rkm(nc)*xm - Rj(sg)*Rln(nc)*xn  -
*           (Ri(sg)*Rkm(nc)*tm - Rj(sg)*Rln(nc)*tn) +
*            Ri(sg)*tm - Rj(sg)*tn +
*            ti(sg) - tj(sg)
*
*       With xn = xm - tmn(sep)
*
*       u = (Ri(sg)*Rkm(nc) - Rj(sg)*Rln(nc))*xm +
*            Rj(sg)*Rln(nc)*(tm - tn) -
*           (Ri(sg)*Rkm(nc)*tm - Rj(sg)*Rln(nc)*tn) +
*            Ri(sg)*tm - Rj(sg)*tn +
*            ti(sg) - tj(sg)
*
*       Requiring that Ri(sg)*Rkm(nc) = Rj(sg)*Rln(nc) we get
*
*       u = Ri(sg)*tm - Rj(sg)*tn + ti(sg) - tj(sg)
*
*       Now tm = tm(initial) + tx
*           tn = tn(initial) + tx = tm(initial) - tmn(sep) + tx
*
*       Thus
*
*****************************************************************
*                                                               *
*       u = (Ri(sg) - Rj(sg)) * tx +                            *
*           (Ri(sg) - Rj(sg)) * tm(initial) +                   *
*           Rj(sg) * tmn(sep) +                                 *
*           ti(sg) - tj(sg)                                     *
*                                                               *
*       With m =   n Harker vectors are generated               *
*       With m =/= n cross vectors are generated                *
*****************************************************************

        do 700 i = 1 , nopsym - 1
          do 650 j = i + 1 , nopsym

            do k = 1 , 3
              do l = 1 , 3
                tmat1( l,k ) = r_sym( l,k,i ) - r_sym( l,k,j )
              end do
            end do

*       Compute maximum length of structure-dependent vector generated by 
*       difference matrix. The test point in tstvec is at upper limits of 
*       the au; normally this will be the point furthest from the origin 
*       in a real search scan. 
*       Here we are only interested in structure invariant vectors so
*       we want this difference matrix to be close to the null matrix.
*       I am not sure what the optimal cutoff here would be; selecting
*       only those cases were the difference matrix is exactly the
*       null matrix would exclude all cases were the ncs is not
*       exactly parallell to a crystallographic axis.

            call vmtply ( tmat1 , tstvec , u )
            tstlen = vlen2 ( u , cell , cosang )
            if ( tstlen*patgrd(1)/cell(1) .ge. tol1 .or. 
     &           tstlen*patgrd(2)/cell(2) .ge. tol1 .or.
     &           tstlen*patgrd(3)/cell(3) .ge. tol1     ) goto 650

*       Null matrix; continue
*       Construct vector transformations

*       Get Ri(sg) - Rj(sg)
            do k = 1 , 3
              do l = 1 , 3
                tmat1( l,k ) = r_sgs( l,k,sgspnt(i) ) - 
     &                         r_sgs( l,k,sgspnt(j) ) 
              end do
            end do

*       Null matrix test again - now we don't want this matrix
*       to be the null matrix since this is what we are going to
*       use in the search.
            call vmtply ( tmat1 , tstvec , u )
            tstlen = vlen2 ( u , cell , cosang )
            if ( tstlen*patgrd(1)/cell(1) .lt. tol0 .and. 
     &           tstlen*patgrd(2)/cell(2) .lt. tol0 .and.
     &           tstlen*patgrd(3)/cell(3) .lt. tol0     ) goto 650

*       Generate translation part for each combination of molecules 
            do 630 k = 1 , 2
*       Get (Ri(sg) - Rj(sg) ) * tm(initial)
              call vmtply ( tmat1 , xyz( 1,k ) , tvec1 )
              do 620 l = 1 , 2
*       Get Rj(sg)*tmn(sep)
                call vecdif ( xyz(1,k) , xyz(1,l) , tvec2 , 3 )
                call vmtply ( r_sgs( 1,1,sgspnt(j) ) , tvec2 , tvec3 )
                do m = 1 , 3
                  tvec1( m ) = tvec1( m ) + tvec3( m ) + 
     &                         t_sgs( m,sgspnt(i) ) -
     &                         t_sgs( m,sgspnt(j) )
                  tvec1( m ) = mod ( tvec1( m ) , 1.0 )
                  if ( abs ( tvec1( m ) ) .lt. 0.0001 ) tvec1( m ) = 0.0
                  if ( tvec1( m ) .lt. 0.0 ) tvec1( m ) = 
     &            tvec1( m ) + 1.0
                end do


                call vxtrct ( tmat1,tvec1,nr,errcod )
                if ( errcod .ne. 0 ) return

                if ( nr .eq. 0 ) then
                  mult( ntrans ) = 1
                  if ( k .ne. l ) then
                    vectyp( ntrans ) = 2
                  else
                    vectyp( ntrans ) = 1
                  endif
                else
                  mult( nr ) = mult( nr ) + 1
                  if ( k .eq. l ) vectyp( nr ) = 1
                endif

 620          continue
 630        continue
 650      continue
 700    continue

        else
          call errmsg ( 'S/R TRNSF unknown vectorset requested > ' //
     &    what , 1 )
        endif

*       Normalize multiplicities
        minmul = 999999999
        do 900 i = 1 , ntrans
          minmul = min ( minmul , mult( i ) )
 900    continue
        do 910 i = 1 , ntrans
          mult( i ) = mult( i ) / minmul
 910    continue

        return
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Error messages
 9000   format(/10x,'** ERROR - Too many symmetry operations',
     &  ' generated in s/r TRNSF; max = ',i5,' found = ',i5,' **' )

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*       Subroutine vxtrct
*
*       Determine if transformation in (r_new,t_new) is a new one
*       by comparing to previously found. Store new transformations
*       in (r_vec,t_vec) in common /symm/.
*
*
*       Arguments :
*       R_new   (i)     rotation matrix of a possible new transformation
*       T_new   (i)     translation vector of a possible new
*                       transformation
*       Nsame   (o)     nsame = 0      new transformation
*                       nsame = n      same as transformation n
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - too many transformations
*
*
*
*       
*       October 31 1991 - sdk
*       February 11 1992 - sdk
*       Determination of polar (indeterminate) axes moved to s/r
*       polaxs
*       INDET now stored in common /symm/ instead of being passed as
*       argument
*       February 15 1989 - sdk
*       Symmetry and grids in common
*       Modularized 18 october 1988 - sdk
************************************************************************
        subroutine vxtrct ( r_new , t_new , nsame , errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'

*       Arguments
        integer         nsame
        integer         errcod
        real            r_new(3,3),t_new(3)


*       Local variables
        integer         i , j , k , m , ic
        real            sext(6)
        real            dmax
        real            tol1 , tol2
        real            tmat1(3,3),tvec1(3),tvec2(3)
        real            r_diff(3,3),u(3),umax(3),umin(3)


*       Data statements
*       Tolerances used in choosing unique transformations
*       (Patterson grid units)
        data            tol1,tol2/3.5,4.5/
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Init
        errcod = 0
        nsame = 0

*       Store large cell for testing closeness of transformations
        do i = 1 , 3
          sext( 2*i-1 ) = -1.0
          sext( 2*i   ) =  1.0
        end do

        do 80 i = 1 , ntrans
*       Loop on previously determined transformations

          do 70 j = 1 , 2 * nopsgs
*       Loop on patterson symmetry

*       Apply patterson symmetry to the (possibly) new transformation
            call mmtply ( r_lgs( 1,1,j ) , r_new , tmat1 )
            call vmtply ( r_lgs( 1,1,j ) , t_new , tvec1 )

*       Apply possible centering
            do 60 ic = 1 , icen
              do k = 1 , 3
                tvec2( k ) = tvec1( k ) + cent( k,ic )
                tvec2( k ) = mod ( tvec2( k ) , 1.0 )
                if ( abs ( tvec2( k ) ) .lt. 0.0001 ) tvec2( k ) = 0.0
                if ( tvec2( k ) .lt. 0 ) tvec2( k ) = tvec2( k ) + 1.0
              end do

*       Compare with previously determined transformations
              do k = 1 , 3
                if ( abs ( ( tvec2( k ) - t_vec( k,i ) ) * patgrd(k) ) 
     &          .ge. tol1 ) goto 60
                do m = 1 , 3
                  r_diff( m,k ) = abs ( tmat1( m,k ) - r_vec( m,k,i ) )
                end do
              end do

*       Check the maximum separation in all three directions
*       between a peak generated by this possible new transformation
*       and all previously found.
*@@@@@  I don't like this test. Should be changed to something else.
*@@@@@  As a matter of fact it is wrong. 940216 SDK
*@@@@@
*
              call maxu ( r_diff,sext,umax,umin,u )
              do k=1,3
                dmax=max(abs(umax(k)+tvec2(k)-t_vec(k,i)),
     &                   abs(umin(k)+tvec2(k)-t_vec(k,i)))
                if ( dmax*patgrd(k).ge.tol2 ) goto 60
              end do


*       If we come here the maximum separation in all directions
*       is less than tol2 - skip this transformation
              nsame = i
              return
 60         continue
 70       continue
 80     continue

*       New transformation  - store it
        ntrans = ntrans + 1
        if ( ntrans .gt. mxvec ) then
          write(6,9000)mxvec,ntrans
          errcod = 1
          call errmsg ( 'Too many transformations generated' , 0 ) 
          return
        endif
        call matcop ( t_new , t_vec( 1,ntrans ) , 3 , 1 )
        call matcop ( r_new , r_vec( 1,1,ntrans ) , 3 , 3 )

        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Error messages
 9000   format(/10x,'** ERROR - Too many transformations found',
     &  ' in s/r VXTRCT; max = ',i5,' found = ',i5,' **' )

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*       Subroutine symapp(nsym,r,t,nxin,xin,nxout,xout,errcod)
*
*       Function:
*       Apply symmetry in r,t to xin, store unique positions in xout
*
*       Arguments :
*       Nsym    (i)     number of symmetry operations
*       R,t     (i)     sym ops, (fractional translations)
*       Nxin    (i)     number of input positions
*       Xin     (i)     input positions (fractional)
*       Nxout   (i/o)   on input : maximum number of generated positions
*                       on output: number of positions generated
*       Xout    (o)     generated positions (fractional)
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*
*@@@@@@@@@@@@@@@@@@@@@@@@@@@
*       Change tol to bump ?
*@@@@@@@@@@@@@@@@@@@@@@@@@@@
*
*       History
*       November 8 1991 - sdk
*       In,out arguments removed. Both input and output coordinates now
*       fractional only.
*       November 1988 - sdk
*       Changed name from geneqp to symapp
*       Modified to output unique list of symmetry related
*       positions only
*       October 1988 - sdk
*       Modified to allow coordinates to be input/output in any of
*       grid 1, grid 2 or normed crystallographic (angstrom).
*       March 28 1988 - sdk
************************************************************************
        subroutine symapp(nsym,r,t,nxin,xin,nxout,xout,errcod)

        implicit none

        include         'rspsgrd.inc'


*       Arguments
        integer         nsym , nxin , nxout
        real            r(3,3,nsym) , t(3,nsym)
        real            xin(3,nxin) , xout(3,nxout)
        integer         errcod

*       Local variables
        integer         i , j , k , m
        integer         nx
        real            tol
        real            xrot(3) , u(3)

*       External functions
        real            vlen

*       Data statements
        data            tol/3.5/
*-----------------------------------------------------------------------
        if ( nsym*nxin .gt. nxout ) then
          write(6,9000)nxout,nsym*nxin
          errcod = 1
          call errmsg ( 'Insufficient space allocated in s/r SYMAPP' , 
     &    0 )
          return
        endif

        nx = 0
*       Loop on input positions
        do 200 i = 1 , nxin

*       Loop on symmetry operations
        do 100 j = 1 , nsym

          call vmtply ( r( 1,1,j ) , xin(1,i) , xrot )
          do k = 1 , 3
            xrot( k ) = xrot( k ) + t( k,j )
            xrot( k ) = mod ( xrot( k ) , 1.0 )
            if ( abs ( xrot( k ) ) .lt. 0.0001 ) xrot( k ) = 0.0
            if ( xrot( k ) .lt. 0 ) xrot( k ) = xrot( k ) + 1.0
          end do

*       Compare with previously found positions
          do k = 1 , nx
            do m = 1 , 3
              u( m ) = xrot( m ) - xout( m,k ) 
            end do
            if ( vlen ( u , cell , cosang ) .lt. tol ) goto 100
          end do

*       If we come here this is a new position - store it
          nx = nx + 1
          do m = 1 , 3
          xout( m,nx ) = xrot( m )
          end do

 100    continue
 200    continue
        nxout = nx
        return
*-----------------------------------------------------------------------
 9000   format(/10x,'** ERROR - Too many positions generated in s/r',
     &  ' SYMAPP, max = ',i5,' requested = ',i5,' **' )
*-----------------------------------------------------------------------
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine CMBSYM
*
*       Function:
*       Combine space group symmetry and non crystallographic symmetry
*
*
*       Comment:
*       The input operators are expected to operate on and result in
*       coordinates in fractions of the unit cell edges. The resulting
*       combined symmetry will the also operate on and result in
*       coordinates in the fractional crystallographic frame.
*
*       Arguments:
*       R1,T1     (i)     SGS operators (or identity)
*       N1        (i)     Number og SGS operators
*       R2,T2     (i)     NCS operators (or identity)
*       N2        (i)     Number of NCS operators
*
*       Results are stored in common /symm/ R_SYM,T_SYM,NOPSYM
*
*       December 3 1993 - sdk
*       Added pointers from symop number to sgs and ncs operators
*       November 5 1991 - sdk
*       Modified for fractional translations in t1 and t2.
*       Ouput (in common /symm/)
*       Written October 31 1991 S. Knight
************************************************************************
        subroutine cmbsym ( r1 , t1 , n1 , r2 , t2 , n2 )


        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'

*       Arguments
        integer         n1 , n2
        real            r1( 3 , 3 , n1 ) , r2( 3 , 3 , n2 )
        real            t1( 3 , n1 ) , t2( 3 , n2 )

*       Local variables
        integer         i , j , k , ict
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        ict = 0

*       Outer loop over sgs
        do 20 i = 1 , n1

*       Inner loop over ncs
          do 10 j = 1 , n2

            ict=ict+1
            sgspnt( ict ) = i
            ncspnt( ict ) = j

            call vmtply ( r1( 1,1,i ) , t2( 1,j ) , t_sym( 1,ict ) )
            call mmtply ( r1( 1,1,i ) , r2( 1,1,j ) , r_sym( 1,1,ict ) )

            do k = 1 , 3
            t_sym( k,ict ) = t_sym( k,ict ) + t1( k,i ) 
            t_sym( k,ict ) = mod ( t_sym( k,ict ) , 1.0 )
            if ( abs ( t_sym( k,ict ) ) .lt. 0.0001 ) 
     &      t_sym( k,ict ) = 0.0
            if ( t_sym( k,ict ) .lt. 0. ) t_sym( k,ict ) =
     &      t_sym( k,ict ) + 1.0
            end do

 10       continue
 20     continue

        return
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*       Subroutine cvcalc ( x1,x2,nx1,nx2,vec,nvec )
*
*       Function:
*       Compute all vectors between positions in x1 and x2
*       Return unique vectors and relative multiplicities
*
*
*       Arguments :
*       X1,x2   (i)     positions to be used for calculating 
*                       crossvectors (fractional)
*       Nx1,nx2 (i)     number of positions in x1 and x2
*       Vec     (o)     coordinates of unique vectors (fractional),
*                       relative multiplicities
*       Nvec    (i/o)   on input : maximum number of vectors
*                       on output: number of unique vectors generated
*
*       History:
*       December 9 1993 - sdk
*       Normalization of multiplicities
*       November 22 1991 - sdk
*       Modified to use external function LGSEQL to compare vectors
*       Written November 1988 - sdk
************************************************************************
        subroutine cvcalc ( x1 , x2 , nx1 , nx2 , vec , nvec )

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'

*       Arguments
        integer         nx1 , nx2 , nvec
        real            x1(3,nx1),x2(3,nx2)
        real            vec(4,nvec)

*       Local variables
        integer         i , j , k
        integer         nv
        integer         minmul
        real            tol
        real            u(3)

*       Data statements
*       Minimum grid separation of vectors in any direction
        data            tol/1.5/

*       External functions
        integer         lgseql
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        nv = 0
        do 200 i = 1 , nx1
        do 100 j = 1 , nx2

*       Compute vector within unit cell limits
        do k = 1 , 3
        u( k ) = x1( k,i ) - x2( k,j )
        u( k ) = mod ( u( k ) , 1.0 )
        if ( abs ( u( k ) ) .lt. 0.0001 ) u( k ) = 0.0
        if ( u( k ) .lt. 0. ) u( k ) = u( k ) + 1.0
        end do


*       Compare with previously found vectors
        do 80 k = 1 , nv
          if ( lgseql ( u , vec( 1,k ) , tol ) .eq. 1 ) then
*       Old vector - skip it
            vec( 4,k ) = vec( 4,k ) + 1
            goto 100
          endif
 80     continue


*       New vector - store it
        nv = nv + 1
        if ( nv .gt. nvec ) then
        write(6,9000)nvec,nv
        stop' ERROR - TOO MANY VECTORS GENERATED'
        endif
        call matcop ( u , vec( 1,nv ) , 3 , 1 )
        vec( 4,nv ) = 1

 100    continue
 200    continue

*       Normalize multiplicities
        minmul = 999999999
        do 210 i = 1 , nv
          minmul = min ( minmul , nint ( vec( 4,i ) ) )
 210    continue
        do 220 i = 1 , nv
          vec( 4,i ) = vec( 4,i ) / minmul
 220    continue

        nvec = nv
        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
*       Error messgaes
 9000   format(/10x,'** ERROR - Too many vectors generated',
     &  ' in s/r CVCALC; max = ',i5,' found = ',i5,' **' )
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*       Subroutine hvcalc ( x1,nx1,vec,nvec )
*
*       Function:
*       Compute all vectors between positions in x1
*       Return unique vectors and relative multiplicities
*
*
*       Arguments :
*       X1      (i)     positions to be used for calculating harker 
*                       vectors 
*       Nx1     (i)     number of positions in x1
*       Vec     (o)     unique vectors (fractional, mod 1.0) and 
*                       relative multiplicities
*       Nvec    (i/o)   on input : maximum number of vectors
*                       on output: number of unique vectors generated
*
*       History:
*       December 9 1993 - sdk
*       Normalization of multiplicities
*       November 22 1991 - sdk
*       Modified to use external function LGSEQL to compare vectors
*       Written November 1988 S. Knight
************************************************************************
        subroutine hvcalc ( x1 , nx1 , vec , nvec )

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'

*       Arguments
        integer         nx1 , nvec
        real            x1(3,nx1)
        real            vec(4,nvec)

*       Local variables
        integer         i , j , k
        integer         nv
        integer         minmul
        real            u(3)
        real            tol

*       Data statements
*       Minimum grid separation of vectors in any direction
        data            tol/1.5/

*       External functions
        integer         lgseql
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        nv = 0
        do 200 i = 1 , nx1 - 1
        do 100 j = i + 1 , nx1

*       Compute vector within unit cell limits
        do k = 1 , 3
        u( k ) = x1( k,i ) - x1( k,j ) 
        u( k ) = mod ( u( k ) , 1.0 )
        if ( abs ( u( k ) ) .lt. 0.0001 ) u( k ) = 0.0
        if ( u( k ) .lt. 0.0 ) u( k ) = u( k ) + 1.0
        end do

*       Compare with previously found vectors
        do 80 k = 1 , nv
          if ( lgseql ( u , vec( 1,k ) , tol ) .eq. 1 ) then
*       Old vector - increase multiplicity count
            vec( 4,k ) = vec( 4,k ) + 1
            goto 100
          endif
 80     continue

*       New vector - store it
        nv = nv + 1
        if ( nv .gt. nvec ) then
        write(6,9000)nvec,nv
        stop' ERROR - TOO MANY VECTORS GENERATED'
        endif
        call matcop ( u , vec( 1,nv ) , 3 , 1 )
        vec( 4,nv ) = 1
 100    continue
 200    continue

*       Normalize multiplicities
        minmul = 999999999
        do 210 i = 1 , nv
          minmul = min ( minmul , nint ( vec( 4,i ) ) )
 210    continue
        do 220 i = 1 , nv
          vec( 4,i ) = vec( 4,i ) / minmul
 220    continue

        nvec = nv
        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
*       Error messages
 9000   format(/10x,'** ERROR - Too many vectors generated',
     &  ' in s/r HVCALC; max = ',i5,' found = ',i5,' **' )
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*       Subroutine maxu ( r , limit , umax , umin , u )
*
*
*       Determine maximum length of vector u given
*       the transformation matrix r and search
*       cell limits limit
*
*       Argumnets :
*
*       R       (i)     transformation matrix
*
*       Limit   (i)     limits for search cell (fractional)
*
*       Umax    (o)     vector with maximal components generated
*
*       Umin    (o)     vector with minimal components generated
*
*       U       (o)     longest vector
*
*       October 1988 sdk
************************************************************************
        subroutine maxu ( r , limit , umax , umin , u )

        implicit none

*       Arguments
        real            limit(6)        
        real            r(3,3),umax(3),umin(3),u(3)

*       Local variables
        integer         i , j


        do i = 1 , 3
          u( i ) = 0.
          umax( i ) = 0.
          umin( i ) = 0.
          do j = 1 , 3
            umax(i)=umax(i)+max(r(i,j)*limit(2*j-1),r(i,j)*limit(2*j))
            umin(i)=umin(i)+min(r(i,j)*limit(2*j-1),r(i,j)*limit(2*j))
          end do
          u( i ) = max ( abs ( umax( i ) ) , abs ( umin( i ) ) )
        end do
        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine POLAXS
*
*       Function:
*       Find polar directions
*       
*       Arguments:
*       Nsym     (i)       number of symmetry operations
*       R        (i)       rotation matrices
*       Npolx    (o)       number of polar axes
*       Polx     (o)       polar axis flag along x,y,z
*                          polx = 0 - polar axis
*                          polx = 1 - non polar axis
*       
*       History:
*       Written February 1992 S. Knight
************************************************************************
        subroutine polaxs ( nsym , r , npolx , polx )

        implicit none


*       Arguments
        integer         nsym , npolx , polx(3)
        real            r(3,3,nsym) 

*       Local variables
        integer         i , j , k , l
        real            dr(3,3) , lbox(6)
        real            u(3) , umax(3) , umin(3)
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Store large cell
        do i = 1 , 3
          lbox( 2*i-1 ) = -1.0
          lbox( 2*i   ) =  1.0
        end do

*       Initiate counter
        npolx = 0

*       Initiate flag
        do i = 1 , 3
          polx( i ) = 0
        end do

*       Loop over symmetry operators 
        do 100 i = 1 , nsym - 1
          do 50  j = i + 1 , nsym

*       Generate difference matrix
            do k = 1 , 3
              do l = 1 , 3
                dr( l,k ) = r( l,k,i ) - r( l,k,j )
              end do
            end do

*       Check for polar axis
            call maxu ( dr,lbox,umax,umin,u )
            do k = 1 , 3
              if ( u( k ) .ge. 0.0001 ) polx( k ) = 1
            end do

 50       continue
 100    continue


*       Count polar axes
*       No polar axis  ==> npolx = 0
*       One polar axis ==> npolx = 1
*       Two polar axes ==> npolx = 2 (no such case)
*       P1             ==> npolx = 3
        do i = 1 , 3
          if ( polx(i) .eq. 0 ) npolx = npolx + 1
        end do


        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine INDAXS
*
*       Function:
*       Find indeterminate directions
*       
*       Arguments:
*       Nvec     (i)       number of vectors
*       R        (i)       vector transformations
*       Indet    (o)       polar axis flag along x,y,z
*                          indet = 0 - indeterminate axis
*                          indet = 1 - non indeterminate axis
*       
*       History:
*       From POLAXS december 6 1993 - sdk
*       Written February 1992 S. Knight
************************************************************************
        subroutine indaxs ( nvec , r , indet )

        implicit none


*       Arguments
        integer         nvec , indet(3)
        real            r(3,3,nvec) 

*       Local variables
        integer         i , j 
        real            lbox(6)
        real            u(3) , umax(3) , umin(3)
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Store large cell
        do 10 i = 1 , 3
          lbox( 2*i-1 ) = -1.0
          lbox( 2*i   ) =  1.0
 10     continue

*       Initiate flag
        do 20 i = 1 , 3
          indet( i ) = 0
 20     continue

*       Loop over vector operators 
        do 100 i = 1 , nvec

*       Check for indeterminate axis
            call maxu ( r(1,1,i),lbox,umax,umin,u )
            do 50 j = 1 , 3
              if ( u( j ) .ge. 0.0001 ) indet( j ) = 1
 50         continue

 100    continue


        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
      subroutine stlim(lspgrp,xyzlim)
c     ================================
c
c Phil Evans 1991
c
c Set appropriate box (asymmetric unit) for space group (true space group)
c     LSPGRP. For high symmetry space groups, this will be more than
c     one asymmetric unit
c
c On entry:
c     lspgrp    true space group (not FFT space group)
c
c On exit
c     xyzlim(2,3)  minimum, maximum limits on x,y,z (fractions of cell)
c                  if space group not recognized, returns xzylim(1,1) = -1.0
c                  Note that the minimum limits (xyzlim(1,)) will always
c                   = 0.0
c
c Space group 164 (P-3m1) added august 12 1992 S. Knight
c 
c
      implicit none
c
      integer lspgrp
      real xyzlim(2,3)
c
      integer i,j
c
      integer numsgp
      parameter (numsgp=89)
      real one,half,thrd,twtd,sixt,quar,eigh,twlt,round,round2
      real onel,halfl,thrdl,sixtl,quarl,eighl
      parameter (round=0.00001, round2=2.0*round)
      parameter (one=1.0+round,half=0.5+round,thrd=1./3.+round,
     $     twtd=2./3.+round,sixt=1./6.+round,
     $     quar=0.25+round,eigh=0.125+round,twlt=1./12.+round)
      parameter (onel=one-round2,halfl=half-round2,thrdl=thrd-round2,
     $     sixtl=sixt-round2,quarl=quar-round2,eighl=eigh-round2)
c
      integer nspgrp(numsgp)
      real asulim(3,numsgp)
c
c  asulim contains maximum limit on x,y,z: the box is always assumed to
c     start at 0,0,0
c
c  Space group numbers
      data nspgrp/
     $   1,   2,   3,    4,   5,  10,  16,   17,  18,1018,  19,   20,
     $  21,  22,  23,   24,  47,  65,  69,   71,  75,  76,  77,   78,
     $  79,  80,  83,   87,  89,  90,  91,   92,  93,  94,  95,   96,
     $  97,  98, 123,  139, 143, 144, 145,  146, 147, 148, 149,  150,
     $ 151, 152, 153,  154, 155, 162, 164,  166, 168, 169, 170,  171,  
     $ 172, 173, 175,  177, 178, 179, 180,  181, 182, 191, 195,  196,  
     $ 197, 198, 199,  200, 202, 204, 207,  208, 209, 210, 211,  212,  
     $ 213, 214, 221,  225, 229/
c
      data asulim/
c        1:  P1          2:  P-1         3:  P2            4:  P21
     $ onel,onel,onel, onel,half,onel, half,onel,onel, onel,halfl,onel,
c        5:  C2         10:  P2/m       16:  P222         17:  P2221
     $ half,halfl,onel,half,half,onel, half,half,onel, half,half,onel,
c       18: P21212    1018: P21212      19: P212121       20:C2221
     $ onel,quar,onel, onel,quar,onel, onel,onel,quar, half,quar,onel,
c       21:  C222       22:  F222       23:  I222         24: I212121
     $ half,quar,onel, quar,quar,onel, half,quar,one,  half,quar,onel,
c       47:  Pmmm       65:  Cmmm       69:  Fmmm         71:  Immm
     $ half,half,half, half,quar,half, quar,quar,half, half,quar,half,
c       75:  P4         76:  P41        77:  P42          78:  P43
     $ half,half,onel, onel,onel,quarl,half,onel,halfl,onel,onel,quarl,
c       79:  I4         80:  I41        83:  P4/m         87:  I4/m
     $ half,half,half, half,onel,quarl,half,half,half, half,half,quar,
c       89: P422        90: P4212       91: P4122         92: P41212
     $ half,half,half, half,half,half, onel,onel,eigh, onel,onel,eigh,
c       93: P4222       94: P42212      95: P4322         96: P43212
     $ half,onel,quar, half,half,half, onel,onel,eigh, onel,onel,eigh,
c       97: I422        98: I4122      123: P4/mmm       139: I4/mmm
     $ half,half,quar, half,onel,eigh, half,half,half, half,half,quar,
c      143:  P3        144:  P31       145: P32          146:  R3
     $ twtd,twtd,onel, onel,onel,thrdl,onel,onel,thrdl,twtd,twtd,thrdl,
c      147:  P-3       148:  R-3       149: P312         150:  P321
     $ twtd,twtd,half, twtd,twtd,sixt, twtd,twtd,half, twtd,twtd,half,
c      151: P3112      152: P3121      153: P3212        154: P3221
     $ onel, onel,sixt,onel,onel,sixt, onel,onel,sixt, onel,onel,sixt,
c      155: R32        162:  P-31m     164: P-3m1
     $ twtd,twtd,sixt, twtd,half,half, twtd,thrd, one,
c      166:  R-3m      168:  P6
     $ twtd,twtd,sixt, twtd,half,onel,
c      169:  P61       170:  P65       171:  P62         172:  P64
     $ onel,onel,sixtl,onel,onel,sixtl,onel,onel,thrdl,onel,onel,thrdl,
c      173:  P63       175:  P6/m      177: P622         178: P6122
     $ twtd,twtd,halfl,twtd,twtd,half, twtd,half,half, onel,onel,twlt,
c      179: P6522      180: P6222      181: P6422        182: P6322
     $ onel,onel,twlt, onel,onel,sixt, onel,onel,sixt, twtd,twtd,quar,
c      191: P6/mmm     195: P23        196: F23          197: I23
     $ twtd,thrd,half, onel,onel,half, quar,quar,onel, onel,onel,half,
C      198: P213       199: I213       200: Pm-3         202: Fm-3
     $ half,half,onel, half,half,half, half,half,half, half,half,quar,
c      204: Im-3       207: P432       208: P4232        209: F432
     $ half,half,half, onel,half,half, half,onel,quar, half,half,half,
c      210: F4132      211: I432       212: P4332        213: P4132
     $ half,onel,eigh, half,half,quar, onel,onel,eigh, onel,onel,eigh,
c      214: I4132      221: Pm-3m      225: Fm-3m        229: Im-3m
     $ half,onel,eigh, half,half,half, half,quar,quar, half,half,quar/
c
      do 10, j=1,numsgp
         if (lspgrp .eq. nspgrp(j)) go to 20
 10   continue
c
c Space group not found
      xyzlim(1,1) = -1.0
      return
c
c
 20   do 30, i=1,3
         xyzlim(1,i) = 0.0
         xyzlim(2,i) = asulim(i,j)
 30   continue
c
      return
c
      end
************************************************************************
************************************************************************
*
*       Subroutine CHKLIM
*
*       Function:
*       Write Laue group name
*       Check limits of Patterson map
*       Set noreduce flag if map limits = unit cell
*       
*       Arguments:
*       Spgr      (i)    space group number
*       Errcod    (o)    error code
*                        errcod = 0 - no error
*                        errcod = 1 - illegal map limits
*                        errcod = 2 - illegal space group
*       
*       
*       History:
*       Written November 24 1991 S. Knight
************************************************************************
        subroutine chklim ( spgr , errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc' 
        include         'rspsgrd.inc' 


*       Arguments
        integer         spgr
        integer         errcod

*       Local variables
        integer         na , nb , nc
        integer         istop
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Initiate noreduce flag
        noreduce = .false.

*       Set limits, report Laue group
************************************************************************
        if ( spgr .eq. 1 ) then

*       Laue group   : P-1 
*       Space groups : any 
*       Map volume   : 0 1 0 1/2 0 1
************************************************************************
          na  = patgrd(1) - 1
          nb  = ( patgrd(2) + 1 ) / 2
          nc  = patgrd(3) - 1
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > P-1'

************************************************************************
        else if ( spgr .eq. 3 .or. spgr .eq. 4 ) then

*       Laue group   : P2/m 2nd setting (sg 10)
*       Space groups : 3 & 4
*       Map volume   : 0 1/2 0 1/2 0 1
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 1 ) / 2
          nc  = patgrd(3) - 1
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > P2/m 2nd setting'

************************************************************************
        else if ( spgr .eq. 5 ) then

*       Laue group   : C2/m (sg 12)
*       Space group  : 5
*       Map volume   : 0 1/2 0 1/4 0 1
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 3 ) / 4
          nc  = patgrd(3) - 1
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > C2/m 2nd setting'

************************************************************************
        else if ( spgr .ge. 16 .and. spgr .le. 19 ) then

*       Laue group   : Pmmm (sg 47)
*       Space groups : 16-19
*       Map volume   : 0 1/2 0 1/2 0 1/2
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 1 ) / 2
          nc  = ( patgrd(3) + 1 ) / 2
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > Pmmm'

************************************************************************
        else if ( spgr .ge. 20 .and. spgr .le. 21 ) then

*       Laue group   : Cmmm (sg 65)
*       Space groups : 20-21
*       Map volume   : 0 1/2 0 1/4 0 1/2
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 3 ) / 4
          nc  = ( patgrd(3) + 1 ) / 2
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > Cmmm'

************************************************************************
        else if ( spgr .eq. 22 ) then

*       Laue group   : Fmmm (sg 69)
*       Space group  : 22
*       Map volume   : 0 1/4 0 1/4 0 1/2 
************************************************************************
          na  = ( patgrd(1) + 3 ) / 4
          nb  = ( patgrd(2) + 3 ) / 4
          nc  = ( patgrd(3) + 1 ) / 2
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > Fmmm'

************************************************************************
        else if ( spgr .ge. 23 .and. spgr .le. 24 ) then

*       Laue group   : Immm (sg 71)
*       Space groups : 23-24
*       Map volume   : 0 1/2 0 1/4 0 1/2 
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 3 ) / 4
          nc  = ( patgrd(3) + 1 ) / 2
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > Immm'

************************************************************************
        else if ( spgr .ge. 75 .and. spgr .le. 78 ) then

*       Laue group   : P4/m (sg 83)
*       Space groups : 75-78
*       Map volume   : 0 1/2 0 1/2 0 1/2 
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 1 ) / 2
          nc  = ( patgrd(3) + 1 ) / 2
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > P4/m'

************************************************************************
        else if ( spgr .ge. 79 .and. spgr .le. 80 ) then

*       Laue group   : I4/m (sg 87)
*       Space groups : 79-80
*       Map volume   : 0 1/2 0 1/2 0 1/4
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 1 ) / 2
          nc  = ( patgrd(3) + 3 ) / 4
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > I4/m'

************************************************************************
        else if ( spgr .ge. 89 .and. spgr .le. 96 ) then

*       Laue group   : P4/mmm (sg 123)
*       Space groups : 89-96
*       Map volume   : 0 1/2 0 1/2 0 1/2
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 1 ) / 2
          nc  = ( patgrd(3) + 1 ) / 2
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > P4/mmm'

************************************************************************
        else if ( spgr .ge. 97 .and. spgr .le. 98 ) then

*       Laue group   : I4/mmm (sg 139)
*       Space groups : 97-98
*       Map volume   : 0 1/2 0 1/2 0 1/4 
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 1 ) / 2
          nc  = ( patgrd(3) + 3 ) / 4
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > I4/mmm'

************************************************************************
        else if ( spgr .ge. 143 .and. spgr .le. 145 ) then

*       Laue group   : P-3 (sg 147)
*       Space groups : 143-145
*       Map volume   : 0 2/3 0 2/3 0 1/2
************************************************************************
          na  = ( 2 * patgrd(1) + 2 ) / 3
          nb  = ( 2 * patgrd(2) + 2 ) / 3
          nc  = ( patgrd(3) + 1 ) / 2
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > P-3'

************************************************************************
        else if ( spgr .eq. 146 ) then

*       Laue group   : R-3 (sg 148) (Hexagonal axes)
*       Space groups : 146 (Hexagonal axes)
*       Map volume   : 0 2/3 0 2/3 0 1/6
************************************************************************
          na  = ( 2 * patgrd(1) + 2 ) / 3
          nb  = ( 2 * patgrd(2) + 2 ) / 3
          nc  = ( patgrd(3) + 5 ) / 6
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > R-3 Hexagonal axes'

************************************************************************
        else if ( spgr .eq. 149 .or. spgr .eq. 151 .or. spgr .eq. 153 ) 
     &  then

*       Laue group   : P-31m (sg 162)
*       Space groups : 149,151,153
*       Map volume   : 0 2/3 0 1/2 0 1/2
************************************************************************
          na  = ( 2 * patgrd(1) + 2 ) / 3
          nb  = ( patgrd(2) + 1 ) / 2
          nc  = ( patgrd(3) + 1 ) / 2
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > P-31m'

************************************************************************
        else if ( spgr .eq. 150 .or. spgr .eq. 152 .or. spgr .eq. 154 )
     &  then

*       Laue group   : P-3m1 (sg 164)
*       Space groups : 150,152,154
*       Map volume   : 0 2/3 0 1/3 0 1
************************************************************************
          na  = ( 2 * patgrd(1) + 2 ) / 3
          nb  = ( patgrd(2) + 2 ) / 3
          nc  = patgrd(3) - 1
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > P-3m1'

************************************************************************
        else if ( spgr .eq. 155 ) then

*       Laue group   : R-3m (sg 166) (Hexagonal axes)
*       Space groups : 155 (Hexagonal axes)
*       Map volume   : 0 2/3 0 2/3 0 1/6
************************************************************************
          na  = ( 2 * patgrd(1) + 2 ) / 3
          nb  = ( 2 * patgrd(2) + 2 ) / 3
          nc  = ( patgrd(3) + 5 ) / 6
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > R-3m Hexagonal axes'

************************************************************************
        else if ( spgr .ge. 168 .and. spgr .le. 173 ) then

*       Laue group   : P6/m (sg 175)
*       Space groups : 168-173
*       Map volume   : 0 2/3 0 2/3 0 1/2
************************************************************************
          na  = ( 2 * patgrd(1) + 2 ) / 3
          nb  = ( 2 * patgrd(2) + 2 ) / 3
          nc  = ( patgrd(3) + 1 ) / 2
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > P6/m'

************************************************************************
        else if ( spgr .ge. 177 .and. spgr .le. 182 ) then

*       Laue group   : P6/mmm (sg 191)
*       Space groups : 177-182
*       Map volume   : 0 2/3 0 1/3 0 1/2
************************************************************************
          na  = ( 2 * patgrd(1) + 2 ) / 3
          nb  = ( patgrd(2) + 2 ) / 3
          nc  = ( patgrd(3) + 1 ) / 2
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > P6/mmm'

************************************************************************
        else if ( spgr .eq. 195 .or. spgr .eq. 198 ) then

*       Laue group   : Pm-3 (sg 200)
*       Space groups : 195,198
*       Map volume   : 0 1/2 0 1/2 0 1/2
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 1 ) / 2
          nc  = ( patgrd(3) + 1 ) / 2
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > Pm-3'

************************************************************************
        else if ( spgr .eq. 196 ) then

*       Laue group   : Fm-3 (sg 202)
*       Space group  : 196
*       Map volume   : 0 1/2 0 1/2 0 1/4
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 1 ) / 2
          nc  = ( patgrd(3) + 3 ) / 4
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > Fm-3'
 
************************************************************************
        else if ( spgr .eq. 197 .or. spgr .eq. 199 ) then

*       Laue group   : Im-3 (sg 204)
*       Space groups : 197,199
*       Map volume   : 0 1/2 0 1/2 0 1/2
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 1 ) / 2
          nc  = ( patgrd(3) + 1 ) / 2
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > Im-3'

************************************************************************
        else if ( spgr .ge. 207 .and. spgr .le. 208 .or.
     &            spgr .ge. 212 .and. spgr .le. 213 ) then

*       Laue group   : Pm-3m (sg 221)
*       Space groups : 207-208,212-213
*       Map volume   : 0 1/2 0 1/2 0 1/2
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 1 ) / 2
          nc  = ( patgrd(3) + 1 ) / 2
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > Pm-3m'

************************************************************************
        else if ( spgr .ge. 209 .and. spgr .le. 210 ) then

*       Laue group   : Fm-3m (sg 225)
*       Space groups : 209-210
*       Map volume   : 0 1/2 0 1/4 0 1/4
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 3 ) / 4
          nc  = ( patgrd(3) + 3 ) / 4
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > Fm-3m'

************************************************************************
        else if ( spgr .eq. 211 .or. spgr .eq. 214 ) then

*       Laue group   : Im-3m (sg 229)
*       Space groups : 211,214
*       Map volume   : 0 1/2 0 1/2 0 1/4
************************************************************************
          na  = ( patgrd(1) + 1 ) / 2
          nb  = ( patgrd(2) + 1 ) / 2
          nc  = ( patgrd(3) + 3 ) / 4
          write(6,'(/10x,a)')'PATTERSON SYMMETRY > Im-3m'

************************************************************************
        else
************************************************************************
          istop = 0
          if (stop_on_error) istop = 1
          call errmsg ( 'Illegal space group' , istop )
          errcod = 1
          return

************************************************************************
        endif
************************************************************************

*       Check limits
        if ( patlim(1) .ne.  0 .or. 
     &       patlim(3) .ne.  0 .or. 
     &       patlim(5) .ne.  0 .or.
     &       patlim(2) .lt. na .or.
     &       patlim(4) .lt. nb .or.
     &       patlim(6) .lt. nc     ) then

          write(6,9000)
          write(6,9100)0,na,0,nb,0,nc,patlim
          istop = 0
          if (stop_on_error) istop = 1
          call errmsg ( 'Wrong limits for the Patterson map', istop )
          errcod = 2
          return

        else

          write(6,2020)
          if ( patlim( 2 ) .ge. patgrd( 1 ) - 1 .and.
     &         patlim( 4 ) .ge. patgrd( 2 ) - 1 .and.
     &         patlim( 6 ) .ge. patgrd( 3 ) - 1      ) then
            noreduce = .true.
          endif

        endif


        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
 2020   format(10x,'Limits of Patterson map tested and found valid')
 9000   format(/10x,'** ERROR - wrong limits on Patterson map **')
 9100   format(10x,'Required map limits with this grid: ' , 6i5,
     &  /10x,      'Limits found:                       ' , 6i5)    
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*       Subroutine SETCEL
*
*       Function:
*       Set cell to (arbitrary) default value
*
*       Arguments: 
*       isg       (i)        space group number
*       cell      (i/o)      unit cell
*
*
*       History:
*       Written March 1996 sdk
*-----------------------------------------------------------------------
        subroutine setcel ( isg , cell )

        implicit none

        integer         isg
        real            cell( 6 )
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

        if ( nint ( cell( 1 ) ) .eq. 0 ) then
*       Cell not set
          cell( 1 ) = 100.
          cell( 2 ) = 100.
          cell( 3 ) = 100.
          cell( 4 ) = 90.
          cell( 5 ) = 90.
          cell( 6 ) = 90.
          if ( isg .ge. 143 .and. isg .le. 182 )cell( 6 ) = 120.
        endif

        if ( isg .ge. 3 .and. isg .le. 5 ) then
*       Monoclinic 2nd setting; alpha = gamma = 90 ?
          if ( nint ( cell( 4 ) ) .ne. 90 .or.
     &         nint ( cell( 6 ) ) .ne. 90 ) then
            call wrnmsg ( 'Cell does not fit space group' )
            write( 6,2000 ) cell
            cell( 1 ) = 100.
            cell( 2 ) = 100.
            cell( 3 ) = 100.
            cell( 4 ) = 90.
            cell( 5 ) = 90.
            cell( 6 ) = 90.
            write( 6,2100 ) cell
          endif
        endif

        if ( isg .ge. 16 .and. isg .le. 24 ) then
*       Orthorhomic; alpha = beta = gamma = 90 ?
          if ( nint ( cell( 4 ) ) .ne. 90 .or.
     &         nint ( cell( 5 ) ) .ne. 90 .or.
     &         nint ( cell( 6 ) ) .ne. 90 ) then
            call wrnmsg ( 'Cell does not fit space group' )
            write( 6,2000 ) cell
            cell( 1 ) = 100.
            cell( 2 ) = 100.
            cell( 3 ) = 100.
            cell( 4 ) = 90.
            cell( 5 ) = 90.
            cell( 6 ) = 90.
            write( 6,2100 ) cell
          endif
        endif

        if ( isg .ge. 75 .and. isg .le. 98 ) then
*       Tetragonal; a = b, alpha = beta = gamma = 90 ?
          if ( nint ( cell( 4 ) ) .ne. 90 .or.
     &         nint ( cell( 5 ) ) .ne. 90 .or.
     &         nint ( cell( 6 ) ) .ne. 90 .or.
     &         abs ( cell( 2 ) - cell( 1 ) ) .gt. 0.0001 ) then
            call wrnmsg ( 'Cell does not fit space group' )
            write( 6,2000 ) cell
            cell( 1 ) = 100.
            cell( 2 ) = 100.
            cell( 3 ) = 100.
            cell( 4 ) = 90.
            cell( 5 ) = 90.
            cell( 6 ) = 90.
            write( 6,2100 ) cell
          endif
        endif
        
        if ( isg .ge. 143 .and. isg .le. 182 ) then
*       Trigonal or hexagonal; a = b. alpha = beta = 90, gamma = 120 ?
*       (only trigonal on hexagonal axes recognized)
          if ( nint ( cell( 4 ) ) .ne. 90 .or.
     &         nint ( cell( 5 ) ) .ne. 90 .or.
     &         nint ( cell( 6 ) ) .ne. 120 .or.
     &         abs ( cell( 2 ) - cell( 1 ) ) .gt. 0.0001 ) then
            call wrnmsg ( 'Cell does not fit space group' )
            write( 6,2000 ) cell
            cell( 1 ) = 100.
            cell( 2 ) = 100.
            cell( 3 ) = 100.
            cell( 4 ) = 90.
            cell( 5 ) = 90.
            cell( 6 ) = 120.
            write( 6,2100 ) cell
          endif
        endif

        if ( isg .ge. 195 .and. isg .le. 214 ) then
*       Cubic; a = b = c. alpha = beta = gamma = 90 ?
          if ( nint ( cell( 4 ) ) .ne. 90 .or.
     &         nint ( cell( 5 ) ) .ne. 90 .or.
     &         nint ( cell( 6 ) ) .ne. 90 .or.
     &         abs ( cell( 2 ) - cell( 1 ) ) .gt. 0.0001 .or. 
     &         abs ( cell( 3 ) - cell( 1 ) ) .gt. 0.0001 .or. 
     &         abs ( cell( 3 ) - cell( 2 ) ) .gt. 0.0001 ) then
            call wrnmsg ( 'Cell does not fit space group' )
            write( 6,2000 ) cell
            cell( 1 ) = 100.
            cell( 2 ) = 100.
            cell( 3 ) = 100.
            cell( 4 ) = 90.
            cell( 5 ) = 90.
            cell( 6 ) = 90.
            write( 6,2100 ) cell
          endif
        endif

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
 2000      format(10x,'Cell    : ',6f8.2)
 2100      format(10x,'Cell changed to arbitrary default',
     &     /10x,'New cell: ',6f8.2,/)
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*       Subroutine REDUCE
*
*       Function:
*       Reduce vector to map volume.
*
*       Comment:
*       The s/r CHKLIM should be called prior to this routine,
*       to check map limits, and to set the noreduce flag if
*       the whole unit cell is kept in memory.
*
*
*       Arguments: 
*       u         (i/o)      vector coordinates (fractional)
*
*
*       History:
*       April 13 1993 - Pete Dunten
*       Fixed bug for Laue groups P6/mmm & Pm-3 
*       August 1992 - sdk:
*       Space group specific routines for trigonal and hexagonal
*       space groups added.
*       November 24 1991 S. Knight:
*       Compiled from separate space group specific reduce routines.
*       Changed to operate on fractional coordinates instead of grid.
*-----------------------------------------------------------------------
        subroutine reduce ( u )

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc' 

*       Argument
        real            u( 3 )

*       Local variables
        integer         i , istop
        real            x
        real            xm , ym , xmy , ymx
        real            sixt , quart , thrd , half , twtd , fvsx
        parameter       ( sixt  = 1./6. )
        parameter       ( quart = 1./4. )
        parameter       ( thrd  = 1./3. )
        parameter       ( half  = 1./2. )
        parameter       ( twtd  = 2./3. )
        parameter       ( fvsx  = 5./6. )

*-----------------------------------------------------------------------
*       Reduce to unit cell
        do i = 1 , 3
          u( i ) = mod ( u( i ) , 1.0 )
          if ( abs ( u( i ) ) .lt. 0.0001 ) u( i ) = 0.0
          if( u( i ) .lt. 0.0 ) u( i ) = u( i ) + 1.0
        end do

*       If full unit cell is stored we're done
        if ( noreduce ) return

*       Reduce to asymmetric unit 
************************************************************************
        if ( ispgrp .eq. 1 ) then

*       Laue group   : P-1 
*       Space groups : any 
*       Map volume   : 0 1 0 1/2 0 1
************************************************************************
          if ( u( 2 ) .ge. half ) then
            do i = 1 , 3
              if ( u( i ) .gt. 0.0 ) u( i ) = 1.0 - u( i )
            end do
          endif

************************************************************************
        else if ( ispgrp .eq. 3 .or. ispgrp .eq. 4 ) then

*       Laue group   : P2/m 2nd setting (sg 10)
*       Space groups : 3 & 4 (P2 & P21)
*       Map volume   : 0 1/2 0 1/2 0 1
************************************************************************
          if ( u(1) .ge. half .and. u(2) .lt. half ) then
            u(1) = 1.0 - u(1)
            if ( u(3) .gt. 0.0 ) u(3) = 1.0 - u(3)
          else 
     &    if ( u(1) .ge. half .and. u(2) .ge. half ) then
            u(1) = 1.0 - u(1)
            u(2) = 1.0 - u(2)
            if ( u(3) .gt. 0.0 ) u(3) = 1.0 - u(3)
          else
     &    if ( u(1) .lt. half .and. u(2) .ge. half ) then
            u(2) = 1.0 - u(2)
          endif

************************************************************************
        else if ( ispgrp .eq. 5 ) then

*       Laue group   : C2/m (sg 12)
*       Space group  : 5 (C2)
*       Map volume   : 0 1/2 0 1/4 0 1
************************************************************************
          if ( u(1) .ge. half .and. u(2) .lt. half ) then
            u(1) = 1.0 - u(1)
            if ( u(3) .gt. 0.0 ) u(3) = 1.0 - u(3)
          else if ( u(1) .ge. half .and. u(2) .ge. half ) then
            u(1) = 1.0 - u(1)
            u(2) = 1.0 - u(2)
            if ( u(3) .gt. 0.0 ) u(3) = 1.0 - u(3)
          else if ( u(1) .lt. half .and. u(2) .ge. half ) then
            u(2) = 1.0 - u(2)
          endif
          if ( u(2) .ge. quart ) then
            u(1) = half - u(1)
            u(2) = half - u(2)
            if ( u(3) .gt. 0.0 ) u(3) = 1.0  - u(3)
          endif

************************************************************************
        else if ( ispgrp .ge. 16 .and. ispgrp .le. 19 ) then

*       Laue group   : Pmmm (sg 47)
*       Space groups : 16-19 (P222,P2221,P21212,P212121)
*       Map volume   : 0 1/2 0 1/2 0 1/2
************************************************************************
          do i = 1 , 3
            if ( u(i) .gt. half ) u(i) = 1.0 - u(i)
          end do

************************************************************************
        else if ( ispgrp .ge. 20 .and. ispgrp .le. 21 ) then

*       Laue group   : Cmmm (sg 65)
*       Space groups : 20-21 (C2221,C222)
*       Map volume   : 0 1/2 0 1/4 0 1/2
************************************************************************
          do i = 1 , 3
            if ( u(i) .gt. half ) u(i) = 1.0 - u(i)
          end do
          if ( u(2) .ge. quart ) then
            u(1) = half - u(1)
            u(2) = half - u(2)
          endif

************************************************************************
        else if ( ispgrp .eq. 22 ) then

*       Laue group   : Fmmm (sg 69)
*       Space group  : 22 (F222)
*       Map volume   : 0 1/4 0 1/4 0 1/2 
************************************************************************
          do i = 1 , 3
            if ( u(i) .gt. half ) u(i) = 1.0 - u(i)
          end do
          if ( u(1) .ge. quart .and. u(2) .lt. quart ) then
            u(1) = half - u(1)
            u(3) = half - u(3)
          else if ( u(1) .ge. quart .and. u(2) .ge. quart ) then
            u(1) = half - u(1)
            u(2) = half - u(2)
          else if ( u(1) .lt. quart .and. u(2) .ge. quart ) then
           u(2) = half - u(2)
           u(3) = half - u(3)
          endif

************************************************************************
        else if ( ispgrp .ge. 23 .and. ispgrp .le. 24 ) then

*       Laue group   : Immm (sg 71)
*       Space groups : 23-24 (I222,I212121)
*       Map volume   : 0 1/2 0 1/4 0 1/2 
************************************************************************
          do i = 1 , 3
            if ( u(i) .gt. half ) u(i) = 1.0 - u(i)
          end do
          if ( u(2) .ge. quart ) then
            u(1) = half - u(1)
            u(2) = half - u(2)
            u(3) = half - u(3)
          endif

************************************************************************
        else if ( ispgrp .ge. 75 .and. ispgrp .le. 78 ) then

*       Laue group   : P4/m (sg 83)
*       Space groups : 75-78 (P4,P41,P42,P43)
*       Map volume   : 0 1/2 0 1/2 0 1/2 
************************************************************************
          if ( u(1) .ge. half .and. u(2) .lt. half .and.
     &    u(3) .lt. half ) then
            x = 1.0 - u(1)
            u(1) = u(2)
            u(2) = x
          else if ( u(1) .lt. half .and. u(2) .ge. half .and.
     &    u(3) .lt. half ) then
            x = u(1)
            u(1) = 1.0 - u(2)
            u(2) = x
          else if ( u(1) .lt. half .and. u(2) .lt. half .and.
     &    u(3) .ge. half ) then
            u(3) = 1.0 - u(3)
          else if ( u(1) .ge. half .and. u(2) .ge. half .and.
     &    u(3) .lt. half ) then
            u(1) = 1.0 - u(1)
            u(2) = 1.0 - u(2)
          else if ( u(1) .ge. half .and. u(2) .lt. half .and.
     &    u(3) .ge. half ) then
            x = 1.0 - u(1)
            u(1) = u(2)
            u(2) = x
            u(3) = 1.0 - u(3)
          else if ( u(1) .lt. half .and. u(2) .ge. half .and.
     &    u(3) .ge. half ) then
            x = u(1)
            u(1) = 1.0 - u(2)
            u(2) = x
            u(3) = 1.0 - u(3)
          else if ( u(1) .ge. half .and. u(2) .ge. half .and.
     &    u(3) .ge. half ) then
            u(1) = 1.0 - u(1)
            u(2) = 1.0 - u(2)
            u(3) = 1.0 - u(3)
          endif

************************************************************************
        else if ( ispgrp .ge. 79 .and. ispgrp .le. 80 ) then

*       Laue group   : I4/m (sg 87)
*       Space groups : 79-80 (I4,I41)
*       Map volume   : 0 1/2 0 1/2 0 1/4
************************************************************************
          if ( u(1) .ge. half .and. u(2) .lt. half .and.
     &    u(3) .lt. half ) then
            x = 1.0 - u(1)
            u(1) = u(2)
            u(2) = x
          else if ( u(1) .lt. half .and. u(2) .ge. half .and.
     &    u(3) .lt. half ) then
            x = u(1)
            u(1) = 1.0 - u(2)
            u(2) = x
          else if ( u(1) .lt. half .and. u(2) .lt. half .and.
     &    u(3) .ge. half ) then
            u(3) = 1.0 - u(3)
          else if ( u(1) .ge. half .and. u(2) .ge. half .and.
     &    u(3) .lt. half ) then
            u(1) = 1.0 - u(1)
            u(2) = 1.0 - u(2)
          else if ( u(1) .ge. half .and. u(2) .lt. half .and.
     &    u(3) .ge. half ) then
            x = 1.0 - u(1)
            u(1) = u(2)
            u(2) = x
            u(3) = 1.0 - u(3)
          else if ( u(1) .lt. half .and. u(2) .ge. half .and.
     &    u(3) .ge. half ) then
            x = u(1)
            u(1) = 1.0 - u(2)
            u(2) = x
            u(3) = 1.0 - u(3)
          else if ( u(1) .ge. half .and. u(2) .ge. half .and.
     &    u(3) .ge. half ) then
            u(1) = 1.0 - u(1)
            u(2) = 1.0 - u(2)
            u(3) = 1.0 - u(3)
          endif
          if ( u(3) .ge. quart ) then
            u(1) = half - u(1)
            u(2) = half - u(2)
            u(3) = half - u(3)
          endif

************************************************************************
        else if ( ispgrp .ge. 89 .and. ispgrp .le. 96 ) then

*       Laue group   : P4/mmm (sg 123)
*       Space groups : 89-96 (P422,P4212,P4122,P41212,P4222,
*                      P42212,P4322,P43212)
*       Map volume   : 0 1/2 0 1/2 0 1/2
************************************************************************
          do i = 1 , 3
            if ( u(i) .gt. half ) u(i) = 1.0 - u(i)
          end do

************************************************************************
        else if ( ispgrp .ge. 97 .and. ispgrp .le. 98 ) then

*       Laue group   : I4/mmm (sg 139)
*       Space groups : 97-98 (I422,I4122)
*       Map volume   : 0 1/2 0 1/2 0 1/4 
************************************************************************
          do i = 1 , 3
            if ( u(i) .gt. half ) u(i) = 1.0 - u(i)
          end do
          if ( u(3) .ge. quart ) then
            u(1) = half - u(1)
            u(2) = half - u(2)
            u(3) = half - u(3)
          endif

************************************************************************
        else if ( ispgrp .ge. 143 .and. ispgrp .le. 145 ) then

*       Laue group   : P-3 (sg 147) 
*       Space group  : 143-145 (P3,P31,P32)
*       Map volume   : 0 2/3 0 2/3 0 1/2
************************************************************************
          if ( u(3) .gt. half ) then
            u(1) = 1.0 - u(1)
            u(2) = 1.0 - u(2)
            u(3) = 1.0 - u(3)
          endif

          if ( ( u(2) .gt. twtd .and. u(1) .le. thrd ) .or.
     &         ( u(1) .gt. twtd .and.
     &           u(2) .gt. thrd .and. u(2) .le. twtd ) .or.
     &         ( u(1) .gt. twtd .and. u(2) .gt. twtd .and.
     &           u(1) .ge. u(2) ) ) then

*       Operation (2): -y, x-y, z
            x = u(1)
            u(1) = 1.0 - u(2)
            u(2) = x - u(2)
            if ( u(2) .lt. 0.0 ) u(2) = u(2) + 1.0

          else if ( ( u(1) .gt. twtd .and. u(2) .le. thrd ) .or.
     &              ( u(2) .gt. twtd .and. 
     &                u(1) .gt. thrd .and. u(1) .le. twtd ) .or.
     &              ( u(1) .gt. twtd .and. u(2) .gt. twtd .and. 
     &                u(1) .lt. u(2) ) ) then

*       Operation (3): y-x, -x, z
            x = u(1)
            u(1) = u(2) - u(1) 
            u(2) = 1.0 - x
            if ( u(1) .lt. 0.0 ) u(1) = u(1) + 1.0

          endif

************************************************************************
        else if ( ispgrp .eq. 146 ) then

*       Laue group   : R-3 (sg 148) (Hexagonal axes)
*       Space group  : 146 (R3 Hexagonal axes)
*       Map volume   : 0 2/3 0 2/3 0 1/6
************************************************************************
*       First get 0 <= z <= 1/6 by centering and lattice translations
*       to point or inverse point
          if ( u(3) .gt. sixt .and. u(3) .le. thrd ) then
            u(1) = twtd - u(1)
            u(2) = thrd - u(2)
            u(3) = thrd - u(3)
            if ( u(1) .lt. 0.0 ) u(1) = u(1) + 1.0
            if ( u(2) .lt. 0.0 ) u(2) = u(2) + 1.0
          else if ( u(3) .gt. thrd .and. u(3) .le. half ) then
            u(1) = mod ( thrd + u(1) , 1.0 )
            u(2) = mod ( twtd + u(2) , 1.0 )
            u(3) = mod ( twtd + u(3) , 1.0 )
          else if ( u(3) .gt. half .and. u(3) .le. twtd ) then
            u(1) = thrd - u(1)
            u(2) = twtd - u(2)
            u(3) = twtd - u(3)
            if ( u(1) .lt. 0.0 ) u(1) = u(1) + 1.0
            if ( u(2) .lt. 0.0 ) u(2) = u(2) + 1.0
          else if ( u(3) .gt. twtd .and. u(3) .le. fvsx ) then
            u(1) = mod ( twtd + u(1) , 1.0 )
            u(2) = mod ( thrd + u(2) , 1.0 )
            u(3) = mod ( thrd + u(3) , 1.0 )
          else if ( u(3) .gt. fvsx ) then
            u(1) = 1.0 - u(1)
            u(2) = 1.0 - u(2)
            u(3) = 1.0 - u(3)
          endif


*       Now move into ua 2/3 2/3 1/6 by rotation (2) or (3) in I.T.
          if ( ( u(2) .gt. twtd .and. u(1) .le. thrd ) .or.
     &         ( u(1) .gt. twtd .and.
     &           u(2) .gt. thrd .and. u(2) .le. twtd ) .or.
     &         ( u(1) .gt. twtd .and. u(2) .gt. twtd .and.
     &           u(1) .ge. u(2) ) ) then

*       Operation (2): -y, x-y, z
            x = u(1)
            u(1) = 1.0 - u(2)
            u(2) = x - u(2)
            if ( u(2) .lt. 0.0 ) u(2) = u(2) + 1.0

          else if ( ( u(1) .gt. twtd .and. u(2) .le. thrd ) .or.
     &              ( u(2) .gt. twtd .and. 
     &                u(1) .gt. thrd .and. u(1) .le. twtd ) .or.
     &              ( u(1) .gt. twtd .and. u(2) .gt. twtd .and. 
     &                u(1) .lt. u(2) ) ) then

*       Operation (3): y-x, -x, z
            x = u(1)
            u(1) = u(2) - u(1) 
            u(2) = 1.0 - x
            if ( u(1) .lt. 0.0 ) u(1) = u(1) + 1.0

          endif

************************************************************************
        else if ( ispgrp .eq. 149 .or. ispgrp .eq. 151 .or. 
     &            ispgrp .eq. 153 ) then

*       Laue group   : P-31m (162)
*       Space groups : 149,151,153 (P312,P3112,P3212)
*       Map volume   : 0 2/3 0 1/2 0 1/2
************************************************************************
          if ( u(3) .gt. half ) then
            u(1) = 1.0 - u(1)
            u(2) = 1.0 - u(2)
            u(3) = 1.0 - u(3)
          endif

          if ( u(1) .le. twtd .and. u(2) .le. half ) return

          xmy = u(1) - u(2)
          ymx = -xmy
          xm  = 1.0 - u(1)
          ym  = 1.0 - u(2)
          if ( xmy .lt. 0.0 ) xmy = xmy + 1.0
          if ( ymx .lt. 0.0 ) ymx = ymx + 1.0

          if ( ym .le. twtd .and. xmy .le. half ) then
            u(1) = ym
            u(2) = xmy
          else if ( ymx .le. twtd .and. xm .le. half ) then
            u(1) = ymx
            u(2) = xm
          else if ( u(2) .le. twtd .and. u(1) .le. half ) then
            x = u(1)
            u(1) = u(2)
            u(2) = x
          else if ( xmy .le. twtd .and. ym .le. half ) then
            u(1) = xmy
            u(2) = ym
          else if ( xm .le. twtd .and. ymx .le. half ) then
            u(1) = xm
            u(2) = ymx
          endif

************************************************************************
        else if ( ispgrp .eq. 150 .or. ispgrp .eq. 152 .or. 
     &            ispgrp .eq. 154 ) then

*       Laue group   : P-3m1 (164)
*       Space groups : 150,152,154 (P321,P3121,P3221)
*       Map volume   : 0 2/3 0 1/3 0 1
************************************************************************
          if ( u(1) .le. twtd .and. u(2) .le. thrd ) return

          xmy = u(1) - u(2)
          ymx = -xmy
          xm  = 1.0 - u(1)
          ym  = 1.0 - u(2)
          if ( xmy .lt. 0.0 ) xmy = xmy + 1.0
          if ( ymx .lt. 0.0 ) ymx = ymx + 1.0

          if ( ym .le. twtd .and. xmy .le. thrd ) then
            u(1) = ym
            u(2) = xmy
          else if ( ymx .le. twtd .and. xm .le. thrd ) then
            u(1) = ymx
            u(2) = xm
          else if ( u(2) .le. twtd .and. u(1) .le. thrd ) then
            x = u(1)
            u(1) = u(2)
            u(2) = x
            u(3) = 1.0 - u(3)
          else if ( xmy .le. twtd .and. ym .le. thrd ) then
            u(1) = xmy
            u(2) = ym
            u(3) = 1.0 - u(3)
          else if ( xm .le. twtd .and. ymx .le. thrd ) then
            u(1) = xm
            u(2) = ymx
            u(3) = 1.0 - u(3)
          else if ( xm .le. twtd .and. ym .le. thrd ) then
            u(1) = xm
            u(2) = ym
            u(3) = 1.0 - u(3)
          else if ( u(2) .le. twtd .and. ymx .le. thrd ) then
            u(1) = u(2)
            u(2) = ymx
            u(3) = 1.0 - u(3)
          else if ( xmy .le. twtd .and. u(1) .le. thrd ) then
            x = u(1)
            u(1) = xmy
            u(2) = x
            u(3) = 1.0 - u(3)
          else if ( ym .le. twtd .and. xm .le. thrd ) then
            u(1) = ym
            u(2) = xm
          else if ( ymx .le. twtd .and. u(2) .le. thrd ) then
            u(1) = ymx
          else if ( u(1) .le. twtd .and. xmy .le. thrd ) then
            u(2) = xmy
          endif

************************************************************************
        else if ( ispgrp .eq. 155 ) then

*       Laue group   : R-3m (sg 166 Hexagonal axes)
*       Space groups : 155 (R32)
*       Map volume   : 0 2/3 0 2/3 0 1/6
************************************************************************
*       First get 0 <= z <= 1/6 by centering and lattice translations
*       to point or inverse point
          if ( u(3) .gt. sixt .and. u(3) .le. thrd ) then
            u(1) = twtd - u(1)
            u(2) = thrd - u(2)
            u(3) = thrd - u(3)
            if ( u(1) .lt. 0.0 ) u(1) = u(1) + 1.0
            if ( u(2) .lt. 0.0 ) u(2) = u(2) + 1.0
          else if ( u(3) .gt. thrd .and. u(3) .le. half ) then
            u(1) = mod ( thrd + u(1) , 1.0 )
            u(2) = mod ( twtd + u(2) , 1.0 )
            u(3) = mod ( twtd + u(3) , 1.0 )
          else if ( u(3) .gt. half .and. u(3) .le. twtd ) then
            u(1) = thrd - u(1)
            u(2) = twtd - u(2)
            u(3) = twtd - u(3)
            if ( u(1) .lt. 0.0 ) u(1) = u(1) + 1.0
            if ( u(2) .lt. 0.0 ) u(2) = u(2) + 1.0
          else if ( u(3) .gt. twtd .and. u(3) .le. fvsx ) then
            u(1) = mod ( twtd + u(1) , 1.0 )
            u(2) = mod ( thrd + u(2) , 1.0 )
            u(3) = mod ( thrd + u(3) , 1.0 )
          else if ( u(3) .gt. fvsx ) then
            u(1) = 1.0 - u(1)
            u(2) = 1.0 - u(2)
            u(3) = 1.0 - u(3)
          endif


*       Now move into ua 2/3 2/3 1/6 by rotation (2) or (3) in IT.
          if ( ( u(2) .gt. twtd .and. u(1) .le. thrd ) .or.
     &         ( u(1) .gt. twtd .and.
     &           u(2) .gt. thrd .and. u(2) .le. twtd ) .or.
     &         ( u(1) .gt. twtd .and. u(2) .gt. twtd .and.
     &           u(1) .ge. u(2) ) ) then

*       Operation (2): -y, x-y, z
            x = u(1)
            u(1) = 1.0 - u(2)
            u(2) = x - u(2)
            if ( u(2) .lt. 0.0 ) u(2) = u(2) + 1.0

          else if ( ( u(1) .gt. twtd .and. u(2) .le. thrd ) .or.
     &              ( u(2) .gt. twtd .and. 
     &                u(1) .gt. thrd .and. u(1) .le. twtd ) .or.
     &              ( u(1) .gt. twtd .and. u(2) .gt. twtd .and. 
     &                u(1) .lt. u(2) ) ) then

*       Operation (3): y-x, -x, z
            x = u(1)
            u(1) = u(2) - u(1) 
            u(2) = 1.0 - x
            if ( u(1) .lt. 0.0 ) u(1) = u(1) + 1.0

          endif

************************************************************************
        else if ( ispgrp .ge. 168 .and. ispgrp .le. 173 ) then

*       Laue group   : P6/m (175)
*       Space groups : 168-173 (P6,P61,P65,P62,P64,P63)
*       Map volume   : 0 2/3 0 1/2 0 1/2
************************************************************************
          if ( u(3) .gt. half ) u(3) = 1.0 - u(3)

          if ( u(1) .le. twtd .and. u(2) .le. half ) return

          xmy = u(1) - u(2)
          ymx = -xmy
          xm  = 1.0 - u(1)
          ym  = 1.0 - u(2)
          if ( xmy .lt. 0.0 ) xmy = xmy + 1.0
          if ( ymx .lt. 0.0 ) ymx = ymx + 1.0

          if ( ym .le. twtd .and. xmy .le. half ) then
            u(1) = ym
            u(2) = xmy
          else if ( ymx .le. twtd .and. xm .le. half ) then
            u(1) = ymx
            u(2) = xm
          else if ( xm .le. twtd .and. ym .le. half ) then
            u(1) = xm
            u(2) = ym
          else if ( u(2) .le. twtd .and. ymx .le. half ) then
            u(1) = u(2)
            u(2) = ymx
          else if ( xmy .le. twtd .and. u(1) .le. half ) then
            u(2) = u(1)
            u(1) = xmy
          endif


************************************************************************
        else if ( ispgrp .ge. 177 .and. ispgrp .le. 182 ) then

*       Laue group   : P6/mmm (191)
*       Space groups : 177-182 (P622,P6122,P6522,P6222,P6422,P6322)
*       Map volume   : 0 2/3 0 1/3 0 1/2
************************************************************************
          if ( u(3) .gt. half ) u(3) = 1.0 - u(3)

          if ( u(1) .le. twtd .and. u(2) .le. thrd ) return

          xmy = u(1) - u(2)
          ymx = -xmy
          xm  = 1.0 - u(1)
          ym  = 1.0 - u(2)
          if ( xmy .lt. 0.0 ) xmy = xmy + 1.0
          if ( ymx .lt. 0.0 ) ymx = ymx + 1.0

          if ( ym .le. twtd .and. xmy .le. thrd ) then
            u(1) = ym
            u(2) = xmy
          else if ( ymx .le. twtd .and. xm .le. thrd ) then
            u(1) = ymx
            u(2) = xm
          else if ( xm .le. twtd .and. ym .le. thrd ) then
            u(1) = xm
            u(2) = ym
          else if ( u(2) .le. twtd .and. ymx .le. thrd ) then
            u(1) = u(2)
            u(2) = ymx
          else if ( xmy .le. twtd .and. u(1) .le. thrd ) then
            u(2) = u(1)
            u(1) = xmy
          else if ( ym .le. twtd .and. xm .le. thrd ) then
            u(1) = ym
            u(2) = xm
          else if ( ymx .le. twtd .and. u(2) .le. thrd ) then
            u(1) = ymx
          else if ( u(1) .le. twtd .and. xmy .le. thrd ) then
            u(2) = xmy
          else if ( u(2) .le. twtd .and. u(1) .le. thrd ) then
            x = u(1)
            u(1) = u(2)
            u(2) = x
          else if ( xmy .le. twtd .and. ym .le. thrd ) then
            u(1) = xmy
            u(2) = ym
          else if ( xm .le. twtd .and. ymx .le. thrd ) then
            u(1) = xm
            u(2) = ymx
          endif

************************************************************************
        else if ( ispgrp .eq. 195 .or. ispgrp .eq. 198 ) then

*       Laue group   : Pm-3 (sg 200)
*       Space groups : 195,198 (P23,P213)
*       Map volume   : 0 1/2 0 1/2 0 1/2
************************************************************************
          do i = 1 , 3
            if ( u(i) .gt. half ) u(i) = 1.0 - u(i)
          end do

************************************************************************
        else if ( ispgrp .eq. 196 ) then

*       Laue group   : Fm-3 (sg 202)
*       Space group  : 196 (F23)
*       Map volume   : 0 1/2 0 1/2 0 1/4
************************************************************************
          do i = 1 , 3
            if ( u(i) .gt. half ) u(i) = 1.0 - u(i)
          end do
          if ( u(3) .ge. quart ) then
            u(1) = half - u(1)
            u(3) = half - u(3)
          endif
 
************************************************************************
        else if ( ispgrp .eq. 197 .or. ispgrp .eq. 199 ) then

*       Laue group   : Im-3 (sg 204)
*       Space groups : 197,199 (I23,I213)
*       Map volume   : 0 1/2 0 1/2 0 1/2
************************************************************************
          do i = 1 , 3
            if ( u(i) .gt. half ) u(i) = 1.0 - u(i)
          end do

************************************************************************
        else if ( ispgrp .ge. 207 .and. ispgrp .le. 208 .or.
     &            ispgrp .ge. 212 .and. ispgrp .le. 213 ) then

*       Laue group   : Pm-3m (sg 221)
*       Space groups : 207-208,212-213 (P432,P4232,P4332,P4132)
*       Map volume   : 0 1/2 0 1/2 0 1/2
************************************************************************
          do i = 1 , 3
            if ( u(i) .gt. half ) u(i) = 1.0 - u(i)
          end do

************************************************************************
        else if ( ispgrp .ge. 209 .and. ispgrp .le. 210 ) then

*       Laue group   : Fm-3m (sg 225)
*       Space groups : 209-210 (F432,F4132)
*       Map volume   : 0 1/2 0 1/4 0 1/4
************************************************************************
          do i = 1 , 3
            if ( u(i) .gt. half ) u(i) = 1.0 - u(i)
          end do
          if ( u(2) .ge. quart .and. u(3) .lt. quart ) then
            u(1) = half - u(1)
            u(2) = half - u(2)
          else if ( u(2) .ge. quart .and. u(3) .ge. quart ) then
            u(2) = half - u(2)
            u(3) = half - u(3)
          else if ( u(2) .lt. quart .and. u(3) .ge. quart ) then
            u(1) = half - u(1)
            u(3) = half - u(3)
          endif

************************************************************************
        else if ( ispgrp .eq. 211 .or. ispgrp .eq. 214 ) then

*       Laue group   : Im-3m (sg 229)
*       Space groups : 211,214 (I432,I4132)
*       Map volume   : 0 1/2 0 1/2 0 1/4
************************************************************************
          do i = 1 , 3
            if ( u(i) .gt. half ) u(i) = 1.0 - u(i)
          end do
          if ( u(3) .ge. quart ) then
            u(1) = half - u(1)
            u(2) = half - u(2)
            u(3) = half - u(3)
          endif

************************************************************************
        else
************************************************************************

          istop = 0
          if (stop_on_error) istop = 1
          call errmsg ( 'Illegal space group' , istop )

************************************************************************
        endif
************************************************************************

        return


*=======================================================================
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine STGRD
*
*       Function:
*       Set score map grid from Patterson grid.
*
*
*       Arguments:
*       None.
*
*
*       History:
*       February 14 1994 - sdk
*       Modified for new MODEs
*       Written November 10 1991 S. Knight
************************************************************************
        subroutine stgrd

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'
        include         'rspsgrd.inc'

*       Local variables
        integer         i

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=


*       Set default score map grid
        if ( index ( vecset , 'SINGLE' ) .ne. 0 ) then
          do i = 1 , 3
            scrgrd( i ) = 2 * patgrd( i )
          end do
        else 
          do i = 1 , 3
            scrgrd( i ) = patgrd( i )
          end do
        endif

        return
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Map handling routines for rsps
*
*       S/r rdhdr       Open PROTEIN or CCP4 file and return nsec,mxyz,
*                       iuvw,mygrid,limit,cell
*       S/r rdmap       Read PROTEIN or CCP4 map into core
*       S/r rdsec       READ one section of PROTEIN/CCP4 map
*       S/r mapstat     Compute statistics for stored map 
*       S/r maptrun     Truncate stored map
*       S/r maprset     Reset peaks in stored map
*       S/r mapscal     Scale stored map
*       S/r maptop      Position map file before first density record
*       S/r mflstat     Get statistics for map on file
*       S/r getr0       Read PROTEIN record type 0
*       S/r getr1       Read PROTEIN record type 1
*       S/r getr2       Read PROTEIN record type 2
*       S/r getr5       Read PROTEIN record type 5
*
************************************************************************
************************************************************************
*
*       Subroutine RDHDR
*
*       Function:
*       Open CCP4 or PROTEIN format map file and return some of the 
*       header info
*
*       Arguments:
*       What    (i)     map format (PROTEIN or CCP4)
*       Iunit   (i)     logical unit number
*       Mapnam  (i)     logical file name
*       Nsec    (o)     number of map sections
*       Mxyz    (o)     extent along x, y, z
*       Iuvw    (o)     sectioning flags along fast, medium, slow axis
*       Grid    (o)     grid along x, y, z
*       Limit   (o)     limits along x, y, z
*       Cell    (o)     cell parameters
*       Titl    (o)     map title
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - MRDHDS error
*                       errcod = 2 - error opening PROTEIN map
*                       errcod = 3 - error reading PROTEIN type 0 header
*                       errcod = 4 - error reading PROTEIN type 1 header
*                       errcod = 5 - error reading PROTEIN type 2 header
*                       errcod = 6 - file does not exist
*
*
*       History:
*       November 19 1991 - sdk
*       Return on error with errcod .ne. 0
*       March 22 1988 - sdk
*       Return header info as arguments instead of in common
*       Written September 1987 S. Knight
************************************************************************
        subroutine rdhdr ( what,iunit,mapnam,nsec,mxyz,iuvw,
     &  grid,limit,cell,titl,errcod )

        implicit none

*       Arguments
        integer         iunit
        character*(*)   what
        character*(*)   mapnam
        integer         nsec
        integer         mxyz(3),iuvw(3),grid(3),limit(6)
        real            cell(6)
        character*80    titl
        integer         errcod

*       Common variables
        integer         mifd(9)
        common/mifd     /mifd

*       Local variables
        integer         i
        integer         nw1 , nu1 , nu2 , nv1 , nv2
        integer         lspgrp , lmode
        integer         nsym
        integer         ipt
        integer         l(6),iperm(3),igrid(3)
        integer         junk(20),title(15)
        integer         ifail
        real            rhmin , rhmax , rhmean , rhrms
        character*80    catlin

*       External functions
        integer         slnblk
        logical         ccpexs
*-----------------------------------------------------------------------
*       Init error code
        errcod = 0

*       Check file exists
        if ( .not. ccpexs ( mapnam ) ) then
          catlin = 'File does not exist > ' // 
     &    mapnam( :slnblk ( mapnam ) ) 
          call errmsg ( catlin , 0 )
          errcod = 6
          return
        endif

        if ( what .eq. 'CCP4' ) then
*       Read CCP4 format header
          ifail = 1
          call mrdhds ( iunit,mapnam,titl,nsec,iuvw,grid,nw1,nu1,nu2,
     &    nv1,nv2,cell,lspgrp,lmode,rhmin,rhmax,rhmean,rhrms,ifail,0 )
          if ( ifail .ne. 1 ) then
            call errmsg ( 'MRDHDS failed reading header' , 0 )
            call mrclos ( iunit )
            errcod = 1
            return
          endif
          l(1) = nu1
          l(2) = nu2
          l(3) = nv1
          l(4) = nv2
          l(5) = nw1
          l(6) = nw1 + nsec - 1
          do i = 1 , 3
*       Extent along x,y,z
            mxyz( iuvw(i) ) = l( 2*i ) -l( 2*i-1 ) + 1
*       Layout along x,y,z
            limit( 2*iuvw(i)-1 ) = l( 2*i-1 )
            limit( 2*iuvw(i)   ) = l( 2*i   )
          end do


        else if (what.eq.'PROTEIN' ) then
*       Read PROTEIN format header
          ifail = 1
          call ccpdpn ( -iunit,mapnam,'READONLY','U',0,ifail )
          if ( ifail .ne. 1 ) then 
            call errmsg ( 'Failed opening PROTEIN format map' , 0 )
            close ( iunit )
            errcod = 2
            return
          endif
*       Read the type 0 header record
          call getr0 ( iunit,junk,errcod )
          if ( errcod .ne. 0 ) then
            call errmsg ( 'Failed reading type 0 header of PROTEIN ' //
     &      'format map' , 0 )
            close ( iunit )
            errcod = 3
            return
          endif
*       Read the type 1 header and symmetry card
          call getr1 ( iunit,cell,nsym,errcod )
          if ( errcod .ne. 0 ) then
            call errmsg ( 'Failed reading type 1 header of PROTEIN ' //
     &      'format map' , 0 )
            close ( iunit )
            errcod = 4
            return
          endif
*       Read the type 2 map index parameter record
          call getr2 ( iunit,iperm,igrid,mifd,title,errcod )
          if ( errcod .ne. 0 ) then
            call errmsg ( 'Failed reading type 2 header of PROTEIN ' //
     &      'format map' , 0 )
            close ( iunit )
            errcod = 5
            return
          endif
*       Copy integer title to character
          write(titl,2000)title

          do i=1,3
            ipt=(i-1)*3+1
*       Extent along x,y,z
            mxyz(iperm(i))=mifd(ipt+1)-mifd(ipt)+1
*       Layout along x,y,z
            limit(2*iperm(i)-1)=mifd(ipt)
            limit(2*iperm(i))=mifd(ipt+1)
*       Grid along x,y,z
            grid(iperm(i))=igrid(i)
          end do
*       Sectioning flags along fast,medium,slow axis
          iuvw(1)=iperm(2)
          iuvw(2)=iperm(1)
          iuvw(3)=iperm(3)
*       Number of sections
          nsec = mxyz( iuvw( 3 ) )

        else

*       Illegal map format
          catlin = 'Illegal map format > ' // 
     &    what( :slnblk ( what ) ) 
          call errmsg ( catlin , 1 )

        endif


        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 2000   format(15a4)

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine RDMAP
*
*       Function:
*       Read PROTEIN or CCP4 map into core
*
*       Arguments:
*       Rhomap      (i/o)     map array
*       Nfast       (i)       number of points in fast direction of map
*       Nslow       (i)       number of points in slow direction of map
*       Nsec        (i)       number of sections in map
*       Rhotyp      (i)       map format flag
*       Iunit       (i)       logical unit of map file
*       Errcod      (o)       error code:
*                             0 - no error
*                             1 - error reading CCP4 map
*                             2 - error reading PROTEIN map
*                             3 - illegal map format flag
*
*       History:
*       Written sometime S. Knight
************************************************************************
        subroutine rdmap ( rhomap , nfast , nslow , nsec , rhotyp ,
     &  iunit , errcod )

        implicit none

*       Arguments
        character*(*)    rhotyp
        integer          nfast , nslow , nsec 
        integer          iunit
        integer          errcod
        real             rhomap( nfast , nslow ,nsec )

*       Local variables
        integer         i
        integer         npps , nrho , ibp0

*       Variables in common
        integer         mifd
        common/mifd     /mifd(9)
*-----------------------------------------------------------------------
*
*       Init
*
        errcod  =  0
        nrho    =  0
        npps    =  nfast * nslow

*
*       Check for valid map type
*
        if ( rhotyp .ne. 'PROTEIN'  .and. rhotyp .ne. 'CCP4' ) then
          errcod = 3
          return
        endif

        do 10 i = 1 , nsec
        if ( rhotyp .eq. 'CCP4' ) then

*       CCP4

          call mgulp ( iunit , rhomap( 1 , 1, i ) , errcod )
          if ( errcod .ne. 0 ) then
            errcod = 1
            return
          endif

        else if ( rhotyp .eq. 'PROTEIN' ) then

*       PROTEIN

          call getr5 ( iunit , mifd , npps , rhomap( 1 , 1 , i ) , 
     &    nrho , ibp0 , errcod )
          if ( errcod .ne. 0 ) then
            errcod = 2
            return
          endif

        endif

 10     continue


*
*       Close map file
*
        if ( rhotyp .eq. 'CCP4' ) then
          call mrclos ( iunit )
        else
          close ( iunit )
        endif

        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine RDSEC
*
*       Function:
*       Read one section from PROTEIN or CCP4 map file
*
*       Arguments:
*       What     (i)     maptype
*       Iunit    (i)     logical unit for read
*       Rho      (i/o)   density array
*       N        (i)     dimension of density array
*       Rhomax   (o)     maximum density
*       Rhomin   (o)     minimum density
*       Rhosum   (o)     sum of density
*       Rhossq   (o)     sum of squares of density
*
*       History:
*       Obscure
************************************************************************
        subroutine rdsec (what,iunit,rho,n,rhomax,rhomin,rhosum,rhossq)

        implicit none

*       Arguments
        integer         iunit
        integer         n
        character*(*)   what
        real            rhomax,rhomin,rhosum,rhossq
        real            rho(n)

*       Variables in common
        integer         mifd
        common/mifd     /mifd(9)

*       Local variables
        integer         i , nrho , ibp0 , ier
        character*80    catlin

*       External functions
        integer         slnblk
*-----------------------------------------------------------------------



        if (what.eq.'CCP4' ) then


          call mgulp ( iunit,rho,ier )
          if ( ier .ne. 0 ) n = -1


        else if (what.eq.'PROTEIN' ) then


          call getr5 ( iunit,mifd,n,rho,nrho,ibp0,ier )
          if (ier.ne.0)n=-1

        else

*       Illegal map format
          catlin = 'Illegal map format > ' // 
     &    what( :slnblk ( what ) ) 
          call errmsg ( catlin , 1 )

        endif

        do i=1,n
        rhomax=max(rhomax,rho(i))
        rhomin=min(rhomin,rho(i))
        rhosum=rhosum+rho(i)
        rhossq=rhossq+rho(i)**2
        end do

        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine MAPSTAT
*
*       Function:
*       Compute statistics for map in core
*
*       Arguments:
*       Rhomap      (i)       map array
*       N           (i)       number of points in map
*       Rhomin      (o)       minimum value in map
*       Rhomax      (o)       maximum value in map
*       Rhoave      (o)       average value of map
*       Rhosig      (o)       standard deviation of map
*       Errcod      (o)       error code:
*                             0 - no error
*                             1 - no points in map
*
*       History:
*       Written August 27 1991 sdk
*       970828 sdk: changed local variables to double precision to
*                   avoid problems due to low precision
************************************************************************
        subroutine mapstat ( rhomap , n , rhomin , rhomax , 
     &  rhoave , rhosig , errcod )

        implicit none

*       Arguments
        integer          n
        integer          errcod
        real             rhomap( n )
        real             rhomin , rhomax , rhoave , rhosig

*       Local variables
        integer          i 
        double precision rho , rhosum , rhossq , varians
        double precision rmin , rmax

*-----------------------------------------------------------------------
*
*       Init
*
        rmin    =  1.e+38
        rmax    = -1.e+38
        rhoave  =  0.
        rhosig  =  0.
        rhosum  =  0.
        rhossq  =  0.
        varians =  0.
        rho     =  0.
        errcod  =  0
*
*       Error trap
*
        if ( n .le. 0 ) then
          errcod = 1
          return
        endif
*
*       Compute statistics
*
        do 100 i = 1 , n
          rho = rhomap( i )
          rmax = max ( rmax , rho )
          rmin = min ( rmin , rho )
          rhosum = rhosum + rho
          rhossq = rhossq + rho **2
 100    continue
        rhomin  = rmin
        rhomax  = rmax
        rhoave  = rhosum / n
        rhosig  = 0.0001
        varians = ( rhossq - n * (rhoave**2) ) / ( n - 1 )
        if ( varians .gt. 0 ) rhosig = sqrt ( varians )

        return
        end

*********************************************************************SDK
************************************************************************
*
*       Subroutine MAPTRUN
*
*       Function:
*       Truncate density in map to dmax
*
*       Arguments:
*       Rhomap    (i/o)  map array
*       N         (i)    dimension of map array
*       Dmax      (i)    truncation limit
*       Dlev      (i)    truncation reset level
*
*       History:
*       Written April 14 1988 S. Knight
************************************************************************
        subroutine maptrun ( rhomap , n , dmax , dlev )

        implicit none

*       Arguments
        integer         n
        real            rhomap( n )
        real            dmax
        real            dlev

*       Local variables
        integer         i
*-----------------------------------------------------------------------
        do 10 i = 1 , n
          if ( rhomap( i ) .ge. dmax ) rhomap( i ) = dlev
 10     continue
        return
*-----------------------------------------------------------------------
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine MAPRSET
*
*       Function:
*       Reset peaks in map
*
*       Comments:
*       The coordinates for reset-peaks are expected to be fractional
*       modulo 1.0
*
*       Arguments:
*       Rhomap      (i/o)     map array
*       Nfast       (i)       number of points along fast direction
*       Nslow       (i)       number of points along slow direction
*       Nsect       (i)       number of sections
*       Nrset       (i)       number of peaks to reset
*       Xrset       (i)       fractional coordinates of peaks to reset
*       Rrset       (i)       reset radii in Angstrom
*       Lrset       (i)       reset levels
*       Mapaxo      (i)       axis order in map
*       Mapgrd      (i)       map grid
*       Cell        (i)       unit cell parameters
*
*
*       History:
*       February 17 1994 - sdk
*       Following a bug report from Dave Schuller (UCI) the following
*       changes were made:
*       Initial check of distance now uses grid coordinates for both
*       the distance and the cutoff.
*       Grid coordinates of reset points are now shifted to array
*       indices.
*       February 10 1994 - sdk
*       Changed reset coordinates from grid to fractional.
*       Unit cell translations now considered when resetting peaks.
*       Written September 1 1992 S. Knight
************************************************************************
        subroutine maprset ( rhomap , nfast , nslow , nsect , 
     &  nrset , xrset , rrset , lrset , mapaxo , mapgrd , cell )

        implicit none


*       Arguments
        integer          nfast , nslow , nsect , nrset
        integer          mapaxo( 3 ) , mapgrd( 3 )
        real             rhomap( nfast , nslow , nsect )
        real             xrset( 3,nrset )
        real             rrset( nrset )
        real             lrset( nrset )
        real             cell( 6 )


*       Local variables
        integer          i , j , k , irset
        integer          d
        real             cosang( 3 )
        real             x( 3 ) , u( 3 ) , xrs( 3 ) 
        real             g1 , g2 , g3
        real             r1 , r2 , r3

*       External functions
        real             vlen
*-----------------------------------------------------------------------
*       Get cosine of cell angles
        call cosabg ( cell( 4 ) , cosang )

*       Get grid along fast, slow, section axes
        g1 = mapgrd( mapaxo( 1 ) )
        g2 = mapgrd( mapaxo( 2 ) )
        g3 = mapgrd( mapaxo( 3 ) )

*       Loop over reset points
        do 400 irset = 1 , nrset

*       Get reset radius in grid units
          r1 = rrset( irset ) * g1 / cell( mapaxo( 1 ) )
          r2 = rrset( irset ) * g2 / cell( mapaxo( 2 ) )
          r3 = rrset( irset ) * g3 / cell( mapaxo( 3 ) )
 
*       Get reset coordinates in grid units
          xrs( 1 ) = nint ( xrset( 1,irset ) * mapgrd( 1 ) ) + 1
          xrs( 2 ) = nint ( xrset( 2,irset ) * mapgrd( 2 ) ) + 1
          xrs( 3 ) = nint ( xrset( 3,irset ) * mapgrd( 3 ) ) + 1

*       Loop over points in map and store grid coordinates along x,y,z
*       of points within cutoff of reset position.
          do 300 i = 1 , nsect

            d = i - xrs( mapaxo( 3 ) )
            if ( d .gt.  g3/2. ) d = d - g3
            if ( d .lt. -g3/2. ) d = d + g3
            if ( abs ( d ) .ge. r3 ) goto 300
            x( mapaxo( 3 ) ) = i

            do 200 j = 1 , nslow

              d = j - xrs( mapaxo( 2 ) )
              if ( d .gt.  g2/2. ) d = d - g2
              if ( d .lt. -g2/2. ) d = d + g2
              if ( abs ( d ) .ge. r2 ) goto 200
              x( mapaxo( 2 ) ) = j

              do 100 k = 1 , nfast

                d = k - xrs( mapaxo( 1 ) )
                if ( d .gt.  g1/2. ) d = d - g1
                if ( d .lt. -g1/2. ) d = d + g1
                if ( abs ( d ) .ge. r1 ) goto 100
                x( mapaxo( 1 ) ) = k

                call vecdif ( x , xrs , u , 3 )
                u( 1 ) = mod ( u( 1 ) / mapgrd( 1 ) , 1.0 )
                u( 2 ) = mod ( u( 2 ) / mapgrd( 2 ) , 1.0 )
                u( 3 ) = mod ( u( 3 ) / mapgrd( 3 ) , 1.0 )
                if ( vlen ( u , cell , cosang ) .lt. rrset( irset ) )
     &          rhomap( k,j,i ) = lrset( irset )

 100          continue
 200        continue
 300      continue
 400    continue

        return
        end

*********************************************************************SDK
************************************************************************
*
*       Subroutine MAPSCAL
*
*       Function:
*       Scale map
*
*       Arguments:
*       Rhomap      (i/o)     map array
*       N           (i)       number of points in map
*       Rhoscl      (i)       map scale factor and shift term
*
*
*       History:
*       Written August 1991 sdk
************************************************************************
        subroutine mapscal ( rhomap , n , rhoscl )

        implicit none

*       Arguments
        integer          n
        real             rhomap( n )
        real             rhoscl( 2 )

*       Local variables
        integer         i
*-----------------------------------------------------------------------
*
*       Apply scale
*
        do 100 i = 1 , n
          rhomap( i ) = rhoscl( 1 ) * rhomap( i ) + rhoscl( 2 )
 100    continue

        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine MAPTOP
*
*       Function:
*       Position map file before first density record
*
*       Comment:
*       The map file must be opened (by call to RDHDR) prior to call
*       of this routine
*
*       Arguments :
*       What    (i)     map format (PROTEIN or CCP4)
*       Iunit   (i)     logical unit number
*       Topsec  (i)     first section in map
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - error reading PROTEIN type 0 header
*                       errcod = 2 - error reading PROTEIN type 1 header
*                       errcod = 3 - error reading PROTEIN type 2 header
*
*
*       History:
*       Written December 5 S. Knight
************************************************************************

        subroutine maptop ( what , iunit , topsec , errcod )

        implicit none

*       Arguments
        character*(*)   what
        integer         iunit
        integer         topsec
        integer         errcod


*       Local variables
        character*80    catlin

*       External function
        integer         slnblk
*-----------------------------------------------------------------------
        errcod = 0
        if ( what .eq. 'CCP4' ) then

          call mposn ( iunit , topsec )

        else if ( what .eq. 'PROTEIN' ) then

*       Rewind file and read the three header records
          rewind ( iunit , iostat = errcod )
          read ( iunit , iostat = errcod )
          read ( iunit , iostat = errcod )
          read ( iunit , iostat = errcod )

        else

*       Illegal map format
          catlin = 'Illegal map format > ' // what( :slnblk ( what ) ) 
          call errmsg ( catlin , 1 )

        endif


        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine MFLSTAT
*
*       Function:
*       Get density statistics from WHAT type file with logical
*       name MAPNAM on unit IUNIT.
*
*       Arguments:
*       What      (i)   maptype (CCP4 or PROTEIN)
*       Mapnam    (i)   file name
*       Iunit     (i)   logical unit for map file
*       Amin      (o)   minimum map value
*       Amax      (o)   maximum map value
*       Ave       (o)   average map value
*       Sigma     (o)   rms map value
*       Errcod    (o)   error code
*                       errcod = 0 - no error
*                       errcod = 1 - MRDHDS error
*                       errcod = 1 - error opening or reading PROTEIN file
*                       errcod = 2 - map too large
*                       errcod = 3 - wrong number of sections
*                       errcod = 4 - unrecognized map format
*                       errcod = 5 - file does not exist
*
*       History:
*       November 6 1991 - sdk
*       Changed error code from logical to integer.
*       Written March 25 1988 S. Knight
************************************************************************
        subroutine mflstat (what,mapnam,iunit,amin,amax,ave,sigma,
     &  errcod)

        implicit none

        include         'rspsdim.inc'

*       Arguments
        character*(*)   what
        character*(*)   mapnam
        integer         iunit
        real            amin , amax , ave ,sigma
        integer         errcod

*       Local variables
        integer         i
        integer         nfast , nslow , nsec , npps
        integer         nw1 , nu1 , nu2 , nv1 ,nv2
        integer         lspgrp , lmode
        integer         mxyz(3),iuvw(3),grid(3),limit(6)
        integer         ifail
        real            cell(6),rho(mxisct)
        real            sum , ssq , varians
        character*80    title , catlin

*       External functions
        integer         slnblk
        logical         ccpexs
*-----------------------------------------------------------------------
        errcod = 0

        if (what.eq.'CCP4' ) then

          if ( .not. ccpexs ( mapnam ) ) then
            catlin = 'File does not exist > ' // 
     &      mapnam( :slnblk ( mapnam ) ) 
            call errmsg ( catlin , 0 )
            errcod = 5
            return
          endif

          ifail = 1
          call mrdhds ( iunit,mapnam,title,nsec,iuvw,mxyz,nw1,nu1,nu2,
     &    nv1,nv2,cell,lspgrp,lmode,amin,amax,ave,sigma,ifail,0 )
          call mrclos ( iunit )
          if ( ifail .ne. 1 ) then
            call errmsg ( 'Error reading CCP4 map header' , 0 )
            errcod = 1
          endif

        else if (what.eq.'PROTEIN' ) then

          if ( .not. ccpexs ( mapnam ) ) then
            catlin = 'File does not exist > ' // 
     &      mapnam( :slnblk ( mapnam ) ) 
            call errmsg ( catlin , 0 )
            errcod = 5
            return
          endif

          call rdhdr ( what,iunit,mapnam,nsec,mxyz,iuvw,
     &    grid,limit,cell,title,errcod )
          if ( errcod .ne. 0 ) then
            call errmsg ( 'Error reading PROTEIN map header' , 0 )
            errcod = 2
            return
          endif
          nfast=mxyz(iuvw(1))
          nslow=mxyz(iuvw(2))
          npps=nfast*nslow
          if (npps.gt.mxisct) then
            call errmsg ( 'Map sections too large' , 0 )
            errcod = 3
            return
          endif
          do i=1,nsec
            call rdsec ( what,iunit,rho,npps,amax,amin,sum,ssq )
            if (npps.lt.0.and.i.ne.nsec) then
              call errmsg ( 'Wrong number of sections in map' , 0 )
              errcod = 4
              return
            endif
          end do
          ave=sum/(npps*nsec)
          sigma=0.0
          varians=ssq/(npps*nsec)-ave**2
          if (varians.gt.0)sigma=sqrt(varians)
          close(iunit)

        else

          catlin = 'Illegal map format > ' //
     &    what( :slnblk ( what ) ) 
          call errmsg ( catlin , 1 )
          errcod = 5

        endif

        return
        end
*********************************************************************SDK
************************************************************************
*       Subroutines getr0,getr1,getr2,getr5 to read in PROTEIN records
*       of type 0,1,2 and 5 (taken from amapin)

      subroutine getr0( lun, title, ierror)

*
*   Ps300 frodo version 6.0
*
*   Subroutine getr0 is used to read in the PROTEIN style file type 0 record
*
*   Source from ps300 frodo version 5.5  (jwp & mas  01-jan-1984)
*
      integer    iblid       ,nwr         ,ift         ,title(20)
      integer    lun         ,ierror      ,i
      integer*2  ip          ,nb
*
*   Blank title
*
      do 100 i  = 1,20
      title(i)    = 0
  100 continue
*
*   Read the type zero record
*
      read (lun,err=900) iblid,ip,nb,nwr,
     &    (title(i),i=1,4),ift,(title(i),i=6,7)
      if (iblid.ne.0)    goto 910
      if (ift.ne.3)      goto 920
      return
*
*   Errors
*
  900 ierror        = 1
      call errmsg ( 'GETR0 > Error reading R0 record' , 0 )
      return
*
  910 ierror        = 2
      call errmsg ( 'GETR0 > Illegal record R0 block-id' , 0 )
      write(6,1100) iblid
      return
*
  920 ierror        = 3
      call errmsg ( 'GETR0 > Illegal record R0 file type' , 0 )
      write(6,1200) ift
      return
*
 1100 format (1x,'GETR0 > Record R0 block-id  ',i5,' (should be 0)' )
 1200 format (1x,'GETR0 > Record R0 file type = ',i5,' (should be 3)' )
      end
************************************************************************
************************************************************************
      subroutine getr1(lun, cell, nsym, ierror)
*
*   Ps300 frodo version 6.0
*
*   Subroutine getr1 is used to read in the PROTEIN style file type 1 record
*
*   Source from ps300 frodo version 5.5  (jwp & mas  01-jan-1984)
*
      real      cell(6)
      integer*2 ip          ,nb
      integer   nsym        ,iblid       ,nwr         ,lun
      integer   ierror      ,i
*
*   Read the type 1 record
*
      read (lun,err=900) iblid,ip,nb,nwr,cell,nsym
      if (iblid.ne.1) goto 910
*
*  Convert cos(angle) to angle
*
      do 100 i  = 4,6
      if (abs(cell(i)).gt.1.0) go to 920
      cell(i) = (180./3.141593)*acos(cell(i))
  100 continue
      return
*
*   Errors
*
  900 ierror       = 1
      call errmsg ( 'GETR1 > Error reading R1 record' , 0 )
      return
*
  910 ierror       = 2
      call errmsg ( 'GETR1 > Illegal record R1 block-id' , 0 )
      write(6,1100) iblid
      return
*
  920 ierror       = 4
      call errmsg ( 'GETR1 > Illegal angle coseine(s)' , 0 )
      write (6,1200)  (cell(i),i=4,6)
      return
*
 1100 format(1x,'GETR1 > Record R1 block-id = ',i5,' (should be 1) ' )
 1200 format(1x,'GETR1 > Angle cosine(s) out of range:', 3f15.6)
      end
************************************************************************
************************************************************************
      subroutine getr2(lun, iperm, igrid, mifd, title, ierror)
*
*   Ps300 frodo version 6.0
*
*   Subroutine getr2 is used to read in the PROTEIN style file type 2 record
*
*   Source from ps300 frodo version 5.5  (jwp & mas  01-jan-1984)
*       Modified to also read comment field october 1987 sdk
*
      integer   i
      integer   iperm(3)    ,igrid(3)    ,mifd(9)     ,iblid
      integer   nwr         ,ipatt       ,lun         ,ierror
      integer   title(15)
      integer*2 ip       ,nb

*   Blank title
*
      do 100 i  = 1,15
      title(i)    = 0
  100 continue
*
*   Read the type 2 record
*
      read (lun,err=900) iblid,ip,nb,nwr,ipatt,iperm,igrid,mifd,title
      if (iblid.ne.2) go to 910
      if (ipatt.ne.1) go to 920
      return
*
*   Errors
*
  900 ierror       = 1
      call errmsg ( 'GETR2 > Error reading R2 record' , 0 )
      return
*
  910 ierror       = 2
      call errmsg ( 'GETR2 > Illegal record R2 block-id' , 0 )
      write (6,1100) iblid
      return
*
  920 ierror       = 5
      call errmsg ( 'GETR2 > Illegal IPATT type' , 0 )
      write (6,1200)  ipatt
      return
*
 1100 format (1x,'GETR2 > Record R2 block-id = ',i5,' (should be 2)' )
 1200 format (1x,'GETR2 > Map IPATT type = ',i5,
     &' (should be 1 for Patterson)' )
      end
************************************************************************
************************************************************************
      subroutine getr5 (lun,mifd,nsize,rho,nrho,ibp0,ierror)
*
*   Ps300 frodo version 6.0
*
*   Subroutine getr5 is used to read in the PROTEIN style file type 5 records
*   Source from ps300 frodo version 5.5  (jwp & mas  01-jan-1984)
*

      integer   maxbuf     ,nsize
      parameter (maxbuf=10000)
      real      tmin        ,tmax        ,tmid        ,trat
      real      r
      real      rho(nsize)  ,buf(maxbuf)
      integer*2 ip          ,nb          ,i2buf(2*maxbuf)
      integer   i4buf(1000) ,blid        ,nwr         ,nlay
      integer   line        ,lnth        ,mifd(9)     ,nrho
      integer   lun         ,ierror      ,nlines      ,lenlin
      integer   ll2         ,nwline      ,irho        ,iline
      integer   ibp0        ,ib          ,ios         ,ibpden
      integer   il          ,ibp
      equivalence (buf(1),i4buf(1),i2buf(1))
*
      ierror    = 0
      nlines    = mifd(2) - mifd(1) + 1
      lenlin    = mifd(5) - mifd(4) + 1
      ll2       = (lenlin+1)/2
      nwline    = ll2 + 3
      irho      = 0
      iline     = 0
*
*   Loop back to here if several blocks for the layer
*
  100 if (ibp0.le.0) then
      read(lun,err=900,iostat=ios)blid,ip,nb,(buf(ib),ib=1,nb)
      if (blid.ne.5) go to 910
      if (nb.lt.4 .or. nb.gt.maxbuf) go to 920
      ibp0 = 0
      end if
*
      if (iline.le.0) then
      nwr       = i4buf(ibp0+1)
      nlay      = i4buf(ibp0+2)
      line      = i4buf(ibp0+3)
      lnth      = i4buf(ibp0+4)
      ibp0      = ibp0 + 4
      iline     = 1
      if (nwr.ne.3) goto 930
      if (line.ne.nlines) goto 940
      if (lnth.ne.lenlin)go to 950
      end if
*
      ibp       = ibp0
  110 if (iline.le.nlines .and. ibp+nwline.le.nb) then
      nwr       = i4buf(ibp+1)
      tmin      = buf(ibp+2)
      tmax      = buf(ibp+3)
      if (nwr.ne.nwline-1) go to 960
      tmid      = 0.5*(tmin + tmax)
      trat      = (tmax-tmin)/65536.
      ibpden    =  2*(ibp + 3)
*
      do 120 il  = 1,lnth
      irho      = irho+1
      ibpden    = ibpden+1
      r         = tmid + trat*i2buf(ibpden)
      rho(irho) = r
  120 continue

      ibp       = ibp + (nwline)
      iline     = iline + 1
      go to 110
      end if
*
      ibp0      = ibp
      if (ibp0.ge.nb) ibp0 = 0
      if (iline.le.nlines) go to 100
      nrho      = nrho + irho
      return
*
*   Errors
*
  900 ierror = 1
      call errmsg ( 'GETR5 > Error reading R5 record' , 0 )
      write (6,1000) ios
      return
*
  910 ierror = 2
      call errmsg ( 'GETR5 > Illegal record R5 block-id' , 0 )
      write (6,1100) blid
      return
*
  920 ierror = 7
      call errmsg ( 'GETR5 > Illegal value in R5 record', 0 )
      write (6,1200) nb, maxbuf
      return
*
  930 ierror = 8
      call errmsg ( 'GETR5 > Illegal value in R5 record', 0 )
      write (6,1300) nwr
      return
*
  940 ierror = 9
      call errmsg ( 'GETR5 > Illegal value in R5 record', 0 )
      write (6,1400) line, nlines
      return
*
  950 ierror = 10
      call errmsg ( 'GETR5 > Illegal value in R5 record', 0 )
      write (6,1500) lnth,lenlin
      return
*
  960 ierror   = 11
      call errmsg ( 'GETR5 > Illegal value in R5 record', 0 )
      write (6,1600) nwr, nwline-1
      return
*
 1000 format (1x,'GETR5 > Error reading R5 record (IOSTAT=',i5,' )' )
 1100 format (1x,'GETR5 > Record R5 block-id = ',i5,' (should be 5)' )
 1200 format (1x,'GETR5 > Value of NB in record R5 =',i8,' <4 or >',i5)
 1300 format (1x,'GETR5 > Value of NWR in record R5 = ',i12,
     &'(should be 3)' )
 1400 format (1x,'GETR5 > Value of LINE in record R5 =',i12,' not', i5)
 1500 format (1x,'GETR5 > Value of LENGTH in record R5 =',i12,' not',
     & i5)
 1600 format (1x,'GETR5 > Value of NWR in record R5 = ',i12,' not',i5)
      end
************************************************************************
************************************************************************
*       RSPS scanning and scoring package
*       Contains modules sscan, vlist, hvscor, cvscor, specpos , 
*       vlspecpos , xtran , onedscan
*
*       Sscan     scan search cell and form scores from the predicted
*                 vector positions. Write out scores as ccp4 map.
*
*       Vlist     write out vectors and densities for selected 
*                 positions
*
*       Hvscor    compute all Harker vectors for a position
*                 Return score and number of low vectors
*
*       Cvscor    compute all crossvectors between two positions
*                 Return score and number of low vectors
*
*       Specpos   Scores for special positions
*
*       Vlspecpos Special Vlist for special positions
*
*       onedXtran     One dimensional scan for N positions, with report
*
*       Onedscan  One dimensional scan for one position, silent
*
************************************************************************
************************************************************************
*
*       Subroutine SSCAN
*
*
*       Function:
*       Scan search cell, compute scores and write them out as ccp4 map.
*       Limits and sectioning for search cell defined in common/gridut/
*       scrlim & scraxo fields (see rspsgrd.inc).
*
*       The transformations that map a point on the search grid g2 to 
*       a point on the patterson grid g1 are in common /symm/ (see
*       rspssym.inc).
*
*
*       Arguments :
*       Iunit   (i)     logical unit number for output score map
*       Patmap  (i)     patterson map densities
*       Patfas  (i)     number of points in fast direction of p'son map
*       Patslo  (i)     number of points in slow direction of p'son map
*       Patsec  (i)     number of sections in p'son map
*       Patsig  (i)     standard deviation of patterson map
*       Scomin  (o)     scoremap minimum value
*       Scomax  (o)     max
*       Scoave  (o)     average
*       Scosig  (o)     sigma
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - SPECPOS error
*
*
*       History:
*       March 3 1994 - sdk
*       Improved tests for molecule bumps
*       February 14 1994 - sdk
*       Modified for new MODEs
*       August 17 1993 - sdk
*       Added scoremap stats arguments
*       August 1991 - sdk
*       New arguments: integer*2 array slice changed to real patmap
*       Written March 1988 S. Knight
************************************************************************
        subroutine sscan ( iunit , patmap , patfas , patslo , patsec ,
     &  patsig , scomin , scomax , scoave , scosig , errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'
        include         'rspspos.inc'
        include         'rspsfil.inc'

*       Arguments
        integer         iunit
        integer         patfas , patslo , patsec
        real            patmap( patfas , patslo , patsec )
        real            patsig
        integer         errcod

*       Local variables
        logical         mfun
        logical         molecules
        logical         spcx
        integer         i , j , k , m , nt
        integer         ij , ik
        integer         ifasti , ifastf
        integer         islowi , islowf
        integer         iseci  , isecf
        integer         nfast , nslow , nsec2
        integer         low
        integer         pgrd1 , pgrd2 , pgrd3
        integer         npkuse
        integer         imult(mxvec)
        integer         ifrcpk
        real            dist
        real            score , rho
        real            fminsc , fmaxsc , avesc , sigsc
        real            scomin , scomax , scoave , scosig
        real            sumscore , susqscore , sumsc , sumsqsc
        real            scorut(mxfast,mxslow)
        real            peak(mxvec)
        real            x(3),xfrac(3),u(3),x1(3),x2(3)
        real            tol
        character*8     date

*       Data
        data            tol/2.5/

*       External functions 
        logical         clash1 , clash2
        integer         sfnblk , slnblk
        integer         lgseql
        real            getrho
        real            vlen
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Make sure flag for special positions is disabled except in 
*       SINGLE vectorsets
        if ( index ( vecset , 'SINGLE' ) .ne. 1 ) special = .false.

*       Check flags
        if ( scrtyp.lt.1.or.scrtyp.gt.3 ) scrtyp = 1

*       Set vectorset MOLECULES flag
        molecules = .false.
        if ( index ( vecset , 'MOLECULES' ) .ne. 0 ) molecules = .true.

*       Set number of peaks to use = all if not min function
        mfun = minfun
        if ( nuse .ge. ntrans ) mfun=.false.
        if ( mfun ) then
          npkuse = nuse
        else
          npkuse = ntrans
        endif

*       Start and end points for scan
        ifasti = nint ( scrlim( 2*scraxo(1)-1 ) * scrgrd( scraxo(1) ) )
        ifastf = nint ( scrlim( 2*scraxo(1)   ) * scrgrd( scraxo(1) ) )
        islowi = nint ( scrlim( 2*scraxo(2)-1 ) * scrgrd( scraxo(2) ) )
        islowf = nint ( scrlim( 2*scraxo(2)   ) * scrgrd( scraxo(2) ) )
        iseci  = nint ( scrlim( 2*scraxo(3)-1 ) * scrgrd( scraxo(3) ) )
        isecf  = nint ( scrlim( 2*scraxo(3)   ) * scrgrd( scraxo(3) ) )
        nfast  = ifastf - ifasti + 1
        nslow  = islowf - islowi + 1
        nsec2  = isecf  - iseci  + 1
        if (nfast.gt.mxfast.or.nslow.gt.mxslow) then
        write(6,9000)nfast,nslow,mxfast,mxslow
        stop' REQUESTED OUTPUT MAP TOO BIG FOR ARRAY SCORUT'
        endif

*       Set peak weights
        do 10 i = 1 , ntrans
          imult(i) = 1
          if ( usemult ) imult(i) = mult(i)
 10     continue

*       Form title
        call ccpdat ( date )
        title = ' '
        title = ' RSPS ' // vecset( :slnblk ( vecset ) ) // ' SCAN ' //
     &  date( :slnblk ( date ) ) // ' > ' // 
     &  titut( sfnblk ( titut ):slnblk ( titut ) )

*       Open output file and set up header record
        call mwrhdl ( iunit,scrfil,title,nsec2,scraxo,scrgrd,
     &  iseci,ifasti,ifastf,islowi,islowf,cell,ispgrp,2 )
*       Copy symmetry data from library file to map
        close ( 3 )
        call msyput ( 3,ispgrp,iunit )
        close ( 3 )


*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Init overall statistics variables
        scomin      =  1.e+38
        scomax      =  0.
        sumscore    =  0.
        susqscore   =  0.

*       Set grid along fast, slow, section
        pgrd1 = patgrd( pataxo( 1 ) )
        pgrd2 = patgrd( pataxo( 2 ) )
        pgrd3 = patgrd( pataxo( 3 ) )

        do 300 i  =  iseci,isecf

*       Init statistics variables for this section
        fminsc  = 1.e+38
        fmaxsc  = 0.
        sumsc   = 0.
        sumsqsc = 0.

*       Set test point section coordinate
        x( scraxo( 3 ) ) = i

        do 200 j  =  islowi,islowf

*       Scoremap pointer
        ij = j - islowi + 1
*       Set test point slow coordinate
        x( scraxo( 2 ) ) = j

        do 100 k  =  ifasti,ifastf

*       Scoremap pointer
          ik = k - ifasti + 1
*       Set test point fast coordinate
          x( scraxo( 1 ) ) = k

*       Fractional coordinates of test point
          do m = 1 , 3
            xfrac( m ) = x( m ) / scrgrd( m )
          end do

*       Init score and low-peak counter, special position flag
          scorut( ik , ij ) = 0.
          low  = 0
          spcx = .false.

*       Init forced-vector counter
          ifrcpk = 0

          do 60 nt = 1 , ntrans

*       Compute vector. 
            call vmtply ( r_vec(1,1,nt) , xfrac , u )
            do m = 1 , 3
              u( m ) = u( m ) + t_vec( m,nt )
            end do 

*       Reduce to map volume
            call reduce ( u )

            if ( vecset .ne. 'SUPERPOSE' ) then
*       Get length of vector
              dist = vlen ( u , cell , cosang )

*       Take care of bumps and special positions
*       (In MOLECULES vectorsets some bumps and special positions
*       will not be detected by this test; see below)
              if ( dist .lt. bump ) then
                if ( special .and. dist .lt. 2. ) then
*       Special position
                  spcx = .true.
                  call specpos ( xfrac,pgrd1,pgrd2,pgrd3,score, 
     &            low,errcod )
                  if ( errcod .ne. 0 ) then
                    errcod = 1
                    return
                  endif
                  if ( low .gt. maxlow ) goto 100
                  goto 80
                else
*       Bump
                  fminsc = min ( fminsc , 0. )
                  goto 100
                endif
              endif
            endif

*       Get density for this vector.
            rho = getrho ( u , pektol , 
     &      patmap , patfas , patslo , patsec ,
     &      pgrd1 , pgrd2 , pgrd3 )

*       Count number of low peaks
            if ( rho .lt. reject ) low = low + imult( nt )
            if ( low .gt. maxlow ) then
              fminsc = min ( fminsc , 0. )
              goto 100
            endif

*       Store peak 
            peak( nt ) = rho 

*       If a specific vector has been requested check if it has bean hit
            if ( force ) ifrcpk = 
     &                   ifrcpk + lgseql ( u , frcpek , tol )

 60       continue

*       In MOLECULES vectorsets we must check separately for clashes 
*       between molecules at equivalent positions since not
*       all intermolecular vectors are represented in the set
*       of vector transformations used. The test is done after
*       collection of peaks to minimize the number of times
*       the test is done.
*       N.B. The current test will fail if a molecule at a special
*       position overlaps with a differently oriented molecule.
 80       if ( molecules ) then
            if ( vecset .eq. 'SINGLE MOLECULES' ) then
              if ( clash1 ( xfrac , bump ) ) then
                fminsc = min ( fminsc , 0. )
                goto 100
              endif
            else if ( vecset .eq. 'MORE MOLECULES' ) then
              do m = 1 , nxyz2
                call matcop ( xyz2( 1,m ) , x2 , 3 , 1 )
                if ( clash1 ( x2 , bump ) .or.
     &               clash2 ( xfrac , x2 , bump ) ) then
                  fminsc = min ( fminsc , 0. )
                  goto 100
                end if
              end do
            else if ( vecset .eq. 'TRANSLATE MOLECULES' ) then
              call vecadd ( xfrac , xyz2( 1,1 ) , x1 , 3 )
              call vecadd ( xfrac , xyz2( 1,2 ) , x2 , 3 )
              if ( clash1 ( x1 , bump ) .or.
     &             clash1 ( x2 , bump ) .or.
     &             clash2 ( x1 , x2 , bump ) ) then
                fminsc = min ( fminsc , 0. )
                goto 100
              end if
            endif
          endif

*       Did we hit forced peak?
          if ( force .and. ifrcpk .eq. 0 ) goto 100

*       Get score (already done for special positions)
          if ( .not. spcx ) call rsscor ( peak , imult , ntrans , 
     &    npkuse , patsig , scrtyp , mfun , usemult , score , errcod )

*       Sums for local statistics
          fminsc   =  min ( fminsc , score )
          fmaxsc   =  max ( fmaxsc , score )
          sumsc    =  sumsc   + score
          sumsqsc  =  sumsqsc + score ** 2

*       Store the score
          scorut( ik,ij ) = score


 100    continue
 200    continue
*       Sums for global statistics
        scomin     =  min ( scomin , fminsc )
        scomax     =  max ( scomax , fmaxsc )
        sumscore   =  sumscore  + sumsc
        susqscore  =  susqscore + sumsqsc

*       Statistics for this section
        if ( iprt .ge. 2 ) then
          avesc = sumsc/(nfast*nslow)
          sigsc = sqrt ( abs ( sumsqsc/(nfast*nslow) - avesc**2 ) )
          if ( sumsqsc .eq. 0. ) then
            fminsc = 0.
            fmaxsc = 0.
          endif
          write(6,2200)i,fminsc,fmaxsc,avesc,sigsc
        endif

*       Write out this section
        call mwrsec ( iunit,scorut,mxfast,mxslow,1,nfast,1,nslow )
 300    continue
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Global statistics
        scoave = sumscore/(nfast*nslow*nsec2)
        scosig = susqscore/(nfast*nslow*nsec2) - scoave**2
        scosig = sqrt( abs( scosig ) )
        if ( susqscore .eq. 0. ) then
          scomax = 0.
          scomin = 0.
        endif
        write(6,2220)scomin,scomax,scoave,scosig

        call mclose ( iunit,scomin,scomax,sumscore,susqscore )
        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Output formats
 2200   format(//10x,'Section number',i8,' completed',/,
     &  10x,'Minimum, maximum, average, sigma of score: ',4f12.2)
 2220   format(///10x,'Global statistics on scores: ',
     &  /,10x,'Minimum score    =',f12.4,
     &  /,10x,'Maximum score    =',f12.4,
     &  /,10x,'Average score    =',f12.4,
     &  /,10x,'Std Dev of score =',f12.4)


*       Error messages
 9000   format(/10x,'** ERROR - requested output map too big for array',
     &  ' scorut in s/r SSCAN **',
     &   /10x,'** Asked for ',2i8,
     &   /10x,'** Allocated ',2i8)
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine VLIST
*
*       Function:
*       Find and write out vectors and densities for heavy atom or
*       ncs axis position xyz.
*
*       Comment:
*       The transformations that map a point on the search grid g2 to 
*       a point on the patterson grid g1 are in common /symm/ (see
*       rspssym.inc).
*
*
*       Arguments :
*       Patmap  (i)     patterson map densities
*       Patfas  (i)     number of points in fast direction of p'son map
*       Patslo  (i)     number of points in slow direction of p'son map
*       Patsec  (i)     number of sections in p'son map
*       Pektot  (i)     number of stored Patterson peaks
*       Patsig  (i)     standard deviation of patterson map
*       Xyz     (i)     position (fractional)
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - error in s/r rsscor
*
*       History:
*       February 14 1994 - sdk
*       Modified for new MODEs
*       August 18 1992 - sdk
*       Header for translation vlist added.
*       November 8 1991 - sdk
*       Modified to operate on single position in fractional coordinates
*       August 1991 - sdk
*       Arguments modified
*       Written March 1988 sdk
************************************************************************
        subroutine vlist (patmap,patfas,patslo,patsec,pektot,patsig,
     &  xyz,errcod)

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'
        include         'rspspos.inc'

*       Arguments
        integer         patfas , patslo , patsec , pektot
        integer         errcod
        real            patmap( patfas , patslo , patsec )
        real            patsig
        real            xyz(3)

*       Local variables
        logical         mfun
        integer         i , j , m , nt
        integer         low
        integer         pgrd1 , pgrd2 , pgrd3
        integer         peknr(mxvec)
        integer         npkuse
        integer         imult(mxvec)
        integer         ihit
        real            pekdist , maxdist
        real            summlt
        real            sumhgt
        real            ssqpos , ssqhgt
        real            rmspos , rmshgt
        real            rmspsw , rmshtw
        real            mindx
        real            rho , score
        real            peak(mxvec)
        real            u(3)
        real            use(3,mxvec)
        real            dist
        real            height

*       Weight for rmsd's in matching index
        parameter       ( rmspsw = 0.8 )
        parameter       ( rmshtw = 0.1 )

*       External functions
        real            getrho
        real            vlen
        real            ppv
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Check there are vectors stored (in P1 there will be none)
        if ( ntrans .le. 0 ) then
           write(6,2000)
           return
        endif

*       Write header
        if ( vecset .eq. 'SINGLE ATOMS' ) then
          if ( .not. usencs ) write(6,2005)xyz
          if ( usencs ) write(6,2006)xyz
        endif
        if ( vecset .eq. 'MORE ATOMS' ) then
          write(6,2010)nxyz2,(xyz2(i,1),i=1,3)
          do i = 2 , nxyz2
            write(6,2012)i,(xyz2(j,i),j=1,3)
          end do
          write(6,2014)xyz
        endif
        if ( vecset .eq. 'TRANSLATE ATOMS' ) write(6,2020)xyz
        if ( vecset .eq. 'SINGLE MOLECULES' ) write(6,2030)xyz
        if ( vecset .eq. 'MORE MOLECULES' ) write(6,2040)xyz
        if ( vecset .eq. 'TRANSLATE MOLECULES' ) write(6,2050)xyz

*       Set score function flag
        if ( scrtyp .lt. 1 .or. scrtyp .gt. 3 ) scrtyp = 1

*       Initiate low-peak counter
        low     =   0

*       Set grid along fast, slow, section
        pgrd1 = patgrd( pataxo( 1 ) )
        pgrd2 = patgrd( pataxo( 2 ) )
        pgrd3 = patgrd( pataxo( 3 ) )

*       Set distance cutoff for peak search
        maxdist = 2.5 * max ( cell( 1 ) / patgrd( 1 ) , 
     &                        cell( 2 ) / patgrd( 2 ) ,
     &                        cell( 3 ) / patgrd( 3 ) )


*       Loop over transformations
        do 100 nt = 1 , ntrans

*       Set peak weights
          imult( nt ) = 1
          if ( usemult ) imult( nt ) = mult( nt )

*       Compute vector
          call vmtply ( r_vec(1,1,nt) , xyz , u )
          do m = 1 , 3
          u( m ) = u( m ) + t_vec( m,nt )
          end do

*       Reduce to map volume
          call reduce ( u )

          if ( index ( vecset , 'SINGLE' ) .ne. 0 ) then
*       Get length of vector
            dist = vlen ( u , cell , cosang )

*       Special positions get special treatment
            if ( dist .lt. 2 ) then
              call vlspecpos ( xyz , pektot , pgrd1 , pgrd2 , 
     &        pgrd3 , errcod )
              if ( errcod .ne. 0 ) errcod = 1
              return
            endif
          endif

*       Get density for this vector.
          rho = getrho ( u , pektol , 
     &    patmap , patfas , patslo , patsec ,
     &    pgrd1 , pgrd2 , pgrd3 )

*       Count number of low peaks
          if ( rho .lt. reject ) low = low + imult( nt )

*       Store vector and peak
          call matcop ( u , use( 1,nt ) , 3 , 1 )
          peak( nt ) = rho 

 100    continue

*       Write out vector, density, multiplicity, closest peak and 
*       distance, rmsd to peaks, matching index
        if ( pektot .ne. 0 ) then
          write(6,2080)
        else
          write(6,2090)
        endif
        ihit = 0
        ssqpos = 0.0
        ssqhgt = 0.0
        sumhgt = 0.0
        summlt = 0.0
        do 200 nt = 1 , ntrans
          summlt = summlt + mult( nt )
          call fndpek ( use( 1,nt ) , peknr( nt ) , pekdist , 
     &    maxdist )
          if ( peknr( nt )  .ne. 0 ) then
            write(6,2100) nt , ( use(i,nt),i=1,3 ) , peak(nt) , 
     &      mult(nt) , peknr( nt ) , pekdist
            ihit = ihit + 1
            ssqpos = ssqpos + pekdist**2
            sumhgt = sumhgt + ppv( peknr( nt ) ) / patsig
          else
            if ( pektot .eq. 0 ) then
              write(6,2150) nt , ( use(i,nt),i=1,3 ) , peak(nt) , 
     &        mult(nt)
            else
              write(6,2155) nt , ( use(i,nt),i=1,3 ) , peak(nt) , 
     &        mult(nt),maxdist
            endif
          endif
 200    continue

*       Rmsd peak positions and peak heights; matching index
        do 300 nt = 1 , ntrans
          if ( peknr( nt ) .ne. 0 ) then
            height = ppv( peknr( nt ) ) / patsig
          else
            height = 0.0
          endif
          ssqhgt = ssqhgt + ( height - (sumhgt/summlt)*mult( nt ) )**2
 300    continue
        rmspos = 999.
        rmshgt  = 999.
        if ( ihit .gt. 0 ) then
          rmspos = sqrt ( ssqpos / ihit )
        endif
        rmshgt  = sqrt ( ssqhgt  / ntrans )
        mindx = ( 1 + ihit ) / 
     &  ( ( 1 + rmspsw*rmspos )*( 1 + rmshtw*rmshgt )*( 1 + ntrans ) )

*       Set number of peaks to use = all if not min function
        mfun = minfun
        if  ( nuse .ge. ntrans ) mfun = .false.
        if ( mfun ) then
          npkuse = nuse
        else
          npkuse = ntrans
        endif

*       Get score
          call rsscor ( peak , imult , ntrans , npkuse , patsig , 
     &    scrtyp , mfun , usemult , score , errcod )
          if ( errcod .ne. 0 ) then
            errcod = 1
            return
          endif

*       Report
        if ( pektot .eq. 0 ) then
          write(6,2200)score,low
        else
          write(6,2210)score,low,rmspos,rmshgt,mindx
        endif


        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Output formats
 2000   format(///10x,'No vector relations stored')
 2005   format(///10x,'**********************************************',
     &  '******************************',/10x,' Harker vectors for a ',
     &  'heavy atom position at ',3f8.4,': ')
 2006   format(///10x,'**********************************************',
     &  '******************************',/10x,' Harker and pseudo-',
     &  'Harker vectors for a heavy atom position at ',3f8.4,': ')
 2010   format(///10x,'**********************************************',
     &  '******************************',/10x,' Cross vectors from ',
     &  i4,' fixed positions:  1',5x,3f8.4)
 2012   format(51x,i3,5x,3f8.4)
 2014   format(54x,'(+ symmetry related positions)',/10x,
     &  ' to an atom at',35x,3f8.4,/54x,
     &  '(+ symmetry related positions)',/)
 2020   format(///10x,'**********************************************',
     &  '******************************',/10x,' Vectors for rigid ',
     &  'fragment translated by ',3f8.4,': ')
 2030   format(///10x,'**********************************************',
     &  '******************************',/10x,' Structure invariant ' ,
     &  'vectors for NCS axes through ',3f8.4,': ')
 2040   format(///10x,'**********************************************',
     &  '******************************',/10x,' Structure invariant ' ,
     &  'cross vectors to a molecule with NCS axes through ',
     &  /10x,3f8.4,': ')
 2050   format(///10x,'**********************************************',
     &  '******************************',/10x,' Structure invariant ' ,
     &  'vectors for two NCS molecules translated by ',
     &  /10x,3f8.4,': ')
 2080   format(//10x,' Vec       U       V       ',
     &  'W        Rho     Multiplicity     Peak   Distance',/10x,
     &  ' ---    ------  ------  ------    -------   ------------',
     &  '     ----   --------' )
 2090   format(//10x,' Vec       U       V       ',
     &  'W        Rho     Multiplicity',/10x,
     &  ' ---    ------  ------  ------    -------   ------------')
 2100   format(10x,i3,3x,3f8.4,f10.2,7x,i4,8x,i5,7x,f4.2)
 2150   format(10x,i3,3x,3f8.4,f10.2,7x,i4)
 2155   format(10x,i3,3x,3f8.4,f10.2,7x,i4,10x,'none    > ',f4.2)
 2200   format(/10x,' Score = ',f8.2,' with ',i5,
     &  ' low peaks',/10x,'************************',
     &  '****************************************************')
 2210   format(/10x,      ' Score               = ',f8.2,' with ',i5,
     &  ' low peaks',/10x,' Rmsd peak positions = ',f8.4, 
     &               /10x,' Rmsd peak heights   = ',f8.4,
     &               /10x,' Matching index      = ',f8.4,
     &               /10x,'******************************',
     &  '**********************************************')

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Real function PPV ( N )
*
*       Function:
*       Return value of peak N in peak list
************************************************************************
        real function ppv ( n )
        integer n
        include         'rspsdim.inc'
        include         'rspsmap.inc'

        ppv = ppval( n )
        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine CVSCOR
*
*
*       Function:
*       Compute all crossvectors between positions x1 and x2
*       Return score, number of vectors used and number of low vectors
*
*
*       Arguments :
*       Patmap  (i)     patterson map densities
*       Patfas  (i)     number of points in fast direction of p'son map
*       Patslo  (i)     number of points in slow direction of p'son map
*       Patsec  (i)     number of sections in p'son map
*       Patsig  (i)     standard deviation of patterson map
*       Rot     (i)     rotations
*       Trans   (i)     translations
*       Nsym    (i)     number of symmetry operations
*       X1,x2   (i)     positions to be used for calculating crossvectors
*       Score   (o)     score
*       Nvec    (o)     number of vectors used
*       Low     (o)     number of vectors .lt. reject
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - SYMAPP error
*
*        
*
*       History:
*       August 1991 - sdk
*       Modified arguments
*       November 1988 - sdk
*       Modified to apply local symmetry (if present) to both input
*       positions. Cvscor (= old cvcalc) now uses subroutines geneqp
*       and cvcalc to compute vectors.
*       Min n function included
*       Written March 30 1988 S. Knight
************************************************************************
        subroutine cvscor (patmap,patfas,patslo,patsec,patsig,rot,trans,
     &  nsym,x1,x2,score,nvec,low,ihit,ssqpos,ssqhgt,errcod)

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'
        include         'rspsgrd.inc'

*       Arguments
        integer         patfas , patslo , patsec , nsym , nvec
        integer         low , ihit
        integer         errcod
        real            patmap( patfas , patslo , patsec )
        real            patsig , score
        real            rot(3,3,nsym),trans(3,nsym)
        real            x1(3),x2(3)
        real            ssqpos , ssqhgt

*       Local variables
        logical         mfun
        integer         i
        integer         pgrd1 , pgrd2 , pgrd3
        integer         imult(mxvec)
        integer         nxout1 , nxout2
        integer         npkuse
        integer         peknr(mxvec)
        real            rho
        real            xout1(3,mxsym) , xout2(3,mxsym)
        real            vec(4,mxvec) , peak(mxvec)
        real            u(3)
        real            pekdist , maxdist
        real            summlt
        real            sumhgt
        real            height


*       External functions
        real            getrho
        real            ppv
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Set score function flag
        if ( scrtyp .lt. 1 .or. scrtyp .gt. 3 ) scrtyp = 1

*       Initialize counters etc
        low     =   0
        ihit    =   0
        nxout1  =   mxsym
        nxout2  =   mxsym
        nvec    =   mxvec
        score   =   0.0
        ssqpos  =   0.0
        ssqhgt  =   0.0
        sumhgt  =   0.0
        summlt  =   0.0

*       Patterson grid along fast, slow, section axis
        pgrd1 = patgrd( pataxo( 1 ) )
        pgrd2 = patgrd( pataxo( 2 ) )
        pgrd3 = patgrd( pataxo( 3 ) )


*       Get all positions symmetry related to x1 and x2
        call symapp ( nsym,rot,trans,1,x1,nxout1,xout1,errcod )
        if ( errcod .ne. 0 ) then
          errcod = 1
          return
        endif
        call symapp ( nsym,rot,trans,1,x2,nxout2,xout2,errcod )
        if ( errcod .ne. 0 ) then
          errcod = 1
          return
        endif

*       Get unique vectors between positions in xout1 and xout2
        call cvcalc ( xout1,xout2,nxout1,nxout2,vec,nvec )


*       Set number of peaks to use = all if not min function
        mfun = minfun
        if  ( nuse .ge. nvec ) mfun = .false.
        if ( mfun ) then
          npkuse = nuse
        else
          npkuse = nvec
        endif

*       Set cutoff distance for peak searching
        maxdist = 2.5 * max ( cell( 1 ) / patgrd( 1 ) , 
     &                        cell( 2 ) / patgrd( 2 ) ,
     &                        cell( 3 ) / patgrd( 3 ) )



*       Loop over all vectors found
        do 100 i = 1 , nvec
          imult( i ) = 1
          if ( usemult ) imult( i ) = vec( 4,i )
          summlt = summlt + vec( 4,i )

*       Put copy in u
          call matcop ( vec( 1,i ) , u , 3 , 1 )

*       Reduce to map volume
          call reduce ( u )

*       Get density
          rho = getrho ( u , pektol , 
     &    patmap , patfas , patslo , patsec ,
     &    pgrd1 , pgrd2 , pgrd3 )

*       Count number of low peaks
          if ( rho .lt. reject ) low = low + imult( i )

*       Store peak
          peak( i ) = rho

*       Look for vector on peak list
          call fndpek ( u , peknr( i ) , pekdist , maxdist )
          if ( peknr( i )  .ne. 0 ) then
            ihit = ihit + 1
            ssqpos = ssqpos + pekdist**2
            sumhgt = sumhgt + ppv( peknr( i ) ) / patsig
          endif
 100    continue

*       SSQ of observed - expected peak height
        do 200 i = 1 , nvec
          if ( peknr( i ) .ne. 0 ) then
            height = ppv( peknr( i ) ) / patsig
          else
            height = 0.0
          endif
          ssqhgt = ssqhgt + ( height - (sumhgt/summlt)*vec( 4,i ) )**2
 200    continue

*       Get score
        call rsscor ( peak , imult , nvec , npkuse , patsig , 
     &  scrtyp , mfun , usemult , score , errcod )


        return
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine HVSCOR
*
*       Function:
*       Compute all harker vectors for position x1
*       Return score, number of vectors used and and number of low 
*       vectors
*
*
*       Arguments :
*       Patmap  (i)     patterson map densities
*       Patfas  (i)     number of points in fast direction of p'son map
*       Patslo  (i)     number of points in slow direction of p'son map
*       Patsec  (i)     number of sections in p'son map
*       Patsig  (i)     standard deviation of patterson map
*       Rot     (i)     rotations
*       Trans   (i)     translations
*       Nsym    (i)     number of symmetry operations
*       X1      (i)     position to be used for calculating harker 
*       Score   (o)     score
*       Nvec    (o)     number of vectors used
*       Low     (o)     number of vectors .lt. reject
*       Ihit    (o)     number of vectors found on peak list
*       SSQPos  (o)     SSQ (observed - predicted) peak position
*       SSQHgt  (o)     SSQ (observed - predicted) peak height
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - SYMAPP error
*
*
*       History:
*       March 1996 - sdk
*       Three arguments added to allow calculation of rmsd's of peak
*       positions and heights: Ihit, SSQPos, SSQHgt
*       August 1991 - SDK
*       Modified arguments
*       Written March 1989 S. Knight
************************************************************************
        subroutine hvscor (patmap,patfas,patslo,patsec,patsig,rot,trans,
     &  nsym,x1,score,nvec,low,ihit,ssqpos,ssqhgt,errcod)

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'
        include         'rspsgrd.inc'

*       Arguments
        integer         patfas , patslo , patsec 
        integer         nsym , nvec , low , ihit
        integer         errcod
        real            patmap( patfas , patslo , patsec )
        real            patsig , score
        real            rot(3,3,nsym),trans(3,nsym)
        real            x1(3)
        real            ssqpos , ssqhgt

*       Local variables
        logical         mfun
        integer         i 
        integer         imult(mxvec)
        integer         nxout1
        integer         pgrd1 , pgrd2 , pgrd3
        integer         npkuse
        integer         peknr(mxvec)
        real            rho
        real            xout1(3,mxsym)
        real            vec(4,mxvec) , peak(mxvec)
        real            u(3)
        real            pekdist , maxdist
        real            summlt
        real            sumhgt
        real            height

*       External functions
        real            getrho
        real            ppv
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Check scoring function flag
        if ( scrtyp .lt. 1 .or. scrtyp .gt. 3 ) scrtyp = 1

*       Initialize counters etc
        low     =   0
        ihit    =   0
        nxout1  =   mxsym
        nvec    =   mxvec
        score   =   0.0
        ssqpos  =   0.0
        ssqhgt  =   0.0
        sumhgt  =   0.0
        summlt  =   0.0

*       Set grid along fast, slow, section
        pgrd1 = patgrd( pataxo( 1 ) )
        pgrd2 = patgrd( pataxo( 2 ) )
        pgrd3 = patgrd( pataxo( 3 ) )


*       Get all positions symmetry related to x1
        call symapp ( nsym,rot,trans,1,x1,nxout1,xout1,errcod )
        if ( errcod .ne. 0 ) then
          errcod = 1
          return
        endif

*       Get unique harker vectors
        if ( nxout1 .le. 1 ) then
           nvec = 0
           return
        endif
        call hvcalc ( xout1,nxout1,vec,nvec )
        if ( nvec .eq. 0 ) return

*       Set number of peaks to use = all if not min function
        mfun = minfun
        if  ( nuse .ge. nvec ) mfun = .false.
        if ( mfun ) then
          npkuse = nuse
        else
          npkuse = nvec
        endif

*       Set cutoff distance for peak searching
        maxdist = 2.5 * max ( cell( 1 ) / patgrd( 1 ) , 
     &                        cell( 2 ) / patgrd( 2 ) ,
     &                        cell( 3 ) / patgrd( 3 ) )


*       Loop over all vectors found
        do 100 i = 1 , nvec
          imult( i ) = 1
          if ( usemult ) imult( i ) = vec( 4,i )
          summlt = summlt + vec( 4,i )

*       Put copy in u
          call matcop ( vec( 1,i ) , u , 3 , 1 )

*       Reduce to map volume
          call reduce ( u )

*       Get density for this vector
          rho = getrho ( u , pektol , 
     &    patmap , patfas , patslo , patsec ,
     &    pgrd1 , pgrd2 , pgrd3 )

*       Count number of low peaks
          if ( rho .lt. reject ) low = low + imult( i )

*       Store peak
          peak( i ) = rho

*       Look for vector on peak list
          call fndpek ( u , peknr( i ) , pekdist , maxdist )
          if ( peknr( i )  .ne. 0 ) then
            ihit = ihit + 1
            ssqpos = ssqpos + pekdist**2
            sumhgt = sumhgt + ppv( peknr( i ) ) / patsig
          endif
 100    continue


*       SSQ of observed - expected peak height
        do 200 i = 1 , nvec
          if ( peknr( i ) .ne. 0 ) then
            height = ppv( peknr( i ) ) / patsig
          else
            height = 0.0
          endif
          ssqhgt = ssqhgt + ( height - (sumhgt/summlt)*vec( 4,i ) )**2
 200    continue

*       Get score
        call rsscor ( peak , imult , nvec , npkuse , patsig , 
     &  scrtyp , mfun , usemult , score , errcod )


        return
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine SPECPOS
*
*       Function:
*       Compute all (pseudo and/or true) Harker vectors for position xin
*       Return score and number of low vectors
*
*
*       Arguments :
*       Xin     (i)     position to be used for calculating vectors
*       Pgrd1   (i)     Patterson grid along fast axis
*       Pgrd2   (i)     Patterson grid along slow axis
*       Pgrd3   (i)     Patterson grid along section axis
*       Score   (o)     score
*       Low     (o)     number of vectors .lt. reject
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - too many vectors generated
*                       errcod = 2 - RSSCOR error
*
*       History:
*       Written December 9 1993 S. Knight
************************************************************************
        subroutine specpos ( xin , pgrd1 , pgrd2 , pgrd3 , score , 
     &  low , errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspsmap.inc'
        include         'rspssym.inc'
        include         'rspsctl.inc'
        include         'rspsgrd.inc'


*       Arguments
        integer         low
        integer         pgrd1 , pgrd2 , pgrd3
        real            score
        real            xin(3)
        integer         errcod

*       Local variables
        logical         mfun
        integer         i , j
        integer         nv
        integer         imult( mxvec )
        integer         npkuse
        real            u( 3 )
        real            use( 3,mxvec )
        real            peak( mxvec )
        real            dist
        real            rho
        real            tol

*       Data statements
*       Minimum grid separation of vectors in any direction
        data            tol/1.5/

*       External functions
        integer         lgseql
        real            getrho
        real            vlen
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Generate all vectors keeping only the unique set
        nv     = 0
        low    = 0
        score  = 0.0
        do 100 i = 1 , ntrans

*       Compute vector
          call vmtply ( r_vec(1,1,i) , xin , u )
          do j = 1 , 3
            u( j ) = u( j ) + t_vec( j,i )
          end do 

*       Reduce to map volume
          call reduce ( u )

*       Get length of vector
          dist = vlen ( u , cell , cosang )

*       Skip null vectors
          if ( dist .lt. 2 ) goto 100

*       Compare with previously found vectors
          do 80 j = 1 , nv
            if ( lgseql ( u , use( 1,j ) , tol ) .eq. 1 ) then
*       Old vector - skip it
              goto 100
            endif
 80       continue

*       New vector - store it 
          nv = nv + 1
          if ( nv .gt. mxvec ) then
            write ( 6,9000 ) mxvec , nv
            errcod = 1
            return
          endif
          call matcop ( u , use( 1,nv ) , 3 , 1 )

*       Get density
          rho = getrho ( u , pektol , 
     &    patmap , patfas , patslo , patsec ,
     &    pgrd1 , pgrd2 , pgrd3 )

*       Count number of low peaks
          if ( rho .lt. reject ) low = low + 1

*       Store peak 
          peak( nv ) = rho 
          imult( nv ) = 1

 100    continue

*       The number of low peaks is scaled by the number 
*       of possible vectors / special-position vectors
        if ( nv .eq. 0 ) then
          low = ntrans
          return
        else
          low = low * real( ntrans ) / real( nv )
        endif
        if ( low .gt. maxlow ) return

*       Set number of peaks to use = all if not min function
        mfun = minfun
        if  ( nuse .ge. nv ) mfun = .false.
        if ( mfun ) then
          npkuse = nuse
        else
          npkuse = nv
        endif

*       Get score
        call rsscor ( peak , imult , nv , npkuse , patsig , 
     &  scrtyp , mfun , usemult , score , errcod )
        if ( errcod .ne. 0 ) errcod = 2

        return
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
*       Error messgaes
 9000   format(/10x,'** ERROR - Too many vectors generated',
     &  ' in s/r SPECPOS; max = ',i5,' found = ',i5,' **' )
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine VLSPECPOS
*
*       Function:
*       Compute all (pseudo and/or true) Harker vectors for position xin
*       Output vector list
*
*
*       Arguments :
*       Xin     (i)     position to be used for calculating vectors
*       Pektot  (i)     number of stored Patterson peaks
*       Pgrd1   (i)     Patterson grid along fast axis
*       Pgrd2   (i)     Patterson grid along slow axis
*       Pgrd3   (i)     Patterson grid along section axis
*       Errcod  (o)     Error code
*                       errcod = 0 - no error
*
*       History:
*       February 16 1994 - sdk
*       Fixed bug in writing vector density
*       Written December 9 1993 S. Knight
************************************************************************
        subroutine vlspecpos ( xin , pektot , pgrd1 , pgrd2 , pgrd3 , 
     &  errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspsmap.inc'
        include         'rspssym.inc'
        include         'rspsctl.inc'
        include         'rspsgrd.inc'


*       Arguments
        integer         pektot
        integer         pgrd1 , pgrd2 , pgrd3
        integer         errcod
        real            xin(3)


*       Local variables
        logical         mfun
        integer         i , j
        integer         nv
        integer         minmul , imult( mxvec )
        integer         low
        integer         peknr(mxvec)
        integer         npkuse
        integer         ihit
        real            pekdist , maxdist
        real            summlt
        real            sumhgt
        real            ssqpos , ssqhgt
        real            rmspos , rmshgt
        real            rmspsw , rmshtw
        real            mindx
        real            u( 3 )
        real            use( 3,mxvec )
        real            peak( mxvec )
        real            dist
        real            rho , score
        real            height
        real            tol

*       Weight for rmsd's in matching index
        parameter       ( rmspsw = 0.8 )
        parameter       ( rmshtw = 0.1 )

*       Data statements
*       Minimum grid separation of vectors in any direction
        data            tol/1.5/

*       External functions
        integer         lgseql
        real            getrho
        real            vlen
        real            ppv
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Set distance cutoff for peak search
        maxdist = 2.5 * max ( cell( 1 ) / patgrd( 1 ) , 
     &                        cell( 2 ) / patgrd( 2 ) ,
     &                        cell( 3 ) / patgrd( 3 ) )


*       Generate all vectors keeping only the unique set
        nv     = 0
        low    = 0
        score  = 0.0
        do 100 i = 1 , ntrans

*       Compute vector
          call vmtply ( r_vec(1,1,i) , xin , u )
          do j = 1 , 3
            u( j ) = u( j ) + t_vec( j,i )
          end do 

*       Reduce to map volume
          call reduce ( u )

*       Get length of vector
          dist = vlen ( u , cell , cosang )

*       Skip null vectors
          if ( dist .lt. 2.0 ) goto 100

*       Compare with previously found vectors
          do 80 j = 1 , nv
            if ( lgseql ( u , use( 1,j ) , tol ) .eq. 1 ) then
*       Old vector - skip it
              imult( j ) = imult( j ) + 1
              goto 100
            endif
 80       continue

*       New vector - store it 
          nv = nv + 1
          if ( nv .gt. mxvec ) then
            write ( 6,9000 ) mxvec , nv
            errcod = 1
            return
          endif
          call matcop ( u , use( 1,nv ) , 3 , 1 )
          imult( nv ) = 1

 100    continue


*       Normalize multiplicities
        minmul = 999999999
        do 200 i = 1 , nv
          minmul = min ( minmul , imult( i ) )
 200    continue
        do 210 i = 1 , nv
          imult( i ) = real ( imult( i ) ) / real ( minmul )
 210    continue


*       Get densities
        do 300 i = 1 , nv
          rho = getrho ( use( 1,i ) , pektol , 
     &    patmap , patfas , patslo , patsec ,
     &    pgrd1 , pgrd2 , pgrd3 )

*       Count number of low peaks
          if ( rho .lt. reject ) low = low + 1

*       Store peak 
          peak( i ) = rho 
 300    continue


*       The number of low peaks is scaled by the number 
*       of possible vectors / special-position vectors
        if ( nv .eq. 0 ) then
          return
        else
          low = low * real( ntrans ) / real( nv )
        endif


*       Write out vector, density, multiplicity, closest peak and 
*       distance, rmsd's and matching index
        write ( 6,2000 )
        if ( pektot .ne. 0 ) then
          write(6,2030)
        else
          write(6,2040)
        endif
        ihit = 0
        ssqpos = 0.0
        ssqhgt = 0.0
        sumhgt = 0.0
        summlt = 0.0
        do 400 i = 1 , nv
          summlt = summlt + imult ( i )
          call fndpek ( use( 1,i ) , peknr( i ) , pekdist , 
     &    maxdist )
          if ( peknr( i ) .ne. 0 ) then
            write(6,2100) i , ( use(j,i),j=1,3 ) , peak( i ) , 
     &      imult(i) , peknr( i ) , pekdist
            ihit = ihit + 1
            ssqpos = ssqpos + pekdist**2
            sumhgt = sumhgt + ppv( peknr( i ) ) / patsig
          else
            if ( pektot .eq. 0 ) then
              write(6,2150) i , ( use(j,i),j=1,3 ) , peak( i ) , 
     &        imult(i)  
            else
              write(6,2155) i , ( use(j,i),j=1,3 ) , peak( i ) , 
     &        imult(i)  
            endif
          endif
 400    continue

*       Rmsd peak positions and peak heights; matching index
        do 500 i = 1 , nv
          if ( peknr( i ) .ne. 0 ) then
            height = ppv( peknr( i ) ) / patsig
          else
            height = 0.0
          endif
          ssqhgt = ssqhgt + ( height - (sumhgt/summlt)*imult( i ) )**2
 500    continue
        rmspos = 999.
        rmshgt = 999.
        if ( ihit .gt. 0 ) then
          rmspos = sqrt ( ssqpos / ihit )
        endif
        rmshgt  = sqrt ( ssqhgt  / nv )
        mindx = ( 1 + ihit ) / 
     &  ( ( 1 + rmspsw*rmspos )*( 1 + rmshtw*rmshgt )*( 1 + ntrans ) )


*       Set number of peaks to use = all if not min function
        mfun = minfun
        if  ( nuse .ge. nv ) mfun = .false.
        if ( mfun ) then
          npkuse = nuse
        else
          npkuse = nv
        endif

*       Get score
        call rsscor ( peak , imult , nv , npkuse , patsig , 
     &  scrtyp , mfun , usemult , score , errcod )
        if ( errcod .ne. 0 ) then
          errcod = 2
          return
        endif

        if ( pektot .eq. 0 ) then
          write(6,2200)score,low
        else
          write(6,2210)score,low,rmspos,rmshgt,mindx
        endif
        return
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
*       Error messages
 9000   format(/10x,'** ERROR - Too many vectors generated',
     &  ' in s/r VLSPECPOS; max = ',i5,' found = ',i5,' **' )
*       Output formats
 2000   format(10x,'Special position' )
 2030   format(//10x,' Vec       U       V       ',
     &  'W        Rho     Multiplicity     Peak   Distance',/10x,
     &  ' ---    ------  ------  ------    -------   ------------',
     &  '     ----   --------' )
 2040   format(//10x,' Vec       U       V       ',
     &  'W        Rho     Multiplicity',/10x,
     &  ' ---    ------  ------  ------    -------   ------------')
 2100   format(10x,i3,3x,3f8.4,f10.2,7x,i4,8x,i5,7x,f4.2)
 2150   format(10x,i3,3x,3f8.4,f10.2,7x,i4)
 2155   format(10x,i3,3x,3f8.4,f10.2,7x,i4,10x,'none    > ',f4.2)
 2200   format(/10x,'Score = ',f8.2,' with ',i5,
     &  ' low peaks',/10x,'************************',
     &  '****************************************************')
 2210   format(/10x,      ' Score               = ',f8.2,' with ',i5,
     &  ' low peaks',/10x,' Rmsd peak positions = ',f8.4, 
     &               /10x,' Rmsd peak heights   = ',f8.4,
     &               /10x,' Matching index      = ',f8.4,
     &               /10x,'******************************',
     &  '**********************************************')
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine XTRAN
*
*       Function:
*       One dimensional translation cross scan
*       
*       Comments:
*       The transformations that map a point in search space to 
*       a point in vector space have to be generated by the
*       apropriate call to GETTNF prior to calling this routine.
*       
*       Arguments:
*       Errcod   (i)    error code
*                       errcod = 0 - no error
*                       errcod = 1 - ONEDSCAN failure
*                       errcod = 2 - QSORT failure
*                       errcod = 3 - orthogonalization error
*       
*       History:
*       Written February 11 1992 S. Knight
************************************************************************
        subroutine xtran ( errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'
        include         'rspsmap.inc'
        include         'rspspos.inc'
        include         'rspsgrd.inc'

*       Arguments
        integer         errcod

*       Local variables
        integer         i , j
        real            xut( 3 )
        real            toort( 3,3 )
        character*3     axslab
        character*8     date
        data            axslab / 'XYZ' /

*       External functions
        integer         sfnblk , slnblk
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Initiate position main array
        call matclr ( xpick , 3 , mxpick )
        call matclr ( value , mxpick , 1 )
        xintot = 0

*       Do one dimensional translation scan
        do 10 i = 1 , ntrnps

          call onedscan ( xscan( 1,i ) , patmap , patfas , 
     &    patslo , patsec , patsig , errcod )
          if ( errcod .eq. 3 ) then
            call wrnmsg ( 'Positions array full, list truncated' )
            errcod = 0
            goto 20
          else if ( errcod .ne. 0 ) then
            call errmsg ( 'Translation scan failed' , 0 )
            errcod = 1
            return
          endif

 10     continue

*       Anything found ?
 20     if ( xintot .eq. 0 ) then
          write(6,2000)
          return
        endif
*       Sort peaks in descending order
        call qsortff ( xintot , xpick , value , errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'Recursion stack overflow in QSORTFF' , 1 )
          errcod = 2
          return
        endif

*       Assign site numbers
        call equsit ( xintot , xpick , sittot , sitbeg , sitend ,
     &  sitpnt , errcod )

*       Form title
        call ccpdat ( date )
        title = ' '
        title = 'RSPS POLARSCAN ' // axslab( scndir:scndir ) //
     &  ' ' // date( :slnblk ( date ) ) // ' > ' // 
     &  titut( sfnblk ( titut ):slnblk ( titut ) )

*       Set matrix for converting fractional crystallographic
*       coordinates to Angstrom orthogonal in PDB standard frame
        call ortho ( cell , 1 , 'TOORT' , toort , errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'Failed getting orthogonalization matrix' , 
     &    0 )
          errcod = 3
          return
        endif

*       Write result
        write(6,2100)xintot
        do 50 i = 1 , xintot
          call vmtply ( toort , xpick( 1,i ) , xut )
          write(6,2200) i , ( xpick( j,i ) , j = 1 , 3 )  ,
     &    xut , value( i ) , sitpnt( i )
 50     continue

        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
 2000   format(/10x,'POLARSCAN >> No positions found')
 2100   format(/10x,'POLARSCAN >> ',i5,' positions found',/10x,
     &  'These are listed in descending order',//12x,'PosnN     ',
     &  ' Fractional coordinates     Angstrom coordinates',
     &  '      Score    Site',/12x,'-----     ',
     &  '------------------------  ------------------------  ',
     &  '---------  ----')
 2200   format(10x,i5,6x,3f8.4,2x,3f8.2,2x,f8.2,4x,i4)
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine ONEDSCAN
*
*
*       Function:
*       Given one or more fixed positions one position with one 
*       coordinate undefined; scan along that axis and find maxima in 
*       score function.
*
*       Comments:
*       The transformations that map a point in search space to 
*       a point in vector space have to be generated by the
*       apropriate call to GETTNF prior to calling this routine.
*
*
*       Arguments :
*       Xscn    (i)     scan position
*       Patmap  (i)     patterson map densities
*       Patfas  (i)     number of points in fast direction of p'son map
*       Patslo  (i)     number of points in slow direction of p'son map
*       Patsec  (i)     number of sections in p'son map
*       Patsig  (i)     standard deviation of patterson map
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - 
*                       errcod = 2 - attempted positions array overflow
*
*
*       History:
*       Written Jan 1992 S. Knight
************************************************************************
        subroutine onedscan ( xscn , patmap , patfas , 
     &  patslo , patsec , patsig , errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'
        include         'rspspos.inc'

*       Arguments
        integer         patfas , patslo , patsec
        real            patmap( patfas , patslo , patsec )
        real            xscn( 3 )
        real            patsig
        integer         errcod

*       Local variables
        logical         mfun
        integer         i , j , nt
        integer         ij 
        integer         iscani , iscanf
        integer         low
        integer         pgrd1 , pgrd2 , pgrd3
        integer         npkuse
        integer         maxlin
        parameter       ( maxlin = mxpatp/1000 )
        integer         imult(mxvec)
        real            dist
        real            score , rho , minsc , maxsc
        real            minlev
        real            scorut(0:maxlin)
        real            x(3) , xfrac(3) , peak( mxvec ) , u(3)

*       External functions
        integer         sgseql
        real            getrho
        real            vlen
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Init min and max
        minsc = 1.e+38
        maxsc = 0.0

*       Check flags
        if (scrtyp.lt.1.or.scrtyp.gt.3)scrtyp=1

*       Set number of peaks to use = all if not min function
        mfun = minfun
        if ( nuse .ge. ntrans ) mfun=.false.
        if ( mfun ) then
          npkuse = nuse
        else
          npkuse = ntrans
        endif

*       Set peak weights
        do 10 i = 1 , ntrans
          imult( i ) = 1
          if ( usemult ) imult( i ) = mult( i )
 10     continue

*       Start and end points for scan ( = whole cell axis )
        iscani = 0
        iscanf = patgrd( scndir )
        if ( iscanf - iscani + 1 .gt. maxlin - 1 ) then
          errcod = 1
          return
        endif

*       Set grid along fast, slow, section of map
        pgrd1 = patgrd( pataxo( 1 ) )
        pgrd2 = patgrd( pataxo( 2 ) )
        pgrd3 = patgrd( pataxo( 3 ) )

*       Copy coordinates of scan point
        call matcop ( xscn , xfrac , 3 , 1 )

*       Scan along scan axis
        do 100 i  =  iscani , iscanf

*       Set score array pointer
          ij = i - patlim( 2*scndir-1 ) + 1

*       Set test point coordinate
          x( scndir ) = i

*       Fractional coordinate of test point
          xfrac( scndir ) = x( scndir ) / patgrd( scndir )

*       Init score and low-peak counter
          scorut( ij ) = 0.
          low     =   0


          do 50 nt = 1 , ntrans
*       Compute vector
            call vmtply ( r_vec(1,1,nt) , xfrac , u )
            do j = 1 , 3
            u( j ) = u( j ) + t_vec( j,nt )
            end do 

*       Reduce to map volume
            call reduce ( u )

*       Get length of vector
            dist = vlen ( u , cell , cosang )

*       Take care of bumps 
            if ( dist .lt. bump ) goto 100

*       Get density for this vector.
            rho = getrho ( u , pektol , 
     &      patmap , patfas , patslo , patsec ,
     &      pgrd1 , pgrd2 , pgrd3 )

*       Count number of low peaks
            if ( rho .lt. reject ) low = low + imult( nt )
            if ( low .gt. maxlow ) goto 100

*       Store peak 
            peak( nt ) = rho 
 50       continue


*       Get score
          call rsscor ( peak , imult , ntrans , npkuse , patsig , 
     &    scrtyp , mfun , usemult , score , errcod )

*       Store the score
          scorut( ij ) = score

*       Min and max
          minsc = min ( minsc , score )
          maxsc = max ( maxsc , score )

 100    continue



*       Return if no positions accepted
        if ( maxsc .lt. 0.00001 ) return

*       Find maxima in line
        minlev = minsc
        if ( minlev .eq. 0 ) minlev = ( maxsc - minsc ) / 4.
        do 200 i = iscani,iscanf

*       Set score array pointer
          ij = i - patlim( 2*scndir-1 ) + 1


*       Max ?
          if ( scorut( ij ) .gt. scorut( ij - 1 ) .and.
     &    scorut( ij ) .gt. scorut( ij + 1 ) ) then
            if ( scorut( ij ) .lt. minlev ) goto 200
*       Set test point coordinate
            x( scndir ) = i
*       Fractional coordinate of test point
            xfrac( scndir ) = x( scndir ) / patgrd( scndir )
*       Check if sgs equivalent to previously found position
            do 150 j = 1 , xintot
              if ( sgseql ( xfrac , xpick( 1,j ) , bump ) 
     &        .eq. 1 ) goto 200
 150        continue
*       Count and store in main coordinate array
            if ( xintot .lt. mxpick ) then
              xintot = xintot + 1
              call matcop ( xfrac , xpick( 1,xintot ) , 3 , 1 )
              call matcop ( scorut( ij ) , value( xintot ) , 1 , 1 )
            else
              call wrnmsg ( 'Attempted positions array overflow' )
              errcod = 2
              return
            endif
          endif

 200    continue

        return
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine RSSCOR
*
*       Scoring subroutine for RSPS03
*
*
*
*  Arguments:
*       rhopek     (i)            array of peak values
*       mltpek     (i)            array of peak multiplities
*       numpek     (i)            number of peaks
*       numuse     (i)            number of peaks to use
*       patsig     (i)            rms of patterson
*       iscrtp     (i)            score type flag:
*                                 iscrtp = 1 - sum function
*                                 iscrtp = 2 - product function
*                                 iscrtp = 3 - harmonic mean
*       minfun     (i)            minimum function flag
*       domult     (i)            flag for multiplicity weights
*       score      (o)            score
*       errcod     (o)            error code:
*                                 0 - no error
*
*
*
*       History:
*       Written Dec 1991 S. Knight
************************************************************************
        subroutine rsscor( rhopek , mltpek , numpek , numuse , patsig ,
     &  iscrtp , minfun , domult , score , errcod )

        implicit none

        include            'rspsdim.inc'


*       Arguments
        logical            minfun , domult
        integer            numpek , numuse , iscrtp , errcod 
        integer            mltpek( numpek )
        real               rhopek( numpek )
        real               patsig
        real               score

*       Local variables
        integer            i , j
        real               rho , p
Csdk        real               w( mxvec )
*-----------------------------------------------------------------------
*       Init
*
        errcod = 0
        score  = 0.0
        if ( iscrtp .eq. 2 ) score = 1.0
*
*       Check input
*
        if ( numpek .eq. 0 ) return
        if ( numuse .eq. 0 ) return
        if ( iscrtp .lt. 1 .or. iscrtp .gt. 3 ) iscrtp = 1
*
*       Apply weights
*
        do 10 i = 1 , numpek
Csdk          w( i ) = 1.0
Csdk          if ( domult ) w( i ) = mltpek( i )
Csdk          rhopek( i ) = w( i ) * rhopek( i ) / patsig
          rhopek( i ) = mltpek( i ) * rhopek( i ) / patsig
 10     continue

*
*       Sort peaks if min function
*
        if ( minfun ) then
          if ( numuse .gt. numpek ) then
            numuse = numpek
          else
*       Sort peaks in ascending order
            do 40 i = 1 , numpek - 1
              p = rhopek( i+1 )
              do 20 j = i , 1 , -1
                if ( p .ge. rhopek( j ) ) then
                  goto 30
                else
                  rhopek( j+1 ) = rhopek( j )
                endif
 20           continue
 30           rhopek( j+1 ) = p
 40         continue
          endif
        endif

*
*       Form sums
*
        do 50 i = 1  , numuse
          if ( iscrtp .eq. 1 ) then

*       Sum 
            score = score + rhopek( i )

          else if ( iscrtp .eq. 2 ) then

*       Product
            rho = max ( rhopek( i ) , 0.001 )
            score = score * rho

          else if ( iscrtp .eq. 3 ) then

*       Harmonic mean
            rho = max ( rhopek( i ) , 0.001 )
            score = score + 1. / rho
          endif
 50     continue

*
*       Average score/peak
*
        score = score / numuse
*
*       Harmonic mean
*
        if ( iscrtp .eq. 3 ) then
          if ( score .ne. 0 ) score = 1. / score
        endif

        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine SETCON
*
*       Function:
*       Set up connect matrix for connect
*
*
*       Arguments :
*       Patmap  (i)     patterson map densities
*       Patfas  (i)     number of points in fast direction of p'son map
*       Patslo  (i)     number of points in slow direction of p'son map
*       Patsec  (i)     number of sections in p'son map
*       R,t     (i)     symmetry operations
*       Nsym    (i)     number of symmetry operations
*       Xyz     (i)     postions (fractional)
*       Sitpnt  (i)     pointers from positions to site nr
*       Nxyz    (i)     number of positions
*       Polar   (i)     polar axis flags
*       Conmtx  (o)     nxyz x nxyz connect table. contmx(i,j)
*                       is set to .true. if positions i and j are
*                       connected
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - SYMAPP error
*
*
*       History:
*       February 1992 - sdk
*       Fixed bug: no check for bumps was done, now skips if bumping
*       Don't skip because of identical equivalence group in P1
*       (here all positions have site = 1!).
*       November 1991 - sdk
*       Introduced search for max density within box around cross
*       vectors.
*       August 1991 - sdk
*       Arguments modified
*       Written March 30 1988 S. Knight
************************************************************************
        subroutine setcon (patmap,patfas,patslo,patsec,r,t,nsym,xyz,
     &  sitpnt,nxyz,polar,conmtx,errcod)

        implicit none

        include         'rspsdim.inc'
        include         'rspsctl.inc'
        include         'rspsgrd.inc'
*
*       Arguments
*
        integer         patfas , patslo , patsec
        integer         nsym
        integer         nxyz
        integer         sitpnt( nxyz)
        integer         polar(3)
        real            patmap( patfas , patslo , patsec )
        real            r(3,3,nsym),t(3,nsym)
        real            xyz(3,nxyz)
        logical         conmtx(nxyz,nxyz)
        integer         errcod
*
*       Local variables
*
*       Patterson grid along fast, slow, medium axis
        integer         pgrd1 , pgrd2 , pgrd3

*       Floating point copy of input positions
        real            x1(3) , x2(3)

*       Symmetry expanded set of positions
        real            xout1(3,mxsym) , xout2(3,mxsym)

*       Number of positions after symmetry expansion
        integer         nxout1 , nxout2

*       Unique set of vectors between symmetry expanded set of coords
        real            vec(4,mxvec)

*       Number of vectors
        integer         nvec

*       Vector coordinates
        real            u(3)

*       Length of vector
        real            dist

*       Loop control variables
        integer         i , j , k

*       Counter for low peaks
        integer         low

*       Peak value at vector
        real            rho

*       Connected pair counter
        integer         concnt

*       Polar axis flag
        integer         polax

*       External functions
        real            getrho
        real            vlen
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Init count of connected pairs
        concnt = 0

*       Set polar axis flag. Polar(i) = 0 if i is polar axis.
*       No polar axis  ==> polax = 3
*       One polar axis ==> polax = 2
*       Two polar axes ==> polax = 1 (no such case)
*       P1             ==> polax = 0
        polax = polar(1) + polar(2) + polar(3)

*       Set grid along fast, slow, section axis
        pgrd1 = patgrd( pataxo( 1 ) )
        pgrd2 = patgrd( pataxo( 2 ) )
        pgrd3 = patgrd( pataxo( 3 ) )

        do 200 i = 1 , nxyz-1
        conmtx( i,i ) = .true.
        do 100 j = i+1 , nxyz
          conmtx( j,i ) = .false.
          conmtx( i,j ) = .false.

*       Skip if sites belong to the same equivalence group
*       (except if P1).
          if ( sitpnt( i ) .eq. sitpnt( j ) .and. polax .ne. 0 ) 
     &    goto 100

          low     =   0
          nxout1  =   mxsym
          nxout2  =   mxsym
          nvec    =   mxvec

          do k=1,3
            x1(k) = xyz(k,i)
            x2(k) = xyz(k,j)
          end do

*       Get all unique positions symmetry related to x1 and x2
          call symapp ( nsym,r,t,1,x1,nxout1,xout1,errcod )
          if ( errcod .ne. 0 ) then
            errcod = 1
            return
          endif
          call symapp ( nsym,r,t,1,x2,nxout2,xout2,errcod )
          if ( errcod .ne. 0 ) then
            errcod = 1
            return
          endif

*       Get unique vectors between positions in xout1 and xout2
          call cvcalc ( xout1,xout2,nxout1,nxout2,vec,nvec )


*       Loop over all vectors found
          do 80 k = 1 , nvec


*       Put copy in u
            call matcop ( vec( 1,k ) , u , 3 , 1 )

*       Reduce to map volume
            call reduce ( u )

*       Get length of vector
            dist = vlen ( u , cell , cosang )

*       Skip if too close
            if ( dist .lt. bump ) goto 100

*       Get density
            rho = getrho ( u , pektol , 
     &      patmap , patfas , patslo , patsec , 
     &      pgrd1 , pgrd2 , pgrd3 )

*       Count number of low peaks
            if ( rho .lt. reject ) low = low + vec( 4,k )
            if ( low .gt. maxlow ) goto 100

 80       continue

          concnt = concnt + 1
          conmtx( j,i ) = .true.
          conmtx( i,j ) = .true.

 100    continue
 200    continue
        conmtx( nxyz,nxyz ) = .true.

        write(6,2000)concnt
 2000   format(/10x,'SETCON >> ',i8,' connected pairs found')

        return

*-----------------------------------------------------------------------
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine CONNECT
*
*       Function:
*       Mask out connected sets from conmtx
*       Conmtx is n x n array where conmtx(i,j) = .true. if
*       positions i and j are connected and .false. otherwise
*
*
*       Arguments :
*       Conmtx  (i)     N x n connect matrix
*       N       (i)     Dimension of conmtx and sitpnt (number of positions)
*       Minset  (i)     Minimum number of members in a set
*
*
*       Algorithm:
*       Example
*
*       Say we have ten positions 1 - 10. Say we have three sets
*       of connected positions:
*       3, 5, 6, 8
*       2, 4, 6, 8, 9
*       1, 3, 7, 10
*
*       The connect matrix then is:
*
*          i 1  2  3  4  5  6  7  8  9 10
*        j
*        1   1  0  1  0  0  0  1  0  0  1
*        2   0  1  0  1  0  1  0  1  1  0
*        3   1  0  1  0  1  1  1  1  0  1
*        4   0  1  0  1  0  1  0  1  1  0
*        5   0  0  1  0  1  1  0  1  0  0
*        6   0  1  1  1  1  1  0  1  1  0
*        7   1  0  1  0  0  0  1  0  0  1
*        8   0  1  1  1  1  1  0  1  1  0
*        9   0  1  0  1  0  1  0  1  1  0
*       10   1  0  1  0  0  0  1  0  0  1
*
*
*
*       a) a trivial case
*       The first connected pair we find are 1 & 3 (from row 1: 
*       conmtx(i,j) = 1 for i = 3, j = 1 ).
*       Combining rows 1 and 3 we find all positions connected to
*       both 1 and 3 (but not necessarily to each other):
*
*
*         1:   1  0  1  0  0  0  1  0  0  1  Current set
*         3:   1  0  1  0  1  1  1  1  0  1
*       ------------------------------------
*       cmsk   1  0  1  0  0  0  1  0  0  1    1 & 3
*       neq1   4
*       cmmb1  1, 3, 7, 10
*
*       So 1, 3 & 7 are connected and so are 1, 3 & 10. We now need
*       to find out whether 7 & 10 are also connected. What we do is
*       combine the current cmsk, (which points to all positions
*       connected to the current set of positions, at this stage 1 & 3)
*       with the row of pointers for each position in the current cmmb1
*       list:
*
*       cmsk   1  0  1  0  0  0  1  0  0  1  Current set
*         1:   1  0  1  0  0  0  1  0  0  1
*       ------------------------------------
*       cmsk   1  0  1  0  0  0  1  0  0  1    1 & 3 (comparison is superfluous
*       neq2   4                                      since 1 & 3 have already
*       cmmb2  1, 3, 7, 10                            been grouped)
*
*       cmsk   1  0  1  0  0  0  1  0  0  1
*         3:   1  0  1  0  1  1  1  1  0  1
*       ------------------------------------
*       cmsk   1  0  1  0  0  0  1  0  0  1    1 & 3 (comparison is superfluous
*       neq2   4                                      since 1 & 3 have already
*       cmmb2  1, 3, 7, 10                            been grouped)
*
*
*       cmsk   1  0  1  0  0  0  1  0  0  1
*         7:   1  0  1  0  0  0  1  0  0  1
*       ------------------------------------
*       cmsk   1  0  1  0  0  0  1  0  0  1    1 & 3 & 7
*       neq2   4
*       cmmb2  1, 3, 7, 10
*
*
*       cmsk   1  0  1  0  0  0  1  0  0  1
*        10:   1  0  1  0  0  0  1  0  0  1
*       ------------------------------------
*       cmsk   1  0  1  0  0  0  1  0  0  1    1 & 3 & 7 & 10
*       neq2   4
*       cmmb2  1, 3, 7, 10
*
*
*
*       b) a less trivial case
*       In row 6 we find that positions 6 & 8 are connected. Combining
*       we get:
*
*
*         6:   0  1  1  1  1  1  0  1  1  0  Current set
*         8:   0  1  1  1  1  1  0  1  1  0
*       ------------------------------------
*       cmsk   0  1  1  1  1  1  0  1  1  0    6 & 8
*       neq1   7
*       cmmb1  2, 3, 4, 5, 6, 8, 9
*
*
*       Combine cmsk with row 2 to find positions connected to 2,6 & 8:
*
*       cmsk   0  1  1  1  1  1  0  1  1  0
*         2:   0  1  0  1  0  1  0  1  1  0
*       ------------------------------------
*       cmsk   0  1  0  1  0  1  0  1  1  0    6 & 8 & 2
*       neq2   5
*       cmmb2  2, 4, 6, 8, 9
*
*       Is the next position in cmmb1 (3) also in cmmb2 ?
*       It is not, so it is not a possible addition to the current set.
*       Therefor we skip it and go to the next position in cmmb1, ie 4,
*       which is also present in the cmmb2 list. Thus
*
*
*       cmsk   0  1  0  1  0  1  0  1  1  0
*         4:   0  1  0  1  0  1  0  1  1  0
*       ------------------------------------
*       cmsk   0  1  0  1  0  1  0  1  1  0    6 & 8 & 2 & 4
*       neq2   5
*       cmmb2  2, 4, 6, 8, 9
*
*
*       Finally, combine the current cmsk with row 9: 
*
*
*       cmsk   0  1  0  1  0  1  0  1  1  0
*         9:   0  1  0  1  0  1  0  1  1  0
*       ------------------------------------
*       cmsk   0  1  0  1  0  1  0  1  1  0    6 & 8 & 2 & 4 & 9
*       neq2   5
*       cmmb2  2, 4, 6, 8, 9
*
*
*
*       History:
*       March 6 1995 - sdk
*       Score table now also lists overall score/position
*       March 1 1995 - sdk
*       Simplified interface, argument list reduced. Dimensions
*       mxgs, mxgm taken directly from mxgset, mxgmem in rspsdim.inc
*       Nmem, Cset, Nset = setsiz,setpos,settot in common /sets/ in
*       rspspos.inc
*       February 22 1995 - sdk
*       Fixed bug in extracting unique sets. Test now coded as
*       logical function EQUSET.
*       Data structure changed: Number of set members now in Nmem.
*       Cset now only contains position pointers.
*       Sets now sorted on scores.
*       Argument SITPNT removed; taken from common on rspspos.inc
*       November 9 1991 - sdk
*       Modified to only report unique sets.
*       Written April 11 1988 S. Knight
************************************************************************
        subroutine connect (conmtx,n,minset)

        implicit none

        include         'rspsdim.inc'
        include         'rspspos.inc'
*
*       Arguments
*
        integer         n , minset
        logical         conmtx(n,n)

*
*       Local variables
*

*       Pointers to positions "connected" to a common pair of positions
        integer         cmmb1(mxpick),cmmb2(mxpick)

*       Temporary vectors for finding connected positions by
*       masking out the unconnected ones.
*       An element in these vectors is set .true. if its index
*       is a pointer to a position connected to the current one
        logical         cmsk(mxpick),tmsk(mxpick)

*       Matrix representation of sets
        logical         setmtx(mxpick,mxgset)

*       Set pointer array for sorting
        integer         setpnt(mxgset)

*       Scratch arrays for reordering sorted sets
        integer         tmpset(mxgmem,mxgset)
        integer         tmpcnt(mxgset)

*       Loop control variables
        integer         i , j , k , m

*       Counters
        integer         neq1 , neq2 , neq3

*       Pointer to old, redundant, position in SETPOS and SETMTX
        integer         iswap

*       Printer output control variables
        integer         is , ie , ileft

*       Counter for matching positions in the cmsk and tmsk vectors
        integer         match

*       Output control
        integer         linlen
        parameter       (linlen=20)
        character*12    fmtlin

*       Error code
        integer         errcod

*       Scratch coordinate array
        real            xscr(3,mxpick)

*       External functions
        logical         equset
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        if(n.gt.mxpick)then
          write(6,9000)mxpick,n
          stop' ERROR - TOO MANY POSITIONS IN S/R CONNECT'
        endif

*       Init
        settot = 0
        do i = 1 , mxgset
          setsiz( i ) = 0
          do j = 1 , mxgmem
            setpos( j,i ) = 0
          end do
        end do
        do i = 1 , mxgset
        do j = 1 , mxpick
          setmtx( j,i ) = .false.
        end do
        end do
        fmtlin = '(1x,    i4)'
        write ( fmtlin(5:8) , '(i4)' ) linlen
 20     do 200 i=1,n-1
        do 100 j=i+1,n

*       If I and J positions are not connected, jump ahead
        if(.not.conmtx(i,j))goto 100

*
*       Combine rows I and J to find all positions connected to both
*       I and J. Jump out if less than the minimum accepted set remains.
*       The combined rows are stored in CMSK which becomes the current
*       mask. The corresponding NEQ1 position pointers are stored in
*       CMMB1.
*
        call msk ( conmtx(1,i),conmtx(1,j),cmsk,cmmb1,n,neq1 )
        if(neq1.lt.minset)goto 100

*
*       Combine the row for the first pointer in CMMB1 with the current 
*       mask to get pointers to all positions connected to I, J and this
*       position. These positions form the new current mask, with the
*       corresponding NEQ2 position pointers stored in CMMB2. Jump out
*       if less than the minimum accepted set remains.
*
        call msk ( conmtx(1,cmmb1(1)),cmsk,cmsk,cmmb2,n,neq2 )
        if(neq2.lt.minset)goto 100

*
*       Now repeat the above procedure for each position still remaining
*       while recursively updating the current mask and the
*       corresponding list of position pointers.
*
        do k=2,neq1
          do m=1,neq2

*       Check if k'th position still remains in list. If it does, update
*       the mask with the corresponding row from the connect matrix.
          if(cmmb1(k).eq.cmmb2(m))then
            call msk ( conmtx(1,cmmb1(k)),cmsk,cmsk,cmmb2,n,neq2 )

*       If the current mask is set to zero (false) in all positions
*       above the current pointer from the initial list this set is
*       completed.
            if(cmmb2(neq2).eq.cmmb1(k))goto 50
          endif

          end do
        end do


 50     if(neq2.gt.mxgmem)then
*
*       This set to large to be stored. As a partial remedy at least
*       list it as is.
*
          write(6,9050)
          if(neq2.le.linlen)then
            write(6,fmtlin)(cmmb2(k),k=1,neq2)
          else
            is=1
            ie=linlen-1
 60         write(6,fmtlin)(cmmb2(k),k=is,is+ie)
            is=is+ie+1
            ileft=neq2-is+1
            if(ileft.lt.ie+1)then
              ie=ileft-1
              write(6,fmtlin)(cmmb2(k),k=is,is+ie)
            else
              goto 60
            endif
          endif


        else if(neq2.ge.minset)then
          iswap=0
*       Compare with previously found sets.
          do 70 k=1,settot
*       If the possible new set is unchanged by masking with an old set
*       it is equivalent to or a subset of the old set.
            call msk ( setmtx(1,k),cmsk,tmsk,cmmb1,n,neq3 )
            call mskcmp ( cmsk,tmsk,n,match )
            if(match.eq.n)goto 100
*       If an old set is unchanged by masking with the possible new set
*       the old set is equivalent to or a subset of the new set.
*       Set iswap to the serial number of that set so that the new set
*       can be inserted in place of the old one in the list of found
*       sets.
            call mskcmp ( setmtx(1,k),tmsk,n,match )
            if(match.eq.n)iswap=k
 70       continue

*       New set found. Check if it is unique. Two sets are equivalent
*       (related by lgs (+origin shift)) if they have the same set of 
*       site numbers.
*SDK    The above assumption is erroneous since it assumes the same 
*SDK    lgs/origin shift has been applied to all sites. 
*SDK    Two sets are equivalent (related by lgs (+origin shift)) 
*       if they have the same set of site numbers
*       AND THE SAME SET OF CROSS VECTORS. Fixed; coded as logical
*       function.
          do 80 k=1,settot
            if ( equset ( setpos(1,k) , setsiz(k) , cmmb2 , neq2 ) ) 
     &      goto 100
 80       continue

*       New unique set - store it if possible. Only count it if not
*       taking the place of a previously found set.
          if(iswap.eq.0)settot=settot+1
          if(settot.gt.mxgset)then
*       Too many sets found - reset minimum number of members
*       in a set and restart
            minset=minset+1
            settot=0
            write(6,9200)minset
            goto 20
          else
*       Still space, allocate a slot
            if(iswap.eq.0)then
              m=settot
            else
              m=iswap
            endif
*       Store number of members
            setsiz(m)=neq2
*       Store pointers
            do k=1,neq2
              setmtx(cmmb2(k),m)=.true.
              setpos(k,m)=cmmb2(k)
            end do
          endif
        endif

 100    continue
 200    continue


*       Sort sets on descending scores
*       First get scores (scores go to array SETSCR in common SETS)
        do i = 1 , settot
          do j = 1 , setsiz(i)
            call rmatcop ( xpick(1,setpos(j,i)) , xscr(1,j) , 3 , 1 )
          end do
          call scrtab ( i , setsiz(i) , setpos(1,i) , xscr , 0 , 
     &    errcod )
          setpnt( i ) = i
        end do

*       Now sort pointers...
        call qsortpf ( settot , setpnt , setscr , errcod )
*       ... and reorganize sets (just a copy-recopy)
        do i = 1 , settot
          call imatcop ( setpos( 1,setpnt(i) ) , tmpset( 1,i ) , 
     &    setsiz( setpnt( i ) ) , 1 )
          tmpcnt( i ) = setsiz( setpnt( i ) )
        end do
        do i = 1 , settot
          setsiz( i ) = tmpcnt( i )
          call imatcop ( tmpset( 1,i ) , setpos( 1,i ) , 
     &    setsiz( i ) , 1 )
        end do


        return


*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 9000   format(/10x,'** ERROR - Too many positions in s/r',
     &  ' CONNECT; max = ',i5,' requested = ',i5,' **')
 9050   format(//10x,'** WARNING - Too big set - not saved! **',
     &  /13x,'members : ')
 9200   format(/10x,'** WARNING - Too many sets found - minimum ',
     &  ' number of members reset to ',i5,' **')
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Logical function EQUSET
*
*       Function:
*       Test if two GETSETS sets are equivalent:
*       Same sites and same cross vectors
*
*       Arguments:
*       Set1    (i)     First set
*       N1      (i)     Number of members in first set
*       Set2    (i)     Second set
*       N2      (i)     Number of members in second set
*
*       Returns true if sets are equivlent, false otherwise
*
*       History:
*       Written February 22 1995 S. Knight
************************************************************************
        logical function equset ( set1 , n1 , set2 , n2 )

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'
        include         'rspspos.inc'

*       Arguments
        integer         n1 , n2
        integer         set1(n1) , set2(n2)

*       Local variables
        integer         maxv
        parameter       ( maxv = mxsym*mxgmem*(mxgmem-1)/2 )
        integer         i , j
        integer         match         
        integer         ipos1 , ipos2
        integer         nxout1 , nxout2
        integer         nvec1 , nvec2
        integer         errcod
        real            x1(3,mxgmem) , x2(3,mxgmem)
        real            xout1(3,mxsym*mxgmem) , xout2(3,mxsym*mxgmem)
        real            vec1(4,maxv) , vec2(4,maxv)
        real            v1(3) , v2(3)


*       External functions
        integer         lgseql
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initialize
        equset = .false.
        errcod = 0
        match  = 0
        nxout1 = nopsym*n1
        nxout2 = nopsym*n2
        nvec1  = nxout1*(nxout1-1)
        nvec2  = nxout2*(nxout2-1)

*       First check if sets are of same size
        if ( n1 .ne. n2 ) return

*       Now compare site numbers for set members
        do i = 1 , n1
          do j = 1 , n2
            if ( sitpnt( set1( i ) ) .eq. 
     &           sitpnt( set2( j ) ) ) match = match + 1
          end do
        end do
*       If site numbers differ the sites are unique
        if ( match .lt. n1 ) return

        
*       Same sites in both sets; compare vector sets

*       Compute vectors between all members in each set
*       This will include Harker vectors as well which
*       is a redundant check but makes the code simpler

*       Copy coordinates
        do i = 1 , n1
          ipos1 = set1( i )
          ipos2 = set2( i )
          call matcop ( xpick( 1,ipos1 ) , x1( 1,i ) , 3 , 1 )
          call matcop ( xpick( 1,ipos2 ) , x2( 1,i ) , 3 , 1 )
        end do
*       Apply symmetry to generate the full unit cell of positions
*       for each set
        call symapp ( nopsym , r_sym , t_sym , n1 , x1 , nxout1 , 
     &  xout1 , errcod )
        call symapp ( nopsym , r_sym , t_sym , n2 , x2 , nxout2 , 
     &  xout2 , errcod )
        if ( errcod .ne. 0 ) call errmsg ( 'Fatal error ' //
     &  ' calling SYMAPP in fn EQUSET' , 1 )
*       Compute vectors between all positions in each set
        call cvcalc ( xout1 , xout1 , nxout1 , nxout1 , vec1 , nvec1 )
        call cvcalc ( xout2 , xout2 , nxout2 , nxout2 , vec2 , nvec2 )

*       If different number of vectors in the sets they are unique
        if ( nvec1 .ne. nvec2 ) return

*       Same number of vectors; compare
        match = 0
        do i = 1 , nvec1
          call matcop ( vec1( 1,i ) , v1 , 3 , 1 )
          do j = 1 , nvec2
            call matcop ( vec2( 1,j ) , v2 , 3 , 1 )
            match = match + lgseql ( v1 , v2 , 1.5 )
          end do
        end do
*       If vectors match sets are equivalent 
        if ( match .ge. nvec1 ) equset = .true.

        return    
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine FNDPEK
*
*       Function:
*       Find closest peak within maxdist Angstrom in peak list.
*
*       Arguments:
*       u       (i)     vector (fractional)
*       peknr   (o)     pointer to closest peak on peaklist
*       pekdist (o)     distance to closest peak
*       maxdist (i)     maximum allowed distance to peak
*
*
*       History:
*       Written November 13 1991 S. Knight
************************************************************************
        subroutine fndpek ( u , peknr , pekdist , maxdist )

        implicit none

        include         'rspsdim.inc'
        include         'rspsgrd.inc'
        include         'rspsmap.inc'


*       Arguments
        real            u( 3 )
        integer         peknr
        real            pekdist
        real            maxdist

*       Local variables
        real            udiff( 3 )
        real            dist
        integer         i

*       External functions
        real            vlen
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        pekdist = maxdist
        do 100 i = 1 , ppnum

          call vecdif ( u , ppuvw( 1,i ) , udiff , 3 )

          dist = vlen ( udiff , cell , cosang )
          if ( dist .lt. pekdist ) then
            pekdist = dist
            peknr = i
          endif

 100    continue


        if ( pekdist .ge. maxdist ) then
          peknr = 0
          pekdist = 0.0
        endif
        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine SCRTAB
*
*       Function:
*       Construct (and write out) score table for set of positions.
*       
*       
*       Arguments:
*       Iset      (i)     Set number
*       Nmem      (i)     Number of positions
*       Xin       (i)     Coordinates
*       Iprint    (i)     Print flag
*                         iprint   =  0 - don't print score table
*                         iprint .ne. 0 - print score table
*       Errcod    (o)     Error code
*                         errcod = 0 - no error
*                         errcod = 1 - HVSCOR error
*                         errcod = 2 - CVSCOR error
*                         errcod = 3 - orthogonalization error
*
*       The overall score is stored in array setscr in common SETS (except
*       for user defined set).
*
*       History:
*       February 24 1995 - sdk
*       Changed argument list: coordinates now passed explicitly
*       instead of through pointers. Pointer argument still kept
*       for reference in output.
*       February 25 1992 - sdk
*       Print flag argument added
*       November 25 1991 - sdk
*       Position pointers now as argument to allow for user defined
*       sets.
*       Written November 18 1991 S. Knight (factored out from RSPS main)
************************************************************************
        subroutine scrtab ( iset , nmem , pspnt , xin , iprint , 
     &  errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspsgrd.inc'
        include         'rspssym.inc'
        include         'rspspos.inc'
        include         'rspsmap.inc'
        include         'rspsctl.inc'


*       Arguments
        integer         iset
        integer         nmem 
        integer         pspnt(nmem)
        integer         iprint
        integer         errcod
        real            xin(3,nmem)

*       Local variables
*       Imem:           pointer to set members
        integer         imem

*       Table of harker & cross vector scores
        real            sctab(mxgmem,mxgmem) 

*       Score and vector count per member
        real            memscr(mxgmem)
        integer         nv_mb

*       Counters for scoretable
*       Harker
        integer         lo_hv , nv_hv
*       Cross
        integer         lo_cv , nv_cv
*       Temporary counters
        integer         nv , lo
*       Peak hit counters
        integer         ihit , ihit_hv , ihit_cv

*       Rms peak variables
        real            ssqpos  , ssqhgt
        real            ssqp_hv , ssqp_cv
        real            ssqh_hv , ssqh_cv
        real            rmspos  , rmshgt
        real            rmsp_hv , rmsp_cv
        real            rmsh_hv , rmsh_cv
        real            rmspsw  , rmshtw
        real            hitfrq , hitf_hv , hitf_cv
        real            mindx

*       Weight for rmsd's in matching index
        parameter       ( rmspsw = 0.8 )
        parameter       ( rmshtw = 0.1 )

*       Overall Harker and cross vector scores
        real            score_hv , score_cv , score_tv , sc

*       Temporary coordinate stores
        real            xut(3)

*       Orthogonalization matrix
        real            toort( 3,3 )

*       Loop control variables
        integer         i , j

*       Variable output format
        character*40    fmtlin
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       Init score and counters for this set
        score_hv = 0.0
        score_cv = 0.0
        score_tv = 0.0
        ssqp_hv  = 0.0
        ssqp_cv  = 0.0
        ssqh_hv  = 0.0
        ssqh_cv  = 0.0
        lo_hv    = 0
        lo_cv    = 0
        nv_hv    = 0
        nv_cv    = 0
        ihit     = 0
        ihit_hv  = 0
        ihit_cv  = 0

        if ( iprint .gt. 0 ) then
*       Set matrix for converting fractional crystallographic
*       coordinates to Angstrom orthogonal in PDB standard frame
          call ortho ( cell , 1 , 'TOORT' , toort , errcod )
          if ( errcod .ne. 0 ) then
            call errmsg ( 'Failed getting orthogonalization matrix' , 
     &      0 )
            errcod = 3
            return
          endif
        endif


*       Loop over all positions in the set
        do 200 i = 1 , nmem
          nv_mb = 0
*       Get harker vector score for this position
          if ( nopsgs .le. 1 ) goto 150
          call hvscor ( patmap , patfas , patslo , patsec , patsig ,
     &    r_sgs , t_sgs , nopsgs , xin(1,i) , sc , nv , lo , ihit ,
     &    ssqpos , ssqhgt , errcod )
          if ( errcod .ne. 0 ) then
            errcod = 1
            return
          endif
          sctab( i,i ) = sc
          if ( scrtyp .eq. 1 ) then
            memscr( i ) = sc * nv
            score_hv = score_hv + memscr( i )
          else if ( scrtyp .eq. 2 ) then
            memscr( i ) = sc * nv
            score_hv = score_hv * memscr( i )
          else if ( scrtyp .eq. 3 ) then
            if ( sc .ne. 0 ) then
              memscr( i ) = nv / sc
              score_hv = score_hv + memscr( i ) 
            endif
          endif

          nv_mb   = nv
          nv_hv   = nv_hv + nv
          lo_hv   = lo_hv + lo
          ihit_hv = ihit_hv + ihit
          ssqp_hv = ssqp_hv + ssqpos
          ssqh_hv = ssqh_hv + ssqhgt

 150      continue
          do 100 j = 1 , nmem
            if ( j .eq. i ) goto 100
*       Get cross vector scores to other members of the set
            call cvscor ( patmap , patfas , patslo , patsec , patsig ,
     &      r_sym , t_sym , nopsym , xin(1,i) , xin(1,j) , sc , nv , 
     &      lo , ihit , ssqpos , ssqhgt , errcod )
            if ( errcod .ne. 0 ) then
              errcod = 2
              return
            endif

            sctab( j,i ) = sc

            if ( scrtyp .eq. 1 ) then
              memscr( i ) = memscr( i ) + sc * nv
              score_cv    = score_cv    + sc * nv
            else if ( scrtyp .eq. 2 ) then
              memscr( i ) = memscr( i ) * sc * nv
              score_cv =    score_cv    * sc * nv
            else if ( scrtyp .eq. 3 ) then
              if ( sc .ne. 0 ) then
                memscr( i ) = memscr( i ) + nv / sc
                score_cv    = score_cv    + nv / sc
              endif
            endif

            nv_mb   = nv_mb + nv
            nv_cv   = nv_cv + nv
            lo_cv   = lo_cv + lo
            ihit_cv = ihit_cv + ihit
            ssqp_cv = ssqp_cv + ssqpos
            ssqh_cv = ssqh_cv + ssqhgt

 100      continue
*         Overall score per member
          if ( scrtyp .eq. 1 .or. scrtyp .eq. 2 ) then
            memscr( i ) = memscr( i ) / nv_mb
          else
            if ( memscr( i ) .ne. 0 ) memscr( i ) = nv_mb / memscr( i )
          endif
 200    continue


*       Overall scores for this set
        nv = nv_hv + nv_cv
        lo = lo_hv + lo_cv
        if ( scrtyp .eq. 1 ) then
          score_tv = ( score_hv + score_cv ) / nv
        else if ( scrtyp .eq. 2 ) then
          if ( nopsgs .eq. 1 ) then
            score_tv = ( score_cv ) / nv
          else
            score_tv = ( score_hv * score_cv ) / nv
          endif
        else if ( scrtyp .eq. 3 ) then
          if ( score_hv + score_cv .ne. 0 ) then
            score_tv = nv / ( score_hv + score_cv )
          endif
        endif
        if ( scrtyp .eq. 1 .or. scrtyp .eq. 2 ) then
          if ( nv_hv .ne. 0 ) score_hv = score_hv / nv_hv
          if ( nv_cv .ne. 0 ) score_cv = score_cv / nv_cv
        else if ( scrtyp .eq. 3 ) then
          if ( score_hv .ne. 0 ) score_hv = nv_hv / score_hv
          if ( score_cv .ne. 0 ) score_cv = nv_cv / score_cv
        endif

*       Peak hit success for this set
        rmspos  = 999.
        rmshgt  = 999.
        rmsp_hv = 999.
        rmsp_cv = 999.
        rmsh_hv = 999.
        rmsh_cv = 999.
        ihit    = ihit_hv + ihit_cv
        ssqpos  = ssqp_hv + ssqp_cv
        ssqhgt  = ssqh_hv + ssqh_cv
        hitfrq  = real ( ihit ) / real ( nv )
        if ( nv_hv .ne. 0 ) hitf_hv = real ( ihit_hv ) / real ( nv_hv )
        if ( nv_cv .ne. 0 ) hitf_cv = real ( ihit_cv ) / real ( nv_cv )
        if ( ihit .ne. 0 ) then
          rmspos = sqrt ( ssqpos / ihit )
        endif
        if ( ihit_hv .ne. 0 ) then
          rmsp_hv = sqrt ( ssqp_hv / ihit_hv )
        endif
        if ( ihit_cv .ne. 0 ) then
          rmsp_cv = sqrt ( ssqp_cv / ihit_cv )
        endif
        rmshgt  = sqrt ( ssqhgt / nv )
        if ( nv_hv .ne. 0 ) rmsh_hv = sqrt ( ssqh_hv / nv_hv )
        if ( nv_cv .ne. 0 ) rmsh_cv = sqrt ( ssqh_cv / nv_cv )
        mindx = ( 1 + ihit ) / 
     &  ( ( 1 + rmspsw*rmspos )*( 1 + rmshtw*rmshgt )*( 1 + nv ) )

*       Put overall score in common if not user defined set
        if ( iset .ne. 0 ) setscr( iset ) = score_tv

        if ( iprint .gt. 0 ) then
*       Write header and coordinates for this set
          if ( iset .ne. 0 ) then
            write(6,2000)iset,nmem,score_tv
          else
            write(6,2050)iset,nmem,score_tv
          endif
          do i = 1 , nmem
            call vmtply ( toort , xin(1,i) , xut )
            imem = pspnt( i )
            if ( iset .ne. 0 ) then
              write(6,2100) imem , (xin(j,i),j=1,3) ,
     &        xut , sitpnt( imem )
            else
              write(6,2150) imem , (xin(j,i),j=1,3) ,
     &        xut 
            endif
          end do

*       Write out score table
          write(6,2200)
          fmtlin = '(10x,'' PosnN'',   (i5,x),''   <Score>'')'
          write(fmtlin(15:17),'(i3)')nmem
          write(6,fmtlin)(pspnt(i),i=1,nmem)
          fmtlin = '(11x,i3,   x,   f6.2,3x,f6.2)'
          do i = 1 , nmem
            write(fmtlin(9:11),'(i3)') (i-1)*6+1
            write(fmtlin(14:16),'(i3)') nmem-i+1
            write(6,fmtlin)pspnt(i),(sctab(j,i),j=i,nmem),memscr(i)
          end do

*       Write harker and cross vector scores; peak hit success
          write(6,2400) nv , nv_hv , nv_cv , lo , lo_hv , lo_cv , 
     &    score_tv , score_hv , score_cv , hitfrq , hitf_hv , hitf_cv ,
     &    rmspos , rmsp_hv , rmsp_cv , rmshgt , rmsh_hv , rmsh_cv , 
     &    mindx
        endif


        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements

 2000   format(//10x,'***********************************************',
     &  '*********************************',/10x,' Set number',i4,'; ',
     &  i4,' members , overall score ',f8.2,//12x,'PosnN     ',
     &  ' Fractional coordinates     Angstrom coordinates',
     &  '    Site',/12x,'-----     ------------------------  ',
     &  '------------------------  ----')
 2050   format(//10x,'***********************************************',
     &  '*********************************',/10x,' Set number',i4,'; ',
     &  i4,' members , overall score ',f8.2,//12x,'PosnN     ',
     &  ' Fractional coordinates     Angstrom coordinates',
     &  /12x,'-----     ------------------------  ',
     &  '------------------------')
 2100   format(10x,i5,6x,3f8.4,2x,3f8.2,3x,i4)
 2150   format(10x,i5,6x,3f8.4,2x,3f8.2)
 2200   format(//10x,' Score table',/,10x,' -----------')
 2400   format(//10x,' Number of vectors      = ',
     &  i10,' (all)',i10,' (Harker)',i10,' (Cross)',
     &          /10x,' Number of low vectors  = ',
     &  i10,' (all)',i10,' (Harker)',i10,' (Cross)',
     &          /10x,' Score                  = ',
     &  f10.2,' (all)',f10.2,' (Harker)',f10.2,' (Cross)',
     &          /10x,' Peak hit frequency     = ',
     &  f10.4,' (all)',f10.4,' (Harker)',f10.4,' (Cross)',
     &          /10x,' Rmsd peak positions    = ',
     &  f10.4,' (all)',f10.4,' (Harker)',f10.4,' (Cross)',
     &          /10x,' Rmsd peak heights      = ',
     &  f10.4,' (all)',f10.4,' (Harker)',f10.4,' (Cross)',
     &          /10x,' Matching index         = ',f10.4,
     &  /10x,'*****************************************************',
     &  '***************************')
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Real function GETRHO
*
*       Function:
*       Return maximum Patterson map value in box around vector.
*       
*       
*       Arguments:
*       U        (i)    vector coordinates (fractional)
*       Pektol   (i)    box is defined by u +/- pektol in all three
*                       directions.
*       Patmap   (i)    Patterson map array
*       Patfas   (i)    number of points along fast axis
*       Patslo   (i)    number of points along slow axis
*       Patsec   (i)    number of sections
*       Pgrd1    (i)    Patterson grid along fast axis
*       Pgrd2    (i)    Patterson grid along slow axis
*       Pgrd3    (i)    Patterson grid along section axis
*
*       History:
*       Modified October 26 1994 - SDK:
*       Make sure array pointers don't overflow array bounds
*       Modified February 7 1994 - SDK:
*       Changed assignement of pointers to map array from using
*       int to nint for better consistency with picking routines
*       Written November 18 1991 S. Knight
************************************************************************
        real function getrho ( u , pektol , patmap , patfas , patslo , 
     &  patsec , pgrd1 , pgrd2 , pgrd3 )

        implicit none

        include         'rspsgrd.inc'


*       Arguments
        integer         pektol
        integer         pgrd1 , pgrd2 , pgrd3
        integer         patfas , patslo , patsec
        real            patmap( patfas , patslo , patsec )
        real            u( 3 )

*       Local variables
        integer         ip1 , ip2 , ip3
        integer         i , j , k
        integer         icnt
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initialize
        getrho = -1.e+38

*       Get pointers to fast,slow,section
        ip1 = min ( nint ( u( pataxo(1) ) * pgrd1 ) + 1 , patfas )
        ip2 = min ( nint ( u( pataxo(2) ) * pgrd2 ) + 1 , patslo )
        ip3 = min ( nint ( u( pataxo(3) ) * pgrd3 ) + 1 , patsec )

*       Search box around vector for maximum density.
*       Points are not reduced so this introduces some
*       error for vectors near the edge of the map.
        icnt = 0
        do 60 k = ip3 - pektol , ip3 + pektol
        if ( k .le. 0 .or. k .gt. patsec ) goto 60
          do 40 j = ip2 - pektol , ip2 + pektol
          if ( j .le. 0 .or. j .gt. patslo ) goto 40
            do 20 i = ip1 - pektol , ip1 + pektol
              if ( i .le. 0 .or. i .gt. patfas ) goto 20
              getrho = max ( getrho , patmap( i , j , k ) )
              icnt = icnt + 1
 20         continue
 40       continue
 60     continue

        if ( icnt .eq. 0 ) then
          call errmsg ( 'Severe out-of-bounds error in function ' //
     &    'GETRHO: program aborted' , 1 )
        endif
        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine EQUSIT
*
*       Function:
*       Find and flag EQUivalent positions, assign to the same SITe.
*       Here, equivalent positions are defined as positions that give
*       the same set of Harker vectors.
*
*       Arguments:
*       Posnum  (i)     number of postions
*       Poscrd  (i)     coordinates
*       Sittot  (o)     number of unique sites
*       Sitbeg  (o)     pointer to first position of a site
*       Sitend  (o)     pointer to last position of a site
*       Sitpnt  (o)     pointer from position to site nr
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - no positions in call
*                       errcod = 2 - SYMAPP error
*
*       History:
*       March 30 1993 - sdk
*       Commented out test for homomorphous positions in space groups
*       with indeterminate axis since this is still not at all doing what
*       it should. Test is now for equivalence of Harker vectors only.
*
*       February 5 1993 - sdk
*       Tried to fix check for homomorphous positions in space groups
*       with indeterminate axis so that it is properly and
*       generally handled for all cases(?).
*
*       March 18 1992 - sdk
*       Added trap for zero positions in call.
*
*       November 22 1991 - sdk
*       Changed vector comparison to use function LGSEQL instead of
*       reduced vector since s/r reduce may not always reduce vectors
*       to one specific asymmetric unit.
*
*       November 13 1991 - sdk
*       Added pointers to first and last position for each site.
*       NOTE though that the method of assigning the start and end of
*       a site presumes that all positions belonging to the same site
*       are grouped together in the list of positions. This should
*       normally be the case since positions are sorted on score, 
*       allthough when two sites have the same score it might not be.
*       
*       Written November 7 1991 S. Knight
************************************************************************
        subroutine equsit ( posnum , poscrd , sittot , sitbeg , sitend ,
     &  sitpnt , errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'

*       Arguments
        integer         posnum , sittot
        integer         sitbeg( posnum ) , sitend( posnum )
        integer         sitpnt( posnum )
        integer         errcod
        real            poscrd( 3 , posnum )

*       Local variables
        integer         nxsym
        real            x( 3 ) , xsym( 3 , mxsgs )
        integer         nv , nvec( mxpick )
        real            hv( 4 , mxvec ) , hvec( 3 , mxvec , mxpick )
        integer         i , j , k , kk , ict , polax
*        real            d
        real            tol
        data            tol / 1.5 /

*       External functions
        integer         lgseql
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initiate error code
        errcod = 0

*       If no positions return
        if ( posnum .eq. 0 ) then
          errcod = 1
          return
        endif

*       Initiate site pointers and counters
        sittot = 0
        call imatclr ( sitbeg , posnum , 1 )
        call imatclr ( sitend , posnum , 1 )
        call imatclr ( sitpnt , posnum , 1 )

*       Initiate local vector arrays
        call imatclr ( nvec , mxpick , 1 )
        call matclr ( hvec , 3 * mxvec , mxpick )

*       Flag polar axis. Polar(i) = 0 if axis i is polar.
*       No  polar axis ==> polax = 3
*       One polar axis ==> polax = 2
*       Two polar axes ==> polax = 1 (no such case)
*       P1             ==> polax = 0
        polax = polar(1) + polar(2) + polar(3)
        if ( polax .eq. 2 ) then
          do 100 i = 1 , 3
            if ( polar(i) .eq. 0 ) polax = i
 100      continue
        else
          polax = 0
        endif
       
*       Loop over positions
        do 500 i = 1 , posnum
*       Copy coordinates
          call matcop ( poscrd( 1,i ) , x , 3 , 1 )

*       Apply space group symmetry
          nxsym = nopsgs
          call symapp ( nopsgs , r_sgs , t_sgs , 1 , x , 
     &    nxsym , xsym , errcod )
          if ( errcod .ne. 0 ) then
            errcod = 2
            return
          endif

*       Get Harker vectors
          nv = mxvec
          call hvcalc ( xsym , nxsym , hv , nv )


*       Compare with Harker vectors of previously found sites
          do 200 j = 1 , sittot
            if ( nv .ne. nvec( j ) ) goto 200

*       Compare Harker vectors
            ict = 0
            do 150 k = 1 , nv
              do 140 kk = 1 , nvec(j)
                ict = ict + lgseql ( hv( 1,k ) , hvec( 1,kk,j ) , tol )
 140          continue
 150        continue
            if ( ict .ge. nv ) then
*       All Harker vectors match
*@@@@
*@@@@   THIS IS STILL NOT CORRECT
*@@@@
*       In space groups with polar axis, check if positions 
*       are identical or related by inversion on this axis
*              if ( polax .ne. 0 ) then
*       Loop from start to end of homomorphous group (site)
*                do 160 k = sitbeg( j ) , sitend( j )
*       If position k not in this homomorphous group go to next
*                  if ( sitpnt( k ) .ne. j ) goto 160
*
*       If x1 + x2 = tranlation component along polar axis both
*       positions belong to the same site
*                  d = poscrd( polax,k ) + poscrd( polax,i )
*                  d = mod ( d , 1.0 )
*       Check out all the symops
*                  do 155 kk = 1 , nopsgs
*                    if ( abs ( d - t_sgs( polax,kk ) ) .lt. 0.001 ) 
*     &              goto 170
* 155              continue
*
* 160            continue
*                goto 200
*              endif


*       Store site number, change pointer to end of site to current position
 170          sitpnt( i ) = j
              sitend( j ) = i
              goto 500
            endif

 200      continue

*       If we come here, this position is not equivalent to any of the
*       previous sites.
*       Increase site count
          sittot = sittot + 1
*       Store vector count and vectors
          nvec( sittot ) = nv
          do 300 j = 1 , nv
            call matcop ( hv( 1,j ) , hvec( 1 , j , sittot ) , 3 , 1 )
 300      end do
*       Store site number
          sitpnt( i ) = sittot
*       Store pointer to beginning of site (this position)
          sitbeg( sittot ) = i
*       Store pointer to current end of site (this position)
          sitend( sittot ) = i 

 500    continue

*       Pointer to end of last site (last position)
        sitend( sittot ) = i

        return
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Integer function SGSEQL
*
*       Function:
*       Inquiry function to determine if two points (in real space)
*       are space group equivalent. Returns SGSEQL = 1 if equivalent, 
*       0 if not.
*       
*       Comments:
*       Coordinates must be modulo 1.0.
*
*
*       Arguments:
*       Pnt1      (i)      first point (fractional)
*       Pnt2      (i)      second point (fractional)
*       Tol       (i)      minimum distance between points (Angstrom)
*       
*       
*       
*       History:
*       Written November 26 1991 S. Knight
************************************************************************
        integer function sgseql ( pnt1 , pnt2 , tol )

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'


*       Arguments
        real            pnt1( 3 ) , pnt2( 3 )
        real            tol
 

*       Local variables
        integer         i , j , k
        real            tpnt1( 3 ) , tpnt2( 3 ) , dvec( 3 )

*       External functions
        real            vlen
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initialize
        sgseql = 0

*       Loop on space group symmetry
        do 100 i = 1 , nopsgs

*       Apply space group symmetry to the first point
          call vmtply ( r_sgs( 1,1,i ) , pnt1 , tpnt1 )
          call vecadd ( tpnt1 , t_sgs( 1,i ) , tpnt2 , 3 )

        do 50 j = 1 , icen

*       Apply possible centering and bring point inside unit cell
          do 20 k = 1 , 3
            tpnt1( k ) = tpnt2( k ) + cent( k,j )
            tpnt1( k ) = mod ( tpnt1( k ) , 1.0 )
            if ( abs ( tpnt1( k ) ) .lt. 0.0001 ) tpnt1( k ) = 0.0
            if ( tpnt1( k ) .lt. 0. ) tpnt1( k ) = tpnt1( k ) + 1.0
 20       continue

*       Check distance
          call vecdif ( tpnt1 , pnt2 , dvec , 3 )
          if ( vlen( dvec,cell,cosang ) .lt. tol ) then
            sgseql = 1
            return
          endif


 50     continue
 100    continue



        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Integer function LGSEQL
*
*       Function:
*       Inquiry function to determine if two points (in vector space)
*       are Laue group equivalent. Returns LGSEQL = 1 if equivalent, 
*       0 if not.
*       
*       Comments:
*       Vector coordinates must be modulo 1.0.
*
*
*       Arguments:
*       Vec1      (i)      first vector (fractional)
*       Vec2      (i)      second vector (fractional)
*       Tol       (i)      maximum PATGRD separation in any direction
*       
*       
*       
*       History:
*       Written November 22 1991 S. Knight
************************************************************************
        integer function lgseql ( vec1 , vec2 , tol )

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'


*       Arguments
        real            vec1( 3 ) , vec2( 3 )
        real            tol
 

*       Local variables
        integer         i , j , k
        integer         match
        real            tvec( 3 ) , tvec1( 3 ) , tvec2( 3 )
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initialize
        lgseql = 0

*       Convert second vector to grid units
        do 10 i = 1 , 3
          tvec2( i ) =  vec2( i ) * patgrd( i )
 10     continue

*       Loop on patterson symmetry
        do 100 i = 1 , 2 * nopsgs

*       Apply patterson symmetry to the first vector
          call vmtply ( r_lgs( 1,1,i ) , vec1 , tvec )

        do 50 j = 1 , icen

*       Apply possible centering and bring vector inside unit cell
*       Convert to grid units
          do 20 k = 1 , 3
            tvec1( k ) = tvec( k ) + cent( k,j )
            tvec1( k ) = mod ( tvec1( k ) , 1.0 )
            if ( abs ( tvec1( k ) ) .lt. 0.0001 ) tvec1( k ) = 0.0
            if ( tvec1( k ) .lt. 0.0 ) tvec1( k ) = tvec1( k ) + 1.0
            tvec1( k ) = tvec1( k ) * patgrd( k )
 20       continue

*       Compare vectors
          call matcmp ( tvec1 , tvec2 , 3 , 1 , match , tol )
          if ( match .eq. 3 ) then
            lgseql = 1
            return
          endif


 50     continue
 100    continue



        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Logical function CLASH1
*
*       Function:
*       Inquiry function to determine if a molecule at a given
*       position clashes with any of its symmetry equivalents.
*       
*       Comments:
*       If the molecule is at a special position it is not
*       considered to clash.
*
*       Arguments:
*       X         (i)      position of molecule (fractional)
*       MolBmp    (i)      Minimum distance between molecules
*       
*       
*       
*       History:
*       Written March 3 1994 S. Knight
************************************************************************
        logical function clash1 ( x , molbmp )

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'


*       Arguments
        real            x( 3 ) , molbmp
 
*       Local variables
        integer         i , j , k
        real            d
        real            tvec1( 3 ) , tvec2( 3 ) , u( 3 )

*       External functions
        real            vlen
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initialize
        clash1 = .false.

*       Apply space group symmetry to input position
        do 100 i = 2 , nopsgs

          call vmtply ( r_sgs( 1,1,i ) , x , tvec1 )
          call vecadd ( tvec1 , t_sgs( 1,i ) , tvec2 , 3 )

        do 50 j = 1 , icen

*       Apply possible centering; reduce to unit cell; compute
*       vector to initial position
          do 20 k = 1 , 3
            tvec2( k ) = tvec2( k ) + cent( k,j )
            tvec2( k ) = mod ( tvec2( k ) , 1.0 )
            if ( abs ( tvec2( k ) ) .lt. 0.0001 ) tvec2( k ) = 0.0
            if ( tvec2( k ) .lt. 0.0 ) tvec2( k ) = tvec2( k ) + 1.0
            u( k ) = mod ( tvec2( k ) - x( k ) , 1.0 )
 20       continue

*       Check if positions clash
           d = vlen ( u , cell , cosang )
           if ( d .gt. 1.0 .and. d .lt. molbmp ) then
             clash1 = .true.
             return
           endif

 50     continue
 100    continue



        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Logical function CLASH2
*
*       Function:
*       Inquiry function to determine if two molecules clash
*       
*       Comments:
*
*       Arguments:
*       X1        (i)      position of molecule 1 (fractional)
*       X2        (i)      position of molecule 2 (fractional)
*       MolBmp    (i)      Minimum distance between molecules
*       
*       
*       
*       History:
*       Written March 3 1994 S. Knight
************************************************************************
        logical function clash2 ( x1 , x2 , molbmp )

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'


*       Arguments
        real            x1( 3 ) ,x2( 3 ) ,  molbmp
 
*       Local variables
        integer         i , j , k
        real            d
        real            tvec1( 3 ) , tvec2( 3 ) , u( 3 )

*       External functions
        real            vlen
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Initialize
        clash2 = .false.

*       Apply space group symmetry to the second input position
        do 100 i = 1 , nopsgs

          call vmtply ( r_sgs( 1,1,i ) , x2 , tvec1 )
          call vecadd ( tvec1 , t_sgs( 1,i ) , tvec2 , 3 )

        do 50 j = 1 , icen

*       Apply possible centering; reduce to unit cell; compute
*       vector to first position
          do 20 k = 1 , 3
            tvec2( k ) = tvec2( k ) + cent( k,j )
            tvec2( k ) = mod ( tvec2( k ) , 1.0 )
            if ( abs ( tvec2( k ) ) .lt. 0.0001 ) tvec2( k ) = 0.0
            if ( tvec2( k ) .lt. 0.0 ) tvec2( k ) = tvec2( k ) + 1.0
            u( k ) = mod ( tvec2( k ) - x1( k ) , 1.0 )
 20       continue

*       Check if positions clash
           d = vlen ( u , cell , cosang )
           if ( d .lt. molbmp ) then
             clash2 = .true.
             return
           endif

 50     continue
 100    continue



        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*       This file contains peak picking routines for rsps :
*
*       S/r PickFile    pick map file (CCP4 or PROTEIN P'son or score)
*       S/r PickCore    pick map in core (Patterson)
*       S/r PickSecF    read and pick map sections (calls pick1)
*       S/r PickSecC    pick map sections in core (calls pick1)
*       S/r Pick1       pick peaks on one section using three sections
*       S/r PickFlt     remove too close peaks
************************************************************************
************************************************************************
*
*       Subroutine PICKFILE
*
*       Function:
*       Pick up to MAXP highest peaks above PLEVEL from map file
*
*       Comment:
*       XCLPNT and PKXCLD are dimensioned MXEXCL to avoid out of bounds
*       error when NXCL = 0.
*
*       Arguments :
*       Maptyp  (i)     map format (must be protein or ccp4)
*       Mapnam  (i)     locical file name for map
*       Iunit   (i)     logical unit number for map
*       Np      (i/o)   on input  : maximum number of peaks to pick
*                       on output : number of peaks found
*       Plevel  (i)     minimum peak height
*       Pkltfr  (i)     limits within which to pick
*       Nxcl    (i)     number of regions to exclude from picking
*       Xclpnt  (i)     pointers to coordinate to use in exclusion selection
*       Pkxcld  (i)     exclusion value (fractional)
*       Xyz     (o)     peak coordinates (fractional)
*       Value   (o)     peak heights
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - RDHDR error
*                       errcod = 2 - map sections too large
*                       errcod = 3 - error getting map statistics
*
*
*       History:
*       February 22 1995 - sdk
*       Booth interpolation of peak max
*       March 1 1994 - sdk
*       Renamed PickFile to distinguish this s/r from PickCore.
*       February 10 1994 - sdk
*       Pick limits now come here fractional instead of in
*       grid units.
*       February 23 1993 - sdk
*       Pick level and increment assignement modified
*       September 9 1992 - sdk
*       Completely rearranged. Now uses standard map picking
*       routines using three stored sections.
*       September 1 1992 - sdk
*       Scaling removed - pick now operates on unmodified map
*       December 3 1991 - sdk
*       Exclude selection changed; now uses pointer to coordinate
*       + single value.
*       November 16 1991 - sdk
*       Scale map before picking. Scale input as argument.
*       November 6 1991 - sdk
*       Return fractional coordinates
*       October 2 1991 - sdk
*       Modified to use unpacked peak coordinates
*       Modified october 1989 - sdk
*       Only peaks within defined parts of the map are selected
*       Written March 21 1988 - sdk
************************************************************************
        subroutine pickfile ( maptyp,mapnam,iunit,np,plevel,pkltfr,
     &  nxcl,xclpnt,pkxcld,xyz,value,errcod )

        implicit none

        include         'rspsdim.inc'
        

*       Arguments
        integer         iunit
        integer         np
        integer         nxcl
        integer         xclpnt( mxexcl )
        integer         errcod
        real            pkltfr( 6 )
        real            pkxcld( mxexcl )
        real            xyz( 3,np )
        real            value( np )
        real            plevel
        character*(*)   maptyp
        character*(*)   mapnam


*       Local variables
        real            ucell( 6 ) 
        real            pincr
        real            rho( mxosct*3 )
        real            rmax , rmin , rave , rsig
        real            x( 3 )
        real            val( mxpik1 )
        real            pek( 3,mxpik1 )
        real            sepmin
        integer         nfast , nslow , nsec , npek
        integer         ict
        integer         i , j
        integer         pklimt( 6 ) , pklm( 6 )
        integer         mxyz( 3 ) , iuvw( 3 ) , grid( 3 ) , limit( 6 )
        character*80    title

        parameter       ( sepmin = 1.1 )
*-----------------------------------------------------------------------
*       Initiate error code
        errcod = 0

*       Check maptype OK
        if ( maptyp .ne. 'CCP4' .and. maptyp .ne. 'PROTEIN' ) then
          write(6,9000)maptyp
          stop
        endif

*       Clear pick coordinate and value arrays
        call matclr ( xyz , 3 , np )
        call matclr ( value , np , 1 )

*       Get map statistics
        call mflstat ( maptyp , mapnam , iunit , rmin,
     &  rmax , rave , rsig , errcod )
        if ( errcod .ne. 0 ) then
          errcod = 3
          return
        endif

*       Check there is density in map
        if ( rmax .eq. 0 ) then
          errcod = 4
          call wrnmsg ( 'No density in map; nothing to pick' )
          np  = 0
          return
        endif

*       Open map and read map header
        call rdhdr ( maptyp , iunit , mapnam , nsec , mxyz , iuvw ,
     &  grid , limit , ucell , title , errcod )
        if ( errcod .ne. 0 ) then
          if ( maptyp .eq. 'CCP4' ) then
            call mrclos ( iunit )
          else
            close ( iunit )
          endif
          errcod = 1
          return
        endif

*       Check that array rho is big enough
        nfast = mxyz( iuvw(1) )
        nslow = mxyz( iuvw(2) )
        if ( nfast*nslow .gt. mxosct ) then
          write(6,9020)nfast*nslow,mxosct
          errcod = 2
          np = 0
          return
        endif

*       Convert fractional pick limits to grid
*       Set limits for pick if not user specified
        if ( pkltfr(1) .gt. pkltfr(2) .and.
     &       pkltfr(3) .gt. pkltfr(4) .and.
     &       pkltfr(5) .gt. pkltfr(6) ) then
          do i = 1 , 6
            pklimt( i ) = limit( i )
          end do
        else
          do i = 1 , 3
            pklimt( 2*i-1 ) = nint ( pkltfr( 2*i-1 ) * grid( i ) )
            pklimt( 2*i   ) = nint ( pkltfr( 2*i   ) * grid( i ) )
          end do
        endif

*       Shift pick limits; store in order fast, medium, slow
        do i = 1 , 3
          pklm( 2*i-1 ) = pklimt( 2*iuvw(i)-1 ) - 
     &                     limit( 2*iuvw(i)-1 ) + 1

          pklm( 2*i   ) = pklimt( 2*iuvw(i)   ) -
     &                     limit( 2*iuvw(i)-1 ) + 1

          if ( pklm( 2*i-1 ) .le. 0 ) pklm( 2*i-1 ) = 1

          pklm( 2*i ) = min ( pklm( 2*i ) , mxyz( iuvw(i) ) )
        end do

*       Set pick level if not delivered
        if ( plevel .eq. 0 ) plevel = max ( 0., rmin + (rmax-rmin)/10.)
        write (6,2000) plevel

*       Set increment to increase pick level if too many peaks found
        pincr = (rmax-plevel)/10.

*       Pick map sections. Peaks are resolved to within a 3x3x3 box
 10     continue

        call picksecf ( maptyp , iunit , rho , nfast , nslow , nsec ,
     &  plevel , pklm , pek , val , npek , errcod )
        if ( errcod .eq. 1 ) then
*       Too many peaks found - restart with higher level
          write(6,9030) plevel , plevel + pincr
          plevel = plevel + pincr
          call maptop ( maptyp , iunit , limit( 2*iuvw(3)-1 ) ,
     &    errcod )
          goto 10
        endif

*       If no peaks found just close files and return
        if ( npek .le. 0 ) goto 40

*       Sort peaks in descending order
        call qsortff ( npek , pek , val , errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'Recursion stack overflow in QSORTFI' , 1 )
        endif

*       Filter out too close peaks
        call pickflt ( npek,pek,val,npek,sepmin )

*       Convert coords to fractional along x,y,z; remove excluded 
*       Save up to np peaks
        ict = 0
        do 30 i = 1 , npek

*       Rearrange peaks x,y,z instead of fast,medium,slow
          do j = 1 , 3
            x( iuvw(j) ) = limit( 2*iuvw(j)-1 ) + pek( j,i ) - 1
          end do

*       Convert to fractional
          do j = 1 , 3
            x( j ) = x( j ) / real ( grid( j ) )
          end do

*       Skip if in exclude region
          do j = 1 , nxcl
            if ( abs ( x( xclpnt(j) ) - pkxcld(j) ) .lt. 0.00001 ) 
     &      goto 30
          end do

*       Count and copy
          if ( ict .lt. np ) then
            ict = ict + 1
            xyz( 1,ict ) = x( 1 )
            xyz( 2,ict ) = x( 2 )
            xyz( 3,ict ) = x( 3 )
            value( ict ) = val( i )
          else
            goto 40
          endif

 30     continue

*       Finished; copy peak counter, close map file
 40     np = ict
        if ( maptyp .eq. 'CCP4' ) then
          call mrclos ( iunit )
        else
          close ( iunit )
        endif


        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
 2000   format(10x,'PICK >> Pick level set to ',f8.2 )
 9000   format(/10x,'** ERROR - Unknown map format: ',a,
     &  ' known map formats: CCP4, PROTEIN **')
 9020   format(/10x,'** ERROR - Map sections too big in s/r PICKFILE',
     &  /10x,'Number of points requested = ',i8,
     &  /10x,'Number of points available = ',i8,
     &  /10x,'Pick aborted **')
 9030   format(/10x,'** WARNING - Too many peaks found - base',
     &  ' level reset from ',f6.1,' to ',f6.1,' **')
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine PICKCORE
*
*       Function:
*       Pick up to MAXP highest peaks above PLEVEL from map in core
*
*       Comment:
*       XCLPNT and PKXCLD are dimensioned MXEXCL to avoid out of bounds
*       error when NXCL = 0.
*
*       Arguments :
*       Pmap    (i)     array of Patterson values
*       Pfast   (i)     Number of points in fast direction
*       Pslow   (i)     Number of points in slow direction
*       Psect   (i)     Number of points in section direction
*       Np      (i/o)   on input  : maximum number of peaks to pick
*                       on output : number of peaks found
*       Plevel  (i)     minimum peak height
*       Pkltfr  (i)     limits within which to pick
*       Nxcl    (i)     number of regions to exclude from picking
*       Xclpnt  (i)     pointers to coordinate to use in exclusion selection
*       Pkxcld  (i)     exclusion value (fractional)
*       HrkOnl  (i)     switch for Harker-only picking
*       Xyz     (o)     peak coordinates (fractional)
*       Value   (o)     peak heights
*       Type    (o)     Vector type flag: 1 = Harker; 2 = non Harker
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - no positive peaks on map
*                       errcod = 2 - map sections too large
*                       errcod = 3 - ORTHO error
*                       errcod = 4 - failed getting Harker plane equations
*
*
*       History:
*       February 1996 - sdk
*       Harker vector flagging introduced. Argument list changed to return
*       type flag; also HrkOnl switch in arg list. This procedure is now
*       appropriate only for Patterson maps.
*       February 22 1995 - sdk
*       Modified for Booth interpolation in PICK1
*       Adapted from PickFile March 1 1994 S. Knight
************************************************************************
        subroutine pickcore ( pmap,pfast,pslow,psect,np,plevel,pkltfr,
     &  nxcl,xclpnt,pkxcld,hrkonl,xyz,value,type,errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'
        include         'rspsmap.inc'
        include         'rspsgrd.inc'
        

*       Arguments
        logical         hrkonl
        integer         np
        integer         nxcl
        integer         xclpnt( mxexcl )
        integer         errcod
        integer         pfast , pslow , psect
        integer         type( np )
        real            pmap( pfast,pslow,psect )
        real            pkltfr( 6 )
        real            pkxcld( mxexcl )
        real            xyz( 3,np )
        real            value( np )
        real            plevel


*       Local variables
        real            pincr
        real            rho( mxosct*3 )
        real            x( 3 ) , x2( 3 )
        real            val( mxpik1 )
        real            pek( 3,mxpik1 )
        real            cut
        real            toort( 3,3 )
        real            sepmin
        integer         npek
        integer         ict
        integer         itp
        integer         i , j
        integer         pklimt( 6 ) , pklm( 6 )
        parameter       ( sepmin = 1.1 )

*       External
        real            plndst
*-----------------------------------------------------------------------
*       Initiate error code
        errcod = 0

*       Clear pick coordinate and value arrays
        call matclr ( xyz , 3 , np )
        call matclr ( value , np , 1 )

*       Check there is density in map
        if ( patmax .eq. 0 ) then
          errcod = 1
          call wrnmsg ( 'No density on map; nothing to pick' )
          return
        endif

*       Check that array rho is big enough
        if ( pfast*pslow .gt. mxosct ) then
          write(6,9020)pfast*pslow,mxosct
          errcod = 2
          np = 0
          return
        endif

*       Convert fractional pick limits to grid
*       Set limits for pick if not user specified
        if ( pkltfr(1) .gt. pkltfr(2) .and.
     &       pkltfr(3) .gt. pkltfr(4) .and.
     &       pkltfr(5) .gt. pkltfr(6) ) then
          do i = 1 , 6
            pklimt( i ) = patlim( i )
          end do
        else
          do i = 1 , 3
            pklimt( 2*i-1 ) = nint ( pkltfr( 2*i-1 ) * patgrd( i ) )
            pklimt( 2*i   ) = nint ( pkltfr( 2*i   ) * patgrd( i ) )
          end do
        endif

*       Shift pick limits; store in order fast, medium, slow
        do i = 1 , 3
          pklm( 2*i-1 ) = pklimt( 2*pataxo(i)-1 ) - 
     &                    patlim( 2*pataxo(i)-1 ) + 1

          pklm( 2*i   ) = pklimt( 2*pataxo(i)   ) -
     &                    patlim( 2*pataxo(i)-1 ) + 1

          if ( pklm( 2*i-1 ) .le. 0 ) pklm( 2*i-1 ) = 1

          pklm( 2*i ) = min ( pklm( 2*i ) , patext( pataxo( i ) ) )
        end do

*       Set pick level if not delivered
        if ( plevel .eq. 0 ) plevel = 
     &  max ( 0., patmin + (patmax-patmin)/10.)
        write (6,2000) plevel

*       Set increment to increase pick level if too many peaks found
        pincr = (patmax-plevel)/10.

*       Pick map sections. Peaks are resolved to within a 3x3x3 box
 10     continue

        call picksecc ( pmap , rho , pfast , pslow , psect ,
     &  plevel , pklm , pek , val , npek , errcod )
        if ( errcod .eq. 1 ) then
*       Too many peaks found - restart with higher level
          write(6,9030) plevel , plevel + pincr
          plevel = plevel + pincr
          goto 10
        endif

*       If no peaks found just close files and return
        if ( npek .le. 0 ) goto 40

*       Sort peaks in descending order
        call qsortff ( npek , pek , val , errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'Recursion stack overflow in QSORTFI' , 1 )
        endif

*       Filter out too close peaks
        call pickflt ( npek,pek,val,npek,sepmin )

*       Set matrix for converting fractional crystallographic
*       coordinates to Angstrom orthogonal in PDB standard frame
        call ortho ( cell , 1 , 'TOORT' , toort , errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'Failed getting orthogonalization matrix' , 0 )
          errcod = 3
          return
        endif

*       Determine Harker plane equations
        call hrkpln ( errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'HRKPLN failed' , 0 )
          errcod = 4
          return
        endif

*       Cutoff for Harker vector flag: max distance from Harker plane
        cut = -1.0
        do i = 1 , 3
          cut = max ( cut , 1.6/patgrd( i ) )
        end do

*       Convert coords to fractional along x,y,z; remove excluded 
*       Save up to np peaks
        ict = 0
        do 30 i = 1 , npek

*       Rearrange peaks x,y,z instead of fast,medium,slow
          do j = 1 , 3
            x( pataxo(j) ) = patlim( 2*pataxo(j)-1 ) + pek( j,i ) - 1
          end do

*       Convert to fractional
          do j = 1 , 3
            x( j ) = x( j ) / real ( patgrd( j ) )
          end do

*       Skip if in exclude region
          do j = 1 , nxcl
            if ( abs ( x( xclpnt(j) ) - pkxcld(j) ) .lt. 0.00001 ) 
     &      goto 30
          end do

*       Count and copy
          if ( ict .lt. np ) then
*       Set vector type flag (1 for Harker, 2 for non Harker)
*       S/r reduce is called prior to the test to make sure it
*       will work regardless of map limits
            itp = 2
            call rmatcop ( x , x2 , 3 , 1 )
            call reduce( x2 )
            do j = 1 , nplanes
              if ( abs ( plndst( x2 , hplane( 1,j ) ) ) .lt. cut ) then
                itp = 1
              endif
            end do
            if ( hrkonl .and. itp .eq. 2 ) goto 30
            ict = ict + 1
            xyz( 1,ict ) = x( 1 )
            xyz( 2,ict ) = x( 2 )
            xyz( 3,ict ) = x( 3 )
            value( ict ) = val( i )
            type( ict )  = itp
          else
            goto 40
          endif

 30     continue

*       Finished; copy count
 40     np = ict

        return

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Format statements
 2000   format(10x,'PICK >> Pick level set to ',f8.2 )
 9020   format(/10x,'** ERROR - map sections too big in s/r PICKCORE',
     &  /10x,'Number of points requested = ',i8,
     &  /10x,'Number of points available = ',i8,
     &  /10x,'Pick aborted **')
 9030   format(/10x,'** WARNING - Too many peaks found - base',
     &  ' level reset from ',f6.1,' to ',f6.1,' **')
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine PICKSECF
*
*       Function:
*       Pick peaks above plevel from map file. Peaks are resolved to 
*       within a box of 3x3x3 grid units.
*
*       Comment:
*
*
*       Arguments :
*       Maptyp    (i)       map format (CCP4 or PROTEIN)
*       Iunit     (i)       logical unit for map
*       Rho       (i)       scratch array for holding three sections of
*                           density
*       Nfast     (i)       number of points in fast direction of map
*       Nslow     (i)       number of points in slow direction of map
*       Nsec      (i)       number of map sections
*       Plevel    (i)       minimum density for peaks
*       Pklim     (i)       limits of pick
*       Pek       (o)       peak coordinates
*       Val       (o)       peak values
*       Npek      (o)       number of peaks
*       Errcod    (o)       error code
*                           errcod = 0 - no error
*                           errcod = 1 - too many peaks found
*
*
*
*       History:
*       March 1 1994 - sdk 
*       Renamed PickSecF to distinguish it from s/r PickSecC which
*       picks a map in core.
*       Written September 9 1992 - S. Knight
************************************************************************
        subroutine picksecf ( maptyp , iunit , rho , nfast , nslow , 
     &  nsec , plevel , pklim , pek , val , npek , errcod )

        implicit none

        include         'rspsdim.inc'

*       Arguments
        integer         iunit
        integer         nfast , nslow , nsec
        integer         pklim( 6 )
        integer         npek
        integer         errcod
        real            rho( nfast,nslow,3 )
        real            plevel
        real            pek( 3,mxpik1 )
        real            val( mxpik1 )
        character*(*)   maptyp

*       Local variables
        integer         isec( 3 ) , lsec
        integer         i
        integer         npps
        real            rmax , rmin , rsum , rssq , rhomax
*-----------------------------------------------------------------------
*       Initiate error code
        errcod = 0

*       Initiate rho matrix
        call matclr ( rho , nfast*nslow , 3 )

*       Set npek < 0 to signal s/r pick1 to initiate peak arrays on 
*       first call to s/r pick1
        npek = -1

*       Initiate section pointers
*       Isec pointers point to slots in rho
*       isec(1) = slot for previous section
*       isec(2) = slot for current section
*       isec(3) = slot to use for next session
        isec(1) = 1
        isec(2) = 2
        isec(3) = 3

*       Number of points/section
        npps = nfast * nslow

*       Read first section and store in middle slot of rho
        rmax = 0.0
        rmin = 0.0
        rsum = 0.0
        rssq = 0.0
        call rdsec ( maptyp , iunit , rho(1,1,isec(2)) , npps ,
     &  rmax , rmin , rsum , rssq )

        do 20 i = 1 , nsec

*       Save max density for current section (last read)
*       Init statistics variables
          rhomax = rmax
          rmax = 0.0
          rmin = 0.0
          rsum = 0.0
          rssq = 0.0

*       Read next section
          if ( i .lt. nsec ) then
            call rdsec ( maptyp , iunit , rho(1,1,isec(3)) , npps ,
     &      rmax , rmin , rsum , rssq )
          endif
          if ( i .eq. 1 ) then
*       Current section is first section; copy next section to previous
*       If there is only one section in the map both sections surrounding
*       the current section will be empty
            call matcop ( rho(1,1,isec(3)) , rho(1,1,isec(1)) , 
     &      npps , 1 )
          endif
          if ( i .eq. nsec ) then
*       Current section is last section; copy previous section to next
            call matcop ( rho(1,1,isec(1)) , rho(1,1,isec(3)) , 
     &      npps , 1 )
          endif

*       Skip if outside pick limits
          if ( i .lt. pklim( 5 ) .or. i .gt. pklim( 6 ) ) goto 15

*       Skip if max density on section too low
          if ( rhomax .lt. plevel ) goto 15

*       Pick current section
          call pick1 ( mxpik1 , npek , pek , val , plevel , 
     &    pklim , rho , nfast , nslow , isec , i , errcod )
          if ( errcod .ne. 0 ) return

*       Reset section pointers
 15       lsec    = isec(1)
          isec(1) = isec(2)
          isec(2) = isec(3)
          isec(3) = lsec

 20     continue

        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine PICKSECC
*
*       Function:
*       Pick peaks above plevel from map in core. Peaks are resolved to 
*       within a box of 3x3x3 grid units.
*
*       Comment:
*
*
*       Arguments :
*       Pmap      (i)       Array of Patterson values
*       Rho       (i)       scratch array for holding three sections of
*                           density
*       Nfast     (i)       number of points in fast direction of map
*       Nslow     (i)       number of points in slow direction of map
*       Nsec      (i)       number of map sections
*       Plevel    (i)       minimum density for peaks
*       Pklim     (i)       limits of pick
*       Pek       (o)       peak coordinates (non-integral grid units)
*       Val       (o)       peak values
*       Npek      (o)       number of peaks
*       Errcod    (o)       error code
*                           errcod = 0 - no error
*                           errcod = 1 - too many peaks found
*
*
*
*       History:
*
*       Written March 1 1994 - S. Knight
************************************************************************
        subroutine picksecc ( pmap, rho , nfast , nslow , 
     &  nsec , plevel , pklim , pek , val , npek , errcod )

        implicit none

        include         'rspsdim.inc'

*       Arguments
        integer         nfast , nslow , nsec
        integer         pklim( 6 )
        integer         npek
        integer         errcod
        real            pmap( nfast,nslow,nsec )
        real            rho( nfast,nslow,3 )
        real            plevel
        real            pek( 3,mxpik1 )
        real            val( mxpik1 )

*       Local variables
        integer         isec( 3 ) , lsec
        integer         i
        integer         npps
        real            rmax , rhomax

*       External functions
        real            fmaxvc
*-----------------------------------------------------------------------
*       Initiate error code
        errcod = 0

*       Initiate rho matrix
        call matclr ( rho , nfast*nslow , 3 )

*       Set npek < 0 to signal s/r pick1 to initiate peak arrays on 
*       first call to s/r pick1
        npek = -1

*       Initiate section pointers
*       Isec pointers point to slots in rho
*       isec(1) = slot for previous section
*       isec(2) = slot for current section
*       isec(3) = slot to use for next session
        isec(1) = 1
        isec(2) = 2
        isec(3) = 3

*       Number of points/section
        npps = nfast * nslow

*       Store first map section in middle slot of rho
        call matcop ( pmap( 1,1,1 ) , rho(1,1,isec(2)) , npps , 1 )

*       Get maximum density
        rmax = fmaxvc ( pmap( 1,1,1 ) , npps )

        do 20 i = 1 , nsec
*       Save max density for current section (last read)
          rhomax = rmax
          rmax = 0.0

*       Store next section
          if ( i .lt. nsec ) then
            call matcop ( pmap( 1,1,i+1 ) , rho(1,1,isec(3)) , 
     &      npps , 1 )
            rmax = fmaxvc ( pmap( 1,1,i+1 ) , npps )
          endif
          if ( i .eq. 1 ) then
*       Current section is first section; copy next section to previous
*       If there is only one section in the map both sections surrounding
*       the current section will be empty
            call matcop ( rho(1,1,isec(3)) , rho(1,1,isec(1)) , 
     &      npps , 1 )
          endif
          if ( i .eq. nsec ) then
*       Current section is last section; copy previous section to next
            call matcop ( rho(1,1,isec(1)) , rho(1,1,isec(3)) , 
     &      npps , 1 )
          endif

*       Skip if outside pick limits
          if ( i .lt. pklim( 5 ) .or. i .gt. pklim( 6 ) ) goto 15

*       Skip if max density on section too low
          if ( rhomax .lt. plevel ) goto 15

*       Pick current section
          call pick1 ( mxpik1 , npek , pek , val , plevel , 
     &    pklim , rho , nfast , nslow , isec , i , errcod )
          if ( errcod .ne. 0 ) return

*       Reset section pointers
 15       lsec    = isec(1)
          isec(1) = isec(2)
          isec(2) = isec(3)
          isec(3) = lsec

 20     continue

        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine PICK1
*
*       Function:
*       Pick peaks above level from map section 
*
*       Comment:
*
*
*       Arguments :
*       N       (i)     maximum number of peaks
*       Np      (i/o)   on input  : if np.lt.0 clear peak and value.
*                                   if np.ge.0 np=number of stored peaks
*                       on output : number of stored peaks
*       Peak    (o)     peak coordinates
*       Value   (o)     peak values 
*       Level   (i)     threshold value for peaks
*       Pklm    (i)     limits within which to pick
*       Rho     (i)     array holding three map sections
*       Nfast   (i)     number of points along fast axis
*       Nslow   (i)     number of points along slow axis
*       Isec    (i)     pointer to previous, current, next section
*       Jsec    (i)     current section number
*       Errcod  (o)     error code
*                       errcod = 0 - no error
*                       errcod = 1 - peak array overflow
*
*       History:
*       February 22 1995 - sdk
*       Booth interpolation
*       September 9 1992 - sdk
*       Current section number renamed JSEC. ISEC points to current
*       section in density array.
*       November 11 1991 - sdk
*       Current section number (ISEC) now provided by calling routine.
*       October 2 1991 - sdk
*       Modified to use unpacked peak coordinates
*       Modified october 1989 - sdk
*       Limits check added
*       Written March 15 1988 - sdk
************************************************************************
        subroutine pick1 ( n,np,peak,value,plevel,pklm,
     &  rho,nfast,nslow,isec,jsec,errcod )

        implicit none

*       Arguments
        integer         n , np , nfast , nslow
        integer         isec(3)
        integer         jsec
        integer         pklm(6)
        integer         errcod
        real            peak(3,n)
        real            value(n),plevel
        real            rho(nfast,nslow,3)

*       Local variables
        integer         i , j , k , ib , ib1 , jb , jb1 , is
        real            dens
        real            r1 , r2 , q , cm
        real            r( 3,2 )
*-----------------------------------------------------------------------
*       Initiate error code
        errcod = 0

*       Initiate peak coordinate and value arrays
        if ( np .lt. 0 ) then
          call matclr ( peak , 3 , n )
          call matclr ( value , n , 1 )
          np = 0
        endif

*       Go through current section within pick limits
        do 200 i = pklm(3) , pklm(4)
        do 100 j = pklm(1) , pklm(2)
          dens = rho( j,i,isec( 2 ) )
          if ( dens .lt. plevel ) goto 100

*       Check nearest neighbours
*       Note that if point is on a border, the density value for 
*       out-of-bounds neighbours is copied from the other-side
*       neighbour. This will generally not be correct but
*       forces the peak to lie on the boundary.
          do is = 1 , 3 
            do ib = i-1 , i+1
              ib1 = ib
              if ( ib1 .le. 0 ) ib1 = i + 1
              if ( ib1 .gt. nslow ) ib1 = i - 1
              do jb = j-1 , j+1
                jb1 = jb
                if ( jb1 .le. 0 ) jb1 = j + 1
                if ( jb1 .gt. nfast ) jb1 = j - 1
                if ( rho( jb1,ib1,is ) .gt. dens ) goto 100
              end do
            end do
          end do

*       This point at least as high as all nearest neighbours; store it.
          np = np + 1
          if ( np .gt. n ) then
            errcod = 1
            return
          else
            peak( 1,np ) = j
            peak( 2,np ) = i
            peak( 3,np ) = jsec
            value(  np ) = dens

*       Interpolation of peak maximum modified from PEAKMAX (CCP4)
*       Uses Booth's parabola procedure
*       Boundary conditions as above
            jb = j - 1
            if ( jb .le. 0 ) jb = j + 1
            r( 1,1 ) = rho( jb,i,isec(2) )

            jb = j + 1
            if ( jb .gt. nfast ) jb = j - 1
            r( 1,2 ) = rho( jb,i,isec(2) )

            ib = i - 1
            if ( ib .le. 0 ) ib = i + 1
            r( 2,1 ) = rho( j,ib,isec(2) )

            ib = i + 1
            if ( ib .gt. nslow ) ib = i - 1
            r( 2,2 ) = rho( j,ib,isec(2) )

            r( 3,1 ) = rho( j,i,isec(1) )
            r( 3,2 ) = rho( j,i,isec(3) )

*       k loops over fast medium slow coordinate
            do 60 k = 1,3
              if ( r( k,2 ) .le. r( k,1 ) ) then
                r1 = dens - r( k,2 )
                r2 = r( k,1 ) - r( k,2 )
                q = -1.0
              else
                r1 = dens - r( k,1 )
                r2 = r( k,2 ) - r( k,1 )
                q = 1.0
              endif

              cm = 0.0
              if ( ( abs ( r1 ) .ge. 0.001 .or. abs ( r2 ) .ge. 0.001 ) 
     &        .and. ( abs ( 2.0*r2 - 4.0*r1 ) .gt. 0.000001 ) ) then 
                cm = ( r2 - 4.0*r1 ) / ( 2.0*r2 - 4.0*r1 ) - 1.0
              endif

              peak( k,np ) = peak( k,np ) + q * cm

 60         continue

          endif

 100    continue
 200    continue

        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine PICKFLT
*
*       Function:
*       Filter out too close peaks.
*
*       Comment:
*       The input list of peaks is assumed to be sorted in descending
*       order of peak value.
*
*       Arguments:
*       Npek      (i)       Number of input peaks
*       Peak      (i/o)     Peak coordinates (grid, arranged fms)
*       Value     (i/o)     Peak values
*       Nout      (i/o)     Max number of peaks (i), number of peaks
*                           found (o)
*       Sepmin    (i)       Minimum separation of peaks in all 
*                           directions (grid units)
*       
*       
*       History:
*       March 6 1996 - sdk
*       Sepmin argument changed from integer to real.
*       February 22 1995 - sdk
*       Peak array changed to real (to allow for interpolated peak
*       coordinates)
*       September 9 1992 - sdk
*       Sorting moved out; limits not checked. Remaining function
*       is filtering only. Obsolete arguments removed. SEPMIN
*       argument added. Subroutine name changed from PICK2 to PICKFLT.
*       December 3 1991 - sdk
*       Arguments modified: input/output coordinates and peak values
*       now occupy the same area (arguments IPKOUT and VALOUT removed).
*       Pick exclude selection now done in calling routine; 
*       corresponding arguments removed.
*       October 2 1991 - sdk
*       Modified to use unpacked peak coordinates
*       Modified october 1989 - sdk
*       Limits check added
*       Written March 15 1988 - sdk
************************************************************************
        subroutine pickflt ( npek,peak,value,nout,sepmin )

        implicit none

        include         'rspsdim.inc'

*       Arguments
        integer         npek , nout
        real            sepmin
        real            peak(3,npek)
        real            value(npek)

*       Local variables
        integer         i , j
        integer         ict
        real            fst1 , slw1 , sec1
        logical         skip(mxpik1)
*-----------------------------------------------------------------------
*       Return if nothing to do
        if ( sepmin .le. 0 ) return

*       Init counter and skip array
        ict = 0
        do i = 1 , npek
        skip( i ) = .false.
        end do

*       Loop over sorted peaks and discard anything within +-sepmin 
*       grid units of a higher peak
        do 100 i = 1 , npek - 1

          if ( skip(i) ) goto 100

          fst1 = peak( 1,i )
          slw1 = peak( 2,i )
          sec1 = peak( 3,i )

          do 10 j = i+1 , npek

            if ( skip(j) ) goto 10
            if ( abs ( fst1 - peak( 1,j ) ) .gt. sepmin ) goto 10
            if ( abs ( slw1 - peak( 2,j ) ) .gt. sepmin ) goto 10
            if ( abs ( sec1 - peak( 3,j ) ) .gt. sepmin ) goto 10
            skip(j) = .true.

 10       continue

          if ( ict .lt. nout ) then
            ict = ict + 1
            peak( 1,ict ) = peak( 1,i )
            peak( 2,ict ) = peak( 2,i )
            peak( 3,ict ) = peak( 3,i )
            value(   ict ) = value(   i )
          endif

          if ( ict .eq. nout ) return

 100    continue


        if ( .not. skip( npek ) .and. ict .lt. nout ) then
          ict = ict + 1
          peak( 1,ict ) = peak( 1,i )
          peak( 2,ict ) = peak( 2,i )
          peak( 3,ict ) = peak( 3,i )
          value(   ict ) = value(   i )
        endif

 200    nout = ict

        return
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine PIKPAT
*
*       Function:
*       Pick Patterson map in core.
*
*
*       Arguments:
*       Errcod    (o)   error code
*                       errcod = 0 - no error
*                       errcod = 1 - error getting peaks
*                       errcod = 2 - peak array overflow
*                       errcod = 3 - recursion stack ovflw in QSORT
*                       errcod = 4 - orthogonalization error
*
*
*       History:
*       February 1996 - sdk
*       Call to s/r PICKCORE changed - s/r now returns vector type
*       flag (1 = Harker, 2 = non Harker).
*       March 1 1994 - sdk
*       Modified to pick map in core instead of file. The resulting
*       peak list thus now reflects the current (possibly modified)
*       Patterson map.
*       940210 - sdk:
*       Don't duplicate already stored peaks when adding peaks
*       931121 - sdk:
*       Pick level assignement removed from here to avoid bad choice 
*       if map has been scaled (patsig then refers to scaled map and 
*       not to the original map which is the one being picked).
*       Written November 10 1991 S. Knight
************************************************************************
        subroutine pikpat ( errcod )

        implicit none

        include         'rspsdim.inc'
        include         'rspssym.inc'
        include         'rspsgrd.inc'
        include         'rspsmap.inc'
        include         'rspsctl.inc'
        include         'rspsfil.inc'

*       Arguments
        integer         errcod

*       Local variables
        integer         i , j , ict
        integer         pektot
        integer         match
        integer         pektyp(mxpick)
        real            pekuvw(3,mxpick) , pekval(mxpick)
        real            toort( 3,3 )
        real            xut( 3 )


        character*1     symbol( 2 )
        data            symbol / 'H' , ' ' /
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
*       Init
        errcod = 0

*       Write header
        write ( 6,2260 )

*       Pick
        call pickcore ( patmap , patfas , patslo , patsec , npick , 
     &  pick_level , pkltfr , nxcl , xclpnt , pkxcld , hrkonl , 
     &  pekuvw , pekval , pektyp , errcod )
        if ( errcod .ne. 0 ) then
          errcod = 1
          call errmsg ( 'PICK error' , 0 )
          return
        endif

*       Return if no peaks found
        if ( npick .eq. 0 ) then
          write ( 6,2265 )
          return
        endif
        
*       Set local pointer to last occupied slot in peak array
*       and check for space
        pektot = ppnum
        if ( pektot + npick .gt. mxpick ) then
          errcod = 2
          call errmsg ( 'Peaks array full - can''t store peaks' , 0 )
          return
        endif

*       Set matrix for converting fractional crystallographic
*       coordinates to Angstrom orthogonal in PDB standard frame
        call ortho ( cell , 1 , 'TOORT' , toort , errcod )
        if ( errcod .ne. 0 ) then
          call errmsg ( 'Failed getting orthogonalization matrix' , 
     &    0 )
          errcod = 4
          return
        endif

*       List new peaks
        write ( 6,2270 ) npick , pick_level
        do 50 i = 1 , npick
          call vmtply ( toort , pekuvw(1,i) , xut )
          write ( 6,2280 ) i , symbol( pektyp( i ) ) ,
     &    ( pekuvw( j,i ) , j = 1 , 3 ) , xut , 
     &    ( nint ( pekuvw( j,i ) * patgrd( j ) ) , j = 1 , 3 ) , 
     &    pekval( i ) , pekval( i )/patsig
 50     continue

*       Copy (uniquely new) peaks to peak array
        ict = ppnum
        do 100 i = 1 , npick
          do j = 1 , ppnum
            call matcmp ( pekuvw( 1,i ), ppuvw( 1,j ) , 
     &      3 , 1 , match , 0.001 )
            if ( match .eq. 3 ) goto 100
          end do
          ict = ict + 1
          if ( ict .gt. mxpick ) then
            errcod = 2
            call wrnmsg ( 'Peaks array full - couldn''t store ' //
     &      'all peaks' )
            ict = ict - 1
            goto 200
          endif
          ppuvw( 1,ict ) = pekuvw( 1,i )
          ppuvw( 2,ict ) = pekuvw( 2,i )
          ppuvw( 3,ict ) = pekuvw( 3,i )
          ppval(   ict ) = pekval( i )
          pptyp(   ict ) = pektyp( i )
 100    continue


*       If adding new peaks, re-sort peak array
 200    if ( ppnum .gt. 0 ) call qsortffi ( ppnum+ict , ppuvw , 
     &  ppval , pptyp , errcod )
        if ( errcod .ne. 0 ) then
          errcod = 3
          call errmsg ( 'Recursion stack overflow in QSORTFFI' , 1 )
        endif
*       Reset global pointer to last occupied slot in peak array
        ppnum = ict


*       Say how many peaks are stored
        write(6,2285)ppnum



        return

*-----------------------------------------------------------------------
*       Format statements
 2260   format(10X,'Picking Patterson map >>')
 2265   format(/10x,'PICK >> No peaks found')
 2270   format(10x,'PICK >> The ',i3,' highest peaks above ',f8.1,
     &  ' are listed in descending order',//10x,'Peak      ',
     &  ' Fractional coordinates     Angstrom coordinates',
     &  '         Grid coordinates       Value      S/N',/10x,
     &  '----      ------------------------  ------------------------',
     &  '    ----------------------  ---------  -------')
 2280   format(8x,i5,1x,a1,4x,3f8.4,2x,3f8.2,2x,3i8,4x,f8.2,f8.1)
 2285   format(//1x,'RSPS >> PICK PATTERSON >> ',i5,' peaks now in ',
     &  'peak list' )

*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        end
*********************************************************************SDK
************************************************************************
*
*       Subroutine TIMER
*
*       Function:
*       Uses ccptim to find cpu and elaps time and then writes it out
*
*       Arguments:
*       Iflag     (i)    start/stop switch (0 = start, 1 = stop)
*       Label     (i)    label to output if iflag = 1
*
*       History:
*       Written April 15 1988 S. Knight
************************************************************************
        subroutine timer ( iflag , label )

        implicit none

*       Arguments
        integer         iflag
        character*(*)   label

*       Local variables
        integer         cpuh,cpum,elph,elpm
        real            cpu,elp,cpus,elps
*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
        call ccptim ( iflag,cpu,elp )
        if ( iflag.eq.1 ) then
          cpuh = int ( cpu / 3600. )
          cpum = int ( ( cpu - 3600 *cpuh ) / 60. )
          cpus = cpu - 60 * cpum - 3600 * cpuh
          elph = int ( elp / 3600. )
          elpm = int ( ( elp - 3600 * elph ) / 60. )
          elps = elp - 60 * elpm - 3600* elph
          write(6,2000)label,cpuh,cpum,cpus,elph,elpm,elps
        endif
        return
 2000   format(/10x,a,t20,' : CPU = ',i3,' h ',i2,' m ',f4.1,' s ',
     &  ' ELAPS = ',i3,' h ',i2,' m ',f4.1,' s ',/ )
        end
*********************************************************************SDK

